Package: mets
Type: Package
Title: Analysis of Multivariate Event Times
Version: 1.3.10
Authors@R: c(person("Klaus K.", "Holst", email="klaus@holst.it", role=c("aut", "cre")),
             person("Thomas Scheike", role=c("aut")))
Author: Klaus K. Holst [aut, cre],
  Thomas Scheike [aut]
Maintainer: Klaus K. Holst <klaus@holst.it>
Description: Implementation of various statistical models for multivariate
    event history data <doi:10.1007/s10985-013-9244-x>. Including multivariate
    cumulative incidence models <doi:10.1002/sim.6016>, and  bivariate random
    effects probit models (Liability models) <doi:10.1016/j.csda.2015.01.014>.
    Modern methods for survival analysis, including regression modelling (Cox, Fine-Gray, 
    Ghosh-Lin, Binomial regression) with fast computation of influence functions.
License: Apache License (== 2.0)
LazyLoad: yes
URL: https://kkholst.github.io/mets/, https://github.com/kkholst/mets
BugReports: https://github.com/kkholst/mets/issues
Depends: R (>= 3.5)
Imports: Rcpp, RcppArmadillo (>= 0.12.8.0), lava (>= 1.9.1), methods,
        mvtnorm, numDeriv, splines, survival (>= 2.43-1), timereg (>=
        1.9.4)
Suggests: cmprsk, icenReg, KernSmooth, knitr, optimx, prodlim,
        riskRegression, rmarkdown, tinytest (>= 1.4.1), ucminf
VignetteBuilder: knitr
ByteCompile: yes
LinkingTo: Rcpp, RcppArmadillo (>= 0.12.8.0), mvtnorm
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-23 08:34:43 UTC; kkzh
Repository: CRAN
Date/Publication: 2026-05-23 10:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-23 15:12:32 UTC; unix
Archs: mets.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(IC,binreg)
S3method(IC,phreg)
S3method(IC,phreg.par)
S3method(bootstrap,timemets)
S3method(coef,binreg)
S3method(coef,biprobit)
S3method(coef,cor)
S3method(coef,doubleFG)
S3method(coef,mets.twostage)
S3method(coef,phreg)
S3method(coef,phreg.par)
S3method(coef,randomcif)
S3method(coef,randomcifrv)
S3method(coef,survd)
S3method(coef,timemets)
S3method(coef,twinlm)
S3method(coef,twinlm.strata)
S3method(compare,twinlm)
S3method(concordance,cor)
S3method(gof,phreg)
S3method(iid,twinlm)
S3method(lifetable,formula)
S3method(lifetable,matrix)
S3method(lines,phreg)
S3method(logLik,biprobit)
S3method(logLik,phreg.par)
S3method(logLik,twinlm)
S3method(logLik,twinlm.strata)
S3method(model.frame,bptwin)
S3method(model.frame,phreg.par)
S3method(model.frame,twinlm)
S3method(plot,BiRecurrent)
S3method(plot,bptwin)
S3method(plot,casewise)
S3method(plot,claytonoakes)
S3method(plot,covariance.recurrent)
S3method(plot,cumh)
S3method(plot,eventpois)
S3method(plot,gof.phreg)
S3method(plot,mets.twostage)
S3method(plot,phreg)
S3method(plot,predictphreg)
S3method(plot,resmean_phreg)
S3method(plot,timemets)
S3method(plot,twinlm)
S3method(plot,twinlm.strata)
S3method(predict,binreg)
S3method(predict,biprobit)
S3method(predict,mets.twostage)
S3method(predict,phreg)
S3method(predict,phreg.par)
S3method(print,Print)
S3method(print,binreg)
S3method(print,binregTSR)
S3method(print,biprobit)
S3method(print,casewise)
S3method(print,claytonoakes)
S3method(print,cor)
S3method(print,cumh)
S3method(print,daggregate)
S3method(print,do.twinlm.strata)
S3method(print,doubleFG)
S3method(print,dreg)
S3method(print,dtable)
S3method(print,gof.phreg)
S3method(print,mets.twostage)
S3method(print,pc.twostage)
S3method(print,phreg)
S3method(print,phreg.par)
S3method(print,phreg_rct)
S3method(print,predictphreg)
S3method(print,randomcif)
S3method(print,randomcifrv)
S3method(print,resmean_phreg)
S3method(print,summary.binregTSR)
S3method(print,summary.biprobit)
S3method(print,summary.bptwin)
S3method(print,summary.claytonoakes)
S3method(print,summary.cor)
S3method(print,summary.pc.twostage)
S3method(print,summary.phreg)
S3method(print,summary.survivalG)
S3method(print,summary.twinlm)
S3method(print,summary.twinlm.group)
S3method(print,summary.twostageREC)
S3method(print,survd)
S3method(print,timemets)
S3method(print,twinlm)
S3method(print,twinlm.strata)
S3method(print,twostageREC)
S3method(residuals,phreg)
S3method(score,biprobit)
S3method(score,twinlm)
S3method(score,twinlm.strata)
S3method(simulate,bptwin)
S3method(summary,binreg)
S3method(summary,binregTSR)
S3method(summary,biprobit)
S3method(summary,bptwin)
S3method(summary,casewise)
S3method(summary,claytonoakes)
S3method(summary,cor)
S3method(summary,cumh)
S3method(summary,doubleFG)
S3method(summary,dreg)
S3method(summary,gof.phreg)
S3method(summary,mets.twostage)
S3method(summary,pc.twostage)
S3method(summary,phreg)
S3method(summary,phreg_rct)
S3method(summary,randomcif)
S3method(summary,randomcifrv)
S3method(summary,recurrent)
S3method(summary,resmean_phreg)
S3method(summary,survd)
S3method(summary,survivalG)
S3method(summary,timemets)
S3method(summary,twinlm)
S3method(summary,twinlm.strata)
S3method(summary,twostageREC)
S3method(twostage,aalen)
S3method(twostage,cox.aalen)
S3method(twostage,coxph)
S3method(twostage,phreg)
S3method(vcov,binreg)
S3method(vcov,biprobit)
S3method(vcov,doubleFG)
S3method(vcov,phreg)
S3method(vcov,phreg.par)
S3method(vcov,survd)
S3method(vcov,timemets)
S3method(vcov,twinlm)
export("dby2<-")
export("dby<-")
export("dcut<-")
export("ddrop<-")
export("dfactor<-")
export("dkeep<-")
export("dlag<-")
export("dnames<-")
export("dnumeric<-")
export("drelev<-")
export("drelevel<-")
export("drename<-")
export("drm<-")
export("dsort<-")
export("dspline<-")
export("dtrans<-")
export("dtransform<-")
export(BinAugmentCifstrata)
export(Bootcovariancerecurrence)
export(BootcovariancerecurrenceS)
export(BootmediatorSurv)
export(Bootphreg)
export(CCbinomial.twostage)
export(ClaytonOakes)
export(Dbvn)
export(EVaddGam)
export(Effbinreg)
export(EventSplit)
export(FG_AugmentCifstrata)
export(FGprediid)
export(FastCoxPLstrataR)
export(GLprediid)
export(Grandom.cif)
export(IC)
export(IIDbaseline.cifreg)
export(IIDbaseline.phreg)
export(Interval)
export(LinSpline)
export(aalenMets)
export(aalenfrailty)
export(ace.family.design)
export(addCums)
export(alpha2kendall)
export(alpha2spear)
export(ascertained.pairs)
export(back2timereg)
export(basecumhaz)
export(basehazplot.phreg)
export(bicomprisk)
export(bicompriskData)
export(binomial.twostage)
export(binomial.twostage.time)
export(binreg)
export(binregATE)
export(binregATEbin)
export(binregCasewise)
export(binregG)
export(binregTSR)
export(binregt)
export(biprobit)
export(biprobit.time)
export(biprobit.vector)
export(blocksample)
export(bplot)
export(bplotdFG)
export(bptwin)
export(bptwin.time)
export(casewise)
export(casewise.bin)
export(casewise.test)
export(cause.pchazard.sim)
export(cif)
export(cif.yearslost)
export(cifreg)
export(cluster.index)
export(coarse.clust)
export(coefmat)
export(concordanceCor)
export(concordanceTwinACE)
export(concordanceTwostage)
export(cor.cif)
export(corsim.prostate)
export(corsim.prostate.random)
export(count.history)
export(count.historyVar)
export(countID)
export(covIntH1dM1IntH2dM2)
export(covarianceRecurrent)
export(covarianceRecurrentS)
export(covfr)
export(covfridstrata)
export(covfridstrataCov)
export(cpred)
export(cumContr)
export(cumODDS)
export(cumsum2strata)
export(cumsumidstratasum)
export(cumsumidstratasumCov)
export(cumsumstrata)
export(cumsumstratasum)
export(dInterval)
export(daggr)
export(daggregate)
export(dby)
export(dby2)
export(dbyr)
export(dcor)
export(dcount)
export(dcut)
export(ddrop)
export(deval)
export(deval2)
export(dfactor)
export(dhead)
export(diffstrata)
export(divide.conquer)
export(divide.conquer.timereg)
export(dkeep)
export(dlag)
export(dlev)
export(dlevel)
export(dlevels)
export(dlist)
export(dmean)
export(dmeansd)
export(dmvn)
export(dnames)
export(dnumeric)
export(doubleFGR)
export(dprint)
export(dquantile)
export(dreg)
export(drelev)
export(drelevel)
export(drename)
export(dreshape)
export(drm)
export(drop.specials)
export(drop.strata)
export(dsample)
export(dscalar)
export(dsd)
export(dsort)
export(dsort2)
export(dspline)
export(dstr)
export(dsubset)
export(dsum)
export(dsummary)
export(dtab)
export(dtable)
export(dtail)
export(dtrans)
export(dtransform)
export(dunique)
export(easy.binomial.twostage)
export(estimate)
export(eventpois)
export(extendCums)
export(familycluster.index)
export(familyclusterWithProbands.index)
export(fast.approx)
export(fast.cluster)
export(fast.pattern)
export(fast.reshape)
export(faster.reshape)
export(folds)
export(force.same.cens)
export(glm_IPTW)
export(gof)
export(gofG.phreg)
export(gofM.phreg)
export(gofZ.phreg)
export(grouptable)
export(haplo.surv.discrete)
export(headstrata)
export(ilap)
export(indexstrata)
export(indexstratarightR)
export(interval.logitsurv.discrete)
export(invsubdist)
export(ipw)
export(ipw2)
export(jumptimes)
export(kendall.ClaytonOakes.twin.ace)
export(kendall.normal.twin.ace)
export(km)
export(kumarsim)
export(kumarsimRCT)
export(lifecourse)
export(lifetable)
export(lin.approx)
export(logitATE)
export(logitIPCW)
export(logitIPCWATE)
export(logitSurv)
export(loglikMVN)
export(make.pairwise.design)
export(matdoubleindex)
export(matplot.mets.twostage)
export(mdi)
export(mediatorSurv)
export(medweight)
export(mets.options)
export(mlogit)
export(mystrata)
export(mystrata2index)
export(nonparcuminc)
export(normalATE)
export(npc)
export(object.defined)
export(or.cif)
export(or2prob)
export(p11.binomial.twostage.RV)
export(pairRisk)
export(pbvn)
export(pcif)
export(phreg)
export(phreg.par)
export(phregR)
export(phreg_IPTW)
export(phreg_rct)
export(piecewise.data)
export(piecewise.twostage)
export(plack.cif)
export(plack.cif2)
export(plotConfRegion)
export(plotConfRegionSE)
export(plotSurvd)
export(plotcr)
export(plotstrata)
export(pmvn)
export(pre.cifs)
export(pred.cif.boot)
export(predictCumhaz)
export(predictGLM)
export(predictPairPlack)
export(predictSurvd)
export(predictdFG)
export(predictlogitSurvd)
export(predictmlogit)
export(prob.exceed.recurrent)
export(prob.exceedBiRecurrent)
export(prob.exceedBiRecurrentStrata)
export(prob.exceedRecurrent)
export(prob.exceedRecurrentStrata)
export(procform)
export(procform3)
export(procformdata)
export(random.cif)
export(randomDes)
export(rchaz)
export(rchazC)
export(rcrisk)
export(rcrisks)
export(readPhreg)
export(readmargsurv)
export(recmarg)
export(recreg)
export(recregIPCW)
export(recurrentMarginal)
export(recurrentMarginalAIPCW)
export(recurrentMarginalAIPCWdata)
export(recurrentMarginalIPCW)
export(resmean.phreg)
export(resmeanATE)
export(resmeanIPCW)
export(revcumsum)
export(revcumsum2strata)
export(revcumsum2stratafdN)
export(revcumsumidstratasum)
export(revcumsumidstratasumCov)
export(revcumsumstrata)
export(revcumsumstratasum)
export(rmst.phreg)
export(rmstATE)
export(rmstIPCW)
export(rmvn)
export(robust.basehaz.phreg)
export(robust.phreg)
export(rpch)
export(rr.cif)
export(scalecumhaz)
export(scoreMVN)
export(setup.cif)
export(showfitsim)
export(sim.base)
export(sim.cause.cox)
export(sim.cif)
export(sim.cifs)
export(sim.cifsRestrict)
export(sim.cox)
export(simAalenFrailty)
export(simBinFam)
export(simBinFam2)
export(simBinPlack)
export(simClaytonOakes)
export(simClaytonOakes.family.ace)
export(simClaytonOakes.twin.ace)
export(simClaytonOakesLam)
export(simClaytonOakesWei)
export(simCompete.simple)
export(simCompete.twin.ace)
export(simCox)
export(simFrailty.simple)
export(simMultistate)
export(simRecurrent)
export(simRecurrentII)
export(simRecurrentTS)
export(simSurvFam)
export(simTTP)
export(simbinClaytonOakes.family.ace)
export(simbinClaytonOakes.pairs)
export(simbinClaytonOakes.twin.ace)
export(simlogitSurvd)
export(simnordic)
export(simnordic.random)
export(simrchaz)
export(simsubdist)
export(simul.cifs)
export(slope.process)
export(squareintHdM)
export(strataAugment)
export(strataC)
export(subdist)
export(summaryGLM)
export(summaryTimeobject)
export(sumstrata)
export(surv.boxarea)
export(survival.twostage)
export(survivalG)
export(tailstrata)
export(test.conc)
export(tetrachoric)
export(tie.breaker)
export(twin.clustertrunc)
export(twin.polygen.design)
export(twinlm)
export(twinlm.strata)
export(twinlm.time)
export(twinsim)
export(twostage)
export(twostageMLE)
export(twostageREC)
export(vecAllStrata)
import(Rcpp)
import(mvtnorm)
import(splines)
import(stats)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.list)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,title)
importFrom(lava,"%++%")
importFrom(lava,"%ni%")
importFrom(lava,"addvar<-")
importFrom(lava,"constrain<-")
importFrom(lava,"covariance<-")
importFrom(lava,"distribution<-")
importFrom(lava,"intercept<-")
importFrom(lava,"kill<-")
importFrom(lava,"latent<-")
importFrom(lava,"parameter<-")
importFrom(lava,"regression<-")
importFrom(lava,Col)
importFrom(lava,Expand)
importFrom(lava,IC)
importFrom(lava,Inverse)
importFrom(lava,Model)
importFrom(lava,blockdiag)
importFrom(lava,bootstrap)
importFrom(lava,cancel)
importFrom(lava,compare)
importFrom(lava,confband)
importFrom(lava,constraints)
importFrom(lava,covariance)
importFrom(lava,coxWeibull.lvm)
importFrom(lava,devcoords)
importFrom(lava,endogenous)
importFrom(lava,estimate)
importFrom(lava,eventTime)
importFrom(lava,expit)
importFrom(lava,getoutcome)
importFrom(lava,gof)
importFrom(lava,iid)
importFrom(lava,information)
importFrom(lava,latent)
importFrom(lava,lava.options)
importFrom(lava,logit)
importFrom(lava,lvm)
importFrom(lava,multigroup)
importFrom(lava,pars)
importFrom(lava,regression)
importFrom(lava,revdiag)
importFrom(lava,score)
importFrom(lava,trim)
importFrom(lava,twostage)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,is.Surv)
importFrom(survival,strata)
importFrom(timereg,Cpred)
importFrom(timereg,Event)
importFrom(timereg,predict.two.stage)
importFrom(timereg,sindex.prodlim)
importFrom(timereg,two.stage)
importFrom(utils,capture.output)
importFrom(utils,getS3method)
importFrom(utils,glob2rx)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(mets, .registration=TRUE)

<!-- badges: start -->
  [![travis](https://travis-ci.com/kkholst/mets.svg?branch=master)](https://travis-ci.com/kkholst/mets)
  [![coverage](https://codecov.io/github/kkholst/mets/coverage.svg?branch=master)](https://codecov.io/github/kkholst/mets?branch=master)
  [![cran](https://www.r-pkg.org/badges/version-last-release/mets)](https://CRAN.R-project.org/package=mets)
  [![cran-dl](https://cranlogs.r-pkg.org/badges/mets)](https://cranlogs.r-pkg.org/downloads/total/last-month/mets)
<!-- badges: end -->




# Multivariate Event Times (`mets`) <img src=man/figures/timerrr.png align="right" height="150">

Implementation of various statistical models for multivariate
event history data <10.1007/s10985-013-9244-x>. Including multivariate
cumulative incidence models <10.1002/sim.6016>, and  bivariate random
effects probit models (Liability models) <10.1016/j.csda.2015.01.014>.
Also contains two-stage binomial modelling that can do pairwise odds-ratio
dependence modelling based marginal logistic regression models. This is an
alternative to the alternating logistic regression approach (ALR).


## Installation


```r
install.packages("mets")
```

The development version may be installed directly from github
(requires [Rtools](https://cran.r-project.org/bin/windows/Rtools/) on windows and [development tools](https://cran.r-project.org/bin/macosx/tools/) (+Xcode) for Mac OS
X):


```r
remotes::install_github("kkholst/mets", dependencies="Suggests")
```

or to get development version


```r
devtools::install_github("kkholst/mets",ref="develop")
```


## Citation

To cite the `mets` package please use one of the following references

> Thomas H. Scheike and Klaus K. Holst and Jacob B. Hjelmborg (2013).
> Estimating heritability for cause specific mortality based on twin studies.
> Lifetime Data Analysis. <http://dx.doi.org/10.1007/s10985-013-9244-x>

> Klaus K. Holst and Thomas H. Scheike Jacob B. Hjelmborg (2015).
> The Liability Threshold Model for Censored Twin Data.
> Computational Statistics and Data Analysis. <http://dx.doi.org/10.1016/j.csda.2015.01.014>

BibTeX:

    @Article{,
      title={Estimating heritability for cause specific mortality based on twin studies},
      author={Scheike, Thomas H. and Holst, Klaus K. and Hjelmborg, Jacob B.},
      year={2013},
      issn={1380-7870},
      journal={Lifetime Data Analysis},
      doi={10.1007/s10985-013-9244-x},
      url={http://dx.doi.org/10.1007/s10985-013-9244-x},
      publisher={Springer US},
      keywords={Cause specific hazards; Competing risks; Delayed entry;
    	    Left truncation; Heritability; Survival analysis},
      pages={1-24},
      language={English}
    }

    @Article{,
      title={The Liability Threshold Model for Censored Twin Data},
      author={Holst, Klaus K. and Scheike, Thomas H. and Hjelmborg, Jacob B.},
      year={2015},
      doi={10.1016/j.csda.2015.01.014},
      url={http://dx.doi.org/10.1016/j.csda.2015.01.014},
      journal={Computational Statistics and Data Analysis}
    }


## Examples


```r
library("mets")
#> Loading required package: timereg
#> Loading required package: survival
#> Loading required package: lava
#> mets version 1.2.8

data(prt) ## Prostate data example (sim)

## Bivariate competing risk, concordance estimates
p33 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                  data=prt, cause=c(2,2), return.data=1, prodlim=TRUE)
#> Strata 'DZ'
#> Strata 'MZ'

p33dz <- p33$model$"DZ"$comp.risk
p33mz <- p33$model$"MZ"$comp.risk

## Probability weights based on Aalen's additive model
prtw <- ipw(Surv(time,status==0)~country, data=prt,
            cluster="id",weight.name="w")

## Marginal model (wrongly ignoring censorings)
bpmz <- biprobit(cancer~1 + cluster(id),
                 data=subset(prt,zyg=="MZ"), eqmarg=TRUE)

## Extended liability model
bpmzIPW <- biprobit(cancer~1 + cluster(id),
                    data=subset(prtw,zyg=="MZ"),
                    weight="w")
smz <- summary(bpmzIPW)

## Concordance
plot(p33mz,ylim=c(0,0.1),axes=FALSE,automar=FALSE,atrisk=FALSE,background=TRUE,background.fg="white")
axis(2); axis(1)

abline(h=smz$prob["Concordance",],lwd=c(2,1,1),col="darkblue")
## Wrong estimates:
abline(h=summary(bpmz)$prob["Concordance",],lwd=c(2,1,1),col="lightgray", lty=2)
```

<img src="man/figures/ex1-1.png" title="plot of chunk ex1" alt="plot of chunk ex1" width="60%" />

- [Multivariate Event Times (`mets`)
  <img src="man/figures/logo.png" align="right" height="250" style="float:right; height:250px;"  alt="mets website">](#multivariate-event-times-mets-)
  - [Installation](#installation)
  - [Citation](#citation)
  - [Examples: Twins Polygenic
    modelling](#examples-twins-polygenic-modelling)
  - [Examples: Twins Polygenic modelling time-to-events
    Data](#examples-twins-polygenic-modelling-time-to-events-data)
  - [Examples: Twins Concordance for time-to-events
    Data](#examples-twins-concordance-for-time-to-events-data)
  - [Examples: Cox model, RMST](#examples-cox-model-rmst)
  - [Examples: Cox model IPTW](#examples-cox-model-iptw)
  - [Examples: Competing risks regression, Binomial
    Regression](#examples-competing-risks-regression-binomial-regression)
  - [Examples: Competing risks regression, Fine-Gray/Logistic
    link](#examples-competing-risks-regression-fine-graylogistic-link)
  - [Examples: Marginal mean for recurrent
    events](#examples-marginal-mean-for-recurrent-events)
  - [Examples: Ghosh-Lin for recurrent
    events](#examples-ghosh-lin-for-recurrent-events)
  - [Examples: Fixed time modelling for recurrent
    events](#examples-fixed-time-modelling-for-recurrent-events)
  - [Examples: Cumulative Medical Cost](#examples-cumulative-medical-cost)
  - [Examples: Regression for RMST/Restricted mean survival for survival
    and competing risks using
    IPCW](#examples-regression-for-rmstrestricted-mean-survival-for-survival-and-competing-risks-using-ipcw)
  - [Examples: Average treatment effects (ATE) for survival or competing
    risks](#examples-average-treatment-effects-ate-for-survival-or-competing-risks)
  - [Examples: While Alive estimands for recurrent
    events](#examples-while-alive-estimands-for-recurrent-events)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start-->

[![R-CMD-check](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml)
[![coverage](https://codecov.io/github/kkholst/mets/coverage.svg)](https://app.codecov.io/github/kkholst/mets?branch=master)
[![cran](https://www.r-pkg.org/badges/version-last-release/mets)](https://CRAN.R-project.org/package=mets)
[![cran-dl](https://cranlogs.r-pkg.org/badges/mets)](https://cranlogs.r-pkg.org/downloads/total/last-month/mets)

<!-- badges: end -->

# Multivariate Event Times (`mets`) <img src="man/figures/logo.png" align="right" height="250" style="float:right; height:250px;"  alt="mets website">

Implementation of various statistical models for multivariate event
history data <doi:10.1007/s10985-013-9244-x>. Including multivariate
cumulative incidence models <doi:10.1002/sim.6016>, and bivariate random
effects probit models (Liability models)
<doi:10.1016/j.csda.2015.01.014>. Modern methods for survival analysis,
including regression modelling (Cox, Fine-Gray, Ghosh-Lin, Binomial
regression) with fast computation of influence functions. Restricted
mean survival time regression and years lost for competing risks.
Average treatment effects and G-computation. All functions can be used
with clusters and will work for large data.

## Installation

``` r
install.packages("mets")
```

The development version may be installed directly from github (requires
[Rtools](https://cran.r-project.org/bin/windows/Rtools/) on windows and
[development tools](https://mac.r-project.org/tools/) (+Xcode) for Mac
OS X):

``` r
remotes::install_github("kkholst/mets", dependencies="Suggests")
```

or to get development version

``` r
remotes::install_github("kkholst/mets",ref="develop")
```

## Citation

To cite the `mets` package please use one of the following references

> Thomas H. Scheike and Klaus K. Holst (2022). A Practical Guide to
> Family Studies with Lifetime Data. Annual Review of Statistics and Its
> Application 9, pp. 47-69. doi:
> <http://dx.doi.org/10.1146/annurev-statistics-040120-024253>

> Thomas H. Scheike and Klaus K. Holst and Jacob B. Hjelmborg (2013).
> Estimating heritability for cause specific mortality based on twin
> studies. Lifetime Data Analysis.
> <http://dx.doi.org/10.1007/s10985-013-9244-x>

> Klaus K. Holst and Thomas H. Scheike Jacob B. Hjelmborg (2015). The
> Liability Threshold Model for Censored Twin Data. Computational
> Statistics and Data Analysis.
> <http://dx.doi.org/10.1016/j.csda.2015.01.014>

BibTeX:

    @Article{,
      title = {A Practical Guide to Family Studies with Lifetime Data},
      author = {Thomas H. Scheike and Klaus K. Holst},
      year = {2014},
      volume = {9},
      pages = {47-69},
      journal = {Annual Review of Statistics and Its Application},
      doi = {10.1146/annurev-statistics-040120-024253},
    }

    @Article{,
      title={Estimating heritability for cause specific mortality based on twin studies},
      author={Scheike, Thomas H. and Holst, Klaus K. and Hjelmborg, Jacob B.},
      year={2013},
      issn={1380-7870},
      journal={Lifetime Data Analysis},
      doi={10.1007/s10985-013-9244-x},
      url={http://dx.doi.org/10.1007/s10985-013-9244-x},
      publisher={Springer US},
      keywords={Cause specific hazards; Competing risks; Delayed entry;
            Left truncation; Heritability; Survival analysis},
      pages={1-24},
      language={English}
    }

    @Article{,
      title={The Liability Threshold Model for Censored Twin Data},
      author={Holst, Klaus K. and Scheike, Thomas H. and Hjelmborg, Jacob B.},
      year={2015},
      doi={10.1016/j.csda.2015.01.014},
      url={http://dx.doi.org/10.1016/j.csda.2015.01.014},
      journal={Computational Statistics and Data Analysis}
    }

## Examples: Twins Polygenic modelling

First considering standard twin modelling (ACE, AE, ADE, and more
models)

``` r
# simulated data with pairs of observations in twins on long #data format
set.seed(1)
d <- twinsim(1000, b1=c(1,-1), b2=c(), acde=c(1,1,0,1))
# Polygenic model with Additive genetic effects, and shared and individual environmental effects (ACE)
ace <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
ace
#>        Estimate Std. Error Z value  Pr(>|z|)
#> y     -0.019439   0.041817 -0.4649     0.642
#> sd(A)  0.902004   0.203739  4.4273 9.544e-06
#> sd(C)  1.137025   0.132852  8.5586 < 2.2e-16
#> sd(E)  1.728992   0.037408 46.2194 < 2.2e-16
#> 
#> MZ-pairs DZ-pairs 
#>     1000     1000 
#> 
#> Variance decomposition:
#>   Estimate 2.5%    97.5%  
#> A 0.15966  0.01867 0.30065
#> C 0.25370  0.13920 0.36820
#> E 0.58664  0.53677 0.63650
#> 
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.15966  0.01867 0.30065
#> 
#>                        Estimate 2.5%    97.5%  
#> Correlation within MZ: 0.41336  0.36229 0.46196
#> Correlation within DZ: 0.33353  0.27933 0.38561
#> 
#> 'log Lik.' -8779.953 (df=4)
#> AIC: 17567.91 
#> BIC: 17590.31
# An AE-model could be fitted as
ae <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id", type="ae")
# AIC
AIC(ae)-AIC(ace)
#> [1] 15.20656
# To adjust for the covariates we simply alter the formula statement
ace2 <- twinlm(y ~ x1+x2, data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
 ## Summary/GOF
summary(ace2)
#>        Estimate Std. Error  Z value Pr(>|z|)
#> y     -0.026049   0.034844  -0.7476   0.4547
#> sd(A)  1.066060   0.072890  14.6256   <2e-16
#> sd(C)  0.980740   0.073569  13.3309   <2e-16
#> sd(E)  0.979980   0.021887  44.7736   <2e-16
#> y~x1   1.006963   0.021900  45.9807   <2e-16
#> y~x2  -0.993802   0.021962 -45.2512   <2e-16
#> 
#> MZ-pairs DZ-pairs 
#>     1000     1000 
#> 
#> Variance decomposition:
#>   Estimate 2.5%    97.5%  
#> A 0.37156  0.27300 0.47012
#> C 0.31446  0.22643 0.40250
#> E 0.31398  0.28381 0.34414
#> 
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.37156  0.27300 0.47012
#> 
#>                        Estimate 2.5%    97.5%  
#> Correlation within MZ: 0.68602  0.65467 0.71502
#> Correlation within DZ: 0.50024  0.45538 0.54257
#> 
#> 'log Lik.' -7449.697 (df=6)
#> AIC: 14911.39 
#> BIC: 14945
```

## Examples: Twins Polygenic modelling time-to-events Data

In the context of time-to-events data we consider the “Liability
Threshold model” with IPCW adjustment for censoring.

First we fit the bivariate probit model (same marginals in MZ and DZ
twins but different correlation parameter). Here we evaluate the risk of
getting cancer before the last double cancer event (95 years)

``` r
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in

b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b0)
#> 
#>                Estimate   Std.Err        Z   p-value    
#> (Intercept)   -1.348188  0.026276 -51.3086 < 2.2e-16 ***
#> atanh(rho) MZ  0.735992  0.087838   8.3789 < 2.2e-16 ***
#> atanh(rho) DZ  0.353023  0.068234   5.1737 2.295e-07 ***
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#>  Total MZ/DZ Complete pairs MZ/DZ
#>  1994/3618   997/1809            
#> 
#>                            Estimate 2.5%    97.5%  
#> Tetrachoric correlation MZ 0.62672  0.51081 0.72024
#> Tetrachoric correlation DZ 0.33905  0.21584 0.45164
#> 
#> MZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.03504  0.02779 0.04409
#> Casewise Concordance 0.39458  0.31876 0.47584
#> Marginal             0.08880  0.08086 0.09743
#> Rel.Recur.Risk       4.44351  3.50521 5.38182
#> log(OR)              2.34131  1.87105 2.81157
#> DZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.01952  0.01449 0.02625
#> Casewise Concordance 0.21983  0.16667 0.28415
#> Marginal             0.08880  0.08086 0.09743
#> Rel.Recur.Risk       2.47556  1.81096 3.14016
#> log(OR)              1.23088  0.81020 1.65156
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.57533  0.25790 0.89276
#> 
#> 
#> Event of interest before time 95
```

Liability threshold model with ACE random effects structure

``` r
b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
           cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b1)
#> 
#>             Estimate  Std.Err        Z p-value    
#> (Intercept) -2.20664  0.16463 -13.4034  <2e-16 ***
#> log(var(A))  0.43260  0.39149   1.1050  0.2691    
#> log(var(C)) -1.98289  2.52342  -0.7858  0.4320    
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#>  Total MZ/DZ Complete pairs MZ/DZ
#>  1994/3618   997/1809            
#> 
#>                    Estimate 2.5%     97.5%   
#> A                   0.57533  0.25790  0.89276
#> C                   0.05139 -0.20836  0.31114
#> E                   0.37328  0.26874  0.47782
#> MZ Tetrachoric Cor  0.62672  0.51081  0.72024
#> DZ Tetrachoric Cor  0.33905  0.21584  0.45164
#> 
#> MZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.03504  0.02779 0.04409
#> Casewise Concordance 0.39458  0.31876 0.47584
#> Marginal             0.08880  0.08086 0.09743
#> Rel.Recur.Risk       4.44351  3.50520 5.38182
#> log(OR)              2.34131  1.87104 2.81157
#> DZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.01952  0.01449 0.02625
#> Casewise Concordance 0.21983  0.16667 0.28415
#> Marginal             0.08880  0.08086 0.09743
#> Rel.Recur.Risk       2.47556  1.81096 3.14016
#> log(OR)              1.23088  0.81020 1.65156
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.57533  0.25790 0.89276
#> 
#> 
#> Event of interest before time 95
```

## Examples: Twins Concordance for time-to-events Data

``` r

data(prt) ## Prostate data example (sim)

## Bivariate competing risk, concordance estimates
p33 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                  data=prt, cause=c(2,2), return.data=1, prodlim=TRUE)
#> Strata 'DZ'
#> Strata 'MZ'

p33dz <- p33$model$"DZ"$comp.risk
p33mz <- p33$model$"MZ"$comp.risk

## Probability weights based on Aalen's additive model (same censoring within pair)
prtw <- ipw(Surv(time,status==0)~country+zyg, data=prt,
            obs.only=TRUE, same.cens=TRUE, 
            cluster="id", weight.name="w")

## Marginal model (wrongly ignoring censorings)
bpmz <- biprobit(cancer~1 + cluster(id), 
                 data=subset(prt,zyg=="MZ"), eqmarg=TRUE)

## Extended liability model
bpmzIPW <- biprobit(cancer~1 + cluster(id),
                    data=subset(prtw,zyg=="MZ"),
                    weights="w")
smz <- summary(bpmzIPW)

## Concordance
plot(p33mz,ylim=c(0,0.1),axes=FALSE, automar=FALSE,atrisk=FALSE,background=TRUE,background.fg="white")
axis(2); axis(1)

abline(h=smz$prob["Concordance",],lwd=c(2,1,1),col="darkblue")
## Wrong estimates:
abline(h=summary(bpmz)$prob["Concordance",],lwd=c(2,1,1),col="lightgray",lty=2)
```

<img src="man/figures/README-prtcomprisk-1.png" width="50%" />

## Examples: Cox model, RMST

We can fit the Cox model and compute many useful summaries, such as
restricted mean survival and standardized treatment effects
(G-estimation). First estimating the standardized survival

``` r
 data(bmt)
 bmt$time <- bmt$time+runif(408)*0.001
 bmt$event <- (bmt$cause!=0)*1
 dfactor(bmt) <- tcell.f~tcell

 ss <- phreg(Surv(time,event)~tcell.f+platelet+age,bmt) 
 summary(survivalG(ss,bmt,50))
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%    P-value
#> risk0   0.6539 0.02708 0.6008 0.7070 9.119e-129
#> risk1   0.5641 0.05973 0.4470 0.6811  3.600e-21
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err    2.5%   97.5% P-value
#> ps0 -0.08982 0.06293 -0.2132 0.03352  0.1535
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>         Estimate  Std.Err       2.5%      97.5%   P-value
#> [ps0] -0.1477619 0.109562 -0.3624994 0.06697567 0.1774462
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.8626365 0.6959347 1.0692695 
#> 
#> Average Treatment effect:  survival-difference (G-estimator) :
#>       Estimate    Std.Err        2.5%     97.5%   P-value
#> ps0 0.08981829 0.06292811 -0.03351854 0.2131551 0.1534889
#> 
#> Average Treatment effect: 1-G (survival)-ratio (G-estimator) :
#> log-ratio: 
#>       Estimate   Std.Err        2.5%     97.5%   P-value
#> [ps0] 0.230711 0.1504459 -0.06415759 0.5255796 0.1251491
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 1.2594952 0.9378572 1.6914390

 sst <- survivalGtime(ss,bmt,n=50)
 plot(sst,type=c("survival","risk","survival.ratio")[1])
```

<img src="man/figures/README-coxrmst-1.png" width="50%" />

Based on the phreg we can also compute the restricted mean survival time
and years lost (via Kaplan-Meier estimates). The function does it for
all times at once and can be plotted as restricted mean survival or
years lost at the different time horizons

``` r
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 
 rm1 <- resmean_phreg(out1, times=c(50))
 summary(rm1)
#>                     strata times    rmean se.rmean    lower    upper years.lost
#> tcell=0, platelet=0      0    50 20.48245 1.411055 17.89542 23.44348   29.51755
#> tcell=0, platelet=1      1    50 28.33071 2.196175 24.33733 32.97934   21.66929
#> tcell=1, platelet=0      2    50 22.74596 4.053717 16.04005 32.25544   27.25404
#> tcell=1, platelet=1      3    50 26.11565 4.230688 19.01112 35.87517   23.88435
 par(mfrow=c(1, 2))
 plot(rm1,se=1)
 plot(rm1,years.lost=TRUE,se=1)
```

<img src="man/figures/README-unnamed-chunk-2-1.png" width="50%" />

For competing risks the years lost can be decomposed into different
causes and is based on the integrated Aalen-Johansen estimators for the
different strata

``` r
 ## years.lost decomposed into causes
drm1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=50)
 par(mfrow=c(1,2)); plot(drm1,cause=1,se=1); title(main="Cause 1"); plot(drm1,cause=2,se=1); title(main="Cause 2")
```

<img src="man/figures/README-yearslostcause-1.png" width="50%" />

``` r
 summary(drm1)
#> $estimate
#>                     strata times   intF_1    intF_2 se.intF_1 se.intF_2
#> tcell=0, platelet=0      0    50 21.36784  8.149711  1.476647  1.094520
#> tcell=0, platelet=1      1    50 12.97924  8.690047  2.047516  1.712441
#> tcell=1, platelet=0      2    50 12.64543 14.608610  4.089981  3.730259
#> tcell=1, platelet=1      3    50 11.80934 12.075008  3.673701  3.890207
#>                     total.years.lost lower_intF_1 upper_intF_1 lower_intF_2
#> tcell=0, platelet=0         29.51755    18.661106     24.46717     6.263606
#> tcell=0, platelet=1         21.66929     9.527297     17.68191     5.905902
#> tcell=1, platelet=0         27.25404     6.708487     23.83649     8.856404
#> tcell=1, platelet=1         23.88435     6.418453     21.72807     6.421784
#>                     upper_intF_2
#> tcell=0, platelet=0     10.60376
#> tcell=0, platelet=1     12.78669
#> tcell=1, platelet=0     24.09685
#> tcell=1, platelet=1     22.70487
```

Computations are again done for all time horizons at once as illustrated
in the plot.

## Examples: Cox model IPTW

We can fit the Cox model with inverse probability of treatment weights
based on logistic regression. The treatment weights can be
time-dependent and then multiplicative weights are applied (see details
and vignette).

``` r
data(bmt)
bmt$time <- bmt$time+runif(408)*0.001
bmt$id <- seq_len(nrow(bmt))
bmt$event <- (bmt$cause!=0)*1
dfactor(bmt) <- tcell.f~tcell

fit <- phreg_IPTW(Surv(time,event)~tcell.f+cluster(id),data=bmt,treat.model=tcell.f~platelet+age) 
summary(fit)
#> 
#>    n events
#>  408    248
#> 
#>  408 clusters
#> coeffients:
#>           Estimate      S.E.   dU^-1/2 P-value
#> tcell.f1 -0.108497  0.199556  0.089653  0.5867
#> 
#> exp(coeffients):
#>          Estimate    2.5%  97.5%
#> tcell.f1  0.89718 0.60676 1.3266
head(IC(fit))
#>    tcell.f1
#> 1 -1.639241
#> 2 -1.669074
#> 3 -1.749761
#> 4 -1.745988
#> 5 -1.625416
#> 6 -1.793372
```

## Examples: Competing risks regression, Binomial Regression

We can fit the logistic regression model at a specific time-point with
IPCW adjustment

``` r
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
# logistic regression with IPCW binomial regression 
out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
summary(out)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.180371  0.126757 -0.428811  0.068068  0.1547
#> tcell       -0.418682  0.345438 -1.095729  0.258364  0.2255
#> platelet    -0.436959  0.240977 -0.909266  0.035349  0.0698
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.83496 0.65128 1.0704
#> tcell        0.65791 0.33430 1.2948
#> platelet     0.64600 0.40282 1.0360
head(IC(out))
#>           [,1]     [,2]    [,3]
#> [1,] -2.834084 1.633524 2.52025
#> [2,] -2.834084 1.633524 2.52025
#> [3,] -2.834084 1.633524 2.52025
#> [4,] -2.834084 1.633524 2.52025
#> [5,] -2.834084 1.633524 2.52025
#> [6,] -2.834084 1.633524 2.52025
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
#>        pred         se     lower     upper
#> 1 0.3503890 0.04848653 0.2553554 0.4454226
#> 2 0.2619201 0.06969710 0.1253138 0.3985265
```

## Examples: Competing risks regression, Fine-Gray/Logistic link

We can fit the Fine-Gray model and the logit-link competing risks model
(using IPCW adjustment). Starting with the logit-link model

``` r
data(bmt)
bmt$time <- bmt$time+runif(nrow(bmt))*0.01
bmt$id <- 1:nrow(bmt)
## logistic link  OR interpretation
 or=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
summary(or)
#> 
#>    n events
#>  408    161
#> 
#>  408 clusters
#> coeffients:
#>           Estimate      S.E.   dU^-1/2 P-value
#> platelet -0.454572  0.235415  0.187997  0.0535
#> age       0.390181  0.097675  0.083636  0.0001
#> 
#> exp(coeffients):
#>          Estimate    2.5%  97.5%
#> platelet  0.63472 0.40013 1.0069
#> age       1.47725 1.21987 1.7889
par(mfrow=c(1,2))
 ## to see baseline 
plot(or)

 # predictions 
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pll <- predict(or,nd)
plot(pll)
```

<img src="man/figures/README-cifreg-1.png" width="50%" />

Similarly, the Fine-Gray model can be estimated using IPCW adjustment

``` r
 ## Fine-Gray model
 fg=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1,propodds=NULL)
 summary(fg)
#> 
#>    n events
#>  408    161
#> 
#>  408 clusters
#> coeffients:
#>           Estimate      S.E.   dU^-1/2 P-value
#> platelet -0.424749  0.180772  0.187820  0.0188
#> age       0.341971  0.079862  0.086284  0.0000
#> 
#> exp(coeffients):
#>          Estimate    2.5%  97.5%
#> platelet  0.65393 0.45884 0.9320
#> age       1.40772 1.20375 1.6462
## baselines 
plot(fg)
```

<img src="man/figures/README-finegrary-1.png" width="50%" />

``` r
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pfg <- predict(fg,nd,se=1)
plot(pfg,se=1)
```

<img src="man/figures/README-finegrary-2.png" width="50%" />

``` r

## influence functions of regression coefficients
head(iid(fg))
#>         platelet           age
#> [1,] 0.004953478  0.0001245648
#> [2,] 0.005348496 -0.0022341772
#> [3,] 0.006069271 -0.0087212019
#> [4,] 0.006043180 -0.0084186443
#> [5,] 0.004732097  0.0011839243
#> [6,] 0.006331457 -0.0121685409
```

and we can get standard errors for predictions based on the influence
functions of the baseline and the regression coefficients (these are
used in the predict function)

``` r
baseid <- iidBaseline(fg,time=40)
FGprediid(baseid,nd)
#>           pred     se-log     lower     upper
#> [1,] 0.2787465 0.23977109 0.1742272 0.4459672
#> [2,] 0.4506249 0.07265694 0.3908134 0.5195901
```

further G-estimation can be done

``` r
 dfactor(bmt) <- tcell.f~tcell
 fg1 <- cifreg(Event(time,cause)~tcell.f+platelet+age,bmt,cause=1,propodds=NULL)
 summary(survivalG(fg1,bmt,50))
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.4332 0.02749 0.3793 0.4871 6.331e-56
#> risk1   0.2726 0.05861 0.1577 0.3875 3.301e-06
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err   2.5%    97.5% P-value
#> ps0  -0.1606 0.06351 -0.285 -0.03609 0.01146
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>        Estimate   Std.Err       2.5%       97.5%    P-value
#> [ps0] -0.463091 0.2211651 -0.8965667 -0.02961528 0.03627159
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.6293354 0.4079679 0.9708190
```

## Examples: Marginal mean for recurrent events

We can estimate the expected number of events non-parametrically and get
standard errors for this estimator

``` r
data(hfactioncpx12)
dtable(hfactioncpx12,~status)
#> 
#> status
#>    0    1    2 
#>  617 1391  124

gl1 <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1,times=1:5)
#> [[1]]
#>       new.time      mean         se   CI-2.5% CI-97.5% strata
#> 325          1 0.8737156 0.06783343 0.7503858 1.017315      0
#> 555          2 1.5718563 0.09572955 1.3949953 1.771140      0
#> 682          3 2.1184963 0.11385721 1.9066915 2.353829      0
#> 748          4 2.6815219 0.15451005 2.3951619 3.002118      0
#> 748.1        5 2.6815219 0.15451005 2.3951619 3.002118      0
#> 
#> [[2]]
#>       new.time      mean         se   CI-2.5%  CI-97.5% strata
#> 284          1 0.7815557 0.06908585 0.6572305 0.9293989      1
#> 499          2 1.4534055 0.10315606 1.2646561 1.6703258      1
#> 601          3 1.9240624 0.12165771 1.6998008 2.1779119      1
#> 645          4 2.3134997 0.14963892 2.0380418 2.6261880      1
#> 645.1        5 2.3134997 0.14963892 2.0380418 2.6261880      1
plot(gl1,se=1)
```

<img src="man/figures/README-recurrentevents-1.png" width="50%" />

## Examples: Ghosh-Lin for recurrent events

We can fit the Ghosh-Lin model for the expected number of events
observed before dying (using IPCW adjustment and get predictions)

``` r
data(hfactioncpx12)
dtable(hfactioncpx12,~status)
#> 
#> status
#>    0    1    2 
#>  617 1391  124

gl1 <- recreg(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1)
#> 
#>     n events
#>  2132   1391
#> 
#>  741 clusters
#> coeffients:
#>             Estimate      S.E.   dU^-1/2 P-value
#> treatment1 -0.110404  0.078656  0.053776  0.1604
#> 
#> exp(coeffients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.89547 0.76754 1.0447

## influence functions of regression coefficients
head(iid(gl1))
#>      treatment1
#> 1 -1.266428e-04
#> 2 -6.112340e-04
#> 3  2.885192e-03
#> 4  1.308207e-03
#> 5  5.404664e-05
#> 6  2.229380e-03
```

and we can get standard errors for predictions based on the influence
functions of the baseline and the regression coefficients

``` r
 nd=data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
 pfg <- predict(gl1,nd,se=1)
 summary(pfg,times=1:5)
#> $pred
#>              Lamt     Lamt     Lamt     Lamt     Lamt
#> strata0 0.8573256 1.592252 2.121181 2.635437 2.635437
#> strata0 0.7677110 1.425817 1.899458 2.359959 2.359959
#> 
#> $se.pred
#>       seLamt     seLamt    seLamt    seLamt    seLamt
#> 1 0.05719895 0.08818784 0.1096157 0.1429941 0.1429941
#> 2 0.05763288 0.09495475 0.1184567 0.1484200 0.1484200
#> 
#> $lower
#>              [,1]     [,2]     [,3]     [,4]     [,5]
#> strata0 0.7522383 1.428458 1.916860 2.369561 2.369561
#> strata0 0.6626698 1.251343 1.680916 2.086276 2.086276
#> 
#> $upper
#>              [,1]     [,2]     [,3]     [,4]     [,5]
#> strata0 0.9770936 1.774827 2.347281 2.931145 2.931145
#> strata0 0.8894025 1.624617 2.146415 2.669546 2.669546
#> 
#> $times
#> [1] 1 2 3 4 5
#> 
#> attr(,"class")
#> [1] "summarypredictrecreg"
 plot(pfg,se=1)
```

<img src="man/figures/README-unnamed-chunk-5-1.png" width="50%" />

The influence functions of the baseline and regression coefficients at a
specific time-point can be obtained

``` r
baseid <- iidBaseline(gl1,time=2)
dd <- data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
GLprediid(baseid,dd)
#>          pred     se-log    lower    upper
#> [1,] 1.596065 0.05530215 1.432113 1.778786
#> [2,] 1.429231 0.06660096 1.254329 1.628521
```

and G-computation

``` r
 hfactioncpx12$age <- (50+rnorm(741)*4)[hfactioncpx12$id]

 GLout <- recreg(Event(entry,time,status)~treatment+age,data=hfactioncpx12,cause=1,death.code=2)
 summary(GLout)
#> 
#>     n events
#>  2132   1391
#> 
#>  2132 clusters
#> coeffients:
#>              Estimate       S.E.    dU^-1/2 P-value
#> treatment1 -0.1131085  0.0640898  0.0538154  0.0776
#> age         0.0086223  0.0079803  0.0066607  0.2799
#> 
#> exp(coeffients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.89305 0.78763 1.0126
#> age         1.00866 0.99301 1.0246
 summary(survivalG(GLout,hfactioncpx12,time=4))
#> G-estimator :
#>       Estimate Std.Err  2.5% 97.5%    P-value
#> risk0    2.640  0.1203 2.404 2.876 1.067e-106
#> risk1    2.358  0.1165 2.130 2.586  3.838e-91
#> 
#> Average Treatment effect: difference (G-estimator) :
#>    Estimate Std.Err    2.5%   97.5% P-value
#> p1  -0.2824  0.1597 -0.5953 0.03059 0.07699
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>        Estimate    Std.Err       2.5%      97.5%    P-value
#> [p1] -0.1131085 0.06408982 -0.2387222 0.01250527 0.07759015
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.8930538 0.7876336 1.0125838
```

## Examples: Fixed time modelling for recurrent events

We can fit a log-link regression model at 2 years for the expected
number of events observed before dying (using IPCW adjustment)

``` r
data(hfactioncpx12)

e2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2,time=2)
summary(e2)
#>    n events
#>  741   1052
#> 
#>  741 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  0.452430  0.060814  0.333236  0.571624  0.0000
#> treatment1  -0.078322  0.093560 -0.261696  0.105052  0.4025
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  1.57213 1.39548 1.7711
#> treatment1   0.92467 0.76974 1.1108
head(iid(e2))
#>            [,1]          [,2]
#> 1  1.959479e-04 -2.266440e-04
#> 2  2.237613e-03 -2.227140e-03
#> 3 -9.349773e-06  1.293789e-03
#> 4 -9.653029e-04  9.653029e-04
#> 5 -1.203962e-04  6.744236e-05
#> 6 -2.861359e-03  2.871831e-03
```

## Examples: Cumulative Medical Cost 

Estimate mean cumulative cost (see also vignette)

``` r
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

## marginal mean using formula  
outNZ <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 
```

For comparison we also compute the IPCW estimates at time 3, using 
the linear model, and note that they are identical. Standard errors are however based on different formula that are asymptotically 
equivalent, and we note that they are very similar.

```{r}
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
head(iid(outNZ3))
```

We also apply the semiparametric proportional cost model with 
IPCW adjustment: 

```{r}
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
plot(propNZ,main="Baselines")
```

## Examples: Regression for RMST/Restricted mean survival for survival and competing risks using IPCW

RMST can be computed using the Kaplan-Meier (via phreg) and the for
competing risks via the cumulative incidence functions, but we can also
get these estimates via IPCW adjustment and then we can do regression

``` r
 ### same as Kaplan-Meier for full censoring model 
 bmt$int <- with(bmt,strata(tcell,platelet))
 out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                         cens.model=~strata(platelet,tcell),model="lin")
 estimate(out)
#>                        Estimate Std.Err  2.5% 97.5%   P-value
#> inttcell=0, platelet=0    13.61  0.8314 11.98 15.24 3.453e-60
#> inttcell=0, platelet=1    18.90  1.2694 16.42 21.39 3.717e-50
#> inttcell=1, platelet=0    16.19  2.4057 11.48 20.91 1.678e-11
#> inttcell=1, platelet=1    17.77  2.4532 12.96 22.58 4.391e-13
 head(iid(out))
#>             [,1] [,2] [,3] [,4]
#> [1,] -0.05341125    0    0    0
#> [2,] -0.05342611    0    0    0
#> [3,] -0.05343207    0    0    0
#> [4,] -0.05341706    0    0    0
#> [5,] -0.05342052    0    0    0
#> [6,] -0.05341259    0    0    0
 ## same as 
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 rm1 <- resmean_phreg(out1,times=30)
 summary(rm1)
#>                     strata times    rmean  se.rmean    lower    upper
#> tcell=0, platelet=0      0    30 13.60584 0.8314012 12.07012 15.33695
#> tcell=0, platelet=1      1    30 18.90350 1.2690639 16.57288 21.56188
#> tcell=1, platelet=0      2    30 16.19410 2.4002390 12.11140 21.65306
#> tcell=1, platelet=1      3    30 17.76830 2.4417528 13.57289 23.26053
#>                     years.lost
#> tcell=0, platelet=0   16.39416
#> tcell=0, platelet=1   11.09650
#> tcell=1, platelet=0   13.80590
#> tcell=1, platelet=1   12.23170
 
 ## competing risks years-lost for cause 1  
 out1 <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                       cens.model=~strata(platelet,tcell),model="lin")
 estimate(out1)
#>                        Estimate Std.Err   2.5%  97.5%   P-value
#> inttcell=0, platelet=0   12.103  0.8507 10.436 13.770 6.168e-46
#> inttcell=0, platelet=1    6.883  1.1739  4.582  9.184 4.533e-09
#> inttcell=1, platelet=0    7.260  2.3529  2.648 11.871 2.033e-03
#> inttcell=1, platelet=1    5.779  2.0921  1.679  9.880 5.737e-03
 ## same as 
 drm1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30)
 summary(drm1)
#> $estimate
#>                     strata times    intF_1   intF_2 se.intF_1 se.intF_2
#> tcell=0, platelet=0      0    30 12.103113 4.291051 0.8506728 0.6160195
#> tcell=0, platelet=1      1    30  6.882894 4.213603 1.1738590 0.9055124
#> tcell=1, platelet=0      2    30  7.259595 6.546309 2.3529175 1.9699198
#> tcell=1, platelet=1      3    30  5.779287 6.452411 2.0920912 2.0811678
#>                     total.years.lost lower_intF_1 upper_intF_1 lower_intF_2
#> tcell=0, platelet=0         16.39416    10.545569    13.890702     3.238664
#> tcell=0, platelet=1         11.09650     4.927208     9.614821     2.765212
#> tcell=1, platelet=0         13.80590     3.846168    13.702396     3.629546
#> tcell=1, platelet=1         12.23170     2.842764    11.749182     3.429056
#>                     upper_intF_2
#> tcell=0, platelet=0     5.685405
#> tcell=0, platelet=1     6.420645
#> tcell=1, platelet=0    11.807030
#> tcell=1, platelet=1    12.141421
```

## Examples: Average treatment effects (ATE) for survival or competing risks

We can compute ATE for survival or competing risks data for the
probability of dying

``` r
 bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
 brs <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,
      treat.model=tcell~platelet+age)
 summary(brs)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -0.19901  0.13098 -0.45574  0.05771  0.1287
#> tcell1      -0.63788  0.35668 -1.33696  0.06120  0.0737
#> platelet    -0.34411  0.24604 -0.82634  0.13811  0.1619
#> age          0.43737  0.10727  0.22712  0.64762  0.0000
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.81954 0.63398 1.0594
#> tcell1       0.52841 0.26264 1.0631
#> platelet     0.70885 0.43765 1.1481
#> age          1.54862 1.25497 1.9110
#> 
#> Average Treatment effects (G-formula) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     0.4288003  0.0275149  0.3748722  0.4827284  0.0000
#> treat1     0.2898471  0.0659033  0.1606789  0.4190153  0.0000
#> treat:1-0 -0.1389532  0.0717737 -0.2796272  0.0017208  0.0529
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.428211  0.027617  0.374084  0.482339  0.0000
#> treat1     0.250336  0.064792  0.123346  0.377325  0.0001
#> treat:1-0 -0.177876  0.070147 -0.315361 -0.040390  0.0112
 head(brs$riskDR.iid)
#>          iidriska      iidriska
#> [1,] -0.001159043 -3.524810e-05
#> [2,] -0.001201108  7.613126e-05
#> [3,] -0.001326534  3.362333e-04
#> [4,] -0.001320393  3.250252e-04
#> [5,] -0.001140791 -9.095525e-05
#> [6,] -0.001398307  4.597688e-04
 head(brs$riskG.iid)
#>        riskGa.iid    riskGa.iid
#> [1,] -0.001190759 -0.0001528426
#> [2,] -0.001242465  0.0001088968
#> [3,] -0.001355317  0.0006916069
#> [4,] -0.001350729  0.0006676909
#> [5,] -0.001164523 -0.0002838563
#> [6,] -0.001404170  0.0009471848
```

or the the restricted mean survival or years-lost to cause 1

``` r
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
#>    n events
#>  408    241
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.852872  0.062472  2.730429  2.975315  0.0000
#> tcell1       0.021472  0.122886 -0.219381  0.262325  0.8613
#> platelet     0.303325  0.090731  0.125495  0.481155  0.0008
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 17.33750 15.33947 19.5958
#> tcell1       1.02170  0.80302  1.2999
#> platelet     1.35435  1.13371  1.6179
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.26491  0.95910 17.38511 21.14472  0.0000
#> treat1    19.68305  2.22794 15.31637 24.04973  0.0000
#> treat:1-0  0.41813  2.41074 -4.30684  5.14310  0.8623
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.28397  0.95792 17.40649 21.16146  0.0000
#> treat1    20.34809  2.54086 15.36811 25.32808  0.0000
#> treat:1-0  1.06412  2.70957 -4.24654  6.37478  0.6945
 head(out$riskDR.iid)
#>         iidriska    iidriska
#> [1,] -0.05143041 0.005890787
#> [2,] -0.05144061 0.005890787
#> [3,] -0.05144470 0.005890787
#> [4,] -0.05143440 0.005890787
#> [5,] -0.05143678 0.005890787
#> [6,] -0.05143133 0.005890787
 head(out$riskG.iid)
#>       riskGa.iid  riskGa.iid
#> [1,] -0.05185784 -0.01866183
#> [2,] -0.05186812 -0.01866485
#> [3,] -0.05187225 -0.01866606
#> [4,] -0.05186186 -0.01866301
#> [5,] -0.05186425 -0.01866372
#> [6,] -0.05185876 -0.01866211

 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
                    treat.model=tcell~platelet)
 summary(out1)
#>    n events
#>  408    157
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.806167  0.069617  2.669721  2.942614  0.0000
#> tcell1      -0.374457  0.247756 -0.860051  0.111137  0.1307
#> platelet    -0.491638  0.164932 -0.814899 -0.168377  0.0029
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 16.54638 14.43594 18.9654
#> tcell1       0.68766  0.42314  1.1175
#> platelet     0.61162  0.44268  0.8450
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    14.53031  0.95690 12.65481 16.40581   0.000
#> treat1     9.99195  2.37789  5.33137 14.65253   0.000
#> treat:1-0 -4.53836  2.57483 -9.58494  0.50822   0.078
#> 
#> Average Treatment effects (double robust) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     14.512256   0.957862  12.634880  16.389632  0.0000
#> treat1      9.362018   2.416771   4.625234  14.098802  0.0001
#> treat:1-0  -5.150238   2.597631 -10.241501  -0.058975  0.0474
```

Here event is 0/1 thus leading to restricted mean and cause taking the
values 0,1,2 produces regression for the years lost due to cause 1.

## Examples: While Alive estimands for recurrent events

We consider an RCT and aim to describe the treatment effect via while
alive estimands

``` r
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
#> 
#> status
#>    0    1    2 
#>  617 1391  124
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
#> While-Alive summaries:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate Std.Err  2.5% 97.5% P-value
#> treatment0    1.859 0.02108 1.817 1.900       0
#> treatment1    1.924 0.01502 1.894 1.953       0
#>  
#>                           Estimate Std.Err    2.5%    97.5% P-value
#> [treatment0] - [treat.... -0.06517 0.02588 -0.1159 -0.01444  0.0118
#> 
#>  Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err  2.5% 97.5%   P-value
#> treatment0    1.572 0.09573 1.384 1.759 1.375e-60
#> treatment1    1.453 0.10315 1.251 1.656 4.376e-45
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....   0.1185  0.1407 -0.1574 0.3943     0.4
#> 
#>  Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>    Estimate Std.Err   2.5%  97.5%   P-value
#> p1   0.8457 0.05264 0.7425 0.9488 4.411e-58
#> p2   0.7555 0.05433 0.6490 0.8619 5.963e-44
#>  
#>             Estimate Std.Err     2.5%  97.5% P-value
#> [p1] - [p2]  0.09022 0.07565 -0.05805 0.2385   0.233
#> 
#>  Null Hypothesis: 
#>   [p1] - [p2] = 0 
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err   2.5%  97.5%   P-value
#> treat0   1.0725  0.1222 0.8331 1.3119 1.645e-18
#> treat1   0.7552  0.0643 0.6291 0.8812 7.508e-32
#>  
#>                     Estimate Std.Err    2.5%  97.5% P-value
#> [treat0] - [treat1]   0.3173  0.1381 0.04675 0.5879 0.02153
#> 
#>  Null Hypothesis: 
#>   [treat0] - [treat1] = 0

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
           death.code=2,trans=.333)
summary(dd,type="log")
#> While-Alive summaries, log-scale:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate  Std.Err   2.5%  97.5% P-value
#> treatment0   0.6199 0.011340 0.5977 0.6421       0
#> treatment1   0.6543 0.007807 0.6390 0.6696       0
#>  
#>                           Estimate Std.Err     2.5%     97.5% P-value
#> [treatment0] - [treat.... -0.03446 0.01377 -0.06145 -0.007478 0.01231
#> 
#>  Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err   2.5%  97.5%   P-value
#> treatment0   0.4523 0.06090 0.3329 0.5716 1.119e-13
#> treatment1   0.3739 0.07097 0.2348 0.5130 1.376e-07
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....  0.07835 0.09352 -0.1049 0.2616  0.4022
#> 
#>  Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>    Estimate Std.Err    2.5%    97.5%   P-value
#> p1  -0.1676 0.06224 -0.2896 -0.04563 7.081e-03
#> p2  -0.2804 0.07192 -0.4214 -0.13947 9.651e-05
#>  
#>             Estimate Std.Err     2.5%  97.5% P-value
#> [p1] - [p2]   0.1128 0.09511 -0.07361 0.2992  0.2356
#> 
#>  Null Hypothesis: 
#>   [p1] - [p2] = 0 
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err    2.5%   97.5%   P-value
#> treat0  -0.3833 0.04939 -0.4801 -0.2865 8.487e-15
#> treat1  -0.5380 0.05666 -0.6491 -0.4270 2.191e-21
#>  
#>                     Estimate Std.Err     2.5%  97.5% P-value
#> [treat0] - [treat1]   0.1548 0.07517 0.007459 0.3021 0.03948
#> 
#>  Null Hypothesis: 
#>   [treat0] - [treat1] = 0
```

---
output:
  github_document:
    toc: true
    toc_depth: 2
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start-->
[![R-CMD-check](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml)
[![coverage](https://codecov.io/github/kkholst/mets/coverage.svg)](https://app.codecov.io/github/kkholst/mets?branch=master)
[![cran](https://www.r-pkg.org/badges/version-last-release/mets)](https://CRAN.R-project.org/package=mets)
[![cran-dl](https://cranlogs.r-pkg.org/badges/mets)](https://cranlogs.r-pkg.org/downloads/total/last-month/mets)

<!-- badges: end -->

```{r  include=FALSE }
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/README-",
  cache = FALSE,
  out.width = "50%"
)
```

# Multivariate Event Times (`mets`) <img src="man/figures/logo.png" align="right" height="250" style="float:right; height:250px;"  alt="mets website">

 Implementation of various statistical models for multivariate
 event history data <doi:10.1007/s10985-013-9244-x>. Including multivariate
 cumulative incidence models <doi:10.1002/sim.6016>, and  bivariate random
 effects probit models (Liability models) <doi:10.1016/j.csda.2015.01.014>.
 Modern methods for survival analysis, including regression modelling (Cox, Fine-Gray, 
 Ghosh-Lin, Binomial regression) with fast computation of influence functions. 
 Restricted mean survival time regression and years lost for competing risks. 
 Average treatment effects and G-computation.  All functions can be used with
 clusters and will work for large data. 

## Installation
```{r load, echo=FALSE, results=FALSE}
library("mets")
```


```{r install, eval=FALSE}
install.packages("mets")
```

The development version may be installed directly from github
(requires [Rtools](https://cran.r-project.org/bin/windows/Rtools/) on windows and [development tools](https://mac.r-project.org/tools/) (+Xcode) for Mac OS
X):


```{r devinstall, eval=FALSE}
remotes::install_github("kkholst/mets", dependencies="Suggests")
```

or to get development version


```{r devinstall2, eval=FALSE}
remotes::install_github("kkholst/mets",ref="develop")
```


## Citation

To cite the `mets` package please use one of the following references

> Thomas H. Scheike and Klaus K. Holst (2022). A Practical Guide to
> Family Studies with Lifetime Data. Annual Review of Statistics and
> Its Application 9, pp. 47-69. doi: <http://dx.doi.org/10.1146/annurev-statistics-040120-024253>

> Thomas H. Scheike and Klaus K. Holst and Jacob B. Hjelmborg (2013).
> Estimating heritability for cause specific mortality based on twin studies.
> Lifetime Data Analysis. <http://dx.doi.org/10.1007/s10985-013-9244-x>

> Klaus K. Holst and Thomas H. Scheike Jacob B. Hjelmborg (2015).
> The Liability Threshold Model for Censored Twin Data.
> Computational Statistics and Data Analysis. <http://dx.doi.org/10.1016/j.csda.2015.01.014>

BibTeX:

    @Article{,
      title = {A Practical Guide to Family Studies with Lifetime Data},
      author = {Thomas H. Scheike and Klaus K. Holst},
      year = {2014},
      volume = {9},
      pages = {47-69},
      journal = {Annual Review of Statistics and Its Application},
      doi = {10.1146/annurev-statistics-040120-024253},
    }

    @Article{,
      title={Estimating heritability for cause specific mortality based on twin studies},
      author={Scheike, Thomas H. and Holst, Klaus K. and Hjelmborg, Jacob B.},
      year={2013},
      issn={1380-7870},
      journal={Lifetime Data Analysis},
      doi={10.1007/s10985-013-9244-x},
      url={http://dx.doi.org/10.1007/s10985-013-9244-x},
      publisher={Springer US},
      keywords={Cause specific hazards; Competing risks; Delayed entry;
    	    Left truncation; Heritability; Survival analysis},
      pages={1-24},
      language={English}
    }

    @Article{,
      title={The Liability Threshold Model for Censored Twin Data},
      author={Holst, Klaus K. and Scheike, Thomas H. and Hjelmborg, Jacob B.},
      year={2015},
      doi={10.1016/j.csda.2015.01.014},
      url={http://dx.doi.org/10.1016/j.csda.2015.01.014},
      journal={Computational Statistics and Data Analysis}
    }

## Examples: Twins Polygenic modelling

First considering standard twin modelling (ACE, AE, ADE, and more  models)

```{r, twinlm1}
# simulated data with pairs of observations in twins on long #data format
set.seed(1)
d <- twinsim(1000, b1=c(1,-1), b2=c(), acde=c(1,1,0,1))
# Polygenic model with Additive genetic effects, and shared and invidual environmental effects (ACE)
ace <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
ace
# An AE-model could be fitted as
ae <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id", type="ae")
# AIC
AIC(ae)-AIC(ace)
# To adjust for the covariates we simply alter the formula statement
ace2 <- twinlm(y ~ x1+x2, data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
 ## Summary/GOF
summary(ace2)
```

## Examples: Twins Polygenic modelling time-to-events Data

In the context of time-to-events data we consider the 
"Liabilty Threshold model" with IPCW adjustment for censoring.

First we fit the bivariate probit model (same marginals in MZ and DZ twins but 
different correlation parameter). Here we evaluate the risk of getting 
cancer before the last double cancer event (95 years)

```{r bptwintime}
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in

b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b0)
```

Liability threshold model with ACE random effects structure

```{r liability_ace1}
b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
           cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b1)
```

## Examples: Twins Concordance for time-to-events Data

```{r prtcomprisk}

data(prt) ## Prostate data example (sim)

## Bivariate competing risk, concordance estimates
p33 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                  data=prt, cause=c(2,2), return.data=1, prodlim=TRUE)

p33dz <- p33$model$"DZ"$comp.risk
p33mz <- p33$model$"MZ"$comp.risk

## Probability weights based on Aalen's additive model (same censoring within pair)
prtw <- ipw(Surv(time,status==0)~country+zyg, data=prt,
            obs.only=TRUE, same.cens=TRUE, 
            cluster="id", weight.name="w")

## Marginal model (wrongly ignoring censorings)
bpmz <- biprobit(cancer~1 + cluster(id), 
                 data=subset(prt,zyg=="MZ"), eqmarg=TRUE)

## Extended liability model
bpmzIPW <- biprobit(cancer~1 + cluster(id),
                    data=subset(prtw,zyg=="MZ"),
                    weights="w")
smz <- summary(bpmzIPW)

## Concordance
plot(p33mz,ylim=c(0,0.1),axes=FALSE, automar=FALSE,atrisk=FALSE,background=TRUE,background.fg="white")
axis(2); axis(1)

abline(h=smz$prob["Concordance",],lwd=c(2,1,1),col="darkblue")
## Wrong estimates:
abline(h=summary(bpmz)$prob["Concordance",],lwd=c(2,1,1),col="lightgray",lty=2)
```

## Examples: Cox model, RMST 

We can fit the Cox model and compute many useful summaries, such as 
restricted mean survival and  stanardized treatment effects (G-estimation).
First estimating the standardized survival 

```{r coxrmst}
 data(bmt)
 bmt$time <- bmt$time+runif(408)*0.001
 bmt$event <- (bmt$cause!=0)*1
 dfactor(bmt) <- tcell.f~tcell

 ss <- phreg(Surv(time,event)~tcell.f+platelet+age,bmt) 
 summary(survivalG(ss,bmt,50))

 sst <- survivalGtime(ss,bmt,n=50)
 plot(sst,type=c("survival","risk","survival.ratio")[1])
```

Based on the phreg we can also compute  the 
restricted mean survival time and years lost (via Kaplan-Meier estimates). The function does it 
for all times at once and can be plotted as restricted mean survival or years lost at the different
time horizons 


```{r}
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 
 rm1 <- resmean.phreg(out1, times=c(50))
 summary(rm1)
 par(mfrow=c(1, 2))
 plot(rm1,se=1)
 plot(rm1,years.lost=TRUE,se=1)
```

For competing risks the years lost can be decomposed into different causes and is 
based on the integrated Aalen-Johansen estimators for the different strata

```{r yearslostcause}
 ## years.lost decomposed into causes
 drm1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=50)
 par(mfrow=c(1,2)); plot(drm1,cause=1,se=1); title(main="Cause 1"); plot(drm1,cause=2,se=1); title(main="Cause 2")
 summary(drm1)
```

Computations are again done for all time horizons at once as illustrated in the plot. 

## Examples: Cox model IPTW 

We can fit the Cox model with inverse probabilty of treatment weights based on 
logistic regression. The treatment weights can be time-dependent and then mutiplicative
weights are applied (see details and vignette). 

```{r coxiptw}
data(bmt)
bmt$time <- bmt$time+runif(408)*0.001
bmt$id <- seq_len(nrow(bmt))
bmt$event <- (bmt$cause!=0)*1
dfactor(bmt) <- tcell.f~tcell

fit <- phreg_IPTW(Surv(time,event)~tcell.f+cluster(id),data=bmt,treat.model=tcell.f~platelet+age) 
summary(fit)
head(IC(fit))
```

## Examples: Competing risks regression, Binomial Regression

We can fit the logistic regression model at a specific time-point with IPCW adjustment

```{r binregcif}
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
# logistic regresion with IPCW binomial regression 
out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
summary(out)
head(IC(out))
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
```

## Examples: Competing risks regression, Fine-Gray/Logistic link

We can fit the Fine-Gray model and the logit-link competing risks model 
(using IPCW adjustment). Starting with the logit-link model 

```{r cifreg}
data(bmt)
bmt$time <- bmt$time+runif(nrow(bmt))*0.01
bmt$id <- 1:nrow(bmt)
## logistic link  OR interpretation
 or=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
summary(or)
par(mfrow=c(1,2))
 ## to see baseline 
plot(or)

 # predictions 
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pll <- predict(or,nd)
plot(pll)
```

Similarly, the Fine-Gray model can be estimated using IPCW adjustment

```{r finegrary}
 ## Fine-Gray model
 fg=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1,propodds=NULL)
 summary(fg)
## baselines 
plot(fg)
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pfg <- predict(fg,nd,se=1)
plot(pfg,se=1)

## influence functions of regression coefficients
head(iid(fg))
```

and we can get standard errors for predictions based on the influence functions of 
the baseline and the regression coefiicients (these are used in the predict function)

```{r}
baseid <- iidBaseline(fg,time=40)
FGprediid(baseid,nd)
```

further G-estimation can be done 

```{r}
 dfactor(bmt) <- tcell.f~tcell
 fg1 <- cifreg(Event(time,cause)~tcell.f+platelet+age,bmt,cause=1,propodds=NULL)
 summary(survivalG(fg1,bmt,50))
```

## Examples: Marginal mean for recurrent events 

We can estimate the expected number of events non-parametrically and 
get standard errors for this estimator

```{r recurrentevents}
data(hfactioncpx12)
dtable(hfactioncpx12,~status)

gl1 <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1,times=1:5)
plot(gl1,se=1)
```

## Examples: Ghosh-Lin for recurrent events 

We can fit the Ghosh-Lin model for the expected number of events observed
before dying (using IPCW adjustment and get predictions)

```{r ghoslin}
data(hfactioncpx12)
dtable(hfactioncpx12,~status)

gl1 <- recreg(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1)

## influence functions of regression coefficients
head(iid(gl1))
```
and we can get standard errors for predictions  based on the influence functions of the baseline 
and the regression coefiicients

```{r}
 nd=data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
 pfg <- predict(gl1,nd,se=1)
 summary(pfg,times=1:5)
 plot(pfg,se=1)
```

The influence functions of the baseline and regression coefficients at 
a specific time-point can be obtained 

```{r}
baseid <- iidBaseline(gl1,time=2)
dd <- data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
GLprediid(baseid,dd)
```
and G-computation 

```{r}
 hfactioncpx12$age <- (50+rnorm(741)*4)[hfactioncpx12$id]

 GLout <- recreg(Event(entry,time,status)~treatment+age,data=hfactioncpx12,cause=1,death.code=2)
 summary(GLout)
 summary(survivalG(GLout,hfactioncpx12,time=4))
```


## Examples: Fixed time modelling for recurrent events 

We can fit a log-link regression model at 2 years for the expected number of events observed
before dying (using IPCW adjustment)

```{r}
data(hfactioncpx12)

e2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2,time=2)
summary(e2)
head(iid(e2))
```

## Examples: Regression for RMST/Restricted mean survival for survival and competing risks  using IPCW  

RMST can be computed using the Kaplan-Meier (via phreg) and the for competing
risks via the cumulative incidence functions, but we can also get these 
estimates via IPCW adjustment and then we can do regression 

```{r}
 ### same as Kaplan-Meier for full censoring model 
 bmt$int <- with(bmt,strata(tcell,platelet))
 out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                         cens.model=~strata(platelet,tcell),model="lin")
 estimate(out)
 head(iid(out))
 ## same as 
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 rm1 <- resmean.phreg(out1,times=30)
 summary(rm1)
 
 ## competing risks years-lost for cause 1  
 out1 <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                       cens.model=~strata(platelet,tcell),model="lin")
 estimate(out1)
 ## same as 
 drm1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30)
 summary(drm1)
```

## Examples: Average treatment effects (ATE) for survival or competing risks 

We can compute ATE for survival or competing risks data for the 
probabilty of dying 

```{r}
 bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
 brs <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,
	  treat.model=tcell~platelet+age)
 summary(brs)
 head(brs$riskDR.iid)
 head(brs$riskG.iid)
```

or the the restricted mean survival or years-lost to cause 1 

```{r}
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 head(out$riskDR.iid)
 head(out$riskG.iid)

 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
                    treat.model=tcell~platelet)
 summary(out1)
```

Here event is 0/1 thus leading to restricted mean and cause taking the values 0,1,2  produces 
regression for the years lost due to cause 1. 

## Examples: While Alive estimands for recurrent events 

We consider an RCT and aim to describe the treatment effect via while alive estimands

```{r}
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")
```

---
output:
  github_document:
  toc: true
  toc_depth: 2
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start-->
[![R-CMD-check](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kkholst/mets/actions/workflows/R-CMD-check.yaml)
[![coverage](https://codecov.io/github/kkholst/mets/coverage.svg)](https://app.codecov.io/github/kkholst/mets?branch=master)
[![cran](https://www.r-pkg.org/badges/version-last-release/mets)](https://CRAN.R-project.org/package=mets)
[![cran-dl](https://cranlogs.r-pkg.org/badges/mets)](https://cranlogs.r-pkg.org/downloads/total/last-month/mets)

<!-- badges: end -->

```{r  include=FALSE }
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/README-",
  cache = FALSE,
  out.width = "50%"
)
```

# Multivariate Event Times (`mets`) <img src="man/figures/logo.png" align="right" height="250" style="float:right; height:250px;"  alt="mets website">

 Implementation of various statistical models for multivariate
 event history data <doi:10.1007/s10985-013-9244-x>. Including multivariate
 cumulative incidence models <doi:10.1002/sim.6016>, and  bivariate random
 effects probit models (Liability models) <doi:10.1016/j.csda.2015.01.014>.
 Modern methods for survival analysis, including regression modelling (Cox, Fine-Gray, 
 Ghosh-Lin, Binomial regression) with fast computation of influence functions. 
 Restricted mean survival time regression and years lost for competing risks. 
 Average treatment effects and G-computation.  All functions can be used with
 clusters and will work for large data. 

## Installation
```{r load, echo=FALSE, results=FALSE}
library("mets")
```


```{r install, eval=FALSE}
install.packages("mets")
```

The development version may be installed directly from github
(requires [Rtools](https://cran.r-project.org/bin/windows/Rtools/) on windows and [development tools](https://mac.r-project.org/tools/) (+Xcode) for Mac OS
X):


```{r devinstall, eval=FALSE}
remotes::install_github("kkholst/mets", dependencies="Suggests")
```

or to get development version


```{r devinstall2, eval=FALSE}
remotes::install_github("kkholst/mets",ref="develop")
```


## Citation

To cite the `mets` package please use one of the following references

> Thomas H. Scheike and Klaus K. Holst (2022). A Practical Guide to
> Family Studies with Lifetime Data. Annual Review of Statistics and
> Its Application 9, pp. 47-69. doi: 10.1146/annurev-statistics-040120-024253

> Thomas H. Scheike and Klaus K. Holst and Jacob B. Hjelmborg (2013).
> Estimating heritability for cause specific mortality based on twin studies.
> Lifetime Data Analysis. <http://dx.doi.org/10.1007/s10985-013-9244-x>

> Klaus K. Holst and Thomas H. Scheike Jacob B. Hjelmborg (2015).
> The Liability Threshold Model for Censored Twin Data.
> Computational Statistics and Data Analysis. <http://dx.doi.org/10.1016/j.csda.2015.01.014>

BibTeX:

    @Article{,
      title = {A Practical Guide to Family Studies with Lifetime Data},
      author = {Thomas H. Scheike and Klaus K. Holst},
      year = {2014},
      volume = {9},
      pages = {47-69},
      journal = {Annual Review of Statistics and Its Application},
      doi = {10.1146/annurev-statistics-040120-024253},
    }

    @Article{,
      title={Estimating heritability for cause specific mortality based on twin studies},
      author={Scheike, Thomas H. and Holst, Klaus K. and Hjelmborg, Jacob B.},
      year={2013},
      issn={1380-7870},
      journal={Lifetime Data Analysis},
      doi={10.1007/s10985-013-9244-x},
      url={http://dx.doi.org/10.1007/s10985-013-9244-x},
      publisher={Springer US},
      keywords={Cause specific hazards; Competing risks; Delayed entry;
    	    Left truncation; Heritability; Survival analysis},
      pages={1-24},
      language={English}
   }

    @Article{,
      title={The Liability Threshold Model for Censored Twin Data},
      author={Holst, Klaus K. and Scheike, Thomas H. and Hjelmborg, Jacob B.},
      year={2015},
      doi={10.1016/j.csda.2015.01.014},
      url={http://dx.doi.org/10.1016/j.csda.2015.01.014},
      journal={Computational Statistics and Data Analysis}
    }

## Examples: Twins Polygenic modelling

First considering standard twin modelling (ACE, AE, ADE, and more  models)

```{r, twinlm1}
# simulated data with pairs of observations in twins on long #data format
set.seed(1)
d <- twinsim(1000, b1=c(1,-1), b2=c(), acde=c(1,1,0,1))
# Polygenic model with Additive genetic effects, and shared and invidual environmental effects (ACE)
ace <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
ace
# An AE-model could be fitted as
ae <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id", type="ae")
# AIC
AIC(ae)-AIC(ace)
# To adjust for the covariates we simply alter the formula statement
ace2 <- twinlm(y ~ x1+x2, data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
 ## Summary/GOF
summary(ace2)
```

## Examples: Twins Polygenic modelling time-to-events Data

In the context of time-to-events data we consider the 
"Liabilty Threshold model" with IPCW adjustment for censoring.

First we fit the bivariate probit model (same marginals in MZ and DZ twins but 
different correlation parameter). Here we evaluate the risk of getting 
cancer before the last double cancer event (95 years)

```{r bptwintime}
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in

b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b0)
```

Liability threshold model with ACE random effects structure

```{r liability_ace1}
b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
           cens.formula=Surv(time,status==0)~zyg, breaks=tau)
summary(b1)
```

## Examples: Twins Concordance for time-to-events Data

```{r prtcomprisk}

data(prt) ## Prostate data example (sim)

## Bivariate competing risk, concordance estimates
p33 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                  data=prt, cause=c(2,2), return.data=1, prodlim=TRUE)

p33dz <- p33$model$"DZ"$comp.risk
p33mz <- p33$model$"MZ"$comp.risk

## Probability weights based on Aalen's additive model (same censoring within pair)
prtw <- ipw(Surv(time,status==0)~country+zyg, data=prt,
            obs.only=TRUE, same.cens=TRUE, 
            cluster="id", weight.name="w")

## Marginal model (wrongly ignoring censorings)
bpmz <- biprobit(cancer~1 + cluster(id), 
                 data=subset(prt,zyg=="MZ"), eqmarg=TRUE)

## Extended liability model
bpmzIPW <- biprobit(cancer~1 + cluster(id),
                    data=subset(prtw,zyg=="MZ"),
                    weights="w")
smz <- summary(bpmzIPW)

## Concordance
plot(p33mz,ylim=c(0,0.1),axes=FALSE, automar=FALSE,atrisk=FALSE,background=TRUE,background.fg="white")
axis(2); axis(1)

abline(h=smz$prob["Concordance",],lwd=c(2,1,1),col="darkblue")
## Wrong estimates:
abline(h=summary(bpmz)$prob["Concordance",],lwd=c(2,1,1),col="lightgray",lty=2)
```

## Examples: Cox model, RMST 

We can fit the Cox model and compute many useful summaries, such as 
restricted mean survival and  stanardized treatment effects (G-estimation).
First estimating the standardized survival 

```{r coxrmst}
 data(bmt)
 bmt$time <- bmt$time+runif(408)*0.001
 bmt$event <- (bmt$cause!=0)*1
 dfactor(bmt) <- tcell.f~tcell

 ss <- phreg(Surv(time,event)~tcell.f+platelet+age,bmt) 
 summary(survivalG(ss,bmt,50))

 sst <- survivalGtime(ss,bmt,n=50)
 plot(sst,type=c("survival","risk","survival.ratio")[1])
```

Based on the phreg we can also compute  the 
restricted mean survival time and years lost (via Kaplan-Meier estimates). The function does it 
for all times at once and can be plotted as restricted mean survival or years lost at the different
time horizons 


```{r}
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 
 rm1 <- resmean.phreg(out1, times=c(50))
 summary(rm1)
 par(mfrow=c(1, 2))
 plot(rm1,se=1)
 plot(rm1,years.lost=TRUE,se=1)
```

For competing risks the years lost can be decomposed into different causes and is 
based on the integrated Aalen-Johansen estimators for the different strata

```{r yearslostcause}
 ## years.lost decomposed into causes
 drm1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=50)
 par(mfrow=c(1,2)); plot(drm1,cause=1,se=1); title(main="Cause 1"); plot(drm1,cause=2,se=1); title(main="Cause 2")
 summary(drm1)
```

Computations are again done for all time horizons at once as illustrated in the plot. 

## Examples: Cox model IPTW 

We can fit the Cox model with inverse probabilty of treatment weights based on 
logistic regression. The treatment weights can be time-dependent and then mutiplicative
weights are applied (see details and vignette). 

```{r coxiptw}
data(bmt)
bmt$time <- bmt$time+runif(408)*0.001
bmt$id <- seq_len(nrow(bmt))
bmt$event <- (bmt$cause!=0)*1
dfactor(bmt) <- tcell.f~tcell

fit <- phreg_IPTW(Surv(time,event)~tcell.f+cluster(id),data=bmt,treat.model=tcell.f~platelet+age) 
summary(fit)
head(IC(fit))
```

## Examples: Competing risks regression, Binomial Regression

We can fit the logistic regression model at a specific time-point with IPCW adjustment

```{r binregcif}
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
# logistic regresion with IPCW binomial regression 
out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
summary(out)
head(IC(out))
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
```

## Examples: Competing risks regression, Fine-Gray/Logistic link

We can fit the Fine-Gray model and the logit-link competing risks model 
(using IPCW adjustment). Starting with the logit-link model 

```{r cifreg}
data(bmt)
bmt$time <- bmt$time+runif(nrow(bmt))*0.01
bmt$id <- 1:nrow(bmt)
## logistic link  OR interpretation
 or=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
summary(or)
par(mfrow=c(1,2))
 ## to see baseline 
plot(or)

 # predictions 
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pll <- predict(or,nd)
plot(pll)
```

Similarly, the Fine-Gray model can be estimated using IPCW adjustment

```{r finegrary}
 ## Fine-Gray model
 fg=cifreg(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1,propodds=NULL)
 summary(fg)
## baselines 
plot(fg)
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pfg <- predict(fg,nd,se=1)
plot(pfg,se=1)

## influence functions of regression coefficients
head(iid(fg))
```

and we can get standard errors for predictions based on the influence functions of 
the baseline and the regression coefiicients (these are used in the predict function)

```{r}
baseid <- iidBaseline(fg,time=40)
FGprediid(baseid,nd)
```

further G-estimation can be done 

```{r}
 dfactor(bmt) <- tcell.f~tcell
 fg1 <- cifreg(Event(time,cause)~tcell.f+platelet+age,bmt,cause=1,propodds=NULL)
 summary(survivalG(fg1,bmt,50))
```

## Examples: Marginal mean for recurrent events 

We can estimate the expected number of events non-parametrically and 
get standard errors for this estimator

```{r recurrentevents}
data(hfactioncpx12)
dtable(hfactioncpx12,~status)

gl1 <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1,times=1:5)
plot(gl1,se=1)
```

## Examples: Ghosh-Lin for recurrent events 

We can fit the Ghosh-Lin model for the expected number of events observed
before dying (using IPCW adjustment and get predictions)

```{r ghoslin}
data(hfactioncpx12)
dtable(hfactioncpx12,~status)

gl1 <- recreg(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2)
summary(gl1)

## influence functions of regression coefficients
head(iid(gl1))
```
and we can get standard errors for predictions  based on the influence functions of the baseline 
and the regression coefiicients

```{r}
 nd=data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
 pfg <- predict(gl1,nd,se=1)
 summary(pfg,times=1:5)
 plot(pfg,se=1)
```

The influence functions of the baseline and regression coefficients at 
a specific time-point can be obtained 

```{r}
baseid <- iidBaseline(gl1,time=2)
dd <- data.frame(treatment=levels(hfactioncpx12$treatment),id=1)
GLprediid(baseid,dd)
```
and G-computation 

```{r}
 hfactioncpx12$age <- (50+rnorm(741)*4)[hfactioncpx12$id]

 GLout <- recreg(Event(entry,time,status)~treatment+age,data=hfactioncpx12,cause=1,death.code=2)
 summary(GLout)
 summary(survivalG(GLout,hfactioncpx12,time=4))
```


## Examples: Fixed time modelling for recurrent events 

We can fit a log-link regression model at 2 years for the expected number of events observed
before dying (using IPCW adjustment)

```{r}
data(hfactioncpx12)

e2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,cause=1,death.code=2,time=2)
summary(e2)
head(iid(e2))
```

## Examples: Regression for RMST/Restricted mean survival for survival and competing risks  using IPCW  

RMST can be computed using the Kaplan-Meier (via phreg) and the for competing
risks via the cumulative incidence functions, but we can also get these 
estimates via IPCW adjustment and then we can do regression 

```{r}
 ### same as Kaplan-Meier for full censoring model 
 bmt$int <- with(bmt,strata(tcell,platelet))
 out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                         cens.model=~strata(platelet,tcell),model="lin")
 estimate(out)
 head(iid(out))
 ## same as 
 out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
 rm1 <- resmean.phreg(out1,times=30)
 summary(rm1)
 
 ## competing risks years-lost for cause 1  
 out1 <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                       cens.model=~strata(platelet,tcell),model="lin")
 estimate(out1)
 ## same as 
 drm1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30)
 summary(drm1)
```

## Examples: Average treatment effects (ATE) for survival or competing risks 

We can compute ATE for survival or competing risks data for the 
probabilty of dying 

```{r}
 bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
 brs <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,
	  treat.model=tcell~platelet+age)
 summary(brs)
 head(brs$riskDR.iid)
 head(brs$riskG.iid)
```

or the the restricted mean survival or years-lost to cause 1 

```{r}
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 head(out$riskDR.iid)
 head(out$riskG.iid)

 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
                    treat.model=tcell~platelet)
 summary(out1)
```

Here event is 0/1 thus leading to restricted mean and cause taking the values 0,1,2  produces 
regression for the years lost due to cause 1. 

## Examples: While Alive estimands for recurrent events 

We consider an RCT and aim to describe the treatment effect via while alive estimands

```{r}
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")
```

message("Bivariate Probit models")
example(biprobit)

message("Polygenic models")
example(bptwin)
example(twinlm)

message("Clayton-Oakes, Two-stage")
example(ClaytonOakes)
example(twostage)

message("Pairwise-Odds-Ratio")
example(binomial.twostage)
example(easy.binomial.twostage)

message("Bivariate competing risks")
example(bicomprisk)

'.onAttach' <- function(libname, pkgname="mets")
  {    
    ## desc <- utils::packageDescription(pkgname)
    ## packageStartupMessage("Loading '", desc$Package, "' package...\n",
    ##                       "\tVersion: ", desc$Version, "\n",
    ##                       "\tOverview: help(package=", desc$Package, ")");
    ## packageStartupMessage(desc$Package, " version ",desc$Version);
  }

.onUnload <- function (libpath) {
  library.dynam.unload("mets", libpath)
}

##' @rdname cif-nonpar
##' @param col colors for curves.
##' @param lty line types for curves.
##' @param legend logical; if TRUE, add legend.
##' @param which which plots to show.
##' @param cause causes to plot.
##' @param ask logical; if TRUE, prompt before new page.
##' @export
plotcr <- function(x,col,lty,legend=TRUE,which=1:2,cause=1:2,
                   ask = prod(par("mfcol")) < length(which) && dev.interactive(), ...) {
    if (inherits(try(find.package("prodlim"),silent=TRUE),"try-error")) {
        stop("Needs prodlim")
    }

  dots <- list(...)
  if (is.null(dots$xlab)) dots$xlab <- "Time"
  if ((!is.data.frame(x) | !is.matrix(x)) && ncol(x)<2) stop("Wrong type of data")  
  if (ncol(x)==2) {
    if (is.null(dots$curvlab)) {
      causes <- setdiff(unique(x[,2]),0)
      dots$curvlab <- seq(length(causes))
    }
    colnames(x)[1:2] <- c("t","status")
    co <- prodlim::prodlim(Hist(t,status)~1,data=data.frame(x))    
    ##    co <- cuminc(x[,1],x[,2])
    ##    if (any(x[,1]<0)) {
    ##      do.call(co,p)
    ## for (i in seq(length(co))) {
    ##   co[[i]]$time <- co[[i]]$time[-1]
    ##   co[[i]]$est <- co[[i]]$est[-1]
    ## }
    ##if (is.null(dots$xlim)) dots$xlim <- range(x[,1])
    ##    }
    ##do.call("plot", c(list(x=co), dots))
    if (is.null(dots$lwd)) dots$lwd <- 1
    if (missing(lty)) lty <- seq_len(length(co$cuminc))
    if (missing(col)) col <- rep(1,length(co$cuminc))
    if (is.null(dots$lwd)) dots$lwd <- 1
    dots$lty <- lty[1]
    dots$col <- col[1]
    do.call("plot", c(list(x=co), dots))
    dots$add <- TRUE
    for (i in seq_len(length(co$cuminc)-1)+1) {
      dots$lty <- lty[i]
      dots$col <- col[i]
      dots$cause <- i
      do.call("plot", c(list(x=co), dots))
    }
    if (legend)
      legend("topleft",names(co$cuminc),col=col,lty=lty,pch=-1)
    return(invisible(co))
  }

  if (ask) {
    oask <- devAskNewPage(TRUE)
    on.exit(devAskNewPage(oask))
  }

  t <- as.matrix(x[,1:2]); cause0 <- as.matrix(x[,3:4])
  causes <- sort(setdiff(unique(cause0),0))
  if (is.null(dots$curvlab))
    dots$curvlab <- seq(1:length(unique(causes)))
  if (missing(col)) col <- c("seagreen","darkred","darkblue","goldenrod","mediumpurple")
  if (1%in%which) {
    plot(t,type="n",...)
    count <- 1
    for (i in causes) {
      points(t[cause0[,1]==causes[i],],col=Col(col[count],0.5),pch=2)
      points(t[cause0[,2]==causes[i],],col=Col(col[count],0.5),pch=6)
      count <- count+1
    }
    points(t[cause0[,1]==0 & cause0[,2]==0,],col=Col("black",0.2),pch=1)  
    if (legend)
    legend("topleft", c("Subj 1, Cause 1", "Subj 2, Cause 1",
                        "Subj 1, Cause 2", "Subj 2, Cause 2",
                        "Double Censoring"), pch=c(2,6,2,6,1), col=c(rep(col[1:length(causes)],each=2),"black"))
  }
  if (2%in%which) {
    dots$curvlab <- NULL
    colnames(x)[1:4] <- c("t1","t2","cause1","cause2")
    co1 <- prodlim::prodlim(Hist(t1,cause1)~1,data.frame(x))
    co2 <- prodlim::prodlim(Hist(t2,cause2)~1,data.frame(x))
    if (is.null(dots$lwd)) dots$lwd <- 1
    if (missing(lty)) lty <- seq_len(length(co1$cuminc))
    if (missing(col)) col <- rep(1,length(co1$cuminc))
    if (is.null(dots$lwd)) dots$lwd <- 1
    dots$lty <- lty[1]
    dots$col <- col[1]
    dots$cause <- cause[1]
    do.call("plot", c(list(x=co1), dots))    
    dots$add <- TRUE
    do.call("plot", c(list(x=co2), dots))
##    for (i in seq_len(length(co1$cuminc)-1)+1) {
    for (i in seq_len(length(cause)-1)+1) {
        dots$lty <- lty[i]
        dots$col <- col[i]
        dots$cause <- cause[i]
        do.call("plot", c(list(x=co1), dots))
        do.call("plot", c(list(x=co2), dots))  
    }
    if (legend && length(cause)>1)
      legend("topleft",names(co1$cuminc)[cause],col=col,lty=lty,pch=-1,bg="white")
  }
}

##' Life-course plot for event life data with recurrent events 
##'
##' @title Life-course plot
##' @param formula Formula (Event(start,slut,status) ~ ...) 
##' @param data data.frame
##' @param id Id variable 
##' @param group group variable
##' @param type Type (line 'l', stair 's', ...)
##' @param lty Line type
##' @param col Colour
##' @param alpha transparency (0-1)
##' @param lwd Line width
##' @param recurrent.col col of recurrence type
##' @param recurrent.lty lty's of  of recurrence type
##' @param legend position of optional id legend
##' @param pchlegend point type legends 
##' @param by make separate plot for each level in 'by' (formula, name of column, or vector)
##' @param status.legend Status legend
##' @param place.sl Placement of status legend
##' @param xlab Label of X-axis
##' @param ylab Label of Y-axis
##' @param add Add to existing device
##' @param ... Additional arguments to lower level arguments
##' @author Thomas Scheike, Klaus K. Holst
##' @export
##' @examples
##' data = data.frame(id=c(1,1,1,2,2),start=c(0,1,2,3,4),slut=c(1,2,4,4,7),
##'                   type=c(1,2,3,2,3),status=c(0,1,2,1,2),group=c(1,1,1,2,2))
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id")
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",recurrent.col="type")
##'
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2)
##' op <- par(mfrow=c(1,2))
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",by=~group)
##' par(op)
##' legends=c("censored","pregnant","married")
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2,status.legend=legends)
##'
lifecourse <- function(formula,data,id="id",group=NULL,
               type="l",lty=1,col=1:10,alpha=0.3,lwd=1,
               recurrent.col=NULL, recurrent.lty=NULL,
               legend=NULL,pchlegend=NULL,
               by=NULL,
               status.legend=NULL,place.sl="bottomright",
               xlab="Time",ylab="",add=FALSE,...) 
{# {{{
    if (!is.null(by)) {
        if (is.character(by) && length(by==1)) {
            by <- data[,by]
        } else if (inherits(by,"formula")) {
            by <- model.frame(by,data,na.action=na.pass)
        }
        cl <- match.call(expand.dots=TRUE)
        cl$by <- NULL
        datasets <- split(data,by)
        res <- c()
        for (d in datasets) {
            cl$data <- d
            res <- c(res, eval(cl,parent.frame()))
        }
        return(invisible(res))
    }
    if (!is.null(group)) {
        if (is.character(group) && length(group==1)) {
            M <- data[,group]
        } else if (inherits(group,"formula")) {
            M <- model.frame(group,data,na.action=na.pass)
        } else {
            M <- group
        }
###        if (!add) plot(formula,data=data,xlab=xlab,ylab=ylab,...,type="n")        
        if (!add) {
	 if (inherits(id,"formula")) id <- all.vars(id)
	 if (inherits(group,"formula")) group <- all.vars(group)
	 if (is.character(id) && length(id)==1) Id <- 
	 y <- getoutcome(formula)
	 x <- attributes(y)$x

    if (length(x)==0) {# {{{

        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster_index(data[,id])
	ccm <- ccid$idclustmat+1
        ccm <- ccm[!is.na(ccm)]
        ###
        x <- data[ccm,id]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

	X <- c(c(t1),c(t2))
	Y <- rep(x,each=2)
	status <- tstat

# }}}
    } else {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster_index(data[,id])
	ccm <- ccid$idclustmat+1
	ccm <- ccm[!is.na(ccm)]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

        x <- data[ccm,x] 

	X <- c(c(t1),c(t2))
	Y <- rep(x,each=2)
	status <- tstat

    }# }}}

	 plot(X,Y,xlab=xlab,ylab=ylab,...,type="n")        

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}

       }

        dd <- split(data,M)
        K <- length(dd)
        if (length(type)<K)        type <- rep(type,K)
        if (length(col)<K)         col <- rep(col,K)
        if (length(lty)<K)         lty <- rep(lty,K)
        if (length(lwd)<K)         lwd <- rep(lwd,K)
        if (length(alpha)<K)       alpha <- rep(alpha,K)
	res <- list()
        for (i in seq_len(K)) {            
           res[[i]] <- lifecourse(formula,data=dd[[i]],id=id,type=type[i],
                     lty=lty[i],col=col[i],lwd=lwd[i],
                     alpha=alpha[i],
		     recurrent.col=recurrent.col,
		     recurrent.lty=recurrent.lty,
                     group=NULL,
                     add=TRUE,...)
        }

        if (!is.null(legend)) {
            graphics::legend(legend,names(dd),lwd=lwd,col=col,lty=lty)
        }
        return(invisible(res))
    }
    
    if (inherits(id,"formula")) id <- all.vars(id)
    if (inherits(group,"formula")) group <- all.vars(group)
    if (is.character(id) && length(id)==1) Id <- 
    y <- getoutcome(formula)
    x <- attributes(y)$x

    if (length(x)==0) {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster_index(data[,id])
	ccm <- ccid$idclustmat+1
        ccm <- ccm[!is.na(ccm)]
        ###
        x <- data[ccm,id]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

	X <- cbind(c(t1),c(t2))
	Y <- matrix(rep(x,each=2),ncol=2,byrow=TRUE)
	status <- tstat

###	print(dim(X)); print(dim(Y)); print(dim(status)); print(summary(X)); print(summary(Y)); 

	if (is.null(recurrent.col))  {
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	} else {
	   cn <- data[ccm,recurrent.col]
	   matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
	}

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}


# }}}
    } else {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster_index(data[,id])
	ccm <- ccid$idclustmat+1
	ccm <- ccm[!is.na(ccm)]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

        x <- data[ccm,x] 

	X <- cbind(c(t1),c(t2))
	Y <- matrix(rep(x,each=2),ncol=2,byrow=TRUE)
	status <- tstat

###	print(dim(X)); print(dim(Y)); print(dim(status)); print(summary(X)); print(summary(Y)); 



	if (is.null(recurrent.col))  cols <- col[1] else { 
            cols <- data[ccm,recurrent.lty]
	}
        if (is.null(recurrent.lty))  ltys <- lty else { 
            ltys <- data[ccm,recurrent.lty]
	}

###	print("hej")
###	print(t(Y))
###	print(t(X))
###	print(type)
###        matplot(t(X),t(Y),type="l",lwd=lwd,col=cols,xlab=xlab,ylab=ylab,add=add,...)

	if (is.null(recurrent.col))  {
###         matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,
###	          col=Col(col[1],alpha[1]),xlab=xlab,ylab=ylab,...)
###	          col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	} else {
            cn <- data[ccm,recurrent.col]
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
       }

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}

    }# }}}

###    return(invisible(data.frame(Y=Y,X=X,status=status)))
    return(invisible(list(Y,X,status)))
}# }}}

##' @export
loglikMVN <- function(yl, yu, status, mu, S, thres) {
  .loglikMVN(
          Yl=as.matrix(yl),
          Yu=as.matrix(yu),
          Status=as.integer(status),
          Mu=as.matrix(mu),
          S=as.matrix(S),
          z=NULL,su=NULL,
          Threshold=as.matrix(thres),
          itol=lava::lava.options()$itol
  )
}

##' @export
scoreMVN <- function(y, m, s, dm, ds) {
    .scoreMVN(
          Y=as.matrix(y),
          Mu=as.matrix(m),
          dMu=as.matrix(dm),
          S=as.matrix(s),
          dS=as.matrix(ds),
          itol=lava::lava.options()$itol
      )
}

##' Recurrent Events Regression with Terminal Event
##'
##' Fits a Ghosh-Lin IPCW (Inverse Probability of Censoring Weighted) Cox-type model for recurrent events
##' in the presence of a terminal event (e.g., death).
##'
##' For the Cox-type model, the expectation is modeled as:
##' \deqn{ E(dN_1(t)|X) = \mu_0(t) dt \exp(X^T \beta) }
##' by solving Cox-type IPCW weighted score equations:
##' \deqn{ \int (Z - E(t)) w(t) dN_1(t) }
##' where \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))},
##' \deqn{E(t) = S_1(t)/S_0(t)}, and \deqn{S_j(t) = \sum X_i^j w_i(t) \exp(X_i^T \beta)}.
##'
##' The IID decomposition of the beta coefficients takes the form:
##' \deqn{ \int (Z - E) w(t) dM_1 + \int q(s)/p(s) dM_c }
##' and is returned as the \code{iid} component.
##'
##' Events, deaths, and censorings are specified via a start-stop structure and the \code{Event} call.
##' The function identifies these via a status vector and cause codes, censoring codes (\code{cens.code}),
##' and death codes (\code{death.code}). See examples and vignettes for details.
##'
##' @param formula Formula with an 'Event' outcome.
##' @param data Data frame containing the variables.
##' @param cause Cause of interest (default is 1).
##' @param death.code Codes for the terminal event/death (default is 2).
##' @param cens.code Code for censoring (default is 0).
##' @param cens.model Formula for a stratified Cox model without covariates used to estimate censoring probabilities.
##' @param weights Weights for the score equations.
##' @param offset Offsets for the model.
##' @param Gc Censoring weights for the time argument. If \code{NULL}, these are calculated using a Kaplan-Meier estimator
##' (should then provide \eqn{G_c(T_i-)}).
##' @param wcomp Weights for composite outcomes (e.g., when \code{cause=c(1,3)}, \code{wcomp} might be \code{c(1,2)}).
##' @param augmentation.type Type of augmentation when an augmentation model is given (options: \code{"lindyn.augment"}, \code{"lin.augment"}).
##' @param marks A mark value vector from the data frame, specifying the mark value at all events.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"recreg"} (extending \code{"phreg"}) containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{iid}{Influence functions for the coefficients.}
##' \item{cumhaz}{Cumulative hazard estimates.}
##' \item{se.cumhaz}{Standard errors for cumulative hazard.}
##'
##' @author Thomas Scheike
##' @seealso \code{\link{recregIPCW}} 
##' @examples
##' ## data with no ties
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' hf$x <- as.numeric(hf$treatment) 
##' dd <- data.frame(treatment=levels(hf$treatment),id=1)
##'
##' gl <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,cause=1,death.code=2)
##' summary(gl)
##' head(iid(gl))
##' pgl <- predict(gl,dd,se=1); 
##' plot(pgl,se=1)
##' 
##' ## censoring stratified after treatment 
##' gls <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,
##' cause=1,death.code=2,cens.model=~strata(treatment))
##' summary(gls)
##' 
##' glss <- recreg(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
##' cause=1,death.code=2,cens.model=~strata(treatment))
##' summary(glss)
##' plot(glss)
##' 
##' ## IPCW at 2 years 
##' ll2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),data=hf,
##' cause=1,death.code=2,time=2,cens.model=~strata(treatment))
##' summary(ll2)
##' 
##' ll2i <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,
##' cause=1,death.code=2,time=2,cens.model=~strata(treatment))
##' summary(ll2i)
##' @aliases marks scalecumhaz GLprediid IIDrecreg 
##' @export
recreg <- function(formula,data,cause=1,death.code=2,cens.code=0,cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,marks=NULL,augmentation.type=c("lindyn.augment","lin.augment"),...)
{ ## {{{
	cl <- match.call()
	outi  <- recregBN(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,marks=marks,...)

	if (!is.null(outi$lindyn.augment)) {
		outA  <- recregBN(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,marks=marks,augmentation=outi[[augmentation.type[1]]],...)
		outi <- outA
	}

	outi$call <- cl
	return(outi)
} ## }}}

##' @export
marks <- function(x) x

recregBN <- function(formula,data=data,cause=c(1),death.code=c(2),cens.code=c(0),cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,marks=NULL,...)
{ ## {{{
cl <- match.call()
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event"))) {
        stop("Expected an 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata))  {
      ns <- grep("strata",names(des$levels))
      strata.name  <-  names(des$levels)[1]
    } else strata.name <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## no use of 
    pos.cluster <- pos.strata <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks

 res <- c(recregN01(data,X,entry,exit,status,id=id,strata=strata,offset=offset,weights=weights,strata.name=strata.name,cens.model=cens.model,cause=cause,
		   death.code=death.code,cens.code=cens.code,Gc=Gc,wcomp=wcomp,
		   case.weights=marks,...),
 list(call=cl,formula=formula,strata.pos=pos.strata,
      cluster.pos=pos.cluster,n=length(status),nevent=sum(status %in% cause)))
	colnames(res$iid) <- names(res$coef)
  res$design <- des

	class(res) <- c("recreg","phreg")
	return(res)
} ## }}}

recregN01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,weights=NULL,
		      strata.name=NULL,beta,stderr=1,method="NR",no.opt=FALSE,propodds=NULL,zero.remove=1,
		      case.weights=NULL,cause=1,death.code=2,cens.code=0,Gc=NULL,cens.model=~+1,augmentation=NULL,
		      cox.prep=TRUE,wcomp=NULL,augment.model=NULL,adm.cens.time=NULL,ftime.augment=NULL,twostage=FALSE,...) { ## {{{
	p <- ncol(X)  # setting up weights, strata, beta and so forth before the action starts
	if (missing(beta)) beta <- rep(0,p)
	if (p==0) X <- cbind(rep(0,length(exit)))
	augmentation.call <- augmentation
	if (is.null(augmentation)) augmentation <- 0
	cause.jumps <- which(status %in% cause)
	if (length(cause.jumps)>0) {
		max.jump <- max(exit[cause.jumps])
		other <- which((status %in% death.code ) & (exit< max.jump))
	} else  {
		###	warning("no jumps of cause type\n"); 
		max.jump <- max(exit)
		other <- which((status %in% death.code ) )
		stop(paste("no jumps of cause type",cause)); 
	}

	n <- length(exit)
	if (is.null(strata)) {
		strata <- rep(0,length(exit))
		nstrata <- 1
		strata.level <- NULL
	} else {
		strata.level <- levels(strata)
		ustrata <- sort(unique(strata))
		nstrata <- length(ustrata)
		strata.values <- ustrata
		if (is.numeric(strata))
			strata <-  fast.approx(ustrata,strata)-1
		else  {
			strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
		}
	}

	if (is.null(entry)) entry <- rep(0,length(exit))
	trunc <- (any(entry>0))
	if (is.null(offset)) offset <- rep(0,length(exit))
	if (is.null(weights)) weights <- rep(1,length(exit))
	if (is.null(case.weights)) case.weights <- rep(1,length(exit))
	if (!is.null(wcomp))  {
		if (length(wcomp)!=length(cause)) stop("weights follow the causes, and length must be the same\n"); 
		wwcomp <- rep(1,length(exit)); 
		k <- 1
		for (i in cause) { wwcomp[status==i] <- wcomp[k];k <- k+1} 
		case.weights <- case.weights* wwcomp
	}
	strata.call <- strata

	call.id <- id 
	conid <- construct_id(id,nrow(X),namesX=rownames(X))
	name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
	orig.id <- id
	exit.call <- exit

	### censoring weights constructed
	whereC <- which(status %in% cens.code)
	time <- exit
	statusC <- c(status %in% cens.code)
	data$id__ <- id  
	data$exit__ <- exit
	data$entry__ <- entry
	data$statusC <- statusC
	data$status__ <- (status %in% cause)*1
	cens.strata <- cens.nstrata <- NULL
	## lag-count to use for augment.model=~Nt+X1+X2
	data <- count_history(data,status="status__",id="id__",types=cause,multitype=TRUE)
	data$Nt <- data[,paste("Count",cause[1],sep="")]

	## augmentation model and remove intercept
	if (!is.null(augment.model)) { XXA <- model.matrix(augment.model,data)[,-1,drop=FALSE]; namesXXA <- colnames(XXA); } else XXA <- NULL

	if ((length(whereC)>0) ) { ## {{{
		if (is.null(Gc)) {
			kmt <- TRUE
			if (class(cens.model)[1]=="formula") {
				formC <- update.formula(cens.model,Surv(entry__,exit__,statusC)~ . +cluster(id__))
				cens.model <- phreg(formC,data)
			}
			if (cens.model$p>0) kmt <- FALSE
			###        Pcens.model <- predict(cens.model,data,times=exit,tminus=TRUE,individual.time=TRUE,se=FALSE,km=kmt)
			Pcens.model <- predict(cens.model,data,times=exit,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)
			Stime <- Pcens.model$surv <- c(Pcens.model$surv)
			## strata from original data
			nCstrata <- cens.model$nstrata
			cens.strata <- Pcens.model$strata
		} else {
			formC <- NULL
			Stime <- Gc
			Pcens.model <- list(time=exit,surv=Gc,strata=0)
			nCstrata <- 1
			cens.strata <- rep(0,length(exit))
		}
	} else { 
		formC <- NULL
		Stime <- Gc  <- rep(1,length(exit))
		Pcens.model <- list(time=exit,surv=Gc,strata=0)
		nCstrata <- 1
		cens.strata <- rep(0,length(exit))
	} ## }}}

	Zcall <- cbind(status,cens.strata,Stime,strata,strata,1) ## to keep track of status and Censoring strata

	trunc <- TRUE
	if (twostage) { ## to use old structure for twostageREC
		xxg <- .Call("FastCoxPrepStrata",entry,exit,(status %in% cause)*1,X,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")
		xxg$nstrata <- nstrata
	}

	## computing terms for those experiencing another cause, need S0, S1, S2
	if ((length(other)>=1)) { ## {{{ 
		weightso <- weights[other]/Stime[other]
		timeoo <- rep(max(exit)+1,length(other))
		if (is.null(adm.cens.time))
			timeoo <- rep(max(exit)+1,length(other)) else timeoo <- adm.cens.time[other] 
		statuso <- rep(cens.code[1],length(other))
		Xo <- X[other,,drop=FALSE]
		offseto <- offset[other]
		entryo <- exit[other]
		ido <- id[other]
		stratao <- strata[other]
		type <- rep(2,length(other))
		###

		entry <- c(entry,entryo)
		exit <- c(exit,timeoo)
		status <- c(status,statuso)
		X <- rbind(X,Xo)
		id <- c(id,ido)
		strata <- c(strata,stratao)
		weights <- c(weights,weightso)
		offset <- c(offset,offseto)
		case.weights <- c(case.weights,case.weights[other])

		Zcallo <-  Zcall[other,,drop=FALSE]
		Zcallo[,6] <- 2
		Zcall <- rbind(Zcall,Zcallo)
		Zcall <- cbind(Zcall,rbind(XXA,XXA[other,,drop=FALSE]))
	} ## }}}

	stat1 <- 1*(status %in% cause)
	xx2 <- .Call("FastCoxPrepStrata",entry,exit,stat1,X,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")

	### remove all the initial zero's to save space/time, that is, the first streak of sign=-1, 
	### that is not important for risk and events
	if (zero.remove==1) { ## {{{
		first <- which(xx2$sign==1)[1]-1
		if (first >1) {
			fentry <- (1:first)
			xx2$id     <- xx2$id[-fentry]
			xx2$time   <- xx2$time[-fentry]
			xx2$status <- xx2$status[-fentry]
			xx2$sign   <- xx2$sign[-fentry]
			xx2$X      <- xx2$X[-fentry,,drop=FALSE]
			xx2$XX     <- xx2$XX[-fentry,,drop=FALSE]
			if (nrow(xx2$ZX)==nrow(xx2$X)) xx2$ZX <- xx2$ZX[-fentry,,drop=FALSE]
			xx2$Z      <- xx2$Z[-fentry,,drop=FALSE]
			xx2$offset <-xx2$offset[-fentry]
			xx2$weights <-xx2$weights[-fentry]
			xx2$caseweights <-xx2$caseweights[-fentry]
			xx2$strata <-xx2$strata[-fentry]
			xx2$jumps <- xx2$jumps-first
		}
	} ## }}}

	jumps <- xx2$jumps+1
	typexx2 <- xx2$Z[,6]
	Xj <- xx2$X[jumps,,drop=FALSE]
	xx2$nstrata <- nstrata
	jumptimes <- xx2$time[jumps]
	strata1jumptimes <- xx2$strata[jumps]
	if ((length(whereC)>0)) {
		###
		rr0 <- xx2$sign*(typexx2==1)
		jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)
		strataCxx2 <- xx2$Z[,2]
		S0iC2  <-  S0iC <- rep(0,length(xx2$status))
		S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
		S0iC[jumpsC] <- 1/S0rrr[jumpsC]
		S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
                ## Gc(t) computed  along all times of combined data-set: data + [D,\infty]
		## not lag because it is lagged later in GcjumpsR 
                Gcxx2 <- exp(cumsumstratasum(log(1 - S0iC), strataCxx2, nCstrata)$sum)
		Gstart <- rep(1,nCstrata)
		Gjumps <- Gcxx2[jumps]

		njumps <- length(jumps)
		GtsS0ooN <-   .Call("_mets_GcjumpsR",Gcxx2,c(xx2$status),strataCxx2,nCstrata,Gstart,njumps)
		Gts <- GtsS0ooN$Gcjumps
	} else {
		Gcxx2 <- rep(1,length(xx2$sign))
		strataCxx2 <- rep(0,length(xx2$sign))
		nCstrata <- 1
		Gstart <- 1
		Gjumps <- 1
		Gts <- matrix(1,length(jumps),1)
	} 

	obj <- function(pp,all=FALSE) { ## {{{

		rr2 <- c(xx2$sign*exp(xx2$X %*% pp + xx2$offset)*xx2$weights)
		rr2now <- c(xx2$sign*exp(xx2$X %*% pp + xx2$offset))
		S0oo <-   .Call("_mets_S0_FGRN",rr2,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gts)$S0
		f  <-  function(x) {
			ll <-   .Call("_mets_S0_FGRN",x,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gts)$S0
		}

		S1oo  <- apply(xx2$X*rr2,2,f)
		S2oo  <- apply(xx2$XX*rr2,2,f)

		S0 <- c(S0oo) ## [jumps,]
		S1 <- S1oo ## [jumps,,drop=FALSE]
		S2 <- S2oo ## [jumps,,drop=FALSE]

		E <- S1/S0
		weightsJ <- xx2$weights[jumps]
		caseweightsJ <- xx2$caseweights[jumps]
		strataJ <- xx2$strata[jumps]
		rr2now <- rr2now[jumps]
		U <- (Xj-E)
		ploglik <- (log(rr2now)-log(S0))*weightsJ*caseweightsJ;

		if (!is.null(propodds)) {
			pow <- c(.Call("cumsumstrataPOR",weightsJ,S0,strataJ,nstrata,propodds,rr2now,PACKAGE="mets")$pow);
			DLam <-.Call("DLambetaR",weightsJ,S0,E,Xj,strataJ,nstrata,propodds,rr2now,PACKAGE="mets")$res;
			Dwbeta <- DLam*rr2now+(pow-1)*Xj
			DUadj  <- .Call("vecMatMat",Dwbeta,U,PACKAGE="mets")$vXZ
		}

		Ut <- caseweightsJ*weightsJ*U
		## E^2, as n x (pxp)
		Et2 <-  .Call("vecCPMat",E,PACKAGE="mets")$XX
		S2S0 <-  S2/S0
		DUt <-  -(S2S0-Et2)
		np <- length(pp)

		if (!is.null(propodds)) {
			Ut  <- pow*Ut
			S0 <- S0/pow
			DUt <- pow*DUt
			DUt <- .Call("XXMatFULL",DUt,np,PACKAGE="mets")$XXf
			if (ncol(DUt)>0) DUt <- DUt+DUadj 
			ploglik <- pow*ploglik
		}

		U  <- apply(Ut,2,sum)
		DUt <- caseweightsJ*weightsJ*DUt
		DU <- -apply(DUt,2,sum)
		np <- length(pp)
		if (ncol(DUt)!=p*p) {
			DU <- matrix(.Call("XXMatFULL",matrix(DU,nrow=1),np,PACKAGE="mets")$XXf,np,np)
		} else  DU <- matrix(DU,p,p)
		ploglik <- sum(ploglik)
		U <- U+augmentation

		out <- list(ploglik=ploglik,gradient=U,hessian=-DU,cox.prep=xx2,
			    hessiantime=DUt,weightsJ=weightsJ,caseweightsJ=caseweightsJ,
			    jumptimes=jumptimes,strata=strataJ,nstrata=nstrata,S0s=S0,
			    time=jumptimes,S0=S0/(caseweightsJ*weightsJ),S2S0=S2S0,E=E,U=Ut,X=Xj,Gjumps=Gjumps)

		nn <- length(xx2$time)
		if (all)
			return(out)
		else
			with(out,structure(-ploglik/nn, gradient=-gradient/nn, hessian=-hessian/nn))
	} ## }}}

	if (length(jumps)==0) no.opt <- TRUE
	opt <- NULL
	if (p>0) {
		if (no.opt==FALSE) {
			if (tolower(method)=="nr") {
				opt <- lava::NR(beta,obj,...)
				opt$estimate <- opt$par
			} else {
				opt <- nlm(obj,beta,...)
				opt$method <- "nlm"
			}
			cc <- opt$estimate;  names(cc) <- colnames(X)
			if (stderr==2) return(cc)
			val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
		} else val <- c(list(coef=beta),obj(beta,all=TRUE))
	} else {
		no.opt <- TRUE
		val <- obj(0,all=TRUE)
	}

	beta.s <- val$coef
	if (is.null(beta.s)) beta.s <- 0
	## getting final S's
	opt <-  val ## obj(beta.s,all=TRUE)

	if (p>0) {
		iH <- - tryCatch(pinv(opt$hessian),error=function(e) matrix(0,nrow(opt$hessian),ncol(opt$hessian)) )
		opt$ihessian <- iH
		opt$no.opt <- FALSE
		dd <- IIDrecreg(xx2,opt,cause=cause,cens.code=cens.code,death.code=death.code,adm.cens=adm.cens.time) 

		Uiid <-  dd$beta.iid.naive 
		UUiid <- dd$beta.iid
		UU <- dd$MGt
		MGc <- dd$MGc

		Uiid <-  (UU+MGc) %*% iH
		UUiid <- UU %*% iH
		var1 <-  crossprod(UUiid)
		varmc <-  crossprod(Uiid)

		## compute regression augmentation for censoring martingale 
		if ((!is.null(augment.model)) & (length(whereC)>0) ) {# ## {{{ 

			CovZXstrata <- function(X,Ej,Z,Sign,strata,nstrata,jumps) 
			{
				strata  <- c(strata); Sign <- c(Sign)
				###	Ej <- Ej[jumps,,drop=FALSE]; Ej <- Ej
				ZE <- apply(Z*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
				XZ  <- .Call("vecMatMat",X,Z)$vXZ;  
				XZ <- apply(XZ*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
				EXZ  <- .Call("vecMatMat",Ej,ZE)$vXZ;  
				out <- XZ-EXZ
				return(out)
			}

			## regress U(s)=\int_s^\infty (Z-E) w(s) dM(s) on agument-model among survivors 
			## U(s) = U(\infty) - \int_0^s (Z-E) w(s)  dM(s)
			## sum (e_i - \bar e) U(s) Y_i(s)

			# construct censoring weights going along with all data, with added [D,\infty], start stop
			rr0 <- c(xx2$sign)*(typexx2==1)
			jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)
			strataCxx2 <- xx2$Z[,2]
			S0iC2  <-  S0iC <- rep(0,length(xx2$status))
			S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
			S0iC[jumpsC] <- 1/S0rrr[jumpsC]
			S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
			S0c <- c(S0rrr[jumpsC])
			## Gc(t) computed  as exp(- Cumhaz) to avoid some "0"s
			Gcj <- Gcxx2 <- exp(-cumsumstrata(S0iC,strataCxx2,nCstrata))[jumpsC]
			#

			XXA <- xx2$Z[,-(1:6),drop=FALSE]
			EXXA <- apply(XXA*c(rr0),2,revcumsumstrata,strataCxx2,nCstrata)
			EA <- EXXA[jumpsC,,drop=FALSE]/S0rrr[jumpsC]
			UA <- (XXA[jumpsC,,drop=FALSE]-EA)

			###
			E2A <- .Call("vecMatMat",EA,EA)$vXZ;  
			XX2A <- .Call("vecMatMat",XXA,XXA)$vXZ;  
			S2A <- apply(XX2A*c(rr0),2,revcumsumstrata,strataCxx2,nCstrata)
			###
			hessiant <- -(S2A[jumpsC,,drop=FALSE]/S0c-E2A)
			hesst <- hessiant

			### X fra GL + tail-death 
			rr <- c(exp(xx2$X %*% beta.s+ xx2$offset)*xx2$weights)*(typexx2==1)
			Zrr <- xx2$X*rr
			ZEdN <- apply(dd$Ut,2,revcumsumstrata,xx2$id,nid)

			covXsZ <-   CovZXstrata(XXA,EA,Zrr,rr0,strataCxx2,nCstrata,jumpsC) 
			covXsrr <-  CovZXstrata(XXA,EA,as.matrix(rr,ncol=1),rr0,strataCxx2,nCstrata,jumpsC) 
			covXsUs3 <- .Call("vecMatMat",covXsrr,dd$EdLam0[jumpsC,,drop=FALSE])$vXZ;  
			covXsUs2 <- covXsZ*dd$cumhaz[jumpsC]-covXsUs3 
			### U(infty)= UU
			Uinfiid <- UU[xx2$id+1,,drop=FALSE]
			fid <- headstrata(xx2$id,nid)
			cZEdN <- ZEdN[fid,,drop=FALSE][xx2$id+1,,drop=FALSE]-ZEdN
			Us1 <- Uinfiid-cZEdN
			covXsUs1 <- CovZXstrata(XXA,EA,Us1,rr0,strataCxx2,nCstrata,jumpsC) 
			## scale with Y_(s) because hessiantime is also scaled with this 
			covXsYs <- (covXsUs1+covXsUs2)/S0c; ## /c(cr2$S0)

			pXXA <- ncol(XXA)
			gammat <-  .Call("CubeMattime",hesst,covXsYs,pXXA,pXXA,pXXA,p,1,0,0,PACKAGE="mets")$XXX
			gammat[is.na(gammat)] <- 0
			gammat[gammat==Inf] <- 0
			namesG <- c(); for (i in 1:p) namesG <- c(namesG,paste(namesXXA,"-e",i,sep=""))
			colnames(gammat) <- namesG
			augmentt <- .Call("CubeMattime",gammat,UA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
			augment.times <- -apply(augmentt,2,sum)
			gain.times <- .Call("CubeMattime",covXsYs,gammat,pXXA,p,pXXA,p,0,1,0,PACKAGE="mets")$XXX
			gain.times <- matrix(apply(gain.times,2,sum),p,p)
			var.augment.times <-  gain.times 

			###
			time.gammat <- timeC <- xx2$time[jumpsC]
			if (is.null(ftime.augment)) {
				### simple default parabola
				maxt <- max(timeC)
				ftime <- timeC*(timeC-maxt)/maxt^2
			} else { 
				if (is.list(ftime.augment)) ftime <- ftime.augment[[1]](timeC) else ftime <- ftime.augment(timeC)
				if (length(ftime.augment)==2) {
					timepar <- ftime.augment[[2]](timeC)
					parap <- lm(gammat~-1+timepar)
					gammat <- parap$fitted.values
				}
			}
			ftime.gamma <- ftime
			varZdN <- matrix(apply(ftime^2*hesst/c(Gcj^2),2,sum),pXXA,pXXA)
			covXYdN <- matrix(apply(ftime*covXsYs/c(Gcj),2,sum),p,pXXA,byrow=TRUE) 
			gamma <- -1*.Call("CubeMattime",matrix(varZdN,nrow=1),matrix(covXYdN,nrow=1),pXXA,pXXA,p,pXXA,1,0,1,PACKAGE="mets")$XXX
			gamma <- matrix(gamma,p,pXXA,byrow=TRUE)
			gamma[is.na(gamma)] <- 0; gamma[gamma==Inf] <- 0
			colnames(gamma) <- namesXXA
			augment <- c(gamma %*% apply(ftime*UA/c(Gcj),2,sum))
			var.augment  <-  gamma %*% t(covXYdN) ###  /(nid^2)

			if (!is.null(augmentation.call)) { ## update variance when called with augmentation
				#### iid magic  for censoring augmentation martingale
				### int_0^infty gamma (e_i - ebar(s)) 1/G_c(s) dM_i^c
				S0iG <- S0i <- rep(0,length(xx2$strata))
				S0iG[jumpsC] <- ftime/(S0rrr[jumpsC]*c(Gcj))
				S0i[jumpsC] <- c(1/S0rrr[jumpsC])
				U <- E <- matrix(0,nrow(xx2$X),pXXA)
				E[jumpsC,] <- EA; 
				U[jumpsC,] <- ftime*UA/c(Gcj)
				cumhaz <- cumsumstrata(S0iG,strataCxx2,nCstrata)
				EdLam0 <- apply(E*S0iG,2,cumsumstrata,strataCxx2,nCstrata)
				MGCt <- U[,drop=FALSE]-(XXA*c(cumhaz)-EdLam0)*c(rr0)
				MGCtiid <- apply(MGCt,2,sumstrata,xx2$id,nid)
				iid.augment <-  (MGCtiid %*% t(gamma)) %*% iH

				gammasEs <- .Call("CubeMattime",gammat,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
				gammasE <- matrix(0,nrow(XXA),p)
				gammatt  <-    matrix(0,nrow(XXA),pXXA*p)
				gammasE[jumpsC,] <- gammasEs
				gammatt[jumpsC,] <- gammat
				gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,strataCxx2,nCstrata)
				gammadLam0 <-   apply(gammatt*S0i,2,cumsumstrata,strataCxx2,nCstrata)
				XgammadLam0 <- .Call("CubeMattime",gammadLam0,XXA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
				Ut <- Et <- matrix(0,nrow(XXA),p)
				Ut[jumpsC,] <- augmentt
				MGCtt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
				MGCttiid <- apply(MGCtt,2,sumstrata,xx2$id,nid)
				iid.augment.times <-  MGCttiid %*% iH
				Uiid.augment <- Uiid-iid.augment
				Uiid.augment.times <- Uiid-iid.augment.times
				## so that iid corresponds to dynamic censorig augmentation 
				Uiid <- Uiid.augment.times
				## scale with G_c(t) to compare with gamma
				gammat <- gammat * c(Gcj)

				var.augment <-  varmc  -  iH %*% var.augment %*% iH
				var.augment.times <-  varmc  +  iH %*% var.augment.times %*% iH
				varmc <- var.augment.times
				var.augment.iid <-  crossprod(Uiid.augment) 
				var.augment.times.iid <- crossprod(Uiid.augment.times) 
			} else {
				var.augment <-  var.augment.times <-  var.augment.iid <-  var.augment.times.iid <- NULL
				Uiid.augment.times <- Uiid.augment <- NULL
			}
		} else {
			iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
			var.augment.times <- var.augment <- NULL
			var.augment.times.iid <- var.augment.iid <- NULL
			Uiid.augment.times <- Uiid.augment <- NULL
			time.gammat <- gamma <- gammat <- NULL
			ftime.gamma <- NULL
			Gcj <- NULL
		}  ## }}}
		### end if (p>0)
	} else {
		iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
		var.augment.times <- var.augment <- NULL
		var.augment.times.iid <- var.augment.iid <- NULL
		Uiid.augment.times <- Uiid.augment <- NULL
		time.gammat <- gamma <- gammat <- NULL
		ftime.gamma <- NULL
		Gcj <- NULL
		varmc <- var1 <- NULL; MGc <- iH <- UUiid <- Uiid <- NULL
	}
	strata <- xx2$strata[jumps]
	cumhaz <- cbind(opt$time,cumsumstrata(1/opt$S0,strata,nstrata))
	colnames(cumhaz)    <- c("time","cumhaz")

	## SE of estimator ignoring some censoring terms
	if (no.opt==FALSE & p!=0) {
		DLambeta.t <- apply(opt$E/c(opt$S0),2,cumsumstrata,strata,nstrata)
		varbetat <-   rowSums((DLambeta.t %*% iH)*DLambeta.t)
		### covariance is 0 for cox model
		### covv <-  apply(covv*DLambeta.t,1,sum) Covariance is "0" by construction
	} else varbetat <- 0
	var.cumhaz <- cumsumstrata(1/opt$S0^2,strata,nstrata)+varbetat
	se.cumhaz <- cbind(jumptimes,(var.cumhaz)^.5)
	colnames(se.cumhaz) <- c("time","se.cumhaz")

	if (!is.null(name.id)) {
		MGc <-  nameme(MGc,name.id)
		Uiid <- nameme(Uiid,name.id)
		UUiid <- nameme(UUiid,name.id)
		Uiid.augment <- nameme(Uiid.augment,name.id)
		Uiid.augment.times  <- nameme(Uiid.augment.times,name.id)
	}

	if (p==0) {beta.s <- NULL; varmc <- NULL;se.coef <- NULL} else se.coef <- diag(varmc)^.5

	out <- list(coef=beta.s,var=varmc,se.coef=se.coef,iid.naive=UUiid,
                    iid=Uiid,ncluster=nid,ihessian=iH,hessian=opt$hessian,var1=var1,se1.coef=diag(var1)^.5,
		    hessianttime=opt$hessianttime,
		    ploglik=opt$ploglik,gradient=opt$gradient,
		    cumhaz=cumhaz,se.cumhaz=se.cumhaz,MGciid=MGc,
		    id=orig.id,call.id=call.id,name.id=name.id,nid=nid,
		    strata.jumps=opt$strata,strata=xx2$strata,
		    nstrata=nstrata,strata.name=strata.name,strata.level=strata.level,
		    propodds=propodds,
		    S0=opt$S0,E=opt$E,S2S0=opt$S2S0,time=opt$time,Ut=opt$U,
		    jumps=jumps,exit=exit.call,
		    p=p,S0s=val$S0s,
		    no.opt=no.opt,##n=nrow(X),nevent=length(jumps),
		    Pcens.model=Pcens.model,Gjumps=Gjumps,
		    cens.code=cens.code,death.code=death.code,cause=cause,
		    augmentation=augmentation.call,
		    var.augment=var.augment,var.augment.times=var.augment.times,
		    var.augment.iid=var.augment.iid,var.augment.times.iid=var.augment.times.iid,
		    lin.augment=c(augment),lindyn.augment=c(augment.times),
		    iid.augment=Uiid.augment,iid.augment.times=Uiid.augment.times,
		    gamma=gamma, gamma.times=gammat, time.gammat=time.gammat,ftime.gamma=ftime.gamma,Gcj=Gcj,
		    adm.cens.time=adm.cens.time,cens.weights=Stime
		    )

	if (cox.prep) out <- c(out,list(cox.prep=xx2))
	if (twostage) out <- c(out,list(cox.prep.twostage=xxg))

	return(out)
} ## }}}

IIDrecreg <- function(coxprep,x,time=NULL,cause=1,cens.code=0,death.code=2,fixbeta=NULL,beta.iid=NULL,adm.cens=NULL,tminus=FALSE)
{ ## {{{
	if (is.null(fixbeta)) 
	if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0

	xx2 <- coxprep
	status <- xx2$Z[,1]
	cause.jumps <- xx2$jumps+1 
	exit <- xx2$time
	max.jump <- max(exit[cause.jumps])+1
	other <- (length(which(status %in% death.code)) > 0)
	whereC <- which( (status %in% cens.code) & xx2$sign==1)
        anyC <- (length(whereC)>0)

	# construct censoring weights going along with all data, with added [D,\infty], start stop
	jumps <- xx2$jumps+1
	jumptimes <- xx2$time[jumps]
	strata1jumptimes <- xx2$strata[jumps]
	Xj <- xx2$X[jumps,,drop=FALSE]
	###
	typexx2 <- xx2$Z[,6]
	rr0 <- xx2$sign*(typexx2==1)
	jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)

	if (anyC) {
		strataCxx2 <- xx2$Z[,2]
		S0iC2  <-  S0iC <- rep(0,length(xx2$status))
		nCstrata <- max(strataCxx2)+1
		S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
		if (length(jumpsC)>0) {
			S0iC[jumpsC] <- 1/S0rrr[jumpsC]
			S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
		}
		## Gc(t) computed  along all times of combined data-set: data + [D,\infty] 
		Gcxx2 <- exp(cumsumstrata(log(1-S0iC),strataCxx2,nCstrata))
		Gstart <- rep(1,nCstrata)
		Gjumps <- Gcxx2[jumps]
	} else  Gcxx2 <- rep(1,length(xx2$strata))
	### all administrative censoring, or no censoring at all 
	if (!anyC) typexx2 <- 1

	### iid version given G_c when covariates are there, iid robust 
	S0i <- rep(0,length(xx2$strata))
	S0i[jumps] <- 1/x$S0
	Z <- xx2$X
	p <- ncol(x$E)
	if ( (!is.null(beta.iid)) | fixbeta==0) {
		U <- E <- matrix(0,nrow(Z),p)
		E[jumps,] <- x$E
		U[jumps,] <- x$U
		EdLam0 <- apply(E*S0i,2,cumsumstrata,xx2$strata,xx2$nstrata)
	} else U <- NULL
	cumhazA <- cumsumstratasum(S0i,xx2$strata,xx2$nstrata,type="all")
	cumhaz <- c(cumhazA$sum)

	## make indicator for times of interest for integrals up to time 
	if (!is.null(time))  btimexx <- (xx2$time<time) else btimexx <- rep(1,length(xx2$time))

	if (fixbeta==0) {
		rr <- c(xx2$sign*exp(Z %*% x$coef + xx2$offset))
		Ht <- apply(E*S0i*btimexx,2,cumsumstrata,xx2$strata,xx2$nstrata); 
	} else { Ht <- NULL; rr <- c(xx2$sign*exp(xx2$offset)) }
	rrw <- rr*c(xx2$weights)

	mid <- max(xx2$id)
	if ( (!is.null(beta.iid)) | fixbeta==0) {
		### Martingale  as a function of time and for all subjects to handle strata
		MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rrw*(typexx2==1)
		UU <- apply(MGt,2,sumstrata,xx2$id,mid+1)
	} else UU <- 0

	if (!is.null(time)) {
		## baseline
		MGAiid <- NULL
		S0i2 <- rep(0,length(xx2$strata))
		ww <- xx2$caseweights*xx2$weights
		S0i2[jumps] <- 1/(x$S0^2*ww[jumps])
		MGAiid <- matrix(0,length(S0i2),1)
		MGAiid2 <- matrix(0,length(S0i2),1)
		cumhazAA <- cumsumstrata(S0i2*btimexx,xx2$strata,xx2$nstrata)
		MGAiid <- S0i*btimexx-cumhazAA*rrw*(typexx2==1)
	} else MGAiid <- NULL

	if (other & anyC) { ## "martingale part" for type-2 after T ## {{{
	   ## tail part with \int (Z_i-E) w_i(t) dM_i = \int_D_i^\tau (Z_i-E) Gc(t) dM_i/Gc(D_i) 
	   rrw2 <- rrw*(typexx2==2)
	   GdL <- c(cumsum2strata(Gcxx2,S0i,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)$res)
	   fff <- function(x) {
		cx  <- cumsum2strata(Gcxx2,x*S0i,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)$res
		return(cx)
	   }
           if ((!is.null(beta.iid)) | fixbeta==0) EGdL  <- apply(E,2,fff)

           if ( ((!is.null(beta.iid)) | fixbeta==0)  ) {
		MGt2  <- -(Z*GdL-EGdL)*rrw2
		UU2 <- apply(MGt2,2,sumstrata,xx2$id,mid+1)
		UU  <-  UU+UU2
	   }

	   dstrata <- mystrata(data.frame(strataCxx2,xx2$strata))
	   ndstrata <- attr(dstrata,"nlevel")
	   lastt <- tailstrata(dstrata-1,ndstrata)

	   if (!is.null(time) ) {
	   HBt <- cumsum2strata(Gcxx2,S0i2*btimexx,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)
	   HBtinf <- HBt$res[lastt][dstrata]-HBt$res
		## baseline
		MGAiid2 <- -HBt$res*c(rrw2)
		MGAiid <- MGAiid+MGAiid2
	   }
	   if ( ((!is.null(beta.iid)) | fixbeta==0) ) {
		Htinf <- GdL[lastt][dstrata]-GdL
		ff <- function(x) x[lastt][dstrata]-x
		EHtinf  <- apply(EGdL,2,ff)
	   }
	} # ## }}}

	if (!is.null(time) )  MGAiid <- apply(MGAiid,2,sumstrata,xx2$id,mid+1)

	## censoring terms for influene functions 
	if (other & anyC) { ##  ## {{{
		### Censoring adjustment for jumps of other type but only for KM-case 
		### first time we see them with type2 event 

		if (is.null(adm.cens)) {
                   rrw2j <- -c(rrw2*(xx2$sign==-1))
		   Xos <- Z*rrw2j
		   rrsx <- cumsumstrata(rrw2j,strataCxx2,nCstrata)
		   Xos <- apply(Xos,2,cumsumstrata,strataCxx2,nCstrata)

		   if ( (!is.null(beta.iid)) | fixbeta==0) q2 <- (Xos*c(Htinf)-EHtinf*c(rrsx))
		   if (!is.null(time))  qB2 <- rrsx*c(HBtinf) 
		} else {
                   lastid <- tailstrata(xx2$id,mid+1)	
	 	   GdLast <- GdL[lastid][xx2$id+1]
		   EGdLast <- EGdL[lastid,,drop=FALSE][xx2$id+1,,drop=FALSE]
                        
                   rrw2j <- -c(rrw2*(xx2$sign==-1))
		   Xos <- Z*rrw2j
                   rrsx <- cumsumstrata(rrw2j,strataCxx2,nCstrata)
		   Xos <- apply(Xos,2,cumsumstrata,strataCxx2,nCstrata)

		   GadmXE2 <-  apply(GdLast*Z*rrw2j,2,cumsumstrata,strataCxx2,nCstrata)
		   EGadmRR2 <- apply(EGdLast*rrw2j,2,cumsumstrata,strataCxx2,nCstrata)
		   qq <- (GadmXE2-EGadmRR2)-(c(GdL)*Xos-EGdL*c(rrsx))

                   if ( (!is.null(beta.iid)) | fixbeta==0) q2 <- qq
                   if (!is.null(time))  {
		       HBlast <- HBt$res[lastid][xx2$id+1]
	               HBadmRR2 <- cumsumstrata(HBlast*rrw2j,strataCxx2,nCstrata)
		       qB2 <- (HBadmRR2-HBt$res*c(rrsx))
                   }

              }

		sss <- headstrata(dstrata-1,ndstrata)
		fff <- function(x) {
		 gtstart <- x[sss]
		 cx  <- cumsum2strata(x,S0iC2,dstrata-1,ndstrata,strataCxx2,nCstrata,gtstart)$res
		 return(cx)
		}

		### Martingale  as a function of time and for all subjects to handle strata
		if ( (!is.null(beta.iid)) | fixbeta==0) {
			EdLam0q2 <- apply(q2,2,fff)
			MGc <- q2*S0iC-EdLam0q2*c(xx2$sign)*(typexx2==1)
			MGc <- apply(MGc,2,sumstrata,xx2$id,mid+1)
		}
		if (!is.null(time) ) {
			EBdLam0q2 <- apply(qB2,2,fff)
			MGBc <- qB2*S0iC-EBdLam0q2*c(xx2$sign)*(typexx2==1)
			MGBc <- apply(MGBc,2,sumstrata,xx2$id,mid+1)
		}

	} else { MGc <- 0; MGBc <- 0}   ## }}}

	if (!is.null(time) & anyC) { MGAiid <- MGAiid+MGBc }  

	if ( (!is.null(beta.iid)) | fixbeta==0) {
		Uiid <-  (UU+MGc) %*% x$ihessian
		Uiid.naive <-  (UU) %*% x$ihessian
	} else {
		Uiid <- beta.iid
		Uiid.naive <- NULL
	} 

	if ( (!is.null(beta.iid)) | fixbeta==0) { #
		Htlast <- tailstrata(xx2$strata,xx2$nstrata)
		HtS <- Ht[Htlast,,drop=FALSE]
	} #

	## sum after id's within strata and order 
	if (!is.null(time))  {
		MGAiids <- c()
		cumhaz.time <- c()
		sus <- sort(unique(xx2$strata))
		fid <- headstrata(xx2$id,mid+1)
		wis <- xx2$strata[fid]

		for (i in sus)  { 
			ws <- 1*(wis==i)
			##
			cumhazs <- rbind(0,x$cumhaz[x$strata[x$jumps]==i,])
			cumhaz.time <- c(cumhaz.time,cpred(cumhazs,time,tminus=tminus)[,2])

			if (fixbeta==0) {
				UU <-  apply(HtS[i+1,]*t(Uiid),2,sum)
				MGAiidl <- ws*MGAiid - UU
			} else MGAiidl <- ws*MGAiid 
			MGAiids <- cbind(MGAiids,MGAiidl)
		}
		colnames(MGAiids) <- paste("strata",sus,sep="")
		names(cumhaz.time) <- paste("strata",sus,sep="")
	} else { sus <- MGAiids <- cumhaz.time <- NULL }

	if (!is.null(x$name.id)) {
		MGAiids <- nameme(MGAiids,x$name.id)
		Uiid    <- nameme(Uiid,x$name.id)
	}

	if (inherits(x,c("cifreg","recreg"))) {
		out <- list(time=time,base.iid=MGAiids,nstrata=xx2$nstrata, beta.iid=Uiid,
			    strata.call=x$strata.call,id=xx2$id,call.id=x$call.id,
			    coef=coef(x),cumhaz=x$cumhaz,cumhaz.strata=x$strata[x$jumps],
			    cumhaz.time=cumhaz.time,strata.time=sus,
			    nstrata=x$nstrata,strata.name=x$strata.name,strata.level=x$strata.level,
			    formula=x$formula,Ut=U)
	} else {
		out <- list(time=time,base.iid=MGAiid,id=xx2$id,beta.iid=Uiid,beta.iid.naive=Uiid.naive, MGt=UU,MGc=MGc,Ut=U,EdLam0=EdLam0,cumhaz=cumhaz)
	}
	class(out) <- "iidBaseline"
	return(out)
}  ## }}}

##' @export
iidBaseline.recreg <- function(object,time=NULL,ft=NULL,fixbeta=NULL,beta.iid=object$iid,tminus=FALSE,...)
{ ## {{{
	if (is.null(object$cox.prep)) stop("must call cifreg/recreg with cox.prep=TRUE\n")
	out <- IIDrecreg(object$cox.prep,object,time=time,fixbeta=fixbeta,beta.iid=beta.iid,adm.cens=object$adm.cens,tminus=tminus,...)
	out$design <- object$design
	return(out)
}  ## }}}

##' @export
GLprediid <- function(...)
{
	out <- FGprediid(...,model="GL")
	return(out)
}

##' @export
IC.recreg <- function(x, time = NULL, ..., wwww) {
    if (!is.null(time)) {
        iid <- IC.phreg(x, time = time, ...)
        return(iid)
    }
    if (is.null(x$iid)) stop("supply 'time' argument")
    res <- with(x, iid * NROW(iid))
    attr(res, "coef") <- coef(x)
    return(res)
}

##' @export
estimate.recreg <- function(x, ..., time = NULL, all=FALSE, baseline.args = list()) {
  if (NCOL(model.matrix(x))==0L & is.null(time)) stop("Non-parametric model; need `time` argument")
  ic <- do.call(IC, c(list(x, time = time,all=all), baseline.args))
  cc <- attr(ic, "coef")
  if (is.null(cc)) cc <- coef(x)
###  lab <- names(cc)
  if (!is.null(time)) {
    lab <- x$strata.level
    if (is.null(lab)) lab <- "mean"
  }
  b <- lava::estimate( coef = cc, IC = ic)
  return(lava::estimate(b, ...))
}



##' @export
plot.recreg <- function(x,se=FALSE,ylab=NULL,...) { #
	if (inherits(x,"recreg") & is.null(ylab)) ylab <- "Mean number"
	if (!se) baseplot(x,se=se,ylab=ylab,...)
	else {
		warning("Standard errors approximative (but too small), use predict and type='cumhaz' \n")
		baseplot(x,se=se,ylab=ylab,...)
	}
} #

##' @export
predict.recreg <- function(object,newdata=NULL,se=FALSE,times=NULL,np=50,...) { #
	if (!se) out <- predict.phreg(object,newdata,se=se,times=times,...)
	else {
		out <- predictrecreg(object,newdata,times=times,np=np,...)
	}
	class(out) <- c("predictrecreg",class(object)[1])
	return(out)
} #

##' @export
summary.predictrecreg <- function(object,type=c("cif","cumhaz","surv")[2],...) { ## {{{
   summary.predictphreg(object,type=type[1],...)
} ## }}}

##' @export
print.predictrecreg <- function(x,type=c("cif","cumhaz","surv")[2],...) 
{ ## }}}
 print.predictphreg(x,type=type[1],...)
} ## }}}


##' @export
plot.predictrecreg <- function(x,se=FALSE,ylab=NULL,type="cumhaz",...)
{ ## {{{
	if (inherits(x,"predictrecreg") & is.null(ylab)) ylab <- "Mean number"
	plotpredictphreg(x,se=se,ylab=ylab,type=type[1],...)
} ## }}}

predictrecreg <- function(x,newdata,times=NULL,individual.time=FALSE,tminus=FALSE,conf.type="log",conf.int=0.95,np=50,...)
{ ## {{{
	if (!inherits(x,c("cifreg","recreg","phreg")))  stop("only for phreg/recreg/cifreg models\n")
	se <- TRUE

	if (is.null(times))  {
		if (is.null(np)) times <- x$cumhaz[,1] else 
			times <- quantile(x$cumhaz[,1],probs=seq(0,1,length=np))
	} 
	if (!is.null(newdata)) xx <- update_design(x$design,data = newdata,response=FALSE) else xx <- x$design
	X <- xx$x
	des <- list()
	des$X <- X
	if (!is.null(xx$strata)) strataNew <- as.numeric(xx$strata)-1 else strataNew <- rep(0,nrow(X))
	des$strata <- strataNew

###	des <- readPhreg(x,newdata)

	if (x$p>0)  {
		RRj <- RR <- c(exp(des$X %*% x$coef))
		Xj <- X <- RR*des$X
	} else { RRj <- RR <- rep(1,length(des$strata)); Xj <- NULL}

	surv <- surv.upper <- surv.lower <- cift <- cif.upper <- cif.lower <- cumhaz <- se.cumhaz <- base.lower <- base.upper <- se.cif <- se.surv <- c()
	j <- 1
	for (tt in times) {
		bt <- iidBaseline(x,time=tt,tminus=tminus)
		covv <- crossprod(with(bt,cbind(base.iid,beta.iid)))
		if (individual.time)  {
			baset <- bt$cumhaz.time[des$strata[j]+1]
			Xbase <- 1*outer(des$strata[j],0:(bt$nstrata-1),"==")
			RRj <- RR[j]; 
			if (x$p>0) Xj <- X[j,,drop=FALSE]*baset else Xj <- NULL
		} else {  
			baset <- bt$cumhaz.time[des$strata+1]
			Xbase <- 1*outer(des$strata,0:(bt$nstrata-1),"==")
			if (x$p>0) Xj <- X*baset else Xj <- NULL
			RRj <- RR
		}
		j <- j+1
		Xall <- cbind(RRj*Xbase,Xj)
		###   
		seLamt <- apply((Xall %*% covv)* Xall,1,sum)^.5
		Lamt <- baset*RRj
		F1t <- 1-exp(-Lamt)
		St <- exp(-Lamt)
		seF1t <- St*seLamt
		seSt <- St*seLamt
		###   
		ciLam <- conftype(Lamt,seLamt,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
		ciSt <- conftype(St,seSt,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)
		ciF1 <- conftype(F1t,seF1t,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)

		surv <- cbind(surv,St)
		se.surv <- cbind(se.surv,seSt)
		se.cif <- cbind(se.cif,seF1t)
		surv.upper <- cbind(surv.upper,ciSt$upper)
		surv.lower <- cbind(surv.lower,ciSt$lower)
		cift <- cbind(cift,F1t)
		cif.upper <- cbind(cif.upper,ciF1$upper)
		cif.lower <- cbind(cif.lower,ciF1$lower)
		cumhaz <- cbind(cumhaz,Lamt)
		se.cumhaz <- cbind(se.cumhaz,seLamt)
		base.upper  <- cbind(base.upper,ciLam$upper)
		base.lower  <- cbind(base.lower,ciLam$lower)
	}

	out <- list(times=times,surv=surv,surv.upper=surv.upper,surv.lower=surv.lower,
		    cumhaz=cumhaz,se.cumhaz=se.cumhaz,cif=cift,cif.upper=cif.upper,cif.lower=cif.lower,
		    cumhaz.upper=base.upper,cumhaz.lower=base.lower,strata=des$strata,X=des$X,RR=RR,
		    se.cif=se.cif,se.surv=se.surv)

	return(out)
} ## }}}

##' IPCW Estimator for Recurrent Events
##'
##' Computes the Inverse Probability of Censoring Weighted (IPCW) estimator for the mean number of recurrent events.
##' Supports various estimators including the Ghosh-Lin and Lawless-Cook estimators.
##'
##' @param formula Formula with an 'Event' outcome.
##' @param data Data frame.
##' @param cause Cause of interest (default is 1).
##' @param cens.code Censoring code (default is 0).
##' @param death.code Death code (default is 2).
##' @param cens.model Formula for the censoring model (default is \code{~1}).
##' @param km Logical; if \code{TRUE}, uses Kaplan-Meier for censoring weights; otherwise uses Cox model.
##' @param times Time points for estimation (required).
##' @param beta Initial values for coefficients (optional).
##' @param offset Offsets.
##' @param type Type of estimator: \code{"II"} (default) or \code{"I"}.
##' @param marks Mark values.
##' @param weights Weights.
##' @param model Model type for the mean: \code{"exp"} (default) or \code{"lin"}.
##' @param no.opt Logical; if \code{TRUE}, skips optimization.
##' @param augmentation Augmentation terms.
##' @param method Optimization method (default is "nr").
##' @param se Logical; if \code{TRUE}, computes standard errors.
##' @param ... Additional arguments.
##'
##' @return An object of class \code{"binreg"} (extending \code{"resmean"}) containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Variance-covariance matrix.}
##' \item{iid}{Influence functions.}
##' \item{times}{Time points.}
##' \item{Y}{Observed IPCW weighted counts (increment IPCW) \deqn{\int_0^t (1/G_c(s)) dN(s)}.}
##'
##' @author Thomas Scheike
##' @seealso \code{\link{recreg}} 
##' @export
recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
		       cens.model=~1,km=TRUE,times=NULL,beta=NULL,offset=NULL,type=c("II","I"),
		       marks=NULL,weights=NULL,model=c("exp","lin"),no.opt=FALSE,augmentation=NULL,method="nr",se=TRUE,...)
{ ## {{{
   ## method=c("incIPCW","IPCW","rate")
   estimator=c("incIPCW")
   cl <- match.call()
   ## {{{ reading design 
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","marks"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster

  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  if (is.null(marks)) marks <- rep(1,length(id))
# }}}

        if (length(times) > 1) stop("recregIPCW currently supports only a single time point.")

	### setting up with artificial names
	data$status__ <-  status 
	data$id__ <-  id
	## lave Countcause
	data <- count_history(data,status="status__",id="id__",types=cause,multitype=TRUE)
	data$Count1__ <- data[,paste("Count",cause[1],sep="")]
	data$death__ <- (status %in% death.code)*1
	data$entry__ <- entry 
	data$exit__ <- exit 
	data$marks__ <- marks 
	statusC <- data$statusC__ <- (status %in% cens.code)*1
	data$status__cause <- (status %in% cause)*1
	data$rid__ <- revcumsumstrata(rep(1,length(entry)),id,nid)
	dexit <- exit
	dstatus <- status
	## to define properly 
	Dtime <- NULL

	formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ .+cluster(id__))
	cr <- phreg(formC,data=data,no.opt=TRUE,no.var=1)
	whereC <- which(status %in% cens.code)

	if (length(whereC)>0) {
		### censoring weights
		strat <- cr$strata[cr$jumps]
		x <- cr
		xx <- x$cox.prep
		S0i2 <- S0i <- rep(0,length(xx$strata))
		S0i[xx$jumps+1] <-  1/x$S0
		S0i2[xx$jumps+1] <- 1/x$S0^2
		## survival at t- to also work in competing risks situation
		if (!km) { 
			cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
			St      <- exp(-cumhazD)
		} else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
	} else  St <- rep(1,length(exit))
	Gc <- St

	###formD <- as.formula(Surv(entry__,exit__,death__)~cluster(id__))
	form1L <- as.formula(Surv(entry__,exit__,status__cause)~Count1__+death__+statusC__+cluster(id__))
	xr <- phreg(form1L,data=data,no.opt=TRUE,no.var=1,Z=matrix(data$marks__,ncol=1))
	###  xr0 <- phreg(form1,data=data,no.opt=TRUE)
	###  dr <- phreg(formD,data=data,no.opt=TRUE,no.var=1)
	###  clgl  <- recurrentMarginal(xr0,dr)
	###  plot(clgl)

	####  First partitioned estimator everywhere n^-1 sum_i \int_0^t m_i(s)/G_c(s) Y_i(s) I(D_i > s) dN_i(s) 
	x <- xr
	xx <- xr$cox.prep
	marksxx <- xx$Z[,1]
	jump1 <- xx$jumps+1
	timeJ <- xx$time[jump1]
	strataN1J <- xx$strata[jump1]
	### Partitioned estimator , same as Ghosh-Lin+Lawless-Cook estimator
	cumhazP <- c(cumsum(marksxx[jump1]/Gc[jump1])/nid)
	cumhazP <- cbind(timeJ,cumhazP)

	if (is.null(times)) stop("time for recurrent events regression must be given\n")

	### setting up regression setting with Y(t) =\int_0^t 1/G(s) dN_i(s)
	if (estimator[1]=="incIPCW") 
		Ydata <- Y <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times)/Gc,xx$id,nid)
	else if (estimator[1]=="IPCW")  {
		obs <- (exit<=time & (!statusC)) | (exit>=time)/Gc
		Ydata <- Y <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times),xx$id,nid)*obs
	} else {
		obs <- (exit<=time & (!statusC)) | (exit>=time)/Gc
		NtD <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times),xx$id,nid)
		Ydata <- Y <- obs*NtD/pmin(times,Dtime)
	}
	nevent <- sum((xx$status!=0)*(xx$time<times))

	lastrecord <- which(data$rid__==1)
	##  1 record per subject 
	Ydata <- Y 
	###
	if (is.null(offset)) offset <- rep(0, length(exit))
	if (is.null(weights)) weights <- rep(1, length(exit))
	###  
	Xorig <- X <- as.matrix(X)
	px <- ncol(X)
	if (is.null(augmentation))  augmentation=rep(0,px)

	## order after id, similar to Y
	oid <- order(id[lastrecord])
	Xdata <- X <- X[lastrecord,,drop=FALSE][oid,,drop=FALSE]
	offset <- offset[lastrecord][oid]
	weights <- weights[lastrecord][oid]
	status <- status[lastrecord][oid]
	exit <- exit[lastrecord][oid]
	idR <- id[lastrecord][oid]
	X2 <- .Call("vecMatMat", X, X)$vXZ
	ph <- 1
	if (is.null(beta)) beta <- rep(0,ncol(X))
	## take iid vession of data 
	dataiid <- data[lastrecord,][oid,]

	if (type[1]=="II") { 
		Gcdata <- suppressWarnings(predict(cr,data,times=dexit,individual.time=TRUE,se=FALSE,km=km,tminus=TRUE)$surv)
		Gcdata[Gcdata<0.000001] <- 0.00001
		Hst <- Y[data$id__ +1]-cumsumstratasum((dexit<times)*marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		#data$Hst <- cumsumstratasum(marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		data$Hst <- Hst
		formC <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
		desform <- update.formula(cens.model,as.formula(paste("~ Hst+ . + cluster(id__)")))
		formC[[3]] <- desform[[2]]

		resC <- phreg(formC,data=data,no.opt=TRUE,no.var=1,Z=Xorig)
		xx <- resC$cox.prep
		Xt <- xx$Z
		S0i2 <- S0i <- rep(0, length(xx$strata))
		S0i[xx$jumps+1] <- 1/resC$S0
		S0i2[xx$jumps+1] <- 1/resC$S0^2
		E <- matrix(0, nrow(xx$X), 1)
		E[xx$jumps+1,] <- resC$E*((resC$jumptimes<times)*1)
		btime <- c(1 * (xx$time < times))
		EdLam0 <- apply(E*c(S0i)*btime,2,cumsumstrata,xx$strata,xx$nstrata)

		mid <- max(idR)
		MGt <- Xt*c(E[, drop = FALSE] - EdLam0 * c(xx$sign))*c(xx$weights)
		MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
		augmentation <- augmentation+apply(MGtiid,2,sum)
		###        
		EXt  <-  apply(Xt*c(xx$sign),2,revcumsumstrata,xx$strata,xx$nstrata)
		IEXhYtdLam0 <- apply(EXt*c(E)*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
		U <- matrix(0,nrow(xx$X),ncol(X))
		U[xx$jumps+1,] <- (resC$jumptimes<times)*E[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)
		MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0*c(xx$sign))*c(xx$weights)
		###
		MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
		MGCiid2 <- MGtiid-MGCiid2
	}  else MGCiid2 <- 0   
	#

	obj <- function(pp, all = FALSE) { ## {{{
		lp <- c(X %*% pp + offset)
		if (model[1] == "exp") p <- exp(lp) else p <- lp
		if (model[1] == "dexp") p <- exp(lp) 
		if (model[1] == "dexp") {
			Dlogl <- weights * X *p * c(Y - p)
			D2logl <- c(weights) *p^2* X2
		} else {
			Dlogl <- weights *  X * c(Y - p)
			if (model[1]=="exp") D2logl <- c(weights) * c(p)* X2 else D2logl <- c(weights) * X2
		}
		if (model[1] == "exp") ploglik <- 0 else ploglik <- sum(weights * (Y - p)^2)
		D2log <- apply(D2logl, 2, sum)
		gradient <- apply(Dlogl, 2, sum) + augmentation
		hessian <- matrix(D2log, length(pp), length(pp))

		if (all) {
			ploglik <- sum(weights * (Y - p)^2)
			ihess <- pinv(hessian)
			beta.iid <- Dlogl %*% ihess
			beta.iid <- apply(beta.iid, 2, sumstrata, idR, max(id) + 1)
			robvar <- crossprod(beta.iid)
			val <- list(par = pp, ploglik = ploglik, gradient = gradient,
				    hessian = hessian, ihessian = ihess, id = idR,
				    Dlogl = Dlogl, iid = beta.iid, robvar = robvar,
				    var = robvar, se.robust = diag(robvar)^0.5)
			return(val)
		}
		structure(-ploglik, gradient = -gradient, hessian = hessian)
	} ## }}}

	## setting default for NR 
	dots <- list(...)
	if (length(dots)==0) {
		if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
		else control <- NULL
	} else control <- dots[[1]]

	p <- ncol(X)
	opt <- NULL
	if (p > 0) {
		if (no.opt == FALSE) {
			if (tolower(method) == "nr") {
				opt <- lava::NR(beta, obj,control=control)
				opt$estimate <- opt$par
			}
			else {
				opt <- nlm(obj, beta, ...)
				opt$method <- "nlm"
			}
			cc <- opt$estimate
			###            if (!se) return(cc)
			val <- c(list(coef = cc), obj(opt$estimate, all = TRUE))
		}
		else val <- c(list(coef = beta), obj(beta, all = TRUE))
	}
	else {
		val <- obj(0, all = TRUE)
	}
	if (length(val$coef) == length(colnames(X))) names(val$coef) <- colnames(X)

	val <- c(val, list(times = times, Y=Y, ncluster=nid, nevent=nevent,n=length(exit),X=X))


	if (se) {
		Gcdata <- suppressWarnings(predict(cr,data,times=dexit,individual.time=TRUE,se=FALSE,km=km,tminus=TRUE)$surv)
		Gcdata[Gcdata<0.000001] <- 0.00001
		## check data sorted in dexit 
		if (type[1]!="II") Hst <- Y[data$id__ +1]-cumsumstratasum((dexit<times)*marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		data$Hst <- Hst
		if (model[1]=="dexp") HstX <-c(exp(as.matrix(Xorig) %*% val$coef))*Xorig*c(data$Hst) else HstX <- Xorig*c(data$Hst) 
		ccn <- paste("nn__nn",1:ncol(Xorig),sep="")
		colnames(HstX) <- ccn
		nncovs <- c()
		for (i in 1:ncol(Xorig)) nncovs <- c(paste(nncovs,paste("+",ccn[i],sep="")))
		formC <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
		desform <- update.formula(cens.model,as.formula(paste("~",nncovs,"+ . + cluster(id__)")))
		formC[[3]] <- desform[[2]]

		data <- cbind(data,HstX)
		resC <- phreg(formC,data=data,no.opt=TRUE,no.var=1)
		xx <- resC$cox.prep
		S0i2 <- S0i <- rep(0, length(xx$strata))
		S0i[xx$jumps + 1] <- 1/resC$S0
		S0i2[xx$jumps + 1] <- 1/resC$S0^2
		E <- U <- matrix(0, nrow(xx$X), ncol(X))
		E[xx$jumps + 1, ] <- resC$E*((resC$jumptimes < times)*1)
		btime <- c(1 * (xx$time < times))
		EdLam0 <- apply(E*c(S0i)*btime,2,cumsumstrata,xx$strata,xx$nstrata)
		MGt <- (E[, drop = FALSE] - EdLam0 * c(xx$sign) )*c(xx$weights)
		MGCiid <- apply(MGt, 2, sumstrata, xx$id, max(id) + 1)
		#MGCiid <- MGCiid-MGCiid2
		MGCiid <- MGCiid+MGCiid2
	} else  MGCiid <- 0 


	val$call <- cl
	val$formula <- formula
	val$model <- model[1]
	val$model.type <- model[1]
	val$Y     <- Ydata
	val$Yipcw <- Ydata
	val$X <- Xdata
	val$id <- orig.id
	val$call.id <- call.id
	val$nid <- nid
	val$name.id <- name.id
	val$iid.naive <- val$iid
	val$naive.var <- val$var
	if (se) { 
		val$MGCiid <- MGCiid
		MGCiid <- MGCiid %*% val$ihessian 
		val$iid <- val$iid + MGCiid
	} else val$MGCiid <- MGCiid 
	oid <- order(name.id)  
	if (is.matrix(val$iid)) 
		if (length(name.id)==nrow(val$iid)) {
			rownames(val$iid) <- name.id
			val$iid <- val$iid[oid,,drop=FALSE]
		}
	if (is.matrix(val$MGCiid))  {
		if (length(name.id)==nrow(val$MGCiid)) rownames(val$MGCiid) <- name.id
		val$MGCiid <- val$MGCiid[oid,,drop=FALSE]
	}

	robvar <- crossprod(val$iid)
	val$var <- val$robvar <- robvar
	val$se.robust <- diag(robvar)^0.5
	val$se.coef <- diag(val$var)^0.5
	val$cens.code <- cens.code
	val$cause <- cause
	val$death.code <- death.code
	val$model.estimator <- estimator
	val$augmentation <- augmentation
	val$type <- type
	val$cumhazP <- cumhazP
	val$nevent <- nevent
	val$design <- des
	class(val) <- c("binreg", "resmean")
	return(val)
} ## }}}

strataAugment <- survival::strata

##' Simulation of Two-Stage Recurrent Events Data
##'
##' Simulates data based on Cox/Cox or Cox/Ghosh-Lin structures for recurrent events with a terminal event.
##'
##' \itemize{
##' \item \code{type=3}: Generates data from a Cox/Cox two-stage model.
##' \item \code{type=2}: Generates data from a Ghosh-Lin/Cox model.
##' }
##'
##' If \code{var.z=0}, data is generated without dependence or frailty.
##' If \code{model="twostage"}, the default is to generate data from a Ghosh-Lin/Cox model.
##' If \code{type=3}, data is generated with marginal Cox models (Cox/Cox).
##'
##' Simulation is based on a linear approximation of the hazard for two-stage models on a time grid.
##' The grid must be sufficiently fine.
##'
##' @param n Number of IDs.
##' @param base1 Baseline cumulative hazard for recurrent events.
##' @param drcumhaz Baseline cumulative hazard for the terminal event.
##' @param var.z Variance of the gamma frailty.
##' @param r1 Relative risk term for the recurrent event baseline.
##' @param rd Relative risk term for the terminal event.
##' @param rc Relative risk term for censoring.
##' @param fz Function for transformation of the frailty term.
##' @param fdz Function for transformation of the frailty term for death.
##' @param model Model type: \code{"twostage"}, \code{"frailty"}, or \code{"shared"}.
##' @param type Type of simulation (default depends on \code{model}).
##' @param cens Censoring rate for exponential censoring.
##' @param share Proportion of shared random effects (for partially shared models).
##' @param nmin Minimum number of points in the time grid (default 100).
##' @param nmax Maximum number of points in the time grid (default 1000).
##'
##' @return A data frame with simulated recurrent events and terminal events, including frailty terms.
##'
##' @author Thomas Scheike
##' @references Scheike (2025), Two-stage recurrent events random effects models, LIDA, to appear.
##' @export
sim_GLcox <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
		     model=c("twostage","frailty","shared"),type=NULL,share=1,cens=NULL,nmin=100,nmax=1000)
{ ## {{{
	## setting up baselines for simulations 
	maxt <- tail(base1[,1],1)
	base1 <- as.matrix(base1); drcumhaz <- as.matrix(drcumhaz)
	nmin <- max(nrow(base1),nrow(drcumhaz),nmin)
	nmin <- min(nmax,nmin)
	seqt <- seq(from=0,to=maxt,length.out=nmin)
	if (base1[1,1]!=0) base1 <- rbind(0,base1) 
	if (drcumhaz[1,1]!=0) drcumhaz <- rbind(0,drcumhaz) 
	base1 <- cbind(seqt, lin_approx(seqt,base1))
	cumD <- cbind(seqt, lin_approx(seqt,drcumhaz))
	###
	St <- exp(-cumD[,2])
	Stm <- cbind(base1[,1],St)
	###
	dbase1 <- diff(c(0,base1[,2]))
	dcum <- cbind(base1[,1],dbase1)
	maxtime <- tail(base1[,1],1)

	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	fz.orig <- fz
	if (is.null(fz)) fz <- function(x) x

	if (var.z[1]>0) {
		z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
		if (share<1) { 
			z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
			z <- z+z2
		} 
		fzz <- fz(z1)
		if (share<1) fzz <- fzz/share; 
		mza <- mean(fzz)
		if (n<10000 & (!is.null(fz.orig))) {
			zl <- rgamma(100000,share/var.z[1])*var.z[1] 
			fzl <- fz(z)
			mza <- mean(fzl)
		} 
	}  else fzz <- z <- z1 <- rep(1,n)

	if (isTRUE(var.z[1] == 0)) model <- "frailty"

	if (is.null(type)) 
		if (model[1]=="twostage") type <- 2 else type <- 1
	## for frailty setting we also consider any function of z 
	if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}

	## survival censoring given X, Z, either twostage or frailty-model 
	if (type>=2) stype <- 2 else stype <- 1
	if (var.z[1]==0) stype <- 1
	### dd <- .Call("_mets_simSurvZ",as.matrix(rbind(c(0,1),Stm)),rd,z,var.z[1],stype)
	dd <- .Call("_mets_simSurvZ",as.matrix(rbind(Stm)),rd,z,var.z[1],stype)
	dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
	if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
	dd$status <- ifelse(dd$time<cens,dd$status,0)
	dd$time <- pmin(dd$time,cens)

	## to avoid R check error
	reverseCountid  <-  death  <- NULL

	if (model[1]=="multiplicative") {
		## other random effect 
		z2 <- rgamma(n,share/var.z[2])*var.z[2] 
		fzz <- z1*z2
		type <- 3
	}
	## type=2 draw recurrent process given X,Z with rate:
	##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
	## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
	## type=3, observed hazards on Cox form among survivors
	## twostage shared<1: W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
	## twostage share=1: or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	## multiplicatve:       W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	dcum <- cbind(base1[,1],dbase1)
	### ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	ll <- .Call("_mets_simGL",as.matrix(dcum),c(St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	colnames(ll) <- c("id","start","stop","death")
	ll <- data.frame(ll)
	ll$death <- dd$status[ll$id+1]
	## add frailty to data for possible validation
	ll$z <- z1[ll$id+1]
	ll$fz <- fzz[ll$id+1]
	## add counts of id
	ids <- countID(ll)
	ll <- cbind(ll,ids[,c(2,4,5)]); 
	ll$status <- 1; 
	ll <- dtransform(ll,status=0,reverseCountid==1)
	ll$statusD <- ll$status
	ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)

	attr(ll,"base1events") <- base1
	attr(ll,"deathcumbase") <- cumD
	return(ll)
} ## }}}


sim_GLcoxC <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
		      model=c("twostage","frailty","shared","multiplicative"),type=NULL,share=1,cens=NULL,nmax=200,by=1)
{ ## {{{
	## setting up baselines for simulations 
	maxt <- tail(base1[,1],1)
	seqt <- seq(by,maxt,by=by)
	base1 <- predictCumhaz(rbind(0, as.matrix(base1)), seqt)
	cumD <- predictCumhaz(rbind(0, as.matrix(drcumhaz)), seqt)
	###
	St <- 1-cumD[,2]
	Stm <- cbind(base1[,1],St)
	###
	dbase1 <- diff(c(0,base1[,2]))
	dcum <- cbind(base1[,1],dbase1)
	maxtime <- tail(base1[,1],1)

	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	fz.orig <- fz
	if (is.null(fz)) fz <- function(x) x

	if (var.z[1]>0) {
		z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
		if (share<1) { 
			z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
			z <- z+z2
		} 
		fzz <- fz(z1)
		if (share<1) fzz <- fzz/share; 
		mza <- mean(fzz)
		if (n<10000 & (!is.null(fz.orig))) {
			zl <- rgamma(100000,share/var.z[1])*var.z[1] 
			fzl <- fz(z)
			mza <- mean(fzl)
		} 
	}  else fzz <- z <- z1 <- rep(1,n)

	if (var.z[1]==0) model <- "frailty"
	if (is.null(type)) 
		if (model[1]=="twostage") type <- 2 else type <- 1
	## for frailty setting we also consider any function of z 
	if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}

	## survival censoring given X, Z, either twostage or frailty-model 
	if (type>=2) stype <- 2 else stype <- 1
	cumHazS <- -log(St)
	dd <- rchaz(cumHazS,rd)
	dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
	if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
	dd$status <- ifelse(dd$time<cens,dd$status,0)
	dd$time <- pmin(dd$time,cens)

	## to avoid R check error
	reverseCountid  <-  death  <- NULL
	## type=2 draw recurrent process given X,Z with rate:
	##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
	## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
	## type=3, observed hazards on Cox form among survivors
	## W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
	## or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	## or W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	dcum <- cbind(base1[,1],dbase1)
	ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	colnames(ll) <- c("id","start","stop","death")
	ll <- data.frame(ll)
	ll$death <- dd$status[ll$id+1]
	## add frailty to data for possible validation
	ll$z <- z1[ll$id+1]
	ll$fz <- fzz[ll$id+1]
	## add counts of id
	ids <- countID(ll)
	ll <- cbind(ll,ids[,c(2,4,5)]); 
	ll$status <- 1; 
	ll <- dtransform(ll,status=0,reverseCountid==1)
	ll$statusD <- ll$status
	ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)

	return(ll)
} ## }}}

sim_recurrentCox <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,X=NULL,r1=NULL,r2=NULL,rd=NULL,rc=NULL, 
			    model=c("not-random","random"),frailty=TRUE,var.z=0.5,death.code=3,alpha=1,...)
{ ## {{{
	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(r2)) r2 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	## to avoid error 
	ctime <- death <- NULL

	################################################################
	### approximate hazards to make marginals fit (approximately)
	################################################################
	###laplace and inverse laplace of gamma
	lap<-function(theta,t) { return( (1+t/theta)^(-theta)) }
	ilap<-function(theta,t) { itheta<-1/theta; return((t^(-itheta)-1)/(itheta)) }

	## addapt to make recurrent mean on cox form with this baseline
	base1 <- cumhaz
	if (is.null(death.cumhaz)) stop("Modification for death in this function otherwise just use sim_recurrentII\n")
	if (is.null(X)) stop("X must be given to link with simulated data\n"); 

	### Cox baseline 
	St <- exp(-cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2])

	## unique hazard combinations Death Relative Risk
	rds <- unique(rd)
	if (model[1]=="random") {
		z <- rgamma(n,1/var.z)*var.z 
		if (n<10000) {
			zl <- rgamma(100000,1/var.z)*var.z 
			mza <- mean(zl^alpha)
		} else mza <- mean(z^alpha)
		rds <- rd 
	}  else z <- rep(1,n)
	dtt <- diff(c(0,base1[,1]))
	dbase1 <- diff(c(0,base1[,2]))
	data <- c()

	XX <- c()
	nks <- 0
	k <- 1
	if (model[1]!="random") {
		for (rdss in rds) {
			lam1ms <- (dbase1)/St^rdss
			where <- which(rdss==rd)
			nk <- length(where)
			cumhaz1 <- cbind(base1[,1],cumsum(lam1ms))
			LamDr <- scalecumhaz(death.cumhaz,rdss) 

			datss <- sim_recurrentII(nk,cumhaz1,cumhaz2,death.cumhaz=LamDr,
						r1=r1[where],r2=NULL,rd=NULL,rc=rc[where],...)
			Xs <- X[where,,drop=FALSE][datss$id,,drop=FALSE]
			XX <- rbind(XX,Xs)
			datss$id <- datss$id+nks
			nks <- nks+nk
			data <- rbind(data,as.matrix(datss))
		}
	}

	if (model[1]=="random") {
		for (k in 1:n) {
			rdss <- rd[k]
			if (frailty==TRUE) lam1ms <- (z[k]^alpha/mza)*(dbase1)/St^(z[k]*rdss) else {
				Stt <-  exp(-z[k]*ilap(1/var.z,St^rdss))
				lam1ms <- (z[k]^alpha/mza)*(dbase1)/Stt
			}
			where <- k
			nk <- 1
			cumhaz1 <- cbind(base1[,1],cumsum(lam1ms))

			if (!frailty) LamDr <- cbind(base1[,1],-log(Stt)) 
			if (frailty) LamDr <- scalecumhaz(death.cumhaz,z[k]*rdss) 

			datss <- sim_recurrentII(nk,cumhaz1,cumhaz2,death.cumhaz=LamDr,
						r1=r1[where],r2=NULL,rd=NULL,rc=rc[where],...)
			Xs <- X[where,,drop=FALSE][datss$id,,drop=FALSE]
			XX <- rbind(XX,Xs)
			datss$id <- datss$id+nks
			nks <- nks+nk
			data <- rbind(data,as.matrix(datss))
		}
	}

	rownames(XX) <- NULL
	rownames(data) <- NULL
	colnames(XX) <- paste("X",1:ncol(X),sep="")
	data <- cbind(data,XX)
	data <- as.data.frame(data)

	dsort(data) <- ~id+entry+time
	data$revnr <- revcumsumstrata(rep(1,nrow(data)),data$id-1,n)
	data$statusD <- data$status
	data <- dtransform(data,statusD=death.code,death==1)

	return(data)
} ## }}}


sim_MarginalMeanCox <- function(n,cens=3/5000,k1=0.1,k2=0,bin=1,Lam1=NULL,Lam2=NULL,LamD=NULL,
			       beta1=rep(0,2),betad=rep(0,2),betac=rep(0,2),X=NULL,...)
{ ## {{{

	### to avoid R-check error
	revnr <- death <- status <- NULL
	p <- length(beta1)

	if (is.null(X))  {
		if (bin==1) X <- matrix(rbinom(n*p,1,0.5),n,p) else  X <- matrix(rnorm(n*p),n,p)
		colnames(X) <- paste("X",1:p,sep="")
	}
	r1 <- exp( X %*% beta1)
	rd <- exp( X %*% betad)
	rc <- exp( X %*% betac)

	if (is.null(Lam2)) Lam2 <- Lam1; 

	rr <- sim_recurrentCox(n,scalecumhaz(Lam1,k1),cumhaz2=scalecumhaz(Lam1,k2),death.cumhaz=LamD,X=X,cens=cens,r1=r1,rd=rd,rc=rc,...)

	if (bin==0) dcut(rr,breaks=4) <- X1g~X1 else rr$X1g <- rr$X1
	if (bin==0) dcut(rr,breaks=4) <- X2g~X2 else rr$X2g <- rr$X2
	return(rr)
}

##' @export
scalecumhaz <- function(cumt,k)
{
	return( t(t(cumt)*c(1,k)))
}

##' @export
GLprediid <- function(...)
{
	out <- FGprediid(...,model="GL")
	return(out)
} ## }}}


sim_GLRA <- function(n,base1,drcumhaz,varz=0,beta=c(0.3,-0.3,-0.3,0.3),rcZ=c(0.5,-0.5),pCA=0.5,pCR=0.5,censA=0.5,censR=0.5,
	    depcens.Adm=0,depcens.R=1,Z=NULL,bin=1) { ## {{{
Count0 <- NULL ## to fix R check, variable name returned below 
if (length(bin)==1) bin <- rep(bin,2)
if (is.null(Z))
Z = cbind((bin[1] == 1) * (2 * rbinom(n, 1, 1/2) - 1)+(bin[1]==0)*rnorm(n),(bin[2]==1)*(rbinom(n, 1, 1/2)) + (bin[2] == 0) * rnorm(n))
colnames(Z) <- paste("Z", 1:ncol(Z), sep = "")
p <- ncol(Z)
r1 <- exp( Z %*% beta[1:p])
rd <- exp( Z %*% beta[(p+1):(2*p)])
rrc <-  exp( Z %*% rcZ)

## generate with Adm censurering 
out <- sim_GLcoxRA(n,base1,drcumhaz,var.z=varz,r1=r1,rd=rd,rrc=rrc,rcA=censA,fz=NULL,fdz=NULL,pCA=pCA,
    model=c("twostage","frailty","shared"),type=NULL,share=1,cens=NULL,nmin=100,nmax=1000,
    depcens.Adm=depcens.Adm)

out$Z1 <- Z[,1][out$id+1]
out$Z2 <- Z[,2][out$id+1]
outID <- countID(out)
out$last <- out$stop[outID$reverseCountid==1][out$id+1]
maxtime <- tail(base1,1)[1]

Rcens <- rbinom(n,1,pCR)
if (depcens.R==1) rrC <- exp(Z %*% rcZ) else rrC <- 1
censorR = censorR = Rcens*pmin(rexp(n, 1)*(1/(censR*rrC)),maxtime)+maxtime*(Rcens==0)
censorA <- out$censorA[out$Countid==1]
censor <- pmin(censorR,censorA)

###lasttime <- out$stop[out$reverseCountid==1]
###laststatusD <- out$statusD[out$reverseCountid==1]
outRA <- out
outRA$censorR <- censorR[outRA$id+1]
outRA <- event_split(outRA,status="statusD",time="stop",name.start="start",name.id="id",cuts="censorR",cens.code=0)
outRA <- count_history(outRA,status="statusD",types=0)
outRA <- subset(outRA,Count0==0)
###print(table(out$statusD))
###print(table(outRA$statusD))

data <- list(out=out,outRA=outRA)
return(data)
} ## }}} 

sim_GLcoxRA <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rrc=NULL,rcA=0.5,fz=NULL,fdz=NULL,
     pCA=0.5,depcens.Adm=1,
     model=c("twostage","frailty","shared"),type=NULL,share=1,cens=NULL,nmin=100,nmax=1000)
{ ## {{{
## setting up baselines for simulations 
maxt <- tail(base1[,1],1)
base1 <- as.matrix(base1); drcumhaz <- as.matrix(drcumhaz)
nmin <- max(nrow(base1),nrow(drcumhaz),nmin)
nmin <- min(nmax,nmin)
seqt <- seq(from=0,to=maxt,length.out=nmin)
if (base1[1,1]!=0) base1 <- rbind(0,base1) 
if (drcumhaz[1,1]!=0) drcumhaz <- rbind(0,drcumhaz) 
base1 <- cbind(seqt, lin_approx(seqt,base1))
cumD <- cbind(seqt, lin_approx(seqt,drcumhaz))
###
St <- exp(-cumD[,2])
Stm <- cbind(base1[,1],St)
###
dbase1 <- diff(c(0,base1[,2]))
dcum <- cbind(base1[,1],dbase1)
maxtime <- tail(base1[,1],1)

if (is.null(r1)) r1 <- rep(1,n)
if (is.null(rd)) rd <- rep(1,n)

fz.orig <- fz
if (is.null(fz)) fz <- function(x) x

if (var.z[1]>0) {
	z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
	if (share<1) { 
		z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
		z <- z+z2
	} 
	fzz <- fz(z1)
	if (share<1) fzz <- fzz/share; 
	mza <- mean(fzz)
	if (n<10000 & (!is.null(fz.orig))) {
		zl <- rgamma(100000,share/var.z[1])*var.z[1] 
		fzl <- fz(z)
		mza <- mean(fzl)
	} 
}  else fzz <- z <- z1 <- rep(1,n)

if (var.z[1]==0) model <- "frailty"
if (is.null(type)) 
	if (model[1]=="twostage") type <- 2 else type <- 1
## for frailty setting we also consider any function of z 
if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}

## survival censoring given X, Z, either twostage or frailty-model 
if (type>=2) stype <- 2 else stype <- 1
if (var.z[1]==0) stype <- 1
### dd <- .Call("_mets_simSurvZ",as.matrix(rbind(c(0,1),Stm)),rd,z,var.z[1],stype)
dd <- .Call("_mets_simSurvZ",as.matrix(rbind(Stm)),rd,z,var.z[1],stype)
dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))

admcens <- rbinom(n,1,pCA)
if (depcens.Adm==1) rrA <- rrc else rrA <- 1
censorA = admcens*pmin(rexp(n, 1)*(1/(rcA*rrA)),maxtime)+ maxtime*(admcens==0)
dd$censorA <- censorA
cens <- censorA
dd$status <- ifelse(dd$time<cens,dd$status,7)
dd$time <- pmin(dd$time,cens)

## to avoid R check error
reverseCountid  <-  death  <- NULL

if (model[1]=="multiplicative") {
	## other random effect 
	z2 <- rgamma(n,share/var.z[2])*var.z[2] 
	fzz <- z1*z2
	type <- 3
}
## type=2 draw recurrent process given X,Z with rate:
##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
## type=3, observed hazards on Cox form among survivors
## twostage shared<1: W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
## twostage share=1: or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
## multiplicatve:       W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
dcum <- cbind(base1[,1],dbase1)
### ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
ll <- .Call("_mets_simGL",as.matrix(dcum),c(St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
colnames(ll) <- c("id","start","stop","death")
ll <- data.frame(ll)
ll$death <- dd$status[ll$id+1]
ll$censorA <- dd$censorA[ll$id+1]
## add frailty to data for possible validation
ll$z <- z1[ll$id+1]
ll$fz <- fzz[ll$id+1]
## add counts of id
ids <- countID(ll)
ll <- cbind(ll,ids[,c(2,4,5)]); 
ll$status <- 1; 
ll <- dtransform(ll,status=7,reverseCountid==1)
ll$statusD <- ll$status
ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)

attr(ll,"base1events") <- base1
attr(ll,"deathcumbase") <- cumD
return(ll)
} ## }}}

boottwostageREC <- function(margsurv,recurrent,data,bootstrap=100,id="id",stepsize=0.5,...) 
{ ## {{{
	n <- max(margsurv$id)
	K <- bootstrap
	formid <- as.formula(paste("~",id))
	rrb <- blocksample(data, size = n*K, formid)
	rrb$strata <- floor((rrb[,id]-0.01)/n)

	outb <- outd <- outr <- c()
	for (i in 1:K)
	{
		rrbs <- subset(rrb,strata==i-1)
		drb <- phreg(margsurv$formula,data=rrbs)
		if (inherits(recurrent,"recreg")) {
			xrb <- recreg(recurrent$formula,data=rrbs,
				      cause=recurrent$cause,death.code=recurrent$death.code,cens.code=recurrent$cens.code,twostage=TRUE)
		} else xrb <- phreg(recurrent$formula,data=rrbs)
		outbl <- tryCatch(twostageREC(drb,xrb,rrbs,numderiv=0,control=list(stepsize=stepsize,iter.max=10),...),error=function(x) NULL)
     if (!is.null(outbl)) outb <- rbind(outb,outbl$coef)
     outd <- rbind(outd,coef(drb))
     outr <- rbind(outr,coef(xrb))
  }
  varb <- cov(outb)
  vard <- cov(outd)
  varr <- cov(outr)

  list(outb=outb,var=varb,se=diag(varb)^.5,se.coxD=diag(vard)^.5,se.coxR=diag(varr)^.5)
} ## }}}


##' Fitting of Two-Stage Recurrent Events Random Effects Model
##'
##' Fits a two-stage random effects model for recurrent events with a terminal event.
##' Marginal models (Cox or Ghosh-Lin) are fitted first and passed to this function.
##'
##' Supports:
##' \itemize{
##' \item Cox/Cox marginals.
##' \item Cox/Ghosh-Lin marginals.
##' \item Fully shared, partly shared, or non-shared random effects.
##' }
##'
##' @param margsurv Marginal model for the terminal event (object of class \code{"phreg"}).
##' @param recurrent Marginal model for recurrent events (object of class \code{"phreg"} or \code{"recreg"}).
##' @param data Data frame used for fitting.
##' @param theta Starting value for total variance of gamma frailty.
##' @param model Model type: \code{"full"} (fully shared), \code{"shared"} (partly shared), or \code{"non-shared"}.
##' @param ghosh.lin Logical; if \code{TRUE}, forces use of Ghosh-Lin marginals based on the recurrent model.
##' @param theta.des Regression design for variance parameters.
##' @param var.link Link function for variance (1 for exponential).
##' @param method Optimization method (default "NR").
##' @param no.opt Logical; if \code{TRUE}, skips optimization.
##' @param weights Weights.
##' @param se.cluster Clusters for SE calculation (GEE style).
##' @param fnu Function to transform \eqn{\nu} (amount shared).
##' @param nufix Logical; if \code{TRUE}, fixes the amount shared.
##' @param nu Starting value for the amount shared.
##' @param numderiv Logical; if \code{TRUE}, uses numerical derivatives.
##' @param derivmethod Method for numerical derivative.
##' @param ... Arguments for the optimizer.
##'
##' @return An object of class \code{"twostageREC"} containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Variance-covariance matrix.}
##' \item{theta}{Dependence parameters.}
##' \item{model}{Model type.}
##'
##' @author Thomas Scheike
##' @references Scheike (2026), Two-stage recurrent events random effects models, LIDA, to appear.
##' @export
twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NULL, model=c("full","shared","non-shared"),ghosh.lin=NULL,
  theta.des = NULL, var.link = 0, method = "NR", no.opt = FALSE, weights = NULL, se.cluster = NULL, 
  fnu=NULL,nufix=0,nu=NULL,numderiv=1,derivmethod=c("simple","Richardson"),...)
{ ## {{{
    newrec <- 1; at.risk <- 1
    if (!inherits(margsurv, "phreg")) stop("Must use phreg for death model\n")
    if (!inherits(recurrent, "phreg")) stop("Must use phreg for recurrent model\n")
    if (is.null(recurrent$cox.prep) & newrec==0) stop("recreg must be called with cox.prep=TRUE\n")
    if (inherits(recurrent, "recreg") & is.null(ghosh.lin))  ghosh.lin <- 1 
    if ((!inherits(recurrent, "recreg")) & is.null(ghosh.lin))  ghosh.lin <- 0 
    if (is.null(recurrent$cox.prep.twostage) & newrec==1 & (ghosh.lin==1)) stop("recreg must be called with twostage=TRUE\n")
    clusters <- margsurv$cox.prep$id
    n <-  max(clusters)+1
    if (is.null(theta.des) == TRUE) ptheta <- 1
    if (is.null(theta.des) == TRUE) theta.des <- matrix(1, n, ptheta) else theta.des <- as.matrix(theta.des)
    ptheta <- ncol(theta.des)
    if (nrow(theta.des) != n) stop("Theta design does not have correct dim")
    if (is.null(theta) == TRUE) {
        if (var.link == 1) theta <- rep(0, ptheta)
        if (var.link == 0) theta <- rep(1, ptheta)
    }
    if (is.null(nu)  & (nufix==0)) { if (is.null(fnu))  nu <-  0.5 else nu <- fnu[[1]](0.5) }
    if (length(theta) != ptheta) theta <- rep(theta[1], ptheta)
    if (length(nu) != ptheta) nu <- rep(nu,ptheta)
    theta.score <- rep(0, ptheta)
    Stheta <- var.theta <- matrix(0, ptheta, ptheta)
    max.clust <- length(unique(clusters))
    theta.iid <- matrix(0, max.clust, ptheta)
    xx <- margsurv$cox.prep
    xr <- recurrent$cox.prep
    if (ghosh.lin & newrec==1) {
	    xr <- recurrent$cox.prep.twostage
    } 
    nn <- length(xx$strata)
    if (is.null(weights)) weights <- rep(1, nn)
    if (length(weights) != nn) stop("Weights do not have right length")
    statusxx <- rep(0, length(xx$strata))
    statusxx[xx$jumps + 1] <- 1
    xx$status <- statusxx
    mid <- max(xx$id) + 1
    Nsum <- cumsumstratasum(statusxx, xx$id, mid, type = "all")
    Ni.tau <- sumstrata(statusxx, xx$id, mid)
    S0i2 <- S0i <- rep(0, length(xx$strata))
    S0i[xx$jumps + 1] <- 1/margsurv$S0
    cumhazD <- cumsumstrata(S0i, xx$strata, xx$nstrata)
    if (!is.null(margsurv$coef)) RR <- exp(xx$X %*% margsurv$coef) else RR <- rep(1, nn)
    HD <- c(cumhazD * RR)
    cumDYt <- sumstrata(HD*xx$sign,xx$id,mid)
    statusx1 <- rep(0, length(xx$strata))
    statusx1[xr$jumps + 1] <- 1
    xr$status <- statusx1
    N1sum <- cumsumstratasum(statusx1, xr$id, mid, type = "all")
    N1i.tau <- sumstrata(statusx1, xr$id, mid)
    S01i2 <- S01i <- rep(0, length(xr$strata))
    S01i[xr$jumps + 1] <- 1/recurrent$S0
    cumhaz1 <- cumsumstrata(S01i, xr$strata, xr$nstrata)
    if (!is.null(recurrent$coef)) RR1 <- exp(xr$X %*% recurrent$coef) else RR1 <- rep(1, nn)
    H1 <- c(cumhaz1 * RR1)
    ###
    ## designs of fixed time covariates and weights
    cc <- cluster_index(xx$id)
    firstid <- cc$firstclustid + 1
    if (max(cc$cluster.size) == 1) stop("No clusters !, maxclust size=1\n")
    ###
    theta.des <- theta.des[xx$id+ 1, , drop = FALSE]
    theta.des <- theta.des[xx$ord + 1, , drop = FALSE]
    weightsid <- weights <- weights[xx$ord + 1]
    weights <- weights[firstid]
    thetaX <- as.matrix(theta.des[firstid, , drop = FALSE])
    Xdeath <- as.matrix(xx$X[firstid,,drop = FALSE])
    Xrecurrent <- as.matrix(xr$X[firstid,,drop = FALSE])
    statusxb <- statusxx+statusx1
    ###
    Nsumb <- Nsum$lagsum+N1sum$lagsum
    rd <- RR[firstid]
    r1 <- RR1[firstid]
    lastid <- tailstrata(xx$id,mid)
    ###
    cumDL <- HD[lastid]
    nuX <- thetaX
    ###
    idD <- xx$id[statusxx==1]

    obj <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(c(par))) else epar <- c(par)
        thetav <- c(as.matrix(theta.des) %*% c(epar))
        thetai <- thetav[firstid]
	###
###        tildeL <- .Call("_mets_tildeLambda1",S01i,cumhazD,r1,rd,thetai,xx$id)
###	if (at.risk==1)  
###		tildeL <- apply(tildeL*c(xr$sign),2,cumsumstrata,xr$id,mid)
        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
        tildeLast <- tildeL[lastid,]
	Ht <- thetav*tildeL[,1]+exp(thetav*HD)
	Hr <- thetai*tildeLast[,1]+exp(thetai*cumDL)
	DHt <- tildeL[,1]+thetav*tildeL[,2]+HD*exp(thetav*HD)
	DHr <- tildeLast[,1]+thetai*tildeLast[,2]+cumDL*exp(thetai*cumDL)
        ###
	D2Ht <- 2*tildeL[,2]+thetav*tildeL[,3]+HD^2*exp(thetav*HD)
	D2Hr <- 2*tildeLast[,2]+thetai*tildeLast[,3]+cumDL^2*exp(thetai*cumDL)
	aHt <- abs(Ht)
	aDHt <- abs(DHt)
	aD2Ht <- abs(D2Ht)
        ###
        l1 <- sumstrata(log(1 + thetav * N1sum$lagsum) * statusxb, xx$id, mid)
        l2 <- sumstrata(statusxb * HD, xx$id, mid)
	if (at.risk==0) l3 <- -(1/thetai + N1i.tau) * log(Hr) 
	if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(1/thetav+N1sum$sum)*log(aHt),xr$id,mid) 
	HtD <- Ht[statusxx==1]
	DHtD <- DHt[statusxx==1]
	D2HtD <- D2Ht[statusxx==1]
        l4 <- -sumstrata(log(HtD),idD,mid)
        logliid <- (l1 + thetai * l2 + l3 + l4) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
        l1s <- sumstrata(N1sum$lagsum/(1 + thetav * N1sum$lagsum) * statusxb, xx$id, mid)
	if (at.risk==0) l3s <- -(1/thetai + N1i.tau) * DHr/Hr + log(Hr)/thetai^2;
	if (at.risk==1) l3s <- sumstrata(c(xr$sign)*(-(1/thetav+N1sum$sum)*DHt/Ht+log(aHt)/thetav^2),xr$id,mid);
	l4s <-  -sumstrata((DHtD/HtD),idD, mid) 
	Dltheta <- (l1s+l2+l3s+l4s)*c(weights) 
	scoreiid <- thetaX * c(Dltheta)
        D2N <- -sumstrata(N1sum$lagsum^2/(1 + thetav * N1sum$lagsum)^2 * statusxb, xx$id, mid)
	if (at.risk==0) 
        Dhes <- (2/thetai^2) * DHr/Hr -(1/thetai + N1i.tau)*(D2Hr*Hr-DHr^2)/Hr^2 - (2/thetai^3) * log(Hr) 
	if (at.risk==1) 
        Dhes <- sumstrata(c(xr$sign)* (
	(2/thetav^2) * aDHt/aHt -(1/thetav + N1sum$sum)*(aD2Ht*Ht-aDHt^2)/aHt^2 -(2/thetav^3) * log(aHt)) ,xr$id,mid)
	D2l4 <- -sumstrata(((D2HtD*HtD-DHtD^2)/HtD^2), idD, mid)
        Dhes <- c(Dhes+D2N+D2l4) * c(weights)
        if (var.link == 1) {
            scoreiid <- scoreiid * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid,
            ploglik = ploglik, gradient = gradient, hessian = hessian,
            hess2 = hess2)
        if (all)
            return(val)
        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
    }


    objNonShared <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(c(par))) else epar <- c(par)
        thetav <- c(as.matrix(theta.des) %*% c(epar))
        thetai <- thetav[firstid]
	###
	Ht <- 1+thetav*H1 
	Hr <- Ht[lastid]
	DHt <- H1
	DHr <- H1[lastid]
        ###
	D2Ht <- 0
	D2Hr <- 0
	aHt <- abs(Ht)
	aDHt <- abs(DHt)
	aHr <- abs(Hr)
        ###
        l1 <- sumstrata(log(1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	if (at.risk==0) l3 <- -(1/thetai + N1i.tau) * log(Hr) 
	if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(1/thetav+N1sum$sum)*log(aHt),xr$id,mid) 
        logliid <- (l1 + l3) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
        l1s <- sumstrata(N1sum$lagsum/(1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	if (at.risk==0) l3s <- -(1/thetai + N1i.tau) * DHr/Hr + log(Hr)/thetai^2;
	if (at.risk==1) l3s <- sumstrata(c(xr$sign)*(-(1/thetav+N1sum$sum)*DHt/Ht+log(aHt)/thetav^2),xr$id,mid);
	Dltheta <- (l1s+l3s)*c(weights) 
	scoreiid <- thetaX * c(Dltheta)
        D2N <- -sumstrata(N1sum$lagsum^2/(1 + thetav * N1sum$lagsum)^2 * statusx1, xx$id, mid)
	if (at.risk==0) 
        Dhes <- (2/thetai^2) * DHr/Hr -(1/thetai + N1i.tau)*(-DHr^2)/Hr^2 - (2/thetai^3) * log(Hr) 
	if (at.risk==1) 
        Dhes <- sumstrata(c(xr$sign)* (
	(2/thetav^2) * aDHt/aHt -(1/thetav + N1sum$sum)*(-aDHt^2)/aHt^2 -(2/thetav^3) * log(aHt)) ,xr$id,mid)
        Dhes <- c(Dhes+D2N) * c(weights)
        if (var.link == 1) {
            scoreiid <- scoreiid * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid,
            ploglik = ploglik, gradient = gradient, hessian = hessian,
            hess2 = hess2)
        if (all)
            return(val)
        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
    }


    ## default is simple identity 
###    if (is.null(fnu)) { fw <- function(x) 1/(1+exp(-x)); Dfw <- function(x) -exp(-x)/(1+exp(-x))^2;} else { 
   if (is.null(fnu)) { fw <- function(x) x; Dfw <- function(x) 1;} else { fw <- fnu[[1]]; Dfw <- fnu[[2]]; } 
   nudes <- theta.des  
   p <- ncol(theta.des)

    objShared <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(par[1:p])) else epar <- c(par[1:p])
        thetav <- c(as.matrix(theta.des) %*% epar)
	if (nufix==1) nu1 <- c(as.matrix(nudes) %*% nu)
	else nu1 <- c(as.matrix(nudes) %*% par[(p+1):2*p])
        nu1i <-nu1[firstid]
	tbeta1 <- fw(nu1); tbeta2 <- 1-tbeta1; 
        thetai <- thetav[firstid]; tbeta1i <- tbeta1[firstid]; tbeta2i <- tbeta2[firstid]
	###
	R <-  exp( - thetav*HD);  DR <- -HD*exp( - thetav* HD); D2R <-  HD^2*exp( - thetav* HD) 
        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
	tildeLast <- tildeL[lastid,]
	Ht <- (thetav/tbeta1)*tildeL[,1]+exp(thetav*HD)
	Hr <- (thetai/tbeta1i)*tildeLast[,1]+exp(thetai*cumDL)
	DHt <- (tildeL[,1]+thetav*tildeL[,2])/tbeta1+HD*exp(thetav*HD)
	DHr <- (tildeLast[,1]+thetai*tildeLast[,2])/tbeta1i+cumDL*exp(thetai*cumDL)
	DHtn <- -(thetav/tbeta1^2)*tildeL[,1]
	DHrn <- -(thetai/tbeta1i^2)*tildeLast[,1]
        ###
	D2Ht <- (2*tildeL[,2]+thetav*tildeL[,3])/tbeta1+HD^2*exp(thetav*HD)
	D2Hr <- (2*tildeLast[,2]+thetai*tildeLast[,3])/tbeta1i+cumDL^2*exp(thetai*cumDL)
        ###
        N <- (tbeta1 + thetav * N1sum$lagsum)/Ht + tbeta2*R
        l1d <- sumstrata(log(N) * statusxx, xx$id, mid)
        l11 <- sumstrata(log(tbeta1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	l1 <- l1d+l11
        l2 <- sumstrata(statusxb * HD, xx$id, mid) 
	l22 <-  -log(tbeta1i)*N1i.tau 
        if (at.risk==0) l3 <- -(tbeta1i/thetai + N1i.tau)*log(Hr) 
        if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(tbeta1/thetav+N1sum$sum)*log(Ht),xr$id,mid) 
        l4 <-  (tbeta1i)*cumDYt
        logliid <- (l1 + thetai*l2+ l22 + l3 + l4) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
	DN1tt <- (Ht*N1sum$lagsum-DHt*(tbeta1+thetav*N1sum$lagsum))
	DN1t <-  DN1tt/Ht^2 
	DNt <- c(DN1t+tbeta2*DR)
        l1ds <- sumstrata((DNt/N)*statusxx,xx$id,mid)
        l11s <- sumstrata((N1sum$lagsum/(tbeta1 + thetav * N1sum$lagsum)) * statusx1, xx$id, mid)
	l1s <- l1ds+l11s
        if (at.risk==0) 
        l3s <- -(tbeta1i/thetai + N1i.tau) * (DHr/Hr) + (tbeta1i/thetai^{2}) * log(Hr) 
        if (at.risk==1) 
	l3s <- sumstrata(c(xr$sign)*(-(tbeta1/thetav+N1sum$sum)*DHt/Ht+log(Ht)*tbeta1/thetav^2),xr$id,mid);
        Dltheta <- (l1s+l2+l3s)*c(weights)
        ### 
	DNtn <- (Ht-DHtn*(tbeta1+thetav*N1sum$lagsum))
	DNn <- DNtn/Ht^2 
        l1dn <- sumstrata( c(DNn-R)/N * statusxx, xx$id, mid)
        l11n <- sumstrata(1/(tbeta1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	l1n <- l1dn+l11n
	l22n <-  -N1i.tau/tbeta1i 
	if (at.risk==0) 
		l3n <- -(tbeta1i/thetai+N1i.tau)*(DHrn/Hr)-(1/thetai)*log(Hr)
	if (at.risk==1) 
        l3n <- -sumstrata(c(xr$sign)*(+(tbeta1/thetav+N1sum$sum)*(DHtn/Ht)+(1/thetav)*log(Ht)), xr$id,mid);
        Dlnu <- (l1n+l22n+l3n+cumDYt)*c(weights)
        ###
	if (nufix==1)
        scoreiid <- thetaX * c(Dltheta)
        else  scoreiid <- cbind(thetaX * c(Dltheta),nuX*c(Dlnu)*Dfw(nu1i))
        ###   ###
        Dl11s <- -sumstrata(N1sum$lagsum^2/(tbeta1 + thetav * N1sum$lagsum)^2 * statusxb, xx$id, mid)
        Dl3s <- (2*tbeta1i/thetai^2) * DHr/Hr -(tbeta1i/thetai+ N1i.tau)*(D2Hr*Hr-DHr^2)/Hr^2 - (2*tbeta1i/thetai^3) * log(Hr) 
        ###
	D2N1t <- (Ht^2*(DHt*N1sum$lagsum-D2Ht*(tbeta1+thetav*N1sum$lagsum)-DHt*N1sum$lagsum)-2*DHt*DN1tt)/Ht^4 
	D2Nt <-  D2N1t+tbeta2*D2R
	Dl1ds <- -sumstrata(((D2Nt*N-DNt^2)/N^2)*statusxx, xx$id, mid)
        Dhes <- c(Dl1ds+Dl11s+Dl3s) * c(weights)
        if (var.link == 1) {
            scoreiid[,1:p] <- scoreiid[,1:p] * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
	if (nufix==0) {
###	    hessiann <- crossprod(nuX, nuX* c(D2nu))
###	    hessiannp <- crossprod(thetaX,nuX*Dtn)
###	    hessian <- cbind(hessian,hessianp)
###	    hessian <- rbind(hessian,cbind(t(hessianp),hessiann))
	}
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid, ploglik = ploglik, gradient = gradient, hessian = -hess2, hess2 = hess2)
        if (all)
            return(val)
###        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
        with(val, structure(-ploglik, gradient = -gradient, hessian = hess2))
    }


   if (model[1]=="shared") obj <- objShared
   if (model[1]=="non-shared") obj <- objNonShared
   if (nufix==0 & model[1]=="shared") par <- c(theta,nu) else par <- theta

    opt <- NULL
    if (no.opt == FALSE) {
        if (tolower(method) == "nr") {
            opt <- lava::NR(par, obj,...)
            opt$estimate <- opt$par
        }
        else {
            opt <- nlm(obj, par,...)
            opt$method <- "nlm"
        }
        cc <- opt$estimate
        val <- c(list(coef = cc), obj(opt$estimate, all = TRUE))
    }
    else val <- c(list(coef = par), obj(par, all = TRUE))
    val$score <- val$gradient
    theta <- matrix(c(val$coef), length(c(val$coef)), 1)

    if (!is.null(colnames(theta.des)))
        thetanames <- colnames(theta.des)
    else thetanames <- paste("dependence", 1:length(c(theta)), sep = "")
    if (nufix==0 & model[1]=="shared") thetanames <-
     thetanames <- c(paste("dependence", 1:p, sep = ""),paste("share", 1:p,sep = ""))

    if (length(thetanames) == length(c(theta))) {
        rownames(theta) <- thetanames
        names(val$coef)  <- thetanames
    }

    if (numderiv==1 & model[1]=="shared") {
	    dobj <- function(p) {
		    oo <- obj(p)
		    return(attr(oo,"gradient"))
	    }
	    hessian <- numDeriv::jacobian(dobj,val$coef,method=derivmethod[1])
	    val$hessian <- -hessian
    }

    hessianI <- pinv(val$hessian)
    val$theta.iid.naive <- val$score.iid %*% hessianI

    if (!is.null(se.cluster))
        if (length(se.cluster) != length(clusters))
            stop("Length of seclusters and clusters must be same\n")
    if (!is.null(se.cluster)) {
        iids <- unique(se.cluster)
        nseclust <- length(iids)
        if (is.numeric(se.cluster))
            se.cluster <- fast.approx(iids, se.cluster) - 1
        else se.cluster <- as.integer(factor(se.cluster, labels = seq(nseclust))) - 1
        val$theta.iid <- apply(val$theta.iid,2,sumstrata,se.cluster, nseclust)
        val$theta.iid.naive <- apply(val$theta.iid.naive,2,sumstrata,se.cluster, nseclust)
    }
    var <- robvar.theta <- var.theta <- crossprod(val$theta.iid)
    naive.var <- crossprod(val$theta.iid.naive)
    val <- c(val, list(theta = theta, var.theta = var,n=mid,p=ncol(thetaX),var.link=var.link,
                       robvar.theta = var, var = var, thetanames = thetanames,
                       model = model[1], se = diag(var)^0.5), 
	     var.naive = naive.var,no.opt=no.opt,ghosh.lin=ghosh.lin)
    class(val) <- "twostageREC"
    attr(val, "clusters") <- clusters
    attr(val, "fnu") <- fw
    attr(val, "Dfnu") <- Dfw
    attr(val, "secluster") <- c(se.cluster)
    attr(val, "var.link") <- var.link
    attr(val, "ptheta") <- ptheta
    attr(val, "n") <- n
    attr(val, "response") <- "survival"
    attr(val, "additive-gamma") <- 0
    attr(val, "twostage") <- "two.stage"
    return(val)
} ## }}}

##' @export
summary.twostageREC <- function(object,vcov=NULL,delta=0,...) { ## {{{
    I <- -pinv(object$hessian)
    if (!is.null(vcov)) V <- vcov else V <- object$var
    ncluster <- object$n
    cc <- estimate(coef=object$coef,vcov=V)$coefmat
    pd <- object$p
    if (object$var.link==1 & object$model=="full") f <- function(p) exp(p)
    if (object$var.link==0 & object$model=="full") f <- function(p) p
    if (object$var.link==1 & object$model=="non-shared") f <- function(p) exp(p)
    if (object$var.link==0 & object$model=="non-shared") f <- function(p) p

    if (object$var.link==1 & object$model=="shared") f <- function(p) c(exp(p[1]),attr(object,"fnu")(p[2]))
    if (object$var.link==0 & object$model=="shared") f <- function(p) c(p[1],attr(object,"fnu")(p[2]))
    if (delta==1 | object$model=="full" | object$model=="non-shared") 
    expC <- lava::estimate(coef=object$coef,vcov=V,f=f)$coefmat ##[,c(1,3,4),drop=FALSE]
    else expC <- apply(cc[,c(1,3,4),drop=FALSE],2,f) 
  n <- object$n
  res <- list(coef=cc,n=n,nevent=object$nevent,ncluster=ncluster,var=V,exp.coef=expC,var.link=object$var.link,
	      ghosh.lin=object$ghosh.lin)
  class(res) <- "summary.twostageREC"
  res
}  ## }}}

##' @export
print.summary.twostageREC  <- function(x,max.strata=5,...) { ## {{{
  if (x$ghosh.lin==0) cat("Cox(recurrent)-Cox(terminal) intensity model\n"); 
  if (x$ghosh.lin==1) cat("Ghosh-Lin(recurrent)-Cox(terminal) mean model\n"); 
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coeffients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    if (x$var.link==1) cat("var=exp(coeffients),shared:\n")
    if (x$var.link==0) cat("var,shared:\n")
    printCoefmat(x$exp.coef,...)
  }
  cat("\n")
}  ## }}}


##' @export
print.twostageREC  <- function(x,...) {
  print(summary(x),...)
} 

# cifreg versus crr-cmprsk
library("tinytest")

# recreg versus recurrentMarginal for non-parametric 
test_recregMarginal <- function() {
 library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf <- tie_breaker(hf,cause=c(0,1,2),cens.code=4)

 ngl <- recreg(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
 cause=1,death.code=2,cens.model=~strata(treatment))
 ### 
 dd <- data.frame(treatment=levels(hf$treatment),id=1,status=0,time=0,entry=0)
 pngl <- predict(ngl,dd,times=3,se=1)
 pgl <- cbind(pngl$cumhaz,pngl$se.cumhaz)
 pgl

 ### 
 meann <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,cause=1,death.code=2)
 smeann <- summary(meann,time=3)
 smeann <- rbind(smeann$pbaseci[[1]],smeann$pbaseci[[2]])
 smeann
 pgl
 dif <- pgl - smeann[,2:3]
 dif

 ## estimate (same) and standard errors (not to far)
 expect_true( ((sum(abs(dif[,1])) < 0.000001) & (sum(abs(dif[,2])) < 0.001)) )
}
test_recregMarginal()


# recregIPCW versus recurrentMarginal for non-parametric 
test_recregMarginalIPCW <- function() {
 library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 ## break all-ties for cause=0,1,2
 hf <- tie_breaker(hf,cause=c(0,1,2),cens.code=4)

 ngl <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		   cens.model=~strata(treatment))
 dd <- data.frame(treatment=levels(hf$treatment),id=1)
 pngl <- predict(ngl,dd,times=3,se=1)
 ### 
 meann <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,cause=1,death.code=2)
 smeann <- summary(meann,time=3)
 smeann <- rbind(smeann$pbaseci[[1]],smeann$pbaseci[[2]])
 smeann
 pngl
 dif <- pngl[,1:2] - smeann[,2:3]
 dif

 ## estimate (same) and standard errors (not to far)
 expect_true( ((sum(abs(dif[,1])) < 0.00001) & (sum(abs(dif[,2])) < 0.001)) )
}
test_recregMarginalIPCW()

# cifreg versus crr-cmprsk
library("tinytest")

# recreg versus recurrentMarginal for non-parametric 
test_recregMarginal <- function() {
 library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf <- tie.breaker(hf,cause=c(0,1,2),cens.code=4)

 ngl <- recreg(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
 cause=1,death.code=2,cens.model=~strata(treatment))
 ### 
 dd <- data.frame(treatment=levels(hf$treatment),id=1,status=0,time=0,entry=0)
 pngl <- predict(ngl,dd,times=3,se=1)
 pgl <- cbind(pngl$cumhaz,pngl$se.cumhaz)
 pgl

 ### 
 meann <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,cause=1,death.code=2)
 smeann <- summary(meann,time=3)
 smeann <- rbind(smeann$pbaseci[[1]],smeann$pbaseci[[2]])
 smeann
 pgl
 dif <- pgl - smeann[,2:3]
 dif

 ## estimate (same) and standard errors (not to far)
 expect_true( ((sum(abs(dif[,1])) < 0.000001) & (sum(abs(dif[,2])) < 0.001)) )
}
test_recregMarginal()


# recregIPCW versus recurrentMarginal for non-parametric 
test_recregMarginalIPCW <- function() {
 library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 ## break all-ties for cause=0,1,2
 hf <- tie.breaker(hf,cause=c(0,1,2),cens.code=4)

 ngl <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		   cens.model=~strata(treatment))
 dd <- data.frame(treatment=levels(hf$treatment),id=1)
 pngl <- predict(ngl,dd,times=3,se=1)
 ### 
 meann <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,cause=1,death.code=2)
 smeann <- summary(meann,time=3)
 smeann <- rbind(smeann$pbaseci[[1]],smeann$pbaseci[[2]])
 smeann
 pngl
 dif <- pngl[,1:2] - smeann[,2:3]
 dif

 ## estimate (same) and standard errors (not to far)
 expect_true( ((sum(abs(dif[,1])) < 0.00001) & (sum(abs(dif[,2])) < 0.001)) )
}
test_recregMarginalIPCW()

##' Multinomial Regression Based on phreg
##'
##' Fits a multinomial regression model for a categorical outcome with \eqn{K} levels:
##' \deqn{ P_i = \frac{ \exp( X^\top \beta_i ) }{ \sum_{j=1}^K \exp( X^\top \beta_j ) } }
##' for \eqn{i=1, \dots, K}, where \eqn{\beta_1 = 0} (baseline category).
##'
##' This ensures that \eqn{\sum_j P_j = 1}. The model is fitted using the \code{phreg} function
##' by expanding the data into a long format with strata for each category.
##'
##' The coefficients represent the log-Relative-Risk (log-RR) relative to the baseline group
##' (the first level of the factor, which can be reset using \code{relevel}).
##'
##' Standard errors are computed based on the sandwich form:
##' \deqn{ D U^{-1} \left( \sum U_i^2 \right) D U^{-1} }
##' where \eqn{U} is the score vector and \eqn{D} is the derivative matrix.
##'
##' Influence functions (possibly robust) can be obtained via the \code{iid()} function.
##' The response variable should be a factor.
##'
##' Can also fit a cumulative odds model as a special case of \code{interval_logitsurv_discrete}.
##'
##' @param formula Formula with the outcome (similar to \code{coxph}). The outcome must be a factor.
##' @param data Data frame containing the variables.
##' @param weights Weights for the score equations.
##' @param offset Offsets for the partial likelihood.
##' @param fix.X Logical; if \code{TRUE}, forces the same coefficients for all categories (except intercepts).
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"mlogit"} (extending \code{"phreg"}) containing:
##' \item{coef}{Matrix of estimated coefficients (rows correspond to categories, columns to covariates).}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{iid}{Influence functions for the coefficients.}
##' \item{nlev}{Number of levels in the outcome.}
##' \item{px}{Number of covariates.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$id <- sample(200,408,replace=TRUE)
##' dfactor(bmt) <- cause1f~cause
##' drelevel(bmt,ref=3) <- cause3f~cause
##' dlevels(bmt)
##'
##' mreg <- mlogit(cause1f~+1+cluster(id),bmt)
##' summary(mreg)
##' head(iid(mreg))
##' dim(iid(mreg))
##'
##' mreg <- mlogit(cause1f~tcell+platelet,bmt)
##' summary(mreg)
##' head(iid(mreg))
##' dim(iid(mreg))
##' 
##' mreg3 <- mlogit(cause3f~tcell+platelet,bmt)
##' summary(mreg3)
##' 
##' ## inverse information standard errors 
##' lava::estimate(coef=mreg3$coef,vcov=mreg3$II)
##' 
##' ## predictions based on seen response or not 
##' ## all probabilities
##' head(predict(mreg,response=FALSE))
##' head(predict(mreg))
##' ## using newdata 
##' newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1),cause1f=c("2","2","0"))
##' ## only probability of seen response 
##' predict(mreg,newdata)
##' ## without response
##' predict(mreg,newdata,response=FALSE)
##' ## given indexx of P(Y=j)
##' predict(mreg,newdata,Y=c(1,2,3))
##' ##  reponse not given 
##' newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1))
##' predict(mreg,newdata)
##' @export
##' @aliases predict 
mlogit <- function(formula,data,offset=NULL,weights=NULL,fix.X=FALSE,...)
{# {{{
  cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

    ## must be a factor 
    if (!is.factor(des$y)) stop("The response variable must be a factor.")

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,nrow(X)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,nrow(X)) 
  } else weights <- des.weights

   if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
   strata.name <- NULL

  res <- mlogit01(X,Y,id=id,strata=NULL,offset=offset,weights=weights,strata.name=strata.name,
		  fix.X=fix.X,formula.call=formula,...) 
  res$design <- des
  return(res)
}# }}}

mlogit01 <- function(X,Y,id=NULL,strata=NULL,offset=NULL,weights=NULL,
       strata.name=NULL,cumhaz=FALSE,
       beta,stderr=TRUE,method="NR",no.opt=FALSE,Z=NULL,
       propodds=NULL,AddGam=NULL,case.weights=NULL,fix.X=FALSE,formula.call=NULL,
  X.call=NULL,Y.call=NULL,...) {# {{{
  p <- ncol(X)
  if (missing(beta)) beta <- rep(0,p)
  if (p==0) X <- cbind(rep(0,length(Y)))
  if (is.null(offset)) offset <- rep(0,length(Y)) 
  if (is.null(weights)) weights <- rep(1,length(Y)) 
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given 
  if (!is.null(Z)) Zcall <- Z
  if (is.null(case.weights)) case.weights <- rep(1,length(Y)) 

  call.id <- id
  if (is.null(id)) id <- 1:nrow(X)

  types <- unique(as.numeric(Y))
  nlev <- length(types)

  nX <- nrow(X)
  idrow <- rep(1:nX,each=nlev)
  X <- X[idrow,,drop=FALSE]
  px <- ncol(X)
  Y <- Y[idrow]
  id <- id[idrow]
  status <- rep(0,nrow(X))
  nY <- as.numeric(Y)

  refg <- 1  ### else refg <- match(ref,types)
  nrefs <- (1:nlev)[-refg]
  for (i in 1:nlev) status[nY==i] <- rep(((1:nlev)==i),sum(nY==i)/nlev)
  time <- id
  strat <- rep(1:nlev,nX)
  XX <- c()
  nn <- c()
  namesX <- colnames(X); 
  namesX <- gsub("^\\(Intercept\\)$", "Intercept", namesX)

  if (!fix.X) {
     for (i in nrefs) { XX <- cbind(XX,X*(strat==i)); 
                      nn<-c(nn, paste(namesX,i,sep="_")) 
     }  
     colnames(XX) <- nn; 
  } else {
     ### different intercepts
     for (i in nrefs) { XX <- cbind(XX,cbind(X[,1]*(strat==i))); 
                        nn<-c(nn, paste(namesX[1],i,sep=".")) 
     }  
     if (fix.X && ncol(X) < 2) stop("fix.X=TRUE requires at least one covariate beyond the intercept.")
     ### same covariate effects 
     XX<- cbind(XX,X[,-1]*(strat!=1))
     nn <- c(nn,colnames(X[,-1]))
     colnames(XX) <- nn; 
  }
  rownames(XX) <- NULL
  colnames(XX) <- nn

  datph = cbind(
      data.frame(
          time = time, status = status, id = id, idrow = idrow
      ),
      XX
  )
  loffset <- offset[idrow]
  lweights<- weights[idrow]

  form <- "Surv(time,status)~strata(idrow)+cluster(id)"
  form <- paste(
  form, "+",
  paste(sprintf("`%s`", colnames(XX)), collapse = "+")
  )

  res <- phreg(as.formula(form), datph, weights = lweights, offset = loffset, ...)
  res$formula <- formula.call
  res$px <- px
  res$nlev <- nlev
  class(res) <- rev(c("phreg","mlogit"))
  return(res)
}# }}}

##' Predictions from Multinomial Regression
##'
##' Computes predicted probabilities for the categorical outcome based on a \code{mlogit} object.
##' Can return probabilities for all categories or only for the observed response.
##'
##' @param object Object of class \code{"mlogit"}.
##' @param newdata Data frame for prediction. If missing, predictions are made for the original data.
##' @param se Logical; if \code{TRUE}, computes standard errors and confidence intervals.
##' @param response Logical; if \code{TRUE} (default), returns probabilities only for the observed response
##' (if \code{newdata} contains the response). If \code{FALSE}, returns probabilities for all categories.
##' @param Y Vector of outcome levels to predict probabilities for (optional).
##' @param level Confidence level for intervals (default 0.95).
##' @param ... Additional arguments.
##'
##' @return A matrix or data frame containing:
##' \item{pred}{Predicted probabilities.}
##' \item{se}{Standard errors (if \code{se=TRUE}).}
##' \item{lower, upper}{Confidence intervals (if \code{se=TRUE}).}
##'
##' If \code{response=FALSE}, columns correspond to the levels of the outcome factor.
##'
##' @author Thomas Scheike
##' @seealso \code{\link{mlogit}}
##' @export
predict.mlogit <- function (object, newdata , se = TRUE, response=TRUE , Y=NULL,level=0.95,...)
{# {{{

  ylev <- levels(object$design$y)
  if (missing(newdata)) {
     X <- object$design$x
     ## take response from newdata if it is there and it is not given  
     if (response) Y <- as.numeric(object$design$y) 
  }  else {
     respindata <- length((grep(all.vars(object$formula)[1],names(newdata))))
     if (respindata>0 & response) x <- update_design(object$design,data = newdata,response=TRUE)
     else x <- update_design(object$design,data = newdata)
     X <- x$x
     ## take response from newdata if it is there and it is not given  
     if (is.null(Y))  { 
         if (response & !is.null(x$y))   
	   Y <- as.numeric(factor(x$y,levels=ylev))
     } else {
	     if (!is.numeric(Y)) 
		     Y <- as.numeric(factor(Y,levels=ylev))
     }
  }

  refg <- 1  ### else refg <- match(ref,types)
  nrefs <- (1:(object$nlev-1))
  px <- ncol(X)
  Xbeta <- c()
  for (i in nrefs) { Xbeta <- cbind(Xbeta,X %*% object$coef[(1:px)+px*(i-1)]);  }

  ppp <- cbind(1,exp(Xbeta))
  spp <- apply(ppp,1,sum)
  pp <- ppp/spp
  colnames(pp) <- ylev
  alpha <- 1-level
  crit <- qnorm(1-alpha/2)

  if (!is.null(Y)) {
	  Yg2 <- 1*(Y>=2)
          pp <- p <- c(mdi(pp,1:length(Y),Y)) 
          pppy <- c(mdi(ppp,1:length(Y),Y)) 
     if (se) {
	     Dppy <-  (spp*Yg2-pppy) 
	     Dp <- c()
             for (i in nrefs) Dp <- cbind(Dp,X*ppp[,i+1]*Dppy/spp^2);  
             covv <- if (!is.null(object$var)) object$var else vcov(object)
             if (is.null(covv)) stop("No variance-covariance matrix available.")
	     se <-  apply((Dp %*% covv) * Dp,1,sum)^.5
	     cmat <- data.frame(pred = p, se = se, lower = p - crit * se, upper = p + crit * se)
             names(cmat)[1:4] <- c("pred", "se", "lower", "upper")
             pp <- cmat
     }
 }

  return(pp)
}# }}}

##' Discrete Time-to-Event Analysis with Interval Censoring
##'
##' Fits a cumulative odds model for discrete time-to-event data, handling interval 
##' censoring where the event time is known only to lie within an interval \eqn{(t_l, t_r]}.
##' The model assumes:
##' \deqn{ \text{logit}(P(T \leq t | x)) = \log(G(t)) + x \beta }
##' where \eqn{G(t)} is the baseline cumulative odds function and \eqn{\beta} are the 
##' regression coefficients. This is equivalent to:
##' \deqn{ P(T \leq t | x) = \frac{G(t) \exp(x \beta)}{1 + G(t) \exp(x \beta)} }
##' 
##' The baseline \eqn{G(t)} is parameterized as the cumulative sum of exponentials 
##' (\eqn{G(t) = \sum \exp(\alpha)}), ensuring positivity. The regression coefficients 
##' describe the log-odds of the event occurring by time \eqn{t}.
##' 
##' The likelihood is maximized over the observed intervals:
##' \deqn{ L = \prod_i [ P(T_i > t_{il} | x_i) - P(T_i > t_{ir} | x_i) ] }
##' where \eqn{t_{il}} and \eqn{t_{ir}} are the left and right endpoints of the interval 
##' for subject \eqn{i}. Right-censored intervals have \eqn{t_{ir} = \infty}.
##'
##' @param formula Formula with an \code{Interval} object (e.g., \code{Interval(entry, time)}) 
##'   on the left-hand side and covariates on the right. Can include \code{cluster()} for 
##'   correlated data.
##' @param data Data frame containing the variables in the formula.
##' @param beta Starting values for the optimization (vector of length \eqn{p + k}, where 
##'   \eqn{p} is the number of covariates and \eqn{k} is the number of time intervals).
##' @param no.opt Logical; if TRUE, skips optimization and returns estimates based on 
##'   the provided \code{beta} (useful for initialization).
##' @param method Optimization method: \code{"NR"} (Newton-Raphson, default) or \code{"nlm"}.
##' @param stderr Logical; if FALSE, returns only the coefficient estimates.
##' @param weights Observation weights (follows ID).
##' @param offsets Offsets (follows ID).
##' @param exp.link Logical; if TRUE, parameterizes increments as \eqn{\exp(\alpha) > 0}.
##' @param increment Logical; if TRUE, uses increments \eqn{dG(t) = \exp(\alpha)} as parameters.
##' @param ... Additional arguments passed to the optimizer (\code{lava::NR} or \code{nlm}).
##' @return An object of class \code{"cumoddsreg"} containing:
##'   \item{coef}{Estimated coefficients (baseline time effects and covariate effects).}
##'   \item{se.coef}{Standard errors of the coefficients.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{iid}{Influence function (IID) decomposition for robust variance estimation.}
##'   \item{ntimes}{Number of distinct time intervals.}
##'   \item{utimes}{Unique time points.}
##'   \item{ploglik}{Log-likelihood at convergence.}
##'   \item{gradient, hessian}{Optimization results.}
##'   \item{call}{Original function call.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. H. (2024). Discrete time survival analysis with interval censoring. mets package documentation.
##' @seealso \code{\link{cumoddsreg}}, \code{\link{predictlogitSurvd}}, \code{\link{simlogitSurvd}}
##' @examples
##' data(ttpd) 
##' dtable(ttpd,~entry+time2)
##' 
##' out <- interval_logitsurv_discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
##' summary(out)
##' head(iid(out)) 
##' 
##' pred <- predictlogitSurvd(out,se=FALSE)
##' plotSurvd(pred)
##' 
##' ttpd <- dfactor(ttpd,fentry~entry)
##' out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
##' summary(out)
##' @aliases Interval dInterval predictlogitSurvd predictSurvd plotSurvd
##' @export
interval_logitsurv_discrete <- function (formula,data,beta=NULL,no.opt=FALSE,method="NR",
	   stderr=TRUE,weights=NULL,offsets=NULL,exp.link=1,increment=1,...)
{ ## {{{ 

  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")

 if (ncol(Y)==2) {
	time2 <- eventtime <- Y[,2]
	entrytime <- Y[,1]
	left <- 0
    } else {
	time2 <- eventtime <- Y[,2]
	status <- delta  <- Y[,3]
	entrytime <- Y[,1]
	left <- 1
	if (max(entrytime)==0) left <- 0
    }

  if (any(entrytime==time2)) stop("left==right not possible for discrete data")

  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}

  X <- model.matrix(Terms, m)
  if (!is.null(intpos  <- attributes(Terms)$intercept))
  X <- X[,-intpos,drop=FALSE]
  if (ncol(X)>0) X.names <- colnames(X) else X.names <- NULL

###  if (!is.null(id)) {
###	  ids <- unique(id)
###	  nid <- length(ids)
###      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
###      id <- as.integer(factor(id,labels=seq(nid)))-1
###     }
###   } else { id <- as.integer(seq_along(time2))-1; nid <- length(time2) }
###   ## orginal id coding into integers 
###   id.orig <- id+1; 

  call.id <- id
  conid <- construct_id(id,nrow(X))
  id <- conid$id; nid <- conid$nid; name.id <- conid$name.id

  ## times 1 -> mutimes , 0 til start
  utimes <- sort(unique(c(time2,entrytime)))
  mutimes <- max(utimes[utimes<Inf])
  n <- length(time2)
  
  ## setting up designs for t_l and t_r
  tR <- tL <- matrix(0,n,mutimes)

  ## design for ]t_l,t_r], for t_l=0 row is  0
  if (increment==0) {
	  tL <- matdoubleindex(tL,1:n,entrytime,rep(1,n))
	  tR <- matdoubleindex(tR,1:n,time2,rep(1,n))
  } else {
     for (i in 1:mutimes) {
        tL[,i] <- (i <= entrytime) 
        tR[,i] <- (i <= time2) 
     }
  }

  ## computing X^2 
  if (ncol(X)>0) {
  X2  <- .Call("vecMatMat",X,X)$vXZ
  XtL  <- .Call("vecMatMat",X,tL)$vXZ
  XtR  <- .Call("vecMatMat",X,tR)$vXZ
  }
  tL2  <- .Call("vecMatMat",tL,tL)$vXZ
  tR2  <- .Call("vecMatMat",tR,tR)$vXZ

  ## weights/offets will follow id 
  if (is.null(weights))  weights <- rep(1,n); #  else wiid <- weights
  if (is.null(offsets))  offsets <- rep(0,n); # else offsets <- offsets

  if (is.null(beta)) {
     beta <- rep(0,ncol(X)+mutimes)
     Set <- 1-cumsum(table(time2))/n
     dHt <- log(diff(c(0,(1/Set-1))))
     beta[1:mutimes] <- dHt[1:mutimes]
  }


obj <- function(pp,all=FALSE)
{ # {{{

if (exp.link==1) theta <- exp(pp[1:mutimes]) else theta  <-  pp[1:mutimes]

if (ncol(X)>0) {
betal   <- pp[-c(1:mutimes)]
Zbeta <- c(X %*% betal+offsets)
} else {Zbeta <- offsets }
xltheta <-  c(tL %*% theta)
xrtheta <-  c(tR %*% theta)
EZbeta <- exp(Zbeta)
GEl <- xltheta * EZbeta
GEr <- xrtheta * EZbeta
Stl <- 1/(1+GEl) 
Str <- 1/(1+GEr) 

############################################
## likelihood
############################################
# {{{
p <- Stl-Str*(time2<Inf)
logp <- log(p)
logl <- weights*logp 
# }}}
############################################
## Derivative 
############################################
# {{{
if (ncol(X)>0) {
DbetaStl <- -X*GEl*Stl^2
DbetaStr <- -(time2<Inf)*X*GEr*Str^2
Dbetalogp <- (DbetaStl-DbetaStr)/p
} 
DgStl <- -tL*EZbeta*Stl^2
DgStr <- -(time2<Inf)*tR*EZbeta*Str^2
Dglogp <- (DgStl-DgStr)/p

if (ncol(X)>0) {
Dlogliid  <- cbind(Dglogp*weights,Dbetalogp*weights)
Dlogl  <- apply(Dlogliid,2,sum)
} else {
	Dlogliid  <- Dglogp*weights
        Dlogl  <- apply(Dlogliid,2,sum)
}
if (exp.link==1) {
Dlogliid[,1:mutimes] <- t( t(Dlogliid[,1:mutimes])*theta)
Dlogl[1:mutimes] <- Dlogl[1:mutimes]*theta
}
# }}}
############################################
## 2nd Derivative 
############################################
# {{{
if (ncol(X)>0) {
D2betaStl <- X2*(GEl^2-GEl)*Stl^3
D2betaStr <- X2*(time2<Inf)*(GEr^2-GEr)*Str^3
DbetagStl <- XtL*            (EZbeta*(GEl-1))*Stl^3
DbetagStr <- XtR*(time2<Inf)*(EZbeta*(GEr-1))*Str^3
}
D2gStl <-             2*tL2*EZbeta^2*Stl^3
D2gStr <- 2*tR2*(time2<Inf)*EZbeta^2*Str^3

DglpDglp       <- .Call("vecMatMat",Dglogp,Dglogp)$vXZ
D2glogp <-    (D2gStl-D2gStr)/p  - DglpDglp
D2g <-    apply(weights*D2glogp,2,sum)

## cross products of derivatives
if (ncol(X)>0) {
DbetalpDbetalp <- .Call("vecMatMat",Dbetalogp,Dbetalogp)$vXZ
DbetalpDglp    <- .Call("vecMatMat",Dbetalogp,Dglogp)$vXZ

D2betalogp <- (D2betaStl-D2betaStr)/p - DbetalpDbetalp
Dbetaglogp <- (DbetagStl-DbetagStr)/p  - DbetalpDglp
D2beta <- apply(weights*D2betalogp,2,sum)
Dbetag <- apply(weights*Dbetaglogp,2,sum)
}

D2log <- matrix(0,length(pp),length(pp))
D2log[1:mutimes,1:mutimes] <- D2g

if (ncol(X)>0) {
###D2log[1:mutimes,(mutimes+1):length(pp)] <- Dbetag
D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag
D2log[1:mutimes,(mutimes+1):length(pp)] <- t(D2log[(mutimes+1):length(pp),1:mutimes])
D2log[(mutimes+1):length(pp),(mutimes+1):length(pp)] <- D2beta
}

diagM <- function(vec)
{# {{{
	p <- length(vec)
	out <- matrix(0,p,p)
	out <- matdoubleindex(out,1:p,1:p,vec)
	return(out)
}
# }}}

if (exp.link==1) {
D2log[1:mutimes,1:mutimes] <- diagM(Dlogl[1:mutimes]) + D2log[1:mutimes,1:mutimes]*(theta %o% theta)  
if (ncol(X)>0) {
D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag*rep(theta,each=length(betal))
###D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag*rep(theta,length(betal))
D2log[1:mutimes,(mutimes+1):length(pp)] <- t(D2log[(mutimes+1):length(pp),1:mutimes])
}
}

# }}}
############################################

ploglik <- sum(logl)
gradient <- Dlogl 
hessian <- D2log

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogliid %*% ihess ## %*% t(Dlogl) 
      beta.iid <- apply(beta.iid,2,sumstrata,id,nid)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
		  iid=beta.iid,robvar=robvar,var=robvar,id=id,
		  se=diag(robvar)^.5,coef=pp,se.coef=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik/nid,gradient=-gradient/nid,hessian=-hessian/nid)
}# }}}

  p <- ncol(X)
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      if (!stderr) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=beta),obj(beta,all=TRUE))

  uu <- utimes[utimes<Inf]
  Xnames <- c(paste("time",uu[-1],sep=""),X.names)
  if (length(val$coef)==length(Xnames)) names(val$coef) <- Xnames

  if (is.matrix(val$iid)) 
	  if (length(name.id)==nrow(val$iid)) rownames(val$iid) <- name.id

  val <- c(list(increment=increment,exp.link=exp.link,ntimes=mutimes,utimes=utimes,
		name.id=name.id,call.id=call.id,nid=nid),val)

  class(val) <- c("cumoddsreg")
  return(val)
} ## }}} 

##' @export
IC.cumoddsreg <- function(x,...) { x$iid*NROW(x$iid) }

##' Cumulative Odds Regression for Discrete Time Data
##'
##' A wrapper function for \code{interval_logitsurv_discrete} that simplifies the 
##' interface for discrete time-to-event data where the event time is observed exactly 
##' or as a factor level. It converts a factor response into an interval-censored format 
##' internally.
##'
##' @param formula Formula with a factor response on the left-hand side (representing 
##'   the event time) and covariates on the right.
##' @param data Data frame.
##' @param ... Arguments passed to \code{interval_logitsurv_discrete}.
##' @return An object of class \code{"cumoddsreg"} with the same structure as 
##'   \code{interval_logitsurv_discrete}.
##' @author Thomas Scheike
##' @seealso \code{\link{interval_logitsurv_discrete}}
##' @export
cumoddsreg <- function (formula,data,...)
{ ## {{{ 
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  nt <- nlevels(Y)
  time2__ <- as.numeric(Y)
  entrytime__ <- time2__-1
  time2__[time2__==nt] <- Inf

  xf <- update.formula(formula,Interval(entrytime__,time2__)~.)
  data$entrytime__ <- entrytime__
  data$time2__ <- time2__
  out <- interval_logitsurv_discrete(xf,data,...)

 return(out)
} ## }}}

##' @export
Interval <- function (time, time2 , ...)
{# {{{
    out <- cbind(time, time2 )
    colnames(out) <- c("entry", "time2")
    class(out) <- "Interval"
    return(out)
}# }}}

##' @export
dInterval <- function (time, time2 ,cuts=NULL,cut.first=0,show=FALSE, ...) 
{# {{{
if (is.null(cuts)) cuts <-  sort(unique(c(time,time2)))
if (min(cuts)> 0)  cuts <- c(cut.first,cuts)
###
lleft <- fast.approx(cuts,time,type="left")
rright <- fast.approx(cuts,time2,type="right")

out <- data.frame(time=time,time2=time2,left=lleft-1,right=rright-1)
attr(out,"cuts") <- cuts 
out$leftd <- cuts[lleft]
out$rightd <- cuts[rright]

if (max(cuts)==Inf) out$right[out$rightd==Inf] <- Inf

if (any(out$time<out$leftd)) warning("left not in [leftd,rightd]\n")
if (any(out$time2>out$rightd)) warning("right  not in [leftd,rightd]\n") 

if (show) {
	mtime <- mmtime <- max(cuts[cuts<Inf])
	n <- length(time)
	if (max(cuts)==Inf) mtime <- mmtime+1
	plot(0,0,xlim=c(0,mtime),ylim=c(0,n),type="n")
	abline(v=cuts,col=3)
	lines(c(min(cuts),mmtime),c(-1,-1),lwd=2)
	if (max(cuts)==Inf) lines(c(mtime,mmtime),c(-1,-1),lwd=2,col=2)
	for (i in 1:n)
	{
	lines(c(out$time[i],min(out$time2[i],mtime)),rep(i,2),col=1)
	lines(c(out$leftd[i],min(out$rightd[i],mtime)),rep(i+0.5,2),col=2)
	}
}

return(out)
}# }}}

##' @export
simlogitSurvd <- function(coef,Z,n=NULL,times=0:6)
{# {{{
  if (missing(Z)) Z <- NULL
  if (!is.null(Z)) n <- nrow(Z) 
  Z <- as.matrix(Z)

  g <- coef[times[-1]]
  beta <- coef[-(times[-1])]
  Gt <- cumsum(c(0,exp(g)))
  if (!is.null(Z)) EZbeta <-  c(exp(Z %*% beta)) else EZbeta <- rep(1,n)
  EZbeta <- rep(EZbeta,each=length(times))
  Gti <- rep(Gt,n)
  GE <- Gti*EZbeta
  Ft <- GE/(1+GE)

  ru <- runif(n)
  out <- data.frame(.Call("wherestrataR",ru,Ft,rep(0:(n-1),each=length(times)),n))
  status <- rep(1,n)
  names(out)[1] <- "time"
  status[out$time==(length(times)-1)] <- 0
  out$status <- status
  out <- cbind(out,Z)

return(out)
}# }}}

##' @export
predictlogitSurvd <- function(x,Z,n=NULL,times=NULL,se=FALSE,type="prob")
{# {{{

if (missing(Z)) Z <- NULL
  if (!is.null(Z))  {
	  n <- nrow(Z)
	  Z <- as.matrix(Z)
  } else { if (is.null(n)) n <- 1; }
  ccc <- x$coef

  ntimes <- x$ntimes
  g <- ccc[1:ntimes]
  beta <- ccc[-(1:ntimes)]
  if (is.null(times)) times <- 0:ntimes 

  Gt <- cumsum(c(0,exp(g)))
  if (!is.null(Z)) EZbeta <-  c(exp(Z %*% beta)) else EZbeta <- rep(1,n)

  if (!se) {# {{{
	  EZbeta <- rep(EZbeta,each=length(times))
	  id <- rep(1:n,each=length(times))
	  Gti <- rep(Gt,n)
	  GE <- Gti*EZbeta
	  Gt <- 1/(1+GE)
	  preds <- data.frame(pred=Gt,id=id,times=rep(times,n))
# }}}
  } else {# {{{

    Ft <- function(p,Zi=1)
    {# {{{
         g <- p[1:ntimes]
         beta <- p[-(1:ntimes)]
         Gt <- cumsum(c(0,exp(g)))
	 if (length(beta)>0) EZ <- exp(sum(Zi*beta)) else EZ <- 1
	 pred <- 1/(1+Gt*EZ)
	 return(pred)
    }# }}}

  preds <- c()
  for (i in 1:length(EZbeta)) {
     if (is.null(x$var)) covv <- vcov(x)  else covv <- x$var
     eud <- estimate(coef=x$coef,vcov=covv,f=function(p) Ft(p,Zi=Z[i,]))
     cmat <- data.frame(eud$coefmat)
     cmat$id <- i
     cmat$times <- times
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- rbind(preds,cmat)
  } 

  }# }}}

return(preds)
} ## }}} 

sim_TTP <- function(coef=NULL,n=100,Xglm=NULL,times=NULL)
{# {{{
	  
  Z <- Xglm  
  if (!is.null(Z)) n <- nrow(Z) 

  if (!is.null(Z)) data <- Z else data <- data.frame(id=1:n)

  if (!is.null(times)) {
     timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
     data <- merge(data,timesf,by.x="id",by.y="id")
     mt <- model.matrix(~factor(times),data)
     nm <- match(c("id","times"),names(data))
     Z <- cbind(mt,data[,-nm])
  }

  p <- c(expit(as.matrix(Z) %*% coef))
  y <- rbinom(length(p),1,p)

  data <- cbind(y,data)
  data <- count_history(data,status="y",id="id",types=1)
  data <- subset(data,data$Count1<=0)

  attr(data,"coef") <- coef
  return(data)
 }# }}}

##' @export
summary.cumoddsreg <- function(object,...) { ## {{{ 
	ntimes <- object$ntimes
	coefb <- coef(object)[1:ntimes]
	vcovb <- object$var[1:object$ntimes,1:object$ntimes]
	outb <- lava::estimate(coef=coefb,vcov=vcovb,...)

	if (length(object$coef)>ntimes) {
		ll <- length(object$coef)
		takex <- (ntimes+1):ll
		coefx <- object$coef[takex]
		vcovx <- object$var[takex,takex]
     	        outx <- lava::estimate(coef=coefx,vcov=vcovx,...)
	        eoutx <- exp(outx$coefmat[,c(1,3,4)])
	        out <- list(baseline=outb,logor=outx,or=eoutx)
	} else out <- list(baseline=outb)
	return(out)
} ## }}} 

##' @export
print.cumoddsreg <- function(x,...) summary(x,...)

##' @export
vcov.cumoddsreg <- function(object,...) return(object$var) 

##' @export
coef.cumoddsreg <- function(object,...) return(object$coef)

##' @export
predictSurvd <- function(ds,Z,times=1:6,se=FALSE,type="prob")
{# {{{
  if (!is.null(Z)) n <- nrow(Z) 
  if (!is.null(Z)) data <- Z else data <- data.frame(id=1:n)
  Z <- data.frame(Z)
  Z$id <- 1:n
  ccc <- ds$coef

  if (!se) {# {{{{{{
	  data <- Z
	  if (!is.null(times)) {
	     timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
	     data <- merge(data,timesf,by.x="id",by.y="id")
	     mt <- model.matrix(~factor(times),data)
	     nm <- match(c("id","times"),names(data))
	     Z <- cbind(mt,data[,-nm])
	  }
	  if (ncol(Z)!=length(c(ccc))) {
		  print(head(Z))
		  print(ccc)
		  stop("dimension of Z not consistent with length of coefficients"); 
	  }

	  p <- c(expit(as.matrix(Z) %*% ccc))

	  preds <- data.frame(p=p,id=data$id,times=data$times)
	  survt <- exp(cumsumstrata(log(1-preds$p),data$id-1,max(data$id)))
	  if (type=="prob") pred <- 1-survt
	  if (type=="surv") pred <- survt
	  if (type=="hazard") pred <- p
	  if (type=="rrm") { ## restricted residual mean 
		  ll <- length(survt)
	        pred <- cumsum(c(1,survt[-ll]))
	  }
	  preds <- cbind(preds,pred)
# }}}
  } else {# {{{

    Ft <- function(p)
    {
	   xp <- as.matrix(Zi) %*% p
	   lam <- expit(xp)
	   st <- cumprod(1-lam)
	   if (type=="prob") st <- 1-st 
	   if (type=="surv") st <- st 
	   if (type=="hazard") st <- lam
           if (type=="rrm") { ## restricted residual mean 
		ll <- length(st)
	        st <- cumsum(c(1,st[-ll]))
	   }
	   return(st)
    }

  preds <- c()
  for (i in 1:nrow(Z)) {
     Zi <- data.frame(Z[i,,drop=FALSE])
     data <- Zi
     if (!is.null(times)) {
        timesf <- data.frame(times=times, id=rep(i, length(times)))
###        timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
        data <- merge(data,timesf,by.x="id",by.y="id")
        mt <- model.matrix(~factor(times),data)
        nm <- match(c("id","times"),names(data))
        Zi <- cbind(mt,data[,-nm])
     }
     if (is.null(ds$var)) covv <- vcov(ds)  else covv <- ds$var
     eud <- estimate(coef=ds$coef,vcov=covv,f=function(p) Ft(p))
     cmat <- data.frame(eud$coefmat)
     cmat$id <- i
     cmat$times <- times
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- rbind(preds,cmat)
  } 

  }# }}}

return(preds)
}# }}}

## }}} 

##' @export
plotSurvd <- function(ds,ids=NULL,add=FALSE,se=FALSE,cols=NULL,ltys=NULL,...)
{# {{{

 if (is.null(ids)) ids <- unique(ds$id)
 if (is.null(cols)) cols <- 1:length(ids)
 if (is.null(ltys)) ltys <- 1:length(ids)

  k <- 1
  fplot <- 0
  for (i in ids) {
	  timei <- ds$time[ds$id==i]
	  predi <- ds$pred[ds$id==i]

 	  if (fplot==0) {
	  if (!add) plot(timei,predi,type="s",col=cols[k],lty=ltys[k],...)
	  if (add) lines(timei,predi,type="s",col=cols[k],lty=ltys[k],...)
	  fplot <- 1
	  } else lines(timei,predi,type="s",col=cols[k],lty=ltys[k],...)

          if (se) {
	  loweri <- ds$lower[ds$id==i]
	  upperi <- ds$upper[ds$id==i]
	  plotConfRegion(timei,cbind(loweri,upperi),col=cols[k])
	  }
	  k <- k+1
  }

} ## }}} 

## {{{ hazard simulation

##' Simulation of Piecewise Constant Hazard Model (Cox)
##'
##' Simulates data from a piecewise constant baseline hazard that can also be of Cox type. 
##' Censors data at the highest value of the break points.
##' 
##' For a piecewise linear cumulative hazard, the inverse is easy to compute. With delayed 
##' entry \eqn{x}, we compute:
##' \deqn{\Lambda^{-1}(\Lambda(x) + E/RR)}
##' where \eqn{RR} are the relative risks and \eqn{E} is exponential with mean 1.
##' This quantity has survival function:
##' \deqn{P(T > t | T>x) = \exp(-RR (\Lambda(t) - \Lambda(x)))}
##'
##' @param cumhazard Cumulative hazard matrix (columns: time, cumulative hazard), or 
##'   piece-constant rates for periods defined by the first column of input.
##' @param rr Relative risk vector for simulations, or alternatively when \code{rr=1} 
##'   specify \code{n}.
##' @param n Number of simulations if \code{rr} not given.
##' @param entry Delayed entry time for simulations (optional).
##' @param cause Name/code of the event cause (default 1).
##' @param extend To extend piecewise constant with constant rate beyond the last break point. 
##'   Default is \code{FALSE}. If \code{TRUE}, extends with average rate over time from 
##'   cumulative. If numeric, uses the given rate.
##' @return A data frame containing:
##'   \item{entry}{Entry times.}
##'   \item{time}{Event/censoring times.}
##'   \item{status}{Event status (1=event, 0=censored).}
##'   \item{rr}{Relative risks used.}
##'   
##'   Attributes include:
##'   \item{cumhaz}{The cumulative hazard used.}
##'   \item{extend.rate}{The extension rate if used.}
##' @author Thomas Scheike
##' @keywords survival simulation
##' @examples
##' chaz <-  c(0,1,1.5,2,2.1)
##' breaks <- c(0,10,   20,  30,   40)
##' cumhaz <- cbind(breaks,chaz)
##' n <- 10
##' X <- rbinom(n,1,0.5)
##' beta <- 0.2
##' rrcox <- exp(X * beta)
##' 
##' pctime <- rchaz(cumhaz,n=10)
##' pctimecox <- rchaz(cumhaz,rrcox,entry=runif(n))
#' @export 
#' @aliases sim_rchaz lin_approx 
rchaz <- function(cumhazard,rr,n=NULL,entry=NULL,cause=1,extend=FALSE)
{# {{{
  if (!is.null(n)) rr <- rep(1,n)
  n <- length(rr)

  breaks <- cumhazard[,1]
  mm <- tail(breaks,1)
  cumh <- cumhazard[,2] 
   ttt <- rexp(n)/rr
   if (cumhazard[1,2]>0)  { ## start cumulative hazard with a 0
###   warning("Safest to start with cumulative hazard 0 to avoid problems\n"); 
      cumhazard <- rbind(c(0,0),cumhazard)
      cumh <- c(0,cumh)
   }
   ###
   if (!is.null(entry)) {
	   if (length(entry)==1) entry <- rep(entry,n) 
	   cumentry <- lin_approx(entry,cumhazard,x=1)
	   if (any(entry>tail(breaks,1)))  stop("Some entry times further out than last cumulative hazard time\n"); 
   } else { entry <- cumentry <- rep(0,n) }
   ###
   ttte <- ttt+cumentry
   rrx <- lin_approx(ttte,cumhazard,x=-1)
   rrx <- ifelse(rrx>mm,mm,rrx)
   status <- rep(0,n)
   status <- ifelse(rrx<mm,cause,status)
   tcum <- tail(cumhazard,1)
   extend.rate <- NULL
   if (is.logical(extend))
	   if (extend) extend.rate <- tcum[2]/tcum[1]
   if (is.numeric(extend)) { extend.rate <- extend; extend <- TRUE;}
   if (extend) {
	   whoe <- which(status==0)
	   nn <- length(whoe)
	   if (nn>0) {
	     textend <- rexp(nn)/(rr[whoe]*extend.rate)
	     rrx[whoe] <- mm+textend
	     status[whoe] <- 1
	   }
   }
   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr)
   colnames(dt) <- c("entry","time","status","rr"); 
   attr(dt,"cumhaz") <- cumhazard
   attr(dt,"extend.rate") <- extend.rate
   return(dt)
}# }}}

##' Multiple Cause Piecewise Constant Hazard Simulation
##'
##' Simulates data from multiple piecewise constant baseline hazards for competing risks. 
##' Takes the minimum of all cause-specific event times and assigns the corresponding cause.
##'
##' @param cumhaz List of cumulative hazard matrices, one for each cause.
##' @param rr Matrix of relative risks (rows = subjects, columns = causes).
##' @param causes Vector of cause codes to assign (default NULL, uses 1,2,...).
##' @param ... Additional arguments passed to \code{rchaz}.
##' @return A data frame with event times and status indicating the cause of the first event.
##' @author Thomas Scheike
##' @seealso \code{\link{rchaz}}
##' @export 
rchazl <- function (cumhaz, rr, causes=NULL,...)
{# {{{
    l <- length(cumhaz)
    tall <- rchaz(cumhaz[[1]], rr[, 1], ...)
    if (l >= 2)
        for (i in 2:l) {
            tall2 <- rchaz(cumhaz[[i]], rr[, i], ...)
###            tall$status <- ifelse(tall$time < tall2$time, tall$status, i * tall2$status)
	    tall$status <- ifelse(tall$time < tall2$time, tall$status,
                      ifelse(tall2$status == 0, 0, i))
            tall$time <- pmin(tall$time, tall2$time)
        }
 if (!is.null(causes)) {
      where <- which(tall$status!=0) 
      tall$status[where] <- causes[tall$status[where]]
 }
    return(tall)
}# }}}

#' @export
lin_approx <- function (x2, xfx, x = 1)
{ ## {{{ 
    breaks <- xfx[, x]
    fx <- xfx[, -x]
    ri <- fast.approx(breaks, x2, type = "left")
    maxindex <- which(ri == length(breaks))
    rip1 <- ri + 1
    rip1[maxindex] <- length(breaks)
    rrr <- (x2 - breaks[ri])/(breaks[rip1] - breaks[ri])
    rrr[maxindex] <- 0
    res <- rrr * (fx[rip1] - fx[ri]) + fx[ri]
    res[is.na(res)] <- tail(fx, 1)
    res[is.na(rrr)] <- fx[ri][is.na(rrr)]
    return(res)
} ## }}}

addCums <- function(cumB,cumA,max=NULL)
{# {{{
 ## take times
 times <- sort(unique(c(cumB[,1],cumA[,1])))
 if (!is.null(max)) times <- times[times<max]
 cumBjx <- lin_approx(times,cumB,x=1)
 cumAjx <- lin_approx(times,cumA,x=1)
 cumBA <- cumBjx+cumAjx
 return(cbind(times,cumBA))
}# }}}

##' Extend Cumulative Hazard Functions to Common Time Range
##'
##' Extends a collection of cumulative hazard functions so that they all
##' cover the same time range. Cumulative hazards that end before the
##' maximum observed time are extrapolated linearly using either a
##' user-supplied rate or the average hazard rate estimated from the
##' existing cumulative hazard.
##'
##' @param cumA a cumulative hazard matrix (two columns: time, cumulative hazard)
##'        or a list of such matrices.
##' @param cumB an optional second cumulative hazard matrix. If provided it is
##'        combined with \code{cumA} into a list.
##' @param extend numeric vector of hazard rates used for extrapolation, one per
##'        cumulative hazard. If \code{NULL} (default), rates are estimated as
##'        the ratio of the last cumulative hazard value to the last time point.
##' @return A list of cumulative hazard matrices extended to the common maximum time.
##' @seealso \code{\link{rchaz}}, \code{\link{rcrisk}}, \code{\link{mets-simulation}}
##' @export
extendCums <- function(cumA,cumB,extend=NULL)
{# {{{
## setup as list to run within loop
if (!is.null(cumB)) { cumA <- list(cumA,cumB); } 

## to also work with strata version where each list contains a list of cumHaz for strata
###matrixlist <- any(unlist(lapply(cumA, function(x) is.data.frame(x[[1]]) | is.matrix(x[[1]]))))
## any stratified components 
basecumhaz <- any(unlist(lapply(cumA, function(x) inherits(x,"basecumhaz"))))
nn <- length(cumA)
nl <- lengths(cumA)
if (basecumhaz) 
	cumA <- unlist(cumA, recursive = FALSE)

restore <- function(flat, lengths) {
  ends <- cumsum(lengths)
  starts <- c(1, head(ends + 1, -1))
  mapply(function(s, e) flat[s:e], starts, ends, SIMPLIFY = FALSE)
}

 maxx <- unlist(lapply(cumA,function(x) tail(x,1)[1]))
 mm <- which.max(maxx)
 haza <- NULL
 if (is.numeric(extend)) 
    haza <- rep(extend, length.out = length(cumA))

 ## extend all that are not at maxtime
for (i in seq(length(cumA))[-mm]) {
  cumB <- cumA[[i]]; 
  if (cumB[1,1]!=0 & cumB[1,2]!=0) cumB <- rbind(0,cumB); 

  ### linear extrapolation of mortality using given dHaz/dt or haza given rate
  if (tail(cumB[,1],1)<maxx[mm]) {
      tailB <- tail(cumB,1)
      cumlast <- tailB[2]
      timelast <- tailB[1]
      if (is.null(haza)) hazb <- cumlast/timelast else hazb <- haza[i]
      cumB <- rbind(cumB,c(maxx[mm],cumlast+hazb*(maxx[mm]-timelast))) 
  }
  cumA[[i]] <- cumB
}

 if (basecumhaz) cumA <- restore(cumA,nl)
 return( setNames(cumA,paste("cum",seq(nn),sep="")))
}# }}}

#' @export
simrchaz <- function(cumhazard,rr,n=NULL,cens=NULL,rrc=NULL,entry=NULL,...)
{# {{{
###   adds censoring to to rchaz call
   if (!is.null(n)) rr <- rep(1,n)
   n <- length(rr)  

   if (is.list(cumhazard)) ptt <- rchazl(cumhazard,rr,entry=entry,...) else 
	   ptt <- rchaz(cumhazard,rr,entry=entry,...)

   if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
      dt <- data.frame(time=pmin(ptt$time,pct),status=ifelse(ptt$time<pct,ptt$status,0))
   } else dt <- ptt[,c("time","status")]

   return(dt)
}# }}}

simCens <- function(cens,rrc=NULL,n=NULL,entry=NULL,...)
{# {{{
   if (is.null(rrc) & is.null(n)) stop("must give either rr or n\n"); 
   if (is.null(rrc)) rrc <- rep(1,n)
   n <- length(rrc)  
   if (is.matrix(cens)) {
	   pct <- rchaz(cens,rrc,entry=entry,...)
	   pct <- pct$time
   } else if (is.numeric(cens)) {
	   pct<- rexp(n)/(rrc*cens)  
           if (!is.null(entry)) pct <- entry+pct
   } else stop("cens must be cumulative hazard or constant rate\n"); 

   return(pct)
} ## }}}

#' Simulation of Piecewise constant hazard models with two causes (Cox).
#' 
#' Simulates data from piecwise constant baseline hazard that can also be of
#' Cox type. Censor data at highest value of the break points for either of the
#' cumulatives, see also sim_phregs  
#' 
#' @param cumA cumulative hazard of cause 1, or list of multiple cumulative hazards
#' @param cumB cumulative hazard of cause 2 or NULL when cumA is a list 
#' @param rr1 number of simulations or vector of relative risk for simuations, or matrix with columns equal to number of hazards in list
#' @param rr2 number of simulations or vector of relative risk for simuations.
#' @param n number of simulation if rr not given, must be given when rr is not given  
#' @param cens to censor further , rate or cumumlative hazard
#' @param rrc retlativ risk for censoring.
#' @param extend to extend the cumulative hazards to largest end-point 
#' @param causes to assign status values for each of the causes, vector of integers 
#' @param ... arguments for rchaz 
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' data(bmt); 
#' n <- 100
#' cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
#' cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)
#'
#' X1 <- bmt[,c("tcell","platelet")]
#' xid <- sample(1:nrow(X1),n,replace=TRUE)
#' Z1 <- X1[xid,]
#' Z2 <- X1[xid,]
#' rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
#' rr2 <- exp(as.matrix(Z2) %*% cox2$coef)
#'
#' d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=2/70)
#' dd <- cbind(d,Z1)
#'
#' d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=cbind(c(1,30,68),c(.01,1,3)))
#' dd <- cbind(d,Z1)
#'
#' par(mfrow=c(1,3))
#' scox0 <- phreg(Surv(time,status==0)~tcell+platelet,data=dd)
#' plot(scox0); lines(cbind(c(1,30,68),c(.01,1,3)),col=2)
#' ##
#' scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
#' scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
#' plot(cox1); plot(scox1,add=TRUE,col=2)
#' plot(cox2); plot(scox2,add=TRUE,col=2)
#' cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
#' 
#' # 3 causes and censoring 
#' d3 <-  rcrisk(list(cox1$cum,cox2$cum,cox1$cum),NULL,n=100,cens=cbind(c(1,30,68),c(.01,1,3)))
#' dtable(d3,~status)
#' 
#' @export 
rcrisk <-function(cumA,cumB,rr1=NULL,rr2=NULL,n=NULL,
		  cens=NULL,rrc=NULL,extend=TRUE,causes=NULL,...)
{# {{{
	if (!is.null(cumB)) {
		cumA <- list(cumA,cumB);
	} else if (!is.list(cumA)) cumA <- list(cumA)
	l <- length(cumA)

    ## --- resolve n and build rr matrix -------------------------------
    ## collect any supplied rr vectors into a list, dropping NULLs
    rr_supplied <- Filter(Negate(is.null), list(rr1, rr2))

    if (length(rr_supplied) == 0) {
        ## no rr given: need n explicitly
        if (is.null(n)) stop("supply 'n' or at least one of 'rr1', 'rr2'\n")
        rr <- matrix(1, n, l)
    } else {
        ## derive n from the first supplied rr
        n <- length(rr_supplied[[1]])
        ## build matrix: supplied columns first, remainder filled with 1
        rr_cols <- vector("list", l)
        rr_cols[[1]] <- if (!is.null(rr1)) rr1 else rep(1, n)
        rr_cols[[2]] <- if (!is.null(rr2)) rr2 else rep(1, n)
        if (l > 2)
            for (i in seq(3, l)) rr_cols[[i]] <- rep(1, n)
        rr <- do.call(cbind, rr_cols)
    }

	if (!is.null(extend))  cumA <- extendCums(cumA,NULL,extend=extend)

	l <- length(cumA)
	## simulate all hazards  for all causes
        ptt <- rchazl(cumA,rr,causes=causes,...)

	## add censoring 
	if (!is.null(cens)) {
		pct <- simCens(cens,rrc=rrc,n=n,...)
		ptt$time <- pmin(ptt$time,pct)
		ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
	}

	return(ptt)
}# }}}

##' Simulation of Output from Cox Model
##'
##' Simulates data that looks like fit from a Cox model. Automatically censors data 
##' for the highest value of the event times by using cumulative hazard.
##'
##' @param cox Output from \code{coxph} or \code{phreg} model fitting.
##' @param n Number of simulations.
##' @param data Data frame to extract covariates for simulations (draws from observed covariates).
##' @param Z Design matrix instead of data.
##' @param rr Vector of relative risks for Cox model.
##' @param strata Vector of strata.
##' @param entry Delayed entry variable for simulation.
##' @param extend Extend possible stratified baselines to largest endpoint.
##' @param cens Censoring specification (matrix = cumulative hazard, scalar = rate).
##' @param rrc Relative risks for Cox-type censoring.
##' @param ... Arguments for \code{rchaz} (e.g., entry-time).
##' @return Data frame with simulated event times, status, and covariates.
##' @author Thomas Scheike
##' @keywords survival simulation
#' @examples
#' data(sTRACE)
#' nsim <- 100
#' coxs <-  phreg(Surv(time,status==9)~strata(chf)+vf+wmi,data=sTRACE)
#' set.seed(100)
#' sim3 <- sim_phreg(coxs,nsim,data=sTRACE)
#' head(sim3)
#' cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim3)
#' cbind(coxs$coef,cc$coef)
#' plot(coxs,col=1); plot(cc,add=TRUE,col=2)
#' 
#' Z <- sim3[,c("vf","chf","wmi")]
#' strata <- sim3[,c("chf")]
#' rr <- exp(as.matrix(Z[,-2]) %*% coef(coxs))
#' sim4 <- sim_phreg(coxs,nsim,data=NULL,rr=rr,strata=strata)
#' sim4 <- cbind(sim4,Z)
#' cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim4)
#' cbind(coxs$coef,cc$coef)
#' plot(coxs,col=1); plot(cc,add=TRUE,col=2)
#' 
#' @export sim_phreg 
sim_phreg <- function(cox,n,data=NULL,Z=NULL,rr=NULL,strata=NULL,
		      entry=NULL,extend=TRUE,cens=NULL,rrc=NULL,...)
{# {{{
	if  (!is.null(data)) {
		scox1 <- draw_phreg(cox,n,data=data,onlyX=TRUE,...)
		dat <- scox1$data
		dat$orig.id <- scox1$id

		if (is.null(strata))  strata <- scox1$strata 
		if (is.null(rr)) rr <- scox1$rr  
		n <- length(rr)
		id <- 1:length(rr)
	} else {
		if (!is.null(Z)) {
		   if (is.data.frame(Z)) znames <- names(Z) else znames <- colnames(Z)
		} else znames <- NULL
		if (is.null(rr) & (!is.null(Z))) rr <- exp(as.matrix(Z) %*% cox$coef)
		if (is.null(rr) & is.null(Z)) rr <- rep(1,n)
		id <- 1:length(rr)
		n <- length(rr)
		dat <- NULL
	}
	if (is.null(strata)) strata <- rep(0,n)

	if (inherits(cox,c("phreg","cifreg"))) cumhaz <- basecumhaz(cox,only=1)
	else {
		if (!is.list(cox)) stop("must be phreg or list of hazards\n") else cumhaz <- cox
	}
	if (!is.null(extend))  cumhaz <- extendCums(cumhaz,NULL,extend=extend)
	ids <- 1:n
	lentry <- NULL

	ptt <- c()
	for (i in unique(strata)) {
		whichi <- which(strata==i)
		cumhazj <- rbind(0,cumhaz[[i+1]])
		if (!is.null(entry)) 
			lentry <- entry[whichi]
		simj <- rchaz(cumhazj,rr[whichi],entry=lentry) 
		simj$id <- ids[whichi]
		ptt  <-  rbind(ptt,simj)
	}
	dsort(ptt) <- ~id

	if (!is.null(cens)) {
		pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
		ptt$time <- pmin(ptt$time,pct)
		ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

##### add correct names to entry,time,status
if (inherits(cox,"phreg"))  {
varsY <- all.vars(update(drop.specials(cox$formula,"cluster"),.~1)) 
if (length(varsY)==3) ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status)
if (length(varsY)==2)  {
	if (!is.null(entry)) { 
		varsY <- c("entry",varsY);  
	        ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status) 
	} else ptt[,varsY] <- cbind(ptt$time,ptt$status)
}
ptt <- dkeep(ptt,varsY)
}

if (!is.null(dat)) ptt <- cbind(ptt,dat)

return(ptt)
}# }}}


draw_phreg <- function(cox,n,data=NULL,Z=NULL,strata=NULL,
                       drawZ=TRUE,fixZ=FALSE,id=NULL,onlyX=TRUE)
{# {{{
###if (!inherits(cox,"phreg")) stop("must be phreg object\n"); 

## give data so that design can be constructed based on model-formula
if (!is.null(data)) {
cid <- countID(data.frame(id=cox$id))
whereid <- which(cid$Countid==1)
if (drawZ==TRUE) xid <- sample(whereid,n,replace=TRUE) else xid <- id
if (onlyX) vars <- all.vars(update(drop.specials(cox$formula,"cluster"),-1~.)) else vars <- all.vars(cox$formula)
dataid <- data[xid,vars,drop=FALSE] 

desX <- readPhreg(cox,dataid,data=FALSE)
Z <- desX$X
strata <- desX$strata
###   xx <- update_design(cox,data = dataid,response=FALSE) 
###   Z <- xx$x
###   if (!is.null(xx$strata)) strata <- as.numeric(xx$strata)-1 else strata <- rep(0,nrow(Z))
} else {  ## Z and strata
   xid <- 1:nrow(Z); 
   n <- nrow(Z); 
   dataid <- Z; 
   if (is.null(strata)) strata <- rep(0,n)
}

nz <- ncol(Z)
if (nz>0) rr <- exp(as.matrix(Z) %*% cox$coef) else rr <- rep(1,nrow(Z))
cumhaz <- rbind(c(0,0),cox$cumhaz)
   if (cox$nstrata>1) {
      stratname <-  substring(cox$strata.name,8,nchar(cox$strata.name)-1)
   } else stratname <- NULL
   model <-c(class(cox),is.null(cox$propodds))

out <- list(Z=Z,cumhaz=cumhaz,rr=rr,id=xid,model=model,strata=strata,data=dataid,stratname=stratname)

return(out)
} ## }}}

draw_phregs <- function(coxs,n,data,onlyX=TRUE,...) { ## {{{ 
   scox1 <- draw_phreg(coxs[[1]],n,data=data,onlyX=onlyX,...)
   datas <- scox1$data
   stratam <-  scox1$strata
   rrm <- scox1$rr

   if (length(coxs)>1) 
   for (i in 2:length(coxs)) {
      coxn <- draw_phreg(coxs[[i]],n,data=data,drawZ=FALSE,id=scox1$id,onlyX=onlyX,...)
      coxndata <- coxn$data
      ind <-  match(colnames(datas),colnames(coxndata),nomatch=0)
      ind <- ind[ind!=0]
      if (length(ind)>0)  datas <- cbind(datas,coxndata[,-ind,drop=FALSE]) else datas <- cbind(datas,coxndata)
      rrm <- cbind(rrm,coxn$rr)
      stratam <- cbind(stratam,coxn$strata)
   }
   datas <- data.frame(datas)
   datas$orig.id <- scox1$id
   out <- list(data=datas,rr=rrm,strata=stratam)

   return(out)
} ## }}} 

setup_phreg  <- function(cumhazard,coef,Znames=NULL,strata=NULL)
{# {{{
    cox <- list()
    cox$cumhaz <- cumhazard
    cox$coef <- coef
    if (is.null(strata)) { strata <- rep(0,nrow(cumhazard)); nstrata <- 1} else nstrata <- max(strata)+1
    cox$strata <- strata
    cox$nstrata <- nstrata
    cox$strata.name <- ""
    cox$jumps <- 1:nrow(cumhazard)
    class(cox) <- c("setup","phreg")
    attr(cox,"znames") <- Znames
    return(cox)
}# }}}


##' Simulation of Cause-Specific Cox Models
##'
##' Simulates data that looks like fit from cause-specific Cox models. Censors data 
##' automatically. When censoring is given in the list of causes, this provides censoring 
##' that looks like the data.
##'
##' @param coxs List of Cox models.
##' @param n Number of simulations.
##' @param data Data frame to extract covariates.
##' @param rr Relative risks.
##' @param strata Strata vector.
##' @param entry Delayed entry.
##' @param extend Extend baselines to largest endpoint.
##' @param cens Censoring specification.
##' @param rrc Relative risks for censoring.
##' @param ... Arguments for \code{rchaz}.
##' @return Data frame with simulated event times, status, and covariates.
##' @author Thomas Scheike
##' @keywords survival simulation
#' @examples
#' data(bmt)
#' nsim <- 100; 
#' 
#' cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet+age,data=bmt)
#' cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
#' coxs <- list(cox1,cox2)
#' ## just calls sim_phregs !
#' dd <- sim_phregs(coxs,nsim,data=bmt,extend=c(0.001))
#' scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet+age,data=dd)
#' scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)
#'
#' cbind(cox1$coef,scox1$coef)
#' cbind(cox2$coef,scox2$coef)
#' par(mfrow=c(1,2))
#' plot(cox1); plot(scox1,add=TRUE); 
#' plot(cox2); plot(scox2,add=TRUE); 
#' 
#' @export sim_phregs
sim_phregs <- function(coxs,n,data=NULL,rr=NULL,strata=NULL,
                       entry=NULL,extend=TRUE,cens=NULL,rrc=NULL,...)
{# {{{
   out <- draw_phregs(coxs,n,data)
   if (is.null(rr)) rr <- out$rr
   if (is.null(strata)) strata <- out$strata
   lentry <- NULL

   cumhazl <- list()
   cumhazl[[1]] <- basecumhaz(coxs[[1]],only=1)
   if (length(coxs)>1) 
   for (i in 2:length(coxs)) 
      cumhazl[[i]] <- basecumhaz(coxs[[i]],only=1)
   if (!is.null(extend)) cumhazl <- extendCums(cumhazl,NULL,extend=extend)

   ## simulate first  time
   simdata <- sim_phreg(cumhazl[[1]],n,data=NULL,rr=rr[,1],strata=strata[,1],entry=entry)
   l <- length(coxs)
   if (l>=2) 
   for (i in 2:l) {
      tall2 <- sim_phreg(cumhazl[[i]],n,rr=rr[,i],strata=strata[,i],entry=entry)
      simdata$status <- ifelse(simdata$time<tall2$time,simdata$status,i*tall2$status)
      simdata$time <- pmin(simdata$time,tall2$time)
   }
   ptt <- simdata

 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

##### add correct names to entry,time,status
if (inherits(coxs[[1]],"phreg"))  {
varsY <- all.vars(update(drop.specials(coxs[[1]]$formula,"cluster"),.~1)) 
if (length(varsY)==3) ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status)
if (length(varsY)==2)  {
	if (!is.null(entry)) { 
		varsY <- c("entry",varsY);  
	        ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status) 
	} else ptt[,varsY] <- cbind(ptt$time,ptt$status)
}
ptt <- dkeep(ptt,varsY)
}

if (!is.null(data)) ptt <- cbind(ptt,out$data)

return(ptt)
}# }}}


## }}} 

## {{{ cumulative incidence 

##' @export sim_cifs
sim_cifs <- function(cifs,n,data=NULL,rr=NULL,strata=NULL,Z=NULL,
                     cens=NULL,rrc=NULL,max.times=NULL,causes=c(1,2),
                     U=NULL,pU=NULL,extend=TRUE,type=NULL,restrict=TRUE,entry=NULL,...)
{# {{{
if (!is.list(cifs)) stop("Cif models in list form\n"); 
if (length(cifs)!=2) stop("Only two models\n"); 

if (!is.null(type)) { 
   if (length(type)==1) type <- rep(type,2)
   model1 <- type[1]; model2 <- type[2];
} else {
   if (is.null(cifs[[1]]$propodds)) model1 <- "cloglog" else model1 <- "logistic" 
   if (is.null(cifs[[2]]$propodds)) model2 <- "cloglog" else model2 <- "logistic" 
}

   out <- draw_phregs(cifs,n,data,Z=Z)
   if (is.null(rr)) rr <- out$rr
   if (is.null(strata)) strata <- out$strata

   cumhazl <- list()
   cumhazl[[1]] <- basecumhaz(cifs[[1]],only=1)
   if (length(cifs)>1) 
   for (i in 2:length(cifs)) 
      cumhazl[[i]] <- basecumhaz(cifs[[i]],only=1)
  if (!is.null(extend)) cumhazl <- extendCums(cumhazl,NULL,extend=extend)

  tau <- tail(cumhazl[[1]][[1]],1)[1]

  ## simulate first  time
  sim1 <- sim_cif(cumhazl[[1]],n,rr=rr[,1],strata=strata[,1],U=U,type=model1,entry=entry)
  sim2 <- sim_cif(cumhazl[[2]],n,rr=rr[,2],strata=strata[,2],U=U,type=model2,entry=entry)

   ## drawing which cause  is observed, 0,1,2
  if (!is.null(entry)) {
     if (restrict) {
	   F1tau <- sim1$F1tau- sim1$F1entry
	   F2tau <- sim2$F1tau-sim2$F1entry
	   F2tau <- F2tau*(1-sim1$F1tau) 
	   F2entry <- sim2$F1entry*(1-sim1$F1tau) 
           St <- (1-sim1$F1entry-F2entry)
     } else { 
	   F1tau <- sim1$F1tau; 
	   F2tau <- sim2$F1tau; 
           St <- (1-sim1$F1entry-sim2$F1entry)
     }
           F1tau <- F1tau/St 
           F2tau <- F2tau/St
  } else {
     F1tau <- sim1$F1tau
     if (restrict) F2tau <- sim2$F1tau * (1-F1tau) else F2tau <- sim2$F1tau
  }

  ptot <- F1tau+F2tau
  if (!is.null(pU)) {
      rt <- 1*(pU< pmin(ptot,1))
      rb <- 1*(pU< F1tau) 
  } else {
      rt <- rbinom(n,1,pmin(ptot,1))
      rb <- rbinom(n,1,F1tau/ptot)
  }
  cause=ifelse(rb==1,1,2)
  time=ifelse(cause==causes[1],sim1$timing,sim2$timing)
  cause <- rt*cause
  time[cause==0] <- tau

  if (is.null(entry)) entry <- 0
  ptt <- data.frame(entry=entry,time=time,status=cause,cause=cause,ptot=ptot)

### adds censoring 
 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }


##### add correct names to entry,time,status
if (inherits(cifs[[1]],"phreg"))  {
varsY <- all.vars(update(drop.specials(cifs[[1]]$formula,"cluster"),.~1)) 
if (length(varsY)==2) ptt[,varsY] <- cbind(ptt$time,ptt$status)
if (length(varsY)==3) ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status)
if (!is.null(entry) & length(varsY)==2) varsY <- c("entry",varsY)
###varsY <- c(varsY,"F1tau","F1entry","timing")
ptt <- dkeep(ptt,varsY)
}
if (!is.null(data)) ptt <- cbind(ptt,out$data)

return(ptt)
}# }}}

sim_subdist <- function(cumhazard,rr,n=NULL,entry=NULL,Sentry=NULL,type="cloglog",startcum=c(0,0),U=NULL,...)
{# {{{
  ## Fine-Gray model cloglog F1= 1-exp(-cum(t)*rr)
  ## logistic                F1= cum(t)*rr/(1+cum(t)*rr)
  ## rr                      F1= cum(t)*rr,  rr=exp(X^t beta) 
  if (!is.null(n)) rr <- rep(1,n)

  logit <- function(p) log(p/(1-p))

  if (cumhazard[1,2]>0)  cumhazard <- rbind(startcum,cumhazard)
  breaks <- cumhazard[,1]
  cumh <- cumhazard[,2] 
  mm <- tail(breaks,1)
  n <- length(rr)
  if (!is.null(entry)) {
       cumhentry <- lin_approx(entry,cumhazard,x=1)
  } else cumhentry <- 0
  F1entry <- 0

  if (is.null(U)) U <- runif(n)

  if (type=="cloglog") {
      if (!is.null(entry)) F1entry <- 1-exp(-cumhentry*rr) 
      F1tau <- 1-exp(-tail(cumh,1)*rr)-F1entry
      ttt <- -log(1-U*F1tau-F1entry)/rr
  } else if (type=="logistic") {
     if (!is.null(entry)) F1entry <- cumhentry*rr/(1+cumhentry*rr) 
     F1tau <- tail(cumh,1)*rr/(1+tail(cumh,1)*rr)-F1entry
     v <- U*F1tau+F1entry 
     ttt <- exp(logit(v))/rr; 
  }  else if (type=="rr" | type=="cif") {
     if (!is.null(entry)) F1entry <- cumhentry
     F1tau <- tail(cumh,1) - F1entry
     ttt <- U*F1tau-F1entry
     ## rr only affects binomial draw 
     F1tau <- (F1tau)*rr 
     F1entry <- (F1entry)*rr 
  } else stop(" cloglog or logistic or give function (fun=) \n"); 
  ###
   rrx <- lin_approx(ttt,cumhazard,x=-1)
  if (is.null(entry)) entry <- 0
  timecause <- rrx
  ###
  rrx <- ifelse(timecause>mm,mm,rrx)

   if (!is.null(entry))  {
      if (is.null(Sentry)) Sentry <- 1-F1entry 
   } else Sentry <- 1

   ## drawing event type using conditional distribution
   status <- rbinom(n,1,F1tau/Sentry) 
   rrx[status==0] <- mm
   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr,
		    F1tau=F1tau+F1entry,F1entry=F1entry,
		    timing=timecause)
   attr(dt,"cumhaz") <- cumhazard
   return(dt)
}# }}}

calcCIF <- function(cumhaz,tau,rr,entry=NULL,type=c("cloglog","logistic","rr")) { ## {{{

  if (!is.null(entry)) {
       cumhentry <- lin_approx(entry,cumhaz,x=1)
  } else F1entry <- NULL

  if (type[1]=="cloglog") {
      if (!is.null(entry)) F1entry <- 1-exp(-cumhentry*rr) 
      F1tau <- 1-exp(-tail(cumh,1)*rr) 
  } else if (type[1]=="logistic") {
     if (!is.null(entry)) F1entry <- cumhentry*rr/(1+cumhentry*rr) 
     F1tau <- tail(cumh,1)*rr/(1+tail(cumh,1)*rr) 
  }  else if (type[1]=="rr" | type=="cif") {
     if (!is.null(entry)) F1entry <- cumhentry
     F1tau <- tail(cumh,1)
  } else stop(" cloglog or logistic or give function (fun=) \n"); 

  return(list(F1tau=F1tau,rr=rr,F1entry=F1entry,entry=entry))
} ## }}} 

#' @export
invsubdist <- function(F1,u,entry=NULL,cond=1,ptrunc=NULL)
{# {{{
  ### computes inverse subdist F1^{-1}(u) 
  ### computes inverse subdist F1^{-1}(u)
  F1max <- tail(F1[,2],1)
  mmax <- tail(F1[,1],1)

  if (!is.null(entry))  F1entry <- subdist(F1,entry)[,2];  
  if (!is.null(entry)) if (is.null(ptrunc)) ptrunc <- 1- F1entry
  if (cond==0 & !is.null(entry))  u<-u*ptrunc+F1entry
  if (cond==1 & !is.null(entry))  u<-u*(F1max-F1entry)+F1entry 
  if (cond==1 & is.null(entry))   u<-u*F1max
  rrx  <- lin_approx(u,F1,x=-1)
  status <- rep(1,length(u))
  status[u>=F1max] <- 0
  dt <- data.frame(time=rrx,status=status,y=u)
  if (!is.null(entry)) dt <- cbind(dt,entry)
  attr(dt,"F1") <- F1
  attr(dt,"Fmax") <- F1max
  return(dt)
}# }}}

#' @export
subdist <- function(F1,times)
{# {{{
  rrx <-   lin_approx(times,F1,x=1)
  dt <- cbind(times,rrx)
  colnames(dt) <- c("time","subdist")
  return(dt)
}# }}}

##' Simulation of Output from Cumulative Incidence Regression Model
##'
##' Simulates data that looks like fit from a fitted cumulative incidence model 
##' (Fine-Gray or logistic).
##'
##' @param cif Output from \code{prop.odds.subdist} or \code{ccr} (\code{cmprsk}), 
##'   or call \code{invsubdist} with cumulative and linear predictor.
##' @param n Number of simulations.
##' @param data Data frame to extract covariates.
##' @param Z Design matrix instead of data.
##' @param rr Relative risks.
##' @param strata Strata vector.
##' @param drawZ Logical; randomly sample from Z.
##' @param cens Censoring specification.
##' @param rrc Relative risks for censoring.
##' @param entry Delayed entry time.
##' @param Sentry Survival related to delayed entry.
##' @param cumstart Start cumulatives at time 0.
##' @param U Uniforms for drawing timing.
##' @param pU Uniforms for drawing event type.
##' @param type Model type: \code{"logistic"}, \code{"cloglog"}, or \code{"rr"}.
##' @param extend Extend piecewise constant with constant rate.
##' @param ... Arguments for \code{sim_subdist}.
##' @return Data frame with simulated event times, status, and covariates.
##' @author Thomas Scheike
##' @keywords survival simulation
##' @examples
#' data(bmt)
#' nsim <- 100
#' 
#' ## logit cumulative incidence regression model 
#' cif <- cifreg(Event(time,cause)~platelet+age,data=bmt,cause=1)
#' estimate(cif)  
#' plot(cif,col=1)
#' simbmt <- sim_cif(cif,nsim,data=bmt)
#' dtable(simbmt,~cause)
#' scif <- cifreg(Event(time,cause)~platelet+age,data=simbmt,cause=1)
#' estimate(scif)
#' plot(scif,add=TRUE,col=2)
#' 
#' ## Fine-Gray cloglog cumulative incidence regression model 
#' cif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
#' estimate(cif)  
#' plot(cif,col=1)
#' simbmt <- sim_cif(cif,nsim,data=bmt)
#' scif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=simbmt,cause=1)
#' estimate(scif)
#' plot(scif,add=TRUE,col=2)
#' 
#' ################################################################
#' #  simulating several causes with specific cumulatives 
#' ################################################################
#' cif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
#' cif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=bmt,cause=2)
#' cifss <-  list(cif1,cif2)
#' simbmt <- sim_cifs(list(cif1,cif2),nsim,data=bmt,extend=0.005)
#' scif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=simbmt,cause=1)
#' scif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=simbmt,cause=2)
#' cbind(cif1$coef,scif1$coef)   
#' ## can be off due to restriction F1+F2<= 1    
#' cbind(cif2$coef,scif2$coef)   
#'     
#' par(mfrow=c(1,2))   
#' ## Cause 1 follows the model    
#' plot(cif1); plot(scif1,add=TRUE,col=1:2,lwd=2)
#' # Cause 2:second cause is modified with restriction to satisfy F1+F2<= 1, so scaled down     
#' plot(cif2); plot(scif2,add=TRUE,col=1:2,lwd=2)
#'
#' @seealso [simul_cifs()]
#' @aliases sim_cif sim_cifs subdist sim_subdist invsubdist
#' @export sim_cif
sim_cif <- function(cif,n,data=NULL,Z=NULL,rr=NULL,strata=NULL,
                    drawZ=TRUE,cens=NULL,rrc=NULL,entry=NULL,Sentry=NULL,
		    cumstart=c(0,0),U=NULL,pU=NULL,type=NULL,extend=NULL,...)
{# {{{
## also extracts coefficients and baseline from cifreg
if (!is.null(data))  {
   des <- 	draw_phreg(cif,n,data=data,Z=Z,drawZ=drawZ,...)
   dats <- des$data
   dats$orig.id <- des$id

   if (is.null(strata))  strata <- des$strata 
   if (is.null(rr)) rr <- des$rr  
   znames <- names(dats); 
    n <- nrow(dats)
} else {
	if (is.null(Z) & is.null(rr)) stop("must give Z or rr ")
	if (!is.null(Z)) {
	  if (is.data.frame(Z)) znames <- names(Z) else znames <- colnames(Z)
	} else znames <- NULL
	if (is.null(rr)) rr <- exp(as.matrix(Z) %*% cif$coef)
	n <- length(rr)
	orig.id <- 1:n
	dats <- NULL
}
if (is.null(strata)) strata <- rep(0,n)

if (inherits(cif,c("phreg","cifreg"))) cumhaz <- basecumhaz(cif,only=1,extend=extend)
else {
if (!is.list(cif)) stop("must be cifreg or list of hazards\n") else cumhaz <- cif
}
if (!is.null(extend))   cumhaz <- extendCums(cumhaz,NULL,extend=extend)

if (is.null(type)) {
if (is.null(cif$propodds)) type <- "cloglog" else type <- "logistic" 
}
ids <- 1:n

lentry <- Slentry <- NULL

## {{{ simulation of cif 
    ptt <- c()
    for (i in unique(strata)) {
      whichi <- which(strata==i)
      cumhazj <- rbind(0,cumhaz[[i+1]])
      if (!is.null(entry)) lentry <- entry[whichi]
      if (!is.null(Sentry)) Slentry <- Sentry[whichi]
      if (!is.null(U)) Ui <- U[whichi] else Ui <- NULL
      simj <- sim_subdist(cumhazj,rr[whichi],type=type,U=Ui,entry=lentry,Sentry=Slentry)
      simj$id <- ids[whichi]
      ptt  <-  rbind(ptt,simj)
    }
    dsort(ptt) <- ~id
### }# }}}

 ### adds censoring 
 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

##### add correct names to entry,time,status if "phreg/cifreg/recreg" object
if (inherits(cif,"phreg"))  {
varsY <- all.vars(update(drop.specials(cif$formula,"cluster"),.~1)) 
if (length(varsY)==2) ptt[,varsY] <- cbind(ptt$time,ptt$status)
if (length(varsY)==3) ptt[,varsY] <- cbind(ptt$entry,ptt$time,ptt$status)
if (!is.null(entry) & length(varsY)==2) varsY <- c("entry",varsY)
varsY <- c(varsY,"F1tau","F1entry","timing")
ptt <- dkeep(ptt,varsY)
} 
if (!is.null(dats)) ptt <- cbind(ptt,dats)

attr(ptt,"type") <- type
attr(ptt,"znames") <- znames
attr(ptt,"cumhaz") <-  cumhaz
return(ptt)
}# }}}


##' @export
simul_cifs <- function(n,rho1,rho2,beta,rc=0.5,depcens=0,rcZ=0.5,
               bin=1,type=c("cloglog","logistic"),rate=1,entry=NULL,
               Z=NULL,U=NULL,pU=NULL,...) {# {{{
    p=length(beta)/2
    tt <- seq(0,6,by=0.1)
    if (length(rate)==1) rate <- rep(rate,2)
    Lam1 <- rho1*(1-exp(-tt/rate[1]))
    Lam2 <- rho2*(1-exp(-tt/rate[2]))

    if (length(bin)==1) bin <- rep(bin,2)
    if (length(rcZ)==1) rcZ <- c(rcZ,0)

    if (is.null(Z)) 
    Z=cbind((bin[1]==1)*(2*rbinom(n,1,1/2)-1)+(bin[1]==0)*rnorm(n),(bin[2]==1)*(rbinom(n,1,1/2))+(bin[2]==0)*rnorm(n))
    p <- ncol(Z)
    colnames(Z) <- paste("Z",1:p,sep="")

    cif1 <- setup_cif(cbind(tt,Lam1),beta[1:p],Znames=colnames(Z),type=type[1])
    cif2 <- setup_cif(cbind(tt,Lam2),beta[(p+1):(2*p)],Znames=colnames(Z),type=type[1])

    data <- sim_cifs(list(cif1,cif2),n,Z=Z,U=U,pU=pU,type=type[1],extend=NULL,entry=entry,...)

    if (is.null(entry)) entry <- 0
    if (!is.null(rc)) {
    if (depcens==0) censor=pmin(entry+rexp(n,1)*(1/rc),6) else censor=pmin(entry+rexp(n,1)*(1/(rc*exp(Z %*% rcZ))),6)
    } else censor <- 6 

    data$status=data$status*(data$time<=censor)
    data$time=pmin(data$time,censor)
    data <- cbind(data,Z)
    attr(data,"Lam1") <- cbind(tt,Lam1)
    attr(data,"Lam2") <- cbind(tt,Lam2)
    return(data)
}# }}}

simul_cifsRA <- function (n, rho1, rho2, beta, rc = 0.5, depcens.R = 0, rcZ = 0.5,pCA=0.5,pCR=0.5,
    bin = 1, type = c("cloglog", "logistic"), rate = 1, Z = NULL,rc2=0.3,depcens.Adm=0)
{# {{{
    p = length(beta)/2
    tt <- seq(0, 6, by = 0.1)
    if (length(rate) == 1)
        rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    if (length(bin) == 1)
        bin <- rep(bin, 2)
    if (length(rcZ) == 1)
        rcZ <- c(rcZ, 0)
    if (is.null(Z))
        Z = cbind((bin[1] == 1) * (2 * rbinom(n, 1, 1/2) - 1) +
            (bin[1] == 0) * rnorm(n), (bin[2] == 1) * (rbinom(n,
            1, 1/2)) + (bin[2] == 0) * rnorm(n))
    colnames(Z) <- paste("Z", 1:2, sep = "")
    p <- ncol(Z)
    cif1 <- setup_cif(cbind(tt, Lam1), beta[1:p], Znames = colnames(Z),
        type = type[1])
    cif2 <- setup_cif(cbind(tt, Lam2), beta[(p + 1):(2 * p)],
        Znames = colnames(Z), type = type[1])
    data <- sim_cifs(list(cif1, cif2), n, Z = Z,extend=NULL)

    admcens <- rbinom(n,1,pCA)
    if (depcens.Adm==1) rrA <- exp( Z %*% rcZ) else rrA <- 0
    censorA = admcens*(rrA+runif(n)*(6-rrA))+ 6*(admcens==0)

    statusA = data$status * (data$time <= censorA)
    timeA = pmin(data$time, censorA)
    statusA[statusA==0] <- 7

    Rcens <- rbinom(n,1,pCR)
    if (depcens.R == 0)  
        censorR = Rcens*runif(n)*6+(Rcens==0)*6
    else censorR = Rcens*pmin(rexp(n, 1) * (1/(rc * exp(Z %*% rcZ))),6) + (Rcens==0)*6
    censor <- ifelse( censorR < censorA, censorR,censorA)

    status = data$status * (data$time <= censor)
    time = pmin(data$time, censor)
    status[status==0 & (censorR > censorA)] <- 7
    ## extra censoring
    data <- data.frame(time=time,status=status,cens.time=censor,censorA=censorA,censorR=censorR,
		       timeA=timeA,statusA=statusA)
    return(cbind(data, Z))
}# }}}


setup_cif  <- function(cumhazard,coef,Znames=NULL,type="logistic")
{# {{{
    cif <- list()
    cif$cumhaz <- cumhazard
    cif$se.cumhaz <- cumhazard
    cif$coef <- coef
    cif$model <- type
    if (type=="logistic") cif$propodds <- 1 else cif$propodds <- NULL
    cif$strata <- rep(0,nrow(cumhazard))
    cif$jumps <- 1:nrow(cumhazard)

    cif$nstrata <- 1
    class(cif) <- "defined"
    attr(cif,"znames") <- Znames
    return(cif)
}# }}}


## }}}

##' @export
kumarsim <- function (n,rho1=0.71,rho2=0.40,rate = c(6.11,24.2),
	      beta=c(-0.67,0.59,-0.55,0.25,0.68,0.18,0.45,0.31),
	      labels= c("gp","dnr","preauto","ttt24(24,300]"),
	      depcens=0,type = c("logistic", "cloglog"),restrict=TRUE,...)
{# {{{
    p = length(beta)/2
    tt <- seq(0, 150, by = 1)
    if (length(rate) == 1)
    rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    
    ## fully saturated model for kumar covariates 
    Zdist <- c(0.21064815,0.02083333,0.05555556,0.01504630,
	       0.13888889,0.15393519, 0.02662037, 0.04398148, 
	       0.04745370, 0.02430556, 0.02199074, 0.03935185,
               0.02430556, 0.05555556, 0.01273148, 0.10879630)
    Zs <- expand.grid(gp=c(0,1),dnr=c(0,1),preauto=c(0,1),ttt24=c(0,1))
    Zs <- dsort(Zs,~ttt24+gp+dnr+preauto)

    samn <- sample(1:16,n,replace=TRUE,prob=c(Zdist))
    Z <- Zs[samn,]

    colnames(Z) <- labels
    cif1 <- setup_cif(cbind(tt, Lam1), beta[1:4], Znames = colnames(Z), type = type[1])
    cif2 <- setup_cif(cbind(tt, Lam2), beta[5:8], Znames = colnames(Z), type = type[1])
    data <- sim_cifs(list(cif2, cif1), n, Z = Z,extend=NULL,restrict=restrict,...)
    stat12 <- which(data$status %in% c(1,2))
    data$status[stat12] <- c(2,1)[data$status[stat12]]

    ## kumar censoring, cox model 
    c0 <- list()     
    c0$cumhaz <- cbind(c(0,20,60,90,160),
		       c(0, 0.07797931, 0.28512764, 0.76116180, 1.95720759))
    c0$coef <- c(1.8503113,-0.6976226,0.5828763,-0.2000003)

    if (depcens == 0)
        censor  <- rchaz(c0$cumhaz,n=n)
    else {
        rrc <- exp(as.matrix(Z) %*% c0$coef)
        censor  <- rchaz(c0$cumhaz,rrc)
    }
    status = data$status * (data$time <= censor$time)
    time = pmin(data$time, censor$time)
    data <- data.frame(time = time, status = status)
    return(cbind(data, Z))
}# }}}

##' @export
kumarsimRCT <- function (n,rho1=0.71,rho2=0.40,rate = c(6.11,24.2),
	      beta=c(-0.67,0.59,-0.55,0.25,0.68,0.18,0.45,0.31),
	      labels= c("gp","dnr","preauto","ttt24(24,300]"),
	      nocens=0,addcens=1,rct=1,
	      type = c("logistic", "cloglog"),restrict=TRUE,
	      censpar=c(1,1,1,1), F1par=c(1,1,1,1), F2par=c(1,1,1,1),
	      treatmodel=c(-0.18,-0.16,0.06,0.24),... )
{# {{{
    p = length(beta)/2
    tt <- seq(0, 150, by = 1)
    if (length(rate) == 1)
    rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    
    ## fully saturated model for kumar covariates 
    Zdist <- c(0.21064815,0.02083333,0.05555556,0.01504630,
	       0.13888889,0.15393519, 0.02662037, 0.04398148, 
	       0.04745370, 0.02430556, 0.02199074, 0.03935185,
               0.02430556, 0.05555556, 0.01273148, 0.10879630)
    Zs <- expand.grid(gp=c(0,1),dnr=c(0,1),preauto=c(0,1),ttt24=c(0,1))
    Zs <- dsort(Zs,~ttt24+gp+dnr+preauto)

    samn <- sample(1:16,n,replace=TRUE,prob=c(Zdist))
    Z <- Zs[samn,]
    ## randomized gp instead
    if (rct==1) Z[,1] <- rbinom(n,1,0.5)
    ## randomized gp given other covariates 
    if (rct==2) {
	    lp <- as.matrix(cbind(1,Z[,-1])) %*% treatmodel 
	    Z[,1] <- rbinom(n,1, expit(lp))
    }
    colnames(Z) <- labels
    cif1 <- setup_cif(cbind(tt, Lam1), F1par*beta[1:4], Znames = colnames(Z), type = type[1])
    cif2 <- setup_cif(cbind(tt, Lam2), F2par*beta[5:8], Znames = colnames(Z), type = type[1])
    data <- sim_cifs(list(cif2, cif1), n, Z = Z,extend=NULL,restrict=restrict,...)
    stat12 <- which(data$status %in% c(1,2))
    data$status[stat12] <- c(2,1)[data$status[stat12]]

   if (nocens==0) {
    ## kumar censoring, cox model 
    c0 <- list()     
    c0$cumhaz <- cbind(c(0,20,60,90,160),
		       c(0, 0.07797931, 0.28512764, 0.76116180, 1.95720759))
    c0$coef <-censpar* c(1.8503113,-0.6976226,0.5828763,-0.2000003)

	if (addcens)  {
	  ## draw from cox gp model  
         rrc <- exp(as.matrix(Z) %*% c(c(1,0,0,0)*c0$coef))
         censorgp  <- rchaz(c0$cumhaz,rrc)
	 ## draw from cox other covariates   
         rrc <- exp(as.matrix(Z) %*% c(c(0,1,1,1) * c0$coef))
         censoroc  <- rchaz(c0$cumhaz,rrc)
	 ## miniumum of the two censoring times 
	 censor <- censorgp
	 censor$time <- pmin(censoroc$time,censorgp$time)
	 censor$status <- pmax(censoroc$status,censorgp$status)
	} else {
        rrc <- exp(as.matrix(Z) %*% c(censpar*c0$coef))
        censor  <- rchaz(c0$cumhaz,rrc)
	}
    status = data$status * (data$time <= censor$time)
    time = pmin(data$time, censor$time)
    data <- data.frame(time = time, status = status)
    }
    return(cbind(data, Z))
}# }}}

######################## ##################### #####################
## illness death competing risks with two causes of death
## First simple model, then with covariates addedd 
## effect of transition into 2 being early for cause 3, and 4
######################## ##################### #####################

##' Illness-Death Competing Risks with Two Causes of Death
##'
##' Simulates data from an illness-death model with two causes of death from the 
##' illness state. Covariate effects can be introduced via relative risk terms.
##'
##' @param cumhaz Cumulative hazard from state 1 to 2.
##' @param death.cumhaz Cumulative hazard of death from state 1.
##' @param death.cumhaz2 Cumulative hazard of death from state 2.
##' @param n Number of simulations.
##' @param rr Relative risks.
##' @param rd Relative risks for death from state 1.
##' @param rd2 Relative risks for death from state 2.
##' @param gamma23 Early effect parameters for death causes.
##' @param gamma24 Early effect parameters for death causes.
##' @param early2 Time threshold for early effect.
##' @param gap.time Gap time indicator.
##' @param max.recurrent Maximum recurrent events.
##' @param cens Censoring specification.
##' @param rrc Censoring relative risks.
##' @param extend Extend hazards.
##' @param ... Additional arguments.
##' @return Data frame with multi-state event history.
##' @author Thomas Scheike
##' @keywords survival simulation
##' @export
sim_multistateII <- function(cumhaz,death.cumhaz,death.cumhaz2,n=NULL,
		    rr=NULL,rd=NULL,rd2=NULL,gamma23=0,gamma24=0,early2=10000,
		    gap.time=FALSE,max.recurrent=100,cens=NULL,rrc=NULL,extend=TRUE,...)
{# {{{
  status <- NULL

  ## cumhaz is cumulative out of state 1 to state 2 
  if (!is.null(n)) rr <- matrix(1,n,1) 
  n <- nrow(rr); 
  ## covariate adjustment 
  if (is.null(rd))   rd  <- matrix(1,n,length(death.cumhaz))
  if (is.null(rd2))  rd2  <- matrix(1,n,length(death.cumhaz2))

  ####
  if (!is.null(cens)) cens <- list(cens)
  cumss  <-  c(list(cumhaz),death.cumhaz,death.cumhaz2,cens)
  ### extend of cumulatives
  for (i in seq_along(cumss))  cumss[[i]] <- rbind(0,cumss[[i]])
  ## extend range of all cumulatives to max range, by linear extension
  cumss <- extendCums(cumss,NULL,extend=extend)
  maxtime <- tail(cumss[[1]],1)[1]

 if (!is.null(cens)) { 
	  cens <- cumss[[1+length(death.cumhaz)+length(death.cumhaz2)+1]]
	  if (is.null(rrc)) rrc <- rep(1,n); 
	  ctime <- rchaz(cens,rrc)$time
  } else ctime <- rep(maxtime,n)

  ## hazards out of 1 and out of 2
###  chaz1 <- cumss[1:3]; 
###  chaz2 <- cumss[4:5]; 
  idx1 <- 1:(1 + length(death.cumhaz))
  idx2 <- (max(idx1) + 1):(max(idx1) + length(death.cumhaz2))
  chaz1 <- cumss[idx1]; chaz2 <- cumss[idx2]
  rr1 <- cbind(rr,rd)
  rr2 <- cbind(rd2)

  ## time out of state 1 
  tall <- rchazl(chaz1,rr1,causes=c(2,3,4))
  tall$id <- 1:n
  tall$status[tall$time>ctime] <- 0; 
  tall$time[tall$time>ctime] <- ctime[tall$time>ctime] 
  tall$from <- 1
  tall$to <- tall$status

  ## simulating out of 2 
  tall2 <- subset(tall,status==2)
  rr23t <- exp(gamma23*(tall2$time<early2))
  rr24t <- exp(gamma24*(tall2$time<early2))
  sim2 <- rchazl(chaz2,rr2[tall2$id,]*cbind(rr23t,rr24t),causes=c(3,4),entry=tall2$time)
  sim2$id <- tall2$id
  ctime2 <- ctime[tall2$id]
  sim2$status[sim2$time>ctime2] <- 0; 
  sim2$time[sim2$time>ctime2] <- ctime2[sim2$time>ctime2] 
  sim2$from <- 2
  sim2$to <- sim2$status
  tall <- rbind(tall,sim2)

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  return(tall)
  }# }}}

##' Simulation of Illness-Death Model
##'
##' Simulates data from a full illness-death model with reversible transitions 
##' and multiple causes of death. Supports various dependence structures via 
##' shared frailties.
##'
##' @param n Number of IDs.
##' @param cumhaz Cumulative hazard from state 1 to 2.
##' @param cumhaz2 Cumulative hazard from state 2 to 1.
##' @param death.cumhaz Cumulative hazard of death from state 1.
##' @param death.cumhaz2 Cumulative hazard of death from state 2.
##' @param rr12 Relative risk for 1->2.
##' @param rr21 Relative risk for 2->1.
##' @param rd13 Relative risk for death 1->3.
##' @param rd23 Relative risk for death 2->3.
##' @param rrc Relative risk for censoring.
##' @param gap.time Gap time indicator. If true simulates gap-times with specified cumulative hazard.
##' @param max.recurrent Maximum recurrent events.
##' @param dependence Dependence structure (0-3).
##' @param var.z Variance of random effects.
##' @param cor.mat Correlation matrix.
##' @param cens Censoring rate.
##' @param extend Extend hazards.
##' @param ... Additional arguments.
##' @return Data frame with multi-state event history.
##' @author Thomas Scheike
##' @keywords survival simulation
##' @examples
##' ########################################
##' ## getting some rates to mimick 
##' ########################################
##' data(CPH_HPN_CRBSI)
##' dr <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi 
##' base4 <- CPH_HPN_CRBSI$mechanical
##' dr2 <- scalecumhaz(dr,1.5)
##' cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
##'
##' iddata <- sim_multistate(100,base1,base1,dr,dr2,cens=cens)
##' dlist(iddata,.~id|id<3,n=0)
##'  
##' ### estimating rates from simulated data  
##' c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
##' c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
##' c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
##' c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
##' ###
##' par(mfrow=c(2,3))
##' plot(c0)
##' lines(cens,col=2) 
##' plot(c3,main="rates 1-> 3 , 2->3")
##' lines(dr,col=1,lwd=2)
##' lines(dr2,col=2,lwd=2)
##' ###
##' plot(c1,main="rate 1->2")
##' lines(base1,lwd=2)
##' ###
##' plot(c2,main="rate 2->1")
##' lines(base1,lwd=2)
##'  
##' @export
sim_multistate <- function(n,cumhaz,cumhaz2,death.cumhaz,death.cumhaz2,
		    rr12=NULL,rr21=NULL,rd13=NULL,rd23=NULL,rrc=NULL,
		    gap.time=FALSE,max.recurrent=100,
		    dependence=0,var.z=0.5,cor.mat=NULL,cens=NULL,extend=TRUE,...) 
{# {{{

  fdeath <- dtime <- NULL # to avoid R-check 
  status <- dhaz <- NULL; dhaz2 <- NULL

  if (dependence==0) { z <- z1 <- z2 <- zd  <- zd2 <-  rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
###	      z <- exp(rnorm(n,1)*var.z[1]^.5)
	      zd2 <- z; z1 <- z; z2 <- z; zd <- z
	      if (!is.null(cor.mat)) { zd <- rep(1,n); }
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; zd2 <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(4*n,1),n,4)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3]+z[,4]^cor.mat[1,4])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3]+z[,4]^cor.mat[2,4])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3]+z[,4]^cor.mat[3,4])
              zd2<- (z[,4]^cor.mat[4,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3]+z[,4]^cor.mat[4,4])
	      z <- cbind(z1,z2,zd,zd2)
      } else stop("dependence 0-3"); # }}}

  ## covariate adjustment 
  if (is.null(rr12))  rr12 <- z1  else rr12 <- rr12*z1
  if (is.null(rr21))  rr21 <- z2  else rr21 <- rr21*z2
  if (is.null(rd13))  rd13 <- zd  else rd13 <- rd13*zd
  if (is.null(rd23))  rd23 <- zd2 else rd23 <- rd23*zd2

  ll <- nrow(cumhaz)
  ### extend of cumulatives
  cumhaz <- rbind(c(0,0),cumhaz)
  cumhaz2 <- rbind(c(0,0),cumhaz2)
  death.cumhaz <- rbind(c(0,0),death.cumhaz)
  death.cumhaz2 <- rbind(c(0,0),death.cumhaz2)

  haz <- haz2 <- NULL
  ## range max of cumhaz and cumhaz2 

  if (!is.null(cens)) {
      if (is.matrix(cens))  {
      out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz,death.cumhaz2,cens),NULL,extend=extend)
      cens <- out$cum5
      }
  } else {
     out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz,death.cumhaz2),NULL,extend=extend)
  }
  cumhaz <- out$cum1
  cumhaz2 <- out$cum2
  cumhazd <- out$cum3
  cumhazd2 <- out$cum4
  max.time <- tail(cumhaz[,1],1)

  tall <- rcrisk(cumhaz,cumhazd,rr12,rd13,cens=cens,extend=NULL)

  tall$id <- 1:n
  ### fixing the first time to event
  tall$death <- 0
  ### cause 2 is death state 3, cause 1 is state 2
  tall <- dtransform(tall,status=3,status==2)
  tall <- dtransform(tall,death=1,status==3)
  tall <- dtransform(tall,status=2,status==1)
  ### dead or censored
  deadid <- (tall$status==3 | tall$status==0)
  tall$from <- 1
  tall$to <- tall$status
  ## id's that are dead: tall[deadid,]
  ## go furhter with those that are not yet dead  or censored
  tt <- tall[!deadid,,drop=FALSE]
  ## also check that we are before max.time
  tt <- subset(tt,tt$time<max.time)

  i <- 1; 
  while ( (nrow(tt)>0) & (i < max.recurrent)) {# {{{
	  i <- i+1
	  nn <- nrow(tt)

	  z1r <- rr12[tt$id]
	  zdr <- rd13[tt$id]
	  z2r <- rr21[tt$id]
	  zd2r <- rd23[tt$id]

	  if (i%%2==0) { ## in state 2
	  ## out of 2 for those in 2
          tt1 <- rcrisk(cumhaz2,cumhazd2,z2r,zd2r,entry=tt$time,cens=cens,extend=NULL)
          tt1$death <- 0
	  ### status 2 is death state 3, status 1 is state 1
	  tt1 <- dtransform(tt1,status=3,status==2)
	  tt1 <- dtransform(tt1,death=1,status==3)
	  tt1$from <- 2
	  tt1$to <- tt1$status
	  ## take id from tt
	  tt1$id <-  tt$id
	  ### add to data 
	  tall <- rbind(tall,tt1,row.names=NULL)

          deadid <- (tt1$status==3 | tt1$status==0)
	  ### those that are still under risk 
	  tt <- tt1[!deadid,,drop=FALSE]
	  ## also keep only those before max.time
          tt <- subset(tt,tt$time<max.time)
		  
	  } else { ## in state 1
	  ## out of 1 for those in 1
          tt1 <- rcrisk(cumhaz,cumhazd,z1r,zdr,entry=tt$time,cens=cens,extend=NULL)

          tt1$death <- 0
	  ### status 2 is death state 3, status 1 is state 2
	  tt1 <- dtransform(tt1,status=3,status==2)
	  tt1 <- dtransform(tt1,death=1,status==3)
	  tt1 <- dtransform(tt1,status=2,status==1)
	  tt1$from <- 1
	  tt1$to <- tt1$status
	  tt1$id <-  tt$id

	  ### add to data 
	  tall <- rbind(tall,tt1,row.names=NULL)

	  ## take id from tt
          deadid <- (tt1$status==3 | tt1$status==0)
	  ### those that are still under risk 
	  tt <- tt1[!deadid,,drop=FALSE]
	  ### also only keep those before max.time
          tt <- subset(tt,tt$time<max.time)
	  }

  }  # }}}

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"death.cumhaz2") <- cumhazd2
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"cens.cumhaz") <- cens
  attr(tall,"z") <- z

  return(tall)
  }# }}}

.onAttach <- function(libname, pkgname) {
    # Suppress the warning about overwriting methods
    suppressWarnings({
        registerS3method("[", "Event", `[.Event`,
            envir = asNamespace(pkgname)
        )
        registerS3method("as.character", "Event", as.character.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("as.matrix", "Event", as.matrix.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("format", "Event", format.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("print", "Event", print.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("rbind", "Event", rbind.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("as.data.frame", "Event", as.data.frame.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("is.na", "Event", is.na.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("length", "Event", length.Event,
            envir = asNamespace(pkgname)
        )
        registerS3method("c", "Event", c.Event,
            envir = asNamespace(pkgname)
        )
    })
}


.onLoad <- function(libname, pkgname) {

}

## n <- 1e4; ACE <- c(1,1,1)/3
## logscale <- -4.5; logshape <- .7
## p2 <- .065
## a2 <- -10; b2 <- 0.15 
## a1 <- 85; b1 <- 0.1

## pmvn(upper=c(q2,q2),sigma=diag(2)*(1-2/3)+2/3)
## pmvn(c(q2,q2),sigma=diag(2)*(1-2/3)+2/3)

## pnorm(q2,sd=1) ## Marginal / Perfect dependence
## pmvn(upper=c(q2,q2),sigma=diag(2)*(1-2/3)+2/3) ## Concordance
## pnorm(q2,sd=1)^2 ## Independence
## (lambdaR <- pmvn(upper=c(q2,q2),sigma=diag(2)*(1-2/3)+2/3)/pnorm(q2,sd=1)^2)

bicomprisksim <- function(n=1e4,
                          ACE=c(1/3,1/3,1/3),
                          logscale=-4.5,logshape=.7,
                          a1=85,b1=0.1,
                          a2=-10,b2=0.15,
                          p2=.065,
                          tt,
                          ...) {

    ACE <- ACE/sum(ACE)
    ialpha <- function(v,a,b) -(log(-v)+a)/b    
    q2 <- qnorm(p2) ## p2: Prostata cancer prevalence 
    p1 <- 1-p2; q1 <- qnorm(p1) ## Death without cancer
    alpha <- function(t,a,b) -exp(-(b*t+a))
    F2s <- function(t) pnorm(alpha(t,b=b2,a=a2)+q2) ## Marginal Cumulative Incidence (cancer)
### Random effects
    R <- diag(2)*.5+.5
    J <- matrix(1,ncol=2,nrow=2)
    I <- diag(2)
    zyg <- rep(c(0,1),each=n)
    A <- rbind(rmvn(n,sigma=R*ACE[1]),rmvn(n,sigma=J*ACE[1]))
    C <- rmvn(2*n,sigma=J*ACE[2])
### Random effects 'death'
    eta1 <- C
### Random effects 'cancer'
    eta2 <- A+C;
### Subject-specific probability of lifetime cancer 
    probcanc <- pnorm(q2+eta2,sd=ACE[3]^.5)   
### Cancer/Death without cancer realizations
    cancertrue <- (runif(length(probcanc))<probcanc)*1+1
### Event times given failure and random effect
    ## inverse P(T<t| eta,cause=1)
    iF1 <- function(u,eta,sd,a,b) (qnorm(u,sd=sd)-eta)/b+a 
    iF2 <- function(u,eta,pr,sd,q,a,b) ialpha(qnorm(u*pr,sd=sd)-eta-q,a=a,b=b)
    u <- runif(length(probcanc))
    t2 <- iF2(u,eta2,probcanc,sd=sqrt(1-sum(ACE[1:2])),q=q2,a=a2,b=b2)
    t1 <- iF1(u,eta1,sd=sqrt(1-sum(ACE[2])),a=a1,b=b1)   
    t0 <- t1; t0[which(cancertrue==2)] <- t2[which(cancertrue==2)] 
### Censoring distributon
    cens <- matrix(rep(rweibull(length(probcanc)/2,shape=exp(logshape),scale=1/exp(logscale)),2),
                   ncol=2)
    ##hist(cens,xlab=c(0,400),200)    
    cause <- cancertrue
    cause[t0>cens] <- 0
    t <- pmin(t0,cens)
    (censtab <- table(cause)/length(cause))    
### Data.frame
    d <- data.frame(t,cause,zyg,cancertrue-1,cens[,1],t0[,1],t0[,2]);
    names(d) <- c("time1","time2","cause1","cause2","zyg","cancertrue1","cancertrue2","cens.time","T01","T02")
### Long format
    dd <- fast.reshape(d)
    dd$cancer <- (dd$cause==2)*1

    if (missing(tt)) tt <- seq(0,max(dd$time))    
    Smz <- J*ACE[1]+J*ACE[2]+I*ACE[3]
    Sdz <- R*ACE[1]+J*ACE[2]+I*ACE[3]
    rr <- alpha(tt,b=b2,a=a2)+q2
    Cmz <- pmvn(upper=cbind(rr,rr),mu=matrix(0,ncol=2,nrow=length(rr)),sigma=Smz)
    Cdz <- pmvn(upper=cbind(rr,rr),mu=matrix(0,ncol=2,nrow=length(rr)),sigma=Sdz)
    
    true <- list(p2=p2,
                 p22mz=pmvn(lower=c(-Inf,-Inf),upper=c(q2,q2),sigma=Smz),
                 p12mz=pmvn(lower=c(q2,-Inf),upper=c(Inf,q2),sigma=Smz),
                 p12mz=pmvn(lower=c(q2,-Inf),upper=c(Inf,q2),sigma=Smz),
                 p11mz=pmvn(lower=c(q2,q2),upper=c(Inf,Inf),sigma=Smz),
                 p22dz=pmvn(lower=c(-Inf,-Inf),upper=c(q2,q2),sigma=Sdz),
                 p12dz=pmvn(lower=c(q2,-Inf),upper=c(Inf,q2),sigma=Sdz),
                 p12dz=pmvn(lower=c(q2,-Inf),upper=c(Inf,q2),sigma=Sdz),
                 p11dz=pmvn(lower=c(q2,q2),upper=c(Inf,Inf),sigma=Sdz),
                 time=tt,
                 F2=F2s(tt),
                 Cmz=Cmz,
                 Cdz=Cdz
                 )
    attributes(dd) <- c(attributes(dd),true)                  
    return(dd)
}

##' Extract Event (Jump) Times
##'
##' Extracts unique event times from survival data, optionally restricting
##' to concordant pairs or specific causes.
##'
##' @param time vector of event/censoring times.
##' @param status vector of status indicators (default TRUE = event).
##' @param id optional cluster identifier for concordant pair times.
##' @param cause optional cause value to select.
##' @param sample optional maximum number of time points to return.
##' @param sample.all logical; if TRUE and sampling, include remaining times.
##' @param strata not used.
##' @param num optional within-cluster numbering variable.
##' @param ... additional arguments.
##' @return Sorted vector of event times.
##' @export
jumptimes <- function(time, status=TRUE, 
                      id,cause,
                      sample,sample.all=TRUE,
                      strata=NULL,num=NULL, ...) {
    if (missing(id)) {
        time <- if (missing(cause)) time[status>0]
                else time[status==cause]
    } else {
        ww <- na.omit(fast.reshape(cbind(time=time,status=status),id=id,num=num))
        statusvar <- grep("status",colnames(ww))
        timevar <- grep("time",colnames(ww))
        if (missing(cause)) {
            idx <- which(rowSums(ww[,statusvar]>0)==length(statusvar))
        } else {
            idx <- which(apply(as.matrix(ww[,statusvar]),1,function(x) all(x==cause)))
        }
            time <- na.omit(do.call(pmax,as.list(ww[idx,timevar])))
    }

    if (!missing(sample) && sample<length(time)) {
        time <- sort(unique(time))
        t0 <- seq(min(time),max(time),length.out=min(sample,length(time)))
        ii <- fast.approx(time,t0)
        ii <- unique(ii)
        time0 <- time[ii]
        if (length(ii)<sample && sample.all) {
            remain <- setdiff(seq(length(time)),ii)
            time0 <- c(time0,time[base::sample(remain,sample-length(ii))])
        }
        time <- time0
    } 
    return(sort(time))
}

## with(prt, jumptimes(time,status==2,id=id,sample=10))

##' Stratified Binomial Regression with cross-validation option 
##'
##' Fits a stratified binomial regression model for the cumulative incidence
##' function (CIF), restricted mean survival time (RMST), or restricted mean
##' time lost (RMTL), using inverse probability of censoring weighting (IPCW).
##' Each stratum may either be fit on its own observations or, when
##' \code{cv.fold = TRUE}, on the complement of its observations (i.e. a
##' leave-one-stratum-out / cross-validation style fit). Standard errors are
##' computed via an iid decomposition that accounts for the estimation of the
##' censoring distribution.
##'
##' @param formula A formula with a \code{\link[survival]{Surv}} or
##'   \code{Event} object on the left-hand side, and covariates, optional
##'   \code{cluster()}, \code{strata()}, \code{offset()}, and \code{weights()}
##'   terms on the right-hand side.
##' @param data A \code{data.frame} containing the variables in \code{formula}.
##' @param cause Value(s) of the status variable that identify the event of
##'   interest (cause 1 by default).
##' @param time The fixed time-point at which the CIF, RMST, or RMTL is
##'   evaluated. Required.
##' @param beta Optional starting values for the regression coefficients.
##'   Either a vector (recycled across strata) or a matrix with one row per
##'   stratum.
##' @param type Character; either \code{"II"} (default) or \code{"I"}. When
##'   \code{"II"}, an additional augmentation term and an extra component of
##'   the censoring-related iid decomposition are computed.
##' @param offset Optional offset term; can also be specified via the formula.
##' @param weights Optional case weights; can also be specified via the
##'   formula.
##' @param cens.weights Optional pre-computed inverse probability of censoring
##'   weights. If supplied, the internal censoring model fit is skipped and
##'   \code{se} is forced to \code{FALSE}.
##' @param cens.model A formula for the censoring model (right-hand side only,
##'   fitted via \code{\link{phreg}}). Defaults to the Kaplan-Meier
##'   (\code{~+1}).
##' @param se Logical. If \code{TRUE} (default), standard errors are computed,
##'   including the contribution from estimating the censoring distribution.
##' @param kaplan.meier Logical. If \code{TRUE} (default), uses the
##'   Kaplan-Meier estimator for censoring weights when the censoring model has
##'   no covariates; otherwise a Cox-type estimator is used internally when
##'   covariates are present.
##' @param cens.code The code in the status variable denoting censoring
##'   (default 0).
##' @param no.opt Logical. If \code{TRUE}, no optimization is performed and the
##'   object is evaluated at the supplied \code{beta}.
##' @param method Optimization method. Currently only \code{"nr"} (Newton-Raphson
##'   via \code{lava::NR}) is supported.
##' @param augmentation Optional augmentation term(s) added to the score
##'   equations, either a vector (recycled across strata) or a matrix with one
##'   row per stratum.
##' @param outcome Character; the outcome scale to model: \code{"cif"}
##'   (default), \code{"rmst"}, or \code{"rmtl"}.
##' @param model Character; the link function: \code{"default"} (chooses
##'   \code{"logit"} for CIF and \code{"exp"} for RMST/RMTL), \code{"logit"},
##'   \code{"exp"}, or \code{"lin"}.
##' @param Ydirect Optional, directly supplied response vector, overriding the
##'   response otherwise constructed from \code{outcome}.
##' @param strata Optional numeric vector defining strata, used when strata are
##'   not specified via \code{strata()} in the formula.
##' @param cv.fold Logical. If \code{TRUE}, for each stratum \code{s} the
##'   regression coefficients are estimated using all observations \emph{not}
##'   in stratum \code{s} (a cross-validation-style fit), rather than the
##'   observations within stratum \code{s} (the default, \code{FALSE}).
##' @param low.memory Logical. If \code{TRUE} only saves key quanties, coef, var, iid and some design structures.
##' @param ... Additional arguments, currently passed as optimizer
##'   \code{control} settings (e.g. \code{tol}, \code{stepsize}) to the
##'   Newton-Raphson routine.
##'
##' @return An object of class \code{c("binregStrata","binreg")}, a list with
##'   components including:
##'   \item{coef}{Estimated regression coefficients (stacked across strata).}
##'   \item{iid}{Subject-level iid decomposition of the coefficients, adjusted
##'     for censoring estimation when \code{se = TRUE}.}
##'   \item{var, robvar}{Robust (sandwich) variance-covariance matrix.}
##'   \item{se.coef, se.robust}{Standard errors derived from \code{var}.}
##'   \item{hessian, ihessian}{Per-stratum Hessians and their (pseudo-)inverses.}
##'   \item{coef_list}{Per-stratum coefficient vectors.}
##'   \item{time, cause, cens.code, model, outcome}{Echoed arguments.}
##'   \item{Y, Yipcw, IPCW}{The constructed response, IPCW-weighted response,
##'     and IPCW weights.}
##'   \item{cens.weights, formC, cens.strata, cens.nstrata}{Censoring model
##'     weights and related information (when internally estimated).}
##'   \item{nstrata, strata_orig, strata_call, cv.fold}{Stratification
##'     information and whether the cross-validation fold scheme was used.}
##'   \item{n, nevent, ncluster, nid, id, name.id}{Sample size and
##'     identifier information.}
##'   \item{call, design}{The matched call and the design object.}
##'
##' @details
##' When \code{cv.fold = FALSE} (the standard case), the coefficients for
##' stratum \code{s} are estimated using only observations with
##' \code{strata == s}. When \code{cv.fold = TRUE}, the coefficients for
##' stratum \code{s} are instead estimated using observations with
##' \code{strata != s}, i.e. the complement; this also affects the
##' construction of the censoring-related iid terms (\code{MGCiid}) and the
##' per-stratum subject-level influence functions used for the robust
##' variance.
##'
##' Censoring weights are computed via a Cox model (\code{\link{phreg}}) on
##' \code{cens.model}, evaluated at \code{pmin(exit, time)}, unless
##' \code{cens.weights} is supplied directly. When \code{se = TRUE}, the
##' additional variability coming from the estimation of the censoring
##' distribution is incorporated into the standard errors via a martingale-type
##' iid decomposition (\code{MGCiid}).
##'
##' @seealso \code{\link{phreg}}, \code{\link{binreg}}
##'
##' @examples
##' \dontrun{
##' library(mets)
##' data(bmt)
##' out <- binregStrata(Event(time, cause) ~ tcell + strata(platelet),
##'                      data = bmt, cause = 1, time = 50)
##' summary(out)
##' }
##'
#' @export
binregStrata <- function(formula, data, cause=1, time=NULL, beta=NULL,
                          type=c("II","I"),
                          offset=NULL, weights=NULL, cens.weights=NULL,
                          cens.model=~+1, se=TRUE,
                          kaplan.meier=TRUE, cens.code=0, no.opt=FALSE,
                          method="nr", augmentation=NULL,
                          outcome=c("cif","rmst","rmtl"),
                          model=c("default","logit","exp","lin"),
                          Ydirect=NULL, strata=NULL, cv.fold=FALSE, low.memory=FALSE,...)
{ # {{{
  monotone <- TRUE
  cl       <- match.call()

  ## ------------------------------------------------------------------
  ## Design
  ## ------------------------------------------------------------------
  des <- proc_design(
    formula, data = data,
    specials = c("offset","weights","cluster","strata"),
    intercept = TRUE
  )
  Y <- des$y
  if (!inherits(Y, c("Event","Surv")))
    stop("Expected a 'Surv' or 'Event'-object")

  if (ncol(Y) == 2) {
    exit  <- Y[,1]; entry <- rep(0, nrow(Y)); status <- Y[,2]
  } else {
    entry <- Y[,1]; exit  <- Y[,2];           status <- Y[,3]
  }

  X           <- des$x
  des.weights <- des$weights
  des.offset  <- des$offset
  id          <- des$cluster

  if (ncol(X) == 0) X <- matrix(nrow=0, ncol=0)

  call.id <- id
  conid   <- construct_id(id, nrow(X), namesX = rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  orig.id <- id

  ## ------------------------------------------------------------------
  ## Weights / offset  (explicit arg > formula-embedded special > default)
  ## ------------------------------------------------------------------
  if (!is.null(offset)) {
    offset <- offset
  } else if (!is.null(des.offset)) {
    offset <- des.offset
  } else {
    offset <- rep(0, length(exit))
  }

  if (!is.null(weights)) {
    weights <- weights
  } else if (!is.null(des.weights)) {
    weights <- des.weights
  } else {
    weights <- rep(1, length(exit))
  }

  ## ------------------------------------------------------------------
  ## Strata  (explicit arg > formula-embedded special > default)
  ## ------------------------------------------------------------------
  des.strata <- des$strata
  if (!is.null(strata)) {
    if (!is.numeric(strata)) stop("strata must be numeric\n")
    ustrata <- sort(unique(strata)); nstrata <- length(ustrata)
    strata  <- fast.approx(ustrata, strata) - 1
  } else if (!is.null(des.strata)) {
    strata  <- des.strata
    ustrata <- sort(unique(strata)); nstrata <- length(ustrata)
    strata  <- fast.approx(ustrata, strata) - 1
  } else {
    strata  <- rep(0, nrow(X)); nstrata <- 1
  }


  ## ------------------------------------------------------------------
  ## Event / censoring indicators
  ## ------------------------------------------------------------------
  statusC  <- 1*(status %in% cens.code)
  statusE  <- (status %in% cause) & (exit <= time)
  if (sum(statusE) == 0) warning("No events of type 1 before time\n")

  kmt     <- kaplan.meier
  ucauses <- sort(unique(status))
  ccc     <- which(ucauses %in% cens.code)
  Causes  <- if (length(ccc) == 0) ucauses else ucauses[-ccc]
  competing <- any(!(Causes %in% cause))

  data$id__    <- id
  data$exit    <- exit
  data$statusC <- statusC

  cens.strata <- cens.nstrata <- NULL
  nevent      <- sum((status %in% cause) * (exit <= time))
  obs         <- (exit <= time & statusC == 0) | (exit >= time)

  ## ------------------------------------------------------------------
  ## Censoring weights
  ## ------------------------------------------------------------------
  if (is.null(cens.weights)) { # {{{
    formC    <- update.formula(cens.model, Surv(exit,statusC) ~ . + cluster(id__))
    resC     <- phreg(formC, data)
    if (resC$p > 0) kmt <- FALSE
    exittime <- pmin(exit, time)
    cens.weights <- suppressWarnings(
      predict(resC, data, times = exittime, individual.time = TRUE,
              se = FALSE, km = kmt, tminus = TRUE)$surv)
    cens.strata  <- resC$strata[order(resC$ord)]
    cens.nstrata <- resC$nstrata
  } else {
    se <- FALSE; resC <- formC <- NULL
  } # }}}

  ## ------------------------------------------------------------------
  ## Response
  ## ------------------------------------------------------------------
  expit <- function(z) 1 / (1 + exp(-z))
  p     <- ncol(X)

  if (is.null(beta))         beta         <- rep(0, p)
  if (is.null(augmentation)) augmentation <- matrix(0, nstrata, p)
  if (!is.matrix(augmentation))
    augmentation <- matrix(augmentation, nstrata, p, byrow = TRUE)

  X  <- as.matrix(X)
  X2 <- .Call("vecCPMat", X)$XX

  if (!is.null(Ydirect)) {
    Y <- Ydirect
  } else {
    if (outcome[1] == "cif") {
      Y <- c((status %in% cause) * (exit <= time))
    } else {
      if (!competing) {
        if (outcome[1] == "rmst") Y <- c(pmin(exit, time))
        else                      Y <- c(time - pmin(exit, time))
      } else {
        Y <- c((status %in% cause) * (time - pmin(exit, time)))
      }
    }
  }
  Yresp <- c(Y)
  Y     <- Yipcw <- c(Y * obs / cens.weights)
  IPCW  <- c(obs / cens.weights)

  ## ------------------------------------------------------------------
  ## MGCiid: censoring IID influence
  ## cv.fold: block s populated with complement rows (strata != s)
  ## standard: block s populated with stratum-s rows
  ## ------------------------------------------------------------------
  MGCiid <- 0
  if (se & monotone) { # {{{
    ord     <- resC$ord
    X       <- X[ord,  , drop = FALSE]
    X2      <- X2[ord, , drop = FALSE]
    status  <- status[ord]
    exit    <- exit[ord]
    weights <- weights[ord]
    offset  <- offset[ord]
    Y       <- Y[ord]
    strata  <- strata[ord]

    xx   <- resC$cox.prep
    S0i2 <- S0i <- rep(0, length(xx$strata))
    S0i[xx$jumps  + 1] <- 1 / resC$S0
    S0i2[xx$jumps + 1] <- 1 / resC$S0^2
    btime <- 1*(exit < time)
    mid   <- max(id)

    ## Block-expanded design: n x (nstrata*p)
    ## cv.fold: block s gets complement rows; standard: block s gets stratum-s rows
    Xs <- matrix(0, nrow(X), nstrata * p)
    for (s in 0:(nstrata - 1)) {
      idx  <- if (cv.fold) which(strata != s) else which(strata == s)
      cols <- s * p + seq_len(p)
      Xs[idx, cols] <- X[idx, ]
    }

    h           <- Mrevcumsumstrata(Xs * Y, xx$strata, xx$nstrata)
    IhdLam0     <- Mcumsumstrata(h * S0i2 * btime, xx$strata, xx$nstrata)
    U           <- matrix(0, nrow(xx$X), ncol(Xs))
    U[xx$jumps + 1,] <- (resC$jumptimes < time) *
                         h[xx$jumps + 1,] / c(resC$S0)
    MGt    <- (U - IhdLam0) * c(xx$weights)
    MGCiid <- Msumstrata(MGt, xx$id, mid + 1)

    if (type[1] == "II") { # {{{
      hYt      <- revcumsumstrata(Y,   xx$strata, xx$nstrata)
      IhdLam0  <- cumsumstrata(hYt * S0i2 * btime, xx$strata, xx$nstrata)
      U2       <- rep(0, length(xx$strata))
      U2[xx$jumps + 1] <- (resC$jumptimes < time) *
                           hYt[xx$jumps + 1] / c(resC$S0)
      MGt2    <- Xs * c(U2 - IhdLam0) * c(xx$weights)
      MGtiid  <- Msumstrata(MGt2, xx$id, mid + 1)
      augmentation <- colSums(MGtiid) + augmentation
      augmentation <- matrix(augmentation, nstrata, p, byrow = TRUE)

      EXt          <- Mrevcumsumstrata(Xs, xx$strata, xx$nstrata)
      IEXhYtdLam0  <- Mcumsumstrata(EXt * c(hYt) * S0i * S0i2 * btime,
                                     xx$strata, xx$nstrata)
      U3 <- matrix(0, nrow(xx$X), ncol(Xs))
      U3[xx$jumps + 1,] <- (resC$jumptimes < time) *
                            hYt[xx$jumps + 1] *
                            EXt[xx$jumps + 1,] / c(resC$S0)^2
      MGt3    <- (U3 - IEXhYtdLam0) * c(xx$weights)
      MGCiid2 <- Msumstrata(MGt3, xx$id, mid + 1)
      MGCiid  <- MGCiid + (MGtiid - MGCiid2)
    } # }}}

    id <- xx$id
  } else {
    MGCiid <- 0
  } # }}}

  ## ------------------------------------------------------------------
  ## Model default
  ## ------------------------------------------------------------------
  if (model[1] == "default") {
    if (outcome[1] == "cif")        model <- "logit"
    if (outcome[1] == "rmst")       model <- "exp"
    if (outcome[1] == "rmtl")       model <- "exp"
    if (outcome[1] == "years-lost") model <- "exp"
  }

  ## ------------------------------------------------------------------
  ## obj(): full objective (used for final all=TRUE call and NR)
  ## ------------------------------------------------------------------
  obj <- function(pp, all = FALSE) { # {{{
    np <- length(pp[[1]])

    ## ----------------------------------------------------------------
    ## cv.fold path: for stratum s, beta_s estimated on complement rows
    ## ----------------------------------------------------------------
    if (cv.fold) {
      ploglik_list  <- vector("list", nstrata)
      gradient_list <- vector("list", nstrata)
      D2log_list    <- vector("list", nstrata)

      for (s in 0:(nstrata - 1)) {
        idx_c <- which(strata != s)          # complement indices
        X_c   <- X[idx_c,  , drop=FALSE]
        X2_c  <- X2[idx_c, , drop=FALSE]
        Y_c   <- Y[idx_c]
        w_c   <- weights[idx_c]
        off_c <- offset[idx_c]
        aug_s <- augmentation[s + 1, ]
        lp_c  <- as.vector(X_c %*% pp[[s + 1]]) + off_c

        if (model[1] == "exp") {
          pr_c  <- exp(lp_c)
          if (monotone) {
            Dl_c  <- w_c * X_c * c(Y_c - pr_c)
            D2l_c <- c(w_c * pr_c) * X2_c
          } else {
            Dl_c  <- w_c * X_c * c((Y_c - pr_c) * pr_c)
            D2l_c <- c(w_c * pr_c^2) * X2_c
          }
          pl_c <- 0
        } else if (model[1] == "lin") {
          pr_c  <- lp_c
          Dl_c  <- w_c * X_c * c(Y_c - pr_c)
          D2l_c <- c(w_c) * X2_c
          pl_c  <- sum(w_c * (Y_c - pr_c)^2)
        } else if (model[1] == "logit") {
          pr_c <- expit(lp_c)
          if (monotone) {
            Dl_c  <- w_c * X_c * c(Y_c - pr_c)
            D2l_c <- c(w_c * pr_c / (1 + exp(lp_c))) * X2_c
          } else {
            varp  <- pr_c / (1 + exp(lp_c))
            Dl_c  <- w_c * X_c * c((Y_c - pr_c) * varp)
            D2l_c <- c(w_c * varp^2) * X2_c
          }
          pl_c <- 0
        } else stop("link must be logit, exp, or lin\n")

        ploglik_list[[s + 1]]  <- pl_c
        gradient_list[[s + 1]] <- -(colSums(Dl_c) + aug_s)
        D2log_list[[s + 1]]    <- colSums(D2l_c)
      }

      sym <- function(x)
        matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                     PACKAGE="mets")$XXf, np, np)
      hessian <- lapply(D2log_list, sym)

      if (all) {
        ihess <- lapply(hessian, pinv)

        ## Per-stratum IID: recompute Dl_c at optimal pp[[s]] separately
        ## for each stratum so blocks are independent and correctly sized.
        ## Obs i in stratum r contributes to beta_s for all s != r;
        ## Msumstrata fills zeros for stratum-s obs (they don't contribute).
        beta.iid <- do.call(cbind, lapply(seq_len(nstrata), function(s) {
          idx_c <- which(strata != (s - 1))
          X_c   <- X[idx_c,  , drop=FALSE]
          Y_c   <- Y[idx_c]
          w_c   <- weights[idx_c]
          off_c <- offset[idx_c]
          id_c  <- id[idx_c]
          lp_c  <- as.vector(X_c %*% pp[[s]]) + off_c

          Dl_c <- switch(model[1],
            exp = {
              pr_c <- exp(lp_c)
              if (monotone) w_c * X_c * c(Y_c - pr_c)
              else          w_c * X_c * c((Y_c - pr_c) * pr_c)
            },
            lin = w_c * X_c * c(Y_c - lp_c),
            logit = {
              pr_c <- expit(lp_c)
              if (monotone) w_c * X_c * c(Y_c - pr_c)
              else {
                varp <- pr_c / (1 + exp(lp_c))
                w_c * X_c * c((Y_c - pr_c) * varp)
              }
            },
            stop("unknown model\n")
          )

          raw <- Dl_c %*% ihess[[s]]
          Msumstrata(raw, id_c, max(id) + 1)
        }))

        robvar <- crossprod(beta.iid)
        return(list(
          par      = pp,             coef      = unlist(pp),
          ploglik  = ploglik_list,   gradient  = gradient_list,
          hessian  = hessian,        ihessian  = ihess,
          id       = id,             Dlogl     = NULL,
          iid      = beta.iid,       robvar    = robvar,
          var      = robvar,         se.robust = diag(robvar)^.5
        ))
      }

      return(structure(ploglik_list,
                       gradient = gradient_list,
                       hessian  = hessian))
    }

# ----------------------------------------------------------------
# Standard non-CV
# ----------------------------------------------------------------
    loffset <- eta <- numeric(length(Y))
    for (s in 0:(nstrata - 1)) {
      idx          <- strata == s
      eta[idx]     <- as.vector(X[idx, , drop=FALSE] %*% pp[[s + 1]])
      loffset[idx] <- offset[idx]
    }
    lp <- eta + loffset

    if (model[1] == "exp") {
      pr     <- exp(lp)
      if (monotone) {
        Dlogl  <- weights * X * c(Y - pr)
        D2logl <- c(weights * pr) * X2
      } else {
        Dlogl  <- weights * X * c((Y - pr) * pr)
        D2logl <- c(weights * pr^2) * X2
      }
    } else if (model[1] == "lin") {
      pr     <- lp
      Dlogl  <- weights * X * c(Y - pr)
      D2logl <- c(weights) * X2
    } else if (model[1] == "logit") {
      pr <- expit(lp)
      if (monotone) {
        Dlogl  <- weights * X * c(Y - pr)
        D2logl <- c(weights * pr / (1 + exp(lp))) * X2
      } else {
        varp   <- pr / (1 + exp(lp))
        Dlogl  <- weights * X * c((Y - pr) * varp)
        D2logl <- c(weights * varp^2) * X2
      }
    } else stop("link functions must be logit, exp, lin\n")

    ploglik  <- sumstrata(weights * (Y - pr)^2, strata, nstrata)
    ploglik  <- matrix(ploglik, nstrata, 1)
    if (model[1] != "lin") ploglik <- 0 * ploglik
    D2log    <- Msumstrata(D2logl,  strata, nstrata)
    gradient <- Msumstrata(Dlogl,   strata, nstrata) + augmentation
    if (!is.matrix(D2log))    D2log    <- matrix(D2log,    1, length(D2log))
    if (!is.matrix(gradient)) gradient <- matrix(gradient, 1, np)

    gradient <- split(-gradient, seq_len(nrow(gradient)))
    ploglik  <- split(-ploglik,  seq_len(nrow(ploglik)))
    D2log    <- split(D2log,     seq_len(nrow(D2log)))

    sym <- function(x)
      matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                   PACKAGE="mets")$XXf, np, np)
    hessian <- lapply(D2log, sym)

    if (all) {
      ihess    <- lapply(hessian, pinv)
      iid_list <- lapply(seq_len(nstrata), function(s) {
        idx  <- strata == (s - 1)
        id_s <- id[idx]
        Dl_s <- Dlogl[idx, , drop=FALSE]
        raw  <- Dl_s %*% ihess[[s]]
        Msumstrata(raw, id_s, max(id) + 1)
      })
      beta.iid <- do.call(cbind, iid_list)
      robvar   <- crossprod(beta.iid)
      return(list(
        par      = pp,           coef      = unlist(pp),
        ploglik  = ploglik,      gradient  = gradient,
        hessian  = hessian,      ihessian  = ihess,
        id       = id,           Dlogl     = Dlogl,
        iid      = beta.iid,     robvar    = robvar,
        var      = robvar,       se.robust = diag(robvar)^.5
      ))
    }
    structure(ploglik, gradient = gradient, hessian = hessian)
  } # }}}

  ## ------------------------------------------------------------------
  ## make_stratum_obj: per-stratum closure for NR
  ## cv.fold: use complement rows; standard: use stratum rows
  ## ------------------------------------------------------------------
  make_stratum_obj <- function(s, pp) { # {{{
    idx       <- if (cv.fold) strata != s else strata == s
    X_s       <- X[idx,  , drop=FALSE]
    X_s2      <- X2[idx, , drop=FALSE]
    Y_s       <- Y[idx]
    weights_s <- weights[idx]
    offset_s  <- offset[idx]
    aug_s     <- augmentation[s + 1, ]

    sym <- function(x, np)
      matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                   PACKAGE="mets")$XXf, np, np)

    function(theta) {
      lp <- as.vector(X_s %*% theta) + offset_s
      if (model[1] == "exp") {
        pr <- exp(lp)
        if (monotone) {
          Dlogl  <- weights_s * X_s * c(Y_s - pr)
          D2logl <- c(weights_s * pr) * X_s2
        } else {
          Dlogl  <- weights_s * X_s * c((Y_s - pr) * pr)
          D2logl <- c(weights_s * pr^2) * X_s2
        }
        ploglik <- 0
      } else if (model[1] == "lin") {
        pr      <- lp
        Dlogl   <- weights_s * X_s * c(Y_s - pr)
        D2logl  <- c(weights_s) * X_s2
        ploglik <- sum(weights_s * (Y_s - pr)^2)
      } else if (model[1] == "logit") {
        pr <- expit(lp)
        if (monotone) {
          Dlogl  <- weights_s * X_s * c(Y_s - pr)
          D2logl <- c(weights_s * pr / (1 + exp(lp))) * X_s2
        } else {
          varp   <- pr / (1 + exp(lp))
          Dlogl  <- weights_s * X_s * c((Y_s - pr) * varp)
          D2logl <- c(weights_s * varp^2) * X_s2
        }
        ploglik <- 0
      } else stop("link functions must be logit, exp, lin\n")

      np   <- length(theta)
      grad <- -(colSums(Dlogl) + aug_s)
      D2   <- colSums(D2logl)
      hess <- sym(D2, np)
      structure(-ploglik, gradient = grad, hessian = hess)
    }
  } # }}}

  ## ------------------------------------------------------------------
  ## Optimise
  ## ------------------------------------------------------------------
  dots    <- list(...)
  control <- if (length(dots) == 0) {
    if (model[1] == "exp") list(tol=1e-10, stepsize=0.5) else NULL
  } else dots[[1]]

  if (!is.matrix(beta)) beta <- matrix(beta, nstrata, p, byrow = TRUE)
  beta <- split(beta, seq_len(nrow(beta)))

  NR_list <- function() {
    Map(function(s) {
      lava::NR(start   = beta[[s + 1]],
                obj     = make_stratum_obj(s, beta),
                control = control)
    }, seq_len(nstrata) - 1L)
  }

  beta_opt <- beta   # fallback

  if (p > 0) {
    if (!no.opt) {
      if (tolower(method) == "nr") {
        tim      <- NR_list()
        beta_opt <- lapply(tim, function(r) r$par)
        cc       <- unlist(beta_opt)
        val      <- c(list(coef = cc), obj(beta_opt, all = TRUE))
        val$gradient <- lapply(tim, function(r) r$gradient)
      } else {
        stop("only NR of lava\n")
      }
    } else {
      val <- c(list(coef = unlist(beta)), obj(beta, all = TRUE))
    }
  } else {
    val <- obj(as.list(rep(0, nstrata)), all = TRUE)
  }

  ## ------------------------------------------------------------------
  ## Coefficient naming
  ## ------------------------------------------------------------------
  if (nstrata == 1 && length(val$coef) == length(colnames(X))) {
    names(val$coef) <- colnames(X)
  } else {
    cnames <- paste(
      rep(paste0("s", 1:nstrata), each = ncol(X)),
      rep(colnames(X), times = nstrata),
      sep = "."
    )
    names(val$coef)   <- cnames
    colnames(val$iid) <- cnames
  }

  ## ------------------------------------------------------------------
  ## Assemble output
  ## ------------------------------------------------------------------
  val <- c(val, list(
    time          = time,          formula      = formula,
    formC         = formC,         cens.weights = cens.weights,
    cens.strata   = cens.strata,   cens.nstrata = cens.nstrata,
    n             = length(exit),  nevent       = nevent,
    ncluster      = nid
  ))
  val$call      <- cl
  val$MGciid    <- MGCiid
  val$call.id   <- call.id
  val$id        <- orig.id
  val$name.id   <- name.id
  val$nid       <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL
  val$coef_list <- beta_opt

  ## ------------------------------------------------------------------
  ## Censoring IID adjustment, per stratum block
  ## iid[, cols_s] += MGCiid[, cols_s] %*% H_s^{-1}
  ## For cv.fold, MGCiid[, cols_s] already encodes complement rows via Xs,
  ## and ihessian[[s]] is from the complement fit
  ## ------------------------------------------------------------------
  if (se) {
    val$iid <- do.call(cbind, lapply(seq_len(nstrata), function(s) {
      cols <- (s - 1) * p + seq_len(p)
      val$iid[, cols] + MGCiid[, cols] %*% val$ihessian[[s]]
    }))
  }

  if (!is.null(name.id)) val$iid <- nameme(val$iid, name.id)
  robvar        <- crossprod(val$iid)
  val$var       <- val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef   <- diag(val$var)^.5

  val$cause        <- cause
  val$cens.code    <- cens.code
  val$augmentation <- augmentation
  val$model        <- model[1]
  val$outcome      <- outcome[1]
  val$Yipcw        <- Yipcw
  val$Y            <- Yresp
  val$IPCW         <- IPCW
  val$Causes       <- Causes
  val$nevent       <- nevent
  val$design       <- des
  val$nstrata      <- nstrata
  val$strata_orig  <- strata
  val$strata_call  <- des.strata
  val$cv.fold      <- cv.fold

  if (low.memory) {
	val$design$y <- NULL
	val$design$data <- NULL
	val$design$x <- NULL
	val$design$strata <- NULL
	val$strata_orig <- NULL
	val$strata_call <- NULL
	val$MGciid <- NULL
	val$iid.naive <- NULL
	val$iid.naive <- NULL
	val$cens.strata <-  NULL
	val$cens.weights <-  NULL
	val$name.id  <-  NULL
  }

  class(val) <- c("binregStrata","binreg")
  return(val)
} # }}}

## {{{  predict, 

## small helper: given flat coef vector, route predictions per stratum
strata_lp <- function(object, Z, strata_new, offset_new) {
  p         <- ncol(Z)
  beta_list <- split(object$coef, rep(seq_len(object$nstrata), each = p))
  eta       <- numeric(nrow(Z))
  for (s in seq_len(object$nstrata)) {
    idx       <- strata_new == (s - 1)
    eta[idx]  <- as.vector(Z[idx, , drop = FALSE] %*% beta_list[[s]]) + offset_new[idx]
  }
  eta
}

##' @export
predict.binregStrata <- function(object, newdata = NULL, se = TRUE,
                                  pred.iid = FALSE, level = 0.95,
                                  strata = NULL,offset=NULL, ...) {  ## {{{ 
  ## design matrix
  if (!is.null(newdata)) {
    x <- update_design(object$design, data = newdata, response = FALSE)
  } else {
    x <- object$design
  }
  Z          <- x$x

  if (!is.null(strata)) {
  strata_new <- as.numeric(strata) - 1
  } else if (!is.null(x$strata)) {
  strata_new <- as.numeric(x$strata) - 1
  } else {
  strata_new <- object$strata_orig
  }

  offset_new <- if (!is.null(offset)) {
  offset
  } else if (!is.null(x$offset)) {
  x$offset
  } else {
  rep(0, nrow(Z))
  }

  ## linear predictor routed per stratum
  lp <- strata_lp(object, Z, strata_new, offset_new)

  ## link
  p <- switch(object$model[1],
    logit = expit(lp),
    exp   = exp(lp),
    lin   = lp,
    stop("unknown model")
  )

## derivative for delta method: same shape as before, per-stratum coef block
  p_col <- ncol(Z)
  Dpv   <- matrix(0, nrow(Z), object$nstrata * p_col)
  for (s in seq_len(object$nstrata)) {
    idx  <- strata_new == (s - 1)
    cols <- (s - 1) * p_col + seq_len(p_col)
    if (object$model[1] == "logit") {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE] * exp(-lp[idx]) * p[idx]^2
    } else if (object$model[1] == "exp") {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE] * p[idx]
    } else {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE]
    }
  }

  preds <- p
  if (se) {
    covv   <- object$var
    se_vec <- apply((Dpv %*% covv) * Dpv, 1, sum)^.5
    crit   <- qnorm(1 - (1 - level) / 2)
    preds  <- data.frame(pred  = p,
                         se    = se_vec,
                         lower = p - crit * se_vec,
                         upper = p + crit * se_vec)
  }
  if (pred.iid) {
    Piid  <- object$iid %*% t(Dpv)
    preds <- list(pred = preds, iid = Piid, id = x$cluster)
  }
  attr(preds, "id") <- x$cluster
  preds
} ## }}} 

## }}} 

## --------------------------------------------------------------------------
## summary.binregStrata
## --------------------------------------------------------------------------
##' @export
summary.binregStrata <- function(object, level = 0.95, ...) { ## {{{
  p       <- length(object$coef) / object$nstrata
  nms     <- colnames(object$design$x)
  n       <- nrow(object$iid)
  crit    <- qnorm(1 - (1 - level) / 2)

  exp_label <- switch(object$model[1],
    logit = "OR",
    exp   = "RR",
    lin   = NULL
  )

  strata_tables     <- vector("list", object$nstrata)
  strata_exp_tables <- vector("list", object$nstrata)

  for (s in seq_len(object$nstrata)) {
    cols  <- (s - 1L) * p + seq_len(p)
    b_s   <- object$coef[cols]
    iid_s <- object$iid[, cols, drop = FALSE]
    e_s   <- lava::estimate(NULL, coef = b_s, IC = iid_s * n)
    cm    <- e_s$coefmat
    rownames(cm) <- if (!is.null(nms)) nms else paste0("X", seq_len(p))
    strata_tables[[s]] <- cm

    if (!is.null(exp_label)) {
      e_exp <- lava::estimate(NULL, coef = b_s, IC = iid_s * n, f = exp)
      cm_exp <- e_exp$coefmat
      rownames(cm_exp) <- rownames(cm)
      strata_exp_tables[[s]] <- cm_exp
    }
  }
  names(strata_tables)     <- paste0("stratum", seq_len(object$nstrata))
  names(strata_exp_tables) <- paste0("stratum", seq_len(object$nstrata))

  structure(
    list(strata_tables     = strata_tables,
         strata_exp_tables = strata_exp_tables,
         exp_label         = exp_label,
         nstrata           = object$nstrata,
         p                 = p,
         model             = object$model[1],
         outcome           = object$outcome[1],
         n                 = object$n,
         nevent            = object$nevent,
         ncluster          = object$ncluster,
         cv.fold           = isTRUE(object$cv.fold),
         call              = object$call),
    class = "summary.binregStrata"
  )
} ## }}}

## --------------------------------------------------------------------------
## print.summary.binregStrata
## --------------------------------------------------------------------------
##' @export
print.summary.binregStrata <- function(x, digits = 3, ...) { ## {{{ 
  cat("\nCall:\n")
  if (!is.null(x$call)) print(x$call)
  cat("\nStratified binreg fit\n")
  cat("  Strata  :", x$nstrata,
      if (x$cv.fold) " [cv.fold -- leave-one-stratum-out CV]" else "", "\n")
  cat("  Model   :", x$model, "\n")
  cat("  Outcome :", x$outcome, "\n")
  cat("  n =", x$n,
      " | events =", x$nevent,
      " | clusters =", x$ncluster, "\n\n")

  for (s in seq_len(x$nstrata)) {
    cat(paste0("--- Stratum ", s,
               if (x$cv.fold)
                 paste0(" (fitted on strata != ", s, ")")
               else "",
               " ---\n"))
    printCoefmat(x$strata_tables[[s]], digits = digits, ...)

    if (!is.null(x$exp_label) && !is.null(x$strata_exp_tables[[s]])) {
      cat(x$exp_label, "= exp(coef):\n")
      em <- x$strata_exp_tables[[s]][, c("Estimate","2.5%","97.5%"), drop = FALSE]
      colnames(em)[1] <- x$exp_label
      print(round(em, digits))
    }
    cat("\n")
  }
  invisible(x)
} ## }}} 

## --------------------------------------------------------------------------
## print.binregStrata  -- brief one-liner per stratum
## --------------------------------------------------------------------------
##' @export
print.binregStrata <- function(x, digits = 3, ...) { ## {{{ 
  cat("\nStratified binreg fit\n")
  cat("  Model:", x$model[1], " | Outcome:", x$outcome[1],
      " | Strata:", x$nstrata,
      if (isTRUE(x$cv.fold)) " [cv.fold]" else "", "\n")
  cat("  n =", x$n,
      "| events =", x$nevent,
      "| clusters =", x$ncluster, "\n\n")

  p   <- length(x$coef) / x$nstrata
  nms <- colnames(x$design$x)
  n   <- nrow(x$iid)

  for (s in seq_len(x$nstrata)) {
    cols <- (s - 1L) * p + seq_len(p)
    b_s  <- x$coef[cols]
    se_s <- sqrt(diag(x$var)[cols])
    mat  <- cbind(Estimate = b_s, `Std.Err` = se_s,
                  `z`      = b_s / se_s,
                  `Pr(>|z|)` = 2 * pnorm(-abs(b_s / se_s)))
    if (!is.null(nms)) rownames(mat) <- nms
    cat(paste0("Stratum ", s,
               if (isTRUE(x$cv.fold))
                 paste0(" (train: strata != ", s, ")")
               else "", ":\n"))
    printCoefmat(mat, digits = digits, ...)
    cat("\n")
  }
  invisible(x)
} ## }}} 

##' @export
coef.binregStrata <- function(object, ...) object$coef

##' @export
vcov.binregStrata <- function(object, ...) object$var

##' @export
IC.binregStrata <- function(x, ...) x$iid * NROW(x$iid)


##' Cross-validated Brier score for competing risks, RMST and RMTL regression
##' using stratified leave-fold-out fits (\code{binregStrata})
##'
##' Fits candidate models via \code{\link[mets]{binregStrata}} with
##' \code{cv.fold = TRUE} and evaluates each using apparent (in-sample) and
##' \emph{k}-fold cross-validated IPCW Brier scores relative to a null model.
##' Supports all outcome types available in \code{binreg}/\code{binregStrata}:
##' cause-specific cumulative incidence (competing risks), restricted mean
##' survival time (RMST), and restricted mean time lost (RMTL).
##'
##' Cross-validation is delegated directly to
##' \code{\link[mets]{binregStrata}} via its \code{strata} and
##' \code{cv.fold = TRUE} arguments: each stratum (fold) gets its own
##' coefficient vector \eqn{\beta_s}, estimated on the complement of that
##' fold. Out-of-fold predictions are obtained by routing each observation to
##' the \eqn{\beta_s} corresponding to its own fold, which was trained on the
##' other folds.
##'
##' The fold membership is stored in an internal \code{fold} column (1-based)
##' in \code{data} and passed to \code{binregStrata} as a 0-based
##' \code{strata} vector. Censoring weights for the underlying competing-risks
##' model are estimated according to \code{cens.model} as usual; since strata
##' already define the folds, \code{cens.model} need not include
##' \code{strata(fold)} explicitly.
##'
##' Brier scores are computed on the \strong{linear scale} via
##' \code{resmeanIPCW(..., model = "lin")} and then log-transformed using
##' \code{lava::estimate(..., f = log)} with the delta method. All stored
##' coefficients, standard errors, and influence functions are therefore on
##' the \strong{log scale}. Use \code{summary(fit, transform = exp)} or
##' \code{estimate(fit, f = exp)} to back-transform to the original Brier
##' scale.
##'
##' The delta-method correction for the cross-validated influence function
##' accounts for the block structure of \code{binregStrata}'s influence
##' function (\eqn{n_{id} \times (\code{nstrata} \times p)}, one
##' \eqn{p}-column block per stratum). For each stratum \eqn{s}, only
##' observations belonging to fold \eqn{s} have a non-zero derivative of
##' their out-of-fold prediction with respect to \eqn{\beta_s}, since
##' \eqn{\beta_s} is exactly the coefficient used to predict fold \eqn{s}.
##' The full gradient is therefore block-sparse, with one non-zero
##' \eqn{p}-length block per stratum, and the correction reduces to a single
##' matrix product against \code{iid(outff_fit)}.
##'
##' Left-truncation (delayed entry) is not supported and will raise an error.
##' Specifically, \code{Event(entry, time, status)} is rejected; use
##' \code{Event(time, status)} or \code{Event(time, status, cause)}.
##'
##' @param formula Two-sided formula of the form
##'   \code{Event(time, status) ~ covariates + cluster(id)} or
##'   \code{Event(time, status, cause) ~ covariates + cluster(id)}.
##'   The \code{cluster(id)} term is optional; if absent an internal
##'   identifier \code{id__} is added automatically assuming independent
##'   observations.
##' @param data A \code{data.frame} containing all variables in
##'   \code{formula} and \code{rhs}.
##' @param time Scalar or numeric vector of evaluation time points. When a
##'   vector is supplied the function loops over time points reusing the same
##'   fold structure and returns a named list of \code{binregCV} objects with
##'   class \code{"binregCV_list"}.
##' @param fold Integer. Number of cross-validation folds / strata
##'   (default 5).
##' @param rhs Named list of one-sided formulas specifying candidate models,
##'   e.g. \code{list(small = ~Z1, full = ~Z1 + Z2)}. If \code{NULL} (default),
##'   the right-hand side of \code{formula} (minus any \code{cluster()} term)
##'   is used as a single candidate named \code{"model1"}.
##' @param rhs0 One-sided formula for the null (intercept-only) model
##'   (default \code{~+1}).
##' @param cens.model Censoring model passed to \code{binregStrata} for the
##'   CV fold fits (default \code{~+1}). Since folds are handled internally
##'   by \code{binregStrata}'s \code{strata}/\code{cv.fold} mechanism, this
##'   does \emph{not} need to include \code{strata(fold)}.
##' @param brier.cens.model Censoring model passed to \code{resmeanIPCW} when
##'   computing Brier scores. Default \code{~+1} gives Kaplan-Meier weights.
##'   Supply a stratified model, e.g. \code{~strata(Z1, Z2)}, for
##'   covariate-adjusted IPCW weights.
##' @param brier.args Named list of additional arguments forwarded to every
##'   \code{resmeanIPCW} call (beyond \code{formula}, \code{data},
##'   \code{time}, \code{Ydirect}, \code{cens.model}, and \code{model} which
##'   are set internally). Typically empty.
##' @param cv.split.index List from \code{folds(n, fold)} or a vector of fold
##'   indices (1:fold) for each observation, to fix the cross-validation
##'   folds. \code{NULL} (default) generates folds internally via
##'   \code{folds(n, fold)}.
##' @param \dots Additional arguments passed to every \code{binreg} /
##'   \code{binregStrata} call, e.g. \code{cause = 2}, \code{outcome = "rmst"},
##'   or \code{offset}.
##'
##' @return An object of class \code{"binregCV"} (single time point) or
##'   \code{"binregCV_list"} (multiple time points), with the following
##'   structure:
##' \describe{
##'   \item{\code{null}}{List with \code{coef}, \code{se}, and \code{iid} for
##'     the null model. \code{coef} and \code{se} are named vectors with
##'     entries \code{bbrier} (apparent, log scale) and \code{bbrierCV}
##'     (cross-validated, log scale). \code{iid} contains the
##'     influence-function matrix for \code{bbrierCV} with delta-method
##'     correction for the stratified CV prediction step and the log
##'     transform.}
##'   \item{\code{models}}{Named list, one element per entry in \code{rhs}.
##'     Each element contains \code{coef} (\code{brier}, \code{brierCV}, log
##'     scale), \code{se}, \code{dcoef} (differences from null: log Brier
##'     ratios), \code{sed}, and \code{iid} (influence-function matrix for
##'     \code{brierCV}, log scale, with CV delta-method correction).}
##'   \item{\code{iid_mat}}{Matrix of dimension \eqn{n \times (1 +
##'     \code{length(rhs)})} with columns \code{null} and one per entry of
##'     \code{rhs}, each the log-scale CV Brier influence function for the
##'     corresponding model. Used by \code{\link{estimate.binregCV}} for joint
##'     inference.}
##'   \item{\code{cv.split.index}}{List of fold index vectors used.}
##'   \item{\code{rhs}, \code{rhs0}}{Processed candidate and null formulas
##'     (cluster terms removed).}
##'   \item{\code{model}}{Link function read from the \code{binreg} fit:
##'     \code{"logit"}, \code{"exp"}, or \code{"lin"}.}
##'   \item{\code{call}, \code{time}, \code{fold}, \code{cens.model},
##'     \code{brier.cens.model}}{Call and settings for printing and
##'     downstream use.}
##' }
##'
##' @section Outcome types:
##' The function supports all outcome types provided by \code{binreg}/
##' \code{binregStrata}:
##' \describe{
##'   \item{Competing risks CIF}{Default; use
##'     \code{Event(time, status, cause)} and pass \code{cause} via \code{...}.
##'     The Brier score measures squared error for the cause-specific CIF.}
##'   \item{RMST}{Pass \code{outcome = "rmst"} via \code{...}. The Brier score
##'     measures squared error for the restricted mean survival time up to
##'     \code{time}.}
##'   \item{RMTL}{Pass \code{outcome = "rmtl"} via \code{...}. The Brier score
##'     measures squared error for the restricted mean time lost.}
##' }
##'
##' @section Scale and inference:
##' All output is on the log scale. To obtain Brier scores on the original
##' scale with confidence intervals, use:
##' \preformatted{
##'   summary(fit, transform = exp)
##'   estimate(fit, f = exp)
##' }
##' Differences on the log scale correspond to log Brier ratios; exponentiate
##' to obtain Brier ratios. For absolute differences on the original scale,
##' compute contrasts after \code{estimate(fit, f = exp)}.
##'
##' @seealso
##'   \code{\link{estimate.binregCV}} for joint inference across models and
##'   time points;
##'   \code{\link{summary.binregCV}} and \code{\link{print.summary_binregCV_multi}}
##'   for formatted output;
##'   \code{\link[mets]{binregStrata}} for the underlying stratified
##'   regression model;
##'   \code{\link[mets]{resmeanIPCW}} for IPCW mean estimation;
##'   \code{\link[lava]{estimate}} for delta-method inference.
##'
##' @examples
##' \dontrun{
##' library(mets)
##' data(bmt)
##' bmt$id <- seq_len(nrow(bmt))
##'
##' ## --- competing risks CIF/Survival (default) ---
##' fit <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs = list(small = ~age, full = ~tcell + platelet + age)
##' )
##' summary(fit)                        ## log scale
##' summary(fit, transform = exp)       ## Brier scale
##'
##' ## multiple time points
##' fitm <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = c(50, 60),
##'   rhs = list(small = ~age, full = ~tcell + platelet + age)
##' )
##' summary(fitm, transform = exp)
##'
##' ## joint inference across models via lava::estimate
##' e <- estimate(fitm)
##' e
##' estimate(e, f = exp)               ## back-transform
##'
##' ## stratified IPCW weights for Brier score
##' fit_strat <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs = list(small = ~age, full = ~tcell + platelet + age),
##'   brier.cens.model = ~strata(tcell)
##' )
##'
##' ## --- RMST --- survival case only
##' fit_rmst <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50, cause = 1:2,
##'   rhs    = list(small = ~age, full = ~tcell + platelet + age),
##'   outcome = "rmst"
##' )
##' summary(fit_rmst, transform = exp)
##'
##' ## --- RMTL --- competing risks or survival (RMTL)
##' fit_rmtl <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs    = list(small = ~age, full = ~tcell + platelet + age),
##'   outcome = "rmtl"
##' )
##' summary(fit_rmtl, transform = exp)
##' }
##' @export
brier_binreg <- function(formula, data, time,
                                     fold             = 5,
                                     rhs              = NULL,
                                     rhs0             = ~+1,
                                     cens.model       = ~+1,
                                     brier.cens.model = ~+1,
                                     brier.args       = list(),
                                     cv.split.index   = NULL,
                                     ...) { ## {{{
  ## ------------------------------------------------------------------
  ## Small helpers (identical to brier_binregStrata)
  ## ------------------------------------------------------------------
  extract_cluster_id <- function(f) {
    tt     <- terms(f, specials = "cluster")
    cl_idx <- attr(tt, "specials")$cluster
    if (is.null(cl_idx)) return(NULL)
    cl_term <- attr(tt, "variables")[[cl_idx + 1L]]
    as.character(cl_term[[2]])
  }
  drop_cluster <- function(f) {
    tt     <- terms(f, specials = "cluster")
    cl_idx <- attr(tt, "specials")$cluster
    if (is.null(cl_idx)) return(f)
    cl_var <- attr(tt, "variables")[[cl_idx + 1L]]
    update(f, paste("~ . -", deparse(cl_var)))
  }
  ## ------------------------------------------------------------------
  ## Parse top-level formula
  ## ------------------------------------------------------------------
  lhs_str  <- deparse(formula[[2]])
  lhs_call <- formula[[2]]
  if (length(lhs_call) == 4L)
    stop("brier_binregStrata_loop does not support delayed entry (left-truncation).\n",
         "  Found: ", deparse(lhs_call))
  time_var   <- as.character(lhs_call[[2]])
  status_var <- as.character(lhs_call[[3]])
  id         <- extract_cluster_id(formula)
  if (is.null(id)) {
    id         <- "id__"
    data[[id]] <- seq_len(nrow(data))
    formula    <- update(formula, paste("~ . + cluster(id__)"))
  }
  base_resmean <- as.formula(paste(lhs_str, "~ +1 + cluster(", id, ")"))
  ## ------------------------------------------------------------------
  ## Parse rhs list
  ## ------------------------------------------------------------------
  formula_rhs_clean <- drop_cluster(
    reformulate(attr(terms(formula), "term.labels"), intercept = FALSE))
  if (is.null(rhs))        rhs <- list(model1 = formula_rhs_clean)
  if (!is.list(rhs))       rhs <- list(model1 = rhs)
  if (is.null(names(rhs)) || any(names(rhs) == ""))
    names(rhs) <- paste0("model", seq_along(rhs))
  rhs  <- lapply(rhs,  drop_cluster)
  rhs0 <- drop_cluster(rhs0)
  build_full <- function(f) {
    labs    <- attr(terms(f), "term.labels")
    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
    as.formula(paste(lhs_str, "~", rhs_str, "+ cluster(", id, ")"))
  }
  if (!id %in% names(data)) stop("id variable '", id, "' not found in data")
  ## ------------------------------------------------------------------
  ## CV fold index -- computed once, reused for all time points
  ## ------------------------------------------------------------------
  as_fold_list <- function(x) {
    if (is.list(x))                      return(x)
    if (is.numeric(x) || is.integer(x)) return(split(seq_along(x), x))
    stop("Unsupported cv.split.index format")
  }
  n  <- nrow(data)
  dd <- if (is.null(cv.split.index)) folds(n, fold) else as_fold_list(cv.split.index)
  data$fold <- 0L
  for (k in seq_len(fold)) data$fold[dd[[k]]] <- k
  strata_vec <- data$fold - 1L
  build_strata_formula <- function(f) {
    labs    <- attr(terms(f), "term.labels")
    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
    as.formula(paste(lhs_str, "~", rhs_str,
                     "+ strata(fold) + cluster(", id, ")"))
  }
  predict_cvS <- function(fit, newdata, se = FALSE) {
    as.numeric(predict(fit, newdata, se = se))
  }
  ## ------------------------------------------------------------------
  ## cv_iid_correctS -- block-sparse version (identical to brier_binregStrata)
  ## ------------------------------------------------------------------

cv_iid_correctS <- function(brier_fit, outff_fit, newdata,
                               pred_vec, ipcw_wt_vec, outcome_vec,
                               binreg_model) { ## {{{ 
    nstrata <- outff_fit$nstrata
    p       <- length(outff_fit$coef) / nstrata
    n       <- length(pred_vec)
    des        <- update_design(outff_fit$design, data = newdata,
                                       response = FALSE)
    X          <- des$x
    strata_idx <- if (!is.null(des$strata)) {
      as.integer(des$strata) - 1L
    } else if ("fold" %in% names(newdata)) {
      as.integer(newdata$fold) - 1L
    } else {
      stop("Cannot determine stratum for newdata: needs 'fold' column.")
    }
###    Xs <- matrix(0.0, n, nstrata * p)
###    for (s in seq_len(nstrata)) {
###      idx  <- which(strata_idx == (s - 1L))
###      cols <- (s - 1L) * p + seq_len(p)
###      Xs[idx, cols] <- X[idx, , drop = FALSE]
###    }
    mu_prime <- switch(binreg_model,
      logit = pred_vec * (1 - pred_vec),
      exp   = pred_vec,
      lin   = rep(1.0, n),
      stop("unrecognised binreg model '", binreg_model, "'")
    )
    resid       <- (outcome_vec - pred_vec) * ipcw_wt_vec

    Dbrier <- -2* Msumstrata(X * mu_prime * resid,strata_idx,nstrata)/n
    Dbrier_full  <-  c(t(Dbrier))

###   Dbrier_full <- -2 * colSums(Xs * mu_prime * resid) / n
###   print(Dbrier_full)

    iid(brier_fit) + iid(outff_fit) %*% Dbrier_full
  } ## }}} 

  eco <- function(bbrier) {
    bbrier$design <- NULL
    bbrier$cens.weights <- bbrier$cens.strata <- bbrier$MGciid <- bbrier$call.id
    bbrier$name.id <- bbrier$iid.naive  <- bbrier$id <- NULL
    bbrier$design$y <- NULL
    bbrier$design$data <- NULL
    bbrier$design$x <- NULL
    bbrier$design$strata <- NULL
    return(bbrier)
  }

  ## ------------------------------------------------------------------
  ## resmeanIPCW wrapper / log-scale estimate helper
  ## ------------------------------------------------------------------
  do_resmean <- function(dat, Ydirect, tt) {
    do.call(resmeanIPCW,
            c(list(base_resmean, dat, time = tt,
                   Ydirect    = Ydirect,
                   cens.model = brier.cens.model,
                   model      = "lin"),
              brier.args))
  }
  log_est <- function(fit, iid_corrected = NULL) {
    if (!is.null(iid_corrected)) {
      nn <- nrow(iid_corrected)
      e <- lava::estimate(NULL,
                          coef = fit$coef,
                          IC   = iid_corrected * nn,
                          f    = log)
    } else {
      e <- estimate(fit, f = log)
    }
    list(coef = as.numeric(e$coef),
         se   = as.numeric(diag(vcov(e))^.5),
         iid  = iid(e))
  }

  ## ------------------------------------------------------------------
  ## Explicit nested for-loops: outer over time, inner over rhs models
  ## ------------------------------------------------------------------
  cl       <- match.call()
  ntime    <- length(time)
  nmodels  <- length(rhs)
  results  <- vector("list", ntime)
  names(results) <- as.character(time)

  for (ti in seq_len(ntime)) {
    tt <- time[ti]

    ## ---------------- null model: apparent ----------------
    outb         <- binreg(build_full(rhs0), data, time = tt, ...)
    binreg_model <- outb$model
    outcome      <- as.numeric(outb$Y)
    ipcw_wt_b    <- as.numeric(outb$IPCW)
    predb        <- predict(outb, data, se = FALSE)
    data$brier_  <- (outcome - predb)^2
    bbrier       <- do_resmean(data, data$brier_, tt)
    bbrier <- eco(bbrier)
    bbrier_log   <- log_est(bbrier)

    ## ---------------- null model: CV ----------------
    f0_strata  <- build_strata_formula(rhs0)
    outfb      <- binregStrata(f0_strata, data = data, time = tt,
                                strata     = strata_vec,
                                cens.model = cens.model,
                                cv.fold    = TRUE, low.memory=TRUE,...)
    predbcv     <- predict_cvS(outfb, data)
    data$brier_ <- (outcome - predbcv)^2
    bbrierCV    <- do_resmean(data, data$brier_, tt)
    bbrierCV    <- eco(bbrierCV)
    iid.bbrierCV <- cv_iid_correctS(bbrierCV, outfb, data,
                                     predbcv, ipcw_wt_b, outcome,
                                     binreg_model)
    outfb$iid <- NULL
    bbrierCV_log <- log_est(bbrierCV, iid_corrected = iid.bbrierCV)

    null_res <- list(
      coef = c(bbrier   = bbrier_log$coef,
               bbrierCV = bbrierCV_log$coef),
      se   = c(bbrier   = bbrier_log$se,
               bbrierCV = bbrierCV_log$se),
      iid  = list(bbrierCV = bbrierCV_log$iid)
    )

    ## ---------------- candidate models: explicit for-loop ----------------
    model_res        <- vector("list", nmodels)
    names(model_res) <- names(rhs)

    for (mi in seq_len(nmodels)) {
      f <- rhs[[mi]]

      ## apparent
      out         <- binreg(build_full(f), data, time = tt, ...)
      outcome_m   <- as.numeric(out$Y)
      ipcw_wt_m   <- as.numeric(out$IPCW)
      pred        <- predict(out, data, se = FALSE)
      data$brier_ <- (outcome_m - pred)^2
      out         <- NULL
      brier       <- do_resmean(data, data$brier_, tt)
      brier       <- eco(brier)
      brier_log   <- log_est(brier)
      dbrier      <- brier_log$coef - bbrier_log$coef
      se.dbrier   <- as.numeric(crossprod(brier_log$iid - bbrier_log$iid)^.5)

      ## CV
      f_strata <- build_strata_formula(f)
      outff    <- binregStrata(f_strata, data = data, time = tt,
                                strata     = strata_vec,
                                cens.model = cens.model,
                                cv.fold    = TRUE, low.memory=TRUE,...)
      pred        <- predict_cvS(outff, data)
      data$brier_ <- (outcome_m - pred)^2
      brierCV     <- do_resmean(data, data$brier_, tt)
      brierCV     <- eco(brierCV)
      iid.brierCV <- cv_iid_correctS(brierCV, outff, data,
                                      pred, ipcw_wt_m, outcome_m, binreg_model)
      outff        <- NULL
      brierCV_log <- log_est(brierCV, iid_corrected = iid.brierCV)
      dbrierCV    <- brierCV_log$coef - bbrierCV_log$coef
      se.dbrierCV <- as.numeric(crossprod(brierCV_log$iid - bbrierCV_log$iid)^.5)

      model_res[[mi]] <- list(
        coef  = c(brier     = brier_log$coef,
                  brierCV   = brierCV_log$coef),
        se    = c(brier     = brier_log$se,
                  brierCV   = brierCV_log$se),
        dcoef = c(dbrier    = dbrier,
                  dbrierCV  = dbrierCV),
        sed   = c(se.dbrier   = se.dbrier,
                  se.dbrierCV = se.dbrierCV),
        iid   = list(brierCV = brierCV_log$iid)
      )
    } ## end inner for over models

    ## iid matrix: n x (1 + nmodels)
    iid_cols <- vector("list", nmodels + 1L)
    iid_cols[[1]] <- bbrierCV_log$iid
    bbrierCV_log$iid <- NULL
    for (mi in seq_len(nmodels)) {
	    iid_cols[[mi + 1L]] <- model_res[[mi]]$iid$brierCV
	    model_res[[mi]]$iid$brierCV <- NULL
    }
    iid_mat <- do.call(cbind, iid_cols)
    colnames(iid_mat) <- c("null", names(rhs))

    results[[ti]] <- structure(
      list(null             = null_res,
           models           = model_res,
           iid_mat          = iid_mat,
           cv.split.index   = dd,
           time             = tt,
           fold             = fold,
           rhs              = rhs,
           rhs0             = rhs0,
           model            = binreg_model,
           cens.model       = cens.model,
           brier.cens.model = brier.cens.model),
      class = "binregCV"
    )
  } ## end outer for over time

  ## ------------------------------------------------------------------
  ## Dispatch: single or multiple time points
  ## ------------------------------------------------------------------
  if (ntime == 1L) {
    out      <- results[[1]]
    out$call <- cl
    out
  } else {
    res        <- results
    res$call   <- cl
    res$rhs    <- rhs
    res$rhs0   <- rhs0
    class(res) <- "binregCV_list"
    res
  }
} ## }}}

## {{{  infrastructure for brier_binreg
## =============================================================================
## estimate.binregCV  -- lava-style joint estimate  (null + all CV models)
## =============================================================================



##' Joint estimate across CV models for a binregCV object
##'
##' @param x A `binregCV` object.
##' @param f Optional transformation function passed to `lava::estimate`.
##' @param ... Not currently used.
##' @export
estimate.binregCV <- function(x , f = NULL, ...) {
  coefs <- c(
    null = x$null$coef["bbrierCV"],
    setNames(
      sapply(x$models, function(m) m$coef["brierCV"]),
      names(x$models)
    )
  )
  iid_mat <- x$iid_mat   # n x (1 + nmodels)
  n       <- nrow(iid_mat)
  e       <- lava::estimate(NULL, coef = coefs, IC = iid_mat * n)
  if (!is.null(f)) e <- estimate(e, f = f)
  e
}


## =============================================================================
## summarize_brier_binregCV  (works for binregCV and binregCV_list)
## =============================================================================
summarize_brier_binregCV <- function(fit,
                                      transform  = NULL,
                                      conf.level = 0.95,
                                      type       = c("cv", "both")) {
  type <- match.arg(type)

  fit_call <- fit$call
  fit_rhs  <- fit$rhs
  fit_rhs0 <- fit$rhs0

  if (inherits(fit, "binregCV")) {
    tname    <- as.character(fit$time)
    fit_call <- fit$call
    fit_rhs  <- fit$rhs
    fit_rhs0 <- fit$rhs0
    fit      <- setNames(list(fit), tname)
  }

  times <- names(fit)[!(names(fit) %in% c("call", "rhs", "rhs0"))]
  if (length(times) == 0L) stop("No time points found in fit")

  pdiff <- lava::pairwise.diff

  col_se <- function(cm) grep("^Std", colnames(cm), value = TRUE)[1]
  col_lo <- function(cm) grep("%",    colnames(cm), value = TRUE)[1]
  col_hi <- function(cm) grep("%",    colnames(cm), value = TRUE)[2]
  col_p  <- function(cm) grep("^P",   colnames(cm), value = TRUE)[1]

  parse_lava_contrast <- function(rn) {
    m    <- gregexpr("(?<=\\[)[^\\]]+(?=\\])", rn, perl = TRUE)
    hits <- regmatches(rn, m)[[1]]
    c(reference = hits[1], model = hits[2])
  }
  strip_t <- function(x) sub("^t[^.]+\\.", "", x)

  coefmat_df <- function(e, time_val) {
    cm <- e$coefmat
    data.frame(
      time  = as.numeric(time_val),
      model = names(coef(e)),
      Brier = as.numeric(cm[, "Estimate"]),
      SE    = as.numeric(cm[, col_se(cm)]),
      lower = as.numeric(cm[, col_lo(cm)]),
      upper = as.numeric(cm[, col_hi(cm)]),
      stringsAsFactors = FALSE
    )
  }

  apparent_coefmat_df <- function(obj, time_val, tfun, cl) {
    z <- qnorm(1 - (1 - cl) / 2)
    coefs <- c(
      null = obj$null$coef["bbrier"],
      setNames(
        sapply(names(obj$models), function(nm) obj$models[[nm]]$coef["brier"]),
        names(obj$models)
      )
    )
    ses <- c(
      null = obj$null$se["bbrier"],
      setNames(
        sapply(names(obj$models), function(nm) obj$models[[nm]]$se["brier"]),
        names(obj$models)
      )
    )
    lower_log <- coefs - z * ses
    upper_log <- coefs + z * ses
    if (!is.null(tfun)) {
      eps    <- sqrt(.Machine$double.eps)
      fprime <- (tfun(coefs + eps) - tfun(coefs - eps)) / (2 * eps)
      data.frame(time  = as.numeric(time_val),
                 model = names(coefs),
                 Brier = as.numeric(tfun(coefs)),
                 SE    = as.numeric(abs(fprime) * ses),
                 lower = as.numeric(tfun(lower_log)),
                 upper = as.numeric(tfun(upper_log)),
                 stringsAsFactors = FALSE)
    } else {
      data.frame(time  = as.numeric(time_val),
                 model = names(coefs),
                 Brier = as.numeric(coefs),
                 SE    = as.numeric(ses),
                 lower = as.numeric(lower_log),
                 upper = as.numeric(upper_log),
                 stringsAsFactors = FALSE)
    }
  }

  models_list          <- vector("list", length(times))
  comps_list           <- vector("list", length(times))
  apparent_models_list <- vector("list", length(times))
  names(models_list) <- names(comps_list) <-
    names(apparent_models_list) <- times

  for (tt in times) {
    obj <- fit[[tt]]

    e <- if (is.null(transform)) estimate(obj)
         else                    estimate(obj, f = transform)
    nmodels <- length(coef(e))
    models_list[[tt]] <- coefmat_df(e, tt)

    if (nmodels > 1L) {
      ec  <- estimate(e, -pdiff(nmodels))
      cmc <- ec$coefmat
      rn  <- rownames(cmc)
      parsed <- t(sapply(rn, parse_lava_contrast))
      comps_list[[tt]] <- data.frame(
        time        = as.numeric(tt),
        model       = strip_t(parsed[, "model"]),
        reference   = strip_t(parsed[, "reference"]),
        delta.Brier = as.numeric(cmc[, "Estimate"]),
        SE          = as.numeric(cmc[, col_se(cmc)]),
        lower       = as.numeric(cmc[, col_lo(cmc)]),
        upper       = as.numeric(cmc[, col_hi(cmc)]),
        P           = as.numeric(cmc[, col_p(cmc)]),
        stringsAsFactors = FALSE
      )
    }

    if (type == "both")
      apparent_models_list[[tt]] <- apparent_coefmat_df(obj, tt, transform, conf.level)
  }

  obj1       <- fit[[times[1]]]
  fit_fold   <- obj1$fold
  if (is.null(fit_rhs))  fit_rhs  <- obj1$rhs
  if (is.null(fit_rhs0)) fit_rhs0 <- obj1$rhs0
  model_list <- c(
    list(`null (rhs0)` = deparse(fit_rhs0)),
    setNames(lapply(fit_rhs, function(f) deparse(f[[length(f)]])),
             names(fit_rhs))
  )

  models_df <- do.call(rbind, models_list)
  comps_df  <- do.call(rbind, Filter(Negate(is.null), comps_list))
  if (is.null(comps_df)) comps_df <- data.frame()
  rownames(models_df) <- NULL
  if (nrow(comps_df) > 0) rownames(comps_df) <- NULL

  apparent_models_df <- if (type == "both")
    do.call(rbind, apparent_models_list) else NULL

  structure(
    list(models               = models_df,
         comparisons          = comps_df,
         apparent.models      = apparent_models_df,
         apparent.comparisons = NULL,
         type                 = type,
         transform            = transform,
         conf.level           = conf.level,
         times                = as.numeric(times),
         call                 = fit_call,
         fold                 = fit_fold,
         model_list           = model_list),
    class = "summary_binregCV_multi"
  )
}


## =============================================================================
## summary methods
## =============================================================================

##' Summary method for binregCV objects
##'
##' @param object A \code{binregCV} object, as returned by
##'   \code{\link{brier_binreg}} for a single evaluation time.
##' @param transform Optional back-transformation function applied to Brier
##'   scores and confidence limits, e.g. \code{exp} to undo the internal log
##'   scale.
##' @param conf.level Confidence level for the reported intervals (default
##'   0.95).
##' @param type Either \code{"cv"} (default) for cross-validated Brier scores
##'   only, or \code{"both"} to also include apparent (in-sample) scores.
##' @param ... Not currently used.
##' @aliases print.binregCV
##' @export
summary.binregCV <- function(object, transform = NULL,
                              conf.level = 0.95,
                              type = c("cv", "both"), ...) {
  summarize_brier_binregCV(object,
                            transform  = transform,
                            conf.level = conf.level,
                            type       = match.arg(type))
}

##' @export
print.binregCV <- function(x, transform = NULL, 
                              conf.level = 0.95, type = c("cv", "both"), ...) { ## {{{ 
  print(summarize_brier_binregCV(x,
                            transform  = transform,
                            conf.level = conf.level,
                            type       = match.arg(type)))
} ## }}} 


##' Summary method for binregCV_list objects
##'
##' @param object A \code{binregCV_list} object, as returned by
##'   \code{\link{brier_binreg}} when \code{time} has length greater than
##'   one.
##' @param transform Optional back-transformation function applied to Brier
##'   scores and confidence limits, e.g. \code{exp} to undo the internal log
##'   scale.
##' @param conf.level Confidence level for the reported intervals (default
##'   0.95).
##' @param type Either \code{"cv"} (default) for cross-validated Brier scores
##'   only, or \code{"both"} to also include apparent (in-sample) scores.
##' @param ... Not currently used.
##' @export
summary.binregCV_list <- function(object, transform = NULL,
                                   conf.level = 0.95,
                                   type = c("cv", "both"), ...) {
  summarize_brier_binregCV(object,
                            transform  = transform,
                            conf.level = conf.level,
                            type       = match.arg(type))
}


## =============================================================================
## print.summary_binregCV_multi
## =============================================================================

##' Print method for summary_binregCV_multi objects
##'
##' @param x A `summary_binregCV_multi` object.
##' @param digits Number of digits to round to.
##' @param ... Not currently used.
##' @export
print.summary_binregCV_multi <- function(x, digits = 4, ...) {
  sep_line <- paste(rep("-", 56), collapse = "")

  rd <- function(df) {
    num_cols <- sapply(df, is.numeric)
    df[num_cols] <- lapply(df[num_cols], round, digits)
    df
  }

  scale_note <- if (is.null(x$transform)) "[log scale]" else "[transformed scale]"
  cat(sprintf("\nCross-validated Brier score summary  %s\n", scale_note))
  if (!is.null(x$call)) { cat("Call: "); print(x$call) }
  cat(sprintf("Evaluation time(s): %s   Folds: %d   CI level: %g%%\n",
              paste(x$times, collapse = ", "), x$fold, 100 * x$conf.level))
  cat("Estimator: binregStrata  |  IID: block-diagonal delta-method correction\n")

  cat("\nModels:\n")
  for (nm in names(x$model_list)) {
    rhs_str <- trimws(gsub("- 1", "", gsub("\\s+", " ", x$model_list[[nm]])))
    cat(sprintf("  %-18s : %s\n", nm, rhs_str))
  }
  cat("\n")

  if (!is.null(x$type) && x$type == "both" && !is.null(x$apparent.models)) {
    cat("Apparent (no CV) Brier scores:\n\n")
    disp_app <- rd(x$apparent.models)[, c("model", "time", "Brier", "SE",
                                           "lower", "upper")]
    for (tt in unique(disp_app$time)) {
      block      <- disp_app[disp_app$time == tt, , drop = FALSE]
      block$time <- NULL
      cat(sprintf("  time = %g\n", tt))
      print(block, row.names = FALSE, quote = FALSE)
      cat(sep_line, "\n")
    }
    cat("Note: apparent comparisons not shown (IID not stored for apparent fits).\n\n")
  }

  cat("CV Brier scores by model:\n\n")
  disp_mod <- rd(x$models)[, c("model", "time", "Brier", "SE", "lower", "upper")]
  for (tt in unique(disp_mod$time)) {
    block      <- disp_mod[disp_mod$time == tt, , drop = FALSE]
    block$time <- NULL
    cat(sprintf("  time = %g\n", tt))
    print(block, row.names = FALSE, quote = FALSE)
    cat(sep_line, "\n")
  }

  if (!is.null(x$comparisons) && nrow(x$comparisons) > 0L) {
    cat("\nModel comparisons (delta CV Brier):\n\n")
    comps <- rd(x$comparisons)
    for (tt in unique(comps$time)) {
      block <- comps[comps$time == tt, , drop = FALSE]
      cat(sprintf("  time = %g\n", tt))
      print(block, row.names = FALSE, quote = FALSE)
      cat(sep_line, "\n")
    }
  }

  if (is.null(x$transform))
    cat("\nNote: use summary(fit, transform = exp) for Brier scores on the original scale.\n")

  invisible(x)
}

## }}} 

## =============================================================================
## estimate.binregCV_list -- joint estimate across ALL time points and models
## =============================================================================
##' @export
estimate.binregCV_list <- function(x, f = NULL, ...) { ## {{{ 
  times <- names(x)[!(names(x) %in% c("call", "rhs", "rhs0"))]
  if (length(times) == 0L) stop("No time points found in object")

  ## Collect per-time coefficient vectors and iid matrices, with
  ## time-qualified names so e.g. "null" at t=50 and t=60 don't collide.
  coef_list <- vector("list", length(times))
  iid_list  <- vector("list", length(times))
  names(coef_list) <- names(iid_list) <- times

  n_ref <- NULL
  for (tt in times) {
    obj <- x[[tt]]
    coefs <- c(
      null = obj$null$coef["bbrierCV"],
      setNames(
        sapply(obj$models, function(m) m$coef["brierCV"]),
        names(obj$models)
      )
    )
    nm <- paste0("t", tt, ".", names(coefs))
    names(coefs) <- nm

    im <- obj$iid_mat                 ## n x (1+nmodels), cols: null, model1, ...
    colnames(im) <- nm

    if (is.null(n_ref)) {
      n_ref <- nrow(im)
    } else if (nrow(im) != n_ref) {
      stop("iid_mat row counts differ across time points (", n_ref,
           " vs ", nrow(im), " at time ", tt, "); cannot combine into a ",
           "single joint estimate. This typically means rows correspond ",
           "to different clusters/ids across time points.")
    }

    coef_list[[tt]] <- coefs
    iid_list[[tt]]  <- im
  }

  coefs_all   <- do.call(c, coef_list)
  iid_mat_all <- do.call(cbind, iid_list)   ## n x (ntimes * (1+nmodels))

  n <- nrow(iid_mat_all)
  e <- lava::estimate(NULL, coef = coefs_all, IC = iid_mat_all * n)
  if (!is.null(f)) e <- estimate(e, f = f)
  e
} ## }}} 

##' Two-Stage Randomization for Survival or Competing Risks Data
##'
##' Estimates the average treatment effect \eqn{E(Y(i,j))} of treatment regime \eqn{(i,j)} 
##' under two-stage randomization. The estimator can be augmented using information from 
##' both randomizations and dynamic censoring augmentation to improve efficiency.
##' 
##' The method solves the estimating equation:
##' \deqn{ \frac{I(\min(T_i,t) < G_i)}{G_c(\min(T_i,t))} I(T \leq t, \epsilon=1) - AUG_0 - AUG_1 + AUG_C - p(i,j) = 0 }
##' where:
##' \itemize{
##'   \item \eqn{AUG_0 = \frac{A_0(i) - \pi_0(i)}{\pi_0(i)} X_0 \gamma_0} uses covariates from \code{augmentR0}
##'   \item \eqn{AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{\pi_1(j)} X_1 \gamma_1} uses covariates from \code{augmentR1}
##'   \item \eqn{AUG_C = \int_0^t \gamma_c(s)^T (e(s) - \bar e(s)) \frac{1}{G_c(s)} dM_c(s)} is the censoring augmentation
##' }
##' 
##' Standard errors are estimated using the influence function of all estimators, enabling 
##' tests of differences to be computed subsequently. The method handles both survival 
##' data and competing risks data, and supports multiple treatment levels.
##'
##' @param formula Formula with outcome (see \code{coxph}), typically \code{Event(entry,time,status)~+1+cluster(id)}.
##' @param data Data frame containing all variables.
##' @param cause Cause of interest for competing risks (default 1).
##' @param time Time point for estimation.
##' @param cens.code Censoring code (default 0).
##' @param beta Starting values for optimization.
##' @param treat.model0 Logistic treatment model for the first randomization.
##' @param treat.model1 Logistic treatment model for the second randomization.
##' @param augmentR0 Covariates for augmentation model of the first randomization.
##' @param augmentR1 Covariates for augmentation model of the second randomization.
##' @param augmentC Covariates for censoring augmentation model.
##' @param cens.model Stratification for censoring model based on observed covariates.
##' @param estpr Logical; estimate randomization probabilities using model (default TRUE).
##' @param response.name Name of response variable (reads from \code{treat.model1} if NULL).
##' @param response.code Code of status indicating response at which 2nd randomization occurs.
##' @param offset Not implemented.
##' @param weights Not implemented.
##' @param cens.weights Can be provided externally.
##' @param kaplan.meier Logical; use Kaplan-Meier for censoring weights rather than exp cumulative hazard.
##' @param no.opt Not implemented.
##' @param method Not implemented.
##' @param augmentation Not implemented.
##' @param outcome Outcome type: \code{"cif"} (cumulative incidence), \code{"rmst"} (restricted mean survival time), 
##'   or \code{"rmst-cause"} (restricted mean time lost for cause).
##' @param model Not implemented, uses linear regression for augmentation.
##' @param Ydirect Use this Y instead of outcome constructed inside the program.
##' @param return.dataw Logical; return weighted data for all treatment regimes.
##' @param pi0 Known randomization probabilities for first randomization.
##' @param pi1 Known randomization probabilities for second randomization.
##' @param cens.time.fixed Logical; use time-dependent weights for censoring estimation.
##' @param outcome.iid Logical; get iid contribution from outcome model.
##' @param meanCs Logical; indicates censoring augmentation is centered by \code{CensAugment.times/n}.
##' @param ... Additional arguments to lower-level functions.
##' @return An object of class \code{"binregTSR"} containing:
##'   \item{riskG}{Simple estimator results (coefficient, SE).}
##'   \item{riskG0}{First randomization augmentation results.}
##'   \item{riskG1}{Second randomization augmentation results.}
##'   \item{riskG01}{Both randomizations augmentation results.}
##'   \item{riskG.iid}{Influence functions for all estimators.}
##'   \item{varG}{Variance-covariance matrices.}
##'   \item{MGc}{Censoring martingale contributions.}
##'   \item{CensAugment.times}{Censoring augmentation terms.}
##'   \item{dynCens.coef}{Dynamic censoring coefficients.}
##'   \item{dataW}{Weighted data (if \code{return.dataw=TRUE}).}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. H. (2024). Two-stage randomization analysis for survival data. mets package documentation.
##' @seealso \code{\link{binreg}}, \code{\link{phreg_rct}}
##' @examples
##' ddf <- mets:::gsim(200,covs=1,null=0,cens=1,ce=2)
##' 
##' bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),ddf$datat,time=2,cause=c(1),
##'         cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
##'         augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
##'         augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
##'         response.code=2)
##' summary(bb) 
##' @export
binregTSR <- function(formula,data,cause=1,time=NULL,cens.code=0,
    response.code=NULL,
    augmentR0=NULL,treat.model0=~+1,augmentR1=NULL,treat.model1=~+1, 
    augmentC=NULL,cens.model=~+1,estpr=c(1,1),response.name=NULL,
    offset=NULL,weights=NULL,cens.weights=NULL,beta=NULL,
    kaplan.meier=TRUE,no.opt=FALSE,method="nr",augmentation=NULL,
    outcome=c("cif","rmst","rmst-cause"),model="exp",Ydirect=NULL,
    return.dataw=0,pi0=0.5,pi1=0.5,cens.time.fixed=1,outcome.iid=1,meanCs=0,...)
{# {{{
  cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

# }}}

  if (is.null(time)) stop("Must give time for construction of survival outome modelling \n"); 
  statusC <- (status %in%cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (is.null(response.code)) stop("Give codes for 2nd treatment start (response), 2nd randomization time \n"); 
  statusR <- (status %in% response.code) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  if (sum(statusR)==0) stop("No Responses before time \n"); 
  kmt <- kaplan.meier

  response__  <-  rid__  <- Count1__ <- NULL

  data$status__ <-  status 
  data$id__ <-  id+1
  data$Count1__ <- cumsumstrata(rep(1,length(entry)),id,nid)
  data$entry__ <- entry 
  data$exit__ <- exit 
  data$statusC__ <- statusC
  data$status__cause <- statusE
  data$rid__ <- revcumsumstrata(rep(1,length(entry)),id,nid)
  csr <- cumsumstratasum(statusR,id,nid) 
  data$response__  <-  csr$sum 
  data$response__lag  <-  csr$lagsum
  cens.strata <- cens.nstrata <- NULL 
  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
###   kmc <- km(formC,data)
  } else formC <- NULL


  ## adjust censoring model according to treat1
  treat.name1 <-  all.vars(treat.model1)[1]
  if (treat.name1 %in% all.vars(formC))  {
	  ## drop treat.name1 from strata, we handle this via the weights
	  ## possibly extend to additional covariates 
	  vv <- all.vars(cens.model)
	  vv <- vv[-grep(treat.name1,vv)]
	  cens.model <- as.formula( paste("~strata(",vv,")"))
	  treat.specific.cens <- 1
  }  else treat.specific.cens <- 0
###  print(cens.model)

  MG.se <- (sum(data$statusC__ & (data$exit__ < time))>0)*1

  ################################################################
  ### iid data consisting of last record, ordered after id   #####
  ################################################################
  id <- id[data$rid__==1]
  oid <- order(id)
  dataiid <- data[data$rid__==1,][oid,]
###  Xorig <- X <- as.matrix(X)
###  Xdata <- X <- X[data$rid__==1,,drop=FALSE][oid,]
  offset <- offset[data$rid__==1][oid]
  weights <- weights[data$rid__==1][oid]
  status <- status[data$rid__==1][oid]
  exit <- exit[data$rid__==1][oid]
###  X2 <- .Call("vecMatMat", X, X)$vXZ
  ph <- 1
  if (is.null(beta)) beta <- rep(0,ncol(X))
  ## take iid vession of data 
  cens.weights <- cens.weights[data$rid__==1][oid]
  response <- (data$response__[data$rid__==1] >0)[oid]
  ### also order Ydirect, check, check
  if (!is.null(Ydirect)) Ydirect <- Ydirect[oid]


  if (is.null(beta)) beta <- rep(0,ncol(X))
###  p <- ncol(X)
###  X <-  as.matrix(X)
###  X2  <- .Call("vecCPMat",X)$XX
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)>=1) Causes <- ucauses[-ccc] else Causes <- ucauses
  obs <- (exit<=time & (status %in% Causes)) | (exit>=time)
  p <- 1

  if (!is.null(Ydirect)) Y <-  Ydirect*obs else {
    if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
    else if (outcome[1]=="rmst") Y <-  c(pmin(exit,time)*obs) 
    else if (outcome[1]=="rmst-cause") Y <- c((status %in% cause)*(time-pmin(exit,time))*obs)
    else stop("outcome not defined") 
  }
  Yorig <- Y
  Y <- Y/cens.weights
  dataiid$Y__ <- Y

 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum((status %in% cause)*(exit<=time))

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

## fitting treatment models for 1st and 2nd randomization 
fittreat <- function(treat.model,data,id,ntreatvar,nlev) {# {{{
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2

out <- list(iidalpha=iidalpha,pA=pA,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

dataR0 <- dataiid
## response==1 data
dataR1 <- subset(dataiid,response__==1)

### treatment is rhs of treat.model 
treat.name0 <-  all.vars(treat.model0)[1]
treatvar0 <- dataR0[,treat.name0]
## make into factor
if (!is.factor(treatvar0))  treatvar0 <- as.factor(treatvar0) 
## treatvar, 1,2,...,nlev or 1,2
treats0 <- treats(treatvar0)

## fit treatment model for first randomization
idR0 <-  id[data$Count1__==1] ## first record with first randomization info
idR0 <- dataiid[,"id__"]
if (estpr[1]==1) {
   fitt0 <- fittreat(treat.model0,dataR0,idR0,treats0$ntreatvar,treats0$nlev)
   iidalpha0 <- fitt0$iidalpha
} else {
   pi0 <- rep(pi0,treats0$nlev)
}

###
treat.name1 <-  all.vars(treat.model1)[1]
## only  look among those randomized 
treatvar1 <- dataR1[,treat.name1]
###treatvar1 <- dataiid[,treat.name1]
if (!is.factor(treatvar1))  treatvar1 <- factor(treatvar1) 

## fit treatment model for second randomization   using combined model
treats1 <- treats(treatvar1)
###idR1 <- id[data$response__==1 & data$rid__==1] ## first record with first randomization info
idR1 <- dataR1[,"id__"]
dataR1[,treat.name1] <- treatvar1

if (estpr[2]==1) {
fit1 <- fittreat(treat.model1,dataR1,idR1,treats1$ntreatvar,treats1$nlev)
##### put iid in matrix after nid 
iidalpha1 <- apply(fit1$iidalpha,2,sumstrata,sort(idR1)-1,nid) ## *(nrow(dataR1)/nid)
}

## read if there are multiple responses recorded and read name of variable, and levels for different responses 
if (length(all.vars(treat.model1)) >=3 & is.null(response.name)) response.name <- all.vars(treat.model1)[3]
if (!is.null(response.name)) {
respvar <- as.factor(dataR1[,response.name])
if (!is.factor(respvar)) stop(paste("Response=",respvar," must be coded as factor \n",sep="")); 
## treatvar, 1,2,...,nlev or 1,2
## drop possible "0" empty level among dataR1 with responses
respvar <- droplevels(respvar)
nlevresp <- nlevels(respvar)
nlevsr <- levels(respvar)
nrespvar <- as.numeric(respvar)
} else {
nlevresp <- 1 
nlevsr <- "Response"
nrespvar <- rep(1 ,nrow(dataR1))
response.name <- "response"
dataiid[,response.name] <- 1
}

if (estpr[2]==0) {
	mpi1 <- matrix(0,nlevresp,treats1$nlev)
	if (length(pi1)!=treats1$nlev) pi1 <- rep(pi1,treats1$nlev)
	for (i in seq(nlevresp)) mpi1[i,] <- pi1[i]
	pi1 <- mpi1
}

## to fit stratified models 
treat.model1s <- update.formula(treat.model1,~+1)
### stratify on A0.f 
treat.model1s <- update.formula(treat.model1s,as.formula(paste("~",treat.name0)) )
treat.model1s <- treat.model1
###print(treat.model1s)

## set up stratified models for response
tv1 <- list(); fitt1 <- list()
combof <- combo <- NULL
for (nnresp in seq(nlevresp)) {
   k <- nnresp
   dataresp <- subset(dataR1,nrespvar==nnresp)
   tv1[[k]] <- treats(treatvar1[nrespvar==nnresp])
   if (k==1)  {combof <-  tv1[[k]]$nlevs; 
               combo <- seq(tv1[[k]]$nlev) }
   else {
	   combof <- expand.grid(combof,tv1[[k]]$nlevs);
	   combo <- expand.grid(combo,seq(tv1[[k]]$nlev))
   }
###   idresp <- idR1[nrespvar==nnresp]
###   fitt1[[k]] <- fittreat(treat.model1s,dataresp,idresp,tv1[[k]]$ntreatvar,tv1[[k]]$nlev)
}
if (nlevresp==1) { combof <- expand.grid(combof); combo <- expand.grid(combo) }

A0 <- as.numeric(dataiid[,treat.name0])
A1 <- as.numeric(dataiid[,treat.name1])
## code possible NA as 0, to avoid problems
A1[is.na(A1)] <- 0

llR0 <- llR <- llR1 <- llR01 <- c()
riskG0 <- riskG <- riskG1 <- riskG01 <- c()
riskG0C <- riskGC <- riskG1C <- riskG01C <- c()
riskG0.iid <- riskG.iid <- riskG1.iid <- riskG01.iid <- c()
riskG0C.iid <- riskGC.iid <- riskG1C.iid <- riskG01C.iid <- c()
nc <- nrow(combo)
MGc <-  Augment <- Augment.times <- Ht <- dataW <- DA1 <- rnames <- c() 
dynCgammat <- list()
cown <- 0


### looping over A0.f response*A1.f
for (i in seq(treats0$nlev)) { ## {{{
for (v in seq(nc))  {
	cown <- cown+1
        namesiv <- paste( paste(treat.name0,"=",treats0$nlevs[i],", ",sep=""),
         	      paste(response.name,"*",treat.name1,"=",
	        	    paste(combof[v,],collapse=","),sep="",collapse=""),sep="")
        rnames <- c(rnames,namesiv)
	## first only one response
	k <- 1
	j <- combo[v,]
	dataij <- dataiid
	pA1j <- rep(1,nrow(dataij))
	responsetype <- as.numeric(as.factor(dataiid[,response.name]))
	if (estpr[1]==1) pA0i <- fitt0$pA else { pA0i <- 0.5; pA0i[i] <- pi0[i]; }
	if (estpr[2]==1) pA1j[fit1$id] <- fit1$pA else pA1j <-c(mdi(pi1,responsetype,A1))

	A1j <- (apply(outer(A1,j,FUN="==")*outer(responsetype,seq(nlevresp),FUN="==") ,1,sum) >=1)
	W <- ((A0==i)/pA0i)*((response==0)+(response!=0)*(A1j)/pA1j)
	W1 <- ((response==0)+(response!=0)*(A1j)/pA1j)
	###### weights for second randomization based on known probabilities
	W1k <- ((response==0)+(response!=0)*(A1j)/pA1j)

       ### possibly estimate censoring weights depending on A0.f and A1.f, with weights and construct new Y 
       ### using data with time-dependent weights for second treatment only 
	if (treat.specific.cens==1) {
            data$WW1__ <- 1
	    data$WW1__[data$response__lag>=1] <- W1k[data$id__][data$response__lag>=1]
            ## use time-fixed weights instead,  
	    if (cens.time.fixed==1) data$WW1__  <-  W1k[data$id__]
            formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
            formC <- update.formula(formC,.~.+status__) 
###	    print(formC)
            resC <- phreg(formC,data,weights=data$WW1__,no.opt=TRUE,no.var=1)
            if (resC$p>0) kmt <- FALSE
            exittime <- pmin(data$exit__,time)
            cens.weights <-c(suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv))
	    ### where the weights are 0, cens.weights are 0, and do not need them there 
	    cens.weights[data$WW1__<=0.00001]  <- 1
            ## strata from original data 
            cens.strata <- resC$strata[order(resC$ord)]
            cens.nstrata <- resC$nstrata
	    ## construct new IPCW with weights from KM, ordered
	    cens.weights <- cens.weights[data$rid__==1][oid]
	    Y <- Yorig/cens.weights
	}

	## check if there are censorings in data consistent with treatment arm
	## update  to check within treatment arm
        MG.se <- (sum(data$statusC__ & (data$exit__ < time) )>0)*1

	DW0 <- W*pA0i
	dataij$W__ <- W
	dataij$YW__ <- W*Y
	dataij$W0 <- ((A0==i)-pA0i)/pA0i
	dataij$W1 <- ((A0==i)/pA0i)*(response!=0)*((A1j)-pA1j)/pA1j
	dataij$W11 <- ((A0==i)/pA0i)*((A1j))/pA1j
	dataij$A0i <- (A0==i)*1
	dataij$A1j <- A1j
	dataij$i__  <-  i
	dataij$j__  <-  v
	dataij$pA0i__  <- pA0i
	dataij$pA1j__  <- pA1j
	dataij$response <- response 
	if (return.dataw==1) dataW <- rbind(dataW,dataij)

	## simple {{{
	ff <- as.formula(YW__ ~ +1)
        ll0 <- lm(ff,data=dataij)
        sll0 <- estimate(ll0)
	## iid ordered after id i data
	iid0 <- ll0$residuals
	h0 <- DW0*Y
	if (estpr[1]==1) {
        DaPsia <-  apply(fitt0$DPai*h0,2,sum,drop=FALSE)
        iidpala0 <- c(DaPsia %*% t(iidalpha0))
	} else iidpala0 <- 0

	if (estpr[2]==1) {
        ddR1 <- response*(A1j)*Y*(A0==i)/pA0i
        ddR1 <- ddR1[response>=1]
        DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
        iidpala1 <- c(DaPsia %*% t(iidalpha1))
	} else iidpala1 <- 0
        ###
        risk.iid <-  (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1)/nid
        llR <- c(coef(ll0)[1],crossprod(risk.iid)^.5)
        riskG <- rbind(riskG,llR)
        riskG.iid <- cbind(riskG.iid,risk.iid)
        # }}}

	### using 1st stage randomization
	if (!is.null(augmentR0)) {# {{{
              varR0 <- all.vars(augmentR0)
	      newnR0 <- paste(c("int",varR0),"__W0",sep="")
	      newaugR0 <- as.formula(paste("~",paste(c("int",varR0),"__W0",sep="",collapse="+")))
	      ff0 <- update.formula(newaugR0,YW__~.)
	      dataij[,newnR0] <- cbind(1,dataij[,varR0]) * dataij$W0
              ###	      
              llR0 <- lm(ff0,data=dataij)
	      iid0 <- llR0$residuals
              ###
	      if (estpr[1]==1) {
		      h0 <- DW0*Y - as.matrix(cbind(1,dataij[,varR0])) %*% coef(llR0)[-1]
		      DaPsia <-  apply(fitt0$DPai*(A0==i)*c(h0),2,sum,drop=FALSE)
		      iidpala0 <- c(DaPsia %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
		      ddR1 <- (A1j)*Y*(A0==i)/pA0i
		      ddR1 <- ddR1[response>=1]
		      DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
		      iidpala1 <- c(DaPsia %*% t(iidalpha1))
	      } else iidpala1 <- 0

	      if (outcome.iid!=0) {
		      miid <- estimate(llR0) 
		      miid <-  miid$IC[,-1]/nid
		      Dma <-  apply(dataij[,newnR0,drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR0 <- c(coef(llR0)[1],crossprod(risk.iid)^.5)
	      riskG0 <- rbind(riskG0,llR0)
	      riskG0.iid <- cbind(riskG0.iid,risk.iid)
	} # }}}

	### using 2nd stage randomization
	if (!is.null(augmentR1)) {# {{{
              varR1 <- all.vars(augmentR1)
	      newnR1 <- paste(c("int",varR1),"__W1",sep="")
	      newaugR1 <- as.formula(paste("~",paste(c("int",varR1),"__W1",sep="",collapse="+")))
	      ff1 <- update.formula(newaugR1,YW__~.)
	      dataij[,newnR1] <- cbind(1,dataij[,varR1]) * dataij$W1

              llR1 <- lm(ff1,data=dataij)
              sllR1 <- estimate(llR1)
	      iid0 <- llR1$residuals
	      ###   ( W Y -  W1 * h1) 
	      if (estpr[1]==1) {
	      h0 <- (llR1$residuals+coef(llR1)[1])*pA0i
              DaPsia <-  apply(fitt0$DPai*(A0==i)*h0,2,sum,drop=FALSE)
	      iidpala0 <- c(DaPsia %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
	      pred1 <- as.matrix(cbind(1,dataij[,varR1])) %*% coef(llR1)[-1]
	      h1 <- Y - pred1 
	      ddR1 <- (A1j)*h1*(A0==i)/pA0i
	      ddR1 <- ddR1[response>=1]
              DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
	      iidpala1 <- c(DaPsia %*% t(iidalpha1))
	      } else iidpala1 <- 0

	      if (outcome.iid!=0) {
		      miid <-  sllR1$IC[,-1]/nid
		      Dma <-  apply(dataij[,newnR1,drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR1 <- c(coef(llR1)[1],crossprod(risk.iid)^.5)
	      riskG1 <- rbind(riskG1,llR1)
	      riskG1.iid <- cbind(riskG1.iid,risk.iid)
	} # }}}

	### using both randomizations
	if ((!is.null(augmentR0)) & (!is.null(augmentR1)) ) {# {{{
	      ff01 <- update(ff0, reformulate(c(".",newnR1)))
              llR01 <- lm(ff01,data=dataij)
	      iid0 <- llR01$residuals
              ###
              varR01 <- all.vars(ff01)[-1]
	      h00 <- as.matrix(cbind(1,dataij[,varR0])) %*% coef(llR01)[newnR0]
	      h01 <- as.matrix(dataij[,newnR1]*pA0i) %*% coef(llR01)[newnR1]
	      h0 <- DW0*Y - h00-h01
	      h11 <- as.matrix(cbind(1,dataij[,varR1])) %*% coef(llR01)[newnR1]
	      h1 <-  Y - h11
              ###
	      if (estpr[1]==1) {
		      DaPsia0 <-  apply(fitt0$DPai*(A0==i)*c(h0),2,sum,drop=FALSE)
		      iidpala0 <- c(DaPsia0 %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
		      ddR1 <- (A1j)*h1*(A0==i)/pA0i
		      ddR1 <- ddR1[response>=1]
		      DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
		      iidpala1 <- c(DaPsia %*% t(iidalpha1))
     	      } else iidpala1 <- 0
	      if (outcome.iid!=0) {
		      miid <- estimate(llR01) 
		      miid <-  miid$IC[,-1]/nid
		      Dma <-  apply(dataij[,c(newnR0,newnR1),drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR01 <- c(coef(llR01)[1],crossprod(risk.iid)^.5)
	      riskG01 <- rbind(riskG01,llR01)
	      riskG01.iid <- cbind(riskG01.iid,risk.iid)
	} # }}}

 if (MG.se) {## {{{ censoring adjustment of variance using KM (could be stratified)
    ### take possibly weighted cox 
    xx <- resC$cox.prep
    icoxS0 <- rep(0,length(resC$S0))
    icoxS0[resC$S0>0] <- 1/resC$S0[resC$S0>0]
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <-  icoxS0
    S0i2[xx$jumps+1] <- icoxS0^2
    ### set up response using that Y from basic data is ordered after id
    Yt <- Y[xx$id+1]
    Wt <- W[xx$id+1]
    ## fixed weights following Y with known strata
    btime <- 1*(xx$time<time)
    Yt <- Yt*Wt
    ## compute function h(s) = \sum_i I(s \leq T_i \leq t), \int h(s)/Ys  dM_i^C(s) 
    h  <-  revcumsumstrata(Yt*xx$sign,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),1)
    U[xx$jumps+1,] <- (resC$jumptimes<=time)*h[xx$jumps+1,]*icoxS0
    MGt <- U[,drop=FALSE]-IhdLam0*xx$sign*c(xx$weights)
    ### Censoring Variance Adjustment  
    MGCiid <- apply(MGt,2,sumstrata,xx$id,nid)

    riskG.iid[,cown] <-     riskG.iid[,cown]+MGCiid/nid 
    riskG0.iid[,cown] <-   riskG0.iid[,cown]+MGCiid/nid 
    riskG1.iid[,cown] <-   riskG1.iid[,cown]+MGCiid/nid 
    riskG01.iid[,cown] <- riskG01.iid[,cown]+MGCiid/nid 
    MGc <- cbind(MGc,MGCiid/nid) 
    Ht <- cbind(Ht,h) 
  }  else { MGCiid <- 0; MGc <- NULL }
 ## }}}

      ### computing censoring augmentation
      if ((!is.null(augmentC)) & MG.se) {# {{{

	data$YW__ <- Y[data$id__]*W[data$id__]
        varsC <- c("YW__",attr(terms(augmentC), "term.labels"))
        formCC <- update(formC, reformulate(c(".", varsC)))
	www <- rep(1,nrow(data))
	if (treat.specific.cens==1)  www <-data$WW1__;  
        cr2 <- phreg(formCC, data = data, no.opt = TRUE, no.var = 1,weights=www, ...)
        xx <- cr2$cox.prep
        icoxS0 <- rep(0,length(cr2$S0))
        icoxS0[cr2$S0>0] <- 1/cr2$S0[cr2$S0>0]
        S0i <- rep(0, length(xx$strata))
        S0i[xx$jumps + 1] <- icoxS0
	km <- TRUE
        if (!km) {
            cumhazD <- c(cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum)
            St <- exp(-cumhazD)
        } else St <- c(exp(cumsumstratasum(log(1 - S0i), xx$strata, xx$nstrata)$lagsum))

  nterms <- cr2$p-1
  dhessian <- cr2$hessianttime
  dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
  ###  matrix(apply(dhessian,2,sum),3,3)
  timeb <- which(cr2$cumhaz[,1]<time)
  ### take relevant \sum H_i(s,t) (e_i - \bar e)
  covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
  ### construct relevant \sum (e_i - \bar e)^2
  Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
  ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
  gammatt <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
  S0 <- cr2$S0[timeb]
  gammatt[is.na(gammatt)] <- 0
  gammatt[gammatt==Inf] <- 0
  Gctb <- St[cr2$cox.prep$jumps+1][timeb]
  augmentt.times <- apply(gammatt*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum)
   augment.times <- sum(augmentt.times)/nid

   Augment.times <- c(Augment.times,augment.times)
   augmentt <- augment.times 

   #### iid magic  for censoring augmentation martingale ## {{{
   ### int_0^infty gamma(t) (e_i - ebar(s)) 1/G_c(s) dM_i^c
   xx <- cr2$cox.prep
   jumpsC <- xx$jumps+1
   rr0 <- xx$sign
   S0i <- rep(0,length(xx$strata))
   S0i[jumpsC] <- c(1/(icoxS0*St[jumpsC]))
   S0i[jumpsC] <- icoxS0

   pXXA <- ncol(cr2$E)-1
   EA <- cr2$E[timeb,-1]
   gammasEs <- .Call("CubeMattime",gammatt,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   gammasE <- matrix(0,length(xx$strata),1)
   gammattt  <-    matrix(0,length(xx$strata),pXXA*1)
   jumpsCt <- jumpsC[timeb]
   gammasE[jumpsCt,] <- gammasEs
   gammattt[jumpsCt,] <- gammatt
   gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
   gammadLam0 <-   apply(gammattt*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
   XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X[,-1],pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   Ut <- Et <- matrix(0,length(xx$strata),1)
   Ut[jumpsCt,] <- augmentt.times
   MGCt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
   MGCiid <- apply(MGCt,2,sumstrata,xx$id,nid)
   MGc <- list(MGc.ipcw=MGc,MGc.augment=MGCiid)

   riskG[cown,]      <- riskG[cown,]+augmentt
   riskG.iid[,cown]  <- riskG.iid[,cown]+MGCiid/nid-(meanCs==0)*augmentt/nid
   riskG0[cown,]     <- riskG0[cown,]+augmentt
   riskG0.iid[,cown] <- riskG0.iid[,cown]+MGCiid/nid-(meanCs==0)*augmentt/nid
   riskG1[cown,]     <- riskG1[cown,]+augmentt
   riskG1.iid[,cown] <- riskG1.iid[,cown]+MGCiid/nid-(meanCs==0)*augmentt/nid
   riskG01[cown,]    <- riskG01[cown,]+augmentt
   riskG01.iid[,cown]<- riskG01.iid[,cown]+MGCiid/nid-(meanCs==0)*augmentt/nid

   dynCgammat  <-  c(dynCgammat,list(gammatt)) 
   ## }}}

    } ## }}}

} } # }}} loop over A0, response*A1.f

varG <- varG0 <- varG1 <- varG01 <- NULL
rownames(riskG) <-  rnames
colnames(riskG) <-  c("coef","se","se-fixed-Gc")[1:2]

   varG <- crossprod(riskG.iid)
   riskG <- cbind(riskG,diag(varG)^.5)[,c(1,3)]
if ((!is.null(augmentR0))) {
   rownames(riskG0) <- rnames
   colnames(riskG0) <- c("coef","se","se-fixed-Gc")[1:2]
   varG0 <- crossprod(riskG0.iid)
   riskG0 <- cbind(riskG0,diag(varG0)^.5)[,c(1,3)]
} 
if ((!is.null(augmentR1))) {
   rownames(riskG1) <- rnames
   colnames(riskG1) <- c("coef","se","se-fixed-Gc")[1:2]
   varG1 <- crossprod(riskG1.iid)
   riskG1 <- cbind(riskG1,diag(varG1)^.5)[,c(1,3)]
} 
if ((!is.null(augmentR0)) & (!is.null(augmentR1)) ) {
   rownames(riskG01) <- rnames
   colnames(riskG01) <- c("coef","se","se-fixed-Gc")[1:2]
   varG01 <- crossprod(riskG01.iid)
   riskG01 <- cbind(riskG01,diag(varG01)^.5)[,c(1,3)]
}

if (!is.null(augmentC) & MG.se) names(Augment.times) <- rnames

###pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
###contrast <- -pdiff(length(nlevs))
###nncont <- c()
###for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
###rownames(contrast) <- nncont
###

riskG <- list(riskG=riskG,riskG0=riskG0,riskG1=riskG1,riskG01=riskG01)

if (!is.null(call.id)) {
	riskG.iid <-  nameme(riskG.iid,name.id)
	riskG0.iid <- nameme(riskG0.iid,name.id)
	riskG1.iid <- nameme(riskG1.iid,name.id)
	riskG01.iid <- nameme(riskG01.iid,name.id)
}

riskG.iid <- list(riskG.iid=riskG.iid,riskG0.iid=riskG0.iid,
		  riskG1.iid=riskG1.iid,riskG01.iid=riskG01.iid,
	  id=id,call.id=call.id,name.id=name.id)
varG <- list(varG=varG,varG0=varG0,varG1=varG1,varG01=varG01)
val <- list(riskG=riskG,varG=varG,riskG.iid=riskG.iid,
	     MGc=MGc,CensAugment.times=Augment.times,
            dynCens.coef=dynCgammat,dataW=dataW)

  class(val) <- "binregTSR"
  return(val)
}# }}}

##' @export
print.binregTSR  <- function(x,...) {# {{{
	print(summary(x),...)
}# }}}

##' @export
summary.binregTSR <- function(object,...) {# {{{
res <- object$riskG
class(res) <- "summary.binregTSR"
return(res)
}# }}}

##' @export
print.summary.binregTSR  <- function(x,...) {# {{{
    cat("Simple estimator :\n")
    print(x$riskG)
    cat("\n")

    cat("First Randomization Augmentation :\n")
    print(x$riskG0)
    cat("\n")

   cat("Second Randomization Augmentation :\n")
   print(x$riskG1)
   cat("\n")

   cat("1st and 2nd Randomization Augmentation :\n")
   print(x$riskG01)
   cat("\n")
} # }}}

   simdataTSR <- function(n,base12,base1d,base2d,...) { ## {{{
   A0 <- rbinom(n,1,0.5)
   A1 <- rbinom(n,1,0.5)
   X0 <- matrix(rbinom(2*n,1,0.5),n,2)
   betaX1 <- c(2.7,-2.7)
   X1 <- matrix(rbinom(2*n,1, expit(X0 %*% betaX1)),n,2)
   cov(cbind(X0,X1))
   ###
   beta13 <- c(-0.3,0.4,-0.5)
   beta14 <- c(-0.3,0.2,-0.5)
   beta23 <- c(-0.3,0.4,-0.5)
   beta24 <- c(-0.3,0.2,-0.5)
   betaR <- c(-0.3,0.4,-0.5)
   gamma23 <- 0.3
   gamma24 <- -0.3
   ###
   rr <- exp( cbind(A0,X0) %*% betaR)
   rr13 <- exp( cbind(A0,X0) %*% beta13)
   rr14 <- exp( cbind(A0,X0) %*% beta14)
   rr23 <- exp( cbind(A1,X1) %*% beta23)
   rr24 <- exp( cbind(A1,X1) %*% beta24)
   rd <- cbind(rr13,rr14)
   rd2 <- cbind(rr23,rr24)
   ###
   iddata <- sim_multistateII(base12,base1d,base2d,rr=rr,rd=rd,rd2=rd2,
			     early2=1800,gamma23=gamma23,gamma24=gamma24,...)
   covX0 <- cbind(A0[iddata$id],X0[iddata$id,])
   colnames(covX0) <- c("A0","X01","X02")
   covX1 <- cbind(A1[iddata$id],X1[iddata$id,])

   iddata  <-  count_history(iddata,types=2)
   ## only covariates after having gone to stage 2
   covX1 <- covX1*c(iddata$Count2)
   colnames(covX1) <- c("A1","X11","X12")
   iddata <- cbind(iddata,covX0,covX1)
   iddata$R3 <- (iddata$entry<1800)*c(iddata$Count2)
   iddata$R4 <- (iddata$entry<1800)*c(iddata$Count2)
   ###

   cid  <-  countID(iddata)
   iddata$cid <- cid$Countid
   iddata$tid <- cid$Totalid
   return(iddata) 
   } ## }}}

gsim <- function(n,null=1,cens=NULL,ce=2,covs=1,
	    beta0=c(0.1,0.5,-0.5),beta1=c(0.4,0.3,0.5,-0.5),betaR=c(-0.3,-0.5,0.5),betac=c(0.3,0.3), 
            beta0R=c(0,0.5,-0.5),beta1R=c(0,0.5,-0.5),tsr=1,int=1)
   {# {{{

   Count2 <- TR <- NULL

   X0 <- matrix(rbinom(2*n,1,0.5),n,2)
   betaX1 <- 0*c(2.7,-2.7)
   X1 <- matrix(rbinom(2*n,1,expit(X0 %*% betaX1)),n,2)

   O10 <- 0.5; O20 <- 4; OR1 <- 1; OR2 <- 2; 
   if (null==0) Os11 <- 8 
   if (null==1) Os11 <- 3
   Os12 <- 3
   if (null==0) Os21 <- 6
   if (null==1) Os21 <- 3
   Os22 <- 3
   ###
   pr1 <- 0.5
   pr2 <- 0.5
   ###
   if (tsr==0) p0 <- expit(cbind(1,X0) %*% beta0R) else p0 <- 0.5
   A0 <- rbinom(n,1,p0)+1
   if (tsr==0) p1 <- expit(cbind(1,X1) %*% beta1R) else p1 <- 0.5
   A1 <- rbinom(n,1,p1)+1
   ###
   if (covs==1) {
   rr01 <- exp( cbind(0,X0) %*% beta0)
   rr02 <- exp( cbind(1,X0) %*% beta0+ int*( apply(X0,1,sum) * beta0[1]))
   rr11 <- exp( cbind(0,0,X1) %*% beta1 )
   rr12 <- exp( cbind(0,1,X1) %*% beta1+ int*( apply(X1,1,sum) * beta1[1]))
   rr21 <- exp( cbind(1,0,X1) %*% beta1 )
   rr22 <- exp( cbind(1,1,X1) %*% beta1+ int*( apply(X1,1,sum)* beta1[1]))
   rrr1 <- exp( cbind(0,X0) %*% betaR)
   rrr2 <- exp( cbind(1,X0) %*% betaR + int*(apply(X0,1,sum)*betaR[1]))
   } else {
	   rr21 <- rr22  <- rr01 <- rr02 <- rr11 <- rr12 <- rrr1 <- rrr2 <- 1
   }
   ###
   TR1 <- runif(n)*OR1*rrr1
   TR2 <- runif(n)*OR2*rrr2

      TCR1 <- TR1 
      TCR2 <- TR2 
   if (cens!=0)  {
	   if (cens==2 | cens==1) {
	      TCR1 <- runif(n)*OR1*ce 
	      TCR2 <- runif(n)*OR2*ce 
	   } 
   }
   ###
   T10 <- rexp(n)*O10*rr01
   T20 <- rexp(n)*O20*rr02
   ### response if (TR1 < T10) or (TR2 < T20)
   R1 <- (TR1 < T10)*1 
   R2 <- (TR2 < T20)*1 
   ###
   Ts11 <- rexp(n)*Os11*exp(-3*TR1)*rr11
   Ts12 <- rexp(n)*Os12*exp(-3*TR1)*rr12
   Ts21 <- rexp(n)*Os21*exp(-1*TR2)*rr21
   Ts22 <- rexp(n)*Os22*exp(-1*TR2)*rr22
   ###
   Tt11 <- TR1 + Ts11
   Tt12 <- TR1 + Ts12
   Tt21 <- TR2 + Ts21
   Tt22 <- TR2 + Ts22
   ###
   TT <- (A0==1)*( (1-R1)*T10 + R1*((A1==1)*Tt11 + (A1==2)*Tt12))+ 
         (A0==2)*( (1-R2)*T20 + R2*((A1==1)*Tt21 + (A1==2)*Tt22))  
###	     
 TT11  <-  (1-R1)*T10 + R1*Tt11
 TT12  <-  (1-R1)*T10 + R1*Tt12
 TT21  <-  (1-R2)*T20 + R2*Tt21
 TT22  <-  (1-R2)*T20 + R2*Tt22
 TTt <- cbind(TT11,TT12,TT21,TT22)
 ###
   TC10 <- rexp(n)*O10*ce
   TC20 <- rexp(n)*O20*ce
   ### response if (TR1 < T10) or (TR2 < T20)
   RC1 <- (TCR1 < TC10)*1 
   RC2 <- (TCR2 < TC20)*1 
   ###
   TCs11 <- rexp(n)*Os11*exp(-3*TCR1)*ce 
   TCs12 <- rexp(n)*Os12*exp(-3*TCR1)*ce 
   TCs21 <- rexp(n)*Os21*exp(-1*TCR2)*ce
   TCs22 <- rexp(n)*Os22*exp(-1*TCR2)*ce
   ###
   TC11 <- TCR1 + TCs11
   TC12 <- TCR1 + TCs12
   TC21 <- TCR2 + TCs21
   TC22 <- TCR2 + TCs22
  ###   
  CC <-  (A0==1)*( (1-RC1)*TC10 + RC1*((A1==1)*TC11 + (A1==2)*TC12))+ 
         (A0==2)*( (1-RC2)*TC20 + RC2*((A1==1)*TC21 + (A1==2)*TC22))  
 CC11  <-  (1-RC1)*TC10 + RC1*TC11
 CC12  <-  (1-RC1)*TC10 + RC1*TC12
 CC21  <-  (1-RC2)*TC20 + RC2*TC21
 CC22  <-  (1-RC2)*TC20 + RC2*TC22
 CCt <- cbind(CC11,CC12,CC21,CC22)
 ##
### CC  <- rexp(n)*ce* exp((A0-1)*betac[1]+(A1-1)*betac[2])
 ###
  if (cens==0) cc <- max(TT)+1 else  {
	  if (cens==1) cc <- rexp(n)*ce*exp((A0-1)*betac[1])
	  if (cens==2) cc <- CC
	  if (cens==3) cc <- CC
  }
  ###
  time <- pmin(TT,cc)
  status <- (TT<cc)*1
  ###  
  data <- data.frame(time=time,T10=T10,Tt11=Tt11,Tt12=Tt12,TT=TT,status=status,A1=A1,A0=A0,R1=R1,R2=R2,R=(A0==1)*R1+(A0==2)*R2,id=1:n,
		     TR=(A0==1)*TR1+(A0==2)*TR2,Resp=(A0==1)*R1+(A0==2)*R2)
  dfactor(data) <- A0.f~A0
  dfactor(data) <- A1.f~A1
  datat <- event_split(data,cuts="TR",name.start="entry")
###  datat <- EventSplit(data,cuts="TR",entry="entry")

  datat <- dtransform(datat,status=2,time==TR)
  datat  <-  count_history(datat,types=2)
  datat  <-  dtransform(datat,A1=0,Count2==0)
  datat$response <- (datat$Count2==1)*1
  datat$A11t <- (datat$A1==1)*1
  datat$A12t <- (datat$A1==2)*1
  datat$A01 <- (datat$A0==1)*1
  datat$A02 <- (datat$A0==2)*1


  covX0 <- X0[datat$id,]
  colnames(covX0) <- c("X01","X02")
  covX1 <- X1[datat$id,]
  ###
  covX1 <- covX1*c(datat$Count2)
  colnames(covX1) <- c("X11","X12")
  datat <- cbind(datat,covX0,covX1)
  datat <- dtransform(datat,TR=0,Count2==0) 

  res <- list(data=data,datat=datat,TTt=TTt,CCt=CCt)
  return(res)
  } # }}}

##' Pepe-Mori Test for Marginal Mean Comparison
##'
##' Performs score-test type tests for proportionality of marginal means in competing 
##' risks data and recurrent events, as presented in Ghosh and Lin (2000). The test 
##' is based on an IPCW (Inverse Probability of Censoring Weighting) formulation.
##' 
##' The function computes several tests:
##' \enumerate{
##'   \item \strong{Pepe-Mori Test}: Tests for equality of marginal mean functions between groups.
##'   \item \strong{Ratio of AUC}: Compares the area under the curve of marginal means.
##'   \item \strong{Difference of AUC}: Tests for difference in areas under the curve.
##'   \item \strong{Score Test}: Tests for proportionality (equivalent to Gray's test for CIF).
##'   \item \strong{Proportionality Test}: Tests the proportional hazards assumption.
##' }
##' 
##' The Pepe-Mori test uses weights based on the number at risk in each group to 
##' construct a weighted integral of the difference in marginal means.
##'
##' @param formula Formula with an \code{Event} object on the left-hand side and 
##'   covariates (typically with \code{strata()} for group comparison) on the right.
##'   Can include \code{cluster(id)} for correlated data.
##' @param data Data frame containing all variables referenced in the formula.
##' @param cause Cause of interest (default 1).
##' @param cens.code Censoring code (default 0).
##' @param death.code Code for death (terminating event, default 2).
##' @param death.code.prop Code for other causes of death for Fine-Gray regression model.
##' @param time Upper limit for Pepe-Mori and AUC integrals. If NULL, defaults to 
##'   the maximum event time for the cause of interest.
##' @param beta Starting values for the score test (default NULL, uses zeros).
##' @param ... Additional arguments passed to lower-level functions.
##' @return An object of class \code{"marginalTest"} containing:
##'   \item{pepe.mori}{Pepe-Mori test results with compare p-value.}
##'   \item{RatioAUC}{Ratio of AUC test results with compare p-value.}
##'   \item{difAUC}{Difference of AUC test results with compare p-value.}
##'   \item{prop.test}{Proportionality test results.}
##'   \item{score.test}{Score test results (equivalent to Gray's test).}
##'   \item{score.iid}{Influence function for the score test.}
##'   \item{time}{Upper time limit used.}
##'   \item{RAUCl, RAUCe}{Raw and transformed AUC estimates.}
##' @author Thomas Scheike
##' @references 
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric Analysis of Recurrent Events and Death. Biometrics, 56, 554--562.
##' @seealso \code{\link{test_logrankRecurrent}}, \code{\link{test_conc}}
##' @examples
##' data(bmt,package="mets")
##' bmt$time <- bmt$time+runif(nrow(bmt))*0.01
##' bmt$id <- 1:nrow(bmt)
##' dcut(bmt) <- age.f~age
##'      
##' fg=cifregFG(Event(time,cause)~tcell,data=bmt,cause=1)
##'  
##' ## computing tests for difference  for CIF
##' pmt <- test_marginalMean(Event(time,cause)~strata(tcell)+cluster(id),data=bmt,cause=1,
##' 			 death.code=1:2,death.code.prop=2,cens.code=0,time=40)
##' summary(pmt) 
##'  
##' pmt$pepe.mori
##' pmt$RatioAUC
##' pmt$prop.test
##' ## score test equialent to Gray's test but variance estimated differently 
##' pmt$score.test
##'  
##' ### age-groups  
##' pmt <- test_marginalMean(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1,
##' 			 death.code=1:2,death.code.prop=2,cens.code=0)
##' summary(pmt) 
##'  
##' ## having a look at the cumulative incidences 
##' cifs <- cif(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1)
##' plot(cifs) 
##'  
##' ## recurrent events   
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' pmt <- test_marginalMean(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
##' 			 cause=1,death.code=2,cens.code=0)
##' summary(pmt) 
##'  
##' @export
test_marginalMean <- function(formula,data,cause=1,cens.code=0,...,death.code=2,death.code.prop=NULL,time=NULL,beta=NULL) { ## {{{ 
cl <- match.call()
###m <- match.call(expand.dots = TRUE)[1:3]
des <- proc_design(formula, data = data, specials = c("offset","weights","cluster","strata","marks"), intercept = FALSE)
Y <- des$y
if (!inherits(Y, c("Event", "Surv"))) {
stop("Expected a 'Surv' or 'Event'-object")
}
if (ncol(Y) == 2) {
exit <- Y[, 1]
entry <- rep(0, nrow(Y))
status <- Y[, 2]
}
else {
entry <- Y[, 1]
exit <- Y[, 2]
status <- Y[, 3]
}
X <- des$x
strata <- des$strata
specials = c("offset", "weights", "cluster", "strata")
Terms <- terms(formula, specials, data = data)
ts <- survival::untangle.specials(Terms, "strata")
if (!is.null(strata))
strata.name <- ts$vars
else strata.name <- NULL
pos.strata <- NULL
des.weights <- des$weights
des.offset <- des$offset
id <- des$cluster
pos.cluster <- NULL

if (is.null(death.code.prop)) death.code.prop <- death.code

data$strata__ <- strata
data$entry__ <- entry
data$exit__ <- exit
nstrata <- as.numeric(strata)

###drop.specials(formula,"strata")
formR <- as.formula(paste("Event(entry__,exit__,status__)~factor(strata__)+cluster(id__)"))
data$status__  <- status
data$id__  <- id

## proportionality test  
proptest <- recreg(formR,data,cause=cause,death.code=death.code.prop,cens.model=~strata(strata__))

nlev <- nlevels(strata)
if (is.null(beta)) beta <- rep(0,nlev-1)
## score test  for prop-factor 
proptest0 <- recreg(formR,data,cause=cause,death.code=death.code.prop,
		   cens.model=~strata(strata__),beta=beta,no.opt=TRUE)
gradient <- matrix(proptest0$gradient,length(proptest0$gradient),1)
iidbeta <-  suppressWarnings(IC(proptest0))
n <- nrow(iidbeta)
iidU0 <- iidbeta %*% proptest0$hessian
score.test <- suppressWarnings(
  summary(estimate(coef=gradient,IC=iidU0), null=0)
)

### varU0 <- crossprod(iidU0)
### logrank.robust <- t(gradient) %*% solve(varU0) %*% gradient
### p.logrank.robust <- 1-pchisq(logrank.robust,nrow(gradient))
### score.test <- list(logrank.robust=logrank.robust,p.logrank.robust=p.logrank.robust,
###		    test.statistic=gradient,iid=iidU0)

ev_times <- exit[status %in% cause]
if (length(ev_times) == 0) stop("No events of the specified cause found; cannot determine time.")
if (is.null(time)) time <- max(ev_times)

## {{{ pepe-mori test 

ddataid <- data.frame(id=id)
cid <- countID(ddataid)
###ns <- c(table(strata[cid$reverse==1]))

data$statusC__ <- 1*(status %in% cens.code) 
data$statusD__ <- 1*(status %in% death.code)
formD <- as.formula(paste("Event(entry__,exit__,statusD__)~+1"))

if (nlev==2) {
	formC <- as.formula(paste("Event(entry__,exit__,statusC__)~strata(strata__)"))
	kms <- km(formC,data)
	kms0 <- phreg(formC,data,no.opt=TRUE)
	kms0$jumptimes
	kms0$strata.jumps
        Yr <- vecAllStrata(kms0$S0,kms0$strata.jumps,kms0$nstrata)
	minRisk <- apply(Yr,1,min)

        if (is.null(time)) {
               kms0 <- phreg(formC,data,no.opt=TRUE)
	       kms0$jumptimes
	       kms0$strata.jumps
               Yr <- vecAllStrata(kms0$S0,kms0$strata.jumps,kms0$nstrata)
	       minRisk <- apply(Yr,1,min)
               time <- max(kms0$jumptimes[minRisk>=5])
        }

        ns <- c(table(strata[cid$reverse==1]))
	n1 <- ns[1]
	n2 <- ns[2]
	## using number at risk for weighting 
	kmss <- cbind(kms$time,t(kms$surv))
	wt <- (n1+n2)*kmss[,2]*kmss[,3]/(n1*kmss[,2]+n2*kmss[,3])
	wt[is.na(wt)] <- 0
	Wt <- cumsum(diff(c(0,kms$time))*wt)

	Wtmark <- lin_approx(pmin(exit,time),rbind(0,cbind(kms$time,Wt)))
	Wfinal <- tail(Wt,1)
	###
	data$pmmark__ <- Wfinal-Wtmark

	pepe.mori <- recregIPCW(formR,data,cause=cause,death.code=death.code,
	    times=time,marks=data$pmmark__,cens.model=~strata(strata__),model="lin")
	pepe.mori <- suppressWarnings(
      summary(estimate(pepe.mori), contrast=lava::contr(2))
    )
} else {

###  formRC <- as.formula(paste("Event(entry__,exit__,status__)~factor(cstrata__)+cluster(id__)"))
###	contr <- contr.iid <- c()
###        for (i in 1:(nlev-1)) {
###
###   	   data$cstrata__  <-  1*(nstrata==i)
###           formC <- as.formula(paste("Event(entry__,exit__,statusC__)~strata(cstrata__)"))
###
###           kms <- km(formC,data)
###           kmss <- cbind(kms$time,t(kms$surv))
###
###           ns <- c(table(data$cstrata__[cid$reverse==1]))
###	   n1 <- ns[1]
###	   n2 <- ns[2]
###	   ## using number at risk for weighting 
###	   kmss <- cbind(kms$time,t(kms$surv))
###	   wt <- (n1+n2)*kmss[,2]*kmss[,3]/(n1*kmss[,2]+n2*kmss[,3])
###	   wt[is.na(wt)] <- 0
###	   Wt <- cumsum(diff(c(0,kms$time))*wt)
###	   Wtmark <- lin_approx(pmin(exit,time),rbind(0,cbind(kms$time,Wt)))
###	   Wfinal <- tail(Wt,1)
###	   ###
###	   data$pmmark__ <- Wfinal-Wtmark
###
###	   pmOut <- recregIPCW(formRC,data,cause=cause,death.code=death.code,
###	       times=time,marks=data$pmmark__,cens.model=~strata(cstrata__),model="lin")
###
###        contr <- c(contr,coef(pmOut)[2])
###	contr.iid <- cbind(contr.iid,pmOut$iid[,2])
###   }
###   var.contr <- crossprod(contr.iid)
###   ###  pepe.mori <- summary(estimate(coef=contr,vcov=var.contr),null=0)
###   pepe.mori <- summary(estimate(coef=contr,IC=contr.iid*nrow(contr.iid)),null=0)
###   p <- length(contr)
###   pepe.mori <- estimate(pepe.mori,lava::contr(1:p))
	pepe.mori <- list()
	pepe.mori$error <- "only for two levels"
        pepe.mori$compare$p.value  <-  NA
}

## }}}

## Ratio of AUC
data$pmmarkAUC__ <- time-exit
## linear model for stability when fitting
RAUCl<- recregIPCW(formR,data,cause=cause,death.code=death.code,
	  times=time,cens.model=~strata(strata__),marks=data$pmmarkAUC__,
	  model="lin")
f <- function(p) p[-1]/p[1]
## reparametrize as baseline on log-scale, and log-ratio contrasts 
RAUCe <- estimate(RAUCl,function(p) c(log(p[1]),log((p[1]+p[-1])/p[1])))
p <- length(coef(RAUCl))
RAUCet <- summary(estimate(RAUCe),contrast=as.list(2:p))
RAUClt <- summary(RAUCl, contrast=as.list(2:p))

proptest <- suppressWarnings(
  summary(estimate(proptest),null=0)
)

out <- list(pepe.mori=pepe.mori,RatioAUC=RAUCet,difAUC=RAUClt,
	    RAUCl=RAUCl,RAUCe=RAUCe,
	    score.test=score.test,score.iid=iidU0,prop.test=proptest,time=time)
class(out) <- "marginalTest"
return(out)
} ## }}} 

##' @export
print.marginalTest  <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.marginalTest <- function(object,...) {# {{{

p.values <- c(object$time,
        object$pepe.mori$compare$p.value,
        object$RatioAUC$compare$p.value,
        object$prop.test$compare$p.value,
        object$score.test$compare$p.value)
p.values <- matrix(p.values,ncol=1)
colnames(p.values) <- "p-value"
rownames(p.values) <- c("time","Pepe-Mori","Ratio-AUC","Proportionality","Proportionality-score-test")

res <- p.values
class(res) <- "summary.marginalTest"
return(res)
}# }}}

##' @export
print.summary.marginalTest <- function(x,...) { ## {{{
  cat("coeffients:\n")
  printCoefmat(x,...)
  cat("\n")
} ## }}}


## proc_design <- mets:::proc_design

gsub2 <- function(pattern, replacement, x, ...) {
  if (length(pattern)!=length(replacement)) {
      pattern <- rep(pattern, length.out=length(replacement))
  }
  result <- x
  for (i in 1:length(pattern)) {
    result <- sub(pattern[i], replacement[i], result, ...)
  }
  result
}

procform <- function(formula=NULL,
                     sep="\\|",
                     nsep=1,
                     return.formula=FALSE,
                     data=NULL,
                     no.match=TRUE,
                     regex=FALSE,
                     return.list=TRUE,
                     specials=NULL,...) {# {{{
  res <- NULL
  if (is.null(formula)) {
    res <- colnames(data)
  } else if (is.character(formula)) {
    if (is.null(data)) {
      res <- unique(formula)
    } else {
      yy <-c()
      for (y0 in formula) {
        y0orig <- y0
        if (!regex) y0 <- glob2rx(y0)
        npos <- grep(y0,names(data),perl=mets.options()$regex.perl)
        if (no.match && length(npos)==0) {
          yy <- union(yy, y0orig)
        } else {
          yy <- union(yy,names(data)[npos])
        }
      }
      res <- unique(yy)
    }
  }
  if (is.numeric(formula)) res <- colnames(data)[formula]
  if (is.character(res)) {
    if (!return.list) return(res)
    if (return.formula) return(as.formula(paste("~",paste(res,collapse="+"))))
    return(list(response=res,predictor=NULL,filter=NULL))
  }


  ## Add parantheses around quotes if it is not a function call
  if (inherits(formula,"formula")) {

    st <- Reduce(paste,deparse(formula))
    strsplit(st,"\"")
    quotepos <- gregexpr("[\"']",st)[[1]]
    if (quotepos[1]>0) {
      sts <- strsplit(st,"[\"']")[[1]]
      foundsep <- any(grepl("|", sts, fixed=TRUE))
      p <- length(quotepos)
      ##repl <- rep(c("(\"","\")"),p)
      for (i in seq(p/2)*2-1) {
        sts[i] <- paste0(sts[i],"(\"")
        sts[i+1] <- paste0(sts[i+1],"\")")
      }
      ## To handle regular expression entered as strings in the formula, we add a 'filter' expression at the end of the formula
      if (!foundsep) sts <- c(sts,"|1")
      formula <- as.formula(paste(sts,collapse=""))
    }
  }

  aa <- attributes(terms(formula,data=data,specials="regex"))
  if (aa$response == 0) {
    res <- NULL
  } else {
    res <- paste(deparse(formula[[2]]), collapse = "")
  }
  filter.expression <- NULL
  foundsep <- FALSE
  pred <- filter <- c()
  if (!is.null(sep) && length(aa$term.labels) > 0) {
    foundsep <- any(grepl(sep,aa$term.labels))
    if (foundsep) {
      if (nsep>1) {
        xc <- gsub(" ","",unlist(lapply(aa$term.labels, function(z) strsplit(z,sep)[[1]])))
        pred <- xc[1]
        filter <- xc[-1]
      } else {
        xc <- gsub(" ","",unlist(lapply(aa$term.labels, function(z) {
          spl <- regexpr(sep,z) ## first appearance
          pred <- substr(z,1,spl-1)
          filter <- substr(z,spl+1,nchar(z))
          return(c(pred,filter))
        })))
        pred <- xc[1]
        filter <- xc[2]
      }
      if (any(pred==".")) {
        f <- as.formula(paste0(paste0(c(res,filter),collapse="+"),"~."))
        x <- attributes(terms(f,data=data))$term.labels
        pred <- x
      }
      if (filter%in%c("0","-1")) {
        filter <- list()
        filter.expression <- NULL
      } else {
        filter.expression <- parse(text=filter)
        filter <- as.list(filter)
      }
    }
  }
  if (!foundsep) pred <- aa$term.labels

  expandst <- function(st) {
    st <- res <- unlist(strsplit(gsub(" ","",st),"\\+"))
    if (any(unlist(lapply(st, function(x) grepl("^\\(",x))))) {
      res <- c()
      for (x in st) {
        if (grepl("^\\(",x)) {
          x <- gsub('\\"',"",x)
          x <- gsub('^\\(',"",x)
          x <- gsub('\\)$',"",x)
          res <- c(res,unlist(procform(x,data=data,regex=regex, no.match=FALSE)$response))
        } else {
          res <- c(res,x)
        }
        res <- unique(res)
      }
    }
    return(res)
  }
  res <- expandst(res)
  pred <- expandst(pred)
  if (any(res==".")) {
    diffset <- c(".",setdiff(pred,res))
    res <- setdiff(union(res,colnames(data)),diffset)
  }
  filter <- lapply(filter, expandst)
  if (!is.null(specials)) {
    foundspec <- replicate(length(specials),c())
    names(foundspec) <- specials
    rmidx <- c()
    spec <- paste0("^",specials,"\\(")
    val <- lapply(spec, function(x) which(grepl(x,pred)))
    for (i in seq_along(val)) {
      if (length(val[[i]])>0) { # special function found
        rmidx <- c(rmidx,val[[i]])
        cleanpred <- gsub("\\)$","",gsub(spec[i],"",pred[val[[i]]]))
        foundspec[[i]] <- c(foundspec[[i]],cleanpred)
      }
    }
    if (length(rmidx)>0)
      pred <- pred[-rmidx]
    if (length(pred)==0) pred <- NULL
    specials <- foundspec
    for (i in seq_along(specials)) if (is.null(specials[[i]])) specials[i] <- NULL
    if (length(specials)==0) specials <- NULL
  }
  if (return.formula) {
    if (foundsep && !is.null(filter)) {
      filter <- lapply(filter,
                       function(z) as.formula(paste0("~", paste0(z,collapse="+"))))
    }
    if (length(pred)>0)
      pred <- as.formula(paste0("~", paste0(pred,collapse="+"), collapse=""))
    if (length(res)>0)
      res <- as.formula(paste0("~", paste0(res,collapse="+"), collapse=""))
    if (!is.null(specials)) {
      specials <- lapply(specials,function(x)
        as.formula(paste0("~", paste0(x,collapse="+"), collapse="")))
    }
  }
  res <- list(response=res,
              predictor=pred,
              filter=filter,
              filter.expression=filter.expression,
              specials=specials)
  if (!return.list) return(unlist(unique(res)))
  return(res)
}

procformdata <- function(formula,
                         data,
                         sep="\\|",
                         na.action=na.pass,
                         do.filter=TRUE,
                         ...) {
    res <- procform(formula,sep=sep,data=data,return.formula=TRUE,...)
    if (inherits(res,"formula")) {
        res <- list(response=res)
    }
    y <- x <- NULL
    filter <- res$filter.expression
    if (!do.filter) {
        filter <- NULL
    }
    ### when filter.expression is expression(1) then also no filter, ts
    if ((!missing(filter))) if (!is.null(filter)) if (as.character(filter)=="1") filter <- NULL

    if (length(res$response)>0) {
        if (is.null(filter)) y <- model.frame(res$response,data=data,na.action=na.action)
        else y <- model.frame(res$response,data=subset(data,eval(filter)),na.action=na.action)
    }
    if (length(res$predictor)>0) {
        if (is.null(filter)) x <- model.frame(res$predictor,data=data,na.action=na.action)
        else x <- model.frame(res$predictor,data=subset(data,eval(filter)),na.action=na.action)

    }

    specials <- NULL
    if (!is.null(res$specials)) {
        specials <- lapply(res$specials,
                      function(x) {
                          if (is.null(filter)) model.frame(x,data=data,na.action=na.action)
                          else model.frame(x,data=subset(data,eval(filter)),na.action=na.action)
                      })
    }

    if (!do.filter) {
        group <- lapply(res$filter, function(x) model.frame(x,data=data,na.action=na.action))
        return(list(response=y,predictor=x,group=group,specials=specials))
    }
    return(list(response=y,predictor=x,specials=specials))
}# }}}

procform2 <- function(y,x=NULL,z=NULL,...) {# {{{
  yx <- procform(y,return.formula=FALSE,...)
  y <- yx$response
  x0 <- yx$predictor
  z0 <- NULL
  if (length(yx$filter)>0) z0 <- yx$filter[[1]]
  if (is.null(x) && length(y)>0) x <- x0
  if (NCOL(x)==0) x <- NULL
  if (length(y)==0) y <- x0
  if (!is.null(x)) {
    x <- unlist(procform(x,return.formula=FALSE,...))
  }
  if (is.null(z)) z <- z0
  if (!is.null(z)) {
    zz <- unlist(procform(z,return.formula=FALSE,...))
  }
  if (is.null(z)) z <- z0
  return(list(y=y,x=x,z=z))
}# }}}

procform3 <- function(y,x=NULL,z=NULL,...) {# {{{
  yx <- procform(y,return.formula=FALSE,...)
  x0 <- yx$predictor
  y  <- yx$response
  if (is.null(yx$predictor)) { x0 <- yx$response ; y <- NULL}

  if (is.null(y))
    if (!is.null(x)) {
      x <- procform(x,return.formula=FALSE,...)
      y <- c(x$predictor,x$response)
    }
  return(list(y=y,x=x0,z=z))
}# }}}

Specials <- function(f,spec,split1=",",split2=NULL,...) {# {{{
  tt <- terms(f,spec)
  pos <- attributes(tt)$specials[[spec]]
  if (is.null(pos)) return(NULL)
  x <- rownames(attributes(tt)$factors)[pos]
  st <- gsub(" ","",x) ## trim
  spec <- unlist(strsplit(st,"[()]"))[[1]]
  res <- substr(st,nchar(spec)+2,nchar(st)-1)
  if (!is.null(split1))
    res <- unlist(strsplit(res,split1))
  res <- as.list(res)
  for (i in seq_along(res)) {
    if (length(grep("~", res[[i]])) > 0) {
      res[[i]] <- as.formula(res[[i]])
    }
  }
  return(structure(res, "name.mf" = x))
}# }}}

decomp.specials <- function (x, pattern = "[()]", sep = ",", ...)
{
  st <- gsub(" ", "", x)
  if (!is.null(pattern))
    st <- rev(unlist(strsplit(st, pattern, ...)))[1]
  unlist(strsplit(st, sep, ...))
}

model.extract2 <- function(frame, component) {
  component <- as.character(substitute(component))
  if (component %in% c("response", "offset")) {
    return(do.call(
      model.extract,
      list(frame = frame, component = component)
    ))
  }
  vname <- paste0("(", component, ")")
  if (!(vname %in% names(frame))) {
    regex <- paste0("^", component, "\\(.*\\)$")
    if (any(grepl(regex, names(frame)))) {
      vname <- grep(regex, names(frame))
      if (length(vname) > 1) stop("model.extract2: non-unique component")
    }
  }
  rval <- frame[[vname]]

  if (!is.null(rval)) {
    if (length(rval) == nrow(frame)) {
      names(rval) <- attr(frame, "row.names")
    } else if (is.matrix(rval) && nrow(rval) == nrow(frame)) {
      t1 <- dimnames(rval)
      dimnames(rval) <- list(
        attr(frame, "row.names"),
        t1[[2L]]
      )
    }
  }
  return(rval)
}

# #' Extract design matrix from data.frame and formula
# #' @title Extract design matrix
# #' @param formula formula
# #' @param data data.frame
# #' @param intercept (logical) If FALSE an intercept is not included in the
# #'   design matrix
# #' @param response (logical) if FALSE the response variable is dropped
# #' @param rm_envir Remove environment
# #' @param ... additional arguments (e.g, specials such weights, offsets, ...)
# #' @param specials character vector specifying functions in the formula that
# #'   should be marked as special in the [terms] object
# #' @param specials.call (call) specials optionally defined as a call-type
# #' @param levels a named list of character vectors giving the full set of
# #'   levels
# #'   to be assumed for each factor
# #' @param design.matrix (logical) if FALSE then only response and specials are
# #'   returned. Otherwise, the design.matrix `x` is als part of the returned
# #'   object.
# #' @param na.action method to handle missing data
# #' @return An object of class 'mets.design'
# #' @examples
# #' n <- 1e3
# #' a <- rbinom(n, 1, 0.5)
# #' t <- rweibull(n, shape=0.5, exp(3 + a))
# #' d <- data.frame(time=t, status=TRUE, a=a, id=seq(n), x=rnorm(n))
# #' des <- proc_design(
# #' Surv(time, status) ~ x + strata(a) + cluster(id),
# #'   specials = c("strata", "cluster"),
# #'   data=d)
# #' new <- update_design(des, head(d))
# #' new$strata
# #' new$cluster
# #' new$y
# #' new$x
proc_design <- function(formula, data, ..., # nolint
                        intercept = FALSE,
                        response = TRUE,
                        rm_envir = FALSE,
                        specials = NULL,
                        specials.call = NULL,
                        levels = NULL,
                        design.matrix = TRUE,
                        na.action = na.omit
                        ) {
  if (inherits(data, c("data.table", "tbl_df"))) {
    data <- as.data.frame(data)
  }
  dots <- substitute(list(...))
  if ("subset" %in% names(dots)) {
    stop("subset is not an allowed specials argument for `proc_design`")
  }
  formulaenv <- new.env(parent = environment(formula))
  formulaenv$cluster <- mets::cluster
  formulaenv$strata <- mets::strata
  formulaenv$weights <- stats::weights
  formulaenv$marks <- mets::marks
  environment(formula) <- formulaenv
  tt <- terms(formula, data = data, specials = specials)
  term.labels <- attr(tt, "term.labels") # predictors

  if (response && inherits(
                    try(model.frame(update(tt, ~1),
                                    data = data,
                                    na.action = na.action),
                        silent = TRUE),
                    "try-error"
                  )) {
    response <- FALSE
  }
  # delete response to generate design matrix when making predictions
  if (!response) tt <- delete.response(tt)

  sterm.list <- c()
  if (length(specials) > 0) {
    des <- attr(tt, "factors")
    for (s in specials) {
      sterm <- rownames(des)[attr(tt, "specials")[[s]]]
      sterm.list <- c(sterm.list, sterm)
    }
    if (length(sterm.list) > 0) {
      # create formula without specials
      if ((nrow(attr(tt, "factors")) - attr(tt, "response")) ==
          length(sterm.list)) {
        # only specials on the rhs, remove everything
        formula <- update(formula, ~1)
      } else {
        # predictors without the specials
        term.labels <- setdiff(
          term.labels,
          unlist(sterm.list)
        )
        ## formula <- update(tt, reformulate(term.labels))
      }
      fst <- lava::trim(paste(deparse(formula), collapse = ""), all = TRUE)
      for (s in sterm.list) {
        fst <- gsub(trim(s, all = TRUE), "", fst, fixed = TRUE)
      }
      fst <- gsub("[\\+]*$", "", fst) # remove potential any trailing '+'
      formula <- as.formula(fst)
      environment(formula) <- formulaenv
    }
  }

  formula0 <- formula
  environment(formula0) <- formulaenv
  if (!response) formula0 <- formula(delete.response(terms(formula)))
  xlev <- levels
  xlev[["response_"]] <- NULL
  if (!design.matrix) { # only extract specials, response
    des <- attr(tt, "factors")
    fs <- update(formula0, ~1)
    if (length(sterm.list) > 0) {
      # formula with only special-terms
      fs <- reformulate(paste(sterm.list, collapse = " + "))
      fs <- update(formula0, fs)
    }
    environment(formula) <- formulaenv
    mf <- model.frame(fs, data=data, na.action = na.action, ...)
  } else { # also extract design matrix
    mf <- model.frame(tt,
                      data = data, ...,
                      xlev = xlev,
                      na.action = na.action,
                      drop.unused.levels = FALSE
                      )
    if (is.null(xlev)) {
      xlev <- .getXlevels(tt, mf)
    }
    xlev0 <- xlev
  }

  y <- NULL
  if (response) {
    y <- tryCatch(
      model.response(mf, type = "any"),
      error = function(...) NULL
    )
    if (is.factor(y) || is.character(y)) {
      ylev <- levels[["response_"]]
      if (!is.null(ylev)) {
        y <- factor(y, levels = ylev)
      } else {
        ylev <- if (is.factor(y)) {
                  levels(y)
                } else if (is.character(y)) {
                  levels(as.factor(y))
                }
        levels[["response_"]] <- ylev
      }
    }
  }

  has_intercept <- attr(tt, "intercept") == 1L
  specials <- union(
    specials,
    names(dots)[-1] # removing "" at first position when calling dots, which
  ) # is a call object

  specials.list <- c()
  specials.var <- c()
  if (length(specials) > 0) {
    for (s in specials) {
      w <- eval(substitute(model.extract2(mf, s), list(s = s)))
      specials.list <- c(specials.list, list(w))
      spec <- Specials(tt, spec = s)
      if (!is.null(unlist(spec))) {
        spec <- structure(unlist(spec), "name.mf" = attr(spec, "name.mf"))
      }
      specials.var <- c(
        specials.var,
        list(spec)
      )
    }
    names(specials.var) <- specials
    names(specials.list) <- specials
    if (length(sterm.list) > 0) {
      if (design.matrix) {
        xlev0[sterm.list] <- NULL
        mf <- model.frame(formula0,
                          data = data, ...,
                          xlev = xlev0,
                          na.action = na.action,
                          drop.unused.levels = FALSE
                          )
      }
    }
  }

  if (!is.null(specials.call)) {
    specials.list2 <- eval(specials.call, data)
    for (n in names(specials.list2)) {
      if (is.null(specials.list[[n]])) {
        specials.list[[n]] <- specials.list2[[n]]
      }
    }
  }

  if (design.matrix) {
    x <- model.matrix(mf, data = structure(data, terms=NULL),
                      xlev = xlev0)
    if (!intercept && has_intercept) {
      has_intercept <- FALSE
      x <- x[, -1, drop = FALSE]
    }
  } else {
    term.labels <- NULL
    x <- NULL
  }

  if (rm_envir) attr(tt, ".Environment") <- NULL
  if (is.null(specials.call)) specials.call <- dots

  xlev[["response_"]] <- levels[["response_"]]
  res <- c(
    list(
      formula = formula, # formula without specials
      terms = tt,
      term.labels = term.labels,
      levels = xlev,
      x = x, y = y,
      design.matrix = design.matrix,
      intercept = has_intercept,
      data = mf,
      ## data[0, ], ## Empty data.frame to capture structure of data
      specials = specials,
      na.action = na.action,
      specials.var = specials.var,
      specials.call = specials.call
    ),
    specials.list
  )
  return(structure(res, class="mets.design"))
}

update_design <- function(object, data = NULL, response = TRUE, ...) {
  if (is.null(data)) data <- object$data
  missing_spec_var <- missing_spec <- c()
  levels <- object$levels
  for (s in names(object$specials.var)) {
    miss <- FALSE
    specials.var <- object$specials.var[[s]]
    if (!is.null(specials.var)) { ## special used in original design object
      for (v in specials.var) {
        # looping through variables used in special-term checking if missing
        if (!v %in% colnames(data)) {
          if (is.null(object$levels[[v]])) {
            data[[v]] <- 0
          } else { # if factor replace with first level
            data[[v]] <- object$levels[[v]][1]
          }
          miss <- TRUE
        }
      }
      if (miss) {
        missing_spec <- c(missing_spec, s)
        missing_spec_var <- c(missing_spec_var, attr(specials.var, "name.mf"))
      }
    }
  }
  if (length(missing_spec) > 0) object$levels[missing_spec_var] <- NULL
  res <- proc_design(object$terms,
                     data = data,
                     design.matrix = object$design.matrix,
                     levels = object$levels,
                     response = response,
                     na.action = object$na.action,
                     intercept = object$intercept,
                     specials = object$specials,
                     specials.call = object$specials.call
                     )
  object$levels <- levels
  res[missing_spec] <- NULL
  return(res)
}

clean_design <- function(object, ...) {
  object$x <- object$x[0, , drop = FALSE]
  object$y <- NULL
  for (i in object$specials) object[[i]] <- NULL
  return(object)
}

##' @export
model.frame.mets.design <- function(formula, data = NULL, ...) {
  if (is.null(data)) return(formula$data)
  model.frame(formula$formula, data = data, ...)
}

### surv.boxarea <- function(left.trunc,right.cens,data,timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1,boxtimevar="boxtime")
###{ ## {{{
###  if (is.null(data[,id])) stop("Wrong cluster variable")
###  if (is.null(data[,timevar])) stop("Wrong time variable")
###  if (is.null(data[,status])) stop("Wrong status variable")
###  data <- data[order(data[,id]),]
###  if (silent<=-1) {
###    message("survboxare()")
###    print(head(data))
###    print(summary(data[,id]))
###  }
###  if (is.null(num)) {
###    idtab <- table(data[,id])
###    num <- "num"
###    while (num%in%names(data)) num <- paste(num,"_",sep="")
###    data[,c(num)] <- unlist(lapply(idtab,seq_len))
###  }
###
###  timevar2 <- paste(timevar,1:2,sep="")
###  status2 <- paste(status,1:2,sep="")
###  num2 <- paste(num,1:2,sep="")
###  covars2 <- NULL; covars.pairs2 <- NULL; 
###  if (length(covars)>0) covars2 <- paste(covars,1:2,sep="")
###  if (length(covars.pairs)>0) covars.pairs2 <- paste(covars.pairs,1:2,sep="")
###
###  if (silent<=-1) {
###    message("survboxare()")
###    print(head(data))
###    print( c(timevar,status,covars,covars.pairs,id,num))
###    print(c(id,num))
###    print(summary(data))
###  }
###  ww0 <- fast.reshape(data[,c(timevar,status,covars,covars.pairs,id,num)],id=id,num=num,labelnum=TRUE)
###  if (silent<=-1) {
###    message("survboxarea(), ww1")
###    print(head(ww0))
###    print(summary(ww0))
###    print(c(timevar2,status2,covars2,covars.pairs2,id,num2)) 
###    ww0 <- data.frame(ww0); 
###    print(table(ww0$status1,ww0$status2))
###  }
###  mleft <-  (ww0[,timevar2[1]]>left.trunc[1]) & (ww0[,timevar2[2]]>left.trunc[2])  ## Both not-truncated
###
###  if (length(na.idx <- which(is.na(mleft)))>0) {
###    ##    warning("Removing incomplete cases", na.idx)
###    mleft <- mleft[-na.idx]
###    ww0 <- ww0[-na.idx,,drop=FALSE]
###  }
###  if (sum(mleft)==0) stop("No data selected\n"); 
###  ww0 <- ww0[which(mleft),,drop=FALSE]
###
###  right1 <- which(ww0[,timevar2[1]] > right.cens[1])
###  right2 <- which(ww0[,timevar2[2]] > right.cens[2])
###  ww0[,timevar2[1]][right1] <- right.cens[1]
###  ww0[,timevar2[2]][right2] <- right.cens[2]
###  ww0[,status2[1]][right1] <- 0
###  ww0[,status2[2]][right2] <- 0
###  truncvar2 <- c("left1","left2")
###  ww0 <- cbind(ww0,left.trunc[1])
###  ww0 <- cbind(ww0,left.trunc[2])
###  colnames(ww0)[c(-1,0) + ncol(ww0)] <- truncvar2
###
###  if (silent<=-1) print(head(ww0))
###  if (silent<=0)
###    message(paste("  Number of joint events:",sum(apply(ww0[,status2],1,sum)==2),"of ",nrow(ww0)),"\n");
###
###  varying <- c(timevar,status,"left",covars)
###  lr.data <- data.frame(fast.reshape(ww0,varying=varying,numname=num))
###  if (silent<=-1) {
###    print("surv.boxarea after fast.reshape"); 
###    print(head(lr.data))
###    print(summary(lr.data[,id]))
###  }
###  lr.data[,boxtimevar] <- lr.data[,timevar]-lr.data[,"left"]
###  return(structure(lr.data,num=num,time=boxtimevar,status=status,covars=covars,id=id,left=left))
###} ## }}}

##' Bivariate Survival Data on Rectangular Regions
##'
##' Restricts bivariate survival data to a rectangular time region defined
##' by left-truncation and right-censoring boundaries, for use with
##' piecewise twostage models.
##'
##' @param left.trunc vector of length 2 giving left truncation times.
##' @param right.cens vector of length 2 giving right censoring times.
##' @param data a data.frame with the survival data.
##' @param timevar name of the time variable.
##' @param status name of the status variable.
##' @param id name of the cluster identifier.
##' @param covars optional covariate names.
##' @param covars.pairs optional pair-level covariate names.
##' @param num within-cluster numbering variable name.
##' @param silent verbosity level (1=silent).
##' @param boxtimevar name for the created box-time variable.
##' @return A data.frame in long format restricted to the specified region.
##' @export
surv_boxarea <- function(left.trunc,right.cens,data,timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1,boxtimevar="boxtime")
{ ## {{{
  if (is.null(data[,id])) stop("Wrong cluster variable")
  if (is.null(data[,timevar])) stop("Wrong time variable")
  if (is.null(data[,status])) stop("Wrong status variable")
  data <- data[order(data[,id]),]
  if (silent<=-1) {
    message("survboxare()")
    print(head(data))
    print(summary(data[,id]))
  }
  if (is.null(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,c(num)] <- unlist(lapply(idtab,seq_len))
  }

###  if (is.null(ssname)) {
###    idtab <- table(data[,id])
###    num <- "num"
###    while (num%in%names(data)) num <- paste(num,"_",sep="")
###    data[,c(num)] <- unlist(lapply(idtab,seq_len))
###  }

  timevar2 <- paste(timevar,1:2,sep="")
  status2 <- paste(status,1:2,sep="")
  num2 <- paste(num,1:2,sep="")
  covars2 <- NULL; covars.pairs2 <- NULL; 
  if (length(covars)>0) covars2 <- paste(covars,1:2,sep="")
  if (length(covars.pairs)>0) covars.pairs2 <- paste(covars.pairs,1:2,sep="")

  if (silent<=-1) {
    message("survboxare()")
    print(head(data))
    print( c(timevar,status,covars,covars.pairs,id,num))
    print(c(id,num))
    print(summary(data))
  }
  ww0 <- fast.reshape(data[,c(timevar,status,covars,covars.pairs,id,num)],id=id,num=num,labelnum=TRUE)
  if (silent<=-1) {
    message("survboxarea(), ww1")
    print(head(ww0))
    print(summary(ww0))
    print(c(timevar2,status2,covars2,covars.pairs2,id,num2)) 
    ww0 <- data.frame(ww0); 
    print(table(ww0$status1,ww0$status2))
  }
  mleft <-  (ww0[,timevar2[1]]>left.trunc[1]) & (ww0[,timevar2[2]]>left.trunc[2])  ## Both not-truncated

  if (length(na.idx <- which(is.na(mleft)))>0) {
    ##    warning("Removing incomplete cases", na.idx)
    mleft <- mleft[-na.idx]
    ww0 <- ww0[-na.idx,,drop=FALSE]
  }
  if (sum(mleft)==0) cat("No data selected\n"); 
  if (sum(mleft)!=0) {
  ww0 <- ww0[which(mleft),,drop=FALSE]

  right1 <- which(ww0[,timevar2[1]] > right.cens[1])
  right2 <- which(ww0[,timevar2[2]] > right.cens[2])
  ww0[,timevar2[1]][right1] <- right.cens[1]
  ww0[,timevar2[2]][right2] <- right.cens[2]
  ww0[,status2[1]][right1] <- 0
  ww0[,status2[2]][right2] <- 0
  truncvar2 <- c("left1","left2")
  ww0 <- cbind(ww0,left.trunc[1])
  ww0 <- cbind(ww0,left.trunc[2])
  colnames(ww0)[c(-1,0) + ncol(ww0)] <- truncvar2
###  ww0[,"intnames1"] <- paste("[",left.trunc[1],",",right.cens[1],")",sep="")
###  ww0[,"intnames2"] <- paste("[",left.trunc[2],",",right.cens[2],")",sep="")

  if (silent<=-1) print(head(ww0))
  if (silent<=0)
    message(paste("  Number of joint events:",sum(apply(ww0[,status2],1,sum)==2),"of ",nrow(ww0)),"\n");

  varying <- c(timevar,status,"left",covars)
  lr.data <- data.frame(fast.reshape(ww0,varying=varying,numname=num))
  if (silent<=-1) {
    print("surv.boxarea after fast.reshape"); 
    print(head(lr.data))
    print(summary(lr.data[,id]))
  }
  lr.data[,boxtimevar] <- lr.data[,timevar]-lr.data[,"left"]
###  print(head(lr.data))
 return(structure(lr.data,num=num,time=boxtimevar,status=status,covars=covars,id=id))
} else return(NULL); 
} ## }}}

dep_cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
                  cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,tol=1e-6,detail=0,
                  clusters=NULL,theta=NULL,theta.des=NULL,step=1,sym=1,weights=NULL,
		  same.cens=FALSE,censoring.weights=NULL,silent=1,entry=NULL,estimator=1,
		  trunkp=1,admin.cens=NULL,control=list(),par.func=NULL,dpar.func=NULL,dimpar=NULL,
		  score.method="nlminb",random.design=NULL,var.link=0,...)
{ ## {{{
  ## {{{ set up data and design
  multi<-0; dscore=1; stab.cens<-FALSE; entry.call<-entry; inverse<-var.link
  notaylor<-1; flex.func<-0; 

  ## extract design and time and cause from cif object 
  time <- cif$response[,"exit"]
  if (is.null(cause)) cause <- cif$response[,"cause"]  ##  attr(cif,"cause"); 
  cause <- as.numeric(cause)
  if (is.null(cens.code)) cens.code <- attr(cif,"cens.code")
  delta<-(cause!=cens.code)
  if (length(cause)!=length(time)) stop("cause and time not of same length\n"); 
  formula <- attr(cif,"Formula")
  ldata <- timereg::aalen.des2(formula(delete.response(terms(formula))),data=data,model="aalen")
  X<-ldata$X;  Z<-ldata$Z;  

  antpers<-nrow(X); 
  if (is.null(Z)==TRUE) {npar<-TRUE; semi<-0;}  else {Z<-as.matrix(Z); npar<-FALSE; semi<-1;}
  if (npar==TRUE) {Z<-matrix(0,antpers,1); pg<-1; fixed<-0;} else {fixed<-1;pg<-ncol(Z);} 
  ng<-antpers;px<-ncol(X);  
  if (is.null(times)) times<-cif$cum[,1]; 
  est<-as.matrix(cif$cum); 
  if (semi==1) gamma<-cif$gamma  else gamma<-0; 
  ntimes<-length(times); 
  if ((cif$model!="additive") && (cif$model!="fg")) 
    stop("Marginal Cumulative incidence model, must be either additive or extended Fine-Gray model\n")
  cif.model <- switch(cif$model,additive=1,fg=2)

  if (cause1!=attr(cif,"causeS")) cat("Cause for marginal model and correlation not the same\n"); 
  if ((cause1[1]!=cause2[1])) {
    if (is.null(cif2)==TRUE) stop("Must provide marginal model for both causes"); 
    formula2<-attr(cif2,"Formula"); 
    ldata2 <- timereg::aalen.des2(formula(delete.response(terms(formula2))),data=data,model="aalen");
    X2<-ldata2$X; Z2<-ldata$Z;  
    if (is.null(Z2)==TRUE) {npar2<-TRUE; semi2<-0;}  else {Z2<-as.matrix(Z2); npar2<-FALSE; semi2<-1;}
    if (npar2==TRUE) {Z2<-matrix(0,antpers,1); pg2<-1; fixed2<-0;} else {fixed2<-1;pg2<-ncol(Z2);} 
    px2<-ncol(X2);  
    est2<-cif2$cum; 
    if (semi2==1) gamma2<-cif2$gamma  else gamma2<-0; 
    est2<-cpred(est2,times);
  } else { 
    X2<-matrix(0,1,1); Z2<-matrix(0,1,1); pg2<-1; px2<-1;  semi2<-0; est2<-matrix(0,1,2); gamma2<-0; 
    npar2<-FALSE; 
  }


### For truncation 
  if (is.null(entry)) entry.call <- NULL else entry.call <- 0
  if (is.null(entry)) entry <- rep(0,antpers); 
  cum1<-cpred(rbind(rep(0,px+1),cif$cum),entry)[,-1];
  if (cif.model==1) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)- (Z %*% gamma )*entry)
  else if (cif.model==2) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)*exp(Z %*% gamma ))
  if ((model!="COR") && (cause1[1]!=cause2[1])) {
    cum2<-cpred(rbind(rep(0,px2+1),cif2$cum),entry)[,-1];
    if (cif.model==1) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)- (Z2 %*% gamma2 )*entry)
    else if (cif.model==2) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)*exp(Z2 %*% gamma2 ))
  } else {cum2 <- cum1; cif2entry <- cif1entry;}
  ## }}}


  ## {{{ censoring model stuff
  cens.weight <- cif$cens.weights ### censoring weights from cif function
  Gcxe <- 1; 
  ### when censoring probs given, wants them so cens.model="user.weights"
  if (!is.null(censoring.weights))  cens.model <- "user.weights"
  if (cens.model!="user.weights") {
    if (is.null(cens.weight) ) {
      if (cens.model=="KM") { ## {{{
        ud.cens<-survival::survfit(Surv(time,cause==cens.code)~+1); 
        Gfit<-cbind(ud.cens$time,ud.cens$surv)
        Gfit<-rbind(c(0,1),Gfit); 
        Gcx<-cpred(Gfit,time)[,2];
        if (is.null(entry.call)==FALSE) Gcxe<-cpred(Gfit,entry)[,2];
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx ## }}}
      } else if (cens.model=="cox") { ## {{{
        if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
        ud.cens<-timereg::cox.aalen(Surv(time,cause==cens.code)~prop(XZ),n.sim=0,robust=0);
        Gcx<-cpred(ud.cens$cum,time)[,2];
        RR<-exp(XZ %*% ud.cens$gamma)
        Gcx<-exp(-Gcx*RR)
        Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
        if (is.null(entry.call)==FALSE) {
          Gcxe<-cpred(Gfit,entry)[,2];
          Gcxe<-exp(-Gcxe*RR)
        }
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx  ## }}}
      } else if (cens.model=="aalen") {  ## {{{
        if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
        ud.cens<-timereg::aalen(Surv(time,cause==cens.code)~XZ,n.sim=0,robust=0);
        Gcx<-cpred(ud.cens$cum,time)[,-1];
        XZ<-cbind(1,XZ); 
        Gcx<-exp(-apply(Gcx*XZ,1,sum))
        Gcx[Gcx>1]<-1; Gcx[Gcx<0]<-0
        Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
        if (is.null(entry.call)==FALSE) {
          Gcxe<-cpred(Gfit,entry)[,-1];
          Gcxe<-exp(-apply(Gcxe*XZ,1,sum))
          Gcxe[Gcxe>1]<-1; Gcxe[Gcxe<0]<-0
        }
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx  ## }}}
      }
    } else { Gcx <- cens.weight; Gctimes <- cens.weight;} 
  } else {
    Gcx <- censoring.weights
    Gctimes <- cens.weight
  } 
  ntimes<-length(times); 
  ## }}}

  ## {{{ set up cluster + theta design + define iid variables 
  if (is.null(clusters)== TRUE) {
    ## take clusters from cif model 
    clusters <- attr(cif,"clusters"); 
    antclust<- length(unique(clusters)); 
    max.clust <- attr(cif,"max.clust")
    if (attr(cif,"coarse.clust")) 
      stop("Max.clust should be NULL in marginal model, or clusters should be given at call \n"); 
  } else {
    clus<-unique(clusters); antclust<-length(clus);
    clusters <- as.integer(factor(clusters, labels = 1:(antclust)))-1;
  }

  outc <- cluster_index(clusters,index.type=TRUE); 
  clustsize <- outc$cluster.size
  maxclust <- outc$maxclust
  clusterindex <- outc$idclust
  if (maxclust==1) stop("No clusters given \n"); 

 if (model!="ARANCIF") { 
  if (is.null(theta.des)==TRUE) { ptheta<-1; theta.des<-matrix(1,antpers,ptheta);
  colnames(theta.des) <- "intercept"; } else theta.des<-as.matrix(theta.des); 
  ptheta<-ncol(theta.des);
  if (is.null(dimpar) & is.null(par.func)) dimpar<-ptheta; 
 }

 if (model=="ARANCIF") { ### different parameters for Additive random effects 
     if (is.null(random.design)) random.design <- matrix(1,antpers,1); 
     dim.rv <- ncol(random.design); 
     if (is.null(theta.des)==TRUE) theta.des<-diag(dim.rv);
     dimpar <- ncol(theta.des); 
 
     if (nrow(theta.des)!=ncol(random.design)) stop("nrow(theta.des)!= ncol(random.design)"); 
 } else random.design <- matrix(0,1,1); 

  if ( (!is.null(par.func)) && is.null(dimpar) ) 
    stop("Must specify dimension of parameters when specifying R-functions\n")

  if ( (is.null(theta)==TRUE) && (model=="ARANCIF")) theta<-rep(0.5,dimpar);
  if (is.null(theta)==TRUE) theta<-rep(0.1,dimpar);
  if (length(theta)!=dimpar) theta<-rep(theta[1],dimpar);

  Biid<-c(); gamma.iid <- 0; B2iid<-c(); gamma2.iid <- 0; 
  if (notaylor==0) {
    for (i in 1:antclust) Biid<-cbind(Biid,cif$B.iid[[i]]); 
    if (npar==TRUE) gamma.iid<-0 else gamma.iid<-cif$gamma.iid;
    if ((model!="COR") && (cause1!=cause2))  {
      B2iid<-c()
      for (i in 1:antclust) B2iid<-cbind(B2iid,cif2$B.iid[[i]]); 
      if (npar2==TRUE) gamma2.iid<-0 else  gamma2.iid<-cif2$gamma.iid;
    } else { B2iid<-Biid; gamma2.iid<-gamma.iid; }
  }
  var.theta<-hess<-matrix(0,dimpar,dimpar); score<-rep(0,dimpar); 
  time.pow<-attr(cif,"time.pow"); 
  #if (sum(time.pow)==0) time.pow<-rep(1,pg); 
  theta.iid<-matrix(0,antclust,dimpar); 

  if ((nrow(theta.des)!=nrow(X)) && (model!="ARANCIF")) 
	  stop("Dependence design not consistent with data\n"); 
  if (length(trunkp)!=antpers) trunkp <- rep(1,antpers)
  if (is.null(weights)==TRUE) weights <- rep(1,antpers); 
  ## }}}

   ###  ## {{{ function and derivative
  if (is.null(par.func)==FALSE)
  {
     flex.func<-1; # use flexible design
  if (is.null(dpar.func)) {
	  cat("Must provide derivative that is used for iid decomposition for SE's\n"); 
	  score.method <- "nlminb"
	  dpar.func <- par.func
  }
  } ## }}}

  Zgamma <-  c(Z %*% gamma); 
  Z2gamma2 <- c(Z2 %*% gamma2); 
  dep.model <- 0;  
  dep.model <- switch(model,COR=1,RR=2,OR=3,RANCIF=4,ARANCIF=5)
  if (dep.model==0) stop("model must be COR, OR, RR, RANCIF, ARANCIF \n"); 
  if (dep.model<=4) rvdes <- matrix(0,1,1); 
  
  obj <- function(par) 
    { ## {{{

    if (is.null(par.func)==TRUE) {
	    Xtheta <- theta.des %*% par; DXtheta <- array(0,c(1,1,1));
    } else {
	  Xtheta <- c(); DXtheta <- array(0,c(length(times),antpers,dimpar));
	  if (is.null(dpar.func))
		  stop("Must also provide derivative of function wrt to parameters")
	  s <- 0
	  for (t in times) {
		  s <- 1+s
		  Xttheta <- par.func(par,t,theta.des)
		  if (length(Xttheta)!=antpers) stop("parfunc(par,t,theta.des) must length n"); 
		  Xtheta <- cbind(Xtheta,Xttheta)
		  Dttheta <- dpar.func(par,t,theta.des); 
		  if (dim(Dttheta)[1]!=antpers || dim(Dttheta)[2]!=dimpar) 
			  stop("dparfunc must return matrix n x dimpar when called on dparfunc(par,t,theta.des)"); 
		  DXtheta[s,,] <- Dttheta
	  }
    }

      outl<-.Call("cor", ## {{{
                 itimes=times,iy=time,icause=cause,iCA1=cause1,iKMc=Gcx, 
                 iz=X,iest=matrix(est[,-1],ncol=ncol(est)-1),iZgamma=c(Zgamma),isemi=semi,izsem=Z, 
                 itheta=c(par),iXtheta=Xtheta,iDXtheta=DXtheta,idimDX=dim(DXtheta),
		 ithetades=theta.des,
                 icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex,
                 iinverse=inverse,iCA2=cause2,
                 ix2=X2,isemi2=semi2,iest2=as.matrix(est2[,-1]),iZgamma2=c(Z2gamma2),
                 iflexfunc=flex.func,iiid=iid,isym=sym,iweights=weights, 
                 isamecens=as.numeric(same.cens),istabcens=as.numeric(stab.cens),
		 iKMtimes=Gctimes,isilent=silent,
                 icifmodel=cif.model,idepmodel=dep.model,
                 iestimator=estimator,ientryage=entry,icif1entry=cif1entry,
                 icif2entry=cif2entry,itrunkp=trunkp,irvdes=random.design,
                 ## Biid,gamma.iid,time.pow, 
                 ## B2iid,gamma2.iid,body(htheta),body(dhtheta),new.env(),
                 PACKAGE="mets"
		 ) 
      ## }}}

      attr(outl,"gradient") <-outl$score 
      if (oout==0) return(outl$ssf) else if (oout==1) return(sum(outl$score^2)) else return(outl)
    } ## }}}

  p <- theta
  iid <- 0;  ###no-iid representation for iterations
  if (score.method=="nr") { ## {{{
    oout <- 2;  ### output control for obj
    if (Nit>0) 
    for (i in 1:Nit)
    {
        oout <- 2
        out <- obj(p)
	hess <- out$Dscore
	if (!is.na(sum(hess))) hessi <- pinv(out$Dscore) else hessi <- hess
        if (detail==1) {## {{{
          print(paste("Fisher-Scoring ===================: it=",i)); 
          cat("theta:");print(c(p))
          cat("score:");print(c(out$score)); 
	  cat("hess:"); print(hess); 
        }## }}}
        delta <- hessi %*% out$score 
	## do not update last iteration 
	if (i<Nit) p <- p-delta* step
	if (is.nan(sum(out$score))) break; 
        if (sum(abs(out$score))<tol) break;
        if (max(delta)>20) { cat("NR increment > 20, lower step zize, increment= \n"); cat(delta); break; }
    }
    if (!is.nan(sum(p))) { ## {{{ iid decomposition
    oout <- 2
    theta <- p
    iid <- 1; 
    out <- obj(p) 
    score <- out$score
    hess <- out$Dscore
    } ## }}} 
    if (detail==1 & Nit==0) {## {{{
          print(paste("Fisher-Scoring ===================: final")); 
          cat("theta:");print(c(p))
          cat("score:");print(c(out$score)); 
	  cat("hess:"); print(hess); 
    }## }}}
    if (!is.na(sum(hess))) hessi <- pinv(out$Dscore) else hessi <- diag(nrow(hess))
    score1 <- score; 
    ## }}}
  } else if (score.method=="nlminb") { ## {{{ nlminb optimizer
    iid <- 0; oout <- 0; 
    tryCatch(opt <- nlminb(theta,obj,control=control),error=function(x) NA)
    if (detail==1) print(opt); 
    iid <- 1; 
    hess <- numDeriv::hessian(obj,opt$par)
    score <- numDeriv::jacobian(obj,opt$par)
    hessi <- pinv(hess);
    theta <- opt$par
    if (detail==1) cat("iid decomposition\n"); 
    oout <- 2; 
    out <- obj(opt$par)
    score1 <- out$score
  ## }}}
  } else if (score.method=="nlm") { ## {{{ nlm optimizer
    iid <- 0; oout <- 0; 
    tryCatch(opt <- nlm(obj,theta,hessian=TRUE,print.level=detail),error=function(x) NA)
    iid <- 1; 
    hess <- opt$hessian
    score <- opt$gradient
    if (detail==1) print(opt); 
    hessi <- pinv(hess);
    theta <- opt$estimate
    if (detail==1) cat("iid decomposition\n"); 
    oout <- 2; 
    out <- obj(opt$estimate)
    score1 <- out$score
  ## }}}
  }  else stop("score.methods = nlm nlminb nr\n"); 

  theta.iid <- out$theta.iid %*% hessi
  if (is.null(par.func)) var.theta  <- t(theta.iid) %*% theta.iid else var.theta <- hessi
  var.theta  <- t(theta.iid) %*% theta.iid 
  if (is.null(par.func)) thetanames <- colnames(theta.des) else
  thetanames <- rep("R-func",dimpar)
  ud <- list(theta=theta,score=score,hess=hess,hessi=hessi,var.theta=var.theta,
             theta.iid=theta.iid,score1=c(score1),thetanames=thetanames,
	     brierscore=out$ssf,p11=out$p11); 
  if (dep.model<=3) class(ud)<-"cor" 
  else if (dep.model==4) class(ud) <- "randomcif" 
  else if (dep.model==6) class(ud) <- "randomcif" 
  else if (dep.model==5) class(ud) <- "randomcifrv" 
  attr(ud, "Formula") <- formula
  attr(ud, "Clusters") <- clusters
  attr(ud,"cause1")<-cause1; attr(ud,"cause2")<-cause2
  attr(ud,"sym")<-sym; 
  attr(ud,"inverse")<-inverse; 
  attr(ud,"antpers")<-antpers; 
  attr(ud,"antclust")<-antclust; 
  attr(ud,"var.link")<-var.link; 
  if (dep.model==4) attr(ud, "Type") <- "randomcif"
  if (dep.model==6) attr(ud, "Type") <- "randomcif"
  if (model=="COR") attr(ud, "Type") <- "cor"
  if (model=="RR") attr(ud, "Type") <- "RR"
  if (model=="OR") attr(ud, "Type") <- "OR-cif"

  if (dep.model==5) attr(ud, "pardes") <- theta.des
  if (dep.model==5) attr(ud, "rv1") <- random.design[1,]
  return(ud);
} ## }}}

##' Cross-odds-ratio, OR or RR risk regression for competing risks
##'
##' Fits a parametric model for the log-cross-odds-ratio for the 
##' predictive effect of for the cumulative incidence curves for \eqn{T_1} 
##' experiencing cause i given that \eqn{T_2} has experienced a cause k :
##' \deqn{
##' \log(COR(i|k))  =  h(\theta,z_1,i,z_2,k,t)=_{default}  \theta^T z =  
##' }
##' with the log cross odds ratio being 
##' \deqn{
##' COR(i|k) = 
##' \frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' O(T_1 \leq t,cause_1=i)}  
##' }
##' the conditional odds divided by the unconditional odds, with the odds
##' being, respectively 
##' \deqn{
##' O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_1=k) = 
##' \frac{
##' P_x(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' P_x((T_1 \leq t,cause_1=i)^c | T_2 \leq t,cause_2=k)}
##' }
##' and 
##' \deqn{
##' O(T_1 \leq t,cause_1=i) = 
##' \frac{P_x(T_1 \leq t,cause_1=i )}{P_x((T_1 \leq t,cause_1=i)^c )}.
##' }
##' Here \eqn{B^c} is the complement event of \eqn{B},
##' \eqn{P_x} is the distribution given covariates 
##' (\eqn{x} are subject specific and \eqn{z} are cluster specific covariates), and 
##' \eqn{h()} is a function that is the simple identity 
##' \eqn{\theta^T z} by default.
##' 
##' The OR dependence measure is given by 
##' \deqn{
##' OR(i,k) = 
##' \log ( 
##' \frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' O(T_1 \leq t,cause_1=i) | T_2 \leq t,cause_2=k)}  
##' }
##' This measure is numerically more stabile than the COR measure, and is symetric in i,k.
##' 
##' The RR dependence measure is given by 
##' \deqn{
##' RR(i,k) = 
##' \log ( 
##' \frac{P(T_1 \leq t,cause_1=i , T_2 \leq t,cause_2=k)}{
##' P(T_1 \leq t,cause_1=i) P(T_2 \leq t,cause_2=k)}  
##' }
##' This measure is numerically more stabile than the COR measure, and is symetric in i,k.
##' 
##' The model is fitted under symmetry (sym=1), i.e., such that it is assumed 
##' that \eqn{T_1} and \eqn{T_2} can be interchanged and leads to
##' the same cross-odd-ratio (i.e.
##' \eqn{COR(i|k) = COR(k|i))}, 
##' as would be expected for twins 
##' or without symmetry as might be the case with mothers and daughters (sym=0). 
##' 
##' \eqn{h()} may be specified as an R-function of the parameters, 
##' see example below, but the default is that it is simply \eqn{\theta^T z}.
##'
##' @aliases or.cif rr.cif
##' @param cif a model object from the timereg::comp.risk function with the
##' marginal cumulative incidence of cause1, i.e., the event of interest, and whose
##' odds the comparision is compared to the conditional odds given cause2
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value. When missing it comes from marginal cif.
##' @param times time-vector that specifies the times used for the estimating euqations for the cross-odds-ratio estimation.
##' @param cause1 specificies the cause considered.
##' @param cause2 specificies the cause that is conditioned on.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param step specifies the step size for the Newton-Raphson algorithm.
##' @param sym specifies if symmetry is used in the model.
##' @param weights weights for estimating equations.
##' @param par.func parfunc
##' @param dpar.func dparfunc
##' @param dimpar dimpar
##' @param score.method "nlminb", can also use "nr".
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param censoring.weights these probabilities are used for the bivariate censoring dist.
##' @param silent 1 to suppress output about convergence related issues.
##' @param ... Not used.
##' @return returns an object of type 'cor'. With the following arguments:
##' \item{theta}{estimate of proportional odds parameters of model.}
##' \item{var.theta}{variance for gamma.  }
##' \item{hess}{the derivative of the used score.}
##' \item{score}{scores at final stage.}
##' \item{score}{scores at final stage.}
##' \item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @author Thomas Scheike
##' @references
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
##' 
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##' library("timereg")
##' data(multcif);
##' multcif$cause[multcif$cause==0] <- 2
##' zyg <- rep(rbinom(200,1,0.5),each=2)
##' theta.des <- model.matrix(~-1+factor(zyg))
##' 
##' times=seq(0.05,1,by=0.05) # to speed up computations use only these time-points
##' add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=1,
##'                n.sim=0,times=times,model="fg",max.clust=NULL)
##' add2 <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=2,
##'                n.sim=0,times=times,model="fg",max.clust=NULL)
##' 
##' out1 <- cor_cif(add,data=multcif,cause1=1,cause2=1)
##' summary(out1)
##' 
##' out2 <- cor_cif(add,data=multcif,cause1=1,cause2=1,theta.des=theta.des)
##' summary(out2)
##' 
##' ##out3 <- cor_cif(add,data=multcif,cause1=1,cause2=2,cif2=add2)
##' ##summary(out3)
##' ###########################################################
##' # investigating further models using parfunc and dparfunc
##' ###########################################################
##' set.seed(100)
##' prt<-sim_nordic_random(2000,cordz=2,cormz=5)
##' prt$status <-prt$cause
##' table(prt$status)
##' 
##' times <- seq(40,100,by=10)
##' cifmod <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=prt,
##'                     cause=1,n.sim=0,
##'                     times=times,conservative=1,max.clust=NULL,model="fg")
##' theta.des <- model.matrix(~-1+factor(zyg),data=prt)
##' 
##' parfunc <- function(par,t,pardes)
##' {
##' par <- pardes %*% c(par[1],par[2]) +
##'        pardes %*% c( par[3]*(t-60)/12,par[4]*(t-60)/12)
##' par
##' }
##' head(parfunc(c(0.1,1,0.1,1),50,theta.des))
##' 
##' dparfunc <- function(par,t,pardes)
##' {
##' dpar <- cbind(pardes, t(t(pardes) * c( (t-60)/12,(t-60)/12)) )
##' dpar
##' }
##' head(dparfunc(c(0.1,1,0.1,1),50,theta.des))
##' 
##' names(prt)
##' or1 <- or_cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
##'               same.cens=TRUE,theta=c(0.6,1.1,0.1,0.1),
##'               par.func=parfunc,dpar.func=dparfunc,dimpar=4,
##'               score.method="nr",detail=1)
##' summary(or1)
##' 
##' ## cor1 <- cor_cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
##' ##                 same.cens=TRUE,theta=c(0.5,1.0,0.1,0.1),
##' ##                 par.func=parfunc,dpar.func=dparfunc,dimpar=4,
##' ##                 control=list(trace=TRUE),detail=1)
##' ## summary(cor1)
##' 
##' ### piecewise contant OR model
##' gparfunc <- function(par,t,pardes)
##' {
##' 	cuts <- c(0,80,90,120)
##' 	grop <- diff(t<cuts)
##' paru  <- (pardes[,1]==1) * sum(grop*par[1:3]) +
##'     (pardes[,2]==1) * sum(grop*par[4:6])
##' paru
##' }
##' 
##' dgparfunc <- function(par,t,pardes)
##' {
##' 	cuts <- c(0,80,90,120)
##' 	grop <- diff(t<cuts)
##' par1 <- matrix(c(grop),nrow(pardes),length(grop),byrow=TRUE)
##' parmz <- par1* (pardes[,1]==1)
##' pardz <- (pardes[,2]==1) * par1
##' dpar <- cbind( parmz,pardz)
##' dpar
##' }
##' head(dgparfunc(rep(0.1,6),50,theta.des))
##' head(gparfunc(rep(0.1,6),50,theta.des))
##' 
##' or1g <- or_cif(cifmod,data=prt,cause1=1,cause2=1,
##'                theta.des=theta.des, same.cens=TRUE,
##'                par.func=gparfunc,dpar.func=dgparfunc,
##'                dimpar=6,score.method="nr",detail=1)
##' summary(or1g)
##' names(or1g)
##' head(or1g$theta.iid)
##' }
##' @export
##' @keywords survival
cor_cif<-function(cif,data,cause=NULL,times=NULL,
                  cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                  clusters=NULL, theta=NULL,theta.des=NULL,step=1,sym=0,weights=NULL, 
		  par.func=NULL,dpar.func=NULL,dimpar=NULL,
		  score.method="nlminb",same.cens=FALSE,censoring.weights=NULL,silent=1,...)
{ ## {{{
  fit <- dep_cif(cif=cif,data=data,cause=cause,model="COR",times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des,par.func=par.func,dpar.func=dpar.func,
		 dimpar=dimpar,
                 step=step,sym=sym,weights=weights,
                 score.method=score.method,same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,...)
  fit$call <- match.call()
  fit
} ## }}}

##' @rdname cif-nonpar
##' @export
rr_cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
                 cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                 clusters=NULL, theta=NULL,theta.des=NULL, step=1,sym=0,weights=NULL,
                 same.cens=FALSE,censoring.weights=NULL,silent=1,par.func=NULL,dpar.func=NULL,dimpar=NULL,
		 score.method="nlminb",entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,...)
{ ## {{{
  fit <- dep_cif(cif=cif,data=data,cause=cause,model="RR",cif2=cif2,times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des, step=step,sym=sym,weights=weights,
                 same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,
		 par.func=par.func,dpar.func=dpar.func,dimpar=dimpar, score.method=score.method,
                 entry=entry,estimator=estimator,trunkp=trunkp,admin.cens=admin.cens,...)
  fit$call <- match.call()
  fit
} ## }}}

##' @rdname cif-nonpar
##' @export
or_cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
                 cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                 clusters=NULL, theta=NULL,theta.des=NULL, step=1,sym=0, weights=NULL,
                 same.cens=FALSE,censoring.weights=NULL,silent=1,par.func=NULL,dpar.func=NULL,dimpar=NULL,
		 score.method="nlminb",entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,...)
{ ## {{{
  fit <- dep_cif(cif=cif,data=data,cause=cause,model="OR",cif2=cif2,times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des, step=step,sym=sym,weights=weights,
                 same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,
		 par.func=par.func,dpar.func=dpar.func,dimpar=dimpar,
		 score.method=score.method,
                 entry=entry,estimator=estimator,trunkp=trunkp,admin.cens=admin.cens,...)
  fit$call <- match.call()
  fit
} ## }}}

##' Fits a random effects  model describing the dependence in the cumulative 
##' incidence curves for subjects within a cluster.  Given the gamma distributed
##' random effects it is assumed that the cumulative incidence curves are indpendent, and
##' that the marginal cumulative incidence curves are on the form
##' \deqn{
##' P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) exp(z^T \beta))
##' }
##' We allow a regression structure for the random effects variances that may depend on
##' cluster covariates.
##'
##' @title Random effects model for competing risks data
##' @param cif a model object from the comp.risk function with the 
##' marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
##' odds the comparision is made with respect to
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value.
##' @param cif2 specificies model for cause2 if different from cause1.
##' @param cause1 cause of first coordinate.
##' @param cause2 cause of second coordinate.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param sym 1 for symmetry 0 otherwise
##' @param step specifies the step size for the Newton-Raphson algorith.m
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param var.link if var.link=1 then var is on log-scale.
##' @param score.method default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.
##' @param entry entry-age in case of delayed entry. Then two causes must be given.
##' @param trunkp gives probability of survival for delayed entry, and related to entry-ages given above.
##' @param ... extra arguments.
##' @return returns an object of type 'cor'. With the following arguments:
##' \item{theta}{estimate of proportional odds parameters of model.}
##' \item{var.theta}{variance for gamma.  }
##' \item{hess}{the derivative of the used score.}
##' \item{score}{scores at final stage.}
##' \item{score}{scores at final stage.}
##' \item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @export
##' @references
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' 
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), work in progress.
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##'  d <- sim_nordic_random(1000,delayed=TRUE,cordz=0.5,cormz=2,lam0=0.3,country=TRUE)
##'  times <- seq(50,90,by=10)
##'  add1 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'  times=times,cause=1,max.clust=NULL)
##' 
##'  ### making group indidcator 
##'  mm <- model.matrix(~-1+factor(zyg),d)
##' 
##'  out1<-random_cif(add1,data=d,cause1=1,cause2=1,theta=1,same.cens=TRUE)
##'  summary(out1)
##' 
##'  out2<-random_cif(add1,data=d,cause1=1,cause2=1,theta=1,
##' 		   theta.des=mm,same.cens=TRUE)
##'  summary(out2)
##' 
##' #########################################
##' ##### 2 different causes
##' #########################################
##' 
##'  add2 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'                   times=times,cause=2,max.clust=NULL)
##'  out3 <- random_cif(add1,data=d,cause1=1,cause2=2,cif2=add2,sym=1,same.cens=TRUE)
##'  summary(out3) ## negative dependence
##' 
##'  out4 <- random_cif(add1,data=d,cause1=1,cause2=2,cif2=add2,theta.des=mm,sym=1,same.cens=TRUE)
##'  summary(out4) ## negative dependence
##' }
##' @keywords survival
##' @author Thomas Scheike
random_cif<-function(cif,data,cause=NULL,cif2=NULL,
                     cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                     clusters=NULL,theta=NULL,theta.des=NULL,sym=1,
                     step=1,same.cens=FALSE,var.link=0,score.method="nr",
                     entry=NULL,trunkp=1,...)
{ ## {{{
  fit <- dep_cif(cif,data=data,cause=cause,model="RANCIF",cif2=cif2,sym=sym,
     cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
     clusters=clusters,theta=theta,theta.des=theta.des,step=step,same.cens=same.cens,
     var.link=var.link,score.method=score.method,entry=entry,trunkp=trunkp,...)
  fit$call <- match.call()
  fit
} ## }}}

##' @rdname cif-nonpar
##' @export
random.cif<-function(cif,...) random_cif(cif,...)

##' @rdname cif-nonpar
##' @export
Grandom.cif<-function(cif,...) Grandom_cif(cif,...)

##'Additive Random effects model for competing risks data for polygenetic modelling
##' 
##'Fits a random effects  model describing the dependence in the cumulative 
##'incidence curves for subjects within a cluster.  Given the gamma distributed
##'random effects it is assumed that the cumulative incidence curves 
##'are indpendent, and that the marginal cumulative incidence curves 
##'are on additive form
##'\deqn{
##'P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) - t z^T \beta)
##'}
##'
##'We allow a regression structure for the indenpendent gamma distributed 
##'random effects  and their variances that may depend on cluster covariates.
##'
##'random.design specificies the random effects for each subject within a cluster. This is
##'a matrix of 1's and 0's with dimension n x d.  With d random effects. 
##'For a cluster with two subjects, we let the random.design rows be 
##' \eqn{v_1} and \eqn{v_2}. 
##'Such that the random effects for subject 
##'1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
##'has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
##'subjects 1's random effect are Gamma distributed with 
##'mean \eqn{\lambda_1/v_1^T \lambda}
##'and variance \eqn{\lambda_1/(v_1^T \lambda)^2}. Note that the random effect 
##'\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
##'
##'The parameters \eqn{(\lambda_1,...,\lambda_d)}
##'are related to the parameters of the model
##'by a regression construction \eqn{pard} (d x k), that links the \eqn{d} 
##'\eqn{\lambda} parameters
##'with the (k) underlying \eqn{\theta} parameters 
##'\deqn{
##'\lambda = pard \theta 
##'}
##'
##'
##'
##' @param cif a model object from the timereg::comp.risk function with the
##' marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
##' odds the comparision is made with respect to
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value.
##' @param cif2 specificies model for cause2 if different from cause1.
##' @param times time points
##' @param cause1 cause of first coordinate.
##' @param cause2 cause of second coordinate.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param step specifies the step size for the Newton-Raphson algorith.m
##' @param sym 1 for symmetri and 0 otherwise
##' @param weights weights for score equations.
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param censoring.weights Censoring probabilities
##' @param silent debug information 
##' @param var.link if var.link=1 then var is on log-scale.
##' @param score.method default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.
##' @param entry entry-age in case of delayed entry. Then two causes must be given.
##' @param estimator estimator
##' @param trunkp gives probability of survival for delayed entry, and related to entry-ages given above.
##' @param admin.cens Administrative censoring
##' @param random.design specifies a regression design of 0/1's for the random effects.
##' @param ... extra arguments.
##' @return returns an object of type 'random.cif'. With the following arguments:
##'\item{theta}{estimate of parameters of model.}
##'\item{var.theta}{variance for gamma.  }
##'\item{hess}{the derivative of the used score.}
##'\item{score}{scores at final stage.}
##'\item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @export
##' @references
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika.
##'
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2013), Biostatitistics.
##'
##' Scheike, Holst, Hjelmborg (2014),  LIDA,  
##' Estimating heritability for cause specific hazards based on twin data
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##'  d <- sim_nordic_random(1000,delayed=TRUE,
##'        cordz=1.0,cormz=2,lam0=0.3,country=TRUE)
##'  times <- seq(50,90,by=10)
##'  addm <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'  times=times,cause=1,max.clust=NULL)
##' 
##'  ### making group indidcator 
##'  mm <- model.matrix(~-1+factor(zyg),d)
##' 
##'  out1m<-random_cif(addm,data=d,cause1=1,cause2=1,theta=1,
##' 		   theta.des=mm,same.cens=TRUE)
##'  summary(out1m)
##'  
##'  ## this model can also be formulated as a random effects model 
##'  ## but with different parameters
##'  out2m<-Grandom_cif(addm,data=d,cause1=1,cause2=1,
##' 		    theta=c(0.5,1),step=1.0,
##' 		    random.design=mm,same.cens=TRUE)
##'  summary(out2m)
##'  1/out2m$theta
##'  out1m$theta
##'  
##'  ####################################################################
##'  ################### ACE modelling of twin data #####################
##'  ####################################################################
##'  ### assume that zygbin gives the zygosity of mono and dizygotic twins
##'  ### 0 for mono and 1 for dizygotic twins. We now formulate and AC model
##'  zygbin <- d$zyg=="DZ"
##' 
##'  n <- nrow(d)
##'  ### random effects for each cluster
##'  des.rv <- cbind(mm,(zygbin==1)*rep(c(1,0)),(zygbin==1)*rep(c(0,1)),1)
##'  ### design making parameters half the variance for dizygotic components
##'  pardes <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0), c(0,1))
##' 
##'  outacem <-Grandom_cif(addm,data=d,cause1=1,cause2=1,
##' 		same.cens=TRUE,theta=c(0.35,0.15),
##'             step=1.0,theta.des=pardes,random.design=des.rv)
##'  summary(outacem)
##' 
##' }
##' @keywords survival
##' @author Thomas Scheike
##' @export
Grandom_cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
clusters=NULL, theta=NULL,theta.des=NULL, weights=NULL, step=1,sym=0,
same.cens=FALSE,censoring.weights=NULL,silent=1,var.link=0,score.method="nr",
entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,random.design=NULL,...)
{ ## {{{
fit <- dep_cif(cif,data=data,cause=cause,model="ARANCIF",cif2=cif2,times=times,
         cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
         clusters=clusters,theta=theta,theta.des=theta.des,step=step,sym=sym,weights=weights,
         same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,var.link=var.link,
	 score.method=score.method,entry=entry,estimator=estimator,
	 random.design=random.design,trunkp=trunkp,admin.cens=admin.cens,...)
    fit$call <- match.call()
    fit
} ## }}}

##' @export
print.summary.cor <- function(x,digits=3,...)
{ ## {{{
  if (x$type=="cor") {
    cat("Cross odds ratio dependence for competing risks\n\n")
    cat("Effect of cause1=",x$cause1," on cause2=",x$cause2,
        " under symmetry=",x$sym,fill=TRUE,sep="")
  } else if (x$type=="RR") {
    cat("Ratio of joint and product of marginals for competing risks\n\n")
    cat("Ratio of cumulative incidence for cause1=",x$cause1," and cause2=",x$cause2,sep=" ")
  } else if (x$type=="OR-cif") {
    cat("OR for dependence for competing risks\n\n")
    cat("OR of cumulative incidence for cause1=",x$cause1," and cause2=",x$cause2,sep=" ")
  }
  cat("\n")
  prmatrix(signif(x$estimates,digits))
  cat("\n")
  if (!is.null(x$marg)) {
    cat(paste("Marginal cumulative incidencen",signif(x$marg,digits),"\n"))
    prmatrix(signif(x$casewise,digits))
    prmatrix(signif(x$concordance,digits))
    cat("\n")
  }
  invisible(x)
} ## }}}

##' Computes concordance and casewise concordance for dependence models for competing risks 
##' models of the type cor_cif, rr_cif or or_cif for the given cumulative incidences and the different dependence
##' measures in the object.
##'
##'
##' @title Summary for dependence models for competing risks
##' @param object object from cor_cif rr_cif or or_cif for dependence between competing risks data for two causes.
##' @param marg.cif a number that gives the cumulative incidence in one time point for which concordance and 
##' casewise concordance are computed.
##' @param marg.cif2 the cumulative incidence for cause 2 for concordance and 
##' casewise concordance are computed. Default is that it is the same as marg.cif.
##' @param digits digits in output.
##' @param ... Additional arguments.
##' @return prints summary for dependence model. 
##' \item{casewise}{gives casewise concordance that is, probability of cause 2 (related to cif2) given that cause 1 (related to cif1)
##' 	has occured.}
##' \item{concordance}{gives concordance that is, probability of cause 2 (related to cif2) and cause 1 (related to cif1).}
##' \item{cif1}{cumulative incidence for cause1.}
##' \item{cif2}{cumulative incidence for cause1.}
##' @references
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
##' 
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' 
##' @author Thomas Scheike
##' @keywords survival
##' @examples
##' ## library("timereg")
##' ## data("multcif",package="mets") # simulated data 
##' ## multcif$cause[multcif$cause==0] <- 2
##' ##  
##' ## times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
##' ## add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),
##' ##                           data=multcif,n.sim=0,times=times,cause=1)
##' ###
##' ## out1<- cor_cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
##' ## summary(out1)
##' ## 
##' ## pad <- predict(add,X=1,se=0,uniform=0)
##' ## summary(out1,marg.cif=pad)
##' @method summary cor
##' @export
summary.cor <- function(object,marg.cif=NULL,marg.cif2=NULL,digits=3,...) { ## {{{
  if (!(attr(object,"class") %in% c("cor","randomcif"))) stop("Must be a cor_cif or randomcif object")
  if (sum(abs(object$score))>0.001) warning("WARNING: check score for convergence\n")

  coefs <- coef(object,...) 

  ocasewise <- oconcordance <- NULL
  if (is.null(marg.cif)==FALSE) { ## {{{ 
    time <- marg.cif$time
    marg.cif <- marg.cif$P1
    if (is.null(marg.cif2)==FALSE) marg.cif2 <- marg.cif2$P1 else {
    if (attr(object,"cause2")==attr(object,"cause1")) marg.cif2 <- marg.cif 
    else stop("causes not the same and second marginal cif not given\n"); 
    }
    pmarg.cif <- marg.cif*marg.cif2

    thetav <- coefs[,1]
    thetavl <- thetav-1.96*coefs[,2]
    thetavu <- thetav+1.96*coefs[,2]
    namev <- object$thetanames
    if (is.null(namev)) namev <- paste("name",1:length(thetav),sep="")

    ocasewise <- oconcordance <- list()
    k <- 0
    for (theta in thetav) { 
       k <- k+1
       thetal <- thetavl[k]; 
       thetau <- thetavu[k]
    if (attr(object,"Type")=="cor") { ## {{{
      concordance <- exp(theta)*pmarg.cif/((1-marg.cif)+exp(theta)*marg.cif)
      conclower  <- exp(thetal)*pmarg.cif/((1-marg.cif)+exp(thetal)*marg.cif)
      concup  <- exp(thetau)*pmarg.cif/((1-marg.cif)+exp(thetau)*marg.cif)
      casewise <- concordance/marg.cif
      caselower  <- conclower/marg.cif
      caseup     <- concup/marg.cif
    } else if (attr(object,"Type")=="RR") {
      casewise<- exp(coefs[,1])*c(marg.cif)
      concordance <- exp(coefs[,1])*pmarg.cif
      caselower  <- marg.cif*exp(thetal)
      caseup     <- marg.cif*exp(thetau)
      conclower  <- pmarg.cif* exp(thetal)
      concup     <- pmarg.cif*exp(thetau)
    } else if (attr(object,"Type")=="OR-cif") {
      casewise<- plack.cif2(marg.cif,marg.cif,c(theta))/marg.cif
      concordance <- plack.cif2(marg.cif,marg.cif,c(theta))
      caselower  <- plack.cif2(marg.cif,marg.cif,thetal)/marg.cif
      caseup  <- plack.cif2(marg.cif,marg.cif,thetau)/marg.cif
      conclower  <- plack.cif2(marg.cif,marg.cif,thetal)
      concup     <- plack.cif2(marg.cif,marg.cif,thetau)
    } else if (attr(object,"Type")=="randomcif") {
	    theta <- 1/theta
	    thetal <- 1/thetal
	    thetau <- 1/thetau
      lam <- 1-marg.cif
      p11<- 1-lam -lam +lap(theta,2*ilap(theta, lam))
      concordance <- p11 
      conclower  <- 1-lam -lam +lap(thetal,2*ilap(thetal, lam))
      concup     <- 1-lam -lam +lap(thetau,2*ilap(thetau, lam))
      casewise <- concordance/marg.cif
      caselower  <- conclower/marg.cif
      caseup     <- concup/marg.cif
    } ## }}} 
    outcase <- cbind(time,casewise,caselower,caseup)
    outconc <- cbind(time,concordance,conclower,concup)
    colnames(outcase) <- c("time","casewise concordance","2.5 %","97.5%")
    colnames(outconc) <- c("time","concordance","2.5 %","97.5%")
	  if (length(thetav)==1) {
	     ocasewise <- outcase
	     oconcordance <- outconc
	  } else {
	     ocasewise[[k]] <- outcase
	     oconcordance[[k]] <- outconc
	  }
  } 

  if (length(thetav)>1) {
  if (length(ocasewise)==length(namev)) {
    names(ocasewise) <- namev
    names(oconcordance) <- namev
  }
  }

  } ## }}} 


  res <- list(casewise=ocasewise,concordance=oconcordance,estimates=coefs,
	      marg.cif=marg.cif, marg.cif2=marg.cif2,type=attr(object,"Type"),
	      sym=attr(object,"sym"),cause1=attr(object,"cause1"),cause2=attr(object,"cause2"))
  class(res) <- "summary.cor"
  res
} ## }}}

##' @export
coef.cor <- function(object,...)
{ ## {{{
  res <- cbind(object$theta, diag(object$var.theta)^0.5)
  se<-diag(object$var.theta)^0.5
  wald <- object$theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  cor<-exp(object$theta)
  res <- as.matrix(cbind(res, wald, waldp,cor,se*cor))
  if (attr(object,"Type")=="cor") 
    colnames(res) <- c("log-Coef.", "SE", "z", "P-val","Cross odds ratio","SE")
  else colnames(res) <- c("log-ratio Coef.", "SE", "z", "P-val","Ratio","SE")
  if (!is.null(object$thetanames)) rownames(res)<-object$thetanames

  return(res)
} ## }}}

##' @export
print.cor<-function(x,digits=3,...)
{ ## {{{
  print(x$call); 
  cat("\n")
  print(summary(x)); 
} ## }}}

##' @export
concordance.cor <- function(object,...) {
    concordanceCor(object,...)
}

##' Concordance for Twins
##'
##' The concordance is the probability that both twins have experienced the 
##' event of interest and is defined as 
##' \deqn{
##'  cor(t) = P(T_1 \leq t, \epsilon_1 =1 , T_2 \leq t, \epsilon_2=1) 
##'  }
##'
##' Similarly, the casewise concordance is 
##' \deqn{
##'  casewise(t) = \frac{cor(t)}{P(T_1 \leq t, \epsilon_1=1) }
##' }
##' that is the probability that twin "2" has the event given that twins "1" has. 
##'
##' @title Concordance Computes concordance and casewise concordance
##' @param object Output from the cor_cif, rr_cif or or_cif function
##' @param cif1 Marginal cumulative incidence
##' @param cif2 Marginal cumulative incidence of other cause (cause2) if  it is different from cause1
##' @param messages  To print messages
##' @param model  Specfifies wich model that is considered if object not given.
##' @param coefs Specfifies  dependence parameters if object is not given.
##' @param ...  Extra arguments, not used.
##' @author Thomas Scheike
##' @aliases concordanceCor concordance.cor 
##' @references
##' Estimating twin concordance for bivariate competing risks twin data
##' Thomas H. Scheike, Klaus K. Holst and Jacob B. Hjelmborg, 
##' Statistics in Medicine 2014, 1193-1204
##' 
##' Estimating Twin Pair Concordance for Age of Onset.
##' Thomas H. Scheike, Jacob V B Hjelmborg, Klaus K. Holst, 2015 
##' in Behavior genetics DOI:10.1007/s10519-015-9729-3
##' 
##' @export 
concordanceCor <- function(object,cif1,cif2=NULL,messages=TRUE,model=NULL,coefs=NULL,...)
{ ## {{{

  if (is.null(model)) { 
    if (!inherits(object, "cor")) stop("Must be a rr_cif, cor_cif or or_cif object")
    model <- attr(object,"Type")
  } 
  if (is.null(coefs)) coefs <- coef(object)
  if (is.null(coefs)) stop("Must give dependence parameters\n"); 

  if (!is.null(object)) { 
    cause1 <-  attr(object,"cause1");
    cause2 <-  attr(object,"cause2"); 
  } else cause1 <- cause2 <- 1

  if (is.null(cif2)==TRUE) cif2 <- cif1; 

  if (messages) {
    if (model=="cor") { ## {{{
      message("Cross odds ratio dependence for competing risks\n\n")
      message("Odds of cause1=",cause1," given cause2=",cause2," relative to Odds of cause1=",cause1,"\n",fill=TRUE,sep="")
    } else if (model=="RR") {
      message("Ratio of joint and product of marginals for competing risks\n\n")
      message("Ratio of cumulative incidence for cause1=",cause1," and cause2=",cause2,sep=" ")
    } else if (model=="OR-cif") {
      message("OR for dependence for competing risks\n\n")
      message("OR of cumulative incidence for cause1=",cause1," and cause2=",cause2,sep=" ")
    } ## }}}
  }

  out <- list()
  for (k in 1:nrow(coefs)) {
    ## {{{
    if (model=="cor") {
      concordance <- exp(coefs[k,1])*cif1*cif2/((1-cif1)+exp(coefs[k,1])*cif1)
      conclower  <- exp(coefs[k,1]-1.96*coefs[k,2])*cif1*cif2/((1-cif1)+exp(coefs[k,1]-1.96*coefs[k,2])*cif1)
      concup  <- exp(coefs[k,1]+1.96*coefs[k,2])*cif1*cif2/((1-cif1)+exp(coefs[k,1]+1.96*coefs[k,2])*cif1)
      casewise <- concordance/cif1
      caselower  <- conclower/cif1
      caseup     <- concup/cif1
    } else if (model=="RR") {
      casewise<- exp(coefs[k,1])*c(cif2)
      concordance <- exp(coefs[k,1])*cif1*cif2
      caselower  <- cif2*exp(coefs[k,1]-1.96*coefs[k,2])
      caseup     <- cif2*exp(coefs[k,1]+1.96*coefs[k,2])
      conclower  <- cif1*cif2* exp(coefs[k,1]-1.96*coefs[k,2])
      concup     <- cif1*cif2*exp(coefs[k,1]+1.96*coefs[k,2])
    } else if (model=="OR-cif") {
      thetal <-  coefs[k,1]-1.96*coefs[k,2]
      thetau <-  coefs[k,1]+1.96*coefs[k,2]
      casewise<- plack.cif2(cif1,cif2,c(coefs[k,1]))/cif1
      concordance <- plack.cif2(cif1,cif2,c(coefs[k,1]))
      caselower  <- plack.cif2(cif1,cif2,thetal)/cif1
      caseup  <- plack.cif2(cif1,cif2,thetau)/cif1
      conclower  <- plack.cif2(cif1,cif2,thetal)
      concup     <- plack.cif2(cif1,cif2,thetau)
    }

    outcase <- cbind(c(casewise),c(caselower),c(caseup))
    outconc <- cbind(c(concordance),c(conclower),c(concup))
    colnames(outcase) <- c("casewise concordance","2.5 %","97.5%")
    colnames(outconc) <- c("concordance","2.5 %","97.5%")
    ## }}}

    out[[k]] <- list(concordance=outconc,casewise=outcase)
    names(out)[k] <- rownames(coefs)[k]
  }

  return(out)
} ## }}}

##' .. content for description (no empty lines) ..
##'
##' @title plack Computes concordance for or.cif based model, that is Plackett random effects model 
##' @aliases plack.cif2
##' @export
##' @param cif1 Cumulative incidence of first argument.
##' @param cif2 Cumulative incidence of second argument.
##' @param object or.cif object with dependence parameters.
##' @author Thomas Scheike
plack_cif <- function(cif1,cif2,object) 
{ ## {{{
  coefs <- coef(object)
  theta <- exp(object$theta); 
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;
  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  valr <- matrix(valr,length(c(theta)),1)
  rownames(valr)=colnames(coefs)
  return(valr); 
} ## }}}

##' @export
plack.cif2 <- function(cif1,cif2,theta) 
{ ## {{{
  theta <- exp(c(theta))
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;

  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  return(valr); 
} ## }}}

##' @rdname cif-nonpar
##' @export
predictPairPlack <- function(cif1,cif2,status1,status2,theta) 
{ ## {{{
  theta <- exp(c(theta))
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;

  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  p11 <- valr; 
  p10 <- cif1-p11
  p01 <- cif2-p11
  p00 <- 1- p10-p01-p11
#
  pred <- (status1==1)*(status2==1)*p11+ (status1==1)*(status2==0)*p10+ 
          (status1==0)*(status2==1)*p01+ (status1==0)*(status2==0)*p00

  return(pred); 
} ## }}}

##' @export
summary.randomcif<-function (object, ...) 
{ ## {{{
  if (!inherits(object, "randomcif")) 
    stop("Must be a random.cif  object")
  cat("Random effect variance for variation due to clusters\n\n")
  cat("Cause", attr(object, "cause1"), "and cause", attr(object, 
                                                         "cause2"), fill = TRUE)
  cat("\n")
  if (sum(abs(object$score)) > 0.001) cat("WARNING: check score for convergence")
  cat("\n")
  coef.randomcif(object, ...)
} ## }}}

##' @export
coef.randomcif<- function (object, digits = 3, ...) 
{ ## {{{
  res <- cbind(object$theta, diag(object$var.theta)^0.5)
  se <- diag(object$var.theta)^0.5
  wald <- object$theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  cor <- object$theta + 1
  res <- as.matrix(cbind(res, wald, waldp, cor, se))
  colnames(res) <- c("Coef.", "SE", "z", "P-val", "Cross odds ratio", "SE")
  if (!is.null(object$thetanames)) rownames(res)<-object$thetanames

  return(res)
} ## }}}

##' @export
print.randomcif<- function (x , digits = 3, ...) 
{ ## {{{
} ## }}}

##' @export
summary.randomcifrv<-function (object, ...) 
{ ## {{{
    if (!inherits(object, "randomcifrv")) 
        stop("Must be a random.cifrv  object")
    cat("Random effect parameters for additive gamma random effects \n\n")
    cat("Cause", attr(object, "cause1"), "and cause", attr(object, 
        "cause2"), fill = TRUE)
    cat("\n")
    if (sum(abs(object$score)) > 1e-06) 
        cat("WARNING: check score for convergence")
    cat("\n")

    res <-     coef.randomcifrv(object, ...)

    var.link <- attr(object,"var.link"); 
    rv1 <- attr(object,"rv1"); 
    theta.des <- attr(object,"pardes"); 
    if (var.link==1) par <- theta.des %*% exp(object$theta) else  par <- theta.des %*% object$theta
    if (var.link==1) {
	      fp <- function(p){  res <- exp(p)/sum(rv1* (theta.des %*% exp(p))); return(res); }
              e <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) fp(p))
              pare <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p))
              res <- list(estimate=res,h=e,exppar=pare)
     } else {
              fp <- function(p) {  p/sum(rv1* (theta.des %*% p)) }
              e <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) fp(p))
              res <- list(estimate=res,h=e)
    }

  res
} ## }}}

##' @export
coef.randomcifrv<- function (object, digits = 3, ...) 
{ ## {{{
    if (attr(object,"inverse")==1) elog <- 1 else elog  <- 0; 
    if (elog==1) theta <- exp(object$theta) else theta <- object$theta 
    se <- diag(object$var.theta)^0.5
    res <- cbind(object$theta, se)
    wald <- object$theta/se
    waldp <- (1 - pnorm(abs(wald))) * 2
    res <- as.matrix(cbind(res, wald, waldp))
    if (elog==0)  colnames(res) <- c("Coef.", "SE", "z", "P-val")
    if (elog==1) res <- cbind(res,exp(object$theta), exp(object$theta)^2*se)  
    if (elog==1) colnames(res) <- c("log-parameter","SE","z","P-val","exp(theta)","SE")

   if (!is.null(object$thetanames)) rownames(res)<-object$thetanames
   prmatrix(signif(res, digits))

} ## }}}

##' @export
print.randomcifrv<- function (x , digits = 3, ...) 
{ ## {{{
 summary(x, ...)
} ## }}}

## summary.cor<-function(object,digits=3,marg.cif=NULL,...)
## { ## {{{
##   if (!inherits(object, "cor")) stop("Must be a cor_cif  object")
##   if (sum(abs(object$score))>0.001) warning("WARNING: check score for convergence\n")
##   coefs <- coef.cor(object,...);

##   outcase <- outconc <- NULL
##   if (is.null(marg.cif)==FALSE) {
##     marg.cif <- max(marg.cif)
##     ## {{{
##     if (attr(object,"Type")=="cor") {
##       concordance <- exp(coefs[,1])*marg.cif^2/((1-marg.cif)+exp(coefs[,1])*marg.cif)
##       conclower  <- exp(coefs[,1]-1.96*coefs[,2])*marg.cif^2/((1-marg.cif)+exp(coefs[,1]-1.96*coefs[,2])*marg.cif)
##       concup  <- exp(coefs[,1]+1.96*coefs[,2])*marg.cif^2/((1-marg.cif)+exp(coefs[,1]+1.96*coefs[,2])*marg.cif)
##       casewise <- concordance/marg.cif
##       caselower  <- conclower/marg.cif
##       caseup     <- concup/marg.cif
##     } else if (attr(object,"Type")=="RR") {
##       casewise<- exp(coefs[,1])*c(marg.cif)
##       concordance <- exp(coefs[,1])*marg.cif^2
##       caselower  <- marg.cif*exp(coefs[,1]-1.96*coefs[,2])
##       caseup     <- marg.cif*exp(coefs[,1]+1.96*coefs[,2])
##       conclower  <- marg.cif^2* exp(coefs[,1]-1.96*coefs[,2])
##       concup     <- marg.cif^2*exp(coefs[,1]+1.96*coefs[,2])
##     } else if (attr(object,"Type")=="OR-cif") {
##       thetal <-  coefs[,1]-1.96*coefs[,2]
##       thetau <-  coefs[,1]+1.96*coefs[,2]
##       casewise<- plack.cif2(marg.cif,marg.cif,c(coefs[,1]))/marg.cif
##       concordance <- plack.cif2(marg.cif,marg.cif,c(coefs[,1]))
##       caselower  <- plack.cif2(marg.cif,marg.cif,thetal)/marg.cif
##       caseup  <- plack.cif2(marg.cif,marg.cif,thetau)/marg.cif
##       conclower  <- plack.cif2(marg.cif,marg.cif,thetal)
##       concup     <- plack.cif2(marg.cif,marg.cif,thetau)
##     }
##     outcase <- cbind(casewise,caselower,caseup)
##     outconc <- cbind(concordance,conclower,concup)
##     rownames(outcase) <- rownames(outconc)  <-  rownames(coefs)
##     colnames(outcase) <- c("casewise concordance","2.5 %","97.5%")
##     colnames(outconc) <- c("concordance","2.5 %","97.5%")
##   }
##   ## }}}
##   res <- list(casewise=outcase,concordance=outconc,estimates=coefs,marg=marg.cif,type=attr(object,"Type"),sym=attr(object,"sym"),cause1=attr(object,"cause1"),cause2=attr(object,"cause2"))
##   class(res) <- "summary.cor"
##   res
## } ## }}}

## coef.cor<-function(object,...)
## { ## {{{
##   res <- cbind(object$theta, diag(object$var.theta)^0.5)
##   se<-diag(object$var.theta)^0.5
##   wald <- object$theta/se
##   waldp <- (1 - pnorm(abs(wald))) * 2
##   cor<-exp(object$theta)
##   res <- as.matrix(cbind(res, wald, waldp,cor,se*cor))
##   if (attr(object,"Type")=="cor") 
##     colnames(res) <- c("log-Coef.", "SE", "z", "P-val","Cross odds ratio","SE")
##   else colnames(res) <- c("log-ratio Coef.", "SE", "z", "P-val","Ratio","SE")
##   if (is.null((rownames(res)))==TRUE) rownames(res)<-rep(" ",nrow(res))

##   return(res)
## } ## }}}

marginalprobit <- function(mu,dmu,S,dS,y,w=NULL,indiv=FALSE,...) {
  sigma <- S^0.5
  alpha <- alpha0 <- pnorm(mu,sd=sigma)
  alpha[y==0] <- 1-alpha[y==0]
  M <- -sigma*dnorm(mu/sigma)
  V <- S*alpha0+mu*M  
  U1 <- 0.5*t(dS)%*%(-alpha0+V/S)/S
  U <- matrix(0,ncol(dmu)+nrow(U1),ncol(U1))
  if (is.null(w)) w <- rep(1,ncol(U))
  for (i in seq(ncol(U))) {
    U[,i] <- c(- dmu[i,,drop=FALSE]*(1/S*M[i]),U1[,i])/alpha[i]*
       ifelse(y[i]==0,-1,1)*w[i]
  }  
  if (indiv) return(structure(t(U),logLik=log(alpha)))
  return(structure(rowSums(U),logLik=sum(log(alpha))))
}

##' Fits Clayton-Oakes or bivariate Plackett (OR) models for binary data
##' using marginals that are on logistic form.
##' If clusters contain more than two times, the algoritm uses a compososite likelihood
##' based on all pairwise bivariate models.
##'
##' The pairwise pairwise odds ratio model provides an alternative to the alternating logistic
##' regression (ALR).
##'
##' The reported standard errors are based on a cluster corrected score equations from the
##' pairwise likelihoods assuming that the marginals are known. This gives correct standard errors
##' in the case of the Odds-Ratio model (Plackett distribution) for dependence, but incorrect standard
##' errors for the Clayton-Oakes types model (that is also called "gamma"-frailty). For the additive gamma version of the
##' standard errors
##' are adjusted for the uncertainty in the marginal models via an iid deomposition using the iid() function of
##' lava. For the clayton oakes model that is not speicifed via the random effects these can be
##' fixed subsequently using the iid influence functions for the marginal model, but typically this does not
##' change much.
##'
##' For the Clayton-Oakes version of the model, given the gamma distributed random effects it is
##' assumed that the probabilities are indpendent, and that the marginal survival functions are on logistic form
##' \deqn{
##' logit(P(Y=1|X)) = \alpha + x^T \beta
##' }
##' therefore conditional on the random effect the probability of the event is
##' \deqn{
##' logit(P(Y=1|X,Z)) = exp( -Z \cdot Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
##' }
##'
##' Can also fit a structured additive gamma random effects model, such
##' the ACE, ADE model for survival data:
##'
##' Now random.design specificies the random effects for each subject within a cluster. This is
##' a matrix of 1's and 0's with dimension n x d.  With d random effects.
##' For a cluster with two subjects, we let the random.design rows be
##'  \eqn{v_1} and \eqn{v_2}.
##' Such that the random effects for subject
##' 1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
##' has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
##' subjects 1's random effect are Gamma distributed with
##' mean \eqn{\lambda_j/v_1^T \lambda}
##' and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
##' \eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
##' It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed over all clusters
##' as it would be for the ACE model below.
##'
##' The DEFAULT parametrization uses the variances of the random effecs (var.par=1)
##' \deqn{
##' \theta_j  = \lambda_j/(v_1^T \lambda)^2
##' }
##'
##' For alternative parametrizations (var.par=0) one can specify how the parameters relate
##' to \eqn{\lambda_j} with the function
##'
##' Based on these parameters the relative contribution (the heritability, h) is
##' equivalent to  the expected values of the random effects  \eqn{\lambda_j/v_1^T \lambda}
##'
##' Given the random effects the probabilities  are independent and on the form
##' \deqn{
##' logit(P(Y=1|X)) = exp( - Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
##' }
##' with the inverse laplace of the gamma distribution with mean 1 and variance lamtot.
##'
##' The parameters \eqn{(\lambda_1,...,\lambda_d)}
##' are related to the parameters of the model
##' by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
##' \eqn{\lambda} parameters
##' with the (k) underlying \eqn{\theta} parameters
##' \deqn{
##' \lambda = theta.des  \theta
##' }
##' here using theta.des to specify these low-dimension association. Default is a diagonal matrix.
##'
##' @references
##' Two-stage binomial modelling
##' @examples
##' library(mets)  ## do not delete this line 
##' set.seed(123)  ## do not delete this line 
##' data(twinstut)
##' twinstut0 <- subset(twinstut, tvparnr<4000)
##' twinstut <- twinstut0
##' twinstut$binstut <- (twinstut$stutter=="yes")*1
##' theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
##' margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
##' bin <- binomial_twostage(margbin,data=twinstut,var.link=1,
##'          clusters=twinstut$tvparnr,theta.des=theta.des,detail=0)
##' summary(bin)
##'
##' twinstut$cage <- scale(twinstut$age)
##' theta.des <- model.matrix( ~-1+factor(zyg)+cage,data=twinstut)
##' bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des)
##' summary(bina)
##'
##' theta.des <- model.matrix( ~-1+factor(zyg)+factor(zyg)*cage,data=twinstut)
##' bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des)
##' summary(bina)
##'
##' ### use of clayton oakes binomial additive gamma model
##' ###########################################################
##' \donttest{ ## Reduce Ex.Timings
##' data <- sim_binClaytonOakes_family_ace(1000,2,1,beta=NULL,alpha=NULL)
##' margbin <- glm(ybin~x,data=data,family=binomial())
##' margbin
##'
##' head(data)
##' data$number <- c(1,2,3,4)
##' data$child <- 1*(data$number==3)
##'
##' ### make ace random effects design
##' out <- ace_family_design(data,member="type",id="cluster")
##' out$pardes
##' head(out$des.rv)
##'
##' bints <- binomial_twostage(margbin,data=data,
##'      clusters=data$cluster,detail=0,var.par=1,
##'      theta=c(2,1),var.link=0,
##'      random.design=out$des.rv,theta.des=out$pardes)
##' summary(bints)
##'
##' data <- sim_binClaytonOakes_twin_ace(1000,2,1,beta=NULL,alpha=NULL)
##' out  <- twin_polygen_design(data,id="cluster",zygname="zygosity")
##' out$pardes
##' head(out$des.rv)
##' margbin <- glm(ybin~x,data=data,family=binomial())
##'
##' bintwin <- binomial_twostage(margbin,data=data,
##'      clusters=data$cluster,var.par=1,
##'      theta=c(2,1),random.design=out$des.rv,theta.des=out$pardes)
##' summary(bintwin)
##' concordanceTwinACE(bintwin)
##' }
##'
##' @keywords binomial regression
##' @author Thomas Scheike
##' @param margbin Marginal binomial model
##' @param data data frame
##' @param method Scoring method "nr", for lava NR optimizer
##' @param detail Detail
##' @param clusters Cluster variable
##' @param silent Debug information
##' @param weights Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.
##' @param theta Starting values for variance components
##' @param theta.des design for dependence parameters, when pairs are given the indeces of the theta-design for this pair, is given in pairs as column 5
##' @param var.link Link function for variance
##' @param var.par parametrization
##' @param var.func when alternative parametrizations are used this function can specify how the paramters are related to the \eqn{\lambda_j}'s.
##' @param iid Calculate i.i.d. decomposition when iid>=1, when iid=2 then avoids adding the uncertainty for marginal paramters for additive gamma model (default).
##' @param notaylor Taylor expansion
##' @param model model
##' @param marginal.p vector of marginal probabilities
##' @param beta.iid iid decomposition of marginal probability  estimates for each subject, if based on GLM model this is computed.
##' @param Dbeta.iid derivatives of marginal model wrt marginal parameters, if based on GLM model this is computed.
##' @param strata strata for fitting: considers only pairs where both are from same strata
##' @param max.clust max clusters
##' @param se.clusters clusters for iid decomposition for roubst standard errors
##' @param numDeriv uses Fisher scoring aprox of second derivative if 0, otherwise numerical derivatives
##' @param random.design random effect design for additive gamma model, when pairs are given the
##' indeces of the pairs random.design rows are given as columns 3:4
##' @param pairs matrix with rows of indeces (two-columns) for the pairs considered in the pairwise composite score, useful for case-control sampling when marginal is known.
##' @param dim.theta dimension of theta when pairs and pairs specific design is given. That is when pairs has 6 columns.
##' @param additive.gamma.sum this is specification of the lamtot in the models via a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number of theta parameters), when null then sums all parameters. Default is a matrix of 1's
##' @param pair.ascertained if pairs are sampled only when there are events in the pair i.e. Y1+Y2>=1.
##' @param case.control if data is case control data for pair call, and here 2nd column of pairs are probands (cases or controls)
##' @param twostage default twostage=1, to fit MLE use twostage=0
##' @param beta is starting value for beta for MLE version
##' @param no.opt for not optimizing 
##' @param ... for NR of lava 
##' @aliases binomial_twostage binomial_twostage_time
##' @export
binomial_twostage <- function(margbin,data=parent.frame(),
     method="nr",detail=0,clusters=NULL,silent=1,weights=NULL,
     theta=NULL,theta.des=NULL,var.link=0,var.par=1,var.func=NULL,
     iid=1,notaylor=1,model="plackett",marginal.p=NULL,beta.iid=NULL,Dbeta.iid=NULL,
     strata=NULL,max.clust=NULL,se.clusters=NULL,numDeriv=0,
     random.design=NULL,pairs=NULL,dim.theta=NULL,additive.gamma.sum=NULL,
     pair.ascertained=0,case.control=0,no.opt=FALSE,twostage=1,beta=NULL,...)
{ ## {{{
    ## {{{ seting up design and variables
    rate.sim <- 1; sym=1;
    if (model=="clayton.oakes"  || model=="gamma") dep.model <- 1 else if (model=="plackett" || model=="or") dep.model <- 2 else stop("Model must by either clayton.oakes or plackett \n");
    antpers <- NROW(data);
    if (!is.null(pairs)) nn <- NROW(pairs) else nn <- 1

### marginal prediction and binomial response, two types of calls ## {{{
    if (inherits(margbin,"glm")) {
        ps <- predict(margbin,newdata=data,type="response")
        if (margbin$family$family!="binomial") warning("not binomial family\n");
        ### takes data to extract response and predictions, these could be different for pairs call
        cause <- data[,all.vars(margbin$formula)[1]]

	if (!is.numeric(cause)) stop(paste("response in data",margbin$formula)[1],"not numeric\n");
	if (is.null(beta.iid))   beta.iid <- lava::iid(margbin,id=clusters)
	if (is.null(Dbeta.iid)) Dbeta.iid <- model.matrix(margbin$formula,data=data) * ps
	if (twostage==0)            Xbeta <- model.matrix(margbin$formula,data=data)
    }
    else if (inherits(margbin,"formula")) {
        margbin <- glm(margbin,data=data,family=binomial())
        ps <- predict(margbin,type="response")
        cause <- margbin$y
	if (twostage==0)            Xbeta <- model.matrix(margbin$formula,data=data)
	if (is.null(Dbeta.iid)) Dbeta.iid <- model.matrix(margbin$formula,data=data) * ps
	if (is.null(beta.iid))   beta.iid <- iid(margbin,id=clusters)
    }  else if (is.null(marginal.p))
        stop("without marginal model, marginal p's must be given\n");

    if (!is.null(marginal.p)) {
        if (length(margbin)!=antpers)
            stop("with marginal margbin is response \n")
        else cause <- margbin
        if (length(marginal.p)!=antpers)
            stop("length same as data dimension  \n")
        else ps <- marginal.p
    }
    ## }}}

    notaylor <- 1
    if (is.null(weights)==TRUE) weights <- rep(1,antpers);
    if (is.null(strata)==TRUE) strata<- rep(1,antpers);
    if (length(strata)!=antpers) stop("Strata must have length equal to number of data points \n");

# {{{ cluster setup
    out.clust <- cluster_index(clusters);
    clusters <- out.clust$clusters
    maxclust <- out.clust$maxclust
    antclust <- out.clust$cluster.size
    clusterindex <- out.clust$idclust
    clustsize <- out.clust$cluster.size
    call.secluster <- se.clusters

    if (is.null(se.clusters)) { se.clusters <- clusters; antiid <- nrow(clusterindex);} else  {
	iids <-  unique(se.clusters);
	antiid <- length(iids);
	if (is.numeric(se.clusters)) se.clusters <-  fast.approx(iids,se.clusters)-1
	 else se.clusters <- as.integer(factor(se.clusters, labels = seq(antiid)))-1
        }
    if (length(se.clusters)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

    if ((!is.null(max.clust))) if (max.clust< antiid) {
        coarse.clust <- TRUE
	qq <- unique(quantile(se.clusters, probs = seq(0, 1, by = 1/max.clust)))
	qqc <- cut(se.clusters, breaks = qq, include.lowest = TRUE)
	se.clusters <- as.integer(qqc)-1
	max.clusters <- length(unique(se.clusters))
	maxclust <- max.clust
	antiid <- max.clusters
    }
# }}}

    if (is.null(beta)==TRUE & twostage==0) beta <- coef(margbin) else beta <- 0
    ## }}}
  dimbeta <- length(beta);

  ### setting design for random variables, in particular with pairs are given
  ddd <- randomDes(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,var.link,clusterindex,dim.theta)

  random.design=ddd$random.design;clusterindex=ddd$clusterindex;
  antpairs=ddd$antpairs;
  theta=ddd$theta;ptheta=ddd$ptheta;theta.des=ddd$theta.des
  pair.structure=ddd$pair.structure; dep.model=ddd$dep.model
  dim.rv <- ddd$dim.rv; additive.gamma.sum=ddd$additive.gamma.sum

  if (dep.model==3) model <- "clayton.oakes"

if (pair.structure==1) {
    if (length(case.control)!=antpairs)         case.control <- rep(case.control[1],antpairs)
    if (length(pair.ascertained)!=antpairs) pair.ascertained <- rep(pair.ascertained[1],antpairs)
    if (any(case.control+pair.ascertained==2))  stop("Each pair is either case.control pair or pair.ascertained \n");
    }

    if (is.null(Dbeta.iid)) Dbeta.iid <- matrix(0,length(cause),1);
    ptrunc <- rep(1,antpers);

    theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);

    obj <- function(par)
    { ## {{{

      if (pair.structure==0 | dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);
      if (pair.structure==1 & dep.model==3) Xtheta <- matrix(0,antpers,1); ## not needed
      if (pair.structure==1 & dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);

      if (twostage==0) epar <- par[seq(1,ptheta)] else epar <- par
      if (twostage==0) { ### update, marginal.p og score for logistic model
	    beta <- par[seq(ptheta+1,ptheta+dimbeta)]
	    lp <- c(Xbeta %*%  beta)
            psu <- exp(lp)/(1+exp(lp))
            dpsu <- psu/(1+exp(lp))
	    Dbeta.iid <- Xbeta * dpsu
	    ps <- psu
      }

      if (var.link==1 & dep.model==3) epar <- c(exp(epar)) else epar <- c(epar)
      partheta <- epar

      if (var.par==1 & dep.model==3) {
       ## from variances to
       if (is.null(var.func)) {
	    sp <- sum(epar)
	    partheta <- epar/sp^2
         } else partheta <- epar; ## par.func(epar)
      }

      if (pair.structure==0)
            outl<-.Call("twostageloglikebin", ## {{{
            icause=cause,ipmargsurv=ps, itheta=c(partheta), ithetades=theta.des, icluster=clusters,
	    iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,iiid=iid,iweights=weights,
	    isilent=silent,idepmodel=dep.model, itrunkp=ptrunc,istrata=strata,iseclusters=se.clusters,
	    iantiid=antiid, irvdes=random.design,iags=additive.gamma.sum,ibetaiid=Dbeta.iid,pa=pair.ascertained,
	    twostage=twostage, PACKAGE="mets")
            ## }}}
      else outl<-.Call("twostageloglikebinpairs", ## {{{
            icause=cause,ipmargsurv=ps, itheta=c(partheta), ithetades=theta.des,
            icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,
	    iiid=iid,iweights=weights,isilent=silent,idepmodel=dep.model,
            itrunkp=ptrunc,istrata=strata,iseclusters=se.clusters,iantiid=antiid,
            irvdes=random.design,
            iags=additive.gamma.sum,ibetaiid=Dbeta.iid,pa=pair.ascertained,twostage=twostage,
	    icasecontrol=case.control,
            PACKAGE="mets")
             ## }}}

      if (detail==3) print(c(par,outl$loglike))

         ## variance parametrization, and inverse.link
	 if (dep.model==3) {# {{{
	    if (var.par==1) {
		 ## from variances to and with sum for all random effects
		 if (is.null(var.func)) {
		 if (var.link==0)  {
	###		 print(c(sp,epar))
		     mm <- matrix(-epar*2*sp,length(epar),length(epar))
		     diag(mm) <- sp^2-epar*2*sp
		 } else {
		    mm <- -c(epar) %o% c(epar)*2*sp
		    diag(mm) <- epar*sp^2-epar^2*2*sp
		 }
		    mm <- mm/sp^4
		 } else mm  <- numDeriv::hessian(var.func,par)
	      } else {
	          if (var.link==0) mm <- diag(length(epar)) else
			  mm <- diag(length(c(epar)))*c(epar)
	      }
              if (twostage==0) {  ### beta for logistic regression also part of model
		      mm0 <- diag(length(par))
		      mm0[1:nrow(mm),1:nrow(mm)] <- mm
		      mm <- mm0
	      }

	      }# }}}


	    if (dep.model==3) {# {{{
	       outl$score <-  t(mm) %*% outl$score
	       if (twostage==1) outl$DbetaDtheta <-  t(mm) %*% outl$DbetaDtheta
	       outl$Dscore <- t(mm) %*% outl$Dscore %*% mm
               if (iid==1) outl$theta.iid <- t(t(mm) %*% t(outl$theta.iid))
	    }# }}}

            attr(outl,"grad") <- attr(outl,"gradient") <-outl$score
	    attr(outl,"hessian") <- outl$Dscore
            outl$gradient <- outl$score 
            outl$hessian <- -outl$Dscore

###            if (oout==0) ret <- c(-1*outl$loglike) else if (oout==1) ret <- sum(outl$score^2) else if (oout==3) ret <- outl$score else ret <- outl

    if (oout==0) ret <- with(outl,structure(-outl$loglike,gradient=-gradient,hessian=-hessian))
    else if (oout==1) ret <- outl$gradient 
    else if (oout==2) ret <- outl

    return(ret)
    } ## }}}

  theta.iid <- NULL
  logl <- NULL
  p <- theta
  if (twostage==0) p <- c(p,beta);
  theta <- p
    
  oout <- 0
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(p,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  ## names(cc) <- colnames(X)
      oout <- 2
      val <- c(list(coef=cc),obj(opt$estimate))
 } else val <- c(list(coef=beta),obj(p))
 theta <- matrix(val$coef,ncol=1)

    if (numDeriv>=1) {
         oout <- 1
         if (detail==1 ) cat("starting numDeriv for second derivative \n");
         val$hessian <- numDeriv::jacobian(obj,p,method="simple")
         if (detail==1 ) cat("finished numDeriv for second derivative \n");
        }

  hess <- val$hessian
  if (!is.na(sum(hess))) hessi <- pinv(val$hessian) else hessi <- diag(nrow(val$hessian))


    ## {{{ handling output
    iid.tot <- NULL
    var.tot <- robvar.theta <- NULL
    beta <- NULL
    if (iid>=1) {
        theta.iid <- val$theta.iid %*% hessi
        if (dep.model==3 & iid!=2  & (!is.null(beta.iid)))
	if (nrow(beta.iid)==nrow(val$theta.iid) & twostage==1) {
	     theta.beta.iid <- (beta.iid %*% t(val$DbetaDtheta) ) %*% hessi
	     theta.iid  <- theta.iid+theta.beta.iid
	     iid.tot <- cbind(theta.iid,beta.iid)
	     var.tot <- crossprod(iid.tot)
	}
        var.theta <- robvar.theta  <- (t(theta.iid) %*% theta.iid)
	if (is.null(var.tot)) var.tot <- var.theta
    } else var.theta <- -1* hessi

  if (inherits(margbin,"glm")) beta <- coef(margbin);
  if (twostage==0) beta <- theta[seq(ptheta,ptheta+dimbeta)]

  if (iid==1) var.theta <- robvar.theta else var.theta <- -hessi
  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(theta),sep="")
   if (length(thetanames)==nrow(theta)) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }

    ud <- list(coef=val$coef,theta=theta,score=val$gradient,hess=hess,hessi=hessi,
	       var.theta=var.theta,model=model,robvar.theta=robvar.theta,
    theta.iid=theta.iid,thetanames=thetanames,loglike=val$loglike,Dscore=val$Dscore,
    margsurv=ps,iid.tot=iid.tot,var.tot=var.tot,beta=beta);
    class(ud)<-"mets.twostage"
    attr(ud, "binomial") <- TRUE
    attr(ud, "ptheta") <- ptheta
    attr(ud, "Formula") <- formula
    attr(ud, "Clusters") <- clusters
    attr(ud,"sym")<-sym;
    attr(ud,"var.link")<-var.link;
    attr(ud,"var.par")<-var.par;
    attr(ud,"var.func")<-var.func;
    attr(ud,"antpers")<-antpers;
    attr(ud,"antclust")<-antclust;
    attr(ud, "Type") <- model
    attr(ud,"DbetaDtheta")<-val$DbetaDtheta;
    attr(ud,"ags")<- additive.gamma.sum
    attr(ud,"twostage")<- twostage
    attr(ud,"pair.ascertained")<- pair.ascertained
    ### to be consistent with structure for survival twostage model
    attr(ud, "additive-gamma") <- (dep.model==3)*1
    attr(ud, "likepairs") <- c(val$likepairs)
    if (dep.model==3 & pair.structure==0) {
       attr(ud, "pardes") <- theta.des
       attr(ud, "theta.des") <- theta.des
       attr(ud, "rv1") <-    random.design[1,]
    }
    if (dep.model==3 & pair.structure==1) {
       nrv <- clusterindex[1,6]
       attr(ud, "theta.des") <- matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "pardes") <-    matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "rv1") <-    random.design[1,]
       attr(ud, "nrv") <- clusterindex[1,6]
    }

    attr(ud, "response") <- "binomial"
    return(ud);
    ## }}}
} ## }}}


##' @rdname twin-design
##' @param theta dependence parameter vector.
##' @param rv1 random-effects design for subject 1.
##' @param rv2 random-effects design for subject 2.
##' @param p1 marginal probability for subject 1.
##' @param p2 marginal probability for subject 2.
##' @param pardes parameter design matrix.
##' @param ags additive gamma sum matrix (optional).
##' @param link link function indicator (0 = identity, 1 = log).
##' @param i index for subject 1.
##' @param j index for subject 2.
##' @export
p11_binomial_twostage_RV <- function(theta,rv1,rv2,p1,p2,pardes,ags=NULL,link=0,i=1,j=1) { ## {{{
	## computes p11 pij for additive gamma binary random effects model
     if (is.null(ags)) ags <- matrix(1,dim(pardes))
     out <- .Call("claytonoakesbinRV",theta,i,j,p1,p2,rv1,rv2,pardes,ags,link,PACKAGE="mets")$like
 return(out)
} ## }}}

##' Concordance Probability from Twostage Model
##'
##' Computes concordance probability (joint probability of both subjects
##' experiencing the event) given dependence parameters and random-effect
##' variance structures from a twostage model.
##'
##' @param p matrix of marginal probabilities (n x 2).
##' @param theta.des parameter design matrix linking theta to lambda parameters.
##' @param additive.gamma.sum optional matrix for additive gamma sums.
##' @param var.par if 1, parameters are rescaled by sum squared.
##' @return A list of concordance tables, one per pair, each containing
##'   \code{pmat} (2x2 probability matrix), \code{casewise} (casewise concordance),
##'   and \code{marg} (marginal probabilities).
##' @rdname twin-design
##' @export
concordanceTwostage<- function(theta,p,rv1,rv2,theta.des,additive.gamma.sum=NULL,link=0,var.par=0,...)
{# {{{
   ### takes dependence paramter from output
      ptheta <- length(theta)

   if (var.par==1) theta <- theta/sum(theta)^2

   nn <- nrow(as.matrix(rv1))
   if (is.matrix(p)==FALSE) { ll <- length(p); p <- matrix(p,ll,2); }
   if (ncol(p)!=2) p <- matrix(p,ncol=2)
   if (is.matrix(rv1)==FALSE) rv1 <- matrix(rv1,nn,length(rv1),byrow=TRUE)
   if (is.matrix(rv2)==FALSE) rv2 <- matrix(rv2,nn,length(rv1),byrow=TRUE)

   if (is.null(additive.gamma.sum)) ags <- matrix(1,ncol(rv1),ptheta) else ags <- additive.gamma.sum


   tabs <- list()
   for (i in 1:nn)
   {
        p1 <- p[i,1]
        p2 <- p[i,2]
	p11 <- p11_binomial_twostage_RV(theta,rv1[i,],rv2[i,],p1,p2,theta.des,ags=ags,link=link)
	p01 <- p[i,1]-p11
	p10 <- p[i,2]-p11
	p00 <- 1-p01-p10+p11
	tabs[[i]] <- list(pmat=matrix(c(p00,p10,p01,p11),2,2),casewise=c(p11/p1,p11/p2),marg=c(p1,p2))
   }

   return(tabs)
}	# }}}

##' Concordance for Twin ACE Models
##'
##' Computes concordance probabilities for twin ACE/ADE models from a
##' binomial twostage object.
##'
##' @param object a binomial twostage model object.
##' @param xmarg optional covariate values for marginal probabilities.
##' @return A list of concordance tables per zygosity group.
##' @rdname twin-design
##' @export
concordanceTwinACE<- function(object,rv1=NULL,rv2=NULL,xmarg=NULL,type="ace",...)
{# {{{

if (type=="ace" | type=="ade") {
if (is.null(rv1))  rv1 <- rbind(c(1,0,0,0,1),c(0,1,1,0,1))
if (is.null(rv2))  rv2 <- rbind(c(1,0,0,0,1),c(0,1,0,1,1))
}
if (type=="ae" | type=="de") {
if (is.null(rv1))  rv1 <- rbind(c(1,0,0,0),c(0,1,1,0))
if (is.null(rv2))  rv2 <- rbind(c(1,0,0,0),c(0,1,0,1))
}

var.par <- attr(object,"var.par")
var.link <- attr(object,"var.link")
theta.des <- attr(object,"theta.des")
twostage <- attr(object,"twostage")

if (twostage==0) {
	pt <- attr(object,"ptheta")
	theta <- object$theta[seq(1,pt)]
        beta <- object$theta[seq(pt+1,length(object$theta))]
	var.theta <- object$robvar.theta[seq(1,pt),seq(1,pt)]
	var.tot <- object$var.theta
} else {
	theta <- object$theta;
	beta <- object$beta
	var.theta <- object$var.theta;
	var.tot <- object$var.tot
}

if (var.link==1) theta <- exp(theta)
###print(beta)
ags <- attr(object,"ags");
if (is.null(xmarg)) {xmarg <- rep(0,length(beta)); xmarg[1] <- 1;}

nn <- nrow(rv1)
if (is.matrix(rv1)==FALSE) rv1 <- matrix(rv1,nn,length(rv1),byrow=TRUE)
if (is.matrix(rv2)==FALSE) rv2 <- matrix(rv2,nn,length(rv1),byrow=TRUE)
if (is.null(ags)) ags <- matrix(1,ncol(rv1),length(theta));

   fp <- function(par) {# {{{
       pp <- par[1:length(theta)];
       beta <- par[(length(theta)+1):length(par)];
       xp  <- sum(xmarg*beta);
       pm <- exp(xp)/(1+exp(xp));
       if (var.par==1) pp <- pp/sum(pp)^2
       p11 <- p11_binomial_twostage_RV(pp,rv1l,rv2l,pm,pm,theta.des,ags=ags,link=0)
       casewise <- p11/pm
       ret <- c(p11,casewise,pm)
       names(ret) <- c("concordance","casewise concordance","marginal")
       return(ret)
   }# }}}


   nnn<-c("MZ","DZ")
   tabs <- list()
   for (i in 1:nn)
   {
      rv1l <- rv1[i,]
      rv2l <- rv2[i,]
      tabs<-c(tabs,setNames(list(tabs[[i]] <- lava::estimate(coef=c(theta,beta),vcov=var.tot,f=function(p) fp(p))), nnn[i]))
   }

   return(tabs)
}	# }}}

##' @export
binomial_twostage_time <- function(formula,data,id,...,silent=1,fix.censweights=1,
breaks=Inf,pairsonly=TRUE,fix.marg=NULL,cens.formula,cens.model="aalen",weights="w") {
   ## {{{
    m <- match.call(expand.dots = FALSE)
    m <- m[match(c("","data"),names(m),nomatch = 0)]
    Terms <- terms(cens.formula,data=data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    M <- eval(m,envir=parent.frame())

    censtime <- model.extract(M, "response")
    status <- censtime[,2]
    time <- censtime[,1]
    timevar <- colnames(censtime)[1]
    outcome <- as.character(terms(formula)[[2]])
    if (is.null(breaks)) breaks <-  quantile(time,c(0.25,0.5,0.75,1))

    outcome0 <- paste(outcome,"_dummy")
    res <- list()
    logor <- cif <- conc <- c()
    k <- 0
    for (tau in rev(breaks)) {
        if ((length(breaks)>1) & (silent==0)) message(tau)
        ### construct min(T_i,tau) or T_i and related censoring variable,
        ### thus G_c(min(T_i,tau)) or G_c(T_i) as weights
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) {
        data0 <- data
        time0 <- time
        status0 <- status
        }
        cond0 <- (time>tau)
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) status0[cond0 & status==1] <- 3
        if (fix.censweights==0) status0[cond0 & status==1] <- 3
        data0[,outcome] <- data[,outcome]
        data0[cond0,outcome] <- FALSE
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) time0[cond0] <- tau
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) {
		data0$S <- survival::Surv(time0,status0==1)
	}
	if ((fix.censweights==1 & k==0) | (fix.censweights==0))
        dataw <- ipw(update(cens.formula,S~.),data=data0,cens.model=cens.model,
		     obsonly=TRUE)
        if ((fix.censweights==1))
		dataw[,outcome] <- (dataw[,outcome])*(dataw[,timevar]<tau)
	marg.bin <- glm(formula,data=dataw,weights=1/dataw[,weights],family="quasibinomial")
        pudz <- predict(marg.bin,newdata=dataw,type="response")
	dataw$pudz <- pudz
	datawdob <- fast.reshape(dataw,id=id)
        datawdob$minw <- pmin(datawdob$w1,datawdob$w2)
        dataw2 <- fast.reshape(datawdob)
	### removes second row of singletons
	dataw2  <- subset(dataw2,!is.na(dataw2$minw))
	k <- k+1
	if (!is.null(fix.marg)) dataw2$pudz <- fix.marg[k]
        suppressWarnings( b <- binomial_twostage(dataw2[,outcome],data=dataw2,clusters=dataw2[,id],marginal.p=dataw2$pudz,weights=1/dataw2$minw,...))
        theta0 <- b$theta[1,1]
        prev <- prev0 <- exp(coef(marg.bin)[1])/(1+exp(coef(marg.bin)[1]))
	if (!is.null(fix.marg)) prev <- fix.marg[k]
        concordance <- plack.cif2(prev,prev,theta0)
	conc <- c(conc,concordance)
	cif <- c(cif,prev0)
	logor <- rbind(logor,coef(b))
    }
    res <- list(varname="Time",var=breaks,concordance=rev(conc),cif=rev(cif),
		time=breaks,call=m,type="time",logor=logor[k:1,])
    return(res)
} ## }}}

##' @export
sim_BinPlack <- function(n,beta=0.3,theta=1,...) { ## {{{
    x1 <- rbinom(n,1,0.5)
    x2 <- rbinom(n,1,0.5)
###
    p1 <- exp(0.5+x1*beta)
    p2 <- exp(0.5+x2*beta)
    p1 <- p1/(1+p1)
    p2 <- p2/(1+p2)
###
    p11 <- plack.cif2(p1,p2,theta)
    p10 <- p1-p11
    p01 <- p2-p11
    p00 <- 1- p10-p01-p11
###
    y1 <- rbinom(n,1,p1)
    y2 <- (y1==1)*rbinom(n,1,p11/p1)+(y1==0)*rbinom(n,1,p01/(1-p1))
    list(x1=x1,x2=x2,y1=y1,y2=y2,id=1:n)
} ## }}}

##' @export
sim_BinFam <- function(n,beta=0.0,rhopp=0.1,rhomb=0.7,rhofb=0.1,rhobb=0.7) { ## {{{
    xc <- runif(n)*0.5
    xm <- rbinom(n,1,0.5+xc);
    xf <- rbinom(n,1,0.5+xc);
    xb1 <- rbinom(n,1,0.3+xc);
    xb2 <- rbinom(n,1,0.3+xc);
###
    rn <- matrix(rnorm(n*4),n,4)
    corm <- matrix( c(1,rhopp,rhomb,rhomb, rhopp,1,rhofb,rhofb, rhomb,rhofb,1,rhobb, rhomb,rhofb,rhobb,1),4,4)
    rnn <- t( corm %*% t(rn))
    zm <- exp(rnn[,1]); zf <- exp(rnn[,2]); zb1 <- exp(rnn[,3]); zb2 <- exp(rnn[,4]);
    pm <- exp(0.5+xm*beta+zm)
    pf <- exp(0.5+xf*beta+zf)
    pf <- pf/(1+pf)
    pm <- pm/(1+pm)
    pb1 <- exp(0.5+xb1*beta+zb1)
    pb1 <- pb1/(1+pb1)
    pb2 <- exp(0.5+xb2*beta+zb2)
    pb2 <- pb2/(1+pb2)
    ym <- rbinom(n,1,pm)
    yf <- rbinom(n,1,pf)
    yb1 <- rbinom(n,1,pb1)
    yb2 <- rbinom(n,1,pb2)
                                        #
    agem <- 20+runif(n)*10
    ageb1 <- 5+runif(n)*10
    data.frame(agem=agem,agef=agem+3+rnorm(n)*2,
               ageb1=ageb1,ageb2=ageb1+1+runif(n)*3,xm=xm,xf=xf,xb1=xb1,xb2=xb2,ym=ym,yf=yf,yb1=yb1,yb2=yb2,id=1:n)
} ## }}}

##' @export
sim_BinFam2 <- function(n,beta=0.0,alpha=0.5,lam1=1,lam2=1,...) { ## {{{
    x1 <- rbinom(n,1,0.5); x2 <- rbinom(n,1,0.5);
    x3 <- rbinom(n,1,0.5); x4 <- rbinom(n,1,0.5);
### random effects speicification of model via gamma distributions
    zf <- rgamma(n,shape=lam1); zb <- rgamma(n,shape=lam2);
    if (length(alpha)!=4) alpha <- rep(alpha[1],4)
    if (length(beta)!=4) beta <- rep(beta[1],4)
    pm <- exp(alpha[1]+x1*beta[1]+zf)
    pf <- exp(alpha[2]+x2*beta[2]+zf)
    pf <- pf/(1+pf)
    pm <- pm/(1+pm)
    pb1 <- exp(alpha[3]+x1*beta[3]+zf+zb)
    pb1 <- pb1/(1+pb1)
    ym <- rbinom(n,1,pm)
    yf <- rbinom(n,1,pf)
    yb1 <- rbinom(n,1,pb1)
    yb2 <- rbinom(n,1,pb1)
                                        #
    data.frame(x1=x1,x2=x2,ym=ym,yf=yf,yb1=yb1,yb2=yb2,id=1:n)
} ## }}}

##' @export
sim_binClaytonOakes_family_ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
###  K <- 10000
  n <- 4 # family members with ace structure
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2;
  varc <- varc/sumpar^2
  ## total variance 1/(varg+varc)
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc+varg
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  mother.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  father.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  env <- rgamma(K,varc)/eta
  mother <- apply(mother.g,1,sum)+env
  father <- apply(father.g,1,sum)+env
  child1 <- apply(cbind(mother.g[,c(1,2)],father.g[,c(1,2)]),1,sum) + env
  child2 <- apply(cbind(mother.g[,c(1,3)],father.g[,c(1,3)]),1,sum) + env
  Gam1 <- cbind(mother,father,child1,child2)
  ## }}}

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  xb1 <- rbinom(K,1,0.5)
  xb2 <- rbinom(K,1,0.5)
  xm <- rbinom(K,1,0.5)
  xf <- rbinom(K,1,0.5)
###
  if (is.null(beta)) beta <- rep(0.3,4)
  if (is.null(alpha)) alpha <- rep(0.5,4)
  pm <- exp(alpha[1]+xm*beta[1]); pf <- exp(alpha[2]+xf*beta[2])
  pb1 <- exp(alpha[3]+xb1*beta[3]); pb2 <- exp(alpha[4]+xb2*beta[4])
  p <- cbind(pm,pf,pb1,pb2)
  p <- p/(1+p)

  vartot <- eta
  pgivenZ <- ilap(vartot,p)
###  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)
  xs <- cbind(xm,xf,xb1,xb2)
  type <- rep(c("mother","father","child","child"),K)

  ud <- data.frame(ybin=c(Ybin),x=c(t(xs)),type=type,cluster=rep(1:K,each=n))

return(ud)
} ## }}}

##' @export
sim_binClaytonOakes_twin_ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
  n <- 2 # twins with ace structure
  ## total variance 1/(varg+varc)
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2;
  varc <- varc/sumpar^2
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc+varg
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  dz.g <-  cbind(rgamma(K/2,varg*0.5)/eta, rgamma(K/2,varg*0.5)/eta, rgamma(K/2,varg*0.5)/eta)
  mz.g <-  rgamma(K/2,varg)/eta
  env1 <- rgamma(K/2,varc)/eta
  env2 <- rgamma(K/2,varc)/eta
  mz   <- mz.g+env1
  dz1 <- apply(dz.g[,c(1,2)],1,sum) + env2
  dz2 <- apply(dz.g[,c(1,3)],1,sum) + env2
  Gam1 <- rbind(cbind(mz,mz),cbind(dz1,dz2))
  ## }}}

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  xb1 <- rbinom(K,1,0.5)
  xb2 <- rbinom(K,1,0.5)
###
  if (is.null(beta)) beta <- rep(0.3,2)
  if (is.null(alpha)) alpha <- rep(0.5,2)
  pb1 <- exp(alpha[1]+xb1*beta[1]);
  pb2 <- exp(alpha[2]+xb2*beta[2])
  p <- cbind(pb1,pb2)
  p <- p/(1+p)

  vartot <- eta
  pgivenZ <- ilap(vartot,p)

###  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)
  xs <- cbind(xb1,xb2)
  type <- rep(c("twin1","twin2"),K)
  zygosity <- rep(c("MZ","DZ"),each=K)

  ud <- data.frame(ybin=c(Ybin),x=c(t(xs)),type=type,cluster=rep(1:K,each=n),zygosity=zygosity)

###names(ud)<-c("ybin","x","cluster","type")
return(ud)
} ## }}}

##' @export
sim_binClaytonOakes_pairs <- function(K,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
###  K <- 10000
  n <- 2 # twins with ace structure
  ## total variance 1/(varg+varc)
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc <- 1/varc
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  rans <- rgamma(K,varc)/eta
  Gam1 <- cbind(rans,rans)
  ## }}}
###  apply(Gam1,2,mean); apply(Gam1,2,var); cor(Gam1)

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  x1 <- rbinom(K,1,0.5)
  x2 <- rbinom(K,1,0.5)
###
###
  if (is.null(beta)) beta <- rep(0.3,2)
  if (is.null(alpha)) alpha <- rep(0.5,2)
  p1 <- exp(alpha[1]+x1*beta[1]);
  p2 <- exp(alpha[1]+x2*beta[2])
  p <- cbind(p1,p2)
  p <- p/(1+p)

  vartot <- eta
###  pgivenZ <- mets:::ilap(vartot,p)
  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)

  xs <- cbind(x1,x2)
  ud <- cbind(c(Ybin),c(t(xs)),rep(1:K,each=n))

  ud <- cbind(ud)
  ud <- data.frame(ud)

names(ud)<-c("ybin","x","cluster")
return(ud)
} ## }}}

cumh <- function(formula,data,...,time,
                 timestrata=quantile(data[,time],c(0.25,0.5,0.75,1)),
                 cens.formula=NULL,cens.model="aalen",
                 cumulative=FALSE, 
                 silent=FALSE) {

  time.  <- substitute(time)
  if (!is.character(time.)) time. <- deparse(time.)
  time <- time.
  if (!is.null(cens.formula)) {    
    m <- match.call(expand.dots = TRUE)[1:3]
    Terms <- terms(cens.formula, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    censMod <- eval(m, parent.frame())
    censtime <- model.extract(m, "response")
    status <- censtime[,2]
###  data[,"_status"] <- 
  }

  res <- list(); i <- 0
  ht <- c()
  outcome <- as.character(terms(formula)[[2]])
  coefs <- c()
  y0 <- data[,outcome]
  for (i in seq(length(timestrata))) {
    t <- timestrata[i]
    data[,outcome] <- y0
    newdata <- data
    if (!cumulative) {
      if (i==1) {
        idx <- data[,time]<t
      } else {
        idx <- (timestrata[i-1]<=data[,time] & data[,time]<t)
      }
    } else {
      newdata <- data
      newdata[,]
      newdata[,outcome] <- data[,outcome]*(data[,time]<t)
      if (!is.null(cens.formula)) {
        
      }      
    }
    if (!silent) {
      message(t)
    }
    if (!cumulative)
      res[[i]] <- summary(twinlm(formula,data=data[idx,],...))
    else {
      
      res[[i]] <- summary(bptwin(formula,data=data,...))
    }

    coefs <- c(coefs, list(res[[i]]$all))
    ht <- rbind(ht,c(t,res[[i]]$heritability[1,]))
  }
  
  coeftype <- c()
  for (i in seq(nrow(coefs[[1]]))) {
    rr <- matrix(unlist(lapply(coefs,function(z) z[i,])),ncol=3,byrow=TRUE)
    colnames(rr) <- colnames(coefs[[1]])
    rr <- cbind(time=timestrata,rr)
    coeftype <- c(coeftype,list(rr)); names(coeftype)[length(coeftype)] <- rownames(coefs[[1]])[i]
  }
  
  rownames(ht) <- timestrata
  colnames(ht) <- c("time","Heritability","Std.Err","2.5%","97.5%")
  res <- (list(time=timestrata, ht=ht,models=res,coef=coefs,
               coeftype=coeftype, timevar=time))
  class(res) <- "cumh"
  res
}


##' @export
summary.cumh <- function(object,...) object 

##' @export
print.cumh <- function(x,type=seq(nrow(x$coef[[1]])),...) {
  for (i in type) {    
    cat(i, ": ", names(x$coeftype)[i], "\n",sep="")
    rr <- matrix(unlist(lapply(x$coef,function(z) z[i,])),ncol=3,byrow=TRUE)
    colnames(rr) <- colnames(x$coef[[1]])
    rr <- cbind(time=x$time,rr)
    print(rr)
  }
  invisible(x)
}


##' @export
plot.cumh <- function(x,...,type=1,lwd=2,col,fillcol,alpha=0.2,ylim=c(0,1),xlab=x$timevar,ylab="Heritability",idx=seq(nrow(x$ht)),legend=TRUE,legendpos="topleft") {

  add <- FALSE
  if (missing(col)) col <- seq(length(type))
  if (alpha>0 & missing(fillcol)) fillcol <- Col(col,alpha)
  count <- 0
  for (tt in type) {
    count <- count+1
    zz <- x$coeftype[[tt]][idx,,drop=FALSE]
    if (!add) {    
      plot(zz[,1:2,drop=FALSE],type="l",ylim=ylim,lwd=lwd,
           ylab=ylab,xlab=xlab,col=col[count],...)
    }
    add <- TRUE
    xx <- with(x, c(zz[,1],rev(zz[,1])))
    yy <- with(x, c(zz[,3],rev(zz[,4]))) 
    polygon(xx,yy,col=fillcol[count])
    lines(zz[,1:2,drop=FALSE],lwd=lwd,col=col[count],...)
  }
  if (legend) graphics::legend(legendpos,names(x$coeftype)[type],col=col,lwd=lwd,lty=1)
  invisible(x)    
}

## ----include=FALSE,echo=FALSE,message=FALSE,warning=FALSE---------------------
options(warn=-1, family="Times")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1
  ##dev="png",
  ##dpi=72,
  ## out.width = "70%")
)
library("mets")

## ----install, eval=FALSE, echo=FALSE------------------------------------------
#  # install.packages("remotes")
#  remotes::install_github("kkholst/mets", dependencies="Suggests")

## ----twinbmi------------------------------------------------------------------
library(mets)
data("twinbmi")
head(twinbmi)

## ----twinwide-----------------------------------------------------------------
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
head(twinwide)

## ----scatterdens, echo=FALSE,message=FALSE,warning=FALSE----------------------
library("cowplot")

scatterdens <- function(x) {
    require(ggplot2)
    sp <- ggplot(x,
                aes_string(colnames(x)[1], colnames(x)[2])) +
        theme_minimal() +
        geom_point(alpha=0.3) + geom_density_2d()
    xdens <- ggplot(x, aes_string(colnames(x)[1],fill=1)) +
        theme_minimal() +
        geom_density(alpha=.5)+
        theme(axis.text.x = element_blank(),
	      legend.position = "none") + labs(x=NULL)
    ydens <- ggplot(x, aes_string(colnames(x)[2],fill=1)) +
        theme_minimal() +
        geom_density(alpha=.5) +
        theme(axis.text.y = element_blank(),
	      axis.text.x = element_text(angle=90, vjust=0),
	      legend.position = "none") +
        labs(x=NULL) +
        coord_flip()
    g <- plot_grid(xdens,NULL,sp,ydens,
                  ncol=2,nrow=2,
                  rel_widths=c(4,1.4),rel_heights=c(1.4,4))
    return(g)
}

## ----scatter1, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in MZ twins"----
mz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
scatterdens(mz)

## ----scatter2, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in DZ twins"----
dz <- log(subset(twinwide, zyg=="DZ")[,c("bmi1","bmi2")])
scatterdens(dz)

## -----------------------------------------------------------------------------
cor.test(mz[,1],mz[,2], method="spearman")

## -----------------------------------------------------------------------------
cor.test(dz[,1],dz[,2], method="spearman")

## ----gee----------------------------------------------------------------------
l0 <- lm(bmi ~ gender + I(age-40), data=twinbmi)
estimate(l0, id=twinbmi$tvparnr)

## -----------------------------------------------------------------------------
library("splines")
l1 <- lm(bmi ~ gender*ns(age,3), data=twinbmi)
marg1 <- estimate(l1, id=twinbmi$tvparnr)

## ----marg1, warning=FALSE,message=FALSE,fig.cap="Marginal association between BMI and Age for males and females."----
dm <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("male"),
	    age=seq(33,61,length.out=50))
df <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("female"),
	    age=seq(33,61,length.out=50))

plot(marg1, function(p) model.matrix(l1,data=dm)%*%p,
     data=dm["age"], ylab="BMI", xlab="Age",
     ylim=c(22,26.5))
plot(marg1, function(p) model.matrix(l1,data=df)%*%p,
     data=df["age"], col="red", add=TRUE)
legend("bottomright", c("Male","Female"),
       col=c("black","red"), lty=1, bty="n")

## -----------------------------------------------------------------------------
dd <- na.omit(twinbmi)

## ----lmsat, eval=FALSE--------------------------------------------------------
#  l0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="sat")

## ----lmflex, eval=FALSE-------------------------------------------------------
#  lf <- twinlm(bmi ~ age+gender, data=dd,DZ="DZ", zyg="zyg", id="tvparnr", type="flex")

## ----lmeqmarg-----------------------------------------------------------------
lu <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="eqmarg")
estimate(lu)

## -----------------------------------------------------------------------------
estimate(lu,lava::contr(5:6,6))

## ----ace----------------------------------------------------------------------
ace0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="ace")
summary(ace0)

## ----include = FALSE, label=setup---------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
  )
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE") || length(list.files("data"))==0
saveobj <- function(obj, null_elements) {
  x <- get(obj, envir=parent.frame())
  if (length(null_elements)>0) {
    print(object.size(x))
    x[null_elements] <- NULL
    attributes(x)[null_elements] <- NULL
    print(object.size(x))
  }
  saveRDS(x, paste0("data/", obj, ".rds"), compress="xz")
}

library(mets)
 cols <- c("darkred","darkblue","black")
 ltys <- c(1,3,2)
 fig_w <- 5
 fig_h <- 5
 savefig <- TRUE

## ----results="hide", echo=FALSE, eval=!fullVignette---------------------------
## To save time building the vignettes on CRAN, we cache time consuming computations
fitco1 <- readRDS("data/fitco1.rds")
fitco2 <- readRDS("data/fitco2.rds")
fitco3 <- readRDS("data/fitco3.rds")
fitco4 <- readRDS("data/fitco4.rds")
fitace <- readRDS("data/fitace.rds")
fitde <- readRDS("data/fitde.rds")
cse <- readRDS("data/cse.rds")
slr <- readRDS("data/slr.rds")
outacem <- readRDS("data/outacem.rds")
b0 <- readRDS("data/b0.rds")
b1 <- readRDS("data/b1.rds")
b2 <- readRDS("data/b2.rds")
a1 <- readRDS("data/a1.rds")
h2 <- readRDS("data/h2.rds")
concMZ <- readRDS("data/concMZ.rds")
s_mz_country <- readRDS("data/s_mz_country.rds")
s_dz_country <- readRDS("data/s_dz_country.rds")

## ----label=data-prt-----------------------------------------------------------
library(mets)
 set.seed(122)
 data(prt)
 
 dtable(prt,~status+cancer)
 dtable(prt,~zyg+country,level=1)

## ----label=survival-marginal--------------------------------------------------
 # Marginal Cox model here stratified on country without covariates 
 margph <- phreg(Surv(time,cancer)~strata(country)+cluster(id),data=prt)
 plot(margph)

## ----label=survival-pairwise1, eval=fullVignette------------------------------
#  # Clayton-Oakes, MLE , overall variance
#  fitco1<-twostageMLE(margph,data=prt,theta=2.7)

## -----------------------------------------------------------------------------
 summary(fitco1)

## ----label=survival-pairwise2, eval=fullVignette------------------------------
#  fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)

## -----------------------------------------------------------------------------
 summary(fitco2)

## ----label=survival-pairwise3, eval=fullVignette------------------------------
#   mm <- model.matrix(~-1+factor(zyg),prt)
#   fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)

## -----------------------------------------------------------------------------
 summary(fitco3)

## ----label=survival-pairwise4, eval=fullVignette------------------------------
#  fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)

## -----------------------------------------------------------------------------
 summary(fitco4)
 round(estimate(coef=fitco4$coef,vcov=fitco4$var.theta)$coefmat[,c(1,3:4)],2)

 ## mz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[2],0,K=1000)$mz.kendall
 ## dz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[1],0,K=1000)$mz.kendall

## ----label=survival-polygenic1, eval=fullVignette-----------------------------
#   ### setting up design for random effects and parameters of random effects
#   desace <- twin.polygen.design(prt,type="ace")
#  
#   ### ace model
#   fitace <- survival.twostage(margph,data=prt,theta=1,
#         clusters=prt$id,var.link=0,model="clayton.oakes",
#         numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)

## -----------------------------------------------------------------------------
 summary(fitace)

## ----label=survival-polygenic2, eval=fullVignette-----------------------------
#   ### ace model with positive random effects variances
#   # fitacee <- survival.twostage(margph,data=prt,theta=1,
#   #      clusters=prt$id,var.link=1,model="clayton.oakes",
#   #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
#   #summary(fitacee)
#  
#   ### ae model
#   #desae <- twin.polygen.design(prt,type="ae")
#   #fitae <- survival.twostage(margph,data=prt,theta=1,
#   #      clusters=prt$id,var.link=0,model="clayton.oakes",
#   #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
#   #summary(fitae)
#  
#   ### de model
#   desde <- twin.polygen.design(prt,type="de")
#   fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
#  numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)
#  

## -----------------------------------------------------------------------------
summary(fitde)

## -----------------------------------------------------------------------------
prt <-  force.same.cens(prt,cause="status")

 dtable(prt,~status+cancer)
 dtable(prt,~status+country)
 dtable(prt,~zyg+country)

## ----label=concordance--------------------------------------------------------
 ## cumulative incidence with cluster standard errors.
 cif1 <- cif(Event(time,status)~strata(country)+cluster(id),prt,cause=2)
 plot(cif1,se=1)

 cifa <- cif(Event(time,status)~+1,prt,cause=2)

 ### concordance estimator, ignoring country differences. 
 p11 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2))
p11mz <- p11$model$"MZ"
p11dz <- p11$model$"DZ" 

## -----------------------------------------------------------------------------
 par(mfrow=c(1,2))
 ## Concordance
 plot(p11mz,ylim=c(0,0.1));
 plot(p11dz,ylim=c(0,0.1));

## ----label=concordance2-------------------------------------------------------
 library(prodlim)
 outm <- prodlim(Hist(time,status)~+1,data=prt)

 cifzyg <- cif(Event(time,status)~+strata(zyg)+cluster(id),data=prt,cause=2)
 cifprt <- cif(Event(time,status)~country+cluster(id),data=prt,cause=2)
     
 times <- 70:100
 cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ")) ## cause is 2 (second cause) 
 cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))
    
 ### concordance for MZ and DZ twins<
 cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
 ccdz <- cc$model$"DZ"
 ccmz <- cc$model$"MZ"
     
 cdz <- casewise(ccdz,outm,cause.marg=2) 
 cmz <- casewise(ccmz,outm,cause.marg=2)

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,col=cols[2:1], lty=ltys[2:1], legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-MZ","concordance-DZ"),col=cols[1:2],lty=ltys[1:2])

 plot(cmz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,col=cols[c(1,3,3)],lty=ltys[c(1,3,3)],
      ylab="Casewise",xlab="Age")
  plot(cdz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(cols[2],NA,NA), lty=ltys[c(2,3,3)], add=TRUE)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[1]))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[2]))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=cols, lty=ltys, bg="white")

 summary(cdz)
 summary(cmz)

 timereg::Cpred(cmz$casewise,80)
 timereg::Cpred(cdz$casewise,80)

## ----label=concordance3-------------------------------------------------------

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-DZ","concordance-MZ"),col=c(1,2),lty=1)
 plot(cmz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,col=c(2,3,3),ylab="Casewise",xlab="Age",lty=c(1,3))
 plot(cdz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(1,3,3), add=TRUE, lty=c(2,3))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=c(2,1,3),lty=1)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=2))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=1))


## ----label=concordance4, eval=fullVignette------------------------------------
#   ### new version of Casewise for specific time-point based on binreg
#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
#   newdata <- data.frame(zyg=c("DZ","MZ"),id=1)
#  
#   ## concordance
#   bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
#   pconc <- predict(bcif1,newdata)
#  
#   ## marginal estimates
#   mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
#   mc <- predict(mbcif1,newdata)
#  
#   ### casewise with improved se's from log-scale
#   cse <- binregCasewise(bcif1,mbcif1)

## -----------------------------------------------------------------------------
 cse 

## ----label=semiparconc, eval=fullVignette-------------------------------------
#   ### semi-parametric modelling of concordance
#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
#   regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
#   regconc
#   ### interaction test
#   wald.test(regconc,coef.null=5:7)
#  
#   regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
#   regconc
#  
#   ## logistic link
#   logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
#   slr <- summary(logitregconc)

## -----------------------------------------------------------------------------
slr
### library(Publish)
### publish(round(slr$exp.coef[,-c(2,5)],2),latex=TRUE,digits=2)

## ----label=additive_gamma, eval=fullVignette----------------------------------
#    times <- seq(50,90,length.out=5)
#    cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
#  		   cause=2,times=times,max.clust=NULL)
#  
#    mm <- model.matrix(~-1+factor(zyg),prt)
#    out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
#  		  theta.des=mm,same.cens=TRUE,step=0.5)
#    summary(out1)
#    round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)
#  
#    desace <- twin.polygen.design(prt,type="ace")
#  
#    outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
#    	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
#           step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
#    ##outacem$score

## -----------------------------------------------------------------------------
  summary(outacem)

 ###  variances
 estimate(coef=outacem$theta,vcov=outacem$var.theta,f=function(p) p/sum(p)^2)

 ## AE polygenic model
 # desae <- twin.polygen.design(prt,type="ae")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #    same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
 #        step=0.5,theta.des=desae$pardes,random.design=desae$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p)     p/sum(p)^2)

 ## AE polygenic model
 # desde <- twin.polygen.design(prt,type="de")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #   same.cens=TRUE,theta=c(0.35),var.link=0,
 #   step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

  times <- 90
  cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

 desde <- twin.polygen.design(prt,type="de")
 outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
	same.cens=TRUE,theta=c(0.35),var.link=0,
        step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 outaem$score
 summary(outaem)
 estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)


## ----label=probit1------------------------------------------------------------
rm(prt)
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in

## ----b0, eval=fullVignette----------------------------------------------------
#  b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)

## -----------------------------------------------------------------------------
summary(b0)

## ----label=liability_ace1, eval=fullVignette----------------------------------
#  b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)

## -----------------------------------------------------------------------------
summary(b1)

## -----------------------------------------------------------------------------
AIC(b0, b1)

## ----label=liability_ace_country, eval=fullVignette---------------------------
#  b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg+country, breaks=95)

## -----------------------------------------------------------------------------
summary(b2)

## ----label=bptime1, eval=fullVignette-----------------------------------------
#  bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg,
#                summary.function=function(x) x, breaks=tt)
#  h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
#  concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))
#  

## -----------------------------------------------------------------------------
par(mfrow=c(1,2))
plot(tt, h2[,1], type="s", lty=1, col=cols[3], xlab="Age", ylab="Heritability", ylim=c(0,1))
lava::confband(tt, h2[,2], h2[,3],polygon=TRUE, step=TRUE, col=lava::Col(cols[3], 0.1), border=NA)
plot(tt, concMZ[,1], type="s", lty=1, col=cols[1], xlab="Age", ylab="Concordance", ylim=c(0,.1))
lava::confband(tt, concMZ[,2], concMZ[,3],polygon=TRUE, step=TRUE, col=lava::Col(cols[1], 0.1), border=NA)

## ----label=biprobittime1------------------------------------------------------
system.time(a.mz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~1, pairs.only=TRUE,
                                breaks=tt))
system.time(a.dz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~1, pairs.only=TRUE,
                               breaks=tt))

#system.time(a.zyg <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, 
#                               cens.formula = Event(time,status==0)~1,
#                               eqmarg=FALSE, fix.cens.weight
#                               breaks=seq(75,100,by=10)))

a.mz
a.dz

plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance", ylim=c(0,0.07))
plot(a.mz, ylim=c(0,.07), col=cols[1], lty=ltys[1], legend=FALSE, add=TRUE)
plot(a.dz, col=cols[2], lty=ltys[2], add=TRUE)

## ----label=biprobittime2, eval=fullVignette-----------------------------------
#  a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
#                                 cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
#                                  breaks=tt)
#  system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
#                                 cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
#                                 breaks=tt))
#  
#  s_mz_country <- summary(a.mz_country)
#  s_dz_country <- summary(a.dz_country)

## -----------------------------------------------------------------------------
s_mz_country
s_dz_country

## ----label=liability_ace_time1, eval=fullVignette-----------------------------
#  ## ACE model (time-varying) with and without adjustment for country
#  a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
#                                zyg="zyg", DZ="DZ",
#                                cens.formula=Surv(time,status==0)~zyg,
#                                breaks=tt)
#  
#  #a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
#  #                              zyg="zyg", DZ="DZ",
#  #                              #cens.formula=Surv(time,status==0)~country+zyg,
#  #                              breaks=tt)

## -----------------------------------------------------------------------------
plot(a.mz, which=c(6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1], lty=ltys[1], legend=NULL, alpha=.1)
plot(a.dz, which=c(6), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, alpha=.1)
legend("topleft", c("MZ tetrachoric correlation", "DZ tetrachoric correlation"),
       col=cols, lty=ltys, lwd=2)

plot(a.mz, which=c(4), xlab="Age", ylab="Relative Recurrence Risk",
     ylim=c(1,20), col=cols[1], lty=ltys[1], legend=NULL, lwd=2, alpha=.1)
plot(a.dz, which=c(4), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, lwd=2, alpha=.1)
legend("topright", c("MZ relative recurrence risk", "DZ relative recurrence risk"),
       col=cols, lty=ltys, lwd=2)

plot(a1, which=c(5,6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1:2], lty=ltys[1:2], lwd=2, alpha=0.1,
     legend=c("MZ tetrachoric correlation", "DZ tetrachoric correlation"))

plot(a1, which=c(1), xlab="Age", ylim=c(0,1), col="black", lty=1, ylab="Heritability", legend=NULL, alpha=.1)

## -----------------------------------------------------------------------------
sessionInfo()

## ----saveobj, results="hide", echo=FALSE, eval=fullVignette-------------------
#  ## To save time building the vignettes on CRAN, we cache time consuming computations
#  
#  rms <- c('id','theta.iid','theta.des','marginal.trunc',
#    'loglikeiid','marginal.surv','theta.iid.naive',
#    'antclust','secluster','cluster.call','trunclikeiid',
#    'logl.iid','score.iid','clusters')
#  tmp <- lapply(as.list(paste0("fitco", 1:4)),
#                function(x) saveobj(x, rms))
#  
#  rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
#  saveobj("fitace", rms)
#  saveobj("fitde", rms)
#  
#  saveobj("cse", NULL)
#  saveobj("slr", NULL)
#  
#  rms <- c("theta.iid", "Clusters", "p11")
#  saveobj("outacem", rms)
#  
#  rms <- c("model.frame", "score", "id", "logLik")
#  saveobj("b0", rms)
#  saveobj("b1", rms)
#  saveobj("b2", rms)
#  saveobj("a1", rms)
#  saveobj("h2", NULL)
#  saveobj("concMZ", NULL)
#  saveobj("s_mz_country", NULL)
#  saveobj("s_dz_country", NULL)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## ----while-alive--------------------------------------------------------------
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")


## ----while-alive-reg----------------------------------------------------------
hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id]
hfactioncpx12$sex <- rbinom(741,1,0.5)[hfactioncpx12$id]

dd <- WA_reg(Event(entry,time,status)~treatment+age+sex+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

## ----wa-composite-------------------------------------------------------------
hfactioncpx12$marks <- runif(nrow(hfactioncpx12))

##ddmg <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
##cause=1:2,death.code=2,marks=hfactioncpx12$marks)
##summary(ddmg)

ddm <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
cause=1:2,death.code=2,marks=hfactioncpx12$status)


## -----------------------------------------------------------------------------
sessionInfo()

###{{{ print.twinlm

##' @export
print.twinlm <- function(x,...) {  
    print(summary(x,...))
    invisible(x)
}

###}}} print.twinlm

###{{{ summary.twinlm

summarygroup.twinlm <- function(object,...) {
    mz <- grep("MZ",names(object$model))
    cc <- lapply(mz,function(i)
                 coef(object$model[[i]],label=TRUE))
    ii <- lapply(cc, function(x)
                 sapply(x, function(i) lava::parpos(object$estimate$model,i)))    
    c0 <- coef(object$estimate)
    c2 <- coef(object$estimate,level=2)
    pnam <- names(c0)
    coefs <- c()
    acde <- c()
    nn <- c()
    lambdas <- c("~a","~c","~d","~e")
    for (i in seq(length(mz))) {
        res <- coef(object$estimate,level=1)[ii[[i]],,drop=FALSE]
        pp <- sapply(lambdas,function(x) ii[[i]][grep(x,rownames(res),fixed=TRUE)])
        nam <- c("A","C","D","E")[which(unlist(lapply(pp,function(x) length(x)>0)))]
        pp <- unlist(pp); names(pp) <- nam
        acde <- c(acde,list(acde.twinlm(object,pp)))
        nn0 <- cc[[i]]
        if (nn0[1]=="mu") nn0[1] <- "(Intercept)"
        for (k in c("a","c","d","e"))
            nn0 <- gsub("lambda["%++%k%++%"]","SD("%++%toupper(k)%++%"):",nn0,fixed=TRUE)
        nam <- rownames(res)
        idx <- unlist(lapply(c("Intercept","SD(","z(A):","z(D):"),function(x) grep(x,nn0,fixed=TRUE)))
        nam.keep.idx <- setdiff(seq(length(nam)),idx)
        nn0[nam.keep.idx] <- unlist(lapply(strsplit(nam[nam.keep.idx],"~"),
                                           function(x) gsub(".2|.1","",x[2])))
        rownames(res) <- nn0
        coefs <- c(coefs,list(res))       
    }; names(coefs) <- names(object$model)[mz]    
    vcov <- vcov(object$estimate)
    suppressWarnings(kinship <- constraints(object$estimate)[,c(1,5,6),drop=FALSE])
    fit <- c(logLik=logLik(object),AIC=AIC(object),BIC=BIC(object))
    structure(list(coef=c0,coefmat=coefs,vcov=vcov,acde=acde,kinship=kinship,fit=fit),class="summary.twinlm.group")
}

##' @export
print.summary.twinlm.group <- function(x,...) {
    for (i in seq(length(x$coefmat))) {
        cat(names(x$coefmat)[i],"\n")
        printCoefmat(x$coefmat[[i]],...)
        print(x$acde[[i]])
        cat("\n")
    }
    cat("\n")
    print(x$kinship)
    cat("\n")
    print(x$fit)
    invisible(x)
}


##' @export
summary.twinlm <- function(object,transform=FALSE,...) {

    if (!is.null(object$group) && !object$group.equal) {
        return(summarygroup.twinlm(object,...))
    }   
    
    e <- object$estimate    
    zygtab <- object$zygtab    
    ## zygtab <- with(object, table(data[,zyg]))
    ## names(zygtab) <- paste(names(zygtab),"pairs",sep="-")

    theta <- pars(e)
    theta.sd <- sqrt(diag(e$vcov))
    myest <- cbind(theta,theta.sd,(Z <- theta/theta.sd),2*(pnorm(abs(Z),lower.tail=FALSE)))
    colnames(myest) <- c("Estimate","Std. Error", "Z value", "Pr(>|z|)")

    if (object$type%in%c("u","flex","sat")) {
        corMZ <- corDZ <- NULL
        if (object$constrain) {
            i1 <- lava::parpos(object$estimate$model,p="atanh(rhoMZ)")[1]
            i2 <- lava::parpos(object$estimate$model,p="atanh(rhoDZ)")[1]
            if (length(i1)>0) {
                corest <- coef(object$estimate,level=0)[c(i1,i2)]
                sdest <- vcov(object$estimate)[cbind(c(i1,i2),c(i1,i2))]^0.5

                ciest <- tanh(cbind(corest,corest)+qnorm(0.975)*cbind(-sdest,sdest))
                corest <- tanh(corest)
                corMZ <- c(corest[1],ciest[1,])
                corDZ <- c(corest[2],ciest[2,])
            }
        }
        aa <- capture.output(e)
        res <- list(estimate=aa, zyg=zygtab,
                    varEst=NULL, varSigma=NULL, heritability=NULL,
                    corMZ=corMZ, corDZ=corDZ, 
                    logLik=logLik(e), AIC=AIC(e), BIC=BIC(e), type=object$type,
                    vcov=vcov(e)                    
                    )                
        class(res) <- "summary.twinlm"
        return(res)
    }

    KinshipGroup <- NULL
    if (!is.null(object$group)) {
        zA.idx <- grep("z(A):",names(coef(object)),fixed=TRUE)
        zD.idx <- grep("z(A):",names(coef(object)),fixed=TRUE)
        KinshipGroup <- constraints(e)[,c(1,5,6),drop=FALSE]
    }

    r1 <- gsub(".1","",coef(Model(Model(e))[[1]],
                            mean=e$meanstructure, silent=TRUE),
               fixed=TRUE)
    rownames(myest) <- seq(nrow(myest))
    idx <- seq(nrow(myest)); 
    rownames(myest)[idx] <- r1

    coefpost <- paste(lava.options()$symbol[1],c("a1","c1","d1","e1"),sep="")
    coefpost2 <- paste(lava.options()$symbol[1],c("a2","c2","d2","e2"),sep="")
    myest.varpos <- unlist(sapply(coefpost,function(x) grep(x,rownames(myest))))  
    lambda.idx <- sapply(coefpost,function(x) grep(x,names(coef(e))))
    lambda.idx2 <- sapply(coefpost2,function(x) grep(x,names(coef(e))))
    for (k in seq_len(length(lambda.idx)))
        if (length(lambda.idx[[k]])==0) lambda.idx[[k]] <- lambda.idx2[[k]] 

    lambda.w <- which(sapply(lambda.idx, function(x) length(x)>0))

    rownames(myest)[myest.varpos] <- paste("sd(",c("A)","C)","D)","E)"),sep="")[lambda.w]

    varEst <- rep(0,4)
    varEst[lambda.w] <- myest[myest.varpos,1]
    varSigma <- matrix(0,4,4);
    varSigma[lambda.w,lambda.w] <- e$vcov[unlist(lambda.idx),unlist(lambda.idx)]

    L <- gaussian("identity")
    if (transform) L <- binomial("logit")
    varcomp <- c()
    genpos <- c()
    pos <- 0L
    {
        varcomp <- c()
        if ("a1"%in%latent(e)) { varcomp <- "lambda[a]"; pos <- pos+1
                                 genpos <- c(genpos,pos) }
        if ("c1"%in%latent(e)) { varcomp <- c(varcomp,"lambda[c]"); pos <- pos+1 }
        if ("d1"%in%latent(e)) { varcomp <- c(varcomp,"lambda[d]"); pos <- pos+1;
                                 genpos <- c(genpos,pos) }
        isOrdinal <- (object$ordinal>0)*1
        if (isOrdinal) varEst[4] <- 1
        if ("e1"%in%latent(e) & !isOrdinal) { varcomp <- c(varcomp,"lambda[e]"); pos <- pos+1 }
        f <- paste("h2~",paste(varcomp,collapse="+"))
        constrain(e, as.formula(f)) <- function(x) {
            L$linkfun(sum(x[genpos]^2)/sum(x^2+isOrdinal))
        }

    }

    ci.logit <- L$linkinv(constraints(e,k=1)["h2",5:6])
    
    h <- function(x) (x[1]^2)/(sum(x^2))
    dh <- function(x) {
        y <- x^2
        cbind(1/sum(y)^2*c(2*x[1]*(sum(y)-y[1]), -2*x[2:4]*y[1]))
    }
    ##  f(x1,x2,x3,x4) = x1/(x1+x2+x3+x4) = x1/s
    ## Quotient rule: (u/v)' = (u'v - uv')/v^2
    ##  f1(x1,x2,x3,x4) = (s - x1)/s^2 = (x2+x3+x4)/s^2
    ##  f2(x1,x2,x3,x4) = -1/(s)^2
    h2 <- function(x) (x[1]^2+x[3]^2)/sum(x^2)
    dh2 <- function(x) {
        y <- x^2
        cbind(1/sum(y)^2*c(
            2*x[1]*(sum(y)-(y[1]+y[3])),
            -2*x[2]*(y[1]+y[3]),
            2*x[3]*(sum(y)-(y[1]+y[3])),
            -2*x[4]*(y[1]+y[3])
            ))
    }
    hval <- cbind(h(varEst), (t(dh(varEst))%*%varSigma%*%(dh(varEst)))^0.5)
    colnames(hval) <- c("Estimate", "Std.Err"); rownames(hval) <- "h squared"
    h2val <- cbind(h2(varEst), (t(dh2(varEst))%*%varSigma%*%(dh2(varEst)))^0.5)
    colnames(h2val) <- c("Estimate", "Std.Err"); rownames(h2val) <- "h squared"
    
    atanhcorMZf <- function(x) atanh(sum(x[1:3]^2)/sum(x^2))
    atanhcorDZf <- function(x) atanh(sum(x[1:3]^2*c(0.5,1,0.25))/sum(x^2))
    
    e1 <- atanhcorMZf(varEst)
    D1 <- numDeriv::grad(atanhcorMZf,varEst)
    s1 <- (t(D1)%*%varSigma%*%(D1))^0.5
    ci1 <- e1+qnorm(0.975)*c(-1,1)*s1[1]
    e2 <- atanhcorDZf(varEst)
    D2 <- numDeriv::grad(atanhcorDZf,varEst)
    s2 <- (t(D2)%*%varSigma%*%(D2))^0.5
    ci2 <- e2+qnorm(0.975)*c(-1,1)*s2[1]
    
    corMZ <- c(tanh(c(e1,ci1)))
    corDZ <- c(tanh(c(e2,ci2)))  
    acde <- acde.twinlm(object,transform=transform)
    coef <- rbind(acde)  

   
    hrow <- rbind(c(h2val,ci.logit));
    rownames(hrow) <- "Broad-sense heritability"
    colnames(hrow)[1:2] <- c("Estimate","Std.Err")
    
    all <- rbind(hrow[,c(1,3,4),drop=FALSE],coef,corMZ,corDZ)
    
    res <- list(estimate=myest, zyg=zygtab, varEst=varEst,
                KinshipGroup=KinshipGroup,
                varSigma=varSigma, heritability=hrow, corMZ=corMZ, corDZ=corDZ,
                acde=acde, logLik=logLik(e), AIC=AIC(e), BIC=BIC(e),
                type=object$type, coef=coef, all=all, vcov=vcov(e));
    class(res) <- "summary.twinlm"
    return(res)
}

###}}} summary.twinlm

###{{{ print.summary.twinlm

##' @export
print.summary.twinlm <- function(x,signif.stars=FALSE,...) {

    if (x$type%in%c("flex","u","sat")) {
        cat(x$estimate,sep="\n")
    } else {

        printCoefmat(x$estimate,signif.stars=signif.stars,...)
        cat("\n")
        print(x$zyg,quote=FALSE)

        if (!is.null(x$acde)) {
            cat("\nVariance decomposition:\n")
            print(RoundMat(x$acde,...),quote=FALSE)
        }
        cat("\n\n")
        ##    cat("Broad-sense heritability (total genetic factors):\n")
        h <- with(x, heritability[,c(1,3,4),drop=FALSE]);
        h <- na.omit(h)
        print(RoundMat(h,...),quote=FALSE)
        cat("\n")
    }
    if (!is.null(x$corMZ)) {
        cc <- with(x, rbind(corMZ,corDZ))        
        rownames(cc)[1:2] <- c("Correlation within MZ:","Correlation within DZ:")
        if (!is.null(x$KinshipGroup)) {
            cc <- rbind(cc,x$KinshipGroup)
        }      
        colnames(cc) <- c("Estimate","2.5%","97.5%")
        print(RoundMat(cc,...),quote=FALSE)
    }
    
    cat("\n")
    print(x$logLik)
    cat("AIC:", x$AIC, "\n")
    cat("BIC:", x$BIC, "\n")
    invisible(x)
}

###}}} print.summary.twinlm

###{{{ compare.twinlm

##' @export
compare.twinlm <- function(object,...) {
    if (length(list(...))==0) return(compare(object$estimate))
    getS3method("compare","default")(object,...)
}

###}}} compare.twinlm

###{{{ plot.twinlm

##' @export
plot.twinlm <- function(x,diag=TRUE,labels=TRUE,...) {
    op <- par(mfrow=c(2,1))
    plot(x$model,...)
    par(op)
}
###}}}

###{{{ vcov.twinlm

##' @export
vcov.twinlm <- function(object,...) {
    return(object$vcov)
}

###}}} vcov.twinlm

###{{{ logLik.twinlm

##' @export
logLik.twinlm <- function(object,...) logLik(object$estimate,...)
###}}} logLik.twinlm

###{{{ score.twinlm
##' @export
score.twinlm <- function(x,...) score(x$estimate,...)
###}}} score.twinlm

###{{{ model.frame.twinlm

##' @export
model.frame.twinlm <- function(formula,...) {
    return(formula$estimate$model$data)
}
###}}} model.frame.twinlm

###{{{ acde

##"acde" <- function(x,...) UseMethod("acde")
acde.twinlm <- function(x,index,transform=FALSE,estimate.return=FALSE,...) {
    if (missing(index)) {
        m <- x$estimate$model$lvm[[1]]
        lambdas <- c("lambda[a]","lambda[c]","lambda[d]","lambda[e]")
        pp <- sapply(lambdas,function(x) grep(x,as.vector(m$par),fixed=TRUE))
        ACDE <- unlist(lapply(pp,function(x) length(x)>0))
        pp <- unlist(lapply(pp[ACDE], function(x) as.vector(m$par)[x[1]]))
        index <- sapply(pp,function(p) lava::parpos(x$estimate$model,p))
        names(index) <- c("A","C","D","E")[ACDE]
    }

    ord <- x$ordinal>0
    ##    f <- function(p) structure(log(p/(1-p)),grad=cbind(0,1/(p*(1-p)),0))    
    if (transform) {        
        suppressWarnings(res <- estimate(x,function(p)
                                         lapply(index,
                                                function(z)
                                                lava::logit((p[z]^2/sum(c(p[index]^2,ord))))),
                                         vcov=vcov(x)))
        if (estimate.return) return(res)
        res <- res$coefmat[,c(1,3,4),drop=FALSE]
        res <- lava::tigol(res)
    } else {
        suppressWarnings(res <- estimate(x,function(p)
                                         lapply(index,
                                                function(z) {
                                                    p[z]^2/sum(c(p[index]^2),ord)
                                                }),                                                
                                         vcov=vcov(x)))
        if (estimate.return) return(res)        
        res <- res$coefmat[,c(1,3,4),drop=FALSE]
    }
    res
}

###}}} acde

##' @export
coef.twinlm <- function(object,...) coef(object$estimate,...)

##' @export
iid.twinlm <- function(x,...) {
    U <- score(x$estimate,indiv=TRUE)
    U <- lapply(U,function(x) {x[is.na(x)] <- 0; return(x)})
    U <- Reduce(rbind,U)
    iI <- vcov(x)
    U%*%iI
}

###{{{ RoundMat

RoundMat <- function(cc,digits = max(3, getOption("digits") - 2),na=TRUE,...) {
  res <- format(round(cc,max(1,digits)),digits=digits)
  if (na) return(res)
  res[grep("NA",res)] <- ""
  res
}

###}}} RoundMat

###{{{ multinomlogit

multinomlogit <- function(x,tr=exp,dtr=exp) {
  n <- length(x)
  ex <- tr(x)
  dex <- dtr(x)
  sx <- sum(ex)+1
  f <- c(ex,1)
  df <- c(dex,0)
  res <- f/sx
  dg <- -dex/sx^2   
  gradient <- matrix(ncol=n,nrow=n+1)
  I <- diag(n+1)
  for (i in seq_len(n)) {
    gradient[,i] <- df[i]*I[i,]/sx+dg[i]*f
  }
  attributes(res)$gradient <- gradient
  return(res)
}

###}}} multinomlogit

###{{{ grouptable

##' Create Group Contingency Table from Clustered Data
##'
##' Creates a contingency table by group from paired/clustered data,
##' optionally combining lower and upper triangles.
##'
##' @param data a data.frame.
##' @param id name of the cluster/pair identifier column.
##' @param group name of the grouping variable (e.g., zygosity).
##' @param var name of the outcome variable to tabulate.
##' @param lower logical; if TRUE, fold upper triangle into lower.
##' @param labels optional labels for levels of \code{var}.
##' @param order optional ordering of factor levels.
##' @param group.labels optional labels for the groups.
##' @param group.order optional ordering of groups.
##' @param combine separator for combining two groups (default \code{" & "}).
##' @param ... additional arguments.
##' @return A table or list of tables.
##' @export
grouptable <- function(data,id,group,var,lower=TRUE,
                       labels,order,
                       group.labels,group.order,
                       combine=" & ",...) {
    if (!missing(order) || !missing(labels)) {
        data[,var] <- as.factor(data[,var])
        if (missing(order)) order <- seq(length(labels))
        if (missing(labels)) labels <- levels(data[,var])
        data[,var] <- factor(data[,var],levels(data[,var])[order],labels=labels[order])
    }
    wide <- fast.reshape(data,id=id,varying=-group)    
    res <- lapply(split(wide,wide[,group]),
                  function(x) {
                      M <- with(x, table(get(paste(var,"1",sep="")),
                                         get(paste(var,"2",sep=""))))
                      if (lower) {
                          M[lower.tri(M)] <- M[lower.tri(M)]+M[upper.tri(M)]
                          M[upper.tri(M)] <- NA
                      }
                      return(M)
                  })
    if (!missing(group.order) && length(group.order)==length(res))
        res <- res[group.order]
    if (!missing(group.labels) && length(group.labels)==length(res))
        names(res) <- group.labels
    if (length(res)==2 && !is.null(combine)) {
        M <- res[[1]]
        M[upper.tri(M)] <- res[[2]][lower.tri(res[[2]])]
        diag(M) <- paste(diag(M),diag(res[[2]]),sep=combine)
        M <- cbind(rownames(M),M)
        M <- rbind(c("",rownames(M)),M)
        colnames(M) <- rownames(M) <- rep("",nrow(M))
        M[1,1] <- paste(names(res),collapse=combine)
        return(structure(M,class="table"))
        return(M);
    }    
    res
}

###}}} grouptable

## Pseudo-inverse
pinv <- function(x, ...) {
  lava::Inverse(x, ...)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
set.seed(101)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean_phreg(out1,times=30)
     summary(rm1)
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     ## same as integrated cumulative incidence 
     rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)

## -----------------------------------------------------------------------------
estimate(out)

measures <- function(p) {
 ratio1 <- p[1]/p[2]; ratio2 <- p[2]/p[1]; dif1 <- p[4]-p[1]; dif2 <- p[3]-p[1]
 m <- c(dif1,dif2,ratio1,ratio2)
 return(m)
}

 labs <- c("dif4-1","dif3-1","ratio 1/2","ratio 2/1")
 estimate(out,f=measures,labels=labs)

## -----------------------------------------------------------------------------
 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
		    treat.model=tcell~platelet)
 summary(out1)

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,time=40,
		    treat.model=tcell~platelet)
 summary(out2)

## -----------------------------------------------------------------------------
sessionInfo()

##' Force Same Censoring Within Clusters
##'
##' Enforces the same censoring time within clusters (pairs) by censoring
##' both subjects at the minimum censoring time when censoring occurs first.
##'
##' @param data a data.frame in long format.
##' @param id name of the cluster/pair identifier column.
##' @param time name of the time variable.
##' @param cause name of the cause/status variable.
##' @param entrytime optional name of left-truncation time variable.
##' @param cens.code value indicating censoring in the cause variable.
##' @return A data.frame with enforced same censoring.
##' @aliases force.same.cens force_same_cens
##' @export
force.same.cens <- function(data,id="id", time="time",cause="cause",entrytime=NULL,cens.code=0)
{ ## {{{ 
  ### no missing values

   ### handle time-variables separately
   data <- dsort(data,id)
   w <- which(names(data) %in% c(time,cause,entrytime,id) )
   datao <- data[,-w]

   data <- data[,c(time,cause,entrytime,id)]

  if (is.null(entrytime)) entry <- rep(0,nrow(data)) else entry <- data[,entrytime]

   censo <- (data[,cause]==cens.code)
   Wide <- fast.reshape(data,id=id)
   time1 <- paste(time,1,sep="")
   time2 <- paste(time,2,sep="")
   stat1 <- paste(cause,1,sep="")
   stat2 <- paste(cause,2,sep="")

   ### enforce same censoring ## {{{ 

   mintime <- pmin(Wide[,time1],Wide[,time2])
   statmin <- ifelse(Wide[,time1]<Wide[,time2],Wide[,stat1],Wide[,stat2])
###
   cens.first <- which(statmin==cens.code)
   Wide[cens.first,time1] <- mintime[cens.first]
   Wide[cens.first,time2] <- mintime[cens.first]
   Wide[cens.first,stat1] <- cens.code
   Wide[cens.first,stat2] <- cens.code

###   idn <- table(data[,id]); rep(mintime,idn)
   
   long <-   fast.reshape(Wide)
   long <- long[!is.na(long[,time]) & !is.na(long[,cause]),]

   ## }}} 

   if (!is.null(entrytime)) { ## {{{ enforce same truncation
      stop("not implemented yet\n")
   entry1 <- paste(entrytime,1,sep="")
   entry2 <- paste(entrytime,2,sep="")
###   trunc.first<- which(statmin==cens.code)
   trunc.max <- pmax(Wide[,entry1],Wide[,entry2])
   Wide[,entry1] <- Wide[,entry2] <- trunc.max
### drop those that enter later
  enter.after <- ( Wide[,entry1] < Wide[,time1]) & (Wide[,entry2] < Wide[,time2])
  Wide <- Wide[enter.after,] 
  } ## }}} 

 long <- cbind(long,datao)

 return(long)
} ## }}} 

##' @rdname force.same.cens
##' @export
force_same_cens <- function(data,id="id", time="time",cause="cause",entrytime=NULL,cens.code=0)
{ ## {{{ 
  ### no missing values

   ### handle time-variables separately
   data <- dsort(data,id)
   w <- which(names(data) %in% c(time,cause,entrytime,id) )
   datao <- data[,-w]

   data <- data[,c(time,cause,entrytime,id)]

  if (is.null(entrytime)) entry <- rep(0,nrow(data)) else entry <- data[,entrytime]

   censo <- (data[,cause]==cens.code)
   Wide <- fast.reshape(data,id=id)
   time1 <- paste(time,1,sep="")
   time2 <- paste(time,2,sep="")
   stat1 <- paste(cause,1,sep="")
   stat2 <- paste(cause,2,sep="")

   ### enforce same censoring ## {{{ 

   mintime <- pmin(Wide[,time1],Wide[,time2])
   statmin <- ifelse(Wide[,time1]<Wide[,time2],Wide[,stat1],Wide[,stat2])
###
   cens.first <- which(statmin==cens.code)
   Wide[cens.first,time1] <- mintime[cens.first]
   Wide[cens.first,time2] <- mintime[cens.first]
   Wide[cens.first,stat1] <- cens.code
   Wide[cens.first,stat2] <- cens.code

###   idn <- table(data[,id]); rep(mintime,idn)
   
   long <-   fast.reshape(Wide)
   long <- long[!is.na(long[,time]) & !is.na(long[,cause]),]

   ## }}} 

   if (!is.null(entrytime)) { ## {{{ enforce same truncation
      stop("not implemented yet\n")
   entry1 <- paste(entrytime,1,sep="")
   entry2 <- paste(entrytime,2,sep="")
###   trunc.first<- which(statmin==cens.code)
   trunc.max <- pmax(Wide[,entry1],Wide[,entry2])
   Wide[,entry1] <- Wide[,entry2] <- trunc.max
### drop those that enter later
  enter.after <- ( Wide[,entry1] < Wide[,time1]) & (Wide[,entry2] < Wide[,time2])
  Wide <- Wide[enter.after,] 
  } ## }}} 

 long <- cbind(long,datao)

 return(long)
} ## }}} 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)

## -----------------------------------------------------------------------------
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

## marginal mean using formula  
outNZ <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 

outN <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
summary(outN,times=3) 

## -----------------------------------------------------------------------------
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
head(iid(outNZ3))

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
head(iid(outN3))

## -----------------------------------------------------------------------------
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
plot(GL,add=TRUE,col=2)

## -----------------------------------------------------------------------------
ooNZ <- prob_exceed_recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
summary(ooNZ,times=3)

## -----------------------------------------------------------------------------
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
head(iid(RecLost))

## -----------------------------------------------------------------------------
sessionInfo()

##' Goodness-of-Fit for Cox PH Regression (Proportionality)
##'
##' Performs cumulative score process residual tests for the proportional hazards (PH) 
##' assumption in Cox regression. The test statistics are based on the cumulative 
##' score process:
##' \deqn{ U(t) = \int_0^t (X_i - E(t) ) d \hat M_i(s) }
##' where \eqn{\hat M_i(s)} are the martingale residuals.
##' 
##' P-values are computed using the Lin, Wei, and Ying (1993) resampling method, which 
##' simulates the asymptotic distribution of the supremum of the score process under 
##' the null hypothesis of proportional hazards.
##'
##' The function supports two types of simulation:
##' \itemize{
##'   \item \strong{Standard}: Uses \eqn{dN_i} (counting process increments) for simulation.
##'   \item \strong{Robust}: Uses \eqn{\hat M_i(t)} (martingale residuals) adjusted for clustering 
##'     if a \code{cluster()} term is present in the model.
##' }
##'
##' @param object A fitted \code{phreg} object (from \code{mets} or \code{survival}).
##' @param n.sim Number of simulations for the resampling procedure (default 1000).
##' @param silent Logical; if TRUE, suppresses timing estimates for long jobs.
##' @param robust Logical; if TRUE, uses robust martingale-based simulations. 
##'   If NULL, defaults to TRUE if a cluster term is detected in the model call.
##' @param ... Additional arguments passed to lower-level functions.
##' @return An object of class \code{"gof.phreg"} containing:
##'   \item{jumptimes}{Event times used in the process.}
##'   \item{supUsim}{Matrix of simulated supremum values for each covariate.}
##'   \item{res}{Matrix with observed supremum (\code{Sup|U(t)|}) and p-values.}
##'   \item{supU}{Observed supremum values.}
##'   \item{pvals}{Vector of p-values for each covariate.}
##'   \item{score}{Cumulative score process values over time.}
##'   \item{simUt}{Simulated score processes.}
##'   \item{type}{Type of test performed ("prop").}
##'   \item{robust}{Logical flag indicating if robust simulation was used.}
##' @author Thomas Scheike and Klaus K. Holst
##' @references 
##' Lin, D. Y., Wei, L. J., & Ying, Z. (1993). Checking the Cox model with cumulative sums of martingale-based residuals. Biometrika, 80(3), 557-572.
##' @seealso \code{\link{gofM_phreg}}, \code{\link{gofZ_phreg}}
##' @examples
##' data(sTRACE)
##' 
##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes, data=sTRACE) 
##' gg <- gof(m1)
##' gg
##' par(mfrow=c(1,3))
##' plot(gg)
##' 
##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes, data=sTRACE) 
##' gg <- gof(m1)
##' 
##' ## Robust simulations with cluster
##' sTRACE$id <- 1:500
##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes+cluster(id), data=sTRACE) 
##' gg <- gof(m1)
##' gg
##' @export
gof.phreg  <- function(object,n.sim=1000,silent=1,robust=NULL,...)
{# {{{

### test for proportionality 
p <- length(object$coef)
if (p==0) stop("Goodness of fit for proportionality, must contain regression effects\n");
nnames <- names(object$coef)
ii <- pinv(object$hessian)
jumptimes <- object$jumptimes
Pt <- object$hessianttime
U <-  object$U
Pt <- apply(Pt,2,cumsum)
Ut <- apply(U,2,cumsum)

nd <- nrow(object$U)
Pt <-  .Call("XXMatFULL",Pt,p,PACKAGE="mets")$XXf
Pt <-  .Call("CubeMat",Pt,ii,PACKAGE="mets")$XXX
sup <- matrix(0,n.sim,nrow(ii))
hatti <- matrix(0,nd,nrow(ii))
obs <- apply(abs(Ut),2,max)

if (is.null(robust)) {
   if (!is.null(object$call.id)) robust <- TRUE else robust <- FALSE
   if (inherits(object,c("cifreg"))) {
	   robust <- FALSE  ## approximative test using MG framework
###	   if (is.null(object$cox.prep)) stop("Must be called with cox.prep=TRUE\n")
   }
}


   ### cluster call or robust \hat M_i(t) based  
if (robust) {# {{{
   xx <- object$cox.prep
   Z <- xx$X
   rrw <- c(xx$sign * exp(Z %*% coef(object) + xx$offset)*xx$weights)
   UdN <- xx$weights[xx$jumps+1]*object$U 
   ### also weights 
   nn <- nrow(Z)

   tt <- system.time(simcox1<- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
     object$S0,object$E,
     10,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps))

   prt <- n.sim*tt[3]/(10*60)
   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
   simcox <- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
	     object$S0,object$E,
	     n.sim,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps)
} else {# }}}
###  or dN_i based  # {{{
   tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
   prt <- n.sim*tt[3]/(10*60)
   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
   simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
}# }}}

sup <-  simcox$supUsim
res <- cbind(obs,simcox$pval)
colnames(res) <- c("Sup|U(t)|","pval")
rownames(res) <- nnames 

if (silent==0) {
cat("Cumulative score process test for Proportionality:\n")
prmatrix(round(res,digits=2))
}

out <- list(jumptimes=object$jumptimes,supUsim=sup,res=res,supU=obs,
    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop",robust=robust)
class(out) <- "gof.phreg"
return(out)
}# }}}

##' Goodness-of-Fit for Cox Covariates (Model Matrix)
##'
##' Tests the functional form of covariates in a Cox PH model by computing cumulative 
##' residuals against a user-specified model matrix. This helps detect non-linear effects 
##' or time-varying coefficients (interaction with time).
##'
##' The test statistic is:
##' \deqn{ U(t) = \int_0^t K^T d \hat M }
##' where \eqn{K} is the model matrix (e.g., a set of basis functions for a continuous covariate) 
##' and \eqn{\hat M} are the martingale residuals.
##'
##' P-values are based on the Lin, Wei, and Ying (1993) resampling method. The plot shows 
##' whether the residuals are consistent with the model across the range of the covariate.
##'
##' @param formula Formula for the Cox regression model.
##' @param data Data frame.
##' @param offset Offset vector.
##' @param weights Weights vector.
##' @param modelmatrix Matrix for cumulating residuals. Typically constructed using 
##'   \code{cumContr()} or manually (e.g., quartiles of a continuous covariate).
##' @param n.sim Number of simulations (default 1000).
##' @param silent Logical; suppresses timing estimates if TRUE.
##' @param ... Additional arguments passed to \code{phreg}.
##' @return An object of class \code{"gof.phreg"} containing:
##'   \item{jumptimes}{Event times.}
##'   \item{supUsim}{Simulated supremum values.}
##'   \item{res}{Matrix with observed supremum and p-values for each column of \code{modelmatrix}.}
##'   \item{score}{Cumulative score process.}
##'   \item{simUt}{Simulated processes.}
##'   \item{Utlast, pval.last}{Supremum and p-value for the final time point (covariate direction).}
##'   \item{type}{Type of test ("modelmatrix").}
##' @author Thomas Scheike and Klaus K. Holst
##' @references 
##' Lin, D. Y., Wei, L. J., & Ying, Z. (1993). Checking the Cox model with cumulative sums of martingale-based residuals. Biometrika, 80(3), 557-572.
##' @seealso \code{\link{gof.phreg}}, \code{\link{gofZ_phreg}}, \code{\link{cumContr}}
##' @examples
##' data(TRACE)
##' set.seed(1)
##' TRACEsam <- blocksample(TRACE, idvar="id", replace=FALSE, 100)
##' dcut(TRACEsam) <- ~. 
##' mm <- model.matrix(~-1+factor(wmicat.4), data=TRACEsam)
##' m1 <- gofM_phreg(Surv(time,status==9)~vf+chf+wmi, data=TRACEsam, modelmatrix=mm)
##' summary(m1)
##' if (interactive()) {
##' par(mfrow=c(2,2))
##' plot(m1)
##' }
##' 
##' ## Cumulative sums in covariates via design matrix
##' mm <- mets:::cumContr(TRACEsam$wmi, breaks=10, equi=TRUE)
##' m1 <- gofM_phreg(Surv(time,status==9)~strata(vf)+chf+wmi, data=TRACEsam,
##'          modelmatrix=mm, silent=0)
##' summary(m1)
##' @export
gofM_phreg  <- function(formula,data,offset=NULL,weights=NULL,modelmatrix=NULL,
			n.sim=1000,silent=1,...)
{# {{{

if (is.null(modelmatrix)) stop(" must give matrix for cumulating residuals\n"); 

cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
p <- length(cox1$coef)
if (p==0) stop("Goodness of fit, must contain regression effects\n");

## put modelmatrix on data, and take y, strata from design
datl <- as.data.frame(modelmatrix)
namesmm <- paste("names",1:ncol(modelmatrix),sep="")
names(datl) <- namesmm
datl[,"y__"] <- cox1$design$y
if (!is.null(cox1$design$strata)) {
datl[,"strata__"] <- cox1$design$strata
formM <- y__~strata(strata__)
formM <- update(formM, paste(". ~ . +",paste(namesmm, collapse = " + ")))
} else {
formM <- as.formula(paste("y__~",paste(namesmm, collapse = " + ")))
}

offsets <- cox1$X %*% cox1$coef
if (!is.null(offset)) offsets <- offsets*offset
coxM <- phreg(formM,datl,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,no.var=1,...)
nnames <- colnames(modelmatrix)

Ut <- apply(coxM$U,2,cumsum)
jumptimes <- coxM$jumptimes
U <- coxM$U
Ubeta <- cox1$U
ii <- -pinv(cox1$hessian)
EE <- .Call("vecMatMat",coxM$E,cox1$E,PACKAGE="mets")$vXZ; 
Pt <- cox1$ZX - EE
Pt <- apply(Pt,2,cumsum)
betaiid <- t(ii %*% t(Ubeta))
obs <- apply(abs(Ut),2,max)
simcox <-  .Call("ModelMatrixTestCox",U,Pt,betaiid,n.sim,obs,PACKAGE="mets")

sup <-  simcox$supUsim
res <- cbind(obs,simcox$pval)
colnames(res) <- c("Sup_t |U(t)|","pval")
rownames(res) <- nnames 

if (silent==0) {
   cat("Cumulative score process test for modelmatrix:\n")
   prmatrix(round(res,digits=2))
}

 ## pvals efter z i model.matrix sup_z | M(z,tau) | 
 Utlast <- max(abs(tail(Ut,1)))
 maxlast <- apply(abs(simcox$last),1,max)
 pval.last <- mean(maxlast>=Utlast)
 res.last <- matrix(c(Utlast,pval.last),1,2)
 colnames(res.last) <- c("Sup_z |U(tau,z)|","pval")
 rownames(res.last) <- "matrixZ"

out <- list(jumptimes=jumptimes,supUsim=simcox$supUsim,res=res,supU=obs,
	    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,
	    simUtlast=simcox$last,Utlast=Utlast,pval.last=pval.last,
	    res.last=res.last, type="modelmatrix")
class(out) <- "gof.phreg"

return(out)
}# }}}

##' Goodness-of-Fit for Cox Covariates (Linearity)
##'
##' Tests the functional form of continuous covariates in a Cox PH model to check for 
##' linearity. It computes cumulative residuals evaluated at a grid of covariate values \eqn{z}.
##' 
##' The test statistic is:
##' \deqn{ U(z, \tau) = \int_0^\tau K(z)^T d \hat M }
##' where \eqn{K(z)} is a design matrix based on indicator functions \eqn{I(Z_i \leq z_l)} 
##' for a grid of points \eqn{z_l}.
##'
##' The p-value is valid but depends on the chosen grid. As the number of break points increases, 
##' this test converges to the original Lin, Wei, and Ying test for linearity.
##'
##' @param formula Formula for the Cox regression.
##' @param data Data frame.
##' @param vars Vector of variable names to test. If NULL, automatically detects continuous 
##'   covariates with more than 2 levels.
##' @param offset Offset vector.
##' @param weights Weights vector.
##' @param breaks Number of break points for the grid (default 50).
##' @param equi Logical; if TRUE, uses equidistant breaks; if FALSE, uses quantiles.
##' @param n.sim Number of simulations (default 1000).
##' @param silent Logical; suppresses timing estimates.
##' @param ... Additional arguments passed to \code{gofM_phreg}.
##' @return An object of class \code{"gof.phreg"} with type "Zmodelmatrix" containing:
##'   \item{res}{Matrix of p-values for each tested variable.}
##'   \item{Zres}{List of \code{gof.phreg} objects, one for each variable.}
##'   \item{type}{Type of test ("Zmodelmatrix").}
##' @author Thomas Scheike and Klaus K. Holst
##' @seealso \code{\link{gofM_phreg}}, \code{\link{cumContr}}
##' @examples
##' data(TRACE)
##' set.seed(1)
##' TRACEsam <- blocksample(TRACE, idvar="id", replace=FALSE, 100)
##' 
##' ## Test linearity of continuous covariates
##' \donttest{ ## Reduce Ex.Timings
##' m1 <- gofZ_phreg(Surv(time,status==9)~strata(vf)+chf+wmi+age, data=TRACEsam)
##' summary(m1) 
##' plot(m1, type="z")
##' }
##' @aliases cumContr 
##' @export
gofZ_phreg  <- function(formula,data,vars=NULL,offset=NULL,weights=NULL,breaks=50,equi=FALSE,
			n.sim=1000,silent=1,...)
{# {{{
if (is.null(vars)) {
     vars <- NULL
      ## find strata var's 
      yxzf <- procform(formula,x=NULL,z=NULL,data=data,do.filter=FALSE)
      avars <- all.vars(formula[-2])
      svar <- grep("strata",yxzf$predictor)
    if (length(svar)>=1) {
	avars <- avars[-svar]
     }
     ## check that it is not a factor and that there are more than 2 levels
     for (vv in avars) {
	  if (length(unique(data[,vv]))>2 & !is.factor(data[,vv]))
	  vars  <-  c(vars,vv)
     }
 }

 if (length(vars)==0) stop("Checking functional form of covariates, must contain regression effects of continous covariates\n")
 gres <- list()
 res <- matrix(0,length(vars),2)
 colnames(res) <- c("Sup_z |U(tau,z)|","pval")
 rownames(res) <- vars

i <- 1
for (vv in vars) {
 modelmatrix <- cumContr(data[,vv],breaks=breaks,equi=equi)
 lres <- gofM_phreg(formula,data,modelmatrix=modelmatrix) 
 lres$xaxs <- attr(modelmatrix,"breaks")

 res[i,] <- c(lres$Utlast,lres$pval.last)
 i <- i+1

 lres <- list(lres)
 names(lres) <- vv
 gres <- c(gres,lres)
}

out <- list(res=res,Zres=gres,type="Zmodelmatrix")
class(out) <- c("gof.phreg")

return(out)
}# }}}

cumContr <- function(data,breaks=4,probs=NULL,equi=TRUE,na.rm=TRUE,unique.breaks=TRUE,...)
 {# {{{
 if (is.vector(data)) {
        if (is.list(breaks))
            breaks <- unlist(breaks)
        if (length(breaks) == 1) {
            if (!is.null(probs)) {
                breaks <- quantile(data, probs, na.rm = na.rm,...)
	        breaks <- breaks[-1]
            }
            else {
                if (!equi) {
                  probs <- seq(0, 1, length.out = breaks + 1)
                  breaks <- quantile(data, probs, na.rm = na.rm, ...)
	          if (unique.breaks) breaks <- unique(breaks)
	          breaks <- breaks[-1]
                }
                if (equi) {
                  rr <- range(data, na.rm = na.rm)
                  breaks <- seq(rr[1], rr[2], length.out = breaks + 1)
	          breaks <- breaks[-1]
                }
            }
        }
        if (sum(duplicated(breaks)) == 0) {
		i <- 0; 
		gm <- matrix(0,length(data),length(breaks))
            for (bb in breaks)  {
		    i <- i+1 
		    gm[,i] <- (data <= bb)*1
	    }
	} else {
            wd <- which(duplicated(breaks))
            mb <- min(diff(breaks[-wd]))
            breaks[wd] <- breaks[wd] + (mb/2) * seq(length(wd))/length(wd)
            i <- 0; gm <- matrix(0,length(data),length(breaks))
            for (bb in breaks)  {
		    i <- i+1 
		    gm[,i] <- (data <= bb)*1
	    }
            warning(paste("breaks duplicated"))
        }
        colnames(gm) <- paste("<=",breaks,sep="")
	attr(gm,"breaks") <- breaks
        return(gm)
    }
 }# }}}

##' @export
plot.gof.phreg <-  function(x,col=3,type=NULL,...)
{# {{{

if (is.null(type)) {
	if (x$type=="prop") type <- "time"
	if (x$type=="modelmatrix" ) type <- "modelmatrix"
	if (x$type=="Zmodelmatrix") type <- "z"
}

if (type=="time" || type=="modelmatrix") {
p <- ncol(x$score)
for (i in 1:p)
{
simU <- x$simUt[,(0:49)*p+i]
rsU <- max(abs(simU))
rsU <- max(rsU,abs(x$score[,i]))
plot(x$jumptimes,x$score[,i],type="s",ylim=c(-rsU,rsU),xlab="",ylab="")
title(main=rownames(x$res)[i])
matlines(x$jumptimes,simU,type="s",lwd=0.3,col=col)
lines(x$jumptimes,x$score[,i],type="s",lwd=1.5)
}
} else {
	if (type=="modelmatrix") {
		obsz <- c(tail(x$score,1))
		times <- 1:length(obsz)
		rsU <- max(max(abs(obsz)),max(abs(x$simUtlast[1:50,])))
		plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
		matlines(times,t(x$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
	        ## redraw with thick to make observed clear 
	        lines(times,obsz,lwd=2,col=1)
	} else {
	   for (i in 1:length(x$Zres))
	   {
	    xr <- x$Zres[[i]]
	    obsz <- c(tail(xr$score,1))
	    times <- xr$xaxs
	    rsU <- max(max(abs(obsz)),max(abs(xr$simUtlast[1:50,])))
	    plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
	    title(main=rownames(x$res)[i])
	    matlines(times,t(xr$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
	    ## redraw with thick to make observed clear 
	    lines(times,obsz,lwd=2,col=1)
           }
	}
}

}# }}}

##' @export
summary.gof.phreg <-  function(object,...)
{# {{{
if (object$type=="prop")
     cat("Cumulative score process test for Proportionality:\n")
else cat("Cumulative residuals versus modelmatrix :\n")
print(object$res)

if (!is.null(object$res.last)) {
   cat("\n")
   cat("Cumulative score process versus covariates (discrete z via model.matrix):\n")
   print(object$res.last)
}

} # }}}

##' @export
print.gof.phreg <-  function(x,...)
{# {{{
if (x$type=="prop")
     cat("Cumulative score process test for Proportionality:\n")
else cat("Cumulative residuals versus modelmatrix :\n")
print(x$res)

if (!is.null(x$res.last)) {
   cat("\n")
   cat("Cumulative score process versus covariates (discrete z via model.matrix):\n")
   print(x$res.last)
}

} # }}}

##' Transform that allows condition 
##'
##' Defines new variables under condition for data frame 
##' @param data is data frame 
##' @param ... new variable definitions including possible if condition
##' @examples
##' data(mena)
##'
##' xx <- dtransform(mena,ll=log(agemena)+twinnum)
##'
##' xx <- dtransform(mena,ll=log(agemena)+twinnum,agemena<15)
##' xx <- dtransform(xx  ,ll=100+agemena,ll2=1000,agemena>15)
##' dsummary(xx,ll+ll2~I(agemena>15))
##' @aliases dtransform dtransform<- dtrans dtrans<- 
##' @export
dtransform <- function(data,...)
{# {{{
    if (is.vector(data)) data <- data.frame(data)

###    if (is.list(...)) e <- eval(substitute(...), data, parent.frame()) else 
###    if (!missing(EXPRLIST)) {
###	   e <- eval(substitute(c(list(...),EXPRLIST)), data, parent.frame())
###    } else
    e <- eval(substitute(list(...)), data, parent.frame())

    tags <- names(e)
    condn  <- match("",tags) 

    if (!is.na(condn)) {
	    condition <- TRUE
	    cond <- e[[condn[1]]]; 
            whereT <- which(cond) 
	    e[[condn]] <- NULL
	    tags <- tags[-condn]
    } else condition <- FALSE
    inx <- match(tags, names(data))
    matched <- !is.na(inx)
    matchedtags <- seq(length(e))[matched]

    if (any(matched)) {
      ### new values replaces old values 
      k <- 1
      if (condition==TRUE) {
	   for (i in inx[matched]) 
	   {
            mk <- matchedtags[k]
	    if (length(e[[mk]])==1) data[whereT,i] <- e[[mk]] else 
		                    data[whereT,i] <- e[[mk]][whereT]
	   k <- k+1
	   }
      } else data[inx[matched]] <- e[matched]
      data <- data.frame(data)
    }

    ### no matched, all new variables 
    if (!all(matched)) {
	    if (condition) 
	    for (i in seq(length(e))[!matched])  {
              if (length(e[[i]])==1) e[[i]] <- rep(e[[i]],nrow(data)) 
	      e[[i]][!cond] <- NA
	    } 
	    data <- cbind(data,data.frame(e[!matched]))
    } 

return(data)
}# }}}


##' @export
dtrans <- function(data,...) dtransform(data,...)

##' @export
`dtrans<-` <- function(data,...,value) {
    dtransform(data,...) <- value
    return(data)
}

##' @export
`dtransform<-` <- function(data,...,value) {
    cl <- match.call()
    cl[[1L]] <- substitute(dtransform)
    a <- substitute(value)
    if (inherits(value,"function")) {
        cl["value"] <- NULL
        names(cl)[names(cl)=="INPUT"] <- ""
        cl[["INPUT"]] <- value
    } else {
        if (is.list(value)) {
            cl[which(names(cl)=="value")] <- NULL
            start <- length(cl)
            for (i in seq_along(value)) {
                cl[start+i] <- value[i]
            }
            if (length(names(value))>0)
                names(cl)[start+seq_along(value)] <- names(value)
        } else {
            names(cl)[which(names(cl)=="value")] <- ""
        }
    }
    eval.parent(cl)

}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)

## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z
 
out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~X)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X,cens.model=~+1)

## -----------------------------------------------------------------------------
###library(speff2trial) 
library(mets)
data(ACTG175)
###
data <- ACTG175[ACTG175$arms==0 | ACTG175$arms==1, ]
data <- na.omit(data[,c("days","cens","arms","strat","cd40","cd80","age")])
data$days <- data$days+runif(nrow(data))*0.01
dfactor(data) <- arms.f~arms
notrun <- 1

if (notrun==0) { 
fit1 <- speffSurv(Surv(days,cens)~cd40+cd80+age,data=data,trt.id="arms",fixed=TRUE)
summary(fit1)
}
# 
# Treatment effect
#             Log HR       SE   LowerCI   UpperCI           p
# Prop Haz  -0.70375  0.12352  -0.94584  -0.46165  1.2162e-08
# Speff     -0.72430  0.12051  -0.96050  -0.48810  1.8533e-09

out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~cd40+cd80+age,augmentC=~cd40+cd80+age)
summary(out)

## -----------------------------------------------------------------------------
dtable(data,~strat+arms)
dfactor(data) <- strat.f~strat
out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~strat.f)
summary(out)

## -----------------------------------------------------------------------------
data(calgb8923)
calgt <- calgb8923

## tm <- At.f~factor(Count2)+age+sex+wbc
## tm <- At.f~factor(Count2)
tm <-  At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=1:10)
summary(pll0,times=1:10)

## -----------------------------------------------------------------------------
library(mets)
data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1

## making time-dependent indicators of going to B1/B2
calgt$A10t <- calgt$A11t <- 0
calgt <- dtransform(calgt,A10t=1,A1==0 & Count2==1)
calgt <- dtransform(calgt,A11t=1,A1==1 & Count2==1)
calgt0 <- subset(calgt,A0==0)

ss0 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==0),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss0)

ss1 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==1),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss1)

## -----------------------------------------------------------------------------
ssf <- phreg_rct(Event(start,time,status)~A0.f+A10t+A11t+cluster(id),
	 data=calgt,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR0=~age+wbc+sex,augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)

## -----------------------------------------------------------------------------
n <- 1000
beta <- 0.15; 
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- scalecumhaz(CPH_HPN_CRBSI$mechanical,0.5)

cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
ce <- 3; betao1 <- 0

varz <- 1; dep=4; X <- z <- rgamma(n,1/varz)*varz
Z0 <- NULL
px <- 0.5
if (betao1!=0) px <- lava::expit(betao1*X)      
A0 <- rbinom(n,1,px)
r1 <- exp(A0*beta[1])
rd <- exp( A0 * 0.15)
rc <- exp( A0 * 0 )
###
rr <-    mets:::simLUCox(n,base1,death.cumhaz=dr,r1=r1,Z0=X,dependence=dep,var.z=varz,cens=ce/5000)
rr$A0 <- A0[rr$id]
rr$z1 <- attr(rr,"z")[rr$id]
rr$lz1 <- log(rr$z1)
rr$X <- rr$lz1 
rr$lX <- rr$z1
rr$statusD <- rr$status
rr <- dtransform(rr,statusD=2,death==1)
rr <- count_history(rr)
rr$Z <- rr$A0
data <- rr
data$Z.f <- as.factor(data$Z)
data$treattime <- 0
data <- dtransform(data,treattime=1,lbnr__id==1)
dlist(data,start+stop+statusD+A0+z1+treattime+Count1~id|id %in% c(4,5))

## -----------------------------------------------------------------------------
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 treat.var="treattime",typesR=c("non","R0"),typesC=c("non","C","dynC"),
	 augmentR0=~z1,augmentC=~z1+Count1)
summary(fit2)

## -----------------------------------------------------------------------------
n <- 500
beta=c(0.3,0.3);betatr=0.3;betac=0;betao=0;betao1=0;ce=3;fixed=1;sim=1;dep=4;varz=1;ztr=0; ce <- 3
## take possible frailty 
Z0 <- rgamma(n,1/varz)*varz
px0 <- 0.5; if (betao!=0) px0 <- expit(betao*Z0)
A0 <- rbinom(n,1,px0)
r1 <- exp(A0*beta[1])
#
px1 <- 0.5; if (betao1!=0) px1 <- expit(betao1*Z0)
A1 <- rbinom(n,1,px1)
r2 <- exp(A1*beta[2])
rtr <- exp(A0*betatr[1])
rr <-  mets:::simLUCox(n,base1,death.cumhaz=dr,cumhaz2=base1,rtr=rtr,betatr=0.3,A0=A0,Z0=Z0,
		r1=r1,r2=r2,dependence=dep,var.z=varz,cens=ce/5000,ztr=ztr)
rr$z1 <- attr(rr,"z")[rr$id]
rr$A1 <- A1[rr$id]
rr$A0 <- A0[rr$id]
rr$lz1 <- log(rr$z1)
rr <- count_history(rr,types=1:2)
rr$A1t <- 0
rr <- dtransform(rr,A1t=A1,Count2==1) 
rr$At.f <- rr$A0
rr$A0.f <- factor(rr$A0)
rr$A1.f <- factor(rr$A1)
rr <- dtransform(rr, At.f = A1, Count2 == 1)
rr$At.f <- factor(rr$At.f)
dfactor(rr)  <-  A0.f~A0
rr$treattime <- 0
rr <- dtransform(rr,treattime=1,lbnr__id==1)
rr$lagCount2 <- dlag(rr$Count2)
rr <- dtransform(rr,treattime=1,Count2==1 & (Count2!=lagCount2))
dlist(rr,start+stop+statusD+A0+A1+A1t+At.f+Count2+z1+treattime+Count1~id|id %in% c(5,10))

## -----------------------------------------------------------------------------
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),treat.var="treattime",
	 treat.model=At.f~factor(Count2),
	 augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)

## -----------------------------------------------------------------------------
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 typesR=c("non","R0"),typesC=c("non","C","dynC"),
         RCT=FALSE,treat.model=Z.f~z1,augmentR0=~z1,augmentC=~z1+Count1,
	 treat.var="treattime")
summary(fit2)

## -----------------------------------------------------------------------------
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
         treat.var="treattime",
	 RCT=FALSE,treat.model=At.f~z1*factor(Count2),
         augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)

## -----------------------------------------------------------------------------
sessionInfo()

##' Discrete Time-to-Event Haplotype Analysis
##'
##' Performs cycle-specific logistic regression to estimate haplotype effects on discrete 
##' time-to-event data, accounting for phase ambiguity. Given observed genotypes \eqn{G} 
##' and unobserved haplotypes \eqn{H}, the method integrates (mixes out) over the possible 
##' haplotype configurations using the conditional probabilities \eqn{P(H|G)}.
##' 
##' The survival function is computed by averaging over the possible haplotypes:
##' \deqn{ S(t|x,G) = E[ S(t|x,H) | G ] = \sum_{h \in G} P(h|G) S(t|x,h) }
##' 
##' The discrete hazard function is modeled using logistic regression:
##' \deqn{ \text{logit}(P(T=t | T \geq t, x, h)) = \alpha_t + x(h) \beta }
##' where \eqn{\alpha_t} are time-specific intercepts (baseline hazards), \eqn{x(h)} is the 
##' regression design constructed from covariates and haplotypes \eqn{h=(h_1, h_2)}, and 
##' \eqn{\beta} are the regression coefficients.
##' 
##' The likelihood is maximized numerically. Standard errors are computed assuming that 
##' \eqn{P(H|G)} is known (i.e., ignoring the uncertainty in haplotype estimation).
##' 
##' The design matrix over possible haplotypes is constructed by merging the covariate 
##' data \eqn{X} with the haplotype probabilities \eqn{Haplos} and applying a user-defined 
##' \code{designfunc}.
##'
##' @param X Design matrix data frame (must be sorted by \code{id} and \code{time}) containing 
##'   the ID, time variable, binary response, and covariates.
##' @param y Name of the response variable (binary, 0/1) in \code{X}.
##' @param time.name Name of the time variable in \code{X} used for sorting and cycle definition.
##' @param Haplos Data frame containing \code{id}, \code{haplo1}, \code{haplo2} (haplotypes as factors), 
##'   and \code{p} (probability \eqn{P(H|G)}).
##' @param id Name of the ID variable in \code{X} and \code{Haplos}.
##' @param desnames Names of the covariate columns in \code{X} to be used in the design matrix.
##' @param designfunc Function that computes the design vector given haplotypes \eqn{h=(h_1, h_2)} 
##'   and covariates. Must return a vector or matrix compatible with the model.
##' @param beta Starting values for the optimization (vector of length \eqn{p + k}, where \eqn{p} 
##'   is the number of covariate effects and \eqn{k} is the number of time cycles).
##' @param no.opt Logical; if TRUE, skips optimization and returns estimates based on the 
##'   provided \code{beta} (useful for initialization or diagnostics).
##' @param method Optimization method: \code{"NR"} (Newton-Raphson, default) or \code{"nlm"}.
##' @param stderr Logical; if FALSE, returns only the coefficient estimates.
##' @param designMatrix Alternative to \code{X} and \code{Haplos}: provides the response and 
##'   design matrix directly (not fully implemented).
##' @param response Alternative to \code{X}: provides the response and design directly (not fully implemented).
##' @param idhap Name of the ID-haplotype variable to specify different haplotypes for different IDs.
##' @param design.only Logical; if TRUE, returns only the design matrices constructed for the analysis.
##' @param covnames Names of covariates to extract from the object for regression output.
##' @param fam Family of the model (default \code{binomial}, currently the only option).
##' @param weights Weights following ID for the GLM component.
##' @param offsets Offsets following ID for the GLM component.
##' @param idhapweights Weights following ID-haplotype for the GLM component (Work in Progress).
##' @param ... Additional arguments passed to the optimizer (\code{lava::NR} or \code{nlm}).
##' @return An object of class \code{"haplosurvd"} containing:
##'   \item{coef}{Estimated coefficients (baseline time effects and haplotype/covariate effects).}
##'   \item{se}{Standard errors of the coefficients.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{se.robust}{Robust standard errors (if available).}
##'   \item{iid}{Influence function (IID) decomposition.}
##'   \item{ploglik}{Log-likelihood at convergence.}
##'   \item{gradient, hessian}{Optimization results.}
##'   \item{Xhap, X, Haplos}{Data and design matrices used.}
##'   \item{nid, nidhap}{Number of IDs and ID-haplotype combinations.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. H. (2024). Discrete time survival analysis with haplotype effects. mets package documentation.
##' @examples
##' ## Some haplotypes of interest
##' types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")
##' 
##' ## Some haplotype frequencies for simulations 
##' data(haplo)
##' hapfreqs <- haplo$hapfreqs 
##'
##' www <- which(hapfreqs$haplotype %in% types)
##' hapfreqs$freq[www]
##'
##' baseline <- hapfreqs$haplotype[9]
##' baseline
##'
##' ## Design function: indicator for presence of any 'types' haplotype
##' designftypes <- function(x, sm=0) {
##'   hap1 <- x[1]
##'   hap2 <- x[2]
##'   if (sm == 0) y <- 1 * ((hap1 == types) | (hap2 == types))
##'   if (sm == 1) y <- 1 * (hap1 == types) + 1 * (hap2 == types)
##'   return(y)
##' }
##'
##' tcoef <- c(-1.93110204, -0.47531630, -0.04118204, -1.57872602, -0.22176426, -0.13836416,
##'            0.88830288, 0.60756224, 0.39802821, 0.32706859)
##' 
##' ghaplos <- haplo$ghaplos
##' haploX  <- haplo$haploX
##' 
##' haploX$time <- haploX$times
##' Xdes <- model.matrix(~ factor(time), haploX)
##' colnames(Xdes) <- paste("X", 1:ncol(Xdes), sep="")
##' X <- dkeep(haploX, ~ id + y + time)
##' X <- cbind(X, Xdes)
##' Haplos <- dkeep(ghaplos, ~ id + "haplo*" + p)
##' desnames <- paste("X", 1:6, sep="")   # Six X's related to 6 cycles 
##' out <- haplo_surv_discrete(X=X, y="y", time.name="time",
##'          Haplos=Haplos, desnames=desnames, designfunc=designftypes) 
##' names(out$coef) <- c(desnames, types)
##' out$coef
##' summary(out)
##' @export
haplo_surv_discrete <- function (X=NULL,y="y",time.name="time",Haplos=NULL,id="id",desnames=NULL,designfunc=NULL,
    beta=NULL,no.opt=FALSE,method="NR",stderr=TRUE,designMatrix=NULL,response=NULL,idhap=NULL,design.only=FALSE,
    covnames=NULL,fam=binomial,weights=NULL,offsets=NULL,idhapweights=NULL,...)
{ ## {{{ 
  cond=NULL

if (is.null(designMatrix)) { 
if (!is.null(Haplos)) { ## with haplo-types {{{
  ## X: y, Xdes, id
  ## Haplos, haplo1, haplo2, id, p (HgivenG)
  bothid <- intersect(X$id,Haplos$id)
  X <- subset(X,id %in% bothid)
  Haplos <- subset(Haplos,id %in% bothid)
  ## new iid starts at 1 
  Haplos$id <- fast.approx(bothid,Haplos$id) 
  iiid <- Haplos$id-1
  X$id <- fast.approx(bothid,X$id) 
  Xo <- X

  Xhap <- merge(X,Haplos,by.x="id",by.y="id")
  Xhap <- dsort2(Xhap,~id+"haplo*"+time)
  Haplos <- dsort2(Haplos,~id+"haplo*")

  time <- Xhap[,time.name]
  tn <- match(time.name,names(Xhap))
  Xhap <- Xhap[,-tn]

  response <- Xhap[,y]
  yn <- match(y,names(Xhap))
  Xhap <- Xhap[,-yn]

  mm <- grep("^haplo", names(Xhap))
  Xhaps <- Xhap[,mm]
  if (!is.null(designfunc)) {
	  Xo <- X <- as.matrix(apply(Xhap[,mm],1,designfunc))
	  if (ncol(X)==nrow(Xhap))  X <- t(X)
	  colnames(X) <- paste("haplo",1:ncol(X),sep="")
	  X <- as.matrix(cbind(Xhap[,desnames],X))
  } else Xo <- X <- as.matrix(Xhap[,desnames])

  ## X(i)^T %*% X(i) for each row 
  X2  <- .Call("vecMatMat",X,X)$vXZ

## creates sub-index for haplo-types within each id
   nmm <- names(Xhap)[mm]
   ms <- mystrata(Xhap[,c("id",nmm)])
	   stratidhap <- ms
   nidhap <- attr(ms,"nlevel")
   nid <- length(unique(Haplos$id))

  ## weights will follow id 
  if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
  ## idhap weights will follow id-hap
  if (is.null(idhapweights))  whapiid <- rep(1,nidhap)  else whapiid <- idhapweights
  ## offsets can follow both Haplo or X design and will then appear in mixed design
  if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

  ### to make the optimizer more flexible and use for interval censored data 
  if (is.null(cond)) cond <- rep(0,nidhap)

   hgiveng <- Haplos$p
# }}}
} else {  ## standard glm {{{ 
  ## X: y, Xdes, id

  id.name <- id
  ## id going from 1 to #id's
  id <- X$id <- fast.approx(unique(X[,id]),X[,id]) 
  Xhap <- Xo <- X
  iiid <- unique(X$id)-1
  nid <- length(iiid)
  stratidhap <- id 
  nidhap <- length(unique(stratidhap))

  ###
  response <- Xhap[,y]
  yn <- match(y,names(Xhap))
  Xhap <- Xhap[,-yn]

  X <- as.matrix(Xhap[,desnames])
  ## X(i)^T %*% X(i) for each row 
  X2  <- .Call("vecMatMat",X,X)$vXZ

  ## weights will follow id 
  if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
  ## idhap weights will follow id-hap
  if (is.null(idhapweights))  wph <- rep(1,nidhap)  else wph <- idhapweights
  ## offsets can follow both Haplo or X design and will then appear in mixed design
  if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

  if (is.null(cond)) cond <- rep(0,nidhap)
  hgiveng <- rep(1,nid)

}# }}}
} else {# {{{
	hgiveng <- designMatrix$p
	X <- designMatrix[,desnames]
	id <- designMatrix[,id]
        iiid <- unique(id)-1

	if (is.null(idhap)) stop("'idhap' must be supplied when 'designMatrix' is provided.")
        stratidhap <- designMatrix[,idhap]
        nidhap <- length(unique(stratidhap))
        nid <- length(iiid)
	design.only <- FALSE

        ## weights will follow id 
        if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
        ## idhap weights will follow id-hap
        if (is.null(idhapweights))  wph <- rep(1,nidhap)  else wph <- idhapweights
        ## offsets can follow both Haplo or X design and will then appear in mixed design
        if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

        if (is.null(cond)) cond <- rep(0,nidhap)

}# }}}

if (!design.only) {

	if (is.null(beta)) beta <- rep(0,ncol(X))

	obj <- function(pp,all=FALSE)
	{ # {{{

	lp <- X %*% pp 
	## plp <- family$linkinv(lp)
	plp <- expit(lp+ offiid)
	nplp <- 1-plp

	logpht <- log(plp)*response+log(nplp)*(1-response)
	pht <-c(exp(logpht))
	Dlogpht <-  X* c(response-plp)
	D2logpht <- c(plp/(1+exp(lp)))*X2

	ph <- c(exp(sumstrata(logpht,stratidhap-1,nidhap)))
	pg <- c(sumstrata(ph*hgiveng,iiid,nid))
	logl <- wiid*log(pg)

	## Derivative 
	Dlogph  <- apply(Dlogpht,2,sumstrata,stratidhap-1,nidhap)
	Dph  <- c(ph)*Dlogph
	Dpg <- apply(Dph*hgiveng,2,sumstrata,iiid,nid)# {{{}}}
	Dlogl    <- wiid*Dpg/pg
	DpgDpg  <- .Call("vecMatMat",Dpg,Dpg)$vXZ

	## 2nd Derivative 
	D2logph <- apply(D2logpht,2,sumstrata,stratidhap-1,nidhap)
	DphDlogph <- .Call("vecMatMat",Dph,Dlogph)$vXZ
	D2ph <- ph*D2logph+DphDlogph
	D2pg <-apply(D2ph*hgiveng,2,sumstrata,iiid,nid)
	D2logi <- wiid*(pg*D2pg-DpgDpg)/pg^2
	D2log <- apply(D2logi,2,sum)
	D2log <- matrix(D2log,ncol(X),ncol(X))

	ploglik <- sum(logl)
	gradient <- apply(Dlogl,2,sum)
	hessian <- D2log

	  if (all) {
	      ihess <- pinv(hessian)
	      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
	      robvar <- crossprod(beta.iid)
	      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
			  iid=beta.iid,robvar=robvar,var=ihess,
			  id=iiid,
			  se=diag(ihess)^.5,se.robust=diag(robvar)^.5)
	      return(val)
	  }  
	 structure(-ploglik,gradient=-gradient,hessian=hessian)
	}# }}}

	  p <- ncol(X)
	  opt <- NULL
	  if (p>0) {
	  if (no.opt==FALSE) {
	      if (tolower(method)=="nr") {
		  opt <- lava::NR(beta,obj,...)
		  opt$estimate <- opt$par
	      } else {
		  opt <- nlm(obj,beta,...)
		  opt$method <- "nlm"
	      }
	      cc <- opt$estimate; 
	      if (!stderr) return(cc)
	      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
	      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
	  } else {
	      val <- obj(0,all=TRUE)
	  }


}  else val <- NULL

  val <- c(list(Xhap=Xhap,X=X,Haplos=Haplos),val)
  class(val) <- "haplosurvd"
  return(val)
} ## }}} 

## uses HaploSurvival package of github install via devtools
## devtools::install_github("scheike/HaploSurvival")
## this is only used for simulations 
## out <- simHaplo(1,100,tcoef,hapfreqs)

##' @export
summary.haplosurvd <- function(object,...) { ## {{{ 
out <- lava::estimate(coef=object$coef,vcov=object$var,...)
return(out)
} ## }}} 

##' @export
print.haplosurvd <- function(x,...) summary(x,...)

##' @export
vcov.haplosurvd <- function(object,...) return(object$var) 

##' @export
coef.haplosurvd <- function(object,...) return(object$coef)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets)
set.seed(100)

data(bmt); 
bmt$time <- bmt$time+runif(nrow(bmt))*0.001
dfactor(bmt) <- tcell~tcell
bmt$event <- (bmt$cause!=0)*1

fg1 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=1)
Gfg1 <- survivalG(fg1,bmt,time=50)
summary(Gfg1)

fg2 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=2)
summary(survivalG(fg2,bmt,time=50))

cif1time <- survivalGtime(fg1,bmt)
plot(cif1time,type="risk"); 


## -----------------------------------------------------------------------------
ss <- phreg(Surv(time,event)~tcell+platelet+age,bmt)
sss <- survivalG(ss,bmt,time=50)
summary(sss)

Gtime <- survivalGtime(ss,bmt)
plot(Gtime)

## -----------------------------------------------------------------------------

## survival situation
sr1 <- binregATE(Event(time,event)~tcell+platelet+age,bmt,cause=1,
		 time=40, treat.model=tcell~platelet+age)
summary(sr1)

## relative risk effect 
estimate(coef=sr1$riskDR,vcov=sr1$var.riskDR,f=function(p) p[2]/p[1],null=1)

## competing risks 
br1 <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,cause=1,
		 time=40,treat.model=tcell~platelet+age)
summary(br1)

## -----------------------------------------------------------------------------
br1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=40)
Gbr1 <- binregG(br1,bmt,Avalues=NULL)
summary(Gbr1)

## contrasting average age to 1+2-sd age, Avalues
Gbr2 <- binregG(br1,bmt,varname="age",Avalues=c(0,1,2))
summary(Gbr2)

## -----------------------------------------------------------------------------
sessionInfo()

pred_weibull <- function(object, X, Z,
                         times,
                         individual.times = FALSE,
                         type = c("surv", "haz", "chaz", "lp"), ...) {
    p <- coef(object)
    V <- vcov(object)
    X <- rbind(X)
    Z <- rbind(Z)
    p1 <- X %*% p[seq_len(ncol(X))] # log-scale
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)] # log-shape
    if (type[1] == "lp") {
      return(cbind(lograte = p1, logshape = p2))
    }
    if (nrow(X) != nrow(Z)) stop("incompatible X and Z matrix")
    n <- nrow(X)
    np <- ncol(X) + ncol(Z)
    xz <- matrix(0, ncol = np, nrow = 2)
    var_arr <- array(dim = c(2, 2, n))
    for (i in seq_len(n)) {
      xz[1, seq_len(ncol(X))] <- X[i, ]
      xz[2, seq_len(ncol(Z)) + ncol(X)] <- Z[i, ]
      var_arr[, , i] <- xz %*% V %*% t(xz)
    }
    par <- cbind(p1, p2)
    nt <- length(times)
    if (individual.times && nt != n) {
        stop("For individual time predictions 'times', 'X' and 'Z' should agree") # nolint
    }
    ## time.fun optional smooth function specifying transformation of
    ##   time-variable. Here the cumulative hazard is assumed to be
    ##   \eqn{H(t)=\lambda g(t)^s}
    ## if (is.null(time.fun)) {
    ##   time.fun <- structure(identity, grad = identity)
    ## }
    ## chaz <- function(par, ...) exp(par[1]) * (time.fun(newtime)**exp(par[2]))
    chaz <- function(par, ...) exp(par[1]) * (newtime**exp(par[2]))
    surv <- function(par, ...) exp(-chaz(par, ...))
    haz <- function(par) {
      res <- exp(par[1] + par[2] +
          (exp(par[2]) - 1) * log(newtime))
                 ## (exp(par[2]) - 1) * log(time.fun(newtime))
                 ## + log(attr(time.fun, "grad")(newtime)))
      res[newtime == 0] <- 0
      return(res)
    }
    if (individual.times) {
      est_arr <- array(dim = c(nt, 4))
    } else {
      est_arr <- array(dim = c(nt, 4, n))
      newtime <- times
    }
    for (i in seq_len(n)) {
      if (individual.times) {
        newtime <- times[i]
      }
      pr <- lava::parameter(summary(
        estimate(
          coef = par[i, ],
          vcov = var_arr[, , i], f = get(type[1])
        ), ...))
      if (individual.times) {
        est_arr[i, ] <- pr[1:4]
      } else {
        est_arr[, , i] <- pr[, 1:4]
      }
    }
    if (individual.times) {
      dimnames(est_arr) <- list(seq_len(nt), colnames(pr)[1:4])
    } else {
      dimnames(est_arr) <- list(
        paste0("t", seq_len(nt)),
        colnames(pr)[1:4], seq_len(n)
      )
    }
  return(est_arr)
}

logl_weibull <- function(p, entry, exit, status,
                         X = NULL, Z = NULL,
                         time.fun = NULL) {
    if (is.null(X)) X <- cbind(rep(1, length(exit)))
    if (is.null(Z)) Z <- cbind(rep(1, length(exit)))
    dt <- 1
    if (!is.null(time.fun)) {
      if (is.null(attr(time.fun, "grad"))) {
          # complex-step derivative
          dt <- numDeriv::grad(time.fun, exit, method = "complex")
      } else {
          # numerical derivative
          dt <- attr(time.fun, "grad")(exit)
      }
      exit <- time.fun(exit)
      entry <- time.fun(entry)
    }
    if (length(p) != ncol(X) + ncol(Z)) stop("wrong parameter length")
    p1 <- X %*% p[seq_len(ncol(X))] # log-scale
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)] # log-shape
    theta <- cbind(exp(p1), exp(p2))
    loghaz <- log(theta[, 1]) + log(theta[, 2]) +
      (theta[, 2] - 1) * log(exit) + log(dt)
    chaz <- function(t) theta[, 1] * t**theta[, 2]
    logl <- status * loghaz - chaz(exit) + chaz(entry)
    logl
}

score_weibull <- function(p, entry, exit, status,
                          X = NULL, Z = NULL,
                          time.fun = NULL) {
    if (is.null(X)) X <- cbind(rep(1, length(exit)))
    if (is.null(Z)) Z <- cbind(rep(1, length(exit)))
    if (!is.null(time.fun)) {
        entry = time.fun(entry)
        exit = time.fun(exit)
    }
    if (length(p) != ncol(X) + ncol(Z)) stop("wrong parameter length")
    p1 <- X %*% p[seq_len(ncol(X))]
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)]
    theta <- exp(cbind(p1, p2))
    dloghaz.dtheta1 <- status / theta[, 1]
    dchaz.dtheta1 <- function(t) {
        idx0 <- which(t == 0)
        res <- t**theta[, 2]
        res[idx0] <- 0 # conv. log(x)*x =0 when x=0
        return(res)
    }
    dloghaz.dtheta2 <- status / theta[, 2] + status * log(exit)
    dchaz.dtheta2 <- function(t) {
        idx0 <- which(t == 0)
        res <- theta[, 1] * log(t) * t**theta[, 2]
        res[idx0] <- 0 # conv. log(x)*x =0 when x=0
        return(res)
    }
    res <- cbind(
        dloghaz.dtheta1 - dchaz.dtheta1(exit) + dchaz.dtheta1(entry),
        dloghaz.dtheta2 - dchaz.dtheta2(exit) + dchaz.dtheta2(entry)
    )
    for (i in seq_len(ncol(X))) X[, i] <- X[, i] * res[, 1] * theta[, 1]
    for (i in seq_len(ncol(Z))) Z[, i] <- Z[, i] * res[, 2] * theta[, 2]
    return(cbind(X, Z))
}

##' @description Fits a Cox-Weibull with cumulative hazard given by \deqn{
##'   \Lambda(t) = \lambda \cdot t^s } where \eqn{s} is the shape parameter, and
##'   \eqn{\lambda} the rate parameter. We here allow a regression model for
##'   both parameters \deqn{\lambda := \exp(\beta^\top X)} \deqn{s :=
##'   \exp(\gamma^\top Z)} as defined by `formula` and `shape.formula`
##'   respectively.
##' @details The parametrization
##' @title Weibull-Cox regression
##' @param formula Formula for proportional hazards. The right-handside must be
##'   an [Event] or [Surv] object (with right-censoring and possibly delayed
##'   entry).
##' @param shape.formula Formula for shape parameter
##' @param data data.frame
##' @param save.data if TRUE the data.frame is stored in the model object (for
##'   predictions and simulations)
##' @param control control arguments to optimization routine [stats::nlmbin]
##' @seealso [mets::phreg()]
##' @aliases phreg_weibull phreg.par
##' @author Klaus Kähler Holst, Thomas Scheike
##' @examples
##' data(sTRACE, package="mets")
##' sTRACE$entry <- 0
##' fit1 <- phreg_weibull(Event(entry, time, status == 9) ~ age,
##'              shape.formula = ~age, data = sTRACE)
##' tt <- seq(0,10, length.out=100)
##' pr1 <- predict(fit1, newdata = sTRACE[1, ], times = tt)
##' fit2 <- phreg(Event(time, status == 9) ~ age, data = sTRACE)
##' pr2 <- predict(fit2, newdata = sTRACE[1, ], se = FALSE)
##' if (interactive()) {
##'    plot(pr2$times, pr2$surv, type="s")
##'    lines(tt, pr1[,1,1], col="red", lwd=2)
##' }
##' @export
##' @return `phreg.par` object
phreg_weibull <- function(formula,
                          shape.formula = ~1,
                          data,
                          save.data = TRUE,
                          control = list()) {
    cl <- match.call()
    des <- proc_design(
      formula,
      data = data,
      specials = c("offset", "weights", "cluster"),
      intercept = TRUE
    )
    des2 <- proc_design(shape.formula, data = data, intercept = TRUE)
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    Z <- des2$x
    obj <- function(p) {
        -.logl_weibull(p, entry, exit, status,
            X = X, Z = Z)
    }
    grad <- function(p, indiv = FALSE) {
      -.score_weibull(p, entry, exit, status,
                      X = X, Z = Z, indiv = indiv)
    }
    if (!is.null(control$start)) {
        p0 <- control$start
        control$init <- NULL
    } else {
        p0 <- rep(0, ncol(X) + ncol(Z))
    }
    op <- stats::nlminb(p0, obj, grad, control = control)
    p <- op$par
    H <- numDeriv::jacobian(grad, p)
    U <- -grad(op$par, indiv=TRUE)
    n <- nrow(U)
    loglik <- structure(-obj(p),
        df = length(p), nobs = n, nall = n,
        class = "logLik"
        )
    ic <- t(pinv(H) %*% t(U)) * n
    names(p) <- c(
      colnames(X),
      paste0("s:", colnames(Z))
    )
    est <- lava::estimate(coef = p, IC = ic)
    est$model.index <- list(ncol(X), ncol(X) + ncol(Z))
    res <- list(
        loglik = loglik,
        response = list(entry = entry, exit = exit, status = status),
        call = cl, opt = op, coef = p,
        hessian = H,
        rate.design = clean_design(des),
        shape.design = clean_design(des2),
        data = NULL,
        estimate = est
    )
    if (save.data) res$data <- data
    return(structure(res, class = "phreg.par"))
}

##' @export
phreg.par <- function(...) {
  warning("`phreg.par` will be deprecated in the next release of mets and replaced by `phreg_weibull`") # nolint
  res <- phreg_weibull(...)
  if (deparse(res$shape.design$terms)!="~1") {
    stop("this function only works with a single shape")
  }
  if (attr(res$rate.design$terms, "intercept") == 0L) {
    stop("model needs intercept")
  }
  np <- length(coef(res))
  tr <- function(p) c(p[1] / exp(p[np]), p[np], p[-c(1, np)])
  estimate(res, tr)
}

##' @export
print.phreg.par <- function(x, ...) {
    cat("\n- Weibull-Cox model -\n\n")
    cat("Call:\n")
    dput(x$call)
    cat("\n")
    cat(sprintf("log-Likelihood: %f", x$loglik), "\n\n")
    dur <- with(x$response, exit - entry)
    n <- length(x$response$status)
    events <- sum(x$response$status)
    nn <- rbind(c(n, events, sum(dur)))
    colnames(nn) <- c("n","events","obs.time")
    rownames(nn) <- ""
    print(nn, quote=FALSE)
    cat("\n")
    print(x$estimate, ...)
    cat("\n")
}

##' @export
vcov.phreg.par <- function(object, ...) {
    vcov(object$estimate)
}

##' @export
coef.phreg.par <- function(object, ...) {
    coef(object$estimate)
}

##' @export
simulate.phreg.par <- function(object,
                               nsim = nrow(data),
                               seed = NULL,
                               data = object$data,
                               cens.model,
                               var.names = c("time", "status"),
                               ...) {
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }
  # bootstrap covariates
  newd <- mets::dsample(size = nsim, data)
  # linear-predictors
  lp <- predict(object, newdata = newd, type = "lp")
  ## simulate event times
  time <- rweibullcox(nrow(lp),
                      rate = exp(lp[, 1]),
                      shape = exp(lp[, 2])
                      )
  if (missing(cens.model)) {
      y <- update_design(object$rate.design, data = data, response = TRUE)$y
      data$cens_ <- !y[, 2]
      data$time_ <- y[, 1]
      cens.model <- phreg_weibull(Surv(time_, cens_) ~ 1, data = data)
  }
  if (is.null(cens.model)) {
    cens <- rep(TRUE, nrow(newd))
  } else {
    cens.par <- exp(predict(cens.model, type = "lp", newdata = newd))
    cens <- rweibullcox(nsim, cens.par[, 1], cens.par[, 2])
  }
  newd[[var.names[1]]] <- pmin(time, cens)
  newd[[var.names[2]]] <- (time <= cens)
  return(newd)
}

##' @export
IC.phreg.par <- function(x, p=coef(x), ...) {
  IC(x$estimate)
}

##' @export
logLik.phreg.par <- function(object, ...) {
  object$loglik
}

##' @export
predict.phreg.par <- function(object,
                              newdata = object$data,
                              times,
                              individual.times = FALSE,
                              type = c("surv", "haz", "chaz", "lp"),
                              level = 0.05,
                              ...) {
  xy <- update_design(object$rate.design, data = newdata, response = TRUE)
  x <- xy$x
  z <- update_design(object$shape.design, data = newdata)$x
  if (type[1] != "lp" && missing(times)) {
    times <- xy$y[, 1]
  }
  pr <- pred_weibull(object,
                     X = x, Z = z,
                     times = times,
                     individual.times = individual.times,
                     level = level,
                     type = type, ...
                     )
  return(pr)
}


##' @export
##' @description Simulate observations from the model with cumulative hazard
##'   given by \deqn{\Lambda(t) = \lambda\cdot t^s} where \eqn{\lambda} is the
##'   \emph{rate parameter} and \eqn{s} is the \emph{shape parameter}.
##' @details [stats::rweibull()] uses a different parametrization with
##'   cumulative hazard given by
##' \deqn{H(t) = (t/b)^a,}
##' i.e., the shape is the same \eqn{a:=s} but the scale paramter \eqn{b}
##' is related to rate paramter \eqn{r} by
##' \deqn{r := b^{-a}}
##' @seealso [stats::rweibull()]
##' @title Simulate observations from a Weibull distribution
##' @param n (integer) number of observations
##' @param rate (numeric) rate parameter (can be a vector of size n)
##' @param shape (numeric) shape parameter (can be a vector of size n)
rweibullcox <- function(n, rate, shape) {
  # F(t) = 1-exp(-H(t)) = 1-exp(-lambda t^s). Inversion=>
  (-log(1-runif(n))/rate)^(1/shape)
}

library(tinytest)

## rate parametrization
set.seed(1)
rate <- 0.2
shape <- 3
d <- data.frame(time=rweibullcox(1e4, rate, shape), status=TRUE)
p <- phreg_weibull(Surv(time, status) ~ 1, data=d)
est <- coef(p) |> exp()
expect_true(mean((c(rate, shape) - est)^2) < 1e-2)

p0 <- survival::survreg(Surv(time, status) ~ 1, data = d)
expect_equivalent(logLik(p), logLik(p0))


## shape parameter regression
set.seed(1)
n <- 1e4
a <- rbinom(n, 1, 0.5)
rate <- 0.2
shape <- 2+a
d <- data.frame(time=rweibullcox(n, rate, shape), status=TRUE)
p <- phreg_weibull(Surv(time, status) ~ 1, ~ factor(a) - 1, data = d)
est <- coef(p) |> exp()
expect_true(mean((c(rate, 2, 3) - est)^2) < 1e-2)

p <- phreg_weibull(Surv(time, status) ~ factor(a) - 1, ~ factor(a) - 1, data = d)
p0 <- survival::survreg(Surv(time, status) ~ 1, data = subset(d, a == 0))
p1 <- survival::survreg(Surv(time, status) ~ 1, data = subset(d, a == 1))
expect_equivalent(logLik(p), logLik(p0)+logLik(p1))


## delayed entry, right-censoring
set.seed(1)
n <- 2e4
rate <- 0.2
shape <- 2
entry <- runif(n)
t0 <- rweibullcox(n, rate, shape)
cens <- rweibullcox(n, 0.5, 1)
time <- pmin(t0, cens)
status <- t0<=cens
d0 <- data.frame(time = time, status = status, entry=entry)
d <- subset(d0, time>entry)

p0 <- phreg_weibull(Surv(time, status) ~ 1, ~ 1, data = d0)
est <- coef(p0) |> exp()
expect_true(mean((c(rate, shape) - est)^2) < 1e-2)

p.wrong <- phreg_weibull(Surv(time, status) ~ 1, ~ 1, data = d)
est <- coef(p.wrong) |> exp()
expect_false(mean((c(rate, shape) - est)^2) < 1e-2)

p <- phreg_weibull(Surv(entry, time, status) ~ 1, ~ 1, data = d)
est <- coef(p) |> exp()
expect_true(mean((c(rate, shape) - est)^2) < 1e-2)

### drop.strata <- function(x) {
###    drop.specials(x, c('strata', 'strataC'))
### }

##' Remove Special Terms from a Formula
##'
##' Removes terms such as \code{strata()} or \code{cluster()} from a
##' formula/terms object.
##'
##' @param x a formula object.
##' @param components character vector of special term names to remove.
##' @param ... additional arguments passed to \code{lava::Specials}.
##' @return The updated formula with an attribute \code{"variables"} containing
##'   the extracted variable names from removed specials.
##' @export
drop.specials <- function(x, components, ...) {
    variables <- c()
    for (comp in components) {
        mm <- lava::Specials(x, comp, ...)
        vars <- unlist(lapply(mm, function(x) strsplit(x,',')))
        variables <- c(variables, list(vars))
        for (i in mm) {
            newf <- as.formula(paste0('.~.-', comp, '(',i,')'))
            x <- update(x, newf)
        }
    }
    names(variables) <- components
    return( structure(x, variables=variables) )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)

## -----------------------------------------------------------------------------
library(mets)
set.seed(1000) # to control output in simulations for p-values below.

## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- sim_recurrent(400,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))

## -----------------------------------------------------------------------------
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)

## -----------------------------------------------------------------------------
summary(out,times=c(1000,2000))

## -----------------------------------------------------------------------------
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrent_marginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

## -----------------------------------------------------------------------------
test_logrankRecurrent(out)

dd <- test_marginalMean(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
dd
summary(dd)
dd$RAUCl

## -----------------------------------------------------------------------------
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust_phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)

## -----------------------------------------------------------------------------
rr <- sim_recurrent_list(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))

## -----------------------------------------------------------------------------
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count_history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

times <- 500*(1:10)
###recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))

## -----------------------------------------------------------------------------
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)

times <- seq(500,5000,by=500)
recEFF1x <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))

out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)

## -----------------------------------------------------------------------------
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
summary(outs)$coef

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)

## -----------------------------------------------------------------------------
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
summary(outipcws)$coef

## -----------------------------------------------------------------------------
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef

## -----------------------------------------------------------------------------
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)

## -----------------------------------------------------------------------------
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])

## -----------------------------------------------------------------------------
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef

## -----------------------------------------------------------------------------
library(mets)
rho1 <- 1; rho2 <- 0.5
rate <- c(1,1)
tt <- seq(0, 6, by = 0.01)
base1 <- cbind(tt,rho1 * (1 - exp(-tt/rate[1])))
drcumhaz <- cbind(tt,rho2 * (1 - exp(-tt/rate[2])))
base13 <- cpred(base1,c(1,3))[,2]

dats <- mets:::sim_GLRA(100,base1,drcumhaz,varz=1)
datsA <- dats[[1]]
dtable(datsA,~statusD)
datsRA <- dats[[2]]
dtable(datsRA,~statusD)


## -----------------------------------------------------------------------------
   ## handling admin censoring with IPCW 
    outRR  <- recreg(Event(start,stop,statusD)~Z1+Z2+cluster(id), datsA, cause = 1, 
		       cens.code = c(7),death.code=3)
    estimate(outRR)

    ## Full Adm-censoring statusA, timeA
    out0  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsA, cause = 1, 
		       cens.code = 9, death.code=3, adm.cens.time=datsA$censorA)
    estimate(out0)

## -----------------------------------------------------------------------------
    ## std Right censoring  on combined censoring time 
    outR  <- recreg(Event(start,stop,statusD)~Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = c(0,7),death.code=3)
    estimate(outR)

    ## Combined R-IPCW + Adm-censoring status, time
    out1  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA)
    estimate(out1)

    ## Combined R-IPCW + Adm-censoring status + censoring modelling time
    out1c  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		     cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA,cens.model=~strata(Z1,Z2))
     estimate(out1c)

## -----------------------------------------------------------------------------
predR0 <- predict(outR,data.frame(Z1=0:1,Z2=0))
plot(predR0,ylim=c(0,1.5))

predR <- predict(outR,data.frame(Z1=0:1,Z2=0),se=1)
plot(predR,se=1,add=TRUE)

pred1c <- predict(out1c,data.frame(Z1=0:1,Z2=0),se=1)
plot(pred1c,se=1,add=TRUE)

## -----------------------------------------------------------------------------
km07 <- km(Event(start,stop,statusD %in% c(0,7))~strata(Z1,Z2),datsRA)
km0 <- km(Event(start,stop,statusD==0)~strata(Z1,Z2),datsRA)
plot(km07,col=1)
plot(km0,add=TRUE,col=2,lwd=2)

## -----------------------------------------------------------------------------
set.seed(100)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,cens=1/5000,type=3) 
dtable(rr,~statusD)
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
glout$gradient

## -----------------------------------------------------------------------------
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

## -----------------------------------------------------------------------------
rr <- mets:::sim_GLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
estimate(recurrent)
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)

## -----------------------------------------------------------------------------
simcoxcox <- sim_recurrent_ts(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim_recurrent_ts(recurrentGL,margsurv,n=10,data=rr)

## -----------------------------------------------------------------------------
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count_history(rr)
dtable(rr,~statusD)

oo <- prob_exceed_recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)

## -----------------------------------------------------------------------------
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))

## -----------------------------------------------------------------------------
rr <- sim_recurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count_history(rr)
dtable(rr,~death+status)

## -----------------------------------------------------------------------------
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)

## ----eval=FALSE---------------------------------------------------------------
#  data(CPH_HPN_CRBSI)
#  dr <- CPH_HPN_CRBSI$terminal
#  base1 <- CPH_HPN_CRBSI$crbsi
#  base4 <- CPH_HPN_CRBSI$mechanical
# 
#   par(mfrow=c(1,3))
#   var.z <- c(0.5,0.5,0.5)
#   # death related to  both causes in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
#   rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count_history(rr,types=1:2)
# ###  cor(attr(rr,"z"))
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  plot(coo,main ="Scenario I")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # death related to  both causes in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count_history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main ="Scenario II")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # positive dependence for N1 and N2 all related in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count_history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main="Scenario III")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # negative dependence for N1 and N2 all related in same way
#   cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count_history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main="Scenario IV")

## -----------------------------------------------------------------------------
sessionInfo()

##' Twostage Survival Model for Multivariate Survival Data
##'
##' Fits Clayton-Oakes or bivariate Plackett models for bivariate survival data
##' using marginals that are on Cox form. The dependence can be modelled via:
##' \enumerate{
##' \item Regression design on dependence parameter.
##' \item Random effects, additive gamma model.
##' }
##'
##' If clusters contain more than two subjects, a composite likelihood based on
##' pairwise bivariate models is used (for full MLE see \code{twostageMLE}).
##'
##' The two-stage model is constructed such that, given gamma distributed random effects,
##' the survival functions are assumed independent, and the marginal survival functions
##' are on Cox form:
##' \deqn{ P(T > t| x) = S(t|x)= \exp( -\exp(x^T \beta) A_0(t) ) }
##'
##' One possibility is to model the variance within clusters via a regression design,
##' specifying a regression structure for the independent gamma distributed random
##' effect for each cluster, such that the variance is given by:
##' \deqn{ \theta = h( z_j^T \alpha) }
##' where \eqn{z} is specified by \code{theta.des}, and a possible link function
##' \code{var.link=1} will use the exponential link \eqn{h(x)=\exp(x)}, and
##' \code{var.link=0} the identity link \eqn{h(x)=x}.
##'
##' The reported standard errors are based on the estimated information from the
##' likelihood assuming that the marginals are known (unlike \code{twostageMLE}
##' and for the additive gamma model below).
##'
##' Can also fit a structured additive gamma random effects model, such as the
##' ACE, ADE model for survival data. In this case, the \code{random.design}
##' specifies the random effects for each subject within a cluster. This is a
##' matrix of 1's and 0's with dimension \eqn{n \times d} (for \eqn{d} random effects).
##'
##' For a cluster with two subjects, the \code{random.design} rows are \eqn{v_1}
##' and \eqn{v_2}. Such that the random effect for subject 1 is \deqn{v_1^T
##' (Z_1,...,Z_d)}, for \eqn{d} random effects. Each random effect has an
##' associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction,
##' subject 1's random effect is Gamma distributed with mean
##' \eqn{\lambda_j/v_1^T \lambda} and variance \eqn{\lambda_j/(v_1^T
##' \lambda)^2}. Note that the random effect \eqn{v_1^T (Z_1,...,Z_d)} has mean
##' 1 and variance \eqn{1/(v_1^T \lambda)}. It is assumed that \eqn{lamtot=v_1^T
##' \lambda} is fixed within clusters as it would be for the ACE model.
##'
##' Based on these parameters, the relative contribution (the heritability, \eqn{h}) is
##' equivalent to the expected values of the random effects: \eqn{\lambda_j/v_1^T \lambda}.
##'
##' The DEFAULT parametrization (\code{var.par=1}) uses the variances of the random effects:
##' \deqn{ \theta_j  = \lambda_j/(v_1^T \lambda)^2 }
##' For alternative parametrizations, specify how the parameters relate to \eqn{\lambda_j}
##' with the argument \code{var.par=0}.
##'
##' For both types of models, the basic model assumptions are that given the
##' random effects of the clusters, the survival distributions within a cluster
##' are independent and on the form: \deqn{ P(T > t| x,z) = \exp( -Z \cdot
##' \text{Laplace}^{-1}(lamtot,lamtot,S(t|x)) ) } with the inverse Laplace of
##' the gamma distribution with mean 1 and variance \eqn{1/lamtot}.
##'
##' The parameters \eqn{(\lambda_1,...,\lambda_d)} are related to the parameters of the model
##' by a regression construction \code{pard} (\eqn{d \times k}), that links the \eqn{d}
##' \eqn{\lambda} parameters with the \eqn{k} underlying \eqn{\theta} parameters:
##' \deqn{ \lambda = theta.des \times \theta }
##' here using \code{theta.des} to specify these low-dimension associations. Default is a diagonal matrix.
##' This can be used to make structural assumptions about the variances of the random-effects
##' as is needed for the ACE model for example.
##'
##' The \code{case.control} option can be used with the pair specification of the pairwise parts
##' of the estimating equations. Here it is assumed that the second subject of each pair is the proband.
##'
##' @param margsurv Marginal model object.
##' @param data Data frame (must be given).
##' @param method Scoring method: \code{"nr"} for lava NR optimizer.
##' @param detail Detail level for output.
##' @param clusters Cluster variable.
##' @param silent Debug information level.
##' @param weights Weights for score equations.
##' @param theta Starting values for variance components.
##' @param theta.des Design for dependence parameters; when pairs are given, the indices of the
##' theta-design for this pair are given in pairs as column 5.
##' @param var.link Link function for variance: 1 for exponential link.
##' @param baseline.iid To adjust for baseline estimation, using \code{phreg} function on same data.
##' @param model Model type: \code{"clayton.oakes"} or \code{"plackett"}.
##' @param marginal.trunc Marginal left truncation probabilities.
##' @param marginal.survival Optional vector of marginal survival probabilities.
##' @param strata Strata for fitting (see examples).
##' @param se.clusters Clusters for SE calculation with IID.
##' @param numDeriv To get numDeriv version of second derivative; otherwise uses sum of squared scores for each pair.
##' @param random.design Random effect design for additive gamma model; when pairs are given, the indices of the
##' pairs' random.design rows are given as columns 3:4.
##' @param pairs Matrix with rows of indices (two-columns) for the pairs considered in the pairwise
##' composite score; useful for case-control sampling when marginal is known.
##' @param dim.theta Dimension of the theta parameter for pairs situation.
##' @param numDeriv.method Method for numerical derivative (e.g., \code{"simple"} to speed up things).
##' @param additive.gamma.sum For \code{two.stage=0}, this is specification of the \code{lamtot} in the models via
##' a matrix that is multiplied onto the parameters theta (dimensions = number of random effects \eqn{\times} number of theta parameters);
##' when \code{NULL}, sums all parameters.
##' @param var.par Is 1 for the default parametrization with the variances of the random effects;
##' \code{var.par=0} specifies that the \eqn{\lambda_j}'s are used as parameters.
##' @param no.opt For not optimizing.
##' @param ... Additional arguments to maximizer NR of lava.
##'
##' @return An object of class \code{"mets.twostage"} containing:
##' \item{theta}{Estimated dependence parameters.}
##' \item{coef}{Coefficients.}
##' \item{score}{Score vector.}
##' \item{hess}{Hessian matrix.}
##' \item{hessi}{Inverse Hessian matrix.}
##' \item{var.theta}{Variance of theta parameters.}
##' \item{robvar.theta}{Robust variance of theta parameters.}
##' \item{loglike}{Log-likelihood value.}
##' \item{theta.iid}{Influence functions for theta.}
##' \item{model}{Model type used.}
##'
##' @author Thomas Scheike
##' @references
##' \itemize{
##' \item Twostage estimation of additive gamma frailty models for survival data. Scheike (2019), work in progress.
##' \item Shih and Louis (1995) Inference on the association parameter in copula models for bivariate survival data, \emph{Biometrics}.
##' \item Glidden (2000), A Two-Stage estimator of the dependence parameter for the Clayton Oakes model, LIDA.
##' \item Measuring early or late dependence for bivariate twin data. Scheike, Holst, Hjelmborg (2015), LIDA.
##' \item Estimating heritability for cause specific mortality based on twins studies. Scheike, Holst, Hjelmborg (2014), LIDA.
##' \item Additive Gamma frailty models for competing risks data. Eriksson and Scheike (2015), \emph{Biometrics}.
##' }
##' @examples
##' data(diabetes)
##'
##' # Marginal Cox model  with treat as covariate
##' margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
##' ### Clayton-Oakes, MLE
##' fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
##' summary(fitco1)
##'
##' \donttest{ ## Reduce Ex.Timings
##' ### Plackett model
##' mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
##' fitp <- survival_twostage(mph,data=diabetes,theta=3.0,Nit=40,
##'                clusters=diabetes$id,var.link=1,model="plackett")
##' summary(fitp)
##'
##' ### Clayton-Oakes
##' fitco2 <- survival_twostage(mph,data=diabetes,theta=0.0,detail=0,
##'                  clusters=diabetes$id,var.link=1,model="clayton.oakes")
##' summary(fitco2)
##' fitco3 <- survival_twostage(margph,data=diabetes,theta=1.0,detail=0,
##'              clusters=diabetes$id,var.link=0,model="clayton.oakes")
##' summary(fitco3)
##'
##' ### without covariates but with stratafied
##' marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
##' fitpa <- survival_twostage(marg,data=diabetes,theta=1.0,
##'                 clusters=diabetes$id,model="clayton.oakes")
##' summary(fitpa)
##'
##'
##' ### Piecewise constant cross hazards ratio modelling
##' ########################################################
##'
##' d <- subset(sim_ClaytonOakes(1000,2,0.5,0,stoptime=2,left=0),!truncated)
##' udp <- piecewise_twostage(c(0,0.5,2),data=d,method="optimize",
##'                           id="cluster",timevar="time",
##'                           status="status",model="clayton.oakes",silent=0)
##' summary(udp)
##' }
##'
##' \donttest{ ## Reduce Ex.Timings
##' ### Same model using the strata option, a bit slower
##' ########################################################
##' ## makes the survival pieces for different areas in the plane
##' ##ud1=surv_boxarea(c(0,0),c(0.5,0.5),data=d,id="cluster",timevar="time",status="status")
##' ##ud2=surv_boxarea(c(0,0.5),c(0.5,2),data=d,id="cluster",timevar="time",status="status")
##' ##ud3=surv_boxarea(c(0.5,0),c(2,0.5),data=d,id="cluster",timevar="time",status="status")
##' ##ud4=surv_boxarea(c(0.5,0.5),c(2,2),data=d,id="cluster",timevar="time",status="status")
##'
##' ## everything done in one call
##' ud <- piecewise_data(c(0,0.5,2),data=d,timevar="time",status="status",id="cluster")
##' ud$strata <- factor(ud$strata);
##' ud$intstrata <- factor(ud$intstrata)
##'
##' ## makes strata specific id variable to identify pairs within strata
##' ## se's computed based on the id variable across strata "cluster"
##' ud$idstrata <- ud$id+(as.numeric(ud$strata)-1)*2000
##'
##' marg2 <- timereg::aalen(Surv(boxtime,status)~-1+factor(num):factor(intstrata),
##'                data=ud,n.sim=0,robust=0)
##' tdes <- model.matrix(~-1+factor(strata),data=ud)
##' fitp2 <- survival_twostage(marg2,data=ud,se.clusters=ud$cluster,clusters=ud$idstrata,
##'                 model="clayton.oakes",theta.des=tdes,step=0.5)
##' summary(fitp2)
##'
##' ### now fitting the model with symmetry, i.e. strata 2 and 3 same effect
##' ud$stratas <- ud$strata;
##' ud$stratas[ud$strata=="0.5-2,0-0.5"] <- "0-0.5,0.5-2"
##' tdes2 <- model.matrix(~-1+factor(stratas),data=ud)
##' fitp3 <- survival_twostage(marg2,data=ud,clusters=ud$idstrata,se.cluster=ud$cluster,
##'                 model="clayton.oakes",theta.des=tdes2,step=0.5)
##' summary(fitp3)
##'
##' ### same model using strata option, a bit slower
##' fitp4 <- survival_twostage(marg2,data=ud,clusters=ud$cluster,se.cluster=ud$cluster,
##'                 model="clayton.oakes",theta.des=tdes2,step=0.5,strata=ud$strata)
##' summary(fitp4)
##' }
##'
##' \donttest{ ## Reduce Ex.Timings
##' ### structured random effects model additive gamma ACE
##' ### simulate structured two-stage additive gamma ACE model
##' data <- sim_ClaytonOakes_twin_ace(2000,2,1,0,3)
##' out <- twin_polygen_design(data,id="cluster")
##' pardes <- out$pardes
##' pardes
##' des.rv <- out$des.rv
##' head(des.rv)
##' aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data,robust=0)
##' ts <- survival_twostage(aa,data=data,clusters=data$cluster,detail=0,
##' 	       theta=c(2,1),var.link=0,step=0.5,
##' 	       random.design=des.rv,theta.des=pardes)
##' summary(ts)
##' }
##' @keywords survival
##' @author Thomas Scheike
##' @aliases survival_twostage twostage_aalen twostage_cox.aalen twostage_coxph twostage_phreg randomDes readmargsurv 
##' @export survival_twostage
survival_twostage <- function(margsurv,data=NULL,
    method="nr",detail=0,clusters=NULL,
    silent=1,weights=NULL,theta=NULL,theta.des=NULL,
    var.link=1,baseline.iid=1,model="clayton.oakes",
    marginal.trunc=NULL,marginal.survival=NULL,strata=NULL,
    se.clusters=NULL,numDeriv=1,random.design=NULL,pairs=NULL,dim.theta=NULL,
    numDeriv.method="simple",additive.gamma.sum=NULL,var.par=1,no.opt=FALSE,...)
{#{{{
  requireNamespace("timereg")
# seting up design and variables
iid <- 1
two.stage <- 1; rate.sim <- 1; sym=1; var.func <- NULL
if (model=="clayton.oakes" || model=="gamma") dep.model <- 1
else if (model=="plackett" || model=="or") dep.model <- 2
else stop("Model must by either clayton.oakes or plackett \n");
start.time <- NULL; ptrunc <- NULL; psurvmarg <- NULL; status <- NULL; score.iid <- NULL
fix.baseline <- 0; convergence.bp <- 1;  ### to control if baseline profiler converges

if (is.null(data)) stop("Must give data \n"); 

  if ((!is.null(margsurv)) | (!is.null(marginal.survival))) fix.baseline <- 1
  antpers <- nrow(data); RR <-  rep(1,antpers);


  if (!is.null(margsurv)) {
     rrr <-  readmargsurv(margsurv,data,clusters)
     psurvmarg <- rrr$psurvmarg; ptrunc <- rrr$ptrunc; start.time <- rrr$entry;
     time2 <- rrr$exit; status <- rrr$status; clusters <- rrr$clusters
  }

  if (is.null(psurvmarg)) psurvmarg <- rep(1,antpers);
  if (!is.null(marginal.survival)) psurvmarg <- marginal.survival
  if (!is.null(marginal.trunc)) ptrunc <- marginal.trunc
  if (is.null(ptrunc)) ptrunc <- rep(1,length(psurvmarg))
  if (is.null(weights)==TRUE) weights <- rep(1,antpers);
  if (is.null(strata)==TRUE) strata<- rep(1,antpers);
  if (length(strata)!=antpers) stop("Strata must have length equal to number of data points \n");

  if (is.null(clusters)) stop("Clusters must be specified \n"); 

  # cluster set up
  cluster.call <- clusters
  out.clust <- cluster_index(clusters);
  clusters <- out.clust$clusters
  maxclust <- out.clust$maxclust
  antclust <- out.clust$cluster.size
  clusterindex <- out.clust$idclust
  clustsize <- out.clust$cluster.size
  call.secluster <- se.clusters

  if (is.null(se.clusters)) { se.clusters <- clusters; antiid <- nrow(clusterindex);} else  {
      iids <-  unique(se.clusters);
      antiid <- length(iids);
      if (is.numeric(se.clusters)) se.clusters <-  fast.approx(iids,se.clusters)-1
       else se.clusters <- as.integer(factor(se.clusters, labels = seq(antiid)))-1
  }
  if (length(se.clusters)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

  #

  ### setting design for random variables, in particular with pairs are given
  ddd <- randomDes(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,var.link,clusterindex,dim.theta)
  random.design=ddd$random.design;clusterindex=ddd$clusterindex;
  antpairs=ddd$antpairs;
  theta=ddd$theta;ptheta=ddd$ptheta;theta.des=ddd$theta.des
  pair.structure=ddd$pair.structure; dep.model=ddd$dep.model
  dim.rv <- ddd$dim.rv; additive.gamma.sum=ddd$additive.gamma.sum

  theta.score<-rep(0,ptheta);
  Stheta<-var.theta<-matrix(0,ptheta,ptheta);
  #
  ascertained <- 0

  obj <- function(par)
  { #

     if (pair.structure==0 | dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par),nrow=ptheta,ncol=1);
     if (pair.structure==1 & dep.model==3) Xtheta <- matrix(0,antpers,1); ## not needed

      if (var.link==1 & dep.model==3) epar <- c(exp(par)) else epar <- c(par)
      partheta <- epar

      if (var.par==1 & dep.model==3) {
	 ## from variances to
	 if (is.null(var.func)) {
	    sp <- sum(epar)
	    partheta <- epar/sp^2
         } else partheta <- epar ## par.func(epar)
      }

      if (pair.structure==0) {
	      outl<-.Call("twostageloglikeRV", # only two stage model for this option
	      icause=status,ipmargsurv=psurvmarg,
	      itheta=c(partheta), ithetades=theta.des,
	      icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex,
	      ivarlink=var.link,iid=iid,iweights=weights,isilent=silent,idepmodel=dep.model,
	      itrunkp=ptrunc,istrata=as.numeric(strata),iseclusters=se.clusters,iantiid=antiid,
	      irvdes=random.design,iags=additive.gamma.sum,iascertained=ascertained,
              PACKAGE="mets") #
      }
      else { # pair-structure
          outl<-.Call("twostageloglikeRVpairs", #
          icause=status,ipmargsurv=psurvmarg, itheta=c(partheta), ithetades=theta.des,
          icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,
	  iiid=iid,iweights=weights,isilent=silent, idepmodel=dep.model,
	  itrunkp=ptrunc,istrata=as.numeric(strata), iseclusters=se.clusters,iantiid=antiid,
	  irvdes=random.design, iags=additive.gamma.sum, iascertained=ascertained,PACKAGE="mets")
	  #
      } #

    if (detail==3) print(c(partheta,outl$loglike))

    ## variance parametrization, and inverse.link
    if (dep.model==3) {
    if (var.par==1) {
         ## from variances to and with sum for all random effects
         if (is.null(var.func)) {
	 if (var.link==0)  {
             mm <- matrix(-epar*2*sp,length(epar),length(epar))
	     diag(mm) <- sp^2-epar*2*sp
	 } else {
            mm <- -c(epar) %o% c(epar)*2*sp
            diag(mm) <- epar*sp^2-epar^2*2*sp
	 }
	    mm <- mm/sp^4
	 } else mm  <- numDeriv::hessian(var.func,par)
      } else {
	   if (var.link==0) mm <- diag(length(epar)) else
			  mm <- diag(length(c(epar)))*c(epar)
      }
      }


    if (dep.model==3) {
       outl$score <-  t(mm) %*% outl$score
       outl$Dscore <- t(mm) %*% outl$Dscore %*% mm
       if (iid==1) { outl$score.iid <- t(t(mm) %*% t(outl$score.iid))
               if (inherits(margsurv,"phreg")) {
	          outl$D1thetal  <- t(t(mm) %*% t(outl$D1thetal))
	          outl$D2thetal  <- t(t(mm) %*% t(outl$D2thetal))
	       }
       }
    }


    outl$gradient <- outl$score 
    outl$hessian <- outl$Dscore

    if (oout==0) ret <- with(outl,structure(outl$loglike,gradient=-gradient,hessian=-hessian))
    else if (oout==1) ret <- outl$gradient 
    else if (oout==2) ret <- outl

    return(ret)
  } #

  score1 <- NULL
  theta.iid <- NULL
  p <- theta

  oout <- 0
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(p,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  ## names(cc) <- colnames(X)
      oout <- 2
      val <- c(list(coef=cc),obj(opt$estimate))
 } else {oout <- 2; val <- c(list(coef=p),obj(p))}

     if (numDeriv>=1) {
	 p <- val$coef
         oout <- 1
         if (detail==1 ) cat("starting numDeriv for second derivative \n");
         val$hessian <- numDeriv::jacobian(obj,p,method=numDeriv.method)
         if (detail==1 ) cat("finished numDeriv for second derivative \n");
        }


    ### update p, but note that score and derivative in fact related to previous p
    if (!is.nan(sum(p))) {
      if (detail==1 && iid==1) cat("iid decomposition\n");
         if (iid==1) {  #
            score.iid <- val$score.iid
	    val$theta.iid <- score.iid
	    theta.iid <- score.iid
	    ucc <-  unique(cluster.call)
	    if (length(ucc)== nrow(theta.iid))
		    rownames(theta.iid) <- unique(cluster.call)
                ### lets iid for theta be just score to start, correction for marginal for phreg call

                if (inherits(margsurv,"phreg") & baseline.iid==1) {  # adjust for baseline when phreg is used

                  if ((margsurv$no.opt) | is.null(margsurv$coef)) fixbeta<- 1 else fixbeta <- 0
		  xx <- margsurv$cox.prep
                  id  <-  xx$id+1
		  cumhazt <- c(rrr$cum)
		  rr <- c(rrr$RR)

		  ## these refers to id given in cluster.call
		  xx <- margsurv$cox.prep
		  D1thetal<- val$D1thetal
		  D2thetal<- val$D2thetal
		  Dlamthetal <- -(D1thetal+D2thetal)
		  ## ordered as original data
                  Fbeta <- t(Dlamthetal) %*% (margsurv$X *c(cumhazt*psurvmarg))

                  ### timeordered
		  ## t- not needed because using S(T-) for survival already
                  Gbasedt <- Dlamthetal[xx$ord+1,,drop=FALSE]*psurvmarg[xx$ord+1]*rr[xx$ord+1]
                  Gbase <- apply(Gbasedt,2,revcumsumstrata,xx$strata,xx$nstrata)

                  if (fixbeta==0) {  ### iid after beta of marginal model
		     Z <- xx$X
		     U <- E <- matrix(0,nrow(xx$X),margsurv$p)
		     E[xx$jumps+1,] <- margsurv$E
		     U[xx$jumps+1,] <- margsurv$U
		     invhess <- -pinv(margsurv$hessian)
		     S0i <- rep(0,length(xx$strata))
		     S0i[xx$jumps+1] <- 1/margsurv$S0
		     cumhaz <- c(cumsumstrata(S0i,xx$strata,xx$nstrata))
		     EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	             rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset))
		     ### Martingale for all subjects
		     MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
		     UUbeta <- apply(MGt,2,sumstrata,id-1,max(id))
		     UUbeta <- UUbeta %*% invhess
		     GbaseEdLam0 <- t(Gbase) %*% (E*S0i)
		     Fbeta   <-  Fbeta -  GbaseEdLam0
		     Fbetaiid <- UUbeta %*% t(Fbeta)
	          }

	  ###  \int_0^\tau (GBase(s) / S_0(s)) dN_i(s) - dLamba_i(s)
	  xx <- margsurv$cox.prep
	  S0i  <- S0i2 <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/margsurv$S0
	  S0i2[xx$jumps+1] <- 1/margsurv$S0^2

	  GbasedN <- Gbase*S0i
	  GbasedLam0 <- apply(Gbase*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)

	  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset)) else rr <- c(xx$sign*exp( xx$offset))
	  MGt <- (GbasedN[,drop=FALSE]-GbasedLam0*rr)*c(xx$weights)
	  MGti <- apply(MGt,2,sumstrata,id-1,max(id))

	  if (fixbeta==0) MGti <-  MGti + Fbetaiid
	  theta.iid <-  -theta.iid + MGti
	  val$theta.iid <- theta.iid
	  } #
   } #
   }

  hess <- val$hessian
  if (!is.na(sum(hess))) hessi <- pinv(val$hessian) else hessi <- diag(nrow(val$hessian))

# handling output
  loglikeiid <- NULL; robvar.theta <- NULL; likepairs <- NULL; marginal.surv <- psurvmarg; 
  marginal.trunc <- ptrunc; 

  if (iid==1) {
  if (dep.model==3 & pair.structure==1) likepairs <- val$likepairs
  if (dep.model==3 & two.stage==0) {
	  hessi <- -1*hessi
	  all.likepairs <- val$all.likepairs
	  colnames(all.likepairs) <- c("surv","dt","ds","dtds","cause1","cause2")
  }
     theta.iid <- val$theta.iid %*% hessi
### rownames not set to make more robust
### if (is.null(call.secluster)) rownames(theta.iid) <- unique(cluster.call) else rownames(theta.iid) <- unique(se.clusters)
     robvar.theta  <- crossprod(theta.iid)
     loglikeiid <- val$loglikeiid
    } else { all.likepairs <- NULL}
  var.theta <- robvar.theta
  if (is.null(robvar.theta)) var.theta <- hessi

  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(theta),sep="")
  theta <- matrix(theta,length(c(theta)),1)
  if (length(thetanames)==nrow(theta)) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }


  ud <- list(theta=matrix(val$coef,ncol=1),coef=val$coef,score=val$gradient,hess=hess,hessi=hessi,var.theta=var.theta,
     model=model,robvar.theta=robvar.theta,
     loglike=val$loglike,loglikeiid=loglikeiid,likepairs=likepairs,
     theta.iid=theta.iid,thetanames=thetanames,
     score1=score1,Dscore=val$Dscore,
     marginal.surv=marginal.surv,marginal.trunc=marginal.trunc,
     se=diag(robvar.theta)^.5,score.iid=-score.iid,theta.des=theta.des,random.design=random.design)
  class(ud) <- "mets.twostage"
  attr(ud,"response") <- "survival"
  attr(ud,"Formula") <- formula
  attr(ud,"clusters") <- clusters
  attr(ud,"cluster.call") <- cluster.call
  attr(ud,"secluster") <- c(se.clusters)
  attr(ud,"sym")<-sym;
  attr(ud,"var.link")<-var.link;
  attr(ud,"var.par")<-var.par;
  attr(ud,"var.func")<-var.func;
  attr(ud,"ptheta")<-ptheta
  attr(ud,"antpers")<-antpers;
  attr(ud,"antclust")<-antclust;
  attr(ud,"Type") <- model
  attr(ud,"twostage") <- two.stage
  attr(ud,"additive-gamma") <- (dep.model==3)*1
  if (!is.null(marginal.trunc)) attr(ud,"trunclikeiid")<- val$trunclikeiid
  if (dep.model==3 & two.stage==0) attr(ud,"all.likepairs")<- all.likepairs
  if (dep.model==3 ) attr(ud,"additive.gamma.sum") <- additive.gamma.sum

if (dep.model==3 & pair.structure==0) {
       attr(ud, "pardes") <- theta.des
       attr(ud, "theta.des") <- theta.des
       attr(ud, "rv1") <-    random.design[1,]
    }
    if (dep.model==3 & pair.structure==1) {
       nrv <- clusterindex[1,6]
       attr(ud, "theta.des") <- matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "pardes") <-    matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "rv1") <-    random.design[1,]
       attr(ud, "nrv") <- clusterindex[1,6]
    }


  return(ud);
  #

} # }}}

##' Survival Twostage Helpers
##'
##' Helper functions for the twostage survival dependence models.
##'
##' \code{survival.twostage} is an alias for \code{survival_twostage}.
##'
##' \code{alpha2kendall} converts the Clayton-Oakes alpha parameter to Kendall's tau.
##'
##' \code{alpha2spear} converts the Clayton-Oakes alpha parameter to Spearman's rho.
##'
##' \code{piecewise_twostage} fits twostage models on piecewise time intervals.
##'
##' \code{piecewise_data} prepares data for piecewise twostage analysis.
##'
##' \code{matplot.mets.twostage} produces matplot of twostage baseline estimates.
##'
##' @name survival-helpers
##' @param object a twostage model object (for matplot method).
##' @param data a data.frame with the survival data.
##' @param id name of the cluster identifier column.
##' @param cut1 vector of cut points for the first time axis.
##' @param cut2 vector of cut points for the second time axis.
##' @param timevar character name of the time variable.
##' @param status character name of the status variable.
##' @param covars optional character vector of covariate names.
##' @param covars.pairs optional covariates at the pair level.
##' @param num character name of the within-cluster number variable.
##' @param method optimization method.
##' @param Nit maximum number of iterations.
##' @param detail level of detail in output.
##' @param silent level of verbosity (1=silent).
##' @param weights optional weights.
##' @param control optimization control list.
##' @param theta initial dependence parameter values.
##' @param theta.des theta design matrix.
##' @param var.link if 1, log-link for variance parameters.
##' @param link if 1, parameters are on log scale (for alpha2kendall/alpha2spear).
##' @param step step size for optimization.
##' @param model dependence model: \code{"plackett"} or \code{"clayton.oakes"}.
##' @param data.return if 1, return data with model fits.
##' @param x a marginal model object (for \code{survival.twostage}).
##' @param ... additional arguments.
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases survival.twostage alpha2kendall alpha2spear
##' @aliases piecewise_twostage piecewise_data matplot.mets.twostage
NULL

##' @rdname survival-helpers
##' @export 
survival.twostage <- function(x,...) survival_twostage(x,...)

##' @export
randomDes <- function(dep.model,random.design,theta.des,theta,antpers,ags,pairs,var.link,clusterindex,dim.theta)
{ ## {{{ 
   additive.gamma.sum <- ags

  if (!is.null(random.design)) { ### different parameters for Additive random effects 
     dep.model <- 3
     dim.rv <- ncol(random.design);
     if (is.null(theta.des)) theta.des<-diag(dim.rv);
  } else {
      random.design <- matrix(0,1,1);  dim.rv <- 1;
      additive.gamma.sum <- matrix(1,1,1);
  }

  if (is.null(theta.des)) ptheta<-1;
  if (is.null(theta.des)) theta.des<-matrix(1,antpers,ptheta); ###  else theta.des<-as.matrix(theta.des);
  ptheta<-ncol(theta.des)

  if (is.matrix(pairs)) {
       if ( ncol(pairs)==6)  ptheta<- dim.theta else ptheta<-ncol(theta.des)
  }

  theta.des <- as.matrix(theta.des)
  if (is.null(theta)==TRUE) {
         if (var.link==1) theta<- rep(-0.7,ptheta);
         if (var.link==0) theta<- rep(exp(-0.7),ptheta);
  }

  if (length(theta)!=ptheta) {
         theta<-rep(theta[1],ptheta);
  }
  theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);
  antpairs <- 1; ### to define

  if (is.null(additive.gamma.sum)) additive.gamma.sum <- matrix(1,dim.rv,ptheta)
  if (!is.null(pairs)) { pair.structure <- 1;} else  pair.structure <- 0;

  if (pair.structure==1 & dep.model==3) { #
  antpairs <- nrow(pairs);

  if ( (ncol(pairs)==2))
  {
      pairs <- cbind(pairs,pairs,1,ncol(random.design))
      theta.des <- matrix(c(theta.des),1,length(c(theta.des)))
  }
   clusterindex <- cbind(pairs[,1:5]-1,ncol(random.design))
  } #

  if (pair.structure==1 & dep.model!=3) { #
       antpairs <- nrow(pairs);
       if ((ncol(pairs)==2)) { ## when only pairs are given we refer to rows of theta.des from 3 column: same as index of pair-1
       pairs <- cbind(pairs,pairs[,1])
       }
       if (ncol(pairs)!=3) stop("with pairstructure and theta.des, 3rd column of pairs must give relevant design from theta.des\n")
       ## index in c are Rindex -1
       clusterindex <- pairs-1;
  }

###  print(head(pairs)); print(head(theta.des))
###  print(dim(pairs)); print(dim(theta.des))

  return(list(random.design=random.design,clusterindex=clusterindex,
	 antpairs=antpairs,pair.structure=pair.structure,
	 dep.model=dep.model,dim.rv=dim.rv, additive.gamma.sum=additive.gamma.sum,
         theta=theta,ptheta=ptheta,theta.des=theta.des))

} ## }}} 

readmargsurv <- function(margsurv,data,clusters)
{ ## {{{ 
start.time <- 0

if (inherits(margsurv,c("aalen","cox.aalen"))) {
	 formula<-attr(margsurv,"Formula");
	 beta.fixed <- attr(margsurv,"beta.fixed")
	 if (is.null(beta.fixed)) beta.fixed <- 1;
	 ldata <- timereg::aalen.des(formula,data=data,model="cox.aalen");
	 id <- attr(margsurv,"id");
	 mclusters <- attr(margsurv,"cluster.call")
	 X<-ldata$X;
	 time<-ldata$time2;
	 Z<-ldata$Z;
	 status<-ldata$status;
	 time2 <- attr(margsurv,"stop");
	 start.time <- attr(margsurv,"start")
	 antpers<-nrow(X);
         if (is.null(Z)==TRUE) {npar<-TRUE; semi<-0;}  else {
		     Z<-as.matrix(Z); npar<-FALSE; semi<-1;}
         if (npar==TRUE) {Z<-matrix(0,antpers,1); pz<-1; fixed<-0;} else {fixed<-1;pz<-ncol(Z);}
	 px<-ncol(X);

         if (is.null(clusters) && is.null(mclusters)) stop("No cluster variabel specified in marginal or twostage call\n");
         if (is.null(clusters)) clusters <- mclusters
	 if (nrow(X)!=length(clusters)) stop("Length of Marginal survival data not consistent with cluster length\n"); 
	 } else if (inherits(margsurv,"phreg")) { 
            ### setting up newdata with factors and strata
                antpers <- nrow(data)
		 rr <- readPhreg(margsurv,data,nr=FALSE)
		 time2 <- rr$exit
		 if (!is.null(rr$entry)) start.time <- rr$entry
		 else start.time <- rep(0,antpers);
		 status <- rr$status
	         if (is.null(clusters)) clusters <- rr$clusters
	
} else { ### coxph 
	  antpers <- margsurv$n
	  id <- 0:(antpers-1)
	  mt <- model.frame(margsurv)
	  Y <- model.extract(mt, "response")
	  if (!inherits(Y, "Surv")) stop("Response must be a survival object")
	  if (attr(Y, "type") == "right") {
	      time2 <- Y[, "time"];
	      status <- Y[, "status"]
		start.time <- rep(0,antpers);
		} else {
		 start.time <- Y[, 1];
		 time2 <- Y[, 2];
		 status <- Y[, 3];
		}
	   Z <- matrix(1,antpers,length(coef(margsurv)));

	   if (is.null(clusters)) stop("must give clusters for coxph\n");
	   cluster.call <- clusters;
	   X <- matrix(1,antpers,1); ### Z <- matrix(0,antpers,1); ### no use for these
	   px <- 1; pz <- ncol(Z);
	   start <- rep(0,antpers);
	   beta.fixed <- 0
	   semi <- 1
}

  if (!is.null(start.time)) {
      if (any(abs(start.time)>0)) lefttrunk <- 1  else lefttrunk <- 0;
  } else lefttrunk <- 0

if (!is.null(margsurv))  {
  if (inherits(margsurv,c("aalen","cox.aalen")))  { 
         resi <- timereg::residualsTimereg(margsurv,data=data)
         RR  <- resi$RR
	 cum <- resi$cumhaz/RR
	 psurvmarg <- exp(-resi$cumhaz);
         ptrunc <- rep(1,length(psurvmarg));
	 if (lefttrunk==1) ptrunc <- exp(-resi$cumhazleft); 
  } else if (inherits(margsurv,"coxph")) {  
    ### some problems here when data is different from data used in margsurv
       residuals <- residuals(margsurv)
       cum <- cumhaz <- status-residuals
       psurvmarg <- exp(-cumhaz);
       cumhazleft <- rep(0,antpers)
       ptrunc <- rep(1,length(psurvmarg));
       RR<- exp(margsurv$linear.predictors+sum(margsurv$means*coef(margsurv)))
       cum <- cum/RR
        if ((lefttrunk==1)) {
         stop("Use cox.aalen function for truncation case \n");
         baseout <- survival::basehaz(margsurv,centered=FALSE);
         cumt <- cbind(baseout$time,baseout$hazard)
	 cumt <- cpred(cumt,start.time)[,2]
	 ptrunc <- exp(-cumt * RR)
	}
  } else if (inherits(margsurv,"phreg")) {

	ppsurvmarg <- predict(margsurv,data,tminus=TRUE,times=time2,individual.time=TRUE,se=FALSE)
        psurvmarg <- ppsurvmarg$surv
        cum <- ppsurvmarg$cumhaz
	RR <- ppsurvmarg$RR
        ptrunc <- rep(1,length(psurvmarg));
        if ((lefttrunk==1)) {
	  ptrunc <- predict(margsurv,data,tminus=TRUE,times=start.time,individual.time=TRUE,se=FALSE)$surv
	}
  }
}

if (is.null(clusters) &  (!inherits(margsurv,"phreg"))) stop("must give clusters")

return(list(psurvmarg=psurvmarg,ptrunc=ptrunc,entry=start.time,exit=time2,
	    status=status,clusters=clusters,cum=cum,RR=RR))

}  ## }}} 

##' Twostage Survival Model Fitted by Pseudo MLE
##'
##' Fits Clayton-Oakes clustered survival data using marginals that are on Cox form
##' in the likelihood for the dependence parameter as in Glidden (2000).
##'
##' The dependence can be modelled via a regression structure for the independent
##' gamma distributed random effects and their variances that may depend on cluster covariates.
##' So:
##' \deqn{ \theta = h( z_j^T \alpha) }
##' where \eqn{z} is specified by \code{theta.des}. The link function can be the exponential
##' when \code{var.link=1}.
##'
##' @param margsurv Marginal model from \code{phreg}.
##' @param data Data frame.
##' @param theta Starting values for variance components.
##' @param theta.des Design for dependence parameters; when pairs are given, this could be a
##' (pairs) \eqn{\times} (number of parameters) \eqn{\times} (max number random effects) matrix.
##' @param var.link Link function for variance; if 1 then uses exponential link.
##' @param method Type of optimizer; default is Newton-Raphson \code{"NR"}.
##' @param no.opt To not optimize, for example to get score and IID for specific theta.
##' @param weights Cluster-specific weights, but given with length equivalent to data-set; weights for score equations.
##' @param se.cluster Specifies how the influence functions are summed before squared when computing the variance.
##' Note that the id from the marginal model is used to construct MLE, and then these scores can be summed with the \code{se.cluster} argument.
##' @param ... Arguments to be passed to optimizer.
##'
##' @return An object of class \code{"mets.twostage"} containing:
##' \item{theta}{Estimated dependence parameters.}
##' \item{coef}{Coefficients.}
##' \item{var.theta}{Variance of theta parameters.}
##' \item{robvar.theta}{Robust variance of theta parameters.}
##' \item{theta.iid}{Influence functions for theta.}
##' \item{theta.iid.naive}{Naive influence functions for theta.}
##' \item{loglike}{Log-likelihood value.}
##'
##' @author Thomas Scheike
##' @references
##' \itemize{
##' \item Measuring early or late dependence for bivariate twin data. Scheike, Holst, Hjelmborg (2015), LIDA.
##' \item Twostage modelling of additive gamma frailty models for survival data. Scheike and Holst, working paper.
##' \item Shih and Louis (1995) Inference on the association parameter in copula models for bivariate survival data, \emph{Biometrics}.
##' \item Glidden (2000), A Two-Stage estimator of the dependence parameter for the Clayton Oakes model, LIDA.
##' }
##' @examples
##' data(diabetes)
##' dd <- phreg(Surv(time, status == 1) ~ treat + cluster(id), diabetes)
##' oo <- twostageMLE(dd, data = diabetes)
##' summary(oo)
##'
##' theta.des <- model.matrix(~ -1 + factor(adult), diabetes)
##' oo <- twostageMLE(dd, data = diabetes, theta.des = theta.des)
##' summary(oo)
##' @keywords survival
##' @author Thomas Scheike
##' @export
twostageMLE <-function(margsurv,data=parent.frame(),
theta=NULL,theta.des=NULL,var.link=0,method="NR",no.opt=FALSE,weights=NULL,se.cluster=NULL,...)
{ ## {{{ 
 if (!inherits(margsurv,"phreg"))  stop("Must use phreg for this \n");
 if (is.null(margsurv$call.id))  stop("Must be called with cluster(id) \n");

 clusters <- margsurv$cox.prep$id
 n <- nrow(margsurv$cox.prep$X)

 if (is.null(theta.des)==TRUE) ptheta<-1;
 if (is.null(theta.des)==TRUE) theta.des<-matrix(1,n,ptheta) else theta.des<-as.matrix(theta.des);
  ptheta<-ncol(theta.des);
  if (nrow(theta.des)!=n) stop("Theta design does not have correct dim");

  if (is.null(theta)==TRUE) {
      if (var.link==1) theta<- rep(0.0,ptheta);
      if (var.link==0) theta<- rep(1.0,ptheta);
  }
  if (length(theta)!=ptheta) theta<-rep(theta[1],ptheta);
  theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);

  max.clust <- length(unique(clusters))
  theta.iid <- matrix(0,max.clust,ptheta)

  xx <- margsurv$cox.prep
  nn <- length(xx$strata)

  if (is.null(weights)) weights <-  rep(1,nn)
  if (length(weights)!=nn) stop("Weights do not have right length")

  statusxx <- rep(0,length(xx$strata))
  statusxx[xx$jumps+1] <- 1
  xx$status <- statusxx
  mid <- max(xx$id)+1

  ## Ni.(t-), Ni.(t)
  Nsum <- cumsumstratasum(statusxx,xx$id,mid,type="all")
  ## Ni.(tau)
  Ni.tau <- sumstrata(statusxx,xx$id,mid)
  ## cumahz(T_i-)
  S0i2 <- S0i <- rep(0, length(xx$strata))
  S0i[xx$jumps + 1] <- 1/margsurv$S0
  cumhazD <- cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum
  if (!is.null(margsurv$coef)) RR <- exp(xx$X %*% margsurv$coef) else RR  <-  rep(1,nn)
  H <- c(cumhazD * RR)

  cc <- cluster_index(xx$id)
  firstid <- cc$firstclustid+1
  if (max(cc$cluster.size)==1) stop("No clusters !, maxclust size=1\n");

  ### order after time-sorted data
  theta.des <- theta.des[xx$ord+1,,drop=FALSE]
  weightsid <- weights <- weights[xx$ord+1]
  ### clusterspecific weights
  weights <- weights[firstid]

  thetaX<- as.matrix(theta.des[firstid,,drop=FALSE])

obj <- function(par,all=FALSE)
{

if (var.link==1) epar <- c(exp(c(par))) else epar <- c(par)

thetav <- c(as.matrix(theta.des) %*%  c(epar))
thetai <-thetav[firstid]

Hs <- sumstrata(H*exp(thetav*H),xx$id,mid)
R <- sumstrata((exp(thetav*H)-1),xx$id,mid) + 1
H2 <- sumstrata(H^2*exp(thetav*H),xx$id,mid)

l1 <- sumstrata(log(1+thetav*Nsum$lagsum)*statusxx,xx$id,mid)
l2 <- sumstrata(statusxx*H,xx$id,mid)
l3 <- -((thetai)^{-1}+Ni.tau) * log(R)
logliid <- (l1 + thetai* l2 + l3)*c(weights)
logl <- sum(logliid)
ploglik <- logl

## first derivative
l1s <- sumstrata(Nsum$lagsum/(1+thetav*Nsum$lagsum)*statusxx,xx$id,mid)
l2s <- (thetai^{-2}) * log(R)
l3s <- -(thetai^{-1}+Ni.tau) * Hs / R
Dltheta <- (l1s+l2s+l3s+l2)*c(weights)
scoreiid <- thetaX* c(Dltheta)

## second derivative
D2N <- -sumstrata(Nsum$lagsum^2/(1+thetav*Nsum$lagsum)^2*statusxx,xx$id,mid)
Dhes  <- c(D2N+(2/thetai^2)*Hs/R-(2/thetai^3)*log(R)-(1/thetai+Ni.tau)*(H2*R-Hs*Hs)/R^2)
Dhes  <-  Dhes * c(weights)

if (var.link==1) {
scoreiid  <-  scoreiid * c(thetai);  
Dhes<- Dhes* thetai^2 + thetai *  Dltheta
}

gradient <- apply(scoreiid,2,sum)
hessian <-  crossprod(thetaX,thetaX*c(Dhes))
hess2 <-    crossprod(scoreiid)

val <- list(id=xx$id,score.iid=scoreiid,logl.iid=logliid,ploglik=ploglik,
            gradient=-gradient,hessian=hessian,hess2=hess2)

if (all) return(val);

with(val, structure(-ploglik,gradient=-gradient,hessian=hessian))
}

  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          opt <- lava::NR(theta,obj,...)
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,theta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;
###   names(cc) <- colnames(theta.des)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=theta),obj(theta,all=TRUE))
  val$score <- val$gradient


  theta <- matrix(c(val$coef),length(c(val$coef)),1)
  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(c(theta)),sep="")
  if (length(thetanames)==length(c(theta))) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }

  hessianI <- pinv(val$hessian)
  val$theta.iid.naive  <-  val$score.iid %*% hessianI

  ### iid due to Marginal model
  biid <- 1
  if (biid==1) {

  if ((margsurv$no.opt) | is.null(margsurv$coef)) fixbeta<- 1 else fixbeta <- 0

   id  <-  xx$id+1
   if (var.link==1) epar <- c(exp(theta)) else epar <- c(theta)

   thetaX<- as.matrix(theta.des[firstid,,drop=FALSE])
   thetav <- c(as.matrix(theta.des) %*%  c(epar))

   Hs <- c(sumstrata(H*exp(thetav*H),xx$id,mid))
   R <- c(sumstrata((exp(thetav*H)-1),xx$id,mid)) + 1
   H2 <- c(sumstrata(H^2*exp(thetav*H),xx$id,mid) )

   Ft <- ((1/(thetav*R[id]))*exp(thetav*H)-(1/thetav+Ni.tau[id])*(1+thetav*H)*exp(thetav*H)/R[id]+statusxx+(1+thetav*Ni.tau[id])*exp(thetav*H)*Hs[id]/R[id]^2)

  if (var.link==1) {
	Ft  <-  Ft * c(exp(theta.des %*% val$coef));
  }
  Gt <- c(RR *Ft * xx$sign * weightsid)
  Ft  <- c( Ft * H * weightsid )
  Fbeta <-  t(theta.des) %*% ( xx$X * Ft )
  Gbase <- apply(Gt* theta.des,2,revcumsumstrata,xx$strata,xx$nstrata)

  ### beta part
  if (fixbeta==0) {
	  Z <- xx$X
	  U <- E <- matrix(0,nrow(xx$X),margsurv$p)
	  E[xx$jumps+1,] <- margsurv$E
	  U[xx$jumps+1,] <- margsurv$U
          invhess <- -pinv(margsurv$hessian)
	  S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/margsurv$S0
	  cumhaz <- c(cumsumstrata(S0i,xx$strata,xx$nstrata))
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset))
	  ### Martingale  as a function of time and for all subjects to handle strata
	  MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
          UUbeta <- apply(MGt,2,sumstrata,id-1,max(id))
	  UUbeta <- UUbeta %*% invhess
	  GbaseEdLam0 <- t(Gbase) %*% (E*S0i)
	  Fbeta   <-  Fbeta -  GbaseEdLam0
          Fbetaiid <- UUbeta %*% t(Fbeta)
  }

  ###  \int_0^\tau (GBase(s) / S_0(s)) dN_i(s) - dLamba_i(s)
  xx <- margsurv$cox.prep
  S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <- 1/margsurv$S0
  S0i2[xx$jumps+1] <- 1/margsurv$S0^2

  GbasedN <- Gbase*S0i
  GbasedLam0 <- apply(Gbase*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)

  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset)) else rr <- c(xx$sign*exp( xx$offset))
  MGt <- (GbasedN[,drop=FALSE]-GbasedLam0*rr)*c(xx$weights)
  MGti <- apply(MGt,2,sumstrata,id-1,max(id))

###  if (fixbeta==0) print(cbind(val$score.iid,MGti,Fbetaiid)) else print(cbind(val$score.iid,MGti))

  if (fixbeta==0) MGti <- MGti+Fbetaiid
  theta.iid <-  val$score.iid + MGti
  theta.iid  <-  theta.iid %*% hessianI
  ## take names from phreg call
  if (!is.null(margsurv$id)) rownames(theta.iid) <- unique(margsurv$id)
  val$theta.iid <- theta.iid

  }

  ### if se.cluster is there we cluster iid-decomp before squaring 
  if (!is.null(se.cluster)) 
	  if (length(se.cluster)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

  if (!is.null(se.cluster)) {
      iids <-  unique(se.cluster);
      nseclust  <- length(iids);
      if (is.numeric(se.cluster)) se.cluster <-  fast.approx(iids,se.cluster)-1
       else se.cluster <- as.integer(factor(se.cluster, labels = seq(nseclust)))-1

      val$theta.iid <-       apply(val$theta.iid,se.cluster,nseclust)
      val$theta.iid.naive <- apply(val$theta.iid.naive,se.cluster,nseclust)
  }

  var <- robvar.theta  <-  var.theta  <-  crossprod(val$theta.iid)
  naive.var  <-   crossprod(val$theta.iid.naive)

  val  <- c(val,list(theta=theta,var.theta=var.theta,robvar.theta=robvar.theta,
		     var=var,thetanames=thetanames,model="clayton.oakes",
	             se=diag(robvar.theta)^.5),
	             var.naive=naive.var)
  class(val) <- "mets.twostage"
  attr(val,"clusters") <- clusters
  attr(val,"secluster") <- c(se.cluster)
  attr(val,"var.link")<-var.link;
  attr(val,"ptheta")<-ptheta
  attr(val,"n")<-n ;
  attr(val,"response")<- "survival"
  attr(val,"additive-gamma")<-0
  attr(val,"twostage") <- "two.stage"

  return(val)
} ## }}}


##' @export
summary.mets.twostage <- function(object,digits = 3,silent=0,theta.des=NULL,...)
{ ## {{{ 
  if (!(inherits(object,"mets.twostage"))) stop("Must be a Two-Stage object")

  var.link<-attr(object,"var.link");
  var.par  <- attr(object,"var.par");
  model <- object$model
  if ((model=="plackett" || model=="or") )         model <- "or"
  if ((model=="clayton.oakes" || model=="gamma") ) model <- "gamma"
  if ((attr(object,"additive-gamma")==1) & (silent==0)) addgam <- TRUE else addgam <- FALSE

  if ((model=="or") && (silent==0)) cat("Dependence parameter for Odds-Ratio (Plackett) model \n");
  if (attr(object,"response")=="binomial") response <- "binomial" else response <- "survival"
  if ((model=="gamma") && (silent==0)) {
	  cat("Dependence parameter for Clayton-Oakes model\n")
	  if (var.par==1 || !addgam) cat("Variance of Gamma distributed random effects \n");
	  if (var.par==0 && addgam) cat("Inverse of variance of Gamma distributed random effects \n");
  }
  if (var.link==1 && silent==0) cat("With log-link \n")
  if ((sum(abs(object$score))>0.0001) & (silent==0))  {
	  cat(" Variance parameters did not converge, allow more iterations.\n");
	  cat(paste("Score:",object$score,"  \n"));
  }
  theta <- object$theta
  if (is.null(rownames(theta)))  rownames(theta) <- paste("dependence",1:nrow(theta),sep="")

  coefs <- coef.mets.twostage(object,response=response,...);

  if (attr(object,"additive-gamma")==1 & (!is.null(object$robvar.theta))  ) {
      var.link <- attr(object,"var.link");
      var.par  <- attr(object,"var.par");
      rv1      <- attr(object,"rv1");
      if (is.matrix(rv1)) rv1 <- rv1[1,]
      theta.des <- attr(object,"pardes");
      ags <- attr(object,"additive.gamma.sum");
      ptheta <- attr(object,"ptheta")
      npar <- nrow(object$theta)
      theta <- object$theta[seq(1,ptheta),1,drop=FALSE]
      robvar.theta <- object$robvar.theta[seq(1,ptheta),seq(1,ptheta)]
      if (var.link==1) par <- theta.des %*% exp(theta) else  par <- theta.des %*% theta

      if (model=="or" || model=="plackett") var.par<-1 ## same as var.par=0 for this model

      if (attr(object,"twostage")==0) {
      }

      if (var.par==0) { #
	      if (var.link==1) {
		  fp <- function(p,d,t){ res <- exp(p*t)/(sum(rv1* c(theta.des %*% matrix(exp(p),ncol=1))))^d;
					     if (t==0) res <- res[1]; return(res); }
                  pare <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) exp(p),labels=rownames(theta))
	      } else {
		      fp <- function(p,d,t) {  res <- (p^t)/(sum(rv1* c(theta.des %*% matrix(p,ncol=1))))^d;
					     if (t==0) res <- res[1]; return(res); }
	          pare <- NULL
	      }

             e      <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
             vare <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,2,1),labels=rownames(theta))
             vartot <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,0))
      } #
      if (var.par==1) { #

	   if (var.link==1) { #
	     fp <- function(p,d,t){  res <- exp(p*t)/(sum(rv1* c(theta.des %*% matrix(exp(p),ncol=1))))^d;
                                     if (t==0) res <- res[1]; return(res); }
             e      <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
             vare   <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) exp(p),labels=rownames(theta))
             vartot <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) sum(exp(p)))
      } else {
              fp <- function(p,d,t) {  res <- (p^t)/(sum(rv1* c(theta.des %*% matrix(p,ncol=1))))^d;
                                     if (t==0) res <- res[1]; return(res); }
              e     <-  lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
              pare  <-  lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,2,1),labels=rownames(theta))
              vartot <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) sum(p))
	      vare <- NULL
	       } #
      } #
      res <- list(estimates=coefs, type=attr(object,"Type"),h=e,vare=vare,vartot=vartot)

  } else {
	 if (var.link==1) { #
	     if (model=="or") {
		or <-  lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p),labels=rownames(theta))
	        res <- list(estimates=coefs,or=or,type=attr(object,"Type"))
	     } else {
	        vargam <-  lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p),labels=rownames(theta))
                res <- list(estimates=coefs,vargam=vargam, type=attr(object,"Type"))
	     }
      } else {
	     if (model=="or") {
             lor <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) log(p),labels=rownames(theta))
	     res <- list(estimates=coefs,log.or=lor,type=attr(object,"Type"))
	     } else  {
             res <- list(estimates=coefs,type=attr(object,"Type"))
	     }
      } #
  }

  class(res) <- "summary.mets.twostage"
  res
} ## }}}


##' @export
coef.mets.twostage <- function(object,var.link=NULL,response="survival",...)
{ #
  pt <- attr(object,"ptheta")
  theta <- object$theta[seq(pt),1]

  if (is.null(object$robvar.theta))
	  var.theta  <-  object$var.theta[seq(1,pt),seq(1,pt),drop=FALSE] else
	  var.theta  <-   object$robvar.theta[seq(1,pt),seq(1,pt),drop=FALSE]
  se <- diag(var.theta)^.5

  if (is.null(var.link))
     if (attr(object,"var.link")==1) vlink <- 1 else vlink <- 0
     else vlink <- var.link
  res <- cbind(theta, se )
  wald <- theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  if (response=="survival") {
       if (object$model=="plackett") {
       spearman <- alpha2spear(theta,link=vlink)
       Dspear <- numDeriv::jacobian(alpha2spear,theta,link=vlink)
       var.spearman <- Dspear %*% var.theta %*%  Dspear
       se.spearman <- diag(var.spearman)^.5
       res <- as.matrix(cbind(res, wald, waldp,spearman,se.spearman))
       if (vlink==1) colnames(res) <- c("log-Coef.", "SE","z", "P-val","Spearman Corr.","SE")
	  else colnames(res) <- c("Coef.", "SE","z", "P-val","Spearman Corr.","SE")
	  if ((!is.null(object$thetanames)) & (nrow(res)==length(object$thetanames))) rownames(res)<-object$thetanames
       }
       if (object$model=="clayton.oakes") {
       kendall <- alpha2kendall(theta,link=vlink)
       Dken <- numDeriv::jacobian(alpha2kendall,theta,link=vlink)
       var.kendall<- Dken %*% var.theta %*%  Dken
       se.kendall <- diag(var.kendall)^.5
       res <- as.matrix(cbind(res, wald, waldp,kendall,se.kendall))
       if (vlink==1) colnames(res) <- c("log-Coef.", "SE","z", "P-val","Kendall tau","SE")
       else colnames(res) <- c("Coef.", "SE","z", "P-val","Kendall tau","SE")
       if ((!is.null(object$thetanames)) & (nrow(res)==length(object$thetanames))) rownames(res)<-object$thetanames
       }
  }
  return(res)
} #

##' @export
print.mets.twostage<-function(x,digits=3,...)
{ #
  cat("\n")
  print(summary(x,silent=0));
} #

##' @export
plot.mets.twostage<-function(x,pointwise.ci=1,robust=0,specific.comps=FALSE,
		level=0.05,
		start.time=0,stop.time=0,add.to.plot=FALSE,mains=TRUE,
                xlab="Time",ylab ="Cumulative regression function",...)
{ #
  if (!(inherits(x, 'two.stage'))) stop("Must be a Two-Stage object")
  object <- x; rm(x);

  B<-object$cum; V<-object$var.cum; p<-dim(B)[[2]];
  if (robust>=1) V<-object$robvar.cum;

  if (sum(specific.comps)==FALSE) comp<-2:p else comp<-specific.comps+1
  if (stop.time==0) stop.time<-max(B[,1]);

  med<-B[,1]<=stop.time & B[,1]>=start.time
  B<-B[med,]; Bs<-B[1,];  B<-t(t(B)-Bs); B[,1]<-B[,1]+Bs[1];
  V<-V[med,]; Vs<-V[1,]; V<-t( t(V)-Vs);
  Vrob<-object$robvar.cum;
  Vrob<-Vrob[med,]; Vrobs<-Vrob[1,]; Vrob<-t( t(Vrob)-Vrobs);

  c.alpha<- qnorm(1-level/2)
  for (v in comp) {
    c.alpha<- qnorm(1-level/2)
    est<-B[,v];ul<-B[,v]+c.alpha*V[,v]^.5;nl<-B[,v]-c.alpha*V[,v]^.5;
    if (add.to.plot==FALSE)
      {
        plot(B[,1],est,ylim=1.05*range(ul,nl),type="s",xlab=xlab,ylab=ylab)
        if (mains==TRUE) title(main=colnames(B)[v]); }
    else lines(B[,1],est,type="s");
    if (pointwise.ci>=1) {
      lines(B[,1],ul,lty=pointwise.ci,type="s");
      lines(B[,1],nl,lty=pointwise.ci,type="s"); }
    if (robust>=1) {
      lines(B[,1],ul,lty=robust,type="s");
      lines(B[,1],nl,lty=robust,type="s"); }
    abline(h=0);
  }
}  #

##' @rdname survival-helpers
##' @export
matplot.mets.twostage <- function(object,...)
{ #
B <- object$baseline
matplot(B[,1],B[,-1],type="s",...)
} #

##' @export
predict.mets.twostage <- function(object,X=NULL,Z=NULL,times=NULL,times2=NULL,theta.des=NULL,diag=TRUE,...)
{ #
time.coef <- data.frame(object$cum)
if (!is.null(times)) {
cum <- cpred(object$cum,times);
cum2 <- cpred(object$cum,times);
} else { cum <- object$cum; cum2 <- object$cum }
if (!is.null(times2)) cum2 <- cpred(object$cum,times2);

if (is.null(X)) X <- 1;
if (is.null(X) & (!is.null(Z))) { Z <- as.matrix(Z);  X <- matrix(1,nrow(Z),1)}
if (is.null(Z) & (!is.null(X)))  {X <- as.matrix(X);  Z <- matrix(0,nrow(X),1); gamma <- 0}

if (diag==FALSE) {
   time.part <-  X %*% t(cum[,-1])
   time.part2 <-  X %*% t(cum2[,-1])
   if (!is.null(object$gamma)) { RR <- exp( Z %*% gamma );
       cumhaz <- t( t(time.part) * RR ); cumhaz2 <- t( t(time.part2) * RR )}
	    else { cumhaz <- time.part;  cumhaz2 <- time.part2;   }
} else {
	time.part <-  apply(as.matrix(X*cum[,-1]),1,sum)
	time.part2 <-  apply(as.matrix(X*cum2[,-1]),1,sum)
}

if (!is.null(object$gamma)) {
	RR<- exp(Z%*%gamma);
	cumhaz <- t( t(time.part) * RR );
	cumhaz2 <- t( t(time.part2) * RR )} else {
		cumhaz <- time.part;  cumhaz2 <- time.part2;
}
S1 <- exp(-cumhaz); S2 <- exp(-cumhaz2)

if (attr(object,"var.link")==1) theta  <- exp(object$theta) else theta <- object$theta
if (!is.null(theta.des)) theta <- c(theta.des %*% object$theta)

if (diag==FALSE) St1t2<- (outer(c(S1)^{-(theta)},c(S2)^{-(theta)},FUN="+") - 1)^(-(1/theta)) else
St1t2<- ((S1^{-(theta)}+S2^{-(theta)})-1)^(-(1/theta))

out=list(St1t2=St1t2,S1=S1,S2=S2,times=times,times2=times2,theta=theta)
return(out)
} #

##' @rdname twin-design
##' @param cr.models formula specifying time and status variables.
##' @param bin logical; if TRUE uses binary (prevalence) ordering rather than time ordering.
##' @export ascertained_pairs
ascertained_pairs <-function (pairs,data,cr.models,bin=FALSE)
{
      timestatus <- all.vars(cr.models)
      ### let first event by second column and only
      ### use pairs where first is event
      apairs <- pairs
      if (bin==TRUE) fj <- ifelse(data[pairs[,1],timestatus[1]] > data[pairs[,2],timestatus[1]],1,2)
      else fj <- ifelse(data[pairs[,1],timestatus[1]] < data[pairs[,2],timestatus[1]],1,2)
      ### change order when 1st comes first
      apairs[fj==1,1] <- pairs[fj==1,2]
      apairs[fj==1,2] <- pairs[fj==1,1]
      ### only take pairs where first is a jump
      if (bin==FALSE) {
	      jmpf <- (data[apairs[,2],timestatus[2]]==1)
	      apairs <- apairs[data[apairs[,2],timestatus[2]]==1,]
              attr(pairs,"jump-first") <- jmpf
      }
      pairs <- apairs
      return(pairs)
} 

##' @rdname survival-helpers
##' @export
alpha2spear <- function(theta,link=1) { #
   if (link==1) theta <- exp(theta)
if (length(theta)>1) {
   out <- c()
   for (thet in theta) {
   if (thet!=1) out <- c(out,( (thet+1)/(thet-1) -2* thet* log(thet)/ (thet-1)^2))
   else out <- c(out,0)
   }
} else { if (theta!=1) out <- ( (theta+1)/(theta-1) -2* theta* log(theta)/ (theta-1)^2) }

return(out)
} #

##' @rdname survival-helpers
##' @export
alpha2kendall <- function(theta,link=0) {  #
   if (link==1) theta <- exp(theta)
   return(1/(1+2/theta))
} #

##' @rdname survival-helpers
##' @export piecewise_twostage
piecewise_twostage <- function(cut1,cut2,data=parent.frame(),timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,
            method="optimize",Nit=100,detail=0,silent=1,weights=NULL,
            control=list(),theta=NULL,theta.des=NULL,var.link=1,
	    step=0.5,model="plackett",data.return=0)
{ #{{{
iid <- 1
ud <- list()
if (missing(cut2)) cut2 <- cut1;
nc1 <- length(cut1); nc2 <- length(cut2)
names1 <- names2 <- c()
theta.mat <- se.theta.mat <- cor.mat <- score.mat <- se.cor.mat <- matrix(0,nc1-1,nc2-1);
clusters <- data[,id]
cluster.call <- clusters
idi <- unique(data[,id]);

if (iid==1) { theta.iid <- matrix(0,length(idi),(nc1-1)*(nc2-1));
              rownames(theta.iid) <- idi
            } else theta.iid <- NULL

thetal <- c()
k <- 0;
for (i1 in 2:nc1)
for (i2 in 2:nc2)
{
k <-(i1-2)*(nc2-1)+(i2-1)
if (silent<=0) cat(paste("Data-set ",k,"out of ",(nc1-1)*(nc2-1)),"\n");
datalr <- surv_boxarea(c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),data,timevar=timevar,
status=status,id=id,covars=covars,covars.pairs=covars.pairs,num=num,silent=silent)
if (silent<=-1) print("back in piecewise.twostage");
if (silent<=-1) print(summary(datalr));
if (silent<=-1) print(head(datalr));
if (silent<=-1) print(summary(datalr[,id]));
 boxlr <- list(left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]))
datalr$tstime <- datalr[,timevar]
datalr$tsstatus <- datalr[,status]
datalr$tsid <- datalr[,id]
###

if (is.null(covars))
f <- as.formula(with(attributes(datalr),paste("Surv(",time,",",status,")~-1+factor(",num,")")))
else f <- as.formula(with(attributes(datalr),paste("Surv(",time,",",status,")~-1+factor(",num,"):",covars)))
marg1 <- timereg::aalen(f,data=datalr,n.sim=0,robust=0)

fitlr<-  survival_twostage(marg1,data=datalr,clusters=datalr$tsid,
,model=model, Nit=Nit,detail=detail,silent=silent,weights=weights,
theta=theta,theta.des=theta.des,var.link=var.link,step=step)
###fitlr<-  survival_twostageCC(marg1,data=datalr,clusters=datalr$tsid,model=model,method=method,
###Nit=Nit,detail=detail,silent=silent,weights=weights,
###baseline.iid=0,control=control,
###theta=theta,theta.des=theta.des,var.link=var.link,step=step)
####
coef <- coef(fitlr)
theta.mat[i1-1,i2-1] <- fitlr$theta
se.theta.mat[i1-1,i2-1] <- fitlr$var.theta^.5
cor.mat[i1-1,i2-1] <- coef[1,5]
se.cor.mat[i1-1,i2-1] <- coef[1,6]
score.mat[i1-1,i2-1] <- fitlr$score
if (data.return==0)
ud[[k]] <- list(index=c(i1,i2),left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]),fitlr=fitlr)
if (data.return==1)
ud[[k]] <- list(index=c(i1,i2),left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]),fitlr=fitlr,data=datalr)
if (i2==2) names1 <- c(names1, paste(cut1[i1-1],"-",cut1[i1]))
if (i1==2) names2 <- c(names2, paste(cut2[i2-1],"-",cut2[i2]))
thetal <- c(thetal,fitlr$theta)

if ((silent<=-1) & (iid==1)) print(head(fitlr$theta.iid));
if ((silent<=-1) & (iid==1)) {
print(idi) ; print(datalr$tsid)
print(dim(fitlr$theta.iid))
print(head(fitlr$theta.iid))
print(dim(theta.iid))
print(length( idi %in% unique(datalr$tsid)))
}
if (iid==1) theta.iid[idi %in% unique(datalr$tsid),k] <-c(fitlr$theta.iid)
###if (iid==1) theta.iid[rownames(fitlr$theta.iid),k] <-  fitlr$theta.iid
}

var.thetal <- NULL
if (iid==1)  var.thetal <- t(theta.iid) %*% theta.iid

colnames(score.mat) <- colnames(cor.mat) <-  colnames(se.cor.mat)  <- colnames(se.theta.mat) <- colnames(theta.mat) <- names1;
rownames(score.mat) <- rownames(cor.mat) <-  rownames(se.cor.mat) <-  rownames(se.theta.mat) <- rownames(theta.mat) <- names2;

ud <- list(model.fits=ud,theta=theta.mat,var.theta=se.theta.mat^2,
	   se.theta=se.theta.mat,thetal=thetal,thetal.iid=theta.iid,var.thetal=var.thetal,model=model,
	   cor=cor.mat,se.cor=se.cor.mat,score=score.mat);
class(ud)<-"pc.twostage"
attr(ud,"var.link")<-var.link;
attr(ud, "Type") <- model
return(ud);
} #}}}

##' @rdname survival-helpers
##' @export piecewise_data
piecewise_data <- function(cut1,cut2,data=parent.frame(),timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1)
{ #
ud <- list()
if (missing(cut2)) cut2 <- cut1;
nc1 <- length(cut1); nc2 <- length(cut2)
dataud <- c()

k <- 0;
for (i1 in 2:nc1)
for (i2 in 2:nc2)
{
k <-(i1-2)*(nc2-1)+(i2-1)
if (silent<=0) cat(paste("Data-set ",k,"out of ",(nc1-1)*(nc2-1)),"\n");
 datalr <- surv_boxarea(c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),data,timevar=timevar,
			status=status,id=id,covars=covars,covars.pairs=covars.pairs,num=num,silent=silent)
if (silent<=-1) print(summary(datalr));
if (silent<=-1) print(head(datalr));
datalr$tstime <- datalr[,timevar]
datalr$tsstatus <- datalr[,status]
datalr$tsid <- datalr[,id]
datalr$strata <- paste( c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),collapse=",",sep="-")
datalr$intstrata <-
c(paste(c(cut1[i1-1],cut1[i1]),collapse=",",sep="-"),paste( c(cut2[i2-1],cut2[i2]),collapse=",",sep="-"))

if (silent<=-1) print(head(datalr));
dataud <- rbind(dataud,datalr)
}

return(data.frame(dataud))
} #

##' @export
summary.pc.twostage <- function(object,var.link=NULL,...)
{ #
  if (!(inherits(object,"pc.twostage"))) stop("Must be a Piecewise constant two-Stage object")

  res <- list(estimates=object$theta,se=object$se.theta,cor=object$cor,se.cor=object$se.cor,
	      model=object$model,score=object$score)
  class(res) <- "summary.pc.twostage"
  attr(res,"var.link")<-attr(object,"var.link");
  attr(res, "Type") <- object$model
  res
} #

##' @export
print.pc.twostage <- function(x,var.link=NULL,...)
{ #
   if (!(inherits(x,"pc.twostage"))) stop("Must be a Piecewise constant two-Stage object")
   print( summary(x,var.link=var.link,...))
} #

##' @export
print.summary.pc.twostage <- function(x,var.link=NULL, digits=3,...)
{ #

  if (is.null(var.link)) { if (attr(x,"var.link")==1) vlink <- 1 else vlink <- 0; } else vlink <- var.link
  print(vlink)

  if (x$model=="plackett") cat("Dependence parameter for Plackett model \n");
  if (x$model=="clayton.oakes") cat("Dependence parameter for Clayton-Oakes model \n");

  if (max(x$score)>0.001) { cat("Score of log-likelihood for parameter estimates (too large?)\n"); print(x$score);cat("\n\n");}

  if (vlink==1) cat("log-coefficient for dependence parameter (SE) \n")  else cat("Dependence parameter (SE) \n");
  print(coefmat(x$estimate,x$se,digits=digits,...))
  cat("\n")

  if (x$model=="plackett") {cat("Spearman Correlation (SE) \n");cor.type <- "Spearman Correlation"; }
  if (x$model=="clayton.oakes") {cat("Kendall's tau (SE) \n"); cor.type <- "Kendall's tau";}

  print(coefmat(x$cor,x$se.cor,digits,...))
  cat("\n")
} #

coefmat <- function(est,stderr,digits=3,...) { #
  myest <- round(10^digits*(est))/10^digits;
  myest <- paste(ifelse(myest<0,""," "),myest,sep="")
  mysd <- round(10^digits*(stderr))/10^digits;
  res <- matrix(paste(format(myest)," (",format(mysd),")",sep=""),ncol=ncol(est))
  dimnames(res) <- dimnames(est)
  colnames(res) <- paste("",colnames(res))
  noquote(res)
} #


##' @export
sim_SurvFam <- function(n,beta=0.0,theta=1,lam0=0.5,lam1=1,lam2=1,ctime=10,...) { #
xm <- rbinom(n,1,0.5); xf <- rbinom(n,1,0.5);
xb1 <- rbinom(n,1,0.5); xb2 <- rbinom(n,1,0.5);
###
zf <- rgamma(n,shape=lam1); zb <- rgamma(n,shape=lam2);
tm <- rexp(n)/(zf*exp(xm*beta)*lam0)
tf <- rexp(n)/(zf*exp(xf*beta)*lam0)
tb1 <- rexp(n)/((zf+zb)*exp(xb1*beta)*2*lam0)
tb2 <- rexp(n)/((zf+zb)*exp(xb2*beta)*2*lam0)
cm <- ifelse(tm<ctime,1,0); cf <- ifelse(tf<ctime,1,0);
cb1 <- ifelse(tb1<ctime,1,0); cb2 <- ifelse(tb2<ctime,1,0);
tm <- ifelse(tm<ctime,tm,ctime); tf <- ifelse(tf<ctime,tf,ctime)
tb1 <- ifelse(tb1<ctime,tb1,ctime); tb2 <- ifelse(tb2<ctime,tb2,ctime)
#
data.frame(xm=xm,xf=xf,xb1=xb1,xb2=xb2,timem=tm,timef=tf,timeb1=tb1,timeb2=tb2,statusm=cm,statusf=cf,
	   statusb1=cb1,statusb2=cb2,id=1:n)
} #

###object.defined <- function(object)
###{
###   exists(as.character(substitute(object)))
###}

##' Twin and Family Random-Effects Design
##'
##' Functions for constructing random-effects design matrices for twin and
##' family models. These designs specify the genetic (A), dominance (D),
##' common environment (C), and unique environment (E) variance components.
##'
##' \code{twin_polygen_design} creates a polygenic random-effects design for
##' twin pairs, distinguishing MZ and DZ twins.
##'
##' \code{twin.polygen.design} is an alias for \code{twin_polygen_design}.
##'
##' \code{ace_family_design} creates designs for nuclear families (mother,
##' father, children).
##'
##' \code{make_pairwise_design} creates pairwise random-effects designs for
##' arbitrary kinship structures.
##'
##' \code{concordanceTwostage} computes concordance probabilities from a
##' twostage model.
##'
##' \code{concordanceTwinACE} computes concordance from a twin ACE model.
##'
##' \code{kendall_ClaytonOakes_twin_ace} and \code{kendall_normal_twin_ace}
##' compute Kendall's tau for Clayton-Oakes and normal-frailty twin ACE models
##' respectively.
##'
##' \code{ascertained_pairs} identifies ascertained (affected) pairs in
##' clustered survival data.
##'
##' \code{p11_binomial_twostage_RV} computes the joint probability P(T1<=t, T2<=t)
##' for the additive gamma binary random effects model.
##'
##' @name twin-design
##' @param data a data.frame with twin/family data.
##' @param id character name of the cluster (pair) identifier column.
##' @param type model type: \code{"ace"}, \code{"ade"}, \code{"ae"}, \code{"de"},
##'   \code{"dce"}, or \code{"un"}.
##' @param ... additional arguments.
##' @return A list with components:
##'   \item{pardes}{parameter design matrix linking random effects to variance parameters.}
##'   \item{des.rv}{random-effects design matrix for subjects.}
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases twin_polygen_design twin.polygen.design ace_family_design
##' @aliases make_pairwise_design ascertained_pairs
##' @aliases concordanceTwostage concordanceTwinACE
##' @aliases p11_binomial_twostage_RV p11.binomial.twostage.RV
##' @aliases kendall_ClaytonOakes_twin_ace kendall.ClaytonOakes.twin.ace
##' @aliases kendall_normal_twin_ace
NULL

##' @export
twin_polygen_design <-function (data,id="id",zyg="DZ",zygname="zyg",type="ace",tv=NULL,...) { #
  ### twin case
  id <- data[,id]
  tv <- diff(c(NA,id))
  tv[tv!=0 | is.na(tv)] <- 1
  tv[tv==0] <- 2

  zygbin <- (data[,zygname]==zyg)*1
  zygdes=model.matrix(~-1+factor(zygbin),data)
  n <- length(zygbin)

  if (type=="ace") { ### ace #
  ### random effects for each cluster
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,1)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2","env")
  pard <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0), c(0,1))
  } #

  if (type=="ae") { ### ae #
  ###AE model
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2")
  pard <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0))[,1,drop=FALSE]
  } #

  if (type=="dce") { ### dce #
  ### DCE
  ### random effects for each cluster
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,1)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2","env")
  pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0), c(0,1))
  } #

  if (type=="ade") { ### ade #
  #ADE
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,DZns,1)
  pard <- rbind(c(1,0),c(0.25,0),c(0.75,0),c(0.75,0),c(0,1),c(0,0.5),c(0,0.5),c(0,0.5) )
  pardes <- matrix(pard,n,16,byrow=TRUE)
  des.rv <- NULL
  } #

  if (type=="adce") { ### adce #
  } #

  if (type=="de") { ### ae #
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2")
  pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0))[,1,drop=FALSE]
  } #

  if (type=="un") { ### ae #
  des.rv <- cbind(zygdes)
  colnames(des.rv) <- c("MZ","DZ")
  pard <- rbind(c(1,0),c(0,1))
  } #

res <- list(pardes=pard,des.rv=des.rv)
return(res)
} #


##' @rdname twin-design
##' @export
twin.polygen.design <-function (x,...) twin_polygen_design(x,...) 

##' @rdname twin-design
##' @param member character name of the family member type column.
##' @param mother value identifying mothers in the member column.
##' @param father value identifying fathers in the member column.
##' @param child value identifying children in the member column.
##' @param child1 column name distinguishing first child from second.
##' @export
ace_family_design <-function (data,id="id",member="type",mother="mother",father="father",child="child",child1="child",type="ace",...) {
#
  ### standard family case
###  nid <- table(data[,id])
  id <- data[,id]

  if (type=="ace") { ### ace #
  ### random effects for each cluster
	  mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2,1)

	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4","env")
	  pard <- rbind( c(0.25,0), c(0.25,0),c(0.25,0), c(0.25,0),
			 c(0.25,0), c(0.25,0),c(0.25,0), c(0.25,0), c(0,1))
  } #

  if (type=="ae") { ### ae #
  ###AE model
          mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2)
	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4")
	  pard <- rbind( c(0.25), c(0.25),c(0.25), c(0.25),
			 c(0.25), c(0.25),c(0.25), c(0.25))
  } #

  if (type=="dce") { ### dce #
  ### DCE
  ### random effects for each cluster
	  stop("not done yet");
          mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2)
	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4")
	  pard <- rbind( c(0.25), c(0.25),c(0.25), c(0.25),
			 c(0.25), c(0.25),c(0.25), c(0.25))

  } #

  if (type=="ade") { ### ade #
  #ADE
    stop("not done yet");
###    pard <- rbind(c(1,0),c(0.25,0),c(0.75,0),c(0.75,0),c(0,1),c(0,0.5),c(0,0.5),c(0,0.5) )
###    pardes <- matrix(pard,n,16,byrow=TRUE)
  } #

  if (type=="adce") { ### adce #
	  stop("not done yet");
  } #

  if (type=="de") { ### ae #
    stop("not done yet");
    pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0))[,1,drop=FALSE]
  } #

  if (type=="un") { ### ae #
	 stop("not done yet");
         pard <- diag(4)
  } #

res <- list(pardes=pard,des.rv=des.rv)
return(res)
} #

##' @rdname twin-design
##' @param pairs matrix of pair indices (n x 2).
##' @param kinship vector of kinship coefficients for each pair.
##' @export
make_pairwise_design  <- function(pairs,kinship,type="ace")
{ #
### makes pairwise random effects design for shared and non-shared random effects
### kinship gives shared genes for each pair

uk <- sort(unique(kinship))
iuk <- fast.approx(uk,kinship)

if (type=="ace")  {
   theta.des <-  matrix(0,length(uk),8)
   i <- 0
   for (uuk in uk)  {
     i <- i+1
     theta.des[i,] <-c( rbind(c(uuk,0), c(1-uuk,0), c(1-uuk,0), c(0,1)))
   }
   random.des <- rbind(c(1,1,0,1),c(1,0,1,1))
   rvs <- rep(4,length(kinship))
   new.pairs <- cbind(pairs,1,2,iuk,4)
}

if (type=="ae") {
  i <- 0
  theta.des <-  matrix(0,length(uk),3)
  for (uuk in uk)  {
    i <- i+1
    theta.des[i,] <- rbind(c(uuk), c(1-uuk), c(1-uuk))
  }
  random.des <- rbind(c(1,1,0),c(1,0,1))
  rvs <- rep(3,length(kinship))
  new.pairs <- cbind(pairs,1,2,iuk,3)
}

if (type=="ade") {
   stop(" not specified yet \n")
   theta.des <-  matrix(0,length(uk),8)
   i <- 0
   for (uuk in uk)  {
      i <- i+1
      theta.des[i,] <- rbind(c((uuk==1)+(uuk!=1)*uuk*0.5,0), c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),
			c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),c(0,1))
   }
   random.des <- rbind(c(1,1,0,1),c(1,0,1,1))
   rvs <- rep(4,length(kinship))
   new.pairs <- cbind(pairs,1,2,iuk,4)
}

if (type=="ad") {
   stop(" not specified yet \n")
   i <- 0
   for (uuk in uk)  {
   i <- i+1
   theta.des[i,] <- rbind(c((uuk==1)+(uuk!=1)*uuk*0.5,0), c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),
			c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),c(0,1))
   random.des <- rbind(c(1,1,0),c(1,0,1))
   new.pairs <- cbind(pairs,1,2,iuk,2)
   }
}

return(list(new.pairs=new.pairs,theta.des=theta.des,random.design=random.des))
} #


EVaddGam <- function(theta,x1,x2,thetades,ags)
{ #
	pars <- thetades %*% theta
	lamtot <- ags %*% theta

	mvar <- pars/lamtot
	vvar <- pars/lamtot^2

	x1mvar <- sum(x1 * mvar)
	x2mvar <- sum(x2 * mvar)
	x1x2vvar <- sum(x1*x2*vvar)

	list(x1m=x1mvar,mx2=x2mvar,
	     dN=x1x2vvar/x2mvar)
} #

##' @export
twostage.aalen <- function(object,...) survival_twostage(object,...)

##' @export
twostage.cox.aalen <- function(object,...) survival_twostage(object,...)

##' @export
twostage.coxph <- function(object,...) survival_twostage(object,...)

##' @export
twostage.phreg <- function(object,...) survival_twostage(object,...)

##' Generate Random Fold Indices for Cross-Validation
##'
##' Splits \code{n} observations into random folds for cross-validation.
##'
##' @param n number of observations.
##' @param folds number of folds (default 10).
##' @return A list of integer vectors, each containing indices for one fold.
##' @export
folds<- function (n, folds = 10) 
{ ## {{{ 
   split(sample(1:n), rep(1:folds, length = n))
} ## }}} 

##' Split a data set and run function 
##'
##' @title Split a data set and run function 
##' @param func called function
##' @param data data-frame
##' @param size size of splits
##' @param splits number of splits (ignored if size is given)
##' @param id optional cluster variable
##' @param ... Additional arguments to lower level functions
##' @author Thomas Scheike, Klaus K. Holst
##' @export
##' @examples
##' ## avoid dependency on timereg
##' ## library(timereg)
##' ## data(TRACE)
##' ## res <- divide_conquer(prop.odds,TRACE,
##' ## 	     formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=200)
divide_conquer <- function(func=NULL,data,size,splits,id=NULL,...)
{ ## {{{
    nn <- nrow(data)
    if (!is.null(id)) {
        if (is.character(id) && length(id)==1) id <- data[,id]
        if (length(id)!=nn) stop("Wrong length of id variable")
        cc <- cluster_index(id)
        if (!missing(size)) splits <- round(cc$uniqueclust/size)
        splits <- min(splits,cc$uniqueclust)
        all.folds <- folds(cc$uniqueclust,splits)
        res <- lapply(all.folds, function(i) {
            idx <- na.omit(as.vector(cc$idclustmat[i,]+1))
            do.call(func, c(list(data=data[idx,]),...))
        })        
        return(res)
    }    
    splits <- round(nn/size)
    all.folds <- folds(nn,splits)
    res <- lapply(all.folds, function(i) 
	do.call(func, c(list(data=data[i,]),...)))
    res
} ## }}} 

##' Calculate summary statistics grouped by variable
##'
##' Calculate summary statistics grouped by
##' @title Calculate summary statistics grouped by
##' @param data Data.frame
##' @param INPUT Input variables (character or formula)
##' @param ... functions
##' @param ID id variable
##' @param ORDER (optional) order variable
##' @param SUBSET (optional) subset expression
##' @param SORT sort order (id+order variable)
##' @param COMBINE If TRUE result is appended to data
##' @param NOCHECK No sorting or check for missing data
##' @param ARGS Optional list of arguments to functions (...)
##' @param NAMES Optional vector of column names
##' @param COLUMN If TRUE do the calculations for each column
##' @param REDUCE Reduce number of redundant rows
##' @param REGEX Allow regular expressions
##' @param ALL if FALSE only the subset will be returned
##' @export
##' @author Klaus K. Holst and Thomas Scheike
##' @details
##' dby2 for column-wise calculations
##' @aliases dby dby<- dby2 dby2<- dbyr
##' @examples
##' n <- 4
##' k <- c(3,rbinom(n-1,3,0.5)+1)
##' N <- sum(k)
##' d <- data.frame(y=rnorm(N),x=rnorm(N),
##'    id=rep(seq(n),k),num=unlist(sapply(k,seq))
##' )
##' d2 <- d[sample(nrow(d)),]
##'
##' dby(d2, y~id, mean)
##' dby(d2, y~id + order(num), cumsum)
##'
##' dby(d,y ~ id + order(num), dlag)
##' dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2))
##' dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2), NAMES=c("l1","l2"))
##'
##' dby(d, y~id + order(num), mean=mean, csum=cumsum, n=length)
##' dby(d2, y~id + order(num), a=cumsum, b=mean, N=length,
##'   l1=function(x) c(NA,x)[-length(x)]
##' )
##'
##' dby(d, y~id + order(num), nn=seq_along, n=length)
##' dby(d, y~id + order(num), nn=seq_along, n=length)
##'
##' d <- d[,1:4]
##' dby(d, x<0) <- list(z=mean)
##' d <- dby(d, is.na(z), z=1)
##'
##' f <- function(x) apply(x,1,min)
##' dby(d, y+x~id, min=f)
##'
##' dby(d,y+x~id+order(num), function(x) x)
##'
##' f <- function(x) { cbind(cumsum(x[,1]),cumsum(x[,2]))/sum(x)}
##' dby(d, y+x~id, f)
##'
##' ## column-wise
##' a <- d
##' dby2(a, mean, median, REGEX=TRUE) <- '^[y|x]'~id
##' a
##' ## wildcards 
##' dby2(a,'y*'+'x*'~id,mean) 
##'
##'
##' ## subset
##' dby(d, x<0) <- list(z=NA)
##' d
##' dby(d, y~id|x>-1, v=mean,z=1)
##' dby(d, y+x~id|x>-1, mean, median, COLUMN=TRUE)
##'
##' dby2(d, y+x~id|x>0, mean, REDUCE=TRUE)
##'
##' dby(d,y~id|x<0,mean,ALL=FALSE)
##'
##' a <- iris
##' a <- dby(a,y=1)
##' dby(a,Species=="versicolor") <- list(y=2)
dby <- function(data,
                INPUT,
                ...,
                ID=NULL,
                ORDER=NULL,
                SUBSET=NULL,
                SORT=0,
                COMBINE=!REDUCE,
                NOCHECK=FALSE,
                ARGS=NULL,
                NAMES,
                COLUMN=FALSE,
                REDUCE=FALSE,
                REGEX=mets.options()$regex,
                ALL=TRUE) {
    if (missing(INPUT)) INPUT <- .~1
    val <- substitute(INPUT)
    INPUT <- try(eval(val),silent=TRUE)
    funs <- substitute(list(...))[-1]
    if (inherits(INPUT,"try-error")) {
        INPUT <- as.formula(paste0(".~1|",deparse(val)))        
    }
    if (inherits(INPUT,c("formula","character"))) {
        INPUT <- procformdata(INPUT,sep="\\|",data=data,na.action=na.pass,do.filter=FALSE,regex=REGEX,specials="order")
        if (is.null(ID)) ID <- INPUT$predictor
        if (is.null(SUBSET)) {
            if (length(INPUT$group)!=0)
                SUBSET <- INPUT$group[[1]]
        }
        if (is.null(ORDER)) {
            if (length(INPUT$specials$order)!=0)
                ORDER <- INPUT$specials$order[[1]]
        }
        INPUT <- INPUT$response
    }

    if (inherits(ID,"formula")) ID <- model.frame(ID,data=data,na.action=na.pass)
    if (inherits(ORDER,"formula")) ORDER <- model.frame(ORDER,data=data,na.action=na.pass)
    if (inherits(SUBSET,"formula")) SUBSET <- model.frame(SUBSET,data=data,na.action=na.pass)
    if (is.null(INPUT)) {
        INPUT <- ID
        ID <- NULL
    }
    noID <- FALSE
    if (length(ID)==0) {
        noID <- TRUE
        ID = rep(1,NROW(INPUT))
    }
    group <- NULL
    if (!COMBINE || length(funs)==0) group <- ID
    if (NCOL(ID)>1) ID <- interaction(ID)
    if (is.data.frame(ID)) {
        ID <- as.numeric(ID[,1,drop=TRUE])
    } else {
        ID <- as.numeric(ID)
    }
    if (is.data.frame(ORDER)) {
        ORDER <- ORDER[,1,drop=TRUE]
    }
    if (!NOCHECK) {
        if (length(ORDER)>0) {
            ord <- order(ID,ORDER,decreasing=SORT,method="radix")
        } else {
            ord <- order(ID,decreasing=SORT,method="radix")
        }
        numerics <- unlist(lapply(INPUT[1,],is.numeric))
        INPUT <- as.matrix(INPUT[ord,which(numerics),drop=FALSE])
        ID <- ID[ord]
        if (is.null(group)) data <- data[ord,,drop=FALSE]
        else {
            if (is.vector(group)) group <- group[ord]
            else group <- group[ord,,drop=FALSE]
        }
        na.idx <- which(is.na(ID))
    } else {
        INPUT <- as.matrix(INPUT)
    }

    if (length(SUBSET)>0) {
        SUBSET <- which(as.matrix(SUBSET))
        INPUT <- INPUT[SUBSET,,drop=FALSE]
        ID <- ID[SUBSET]
    }
    if (length(funs)==0) {
        if (noID) return(INPUT)
        return(cbind(INPUT,group))
    }

    if (NROW(INPUT)>0) {
    resl <- lapply(funs, function(fun_) {
        env <- new.env()
        isfun <- tryCatch(is.function(eval(fun_)),error=function(...) FALSE)
        if (isfun) {
            env$fun_ <- eval(fun_)
            if (length(ARGS)>0) {
                fun_ <- eval(fun_)
                ff <- function(...) do.call(fun_,c(list(...),ARGS))
                expr <- quote(ff(x))
            } else {
                expr <- quote(fun_(x))
            }
        } else {
            expr <- fun_
        }
        x <- .ApplyBy2(INPUT, ID, F = expr, Env = env, Argument = "x", Columnwise = COLUMN)
        return(structure(x$res, clustersize = x$clustersize))
    })
    res <- Reduce(cbind,resl)
    } else {
        resl <- NULL
        res <- NULL
    }

    dn <- NULL
    ## Setting column names
    if (missing(NAMES)) {
        a <- match.call(expand.dots=FALSE)
        ff <- lapply(a[["..."]],deparse)
        fn <- lapply(ff,deparse)
        dn <- names(ff)
        if (is.null(dn)) dn <- rep("",length(ff))
        idx <- which(dn=="")
        if (length(idx)>0) {
            newn <- unlist(ff[idx])
            dn[idx] <- newn[seq_along(idx)]
            fcall <- grepl("^function",dn)
            if (any(fcall))
                dn[which(fcall)] <- which(fcall)
        }
        numcolf <- unlist(lapply(resl, NCOL))
        nn <- c()
        if (COLUMN) {
            colnames(INPUT)
            dn <- unlist(lapply(dn, function(x) paste(x,colnames(INPUT),sep=".")))
        }
        for (i in seq_along(resl)) {
            nc <- numcolf[i]
            curnam <- dn[i]
            if (COLUMN) {
                nc <- nc/NCOL(INPUT)
                pos <- NCOL(INPUT)*(i-1)+1
                pos <- seq(pos,pos+NCOL(INPUT)-1)
                curnam <- dn[pos]
            }
            if (nc>1) {
                nn <- c(nn, unlist(lapply(curnam, function(x) paste0(x,seq(nc)))))
            }
            else nn <- c(nn,curnam)
        }
        NAMES <- nn
    }

    try(colnames(res) <- NAMES,silent=TRUE)
    numidx <- grep("^[0-9]",colnames(res)) ## column names starting with digit
    if (length(numidx)>0)
       colnames(res)[numidx] <- paste0("_",colnames(res)[numidx])

    if (!NOCHECK && length(na.idx)>0) {
        res[na.idx,] <- NA
    }
    cl <- attr(resl[[1L]],"clustersize")
    if (COMBINE) {
        if (is.null(NAMES) & is.null(res) & length(dn)>0) {
            res <- matrix(NA,nrow(data),length(dn))
            colnames(res) <- dn
        }        
        nn <- colnames(res)
        nc <- ifelse(is.null(res), 0, ncol(res))
        res0 <- matrix(NA,nrow(data),nc)
        colnames(res0) <- nn
        didx <- which(colnames(data)%in%nn)
        if (length(didx)>0) {
            ridx <- match(colnames(data)[didx],nn)
            res0[,ridx] <- as.matrix(data[,didx])
            data <- data[,-didx,drop=FALSE]
        }
        if (length(SUBSET)>0) {
            res0[SUBSET,] <- res
            res <- cbind(data,res0)
        } else {
            if (!is.null(res))
                res <- cbind(data,res)
            else
                res <- data
        }

    } else {
        if (!noID) {
            if (length(SUBSET)>0) group <- group[SUBSET,,drop=FALSE]
            res <- cbind(group,res)
        }
    }
    if (REDUCE!=0) {
        if (REDUCE<0) { # Grab the last observation
            idx <- cumsum(cl)
        } else { # Grab the first
            idx <- cumsum(c(1,cl[-length(cl)]))
        }
        res <- res[unique(idx),,drop=FALSE]
        if (NROW(res)==1) {
            rownames(res) <- ""
        } else {
            rownames(res) <- NULL
        }
    }
    if (!ALL) {
        return(res[SUBSET,,drop=FALSE])
    }
    return(res)
}


##' @export
"dby<-" <- function(data,INPUT,...,value) {
    cl <- match.call()
    cl[[1L]] <- substitute(dby)
    a <- substitute(value)
    if (inherits(value,"formula")) {
        cl["value"] <- NULL
        names(cl)[names(cl)=="INPUT"] <- ""
        cl[["INPUT"]] <- value
    } else {
        if (is.list(value)) {
            cl[which(names(cl)=="value")] <- NULL
            start <- length(cl)
            for (i in seq_along(value)) {
                cl[start+i] <- value[i]
            }
            if (length(names(value))>0)
                names(cl)[start+seq_along(value)] <- names(value)
        } else {
            names(cl)[which(names(cl)=="value")] <- ""
        }
    }
    eval.parent(cl)
}

##' @export
dby2 <- function(data,INPUT,...) {
    cl <- match.call()
    cl[[1L]] <- substitute(dby)
    cl[["COLUMN"]] <- TRUE
    eval.parent(cl)
}

##' @export
"dby2<-" <- function(data,INPUT,...,value) {
    cl <- match.call()
    cl[[1L]] <- substitute(`dby<-`)
    cl[["COLUMN"]] <- TRUE
    eval.parent(cl)
}

##' @export
dbyr <- function(data,INPUT,...,COLUMN=FALSE) {
    cl <- match.call()
    cl[[1L]] <- substitute(`dby`)
    cl[["REDUCE"]] <- TRUE
    cl[["COLUMN"]] <- COLUMN
    eval.parent(cl)
}

# dutils
library("tinytest")

test_dsort <- function() {
  data("hubble",package="lava")
  expect_equivalent(
              order(dsort(hubble, ~sigma)$sigma),
              seq_len(nrow(hubble)))
}
test_dsort()

test_dsort2 <- function() {
  data("hubble",package="lava")
  h1 <- hubble
  drename(h1,fun=toupper) <- ~.
  expect_equivalent(colnames(h1),
                              toupper(names(hubble)))
}
test_dsort2()

test_daggregate <- function() {
  dd <- data.frame(a = 1:20, b = 20:1,
                   g1 = rep(0:1, 10),
                   g2 = rep(0:1, each = 10),
                   g3 = rbinom(20, 1, 0.5))
  dd$g1[1:2] <- NA
  dd$g2[2:3] <- NA
  dd$a[3:7] <- NA
  ##dcor(dd,.~g1+g2)

  r1 <- dcor(dd, "[acg][12]*$", regex=TRUE, use="pairwise")
  r2 <- cor(dd[,c("a","g1","g2")], use="pairwise")
  expect_equal(norm(r1-r2), 0)
  r1 <- dcor(dd, "[ab]",subset=is.na(g1), regex=TRUE, use="pairwise")
  dd0 <- subset(dd, is.na(g1))
  r2 <- cor(dd[, c("a","b")], use="pairwise")
  expect_equal(norm(r1-r2), 0)
  suppressWarnings(r1 <- dcor(dd, ~.|is.na(g1)|is.na(g2))) # do not warn about zero sd
  dd0 <- subset(dd, is.na(g1) | is.na(g2), select=-c(g1,g2))
  suppressWarnings(r2 <- dcor(dd0)) # do not warn about zero sd
  expect_equal(r1,r2)
  r1 <- dcor(dd, use="pairwise")
  expect_equal(norm(r1-cor(dd, use="pairwise")), 0)
}
test_daggregate()


test_dby <- function() {
  dd <- dby2(iris, . ~ Species, mean, median, REDUCE=T)
  val <- dreshape(dd,varying=list(mean="mean*",median="median*"),dropid=TRUE)
  val$num <- gsub("mean.","",val$num)

  val <- dreshape(dd,varying=c("mean","median"),dropid=TRUE)
  val$num <- gsub("^.","",val$num)
    
  expect_true(ncol(val)==4)
  expect_true(nrow(val)==3*4)
  val0 <- subset(val, Species=="setosa" & num=="Sepal.Width")
  val1 <- subset(iris, Species=="setosa", select="Sepal.Width")[,1]
  expect_true(abs(val0[1,"mean"]-mean(val1))<1e-16)
  expect_true(abs(val0[1,"median"]-median(val1))<1e-16)
}
test_dby()

test_dsample <- function() {
  n <- 10
  d <- data.frame(id=rep(0:1,5),x=seq(0,1,length.out=10),y=seq(1,0,length.out=10),
                  id2=rep(1:5,each=2))
  d1 <- dsample(d, ~id, size=3)
  expect_true(ncol(d1)==5)
  expect_true(nrow(d1)==15)

  d2 <- dsample(d, .~id|x>0.5, size=3)
  expect_true(ncol(d2)==4)
  expect_true(all(d2$x>0))

  d3 <- dsample(d, .~id+id2, size=3)
  expect_true(ncol(d3)==3)

  d4 <- dsample(d, .~id+id2|x>0, size=3)
  expect_true(ncol(d4)==3)
  expect_true(all(d4$x>0))
}
test_dsample()

context("dutils")

test_that("dsort", {
    data("hubble",package="lava")
    expect_equivalent(order(dsort(hubble, ~sigma)$sigma),
                      seq_len(nrow(hubble)))
})

test_that("dsort", {
    data("hubble",package="lava")
    h1 <- hubble
    drename(h1,fun=toupper) <- ~.    
    expect_equivalent(colnames(h1),toupper(names(hubble)))
})


test_that("daggregate", {    
    dd <- data.frame(a=1:20,b=20:1,
                    g1=rep(0:1,10),
                    g2=rep(0:1,each=10),
                    g3=rbinom(20,1,0.5))
    dd$g1[1:2] <- NA
    dd$g2[2:3] <- NA
    dd$a[3:7] <- NA
    ##dcor(dd,.~g1+g2)

    r1 <- dcor(dd, "[acg][12]*$", regex=TRUE, use="pairwise")
    r2 <- cor(dd[,c("a","g1","g2")], use="pairwise")
    expect_equal(norm(r1-r2), 0)
    r1 <- dcor(dd, "[ab]",subset=is.na(g1), regex=TRUE, use="pairwise")
    dd0 <- subset(dd, is.na(g1))
    r2 <- cor(dd[, c("a","b")], use="pairwise")
    expect_equal(norm(r1-r2), 0)
    suppressWarnings(r1 <- dcor(dd, ~.|is.na(g1)|is.na(g2))) # do not warn about zero sd
    dd0 <- subset(dd, is.na(g1) | is.na(g2), select=-c(g1,g2))
    suppressWarnings(r2 <- dcor(dd0)) # do not warn about zero sd
    expect_equal(r1,r2)
    r1 <- dcor(dd, use="pairwise")
    expect_equal(norm(r1-cor(dd, use="pairwise")), 0)
})

test_that("dby", {
    dd <- dby2(iris, . ~ Species, mean, median, REDUCE=T)
    val <- dreshape(dd,varying=list(mean="mean*",median="median*"),dropid=TRUE)
    val$num <- gsub("mean.","",val$num)

    val <- dreshape(dd,varying=c("mean","median"),dropid=TRUE)
    val$num <- gsub("^.","",val$num)
    
    expect_true(ncol(val)==4)
    expect_true(nrow(val)==3*4)
    val0 <- subset(val, Species=="setosa" & num=="Sepal.Width")
    val1 <- subset(iris, Species=="setosa", select="Sepal.Width")[,1]
    expect_true(abs(val0[1,"mean"]-mean(val1))<1e-16)
    expect_true(abs(val0[1,"median"]-median(val1))<1e-16)
})

test_that("dsample", {
    n <- 10
    d <- data.frame(id=rep(0:1,5),x=seq(0,1,length.out=10),y=seq(1,0,length.out=10),
                   id2=rep(1:5,each=2))
    d1 <- dsample(d, ~id, size=3)
    expect_true(ncol(d1)==5)
    expect_true(nrow(d1)==15)

    d2 <- dsample(d, .~id|x>0.5, size=3)
    expect_true(ncol(d2)==4)
    expect_true(all(d2$x>0))
    
    d3 <- dsample(d, .~id+id2, size=3)
    expect_true(ncol(d3)==3)

    d4 <- dsample(d, .~id+id2|x>0, size=3)
    expect_true(ncol(d4)==3)
    expect_true(all(d4$x>0))

    
})

##' Non-parametric Cumulative Incidence Functions
##'
##' Functions for computing and visualizing non-parametric cumulative incidence
##' estimates, as well as dependence measures (odds ratio, relative risk) for
##' bivariate competing risks data.
##'
##' \code{npc} computes bivariate non-parametric cumulative incidence using
##' inverse-probability-of-censoring weights.
##'
##' \code{nonparcuminc} computes univariate non-parametric cumulative incidence
##' for multiple causes.
##'
##' \code{plotcr} plots cumulative incidence curves for competing risks using
##' the prodlim package.
##'
##' \code{or_cif} fits an odds-ratio model for bivariate cumulative incidence.
##'
##' \code{rr_cif} fits a relative-risk model for bivariate cumulative incidence.
##'
##' \code{random.cif} and \code{Grandom.cif} are aliases for \code{random_cif}
##' and \code{Grandom_cif} (random effects CIF models).
##'
##' \code{predictPairPlack} predicts pairwise joint probabilities under a
##' Plackett (odds-ratio) dependence model.
##'
##' \code{matplot.mets.twostage} produces matrix-plots of concordance over time
##' from a twostage object.
##'
##' @name cif-nonpar
##' @param T matrix with columns: time1, time2, status1, status2 (for \code{npc}).
##' @param cause vector of length 2 specifying causes of interest (for \code{npc}).
##' @param same.cens logical; if TRUE, uses joint censoring weights.
##' @param sep logical; if TRUE, uses separate censoring models for each subject.
##' @param cif a cumulative incidence model object (from timereg).
##' @param data a data.frame with the variables.
##' @param cif2 optional second CIF model if different from first.
##' @param times time points for evaluation.
##' @param cause1 cause for first coordinate.
##' @param cause2 cause for second coordinate.
##' @param cens.code censoring code value.
##' @param cens.model censoring model type (default \code{"KM"}).
##' @param Nit maximum number of iterations.
##' @param detail level of output detail.
##' @param clusters cluster variable name or vector.
##' @param theta dependence parameter(s).
##' @param theta.des design matrix for theta.
##' @param step step size for optimization.
##' @param sym if 1, symmetric dependence structure.
##' @param weights optional weights.
##' @param censoring.weights optional pre-computed censoring weights.
##' @param silent verbosity level.
##' @param par.func optional parameter function.
##' @param dpar.func optional derivative of parameter function.
##' @param dimpar dimension of parameter vector.
##' @param score.method optimization method (default \code{"nlminb"}).
##' @param entry optional entry time variable.
##' @param estimator estimator type.
##' @param trunkp truncation probability.
##' @param admin.cens administrative censoring time.
##' @param cif1 CIF values for subject 1 (for \code{predictPairPlack}).
##' @param status1 status for subject 1.
##' @param status2 status for subject 2.
##' @param x data matrix or competing risks object.
##' @param ... additional arguments.
##' @return For \code{npc}: matrix with columns (time, cumulative incidence).
##'   For \code{nonparcuminc}: matrix with time and cause-specific cumulative incidences.
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases npc nonparcuminc plotcr or_cif rr_cif random.cif Grandom.cif
##' @aliases predictPairPlack
NULL

##' @rdname cif-nonpar
##' @export
npc <- function(T,cause,same.cens=TRUE,sep=FALSE) {
  mtime <- apply(T[,1:2],1,max)
  ot <- order(mtime)
  mtime <- mtime[ot]
  T <- T[ot,]
  if (!sep) {
    time1 <- as.vector(T[,1:2]); status1 <- as.vector(T[,3:4])
    ud.cens1<-survival::survfit(Surv(time1,status1==0)~+1);
    Gfit1<-cbind(ud.cens1$time,ud.cens1$surv)
    Gfit2 <- Gfit1<-rbind(c(0,1),Gfit1);
  } else {
    time1 <- as.vector(T[,1]); status1 <- as.vector(T[,3])
    ud.cens1<-survival::survfit(Surv(time1,status1==0)~+1);
    time2 <- as.vector(T[,2]); status2 <- as.vector(T[,4])
    ud.cens2<-survival::survfit(Surv(time2,status2==0)~+1);
    Gfit1<-cbind(ud.cens1$time,ud.cens1$surv)
    Gfit1<-rbind(c(0,1),Gfit1);
    Gfit2<-cbind(ud.cens2$time,ud.cens2$surv)
    Gfit2<-rbind(c(0,1),Gfit2);
  }
  i1 <- fast.approx(Gfit1[,1],T[,1])
  cweights1<-fast.approx(,Gfit1[,2])[[1]]
  cweights2<-fast.approx(Gfit2[,1],T[,2],Gfit2[,2])[[1]];
  weight11 <- apply(cbind(cweights1,cweights2),1,min)

  if (same.cens) {
    conc <- (T[,3]==cause[1])*(T[,4]==cause[2])/weight11
  } else {
    conc <-(T[,3]==cause[1])*(T[,4]==cause[2])/(cweights1*cweights2);
  }
  mtime <- mtime[!is.na(conc)]
  conc <- conc[!is.na(conc)]
  cbind(mtime,cumsum(conc)/length(conc))
}

##' @rdname cif-nonpar
##' @param t vector of event/censoring times (for \code{nonparcuminc}).
##' @param status vector of status codes (for \code{nonparcuminc}).
##' @param cens censoring code (default 0).
##' @export
nonparcuminc <- function(t,status,cens=0) {
  ord <- order(t); t <- t[ord]; status <- status[ord]
  ud.cens<-survival::survfit(Surv(t,status==cens)~1)
  Gfit<-cbind(ud.cens$time,ud.cens$surv)
  Gfit<-rbind(c(0,1),Gfit);
  causes <- setdiff(unique(status),cens)
  cweight<-fast.approx(Gfit[,1],t,Gfit[,2])[[1]];
  cc <- t
  for (i in 1:length(causes)) {
    c1 <- status==causes[i]
    cc <- cbind(cc,cumsum(c1/cweight)/length(c1))
  }
  return(cc)
}

##' @export
`lifetable` <- function(x,...) UseMethod("lifetable")

##' Create simple life table 
##' 
##' @title Life table
##' @param x time formula (Surv) or matrix/data.frame with columns time,status or entry,exit,status
##' @param strata strata
##' @param data data.frame
##' @param breaks time intervals
##' @param weights weights variable
##' @param confint if TRUE 95\% confidence limits are calculated
##' @param ... additional arguments to lower level functions
##' @author Klaus K. Holst
##' @aliases lifetable lifetable.matrix lifetable.formula
##' @usage
##'  \method{lifetable}{matrix}(x, strata = list(), breaks = c(),
##'    weights=NULL, confint = FALSE, ...)
##'
##'  \method{lifetable}{formula}(x, data=parent.frame(), breaks = c(),
##'    weights=NULL, confint = FALSE, ...)
##' @examples
##' library(timereg)
##' data(TRACE)
##' 
##' d <- with(TRACE,lifetable(Surv(time,status==9)~sex+vf,breaks=c(0,0.2,0.5,8.5)))
##' lava::estimate(glm(events ~ offset(log(atrisk))+factor(int.end)*vf + sex*vf,
##'             data=d,poisson))
##' @export
lifetable.matrix <- function(x,strata=list(),breaks=c(),weights=NULL,confint=FALSE,...) {
    if (ncol(x)==3) {
        status <- x[,3]
        entry <- x[,1]
        time <- x[,2]
    } else {
        status <- x[,2]
        time <- x[,1]
        entry <- rep(0,length(time))
    }
    LifeTable(time,status,entry,strata=strata,breaks=breaks,weights=weights,confint,...)
}

##' @export
lifetable.formula <- function(x,data=parent.frame(),breaks=c(),weights=NULL,confint=FALSE,...) {
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("x", "data", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)
    names(mf)[which(names(mf)=="x")] <- "formula"    
    mf <- eval(mf, parent.frame())
    weights <- as.vector(model.weights(mf))    
    ##mf <- model.frame(x,data)
    Y <- model.extract(mf, "response")
    Terms <- terms(x, data = data)
    if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    strata <- list()
    X <- model.matrix(Terms, data)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)>0) {
        strata <- as.list(model.frame(Terms,data)[,-1,drop=FALSE])
    }
    LifeTable(exit,status,entry,
              strata=strata,breaks=breaks,weights=weights,confint=confint,...)       
}

## lifetable.data.table <- function(x,entry,exit,status,strata,breaks,...) {
##     requireNamespace("data.table")
##     breaks <- sort(unique(breaks))
##     nbreaks <- length(breaks)
##     ## prs <- parse(text=paste0("pmin(x,",exit,")-pmax(x-1,",entry,")"))
##     ## dt[,eval(prs),by=strata]
##     system.time(dur1 <- x[,
##                            lapply(breaks[-1],function(x)
##                               eval(parse(text=paste0("pmin(x,",exit,")-pmax(x-1,",entry,")")))
##                                  ),by=strata])
##     dur1[dur1<0] <- NA
##     system.time(endur1 <- x[,
##                             lapply(breaks[-nbreaks],function(x)
##                                 eval(parse(text=paste0("x+1-pmax(x,",entry,")")))
##                                    ),by=strata])
##     enter <- dur1[,lapply(.SD,function(x) sum(!is.na(x))),by=strata]
##     atrisk <- dur1[,lapply(.SD,function(x) sum(x,na.rm=TRUE)),by=strata]
##     vidx <- seq(length(strata)+1,ncol(atrisk))
##     suppressWarnings(names(atrisk)[vidx] <- paste0("_R",seq_along(vidx)))   
##     system.time(eventcens <- x[,
##                                lapply(breaks[-1],function(x)
##                                    eval(parse(text=paste0("((pmin(x,",exit,")-pmax(x-1,",entry,"))<(x-pmax(x-1,",entry,")))*(1+",status,")")))
##                                            ),by=strata])
##     lost <- eventcens[,lapply(.SD,function(x) sum(x==1,na.rm=TRUE)),by=strata]
##     events <- eventcens[,lapply(.SD,function(x) sum(x==2,na.rm=TRUE)),by=strata]
##     suppressWarnings(names(events)[vidx] <- paste0("_E",seq_along(vidx)))
##     res <- fast.reshape(cbind(events,atrisk[,vidx,with=FALSE]))
##     return(res)
##     res <- subset(data.frame(enter=enter,
##                              atrisk=atrisk,
##                              lost=lost,
##                              events=events,
##                              ## int.start=c(-Inf,breaks),
##                              ## int.end=c(breaks,Inf),
##                              int.start=breaks[-length(breaks)],
##                              int.end=breaks[-1],
##                              surv=0,
##                              rate=events/atrisk))
## }



LifeTable <- function(time,status,entry=NULL,weights=NULL,strata=list(),breaks=c(),confint=FALSE,interval=TRUE,mesg=FALSE) {
    if (is.null(entry)) entry <- rep(0,NROW(time))
    if (mesg) message(dim(time))
    if ((is.matrix(time) || is.data.frame(time)) && ncol(time)>1) {
        if (ncol(time)>=3L) {
            if (ncol(time)==4L) weights <- time[,4]
            status <- time[,3]
            entry <- time[,1]
            time <- time[,2]            
        } else {
            status <- time[,2]
            time <- time[,1]
            entry <- rep(0,length(time))
        }
    }

    if (mesg) message(dim(time))
    if ((is.matrix(time) || is.data.frame(time)) && ncol(time)>1) {
        if (ncol(time)==3) {
            status <- time[,3]
            entry <- time[,1]
            time <- time[,2]
        } else {
            status <- time[,2]
            time <- time[,1]
            entry <- rep(0,length(time))
        }
    }
    if (length(strata)>0) {
        a <- by(cbind(entry,time,status,weights), strata,
                FUN=LifeTable, breaks=breaks, confint=confint)
        cl <- lapply(strata,class)
        nulls <- which(unlist(lapply(a,is.null)))
        nonnulls <- setdiff(seq_along(a),nulls)
        nn <- do.call("expand.grid",attributes(a)$dimnames)
        if (length(nulls)>0) nn <- nn[-nulls,,drop=FALSE]
        nam <- nn[rep(seq(NROW(nn)),each=NROW(a[[nonnulls[1]]])),,drop=FALSE]
        xx <- list()
        for (i in seq(ncol(nam))) {
            if (cl[i]%in%c("numeric","integer"))
                xx <- c(xx,list(as.numeric(as.character(nam[,i]))))
            else                    
                xx <- c(xx, list(do.call(paste("as.",as.character(cl[i]),sep=""),list(nam[,i]))))
        }
        xx <- as.data.frame(xx); colnames(xx) <- colnames(nam)
        res <- Reduce("rbind",a)
        res <- cbind(res,xx)
        return(res)
    }
    if (length(breaks)==0) breaks <- c(0,max(time,na.rm=TRUE))
    if (length(breaks)==1) breaks <- c(0,breaks)
    breaks <- sort(unique(breaks))
    ## en <- matrix(unlist(lapply(c(-Inf, breaks),function(x) pmax(x,entry))),
    ##               ncol=length(breaks)+1)
    ## ex <- matrix(unlist(lapply(c(breaks, Inf),function(x) pmin(x,time))),
    ##               ncol=length(breaks)+1)
    en <- matrix(unlist(lapply(breaks[-length(breaks)],function(x) pmax(x,entry))),
                  ncol=length(breaks)-1)
    ex <- matrix(unlist(lapply(breaks[-1],function(x) pmin(x,time))),
                  ncol=length(breaks)-1)
    dur <- ex-en
    dur[dur<=0] <- NA
    eventcens <- dur; eventcens[!is.na(dur)] <- 0
    lastobs <- apply(eventcens,1,function(x) tail(which(!is.na(x)),1))
    eventcens[cbind(seq(nrow(eventcens)),lastobs)] <- status+1
    if (!is.null(weights)) {
        W <- weights %x% rbind(rep(1,ncol(dur)))
        W[is.na(dur)] <- NA
        enter <- colSums(W,na.rm=TRUE)
        atrisk <- colSums(dur*W,na.rm=TRUE)
        Sum <- function(x) { res <- sum(x,na.rm=TRUE); ifelse(length(res)>0L, res, 0L) } 
        lost <- apply(eventcens, 2, function(x) res <- Sum(weights[which(x==1)]))
        events <- apply(eventcens, 2, function(x) res <- Sum(weights[which(x==2)]))
    } else {
        enter <- colSums(!is.na(dur))
        atrisk <- colSums(dur,na.rm=TRUE)
        lost <- colSums(eventcens==1,na.rm=TRUE)
        events <- colSums(eventcens==2,na.rm=TRUE)
    }
    rate <- events/atrisk; rate[is.nan(rate)] <- 0
    res <- subset(data.frame(enter=enter,
                             atrisk=atrisk,
                             lost=lost,
                             events=events,
                             ## int.start=c(-Inf,breaks),
                             ## int.end=c(breaks,Inf),
                             int.start=breaks[-length(breaks)],
                             int.end=breaks[-1],
                             rate=rate))
    if (interval) res$interval <- factor(paste0("[",res$int.start,";",res$int.end,")"))
    cumsum.na <- function(x,...) { x[is.na(x)] <- 0; cumsum(x) }
    if (length(strata)==0) res$surv <- with(res, exp(-cumsum.na(rate*(int.end-int.start))))    
    if (confint) {
        ff <- events ~ offset(log(atrisk))
        if (length(breaks)>2) ff <- update(ff,.~.+factor(int.end)-1)
        g <- glm(ff,data=res,poisson)
        suppressMessages(ci <- rbind(exp(stats::confint(g))))
        res[,"2.5%"] <- ci[,1]
        res[,"97.5%"] <- ci[,2]
    }
    res
}

eh <- function(formula,intervals,family=poisson(log),...) {
    if (missing(intervals)) stop("Please supply list of time-intervals")
    g <- glm(formula,family=family,...)
    structure(g,class=c("glm","eh"),intervals=intervals)
    
}

##' Summary for survival analyses via the 'lifetable' function
##'
##' Summary for survival analyses via the 'lifetable' function 
##' @title Extract survival estimates from lifetable analysis
##' @param object glm object (poisson regression)
##' @param ... Contrast arguments
##' @param timevar Name of time variable
##' @param time Time points (optional)
##' @param int.len Time interval length (optional)
##' @param confint If TRUE confidence limits are supplied
##' @param level Level of confidence limits
##' @param individual Individual predictions
##' @param length.out Length of time vector
##' @aliases eventpois pcif 
##' @export
##' @author Klaus K. Holst
eventpois <- function(object,...,timevar,time,int.len,confint=FALSE,level=0.95,individual=FALSE,length.out=25) {      
    if (missing(timevar)) {
        timevar <- names(attributes(object)$intervals)[1]
        times <- attributes(object)$intervals
        int.len <- diff(times)
    }
    nn <- names(coef(object))
    if (is.null(timevar)) {
        idx <- unlist(sapply(c("\\(Intercept\\)","bs\\(","ns\\("),function(x) grep(x,nn)))
        keep <- setdiff(seq_along(nn),idx)
        cc <- estimate(object,exp,keep=keep,labels=nn[keep])$coefmat
        colnames(cc)[1] <- "RR"
        cc[,5] <- coef(summary(object))[keep,4]
        return(cc)
    }
    timevar_re0 <- gsub("\\$|\\^","",glob2rx(timevar))
    timevar_re <- paste0(timevar_re0,"[0-9]+\\.*[0-9]*")
    idx <- regexpr(timevar_re,nn)
    dots <- list(...)
    if (all(idx<0)) {
        timevar_re0 <- gsub("\\$|\\^","",glob2rx(paste0("factor(",timevar,")")))
        timevar_re <- paste0(timevar_re0,"[0-9]+\\.*[0-9]*")
        idx <- regexpr(timevar_re,nn)
    }
    
    tvar <- unique(regmatches(nn,idx))
    if (missing(time)) {        
        time <- sort(unique(as.numeric(gsub(timevar_re0,"",tvar))))
        if (length(time)==0) {
            i0 <- seq(nrow(object$data))
            rg <- range(object$data[i0,timevar],na.rm=TRUE)        
            time <- seq(rg[1],rg[2],length.out=length.out)
        } else {
            ##time <- seq(1,rg[2])
        }
    }
    if (missing(int.len)) {
        int.len <- diff(c(0,time))
    } else if (int.len==1) int.len <- rep(int.len,nrow(res))
    tt <- terms(object)
    offsetvar <- NULL
    if (attr(tt,"offset")) {
        offsetvar <- all.vars(tt)[attr(tt,"offset")]
        dots[[offsetvar]] <- 1
    }
    responsevar <- getoutcome(tt)
    vv <- setdiff(all.vars(formula(tt)),c(offsetvar,responsevar))
    dots0 <- dots
    dots[[timevar]] <- time
    dotnames <- c()
    if (individual) {
        newdata <- as.data.frame(dots)
        for (v in vv) {
            if (v%ni%names(dots)) newdata[,v] <- object$data[1,v]
            else dotnames <- c(dotnames,v)
        }    
    } else {
        for (v in vv) {            
            if (v%ni%names(dots)) {
                dots[[v]] <- object$data[1,v]
            } else {
                dotnames <- c(dotnames,v)
            }
        }        
        args <- c(list(`_data`=model.frame(object)),dots)
        newdata <- do.call(Expand, args)
        newdata <- dsort(newdata,c(setdiff(dotnames,timevar),timevar))
    }

    Terms <- delete.response(tt)    
    m <- model.frame(Terms, newdata, xlev = object$xlevels)
    X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    ## NA-robust matrix product
    beta0 <- coef(object); beta0[is.na(beta0)] <- -.Machine$double.xmax    
    V <- vcov(object)    
    V0 <- structure(matrix(0,length(beta0),length(beta0)),dimnames=list(names(beta0),names(beta0)))
    idx <- which(rownames(V0)%in%rownames(V))
    V0[idx,idx] <- V
    coefs <- X%*%beta0
    res <- cbind(time-int.len,int.len,exp(coefs))
    colnames(res) <- c("time","int.len","rate")    
    if (nrow(coefs)<=length(time)) {
        if (!confint) {
            res <- cbind(res,cumsum(res[,3]*int.len))
            res <- cbind(res,exp(-res[,4]))
        } else {
            S0 <- X%*%V0%*%t(X)
            system.time(e0 <- estimate(NULL,coef=coefs,vcov=S0,function(x) cumsum(exp(x)*int.len))) ## Cumulative hazard
            system.time(e1 <- estimate(e0, function(x) exp(-x),level=level)) ## Survival
            res <- cbind(res,e0$coefmat[,1],e1$coefmat[,c(1,3:4)])
        }
        colnames(res)[4:5] <- c("chaz","surv") 
    }
    if (!is.null(offsetvar)) newdata[,offsetvar] <- NULL

    times <- list()
    if (!individual) {
        aa <- unique(newdata[,setdiff(dotnames,c(timevar,offsetvar)),drop=FALSE])
        vv <- colnames(aa)
        for (i in seq(nrow(aa))) {
            idx <- seq(nrow(object$data))
            for (j in seq_along(vv)) {
                val <- aa[i,j]
                if (is.factor(val)) val <- as.character(val)
                idx <- intersect(idx,which(object$data[,vv[j]]==val))
            }
            times <- c(times, list(sort(unique(object$data[idx,timevar]))))
        }
    }

    structure(as.data.frame(cbind(res,newdata)),
              args=dots0,
              variables=aa,
              times=times,
              individual=individual,
              class=c("eventpois","data.frame")
              )
}

##' @export
plot.eventpois <- function(x,var,confint=TRUE,cont=TRUE,length.out=200,type="surv",
                           line.type="l",lty=c(1:8),col=1,lwd=1,add=FALSE,
                           use.time=TRUE,
                           legend,                           
                           xlab="Time",ylab="Survival probability",xlim,ylim,...) {
    type <- agrep(type,c("clogclog","hazard","survival"))
    if (!add) {
        if (missing(xlim)) xlim <- c(0,max(x[,1]+x[,2]))
        if (missing(ylim)) {
            ylim <- switch(type,
                           '1'=c(-30,max(log(sum(x[,2]*x[,3])))),
                           '2'=c(0,max(x[,3])),
                           c(0,1))
        }
        plot(0,type="n",xlab=xlab,ylab=ylab,xlim=xlim,ylim=ylim,...)
    }
    ##args <- attr(x,"args")
    ##aa <- expand.grid(args)
    aa <- attr(x,"variables")
    vv <- colnames(aa)    
    if (attr(x,"individual")) aa <- aa[1,,drop=FALSE]
    if (missing(legend)) legend <- nrow(aa)>1
    if (length(lty)<nrow(aa)) lty <- rep(lty,nrow(aa))
    if (length(col)<nrow(aa)) col <- rep(col,nrow(aa))
    if (length(lwd)<nrow(aa)) lwd <- rep(lwd,nrow(aa))
    for (i in seq(nrow(aa))) {
        idx <- seq(nrow(x))
        
        if (!attr(x,"individual"))
        for (j in seq_along(vv)) {
            val <- aa[i,j]
            if (is.factor(val)) val <- as.character(val)
            idx <- intersect(idx,which(x[,vv[j]]==val))
        }
        idx <- intersect(idx,which(x[,3]>0))
        e0 <- x[idx,,drop=FALSE]
        dt <- tail(e0,1)        
        time0 <- c(e0[,1],dt[,1]+dt[,2])
        if (use.time) {
            time0 <- attr(x,"time")[[i]]-1
            e0 <- e0[na.omit(match(time0,e0[,"time"])),,drop=FALSE]
            time0 <- time0[match(e0[,"time"],time0)]
            dt <- tail(e0,1)
            time0 <- c(time0,dt[,1]+dt[,2])
        }
        chaz0 <- cumsum(e0[,3]*e0[,2])
        tt <- seq(min(time0),max(time0),length.out=length.out)
        ff <- approxfun(time0,c(0,chaz0),method="linear")
        if (type==3) {
            lines(tt,exp(-ff(tt)),lty=lty[i],col=col[i],lwd=lwd[i],...)
        } else if (type==1) {
            lines(tt,log(ff(tt)),lty=lty[i],col=col[i],lwd=lwd[i],...)
        } else {
            lines(time0,e0[,3],lty=lty[i],col=col[i],lwd=lwd[i],...)
        }
    }
    if (legend) {
        lab <- apply(aa,1,function(x) paste(x,collapse=","))
        graphics::legend("bottomleft",lab,lty=lty,col=col,lwd=lwd,bg="white")
    }
    return(aa)    
    ## if (surv) lines(x[,c(1,5)],lty=lty[1],type=type,...)
    ## else lines(x[,c(1,3)],lty=lty[1],type=type,...)
    ## if (ncol(x)>4 && confint && surv) {
    ##     lines(x[,c(1,6)],lty=lty[2],type=type,...)
    ##     lines(x[,c(1,7)],lty=lty[2],type=type,...)
    ## }
}


##' @export
pcif <- function(models,timevar,survival,...,delta=0.01,stop.time,trunc=TRUE) {
    if (!is.list(models)) models <- list(models)
    ee <- lapply(models,eventpois,timevar=timevar,...)
    t0 <- c(0,ee[[1]][,timevar])
    int.len <- diff(t0)
    if (!missing(survival)) {
        G <- c(1,eventpois(survival,...)$surv)        
    } else {
        chaz <- c(0,Reduce("+",lapply(ee,function(x) x$chaz)))
        G <- exp(-chaz)
    }
    rates <- lapply(ee,function(x) x$rate)
    if (missing(stop.time)) stop.time <- max(ee[[1]][,1])
    tt <- seq(0,stop.time,by=delta)
    GG <- approx(x=t0,y=G,xout=tt,method="linear")
    rr <- lapply(rates,function(rate)
        approx(x=t0,y=c(rate,0),xout=tt,method="constant",f=0))    
    FF <- lapply(rr,function(x) {
        cif0 <- c(0,cumsum(x$y*GG$y*delta))
        cif0 <- cif0[-length(cif0)]
    })
    S <- Reduce("+",FF)
    idx <- which(S>1)
    if (trunc & length(idx)>0) {
        for (i in seq_along(FF)) {
            FF[[i]][idx] <- FF[[i]][idx]/S[idx]
        }
    }
    FF0 <- Reduce(cbind,FF); colnames(FF0) <- paste0("F",seq(ncol(FF0)))
    as.data.frame(cbind(time=tt,FF0,G=GG$y))
}

#' @export
plotConfregion <- function(sfit,add=TRUE,polygon=TRUE,cols=1,ltys=1,...)
{# {{{
  nn <- names(sfit$strata)
  if (is.null(nn)) ll <- 1 else ll <- length(nn)

  if (length(cols)!=ll) cols <- seq(cols[1],ll) else cols <- cols
  if (length(cols)!=ll) ltys <- seq(ltys[1],ll)  else ltys <- ltys

ss <- 1
for (i in seq(ll))
{
   if (ll>1 & i>1) ss <- ss+sfit$strata[i-1]
   if (ll==1) index <- 1:length(sfit$time)
   else index <- ss:(ss+sfit$strata[i]-1)

   nl <- cbind(sfit$time[index],sfit$lower[index])
   ul <- cbind(sfit$time[index],sfit$upper[index])

   ## if call is from survfit type with hazard plot "fun="cumhaz"))
   ## checks if ... contains "fun="cumhaz"
   if (hasArg("fun")) {
      nl <- cbind(sfit$time[index],-log(sfit$lower[index]))
      ul <- cbind(sfit$time[index],-log(sfit$upper[index]))
   }

  if (!polygon) {
      lines(nl,type="s",col=cols[i],lty=ltys[i]+1,lwd=3,...)
      lines(ul,type="s",col=cols[i],lty=ltys[i]+1,lwd=3,...)
  } else {
	 ll <- length(nl[,1])
         timess <- nl[,1]
         ttp <- c(timess[1],rep(timess[-c(1,ll)],each=2),timess[ll])
         tt <- c(ttp,rev(ttp))
         yy <- c(rep(nl[-ll,2],each=rep(2)),rep(rev(ul[-ll,2]),each=2))
         col.alpha<-0.1
         col.ci<-cols[i]
         col.trans <- sapply(col.ci, FUN=function(x) 
           do.call(grDevices::rgb,as.list(c(grDevices::col2rgb(x)/255,col.alpha))))
	 polygon(tt,yy,lty=0,col=col.trans)
      }
}

}# }}}

#' @export
kmplot<- function(x,add=FALSE,loc=NULL,col=NULL,lty=NULL,conf.int=TRUE,polygon=TRUE,add.legend=TRUE,...)
{ ## {{{ 
	### default location if loc not given 
	if (is.null(loc)) { 
	    if (min(x$surv)>0.7) loc <- "bl" else loc <- "bl"
            if (hasArg("fun")) loc <- "tl"
	}
	if (loc=="bl") loc <- "bottomleft"
	else if (loc=="br") loc <- "bottomright"
	else if (loc=="tr") loc <- "topright"
	else if (loc=="tl") loc <- "topleft"
	else loc <- "bottomleft"
	nn <- names(x$strata)
	if (is.null(nn)) ll <- 1 else ll <- length(nn)
        if (is.null(col)) cols <- seq(ll)  else cols <- col
        if (is.null(lty)) ltys <- seq(ll)  else ltys <- lty
	plot(x,col=cols,lty=ltys,conf.int=FALSE,...)
	if (!is.null(nn) & add.legend) legend(loc,legend=names(x$strata),col=cols,lty=ltys)
	if (conf.int) {
	    plotConfregion(x,add=TRUE,ltys=ltys,cols=cols,polygon=polygon,...)
	}

} ## }}}


##' @export
plotConfRegion <- function(x,band,add=TRUE,polygon=TRUE,col=1,type="s",...)
{# {{{
nl <- cbind(x,band[,1])
ul <- cbind(x,band[,2])

  if (!polygon) {
      lines(nl,type=type,...)
      lines(ul,type=type,...)
      } else {
	 ll <- length(nl[,1])
         timess <- nl[,1]
         ttp <- c(timess[1],rep(timess[-c(1,ll)],each=2),timess[ll])
         tt <- c(ttp,rev(ttp))
         yy <- c(rep(nl[-ll,2],each=rep(2)),rep(rev(ul[-ll,2]),each=2))
         col.alpha<-0.1
         col.ci<-col[1]
         col.trans <- sapply(col.ci, FUN=function(x) 
           do.call(grDevices::rgb,as.list(c(grDevices::col2rgb(x)/255,col.alpha))))
	 polygon(tt,yy,lty=0,col=col.trans,...)
      }
}# }}}

##' @export
plotConfRegionSE <- function(x,est,se,...)
{# {{{
ul <- est+1.96*se; nl <- est-1.96*se
plotConfRegion(x,cbind(nl,ul),...)
}# }}

##' Binomial Regression for Censored Competing Risks Data
##'
##' Fits a binomial regression model for a specific time point in the presence of 
##' right-censored data and competing risks. This function implements the 
##' Inverse Probability of Censoring Weighted (IPCW) estimating equation approach.
##'
##' The model estimates the probability:
##' \deqn{P(T \leq t, \epsilon=1 | X ) = \text{expit}( X^T \beta) }
##'
##' Based on binomial regresion IPCW response estimating equation: 
##' \deqn{ X ( \Delta^{ipcw}(t) I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 }
##' where \deqn{\Delta^{ipcw}(t) = I((min(t,T)< C)/G_c(min(t,T)-)} is 
##' IPCW adjustment of the response \deqn{Y(t)= I(T \leq t, \epsilon=1 )}.  
##' Two types of estimators are available:
##' \itemize{
##'   \item \code{type="I"}: Solves the standard IPCW estimating equation.
##'   \item \code{type="II"}: Adds a censoring augmentation term for efficiency gains, 
##'   solving \deqn{X \int E(Y(t)| T>s)/G_c(s) d \hat M_c}.
##' }
##'
##' Alternatively, \code{logitIPCW} performs a standard logistic regression with 
##' IPCW weights applied directly to the likelihood. Thus solving 
##' \deqn{ X  I(min(T_i,t) < G_i)/G_c(min(T_i ,t)) ( I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 } 
##' a standard logistic regression with IPCW weights.
##'
##' The variance estimation is based on squared influence functions, with options 
##' for naive variance (assuming known censoring) and robust variance (accounting 
##' for censoring model estimation).
##'
##' Censoring model may depend on strata (cens.model=~strata(gX)). 
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime data analysis}, 29, 441–482.
##'   \item Scheike TH, Zhang MJ, Gerds TA (2008). "Predicting cumulative incidence probability by direct binomial regression." \emph{Biometrika}, 95(1), 205–220.
##' }
##'
##' @param formula A formula object specifying the outcome and covariates. 
##'   The outcome must be an \code{Event} object (\code{Event(time, cause)}).
##' @param data A data frame containing the variables in the formula.
##' @param cause Numeric vector or scalar indicating the cause of interest for the competing risks.
##' @param time Numeric scalar indicating the time point of interest for the cumulative incidence.
##' @param beta Optional numeric vector of starting values for the coefficients. Defaults to zeros.
##' @param type Character string. Either \code{"I"} (classic binomial regression) or \code{"II"} (adds augmentation term).
##' @param offset Optional numeric vector of offsets for the linear predictor.
##' @param weights Optional numeric vector of weights for the score equations.
##' @param cens.weights Optional numeric vector of pre-calculated censoring weights. If NULL, they are estimated internally.
##' @param cens.model A formula specifying the censoring model. Defaults to \code{~+1}. Can include strata (e.g., \code{~strata(group)}).
##' @param se Logical. If TRUE, computes standard errors based on IPCW influence functions.
##' @param kaplan.meier Logical. If TRUE, uses Kaplan-Meier estimator for IPCW weights; otherwise uses exponential baseline.
##' @param cens.code Numeric code representing censored observations in the status variable. Defaults to 0.
##' @param no.opt Logical. If TRUE, optimization is skipped and starting values are used.
##' @param method Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.
##' @param augmentation Optional numeric vector for additional augmentation terms.
##' @param outcome Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function), \code{"rmst"} (Restricted Mean Survival Time), or \code{"rmtl"} (Restricted Mean Time Lost).
##' @param model Character string. Link function: \code{"default"} (auto-selects based on outcome), \code{"logit"}, \code{"exp"}, or \code{"lin"} (identity).
##' @param Ydirect Optional numeric vector. If provided, this outcome is used instead of constructing one from \code{outcome}. Useful for custom IPCW adjustments.
##'
##' @section Outcome definition:
##' The observed outcome \eqn{Y_i} constructed from \code{outcome} depends on whether
##' competing risks are present. Competing risks are detected automatically: the data are
##' considered to have competing risks when causes other than those specified in \code{cause}
##' are observed (i.e. \code{any(!(Causes \%in\% cause))}, where \code{Causes} are all
##' non-censoring event codes found in the data).
##'
##' \strong{No competing risks} (all observed events belong to \code{cause}):
##' \itemize{
##'   \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i = 1)}. Cumulative incidence.
##'   \item \code{"rmst"}: \eqn{Y_i = \min(T_i, t)}. Restricted mean survival time.
##'   \item \code{"rmtl"}: \eqn{Y_i = t - \min(T_i, t)}. Restricted mean time lost.
##' }
##'
##' \strong{Competing risks} (causes beyond \code{cause} are observed in the data):
##' \itemize{
##'   \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i \in \code{cause})}. Cumulative
##'     incidence for the cause(s) of interest.
##'   \item \code{"rmst"} or \code{"rmtl"}: \eqn{Y_i = I(\epsilon_i \in \code{cause})(t - \min(T_i, t))}.
##'     Cause-specific years lost, accumulated only for subjects who experience an event in \code{cause}.
##' }
##'
##' The default link function (\code{model = "default"}) is \code{"logit"} for \code{"cif"} and
##' \code{"exp"} for \code{"rmst"} and \code{"rmtl"}.
##' If \code{Ydirect} is supplied, outcome construction is bypassed entirely.
##' @param ... Additional arguments passed to lower-level optimization functions.
##'
##' @return An object of class \code{"binreg"} containing coefficients, variance-covariance matrix, 
##'   influence functions (\code{iid}), and model details.
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' # logistic regresion with IPCW binomial regression 
##' out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
##' summary(out)
##' head(iid(out))
##'
##' predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
##'
##' outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
##' summary(outs)
##'
##' ## glm with IPCW weights 
##' outl <- logitIPCW(Event(time,cause)~tcell+platelet,bmt,time=50)
##' summary(outl)
##'
##' ##########################################
##' ### risk-ratio of different causes #######
##' ##########################################
##' data(bmt)
##' bmt$id <- 1:nrow(bmt)
##' bmt$status <- bmt$cause
##' bmt$strata <- 1
##' bmtdob <- bmt
##' bmtdob$strata <-2
##' bmtdob <- dtransform(bmtdob,status=1,cause==2)
##' bmtdob <- dtransform(bmtdob,status=2,cause==1)
##' ###
##' bmtdob <- rbind(bmt,bmtdob)
##' dtable(bmtdob,cause+status~strata)
##' 
##' cif1 <- cif(Event(time,cause)~+1,bmt,cause=1)
##' cif2 <- cif(Event(time,cause)~+1,bmt,cause=2)
##' plot(cif1)
##' plot(cif2,add=TRUE,col=2)
##' 
##' cifs1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=50)
##' cifs2 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=2,time=50)
##' summary(cifs1)
##' summary(cifs2)
##' 
##' cifdob <- binreg(Event(time,status)~-1+factor(strata)+
##' 	 tcell*factor(strata)+platelet*factor(strata)+age*factor(strata)
##' 	 +cluster(id),bmtdob,cause=1,time=50,cens.model=~strata(strata))
##' summary(cifdob)
##' head(iid(cifdob)) 
##' 
##' newdata <- data.frame(tcell=1,platelet=1,age=0,strata=1:2,id=1)
##' riskratio <- function(p) {
##'   cifdob$coef <- p
##'   p <- predict(cifdob,newdata,se=0)
##'   return(p[1]/p[2])
##' }
##' lava::estimate(cifdob,f=riskratio)
##' 
##' predict(cifdob,newdata)
##' (p1 <- predict(cifs1,newdata))
##' (p2 <- predict(cifs2,newdata))
##' p1[1,1]/p2[1,1]
##' 
##' @aliases logitIPCW binregt
##' @export
binreg <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("II","I"),
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),
	   Ydirect=NULL,...)
{# {{{
	monotone <- TRUE
cl <- match.call()# {{{
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- 1*(status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing <- any(!(Causes %in% cause))
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

 nevent <- sum((status %in% cause)*(exit<=time))
 ## if event before time or alive, then uncensored, equality for both censored and events  
 obs <- (exit<=time & (statusC==0)) | (exit>=time)

  if (is.null(cens.weights))  { ## {{{
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else { se <- FALSE; resC <- formC <- NULL} ## }}}
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  if (is.null(augmentation))  augmentation=rep(0,p)
  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX

 if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst")
	     Y <-  c(pmin(exit,time))
             else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
  Yresp <- c(Y)
  Y <- Yipcw <- c(Y*obs/cens.weights)
  IPCW <- c(obs/cens.weights)

 if (se & monotone) {## {{{ censoring adjustment of variance  and infinitissimal jacknife pseudo augmentation
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    X2 <-  X2[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    Y <- Y[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    h  <-  Mrevcumsumstrata(X*Y,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    btime <- 1*(exit<time)
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    IhdLam0 <- Mcumsumstrata(h*S0i2*btime,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)
###    MGCiid <- Msumstrata(MGt,2,sumstrata,xx$id,mid+1)
    MGCiid <- Msumstrata(MGt,xx$id,mid+1)

   if (type[1]=="II") { ##  pseudo-value type augmentation
    hYt  <-  revcumsumstrata(Y,xx$strata,xx$nstrata)
    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
    U <- rep(0,length(xx$strata))
    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
    MGt <- X*c(U-IhdLam0)*c(xx$weights)
###    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
    MGtiid <- Msumstrata(MGt,xx$id,mid+1)
    augmentation  <-  colSums(MGtiid) + augmentation
    ###
    EXt  <-  Mrevcumsumstrata(X,xx$strata,xx$nstrata)
    IEXhYtdLam0 <- Mcumsumstrata(EXt*c(hYt)*S0i*S0i2*btime,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
    ###
    MGCiid2 <- Msumstrata(MGt2,xx$id,mid+1)
    ### Censoring Variance Adjustment 
    MGCiid <- MGCiid+(MGtiid-MGCiid2)
   }
   ## use data ordered by time (keeping track of id also)
   id <- xx$id
   }  else {
	  MGCiid <- 0
  }## }}}

 ## default links
 if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
	 if (outcome[1]=="years-lost") model <- "exp"
 }

obj <- function(pp,all=FALSE)
{ # {{{
lp <- c(X %*% pp+offset)

    if (model[1]=="exp") {
	 p <- exp(lp) 
         D2logl <- c(weights*p)*X2 
	 if (monotone) {
	    Dlogl <- weights*X*c(Y-p) 
            D2logl <- c(weights*p)*X2 
	 } else { 
	    Dlogl <- weights*X*c((Y-p)*p)
            D2logl <- c(weights*p^2)*X2 
	 }
     } else if (model[1]=="lin") {
	 p <- lp
         D2logl <- c(weights)*X2
         Dlogl <- weights*X*c(Y-p)
    } else if (model[1]=="logit") {
	p <- expit(lp)
        if (monotone) {
           D2logl <- c(weights*p/(1+exp(lp)))*X2
           Dlogl <- weights*X*c(Y-p)
	} else {
           varp <- p/(1+exp(lp))
           Dlogl <- weights*X*c((Y-p)*varp)
           D2logl <- c(weights*varp^2)*X2
	}
} else stop("link functions must be logit,exp,lin\n") 
ploglik <- sum(weights*(Y-p)^2)
## to avoid using ploglik for fitting, with exp,logit only solve estimating equation
if (model[1]!="lin") ploglik <- 0

###D2log <- apply(D2logl,2,sum)
###gradient <- apply(Dlogl,2,sum)+augmentation
D2log <- colSums(D2logl)
gradient <- colSums(Dlogl)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ploglik <- sum(weights*(Y-p)^2)
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
###      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      beta.iid <-  Msumstrata(beta.iid,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

## setting default for NR 
dots <- list(...)
if (length(dots)==0) {
   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
   else control <- NULL
} else control <- dots[[1]]

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
###	      if (!se) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else {
	      val <- c(list(coef=beta),obj(beta,all=TRUE))
  }
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid))

  if ( (!monotone) & se) { ## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    lp <- c(X %*% val$coef+offset)
    Y <- Y[ord]

   if (model[1]=="exp") {
	 p <- exp(lp) 
         Yglm <- weights*X*c(Y*p)
    } else if (model[1]=="lin") {
	 p <- lp
         Yglm <- weights*X*c(Y)
    } else if (model[1]=="logit") {
	 p <- expit(lp) 
         varp <- p/(1+exp(lp))
         Yglm <- weights*X*c(Y*varp)
    }

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
  }## }}}

  ## do not need this, since have iid
  val$Dlogl  <- NULL

  val$call <- cl
  val$MGciid <- MGCiid 
  val$call.id <- call.id
  val$id <- orig.id
  val$name.id <- name.id
  val$nid <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL 
  if (se)  val$iid  <- val$iid+(MGCiid %*% val$ihessian)
  if (!is.null(name.id)) val$iid <- nameme(val$iid,name.id)
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$augmentation <- augmentation
  val$model <- model[1]
  val$outcome <- outcome[1]
  val$Yipcw <- Yipcw
  val$Y     <- Yresp
  val$IPCW   <- IPCW
  val$Causes <- Causes
  val$nevent <- nevent
  val$design <- des

  class(val) <- "binreg"
  return(val)
}# }}}

###binreg <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("II","I"),
###	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
###	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
###	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),
###	   Ydirect=NULL,...)
###{# {{{
###	monotone <- TRUE
###cl <- match.call()# {{{
######    m <- match.call(expand.dots = TRUE)[1:3]
###    des <- proc_design(
###        formula,
###        data = data,
###        specials = c("offset", "weights", "cluster"),
###        intercept = TRUE
###    )
###    Y <- des$y
###    if (!inherits(Y, c("Event", "Surv"))) {
###        stop("Expected a 'Surv' or 'Event'-object")
###    }
###    if (ncol(Y) == 2) {
###        exit <- Y[, 1]
###        entry <- rep(0, nrow(Y))
###        status <- Y[, 2]
###    } else {
###        entry <- Y[, 1]
###        exit <- Y[, 2]
###        status <- Y[, 3]
###    }
###    X <- des$x
###    des.weights <- des$weights
###    des.offset  <- des$offset
###    id      <- des$cluster
###  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
###
###  ### possible handling of id to code from 0:(antid-1)
###  call.id <- id 
###  conid <- construct_id(id,nrow(X),namesX=rownames(X))
###  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
###  ## id before time-sorting later 
###  orig.id <- id
###
###  ## take offset and weight first from formula, but then from arguments
###  if (is.null(des.offset)) {
###	  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  } else offset <- des.offset
###  if (is.null(des.weights)) {
###	  if (is.null(weights)) weights <- rep(1,length(exit)) 
###  } else weights <- des.weights
#### }}}
###
###  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
###  statusC <- 1*(status %in% cens.code) 
###  statusE <- (status %in% cause) & (exit<= time) 
###  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
###  kmt <- kaplan.meier
###
###  ucauses  <-  sort(unique(status))
###  ccc <- which(ucauses %in% cens.code)
###  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
###  competing <- any(!(Causes %in% cause))
###  data$id__ <- id
###  data$exit <- exit
###  data$statusC <- statusC 
###  cens.strata <- cens.nstrata <- NULL 
###
### nevent <- sum((status %in% cause)*(exit<=time))
### ## if event before time or alive, then uncensored, equality for both censored and events  
### obs <- (exit<=time & (statusC==0)) | (exit>=time)
###
###  if (is.null(cens.weights))  { ## {{{
###      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
###      resC <- phreg(formC,data)
###      if (resC$p>0) kmt <- FALSE
###      exittime <- pmin(exit,time)
###      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
###      ## strata from original data 
###      cens.strata <- resC$strata[order(resC$ord)]
###      cens.nstrata <- resC$nstrata
###  } else { se <- FALSE; resC <- formC <- NULL} ## }}}
###  expit  <- function(z) 1/(1+exp(-z)) ## expit
###
###  p <- ncol(X)
###  if (is.null(beta)) beta <- rep(0,p)
###  if (is.null(augmentation))  augmentation=rep(0,p)
###  X <-  as.matrix(X)
###  X2  <- .Call("vecCPMat",X)$XX
###
### if (!is.null(Ydirect)) Y <-  Ydirect else {
###     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
###     else { if (!competing) {
###	     if (outcome[1]=="rmst")
###	     Y <-  c(pmin(exit,time))
###             else Y <-  c((time-pmin(exit,time)))
###            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
###     }
###  }
###  Yresp <- c(Y)
###  Y <- Yipcw <- c(Y*obs/cens.weights)
###  IPCW <- c(obs/cens.weights)
###
### if (se & monotone) {## {{{ censoring adjustment of variance  and infinitissimal jacknife pseudo augmentation
###    ### order of sorted times
###    ord <- resC$ord
###    X <-  X[ord,,drop=FALSE]
###    X2 <-  X2[ord,,drop=FALSE]
###    status <- status[ord]
###    exit <- exit[ord]
###    weights <- weights[ord]
###    offset <- offset[ord]
###    Y <- Y[ord]
###
###    xx <- resC$cox.prep
###    S0i2 <- S0i <- rep(0,length(xx$strata))
###    S0i[xx$jumps+1]  <- 1/resC$S0
###    S0i2[xx$jumps+1] <- 1/resC$S0^2
###    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
###    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
###    ### Cens-Martingale as a function of time and for all subjects to handle strata 
###    btime <- 1*(exit<time)
###    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
###    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
###
###    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
###    mid <- max(id)
###    MGCiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
###
###   if (type[1]=="II") { ##  pseudo-value type augmentation
###    hYt  <-  revcumsumstrata(Y,xx$strata,xx$nstrata)
###    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
###    U <- rep(0,length(xx$strata))
###    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
###    MGt <- X*c(U-IhdLam0)*c(xx$weights)
###    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
###    augmentation  <-  apply(MGtiid,2,sum) + augmentation
###    ###
###    EXt  <-  apply(X,2,revcumsumstrata,xx$strata,xx$nstrata)
###    IEXhYtdLam0 <- apply(EXt*c(hYt)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
###    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
###    ###
###    MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
###    ### Censoring Variance Adjustment 
###    MGCiid <- MGCiid+(MGtiid-MGCiid2)
###   }
###   ## use data ordered by time (keeping track of id also)
###   id <- xx$id
###   }  else {
###	  MGCiid <- 0
###  }## }}}
###
### ## default links
### if (model[1]=="default") {
###	 if (outcome[1]=="cif") model <- "logit"
###	 if (outcome[1]=="rmst") model <- "exp"
###	 if (outcome[1]=="rmtl") model <- "exp"
###	 if (outcome[1]=="years-lost") model <- "exp"
### }
###
###obj <- function(pp,all=FALSE)
###{ # {{{
###lp <- c(X %*% pp+offset)
###
###    if (model[1]=="exp") {
###	 p <- exp(lp) 
###         D2logl <- c(weights*p)*X2 
###	 if (monotone) {
###	    Dlogl <- weights*X*c(Y-p) 
###            D2logl <- c(weights*p)*X2 
###	 } else { 
###	    Dlogl <- weights*X*c((Y-p)*p)
###            D2logl <- c(weights*p^2)*X2 
###	 }
###     } else if (model[1]=="lin") {
###	 p <- lp
###         D2logl <- c(weights)*X2
###         Dlogl <- weights*X*c(Y-p)
###    } else if (model[1]=="logit") {
###	p <- expit(lp)
###        if (monotone) {
###           D2logl <- c(weights*p/(1+exp(lp)))*X2
###           Dlogl <- weights*X*c(Y-p)
###	} else {
###           varp <- p/(1+exp(lp))
###           Dlogl <- weights*X*c((Y-p)*varp)
###           D2logl <- c(weights*varp^2)*X2
###	}
###} else stop("link functions must be logit,exp,lin\n") 
###ploglik <- sum(weights*(Y-p)^2)
##### to avoid using ploglik for fitting, with exp,logit only solve estimating equation
###if (model[1]!="lin") ploglik <- 0
###
###D2log <- apply(D2logl,2,sum)
###gradient <- apply(Dlogl,2,sum)+augmentation
###np <- length(pp)
###hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)
###
###  if (all) {
###      ploglik <- sum(weights*(Y-p)^2)
###      ihess <- pinv(hessian)
###      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
###      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
###      robvar <- crossprod(beta.iid)
###      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
###	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
###      return(val)
###  }  
### structure(-ploglik,gradient=-gradient,hessian=hessian)
###}# }}}
###
##### setting default for NR 
###dots <- list(...)
###if (length(dots)==0) {
###   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
###   else control <- NULL
###} else control <- dots[[1]]
###
###  p <- ncol(X)
###  opt <- NULL
###  if (p>0) {
###  if (no.opt==FALSE) {
###      if (tolower(method)=="nr") {
###          tim <- system.time(opt <- lava::NR(beta,obj,control=control))
###	  opt$timing <- tim
###	  opt$estimate <- opt$par
###      } else {
###	  opt <- nlm(obj,beta,...)
###	  opt$method <- "nlm"
###      }
###      cc <- opt$estimate; 
######	      if (!se) return(cc)
###      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
###      } else {
###	      val <- c(list(coef=beta),obj(beta,all=TRUE))
###  }
###  } else {
###      val <- obj(0,all=TRUE)
###  }
###
###  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
###  val <- c(val,list(time=time,formula=formula,formC=formC,
###    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
###    n=length(exit),nevent=nevent,ncluster=nid))
###
###  if ( (!monotone) & se) { ## {{{ censoring adjustment of variance 
###    ### order of sorted times
###    ord <- resC$ord
###    X <-  X[ord,,drop=FALSE]
###    status <- status[ord]
###    exit <- exit[ord]
###    weights <- weights[ord]
###    offset <- offset[ord]
###    lp <- c(X %*% val$coef+offset)
###    Y <- Y[ord]
###
###   if (model[1]=="exp") {
###	 p <- exp(lp) 
###         Yglm <- weights*X*c(Y*p)
###    } else if (model[1]=="lin") {
###	 p <- lp
###         Yglm <- weights*X*c(Y)
###    } else if (model[1]=="logit") {
###	 p <- expit(lp) 
###         varp <- p/(1+exp(lp))
###         Yglm <- weights*X*c(Y*varp)
###    }
###
###    xx <- resC$cox.prep
###    S0i2 <- S0i <- rep(0,length(xx$strata))
###    S0i[xx$jumps+1]  <- 1/resC$S0
###    S0i2[xx$jumps+1] <- 1/resC$S0^2
###    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
###    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
###    ### Cens-Martingale as a function of time and for all subjects to handle strata 
###    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
###    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
###    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
###  }## }}}
###
###  val$call <- cl
###  val$MGciid <- MGCiid 
###  val$call.id <- call.id
###  val$id <- orig.id
###  val$name.id <- name.id
###  val$nid <- nid
###  val$iid.naive <- val$iid
###  val$naive.var <- NULL 
###  if (se)  val$iid  <- val$iid+(MGCiid %*% val$ihessian)
###  if (!is.null(name.id)) val$iid <- nameme(val$iid,name.id)
###  robvar <- crossprod(val$iid)
###  val$var <-  val$robvar <- robvar
###  val$se.robust <- diag(robvar)^.5
###  val$se.coef <- diag(val$var)^.5
###  val$cause <- cause
###  val$cens.code <- cens.code 
###  val$augmentation <- augmentation
###  val$model <- model[1]
###  val$outcome <- outcome[1]
###  val$Yipcw <- Yipcw
###  val$Y     <- Yresp
###  val$IPCW   <- IPCW
###  val$Causes <- Causes
###  val$nevent <- nevent
###  val$design <- des
###
###  class(val) <- "binreg"
###  return(val)
###}# }}}


#' Influence curve components for binomial regression ATE
#'
#' Extracts and scales influence function (IC) components from a fitted
#' binomial regression ATE object or raw IC matrix. Depending on the object
#' class and requested `type`, returns one or more components combined by
#' column-binding.
#'
#' @param x An object of class \code{ATE}, or an object containing an
#'   \code{iid} component.
#' @param type Character vector specifying which influence curve components
#'   to return. Possible values are:
#'   \describe{
#'     \item{coef}{Influence curve for regression coefficients}
#'     \item{DR}{Doubly robust ATE influence curve}
#'     \item{G}{G-computation ATE influence curve}
#'   }
#'   If \code{x} inherits from \code{ATE} and \code{type} is missing,
#'   the default is \code{c("DR", "G")}.
#' @param ... Currently unused.
#'
#' @return A numeric matrix obtained by column-binding the selected
#' influence curve components. Each column corresponds to one element
#' of \code{type}, with column names preserved.
#'
#' @details
#' For objects of class \code{ATE}, the function extracts and rescales
#' the stored influence curve components:
#'
#' \itemize{
#'   \item \code{x$iid} for coefficient ICs
#'   \item \code{x$riskDR.iid} for doubly robust ATE ICs
#'   \item \code{x$riskG.iid} for G-computation ICs
#' }
#'
#' Each component is multiplied by its sample size (\code{NROW(.)}).
#'
#' For non-\code{ATE} objects, the function returns
#' \code{NROW(x$iid) * x$iid}.
#'
#' @examples
#' \dontrun{
#' data(bmt)
#' dfactor(bmt)  <-  ~.
#'
#' fit  <- binregATE(Event(time,cause)~tcell.f+platelet+age,bmt,time=50,cause=1,
#'	  treat.model=tcell.f~platelet+age)
#
#' # default ATE behavior (DR + G)
#' head(IC(fit))
#'
#' # only DR component
#' head(IC(fit, "DR"))
#'
#' # multiple components
#' head(IC(fit, c("coef", "DR")))
#' }
#'
#' @export
IC.binreg <- function(x, type = c("coef", "DR", "G"), ...) { ## {{{ 

  # Default override for ATE objects
  if (inherits(x, "ATE") && missing(type)) {
    type <- c("DR", "G")
  }

  type <- match.arg(type, several.ok = TRUE)

  if (!inherits(x, "ATE")) {
    return(x$iid * NROW(x$iid))
  }

  comps <- list()

  for (t in type) {

    ic_t <- switch(t,
      "coef" = NROW(x$iid) * x$iid,
      "DR"   = NROW(x$riskDR.iid) * x$riskDR.iid,
      "G"    = NROW(x$riskG.iid) * x$riskG.iid,
      stop("Unknown type: ", t)
    )

    comps[[t]] <- ic_t
  }

  out <- do.call(cbind, comps)

  return(out)
}  ## }}} 

##' @export
print.binreg  <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.binreg <- function(object,...) {# {{{

if (!is.null(object$gradient)) { ## write warning if gradient not small 
gradient <- max(abs(object$gradient))
if (gradient > 0.000001) { cat("gradient:\n"); print(object$gradient) }
}

cc  <- estimate(coef=object$coef,vcov=object$var)$coefmat
V=object$var

res <- list(coef=cc,n=object$n,nevent=object$nevent,strata=NULL,
	    ncluster=object$ncluster,var=V,model=object$model[1])

if (object$model[1]=="exp" | object$model[1]=="logit") {
expC <- exp(lava::estimate(coef=coef(object),vcov=object$var)$coefmat[,c(1,3,4),drop=FALSE])
res <- c(res,list(exp.coef=expC))
}

## to add marginal estimates for binregATE 
if (!is.null(object$riskDR))  {
    marginalDR <- estimate(coef=object$riskDR,vcov=object$var.riskDR)$coefmat
    difmarginalDR <- estimate(coef=object$difriskDR,vcov=as.matrix(object$var.difriskDR))$coefmat
###    rownames(difmarginalDR) <- "differenceDR"
    marginalDR <- rbind(marginalDR,difmarginalDR)

    marginalG <- estimate(coef=object$riskG,vcov=object$var.riskG)$coefmat
    difG <- estimate(coef=object$difriskG,vcov=as.matrix(object$var.difriskG))$coefmat
###    rownames(difG) <- "differenceG"
    marginalG <- rbind(marginalG,difG) 
    res <- c(res,list(ateG=marginalG,ateDR=marginalDR))

    if (!is.null(object$attc)) {
    attc <- estimate(coef=object$attc,vcov=object$var.attc)$coefmat
    res <- c(res,list(attc=attc))
    }
}

class(res) <- "summary.binreg"
return(res)
}# }}}

##' @export
print.summary.binreg <- function(x,max.strata=5,...) { ## {{{

  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- levels(x$strata); colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  if (length(x$strata)>max.strata) {
      nn <- rbind(c(colSums(nn),length(x$strata)));
      colnames(nn) <- c("n","events","stratas")
      rownames(nn) <- ""
  } 
  print(nn,quote=FALSE)  
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coeffients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    if (x$model[1]=="exp" | x$model[1]=="logit") {
    cat("exp(coeffients):\n")
    printCoefmat(x$exp.coef,...)
    }
  }
  cat("\n")

 ## for binreg ATE
 if (!is.null(x$ateDR)) {
    cat("Average Treatment effects (G-formula) :\n")
    printCoefmat(x$ateG,...)
    cat("\n")

    cat("Average Treatment effects (double robust) :\n")
    printCoefmat(x$ateDR,...)
    cat("\n")

###    if (!is.null(x$attc)) {
###    cat("Average Treatment effects on Treated/Non-Treated (DR) :\n")
###    printCoefmat(x$attc,...)
###    cat("\n")
###    }

  }
  cat("\n")

} ## }}}

###}}} 


##' @export
vcov.binreg <- function(object,...) {# {{{
	return(object$var)
}# }}}

##' @export
coef.binreg <- function(object,...) {# {{{
	return(object$coef)
}# }}}

##' @export
predict.binreg <- function(object,newdata=NULL,se=TRUE,pred.iid=FALSE,level=0.95,design=FALSE,...)
{# {{{
   if (!is.null(newdata)) x <- update_design(object$design,data = newdata,response=FALSE) else x <- object$design
  Z <- x$x

  if (object$model[1]=="logit") {
  lp <- c(Z %*% object$coef)
  p <- expit(lp)
  preds <- p
  alpha <- 1-level

  if (se) {
     if (is.null(object$var)) covv <- vcov(object)  else covv <- object$var
     Dpv <- Z*exp(-lp)*p^2
     se <- apply((Dpv %*% covv)* Dpv,1,sum)^.5
     crit <- qnorm(1-alpha/2)
     cmat <- data.frame(pred=p,se=se,lower=p-1.96*se,upper=p+1.96*se)
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- cmat
  }
  if (pred.iid) {
      Piid <- object$iid  %*% t(Dpv)
      preds <- list(pred=preds,iid=Piid)
  }

  } else {

  lp <- c(Z %*% object$coef)
  if (object$model[1]=="exp") p <- exp(lp) else p <- lp
  preds <- p

  level <- 0.95
  alpha <- 1-level
  crit <- qnorm(1-alpha/2)

  if (se) {
     if (is.null(object$var)) covv <- vcov(object)  else covv <- object$var
###     if (object$model[1]=="dexp") Dpv <- Z*p^2 else 
     if (object$model[1]=="exp") Dpv <- Z*p else Dpv <- Z 
     se <- apply((Dpv %*% covv)* Dpv,1,sum)^.5
     cmat <- data.frame(pred=p,se=se,lower=p-crit*se,upper=p+crit*se)
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- cmat
  }
  if (pred.iid) {
      Piid <- object$iid  %*% t(Dpv)
      preds <- list(pred=preds,iid=Piid,id=x$cluster)
  }

  }

  attr(preds,"id") <- x$cluster
  if (design) attr(preds,"x") <- Z

return(preds)
} # }}}

##' @export
binregt <- function(formula,data,cause=1,time=NULL,beta=NULL,
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,...)
{# {{{
  cl <- match.call()# {{{
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

  call.id <- id
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting
  orig.id <- id

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

# }}}




  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in% cens.code) 
  statusE <- (status %in% cause)*outer(exit,time,"<=") 
  if (any(apply(statusE,2,sum))==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      cens.weights <- c()
      for (tt in time)  {
      exittime <- pmin(exit,tt)
      cens.weights1 <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      cens.weights <- cbind(cens.weights,cens.weights1)
      }
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  if (length(time)>1) {
      Yt <- outer(exit,time,"<=") 
      nevent <- apply(c(status %in% cause)*Yt,2,sum)
      Y <- c((status %in%cause)/cens.weights)*Yt
  } else {
      Y <- c((status %in%cause)*(exit<=time)/cens.weights)
      nevent <- sum((status %in%cause)*(exit<=time))
  }

  ppt <- length(time)
  p <- pd <- ncol(X)+(length(time)-1)
  if (is.null(beta)) beta <- rep(0,ncol(X)+ppt-1)
  X <-  as.matrix(X)
  X2o  <- .Call("vecMatMat",X[,-1,drop=FALSE],X[,-1,drop=FALSE])$vXZ
###  X2  <- .Call("vecMatMat",X,X)$vXZ

  if (is.null(augmentation))  augmentation=rep(0,p)

obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X[,-1,drop=FALSE] %*% pp[-seq(1,ppt)])
lp <- outer(lp,pp[1:ppt],"+")
p <- expit(lp)
ploglik <- sum(weights*(Y-p)^2)

Dloglt <- weights*(Y-p)
Dloglo <- weights*X[,-1]*apply((Y-p),1,sum)
Dlogl <- cbind(Dloglt,Dloglo)

wwp <- (weights*p/(1+exp(lp)))
wwpt <- apply(wwp,2,sum)
wwpo <- apply(wwp,1,sum)
D2loglo <- c(wwpo)*X2o
hessian <- matrix(0,pd,pd)
hessian[seq(1,ppt),seq(1,ppt)] <- diag(wwpt,ppt,ppt)
for (i in 1:ppt) hessian[i,(ppt+1):pd] <- apply(wwp[,i]*X[,-1,drop=FALSE],2,sum)
hessian[(ppt+1):pd,1:ppt]  <- t(hessian[1:ppt,(ppt+1):pd])
hessian[(ppt+1):pd,(ppt+1):pd] <- c(apply(D2loglo,2,sum))

gradient <- apply(Dlogl,2,sum)+augmentation

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik/nid,gradient=-gradient/nid,hessian=hessian/nid)
}# }}}

	  p <- pd 
	  opt <- NULL
	  if (p>0) {
	  if (no.opt==FALSE) {
	      if (tolower(method)=="nr") {
		  tim <- system.time(opt <- lava::NR(beta,obj,...))
		  opt$timing <- tim
		  opt$estimate <- opt$par
	      } else {
		  opt <- nlm(obj,beta,...)
		  opt$method <- "nlm"
	      }
	      cc <- opt$estimate; 
	      if (!se) return(cc)
	      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
	      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
	  } else {
	      val <- obj(0,all=TRUE)
	  }

	  if (length(time)==1) {
	  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X) }
	  else names(val$coef) <- c(paste("Intercept",time,sep=""),colnames(X)[-1])
	  val <- c(val,list(time=time,formula=formula,formC=formC,
	    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
	    n=length(exit),nevent=nevent,ncluster=nid))
	  

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    if (length(time)>1) {
         Yt <- outer(exit,time,"<=") 
         Y <- c((status%in% cause)/cens.weights)*Yt
	 Ys <- apply(Y,1,sum)
     } else {
         Ys <- Y <- c((status%in%cause)*(exit<=time)/cens.weights)
     }

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(cbind(Y,X[,-1]*Ys),2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    btime <- 1*outer(exit,time,"<") 
    IhdLam0 <- apply(h*c(S0i2),2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(X),pd)
    jtime <- outer(resC$jumptimes,time,"<")*1
    U[xx$jumps+1,] <- (h[xx$jumps+1,])/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
 
   }  else {
	  MGCiid <- 0
  }## }}}

  val$call <- cl
  val$MGciid <- MGCiid
  val$call.id <- call.id
  val$name.id <- name.id
  val$id <- orig.id
  val$nid <- nid
  val$iid.naive <- val$iid 
  if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian) else val$iid  <- val$iid
  val$naive.var <- val$var
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$model <- "logit"
  val$design <- des


  class(val) <- "binreg"
  return(val)
}# }}}

##' @export
logitIPCW <- function(formula,data,cause=1,time=NULL,beta=NULL,
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
###  m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status%in% cens.code) 
  statusE <- (status%in%cause) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  statusC <- (status %in%cens.code) 
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  if (is.null(beta)) beta <- rep(0,ncol(X))
  p <- ncol(X)

  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX
  obs <- (exit<=time & status!=cens.code) | (exit>=time)
  weights <- obs*weights/c(cens.weights)
  cens.weights <- c(cens.weights)

  if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst")
	     Y <-  c(pmin(exit,time)) 
             else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
  Yipcw <- Y
 
 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum(Y)

 ## default links
 if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
	 if (outcome[1]=="years-lost") model <- "exp"
 }


obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X %*% pp+offset)

    if (model[1]=="exp") {
	 p <- exp(lp) 
         D2logl <- c(weights*p)*X2 
     } else if (model[1]=="lin") {
	 p <- lp
         D2logl <- c(weights)*X2
     } else if (model[1]=="logit") {
	p <- expit(lp)
        D2logl <- c(weights*p/(1+exp(lp)))*X2
     } else stop("link functions must be logit,exp,lin\n") 
ploglik <- sum(weights*(Y-p)^2)

if (model[1]=="exp") ploglik <- 0
Dlogl <- weights*X*c(Y-p)
D2log <- apply(D2logl,2,sum)
gradient <- apply(Dlogl,2,sum)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

## setting default for NR 
dots <- list(...)
if (length(dots)==0) {
   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
   else control <- NULL
} else control <- dots[[1]]

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid,weights=weights))
  
 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    lp <- c(X %*% val$coef+offset)
    p <- expit(lp)
    Yglm <- weights*c(Y[ord]-p) # *(exit<=time)

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
  }  else {
	  MGCiid <- 0
  }## }}}

    val$Y <- Yipcw
    val$model <- model[1]
    val$outcome <- outcome[1]
    val$call <- cl
    val$MGciid <- MGCiid
    val$id <- orig.id
    val$call.id <- call.id
    val$name.id <- name.id
    val$nid <- nid
    val$iid.naive <- val$iid 
    if (se)
    val$iid  <- val$iid+(MGCiid %*% val$ihessian)
    if (!is.null(call.id)) val$iid <- nameme(val$iid,name.id)
    val$naive.var <- val$var
    robvar <- crossprod(val$iid)
    val$var <-  val$robvar <- robvar
    val$se.robust <- diag(robvar)^.5
    val$se.coef <- diag(val$var)^.5
    val$cause <- cause
    val$cens.code <- cens.code 
    val$augmentation <- augmentation
    val$design <- des

  class(val) <- "binreg"
  return(val)
}# }}}


##' Average Treatment Effect for Censored Competing Risks Data using Binomial Regression
##'
##' Estimates the average treatment effect (ATE) \eqn{E(Y(1) - Y(0))} for censored 
##' competing risks data using binomial regression with Inverse Probability of 
##' Censoring Weighting (IPCW).
##'
##' Under standard causal assumptions, the ATE can be estimated. These assumptions include:
##' \itemize{
##'   \item \strong{Consistency}: The observed outcome equals the potential outcome under the observed treatment.
##'   \item \strong{Ignorability}: \eqn{(Y(1), Y(0)) \perp A | X} (treatment assignment is independent of potential outcomes given covariates).
##'   \item \strong{Positivity}: All treatment levels have non-zero probability given covariates.
##' }
##'
##' The first covariate in the competing risks regression model must be the treatment variable, 
##' which should be coded as a factor. If the factor has more than two levels, multinomial 
##' logistic regression (\code{mlogit}) is used for propensity score modeling. In the absence 
##' of censoring, this reduces to ordinary logistic regression.
##'
##' The ATE is estimated using standard doubly robust estimating equations that are 
##' IPCW-censoring adjusted. As an alternative to binomial regression, 
##' \code{logitIPCWATE} provides an IPCW-weighted version of standard logistic regression.
##'
##' When \code{typeATE = "II"}, the estimating equation is augmented with:
##' \deqn{ (A/\pi(X)) \int E( O(t) | T \geq t, S(X))/ G_c(t,S(X)) d \hat M_c(s) }
##' when estimating the mean outcome for the treated group.
##'
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
##' }
##'
##' @param formula A formula object specifying the outcome and covariates (see \code{coxph}). 
##'   The first covariate should be the treatment variable coded as a factor.
##' @param data A data frame containing the variables in the formula.
##' @param cause Numeric scalar indicating the cause of interest for competing risks.
##' @param time Numeric scalar indicating the time point of interest.
##' @param beta Optional numeric vector of starting values for the coefficients.
##' @param treat.model A formula specifying the logistic treatment model given covariates 
##'   (e.g., \code{treatment ~ covariate1 + covariate2}).
##' @param cens.model A formula specifying the censoring model. Only stratified Cox models 
##'   without covariates are supported (e.g., \code{~ strata(group)}).
##' @param offset Optional numeric vector of offsets for the partial likelihood.
##' @param weights Optional numeric vector of weights for the score equations.
##' @param cens.weights Optional numeric vector of pre-calculated censoring weights. 
##'   If \code{NULL}, weights are estimated internally.
##' @param se Logical. If \code{TRUE}, computes standard errors with IPCW adjustment. 
##'   If \code{FALSE}, assumes IPCW weights are known.
##' @param type Character string. Either \code{"I"} (classic binomial regression) or 
##'   \code{"II"} (adds augmentation term for efficiency).
##' @param kaplan.meier Logical. If \code{TRUE}, uses Kaplan-Meier for IPCW weights; 
##'   otherwise uses \eqn{\exp(-\text{Baseline})}.
##' @param cens.code Numeric code representing censored observations in the status variable.
##' @param no.opt Logical. If \code{TRUE}, optimization is skipped and starting values are used.
##' @param method Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.
##' @param augmentation Optional numeric vector for augmenting binomial regression.
##' @param outcome Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function, 
##'   \eqn{F(t|X)}), \code{"rmst"} (Restricted Mean Survival Time, \eqn{E(\min(T, t) | X)}), 
##'   or \code{"rmtl"} (Restricted Mean Time Lost, \eqn{E(I(\epsilon=\text{cause})(t - \min(T,t)) | X)}).
##' @param model Character string. Link function for the outcome model: \code{"exp"} or 
##'   \code{"lin"} (identity). For \code{"cif"}, \code{"logit"} is typically used.
##' @param Ydirect Optional numeric vector. Use this outcome Y with IPCW version instead of 
##'   constructing one from \code{outcome}.
##' @param typeATE Character string. Either \code{"II"} (censoring augmentation of the 
##'   estimating equation) or \code{"I"} (standard).
##' @param ... Additional arguments passed to lower-level functions (e.g., \code{binreg} 
##'   that fits the outcome model).
##'
##' @return An object of class \code{c("binreg", "ATE")} containing:
##' \item{coef}{Estimated coefficients from the outcome model.}
##' \item{riskDR}{Double-robust marginal risk estimates for each treatment level.}
##' \item{riskG}{G-formula marginal risk estimates for each treatment level.}
##' \item{difriskDR}{Difference in risks (ATE) using double-robust estimator.}
##' \item{difriskG}{Difference in risks (ATE) using G-formula estimator.}
##' \item{riskDR.iid, riskG.iid}{Influence functions for marginal risk estimates.}
##' \item{var, var.riskDR, var.riskG}{Variance-covariance matrices.}
##' \item{se.coef, se.riskDR, se.riskG}{Standard errors.}
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{binreg}}, \code{\link{logitIPCWATE}}, \code{\link{logitATE}}, 
##'   \code{\link{binregG}}
##' @examples
##' data(bmt)
##' dfactor(bmt)  <-  ~.
##'
##' brs <- binregATE(Event(time,cause)~tcell.f+platelet+age,bmt,time=50,cause=1,
##'	  treat.model=tcell.f~platelet+age)
##' summary(brs)
##' head(brs$riskDR.iid)
##' head(brs$riskG.iid)
##'
##' brsi <- binregATE(Event(time,cause)~tcell.f+tcell.f*platelet+tcell.f*age,bmt,time=50,cause=1,
##'	  treat.model=tcell.f~platelet+age)
##' summary(brsi)
##' head(brs$riskDR.iid)
##' head(brs$riskG.iid)
##' @seealso [kumarsim()] [kumarsimRCT()]
##' @aliases logitIPCWATE logitATE normalATE
##' @export
binregATE <- function(formula,data,cause=1,time=NULL,beta=NULL,treat.model=~+1,cens.model=~+1,
   offset=NULL,weights=NULL,cens.weights=NULL,se=TRUE,type=c("II","I"),
   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,typeATE="II",...)
{# {{{
  cl <- match.call()# {{{
###  m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

 call.id <- id;
 conid <- construct_id(id,nrow(X),namesX=rownames(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 idclust <- id; nclust <- nid

 ## take each data-record with its own iid or organize after idclust later 
 nid <- length(id); 
 id <- 0:(nid-1); 

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  ## {{{ setting up status, response, censoring model 
  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in%cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  statusC <- (status %in%cens.code) 
  data$id__ <- id 
  data$exit <- exit
  data$statusC <- statusC 
  n <- length(exit)
  cens.strata <- cens.nstrata <- NULL 

  call.cw <- cens.weights
  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  X <-  as.matrix(X)
  ###  X2  <- .Call("vecMatMat",X,X)$vXZ
  X2  <- .Call("vecCPMat",X)$XX
  ###
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)>=1) Causes <- ucauses[-ccc] else Causes <- ucauses
  obs <- (exit<=time & (status %in% Causes)) | (exit>=time)

 if (!is.null(Ydirect)) Y <-  Ydirect*obs/cens.weights else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time)/cens.weights)
     else { if (!competing) {
	     if (outcome[1]=="rmst") Y <-  c(pmin(exit,time)*obs)/cens.weights 
	     else Y <-  c((time-pmin(exit,time))*obs)/cens.weights 
            } else Y <- c((status %in% cause)*(time-pmin(exit,time))*obs)/cens.weights
     }
  }
 nevent <- sum((status %in% cause)*(exit<=time))

 ## }}}

## change id from call to id__ to get ordering as data
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))

  val <- binreg(formula,data,cause=cause,time=time,beta=beta,type=type,
	cens.model=cens.model,se=se,kaplan.meier=kaplan.meier,
        offset=offset,weights=weights,cens.weights=call.cw,
        cens.code=cens.code,no.opt=no.opt,method=method,augmentation=augmentation,
        outcome=outcome[1],model=model[1],Ydirect=Ydirect,...)

model <- val$model[1]

# {{{ computation of ate, att, atc and their influence functions

### treatment is rhs of treat.model 
if (length(all.vars(treat.model))==0) 
{ respformula  <-  as.formula( paste(all.vars(formula)[3],"~."))
  treat.model  <- update.formula(treat.model,respformula)
###  cat("Using propensity score model :\n "); print(treat.model)
}
treat.name <-  all.vars(treat.model)[1]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 
## treatvar, 1,2,...,nlev or 1,2
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
ytreat <- ntreatvar-1

## dropping cluster here 
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(treatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(treatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2
   p1lp <-   X %*% val$coef+offset
   if (model[1]=="logit") { p1 <- expit(p1lp) } else {
    if (model[1]=="exp") { p1 <- exp(p1lp); } else { p1 <- p1lp;}
   }


k <- 0
DePsia <- DariskG <- DaPsia <- list(); 
pia <- Ya <- riskG <- riska <- c(); 
datA <- dkeep(data,x=all.vars(formula))
xlev <- lapply(datA,levels)
formulanc <- drop.specials(formula,"cluster")
a <- nlevs[1]
for (a in nlevs) {# {{{
	k <- k+1
	datA[,treat.name] <- a
	Xa <- model.matrix(formulanc[-2],datA,xlev=xlev)
        lpa <- Xa %*% val$coef+offset
	if (model[1]=="logit") {
	   ma <- expit(lpa); Dma  <-  Xa*c(ma/(1+exp(lpa)))
	} else {
	    if (model[1]=="exp") { ma <- exp(lpa);  Dma<-c(ma)*Xa; } else { ma <- lpa; Dma <- Xa }
	}
	paka <- pal[,k]
	riska <- cbind(riska,((treatvar==a)/paka)*(Y-ma)+ma)
	riskG <- cbind(riskG,ma)
	Ya <- cbind(Ya,Y*((treatvar==a)/paka))
	pia <- cbind(pia,((treatvar==a)/paka))
        DariskG[[k]] <- apply(Dma,2,sum)
        DePsia[[k]] <-  apply(Dma*(1-(treatvar==a)/paka),2,sum)
        DaPsia[[k]] <-  apply(DPai*(treatvar==a)*c(Y-p1),2,sum)
}# }}}

augmentationATE <- 0
 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    Xtreat <- Xtreat[ord,,drop=FALSE]
    ytreat <- ytreat[ord]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    Ya <- Ya[ord,]
    pal <- pal[ord]
    piaC <- pia[ord,]
    cens.weights <- cens.weights[ord]
    lp <- c(X %*% val$coef+offset)
    obs <- obs[ord]
    p <- expit(lp)

  Y <- Y[ord]
  Yc <- matrix(Y,ncol=1)

 Y <- Y*weights 
 xx <- resC$cox.prep
 S0i2 <- S0i <- rep(0,length(xx$strata))
 S0i[xx$jumps+1]  <- 1/resC$S0
 S0i2[xx$jumps+1] <- 1/resC$S0^2
 ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
 ## to make \int h(s)/Ys  dM_i^C(s) 
 has  <-  apply(Ya,2,revcumsumstrata,xx$strata,xx$nstrata)
###    hattc  <-  apply(cbind(ytreat-pal*(1-ytreat)/(1-pal),-(1-ytreat)+(1-pal)*ytreat/pal)*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    btime <- 1*(exit<time)
    IhdLamhas <- apply(has*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    Uas <- matrix(0,nrow(xx$X),ncol(has))
    Uas[xx$jumps+1,] <- (resC$jumptimes<time)*has[xx$jumps+1,] /c(resC$S0)
    MGtas <- (Uas[,drop=FALSE]-IhdLamhas)*c(xx$weights)

    if (typeATE=="II") {
            h  <-  apply(Yc,2,revcumsumstrata,xx$strata,xx$nstrata)
	    IhdLam <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
	    U <- matrix(0,nrow(xx$X),1)
	    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
	    MGateII <- piaC*c(U[,drop=FALSE]-IhdLam)*c(xx$weights)
	    augmentationATE <- apply(MGateII,2,sum)

	    hapi  <-  apply(piaC,2,revcumsumstrata,xx$strata,xx$nstrata)
	    IhaidLam <- apply(hapi*c(h)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
	    Uai <- matrix(0,nrow(xx$X),ncol(hapi))
	    Uai[xx$jumps+1,] <- (resC$jumptimes<time)*c(h[xx$jumps+1,])*hapi[xx$jumps+1,]/c(resC$S0)^2
	    MGait <- (Uai[,drop=FALSE]-IhaidLam)*c(xx$weights)
	    MGtII <- MGateII - MGait
    }
    ### Censoring Variance Adjustment \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(xx$id)+1
    MGCiidas <- apply(MGtas,2,sumstrata,xx$id,mid)
    if (typeATE=="II") {
        MGtII <-  apply(MGtII,2,sumstrata,xx$id,mid)
        MGCiidas  <-  MGCiidas+MGtII
    } else MGtII <- 0
 
  }  else { MGCiid <- MGCiidas <- 0 }
## }}}

################################
### estimates risk, att, atc
################################
###val$riskDR <- apply(riska,2,mean)
val$riskDR <- apply(riska,2,mean)+augmentationATE/nid
val$riskG<-  apply(riskG,2,mean)
names(val$riskDR) <- paste("treat",nlevs,sep="")
names(val$riskG) <- paste("treat",nlevs,sep="")

################################
## iid's of marginal risk estimates 
################################

k <- 1
iidrisk <- c()
riskG.iid <- c()
for (a in nlevs) {
	iidbasea <- c(sumstrata(riska[,k]-val$riskDR[k],id,nid))
	iidcifa <- c(DePsia[[k]] %*% t(val$iid))
	iidpala <- c(DaPsia[[k]] %*% t(iidalpha))
	if (se)  iidGca <- MGCiidas[,k] else iidGca<-0 
        ###
	iidriskDR <- (iidbasea+iidcifa+iidpala+iidGca)/nid
        iidrisk <- cbind(iidrisk,iidriskDR)
	iidriskG <- c(sumstrata(riskG[,k]-val$riskG[k],id,nid))
	iidriskG <- c(iidriskG)/nid+c(DariskG[[k]] %*% t(val$iid))/nid
        riskG.iid <- cbind(riskG.iid,iidriskG)
	k <- k+1
}
# }}}

val$call.id <- call.id
val$name.id  <- name.id
val$id <- idclust
val$nid  <- nclust

# {{{ output variances and se for ate; cluster correction
val$iid <- apply(val$iid,2,sumstrata,idclust,nclust)
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$MGciid <- NULL
val$iid.naive <- NULL

## outcome model from binreg
iidrisk <- apply(iidrisk,2,sumstrata,idclust,nclust)
val$var.riskDR <- crossprod(iidrisk); 
val$se.riskDR <- diag(val$var.riskDR)^.5
val$riskDR.iid <- iidrisk

pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
contrast <- -pdiff(length(nlevs))
nncont <- c()
for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
rownames(contrast) <- nncont

mm <- estimate(coef=val$riskDR,vcov=val$var.riskDR,f=contrast)
val$difriskDR <- mm$coef 
names(val$difriskDR) <-  rownames(contrast) 
val$var.difriskDR <- mm$vcov 
val$se.difriskDR <- diag(val$var.difriskDR)^.5

riskG.iid <- apply(riskG.iid,2,sumstrata,idclust,nclust)
###
val$riskG.iid <- riskG.iid
val$var.riskG <- crossprod(val$riskG.iid)
val$se.riskG <- diag(val$var.riskG)^.5
###
mm <- estimate(coef=val$riskG,vcov=val$var.riskG,f=contrast)
val$difriskG <- mm$coef 
names(val$difriskG) <-  rownames(contrast) 
val$var.difriskG <- mm$vcov 
val$se.difriskG <- diag(val$var.difriskG)^.5

###### DR-estimator, G add names to iid 
if (!is.null(call.id)) {
    val$riskDR.iid <- nameme(iidrisk,name.id)
    val$riskG.iid  <- nameme(riskG.iid,name.id)
    val$iid        <- nameme(val$iid,name.id)
}
## }}}

  class(val) <- c("binreg","ATE")
  return(val)
}# }}}

##' @export
estimate.ATE  <- function(x,type=c("all","DR","G"),...)
{
if (type[1]=="all") {
iidGDR <- cbind(x$riskG.iid,x$riskDR.iid)
coefs <- c(x$riskG,x$riskDR)
names(coefs)  <- c(paste("G",names(x$riskG),sep="-"), paste("DR",names(x$riskG),sep="-"))
} else if (type[1]=="DR") {
iidGDR <- x$riskDR.iid
coefs <- x$riskDR
names(coefs)  <- paste("DR",names(x$riskG),sep="-")
} else {
iidGDR <- x$riskG.iid
coefs <- x$riskG
names(coefs)  <- paste("G",names(x$riskG),sep="-")
}
out <-estimate(coef=coefs,IC=iidGDR*nrow(iidGDR),...)
return(out)
}


##' G-Estimator for Binomial Regression Model (Standardized Estimates)
##'
##' Computes the G-estimator (G-formula) for standardized risk estimates based on a 
##' fitted \code{binreg} object. The G-estimator standardizes predictions over the 
##' covariate distribution in the data:
##' \deqn{ \hat F(t, A=a) = n^{-1} \sum_{i=1}^n \hat F(t, A=a, Z_i) }
##'
##' This function assumes that the first covariate in the original model formula 
##' represents the treatment or exposure variable (\eqn{A}). It calculates the 
##' marginal risk for specified values of \eqn{A} by averaging the conditional 
##' predictions over the observed covariate distribution \eqn{Z}.
##'
##' The function returns influence functions for these risk estimates, allowing 
##' for the computation of standard errors and confidence intervals.
##'
##' If the first covariate is a factor, contrasts between all levels are computed 
##' automatically. If it is continuous, specific values must be provided via 
##' \code{Avalues}.
##'
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
##' }
##'
##' @param x An object of class \code{"binreg"} obtained from \code{binreg()} or \code{logitIPCW()}.
##' @param data A data frame containing the covariates to be used for averaging the risk 
##'   estimates. This should ideally be the same data used to fit the model, or a 
##'   representative sample.
##' @param Avalues Numeric or factor vector specifying the values of the first covariate 
##'   (\eqn{A}) for which to compute standardized risks. 
##'   \itemize{
##'     \item If the first covariate is a factor and \code{Avalues} is \code{NULL}, 
##'       all levels of the factor are used.
##'     \item If the first covariate is continuous, \code{Avalues} must be provided.
##'   }
##' @param varname Optional character string specifying the name of the variable to be 
##'   treated as the treatment/exposure variable. If \code{NULL}, the first variable 
##'   in the model formula is used.
##'
##' @return An object of class \code{"survivalG"} containing:
##' \item{risk}{A table of standardized risk estimates for each value of \code{Avalues}.}
##' \item{risk.iid}{Influence functions for the standardized risk estimates.}
##' \item{difference}{Pairwise differences in risks between levels of \code{A}.}
##' \item{ratio}{Risk ratios between levels of \code{A}.}
##' \item{vcov}{Variance-covariance matrix of the risk estimates.}
##' \item{model}{The link function used in the original model.}
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{binreg}}, \code{\link{binregATE}}
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' bmt$event <- (bmt$cause!=0)*1
##' 
##' b1 <- binreg(Event(time,cause)~age+tcell+platelet,bmt,cause=1,time=50)
##' sb1 <- binregG(b1,bmt,Avalues=c(0,1,2))
##' summary(sb1)
##' @export
binregG <- function(x,data,Avalues=NULL,varname=NULL)
{# {{{

### dealing with first variable 
if (is.null(varname))  {
   treat.name <- all.vars(update.formula(x$formula,1~.))[1]
} else treat.name <- varname
treatvar <- data[,treat.name]

if (is.factor(treatvar)) {
   ## treatvar, levels to look at 
   if (is.null(Avalues)) nlevels <- levels(treatvar) else nlevels <- Avalues
} else {
   nlevels <- Avalues
}
if (is.null(nlevels)) stop("must specify values")

desx <- update_design(x$design,data=data)
if (is.null(desx$cluster)) { ## data must be same as data used for x
   id <- x$id
} else {
  id <- desx$cluster-1
}
datA <- data

DariskG <- list()
risks <- c()
k <- 0
for (a in nlevels) {# {{{
k <- k+1
datA[,treat.name] <- a
pa <- predict.binreg(x,datA,se=0,design=TRUE)
Xa <- attr(pa,"x")
lpa <- Xa %*% x$coef
if (x$model[1]=="logit") {
###   pa <- expit(lpa)
   Dma  <-  Xa*c(pa/(1+exp(lpa)))
} else if (x$model[1]=="exp") {
###   pa <- exp(lpa)
   Dma  <-  Xa*c(pa) 
} else {
###   pa <- lpa
   Dma  <-  Xa
}
risks <- cbind(risks,pa)
DariskG[[k]] <- apply(Dma,2,sum)
}# }}}

Grisk <- apply(risks,2,mean)
Gest <- list(Gest=Grisk,iid=t(t(risks)-Grisk))

nid <- max(x$id)+1
n <-  ndata <- length(unique(id))
risk.iid <- apply(Gest$iid,2,sumstrata,x$id,nid)/n 
DGiid <- risk.iid

for (a in seq_along(nlevels)) {
	DGiid[,1] <- c(x$iid  %*% DariskG[[a]])/n
	risk.iid[,a] <- risk.iid[,a]+ c(x$iid  %*% DariskG[[a]])/n
}
vv <- crossprod(risk.iid)

Gout <- estimate(coef=Gest$Gest,vcov=vv,labels=paste("risk",nlevels,sep=""))
ed <-  summary(estimate(coef=Gest$Gest,vcov=vv),f=function(p) p[-1]-p[1])
rd <- summary(estimate(coef=Gest$Gest,vcov=vv),f=function(p) log(p[-1]/p[1]),null=0)
out <- list(risk.iid=risk.iid,risk=Gout,difference=ed,ratio=rd,vcov=vv,model=x$model[1])
class(out) <- "survivalG"
return(out)
} ## }}}

##' @export
logitIPCWATE <- function(formula,data,cause=1,time=NULL,beta=NULL,treat.model=~+1, cens.model=~+1,
	   offset=NULL,weights=NULL,cens.weights=NULL,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
           outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster


 call.id <- id;
 conid <- construct_id(id,nrow(X),namesX=rownames(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 idclust <- id; nclust <- nid

 ## make all computations for each subject, later sum influence functions and sort after idclust 
 nid <- length(id); id <- 0:(nid-1); 

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  statusC <- (status %in% cens.code) 
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  n <- length(exit)

  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
###      cens.weights <- predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL

  if (is.null(beta)) beta <- rep(0,ncol(X))
  p <- ncol(X)

  X <-  as.matrix(X)
  X2  <- .Call("vecMatMat",X,X)$vXZ
  obs <- (exit<=time & status!=cens.code) | (exit>=time)
  weights <- obs*weights/c(cens.weights)
  cens.weights <- c(cens.weights)

  Y <- c((status==cause)*(exit<=time))

  if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst") Y <-  c(pmin(exit,time))
	     else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
 Yout <- Y
 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum(Y)

 ## default links
if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
 }

obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X %*% pp+offset)
if (model[1]=="exp") {
    p <- exp(lp) 
    D2logl <- c(weights*p)*X2 
} else if (model[1]=="lin") {
    p <- lp
    D2logl <- c(weights)*X2
} else if (model[1]=="logit") {
   p <- expit(lp)
   D2logl <- c(weights*p/(1+exp(lp)))*X2
} 

ploglik <- sum(weights*(Y-p)^2)
Dlogl <- weights*X*c(Y-p)
D2log <- apply(D2logl,2,sum)
gradient <- apply(Dlogl,2,sum)+augmentation
hessian <- matrix(D2log,length(pp),length(pp))

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,
	 iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid,weights=weights))
  
# {{{ computation of ate, att, atc and their influence functions
if (length(all.vars(treat.model))==0) 
{ respformula  <-  as.formula( paste(all.vars(formula)[3],"~."))
  treat.model  <- update.formula(treat.model,respformula)
###  cat("Using propensity score model :\n "); print(treat.model)
}

## dropping cluster here 
treat.model <- drop.specials(treat.model,"cluster")
treat <- glm(treat.model,data,family="binomial")
Xtreat <- model.matrix(treat$formula,data)
ytreat <- treat$y

lpa <- treat$linear.predictors 
pal <- expit(lpa)
iidalpha <- iid(treat,id=id)

### treatment is rhs of treat.model 
treat.name <-  all.vars(treat.model)[1]
dat0 <- data
dat0[,treat.name] <- 0 ## treat.contrast[1]
dat1 <- data
dat1[,treat.name] <- 1 ## treat.contrast[2]

formulanc <- drop.specials(formula,"cluster")

X1 <- model.matrix(formulanc[-2],dat1)
X0 <- model.matrix(formulanc[-2],dat0)

p11lp <- X1 %*% val$coef+offset
p10lp <- X0 %*% val$coef+offset
p1lp <-   X %*% val$coef+offset

if (model[1]=="exp") {
   p1 <- exp(p1lp); p10 <- exp(p10lp); p11 <- exp(p11lp);
} else if (model[1]=="lin") {
   p1 <- p1lp; p10 <- p10lp; p11 <- p11lp; 
} else if (model[1]=="logit") {
    p1 <- expit(p1lp); p10 <- expit(p10lp); p11 <- expit(p11lp);
} else stop("link functions must be logit,exp,lin\n") 

Yorig <- Y
Y <- c(Y*obs/cens.weights)
risk1 <- ytreat*(Y-p11)/pal+p11
risk0 <- (1-ytreat)*(Y-p10)/(1-pal)+p10
riskG1 <- p11
riskG0 <- p10

ntreat <- sum(ytreat)
###att <- ytreat*Y-(pal*(1-ytreat)*Y + (ytreat - pal)* p10)/(1-pal)
###atc <- ((1-pal)*ytreat*Y - ((ytreat - pal)* p11)/pal)-(1-ytreat)*Y

if (model[1]=="logit") {
	Dp1 <- X * c(p1/(1+exp(p1lp)))
	Dp11 <- X1 * c(p11/(1+exp(p11lp)))
	Dp10 <- X0 * c(p10/(1+exp(p10lp)))
} else if (model[1]=="lin") {
      Dp1 <- X; Dp11 <- X1; Dp10 <- X0;
} else {
      Dp1 <- c(p1)*X; Dp11 <- c(p11)*X1; Dp10 <- c(p10)*X0;
}

Dpai <-  - Xtreat * exp(-lpa)
D1mpai <-   Xtreat * exp(lpa)

DaPsi1 <-  apply( Dpai * ytreat * c( Y - p1),2,sum)
DaPsi0 <-  apply( D1mpai * (1-ytreat) * c( Y - p1),2,sum)
DePsi1 <-  apply( Dp11 * ( 1- ytreat/pal),2,sum)
DePsi0 <-  apply( Dp10 * ( 1- (1-ytreat)/(1-pal)),2,sum)

###DePsiatt <- - apply( Dp10* (ytreat-pal)/(1-pal),2,sum)
###DaPsiatt <- apply(c((1-ytreat)*(Y-p10))*D1mpai,2,sum)
###
###DePsiatc <- -apply( Dp11* (ytreat-pal)/pal,2,sum)
###DaPsiatc <- apply(c(ytreat*(Y-p11))*Dpai,2,sum)

DriskG1 <- apply(Dp11,2,sum)
DriskG0 <- apply(Dp10,2,sum)
DdifriskG <- DriskG1-DriskG0

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    Xtreat <- Xtreat[ord,,drop=FALSE]
    ytreat <- ytreat[ord]
    pal <-   pal[ord]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    lp <- c(X %*% val$coef+offset)
    if (model[1]=="exp") { p <- exp(lp) } else if (model[1]=="lin") { p <- lp } else if (model[1]=="logit") {p <- expit(lp) } 
    Yglm <- weights*(Yout[ord]-p)
    Y <- Yout[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    h10  <-  apply(cbind((ytreat/pal)*Y,I(ytreat==0)/(1-pal))*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
###  hattc  <-  apply(cbind(ytreat-pal*(1-ytreat)/(1-pal),-(1-ytreat)+(1-pal)*ytreat/pal)*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<=time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<=time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    IhdLamh10 <- apply(h10*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U10 <- matrix(0,nrow(xx$X),2)
    U10[xx$jumps+1,] <- h10[xx$jumps+1,] /c(resC$S0)
    MGt10 <- (U10[,drop=FALSE]-IhdLamh10)*c(xx$weights)

###    IhdLamhattc <- apply(hattc*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
###    Uattc <- matrix(0,nrow(xx$X),2)
###    Uattc[xx$jumps+1,] <- hattc[xx$jumps+1,]/c(resC$S0)
###    MGtattc <- (Uattc[,drop=FALSE]-IhdLamhattc)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)+1
    MGCiid <- apply(MGt,2,sumstrata,xx$id,mid)
    MGCiid10 <- apply(MGt10,2,sumstrata,xx$id,mid)
###    MGCiidattc <- apply(MGtattc,2,sumstrata,xx$id,mid)
  }  else { MGCiidattc <- MGCiid <- 0; MGCiid10 <- 0 }
## }}}

val$MGciid <- MGCiid
val$MGciid10 <- MGCiid10
val$id <- id
val$call.id <- call.id
val$name.id <- name.id
val$nid <- nid
val$iid.naive <- val$iid 
if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian)
val$naive.var <- val$var
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$cause <- cause
val$cens.code <- cens.code 

### estimates risk, att, atc
val$riskDR <- c(mean(risk1),mean(risk0))
val$riskG<- c(mean(riskG1),mean(riskG0))

###val$att <- sum(att)/ntreat
###val$atc <- sum(atc)/(n-ntreat)
###val$attc <- c(val$att,val$atc)
###names(val$attc) <- c("ATT","ATC")

names(val$riskDR) <- paste("treat",1:0,sep="-")
names(val$riskG) <- paste("treat",1:0,sep="-")

## iid's of marginal risk estimates 
iidbase1 <- c(risk1-val$riskDR[1])
iidcif1 <- c(c(DePsi1) %*% t(val$iid))
iidpal1 <- c(c(DaPsi1) %*% t(iidalpha))
if (se)  {
iidGc1 <- MGCiid10[,1]; iidGc0 <- MGCiid10[,2]
###iidGatt <-  MGCiidattc[,1]; iidGatc <-  MGCiidattc[,2]
}  else { iidGc1 <- iidGatt  <- iidGatc  <- iidGc0  <- 0 } 

iidbase0 <- c(risk0-val$riskDR[2])
iidcif0 <- c(c(DePsi0) %*% t(val$iid))
iidpal0 <- c(c(DaPsi0) %*% t(iidalpha))

iidrisk1 <- iidbase1+iidcif1+iidpal1+iidGc1
iidrisk0 <- iidbase0+iidcif0+iidpal0+iidGc0
difriskiid <- (iidrisk1-iidrisk0)/n

iidrisk <- cbind(iidrisk1,iidrisk0)/n

###iidatt <- att-ytreat*val$att
###iidatc <- atc-(1-ytreat)*val$atc
###iidcifatt <- c(c(DePsiatt) %*% t(val$iid))
###iidpalatt <- c(c(DaPsiatt) %*% t(iidalpha))  
###iidcifatc <- c(c(DePsiatc) %*% t(val$iid))
###iidpalatc <- c(c(DaPsiatc) %*% t(iidalpha)) 
###iidatt <- iidatt+iidcifatt+iidpalatt+iidGatt
###iidatc <- iidatc+iidcifatc+iidpalatc+iidGatc

# }}}


val$call.id <- call.id
val$name.id  <- name.id

# {{{ output variances and se for ate; cluster correction
val$id <- idclust
val$nid  <- nclust

## make iid form before combining into clusters 
riskG.iid <- cbind(c(p11-val$riskG[1]) + c(DriskG1 %*% t(val$iid)),
	          c(p10-val$riskG[2]) + c(DriskG0 %*% t(val$iid)))/n

## outcome model from IPCW logistic regression 
val$iid <- apply(val$iid,2,sumstrata,idclust,nclust)
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$MGciid <- NULL
val$iid.naive <- NULL

## DR estimates 
iidrisk <- apply(iidrisk,2,sumstrata,idclust,nclust)
val$var.riskDR <- crossprod(iidrisk); 
val$se.riskDR <- diag(val$var.riskDR)^.5
val$riskDR.iid <- iidrisk

###nlevs <- 2
###pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
###contrast <- -pdiff(length(nlevs))
###nncont <- c()
###for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
###rownames(contrast) <- nncont
contrast <- rbind(c(1,-1))

mm <- estimate(coef=val$riskDR,vcov=val$var.riskDR,f=rbind(c(1,-1)))
val$difriskDR <- mm$coef 
names(val$difriskDR) <-  rownames(contrast) 
val$var.difriskDR <- mm$vcov 
val$se.difriskDR <- diag(val$var.difriskDR)^.5

riskG.iid <- apply(riskG.iid,2,sumstrata,idclust,nclust)
###
val$riskG.iid <- riskG.iid
val$var.riskG <- crossprod(riskG.iid)
val$se.riskG <- diag(val$var.riskG)^.5
###
mm <- estimate(coef=val$riskG,vcov=val$var.riskG,f=contrast)
val$difriskG <- mm$coef 
names(val$difriskG) <-  rownames(contrast) 
val$var.difriskG <- mm$vcov 
val$se.difriskG <- diag(val$var.difriskG)^.5

### DR-estimator ; G -estimator sort after namid; also outcome model
if (!is.null(call.id)) {
    val$riskDR.iid <- nameme(iidrisk,name.id)
    val$riskG.iid  <- nameme(riskG.iid,name.id)
    val$iid <-       nameme(val$iid,name.id)
}
## }}}

  val$model <- model[1]
  val$outcome <- outcome[1]
  val$call.id <- call.id
  class(val) <- c("binreg","ATE")
  return(val)
}# }}}

##' @export
logitATE <- function(formula,data,binreg=TRUE,...)
{# {{{
   ## use IPCW machine in no-censoring case
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster", "offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (inherits(Y,"Event")) {
    if (binreg) out <- binregATE(formula,data,...) else out <- logitIPCWATE(formula,data,...)
    } else {
      response <- all.vars(formula)[1]
      Ydirect <-  as.numeric(data[,response]) 
      data$time <- 2
      data$event <- 1
      time <- 2
      Survform <-  update.formula(formula,Event(time,event)~.)
      n <- nrow(data)
      if (binreg)
      out <- binregATE(Survform,data,se=0,cens.weights=rep(1,n),time=time,Ydirect=Ydirect,...)
      else
      out <- logitIPCWATE(Survform,data,se=0,cens.weights=rep(1,n),time=time,Ydirect=Ydirect,...)
    }

   return(out)
}# }}}

##' @export
normalATE <- function(formula,data,binreg=TRUE,model="lin",...)
{# {{{
   if (binreg) 
   out <- logitATE(formula,data,binreg=binreg,model=model,outcome="rmst",...)
   else 
   out <- logitATE(formula,data,binreg=binreg,...)
   return(out)
}# }}}

library("tinytest")
library(mets)
library(riskRegression)
## break ties
set.seed(100)
data(bmt)
bmt$time <- bmt$time+runif(408)*.001
dfactor(bmt)  <-  ~tcell+platelet
bmt$id <- 1:408

test_ate_wglm <- function() { ## {{{
brs <- logitIPCW(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1)
estimate(brs)
###
lbrs <- logitIPCWATE(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,treat.model=tcell~platelet+age)
###     
eCR.wglm <- wglm(Surv(time,cause)~ tcell.f+platelet+age,
	      formula.censor =~+1, times = 50, data = bmt, cause = 1, product.limit = TRUE)
summary(eCR.wglm)
estimate(lbrs)$coefmat
estimate(brs)$coefmat
####
awglm <- ate(eCR.wglm, data = bmt, times = 50, treatment = tcell.f~platelet+age, 
	     verbose = FALSE,estimator=c("G-formula","AIPTW","IPTW"))
aa <- confint(awglm)
aa$diffRisk
aa$meanRisk
summary(lbrs)$ateG
summary(lbrs)$ateDR
gg <- aa$meanRisk[2:1,c(4,5)]
dr <- aa$meanRisk[6:5,c(4,5)]
ddr  <-  dr- summary(lbrs)$ateDR[1:2,1:2]
estdr <- ddr[,1]
dgg  <-  gg-summary(lbrs)$ateG[1:2,1:2]
###
Giid <- max(abs(cbind(awglm$iid$GFORMULA[[1]], awglm$iid$GFORMULA[[2]])- lbrs$riskG.iid[,c(2,1)]))
DRiid <- max(abs(cbind(awglm$iid$AIPTW[[1]], awglm$iid$AIPTW[[2]])- lbrs$riskDR.iid[,c(2,1)]))
dgdr <- max( abs(c(unlist(ddr),unlist(dgg))))
c(dgdr,Giid,DRiid)
###
dgg <- max( abs(unlist(dgg)))
ddr <- max( abs(c(unlist(ddr))))
c(dgg,max(abs(estdr)),ddr,Giid,DRiid)
expect_true((dgg < 0.00001) &  (max(abs(estdr)) < 0.00001) & Giid < 0.00001) 
## standard error for DR is missing a censoring term from ate function

brs <- logitIPCW(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,
       cens.model=~strata(tcell,platelet))
lbrs <- logitIPCWATE(Event(time,cause)~tcell+platelet+age,bmt,time=50,cause=1,
       treat.model=tcell~platelet+age,cens.model=~strata(tcell,platelet),se=TRUE)
###     
eCR.wglm <- wglm(Surv(time,cause)~ tcell.f+platelet+age,
	      formula.censor =~strata(tcell,platelet), times = 50, data = bmt, cause = 1, product.limit = TRUE)
d <- summary(eCR.wglm)
estimate(brs)$coefmat
estimate(lbrs)$coefmat
dcoef <- max(abs(d[[1]][,c(1,2)] - estimate(brs)$coefmat[,c(1,2)]))
###
awglm <- ate(eCR.wglm, data = bmt, times = 50, treatment = tcell.f~platelet+age, 
	     verbose = FALSE,estimator=c("G-formula","AIPTW","IPTW"))
summary(awglm)
aa <- confint(awglm)
aa$diffRisk
aa$meanRisk
summary(lbrs)$ateG
summary(lbrs)$ateDR
gg <- aa$meanRisk[2:1,c(4,5)]
dr <- aa$meanRisk[6:5,c(4,5)]
ddr  <-  dr- summary(lbrs)$ateDR[1:2,1:2]
ddr
estdr <- ddr[,1]
dgg  <-  gg-summary(lbrs)$ateG[1:2,1:2]
dgg
###
Giid <- max(abs(cbind(awglm$iid$GFORMULA[[1]], awglm$iid$GFORMULA[[2]])- lbrs$riskG.iid[,c(2,1)]))
DRiid <- max(abs(cbind(awglm$iid$AIPTW[[1]],awglm$iid$AIPTW[[2]])- lbrs$riskDR.iid[,c(2,1)]))
###
dgg <- max( abs(unlist(dgg)))
ddr <- max( abs(c(unlist(ddr))))
c(dgg,max(abs(estdr)),ddr,Giid,DRiid)
expect_true((dgg < 0.00001) &  (max(abs(estdr)) < 0.00001) & Giid < 0.00001) 
## standard error for DR is missing a censoring term from ate function

################################################################
## no censoring case ###########################################
################################################################
bmt$bin <- (bmt$cause %in% c(1,2))
## binregATE based 
lbrs <- logitATE(bin~tcell.f+platelet+age,bmt,treat.model=tcell.f~platelet+age)
## logitIPCWATE
llbrs <- logitATE(bin~tcell.f+platelet+age,bmt,treat.model=tcell.f~platelet+age,binreg=FALSE)
###     
eCR.wglm <- glm(bin~tcell.f+platelet+age,data=bmt,family=binomial())
## robust se's
estimate(eCR.wglm)
estimate(lbrs)$coefmat
estimate(llbrs)$coefmat
###
awglm <- ate(eCR.wglm, data = bmt, times = 50, treatment = tcell.f~platelet+age, verbose = FALSE,estimator=c("G-formula","AIPTW","IPTW"))
aa <- confint(awglm)
aa$diffRisk
aa$meanRisk
summary(lbrs)$ateG
summary(llbrs)$ateG
summary(lbrs)$ateDR
summary(llbrs)$ateDR
gg <- aa$meanRisk[1:2,c(4,5)]
dr <- aa$meanRisk[5:6,c(4,5)]
ddr  <-  dr- summary(lbrs)$ateDR[1:2,1:2]
dgg  <-  gg- summary(lbrs)$ateG[1:2,1:2]
#
###
Giid <- max(abs(cbind(awglm$iid$GFORMULA[[1]],awglm$iid$GFORMULA[[2]])-lbrs$riskG.iid[,c(1,2)]))
DRiid <- max(abs(cbind(awglm$iid$AIPTW[[1]],awglm$iid$AIPTW[[2]])-lbrs$riskDR.iid[,c(1,2)]))
c(Giid,DRiid)
Giid2 <- max(abs(cbind(awglm$iid$GFORMULA[[1]],awglm$iid$GFORMULA[[2]])-llbrs$riskG.iid[,c(2,1)]))
DRiid2 <- max(abs(cbind(awglm$iid$AIPTW[[1]],awglm$iid$AIPTW[[2]])-llbrs$riskDR.iid[,c(2,1)]))
c(Giid2,DRiid2)
dgdr <- max( abs(c(unlist(ddr),unlist(dgg))))
c(dgdr,Giid,DRiid)
dgg <- max( abs(unlist(dgg)))
ddr <- max( abs(c(unlist(ddr))))
c(dgg,ddr,Giid,DRiid)
expect_true((dgg < 0.00001) &  (ddr < 0.00001) & (max(c(Giid,DRiid)) < 0.00001)) 
} ## }}}
test_ate_wglm()

# binreg simple-icpw versus prodlim
library("tinytest")

test_binreg_cif <- function() {

  library(mets)
  data(bmt)
  bmt$time <-  bmt$time + runif(nrow(bmt)) * 0.001
  out <- binreg(Event(time, cause) ~ -1+factor(platelet),
                bmt, time=30, cens.model=~strata(platelet))

  summary(out)
  pp <- predict(out, data.frame(platelet = 0:1, tcell = 0:1))
  pp

  cif1 <- cif(Event(time,cause) ~ strata(platelet), bmt)
  scif <- summary(cif1,time=30)
  cif50 <- rbind(scif$pbaseci[[1]],scif$pbaseci[[2]])
  cif50
  dif <- cif50[,2:3] - pp[,1:2]
  dif

  ## estimate (same) and standard errors (close)
  expect_true( ((sum(abs(dif[,1])) < 0.0001) & (sum(abs(dif[,2])) < 0.0002)) )
}
test_binreg_cif()


test_binreg_cifClust <- function() {

  library(mets)
  data(bmt)
  bmt$time <-  bmt$time + runif(nrow(bmt)) * 0.001
  bmt$id2 <- sample(1:100,408,replace=TRUE)
  bmt$id <- 1:408
  bmt2 <- cbind(bmt,bmt)
  bmt2$time <- bmt2$time+runif(nrow(bmt2)) * 0.001

  out <- binreg(Event(time, cause) ~ -1+factor(platelet)+cluster(id),
                bmt2, time=30, cens.model=~strata(platelet))

  summary(out)
  pp <- predict(out, data.frame(platelet = 0:1, tcell = 0:1))
  pp

  cif1 <- cif(Event(time,cause) ~ strata(platelet)+cluster(id), bmt2)
  scif <- summary(cif1,time=30)
  cif50 <- rbind(scif$pbaseci[[1]],scif$pbaseci[[2]])
  cif50
  dif <- cif50[,2:3] - pp[,1:2]
  dif

  ## estimate (same) and standard errors (close)
  expect_true( ((sum(abs(dif[,1])) < 0.0001) & (sum(abs(dif[,2])) < 0.0002)) )
}
test_binreg_cifClust()

# cifreg versus crr-cmprsk
library("tinytest")

test_cluster <- function() {
  library(mets)
  set.seed(100)
  data(bmt)
  bmt$id <- 1:nrow(bmt)
  bmt2 <- rbind(bmt,bmt)
  bmt2$time <-  bmt2$time + runif(nrow(bmt2)) * 0.001

  fg <- cifregFG(Event(time,cause)~tcell+age+platelet,bmt)
  estimate(fg)
  ###
  fg2 <- cifregFG(Event(time,cause)~tcell+age+platelet+cluster(id),bmt2)
  estimate(fg2)
  d1 <- max(abs(fg2$se.coef-fg$se.coef))

  fg <- cifreg(Event(time,cause)~tcell+age+platelet,bmt)
  estimate(fg)
  ###
  fg2 <- cifreg(Event(time,cause)~tcell+age+platelet+cluster(id),bmt2)
  estimate(fg2)
  d2 <- max(abs(fg2$se.coef-fg$se.coef))

  fg <- phreg(Event(time,cause!=0)~tcell+age+platelet,bmt)
  estimate(fg)
  ###
  fg2 <- phreg(Event(time,cause!=0)~tcell+age+platelet+cluster(id),bmt2)
  estimate(fg2)
  d3 <- max(abs(fg2$var-fg$var))

  fg <- binreg(Event(time,cause!=0)~tcell+age+platelet,bmt,time=30)
  estimate(fg)
  ###
  fg2 <- binreg(Event(time,cause!=0)~tcell+age+platelet+cluster(id),bmt2,time=30)
  estimate(fg2)
  d4 <- max(abs(fg2$se.coef-fg$se.coef))

  dfactor(bmt) <- tcell.f~tcell
  dfactor(bmt2) <- tcell.f~tcell
###
  fg <- binregATE(Event(time,cause!=0)~tcell.f+age+platelet,bmt,time=30,
		  treat.model=tcell.f~age+platelet,cens.model=~strata(tcell,platelet))
  sfg <- summary(fg)
  ###
  fg2 <- binregATE(Event(time,cause!=0)~tcell.f+age+platelet+cluster(id),bmt2,time=30,
		  treat.model=tcell.f~age+platelet,cens.model=~strata(tcell,platelet))
  sfg2 <- summary(fg2)
  ateg <- max(abs(sfg$ateG[,2]-sfg2$ateG[,2] ))
  ateDR <- max(abs(sfg$ateDR[,2]-sfg2$ateDR[,2] ))

  dd <- max(c(d1,d2,d3,d4,ateg,ateDR))

  expect_true( ((d3 < 0.0011) & (max(c(d1,d2,d4,ateg,ateDR)) < 0.001)) )
} 
test_cluster()

##' Clayton-Oakes frailty model
##'
##' @title Clayton-Oakes model with piece-wise constant hazards
##' @param formula formula specifying the marginal proportional (piecewise constant) hazard structure with the right-hand-side being a survival object (Surv) specifying the entry time (optional), the follow-up time, and event/censoring status at follow-up. The clustering can be specified using the special function \code{cluster} (see example below).
##' @param data Data frame
##' @param cluster Variable defining the clustering (if not given in the formula)
##' @param var.formula Formula specifying the variance component structure (if not given via the cluster special function in the formula) using a linear model with log-link.
##' @param cuts Cut points defining the piecewise constant hazard
##' @param type when equal to \code{two.stage}, the Clayton-Oakes-Glidden estimator will be calculated via the \code{timereg} package
##' @param start Optional starting values
##' @param control Control parameters to the optimization routine
##' @param var.invlink Inverse link function for variance structure model
##' @param ... Additional arguments
##' @author Klaus K. Holst
##' @examples
##' set.seed(1)
##' d <- subset(sim_ClaytonOakes(500,4,2,1,stoptime=2,left=2),truncated)
##' e <- ClaytonOakes(survival::Surv(lefttime,time,status)~x+cluster(~1,cluster),
##'                   cuts=c(0,0.5,1,2),data=d)
##' e
##'
##' d2 <- sim_ClaytonOakes(500,4,2,1,stoptime=2,left=0)
##' d2$z <- rep(1,nrow(d2)); d2$z[d2$cluster%in%sample(d2$cluster,100)] <- 0
##' ## Marginal=Cox Proportional Hazards model:
##' ## ts <- ClaytonOakes(survival::Surv(time,status)~timereg::prop(x)+cluster(~1,cluster),
##' ##                   data=d2,type="two.stage")
##' ## Marginal=Aalens additive model:
##' ## ts2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),
##' ##                    data=d2,type="two.stage")
##' ## Marginal=Piecewise constant:
##' e2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~-1+factor(z),cluster),
##'                    cuts=c(0,0.5,1,2),data=d2)
##' e2
##'
##' e0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~-1+factor(z),cluster),
##'                    cuts=c(0,0.5,1,2),data=d2)
##' ##ts0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~1,cluster),
##' ##                   data=d2,type="two.stage")
##' ##plot(ts0)
##' ##plot(e0)
##'
##' e3 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),cuts=c(0,0.5,1,2),
##'                    data=d,var.invlink=identity)
##' e3
##' @export
ClaytonOakes <- function(formula,data=parent.frame(),cluster,var.formula=~1,cuts=NULL,type="piecewise",start,control=list(),var.invlink=exp,...) {

  mycall <- match.call()
  dots <- list(...)
  formulaId <- Specials(formula,"cluster") 
  formulaStrata <- Specials(formula,"strata")
  formulaSt <- "~."
  formulaProp <- Specials(formula,"prop")
  if (!is.null(formulaId)) {
    var.formulaId <- ~1
    if (length(formulaId)>1) {
      var.formula <- as.formula(formulaId[[1]])
      formulaId <- formulaId[[2]]
    }
    cluster <- formulaId
    mycall$cluster <- cluster
    formulaSt <- paste(formulaSt,paste("-cluster(",paste(var.formula,collapse=""),
                                       ",",formulaId,")"))
  }
  formulaSt <- paste(formulaSt,paste("-strata(",paste(formulaStrata,collapse="+"),")"))
  formula <- update(formula,formulaSt)
  
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  if (missing(cluster)) stop("Missing 'cluster' variable")
  ngamma <- 0
  data <- data[order(data[,cluster]),]
  Z <- model.matrix(var.formula,data)
  ngamma <- ncol(Z)

  if (type!="piecewise") {
	  stop("two.stage from timereg not called any more \n"); 
    timeregmod <- ifelse(length(formulaProp)>0,"cox.aalen","aalen")
    timeregmod <- get(timeregmod, asNamespace("timereg"))
    if (is.null(dots$robust)) dots$robust <- 0
    args <- c(list(formula=formula,data=data,max.clust=NULL,clusters=data[,cluster]),dots)
    marg <- do.call(timeregmod, args)    
    return(two.stage(marg,data=data,theta.des=Z,var.link=1,...))
  }
  
  
  timevar <- terms(formula)[[2]]
  if (is.call(timevar)) {
    delayedentry <- (length(timevar)==4)*1
    entry <- NULL
    if (delayedentry==1)
      entry <- as.character(timevar[[2]])
    causes <- timevar[[3+delayedentry]]
    timevar <- timevar[[2+delayedentry]]
  }  
  timevar <- as.character(timevar)
  causes <- as.character(causes)
  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  X <- NULL
  nbeta <- 0  
  if (length(covars)>0) {
##    X <- model.matrix(as.formula(paste("~-1+",paste(covars,collapse="+"))),data)
    X <- model.matrix(update(formula,.~.+1),data)[,-1,drop=FALSE]
    nbeta <- ncol(X)
  }
  
  if (is.data.frame(data)) {
    mydata <- data.frame(T=data[,timevar],status=data[,causes],cluster=data[,cluster],entry=0)
    if (!is.null(entry)) {
      mydata$entry <- data[,entry]
    }
  } else {
    mydata <- data.frame(T=get(timevar,envir=data),status=get(causes,envir=data),cluster=get(cluster,envir=data),entry=0)
    if (!is.null(entry))      
      mydata$entry <- get(entry,envir=data)
  }
  if (is.null(cuts)) {
    cuts <- c(0,max(mydata$T))
  }
  if (max(mydata$T)>tail(cuts,1)) stop("Interval does not embed time observations")
  if (any(with(mydata, T<entry))) stop("Entry time occuring after event")

  ucluster <- unique(mydata$cluster)
  
  npar <- length(cuts)-1
  if (!is.null(X)) npar <- npar+ncol(X)
  npar <- npar+ncol(Z)
  p0 <- rep(0.1,npar)
  if (!missing(start)) p0 <- c(start,rep(0,max(0,length(npar)-length(start))))
  if (!is.null(dots$theta)) p0[1] <- dots$theta

  invlinkname <- as.character(substitute(var.invlink))

  
  obj <- function(p) {
    varpar <- p[seq(ngamma)]
    p <- p[-seq(ngamma)]
    ##    theta0 <- rep(exp(varpar),length(ucluster));
    theta0 <- Z%*%varpar
    if (exists(invlinkname)) theta0 <- var.invlink(theta0)
    multhaz <- rep(1,nrow(mydata))
    if (!is.null(X)) {
      nbeta <- ncol(X)
      beta <- p[seq(nbeta)]
      p <- p[-seq(nbeta)]
      multhaz <- exp(X%*%beta)
    }
    if (!is.null(dots$theta)) theta0 <- cbind(rep(dots$theta,length(theta0)))
    res <- .Call("claytonoakes",
           ds=mydata$status,ts=mydata$T,es=mydata$entry,
           allcs=mydata$cluster,cs=ucluster, cuts=cuts,
           hs=exp(p),mulths=multhaz,
           var=theta0,PACKAGE="mets")$logLik    
    return(-res)
  }
  opt <- tryCatch(nlminb(p0,obj,control=control),error=function(x) NULL)
  if (is.null(opt)) stop("Critical optmization problem")
  if (any(is.na(opt$par)) | any(!is.finite(opt$par)) | any(is.nan(opt$par)) ) {
    V <- matrix(NA,length(p0),length(p0))
  } else {    
    I <- numDeriv::hessian(obj,opt$par)
    ee <- tryCatch(eigen(I),error=function(x) NULL); 
    if (!is.null(ee)) {
      threshold <- 1e-12
      idx <- ee$values>threshold
      ee$values[idx] <- 1/ee$values[idx];
      if (!all(idx))
        ee$values[!idx] <- 0
      V <- with(ee, vectors%*%diag(values)%*%t(vectors))
    } else {
      V <- matrix(NA,length(p0),length(p0))
    }
  }
  res <- list(coef=opt$par,vcov=V,cuts=cuts,nbeta=nbeta,ngamma=ngamma,betanames=colnames(X),gammanames=colnames(Z),opt=opt,invlink=var.invlink,invlinkname=invlinkname)
  class(res) <- "claytonoakes"
  return(res)
}

##################################################

##' @export
print.claytonoakes <- function(x,...) {
  print(summary(x))
}

##' @export
print.summary.claytonoakes <- function(x,...) {
  printCoefmat(x$coef[,c(1,3,4)],...)
  cat("\nDependence parameters:\n")
  printCoefmat(x$var,...)
  invisible(x)
}

##' @export
summary.claytonoakes <- function(object,...) {
  mycoef <- matrix(nrow=length(object$coef),ncol=4)
  mycoef[,1:2] <- cbind(object$coef,sqrt(diag(object$vcov)))
  mycoef[,3:4] <- cbind(mycoef[,1]-qnorm(0.975)*mycoef[,2],mycoef[,1]+qnorm(0.975)*mycoef[,2])
  colnames(mycoef) <- c("Estimate","Std.Err","2.5%","97.5%")
  if (length(object$cuts))
  cutnames <- levels(cut(0,breaks=object$cuts))
  varname <- switch(object$invlinkname,exp="log-Var:",identity="Var:",paste("inv",object$invlinkname,"-Var:",sep=""))
  rownames(mycoef) <- c(paste(varname,object$gammanames,sep=""),object$betanames,cutnames)
  mycoef[-seq(object$ngamma),] <- exp(mycoef[-seq(object$ngamma),])
  varcoef <- object$invlink(mycoef[seq(object$ngamma),c(1,3,4),drop=FALSE])
  rownames(varcoef) <- object$gammanames
  varcoef <- cbind(varcoef,1/(1+2/varcoef))
  colnames(varcoef)[c(1,4)] <- c("Variance","Kendall's tau")  
  res <- list(coef=mycoef,var=varcoef)
  class(res) <- "summary.claytonoakes"
  res
}

##' @export
plot.claytonoakes <- function(x,chaz=TRUE,add=!is.null(dev.list()),col="darkblue",...) {
  haz <- summary(x)$coef[-seq(x$nbeta+x$ngamma),,drop=FALSE]
  t <- x$cuts
  L <- approxfun(t,f=1,cumsum(c(0,haz[,1]*diff(t))),method="linear")
  if (add) {
    lines(t,L(t),col=col,...)
  } else {
    plot(t,L(t),type="l",col=col,...)
  }
  invisible(x)  
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets)

data(ttpd) 
dtable(ttpd,~entry+time2)
out <- interval_logitsurv_discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
summary(out)

dfactor(ttpd) <- entry.f~entry
out <- cumoddsreg(entry.f~X1+X2+X3+X4,ttpd)
summary(out)

## -----------------------------------------------------------------------------
set.seed(1000) # to control output in simulations for p-values below.
n <- 200
Z <- matrix(rbinom(n*4,1,0.5),n,4)
outsim <- simlogitSurvd(out$coef,Z)
outsim <- transform(outsim,left=time,right=time+1)
outsim <- dtransform(outsim,right=Inf,status==0)
outss <- interval_logitsurv_discrete(Interval(left,right)~+X1+X2+X3+X4,outsim)
summary(outss)

pred <- predictlogitSurvd(out,se=TRUE)
plotSurvd(pred,se=TRUE)

## -----------------------------------------------------------------------------
test <- 0 
if (test==1) {

require(icenReg)
data(IR_diabetes)
IRdia <- IR_diabetes
## removing fully observed data in continuous version, here making it a discrete observation 
IRdia <- dtransform(IRdia,left=left-1,left==right)
dtable(IRdia,~left+right,level=1)

ints <- with(IRdia,dInterval(left,right,cuts=c(0,5,10,20,30,40,Inf),show=TRUE) )
}

## -----------------------------------------------------------------------------
if (test==1) {
ints$Ileft <- ints$left
ints$Iright <- ints$right
IRdia <- cbind(IRdia,data.frame(Ileft=ints$Ileft,Iright=ints$Iright))
dtable(IRdia,~Ileft+Iright)
# 
#       Iright   1   2   3   4   5 Inf
# Ileft                               
# 0             10   1  34  25   4   0
# 1              0  55  19  17   1   1
# 2              0   0 393  16   4   0
# 3              0   0   0 127   1   0
# 4              0   0   0   0  21   0
# 5              0   0   0   0   0   2

outss <- interval.logitsurv.discrete(Interval(Ileft,Iright)~+gender,IRdia)
#            Estimate Std.Err    2.5%    97.5%   P-value
# time1        -3.934  0.3316 -4.5842 -3.28418 1.846e-32
# time2        -2.042  0.1693 -2.3742 -1.71038 1.710e-33
# time3         1.443  0.1481  1.1530  1.73340 1.911e-22
# time4         3.545  0.2629  3.0295  4.06008 1.976e-41
# time5         6.067  0.7757  4.5470  7.58784 5.217e-15
# gendermale   -0.385  0.1691 -0.7165 -0.05351 2.283e-02
summary(outss)
outss$ploglik
# [1] -646.1946

fit <- ic_sp(cbind(Ileft, Iright) ~ gender, data = IRdia, model = "po")
# 
# Model:  Proportional Odds
# Dependency structure assumed: Independence
# Baseline:  semi-parametric 
# Call: ic_sp(formula = cbind(Ileft, Iright) ~ gender, data = IRdia, 
#     model = "po")
# 
#            Estimate Exp(Est)
# gendermale    0.385     1.47
# 
# final llk =  -646.1946 
# Iterations =  6 
# Bootstrap Samples =  0 
# WARNING: only  0  bootstrap samples used for standard errors. 
# Suggest using more bootstrap samples for inference
summary(fit)

## sometimes NR-algorithm needs modifications of stepsize to run 
## outss <- interval_logitsurv_discrete(Interval(Ileft,Iright)~+gender,IRdia,control=list(trace=TRUE,stepsize=1.0))
}


## -----------------------------------------------------------------------------

data(ttpd) 
dtable(ttpd,~entry+time2)
ttpd <- dfactor(ttpd,fentry~entry)
out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
summary(out)

out$ploglik

if (test==1) {
### library(ordinal)
### out1 <- clm(fentry~X1+X2+X3+X4,data=ttpd)
### summary(out1)

# formula: fentry ~ X1 + X2 + X3 + X4
# data:    ttpd
# 
#  link  threshold nobs logLik   AIC     niter max.grad cond.H 
#  logit flexible  1000 -1676.46 3372.91 6(2)  1.17e-12 5.3e+02
# 
# Coefficients:
#    Estimate Std. Error z value Pr(>|z|)    
# X1  -0.9913     0.1171  -8.465  < 2e-16 ***
# X2  -0.6962     0.1156  -6.021 1.74e-09 ***
# X3  -0.3466     0.1150  -3.013  0.00259 ** 
# X4  -0.3223     0.1147  -2.810  0.00495 ** 
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Threshold coefficients:
#     Estimate Std. Error z value
# 0|1  -2.0064     0.1461 -13.733
# 1|2  -1.3940     0.1396  -9.984
# 2|3  -0.7324     0.1347  -5.435
# 3|4  -0.6266     0.1343  -4.667
# 4|5  -0.1814     0.1333  -1.361
# 5|6   0.2123     0.1342   1.582
}


## -----------------------------------------------------------------------------
sessionInfo()

##' Restricted IPCW Mean for Censored Survival Data
##'
##' Provides a fast implementation of Inverse Probability of Censoring Weighting (IPCW) 
##' regression for a single time point. It fits the model:
##' \deqn{ E( \min(T, t) | X ) = \exp( X^T \beta) }
##' or, in the case of competing risks data:
##' \deqn{ E( I(\epsilon=1) (t - \min(T, t)) | X ) = \exp( X^T \beta) }
##' which represents the "Years Lost Due to Cause" (RMTL).
##'
##' The method solves the binomial regression IPCW response estimating equation:
##' \deqn{ X \left( \frac{\Delta(\min(T,t)) Y}{G_c(\min(T,t))} - \exp( X^T \beta) \right) = 0 }
##' where \eqn{\Delta(\min(T,t)) = I(\min(T,t) \leq C)} is the indicator of being uncensored 
##' at the time of interest.
##'
##' When the status variable is binary, the outcome is assumed to be \eqn{Y = \min(T,t)} (RMST).
##' If the status has more than two levels (competing risks), the outcome is 
##' \eqn{Y = (t - \min(T,t)) I(\text{status}=\text{cause})} (RMTL for a specific cause).
##'
##' The function supports:
##' \itemize{
##'   \item \strong{IPCW Adjustment}: Weights by the inverse of the censoring survival probability.
##'   \item \strong{Augmentation}: Can include an augmentation term (type="II" or "III") to improve efficiency 
##'     and robustness (Double Robust estimation).
##'   \item \strong{Variance Estimation}: Based on the influence function, including adjustments for 
##'     the estimation of the censoring model.
##' }
##'
##' @param formula Formula with an \code{Event} outcome (e.g., \code{Event(time, cause)}).
##' @param data Data frame containing the variables.
##' @param outcome Outcome type: \code{"rmst"} (Restricted Mean Survival Time) or 
##'   \code{"rmtl"} (Restricted Mean Time Lost).
##' @param ... Additional arguments passed to \code{binreg}, such as \code{time}, \code{cause}, 
##'   \code{cens.model}, \code{model}, \code{type}, etc.
##' @return An object of class \code{"binreg"} containing:
##'   \item{coef}{Coefficient estimates.}
##'   \item{se}{Standard errors.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{iid}{Influence function decomposition.}
##'   \item{naive.var}{Variance under known censoring model (if applicable).}
##'   \item{time}{Time point used.}
##'   \item{outcome}{Type of outcome analyzed.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.
##' @seealso \code{\link{binreg}}, \code{\link{resmeanATE}}, \code{\link{rmstIPCW}}
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001
##' 
##' # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
##' out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'                 time=50, cens.model=~strata(platelet), model="exp")
##' summary(out)
##' 
##' \donttest{ ## Reduce Ex.Timings
##' ## Weighted GLM version (RMST)
##' out2 <- logitIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'             time=50, cens.model=~strata(platelet), model="exp", outcome="rmst")
##' summary(out2)
##' 
##' ### Time-lost (RMTL)
##' outtl <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'                 time=50, cens.model=~strata(platelet), model="exp", outcome="rmtl")
##' summary(outtl)
##' 
##' ### Same as Kaplan-Meier for full censoring model 
##' bmt$int <- with(bmt, strata(tcell, platelet))
##' out <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30,
##'                              cens.model=~strata(platelet, tcell), model="lin")
##' estimate(out)
##' out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet), data=bmt)
##' rm1 <- resmean_phreg(out1, times=30)
##' summary(rm1)
##' 
##' ### Years lost regression
##' outl <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30, outcome="years-lost",
##'                              cens.model=~strata(platelet, tcell), model="lin")
##' estimate(outl)
##' 
##' ## Competing risks years-lost for cause 1  
##' out <- resmeanIPCW(Event(time,cause)~-1+int, bmt, time=30, cause=1,
##'                             cens.model=~strata(platelet, tcell), model="lin")
##' estimate(out)
##' ## Same as integrated cumulative incidence 
##' rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet), data=bmt, times=30)
##' summary(rmc1)
##' }
##' @aliases rmstIPCW 
##' @export
resmeanIPCW  <- function(formula,data,outcome=c("rmst","rmtl"),...)
{# {{{
   out <- binreg(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

##' @export
rmstIPCW <- function(formula,data,outcome=c("rmst","rmtl"),...)
{# {{{
   out <- binreg(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

preprrm <- function(cs,ss,X,times,data,model="exp") 
{# {{{

ttimes <- ss$cumhaz[,1]
ttimes <- ttimes[(ttimes< times)]
ttimes <- c(ttimes,times)
## cens model 
pcs <- predict(cs,data,se=FALSE,times=ttimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
## survival model 
ps <- predict(ss,data,se=FALSE,times=ttimes)
###dLamt <- apply(cbind(0,ps$cumhaz),1,diff)
pst0 <- c(tail(t(ps$surv),n=1))
dtime <- diff(c(0,ttimes))
RRM <-  apply(dtime * t(ps$surv),2,cumsum)
RRMt0 <- c(tail(RRM,n=1))
dF <- -apply(cbind(1,ps$surv),1,diff)
I <- apply(ttimes^2/t(pcs$surv)*dF,2,sum)+times^2*pst0/pcst0 -RRMt0*apply(ttimes/t(pcs$surv)*dF,2,sum)-RRMt0*times*pst0/pcst0 
varY <- apply(ttimes^2*dF,2,sum)+times^2*pst0 - RRMt0^2

ctimes <- cs$cumhaz[,1]
ctimes <- ctimes[(ctimes< times)]
## cens model 
pcs <- predict(cs,data,se=FALSE,times=ctimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
dLamc <- apply(cbind(0,pcs$cumhaz),1,diff)
## survival model 
ps <- predict(ss,data,se=FALSE,times=ctimes)
pst0 <- c(tail(t(ps$surv),n=1))
Fsst0 <- 1-pst0
dtime <- diff(c(0,ctimes))
###dS <- -apply(cbind(1,ps$surv),1,diff)
RRM <-  apply(dtime * t(ps$surv),2,cumsum)
###
RRMt0 <- c(tail(RRM,n=1))
dF <- dRRM <- t(RRMt0-t(RRM)) + ctimes*t(ps$surv)
II <- apply((dF^2/t(ps$surv*pcs$surv))*dLamc,2,sum)-RRMt0*apply((dF/t(pcs$surv))*dLamc,2,sum) 
if (model=="exp") DbetaF <- RRMt0 else DbetaF <- 1
h <- DbetaF/(I-II) 
###varYII <- 2*apply(ctimes*dtime*ps$surv,1,sum)-RRMt0^2

cmtimes <- matrix(ctimes,nrow(dLamc),ncol(dLamc))
tttimes <- matrix(pmin(data[,"time"],times),nrow(dLamc),ncol(dLamc),byrow=TRUE)
Augmentf <- (dF/t(ps$surv*pcs$surv))
AugmentC <- apply(Augmentf*dLamc*(cmtimes<=tttimes),2,sum)
###
n <- nrow(data)
AdN <- rep(0,n)
jc <- (1:nrow(data))[cs$ord][cs$jumps]
jc <-jc[1:length(ctimes)] 
AdN[jc] <- mdi(Augmentf,1:length(ctimes),jc)
Mc <- AdN- AugmentC

ph <- 1
if (model=="exp") ph <- RRMt0
Xaugment <- apply(X*Mc*ph,2,sum)
augment <- apply(X*Mc*h,2,sum)
hh <- (DbetaF/varY)
Faugment <- apply(X*hh*Mc,2,sum)
return(list(Mc=Mc,Xaugment=Xaugment,Faugment=Faugment,hXaugment=augment,h=h,hh=hh,varY=varY,RRMt0=RRMt0))
}# }}}


##' Average Treatment Effect for Restricted Mean Time
##'
##' Estimates the Average Treatment Effect (ATE) for Restricted Mean Survival Time (RMST) 
##' or Restricted Mean Time Lost (RMTL) in censored competing risks data using IPCW.
##' 
##' Under standard causal assumptions (Consistency, Ignorability, Positivity), the ATE is 
##' estimated as \eqn{E(Y(1) - Y(0))}, where \eqn{Y(a)} is the potential outcome under treatment \eqn{a}.
##' The method uses double robust estimating equations that are IPCW-adjusted for censoring.
##'
##' The first covariate in the formula must be the treatment effect (a factor). If the factor 
##' has more than two levels, multinomial logistic regression (mlogit) is used for propensity 
##' score modeling.
##'
##' @param formula Formula with an \code{Event} outcome. The first covariate must be the treatment factor.
##' @param data Data frame.
##' @param model Link function: \code{"exp"} (exponential) or \code{"lin"} (identity).
##' @param outcome Outcome type: \code{"rmst"} or \code{"rmtl"}.
##' @param ... Additional arguments passed to \code{binregATE}, such as \code{time}, \code{treat.model}, 
##'   \code{augmentR0}, \code{augmentC}, etc.
##' @return An object of class \code{"binregATE"} containing:
##'   \item{riskG}{Simple IPCW estimator results.}
##'   \item{riskDR}{Double Robust estimator results.}
##'   \item{riskG.iid, riskDR.iid}{Influence functions.}
##'   \item{coef}{Treatment effect estimates.}
##'   \item{se}{Standard errors.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.
##' 
##' Scheike, T. and Tanaka, S. (2025). Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause. WIP.
##' @seealso \code{\link{binregATE}}, \code{\link{ratioATE}}
##' @examples
##' data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
##' 
##' out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, 
##'                   treat.model=tcell~platelet, outcome="rmtl")
##' summary(out)
##' 
##' out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40,
##'                    treat.model=tcell~platelet, outcome="rmtl")
##' summary(out1)
##' 
##' @aliases rmstATE 
##' @export
resmeanATE <- function(formula,data,model="exp",outcome=c("rmst","rmtl"),...)
{# {{{
out <- 	binregATE(formula,data,outcome=outcome[1],model=model,...) 
return(out)
}# }}}

##' @export
rmstATE <- function(formula,data,model="exp",outcome=c("rmst","rmtl"),...)
{# {{{
out <- 	binregATE(formula,data,outcome=outcome[1],model=model,...) 
return(out)
}# }}}

##' Ratio of Average Treatment Effects
##'
##' Computes the ratio of two Average Treatment Effects (ATEs), typically comparing the 
##' ATE for a specific cause (e.g., RMTL due to cause 1) to the ATE for the total RMTL.
##' 
##' The function transforms the estimates (e.g., using log) to compute the ratio and 
##' its standard error using the delta method and the joint influence functions.
##'
##' @param rmtl Object containing the ATE for the total RMTL (from \code{resmeanATE}).
##' @param rmtl1 Object containing the ATE for the specific cause RMTL (from \code{resmeanATE}).
##' @param h Transformation function (e.g., \code{log}) applied to the ratio. Default is identity.
##' @param null Value under the null hypothesis for the ratio (default 1).
##' @return A list containing:
##'   \item{ratioG}{Ratio based on the simple IPCW estimator.}
##'   \item{ratioDR}{Ratio based on the double robust estimator.}
##' @author Thomas Scheike
##' @seealso \code{\link{resmeanATE}}
##' @examples
##' data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
##' out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, outcome="rmtl")
##' out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40, outcome="rmtl")
##' ratioATE(out, out1, h=log)
##' @export
ratioATE <- function(rmtl, rmtl1, h=NULL, null=1)
##' @export
ratioATE <- function(rmtl,rmtl1,h=NULL,null=1) { ## {{{

if (is.null(h)) h <- function(x) x

coefG <- c(rmtl$riskG,rmtl1$riskG)
iidG <- cbind( rmtl$riskG.iid, rmtl1$riskG.iid)
covG <- crossprod(iidG)
###
coefDR <- c(rmtl$riskDR,rmtl1$riskDR)
iidDR <- cbind(rmtl$riskDR.iid, rmtl1$riskDR.iid)
covDR <- crossprod(iidDR)

ratioG <- summary(estimate(coef=coefG,vcov=covG,f=function(p) h(c(p[3:4]/p[1:2],p[1]*p[4]/(p[2]*p[3]),p[2]*p[3]/(p[1]*p[4])))),null=null)
ratioDR <- summary(estimate(coef=coefDR,vcov=covDR,f=function(p) h(c(p[3:4]/p[1:2],p[1]*p[4]/(p[2]*p[3]),p[2]*p[3]/(p[1]*p[4])))),null=null)
out <- list(ratioG=ratioG,ratioDR=ratioDR)
return(out)

} ## }}}

context("RMST binreg versus resmean.phreg, cif.yearslost")

test_that("resmeanIPCW", {
     set.seed(101)
     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean.phreg(out1,times=30)
     expect_true( (sum(abs(rm1$intkmtimes[,3] - coef(out)))<0.0001) & 
		 (sum(abs(rm1$intkmtimes[,4] - out$se.coef))<0.05) )
})

     
test_that("cif.yearslost", {
     set.seed(101)
     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     ### same as cif integral for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))

     ## competing risks years-lost for cause 1  
     outc <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     ## same as integrated cumulative incidence 
     rmc1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     expect_true( (sum(abs(rmc1$intkmtimes[,3] - coef(outc)))<0.0001) & 
  		 (sum(abs(rmc1$intkmtimes[,5] - outc$se.coef))<0.05) )

})

## RMST binreg versus resmean.phreg, cif.yearslost

test_resmeanIPCW <- function() {
  set.seed(101)
  data(bmt)
  bmt$time <- bmt$time+runif(nrow(bmt))*0.001

### same as Kaplan-Meier for full censoring model
  bmt$int <- with(bmt,strata(tcell,platelet))
  out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                     cens.model=~strata(platelet,tcell),model="lin")
  estimate(out)
  out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
  rm1 <- resmean.phreg(out1,times=30)
  expect_true( (sum(abs(rm1$intkmtimes[,3] - coef(out)))<0.0001) &
               (sum(abs(rm1$intkmtimes[,4] - out$se.coef))<0.05) )
}
test_resmeanIPCW()

test_cif_yearslost <- function() {
  set.seed(101)
  data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

### same as cif integral for full censoring model
  bmt$int <- with(bmt,strata(tcell,platelet))

  ## competing risks years-lost for cause 1
  outc <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                      cens.model=~strata(platelet,tcell),model="lin")
  ## same as integrated cumulative incidence
  rmc1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
  expect_true( (sum(abs(rmc1$intkmtimes[,3] - coef(outc)))<0.0001) &
               (sum(abs(rmc1$intkmtimes[,5] - outc$se.coef))<0.05) )
}
test_cif_yearslost()

##' @export
or2prob <- function(OR, marg) {
    p1 <- marg[1]
    p2 <- marg[2]
    if (length(marg)==1) p2 <- p1
    if (OR==1) {
        PP <- outer(c(p1, 1-p1), c(p2, 1-p2))
    } else {
        ## OR (p11*p00)/(p10*p01) = p11*(1+p11-p1-p2)/(p1-p11)*(p2-p11)
        ## (OR-1)p11^2-(OR(p1+p2)+(1-p1-p2))*p11+OR*p1*p2 = 0
        ## (1-OR)p11^2+(OR(p1+p2)+(1-p1-p2))*p11-OR*p1*p2 = 0
        a <- 1  -OR
        b <- 1 + (p1+p2) * (OR-1)
        ac <- -OR * p1 * p2 * a
        d <- sqrt(b^2 - 4*ac)
        ## Only one solution
        p11 <- (-b+d) / (2*a)
        PP <- c(p11, p1-p11, p2-p11)
        PP <- c(PP, 1-sum(PP))
    }
    structure(matrix(PP, 2), marg=c(p1, p2))
}

##' Estimate parameters from odds-ratio
##'
##' Calculate tetrachoric correlation of probabilities from odds-ratio
##' @param P Joint probabilities or marginals (if OR is given)
##' @param OR Odds-ratio
##' @param approx If TRUE an approximation of the tetrachoric correlation is
##'   used
##' @param ... Additional arguments
##' @export
##' @aliases or2prob tetrachoric
##' @examples
##' tetrachoric(0.3,1.25) # Marginal p1=p2=0.3, OR=2
##' P <- matrix(c(0.1,0.2,0.2,0.5),2)
##' prod(diag(P))/prod(lava::revdiag(P))
##' ##mets:::assoc(P)
##' tetrachoric(P)
##' or2prob(2,0.1)
##' or2prob(2,c(0.1,0.2))
tetrachoric <- function(P, OR, approx = 0, ...) {
    if (!missing(OR)) {
        ## Assuming P[1],P[2] is the marginals
        P <- or2prob(OR, P)
        p1 <- attributes(P)$marg[1]
        p2 <- attributes(P)$marg[2]
    } else {
        ## Assuming P contains the joint probabilities
        if (is.vector(P)) {
            if (length(P)==3) P <- c(P, 1-sum(P))
            P <- matrix(P, 2)
        }
        if (!all.equal(sum(P), 1)) stop("Not a probability matrix")
        p1 <- colSums(P)[1]
        p2 <- rowSums(P)[1]
    }
    if (approx>0) { ## Bonnet & Price 2005
        k <- (1-abs(p1-p2) / 5 - (.5-min(p1, p2))^2)/2
        if (missing(OR)) OR <- prod(diag(P)) / prod(revdiag(P))
        return(cos(pi / (1+OR^k)))
    }
    q1 <- qnorm(p1)
    q2 <- qnorm(p2)
    lo <- rbind(c(0, 0), c(0, -Inf), c(-Inf, 0), c(-Inf, -Inf))
    hi <- rbind(c(Inf, Inf), c(Inf, 0), c(0, Inf), c(0, 0))
    mu <- cbind(q1, q2) %x% cbind(rep(1, 4))
    obj <- function(r) {
        Pr <- pmvn(lower = lo, upper = hi, mu = mu, sigma = r, cor = TRUE)
        return(mean(abs(P - Pr)^2))
    }
    optimize(obj, interval=c(-1, 1))$minimum
}

assoc <- function(x, id, ...) {
    N <- sum(x)
    P <- x
    if (N != 1) P <- P/N
    OR <- prod(diag(P))/prod(revdiag(P))
    rho <- tetrachoric(P)
    list(P=P, OR=OR, rho=rho)
}

##' @export
biprobit.time <- function(formula,data,id,...,
                          breaks=NULL,n.times=20,
                          pairs.only=TRUE,fix.cens.weights=TRUE,
                          cens.formula,cens.model="aalen",
                          weights.fun=function(x) { u <- min(x); ifelse(u==0, 0, 1/min(u)) },
                          weights="w",messages=FALSE,
                          return.data=FALSE,theta.formula=~1,
                          trunc.weights="w2",
                          estimator="biprobit", summary.function) {
## {{{ 
    m <- match.call(expand.dots = FALSE)
    m <- m[match(c("","data"),names(m),nomatch = 0)]
    Terms <- terms(cens.formula,data=data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    M <- eval(m,envir=parent.frame())

    censtime <- model.extract(M, "response")
    if (pairs.only) {
        ii <- sort(as.matrix(na.omit(fast.reshape(seq(nrow(data)),id=data[,id]))))
        data <- data[ii,,drop=FALSE]
        censtime <- censtime[ii]
    }

    ltimes <- 0
    if (ncol(censtime)==3) {## {{{ ## Calculate probability of not being truncated via Clayton-Oakes (ad hoc combining causes)
        status <- censtime[,3]
        noncens <- !status
        time <- censtime[,2]
        ltimes <- censtime[,1]        
        data$truncsurv <- Surv(ltimes,time,noncens)
        trunc.formula <- update(formula,truncsurv~.)       
        ud.trunc <- timereg::aalen(trunc.formula,data=data,robust=0,n.sim=0,residuals=0,silent=1,max.clust=NULL,
                          clusters=data[,id], ...)
        X <- model.matrix(trunc.formula,data)
        ##dependX0 <- model.matrix(theta.formula,data)
        dependX0 <- X       
        twostage.fit <- two.stage(ud.trunc,
                                 data=data,robust=0,detail=0,
                                 theta.des=dependX0)#,Nit=20,step=1.0,notaylor=1)        
        Xnam <- colnames(X)
        ww <- fast.reshape(cbind(X,".num"=seq(nrow(X)),".lefttime"=ltimes),varying=c(".num",".lefttime"),id=data[,id])
        dependX <- as.matrix(ww[,Xnam,drop=FALSE])
        nottruncpair.prob <- predict.two.stage(twostage.fit,X=ww[,Xnam],
                                               times=ww[,".lefttime1"],times2=ww[,".lefttime2"],
                                               theta.des=dependX)$St1t2
        data[,trunc.weights] <- 0
        data[ww[,".num1"],trunc.weights] <- nottruncpair.prob
        data[ww[,".num2"],trunc.weights] <- nottruncpair.prob
        
    } else {  
        status <- censtime[,2]
        time <- censtime[,1]
    } ## }}} 

    outcome <- as.character(terms(formula)[[2]])
    jj <- jumptimes(time,data[,outcome],data[,id],sample=n.times)
    lastjump <- tail(jj,1)
    if (is.null(breaks)) {
        breaks <- jj
    }
    if (any(breaks>lastjump)) {
        breaks <- unique(pmin(breaks,tail(jj,1)))        
        if (messages) message("Looking at event before time ",max(breaks))
    }    

    outcome0 <- paste(outcome,"_dummy")
    res <- list(); k <- 0
    breaks <- rev(breaks)
    ids <- c()
    for (tau in breaks) {
        if (length(breaks)>1 && messages) message(tau)
        ## construct min(T_i,tau) or T_i and related censoring variable, 
        ## thus G_c(min(T_i,tau)) or G_c(T_i) as weights
        if ((fix.cens.weights==1 & k==0) | (fix.cens.weights==0)) {
            data0 <- data
            time0 <- time
            status0 <- status
        }
        cond0 <- time0>tau
        if (!fix.cens.weights) {
            status0[cond0 & status==1] <- 3 ## Not-censored if T>tau
        }
        data0[,outcome] <- data[outcome]
        data0[cond0,outcome] <- FALSE ## Non-case if T>tau
        if (!fix.cens.weights) time0[cond0] <- tau
        if ((fix.cens.weights & k==0) | (!fix.cens.weights)) {
            
            if (ncol(censtime)==3) { ## truncation...
                suppressWarnings(data0$S <- Surv(ltimes,time0,status0==1))
            } else {
                data0$S <- Surv(time0,status0==1)
            }
            ## data0$status0 <- status0
            ## data0$time0 <- time0
            ## data0$y0 <- data0[,outcome]
          dataw <- ipw(update(cens.formula,S~.),
                       data=subset(data0,ltimes<time0), cens.model=cens.model,
                       weight.name=weights, obs.only=TRUE, cluster=id)
        }
        if (fix.cens.weights) {
            timevar <- dataw$S[, 1]
            dataw[,outcome] <- (dataw[,outcome])*(timevar<tau)
        }
        k <- k+1
        if (ncol(censtime)==3) { ## truncation...
            
        }
        if (return.data) return(dataw)
        if (ncol(censtime)==3) { ## truncation...
            dataw[,weights] <- dataw[,weights]*dataw[,trunc.weights]
        }
        args <- c(list(x=formula,data=dataw,id=id,
                       weights=weights, weights.fun=weights.fun,
                       pairs.only=pairs.only), list(...))
        ids <- c(ids, list(rownames(dataw)))
        suppressWarnings(b <- do.call(estimator, args))
        ## suppressWarnings(b <- biprobit(formula, data=dataw, id=id, weights=weights, pairs.only=pairs.only,...))
        if (length(breaks)>1) res <- c(res, list(summary(b,...)))
    }
    if (length(breaks)==1) {
        return(structure(b,time=breaks))
    }
    if (missing(summary.function)) {
        summary.function <- function(x,...) x$all
    } 
    mycoef <- lapply(rev(res),function(x) summary.function(x))
        res <- list(varname="Time",
                    var=rev(breaks),
                    coef=mycoef,
                    summary=rev(res),
                    call=m, type="time", id=ids)
    class(res) <- "timemets"
    return(res)
} ## }}}

biprobit.time2 <- function(formula,data,id,...,
                          breaks=Inf,pairs.only=TRUE,
                          cens.formula,cens.model="aalen",weights="w") {
## {{{ 

    m <- match.call(expand.dots = FALSE)
    m <- m[match(c("","data"),names(m),nomatch = 0)]
    Terms <- terms(cens.formula,data=data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    M <- eval(m,envir=parent.frame())

    censtime <- model.extract(M, "response")
    status <- censtime[,2]
    time <- censtime[,1]
    outcome <- as.character(terms(formula)[[2]])    
    if (is.null(breaks)) breaks <-  quantile(time,c(0.25,0.5,0.75,1))

    outcome0 <- paste(outcome,"_dummy")
    res <- list()
    for (tau in breaks) {
        if (length(breaks)>1) message(tau)
        data0 <- data
        time0 <- time
        cond0 <- time0>tau
        status0 <- status
        status0[cond0 & status==1] <- 3 ## Censored
        data0[cond0,outcome] <- FALSE
        time0[cond0] <- tau
        data0$S <- survival::Surv(time0,status0==1)        
        dataw <- ipw(update(cens.formula,S~.), data=data0, cens.model=cens.model,
                     cluster=id,weight.name=weights,obs.only=TRUE)
        message("control")
        suppressWarnings(b <- biprobit(formula, data=dataw, id=id, weights=weights, pairs.only=pairs.only,...))
        res <- c(res,list(summary(b)))
    }
    if (length(breaks)==1) return(b)
    res <- list(varname="Time",var=breaks,coef=lapply(res,function(x) x$all),summary=res,call=m,type="time")
    class(res) <- "timemets"
    return(res)    
} ## }}} 

biprobit.time.trunc.test <- function(formula,data,id,...,
                          breaks=NULL,n.times=20,pairs.only=TRUE,fix.cens.weights=FALSE,
                          cens.formula,cens.model="aalen",weights="w",messages=FALSE,
                          return.data=FALSE,theta.formula=~1,trunc.weights="w2",
                          estimator="biprobit", summary.function) {
## {{{ 
    
    m <- match.call(expand.dots = FALSE)
    m <- m[match(c("","data"),names(m),nomatch = 0)]
    Terms <- terms(cens.formula,data=data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    M <- eval(m,envir=parent.frame())

    censtime <- model.extract(M, "response")
    if (pairs.only) {
        ii <- sort(as.matrix(na.omit(fast.reshape(seq(nrow(data)),id=data[,id]))))
        data <- data[ii,,drop=FALSE]
        censtime <- censtime[ii]
    }

    ltimes <- 0
    if (ncol(censtime)==3) {  ## Calculate probability of not being truncated via Clayton-Oakes (ad hoc combining causes)
        status <- censtime[,3]
        noncens <- !status
        time <- censtime[,2]
        ltimes <- censtime[,1]        
        data$truncsurv <- Surv(ltimes,time,noncens)
        trunc.formula <- update(formula,truncsurv~.)       
        ud.trunc <- timereg::aalen(trunc.formula,data=data,robust=0,n.sim=0,residuals=0,silent=1,max.clust=NULL,
                          clusters=data[,id], ...)
        X <- model.matrix(trunc.formula,data)
        ##dependX0 <- model.matrix(theta.formula,data)
        dependX0 <- X       
        twostage.fit <- two.stage(ud.trunc,
                                 data=data,robust=0,detail=0,
                                 theta.des=dependX0)#,Nit=20,step=1.0,notaylor=1)        
        Xnam <- colnames(X)
        ww <- fast.reshape(cbind(X,".num"=seq(nrow(X)),".lefttime"=ltimes),varying=c(".num",".lefttime"),id=data[,id])
        dependX <- as.matrix(ww[,Xnam,drop=FALSE])
        nottruncpair.prob <- predict.two.stage(twostage.fit,X=ww[,Xnam],
                                               times=ww[,".lefttime1"],times2=ww[,".lefttime2"],
                                               theta.des=dependX)$St1t2
        data[,trunc.weights] <- 0
        data[ww[,".num1"],trunc.weights] <- nottruncpair.prob
        data[ww[,".num2"],trunc.weights] <- nottruncpair.prob
        
    } else {
        status <- censtime[,2]
        time <- censtime[,1]
    }

    outcome <- as.character(terms(formula)[[2]])
    jj <- jumptimes(time,data[,outcome],data[,id],sample=n.times)
    lastjump <- tail(jj,1)
    if (is.null(breaks)) {
        breaks <- jj
    }
    if (any(breaks>lastjump)) {
        breaks <- unique(pmin(breaks,tail(jj,1)))        
        if (messages) message("Looking at event before time ",max(breaks))
    }    

    outcome0 <- paste(outcome,"_dummy")
    res <- list(); k <- 0
    breaks <- rev(breaks)
    for (tau in breaks) {
        if (length(breaks)>1 && messages) message(tau)
        ## construct min(T_i,tau) or T_i and related censoring variable, 
        ## thus G_c(min(T_i,tau)) or G_c(T_i) as weights
        if ((fix.cens.weights==1 & k==0) | (fix.cens.weights==0)) {
            data0 <- data
            time0 <- time
            status0 <- status
        }
        cond0 <- time0>tau
        if (!fix.cens.weights) {
            status0[cond0 & status==1] <- 3 ## Not-censored if T>tau
        }
        data0[,outcome] <- data[outcome]
        data0[cond0,outcome] <- FALSE ## Non-case if T>tau
        if (!fix.cens.weights) time0[cond0] <- tau
        if ((fix.cens.weights & k==0) | (!fix.cens.weights)) {
            
            if (ncol(censtime)==3) { ## truncation...
                suppressWarnings(data0$S <- Surv(ltimes,time0,status0==1))
            } else {
                data0$S <- Surv(time0,status0==1)
            }
            ## data0$status0 <- status0
            ## data0$time0 <- time0
            ## data0$y0 <- data0[,outcome]
            dataw <- ipw(update(cens.formula,S~.), data=subset(data0,ltimes<time0), cens.model=cens.model,
                         weight.name=weights,obs.only=TRUE,cluster=id)            
        }
        if (fix.cens.weights) {
            timevar <- dataw$S[,1]
            dataw[,outcome] <- (dataw[,outcome])*(timevar<tau)
        }
        k <- k+1
        if (ncol(censtime)==3) { ## truncation...
            
        }
        if (return.data) return(dataw)
        if (ncol(censtime)==3) { ## truncation...
            dataw[,weights] <- dataw[,weights]*dataw[,trunc.weights]
        }        
        args <- c(list(x=formula,data=dataw,id=id,weights=weights, pairs.only=pairs.only), list(...))
        suppressWarnings(b <- do.call(estimator, args))
        ## suppressWarnings(b <- biprobit(formula, data=dataw, id=id, weights=weights, pairs.only=pairs.only,...))
        if (length(breaks)>1) res <- c(res,list(summary(b,...)))
    }
    if (length(breaks)==1) {
        return(structure(b,time=breaks))
    }
    if (missing(summary.function)) {
        summary.function <- function(x,...) x$all
    } 
    mycoef <- lapply(rev(res),function(x) summary.function(x))
    res <- list(varname="Time",var=rev(breaks),coef=mycoef,summary=rev(res),call=m,type="time")
    class(res) <- "timemets"
    return(res)    
} ## }}} 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)
###
n <- 400
kumar <- kumarsim(n,depcens=1)
kumar$cause <- kumar$status
kumar$ttt24 <- kumar[,6]
dtable(kumar,~cause)
dfactor(kumar) <- gp.f~gp
kumar$id <- 1:n
kumar$idc <- sample(100,n,TRUE)
kumar$ids <- sample(n,n)
kumar$id2 <- sample(n,n)
kumar2 <- kumar[order(kumar$id2),]
kumar$int <- interaction(kumar$gp,kumar$dnr)
kumar2$int <- interaction(kumar2$gp,kumar2$dnr)
clust <- 0

b2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(b2)

b5 <- binregATE(Event(time,cause)~int+preauto+ttt24,kumar,cause=2,
		treat.model=int~preauto+ttt24,cens.code=0,time=60)
summary(b5)

## -----------------------------------------------------------------------------
ib2 <- logitIPCWATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(ib2)

ib5 <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,
	 time=60,cens.model=~strata(gp,dnr))
summary(ib5)

ibs <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,time=60)
summary(ibs)

check <- 0
if (check==1) {
require(riskRegression)
e.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, formula.censor = Surv(time,cause==0)~+1, times = 60, data = kumar, product.limit=TRUE,cause=2)
summary(e.wglm)$coef
estimate(ibs)

es.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, 
formula.censor = Surv(time,cause==0)~strata(gp,dnr), times = 60, 
data = kumar, product.limit=TRUE,cause=2)
summary(es.wglm)$coef
estimate(ib5)
}

## -----------------------------------------------------------------------------
kumar$cause2 <- 1*(kumar$cause==2)

b3 <- logitATE(cause2~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)

###library(targeted)
###b3a <- ate(cause2~gp.f|dnr+preauto+ttt24| dnr+preauto+ttt24,kumar,family=binomial)
###summary(b3a)

## calculate also relative risk
estimate(coef=b3$riskDR,vcov=b3$var.riskDR,f=function(p) p[1]/p[2])

## -----------------------------------------------------------------------------
b3 <- normalATE(time~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)

## -----------------------------------------------------------------------------
sessionInfo()

##' While-Alive Estimands for Recurrent Events
##'
##' Computes the "While-Alive" estimands for recurrent events in the presence of a 
##' terminal event (death). These estimands address the challenge of defining meaningful 
##' treatment effects when death prevents further observation of recurrent events.
##' 
##' The function estimates two primary quantities:
##' \enumerate{
##'   \item \strong{Ratio of Means}: \deqn{E(N(\min(D,t))) / E(\min(D,t))}
##'     The expected number of events up to time \eqn{t} (censored by death \eqn{D}) 
##'     divided by the expected time alive up to \eqn{t}.
##'   \item \strong{Mean of Events per Time Unit}: \deqn{E(N(\min(D,t)) / \min(D,t))}
##'     The expected rate of events per unit of time alive.
##' }
##' 
##' Estimation is based on Inverse Probability of Censoring Weighting (IPCW) to handle 
##' administrative censoring and death. The method can be augmented with covariates 
##' (double robust estimation) to improve efficiency and robustness.
##'
##' @param formula Formula with an \code{Event} object. The first covariate on the RHS 
##'   must be a factor representing the treatment group. Can include \code{cluster(id)}.
##' @param data Data frame containing all variables referenced in the formula.
##' @param time Time point \eqn{t} for estimation. If NULL, defaults to the maximum event time.
##' @param cens.code Numeric code for censoring (default 0).
##' @param cause Numeric code for the recurrent event of interest (default 1).
##' @param death.code Numeric code for the terminal event/death (default 2).
##' @param trans Power transformation for the mean of events per time-unit (default NULL, i.e., linear).
##' @param cens.formula Formula for the censoring model. Default is \code{~strata(treatment)}.
##' @param augmentR Formula for covariate augmentation in the randomization model 
##'   (e.g., \code{~age+sex}). Improves efficiency.
##' @param augmentC Formula for covariate augmentation in the censoring model. 
##'   Enables double robustness.
##' @param type Type of augmentation for the binomial regression call. Default is "I" 
##'   if \code{augmentC} is given, otherwise "II".
##' @param marks Optional marks for composite outcome situations (e.g., distinguishing 
##'   event types in a composite endpoint).
##' @param ... Additional arguments passed to \code{binregATE}.
##' @return An object of class \code{"WA"} containing:
##'   \item{RAW}{List of raw estimates: RMST, mean number of events, ratio of means, 
##'     and their log-transformed versions with standard errors.}
##'   \item{ET}{List of estimated treatment effects: risk difference for the mean rate 
##'     (\code{riskDR}) and optionally the augmented version (\code{riskDRC}).}
##'   \item{time}{The time point used for estimation.}
##'   \item{cause, death.code, cens.code}{Codes used.}
##'   \item{augmentR, augmentC}{Formulas used for augmentation.}
##'   
##'   The object includes influence functions (IID) for all estimators, allowing for 
##'   further variance calculations or combination with other estimators.
##' @author Thomas Scheike
##' @references 
##' Ragni, A., Martinussen, T., & Scheike, T. H. (2023). Nonparametric estimation of the Patient Weighted While-Alive Estimand. arXiv preprint.
##' 
##' Mao, L. (2023). Nonparametric inference of general while-alive estimands for recurrent events. Biometrics, 79(3), 1749–1760.
##' 
##' Schmidli, H., Roger, J. H., & Akacha, M. (2023). Estimands for recurrent event endpoints in the presence of a terminal event. Statistics in Biopharmaceutical Research, 15(2), 238–248.
##' @examples
##' data(hfactioncpx12)
##' dtable(hfactioncpx12,~status)
##' dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),data=hfactioncpx12,
##'                    time=2,death.code=2)
##' summary(dd)
##' @export
WA_recurrent <- function(formula,data,time=NULL,cens.code=0,cause=1,death.code=2,
	 trans=NULL,cens.formula=NULL,augmentR=NULL,augmentC=NULL,type=NULL,marks=NULL,...)
{ ## {{{
  cl <- match.call() ## {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","marks"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

   call.id <- id
   conid <- construct_id(id, nrow(X), namesX = rownames(X))
   name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
   orig.id <- id
   data$id__ <- id 
    if (is.null(des.offset)) {
        if (is.null(offset))
            offset <- rep(0, length(exit))
    } else offset <- des.offset
    if (is.null(des.weights)) {
        if (is.null(weights))
            weights <- rep(1, length(exit))
    } else weights <- des.weights
    if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  ## }}}

## use sorted id for all things  and here indentify last record of each subject
cid <- countID(data,"id__",sorted=TRUE)
rrR <- subset(data,cid$reverseCountid==1)

## first var on rhs of formula
vars <- all.vars(formula)
treat.name <- vars[4]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
ntreatvar <- as.numeric(treatvar) - 1
statusD <- rrR[, vars[3]] %in% death.code
rrR[["statusD__"]] <- statusD
treat.formula <- treat.model <- as.formula(paste(treat.name,"~+1",sep=""))
if (is.null(cens.formula)) cens.formula <- as.formula( paste("~strata(",treat.name,")",collapse=""))
formC <- as.formula( paste("Event(",vars[1],",",vars[2],",",vars[3],"%in%", cens.code,")~+cluster(id__)",collapse=""))
formD <- as.formula( paste("Event(",vars[2], ",statusD__)~-1+",treat.name,"+cluster(id__)",collapse=""))
form1 <- as.formula( paste("Event(",vars[2],",",vars[3],")~-1+",treat.name,"+cluster(id__)",collapse=""))

  ## drop id as cluster variable and use id__
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))
## take out intercept, to get mean in treated/non-treated
formrec <- update(formula, reformulate(c(".", "-1")))

if (!is.null(augmentR)) {
   varsR <- c(attr(terms(augmentR), "term.labels"))
   form1X <- update(form1, reformulate(c(".", varsR)))
} else form1X <- form1

## ratio of means ## {{{
dd <- rmstIPCW(formD,data=rrR,cause=1,cens.code=0,cens.model=cens.formula,time=time, model="lin")
ddN <- recregIPCW(formrec,data=data,cause=cause,death.code=death.code,cens.code=cens.code, cens.model=cens.formula,times=time,model="lin",marks=marks)

treatdata <- list()
treatdata[[treat.name]] <- nlevs
treatdata <- as.data.frame(treatdata)

f <- function(p,logg=0) {
ddN$coef <- p[1:2]
dd$coef <- p[3:4]
prmst <- predict(dd,treatdata,se=0)
prec <- predict(ddN,treatdata,se=0)
out <- prec/prmst
if (logg==1) out <- log(out)
return(out)
}

cc <- c(ddN$coef,dd$coef)
cciid <- cbind(ddN$iid,dd$iid)
ratio.means  <- estimate(coef=cc,vcov=crossprod(cciid),f=f)
###			 f=function(p) (p[1:2]/p[3:4]))
ratio.means.log <- estimate(coef=cc,vcov=crossprod(cciid),f=function(p) log((p[1:2]/p[3:4])))
RAW <- list(iid=cciid,coef=cc,time=time,rmst=dd,meanN=ddN,ratio.means=ratio.means,ratio.means.log=ratio.means.log)
## }}}

data <- count_history(data,id="id__",lag=TRUE,types=cause,status=vars[3],marks=marks,multitype=TRUE)

nameCount <- paste("Count",cause[1],sep="")
formulaCount <- update.formula(formula,.~+1)
cform <- as.formula(paste("~",nameCount,"+cluster(id__)",sep=""))
formulaCount <- update.formula(formulaCount,cform)

## While-Alive mean of events per time-unit 
## with possible marks for death.codes 
if (any(cause %in% death.code)) {
	wd <- match(cause,death.code,nomatch=0)
	mark.codes <- death.code[wd]
} else mark.codes <- NULL
dataDmin <- evalTerminal(formulaCount,data=data,time=time,death.code=death.code, mark.codes=mark.codes,marks=marks)

### setting new response , Ratio of composite outcome
rrR[,"ratio__"] <- dataDmin[cid$reverseCountid==1,"ratio"]
if (!is.null(trans)) {
     rrR[,"ratio__"] <- rrR[,"ratio__"]^trans
}
Yr <- rrR[,"ratio__"]

if (is.null(type)) {
    if (is.null(augmentC)) type <- "II" else type <- "I"
}

outae <- binregATE(form1X,rrR,cause=death.code,time=time,treat.model=treat.formula,
   cens.code=cens.code,Ydirect=Yr,outcome="rmst",model="lin",cens.model=cens.formula,
   type=type[1],...) 
ET <- list(riskDR=outae)

##  Yipcw as it comes in data frame 
data[,"ratio__"] <- outae$Yipcw[cid$indexid+1]

if (!is.null(augmentC)) { ## {{{
dc0 <- dynCensAug(formC,subset(data,ntreatvar==0),augmentC=augmentC,response="ratio__",time=time)
dc1 <- dynCensAug(formC,subset(data,ntreatvar==1),augmentC=augmentC,response="ratio__",time=time)
###nn <- table(rr$treat)
nid <- nrow(rrR)

treats <- function(treatvar) { treatvar <- droplevels(treatvar)# {{{
        nlev <- nlevels(treatvar)
        nlevs <- levels(treatvar)
        ntreatvar <- as.numeric(treatvar)
        return(list(nlev = nlev, nlevs = nlevs, ntreatvar = ntreatvar))
}  

fittreat <- function(treat.model, data, id, ntreatvar, nlev) {
if (nlev == 2) {
    treat.model <- drop.specials(treat.model, "cluster")
    treat <- glm(treat.model, data, family = "binomial")
    iidalpha <- lava::iid(treat, id =id)
    lpa <- treat$linear.predictors
    pal <- expit(lpa)
    pal <- cbind(1 - pal, pal)
    ppp <- (pal/pal[, 1])
    spp <- 1/pal[, 1]
} else {
    treat.modelid <- update.formula(treat.model, . ~ . + cluster(id__))
    treat <- mlogit(treat.modelid, data)
    iidalpha <- lava::iid(treat)
    pal <- predict(treat, data, se = 0, response = FALSE)
    ppp <- (pal/pal[, 1])
    spp <- 1/pal[, 1]
}
Xtreat <- model.matrix(treat.model, data)
tvg2 <- 1 * (ntreatvar >= 2)
pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
pppy <- c(mdi(ppp, 1:length(ntreatvar), ntreatvar))
Dppy <- (spp * tvg2 - pppy)
Dp <- c()
for (i in seq(nlev - 1)) Dp <- cbind(Dp, Xtreat * ppp[,
    i + 1] * Dppy/spp^2)
DPai <- -1 * Dp/pA^2
out <- list(iidalpha = iidalpha, pA = pA, Dp = Dp, pal = pal,
    ppp = ppp, spp = spp, id = id, DPai = DPai)
return(out)
} # }}}

expit <- function(x) 1/(1 + exp(-x))
idW <- rrR[,"id__"]
treatsvar <- rrR[,treat.name]
treats <- treats(treatsvar)

fitt <- fittreat(treat.model, rrR, idW, treats$ntreatvar, treats$nlev)
iidalpha0 <- fitt$iidalpha
wPA <- c(fitt$pA)
## ordered after idW
wPA <- wPA[order(idW)]

riskDRC <- outae$riskDR+c(dc0$augment,dc1$augment)/nid
MGC0 <- sumstrata(dc0$MGCiid/c(wPA[dc0$id+1]),dc0$id,nid)*dc0$n
MGC1 <- sumstrata(dc1$MGCiid/c(wPA[dc1$id+1]),dc1$id,nid)*dc1$n
MGC <- cbind(MGC0,MGC1)
ccaugment <- apply(MGC,2,sum)

riskDRC <- outae$riskDR+ccaugment/nid
iidDRC <- outae$riskDR.iid+MGC/nid
varDRC <- crossprod(iidDRC)
se.riskDRC <- diag(varDRC)^.5

riskDRC <- list(riskDRC=riskDRC,iid=iidDRC,var=varDRC,coef=riskDRC,se=se.riskDRC)
ET <- list(riskDRC=riskDRC,riskDR=outae)
###	   MGC=MGC,Y=data[,"ratio__"],dc0=dc0,dc1=dc1,wPA=wPA)
} ## }}}

###### sort iid after name.id and put as rownames
if (!is.null(call.id)) {
	ET$riskDRC$iid <- nameme(ET$riskDRC$iid,name.id)
	ET$riskDR$riskG.iid <- nameme(outae$riskG.iid,name.id)
	ET$riskDR$riskDR.iid <- nameme(outae$riskG.iid,name.id)
	ET$riskDR$iid <- nameme(ET$riskDR$iid,name.id)
}

out <- list(time=time,id=id,call.id=call.id,name.id=name.id,nid=nid,
	    trans=trans,cause=cause,cens.code=cens.code,death.code,
	    RAW=RAW,ET=ET,augmentR=augmentR,augmentC=augmentC)
class(out) <- "WA"
return(out)
} ## }}}

##' While-Alive Regression for Recurrent Events
##'
##' Performs regression analysis for the "While-Alive" mean of events per time unit, 
##' defined as \eqn{Z(t) = N(\min(D,t)) / \min(D,t)}. This function models how covariates 
##' affect the rate of recurrent events per unit of time alive.
##' 
##' The estimation is based on IPCW (Inverse Probability of Censoring Weighting) and 
##' calls \code{binreg} after constructing the outcome variable. It supports double 
##' robust estimation if covariate augmentation is specified.
##'
##' @param formula Formula with regression design. The first covariate on the RHS must 
##'   be the treatment factor. Can include other covariates and \code{cluster(id)}.
##' @param data Data frame.
##' @param time Time point \eqn{t} for estimation.
##' @param cens.code Censoring code.
##' @param cause Event cause code.
##' @param death.code Death code.
##' @param marks Marks for composite outcomes.
##' @param trans Power transformation for the outcome (default 1).
##' @param ... Additional arguments passed to \code{binreg}.
##' @return An object of class \code{"binreg"} containing coefficient estimates, 
##'   standard errors, confidence intervals, and influence functions for the 
##'   regression of the event rate per time alive.
##' @author Thomas Scheike
##' @references 
##' Ragni, A., Martinussen, T., & Scheike, T. H. (2023). Nonparametric estimation of the Patient Weighted While-Alive Estimand. arXiv preprint.
##' @examples
##' data(hfactioncpx12)
##' hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id] 
##' dtable(hfactioncpx12,~status)
##' ## exp-link regression 
##' dd <- WA_reg(Event(entry,time,status)~treatment+age+cluster(id),data=hfactioncpx12,
##'                     time=2,death.code=2)
##' summary(dd)
##' @export
WA_reg <- function(formula,data,time=NULL,cens.code=0,cause=1,death.code=2,marks=NULL,...,trans=1)
{ ## {{{
  cl <- match.call() ## {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","marks"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

   call.id <- id
   conid <- construct_id(id, nrow(X), namesX = rownames(X))
   name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
   orig.id <- id
   data$id__ <- id 
    if (is.null(des.offset)) {
        if (is.null(offset))
            offset <- rep(0, length(exit))
    } else offset <- des.offset
    if (is.null(des.weights)) {
        if (is.null(weights))
            weights <- rep(1, length(exit))
    } else weights <- des.weights
    if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  ## }}}

    data$status__ <- status

## use sorted id for all things  and here indentify last record of each subject
cid <- countID(data,"id__",sorted=TRUE)
rrR <- subset(data,cid$reverseCountid==1)

data <- count_history(data,id="id__",lag=TRUE,types=cause,status="status__",marks=marks,multitype=TRUE)
nameCount <- paste("Count",cause[1],sep="")

formulaCount <- update.formula(formula,.~+1)
cform <- as.formula(paste("~",nameCount,"+cluster(id__)",sep=""))
formulaCount <- update.formula(formulaCount,cform)

## While-Alive mean of events per time-unit 
## with possible marks for death.codes 
if (any(cause %in% death.code)) {
	wd <- match(cause,death.code,nomatch=0)
	mark.codes <- death.code[wd]
} else mark.codes <- NULL

dataDmin <- evalTerminal(formulaCount,data=data,time=time,death.code=death.code, mark.codes=mark.codes,marks=marks)

### setting new response , Ratio of composite outcome
rrR[,"ratio__"] <- dataDmin[cid$reverseCountid==1,"ratio"]
if (!is.null(trans)) {
     rrR[,"ratio__"] <- rrR[,"ratio__"]^trans
}
Yr <- rrR[,"ratio__"]

outae <- binreg(formula,rrR,cause=death.code,time=time,cens.code=cens.code,Ydirect=Yr,outcome="rmst",...)

return(outae)
} ## }}}

##' @export
print.WA  <- function(x,type="log",...) {# {{{
  print(summary(x),type=type,...)
}# }}}

##' @export
summary.WA <- function(object,type="p",augtype=NULL,...) {# {{{

rmst <- estimate(object$RAW$rmst)
rmst.test <- estimate(rmst,f=rbind(c(1,-1)))
rmst.log <- estimate(rmst,function(p) log(p))
rmst.test.log <- estimate(rmst.log,f=rbind(c(1,-1)))

meanNtD <- estimate(object$RAW$meanN)
meanNtD.test <- estimate(meanNtD,f=rbind(c(1,-1)))
meanNtD.log <- estimate(meanNtD,function(p) log(p))
meanNtD.test.log <- estimate(meanNtD.log,f=rbind(c(1,-1)))

eer <- estimate(object$RAW$ratio.means)
eedr <- estimate(eer,f=rbind(c(1,-1)))

if (is.null(augtype)) {
   augtype <- "riskDR"
   if (!is.null(object$ET$riskDRC)) augtype <- "riskDRC" 
}
if (augtype=="riskDR")
ee <- estimate(coef=object$ET$riskDR$riskDR,vcov=object$ET$riskDR$var.riskDR)
if (augtype=="riskDRC")
ee <- estimate(coef=object$ET$riskDRC$coef,vcov=object$ET$riskDRC$var)
eed <- estimate(ee,f=rbind(c(1,-1)))
eer.log <- object$RAW$ratio.means.log
eedr.log <- estimate(eer.log,f=rbind(c(1,-1)))
eelog <-  estimate(ee,function(p) log(p))
eedlog <- estimate(eelog,f=rbind(c(1,-1)))

res <- list(rmst=rmst,rmst.test=rmst.test,meanNtD=meanNtD,meanNtD.test=meanNtD.test,
            ratio=eer,test.ratio=eedr,meanpt=ee,test.meanpt=eed)
reslog <- list(rmst=rmst.log,rmst.test=rmst.test.log,
               meanNtD=meanNtD.log,meanNtD.test=meanNtD.test.log,
    ratio=eer.log,test.ratio=eedr.log,meanpt=eelog,test.meanpt=eedlog)
class(res) <- "summary.WA"
class(reslog) <- "summary.WA"
attr(res,"log") <- (type!="p")
attr(reslog,"log") <- (type!="p")

if (type=="p") return(res) else return(reslog)
}# }}}

##' @export
print.summary.WA <- function(x,...)
{# {{{

if (attr(x,"log")) 
cat(paste("While-Alive summaries, log-scale:",x$time,"\n\n"))
else
cat(paste("While-Alive summaries:",x$time,"\n\n"))

cat("RMST,  E(min(D,t)) \n")
print(x$rmst)
cat(" \n")
print(x$rmst.test)

cat("mean events, E(N(min(D,t))): \n")
print(x$meanNtD)
cat(" \n")
print(x$meanNtD.test)
cat("_______________________________________________________ \n")

cat("Ratio of means E(N(min(D,t)))/E(min(D,t)) \n")
print(x$ratio)
cat(" \n")
print(x$test.ratio)
cat("_______________________________________________________ \n")

cat("Mean of Events per time-unit E(N(min(D,t))/min(D,t)) \n")
print(x$meanpt)
cat(" \n")
print(x$test.meanpt)
} # }}}

dynCensAug <- function(formC,data,augmentC=~+1,response="Yipcw",time=NULL,Z=NULL) { ## {{{

if (is.null(time)) stop("must give time of response \n")
   data$Y__ <- data[,response]
   varsC <- c("Y__",attr(terms(augmentC), "term.labels"))
   formCC <- update(formC, reformulate(c(".", varsC)))
   cr2 <- phreg(formCC, data = data, no.opt = TRUE, no.var = 1,Z=Z)
   xx <- cr2$cox.prep
   icoxS0 <- rep(0,length(cr2$S0))
   icoxS0[cr2$S0>0] <- 1/cr2$S0[cr2$S0>0]
   S0i <- rep(0, length(xx$strata))
   S0i[xx$jumps + 1] <- icoxS0
   km <- TRUE
   if (!km) {
        cumhazD <- c(cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum)
        St <- exp(-cumhazD)
   } else St <- c(exp(cumsumstratasum(log(1 - S0i), xx$strata, xx$nstrata)$lagsum))

   nterms <- cr2$p-1
   dhessian <- cr2$hessianttime
   dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
   ###  matrix(apply(dhessian,2,sum),3,3)
   timeb <- which(cr2$cumhaz[,1]<time)
   ### take relevant \sum H_i(s,t) (e_i - \bar e)
   covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
   ### construct relevant \sum (e_i - \bar e)^2
   Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
   ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
   gammatt <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
   S0 <- cr2$S0[timeb]
   gammatt[is.na(gammatt)] <- 0
   gammatt[gammatt==Inf] <- 0
   Gctb <- St[cr2$cox.prep$jumps+1][timeb]
   augmentt.times <- apply(gammatt*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum)
   augment.times <- sum(augmentt.times)
   if (!is.null(Z)) {
             Zj <- cr2$cox.prep$Z[cr2$cox.prep$jumps+1,][timeb]
             Xaugment.times <- apply( augmentt.times*Zj,2,sum)
   }

   p <- 1
   #### iid magic  for censoring augmentation martingale #
   ### int_0^infty gamma(t) (e_i - ebar(s)) 1/G_c(s) dM_i^c
   xx <- cr2$cox.prep
   nid <- max(xx$id)+1
   jumpsC <- xx$jumps+1
   rr0 <- xx$sign
   S0i <- rep(0,length(xx$strata))
   S0i[jumpsC] <- c(1/(icoxS0*St[jumpsC]))
   S0i[jumpsC] <- icoxS0
   xxtime <- 1*c(xx$time<time)

   pXXA <- ncol(cr2$E)-1
   EA <- cr2$E[timeb,-1,drop=FALSE]
   gammasEs <- .Call("CubeMattime",gammatt,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   gammasE <- matrix(0,length(xx$strata),1)
   gammattt  <-    matrix(0,length(xx$strata),pXXA*1)
   jumpsCt <- jumpsC[timeb]
   gammasE[jumpsCt,] <- gammasEs
   gammattt[jumpsCt,] <- gammatt
   gammaEsdLam0 <- apply(gammasE*S0i*xxtime,2,cumsumstrata,xx$strata,xx$nstrata)
   gammadLam0 <-   apply(gammattt*S0i*xxtime,2,cumsumstrata,xx$strata,xx$nstrata)
   XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X[,-1,drop=FALSE],pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   Ut <- Et <- matrix(0,length(xx$strata),1)
   Ut[jumpsCt,] <- augmentt.times
   MGCt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
   MGCiid <- apply(MGCt,2,sumstrata,xx$id,nid)
   MGCiid <- MGCiid/nid

   nid <- max(cr2$id)+1
   res <- list(MGCiid=MGCiid,gammat=gammatt,augment=augment.times,
	       id=cr2$name.id,n=nid)
} ## }}}

evalTerminal <- function(formula,data=data,death.code=2,time=NULL,marks=NULL,mark.codes=NULL)
{# {{{
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    id <- strata <- NULL
    if (!is.null(attributes(Terms)$specials$cluster)) {
        ts <- survival::untangle.specials(Terms, "cluster")
        pos.cluster <- ts$terms
        Terms  <- Terms[-ts$terms]
        id <- m[[ts$vars]]
    } else pos.cluster <- NULL
###    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
###        ts <- survival::untangle.specials(Terms, "strata")
###        pos.strata <- ts$terms
###        Terms  <- Terms[-ts$terms]
###        strata <- m[[ts$vars]]
###        strata.name <- ts$vars
###    }  else { strata.name <- NULL; pos.strata <- NULL}
###    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
###        ts <- survival::untangle.specials(Terms, "offset")
###        Terms  <- Terms[-ts$terms]
###        offset <- m[[ts$vars]]
###    }
    X <- model.matrix(Terms, m)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## }}}

   if (is.null(time)) time <- max(exit)+1

###   if (!is.null(id)) {
###	   call.id <- id
###        ids <- unique(id)
###        nid <- length(ids)
###        if (is.numeric(id))
###            id <-  fast.approx(ids,id)-1
###        else  {
###            id <- as.integer(factor(id,labels=seq(nid)))-1
###        }
###    } else { call.id <- id <- as.integer(seq_along(entry))-1;  nid <- nrow(X); }
###    ## orginal id coding into integers 1:...
###   id <- id+1

###   ## new id 1,2,.... and so on, referring to rows of data
   call.id <- id
   conid <- construct_id(id,length(exit))
   id <- conid$id+1; nid <- conid$nid; 

   call.marks <- marks
   if (is.null(marks)) marks <- rep(1,length(id))

###   dd <- data.frame(id=id)
###   dd <- countID(dd,sorted=TRUE)
###   ## new id 1,2,.... and so on, referring to rows of data
###   id <- dd$indexid+1

 ###
 indexD  <- which(exit <= time & (status %in% death.code))
 indexDM  <- which(exit <= time & (status %in% mark.codes))
 idD <- id[indexD]
 idDM <- id[indexDM]
 obsid <- MarkDid <- Dmintid <- rep(0,nid)
 Dmintid[idD] <- exit[indexD]
 MarkDid[idDM] <- marks[idDM]
 ### 
 indexA<- which(entry <= time & time <= exit)
 idA <- id[indexA]
 Dmintid[idA] <- time
 obsid[idA] <- 1
 Dmint <- Dmintid[id]
 obsid[idA] <- 1
 obsid[idD] <- 1
 ###
 XminDtid <- matrix(0,nid,ncol(X))
 colnames(XminDtid) <- colnames(X)
 XminDtid[idD,] <- X[indexD,]
 XminDtid[idA,]  <- X[indexA,]
 if (length(idDM)>0) XminDtid  <- XminDtid+MarkDid
 XminDt <- XminDtid[id,,drop=FALSE]
 ratio <- rep(0,length(exit))
 obs <- obsid[id]
 ratio[obs==1] <- XminDt[obs==1]/Dmint[obs==1]

 nX <- colnames(XminDt) <- paste(colnames(X),"minDt",sep="") 
 if (ncol(X)==1) nR <- "ratio" else nR <- paste(colnames(X),"ratio",sep="") 
 dd <- data.frame(cbind(XminDt,Dmint,id,call.id,obs,ratio))
 colnames(dd) <- c(nX,"minDt","nid","call.id","uncensored",nR)

 return(dd)
} # }}}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)

n <- 200
ddf <- mets:::gsim(n,covs=1,null=0,cens=1,ce=1,betac=c(0.3,1))
true <- apply(ddf$TTt<2,2,mean)
true
datat <- ddf$datat
## set-random response on data, only relevant after status==2 
response <- rbinom(n,1,0.5)
datat$response <- as.factor(response[datat$id]*datat$Count2)
datat$A000 <- as.factor(1)
datat$A111 <- as.factor(1)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f,
		augmentR1=~X11+X12+TR, augmentR0=~X01+X02,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f))
bb

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]/p[2],p[3]/p[4]))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]-p[2],p[3]-p[4]))


## -----------------------------------------------------------------------------

## 2 levels for each response , fixed weights 
datat$response.f <- as.factor(datat$response)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(0,0),pi0=0.5,pi1=0.5)
bb

## 2 levels for each response ,  estimated treat probabilities
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb


## 2 and 3 levels for each response , fixed weights 
datat$A1.23.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.23.f+response)
datat <- dtransform(datat,A1.23.f=2+rbinom(nrow(datat),1,0.5),
		    Count2==1 & A1.23.f==2 & response==0)
dtable(datat,~A1.23.f+response)
datat$A1.23.f <- as.factor(datat$A1.23.f)
dtable(datat,~A1.23.f+response|Count2==1)
###
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1,treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
        estpr=c(1,0),pi1=c(0.3,0.5))
bb


## 2 and 3 levels for each response , estimated 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb


## 2 and 1 level for each response 
datat$A1.21.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.21.f+response|Count2==1)
datat <- dtransform(datat,A1.21.f=1,Count2==1 & response==1)
dtable(datat,~A1.21.f+response|Count2==1)
datat$A1.21.f <- as.factor(datat$A1.21.f)
dtable(datat,~A1.21.f+response|Count2==1)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb

## known weights 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,
	cens.model=~strata(A0.f),estpr=c(1,0),pi1=c(0.5,1))
bb

## -----------------------------------------------------------------------------
data(calgb8923)
calgt <- calgb8923

tm=At.f~factor(Count2)+age+sex+wbc
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=12)
summary(pll0,times=12)

## -----------------------------------------------------------------------------
sessionInfo()

##' Marginal mean estimation for recurrent events with a terminal event
##'
##' Estimates the marginal mean number of recurrent events over time in the
##' presence of a competing terminal event (e.g. death), using the nonparametric
##' estimator of Ghosh and Lin (2000). Two proportional hazards models are fitted
##' internally—one for the recurrent event rate and one for the terminal
##' event—and combined to form the estimator
##' \deqn{\mu(t) = \int_0^t S(u-)\,dR(u),}
##' where \eqn{S(u)} is the marginal survival probability at the baseline covariate
##' level and \eqn{dR(u)} is the baseline recurrent event rate among survivors.
##' Robust (sandwich) standard errors are computed via the influence-function
##' approach of Ghosh and Lin (2000).
##'
##' Jump times must be unique within each stratum. If ties are present, use
##' \code{\link{tie_breaker}} to resolve them before calling this function.
##'
##' @param formula A formula with an \code{\link{Event}} response on the left-hand
##'   side, specifying entry time, exit time, and event status. The right-hand side
##'   may include \code{cluster()} to identify subjects and \code{strata()} for a
##'   stratified analysis. A \code{cluster()} term is required.
##' @param data A data frame containing all variables in \code{formula}.
##' @param cause Integer code(s) for the recurrent event of interest. Default is
##'   \code{1}.
##' @param death.code Integer code(s) for the terminal event. Default is \code{2}.
##' @param test Logical. If \code{TRUE}, a logrank-type test comparing strata is
##'   computed and stored as an attribute of the result. Default is \code{FALSE}.
##' @param ... Further arguments passed to \code{\link{phreg}}.
##'
##' @return An object of class \code{"recurrent"} with the following components:
##'   \item{mu}{Estimated marginal mean \eqn{\mu(t)} at each jump time.}
##'   \item{se.mu}{Robust standard error of \code{mu}.}
##'   \item{times}{Jump times at which estimates are computed.}
##'   \item{St}{Marginal survival estimate \eqn{S(t)} at each jump time.}
##'   \item{cumhaz}{Two-column matrix of \code{(time, mu)}, suitable for plotting.}
##'   \item{se.cumhaz}{Two-column matrix of \code{(time, se.mu)}.}
##'   The object carries three attributes: \code{"logrank"} (the test result when
##'   \code{test = TRUE}, otherwise \code{NULL}), \code{"cause"}, and
##'   \code{"death.code"}.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{test_logrankRecurrent}}, \code{\link{tie_breaker}},
##'   \code{\link{prob_exceed_recurrent}}
##'
##' @references
##' Cook, R. J. and Lawless, J. F. (1997). Marginal analysis of recurrent events
##' and a terminating event. \emph{Statistics in Medicine}, 16, 911--924.
##'
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
##' and death. \emph{Biometrics}, 56, 554--562.
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' hf$x <- as.numeric(hf$treatment)
##'
##' ## Fit nonparametric baseline models for recurrent events and death
##' xr <- phreg(Surv(entry, time, status == 1) ~ cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ cluster(id), data = hf)
##'
##' par(mfrow = c(1, 3))
##' plot(dr, se = TRUE); title(main = "Death")
##' plot(xr, se = TRUE); title(main = "Recurrent events")
##'
##' ## Compare naive and robust standard errors for the recurrent event rate
##' rxr <- robust_phreg(xr, fixbeta = 1)
##' plot(rxr, se = TRUE, robust = TRUE, add = TRUE, col = 4)
##'
##' ## Marginal mean via formula interface
##' outN <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
##'                            data = hf, cause = 1, death.code = 2)
##' plot(outN, se = TRUE, col = 2, add = TRUE)
##' summary(outN, times = 1:5)
##'
##' ## Stratified analysis with logrank test
##' out <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
##'                           data = hf, cause = 1, death.code = 2, test = TRUE)
##' plot(out, se = TRUE, ylab = "Marginal mean", col = 1:2)
##' attr(out, "logrank")
##' summary(out, times = 1:5)
##'
##' ## Influence-function (iid) decomposition at a fixed time point
##' head(iid(outN, time = 3))
##'
##' @aliases recurrent_marginalAIPCW recurrentMarginalPhreg iidRecurrent
##' @export
recurrent_marginal <- function(formula,data,cause=1,...,death.code=2,test=FALSE)
{# {{{
  if (missing(formula)) { # Fall-back to recurrentMarginalPhreg for backward compatibility <= 1.3.5
    return(recurrentMarginalPhreg(...))
  }
  if (inherits(formula,"phreg")) { # Fall-back to recurrentMarginalPhreg
    if (inherits(data, "phreg")) {
        return(recurrentMarginalPhreg(formula, data, ...))
    } else {
        stop("You can give 2 phreg objects \n")
    }
  }

  ## {{{
  cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    specials = c("offset", "weights", "cluster","strata")
    Terms <- terms(formula, specials, data = data)
    ts <- survival::untangle.specials(Terms, "strata")
    if (!is.null(strata)) strata.name <- ts$vars else strata.name <- NULL
###    if (!is.null(strata)) strata.name <- names(des$xlevels) 
###    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of 
    pos.cluster <- NULL
 ## }}}
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

 call.id <- id;
 conid <- construct_id(id,nrow(X),as.data=TRUE)
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid

  statusE <- 1*(status %in% cause)
###  if (is.null(death.code)) statusD <- 1*(!(status %in% cens.code)) else 
  statusD <- 1*(status %in% death.code)
  data$statusE__ <- statusE
  data$statusD__ <- statusD
  data$strata__  <- strata

  if (is.null(call.id))  stop("must specify 'cluster()'\n"); 
  ### setting up formulae for the two phreg (cause of interest and death)
###  if (is.null(call.id)) { 
###     stop("must give id\n"); 
###     formid <- update.formula(formula,~.+cluster(id)) 
###     data$id <- id
###     tt <- terms(formid)
###     tt <- delete.response(tt)
###     formid <- formula(tt)
###  }  else 
  formid <- formula
  tt <- terms(formid)
  tt <- delete.response(tt)
  formid <- formula(tt)

  data$exit__ <- exit
  if (ncol(Y)==3) {
     data$entry__ <- entry
     formE <- as.formula(paste("Surv(entry__,exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(entry__,exit__,statusD__)~+1"))
  } else {
     formE <- as.formula(paste("Surv(exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(exit__,statusD__)~+1"))
  }
 formE <- update.formula(formE,formid)
 formD <- update.formula(formD,formid)
 ## drop marks from terminal event formula
 formD <- drop.specials(formD,"marks")

  if (sum(statusE)==0) warning("No events of type 1\n"); 
  coxE <- phreg(formE,data=data,...)
  coxS <- phreg(formD,data=data,...)

  ### cif 
  if (sum(statusE)>0) meano <- recurrentMarginalPhreg(coxE,coxS) else meano <- coxE
  ## to work with predict function
  meano$no.opt <- TRUE

  if (test) {
	  logrank <- logrankRecurrentBase(coxE,coxS)
  } else logrank <- NULL

  attr(meano,"logrank") <- logrank
  attr(meano,"cause") <- cause
  attr(meano,"death.code") <- death.code
  return(meano)
}# }}}

##' @rdname recurrent_marginal
##' @export
recurrentMarginal <- function(formula,data,...) recurrent_marginal(formula,data,...)

##' @export
recurrentMarginalPhreg <- function(recurrent,death,fixbeta=NULL,km=TRUE)
{# {{{
  xr <- recurrent
  dr <- death 

  ### sets fixbeta based on whether xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) 
  if ((xr$no.opt) | is.null(xr$coef)) fixbeta<- 1 else fixbeta <- 0

  ### marginal expected events  int_0^t G(s) \lambda_r(s) ds 
  # {{{
  strat <- dr$strata[dr$jumps]
  ###
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## survival at t- to also work in competing risks situation
  if (!km) { 
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  x <- xr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  cumhazR <-  cbind(xx$time,cumsumstrata(S0i,xx$strata,xx$nstrata))
  cumhazDR <- cbind(xx$time,cumsumstrata(St*S0i,xx$strata,xx$nstrata))
  mu <- cumhazDR[,2]
# }}}

  ### robust standard errors 
  ### 1. sum_k ( int_0^t S(s)/S_0^r(s) dM_k.^r(s) )^2
 resIM1 <-  squareintHdM(xr,ft=St,fixbeta=fixbeta)
 ### 2. mu(t)^2 * sum_k ( int_0^t 1/S_0^d(s) dM_k.^d(s) )^2
 resIM2 <-  squareintHdM(dr,ft=NULL,fixbeta=fixbeta)
  ### 3. sum_k( int_0^t mu(s) /S_0^d(s) dM_k.^d(s))^2
 resIM3 <-  squareintHdM(dr,ft=mu,fixbeta=fixbeta)

 varA <-  resIM1$varInt+mu^2*resIM2$varInt+resIM3$varInt 

## covariances between different terms  13 23  12 12
## to allow different strata for xr and dr, but still nested strata
 if ((xr$nstrata>1 & dr$nstrata==1)) {
    cM1M3 <- covIntH1dM1IntH2dM2(resIM1,resIM3,fixbeta=fixbeta,mu=NULL)
    cM1M2 <- covIntH1dM1IntH2dM2(resIM1,resIM2,fixbeta=fixbeta,mu=mu)
 } else  {
    cM1M3 <- covIntH1dM1IntH2dM2(resIM3,resIM1,fixbeta=fixbeta,mu=NULL)
    cM1M2 <- covIntH1dM1IntH2dM2(resIM2,resIM1,fixbeta=fixbeta,mu=mu)
 }
 cM2M3 <- covIntH1dM1IntH2dM2(resIM2,resIM3,fixbeta=fixbeta,mu=mu)

 varA <- varA+2*cM1M3$cov12A-2*cM1M2$cov12A-2*cM2M3$cov12A 
### varA <- varA-2*cM1M3$cov12A+2*cM1M2$cov12A+2*cM2M3$cov12A 

 cov12aa <- cov13aa <- cov23aa <- 0

 if (fixbeta==0) {
    varA <-varA + cM2M3$covbeta - cM1M3$covbeta + cM1M2$covbeta 
 }

 varrs <- data.frame(mu=mu,cumhaz=mu,se.mu=varA^.5,time=xr$time,
		     se.cumhaz=varA^.5,strata=xr$strata,St=St)
 varrs <- varrs[c(xr$cox.prep$jumps)+1,]

 ### to use basehazplot.phreg
 ### making output such that basehazplot can work also
 out <- list(mu=varrs$mu,se.mu=varrs$se.mu,times=varrs$time,
     St=varrs$St,
     cumhaz=cbind(varrs$time,varrs$mu),se.cumhaz=cbind(varrs$time,varrs$se.mu),
     strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),
     strata.name=xr$strata.name,strata.level=recurrent$strata.level,call=xr$call,
     formula=xr$formula,no.opt=TRUE,exit=varrs$time,p=0)
 attr(out,"recurrent") <-  recurrent
 attr(out,"death") <- death
 class(out) <- rep("recurrent",2)
 return(out)
}# }}}

##' @export
plot.recurrent <- function(x,ylab=NULL,test=TRUE,...) {# {{{
 if (inherits(x,"recurrent") & is.null(ylab)) ylab <- "Mean events"
 baseplot(x,ylab=ylab,...)
 logrank <- attr(x,"logrank")
 if (!is.null(logrank) & test)
 legend("bottomright",paste("Logrank test, p=",round(logrank$coefmat[,5],3)))
}# }}}

##' @export
summary.recurrent <- function(object,...) {# {{{
out <- summaryRecurrentobject(object,...)
return(out)
}# }}}

summaryRecurrentobject <- function(object,times=NULL,strata=NULL,estimates=FALSE,
			      name="mean",cumhaz="cumhaz",se.cumhaz="se.cumhaz",robust=FALSE,
	      conf.type=c("log","log-log","plain"),...) {# {{{
base <- basecumhaz(object,joint=1,robust=robust,cumhaz=cumhaz,se.cumhaz=se.cumhaz)
nstrata <- object$nstrata
stratobs <- attr(base,"stratobs")

baseci <- rep(list(NULL),nstrata)
if (length(stratobs)>0) 
 for (i in stratobs) {
   cumhaz <- base[[i+1]]$cumhaz
   if (nrow(cumhaz)>=1) {
     mu <- base[[i+1]]$cumhaz[,2]
     se.mu <- base[[i+1]]$cumhaz[,3]
	   if (length(mu)>=1) {
	   conf <- conftype(mu,se.mu,conf.type=conf.type[1],...)
	   out <- data.frame(times=cumhaz[,1],mu=mu,se.mu=se.mu,lower=conf$lower,upper=conf$upper,strata=i)
	   names(out) <- c("times",name,"se","CI-2.5%","CI-97.5%","strata")
	   baseci[[i+1]] <- out
	   } 
   } 
   }

i <- stratobs
pbaseci <- NULL
if (!is.null(times)) {
 if (length(stratobs)>0) 
 for (i in stratobs) 
if (!is.null(baseci[[i+1]])) pbaseci[[i+1]] <- predictCumhaz(rbind(0,baseci[[i+1]]),times) 
}
out <- list(baseci=baseci,pbaseci=pbaseci,times=times)

class(out) <- "summary.recurrent"
return(out)
}# }}}

##' @export
print.summary.recurrent  <- function(x,...) {# {{{
if (is.null(x$times)) print(x$baseci) else print(x$pbaseci)
} # }}}

iidRecurrent <- function(recurrent,death,wt=NULL,km=TRUE,start=0,time=NULL) { ## {{{ 
xr <- recurrent
dr <- death 
if (is.null(time)) time <- max(xr$jumptimes)
### marginal expected events  int_0^t w(s) S(s) \lambda_r(s) ds 
### weight is considered known 
## {{{
###
x <- dr
xx <- x$cox.prep
xstrata <- xx$strata
nstrata <- xx$nstrata

S0i2 <- S0i <- rep(0,length(xstrata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2
## survival at t- to also work in competing risks situation
if (!km) { 
cumhazD <- c(cumsumstratasum(S0i,xstrata,nstrata)$lagsum)
St      <- exp(-cumhazD)
} else St <- c(exp(cumsumstratasum(log(1-S0i),xstrata,nstrata)$lagsum))
x <- xr
xx <- x$cox.prep
S0i2 <- S0i <- rep(0,length(xstrata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2

btime <- (xx$time<=time  & xx$time>start)*1
wt.call <- wt
if (is.null(wt)) wt <- rep(1,length(xx$time))

cumhazR <-  cbind(xx$time,cumsumstrata(S0i*btime,xstrata,nstrata))
meanN <- cumhazDR <- cbind(xx$time,cumsumstrata(wt*St*S0i*btime,xstrata,nstrata))
mu <- cumhazDR[,2]
meanN <- meanN[xx$jumps+1,]

cumhazSR <- cumsumstrata(wt*St*S0i2*btime, xstrata, nstrata)
rr <- c(xx$sign * exp( xx$offset))
MSGt <- wt*St*S0i*btime - cumhazSR*rr*c(xx$weights)
id <- xx$id
MSGtiid <- apply(MSGt,2,sumstrata,id,max(id)+1)

x <- dr
xx <- x$cox.prep
rr <- c(xx$sign * exp( xx$offset))
S0i2 <- S0i <- rep(0,length(xx$strata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2

cumhazD <-  cumsumstrata(S0i2*btime,xstrata,nstrata)
cumhazmuD <-  cumsumstrata(mu*S0i2*btime,xstrata,nstrata)
tails <- tailstrata(xstrata,nstrata)
mut <- mu[tails][xstrata+1]

mutMDt <- mut*S0i*btime-mut*cumhazD*rr*c(xx$weights)
mutMDtiid <- apply(mutMDt,2,sumstrata,id,max(id)+1)

MmuDt <- mu*S0i*btime-cumhazmuD*rr*c(xx$weights)
MmuDtiid <- apply(MmuDt,2,sumstrata,id,max(id)+1)

MGAiid <- MSGtiid-mutMDtiid+MmuDtiid
mid <- max(id)+1

strataid <- xstrata[headstrata(xx$id,mid)]
id <- 1:nrow(MGAiid)

MGAiids <- matrix(0,nrow(MGAiid),nstrata)
cumhaz.time <- c()
sus <- sort(unique(strataid))
for (i in sus)  { 
 wi <- which(strataid==i)
 MGAiids[wi,i+1] <- MGAiid[wi,]
}
MGAiid <- MGAiids
if (is.matrix(MGAiid)) {
colnames(MGAiid) <- paste("strata",sus,sep="")
if (!is.null(x$call.id)) MGAiid <- nameme(MGAiid,x$name.id) 
}

var.iid <- crossprod(MGAiid)
se <- diag(var.iid)^.5
## }}} 

out <- list(iid=MGAiid,wt=wt.call,time=time,start=start,meanN=meanN,var=var.iid,se=diag(var.iid)^.5,mut=mu[tails])
return(out)
} ## }}}

##' @export
estimate.recurrent <- function(x,time=NULL,...) { ## {{{ 
   if (is.null(time)) stop("must give time for iid decomposition \n")
   ic <- IC(x,time=time)
   out <- estimate(coef=attr(ic,"coefs"),IC=ic,...)
return(out)
} ## }}}

##' @export
iid.recurrent <- function(x,...) { ## {{{ 
iid <- iidRecurrent(attr(x,"recurrent"),attr(x,"death"),...)
out <- iid$iid
attr(out,"coefs") <- iid$mut 
attr(out,"time") <-  iid$time
return(out)
} ## }}}

##' @export
IC.recurrent <- function(x,...) { ## {{{ 
iid <- iid(x,...)
ic <- iid*nrow(iid)
return(ic)
} ## }}}

##' Logrank-type test for comparing recurrent event marginal means between groups
##'
##' Tests whether the marginal mean number of recurrent events differs across
##' groups (strata), extending the classical logrank test to the setting of
##' recurrent events with a competing terminal event. The test statistic is
##' \deqn{z = \int_0^\tau w(s)\bigl[d\hat\mu_1(s) - d\hat\mu_2(s)\bigr],}
##' where \eqn{w(s)} is a weight function and \eqn{\hat\mu_j(s)} is the estimated
##' marginal mean for group \eqn{j}. Variance is estimated robustly via the
##' influence functions of Ghosh and Lin (2000).
##'
##' Three weight schemes are available:
##' \describe{
##'   \item{\code{"I"}}{(Default) \eqn{w(t) = R_1(t) R_2(t) / (R_1(t) + R_2(t))},
##'     where \eqn{R_j(t) = Y_j(t) / \hat S_j(t-)}. Analogous to the standard
##'     logrank weight.}
##'   \item{\code{"II"}}{\eqn{w(t) = Y_j(t)}, the raw risk-set size. Equivalent
##'     to using observed counts without survival adjustment.}
##'   \item{\code{"III"}}{A modified weight incorporating the cumulative incidence,
##'     analogous to Gray's test for competing risks.}
##' }
##'
##' @param recurrent Either a \code{"recurrent"} object returned by
##'   \code{\link{recurrent_marginal}}, or a \code{"phreg"} object for the
##'   recurrent event model (in which case \code{death} must also be supplied).
##' @param death A \code{"phreg"} object for the terminal event model. Required
##'   when \code{recurrent} is a \code{"phreg"} object; ignored otherwise.
##' @param weight Character string specifying the weight scheme: \code{"I"},
##'   \code{"II"}, or \code{"III"}. Default is \code{"I"}. For competing risks \code{"III"} 
##'    gives Gray's test with robust standard errors.
##' @param km Logical. If \code{TRUE} (default), the Kaplan-Meier estimator is
##'   used for the survival probability \eqn{S(t)}; otherwise the Nelson-Aalen
##'   estimator is used.
##' @param start Left truncation time for the integration. Default is \code{0}.
##' @param stop Right truncation time for the integration. Defaults to the last
##'   observed jump time.
##' @param at.risk Minimum combined risk-set size below which the weight is set
##'   to zero. Default is \code{5}.
##' @param cluster.id Optional vector of cluster identifiers for aggregating
##'   influence functions across clusters before forming the test statistic.
##' @param ... Currently unused.
##'
##' @return An object of class \code{"estimate"} (from the \pkg{lava} package)
##'   with the following components:
##'   \item{coef}{The weighted difference in marginal means between groups.}
##'   \item{se}{Robust standard error of the test statistic.}
##'   \item{lower, upper}{95\% confidence interval bounds.}
##'   \item{p.value}{Two-sided p-value for the null hypothesis of no difference.}
##'   The object also carries an \code{iid} attribute containing the
##'   subject-level influence function decomposition of the test statistic,
##'   which can be used for further inference or combination with other estimators.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{logrankRecurrentBase}}
##'
##' @references
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
##' and death. \emph{Biometrics}, 56, 554--562.
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##'
##' ## Test using two separate phreg models
##' xr <- phreg(Surv(entry, time, status == 1) ~ strata(treatment) + cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ strata(treatment) + cluster(id), data = hf)
##' out <- test_logrankRecurrent(xr, dr, stop = 5)
##' summary(out)
##'
##' ## Equivalently, using a recurrent_marginal object directly
##' outN <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
##'                            data = hf, cause = 1, death.code = 2)
##' test_logrankRecurrent(outN)
##'
##' @aliases logrankRecurrentBase
##' @export
test_logrankRecurrent <- function(recurrent,death,
              weight=c("I","II","III"),km=TRUE,start=0,stop=NULL,at.risk=5,cluster.id=NULL,...) { ## {{{ 
  if (inherits(recurrent,"phreg")) { # Fall-back to recurrentMarginalPhreg
    if (inherits(death, "phreg")) {
        return(logrankRecurrentBase(recurrent,death,
		weight=weight[1],km=km,start=start,stop=stop,at.risk=at.risk,
		cluster.id=cluster.id))
     } 
     return(NULL)
  } else if (inherits(recurrent,"recurrent")) { # Fall-back to recurrentMarginalPhreg
        return(logrankRecurrentBase(attr(recurrent,"recurrent"),attr(recurrent,"death"),
		weight=weight[1],km=km,start=start,stop=stop,at.risk=at.risk,cluster.id=cluster.id))
 } else stop("input either output from recurrentMarginal or two phreg's\n")

} ## }}}

##' @export
logrankRecurrentBase <- function(recurrent,death,weight=c("I","II","III"),km=TRUE,
       start=0,stop=NULL,at.risk=0,cluster.id=NULL) { ## {{{ 
  xr <- recurrent
  dr <- death 
  if (is.null(stop)) stop <- max(xr$jumptimes)

  nlev <- xr$nstrata
  ###
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  n <- dr$n

  Yrr <- revcumsumstrata(xx$sign,xx$strata,xx$nstrata)
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/Yrr[xx$jumps+1]

  Sta <- cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)
  St <- c(exp(Sta$lagsum))
  kmss <- exp(Sta$alllagsum)
  Yss <- .Call("_mets_riskstrataR",xx$sign,xx$strata,xx$nstrata)$risk

  if (weight[1]=="I") {
	  Rss <-  Yss/kmss
	  Rss[Yss==0] <- 0 
  }
  if (weight[1]=="III") {
     xx <- xr$cox.prep
     S0ri <- rep(0,length(xx$strata))
     S0ri[xx$jumps+1] <-  1/Yrr[xx$jumps+1]
     cifsm <- cumsumstratasum(St*S0ri,xx$strata,xx$nstrata)
     F1c <-  1-cifsm$alllagsum 
     Rss <-  F1c*Yss/kmss
     Rss[Yss==0] <- 0 
     F1ci <- rep(0,length(xx$strata))
     F1ci[xx$jumps+1] <-  1/(1-cifsm$lagsum[xx$jumps+1])
  }
  if (weight[1]=="II") {
     Rss <- Yss 
  }
  Rp <- apply(Rss,1,sum,na.rm=TRUE)
  Rssstrat <- mdi(Rss,1:length(xx$strata),xx$strata+1)

  contr <- contr.iid <- c()
  i <- 1
  for (i in 1:(nlev-1)) {
     wt <- rep(0,length(xx$strata))
     strati  <- which(xx$strata==(i-1))
     wt <- Rss[,i]*Rssstrat/Rp
     wt[strati] <- Rss[strati,i]*(Rp[strati]-Rss[strati,i])/Rp[strati]
     wt[is.na(wt)] <- 0
      if (at.risk>0) {
        minrisk <- pmin(Yss[,i],apply(Yss[,-i,drop=FALSE],1,sum))
        wt[minrisk<=at.risk] <- 0
      }
     ## adjust weight to get Grays test with [1-F_1(s-)]^-1
     if (weight[1]=="III") wt <- wt*F1ci

      lrt <- iidRecurrent(xr,dr,wt=wt)
      diff(lrt$mut)
      if (!is.null(cluster.id)) {
	      nid <- length(unique(cluster.id))
	      conid <- construct_id(cluster.id,nid)
	      lrt$iid <- apply(lrt$iid,2,sumstrata,conid$id,conid$nid)
      }
      diffi <- lrt$mut[i]-sum(lrt$mut[-i])
      contr <- c(contr,diffi)
      contr.iid <- cbind(contr.iid,lrt$iid[,i]-apply(lrt$iid[,-i,drop=FALSE],1,sum))
   }
   logrank <- summary(estimate(coef=contr,IC=contr.iid*nrow(contr.iid)),null=0)
   p <- length(contr)

   return(logrank)
} ## }}} 

##' @export
recurrent_marginalAIPCW <- function(formula,data=NULL,cause=1,cens.code=0,death.code=2,
	  cens.model=~1,km=TRUE,times=NULL,augment.model=~Nt,...)
{# {{{
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    id <- strata <- NULL
    if (!is.null(attributes(Terms)$specials$cluster)) {
        ts <- survival::untangle.specials(Terms, "cluster")
        pos.cluster <- ts$terms
        Terms  <- Terms[-ts$terms]
        id <- m[[ts$vars]]
    } else pos.cluster <- NULL
    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
        ts <- survival::untangle.specials(Terms, "strata")
        pos.strata <- ts$terms
        Terms  <- Terms[-ts$terms]
        strata <- m[[ts$vars]]
        strata.name <- ts$vars
    }  else { strata.name <- NULL; pos.strata <- NULL}
    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
        ts <- survival::untangle.specials(Terms, "offset")
        Terms  <- Terms[-ts$terms]
        offset <- m[[ts$vars]]
    }
    X <- model.matrix(Terms, m)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## }}}

   if (!is.null(id)) {
        ids <- unique(id)
        nid <- length(ids)
        if (is.numeric(id))
            id <-  fast.approx(ids,id)-1
        else  {
            id <- as.integer(factor(id,labels=seq(nid)))-1
        }
    } else { id <- as.integer(seq_along(entry))-1;  nid <- nrow(X); }
    ## orginal id coding into integers 1:...
    id.orig <- id+1;

   ### setting up with artificial names
 data$status__ <-  status 
 data$id__ <-  id
 ## lave Countcause
 data <- count_history(data,status="status__",id="id__",types=cause,multitype=TRUE)
 data$Count1__ <- data[,paste("Count",cause[1],sep="")]
 data$death__ <- (status %in% death.code)*1
 data$entry__ <- entry 
 data$exit__ <- exit 
 data$statusC__ <- (status %in% cens.code)*1
 data$status__cause <- (status %in% cause)*1

  xr <- phreg(Surv(entry__,exit__,status__cause)~Count1__+death__+cluster(id__),data=data,no.opt=TRUE,no.var=1)

  formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
  cr <- phreg(formC,data=data)
  whereC <- which(status %in% cens.code)

  if (length(whereC)>0) {# {{{
  ### censoring weights
  strat <- cr$strata[cr$jumps]
  x <- cr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## survival at t- to also work in competing risks situation
  if (!km) { 
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  } else  St <- rep(1,length(xx$strata))
  Gc <- St
  ## }}}

  
formD <- as.formula(Surv(entry__,exit__,death__)~cluster(id__))
form1L <- as.formula(Surv(entry__,exit__,status__cause)~Count1__+death__+statusC__+cluster(id__))
form1 <- as.formula(Surv(entry__,exit__,status__cause)~cluster(id__))

 xr <- phreg(form1L,data=data,no.opt=TRUE,no.var=1)
 dr <- phreg(formD,data=data,no.opt=TRUE,no.var=1)

 ### augmenting partioned estimator computing \hat H_i(s,t) for fixed t
 data$Gctrr <- exp(-cpred(rbind(0,cr$cumhaz),exit)[,2])

 ### cook-lawless-ghosh-lin
 xr0 <- phreg(form1,data=data,no.opt=TRUE)
 clgl  <- recurrentMarginalPhreg(xr0,dr)
### bplot(clgl,se=1); print(cpred(clgl$cumhaz,times)); print(cpred(clgl$se.cumhaz,times)); 

  ####  First \mu_ipcw(t) \sum_i I(T_i /\ t \leq C_i)/G_c(T_i /\ t ) N_(T_i /\ t) {{{
  x <- xr
  xx <- xr$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## stay with N(D_i) when t is large so no -1 when death
  ## xx$X[,1] er Count1 dvs N(t-)
  Nt <- revcumsumstrata(xx$X[,1]*xx$sign,xx$strata,xx$nstrata)
  Nt <- Nt/Gc
  ## counting N(D) forward in time skal ikke checke ud når man dør N_(D_i) er i spil efter D_i
  NtD <- cumsumstrata(xx$X[,1]*(xx$X[,2]==1)*(xx$sign==1)/Gc,xx$strata,xx$nstrata)
  jump1 <- xx$jumps+1
  timeJ <- xx$time[jump1]
  avNtD <- (NtD+Nt)[jump1]/nid
  strataN1J <- xx$strata[jump1]

  varIPCW1 <- NULL
  seIPCW1 <- NULL

  ### IPCW estimator 
  cumhaz <- cbind(timeJ,avNtD)
# }}}

  ### Partitioned estimator , same as Lin, Lawless & Cook estimator {{{
  cumhazP <- c(cumsumstrata(1/Gc[jump1],strataN1J,xx$nstrata)/nid)
  cumhazP <- cbind(timeJ,cumhazP)

  ### variance of partitioned estimator 

  ### calculate E(H,s,t) = E(H,t) - E(H,s) 
  ### E(H,t) = 1/(S(t)*n) \sum_ \int Y_i(s)/G_c(s) dN_{1i} = 1/(S(t)) (\mu_p(t) - \mu_p(s)) 
  ### \hat H_i for all subjects, and look at together with Hst, eHst 
  ### for censoring martingale 
  data$Nt <- data$Count1__
  data$Nt2 <- data$Nt^2
  data$expNt <- exp(-data$Nt)
  data$NtexpNt <- data$Nt*exp(-data$Nt)

  Gcdata <- exp(-cpred(rbind(0,cr$cumhaz),exit)[,2])

  form <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
  desform <- update.formula(augment.model,~Hst + . + cluster(id__))
  form[[3]] <- desform[[2]]
  nterms <- length(all.vars(form[[3]]))-1

  if (!is.null(times)) {
       semuPA <-  muPA <- semuPA.times <-  muPA.times <- rep(0,length(times))
       ww <- fast.approx(timeJ,times,type="left")
       muP.times <- cumhazP[ww,2]
       semuP.times <- clgl$se.mu[ww]

  for (i in seq_along(times)) {
     timel <- times[i]
     data$Hst <- revcumsumstrata((exit<timel)*(status %in% cause)/Gcdata,id,nid)
     cr2 <- phreg(form,data=data,no.opt=TRUE,no.var=1)
     nterms <- cr2$p-1

     dhessian <- cr2$hessianttime
     dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
     ###  matrix(apply(dhessian,2,sum),3,3)
     timeb <- which(cr$cumhaz[,1]<timel)
     ### take relevant \sum H_i(s,t) (e_i - \bar e)
     covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
     ### construct relevant \sum (e_i - \bar e)^2
     Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
     ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
     gammahat <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
     S0 <- cr$S0[timeb]
     gammahat[is.na(gammahat)] <- 0
     gammahat[gammahat==Inf] <- 0
     Gctb <- Gc[cr$cox.prep$jumps+1][timeb]
     augment.times <- sum(apply(gammahat*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum))/nid
     mterms <- length(terms)
     mterms <- nterms
     ###
     varZ <- matrix(apply(Pt/Gctb^2,2,sum),mterms,mterms)
     gamma <- .Call("CubeVec",matrix(c(varZ),nrow=1),matrix(apply(covts/Gctb,2,sum),nrow=1),1,PACKAGE="mets")$XXbeta
     gamma <- c(gamma)
     gamma[is.na(gamma)] <- 0
     gamma[gamma==Inf] <- 0
     augment <- sum(apply(gamma*t(cr2$U[timeb,1+1:nterms,drop=FALSE])/Gctb,2,sum))/nid
     ###
     muPA[i] <- muP.times[i]+augment
     semuPA[i] <- (semuP.times[i]^2 +(gamma %*% varZ %*% gamma)/nid^2)^.5
     muPA.times[i] <- muP.times[i]+augment.times
     semuPA.times[i] <- (semuP.times[i]^2+sum(gammahat*.Call("CubeVec",Pt,gammahat,0,PACKAGE="mets")$XXbeta)/(nid^2))^.5
  }
  } else { Gctb <- NULL;gammahat <- NULL; 
  muPA.times <-semuPA.times <- muPA  <- semuPA <- NULL }
# }}}

  return(list(censoring.weights=Gctb,muP.all=cumhazP,Gcjump=Gc[jump1],gamma=gamma,gamma.time=gammahat,times=times,
  muP=muP.times,semuP=semuP.times, muPAt=muPA.times,semuPAt=semuPA.times, muPA=muPA,semuPA=semuPA))
}# }}}

##' Summarize a Time-Varying Estimate with Confidence Bands
##'
##' Extracts estimates at specified time points with confidence intervals.
##'
##' @param mutimes vector of estimation time points.
##' @param mu vector or matrix of estimates.
##' @param se.mu vector or matrix of standard errors (optional).
##' @param times time points at which to evaluate (default: \code{mutimes}).
##' @param type confidence interval type: \code{"log"} or \code{"plain"}.
##' @param level confidence level (default 0.95).
##' @param ... additional arguments.
##' @return A data.frame with columns: times, mean, se-mean, CI-2.5\%, CI-97.5\%.
##' @export
summaryTimeobject <-function(mutimes,mu,se.mu=NULL,times=NULL,type="log",level=0.95,...) {# {{{
 if (is.null(times)) times <- mutimes

 where <- fast.approx(c(0,mutimes),times,type="left")

 crit <- qnorm(1-(1-level)/2)
 ##  see if object is vector or matrix
 if (is.matrix(mu)) mu <- rbind(0,mu)[where,] else mu <- c(0,mu)[where]
 if (!is.null(se.mu)) {
     if (is.matrix(se.mu)) se.mu <- rbind(0,se.mu)[where,] else se.mu <- c(0,se.mu)[where]
 se.logmu=se.mu/mu
 if (type=="log") {
 lower <- exp(log(mu) - crit*se.logmu)
 upper <- exp(log(mu) + crit*se.logmu)
 } else {
 lower <- mu - crit*se.mu
 upper <- mu + crit*se.mu
 }
 } else {se.mu <- se.logmu <- lower <- upper <- NA}

 out <- data.frame(times=times,mu=mu,se.mu=se.mu,lower=lower,upper=upper)
 names(out) <- c("times","mean","se-mean","CI-2.5%","CI-97.5%")
 return(out)
}# }}}

recmarg <- function(recurrent,death,Xr=NULL,Xd=NULL,km=TRUE,...)
{# {{{
  xr <- recurrent
  dr <- death 

  if (!is.null(Xr)) rr <- exp(sum(xr$coef * Xr)) else rr <- 1
  if (!is.null(Xd)) rrd <- exp(sum(dr$coef * Xd)) else rrd <- 1

  ### marginal expected events  int_0^t G(s) \lambda_r(s) ds 
  # {{{
  strat <- dr$strata[dr$jumps]
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  if (!km) { 
     cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
     St      <- exp(-cumhazD*rrd)
  } else St <- exp(rrd*c(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  ###
  x <- xr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  cumhazDR <- cbind(xx$time,cumsumstrata(St*S0i,xx$strata,xx$nstrata))
  mu <- rr*cumhazDR[,2]
# }}}

 varrs <- data.frame(mu=mu,time=xr$time,strata=xr$strata,St=St)
 varrs  <-  varrs[c(xr$cox.prep$jumps)+1,]

 ### to use basehazplot.phreg
 ### making output such that basehazplot can work also
 out <- list(mu=varrs$mu,time=varrs$time,
	     St=varrs$St,cumhaz=cbind(varrs$time,varrs$mu),
             strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),
	     strata.name=xr$strata.name,strata.level=recurrent$strata.level)
 return(out)
}# }}}

squareintHdM <- function(phreg,ft=NULL,fixbeta=NULL,beta.iid=NULL,...)
{# {{{
###  sum_k ( int_0^t f(s)/S_0^r(s) dM_k.^r(s) )^2
###  strata "r" from object and "k" id from cluster 
  if (!inherits(phreg,"phreg")) stop("Must be phreg object\n"); 

  ### sets fixbeta based on whether xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) 
  if ((phreg$no.opt) | is.null(phreg$coef)) fixbeta<- 1 else fixbeta <- 0

  x <- phreg
  xx <- x$cox.prep
  ww <- xx$caseweights*xx$weights
 
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/(x$S0^2*ww[xx$jumps+1])
  Z <- xx$X
  U <- E <- matrix(0,nrow(xx$X),x$p)
  E[xx$jumps+1,] <- x$E
  U[xx$jumps+1,] <- x$U
  cumhaz <- cbind(xx$time,cumsumstrata(S0i,xx$strata,xx$nstrata))
  if (is.null(ft))  ft <- rep(1,length(xx$time))
  cumS0i2 <- c(cumsumstrata(ft*S0i2,xx$strata,xx$nstrata))
  if (fixbeta==0) {
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Ht <- apply(ft*E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
  } else Ht <- NULL
  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
  else rr <- c(xx$sign*exp(xx$offset))
  id <-   xx$id
  mid <- max(id)+1
  ### also weights 
  w <- c(xx$weights)
  xxx <- (ft*S0i-rr*cumS0i2*w)
  ssf <- cumsumidstratasum(xxx,id,mid,xx$strata,xx$nstrata)$sumsquare
  ss <-  revcumsumidstratasum(w*rr,id,mid,xx$strata,xx$nstrata)$lagsumsquare*cumS0i2^2
  covv <- covfridstrata(xxx,w*rr,id,mid,xx$strata,xx$nstrata)$covs*cumS0i2
  varA1 <- c(ssf+ss-2*covv)

  vbeta <- betaiidR <- NULL
  if (fixbeta==0) {# {{{
    if (!is.null(beta.iid))  betaiidR <- beta.iid else {
	     invhess <- -pinv(x$hessian)
	     MGt <- U[,drop=FALSE]-(Z*cumhaz[,2]-EdLam0)*rr*c(xx$weights)
	     UU <- apply(MGt,2,sumstrata,id,mid)
	     betaiidR <- UU %*% invhess
     }
     vbeta <- crossprod(betaiidR)
     varbetat <-   rowSums((Ht %*% vbeta)*Ht)
     ### writing each beta for all individuals 
     betakt <- betaiidR[id+1,,drop=FALSE]
     covk1 <- apply(xxx*betakt,2,cumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="sum")
     covk2 <- apply(w*rr*betakt,2,revcumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="lagsum")
     covk2 <- covk2*cumS0i2
     covv <- covk1-covk2
     varA1 <- varA1+varbetat-2*apply(covv*Ht,1,sum)
  }# }}}

  return(list(xx=xx,Ht=Ht,varInt=varA1,xxx=xxx,rr=rr,
	      cumhaz=cumhaz,cumS0i2=cumS0i2,mid=mid,id=id,
	      betaiid=betaiidR,vbeta=vbeta,covv=covv))
} # }}}

covIntH1dM1IntH2dM2 <- function(square1,square2,fixbeta=1,mu=NULL)
{# {{{

 ### strata and id same for two objects 
 xx <- square1$xx; xx2 <- square2$xx
 xxxR <- square1$xxx;     xxxD1 <- square2$xxx
 rrR  <- square1$rr;       rrD1 <- square2$rr
 id   <- id1 <- square1$id; id2 <- square2$id
 mid  <- square1$mid; w <- c(xx$weights)

 if (is.null(mu)) mu <- rep(1,length(xx$strata))

 cov12 <-  c(cumsumidstratasumCov(xxxR,xxxD1,id,mid,xx$strata,xx$nstrata)$sumsquare)
 cov122 <- c(revcumsumidstratasumCov(w*rrR,w*rrD1,id,mid,xx$strata,xx$nstrata)$lagsumsquare)*c(square1$cumS0i2*square2$cumS0i2)
 cov123 <- covfridstrataCov(xxxR,w*rrR,xxxD1,w*rrD1,id,mid,xx$strata,xx$nstrata)$covs*c(square2$cumS0i2)
 cov124 <- covfridstrataCov(xxxD1,w*rrD1,xxxR,w*rrR,id,mid,xx$strata,xx$nstrata)$covs*c(square1$cumS0i2)
 cov12A <- c(cov12+cov122+cov123+cov124)
 
 test <- 0
 if (test==1) {# {{{
	 print("________cov cov ___________________________"); 
	 print(summary(c(xxxR))); print(summary(c(xxxD1))); 
	 print(summary(c(rrD1))); print(summary(c(rrR)))
	 print(summary(c(square1$cumS0i2))); print(summary(c(square2$cumS0i2)));
	 print("-----------"); 
	 print(summary(cov12)); 
	 print(summary(cov122)); 
	 print(summary(c(cov123))); 
	 print(summary(c(cov124))); 
	 print("______________________________________"); 
	 jumps <- c(square1$xx$jumps,square2$xx$jumps)+1
	 print(summary(jumps))
         print(summary(cov12[jumps])); 
	 print(summary(cov122[jumps])); 
	 print(summary(c(cov123[jumps]))); 
	 print(summary(c(cov124[jumps]))); 
 }# }}}

 cov12aa <- 0
 if (fixbeta==0) {
 ### covariances between different terms and  beta's 
 # {{{
	 betaiidR <- square1$betaiid; betaiidD <- square2$betaiid
	 HtR <- square1$Ht; HtD <- square2$Ht
	 covbetaRD <- t(betaiidR) %*% betaiidD
	 covbeta <-   -1*rowSums((HtR %*% covbetaRD)*HtD)

	 ### cov12 wrt betaD and betaR
	 betakt <- betaiidD[id1+1,,drop=FALSE]
	 covk1 <-apply(xxxR*betakt,2,cumsumidstratasum,id1,mid,xx$strata,xx$nstrata,type="sum")
	 covk2 <-apply(w*rrR*betakt,2,revcumsumidstratasum,id1,mid,xx$strata,xx$nstrata,type="lagsum")
	 covk2 <- c(covk2)*c(square1$cumS0i2)
	 covRD12 <- apply((covk1-covk2)*HtD,1,sum)
	 betakt <- betaiidR[id2+1,,drop=FALSE]
	 covk1 <-apply(xxxD1*betakt,2,cumsumidstratasum,id2,mid,xx2$strata,xx$nstrata,type="sum")
	 covk2 <-apply(w*rrD1*betakt,2,revcumsumidstratasum,id2,mid,xx2$strata,xx$nstrata,type="lagsum")
	 covk2 <- c(covk2)*c(square2$cumS0i2)
	 covRD21 <- apply((covk1-covk2)*HtR,1,sum)
	 cov12aa <- 2*(covbeta + covRD12+covRD21)
	 test <- 0
	 if (test==1) {
		 print("--------------")
		 print(summary(2*mu*covbeta))
		 print(summary(2*mu*covRD12))
		 print(summary(2*mu*covRD21))
		 print("--------------")
	 }
 } # }}}

 cov12 <- (cov12A-cov12aa)*mu

 return(list(cov=cov12,cov12A=cov12A*mu,covbeta=cov12aa*mu))
} # }}}

##' Break ties in event times for recurrent event data
##'
##' Resolves tied event times in a counting-process dataset by adding a small
##' random perturbation to duplicated exit times of event rows. This is a
##' preprocessing step required by \code{\link{recurrent_marginal}} and related
##' functions, which assume unique jump times within each stratum.
##'
##' A tie is defined as an event exit time (rows where \code{status} is a cause
##' code) that coincides with another exit time in the dataset. When
##' \code{exit.unique = TRUE} (default), a tie is flagged whenever an event time
##' also appears among any other exit times (censored or event). When
##' \code{exit.unique = FALSE}, only exact ties between two event rows are
##' resolved.
##'
##' Tied event times are perturbed by adding \eqn{U \cdot \delta} where
##' \eqn{U \sim \text{Uniform}(0, 1)} and \eqn{\delta} is \code{ddt} (defaulting
##' to half the smallest observed positive gap between consecutive exit times).
##' When subject IDs are provided via \code{id}, the corresponding interval start
##' time of the immediately following row for the same subject is updated to
##' maintain a valid counting-process structure, and a logical \code{tiebreaker}
##' column is added to flag affected rows.
##'
##' @param data A data frame in counting-process format, sorted by subject and
##'   time.
##' @param stop Name of the column containing interval exit (stop) times.
##'   Default is \code{"time"}.
##' @param start Name of the column containing interval entry (start) times,
##'   used to update the following row when \code{id} is supplied. Default is
##'   \code{"entry"}.
##' @param status Name of the column containing event status codes. Default is
##'   \code{"status"}.
##' @param id Name of the column containing subject identifiers. If supplied,
##'   the start time of the next interval for the same subject is adjusted to
##'   match the perturbed stop time, preserving interval continuity. Default is
##'   \code{NULL} (no adjustment made).
##' @param ddt Maximum perturbation size. Tied event times are shifted by a
##'   uniform draw on \eqn{[0, \text{ddt}]}. If \code{NULL} (default), \code{ddt}
##'   is set to half the smallest positive gap between any two consecutive exit
##'   times in the data.
##' @param exit.unique Logical. If \code{TRUE} (default), an event time is
##'   considered tied whenever it coincides with \emph{any} exit time in the
##'   data (including censored rows). If \code{FALSE}, only ties among event rows
##'   are resolved.
##' @param cause Integer vector of status codes that identify events (non-censored
##'   rows). If \code{NULL} (default), all non-censoring status values are treated
##'   as events.
##' @param cens.code Integer code(s) for censoring. Rows with this status are
##'   never perturbed. Default is \code{0}.
##' @param seed Optional integer passed to \code{set.seed()} before drawing the
##'   perturbations, making results reproducible. Default is \code{NULL} (no seed
##'   set).
##'
##' @return The input data frame \code{data} with tied event exit times
##'   perturbed. If \code{id} is supplied, an additional logical column
##'   \code{tiebreaker} marks rows whose start time was adjusted as a consequence
##'   of a perturbation in the preceding row.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{test_logrankRecurrent}}
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##'
##' ## Check for ties in event exit times
##' ev <- hf[hf$status == 1, ]
##' any(duplicated(ev$time))
##'
##' ## Resolve ties before fitting the marginal mean model
##' hf_clean <- tie_breaker(hf, stop = "time", start = "entry",
##'                          status = "status", id = "id",
##'                          cause = 1, cens.code = 0)
##'
##' out <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
##'                            data = hf_clean, cause = 1, death.code = 2)
##' summary(out, times = 1:5)
##'
##' @export
tie_breaker <- function(data, stop = "time", start = "entry", status = "status",
                        id = NULL, cause = NULL, cens.code = 0,
                        exit.unique = TRUE, ddt = NULL, seed = NULL)
{# {{{
   id.col <- id
   if (!is.null(id)) id <- data[, id]
   ord  <- 1:nrow(data)
   stat <- data[, status]
   time <- data[, stop]

   ## determine event codes, excluding censoring
   if (is.null(cause)) cause <- unique(stat)
   type0 <- which(cause %in% cens.code)
   if (length(type0) > 0) cause <- cause[-type0]
   jumps <- stat %in% cause

   ## detect tied event times
   time1 <- data[jumps,  stop]
   time0 <- data[!jumps, stop]
   lt0   <- length(time0)
   ddp   <- duplicated(c(time0, time1))
   if (exit.unique)
     ties <- ddp[(lt0 + 1):nrow(data)]
   else
     ties <- duplicated(c(time1))

   nties <- sum(ties)

   ## initialise tiebreaker column unconditionally
   data[, "tiebreaker"] <- FALSE

   if (nties >= 1) {
     message(nties, " tied event time(s) found and perturbed.")

     ordties <- ord[jumps][ties]

     ## compute ddt from sorted unique event-time gaps, not row differences
     if (is.null(ddt)) {
       ut  <- sort(unique(time[jumps]))
       gaps <- diff(ut)
       ddt <- min(gaps[gaps > 0]) * 0.5
     }

     ## apply reproducible perturbation
     if (!is.null(seed)) set.seed(seed)
     time[ordties] <- time[ordties] + runif(nties) * ddt
     data[ordties, stop] <- time[ordties]

     ## mark perturbed rows
     data[ordties, "tiebreaker"] <- TRUE

     ## when id is available, propagate stop -> start for the next interval
     ## of the same subject to maintain a valid counting-process structure
     if (!is.null(id.col)) {
       ties_flag  <- ord %in% ordties
       lagties    <- dlag(ties_flag)
       change.start <- lagties == TRUE & id == dlag(id)
       change.start[is.na(change.start)] <- FALSE
       ocs <- ord[change.start]
       data[ocs, start] <- data[ocs - 1, stop]
     }
   }

   return(data)
 } # }}}


##' Simulate recurrent events with two event types and a terminal event
##'
##' Simulates recurrent event data with up to two distinct event types and an
##' optional terminal event (death), based on user-supplied cumulative hazard
##' functions. Dependence between processes can be introduced via shared or
##' correlated gamma-distributed frailties.
##'
##' The simulation proceeds by sequentially drawing the next event time from the
##' specified cumulative hazards, taking the minimum of the two recurrent event
##' times, and stopping each subject at death or administrative censoring.
##'
##' Dependence between processes is controlled by \code{dependence}:
##' \describe{
##'   \item{\code{0}}{Independence: all subjects have frailty fixed at 1.}
##'   \item{\code{1}}{Shared frailty: all processes share a single
##'     gamma-distributed random effect with mean 1 and variance \code{var.z}.}
##'   \item{\code{4}}{Recurrent-event frailty only: the two recurrent event
##'     processes share a gamma frailty but the terminal event is independent.}
##' }
##' For more complex correlation structures across two event types and death, use
##' \code{\link{sim_recurrentTS}}.
##'
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the first type of recurrent event.
##' @param cumhaz2 Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the second type of recurrent event.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event. If \code{NULL}, no terminal event
##'   is simulated and follow-up ends at the end of \code{cumhaz}.
##' @param r1 Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the first event type.
##' @param r2 Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the second event type.
##' @param rd Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the terminal event.
##' @param rc Optional numeric vector of length \code{n} with subject-specific
##'   multipliers for the exponential censoring rate.
##' @param gap.time Logical. If \code{TRUE}, event times are drawn as gap times
##'   (time since the last event) rather than calendar times. Default is
##'   \code{FALSE}.
##' @param max.recurrent Maximum number of recurrent events allowed per subject.
##'   Default is \code{100}.
##' @param dependence Integer specifying the frailty structure. One of \code{0}
##'   (independence), \code{1} (shared gamma frailty), or \code{4} (shared frailty
##'   for recurrent events only). See Details.
##' @param var.z Variance of the gamma-distributed frailty. Default is \code{1}.
##' @param cor.mat Correlation matrix for the random effects. Used when
##'   \code{dependence = 2} (in \code{\link{sim_recurrent_list}}).
##' @param cens Rate of exponential censoring. If \code{NULL} (default), no
##'   additional censoring is applied.
##' @param ... Further arguments passed to \code{\link{sim_recurrent_list}}.
##'
##' @return A data frame in counting-process format (one row per event interval
##'   per subject) with columns:
##'   \item{id}{Subject identifier.}
##'   \item{start, entry}{Interval start time.}
##'   \item{stop, time}{Interval end time (event or censoring time).}
##'   \item{status}{Event type at \code{stop}: \code{1} or \code{2} for a
##'     recurrent event of the corresponding type, \code{0} for censoring.}
##'   \item{death}{Indicator for a terminal event (\code{1}) or censoring/survival
##'     (\code{0}).}
##'   Attributes \code{"cumhaz"}, \code{"death.cumhaz"}, \code{"rr"}, and
##'   \code{"rd"} store the inputs used for simulation.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{sim_recurrent}}, \code{\link{sim_recurrent_list}},
##'   \code{\link{sim_recurrentTS}}
##'
##' @examples
##' data(CPH_HPN_CRBSI)
##' dr    <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi
##' base4 <- CPH_HPN_CRBSI$mechanical
##'
##' ## Single recurrent event type, with and without terminal event
##' rr <- sim_recurrent(5, base1)
##' dlist(rr, . ~ id, n = 0)
##'
##' rr <- sim_recurrent(5, base1, death.cumhaz = dr)
##' dlist(rr, . ~ id, n = 0)
##'
##' ## Verify that estimated rates recover the true baselines (increase n for precision)
##' rr <- sim_recurrent(100, base1, death.cumhaz = dr)
##' par(mfrow = c(1, 3))
##' mets:::showfitsim(causes = 1, rr, dr, base1, base1)
##'
##' ## Shared frailty across all processes
##' rr <- sim_recurrent(100, base1, death.cumhaz = dr, dependence = 1, var.z = 0.4)
##' dtable(rr, ~death + status)
##'
##' ## Two event types; second type uses the mechanical complication rate
##' set.seed(100)
##' rr <- sim_recurrentII(100, base1, base4, death.cumhaz = dr)
##' dtable(rr, ~death + status)
##' par(mfrow = c(2, 2))
##' mets:::showfitsim(causes = 2, rr, dr, base1, base4)
##'
##' ## Three event types and two causes of death via sim_recurrent_list
##' set.seed(100)
##' cumhaz <- list(base1, base1, base4)
##' drl    <- list(dr, base4)
##' rr     <- sim_recurrent_list(100, cumhaz, death.cumhaz = drl, dependence = 0)
##' dtable(rr, ~death + status)
##' mets:::showfitsimList(rr, cumhaz, drl)
##'
##' @export
##' @aliases sim_recurrentII sim_recurrent_list
sim_recurrentII <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,r1=NULL,r2=NULL,rd=NULL,rc=NULL,dependence=0,var.z=1,
			   cor.mat=NULL,cens=NULL,gap.time=FALSE,max.recurrent=100,...) 
{# {{{
cumhazL <- list(cumhaz,cumhaz2)
rr <- cbind(r1,r2)
if (!is.null(death.cumhaz)) death.cumhaz <- list(death.cumhaz)
if (!is.null(r1)) {
	if (!is.null(r2)) r2 <- rep(1,length(r1))
	rr <- cbind(r1,r2)
}
data <-     sim_recurrent_list(n,cumhazL,death.cumhaz=death.cumhaz,rr=rr,
		     rd=rd,rc=rc,dependence=dependence,var.z=var.z,
		     cor.mat=cor.mat,cens=cens,gap.time=gap.time,
		     max.recurrent=max.recurrent,...)
return(data)
}# }}}

##' Simulate recurrent events with a single event type and a terminal event
##'
##' A convenience wrapper around \code{\link{sim_recurrentII}} for the common
##' case of a single recurrent event type. Frailty and censoring options are
##' passed through to \code{\link{sim_recurrent_list}}.
##'
##' @inherit sim_recurrentII examples author
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the recurrent event.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event. If \code{NULL}, no terminal event
##'   is included.
##' @param r1 Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the recurrent event.
##' @param rd Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the terminal event.
##' @param rc Optional numeric vector of length \code{n} of subject-specific
##'   multipliers for the exponential censoring rate.
##' @param ... Further arguments passed to \code{\link{sim_recurrent_list}},
##'   including \code{dependence}, \code{var.z}, \code{gap.time}, and
##'   \code{max.recurrent}.
##' @export
sim_recurrent <- function(n,cumhaz,death.cumhaz=NULL,r1=NULL,rd=NULL,rc=NULL,...) 
{# {{{
## wrapper for sim_recurrentII without type-2 events
if (!is.null(death.cumhaz)) death.cumhaz <- list(death.cumhaz)
if (!is.null(r1)) r1 <- as.matrix(r1,ncol=1)
if (!is.null(rd)) rd <- as.matrix(rd,ncol=1)

rr <- sim_recurrent_list(n,list(cumhaz),death.cumhaz=death.cumhaz,rr=r1,rd=rd,rc=rc,...)
return(rr)
}# }}}

##' @export
sim_recurrent_list <- function(n,cumhaz,death.cumhaz=NULL,rr=NULL,rd=NULL,rc=NULL,zzr=NULL,zzd=NULL,
  gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,dependence=0,var.z=1,cor.mat=NULL,cens=NULL,extend=TRUE,...) 
{# {{{
  status <- fdeath <- dtime <- NULL # to avoid R-check 

  if (dependence==0) { # {{{
	  z <- z1 <- z2 <- zd <- rep(1,n) 
	  if (is.null(zzr)) zzr <- matrix(z,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     } else if (dependence==1) {
	  z <- rgamma(n,1/var.z[1])*var.z[1]
	  z1 <- z; z2 <- z; zd <- z
	  if (is.null(zzr)) zzr <- matrix(z,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; zd <- rep(1,n) 
	  if (is.null(zzr)) zzr <- matrix(zz,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     }    else stop("dependence 0,1,4"); # }}}

   if (is.null(rr)) rr <- matrix(1,n,length(cumhaz))
   if (!is.null(death.cumhaz))
   if (is.null(rd)) rd <- matrix(1,n,length(death.cumhaz))
   if (is.null(rc)) rc <- rep(1,n)

  ## extend cumulative for death to full range  of cause 1
  if (!is.null(death.cumhaz)) {
     out <- extendCums(c(cumhaz,death.cumhaz),NULL,extend=extend)
     l <- length(cumhaz)
     ld <- length(death.cumhaz)
     cumhaz <- out[1:l]
     cumhazd <- out[(l+1):(l+ld)]
  } else {
     if (length(cumhaz)>1) {
	     out <- extendCums(cumhaz,NULL,extend=extend)
	     l <- length(cumhaz)
	     cumhaz <- out[1:l]
      }
  }
  max.time <- tail(cumhaz[[1]][,1],1)

  rrz <- rr*zzr
  tall <- rchazl(cumhaz,rrz) 
  tall$id <- 1:n

### death time simulated
  if (!is.null(death.cumhaz)) {# {{{
          rrzd <- rd*zzd
	  timed   <- rchazl(cumhazd,rrzd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }
# }}}


### fixing the first time to event
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  i <- 0; 
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i <= max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
	  tt <- rchazl(cumhaz,rrz[still$id,,drop=FALSE],entry=(1-gap.time)*still$time) 
	  if (i==max.recurrent) {
		  tt$time <- max.time; tt$status <- 0
	  }
	  if (gap.time) {
		  tt$entry <- still$time
		  tt$time  <- tt$time+still$time
		  if (i==max.recurrent) { tt$time <- max.time; tt$status <- 0}
	  }
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  } 

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time
  tall$rr <- NULL

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"rr") <- rr
  attr(tall,"rd") <- rd
  attr(tall,"z") <- zzr
  attr(tall,"zd") <- zzd

  return(tall)
  }# }}}

showfitsimList <- function(rr,cumhaz,dr,...)  {# {{{
 colp <- max(length(cumhaz),length(dr))
 par(mfrow=c(2,colp))
 if (!is.null(cumhaz)) {
 for (i in 1:length(cumhaz)) {
	 pp <- phreg(Surv(entry,time,status==i)~+1,rr)
	 plot(pp,main=paste("status=",i),...); 
	 lines(cumhaz[[i]],col=2,lwd=2)
 }
 }
 if (!is.null(dr)) {
 for (i in 1:length(dr)) {
	 pp <- phreg(Surv(entry,time,death==i)~+1,rr)
	 plot(pp,main=paste("death=",i),...); 
	 lines(dr[[i]],col=2,lwd=2)
 }
 }
}
# }}}

showfitsim <- function(causes=2,rr,dr,base1,base4,which=1:3) 
{# {{{
if (1 %in% which) {
  drr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
  basehazplot.phreg(drr,ylim=c(0,8))
  lines(dr,col=2)
}
###
if (2 %in% which) {
  xrr <- phreg(Surv(entry,time,status==1)~cluster(id),data=rr)
  basehazplot.phreg(xrr,add=TRUE)
###  basehazplot.phreg(xrr)
  lines(base1,col=2)
  if (causes>=2) {
	  xrr2 <- phreg(Surv(entry,time,status==2)~cluster(id),data=rr)
	  basehazplot.phreg(xrr2,add=TRUE)
	  lines(base4,col=2)
  }
  }
if (3 %in% which) {
  meanr1 <-   recurrentMarginalPhreg(xrr,drr)
  basehazplot.phreg(meanr1,se=TRUE)
  if (causes>=2) {
	  meanr2 <-   recurrentMarginalPhreg(xrr2,drr)
	  basehazplot.phreg(meanr2,se=TRUE,add=TRUE,col=2)
  }
}
}# }}}

##' Simulate recurrent events from a two-stage Cox or Ghosh-Lin model
##'
##' Simulates recurrent event data from a fitted two-stage model, where the
##' recurrent event process and the terminal event are each described by a
##' separate fitted model. The recurrent event model may be either a Cox
##' proportional hazards model (\code{phreg}) or a Ghosh-Lin marginal rate model
##' (\code{recreg}); the terminal event model must be a Cox model (\code{phreg}).
##'
##' Covariates are drawn by bootstrap from \code{data} (if supplied), and
##' subject-specific relative risks and strata are derived from the fitted
##' model objects. Stratified baselines are fully supported. When
##' \code{dependence} is \code{NULL} (default), the simulation uses the
##' two-stage structure from \code{\link{sim_GLcox}}; setting \code{dependence}
##' to an integer falls back to \code{\link{sim_recurrent_list}} with the
##' corresponding frailty model.
##'
##' @param cox1 A fitted \code{phreg} object for the recurrent event rate, or a
##'   fitted \code{recreg} (Ghosh-Lin) object. The model type is detected
##'   automatically from the class.
##' @param coxd A fitted \code{phreg} object for the terminal event. May be
##'   \code{NULL} if no terminal event is modelled.
##' @param n Number of subjects to simulate. Default is \code{1}.
##' @param data The data frame on which \code{cox1} and \code{coxd} were fitted,
##'   used to draw covariate values for the simulated subjects. If \code{NULL},
##'   covariates must be supplied via \code{r1}, \code{rd}, \code{strata1}, and
##'   \code{stratad}.
##' @param type Simulation type: \code{"default"} (auto-detected from class of
##'   \code{cox1}), \code{"cox-cox"}, or \code{"gl-cox"}.
##' @param id Name of the subject identifier variable in \code{data}. Default is
##'   \code{"id"}.
##' @param varz Variance of the frailty distribution in the two-stage model.
##'   Default is \code{1}.
##' @param share Proportion of the shared frailty assigned to the recurrent event
##'   process in the partial-sharing model. Default is \code{1}.
##' @param cens Rate of exponential censoring. Default is \code{0.001}.
##' @param scale1 Scalar multiplier for the baseline cumulative hazard of the
##'   recurrent event process. Default is \code{1}.
##' @param scaled Scalar multiplier for the baseline cumulative hazard of the
##'   terminal event. Default is \code{1}.
##' @param dependence If non-\code{NULL}, falls back to
##'   \code{\link{sim_recurrent_list}} with this frailty structure (see
##'   \code{\link{sim_recurrentII}} for valid values). Default is \code{NULL}.
##' @param r1 Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the recurrent event, used when \code{data = NULL}.
##' @param rd Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the terminal event, used when \code{data = NULL}.
##' @param rc Optional numeric vector of length \code{n} of subject-specific
##'   censoring rate multipliers.
##' @param strata1 Optional integer vector of length \code{n} specifying the
##'   stratum index (0-based) for the recurrent event model, used when
##'   \code{data = NULL}.
##' @param stratad Optional integer vector of length \code{n} specifying the
##'   stratum index (0-based) for the terminal event model, used when
##'   \code{data = NULL}.
##' @param death.code Integer status code used for the terminal event in the
##'   output \code{status} column. Default is \code{3}.
##' @param ... Further arguments passed to \code{\link{sim_GLcox}}, including
##'   \code{nmin} and \code{nmax} for the linear approximation grid.
##'
##' @return A data frame in counting-process format with one row per event
##'   interval per subject. Column names match those in the original model
##'   formula (entry, exit, and status variables). Additional columns include
##'   \code{id} and the covariates drawn from \code{data} (if supplied). The
##'   terminal event is coded as \code{death.code} in the status variable;
##'   recurrent events are coded as \code{1}.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{sim_recurrent_list}},
##'   \code{\link{sim_GLcox}}
##'
##' @references
##' Scheike, T. H. (2026). Two-stage recurrent events random effects models.
##' \emph{Lifetime Data Analysis}.
##'
##' @examples
##' data(hfactioncpx12)
##' hf    <- hfactioncpx12
##' hf$x  <- as.numeric(hf$treatment)
##' n     <- 100
##'
##' ## Cox-Cox two-stage model
##' xr <- phreg(Surv(entry, time, status == 1) ~ x + cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ x + cluster(id), data = hf)
##' simcoxcox <- sim_recurrent_ts(xr, dr, n = n, data = hf, death.code = 2)
##'
##' ## Ghosh-Lin/Cox two-stage model
##' recGL  <- recreg(Event(entry, time, status) ~ x + cluster(id), hf, death.code = 2)
##' simglcox <- sim_recurrent_ts(recGL, dr, n = n, data = hf, death.code = 2)
##'
#' @export sim_recurrent_ts
sim_recurrent_ts <- function(cox1,coxd=NULL,
             n=1, data=NULL,type=c("default","cox-cox","gl-cox"),
             id="id",varz=1,share=1,cens=0.001,
      scale1=1,scaled=1,dependence=NULL,
 r1=NULL,rd=NULL,rc=NULL,strata1=NULL,stratad=NULL,death.code=3,
                          ...) {# {{{
## exp censoring default
statusD <- death <- NULL

if (type[1]=="default" & inherits(cox1,"recreg")) type <- "gl-cox" 
if (type[1]=="default" & inherits(cox1,"phreg")) type <- "cox-cox" 
if (type[1]=="cox-cox") type <- 3 else type <- 2

if (!is.null(data)) {
   coxs <- list(cox1,coxd)
   rrdata <- draw_phregs(coxs,n,data=data)
   rr1 <- rrdata$rr[,1]
   rstrata1 <- rrdata$strata[,1]

   if (!is.null(coxd)) {
      rrd <- rrdata$rr[,2]
      rstratad <- rrdata$strata[,2]
   }
} else { 
	data <- c()
	rr1 <- rep(1,n)
	rrd <- rep(1,n)
	rstrata1 <- rep(0,n)
	rstratad <- rep(0,n)
}

if (is.null(rc)) rc <- rep(1,n)

if (!is.null(coxd))    {
    coxd$cumhaz <- scalecumhaz(coxd$cumhaz,scaled); 
    LamD <- basecumhaz(coxd,only=1)
} else LamD <- NULL
cox1$cumhaz <-   scalecumhaz(cox1$cumhaz,scale1); 
Lam1 <- basecumhaz(cox1,only=1)

### when data not given then use r1,rd, strata1,stratad
if (is.null(r1)) r1 <- rr1
if (is.null(rd)) rd <- rrd
if (is.null(strata1)) strata1 <- rstrata1
if (is.null(stratad)) stratad <- rstratad


if (is.null(dependence) & (!is.null(LamD))) {
 ## go through combined strata for the two models 
strat1d <- mystrata(data.frame(strata1,stratad))
rrs <- c()
for (j in 1:attr(strat1d,"nlevel")) {
     r1i <- which(strat1d==j)
     strata1ss <- strata1[r1i[1]]+1
     stratadss <- stratad[r1i[1]]+1
     Lam1s <- Lam1[[strata1ss]]
     LamDs <- LamD[[stratadss]]
     ns <- length(r1i)
     rrss <- sim_GLcox(ns,Lam1s,LamDs,var.z=varz,r1=r1[r1i],rd=rd[r1i],rc=rc[r1i],
		model="twostage",cens=cens,type=type,share=share,...)
    rrss$ids <- rrss$id
    rrss$id <- r1i[rrss$id+1]
    rrs <- rbind(rrs,rrss)
}			  
rrs <- dtransform(rrs,statusD=death.code,statusD==3)
} else { 
if (is.null(dependence)) dependence <- 0
if (!is.null(LamD)) 
rrs <- sim_recurrent_list(n,Lam1,death.cumhaz=LamD,rr=matrix(r1,ncol=1),rd=matrix(rd,ncol=1),rc=rc,cens=cens,var.z=varz,dependence=dependence)
else rrs <- sim_recurrent_list(n,list(Lam1),rr=matrix(r1,ncol=1),rc=rc,cens=cens,var.z=varz,dependence=dependence)
rrs$Z <- attr(rrs,"z")[rrs$id+1]
rrs$statusD <- rrs$status
if (!is.null(LamD))  {
   rrs <- dtransform(rrs,statusD=death.code,death==1)
}
}

## add correct names to entry,time,status if possible
if (inherits(cox1,"phreg")) {
   varsY <- all.vars(update(drop.specials(cox1$formula,"cluster"),.~1)) 
   rrs[,varsY] <- cbind(rrs$start,rrs$stop,rrs$statusD)
   rrs <- dkeep(rrs,c(varsY,"id"))
}

## add covariates, 
if (!is.null(data)) rrs <- cbind(rrs,rrdata$data[rrs$id,])

return(rrs)
}
# }}}

sim_RecurrentIIHist <- function(n,cumhaz,death.cumhaz,cens=NULL,rr=NULL,rc=NULL,rd=NULL,
	    max.recurrent=100,dependence=0,var.z=0.22,cor.mat=NULL,
	    HistN1=~I(Nt^.5),HistD=~I(Nt^.5),HistN1.beta=c(1.0),HistD.beta=c(1.0),...) 
{# {{{

  ctime <- fdeath <- dtime <- NULL # to avoid R-check 
  status <- dhaz <- NULL; dhaz2 <- NULL

  if (dependence==0) { z <- z1 <- zc <- zd  <-  rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
###	      z <- exp(rnorm(n,1)*var.z[1]^.5)
	      z1 <- z; z2 <- z; zd <- z
	      if (!is.null(cor.mat)) { zd <- rep(1,n); }
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
###	      print(summary(z))
###	      print(cor(z))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
###	      print(summary(z))
###	      print(cor(z))
      } else stop("dependence 0-3"); # }}}

  ## covariate adjustment 
  if (is.null(rr))  rr <- z1; 
  if (is.null(rc))  rc <- zc; 
  if (is.null(rd))  rd <- zd; 
  if (length(rr)!=n) rr <- rep(rr[1],n)
  if (length(rc)!=n) rc <- rep(rc[1],n)
  if (length(rd)!=n) rd <- rep(rd[1],n)

  ll <- nrow(cumhaz)
  ### extend of cumulatives
  cumhaz <- rbind(c(0,0),cumhaz)
  death.cumhaz <- rbind(c(0,0),death.cumhaz)

  if (!is.null(cens)) {
	  if (is.matrix(cens))  {
             out <- extendCums(list(cumhaz,death.cumhaz,cens),NULL)
   	     cumcens <- out$cum3
	  } else {
             out <- extendCums(list(cumhaz,death.cumhaz),NULL)
	  }
  } else {
     out <- extendCums(list(cumhaz,death.cumhaz),NULL)
  }
  cumhaz <- out$cum1
  cumhazd <- out$cum2
  max.time <- tail(cumhaz[,1],1)

  ### draw censorting times 
  if (!is.null(cens)) { 
	  if (is.matrix(cens)) sdata <- rchaz(cens,rc) else 
          sdata <- data.frame(entry=0,time=pmin(max.time,rexp(n)/(c(rc)*cens)),
			      status=0,id=1:n)
  } else sdata <- data.frame(entry=0,time=rep(max.time,n),status=0,id=1:n)
  sdata$ctime <- sdata$time
  sdata$Nt <- 0

  i <- 0; 
  tall <- c()
  still <- tt1 <-  sdata
  ## start at 0
  tt1$time <- still$time <- 0
  while (any((still$time<still$ctime) &  (i < max.recurrent))) { ## {{{
	  still$Nt <- i
	  nn <- nrow(still)
	  z1r <- rr[still$id]
	  zdr <- rd[still$id]

	  m1 <- model.matrix(HistN1,still)[,-1,drop=FALSE]
	  md <- model.matrix(HistD,still)[,-1,drop=FALSE]
	  r1h <- c(exp(m1 %*% HistN1.beta))
	  rdh <- c(exp(md %*% HistN1.beta))

          tt1 <- rcrisk(cumhaz,cumhazd,z1r*r1h,zdr*rdh,entry=still$time)
	  tt1 <- cbind(tt1,dkeep(still,~id+ctime),row.names=NULL)
	  tt1 <- dtransform(tt1,status=0,time>ctime)
	  tt1 <- dtransform(tt1,time=ctime,time>ctime)
	  tt1$Nt <- i
          ## remove dead from tt1 to get those still there 
          deadid <- (tt1$status %in% c(0,2))
	  ### those that are still under risk 
	  still <- tt1[!deadid,,drop=FALSE]
	  ## also keep only those before max.time
          still <- subset(still,still$time<max.time)
	  tall <- rbind(tall,tt1,row.names=NULL)
	  i <- i+1
  }  # }}}

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time
  tall <- dkeep(tall,~id+entry+time+status+ctime+Nt+start+stop)

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cens.cumhaz") <- cens

  return(tall)
  }# }}}

##' Simulate recurrent events from a two-stage model with structured gamma frailties
##'
##' Simulates recurrent event data with two event types and a terminal event,
##' using a parametric two-stage frailty model. The construction ensures that the
##' marginal rates are approximately correct: conditional on survival,
##' \eqn{E(dN_j \mid D > t) \approx} \code{cumhazj}, and the hazard of death
##' equals \code{death.cumhaz}.
##'
##' The frailty structure uses three gamma random variables \eqn{Z_{d1}},
##' \eqn{Z_{d2}}, \eqn{Z_{12}} to induce dependence:
##' \deqn{Z_\text{death} = Z_{d1} + Z_{d2}, \quad
##'       Z_1 = Z_{d1}^{\nu_1} Z_{12}, \quad
##'       Z_2 = Z_{d2}^{\nu_2} Z_{12}^{\nu_3}.}
##' The parameters \code{share1} and \code{vargamD} control how the death frailty
##' splits between the two components, and \code{vargam12} controls the shared
##' recurrent-event frailty. Setting \eqn{\nu = (1,1,1)} with \code{share1 = 0.5}
##' gives a symmetric structure; varying \eqn{\nu} allows asymmetric dependence.
##'
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the target
##'   marginal cumulative rate of the first recurrent event type.
##' @param cumhaz2 Two-column matrix \code{(time, cumhaz)} giving the target
##'   marginal cumulative rate of the second recurrent event type.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event.
##' @param nu Numeric vector of length 3: the powers \eqn{(\nu_1, \nu_2, \nu_3)}
##'   applied to the frailty components (see Details). Must satisfy
##'   \eqn{\nu_j > -1/\text{shape}}. Default is \code{rep(1, 3)}.
##' @param share1 Proportion of the total death frailty variance assigned to the
##'   first component \eqn{Z_{d1}}. The remainder goes to \eqn{Z_{d2}}. Must be
##'   in \eqn{(0, 1)}. Default is \code{0.3}.
##' @param vargamD Total variance of the death frailty \eqn{Z_\text{death}}.
##'   Default is \code{2}.
##' @param vargam12 Variance of the shared recurrent-event frailty \eqn{Z_{12}}.
##'   Default is \code{0.5}.
##' @param gap.time Logical. If \code{TRUE}, event times are drawn as gap times
##'   rather than calendar times. Default is \code{FALSE}.
##' @param max.recurrent Maximum number of recurrent events per subject. Default
##'   is \code{100}.
##' @param cens Rate of exponential censoring. If \code{NULL} (default), no
##'   administrative censoring is applied.
##' @param ... Further arguments passed to lower-level simulation functions.
##'
##' @return A data frame in counting-process format (one row per event interval
##'   per subject) with columns \code{id}, \code{start}, \code{stop},
##'   \code{entry}, \code{time}, \code{status}, and \code{death}. Attributes
##'   store the (possibly adjusted) cumulative hazards used in simulation and
##'   the frailty parameters.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{sim_recurrentII}}, \code{\link{sim_recurrent_ts}}
##'
##' @examples
##' data(CPH_HPN_CRBSI)
##' dr    <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi
##' base4 <- CPH_HPN_CRBSI$mechanical
##'
##' rr <- sim_recurrentTS(1000, base1, base4, death.cumhaz = dr)
##' dtable(rr, ~death + status)
##' mets:::showfitsim(causes = 2, rr, dr, base1, base4)
##'
##' @export
sim_recurrentTS <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,
		    nu=rep(1,3),share1=0.3,vargamD=2,vargam12=0.5,
		    gap.time=FALSE,max.recurrent=100,cens=NULL,...) 
{# {{{

k <- 1
nu1 <- nu[1]; nu2 <- nu[2]; nu3 <- nu[3]
###nu1 <- 1; nu2 <- 1; nu3 <- 0.4
share2 <- (1-share1)
vargam <- vargamD
vargam12 <- 0.5
agam1 <- share1/vargam
agam2 <- share2/vargam
betagam=1/vargam
gamma1  <- rep(rgamma(n,agam1)*vargam,each=k)
gamma2  <- rep(rgamma(n,agam2)*vargam,each=k)
agam12 <- 1/vargam12
betagam12 <- 1/vargam12
gamma12 <- rep(rgamma(n,agam12)*vargam12,each=k)
agamD <- agam1+agam2
z1 <- (gamma1^nu1)*gamma12
z2 <- (gamma2^nu2)*gamma12^nu3
gamD <- gamma1+gamma2
zd <- gamD
egamma12nu3 <- (gamma(agam12+nu3)/gamma(agam12))*1/(betagam12)^nu3
zs <- cbind(z1,z2,zd)

 status <- fdeath <- dtime <- NULL # to avoid R-check 
 dhaz <- haz2 <- dhaz <- NULL

 ll <- nrow(cumhaz)
 max.time <- tail(cumhaz[,1],1)

 ################################################################
 ### approximate hazards to make marginals fit (approximately)
 ################################################################
 ## step-size set to one and range to that of base1
 base1 <- predictCumhaz(rbind(0,as.matrix(cumhaz)),1:round(tail(cumhaz[,1],1)) )
 if (!is.null(death.cumhaz)) 
     death.cumhaz <- predictCumhaz(rbind(0,as.matrix(death.cumhaz)),base1[,1])

 orig.death <- death.cumhaz
 dbase1 <- death.cumhaz
 gt <- exp(vargam*dbase1[,2]) 
 dtt <- diff(c(0,dbase1[,1]))
 lams <- (diff(c(0,dbase1[,2]))/dtt)*gt
 death.cumhaz <- cbind(dbase1[,1],cumsum(dtt*lams))

 dbase1 <- cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2]
 dtt <- diff(c(0,base1[,1]))
 gt <- (gamma(agam1+nu1)/gamma(agam1))*(1/(betagam+dbase1))^nu1
 lams <- (diff(c(0,base1[,2]))/dtt)*(1/gt)
 cumhaz <- cbind(base1[,1],cumsum(dtt*lams))

 base1 <- cumhaz2
 dbase1 <- cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2]
 dtt <- diff(c(0,base1[,1]))
 gt <- (gamma(agam2+nu2)/gamma(agam2))*(1/(betagam+dbase1))^nu2
 lams <-(1/egamma12nu3)*(diff(c(0,base1[,2]))/dtt)*(1/gt)
 cumhaz2 <- cbind(base1[,1],cumsum(dtt*lams))

 cumhaz <- rbind(c(0,0),cumhaz)
 cumhaz2 <- rbind(c(0,0),cumhaz2)
 death.cumhaz <- rbind(c(0,0),death.cumhaz)

## range max of cumhaz and cumhaz2 
  out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz),NULL)
  cumhaz <- out$cum1
  cumhaz2 <- out$cum2
  cumhazd <- out$cum3
  max.time <- tail(cumhaz[,1],1)

### recurrent first time
  tall1 <- rchaz(cumhaz,rr=z1)
  tall2 <- rchaz(cumhaz2,rr=z2)
  tall <- tall1 
  tall$status <- ifelse(tall1$time<tall2$time,tall1$status,2*tall2$status)
  tall$time <- ifelse(tall1$time<tall2$time,tall1$time,tall2$time)
  tall$id <- 1:n
  tall$rr2 <- tall2$rr
### death time simulated
  if (!is.null(death.cumhaz)) {# {{{
	  timed   <- rchaz(cumhazd,rr=zd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/cens
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/cens
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }# }}}

  ### fixing the first time to event
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  ### setting aside memory 
  tt1 <- tt2 <- tt
  i <- 1; 
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i < max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
          tt1 <- rchaz(cumhaz,rr=z1[still$id],entry=still$time)
          tt2 <- rchaz(cumhaz2,rr=z2[still$id],entry=still$time)
	  tt <- tt1
          tt$status <- ifelse(tt1$time<=tt2$time,tt1$status,2*tt2$status)
          tt$time <-   ifelse(tt1$time<=tt2$time,tt1$time,tt2$time)
	  tt$rr2 <- tt2$rr
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  }
  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"zs") <- zs

  attr(tall,"gamma.death") <- c(agam1,agam2,betagam,vargamD)
  attr(tall,"gamma.N12") <-   c(agam12,betagam12,vargam12)

  return(tall)
  }# }}}

##' Compute cumulative event counts as time-dependent covariates
##'
##' For each subject and each row of \code{data}, counts the number of prior
##' events of specified types in the recurrent event history. The resulting
##' count columns can be used as time-dependent covariates in subsequent models,
##' e.g. to capture event-history dependence in the recurrent event rate.
##'
##' When \code{lag = TRUE} (default), the count at each row reflects events that
##' occurred strictly before the current time point (i.e. \eqn{N(t-)}), making
##' it suitable as a left-continuous covariate in counting-process models.
##'
##' @param data A data frame in counting-process format, with one row per event
##'   interval per subject.
##' @param status Name of the column containing event status codes. Default is
##'   \code{"status"}.
##' @param id Name of the column containing subject identifiers. Default is
##'   \code{"id"}.
##' @param types Integer vector of status codes to count. Each value in
##'   \code{types} generates one new count column (when \code{multitype = FALSE})
##'   or contributes to a single combined count (when \code{multitype = TRUE}).
##'   Default is \code{1}.
##' @param names.count Prefix for the names of the new count columns. The
##'   status code is appended, e.g. \code{"Count1"}, \code{"Count2"}.
##'   Default is \code{"Count"}.
##' @param lag Logical. If \code{TRUE} (default), the count at each row is the
##'   number of events strictly before the current time (\eqn{N(t-)}). If
##'   \code{FALSE}, events at the current time are included (\eqn{N(t)}).
##' @param multitype Logical. If \code{TRUE}, events with status in \code{types}
##'   are aggregated into a single count column, optionally weighted by
##'   \code{marks}. If \code{FALSE} (default), a separate count column is created
##'   for each value in \code{types}.
##' @param marks Optional numeric vector of weights applied to events when
##'   \code{multitype = TRUE}. If \code{NULL} (default), each event has weight 1.
##'
##' @return The input data frame \code{data} with one or more new integer columns
##'   appended. With \code{multitype = FALSE}, columns are named
##'   \code{paste0(names.count, k)} for each \code{k} in \code{types}; with
##'   \code{multitype = TRUE}, a single column named
##'   \code{paste0(names.count, types[1])} is added. An internal bookkeeping
##'   column \code{lbnr__id} is also added.
##'
##' @author Thomas Scheike
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' dtable(hf, ~status)
##'
##' ## Separate counts for event types 1 and 2
##' rr <- count_history(hf, types = 1:2, id = "id", status = "status")
##' dtable(rr, ~"Count*" + status, level = 1)
##'
##' @export
count_history <- function(data,status="status",id="id",types=1,names.count="Count",lag=TRUE,multitype=FALSE,marks=NULL)
{# {{{
stat <- data[,status]

## also allowing marks  when multitype=TRUE
if (multitype & is.null(marks))  marks <- rep(1,nrow(data))

clusters <- data[,id]
if (is.numeric(clusters)) {
      clusters <- fast.approx(unique(clusters), clusters) - 1
      max.clust <- max(clusters)
}
else {
     max.clust <- length(unique(clusters))
     clusters <- as.integer(factor(clusters, labels = seq(max.clust))) - 1
}

data[,"lbnr__id"] <- cumsumstrata(rep(1,nrow(data)),clusters,max.clust+1) 
if (!multitype) {
for (i in types)  {
if (lag==TRUE)
data[,paste(names.count,i,sep="")] <- 
   cumsumidstratasum((stat==i),rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,paste(names.count,i,sep="")] <- 
   cumsumidstratasum((stat==i),rep(0,nrow(data)),1,clusters,max.clust+1)$sum 
}
} else {
if (lag==TRUE)
data[,paste(names.count,types[1],sep="")] <- 
   cumsumidstratasum((stat %in% types)*marks,rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,paste(names.count,types[1],sep="")] <- 
   cumsumidstratasum((stat %in% types)*marks,rep(0,nrow(data)),1,clusters,max.clust+1)$sum 
}

return(data)
}# }}}

count_historyVar <- function(data,var="status",id="id",names.count="Count",lag=TRUE)
{# {{{
vvar <- data[,var]

clusters <- data[,id]
if (is.numeric(clusters)) {
      clusters <- fast.approx(unique(clusters), clusters) - 1
      max.clust <- max(clusters)
}
else {
     max.clust <- length(unique(clusters))
     clusters <- as.integer(factor(clusters, labels = seq(max.clust))) - 1
}

data[,"lbnr__id"] <- cumsumstrata(rep(1,nrow(data)),clusters,max.clust+1) 
if (lag==TRUE)
data[,names.count] <- 
   cumsumidstratasum(vvar,rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,names.count] <- 
   cumsumidstratasum(vvar,rep(0,nrow(data)),1,clusters,max.clust+1)$sum 

return(data)
}# }}}

##' Estimate the probability of exceeding k recurrent events by time t
##'
##' Estimates \eqn{P(N(t) \geq k)} as a function of time \eqn{t}, for a range of
##' thresholds \eqn{k}, in the presence of a terminal event (death). The estimator
##' is based on the cumulative incidence of "reaching \eqn{k} events", treating
##' death as a competing risk. Confidence intervals are computed on the log or
##' plain scale.
##'
##' For each threshold \eqn{k} in \code{exceed}, the function identifies the
##' first time each subject reaches \eqn{k} events, then fits a competing risks
##' model (\code{\link{cif}}) with "reaching \eqn{k} events" as the event of
##' interest and death as the competing event. Strata are supported. When
##' \code{marks} is \code{NULL}, each event contributes equally; otherwise events
##' are weighted by their mark values before cumulative counts are formed.
##'
##' @param formula A formula with an \code{\link{Event}} response giving the
##'   exit time and status (and optionally entry time). The right-hand side may
##'   include \code{cluster()} and \code{strata()}.
##' @param data A data frame containing all variables in \code{formula}.
##' @param cause Integer code for the recurrent event of interest. Default is
##'   \code{1}.
##' @param death.code Integer code for the terminal event. Default is \code{2}.
##' @param cens.code Integer code for censoring. Default is \code{0}.
##' @param exceed Integer vector of thresholds \eqn{k} to evaluate. If
##'   \code{NULL} (default), all observed cumulative counts are used.
##' @param marks Optional numeric vector of event weights. If non-\code{NULL},
##'   cumulative counts are formed as weighted sums of events rather than simple
##'   counts. Must have the same length as \code{nrow(data)}.
##' @param all.cifs Logical. If \code{TRUE}, the fitted \code{cif} object for
##'   each threshold is returned in \code{cif.exceed}. Default is \code{FALSE}.
##' @param return.data Logical. If \code{TRUE}, the constructed dataset for each
##'   threshold is returned in \code{dataList}. Default is \code{FALSE}.
##' @param conf.type Type of confidence interval transformation: \code{"log"}
##'   (default) or \code{"plain"}.
##' @param level Confidence level. Default is \code{0.95}.
##' @param ... Further arguments passed to \code{\link{cif}}.
##'
##' @return An object of class \code{"exceed"} with the following components:
##'   \item{time}{Vector of evaluation time points.}
##'   \item{prob}{Array of dimension \code{(length(time), length(exceed) + 1,
##'     nstrata)} containing \eqn{P(N(t) \geq k)} for each threshold and stratum.
##'     The first column gives \eqn{P(N(t) < \text{exceed}[1])}.}
##'   \item{se.prob}{Standard errors of \code{prob}.}
##'   \item{lower, upper}{Pointwise confidence interval bounds.}
##'   \item{meanN}{Estimated mean number of events \eqn{E(N(t))} (single stratum
##'     only; \code{NULL} for stratified analyses).}
##'   \item{meanN2, varN}{Second moment and variance of \eqn{N(t)} (single
##'     stratum only).}
##'   \item{exceed}{Thresholds evaluated (excluding zero).}
##'   \item{cif.exceed}{List of fitted \code{cif} objects (if
##'     \code{all.cifs = TRUE}).}
##'   \item{dataList}{List of datasets for each threshold (if
##'     \code{return.data = TRUE}).}
##'   \item{nstrata, strata.levels, strata.name}{Stratification information.}
##'   Use \code{plot()} and \code{summary()} methods for visualisation and
##'   tabulation.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{cif}}
##'
##' @references
##' Scheike, T. H., Eriksson, L., and Tribler, P. (2019). The mean, variance and
##' correlation for bivariate recurrent events with a terminal event.
##' \emph{Journal of the Royal Statistical Society, Series C}, 68(5).
##'
##' @examples
##' data(hfactioncpx12)
##' dtable(hfactioncpx12, ~status)
##'
##' oo <- prob_exceed_recurrent(Event(entry, time, status) ~ cluster(id),
##'                              hfactioncpx12, cause = 1, death.code = 2)
##' plot(oo)
##' summary(oo, times = c(1, 2, 5))
##' @export
prob_exceed_recurrent <- function(formula,data,cause=1,death.code=2,cens.code=0, exceed=NULL,marks=NULL,all.cifs=FALSE,
				  return.data=FALSE,conf.type=c("log","plain"),level=0.95,...)
{# {{{
    cifmets <- TRUE
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("marks","strata","offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    strata <- des$strata
    if (!is.null(strata))  {
      ns <- grep("strata",names(des$levels))
      strata.name  <-  names(des$levels)[ns]
    } else strata.name <- NULL
    ## no use of 
    pos.cluster <- pos.strata <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  marks.call <- marks
  if (is.null(marks)) marks <- rep(1,length(exit))
  ## }}}
  if (!is.null(strata)) {
	  nstrata <- nlevels(strata) 
          strata.levels <- levels(strata)
  } else { strata.levels <- NULL; nstrata <- 1}

 call.id <- id;
 conid <- construct_id(id,nrow(X),as.data=TRUE)
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 data$id__  <- id

 times <- NULL
 statusD <- (status %in% death.code)*1
 statusE <- (status %in% cause)*1
 if (sum(statusE)==0) stop("none of type events")
 if (!is.null(strata) & !cifmets) stop("strata only for cifmets=TRUE\n")
 allvars <- all.vars(formula)
 if (is.null(times)) times <- sort(unique(exit[statusE==1]))

 countE <- cumsumstrata(statusE*marks,id,nid)
 if (is.null(exceed)) exceed <- sort(unique(countE))
 w0 <- which(exceed==0)
 if (length(w0)>=1) exceed <- exceed[-w0]

 if (is.null(marks.call)) {
    mc <- max(countE)+1
    idcount <- id*mc+countE
    idcount <- cumsumstrata(rep(1,length(idcount)),idcount,mc*(nid+1))
 } else {
    ex <- outer(countE,exceed,">=")[,1,]
    exC <- apply(ex,2,cumsumstrata,id,nid)
    idcount <- exC
 }

if (!cifmets) {
  ## can not handle start stop and id, so uses MG standard errors 
  pp <- as.formula(paste("Hist(entry=",allvars[1],",",allvars[2],",statN)~+1",sep=""))
  ###rhs <- update(formula,-1~.)
  ###form <- as.formula(update.formula(rhs,pp))
  form <- as.formula(pp)
} else {
###  pp <- as.formula(paste("Event(",allvars[1],",",allvars[2],",statN)~.",sep=""))
  pp <- as.formula(paste("Event(",allvars[2],",statN)~.",sep=""))
  rhs <- update(formula,-1~.)
  form <- as.formula(update.formula(rhs,pp))
}

cif.exceed <- NULL
dataList <- NULL
if (all.cifs) cif.exceed <- list() 
if (return.data) dataList <- list() 
se.probs <- probs <- lower <- upper <- array(0,c(length(times),length(exceed)+1,nstrata))
i <- 1
for (n1 in exceed) {# {{{
	i <- i+1
	### first time that get to n1
        if (is.null(marks.call)) keep <- (countE<n1 ) | (countE==n1 & idcount==1) else keep <- exC[,i-1]<=1 
###	keep <- (countE<n1 ) | (firsttoE==n1)

	### status, censoring, get to n1, or die
        statN <- rep(0,nrow(data))
        if (is.null(marks.call)) statN[countE==n1] <- 1 else statN[exC[,i-1]==1] <- 1
	statN[statusD==1] <- 2
	statN <- statN[keep]
	dataN <- data[keep,]
	dataN$statN <- statN

        idN <- revcumsumstrata(rep(1,nrow(dataN)),dataN$id__,nid)
        dataN <- subset(dataN,idN==1)
	if (return.data) dataList[[i-1]] <- dataN

   pN1 <-  suppressWarnings(cif(form,data=dataN))
   if (all.cifs) cif.exceed[[i-1]] <- pN1

###   lpN1 <- basecumhaz(pN1)
   where <- predictCumhaz(c(0,pN1$times),times)
###  where <- fast.approx(c(0,pN1$times),times,type="left")
###   print(c(length(pN1$mu),length(pN1$strata),pN1$nstrata))
   if (length(pN1$mu)>=1) {
   cifs <- vecAllStrata(pN1$mu,pN1$strata,pN1$nstrata)
   se.cifs <- vecAllStrata(pN1$se.mu,pN1$strata,pN1$nstrata)
   probs[,i,] <- rbind(0,cifs)[where,]
   se.probs[,i,] <- rbind(0,se.cifs)[where,]

   lu <- conftype(probs[,i,],se.probs[,i,],conf.type=conf.type[1],
         	 restrict=c("prob"),conf.int=level)
   lower[,i,] <- lu$lower
   upper[,i,] <- lu$upper
   } else {
   probs[,i,] <- se.probs[,i,] <- lower[,i,] <- upper[,i,] <- 0
   }

   ## surviving first level
   if (i==2) { probs[,1,]  <- 1-probs[,2,]; 
	    se.probs[,1,] <- se.probs[,2,]; 
	    lower[,1,] <- 1-lower[,2,]; 
	    upper[,1,] <- 1-upper[,2,]; 
   }

}# }}}

if (is.null(marks.call) & nstrata==1) {
dp <- -t(apply(cbind(probs[,-1,1],0),1,diff))
meanN <- apply(probs[,-1,1,drop=FALSE],1,sum)
meanN2 <- apply(t(exceed^2 * t(dp)),1,sum)
varN <- meanN2-meanN^2
} else dp <- meanN <- meanN2 <- varN <- NULL
 
colnames(probs) <- c(paste("N<",exceed[1],sep=""),paste("exceed>=",exceed,sep=""))
colnames(se.probs) <- c(paste("N<",exceed[1],sep=""),paste("exceed>=",exceed,sep=""))

out <- list(time=times,times=times,prob=probs,se.prob=se.probs,meanN=meanN,
	  lower=lower,upper=upper,meanN2=meanN2,varN=varN,exceed=exceed[-1],formula=form,
 cif.exceed=cif.exceed,dataList=dataList,
 nstrata=nstrata,strata.levels=strata.levels,strata.name=strata.name)
class(out) <- "exceed"
return(out)
}# }}}

##' @export
plot.exceed <- function(x,types=NULL,se=1,where=0.6,legend=NULL,strata=NULL,...) { ## {{{ 
if (is.null(types)) types <- 1:ncol(x$prob)
if (is.null(strata)) stratas <- seq(x$nstrata) else stratas <- strata
for (j in stratas) {
matplot(x$time,x$prob[,types,j],type="s",xlab="time",ylab="probabilty",xlim=range(c(0,x$times)),...)
if (x$nstrata>1) title(main=paste(x$strata.name,"=",x$strata.levels[j],sep=""))
if (se==1) 
for (i in types) plotConfRegion(x$time, cbind(x$lower[,i,j],x$upper[,i,j]) ,col=i)
if (!is.null(where)) {
if (is.null(legend)) legend <- colnames(x$prob)[types]
legend(0,0.6,legend=legend,lty=types,col=types)
}
}
} ## }}}

##' @export
summary.exceed <- function(object,times=NULL,types=NULL,strata=NULL,...) { ## {{{ 

if (is.null(types)) types <- 1:ncol(object$prob)
if (is.null(strata)) stratas <- seq(object$nstrata) else stratas <- strata
out <- list()

for (j in stratas) {
prob <- cbind(object$time,object$prob[,types,j])
se <- cbind(object$time,object$se.prob[,types,j])
lower <- cbind(object$time,object$lower[,types,j])
upper <- cbind(object$time,object$upper[,types,j])
if (is.null(times)) 
outl <- list(prob=prob,se=se,lower=lower,upper=upper)
else {
   rows <- fast.approx(object$time,times,type="left")
   probt <- cbind(times,prob[rows,])
   set <-   cbind(times,se[rows,])
   lowert <- cbind(times,lower[rows,])
   uppert <- cbind(times,upper[rows,])
   outl <- list(prob=probt,se=set,lower=lowert,upper=uppert)
}
if (length(stratas)>1) out[[j]] <- outl else out <- outl 
}
if (length(stratas)>1) names(out) <- object$strata.levels[stratas]
return(out)
} ## }}}

##' @export
print.exceed  <- function(x,...) summary.exceed(x,...)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"  
)

## -----------------------------------------------------------------------------
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations

## -----------------------------------------------------------------------------
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)

## -----------------------------------------------------------------------------
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim_phreg(cox1,n,data=bmt)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)

 ## modify regression coefficients 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim_phreg(cox10,n,data=bmt)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)

## -----------------------------------------------------------------------------
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)

## -----------------------------------------------------------------------------
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim_phreg(coxs,nsim,data=NULL,strata=strata)

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim_phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)

## ----stratified---------------------------------------------------------------
 ## competing risks with phreg 
 cox0 <- phreg(Surv(time,cause==0)~tcell+platelet,data=bmt)
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
 coxs <- list(cox0,cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)

 ## verify cause-specific hazards match fitted model; increase n for better agreement
 scox0 <- phreg(Surv(time,cause==1)~tcell+platelet,data=dd)
 scox1 <- phreg(Surv(time,cause==2)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==3)~strata(tcell)+platelet,data=dd)
 cbind(cox0$coef,scox0$coef)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,3))
 plot(cox0); plot(scox0,add=TRUE,col=2); 
 plot(cox1); plot(scox1,add=TRUE,col=2); 
 plot(cox2); plot(scox2,add=TRUE,col=2); 
 
 ########################################
 ## second example 
 ########################################

 cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
 coxs <- list(cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE); 
 plot(cox2); plot(scox2,add=TRUE); 

## -----------------------------------------------------------------------------
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim_phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,cause==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)

## ----sim-phregs, label="entry-stratified"-------------------------------------
data(bmt)
cox0 <- phreg(Surv(time, cause == 0) ~ tcell + platelet+age,          data = bmt)
cox1 <- phreg(Surv(time, cause == 1) ~ tcell + platelet+age,          data = bmt)
cox2 <- phreg(Surv(time, cause == 2) ~ strata(tcell) + platelet+age,  data = bmt)

nsim <- 800
entry <- rbinom(nsim, 1, 0.5) * runif(nsim) * 60

dd <- sim_phreg(cox0,nsim, data = bmt,entry=entry)
scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
cbind(cox0$coef, scox0$coef)
par(mfrow = c(2, 2))
plot(cox0); plot(scox0, add = TRUE, col = 2)

dd <- sim_phregs(list(cox0, cox1, cox2), nsim, data = bmt,entry=entry)

scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
scox1 <- phreg(Surv(entry,time, cause == 2) ~ tcell + platelet+age,         data = dd)
scox2 <- phreg(Surv(entry,time, cause == 3) ~ strata(tcell) + platelet+age, data = dd)

cbind(cox0$coef, scox0$coef)
cbind(cox1$coef, scox1$coef)
cbind(cox2$coef, scox2$coef)

plot(cox0); plot(scox0, add = TRUE, col = 2)
plot(cox1); plot(scox1, add = TRUE, col = 2)
plot(cox2); plot(scox2, add = TRUE, col = 2)

## ----weibull1-----------------------------------------------------------------
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)

## ----weibull_sim--------------------------------------------------------------
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)

## -----------------------------------------------------------------------------
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)

## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- sim_multistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)


## -----------------------------------------------------------------------------
library(mets)
nsim <- 400
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
plot(ciff)
lines(attr(dats,"Lam1"))

## -----------------------------------------------------------------------------
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
 cbind(cif2$coef,scif2$coef)
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)

## -----------------------------------------------------------------------------
 data(bmt)
 ## two cloglog 
 cif1 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=1)
 cif2 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=2)

 nsim <- 800
 entry <- rbinom(nsim,1,0.5)*runif(nsim)*60
 dd <- sim_cif(cif1,nsim,data=bmt,entry=entry)

 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)

## -----------------------------------------------------------------------------
 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt,entry=entry)

 ## logistic link; nonparametric Aalen-Johansen with delayed entry
 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)

## -----------------------------------------------------------------------------
rho1 <- 0.3; rho2 <- 4
set.seed(100)
beta=c(0.3,-0.3,-0.3,0.3)
dep=0
rc <- 0.9

n <- nsim
entry <- rbinom(n,1,0.5)*runif(n)*6
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7),entry=entry) 
###
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,ylim=c(0,0.4))
###

## and without delayed entry for comparison
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7)) 
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,add=TRUE)

## true baseline cif, cloglog link 
baset <- attr(data,"Lam1")[,2]
timet <- attr(data,"Lam1")[,1]
F1base <- 1-exp(-baset)
lines(timet,F1base,lwd=3)

## -----------------------------------------------------------------------------
library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 200

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
 estimate(dr)

 simcoxcox <- sim_recurrent_ts(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)


## -----------------------------------------------------------------------------
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
 estimate(dr)

 simglcox <- sim_recurrent_ts(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(entry,time,status)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
 plot(drs)
 plot(dr,add=TRUE)


## -----------------------------------------------------------------------------
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)
hf$age <- rnorm(741)[hf$id]
hf$Z1 <- rbinom(741,1,0.5)[hf$id]

xr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400 
rr <- sim_recurrent_ts(xr,dr,n=n,data=hf)
rxr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=rr)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
estimate(rxr)
estimate(dr)
estimate(rdr)
plot(xr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
###

glr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=hf,death.code=2)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400
rr <- sim_recurrent_ts(glr,dr,n=n,data=hf)
rxr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=rr,death.code=3)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
estimate(rxr)
estimate(dr)
estimate(rdr)
plot(glr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)

## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 400
 rr <- sim_recurrent(n,base1,death.cumhaz=dr)
 ###
 mets:::showfitsim(causes=1,rr,dr,base1,base1,which=1:2)

 rr <- sim_recurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
 mets:::showfitsim(causes=2,rr,dr,base1,base4,which=1:2)

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- sim_recurrent_list(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
 mets:::showfitsimList(rr,cumhaz,drl) 

## -----------------------------------------------------------------------------
sessionInfo()

##' IPTW logistic regression, Inverse Probabibilty of Treatment Weighted binreg 
##'
##' Fits logistic regression model with treatment weights \deqn{ w(A)= \sum_a I(A=a)/P(A=a|X) }, computes
##' standard errors via influence functions that are returned as the IID argument. 
##' Propensity scores are fitted using either logistic regression (glm) or the multinomial model (mlogit) when more
##' than two categories for treatment. The treatment needs to be a factor and is identified on the rhs
##' of the "treat.model". Can handle right censored binreg type estimating equations with IPTW weights. 
##'
##' Also works with cluster argument. 
##'
##' @param formula for binreg 
##' @param data data-frame for estimation 
##' @param treat.model propensity score model (binary or multinomial) 
##' @param weights may be given, and then uses weights*w(A) as the weights
##' @param estpr to estimate propensity scores and get infuence function contribution to uncertainty
##' @param pi0 fixed simple weights 
##' @param ...  arguments for  binreg call
##' @author Thomas Scheike
##' @examples
##' 
##' data(bmt)
##' dfactor(bmt) <- platelet.f~platelet
##' ## logistic modelling of cumulative incidence 
##' gg <- binreg_IPTW(Event(time,cause)~platelet.f+age,bmt,
##' 	       treat.model=platelet.f~age,time=30)
##' summary(gg)
##' head(iid(gg))
##' 
##' ## logistic modelling  
##' gg <- binreg_IPTW(tcell~platelet.f+age,bmt,
##' 	       treat.model=platelet.f~age)
##' summary(gg)
##' @export
binreg_IPTW <- function(formula,data,treat.model=NULL,weights=NULL,estpr=1,pi0=0.5,...) {# {{{
    cl <- match.call()# {{{
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (inherits(Y, c("Event", "Surv"))) {
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(Y)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(Y)) 
  } else weights <- des.weights
# }}}

  data$id__  <-  id

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

fittreat <- function(treat.model,data,id,ntreatvar,nlev) {# {{{
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2

out <- list(iidalpha=iidalpha,pA=pA,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

expit <- function(x) 1/(1+exp(-x))

treat.name <-  all.vars(treat.model)[1]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 

treats <- treats(treatvar)
if (estpr[1]==1) {
   fitt <- fittreat(treat.model,data,id,treats$ntreatvar,treats$nlev)
   iidalpha0 <- fitt$iidalpha
} else {
   ## assumes constant fixed prob over groups
   wPA <- ifelse(treats$ntreatvar==2,pi0[1],1-pi0[1])         
}
wPA <- c(1/fitt$pA)
if (is.null(weights)) ww <- wPA else ww <- weights*wPA
data$ww <- ww

## fit the weighted model and bring the derivatives to bring them along
###glmw <- suppressWarnings(glm(formula,data,weights=ww,family=family,...))
argss <- list(...)

if (!inherits(Y, c("Event", "Surv"))) {
   Yn <- as.numeric(Y)
   data$event__  <- Yn
   data$time__  <-  2
   binreg.formula <-  update.formula(formula,Event(time__,event__)~.)
   cens.code <- max(Y)+1
   argss[["cens.code"]]  <- cens.code
   argss[["time"]] <- 3
   if (!"cause" %in% names(argss)) {
        argss[["cause"]] <- max(Yn)  # largest value of Yn
   } 
} else {
	binreg.formula <- formula
        if (!"cens.code" %in% names(argss)) {
           argss[["cens.code"]]  <- 0   # default value
        } 
}
argss[["weights"]] <- ww
argss[["data"]] <-  data
argss[["formula"]] <- binreg.formula

glmw <- do.call("binreg",argss)
glm.iid <- iid(glmw)
ihess <- glmw$ihessian

check.derivative <- 0
### for checking derivative 
if (check.derivative==1) {
argsstest <- argss
argsstest$no.opt <- TRUE
argsstest$beta <- coef(glmw)
fpar <- glm(treat.model,data,family=binomial)
mm <- as.matrix(model.matrix(treat.model,data))
cpar <- coef(fpar)
X <- model.matrix(formula,data)
ff <- function(par) {
pa <- 	lava::expit(mm %*% par)
www <- 1/ifelse(data$platelet== 1, pa,(1 - pa))
argsstest$weights <- www
pp <- do.call("binreg",argsstest)
gradient  <- pp$gradient
return(gradient)
}
###print(ff(cpar))
###library(numDeriv)
###gf <- jacobian(ff,cpar)
###print(gf)
}

### iid after propensity model 
### computing  derivatives 
if (estpr[1]==1) {
X <- glmw$design$x
res <- c(glmw$Yipcw -predict(glmw,data,se=0))
XY <- X*res 
###XWY <- X*ww*res
###print( apply(XWY,2,sum))
DUa  <-  t(fitt$DPai) %*% XY  
iidpal <- iidalpha0 %*% DUa ## /nid
glmw$naive.var  <- crossprod(glm.iid)
glm.iid <-   glm.iid + iidpal %*% ihess
glmw$DUproppar <- DUa
} 

glmw$naive.iid <- glmw$iid
glmw$iid <- glm.iid
glmw$var  <-  crossprod(glm.iid)
glmw$IPTW <- ww
## add some arguments to comply with summary.binreg
glmw$n <- nrow(data)
glmw$ncluster <- nid
glmw$nevent <- sum(glmw$Yipcw)
class(glmw) <- c("binreg","IPTW")

return(glmw)
}# }}}

##' @export
predictGLM <- function(object,newdata,id=NULL,fun=NULL,link.conf=TRUE,...) {# {{{
    tt <- terms(object)
    if (missing(newdata) || is.null(newdata)) {
        m <- X <- model.matrix(object)
        mmDone <- TRUE
        offset <- object$offset
    } else {
        Terms <- delete.response(tt)
        m <- model.frame(Terms, newdata, xlev = object$xlevels)
        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    }
    offsets <- rep(0,nrow(X))
    if (!is.null(object$offset)) offsets <- m[, grep("offset", names(m))] 

if (!is.null(object$family$linkinv)) linkinv <- object$family$linkinv  else linkinv <- function(x) x
f <- function(p) { pp <- X %*% p+offsets; return(linkinv(pp)); }
fl <- function(p) { pp <- X %*% p+offsets; return(pp); }
if (!is.null(fun))  f <- fun

if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)

if (link.conf) { 
     if (!is.null(id)) resl <- estimate(object,f=fl,id=id,...) else resl <- estimate(object,f=fl,...)
     res <- linkinv(resl$coefmat[,c(1,3,4)]) 
} else {
if (!is.null(id)) res <- estimate(object,f=f,id=id,...) else res <- estimate(object,f=f,...)
res <- res$coefmat
}

return(list(coef=coef,pred=res))
}
# }}}

##' Reporting OR (exp(coef)) from glm with binomial link and glm predictions
##' 
##' Reporting OR from glm with binomial link  and glm predictions
##' 
##' @param object glm output 
##' @param id possible id for cluster corrected standard errors
##' @param fun possible function for non-standard predictions based on object
##' @param ... arguments of estimate of lava for example level=0.95 
##' @author Thomas Scheike
##' @export
##' @examples
##' data(sTRACE)
##' sTRACE$id <- sample(1:100,nrow(sTRACE),replace=TRUE)
##'
##' model <- glm(I(status==9)~sex+factor(diabetes)+age,data=sTRACE,family=binomial)
##' summaryGLM(model)
##' summaryGLM(model,id=sTRACE$id)
##'
##' nd <- data.frame(sex=c(0,1),age=67,diabetes=1)
##' predictGLM(model,nd)
##' @aliases predictGLM 
summaryGLM <- function(object,id=NULL,fun=NULL,...) {# {{{

f <- function(p) exp(p)

est <- if (!is.null(id)) estimate(object, id=id, ...) else estimate(object, ...)
coef <- est
res  <- exp(est$coefmat[, c(1, 3, 4)])

if (!is.null(fun))  {
if (!is.null(id)) resl <- estimate(object,id=id,f=fun,...) else resl <- estimate(object,f=fun,...)
fout <- resl$coefmat
} else fout <- NULL

return(list(coef=coef,or=res,fout=fout))
}
# }}}

waldTest <- function(object,...)
{# {{{
Vars <-attr(terms(formula(object)),"term.labels")
## remove possible cluster term
clusterTerm<- grep("^cluster[(][A-z0-9._:]*",Vars,perl=TRUE)
if (length(clusterTerm)==1) Vars <- Vars[-clusterTerm]
Varf <- names(attr(attr(object$model,"terms"),"dataClasses"))[grep("factor",attr(attr(object$model,"terms"),"dataClasses"))]
txtVar<-names(coef(object))
###
test.matrix <- matrix(0,length(Vars),3)
colnames(test.matrix) <- c("Chisq","df","p.value")
rownames(test.matrix) <- Vars
k <- 1
for (ff in Vars) {
	pos <- grep(ff,txtVar,fixed=TRUE)
        wt <- estimate(object,as.list(pos),...)$compare
###	wt <- estimate(object,as.list(3:5))
	test.matrix[k,] <- unlist(wt[1:3])
	k <- k+1
}
return(test.matrix)
}# }}}

#' @keywords internal
"_PACKAGE"

##' Analysis of Multivariate Events
##'
##' Implementation of various statistical models for multivariate
##' event history data. Including multivariate cumulative incidence models,
##' and bivariate random effects probit models (Liability models)
##'
##' @name mets-package
##' @author Klaus K. Holst and Thomas Scheike
##' @useDynLib mets, .registration=TRUE
##' @import stats splines Rcpp mvtnorm
##' @importFrom lava iid estimate bootstrap compare score information
##' @importFrom lava twostage %++% %ni% addvar<- blockdiag cancel Col confband constrain<-
##' @importFrom lava constraints covariance covariance<- coxWeibull.lvm devcoords
##' @importFrom lava distribution<- endogenous eventTime Expand getoutcome gof intercept<-
##' @importFrom lava Inverse kill<- latent latent<- lava.options lvm Model multigroup
##' @importFrom lava parameter parameter<- pars regression regression<- revdiag
##' @importFrom lava trim IC expit logit
##' @importFrom survival Surv is.Surv concordance strata cluster finegray
##' @importFrom timereg two.stage predict.two.stage
##' @importFrom utils head tail getS3method glob2rx capture.output
##' @importFrom graphics matplot lines plot polygon par points abline title
##' @importFrom graphics matlines legend mtext layout axis barplot mosaicplot
##' @importFrom methods hasArg
##' @importFrom RcppArmadillo armadillo_set_number_of_omp_threads
##' @importFrom grDevices dev.list devAskNewPage dev.interactive
##' @aliases mets mets-package
NULL

##' @export
lava::iid

##' @export
lava::IC

##' @export
lava::twostage

##' @export
lava::estimate

##' @export
lava::gof

##' @export
survival::strata

##' @export
survival::Surv

##' @export
survival::cluster

##' np data set
##'
##' @name np
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Migraine data
##'
##' @name migr
##' @docType data
##' @keywords data
NULL

##' Dermal ridges data (families)
##'
##' Data on dermal ridge counts in left and right hand in (nuclear) families
##' @name dermalridges
##' @docType data
##' @keywords data
##' @format Data on 50 families with ridge counts in left and right
##' hand for moter, father and each child. Family id in 'family' and
##' gender and child number in 'sex' and 'child'.
##' @source Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
##' asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
##' pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
##' @examples
##' data(dermalridges)
##' fast.reshape(dermalridges,id="family",varying=c("child.left","child.right","sex"))
NULL

##' Dermal ridges data (monozygotic twins)
##'
##' Data on dermal ridge counts in left and right hand in (nuclear) families
##' @name dermalridgesMZ
##' @docType data
##' @keywords data
##' @format Data on dermal ridge counts (left and right hand) in 18
##' monozygotic twin pairs.
##' @source Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
##' asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
##' pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
##' @examples
##' data(dermalridgesMZ)
##' fast.reshape(dermalridgesMZ,id="id",varying=c("left","right"))
NULL

##' CALGB 8923, twostage randomization SMART design
##'
##' Data from CALGB 8923 
##' @name calgb8923 
##' @docType data
##' @keywords data
##' @format Data from smart design
##' id: id of subject
##' status : 1-death, 2-response for second randomization, 0-censoring
##' A0 : treament at first randomization
##' A1 : treament at second randomization
##' At.f : treament given at record (A0 or A1)
##' TR : time of response 
##' sex : 0-males, 1-females 
##' consent: 1 if agrees to 2nd randomization, censored if not
##' R: 1 if response 
##' trt1: A0
##' trt2: A1
##' @source  https://github.com/ycchao/code_Joint_model_SMART
##' @examples
##' data(calgb8923)
NULL

##' ACTG175, block randomized study from speff2trial package
##'
##' Data from speff2trial
##' @name ACTG175
##' @docType data
##' @keywords data
##' @format Randomized study 
##' @source  Hammer et al. 1996, speff2trial package.
##' @examples
##' data(ACTG175)
NULL

##' hfaction, subset of block randomized study HF-ACtion from WA package  
##'
##' Data from HF-action trial slightly modified from WA package,
##' consisting of 741 nonischemic patients with baseline
##' cardiopulmonary test duration less than or equal to 12 minutes.
##' @name hfactioncpx12
##' @docType data
##' @keywords data
##' @format Randomized study 
##' status : 1-event, 2-death, 0-censoring
##' treatment : 1/0
##' @source  WA package, Connor et al. 2009
##' @examples
##' data(hfactioncpx12)
NULL


##' Menarche data set
##'
##' @name mena
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Multivariate Cumulative Incidence Function example data set
##'
##' @name multcif
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Stutter data set
##'
##' Based on nation-wide questionnaire answers from 33,317 Danish twins
##' @format
##' tvparnr: twin-pair id
##' zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
##' stutter: stutter status (yes/no)
##' age: age
##' nr: number within twin-pair
##' @name twinstut
##' @docType data
##' @keywords data
NULL

##' BMI data set
##'
##' @format
##' Self-reported BMI-values on 11,411 subjects
##'
##' tvparnr: twin id
##' bmi: BMI (m/kg^2)
##' age: Age
##' gender: (male/female)
##' zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
##' @name twinbmi
##' @docType data
##' @keywords data
NULL

##' Prostate data set
##'
##' @name prt
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Rates for HPN program for patients of Copenhagen Cohort
##'
##' @name CPH_HPN_CRBSI 
##' @format
##'  crbsi: cumulative rate of catheter related bloodstream infection in HPN patients of Copenhagen
##'  mechanical: cumulative rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen
##' trombo: cumulative rate of Occlusion/Thrombosis complication for catheter of HPN patients of Copenhagen
##' terminal: rate of terminal event, patients leaving the HPN program
##' @docType data
##' @keywords data
##' @source Estimated data
NULL

##' haplo fun data 
##'
##' @name haplo 
##' @format
##' hapfreqs : haplo frequencies
##' haploX:  covariates and response for haplo survival discrete survival
##' ghaplos:  haplo-types for subjects of haploX data
##' @docType data
##' @keywords data
##' @source Estimated data
NULL

##' ttpd discrete survival data on interval form
##'
##' @name ttpd
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' @export
dreshape <- function(data,...) fast.reshape(data,...)

##' Fast reshape
##'
##' Fast reshape/tranpose of data
##' @param data data.frame or matrix
##' @param varying Vector of prefix-names of the time varying variables. Optional for Long->Wide reshaping.
##' @param id id-variable. If omitted then reshape Wide->Long. 
##' @param num Optional number/time variable
##' @param sep String seperating prefix-name with number/time
##' @param keep Vector of column names to keep
##' @param idname Name of id-variable (Wide->Long)
##' @param numname Name of number-variable (Wide->Long)
##' @param factor If true all factors are kept (otherwise treated as character)
##' @param idcombine If TRUE and \code{id} is vector of several variables, the unique id is combined from all the variables.
##' Otherwise the first variable is only used as identifier.
##' @param labelnum If TRUE varying variables in wide format (going from long->wide) are labeled 1,2,3,... otherwise use 'num' variable. In long-format (going from wide->long) varying variables matching 'varying' prefix are only selected if their postfix is a number.
##' @param labels Optional labels for the number variable
##' @param regex Use regular expressions
##' @param dropid Drop id in long format (default FALSE)
##' @param ... Optional additional arguments
##' @author Thomas Scheike, Klaus K. Holst
##' @aliases fast.reshape dreshape
##' @export
##' @examples
##' m <- lava::lvm(c(y1,y2,y3,y4)~x)
##' d <- lava::sim(m,5)
##' d
##' fast.reshape(d,"y")
##' fast.reshape(fast.reshape(d,"y"),id="id")
##' 
##' ##### From wide-format
##' (dd <- fast.reshape(d,"y"))
##' ## Same with explicit setting new id and number variable/column names
##' ## and seperator "" (default) and dropping x
##' fast.reshape(d,"y",idname="a",timevar="b",sep="",keep=c())
##' ## Same with 'reshape' list-syntax
##' fast.reshape(d,list(c("y1","y2","y3","y4")),labelnum=TRUE)
##' 
##' ##### From long-format
##' fast.reshape(dd,id="id")
##' ## Restrict set up within-cluster varying variables
##' fast.reshape(dd,"y",id="id")
##' fast.reshape(dd,"y",id="id",keep="x",sep=".")
##' 
##' #####
##' x <- data.frame(id=c(5,5,6,6,7),y=1:5,x=1:5,tv=c(1,2,2,1,2))
##' x
##' (xw <- fast.reshape(x,id="id"))
##' (xl <- fast.reshape(xw,c("y","x"),idname="id2",keep=c()))
##' (xl <- fast.reshape(xw,c("y","x","tv")))
##' (xw2 <- fast.reshape(xl,id="id",num="num"))
##' fast.reshape(xw2,c("y","x"),idname="id")
##' 
##' ### more generally:
##' ### varying=list(c("ym","yf","yb1","yb2"), c("zm","zf","zb1","zb2"))
##' ### varying=list(c("ym","yf","yb1","yb2")))
##' 
##' ##### Family cluster example
##' d <- mets:::sim_BinFam(3)
##' d
##' fast.reshape(d,var="y")
##' fast.reshape(d,varying=list(c("ym","yf","yb1","yb2")))
##' 
##' d <- lava::sim(lava::lvm(~y1+y2+ya),10)
##' d
##' (dd <- fast.reshape(d,"y"))
##' fast.reshape(d,"y",labelnum=TRUE)
##' fast.reshape(dd,id="id",num="num")
##' fast.reshape(dd,id="id",num="num",labelnum=TRUE)
##' fast.reshape(d,c(a="y"),labelnum=TRUE) ## New column name
##' 
##' 
##' ##### Unbalanced data
##' m <- lava::lvm(c(y1,y2,y3,y4)~ x+z1+z3+z5)
##' d <- lava::sim(m,3)
##' d
##' fast.reshape(d,c("y","z"))
##' 
##' ##### not-varying syntax:
##' fast.reshape(d,-c("x"))
##' 
##' ##### Automatically define varying variables from trailing digits
##' fast.reshape(d)
##' 
##' ##### Prostate cancer example
##' data(prt)
##' head(prtw <- fast.reshape(prt,"cancer",id="id"))
##' ftable(cancer1~cancer2,data=prtw)
##' rm(prtw)
fast.reshape <- function(data,varying,id,num,sep="",keep,
                 idname="id",numname="num",factor=FALSE,
                 idcombine=TRUE,labelnum=FALSE,labels,
                 regex=mets.options()$regex,
                 dropid=FALSE, ...) {
    if (!is.data.frame(data) & is.list(data)) {
        data <- as.data.frame(data)
    } else {
        if (NCOL(data)==1) data <- cbind(data)
    }
    nn <- colnames(data)
    if (!missing(varying)) {
        varsubst <- substitute(varying)
        if (as.character(varsubst)[1]=="-") {
            notvarying <- varsubst[[-1]]
            vars0 <- setdiff(nn,eval(notvarying,parent.frame()))
            ##numstr <- gsub("([1-9]\\d+)$","",vars0)
            ##numstr_sanspre0 <- gsub("(^0+)",num)
            varying <- unique(gsub("([1-9]|[1-9]\\d+)$","",vars0))
        }
        if (!missing(id)) varying <- setdiff(varying,id)
        if (!missing(num)) varying <- setdiff(varying,num)
    }
        
    if (missing(id)) {
        ## reshape from wide to long format. 
        nsep <- nchar(sep)
        if (missing(varying)) {#stop("Prefix of time-varying variables needed")
            ## Find all variable names with trailing digits (and leading zeros)
            vars0 <- grep("([1-9]|[1-9]\\d+)$",nn);
            varying <- unique(gsub("([1-9]|[1-9]\\d+)$","",nn[vars0]))
        }
        if (is.list(varying)) {
            orig <- varying
            for (i in seq_along(varying)) {
                elem <- varying[[i]]
                if (is.numeric(elem[i])) {
                    varying[[i]] <- colnames(data)[elem]
                }
                if (is.character(elem[i])) {
                    ii <- elem%in%colnames(data)
                    if (!all(ii)) {
                        newelem <- c()
                        for (j in seq_along(elem)) {
                            if (ii[j]) newelem <- c(newelem,elem[j])
                            else {
                                if (!regex) elem[j] <- glob2rx(elem[j])
                                newelem <- c(newelem,
                                            grep(elem[j],colnames(data),value=TRUE))
                            }
                        }
                        varying[[i]] <- newelem
                    }
                }
            }
        }
        ## nl <- as.list(seq_along(data)); names(nl) <- nn
        ## varying <- eval(substitute(varying),nl,parent.frame())
        vnames <- NULL
        ncvar <- sapply(varying,nchar)
        newlist <- c()
        numlev <- TRUE
        all_levels <- c()
        thelevels <- c()
        if (!is.list(varying)) {
            for (i in seq_len(length(varying))) {
                ii <- which(varying[i]==substr(nn,1,ncvar[i]))                
                thelevel <- substring(nn[ii],ncvar[i]+1+nsep)
                if (labelnum) {
                    ii0 <- suppressWarnings(which(!is.na(as.numeric(thelevel))))
                    ii <- ii[ii0]
                    thelevel <- thelevel[ii0]
                }
                all_levels <- union(all_levels,thelevel)
                thelevels <- c(thelevels,list(thelevel))
                suppressWarnings(tt <- as.numeric(thelevel))
                newlist <- c(newlist,list(nn[ii[order(tt)]]))
            }
            len <- unlist(lapply(newlist,length))
            for (i in seq_len(length(varying))) {
                if (len[i]<length(all_levels)) {
                    pp <- setdiff(all_levels,thelevels[[i]])
                    vv <- paste(varying[i],pp,sep=sep)
                    data[,vv] <- NA
                    newlist[[i]] <- paste(varying[i],all_levels,sep=sep)
                }                    
            }
            if (any(is.na(suppressWarnings(as.numeric(all_levels))))) {
                numlev <- FALSE
            } else {
                all_levels <- as.numeric(all_levels)
            }
            thelevels <- all_levels
            vnames0 <- names(varying) 
            vnames <- varying
            if (!is.null(vnames0)) {
                vidx <- which(vnames0!="")
                vnames[vidx] <- vnames0[vidx]
            }
            varying <- newlist
        } else {
            if (labelnum) {
                thelevels <- seq(length(varying[[1]]))
            } else {
                thelevels <- varying[[1]]
            }
        }
        if (!missing(labels)) thelevels <- labels
        is_df <- is.data.frame(data)
        oldreshape <- FALSE
        if (is_df) {
            ## D0 <- droplevels(data)[1,,drop=FALSE]
            D0 <- data[1,,drop=FALSE]
            classes <- unlist(lapply(D0,class))
            dim <- unlist(lapply(D0,NCOL))
            if (any(dim>1) || !all(classes%in%c("numeric","logical","integer","matrix","factor","character"))) { ## e.g. Surv columns 
                oldreshape <- TRUE
            } ## else {
            ##     chars <- which(classes%in%c("character"))
            ##     factors <- which(classes%in%c("factor"))
            ##     for (j in chars) data[,j] <- as.factor(data[,j])
            ##     if (length(c(chars,factors))>0) {
            ##         for (k in varying) {
            ##             if (any(nn[c(chars,factors)]%in%k)) {
            ##                 lev <- lapply(data[1,k],levels)
            ##                 allsame <- unlist(lapply(lev,function(x)
            ##                                          identical(x,lev[[1]])))
            ##                 if (!all(allsame))
            ##                     for (j in k) data[,j] <- factor(data[,j],levels=lev)
            ##             }
            ##         }
            ##         classes[chars] <- "factor"
            ##         D0 <- data[1,,drop=FALSE]
            ##     }            
            ##     data <- data.matrix(data)
            ## }
        }
        if (is.null(vnames)) {
            vnames <- unlist(lapply(varying,function(x) x[1]))
            if (!is.null(names(vnames))) vnames <- names(vnames)
        }

        if (oldreshape) {
            ## Fall-back to stats::reshape
            return(
                structure(reshape(as.data.frame(data),varying=varying,direction="long",v.names=vnames,timevar=numname,idvar=idname,...),
                          class=c("fast.reshape","data.frame"),
                          direction="wide",
                          varying=varying))
        }

        fixed <- setdiff(nn,unlist(c(varying,numname)))
        if (!missing(keep)) fixed <- intersect(fixed,c(keep,idname,numname))
        nfixed <- length(fixed)
        nvarying <- length(varying)
        nclusts <- unlist(lapply(varying,length))
        ##        nclust <- length(varying[[1]])
        nclust <- max(nclusts)
        
        if (any(nclusts!=nclust)) stop("Different length of varying vectors!")
        data <- data[,c(fixed,unlist(varying)),drop=FALSE]
        long <- as.data.frame(.Call("FastLong2",
                                    idata=data,
                                    inclust=as.integer(nclust),
                                    as.integer(nfixed),
                                    as.integer(nvarying),PACKAGE="mets"
                                    ));


        if (numname%in%fixed) {
            while (numname%in%c(fixed)) numname <- paste(numname,"_",sep="")
        }
        if (idname%in%fixed) {
            long <- long[,-(ncol(long)-1)]
            cnames <- c(fixed,vnames,numname)
        } else {
            cnames <- c(fixed,vnames,idname,numname)
        }
        ##  while (idname%in%c(fixed,vnames,numname)) idname <- paste(idname,"_",sep="")
        ##  while (numname%in%c(fixed,vnames)) numname <- paste(numname,"_",sep="")
        colnames(long) <- cnames
        if (!numlev) {
            long[,numname] <- base::factor(long[,numname],labels=thelevels)
        } else {
            if (!identical(order(thelevels),thelevels))
                long[,numname] <- thelevels[long[,numname]]
        } 

        if (is_df && factor) { ## Recreate classes            
            vars.orig <- c(fixed,unlist(lapply(varying,function(x) x[1])))
            vars.new <- c(fixed,vnames)
            factors <- which("factor"==classes[vars.orig])
            lev <- lapply(data[1,factors],levels)
            count <- 0
            for (i in factors) {
                count <- count+1
                long[,vars.new[i]] <- base::factor(long[,vars.new[i]],levels=lev[[count]])
            }
        }
        if (dropid) {
            ii <- which(colnames(long)%in%c(idname)) ##,numname
            long <- long[,-ii,drop=FALSE]
        }
        return(
            structure(long,
                      class=c("fast.reshape","data.frame"),
                      type="wide",
                      varying=varying))
    }


##################################################
### Long to wide format:
##################################################
    numvar <- idvar <- NULL 
    if (is.character(id)) {
        idvar <- id
        if (length(id)==1) {
            id <- data[,idvar,drop=TRUE]
        } else {
            if (idcombine)
                id <- interaction(as.data.frame(data[,idvar,drop=FALSE]),drop=TRUE)
            else
                id <- data[,idvar[1],drop=TRUE]
        } 
    } else {
        if (length(id)!=nrow(data)) stop("Length of ids and data-set does not agree")
    }
    
    unum <- NULL
    if (!missing(num) && !is.null(num)) {
        if (is.character(num)) {
            numvar <- num
            if (is.character(data[1,num,drop=TRUE])) {
                data[,num] <- as.factor(data[,num,drop=TRUE])
            }
            num <- as.integer(data[,num,drop=TRUE])
            if (!labelnum) unum <- sort(unique(data[,numvar,drop=TRUE]))
        } else {
            if (length(num)!=nrow(data)) stop("Length of time and data-set does not agree")
            if (!labelnum) unum <- unique(num)
        }
    } else {
        num <- NULL
    }
    
    if (any(nn=="")) data <- data.frame(data)

    clustud <- cluster_index(id,num=num)
    maxclust <- clustud$maxclust
    idclust <- clustud$idclust  
    obs1 <- clustud$firstclustid+1 ## as.vector(apply(idclust,1,function(x) na.omit(x)[1]))+1

    if (!is.null(numvar)) {
        ii <- which(colnames(data)==numvar)
        data <- data[,-ii,drop=FALSE]
    }

    if (!missing(keep)) {
        keepers <- c(keep,idvar)
        if (!missing(varying)) keepers <- c(keepers,varying)
        ii <- which(colnames(data)%in%keepers)
        data <- data[,ii,drop=FALSE]
    }

    if (missing(varying)) varying <- setdiff(colnames(data),c(idvar))
    vidx <- match(varying,colnames(data))
    N <- nrow(idclust)
    p <- length(varying)
    P <- NCOL(data)  
    fixidx <- setdiff(seq(P),vidx)
    if (is.matrix(data) || (all(apply(data[1,,drop=FALSE],2,is.numeric)) & length(unlist(data[1,]))==length(data[1,]) )) {
        ## Everything numeric - we can work with matrices
        dataw <- matrix(NA, nrow = N, ncol = p * (maxclust-1) + ncol(data))
        dataw[,fixidx] <- as.matrix(data[obs1,fixidx,drop=FALSE])
        mnames <- colnames(data)
        if (!is.null(unum)) {
            mnames[vidx] <- paste(mnames[vidx],unum[1],sep=sep)
        } else {
            mnames[vidx] <- paste(mnames[vidx],1,sep=sep)
        }
        if (p>0) {
            for (i in seq_len(maxclust)) {
                idx <- idclust[, i] + 1
                pos <- vidx
                if (i>1) {
                    pos <- P+seq(p)+p*(i-2)
                }
                dataw[which(!is.na(idx)), pos] <-
                    as.matrix(data[na.omit(idx),vidx,drop=FALSE])      
            }
            if (!is.null(unum)) {
                postn <- unum[-1]
            } else {
                postn <- seq_len(maxclust-1)+1
            }
            ##if (is.null(numname)) postn <- idlev[postn]
            mnames <- c(mnames,
                        as.vector(t(outer(postn,varying,function(x,y) paste(y,x,sep=sep)))))      
        }
        colnames(dataw) <- mnames
        return(structure(as.data.frame(dataw),class=c("fast.reshape","data.frame"),
                         varying=varying,direction="long"))
    }

    ## Potentially slower with data.frame where we use cbind
    for (i in seq_len(maxclust)) {
        if (i==1) {
            dataw <- data[obs1,,drop=FALSE]
            mnames <- names(data);
            dataw[,vidx] <- data[idclust[,i]+1,vidx,drop=FALSE]
            if (!is.null(unum)) 
                mnames[vidx] <- paste(varying,sep,unum[i],sep="")
            else 
                mnames[vidx] <- paste(mnames[vidx],sep,i,sep="")
        } else {
            dataw <- cbind(dataw,data[idclust[,i]+1,varying,drop=FALSE])
            if (!is.null(unum)) 
                mnames <- c(mnames,paste(varying,sep,unum[i],sep=""))
            else
                mnames <- c(mnames,paste(varying,sep,i,sep=""))
        }
    }
    names(dataw) <- mnames
    return(structure(dataw,class=c("fast.reshape","data.frame"),
                     varying=varying,type="long"))
} 

simple.reshape <- function (data, id = "id", num = NULL) {
    cud <- cluster_index(data[, c(id)], num = num, Rindex = 1)
    N <- nrow(cud$idclust)
    p <- ncol(data)
    dataw <- matrix(NA, nrow = N, ncol = p * cud$maxclust)
    for (i in seq_len(cud$maxclust)) {
        dataw[, seq(p) + (i - 1) * p] <- as.matrix(data[cud$idclust[, i] + 1, ])
    }
    colnames(dataw) <- paste(names(data), rep(seq_len(cud$maxclust), each = p), sep = ".")
    return(dataw)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.ApplyBy2 <- function(idata, icluster, F, Env, Argument = "x", Columnwise = 0L, Reduce = 0L, epsilon = 1.0e-16) {
    .Call(`_mets_ApplyBy2`, idata, icluster, F, Env, Argument, Columnwise, Reduce, epsilon)
}

.ApplyBy <- function(idata, icluster, f) {
    .Call(`_mets_ApplyBy`, idata, icluster, f)
}

S0FG <- function(ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart) {
    .Call(`_mets_S0_FG_GcR`, ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart)
}

Gcjumps <- function(iGc, istatus, istrata2, instrata2, iGcstart, injumps) {
    .Call(`_mets_GcjumpsR`, iGc, istatus, istrata2, instrata2, iGcstart, injumps)
}

S0FGN <- function(ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps) {
    .Call(`_mets_S0_FGRN`, ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps)
}

riskstrataR <- function(ia, istrata, instrata) {
    .Call(`_mets_riskstrataR`, ia, istrata, instrata)
}

cumsumstratasumR <- function(ia, istrata, instrata, iallstrata) {
    .Call(`_mets_cumsumstratasumR`, ia, istrata, instrata, iallstrata)
}

.scoreMVN <- function(Y, Mu, dMu, S, dS, itol = 0.0) {
    .Call(`_mets_scoreMVN`, Y, Mu, dMu, S, dS, itol)
}

.loglikMVN <- function(Yl, Yu, Status, Mu, S, Threshold, z, su, itol) {
    .Call(`_mets_loglikMVN`, Yl, Yu, Status, Mu, S, Threshold, z, su, itol)
}

.dmvn <- function(u, mu, rho) {
    .Call(`_mets_dmvn`, u, mu, rho)
}

.rmvn <- function(n, mu, rho) {
    .Call(`_mets_rmvn`, n, mu, rho)
}

.logl_weibull <- function(par, entry, exit, status, X, Z) {
    .Call(`_mets_logl_weibull`, par, entry, exit, status, X, Z)
}

.score_weibull <- function(par, entry, exit, status, X, Z, indiv = TRUE) {
    .Call(`_mets_score_weibull`, par, entry, exit, status, X, Z, indiv)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_mets_RcppExport_registerCCallable`)
})

##' Fast approximation
##' 
##' Fast approximation
##' @param time Original ordered time points
##' @param new.time New time points
##' @param equal If TRUE a list is returned with additional element
##' @param type Type of matching, nearest index, nearest greater than
##'     or equal (right), number of elements smaller than y otherwise
##'     the closest value above new.time is returned.
##' @param sorted Set to true if new.time is already sorted
##' @param ... Optional additional arguments
##' @author Klaus K. Holst
##' @examples
##' id <- c(1,1,2,2,7,7,10,10)
##' fast.approx(unique(id),id)
##' 
##' t <- 0:6
##' n <- c(-1,0,0.1,0.9,1,1.1,1.2,6,6.5)
##' fast.approx(t,n,type="left")
##' @aliases  indexstrata predictCumhaz cpred
##' @export
fast.approx <- function(time,new.time,equal=FALSE,type=c("nearest","right","left"),sorted=FALSE,...) {# {{{
    if (!sorted) {
        ord <- order(new.time,decreasing=FALSE)
        new.time <- new.time[ord]
    }
    A <- NULL
    if (NCOL(time)>1) {
        A <- time
        time <- A[,1,drop=TRUE]
    }
    if (is.unsorted(time)) warnings("'time' will be sorted")
    type <- agrep(type[1],c("nearest","right","left"))-1
    res <- .Call("FastApprox",
                 time=sort(time),
                 newtime=new.time,
                 equal=equal,
                 type=type,
                 PACKAGE="mets")
    if (!sorted) {
        oord <- order(ord)
        if (!equal) return(res[oord])
        res <- lapply(res,function(x) x[oord])
    }
    if (!is.null(A)) {
        A[res,,drop=FALSE]
    }
    return(res)
}# }}}

##' @export
predictCumhaz <- function (cum, new.time, type = c("left", "right", "nearest"),tminus=FALSE,tplus=FALSE,return.index=TRUE,...)
{# {{{
   if (NCOL(cum)>1) { cumh <- cum[,-1,drop=FALSE]; time <- cum[,1]} else time <- cum
   equal <- FALSE
   if (type[1]=="left" & tminus) equal <- TRUE
   if (type[1]=="right" & tplus) equal <- TRUE
   index <- fast.approx(time,new.time,type=type[1],equal=equal,...)
   if (equal) {
   equali <- which(index$eq!=0)
   index <- index$idx
   if (length(equali)>=1 & tminus & type[1]=="left") {
	   index[equali] <- index[equali] -1
   }
   if (length(equali)>=1 & tplus & type[1]=="right") {
	   index[equali] <- pmin(length(time),index[equali] +1)
   }
   if (any(index==0))  { 
      index[index==0] <- 1 
   }
   }
   if (NCOL(cum)>1) {
	   res <- cbind(new.time, cumh[index,])
   } else {
	   if (return.index) res <-  index else res <- time[index]
   }
   return(res)
}# }}}

##' @export
cpred <- function(...) predictCumhaz(...)

##' @export
indexstrata <- function(jump.times,eval.times,jump.strata=NULL,eval.strata=NULL,nstrata=NULL,start.time=NULL,...)
{   # {{{
	stopifnot(is.numeric(jump.times))
	stopifnot(is.numeric(eval.times))
	if (is.null(jump.strata)) jump.strata <- rep(0,length(jump.times))
	if (is.null(eval.strata)) eval.strata <- rep(0,length(eval.times))
###	if (any(eval.strata<0) | any(eval.strata>nstrata-1)) stop("eval.strata index not ok\n"); 
###	if (any(jump.strata<0) | any(jump.strata>nstrata-1)) stop("jump.strata index not ok\n"); 
	N <- length(jump.times)
	index <- rep(0,length(eval.times))
	
	for (i in unique(eval.strata)) {
	   wherej <- which(eval.strata==i)
	   whereJ <- which(jump.strata==i)
	   if (!is.null(start.time)) iindex <- predictCumhaz(c(start.time,jump.times[whereJ]),eval.times[wherej],...)
	   else iindex <- predictCumhaz(jump.times[whereJ],eval.times[wherej],...)
	iindexn0 <- which(iindex!=0)
        if (length(iindexn0)>0)
	   index[wherej[iindexn0]] <- whereJ[iindex[iindexn0]]
	}
	return(index)
}# }}}

## fastapprox
library("tinytest")

test_fastapprox <- function() {
  set.seed(1)
  x <- sort(rnorm(1e5))
  y <- rnorm(1e3)
  val <- x[fast.approx(x, y)]
  val2 <- sapply(y, function(y) x[which.min(abs(x - y))])
  expect_identical(val, val2)
  ##rbenchmark::benchmark(fast.approx(x,y,type="left"),replications=1000)
  ##rbenchmark::benchmark(prodlim::sindex(x,y),replications=1000)
  val <- fast.approx(x, y, type="left") # Number of observations in x less than y
  val3 <- prodlim::sindex(x, y)
  expect_identical(val, val3)
}
test_fastapprox()

context("Fast.approx")

test_that("fast approx I", {
    set.seed(1)
    x <- sort(rnorm(1e5))
    y <- rnorm(1e3)
    val <- x[fast.approx(x,y)]
    val2 <- sapply(y, function(y) x[which.min(abs(x-y))])
    expect_identical(val,val2)
    ##rbenchmark::benchmark(fast.approx(x,y,type="left"),replications=1000)
    ##rbenchmark::benchmark(prodlim::sindex(x,y),replications=1000)    
    val <- fast.approx(x,y,type="left") # Number of observations in x less than y
    val3 <- prodlim::sindex(x,y)
    expect_identical(val,val3)    
})

nameme <- function(iid,name.id,sort=TRUE) { ## {{{
if (is.matrix(iid))
	if (nrow(iid)==length(name.id)) {
		rownames(iid) <- name.id
###		if (sort)  {
###		oid <- order(name.id);
###		iid <- iid[oid,,drop=FALSE];
###		}
}
return(iid)
} ## }}}

construct_id <- function(id,nid,namesX=NULL,as.data=FALSE) { ## {{{
  call.id <- id

  if (!is.null(id)) {
	  ids <- unique(id)
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
     name.id <- sort(ids)
   } else {
	   id <- 1:nid-1;
	   ids <- id+1
	   order.ids <- ids
	   name.id <- NULL
	   ## Use rownames of design matrix as labels when id was not provided
           name.id <- if (!is.null(namesX)) namesX else NULL 
   }
   ## orginal id coding into integers
   ## id from 0,1,...,nid-1

    if (as.data) {
        id  <- (0:(nid - 1))[order(ids)][id +1]
        name.id <- ids
    }

  return(list(call.id=call.id,id=id,nid=nid,unique.id=ids,name.id=name.id))
} ## }}}

###{{{ phreg01
phreg01 <- function(X,entry,exit,status,id=NULL,strata=NULL, offset=NULL,weights=NULL,strata.name=NULL,cumhaz=TRUE,
    beta,stderr=TRUE,method="NR", no.opt=FALSE,Z=NULL,propodds=NULL,AddGam=NULL,
    marks=NULL,no.var=0,augmentation=0,...) {
  p <- ncol(X)
  if (missing(beta)) beta <- rep(0,p)
  if (p==0) X <- cbind(rep(0,length(exit)))

  if (is.null(strata)) {
	  strata <- rep(0,length(exit)); nstrata <- 1; strata.level <- NULL;
  } else {
	  strata.level <- levels(strata)
	  ustrata <- sort(unique(strata))
	  nstrata <- length(ustrata)
	  strata.values <- ustrata
      if (is.numeric(strata)) strata <-  fast.approx(ustrata,strata)-1 else  {
      strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
    }
  }
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given
  if (!is.null(Z)) Zcall <- Z

  ## possible casewights to use for bootstrapping and other things
  if (is.null(marks)) marks <- rep(1,length(exit))
  trunc <- (any(entry>0))
###  if (!trunc) entry <- rep(0,length(exit))

  call.id <- id;
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  id <- conid$id; nid <- conid$nid; name.id <- conid$name.id

  dd <- .Call("FastCoxPrepStrata", entry,exit,status,X,id,trunc,strata,weights,offset,Zcall,marks,PACKAGE="mets")

  dd$nstrata <- nstrata
   obj <- function(pp,U=FALSE,all=FALSE) {# {{{
      if (length(dd$jumps) >0) {
	      if (is.null(propodds) & is.null(AddGam)) {
		val <- with(dd, .Call("FastCoxPLstrata",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,caseweights,PACKAGE="mets"))
	      } else if (is.null(AddGam))
		val <- with(dd, .Call("FastCoxPLstrataPO",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,propodds,PACKAGE="mets"))
	      else val <- with(dd, .Call("FastCoxPLstrataAddGam",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,
					 AddGam$theta,AddGam$dimthetades,AddGam$thetades,AddGam$ags,AddGam$varlink,AddGam$dimjumprv,AddGam$jumprv,
					 AddGam$JumpsCauses,PACKAGE="mets"))
	      val$gradient <- val$gradient+augmentation
      } else {
	      val <- list(jumps=NULL,ploglik=NULL,U=NULL,gradient=NULL,hessian=NULL,hessiantime=NULL,S2S0=NULL,E=NULL,S0=NULL)
      }

	  if (all) {
	      val$time <- dd$time
	      ### plus 1 for R index
	      val$ord <- dd$ord+1
	      val$jumps <- dd$jumps+1
	      val$jumptimes <- val$time[val$jumps]
	      val$weightsJ <- dd$weights[val$jumps]
	      val$marks <- dd$marks[val$jumps]
	      val$nevent <- length(val$S0)
	      val$nstrata <- dd$nstrata
	      val$strata <- dd$strata
	      val$strata.jumps <- val$strata[val$jumps]
	      return(val)
	  }
          n <- length(dd$time)
	 with(val,structure(-ploglik/n,gradient=-gradient/n,hessian=-hessian/n))
	}# }}}

  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(beta,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  names(cc) <- colnames(X)
      if (!stderr) return(cc)
      val <- c(list(coef=cc), obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta), obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL
  II <- NULL
  if (no.opt==FALSE & p!=0) {
         II <- - tryCatch(pinv(val$hessian),error=function(e) matrix(0,nrow(val$hessian),ncol(val$hessian)) )
  } else II <- matrix(0,p,p)

  ## Brewslow estimator, to handle also possible weights, caseweights that are 0
  ww <- val$caseweightsJ * val$weightsJ
  val$S0[abs(ww)<.00000001] <- 0

  ### computes Breslow estimator
  if (cumhaz==TRUE & (length(val$jumps)>0)) { # {{{
	 strata <- val$strata[val$jumps]
	 nstrata <- val$nstrata
	 jumptimes <- val$jumptimes

	 ## Brewslow estimator, to handle also possible weights/case-weights, and caseweights that are 0
	 S0i2 <- S0i <- rep(0,length(val$S0))
         wwJ <- val$caseweightsJ*val$weightsJ
	 S0i[val$S0>0] <- 1/val$S0[val$S0>0]
	 S0i2[val$S0>0] <- 1/(val$S0[val$S0>0]^2*wwJ[val$S0>0])
	 cumhaz <- cbind(jumptimes,cumsumstrata(S0i,strata,nstrata))
	 if ((no.opt==FALSE & p!=0)) {
	     DLambeta.t <- apply(val$E*S0i,2,cumsumstrata,strata,nstrata)
	     varbetat <-   rowSums((DLambeta.t %*% II)*DLambeta.t)
	 ### covv <-  apply(covv*DLambeta.t,1,sum) Covariance is "0" by construction
	 } else varbetat <- 0
	 var.cumhaz <- cumsumstrata(S0i2,strata,nstrata)+varbetat
	 se.cumhaz <- cbind(jumptimes,(var.cumhaz)^.5)

	 colnames(cumhaz)    <- c("time","cumhaz")
	 colnames(se.cumhaz) <- c("time","se.cumhaz")
 } # }}}
 else {cumhaz <- se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL}

  res <- c(val,
           list(cox.prep=dd,
                strata.call=strata.call, strata.level=strata.level,
                entry=entry, exit=exit, status=status, X=X,
                p=p, offsets=offset, weights=weights,
                id=id,call.id=call.id,nid=nid,name.id=name.id,
                opt=opt, no.opt=no.opt, cumhaz=cumhaz, se.cumhaz=se.cumhaz,
                lcumhaz=lcumhaz, lse.cumhaz=lse.cumhaz, ihessian=II,
                II=II,strata.name=strata.name,propodds=propodds))
  class(res) <- "phreg"

  ## also computing robust variance
  if (p>0 & no.var==0) {
  beta.iid <- iid(res)
  phvar <- crossprod(beta.iid)
###  estimate.beta <- estimate(coef=val$coef,IC=nrow(beta.iid)*beta.iid,id=name.id)
###  res$estimate.beta <- estimate.beta
  colnames(phvar) <- rownames(phvar) <- names(res$coef)
  res$var <- phvar
  res$beta.iid <- beta.iid
  } else res$var <- NULL

  return(res)
}

###}}} phreg0

###{{{ phreg

##' Fast Cox Proportional Hazards Regression
##'
##' Fits a Cox proportional hazards model using a fast C++ backend.
##' Robust variance (sandwich estimator) is the default variance estimate in the summary.
##'
##' The influence functions (IID decomposition) follow the numerical order of the given cluster variable.
##' Ordering the results by `$id` will align the IID terms with the original dataset order.
##'
##' @param formula Formula with a 'Surv' or 'Event' outcome (similar to \code{coxph}).
##' @param data Data frame containing the variables.
##' @param offset Offsets for the Cox model linear predictor.
##' @param weights Weights for the Cox score equations.
##' @param ... Additional arguments passed to lower-level functions (e.g., optimization controls).
##'
##' @return An object of class \code{"phreg"} containing:
##' \item{coef}{Vector of estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{beta.iid}{Influence functions for the regression coefficients.}
##' \item{cumhaz}{Matrix of cumulative hazard estimates (time, cumhaz).}
##' \item{se.cumhaz}{Matrix of standard errors for the cumulative hazard.}
##' \item{cox.prep}{List containing preprocessed data for the Cox model.}
##' \item{opt}{Optimization results (if optimization was performed).}
##' \item{call}{The matched call.}
##'
##' @author Klaus K. Holst, Thomas Scheike
##' @seealso \code{\link{plot.phreg}}, \code{\link{predict.phreg}}, \code{\link{robust_phreg}}
##' @aliases phreg robust_phreg
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' # Fit model with clustering
##' out1 <- phreg(Surv(time, status == 9) ~ wmi + age + strata(vf, chf) + cluster(id), data = TRACE)
##' summary(out1)
##'
##' # Plotting baselines
##' par(mfrow = c(1, 2))
##' plot(out1)
##'
##' # Computing robust variance for baseline
##' rob1 <- robust_phreg(out1)
##' plot(rob1, se = TRUE, robust = TRUE)
##'
##' # IID decomposition for regression parameters
##' head(iid(out1))
##'
##' # IID decomposition for baseline at a specific time-point
##' Aiiid <- iid(out1, time = 30)
##' head(Aiiid)
##'
##' # Combined IID decomposition (beta and baseline)
##' dd <- iidBaseline(out1, time = 30)
##' head(dd$beta.iid)
##' head(dd$base.iid)
##'
##' # Stratified model
##' outs <- phreg(Surv(time, status == 9) ~ strata(vf, wmicat.4) + cluster(id), data = TRACE)
##' summary(outs)
##' par(mfrow = c(1, 2))
##' plot(outs)
##' @export
phreg <- function(formula,data,offset=NULL,weights=NULL,...) {# {{{
  cl <- match.call() ## {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    marks <- des$marks
    if (!is.null(strata))  {
      strata.name <- attr(des$specials.var$strata, "name.mf")
    } else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit))
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit))
  } else weights <- des.weights
 ## }}}

 res <- c(
     phreg01(X, entry, exit, status,
         id = id, strata = strata, offset = offset,
         weights = weights, strata.name = strata.name, marks = marks, ...
     ),
     list(
         call = cl, formula = formula, strata.pos = pos.strata,
         cluster.pos = pos.cluster, n = length(exit)
     )
 )
  res$model.frame <- proc_design(des$terms, data)$data #, specials="cluster")$data
  des$data <- data[0,]
  res$design <- des
  class(res) <- "phreg"

  res
}# }}}

readPhreg <- function (object, newdata, nr=TRUE,data=TRUE, ...)
{# {{{
   respindata <- length((grep(all.vars(object$formula)[1],names(newdata))))
   if (respindata>0) des <- update_design(object$design,data = newdata,response=TRUE)
   else des <- update_design(object$design,data = newdata)

   clusters <- des$cluster
   X <- des$x
   if (data) {
       xxr <- drop.specials(object$formula,"cluster")
       vars <- all.vars( update.formula(xxr,1~.))
       data <- newdata[,vars,drop=FALSE]
   } else data <- NULL

   if (!is.null(des$strata)) strataNew <- as.numeric(des$strata)-1 else strataNew <- rep(0,nrow(X))

    if (!is.null(des$y)) {
	    Y <- des$y
	    if (!inherits(Y, c("Event", "Surv"))) {
		stop("Expected a 'Surv' or 'Event'-object")
	    }
	    if (ncol(Y) == 2) {
		exit <- Y[, 1]
		entry <- rep(0, nrow(Y))
		status <- Y[, 2]
	    } else {
		entry <- Y[, 1]
		exit <- Y[, 2]
		status <- Y[, 3]
	    }
    } else {status <- exit <- entry <- NULL; }

return(list(data=data,X=X,strata=strataNew,entry=entry,exit=exit,status=status,clusters=clusters))
}# }}}


### }}} phreg

### {{{ iid & Robust variances

transformation_phreg <- function(type, lp = TRUE) {
  # p[1]: linear predictor
  # p[2..]: baseline cumulative hazard in different strata
  if (type == "risk") {
    f <- ifelse(lp,
      function(p) {
        res <- numeric(length(p) - 1)
        for (i in seq(2, length(p))) {
          res[i - 1] <- 1 - exp(-p[i])**exp(p[1])
        }
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)[-1]
        }
        return(res)
      },
      function(p) {
        res <- 1 - exp(-p)
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)
        }
        return(res)
      }
    )
    return(f)
  }
  if (type == "surv") {
    f <- ifelse(lp,
      function(p) {
        res <- numeric(length(p) - 1)
        for (i in seq(2, length(p))) {
          res[i - 1] <- exp(-p[i])**exp(p[1])
        }
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)[-1]
        }
        return(res)
      },
      function(p) {
        res <- exp(-p)
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)
        }
        return(res)
      }
    )
    return(f)
  }
  ## type == "chaz"
  f <- ifelse(lp,
    function(p) {
      res <- numeric(length(p) - 1)
      for (i in seq(2, length(p))) {
        res[i - 1] <- p[i]*exp(p[1])
      }
      if (length(res) == 1L) {
        names(res) <- type
      } else {
        names(res) <- names(p)[-1]
      }
      return(res)
    },
    function(p) {
      if (length(p) == 1L) {
        names(p) <- type
      }
      return(p)
    }
  )
  return(f)
}

##' @export
estimate.phreg <- function(x, ..., time = NULL,
			   newdata = NULL, X = NULL,all=FALSE,
                           type = c("chaz", "surv", "risk"),
                           baseline.args = list()) { ## {{{ 
  if (NCOL(model.matrix(x))==0L & is.null(time)) stop("Non-parametric model; need `time` argument")
  if (!is.null(newdata) || !is.null(X)) {
    if (is.null(X)) {
      X <- model.matrix(x, data = newdata)
    }
    if (NROW(X) > 1) {
      X <- X[1, ]
      warning("Only using first row of the data")
    }
    if (all(X == 0L)) X <- NULL
  }
  if (!is.null(X)) {
    lp <- estimate(x, rbind(X), labels = "lp")
    if (is.null(time)) {
      return(estimate(lp, ...))
    }
    base <- estimate(x,
      time = time,
      type = "chaz",
      baseline.args = baseline.args
    )
    res <- transform(merge(lp, base,
                           paired = TRUE
                           ),
                     transformation_phreg(tolower(type[1]))
                     )
    return(estimate(res, ...))
  }
  ic <- do.call(IC, c(list(x, time = time,all=all), baseline.args))
  cc <- attr(ic, "coef")
  if (is.null(cc)) cc <- coef(x)
  lab <- names(cc)
###  if (!is.null(time) && length(cc) > 1 || is.null(lab)) lab <- x$strata.level
###  b <- lava::estimate(coef = cc, IC = ic, labels = lab)
  b <- lava::estimate(coef = cc, IC = ic)
  if (!is.null(time)) {
    b <- transform(b, transformation_phreg(tolower(type[1]), FALSE))
  }
  return(estimate(b, ...))
} ## }}} 

##' Influence Functions for phreg objects
##'
##' Computes the influence functions (IID decomposition) for the regression coefficients
##' and/or the baseline cumulative hazard at a specific time point.
##'
##' @param x Object of class \code{"phreg"}.
##' @param type Type of influence function: \code{"robust"} (default) or \code{"martingale"}.
##' @param all Logical; if \code{TRUE}, returns both beta and baseline influence functions.
##' @param time Time point for baseline influence function (required if baseline is requested).
##' @param baseline Arguments for baseline estimation.
##' @param ... Additional arguments.
##'
##' @return A matrix of influence functions. If \code{all=TRUE}, columns correspond to
##' regression coefficients and baseline cumulative hazard. Attributes include \code{coef}
##' and \code{time}.
##'
##' @author Thomas Scheike
##' @seealso \code{\link{phreg}}, \code{\link{iidBaseline}}
##' @export
IC.phreg  <- function(x,type="robust",all=FALSE,time=NULL,baseline=NULL,...) {# {{{
  if (x$p==0 & is.null(time)) stop("Non-parametric model; need `time` argument")
  if (!is.null(time)) {
    iid <- iidBaseline(x, time = time, ...)
    if (all)  {
            res <- with(iid, cbind(beta.iid,base.iid) * NROW(base.iid))
	    coefs <- c(iid$coef,iid$cumhaz.time) 
	    colnames(res) <- names(coefs)
            attr(res, "coef") <- coefs
    } else { 
            res <- with(iid, base.iid * NROW(base.iid))
	    attr(res, "coef") <- c(iid$cumhaz.time)
    }
    attr(res, "strata.level") <- iid$strata.level
    attr(res, "time") <- time
    return(res)
  }
    classes1 <- "mlogit"
    if ((length(class(x)) == 1) || inherits(x, classes1)) {
        invhess <- -pinv(x$hessian)
        orig.order <- FALSE
        if (is.null(x$propodds)) {
            if (type == "robust") {
                xx <- x$cox.prep
                ii <- invhess
                S0i <- rep(0, length(xx$strata))
                S0i[xx$jumps + 1] <- 1/x$S0
                Z <- xx$X
                U <- E <- matrix(0, nrow(xx$X), x$p)
                E[xx$jumps + 1, ] <- x$E
                U[xx$jumps + 1, ] <- x$U
                cumhaz <- cbind(xx$time, cumsumstrata(S0i, xx$strata,
                  xx$nstrata))
                EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata,
                  xx$nstrata)
                rr <- c(xx$sign * exp(Z %*% coef(x) + xx$offset))
                MGt <- U[, drop = FALSE] - (Z * cumhaz[, 2] -
                  EdLam0) * rr * c(xx$weights)
                if (orig.order) {
                  oo <- (1:nrow(xx$X))[xx$ord + 1]
                  oo <- order(oo)
                  MGt <- MGt[oo, , drop = FALSE]
                  id <- xx$id[oo]
                }
                else id <- xx$id
            }
            else {
                MGt <- x$U
                MG.base <- 1/x$S0
            }
        } else {
            xx <- x$cox.prep
            ii <- invhess
            S0i <- rep(0, length(xx$strata))
            S0i[xx$jumps + 1] <- 1/x$S0
            Z <- xx$X
            U <- E <- matrix(0, nrow(xx$X), x$p)
            E[xx$jumps + 1, ] <- x$E
            U[xx$jumps + 1, ] <- x$U
            cumhazA <- cumsumstratasum(S0i, xx$strata, xx$nstrata,
                type = "all")
            cumhaz <- c(cumhazA$sum)
            cumhazm <- cumhazA$lagsum
            EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata,
                xx$nstrata)
            rr <- c(xx$sign * exp(Z %*% coef(x) + xx$offset))
            rro <- c(exp(Z %*% coef(x) + xx$offset))
            S0star <- revcumsumstrata(rr/(1 + rro * cumhazm),
                xx$strata, xx$nstrata)
            S0 <- revcumsumstrata(rr, xx$strata, xx$nstrata)
            S1 <- apply(Z * rr, 2, revcumsumstrata, xx$strata,
                xx$nstrata)
            Et <- S1/c(S0)
            lt <- apply((Z - Et) * c(rr * rro/(1 + rro * cumhazm)),
                2, revcumsumstrata, xx$strata, xx$nstrata)
            Estar <- S0star/S0
            EstardLam <- cumsumstrata(Estar * S0i, xx$strata,
                xx$nstrata)
            k <- exp(-EstardLam)
            basecor <- apply(lt * c(k * S0i), 2, revcumsumstrata,
                xx$strata, xx$nstrata)
            basecor <- basecor/c(k * S0)
            www <- x$propoddsW * x$weightsJ
            U[xx$jumps + 1, ] <- U[xx$jumps + 1, ] - c(www) *
                basecor[xx$jumps + 1, ]
            MGt <- U
            if (type == "robust") {
                baseDLam0 <- apply(basecor * S0i, 2, cumsumstrata,
                  xx$strata, xx$nstrata)
                MGt <- U[, drop = FALSE] - (Z * cumhaz - EdLam0 -
                  baseDLam0) * rr * c(xx$weights)
            }
            else MGt <- MGt[xx$jumps + 1, ]
            MG.base <- 1/x$S0
            id <- xx$id
        }
        ncluster <- NULL
        if (type == "robust" & (!is.null(x$id) | any(x$entry >
            0))) {
            if (type == "martingale")
                id <- x$id[x$jumps]
            UU <- apply(MGt, 2, sumstrata, id, max(id) + 1)
            ncluster <- nrow(UU)
        }
        else {
            UU <- MGt
        }
        res <- structure(UU %*% invhess, invhess = invhess, ncluster = ncluster)
        if (is.null(colnames(res))) colnames(res) <- names(coef(x))
	if (!is.null(x$name.id)) res <- nameme(res, x$name.id)
        res <- res * NROW(res)
        return(res)
    }
    else if (inherits(x, c("cifreg", "recreg", "IPTW"))) {
        if (inherits(x, c("IPTW")))
            res <- x$IID
        else res <- x$iid
        if (is.null(colnames(res)))
            colnames(res) <- names(coef(x))
        res <- res * NROW(res)
        return(res)
    }
} ## }}}

##' Influence Functions or IID Decomposition of Baseline
##'
##' Computes the influence functions for the baseline cumulative hazard (and optionally
##' regression coefficients) for \code{phreg}, \code{recreg}, or \code{cifregFG} objects.
##'
##' The decomposition is based on the formula:
##' \deqn{ \sum_i \int_0^t \frac{f(s)}{S_0(s)} dM_{ki}(s) - P(t) \beta_k }
##' where \eqn{k} denotes the stratum and \eqn{i} the cluster.
##'
##' @param object Object of class \code{"phreg"}, \code{"recreg"}, or \code{"cifregFG"}.
##' @param time Time point for baseline IID (required).
##' @param ft Function to compute IID of baseline integrated against \eqn{f(t)}.
##' @param fixbeta Logical; if \code{TRUE}, fixes the coefficients (useful for specific tests).
##' @param beta.iid Optional matrix of beta influence functions to use.
##' @param tminus Logical; if \code{TRUE}, computes predictions at \eqn{t-} (strictly before \eqn{t}), useful for IPCW techniques.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"iidBaseline"} containing:
##' \item{time}{Time point.}
##' \item{base.iid}{Influence functions for the baseline.}
##' \item{beta.iid}{Influence functions for the regression coefficients.}
##' \item{cumhaz.time}{Cumulative hazard at the specified time.}
##' \item{strata}{Strata indices.}
##'
##' @author Thomas Scheike
##' @seealso \code{\link{phreg}}, \code{\link{recreg}}, \code{\link{cifreg}}
##' @export
iidBaseline <- function(object, time = NULL, ft = NULL, fixbeta = NULL, beta.iid = NULL, tminus = FALSE, ...) UseMethod("iidBaseline")

##' @export
iidBaseline.phreg <- function(object,time=NULL,ft=NULL,fixbeta=NULL,beta.iid=NULL,tminus=FALSE,...)
{# {{{
###  sum_i int_0^t f(s)/S_0(s) dM_{ki}(s) - P(t) \beta_k
###  with possible strata and cluster "k", and i in clusters
  x <- object
  if (!inherits(x,"phreg")) stop("Must be phreg object\n");
  if (is.null(time)) stop("Must give time for iid of baseline")

  if (!is.null(x$propodds))  stop("Only for Cox model")
  ### sets fixbeta based on  wheter xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) {
    fixbeta <- if ((x$no.opt) | is.null(x$coef)) 1 else 0
 }


  xx <- x$cox.prep
  btimexx <- c(1*(xx$time < time))

  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <- 1/x$S0
  ww <- xx$caseweights*xx$weights
  S0i2[xx$jumps+1] <- 1/(x$S0^2*ww[xx$jumps+1])
  Z <- xx$X
  U <- E <- matrix(0,nrow(xx$X),x$p)
  E[xx$jumps+1,] <- x$E
  U[xx$jumps+1,] <- x$U
  ### only up to time t for A
  if (is.null(ft))  ft <- rep(1,length(xx$time))
  if (is.matrix(ft)) ft <- c(ft)
  cumhaz <- c(cumsumstrata(ft*S0i,xx$strata,xx$nstrata))
  cumS0i2 <- c(cumsumstrata(ft*S0i2*btimexx,xx$strata,xx$nstrata))
  if (fixbeta==0) {
	  EdLam0 <- apply(ft*E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Ht <- apply(ft*E*S0i*btimexx,2,cumsumstrata,xx$strata,xx$nstrata)
  } else Ht <- NULL
  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset)) else rr <- c(xx$sign*exp(xx$offset))
  id <-   xx$id
  mid <- max(id)+1
  MGAiid <- ft*S0i*btimexx-cumS0i2*rr*c(xx$weights)

  MGtiid <- NULL
  if (fixbeta==0) {# {{{
     if (!is.null(beta.iid)) MGtiid <- beta.iid else {
     invhess <- -pinv(x$hessian)
     MGt <- ft*U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
     MGt <- MGt %*% invhess
     MGtiid <- apply(MGt,2,sumstrata,id,mid)
    }
     ## Ht efter strata
     Htlast <- tailstrata(xx$strata,xx$nstrata)
     HtS <- Ht[Htlast,,drop=FALSE]
     if (!is.null(x$call.id)) MGtiid <- nameme(MGtiid,x$name.id)
  }# }}}

 ### \hat beta - \beta = \sum_i \beta_i  (iid)
 ### iid after baseline:
 ### \hat A_s-A_s=\sum_{i clusters} \sum_{j: i(j)=i, s(j)=s} \int_0^t 1/S_0 dM^s_{i,j} - P^s(t) \sum_i \beta_i
 ### = \sum_{i clusters} ( \sum_{j \in i(j)=i, s(j)=s} \int_0^t 1/S_0 dM^s_j - P^s(t) \beta_i )

 ## sum after id's within strata and order
 MGAiids <- c()
 cumhaz.time <- c()
 sus <- sort(unique(xx$strata))
 for (i in sus)  {
	 wi <- which(xx$strata==i)
         MGAiidl <- sumstrata(MGAiid[xx$strata==i],xx$id[xx$strata==i],mid)
	 cumhazs <- rbind(0,x$cumhaz[x$strata[x$jumps]==i,])
	 cumhaz.time <- c(cumhaz.time,cpred(cumhazs,time,tminus=tminus)[,2])

        if (fixbeta==0) {
           UU <-  apply(HtS[i+1,]*t(MGtiid),2,sum)
           MGAiidl <- MGAiidl - UU
         }
         MGAiids <- cbind(MGAiids,MGAiidl)
 }
 names(cumhaz.time) <- paste("strata",sus,sep="")
 MGAiid <- MGAiids
 if (is.matrix(MGAiid)) {
    colnames(MGAiid) <- paste("strata",sus,sep="")
   if (!is.null(x$name.id)) MGAiid <- nameme(MGAiid,x$name.id)
    ###    if (length(x$name.id)==nrow(MGAiid)) rownames(MGAiid) <- x$name.id
 }

 cumhaz <- x$cumhaz
 out <- list(time=time,base.iid=MGAiid,strata=xx$strata,nstrata=xx$nstrata,coef=x$coef,
	     cumhaz=cumhaz,cumhaz.time=cumhaz.time,
	     id=id,beta.iid=MGtiid,formula=x$formula,
	     design=object$design)
 class(out) <- c("iidBaseline","phreg")
 return(out)
} # }}}

##' @export
summary.iidBaseline  <- function(object,...) { ## {{{
cat("iid decomposition of object phreg/cifregFG/recreg \n")
} # }}}

##' @export
print.iidBaseline <- function(x,...) { ## {{{
	x$base.iid
} # }}}


##' @export
residuals.phreg  <- function(object,cumsum=FALSE,...) {# {{{
  orig.order <- FALSE
  x <- object

if (is.null(x$propodds)) { # {{{ cox model
	  xx <- x$cox.prep
	  dN <- S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  dN[xx$jumps+1] <- 1
	  cumhaz <- cumsumstrata(S0i,xx$strata,xx$nstrata)
	  Z <- xx$X
	  if (is.null(coef(x)))
		  rr <- c(xx$sign*exp(xx$offset))
          else
	  rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
	  ### dMartingale  as a function of time and for all subjects to handle strata
	  Lamt <- (cumhaz)*rr*c(xx$weights)
	  dMGt <- dN-Lamt
	  if (orig.order) {
	     oo <- (1:nrow(xx$X))[xx$ord+1]
	     oo <- order(oo)
	     ### back to order of iid variable
	     dMGt <- dMGt[oo,,drop=FALSE]   ## sum after id later so not needed
	     id <- xx$id[oo]
	  } else id <-  xx$id
} else { ## }}}
	# {{{  prop-odds model logitSurv
	  xx <- x$cox.prep
	  dN <- S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  dN[xx$jumps+1] <- 1
	  U <- E <- matrix(0,nrow(xx$X),x$p)
	  E[xx$jumps+1,] <- x$E
	  U[xx$jumps+1,] <- x$U
	  S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  cumhazA <- cumsumstratasum(S0i,xx$strata,xx$nstrata,type="all")
	  cumhaz <- c(cumhazA$sum)
	  cumhazm <- cumhazA$lagsum
          ###
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Z <- xx$X
	  rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
	  rro <- c(exp(Z %*% coef(x) + xx$offset))
          S0star <- revcumsumstrata(rr/(1+rro*cumhazm),xx$strata,xx$nstrata)
          S0 <- revcumsumstrata(rr,xx$strata,xx$nstrata)
          S1 <- apply(Z*rr,2,revcumsumstrata,xx$strata,xx$nstrata)
	  Et <- S1/c(S0)
          lt <- apply((Z-Et)*c(rr*rro/(1+rro*cumhazm)),2,revcumsumstrata,xx$strata,xx$nstrata)
	  Estar <- S0star/S0
	  EstardLam <- cumsumstrata(Estar*S0i,xx$strata,xx$nstrata)
	  k <- exp(-EstardLam)
	  basecor <- apply(lt*c(k*S0i),2,revcumsumstrata,xx$strata,xx$nstrata)
	  basecor <- basecor/c(k*S0)
	  www <- x$propoddsW*x$weightsJ
	  U[xx$jumps+1,] <- U[xx$jumps+1,] - c(www)* basecor[xx$jumps+1,]
	  baseDLam0 <- apply(basecor*S0i,2,cumsumstrata,xx$strata,xx$nstrata)

	  ### Martingale  as a function of time and for all subjects to handle strata
	  MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0-baseDLam0)*rr*c(xx$weights)

	  if (orig.order) {
	     oo <- (1:nrow(xx$X))[xx$ord+1]
	     oo <- order(oo)
	     ### back to order of data-set
	     MGt <- MGt[oo,,drop=FALSE]
	     id <- xx$id[oo]
	  } else id <-  xx$id
  }# }}}

  ncluster <- NULL
  if (!cumsum) {
	  mid <- max(id)+1
    Mt <- sumstrata(dMGt,id,mid)
    cumhaz <- sumstrata(Lamt,id,mid)
    out <- list(residuals=c(Mt),cumhaz=c(cumhaz))
  } else {
     mid <- max(id)+1
     out <- data.frame(dMGt=c(dMGt),Lamt=Lamt,status=dN,
		time=c(xx$time),id=c(xx$id)+1,sign=xx$sign)
     dsort(out) <- ~time+status+id-sign
     Mt <- cumsumstrata(out$dMGt,out$id-1,mid)
     cumhaz <- cumsumstrata(out$Lamt,out$id-1,mid)
    out <- cbind(out,Mt,cumhaz)
    out <- subset(out,sign==1)
    ddrop(out)  <- ~sign+dMGt+Lamt
  }

  return(out)
} # }}}

##' Robust Baseline Hazard Standard Errors
##'
##' Computes robust (sandwich) standard errors for the cumulative baseline
##' hazard from a \code{phreg} object.
##'
##' @param x a \code{phreg} object.
##' @param type type of standard error (default \code{"robust"}).
##' @param fixbeta if non-NULL, fixes beta at given value.
##' @param ... additional arguments passed to \code{squareintHdM}.
##' @return A list with \code{cumhaz}, \code{se.cumhaz}, and \code{strata}.
##' @rdname phreg-helpers
##' @export
robust.basehaz.phreg  <- function(x,type="robust",fixbeta=NULL,...) {# {{{

  IsdM <- squareintHdM(x,ft=NULL,fixbeta=fixbeta,...)
  varA <-   IsdM$varInt[x$jumps]
  strata <- x$strata[x$jumps]
  cumhaz <- x$cumhaz
  se.cumhaz <- cbind(cumhaz[,1],varA^.5)
  colnames(se.cumhaz) <- c("time","se.cumhaz")

  return(list(cumhaz=cumhaz,se.cumhaz=se.cumhaz,strata=strata))
} # }}}

##' @export robust_phreg
robust_phreg  <- function(x,fixbeta=NULL,...) {

  if (is.null(fixbeta))
  if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0

 if (fixbeta==0)  {
    beta.iid <- iid(x)
    robvar <- crossprod(beta.iid)
 } else robvar <- beta.iid <- NULL
 baseline <- robust.basehaz.phreg(x,fixbeta=fixbeta,...);
 ## add arguments so that we can call basehazplot.phreg
 res <- c(x,list(beta.iid=beta.iid,robvar=robvar,robse.cumhaz=baseline$se.cumhaz))
 class(res) <- "phreg"
 return(res)
}

###}}}

###{{{ phreg: coef vcov print summary print.summary  plot lines

##' @export
coef.phreg  <- function(object,...) { ## {{{
  object$coef
} ## }}}

##' @export
vcov.phreg  <- function(object,...) {     ## {{{
 if (is.null(coef(object))) return(NULL)
 if ((length(class(object))==1) & inherits(object,"phreg")) {
  res <- object$var
  if (!is.null(res)) colnames(res) <- rownames(res) <- names(coef(object))
} else {
  res <- object$var
  if (!is.null(res)) colnames(res) <- rownames(res) <- names(coef(object))
}
  res
} ## }}}

##' @export
summary.phreg <- function(object,type=c("robust","martingale"),augment.type=c("var.augment.times","var.augmenti"),...) { ## {{{
  expC <- cc <- ncluster <- V <- NULL

   if (length(object$p)>0 & object$p>0 & (!object$no.opt)) {
    I <- -pinv(object$hessian)
    if ( (length(class(object))==2) && ( inherits(object,c("cifreg","recreg")))) {
	    V <- object$var
	    ncluster <- object$ncluster ## nrow(object$Uiid)
            if (!is.null(object$augmentation)) { V <- object[[augment.type[1]]]; }
    } else  {  ## phreg
	    V <- vcov(object,type=type[1])
            ncluster <- object$n
    }
    cc <- cbind(coef(object),diag(V)^0.5,diag(I)^0.5)
    cc  <- cbind(cc,2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
    colnames(cc) <- c("Estimate","S.E.","dU^-1/2","P-value")
    if (length(class(object))==1) if (!is.null(ncluster <- attributes(V)$ncluster))
    rownames(cc) <- names(coef(object))
    expC <- exp(lava::estimate(coef=coef(object),vcov=V)$coefmat[,c(1,3,4),drop=FALSE])
  }
  Strata <- levels(object$strata)
  n <- object$n
  res <- list(coef=cc,n=n,nevent=object$nevent,strata=Strata,ncluster=ncluster,var=V,exp.coef=expC)
  class(res) <- "summary.phreg"
  res
} ## }}}

##' Summarize Baseline Hazard from phreg
##'
##' Summarizes cumulative baseline hazard estimates from a phreg object,
##' optionally with robust standard errors.
##'
##' @param object a \code{phreg} object.
##' @param robust logical; if TRUE, uses robust standard errors.
##' @param ... additional arguments.
##' @return An object of class \code{"summary.recurrent"}.
##' @rdname phreg-helpers
##' @export
summarybase.phreg <- function(object,robust=FALSE,...) { ## {{{
  out <- summaryRecurrentobject(object,robust=robust,...)
  class(out) <- "summary.recurrent"
  return(out)
}# }}}


##' @export
print.phreg  <- function(x,...) { ## {{{
  cat("Call:\n")
  dput(x$call)
  cat("\n")

  Strata <- levels(x$strata)
  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- Strata; colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  print(nn, quote=FALSE)

  if (!is.null(x$coef) && length(x$coef) > 0) {
    cat("\ncoefficients:\n")
    ## V <- x$var
    ## if (!is.null(V)) {
    ##   cc <- cbind(coef(x), diag(V)^0.5)
    ##   cc <- cbind(cc, 2*(pnorm(abs(cc[,1]/cc[,2]), lower.tail=FALSE)))
    ##   colnames(cc) <- c("Estimate", "S.E.", "P-value")
    ##   rownames(cc) <- names(coef(x))
    ##   printCoefmat(cc, P.values=TRUE, has.Pvalue=TRUE, ...)
    ## } else {
    print(coef(x))
    ## }
  }
  cat("\n")
  invisible(x)
}  ## }}}

##' Plotting the baselines of stratified Cox
##'
##' Plotting the baselines of stratified Cox
##' @param x phreg object
##' @param ... Additional arguments to baseplot funtion
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases baseplot bplot  basecumhaz plotConfRegion  plotConfRegionSE plotstrata kmplot plotConfregion
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' out1 <- phreg(Surv(time,status==9)~vf+chf+strata(wmicat.4),data=TRACE)
##'
##' par(mfrow=c(2,2))
##' plot(out1)
##' plot(out1,stratas=c(0,3))
##' plot(out1,stratas=c(0,3),col=2:3,lty=1:2,se=TRUE)
##' plot(out1,stratas=c(0),col=2,lty=2,se=TRUE,polygon=FALSE)
##' plot(out1,stratas=c(0),col=matrix(c(2,1,3),1,3),lty=matrix(c(1,2,3),1,3),se=TRUE,polygon=FALSE)
##' @aliases plotConfRegion  plotConfRegionSE plotstrata kmplot plotConfregion
##' @export
plot.phreg  <- function(x,...)  baseplot(x,...)

##' @export
lines.phreg <- function(x,...,add=TRUE) plot(x,...,add=add)

##' @export
bplot <- function(x,...,add=TRUE) plot(x,...,add=add)

##' @export
print.summary.phreg  <- function(x,max.strata=5,...) { ## {{{

  if (length(class(x))==2 & inherits(x,"cifreg")) cat("Competing risks regression \n");
  if (!is.null(x$propodds)) {
       cat("Proportional odds model, log-OR regression \n");
  } else cat("\n")

  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- levels(x$strata); colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  if (length(x$strata)>max.strata) {
      nn <- rbind(c(colSums(nn),length(x$strata)));
      colnames(nn) <- c("n","events","stratas")
      rownames(nn) <- ""
  }
  print(nn,quote=FALSE)
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coefficients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    cat("exp(coefficients):\n")
    printCoefmat(x$exp.coef,...)
  }
  cat("\n")

} ## }}}

###}}}

##' Confidence Intervals with Transformations
##'
##' Computes confidence intervals using log or plain transformations,
##' with optional restrictions to positive values or probability scale.
##'
##' @param x point estimate(s).
##' @param std.err standard error(s).
##' @param conf.type type of transformation: \code{"log"} or \code{"plain"}.
##' @param restrict restriction: \code{"positive"}, \code{"prob"}, or \code{"none"}.
##' @param conf.int confidence level (default 0.95).
##' @return A list with \code{upper}, \code{lower}, \code{conf.type}, and \code{conf.int}.
##' @rdname phreg-helpers
##' @aliases conftype summarybase.phreg robust.basehaz.phreg
##' @export
conftype <- function(x,std.err,conf.type=c("log","plain"),restrict=c("positive","prob","none"),conf.int=0.95)
{ ## {{{
zval <- qnorm(1 - (1 - conf.int)/2, 0, 1)
 if (conf.type[1] == "plain") {# {{{
    temp1 <- x + zval * std.err
    temp2 <- x - zval * std.err
    if (restrict[1]=="prob") cix <- list(upper = pmin(temp1, 1), lower = pmax(temp2, 0), conf.type = "plain", conf.int = conf.int)
    if (restrict[1]=="positive") cix <- list(upper = pmax(temp1, 0), lower = pmax(temp2, 0), conf.type = "plain", conf.int = conf.int)
    if (restrict[1]=="none") cix <- list(upper = temp1, lower = temp2, conf.type = "plain", conf.int = conf.int)
 }
 if (conf.type[1] == "log") {
    xx <- ifelse(x <= 0, NA, x)
    temp1 <- ifelse(x <= 0, NA, exp(log(xx) + zval * std.err/xx))
    temp2 <- ifelse(x <= 0, NA, exp(log(xx) - zval * std.err/xx))
    if (restrict[1]=="prob") cix <- list(upper = pmin(temp1, 1), lower = temp2, conf.type = "log", conf.int = conf.int)
    else cix <- list(upper = temp1, lower = temp2, conf.type = "log", conf.int = conf.int)
 }
### if (conf.type[1] == "log-log") {
###    who <- (x == 0 | x == 1)
###    xx <- ifelse(who, NA, x)
###    temp1 <- exp(-exp(log(-log(xx)) + zval * std.err/log(xx)))
###    temp2 <- exp(-exp(log(-log(xx)) - zval * std.err/log(xx)))
###    if (restrict[1]=="prob") {
###	    temp3 <- ifelse(x == 0, NA, 1)
###	    temp1 <- ifelse(who, temp3, temp1)
###	    temp2 <- ifelse(who, temp3, temp2)
###    } else {
###    cix <- list(upper = temp1, lower = temp2, conf.type = "log-log", conf.int = conf.int)
###    }
### }# }}}
 return(cix)
}# }}}

basecumhaz <- function(x,type=c("list"),only=0,joint=0,cumhaz="cumhaz",se.cumhaz="se.cumhaz",robust=FALSE,columns=NULL,...) {# {{{
   ## all strata
   stratobs <- x$strata[x$jumps]
   ustratobs <- sort(unique(stratobs))
   stratas <- 0:(x$nstrata-1)
   ## default 
   if (is.null(columns)) columns <- 1:2

   se.cum <- cum <- c()
   se.cum <- cum <- x[[cumhaz]]
   if (robust==TRUE) {
	   ## take robse if there otherwise stay with se.cumhaz
	   secum <- x$robse.cumhaz
	   if (is.null(secum)) secum <- x[[se.cumhaz]]
   } else secum  <- x[[se.cumhaz]]
   if (is.null(secum)) nose <- TRUE else nose <- FALSE

   out <- rep(list(NULL),x$nstrata)
   if (length(ustratobs)>0)
   for (i in ustratobs) {
	   if (!is.null(x[[cumhaz]])) {
	   cumhazard <- x[[cumhaz]][stratobs==i,columns,drop=FALSE]
           nr <- nrow(cumhazard)
	   if (nr>=1) {
		   if (!nose) se.cum <- secum[stratobs==i,columns,drop=FALSE] else se.cum <- NULL
		   if (only==0) {
			  if (joint==0) out[[i+1]] <- list(cumhaz=cumhazard,se.cumhaz=se.cum,strata=i) else
		                        out[[i+1]] <- list(cumhaz=cbind(cumhazard,se.cum[,2],i))
	           } else out[[i+1]] <- cumhazard
         }
	 }
   }

   class(out) <- "basecumhaz"
   attr(out,"stratobs") <- ustratobs
   return(out)
}# }}}

baseplot  <- function(x,se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,
     lty=NULL,col=NULL,lwd=NULL,legend=TRUE,legend.args=list(),
     ylab="Cumulative hazard",xlab="time",
     polygon=TRUE,level=0.95,stratas=NULL,robust=FALSE,cumhaz="cumhaz",
     se.cumhaz="se.cumhaz",
     conf.type=c("log","plain"),restrict = c("positive","prob", "none"),...) {# {{{

###   if (!inherits(x,"phreg")) stop("must be phreg/recreg/ \n")

   base <- basecumhaz(x,joint=1,robust=robust,cumhaz=cumhaz,se.cumhaz=se.cumhaz)
   nstrata <- x$nstrata
   stratobs <- attr(base,"stratobs")
   ###
   if (is.null(stratas)) stratas <- stratobs else {
         wm <-   match(stratas,stratobs)
         wm <- wm[!is.na(wm)]
         if (length(wm)==0) stratas <- rep(0,0) else stratas <- stratobs[wm]
   }

   ## all confidence intervals for relevant strata  to make ylim
   if (length(stratobs)>0) {
   xx <- conftype(x$cumhaz[,2],x$se.cumhaz[,2],conf.type=conf.type[1],restrict=restrict[1],conf.int=level)

   if (is.null(xlim)) {
	   xlim <-  range(x$cumhaz[,1],na.rm=TRUE)
   }
   if (se) {
       if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz)) stop("phreg must be with cumhazard=TRUE\n");
       rrse <- range(c(xx$lower,xx$upper),na.rm=TRUE)
       if (is.null(ylim))
       {
        if (Inf %in% abs(rrse)) ylim <- range(x$cumhaz[,2],na.rm=TRUE) else ylim <- rrse
       }
   } else {
       if (is.null(ylim)) ylim <- range(x$cumhaz[,2],na.rm=TRUE)
   }

   ## all strata
   ## {{{
   ltys <- lty; cols <- col; lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   stratnames <- paste(stratn,lstrata,sep=":")

      if (!is.matrix(lty)) {
         if (is.null(lty)) lty <- ltys <- 1:length(stratas)
         if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas)) else ltys <- lty
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) col <- cols <- 1:length(stratas)
	 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas)) else cols <- col
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwd <- lwds <- rep(1,length(stratas))
	 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas)) else lwds <- lwd
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)
  ## }}}

  first <- 1
  j <- 1
  for (i in stratas)  {
	if (!is.null(base[[i+1]])) {
	cumhazard <- base[[i+1]]$cumhaz
	if (nrow(cumhazard)>1) {
        if (add | first==0) lines(cumhazard,type="s",lty=ltys[j,1],col=cols[j,1],lwd=lwds[j,1])
        else {
	  first <- 0
          plot(cumhazard,type="s",lty=ltys[j,1],col=cols[j,1],lwd=lwds[j,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
        }
        if (se==TRUE) {
            xx <- conftype(cumhazard[,2],cumhazard[,3],conf.type=conf.type[1],restrict=restrict[1],conf.int=level)
            nl <- cbind(cumhazard[,1],xx$lower); ul <- cbind(cumhazard[,1],xx$upper)
	      if (!polygon) {
		  lines(nl,type="s",lty=ltys[j,2],col=cols[j,2],lwd=lwds[j,2],...)
		  lines(ul,type="s",lty=ltys[j,3],col=cols[j,3],lwd=lwds[j,3],...)
	      } else plotConfRegion(nl[,1],cbind(nl[,2],ul[,2]),col=cols[j,1])
        }
     }
     }
    j <- j+1
  }

     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }
   } else {
	   if (!add) plot(0,0,ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
   }
}# }}}

###{{{ phreg: basehazplot.phreg predictphreg 

basehazplot.phreg  <- function(x,se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,
			       lwd=NULL,legend=TRUE,legend.args=list(),
                   ylab=NULL,xlab=NULL,polygon=TRUE,level=0.95,
			       stratas=NULL,robust=FALSE,conf.type=c("plain","log"),...) {# {{{
   if (inherits(x,"phreg") & is.null(ylab)) ylab <- "Cumulative hazard"
   if (is.null(xlab)) xlab <- "time"
   level <- -qnorm((1-level)/2)
   rr <- range(x$cumhaz[,-1])
   strat <- x$strata[x$jumps]
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$cumhaz[,1])
   if (se==TRUE) {
	   if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz)) stop("phreg must be with cumhazard=TRUE\n");
       if (conf.type[1]=="plain")
       rrse <- range(c(x$cumhaz[,-1]+level*x$se.cumhaz[,-1]))
       else { ## type="log"
	       relse <- exp(level*x$se.cumhaz[,-1]/x$cumhaz[,-1])
	       rrse <- range(x$cumhaz[,-1]/relse,x$cumhaz[,-1]*relse)
       }
       if (inherits(x,"km")) rrse <- c(min(x$lower,na.rm=TRUE),1)
       if (is.null(ylimo)) ylim <- rrse
   }

   ## all strata
   if (is.null(stratas)) stratas <- 0:(x$nstrata-1)
   ltys <- lty
   cols <- col
   lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   stratnames <- paste(stratn,lstrata,sep=":")

      if (!is.matrix(lty)) {
         if (is.null(lty)) lty <- ltys <- 1:length(stratas)
         if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas)) else ltys <- lty
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) col <- cols <- 1:length(stratas)
	 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas)) else cols <- col
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwd <- lwds <- rep(1,length(stratas))
	 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas)) else lwds <- lwd
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)


  first <- 0
  for (i in seq(stratas)) {
      j <- stratas[i]
        cumhazard <- x$cumhaz[strat==j,,drop=FALSE]
        if (!is.null(cumhazard)) {
	if (nrow(cumhazard)>1) {
        if (add | first==1)
        lines(cumhazard,type="s",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1])
       else {
	  first <- 1
          plot(cumhazard,type="s",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
       }
       if (se==TRUE) {
	    if (robust==TRUE) secumhazard  <- x$robse.cumhaz[strat==j,,drop=FALSE]
	    else secumhazard <- x$se.cumhaz[strat==j,,drop=FALSE]
	    xx <- cumhazard[,2]
	    std.err <- secumhazard[,2]
	 if (conf.type[1] == "log") {
	    temp1 <-  exp(log(xx) + level * std.err/xx)
	    temp2 <-  exp(log(xx) - level * std.err/xx)
	    ul = cbind(cumhazard[,1],temp1);
	    nl <- cbind(cumhazard[,1],temp2);
	 }
	 if (conf.type[1] == "plain") {
		 ul <-cbind(cumhazard[,1],cumhazard[,2]+level*secumhazard[,2])
		 nl <-cbind(cumhazard[,1],cumhazard[,2]-level*secumhazard[,2])
	 }
	 if (inherits(x,"km")) { ul[,2] <- x$upper[x$strata==j];
	                          nl[,2] <- x$lower[x$strata==j];
	                          wna <- which(is.na(ul[,2]))
	                          ul[wna,2] <- 0
	                          nl[wna,2] <- 0

	 }
	      if (!polygon) {
		  lines(nl,type="s",lty=ltys[i,2],col=cols[i,2],lwd=lwds[i,2],...)
		  lines(ul,type="s",lty=ltys[i,3],col=cols[i,3],lwd=lwds[i,3],...)
	      } else plotConfRegion(nl[,1],cbind(nl[,2],ul[,2]),col=cols[j+1])
        }
     }
     }
    }

     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }

}# }}}

###}}} plot

plotpredictphreg <- function(x,se=FALSE,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,
       type=c("surv","cumhaz","cif"),ylab=NULL,xlab=NULL,
       polygon=TRUE,whichx=NULL,robust=FALSE,...) {# {{{
   if (type[1]=="surv" & is.null(ylab)) ylab <- "Surival probability"
   if (type[1]=="cumhaz" & is.null(ylab)) ylab <- "Cumulative hazard"
   if (type[1]=="cif" & is.null(ylab)) ylab <- "Cumulative probability"

   if (is.null(xlim)) xlim <- range(x$times)
   if (is.null(xlab)) xlab <- "time"
   if (is.null(x$se.cumhaz) & se==TRUE)  {
	  warning("predict.phreg must be with se=TRUE\n");
          se <- FALSE
   }
   ylimo <- ylim
   nlower <- paste(type[1],".lower",sep="")
   nupper <- paste(type[1],".upper",sep="")
   xx <- x[[type[1]]]

   if (se==TRUE) {
      upper <- x[[nupper]]
      lower <- x[[nlower]]
      if (is.null(x$se.cumhaz)) stop("predict.phreg/cifreg/recreg must be with se=TRUE\n");
      rrse <- range(c(0,x[[nlower]],x[[nupper]]),na.rm=TRUE)
      if (is.null(ylimo)) ylim <- rrse
   } else {
      rrse <- range(c(xx),na.rm=TRUE)
      if (is.null(ylimo)) ylim <- rrse
   }

   ## all covriates
   nx <- nrow(x$surv)
   if (is.null(whichx)) whichx <- 1:nx
   stratas <- whichx
   ltys <- lty
   cols <- col

   if (length(whichx)>0 ) { ## with X

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(whichx) else
		 if (length(lty)!=length(whichx)) ltys <- rep(lty[1],length(whichx)) else ltys <- lty
      } else ltys <- lty

      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
   } else {
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)

  i <- 1
  j <- whichx[i]

   if (!add)
  plot(x$times,xx[j,],type="s",lty=ltys[i,1],col=cols[i,1],ylim=ylim,ylab=ylab,xlim=xlim,xlab=xlab,...)
  else lines(x$times,xx[j,],type="s", lty=ltys[i,1],col=cols[i,1],...)
  if (length(whichx)>1)
  for (i in seq(2,length(whichx))) lines(x$times,xx[whichx[i],],type="s",lty=ltys[i,1],col=cols[i,1],...)

    if (se==TRUE) {
    for (i in seq(1,length(whichx))) {
      j <- whichx[i]
      ul <- upper[j,]; nl <- lower[j,];
      if (!polygon) {
      lines(x$times,nl,type="s",lty=ltys[i,2],col=cols[i,2])
      lines(x$times,ul,type="s",lty=ltys[i,3],col=cols[i,3])
      } else plotConfRegion(x$times,cbind(nl,ul),col=cols[i,1])
    }
  }

  where <- "topleft";
  if (type[1]=="surv") where <-  "topright"

}# }}}

###{{{ predict.phreg print.predict plot.predict.phreg

##' Predictions from Proportional Hazards Model
##'
##' Computes predictions for survival probability, cumulative hazard, or risk at specified
##' time points for new data or existing data. Includes standard errors and confidence intervals.
##'
##' @param object Object of class \code{"phreg"}.
##' @param newdata Data frame for prediction. If \code{NULL}, predictions are made for the original data.
##' @param times Time points for prediction. Defaults to all unique event times in the model.
##' @param individual.time Logical; if \code{TRUE}, uses one time per subject (requires \code{newdata} and \code{times} to be same length).
##' @param tminus Logical; if \code{TRUE}, predicts at \eqn{t-} (strictly before \eqn{t}).
##' @param se Logical; if \code{TRUE}, computes standard errors and confidence intervals.
##' @param robust Logical; if \code{TRUE}, uses robust standard errors (default for most functions).
##' @param conf.type Transformation for survival estimates: \code{"log"} (default) or \code{"plain"}.
##' @param conf.int Confidence level (default 0.95).
##' @param km Logical; if \code{TRUE}, uses Kaplan-Meier product-limit for baseline; otherwise uses exponential of cumulative baseline.
##' @param ... Additional arguments for plotting functions.
##'
##' @return An object of class \code{"predictphreg"} containing:
##' \item{surr}{Matrix of survival probabilities.}
##' \item{cumhaz}{Matrix of cumulative hazards.}
##' \item{cif}{Matrix of cumulative incidence functions (if applicable).}
##' \item{times}{Vector of time points.}
##' \item{surv.upper, surv.lower}{Confidence bounds for survival.}
##' \item{RR}{Relative risks.}
##'
##' @author Thomas Scheike
##' @export
predict.phreg <- function(object,newdata,times=NULL,individual.time=FALSE,tminus=FALSE,se=TRUE,robust=FALSE,conf.type="log",conf.int=0.95,km=FALSE,...)
{# {{{ default is all time-points from the object
### take baseline and strata from object# {{{
ocumhaz <- object$cumhaz
jumptimes <- ocumhaz[,1]
chaz <- ocumhaz[,2]
if (is.null(object$nstrata)) {  ## try to make more robust
nstrata <- 1;
strata <- rep(1,length(jumptimes))
} else {
nstrata <- object$nstrata
strata <- object$strata[object$jumps]
}
if (length(jumptimes)==0) se <- FALSE
if (se) {
if (!robust) {
   se.chaz <- object$se.cumhaz[,2]
   varbeta <- object$ihessian
   if (!object$no.opt) Pt <- apply(object$E/c(object$S0),2,cumsumstrata,strata,nstrata)
   else Pt <- 0
} else { ## only relevant for phreg objects
  if ((object$no.opt) | is.null(object$coef)) fixbeta<- 1 else fixbeta <- 0
  IsdM <- squareintHdM(object,ft=NULL,fixbeta=fixbeta,...)
  ###
  se.chaz <-   IsdM$varInt[object$jumps]^.5
  covv <- IsdM$covv[object$jumps,,drop=FALSE]
  varbeta <- IsdM$vbeta
  Pt <- IsdM$Ht[object$jumps,,drop=FALSE]
}
} # }}}

### setting up newdata with factors and strata
if (!is.null(newdata)) 
x <- update_design(object$design,data = newdata,response=FALSE)
else x <- object$design
X <- x$x
if (!is.null(x$strata)) strataNew <- as.numeric(x$strata)-1 else strataNew <- rep(0,nrow(X))

if (is.null(times)) times <- sort(unique(c(object$exit)))
if (individual.time & is.null(times)) times <- c(object$exit)
if (individual.time & length(times)==1) times <- rep(times,length(object$exit))

    se.cumhaz <- NULL
    if (!individual.time) {
       pcumhaz <- surv <- matrix(0,nrow(X),length(times))
       if (se) se.cumhaz <- matrix(0,nrow(X),length(times))
    } else {
        pcumhaz <- surv <- matrix(0,nrow(X),1)
        if (se) se.cumhaz <- matrix(0,nrow(X),1)
    }
    hazt <- length(times)

    for (j in unique(strataNew)) {
        where <- predictCumhaz(c(0,jumptimes[strata==j]),times,type="left",tminus=tminus)
	plhazt <- hazt <- c(0,chaz[strata==j])
	if (km) { plhazt <- suppressWarnings(c(1,exp(cumsum(log(1-diff(hazt))))));  plhazt[is.na(hazt)] <- 0 }
	if (se) se.hazt <- c(0,se.chaz[strata==j])
	Xs <- X[strataNew==j,,drop=FALSE]
        if (object$p==0) RR <- rep(1,nrow(Xs)) else RR <- c(exp( Xs %*% coef(object)))
	if (se)  { # {{{ based on Hazard's
		if (object$p>0) {
			Ps <- Pt[strata==j,,drop=FALSE]
			Ps <- rbind(0,Ps)[where,,drop=FALSE]
			Xbeta <- Xs %*% varbeta
			seXbeta <- rowSums(Xbeta*Xs)^.5
			cov2 <- cov1 <- Xbeta %*% t(Ps*hazt[where])
		        if (robust)	{
			   covvs <- covv[strata==j,,drop=FALSE]
			   covvs <- rbind(0,covvs)[where,,drop=FALSE]
                           covv1 <- Xs %*% t((covvs*hazt[where]))
			   cov1 <- cov1-covv1
			}
		} else cov1 <- 0
	}# }}}
   haztw <- hazt[where]
   if (se) se.haztw <- se.hazt[where]
	if (is.null(object$propodds)) {
           plhaztw <- plhazt[where]
 	   if (!individual.time) pcumhaz[strataNew==j,]  <- RR%o%haztw else pcumhaz[strataNew==j,] <- RR*haztw[strataNew==j]
           if (!km) {
	     if (!individual.time) surv[strataNew==j,]  <- exp(- RR%o%haztw)
	     else surv[strataNew==j,]  <- exp(-RR*haztw[strataNew==j])
	   } else {
             if (!individual.time) surv[strataNew==j,]  <- exp( RR %o% log(plhaztw))
	     else surv[strataNew==j,]  <- plhaztw[strataNew==j]^RR
	   }
	} else {
	  if (!individual.time) surv[strataNew==j,]  <- 1/(1+RR%o%haztw)
          else surv[strataNew==j,]  <- 1/(1+RR*haztw[strataNew==j])
	}
	if (se) {# {{{
	    if (object$p>0)  {
	       if (!individual.time) se.cumhaz[strataNew==j,]  <-
		     ((RR %o% se.haztw)^2+(c(RR*seXbeta) %o% haztw)^2-2*RR^2*cov1)^.5
	        else se.cumhaz[strataNew==j,]  <- RR* (se.haztw^2+(c(seXbeta)*haztw)^2-2*diag(cov1))^.5
	    } else {
	       if (!individual.time) se.cumhaz[strataNew==j,]  <- RR %o% (se.haztw)
	        else se.cumhaz[strataNew==j,]  <- RR* se.haztw[strataNew==j]
	    }
	}# }}}
    }

  if (se)  {
     cibase <- conftype(pcumhaz,se.cumhaz,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
     cisurv <- conftype(surv,surv*se.cumhaz,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)
  } else {
     cibase <- list(upper=NULL,lower=NULL,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
     cisurv <- list(upper=NULL,lower=NULL,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
  }

 if (object$p>0) RR <-  exp(X %*% coef(object)) else RR <- rep(1,nrow(X))

### non-cox setting
if (!is.null(object$propodds)) pcumhaz <- -log(surv)

 out <- list(surv=surv,times=times,
    surv.upper=cisurv$upper,surv.lower=cisurv$lower,cumhaz=pcumhaz,se.cumhaz=se.cumhaz,
    se.surv=surv*se.cumhaz,se.cif=surv*se.cumhaz,
    cif=1-surv,cif.lower=1-cisurv$upper,cif.upper=1-cisurv$lower,
    cumhaz.upper=cibase$upper,cumhaz.lower=cibase$lower,strata=strataNew,X=X,RR=RR,id=x$cluster)

class(out) <- c("predictphreg",class(object)[1])
return(out)
}# }}}

##' @export
model.frame.phreg <- function(formula, data = NULL, ...) {
    if (is.null(data)) {
        return(formula$model.frame)
    }
    model.frame(formula$design$formula, data = data, ...)
}

##' @export
model.matrix.phreg <- function(object, data=NULL, ...) {
  if (is.null(data)) return(object$design$x)
  update_design(object$design, data=data, ...)$x
}

##' @export
summary.predictphreg <- function(object, type = c("surv", "cumhaz", "cif")[1],
                                 times = NULL, np = 10, extend = FALSE,
                                 digits = 4, ...) { ## {{{ 
  # ---- resolve time indices ----------------------------------------
  call.times <- times
  if (is.null(times)) {
    indexcol <- seq(ncol(object$surv))
    times    <- object$times
  } else {
    if (!is.numeric(times))
      stop("'times' must be numeric or NULL\n")
    if (extend) {
      indexcol <- predictCumhaz(c(0, object$times), times, return.index = TRUE)
    } else {
      indexcol <- predictCumhaz(object$times, times, return.index = TRUE)
      times[indexcol == 0] <- NA
      if (any(indexcol == 0))
        warning("Some requested times are before the first event time, returning NA\n")
    }
  }

  # ---- resolve subject (row) indices --------------------------------
  n_subj <- nrow(object[[type[1]]])
  if (is.null(np)) {
    ids <- seq(n_subj)
  } else {
    if (!is.numeric(np))
      stop("'np' must be row ids, a count, or NULL\n")
    if (length(np) > 1)       ids <- np           # explicit ids
    else if (np >= n_subj)    ids <- seq(n_subj)  # np >= data size
    else                      ids <- seq(np)       # first np rows
  }

  # ---- extract the requested quantity --------------------------------
  qty <- type[1]
  if (is.null(object[[qty]]))
    stop(sprintf("Type '%s' not found in predict object\n", qty))

  mat      <- object[[qty]]
  lower    <- object[[paste0(qty, ".lower")]]
  upper    <- object[[paste0(qty, ".upper")]]
  se.out   <- object[[paste0("se.", qty)]]

  # ---- extend with time-0 boundary if requested ---------------------
  if (extend) {
    boundary <- if (qty == "surv") 1 else 0
    mat    <- cbind(boundary, mat)
    se.out <- if (!is.null(se.out)) cbind(0, se.out) else NULL
    lower  <- if (!is.null(lower))  cbind(boundary, lower)  else NULL
    upper  <- if (!is.null(upper))  cbind(boundary, upper)  else NULL
  }

  # ---- slice and return ---------------------------------------------
  has_ci <- length(lower) > 1
  if (has_ci) {
    out <- list(
      pred    = mat[ids, indexcol, drop = FALSE],
      se.pred = se.out[ids, indexcol, drop = FALSE],
      lower   = lower[ids, indexcol, drop = FALSE],
      upper   = upper[ids, indexcol, drop = FALSE],
      times   = times,
      rows    = ids
    )
  } else {
    out <- list(
      pred  = mat[ids, indexcol, drop = FALSE],
      times = times,
      rows  = ids
    )
  }

  out$call.times <- call.times
  out$type       <- qty
  class(out)     <- "summary.predictphreg"
  out
} ## }}} 

##' @export
print.summary.predictphreg <- function(x, digits = 4, ...) { ## {{{
  has_ci <- !is.null(x$lower)
  cat(sprintf("Predictions of type '%s'\n", x$type))
  cat(sprintf("  Showing subjects: %s\n",
              paste(x$rows, collapse = ", ")))
  cat(sprintf("  Showing times:    %s\n\n",
              paste(round(x$times, digits), collapse = ", ")))

  for (i in seq_along(x$rows)) {
    cat(sprintf("-- Subject %d --\n", x$rows[i]))
    entry <- data.frame(time = round(x$times, digits),
                        pred = round(x$pred[i, ], digits))
    names(entry)[2] <- x$type
    if (has_ci) {
      entry$se    <- round(x$se.pred[i, ], digits)
      entry$lower <- round(x$lower[i, ],   digits)
      entry$upper <- round(x$upper[i, ],   digits)
    }
    print(entry, row.names = FALSE)
    cat("\n")
  }
  invisible(x)
} ## }}} 

##' @export
print.predictphreg <- function(x, type = c("surv", "cumhaz", "cif")[1],
                               times = NULL, np = 10, digits = 4, ...) { ## {{{ 
  # Run summary to get the sliced data
  s <- summary(x, type = type[1], times = times, np = np, ...)

  n_subj  <- nrow(x[[type[1]]])
  n_times <- ncol(x[[type[1]]])
  has_ci  <- !is.null(s$lower)

  cat(sprintf("Predictions of type '%s'\n", type[1]))
  cat(sprintf("  Subjects: %d total, showing %d\n", n_subj,  length(s$rows)))
  cat(sprintf("  Times:    %d total, showing %d\n", n_times, length(s$times)))
  if (has_ci) cat("  Confidence intervals available\n")
  cat("\n")

  # Build a long-format data frame: one row per (subject, time)
  rows <- s$rows
  tms  <- round(s$times, digits)
  pred <- s$pred

  df_list <- vector("list", length(rows))
  for (i in seq_along(rows)) {
    entry <- data.frame(
      subject = rows[i],
      time    = tms,
      pred    = round(pred[i, ], digits)
    )
    names(entry)[3] <- type[1]
    if (has_ci) {
      entry$lower <- round(s$lower[i, ], digits)
      entry$upper <- round(s$upper[i, ], digits)
      entry$se    <- round(s$se.pred[i, ], digits)
    }
    df_list[[i]] <- entry
  }
  out_df <- do.call(rbind, df_list)
  rownames(out_df) <- NULL

  print(out_df, row.names = FALSE)
  invisible(x)
} ## }}} 


##' @export
plot.predictphreg  <- function(x,se=FALSE,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,type=c("surv","cumhaz","cif"),ylab=NULL,xlab=NULL,
    polygon=TRUE,level=0.95,whichx=NULL,robust=FALSE,...) {# {{{
   if (type[1]=="cumhaz" & is.null(ylab)) ylab <- "Cumulative hazard"
   if (type[1]=="cif" & is.null(ylab)) ylab <- "Cumulative probability"
   if (type[1]=="surv" & is.null(ylab)) ylab <- "Surival probability"

   if (is.null(xlab)) xlab <- "time"
   if (type[1]=="cif") rr <- c(0,1)
   if (type[1]=="surv") rr <- c(0,1)
   if (type[1]=="cumhaz") rr <- range(c(0,x$cumhaz))
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$times)
   if (is.null(x$se.cumhaz) & se==TRUE)  {
	  warning("predict.phreg must be with se=TRUE\n");
          se <- FALSE
   }
   if (se==TRUE) {
      if (is.null(x$se.cumhaz)) stop("predict.phreg must be with se=TRUE\n");
   if (type[1]=="surv") rrse <- range(c(x$surv.upper,x$surv.lower,na.rm=TRUE))
   if (type[1]=="surv") rrse <- range(c(x$surv.upper,x$surv.lower,na.rm=TRUE))
   if (type[1]=="cumhaz") {
	   cumhaz.upper <- x$cumhaz+level*x$se.cumhaz
	   cumhaz.lower <- x$cumhaz-level*x$se.cumhaz
       rrse <- range(c(cumhaz.upper,cumhaz.lower),na.rm=TRUE)
   }
   if (type[1]=="surv") rrse <- c(0,1)
   if (type[1]=="cif") rrse <- c(0,1)
   if (type[1]=="cumhaz") rrse <- c(max(0,rrse[1]),rrse[2])
   if (is.null(ylimo)) ylim <- rrse
   }

   ## all covriates
   nx <- nrow(x$surv)
   if (is.null(whichx)) whichx <- 1:nx
   stratas <- whichx

   ltys <- lty
   cols <- col

   if (length(whichx)>0 ) { ## with X

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(whichx) else
		 if (length(lty)!=length(whichx)) ltys <- rep(lty[1],length(whichx)) else ltys <- lty
      } else ltys <- lty

      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
   } else {
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)

  i <- 1
  j <- whichx[i]

  recreg <- cifreg <- FALSE
  if ((length(class(x))==2) && (substr(class(x)[2],1,3)=="cif")) cifreg <- TRUE
###  if ((length(class(x))==2) && (substr(class(x)[2],1,3)=="rec")) recreg <- TRUE
  if (type[1]=="surv") {
	  xx <- x$surv
	  if (cifreg)  xx <- x$cif
	  if (recreg)  xx <- x$mean
  } else if (type[1]=="cif") {
	  xx <- 1-x$surv
  } else xx <- x$cumhaz
  if (se) {
     if (type[1]=="surv") {
	     upper <- x$surv.upper; lower <- x$surv.lower
	     if (cifreg) { upper <- x$cif.upper; lower <- x$cif.lower}
	     if (recreg) { upper <- x$mean.upper; lower <- x$mean.lower}
     } else if (type[1]=="cif") {
	     upper <- 1-x$surv.lower; lower <- 1-x$surv.upper
     } else { upper <- cumhaz.upper; lower <- cumhaz.lower}
  }

  if (!add)
  plot(x$times,xx[j,],type="s",lty=ltys[i,1],col=cols[i,1],ylim=ylim,ylab=ylab,xlim=xlim,xlab=xlab,...)
  else lines(x$times,xx[j,],type="s", lty=ltys[i,1],col=cols[i,1],...)
  if (length(whichx)>1)
  for (i in seq(2,length(whichx))) lines(x$times,xx[whichx[i],],type="s",lty=ltys[i,1],col=cols[i,1],...)

    if (se==TRUE) {
    for (i in seq(1,length(whichx))) {
      j <- whichx[i]
      ul <- upper[j,]; nl <- lower[j,];
      if (!polygon) {
      lines(x$times,nl,type="s",lty=ltys[i,2],col=cols[i,2])
      lines(x$times,ul,type="s",lty=ltys[i,3],col=cols[i,3])
      } else plotConfRegion(x$times,cbind(nl,ul),col=cols[i,1])
    }
  }

  where <- "topleft";
  if (type[1]=="surv") where <-  "topright"

}# }}}

###}}} predict

# {{{ Restricted mean for stratified Kaplan-Meier with martingale standard errors

##' Restricted Mean for Stratified Kaplan-Meier or Cox Model
##'
##' Computes the Restricted Mean Survival Time (RMST) for stratified Kaplan-Meier or
##' stratified Cox models with martingale standard errors.
##'
##' The standard error is computed using linear interpolation between standard errors at jump-times.
##' This allows plotting the restricted mean as a function of time.
##'
##' Years lost can be computed based on this and decomposed into years lost for different causes
##' using the \code{cif_yearslost} function.
##'
##' @param x Object of class \code{"phreg"}.
##' @param times Possible times for which to report restricted mean. If \code{NULL}, reports for all event times.
##' @param covs Possible covariates for Cox model adjustment.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"resmean_phreg"} containing:
##' \item{rmst}{Matrix of restricted mean survival times.}
##' \item{se.rmst}{Standard errors for RMST.}
##' \item{intkmtimes}{Restricted mean at specified times.}
##' \item{years.lost}{Years lost (if applicable).}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' out1 <- phreg(Surv(time, cause != 0) ~ strata(tcell, platelet), data = bmt)
##'
##' rm1 <- resmean_phreg(out1, times = 10 * (1:6))
##' summary(rm1)
##' e1 <- estimate(rm1)
##' par(mfrow = c(1, 2))
##' plot(rm1, se = 1)
##' plot(rm1, years.lost = TRUE, se = 1)
##'
##' ## Comparing populations
##' rm1 <- resmean_phreg(out1, times = 40)
##' e1 <- estimate(rm1)
##' estimate(e1, rbind(c(1, -1, 0, 0)))
##' @aliases rmst_phreg resmean_phreg
##' @export
resmean_phreg <- function(x,times=NULL,covs=NULL,...)
{# {{{
  ii <- invhess <- x$II

 if (!is.null(times)) {
	   tt <- expand.grid(times,0:(x$nstrata-1))
           mm <- cbind(x$jumptimes,x$strata.jumps,1)
           mm <- rbind(mm,cbind(tt[,1],tt[,2],0))
           ord <- order(mm[,1])
           mm <- mm[ord,]
	   phd <- which(mm[,3]==1)
	   newtimes <- which(mm[,3]==0)
	   S0i <- S0i2 <- rep(0,nrow(mm))
	   S0i[phd] <- c(1/x$S0)
	   S0ii2 <- c(1/(x$S0*(x$S0-1)))
	   S0ii2[x$S0==1] <- 0
	   S0i2[phd] <- S0ii2
           strata.jumps <- mm[,2]
	   nstrata <- x$nstrata;
	   jumptimes <- mm[,1]
	   E <- matrix(0,nrow(mm),x$p)
	   E[phd,] <- x$E
    }   else {
	   strata.jumps <- x$strata.jumps;
	   nstrata <- x$nstrata;
	   jumptimes <- x$jumptimes;
	   S0i <- c(1/x$S0)
	   S0i2 <- c(1/(x$S0*(x$S0-1)))
	   S0i2[x$S0==1] <- 0
	   E <- x$E
 }

  cumhaz <- cumsumstrata(S0i,strata.jumps,nstrata)
  var.cumhazMG <- cumsumstrata(S0i2,strata.jumps,nstrata)

  ## baseline survival
  km <- exp(cumsumstratasum(log(1-S0i),strata.jumps,nstrata,type="lagsum"))

  ## If covariates compute rr = exp( X beta)  and adjust km
  if (!is.null(covs)) { rr <- exp(sum(covs * coef(x))) } else rr <- 1
  km <- km^rr

  ## start integral in 0
  dtime <- c(diffstrata(jumptimes,strata.jumps,nstrata))
  intkm <-  cumsumstrata( c(km)*dtime,strata.jumps,nstrata)

  ### variance of baseline term
  var.intkmcumhaz <- cumsumstrata(intkm*S0i2,strata.jumps,nstrata)
  var.intkm2cumhaz <- cumsumstrata(intkm^2*S0i2,strata.jumps,nstrata)
  var.resmean <- intkm^2*var.cumhazMG+var.intkm2cumhaz-2*intkm*var.intkmcumhaz

   if (!is.null(x$coef)) {
      intp <- apply(E*S0i,2,cumsumstratasum,strata.jumps,nstrata,type="lagsum")
      intpS <-  apply(intp*c(km)*dtime,2,cumsumstrata,strata.jumps,nstrata)
      Dbeta <- -intpS
      if (!is.null(covs)) {
	      intLam <- cumsumstratasum(S0i,strata.jumps,nstrata,type="lagsum")
	      intLamS <-  cumsumstrata( c(intLam)*km*dtime,strata.jumps,nstrata)
	      Dbeta <- Dbeta + matrix(covs,nrow=nrow(intLam),ncol=length(covs),byrow=TRUE)*c(intLamS)
      }
      varbetat <-   rowSums((Dbeta %*% x$II)*Dbeta)
      var.resmean <- rr^2*(var.resmean+varbetat)
  } else {
      varbetat <- 0
  }

  time<- jumptimes
  meanm <- cbind(time,intkm)
  se.resmean <- var.resmean^.5
  se.resmean[var.resmean<0] <- 0
  se.mm <- cbind(time,se.resmean)

  ### make output at specified times
  if (!is.null(times)) {
    intkmtimes <- meanm[newtimes,,drop=FALSE]
    se.intkmtimes <- se.resmean[newtimes]
    skmtimes <- mm[newtimes,2]
    years.lost <- intkmtimes[,1]-intkmtimes[,2]
    intkmtimes <- cbind(skmtimes,intkmtimes,se.intkmtimes,years.lost)
    colnames(intkmtimes) <- c("strata","times","rmean","se.rmean","years.lost")
    rownames(intkmtimes) <- rep(x$strata.level,length(times));
    intkmtimes=data.frame(intkmtimes)
###    logintkmtimes=cbind(intkmtimes[,1:2],log(intkmtimes[,3]),se.intkmtimes/intkmtimes[,3])
###    colnames(logintkmtimes) <- c("strata","times","log-rmean","log-se.rmean")
     coef <- intkmtimes[,3];
     var  <- diag(intkmtimes[,4]^2)
  } else {
	  coef <- var <- intkmtimes <- se.intkmtimes <- NULL
  }

  rmst <- cbind(meanm[,1],vecAllStrata(meanm[,2],strata.jumps,nstrata))
  se.rmst <- cbind(se.mm[,1],vecAllStrata(se.mm[,2],strata.jumps,nstrata))

 out <- list(cumhaz=meanm,se.cumhaz=se.mm,covs=covs,
       time=time,strata=strata.jumps,nstrata=nstrata,
       jumps=1:length(km),strata.name=x$strata.name,
       strata.level=x$strata.level,
       intkmtimes=intkmtimes, rmst=rmst,se.rmst=se.rmst, coef=coef,var=var)
class(out) <- c("resmean_phreg")
return(out)
}# }}}

##' @export
vcov.resmean_phreg <- function(object,cause=1,...)
{# {{{
  if (is.null(object$intkmtimes)) stop("Only for the times given in the call 'times' \n");
  if (is.na(match("rmean",names(object$intkmtimes)))) name <- paste("se.intF_",cause,sep="") else name <- "se.rmean"
  if (!(name %in% colnames(object$intkmtimes))) stop( paste("Cause not consistent with names, looks for ",name,"\n"))

  rest <- object$intkmtimes[,name]
  inttimes <- object$intkmtimes[,2]
  times <- unique(object$intkmtimes[,2])

  if (length(times)==1)   {
   res <- rest 
   out <- diag(res^2, nrow=length(res))
   if (length(res) == length(object$strata.level)) rownames(out) <- colnames(out) <- object$strata.level
  } else {
	  out <- list()
	  k <- 1
          for (tt in times)  {
		  res <- rest[tt==inttimes]
                  if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
                 res <- diag(res^2, nrow=length(res))
                 if (nrow(res) == length(object$strata.level)) rownames(res) <- colnames(res) <- object$strata.level
		  out[[k]] <- res
		  k <- k+1
           }
  }

  return(out)
}# }}}

##' @export
coef.resmean_phreg <- function(object,cause=1,...) 
{# {{{
  if (is.null(object$intkmtimes)) stop("Only for the times given in the call 'times' \n");
  if (is.na(match("rmean",names(object$intkmtimes)))) name <- paste("intF_",cause,sep="") else name <- "rmean"
  if (!(name %in% colnames(object$intkmtimes))) stop( paste("Cause not consistent with names, looks for ",name,"\n"))
  rest <- object$intkmtimes[,name]
  inttimes <- object$intkmtimes[,2]
  times <- unique(object$intkmtimes[,2])

  if (length(times)==1)   {
   res <- rest 
   if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
   out <- res
  } else {
	  out <- list()
	  k <- 1
          for (tt in times)  {
		  res <- rest[tt==inttimes]
                  if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
		  out[[k]] <- res
		  k <- k+1
           }
  }

  return(out)
}# }}}

##' @export
estimate.resmean_phreg <- function(x,cause=1,...)
{# {{{
  if (is.null(x$intkmtimes)) stop("Only for the times given in the call 'times' \n");

  if (is.na(match("rmean",names(x$intkmtimes)))) name <- paste("se.intF_",cause,sep="") else name <- "se.rmean"

  coeft <- coef(x,cause=cause,...)
  vcovt <- vcov(x,cause=cause,...)
  times <- unique(x$intkmtimes[,2])
  if (length(times)==1)  
   out <- estimate(coef=coeft,vcov=vcovt,...)
  else {
	  out <- list()
          for (k in seq_along(times)) {
	  out[[k]] <- estimate(coef=coeft[[k]],vcov=vcovt[[k]],...)
	  }
  }

  return(out)
}# }}}



##' @export
rmst_phreg <- function(x,times=NULL,covs=NULL,...)
{# {{{
out <- resmean_phreg(x,times=times,covs=covs,...)
return(out)
}# }}}

##' Restricted Mean Time Lost for Competing Risks
##'
##' Computes the Restricted Mean Time Lost (RMTL) for competing risks based on the
##' integrated Aalen-Johansen estimator.
##'
##' A set of time points can be given to be returned in the summary, but the function
##' computes years-lost for all event times and can be plotted/viewed.
##' The RMTL for a specific time-point can also be computed using the \code{rmstIPCW} function.
##'
##' @param formula Formula for \code{phreg} object with \code{strata} to indicate strata, or \code{+1} if no strata.
##' @param data Data frame for calculations.
##' @param times Possible times for which to report restricted mean. Summary displays estimates for these times.
##' @param cens.code Censoring code (needed to separate event codes from censorings).
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"resmean_phreg"} containing:
##' \item{cumhaz}{Matrix of cumulative hazards (years lost).}
##' \item{se.cumhaz}{Standard errors.}
##' \item{intF1times}{Years lost at specified times.}
##' \item{causes}{Vector of cause codes.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##'
##' ## Years lost decomposed into causes
##' drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = c(40, 50))
##' par(mfrow = c(1, 2))
##' plot(drm1, cause = 1, se = 1)
##' plot(drm1, cause = 2, se = 1)
##' summary(drm1)
##' estimate(drm1, cause = 1)
##' estimate(drm1, cause = 2)
##'
##' ## Comparing populations
##' drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = 40)
##' summary(drm1, contrast = list(1:4))
##' e1 <- estimate(drm1)
##' estimate(e1, rbind(c(1, -1, 0, 0)))
##' @export
cif_yearslost <- function(formula,data=data,cens.code=0,times=NULL,...)
{# {{{
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
###  if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
  if (!inherits(Y,c("Event","Surv"))) stop("Expected a 'Surv' or 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }

  x <- phreg(formula,data=data,no.opt=TRUE,no.var=1,Z=as.matrix(status,ncol=1),...)
  causes <- sort(unique(x$cox.prep$Z[,1]))
  ccc <- which(causes %in% cens.code)
  if (length(ccc)>=1) causes <- causes[-ccc]

 if (!is.null(times)) {# {{{
	   tt <- expand.grid(times,0:(x$nstrata-1))
###           cause.jumps <- x$U+x$E
           cause.jumps <- x$cox.prep$Z[x$cox.prep$jumps+1,1]
           mm <- cbind(x$jumptimes,x$strata.jumps,1,cause.jumps)
           mm <- rbind(mm,cbind(tt[,1],tt[,2],0,cens.code))
           ord <- order(mm[,1])
           mm <- mm[ord,]
	   phd <- which(mm[,3]==1)
	   newtimes <- which(mm[,3]==0)
	   S0i <- S0i2 <- rep(0,nrow(mm))
	   S0i[phd] <- c(1/x$S0)
	   S0ii2 <- c(1/(x$S0*(x$S0-1)))
	   S0ii2[x$S0==1] <- 0
	   S0i2[phd] <- S0ii2
           strata.jumps <- mm[,2]
	   nstrata <- x$nstrata;
	   jumptimes <- mm[,1]
	   cause.jumps <- mm[,4]
    }   else {
	   strata.jumps <- x$strata.jumps;
	   nstrata <- x$nstrata;
	   jumptimes <- x$jumptimes;
           cause.jumps <- x$cox.prep$Z[x$cox.prep$jumps+1]
	   S0i <- c(1/x$S0)
	   S0i2 <- c(1/(x$S0*(x$S0-1)))
	   S0i2[x$S0==1] <- 0
 }# }}}

 ### formula from Pepe-Mori: SIM 93, 737-
 pepemori <- 0
 ### formula from PKA SIM 2013, not corect, missing some terms
 pka <- 0

  years.lostF1 <- se.years.lostF1m  <- se.years.lostF1 <- se.years.lostF1pm <- c()
  for (i in seq_along(causes)) {
	  jumpsi <- (cause.jumps==causes[i])*1
	  jumpsni <- (cause.jumps %in% causes[-i])*1
	  cumhaz <- cumsumstrata(S0i*jumpsi,strata.jumps,nstrata)
	  var.cumhazMG <- cumsumstrata(S0i2*jumpsi,strata.jumps,nstrata)
	  var.cumhazniMG <- cumsumstrata(S0i2*jumpsni,strata.jumps,nstrata)

	  ## baseline survival
	  km <- exp(cumsumstratasum(log(1-S0i),strata.jumps,nstrata,type="lagsum"))
	  F1 <- cumsumstratasum(jumpsi*km*S0i,strata.jumps,nstrata,type="lagsum")
	  ### 1 - F2  = S + F1
	  F1n1 <- km+F1

	  ## start integral in 0
	  dtime <- c(diffstrata(jumptimes,strata.jumps,nstrata))
	  intkm <-  cumsumstrata( c(km)*dtime,strata.jumps,nstrata)
	  intF1 <- cumsumstrata( c(F1)*dtime,strata.jumps,nstrata)

	  years.lostF1 <- cbind(years.lostF1,intF1)

	  if (pka==1) {
	  ### variance of baseline term  \int_0^t 1/Y^2 \int_s^t S(u) du  dN^i{{{
	  var.intkmcumhaz <- cumsumstrata(jumpsi*intkm*S0i2,strata.jumps,nstrata)
	  var.intkm2cumhaz <- cumsumstrata(jumpsi*intkm^2*S0i2,strata.jumps,nstrata)
	  var.resmean1 <- intkm^2*var.cumhazMG+var.intkm2cumhaz-2*intkm*var.intkmcumhaz

	  ### variance of baseline term  \int_0^t 1/Y^2 \int_s^t F_i(u) du  dN^-i
	  var.intF1cumhaz <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  var.intF12cumhaz <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  var.resmean2 <- intF1^2*var.cumhazniMG+var.intF12cumhaz-2*intF1*var.intF1cumhaz

	  var.intF1 <- (var.resmean1+var.resmean2)

          ## possibly negative due to rounding errors
	  var.intF1[var.intF1<0] <- 0
	  se.intF1 <- var.intF1^.5
	  se.years.lostF1 <- cbind(se.years.lostF1,se.intF1) # }}}
	  }

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s) - (t-s) (1-Fn1(s))) dN_1^i{{{
	  vars.1 <- cumsumstrata(jumpsi*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazMG+vars.1-2*intF1*vars.2

	  vars.3 <- cumsumstrata(jumpsi*F1n1^2*S0i2,strata.jumps,nstrata)
	  vars.4 <- cumsumstrata(jumpsi*F1n1^2*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsi*F1n1^2*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- jumptimes^2*vars.3+vars.4-2*jumptimes*vars.5

          vars.6 <- cumsumstrata(jumpsi*F1n1*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsi*F1n1*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsi*F1n1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsi*F1n1*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7
 	  varsj1 <- vars.11+vars.21-2*vars.31

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s) - (t-s) F1(s)) dN_n1^i
	  vars.1 <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazniMG+vars.1-2*intF1*vars.2

	  vars.3 <- cumsumstrata(jumpsni*F1^2*S0i2,strata.jumps,nstrata)
	  vars.4 <- cumsumstrata(jumpsni*F1^2*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsni*F1^2*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- jumptimes^2*vars.3+vars.4-2*jumptimes*vars.5

          vars.6 <- cumsumstrata(jumpsni*F1*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsni*F1*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsni*F1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsni*F1*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7
	  varsnj1 <- vars.11+vars.21-2*vars.31

	  varss <-  varsj1+varsnj1
          varss[varss<0] <- 0
	  se.intF1m <- varss^.5
	  se.years.lostF1m <- cbind(se.years.lostF1m,se.intF1m)
# }}}

	  if (pepemori==1) {
          ### variance Pepe-Mori {{{
          ### variance of dN_1 term  \int_0^t 1/Y^2   dN_n1^i
	  vars.1 <- cumsumstrata(jumpsi*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazMG+vars.1-2*intF1*vars.2

	  vars.4 <- cumsumstrata(jumpsi*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsi*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- F1n1^2*(jumptimes^2*var.cumhazMG+vars.4-2*jumptimes*vars.5)

          vars.6 <- cumsumstrata(jumpsi*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsi*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsi*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- F1n1*(jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7)
 	  varsj1 <- vars.11+vars.21-2*vars.31

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s)) dN_n1^i
	  vars.1 <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazniMG+vars.1-2*intF1*vars.2

	  varspm <- vars.11+varsj1

          varspm[varspm<0] <- 0
	  se.intF1pm <- varspm^.5
	  se.years.lostF1pm <- cbind(se.years.lostF1pm,se.intF1pm)
# }}}
	  }
 }
 years.lostF1 <- cbind(jumptimes,years.lostF1)
 colnames(years.lostF1) <- c("time",paste("intF_",causes,sep=""))

  ### make output at specified times
  if (!is.null(times)) {
    intF1times <- years.lostF1[newtimes,]
    years.lost <- apply(intF1times[,-1,drop=FALSE],1,sum)
    if (pka==1) se.intF1times <- se.years.lostF1[newtimes,-1]
    se.intF1mtimes <- se.years.lostF1m[newtimes,]
    stratatimes <- strata.jumps[newtimes]
    intF1mtimes <- cbind(stratatimes,intF1times,se.intF1mtimes,years.lost)
    colnames(intF1mtimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes),"total-years-lost")
    rownames(intF1mtimes) <- rep(x$strata.level,length(times));
    intF1mtimes=data.frame(intF1mtimes)
    if (pka==1) {
	    intF1pkatimes <- cbind(stratatimes,intF1times,se.intF1times)
            colnames(intF1pkatimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes))
            intF1pkatimes=data.frame(intF1times)
    } else intF1pkatimes <- NULL
    if (pepemori==1) {
          se.intF1pmtimes <- se.years.lostF1pm[newtimes,]
          intF1pmtimes <- cbind(stratatimes,intF1times,se.intF1pmtimes)
          colnames(intF1pmtimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes))
         intF1pmtimes=data.frame(intF1pmtimes)
    } else intF1pmtimes <- NULL
  } else coef <- var <- intF1pmtimes <-intF1mtimes <- intF1pkatimes <- NULL

 se.years.lostF1 <- cbind(jumptimes,se.years.lostF1m)
 colnames(se.years.lostF1) <- c("time",paste("intF_",causes,sep=""))

## name things to make use of other programs
 out <- list(cumhaz=years.lostF1,se.cumhaz=se.years.lostF1,
       time=jumptimes, strata=strata.jumps,nstrata=nstrata,
       jumps=1:length(km),strata.name=x$strata.name,
       strata.level=x$strata.level,
       intkmtimes=intF1mtimes, intF1times=intF1mtimes,
       intF1pmtimes=intF1pmtimes, intF1pkatimes=intF1pkatimes,causes=causes
       )
class(out) <- c("resmean_phreg")
return(out)
}# }}}

##' @export
summary.resmean_phreg <- function(object,level=0.95,contrast=NULL,...)
{# {{{

if (is.null(object$intkmtimes)) {
	p <- ncol(object$cumhaz)
	outl <- basecumhaz(object,columns=1:p) 
	outl$strata.name  <- object$strata.name
	outl$strata.level <- object$strata.level
	outl$causes       <- object$causes
} else  {

out <- object$intkmtimes
if (ncol(out)==5) {
   mu <- out[,3]; se <- out[,4];
   xx <- conftype(mu,se,conf.int=level,...)
   lower <- xx$lower
   upper <- xx$upper
   years.lost <- out[,5]
   out <- cbind(out[,-5],lower,upper,years.lost)
   if (!is.null(contrast)) {
	   test <- estimate(object,f=contrast)
	   out <- list(estimates=out,test=test)
   }
   outl <- out
} else {
	p <- ncol(out)
        nn <- names(out)[1:2]
        nl <- names(out)[p]
        outl <- list()
        oute <- list()
	k <- 1
	for (i in object$causes) {
                   name <- paste("intF_",i,sep="")
                   sename <- paste("se.intF_",i,sep="")
		   mu <- out[,name];
		   se <- out[,sename];
                   xx <- conftype(mu,se,conf.int=level,...)
		   loweri <- paste0("lower_",name)
		   upperi <- paste0("upper_",name)
		   out[,loweri] <- xx$lower
		   out[,upperi] <- xx$upper
		   xo <- out[,c(nn,name,sename)]
		   xo[,loweri] <- xx$lower
		   xo[,upperi] <- xx$upper
		   oute[[name]] <- xo
		   if (!is.null(contrast)) {
             test <- estimate(coef=mu,vcov=diag(se^2),f=contrast)
		   testname <- paste0("test",name)
		   outl[[testname]] <- test
		   }
	}
###        outl$estimate <- out
	outl$estimate <- oute
	outl$total.years.lost <- out[,nl]
}
}

return(outl)
}# }}}


##' @export
print.resmean_phreg <- function(x,...)
{# {{{
print(summary.resmean_phreg(x,...))
}# }}}

##' @export
plot.resmean_phreg <- function(x, se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,
                               lty=NULL,col=NULL,lwd=NULL,legend=TRUE,legend.args=list(),
                               ylab=NULL,xlab=NULL,
                               polygon=TRUE,level=0.95,stratas=NULL,robust=FALSE,years.lost=FALSE,cause=1,
                               conf.type=c("log","plain"),...) {# {{{

	if (inherits(x,"phreg") & is.null(ylab)) ylab <- "Cumulative hazard"
	if (inherits(x,"km") & is.null(ylab)) ylab <- "Survival probability"
	if (inherits(x,"cif") & is.null(ylab)) ylab <- "Probability"
	if (inherits(x,"resmean_phreg") & is.null(ylab)) ylab <- "Restricted residual mean life"
	if (inherits(x,"resmean_phreg") & !is.null(x$intF1times)) ylab <- "Years lost up to t: t - E(min(T,t))"
	if (years.lost) ylab <- "Years lost up to t: t - E(min(T,t))"
	if (is.null(xlab)) xlab <- "time"
	ci.level <- level
   level <- -qnorm((1-level)/2)
   if (years.lost) rr <- range(x$cumhaz[,1]-x$cumhaz[,cause+1])  else rr <- range(x$cumhaz[,cause+1])
   strat <- x$strata[x$jumps]
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$cumhaz[,1])
   if (se==TRUE) {
      if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz) )
		   stop("phreg must be with cumhazard=TRUE\n");
   if (years.lost) rrse <- range(c(x$cumhaz[,1]-x$cumhaz[,cause+1]+level*x$se.cumhaz[,cause+1])) else
       rrse <- range(c(x$cumhaz[,cause+1]+level*x$se.cumhaz[,cause+1]))
       if (is.null(ylimo)) ylim <- rrse
   }

   ## all strata
   if (is.null(stratas)) stratas <- 0:(x$nstrata-1)
   ltys <- lty
   cols <- col
   lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   ## stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   ## stratnames <- paste(stratn,lstrata,sep=":")
   stratnames <- lstrata

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(stratas) else if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas))
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwds <- rep(1,length(stratas)) else
		 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas))
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)

  if (years.lost) x$cumhaz[,cause+1] <- x$cumhaz[,1]-x$cumhaz[,cause+1]
  first <- 0
  for (i in seq(stratas)) {
      j <- stratas[i]
        cumhazard <- x$cumhaz[strat==j,c(1,cause+1),drop=FALSE]
        if (!is.null(cumhazard)) {
	if (nrow(cumhazard)>1) {
        if (add | first==1)
        lines(cumhazard,type="l",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1])
       else {
	  first <- 1
          plot(cumhazard,type="l",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
       }
       if (se==TRUE) {
	    if (robust==TRUE) secumhazard  <- x$robse.cumhaz[strat==j,c(1,cause+1),drop=FALSE]
	    else secumhazard <- x$se.cumhaz[strat==j,c(1,cause+1),drop=FALSE]
            xx <- conftype(cumhazard[,2],secumhazard[,2],conf.type=conf.type[1],restrict="positive",conf.int=ci.level)
	    ul <-cbind(cumhazard[,1],xx$upper)
	    nl <-cbind(cumhazard[,1],xx$lower)
	    ul[is.na(ul)] <- 0
	    nl[is.na(nl)] <- 0
		 if (inherits(x,"km")) { ul[,2] <- x$upper[x$strata==j];
		                          nl[,2] <- x$lower[x$strata==j];
		                          wna <- which(is.na(ul[,2]))
		                          ul[wna,2] <- 0
		                          nl[wna,2] <- 0

		 }
	      if (!polygon) {
		  lines(nl,type="l",lty=ltys[i,2],col=cols[i,2],lwd=lwds[i,2])
		  lines(ul,type="l",lty=ltys[i,3],col=cols[i,3],lwd=lwds[i,3])
	      } else {
                 ## type="l" confidence regions
		 ll <- length(nl[,1])
		 timess <- nl[,1]
		 tt <- c(timess,rev(timess))
		 yy <- c(nl[,2],rev(ul[,2]))
		 col.alpha<-0.1
		 col.ci<-cols[i,]
		 col.trans <- sapply(col.ci, FUN=function(x)
		   do.call(grDevices::rgb,as.list(c(grDevices::col2rgb(x)/255,col.alpha))))
		 polygon(tt,yy,lty=0,col=col.trans)
	      }
        }
     }
     }
    }
     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }

}# }}}


# }}}

##' IPTW Cox Regression (Inverse Probability of Treatment Weighted)
##'
##' Fits a Cox model with treatment weights \deqn{w(A) = \sum_a I(A=a)/\pi(a|X)}, where
##' \deqn{\pi(a|X) = P(A=a|X)}.
##'
##' Standard errors are computed via influence functions that are returned as the IID argument.
##' Propensity scores are fitted using either logistic regression (\code{glm}) or the multinomial
##' model (\code{mlogit}) when there are more than two treatment categories.
##'
##' The treatment variable must be a factor and is identified on the RHS of the \code{treat.model}.
##' Recurrent events can be considered with a start-stop structure, requiring \code{cluster(id)}.
##' Robust standard errors are computed in all cases.
##'
##' Time-dependent propensity score weights can be computed when \code{treat.var} is used. This weight
##' be 1 at the time of first (A_0) and 2nd treatment (A_1), then uses weights \deqn{w_0(A_0) * w_1(A_1)^{t>T_r}} where \deqn{T_r} is
##' time of 2nd randomization. The weights are constructed using a \code{glm} or \code{mlogit} model based on the data where 
##' \code{treat.var=1}. The propensity score can be constructed for any number of treatments in a similar manner. 
##'
##' @param formula Formula for \code{phreg}.
##' @param data Data frame for risk averaging.
##' @param treat.model Propensity score model (binary or multinomial).
##' @param treat.var A 1/0 variable indicating when treatment is given (for time-dependent weights).
##' @param weights Optional weights to multiply with the IPTW weights.
##' @param estpr (=1, default) to estimate propensity scores and include their uncertainty in the influence function.
##' @param pi0 Fixed simple weights (if \code{estpr=0}).
##' @param se.cluster To compute GEE-type standard errors when additional cluster structure is present.
##' @param ... Arguments for \code{phreg} call.
##'
##' @return An object of class \code{"phreg"} with additional IPTW components:
##' \item{IID}{Influence functions including propensity score uncertainty.}
##' \item{iptw}{IPTW weights used.}
##' \item{naive.var}{Naive variance ignoring propensity score uncertainty.}
##'
##' @author Thomas Scheike
##' @examples
##' data <- mets:::simLT(0.7, 100, beta = 0.3, betac = 0, ce = 1, betao = 0.3)
##' dfactor(data) <- Z.f ~ Z
##' out <- phreg_IPTW(Surv(time, status) ~ Z.f, data = data, treat.model = Z.f ~ X)
##' summary(out)
##' @export
phreg_IPTW <- function (formula, data,treat.model = NULL, treat.var = NULL,weights = NULL, estpr = 1, pi0 = 0.5,se.cluster=NULL,...)
{# {{{
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata)) strata.name <- names(des$xlevels)
    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL

 call.id <- id;
 conid <- construct_id(id,length(exit))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid

 ## take offset and weight first from formula, but then from arguments
 offset <- NULL
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit))
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit))
  } else weights <- des.weights


    if (!is.null(se.cluster)) {
        sds <- sort(unique(se.cluster))
        sid <- length(sds)
        if (is.numeric(se.cluster))
            se.cluster <- fast.approx(sds, se.cluster) - 1
        else se.cluster <- as.integer(factor(se.cluster, labels = seq(sid))) - 1
    }

    id <- id + 1
    nid <- length(unique(id))
    data$id__ <- id
    data$cid__ <- cumsumstrata(rep(1, length(id)), id - 1, nid)
    treats <- function(treatvar) {
        treatvar <- droplevels(treatvar)
        nlev <- nlevels(treatvar)
        nlevs <- levels(treatvar)
        ntreatvar <- as.numeric(treatvar)
        return(list(nlev = nlev, nlevs = nlevs, ntreatvar = ntreatvar))
    }
    fittreat <- function(treat.model, data, id, ntreatvar, nlev) {
        if (nlev == 2) {
            treat.model <- drop.specials(treat.model, "cluster")
            treat <- glm(treat.model, data, family = "binomial")
            iidalpha <- lava::iid(treat, id = id)
            lpa <- treat$linear.predictors
            pal <- expit(lpa)
            pal <- cbind(1 - pal, pal)
            ppp <- (pal/pal[, 1])
            spp <- 1/pal[, 1]
        }
        else {
            treat.modelid <- update.formula(treat.model, . ~ . + cluster(id__))
            treat <- mlogit(treat.modelid, data)
            iidalpha <- lava::iid(treat)
            pal <- predict(treat, data, se = 0, response = FALSE)
            ppp <- (pal/pal[, 1])
            spp <- 1/pal[, 1]
        }
        Xtreat <- model.matrix(treat.model, data)
        tvg2 <- 1 * (ntreatvar >= 2)
        pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
        pppy <- c(mdi(ppp, 1:length(ntreatvar), ntreatvar))
        Dppy <- (spp * tvg2 - pppy)
        Dp <- c()
        for (i in seq(nlev - 1)) Dp <- cbind(Dp, Xtreat * ppp[, i + 1] * Dppy/spp^2)
        DPai <- -1 * Dp/pA^2
        out <- list(iidalpha = iidalpha, pA = pA, Dp = Dp, pal = pal,
            ppp = ppp, spp = spp, id = id, DPai = DPai)
        return(out)
    }
    expit <- function(x) 1/(1 + exp(-x))
    if (!is.null(treat.var)) {
        weightWT <- data[, treat.var]
        whereW <- which(weightWT == 1)
        CountW <- cumsumstrata(weightWT, id - 1, nid)
        dataW <- data[whereW, ]
        idW <- id[whereW]
    } else {
        whereW <- 1:nrow(data)
        dataW <- data
        idW <- id
        CountW <- cumsumstrata(rep(1,nrow(data)), id-1,nid)
    }
    treat.name <- all.vars(treat.model)[1]
    treatvar <- dataW[, treat.name]
    if (!is.factor(treatvar))
        stop(paste("treatment=", treat.name, " must be coded as factor \n",
            sep = ""))
    treats <- treats(treatvar)
    wlPA <- ww <- rep(1, nrow(data))
    idWW <- mystrata2index(cbind(id, CountW))
    if (estpr[1] == 1) {
        fitt <- fittreat(treat.model, dataW, idW, treats$ntreatvar, treats$nlev)
        iidalpha0 <- fitt$iidalpha
        wPA <- c(fitt$pA)
        DPai <- fitt$DPai
    } else {
        if (length(pi0)==length(treats$ntreatvar)) wPA <- pi0
	else wPA <- ifelse(treats$ntreatvar==2,pi0[1],1-pi0[1])
        ## pi0 <- rep(pi0, treats$nlev)
        DPai <- matrix(0, nrow(data), 1)
    }
    ww <- rep(1, nrow(data))
    ww[whereW] <- wPA
    wlPA <- exp(cumsumstrata(log(ww), idWW - 1, attr(idWW, "nlevel")))
    wwt <- c(exp(cumsumstrata(log(ww), id - 1, nid)))
    ## P(t) = P_0 * P_1^(I(t>T1)), time-dependent weights
    ## DP = P(t) \sum_j P_j I(t> TJ) (-DP_j/P_j^2)
    if (estpr[1] == 1) {
        DPait <- matrix(0, nrow(data), ncol(DPai))
        DPait[whereW, ] <- DPai
        DPait <- apply(DPait * c(wlPA), 2, cumsumstrata, id - 1, nid)/wwt
    } else DPait <- NULL
    if (is.null(weights)) ww <- 1/wwt else ww <- weights/wwt
    phw <- phreg(formula, data, weights = ww, Z = DPait, ...)

###    check.derivative <- 0
###	if (check.derivative == 1) { ## {{{
###	### for checking derivative
###	fpar <- glm(treat.model,dataw,family=binomial)
###	mm <- model.matrix(treat.model,dataw)
###	cpar <- coef(fpar)
###
###	ff <- function(par,base=0) {
###	pa <-        expit(mm %*% par)
###	www <- ifelse(dataw[,treat.name] == "1", pa, 1 - pa)
###	ww <- rep(1, nrow(data))
###	ww[wherew] <- www
###	wlpa <- exp(cumsumstrata(log(ww), idww - 1, attr(idww, "nlevel")))
###	wwwt <- exp(cumsumstrata(log(ww), id - 1, nid))
###
###	pp <- phreg(formula,data,weights=1/wwwt,no.opt=true,beta=coef(phw))
###	if (base==1) po <- c(pp$cumhaz[,2]) else po <- pp$gradient
###	return(po)
###	}
###
###	print(ff(cpar))
###	gf <- jacobian(ff,cpar)
###	print(t(gf))
###        ###
###        print(ff(cpar,base=1))
###	gf <- jacobian(ff,cpar,base=1)
###	print(gf)
###	print("___________________________________");
###	} ## }}}

if (estpr[1] == 1) {
	xx <- phw$cox.prep
	nid <- max(xx$id)
	S0i <- rep(0, length(xx$strata))
	wPAJ <- xx$weights[xx$jumps+1]*xx$caseweights[xx$jumps+1]
	Xt <- xx$X
	S0 <- phw$S0 * wPAJ
	S0i[xx$jumps + 1] <- 1/S0
	U <- E <- matrix(0, nrow(xx$X), phw$p)
	U[xx$jumps + 1, ] <- phw$U/wPAJ
	E[xx$jumps + 1, ] <- phw$E


        if (phw$p>0) {
	rr <- c(xx$sign * exp(Xt %*% coef(phw) + xx$offset) * xx$weights)
	rrnw <- c(xx$sign * exp(Xt %*% coef(phw) + xx$offset))
        } else {
	rr <- c(xx$sign * exp(xx$offset) * xx$weights)
	rrnw <- c(xx$sign * exp(xx$offset))
        }
	DWX = .Call("vecMatMat", xx$Z, Xt)$vXZ
	S1 = apply(Xt * rr, 2, revcumsumstrata, xx$strata, xx$nstrata)
	###S00 = revcumsumstrata( rr, xx$strata, xx$nstrata)
	DS1 = apply(DWX * rrnw, 2, revcumsumstrata, xx$strata, xx$nstrata)
	DS0 = apply(xx$Z * rrnw, 2, revcumsumstrata, xx$strata, xx$nstrata)
	if (phw$p>0) {
		DS0S1 = .Call("vecMatMat", DS0[xx$jumps + 1, , drop = FALSE],S1[xx$jumps + 1, , drop = FALSE])$vXZ
		DUa2 <- apply(wPAJ * DS1[xx$jumps + 1, , drop = FALSE]/c(S0),2, sum) - apply(wPAJ * DS0S1/c(S0^2), 2, sum)
		DUa2 <- matrix(DUa2, ncol(fitt$DPai), phw$p)
		DUa1 <- t(xx$Z[xx$jumps + 1, ]) %*% (phw$U/wPAJ)
		DUa <- DUa1 - DUa2
		iidpal <- iidalpha0 %*% DUa
		iidbeta <- lava::iid(phw) + iidpal %*% phw$ihess
		phw$DUa <- DUa
		phw$IID <- iidbeta
		phw$naive.var <- phw$var
		phw$var <- crossprod(iidbeta)
	} else iidbeta <- NULL

        ###
	DAw <- apply(xx$caseweights[xx$jumps+1]*xx$Z[xx$jumps + 1, ,drop=FALSE]/c(S0),2,cumsumstrata,phw$strata.jumps,xx$nstrata)
	DA2 <- apply(wPAJ*DS0[xx$jumps + 1, , drop = FALSE]/c(S0)^2,2,cumsumstrata,phw$strata.jumps,xx$nstrata)
	DAt <- 1*(DAw-DA2)
	phw <- robust_phreg(phw,beta.iid=iidbeta)
	varA <- phw$robse.cumhaz[,2]^2
	phw$naive.se.cumhaz <- phw$robse.cumhaz
	vtheta <- crossprod(iidalpha0)
        varthetat <-   rowSums((DAt %*% vtheta)*DAt)
        ###
        ww <- xx$caseweights*xx$weights
	S0i2 <- rep(0,length(xx$strata))
        S0i2[xx$jumps+1] <- 1/(phw$S0^2*ww[xx$jumps+1])
        cumS0i2 <- c(cumsumstrata(S0i2,xx$strata,xx$nstrata))
        xxx <- (S0i-rr*cumS0i2)
	id <- xx$id
	mid <- max(xx$id)+1
        thetat <- iidalpha0[id+1,,drop=FALSE]
        covk1 <- apply(S0i*thetat,2,cumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="sum")
        covk2 <- apply(rr*thetat,2,revcumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="lagsum")
        ###
        covk1 <- apply(S0i*thetat,2,cumsumstrata,xx$strata,xx$nstrata)
        covk2 <- apply(rr*thetat,2,revcumsumstrata,xx$strata,xx$nstrata)
        ###
        covk2 <- covk2*cumS0i2
        covv <- (covk1-covk2)[xx$jumps+1,,drop=FALSE]
        covvt <- 2*apply(covv*DAt,1,sum)
        varA <- varA+varthetat+2*apply(covv*DAt,1,sum)

	Ht <- NULL
	if (phw$p>0) {
		covbetatheta <- t(iidalpha0) %*%  iidbeta
		Ht <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
		Ht <- Ht[xx$jumps+1,,drop=FALSE]
		covvBT <- 2* rowSums((Ht %*% t(covbetatheta))*DAt)
		varA <- varA + covvBT
	}
	phw$robse.cumhaz[,2] <- varA^.5
	phw$se.cumhaz[,2] <- varA^.5
	phw$var.cumhaz <- cbind(phw$se.cumhaz[,1],varA)
	resAiid <- list(DAt=DAt,Ht=Ht,iidalpha0=iidalpha0,iidbeta=phw$IID)
	phw$resAiid <- resAiid
	phw$iptw <- ww
} else {
   phw$iptw <- ww
   phw <- robust_phreg(phw)
   phw$naive.se.cumhaz <- phw$se.cumhaz
   phw$se.cumhaz <- phw$robse.cumhaz
   phw$naive.var <- phw$var
}


if (is.matrix(phw$IID))
	if (nrow(phw$IID)==length(name.id)) rownames(phw$IID) <- name.id

if (!is.null(se.cluster)) {
	phw$IID.simple <- phw$IID
	phw$var.simple <- phw$var
	phw$IID <- apply(phw$IID,2,sumstrata,se.cluster,sid)
	phw$var <- crossprod(phw$IID)
	phw$se.cluster <- se.cluster
}
class(phw) <- c("phreg","IPTW")

return(phw)
}# }}}


##' G-Estimator for Cox and Fine-Gray Models
##'
##' Computes the G-estimator (G-formula) for standardized survival or cumulative incidence estimates:
##' \deqn{ \hat S(t, A=a) = n^{-1} \sum_i \hat S(t, A=a, Z_i) }
##'
##' Based on a \code{phreg} or \code{cifreg} object. Provides influence functions for these risk estimates,
##' allowing for standard error computation.
##'
##' If the first covariate is a factor, contrasts between all levels are computed automatically.
##' If it is continuous, specific values must be provided via \code{Avalues}.
##'
##' @param x Object of class \code{"phreg"} or \code{"cifreg"}.
##' @param data Data frame for risk averaging. Must be part of the data used for fitting unless \code{same.data=FALSE}.
##' @param time Time point for estimation.
##' @param Avalues Values to compare for the first covariate \eqn{A}.
##' @param varname Name of the variable to be treated as the treatment/exposure variable (default is the first variable).
##' @param same.data Logical; assumes the same data is used for fitting and averaging.
##' @param First Logical; if \code{TRUE}, uses only the first record for G-averaging (useful for start-stop structures).
##'
##' @return An object of class \code{"survivalG"} containing:
##' \item{risk}{Standardized risk estimates.}
##' \item{risk.iid}{Influence functions for the risk estimates.}
##' \item{difference}{Pairwise differences in risks.}
##' \item{ratio}{Risk ratios.}
##' \item{survival.ratio}{Survival ratios (for \code{phreg}).}
##' \item{survival.difference}{Survival differences (for \code{phreg}).}
##'
##' @author Thomas Scheike
##' @aliases survivalGtime
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' bmt$event <- (bmt$cause != 0) * 1
##' bmt$id <- 1:408
##' dfactor(bmt) <- tcell.f ~ tcell
##'
##' # Fine-Gray model
##' fg1 <- cifregFG(Event(time, cause) ~ tcell.f + platelet + age, bmt,
##'               cause = 1, cox.prep = TRUE)
##' summary(survivalG(fg1, bmt, 50))
##'
##' \donttest{ ## Reduce Ex.Timings
##' # Cox model
##' ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age, bmt)
##' summary(survivalG(ss, bmt, 50))
##'
##' # Stratified Cox model
##' ss <- phreg(Surv(time, event) ~ strata(tcell.f) + platelet + age, bmt)
##' summary(survivalG(ss, bmt, 50))
##'
##' # Time-varying G-estimates
##' sst <- survivalGtime(ss, bmt, n = 50)
##' plot(sst)
##'
##' # Among treated (specify id to link influence functions)
##' ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age + cluster(id), bmt)
##' summary(survivalG(ss, subset(bmt, tcell == 1), 50))
##' }
##' @aliases survivalGtime
##' @export
survivalG <- function(x,data,time=NULL,Avalues=NULL,varname=NULL,same.data=TRUE,First=FALSE)
{# {{{
if (is.null(time)) stop("Give time for estimation of survival/cumulative incidence\n")
if (inherits(x,c("cifreg","phreg","recreg")))
Aiid <- iidBaseline(x,time=time) else stop("Must be cifregFG/phreg/recreg object \n");

### dealing with first variable
if (is.null(varname))  {
   treat.name <- all.vars(update.formula(x$formula,1~.))[1]
} else treat.name <- varname
treatvar <- data[,treat.name]

Alevels <- NULL
if (is.factor(treatvar)) {
   ## treatvar, levels to look at
   if (is.null(Avalues)) Alevels <- levels(treatvar) else Alevels <- Avalues
} else {
   Alevels <- Avalues
}
if (is.null(Alevels)) stop("must either have a factor as first variable or give Avalues for risk average")

## for recreg take first record only of each subject
cid <- countID(data.frame(id=x$id))
if (inherits(x,"recreg") | First) {
    FirstId <- which(cid$Countid==1)
    data <- data[FirstId,]
}

desx <- update_design(x$design,data=data)
if (is.null(desx$cluster)) { ## data must be same as data used for x
   id <- x$id
} else {
  id <- desx$cluster-1
}
datA <- data


k <- 1; risks <- c(); DariskG <- list()
for (a in Alevels) { ## {{{
   datA[,treat.name] <- a
   pp <- predict(x,datA,time,se=0)
   Xbase <- 1*outer(pp$strata,0:(x$nstrata-1),"==")

 if (inherits(x,"cifreg")) {
    ps0 <- c(pp$cif);
    Dma  <- cbind(c(pp$RR)*c(1-ps0)*Xbase,c((1-ps0)*pp$cumhaz)*pp$X);
 } else if (inherits(x,"recreg")) {
    ps0 <- pp$cumhaz;
    Dma <-  cbind(c(pp$RR)*Xbase,c(pp$cumhaz)*pp$X)
 } else if (inherits(x,"phreg"))  {
    ps0 <- c(pp$surv);
    Dma  <- -cbind(c(pp$RR)*ps0*Xbase,c(ps0*pp$cumhaz)*pp$X);
 }
 risks <- cbind(risks,ps0)
 DariskG[[k]] <- apply(Dma,2,sum)
 k <- k+1
} ## }}}

predictAiid <- NULL
###
Grisk <- apply(risks,2,mean)
risk.iid  <- t(t(risks)-Grisk)
### order after all id in data
nid <- max(x$id)+1
ndata <- length(unique(id))

## after id.data, if sub-data then still ok sorted after id.data=0:nid
risk.iid <- apply(risk.iid,2,sumstrata,id,nid)/ndata

## sorted after x$id, so is risk.iid
coxiid <- cbind(Aiid$base.iid,Aiid$beta.iid)

if (same.data) {
   for (a in seq_along(Alevels)) risk.iid[,a] <- risk.iid[,a]+ coxiid %*% DariskG[[a]]/ndata
   vv <- crossprod(risk.iid)
} else {
   predictAiid <- matrix(0,ndata,ncol(risks))
   for (a in seq_along(Alevels))  {
###	risk.iid[,a] <- risk.iid[,a]
	predictAiid[,a] <- coxiid %*% DariskG[[a]]/ndata
   }
   risk.iid <- rbind(risk.iid,predictAiid)
   vv <- crossprod(risk.iid)
}

if (inherits(x,"cifreg") | inherits(x,"recreg")) {
out <- estimate(coef=Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
ed <- estimate(coef=Grisk,vcov=vv,f=function(p) (p[-1])-(p[1]))
rd <- estimate(coef=Grisk,vcov=vv,f=function(p) log(p[-1]/p[1]))
out <- list(risk.iid=risk.iid,risk=out,difference=ed,ratio=rd,vcov=vv)
} else if (inherits(x,"phreg"))  {
out <- estimate(coef=1-Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
sout <- estimate(coef=Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
ed <- estimate(coef=Grisk,vcov=vv,f=function(p) (1-p[-1])-(1-p[1]))
rd <- estimate(coef=Grisk,vcov=vv,f=function(p) log((1-p[-1])/(1-p[1])))
srd <- estimate(coef=Grisk,vcov=vv,f=function(p) log(p[-1]/p[1]))
ssd <- estimate(coef=Grisk,vcov=vv,f=function(p) p[-1]-p[1])
out <- list(risk.iid=risk.iid,survivalG=sout,risk=out,difference=ed,ratio=rd,
	    survival.ratio=srd, survival.difference=ssd,vcov=vv)
}

class(out) <- "survivalG"
attr(out,"levels") <- Alevels
return(out)
} ## }}}

##' @export
estimate.survivalG  <- function(x,...)
{
out <-estimate(coef=x$risk$coefmat[,1],IC=x$risk.iid*nrow(x$risk.iid),...)
return(out)
}

###{{{ summary  survivalG

##' @export
print.survivalG <- function(x,...) {
  print(summary(x,...))
}

##' @export
summary.survivalG <- function(object,...) {
  res <- list(risk=object$risk,difference=object$difference,ratio=object$ratio,
	      survival.ratio=object$survival.ratio,survival.difference=object$survival.difference)
  class(res) <- "summary.survivalG"
  res
}

##' @export
print.summary.survivalG  <- function(x,...) {
    cat("G-estimator :\n")
    print(x[["risk"]],...)
    cat("\n")

    cat("Average Treatment effect: difference (G-estimator) :\n")
    print(x$difference,...)
    cat("\n")

    cat("Average Treatment effect: ratio (G-estimator) :\n")
    cat("log-ratio: \n")
    print(x$ratio$coefmat,...)
    cat("ratio: \n")
    ratio <- exp(x$ratio$coefmat[,c(1,3:4)])
    print(ratio,...)
    cat("\n")

    if (!is.null(x$survival.difference)) {
    cat("Average Treatment effect:  survival-difference (G-estimator) :\n")
    print(x$survival.difference$coefmat,...)
    cat("\n")
    }

    if (!is.null(x$survival.ratio)) {
    cat("Average Treatment effect: 1-G (survival)-ratio (G-estimator) :\n")
    cat("log-ratio: \n")
    print(x$survival.ratio$coefmat,...)
    cat("ratio: \n")
    ratio <- exp(x$survival.ratio$coefmat[,c(1,3:4)])
    print(ratio,...)
    cat("\n")
    }

}

###}}} summary

##' @export
survivalGtime <- function(x,data,time=NULL,n=100,...)
{# {{{

if (is.null(time)) {
       rr <- range(x$cumhaz[,1])
       if (!is.null(n)) time <- seq(rr[1],rr[2],length=n) else time <- x$cumhaz[,1]
}

survivalGt <- risk <- difference <- ratio <- survival.ratio <- c()

for (tt in time) {
  Gt <- survivalG(x,data,time=tt,...)
  strata <- strata(rownames(Gt$risk$coefmat))
  survivalGt <- rbind(survivalGt,cbind(tt,Gt$survivalG$coefmat))
  risk <- rbind(risk,cbind(tt,Gt$risk$coefmat))
  difference <- rbind(difference,cbind(tt,Gt$difference$coefmat))
  ratio <- rbind(ratio,cbind(tt,exp(Gt$ratio$coefmat)))
  if (!is.null(Gt$survival.ratio$coefmat))
  survival.ratio <- rbind(survival.ratio,cbind(tt,exp(Gt$survival.ratio$coefmat)))
}
  colnames(survivalGt)[1] <- "time"
  colnames(risk)[1] <- "time"
  colnames(difference)[1] <- "time"
  colnames(ratio)[1] <- "time"
  if (!is.null(survival.ratio))
  colnames(survival.ratio)[1] <- "time"
  strata <- strata(rownames(risk))
out <- list(time=time,survivalG=survivalGt,risk=risk,difference=difference,
	    ratio=ratio,survival.ratio=survival.ratio,strata=strata)

class(out) <- "survivalGtime"
return(out)
} ## }}}

##' @export
plot.survivalGtime <- function(x,type=c("survival","risk","survival.ratio","difference","ratio"),ylim=NULL,legend=NULL,...) {# {{{

  ## to deal with fine-gray based things, that do not contain survival estimates
  if ((ncol(x$survivalG)==1) & type[1]=="survival") type <- "risk"
  us <- unique(x$strata)
  cols <- 1:length(us)
  ltys <- cols
  ss0 <- x$strata==us[1]

  if (type[1]=="survival") {
  if (is.null(ylim))  ylim  <- range(x$survivalG[,c(4,5)])
  plot(x$time,x$survivalG[ss0,2],type="s",xlab="time",ylab="Survival",col=cols[1],lty=ltys[1],ylim=ylim,...)
  plotConfRegion(x$time,x$survivalG[ss0,c(4,5)],col=cols[1])

  k <- 2
for (ss in us[-1]) {
   ss0 <- x$strata==ss
   lines(x$time,x$survivalG[ss0,2],type="s",ylim=c(0,1),col=cols[k],lty=ltys[k])
   plotConfRegion(x$time,x$survivalG[ss0,c(4,5)],col=cols[k])
   k <- k+1
}
  }

  if (type[1]=="risk") {
   if (is.null(ylim))  ylim  <- range(x$risk[,c(4,5)])
   plot(x$time,x$risk[ss0,2],type="s",xlab="time",ylab="risk",col=cols[1],lty=ltys[1],ylim=ylim,...)
   plotConfRegion(x$time,x$risk[ss0,c(4,5)],col=cols[1])
   k <- 2
   for (ss in us[-1]) {
   ss0 <- x$strata==ss
   lines(x$time,x$risk[ss0,2],type="s",ylim=c(0,1),col=cols[k],lty=ltys[k])
   plotConfRegion(x$time,x$risk[ss0,c(4,5)],col=cols[k])
   k <- k+1
   }
  }

  if (type[1]=="difference")  {
	  if (is.null(ylim))  ylim  <- range(x$risk[,c(4,5)])
  plot(x$time,x$difference[,2],type="s",xlab="time",ylab="difference in risk",ylim=range(x$difference[,2]),col=cols[1],lty=ltys[1],...)
  plotConfRegion(x$time,x$difference[,c(4,5)],col=cols[1])
  }
  if (type[1]=="ratio")  {
  if (is.null(ylim))  ylim  <- range(x$ratio[,c(4,5)])
  plot(x$time,x$ratio[,2],type="s",ylim=ylim,xlab="time",ylab="ratio",col=cols[1],lty=ltys[1],...)
  plotConfRegion(x$time,x$ratio[,c(4,5)],col=cols[1])
}
 if (type[1]=="survival.ratio")  {
   if (is.null(ylim))  ylim  <- range(x$survival.ratio[,c(4,5)])
  plot(x$time,x$survival.ratio[,2],type="s",xlab="time",ylab="risk",col=cols[1],lty=ltys[1],ylim=ylim,...)
  plotConfRegion(x$time,x$survival.ratio[,c(4,5)],col=cols[1])
}

 if (is.null(legend)) legend("topleft",levels(x$strata),col=cols,lty=ltys)

}
# }}}

##' Fast Additive Hazards Model with Robust Standard Errors
##'
##' Fits a fast Lin-Ying additive hazards model with a possibly stratified baseline.
##' Robust variance is the default variance estimate in the summary.
##'
##' Influence functions (IID) follow the numerical order of the given cluster variable.
##' Ordering by \code{$id} aligns the IID terms with the dataset order.
##'
##' @param formula Formula with a 'Surv' outcome (similar to \code{coxph}).
##' @param data Data frame.
##' @param no.baseline Logical; if \code{TRUE}, fits the model without baseline hazard estimation.
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"aalenMets"} (extends \code{"phreg"}) containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{iid}{Influence functions.}
##' \item{intZHZ}{Integrated ZHZ matrix.}
##' \item{gamma}{Coefficient estimates.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' out <- aalenMets(Surv(time, cause == 1) ~ tcell + platelet + age, data = bmt)
##' summary(out)
##'
##' ## Comparison with timereg::aalen
##' ## out2 <- timereg::aalen(
##' ##   Surv(time, cause == 1) ~ const(tcell) + const(platelet) + const(age),
##' ##   data = bmt
##' ## )
##' ## summary(out2)
##' @export
aalenMets <- function(formula,data=data,no.baseline=FALSE,...)
{# {{{
formula.call <- formula
x <- phreg(formula,data=data,no.opt=TRUE,...)
xx <- x$cox.prep

### computation of intZHZ matrix, and gamma and baseline ##################
# {{{
eXb <- c(xx$sign) * c(xx$weights)
S0 = c(revcumsumstrata(eXb,xx$strata,xx$nstrata))
S0[S0==0] <- 1
E=apply(eXb*xx$X,2,revcumsumstrata,xx$strata,xx$nstrata)/S0;
if (no.baseline) E <- 0*E
###if (!no.int) E <- cbind(1,E)
###if (no.int) X <-  xx$X else X <- cbind(1,xx$X)
###if (no.int) XX <-  xx$XX else { XX <- .Call("vecCPMat",X)$XX }
X <- xx$X; XX <- xx$XX
###
S2=apply(eXb*XX,2,revcumsumstrata,xx$strata,xx$nstrata)
E2  <- .Call("vecCPMat",E)$XX
###
dts <- c(diffstrata(xx$time,xx$strata,xx$nstrata))
dt <- diff(c(0,xx$time))
###
intZbar <- apply(E*dts,2,cumsumstrata,xx$strata,xx$nstrata)
intZHZt <- apply((S2-S0*E2)*dts,2,cumsum)
p <- ncol(E)
intZHZ <-  matrix(.Call("XXMatFULL",tail(intZHZt,1),p,PACKAGE="mets")$XXf,p,p)

IintZHZ  <-  pinv(intZHZ)
intZHdN <- matrix(x$gradient,ncol(E),1)
XJ <- X[xx$jumps+1,]
if (no.baseline) intZHdN <- matrix(apply(XJ,2,sum),ncol(E),1)
gamma <- IintZHZ %*% intZHdN
###if (!no.int) nn <- c("int",colnames(x$X)) else
nn <- colnames(x$X)
rownames(gamma)  <-  nn
x$cumhaz[,2] <- x$cumhaz[,2]- intZbar[xx$jumps+1,] %*% gamma
# }}}

### iid gamma #########################################
# {{{
id <- xx$id
if (no.baseline==FALSE) {
  mm <-  X * c(X %*% gamma) * c(xx$time)+ apply( E* c(E %*% gamma)*dts,2,cumsumstrata,xx$strata,xx$nstrata) -  X* c(intZbar %*% gamma) - c(X %*% gamma)* intZbar
  mm <- c(xx$weights*xx$sign) * mm
  mm <- apply(mm,2,sumstrata,id,max(id)+1)
  ###
  MGt <- t(x$hessian %*% t(lava::iid(x))) + mm
  XJ2l  <- .Call("vecCPMat",x$U)$XX
  XJ2l <- matrix(apply(XJ2l,2,sum),nrow=1)
  varmg <-  matrix(.Call("XXMatFULL",XJ2l,p,PACKAGE="mets")$XXf,p,p)
  varmg <- IintZHZ %*% varmg %*% IintZHZ
} else {
  mm <-  X * c(X %*% gamma) * c(xx$time)
  mm <- c(xx$weights*xx$sign) * mm
  mm <- apply(mm,2,sumstrata,id,max(id)+1)
  XJ2l  <- .Call("vecCPMat",XJ)$XX
  XJ2l <- matrix(apply(XJ2l,2,sum),nrow=1)
  varmg <-  matrix(.Call("XXMatFULL",XJ2l,p,PACKAGE="mets")$XXf,p,p)
  varmg <- IintZHZ %*% varmg %*% IintZHZ
  XdN  <- apply(XJ,2,sumstrata,id[xx$jumps+1],max(id)+1)
  MGt <-  XdN-mm
}
iid <-  MGt %*% IintZHZ
# }}}

### output  #########################################
# {{{
coef <- c(gamma)
names(coef) <- nn
x$coef <- coef
x$var <- crossprod(iid)
x$varmg <- varmg
x$iid <- iid
x$intZHdN <- intZHdN
x$intZHZ  <-  intZHZ
x$formula <- formula.call
x$ihessian <- IintZHZ
## score of gamma
x$gradient <- c(intZHdN-intZHZ %*% gamma)

x$no.opt <- FALSE
class(x) <- c(class(x),"aalenMets")
# }}}

return(x)
}# }}}

##' Kaplan-Meier with Robust Standard Errors
##'
##' Computes Kaplan-Meier estimates with robust standard errors.
##' Robust variance is the default and is obtained from the \code{predict} call.
##'
##' @param formula Formula with 'Surv' or 'Event' outcome.
##' @param data Data frame.
##' @param km Logical; if \code{TRUE}, returns Kaplan-Meier estimates; otherwise returns Nelson-Aalen based estimates.
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"km"} (extends \code{"predictphreg"}) containing:
##' \item{surr}{Survival probabilities.}
##' \item{se.surv}{Standard errors.}
##' \item{lower, upper}{Confidence intervals.}
##'
##' @author Thomas Scheike
##' @examples
##' data(sTRACE)
##' sTRACE$cluster <- sample(1:100, 500, replace = TRUE)
##' out1 <- km(Surv(time, status == 9) ~ strata(vf, chf), data = sTRACE)
##' out2 <- km(Surv(time, status == 9) ~ strata(vf, chf) + cluster(cluster), data = sTRACE)
##'
##' summary(out1, times = 1:3)
##' summary(out2, times = 1:3)
##'
##' par(mfrow = c(1, 2))
##' plot(out1, se = TRUE)
##' plot(out2, se = TRUE)
##' @export
km <- function(formula,data=data,km=TRUE,...)
{# {{{
res <- phreg(formula,data=data,...)

rhs <- update(formula,-1~.)
varss <- all.vars(formula)
## find all strata
first <- c(headstrata(res$strata.call,res$nstrata))

ddf <- data[first,varss]
pres <- predict(res,ddf,robust=TRUE,km=km,...)
pres$formula <- formula
pres$call <- match.call()

attr(pres,"data") <- ddf
class(pres) <- c("km","predictphreg")
return(pres)
}# }}}

##' @export
summary.km <- function(object,times=NULL,type=c("cif","cumhaz","surv")[3],...) { ## {{{
   out <- summary.predictphreg(object,times=times,type=type[1],...)
   return(out)
} ## }}}

##' @export
plot.km <- function(x,...) { ## {{{
   plot.predictphreg(x,...)
}# }}}

##' Cumulative Incidence with Robust Standard Errors
##'
##' Computes cumulative incidence functions with robust standard errors.
##'
##' @param formula Formula with 'Event' outcome and \code{strata} (only!).
##' @param data Data frame.
##' @param cause Cause of interest (default is \code{NULL}, which looks at all causes).
##' @param cens.code Censoring code (default is \code{"0"}).
##' @param death.code Alternative to \code{cens.code}; specifies codes of death.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"cif"} (extends \code{"phreg"}) containing:
##' \item{cumhaz}{Cumulative incidence estimates.}
##' \item{se.cumhaz}{Standard errors.}
##' \item{cause}{Cause of interest.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$cluster <- sample(1:100, 408, replace = TRUE)
##' out1 <- cif(Event(time, cause) ~ +1, data = bmt, cause = 1)
##' out2 <- cif(Event(time, cause) ~ +1 + cluster(cluster), data = bmt, cause = 1)
##'
##' par(mfrow = c(1, 2))
##' plot(out1, se = TRUE)
##' plot(out2, se = TRUE)
##' @export
cif <- function(formula,data=data,cause=1,cens.code=0,death.code=NULL,...)
{# {{{
  cl <- match.call()
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata)) strata.name <- names(des$xlevels)
    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  id.orig <- id;
  if (!is.null(id)) {
	  ids <- sort(unique(id))
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
   } else id <- as.integer(seq_along(exit))-1;

  statusE <- 1*(status %in% cause)
  if (is.null(death.code)) statusD <- 1*(!(status %in% cens.code)) else statusD <- 1*(status %in% death.code)
  data$statusE__ <- statusE
  data$statusD__ <- statusD
  data$strata__ <- strata

  ### setting up formulae for the two phreg (cause of interest and death)
  if (is.null(id.orig)) {
     formid <- update.formula(formula,~.+cluster(id))
     data$id <- id
  } else formid <- formula
  tt <- terms(formid)
  tt <- delete.response(tt)
  formid <- formula(tt)

  data$exit__ <- exit
  if (ncol(Y)==3) {
     data$entry__ <- entry
     formE <- as.formula(paste("Surv(entry__,exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(entry__,exit__,statusD__)~+1"))
  } else {
     formE <- as.formula(paste("Surv(exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(exit__,statusD__)~+1"))
  }
 formE <- update.formula(formE,formid)
 formD <- update.formula(formD,formid)

  if (sum(statusE)==0) warning("No events of type 1\n");
  coxE <- phreg(formE,data=data,...)
  coxS <- phreg(formD,data=data,...)

  ### cif
  if (sum(statusE)>0) cifo <- recurrentMarginalPhreg(coxE,coxS) else cifo <- coxE
  ## to work with predict function
  ##  cifo$no.opt <- TRUE

  ### to use basehazplot.phreg
  class(cifo) <- c("cif","phreg")
  attr(cifo,"cause") <- cause
  attr(cifo,"cens.code") <- cens.code
  attr(cifo,"death.code") <- death.code
  return(cifo)
}# }}}

##' @export
plot.cif <- function(x,se=FALSE,ylab=NULL,ylim=c(0,1),conf.type=c("log","plain"),...) { ## {{{
   if (inherits(x,"cif") & is.null(ylab)) ylab <- "Probability"
   baseplot(x,se=se,ylab=ylab,ylim=ylim,restrict="prob",conf.type=conf.type[1],...)
} ## }}}

##' @export
summary.cif <- function(object,se=FALSE,ylab=NULL,times=NULL,conf.type=c("log","plain"),...) { ## {{{
   if (inherits(object,"cif") & is.null(ylab)) ylab <- "Probability"
   out <- summaryRecurrentobject(object,name="probability",times=times,conf.type=conf.type[1],restrict="prob",...)
   return(out)
} ## }}}

##' Proportional Odds Survival Model
##'
##' Fits a semiparametric proportional odds model where:
##' \deqn{ \mbox{logit}(S(t|x)) = \log(\Lambda(t)) + x \beta }
##' Thus, covariate effects represent the odds ratio (OR) of survival.
##'
##' This is equivalent to using a hazards model:
##' \deqn{ Z \lambda(t) \exp(x \beta) }
##' where \eqn{Z} is gamma distributed with mean and variance 1.
##'
##' @param formula Formula with 'Surv' outcome (similar to \code{coxph}).
##' @param data Data frame.
##' @param offset Offsets for \eqn{\exp(x \beta)} terms.
##' @param weights Weights for score equations.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"phreg"} with \code{propodds=1}.
##'
##' @author Thomas Scheike
##' @references
##' Eriksson, Frank, Li, Jianing, Scheike, Thomas, and Zhang, Mei-Jie (2015).
##' "The proportional odds cumulative incidence model for competing risks."
##' \emph{Biometrics}, 71(3), 687--695.
##'
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' out1 <- logitSurv(Surv(time, status == 9) ~ vf + chf + strata(wmicat.4), data = TRACE)
##' summary(out1)
##' gof(out1)
##' plot(out1)
##' @export
logitSurv <- function(formula,data,offset=NULL,weights=NULL,...)
{# {{{
   out <- phreg(formula,data,offset=offset,weights=weights,propodds=1,...)
   out$se.cumhaz <- NULL
   return(out)
}# }}}

## {{{ Utility functions, cumsumstrata ....

tailstrata <- function(strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("tailstrataR",length(strata),strata,nstrata,PACKAGE="mets")
if (any(res$found<0.5))  { warning("Not all strata found");  cat((1:nstrata)[res$found>0.5]); }
return(res$where)
}# }}}

headstrata <- function(strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("headstrataR",length(strata),strata,nstrata,PACKAGE="mets")
if (any(res$found<0.5))  { warning("Not all strata found");  cat((1:nstrata)[res$found>0.5]); }
return(res$where)
}# }}}

##' Stratified Cumulative and Summary Operations
##'
##' Low-level helper functions for computing sums, cumulative sums, and
##' reverse cumulative sums within strata groups, as well as matrix
##' double-indexing.
##'
##' @name strata-numeric
##' @param x numeric vector (or matrix for \code{matdoubleindex}).
##' @param strata integer vector of strata indices (0-based).
##' @param nstrata number of distinct strata.
##' @param rows row indices for \code{matdoubleindex}.
##' @param cols column indices for \code{matdoubleindex}.
##' @param xvec optional values to assign at indexed positions.
##' @param ... additional arguments (for \code{mdi}).
##' @return Numeric vector of the same length as \code{x} (or modified matrix).
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases sumstrata cumsumstrata revcumsumstrata revcumsum matdoubleindex mdi
NULL

##' @rdname strata-numeric
##' @export
sumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("sumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
cumsumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("cumsumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @export
diffstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("diffstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
revcumsumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("revcumsumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

revcumsum2strata <- function(x,strata,nstrata,strata2,nstrata2,lag=FALSE)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
res <- .Call("revcumsum2strataR",as.double(x),strata,nstrata,strata2,nstrata2,PACKAGE="mets")
return(res)
}# }}}

revcumsum2stratafdN <- function(x,y,strata,nstrata,strata2,nstrata2,startx)
{# {{{
if (any(strata<0) | any(strata>nstrata-1))    stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
if (length(x)!=length(y)) stop("length of x and y must be same\n");
res <- .Call("revcumsum2stratafdNR",as.double(x),as.double(y),strata,nstrata,strata2,nstrata2,as.double(startx),PACKAGE="mets")
return(res)
}# }}}


cumsum2strata <- function(x,y,strata,nstrata,strata2,nstrata2,startx)
{# {{{
if (any(strata<0) | any(strata>nstrata-1))    stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
res <- .Call("cumsum2strataR",as.double(x),as.double(y),strata,nstrata,strata2,nstrata2,as.double(startx),PACKAGE="mets")
return(res)
}# }}}


##' @export
vecAllStrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
res <- .Call("vecAllStrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
revcumsum <- function(x)
{# {{{
res <- .Call("revcumsumR",x,PACKAGE="mets")$res
return(res)
}# }}}

revcumsumstratasum <- function(x,strata,nstrata,type="all")
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
if (type=="sum")    res <- .Call("revcumsumstratasumR",x,strata,nstrata)$sum
if (type=="lagsum") res <- .Call("revcumsumstratasumR",x,strata,nstrata)$lagsum
if (type=="all")    res <- .Call("revcumsumstratasumR",x,strata,nstrata)
return(res)
}# }}}

cumsumstratasum <- function(x,strata,nstrata,type="all")
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n"); 
if (type=="sum")    res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,0)$sum
if (type=="lagsum") res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,0)$lagsum
if (type=="all")    res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,1)
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
matdoubleindex <- function(x,rows,cols,xvec=NULL)
{# {{{
if (!is.matrix(x)) stop("x must be matrix")
ncols <- ncol(x); nrows <- nrow(x)
## to avoid warnings when going to C, get rid of Inf
cols[cols==Inf] <- ncol(x)+1;
rows[rows==Inf] <- nrow(x)+1;
if (length(rows)==1) rows <- rep(rows,length(cols))
if (length(cols)==1) cols <- rep(cols,length(rows))
if (length(cols)!=length(rows)) stop("rows and cols different lengths\n");
if (is.null(xvec)) { assign <- 0; xvec <- 1} else {
	assign <- 1;
        if (length(cols)!=length(xvec)) stop("rows and cols and xvec differ \n");
}
res <- .Call("Matdoubleindex",x,rows-1,cols-1,length(cols),assign,xvec)$mat
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
mdi <- function(x,...) matdoubleindex(x,...)

covfr  <- function(x,y,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfR",x,y,strata,nstrata)
return(res)
}# }}}

revcumsumidstratasum <- function(x,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$sum
if (type=="lagsum")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$lagsum
if (type=="lagsumsquare") res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$lagsumsquare
if (type=="all")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)
return(res)
}# }}}

revcumsumidstratasumCov <- function(x,y,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$sum
if (type=="lagsum")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$lagsum
if (type=="lagsumsquare") res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$lagsumsquare
if (type=="all")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

cumsumidstratasumCov <- function(x,y,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")   res <- .Call("cumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$sum
else res <- .Call("cumsumidstratasumCovR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

cumsumidstratasum <- function(x,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")   res <- .Call("cumsumidstratasumR",x,id,nid,strata,nstrata)$sum
else res <- .Call("cumsumidstratasumR",x,id,nid,strata,nstrata)
return(res)
}# }}}

covfridstrata  <- function(x,y,id,nid,strata,nstrata)
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfstrataR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

covfridstrataCov  <- function(x,y,x1,y1,id,nid,strata,nstrata)
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfstrataCovR",x,y,x1,y1,id,nid,strata,nstrata)
return(res)
}# }}}


## }}}


Mcumsumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("McumsumstrataR", A, strata, as.integer(nstrata), FALSE)$res
}

Mrevcumsumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("McumsumstrataR", A, strata, as.integer(nstrata), TRUE)$res
}


Msumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("MsumstrataR", A, strata, as.integer(nstrata))$res
}

library("tinytest")
library(mets) 
###
set.seed(100)
###
data(hfactioncpx12)
hf <- hfactioncpx12
## break ties after cause and censoring
hf <- tie_breaker(hf,id="id",cause=c(1,0),cens.code=5)
hf$idn <- hf$id
hf$x <- as.numeric(hf$treatment) 
hf$z <- rnorm(741)[hf$id]
hf$nn <- 1:nrow(hf)
hf$idc <- 5+sample(400,741,TRUE)[hf$id]
###
hf$id2 <- sample(1:741,741)[hf$id]
hf$ido <- hf$id
hf$id <- NULL
###
hf2 <- dsort(hf,~id2+entry+time,return.order=TRUE)

test_phregClust <- function() {
## {{{  works  phreg 

####################################################################################

gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(idn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z+cluster(idn),data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
###
gg <- survivalG(gl,hf,time=1,First=TRUE)
gg2 <- survivalG(gl2,hf2,time=1,First=TRUE)
d3 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d4 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d5 <- max(abs(t(head(gg$risk.iid)- head(gg2$risk.iid))))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
d6 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d5 < 0.000000001 ))
expect_true( (d4< 0.000000001) & ( d5 < 0.000000001 ))
expect_true( (d6< 0.000000001) )
c(d1,d2,d3,d4,d5,d6)


gl <- phreg(Event(entry,time,status==1)~treatment+z,data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4,d5)
b1 <- iidBaseline(gl,time=1)


gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
d5 <- max(abs(t(head(gg$risk.iid)- head(gg2$risk.iid))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d1
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
## data her skal svare til fit 
gg2 <- survivalG(gl2,hf,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d2
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
d5 <- max(abs(gg$risk.iid-gg2$risk.iid))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

###gg <- survivalG(gl,hf2,time=1); gg2 <- survivalG(gl2,hf2,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###d2
######
###gg <- survivalG(gl,hf2,time=1); gg2 <- survivalG(gl2,hf,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###cat("survivalG på data i anden order end fit")
###d2
######
###gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf2)
###gg <- survivalG(gl,hf,time=1); gg2 <- survivalG(gl2,hf2,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###d2

## }}}
}
test_phregClust()

test_recregClust <- function() {
## {{{  works recreg, cifreg, 

gl <- recreg(Event(entry,time,status)~treatment+z+cluster(idn),data=hf,cause=1,death.code=2)
###
gl2 <- recreg(Event(entry,time,status)~treatment+z+cluster(idn),data=hf2,cause=1,death.code=2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat))
d2 <- max(abs(iid(gl)-iid(gl2)))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d5 <- max(abs(gg$risk.iid-gg2$risk.iid))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
###
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( d5 < 0.000000001)
c(d1,d2,d3,d4,max(diid),d5)


gl <- recreg(Event(entry,time,status)~treatment+z+cluster(id2),data=hf,cause=1,death.code=2)
###
gl2 <- recreg(Event(entry,time,status)~treatment+z+cluster(id2),data=hf2,cause=1,death.code=2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
#
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4,max(diid))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)



   ## censoring stratified after treatment 
     gl <- recreg(Event(entry,time,status)~treatment+cluster(idn),data=hf,
     cause=1,death.code=2,cens.model=~strata(treatment))
      gl2 <- recreg(Event(entry,time,status)~treatment+cluster(idn),data=hf2,
     cause=1,death.code=2,cens.model=~strata(treatment))
     d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
     d2 <- max(abs(head(iid(gl))- head(iid(gl2))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
 

####################################################################################


gl <- cifregFG(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1)
gl2 <- cifregFG(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
class(gl)
d2 <- max(abs(iid(gl)- iid(gl2)))
head(iid(gl))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
###
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))

gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
d4 <- max(abs(cbind(b1$base.iid,b1$beta.iid)- cbind(b2$base.iid,b2$beta.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))
max(abs(gl2$MGciid-gl$MGciid))



gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z,data=hf,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
d4 <- max(abs(cbind(b1$base.iid,b1$beta.iid)- cbind(b2$base.iid,b2$beta.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))
max(abs(gl2$MGciid-gl$MGciid))

############################ ok også her 

gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z,data=hf2,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
gg <- survivalG(gl,hf2,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

## }}}
}
test_recregClust() 

test_binregClust <- function() {
## {{{  works binreg, binregG, binregATE 

gl <- binreg(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
head(iid(gl))
head(iid(gl))
head(bbg$risk.iid)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs(t((bbg$risk.iid)-(bbg2$risk.iid))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


gl <- binreg(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t((iid(gl))- (iid(gl2)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs((bbg$risk.iid)-(bbg2$risk.iid)))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###
###

gla <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
gla2 <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgla2 <- summary(gla2)
sgla <- summary(gla)
gdif <- sgla$ateG- sgla2$ateG
d2 <- max(abs(gla$iid-gla2$iid))
d2
d3 <- max(abs(gdif))
d4 <- max(abs(sgla$ateDR-sgla2$ateDR))
d4
dd2 <- max(abs(gla2$riskG.iid-gla$riskG.iid))
dd3 <- max(abs(gla2$riskDR.iid-gla$riskDR.iid))
expect_true( (max(d1,d2,d3,d4)< 0.000000001) )
expect_true( (dd2< 0.000000001) & ( dd3 < 0.000000001 ))

gl <- binreg(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(iid(gl)- iid(gl2)))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs(head(bbg$risk.iid)-head(bbg2$risk.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###

###
gla <- binregATE(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
gla2 <- binregATE(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgla2 <- summary(gla2)
sgla <- summary(gla)
g3 <- max(abs(sgla$ateG- sgla2$ateG))
g3
d2 <- max(abs(gla$iid-gla2$iid))
d4 <- max(abs(sgla$ateDR-sgla2$ateDR))
c(d2,d3,d4)
dd2 <- max(abs(gla2$riskG.iid-gla$riskG.iid))
dd3 <- max(abs(gla2$riskDR.iid-gla$riskDR.iid))
c(dd2,dd3)
expect_true( (max(d1,d2,d3,d4)< 0.000000001) )
expect_true( (dd2< 0.000000001) & ( dd3 < 0.000000001 ))

##################################################################################

gl <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
sgl <- summary(gl)
###
gl2 <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgl2 <- summary(gl2)
d0 <- max(abs(sgl$coef - sgl2$coef ))
d1 <- max(abs(sgl$ateG-sgl2$ateG))
d2 <- max(abs(sgl$ateDR-sgl2$ateDR))
d3 <- max(abs(cbind(gl$riskG.iid,gl$riskDR.iid)- cbind(gl2$riskG.iid,gl2$riskDR.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) )
expect_true( (d0< 0.000000001) )
c(d0,d1,d2,d3)

## }}}
}
test_binregClust()

test_recgetIPCWClust <- function() {
## {{{ recregIPCW 

 ## IPCW at 2 years 
gl <- recregIPCW(Event(entry,time,status)~treatment+z+cluster(idn),data=hf,
     cause=1,death.code=2,time=2,cens.model=~strata(treatment))
gl2 <- recregIPCW(Event(entry,time,status)~treatment+z+cluster(idn),data=hf2,
     cause=1,death.code=2,time=2,cens.model=~strata(treatment))
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(iid(gl))- head(iid(gl2))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)

## }}} 
}
test_recgetIPCWClust()

test_binregRatioClust <- function() {
## {{{ also works  binregRatio, 

gl <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3,type="II")
gl2 <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3,type="II")
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(gl$iid)- head(gl2$iid)))
d3 <- max(abs(head(gl$iidI)- head(gl2$iidI)))
d4 <- max(abs(gl$coefI-gl2$coefI))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


gl <- binregRatio(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1,time=3)
gl2 <- binregRatio(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(gl$iid)- head(gl2$iid)))
###
d3 <- max(abs(head(gl$iidI)- head(gl2$iidI)))
d4 <- max(abs(gl$coefI-gl2$coefI))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)

gl <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,
		  time=3,type="II")
gl2 <- binregRatio(Event(time,status)~treatment+z,data=hf2,cause=1,time=3,type="II")
estimate(gl)$coefmat 
estimate(gl2)$coefma
d1
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d1
expect_true( (d1< 0.000000001) )

## }}}
}
test_binregRatioClust()

test_binregTSRClust <- function() {
## {{{ works binregTSR 

 library(mets)
 ddf <- mets:::gsim(200,covs=1,null=0,cens=1,ce=2)
 datat <- ddf$datat
 datat$id2 <- sample(200,200)[datat$id]
 datat2 <- dsort(datat,~id2+time)
###     
 bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
###
  bb2 <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat2,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
  d1 <- 0
  for (i in 1:4) d1 <- max(d1,max(abs(bb2$riskG[[i]]-bb$riskG[[i]])))
  d2 <- 0
  for (i in 1:4) d2 <- max(d2,max(abs(bb2$riskG.iid[[i]]-bb$riskG.iid[[i]])))
  expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
 print( c(d1,d2))

 bb <- binregTSR(Event(entry,time,status)~+1+cluster(id2),datat,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
###
  bb2 <- binregTSR(Event(entry,time,status)~+1+cluster(id2),datat2,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
  d1 <- 0
  for (i in 1:4) d1 <- max(d1,max(abs(bb2$riskG[[i]]-bb$riskG[[i]])))
  d2 <- 0
  for (i in 1:4) d2 <- max(d2,max(abs(bb2$riskG.iid[[i]]-bb$riskG.iid[[i]])))
  expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
  print(c(d1,d2))

  ## }}}
}
test_binregTSRClust()

test_phregIPTWClust <- function() {
## {{{ phreg_IPTW

gl <- phreg_IPTW(Event(entry,time,status==1)~treatment+cluster(idn),data=hf, treat.model=treatment~z)
gl2 <-phreg_IPTW(Event(entry,time,status==1)~treatment+cluster(idn),data=hf2, treat.model=treatment~z)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl2))- head(iid(gl)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)

library(mets)
data <- mets:::simLT(0.7,200,beta=0.3,betac=0,ce=1,betao=0.3)
dfactor(data) <- Z.f~Z
data$id2 <- sample(200,200)
data2 <- data[order(data$id2),]
out <- phreg_IPTW(Surv(time,status)~Z.f+cluster(id2),data=data,treat.model=Z.f~X)
out2 <- phreg_IPTW(Surv(time,status)~Z.f+cluster(id2),data=data2,treat.model=Z.f~X)
d1 <- max(abs(iid(out)-iid(out2)))
d2 <- max(abs(estimate(out)$coefmat-estimate(out2)$coefmat))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1
calgt$id2 <- sample(388,388)[calgt$id]
calgt2 <- dsort(calgt,~id2+time)
###
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f
tm=At.f~factor(Count2)+age+sex+wbc
###
gl <- phreg_IPTW(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,treat.model=tm,treat.var="treatvar")
gl2 <- phreg_IPTW(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,treat.model=tm,treat.var="treatvar")
d1 <- max(abs(estimate(gl2)$coefmat- estimate(gl)$coefmat))
d2 <- max(abs(gl$IID-gl2$IID))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

## }}}
}
test_phregIPTWClust()

test_phregRCT <- function() {
## {{{ phreg_rct


gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(idn),data=hf,augmentR0=~z,augmentC=~z+Count1)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(idn),data=hf2,augmentR0=~z,augmentC=~z+Count1)
d1 <- max(abs(gl$coefs-gl2$coefs))
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)

gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf,augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf2, augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE)
d1 <- max(abs(gl$coefs-gl2$coefs))
###gl$coefs- gl2$coefs
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)

gl$AugR0-gl2$AugR0
gl$AugCdyn-gl2$AugCdyn

###gl$data.augR0$ea- gl2$data.augR0$ea
###gl$data.augR0$XR- gl2$data.augR0$XR
###gl$data.augR0$XRpi- gl2$data.augR0$XRpi
###gl$data.augR0$p0- gl2$data.augR0$p0
######
###names(gl$data.augR0)

gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf,augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE,estpr=0)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf2, augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE,estpr=0)
d1 <- max(abs(gl$coefs-gl2$coefs))
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)


data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1
set.seed(100)
calgt$id2 <- sample(388,388)[calgt$id]
calgt2 <- dsort(calgt,~id2+time)
###
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f
tm=At.f~factor(Count2)+age+sex+wbc
###
gl <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,RCT=TRUE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
gl2 <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,RCT=TRUE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
d1 <- max(abs(gl2$coefs- gl$coefs))
d2 <- 0
for (i in 1:length(gl$iid)) d2 <- max(d2,max(abs(head(gl$iid[[i]])- head(gl2$iid[[i]]))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
d3 <- max( gl$AugR0-gl2$AugR0, gl$AugR1-gl2$AugR1, gl$AugR01-gl2$AugR01, gl$AugCdyn-gl2$AugCdyn)
expect_true( (d3< 0.000000001) )
###
c(gl$AugR0-gl2$AugR0, gl$AugR1-gl2$AugR1, gl$AugR01-gl2$AugR01, gl$AugCdyn-gl2$AugCdyn)


gl <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,RCT=FALSE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
gl2 <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,RCT=FALSE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
d1 <- max(abs(gl2$coefs- gl$coefs))
d2 <- 0
for (i in 1:length(gl$iid)) d2 <- max(d2,max(abs(head(gl$iid[[i]])- head(gl2$iid[[i]]))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
d3 <- max( gl$AugR0-gl2$AugR0,gl$AugR1-gl2$AugR1,gl$AugR01-gl2$AugR01,gl$AugCdyn-gl2$AugCdyn)
expect_true( (d3< 0.000000001) )


## }}}
}
test_phregRCT()

test_WAClust <- function() {
## {{{ WA_recurrent

w1 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf,time=2)
w2 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf2,time=2)
s1 <- summary(w1)
s2 <- summary(w2)
s1
s2
d1 <- max(abs(s2$rmst$coefmat- s1$rmst$coefmat))
d2 <- max(abs(s2$meanNtD$coefmat- s2$meanNtD$coefmat))
d3 <- max(abs(s2$ratio$coefmat - s1$ratio$coefmat))
d4 <- max(abs(s2$meanpt$coefmat- s1$meanpt$coefmat))
c(d1,d2,d3,d4)
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###
d1 <- max(abs(w1$ET$riskDR$riskG.iid- w2$ET$riskDR$riskG.iid))
d2 <- max(abs(w1$ET$riskDR$riskDR.iid- w2$ET$riskDR$riskDR.iid))
d3 <- max(abs(w1$ET$riskDR$iid -w2$ET$riskDR$iid))
head(w1$ET$riskDR$iid) 
head(w2$ET$riskDR$iid)
d4 <- max(abs((w1$RAW$iid- w2$RAW$iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


w1 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf,time=2,augmentC=~z+Count1,augmentR=~z)
w2 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf2,time=2,augmentC=~z+Count1,augmentR=~z)
###
s1 <- summary(w1)
s2 <- summary(w2)
d1 <- max(abs(s2$rmst$coefmat- s1$rmst$coefmat))
d3 <- max(abs(s2$meanNtD$coefmat- s2$meanNtD$coefmat))
d4 <- max(abs(s2$ratio$coefmat - s1$ratio$coefmat))
d5 <- max(abs(s2$meanpt$coefmat- s1$meanpt$coefmat))
c(d1,d2,d3,d4,d5)
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)
###
d4 <- max(abs((w1$RAW$iid- w2$RAW$iid)))
d1 <- max(abs(head(w1$ET$riskDR$riskG.iid)- head(w2$ET$riskDR$riskG.iid)))
d2 <- max(abs(head(w1$ET$riskDR$riskDR.iid)- head(w2$ET$riskDR$riskDR.iid)))
d3 <- max(abs(head(w1$ET$riskDR$iid) - head(w2$ET$riskDR$iid)))
d5 <- max(abs(head(w1$ET$riskDRC$iid) - head(w2$ET$riskDRC$iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

## }}}
}
test_WAClust()

## RMST binreg versus resmean.phreg, cif.yearslost

test_resmeanIPCW <- function() {
  set.seed(101)
  data(bmt)
  bmt$time <- bmt$time+runif(nrow(bmt))*0.001

### same as Kaplan-Meier for full censoring model
  bmt$int <- with(bmt,strata(tcell,platelet))
  out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                     cens.model=~strata(platelet,tcell),model="lin")
  estimate(out)
  out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
  rm1 <- resmean_phreg(out1,times=30)
  expect_true( (sum(abs(rm1$intkmtimes[,3] - coef(out)))<0.0001) &
               (sum(abs(rm1$intkmtimes[,4] - out$se.coef))<0.05) )
}
test_resmeanIPCW()

test_cif_yearslost <- function() {
  set.seed(101)
  data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

### same as cif integral for full censoring model
  bmt$int <- with(bmt,strata(tcell,platelet))

  ## competing risks years-lost for cause 1
  outc <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                      cens.model=~strata(platelet,tcell),model="lin")
  ## same as integrated cumulative incidence
  rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
  expect_true( (sum(abs(rmc1$intkmtimes[,3] - coef(outc)))<0.0001) &
               (sum(abs(rmc1$intkmtimes[,5] - outc$se.coef))<0.05) )
}
test_cif_yearslost()

context("binreg simple-icpw versus prodlim")

test_that("binreg-cif", {
data(bmt)
bmt$time <-  bmt$time+runif(nrow(bmt))*0.001
out <- binreg(Event(time,cause)~-1+factor(platelet),bmt,time=50,cens.model=~strata(platelet))
summary(out)
pp <- predict(out,data.frame(platelet=0:1,tcell=0:1))

cif1 <- cif(Event(time,cause)~strata(platelet),bmt)
cifs <- cbind(cif1$cumhaz[,1],vecAllStrata(cif1$cumhaz[,2],cif1$strata,cif1$nstrata))
pcif50 <- cpred(cifs,50)[,2]

se.cifs <- cbind(cif1$cumhaz[,1],vecAllStrata(cif1$se.cumhaz[,2],cif1$strata,cif1$nstrata))
se.pcif50 <- cpred(se.cifs,50)[,2]

## estimate (same) and standard errors (close)
expect_true( ((sum(abs(pcif50-pp[,1])))<0.0001) & (sum(abs(pp[,2]-cpred(se.cifs,50)[,2])) < 0.0002))
})

context("Influence functions, robust standard errors")

fnorm <-

test_that("phreg.iid", {
  data(TRACE)
  out1 <- phreg(Surv(time,status==9)~vf+chf, data=TRACE)
  expect_true(norm(estimate(out1)$vcov - solve(-out1$hessian))<1)
})

test_that("twinlm.iid", {
  set.seed(1)
  d <- twinsim(100,b1=c(1,-1),b2=c(),acde=c(1,1,0,1))
  l <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
  expect_true(norm(estimate(l)$vcov - vcov(l))<1)
})

test_that("biprobitid", {
  set.seed(1)
  d <- twinsim(100,b1=c(1,-1),b2=c(),acde=c(1,1,0,1))
  l <- biprobit(y0 ~ x1, id="id", rho=~x1, data=d)
  expect_true(norm(estimate(l)$vcov - vcov(l))<1)
})

library("tinytest")
library(mets) 
###
set.seed(100)
###
data(hfactioncpx12)
hf <- hfactioncpx12
## break ties after cause and censoring
hf <- tie.breaker(hf,id="id",cause=c(1,0),cens.code=5)
hf$idn <- hf$id
hf$x <- as.numeric(hf$treatment) 
hf$z <- rnorm(741)[hf$id]
hf$nn <- 1:nrow(hf)
hf$idc <- 5+sample(400,741,TRUE)[hf$id]
###
hf$id2 <- sample(1:741,741)[hf$id]
hf$ido <- hf$id
hf$id <- NULL
###
hf2 <- dsort(hf,~id2+entry+time,return.order=TRUE)

test_phregClust <- function() {
## {{{  works  phreg 

####################################################################################

gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(idn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z+cluster(idn),data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
###
gg <- survivalG(gl,hf,time=1,First=TRUE)
gg2 <- survivalG(gl2,hf2,time=1,First=TRUE)
d3 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d4 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d5 <- max(abs(t(head(gg$risk.iid)- head(gg2$risk.iid))))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
d6 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d5 < 0.000000001 ))
expect_true( (d4< 0.000000001) & ( d5 < 0.000000001 ))
expect_true( (d6< 0.000000001) )
c(d1,d2,d3,d4,d5,d6)


gl <- phreg(Event(entry,time,status==1)~treatment+z,data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4,d5)
b1 <- iidBaseline(gl,time=1)


gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
d5 <- max(abs(t(head(gg$risk.iid)- head(gg2$risk.iid))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

gl <- phreg(Event(entry,time,status==1)~treatment+z+cluster(nn),data=hf)
gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d1
expect_true( (d1<0.0000000001) )
gg <- survivalG(gl,hf,time=1)
## data her skal svare til fit 
gg2 <- survivalG(gl2,hf,time=1)
d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
d2
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d4 <- max(abs(gg$ratio$coefmat- gg2$ratio$coefmat))
d5 <- max(abs(gg$risk.iid-gg2$risk.iid))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

###gg <- survivalG(gl,hf2,time=1); gg2 <- survivalG(gl2,hf2,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###d2
######
###gg <- survivalG(gl,hf2,time=1); gg2 <- survivalG(gl2,hf,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###cat("survivalG på data i anden order end fit")
###d2
######
###gl2 <-phreg(Event(entry,time,status==1)~treatment+z,data=hf2)
###gg <- survivalG(gl,hf,time=1); gg2 <- survivalG(gl2,hf2,time=1)
###d2 <- max(abs(gg$survivalG$coefmat- gg2$survivalG$coefmat))
###d2

## }}}
}
test_phregClust()

test_recregClust <- function() {
## {{{  works recreg, cifreg, 

gl <- recreg(Event(entry,time,status)~treatment+z+cluster(idn),data=hf,cause=1,death.code=2)
###
gl2 <- recreg(Event(entry,time,status)~treatment+z+cluster(idn),data=hf2,cause=1,death.code=2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat))
d2 <- max(abs(iid(gl)-iid(gl2)))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
d5 <- max(abs(gg$risk.iid-gg2$risk.iid))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
###
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( d5 < 0.000000001)
c(d1,d2,d3,d4,max(diid),d5)


gl <- recreg(Event(entry,time,status)~treatment+z+cluster(id2),data=hf,cause=1,death.code=2)
###
gl2 <- recreg(Event(entry,time,status)~treatment+z+cluster(id2),data=hf2,cause=1,death.code=2)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
#
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4,max(diid))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)



   ## censoring stratified after treatment 
     gl <- recreg(Event(entry,time,status)~treatment+cluster(idn),data=hf,
     cause=1,death.code=2,cens.model=~strata(treatment))
      gl2 <- recreg(Event(entry,time,status)~treatment+cluster(idn),data=hf2,
     cause=1,death.code=2,cens.model=~strata(treatment))
     d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
     d2 <- max(abs(head(iid(gl))- head(iid(gl2))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
 

####################################################################################


gl <- cifregFG(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1)
gl2 <- cifregFG(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
class(gl)
d2 <- max(abs(iid(gl)- iid(gl2)))
head(iid(gl))
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d3 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
###
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
###
d4 <- max(abs(t(head(cbind(b1$base.iid,b1$beta.iid))- head(cbind(b2$base.iid,b2$beta.iid)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))

gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
d4 <- max(abs(cbind(b1$base.iid,b1$beta.iid)- cbind(b2$base.iid,b2$beta.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))
max(abs(gl2$MGciid-gl$MGciid))



gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z,data=hf,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
###
gg <- survivalG(gl,hf,time=1)
gg2 <- survivalG(gl2,hf,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
b1 <- iidBaseline(gl,time=1)
b2 <- iidBaseline(gl2,time=1)
diid <- c(max(abs(IC(gl)-IC(gl2))), max(abs(iid(gl)-iid(gl2))), max(abs(iid(gl,time=1)-iid(gl2,time=1))),
max(abs(IC(gl,time=1)-IC(gl2,time=1))),
max(abs(b1$base.iid- iid(gl,time=1))),
max(abs(b2$base.iid- iid(gl2,time=1))),
max(abs(b2$beta.iid- iid(gl2))),
max(abs(b1$beta.iid- iid(gl))))
expect_true( (max(diid)< 0.000000001))
d4 <- max(abs(cbind(b1$base.iid,b1$beta.iid)- cbind(b2$base.iid,b2$beta.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( max(diid)< 0.000000001)
c(d1,d2,d3,d4,max(diid))
max(abs(gl2$MGciid-gl$MGciid))

############################ ok også her 

gl <- cifregFG(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,cens.code=0)
gl2 <- cifregFG(Event(time,status)~treatment+z,data=hf2,cause=1,cens.code=0)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat )) 
gg <- survivalG(gl,hf2,time=1)
gg2 <- survivalG(gl2,hf2,time=1)
d2 <- max(abs(gg$risk$coefmat- gg2$risk$coefmat ))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

## }}}
}
test_recregClust() 

test_binregClust <- function() {
## {{{  works binreg, binregG, binregATE 

gl <- binreg(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl))- head(iid(gl2)))))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
head(iid(gl))
head(iid(gl))
head(bbg$risk.iid)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs(t((bbg$risk.iid)-(bbg2$risk.iid))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


gl <- binreg(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t((iid(gl))- (iid(gl2)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs((bbg$risk.iid)-(bbg2$risk.iid)))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###
###

gla <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
gla2 <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgla2 <- summary(gla2)
sgla <- summary(gla)
gdif <- sgla$ateG- sgla2$ateG
d2 <- max(abs(gla$iid-gla2$iid))
d2
d3 <- max(abs(gdif))
d4 <- max(abs(sgla$ateDR-sgla2$ateDR))
d4
dd2 <- max(abs(gla2$riskG.iid-gla$riskG.iid))
dd3 <- max(abs(gla2$riskDR.iid-gla$riskDR.iid))
expect_true( (max(d1,d2,d3,d4)< 0.000000001) )
expect_true( (dd2< 0.000000001) & ( dd3 < 0.000000001 ))

gl <- binreg(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3)
gl2 <- binreg(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(iid(gl)- iid(gl2)))
bbg<- binregG(gl,hf)
bbg2<- binregG(gl2,hf2)
d3 <- max(abs(bbg$risk$coefmat- bbg2$risk$coefmat))
d4 <- max(abs(head(bbg$risk.iid)-head(bbg2$risk.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###

###
gla <- binregATE(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
gla2 <- binregATE(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgla2 <- summary(gla2)
sgla <- summary(gla)
g3 <- max(abs(sgla$ateG- sgla2$ateG))
g3
d2 <- max(abs(gla$iid-gla2$iid))
d4 <- max(abs(sgla$ateDR-sgla2$ateDR))
c(d2,d3,d4)
dd2 <- max(abs(gla2$riskG.iid-gla$riskG.iid))
dd3 <- max(abs(gla2$riskDR.iid-gla$riskDR.iid))
c(dd2,dd3)
expect_true( (max(d1,d2,d3,d4)< 0.000000001) )
expect_true( (dd2< 0.000000001) & ( dd3 < 0.000000001 ))

##################################################################################

gl <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf,cause=1,time=3,
		treat.model=treatment~z)
sgl <- summary(gl)
###
gl2 <- binregATE(Event(time,status)~treatment+z+cluster(idn),data=hf2,cause=1,time=3,
		treat.model=treatment~z)
sgl2 <- summary(gl2)
d0 <- max(abs(sgl$coef - sgl2$coef ))
d1 <- max(abs(sgl$ateG-sgl2$ateG))
d2 <- max(abs(sgl$ateDR-sgl2$ateDR))
d3 <- max(abs(cbind(gl$riskG.iid,gl$riskDR.iid)- cbind(gl2$riskG.iid,gl2$riskDR.iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) )
expect_true( (d0< 0.000000001) )
c(d0,d1,d2,d3)

## }}}
}
test_binregClust()

test_recgetIPCWClust <- function() {
## {{{ recregIPCW 

 ## IPCW at 2 years 
gl <- recregIPCW(Event(entry,time,status)~treatment+z+cluster(idn),data=hf,
     cause=1,death.code=2,time=2,cens.model=~strata(treatment))
gl2 <- recregIPCW(Event(entry,time,status)~treatment+z+cluster(idn),data=hf2,
     cause=1,death.code=2,time=2,cens.model=~strata(treatment))
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(iid(gl))- head(iid(gl2))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)

## }}} 
}
test_recgetIPCWClust()

test_binregRatioClust <- function() {
## {{{ also works  binregRatio, 

gl <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf,cause=1,time=3,type="II")
gl2 <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,time=3,type="II")
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(gl$iid)- head(gl2$iid)))
d3 <- max(abs(head(gl$iidI)- head(gl2$iidI)))
d4 <- max(abs(gl$coefI-gl2$coefI))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


gl <- binregRatio(Event(time,status)~treatment+z+cluster(id2),data=hf,cause=1,time=3)
gl2 <- binregRatio(Event(time,status)~treatment+z+cluster(id2),data=hf2,cause=1,time=3)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(head(gl$iid)- head(gl2$iid)))
###
d3 <- max(abs(head(gl$iidI)- head(gl2$iidI)))
d4 <- max(abs(gl$coefI-gl2$coefI))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)

gl <- binregRatio(Event(time,status)~treatment+z+cluster(nn),data=hf2,cause=1,
		  time=3,type="II")
gl2 <- binregRatio(Event(time,status)~treatment+z,data=hf2,cause=1,time=3,type="II")
estimate(gl)$coefmat 
estimate(gl2)$coefma
d1
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d1
expect_true( (d1< 0.000000001) )

## }}}
}
test_binregRatioClust()

test_binregTSRClust <- function() {
## {{{ works binregTSR 

 library(mets)
 ddf <- mets:::gsim(200,covs=1,null=0,cens=1,ce=2)
 datat <- ddf$datat
 datat$id2 <- sample(200,200)[datat$id]
 datat2 <- dsort(datat,~id2+time)
###     
 bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
###
  bb2 <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat2,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
  d1 <- 0
  for (i in 1:4) d1 <- max(d1,max(abs(bb2$riskG[[i]]-bb$riskG[[i]])))
  d2 <- 0
  for (i in 1:4) d2 <- max(d2,max(abs(bb2$riskG.iid[[i]]-bb$riskG.iid[[i]])))
  expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
 print( c(d1,d2))

 bb <- binregTSR(Event(entry,time,status)~+1+cluster(id2),datat,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
###
  bb2 <- binregTSR(Event(entry,time,status)~+1+cluster(id2),datat2,time=2,cause=c(1),
             cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
             augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
             augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
             response.code=2)
  d1 <- 0
  for (i in 1:4) d1 <- max(d1,max(abs(bb2$riskG[[i]]-bb$riskG[[i]])))
  d2 <- 0
  for (i in 1:4) d2 <- max(d2,max(abs(bb2$riskG.iid[[i]]-bb$riskG.iid[[i]])))
  expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
  print(c(d1,d2))

  ## }}}
}
test_binregTSRClust()

test_phregIPTWClust <- function() {
## {{{ phreg_IPTW

gl <- phreg_IPTW(Event(entry,time,status==1)~treatment+cluster(idn),data=hf, treat.model=treatment~z)
gl2 <-phreg_IPTW(Event(entry,time,status==1)~treatment+cluster(idn),data=hf2, treat.model=treatment~z)
d1 <- max(abs(estimate(gl)$coefmat - estimate(gl2)$coefmat ))
d2 <- max(abs(t(head(iid(gl2))- head(iid(gl)))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)

library(mets)
data <- mets:::simLT(0.7,200,beta=0.3,betac=0,ce=1,betao=0.3)
dfactor(data) <- Z.f~Z
data$id2 <- sample(200,200)
data2 <- data[order(data$id2),]
out <- phreg_IPTW(Surv(time,status)~Z.f+cluster(id2),data=data,treat.model=Z.f~X)
out2 <- phreg_IPTW(Surv(time,status)~Z.f+cluster(id2),data=data2,treat.model=Z.f~X)
d1 <- max(abs(iid(out)-iid(out2)))
d2 <- max(abs(estimate(out)$coefmat-estimate(out2)$coefmat))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1
calgt$id2 <- sample(388,388)[calgt$id]
calgt2 <- dsort(calgt,~id2+time)
###
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f
tm=At.f~factor(Count2)+age+sex+wbc
###
gl <- phreg_IPTW(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,treat.model=tm,treat.var="treatvar")
gl2 <- phreg_IPTW(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,treat.model=tm,treat.var="treatvar")
d1 <- max(abs(estimate(gl2)$coefmat- estimate(gl)$coefmat))
d2 <- max(abs(gl$IID-gl2$IID))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))

## }}}
}
test_phregIPTWClust()

test_phregRCT <- function() {
## {{{ phreg_rct


gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(idn),data=hf,augmentR0=~z,augmentC=~z+Count1)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(idn),data=hf2,augmentR0=~z,augmentC=~z+Count1)
d1 <- max(abs(gl$coefs-gl2$coefs))
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)

gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf,augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf2, augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE)
d1 <- max(abs(gl$coefs-gl2$coefs))
###gl$coefs- gl2$coefs
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)

gl$AugR0-gl2$AugR0
gl$AugCdyn-gl2$AugCdyn

###gl$data.augR0$ea- gl2$data.augR0$ea
###gl$data.augR0$XR- gl2$data.augR0$XR
###gl$data.augR0$XRpi- gl2$data.augR0$XRpi
###gl$data.augR0$p0- gl2$data.augR0$p0
######
###names(gl$data.augR0)

gl <- phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf,augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE,estpr=0)
###
gl2 <-phreg_rct(Event(entry,time,status==1)~treatment+cluster(id2),data=hf2, augmentR0=~z,augmentC=~z+Count1,return.augmentR0=TRUE,estpr=0)
d1 <- max(abs(gl$coefs-gl2$coefs))
d2 <- max(abs(head(gl$iid[[1]])- head(gl2$iid[[1]])))
d3 <- max(abs(head(gl$iid[[2]])- head(gl2$iid[[2]])))
expect_true( (d1< 0.000000001) )
expect_true( (d2< 0.000000001) & ( d3 < 0.000000001 ))
c(d1,d2,d3)


data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1
set.seed(100)
calgt$id2 <- sample(388,388)[calgt$id]
calgt2 <- dsort(calgt,~id2+time)
###
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f
tm=At.f~factor(Count2)+age+sex+wbc
###
gl <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,RCT=TRUE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
gl2 <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,RCT=TRUE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
d1 <- max(abs(gl2$coefs- gl$coefs))
d2 <- 0
for (i in 1:length(gl$iid)) d2 <- max(d2,max(abs(head(gl$iid[[i]])- head(gl2$iid[[i]]))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
d3 <- max( gl$AugR0-gl2$AugR0, gl$AugR1-gl2$AugR1, gl$AugR01-gl2$AugR01, gl$AugCdyn-gl2$AugCdyn)
expect_true( (d3< 0.000000001) )
###
c(gl$AugR0-gl2$AugR0, gl$AugR1-gl2$AugR1, gl$AugR01-gl2$AugR01, gl$AugCdyn-gl2$AugCdyn)


gl <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt,RCT=FALSE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
gl2 <- phreg_rct(Event(start,time,status==1)~A0+A10+cluster(id2),calgt2,RCT=FALSE,treat.model=tm,treat.var="treatvar",
		augmentR0=~sex+age+wbc,augmentR1=~sex+wbc+TR,augmentC=~sex+age+wbc)
d1 <- max(abs(gl2$coefs- gl$coefs))
d2 <- 0
for (i in 1:length(gl$iid)) d2 <- max(d2,max(abs(head(gl$iid[[i]])- head(gl2$iid[[i]]))))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
c(d1,d2)
d3 <- max( gl$AugR0-gl2$AugR0,gl$AugR1-gl2$AugR1,gl$AugR01-gl2$AugR01,gl$AugCdyn-gl2$AugCdyn)
expect_true( (d3< 0.000000001) )


## }}}
}
test_phregRCT()

test_WAClust <- function() {
## {{{ WA_recurrent

w1 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf,time=2)
w2 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf2,time=2)
s1 <- summary(w1)
s2 <- summary(w2)
s1
s2
d1 <- max(abs(s2$rmst$coefmat- s1$rmst$coefmat))
d2 <- max(abs(s2$meanNtD$coefmat- s2$meanNtD$coefmat))
d3 <- max(abs(s2$ratio$coefmat - s1$ratio$coefmat))
d4 <- max(abs(s2$meanpt$coefmat- s1$meanpt$coefmat))
c(d1,d2,d3,d4)
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)
###
d1 <- max(abs(w1$ET$riskDR$riskG.iid- w2$ET$riskDR$riskG.iid))
d2 <- max(abs(w1$ET$riskDR$riskDR.iid- w2$ET$riskDR$riskDR.iid))
d3 <- max(abs(w1$ET$riskDR$iid -w2$ET$riskDR$iid))
head(w1$ET$riskDR$iid) 
head(w2$ET$riskDR$iid)
d4 <- max(abs((w1$RAW$iid- w2$RAW$iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
c(d1,d2,d3,d4)


w1 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf,time=2,augmentC=~z+Count1,augmentR=~z)
w2 <- WA_recurrent(Event(entry,time,status)~treatment+cluster(idn),data=hf2,time=2,augmentC=~z+Count1,augmentR=~z)
###
s1 <- summary(w1)
s2 <- summary(w2)
d1 <- max(abs(s2$rmst$coefmat- s1$rmst$coefmat))
d3 <- max(abs(s2$meanNtD$coefmat- s2$meanNtD$coefmat))
d4 <- max(abs(s2$ratio$coefmat - s1$ratio$coefmat))
d5 <- max(abs(s2$meanpt$coefmat- s1$meanpt$coefmat))
c(d1,d2,d3,d4,d5)
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)
###
d4 <- max(abs((w1$RAW$iid- w2$RAW$iid)))
d1 <- max(abs(head(w1$ET$riskDR$riskG.iid)- head(w2$ET$riskDR$riskG.iid)))
d2 <- max(abs(head(w1$ET$riskDR$riskDR.iid)- head(w2$ET$riskDR$riskDR.iid)))
d3 <- max(abs(head(w1$ET$riskDR$iid) - head(w2$ET$riskDR$iid)))
d5 <- max(abs(head(w1$ET$riskDRC$iid) - head(w2$ET$riskDRC$iid)))
expect_true( (d1< 0.000000001) & ( d2 < 0.000000001 ))
expect_true( (d3< 0.000000001) & ( d4 < 0.000000001 ))
expect_true( (d5< 0.000000001) )
c(d1,d2,d3,d4,d5)

## }}}
}
test_WAClust()

##' Simulation Helper Functions
##'
##' Internal simulation functions used for generating data from various
##' survival, competing risks, frailty, and twin/family models. These
##' functions are primarily intended for use in examples and testing.
##'
##' @section Survival and Competing Risks:
##' \describe{
##'   \item{\code{simrchaz}}{Simulate from cumulative hazard via inverse CDF.}
##'   \item{\code{simul_cifs}}{Simulate competing risks data from cumulative incidence functions.}
##'   \item{\code{simlogitSurvd}}{Simulate survival data using logistic model.}
##'   \item{\code{kumarsim}}{Simulate competing risks data (Kumaraswamy-type).}
##'   \item{\code{kumarsimRCT}}{Simulate RCT competing risks data (Kumaraswamy-type).}
##' }
##'
##' @section Clayton-Oakes and Frailty Models:
##' \describe{
##'   \item{\code{sim_ClaytonOakes_family_ace}}{Simulate family data from Clayton-Oakes ACE model.}
##'   \item{\code{sim_ClaytonOakes_twin_ace}}{Simulate twin data from Clayton-Oakes ACE model.}
##'   \item{\code{sim_Compete_twin_ace}}{Simulate twin competing risks data with ACE frailty.}
##'   \item{\code{sim_Compete_simple}}{Simulate competing risks data with shared frailty.}
##'   \item{\code{sim_Frailty_simple}}{Simulate survival data with shared frailty.}
##'   \item{\code{sim_SurvFam}}{Simulate family survival data with shared frailty.}
##' }
##'
##' @section Binomial Twin/Family Models:
##' \describe{
##'   \item{\code{sim_BinPlack}}{Simulate paired binary data (Plackett model).}
##'   \item{\code{sim_BinFam}}{Simulate binary family data.}
##'   \item{\code{sim_BinFam2}}{Simulate binary family data (alternative parameterization).}
##'   \item{\code{sim_binClaytonOakes_family_ace}}{Simulate binary family data (Clayton-Oakes ACE).}
##'   \item{\code{sim_binClaytonOakes_twin_ace}}{Simulate binary twin data (Clayton-Oakes ACE).}
##'   \item{\code{sim_binClaytonOakes_pairs}}{Simulate binary paired data (Clayton-Oakes).}
##'   \item{\code{sim_bptwin}}{Simulate from a biprobit twin model.}
##' }
##'
##' @section Nordic Twin Studies:
##' \describe{
##'   \item{\code{sim_nordictwin}}{Simulate Nordic twin registry data.}
##'   \item{\code{sim_nordic_random}}{Simulate Nordic twin data with random effects.}
##'   \item{\code{corsim_prostate_random}}{Simulate correlated prostate cancer data with random effects.}
##' }
##'
##' @name mets-simulation
##' @aliases simrchaz simul_cifs simlogitSurvd kumarsim kumarsimRCT
##' @aliases  sim_ClaytonOakes_family_ace sim_ClaytonOakes_twin_ace
##' @aliases  sim_Compete_twin_ace sim_Compete_simple sim_Frailty_simple
##' @aliases  sim_BinPlack sim_BinFam sim_BinFam2
##' @aliases  sim_binClaytonOakes_family_ace sim_binClaytonOakes_twin_ace
##' @aliases  sim_binClaytonOakes_pairs sim_bptwin sim_SurvFam
##' @aliases  sim_nordictwin sim_nordic_random corsim_prostate_random
##' @keywords internal
NULL

##' Lu-Tsiatis More Efficient Log-Rank for Randomized Studies with Baseline Covariates
##'
##' Efficient implementation of the Lu-Tsiatis improvement using baseline covariates, 
##' extended to competing risks and recurrent events. The results are almost equivalent 
##' to the \code{speffSurv} function of the \code{speff2trial} package in the survival case. 
##' A dynamic censoring augmentation regression is also computed to gain additional 
##' efficiency from the censoring augmentation. The function handles two-stage 
##' randomizations and recurrent events (start,stop) with cluster structure.
##' 
##' The method improves the efficiency of the log-rank test by utilizing auxiliary 
##' baseline covariates to reduce variance, particularly useful in randomized clinical 
##' trials (RCTs) where covariate adjustment can increase power.
##'
##' @param formula Formula with \code{Surv} or \code{Event} outcome (see \code{coxph}) and 
##'   treatment variable (randomization 0/1). The treatment variable must be the first 
##'   covariate on the right-hand side.
##' @param data Data frame containing all variables referenced in the formula.
##' @param cause Numeric code for the event of interest in competing risks or recurrent events.
##' @param cens.code Numeric code for censoring in competing risks or recurrent events.
##' @param weights Weights to be used for \code{phreg}.
##' @param typesR Character vector specifying augmentation types for randomization 
##'   (options: "R0" for baseline, "R1" for post-baseline, "R01" for both).
##' @param typesC Character vector specifying augmentation types for censoring 
##'   (options: "C" for static, "dynC" for dynamic).
##' @param augmentR0 Formula for the first randomization augmentation (e.g., \code{~age+sex}).
##' @param augmentR1 Formula for the second randomization augmentation (e.g., \code{~age+sex}).
##' @param augmentC Formula for the censoring augmentation (e.g., \code{~age+sex}).
##' @param treat.model Propensity score model formula (default is \code{~+1}, assuming RCT).
##' @param RCT Logical; if FALSE, uses propensity score adjustment for marginal model.
##' @param treat.var Variable indicating treatment times in two-stage randomization.
##' @param km Logical; use Kaplan-Meier for censoring weights (stratified on treatment).
##' @param level Confidence level for intervals (default 0.95).
##' @param cens.model Censoring model formula (default is \code{~strata(treatment)}).
##' @param estpr Numeric code (1/0); estimate propensity scores or not (default TRUE).
##' @param pi0 Fixed propensity scores for randomizations (if not estimating).
##' @param base.augment Logical; covariate augment baselines (only for R0 augmentation).
##' @param return.augmentR0 Logical; return augmentation data.
##' @param mlogit Logical; use multinomial logistic regression for propensity scores.
##' @param ... Additional arguments passed to \code{phreg} function.
##' @return An object of class \code{"phreg_rct"} containing:
##'   \item{coefs}{Coefficient estimates for the treatment effect.}
##'   \item{iid}{Influence function (IID) decomposition for variance estimation.}
##'   \item{AugR0, AugR1, AugCdyn, AugClt}{Augmentation terms for different strategies.}
##'   \item{cumhaz}{Cumulative hazards.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{se}{Standard errors.}
##'   \item{call}{Original function call.}
##'   \item{formula}{Formula used.}
##'   \item{data}{The data used (if requested).}
##'   
##'   The object includes results for different augmentation combinations (R0, R1, R01, C, dynC).
##' @author Thomas Scheike
##' @references 
##' Lu, T. and Tsiatis, A. A. (2008), Improving the efficiency of the log-rank test using auxiliary covariates, Biometrika, 95, 679--694.
##' 
##' Scheike, T. H., Nerstroem, C. and Martinussen, T. (2026), Randomized clinical trials and the proportional hazards model for recurrent events, TEST.
##' @examples
##' ## Lu, Tsiatis simulation
##' data <- mets:::simLT(0.7,100)
##' dfactor(data) <- Z.f~Z
##' 
##' out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
##' summary(out)
##' @export
phreg_rct <- function(formula,data,cause=1,cens.code=0,
     typesR=c("R0","R1","R01"),typesC=c("C","dynC"),weights=NULL,
     augmentR0=NULL,augmentR1=NULL,augmentC=NULL,treat.model=~+1,RCT=TRUE,
     treat.var=NULL,km=TRUE,level=0.95,cens.model=NULL,estpr=1,pi0=0.5,
     base.augment=FALSE,return.augmentR0=FALSE,mlogit=FALSE,...) {# {{{
  Z <- typeII <- NULL
  cl <- match.call()# {{{
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (!inherits(Y,c("Event","Surv"))) stop("Expected a 'Surv' or 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
    mstatus <- matrix(status,ncol=1)
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
    mstatus <- matrix(status,ncol=1)
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL

 call.id <- id;
 conid <- construct_id(id,length(exit))
 name.id <- conid$name.id; id <- conid$id+1; nid <- conid$nid

 data$id__  <-  id
 data$cid__ <- cumsumstrata(rep(1,length(id)),id-1,nid)
 expit <- lava::expit

sides <- function(formula,vars) {# {{{
lhs <- update(formula,.~+1)
rhs <- update(formula,-1~.)
if (all.vars(lhs)[1]==".") 
   formula <- update.formula(formula,as.formula(paste(vars,"~.")))
return(list(formula=formula,lhs=lhs,rhs=rhs))
}
# }}}

ssform <- sides(formula,"")
varss <- all.vars(ssform$lhs)

## first varaible on rhs of formula
## also candidate for treat variable
streat.name <-  all.vars(ssform$rhs)[1]

treatform <- sides(treat.model,streat.name)
treat.formula <- treatform$formula
treat.name <-  all.vars(treat.formula)[1]
## }}}

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

fittreat <- function(treat.model,data,id,ntreatvar,nlev,mlogit=FALSE) {# {{{
if (nlev==2 & !mlogit) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   DpA <- matrix(0, nrow(Xtreat), (nlev - 1) * ncol(Xtreat))
   for (i in seq(nlev - 1))
   DpA[, ((i-1)*ncol(Xtreat) + 1):(i*ncol(Xtreat))] <- Xtreat * ppp[,i+1] * Dppy / spp^2
###   for (i in seq(nlev-1)) DpA <- cbind(DpA,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*DpA/pA^2

   ## Dp binomial
   Dp <- Xtreat*(pal[,1]*(1-pal[,1]))

out <- list(iidalpha=iidalpha,pA=pA,Dp=Dp,DpA=Dp,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

if (!is.null(treat.var)) { # {{{
	## time-changing weights
	weightWT <- data[,treat.var]
	whereW <- which(weightWT==1)
	CountW <- cumsumstrata(weightWT,id-1,nid)
        dataW <- data[whereW,]; 
	CountWW <- CountW[whereW]
        idW <- id[whereW]; } 
else { ## only first record is associated with treatment  
	weightWT <- rep(1,nrow(data))
	cid <- cumsumstrata(weightWT,id-1,nid)
	## first record of each subject is treatment 
	whereW <- which(cid==1)
	CountW <- cumsumstrata((cid==1)*1,id-1,nid)
	## constant weights 
	CountWW <- CountW[whereW]
	dataW <- data[whereW,]
        idW <- id[whereW]
	if (!is.null(augmentR1)) warning("must specify treat.var to indicate where treatment is given\nassumes now that treatment is given only at start\n")
} 
# }}}

treatvar <- dataW[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be factor \n",sep="")); 
treats <- treats(treatvar)

if (estpr[1]==1 ) {
   fitt <- fittreat(treat.formula,dataW,idW,treats$ntreatvar,treats$nlev,mlogit=mlogit)
   pi0 <- fitt$pal[,-1]
   ## p(A) 
   wPA <- c(fitt$pA)
   DPai <- fitt$DPai
} else {
   ## assumes constant fixed prob over groups
   wPA <- ifelse((treats$ntreatvar)[idW]==2,pi0[1],1-pi0[1])         
   pi0 <- rep(pi0[1],length(idW))
}

## construct multiplicative weights, with possible start stop structure
## put propensity score weights at time of weight change, only when RCT=FALSE
ww <-  rep(1,nrow(data))
ww[whereW] <- wPA
wwt <- exp(cumsumstrata(log(ww),id-1,nid))

## set propensity score weights for Cox model's below 
if (!RCT) ww <- 1/wwt  else ww <- rep(1,nrow(data))

if (!is.null(weights)) ww <- weights*ww

rsss <- all.vars(formula)
if (ncol(Y)==2) 
rformulaS <-as.formula( paste("Surv(",rsss[1],",",rsss[2],"==",cause,")~."))
else 
rformulaS <-as.formula( paste("Surv(",rsss[1],",",rsss[2],",",rsss[3],"==",cause,")~."))
formula <- update(formula,rformulaS)

## change id from call to id__
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))

if (RCT) {
### ... for phreg
fit0 <- phreg(formula,data=data,...)
if (fit0$p>0) eaM <- ea <- ea.iid <- (lava::iid(fit0) %*% fit0$hessian)
else ea <- eaM  <- ea.iid <- matrix(0,max(fit0$id),1)
} else {
fit0 <- phreg_IPTW(formula,data=data,treat.model=treat.formula,treat.var=treat.var,estpr=estpr,pi0=pi0,...)
if (fit0$p>0) {
ea <- ea.iid <- fit0$IID %*% fit0$hessian
## iid without Taylor expansion in weights, to use for censoring augmentation
eaM <- fit0$beta.iid %*% fit0$hessian
} else  ea <- eaM  <- ea.iid <- matrix(0,max(fit0$id),1)
}


data.augR0 <- NULL
AugR0 <- AugR1 <- AugR01 <- rep(0,ncol(ea))
AugR0.iid <-  AugR1.iid <-  AugR01.iid <- matrix(0,nrow(ea),ncol(ea))
if (!is.null(augmentR0)) {# {{{
   ## design 
   ff0 <- sides(augmentR0,all.vars(ssform$rhs)[1])
   dataW0 <- subset(dataW,CountWW==1)
   idW0 <- idW[CountWW==1]
   XR <- model.matrix(augmentR0,dataW0) 
   Z0 <- dataW0[,all.vars(ff0$formula)[1]]
   if (is.factor(Z0)) Z0 <- as.numeric(Z0)-1
   ## order after idW0
   XR[idW0, ] <-  XR 
   Z0[idW0] <- Z0
   strata0 <- piW0 <- rep(0,length(idW0))
   piW0[idW0] <- pi0[CountWW==1]
   ## strata0 0 ??  
   strata0 <- fit0$strata.call[idW0]

   XRpi <- (Z0-piW0)*XR
   XR0pi <- XRpi 
   if (estpr[1]==1) {
       Dp0 <- matrix(0,nid,ncol(fitt$Dp))
       Dp0[idW0,]  <- fitt$Dp[CountWW==1,]
    }
   if (return.augmentR0)  data.augR0 <- list(ea=ea,XRpi=XRpi,XR=XR,A=Z0,p0=piW0)

   augR0fit <- lm(ea~-1+XRpi)
   XRgamma <- augR0fit$fitted.values/(Z0-piW0)
   AugR0.iid <- matrix(augR0fit$fitted.values,ncol=ncol(ea))
   AugR0 <- apply(AugR0.iid,2,sum)


   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
	 Dp0f <- crossprod(Dp0,XRgamma) 
         iid.treat <- fitt$iidalpha %*% Dp0f
	 ### same for RCT \hat \pi(,X)
         AugR0.iid <- AugR0.iid - iid.treat
      } 
      ## oucome model iid 
      if (!RCT) {
         DpO <- apply(XRpi,2,sum)
         iid.outcome <- iid(augR0fit)
###         iid.outcome <- iid.outcome %*% DpO
###         AugR0.iid <- AugR0.iid-iid.outcome
      }

} # }}}

if (!is.null(augmentR1)) {# {{{
   ff1 <- sides(augmentR1,all.vars(ssform$rhs)[2])
   dataW1 <- subset(dataW,CountWW==2)
   idW1 <- idW[CountWW==2]
   XR11 <- model.matrix(augmentR1,dataW1) #[,-1,drop=FALSE]
   XR1 <- matrix(0,nid,ncol(XR11))
   XR1[idW1,] <- XR11
   Z1 <- dataW1[,all.vars(ff1$formula)[1]]
   if (is.factor(Z1)) Z1 <- as.numeric(Z1)-1
   piW1 <- pi0[CountWW==2]
   Z1p <- (Z1-piW1)
   Z1p1  <-  rep(0,nid)
   Z1p1[idW1] <- Z1p
   XR1pi <- Z1p1*XR1

   if (estpr[1]==1) {
	 Dp1 <- matrix(0,nid,ncol(fitt$Dp))
	 Dp1[idW1,]  <- fitt$Dp[CountWW==2,]
   }

   augR1fit <- lm(ea~-1+XR1pi)
   XRgamma <- XR1 %*%  coef(augR1fit)
   AugR1.iid <- augR1fit$fitted.values 
   AugR1 <- apply(AugR1.iid,2,sum)

   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
	 Dp1f <- crossprod(Dp1,XRgamma) 
         iid.treat <- fitt$iidalpha %*% Dp1f
	 ### same for RCT \hat \pi(,X)
         AugR1.iid <- AugR1.iid - iid.treat
      } 

      ## oucome model iid 
      if (!RCT) {
###         DpO <- apply(XR1pi,2,sum)
###         iid.outcome <- iid(augR1fit)
###         iid.outcome <- iid.outcome %*% rep(DpO,ncol(ea))
###         AugR1.iid <- AugR1.iid-iid.outcome
      }

} # }}}

if (!is.null(augmentR0) & !is.null(augmentR1)) {# {{{
   XRbpi <- cbind(XRpi,XR1pi)
   XRb <- cbind(XR,XR1)
   xxi <- pinv(crossprod(XRbpi))

   augR01fit <- lm(ea~-1+XRbpi)
   XRgamma <- XRb %*%  coef(augR01fit)
   AugR01.iid <- augR01fit$fitted.values 
   AugR01 <- apply(AugR01.iid,2,sum)

   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
         XRgamma0 <- XRpi %*% coef(augR01fit)[1:ncol(XRpi),]
         XRgamma1 <- XR1pi %*% coef(augR01fit)[-(1:ncol(XRpi)),]
         Dp01f <- crossprod(Dp0,XRgamma0)+crossprod(Dp1,XRgamma1)
         iid.treat <- fitt$iidalpha %*% Dp01f
         AugR01.iid <- AugR01.iid - iid.treat
      } 
      ## oucome model iid 
      if (!RCT) {
         DpO <- apply(XRbpi,2,sum)
###         iid.outcome <- iid(augR01fit)
###         iid.outcome <- iid.outcome %*% rep(DpO,ncol(ea))
###         AugR01.iid <- AugR1.iid-iid.outcome
      }

} else {AugR01.iid <- AugR0.iid+AugR1.iid; AugR01 <- AugR0+AugR1;  }
# }}}

varC.improve <- 0; formulaC <- NULL
AugC <- AugC.times <- AugClt <- rep(0,ncol(ea)) 
AugC.iid <- AugClt.iid <- matrix(0,nrow(ea),ncol(ea))

## compute regression augmentation for censoring martingale 
if ((!is.null(augmentC))) {## {{{

xxx <- fit0$cox.prep
### X fra GL + tail-death 
rr <- c(exp(xxx$X %*% coef(fit0)+ xxx$offset)*xxx$weights)
Zrr <- xxx$X*rr
S0i <- rep(0,nrow(xxx$X))
jumps <- xxx$jumps+1
S0i[jumps] <- 1/fit0$S0
E <- U <- matrix(0,nrow(Zrr),ncol(Zrr))
U[jumps,] <- fit0$U
E[jumps,] <- fit0$E
ZEdN <- apply(U,2,revcumsumstrata,xxx$id,nid)
cumhaz <- cumsumstrata(S0i,xxx$strata,xxx$nstrata)
EdLam0 <- apply(E*S0i,2,cumsumstrata,xxx$strata,xxx$nstrata)

mmA <- cbind(status,weightWT,CountW)

## formulac with or without start,stop formulation
if (is.null(cens.model)) 
  cens.model <- as.formula(paste("~strata(",treat.name,")+cluster(id__)"))
else cens.model <- update.formula(cens.model,.~.+cluster(id__))
rrss <- all.vars(ssform$lhs)
if (length(rrss)==2) 
formulaC <-as.formula( paste("Surv(",rrss[1],",",rrss[2],"==",cens.code,")~."))
else 
formulaC <-as.formula( paste("Surv(",rrss[1],",",rrss[2],",",rrss[3],"==",cens.code,")~."))
formulaC <- update.formula(formulaC,cens.model)
###
varsC <- attr(terms(augmentC),"term.labels")
formCC <- update(formulaC, reformulate(c(".", varsC)))

Cfit0 <- phreg(formCC,data=data,no.opt=TRUE,no.var=1,Z=mmA,...)

  ### computing weights for censoring terms
  x <- Cfit0
  xx <- x$cox.prep
  S0i <- rep(0,length(xx$strata))
  jumpsC <- xx$jumps+1
  S0i[jumpsC] <-  1/x$S0
  ## G_c survival at t- 
  if (!km) {
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  ###
  rr0 <- c(xx$sign)
  XXA <- xx$X
  EA <- Cfit0$E
  UA <- Cfit0$U
  dhessian <- Cfit0$hessianttime
  hesst <-  .Call("XXMatFULL",dhessian,Cfit0$p,PACKAGE="mets")$XXf

CovZXstrata <- function(X,Ej,Z,Sign,strata,nstrata,jumps) 
{# {{{
		strata  <- c(strata); Sign <- c(Sign)
         ###	Ej <- Ej[jumps,,drop=FALSE]; Ej <- Ej
		ZE <- apply(Z*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
		XZ  <- .Call("vecMatMat",X,Z)$vXZ;  
		XZ <- apply(XZ*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
		EXZ  <- .Call("vecMatMat",Ej,ZE)$vXZ;  
		out <- XZ-EXZ
		return(out)
}# }}}

fid <- headstrata(xxx$id,nid)

  ## {{{
  if (any(CountWW==2)) {
  rrd <- rr-rr[fid][xxx$id+1]
  Zrrd <- Zrr-Zrr[fid,,drop=FALSE][xxx$id+1,]

  ## place to start with Lam_o(T_R) 
  jumpsW <- which((xx$Z[,2]==1)*(xx$Z[,3]==2)*(xx$sign==-1)==1)
  EdLamTR <- XcumTR <- matrix(0,length(xxx$strata),ncol(xxx$X))
  XcumTR[jumpsW,] <-  Zrrd[jumpsW,]*cumhaz[jumpsW]
  XcumTR <- apply(XcumTR,2,cumsumstrata,xx$id,nid)

  ###  dd <-   cbind(XcumTR,xx$id,xxx$id,xx$status,xx$sign,xxx$sign,xx$Z,xx$time,rr)
  ###  jumpsW; dd[dd[,4]==61,]; dd[dd[,4]==20,]; dd[dd[,4]==95,]; cumhaz[jumpsW]; 

  EdLamTR[jumpsW,] <- EdLam0[jumpsW,]*rrd[jumpsW]
  EdLamTR <- apply(EdLamTR,2,cumsumstrata,xx$id,nid)

  ###  XcumTR <- xx$X*c(cumTR)
  covXTRsZ <-   CovZXstrata(XXA,EA,XcumTR,rr0,xx$strata,xx$nstrata,jumpsC) 
  covELTRsZ <-  CovZXstrata(XXA,EA,EdLamTR,rr0,xx$strata,xx$nstrata,jumpsC) 
  covttt <- covXTRsZ-covELTRsZ
  } else  covttt <- 0
  ## }}}

###  dd <-   cbind(cumTR,xxx$id,xx$sign,xx$Z,xx$time) 

covXsZ <-   CovZXstrata(XXA,EA,Zrr,rr0,xx$strata,xx$nstrata,jumpsC) 
covXsrr <-  CovZXstrata(XXA,EA,as.matrix(rr,ncol=1),rr0,xx$strata,xx$nstrata,jumpsC) 
covXsUs3 <- .Call("vecMatMat",covXsrr,EdLam0[jumpsC,,drop=FALSE])$vXZ;  
covXsUs2 <- covXsZ*cumhaz[jumpsC]-covXsUs3 
### U(infty)= UU
Uinfiid <- -eaM[xx$id+1,,drop=FALSE]
cZEdN <- ZEdN[fid,,drop=FALSE][xx$id+1,,drop=FALSE]-ZEdN
Us1 <- Uinfiid-cZEdN
covXsUs1 <- CovZXstrata(XXA,EA,Us1,rr0,xx$strata,xx$nstrata,jumpsC) 
## scale with Y_(s) because hessiantime is also scaled with this 
covXsYs <- (covXsUs1+covXsUs2-1*covttt)/c(Cfit0$S0); 

p <- ncol(ea)
pXXA <- ncol(XXA)
gammat <-  -.Call("CubeMattime",hesst,covXsYs,pXXA,pXXA,pXXA,p,1,0,0,PACKAGE="mets")$XXX
### solve(matrix(hesst[1,],3,3)) %*% matrix(covXsYs[1,],3,2)
gammat[is.na(gammat)] <- 0
gammat[!is.finite(gammat)] <- 0
augmentt <- .Call("CubeMattime",gammat,UA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
AugCdyn <-  apply(augmentt,2,sum)
gain.times <- .Call("CubeMattime",covXsYs,gammat,pXXA,p,pXXA,p,0,1,0,PACKAGE="mets")$XXX
vardynC.improve  <- matrix(apply(gain.times,2,sum),p,p)

## regress U(s)=\int_s^\infty (Z-E) w(s) dM(s) on agument-model among survivors 
## U(s) = U(\infty) - \int_0^s (Z-E) w(s)  dM(s)
## sum (e_i - \bar e) U(s) Y_i(s)
  ## Lu-Tsiatis augmentation 
  out1 <- iidBaseline(Cfit0,ft=1/St,time=0,fixbeta=0)
  Hiid <- (out1$beta.iid %*% Cfit0$hessian)
  xxi <- pinv(crossprod(Hiid))
  ###
  for (i in 1:ncol(ea)) {
     gamma <- xxi %*% crossprod(Hiid, eaM[,i])
     AugClt.iid[,i] <- Hiid %*%  gamma
     AugClt[i] <- sum(AugClt.iid[,i])
  }

Gcj <- St[jumpsC]
varZdN <- matrix(apply(hesst/c(Gcj^2),2,sum),pXXA,pXXA)
covXYdN <- matrix(apply(covXsYs/c(Gcj),2,sum),p,pXXA,byrow=TRUE) 
gamma <- -1*.Call("CubeMattime",matrix(varZdN,nrow=1),matrix(covXYdN,nrow=1),pXXA,pXXA,p,pXXA,1,0,1,PACKAGE="mets")$XXX
gamma <- matrix(gamma,p,pXXA,byrow=TRUE)
gamma[is.na(gamma)] <- 0; 
gamma[!is.finite(gamma)] <- 0
augment <- c(gamma %*% apply(UA/c(Gcj),2,sum))
var.Clt.improve <-  gamma %*% t(covXYdN) ###  /(nid^2)

#### iid magic  for censoring augmentation martingale{{{
### int_0^infty gamma (e_i - ebar(s)) 1/G_c(s) dM_i^c
S0iG <- S0i <- rep(0,length(xx$strata))
S0iG[jumpsC] <- 1/c(x$S0*Gcj)
S0i[jumpsC] <-  1/x$S0
U <- E <- matrix(0,nrow(xx$X),pXXA)
E[jumpsC,] <- EA; 
U[jumpsC,] <- UA/c(Gcj)
cumhaz <- cumsumstrata(S0iG,xx$strata,xx$nstrata)
EdLam0 <- apply(E*S0iG,2,cumsumstrata,xx$strata,xx$nstrata)

gammasEs <- .Call("CubeMattime",gammat,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
gammasE <- matrix(0,nrow(XXA),p)
gammatt  <-    matrix(0,nrow(XXA),pXXA*p)
gammasE[jumpsC,] <- gammasEs
gammatt[jumpsC,] <- gammat
gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
gammadLam0 <-   apply(gammatt*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
Ut <- Et <- matrix(0,nrow(XXA),p)
Ut[jumpsC,] <- augmentt
MGCtt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
MGCttiid <- apply(MGCtt,2,sumstrata,xx$id,nid)
AugCdyn.iid <-  MGCttiid ## %*% iH
# }}}

} else {
iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
var.augment.times <- var.augment <- NULL
var.augment.times.iid <- var.augment.iid <- NULL
Uiid.augment.times <- Uiid.augment <- NULL
time.gammat <- gamma <- gammat <- NULL
ftime.gamma <- NULL
Gcj <- NULL
} ## }}}

## Fitting all models with augmentation terms 
if (fit0$p>0) {
coefMarg <- summary(estimate(coef = coef(fit0), vcov = fit0$var), level = level)$coefmat
var.names <- rownames(coefMarg)
rownames(coefMarg) <- paste("Marginal",rownames(coefMarg),sep="-")
coefs <- coefMarg
} else coefs <- NULL

if (length(typesR)!=3) typesRR <- typesR else 
typesRR <- typesR[c(!is.null(augmentR0),!is.null(augmentR1),!is.null(augmentR0) & !is.null(augmentR1)) ]

if (length(typesC)!=2) typesCC <- typesC else 
typesCC <- typesC[rep(!is.null(augmentC),length(typesC))]
if (is.null(augmentC)) {
	AugCdyn <- AugClt <- AugCdyn.iid <- AugClt.iid <-  0
	typesCC <- "none"
}
if (length(typesRR)==0) { typesRR <- "none" }


baselinecox <- cumhazR0 <- cumhaz <-  se.cumhaz <- se.R0cumhaz <- NULL
iidn <- c()
varbeta <- iid <- fitt_list <- list(); j <- 0
for (typeR in typesRR) 
for (typeC in typesCC) {# {{{
if (typeR!=typeC) {
   j <- j+1
   AugR <- (typeR=="R0")*AugR0+ (typeR=="R1")*AugR1+(typeR=="R01")*AugR01 +0 
   AugR.iid <- 0+(typeR=="R0")*AugR0.iid+(typeR=="R1")*AugR1.iid + (typeR=="R01")*AugR01.iid
   AugC <- (typeC=="C")*AugClt+(typeC=="dynC")*AugCdyn+0
   Aug <- AugR+AugC

   if (fit0$p>0) iid[[j]] <- -1*(ea.iid-AugR.iid ) %*% fit0$ihessian else iid[[j]] <- 0
   var.beta <- crossprod(iid[[j]])
   AugC.iid <- 0+(typeC=="C")*AugClt.iid+ (typeC=="dynC")*AugCdyn.iid
   if (typeC=="dynC") {
       var.beta <- var.beta - fit0$ihessian %*% vardynC.improve%*% fit0$ihessian
   }
   if (typeC=="C") {
       var.beta <- var.beta - fit0$ihessian %*% var.Clt.improve%*% fit0$ihessian
   }
   if (fit0$p>0) varbeta[[j]] <- var.beta

   if (fit0$p>0) 
   fitts <- phreg(formula,data=data,augmentation=Aug,no.var=1,weights=ww,...)
   else fitts <- fit0

   ## only baseline augment with R0 augmentation
   if (base.augment & typeR == "R0" & (typeC != "dynC" & typeC != "C")) { ## {{{
      xxx <- fitts$cox.prep
      xx <- crossprod(XR0pi)
      xxi <- pinv(xx)
      XRpit <- XR0pi[xxx$id+1,]
      if (fit0$p>0) rr <- c(exp(xxx$X %*% coef(fitts)+ xxx$offset)*xxx$weights)
      else rr <- c(exp(xxx$offset)*xxx$weights)
      rrs <- rr*c(xxx$sign)
      XRE <- apply(XRpit*rrs,2,revcumsumstrata,xxx$strata,xxx$nstrata)
      S0i2 <- S0i <- rep(0,nrow(xxx$X))
      jumps <- xxx$jumps+1
      S0i[jumps] <- 1/fitts$S0
      ww_base <- xxx$caseweights*xxx$weights
      S0i2[jumps] <- 1/(fitts$S0^2*ww_base[xxx$jumps+1])
      ###
      U <- matrix(0,nrow(XRE),ncol(XRE))
      U[jumps,] <- XRpit[jumps,]*S0i[jumps]
      NXRE <- apply(U,2,cumsumstrata,xxx$strata,xxx$nstrata)[jumps,]
      XREdLam0 <- apply(XRE*S0i2,2,cumsumstrata,xxx$strata,xxx$nstrata)[jumps,]
      ns <- c(sumstrata(rep(1,nid),xxx$strata[headstrata(xxx$id,nid)],xxx$nstrata))[xxx$strata[jumps,]+1]
###      if (RCT) covBase <- (NXRE-XREdLam0) else covBase <- (NXRE-XREdLam0)
      covBase <- (NXRE-XREdLam0) 

      gamR0Base <- (covBase) %*% xxi
      XRs <- apply(XR0pi,2,sum)
      R0baseline.augment <-  c(XRs %*% t(gamR0Base))
      R0baseline.reduction<- apply(covBase*gamR0Base,1,sum)
      ## using augmented estimator of beta 
      if (fit0$p>0) fitr0 <- robust_phreg(fitts,beta.iid=iid[[j]])
      else fitr0 <- robust_phreg(fit0)
      cumhazR0 <- cumhaz <- fitts$cumhaz
      cumhazR0[,2] <- cumhaz[,2]-R0baseline.augment
      se.R0cumhaz <- se.cumhaz <- fitr0$robse.cumhaz
      se.R0cumhaz[,2] <- (se.cumhaz[,2]^2-R0baseline.reduction)^.5
      baselinecox <- list(phreg=fitts,beta.iid=iid[[j]],XR0pi=XR0pi,gamR0Base=gamR0Base)
   } else baselinecox <- cumhazR0 <- cumhaz <-  se.cumhaz <- se.R0cumhaz <- NULL  ## }}}

   if (fit0$p>0)  {
     coeffitt <- summary(estimate(coef=coef(fitts),vcov=var.beta),level=level)$coefmat
     nnn <- paste(typeR,typeC,sep="_")
     iidn <- c(iidn,nnn)
     rownames(coeffitt) <- paste(nnn,var.names,sep=":")
     coefs <- rbind(coefs,coeffitt)
     if (!is.null(augmentC)) iid[[j]] <- iid[[j]] + AugC.iid%*% fit0$ihessian
   }
   } 
}
names(iid) <- iidn
# }}}

### sort iid after name.id and put as rownames
if (!is.null(call.id)) {
ea.iid <- nameme(ea.iid,name.id)
for (l in 1:length(iid)) iid[[l]] <- nameme(iid[[l]],name.id)
AugC.iid <- nameme(AugC.iid,name.id)
AugClt.iid <- nameme(AugClt.iid,name.id)
}

out <- list(marginal=fit0,AugR0=AugR0,AugR1=AugR1,AugR01=AugR01,AugCdyn=AugCdyn,
    AugClt=AugClt,
    coefs=coefs,iid=iid,AugC.iid=AugC.iid,AugClt.iid=AugClt.iid,Cox.iid=ea.iid,
    formula=formula,formulaC=formulaC,treat.model=treat.model,
    id=id,call.id=call.id,name.id=name.id,
    cumhaz=cumhazR0,se.cumhaz=se.R0cumhaz,
    cumhaz.noAug=cumhaz,se.cumhaz.noAug=se.cumhaz, 
    strata=fit0$strata.jumps,nstrata=fit0$nstrata,jumps=seq_along(fit0$strata.jumps),
    strata.name=fit0$strata.name,strata.level=fit0$strata.level,
    baselinecox=baselinecox,data.augR0=data.augR0,var.beta=varbeta)
class(out) <- "phreg_rct"
return(out)
} ## }}} 


##' @export
plot.phreg_rct  <- function(x,...)  { ## {{{
	if (!is.null(x$cumhaz)) baseplot(x,...) 
} ## }}}

##' @export
print.phreg_rct <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.phreg_rct <- function(object,...) {# {{{
if (!is.null(object$coefs)) res  <- object$coefs else res <- "only baseline"
class(res) <- "summary.phreg_rct"
return(res)
}# }}}

simLT <- function(rho,n,beta=0,betac=0,ce=1,betao=0)
{# {{{
Sigma <- matrix(c(1,rho,rho,1),2,2)
M <- t(chol(Sigma))
# M %*% t(M)
Z <- matrix(rnorm(2*n),2,n) # 2 rows, N/2 columns
XY <- t(M %*% Z) 
###
X <- XY[,1]
Y <- XY[,2]
if (betao!=0) px <- expit(X*betao) else px <- 0.5
Z <- rbinom(n,1,px)
TT <- -exp(Z*beta)*log(1-pnorm(Y))
C <- exp(Z*betac)*rexp(n)*ce
status <- (TT<C)
mean(status)
time <- pmin(TT,C)
data <- data.frame(time=time,status=status,X=X,Z=Z)
return(data)
}# }}}

simLTTS <- function(rho,n,beta=c(0,0),betac=0,ce=1,cr=0.5,betao=0.4)
{# {{{

## to avoid R-check
TR <- Z1.f <- count2 <- X1 <- NULL

sigma <- matrix(rho,4,4)
diag(sigma) <- 1
m <- t(chol(sigma))
# m %*% t(m)
z <- matrix(rnorm(4*n),4,n) # 2 rows, n/2 columns
xy <- t(m %*% z) 
zz <- matrix(rnorm(4*n),4,n) # 2 rows, n/2 columns
xyz <- t(m %*% zz) 
###
x <- xy[,1]
y <- xy[,2]
tr <- xy[,3]
x1 <- xyz[,1]
y1 <- xyz[,2]
if (betao!=0) px <- lava::expit(x*betao) else px <- 0.5
if (betao!=0) px1 <- lava::expit(x1*betao+tr) else px1 <- 0.5
z0 <- rbinom(n,1,px)
z1 <- rbinom(n,1,px1)
tt0 <- -exp(z0*beta[1])*log(1-pnorm(y))
tt1 <- -exp(z0*beta[1]+z1*beta[2])*log(1-pnorm(y1))  ## log(1-pnorm(y1))
tr <- exp(z0*beta[1])*rexp(n)*cr 
tt <- ifelse(tt0<tr,tt0,tr)+(tt0>tr)*(tt1)
c <- exp(z0*betac)*rexp(n)*ce
status <- (tt<c)
time <- pmin(tt,c)
data <- data.frame(time=time,status=status,X=x,X1=x1,Z0=z0,Z1=z1,TR=tr)
data <- event_split(data,cuts="TR")
###data <- EventSplit(data,cuts="TR")
data <- dtransform(data,status=2,TR==time)
data <- dtransform(data,Z1=0,start<TR)
data$count2 <- 1*(data$start==data$TR)
data$Count2 <- 1*(data$start==data$TR)
data$cw <- 1
data$Zt.f <- factor(data$Z0)
data$Z0.f <- factor(data$Z0)
data$Z1.f <- factor(data$Z1)
data$Xt <- data$X
data <- dtransform(data,Zt.f=Z1.f,count2==1)
data <- dtransform(data,Xt=X1,count2==1)
data$X0 <- data$X
data$X1t <- 0
data <- dtransform(data,X1t=X1,count2==1)

return(data)
}# }}}

simLUCox <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,r1=NULL,r2=NULL,rd=NULL,
		     rc=NULL,rtr=NULL,ztr=0,betatr=0.3,A0=NULL,Z0=NULL,efTR=0,
   gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,dependence=4,var.z=1,cor.mat=NULL,cens=NULL,...) 
{# {{{
  status <- death <- fdeath <- dtime <- NULL # to avoid R-check 

  if (dependence==0) { z <- z1 <- z2 <- zd <- rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- z; z2 <- z; zd <- z 
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
      } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; 
	      zd <- rep(1,n) 
	      z <- z1
      }      else stop("dependence 0-4"); # }}}

      if (!is.null(Z0)) {
	      if (dependence==1) zd <- Z0
	      z <- z1 <- z2 <- Z0
      }

   if (is.null(r1)) r1 <- rep(1,n)
   if (is.null(r2)) r2 <- rep(1,n)
   if (is.null(rd)) rd <- rep(1,n)
   if (is.null(rtr)) rtr <- rep(1,n)
   if (is.null(rc)) rc <- rep(1,n)

  ## extend cumulative for death to full range  of cause 1
  if (!is.null(death.cumhaz)) {
	  if (!is.null(cumhaz2))  {
	     out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz),NULL)
	     cumhaz <- out$cum1
	     cumhaz2 <- out$cum2
	     cumhazd <- out$cum3
	  } else  {
	     out <- extendCums(list(cumhaz,death.cumhaz),NULL)
	     cumhaz <- out$cum1
	     cumhazd <- out$cum2
	     cumhaz2 <- NULL
     }
  }  else cumhaz <- rbind(c(0,0),cumhaz)

  ll <- nrow(cumhaz)
  max.time <- tail(cumhaz[,1],1)

### recurrent first time
  tall1 <- rchaz(cumhaz,r1*z1)
  if (ztr==1)  {
  ## should deal with selection, we know that TR has not happened, when A1(t)=0
  cum1A00 <- cbind(cumhaz[,1],cumsum(c(0,diff(cumhaz[,2]))*(1+var.z*cumhaz2[,2])))
  rtr1 <- exp(betatr)
  cum1A01 <- cbind(cumhaz[,1],cumsum(c(0,diff(cumhaz[,2]))*(1+var.z*cumhaz2[,2]*rtr1)))
  tallA0 <- rchaz(cum1A00,z1[A0==0]*r1[A0==0])
  tallA1 <- rchaz(cum1A01,z1[A0==1]*r1[A0==1])
  tall1[A0==0,] <- tallA0
  tall1[A0==1,] <- tallA1
  } else { cum1A00 <-cum1A01 <-  cumhaz; }
  tall <- tall1
  tall$id <- 1:n
  tall$tr <- 0
  tall$sel <- 1
  tall$timetr <- 0


  ## if tr occurs put tr=1
  if (!is.null(cumhaz2))  {
if (ztr==1) tall2 <- rchaz(cumhaz2,z1*rtr)
if (ztr==0) tall2 <- rchaz(cumhaz2,rtr)
       tall$status <- ifelse(tall1$time<tall2$time,tall1$status,2*tall2$status)
       tall$time <- ifelse(tall1$time<tall2$time,tall1$time,tall2$time)
       tall$tr <- ifelse(tall2$time<tall1$time,1,0)
       tall$timetr <- ifelse(tall2$time<tall1$time,tall2$time,0)
if (ztr==1) {
       ww <- (tall2$time<=tall1$time)
       baseTR <- cpred(cumhaz2,tall2$time[ww])
       basesel <- rtr[ww]*baseTR[,2]
       tall$sel[ww] <- (1+var.z*basesel)/(1+var.z)
  }
  }

### death time simulated
  if (!is.null(death.cumhaz)) {
	  timed   <- rchaz(cumhazd,zd*rd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }

### fixing the first time to event, if death or censoring occurs before
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  i <- 1
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i < max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
	  tr <- still$tr
 	  sel <- still$sel
 	  timetr <- still$timetr
	  r1tr <- rep(1,nn)
###	  if (efTR!=0) r1tr <- exp(efTR*timetr); 
	  rrt <- r1tr*r1[still$id]*(1-tr)+r1tr*r1[still$id]*r2[still$id]*tr
        if (ztr==0) tt1 <- rchaz(cumhaz,rrt*z1[still$id],entry=(1-gap.time)*still$time)
	if (ztr==1)  {

             ## sets up dataframe, overwrites below 
             tt1 <- rchaz(cumhaz,rrt*z1[still$id],entry=(1-gap.time)*still$time)
		if (any(tr==0)) {
			rz <- r1[still$id]*z1[still$id]
			A0s <- A0[still$id]
			if (any(A0s[tr==0]==0)) {
                          ww0 <-  A0s[tr==0]==0
			  tallA0 <- rchaz(cum1A00,rz[ww0],entry=still$time[ww0])
			  tt1[A0s[tr==0]==0,] <- tallA0
			}
			if (any(A0s[tr==0]==1)) {
                          ww1 <-  A0s[tr==0]==1
			  tallA0 <- rchaz(cum1A00,rz[ww1],entry=still$time[ww1])
			  tt1[A0s[tr==0]==1,] <- tallA0
			}
		}
		if (any(tr==1)) {
		    rrt <- r1[still$id]*r2[still$id]*sel[still$id]*z1[still$id]
		    rrt <- rrt[tr==1]
		    tt11 <- rchaz(cumhaz,rrt,entry=(1-gap.time)*still$time[tr==1])
		    tt1[tr==1,] <- tt11
		}
	}
	tt <- tt1
	tt$tr <- tr; tt$sel <- sel; tt$timetr <- timetr; 
	  if (!is.null(cumhaz2))  {
              if (ztr==0)  tt2 <- rchaz(cumhaz2,rtr[still$id],entry=(1-gap.time)*still$time)
              if (ztr==1)  tt2 <- rchaz(cumhaz2,rtr[still$id]*z1[still$id],entry=(1-gap.time)*still$time)
	  ## modify to t2=tr  only when tr==0
          tt$status <- ifelse(tt2$time<=tt1$time & tr==0,2*tt2$status,tt1$status)
          tt$time <-   ifelse(tt2$time<=tt1$time & tr==0,tt2$time,tt1$time)
          tt$timetr <-   ifelse(tt2$time<=tt1$time & tr==0,tt2$time,timetr)
	  if (ztr==1) {
             ww <- (tt2$time<=tt1$time & tr==0)
             Base2TR <- cpred(cumhaz2,tt$time[ww])
             basesel <- r1[still$id][ww]*Base2TR[,2]
             tt$sel[ww] <- (1+var.z*basesel)/(1+var.z)
	  }
          tt$tr[tt2$time<=tt1$time & tr==0] <- 1
	  }
	  if (gap.time) {
		  tt$entry <- still$time
		  tt$time  <- tt$time+still$time
	  }
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tt <- tt[,colnames(tall)]
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  } 
  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  tall$statusD <- tall$status
  tall <- dtransform(tall,statusD=3,death==1)

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"z") <- z1

  return(tall)
  }# }}}

simFrail <- function(n,cumhaz,r1=NULL,rc=NULL,dependence=1,var.z=1,cens=NULL,...) 
{# {{{

 cor.mat <- death <- NULL

  if (dependence==0) { z <- z1 <- z2 <- zd <- rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- z; z2 <- z; zd <- z 
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
      } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; zd <- rep(1,n) 
	      z <- z1
      }      else stop("dependence 0-4"); # }}}

   if (is.null(r1)) r1 <- rep(1,n)
   if (is.null(rc)) rc <- rep(1,n)

  ll <- nrow(cumhaz)

 tall   <- rchaz(cumhaz,z1*r1)
  if (!is.null(cens)) { 
     ctime <- rexp(n)/(rc*cens)
     tall$status[tall$time>ctime] <- 0; 
     tall$time[tall$time>ctime] <- ctime[tall$time>ctime] 
  }
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"z") <- z1

  return(tall)
  }# }}}

#' The Bone Marrow Transplant Data
#' 
#' Bone marrow transplant data with 408 rows and 5 columns.
#' 
#' 
#' @format The data has 408 rows and 5 columns. \describe{ \item{cause}{a
#' numeric vector code.  Survival status. 1: dead from treatment related
#' causes, 2: relapse , 0: censored.} \item{time}{ a numeric vector. Survival
#' time.  } \item{platelet}{a numeric vector code. Plalelet 1: more than 100 x
#' \eqn{10^9} per L, 0: less.} \item{tcell}{a numeric vector. T-cell depleted
#' BMT 1:yes, 0:no.} \item{age}{a numeric vector code. Age of patient, scaled
#' and centered ((age-35)/15).} }
#' @references NN
#' @name bmt
#' @docType data
#' @source Simulated data
#' @keywords package 
#' @examples
#' 
#' data(bmt)
#' names(bmt)
#' 
NULL


#' The Diabetic Retinopathy Data
#' 
#' The data was colleceted to test a laser treatment for delaying blindness in
#' patients with dibetic retinopathy. The subset of 197 patiens given in Huster
#' et al. (1989) is used.
#' 
#' 
#' @format This data frame contains the following columns: \describe{
#' \item{id}{a numeric vector. Patient code.} \item{agedx}{a numeric vector.
#' Age of patient at diagnosis.} \item{time}{a numeric vector. Survival time:
#' time to blindness or censoring.} \item{status}{ a numeric vector code.
#' Survival status. 1: blindness, 0: censored.} \item{trteye}{a numeric vector
#' code. Random eye selected for treatment. 1: left eye 2: right eye.}
#' \item{treat}{a numeric vector. 1: treatment 0: untreated.} \item{adult}{a
#' numeric vector code. 1: younger than 20, 2: older than 20.} }
#' @source Huster W.J. and Brookmeyer, R. and Self. S. (1989) MOdelling paired
#' survival data with covariates, Biometrics 45, 145-56.
#' @name  diabetes
#' @docType data
#' @keywords package 
#' @examples
#' 
#' data(diabetes)
#' names(diabetes)
#' 
NULL


#' The Melanoma Survival Data
#' 
#' The melanoma data frame has 205 rows and 7 columns.  It contains data
#' relating to survival of patients after operation for malignant melanoma
#' collected at Odense University Hospital by K.T.  Drzewiecki.
#' 
#' 
#' @format This data frame contains the following columns: \describe{
#' \item{no}{ a numeric vector. Patient code. } \item{status}{ a numeric vector
#' code. Survival status. 1: dead from melanoma, 2: alive, 3: dead from other
#' cause. } \item{days}{ a numeric vector. Survival time. } \item{ulc}{ a
#' numeric vector code. Ulceration, 1: present, 0: absent. } \item{thick}{ a
#' numeric vector. Tumour thickness (1/100 mm). } \item{sex}{ a numeric vector
#' code. 0: female, 1: male. } }
#' @source Andersen, P.K., Borgan O, Gill R.D., Keiding N. (1993),
#' \emph{Statistical Models Based on Counting Processes}, Springer-Verlag.
#' 
#' Drzewiecki, K.T., Ladefoged, C., and Christensen, H.E. (1980), Biopsy and
#' prognosis for cutaneous malignant melanoma in clinical stage I. Scand. J.
#' Plast. Reconstru. Surg. 14, 141-144.
#' @name  melanoma 
#' @docType data
#' @keywords package 
#' @examples
#' 
#' data(melanoma)
#' names(melanoma)
#' 
NULL





#' The TRACE study group of myocardial infarction
#' 
#' The TRACE data frame contains 1877 patients and is a subset of a data set
#' consisting of approximately 6000 patients.  It contains data relating
#' survival of patients after myocardial infarction to various risk factors.
#' 
#' sTRACE is a subsample consisting of 300 patients.
#' 
#' tTRACE is a subsample consisting of 1000 patients.
#' 
#' 
#' @aliases TRACE sTRACE tTRACE
#' @format This data frame contains the following columns: \describe{
#' \item{id}{a numeric vector. Patient code. } \item{status}{ a numeric vector
#' code. Survival status. 9: dead from myocardial infarction, 0: alive, 7: dead
#' from other causes.  } \item{time}{ a numeric vector. Survival time in years.
#' } \item{chf}{ a numeric vector code. Clinical heart pump failure, 1:
#' present, 0: absent. } \item{diabetes}{ a numeric vector code. Diabetes, 1:
#' present, 0: absent. } \item{vf}{ a numeric vector code. Ventricular
#' fibrillation, 1: present, 0: absent. } \item{wmi}{ a numeric vector.
#' Measure of heart pumping effect based on ultrasound measurements where 2 is
#' normal and 0 is worst. } \item{sex}{ a numeric vector code. 1: female, 0:
#' male. } \item{age}{ a numeric vector code. Age of patient. } }
#' @source The TRACE study group.
#' 
#' Jensen, G.V., Torp-Pedersen, C., Hildebrandt, P., Kober, L., F. E. Nielsen,
#' Melchior, T., Joen, T. and P. K. Andersen (1997), Does in-hospital
#' ventricular fibrillation affect prognosis after myocardial infarction?,
#' European Heart Journal 18, 919--924.
#' @name  TRACE
#' @docType data
#' @keywords package 
#' @examples
#' 
#' data(TRACE)
#' names(TRACE)
#' 
NULL

##' Scatterplot with contours of the (kernel) estimated density
##'
##' @title Scatter plot function
##' @param data bivariate data to plot (data.frame or matrix with 2 columns)
##' @param marginal.args argumemts to marginal estimator (`density` continuous
##'   data, `barplot` for categorical )
##' @param kernsmooth.args arguments to 2d-kernel smoother
##' @param xlab x-axis label
##' @param ylab y-axis label
##' @param col color of points
##' @param col2 color of contour / density plot
##' @param alpha transparency level of points
##' @param grid should grid be added to the plot
##' @param side.plot If TRUE subplots of the marginal distributions are added to
##'   the plot
##' @param ... arguments to lower level plot functions
##' @author Klaus Kähler Holst
##' @export
##' @examples
##' data("twinbmi", package="mets")
##' twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
##' datamz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
##'
##' # continuous variables
##' plot_twin(datamz)
##'
##' # categorical variables
##' datamz2 <- datamz
##' datamz2[, 1] <- cut(datamz[, 1], 4)
##' datamz2[, 2] <- cut(datamz[, 2], 4)
##' plot_twin(datamz2, color = TRUE)
##'
##' # survival variables
##' cens1 <- rbinom(nrow(datamz), 1, 0.5)
##' cens2 <- rbinom(nrow(datamz), 1, 0.5)
##' datamz2[, 1] <- Event(datamz[, 1], cens1)
##' datamz2[, 2] <- suppressWarnings(Event(datamz[, 2], cens2))
##' plot_twin(datamz2)
##'
##' rm(datamz, datamz2, cens1, cens2)
plot_twin <- function (data,
                       marginal.args = list(),
                       kernsmooth.args = list(),
                       xlab, ylab,
                       col = "black", col2 = "lightblue",
                       alpha = 0.3,
                       grid = TRUE,
                       side.plot = TRUE,
                       ...
                       ) {

  def.par <- par(no.readonly = TRUE) # save default, for resetting...
  if (!ncol(data) > 1) stop("data much contain at least 2 columns")
  data <- na.omit(data[, 1:2])
  if (missing(xlab)) xlab <- colnames(data)[1]
  if (missing(ylab)) ylab <- colnames(data)[2]
  x <- data[, 1, drop = TRUE]
  y <- data[, 2, drop = TRUE]
  if (class(x)[1] != class(y)[1]) {
      stop("Expecting tabular data with 2 columns of the same type")
  }
  if (side.plot) {
      layout(
          matrix(c(2, 4, 1, 3), 2, 2, byrow = TRUE),
          c(3, 1), c(1, 3), TRUE
      )
  }
  par(mar = c(4, 4, 1, 1))
  if (inherits(x, c("character", "factor"))) {
    tt <- prop.table(table(data))
    tt <- tt[, rev(seq_len(ncol(tt)))]
    mosaicplot(tt,
        main = "",
        xlab = xlab,
        ylab = ylab,
        ...
        )
    if (side.plot) {
        do.call(barplot, c(list(table(data[, 1])), marginal.args))
        do.call(barplot, c(list(table(data[, 2]), horiz = TRUE), marginal.args))
    }
    par(def.par)
    return(invisible())
  }
  if (inherits(x, c("Surv", "Event"))) {
    x0 <- x[, 1]
    y0 <- y[, 1]
    rg <- range(x0, y0)
    plot(x0, y0,
        xlim = rg, ylim = rg,
        xlab = "", ylab = "",
        type = "n",
        ...
        )
    if (grid) graphics::grid()
    cens1 <- !x[, 2]
    cens2 <- !y[, 2]
    if (any(cens1)) {
      idx <- which(cens1)
      points(x0[idx], y0[idx],
          pch = 3,
          col = lava::Col(col, alpha)
          )
    }
    if (any(cens2)) {
      idx <- which(cens2)
      points(x0[idx], y0[idx],
          pch = 5,
          col = lava::Col(col, alpha)
          )
    }
    if (any((!cens1) & (!cens2))) {
        idx <- which(!cens1 & !cens2)
        points(x0[idx], y0[idx],
            pch = 16, col = lava::Col(col, alpha)
        )
    }
    if (!side.plot) {
      par(def.par)
      return(invisible())
    }
    p1 <- phreg(t ~ 1, data.frame(t = x))
    p2 <- phreg(t ~ 1, data.frame(t = y))
    chaz1 <- basecumhaz(p1)[[1]]$cumhaz
    chaz2 <- basecumhaz(p2)[[1]]$cumhaz
    chaz1[, 2] <- exp(-chaz1[, 2])
    chaz2[, 2] <- exp(-chaz2[, 2])
    top <- max(chaz2[,2], chaz1[,2])
    ## par(mar = c(3, 3, 1, 1))
    plot(chaz1[, 1], chaz1[, 2],
        axes = FALSE,
        xlim = rg, ylim = c(0, top),
        type = "s",
        xlab = xlab,
        ylab = "Surv. prob."
        )
    if (grid)  grid()
    axis(2)
    axis(1)
    ## par(mar = c(3, 3, 1, 1))
    plot(chaz2[, 2], chaz2[, 1],
        axes = FALSE,
        ylim = rg, , xlim = c(0, top),
        type = "s",
        ylab = ylab,
        xlab = "Surv. prob."
        )
    if (grid)  grid()
    axis(1)
    axis(2)
    plot(1, type = "n", axes = FALSE, xlab="", ylab="")
    legend("left",
           c("no cens", "1. cens", "2. cens"),
           pch = c(16, 3, 5),
           bty = "n"
    )

    par(def.par)
    return(invisible())
  }

  if (is.null(kernsmooth.args$bandwidth)) { # bandwidth
      kernsmooth.args$bandwidth <- as.numeric(apply(data, 2, sd)) *
          nrow(data)^(-1 / 5)
  }
  d1 <- do.call(density, c(list(x), marginal.args))
  d2 <- do.call(density, c(list(y), marginal.args))
  rg <- range(x, y)
  top <- max(c(d1$y, d2$y)) # nolint
  plot(x, y,
       xlim = rg, ylim = rg,
       xlab = "", ylab = "",
       pch = 16, col = lava::Col(col, alpha),
       ...
       )
  graphics::grid()
  est <- do.call(KernSmooth::bkde2D, c(list(data), kernsmooth.args))
  with(est, contour(x1, x2, fhat, add = TRUE, col = col2))
  if (side.plot) {
    with(d1, plot(x, y,
      xlab = "", ylab = "",
      type = "l", lwd = 0.5,
      axes = FALSE, ylim = c(0, top)
    ))
    x <- with(d1, c(x, rev(x)))
    y <- with(d1, c(y, rep(0, length(y))))
    polygon(x, y, lty = 0, col = col2)
    axis(2)
    mtext(xlab, 1, line = 2)
    with(d2, plot(y, x,
       xlab = "", ylab = "",
       type = "l", lwd = 0.5,
       axes = FALSE, xlim = c(0, top)
     ))
    x <- with(d2, c(x, rev(x)))
    y <- with(d2, c(y, rep(0, length(y))))
    polygon(y, x, lty = 0, col = col2)
    axis(1)
    mtext(ylab, side = 2, line = 2, srt = 20)
  }
  par(def.par)
}

##' Internal function.
##' Calculates Inverse Probability of Censoring
##' Weights (IPCW) and adds them to a data.frame
##'
##' @title Inverse Probability of Censoring Weights
##' @param formula Formula specifying the censoring model 
##' @param data data frame
##' @param cluster clustering variable
##' @param same.cens For clustered data, should same censoring be assumed (bivariate probability calculated as mininum of the marginal probabilities)
##' @param obs.only Return data with uncensored observations only
##' @param weight.name Name of weight variable in the new data.frame
##' @param indi.weight Name of individual censoring weight  in the new data.frame
##' @param trunc.prob If TRUE truncation probabilities are also calculated and stored in 'weight.name2' (based on Clayton-Oakes gamma frailty model)
##' @param weight.name2 Name of truncation probabilities
##' @param cens.model Censoring model (default Aalens additive model)
##' @param pairs For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)
##' @param theta.formula Model for the dependence parameter in the Clayton-Oakes model (truncation only)
##' @param ... Additional arguments to censoring model 
##' @author Klaus K. Holst
##' @examples
##' \dontrun{
##' data("prt",package="mets")
##' prtw <- ipw(Surv(time,status==0)~country, data=prt[sample(nrow(prt),5000),],
##'             cluster="id",weight.name="w")
##' plot(0,type="n",xlim=range(prtw$time),ylim=c(0,1),xlab="Age",ylab="Probability")
##' count <- 0
##' for (l in unique(prtw$country)) {
##'     count <- count+1
##'     prtw <- prtw[order(prtw$time),]
##'     with(subset(prtw,country==l),
##'          lines(time,w,col=count,lwd=2))
##' }
##' legend("topright",legend=unique(prtw$country),col=1:4,pch=-1,lty=1)
##' }
##' @export
ipw <- function(formula,data,cluster,
                same.cens=FALSE,obs.only=FALSE,
                weight.name="w",
                trunc.prob=FALSE,weight.name2="wt",indi.weight="pr",
                cens.model="aalen", pairs=FALSE,
                theta.formula=~1, ...) { 
## {{{
    ##iid=TRUE,

    ##cens.args <- c(list(formula,n.sim=0,robust=0,data=eval(data)),list(...))
    if (tolower(cens.model)%in%c("weibull","phreg.par","phreg.weibull")) {
        ud.cens <- phreg_weibull(formula, data, ...)
        pr <- predict(ud.cens,
                      newdata = data, times = ud.cens$response$exit,
                      individual.times = TRUE)[, 1]
        noncens <- which(!ud.cens$status)        
    } else { ## {{{ 
        m <- match.call(expand.dots = FALSE)
        m <- m[match(c("", "formula", "data", "subset", "na.action"), 
                     names(m), nomatch = 0)]
        special <- c("strata", "factor", "NN", "cluster", "dummy")
        if (missing(data)) 
            Terms <- terms(formula)
        else Terms <- terms(formula, data = data)
        m$formula <- Terms
        m[[1]] <- as.name("model.frame")
        M <- eval(m, parent.frame())
        censtime <- model.extract(M, "response")
        if (ncol(censtime)==3) {
            status <- censtime[,3]
            otimes <- censtime[,2]
            ltimes <- censtime[,1]
        } else {
            status <- censtime[,2]
            otimes <- censtime[,1]
        }
        noncens <- !status
        if (is.null(attr(terms(formula,"prop"),"specials")$prop)) {
            ud.cens <- timereg::aalen(formula,n.sim=0,robust=0,data=data,...)
            XZ <- model.matrix(formula,data)
###         Gcx <- ud.cens$cum[prodlim::sindex(ud.cens$cum[,1],otimes),-1]
            Gcx<-cpred(ud.cens$cum,otimes)[,-1];            
            Gcx<-exp(-apply(Gcx*XZ,1,sum))            
        } else {
            ud.cens <- timereg::cox.aalen(formula,n.sim=0,robust=0,data=data,...)
            XZ <- model.matrix(formula,data)
            Gcx<-cpred(ud.cens$cum,otimes)[,-1];
            Gcx<-exp(-apply(Gcx*XZ,1,sum))            
        }
        ##ud.cens <- do.call(cens.model,cens.args)
        Gcx[Gcx>1]<-1; Gcx[Gcx<0]<-0
        pr <- Gcx
	data[,indi.weight] <- Gcx
    } ## }}} 

     if (trunc.prob) stop("Under development\n"); 
    if (trunc.prob & ncol(censtime)==3) { ## truncation ## {{{ 
        data$truncsurv <- Surv(ltimes,otimes,noncens)
        trunc.formula <- update(formula,truncsurv~.)        
        ud.trunc <- timereg::aalen(trunc.formula,data=data,robust=0,n.sim=0,residuals=0,silent=1)
        dependX0 <- model.matrix(theta.formula,data)        
        twostage.fit <- timereg::two.stage(ud.trunc,data=data,robust=0,detail=0,
			         clusters=data[,cluster],
				 theta.des=dependX0)#,Nit=20,step=1.0,notaylor=1)
	pre.theta <- dependX0 %*% twostage.fit$theta
        X <- model.matrix(formula,data)
        Xnam <- colnames(X)
	X <- cbind(X,pre.theta)
	colnames(X)[ncol(X)] <- "pre.theta"
        ww <- fast.reshape(cbind(X,".num"=seq(nrow(X)),".lefttime"=ltimes),varying=c(".num",".lefttime"),id=data[,cluster])
	if (anyNA(ww)) stop("not balanced for predict two stage\n");  
        Prob <- timereg::predict.two.stage(twostage.fit,X=ww[,Xnam],
                                  times=ww[,".lefttime1"],times2=ww[,".lefttime2"],
                                  theta=ww$pre.theta)
        P0 <- numeric(nrow(X))
        P0[ww[,".num1"]] <- Prob$St1t2
        P0[ww[,".num2"]] <- Prob$St1t2
        data[,weight.name2] <- P0
	print(summary(P0))
    }
    data[,weight.name] <- pr
    ## }}} 
        
    if (same.cens & missing(cluster)) message("no cluster for same-cens given \n"); 

    if (same.cens & !missing(cluster)) { ## {{{         
###        message("Minimum weights...")
        myord <- order(data[,cluster])
        data <- data[myord,,drop=FALSE]
        id <-  table(data[,cluster])
        if (pairs) {
            gem <- data[,cluster]%in%(names(id)[id==2])
            id <- id[id==2]
            data <- data[gem,]
        }
        d0 <- subset(data,select=c(cluster,weight.name))
        noncens <- with(data,!eval(terms(formula)[[2]][[3]]))
        d0[,"observed."] <- noncens
        timevar <- paste("_",cluster,weight.name,sep="")
        d0[,timevar] <- unlist(lapply(id,seq))
        Wide <- reshape(d0,direction="wide",timevar=timevar,idvar=cluster)
        W <- apply(Wide[,paste(weight.name,1:2,sep=".")],1,
                   function(x) min(x,na.rm=TRUE))
        Wmarg <- d0[,weight.name]
        data[,weight.name] <- 1/Wmarg
        Wmin <- rep(W,id)
        
        obs1only <- rep(with(Wide, observed..1 & (is.na(observed..2) | !observed..2)),id)
        obs2only <- rep(with(Wide, observed..2 & (is.na(observed..1) | !observed..1)),id)
        obsOne <- which(na.omit(obs1only|obs2only))
        obsBoth <- rep(with(Wide, !is.na(observed..1) & !is.na(observed..2) & observed..2 & observed..1),id)

        data[obsBoth,weight.name] <-
            ifelse(noncens[obsBoth],1/Wmin[obsBoth],0)    
        data[obsOne,weight.name] <-
            ifelse(noncens[obsOne],1/Wmarg[obsOne],0)
    } ## }}} 

    if (obs.only)
        data <- data[noncens,,drop=FALSE]
    
    return(data)    
} ## }}} 

##' Internal function.
##' Calculates Inverse Probability of Censoring and Truncation 
##' Weights and adds them to a data.frame
##'
##' @title Inverse Probability of Censoring Weights
##' @param data data frame
##' @param times possible time argument for speciying a maximum value of time tau=max(times), to specify when things are considered censored or not.
##' @param entrytime nam of entry-time for truncation. 
##' @param time name of time variable on data frame.  
##' @param cause name of cause indicator on data frame.
##' @param same.cens For clustered data, should same censoring be assumed and same truncation (bivariate probability calculated as mininum of the marginal probabilities)
##' @param cluster name of clustering variable
##' @param pairs For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)
##' @param strata  name of strata variable to get weights stratified.
##' @param obs.only Return data with uncensored observations only
##' @param cens.formula model for Cox models for truncation and right censoring times.
##' @param cens.code censoring.code 
##' @param pair.cweight Name of weight variable in the new data.frame for right censorig of pairs
##' @param pair.tweight Name of weight variable in the new data.frame for left truncation of pairs
##' @param pair.weight Name of weight variable in the new data.frame for right censoring and left truncation of pairs
##' @param cname Name of weight variable in the new data.frame for right censoring of individuals
##' @param tname Name of weight variable in the new data.frame for left truncation of individuals
##' @param weight.name Name of weight variable in the new data.frame for right censoring and left truncation of individuals
##' @param prec.factor To let tied censoring and truncation times come after the death times. 
##' @param ... Additional arguments to censoring model 
##' @author Thomas Scheike 
##' @examples
##' library("timereg")
##' set.seed(1)
##' d <- sim_nordic_random(5000,delayed=TRUE,ptrunc=0.7,
##'       cordz=0.5,cormz=2,lam0=0.3,country=FALSE)
##' d$strata <- as.numeric(d$country)+(d$zyg=="MZ")*4
##' times <- seq(60,100,by=10)
##' ## c1 <- timereg::comp.risk(Event(time,cause)~1+cluster(id),data=d,cause=1,
##' ## 	model="fg",times=times,max.clust=NULL,n.sim=0)
##' ## mm=model.matrix(~-1+zyg,data=d)
##' ## out1<-random_cif(c1,data=d,cause1=1,cause2=1,same.cens=TRUE,theta.des=mm)
##' ## summary(out1)
##' ## pc1 <- predict(c1,X=1,se=0)
##' ## plot(pc1)
##' ## 
##' ## dl <- d[!d$truncated,]
##' ## dl <- ipw2(dl,cluster="id",same.cens=TRUE,time="time",entrytime="entry",cause="cause",
##' ##            strata="strata",prec.factor=100)
##' ## cl <- timereg::comp.risk(Event(time,cause)~+1+
##' ## 		cluster(id),
##' ##  		data=dl,cause=1,model="fg",
##' ## 		weights=dl$indi.weights,cens.weights=rep(1,nrow(dl)),
##' ##           times=times,max.clust=NULL,n.sim=0)
##' ## pcl <- predict(cl,X=1,se=0)
##' ## lines(pcl$time,pcl$P1,col=2)
##' ## mm=model.matrix(~-1+factor(zyg),data=dl)
##' ## out2<-random_cif(cl,data=dl,cause1=1,cause2=1,theta.des=mm,
##' ##                  weights=dl$weights,censoring.weights=rep(1,nrow(dl)))
##' ## summary(out2)
##' @export
ipw2 <- function(data,times=NULL,entrytime=NULL,time="time",cause="cause",
     same.cens=FALSE,cluster=NULL,pairs=FALSE,
     strata=NULL,obs.only=TRUE,cens.formula=NULL,cens.code=0,
     pair.cweight="pcw",pair.tweight="ptw",pair.weight="weights",
     cname="cweights",tname="tweights",weight.name="indi.weights",
     prec.factor=100,...)
{ ## {{{ 
   ### first calculates weights based on marginal
   ### estimators  of censoring and truncation

## {{{  weights, up at T_i or min(T_i,max(times))
   if (is.null(times)) times <- max(data[,time])
   if (is.null(entrytime)) entry <- rep(0,nrow(data)) else entry <- data[,entrytime]
   mtt <- max(times)
   prec <- .Machine$double.eps * prec.factor
   trunc.model <- cens.model <- NULL ## output of Cox models for entry cens

   if (is.null(cens.formula)) { 
   if (is.null(strata)) { ## {{{ 
	   if (!is.null(entrytime)) {
		   surv.trunc <- survival::survfit(Surv(-data[,time],-entry+prec,rep(1,nrow(data))) ~ 1) 
		   trunc.dist <- summary(surv.trunc)
		   trunc.dist$time <- rev(-trunc.dist$time)
		   trunc.dist$surv <- c(rev(trunc.dist$surv)[-1], 1)
		   Lfit <-cpred(cbind(trunc.dist$time,trunc.dist$surv),data[,time],strict=TRUE)
		   Lw <- Lfit[,2]
	   } else { Lw <- 1; }
	   if (!is.null(entrytime)) 
	   ud.cens<- survival::survfit(Surv(entry,data[,time],data[,cause]==0)~+1) 
           else ud.cens<- survival::survfit(Surv(data[,time],data[,cause]==0)~+1) 
	   Gfit<-cbind(ud.cens$time,ud.cens$surv)
	   Gfit<-rbind(c(0,1),Gfit); 
	   Gcx<-cpred(Gfit,pmin(mtt,data[,time]),strict=TRUE)[,2];
           weights <- 1/(Lw*Gcx); 
	   cweights <-  Gcx; 
	   tweights <-  Lw; 
   ### ## }}} 
   } else { ## {{{ 
	   ### compute for each strata and combine 
	  vstrata <- as.numeric(factor(data[,strata]))
          weights <- rep(1,nrow(data))
          cweights <- rep(1,nrow(data))
          tweights <- rep(1,nrow(data))
	  for (i in unique(vstrata)) { ## {{{ for each strata
	       who <- (vstrata == i)
	       if (sum(who) <= 1) stop(paste("strata",i,"less than 1 observation\n")); 
	   datas <- subset(data,who)
	   entrytimes <- entry[who]
	   if (!is.null(entrytime)) {
		   surv.trunc <- survival::survfit(Surv(-datas[,time],-entrytimes+prec,rep(1,nrow(datas))) ~ +1) 
		   trunc.dist <- summary(surv.trunc)
		   trunc.dist$time <- rev(-trunc.dist$time)
		   trunc.dist$surv <- c(rev(trunc.dist$surv)[-1], 1)
		   Lfit <-cpred(cbind(trunc.dist$time,trunc.dist$surv),datas[,time],strict=TRUE)
		   Lw <- Lfit[,2]
	   } else {Lw <- 1; }
	   if (!is.null(entrytime)) 
	   ud.cens<- survival::survfit(Surv(entrytimes,datas[,time],datas[,cause]==0)~+1) 
           else ud.cens<- survival::survfit(Surv(entrytimes,datas[,time],datas[,cause]==0)~+1) 
	   Gfit<-cbind(ud.cens$time,ud.cens$surv)
	   Gfit<-rbind(c(0,1),Gfit); 
	   Gcx<-cpred(Gfit,pmin(mtt,datas[,time]),strict=TRUE)[,2];
	   weights[who]<-  1/(Lw*Gcx); 
	   cweights[who] <-  Gcx; 
	   tweights[who] <-  Lw; 
          } ## }}} 
   } ## }}} 
   } else { ### cens.formula Cox models  ## {{{
        X <- model.matrix(cens.formula,data=data)[,-1,drop=FALSE]; 

	if (!is.null(entrytime)) {
###		trunc.model <- timereg::cox.aalen(Surv(-data[,time],-entrytime+prec,rep(1,nrow(data))) ~ prop(X))
		trunc.model <- survival::coxph(Surv(-data[,time],-entrytime+prec,rep(1,nrow(data))) ~ X) 
###                baseout <- cens.model$cum 
		baseout <- survival::basehaz(trunc.model,centered=FALSE); 
		baseout <- cbind(rev(-baseout$time),rev(baseout$hazard))
	###
		Lfit <-cpred(baseout,data[,time],strict=TRUE)[,-1]
		RR<-exp(as.matrix(X) %*% coef(trunc.model))
		Lfit<-exp(-Lfit*RR)
		Lw <- Lfit
        } else {Lw <- 1; }
###
	if (!is.null(entrytime)) 
	cens.model <- survival::coxph(Surv(entrytime,data[,time],data[,cause]==0)~+X) 
        else cens.model <- survival::coxph(Surv(data[,time],data[,cause]==0)~+X) 
        baseout <- survival::basehaz(cens.model,centered=FALSE); 
###        baseout <- cens.model$cum 
	baseout <- cbind(baseout$time,baseout$hazard)
	Gfit<-cpred(baseout,pmin(mtt,data[,time]),strict=TRUE)[,2];
	RR<-exp(as.matrix(X) %*% coef(cens.model))
	Gfit<-exp(-Gfit*RR)
        weights <- 1/(Lw*Gfit); 
        cweights <- Gfit
        tweights <- Lw
   } ## }}} 

   data[,weight.name] <- weights
   data[,"cw"] <- 1

   if (!is.null(entrytime)) {
   mint <- min(tweights); maxt <- min(tweights) 
   if (mint<0 | mint>1) warning("min(truncation weights) strange, maybe prec.factor should be different\n")
   if (maxt<0 | maxt>1) warning("max(truncation weights) strange, maybe prec.factor should be different\n")
   }

   if (!is.null(entrytime)) {
   attr(data,"trunc.model") <- trunc.model
   attr(data,"cens.model") <- cens.model 
   }
   data[,cname] <- cweights
   data[,tname] <- tweights
## }}} 

  observed <- ((data[,time]>mtt & data[,cause]==cens.code)) | (data[,cause]!=cens.code)
  data[,"observed."] <- observed

  if (same.cens & is.null(cluster)) message("no cluster for same-cens given \n"); 
    if (same.cens & !is.null(cluster)) { ## {{{         
###        message("Minimum weights.cens..")
###        message("Maximum weights.trunc..")
       myord <- order(data[,cluster])
       data <- data[myord,,drop=FALSE]
       id <-  table(data[,cluster])
       observed <- ((data[,time]>mtt & data[,cause]==cens.code)) | (data[,cause]!=cens.code)
       d0 <- data[,c(cluster,cname,tname,time,cause,strata)]
       noncens <- observed
       d0[,"observed."] <-observed
       timevar <- paste("_",cluster,cname,sep="")
       d0[,timevar] <- unlist(lapply(id,seq))
###    print(head(d0))
###    Wide <- reshape(d0,direction="wide",timevar=timevar,idvar=cluster)
       Wide <- fast.reshape(d0,id=cluster)
       ### censoring same cens

       cause1 <- paste(cause,"1",sep=""); cause2 <- paste(cause,"2",sep="")
       obs1 <- "observed.1"; obs2 <- "observed.2"
       time1 <- paste(time,"1",sep=""); time2 <- paste(time,"1",sep="")
###       print(c(time1,time2,cause1,cause2))
###       print(head(Wide))

###       tmax <- apply(Wide[,paste(time,1:2,sep="")],1,
###                   function(x) max(x)) ## NA when there is just one
###       maxstat <- ifelse(Wide[,time1]> Wide[,time2],Wide[,cause1],Wide[,cause2])
###       obsstat <- ifelse(Wide[,time1]> Wide[,time2],Wide[,obs1],Wide[,obs2])*1
###       print(head(tmax,10)); print(head(maxstat,10))
###       print(names(Wide))
###       datp <- data.frame(time=tmax,cause=obsstat,strata=Wide[,paste(strata,"1",sep="")])
###       print(head(datp))
###       datp <- pred.stratKM(datp)
###       print(head(tmax,datp))
###
###       ud.cens<- survival::survfit(Surv(tmax,obsstat==0)~+1) 
###       Gfit<-cbind(ud.cens$time,ud.cens$surv)
###       Gfit<-rbind(c(0,1),Gfit); 
###       tmaxna <- tmax; tmaxna[is.na(tmax)] <- 0
###       Gcx<-cpred(Gfit,pmin(mtt,tmaxna),strict=TRUE)[,2];
###       Wd <- Gcx
###
###      data[,"tmax"] <- rep(tmax,id)
###	data[,"stattmax"] <- rep(maxstat,id)

       W <- apply(Wide[,paste(cname,1:2,sep="")],1,
                   function(x) min(x)) ## NA when there is just one
###                   function(x) min(x,na.rm=TRUE))
        Wmin <- rep(W,id)
	data[,pair.cweight] <- 1/Wmin
###	data[,paste(pair.cweight,"max",sep="")] <- 1/rep(Wd,id)

	### when pair-weight is NA takes individual weight
	naW <- is.na(Wmin)
	data[naW,pair.cweight] <- 1/data[naW,cname]
	data[naW,paste(pair.cweight,"max",sep="")] <- 1/data[naW,cname] 

   ### truncation same truncation  
   if (!is.null(entrytime)) { 
        Wt <- apply(Wide[,paste(tname,1:2,sep="")],1,
                   function(x) min(x)) ## NA when there is just one
###        Wtmarg <- d0[,tname]
###        data[,pair.tweight] <- 0; ##1/Wtmarg
        Wtmax <- rep(Wt,id)
	data[,pair.tweight] <- 1/Wtmax

	### when pair-weight is NA takes individual weight
	naW <- is.na(Wtmax)
	data[naW,pair.tweight] <- 1/data[naW,tname]
   }

    if (!is.null(entrytime)) { 
    data[,pair.weight] <- data[,pair.cweight]*data[,pair.tweight]
    } else data[,pair.weight] <- data[,pair.cweight]
    } ## }}} 

    if (obs.only) { data <- data[observed,] } 

    id <-  table(data[,cluster])
    if (pairs) {
          gem <- data[,cluster]%in%(names(id)[id==2])
          id <- id[id==2]
          data <- data[gem,]
    } else data[,"pairs."] <- data[,cluster]%in%(names(id)[id==2])

   return(data)
} ## }}} 

##' Fast Cluster Index Conversion
##'
##' Converts a cluster variable to consecutive numeric indices using compiled code.
##'
##' @param x integer vector of cluster identifiers.
##' @param ... additional arguments (not used).
##' @return Integer vector of 0-based cluster indices.
##' @export
fast.cluster <- function(x,...) {
    arglist <- list("FastCluster",
                    time=as.integer(x),
                    PACKAGE="mets")
    res <- do.call(".Call",arglist)
    return(as.vector(res))
}

##' Cumulative Incidence Function (CIF) Regression
##'
##' Fits a regression model for the cumulative incidence function (CIF) in the presence of competing risks.
##' Supports two link functions:
##' \itemize{
##' \item \code{propodds=1} (default): Logistic link model (logit of CIF), providing Odds Ratio (OR) interpretations.
##' \item \code{propodds=NULL}: Fine-Gray (cloglog) regression model, providing subdistribution hazard ratio interpretations.
##' }
##'
##' For the Fine-Gray model, the score equations are:
##' \deqn{ \int (X - E(t)) Y_1(t) w(t) dM_1 }
##' summed over clusters and returned as \code{iid.naive} (multiplied by the inverse of the second derivative).
##' Here, \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))},
##' \deqn{E(t) = S_1(t)/S_0(t)}, and \deqn{S_j(t) = \sum X_i^j Y_{i1}(t) w_i(t) \exp(X_i^T \beta)}.
##'
##' The full influence function (IID decomposition) for the beta coefficients includes a censoring term:
##' \deqn{ \int (X - E(t)) Y_1(t) w(t) dM_1 + \int q(s)/p(s) dM_c }
##' which is returned as the \code{iid} component.
##'
##' For the logistic link model, standard errors may be slightly underestimated because uncertainty from the
##' recursive baseline estimation is not fully accounted for. For smaller datasets, it is recommended to use
##' the \code{prop.odds.subdist} function from the \pkg{timereg} package (which uses more efficient weights)
##' or to bootstrap the standard errors.
##'
##' @param formula Formula with an 'Event' outcome.
##' @param data Data frame containing the variables.
##' @param propodds Logical; if \code{1} (default), fits the logit link model. If \code{NULL}, fits the Fine-Gray model.
##' @param cause Cause of interest (default is 1).
##' @param cens.code Code for censoring (default is 0).
##' @param no.codes Event codes to be ignored when identifying competing causes (useful for administrative censoring).
##' @param death.code Codes for death (terminal events). If \code{NULL}, defaults to all remaining codes (excluding \code{cause}, \code{cens.code}, and \code{no.codes}).
##' @param ... Additional arguments passed to \code{recreg}.
##'
##' @return An object of class \code{"cifreg"} (extending \code{"phreg"}) containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{iid}{Influence functions for the coefficients.}
##' \item{cumhaz}{Cumulative incidence estimates.}
##' \item{propodds}{Indicator of the link function used.}
##'
##' @author Thomas Scheike
##' @seealso \code{\link{cifregFG}}, \code{\link{recreg}}, \code{\link{gofFG}}
##' @aliases vecAllStrata diffstrata FGprediid indexstratarightR gofFG 
##' @examples
##' ## data with no ties
##' data(bmt,package="mets")
##' bmt$time <- bmt$time+runif(nrow(bmt))*0.01
##' bmt$id <- 1:nrow(bmt)
##'
##' ## logistic link  OR interpretation
##' or=cifreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' summary(or)
##' par(mfrow=c(1,2))
##' plot(or)
##' nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
##' por <- predict(or,nd)
##' plot(por)
##'
##' ## approximate standard errors 
##' por <-mets:::predict.phreg(or,nd)
##' plot(por,se=1)
##'
##' ## Fine-Gray model
##' fg=cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' summary(fg)
##' ##fg=recreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1,death.code=2)
##' ##summary(fg)
##' plot(fg)
##' nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
##' pfg <- predict(fg,nd,se=1)
##' plot(pfg,se=1)
##' 
##' ## bt <- iidBaseline(fg,time=30)
##' ## bt <- IIDrecreg(fg$cox.prep,fg,time=30)
##' 
##' ## not run to avoid timing issues
##' ## gofFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' 
##' sfg <- cifregFG(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
##' summary(sfg)
##' plot(sfg)
##'
##' ### predictions with CI based on iid decomposition of baseline and beta
##' ### these are used in the predict function above
##' fg <- cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' Biid <- iidBaseline(fg,time=20)
##' pfg1 <- FGprediid(Biid,nd)
##' pfg1
##' @aliases vecAllStrata diffstrata FGprediid indexstratarightR gofFG 
##' @export
cifreg  <- function(formula,data,propodds=1,cause=1,cens.code=0,no.codes=NULL,death.code=NULL,...)
{# {{{
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset","strataAugment")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y, "Event")) stop("Expected a 'Event'-object, with codes for terminal events (death.code if any), censoring (cens.code), and event of interest (cause)")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }

    ## default version of codes, otherwise call recregN directly
    all.codes <-  unique(status)
    codes <- c(cause,cens.code) 
    if (!is.null(no.codes)) codes <- c(codes,no.codes) 
    mcodes <- match(codes,all.codes,nomatch=0)
    ## default death.code are all other codes 
    if (is.null(death.code)) death.code <- all.codes[-mcodes]

    cif <- recreg(formula,data,propodds=propodds,cause=cause,cens.code=cens.code,death.code=death.code,...)
    class(cif) <- c("cifreg","phreg")
    cif$call <- cl
return(cif)
} # }}}

##' Fine-Gray Cumulative Incidence Function Regression
##'
##' Convenience wrapper for \code{cifreg} that specifically fits the Fine-Gray model
##' by setting \code{propodds=NULL}. This provides subdistribution hazard ratio interpretations.
##'
##' @param formula Formula with an 'Event' outcome.
##' @param data Data frame.
##' @param propodds Set to \code{NULL} to fit the Fine-Gray model (default behavior of this function).
##' @param ... Additional arguments passed to \code{cifreg}.
##'
##' @return An object of class \code{"cifreg"} (extending \code{"phreg"}) with \code{propodds=NULL}.
##'
##' @author Thomas Scheike
##' @seealso \code{\link{cifreg}} 
##' @export
cifregFG  <- function(formula,data,propodds=NULL,...)
{# {{{
cif <- cifreg(formula,data,propodds=propodds,...)
return(cif)
} # }}}

##' @export
iidBaseline.cifreg <- function(object,time=NULL,...)
{# {{{
  out <- iidBaseline.recreg(object,time=time,...)
  return(out)
} # }}}


##' @export
IC.cifreg <- function(x, time=NULL,all=FALSE,...) {# {{{
  if (!is.null(time)) {
    iid <- iidBaseline.recreg(x, time=time,...)
    if (all)  {
       res <- with(iid, cbind(beta.iid,base.iid) * NROW(base.iid))
       coefs <- c(iid$coef,iid$cumhaz.time) 
       colnames(res) <- names(coefs)
       attr(res, "coef") <- coefs
    } else { 
        res <- with(iid, base.iid * NROW(base.iid))
        attr(res, "coef") <- c(iid$cumhaz.time)
    }

    attr(res, "strata.level") <- iid$strata.level
    attr(res, "time") <- time
    return(res)
  }
  res <- with(x, iid * NROW(iid))
  return(res)
}
# }}}

##' @export
estimate.cifreg <- function(x, ..., time = NULL, all=FALSE, baseline.args = list()) {
  if (NCOL(model.matrix(x))==0L & is.null(time)) stop("Non-parametric model; need `time` argument")
  ic <- do.call(IC, c(list(x, time = time,all=all), baseline.args))
  cc <- attr(ic, "coef")
  if (is.null(cc)) cc <- coef(x)
###  lab <- names(cc)
  if (!is.null(time)) {
    lab <- x$strata.level
    if (is.null(lab)) lab <- "cif"
  }
  b <- lava::estimate( coef = cc, IC = ic)
  return(lava::estimate(b, ...))
}

##' @export
plot.cifreg <- function(x,se=FALSE,ylab=NULL,...) { ## {{{
   if (inherits(x,"cifreg") & is.null(ylab)) ylab <- "Cumulative Hazard"
if (!se) baseplot(x,se=se,ylab=ylab,...)
else {
    warning("Standard errors approximative (but too small), use predict and type='cumhaz' \n")
    baseplot(x,se=se,ylab=ylab,...)
}
} ## }}}

##' @export
predict.cifreg <- function(object,newdata=NULL,se=FALSE,times=NULL,np=50,...) { ## {{{
if (isTRUE(object$propodds==1) && se) {se <- FALSE; warning("standard errors not computed for logit link\n"); }
if (!se) out <- predict.phreg(object,newdata,se=se,times=times,...)
else {
  out <- predictrecreg(object,newdata,times=times,np=np,...)
}
class(out) <- c("predictcifreg",class(object)[1])
return(out)
} ## }}}

##' @export
summary.predictcifreg <- function(object,times=NULL,type=c("cif","cumhaz","surv")[1],...) {# {{{
ret <- summary.predictrecreg(object,type=type[1],times=times,...)
return(ret)
}# }}}

##' @export
print.predictcifreg <- function(x,...) {# {{{
out <- summary.predictrecreg(x,...)
cat("Predictions displayed, for rows:\n")
print(out$rows)
if (!is.null(out$call.times))  {
  cat("Predictions based on predict object, for times:\n")
  print(out$times)
}
print(out)
}# }}}


##' @export
plot.predictcifreg <- function(x,se=FALSE,ylab=NULL,type="cif",...)
{ ## {{{
  if (inherits(x,"predictcifreg") & is.null(ylab)) ylab <- "Probability"
  plotpredictphreg(x,se=se,ylab=ylab,type=type[1],...)
} ## }}}

##' @export
gofFG <- function(formula,data,cause=1,cens.code=0,cens.model=NULL,...)
{# {{{
fgform <- update(formula, paste("Surv(fgstart, fgstop, fgstatus) ~ .+cluster(id__)"))
## assumes simple Surv(time,status) is given 
vars <- all.vars(formula)
data$id__ <- 1:nrow(data)
formid <- update.formula(formula,paste(".~.+id__"))

## process types to get type of interst and other types 
status <- data[,vars[2]]
types <- unique(status)
mm <- match(c(cens.code,cause),types)
mm <- mm[!is.na(mm)]
statusS <- status
if (length(types[-mm]) == 0) stop("No competing risk codes found; gofFG requires at least one competing event.")
statusS[!(status %in% c(cens.code,cause))] <- types[-mm][1]
typesF <- c(cens.code,cause,types[-mm][1])
###
###data[,vars[2]] <- factor(statusS,typesF,c("censoring","cause","ocause"))
data[,vars[2]] <- factor(statusS,typesF,typesF)

Xs <- vars[-(1:2)]
modP <- paste(Xs,collapse="+") 
formid <- as.formula(paste("Surv(",vars[1],",",vars[2],")~",modP,"+id__"))
if (!is.null(cens.model)) {
Cstrata <- as.character(cens.model)
formid <- as.formula(paste("Surv(",vars[1],",",vars[2],")~",modP,"+",Cstrata[2],"+id__"))
}

fgdata <- finegray(formid,data=data)
fgcph <- phreg(fgform,data=fgdata,weights=fgdata$fgwt,...)
ggmg <- gof(fgcph)
return(ggmg)
}# }}}

#####' @export
indexstratarightR <- function(timeo,stratao,jump,js,nstrata,type="right")# {{{
{
###    if (any(stratao < 0 | stratao >= nstrata))
###        stop("time-strata strata not between 0-(nstrata-1)\n")
###    if (any(js < 0 | js >= nstrata))
###        stop("jump-strata strata not between 0-(nstrata-1)\n")
    mm <- cbind(timeo, stratao, 1:length(timeo), 0)
    mm <- rbind(mm, cbind(jump, js, 1:length(jump), 1))
    ord <- order(mm[, 1], mm[, 4])
    mm <- mm[ord, ]
    if (type == "right") right <- 1 else right <- 0
    ires <- .Call("indexstrataR", mm[, 2], mm[, 3], mm[, 4], nstrata,right)
    res <- ires$res
    or2 <- order(res[,2])
    res <- res[or2,1]
    reso <- which(res==0) 
    if (length(reso)>0) {
       jso <- js[reso]
        # strata indices from C are 0-based, so compare against s-1
       for (s in 1:nstrata) if (length(jso)>0) res[reso[jso==s-1]] <- ires$maxmin[s] 
    }
    return(res)
} ## }}}

##' @export
FGprediid <- function(iidBase,newdata,conf.type=c("log","cloglog","plain"),model="FG")
{# {{{
  if (!inherits(iidBase,"iidBaseline")) stop("must be a iidBaseline object with iid decomposition \n"); 
	if (!is.null(newdata)) xx <- update_design(iidBase$design,data = newdata,response=FALSE) else xx <- iidBase$design
	X <- xx$x
	des <- list()
	des$X <- X
	if (!is.null(xx$strata)) strataNew <- as.numeric(xx$strata)-1 else strataNew <- rep(0,nrow(X))
###	des$strata <- strataNew

###  des <- readPhreg(iidBase,newdata)
  strata <- strataNew

  if (!is.null(iidBase$beta.iid))  { 
	  fixbeta <- 0; beta.iid <- iidBase$beta.iid; X <- des$X; p <- ncol(beta.iid); 
  } else { fixbeta <- 1; beta.iid <- 0; X <- matrix(0,1,1); p <- 1 }

  sus <- sort(unique(strata))
  At <- iidBase$cumhaz.time[match(sus,iidBase$strata.time)]

   if (nrow(X)==0) X <- matrix(0,1,p)
   if (ncol(X)!=p) stop("X and coef does not match \n"); 

   Ft <- function(coef,Xi=rep(0,length(p)),type="log") {
	   base <- coef[1]
	   p <- coef[-1]
   if (model=="FG") {
       if (type=="log")     y <- log(1-exp(-base*exp(Xi %*% p)))
       if (type=="plain")   y <- 1-exp(-base*exp(Xi %*% p))
       if (type=="cloglog") y <- log(-log(1-exp(-base*exp(Xi %*% p))))
   } else { ## Ghosh-Lin model
       if (type=="log")     y <- log(base*exp(Xi %*% p))
       if (type=="plain")   y <- base*exp(Xi %*% p)
   }
       return(y)
   }
   Ftback <- function(p,type="log")
   {
       if (type=="log")     y <- exp(p)
       if (type=="plain")   y <- p
       if (type=="cloglog") y <- exp(-exp(p)) 
       return(y)
   }

   preds <- matrix(0,length(strata),4)

   k <- 0
   for (i in sus) {
      wheres <- which(strata==i) 
      wi <- match(i,iidBase$strata.time)
      At <- iidBase$cumhaz.time[wi]
	if (fixbeta==0)  {
	   iidAB <- cbind(iidBase$base.iid[,wi],iidBase$beta.iid)
	   covv <- crossprod(iidAB)
	   coeff <- c(At,iidBase$coef)
	   for (j in wheres)  {
	      Xj <- X[j,]
	      eud <- estimate(coef=coeff,vcov=covv,f=function(p) Ft(p,Xi=Xj,type=conf.type[1]))
	      cmat <- eud$coefmat
	      cmat <- c(cmat[,-5])
	      cicmat <- Ftback(cmat[c(1,3:4)],type=conf.type[1])
	      cmat[c(1,3:4)] <- cicmat
	      preds[j,] <- c(cmat)
	   } 
	} else {
	      iidAB <- cbind(iidBase$base.iid[,wi])
	      covv <- crossprod(iidAB)
	      coeff <- c(At)
	      eud <- estimate(coef=coeff,vcov=covv,f=function(p) Ft(p,Xi=1,type=conf.type[1]))
	      cmat <- eud$coefmat
	      cmat <- c(cmat[,-5])
	      cicmat <- Ftback(cmat[c(1,3:4)],type=conf.type[1])
	      cmat[c(1,3:4)] <- cicmat
	      preds[wheres,] <- matrix(c(cmat),length(wheres),4,byrow=TRUE)
	}
}

colnames(preds) <- c("pred",paste("se",conf.type[1],sep="-"),"lower","upper")

return(preds)
}# }}}

###strataC <- survival::strata

# cifreg versus crr-cmprsk
library("tinytest")

test_cifreg <- function() {
  library(mets)
  set.seed(100)
  data(bmt)
  bmt$time <-  bmt$time + runif(nrow(bmt)) * 0.001

  fg <- cifregFG(Event(time,cause)~tcell+age+platelet,bmt)

  if (requireNamespace("cmprsk")) {
    library("cmprsk")
    mm <- model.matrix(~ tcell + age + platelet, bmt)[, -1]
    cr <- with(bmt, crr(time,cause,mm))
    mm <-  cbind(cr$coef,
                 diag(cr$var)^.5,
                 fg$coef,
                 fg$se.coef,
                 cr$coef - fg$coef,
                 diag(cr$var)^.5 - fg$se.coef)
    mm
    ## estimate (same) and standard errors (close close)
    expect_true( ((sum(abs(mm[, 5]))) < 0.0001) & ((sum(abs(mm[,6])))<0.0001)   )
  }
}
test_cifreg()

##' @export
coef.timemets <- function(object,...) {    
    res <- unlist(lapply(object$summary,function(x) x$estimate[,1]))
    if (!is.null(names(object$var))) {
        nn <- names(object$var)[seq(length(object$summary))]
    } else {
        nn <- object$var[seq(length(object$summary))]
    }
    names(res) <- paste(names(res),rep(nn,length(res)/length(nn)),sep=":")
    return(res)
}

##' @export
vcov.timemets <- function(object,...) {
    if (object$type=="strata") {
        return(do.call(blockdiag,
                       lapply(object$summary, function(x) x$vcov)))
    }
}


##' @export
twinlm.strata <- function(formula,data,var,breaks,quantiles,...) {
    varname <- ""
    if (is.character(var) && length(var)==1) {
        varname <- var
        var <- data[,var]
    }
    ## 
    if (!missing(quantiles)) breaks <- quantile(var,quantiles,include.lowest=TRUE)
    if (!missing(breaks)) {
        var <- cut(var,c(breaks))
    }
    if (!inherits(var,c("character","factor","logical"))) stop("Expected factor, character, or logical vector")
    lev <- na.omit(unique(var))
    res <- list()
    for (i in seq_along(lev)) {
        tau <- lev[i]
        if (length(lev)>1) message(tau)
        data0 <- data[var==tau,,drop=FALSE]
        suppressWarnings(b <- twinlm(formula,data=data0,...))
        res <- c(res,list(summary(b)))
    }
    if (length(lev)==1) return(b)
    if (!missing(breaks)) lev <- breaks
    coef <- c(lapply(res,function(x) x$all),list(res[[length(res)]]$all))
    res <- list(varname=varname,var=lev,coef=coef,summary=res,type="strata")
    class(res) <- "timemets"
    return(res)
}


## data(prt)
## bb <- twinlm.time(cancer~country,data=prt,id="id",zyg="zyg",DZ="DZ",cens.formula=Surv(time,status==0)~zyg,breaks=seq(70,90,by=4))
## plot(bb,which=c(7,11),ylim=c(0,28),legendpos="topright",col=c("darkred","darkblue"),lty=c(1,2),legend=c("MZ","DZ"),ylab="Relative recurrence risk ratio")
## plot(bb,which=c(7,11),ylim=c(0,28),legendpos="topright",col=c("darkred","darkblue"),lty=c(1,2),legend=c("MZ","DZ"),ylab="Relative recurrence risk ratio",type="l")

##' @export
twinlm.time <- function(formula,...) {
    biprobit.time(formula,estimator="bptwin",...)
}

##' @export
bptwin.time <- function(formula,...) {
    biprobit.time(formula,estimator="bptwin",...)
}

##' @export
summary.timemets <- function(object,which=seq(nrow(object$coef[[1]])),...) {
    res <- list()
    for (i in which) {    
        rr <- matrix(unlist(lapply(object$coef,function(z) z[i,])),ncol=3,byrow=TRUE)
        colnames(rr) <- colnames(object$coef[[1]])
        rr <- cbind(object$var,as.data.frame(rr[seq_along(object$var),,drop=FALSE]))
        colnames(rr)[1] <- object$varname
        res <- c(res,list(rr))
    }
    names(res) <- rownames(object$coef[[1]])[which]
    return(res)
}


##' @export
print.timemets <- function(x,tail,row.names=FALSE,digits=4,width=10,...) {
    res <- summary(x,...)
    if (length(res)==0) {
        return(invisible(print(x$coef)))
    }
    if (!is.null(x$summary[[1]]$ncontrast) && x$summary[[1]]$ncontrast>1) {
        cat("Contrasts:\n")
        for (i in seq(x$summary[[1]]$ncontrasts)) {
            cat("   c",i,":\n",sep="")
            cat("\tDependence ", x$summary[[1]]$par[[i]]$corref, "\n")
            if (x$summary[[1]]$model$eqmarg) {
                cat("\tMean       ", x$summary[[1]]$par[[i]]$mref1, "\n")
            } else {
                cat("\tMean 1     ", x$summary[[1]]$par[[i]]$mref1, "\n")
                cat("\tMean 2     ", x$summary[[1]]$par[[i]]$mref2, "\n")
            }
        }
    }
    ## }
    ## if (mcontr2 || (x$contrast & !x$model$eqmarg)) {
    ##     cat("\tMean 1       ", x$par[[i]]$mref1, "\n")
    ##     cat("\tMean 2       ", x$par[[i]]$mref2, "\n")
    ## } 
    ## if (mcontr1 || (x$contrast & x$model$eqmarg))
    ##     cat("\tMean         ", x$par[[i]]$mref1, "\n")
    
    
    M <- res[[1]][,1]
    nn <- c()
    for (i in seq_along(res)) {
        nn <- c(nn,names(res)[i])
        M <- cbind(M,res[[i]][,2])
    }
    nn0 <- cbind(paste(seq(ncol(M)-1),":",nn,sep=""))
    colnames(nn0) <- ""; rownames(nn0) <- rep("",nrow(nn0))
    print(nn0,quote=FALSE)
    cat("\n")
    nn <- unlist(lapply(nn,
                        function(x) {
                            res <- toString(x,width)
                            if (nchar(res)==nchar(x)) return(x)
                            substr(res,1,nchar(res)-1)
                        }))
    nn <- paste(seq(ncol(M)-1),":",nn,sep="")
    colnames(M) <- c("Time",nn)
    if (!missing(tail)) {
        print(utils::tail(round(M,digits=digits),tail),row.names=row.names)
    } else {
        print(round(M,digits=digits),row.names=row.names)
    }
    invisible(M)
}

##' @export
plot.timemets <- function(x,...,which=1,
                            type="s",
                            lwd=2,lty=1,col,fillcol,alpha=0.2,
                            xlab=x$varname,
                            ylab="",idx=seq_along(x$var),
                            lasttick=TRUE,add=FALSE,
                            legend=TRUE,legendpos="topleft") {
    ss <- summary(x,which)
    if (missing(col)) col <- seq_along(which)
    if (length(col)==1) col <- rep(col,length(which))
    if (length(lwd)==1) lwd <- rep(lwd,length(which))
    if (length(lty)==1) lty <- rep(lty,length(which))
    if (alpha>0 & missing(fillcol)) fillcol <- Col(col,alpha)
    count <- 0
    if (add) dev <- devcoords()
    for (tt in seq_along(which)) {
        count <- count+1
        zz <- ss[[tt]][idx,,drop=FALSE]
        if (!add) {
            plot(zz[,1:2,drop=FALSE],lty=0,
                 ylab=ylab,xlab=xlab,type="n",...)
            dev <- devcoords()
        }
        if (lasttick && type=="s" && !is.factor(zz[,1])) {
            zz2 <- rbind(zz,tail(zz,1))
            zz2[nrow(zz2),1] <- dev$fig.x2
            confband(zz2[,1],zz2[,3],zz2[,4],polygon=TRUE,step=(type=="s"),col=fillcol[count],border=0)
        } else {
            if (is.factor(zz[,1])) {
                confband(x=seq(nrow(zz)),lower=zz[,3],upper=zz[,4],center=zz[,2],col=col[count],lwd=lwd[count],lty=lty[count],...)
            } else {
                confband(zz[,1],zz[,3],zz[,4],polygon=TRUE,step=(type=="s"),col=fillcol[count],border=0)
            }
        }
        add <- TRUE
        ## if (lasttick && type=="s") {
        ##     axis(4,at=zz[nrow(zz),3],labels=FALSE,
        ##          lwd=lwd[count],col=fillcol[count],tcl=.5,...)
        ##     axis(4,at=zz[nrow(zz),4],labels=FALSE,
        ##          lwd=lwd[count],col=fillcol[count],tcl=.5,...)
        ## }
        if (!is.factor(zz[,1]))
            lines(zz[,1:2,drop=FALSE],lwd=lwd[count],lty=lty[count],col=col[count],type=type,...)        
    }
    if (!is.null(legend) || (is.logical(legend) && !legend[1])) {
        if (is.logical(legend) || length(legend)==1) legend <- rownames(x$coef[[1]])[which]
        graphics::legend(legendpos,legend=legend,col=col,lwd=lwd,lty=lty)
    }
    invisible(x)
}


##' @export
bootstrap.timemets <- function(x,R=1000,...) {
    
}

by2mat <- function(x,nam,...) {
    nulls <- which(unlist(lapply(x,is.null)))
    nonnulls <- setdiff(seq_along(x),nulls)
    nn <- do.call("expand.grid",attributes(x)$dimnames)
    if (length(nulls)>0) nn <- nn[-nulls,,drop=FALSE]
    res <- Reduce("rbind",x)
    if (is.null(colnames(res)) && !missing(nam)) {
        colnames(res) <- nam[seq(length(ncol(res)))]
    }
    suppressWarnings(res <- cbind(nn,res)) ## no warnings on row-names
    for (i in seq(ncol(res)-1)+1) {
        if (is.list(res[,i])) {
            if (!is.null(nn <- names(res[,i][[1]])))
                colnames(res)[i] <- paste0(colnames(res)[i],"(",paste0(nn,collapse=","),")")
        }
    }
    a <- rownames(x[[1]])
    res$"_var" <- a
    rownames(res) <- seq(nrow(res))
    return(res)
}


##' aggregating for for data frames
##'
##' aggregating for for data frames
##' @examples
##' data("sTRACE")
##' daggregate(iris, "^.e.al", x="Species", fun=cor, regex=TRUE)
##' daggregate(iris, Sepal.Length+Petal.Length ~Species, fun=summary)
##' daggregate(iris, log(Sepal.Length)+I(Petal.Length>1.5) ~ Species,
##'                  fun=summary)
##' daggregate(iris, "*Length*", x="Species", fun=head)
##' daggregate(iris, "^.e.al", x="Species", fun=tail, regex=TRUE)
##' daggregate(sTRACE, status~ diabetes, fun=table)
##' daggregate(sTRACE, status~ diabetes+sex, fun=table)
##' daggregate(sTRACE, status + diabetes+sex ~ vf+I(wmi>1.4), fun=table)
##' daggregate(iris, "^.e.al", x="Species",regex=TRUE)
##' dlist(iris,Petal.Length+Sepal.Length ~ Species |Petal.Length>1.3 & Sepal.Length>5,
##'             n=list(1:3,-(3:1)))
##' daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5))
##' daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5),
##'                  fun=table)
##'
##' dsum(iris, .~Species, matrix=TRUE, missing=TRUE)
##'
##' par(mfrow=c(1,2))
##' data(iris)
##' drename(iris) <- ~.
##' daggregate(iris,'sepal*'~species|species!="virginica",fun=plot)
##' daggregate(iris,'sepal*'~I(as.numeric(species))|I(as.numeric(species))!=1,fun=summary)
##'
##' dnumeric(iris) <- ~species
##' daggregate(iris,'sepal*'~species.n|species.n!=1,fun=summary)
##'
##' @export
##' @param data data.frame
##' @param y name of variable, or formula, or names of variables on data frame.
##' @param x name of variable, or formula, or names of variables on data frame.
##' @param subset subset expression
##' @param ... additional arguments to lower level functions
##' @param fun function defining aggregation
##' @param regex interpret x,y as regular expressions
##' @param missing Missing used in groups (x)
##' @param remove.empty remove empty groups from output
##' @param matrix if TRUE a matrix is returned instead of an array
##' @param silent suppress messages
##' @param na.action How model.frame deals with 'NA's
##' @param convert if TRUE try to coerce result into matrix. Can also be a user-defined function
##' @aliases daggr
daggregate <- function(data,y=NULL,x=NULL,subset,...,fun="summary",regex=mets.options()$regex, missing=FALSE, remove.empty=FALSE, matrix=FALSE, silent=FALSE, na.action=na.pass, convert=NULL)
{# {{{
    if (is.vector(data)) data <- data.frame(data)
    subs <- substitute(subset)
    if (!base::missing(subs)) {
        expr <- suppressWarnings(inherits(try(subset,silent=TRUE),"try-error"))
        if (expr) data <- data[which(eval(subs,envir=data)),,drop=FALSE]
        else data[subset,,drop=FALSE]
    }
    if (is.null(y)) y <- colnames(data)
    if (inherits(y,"formula")) {

        yx <- procformdata(y,sep="\\|",data=data,na.action=na.action,regex=regex,...)
        y <- yx$response
        x0 <- yx$predictor
        if (is.null(x) && length(y)>0) x <- x0
        if (NCOL(x)==0) x <- NULL
        if (length(y)==0) {
            y <- x0
        }
    } else {
        yy <- c()
        for (y0 in y) {
            if (!regex) y0 <- glob2rx(y0)
            n <- grep(y0,names(data),perl=mets.options()$regex.perl)
            yy <- union(yy,names(data)[n])
        }
        y <- data[,yy,drop=FALSE]
    }
    if (is.character(x) && length(x)<NROW(data)) {
        xx <- c()
        for (x0 in x) {
            if (!regex) x0 <- glob2rx(x0)
            n <- grep(x0,names(data),perl=mets.options()$regex.perl)
            xx <- union(xx,names(data)[n])
        }
        x <- data[,xx,drop=FALSE]
    }
    if (inherits(x,"formula")) {
        x <- model.frame(x,data=data,na.action=na.action)
    }
    if (!is.null(x)) {
        xidx <- na.omit(match(colnames(x),colnames(y)))
        if (length(xidx)>0) y <- y[,-xidx,drop=FALSE]
    }
    if (is.character(fun)) fun <- get(fun)
    if (!is.null(convert) && is.logical(convert)) {
        if (convert) convert <- as.matrix
        else convert <- NULL
    }
    if (!is.null(convert)) {
        fun_ <- fun
        fun <- function(x,...) fun_(convert(x,...))
    }
    if (!is.null(x)) {
        if (missing) {
            x[is.na(x)] <- 'NA'
        }
        if (silent) {
                capture.output(res <- by(y,x,fun,...))
        } else {
            res <- by(y,x,fun,...)
        }
        if (remove.empty) {
            # ... need to abandon 'by'?
        }
        if (matrix) {
            res <- by2mat(res,colnames(y))
        }
        return(structure(res, ngroupvar=NCOL(x),
                         class=c("daggregate",class(res))))
    }
    if (silent)
        capture.output(res <- do.call(fun, c(list(y),list(...))))
    else
        res <- do.call(fun, c(list(y), list(...)))
    if (is.null(res)) return(invisible(NULL))
    structure(res, ngroupvar=0, class=c("daggregate", class(res)))
}# }}}

##' @export
daggr <- function(data,...,convert=as.matrix) daggregate(data,...,convert=convert)

##' @export
print.daggregate <- function(x,quote=FALSE,...) {
    attr(x,c("ngroupvar")) <- NULL
    class(x) <- class(x)[-1]
    print(x,quote=quote,...)
}


##' @export
dhead <- function(data,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) utils::head(z,...))

##' @export
dtail <- function(data,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) utils::tail(z,...))

##' @export
dsummary <- function(data=NULL,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) base::summary(z,...))

##' @export
dstr <- function(data,y=NULL,x=NULL,...) invisible(daggregate(data,y,x,fun=function(z) utils::str(z,...)))

##' @export
dunique <- function(data,y=NULL,x=NULL,...) invisible(daggregate(data,y,x,fun=function(z) base::unique(z,...)))


##' summary, tables, and correlations for data frames
##'
##' summary, tables, and correlations for data frames
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x possible group variable
##' @param use how to handle missing values
##' @param ... Optional additional arguments
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##' data("sTRACE",package="timereg")
##' dt<- sTRACE
##' dt$time2 <- dt$time^2
##' dt$wmi2 <- dt$wmi^2
##' head(dt)
##'
##' dcor(dt)
##'
##' dcor(dt,~time+wmi)
##' dcor(dt,~time+wmi,~vf+chf)
##' dcor(dt,time+wmi~vf+chf)
##'
##' dcor(dt,c("time*","wmi*"),~vf+chf)
##' @aliases dsummary dstr dcor dsubset dquantile dcount dmean dmeansd dscalar deval deval2 dsum dsd
##' @export
dcor <- function(data,y=NULL,x=NULL,use="pairwise.complete.obs",...) daggregate(data,y,x,...,fun=function(z,...) stats::cor(z,use=use,...))

##' @export
dscalar <- function(data,y=NULL,x=NULL,...,na.rm=TRUE,matrix=TRUE,fun=base::mean) {
    daggregate(data,y,x,matrix=matrix,...,
               fun=function(z,...) {
                   if (is.matrix(z)) {
                       apply(z,2,function(x)
                           suppressWarnings(tryCatch(fun(x,na.rm=na.rm,...),error=function(e) return(NA))))
                   } else {
                       unlist(lapply(z,function(x) {
                           suppressWarnings(tryCatch(fun(x,na.rm=na.rm,...),error=function(e) return(NA)))
                       }))
                   }
               })
}


Summary <- function(object,na.rm=TRUE,...) {
    if (is.numeric(object)) {
        x <- c(summary(object,...),sd=sd(object,na.rm=TRUE))
    } else {
        x <- summary(object,...)
    }
    ## Formatting
    xx <- x
    if (is.numeric(x) || is.complex(x)) {
        finite <- is.finite(x)
        xx[finite] <- zapsmall(x[finite])
      }
    m <- match("NA's", names(xx), 0)
    if (inherits(x, "Date") || inherits(x, "POSIXct")) {
        xx <- if (length(a <- attr(x, "NAs")))
                 c(format(xx), `NA's` = as.character(a))
             else format(xx)
    }
    else if (m && !is.character(x))
        xx <- c(format(xx[-m]), `NA's` = as.character(xx[m]))
    xx
}

##' @export
deval2 <- function(data,...,matrix=simplify,simplify=TRUE)  deval(data,matrix=TRUE,simplify=TRUE,...)

##' @export
deval <- function(data,y=NULL,x=NULL,...,matrix=FALSE,fun=Summary,simplify=FALSE) {
    if (is.list(fun)) {
        newf <- function(x,...) {
            unlist(lapply(fun,function(f) f(x,...), ...))
        }
    }
    else newf <- fun
    res <- daggregate(data,y,x,matrix=matrix,...,
                     fun=function(z) lapply(z,function(x) {
                         suppressWarnings(tryCatch(newf(x,...),error=function(e) return(NA)))
                     }))
    if (simplify) {
        for (i in seq_len(ncol(res))) {
            if (is.list(res[,i])) res[,i] <- unlist(res[,i])
        }
        ## }
    }
    res
}


##' @export
dmeansd <- function(data,...)  {
	mm <- dscalar(data,fun=base::mean,...)
	vv <- dscalar(data,fun=stats::sd,...)
	colnames(vv) <- paste("sd.",colnames(vv),sep="")
	cbind(mm,vv)
}


##' @export
dmean <- function(data,...) dscalar(data,fun=base::mean,...)

##' @export
dsum <- function(data,...) dscalar(data,fun=base::sum,...)

##' @export
dsd <- function(data,...) dscalar(data,fun=stats::sd,...)


##' @export
dcount <- function(data,x=NULL,...,na.rm=TRUE) {
    res <- rbind(daggregate(data,x=x,matrix=TRUE,...,fun=function(z,...) NROW(z)))
    res[is.na(res)] <- 0
    rownames(res) <- seq(nrow(res))
    colnames(res)[ncol(res)] <- "count"
    res
}


##' @export
dsubset <- function(data,...) {
    daggregate(data,...,fun=function(z) z)
}


##' @export
dquantile <- function(data,y=NULL,x=NULL,probs=seq(0,1,by=1/breaks),breaks=4,matrix=TRUE,reshape=FALSE,...,na.rm=TRUE) {
    a <- daggregate(data,y,x,matrix=FALSE,...,fun=function(z,...) apply(z,2,function(x,...) quantile(x,probs=probs,na.rm=na.rm,...)))
    if (matrix) {
        res <- by2mat(a)
        xidx <- seq_len(attr(a, "ngroupvar"))
        if (!reshape || is.null(res[,"_var"]) || length(xidx)==0) return(res)
        res <- dreshape(res, id=colnames(res)[xidx], num="_var",sep="_")
        return(res)
    }
    return(a)
}

##' @export
summary.bptwin <- function(object,level=0.05,transform=FALSE,...) {
  logit <- function(p) log(p/(1-p))
  tigol <- function(z) 1/(1+exp(-z))
  dlogit <- function(p) 1/(p*(1-p))
  dtigol <- function(z) tigol(z)^2*exp(-z)
  trnam <- " "
  vcoef1 <- paste("var(",c("A","C","D"),")",sep="")
  if (object$transform$invname!="") {
      vcoef1 <- paste(object$transform$invname,"(",vcoef1,")",sep="")
  }
  vcoef2 <- paste("atanh(",
                  c(paste("rho)","MZ",sep=trnam),
                    paste("rho)","DZ",sep=trnam)),sep="")
  idx1 <- na.omit(match(vcoef1,names(coef(object))))
  idx2 <- na.omit(match(vcoef2,names(coef(object))))
  CIs <- c()
  alpha <- level/2
  CIlab <- paste(c(alpha*100,100*(1-alpha)),"%",sep="")
  V <- c()
  if (length(idx2)>0) {
    idx <- idx2
    V <- vcov(object)[idx,idx]
    arho <- coef(object)[idx2[1:2]]
    mz <- multinomlogit(coef(object)[idx2[1]]); names(mz) <- c("U","E")
    dz <- multinomlogit(coef(object)[idx2[2]]); names(dz) <- c("U","E")
    cc <- tanh(arho)
    names(cc) <- c("Tetrachoric correlation MZ","Tetrachoric correlation DZ")
    corMZ <- cc[1]; corDZ <- cc[2]
    D <- diag(object$tr$dtr(arho))
    h <- function(x) 2*(x[1]-x[2])
    dh <- function(x) c(2,-2)
    i1 <- 1:2
    corr <- NULL
  }
  if (length(idx1)>0) {
      idx <- idx1
      V <- vcov(object)[idx,idx]
      ACD <- match(names(coef(object))[idx1],vcoef1)
      nn <- c(c("A","C","D")[ACD],"E")
      dzsc <- c(1/2,1,1/4)[ACD]
      pp <- coef(object)[idx1]
      cc <- multinomlogit(pp,object$transform$tr,object$transform$dtr); names(cc) <- nn
      D <- attributes(cc)$gradient
      vcovACDE <- (D%*%V%*%t(D))
      if (transform) {
          cc2 <- logit(cc)
          D2 <- diag(dlogit(cc))
          DD <- D2%*%D
          Vc2 <- DD%*%V%*%t(DD)
          CIs <- tigol(cc2%x%cbind(1,1)+diag(Vc2)^0.5%x%cbind(-1,1)*qnorm(1-alpha))
      } else {
          CIs <- cbind(cc-qnorm(1-alpha)*sqrt(diag(vcovACDE)),cc+qnorm(1-alpha)*sqrt(diag(vcovACDE)))
      }
      K <- length(ACD)
      Ki <- seq_len(K)
      corMZ <- sum(cc[Ki]); corDZ <- sum(cc[Ki]*dzsc)
      i1 <- na.omit(match(c("D","A"),nn))
      h <- function(x) sum(x)
      dh <- function(x) rep(1,length(i1))
      ##    dh <- function(x) { res <- rep(0,length(x)); res[i1] <- 1; res }
      ##    h <- function(x) 2*(sum(x[i1])-sum(x[i1]*dzsc))
      ##    dh <- function(x) 2*(1-dzsc)
      
  }
  Vc <- D%*%V%*%t(D)
  datanh <- function(r) 1/(1-r^2)
  if (length(idx1)>0) {
    pp <- coef(object)[idx]
    b <- cbind(rep(1,K))
    corMZ.sd <- (t(b)%*%Vc[Ki,Ki]%*%b)[1]^0.5
    corDZ.sd <- (t(dzsc)%*%Vc[Ki,Ki]%*%dzsc)[1]^0.5    
    corr <- rbind(c(corMZ,corMZ.sd),c(corDZ,corDZ.sd))
    zrho <- atanh(corr[,1])
    zrho.var <- datanh(corr[,1])^2*corr[,2]^2
    corr <- cbind(corr, tanh(zrho%x%cbind(1,1)+zrho.var^0.5%x%cbind(-1,1)*qnorm(1-alpha)))
    rownames(corr) <- c("MZ Tetrachoric Cor","DZ Tetrachoric Cor")    
  } else {   
    zrho <- atanh(cc)
    zrho.var <- datanh(cc)^2*diag(Vc)
    CIs <- tanh(zrho%x%cbind(1,1)+zrho.var^0.5%x%cbind(-1,1)*qnorm(1-alpha))
  }
  newcoef <- rbind(cbind(cc,diag(Vc)^0.5,CIs),corr);
  ##  CIs <- rbind(CIs,c(NA,NA),c(NA,NA))
  ##  newcoef <- cbind(newcoef,CIs)
  colnames(newcoef) <- c("Estimate","Std.Err",CIlab)
  H <- h(cc[i1])
  hstd <- (t(dh(cc[i1]))%*%Vc[i1,i1]%*%dh(cc[i1]))^.5
  if (!transform) {
      ci <- c(H-hstd*qnorm(1-alpha),H+hstd*qnorm(1-alpha))
  } else {
      logith <- function(x) logit(h(x))  
      dlogith <- function(x) dlogit(h(x))*dh(x)  
      Dlh <- dlogith(cc[i1])
      sdlh <- (t(Dlh)%*%Vc[i1,i1]%*%(Dlh))[1]^0.5      
      suppressWarnings(ci <- tigol(logith(cc[i1]) + qnorm(1-alpha)*c(-1,1)*sdlh))
  }

  rhoOS <- NULL
  if (object$OS) {
    rEst <- object$coef[nrow(object$coef),1]
    rSE <- object$coef[nrow(object$coef),2]
    rhoOS <- tanh(rbind(c(rEst,rEst+qnorm(1-alpha)*c(-1,1)*rSE)))
    colnames(rhoOS) <- c("Estimate",CIlab)
    if (length(idx1)==0) {
      rownames(rhoOS) <- "Tetrachoric correlation OS"
    } else {
      rownames(rhoOS) <- "Kinship OS"    
    }
  }
      
  concordance <-  conditional <- marg <- c()
  probs <- function(p,idx=1) {
    if (idx==0) {
      m <- p[1]
      ##else m <- p[length(object$midx0)+1]
      S <- object$SigmaFun(p)
      conc1 <- pmvn(upper=c(m,m),sigma=S[[1]])
      conc2 <- pmvn(upper=c(m,m),sigma=S[[2]])
      marg <- pnorm(m,sd=S[[1]][1,1]^0.5)      
      return(logit((conc1-conc2)/(marg*(1-marg))))
    }
    S <- (object$SigmaFun(p))[[idx]]
    m <- 0
    if((object$npar$intercept==1 & idx==1) | object$eqmean) m <- p[1]
    else m <- p[length(object$midx0)+1]
    mu.cond <- function(x) m+S[1,2]/S[2,2]*(x-m)
    var.cond <- S[1,1]-S[1,2]^2/S[2,2]    
    conc <- pmvn(upper=c(m,m),sigma=S)
    disconc <- pmvn(lower=c(-Inf,m),upper=c(m,Inf),sigma=S)
    marg <- pnorm(m,sd=S[1,1]^0.5)
    cond <- conc/marg
    discond <- disconc/(1-marg)
    logOR <- log(cond)-log(1-cond)-log(discond)+log(1-discond)
    lambdaR <- cond/marg
    c(logit(c(conc,cond,marg)),lambdaR,logOR)
  }

  mycoef <- coef(object)
  ## formals(probs) <- alist(p=,idx=0)
  ## hp <- probs(mycoef)
  ## Dhp <- numDeriv::grad(probs,mycoef)
  ## shp <- diag(t(Dhp)%*%vcov(object)%*%(Dhp))^0.5
  
  formals(probs) <- alist(p=,idx=1)
  probMZ <- probs(mycoef)
  
  Dp0 <- numDeriv::jacobian(probs,mycoef)
  formals(probs) <- alist(p=,idx=2)
  probDZ <- probs(mycoef)
  Dp1 <- numDeriv::jacobian(probs,mycoef)
  sprobMZ <- diag((Dp0)%*%vcov(object)%*%t(Dp0))^0.5
  sprobDZ <- diag((Dp1)%*%vcov(object)%*%t(Dp1))^0.5
  probMZ <- cbind(probMZ,probMZ-qnorm(1-alpha)*sprobMZ,probMZ+qnorm(1-alpha)*sprobMZ)
  probMZ[1:3,] <- tigol(probMZ[1:3,])
  probDZ <- cbind(probDZ,probDZ-qnorm(1-alpha)*sprobDZ,probDZ+qnorm(1-alpha)*sprobDZ)
  probDZ[1:3,] <- tigol(probDZ[1:3,])
  rownames(probMZ) <- rownames(probDZ) <- c("Concordance","Casewise Concordance","Marginal","Rel.Recur.Risk","log(OR)")
  colnames(probMZ) <- colnames(probDZ) <- c("Estimate",CIlab)
 
  ## mu <- coef(object)[c(object$bidx0[1],object$bidx1[1])]
  ## Sigma <- list(object$Sigma0,object$Sigma1)
  ## for (i in 1:2) {
  ##   conc <- function()
  ##   mu.cond <- function(x) mu+Sigma[[i]][1,2]/Sigma[[i]][2,2]*(x-mu[i])
  ##   var.cond <- Sigma[[i]][1,1]-Sigma[[i]][1,2]^2/Sigma[[i]][2,2]    
  ##   cc0 <- pmvn(upper=c(mu[i],mu[i]),sigma=Sigma[[i]])
  ##   px <- pnorm(mu[i],sd=Sigma[[i]][2,2]^0.5)
  ##   concordance <- c(concordance,cc0)
  ##   marg <- c(marg,px)
  ##   conditional <- c(conditional,cc0/px)
  ## }
  ## names(concordance) <- names(conditional) <- c("MZ","DZ")
  
  hval <- rbind(c(H,hstd,ci)); colnames(hval) <- c("Estimate","Std.Err",CIlab);
  if (hval[1]>1) hval[1,] <- c(1,NaN,NaN,NaN)

##  hval <- rbind(hval, tigol(c(hp,NA,hp-qnorm(1-alpha)*shp,hp+qnorm(1-alpha)*shp)))
  rownames(hval) <- c("Broad-sense heritability")##,"Risk-scale Heritability")

  Nstr <- object$N
  nN <- ncol(object$N)
  ngroups <- ifelse(object$OS,3,2)
  postn  <- "MZ/DZ"; if (object$OS) postn <- paste(postn,"OS",sep="/")
  npos <- seq(ngroups)
  Nstr <- cbind(paste(Nstr[npos],collapse="/"),
                paste(Nstr[npos+ngroups],collapse="/"),
                paste(Nstr[npos+2*ngroups],collapse="/"))
  rownames(Nstr) <- ""
  colnames(Nstr) <-
    unlist(lapply(strsplit(colnames(object$N)[(1:3)*ngroups],".",fixed=TRUE),
                                  function(x) paste(x[1], postn)))

  all  <-  rbind(hval[,c(1,3,4),drop=FALSE],newcoef[,c(1,3,4),drop=FALSE])
  allprob <- rbind(probMZ,probDZ); rownames(allprob) <-
      c(paste("MZ",rownames(probMZ)),paste("DZ",rownames(probDZ)))
  all <- rbind(all,allprob)  
  
  cc <- object$coef; cc[,2] <- diag(vcov(object))^.5;
  cc[,3] <- cc[,1]/cc[,2]; cc[,4] <- 2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE))
  res <- list(heritability=hval,
              par=cc,
              probMZ=probMZ, probDZ=probDZ, Nstr=Nstr,
              rhoOS=rhoOS,
              coef=newcoef, all=all,
              vcov=vcov(object),
              AIC=AIC(object),
              time=attributes(object)$time,
              logLik=logLik(object)) ##, concordance=concordance, conditional=conditional)

  class(res) <- "summary.bptwin"
  res
}

##' @export
print.summary.bptwin <- function(x,digits = max(3, getOption("digits") - 2),...) {
  cat("\n")
  printCoefmat(x$par,digits=digits,...)
  cat("\n")
  ##  x$Nstr <- x$Nstr[,which((colnames(x$Nstr)!="Complete MZ/DZ")),drop=FALSE]
  NN <- x$Nstr[,c(1,3),drop=FALSE]; colnames(NN)[1] <- gsub("Complete ","",colnames(NN)[1])
  print(NN,quote=FALSE)
  cat("\n")
  cc <- rbind(x$coef[,-2,drop=FALSE],x$rhoOS)
  print(RoundMat(cc,digits=digits),quote=FALSE)
  cat("\nMZ:\n");
  print(RoundMat(x$probMZ,digits=digits),quote=FALSE)
  cat("DZ:\n")
  print(RoundMat(x$probDZ,digits=digits),quote=FALSE)
##  cat("\nConcordance (MZ; DZ):\t\t", x$concordance,"\n")
##  cat("Case-wise concordance (MZ; DZ):\t", x$conditional,"\n\n")
  cat("\n")
  print(RoundMat(x$heritability[,-2,drop=FALSE],digits=digits),quote=FALSE)
  cat("\n")
  if (!is.null(x$time)) {
      cat("\n")
      cat("Event of interest before time ", x$time, "\n", sep="")
  }

}

ExMarg <- function(Y0,XX0,W0,dS0,midx1=seq(ncol(XX0)/2),midx2=seq(ncol(XX0)/2)+ncol(XX0)/2,eqmarg=TRUE,allmarg=FALSE,Z0=NULL,id=NULL,...) {  
  ii1 <- which(is.na(Y0[,2]) & !is.na(Y0[,1]))
  ii2 <- which(is.na(Y0[,1]) & !is.na(Y0[,2]))
  ii0 <- which(is.na(Y0[,1]) & is.na(Y0[,2]))
  margidx <- c(ii1,ii2)
  id1 <- id2 <-  NULL
  both <- setdiff(seq(nrow(Y0)),c(ii1,ii2,ii0))
  idB <- seq_len(length(both))
  if (allmarg) {
    id1 <- c(seq_len(length(ii1))+length(idB), idB)
    id2 <- c(seq_len(length(ii2))+length(idB)+length(id1), idB)
    ii1 <- c(ii1,both)
    ii2 <- c(ii2,both)
  }
  Y0_marg <- XX0_marg <- X0_marg1 <- X0_marg2 <- dS0_marg <- W0_marg <- NULL
  id0 <- id
  idmarg0 <- NULL
  if (length(margidx)>0) {
      Y0_marg <- cbind(c(Y0[ii1,1],Y0[ii2,2]))
      idmarg0 <- c(id[ii1],id[ii2])     
      X0_marg1 <- XX0[ii1,midx1,drop=FALSE]
      X0_marg2 <- XX0[ii2,midx2,drop=FALSE]
      dS0_marg <- dS0[,1,drop=FALSE]
      if (eqmarg) {
          XX0_marg <- rbind(X0_marg1,X0_marg2)
      } else {
          XX0_marg <- XX0[c(ii1,ii2),,drop=FALSE]
      }    
      if (!is.null(W0)) {
      W0_marg <- cbind(c(W0[ii1,1],W0[ii2,2]))
      W0 <- W0[-c(margidx,ii0),,drop=FALSE]
      }
      id0 <- id[-c(margidx,ii0)]
      Y0 <- Y0[-c(margidx,ii0),,drop=FALSE]
      if (!is.null(Z0))  Z0 <- Z0[-c(margidx,ii0),,drop=FALSE]
      XX0 <- XX0[-c(margidx,ii0),,drop=FALSE]    
  }
  
  res <- list(Y0=Y0,XX0=XX0,W0=W0,
              Y0_marg=Y0_marg, XX0_marg=XX0_marg,
              X0_marg1=X0_marg1, X0_marg2=X0_marg2,
              dS0_marg=dS0_marg, W0_marg=W0_marg,              
              id=idB, idmarg=c(id1,id2),
              ii1=ii1,
              id0=id0,idmarg0=idmarg0, ## Original id's
              margidx=margidx,
              Z0=Z0)
}

if (requireNamespace("tinytest", quietly = TRUE)) {
  options(Ncpus = 2)
  Sys.setenv(OMP_THREAD_LIMIT = 2)
  Sys.setenv(OMP_NUM_THREADS = 2)
  Sys.setenv(OPENBLAS_NUM_THREADS = 2)
  Sys.setenv(ARMA_OPENMP_THREADS = 2)
  RcppArmadillo::armadillo_set_number_of_omp_threads(2)
  tinytest::test_package("mets")
}

##' Estimation of Concordance in Bivariate Competing Risks Data
##'
##' Estimates the bivariate cumulative incidence function (concordance) for paired 
##' data (e.g., twins, family members) in the presence of competing risks. The function 
##' handles both the IPCW (Inverse Probability of Censoring Weighting) estimator and 
##' the Aalen-Johansen estimator (via \code{prodlim}).
##' 
##' The concordance function \eqn{C(t)} is defined as the probability that both members 
##' of a pair experience the event of interest by time \eqn{t}:
##' \deqn{ C(t) = P(T_1 \leq t, T_2 \leq t, \epsilon_1 = k, \epsilon_2 = k) }
##' where \eqn{T_i} is the event time and \eqn{\epsilon_i} is the cause of failure for individual \eqn{i}.
##' 
##' The function supports:
##' \itemize{
##'   \item Stratified analysis (e.g., by zygosity in twin studies).
##'   \item Clustering for robust standard errors.
##'   \item Both IPCW and Aalen-Johansen estimation methods.
##'   \item Resampling for uniform standard errors.
##'   \item IID decomposition for further inference (e.g., casewise concordance tests).
##' }
##'
##' @param formula Formula with an \code{Event} object on the left-hand side. The right-hand 
##'   side specifies the covariate structure, including \code{strata()} for grouping (e.g., MZ/DZ) 
##'   and \code{id()} for pairing.
##' @param data Data frame containing the variables.
##' @param cause Vector of cause codes for which to estimate the bivariate cumulative incidence 
##'   (default \code{c(1,1)}).
##' @param cens Censoring code (default 0).
##' @param causes Vector of all possible causes (optional, inferred from data if missing).
##' @param indiv Variable indicating individual within a pair (optional, inferred from \code{id}).
##' @param strata Variable for stratification (optional, can be specified in formula).
##' @param id Clustering variable (pair ID). Required.
##' @param num Variable for numbering individuals within pairs (optional, auto-generated if missing).
##' @param max.clust Maximum number of clusters to use for IID decomposition in \code{timereg::comp.risk}. 
##'   If NULL, uses all clusters. Useful for large datasets to speed up computation.
##' @param marg Optional marginal cumulative incidence object (from \code{comp.risk}) to compute 
##'   standard errors for same-cluster comparisons in subsequent \code{casewise.test()}.
##' @param se.clusters Vector of cluster indices or column name in \code{data} for standard error calculation. 
##'   Defaults to the \code{id} variable.
##' @param wname Name of an additional weight variable for paired competing risks data.
##' @param prodlim Logical; if TRUE, uses the \code{prodlim} (Aalen-Johansen) estimator instead of 
##'   the IPCW estimator based on \code{comp.risk}. These are equivalent in the absence of covariates.
##' @param messages Control amount of output (0 = silent, 1 = messages).
##' @param model Type of competing risk model for \code{comp.risk} (default "fg" for Fine-Gray).
##' @param return.data If 1, returns the reshaped data; if 2, returns only the data; otherwise returns the model.
##' @param uniform Logical; if TRUE, computes uniform standard errors based on resampling.
##' @param conservative Logical; if TRUE, uses conservative standard errors (recommended for large datasets).
##' @param resample.iid Logical; if TRUE, returns IID residual processes for further computations.
##' @param ... Additional arguments passed to \code{timereg::comp.risk}.
##' @return An object of class \code{"bicomprisk"} (or \code{"bicomprisk.strata"} if stratified) containing:
##'   \item{model}{List of fitted models for each stratum (or a single model).}
##'   \item{strata}{Names of strata (if applicable).}
##'   \item{N}{Number of strata (if applicable).}
##'   \item{time}{Event times.}
##'   \item{P1}{Bivariate cumulative incidence estimates.}
##'   \item{se.P1}{Standard errors of the estimates.}
##'   \item{P1.iid}{IID decomposition (if \code{resample.iid=TRUE}).}
##'   \item{clusters}{Cluster assignments (if \code{marg} or \code{se.clusters} provided).}
##' @author Thomas Scheike, Klaus K. Holst
##' @references 
##' Scheike, T. H.; Holst, K. K. & Hjelmborg, J. B. (2014). Estimating twin concordance for bivariate competing risks twin data. Statistics in Medicine, 33, 1193-1204.
##' @seealso \code{\link{test_casewise}}, \code{\link{casewise}}
##' @examples
##' library("timereg")
##' 
##' ## Simulated data example
##' prt <- sim_nordic_random(2000,delayed=TRUE,ptrunc=0.7,
##'		      cordz=0.5,cormz=2,lam0=0.3)
##' ## Bivariate competing risk, concordance estimates
##' p11 <- bicomprisk(Event(time,cause)~strata(zyg)+id(id),data=prt,cause=c(1,1))
##' 
##' p11mz <- p11$model$"MZ"
##' p11dz <- p11$model$"DZ"
##' par(mfrow=c(1,2))
##' ## Concordance
##' plot(p11mz,ylim=c(0,0.1));
##' plot(p11dz,ylim=c(0,0.1));
##' 
##' ## Entry time, truncation weighting
##' ### Other weighting procedure
##' prtl <-  prt[!prt$truncated,]
##' prt2 <- ipw2(prtl,cluster="id",same.cens=TRUE,
##'      time="time",cause="cause",entrytime="entry",
##'      pairs=TRUE,strata="zyg",obs.only=TRUE)
##' 
##' prt22 <- fast.reshape(prt2,id="id")
##' 
##' prt22$event <- (prt22$cause1==1)*(prt22$cause2==1)*1
##' prt22$timel <- pmax(prt22$time1,prt22$time2)
##' ipwc <- timereg::comp.risk(Event(timel,event)~-1+factor(zyg1),
##'   data=prt22,cause=1,n.sim=0,model="rcif2",times=50:90,
##'   weights=prt22$weights1,cens.weights=rep(1,nrow(prt22)))
##' 
##' p11wmz <- ipwc$cum[,2]
##' p11wdz <- ipwc$cum[,3]
##' lines(ipwc$cum[,1],p11wmz,col=3)
##' lines(ipwc$cum[,1],p11wdz,col=3)
##' @aliases bicompriskData
##' @export
bicomprisk <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
 strata=NULL, id,num, max.clust=1000, marg=NULL,se.clusters=NULL,wname=NULL,
 prodlim=FALSE,messages=TRUE,model,return.data=0,uniform=0,conservative=1,resample.iid=1,...) {
## {{{

  mycall <- match.call()
  formulaId <- unlist(Specials(formula,"id"))
  formulaIndiv <- Specials(formula,"indiv")
  formulaStrata <- unlist(Specials(formula,"strata"))
  formulaSt <- paste("~.-id(",formulaId,")",
                     "-strata(",paste(formulaStrata,collapse="+"),")",
                     "-indiv(",paste(formulaIndiv,collapse="+"),")")
  formula <- update(formula,formulaSt)

  ### ts  11/10
  ## if (substr(as.character(formula)[2],1,4)=="Hist") {
  ##     stop("Since version : The left hand side of the formula must be specified as
  ##     Event(time, event) or with non default censoring codes Event(time, event, cens.code=0).")
  ## }


  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  indiv <- formulaIndiv
  if (!is.null(formulaIndiv)) {
    mycall$indiv <- indiv
  }
  if (missing(id)) stop("Missing 'id' variable")

  ### setting up cluster level for iid decomposition
  if (is.null(se.clusters) & is.null(marg))  lse.clusters <- data[,c(id)]
  else {
    if (is.null(se.clusters)) {
      lse.clusters <- marg$clusters
###      if (!is.null(max.clust)) {   }
    } else {
      if (is.character(se.clusters)) {
        lse.clusters <- data[,se.clusters]
      } else {
        lse.clusters <- se.clusters
      }
    }
  }
  if (length(lse.clusters)!=nrow(data)) stop("'se.clusters' and 'data' does not match!")
  se.clusters.call <- se.clusters

  data <- data.frame(cbind(data,lse.clusters))

  timevar <- terms(formula)[[2]]
  ##  hh <- with(data,eval(timevar))
  if (is.call(timevar)) {
    causes <- timevar[[3]]
    timevar <- timevar[[2]]
  }
  timevar <- as.character(timevar)
  causes <- as.character(causes)

  if (!is.null(strata)) {
    fac <- interaction(data[,strata])
    dd <- split(data,fac)
    nn <- unlist(lapply(dd,nrow))
    dd[which(nn==0)] <- NULL
    if (length(dd)>1) {
      fit <- lapply(seq_len(length(dd)),function(i) {
        if (messages>0) message("Strata '",names(dd)[i],"'")
        idx <- which(fac==names(dd)[i])
        mycall$se.clusters <- lse.clusters[idx]
        mycall$formula <- formula
        mycall$data <- dd[[i]]
        eval(mycall)
      })
      res <- list(model=fit)
      res$strata <- names(res$model) <- names(dd)
      class(res) <- c("bicomprisk.strata","twinlm.strata")
      res$N <- length(dd)
      return(res)
    }
  }

  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  ### adds weights,
  if (!is.null(wname)) covars <- c(covars,wname)

  indiv2 <- covars2 <- NULL

  data <- data[order(data[,id]),]
  idtab <- table(data[,id])
  ##which(data[,id]%in%names(idtab==2))
  data <- data[which(data[,id]%in%names(idtab==2)),]
  if (missing(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,num] <- unlist(lapply(idtab,seq_len))
  }

  oldreshape <- 0
  if (oldreshape==1) sep="." else sep=""
  timevar2 <- paste(timevar,1:2,sep=sep)
  causes2 <- paste(causes,1:2,sep=sep)
  if (length(covars)>0)
    covars2 <- paste(covars,1,sep=sep)
  for (i in seq_len(length(indiv)))
  indiv2 <- c(indiv2, paste(indiv[i],1:2,sep=sep))

  if (oldreshape==1)
  ww0 <- reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],
         direction="wide",idvar=id,timevar=num)[,c(id,"lse.clusters.1",timevar2,causes2,indiv2,covars2)]
  else
  ww0 <- fast.reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],id=id,num=data$num,labelnum=TRUE)[,c(id,"lse.clusters1",timevar2,causes2,indiv2,covars2)]
  ww0 <- na.omit(ww0)

  status <- rep(0,nrow(ww0))
  time <- ww0[,timevar2[1]]

  ## {{{ (i,j) causes
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cause[2])
  if (length(idx2)>0) {
      status[idx2] <- 1
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,max)
  }

  ##(0,0), (0,j)
  idx2 <- which(ww0[,causes2[1]]==cens & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(ic,0), (ic,j)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(i,0)
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cens)
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  ##(ic,jc)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,min)
  }

  ##(0,jc),(i,jc)
  idx2 <- which((ww0[,causes2[1]]==cens | ww0[,causes2[1]]==cause[1]) & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  mydata0 <- mydata <- data.frame(time,status,ww0[,covars2],ww0[,indiv2])
  names(mydata) <- c(timevar,causes,covars,indiv2)
  ## }}}

  if (return.data==2) return(list(data=mydata)) else {
  if (!prodlim) {
    ff <- paste("Event(",timevar,",",causes,",cens.code=",cens,") ~ 1",sep="")
    if (!is.null(wname)) covars <- covars[-which(covars %in% c(wname))]
    if (length(c(covars,indiv))>0) {
     xx <- c(covars,indiv2)
     for (i in seq_len(length(xx)))
	     xx[i] <- paste("const(",xx[i],")",sep="")
      ff <- paste(c(ff,xx),collapse="+")
      if (missing(model)) model <- "fg"
    }
    if (missing(model)) model <- "fg"
    ### clusters for iid construction
    lse.clusters <- NULL
    if (!is.null(se.clusters.call)) {
        lse.clusters <- ww0[,"lse.clusters1"]
    }

###    if (!(is.null(wname))) mydata <- ipw2(mydata,time=timevar,cause=causes) #,cens.code=cens)

    if (is.null(wname)) {
	    add<- timereg::comp.risk(as.formula(ff),data=mydata,
	    cause=1,n.sim=0,resample.iid=resample.iid,model=model,conservative=conservative,
	    clusters=lse.clusters, max.clust=max.clust,...)
    } else {
	    add<-timereg::comp.risk(as.formula(ff),data=mydata,
	    cause=1,n.sim=0,resample.iid=resample.iid,model=model,conservative=conservative,
	    clusters=lse.clusters, max.clust=max.clust,
	    weights=mydata[,wname]*mydata$indi.weights,cens.weights=rep(1,nrow(mydata)),...)
    }

    padd <- predict(add,X=1,se=1,uniform=uniform,resample.iid=resample.iid)
    padd$cluster.names <- lse.clusters
  } else {
      if (!requireNamespace("prodlim",quietly=TRUE)) stop("prodlim requested but not installed")
    ff <- as.formula(paste("prodlim::Hist(",timevar,",",causes,")~",paste(c("1",covars,indiv2),collapse="+")))
    padd <- prodlim::prodlim(ff,data=mydata)
  }
###  class(padd) <- c("bicomprisk",class(padd))
 if (return.data==1) return(list(comp.risk=padd,data=mydata)) else return(padd)
  }
## }}}
}

##' @export
bicompriskData <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
 strata=NULL, id,num, se.clusters=NULL,wname=NULL, ...) {
## {{{

  mycall <- match.call()
  formulaId <- unlist(Specials(formula,"id"))
  formulaIndiv <- Specials(formula,"indiv")
  formulaStrata <- unlist(Specials(formula,"strata"))
  formulaSt <- paste("~.-id(",formulaId,")",
                     "-strata(",paste(formulaStrata,collapse="+"),")",
                     "-indiv(",paste(formulaIndiv,collapse="+"),")")
  formula <- update(formula,formulaSt)

  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  indiv <- formulaIndiv
  if (!is.null(formulaIndiv)) {
    mycall$indiv <- indiv
  }
  if (missing(id)) stop("Missing 'id' variable")

  ### setting up cluster level for iid decomposition
  if (is.null(se.clusters))  lse.clusters <- data[,c(id)]
  if (length(lse.clusters)!=nrow(data)) stop("'se.clusters' and 'data' does not match!")
  se.clusters.call <- se.clusters


  data <- data.frame(cbind(data,lse.clusters))

  timevar <- terms(formula)[[2]]
  ##  hh <- with(data,eval(timevar))
  if (is.call(timevar)) {
    causes <- timevar[[3]]
    timevar <- timevar[[2]]
  }
  timevar <- as.character(timevar)
  causes <- as.character(causes)

  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  covars <- c(covars,strata)
  ### adds weights,
  if (!is.null(wname)) covars <- c(covars,wname,strata)

  indiv2 <- covars2 <- NULL

  data <- data[order(data[,id]),]
  idtab <- table(data[,id])
  ##which(data[,id]%in%names(idtab==2))
  data <- data[which(data[,id]%in%names(idtab==2)),]
  if (missing(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,num] <- unlist(lapply(idtab,seq_len))
  }

  sep <- ""
  timevar2 <- paste(timevar,1:2,sep=sep)
  causes2 <- paste(causes,1:2,sep=sep)
  if (length(covars)>0)
    covars2 <- paste(covars,1,sep=sep)
  for (i in seq_len(length(indiv)))
  indiv2 <- c(indiv2, paste(indiv[i],1:2,sep=sep))

  ww0 <- fast.reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],id=id,num=data$num,labelnum=TRUE)[,c(id,"lse.clusters1",timevar2,causes2,indiv2,covars2)]
  ww0 <- na.omit(ww0)

  status <- rep(0,nrow(ww0))
  time <- ww0[,timevar2[1]]

  ## {{{ (i,j) causes
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cause[2])
  if (length(idx2)>0) {
      status[idx2] <- 1
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,max)
  }

  ##(0,0), (0,j)
  idx2 <- which(ww0[,causes2[1]]==cens & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(ic,0), (ic,j)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(i,0)
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cens)
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  ##(ic,jc)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,min)
  }

  ##(0,jc),(i,jc)
  idx2 <- which((ww0[,causes2[1]]==cens | ww0[,causes2[1]]==cause[1]) & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[2]]
  }


  mydata0 <- mydata <- data.frame(time,status,ww0[,covars2],ww0[,indiv2],ww0[,id])

  names(mydata) <- c(timevar,causes,covars,indiv2,id)
  ## }}}

  return(mydata)
} ## }}}


## plot.bicomprisk <- function(x,add=FALSE,...) {
##   if ("predict.timereg"%in%class(a)) {
##     if (!add) { plot.predict.timereg(x,...) }
##     else {
##       with(x,lines(time,P1,...))
##     }
##   } else {
##     plot(x,...)
##   }
##   return(invisible(x))
##}

##' event_split (SurvSplit).
##' 
##' Constructs start stop formulation of event time data after a variable in
##' the data.set. Similar to SurvSplit of the survival package but can also
##' split after random time given in data frame.
##' 
##' 
##' @param data data to be split
##' @param time time variable.
##' @param status status variable.
##' @param cuts cuts variable or numeric cut (only one value)
##' @param name.id name of id variable.
##' @param name.start name of start variable in data, start can also be numeric "0"
##' @param cens.code code for the censoring.
##' @param order.id order data after id and start.
##' @param time.group make variable "before"."cut" that keeps track of wether start,stop is before (1) or after cut (0).
##' @author Thomas Scheike
##' @keywords survival
##' @examples
##' 
##' set.seed(1)
##' d <- data.frame(event=round(5*runif(5),2),start=1:5,time=2*1:5,
##' 		status=rbinom(5,1,0.5),x=1:5)
##' d
##' 
##' d0 <- event_split(d,cuts="event",name.start=0)
##' d0
##' 
##' dd <- event_split(d,cuts="event")
##' dd
##' ddd <- event_split(dd,cuts=3.5)
##' ddd
##' event_split(ddd,cuts=5.5)
##' 
##' ### successive cutting for many values 
##' dd <- d
##' for  (cuts in seq(2,3,by=0.3)) dd <- event_split(dd,cuts=cuts)
##' dd
##' 
##' ###########################################################################
##' ### same but for situation with multiple events along the time-axis
##' ###########################################################################
##' d <- data.frame(event1=1:5+runif(5)*0.5,start=1:5,time=2*1:5,
##' 		status=rbinom(5,1,0.5),x=1:5,start0=0)
##' d$event2 <- d$event1+0.2
##' d$event2[4:5] <- NA 
##' d
##' 
##' d0 <- event_split(d,cuts="event1",name.start="start",time="time",status="status")
##' d0
##' ###
##' d00 <- event_split(d0,cuts="event2",name.start="start",time="time",status="status")
##' d00
##' 
##' @export
event_split <- function(data,
		time="time",status="status",cuts="cuts",name.id="id",
		name.start="start", cens.code=0,order.id=TRUE, time.group=FALSE)
{ ## {{{
    n <- nrow(data)
    new.time <- data[,time]
    new.status <- data[,status]

    if (is.numeric(cuts)) {
        cutname <- paste("cut",cuts,sep=".")
        data[,cutname] <- cuts
    } else cutname <- cuts
    new.cuts <- data[,cutname]

    if (is.numeric(name.start)) {
	    start0 <- name.start
	    name.start <- paste("start",name.start,sep=".")
            data[,name.start] <- start0
    }  


    if ((name.start %in% names(data))) {
      new.start <- data[,name.start]
    } else new.start <- rep(0,n)

    if (any(new.start>= new.time)) cat("any(new.start>= new.time) is TRUE\n")

    if ((name.id %in% names(data))) idl <- data[,name.id] else {
	    idl <- 1:n
	    data[,name.id] <- idl 
    }

    ## only split if cut not already among times
    splits <- which(new.cuts<new.time & new.start<new.cuts)

    if (length(splits)) {
	    rows  <- c(1:n,splits)
	    new.time <-   c(new.time,new.time[splits])
	    new.start <-  c(new.start,new.cuts[splits])
	    new.status <- c(new.status,new.status[splits])
	    new.ccc <-    c(new.cuts,new.cuts[splits])
	    idl <- c(idl,idl[splits])
	    new.time[splits] <- new.cuts[splits]
	    new.status[splits] <- cens.code
	    data <- data[rows,]
	    data[,time] <- new.time
	    data[,status] <- new.status
	    data[,name.start] <- new.start
	    data[,name.id] <- idl
    }

    if (time.group) {
      group.time <- paste("before",cutname,sep=".")
      data[,group.time] <- 1*(data[,name.start]<data[,cutname]) ## sc(rep(1,n),rep(0,length(splits)))
    } 

    if (order.id) data <- data[order(idl,new.start),] 
    rownames(data) <- NULL

    return(data)
}  ## }}}

##' Event split with two time-scales, time and gaptime 
##'
##' Cuts time for two time-scales, as event.split 
##'
##' @param data data to be split
##' @param time time variable.
##' @param status status variable.
##' @param entry name of entry variable.
##' @param cuts cuts variable or numeric cut (only one value)
##' @param name.id name of id variable.
##' @param gaptime gaptime variable.
##' @param gaptime.entry name of entry variable for gaptime.
##' @param cuttime to cut after time or gaptime
##' @param cens.code code for the censoring.
##' @param order.id order data after id and start.
##' @author Thomas Scheike
##' @keywords survival
##' @examples
##' rr  <- data.frame(time=c(500,1000),start=c(0,500),status=c(1,1),id=c(1,1))
##' rr$gaptime <-  rr$time-rr$start
##' rr$gapstart <- 0
##'
##' rr1 <- event_split2(rr,cuts=600,cuttime="time",   gaptime="gaptime",gaptime.entry="gapstart")
##' rr2 <- event_split2(rr1,cuts=100,cuttime="gaptime",gaptime="gaptime",gaptime.entry="gapstart")
##'
##' dlist(rr1,start-time+status+gapstart+gaptime~id)
##' dlist(rr2,start-time+status+gapstart+gaptime~id)
##'
##' @export 
event_split2 <- function(data,
		time="time",status="status",entry="start",cuts="cuts",name.id="id",
		gaptime=NULL,gaptime.entry=NULL,cuttime=c("time","gaptime"),
		cens.code=0,order.id=TRUE)
{
## {{{ 
    n <- nrow(data)
    new.time <- data[,time]
    new.status <- data[,status]

    if (!is.null(gaptime)) { new.gaptime <- data[,gaptime]; nngap <- new.gaptime;}
    if (!is.null(gaptime.entry)) new.gapstart <- data[,gaptime.entry]

    if (is.numeric(cuts)) {
	    new.cuts <- rep(cuts,nrow(data))
    } else { new.cuts  <-  data[,cuts] }

    if (is.numeric(entry)) {
	    start0 <- entry
	    name.entry <- paste("start",entry,sep=".")
            data[,name.entry] <- start0
    }  else name.entry <- entry

    if ((name.entry %in% names(data))) {
      new.start <- data[,name.entry]
    } else new.start <- rep(0,n)
    nnstart <- new.start

    if (any(new.start>= new.time)) cat("any(new.start>= new.time) is TRUE\n")

    if ((name.id %in% names(data))) idl <- data[,name.id] else {
	    idl <- 1:n
	    data[,name.id] <- idl 
    }

    if (cuttime[1]=="time") {# {{{
    splits <- which(new.cuts<new.time & new.start<new.cuts)

    if (length(splits)) {
	    rows  <- c(1:n,splits)
	    new.time <-   c(new.time,new.time[splits])
	    new.start <-  c(new.start,new.cuts[splits])
	    new.time[splits] <- new.cuts[splits]
	    new.status <- c(new.status,new.status[splits])
	    new.status[splits] <- cens.code
	    idl <- c(idl,idl[splits]) 

	    if (!is.null(gaptime)) {
	       new.gapstart <- c(new.gapstart,new.cuts[splits]-nnstart[splits]+new.gapstart[splits])
               new.gaptime <-   c(new.gaptime,new.gaptime[splits])
	       new.gaptime[splits] <- new.cuts[splits]- nnstart[splits]+new.gapstart[splits]
            }

	    data <- data[rows,]
	    data[,time] <- new.time
	    data[,status] <- new.status
	    data[,name.entry] <- new.start
	    data[,name.id] <- idl

	    if (!is.null(gaptime)) data[,gaptime]  <-  new.gaptime 
	    if (!is.null(gaptime)) data[,gaptime.entry] <- new.gapstart
    }

    if (order.id) data <- data[order(idl,new.start),] # }}}
    } else {# {{{
    splits <- which(new.cuts<new.gaptime & new.gapstart<new.cuts)

    if (length(splits)) {
	rows  <- c(1:n,splits)
	new.gaptime <-   c(new.gaptime,new.gaptime[splits])
        new.gapstart <-  c(new.gapstart,new.cuts[splits])
        new.gaptime[splits] <- new.cuts[splits]
        new.status <- c(new.status,new.status[splits])
        new.status[splits] <- cens.code
        idl <- c(idl,idl[splits]) 

        new.time  <- c(new.time,new.time[splits])
        new.time[splits] <- new.start[splits]+(new.gaptime[splits]-new.gapstart[splits])
        new.start  <-   c(new.start,new.start[splits]+(new.gaptime[splits]-new.gapstart[splits]))

        data <- data[rows,]
        data[,time] <- new.time
        data[,status] <- new.status
        data[,name.entry] <- new.start
        data[,name.id] <- idl
        ###
        data[,gaptime]  <-  new.gaptime 
        data[,gaptime.entry] <- new.gapstart
        if (order.id) data <- data[order(idl,new.start),] 
       }

    }# }}}

    return(data)
    ## }}} 
} 

#' Event history object
#'
#' Constructur for Event History objects
#'
#' ... content for details
#'
#' @aliases Event as.character.Event as.matrix.Event
#' @aliases [.Event format.Event
#' @aliases print.Event rbind.Event summary.Event
#' @param time Time
#' @param time2 Time 2
#' @param cause Cause
#' @param cens.code Censoring code (default 0)
#' @param ... Additional arguments
#' @return Object of class Event (a matrix)
#' @author Klaus K. Holst and Thomas Scheike
#' @examples
#'
#' 	t1 <- 1:10
#' 	t2 <- t1+runif(10)
#' 	ca <- rbinom(10,2,0.4)
#' 	(x <- Event(t1,t2,ca))
#' @export
Event <- function(time,time2=TRUE,cause=NULL,cens.code=0,...) {# {{{
  if (missing(cause)) {
    if (is.factor(time2) | is.character(time2)) {
      stop("cause should be numeric\n")
    }
  }
  else { if (is.factor(cause) | is.character(cause))
           stop("cause should be numeric\n");  }
  out <- cbind(time, time2, cause)
  if (any(is.na(out))) warning("missing values in Event object\n")
  if (!missing(cause)) {
    colnames(out) <- c("entry", "exit", "cause")
    tmp <- (out[,1]>out[,2])
    if (any(tmp)) warning("entry time later than exit time\n")
  } else {
    colnames(out) <- c("exit", "cause")
  }
  class(out) <- "Event"
  attr(out, "cens.code") <- cens.code
  return(out)
}
# }}}

# exported in zzz.R
#' @exportS3Method NULL
as.matrix.Event <- function(x, ...) structure(x, class="matrix")

# exported in zzz.R
#' @exportS3Method NULL
as.data.frame.Event <- function(x, ...) {
  as.data.frame.model.matrix(x, ...)
}

# exported in zzz.R
#' @exportS3Method NULL
as.character.Event <- function(x, ...) {
  if (ncol(x) == 3) {
    res <- paste(
        "(", format(x[, 1], ...), ";",
        format(x[, 2], ...), ":",
        format(x[, 3], ...), "]",
        sep = ""
    )
  } else {
    res <- paste(format(x[, 1], ...), ":", format(x[, 2], ...), sep = "")
  }
  return(res)
}

# exported in zzz.R
#' @exportS3Method NULL
format.Event <- function(x, ...) format(as.character.Event(x), ...)

# exported in zzz.R
#' @exportS3Method NULL
is.na.Event <- function(x) {
    apply(x, 1, function(r) any(is.na(r)))
}

# exported in zzz.R
#' @exportS3Method NULL
print.Event <- function(x, ...) {
    print(as.character(x), ..., quote=FALSE)
}

# exported in zzz.R
#' @exportS3Method NULL
summary.Event <- function(object,...) {
  cat(paste("cens.code=",attr(object,"cens.code"),"\n"))
  cat("causes:\n")
  print(table(object[,"cause"]))
  cat("exit:\n")
  print(summary(object[,"exit"]))
  if (ncol(object)==3) {
    cat("entry:\n")
    print(summary(object[,"entry"]))
    cat("exit-entry:\n")
    print(summary(object[,"exit"]- object[,"entry"]))
  }
}

# exported in zzz.R
#' @exportS3Method NULL
"[.Event" <- function (x, i, j, drop = FALSE) {
  if (missing(j)) {
    atr <- attributes(x)
    class(x) <- "matrix"
    x <- x[i, , drop = FALSE]
    class(x) <- "Event"
    atr.keep <- c("cens.code", "entry")
    attributes(x)[atr.keep] <- atr[atr.keep]
    x
  }
  else {
    class(x) <- "matrix"
    NextMethod("[")
  }
}

# exported in zzz.R
#' @exportS3Method NULL
rbind.Event <- function(...) {
    dots <- list(...)
    cens.code <- attributes(dots[[1]])$cens.code
    type <- attributes(dots[[1]])$type
    ncol <- dim(dots[[1]])[2]
    nrow <- unlist(lapply(dots, nrow))
    cnrow <- c(0, cumsum(nrow))
    M <- matrix(ncol = ncol, nrow = sum(nrow))
    for (i in seq_len(length(dots))) {
        M[(cnrow[i] + 1):cnrow[i + 1], ] <- dots[[i]]
    }
    x <- c()
    for (i in seq_len(ncol(M))) x <- c(x, list(M[, i]))
    x <- c(x, list(cens.code = cens.code))
    do.call("Event", x)
}

# exported in zzz.R
#' @exportS3Method NULL
length.Event <- function(x) nrow(x)

# exported in zzz.R
#' @exportS3Method NULL
c.Event <- function(...) {
    objects <- list(...)
    if (!all(unlist(lapply(objects, function(x) inherits(x, "Event")))) &&
        all.equal(unlist(lapply(objects, NCOL)))) {
        stop("All elements should be `Event` of the same type")
    }
    Reduce(rbind, objects)
}

## Event function

library("tinytest")

test_event <- function() {

  dat <- data.frame(
    s = mets::Event(runif(10), rbinom(10, 1, 0.5)),
    y = runif(10),
    z = letters[1:10]
  )

  # Event, length.Event
  expect_true(nrow(dat) == 10)
  # c.Evente
  expect_true(length(c(dat$s, dat$s)) == 20)
  # as.matrix.Event
  expect_true(all(dim(as.matrix(dat$s)) == c(10, 2)))

}
test_event()

##' Percentage of Years Lost Due to a Cause Regression
##'
##' Estimates the percentage of the restricted mean time lost (RMTL) that is attributable 
##' to a specific cause and models how covariates affect this percentage using IPCW regression.
##' 
##' Let the total years lost be \eqn{Y = t - \min(T, t)} and the years lost due to cause 1 be 
##' \eqn{Y_1 = I(\epsilon=1) (t - \min(T, t))}. The function models the ratio:
##' \deqn{ \text{logit}\left( \frac{E(Y_1 | X)}{E(Y | X)} \right) = X^T \beta }
##' 
##' Estimation is based on a binomial regression IPCW response estimating equation:
##' \deqn{ X \left( \Delta^{\text{ipcw}}(t) \left( Y \cdot \text{expit}(X^T \beta) - Y_1 \right) \right) = 0 }
##' where \eqn{\Delta^{\text{ipcw}}(t) = I(\min(t,T) < C) / G_c(\min(t,T))} is the IPCW adjustment.
##'
##' The function supports three types of estimators:
##' \itemize{
##'   \item \code{"I"}: Classical outcome IPCW regression (no augmentation).
##'   \item \code{"II"}: Adds a censoring augmentation term \eqn{X \int E(Z(t)| T>s)/G_c(s) d \hat M_c} 
##'     to improve efficiency (requires an initial estimate of \eqn{\beta}).
##'   \item \code{"III"}: Adds a more complex augmentation term separating the expectations of 
##'     \eqn{Y} and \eqn{Y_1} for further efficiency gains.
##' }
##'
##' The variance is based on the squared influence functions (IID). A "naive" variance 
##' (assuming known censoring) is also provided for comparison.
##'
##' @param formula Formula with an outcome (see \code{coxph}). The first covariate on the RHS 
##'   is typically the treatment or group indicator. Can include \code{cluster(id)}.
##' @param data Data frame containing the variables.
##' @param cause Numeric code of the cause of interest.
##' @param time Time point \eqn{t} for the analysis. Required.
##' @param beta Starting values for optimization (default NULL, uses zeros).
##' @param type Type of estimator: \code{"I"} (IPCW only), \code{"II"} (IPCW + augmentation), 
##'   or \code{"III"} (IPCW + complex augmentation). Default is \code{"III"}.
##' @param offset Offsets for the partial likelihood.
##' @param weights Weights for the score equations.
##' @param cens.weights External censoring weights (if provided, \code{cens.model} is ignored).
##' @param cens.model Formula for the censoring model (default \code{~+1}, stratified KM). 
##'   Can include \code{strata()} for stratified censoring.
##' @param se Logical; if TRUE, computes standard errors based on IPCW (default TRUE).
##' @param relative.to.causes If not NULL, compares the RMTL of the specified \code{cause} 
##'   to the RMTL of the causes in this vector (the denominator becomes the sum of these causes).
##' @param kaplan.meier Logical; if TRUE, uses Kaplan-Meier for IPCW weights; if FALSE, 
##'   uses \eqn{\exp(-\text{cumulative hazard})}.
##' @param cens.code Censoring code (default 0).
##' @param no.opt Logical; if TRUE, skips optimization and uses \code{beta} directly.
##' @param method Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.
##' @param augmentation Initial augmentation term (used for type "II" and "III").
##' @param outcome Outcome type: \code{"rmtl"} (years lost) or \code{"cif"} (cumulative incidence).
##' @param model Link function: \code{"logit"} (default), \code{"exp"}, or \code{"lin"}.
##' @param Ydirect Matrix with two columns (numerator, denominator) to use directly as the response.
##' @param ... Additional arguments passed to lower-level functions.
##' @return An object of class \code{"binreg"} and \code{"ratio"} containing:
##'   \item{coef}{Coefficient estimates.}
##'   \item{se.coef}{Standard errors.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{iid}{Influence function decomposition (with censoring adjustment).}
##'   \item{iidI}{Influence function without censoring adjustment.}
##'   \item{naive.var}{Variance assuming known censoring.}
##'   \item{time}{Time point used.}
##'   \item{cause}{Cause of interest.}
##'   \item{Causes}{Set of causes considered in the denominator.}
##'   \item{Yipcw}{IPCW-adjusted response matrix.}
##'   \item{coefI, varI}{Results from the initial (type "I") fit.}
##'   \item{augmentation}{Final augmentation term used.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. & Tanaka, S. (2025). Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause. WIP.
##' @seealso \code{\link{resmeanIPCW}}, \code{\link{binreg}}
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' 
##' rmtl30 <- rmstIPCW(Event(time,cause!=0)~platelet+tcell+age, bmt, time=30, cause=1, outcome="rmtl")
##' rmtl301 <- rmstIPCW(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1)
##' rmtl302 <- rmstIPCW(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=2)
##' 
##' estimate(rmtl30)
##' estimate(rmtl301)
##' estimate(rmtl302)
##' 
##' ## Percentage of total RMTL due to cause 1
##' rmtlratioI <- rmtlRatio(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1)
##' summary(rmtlratioI)
##' 
##' newdata <- data.frame(platelet=1, tcell=1, age=1)
##' pp <- predict(rmtlratioI, newdata)
##' pp
##' 
##' ## Percentage of total cumulative incidence due to cause 1
##' cifratio <- binregRatio(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1, model="cif")
##' summary(cifratio)
##' pp <- predict(cifratio, newdata)
##' pp
##' @aliases rmtlRatio
##' @export
binregRatio <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("III","II","I"),
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,relative.to.causes=NULL,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("rmtl","cif"),model=c("logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula, data = data, specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

 call.id <- id;
 conid <- construct_id(id,nrow(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- 1*(status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 
  if (!is.null(relative.to.causes)) Causes <- relative.to.causes
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

 nevent <- sum((status %in% cause)*(exit<=time))
 ## if event before time or alive, then uncensored, equality for both censored and events  
 obs <- (exit<=time & (statusC==0)) | (exit>=time)

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else { se <- FALSE; resC <- formC <- NULL}
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  if (is.null(augmentation))  augmentation=rep(0,p)
  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX

 if (!is.null(Ydirect)) Y <-  Ydirect*obs/cens.weights else {
     if (outcome[1]=="cif") Y <- cbind( c((status %in% Causes)*(exit<=time)/cens.weights) , c((status %in% cause)*(exit<=time)/cens.weights) )
     else { 
            Y <- cbind(c((status %in% Causes)*(time-pmin(exit,time))*obs)/cens.weights, c((status %in% cause)*(time-pmin(exit,time))*obs)/cens.weights)
     }
  }
  Yipcw <- Y
  cens.weights.origsort <- obs/cens.weights 

obj <- function(pp,all=FALSE)
{ # {{{
lp <- c(X %*% pp+offset)

if (model[1]=="logit") {
	p <- expit(lp)
        D2logl <- c(Y[,1]*weights*p/(1+exp(lp)))*X2
} else if (model[1]=="exp") {
	p <- exp(lp)
        D2logl <- c(Y[,1]*p*weights)*X2
} else {
	p <- lp
        D2logl <- c(Y[,1]*weights)*X2
}
Dlogl <- weights*X*c(Y[,1]*p-Y[,2])
D2log <- apply(D2logl,2,sum)
ploglik <- 0   ###sum(weights*(Y[,1]*p-Y[,2])^2)

gradient <- apply(Dlogl,2,sum)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ploglik <- sum(weights*(Y[,1]*p-Y[,2])^2)
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(0,gradient=-gradient/nid,hessian=-hessian/nid)
}# }}}

 if (model[1]=="exp") control <- list(stepsize=0.5)  else control <- NULL
  

  ## first run without pseudo-value augmentation
###  p <- ncol(X)
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=beta),obj(beta,all=TRUE))

 coefI <- val$coef
 gradientI <- val$gradient
 hessianI <- val$hessian
 ihessianI <- val$ihessian
 iidI <- val$iid

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    X2 <-  X2[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    Y <- Y[ord,]
    lp <- c(X %*% val$coef+offset)
    if (model[1]=="logit")  p <- expit(lp) else if (model[1]=="exp") p <- exp(lp)  else p <- lp
    Yo <- Y[,1]*p-Y[,2]
    id <- id[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yo,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    btime <- 1*(exit<time)
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)
    MGCiidI <- MGCiid <- apply(MGt,2,sumstrata,xx$id,mid+1)

   if (type[1]=="II") { ##  pseudo-value type augmentation
    hYt  <-  revcumsumstrata(Yo,xx$strata,xx$nstrata)
    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
    U <- rep(0,length(xx$strata))
    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
    MGt <- X*c(U-IhdLam0)*c(xx$weights)
    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
    augmentation  <-  apply(MGtiid,2,sum) + augmentation
    ###
    EXt  <-  apply(X,2,revcumsumstrata,xx$strata,xx$nstrata)
    IEXhYtdLam0 <- apply(EXt*c(hYt)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
    ###
    MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
    ### Censoring Variance Adjustment 
    MGCiid <- MGCiid+(MGtiid-MGCiid2)
   }
   if (type[1]=="III") { ##  pseudo-value type augmentation
    hYt  <-  apply(Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    IhdLam0 <- apply(hYt*c(S0i2)*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- rep(0,length(xx$strata))
    U[xx$jumps+1] <- (resC$jumptimes<time)*(hYt[xx$jumps+1,1]*p[xx$jumps+1]-hYt[xx$jumps+1,2])/c(resC$S0)
    MGt <- X*c(U-IhdLam0[,1]*p+IhdLam0[,2])*c(xx$weights)

    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
    augmentation  <-  apply(MGtiid,2,sum) + augmentation
    ###
    EXt  <-  apply(X,2,revcumsumstrata,xx$strata,xx$nstrata)
    EXFt  <-  apply(X*p,2,revcumsumstrata,xx$strata,xx$nstrata)

    IEXhY1tdLam0 <- apply(EXt*hYt[,2]*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    IEXhYtdLam0 <- apply(EXFt*hYt[,1]*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)

    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*(hYt[xx$jumps+1,1]*EXFt[xx$jumps+1,]-hYt[xx$jumps+1,2]*EXt[xx$jumps+1,]) /c(resC$S0)^2
    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0+IEXhY1tdLam0)*c(xx$weights)
    ###
    MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
    ### Censoring Variance Adjustment 
    MGCiid <- MGCiid+(MGtiid-MGCiid2)
   }
   ## use data ordered by time (keeping track of id also)
   ## since X; Y and so forth are ordered in time
   ###   id <- xx$id
   }  else {
	 MGCiidI <-  MGCiid <- 0
  }## }}}

  ## first run without pseudo-value augmentation, then run with augmentation, if type="II"
 if (type[1]!="I") {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(coefI,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,coefI,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
###	      if (!se) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
 }

 if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid))


  val$call <- cl
  val$MGciid <- MGCiid
  val$id <- orig.id
  val$call.id <- call.id
  val$name.id <- name.id
  val$nid <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL 
  val$iidI <- iidI
  if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian)
  if (se) val$iidI  <- iidI+(MGCiidI %*% ihessianI)
  if (!is.null(call.id)) {
  val$iid <- nameme(val$iid,name.id)
  val$iidI <- nameme(val$iidI,name.id)
  }
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$augmentation <- augmentation
  val$model <- model[1]
  val$outcome <- outcome[1]
  val$Yipcw <- Yipcw
  val$cens.weights <- cens.weights.origsort 
  val$Causes <- Causes
  val$nevent <- nevent

  val$coefI <- coefI
  val$varI <- crossprod(val$iidI)
  val$se.coefI <- diag(val$varI)^.5
  val$gradientI <-  gradientI
  val$hessianI <- hessianI
  val$ihessianI <- ihessianI
  val$design <- des

  class(val) <- c("binreg","ratio")
  return(val)
}# }}}

##' @export
rmtlRatio <- function(formula,data,outcome=c("rmtl"),...)
{# {{{
   out <- binregRatio(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

data(prt)
dim(prt)
table(prt$status)
### 21000 7000 1000

library(mets)
###
set.seed(100)
prt<-simnordic(7500,cordz=3,cormz=4,cratemz=1.0,cratedz=1.0)
prt$status <-prt$cause
table(prt$status)

prt<-simnordic(7500,cordz=2.0,cormz=3.,pcensmz=0.0,pcensdz=0.0,cratemz=200.4,cratedz=100.4)
prt$status <-prt$cause
prop.table(table(prt$status))
prt$cancer <- (prt$status==1)

prt<-simnordic(7500,cordz=1,cormz=3,pcensmz=0.9,pcensdz=0.9,cratemz=0.4,cratedz=0.4)
prt$status <-prt$cause
prop.table(table(prt$status))


prt<-simnordic(75000,cordz=2.0,cormz=3.,pcensmz=0.0,pcensdz=0.0,cratemz=200.4,cratedz=100.4)
prt$status <-prt$cause
table(prt$status)
prt$cancer <- (prt$status==1)
###
bp3 <- bptwin(cancer~1,zyg="zyg",DZ="DZ",id="id",type="ace",data=prt)
summary(bp3)

coef(bp3)
exp(-0.1)/( exp(-0.1)+exp(-0.77)+1)
exp(-0.77)/( exp(-0.1)+exp(-0.77)+1)

gem <- c()
for (pcens in seq(0,0.95,length=5))
{
prt<-simnordic(7500,cordz=3,cormz=4,pcensmz=pcens,pcensdz=pcens,cratemz=0.4,cratedz=0.4)
prt$status <-prt$cause
tt <- table(prt$status)
if (length(tt)==2) tt <- c(0,tt)
prt$cancer <- (prt$status==1)
###
bp3 <- bptwin(cancer~1,zyg="zyg",DZ="DZ",id="id",type="ace",data=prt)
summary(bp3)
###
gem <- rbind(gem,c(pcens,tt,coef(bp3)))
}
gem


gemmzdz <- c()
concmz <- concdz <- marg <- cens <- gemmzdzc <- matrix(0,5,5)
cens <- gemmzdzc <- matrix(0,5,5)
gemmzdza <- matrix(0,5,5)
j <- i <- 0 
for (pcensmz in seq(0,0.95,length=5))
{
i <- i+1
j <- 0
for (pcensdz in seq(0,0.95,length=5))
{
j <- j+1
prt<-simnordic(100000,cordz=1.5,cormz=3,pcensmz=pcensmz,pcensdz=pcensdz,cratemz=0.9,cratedz=0.9)
prt$status <-prt$cause
tt <- table(prt$status)
if (length(tt)==2) tt <- c(0,tt)
prt$cancer <- (prt$status==1)
###
bp3 <- bptwin(cancer~1,zyg="zyg",DZ="DZ",id="id",type="ace",data=prt)
ud <- summary(bp3)
###
gemmzdz <- rbind(gemmzdz,c(pcensmz,pcensdz,tt,coef(bp3)))
print(c(i,j))
gemmzdza[i,j] <- coef(bp3)[2]
gemmzdzc[i,j] <- coef(bp3)[3]
marg[i,j] <- ud$probMZ[3,1]
concmz[i,j] <- ud$probMZ[1,1]
concdz[i,j] <- ud$probDZ[1,1]
cens[i,j] <- tt[1]/sum(tt)
}
}
###
gemmzdz
gemmzdza
gemmzdzc
###
h <- exp(gemmzdza)/(exp(gemmzdza)+exp(gemmzdzc)+1)
c <- exp(gemmzdzc)/(exp(gemmzdza)+exp(gemmzdzc)+1)
###
round(h,2)
round(c,2)
round(cens,2)
###
###h2.3 <- h 
###c2.3 <- c

h15.3 <- h 
c15.3 <- c

round(h1.3,2)
round(c1.3,2)

round(h2.3,2)
round(c2.3,2)

save(h1.3,file="h13.rda"); save(c1.3,file="c13.rda")
save(h2.3,file="h23.rda"); save(c2.3,file="c23.rda")
save(concmz,file="concMZ23.rda"); save(marg,file="marg23.rda")

table(prt$zyg,prt$status)

library(latextable)
cbind(h1.3,c1.3)

########################################################################################
#####################   MC
########################################################################################
library(doMC)
library(mets)
registerDoMC()

onerun <- function(k,cordz=1,cormz=3)
{#{{{
print(k)
pcensmz  <- seq(0,0.95,length=5)
j <- (k%%5)
j[j==0] <- 5
i <-ceiling(k/5)
print(c(i,j))
prt<-simnordic(300000,cordz=cordz,cormz=cormz,pcensmz=pcensmz[i],pcensdz=pcensmz[j],cratemz=0.3,cratedz=0.3)
prt$status <-prt$cause
cmzdz <- table(prt$status,prt$zyg)
if (nrow(cmzdz)==3) cmzdz <- rbind(c(0,0),cmzdz)
tt <- table(prt$status)
###print(tt)
if (length(tt)==2) tt <- c(0,tt)
prt$cancer <- (prt$status==1)
###
bp3 <- bptwin(cancer~1,zyg="zyg",DZ="DZ",id="id",type="ace",data=prt)
ud <- summary(bp3)
###
return(list(index=c(i,j),cmzdz=cmzdz,status=tt,pcensmz=pcensmz[i],pcensdz=pcensmz[j],bp3=bp3))
}#}}}
ud <- onerun(25)
prop.table(ud$status)

res <- c()
res <- foreach (i=1:25) %dopar% onerun(i,cordz=1,cormz=3)
###res23 <- res
res13 <- res
###res153 <- res

gemmzdz <- c()
concmz <- concdz <- marg <- cens <- gemmzdzc <- matrix(0,5,5)
cens <- gemmzdzc <- matrix(0,5,5)
gemmzdza <- matrix(0,5,5)
for (k in 1:length(res)) {
j <- (k%%5)
j[j==0] <- 5
i <-ceiling(k/5)
print(c(i,j))
print(res[[k]]$index)
print(c(res[[k]]$pcensmz,res[[k]]$pcensdz)); 
bp3 <- res[[k]]$bp3
ud <- summary(bp3)
gemmzdza[i,j] <- coef(bp3)[2]
gemmzdzc[i,j] <- coef(bp3)[3]
marg[i,j] <- ud$probMZ[3,1]
concmz[i,j] <- ud$probMZ[1,1]
concdz[i,j] <- ud$probDZ[1,1]
tt <- res[[k]]$status
cens[i,j] <- tt[1]/sum(tt)
}
cens
h <- exp(gemmzdza)/(exp(gemmzdza)+exp(gemmzdzc)+1)
c <- exp(gemmzdzc)/(exp(gemmzdza)+exp(gemmzdzc)+1)
###
round(h,2)
round(c,2)
round(cens,2)
#

###h153 <- h
###c153 <- c
###cens153 <- cens
###marg153 <- marg
###conc153 <- concmz 

###h13 <- h
###c13 <- c
###cens13 <- cens

h23 <- h
c23 <- c
cens23 <- cens
conc23 <- concmz
marg23 <- marg

round(h13,2)
round(h23,2)

save(h13,file="h13.rda")
save(h23,file="h23.rda")
###
save(c13,file="c13.rda")
save(c23,file="c23.rda")
###
save(marg23,file="marg23.rda")
save(conc23,file="conc23.rda")

##' @export
plotcr <- function(x,col,lty,legend=TRUE,which=1:2,cause=1:2,
                   ask = prod(par("mfcol")) < length(which) && dev.interactive(), ...) {
    if (inherits(try(find.package("prodlim"),silent=TRUE),"try-error")) {
        stop("Needs prodlim")
    }

  dots <- list(...)
  if (is.null(dots$xlab)) dots$xlab <- "Time"
  if ((!is.data.frame(x) | !is.matrix(x)) && ncol(x)<2) stop("Wrong type of data")  
  if (ncol(x)==2) {
    if (is.null(dots$curvlab)) {
      causes <- setdiff(unique(x[,2]),0)
      dots$curvlab <- seq(length(causes))
    }
    colnames(x)[1:2] <- c("t","status")
    co <- prodlim::prodlim(Hist(t,status)~1,data=data.frame(x))    
    ##    co <- cuminc(x[,1],x[,2])
    ##    if (any(x[,1]<0)) {
    ##      do.call(co,p)
    ## for (i in seq(length(co))) {
    ##   co[[i]]$time <- co[[i]]$time[-1]
    ##   co[[i]]$est <- co[[i]]$est[-1]
    ## }
    ##if (is.null(dots$xlim)) dots$xlim <- range(x[,1])
    ##    }
    ##do.call("plot", c(list(x=co), dots))
    if (is.null(dots$lwd)) dots$lwd <- 1
    if (missing(lty)) lty <- seq_len(length(co$cuminc))
    if (missing(col)) col <- rep(1,length(co$cuminc))
    if (is.null(dots$lwd)) dots$lwd <- 1
    dots$lty <- lty[1]
    dots$col <- col[1]
    do.call("plot", c(list(x=co), dots))
    dots$add <- TRUE
    for (i in seq_len(length(co$cuminc)-1)+1) {
      dots$lty <- lty[i]
      dots$col <- col[i]
      dots$cause <- i
      do.call("plot", c(list(x=co), dots))
    }
    if (legend)
      legend("topleft",names(co$cuminc),col=col,lty=lty,pch=-1)
    return(invisible(co))
  }

  if (ask) {
    oask <- devAskNewPage(TRUE)
    on.exit(devAskNewPage(oask))
  }

  t <- as.matrix(x[,1:2]); cause0 <- as.matrix(x[,3:4])
  causes <- sort(setdiff(unique(cause0),0))
  if (is.null(dots$curvlab))
    dots$curvlab <- seq(1:length(unique(causes)))
  if (missing(col)) col <- c("seagreen","darkred","darkblue","goldenrod","mediumpurple")
  if (1%in%which) {
    plot(t,type="n",...)
    count <- 1
    for (i in causes) {
      points(t[cause0[,1]==causes[i],],col=Col(col[count],0.5),pch=2)
      points(t[cause0[,2]==causes[i],],col=Col(col[count],0.5),pch=6)
      count <- count+1
    }
    points(t[cause0[,1]==0 & cause0[,2]==0,],col=Col("black",0.2),pch=1)  
    if (legend)
    legend("topleft", c("Subj 1, Cause 1", "Subj 2, Cause 1",
                        "Subj 1, Cause 2", "Subj 2, Cause 2",
                        "Double Censoring"), pch=c(2,6,2,6,1), col=c(rep(col[1:length(causes)],each=2),"black"))
  }
  if (2%in%which) {
    dots$curvlab <- NULL
    colnames(x)[1:4] <- c("t1","t2","cause1","cause2")
    co1 <- prodlim::prodlim(Hist(t1,cause1)~1,data.frame(x))
    co2 <- prodlim::prodlim(Hist(t2,cause2)~1,data.frame(x))
    if (is.null(dots$lwd)) dots$lwd <- 1
    if (missing(lty)) lty <- seq_len(length(co1$cuminc))
    if (missing(col)) col <- rep(1,length(co1$cuminc))
    if (is.null(dots$lwd)) dots$lwd <- 1
    dots$lty <- lty[1]
    dots$col <- col[1]
    dots$cause <- cause[1]
    do.call("plot", c(list(x=co1), dots))    
    dots$add <- TRUE
    do.call("plot", c(list(x=co2), dots))
##    for (i in seq_len(length(co1$cuminc)-1)+1) {
    for (i in seq_len(length(cause)-1)+1) {
        dots$lty <- lty[i]
        dots$col <- col[i]
        dots$cause <- cause[i]
        do.call("plot", c(list(x=co1), dots))
        do.call("plot", c(list(x=co2), dots))  
    }
    if (legend && length(cause)>1)
      legend("topleft",names(co1$cuminc)[cause],col=col,lty=lty,pch=-1,bg="white")
  }
}

##' Life-course plot for event life data with recurrent events 
##'
##' @title Life-course plot
##' @param formula Formula (Event(start,slut,status) ~ ...) 
##' @param data data.frame
##' @param id Id variable 
##' @param group group variable
##' @param type Type (line 'l', stair 's', ...)
##' @param lty Line type
##' @param col Colour
##' @param alpha transparency (0-1)
##' @param lwd Line width
##' @param recurrent.col col of recurrence type
##' @param recurrent.lty lty's of  of recurrence type
##' @param legend position of optional id legend
##' @param pchlegend point type legends 
##' @param by make separate plot for each level in 'by' (formula, name of column, or vector)
##' @param status.legend Status legend
##' @param place.sl Placement of status legend
##' @param xlab Label of X-axis
##' @param ylab Label of Y-axis
##' @param add Add to existing device
##' @param ... Additional arguments to lower level arguments
##' @author Thomas Scheike, Klaus K. Holst
##' @export
##' @examples
##' data = data.frame(id=c(1,1,1,2,2),start=c(0,1,2,3,4),slut=c(1,2,4,4,7),
##'                   type=c(1,2,3,2,3),status=c(0,1,2,1,2),group=c(1,1,1,2,2))
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id")
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",recurrent.col="type")
##'
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2)
##' op <- par(mfrow=c(1,2))
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",by=~group)
##' par(op)
##' legends=c("censored","pregnant","married")
##' ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2,status.legend=legends)
##'
lifecourse <- function(formula,data,id="id",group=NULL,
               type="l",lty=1,col=1:10,alpha=0.3,lwd=1,
               recurrent.col=NULL, recurrent.lty=NULL,
               legend=NULL,pchlegend=NULL,
               by=NULL,
               status.legend=NULL,place.sl="bottomright",
               xlab="Time",ylab="",add=FALSE,...) 
{# {{{
    if (!is.null(by)) {
        if (is.character(by) && length(by==1)) {
            by <- data[,by]
        } else if (inherits(by,"formula")) {
            by <- model.frame(by,data,na.action=na.pass)
        }
        cl <- match.call(expand.dots=TRUE)
        cl$by <- NULL
        datasets <- split(data,by)
        res <- c()
        for (d in datasets) {
            cl$data <- d
            res <- c(res, eval(cl,parent.frame()))
        }
        return(invisible(res))
    }
    if (!is.null(group)) {
        if (is.character(group) && length(group==1)) {
            M <- data[,group]
        } else if (inherits(group,"formula")) {
            M <- model.frame(group,data,na.action=na.pass)
        } else {
            M <- group
        }
###        if (!add) plot(formula,data=data,xlab=xlab,ylab=ylab,...,type="n")        
        if (!add) {
	 if (inherits(id,"formula")) id <- all.vars(id)
	 if (inherits(group,"formula")) group <- all.vars(group)
	 if (is.character(id) && length(id)==1) Id <- 
	 y <- getoutcome(formula)
	 x <- attributes(y)$x

    if (length(x)==0) {# {{{

        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster.index(data[,id])
	ccm <- ccid$idclustmat+1
        ccm <- ccm[!is.na(ccm)]
        ###
        x <- data[ccm,id]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

	X <- c(c(t1),c(t2))
	Y <- rep(x,each=2)
	status <- tstat

# }}}
    } else {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster.index(data[,id])
	ccm <- ccid$idclustmat+1
	ccm <- ccm[!is.na(ccm)]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

        x <- data[ccm,x] 

	X <- c(c(t1),c(t2))
	Y <- rep(x,each=2)
	status <- tstat

    }# }}}

	 plot(X,Y,xlab=xlab,ylab=ylab,...,type="n")        

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}

       }

        dd <- split(data,M)
        K <- length(dd)
        if (length(type)<K)        type <- rep(type,K)
        if (length(col)<K)         col <- rep(col,K)
        if (length(lty)<K)         lty <- rep(lty,K)
        if (length(lwd)<K)         lwd <- rep(lwd,K)
        if (length(alpha)<K)       alpha <- rep(alpha,K)
	res <- list()
        for (i in seq_len(K)) {            
           res[[i]] <- lifecourse(formula,data=dd[[i]],id=id,type=type[i],
                     lty=lty[i],col=col[i],lwd=lwd[i],
                     alpha=alpha[i],
		     recurrent.col=recurrent.col,
		     recurrent.lty=recurrent.lty,
                     group=NULL,
                     add=TRUE,...)
        }

        if (!is.null(legend)) {
            graphics::legend(legend,names(dd),lwd=lwd,col=col,lty=lty)
        }
        return(invisible(res))
    }
    
    if (inherits(id,"formula")) id <- all.vars(id)
    if (inherits(group,"formula")) group <- all.vars(group)
    if (is.character(id) && length(id)==1) Id <- 
    y <- getoutcome(formula)
    x <- attributes(y)$x

    if (length(x)==0) {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster.index(data[,id])
	ccm <- ccid$idclustmat+1
        ccm <- ccm[!is.na(ccm)]
        ###
        x <- data[ccm,id]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

	X <- cbind(c(t1),c(t2))
	Y <- matrix(rep(x,each=2),ncol=2,byrow=TRUE)
	status <- tstat

###	print(dim(X)); print(dim(Y)); print(dim(status)); print(summary(X)); print(summary(Y)); 

	if (is.null(recurrent.col))  {
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	} else {
	   cn <- data[ccm,recurrent.col]
	   matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
	}

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}


# }}}
    } else {# {{{
        y <- response <- all.vars( update(formula,.~+1))
        ###
	ccid <- cluster.index(data[,id])
	ccm <- ccid$idclustmat+1
	ccm <- ccm[!is.na(ccm)]

	if (length(response)==3) { 
	t1 <-    data[ccm,response[1]] 
	t2 <-    data[ccm,response[2]] 
	tstat <- data[ccm,response[3]]
	} else {
	t1 <-    rep(0,length(ccm))
	t2 <-    data[ccm,response[1]] 
	tstat <- data[ccm,response[2]]
	}

        x <- data[ccm,x] 

	X <- cbind(c(t1),c(t2))
	Y <- matrix(rep(x,each=2),ncol=2,byrow=TRUE)
	status <- tstat

###	print(dim(X)); print(dim(Y)); print(dim(status)); print(summary(X)); print(summary(Y)); 



	if (is.null(recurrent.col))  cols <- col[1] else { 
            cols <- data[ccm,recurrent.lty]
	}
        if (is.null(recurrent.lty))  ltys <- lty else { 
            ltys <- data[ccm,recurrent.lty]
	}

###	print("hej")
###	print(t(Y))
###	print(t(X))
###	print(type)
###        matplot(t(X),t(Y),type="l",lwd=lwd,col=cols,xlab=xlab,ylab=ylab,add=add,...)

	if (is.null(recurrent.col))  {
###         matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,
###	          col=Col(col[1],alpha[1]),xlab=xlab,ylab=ylab,...)
###	          col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=col[1],xlab=xlab,ylab=ylab,add=add,...)
	} else {
            cn <- data[ccm,recurrent.col]
	    if (!is.null(recurrent.lty))   {
            cl <- data[ccm,recurrent.lty]
            matplot(t(X),t(Y),type=type,lty=cl,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
	    } else matplot(t(X),t(Y),type=type,lty=lty,lwd=lwd,col=cn,xlab=xlab,ylab=ylab,add=add,...)
       }

       if (!is.null(status.legend)) {# {{{
	    if (is.null(pchlegend)) {
	       if (length(status.legend)!=length(unique(status)))
		   {
		           warning("Not all legends represented, legends could be wrong give pchlegend\n"); 
			   print(cbind(status.legend,sort(unique(status))))
		   }
                    points(t2,x,pch=status)
		    graphics::legend(place.sl,legend=status.legend,pch=sort(unique(status)))
	    } else { 
            points(t2,x,pch=pchlegend[status])
            graphics::legend(place.sl,legend=status.legend,pch=pchlegend)
       }
       }# }}}

    }# }}}

###    return(invisible(data.frame(Y=Y,X=X,status=status)))
    return(invisible(list(Y,X,status)))
}# }}}

##' @export
loglikMVN <- function(yl,yu,status,mu,S,thres) {
    .Call("_mets_loglikMVN",
          yl=as.matrix(yl),
          yu=as.matrix(yu),
          status=as.integer(status),
          mu=as.matrix(mu),dmu=NULL,s=as.matrix(S),ds=NULL,
          z=NULL,su=NULL,dsu=NULL,
          threshold=as.matrix(thres),dthreshold=NULL,
          score=FALSE, itol=lava.options()$itol,
          PACKAGE="mets")
}


##' @export
scoreMVN <- function(y,mu,S,dmu,dS) {
    .Call("_mets_loglikMVN",
          yl=as.matrix(y),
          yu=NULL,
          status=NULL,
          mu=as.matrix(mu),dmu=as.matrix(dmu),
          s=as.matrix(S),ds=as.matrix(dS),
          z=NULL,su=NULL,dsu=NULL,
          threshold=NULL,dthreshold=NULL,
          score=TRUE, itol=lava.options()$itol,
          PACKAGE="mets")
}

##' @export
dsample <- function(data,x,size=NULL,replace=TRUE,...) {
    if (missing(x)) {
        if (is.null(size)) size <- NROW(data)
        return(data[sample.int(NROW(data), size, replace=replace),,drop=FALSE])
    }
    inp <- procform(x,data=data,return.formula=TRUE)
    if (length(inp$filter.expression)>0)
        data <- subset(data,eval(inp$filter.expression))
    if (!is.null(inp$predictor))
        idvar <- model.frame(inp$predictor, data=data, na.action=na.pass,...)
    if (!is.null(inp$response)) {
        if (is.null(size)) size <- NROW(data)
        data <- model.frame(inp$response, data=data, na.action=na.pass,...)
    }
    if (is.null(inp$predictor)) {
        if (is.null(size)) size <- NROW(data)
        return(data[sample.int(NROW(data), size, replace=replace),,drop=FALSE])
    }
    blocksample(data,idvar=idvar,size=size,replace=replace,...)
}

##' Sample blockwise from clustered data
##'
##' @title Block sampling
##' @param data Data frame
##' @param idvar Column defining the clusters
##' @param size Size of samples
##' @param replace Logical indicating wether to sample with replacement
##' @param \dots additional arguments to lower level functions
##' @return \code{data.frame}
##' @author Klaus K. Holst
##' @keywords models utilities
##' @aliases blocksample dsample
##' @details Original id is stored in the attribute 'id'
##' @export
##' @examples
##'
##' d <- data.frame(x=rnorm(5), z=rnorm(5), id=c(4,10,10,5,5), v=rnorm(5))
##' (dd <- blocksample(d,size=20,~id))
##' attributes(dd)$id
##'
##' \dontrun{
##' blocksample(data.table::data.table(d),1e6,~id)
##' }
##'
##'
##' d <- data.frame(x=c(1,rnorm(9)),
##'                z=rnorm(10),
##'                id=c(4,10,10,5,5,4,4,5,10,5),
##'                id2=c(1,1,2,1,2,1,1,1,1,2),
##'                v=rnorm(10))
##' dsample(d,~id, size=2)
##' dsample(d,.~id+id2)
##' dsample(d,x+z~id|x>0,size=5)
##'
blocksample <- function(data, size, idvar=NULL, replace=TRUE, ...) {
    if (is.null(idvar)) {
        return(data[sample(NROW(data),size,replace=replace),,drop=FALSE])
    }
    if (inherits(idvar,"formula")) {
        idvar <- all.vars(idvar)
    }
    if (NROW(idvar)==nrow(data)) {
        id0 <- idvar
    } else {
        if (inherits(data,"data.table")) {
            id0 <- as.data.frame(data[,idvar,with=FALSE])[,1]
        } else id0 <- data[,idvar]
    }
    if (NCOL(id0)>1) {
        id0 <- interaction(as.data.frame(id0))
    }
    id0 <- as.matrix(id0)[,1,drop=TRUE]
    ii <-  cluster.index(as.matrix(id0))
    size <- ifelse(missing(size) || is.null(size),ii$uniqueclust,size)
    ids <- sample(seq(ii$uniqueclust), size=size,replace=replace)
    idx <- na.omit(as.vector(t(ii$idclustmat[ids,])))+1
    newid <- rep(seq(size), ii$cluster.size[ids])
    oldid <- id0[idx]
    res <- data[idx,]
    if (is.character(idvar) && length(idvar)==1) {
        res[,idvar] <- newid
    } else {
        res <- cbind(res,id=newid)
        colnames(res) <- make.unique(colnames(res))
    }
    attributes(res)$id <- oldid
    return(res)
}

##' Set global options for \code{mets}
##'
##' Extract and set global parameters of \code{mets}. 
##'
##' \itemize{
##'   \item \code{regex}: If TRUE character vectors will be interpreted as regular expressions (\code{dby}, \code{dcut}, ...)
##'   \item \code{silent}: Set to \code{FALSE} to disable various output messages
##' }
##' @param ... Arguments
##' @return \code{list} of parameters
##' @keywords models
##' @examples
##' \dontrun{
##' mets.options(regex=TRUE)
##' }
##' @export
mets.options <- function(...) {
    dots <- list(...)
    newopt <- curopt <- get("options",envir=mets.env)
    if (length(dots)==0)
        return(curopt)
    if (length(dots)==1 && is.list(dots[[1]]) && is.null(names(dots))) {
        dots <- dots[[1]]
    }
    idx <- which(names(dots)!="")
    newopt[names(dots)[idx]] <- dots[idx]
    assign("options",newopt,envir=mets.env)
    invisible(curopt)
}

mets.env <- new.env()
assign("options",
       list(debug=FALSE,
            regex=FALSE,
            regex.perl=FALSE            
            ), envir=mets.env)

##' @export
jumptimes <- function(time, status=TRUE, 
                      id,cause,
                      sample,sample.all=TRUE,
                      strata=NULL,num=NULL, ...) {
    if (missing(id)) {
        time <- if (missing(cause)) time[status>0]
                else time[status==cause]
    } else {
        ww <- na.omit(fast.reshape(cbind(time=time,status=status),id=id,num=num))
        statusvar <- grep("status",colnames(ww))
        timevar <- grep("time",colnames(ww))
        if (missing(cause)) {
            idx <- which(rowSums(ww[,statusvar]>0)==length(statusvar))
        } else {
            idx <- which(apply(as.matrix(ww[,statusvar]),1,function(x) all(x==cause)))
        }
            time <- na.omit(do.call(pmax,as.list(ww[idx,timevar])))
    }

    if (!missing(sample) && sample<length(time)) {
        time <- sort(unique(time))
        t0 <- seq(min(time),max(time),length.out=min(sample,length(time)))
        ii <- fast.approx(time,t0)
        ii <- unique(ii)
        time0 <- time[ii]
        if (length(ii)<sample && sample.all) {
            remain <- setdiff(seq(length(time)),ii)
            time0 <- c(time0,time[base::sample(remain,sample-length(ii))])
        }
        time <- time0
    } 
    return(sort(time))
}

## with(prt, jumptimes(time,status==2,id=id,sample=10))

library(mets)
###
set.seed(1000)
data <- simClaytonOakes.family.ace(8000,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)
out <- ace.family.design(data,member="type",id="cluster")
### 8 random effects with 1/4 * var.gene, and one shared environment  1 * var.env
out$pardes
head(out$des.rv)
###
aa <- phreg(Surv(time,status)~+cluster(cluster),data=data)

## {{{ additive gamma models with and without pair call 
### make ace random effects design

### simple random effects call 
ts0 <- survival.twostage(aa,data=data,clusters=data$cluster,
	detail=1,var.par=1,var.link=0,
        theta=c(2,1),
        random.design=out$des.rv,theta.des=out$pardes)
summary(ts0)

### simple random effects call 
ts1 <- survival.twostage(aa,data=data,clusters=data$cluster,
	detail=1,var.par=0,var.link=0,
        theta=c(2,1)/9,
        random.design=out$des.rv,theta.des=out$pardes)
summary(ts1)
### parameters c(2,1)/(2+1)^2

checkderiv=1
if (checkderiv==1) {# {{{

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	detail=1,numDeriv=1,Nit=0,var.par=1,
        theta=log(c(2,1)/9),var.link=1,step=1.0,
        random.design=out$des.rv,theta.des=out$pardes)
ts0$score
ts0$score1

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	detail=1,numDeriv=1,Nit=0,var.par=1,
        theta=c(2,1)/9,var.link=0,step=1.0,
        random.design=out$des.rv,theta.des=out$pardes)
ts0$score
ts0$score1


ts0 <- twostage(aa,data=data,clusters=data$cluster,
	detail=1,numDeriv=1,Nit=0,var.par=0,
        theta=log(c(2,1)),var.link=1,step=1.0,
        random.design=out$des.rv,theta.des=out$pardes)
ts0$score
ts0$score1

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	detail=1,numDeriv=1,Nit=0,var.par=0,
        theta=c(2,1),var.link=0,step=1.0,
        random.design=out$des.rv,theta.des=out$pardes)
ts0$score
ts0$score1

}# }}}


### now specify fitting via specific pairs 
### first construct all pairs 
mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
## make all pairs and pair specific design and pardes 
## same as ts0 but pairs specified 
tsp <- twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1)/9,var.link=0,step=1.0,var.par=0,
               random.design=out$des.rv,detail=0,
               theta.des=out$pardes,pairs=pairs)
summary(tsp)

tsp1 <- twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1),var.link=0,step=1.0,var.par=1,
               random.design=out$des.rv,detail=0,
               theta.des=out$pardes,pairs=pairs)
summary(tsp1)


 source("../../R/twostage.R")
aa <- aalen(Surv(time,status)~+1,data=data,robust=0)

tsp2 <- twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1),var.link=0,step=1.0,
               random.design=out$des.rv,detail=0,
               theta.des=out$pardes,pairs=pairs)
summary(tsp2)

c(tsp1$marginal.surv-tsp2$marginal.surv)


### random sample of pairs 
ssid <- sort(sample(1:32000,20000))
###
### take some of all 
tsd <- twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1)/10,var.link=0,step=1.0,
               random.design=out$des.rv,iid=1,
	      theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)

str(aa)
aa$id


### same analyses but now gives only data that is used in the relevant pairs 
ids <- sort(unique(c(pairs[ssid,])))
###
pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)

## this requires that pair.new refers to id's in dataid (survival, status and so forth)
## random.design and theta.des are constructed to be the array 3 dims via individual specfication from ace.family.design
dataid <- dsort(data[ids,],"cluster")
outid <- ace.family.design(dataid,member="type",id="cluster")
outid$pardes
head(outid$des.rv)
###
tsid <- twostage(aa,data=dataid,clusters=dataid$cluster,
               theta=c(2,1)/10,var.link=0,step=1.0,
               random.design=outid$des.rv,iid=1,
               theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)
coef(tsdid)
coef(tsd)
### same as tsd 


### now direct specification of random.design and theta.design 
### rather than taking the rows of the des.rv for the relevant pairs
### can make a pair specific specification of random effects 

pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new)
head(pair.types)

### here makes pairwise design , simpler random.design og pardes, parameters
### stil varg, varc 
### mother, child, share half rvm=c(1,1,0) rvc=c(1,0,1),
### thetadesmcf=rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))
###
### father, child, share half rvf=c(1,1,0) rvc=c(1,0,1), 
### thetadescf=rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))
###
### child, child,  share half rvc=c(1,1,0) rvc=c(1,0,1),
### thetadesmf=rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))
###
### mother, father, share 0 rvm=c(1,0) rvf=c(0,1), 
### thetadesmf=rbind(c(1,0),c(1,0),c(0,1))

theta.des  <- array(0,c(4,2,nrow(pair.new)))
random.des <- array(0,c(2,4,nrow(pair.new)))
### random variables in each pair 
rvs <- c()
for (i in 1:nrow(pair.new))
{
	if (pair.types[i,1]=="mother" & pair.types[i,2]=="father")
	{
	theta.des[,,i] <- rbind(c(1,0),c(1,0),c(0,1),c(0,0))
       	random.des[,,i] <- rbind(c(1,0,1,0),c(0,1,1,0))
	rvs <- c(rvs,3)
	} else {
  	theta.des[,,i] <- rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))
	random.des[,,i] <- rbind(c(1,1,0,1),c(1,0,1,1))
	rvs <- c(rvs,4)
	}
}
### 3 rvs here 
random.des[,,7]
theta.des[,,7]
### 4 rvs here 
random.des[,,1]
theta.des[,,1]
head(rvs)

tsdid2 <- twostage(aa,data=dataid,clusters=dataid$cluster,
               theta=c(2,1)/10,var.link=0,step=1.0,
               random.design=random.des,
               theta.des=theta.des,pairs=pair.new,pairs.rvs=rvs)
summary(tsdid2)
tsd$theta
tsdid2$theta
tsdid$theta


### simpler specification via kinship coefficient for each pair

kinship  <- c()
for (i in 1:nrow(pair.new))
{
if (pair.types[i,1]=="mother" & pair.types[i,2]=="father") pk1 <- 0 else pk1 <- 0.5
kinship <- c(kinship,pk1)
}
head(kinship,n=10)

out <- make.pairwise.design(pair.new,kinship,type="ace") 
names(out)
### 4 rvs here , here independence since shared component has variance 0 !
out$random.des[,,9]
out$theta.des[,,9]


tsdid3 <- twostage(aa,data=dataid,clusters=dataid$cluster,
               theta=c(2,1)/10,var.link=0,step=1.0,
               random.design=out$random.design,
               theta.des=out$theta.des,pairs=pair.new,pairs.rvs=out$ant.rvs)
summary(tsdid3)
coef(tsdid3)

### same as above  tsdid2

## }}} 

##### simple models, test for pairs structure ## {{{ 

library(mets)
source("../R/twostage.R")
ts0 <- twostage(aa,data=data,clusters=data$cluster,
	detail=0,numDeriv=1,Nit=10,
        theta=c(0.17),var.link=0,step=1.0)
summary(ts0)
ts0$score; ts0$score1
ts0$Dscore; ts0$hess

mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
head(pairs,n=12)
tail(pairs,n=12)
dim(pairs)
#
cc <- cluster.index(data$cluster)
###
ts0 <- twostage(aa,data=data,clusters=data$cluster,
        detail=1,Nit=0,
        theta=ts0$theta,var.link=0,pairs=pairs)
summary(ts0)


## {{{  simple models with pair call 

library(mets)

set.seed(100)
data <- simClaytonOakes.family.ace(8000,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

### make ace random effects design
out <- ace.family.design(data,member="type",id="cluster")
out$pardes
head(out$des.rv)

### makes marginal model (same for all) 
aa <- aalen(Surv(time,status)~+1,data=data,robust=0)


mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
head(pairs,n=12)
tail(pairs,n=12)
dim(pairs)
#

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,
        theta=c(0.2),var.link=0,step=1.0)
summary(ts0)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,numDeriv=1,
        theta=c(0.2),var.link=0,step=1.0,pairs=pairs)
summary(ts0)
ts0$score
ts0$score1

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,
        theta=c(0.2),var.link=0,step=1.0,model="plackett")
summary(ts0)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,
        theta=c(0.2),var.link=0,step=1.0,model="plackett",pairs=pairs)
summary(ts0)



theta.des <- model.matrix(~x1,data=data)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,theta.des=theta.des,
        theta=c(0.2),var.link=0,step=1.0)
summary(ts0)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,theta.des=theta.des,
        theta=c(0.2),var.link=0,step=1.0,pairs=pairs)
summary(ts0)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,theta.des=theta.des,
        theta=c(0.2),var.link=0,step=1.0,model="plackett")
summary(ts0)

ts0 <- twostage(aa,data=data,clusters=data$cluster,
	 detail=1,Nit=10,theta.des=theta.des,
        theta=c(0.2),var.link=0,step=1.0,model="plackett",pairs=pairs)
summary(ts0)



## }}} 


## }}} 

##' Regression for data frames with dutility call
##'
##' Regression for data frames with dutility call
##' @param data data frame
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x name of variable, or fomula, or names of variables on data frame.
##' @param z name of variable, or fomula, or names of variables on data frame.
##' @param x.oneatatime x's one at a time
##' @param x.base.names base covarirates
##' @param z.arg what is Z, c("clever","base","group","condition"), clever decides based on type of Z, base means that Z is used as fixed baseline covaraites for all X, group means the analyses is done based on groups of Z, and condition means that Z specifies a condition on the data
##' @param fun. function  lm is default
##' @param summary. summary to use
##' @param regex regex
##' @param convert convert
##' @param doSummary doSummary or not
##' @param special special's
##' @param equal to do pairwise stuff
##' @param test development argument
##' @param ... Additional arguments for fun
##' @author Klaus K. Holst, Thomas Scheike
##' @examples##'
##' data(iris)
##' dat <- iris
##' drename(dat) <- ~.
##' names(dat)
##' set.seed(1)
##' dat$time <- runif(nrow(dat))
##' dat$time1 <- runif(nrow(dat))
##' dat$status <- rbinom(nrow(dat),1,0.5)
##' dat$S1 <- with(dat, Surv(time,status))
##' dat$S2 <- with(dat, Surv(time1,status))
##' dat$id <- 1:nrow(dat)
##'
##' mm <- dreg(dat, "*.length"~"*.width"|I(species=="setosa" & status==1))
##' mm <- dreg(dat, "*.length"~"*.width"|species+status)
##' mm <- dreg(dat, "*.length"~"*.width"|species)
##' mm <- dreg(dat, "*.length"~"*.width"|species+status,z.arg="group")
##'
##' \donttest{ ## Reduce Ex.Timings
##' y <- "S*"~"*.width"
##' xs <- dreg(dat, y, fun.=phreg)
##' xs <- dreg(dat, y, fun.=survdiff)
##'
##' y <- "S*"~"*.width"
##' xs <- dreg(dat, y, x.oneatatime=FALSE, fun.=phreg)
##'
##' ## under condition
##' y <- S1~"*.width"|I(species=="setosa" & sepal.width>3)
##' xs <- dreg(dat, y, z.arg="condition", fun.=phreg)
##' xs <- dreg(dat, y, fun.=phreg)
##'
##' ## under condition
##' y <- S1~"*.width"|species=="setosa"
##' xs <- dreg(dat, y, z.arg="condition", fun.=phreg)
##' xs <- dreg(dat, y, fun.=phreg)
##'
##' ## with baseline  after |
##' y <- S1~"*.width"|sepal.length
##' xs <- dreg(dat, y, fun.=phreg)
##'
##' ## by group by species, not working
##' y <- S1~"*.width"|species
##' ss <- split(dat, paste(dat$species, dat$status))
##'
##' xs <- dreg(dat, y, fun.=phreg)
##'
##' ## species as base, species is factor so assumes that this is grouping
##' y <- S1~"*.width"|species
##' xs <- dreg(dat, y, z.arg="base", fun.=phreg)
##'
##' ##  background var after | and then one of x's at at time
##' y <- S1~"*.width"|status+"sepal*"
##' xs <- dreg(dat, y, fun.=phreg)
##'
##' ##  background var after | and then one of x's at at time
##' ##y <- S1~"*.width"|status+"sepal*"
##' ##xs <- dreg(dat, y, x.oneatatime=FALSE, fun.=phreg)
##' ##xs <- dreg(dat, y, fun.=phreg)
##'
##' ##  background var after | and then one of x's at at time
##' ##y <- S1~"*.width"+factor(species)
##' ##xs <- dreg(dat, y, fun.=phreg)
##' ##xs <- dreg(dat, y, fun.=phreg, x.oneatatime=FALSE)
##'
##' y <- S1~"*.width"|factor(species)
##' xs <- dreg(dat, y, z.arg="base", fun.=phreg)
##'
##' y <- S1~"*.width"|cluster(id)+factor(species)
##' xs <- dreg(dat, y, z.arg="base", fun.=phreg)
##' xs <- dreg(dat, y, z.arg="base", fun.=coxph)
##'
##' ## under condition with groups
##' y <- S1~"*.width"|I(sepal.length>4)
##' xs <- dreg(subset(dat, species=="setosa"), y,z.arg="group",fun.=phreg)
##'
##' ## under condition with groups
##' y <- S1~"*.width"+I(log(sepal.length))|I(sepal.length>4)
##' xs <- dreg(subset(dat, species=="setosa"), y,z.arg="group",fun.=phreg)
##'
##' y <- S1~"*.width"+I(dcut(sepal.length))|I(sepal.length>4)
##' xs <- dreg(subset(dat,species=="setosa"), y,z.arg="group",fun.=phreg)
##'
##' ff <- function(formula,data,...) {
##'  ss <- survfit(formula,data,...)
##'  kmplot(ss,...)
##'  return(ss)
##' }
##'
##' if (interactive()) {
##' dcut(dat) <- ~"*.width"
##' y <- S1~"*.4"|I(sepal.length>4)
##' par(mfrow=c(1, 2))
##' xs <- dreg(dat, y, fun.=ff)
##' }
##' }
##'
##' @export
dreg <- function(data,y,x=NULL,z=NULL,x.oneatatime=TRUE,
	 x.base.names=NULL,z.arg=c("clever","base","group","condition"),
         fun.=lm,summary.=summary,regex=FALSE,convert=NULL,doSummary=TRUE,
	 special=NULL,equal=TRUE,test=1,...) {# {{{
### z.arg=clever,  if z is logical then condition
###                if z is factor  then group variable
###                if z is numeric then baseline covariate
### ... further arguments to fun

###  fun <- as.character(substitute(fun))
###    if (is.character(fun))
###        fun <- get(fun)
###    if (!is.null(convert) && is.logical(convert)) {
###        if (convert)
###            convert <- as.matrix
###        else convert <- NULL
###    }
###    if (!is.null(convert)) {
###        fun_ <- fun
###        fun <- function(x, ...) fun_(convert(x, ...))
###    }
###    print(fun)
###    print(str(fun))

 yxzf <- procform(y,x=x,z=z,data=data,do.filter=FALSE,regex=regex)
 yxz <- procformdata(y,x=x,z=z,data=data,do.filter=FALSE,regex=regex)
### print(yxz); print(yxzf)

 ## remove blank, to able to use also 	+1 on right hand side
 if (any(yxzf$predictor==""))
    yxzf$predictor <- yxzf$predictor[-which(yxzf$predictor=="")]

 yy <- yxz$response
 xx <- yxz$predictor
 ### group is list, so zz is data.frame
 if ((length(yxzf$filter))==0) zz <- NULL else if ((length(yxzf$filter[[1]])==1 & yxzf$filter[[1]][1]=="1"))
      zz <- NULL else   zz <- yxz$group[[1]]

 if (!is.null(zz)) {# {{{
 if (z.arg[1]=="clever")
 {
   if ((ncol(zz)==1) & is.logical(zz[1,1])) z.arg[1] <- "condition"
       else if ((ncol(zz)==1) & is.factor(zz[,1])) z.arg[1] <- "group"
       else  z.arg[1] <- "base"
  }
  }# }}}
### print(z.arg)

 basen <- NULL
 if (z.arg[1]=="base")
     basen <- yxzf$filter[[1]]
 if (z.arg[1]=="condition")
     data <- subset(data,eval(yxzf$filter.expression))
 if (z.arg[1]=="group")
     group <- interaction(zz) else group <- rep(1,nrow(data))
 if (z.arg[1]=="group")  levell <- levels(group) else levell <-1


 res <- sum <- list()
 if (test==1) {
 if (is.null(summary)) sum <- NULL
 for (g in levell) {# {{{
 if (equal==TRUE) datal <- subset(data,group==g)
 else datal <- subset(data,group!=g)
 for (y in yxzf$response) {# {{{
	 if (x.oneatatime)  {
		 for (x in yxzf$predictor) {
	             if (length(c(x,basen))>1)
                     basel <- paste(c(x,basen),collapse="+")
		     else basel <- c(x,basen)
	             form <- as.formula(paste(y,"~",basel))
		     if (!is.null(special)) form <- timereg::timereg.formula(form,special=special)
###	     capture.output()
             val <- do.call(fun.,c(list(formula=form),list(data=datal),list(...)))
             val <- list(val)
	     nn <- paste(y,"~",basel)
	     if (z.arg[1]=="group") {
		     if (equal==TRUE) nn <- paste(nn,"|",g)  else nn <- paste(nn,"| not",g);
	     }
	     names(val) <- nn
	     ## to avoid call stuff
	     val[[1]]$call <- nn
             res <- c(res, val)
	     if (doSummary) {
	        sval <- list(do.call(summary.,list(val[[1]])))
                names(sval) <- nn
	        sum <- c(sum, sval)
	    }
	 }
	 } else {
             basel <- paste(c(yxzf$predictor,basen),collapse="+")
             form <- as.formula(paste(y,"~",basel))
	     if (!is.null(special)) form <- timereg::timereg.formula(form,special=special)
###	     capture.output()
             val <- do.call(fun.,c(list(formula=form),list(data=datal),list(...)))
	     nn <- paste(y,"~",basel)
	     if (z.arg[1]=="group") {
		     if (equal==TRUE) nn <- paste(nn,"|",g)  else nn <- paste(nn,"| not",g);
	     }
             val <- list(val)
	     names(val) <- paste(y,"~",basel)
	     ## to avoid call stuff
	     val[[1]]$call <- nn
             res <- c(res, val)
	     if (doSummary) {
	       sval <- list(do.call(summary.,list(val[[1]])))
	       names(sval) <- nn
	       sum <- c(sum, sval)
	     }

	 }
 }# }}}
 }# }}}
 }

   res <- list(reg=res,summary=sum)
###       res <- list(setNames(res,funn),summary=sum,...)
   class(res) <- "dreg"
###   structure(res,ngrouvar=0,class="dreg")
   return(res)

}# }}}

##' @export
print.dreg <- function(x,sep="-",...) {# {{{
    sep <- paste(rep(sep,50,sep=""),collapse="")
    sep <-  paste(sep,"\n")
    nn <-  names(x$reg)
    for (i in seq_along(x$reg)) {
        cat(paste("Model=",nn[i],"\n"))
            print(x$reg[[i]],...)
        cat(sep)
    }

}# }}}

##' @export
summary.dreg <- function(object,sep="-",...) {# {{{
    x <- object
    sep <- paste(rep(sep,50,sep=""),collapse="")
    sep <-  paste(sep,"\n")
###    cat(sep)
###    if (inherits(x$lm, c("lm"))) {
###        print(x$lm)
###        if (!is.null(x$summary)) print(x$summary)
###        return(invisible(x))
###    }
if (!is.null(x$summary)) {
    nn <-  names(x$summary)
    for (i in seq_along(x$summary)) {
        cat(paste("Model=",nn[i],"\n"))
        if (!is.null(x$summary))
            print(x$summary[[i]],...)
        else print(x$reg[[i]],...)
        cat(sep)
    }
}

}# }}}

propOdds <- function(time,status,X,
                         ...) {  
  id <- 1:length(time)
  theta <- 1
  dix <- which(status==1)
  fB <- fast.approx(time[dix],time)
  cc <- cluster.index(id)
  ncluster <- length(cc$clusters)
  U <- function(beta,indiv=FALSE) {
      B <- .Call("pBhat",as.integer(status),X,beta,as.integer(cc$clusters),
                cc$idclust,as.integer(cc$cluster.size),PACKAGE="mets")$B
      Ba <- B[fB$pos+1,,drop=FALSE]
      Hij <- as.vector(X*Ba)
      res <- .Call("Uhat",as.integer(status),Hij,theta,
                  cc$idclust,as.integer(cc$cluster.size),PACKAGE="mets")
      if (!indiv) res <- mean(res)
      res
  }
  if (missing(theta)) theta <- 0.1
  if (is.list(theta)) return(lapply(theta,function(x) U(x,...)))

  op <- nlminb(theta,function(x) U(x)^2)
  uu <- U(op$par,TRUE)
  du <- numDeriv::grad(U,op$par)
  return(list(theta=op$par, sd=(mean(uu^2)/du^2/ncluster)^0.5))
}
                  
  

#####' @export
###surv.boxarea <- function(left.trunc,right.cens,data,timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1,boxtimevar="boxtime")
###{ ## {{{
###  if (is.null(data[,id])) stop("Wrong cluster variable")
###  if (is.null(data[,timevar])) stop("Wrong time variable")
###  if (is.null(data[,status])) stop("Wrong status variable")
###  data <- data[order(data[,id]),]
###  if (silent<=-1) {
###    message("survboxare()")
###    print(head(data))
###    print(summary(data[,id]))
###  }
###  if (is.null(num)) {
###    idtab <- table(data[,id])
###    num <- "num"
###    while (num%in%names(data)) num <- paste(num,"_",sep="")
###    data[,c(num)] <- unlist(lapply(idtab,seq_len))
###  }
###
###  timevar2 <- paste(timevar,1:2,sep="")
###  status2 <- paste(status,1:2,sep="")
###  num2 <- paste(num,1:2,sep="")
###  covars2 <- NULL; covars.pairs2 <- NULL; 
###  if (length(covars)>0) covars2 <- paste(covars,1:2,sep="")
###  if (length(covars.pairs)>0) covars.pairs2 <- paste(covars.pairs,1:2,sep="")
###
###  if (silent<=-1) {
###    message("survboxare()")
###    print(head(data))
###    print( c(timevar,status,covars,covars.pairs,id,num))
###    print(c(id,num))
###    print(summary(data))
###  }
###  ww0 <- fast.reshape(data[,c(timevar,status,covars,covars.pairs,id,num)],id=id,num=num,labelnum=TRUE)
###  if (silent<=-1) {
###    message("survboxarea(), ww1")
###    print(head(ww0))
###    print(summary(ww0))
###    print(c(timevar2,status2,covars2,covars.pairs2,id,num2)) 
###    ww0 <- data.frame(ww0); 
###    print(table(ww0$status1,ww0$status2))
###  }
###  mleft <-  (ww0[,timevar2[1]]>left.trunc[1]) & (ww0[,timevar2[2]]>left.trunc[2])  ## Both not-truncated
###
###  if (length(na.idx <- which(is.na(mleft)))>0) {
###    ##    warning("Removing incomplete cases", na.idx)
###    mleft <- mleft[-na.idx]
###    ww0 <- ww0[-na.idx,,drop=FALSE]
###  }
###  if (sum(mleft)==0) stop("No data selected\n"); 
###  ww0 <- ww0[which(mleft),,drop=FALSE]
###
###  right1 <- which(ww0[,timevar2[1]] > right.cens[1])
###  right2 <- which(ww0[,timevar2[2]] > right.cens[2])
###  ww0[,timevar2[1]][right1] <- right.cens[1]
###  ww0[,timevar2[2]][right2] <- right.cens[2]
###  ww0[,status2[1]][right1] <- 0
###  ww0[,status2[2]][right2] <- 0
###  truncvar2 <- c("left1","left2")
###  ww0 <- cbind(ww0,left.trunc[1])
###  ww0 <- cbind(ww0,left.trunc[2])
###  colnames(ww0)[c(-1,0) + ncol(ww0)] <- truncvar2
###
###  if (silent<=-1) print(head(ww0))
###  if (silent<=0)
###    message(paste("  Number of joint events:",sum(apply(ww0[,status2],1,sum)==2),"of ",nrow(ww0)),"\n");
###
###  varying <- c(timevar,status,"left",covars)
###  lr.data <- data.frame(fast.reshape(ww0,varying=varying,numname=num))
###  if (silent<=-1) {
###    print("surv.boxarea after fast.reshape"); 
###    print(head(lr.data))
###    print(summary(lr.data[,id]))
###  }
###  lr.data[,boxtimevar] <- lr.data[,timevar]-lr.data[,"left"]
###  return(structure(lr.data,num=num,time=boxtimevar,status=status,covars=covars,id=id,left=left))
###} ## }}}

##' @export
surv.boxarea <- function(left.trunc,right.cens,data,timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1,boxtimevar="boxtime")
{ ## {{{
  if (is.null(data[,id])) stop("Wrong cluster variable")
  if (is.null(data[,timevar])) stop("Wrong time variable")
  if (is.null(data[,status])) stop("Wrong status variable")
  data <- data[order(data[,id]),]
  if (silent<=-1) {
    message("survboxare()")
    print(head(data))
    print(summary(data[,id]))
  }
  if (is.null(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,c(num)] <- unlist(lapply(idtab,seq_len))
  }

###  if (is.null(ssname)) {
###    idtab <- table(data[,id])
###    num <- "num"
###    while (num%in%names(data)) num <- paste(num,"_",sep="")
###    data[,c(num)] <- unlist(lapply(idtab,seq_len))
###  }

  timevar2 <- paste(timevar,1:2,sep="")
  status2 <- paste(status,1:2,sep="")
  num2 <- paste(num,1:2,sep="")
  covars2 <- NULL; covars.pairs2 <- NULL; 
  if (length(covars)>0) covars2 <- paste(covars,1:2,sep="")
  if (length(covars.pairs)>0) covars.pairs2 <- paste(covars.pairs,1:2,sep="")

  if (silent<=-1) {
    message("survboxare()")
    print(head(data))
    print( c(timevar,status,covars,covars.pairs,id,num))
    print(c(id,num))
    print(summary(data))
  }
  ww0 <- fast.reshape(data[,c(timevar,status,covars,covars.pairs,id,num)],id=id,num=num,labelnum=TRUE)
  if (silent<=-1) {
    message("survboxarea(), ww1")
    print(head(ww0))
    print(summary(ww0))
    print(c(timevar2,status2,covars2,covars.pairs2,id,num2)) 
    ww0 <- data.frame(ww0); 
    print(table(ww0$status1,ww0$status2))
  }
  mleft <-  (ww0[,timevar2[1]]>left.trunc[1]) & (ww0[,timevar2[2]]>left.trunc[2])  ## Both not-truncated

  if (length(na.idx <- which(is.na(mleft)))>0) {
    ##    warning("Removing incomplete cases", na.idx)
    mleft <- mleft[-na.idx]
    ww0 <- ww0[-na.idx,,drop=FALSE]
  }
  if (sum(mleft)==0) cat("No data selected\n"); 
  if (sum(mleft)!=0) {
  ww0 <- ww0[which(mleft),,drop=FALSE]

  right1 <- which(ww0[,timevar2[1]] > right.cens[1])
  right2 <- which(ww0[,timevar2[2]] > right.cens[2])
  ww0[,timevar2[1]][right1] <- right.cens[1]
  ww0[,timevar2[2]][right2] <- right.cens[2]
  ww0[,status2[1]][right1] <- 0
  ww0[,status2[2]][right2] <- 0
  truncvar2 <- c("left1","left2")
  ww0 <- cbind(ww0,left.trunc[1])
  ww0 <- cbind(ww0,left.trunc[2])
  colnames(ww0)[c(-1,0) + ncol(ww0)] <- truncvar2
###  ww0[,"intnames1"] <- paste("[",left.trunc[1],",",right.cens[1],")",sep="")
###  ww0[,"intnames2"] <- paste("[",left.trunc[2],",",right.cens[2],")",sep="")

  if (silent<=-1) print(head(ww0))
  if (silent<=0)
    message(paste("  Number of joint events:",sum(apply(ww0[,status2],1,sum)==2),"of ",nrow(ww0)),"\n");

  varying <- c(timevar,status,"left",covars)
  lr.data <- data.frame(fast.reshape(ww0,varying=varying,numname=num))
  if (silent<=-1) {
    print("surv.boxarea after fast.reshape"); 
    print(head(lr.data))
    print(summary(lr.data[,id]))
  }
  lr.data[,boxtimevar] <- lr.data[,timevar]-lr.data[,"left"]
###  print(head(lr.data))
 return(structure(lr.data,num=num,time=boxtimevar,status=status,covars=covars,id=id))
} else return(NULL); 
} ## }}}

##' @export
logLik.biprobit <- function(object,indiv=FALSE,...) {
  if (indiv) return(object$logLik)
  n <- sum(object$N[1])
  p <- length(coef(object))
  loglik <- sum(object$logLik)
  attr(loglik, "nall") <- n
  attr(loglik, "nobs") <- n
  attr(loglik, "df") <- p
  class(loglik) <- "logLik"        
  return(loglik)
}

*
* This file contains a test program and functions TVTL (trivariate normal 
* and t), BVTL (bivariate t), BVND (bivariate normal), STUDNT (univariate
* t), PHID (univariate normal), plus some support functions.
* The file is self contained and should compile without errors on (77) 
* standard Fortran compilers. The test program demonstrates the use of
* TVTL for computing trivariate distribution values 20 test problems
* with NU = 0 (normal case), 3, 6, 9, and 12.
*
* The software is based on work described in the paper
*  "Numerical Computation of Rectangular Bivariate and Trivariate Normal
*    and t Probabilities", by
*
*          Alan Genz
*          Department of Mathematics
*          Washington State University
*          Pullman, WA 99164-3113
*          Email : alangenz@wsu.edu
*
*
*       Original source available from
*       http://www.math.wsu.edu/faculty/genz/software/fort77/tvpack.f
*
c$$$      PROGRAM TVTST
c$$$      INTEGER I, J, NU, NT
c$$$      PARAMETER ( NT = 20 )
c$$$      DOUBLE PRECISION TVTL, LIMIT(3,NT), SIGMA(3,NT), EPS, V
c$$$      DATA ( LIMIT(I,1), I = 1, 3 ), ( SIGMA(I,1), I = 1, 3 )
c$$$     &     / .5D0, .5D0, .8D0, .1D0, .6D0, .8D0 /
c$$$      DATA ( LIMIT(I,2), I = 1, 3 ), ( SIGMA(I,2), I = 1, 3 )
c$$$     &     / -2.5D0, .5D0, .8D0, .1D0, -.6D0, -.8D0 /
c$$$      DATA ( LIMIT(I,3), I = 1, 3 ), ( SIGMA(I,3), I = 1, 3 )
c$$$     &     / 1.5D0, .5D0, .8D0, .1D0, .6D0, .8D0 /
c$$$      DATA ( LIMIT(I,4), I = 1, 3 ), ( SIGMA(I,4), I = 1, 3 )
c$$$     &     / .5D0, .5D0, .8D0, .1D0, -.6D0, -.8D0 /
c$$$      DATA ( LIMIT(I,5), I = 1, 3 ), ( SIGMA(I,5), I = 1, 3 )
c$$$     &     / .5D0, .5D0, .8D0, .1D0, -.5D0, .5D0 /
c$$$      DATA ( LIMIT(I,6), I = 1, 3 ), ( SIGMA(I,6), I = 1, 3 )
c$$$     &     / -1.5D0, .5D0, .8D0, .1D0, -.5D0, .5D0 /
c$$$      DATA ( LIMIT(I,7), I = 1, 3 ), ( SIGMA(I,7), I = 1, 3 )
c$$$     &     / 1.5D0, .5D0, .8D0, .1D0, .5D0, -.5D0 /
c$$$      DATA ( LIMIT(I,8), I = 1, 3 ), ( SIGMA(I,8), I = 1, 3 )
c$$$     &     / -.5D0, 1D0, 1.2D0, -.4D0, .2D0, .7D0 /
c$$$      DATA ( LIMIT(I,9), I = 1, 3 ), ( SIGMA(I,9), I = 1, 3 )
c$$$     &     / 1D0, 1D0, 2D0, .4D0, .8D0, .8D0 /
c$$$      DATA ( LIMIT(I,10), I = 1, 3 ), ( SIGMA(I,10), I = 1, 3 )
c$$$     &     / 1D0, 2D0, 1D0, .4D0, .8D0, .8D0 /
c$$$      DATA ( LIMIT(I,11), I = 1, 3 ), ( SIGMA(I,11), I = 1, 3 )
c$$$     &     / -2D0, -2D0, -2D0, .4D0, .8D0, .8D0 /
c$$$      DATA ( LIMIT(I,12), I = 1, 3 ), ( SIGMA(I,12), I = 1, 3 )
c$$$     *     / 1D0, 2D0, 3D0, -.998D0, -0.248D0, 0.248D0 /
c$$$      DATA ( LIMIT(I,13), I = 1, 3 ), ( SIGMA(I,13), I = 1, 3 )
c$$$     *     / -1D0, 2D0, 3D0, .25D0, 0.25D0, 0.25D0 /
c$$$      DATA ( LIMIT(I,14), I = 1, 3 ), ( SIGMA(I,14), I = 1, 3 )
c$$$     *     /  1D0, 1D0, 3D0, .998D0, 0.2482D0, 0.2487D0 /
c$$$      DATA ( LIMIT(I,15), I = 1, 3 ), ( SIGMA(I,15), I = 1, 3 )
c$$$     *     /  1D0, 1D0, 3D0, .998D0, 0.5D0, 0.5D0 /
c$$$      DATA ( LIMIT(I,16), I = 1, 3 ), ( SIGMA(I,16), I = 1, 3 )
c$$$     *     /  1D0, 1D0, 3D0, .99D0, 0.99D0, 0.99D0 /
c$$$      DATA ( LIMIT(I,17), I = 1, 3 ), ( SIGMA(I,17), I = 1, 3 )
c$$$     *     /  1D0, 2D0, 3D0, -1D0, -.99D0, .99D0 /
c$$$      DATA ( LIMIT(I,18), I = 1, 3 ), ( SIGMA(I,18), I = 1, 3 )
c$$$     *     /  1D0, 2D0, 3D0, 1D0, -.99D0, -.99D0 /
c$$$      DATA ( LIMIT(I,19), I = 1, 3 ), ( SIGMA(I,19), I = 1, 3 )
c$$$     *     /  1D0, -1D0, 1D0, .998D0, -0.2482D0, -0.2482D0 /
c$$$      DATA ( LIMIT(I,NT), I = 1, 3 ), ( SIGMA(I,NT), I = 1, 3 )
c$$$     *     /  1D0, -1D0, 2D0, .99992D0, 0.64627D0, 0.63975D0 /
c$$$      EPS = 1D-6
c$$$      PRINT '(''      Trivariate t Test with EPS ='', E10.1)', EPS
c$$$      DO NU = 0, 12, 3
c$$$         PRINT '(''NU   B1   B2   B3    R21      R31      R32    TVT'')'
c$$$         DO J = 1, NT 
c$$$            V = TVTL( NU, LIMIT(1,J), SIGMA(1,J), EPS )
c$$$            PRINT '(I2,3F5.1,3F9.5,F13.10)', NU, 
c$$$     &           ( LIMIT(I,J), I = 1, 3 ), ( SIGMA(I,J), I = 1, 3 ), V
c$$$         END DO
c$$$      END DO
c$$$      END
*
      DOUBLE PRECISION FUNCTION TVTL( NU, H, R, EPSI )
*    
*     A function for computing trivariate normal and t-probabilities.
*     This function uses algorithms developed from the ideas 
*     described in the papers:
*       R.L. Plackett, Biometrika 41(1954), pp. 351-360.
*       Z. Drezner, Math. Comp. 62(1994), pp. 289-294.
*     with adaptive integration from (0,0,1) to (0,0,r23) to R. 
*
*      Calculate the probability that X(I) < H(I), for I = 1,2,3     
*    NU   INTEGER degrees of freedom; use NU = 0 for normal cases.
*    H    REAL array of uppoer limits for probability distribution 
*    R    REAL array of three correlation coefficients, R should 
*         contain the lower left portion of the correlation matrix r. 
*         R should contains the values r21, r31, r23 in that order.
*   EPSI  REAL required absolute accuracy; maximum accuracy for most
*          computations is approximately 1D-14
* 
*    The software is based on work described in the paper
*     "Numerical Computation of Rectangular Bivariate and Trivariate
*      Normal and t Probabilities", by the code author:
*
*       Alan Genz
*       Department of Mathematics
*       Washington State University
*       Pullman, WA 99164-3113
*       Email : alangenz@wsu.edu
*
      EXTERNAL TVTMFN
      INTEGER NU, NUC
      DOUBLE PRECISION H(3), H1, H2, H3, R(3), R12, R13, R23, EPSI
      DOUBLE PRECISION ONE, ZRO, EPS, TVT
* ZROS(3), HS(3)
      DOUBLE PRECISION RUA, RUB, AR, RUC, PT, BVTL, PHID, ADONET
      PARAMETER ( ZRO = 0, ONE = 1 )
      COMMON /TVTMBK/ H1, H2, H3, R23, RUA, RUB, AR, RUC, NUC
      EPS = MAX( 1D-14, EPSI )
      PT = ASIN(ONE)
      NUC = NU
      H1 = H(1)
      H2 = H(2)
      H3 = H(3)
      R12 = R(1)
      R13 = R(2)
      R23 = R(3)
*
*     Sort R's and check for special cases
*
      IF ( ABS(R12) .GT. ABS(R13) ) THEN
         H2 = H3
         H3 = H(2)
         R12 = R13
         R13 = R(1)
      END IF
      IF ( ABS(R13) .GT. ABS(R23) ) THEN
         H1 = H2
         H2 = H(1)
         R23 = R13
         R13 = R(3)
      END IF
      TVT = 0
      IF ( ABS(H1) + ABS(H2) + ABS(H3) .LT. EPS ) THEN 
         TVT = ( 1 + ( ASIN(R12) + ASIN(R13) + ASIN(R23) )/PT )/8
      ELSE IF ( NU .LT. 1 .AND. ABS(R12) + ABS(R13) .LT. EPS ) THEN 
         TVT = PHID(H1)*BVTL( NU, H2, H3, R23 )
      ELSE IF ( NU .LT. 1 .AND. ABS(R13) + ABS(R23) .LT. EPS ) THEN 
         TVT = PHID(H3)*BVTL( NU, H1, H2, R12 )
      ELSE IF ( NU .LT. 1 .AND. ABS(R12) + ABS(R23) .LT. EPS ) THEN 
         TVT = PHID(H2)*BVTL( NU, H1, H3, R13 )
      ELSE IF ( 1 - R23 .LT. EPS ) THEN
         TVT = BVTL( NU, H1, MIN( H2, H3 ), R12 )
      ELSE IF ( R23 + 1 .LT. EPS ) THEN
         IF  ( H2 .GT. -H3 ) 
     &        TVT = BVTL( NU, H1, H2, R12 ) - BVTL( NU, H1, -H3, R12 )
      ELSE
*
*        Compute singular TVT value
*
         IF ( NU .LT. 1 ) THEN
            TVT = BVTL( NU, H2, H3, R23 )*PHID(H1)
         ELSE IF ( R23 .GE. 0 ) THEN
            TVT = BVTL( NU, H1, MIN( H2, H3 ), ZRO )
         ELSE IF ( H2 .GT. -H3 ) THEN 
            TVT = BVTL( NU, H1, H2, ZRO ) - BVTL( NU, H1, -H3, ZRO )
         END IF
*
*        Use numerical integration to compute probability
*
*
         RUA = ASIN( R12 )
         RUB = ASIN( R13 )
         AR = ASIN( R23)
         RUC = SIGN( PT, AR ) - AR
         TVT = TVT + ADONET( TVTMFN, ZRO, ONE, EPS )/( 4*PT )
      END IF
      TVTL = MAX( ZRO, MIN( TVT, ONE ) ) 
      END
*
      DOUBLE PRECISION FUNCTION TVTMFN( X )
*
*     Computes Plackett formula integrands
*
      INTEGER NU
      DOUBLE PRECISION X, H1, H2, H3, R23, RUA, RUB, AR, RUC
      DOUBLE PRECISION R12, RR2, R13, RR3, R, RR, ZRO, PNTGND
      PARAMETER ( ZRO = 0 )    
      COMMON /TVTMBK/ H1, H2, H3, R23, RUA, RUB, AR, RUC, NU
      TVTMFN = 0
      CALL SINCS( RUA*X, R12, RR2 )
      CALL SINCS( RUB*X, R13, RR3 )
      IF ( ABS(RUA) .GT. 0 ) 
     &     TVTMFN = TVTMFN + RUA*PNTGND( NU, H1,H2,H3, R13,R23,R12,RR2 )
      IF ( ABS(RUB) .GT. 0 )
     &     TVTMFN = TVTMFN + RUB*PNTGND( NU, H1,H3,H2, R12,R23,R13,RR3 )
      IF ( NU .GT. 0 ) THEN
         CALL SINCS( AR + RUC*X, R, RR )
         TVTMFN = TVTMFN - RUC*PNTGND( NU, H2, H3, H1, ZRO, ZRO, R, RR )
      END IF
      END      
*
      SUBROUTINE SINCS( X, SX, CS )
*
*     Computes SIN(X), COS(X)^2, with series approx. for |X| near PI/2
*
      DOUBLE PRECISION X, SX, CS, PT, EE
      PARAMETER ( PT = 1.57079632679489661923132169163975D0 )
      EE = ( PT - ABS(X) )**2
      IF ( EE .LT. 5D-5 ) THEN
         SX = SIGN( 1 - EE*( 1 - EE/12 )/2, X )
         CS = EE*( 1 - EE*( 1 - 2*EE/15 )/3 )
      ELSE
         SX = SIN(X)
         CS = 1 - SX*SX
      END IF
      END
*
      DOUBLE PRECISION FUNCTION PNTGND( NU, BA, BB, BC, RA, RB, R, RR )
*
*     Computes Plackett formula integrand
*
      INTEGER NU
      DOUBLE PRECISION BA, BB, BC, RA, RB, R, RR
      DOUBLE PRECISION DT, FT, BT, PHID, STUDNT
      PNTGND = 0
      DT = RR*( RR - ( RA - RB )**2 - 2*RA*RB*( 1 - R ) )
      IF ( DT .GT. 0 ) THEN
         BT = ( BC*RR + BA*( R*RB - RA ) + BB*( R*RA -RB ) )/SQRT(DT) 
         FT = ( BA - R*BB )**2/RR + BB*BB
         IF ( NU .LT. 1 ) THEN
            IF ( BT .GT. -10 .AND. FT .LT. 100 ) THEN
               PNTGND = EXP( -FT/2 )
               IF ( BT .LT. 10 ) PNTGND = PNTGND*PHID(BT)
            END IF
         ELSE
            FT = SQRT( 1 + FT/NU )
            PNTGND = STUDNT( NU, BT/FT )/FT**NU
         END IF
      END IF
      END 
*
      DOUBLE PRECISION FUNCTION ADONET( F, A, B, TOL )
*
*     One Dimensional Globally Adaptive Integration Function
*
      EXTERNAL F
      DOUBLE PRECISION F, A, B, TOL
      INTEGER NL, I, IM, IP
      PARAMETER ( NL = 100 )
      DOUBLE PRECISION EI(NL), AI(NL), BI(NL), FI(NL), FIN, ERR, KRNRDT
      COMMON /ABLK/ ERR, IM
      AI(1) = A
      BI(1) = B
      FIN = 0
      ERR = 1
      IP = 1
      IM = 1
      DO WHILE ( 4*ERR .GT. TOL .AND. IM .LT. NL ) 
         IM = IM + 1
         BI(IM) = BI(IP)
         AI(IM) = ( AI(IP) + BI(IP) )/2
         BI(IP) = AI(IM)
         FI(IP) = KRNRDT( AI(IP), BI(IP), F, EI(IP) )
         FI(IM) = KRNRDT( AI(IM), BI(IM), F, EI(IM) )
         ERR = 0
         FIN = 0
         DO I = 1, IM
            IF ( EI(I) .GT. EI(IP) ) IP = I
            FIN = FIN + FI(I)
            ERR = ERR + EI(I)**2
         END DO
         ERR = SQRT( ERR )
      END DO
      ADONET = FIN
      END
*
      DOUBLE PRECISION FUNCTION KRNRDT( A, B, F, ERR )
*
*     Kronrod Rule
*
      DOUBLE PRECISION A, B, ERR, T, CEN, F, FC, WID, RESG, RESK
*
*        The abscissae and weights are given for the interval (-1,1);
*        only positive abscissae and corresponding weights are given.
*
*        XGK    - abscissae of the 2N+1-point Kronrod rule: 
*                 XGK(2), XGK(4), ...  N-point Gauss rule abscissae; 
*                 XGK(1), XGK(3), ...  optimally added abscissae.
*        WGK    - weights of the 2N+1-point Kronrod rule.
*        WG     - weights of the N-point Gauss rule.
*
      INTEGER J, N
      PARAMETER ( N = 11 )
      DOUBLE PRECISION WG(0:(N+1)/2), WGK(0:N), XGK(0:N) 
      SAVE WG, WGK, XGK
      DATA WG( 0)/ 0.2729250867779007D+00/
      DATA WG( 1)/ 0.5566856711617449D-01/
      DATA WG( 2)/ 0.1255803694649048D+00/
      DATA WG( 3)/ 0.1862902109277352D+00/
      DATA WG( 4)/ 0.2331937645919914D+00/
      DATA WG( 5)/ 0.2628045445102478D+00/
*
      DATA XGK( 0)/ 0.0000000000000000D+00/
      DATA XGK( 1)/ 0.9963696138895427D+00/
      DATA XGK( 2)/ 0.9782286581460570D+00/
      DATA XGK( 3)/ 0.9416771085780681D+00/
      DATA XGK( 4)/ 0.8870625997680953D+00/
      DATA XGK( 5)/ 0.8160574566562211D+00/
      DATA XGK( 6)/ 0.7301520055740492D+00/
      DATA XGK( 7)/ 0.6305995201619651D+00/
      DATA XGK( 8)/ 0.5190961292068118D+00/
      DATA XGK( 9)/ 0.3979441409523776D+00/
      DATA XGK(10)/ 0.2695431559523450D+00/
      DATA XGK(11)/ 0.1361130007993617D+00/
*
      DATA WGK( 0)/ 0.1365777947111183D+00/
      DATA WGK( 1)/ 0.9765441045961290D-02/
      DATA WGK( 2)/ 0.2715655468210443D-01/
      DATA WGK( 3)/ 0.4582937856442671D-01/
      DATA WGK( 4)/ 0.6309742475037484D-01/
      DATA WGK( 5)/ 0.7866457193222764D-01/
      DATA WGK( 6)/ 0.9295309859690074D-01/
      DATA WGK( 7)/ 0.1058720744813894D+00/
      DATA WGK( 8)/ 0.1167395024610472D+00/
      DATA WGK( 9)/ 0.1251587991003195D+00/
      DATA WGK(10)/ 0.1312806842298057D+00/
      DATA WGK(11)/ 0.1351935727998845D+00/
*
*           Major variables
*
*           CEN  - mid point of the interval
*           WID  - half-length of the interval
*           RESG - result of the N-point Gauss formula
*           RESK - result of the 2N+1-point Kronrod formula
*
*           Compute the 2N+1-point Kronrod approximation to
*            the integral, and estimate the absolute error.
*
      WID = ( B - A )/2
      CEN = ( B + A )/2
      FC = F(CEN)
      RESG = FC*WG(0)
      RESK = FC*WGK(0)
      DO J = 1, N
         T = WID*XGK(J) 
         FC = F( CEN - T ) + F( CEN + T )
         RESK = RESK + WGK(J)*FC
         IF( MOD( J, 2 ) .EQ. 0 ) RESG = RESG + WG(J/2)*FC
      END DO
      KRNRDT = WID*RESK
      ERR = ABS( WID*( RESK - RESG ) )
      END
*
*      DOUBLE PRECISION FUNCTION STUDNT( NU, T )
*
*     Student t Distribution Function
*
*                       T
*         STUDNT = C   I  ( 1 + y*y/NU )**( -(NU+1)/2 ) dy
*                   NU -INF
*
*      INTEGER NU, J
*      DOUBLE PRECISION T, ZRO, ONE, PI, PHID
*      DOUBLE PRECISION CSSTHE, SNTHE, POLYN, TT, TS, RN
*      PARAMETER ( ZRO = 0, ONE = 1 )
*      PI = ACOS(-ONE)
*      IF ( NU .LT. 1 ) THEN
*         STUDNT = PHID( T )
*      ELSE IF ( NU .EQ. 1 ) THEN
*         STUDNT = ( 1 + 2*ATAN(T)/PI )/2
*      ELSE IF ( NU .EQ. 2 ) THEN
*         STUDNT = ( 1 + T/SQRT( 2 + T*T ))/2
*      ELSE 
*         TT = T*T
*         CSSTHE = 1/( 1 + TT/NU )
*         POLYN = 1
*         DO J = NU-2, 2, -2
*            POLYN = 1 + ( J - 1 )*CSSTHE*POLYN/J
*         END DO
*         IF ( MOD( NU, 2 ) .EQ. 1 ) THEN
*            RN = NU
*            TS = T/SQRT(RN)
*            STUDNT = ( 1 + 2*( ATAN(TS) + TS*CSSTHE*POLYN )/PI )/2
*         ELSE
*            SNTHE = T/SQRT( NU + TT )
*            STUDNT = ( 1 + SNTHE*POLYN )/2
*         END IF
*         STUDNT = MAX( ZRO, MIN( STUDNT, ONE ) )
*      ENDIF
*      END
*
      DOUBLE PRECISION FUNCTION BVTL( NU, DH, DK, R )
*
*     A function for computing bivariate t probabilities.
*
*       Alan Genz
*       Department of Mathematics
*       Washington State University
*       Pullman, WA 99164-3113
*       Email : alangenz@wsu.edu
*
*    This function is based on the method described by 
*        Dunnett, C.W. and M. Sobel, (1954),
*        A bivariate generalization of Student's t-distribution
*        with tables for certain special cases,
*        Biometrika 41, pp. 153-169.
*
* BVTL - calculate the probability that X < DH and Y < DK. 
*
* parameters
*
*   NU number of degrees of freedom
*   DH 1st lower integration limit
*   DK 2nd lower integration limit
*   R   correlation coefficient
*
      INTEGER NU, J, HS, KS
      DOUBLE PRECISION DH, DK, R
      DOUBLE PRECISION TPI, PI, ORS, HRK, KRH, BVT, SNU, BVND, STUDNT
      DOUBLE PRECISION GMPH, GMPK, XNKH, XNHK, QHRK, HKN, HPK, HKRN
      DOUBLE PRECISION BTNCKH, BTNCHK, BTPDKH, BTPDHK, ONE, EPS
      PARAMETER ( ONE = 1, EPS = 1D-15 )
      IF ( NU .LT. 1 ) THEN
         BVTL = BVND( -DH, -DK, R )
      ELSE IF ( 1 - R .LE. EPS ) THEN
            BVTL = STUDNT( NU, MIN( DH, DK ) )
      ELSE IF ( R + 1  .LE. EPS ) THEN
         IF ( DH .GT. -DK )  THEN
            BVTL = STUDNT( NU, DH ) - STUDNT( NU, -DK )
         ELSE
            BVTL = 0
         END IF
      ELSE
         PI = ACOS(-ONE)
         TPI = 2*PI
         SNU = NU
         SNU = SQRT(SNU)
         ORS = 1 - R*R  
         HRK = DH - R*DK  
         KRH = DK - R*DH  
         IF ( ABS(HRK) + ORS .GT. 0 ) THEN
            XNHK = HRK**2/( HRK**2 + ORS*( NU + DK**2 ) ) 
            XNKH = KRH**2/( KRH**2 + ORS*( NU + DH**2 ) ) 
         ELSE
            XNHK = 0  
            XNKH = 0  
         END IF
         HS = INT( SIGN( ONE, DH - R*DK ) )
         KS = INT ( SIGN( ONE, DK - R*DH ) )
         IF ( MOD( NU, 2 ) .EQ. 0 ) THEN
            BVT = ATAN2( SQRT(ORS), -R )/TPI 
            GMPH = DH/SQRT( 16*( NU + DH**2 ) )  
            GMPK = DK/SQRT( 16*( NU + DK**2 ) )  
            BTNCKH = 2*ATAN2( SQRT( XNKH ), SQRT( 1 - XNKH ) )/PI  
            BTPDKH = 2*SQRT( XNKH*( 1 - XNKH ) )/PI 
            BTNCHK = 2*ATAN2( SQRT( XNHK ), SQRT( 1 - XNHK ) )/PI  
            BTPDHK = 2*SQRT( XNHK*( 1 - XNHK ) )/PI 
            DO J = 1, NU/2
               BVT = BVT + GMPH*( 1 + KS*BTNCKH ) 
               BVT = BVT + GMPK*( 1 + HS*BTNCHK ) 
               BTNCKH = BTNCKH + BTPDKH  
               BTPDKH = 2*J*BTPDKH*( 1 - XNKH )/( 2*J + 1 )  
               BTNCHK = BTNCHK + BTPDHK  
               BTPDHK = 2*J*BTPDHK*( 1 - XNHK )/( 2*J + 1 )  
               GMPH = GMPH*( 2*J - 1 )/( 2*J*( 1 + DH**2/NU ) ) 
               GMPK = GMPK*( 2*J - 1 )/( 2*J*( 1 + DK**2/NU ) ) 
            END DO
         ELSE
            QHRK = SQRT( DH**2 + DK**2 - 2*R*DH*DK + NU*ORS )  
            HKRN = DH*DK + R*NU  
            HKN = DH*DK - NU  
            HPK = DH + DK 
            BVT = ATAN2( -SNU*( HKN*QHRK + HPK*HKRN ),
     &                          HKN*HKRN-NU*HPK*QHRK )/TPI
            IF ( BVT .LT. -EPS ) BVT = BVT + 1
            GMPH = DH/( TPI*SNU*( 1 + DH**2/NU ) )  
            GMPK = DK/( TPI*SNU*( 1 + DK**2/NU ) )  
            BTNCKH = SQRT( XNKH )  
            BTPDKH = BTNCKH 
            BTNCHK = SQRT( XNHK )  
            BTPDHK = BTNCHK  
            DO J = 1, ( NU - 1 )/2
               BVT = BVT + GMPH*( 1 + KS*BTNCKH ) 
               BVT = BVT + GMPK*( 1 + HS*BTNCHK ) 
               BTPDKH = ( 2*J - 1 )*BTPDKH*( 1 - XNKH )/( 2*J )  
               BTNCKH = BTNCKH + BTPDKH  
               BTPDHK = ( 2*J - 1 )*BTPDHK*( 1 - XNHK )/( 2*J )  
               BTNCHK = BTNCHK + BTPDHK  
               GMPH = 2*J*GMPH/( ( 2*J + 1 )*( 1 + DH**2/NU ) ) 
               GMPK = 2*J*GMPK/( ( 2*J + 1 )*( 1 + DK**2/NU ) ) 
            END DO
         END IF
         BVTL = BVT 
      END IF
*     END BVTL
      END
*
*      DOUBLE PRECISION FUNCTION PHID(Z)
*     
*     Normal distribution probabilities accurate to 1d-15.
*     Reference: J.L. Schonfelder, Math Comp 32(1978), pp 1232-1240. 
*     
*      INTEGER I, IM
*      DOUBLE PRECISION A(0:43), BM, B, BP, P, RTWO, T, XA, Z
*      PARAMETER( RTWO = 1.414213562373095048801688724209D0, IM = 24 )
*      SAVE A
*      DATA ( A(I), I = 0, 43 )/
*     &    6.10143081923200417926465815756D-1,
*     &   -4.34841272712577471828182820888D-1,
*     &    1.76351193643605501125840298123D-1,
*     &   -6.0710795609249414860051215825D-2,
*     &    1.7712068995694114486147141191D-2,
*     &   -4.321119385567293818599864968D-3, 
*     &    8.54216676887098678819832055D-4, 
*     &   -1.27155090609162742628893940D-4,
*     &    1.1248167243671189468847072D-5, 3.13063885421820972630152D-7,      
*     &   -2.70988068537762022009086D-7, 3.0737622701407688440959D-8,
*     &    2.515620384817622937314D-9, -1.028929921320319127590D-9,
*     &    2.9944052119949939363D-11, 2.6051789687266936290D-11,
*     &   -2.634839924171969386D-12, -6.43404509890636443D-13,
*     &    1.12457401801663447D-13, 1.7281533389986098D-14, 
*     &   -4.264101694942375D-15, -5.45371977880191D-16,
*     &    1.58697607761671D-16, 2.0899837844334D-17, 
*     &   -5.900526869409D-18, -9.41893387554D-19, 2.14977356470D-19, 
*     &    4.6660985008D-20, -7.243011862D-21, -2.387966824D-21, 
*     &    1.91177535D-22, 1.20482568D-22, -6.72377D-25, -5.747997D-24,
*     &   -4.28493D-25, 2.44856D-25, 4.3793D-26, -8.151D-27, -3.089D-27, 
*     &    9.3D-29, 1.74D-28, 1.6D-29, -8.0D-30, -2.0D-30 /       
*     
*      XA = ABS(Z)/RTWO
*      IF ( XA .GT. 100 ) THEN
*         P = 0
*      ELSE
*         T = ( 8*XA - 30 ) / ( 4*XA + 15 )
*         BM = 0
*         B  = 0
*         DO I = IM, 0, -1 
*            BP = B
*            B  = BM
*            BM = T*B - BP  + A(I)
*         END DO
*         P = EXP( -XA*XA )*( BM - BP )/4
*      END IF
*      IF ( Z .GT. 0 ) P = 1 - P
*      PHID = P
*      END
*
      DOUBLE PRECISION FUNCTION BVND( DH, DK, R )
*
*     A function for computing bivariate normal probabilities.
*
*       Alan Genz
*       Department of Mathematics
*       Washington State University
*       Pullman, WA 99164-3113
*       Email : alangenz@wsu.edu
*
*    This function is based on the method described by 
*        Drezner, Z and G.O. Wesolowsky, (1989),
*        On the computation of the bivariate normal integral,
*        Journal of Statist. Comput. Simul. 35, pp. 101-107,
*    with major modifications for double precision, and for |R| close to 1.
*
* BVND calculates the probability that X > DH and Y > DK.
*      Note: Prob( X < DH, Y < DK ) = BVND( -DH, -DK, R ).
*
* Parameters
*
*   DH  DOUBLE PRECISION, integration limit
*   DK  DOUBLE PRECISION, integration limit
*   R   DOUBLE PRECISION, correlation coefficient
*
      DOUBLE PRECISION DH, DK, R, TWOPI 
      INTEGER I, IS, LG, NG
      PARAMETER ( TWOPI = 6.283185307179586D0 ) 
      DOUBLE PRECISION X(10,3), W(10,3), AS, A, B, C, D, RS, XS, BVN 
      DOUBLE PRECISION PHID, SN, ASR, H, K, BS, HS, HK
*     Gauss Legendre Points and Weights, N =  6
      DATA ( W(I,1), X(I,1), I = 1,3) /
     &  0.1713244923791705D+00,-0.9324695142031522D+00,
     &  0.3607615730481384D+00,-0.6612093864662647D+00,
     &  0.4679139345726904D+00,-0.2386191860831970D+00/
*     Gauss Legendre Points and Weights, N = 12
      DATA ( W(I,2), X(I,2), I = 1,6) /
     &  0.4717533638651177D-01,-0.9815606342467191D+00,
     &  0.1069393259953183D+00,-0.9041172563704750D+00,
     &  0.1600783285433464D+00,-0.7699026741943050D+00,
     &  0.2031674267230659D+00,-0.5873179542866171D+00,
     &  0.2334925365383547D+00,-0.3678314989981802D+00,
     &  0.2491470458134029D+00,-0.1252334085114692D+00/
*     Gauss Legendre Points and Weights, N = 20
      DATA ( W(I,3), X(I,3), I = 1, 10 ) /
     &  0.1761400713915212D-01,-0.9931285991850949D+00,
     &  0.4060142980038694D-01,-0.9639719272779138D+00,
     &  0.6267204833410906D-01,-0.9122344282513259D+00,
     &  0.8327674157670475D-01,-0.8391169718222188D+00,
     &  0.1019301198172404D+00,-0.7463319064601508D+00,
     &  0.1181945319615184D+00,-0.6360536807265150D+00,
     &  0.1316886384491766D+00,-0.5108670019508271D+00,
     &  0.1420961093183821D+00,-0.3737060887154196D+00,
     &  0.1491729864726037D+00,-0.2277858511416451D+00,
     &  0.1527533871307259D+00,-0.7652652113349733D-01/
      SAVE X, W
      IF ( ABS(R) .LT. 0.3 ) THEN
         NG = 1
         LG = 3
      ELSE IF ( ABS(R) .LT. 0.75 ) THEN
         NG = 2
         LG = 6
      ELSE 
         NG = 3
         LG = 10
      ENDIF
      H = DH
      K = DK 
      HK = H*K
      BVN = 0
      IF ( ABS(R) .LT. 0.925 ) THEN
         IF ( ABS(R) .GT. 0 ) THEN 
            HS = ( H*H + K*K )/2
            ASR = ASIN(R)
            DO I = 1, LG
               DO IS = -1, 1, 2
                  SN = SIN( ASR*(  IS*X(I,NG) + 1 )/2 )
                  BVN = BVN + W(I,NG)*EXP( ( SN*HK-HS )/( 1-SN*SN ) )
               END DO
            END DO
            BVN = BVN*ASR/( 2*TWOPI )
         ENDIF
         BVN = BVN + PHID(-H)*PHID(-K)
      ELSE
         IF ( R .LT. 0 ) THEN
            K = -K
            HK = -HK
         ENDIF
         IF ( ABS(R) .LT. 1 ) THEN
            AS = ( 1 - R )*( 1 + R )
            A = SQRT(AS)
            BS = ( H - K )**2
            C = ( 4 - HK )/8 
            D = ( 12 - HK )/16
            ASR = -( BS/AS + HK )/2
            IF ( ASR .GT. -100 ) BVN = A*EXP(ASR)
     &             *( 1 - C*( BS - AS )*( 1 - D*BS/5 )/3 + C*D*AS*AS/5 )
            IF ( -HK .LT. 100 ) THEN
               B = SQRT(BS)
               BVN = BVN - EXP( -HK/2 )*SQRT(TWOPI)*PHID(-B/A)*B
     &                    *( 1 - C*BS*( 1 - D*BS/5 )/3 ) 
            ENDIF
            A = A/2
            DO I = 1, LG
               DO IS = -1, 1, 2
                  XS = ( A*(  IS*X(I,NG) + 1 ) )**2
                  RS = SQRT( 1 - XS )
                  ASR = -( BS/XS + HK )/2
                  IF ( ASR .GT. -100 ) THEN
                     BVN = BVN + A*W(I,NG)*EXP( ASR )
     &                    *( EXP( -HK*( 1 - RS )/( 2*( 1 + RS ) ) )/RS        
     &                    - ( 1 + C*XS*( 1 + D*XS ) ) )
                  END IF
               END DO
            END DO
            BVN = -BVN/TWOPI
         ENDIF
         IF ( R .GT. 0 ) THEN
            BVN =  BVN + PHID( -MAX( H, K ) )
         ELSE
            BVN = -BVN 
            IF ( K .GT. H ) BVN = BVN + PHID(K) - PHID(H) 
         ENDIF
      ENDIF
      BVND = BVN
      END

##' tables for data frames
##'
##' tables for data frames
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x name of variable, or fomula, or names of variables on data frame.
##' @param ... Optional additional arguments
##' @param level  1 for all marginal tables, 2 for all 2 by 2 tables, and null for the full table, possible versus group variable
##' @param response For level=2, only produce tables with columns given by 'response' (index)
##' @param flat  produce flat tables
##' @param total add total counts/proportions
##' @param prop Proportions instead of counts (vector of margins)
##' @param summary summary function
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##' data("sTRACE",package="timereg")
##'
##' dtable(sTRACE,~status)
##' dtable(sTRACE,~status+vf)
##' dtable(sTRACE,~status+vf,level=1)
##' dtable(sTRACE,~status+vf,~chf+diabetes)
##'
##' dtable(sTRACE,c("*f*","status"),~diabetes)
##' dtable(sTRACE,c("*f*","status"),~diabetes, level=2)
##' dtable(sTRACE,c("*f*","status"),level=1)
##'
##' dtable(sTRACE,~"*f*"+status,level=1)
##' dtable(sTRACE,~"*f*"+status+I(wmi>1.4)|age>60,level=2)
##' dtable(sTRACE,"*f*"+status~I(wmi>0.5)|age>60,level=1)
##' dtable(sTRACE,status~dcut(age))
##'
##' dtable(sTRACE,~status+vf+sex|age>60)
##' dtable(sTRACE,status+vf+sex~+1|age>60, level=2)
##' dtable(sTRACE,.~status+vf+sex|age>60,level=1)
##' dtable(sTRACE,status+vf+sex~diabetes|age>60)
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, flat=FALSE)
##'
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, level=1)
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2)
##'
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=1, total=TRUE)
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=2, total=TRUE)
##' dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=1:2, summary=summary)
##'
##' @aliases dtable dtab
##' @export
dtable <- function(data,y=NULL,x=NULL,...,level=-1,response=NULL,flat=TRUE,total=FALSE,prop=FALSE,summary=NULL) {
       	daggregate(data,y,x,...,
               fun=function(z) {
                   res <- sum <- c()
                   if (level==1 || ncol(z)==1) {
                       for (i in seq_len(ncol(z))) {
                           nn <- colnames(z)[i]
                           val <- table(z[,i],...)
			   if (prop[1]>0) val <- prop.table(val)
                           names(attr(val,"dimnames")) <- nn
                           val <- list(val)
                           names(val) <- nn
                           res <- c(res, val)
                           if (!is.null(summary)) {
                               sval <- list(do.call(summary,list(val[[1]])))
                               names(sval) <- nn
                               c(sum, sval)
                           }
                       }
                       res <- list(table=res,summary=sum,...)
                       class(res) <- "dtable"
                       return(res)
                   }
                   if (level>1) {
                       if (level>2) response <- ncol(z)
                       idx1 <- seq(1,ncol(z)-1)
                       if (level>2 || !is.null(response)) {
                           idx1 <- response
                           idx2 <- setdiff(seq(ncol(z)),idx1)
                       }
                       for (i in idx1)  {
                           if (!(level>2 || !is.null(response))) {
                               idx2 <- seq(i+1,ncol(z))
                           }
                           for (j in idx2) {
                               n1 <- colnames(z)[j]
                               n2 <- colnames(z)[i]
                               val <- table(z[,c(j,i)],...)
                               if (prop[1]>0) {
                                   if (all(1:2 %in% prop)) {
                                       val <- prop.table(val)
                                   } else {
                                       val <- prop.table(val,prop)
                                   }
                               }
                               if (total) {
                                   tot <- prop
                                   if (length(prop)==1) tot <- setdiff(1:2,prop)
                                       val <- addmargins(val,tot)
                               }
                               val <- list(val)
                               names(val) <- paste0(n1,", ",n2)
                               res <- c(res, val)
                               if (!is.null(summary)) {
                                   sval <- list(do.call(summary,list(val[[1]])))
                                   #names(sval) <- names(val)
                                   sum <- c(sum, sval)
                               }
                           }
                       }
                       res <- list(table=res,summary=sum)
                       class(res) <- "dtable"
                       return(res)
                   }

                   res <- table(z,...)
                   if (!is.null(summary)) {
                       sum <- do.call(summary,c(list(res),list(...)))
                   }
                   if (prop[1]>0) res <- prop.table(res,prop)
                   if (total>0) res <- addmargins(res,prop)
                   if (flat) res <- ftable(res,...)
                   res <- list(table=res,summary=sum)
                   class(res) <- "dtable"
                   return(res)
               })
}

##' @export
dtab <- function(data,y=NULL,x=NULL,...) dtable(data,y=NULL,x=NULL,...)

##' @export
print.dtable <- function(x,sep="\n",...) {
    cat(sep)
    if (inherits(x$table, c("table","ftable"))) {
        print(x$table)
        if (!is.null(x$summary)) print(x$summary)
        return(invisible(x))
    }

    for (i in seq_along(x$table)) {
        print(x$table[[i]],...)
        if (!is.null(x$summary))
            print(x$summary[[i]],...)
        cat(sep)
    }

}

## ----include=FALSE,echo=FALSE,message=FALSE,warning=FALSE---------------------
options(warn=-1, family="Times")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  #dev="png",
  #dpi=72,
  out.width = "70%")
library("mets")

## ----install, eval=FALSE, echo=FALSE------------------------------------------
#  # install.packages("remotes")
#  remotes::install_github("kkholst/mets", dependencies="Suggests")

## ----twinbmi------------------------------------------------------------------
library(mets)
data("twinbmi")
head(twinbmi)

## ----twinwide-----------------------------------------------------------------
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
head(twinwide)

## ----scatterdens, echo=FALSE,message=FALSE,warning=FALSE----------------------
library("cowplot")

scatterdens <- function(x) {
    require(ggplot2)
    sp <- ggplot(x,
                aes_string(colnames(x)[1], colnames(x)[2])) +
        theme_minimal() +
        geom_point(alpha=0.3) + geom_density_2d()
    xdens <- ggplot(x, aes_string(colnames(x)[1],fill=1)) +
        theme_minimal() +
        geom_density(alpha=.5)+
        theme(axis.text.x = element_blank(),
	      legend.position = "none") + labs(x=NULL)
    ydens <- ggplot(x, aes_string(colnames(x)[2],fill=1)) +
        theme_minimal() +
        geom_density(alpha=.5) +
        theme(axis.text.y = element_blank(),
	      axis.text.x = element_text(angle=90, vjust=0),
	      legend.position = "none") +
        labs(x=NULL) +
        coord_flip()
    g <- plot_grid(xdens,NULL,sp,ydens,
                  ncol=2,nrow=2,
                  rel_widths=c(4,1.4),rel_heights=c(1.4,4))
    return(g)
}

## ----scatter1, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in MZ twins"----
mz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
scatterdens(mz)

## ----scatter2, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in DZ twins"----
dz <- log(subset(twinwide, zyg=="DZ")[,c("bmi1","bmi2")])
scatterdens(dz)

## -----------------------------------------------------------------------------
cor.test(mz[,1],mz[,2], method="spearman")

## -----------------------------------------------------------------------------
cor.test(dz[,1],dz[,2], method="spearman")

## ----gee----------------------------------------------------------------------
l0 <- lm(bmi ~ gender + I(age-40), data=twinbmi)
estimate(l0, id=twinbmi$tvparnr)

## -----------------------------------------------------------------------------
library("splines")
l1 <- lm(bmi ~ gender*ns(age,3), data=twinbmi)
marg1 <- estimate(l1, id=twinbmi$tvparnr)

## ----marg1, warning=FALSE,message=FALSE,fig.cap="Marginal association between BMI and Age for males and females."----
dm <- Expand(twinbmi,
	    bmi=0,
	    gender=c("male"),
	    age=seq(33,61,length.out=50))
df <- Expand(twinbmi,
	    bmi=0,
	    gender=c("female"),
	    age=seq(33,61,length.out=50))

plot(marg1, function(p) model.matrix(l1,data=dm)%*%p,
     data=dm["age"], ylab="BMI", xlab="Age",
     ylim=c(22,26.5))
plot(marg1, function(p) model.matrix(l1,data=df)%*%p,
     data=df["age"], col="red", add=TRUE)
legend("bottomright", c("Male","Female"),
       col=c("black","red"), lty=1, bty="n")

## -----------------------------------------------------------------------------
dd <- na.omit(twinbmi)

## ----lmsat, eval=FALSE--------------------------------------------------------
#  l0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="sat")

## ----lmflex, eval=FALSE-------------------------------------------------------
#  lf <- twinlm(bmi ~ age+gender, data=dd,DZ="DZ", zyg="zyg", id="tvparnr", type="flex")

## ----lmeqmarg-----------------------------------------------------------------
lu <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="eqmarg")
estimate(lu)

## -----------------------------------------------------------------------------
estimate(lu,contr(5:6,6))

## ----ace----------------------------------------------------------------------
ace0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="ace")
summary(ace0)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)

n <- 400
kumar <- kumarsim(n,depcens=1)
kumar$cause <- kumar$status
kumar$ttt24 <- kumar[,6]
dtable(kumar,~cause)
dfactor(kumar) <- gp.f~gp

### censoring model must be adapted to size of data
###c2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
###	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr,preauto,ttt24))
###summary(c2)

c2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(c2)


c1 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
		treat.model=gp.f~dnr+preauto+ttt24,time=60)
summary(c1)

kumar$int <- interaction(kumar$gp,kumar$dnr)

b5 <- binregATE(Event(time,cause)~int+preauto+ttt24,kumar,cause=2,
		treat.model=int~preauto+ttt24,cens.code=0,time=60)
summary(b5)

## -----------------------------------------------------------------------------
kumar$cause2 <- 1*(kumar$cause==2)

b3 <- logitATE(cause2~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)

## calculate also relative risk
estimate(coef=b3$riskDR,vcov=b3$var.riskDR,f=function(p) p[1]/p[2])

## -----------------------------------------------------------------------------
b3 <- normalATE(time~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)

## -----------------------------------------------------------------------------
sessionInfo()

##' @export
print.summary.biprobit <- function(x,digits = max(3, getOption("digits") - 2),...) {
  cat("\n")
  printCoefmat(x$coef,digits=digits,...)
  S <- x$score;  names(S) <- rep("",length(S))
  cat("\nlogLik:", sum(x$logLik));
  cat("  mean(score^2):", formatC(mean(S^2),...), "\n");
  print(x$N,quote=FALSE)
  if (!is.null(x$msg)) {
      cat(x$msg,"\n")
  }

  for (i in seq(x$ncontrasts)) {
      corcontr <- x$model$zlen>1
      mcontr2 <- !x$model$eqmarg && x$model$blen>2
      mcontr1 <- x$model$eqmarg & x$model$blen>1
      if (corcontr || mcontr2 || mcontr1 || x$contrast) cat("\nContrast:\n")
      if (corcontr || x$contrast) {
          cat("\tDependence   ", x$par[[i]]$corref, "\n")
      }
      if (mcontr2 || (x$contrast & !x$model$eqmarg)) {
          cat("\tMean 1       ", x$par[[i]]$mref1, "\n")
          cat("\tMean 2       ", x$par[[i]]$mref2, "\n")
      } 
      if (mcontr1 || (x$contrast & x$model$eqmarg))
          cat("\tMean         ", x$par[[i]]$mref1, "\n")

      if (!is.null(x$varcomp)) {
          cat("\n")
          ##res <- x$varcomp
          res <- c()
          P <- x$prob[seq(x$nstat)+x$nstat*(i-1),,drop=FALSE]
          if (!is.null(P)) {
              res <- rbind(res,P)
          }
          idx <- unlist(sapply(c("Concordance","Marginal","P\\(Y"),function(x) grep(x,rownames(res))))
          idx2 <- setdiff(seq(nrow(res)),idx)
          res2 <- rbind(res[idx2,],rep(NA,ncol(res)),res[idx,])
          nn <- rownames(res2)
          rownames(res2) <- unlist(lapply(nn,function(x) gsub(paste("c",i,":",sep=""),"",x)))
          print(RoundMat(res2,digits=digits,na=FALSE),quote=FALSE)
      }

  }
  if (!is.null(x$time)) {
      cat("\n")
      cat("Event of interest before time ", x$time, "\n", sep="")
  }
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)

## -----------------------------------------------------------------------------
library(mets)
set.seed(1000) # to control output in simulatins for p-values below.

## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- simRecurrent(200,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))

## -----------------------------------------------------------------------------
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust.phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrentMarginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)

## -----------------------------------------------------------------------------
summary(out,times=c(1000,2000))

## -----------------------------------------------------------------------------
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust.phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

## -----------------------------------------------------------------------------
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust.phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)

## -----------------------------------------------------------------------------
rr <- simRecurrentList(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))

## -----------------------------------------------------------------------------
rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count.history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrentMarginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

times <- 500*(1:10)
###recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))

## -----------------------------------------------------------------------------
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::simGLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)

times <- seq(500,5000,by=500)
recEFF1x <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))

out <- recurrentMarginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)

## -----------------------------------------------------------------------------
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
summary(outs)$coef

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)

## -----------------------------------------------------------------------------
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
summary(outipcws)$coef

## -----------------------------------------------------------------------------
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef

## -----------------------------------------------------------------------------
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)

## -----------------------------------------------------------------------------
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])

## -----------------------------------------------------------------------------
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef

## -----------------------------------------------------------------------------
set.seed(100)
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
###
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
###
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
glout$gradient

## -----------------------------------------------------------------------------
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::simGLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

## -----------------------------------------------------------------------------
rr <- mets:::simGLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
estimate(recurrent)
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)

## -----------------------------------------------------------------------------
simcoxcox <- sim.recurrent(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim.recurrent(recurrentGL,margsurv,n=10,data=rr)

## -----------------------------------------------------------------------------
rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count.history(rr)
dtable(rr,~statusD)

oo <- prob.exceed.recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)

## -----------------------------------------------------------------------------
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))

## -----------------------------------------------------------------------------
rr <- simRecurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count.history(rr)
dtable(rr,~death+status)

## -----------------------------------------------------------------------------
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)

## ----eval=FALSE---------------------------------------------------------------
#  data(CPH_HPN_CRBSI)
#  dr <- CPH_HPN_CRBSI$terminal
#  base1 <- CPH_HPN_CRBSI$crbsi
#  base4 <- CPH_HPN_CRBSI$mechanical
# 
#   par(mfrow=c(1,3))
#   var.z <- c(0.5,0.5,0.5)
#   # death related to  both causes in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
#   rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count.history(rr,types=1:2)
# ###  cor(attr(rr,"z"))
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  plot(coo,main ="Scenario I")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # death related to  both causes in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count.history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main ="Scenario II")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # positive dependence for N1 and N2 all related in same way
#   cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count.history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main="Scenario III")

## ----eval=FALSE---------------------------------------------------------------
#   var.z <- c(0.5,0.5,0.5)
#   # negative dependence for N1 and N2 all related in same way
#   cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
#   rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
#   rr <- count.history(rr,types=1:2)
# ###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
# ###  par(mfrow=c(1,3))
# ###  plot(coo,main="Scenario IV")

## -----------------------------------------------------------------------------
sessionInfo()

##' Fast pattern
##'
##' @title Fast pattern
##' @param x Matrix (binary) of patterns. Optionally if \code{y} is
##' also passed as argument, then the pattern matrix is defined as the
##' elements agreeing in the two matrices.
##' @param y Optional matrix argument with same dimensions as \code{x} (see above)
##' @param categories Default 2 (binary)
##' @param ... Optional additional arguments
##' @author Klaus K. Holst
##' @export
##' @examples
##' X <- matrix(rbinom(100,1,0.5),ncol=4)
##' fast.pattern(X)
##'
##' X <- matrix(rbinom(100,3,0.5),ncol=4)
##' fast.pattern(X,categories=4)
fast.pattern <- function(x,y,categories=2,...) {
  if (missing(y)) y <- NULL
   .Call("FastPattern",as.matrix(x),y,as.integer(categories),PACKAGE="mets")
}
 

##' Efficient IPCW for binary data 
##'
##' Simple version of comp.risk function of timereg for just one time-point thus fitting the model 
##' \deqn{E(T \leq t | X ) = expit( X^T beta) }
##'
##' Based on binomial regresion IPCW response estimating equation: 
##' \deqn{ X  ( \Delta (T \leq t)/G_c(T_i-) - expit( X^T beta)) = 0 }
##' for IPCW adjusted responses. 
##'
##' Based on binomial regresion IPCW response estimating equation: 
##' \deqn{ h(X) X ( \Delta (T \leq t)/G_c(T_i-) - expit( X^T beta)) = 0 }
##' for IPCW adjusted responses where $h$ is given as an argument together with iid of censoring with h. 
##' By using appropriately the h argument we can also do the efficient IPCW estimator estimator this works 
##' the prepsurv and prepcif for survival or competing risks data. In this case also the censoring martingale 
##' should be given for variance calculation and this also comes out of the prepsurv or prepcif functions. 
##' (Experimental version at this stage).
##' 
##' Variance is based on  \deqn{ \sum w_i^2 } also with IPCW adjustment, and naive.var is variance 
##' under known censoring model. 
##'
##' Censoring model may depend on strata. 
##'
##' @param formula formula with outcome (see \code{coxph})
##' @param data data frame
##' @param cause cause of interest
##' @param time  time of interest 
##' @param beta starting values 
##' @param offset offsets for partial likelihood 
##' @param weights for score equations 
##' @param cens.weights censoring weights 
##' @param cens.model only stratified cox model without covariates
##' @param se to compute se's  based on IPCW 
##' @param kaplan.meier uses Kaplan-Meier for IPCW in contrast to exp(-Baseline)
##' @param cens.code gives censoring code
##' @param no.opt to not optimize 
##' @param method for optimization 
##' @param model exp or linear 
##' @param augmentation to augment binomial regression 
##' @param h  h for estimating equation 
##' @param MCaugment iid of h and censoring model 
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @export
Effbinreg <- function(formula,data,cause=1,time=NULL,beta=NULL,
   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",
   augmentation=NULL,h=NULL,MCaugment=NULL,...)
{# {{{
  cl <- match.call()# {{{
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    stop("only right censored data, will not work for delayed entry\n"); 
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}
  if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
    ts <- survival::untangle.specials(Terms, "offset")
    Terms  <- Terms[-ts$terms]
    offset <- m[[ts$vars]]
  }  
  X <- model.matrix(Terms, m)
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  if (!is.null(id)) {
          orig.id <- id
	  ids <- sort(unique(id))
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
   } else { orig.id <- NULL; nid <- nrow(X); id <- as.integer(seq_along(exit))-1; ids <- NULL}
  ### id from call coded as numeric 1 -> 
  id.orig <- id; 

  if (is.null(offset)) offset <- rep(0,length(exit)) 
  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status==cens.code) 
  statusE <- (status==cause) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  statusC <- (status==cens.code) 
  data$id <- id
  data$exit <- exit
  data$statusC <- statusC 

  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL

  if (is.null(beta)) beta <- rep(0,ncol(X))
  p <- ncol(X)

  X <-  as.matrix(X)
  X2  <- .Call("vecMatMat",X,X)$vXZ
  Y <- c((status==cause)*(exit<=time)/cens.weights)
  nevent <- sum((status==cause)*(exit<=time))

  if (is.null(augmentation))  augmentation=rep(0,p)
  if (is.null(h))  h <- rep(1,length(exit))

obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X %*% pp+offset)
p <- expit(lp)
ploglik <- sum(weights*(Y-p)^2)

Dlogl <- weights*h*X*c(Y-p)
D2logl <- c(weights*h*p/(1+exp(lp)))*X2
D2log <- apply(D2logl,2,sum)
gradient <- apply(Dlogl,2,sum)+augmentation
hessian <- matrix(D2log,length(pp),length(pp))

  if (all) {
      ihess <- solve(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      if (!se) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    model.frame=m,n=length(exit),nevent=nevent,ncluster=nid))

 if (!is.null(MCaugment)) { se <- FALSE;}

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    lp <- c(X %*% val$coef+offset)
    p <- expit(lp)
    Y <- c((status==cause)*weights*(exit<=time)/cens.weights)
    h <- h[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    ht  <-  apply(h*X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply(ht*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*ht[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
 
   }  else {
	  MGCiid <- 0
  }## }}}

  if (!is.null(MCaugment)) { MGCiid <- MCaugment*h*X}
  val$MGciid <- MGCiid
  val$MGtid <- id
  val$orig.id <- orig.id
  val$iid.origid <- ids 
  val$iid.naive <- val$iid 
  val$iid  <- beta.iid <- val$iid+(MGCiid %*% val$ihessian)
  beta.iid <- apply(beta.iid,2,sumstrata,id,max(id)+1)
  val$iid <- beta.iid
  val$naive.var <- val$var
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5

  class(val) <- "binreg"
  return(val)
}# }}}

prepsurv <- function(cs,ss,X,times,data) 
{# {{{
ttimes <- ss$cumhaz[,1]
ttimes <- ttimes[(ttimes< times)]
pcs <- predict(cs,data,se=FALSE,times=ttimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
###
## survival model 
ps <- predict(ss,data,se=FALSE,times=ttimes)
dLamt <- apply(cbind(0,ps$cumhaz),1,diff)
pst0 <- c(tail(t(ps$surv),n=1))
Fsst0 <- 1-pst0
###
L <- apply((1/t(ps$surv*pcs$surv))*dLamt,2,sum)
L0 <- (1/pst0-1)

ctimes <- cs$cumhaz[,1]
ctimes <- ctimes[(ctimes< times)]
pcs <- predict(cs,data,se=FALSE,times=ctimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
dLamc <- apply(cbind(0,pcs$cumhaz),1,diff)

###cctimes <- cs$cumhaz[,1]
###Lamcc <- cs$cumhaz[,2] %o%  exp(X[,2] * coef(cs)) 
###dLamcc <- diff(c(0,cs$cumhaz[cctimes<times,2])) %o%  exp(X[,2] * coef(cs)) 
###Lamcc[cctimes<times,]- t(pcs$cumhaz)
###matplot(ctimes, t(exp(-pcs$cumhaz)),type="s",lwd=0.5)

## survival model 
ps <- predict(ss,data,se=FALSE,times=ctimes)
pst0 <- c(tail(t(ps$surv),n=1))
Fsst0 <- 1-pst0
###L <- 1/(pst0*pcst0) - 1 - apply( (1/t(ps$surv*pcs$surv))*dLamc,2,sum)
h <- Fsst0/(pst0*L)

cmtimes <- matrix(ctimes,nrow(dLamc),ncol(dLamc))
tttimes <- matrix(pmin(data[,"time"],times),nrow(dLamc),ncol(dLamc),byrow=TRUE)
## augment
dF <- t(Fsst0-((1-ps$surv)))
Augmentf <- dF/t(ps$surv*pcs$surv)
AugmentC <- apply(Augmentf*dLamc*(cmtimes<tttimes),2,sum)
###
n <- nrow(data)
AdN <- rep(0,n)
jc <- (1:nrow(data))[cs$ord][cs$jumps]
jc <-jc[1:length(ctimes)] 
AdN[jc] <- mdi(Augmentf,1:length(ctimes),jc)
Augment <- AdN-AugmentC

saugment <- apply(X*Augment,2,sum)
augment <- apply(X*Augment*h,2,sum)
return(list(L=L,L0=L0,Mc=Augment,AugmentC=AugmentC,Xaugment=saugment,hXaugment=augment,h=h))
}# }}}

prepcif <- function(cs,ps,cif1,X,times,data) 
{# {{{

ctimes <- cs$cumhaz[,1]
ctimes <- ctimes[(ctimes< times)]
pcif1 <- predict(cif1,data,se=FALSE,times=ctimes)
pcift0 <- c(tail(t(pcif1$cif),n=1))
###
ctimes <- cs$cumhaz[,1]
ctimes <- ctimes[(ctimes< times)]
pcs <- predict(cs,data,se=FALSE,times=ctimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
###
pps <- predict(ps,data,se=FALSE,times=ctimes)
dim(pps$surv)
pst0 <- c(tail(t(pps$surv),n=1))
psst0 <- 1-pst0
###
dLamc <- apply(cbind(0,pcs$cumhaz),1,diff)
Ia <- (1-pcift0)* ( pcift0/pcst0 - apply( t(pcif1$cif/pcs$surv)*dLamc,2,sum) )

times1 <- cif1$cumhaz[,1]
times1 <- times1[times1<times]
ppcs <- predict(cs,data,se=FALSE,times=times1)
ppcif1 <- predict(cif1,data,se=FALSE,times=times1)
dF1 <- apply(cbind(0,ppcif1$cif),1,diff)
I <- (1-pcift0)* apply(1/t(ppcs$surv)*dF1,2,sum)
###
ddF1 <-  t(((pcift0)-((pcif1$cif))))
II <- apply(ddF1^2/(t(pps$surv*pcs$surv))*dLamc,2,sum)-pcift0*apply(ddF1/(t(pcs$surv))*dLamc,2,sum) 
###
h <- pcift0*(1-pcift0)/(I-II) 
hc <- pcift0*(1-pcift0)/(Ia-II) 
###
cmtimes <- matrix(ctimes,nrow(dLamc),ncol(dLamc))
tttimes <- matrix(pmin(data[,"time"],times),nrow(dLamc),ncol(dLamc),byrow=TRUE)
med <- cmtimes<tttimes
## augment
dF <- t(((pcift0)-((pcif1$cif))))
Augmentf <- dF/t(pps$surv*pcs$surv)
AugmentC <- apply(Augmentf*dLamc*med,2,sum)
###
n <- nrow(data)
AdN <- rep(0,n)
dc <- length(ctimes)
jc <- (1:nrow(data))[cs$ord][cs$jumps]
jc <-jc[1:dc] 
AdN[jc] <- mdi(Augmentf,1:dc,jc)
Augment <- AdN- AugmentC
###
###X <- model.matrix(~formula,data)
saugment <- apply(X*Augment,2,sum)
augment <- apply(X*Augment*h,2,sum)

return(list(Mc=Augment,AugmentC=AugmentC,Xaugment=saugment,hXaugment=augment,h=h,hc=hc))
}# }}}

##' Liability-threshold model for twin data
##'
##' @aliases bptwin twinlm.time bptwin.time
##' @title Liability model for twin data
##' @seealso \code{\link{twinlm}}, \code{\link{twinlm.time}}, \code{\link{twinlm.strata}}, \code{\link{twinsim}}
##' @param x Formula specifying effects of covariates on the response.
##' @param data \code{data.frame} with one observation pr row. In
##'     addition a column with the zygosity (DZ or MZ given as a factor) of
##'     each individual much be
##'     specified as well as a twin id variable giving a unique pair of
##'     numbers/factors to each twin pair.
##' @param id The name of the column in the dataset containing the twin-id variable.
##' @param zyg The name of the column in the dataset containing the
##'     zygosity variable.
##' @param DZ Character defining the level in the zyg variable
##' corresponding to the dyzogitic twins. 
##' @param group Optional. Variable name defining group for interaction analysis (e.g., gender)
##' @param num Optional twin number variable
##' @param weights Weight matrix if needed by the chosen estimator (IPCW)
##' @param weights.fun Function defining a single weight each individual/cluster
##' @param strata Strata
##' @param messages Control amount of messages shown 
##' @param control Control argument parsed on to the optimization routine. Starting values may be parsed as '\code{start}'.
##' @param type Character defining the type of analysis to be
##'     performed. Should be a subset of "acde" (additive genetic factors, common
##'     environmental factors, dominant
##'     genetic factors, unique environmental factors).
##' @param eqmean Equal means (with type="cor")?
##' @param pairs.only Include complete pairs only?
##' @param stderr Should standard errors be calculated?
##' @param robustvar If TRUE robust (sandwich) variance estimates of the variance are used
##' @param p Parameter vector p in which to evaluate log-Likelihood and score function
##' @param indiv If TRUE the score and log-Likelihood contribution of each twin-pair
##' @param constrain Development argument
##' @param samecens Same censoring
##' @param allmarg Should all marginal terms be included
##' @param varlink Link function for variance parameters
##' @param ... Additional arguments to lower level functions
##' @author Klaus K. Holst
##' @export
##' @examples
##' data(twinstut)
##' b0 <- bptwin(stutter~sex,
##'              data=droplevels(subset(twinstut,zyg%in%c("mz","dz"))),
##'              id="tvparnr",zyg="zyg",DZ="dz",type="ae")
##' summary(b0)
bptwin <- function(x, data, id, zyg, DZ, group=NULL,
                   num=NULL,
                   weights=NULL,
                   weights.fun=function(x) ifelse(any(x<=0), 0, max(x)),
                   strata=NULL,
                   messages=1,
                   control=list(trace=0),
                   type="ace",
                   eqmean=TRUE,
                   pairs.only=FALSE,
                   samecens=TRUE,
                   allmarg=samecens&!is.null(weights),
                   stderr=TRUE,                  
                   robustvar=TRUE,
                   p, indiv=FALSE,
                   constrain,
                   varlink,
                   ...) {
    
###{{{ setup

  mycall <- match.call()
  formulaId <- unlist(Specials(x,"cluster"))
  formulaStrata <- unlist(Specials(x,"strata"))
  formulaSt <- paste("~.-cluster(",formulaId,")-strata(",paste(formulaStrata,collapse="+"),")")
  formula <- update(x,formulaSt)
  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) strata <- formulaStrata
  mycall$formula <- formula
 
  if (!is.null(strata)) {
    dd <- split(data,interaction(data[,strata]))
    nn <- unlist(lapply(dd,nrow))
    dd[which(nn==0)] <- NULL
    if (length(dd)>1) {
      fit <- lapply(seq(length(dd)),function(i) {
        if (messages>0) message("Strata '",names(dd)[i],"'")
        mycall$data <- dd[[i]]
        eval(mycall)
      })
      res <- list(model=fit)
      res$strata <- names(res$model) <- names(dd)
      class(res) <- c("twinlm.strata","biprobit")
      res$coef <- unlist(lapply(res$model,coef))
      res$vcov <- blockdiag(lapply(res$model,vcov.biprobit))
      res$N <- length(dd)
      res$idx <- seq(length(coef(res$model[[1]])))
      rownames(res$vcov) <- colnames(res$vcov) <- names(res$coef)
      return(res)
    }
  }

##################################################
### No strata
  if (is.null(control$method)) {
      if (!samecens & !is.null(weights)) {
        control$method <- "bhhh"
      } else {
        if (requireNamespace("ucminf",quietly=TRUE)) {
          control$method <- "gradient"
        } else control$method <- "nlminb"
      }
  }
  if (tolower(type)=="cor") type <- "u"
  if (length(grep("flex",tolower(type)))>0) { type <- "u"; eqmean <- FALSE }

  yvar <- paste(deparse(formula[[2]]),collapse="")
  data <- data[order(data[,id]),]
  idtab <- table(data[,id])
  if (sum(idtab>2)) stop("More than two individuals with the same id ")

  if (pairs.only) {
    data <- data[as.character(data[,id])%in%names(idtab)[idtab==2],]
    idtab <- table(data[,id])
  }
  if (is.logical(data[,yvar])) data[,yvar] <- data[,yvar]*1
  if (is.factor(data[,yvar])) data[,yvar] <- as.numeric(data[,yvar])-1  

  idx2 <- NULL
  if (missing(DZ)) {
    DZ <- levels(as.factor(data[,zyg]))[1]
    message("Using '",DZ,"' as DZ",sep="")
  }
  OS <- NULL
  OSon <- FALSE
  if (!is.null(OS)) {
    idx2 <- which(data[,zyg]==OS)
    OSon <- TRUE
    if (length(idx2)==0) {
      warning("No OS twins found")
      OSon <- FALSE
    }
  }
  idx1 <- which(data[,zyg]==DZ) ## DZ
  if (length(idx1)==0) stop("No DZ twins found")
  idx0 <- which(!(data[,zyg]%in%c(DZ,OS))) ## MZ
  if (length(idx1)==0) stop("No MZ twins found")
  zyg2 <- rep(1,nrow(data)); zyg2[idx0] <- 0; zyg2[idx2] <- 2
  data[,zyg] <- zyg2 ## MZ=0, DZ=1, OS=2

  
  opt <- options(na.action="na.pass")
  Data <- cbind(model.matrix(formula,data),data[,c(yvar,id,zyg,weights,num)])
  options(opt)

  rnames1 <- setdiff(colnames(Data),c(yvar,id,weights,zyg,num))
  nx <- length(rnames1) 
  if (nx==0) stop("Zero design not allowed")

  bidx0 <- seq(nx)
  midx0 <- bidx0; midx1 <- midx0+nx
  dS0. <- rbind(rep(1,4),rep(1,4),rep(1,4)) ## MZ
  dS1. <- rbind(c(1,.5,.5,1),rep(1,4),c(1,.25,.25,1)) ## DZ
  dS2.  <- rbind(c(1,0,0,1),rep(1,4),c(1,0,0,1),c(0,1,1,0))
  
  ##mytr <- function(x) x; dmytr <- function(x) 1
  ##mytr <- function(x) x^2; dmytr <- function(x) 2*x
  ##mytr <- function(z) 1/(1+exp(-z)); dmytr <- function(z) exp(-z)/(1+exp(-z))^2
  ACDU <- sapply(c("a","c","d","e","u"),function(x) length(grep(x,tolower(type)))>0)
  ACDU <- c(ACDU,os=OSon)

  if (missing(varlink) || (!is.null(varlink) && varlink%in%"log")) {
      mytr <- exp; dmytr <- exp; myinvtr <- log
      trname <- "exp"; invtrname <- "log"
  } else {
      mytr <- myinvtr <- identity; dmytr <- function(x) rep(1,length(x))
      trname <- ""; invtrname <- ""
  }
  
  dmytr2 <- function(z) 4*exp(2*z)/(exp(2*z)+1)^2
  mytr2 <- tanh;  myinvtr2 <- atanh
  trname2 <- "tanh"; invtrname2 <- "atanh"

  if (OSon) {
    ## logit <- function(p) log(p/(1-p))
    ## tigol <- function(z) 1/(1+exp(-z))
    ## dlogit <- function(p) 1/(p*(1-p))
    ## dtigol <- function(z) tigol(z)^2*exp(-z)    
    ## mytr <- function(p) c(exp(p[-length(p)]),tigol(p[length(p)]))
    ## myinvtr <- function(z) c(log(z[-length(z)]),logit(z[length(z)]))
    ## dmytr <- function(p) c(exp(p[-length(p)]),dtigol(p[length(p)]))
    mytr <- function(x) c(exp(x[-length(x)]),mytr2(x[length(x)]))
    myinvtr <- function(z) c(log(z[-length(z)]),myinvtr2(z[length(z)]))
    dmytr <- function(x) c(exp(x[-length(x)]),dmytr2(x[length(x)]))
  }

  if (ACDU["u"]) {
    ##      datanh <- function(r) 1/(1-r^2)
    dmytr <- dmytr2
    mytr <- mytr2;  myinvtr <- myinvtr2
    trname <- trname2; invtrname <- invtrname2
    dS0 <- rbind(c(0,1,1,0))
    vidx0 <- 1
    vidx1 <- 2
    vidx2 <- 3
    dS2 <- dS1 <- dS0
    nvar <- length(vidx0)+length(vidx1)
    if (OSon) nvar <- nvar+length(vidx2)
  } else {
    nvar <- sum(ACDU[1:3])
    vidx0 <- vidx1 <- seq(nvar); vidx2 <- seq(nvar+1)
    if (OSon) nvar <- nvar+1
    dS0 <- dS0.[ACDU[1:3],,drop=FALSE]
    dS1 <- dS1.[ACDU[1:3],,drop=FALSE]
    dS2 <- dS2.[which(c(ACDU[1:3],TRUE)),,drop=FALSE]
  }  
  if (eqmean) {
    bidx2 <- bidx1 <- bidx0
  } else {
    bidx1 <- bidx0+nx
    bidx2 <- bidx1+nx    
    if (OSon) nx <- 3*nx else nx <- 2*nx;
  }
  
  vidx0 <- vidx0+nx; vidx1 <- vidx1+nx; vidx2 <- vidx2+nx
  vidx <- nx+seq_len(nvar)
  midx <- seq_len(nx)
  plen <- nx+nvar

  Am <- matrix(c(1,.5,.5,1),ncol=2)
  Dm <- matrix(c(1,.25,.25,1),ncol=2)
  Vm <- matrix(c(1,0,0,1),ncol=2)
  Rm <- matrix(c(0,1,1,0),ncol=2)

##################################################

  ## system.time(Wide <- reshape(as.data.frame(Data),idvar=c(id,zyg),timevar=time,direction="wide"))
  ##  system.time(Wide <- as.data.frame(fast.reshape(Data,id=c(id),sep=".")))
  Wide <- as.data.frame(fast.reshape(Data,id=c(id,zyg),sep=".",idcombine=FALSE,labelnum=TRUE))
  yidx <- paste(yvar,1:2,sep=".")
  rmidx <- c(id,yidx,zyg)
    
  W0 <- W1 <- W2 <- NULL
  if (!is.null(weights)) {
    widx <- paste(weights,1:2,sep=".")
    rmidx <- c(rmidx,widx)
    W0 <- as.matrix(Wide[which(Wide[,zyg]==0),widx,drop=FALSE])
    W1 <- as.matrix(Wide[which(Wide[,zyg]==1),widx,drop=FALSE])
    W2 <- as.matrix(Wide[which(Wide[,zyg]==2),widx,drop=FALSE])
  }
  XX <- as.matrix(Wide[,setdiff(colnames(Wide),rmidx)])
  XX[is.na(XX)] <- 0

  Y0 <- as.matrix(Wide[which(Wide[,zyg]==0),yidx,drop=FALSE])
  Y1 <- as.matrix(Wide[which(Wide[,zyg]==1),yidx,drop=FALSE])
  Y2 <- as.matrix(Wide[which(Wide[,zyg]==2),yidx,drop=FALSE])
  XX0 <- XX[which(Wide[,zyg]==0),,drop=FALSE]
  XX1 <- XX[which(Wide[,zyg]==1),,drop=FALSE]
  XX2 <- XX[which(Wide[,zyg]==2),,drop=FALSE]
  
##################################################

###}}} setup

###{{{ Mean/Var function

  ##Marginals etc.
  MyData0 <- ExMarg(Y0,XX0,W0,dS0,eqmarg=TRUE,allmarg=allmarg)
  MyData1 <- ExMarg(Y1,XX1,W1,dS1,eqmarg=TRUE,allmarg=allmarg)
  MyData2 <- ExMarg(Y2,XX2,W2,dS2,eqmarg=TRUE,allmarg=allmarg)

  N <- cbind(length(idx0),length(idx1),length(idx2)); 
  N <- cbind(N,
             2*nrow(MyData0$Y0)+if (!pairs.only) NROW(MyData0$Y0_marg) else 0, 
             2*nrow(MyData1$Y0)+if (!pairs.only) NROW(MyData1$Y0_marg) else 0,
             2*nrow(MyData2$Y0)+if (!pairs.only) NROW(MyData2$Y0_marg) else 0,
             NROW(MyData0$Y0),NROW(MyData1$Y0),NROW(MyData2$Y0))

  
  colnames(N) <- c("Total.MZ","Total.DZ","Total.OS","Complete.MZ","Complete.DZ","Complete.OS","Complete pairs.MZ","Complete pairs.DZ","Complete pairs.OS")
  rownames(N) <- rep("",nrow(N))
  if (!OSon) N <- N[,-c(3,6,9),drop=FALSE]
  
  if (samecens & !is.null(weights)) {
    MyData0$W0 <- cbind(apply(MyData0$W0,1,weights.fun))
    if (!is.null(MyData0$Y0_marg))
      MyData0$W0_marg <- cbind(apply(MyData0$W0_marg,1,weights.fun))

    MyData1$W0 <- cbind(apply(MyData1$W0,1,weights.fun))
    if (!is.null(MyData1$Y0_marg))
        MyData1$W0_marg <- cbind(apply(MyData1$W0_marg,1,weights.fun))

    MyData2$W0 <- cbind(apply(MyData2$W0,1,weights.fun))
    if (!is.null(MyData2$Y0_marg))
        MyData2$W0_marg <- cbind(apply(MyData2$W0_marg,1,weights.fun))
  }

  rm(Y0,XX0,W0,Y1,XX1,W1,Y2,XX2,W2)
  
  Sigma <- function(p0) {
    Sigma2 <- NULL
    p0[vidx] <- mytr(p0[vidx])    
    if (ACDU["u"]) {
      pos0 <- ifelse(OSon, plen-2, plen-1)
      Sigma0 <- diag(2) + p0[pos0]*Rm
      Sigma1 <- diag(2) + p0[pos0+1]*Rm
      if (OSon) Sigma2 <- diag(2) + p0[pos0+2]*Rm
    } else {
      ii <- ACDU; ii[4:5] <- FALSE
      pv <- ACDU*1;  pv[ii] <- p0[vidx]
      Sigma0 <- Vm*pv["e"] + pv["a"] + pv["c"] + pv["d"]
      Sigma1 <- Vm*pv["e"] + pv["a"]*Am + pv["c"] + pv["d"]*Dm
      Sigma2 <- Vm*pv["e"] + pv["c"] + (pv["a"] + pv["d"])*Vm +
        pv["os"]*(pv["a"] + pv["d"])*Rm
      if (OSon) {
        dS2 <- dS2.
        dS2[c(1,3),2:3] <- pv["os"]
        dS2[4,2:3] <- pv["a"]+pv["d"]
        dS2 <- dS2[which(c(ACDU[1:3],TRUE)),]
      }
    }
    return(list(Sigma0=Sigma0,Sigma1=Sigma1,Sigma2=Sigma2,dS2=dS2))
  }

  env <- new.env(parent=baseenv())
  #environment(mytr) <- baseenv()
  for (v in c("ACDU","mytr","vidx","OSon","plen",
              "Rm","Am","Vm","Dm","dS2.","dS2")) {
    assign(v, get(v), envir=env)
  }
  environment(Sigma) <- env

###}}} Mean/Var function
  
###{{{ U  

  p0 <- rep(-1,plen); ##p0[vidx] <- 0
  if (!missing(varlink) && is.null(varlink)) p0 <- rep(0.5,plen)
  if (OSon) p0[length(p0)] <- 0.3
  if (type=="u")
    p0[vidx] <- 0.3
  if (!is.null(control$start)) {
    p0 <- control$start
    control$start <- NULL
  } else {
    X <- rbind(MyData0$XX0[,midx0,drop=FALSE],MyData0$XX0[,midx1,drop=FALSE])
    Y <- rbind(MyData0$Y0[,1,drop=FALSE],MyData0$Y0[,2,drop=FALSE])
    g <- suppressWarnings(glm(Y~-1+X,family=binomial(probit)))
    p0[midx] <- coef(g)
  }
  
  U <- function(p,indiv=FALSE) {
    b0 <- cbind(p[bidx0])
    b1 <- cbind(p[bidx1])
    b2 <- cbind(p[bidx2])
    b00 <- b0; b11 <- b1; b22 <- b2
    ## if (bound) p[vidx] <- min(p[vidx],20)
    S <- Sigma(p)
    lambda <- eigen(S$Sigma0)$values
    if (any(lambda<1e-12 | lambda>1e9)) stop("Variance matrix out of bounds")
    
    Mu0 <- with(MyData0, cbind(XX0[,midx0,drop=FALSE]%*%b00,
                               XX0[,midx1,drop=FALSE]%*%b00))
    U0 <- with(MyData0, .Call("biprobit0",
                             Mu0,
                             S$Sigma0,dS0,Y0,XX0,W0,!is.null(W0),samecens,
                             PACKAGE="mets"))

    if (!is.null(MyData0$Y0_marg) &&!pairs.only) {
      mum <- with(MyData0, XX0_marg%*%b00)
      dSmarg <- dS0[,1,drop=FALSE]
       U_marg <- with(MyData0, .Call("uniprobit",
                                   mum,XX0_marg,
                                   S$Sigma0[1,1],t(dSmarg),Y0_marg,
                                   W0_marg,!is.null(W0_marg),TRUE,
                                   PACKAGE="mets"))
      U0$score <- rbind(U0$score,U_marg$score)
      U0$loglik <- c(U0$loglik,U_marg$loglik)
    }

    Mu1 <- with(MyData1, cbind(XX0[,midx0,drop=FALSE]%*%b11,
                               XX0[,midx1,drop=FALSE]%*%b11))

    U1 <- with(MyData1, .Call("biprobit0",
                             Mu1,
                             S$Sigma1,dS1,Y0,XX0,W0,!is.null(W0),samecens,
                             PACKAGE="mets"))
    if (!is.null(MyData1$Y0_marg) &&!pairs.only) {
      mum <- with(MyData1, XX0_marg%*%b11)
      dSmarg <- dS1[,1,drop=FALSE]
      U_marg <- with(MyData1, .Call("uniprobit",
                                   mum,XX0_marg,
                                   S$Sigma1[1,1],t(dSmarg),Y0_marg,
                                   W0_marg,!is.null(W0_marg),TRUE,
                                   PACKAGE="mets"))
      U1$score <- rbind(U1$score,U_marg$score)
      U1$loglik <- c(U1$loglik,U_marg$loglik)
    }

    U2 <- val2 <- NULL
    if (OSon) {
      Mu2 <- with(MyData2, cbind(XX0[,midx0,drop=FALSE]%*%b22,
                                 XX0[,midx1,drop=FALSE]%*%b22))
      U2 <- with(MyData2, .Call("biprobit0",
                               Mu2,
                               S$Sigma2,S$dS2,Y0,XX0,W0,!is.null(W0),samecens,
                               PACKAGE="mets"))
      if (!is.null(MyData2$Y0_marg) &&!pairs.only) {
        mum <- with(MyData2, XX0_marg%*%b22)
        dSmarg <- S$dS2[,1,drop=FALSE]
        U_marg <- with(MyData2, .Call("uniprobit",
                                     mum,XX0_marg,
                                     S$Sigma2[1,1],t(dSmarg),Y0_marg,
                                     W0_marg,!is.null(W0_marg),TRUE,
                                     PACKAGE="mets"))
        U2$score <- rbind(U2$score,U_marg$score)
        U2$loglik <- c(U2$loglik,U_marg$loglik)
      }
    }

    
    if (indiv) {
      ll0 <- U0$loglik
      ll1 <- U1$loglik
      val0 <- U0$score[MyData0$id,,drop=FALSE]
      val1 <- U1$score[MyData1$id,,drop=FALSE]
      
      N0 <- length(MyData0$id)
      idxs0 <- seq_len(N0)
      if (length(MyData0$margidx)>0) {
        for (i in seq_len(N0)) {
          idx0 <- which((MyData0$idmarg)==(MyData0$id[i]))+N0
          idxs0 <- c(idxs0,idx0)
          val0[i,] <- val0[i,]+colSums(U0$score[idx0,,drop=FALSE])
        }
        val0 <- rbind(val0, U0$score[-idxs0,,drop=FALSE])
        ll0 <- c(ll0,ll0[-idxs0])
      }
      
      N1 <- length(MyData1$id)
      idxs1 <- seq_len(N1)
      if (length(MyData1$margidx)>0) {
        for (i in seq_len(N1)) {
          idx1 <- which((MyData1$idmarg)==(MyData1$id[i]))+N1
          idxs1 <- c(idxs1,idx1)
          val1[i,] <- val1[i,]+colSums(U1$score[idx1,,drop=FALSE])
        }
        val1 <- rbind(val1, U1$score[-idxs1,,drop=FALSE])
        ll1 <- c(ll1,ll1[-idxs1])        
      }

      if (OSon) {
        ll2 <- U2$loglik
        val2 <- U2$score[MyData2$id,,drop=FALSE]
        N2 <- length(MyData2$id)
        idxs2 <- seq_len(N2)
        if (length(MyData2$margidx)>0) {
          for (i in seq_len(N2)) {
            idx2 <- which((MyData2$idmarg)==(MyData2$id[i]))+N2
            idxs2 <- c(idxs2,idx2)
            val2[i,] <- val2[i,]+colSums(U2$score[idx2,,drop=FALSE])
          }
          val2 <- rbind(val2, U2$score[-idxs2,,drop=FALSE])
          ll2 <- c(ll2,ll2[-idxs2])        
        }
      } 
      
      val <- matrix(0,ncol=plen,nrow=nrow(val0)+nrow(val1) + NROW(val2))
      val[seq_len(nrow(val0)),c(bidx0,vidx0)] <- val0
      val[nrow(val0)+seq_len(nrow(val1)),c(bidx1,vidx1)] <- val1
      if (OSon) {
        val[nrow(val0)+nrow(val1)+seq_len(nrow(val2)),c(bidx2,vidx2)] <- val2
      }
      trp <- dmytr(p[vidx])
      for (i in seq(length(vidx))) {
        val[,vidx[i]] <- val[,vidx[i]]*trp[i]
      }
      attributes(val)$logLik <- c(U0$loglik,U1$loglik,U2$loglik)
      return(val)
      
    }
    val <- numeric(plen)
    val[c(bidx0,vidx0)] <- colSums(U0$score)
    val[c(bidx1,vidx1)] <- val[c(bidx1,vidx1)]+colSums(U1$score)
    if (OSon) val[c(bidx2,vidx2)] <- val[c(bidx2,vidx2)]+colSums(U2$score)
    val[vidx] <- val[vidx]*dmytr(p[vidx])
    attributes(val)$logLik <- sum(U0$loglik)+sum(U1$loglik)+sum(U2$loglik)
    return(val)
  }

###}}} U

###{{{ optim

  if (!missing(p)) return(U(p,indiv=indiv))


  f <- function(p) crossprod(U(p))[1]
  f0 <- function(p) -sum(attributes(U(p))$logLik)
  g0 <- function(p) -as.numeric(U(p))
  h0 <- function(p) crossprod(U(p,indiv=TRUE))

  
  if (!missing(constrain)) {
    freeidx <- is.na(constrain)
    f <- function(p) {      
      p1 <- constrain; p1[freeidx] <- p
      res <- U(p1)[freeidx]
      crossprod(res)[1]
    }
    f0 <- function(p) {
      p1 <- constrain; p1[freeidx] <- p
      -sum(attributes(U(p1))$logLik)
    }
    g0 <- function(p) {
      p1 <- constrain; p1[freeidx] <- p
      -as.numeric(U(p1)[freeidx])
    }
    p0 <- p0[is.na(constrain)]
  }


  ## Derivatives, Sanity check 
  ## ff <- function(p) attributes(U(p,indiv=FALSE))$logLik
  ## pp <- c(0,-.1,.1,0.5)
  ## numDeriv::grad(ff,pp)
  ## U(pp,indiv=FALSE)

  
  controlstd <- list(hessian=0)
  controlstd[names(control)] <- control
  control <- controlstd
  
  nlminbopt <- intersect(names(control),c("eval.max","iter.max","trace","abs.tol","rel.tol","x.tol","step.min"))
  ucminfopt <- intersect(names(control),c("trace","grtol","xtol","stepmax","maxeval","grad","gradstep","invhessian.lt"))
  optimopt <- names(control) 

  if (control$method=="optimx") {
    if (!requireNamespace("optimx")) stop("Package `optimx` required.")
    if (is.null(control$lower)) control$lower <- -Inf
    if (is.null(control$upper)) control$upper <- Inf
    if (is.null(control$optimx.method)) control$optimx.method <- c("BFGS","Nelder-Mead")
  }

  op <- switch(tolower(control$method),
               nlminb=nlminb(p0,f0,gradient=g0,control=control[nlminbopt]),
               optim=optim(p0,fn=f0,gr=g0,control=control[ucminfopt]),
               optimx=optimx::optimx(p0, f0, g0, ...,
                                     lower=control$lower,
                                     upper=control$upper,
                                     method=control$optimx.method,
                                     control=control$optimx.control),
               ucminf=,
               quasi=,
               gradient=ucminf::ucminf(p0,fn=f0,gr=g0,control=control[ucminfopt],hessian=0),
               ## ,
               ## bhhh={
               ##   controlnr <- list(stabil=FALSE,
               ##                     gamma=0.1,
               ##                     gamma2=1,
               ##                     ngamma=5,
               ##                     iter.max=200,
               ##                     epsilon=1e-12,
               ##                     tol=1e-9,
               ##                     trace=1,
               ##                     stabil=FALSE)
               ##   controlnr[names(control)] <- control
               ##   lava:::NR(start=p0,NULL,g0, h0,control=controlnr)
##               },
               ##                 op <- switch(mycontrol$method,
               ##                              ucminf=ucminf(p0,f,control=mycontrol[ucminfopt],hessian=F),
               ##                optim=optim(p0,f,control=mycontrol[ucminfopt],...),
                 nlminb(p0,f,control=control[nlminbopt]))

  if (inherits(op, "optimx")) {
    val <- op$value
    if (length(val)==1) {
      pp <- as.numeric(coef(op))
    } else {
      idx <- which.min(op$value)
      pp <- as.numeric(coef(op)[idx,])
    }
    op <- list(par=pp, opt=op)
  }
  if (stderr) {
    UU <- U(op$par,indiv=TRUE)    
    I <- -numDeriv::jacobian(U,op$par)
    tol <- 1e-15
    iI <- Inverse(I,tol)
    V <- iI
    sqrteig <- attributes(V)$sqrteig
    J <- NULL
    if (robustvar) {
      J <- crossprod(UU)
      V <- iI%*%J%*%iI
    }
    if (any(sqrteig<tol)) warning("Near-singular covariance matrix (pseudo-inverse used)")
  } else {
    UU <- matrix(NA,ncol=length(op$par),nrow=1)
    I <- J <- V <- matrix(NA,ncol=length(op$par),nrow=length(op$par))
  }

###}}} optim

###{{{ return

  suppressWarnings(cc <- cbind(op$par,sqrt(diag(V))))
  cc <- cbind(cc,cc[,1]/cc[,2],2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
  colnames(cc) <- c("Estimate","Std.Err","Z","p-value")
  vnames1 <- NULL
  trnam <- " "

  if (!eqmean) {
    rnam <- rnames1
    rnames1 <- c(paste(rnam,"MZ",sep=trnam),paste(rnam,"DZ",sep=trnam))
    if (OSon) rnames1 <- c(rnames1,paste(rnam,"OS",sep=trnam))

  }

  if (ACDU["u"]) {
    groups <- c("MZ","DZ"); if (OSon) groups <- c(groups,"OS")
    rnames <- c(rnames1,paste(c("atanh(rho)","atanh(rho)"),groups,sep=trnam))
  } else {
      rnames0 <- c("var(A)","var(C)","var(D)")[ACDU[1:3]]
      if (invtrname!="")
          rnames0 <- paste(invtrname,"(",rnames0,")",sep="")
      rnames <- c(rnames1,rnames0)
      if (OSon) rnames <- c(rnames,"atanh(rho(G[OS]))")
  }
  if (!missing(constrain)) rnames <- rnames[freeidx]
  rownames(cc) <- rnames
  rownames(V) <- colnames(V) <- rnames
  S <- Sigma(op$par)

  npar <- list(intercept=attributes(terms(formula))$intercept,
               pred=nrow(attributes(terms(formula))$factor)-1,
               var=sum(ACDU[-4]),
               ACDU=ACDU[-4]*1)
  
  npar[unlist(lapply(npar,length))==0] <- 0

  environment(mytr) <- baseenv()
  environment(myinvtr) <- baseenv()
  environment(dmytr) <- baseenv()

  val <- list(coef=cc,vcov=V,bread=iI,I=I,score=UU,logLik=attributes(UU)$logLik,opt=op,
              id=Wide[,id], model.frame=Wide,
              Sigma0=S$Sigma0, Sigma1=S$Sigma1, Sigma2=S$Sigma2,
              dS0=dS0, dS1=dS1, dS2=dS2,
              N=N,
              midx0=midx0, midx1=midx1,
              vidx0=vidx0, vidx1=vidx1, vidx2=vidx2,
              eqmean=eqmean, I=I,J=J, robustvar=robustvar,
              transform=list(tr=mytr, invtr=myinvtr, dtr=dmytr,
                  name=trname, invname=invtrname),
              SigmaFun=Sigma,
              npar=npar,
              OS=OSon
              )
  class(val) <- c("bptwin","biprobit")
  return(val)
}

###}}} return

##' @export
model.frame.bptwin <- function(formula,...) {
    formula$model.frame
}

###{{{ RoundMat

RoundMat <- function(cc,digits = max(3, getOption("digits") - 2),na=TRUE,...) {
    res <- format(round(cc,max(1,digits)),digits=digits)
    if (na) return(res)
    res[grep("NA",res)] <- ""
    res
}

###}}} RoundMat

###{{{ multinomlogit

multinomlogit <- function(x,tr=exp,dtr=exp) {
  n <- length(x)
  ex <- tr(x)
  dex <- dtr(x)
  sx <- sum(ex)+1
  f <- c(ex,1)
  df <- c(dex,0)
  res <- f/sx
  dg <- -dex/sx^2   
  gradient <- matrix(ncol=n,nrow=n+1)
  I <- diag(n+1)
  for (i in seq_len(n)) {
    gradient[,i] <- df[i]*I[i,]/sx+dg[i]*f
  }
  attributes(res)$gradient <- gradient
  return(res)
}

###}}} multinomlogit

###{{{ grouptable

##' @export
grouptable <- function(data,id,group,var,lower=TRUE,
                       labels,order,
                       group.labels,group.order,
                       combine=" & ",...) {
    if (!missing(order) || !missing(labels)) {
        data[,var] <- as.factor(data[,var])
        if (missing(order)) order <- seq(length(labels))
        if (missing(labels)) labels <- levels(data[,var])
        data[,var] <- factor(data[,var],levels(data[,var])[order],labels=labels[order])
    }
    wide <- fast.reshape(data,id=id,varying=-group)    
    res <- lapply(split(wide,wide[,group]),
                  function(x) {
                      M <- with(x, table(get(paste(var,"1",sep="")),
                                         get(paste(var,"2",sep=""))))
                      if (lower) {
                          M[lower.tri(M)] <- M[lower.tri(M)]+M[upper.tri(M)]
                          M[upper.tri(M)] <- NA
                      }
                      return(M)
                  })
    if (!missing(group.order) && length(group.order)==length(res))
        res <- res[group.order]
    if (!missing(group.labels) && length(group.labels)==length(res))
        names(res) <- group.labels
    if (length(res)==2 && !is.null(combine)) {
        M <- res[[1]]
        M[upper.tri(M)] <- res[[2]][lower.tri(res[[2]])]
        diag(M) <- paste(diag(M),diag(res[[2]]),sep=combine)
        M <- cbind(rownames(M),M)
        M <- rbind(c("",rownames(M)),M)
        colnames(M) <- rownames(M) <- rep("",nrow(M))
        M[1,1] <- paste(names(res),collapse=combine)
        return(structure(M,class="table"))
        return(M);
    }    
    res
}

###}}} grouptable

##' Simulate twin data from a linear normal ACE/ADE/AE model.
##'
##' @title Simulate twin data
##' @author Klaus K. Holst
##' @export
##' @seealso \code{\link{twinlm}}
##' @keywords models
##' @keywords regression
##' @param nMZ Number of monozygotic twin pairs
##' @param nDZ Number of dizygotic twin pairs
##' @param b1 Effect of covariates (labelled x1,x2,...) of type 1. One
##'     distinct covariate value for each twin/individual.
##' @param b2 Effect of covariates (labelled g1,g2,...) of type 2. One
##'     covariate value for each twin pair.
##' @param mu Intercept parameter.
##' @param acde Variance of random effects (in the order A,C,D,E)
##' @param randomslope Logical indicating wether to include random slopes of
##'     the variance components w.r.t. x1,x2,...
##' @param threshold Treshold used to define binary outcome y0
##' @param cens Logical variable indicating whether to censor outcome
##' @param wide Logical indicating if wide data format should be returned
##' @param ... Additional arguments parsed on to lower-level functions
twinsim <- function(nMZ=100,nDZ=nMZ,b1=c(),b2=c(),mu=0,acde=c(1,1,0,1),randomslope=NULL,threshold=0,cens=FALSE,wide=FALSE,...) {
  n <- nMZ+nDZ
  sA <- acde[1]^0.5; sC <- acde[2]^0.5; sD <- acde[3]^0.5; sE <- acde[4]^0.5; 
  A.MZ <- rnorm(nMZ,sd=sA); A.MZ <- cbind(A.MZ,A.MZ)
  D.MZ <- rnorm(nMZ,sd=sD); D.MZ <- cbind(D.MZ,D.MZ)  
  S2 <- matrix(c(0,1,1,0),2)
  A.DZ <- sA*rmvn(nDZ,sigma=diag(2)+S2*0.5)
  D.DZ <- sD*rmvn(nDZ,sigma=diag(2)+S2*0.25)
  C.MZ <- rnorm(nMZ,sd=sC)
  C.DZ <- rnorm(nDZ,sd=sC)
  yMZ <- mu + A.MZ + cbind(C.MZ,C.MZ) + D.MZ + cbind(rnorm(nMZ,sd=sE),rnorm(nMZ,sd=sE))
  yDZ <- mu + A.DZ + cbind(C.DZ,C.DZ) + D.DZ + cbind(rnorm(nDZ,sd=sE),rnorm(nDZ,sd=sE))
  y <- rbind(yMZ,yDZ)
  if (length(b1)>0) {
    x1 <- rmvn(n,sigma=diag(length(b1)))
    x2 <- rmvn(n,sigma=diag(length(b1)))
    y <- y+cbind(x1%*%b1,x2%*%b1)
  }
  if (length(b2)>0) {
    g <- rmvn(n,sigma=diag(length(b2)))
    ge <- g%*%b2
    y <- y+cbind(ge,ge)
  }
  Cens <- ifelse(cens,rep(Inf,nMZ+nDZ),rnorm(nMZ+nDZ,threshold+1))
  d <- data.frame(id=seq(n),y=y,zyg=c(rep("MZ",nMZ),rep("DZ",nDZ)),
                  cens=Cens)
  vary <- list(c("y1","y2"))
  vnames <- c("y")  
  colnames(d)[2:3] <- vary[[1]]
  if (length(b1)>0) {
    d <- cbind(d,x1=x1,x2=x2);
    if (length(b1)==1) {
        colnames(d)[1:2+ncol(d)-2] <- c("x11","x21")
    }
    vary <- c(vary,lapply(seq(length(b1)),FUN=function(x) paste(c("x1","x2"),x,sep="")))
    vnames <- c(vnames,paste("x",seq_len(length(b1)),sep=""))
  }  
  if (length(b2)>0) {
      d <- cbind(d,g=g)
  }  
  names(vary) <- vnames
  colnames(d) <- sub(".","",colnames(d),fixed=TRUE)
  if (wide) return(d)
  dd <- fast.reshape(d,idname="id",varying=vary)
  
  dd$status <- dd$y<dd$cens
  dd$y0 <- (dd$y>threshold)
  dd$y1 <- (dd$y>threshold & dd$y<cens)*1
  ## S.MZ <- diag(2)*vE+vC+vA
  ## S.DZ <- diag(2)*(vE+vA) + rho*S2*vA + vC
  ## Mu <- (threshold-mu)
  ## probs <- c(marginal=pnorm(mu,threshold,S.MZ[1]^0.5),concordance=pmvnorm(lower=c(threshold,threshold),mean=c(mu,mu),sigma=S.MZ))
  ## probs <- c(probs,casewise=probs[2]/probs[1])
  ## probs
  return(dd)
}

##' 2 Stage Randomization for Survival Data or competing Risks Data 
##'
##' Under two-stage randomization we can estimate the average treatment effect E(Y(i,j)) of treatment regime (i,j). 
##' The estimator can be agumented in different ways: using the two randomizations and the dynamic censoring augmetatation.
##' The treatment's must be given as factors. 
##'
##' The solved estimating eqution is 
##' \deqn{  (  I(min(T_i,t) < G_i)/G_c(min(T_i ,t)) I(T \leq t, \epsilon=1 ) - AUG_0 - AUG_1 + AUG_C  -  p(i,j)) = 0 }
##' where  using the covariates from augmentR0
##' \deqn{ AUG_0 = \frac{A_0(i) - \pi_0(i)}{ \pi_0(i)} X_0 \gamma_0}
##' and  using the covariates from augmentR1
##' \deqn{ AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{ \pi_1(j)} X_1 \gamma_1}
##' and   the censoring augmentation is 
##' \deqn{  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s) }
##' where 
##' \deqn{ \gamma_c(s)} is chosen to minimize the variance given the dynamic  covariates specified by augmentC.
##'
##' In the observational case, we can use propensity score modelling and outcome modelling (using linear regression).
##' 
##' Standard errors are estimated using the influence function  of all estimators and tests of differences can therefore be computed
##' subsequently.
##'
##' @param formula formula with outcome (see \code{coxph})
##' @param data data frame
##' @param cause cause of interest
##' @param time  time of interest 
##' @param cens.code gives censoring code
##' @param beta starting values 
##' @param treat.model0 logistic treatment model for 1st randomization
##' @param treat.model1 logistic treatment model for 2ndrandomization
##' @param augmentR0 augmentation model for  1st randomization
##' @param augmentR1 augmentation model for  2nd randomization
##' @param augmentC augmentation model for censoring 
##' @param cens.model stratification for censoring model based on observed covariates
##' @param estpr estimate randomization probabilities using model
##' @param response.name can give name of response variable, otherwise reads this as first variable of treat.model1
##' @param response.code code of status of survival data that indicates a response at which 2nd randomization is performed
##' @param offset not implemented 
##' @param weights not implemented 
##' @param cens.weights can be given 
##' @param kaplan.meier  for censoring weights, rather than exp cumulative hazard
##' @param no.opt not implemented 
##' @param method not implemented 
##' @param augmentation not implemented 
##' @param outcome can be c("cif","rmst","rmst-cause")
##' @param model  not implemented, uses linear regression for augmentation
##' @param Ydirect use this Y instead of outcome constructed inside the program (e.g. I(T< t, epsilon=1)), see binreg for more on this
##' @param return.dataw to return weighted data for all treatment regimes
##' @param pi0 set up known randomization probabilities 
##' @param pi1 set up known randomization probabilities 
##' @param cens.time.fixed to use time-dependent weights for censoring estimation using weights 
##' @param outcome.iid to get iid contribution from outcome model (here linear regression working models). 
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' 
##' ddf <- mets:::gsim(200,covs=1,null=0,cens=1,ce=2)
##' 
##' bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),ddf$datat,time=2,cause=c(1),
##'         cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
##'         augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
##'         augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
##'         response.code=2)
##' summary(bb) 
##' @export
binregTSR <- function(formula,data,cause=1,time=NULL,cens.code=0,
		      response.code=NULL,
      augmentR0=NULL,treat.model0=~+1,augmentR1=NULL,treat.model1=~+1, 
      augmentC=NULL,cens.model=~+1,estpr=c(1,1),response.name=NULL,
      offset=NULL,weights=NULL,cens.weights=NULL,beta=NULL,
      kaplan.meier=TRUE,no.opt=FALSE,method="nr",augmentation=NULL,
      outcome=c("cif","rmst","rmst-cause"),model="exp",Ydirect=NULL,
      return.dataw=0,pi0=0.5,pi1=0.5,cens.time.fixed=1,outcome.iid=1,...)
{# {{{
  cl <- match.call()# {{{
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}
  if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
    ts <- survival::untangle.specials(Terms, "offset")
    Terms  <- Terms[-ts$terms]
    offset <- m[[ts$vars]]
  }  
  X <- model.matrix(Terms, m)
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  if (!is.null(id)) {
          orig.id <- id
	  ids <- sort(unique(id))
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
  } else { orig.id <- NULL; nid <- nrow(X); id <- as.integer(seq_along(exit))-1; ids <- NULL}
  ### id from call coded as numeric 1 -> 
  id <- id+1
  nid <- length(unique(id))

  if (is.null(offset)) offset <- rep(0,length(exit)) 
  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  if (is.null(time)) stop("Must give time for construction of survival outome modelling \n"); 
  statusC <- (status %in%cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (is.null(response.code)) stop("Give codes for 2nd treatment start (response), 2nd randomization time \n"); 
  statusR <- (status %in% response.code) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  if (sum(statusR)==0) stop("No Responses before time \n"); 
  kmt <- kaplan.meier

  response__  <-  rid__  <- Count1__ <- NULL

  data$status__ <-  status 
  data$id__ <-  id
  data$Count1__ <- cumsumstrata(rep(1,length(entry)),id-1,nid)
  data$entry__ <- entry 
  data$exit__ <- exit 
  data$statusC__ <- statusC
  data$status__cause <- statusE
  data$rid__ <- revcumsumstrata(rep(1,length(entry)),id-1,nid)
  csr <- cumsumstratasum(statusR,id-1,nid) 
  data$response__  <-  csr$sum 
  data$response__lag  <-  csr$lagsum
  cens.strata <- cens.nstrata <- NULL 
  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
###   kmc <- km(formC,data)
  } else formC <- NULL


  ## adjust censoring model according to treat1
  treat.name1 <-  all.vars(treat.model1)[1]
  if (treat.name1 %in% all.vars(formC))  {
	  ## drop treat.name1 from strata, we handle this via the weights
	  ## possibly extend to additional covariates 
	  vv <- all.vars(cens.model)
	  vv <- vv[-grep(treat.name1,vv)]
	  cens.model <- as.formula( paste("~strata(",vv,")"))
	  treat.specific.cens <- 1
  }  else treat.specific.cens <- 0
###  print(cens.model)

  MG.se <- (sum(data$statusC__ & (data$exit__ < time))>0)*1

  ################################################################
  ### iid data consisting of last record, ordered after id   #####
  ################################################################
  id <- id[data$rid__==1]
  oid <- order(id)
  dataiid <- data[data$rid__==1,][oid,]
###  Xorig <- X <- as.matrix(X)
###  Xdata <- X <- X[data$rid__==1,,drop=FALSE][oid,]
  offset <- offset[data$rid__==1][oid]
  weights <- weights[data$rid__==1][oid]
  status <- status[data$rid__==1][oid]
  exit <- exit[data$rid__==1][oid]
###  X2 <- .Call("vecMatMat", X, X)$vXZ
  ph <- 1
  if (is.null(beta)) beta <- rep(0,ncol(X))
  ## take iid vession of data 
  cens.weights <- cens.weights[data$rid__==1][oid]
  response <- (data$response__[data$rid__==1] >0)[oid]
  ### also order Ydirect, check, check
  if (!is.null(Ydirect)) Ydirect <- Ydirect[oid]


  if (is.null(beta)) beta <- rep(0,ncol(X))
###  p <- ncol(X)
###  X <-  as.matrix(X)
###  X2  <- .Call("vecCPMat",X)$XX
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)>=1) Causes <- ucauses[-ccc] else Causes <- ucauses
  obs <- (exit<=time & (status %in% Causes)) | (exit>=time)
  p <- 1

  if (!is.null(Ydirect)) Y <-  Ydirect*obs else {
    if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
    else if (outcome[1]=="rmst") Y <-  c(pmin(exit,time)*obs) 
    else if (outcome[1]=="rmst-cause") Y <- c((status %in% cause)*(time-pmin(exit,time))*obs)
    else stop("outcome not defined") 
  }
  Yorig <- Y
  Y <- Y/cens.weights
  dataiid$Y__ <- Y

 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum((status %in% cause)*(exit<=time))

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

## fitting treatment models for 1st and 2nd randomization 
fittreat <- function(treat.model,data,id,ntreatvar,nlev) {# {{{
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predictmlogit(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2

out <- list(iidalpha=iidalpha,pA=pA,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

dataR0 <- dataiid
## response==1 data
dataR1 <- subset(dataiid,response__==1)

### treatment is rhs of treat.model 
treat.name0 <-  all.vars(treat.model0)[1]
treatvar0 <- dataR0[,treat.name0]
## make into factor
if (!is.factor(treatvar0))  treatvar0 <- as.factor(treatvar0) 
## treatvar, 1,2,...,nlev or 1,2
treats0 <- treats(treatvar0)

## fit treatment model for first randomization
idR0 <-  id[data$Count1__==1] ## first record with first randomization info
idR0 <- dataiid[,"id__"]
if (estpr[1]==1) {
   fitt0 <- fittreat(treat.model0,dataR0,idR0,treats0$ntreatvar,treats0$nlev)
   iidalpha0 <- fitt0$iidalpha
} else {
   pi0 <- rep(pi0,treats0$nlev)
}

###
treat.name1 <-  all.vars(treat.model1)[1]
## only  look among those randomized 
treatvar1 <- dataR1[,treat.name1]
###treatvar1 <- dataiid[,treat.name1]
if (!is.factor(treatvar1))  treatvar1 <- factor(treatvar1) 

## fit treatment model for second randomization   using combined model
treats1 <- treats(treatvar1)
###idR1 <- id[data$response__==1 & data$rid__==1] ## first record with first randomization info
idR1 <- dataR1[,"id__"]
dataR1[,treat.name1] <- treatvar1
if (estpr[2]==1) {
fit1 <- fittreat(treat.model1,dataR1,idR1,treats1$ntreatvar,treats1$nlev)
## put iid in matrix after nid 
iidalpha1 <- apply(fit1$iidalpha,2,sumstrata,sort(idR1)-1,nid) ## *(nrow(dataR1)/nid)
}

## read if there are multiple responses recorded and read name of variable, and levels for different responses 
if (length(all.vars(treat.model1)) >=3 & is.null(response.name)) response.name <- all.vars(treat.model1)[3]
if (!is.null(response.name)) {
respvar <- as.factor(dataR1[,response.name])
if (!is.factor(respvar)) stop(paste("Response=",respvar," must be coded as factor \n",sep="")); 
## treatvar, 1,2,...,nlev or 1,2
## drop possible "0" empty level among dataR1 with responses
respvar <- droplevels(respvar)
nlevresp <- nlevels(respvar)
nlevsr <- levels(respvar)
nrespvar <- as.numeric(respvar)
} else {
nlevresp <- 1 
nlevsr <- "Response"
nrespvar <- rep(1 ,nrow(dataR1))
response.name <- "response"
dataiid[,response.name] <- 1
}

if (estpr[2]==0) {
	mpi1 <- matrix(0,nlevresp,treats1$nlev)
	if (length(pi1)!=treats1$nlev) pi1 <- rep(pi1,treats1$nlev)
	for (i in seq(nlevresp)) mpi1[i,] <- pi1[i]
	pi1 <- mpi1
}

## to fit stratified models 
treat.model1s <- update.formula(treat.model1,~+1)
### stratify on A0.f 
treat.model1s <- update.formula(treat.model1s,as.formula(paste("~",treat.name0)) )
treat.model1s <- treat.model1
###print(treat.model1s)

## set up stratified models for response
tv1 <- list(); fitt1 <- list()
combof <- combo <- NULL
for (nnresp in seq(nlevresp)) {
   k <- nnresp
   dataresp <- subset(dataR1,nrespvar==nnresp)
   tv1[[k]] <- treats(treatvar1[nrespvar==nnresp])
   if (k==1)  {combof <-  tv1[[k]]$nlevs; 
               combo <- seq(tv1[[k]]$nlev) }
   else {
	   combof <- expand.grid(combof,tv1[[k]]$nlevs);
	   combo <- expand.grid(combo,seq(tv1[[k]]$nlev))
   }
###   idresp <- idR1[nrespvar==nnresp]
###   fitt1[[k]] <- fittreat(treat.model1s,dataresp,idresp,tv1[[k]]$ntreatvar,tv1[[k]]$nlev)
}
if (nlevresp==1) { combof <- expand.grid(combof); combo <- expand.grid(combo) }

A0 <- as.numeric(dataiid[,treat.name0])
A1 <- as.numeric(dataiid[,treat.name1])
## code possible NA as 0, to avoid problems
A1[is.na(A1)] <- 0

llR0 <- llR <- llR1 <- llR01 <- c()
riskG0 <- riskG <- riskG1 <- riskG01 <- c()
riskG0C <- riskGC <- riskG1C <- riskG01C <- c()
riskG0.iid <- riskG.iid <- riskG1.iid <- riskG01.iid <- c()
riskG0C.iid <- riskGC.iid <- riskG1C.iid <- riskG01C.iid <- c()
nc <- nrow(combo)
MGc <-  Augment <- Augment.times <- Ht <- dataW <- DA1 <- rnames <- c() 
dynCgammat <- list()
cown <- 0


### looping over A0.f response*A1.f
for (i in seq(treats0$nlev)) { ## {{{
for (v in seq(nc))  {
	cown <- cown+1
        namesiv <- paste( paste(treat.name0,"=",treats0$nlevs[i],", ",sep=""),
         	      paste(response.name,"*",treat.name1,"=",
	        	    paste(combof[v,],collapse=","),sep="",collapse=""),sep="")
        rnames <- c(rnames,namesiv)
	## first only one response
	k <- 1
	j <- combo[v,]
	dataij <- dataiid
	pA1j <- rep(1,nrow(dataij))
	responsetype <- as.numeric(as.factor(dataiid[,response.name]))
	if (estpr[1]==1) pA0i <- fitt0$pA else { pA0i <- 0.5; pA0i[i] <- pi0[i]; }
	if (estpr[2]==1) pA1j[fit1$id] <- fit1$pA else pA1j <-c(mdi(pi1,responsetype,A1))

	A1j <- (apply(outer(A1,j,FUN="==")*outer(responsetype,seq(nlevresp),FUN="==") ,1,sum) >=1)
	W <- ((A0==i)/pA0i)*((response==0)+(response!=0)*(A1j)/pA1j)
	W1 <- ((response==0)+(response!=0)*(A1j)/pA1j)
	###### weights for second randomization based on known probabilities
	W1k <- ((response==0)+(response!=0)*(A1j)/pA1j)

       ### possibly estimate censoring weights depending on A0.f and A1.f, with weights and construct new Y 
       ### using data with time-dependent weights for second treatment only 
	if (treat.specific.cens==1) {
            data$WW1__ <- 1
	    data$WW1__[data$response__lag>=1] <- W1k[data$id__][data$response__lag>=1]
            ## use time-fixed weights instead,  
	    if (cens.time.fixed==1) data$WW1__  <-  W1k[data$id__]
            formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
            formC <- update.formula(formC,.~.+status__) 
###	    print(formC)
            resC <- phreg(formC,data,weights=data$WW1__,no.opt=TRUE,no.var=1)
            if (resC$p>0) kmt <- FALSE
            exittime <- pmin(data$exit__,time)
            cens.weights <-c(suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv))
	    ### where the weights are 0, cens.weights are 0, and do not need them there 
	    cens.weights[data$WW1__<=0.00001]  <- 1
            ## strata from original data 
            cens.strata <- resC$strata[order(resC$ord)]
            cens.nstrata <- resC$nstrata
	    ## construct new IPCW with weights from KM, ordered
	    cens.weights <- cens.weights[data$rid__==1][oid]
	    Y <- Yorig/cens.weights
	}

	## check if there are censorings in data consistent with treatment arm
	## update  to check within treatment arm
        MG.se <- (sum(data$statusC__ & (data$exit__ < time) )>0)*1

	DW0 <- W*pA0i
	dataij$W__ <- W
	dataij$YW__ <- W*Y
	dataij$W0 <- ((A0==i)-pA0i)/pA0i
	dataij$W1 <- ((A0==i)/pA0i)*(response!=0)*((A1j)-pA1j)/pA1j
	dataij$W11 <- ((A0==i)/pA0i)*((A1j))/pA1j
	dataij$A0i <- (A0==i)*1
	dataij$A1j <- A1j
	dataij$i__  <-  i
	dataij$j__  <-  v
	dataij$pA0i__  <- pA0i
	dataij$pA1j__  <- pA1j
	dataij$response <- response 
	if (return.dataw==1) dataW <- rbind(dataW,dataij)

	## simple {{{
	ff <- as.formula(YW__ ~ +1)
        ll0 <- lm(ff,data=dataij)
        sll0 <- estimate(ll0)
	## iid ordered after id i data
	iid0 <- ll0$residuals
	h0 <- DW0*Y
	if (estpr[1]==1) {
        DaPsia <-  apply(fitt0$DPai*h0,2,sum,drop=FALSE)
        iidpala0 <- c(DaPsia %*% t(iidalpha0))
	} else iidpala0 <- 0

	if (estpr[2]==1) {
        ddR1 <- response*(A1j)*Y*(A0==i)/pA0i
        ddR1 <- ddR1[response>=1]
        DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
        iidpala1 <- c(DaPsia %*% t(iidalpha1))
	} else iidpala1 <- 0
        ###
        risk.iid <-  (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1)/nid
        llR <- c(coef(ll0)[1],crossprod(risk.iid)^.5)
        riskG <- rbind(riskG,llR)
        riskG.iid <- cbind(riskG.iid,risk.iid)
        # }}}

	### using 1st stage randomization
	if (!is.null(augmentR0)) {# {{{
              varR0 <- all.vars(augmentR0)
	      newnR0 <- paste(c("int",varR0),"__W0",sep="")
	      newaugR0 <- as.formula(paste("~",paste(c("int",varR0),"__W0",sep="",collapse="+")))
	      ff0 <- update.formula(newaugR0,YW__~.)
	      dataij[,newnR0] <- cbind(1,dataij[,varR0]) * dataij$W0
              ###	      
              llR0 <- lm(ff0,data=dataij)
	      iid0 <- llR0$residuals
              ###
	      if (estpr[1]==1) {
		      h0 <- DW0*Y - as.matrix(cbind(1,dataij[,varR0])) %*% coef(llR0)[-1]
		      DaPsia <-  apply(fitt0$DPai*(A0==i)*c(h0),2,sum,drop=FALSE)
		      iidpala0 <- c(DaPsia %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
		      ddR1 <- (A1j)*Y*(A0==i)/pA0i
		      ddR1 <- ddR1[response>=1]
		      DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
		      iidpala1 <- c(DaPsia %*% t(iidalpha1))
	      } else iidpala1 <- 0

	      if (outcome.iid!=0) {
		      miid <- estimate(llR0) 
		      miid <-  miid$IC[,-1]/nid
		      Dma <-  apply(dataij[,newnR0,drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR0 <- c(coef(llR0)[1],crossprod(risk.iid)^.5)
	      riskG0 <- rbind(riskG0,llR0)
	      riskG0.iid <- cbind(riskG0.iid,risk.iid)
	} # }}}

	### using 2nd stage randomization
	if (!is.null(augmentR1)) {# {{{
              varR1 <- all.vars(augmentR1)
	      newnR1 <- paste(c("int",varR1),"__W1",sep="")
	      newaugR1 <- as.formula(paste("~",paste(c("int",varR1),"__W1",sep="",collapse="+")))
	      ff1 <- update.formula(newaugR1,YW__~.)
	      dataij[,newnR1] <- cbind(1,dataij[,varR1]) * dataij$W1

              llR1 <- lm(ff1,data=dataij)
              sllR1 <- estimate(llR1)
	      iid0 <- llR1$residuals
	      ###   ( W Y -  W1 * h1) 
	      if (estpr[1]==1) {
	      h0 <- (llR1$residuals+coef(llR1)[1])*pA0i
              DaPsia <-  apply(fitt0$DPai*(A0==i)*h0,2,sum,drop=FALSE)
	      iidpala0 <- c(DaPsia %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
	      pred1 <- as.matrix(cbind(1,dataij[,varR1])) %*% coef(llR1)[-1]
	      h1 <- Y - pred1 
	      ddR1 <- (A1j)*h1*(A0==i)/pA0i
	      ddR1 <- ddR1[response>=1]
              DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
	      iidpala1 <- c(DaPsia %*% t(iidalpha1))
	      } else iidpala1 <- 0

	      if (outcome.iid!=0) {
		      miid <-  sllR1$IC[,-1]/nid
		      Dma <-  apply(dataij[,newnR1,drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR1 <- c(coef(llR1)[1],crossprod(risk.iid)^.5)
	      riskG1 <- rbind(riskG1,llR1)
	      riskG1.iid <- cbind(riskG1.iid,risk.iid)
	} # }}}

	### using both randomizations
	if ((!is.null(augmentR0)) & (!is.null(augmentR1)) ) {# {{{
	      ff01 <- update(ff0, reformulate(c(".",newnR1)))
              llR01 <- lm(ff01,data=dataij)
	      iid0 <- llR01$residuals
              ###
              varR01 <- all.vars(ff01)[-1]
	      h00 <- as.matrix(cbind(1,dataij[,varR0])) %*% coef(llR01)[newnR0]
	      h01 <- as.matrix(dataij[,newnR1]*pA0i) %*% coef(llR01)[newnR1]
	      h0 <- DW0*Y - h00-h01
	      h11 <- as.matrix(cbind(1,dataij[,varR1])) %*% coef(llR01)[newnR1]
	      h1 <-  Y - h11
              ###
	      if (estpr[1]==1) {
		      DaPsia0 <-  apply(fitt0$DPai*(A0==i)*c(h0),2,sum,drop=FALSE)
		      iidpala0 <- c(DaPsia0 %*% t(fitt0$iidalpha))
	      } else iidpala0 <- 0
              ###
	      if (estpr[2]==1) {
		      ddR1 <- (A1j)*h1*(A0==i)/pA0i
		      ddR1 <- ddR1[response>=1]
		      DaPsia <-  apply(fit1$DPai*c(ddR1),2,sum,drop=FALSE)
		      iidpala1 <- c(DaPsia %*% t(iidalpha1))
     	      } else iidpala1 <- 0
	      if (outcome.iid!=0) {
		      miid <- estimate(llR01) 
		      miid <-  miid$IC[,-1]/nid
		      Dma <-  apply(dataij[,c(newnR0,newnR1),drop=FALSE],2,sum,drop=FALSE)
		      miid <-  c(Dma %*% t(miid))
	      } else miid <- 0

              risk.iid <- (iid0+estpr[1]*iidpala0+estpr[2]*iidpala1+outcome.iid*miid)/nid
	      llR01 <- c(coef(llR01)[1],crossprod(risk.iid)^.5)
	      riskG01 <- rbind(riskG01,llR01)
	      riskG01.iid <- cbind(riskG01.iid,risk.iid)
	} # }}}

 if (MG.se) {## {{{ censoring adjustment of variance using KM (could be stratified)
    ### take possibly weighted cox 
    xx <- resC$cox.prep
    icoxS0 <- rep(0,length(resC$S0))
    icoxS0[resC$S0>0] <- 1/resC$S0[resC$S0>0]
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <-  icoxS0
    S0i2[xx$jumps+1] <- icoxS0^2
    ### set up response using that Y from basic data is ordered after id
    Yt <- Y[xx$id+1]
    Wt <- W[xx$id+1]
    ## fixed weights following Y with known strata
    btime <- 1*(xx$time<time)
    Yt <- Yt*Wt
    ## compute function h(s) = \sum_i I(s \leq T_i \leq t), \int h(s)/Ys  dM_i^C(s) 
    h  <-  revcumsumstrata(Yt*xx$sign,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),1)
    U[xx$jumps+1,] <- (resC$jumptimes<=time)*h[xx$jumps+1,]*icoxS0
    MGt <- U[,drop=FALSE]-IhdLam0*xx$sign*c(xx$weights)
    ### Censoring Variance Adjustment  
    MGCiid <- apply(MGt,2,sumstrata,xx$id,nid)

    riskG.iid[,cown] <-     riskG.iid[,cown]+MGCiid/nid 
    riskG0.iid[,cown] <-   riskG0.iid[,cown]+MGCiid/nid 
    riskG1.iid[,cown] <-   riskG1.iid[,cown]+MGCiid/nid 
    riskG01.iid[,cown] <- riskG01.iid[,cown]+MGCiid/nid 
    MGc <- cbind(MGc,MGCiid/nid) 
    Ht <- cbind(Ht,h) 
  }  else { MGCiid <- 0; MGc <- NULL }
 ## }}}

      ### computing censoring augmentation
      if ((!is.null(augmentC)) & MG.se) {# {{{

	data$YW__ <- Y[data$id__]*W[data$id__]
        varsC <- c("YW__",attr(terms(augmentC), "term.labels"))
        formCC <- update(formC, reformulate(c(".", varsC)))
	www <- rep(1,nrow(data))
	if (treat.specific.cens==1)  www <-data$WW1__;  
        cr2 <- phreg(formCC, data = data, no.opt = TRUE, no.var = 1,weights=www, ...)
        xx <- cr2$cox.prep
        icoxS0 <- rep(0,length(cr2$S0))
        icoxS0[cr2$S0>0] <- 1/cr2$S0[cr2$S0>0]
        S0i <- rep(0, length(xx$strata))
        S0i[xx$jumps + 1] <- icoxS0
	km <- TRUE
        if (!km) {
            cumhazD <- c(cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum)
            St <- exp(-cumhazD)
        } else St <- c(exp(cumsumstratasum(log(1 - S0i), xx$strata, xx$nstrata)$lagsum))

     nterms <- cr2$p-1
     dhessian <- cr2$hessianttime
     dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
     ###  matrix(apply(dhessian,2,sum),3,3)
     timeb <- which(cr2$cumhaz[,1]<time)
     ### take relevant \sum H_i(s,t) (e_i - \bar e)
     covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
     ### construct relevant \sum (e_i - \bar e)^2
     Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
     ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
     gammatt <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
     S0 <- cr2$S0[timeb]
     gammatt[is.na(gammatt)] <- 0
     gammatt[gammatt==Inf] <- 0
     Gctb <- St[cr2$cox.prep$jumps+1][timeb]
     augmentt.times <- apply(gammatt*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum)
     augment.times <- sum(augmentt.times)/nid

     Augment.times <- c(Augment.times,augment.times)
     augmentt <- augment.times 

   #### iid magic  for censoring augmentation martingale ## {{{
   ### int_0^infty gamma(t) (e_i - ebar(s)) 1/G_c(s) dM_i^c
   xx <- cr2$cox.prep
   jumpsC <- xx$jumps+1
   rr0 <- xx$sign
   S0i <- rep(0,length(xx$strata))
   S0i[jumpsC] <- c(1/(icoxS0*St[jumpsC]))
   S0i[jumpsC] <- icoxS0

   pXXA <- ncol(cr2$E)-1
   EA <- cr2$E[timeb,-1]
   gammasEs <- .Call("CubeMattime",gammatt,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   gammasE <- matrix(0,length(xx$strata),1)
   gammattt  <-    matrix(0,length(xx$strata),pXXA*1)
   jumpsCt <- jumpsC[timeb]
   gammasE[jumpsCt,] <- gammasEs
   gammattt[jumpsCt,] <- gammatt
   gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
   gammadLam0 <-   apply(gammattt*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
   XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X[,-1],pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   Ut <- Et <- matrix(0,length(xx$strata),1)
   Ut[jumpsCt,] <- augmentt.times
   MGCt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
   MGCiid <- apply(MGCt,2,sumstrata,xx$id,nid)

   riskG[cown,]      <- riskG[cown,]+augmentt
   riskG.iid[,cown]  <- riskG.iid[,cown]+MGCiid/nid 
   riskG0[cown,]     <- riskG0[cown,]+augmentt
   riskG0.iid[,cown] <- riskG0.iid[,cown]+MGCiid/nid 
   riskG1[cown,]     <- riskG1[cown,]+augmentt
   riskG1.iid[,cown] <- riskG1.iid[,cown]+MGCiid/nid 
   riskG01[cown,]    <- riskG01[cown,]+augmentt
   riskG01.iid[,cown]<- riskG01.iid[,cown]+MGCiid/nid 

   dynCgammat  <-  c(dynCgammat,list(gammatt)) 
   ## }}}

    } ## }}}

} } # }}} loop over A0, response*A1.f

varG <- varG0 <- varG1 <- varG01 <- NULL
rownames(riskG) <-  rnames
colnames(riskG) <-  c("coef","se","se-fixed-Gc")[1:2]


   varG <- crossprod(riskG.iid)
   riskG <- cbind(riskG,diag(varG)^.5)[,c(1,3)]
if ((!is.null(augmentR0))) {
   rownames(riskG0) <- rnames
   colnames(riskG0) <- c("coef","se","se-fixed-Gc")[1:2]
   varG0 <- crossprod(riskG0.iid)
   riskG0 <- cbind(riskG0,diag(varG0)^.5)[,c(1,3)]
} 
if ((!is.null(augmentR1))) {
   rownames(riskG1) <- rnames
   colnames(riskG1) <- c("coef","se","se-fixed-Gc")[1:2]
   varG1 <- crossprod(riskG1.iid)
   riskG1 <- cbind(riskG1,diag(varG1)^.5)[,c(1,3)]
} 
if ((!is.null(augmentR0)) & (!is.null(augmentR1)) ) {
   rownames(riskG01) <- rnames
   colnames(riskG01) <- c("coef","se","se-fixed-Gc")[1:2]
   varG01 <- crossprod(riskG01.iid)
   riskG01 <- cbind(riskG01,diag(varG01)^.5)[,c(1,3)]
}

if (!is.null(augmentC) & MG.se) names(Augment.times) <- rnames

###pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
###contrast <- -pdiff(length(nlevs))
###nncont <- c()
###for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
###rownames(contrast) <- nncont
###

riskG <- list( riskG=riskG, riskG0=riskG0,riskG1=riskG1,riskG01=riskG01)
riskG.iid <- list(riskG0.iid=riskG0.iid,riskG1.iid=riskG1.iid,riskG01.iid=riskG01.iid,
		  riskG.iid=riskG.iid,id=id,orig.id=orig.id)
varG <- list(varG=varG, varG0=varG0, varG1=varG1, varG01=varG01)
val <- list( riskG.iid=riskG.iid, MGc=MGc, CensAugment.times=Augment.times,
            dynCens.coef=dynCgammat,riskG=riskG,varG=varG,dataW=dataW)

  class(val) <- "binregTSR"
  return(val)
}# }}}

##' @export
print.binregTSR  <- function(x,...) {# {{{
	print(summary(x),...)
}# }}}

##' @export
summary.binregTSR <- function(object,...) {# {{{
res <- object$riskG
class(res) <- "summary.binregTSR"
return(res)
}# }}}

##' @export
print.summary.binregTSR  <- function(x,...) {# {{{
    cat("Simple estimator :\n")
    print(x$riskG)
    cat("\n")

    cat("First Randomization Augmentation :\n")
    print(x$riskG0)
    cat("\n")

   cat("Second Randomization Augmentation :\n")
   print(x$riskG1)
   cat("\n")

   cat("1st and 2nd Randomization Augmentation :\n")
   print(x$riskG01)
   cat("\n")
} # }}}

   simdataTSR <- function(n,base12,base1d,base2d,...) { ## {{{
   A0 <- rbinom(n,1,0.5)
   A1 <- rbinom(n,1,0.5)
   X0 <- matrix(rbinom(2*n,1,0.5),n,2)
   betaX1 <- c(2.7,-2.7)
   X1 <- matrix(rbinom(2*n,1, expit(X0 %*% betaX1)),n,2)
   cov(cbind(X0,X1))
   ###
   beta13 <- c(-0.3,0.4,-0.5)
   beta14 <- c(-0.3,0.2,-0.5)
   beta23 <- c(-0.3,0.4,-0.5)
   beta24 <- c(-0.3,0.2,-0.5)
   betaR <- c(-0.3,0.4,-0.5)
   gamma23 <- 0.3
   gamma24 <- -0.3
   ###
   rr <- exp( cbind(A0,X0) %*% betaR)
   rr13 <- exp( cbind(A0,X0) %*% beta13)
   rr14 <- exp( cbind(A0,X0) %*% beta14)
   rr23 <- exp( cbind(A1,X1) %*% beta23)
   rr24 <- exp( cbind(A1,X1) %*% beta24)
   rd <- cbind(rr13,rr14)
   rd2 <- cbind(rr23,rr24)
   ###
   iddata <- simMultistateII(base12,base1d,base2d,rr=rr,rd=rd,rd2=rd2,
			     early2=1800,gamma23=gamma23,gamma24=gamma24,...)
   covX0 <- cbind(A0[iddata$id],X0[iddata$id,])
   colnames(covX0) <- c("A0","X01","X02")
   covX1 <- cbind(A1[iddata$id],X1[iddata$id,])

   iddata  <-  count.history(iddata,types=2)
   ## only covariates after having gone to stage 2
   covX1 <- covX1*c(iddata$Count2)
   colnames(covX1) <- c("A1","X11","X12")
   iddata <- cbind(iddata,covX0,covX1)
   iddata$R3 <- (iddata$entry<1800)*c(iddata$Count2)
   iddata$R4 <- (iddata$entry<1800)*c(iddata$Count2)
   ###

   cid  <-  countID(iddata)
   iddata$cid <- cid$Countid
   iddata$tid <- cid$Totalid
   return(iddata) 
   } ## }}}

######################## ##################### #####################
## illness death competing risks with two causes of death
## First simple model, then with covariates addedd 
## effect of transition into 2 being early for cause 3, and 4
######################## ##################### #####################
simMultistateII <- function(cumhaz,death.cumhaz,death.cumhaz2,n=NULL,
		    rr=NULL,rd=NULL,rd2=NULL,gamma23=0,gamma24=0,early2=10000,
		    gap.time=FALSE,max.recurrent=100,cens=NULL,rrc=NULL,...) 
{# {{{

  status <- NULL

  ## cumhaz is cumulative out of state 1 to state 2 
  if (!is.null(n)) rr <- matrix(1,n,1) 
  n <- nrow(rr); 
  ## covariate adjustment 
  if (is.null(rd))   rd  <- matrix(1,n,length(death.cumhaz))
  if (is.null(rd2))  rd2  <- matrix(1,n,length(death.cumhaz2))

  ####
  if (!is.null(cens)) cens <- list(cens)
  cumss  <-  c(list(cumhaz),death.cumhaz,death.cumhaz2,cens)
  ### extend of cumulatives
  for (i in seq_along(cumss))  cumss[[i]] <- rbind(0,cumss[[i]])
  ## extend range of all cumulatives to max range, by linear extension
  cumss <- extendCums(cumss,NULL)
  maxtime <- tail(cumss[[1]],1)[1]

 if (!is.null(cens)) { 
	  cens <- cumss[[1+length(death.cumhaz)+length(death.cumhaz2)+1]]
	  if (is.null(rrc)) rrc <- rep(1,n); 
	  ctime <- rchaz(cens,rrc)$time
  } else ctime <- rep(maxtime,n)

  ## hazards out of 1 and out of 2
  chaz1 <- cumss[1:3]; rr1 <- cbind(rr,rd)
  chaz2 <- cumss[4:5]; rr2 <- cbind(rd2)

  ## time out of state 1 
  tall <- rcrisks(chaz1,rr1,causes=c(2,3,4))
  tall$id <- 1:n
  tall$status[tall$time>ctime] <- 0; 
  tall$time[tall$time>ctime] <- ctime[tall$time>ctime] 
  tall$from <- 1
  tall$to <- tall$status

  ## simulating out of 2 
  tall2 <- subset(tall,status==2)
  rr23t <- exp(gamma23*(tall2$time<early2))
  rr24t <- exp(gamma24*(tall2$time<early2))
  sim2 <- rcrisks(chaz2,rr2[tall2$id,]*cbind(rr23t,rr24t),causes=c(3,4),entry=tall2$time)
  sim2$id <- tall2$id
  ctime2 <- ctime[tall2$id]
  sim2$status[sim2$time>ctime2] <- 0; 
  sim2$time[sim2$time>ctime2] <- ctime2[sim2$time>ctime2] 
  sim2$from <- 2
  sim2$to <- sim2$status
  tall <- rbind(tall,sim2)

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  return(tall)
  }# }}}

gsim <- function(n,null=1,cens=NULL,ce=2,covs=1,
	    beta0=c(0.1,0.5,-0.5),beta1=c(0.4,0.3,0.5,-0.5),betaR=c(-0.3,-0.5,0.5),betac=c(0.3,0.3), 
            beta0R=c(0,0.5,-0.5),beta1R=c(0,0.5,-0.5),tsr=1)
   {# {{{

   Count2 <- TR <- NULL

   X0 <- matrix(rbinom(2*n,1,0.5),n,2)
   betaX1 <- 0*c(2.7,-2.7)
   X1 <- matrix(rbinom(2*n,1,expit(X0 %*% betaX1)),n,2)

   O10 <- 0.5; O20 <- 4; OR1 <- 1; OR2 <- 2; 
   if (null==0) Os11 <- 8 
   if (null==1) Os11 <- 3
   Os12 <- 3
   if (null==0) Os21 <- 6
   if (null==1) Os21 <- 3
   Os22 <- 3
   ###
   pr1 <- 0.5
   pr2 <- 0.5
   ###
   if (tsr==0) p0 <- expit(cbind(1,X0) %*% beta0R) else p0 <- 0.5
   A0 <- rbinom(n,1,p0)+1
   if (tsr==0) p1 <- expit(cbind(1,X1) %*% beta1R) else p1 <- 0.5
   A1 <- rbinom(n,1,p1)+1
   ###
   if (covs==1) {
   rr01 <- exp( cbind(0,X0) %*% beta0)
   rr02 <- exp( cbind(1,X0) %*% beta0)
   rr11 <- exp( cbind(0,0,X1) %*% beta1 )
   rr12 <- exp( cbind(0,1,X1) %*% beta1 )
   rr21 <- exp( cbind(1,0,X1) %*% beta1 )
   rr22 <- exp( cbind(1,1,X1) %*% beta1 )
   rrr1 <- exp( cbind(0,X0) %*% betaR)
   rrr2 <- exp( cbind(1,X0) %*% betaR)
   } else {
	   rr21 <- rr22  <- rr01 <- rr02 <- rr11 <- rr12 <- rrr1 <- rrr2 <- 1
   }
   ###
   TR1 <- runif(n)*OR1*rrr1
   TR2 <- runif(n)*OR2*rrr2

      TCR1 <- TR1 
      TCR2 <- TR2 
   if (cens!=0)  {
	   if (cens==2 | cens==1) {
	      TCR1 <- runif(n)*OR1*ce 
	      TCR2 <- runif(n)*OR2*ce 
	   } 
   }
   ###
   T10 <- rexp(n)*O10*rr01
   T20 <- rexp(n)*O20*rr02
   ### response if (TR1 < T10) or (TR2 < T20)
   R1 <- (TR1 < T10)*1 
   R2 <- (TR2 < T20)*1 
   ###
   Ts11 <- rexp(n)*Os11*exp(-3*TR1)*rr11
   Ts12 <- rexp(n)*Os12*exp(-3*TR1)*rr12
   Ts21 <- rexp(n)*Os21*exp(-1*TR2)*rr21
   Ts22 <- rexp(n)*Os22*exp(-1*TR2)*rr22
   ###
   Tt11 <- TR1 + Ts11
   Tt12 <- TR1 + Ts12
   Tt21 <- TR2 + Ts21
   Tt22 <- TR2 + Ts22
   ###
   TT <- (A0==1)*( (1-R1)*T10 + R1*((A1==1)*Tt11 + (A1==2)*Tt12))+ 
         (A0==2)*( (1-R2)*T20 + R2*((A1==1)*Tt21 + (A1==2)*Tt22))  
###	     
 TT11  <-  (1-R1)*T10 + R1*Tt11
 TT12  <-  (1-R1)*T10 + R1*Tt12
 TT21  <-  (1-R2)*T20 + R2*Tt21
 TT22  <-  (1-R2)*T20 + R2*Tt22
 TTt <- cbind(TT11,TT12,TT21,TT22)
 ###
   TC10 <- rexp(n)*O10*ce
   TC20 <- rexp(n)*O20*ce
   ### response if (TR1 < T10) or (TR2 < T20)
   RC1 <- (TCR1 < TC10)*1 
   RC2 <- (TCR2 < TC20)*1 
   ###
   TCs11 <- rexp(n)*Os11*exp(-3*TCR1)*ce 
   TCs12 <- rexp(n)*Os12*exp(-3*TCR1)*ce 
   TCs21 <- rexp(n)*Os21*exp(-1*TCR2)*ce
   TCs22 <- rexp(n)*Os22*exp(-1*TCR2)*ce
   ###
   TC11 <- TCR1 + TCs11
   TC12 <- TCR1 + TCs12
   TC21 <- TCR2 + TCs21
   TC22 <- TCR2 + TCs22
  ###   
  CC <-  (A0==1)*( (1-RC1)*TC10 + RC1*((A1==1)*TC11 + (A1==2)*TC12))+ 
         (A0==2)*( (1-RC2)*TC20 + RC2*((A1==1)*TC21 + (A1==2)*TC22))  
 CC11  <-  (1-RC1)*TC10 + RC1*TC11
 CC12  <-  (1-RC1)*TC10 + RC1*TC12
 CC21  <-  (1-RC2)*TC20 + RC2*TC21
 CC22  <-  (1-RC2)*TC20 + RC2*TC22
 CCt <- cbind(CC11,CC12,CC21,CC22)
 ##
### CC  <- rexp(n)*ce* exp((A0-1)*betac[1]+(A1-1)*betac[2])
 ###
  if (cens==0) cc <- max(TT)+1 else  {
	  if (cens==1) cc <- rexp(n)*ce*exp((A0-1)*betac[1])
	  if (cens==2) cc <- CC
	  if (cens==3) cc <- CC
  }
  ###
  time <- pmin(TT,cc)
  status <- (TT<cc)*1
  ###  
  data <- data.frame(time=time,T10=T10,Tt11=Tt11,Tt12=Tt12,TT=TT,status=status,A1=A1,A0=A0,R1=R1,R2=R2,R=(A0==1)*R1+(A0==2)*R2,id=1:n,
		     TR=(A0==1)*TR1+(A0==2)*TR2,Resp=(A0==1)*R1+(A0==2)*R2)
  dfactor(data) <- A0.f~A0
  dfactor(data) <- A1.f~A1
  datat <- timereg::event.split(data,cuts="TR",name.start="entry")

  datat <- dtransform(datat,status=2,time==TR)
  datat  <-  count.history(datat,types=2)
  datat  <-  dtransform(datat,A1=0,Count2==0)
  datat$response <- (datat$Count2==1)*1
  datat$A11t <- (datat$A1==1)*1
  datat$A12t <- (datat$A1==2)*1
  datat$A01 <- (datat$A0==1)*1
  datat$A02 <- (datat$A0==2)*1


  covX0 <- X0[datat$id,]
  colnames(covX0) <- c("X01","X02")
  covX1 <- X1[datat$id,]
  ###
  covX1 <- covX1*c(datat$Count2)
  colnames(covX1) <- c("X11","X12")
  datat <- cbind(datat,covX0,covX1)
  datat <- dtransform(datat,TR=0,Count2==0) 

  res <- list(data=data,datat=datat,TTt=TTt,CCt=CCt)
  return(res)
  } # }}}

##' @export
force.same.cens <- function(data,id="id", time="time",cause="cause",entrytime=NULL,cens.code=0)
{ ## {{{ 
  ### no missing values

   ### handle time-variables separately
   data <- dsort(data,id)
   w <- which(names(data) %in% c(time,cause,entrytime,id) )
   datao <- data[,-w]

   data <- data[,c(time,cause,entrytime,id)]

  if (is.null(entrytime)) entry <- rep(0,nrow(data)) else entry <- data[,entrytime]

   censo <- (data[,cause]==cens.code)
   Wide <- fast.reshape(data,id=id)
   time1 <- paste(time,1,sep="")
   time2 <- paste(time,2,sep="")
   stat1 <- paste(cause,1,sep="")
   stat2 <- paste(cause,2,sep="")

   ### enforce same censoring ## {{{ 

   mintime <- pmin(Wide[,time1],Wide[,time2])
   statmin <- ifelse(Wide[,time1]<Wide[,time2],Wide[,stat1],Wide[,stat2])
###
   cens.first <- which(statmin==cens.code)
   Wide[cens.first,time1] <- mintime[cens.first]
   Wide[cens.first,time2] <- mintime[cens.first]
   Wide[cens.first,stat1] <- cens.code
   Wide[cens.first,stat2] <- cens.code

###   idn <- table(data[,id]); rep(mintime,idn)
   
   long <-   fast.reshape(Wide)
   long <- long[!is.na(long[,time]) & !is.na(long[,cause]),]

   ## }}} 

   if (!is.null(entrytime)) { ## {{{ enforce same truncation
      stop("not implemented yet\n")
   entry1 <- paste(entrytime,1,sep="")
   entry2 <- paste(entrytime,2,sep="")
###   trunc.first<- which(statmin==cens.code)
   trunc.max <- pmax(Wide[,entry1],Wide[,entry2])
   Wide[,entry1] <- Wide[,entry2] <- trunc.max
### drop those that enter later
  enter.after <- ( Wide[,entry1] < Wide[,time1]) & (Wide[,entry2] < Wide[,time2])
  Wide <- Wide[enter.after,] 
  } ## }}} 

 long <- cbind(long,datao)

 return(long)
} ## }}} 

##' Recurrent events regression with terminal event 
##'
##' Fits Ghosh-Lin IPCW Cox-type model
##'
##' For Cox type model :
##' \deqn{
##' E(dN_1(t)|X) = \mu_0(t)dt exp(X^T \beta)
##' }
##' by solving Cox-type IPCW weighted score equations 
##' \deqn{
##'  \int (Z - E(t)) w(t) dN_1(t) 
##' }
##' where \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))} and
##' \deqn{E(t) = S_1(t)/S_0(t)} and \deqn{S_j(t) = \sum X_i^j w_i(t) \exp(X_i^T \beta)}.
##'
##'
##' The iid decomposition of the beta's are on the form
##' \deqn{
##' \int (Z - E ) w(t) dM_1 + \int q(s)/p(s) dM_c
##' }
##' and returned as iid.
##'
##' Events, deaths and censorings are specified via stop start structure and the Event call, that via a status vector 
##' and cause (code), censoring-codes (cens.code) and death-codes (death.code) indentifies these. See example and vignette. 
##'
##' @param formula formula with 'Event' outcome
##' @param data data frame
##' @param cause of interest (1 default)
##' @param death.code codes for death (terminating event, 2 default)
##' @param cens.code code of censoring (0 default)
##' @param cens.model for stratified Cox model without covariates
##' @param weights weights for score equations
##' @param offset offsets for model
##' @param Gc censoring weights for time argument, default is to calculate these with a Kaplan-Meier estimator, should then give G_c(T_i-)
##' @param wcomp weights for composite outcome, so when cause=c(1,3), we might have wcomp=c(1,2).
##' @param augmentation.type of augmentation when augmentation model is given 
##' @param marks  a mark value can be specified, this is vector from the data-frame where the mark value can be found at all events
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' ## data with no ties
##' library(mets)
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' hf$x <- as.numeric(hf$treatment) 
##' dd <- data.frame(treatment=levels(hf$treatment),id=1)
##'
##' gl <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,cause=1,death.code=2)
##' summary(gl)
##' head(iid(gl))
##' pgl <- predict(gl,dd,se=1); plot(pgl,se=1)
##' 
##' ## censoring stratified after treatment 
##' gls <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,
##' cause=1,death.code=2,cens.model=~strata(treatment))
##' summary(gls)
##' 
##' glss <- recreg(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
##' cause=1,death.code=2,cens.model=~strata(treatment))
##' summary(glss)
##' plot(glss)
##' 
##' ## IPCW at 2 years 
##' ll2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),data=hf,
##' cause=1,death.code=2,time=2,cens.model=~strata(treatment))
##' summary(ll2)
##' 
##' ll2i <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,
##' cause=1,death.code=2,time=2,cens.model=~strata(treatment))
##' summary(ll2i)
##' @aliases marks strataAugment scalecumhaz GLprediid recregIPCW IIDrecreg predicttime
##' @export
recreg <- function(formula,data,cause=1,death.code=2,cens.code=0,cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,marks=NULL,augmentation.type=c("lindyn.augment","lin.augment"),...)
{ ## {{{
	cl <- match.call()
	outi  <- recregBN(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,marks=marks,...)

	if (!is.null(outi$lindyn.augment)) {
		outA  <- recregBN(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,marks=marks,augmentation=outi[[augmentation.type[1]]],...)
		outi <- outA
	}

	outi$call <- cl
	return(outi)
} ## }}}

##' @export
marks <- function(x) x

recregBN <- function(formula,data=data,cause=c(1),death.code=c(2),cens.code=c(0),cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,marks=NULL,...)
{ ## {{{
cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event"))) {
        stop("Expected an 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata))  {
      ns <- grep("strata",names(des$levels))
      strata.name  <-  names(des$levels)[1]
    } else strata.name <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## no use of 
    pos.cluster <- pos.strata <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks

 res <- c(recregN01(data,X,entry,exit,status,id=id,strata=strata,offset=offset,weights=weights,strata.name=strata.name,cens.model=cens.model,cause=cause,
		   death.code=death.code,cens.code=cens.code,Gc=Gc,wcomp=wcomp,
		   case.weights=marks,...),
 list(call=cl,formula=formula,strata.pos=pos.strata,
      cluster.pos=pos.cluster,n=length(status),nevent=sum(status %in% cause)))
	colnames(res$iid) <- names(res$coef)
  res$design <- des

	class(res) <- c("recreg","phreg")
	return(res)
} ## }}}

recregN01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,weights=NULL,
		      strata.name=NULL,beta,stderr=1,method="NR",no.opt=FALSE,propodds=NULL,zero.remove=1,
		      case.weights=NULL,cause=1,death.code=2,cens.code=0,Gc=NULL,cens.model=~+1,augmentation=NULL,
		      cox.prep=TRUE,wcomp=NULL,augment.model=NULL,adm.cens.time=NULL,ftime.augment=NULL,twostage=FALSE,...) { ## {{{
	p <- ncol(X)  # setting up weights, strata, beta and so forth before the action starts
	if (missing(beta)) beta <- rep(0,p)
	if (p==0) X <- cbind(rep(0,length(exit)))
	augmentation.call <- augmentation
	if (is.null(augmentation)) augmentation <- 0
	cause.jumps <- which(status %in% cause)
	if (length(cause.jumps)>0) {
		max.jump <- max(exit[cause.jumps])
		other <- which((status %in% death.code ) & (exit< max.jump))
	} else  {
		###	warning("no jumps of cause type\n"); 
		max.jump <- max(exit)
		other <- which((status %in% death.code ) )
	}

	n <- length(exit)
	if (is.null(strata)) {
		strata <- rep(0,length(exit))
		nstrata <- 1
		strata.level <- NULL
	} else {
		strata.level <- levels(strata)
		ustrata <- sort(unique(strata))
		nstrata <- length(ustrata)
		strata.values <- ustrata
		if (is.numeric(strata))
			strata <-  fast.approx(ustrata,strata)-1
		else  {
			strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
		}
	}

	if (is.null(entry)) entry <- rep(0,length(exit))
	trunc <- (any(entry>0))
	if (is.null(offset)) offset <- rep(0,length(exit))
	if (is.null(weights)) weights <- rep(1,length(exit))
	if (is.null(case.weights)) case.weights <- rep(1,length(exit))
	if (!is.null(wcomp))  {
		if (length(wcomp)!=length(cause)) stop("weights follow the causes, and length must be the same\n"); 
		wwcomp <- rep(1,length(exit)); 
		k <- 1
		for (i in cause) { wwcomp[status==i] <- wcomp[k];k <- k+1} 
		case.weights <- case.weights* wwcomp
	}
	strata.call <- strata

	call.id <- id 
	conid <- construct_id(id,nrow(X))
	name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
	orig.id <- id

	### censoring weights constructed
	whereC <- which(status %in% cens.code)
	time <- exit
	statusC <- c(status %in% cens.code)
	data$id__ <- id  
	data$exit__ <- exit
	data$entry__ <- entry
	data$statusC <- statusC
	data$status__ <- (status %in% cause)*1
	cens.strata <- cens.nstrata <- NULL
	## lag-count to use for augment.model=~Nt+X1+X2
	data <- count.history(data,status="status__",id="id__",types=cause,multitype=TRUE)
	data$Nt <- data[,paste("Count",cause[1],sep="")]

	## augmentation model and remove intercept
	if (!is.null(augment.model)) { XXA <- model.matrix(augment.model,data)[,-1,drop=FALSE]; namesXXA <- colnames(XXA); } else XXA <- NULL

	if ((length(whereC)>0) ) { ## {{{
		if (is.null(Gc)) {
			kmt <- TRUE
			if (class(cens.model)[1]=="formula") {
				formC <- update.formula(cens.model,Surv(entry__,exit__,statusC)~ . +cluster(id__))
				cens.model <- phreg(formC,data)
			}
			if (cens.model$p>0) kmt <- FALSE
			###        Pcens.model <- predict(cens.model,data,times=exit,tminus=TRUE,individual.time=TRUE,se=FALSE,km=kmt)
			Pcens.model <- predict(cens.model,data,times=exit,individual.time=TRUE,se=FALSE,km=kmt)
			Stime <- Pcens.model$surv <- c(Pcens.model$surv)
			## strata from original data
			nCstrata <- cens.model$nstrata
			cens.strata <- Pcens.model$strata
		} else {
			formC <- NULL
			Stime <- Gc
			Pcens.model <- list(time=exit,surv=Gc,strata=0)
			nCstrata <- 1
			cens.strata <- rep(0,length(exit))
		}
	} else { 
		formC <- NULL
		Stime <- Gc  <- rep(1,length(exit))
		Pcens.model <- list(time=exit,surv=Gc,strata=0)
		nCstrata <- 1
		cens.strata <- rep(0,length(exit))
	} ## }}}

	Zcall <- cbind(status,cens.strata,Stime,strata,strata,1) ## to keep track of status and Censoring strata

	trunc <- TRUE
	if (twostage) { ## to use old structure for twostageREC
		xxg <- .Call("FastCoxPrepStrata",entry,exit,(status %in% cause)*1,X,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")
		xxg$nstrata <- nstrata
	}

	## computing terms for those experiencing another cause, need S0, S1, S2
	if ((length(other)>=1)) { ## {{{ 
		weightso <- weights[other]/Stime[other]
		timeoo <- rep(max(exit)+1,length(other))
		if (is.null(adm.cens.time))
			timeoo <- rep(max(exit)+1,length(other)) else timeoo <- adm.cens.time[other] 
		statuso <- rep(cens.code[1],length(other))
		Xo <- X[other,,drop=FALSE]
		offseto <- offset[other]
		entryo <- exit[other]
		ido <- id[other]
		stratao <- strata[other]
		type <- rep(2,length(other))
		###

		entry <- c(entry,entryo)
		exit <- c(exit,timeoo)
		status <- c(status,statuso)
		X <- rbind(X,Xo)
		id <- c(id,ido)
		strata <- c(strata,stratao)
		weights <- c(weights,weightso)
		offset <- c(offset,offseto)
		case.weights <- c(case.weights,case.weights[other])

		Zcallo <-  Zcall[other,,drop=FALSE]
		Zcallo[,6] <- 2
		Zcall <- rbind(Zcall,Zcallo)
		Zcall <- cbind(Zcall,rbind(XXA,XXA[other,,drop=FALSE]))
	} ## }}}

	stat1 <- 1*(status %in% cause)
	xx2 <- .Call("FastCoxPrepStrata",entry,exit,stat1,X,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")

	### remove all the initial zero's to save space/time, that is, the first streak of sign=-1, 
	### that is not important for risk and events
	if (zero.remove==1) { ## {{{
		first <- which(xx2$sign==1)[1]-1
		if (first >1) {
			fentry <- (1:first)
			xx2$id     <- xx2$id[-fentry]
			xx2$time   <- xx2$time[-fentry]
			xx2$status <- xx2$status[-fentry]
			xx2$sign   <- xx2$sign[-fentry]
			xx2$X      <- xx2$X[-fentry,,drop=FALSE]
			xx2$XX     <- xx2$XX[-fentry,,drop=FALSE]
			if (nrow(xx2$ZX)==nrow(xx2$X)) xx$ZX <- xx2$ZX[-fentry,,drop=FALSE]
			xx2$Z      <- xx2$Z[-fentry,,drop=FALSE]
			xx2$offset <-xx2$offset[-fentry]
			xx2$weights <-xx2$weights[-fentry]
			xx2$caseweights <-xx2$caseweights[-fentry]
			xx2$strata <-xx2$strata[-fentry]
			xx2$jumps <- xx2$jumps-first
		}
	} ## }}}

	jumps <- xx2$jumps+1
	typexx2 <- xx2$Z[,6]
	Xj <- xx2$X[jumps,,drop=FALSE]
	xx2$nstrata <- nstrata
	jumptimes <- xx2$time[jumps]
	strata1jumptimes <- xx2$strata[jumps]
	if ((length(whereC)>0)) {
		###
		rr0 <- xx2$sign*(typexx2==1)
		jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)
		strataCxx2 <- xx2$Z[,2]
		S0iC2  <-  S0iC <- rep(0,length(xx2$status))
		S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
		S0iC[jumpsC] <- 1/S0rrr[jumpsC]
		S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
  ## Gc(t) computed  along all times of combined data-set: data + [D,\infty]
        Gcxx2 <- exp(cumsumstrata(log(1 - S0iC), strataCxx2, nCstrata))
		Gstart <- rep(1,nCstrata)
		Gjumps <- Gcxx2[jumps]

		njumps <- length(jumps)
		GtsS0ooN <-   .Call("_mets_GcjumpsR",Gcxx2,c(xx2$status),strataCxx2,nCstrata,Gstart,njumps)
		Gts <- GtsS0ooN$Gcjumps
	} else {
		Gcxx2 <- rep(1,length(xx2$sign))
		strataCxx2 <- rep(0,length(xx2$sign))
		nCstrata <- 1
		Gstart <- 1
		Gjumps <- 1
		Gts <- matrix(1,length(jumps),1)
	} 

	obj <- function(pp,all=FALSE) { ## {{{

		rr2 <- c(xx2$sign*exp(xx2$X %*% pp + xx2$offset)*xx2$weights)
		rr2now <- c(xx2$sign*exp(xx2$X %*% pp + xx2$offset))

		###     S0ooN <-   .Call("_mets_S0_FG_GcR",rr2,Gcxx2,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gstart)

		###     S0ooA <-   .Call("_mets_S0_FGRN",rr2,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gts)
		S0oo <-   .Call("_mets_S0_FGRN",rr2,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gts)$S0

		f  <-  function(x) {
			ll <-   .Call("_mets_S0_FGRN",x,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gts)$S0
		}
		###	f  <-  function(x) {
		###           ll <-   .Call("_mets_S0_FG_GcR",x,Gcxx2,typexx2-1,c(xx2$status),xx2$strata,xx2$nstrata,strataCxx2,nCstrata,Gstart)$S0
		###	}

		S1oo  <- apply(xx2$X*rr2,2,f)
		S2oo  <- apply(xx2$XX*rr2,2,f)

		S0 <- c(S0oo) ## [jumps,]
		S1 <- S1oo ## [jumps,,drop=FALSE]
		S2 <- S2oo ## [jumps,,drop=FALSE]

		E <- S1/S0
		weightsJ <- xx2$weights[jumps]
		caseweightsJ <- xx2$caseweights[jumps]
		strataJ <- xx2$strata[jumps]
		rr2now <- rr2now[jumps]
		U <- (Xj-E)
		ploglik <- (log(rr2now)-log(S0))*weightsJ*caseweightsJ;

		if (!is.null(propodds)) {
			pow <- c(.Call("cumsumstrataPOR",weightsJ,S0,strataJ,nstrata,propodds,rr2now,PACKAGE="mets")$pow);
			DLam <-.Call("DLambetaR",weightsJ,S0,E,Xj,strataJ,nstrata,propodds,rr2now,PACKAGE="mets")$res;
			Dwbeta <- DLam*rr2now+(pow-1)*Xj
			DUadj  <- .Call("vecMatMat",Dwbeta,U,PACKAGE="mets")$vXZ
		}

		Ut <- caseweightsJ*weightsJ*U
		## E^2, as n x (pxp)
		Et2 <-  .Call("vecCPMat",E,PACKAGE="mets")$XX
		S2S0 <-  S2/S0
		DUt <-  -(S2S0-Et2)
		np <- length(pp)

		if (!is.null(propodds)) {
			Ut  <- pow*Ut
			S0 <- S0/pow
			DUt <- pow*DUt
			DUt <- .Call("XXMatFULL",DUt,np,PACKAGE="mets")$XXf
			if (ncol(DUt)>0) DUt <- DUt+DUadj 
			ploglik <- pow*ploglik
		}

		U  <- apply(Ut,2,sum)
		DUt <- caseweightsJ*weightsJ*DUt
		DU <- -apply(DUt,2,sum)
		np <- length(pp)
		if (ncol(DUt)!=p*p) {
			DU <- matrix(.Call("XXMatFULL",matrix(DU,nrow=1),np,PACKAGE="mets")$XXf,np,np)
		} else  DU <- matrix(DU,p,p)
		ploglik <- sum(ploglik)
		U <- U+augmentation

		out <- list(ploglik=ploglik,gradient=U,hessian=-DU,cox.prep=xx2,
			    hessiantime=DUt,weightsJ=weightsJ,caseweightsJ=caseweightsJ,
			    jumptimes=jumptimes,strata=strataJ,nstrata=nstrata,S0s=S0,
			    time=jumptimes,S0=S0/(caseweightsJ*weightsJ),S2S0=S2S0,E=E,U=Ut,X=Xj,Gjumps=Gjumps)

		nn <- length(xx2$time)
		if (all)
			return(out)
		else
			with(out,structure(-ploglik/nn, gradient=-gradient/nn, hessian=-hessian/nn))
	} ## }}}

	if (length(jumps)==0) no.opt <- TRUE
	opt <- NULL
	if (p>0) {
		if (no.opt==FALSE) {
			if (tolower(method)=="nr") {
				opt <- lava::NR(beta,obj,...)
				opt$estimate <- opt$par
			} else {
				opt <- nlm(obj,beta,...)
				opt$method <- "nlm"
			}
			cc <- opt$estimate;  names(cc) <- colnames(X)
			if (stderr==2) return(cc)
			val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
		} else val <- c(list(coef=beta),obj(beta,all=TRUE))
	} else {
		no.opt <- TRUE
		val <- obj(0,all=TRUE)
	}

	beta.s <- val$coef
	if (is.null(beta.s)) beta.s <- 0
	## getting final S's
	opt <-  val ## obj(beta.s,all=TRUE)

	if (p>0) {
		iH <- - tryCatch(solve(opt$hessian),error=function(e) matrix(0,nrow(opt$hessian),ncol(opt$hessian)) )
		opt$ihessian <- iH
		opt$no.opt <- FALSE
		dd <- IIDrecreg(xx2,opt,cause=cause,cens.code=cens.code,death.code=death.code,adm.cens=adm.cens.time) 

		Uiid <-  dd$beta.iid.naive 
		UUiid <- dd$beta.iid
		UU <- dd$MGt
		MGc <- dd$MGc

		Uiid <-  (UU+MGc) %*% iH
		UUiid <- UU %*% iH
		var1 <-  crossprod(UUiid)
		varmc <-  crossprod(Uiid)

		## compute regression augmentation for censoring martingale 
		if ((!is.null(augment.model)) & (length(whereC)>0) ) {# ## {{{ 

			CovZXstrata <- function(X,Ej,Z,Sign,strata,nstrata,jumps) 
			{
				strata  <- c(strata); Sign <- c(Sign)
				###	Ej <- Ej[jumps,,drop=FALSE]; Ej <- Ej
				ZE <- apply(Z*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
				XZ  <- .Call("vecMatMat",X,Z)$vXZ;  
				XZ <- apply(XZ*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
				EXZ  <- .Call("vecMatMat",Ej,ZE)$vXZ;  
				out <- XZ-EXZ
				return(out)
			}

			## regress U(s)=\int_s^\infty (Z-E) w(s) dM(s) on agument-model among survivors 
			## U(s) = U(\infty) - \int_0^s (Z-E) w(s)  dM(s)
			## sum (e_i - \bar e) U(s) Y_i(s)

			# construct censoring weights going along with all data, with added [D,\infty], start stop
			rr0 <- c(xx2$sign)*(typexx2==1)
			jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)
			strataCxx2 <- xx2$Z[,2]
			S0iC2  <-  S0iC <- rep(0,length(xx2$status))
			S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
			S0iC[jumpsC] <- 1/S0rrr[jumpsC]
			S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
			S0c <- c(S0rrr[jumpsC])
			## Gc(t) computed  as exp(- Cumhaz) to avoid some "0"s
			Gcj <- Gcxx2 <- exp(-cumsumstrata(S0iC,strataCxx2,nCstrata))[jumpsC]
			#

			XXA <- xx2$Z[,-(1:6),drop=FALSE]
			EXXA <- apply(XXA*c(rr0),2,revcumsumstrata,strataCxx2,nCstrata)
			EA <- EXXA[jumpsC,,drop=FALSE]/S0rrr[jumpsC]
			UA <- (XXA[jumpsC,,drop=FALSE]-EA)

			###
			E2A <- .Call("vecMatMat",EA,EA)$vXZ;  
			XX2A <- .Call("vecMatMat",XXA,XXA)$vXZ;  
			S2A <- apply(XX2A*c(rr0),2,revcumsumstrata,strataCxx2,nCstrata)
			###
			hessiant <- -(S2A[jumpsC,,drop=FALSE]/S0c-E2A)
			hesst <- hessiant

			### X fra GL + tail-death 
			rr <- c(exp(xx2$X %*% beta.s+ xx2$offset)*xx2$weights)*(typexx2==1)
			Zrr <- xx2$X*rr
			ZEdN <- apply(dd$Ut,2,revcumsumstrata,xx2$id,nid)

			covXsZ <-   CovZXstrata(XXA,EA,Zrr,rr0,strataCxx2,nCstrata,jumpsC) 
			covXsrr <-  CovZXstrata(XXA,EA,as.matrix(rr,ncol=1),rr0,strataCxx2,nCstrata,jumpsC) 
			covXsUs3 <- .Call("vecMatMat",covXsrr,dd$EdLam0[jumpsC,,drop=FALSE])$vXZ;  
			covXsUs2 <- covXsZ*dd$cumhaz[jumpsC]-covXsUs3 
			### U(infty)= UU
			Uinfiid <- UU[xx2$id+1,,drop=FALSE]
			fid <- headstrata(xx2$id,nid)
			cZEdN <- ZEdN[fid,,drop=FALSE][xx2$id+1,,drop=FALSE]-ZEdN
			Us1 <- Uinfiid-cZEdN
			covXsUs1 <- CovZXstrata(XXA,EA,Us1,rr0,strataCxx2,nCstrata,jumpsC) 
			## scale with Y_(s) because hessiantime is also scaled with this 
			covXsYs <- (covXsUs1+covXsUs2)/S0c; ## /c(cr2$S0)

			pXXA <- ncol(XXA)
			gammat <-  .Call("CubeMattime",hesst,covXsYs,pXXA,pXXA,pXXA,p,1,0,0,PACKAGE="mets")$XXX
			gammat[is.na(gammat)] <- 0
			gammat[gammat==Inf] <- 0
			namesG <- c(); for (i in 1:p) namesG <- c(namesG,paste(namesXXA,"-e",i,sep=""))
			colnames(gammat) <- namesG
			augmentt <- .Call("CubeMattime",gammat,UA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
			augment.times <- -apply(augmentt,2,sum)
			gain.times <- .Call("CubeMattime",covXsYs,gammat,pXXA,p,pXXA,p,0,1,0,PACKAGE="mets")$XXX
			gain.times <- matrix(apply(gain.times,2,sum),p,p)
			var.augment.times <-  gain.times 

			###
			time.gammat <- timeC <- xx2$time[jumpsC]
			if (is.null(ftime.augment)) {
				### simple default parabola
				maxt <- max(timeC)
				ftime <- timeC*(timeC-maxt)/maxt^2
			} else { 
				if (is.list(ftime.augment)) ftime <- ftime.augment[[1]](timeC) else ftime <- ftime.augment(timeC)
				if (length(ftime.augment)==2) {
					timepar <- ftime.augment[[2]](timeC)
					parap <- lm(gammat~-1+timepar)
					gammat <- parap$fitted.values
				}
			}
			ftime.gamma <- ftime
			varZdN <- matrix(apply(ftime^2*hesst/c(Gcj^2),2,sum),pXXA,pXXA)
			covXYdN <- matrix(apply(ftime*covXsYs/c(Gcj),2,sum),p,pXXA,byrow=TRUE) 
			gamma <- -1*.Call("CubeMattime",matrix(varZdN,nrow=1),matrix(covXYdN,nrow=1),pXXA,pXXA,p,pXXA,1,0,1,PACKAGE="mets")$XXX
			gamma <- matrix(gamma,p,pXXA,byrow=TRUE)
			gamma[is.na(gamma)] <- 0; gamma[gamma==Inf] <- 0
			colnames(gamma) <- namesXXA
			augment <- c(gamma %*% apply(ftime*UA/c(Gcj),2,sum))
			var.augment  <-  gamma %*% t(covXYdN) ###  /(nid^2)

			if (!is.null(augmentation.call)) { ## update variance when called with augmentation
				#### iid magic  for censoring augmentation martingale
				### int_0^infty gamma (e_i - ebar(s)) 1/G_c(s) dM_i^c
				S0iG <- S0i <- rep(0,length(xx2$strata))
				S0iG[jumpsC] <- ftime/(S0rrr[jumpsC]*c(Gcj))
				S0i[jumpsC] <- c(1/S0rrr[jumpsC])
				U <- E <- matrix(0,nrow(xx2$X),pXXA)
				E[jumpsC,] <- EA; 
				U[jumpsC,] <- ftime*UA/c(Gcj)
				cumhaz <- cumsumstrata(S0iG,strataCxx2,nCstrata)
				EdLam0 <- apply(E*S0iG,2,cumsumstrata,strataCxx2,nCstrata)
				MGCt <- U[,drop=FALSE]-(XXA*c(cumhaz)-EdLam0)*c(rr0)
				MGCtiid <- apply(MGCt,2,sumstrata,xx2$id,nid)
				iid.augment <-  (MGCtiid %*% t(gamma)) %*% iH

				gammasEs <- .Call("CubeMattime",gammat,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
				gammasE <- matrix(0,nrow(XXA),p)
				gammatt  <-    matrix(0,nrow(XXA),pXXA*p)
				gammasE[jumpsC,] <- gammasEs
				gammatt[jumpsC,] <- gammat
				gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,strataCxx2,nCstrata)
				gammadLam0 <-   apply(gammatt*S0i,2,cumsumstrata,strataCxx2,nCstrata)
				XgammadLam0 <- .Call("CubeMattime",gammadLam0,XXA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
				Ut <- Et <- matrix(0,nrow(XXA),p)
				Ut[jumpsC,] <- augmentt
				MGCtt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
				MGCttiid <- apply(MGCtt,2,sumstrata,xx2$id,nid)
				iid.augment.times <-  MGCttiid %*% iH
				Uiid.augment <- Uiid-iid.augment
				Uiid.augment.times <- Uiid-iid.augment.times
				## so that iid corresponds to dynamic censorig augmentation 
				Uiid <- Uiid.augment.times
				## scale with G_c(t) to compare with gamma
				gammat <- gammat * c(Gcj)

				var.augment <-  varmc  -  iH %*% var.augment %*% iH
				var.augment.times <-  varmc  +  iH %*% var.augment.times %*% iH
				varmc <- var.augment.times
				var.augment.iid <-  crossprod(Uiid.augment) 
				var.augment.times.iid <- crossprod(Uiid.augment.times) 
			} else {
				var.augment <-  var.augment.times <-  var.augment.iid <-  var.augment.times.iid <- NULL
				Uiid.augment.times <- Uiid.augment <- NULL
			}
		} else {
			iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
			var.augment.times <- var.augment <- NULL
			var.augment.times.iid <- var.augment.iid <- NULL
			Uiid.augment.times <- Uiid.augment <- NULL
			time.gammat <- gamma <- gammat <- NULL
			ftime.gamma <- NULL
			Gcj <- NULL
		}  ## }}}
		### end if (p>0)
	} else {
		iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
		var.augment.times <- var.augment <- NULL
		var.augment.times.iid <- var.augment.iid <- NULL
		Uiid.augment.times <- Uiid.augment <- NULL
		time.gammat <- gamma <- gammat <- NULL
		ftime.gamma <- NULL
		Gcj <- NULL
		varmc <- var1 <- 0; MGc <- iH <- UUiid <- Uiid <- NULL
	}
	strata <- xx2$strata[jumps]
	cumhaz <- cbind(opt$time,cumsumstrata(1/opt$S0,strata,nstrata))
	colnames(cumhaz)    <- c("time","cumhaz")

	## SE of estimator ignoring some censoring terms
	if (no.opt==FALSE & p!=0) {
		DLambeta.t <- apply(opt$E/c(opt$S0),2,cumsumstrata,strata,nstrata)
		varbetat <-   rowSums((DLambeta.t %*% iH)*DLambeta.t)
		### covariance is 0 for cox model
		### covv <-  apply(covv*DLambeta.t,1,sum) Covariance is "0" by construction
	} else varbetat <- 0
	var.cumhaz <- cumsumstrata(1/opt$S0^2,strata,nstrata)+varbetat
	se.cumhaz <- cbind(jumptimes,(var.cumhaz)^.5)
	colnames(se.cumhaz) <- c("time","se.cumhaz")

	if (!is.null(call.id)) {
		MGc <-  nameme(MGc,name.id)
		Uiid <- nameme(Uiid,name.id)
		UUiid <- nameme(UUiid,name.id)
		Uiid.augment <- nameme(Uiid.augment,name.id)
		Uiid.augment.times  <- nameme(Uiid.augment.times,name.id)
	}

	out <- list(coef=beta.s,var=varmc,se.coef=diag(varmc)^.5,iid.naive=UUiid,
   iid=Uiid,ncluster=nid,ihessian=iH,hessian=opt$hessian,var1=var1,se1.coef=diag(var1)^.5,
		    hessianttime=opt$hessianttime,
		    ploglik=opt$ploglik,gradient=opt$gradient,
		    cumhaz=cumhaz,se.cumhaz=se.cumhaz,MGciid=MGc,
		    id=orig.id,call.id=call.id,name.id=name.id,nid=nid,
		    strata.jumps=opt$strata,strata=xx2$strata,
		    nstrata=nstrata,strata.name=strata.name,strata.level=strata.level,
		    propodds=propodds,
		    S0=opt$S0,E=opt$E,S2S0=opt$S2S0,time=opt$time,Ut=opt$U,
		    jumps=jumps,exit=exit,
		    p=p,S0s=val$S0s,
		    no.opt=no.opt,##n=nrow(X),nevent=length(jumps),
		    Pcens.model=Pcens.model,Gjumps=Gjumps,
		    cens.code=cens.code,death.code=death.code,cause=cause,
		    augmentation=augmentation.call,
		    var.augment=var.augment,var.augment.times=var.augment.times,
		    var.augment.iid=var.augment.iid,var.augment.times.iid=var.augment.times.iid,
		    lin.augment=c(augment),lindyn.augment=c(augment.times),
		    iid.augment=Uiid.augment,iid.augment.times=Uiid.augment.times,
		    gamma=gamma, gamma.times=gammat, time.gammat=time.gammat,ftime.gamma=ftime.gamma,Gcj=Gcj,
		    adm.cens.time=adm.cens.time,cens.weights=Stime
		    )

	if (cox.prep) out <- c(out,list(cox.prep=xx2))
	if (twostage) out <- c(out,list(cox.prep.twostage=xxg))

	return(out)
} ## }}}

##' @export
IIDrecreg <- function(coxprep,x,time=NULL,cause=1,cens.code=0,death.code=2,fixbeta=NULL,beta.iid=NULL,adm.cens=NULL,tminus=FALSE)
{ ## {{{
	if (is.null(fixbeta)) 
	if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0

	xx2 <- coxprep
	status <- xx2$Z[,1]
	cause.jumps <- xx2$jumps+1 
	exit <- xx2$time
	max.jump <- max(exit[cause.jumps])+1
	other <- (length(which(status %in% death.code)) > 0)
	whereC <- which( (status %in% cens.code) & xx2$sign==1)
        anyC <- (length(whereC)>0)

	# construct censoring weights going along with all data, with added [D,\infty], start stop
	jumps <- xx2$jumps+1
	jumptimes <- xx2$time[jumps]
	strata1jumptimes <- xx2$strata[jumps]
	Xj <- xx2$X[jumps,,drop=FALSE]
	###
	typexx2 <- xx2$Z[,6]
	rr0 <- xx2$sign*(typexx2==1)
	jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1 & typexx2==1)

	if (anyC) {
		strataCxx2 <- xx2$Z[,2]
		S0iC2  <-  S0iC <- rep(0,length(xx2$status))
		nCstrata <- max(strataCxx2)+1
		S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
		if (length(jumpsC)>0) {
			S0iC[jumpsC] <- 1/S0rrr[jumpsC]
			S0iC2[jumpsC] <- 1/S0rrr[jumpsC]^2
		}
		## Gc(t) computed  along all times of combined data-set: data + [D,\infty] 
		Gcxx2 <- exp(cumsumstrata(log(1-S0iC),strataCxx2,nCstrata))
		Gstart <- rep(1,nCstrata)
		Gjumps <- Gcxx2[jumps]
	} else  Gcxx2 <- rep(1,length(xx2$stata))
	### all administrative censoring, or no censoring at all 
	if (!anyC) typexx2 <- 1

	### iid version given G_c when covariates are there, iid robust 
	S0i <- rep(0,length(xx2$strata))
	S0i[jumps] <- 1/x$S0
	Z <- xx2$X
	p <- ncol(x$E)
	if ( (!is.null(beta.iid)) | fixbeta==0) {
		U <- E <- matrix(0,nrow(Z),p)
		E[jumps,] <- x$E
		U[jumps,] <- x$U
		EdLam0 <- apply(E*S0i,2,cumsumstrata,xx2$strata,xx2$nstrata)
	} else U <- NULL
	cumhazA <- cumsumstratasum(S0i,xx2$strata,xx2$nstrata,type="all")
	cumhaz <- c(cumhazA$sum)

	## make indicator for times of interest for integrals up to time 
	if (!is.null(time))  btimexx <- (xx2$time<time) else btimexx <- rep(1,length(xx2$time))

	if (fixbeta==0) {
		rr <- c(xx2$sign*exp(Z %*% x$coef + xx2$offset))
		Ht <- apply(E*S0i*btimexx,2,cumsumstrata,xx2$strata,xx2$nstrata); 
	} else { Ht <- NULL; rr <- c(xx2$sign*exp(xx2$offset)) }
	rrw <- rr*c(xx2$weights)

	mid <- max(xx2$id)
	if ( (!is.null(beta.iid)) | fixbeta==0) {
		### Martingale  as a function of time and for all subjects to handle strata
		MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rrw*(typexx2==1)
		UU <- apply(MGt,2,sumstrata,xx2$id,mid+1)
	} else UU <- 0

	if (!is.null(time)) {
		## baseline
		MGAiid <- NULL
		S0i2 <- rep(0,length(xx2$strata))
		ww <- xx2$caseweights*xx2$weights
		S0i2[jumps] <- 1/(x$S0^2*ww[jumps])
		MGAiid <- matrix(0,length(S0i2),1)
		MGAiid2 <- matrix(0,length(S0i2),1)
		cumhazAA <- cumsumstrata(S0i2*btimexx,xx2$strata,xx2$nstrata)
		MGAiid <- S0i*btimexx-cumhazAA*rrw*(typexx2==1)
	} else MGAiid <- NULL

	if (other & anyC) { ## "martingale part" for type-2 after T ## {{{
	   ## tail part with \int (Z_i-E) w_i(t) dM_i = \int_D_i^\tau (Z_i-E) Gc(t) dM_i/Gc(D_i) 
	   rrw2 <- rrw*(typexx2==2)
	   GdL <- c(cumsum2strata(Gcxx2,S0i,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)$res)
	   fff <- function(x) {
		cx  <- cumsum2strata(Gcxx2,x*S0i,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)$res
		return(cx)
	   }
           if ((!is.null(beta.iid)) | fixbeta==0) EGdL  <- apply(E,2,fff)

           if ( ((!is.null(beta.iid)) | fixbeta==0)  ) {
		MGt2  <- -(Z*GdL-EGdL)*rrw2
		UU2 <- apply(MGt2,2,sumstrata,xx2$id,mid+1)
		UU  <-  UU+UU2
	   }

	   dstrata <- mystrata(data.frame(strataCxx2,xx2$strata))
	   ndstrata <- attr(dstrata,"nlevel")
	   lastt <- tailstrata(dstrata-1,ndstrata)

	   if (!is.null(time) ) {
	   HBt <- cumsum2strata(Gcxx2,S0i2*btimexx,strataCxx2,nCstrata,xx2$strata,xx2$nstrata,Gstart)
	   HBtinf <- HBt$res[lastt][dstrata]-HBt$res
###	   if ( is.null(adm.cens)) {
		## baseline
		MGAiid2 <- -HBt$res*c(rrw2)
		MGAiid <- MGAiid+MGAiid2
###		MGAiid <- apply(MGAiid,2,sumstrata,xx2$id,mid+1)
###	   }
	   }
	   if ( ((!is.null(beta.iid)) | fixbeta==0) ) {
		Htinf <- GdL[lastt][dstrata]-GdL
		ff <- function(x) x[lastt][dstrata]-x
		EHtinf  <- apply(EGdL,2,ff)
	   }
	} # ## }}}

	if (!is.null(time) )  MGAiid <- apply(MGAiid,2,sumstrata,xx2$id,mid+1)

	## censoring terms for influene functions 
	if (other & anyC) { ##  ## {{{
		### Censoring adjustment for jumps of other type but only for KM-case 
		### first time we see them with type2 event 

		if (is.null(adm.cens)) {
                   rrw2j <- -c(rrw2*(xx2$sign==-1))
		   Xos <- Z*rrw2j
		   rrsx <- cumsumstrata(rrw2j,strataCxx2,nCstrata)
		   Xos <- apply(Xos,2,cumsumstrata,strataCxx2,nCstrata)

		   if ( (!is.null(beta.iid)) | fixbeta==0) q2 <- (Xos*c(Htinf)-EHtinf*c(rrsx))
		   if (!is.null(time))  qB2 <- rrsx*c(HBtinf) 
		} else {
                   lastid <- tailstrata(xx2$id,mid+1)	
	 	   GdLast <- GdL[lastid][xx2$id+1]
		   EGdLast <- EGdL[lastid,,drop=FALSE][xx2$id+1,,drop=FALSE]
                        
		   GadmXE2 <- apply(GdLast*Z*rrw2,2,revcumsumstrata,strataCxx2,nCstrata)
		   XE2 <- apply(Z*rrw2,2,revcumsumstrata,strataCxx2,nCstrata)
		   EGadmRR2 <- apply(EGdLast*rrw2,2,revcumsumstrata,strataCxx2,nCstrata)
		   RR2 <- revcumsumstrata(rrw2,strataCxx2,nCstrata)

		   q11 <- GadmXE2-XE2*GdL
		   q21 <- EGadmRR2-c(RR2)*EGdL
                   ###
		   qq <- q11-q21

                   if ( (!is.null(beta.iid)) | fixbeta==0) q2 <- qq
                   if (!is.null(time))  {
		       HBlast <- HBt$res[lastid][xx2$id+1]
	               HBadmRR2 <- revcumsumstrata(HBlast*rrw2,strataCxx2,nCstrata)
		       qB2 <- (HBadmRR2-HBt$res*RR2)
                   }

              }

		sss <- headstrata(dstrata-1,ndstrata)
		fff <- function(x) {
		 gtstart <- x[sss]
		 cx  <- cumsum2strata(x,S0iC2,dstrata-1,ndstrata,strataCxx2,nCstrata,gtstart)$res
		 return(cx)
		}

		### Martingale  as a function of time and for all subjects to handle strata
		if ( (!is.null(beta.iid)) | fixbeta==0) {
			EdLam0q2 <- apply(q2,2,fff)
			MGc <- q2*S0iC-EdLam0q2*c(xx2$sign)*(typexx2==1)
			MGc <- apply(MGc,2,sumstrata,xx2$id,mid+1)
		}
		if (!is.null(time) ) {
			EBdLam0q2 <- apply(qB2,2,fff)
			MGBc <- qB2*S0iC-EBdLam0q2*c(xx2$sign)*(typexx2==1)
			MGBc <- apply(MGBc,2,sumstrata,xx2$id,mid+1)
		}

	} else { MGc <- 0; MGBc <- 0}   ## }}}

	if (!is.null(time) & anyC) { MGAiid <- MGAiid+MGBc }  

	if ( (!is.null(beta.iid)) | fixbeta==0) {
		Uiid <-  (UU+MGc) %*% x$ihessian
		Uiid.naive <-  (UU) %*% x$ihessian
	} else {
		Uiid <- beta.iid
		Uiid.naive <- NULL
	} 

	if ( (!is.null(beta.iid)) | fixbeta==0) { #
		Htlast <- tailstrata(xx2$strata,xx2$nstrata)
		HtS <- Ht[Htlast,,drop=FALSE]
	} #

	## sum after id's within strata and order 
	if (!is.null(time))  {
		MGAiids <- c()
		cumhaz.time <- c()
		sus <- sort(unique(xx2$strata))
		fid <- headstrata(xx2$id,mid+1)
		wis <- xx2$strata[fid]

		for (i in sus)  { 
			ws <- 1*(wis==i)
			##
			cumhazs <- rbind(0,x$cumhaz[x$strata[x$jumps]==i,])
			cumhaz.time <- c(cumhaz.time,cpred(cumhazs,time,tminus=tminus)[,2])

			if (fixbeta==0) {
				UU <-  apply(HtS[i+1,]*t(Uiid),2,sum)
				MGAiidl <- ws*MGAiid - UU
			} else MGAiidl <- ws*MGAiid 
			MGAiids <- cbind(MGAiids,MGAiidl)
		}
		colnames(MGAiids) <- paste("strata",sus,sep="")
		names(cumhaz.time) <- paste("strata",sus,sep="")
	} else { sus <- MGAiids <- cumhaz.time <- NULL }

	if (!is.null(x$call.id)) {
		MGAiids <- nameme(MGAiids,x$name.id)
		Uiid    <- nameme(Uiid,x$name.id)
	}

	if (inherits(x,c("cifreg","recreg"))) {
		out <- list(time=time,base.iid=MGAiids,nstrata=xx2$nstrata, beta.iid=Uiid,
			    strata.call=x$strata.call,id=xx2$id,call.id=x$call.id,
			    coef=coef(x),cumhaz=x$cumhaz,cumhaz.strata=x$strata[x$jumps],
			    cumhaz.time=cumhaz.time,strata.time=sus,
			    nstrata=x$nstrata,strata.name=x$strata.name,strata.level=x$strata.level,
			    formula=x$formula,Ut=U)
	} else {
		out <- list(time=time,base.iid=MGAiid,id=xx2$id,beta.iid=Uiid,beta.iid.naive=Uiid.naive, MGt=UU,MGc=MGc,Ut=U,EdLam0=EdLam0,cumhaz=cumhaz)
	}
	return(out)
}  ## }}}

##' @export
iidBaseline.recreg <- function(object,time=NULL,ft=NULL,fixbeta=NULL,beta.iid=object$iid,tminus=FALSE,...)
{ ## {{{
	if (is.null(object$cox.prep)) stop("must call cifreg/recreg with cox.prep=TRUE\n")
	out <- IIDrecreg(object$cox.prep,object,time=time,fixbeta=fixbeta,beta.iid=beta.iid,adm.cens=object$adm.cens,tminus=tminus,...)
	out$design <- object$design
	return(out)
}  ## }}}

##' @export
GLprediid <- function(...)
{
	out <- FGprediid(...,model="GL")
	return(out)
}

##' @export
IC.recreg <- function(x, time = NULL, ..., wwww) {
    if (!is.null(time)) {
        iid <- IC.phreg(x, time = time, ...)
        return(iid)
    }
    if (is.null(x$iid)) stop("supply 'time' argument")
    res <- with(x, iid * NROW(iid))
    attr(res, "coef") <- coef(x)
    return(res)
}

##' @export
plot.recreg <- function(x,se=FALSE,ylab=NULL,...) { #
	if (inherits(x,"recreg") & is.null(ylab)) ylab <- "Mean number"
	if (!se) baseplot(x,se=se,ylab=ylab,...)
	else {
		warning("Standard errors approximative (but too small), use predict and type='cumhaz' \n")
		baseplot(x,se=se,ylab=ylab,...)
	}
} #

##' @export
predict.recreg <- function(object,newdata,se=FALSE,times=NULL,np=50,...) { #
	if (!se) out <- predict.phreg(object,newdata,se=se,times=times,...)
	else {
		out <- predictrecreg(object,newdata,times=times,np=np,...)
	}
	class(out) <- c("predictrecreg",class(object)[1])
	return(out)
} #

##' @export
summary.predictrecreg <- function(object,times=NULL,strata=NULL,type=c("cif","cumhaz","surv")[2],np=10,...) { ## {{{
	if (!is.null(times)) {
		indexcol <- predictCumhaz(c(0,object$times),times,return.index=TRUE)
	} else {
		## all predictions
		nl <- length(object$times)+1 
		indexcol  <- seq(1,nl,length=np)
		times <- c(0,object$times)[indexcol]
		print("summary.predictrecreg")
		print(times)
	}

	out <- object[[type[1]]]
	nlower <- paste(type[1],".lower",sep="")
	nupper <- paste(type[1],".upper",sep="")
	lower <- object[[nlower]]
	upper <- object[[nupper]]
	nse <-  paste("se.",type[1],sep="")
	se.out  <- object[[paste("se.",type[1],sep="")]]
	if (type[1]=="surv") {
		out <- cbind(1,out) 
		if (!is.null(se.out)) se.out <- cbind(0,se.out)
		if (!is.null(lower)) lower <- cbind(1,lower) 
		if (!is.null(upper)) upper <- cbind(1,upper) 
	} else { 
		out <- cbind(0,out)
		if (!is.null(se.out)) se.out <- cbind(0,se.out)
		if (!is.null(lower)) lower <- cbind(0,lower) 
		if (!is.null(upper)) upper <- cbind(0,upper) 
	}
	if (length(lower)>1) { se <- 1; } else  { se <- 0; lower <- upper <- NULL}

	if (!is.null(lower)) ret <- list(pred=out[,indexcol],se.pred=se.out[,indexcol],lower=lower[,indexcol],upper=upper[,indexcol],times=times)
	else  ret <- list(pred=out[,indexcol],times=times)
	rownames(ret) <- NULL
	###ret$strata <- object$strata; ret$X <- object$X; ret$RR <- object$RR
	class(ret) <- "summarypredictrecreg"
	return(ret)
} ## }}}


##' @export
plot.predictrecreg <- function(x,se=FALSE,ylab=NULL,type="cumhaz",...)
{ ## {{{
	if (inherits(x,"predictrecreg") & is.null(ylab)) ylab <- "Mean number"
	plotpredictphreg(x,se=se,ylab=ylab,type=type[1],...)
} ## }}}

##' @export
predictrecreg <- function(x,newdata,times=NULL,individual.time=FALSE,tminus=FALSE,conf.type="log",conf.int=0.95,np=50,...)
{ ## {{{
	if (!inherits(x,c("cifreg","recreg","phreg")))  stop("only for phreg/recreg/cifreg models\n")
	se <- TRUE

	if (is.null(times))  {
		if (is.null(np)) times <- x$cumhaz[,1] else 
			times <- quantile(x$cumhaz[,1],probs=seq(0,1,length=np))
	} 
	des <- readPhreg(x,newdata)

	if (x$p>0)  {
		RRj <- RR <- c(exp(des$X %*% x$coef))
		Xj <- X <- RR*des$X
	} else { RRj <- RR <- rep(1,length(des$strata)); Xj <- NULL}

	surv <- surv.upper <- surv.lower <- cift <- cif.upper <- cif.lower <- cumhaz <- se.cumhaz <- base.lower <- base.upper <- se.cif <- se.surv <- c()
	j <- 1
	for (tt in times) {
		bt <- iidBaseline(x,time=tt,tminus=tminus)
		covv <- crossprod(with(bt,cbind(base.iid,beta.iid)))
		if (individual.time)  {
			baset <- bt$cumhaz.time[des$strata[j]+1]
			Xbase <- 1*outer(des$strata[j],0:(bt$nstrata-1),"==")
			RRj <- RR[j]; 
			if (x$p>0) Xj <- X[j,,drop=FALSE]*baset else Xj <- NULL
		} else {  
			baset <- bt$cumhaz.time[des$strata+1]
			Xbase <- 1*outer(des$strata,0:(bt$nstrata-1),"==")
			if (x$p>0) Xj <- X*baset else Xj <- NULL
			RRj <- RR
		}
		j <- j+1
		Xall <- cbind(RRj*Xbase,Xj)
		###   
		seLamt <- apply((Xall %*% covv)* Xall,1,sum)^.5
		Lamt <- baset*RRj
		F1t <- 1-exp(-Lamt)
		St <- exp(-Lamt)
		seF1t <- St*seLamt
		seSt <- St*seLamt
		###   
		ciLam <- conftype(Lamt,seLamt,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
		ciSt <- conftype(St,seSt,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)
		ciF1 <- conftype(F1t,seF1t,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)

		surv <- cbind(surv,St)
		se.surv <- cbind(se.surv,seSt)
		se.cif <- cbind(se.cif,seF1t)
		surv.upper <- cbind(surv.upper,ciSt$upper)
		surv.lower <- cbind(surv.lower,ciSt$lower)
		cift <- cbind(cift,F1t)
		cif.upper <- cbind(cif.upper,ciF1$upper)
		cif.lower <- cbind(cif.lower,ciF1$lower)
		cumhaz <- cbind(cumhaz,Lamt)
		se.cumhaz <- cbind(se.cumhaz,seLamt)
		base.upper  <- cbind(base.upper,ciLam$upper)
		base.lower  <- cbind(base.lower,ciLam$lower)
	}

	out <- list(times=times,surv=surv,surv.upper=surv.upper,surv.lower=surv.lower,
		    cumhaz=cumhaz,se.cumhaz=se.cumhaz,cif=cift,cif.upper=cif.upper,cif.lower=cif.lower,
		    cumhaz.upper=base.upper,cumhaz.lower=base.lower,strata=des$strata,X=des$X,RR=RR,
		    se.cif=se.cif,se.surv=se.surv)

	return(out)
} ## }}}

##' @export
recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
		       cens.model=~1,km=TRUE,times=NULL,beta=NULL,offset=NULL,type=c("II","I"),
		       marks=NULL,weights=NULL,model=c("exp","lin"),no.opt=FALSE,augmentation=NULL,method="nr",se=TRUE,...)
{ ## {{{
   ## method=c("incIPCW","IPCW","rate")
   estimator=c("incIPCW")
   cl <- match.call()
   ## {{{ reading design 
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","marks"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster

  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  if (is.null(marks)) marks <- rep(1,length(id))
# }}}

	### setting up with artificial names
	data$status__ <-  status 
	data$id__ <-  id
	## lave Countcause
	data <- count.history(data,status="status__",id="id__",types=cause,multitype=TRUE)
	data$Count1__ <- data[,paste("Count",cause[1],sep="")]
	data$death__ <- (status %in% death.code)*1
	data$entry__ <- entry 
	data$exit__ <- exit 
	data$marks__ <- marks 
	statusC <- data$statusC__ <- (status %in% cens.code)*1
	data$status__cause <- (status %in% cause)*1
	data$rid__ <- revcumsumstrata(rep(1,length(entry)),id,nid)
	dexit <- exit
	dstatus <- status
	## to define properly 
	Dtime <- NULL

	formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ .+cluster(id__))
	cr <- phreg(formC,data=data,no.opt=TRUE,no.var=1)
	whereC <- which(status %in% cens.code)

	if (length(whereC)>0) {
		### censoring weights
		strat <- cr$strata[cr$jumps]
		x <- cr
		xx <- x$cox.prep
		S0i2 <- S0i <- rep(0,length(xx$strata))
		S0i[xx$jumps+1] <-  1/x$S0
		S0i2[xx$jumps+1] <- 1/x$S0^2
		## survival at t- to also work in competing risks situation
		if (!km) { 
			cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
			St      <- exp(-cumhazD)
		} else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
	} else  St <- rep(1,length(exit))
	Gc <- St

	###formD <- as.formula(Surv(entry__,exit__,death__)~cluster(id__))
	form1L <- as.formula(Surv(entry__,exit__,status__cause)~Count1__+death__+statusC__+cluster(id__))
	xr <- phreg(form1L,data=data,no.opt=TRUE,no.var=1,Z=matrix(data$marks__,ncol=1))
	###  xr0 <- phreg(form1,data=data,no.opt=TRUE)
	###  dr <- phreg(formD,data=data,no.opt=TRUE,no.var=1)
	###  clgl  <- recurrentMarginal(xr0,dr)
	###  plot(clgl)

	####  First partitioned estimator everywhere n^-1 sum_i \int_0^t m_i(s)/G_c(s) Y_i(s) I(D_i > s) dN_i(s) 
	x <- xr
	xx <- xr$cox.prep
	marksxx <- xx$Z[,1]
	jump1 <- xx$jumps+1
	timeJ <- xx$time[jump1]
	strataN1J <- xx$strata[jump1]
	### Partitioned estimator , same as Ghosh-Lin+Lawless-Cook estimator
	cumhazP <- c(cumsum(marksxx[jump1]/Gc[jump1])/nid)
	cumhazP <- cbind(timeJ,cumhazP)

	if (is.null(times)) stop("time for recurrent events regression must be given\n")

	### setting up regression setting with Y(t) =\int_0^t 1/G(s) dN_i(s)
	if (estimator[1]=="incIPCW") 
		Ydata <- Y <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times)/Gc,xx$id,nid)
	else if (estimator[1]=="IPCW")  {
		obs <- (exit<=time & (!statusC)) | (exit>=time)/Gc
		Ydata <- Y <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times),xx$id,nid)*obs
	} else {
		obs <- (exit<=time & (!statusC)) | (exit>=time)/Gc
		NtD <- sumstrata(marksxx*(xx$status!=0)*(xx$time<times),xx$id,nid)
		Ydata <- Y <- obs*NtD/pmin(times,Dtime)
	}
	nevent <- sum((xx$status!=0)*(xx$time<times))

	lastrecord <- which(data$rid__==1)
	##  1 record per subject 
	Ydata <- Y 
	###
	if (is.null(offset)) offset <- rep(0, length(exit))
	if (is.null(weights)) weights <- rep(1, length(exit))
	###  
	Xorig <- X <- as.matrix(X)
	px <- ncol(X)
	if (is.null(augmentation))  augmentation=rep(0,px)

	## order after id, similar to Y
	oid <- order(id[lastrecord])
	Xdata <- X <- X[lastrecord,,drop=FALSE][oid,,drop=FALSE]
	offset <- offset[lastrecord][oid]
	weights <- weights[lastrecord][oid]
	status <- status[lastrecord][oid]
	exit <- exit[lastrecord][oid]
	idR <- id[lastrecord][oid]
	X2 <- .Call("vecMatMat", X, X)$vXZ
	ph <- 1
	if (is.null(beta)) beta <- rep(0,ncol(X))
	## take iid vession of data 
	dataiid <- data[lastrecord,][oid,]

	if (type[1]=="II") { 
		Gcdata <- suppressWarnings(predict(cr,data,times=dexit,individual.time=TRUE,se=FALSE,km=km,tminus=TRUE)$surv)
		Gcdata[Gcdata<0.000001] <- 0.00001
		Hst <- Y[data$id__ +1]-cumsumstratasum((dexit<times)*marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		#data$Hst <- cumsumstratasum(marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		data$Hst <- Hst
		formC <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
		desform <- update.formula(cens.model,as.formula(paste("~ Hst+ . + cluster(id__)")))
		formC[[3]] <- desform[[2]]

		resC <- phreg(formC,data=data,no.opt=TRUE,no.var=1,Z=Xorig)
		xx <- resC$cox.prep
		Xt <- xx$Z
		S0i2 <- S0i <- rep(0, length(xx$strata))
		S0i[xx$jumps+1] <- 1/resC$S0
		S0i2[xx$jumps+1] <- 1/resC$S0^2
		E <- matrix(0, nrow(xx$X), 1)
		E[xx$jumps+1,] <- resC$E*((resC$jumptimes<times)*1)
		btime <- c(1 * (xx$time < times))
		EdLam0 <- apply(E*c(S0i)*btime,2,cumsumstrata,xx$strata,xx$nstrata)

		mid <- max(idR)
		MGt <- Xt*c(E[, drop = FALSE] - EdLam0 * c(xx$sign))*c(xx$weights)
		MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
		augmentation <- augmentation+apply(MGtiid,2,sum)
		###        
		EXt  <-  apply(Xt*c(xx$sign),2,revcumsumstrata,xx$strata,xx$nstrata)
		IEXhYtdLam0 <- apply(EXt*c(E)*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
		U <- matrix(0,nrow(xx$X),ncol(X))
		U[xx$jumps+1,] <- (resC$jumptimes<times)*E[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)
		MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0*c(xx$sign))*c(xx$weights)
		###
		MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
		MGCiid2 <- MGtiid-MGCiid2
	}  else MGCiid2 <- 0   
	#

	obj <- function(pp, all = FALSE) { ## {{{
		lp <- c(X %*% pp + offset)
		if (model[1] == "exp") p <- exp(lp) else p <- lp
		if (model[1] == "dexp") p <- exp(lp) 
		if (model[1] == "dexp") {
			Dlogl <- weights * X *p * c(Y - p)
			D2logl <- c(weights) *p^2* X2
		} else {
			Dlogl <- weights *  X * c(Y - p)
			if (model[1]=="exp") D2logl <- c(weights) * c(p)* X2 else D2logl <- c(weights) * X2
		}
		if (model[1] == "exp") ploglik <- 0 else ploglik <- sum(weights * (Y - p)^2)
		D2log <- apply(D2logl, 2, sum)
		gradient <- apply(Dlogl, 2, sum) + augmentation
		hessian <- matrix(D2log, length(pp), length(pp))

		if (all) {
			ploglik <- sum(weights * (Y - p)^2)
			ihess <- solve(hessian)
			beta.iid <- Dlogl %*% ihess
			beta.iid <- apply(beta.iid, 2, sumstrata, idR, max(id) + 1)
			robvar <- crossprod(beta.iid)
			val <- list(par = pp, ploglik = ploglik, gradient = gradient,
				    hessian = hessian, ihessian = ihess, id = idR,
				    Dlogl = Dlogl, iid = beta.iid, robvar = robvar,
				    var = robvar, se.robust = diag(robvar)^0.5)
			return(val)
		}
		structure(-ploglik, gradient = -gradient, hessian = hessian)
	} ## }}}

	## setting default for NR 
	dots <- list(...)
	if (length(dots)==0) {
		if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
		else control <- NULL
	} else control <- dots[[1]]

	p <- ncol(X)
	opt <- NULL
	if (p > 0) {
		if (no.opt == FALSE) {
			if (tolower(method) == "nr") {
				tim <- system.time(opt <- lava::NR(beta, obj,control=control))
				opt$timing <- tim
				opt$estimate <- opt$par
			}
			else {
				opt <- nlm(obj, beta, ...)
				opt$method <- "nlm"
			}
			cc <- opt$estimate
			###            if (!se) return(cc)
			val <- c(list(coef = cc), obj(opt$estimate, all = TRUE))
		}
		else val <- c(list(coef = beta), obj(beta, all = TRUE))
	}
	else {
		val <- obj(0, all = TRUE)
	}
	if (length(val$coef) == length(colnames(X))) names(val$coef) <- colnames(X)

	val <- c(val, list(times = times, Y=Y, ncluster=nid, nevent=nevent,n=length(exit),X=X))


	if (se) {
		Gcdata <- suppressWarnings(predict(cr,data,times=dexit,individual.time=TRUE,se=FALSE,km=km,tminus=TRUE)$surv)
		Gcdata[Gcdata<0.000001] <- 0.00001
		## check data sorted in dexit 
		if (type[1]!="II") Hst <- Y[data$id__ +1]-cumsumstratasum((dexit<times)*marks*(dstatus %in% cause)/Gcdata,data$id__,nid)$lagsum
		data$Hst <- Hst
		if (model[1]=="dexp") HstX <-c(exp(as.matrix(Xorig) %*% val$coef))*Xorig*c(data$Hst) else HstX <- Xorig*c(data$Hst) 
		ccn <- paste("nn__nn",1:ncol(Xorig),sep="")
		colnames(HstX) <- ccn
		nncovs <- c()
		for (i in 1:ncol(Xorig)) nncovs <- c(paste(nncovs,paste("+",ccn[i],sep="")))
		formC <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
		desform <- update.formula(cens.model,as.formula(paste("~",nncovs,"+ . + cluster(id__)")))
		formC[[3]] <- desform[[2]]

		data <- cbind(data,HstX)
		resC <- phreg(formC,data=data,no.opt=TRUE,no.var=1)
		xx <- resC$cox.prep
		S0i2 <- S0i <- rep(0, length(xx$strata))
		S0i[xx$jumps + 1] <- 1/resC$S0
		S0i2[xx$jumps + 1] <- 1/resC$S0^2
		E <- U <- matrix(0, nrow(xx$X), ncol(X))
		E[xx$jumps + 1, ] <- resC$E*((resC$jumptimes < times)*1)
		btime <- c(1 * (xx$time < times))
		EdLam0 <- apply(E*c(S0i)*btime,2,cumsumstrata,xx$strata,xx$nstrata)
		MGt <- (E[, drop = FALSE] - EdLam0 * c(xx$sign) )*c(xx$weights)
		MGCiid <- apply(MGt, 2, sumstrata, xx$id, max(id) + 1)
		#MGCiid <- MGCiid-MGCiid2
		MGCiid <- MGCiid+MGCiid2
	} else  MGCiid <- 0 


	val$call <- cl
	val$formula <- formula
	val$model <- model[1]
	val$model.type <- model[1]
	val$Y <- Ydata
	val$X <- Xdata
	val$id <- orig.id
	val$call.id <- call.id
	val$nid <- nid
	val$name.id <- name.id
	val$iid.naive <- val$iid
	val$naive.var <- val$var
	if (se) { 
		val$MGCiid <- MGCiid
		MGCiid <- MGCiid %*% val$ihessian 
		val$iid <- val$iid + MGCiid
	} else val$MGCiid <- MGCiid 
	if (is.matrix(val$iid)) 
		if (length(name.id)==nrow(val$iid)) {
			rownames(val$iid) <- name.id
			oid <- order(name.id)  
			val$iid <- val$iid[oid,,drop=FALSE]
		}
	if (is.matrix(val$MGCiid))  {
		if (length(name.id)==nrow(val$MGCiid)) rownames(val$MGCiid) <- name.id
		val$MGCiid <- val$MGCiid[oid,,drop=FALSE]
	}

	robvar <- crossprod(val$iid)
	val$var <- val$robvar <- robvar
	val$se.robust <- diag(robvar)^0.5
	val$se.coef <- diag(val$var)^0.5
	val$cens.code <- cens.code
	val$cause <- cause
	val$death.code <- death.code
	val$model.estimator <- estimator
	val$augmentation <- augmentation
	val$type <- type
	val$cumhazP <- cumhazP
	val$nevent <- nevent
	val$design <- des
	class(val) <- c("binreg", "resmean")
	return(val)
} ## }}}

strataAugment <- survival::strata

##' Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure 
##'
##' Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure. type=3 will generate
##' Cox/Cox twostage mode, type=2 will generate Ghosh-Lin/Cox model. 
##' If the variance is var.z=0, then generates data without any dependence or frailty. If model="twostage" then default is to generate data from Ghosh-Lin/Cox model, and
##' if type=3 then will generate data with marginal Cox models (Cox/Cox). 
##' Simulation based on linear aproximation of hazard for two-stage models based on grid on time-scale. Must be sufficientyly fine. 
##'
##' Must specify baselines of recurrent events and terminal event and possible covariate effects.
##'
##' @param n number of id's 
##' @param base1 baseline for cox/ghosh-lin models
##' @param drcumhaz baseline for terminal event
##' @param var.z variance of gamma frailty 
##' @param r1 relative risk term for baseline 
##' @param rd relative risk term for terminal event 
##' @param rc relative risk term for censorings
##' @param fz possible transformation (function) of frailty term 
##' @param fdz possible transformation (function) of frailty term for death 
##' @param model twostage, frailty, shared (partly shared two-stage model)
##' @param type type of simulation, default is decided based on model
##' @param cens to right censor
##' @param share to fit patly shared random effects model
##' @param cens censoring rate for exponential censoring
##' @param nmin default 100, at least nmin or number of rows of the two-baselines max(nmin,nrow(base1),nrow(drcumhaz)) points in time-grid from 0 to maximum time for base1
##' @param nmax default 1000, at most nmax points in time-grid 
##' @references 
##' Scheike (2025), Two-stage recurrent events random effects models, LIDA, to appear
##' @export
simGLcox <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
		     model=c("twostage","frailty","shared"),type=NULL,share=1,cens=NULL,nmin=100,nmax=1000)
{ ## {{{
	## setting up baselines for simulations 
	maxt <- tail(base1[,1],1)
	base1 <- as.matrix(base1); drcumhaz <- as.matrix(drcumhaz)
	nmin <- max(nrow(base1),nrow(drcumhaz),nmin)
	nmin <- min(nmax,nmin)
	seqt <- seq(from=0,to=maxt,length.out=nmin)
	if (base1[1,1]!=0) base1 <- rbind(0,base1) 
	if (drcumhaz[1,1]!=0) drcumhaz <- rbind(0,drcumhaz) 
	base1 <- cbind(seqt, lin.approx(seqt,base1))
	cumD <- cbind(seqt, lin.approx(seqt,drcumhaz))
	###
	St <- exp(-cumD[,2])
	Stm <- cbind(base1[,1],St)
	###
	dbase1 <- diff(c(0,base1[,2]))
	dcum <- cbind(base1[,1],dbase1)
	maxtime <- tail(base1[,1],1)

	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	fz.orig <- fz
	if (is.null(fz)) fz <- function(x) x

	if (var.z[1]>0) {
		z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
		if (share<1) { 
			z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
			z <- z+z2
		} 
		fzz <- fz(z1)
		if (share<1) fzz <- fzz/share; 
		mza <- mean(fzz)
		if (n<10000 & (!is.null(fz.orig))) {
			zl <- rgamma(100000,share/var.z[1])*var.z[1] 
			fzl <- fz(z)
			mza <- mean(fzl)
		} 
	}  else fzz <- z <- z1 <- rep(1,n)

	if (var.z[1]==0) model <- "frailty"
	if (is.null(type)) 
		if (model[1]=="twostage") type <- 2 else type <- 1
	## for frailty setting we also consider any function of z 
	if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}

	## survival censoring given X, Z, either twostage or frailty-model 
	if (type>=2) stype <- 2 else stype <- 1
	if (var.z[1]==0) stype <- 1
	### dd <- .Call("_mets_simSurvZ",as.matrix(rbind(c(0,1),Stm)),rd,z,var.z[1],stype)
	dd <- .Call("_mets_simSurvZ",as.matrix(rbind(Stm)),rd,z,var.z[1],stype)
	dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
	if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
	dd$status <- ifelse(dd$time<cens,dd$status,0)
	dd$time <- pmin(dd$time,cens)

	## to avoid R check error
	reverseCountid  <-  death  <- NULL

	if (model[1]=="multiplicative") {
		## other random effect 
		z2 <- rgamma(n,share/var.z[2])*var.z[2] 
		fzz <- z1*z2
		type <- 3
	}
	## type=2 draw recurrent process given X,Z with rate:
	##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
	## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
	## type=3, observed hazards on Cox form among survivors
	## twostage shared<1: W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
	## twostage share=1: or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	## multiplicatve:       W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	dcum <- cbind(base1[,1],dbase1)
	### ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	ll <- .Call("_mets_simGL",as.matrix(dcum),c(St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	colnames(ll) <- c("id","start","stop","death")
	ll <- data.frame(ll)
	ll$death <- dd$status[ll$id+1]
	## add frailty to data for possible validation
	ll$z <- z1[ll$id+1]
	ll$fz <- fzz[ll$id+1]
	## add counts of id
	ids <- countID(ll)
	ll <- cbind(ll,ids[,c(2,4,5)]); 
	ll$status <- 1; 
	ll <- dtransform(ll,status=0,reverseCountid==1)
	ll$statusD <- ll$status
	ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)

	attr(ll,"base1events") <- base1
	attr(ll,"deathcumbase") <- cumD
	return(ll)
} ## }}}


simGLcoxC <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
		      model=c("twostage","frailty","shared","multiplicative"),type=NULL,share=1,cens=NULL,nmax=200,by=1)
{ ## {{{
	## setting up baselines for simulations 
	maxt <- tail(base1[,1],1)
	seqt <- seq(by,maxt,by=by)
	base1 <- predictCumhaz(rbind(0, as.matrix(base1)), seqt)
	cumD <- predictCumhaz(rbind(0, as.matrix(drcumhaz)), seqt)
	###
	St <- 1-cumD[,2]
	Stm <- cbind(base1[,1],St)
	###
	dbase1 <- diff(c(0,base1[,2]))
	dcum <- cbind(base1[,1],dbase1)
	maxtime <- tail(base1[,1],1)

	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	fz.orig <- fz
	if (is.null(fz)) fz <- function(x) x

	if (var.z[1]>0) {
		z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
		if (share<1) { 
			z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
			z <- z+z2
		} 
		fzz <- fz(z1)
		if (share<1) fzz <- fzz/share; 
		mza <- mean(fzz)
		if (n<10000 & (!is.null(fz.orig))) {
			zl <- rgamma(100000,share/var.z[1])*var.z[1] 
			fzl <- fz(z)
			mza <- mean(fzl)
		} 
	}  else fzz <- z <- z1 <- rep(1,n)

	if (var.z[1]==0) model <- "frailty"
	if (is.null(type)) 
		if (model[1]=="twostage") type <- 2 else type <- 1
	## for frailty setting we also consider any function of z 
	if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}

	## survival censoring given X, Z, either twostage or frailty-model 
	if (type>=2) stype <- 2 else stype <- 1
	cumHazS <- -log(St)
	dd <- rchaz(cumHazS,rd)
	dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
	if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
	dd$status <- ifelse(dd$time<cens,dd$status,0)
	dd$time <- pmin(dd$time,cens)

	## to avoid R check error
	reverseCountid  <-  death  <- NULL
	## type=2 draw recurrent process given X,Z with rate:
	##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
	## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
	## type=3, observed hazards on Cox form among survivors
	## W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
	## or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	## or W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
	dcum <- cbind(base1[,1],dbase1)
	ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
	colnames(ll) <- c("id","start","stop","death")
	ll <- data.frame(ll)
	ll$death <- dd$status[ll$id+1]
	## add frailty to data for possible validation
	ll$z <- z1[ll$id+1]
	ll$fz <- fzz[ll$id+1]
	## add counts of id
	ids <- countID(ll)
	ll <- cbind(ll,ids[,c(2,4,5)]); 
	ll$status <- 1; 
	ll <- dtransform(ll,status=0,reverseCountid==1)
	ll$statusD <- ll$status
	ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)

	return(ll)
} ## }}}


simRecurrentCox <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,X=NULL,r1=NULL,r2=NULL,rd=NULL,rc=NULL, 
			    model=c("not-random","random"),frailty=TRUE,var.z=0.5,death.code=3,alpha=1,...)
{ ## {{{
	if (is.null(r1)) r1 <- rep(1,n)
	if (is.null(r2)) r2 <- rep(1,n)
	if (is.null(rd)) rd <- rep(1,n)
	if (is.null(rc)) rc <- rep(1,n)

	## to avoid error 
	ctime <- death <- NULL

	################################################################
	### approximate hazards to make marginals fit (approximately)
	################################################################
	###laplace and inverse laplace of gamma
	lap<-function(theta,t) { return( (1+t/theta)^(-theta)) }
	ilap<-function(theta,t) { itheta<-1/theta; return((t^(-itheta)-1)/(itheta)) }

	## addapt to make recurrent mean on cox form with this baseline
	base1 <- cumhaz
	if (is.null(death.cumhaz)) stop("Modification for death in this function otherwise just use simRecurrentII\n")
	if (is.null(X)) stop("X must be given to link with simulated data\n"); 

	### Cox baseline 
	St <- exp(-cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2])

	## unique hazard combinations Death Relative Risk
	rds <- unique(rd)
	if (model[1]=="random") {
		z <- rgamma(n,1/var.z)*var.z 
		if (n<10000) {
			zl <- rgamma(100000,1/var.z)*var.z 
			mza <- mean(zl^alpha)
		} else mza <- mean(z^alpha)
		rds <- rd 
	}  else z <- rep(1,n)
	dtt <- diff(c(0,base1[,1]))
	dbase1 <- diff(c(0,base1[,2]))
	data <- c()

	XX <- c()
	nks <- 0
	k <- 1
	if (model[1]!="random") {
		for (rdss in rds) {
			lam1ms <- (dbase1)/St^rdss
			where <- which(rdss==rd)
			nk <- length(where)
			cumhaz1 <- cbind(base1[,1],cumsum(lam1ms))
			LamDr <- scalecumhaz(death.cumhaz,rdss) 

			datss <- simRecurrentII(nk,cumhaz1,cumhaz2,death.cumhaz=LamDr,
						r1=r1[where],r2=NULL,rd=NULL,rc=rc[where],...)
			Xs <- X[where,,drop=FALSE][datss$id,,drop=FALSE]
			XX <- rbind(XX,Xs)
			datss$id <- datss$id+nks
			nks <- nks+nk
			data <- rbind(data,as.matrix(datss))
		}
	}

	if (model[1]=="random") {
		for (k in 1:n) {
			rdss <- rd[k]
			if (frailty==TRUE) lam1ms <- (z[k]^alpha/mza)*(dbase1)/St^(z[k]*rdss) else {
				Stt <-  exp(-z[k]*ilap(1/var.z,St^rdss))
				lam1ms <- (z[k]^alpha/mza)*(dbase1)/Stt
			}
			where <- k
			nk <- 1
			cumhaz1 <- cbind(base1[,1],cumsum(lam1ms))

			if (!frailty) LamDr <- cbind(base1[,1],-log(Stt)) 
			if (frailty) LamDr <- scalecumhaz(death.cumhaz,z[k]*rdss) 

			datss <- simRecurrentII(nk,cumhaz1,cumhaz2,death.cumhaz=LamDr,
						r1=r1[where],r2=NULL,rd=NULL,rc=rc[where],...)
			Xs <- X[where,,drop=FALSE][datss$id,,drop=FALSE]
			XX <- rbind(XX,Xs)
			datss$id <- datss$id+nks
			nks <- nks+nk
			data <- rbind(data,as.matrix(datss))
		}
	}

	rownames(XX) <- NULL
	rownames(data) <- NULL
	colnames(XX) <- paste("X",1:ncol(X),sep="")
	data <- cbind(data,XX)
	data <- as.data.frame(data)

	dsort(data) <- ~id+entry+time
	data$revnr <- revcumsumstrata(rep(1,nrow(data)),data$id-1,n)
	data$statusD <- data$status
	data <- dtransform(data,statusD=death.code,death==1)

	return(data)
} ## }}}


simMarginalMeanCox <- function(n,cens=3/5000,k1=0.1,k2=0,bin=1,Lam1=NULL,Lam2=NULL,LamD=NULL,
			       beta1=rep(0,2),betad=rep(0,2),betac=rep(0,2),X=NULL,...)
{ ## {{{

	### to avoid R-check error
	revnr <- death <- status <- NULL
	p <- length(beta1)

	if (is.null(X))  {
		if (bin==1) X <- matrix(rbinom(n*p,1,0.5),n,p) else  X <- matrix(rnorm(n*p),n,p)
		colnames(X) <- paste("X",1:p,sep="")
	}
	r1 <- exp( X %*% beta1)
	rd <- exp( X %*% betad)
	rc <- exp( X %*% betac)

	if (is.null(Lam2)) Lam2 <- Lam1; 

	rr <- simRecurrentCox(n,scalecumhaz(Lam1,k1),cumhaz2=scalecumhaz(Lam1,k2),death.cumhaz=LamD,X=X,cens=cens,r1=r1,rd=rd,rc=rc,...)

	if (bin==0) dcut(rr,breaks=4) <- X1g~X1 else rr$X1g <- rr$X1
	if (bin==0) dcut(rr,breaks=4) <- X2g~X2 else rr$X2g <- rr$X2
	return(rr)
}

##' @export
scalecumhaz <- function(cumt,k)
{
	return( t(t(cumt)*c(1,k)))
}

##' @export
GLprediid <- function(...)
{
	out <- FGprediid(...,model="GL")
	return(out)
} ## }}}


boottwostageREC <- function(margsurv,recurrent,data,bootstrap=100,id="id",stepsize=0.5,...) 
{ ## {{{
	n <- max(margsurv$id)
	K <- bootstrap
	formid <- as.formula(paste("~",id))
	rrb <- blocksample(data, size = n*K, formid)
	rrb$strata <- floor((rrb[,id]-0.01)/n)

	outb <- outd <- outr <- c()
	for (i in 1:K)
	{
		rrbs <- subset(rrb,strata==i-1)
		drb <- phreg(margsurv$formula,data=rrbs)
		if (inherits(recurrent,"recreg")) {
			xrb <- recreg(recurrent$formula,data=rrbs,
				      cause=recurrent$cause,death.code=recurrent$death.code,cens.code=recurrent$cens.code,twostage=TRUE)
		} else xrb <- phreg(recurrent$formula,data=rrbs)
		outbl <- tryCatch(twostageREC(drb,xrb,rrbs,numderiv=0,control=list(stepsize=stepsize,iter.max=10),...),error=function(x) NULL)
     if (!is.null(outbl)) outb <- rbind(outb,outbl$coef)
     outd <- rbind(outd,coef(drb))
     outr <- rbind(outr,coef(xrb))
  }
  varb <- cov(outb)
  vard <- cov(outd)
  varr <- cov(outr)

  list(outb=outb,var=varb,se=diag(varb)^.5,se.coxD=diag(vard)^.5,se.coxR=diag(varr)^.5)
} ## }}}

##' Fittting of Two-stage recurrent events random effects model based on Cox/Cox or Cox/Ghosh-Lin marginals 
##'
##' Fittting of Two-stage recurrent events random effects model based on Cox/Cox or Cox/Ghosh-Lin marginals. Random
##' effects model fore recurrent events with terminal  event. Marginal models fitted first and given to twostageREC function.
##'
##' @param margsurv marginal model for terminal event 
##' @param recurrent marginal model for recurrent events
##' @param data used for fitting
##' @param theta starting value for total variance of gamma frailty
##' @param model can fully shared "full", partly shared "shared", or non-shared where the random effect acts only on recurrent events
##' @param ghosh.lin to force use ghosh.lin marginals based on recurrent (taking baseline and coefficients) 
##' @param theta.des regression design for variance
##' @param var.link possible link  function 1 is exponential link
##' @param method NR
##' @param no.opt to not optimize
##' @param weights possible weights
##' @param se.cluster  to combine influence functions for naive variance based on these clusters GEE style
##' @param fnu a function to make transformation for nu (amount shared)
##' @param nufix to fix the amount shared
##' @param nu starting value for amount shared
##' @param numderiv uses numerical derivatives for some derivatives
##' @param derivmethod method for numerical derivative
##' @param ... arguments for 
##' @references 
##' Scheike (2026), Two-stage recurrent events random effects models, LIDA, to appear
##' @export
twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NULL, model=c("full","shared","non-shared"),ghosh.lin=NULL,
  theta.des = NULL, var.link = 0, method = "NR", no.opt = FALSE, weights = NULL, se.cluster = NULL, 
  fnu=NULL,nufix=0,nu=NULL,numderiv=1,derivmethod=c("simple","Richardson"),...)
{ ## {{{
    newrec <- 1; at.risk <- 1
    if (!inherits(margsurv, "phreg")) stop("Must use phreg for death model\n")
    if (!inherits(recurrent, "phreg")) stop("Must use phreg for recurrent model\n")
    if (is.null(recurrent$cox.prep) & newrec==0) stop("recreg must be called with cox.prep=TRUE\n")
    if (inherits(recurrent, "recreg") & is.null(ghosh.lin))  ghosh.lin <- 1 
    if ((!inherits(recurrent, "recreg")) & is.null(ghosh.lin))  ghosh.lin <- 0 
    if (is.null(recurrent$cox.prep.twostage) & newrec==1 & (ghosh.lin==1)) stop("recreg must be called with twostage=TRUE\n")
    clusters <- margsurv$cox.prep$id
    n <-  max(clusters)+1
    if (is.null(theta.des) == TRUE) ptheta <- 1
    if (is.null(theta.des) == TRUE) theta.des <- matrix(1, n, ptheta) else theta.des <- as.matrix(theta.des)
    ptheta <- ncol(theta.des)
    if (nrow(theta.des) != n) stop("Theta design does not have correct dim")
    if (is.null(theta) == TRUE) {
        if (var.link == 1) theta <- rep(0, ptheta)
        if (var.link == 0) theta <- rep(1, ptheta)
    }
    if (is.null(nu)  & (nufix==0)) { if (is.null(fnu))  nu <-  0.5 else nu <- fnu[[1]](0.5) }
    if (length(theta) != ptheta) theta <- rep(theta[1], ptheta)
    if (length(nu) != ptheta) nu <- rep(nu,ptheta)
    theta.score <- rep(0, ptheta)
    Stheta <- var.theta <- matrix(0, ptheta, ptheta)
    max.clust <- length(unique(clusters))
    theta.iid <- matrix(0, max.clust, ptheta)
    xx <- margsurv$cox.prep
    xr <- recurrent$cox.prep
    if (ghosh.lin & newrec==1) {
	    xr <- recurrent$cox.prep.twostage
    } 
    nn <- length(xx$strata)
    if (is.null(weights)) weights <- rep(1, nn)
    if (length(weights) != nn) stop("Weights do not have right length")
    statusxx <- rep(0, length(xx$strata))
    statusxx[xx$jumps + 1] <- 1
    xx$status <- statusxx
    mid <- max(xx$id) + 1
    Nsum <- cumsumstratasum(statusxx, xx$id, mid, type = "all")
    Ni.tau <- sumstrata(statusxx, xx$id, mid)
    S0i2 <- S0i <- rep(0, length(xx$strata))
    S0i[xx$jumps + 1] <- 1/margsurv$S0
    cumhazD <- cumsumstrata(S0i, xx$strata, xx$nstrata)
    if (!is.null(margsurv$coef)) RR <- exp(xx$X %*% margsurv$coef) else RR <- rep(1, nn)
    HD <- c(cumhazD * RR)
    cumDYt <- sumstrata(HD*xx$sign,xx$id,mid)
    statusx1 <- rep(0, length(xx$strata))
    statusx1[xr$jumps + 1] <- 1
    xr$status <- statusx1
    N1sum <- cumsumstratasum(statusx1, xr$id, mid, type = "all")
    N1i.tau <- sumstrata(statusx1, xr$id, mid)
    S01i2 <- S01i <- rep(0, length(xr$strata))
    S01i[xr$jumps + 1] <- 1/recurrent$S0
    cumhaz1 <- cumsumstrata(S01i, xr$strata, xr$nstrata)
    if (!is.null(recurrent$coef)) RR1 <- exp(xr$X %*% recurrent$coef) else RR1 <- rep(1, nn)
    H1 <- c(cumhaz1 * RR1)
    ###
    ## designs of fixed time covariates and weights
    cc <- cluster.index(xx$id)
    firstid <- cc$firstclustid + 1
    if (max(cc$cluster.size) == 1) stop("No clusters !, maxclust size=1\n")
    ###
    theta.des <- theta.des[xx$id+ 1, , drop = FALSE]
    theta.des <- theta.des[xx$ord + 1, , drop = FALSE]
    weightsid <- weights <- weights[xx$ord + 1]
    weights <- weights[firstid]
    thetaX <- as.matrix(theta.des[firstid, , drop = FALSE])
    Xdeath <- as.matrix(xx$X[firstid,,drop = FALSE])
    Xrecurrent <- as.matrix(xr$X[firstid,,drop = FALSE])
    statusxb <- statusxx+statusx1
    ###
    Nsumb <- Nsum$lagsum+N1sum$lagsum
    rd <- RR[firstid]
    r1 <- RR1[firstid]
    lastid <- tailstrata(xx$id,mid)
    ###
    cumDL <- HD[lastid]
    nuX <- thetaX
    ###
    idD <- xx$id[statusxx==1]

    obj <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(c(par))) else epar <- c(par)
        thetav <- c(as.matrix(theta.des) %*% c(epar))
        thetai <- thetav[firstid]
	###
###        tildeL <- .Call("_mets_tildeLambda1",S01i,cumhazD,r1,rd,thetai,xx$id)
###	if (at.risk==1)  
###		tildeL <- apply(tildeL*c(xr$sign),2,cumsumstrata,xr$id,mid)
        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
        tildeLast <- tildeL[lastid,]
	Ht <- thetav*tildeL[,1]+exp(thetav*HD)
	Hr <- thetai*tildeLast[,1]+exp(thetai*cumDL)
	DHt <- tildeL[,1]+thetav*tildeL[,2]+HD*exp(thetav*HD)
	DHr <- tildeLast[,1]+thetai*tildeLast[,2]+cumDL*exp(thetai*cumDL)
        ###
	D2Ht <- 2*tildeL[,2]+thetav*tildeL[,3]+HD^2*exp(thetav*HD)
	D2Hr <- 2*tildeLast[,2]+thetai*tildeLast[,3]+cumDL^2*exp(thetai*cumDL)
	aHt <- abs(Ht)
	aDHt <- abs(DHt)
	aD2Ht <- abs(D2Ht)
        ###
        l1 <- sumstrata(log(1 + thetav * N1sum$lagsum) * statusxb, xx$id, mid)
        l2 <- sumstrata(statusxb * HD, xx$id, mid)
	if (at.risk==0) l3 <- -(1/thetai + N1i.tau) * log(Hr) 
	if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(1/thetav+N1sum$sum)*log(aHt),xr$id,mid) 
	HtD <- Ht[statusxx==1]
	DHtD <- DHt[statusxx==1]
	D2HtD <- D2Ht[statusxx==1]
        l4 <- -sumstrata(log(HtD),idD,mid)
        logliid <- (l1 + thetai * l2 + l3 + l4) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
        l1s <- sumstrata(N1sum$lagsum/(1 + thetav * N1sum$lagsum) * statusxb, xx$id, mid)
	if (at.risk==0) l3s <- -(1/thetai + N1i.tau) * DHr/Hr + log(Hr)/thetai^2;
	if (at.risk==1) l3s <- sumstrata(c(xr$sign)*(-(1/thetav+N1sum$sum)*DHt/Ht+log(aHt)/thetav^2),xr$id,mid);
	l4s <-  -sumstrata((DHtD/HtD),idD, mid) 
	Dltheta <- (l1s+l2+l3s+l4s)*c(weights) 
	scoreiid <- thetaX * c(Dltheta)
        D2N <- -sumstrata(N1sum$lagsum^2/(1 + thetav * N1sum$lagsum)^2 * statusxb, xx$id, mid)
	if (at.risk==0) 
        Dhes <- (2/thetai^2) * DHr/Hr -(1/thetai + N1i.tau)*(D2Hr*Hr-DHr^2)/Hr^2 - (2/thetai^3) * log(Hr) 
	if (at.risk==1) 
        Dhes <- sumstrata(c(xr$sign)* (
	(2/thetav^2) * aDHt/aHt -(1/thetav + N1sum$sum)*(aD2Ht*Ht-aDHt^2)/aHt^2 -(2/thetav^3) * log(aHt)) ,xr$id,mid)
	D2l4 <- -sumstrata(((D2HtD*HtD-DHtD^2)/HtD^2), idD, mid)
        Dhes <- c(Dhes+D2N+D2l4) * c(weights)
        if (var.link == 1) {
            scoreiid <- scoreiid * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid,
            ploglik = ploglik, gradient = gradient, hessian = hessian,
            hess2 = hess2)
        if (all)
            return(val)
        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
    }


    objNonShared <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(c(par))) else epar <- c(par)
        thetav <- c(as.matrix(theta.des) %*% c(epar))
        thetai <- thetav[firstid]
	###
	Ht <- 1+thetav*H1 
	Hr <- Ht[lastid]
	DHt <- H1
	DHr <- H1[lastid]
        ###
	D2Ht <- 0
	D2Hr <- 0
	aHt <- abs(Ht)
	aDHt <- abs(DHt)
	aHr <- abs(Hr)
        ###
        l1 <- sumstrata(log(1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	if (at.risk==0) l3 <- -(1/thetai + N1i.tau) * log(Hr) 
	if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(1/thetav+N1sum$sum)*log(aHt),xr$id,mid) 
        logliid <- (l1 + l3) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
        l1s <- sumstrata(N1sum$lagsum/(1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	if (at.risk==0) l3s <- -(1/thetai + N1i.tau) * DHr/Hr + log(Hr)/thetai^2;
	if (at.risk==1) l3s <- sumstrata(c(xr$sign)*(-(1/thetav+N1sum$sum)*DHt/Ht+log(aHt)/thetav^2),xr$id,mid);
	Dltheta <- (l1s+l3s)*c(weights) 
	scoreiid <- thetaX * c(Dltheta)
        D2N <- -sumstrata(N1sum$lagsum^2/(1 + thetav * N1sum$lagsum)^2 * statusx1, xx$id, mid)
	if (at.risk==0) 
        Dhes <- (2/thetai^2) * DHr/Hr -(1/thetai + N1i.tau)*(-DHr^2)/Hr^2 - (2/thetai^3) * log(Hr) 
	if (at.risk==1) 
        Dhes <- sumstrata(c(xr$sign)* (
	(2/thetav^2) * aDHt/aHt -(1/thetav + N1sum$sum)*(-aDHt^2)/aHt^2 -(2/thetav^3) * log(aHt)) ,xr$id,mid)
        Dhes <- c(Dhes+D2N) * c(weights)
        if (var.link == 1) {
            scoreiid <- scoreiid * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid,
            ploglik = ploglik, gradient = gradient, hessian = hessian,
            hess2 = hess2)
        if (all)
            return(val)
        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
    }


    ## default is simple identity 
###    if (is.null(fnu)) { fw <- function(x) 1/(1+exp(-x)); Dfw <- function(x) -exp(-x)/(1+exp(-x))^2;} else { 
   if (is.null(fnu)) { fw <- function(x) x; Dfw <- function(x) 1;} else { fw <- fnu[[1]]; Dfw <- fnu[[2]]; } 
   nudes <- theta.des  
   p <- ncol(theta.des)

    objShared <- function(par, all = FALSE) {
        if (var.link == 1) epar <- c(exp(par[1:p])) else epar <- c(par[1:p])
        thetav <- c(as.matrix(theta.des) %*% epar)
	if (nufix==1) nu1 <- c(as.matrix(nudes) %*% nu)
	else nu1 <- c(as.matrix(nudes) %*% par[(p+1):2*p])
        nu1i <-nu1[firstid]
	tbeta1 <- fw(nu1); tbeta2 <- 1-tbeta1; 
        thetai <- thetav[firstid]; tbeta1i <- tbeta1[firstid]; tbeta2i <- tbeta2[firstid]
	###
	R <-  exp( - thetav*HD);  DR <- -HD*exp( - thetav* HD); D2R <-  HD^2*exp( - thetav* HD) 
        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
	tildeLast <- tildeL[lastid,]
	Ht <- (thetav/tbeta1)*tildeL[,1]+exp(thetav*HD)
	Hr <- (thetai/tbeta1i)*tildeLast[,1]+exp(thetai*cumDL)
	DHt <- (tildeL[,1]+thetav*tildeL[,2])/tbeta1+HD*exp(thetav*HD)
	DHr <- (tildeLast[,1]+thetai*tildeLast[,2])/tbeta1i+cumDL*exp(thetai*cumDL)
	DHtn <- -(thetav/tbeta1^2)*tildeL[,1]
	DHrn <- -(thetai/tbeta1i^2)*tildeLast[,1]
        ###
	D2Ht <- (2*tildeL[,2]+thetav*tildeL[,3])/tbeta1+HD^2*exp(thetav*HD)
	D2Hr <- (2*tildeLast[,2]+thetai*tildeLast[,3])/tbeta1i+cumDL^2*exp(thetai*cumDL)
        ###
        N <- (tbeta1 + thetav * N1sum$lagsum)/Ht + tbeta2*R
        l1d <- sumstrata(log(N) * statusxx, xx$id, mid)
        l11 <- sumstrata(log(tbeta1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	l1 <- l1d+l11
        l2 <- sumstrata(statusxb * HD, xx$id, mid) 
	l22 <-  -log(tbeta1i)*N1i.tau 
        if (at.risk==0) l3 <- -(tbeta1i/thetai + N1i.tau)*log(Hr) 
        if (at.risk==1) l3 <- - sumstrata(c(xr$sign)*(tbeta1/thetav+N1sum$sum)*log(Ht),xr$id,mid) 
        l4 <-  (tbeta1i)*cumDYt
        logliid <- (l1 + thetai*l2+ l22 + l3 + l4) * c(weights)
        logl <- sum(logliid)
        ploglik <- logl
        ###
	DN1tt <- (Ht*N1sum$lagsum-DHt*(tbeta1+thetav*N1sum$lagsum))
	DN1t <-  DN1tt/Ht^2 
	DNt <- c(DN1t+tbeta2*DR)
        l1ds <- sumstrata((DNt/N)*statusxx,xx$id,mid)
        l11s <- sumstrata((N1sum$lagsum/(tbeta1 + thetav * N1sum$lagsum)) * statusx1, xx$id, mid)
	l1s <- l1ds+l11s
        if (at.risk==0) 
        l3s <- -(tbeta1i/thetai + N1i.tau) * (DHr/Hr) + (tbeta1i/thetai^{2}) * log(Hr) 
        if (at.risk==1) 
	l3s <- sumstrata(c(xr$sign)*(-(tbeta1/thetav+N1sum$sum)*DHt/Ht+log(Ht)*tbeta1/thetav^2),xr$id,mid);
        Dltheta <- (l1s+l2+l3s)*c(weights)
        ### 
	DNtn <- (Ht-DHtn*(tbeta1+thetav*N1sum$lagsum))
	DNn <- DNtn/Ht^2 
        l1dn <- sumstrata( c(DNn-R)/N * statusxx, xx$id, mid)
        l11n <- sumstrata(1/(tbeta1 + thetav * N1sum$lagsum) * statusx1, xx$id, mid)
	l1n <- l1dn+l11n
	l22n <-  -N1i.tau/tbeta1i 
	if (at.risk==0) 
		l3n <- -(tbeta1i/thetai+N1i.tau)*(DHrn/Hr)-(1/thetai)*log(Hr)
	if (at.risk==1) 
        l3n <- -sumstrata(c(xr$sign)*(+(tbeta1/thetav+N1sum$sum)*(DHtn/Ht)+(1/thetav)*log(Ht)), xr$id,mid);
        Dlnu <- (l1n+l22n+l3n+cumDYt)*c(weights)
        ###
	if (nufix==1)
        scoreiid <- thetaX * c(Dltheta)
        else  scoreiid <- cbind(thetaX * c(Dltheta),nuX*c(Dlnu)*Dfw(nu1i))
        ###   ###
        Dl11s <- -sumstrata(N1sum$lagsum^2/(tbeta1 + thetav * N1sum$lagsum)^2 * statusxb, xx$id, mid)
        Dl3s <- (2*tbeta1i/thetai^2) * DHr/Hr -(tbeta1i/thetai+ N1i.tau)*(D2Hr*Hr-DHr^2)/Hr^2 - (2*tbeta1i/thetai^3) * log(Hr) 
        ###
	D2N1t <- (Ht^2*(DHt*N1sum$lagsum-D2Ht*(tbeta1+thetav*N1sum$lagsum)-DHt*N1sum$lagsum)-2*DHt*DN1tt)/Ht^4 
	D2Nt <-  D2N1t+tbeta2*D2R
	Dl1ds <- -sumstrata(((D2Nt*N-DNt^2)/N^2)*statusxx, xx$id, mid)
        Dhes <- c(Dl1ds+Dl11s+Dl3s) * c(weights)
        if (var.link == 1) {
            scoreiid[,1:p] <- scoreiid[,1:p] * c(thetai)
            Dhes <- Dhes * thetai^2 + thetai * Dltheta
        }
        gradient <- apply(scoreiid, 2, sum)
        hessian <- crossprod(thetaX, thetaX * c(Dhes))
	if (nufix==0) {
###	    hessiann <- crossprod(nuX, nuX* c(D2nu))
###	    hessiannp <- crossprod(thetaX,nuX*Dtn)
###	    hessian <- cbind(hessian,hessianp)
###	    hessian <- rbind(hessian,cbind(t(hessianp),hessiann))
	}
        hess2 <- crossprod(scoreiid)
        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid, ploglik = ploglik, gradient = gradient, hessian = -hess2, hess2 = hess2)
        if (all)
            return(val)
###        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
        with(val, structure(-ploglik, gradient = -gradient, hessian = hess2))
    }


   if (model[1]=="shared") obj <- objShared
   if (model[1]=="non-shared") obj <- objNonShared
   if (nufix==0 & model[1]=="shared") par <- c(theta,nu) else par <- theta

    opt <- NULL
    if (no.opt == FALSE) {
        if (tolower(method) == "nr") {
            opt <- lava::NR(par, obj,...)
            opt$estimate <- opt$par
        }
        else {
            opt <- nlm(obj, par,...)
            opt$method <- "nlm"
        }
        cc <- opt$estimate
        val <- c(list(coef = cc), obj(opt$estimate, all = TRUE))
    }
    else val <- c(list(coef = par), obj(par, all = TRUE))
    val$score <- val$gradient
    theta <- matrix(c(val$coef), length(c(val$coef)), 1)

    if (!is.null(colnames(theta.des)))
        thetanames <- colnames(theta.des)
    else thetanames <- paste("dependence", 1:length(c(theta)), sep = "")
    if (nufix==0 & model[1]=="shared") thetanames <-
     thetanames <- c(paste("dependence", 1:p, sep = ""),paste("share", 1:p,sep = ""))

    if (length(thetanames) == length(c(theta))) {
        rownames(theta) <- thetanames
        names(val$coef)  <- thetanames
    }

    if (numderiv==1 & model[1]=="shared") {
	    dobj <- function(p) {
		    oo <- obj(p)
		    return(attr(oo,"gradient"))
	    }
	    hessian <- numDeriv::jacobian(dobj,val$coef,method=derivmethod[1])
	    val$hessian <- -hessian
    }

    hessianI <- solve(val$hessian)
    val$theta.iid.naive <- val$score.iid %*% hessianI

    if (!is.null(se.cluster))
        if (length(se.cluster) != length(clusters))
            stop("Length of seclusters and clusters must be same\n")
    if (!is.null(se.cluster)) {
        iids <- unique(se.cluster)
        nseclust <- length(iids)
        if (is.numeric(se.cluster))
            se.cluster <- fast.approx(iids, se.cluster) - 1
        else se.cluster <- as.integer(factor(se.cluster, labels = seq(nseclust))) - 1
        val$theta.iid <- apply(val$theta.iid,2,sumstrata,se.cluster, nseclust)
        val$theta.iid.naive <- apply(val$theta.iid.naive,2,sumstrata,se.cluster, nseclust)
    }
    var <- robvar.theta <- var.theta <- crossprod(val$theta.iid)
    naive.var <- crossprod(val$theta.iid.naive)
    val <- c(val, list(theta = theta, var.theta = var,n=mid,p=ncol(thetaX),var.link=var.link,
                       robvar.theta = var, var = var, thetanames = thetanames,
                       model = model[1], se = diag(var)^0.5), 
	     var.naive = naive.var,no.opt=no.opt,ghosh.lin=ghosh.lin)
    class(val) <- "twostageREC"
    attr(val, "clusters") <- clusters
    attr(val, "fnu") <- fw
    attr(val, "Dfnu") <- Dfw
    attr(val, "secluster") <- c(se.cluster)
    attr(val, "var.link") <- var.link
    attr(val, "ptheta") <- ptheta
    attr(val, "n") <- n
    attr(val, "response") <- "survival"
    attr(val, "additive-gamma") <- 0
    attr(val, "twostage") <- "two.stage"
    return(val)
} ## }}}

##' @export
summary.twostageREC <- function(object,vcov=NULL,delta=0,...) { ## {{{
    I <- -solve(object$hessian)
    if (!is.null(vcov)) V <- vcov else V <- object$var
    ncluster <- object$n
    cc <- estimate(coef=object$coef,vcov=V)$coefmat
    pd <- object$p
    if (object$var.link==1 & object$model=="full") f <- function(p) exp(p)
    if (object$var.link==0 & object$model=="full") f <- function(p) p
    if (object$var.link==1 & object$model=="non-shared") f <- function(p) exp(p)
    if (object$var.link==0 & object$model=="non-shared") f <- function(p) p

    if (object$var.link==1 & object$model=="shared") f <- function(p) c(exp(p[1]),attr(object,"fnu")(p[2]))
    if (object$var.link==0 & object$model=="shared") f <- function(p) c(p[1],attr(object,"fnu")(p[2]))
    if (delta==1 | object$model=="full" | object$model=="non-shared") 
    expC <- lava::estimate(coef=object$coef,vcov=V,f=f)$coefmat ##[,c(1,3,4),drop=FALSE]
    else expC <- apply(cc[,c(1,3,4),drop=FALSE],2,f) 
  n <- object$n
  res <- list(coef=cc,n=n,nevent=object$nevent,ncluster=ncluster,var=V,exp.coef=expC,var.link=object$var.link,
	      ghosh.lin=object$ghosh.lin)
  class(res) <- "summary.twostageREC"
  res
}  ## }}}

##' @export
print.summary.twostageREC  <- function(x,max.strata=5,...) { ## {{{
  if (x$ghosh.lin==0) cat("Cox(recurrent)-Cox(terminal) intensity model\n"); 
  if (x$ghosh.lin==1) cat("Ghosh-Lin(recurrent)-Cox(terminal) mean model\n"); 
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coeffients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    if (x$var.link==1) cat("var=exp(coeffients),shared:\n")
    if (x$var.link==0) cat("var,shared:\n")
    printCoefmat(x$exp.coef,...)
  }
  cat("\n")
}  ## }}}


##' @export
print.twostageREC  <- function(x,...) {
  print(summary(x),...)
} 

##' Discrete time to event interval censored data 
##'
##' We consider the cumulative odds model 
##' \deqn{
##'    P(T \leq t | x) =  \frac{G(t) \exp(x \beta) }{1 + G(t) exp( x \beta) }
##' }
##' or equivalently 
##' \deqn{
##'    logit(P(T \leq t | x)) = log(G(t)) + x \beta
##' }
##' and we can thus also compute the probability of surviving 
##' \deqn{
##'    P(T >t | x) =  \frac{1}{1 + G(t) exp( x \beta) }
##' }
##' 
##' The baseline \eqn{G(t)} is written as \eqn{cumsum(exp(\alpha))} and this is not the standard
##' parametrization that takes log of \eqn{G(t)} as the parameters. Note that the regression 
##' coefficients are describing the probability of dying before or at time t. 
##' 
##' Input are intervals given by ]t_l,t_r] where t_r can be infinity for right-censored intervals 
##' When truly discrete ]0,1] will be an observation at 1, and  ]j,j+1] will be an observation at j+1.
##' Can be used for fitting the usual ordinal regression model (with logit link) that in contrast, however, 
##' describes the probibility of surviving time t (thus leads to -beta).
##' 
##' Likelihood is maximized:
##' \deqn{
##'  \prod  P(T_i >t_{il} | x) - P(T_i> t_{ir}| x) 
##' }
##' 
##' @param formula  formula
##' @param data  data 
##' @param beta starting values 
##' @param no.opt optimization TRUE/FALSE 
##' @param method NR, nlm 
##' @param stderr to return only estimate 
##' @param weights weights following id for GLM 
##' @param offsets following id  for GLM
##' @param exp.link parametrize increments exp(alpha) > 0
##' @param increment using increments dG(t)=exp(alpha) as parameters
##' @param ... Additional arguments to lower level funtions lava::NR  optimizer or nlm
##' @author Thomas Scheike
##' @examples
##' library(mets)
##' data(ttpd) 
##' dtable(ttpd,~entry+time2)
##' 
##' out <- interval.logitsurv.discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
##' summary(out)
##' head(iid(out)) 
##' 
##' pred <- predictlogitSurvd(out,se=FALSE)
##' plotSurvd(pred)
##' 
##' ttpd <- dfactor(ttpd,fentry~entry)
##' out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
##' summary(out)
##' head(iid(out)) 
##' 
##' @aliases Interval dInterval simlogitSurvd predictlogitSurvd cumoddsreg simTTP predictSurvd plotSurvd 
##' @export
interval.logitsurv.discrete <- function (formula,data,beta=NULL,no.opt=FALSE,method="NR",
	   stderr=TRUE,weights=NULL,offsets=NULL,exp.link=1,increment=1,...)
{ ## {{{ 

  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")

 if (ncol(Y)==2) {
	time2 <- eventtime <- Y[,2]
	entrytime <- Y[,1]
	left <- 0
    } else {
	time2 <- eventtime <- Y[,2]
	status <- delta  <- Y[,3]
	entrytime <- Y[,1]
	left <- 1
	if (max(entrytime)==0) left <- 0
    }

  if (any(entrytime==time2)) stop("left==right not possible for discrete data")

  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}

  X <- model.matrix(Terms, m)
  if (!is.null(intpos  <- attributes(Terms)$intercept))
  X <- X[,-intpos,drop=FALSE]
  if (ncol(X)>0) X.names <- colnames(X) else X.names <- NULL

###  if (!is.null(id)) {
###	  ids <- unique(id)
###	  nid <- length(ids)
###      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
###      id <- as.integer(factor(id,labels=seq(nid)))-1
###     }
###   } else { id <- as.integer(seq_along(time2))-1; nid <- length(time2) }
###   ## orginal id coding into integers 
###   id.orig <- id+1; 

  call.id <- id
  conid <- construct_id(id,nrow(X))
  id <- conid$id; nid <- conid$nid; name.id <- conid$name.id

  ## times 1 -> mutimes , 0 til start
  utimes <- sort(unique(c(time2,entrytime)))
  mutimes <- max(utimes[utimes<Inf])
  n <- length(time2)
  
  ## setting up designs for t_l and t_r
  tR <- tL <- matrix(0,n,mutimes)

  ## design for ]t_l,t_r], for t_l=0 row is  0
  if (increment==0) {
	  tL <- matdoubleindex(tL,1:n,entrytime,rep(1,n))
	  tR <- matdoubleindex(tL,1:n,time2,rep(1,n))
  } else {
     for (i in 1:mutimes) {
        tL[,i] <- (i <= entrytime) 
        tR[,i] <- (i <= time2) 
     }
  }

  ## computing X^2 
  if (ncol(X)>0) {
  X2  <- .Call("vecMatMat",X,X)$vXZ
  XtL  <- .Call("vecMatMat",X,tL)$vXZ
  XtR  <- .Call("vecMatMat",X,tR)$vXZ
  }
  tL2  <- .Call("vecMatMat",tL,tL)$vXZ
  tR2  <- .Call("vecMatMat",tR,tR)$vXZ

  ## weights/offets will follow id 
  if (is.null(weights))  weights <- rep(1,n); #  else wiid <- weights
  if (is.null(offsets))  offsets <- rep(0,n); # else offsets <- offsets

  if (is.null(beta)) {
     beta <- rep(0,ncol(X)+mutimes)
     Set <- 1-cumsum(table(time2))/n
     dHt <- log(diff(c(0,(1/Set-1))))
     beta[1:mutimes] <- dHt[1:mutimes]
  }


obj <- function(pp,all=FALSE)
{ # {{{

if (exp.link==1) theta <- exp(pp[1:mutimes]) else theta  <-  pp[1:mutimes]

if (ncol(X)>0) {
betal   <- pp[-c(1:mutimes)]
Zbeta <- c(X %*% betal+offsets)
} else {Zbeta <- offsets }
xltheta <-  c(tL %*% theta)
xrtheta <-  c(tR %*% theta)
EZbeta <- exp(Zbeta)
GEl <- xltheta * EZbeta
GEr <- xrtheta * EZbeta
Stl <- 1/(1+GEl) 
Str <- 1/(1+GEr) 

############################################
## likelihood
############################################
# {{{
p <- Stl-Str*(time2<Inf)
logp <- log(p)
logl <- weights*logp 
# }}}
############################################
## Derivative 
############################################
# {{{
if (ncol(X)>0) {
DbetaStl <- -X*GEl*Stl^2
DbetaStr <- -(time2<Inf)*X*GEr*Str^2
Dbetalogp <- (DbetaStl-DbetaStr)/p
} 
DgStl <- -tL*EZbeta*Stl^2
DgStr <- -(time2<Inf)*tR*EZbeta*Str^2
Dglogp <- (DgStl-DgStr)/p

if (ncol(X)>0) {
Dlogliid  <- cbind(Dglogp*weights,Dbetalogp*weights)
Dlogl  <- apply(Dlogliid,2,sum)
} else {
	Dlogliid  <- Dglogp*weights
        Dlogl  <- apply(Dlogliid,2,sum)
}
if (exp.link==1) {
Dlogliid[,1:mutimes] <- t( t(Dlogliid[,1:mutimes])*theta)
Dlogl[1:mutimes] <- Dlogl[1:mutimes]*theta
}
# }}}
############################################
## 2nd Derivative 
############################################
# {{{
if (ncol(X)>0) {
D2betaStl <- X2*(GEl^2-GEl)*Stl^3
D2betaStr <- X2*(time2<Inf)*(GEr^2-GEr)*Str^3
DbetagStl <- XtL*            (EZbeta*(GEl-1))*Stl^3
DbetagStr <- XtR*(time2<Inf)*(EZbeta*(GEr-1))*Str^3
}
D2gStl <-             2*tL2*EZbeta^2*Stl^3
D2gStr <- 2*tR2*(time2<Inf)*EZbeta^2*Str^3

DglpDglp       <- .Call("vecMatMat",Dglogp,Dglogp)$vXZ
D2glogp <-    (D2gStl-D2gStr)/p  - DglpDglp
D2g <-    apply(weights*D2glogp,2,sum)

## cross products of derivatives
if (ncol(X)>0) {
DbetalpDbetalp <- .Call("vecMatMat",Dbetalogp,Dbetalogp)$vXZ
DbetalpDglp    <- .Call("vecMatMat",Dbetalogp,Dglogp)$vXZ

D2betalogp <- (D2betaStl-D2betaStr)/p - DbetalpDbetalp
Dbetaglogp <- (DbetagStl-DbetagStr)/p  - DbetalpDglp
D2beta <- apply(weights*D2betalogp,2,sum)
Dbetag <- apply(weights*Dbetaglogp,2,sum)
}

D2log <- matrix(0,length(pp),length(pp))
D2log[1:mutimes,1:mutimes] <- D2g

if (ncol(X)>0) {
###D2log[1:mutimes,(mutimes+1):length(pp)] <- Dbetag
D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag
D2log[1:mutimes,(mutimes+1):length(pp)] <- t(D2log[(mutimes+1):length(pp),1:mutimes])
D2log[(mutimes+1):length(pp),(mutimes+1):length(pp)] <- D2beta
}

diagM <- function(vec)
{# {{{
	p <- length(vec)
	out <- matrix(0,p,p)
	out <- matdoubleindex(out,1:p,1:p,vec)
	return(out)
}
# }}}

if (exp.link==1) {
D2log[1:mutimes,1:mutimes] <- diagM(Dlogl[1:mutimes]) + D2log[1:mutimes,1:mutimes]*(theta %o% theta)  
if (ncol(X)>0) {
D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag*rep(theta,each=length(betal))
###D2log[(mutimes+1):length(pp),1:mutimes] <- Dbetag*rep(theta,length(betal))
D2log[1:mutimes,(mutimes+1):length(pp)] <- t(D2log[(mutimes+1):length(pp),1:mutimes])
}
}

# }}}
############################################

ploglik <- sum(logl)
gradient <- Dlogl 
hessian <- D2log

  if (all) {
      ihess <- solve(hessian)
      beta.iid <- Dlogliid %*% ihess ## %*% t(Dlogl) 
      beta.iid <- apply(beta.iid,2,sumstrata,id,nid)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
		  iid=beta.iid,robvar=robvar,var=robvar,ihessian=ihess,id=id,
		  se=diag(robvar)^.5,coef=pp,se.coef=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik/nid,gradient=-gradient/nid,hessian=-hessian/nid)
}# }}}

  p <- ncol(X)
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      if (!stderr) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=beta),obj(beta,all=TRUE))

  uu <- utimes[utimes<Inf]
  Xnames <- c(paste("time",uu[-1],sep=""),X.names)
  if (length(val$coef)==length(Xnames)) names(val$coef) <- Xnames

  if (is.matrix(val$iid)) 
	  if (length(name.id)==nrow(val$iid)) rownames(val$iid) <- name.id

  val <- c(list(increment=increment,exp.link=exp.link,ntimes=mutimes,utimes=utimes,
		name.id=name.id,call.id=call.id,nid=nid),val)

  class(val) <- c("cumoddsreg")
  return(val)
} ## }}} 

##' @export
IC.cumoddsreg <- function(x,...) { x$iid*NROW(x$iid) }

##' @export
cumoddsreg <- function (formula,data,...)
{ ## {{{ 
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  nt <- nlevels(Y)
  time2__ <- as.numeric(Y)
  entrytime__ <- time2__-1
  time2__[time2__==nt] <- Inf

  xf <- update.formula(formula,Interval(entrytime__,time2__)~.)
  data$entrytime__ <- entrytime__
  data$time2__ <- time2__
  out <- interval.logitsurv.discrete(xf,data,...)

 return(out)
} ## }}}

##' @export
Interval <- function (time, time2 , ...)
{# {{{
    out <- cbind(time, time2 )
    colnames(out) <- c("entry", "time2")
    class(out) <- "Interval"
    return(out)
}# }}}

##' @export
dInterval <- function (time, time2 ,cuts=NULL,cut.first=0,show=FALSE, ...) 
{# {{{
if (is.null(cuts)) cuts <-  sort(unique(time,time2))
if (min(cuts)> 0)  cuts <- c(cut.first,cuts)
###
lleft <- fast.approx(cuts,time,type="left")
rright <- fast.approx(cuts,time2,type="right")

out <- data.frame(time=time,time2=time2,left=lleft-1,right=rright-1)
attr(out,"cuts") <- cuts 
out$leftd <- cuts[lleft]
out$rightd <- cuts[rright]

if (max(cuts)==Inf) out$right[out$rightd==Inf] <- Inf

if (any(out$time<out$leftd)) warning("left not in [leftd,rightd]\n")
if (any(out$time2>out$rightd)) warning("right  not in [leftd,rightd]\n") 

if (show) {
	mtime <- mmtime <- max(cuts[cuts<Inf])
	n <- length(time)
	if (max(cuts)==Inf) mtime <- mmtime+1
	plot(0,0,xlim=c(0,mtime),ylim=c(0,n),type="n")
	abline(v=cuts,col=3)
	lines(c(min(cuts),mmtime),c(-1,-1),lwd=2)
	if (max(cuts)==Inf) lines(c(mtime,mmtime),c(-1,-1),lwd=2,col=2)
	for (i in 1:n)
	{
	lines(c(out$time[i],min(out$time2[i],mtime)),rep(i,2),col=1)
	lines(c(out$leftd[i],min(out$rightd[i],mtime)),rep(i+0.5,2),col=2)
	}
}

return(out)
}# }}}

##' @export
simlogitSurvd <- function(coef,Z,n=NULL,times=0:6)
{# {{{
  if (missing(Z)) Z <- NULL
  if (!is.null(Z)) n <- nrow(Z) 
  Z <- as.matrix(Z)

  g <- coef[times[-1]]
  beta <- coef[-(times[-1])]
  Gt <- cumsum(c(0,exp(g)))
  if (!is.null(Z)) EZbeta <-  c(exp(Z %*% beta)) else EZbeta <- rep(1,n)
  EZbeta <- rep(EZbeta,each=length(times))
  Gti <- rep(Gt,n)
  GE <- Gti*EZbeta
  Ft <- GE/(1+GE)

  ru <- runif(n)
  out <- data.frame(.Call("wherestrataR",ru,Ft,rep(0:(n-1),each=length(times)),n))
  status <- rep(1,n)
  names(out)[1] <- "time"
  status[out$time==(length(times)-1)] <- 0
  out$status <- status
  out <- cbind(out,Z)

return(out)
}# }}}

##' @export
predictlogitSurvd <- function(x,Z,n=NULL,times=NULL,se=FALSE,type="prob")
{# {{{

if (missing(Z)) Z <- NULL
  if (!is.null(Z))  {
	  n <- nrow(Z)
	  Z <- as.matrix(Z)
  } else { if (is.null(n)) n <- 1; }
  ccc <- x$coef

  ntimes <- x$ntimes
  g <- ccc[1:ntimes]
  beta <- ccc[-(1:ntimes)]
  if (is.null(times)) times <- 0:ntimes 

  Gt <- cumsum(c(0,exp(g)))
  if (!is.null(Z)) EZbeta <-  c(exp(Z %*% beta)) else EZbeta <- rep(1,n)

  if (!se) {# {{{
	  EZbeta <- rep(EZbeta,each=length(times))
	  id <- rep(1:n,each=length(times))
	  Gti <- rep(Gt,n)
	  GE <- Gti*EZbeta
	  Gt <- 1/(1+GE)
	  preds <- data.frame(pred=Gt,id=id,times=rep(times,n))
# }}}
  } else {# {{{

    Ft <- function(p,Zi=1)
    {# {{{
         g <- p[1:ntimes]
         beta <- p[-(1:ntimes)]
         Gt <- cumsum(c(0,exp(g)))
	 if (length(beta)>0) EZ <- exp(sum(Zi*beta)) else EZ <- 1
	 pred <- 1/(1+Gt*EZ)
	 return(pred)
    }# }}}

  preds <- c()
  for (i in 1:length(EZbeta)) {
     if (is.null(x$var)) covv <- vcov(x)  else covv <- x$var
     eud <- estimate(coef=x$coef,vcov=covv,f=function(p) Ft(p,Zi=Z[i,]))
     cmat <- data.frame(eud$coefmat)
     cmat$id <- i
     cmat$times <- times
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- rbind(preds,cmat)
  } 

  }# }}}

return(preds)
} ## }}} 

##' @export
simTTP <- function(coef=NULL,n=100,Xglm=NULL,times=NULL)
{# {{{
	  
  Z <- Xglm  
  if (!is.null(Z)) n <- nrow(Z) 

  if (!is.null(Z)) data <- Z else data <- data.frame(id=1:n)

  if (!is.null(times)) {
     timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
     data <- merge(data,timesf,by.x="id",by.y="id")
     mt <- model.matrix(~factor(times),data)
     nm <- match(c("id","times"),names(data))
     Z <- cbind(mt,data[,-nm])
  }

  p <- c(expit(as.matrix(Z) %*% coef))
  y <- rbinom(length(p),1,p)

  data <- cbind(y,data)
  data <- count.history(data,status="y",id="id",types=1)
  data <- subset(data,data$Count1<=0)

  attr(data,"coef") <- beta
  return(data)
 }# }}}

##' @export
summary.cumoddsreg <- function(object,...) { ## {{{ 
	ntimes <- object$ntimes
	coefb <- coef(object)[1:ntimes]
	vcovb <- object$var[1:object$ntimes,1:object$ntimes]
	outb <- lava::estimate(coef=coefb,vcov=vcovb,...)

	if (length(object$coef)>ntimes) {
		ll <- length(object$coef)
		takex <- (ntimes+1):ll
		coefx <- object$coef[takex]
		vcovx <- object$var[takex,takex]
     	        outx <- lava::estimate(coef=coefx,vcov=vcovx,...)
	        eoutx <- exp(outx$coefmat[,c(1,3,4)])
	        out <- list(baseline=outb,logor=outx,or=eoutx)
	} else out <- list(baseline=outb)
	return(out)
} ## }}} 


##' @export
print.cumoddsreg <- function(x,...) summary(x,...)

##' @export
vcov.cumoddsreg <- function(object,...) return(object$var) 

##' @export
coef.cumoddsreg <- function(object,...) return(object$coef)

##' @export
predictSurvd <- function(ds,Z,times=1:6,se=FALSE,type="prob")
{# {{{
  if (!is.null(Z)) n <- nrow(Z) 
  if (!is.null(Z)) data <- Z else data <- data.frame(id=1:n)
  Z <- data.frame(Z)
  Z$id <- 1:n
  ccc <- ds$coef

  if (!se) {# {{{{{{
	  data <- Z
	  if (!is.null(times)) {
	     timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
	     data <- merge(data,timesf,by.x="id",by.y="id")
	     mt <- model.matrix(~factor(times),data)
	     nm <- match(c("id","times"),names(data))
	     Z <- cbind(mt,data[,-nm])
	  }
	  if (ncol(Z)!=length(c(ccc))) {
		  print(head(Z))
		  print(ccc)
		  stop("dimension of Z not consistent with length of coefficients"); 
	  }

	  p <- c(expit(as.matrix(Z) %*% ccc))

	  preds <- data.frame(p=p,id=data$id,times=data$times)
	  survt <- exp(cumsumstrata(log(1-preds$p),data$id-1,6))
	  if (type=="prob") pred <- 1-survt
	  if (type=="surv") pred <- survt
	  if (type=="hazard") pred <- p
	  if (type=="rrm") { ## restricted residual mean 
		  ll <- length(survt)
	        pred <- cumsum(c(1,survt[-ll]))
	  }
	  preds <- cbind(preds,pred)
# }}}
  } else {# {{{

    Ft <- function(p)
    {
	   xp <- as.matrix(Zi) %*% p
	   lam <- expit(xp)
	   st <- cumprod(1-lam)
	   if (type=="prob") st <- 1-st 
	   if (type=="surv") st <- st 
	   if (type=="hazard") st <- lam
           if (type=="rrm") { ## restricted residual mean 
		ll <- length(st)
	        st <- cumsum(c(1,st[-ll]))
	   }
	   return(st)
    }

  preds <- c()
  for (i in 1:nrow(Z)) {
     Zi <- data.frame(Z[i,,drop=FALSE])
     data <- Zi
     if (!is.null(times)) {
        timesf <- data.frame(times=rep(times,n),id=rep(1:n,each=length(times)))
        data <- merge(data,timesf,by.x="id",by.y="id")
        mt <- model.matrix(~factor(times),data)
        nm <- match(c("id","times"),names(data))
        Zi <- cbind(mt,data[,-nm])
     }
     if (is.null(ds$var)) covv <- vcov(ds)  else covv <- ds$var
     eud <- estimate(coef=ds$coef,vcov=covv,f=function(p) Ft(p))
     cmat <- data.frame(eud$coefmat)
     cmat$id <- i
     cmat$times <- times
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- rbind(preds,cmat)
  } 

  }# }}}

return(preds)
}# }}}

## }}} 

##' @export
plotSurvd <- function(ds,ids=NULL,add=FALSE,se=FALSE,cols=NULL,ltys=NULL,...)
{# {{{

 if (is.null(ids)) ids <- unique(ds$id)
 if (is.null(cols)) cols <- 1:length(ids)
 if (is.null(ltys)) ltys <- 1:length(ids)

  k <- 1
  fplot <- 0
  for (i in ids) {
	  timei <- ds$time[ds$id==i]
	  predi <- ds$pred[ds$id==i]

 	  if (fplot==0) {
	  if (!add) plot(timei,predi,type="s",col=cols[k],lty=ltys[k],...)
	  if (add) lines(timei,predi,type="s",col=cols[k],lty=ltys[k],...)
	  fplot <- 1
	  } else lines(timei,predi,type="s",col=cols[k],lty=ltys[k],...)

          if (se) {
	  loweri <- ds$lower[ds$id==i]
	  upperi <- ds$upper[ds$id==i]
	  plotConfRegion(timei,cbind(loweri,upperi),col=cols[k])
	  }
	  k <- k+1
  }

} ## }}} 

dep.cif<-function(cif,data,cause=NULL,model="OR",cif2=NULL,times=NULL,
                  cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,tol=1e-6,detail=0,
                  clusters=NULL,theta=NULL,theta.des=NULL,step=1,sym=1,weights=NULL,
		  same.cens=FALSE,censoring.weights=NULL,silent=1,entry=NULL,estimator=1,
		  trunkp=1,admin.cens=NULL,control=list(),par.func=NULL,dpar.func=NULL,dimpar=NULL,
		  score.method="nlminb",random.design=NULL,var.link=0,...)
{ ## {{{
  ## {{{ set up data and design
  multi<-0; dscore=1; stab.cens<-FALSE; entry.call<-entry; inverse<-var.link
  notaylor<-1; flex.func<-0; 

  ## extract design and time and cause from cif object 
  time <- cif$response[,"exit"]
  if (is.null(cause)) cause <- cif$response[,"cause"]  ##  attr(cif,"cause"); 
  cause <- as.numeric(cause)
  if (is.null(cens.code)) cens.code <- attr(cif,"cens.code")
  delta<-(cause!=cens.code)
  if (length(cause)!=length(time)) stop("cause and time not of same length\n"); 
  formula <- attr(cif,"Formula")
  ldata <- timereg::aalen.des2(formula(delete.response(terms(formula))),data=data,model="aalen")
  X<-ldata$X;  Z<-ldata$Z;  

  antpers<-nrow(X); 
  if (is.null(Z)==TRUE) {npar<-TRUE; semi<-0;}  else {Z<-as.matrix(Z); npar<-FALSE; semi<-1;}
  if (npar==TRUE) {Z<-matrix(0,antpers,1); pg<-1; fixed<-0;} else {fixed<-1;pg<-ncol(Z);} 
  ng<-antpers;px<-ncol(X);  
  if (is.null(times)) times<-cif$cum[,1]; 
  est<-as.matrix(cif$cum); 
  if (semi==1) gamma<-cif$gamma  else gamma<-0; 
  ntimes<-length(times); 
  if ((cif$model!="additive") && (cif$model!="fg")) 
    stop("Marginal Cumulative incidence model, must be either additive or extended Fine-Gray model\n")
  cif.model <- switch(cif$model,additive=1,fg=2)

  if (cause1!=attr(cif,"causeS")) cat("Cause for marginal model and correlation not the same\n"); 
  if ((cause1[1]!=cause2[1])) {
    if (is.null(cif2)==TRUE) stop("Must provide marginal model for both causes"); 
    formula2<-attr(cif2,"Formula"); 
    ldata2 <- timereg::aalen.des2(formula(delete.response(terms(formula2))),data=data,model="aalen");
    X2<-ldata2$X; Z2<-ldata$Z;  
    if (is.null(Z2)==TRUE) {npar2<-TRUE; semi2<-0;}  else {Z2<-as.matrix(Z2); npar2<-FALSE; semi2<-1;}
    if (npar2==TRUE) {Z2<-matrix(0,antpers,1); pg2<-1; fixed2<-0;} else {fixed2<-1;pg2<-ncol(Z2);} 
    px2<-ncol(X2);  
    est2<-cif2$cum; 
    if (semi2==1) gamma2<-cif2$gamma  else gamma2<-0; 
    est2<-cpred(est2,times);
  } else { 
    X2<-matrix(0,1,1); Z2<-matrix(0,1,1); pg2<-1; px2<-1;  semi2<-0; est2<-matrix(0,1,2); gamma2<-0; 
    npar2<-FALSE; 
  }


### For truncation 
  if (is.null(entry)) entry.call <- NULL else entry.call <- 0
  if (is.null(entry)) entry <- rep(0,antpers); 
  cum1<-cpred(rbind(rep(0,px+1),cif$cum),entry)[,-1];
  if (cif.model==1) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)- (Z %*% gamma )*entry)
  else if (cif.model==2) cif1entry  <-  1-exp(-apply(X*cum1,1,sum)*exp(Z %*% gamma ))
  if ((model!="COR") && (cause1[1]!=cause2[1])) {
    cum2<-cpred(rbind(rep(0,px2+1),cif2$cum),entry)[,-1];
    if (cif.model==1) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)- (Z2 %*% gamma2 )*entry)
    else if (cif.model==2) cif2entry  <-  1-exp(-apply(X2*cum2,1,sum)*exp(Z2 %*% gamma2 ))
  } else {cum2 <- cum1; cif2entry <- cif1entry;}
  ## }}}


  ## {{{ censoring model stuff
  cens.weight <- cif$cens.weights ### censoring weights from cif function
  Gcxe <- 1; 
  ### when censoring probs given, wants them so cens.model="user.weights"
  if (!is.null(censoring.weights))  cens.model <- "user.weights"
  if (cens.model!="user.weights") {
    if (is.null(cens.weight) ) {
      if (cens.model=="KM") { ## {{{
        ud.cens<-survival::survfit(Surv(time,cause==cens.code)~+1); 
        Gfit<-cbind(ud.cens$time,ud.cens$surv)
        Gfit<-rbind(c(0,1),Gfit); 
        Gcx<-cpred(Gfit,time)[,2];
        if (is.null(entry.call)==FALSE) Gcxe<-cpred(Gfit,entry)[,2];
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx ## }}}
      } else if (cens.model=="cox") { ## {{{
        if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
        ud.cens<-timereg::cox.aalen(Surv(time,cause==cens.code)~prop(XZ),n.sim=0,robust=0);
        Gcx<-cpred(ud.cens$cum,time)[,2];
        RR<-exp(XZ %*% ud.cens$gamma)
        Gcx<-exp(-Gcx*RR)
        Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
        if (is.null(entry.call)==FALSE) {
          Gcxe<-cpred(Gfit,entry)[,2];
          Gcxe<-exp(-Gcxe*RR)
        }
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx  ## }}}
      } else if (cens.model=="aalen") {  ## {{{
        if (npar==TRUE) XZ<-X[,-1] else XZ<-cbind(X,Z)[,-1];
        ud.cens<-timereg::aalen(Surv(time,cause==cens.code)~XZ,n.sim=0,robust=0);
        Gcx<-cpred(ud.cens$cum,time)[,-1];
        XZ<-cbind(1,XZ); 
        Gcx<-exp(-apply(Gcx*XZ,1,sum))
        Gcx[Gcx>1]<-1; Gcx[Gcx<0]<-0
        Gfit<-rbind(c(0,1),cbind(time,Gcx)); 
        if (is.null(entry.call)==FALSE) {
          Gcxe<-cpred(Gfit,entry)[,-1];
          Gcxe<-exp(-apply(Gcxe*XZ,1,sum))
          Gcxe[Gcxe>1]<-1; Gcxe[Gcxe<0]<-0
        }
        Gcx <- Gcx/Gcxe; 
        Gctimes<- Gcx  ## }}}
      }
    } else { Gcx <- cens.weight; Gctimes <- cens.weight;} 
  } else {
    Gcx <- censoring.weights
    Gctimes <- cens.weight
  } 
  ntimes<-length(times); 
  ## }}}

  ## {{{ set up cluster + theta design + define iid variables 
  if (is.null(clusters)== TRUE) {
    ## take clusters from cif model 
    clusters <- attr(cif,"clusters"); 
    antclust<- length(unique(clusters)); 
    max.clust <- attr(cif,"max.clust")
    if (attr(cif,"coarse.clust")) 
      stop("Max.clust should be NULL in marginal model, or clusters should be given at call \n"); 
  } else {
    clus<-unique(clusters); antclust<-length(clus);
    clusters <- as.integer(factor(clusters, labels = 1:(antclust)))-1;
  }

  outc <- cluster.index(clusters,index.type=TRUE); 
  clustsize <- outc$cluster.size
  maxclust <- outc$maxclust
  clusterindex <- outc$idclust
  if (maxclust==1) stop("No clusters given \n"); 

 if (model!="ARANCIF") { 
  if (is.null(theta.des)==TRUE) { ptheta<-1; theta.des<-matrix(1,antpers,ptheta);
  colnames(theta.des) <- "intercept"; } else theta.des<-as.matrix(theta.des); 
  ptheta<-ncol(theta.des);
  if (is.null(dimpar) & is.null(par.func)) dimpar<-ptheta; 
 }

 if (model=="ARANCIF") { ### different parameters for Additive random effects 
     if (is.null(random.design)) random.design <- matrix(1,antpers,1); 
     dim.rv <- ncol(random.design); 
     if (is.null(theta.des)==TRUE) theta.des<-diag(dim.rv);
     dimpar <- ncol(theta.des); 
 
     if (nrow(theta.des)!=ncol(random.design)) stop("nrow(theta.des)!= ncol(random.design)"); 
 } else random.design <- matrix(0,1,1); 

  if ( (!is.null(par.func)) && is.null(dimpar) ) 
    stop("Must specify dimension of parameters when specifying R-functions\n")

  if ( (is.null(theta)==TRUE) && (model=="ARANCIF")) theta<-rep(0.5,dimpar);
  if (is.null(theta)==TRUE) theta<-rep(0.1,dimpar);
  if (length(theta)!=dimpar) theta<-rep(theta[1],dimpar);

  Biid<-c(); gamma.iid <- 0; B2iid<-c(); gamma2.iid <- 0; 
  if (notaylor==0) {
    for (i in 1:antclust) Biid<-cbind(Biid,cif$B.iid[[i]]); 
    if (npar==TRUE) gamma.iid<-0 else gamma.iid<-cif$gamma.iid;
    if ((model!="COR") && (cause1!=cause2))  {
      B2iid<-c()
      for (i in 1:antclust) B2iid<-cbind(B2iid,cif2$B.iid[[i]]); 
      if (npar2==TRUE) gamma2.iid<-0 else  gamma2.iid<-cif2$gamma.iid;
    } else { B2iid<-Biid; gamma2.iid<-gamma.iid; }
  }
  var.theta<-hess<-matrix(0,dimpar,dimpar); score<-rep(0,dimpar); 
  time.pow<-attr(cif,"time.pow"); 
  #if (sum(time.pow)==0) time.pow<-rep(1,pg); 
  theta.iid<-matrix(0,antclust,dimpar); 

  if ((nrow(theta.des)!=nrow(X)) && (model!="ARANCIF")) 
	  stop("Dependence design not consistent with data\n"); 
  if (length(trunkp)!=antpers) trunkp <- rep(1,antpers)
  if (is.null(weights)==TRUE) weights <- rep(1,antpers); 
  ## }}}

   ###  ## {{{ function and derivative
  if (is.null(par.func)==FALSE)
  {
     flex.func<-1; # use flexible design
  if (is.null(dpar.func)) {
	  cat("Must provide derivative that is used for iid decomposition for SE's\n"); 
	  score.method <- "nlminb"
	  dpar.func <- par.func
  }
  } ## }}}

  Zgamma <-  c(Z %*% gamma); 
  Z2gamma2 <- c(Z2 %*% gamma2); 
  dep.model <- 0;  
  dep.model <- switch(model,COR=1,RR=2,OR=3,RANCIF=4,ARANCIF=5)
  if (dep.model==0) stop("model must be COR, OR, RR, RANCIF, ARANCIF \n"); 
  if (dep.model<=4) rvdes <- matrix(0,1,1); 
  
  obj <- function(par) 
    { ## {{{

    if (is.null(par.func)==TRUE) {
	    Xtheta <- theta.des %*% par; DXtheta <- array(0,c(1,1,1));
    } else {
	  Xtheta <- c(); DXtheta <- array(0,c(length(times),antpers,dimpar));
	  if (is.null(dpar.func))
		  stop("Must also provide derivative of function wrt to parameters")
	  s <- 0
	  for (t in times) {
		  s <- 1+s
		  Xttheta <- par.func(par,t,theta.des)
		  if (length(Xttheta)!=antpers) stop("parfunc(par,t,theta.des) must length n"); 
		  Xtheta <- cbind(Xtheta,Xttheta)
		  Dttheta <- dpar.func(par,t,theta.des); 
		  if (dim(Dttheta)[1]!=antpers || dim(Dttheta)[2]!=dimpar) 
			  stop("dparfunc must return matrix n x dimpar when called on dparfunc(par,t,theta.des)"); 
		  DXtheta[s,,] <- Dttheta
	  }
    }

      outl<-.Call("cor", ## {{{
                 itimes=times,iy=time,icause=cause,iCA1=cause1,iKMc=Gcx, 
                 iz=X,iest=matrix(est[,-1],ncol=ncol(est)-1),iZgamma=c(Zgamma),isemi=semi,izsem=Z, 
                 itheta=c(par),iXtheta=Xtheta,iDXtheta=DXtheta,idimDX=dim(DXtheta),
		 ithetades=theta.des,
                 icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex,
                 iinverse=inverse,iCA2=cause2,
                 ix2=X2,isemi2=semi2,iest2=as.matrix(est2[,-1]),iZgamma2=c(Z2gamma2),
                 iflexfunc=flex.func,iiid=iid,isym=sym,iweights=weights, 
                 isamecens=as.numeric(same.cens),istabcens=as.numeric(stab.cens),
		 iKMtimes=Gctimes,isilent=silent,
                 icifmodel=cif.model,idepmodel=dep.model,
                 iestimator=estimator,ientryage=entry,icif1entry=cif1entry,
                 icif2entry=cif2entry,itrunkp=trunkp,irvdes=random.design,
                 ## Biid,gamma.iid,time.pow, 
                 ## B2iid,gamma2.iid,body(htheta),body(dhtheta),new.env(),
                 PACKAGE="mets"
		 ) 
      ## }}}

      attr(outl,"gradient") <-outl$score 
      if (oout==0) return(outl$ssf) else if (oout==1) return(sum(outl$score^2)) else return(outl)
    } ## }}}

  p <- theta
  iid <- 0;  ###no-iid representation for iterations
  if (score.method=="nr") { ## {{{
    oout <- 2;  ### output control for obj
    if (Nit>0) 
    for (i in 1:Nit)
    {
        oout <- 2
        out <- obj(p)
	hess <- out$Dscore
	if (!is.na(sum(hess))) hessi <- lava::Inverse(out$Dscore) else hessi <- hess 
        if (detail==1) {## {{{
          print(paste("Fisher-Scoring ===================: it=",i)); 
          cat("theta:");print(c(p))
          cat("score:");print(c(out$score)); 
	  cat("hess:"); print(hess); 
        }## }}}
        delta <- hessi %*% out$score 
	## do not update last iteration 
	if (i<Nit) p <- p-delta* step
	if (is.nan(sum(out$score))) break; 
        if (sum(abs(out$score))<tol) break;
        if (max(delta)>20) { cat("NR increment > 20, lower step zize, increment= \n"); cat(delta); break; }
    }
    if (!is.nan(sum(p))) { ## {{{ iid decomposition
    oout <- 2
    theta <- p
    iid <- 1; 
    out <- obj(p) 
    score <- out$score
    hess <- out$Dscore
    } ## }}} 
    if (detail==1 & Nit==0) {## {{{
          print(paste("Fisher-Scoring ===================: final")); 
          cat("theta:");print(c(p))
          cat("score:");print(c(out$score)); 
	  cat("hess:"); print(hess); 
    }## }}}
    if (!is.na(sum(hess))) hessi <- lava::Inverse(out$Dscore) else hessi <- diag(nrow(hess))
    score1 <- score; 
    ## }}}
  } else if (score.method=="nlminb") { ## {{{ nlminb optimizer
    iid <- 0; oout <- 0; 
    tryCatch(opt <- nlminb(theta,obj,control=control),error=function(x) NA)
    if (detail==1) print(opt); 
    iid <- 1; 
    hess <- numDeriv::hessian(obj,opt$par)
    score <- numDeriv::jacobian(obj,opt$par)
    hessi <- lava::Inverse(hess); 
    theta <- opt$par
    if (detail==1) cat("iid decomposition\n"); 
    oout <- 2; 
    out <- obj(opt$par)
    score1 <- out$score
  ## }}}
  } else if (score.method=="nlm") { ## {{{ nlm optimizer
    iid <- 0; oout <- 0; 
    tryCatch(opt <- nlm(obj,theta,hessian=TRUE,print.level=detail),error=function(x) NA)
    iid <- 1; 
    hess <- opt$hessian
    score <- opt$gradient
    if (detail==1) print(opt); 
    hessi <- lava::Inverse(hess); 
    theta <- opt$estimate
    if (detail==1) cat("iid decomposition\n"); 
    oout <- 2; 
    out <- obj(opt$estimate)
    score1 <- out$score
  ## }}}
  }  else stop("score.methods = nlm nlminb nr\n"); 

  theta.iid <- out$theta.iid %*% hessi
  if (is.null(par.func)) var.theta  <- t(theta.iid) %*% theta.iid else var.theta <- hessi
  var.theta  <- t(theta.iid) %*% theta.iid 
  if (is.null(par.func)) thetanames <- colnames(theta.des) else
  thetanames <- rep("R-func",dimpar)
  ud <- list(theta=theta,score=score,hess=hess,hessi=hessi,var.theta=var.theta,
             theta.iid=theta.iid,score1=c(score1),thetanames=thetanames,
	     brierscore=out$ssf,p11=out$p11); 
  if (dep.model<=3) class(ud)<-"cor" 
  else if (dep.model==4) class(ud) <- "randomcif" 
  else if (dep.model==6) class(ud) <- "randomcif" 
  else if (dep.model==5) class(ud) <- "randomcifrv" 
  attr(ud, "Formula") <- formula
  attr(ud, "Clusters") <- clusters
  attr(ud,"cause1")<-cause1; attr(ud,"cause2")<-cause2
  attr(ud,"sym")<-sym; 
  attr(ud,"inverse")<-inverse; 
  attr(ud,"antpers")<-antpers; 
  attr(ud,"antclust")<-antclust; 
  attr(ud,"var.link")<-var.link; 
  if (dep.model==4) attr(ud, "Type") <- "randomcif"
  if (dep.model==6) attr(ud, "Type") <- "randomcif"
  if (model=="COR") attr(ud, "Type") <- "cor"
  if (model=="RR") attr(ud, "Type") <- "RR"
  if (model=="OR") attr(ud, "Type") <- "OR-cif"

  if (dep.model==5) attr(ud, "pardes") <- theta.des
  if (dep.model==5) attr(ud, "rv1") <- random.design[1,]
  return(ud);
} ## }}}

##' Cross-odds-ratio, OR or RR risk regression for competing risks
##'
##' Fits a parametric model for the log-cross-odds-ratio for the 
##' predictive effect of for the cumulative incidence curves for \eqn{T_1} 
##' experiencing cause i given that \eqn{T_2} has experienced a cause k :
##' \deqn{
##' \log(COR(i|k))  =  h(\theta,z_1,i,z_2,k,t)=_{default}  \theta^T z =  
##' }
##' with the log cross odds ratio being 
##' \deqn{
##' COR(i|k) = 
##' \frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' O(T_1 \leq t,cause_1=i)}  
##' }
##' the conditional odds divided by the unconditional odds, with the odds
##' being, respectively 
##' \deqn{
##' O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_1=k) = 
##' \frac{
##' P_x(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' P_x((T_1 \leq t,cause_1=i)^c | T_2 \leq t,cause_2=k)}
##' }
##' and 
##' \deqn{
##' O(T_1 \leq t,cause_1=i) = 
##' \frac{P_x(T_1 \leq t,cause_1=i )}{P_x((T_1 \leq t,cause_1=i)^c )}.
##' }
##' Here \eqn{B^c} is the complement event of \eqn{B},
##' \eqn{P_x} is the distribution given covariates 
##' (\eqn{x} are subject specific and \eqn{z} are cluster specific covariates), and 
##' \eqn{h()} is a function that is the simple identity 
##' \eqn{\theta^T z} by default.
##' 
##' The OR dependence measure is given by 
##' \deqn{
##' OR(i,k) = 
##' \log ( 
##' \frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
##' O(T_1 \leq t,cause_1=i) | T_2 \leq t,cause_2=k)}  
##' }
##' This measure is numerically more stabile than the COR measure, and is symetric in i,k.
##' 
##' The RR dependence measure is given by 
##' \deqn{
##' RR(i,k) = 
##' \log ( 
##' \frac{P(T_1 \leq t,cause_1=i , T_2 \leq t,cause_2=k)}{
##' P(T_1 \leq t,cause_1=i) P(T_2 \leq t,cause_2=k)}  
##' }
##' This measure is numerically more stabile than the COR measure, and is symetric in i,k.
##' 
##' The model is fitted under symmetry (sym=1), i.e., such that it is assumed 
##' that \eqn{T_1} and \eqn{T_2} can be interchanged and leads to
##' the same cross-odd-ratio (i.e.
##' \eqn{COR(i|k) = COR(k|i))}, 
##' as would be expected for twins 
##' or without symmetry as might be the case with mothers and daughters (sym=0). 
##' 
##' \eqn{h()} may be specified as an R-function of the parameters, 
##' see example below, but the default is that it is simply \eqn{\theta^T z}.
##'
##' @aliases or.cif rr.cif
##' @param cif a model object from the timereg::comp.risk function with the
##' marginal cumulative incidence of cause1, i.e., the event of interest, and whose
##' odds the comparision is compared to the conditional odds given cause2
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value. When missing it comes from marginal cif.
##' @param times time-vector that specifies the times used for the estimating euqations for the cross-odds-ratio estimation.
##' @param cause1 specificies the cause considered.
##' @param cause2 specificies the cause that is conditioned on.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param step specifies the step size for the Newton-Raphson algorithm.
##' @param sym specifies if symmetry is used in the model.
##' @param weights weights for estimating equations.
##' @param par.func parfunc
##' @param dpar.func dparfunc
##' @param dimpar dimpar
##' @param score.method "nlminb", can also use "nr".
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param censoring.weights these probabilities are used for the bivariate censoring dist.
##' @param silent 1 to suppress output about convergence related issues.
##' @param ... Not used.
##' @return returns an object of type 'cor'. With the following arguments:
##' \item{theta}{estimate of proportional odds parameters of model.}
##' \item{var.theta}{variance for gamma.  }
##' \item{hess}{the derivative of the used score.}
##' \item{score}{scores at final stage.}
##' \item{score}{scores at final stage.}
##' \item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @author Thomas Scheike
##' @references
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
##' 
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' @examples
##' library("timereg")
##' data(multcif);
##' multcif$cause[multcif$cause==0] <- 2
##' zyg <- rep(rbinom(200,1,0.5),each=2)
##' theta.des <- model.matrix(~-1+factor(zyg))
##' 
##' times=seq(0.05,1,by=0.05) # to speed up computations use only these time-points
##' add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=1,
##'                n.sim=0,times=times,model="fg",max.clust=NULL)
##' add2 <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=2,
##'                n.sim=0,times=times,model="fg",max.clust=NULL)
##' 
##' out1 <- cor.cif(add,data=multcif,cause1=1,cause2=1)
##' summary(out1)
##' 
##' out2 <- cor.cif(add,data=multcif,cause1=1,cause2=1,theta.des=theta.des)
##' summary(out2)
##' 
##' ##out3 <- cor.cif(add,data=multcif,cause1=1,cause2=2,cif2=add2)
##' ##summary(out3)
##' ###########################################################
##' # investigating further models using parfunc and dparfunc
##' ###########################################################
##' \donttest{ ## Reduce Ex.Timings
##' set.seed(100)
##' prt<-simnordic.random(2000,cordz=2,cormz=5)
##' prt$status <-prt$cause
##' table(prt$status)
##' 
##' times <- seq(40,100,by=10)
##' cifmod <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=prt,
##'                     cause=1,n.sim=0,
##'                     times=times,conservative=1,max.clust=NULL,model="fg")
##' theta.des <- model.matrix(~-1+factor(zyg),data=prt)
##' 
##' parfunc <- function(par,t,pardes)
##' {
##' par <- pardes %*% c(par[1],par[2]) +
##'        pardes %*% c( par[3]*(t-60)/12,par[4]*(t-60)/12)
##' par
##' }
##' head(parfunc(c(0.1,1,0.1,1),50,theta.des))
##' 
##' dparfunc <- function(par,t,pardes)
##' {
##' dpar <- cbind(pardes, t(t(pardes) * c( (t-60)/12,(t-60)/12)) )
##' dpar
##' }
##' head(dparfunc(c(0.1,1,0.1,1),50,theta.des))
##' 
##' names(prt)
##' or1 <- or.cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
##'               same.cens=TRUE,theta=c(0.6,1.1,0.1,0.1),
##'               par.func=parfunc,dpar.func=dparfunc,dimpar=4,
##'               score.method="nr",detail=1)
##' summary(or1)
##' 
##'  cor1 <- cor.cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
##'                  same.cens=TRUE,theta=c(0.5,1.0,0.1,0.1),
##'                  par.func=parfunc,dpar.func=dparfunc,dimpar=4,
##'                  control=list(trace=TRUE),detail=1)
##' summary(cor1)
##' 
##' ### piecewise contant OR model
##' gparfunc <- function(par,t,pardes)
##' {
##' 	cuts <- c(0,80,90,120)
##' 	grop <- diff(t<cuts)
##' paru  <- (pardes[,1]==1) * sum(grop*par[1:3]) +
##'     (pardes[,2]==1) * sum(grop*par[4:6])
##' paru
##' }
##' 
##' dgparfunc <- function(par,t,pardes)
##' {
##' 	cuts <- c(0,80,90,120)
##' 	grop <- diff(t<cuts)
##' par1 <- matrix(c(grop),nrow(pardes),length(grop),byrow=TRUE)
##' parmz <- par1* (pardes[,1]==1)
##' pardz <- (pardes[,2]==1) * par1
##' dpar <- cbind( parmz,pardz)
##' dpar
##' }
##' head(dgparfunc(rep(0.1,6),50,theta.des))
##' head(gparfunc(rep(0.1,6),50,theta.des))
##' 
##' or1g <- or.cif(cifmod,data=prt,cause1=1,cause2=1,
##'                theta.des=theta.des, same.cens=TRUE,
##'                par.func=gparfunc,dpar.func=dgparfunc,
##'                dimpar=6,score.method="nr",detail=1)
##' summary(or1g)
##' names(or1g)
##' head(or1g$theta.iid)
##' }
##' @export
##' @keywords survival
cor.cif<-function(cif,data,cause=NULL,times=NULL,
                  cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                  clusters=NULL, theta=NULL,theta.des=NULL,step=1,sym=0,weights=NULL, 
		  par.func=NULL,dpar.func=NULL,dimpar=NULL,
		  score.method="nlminb",same.cens=FALSE,censoring.weights=NULL,silent=1,...)
{ ## {{{
  fit <- dep.cif(cif=cif,data=data,cause=cause,model="COR",times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des,par.func=par.func,dpar.func=dpar.func,
		 dimpar=dimpar,
                 step=step,sym=sym,weights=weights,
                 score.method=score.method,same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,...)
  fit$call <- match.call()
  fit
} ## }}}

##' @export
rr.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
                 cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                 clusters=NULL, theta=NULL,theta.des=NULL, step=1,sym=0,weights=NULL,
                 same.cens=FALSE,censoring.weights=NULL,silent=1,par.func=NULL,dpar.func=NULL,dimpar=NULL,
		 score.method="nlminb",entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,...)
{ ## {{{
  fit <- dep.cif(cif=cif,data=data,cause=cause,model="RR",cif2=cif2,times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des, step=step,sym=sym,weights=weights,
                 same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,
		 par.func=par.func,dpar.func=dpar.func,dimpar=dimpar, score.method=score.method,
                 entry=entry,estimator=estimator,trunkp=trunkp,admin.cens=admin.cens,...)
  fit$call <- match.call()
  fit
} ## }}}
##' @export
or.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
                 cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                 clusters=NULL, theta=NULL,theta.des=NULL, step=1,sym=0, weights=NULL,
                 same.cens=FALSE,censoring.weights=NULL,silent=1,par.func=NULL,dpar.func=NULL,dimpar=NULL,
		 score.method="nlminb",entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,...)
{ ## {{{
  fit <- dep.cif(cif=cif,data=data,cause=cause,model="OR",cif2=cif2,times=times,
                 cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
                 clusters=clusters,theta=theta,theta.des=theta.des, step=step,sym=sym,weights=weights,
                 same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,
		 par.func=par.func,dpar.func=dpar.func,dimpar=dimpar,
		 score.method=score.method,
                 entry=entry,estimator=estimator,trunkp=trunkp,admin.cens=admin.cens,...)
  fit$call <- match.call()
  fit
} ## }}}

##' Fits a random effects  model describing the dependence in the cumulative 
##' incidence curves for subjects within a cluster.  Given the gamma distributed
##' random effects it is assumed that the cumulative incidence curves are indpendent, and
##' that the marginal cumulative incidence curves are on the form
##' \deqn{
##' P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) exp(z^T \beta))
##' }
##' We allow a regression structure for the random effects variances that may depend on
##' cluster covariates.
##'
##' @title Random effects model for competing risks data
##' @param cif a model object from the comp.risk function with the 
##' marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
##' odds the comparision is made with respect to
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value.
##' @param cif2 specificies model for cause2 if different from cause1.
##' @param cause1 cause of first coordinate.
##' @param cause2 cause of second coordinate.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param sym 1 for symmetry 0 otherwise
##' @param step specifies the step size for the Newton-Raphson algorith.m
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param var.link if var.link=1 then var is on log-scale.
##' @param score.method default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.
##' @param entry entry-age in case of delayed entry. Then two causes must be given.
##' @param trunkp gives probability of survival for delayed entry, and related to entry-ages given above.
##' @param ... extra arguments.
##' @return returns an object of type 'cor'. With the following arguments:
##' \item{theta}{estimate of proportional odds parameters of model.}
##' \item{var.theta}{variance for gamma.  }
##' \item{hess}{the derivative of the used score.}
##' \item{score}{scores at final stage.}
##' \item{score}{scores at final stage.}
##' \item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @export
##' @references
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' 
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), work in progress.
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##'  d <- simnordic.random(5000,delayed=TRUE,cordz=0.5,cormz=2,lam0=0.3,country=TRUE)
##'  times <- seq(50,90,by=10)
##'  add1 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'  times=times,cause=1,max.clust=NULL)
##' 
##'  ### making group indidcator 
##'  mm <- model.matrix(~-1+factor(zyg),d)
##' 
##'  out1<-random.cif(add1,data=d,cause1=1,cause2=1,theta=1,same.cens=TRUE)
##'  summary(out1)
##' 
##'  out2<-random.cif(add1,data=d,cause1=1,cause2=1,theta=1,
##' 		   theta.des=mm,same.cens=TRUE)
##'  summary(out2)
##' 
##' #########################################
##' ##### 2 different causes
##' #########################################
##' 
##'  add2 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'                   times=times,cause=2,max.clust=NULL)
##'  out3 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,sym=1,same.cens=TRUE)
##'  summary(out3) ## negative dependence
##' 
##'  out4 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,theta.des=mm,sym=1,same.cens=TRUE)
##'  summary(out4) ## negative dependence
##' }
##' @keywords survival
##' @author Thomas Scheike
random.cif<-function(cif,data,cause=NULL,cif2=NULL,
                     cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
                     clusters=NULL,theta=NULL,theta.des=NULL,sym=1,
                     step=1,same.cens=FALSE,var.link=0,score.method="nr",
                     entry=NULL,trunkp=1,...)
{ ## {{{
  fit <- dep.cif(cif,data=data,cause=cause,model="RANCIF",cif2=cif2,sym=sym,
     cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
     clusters=clusters,theta=theta,theta.des=theta.des,step=step,same.cens=same.cens,
     var.link=var.link,score.method=score.method,entry=entry,trunkp=trunkp,...)
  fit$call <- match.call()
  fit
} ## }}}

##'Additive Random effects model for competing risks data for polygenetic modelling
##' 
##'Fits a random effects  model describing the dependence in the cumulative 
##'incidence curves for subjects within a cluster.  Given the gamma distributed
##'random effects it is assumed that the cumulative incidence curves 
##'are indpendent, and that the marginal cumulative incidence curves 
##'are on additive form
##'\deqn{
##'P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) - t z^T \beta)
##'}
##'
##'We allow a regression structure for the indenpendent gamma distributed 
##'random effects  and their variances that may depend on cluster covariates.
##'
##'random.design specificies the random effects for each subject within a cluster. This is
##'a matrix of 1's and 0's with dimension n x d.  With d random effects. 
##'For a cluster with two subjects, we let the random.design rows be 
##' \eqn{v_1} and \eqn{v_2}. 
##'Such that the random effects for subject 
##'1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
##'has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
##'subjects 1's random effect are Gamma distributed with 
##'mean \eqn{\lambda_1/v_1^T \lambda}
##'and variance \eqn{\lambda_1/(v_1^T \lambda)^2}. Note that the random effect 
##'\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
##'
##'The parameters \eqn{(\lambda_1,...,\lambda_d)}
##'are related to the parameters of the model
##'by a regression construction \eqn{pard} (d x k), that links the \eqn{d} 
##'\eqn{\lambda} parameters
##'with the (k) underlying \eqn{\theta} parameters 
##'\deqn{
##'\lambda = pard \theta 
##'}
##'
##'
##'
##' @param cif a model object from the timereg::comp.risk function with the
##' marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
##' odds the comparision is made with respect to
##' @param data a data.frame with the variables.
##' @param cause specifies the causes  related to the death
##' times, the value cens.code is the censoring value.
##' @param cif2 specificies model for cause2 if different from cause1.
##' @param times time points
##' @param cause1 cause of first coordinate.
##' @param cause2 cause of second coordinate.
##' @param cens.code specificies the code for the censoring if NULL then uses the one from the marginal cif model.
##' @param cens.model specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"
##' @param Nit number of iterations for Newton-Raphson algorithm.
##' @param detail if 0 no details are printed during iterations, if 1 details are given.
##' @param clusters specifies the cluster structure.
##' @param theta specifies starting values for the cross-odds-ratio parameters of the model.
##' @param theta.des specifies a regression design for the cross-odds-ratio parameters.
##' @param step specifies the step size for the Newton-Raphson algorith.m
##' @param sym 1 for symmetri and 0 otherwise
##' @param weights weights for score equations.
##' @param same.cens if true then censoring within clusters are assumed to be the same variable, default is independent censoring.
##' @param censoring.weights Censoring probabilities
##' @param silent debug information 
##' @param var.link if var.link=1 then var is on log-scale.
##' @param score.method default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.
##' @param entry entry-age in case of delayed entry. Then two causes must be given.
##' @param estimator estimator
##' @param trunkp gives probability of survival for delayed entry, and related to entry-ages given above.
##' @param admin.cens Administrative censoring
##' @param random.design specifies a regression design of 0/1's for the random effects.
##' @param ... extra arguments.
##' @return returns an object of type 'random.cif'. With the following arguments:
##'\item{theta}{estimate of parameters of model.}
##'\item{var.theta}{variance for gamma.  }
##'\item{hess}{the derivative of the used score.}
##'\item{score}{scores at final stage.}
##'\item{theta.iid}{matrix of iid decomposition of parametric effects.}
##' @export
##' @references
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika.
##'
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2013), Biostatitistics.
##'
##' Scheike, Holst, Hjelmborg (2014),  LIDA,  
##' Estimating heritability for cause specific hazards based on twin data
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##'  d <- simnordic.random(5000,delayed=TRUE,
##'        cordz=1.0,cormz=2,lam0=0.3,country=TRUE)
##'  times <- seq(50,90,by=10)
##'  addm <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
##'  times=times,cause=1,max.clust=NULL)
##' 
##'  ### making group indidcator 
##'  mm <- model.matrix(~-1+factor(zyg),d)
##' 
##'  out1m<-random.cif(addm,data=d,cause1=1,cause2=1,theta=1,
##' 		   theta.des=mm,same.cens=TRUE)
##'  summary(out1m)
##'  
##'  ## this model can also be formulated as a random effects model 
##'  ## but with different parameters
##'  out2m<-Grandom.cif(addm,data=d,cause1=1,cause2=1,
##' 		    theta=c(0.5,1),step=1.0,
##' 		    random.design=mm,same.cens=TRUE)
##'  summary(out2m)
##'  1/out2m$theta
##'  out1m$theta
##'  
##'  ####################################################################
##'  ################### ACE modelling of twin data #####################
##'  ####################################################################
##'  ### assume that zygbin gives the zygosity of mono and dizygotic twins
##'  ### 0 for mono and 1 for dizygotic twins. We now formulate and AC model
##'  zygbin <- d$zyg=="DZ"
##' 
##'  n <- nrow(d)
##'  ### random effects for each cluster
##'  des.rv <- cbind(mm,(zygbin==1)*rep(c(1,0)),(zygbin==1)*rep(c(0,1)),1)
##'  ### design making parameters half the variance for dizygotic components
##'  pardes <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0), c(0,1))
##' 
##'  outacem <-Grandom.cif(addm,data=d,cause1=1,cause2=1,
##' 		same.cens=TRUE,theta=c(0.35,0.15),
##'             step=1.0,theta.des=pardes,random.design=des.rv)
##'  summary(outacem)
##' 
##' }
##' @keywords survival
##' @author Thomas Scheike
##' @export
Grandom.cif<-function(cif,data,cause=NULL,cif2=NULL,times=NULL,
cause1=1,cause2=1,cens.code=NULL,cens.model="KM",Nit=40,detail=0,
clusters=NULL, theta=NULL,theta.des=NULL, weights=NULL, step=1,sym=0,
same.cens=FALSE,censoring.weights=NULL,silent=1,var.link=0,score.method="nr",
entry=NULL,estimator=1,trunkp=1,admin.cens=NULL,random.design=NULL,...)
{ ## {{{
fit <- dep.cif(cif,data=data,cause=cause,model="ARANCIF",cif2=cif2,times=times,
         cause1=cause1,cause2=cause2,cens.code=cens.code,cens.model=cens.model,Nit=Nit,detail=detail,
         clusters=clusters,theta=theta,theta.des=theta.des,step=step,sym=sym,weights=weights,
         same.cens=same.cens,censoring.weights=censoring.weights,silent=silent,var.link=var.link,
	 score.method=score.method,entry=entry,estimator=estimator,
	 random.design=random.design,trunkp=trunkp,admin.cens=admin.cens,...)
    fit$call <- match.call()
    fit
} ## }}}

##' @export
print.summary.cor <- function(x,digits=3,...)
{ ## {{{
  if (x$type=="cor") {
    cat("Cross odds ratio dependence for competing risks\n\n")
    cat("Effect of cause1=",x$cause1," on cause2=",x$cause2,
        " under symmetry=",x$sym,fill=TRUE,sep="")
  } else if (x$type=="RR") {
    cat("Ratio of joint and product of marginals for competing risks\n\n")
    cat("Ratio of cumulative incidence for cause1=",x$cause1," and cause2=",x$cause2,sep=" ")
  } else if (x$type=="OR-cif") {
    cat("OR for dependence for competing risks\n\n")
    cat("OR of cumulative incidence for cause1=",x$cause1," and cause2=",x$cause2,sep=" ")
  }
  cat("\n")
  prmatrix(signif(x$estimates,digits))
  cat("\n")
  if (!is.null(x$marg)) {
    cat(paste("Marginal cumulative incidencen",signif(x$marg,digits),"\n"))
    prmatrix(signif(x$casewise,digits))
    prmatrix(signif(x$concordance,digits))
    cat("\n")
  }
  invisible(x)
} ## }}}

##' Computes concordance and casewise concordance for dependence models for competing risks 
##' models of the type cor.cif, rr.cif or or.cif for the given cumulative incidences and the different dependence
##' measures in the object.
##'
##'
##' @title Summary for dependence models for competing risks
##' @param object object from cor.cif rr.cif or or.cif for dependence between competing risks data for two causes.
##' @param marg.cif a number that gives the cumulative incidence in one time point for which concordance and 
##' casewise concordance are computed.
##' @param marg.cif2 the cumulative incidence for cause 2 for concordance and 
##' casewise concordance are computed. Default is that it is the same as marg.cif.
##' @param digits digits in output.
##' @param ... Additional arguments.
##' @return prints summary for dependence model. 
##' \item{casewise}{gives casewise concordance that is, probability of cause 2 (related to cif2) given that cause 1 (related to cif1)
##' 	has occured.}
##' \item{concordance}{gives concordance that is, probability of cause 2 (related to cif2) and cause 1 (related to cif1).}
##' \item{cif1}{cumulative incidence for cause1.}
##' \item{cif2}{cumulative incidence for cause1.}
##' @references
##' Cross odds ratio Modelling of dependence for
##' Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 
##' 
##' A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
##' Scheike, Zhang, Sun, Jensen (2010), Biometrika. 
##' 
##' @author Thomas Scheike
##' @keywords survival
##' @examples
##' ## library("timereg")
##' ## data("multcif",package="mets") # simulated data 
##' ## multcif$cause[multcif$cause==0] <- 2
##' ##  
##' ## times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
##' ## add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),
##' ##                           data=multcif,n.sim=0,times=times,cause=1)
##' ###
##' ## out1<-cor.cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
##' ## summary(out1)
##' ## 
##' ## pad <- predict(add,X=1,se=0,uniform=0)
##' ## summary(out1,marg.cif=pad)
##' @method summary cor
##' @export
summary.cor <- function(object,marg.cif=NULL,marg.cif2=NULL,digits=3,...) { ## {{{
  if (!(attr(object,"class") %in% c("cor","randomcif"))) stop("Must be a cor.cif or randomcif object")
  if (sum(abs(object$score))>0.001) warning("WARNING: check score for convergence\n")

  coefs <- coef(object,...) 

  ocasewise <- oconcordance <- NULL
  if (is.null(marg.cif)==FALSE) { ## {{{ 
    time <- marg.cif$time
    marg.cif <- marg.cif$P1
    if (is.null(marg.cif2)==FALSE) marg.cif2 <- marg.cif2$P1 else {
    if (attr(object,"cause2")==attr(object,"cause1")) marg.cif2 <- marg.cif 
    else stop("causes not the same and second marginal cif not given\n"); 
    }
    pmarg.cif <- marg.cif*marg.cif2

    thetav <- coefs[,1]
    thetavl <- thetav-1.96*coefs[,2]
    thetavu <- thetav+1.96*coefs[,2]
    namev <- object$thetanames
    if (is.null(namev)) namev <- paste("name",1:length(thetav),sep="")

    ocasewise <- oconcordance <- list()
    k <- 0
    for (theta in thetav) { 
       k <- k+1
       thetal <- thetavl[k]; 
       thetau <- thetavu[k]
    if (attr(object,"Type")=="cor") { ## {{{
      concordance <- exp(theta)*pmarg.cif/((1-marg.cif)+exp(theta)*marg.cif)
      conclower  <- exp(thetal)*pmarg.cif/((1-marg.cif)+exp(thetal)*marg.cif)
      concup  <- exp(thetau)*pmarg.cif/((1-marg.cif)+exp(thetau)*marg.cif)
      casewise <- concordance/marg.cif
      caselower  <- conclower/marg.cif
      caseup     <- concup/marg.cif
    } else if (attr(object,"Type")=="RR") {
      casewise<- exp(coefs[,1])*c(marg.cif)
      concordance <- exp(coefs[,1])*pmarg.cif
      caselower  <- marg.cif*exp(thetal)
      caseup     <- marg.cif*exp(thetau)
      conclower  <- pmarg.cif* exp(thetal)
      concup     <- pmarg.cif*exp(thetau)
    } else if (attr(object,"Type")=="OR-cif") {
      casewise<- plack.cif2(marg.cif,marg.cif,c(theta))/marg.cif
      concordance <- plack.cif2(marg.cif,marg.cif,c(theta))
      caselower  <- plack.cif2(marg.cif,marg.cif,thetal)/marg.cif
      caseup  <- plack.cif2(marg.cif,marg.cif,thetau)/marg.cif
      conclower  <- plack.cif2(marg.cif,marg.cif,thetal)
      concup     <- plack.cif2(marg.cif,marg.cif,thetau)
    } else if (attr(object,"Type")=="randomcif") {
	    theta <- 1/theta
	    thetal <- 1/thetal
	    thetau <- 1/thetau
      lam <- 1-marg.cif
      p11<- 1-lam -lam +lap(theta,2*ilap(theta, lam))
      concordance <- p11 
      conclower  <- 1-lam -lam +lap(thetal,2*ilap(thetal, lam))
      concup     <- 1-lam -lam +lap(thetau,2*ilap(thetau, lam))
      casewise <- concordance/marg.cif
      caselower  <- conclower/marg.cif
      caseup     <- concup/marg.cif
    } ## }}} 
    outcase <- cbind(time,casewise,caselower,caseup)
    outconc <- cbind(time,concordance,conclower,concup)
    colnames(outcase) <- c("time","casewise concordance","2.5 %","97.5%")
    colnames(outconc) <- c("time","concordance","2.5 %","97.5%")
	  if (length(thetav)==1) {
	     ocasewise <- outcase
	     oconcordance <- outconc
	  } else {
	     ocasewise[[k]] <- outcase
	     oconcordance[[k]] <- outconc
	  }
  } 

  if (length(thetav)>1) {
  if (length(ocasewise)==length(namev)) {
    names(ocasewise) <- namev
    names(oconcordance) <- namev
  }
  }

  } ## }}} 


  res <- list(casewise=ocasewise,concordance=oconcordance,estimates=coefs,
	      marg.cif=marg.cif, marg.cif2=marg.cif2,type=attr(object,"Type"),
	      sym=attr(object,"sym"),cause1=attr(object,"cause1"),cause2=attr(object,"cause2"))
  class(res) <- "summary.cor"
  res
} ## }}}

##' @export
coef.cor <- function(object,...)
{ ## {{{
  res <- cbind(object$theta, diag(object$var.theta)^0.5)
  se<-diag(object$var.theta)^0.5
  wald <- object$theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  cor<-exp(object$theta)
  res <- as.matrix(cbind(res, wald, waldp,cor,se*cor))
  if (attr(object,"Type")=="cor") 
    colnames(res) <- c("log-Coef.", "SE", "z", "P-val","Cross odds ratio","SE")
  else colnames(res) <- c("log-ratio Coef.", "SE", "z", "P-val","Ratio","SE")
  if (!is.null(object$thetanames)) rownames(res)<-object$thetanames

  return(res)
} ## }}}

##' @export
print.cor<-function(x,digits=3,...)
{ ## {{{
  print(x$call); 
  cat("\n")
  print(summary(x)); 
} ## }}}

##' @export
concordance.cor <- function(object,...) {
    concordanceCor(object,...)
}

##' Concordance for Twins
##'
##' The concordance is the probability that both twins have experienced the 
##' event of interest and is defined as 
##' \deqn{
##'  cor(t) = P(T_1 \leq t, \epsilon_1 =1 , T_2 \leq t, \epsilon_2=1) 
##'  }
##'
##' Similarly, the casewise concordance is 
##' \deqn{
##'  casewise(t) = \frac{cor(t)}{P(T_1 \leq t, \epsilon_1=1) }
##' }
##' that is the probability that twin "2" has the event given that twins "1" has. 
##'
##' @title Concordance Computes concordance and casewise concordance
##' @param object Output from the cor.cif, rr.cif or or.cif function
##' @param cif1 Marginal cumulative incidence
##' @param cif2 Marginal cumulative incidence of other cause (cause2) if  it is different from cause1
##' @param messages  To print messages
##' @param model  Specfifies wich model that is considered if object not given.
##' @param coefs Specfifies  dependence parameters if object is not given.
##' @param ...  Extra arguments, not used.
##' @author Thomas Scheike
##' @aliases concordanceCor concordance.cor 
##' @references
##' Estimating twin concordance for bivariate competing risks twin data
##' Thomas H. Scheike, Klaus K. Holst and Jacob B. Hjelmborg, 
##' Statistics in Medicine 2014, 1193-1204
##' 
##' Estimating Twin Pair Concordance for Age of Onset.
##' Thomas H. Scheike, Jacob V B Hjelmborg, Klaus K. Holst, 2015 
##' in Behavior genetics DOI:10.1007/s10519-015-9729-3
##' 
##' @export 
concordanceCor <- function(object,cif1,cif2=NULL,messages=TRUE,model=NULL,coefs=NULL,...)
{ ## {{{

  if (is.null(model)) { 
    if (!inherits(object, "cor")) stop("Must be a rr.cif, cor.cif or or.cif object")
    model <- attr(object,"Type")
  } 
  if (is.null(coefs)) coefs <- coef(object)
  if (is.null(coefs)) stop("Must give dependence parameters\n"); 

  if (!is.null(object)) { 
    cause1 <-  attr(object,"cause1");
    cause2 <-  attr(object,"cause2"); 
  } else cause1 <- cause2 <- 1

  if (is.null(cif2)==TRUE) cif2 <- cif1; 

  if (messages) {
    if (model=="cor") { ## {{{
      message("Cross odds ratio dependence for competing risks\n\n")
      message("Odds of cause1=",cause1," given cause2=",cause2," relative to Odds of cause1=",cause1,"\n",fill=TRUE,sep="")
    } else if (model=="RR") {
      message("Ratio of joint and product of marginals for competing risks\n\n")
      message("Ratio of cumulative incidence for cause1=",cause1," and cause2=",cause2,sep=" ")
    } else if (model=="OR-cif") {
      message("OR for dependence for competing risks\n\n")
      message("OR of cumulative incidence for cause1=",cause1," and cause2=",cause2,sep=" ")
    } ## }}}
  }

  out <- list()
  for (k in 1:nrow(coefs)) {
    ## {{{
    if (model=="cor") {
      concordance <- exp(coefs[k,1])*cif1*cif2/((1-cif1)+exp(coefs[k,1])*cif1)
      conclower  <- exp(coefs[k,1]-1.96*coefs[k,2])*cif1*cif2/((1-cif1)+exp(coefs[k,1]-1.96*coefs[k,2])*cif1)
      concup  <- exp(coefs[k,1]+1.96*coefs[k,2])*cif1*cif2/((1-cif1)+exp(coefs[k,1]+1.96*coefs[k,2])*cif1)
      casewise <- concordance/cif1
      caselower  <- conclower/cif1
      caseup     <- concup/cif1
    } else if (model=="RR") {
      casewise<- exp(coefs[k,1])*c(cif2)
      concordance <- exp(coefs[k,1])*cif1*cif2
      caselower  <- cif2*exp(coefs[k,1]-1.96*coefs[k,2])
      caseup     <- cif2*exp(coefs[k,1]+1.96*coefs[k,2])
      conclower  <- cif1*cif2* exp(coefs[k,1]-1.96*coefs[k,2])
      concup     <- cif1*cif2*exp(coefs[k,1]+1.96*coefs[k,2])
    } else if (model=="OR-cif") {
      thetal <-  coefs[k,1]-1.96*coefs[k,2]
      thetau <-  coefs[k,1]+1.96*coefs[k,2]
      casewise<- plack.cif2(cif1,cif2,c(coefs[k,1]))/cif1
      concordance <- plack.cif2(cif1,cif2,c(coefs[k,1]))
      caselower  <- plack.cif2(cif1,cif2,thetal)/cif1
      caseup  <- plack.cif2(cif1,cif2,thetau)/cif1
      conclower  <- plack.cif2(cif1,cif2,thetal)
      concup     <- plack.cif2(cif1,cif2,thetau)
    }

    outcase <- cbind(c(casewise),c(caselower),c(caseup))
    outconc <- cbind(c(concordance),c(conclower),c(concup))
    colnames(outcase) <- c("casewise concordance","2.5 %","97.5%")
    colnames(outconc) <- c("concordance","2.5 %","97.5%")
    ## }}}

    out[[k]] <- list(concordance=outconc,casewise=outcase)
    names(out)[k] <- rownames(coefs)[k]
  }

  return(out)
} ## }}}

##' .. content for description (no empty lines) ..
##'
##' @title plack Computes concordance for or.cif based model, that is Plackett random effects model 
##' @aliases plack.cif2
##' @export
##' @param cif1 Cumulative incidence of first argument.
##' @param cif2 Cumulative incidence of second argument.
##' @param object or.cif object with dependence parameters.
##' @author Thomas Scheike
plack.cif <- function(cif1,cif2,object) 
{ ## {{{
  coefs <- coef(object)
  theta <- exp(object$theta); 
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;
  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  valr <- matrix(valr,length(c(theta)),1)
  rownames(valr)=colnames(coefs)
  return(valr); 
} ## }}}

##' @export
plack.cif2 <- function(cif1,cif2,theta) 
{ ## {{{
  theta <- exp(c(theta))
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;

  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  return(valr); 
} ## }}}

##' @export
predictPairPlack <- function(cif1,cif2,status1,status2,theta) 
{ ## {{{
  theta <- exp(c(theta))
  cif1 <- c(cif1); cif2 <- c(cif2)
  cifs=cif1+cif2; 

  valn=2*(theta-1); 
  val1=(1+(theta-1)*(cifs))-( ((1+(theta-1)*cifs))^2-4*cif1*cif2*theta*(theta-1))^0.5; 
  vali=cif1*cif2;

  valr <- vali;
  valr[valn!=0] <- val1/valn; 

  p11 <- valr; 
  p10 <- cif1-p11
  p01 <- cif2-p11
  p00 <- 1- p10-p01-p11
#
  pred <- (status1==1)*(status2==1)*p11+ (status1==1)*(status2==0)*p10+ 
          (status1==0)*(status2==1)*p01+ (status1==0)*(status2==0)*p00

  return(pred); 
} ## }}}

##' @export
summary.randomcif<-function (object, ...) 
{ ## {{{
  if (!inherits(object, "randomcif")) 
    stop("Must be a random.cif  object")
  cat("Random effect variance for variation due to clusters\n\n")
  cat("Cause", attr(object, "cause1"), "and cause", attr(object, 
                                                         "cause2"), fill = TRUE)
  cat("\n")
  if (sum(abs(object$score)) > 0.001) cat("WARNING: check score for convergence")
  cat("\n")
  coef.randomcif(object, ...)
} ## }}}

##' @export
coef.randomcif<- function (object, digits = 3, ...) 
{ ## {{{
  res <- cbind(object$theta, diag(object$var.theta)^0.5)
  se <- diag(object$var.theta)^0.5
  wald <- object$theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  cor <- object$theta + 1
  res <- as.matrix(cbind(res, wald, waldp, cor, se))
  colnames(res) <- c("Coef.", "SE", "z", "P-val", "Cross odds ratio", "SE")
  if (!is.null(object$thetanames)) rownames(res)<-object$thetanames

  return(res)
} ## }}}

##' @export
print.randomcif<- function (x , digits = 3, ...) 
{ ## {{{
} ## }}}

##' @export
summary.randomcifrv<-function (object, ...) 
{ ## {{{
    if (!inherits(object, "randomcifrv")) 
        stop("Must be a random.cifrv  object")
    cat("Random effect parameters for additive gamma random effects \n\n")
    cat("Cause", attr(object, "cause1"), "and cause", attr(object, 
        "cause2"), fill = TRUE)
    cat("\n")
    if (sum(abs(object$score)) > 1e-06) 
        cat("WARNING: check score for convergence")
    cat("\n")

    res <-     coef.randomcifrv(object, ...)

    var.link <- attr(object,"var.link"); 
    rv1 <- attr(object,"rv1"); 
    theta.des <- attr(object,"pardes"); 
    if (var.link==1) par <- theta.des %*% exp(object$theta) else  par <- theta.des %*% object$theta
    if (var.link==1) {
	      fp <- function(p){  res <- exp(p)/sum(rv1* (theta.des %*% exp(p))); return(res); }
              e <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) fp(p))
              pare <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p))
              res <- list(estimate=res,h=e,exppar=pare)
     } else {
              fp <- function(p) {  p/sum(rv1* (theta.des %*% p)) }
              e <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) fp(p))
              res <- list(estimate=res,h=e)
    }

  res
} ## }}}

##' @export
coef.randomcifrv<- function (object, digits = 3, ...) 
{ ## {{{
    if (attr(object,"inverse")==1) elog <- 1 else elog  <- 0; 
    if (elog==1) theta <- exp(object$theta) else theta <- object$theta 
    se <- diag(object$var.theta)^0.5
    res <- cbind(object$theta, se)
    wald <- object$theta/se
    waldp <- (1 - pnorm(abs(wald))) * 2
    res <- as.matrix(cbind(res, wald, waldp))
    if (elog==0)  colnames(res) <- c("Coef.", "SE", "z", "P-val")
    if (elog==1) res <- cbind(res,exp(object$theta), exp(object$theta)^2*se)  
    if (elog==1) colnames(res) <- c("log-parameter","SE","z","P-val","exp(theta)","SE")

   if (!is.null(object$thetanames)) rownames(res)<-object$thetanames
   prmatrix(signif(res, digits))

} ## }}}

##' @export
print.randomcifrv<- function (x , digits = 3, ...) 
{ ## {{{
 summary(x, ...)
} ## }}}

## summary.cor<-function(object,digits=3,marg.cif=NULL,...)
## { ## {{{
##   if (!inherits(object, "cor")) stop("Must be a cor.cif  object")
##   if (sum(abs(object$score))>0.001) warning("WARNING: check score for convergence\n")
##   coefs <- coef.cor(object,...);

##   outcase <- outconc <- NULL
##   if (is.null(marg.cif)==FALSE) {
##     marg.cif <- max(marg.cif)
##     ## {{{
##     if (attr(object,"Type")=="cor") {
##       concordance <- exp(coefs[,1])*marg.cif^2/((1-marg.cif)+exp(coefs[,1])*marg.cif)
##       conclower  <- exp(coefs[,1]-1.96*coefs[,2])*marg.cif^2/((1-marg.cif)+exp(coefs[,1]-1.96*coefs[,2])*marg.cif)
##       concup  <- exp(coefs[,1]+1.96*coefs[,2])*marg.cif^2/((1-marg.cif)+exp(coefs[,1]+1.96*coefs[,2])*marg.cif)
##       casewise <- concordance/marg.cif
##       caselower  <- conclower/marg.cif
##       caseup     <- concup/marg.cif
##     } else if (attr(object,"Type")=="RR") {
##       casewise<- exp(coefs[,1])*c(marg.cif)
##       concordance <- exp(coefs[,1])*marg.cif^2
##       caselower  <- marg.cif*exp(coefs[,1]-1.96*coefs[,2])
##       caseup     <- marg.cif*exp(coefs[,1]+1.96*coefs[,2])
##       conclower  <- marg.cif^2* exp(coefs[,1]-1.96*coefs[,2])
##       concup     <- marg.cif^2*exp(coefs[,1]+1.96*coefs[,2])
##     } else if (attr(object,"Type")=="OR-cif") {
##       thetal <-  coefs[,1]-1.96*coefs[,2]
##       thetau <-  coefs[,1]+1.96*coefs[,2]
##       casewise<- plack.cif2(marg.cif,marg.cif,c(coefs[,1]))/marg.cif
##       concordance <- plack.cif2(marg.cif,marg.cif,c(coefs[,1]))
##       caselower  <- plack.cif2(marg.cif,marg.cif,thetal)/marg.cif
##       caseup  <- plack.cif2(marg.cif,marg.cif,thetau)/marg.cif
##       conclower  <- plack.cif2(marg.cif,marg.cif,thetal)
##       concup     <- plack.cif2(marg.cif,marg.cif,thetau)
##     }
##     outcase <- cbind(casewise,caselower,caseup)
##     outconc <- cbind(concordance,conclower,concup)
##     rownames(outcase) <- rownames(outconc)  <-  rownames(coefs)
##     colnames(outcase) <- c("casewise concordance","2.5 %","97.5%")
##     colnames(outconc) <- c("concordance","2.5 %","97.5%")
##   }
##   ## }}}
##   res <- list(casewise=outcase,concordance=outconc,estimates=coefs,marg=marg.cif,type=attr(object,"Type"),sym=attr(object,"sym"),cause1=attr(object,"cause1"),cause2=attr(object,"cause2"))
##   class(res) <- "summary.cor"
##   res
## } ## }}}

## coef.cor<-function(object,...)
## { ## {{{
##   res <- cbind(object$theta, diag(object$var.theta)^0.5)
##   se<-diag(object$var.theta)^0.5
##   wald <- object$theta/se
##   waldp <- (1 - pnorm(abs(wald))) * 2
##   cor<-exp(object$theta)
##   res <- as.matrix(cbind(res, wald, waldp,cor,se*cor))
##   if (attr(object,"Type")=="cor") 
##     colnames(res) <- c("log-Coef.", "SE", "z", "P-val","Cross odds ratio","SE")
##   else colnames(res) <- c("log-ratio Coef.", "SE", "z", "P-val","Ratio","SE")
##   if (is.null((rownames(res)))==TRUE) rownames(res)<-rep(" ",nrow(res))

##   return(res)
## } ## }}}

###{{{ twinlm

##' Fits a classical twin model for quantitative traits.
##'
##' @title Classic twin model for quantitative traits
##' @return   Returns an object of class \code{twinlm}.
##' @author Klaus K. Holst
##' @seealso \code{\link{bptwin}}, \code{\link{twinlm.time}}, \code{\link{twinlm.strata}}, \code{\link{twinsim}}
##' @aliases twinlm twinlm.strata
##' @export
##' @examples
##' ## Simulate data
##' set.seed(1)
##' d <- twinsim(1000,b1=c(1,-1),b2=c(),acde=c(1,1,0,1))
##' ## E(y|z1,z2) = z1 - z2. var(A) = var(C) = var(E) = 1
##' 
##' ## E.g to fit the data to an ACE-model without any confounders we simply write
##' ace <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
##' ace
##' ## An AE-model could be fitted as
##' ae <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id", type="ae")
##' ## LRT:
##' lava::compare(ae,ace)
##' ## AIC
##' AIC(ae)-AIC(ace)
##' ## To adjust for the covariates we simply alter the formula statement
##' ace2 <- twinlm(y ~ x1+x2, data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
##' ## Summary/GOF
##' summary(ace2)
##' \donttest{ ## Reduce Ex.Timings
##' ## An interaction could be analyzed as:
##' ace3 <- twinlm(y ~ x1+x2 + x1:I(x2<0), data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
##' ace3
##' ## Categorical variables are also supported 
##' d2 <- transform(d,x2cat=cut(x2,3,labels=c("Low","Med","High")))
##' ace4 <- twinlm(y ~ x1+x2cat, data=d2, DZ="DZ", zyg="zyg", id="id", type="ace")
##' }
##' @keywords models
##' @keywords regression
##' @param formula Formula specifying effects of covariates on the response
##' @param data \code{data.frame} with one observation pr row. In
##'     addition a column with the zygosity (DZ or MZ given as a factor) of
##'     each individual much be
##'     specified as well as a twin id variable giving a unique pair of
##'     numbers/factors to each twin pair
##' @param id The name of the column in the dataset containing the twin-id variable.
##' @param zyg The name of the column in the dataset containing the
##'     zygosity variable
##' @param DZ Character defining the level in the zyg variable
##'     corresponding to the dyzogitic twins. If this argument is missing,
##'     the reference level (i.e. the first level) will be interpreted as
##'     the dyzogitic twins
##' @param group Optional. Variable name defining group for interaction analysis (e.g., gender)
##' @param group.equal If TRUE marginals of groups are asummed to be the same
##' @param strata Strata variable name
##' @param weights Weights matrix if needed by the chosen estimator. For use
##'     with Inverse Probability Weights
##' @param type Character defining the type of analysis to be
##'     performed. Can be a subset of "aced" (additive genetic factors, common
##'     environmental factors, unique environmental factors, dominant
##'     genetic factors). Other choices are:
##'     \itemize{
##'     \item {"0" (or "sat"): Saturated model where twin 1 and twin 2 within each twin
##'            pair may have a different marginal distribution.}
##'     \item {"1" (or "flex","zyg"): Within twin pairs the marginal distribution is
##'            the same, but the marginal distribution may differ between MZ and DZ
##'            twins. A free correlation structure within MZ and DZ twins. }
##'     \item {"2" (or "u", "eqmarg"): All individuals have the same marginals
##'            but a free correlation structure within MZ and DZ twins.}
##'     }
##'     The default value is an additive polygenic model \code{type="ace"}.
##' @param twinnum The name of the column in the dataset numbering the
##'     twins (1,2). If it does not exist in \code{data} it will
##'     automatically be created.
##' @param binary If \code{TRUE} a liability model is fitted. Note that if the right-hand-side of the formula is a factor, character vector, og logical variable, then the liability model is automatically chosen (wrapper of the \code{bptwin} function).
##' @param ordinal If non-zero (number of bins) a liability model is fitted.
##' @param keep Vector of variables from \code{data} that are not
##'     specified in \code{formula}, to be added to data.frame of the SEM
##' @param estimator Choice of estimator/model
##' @param constrain Development argument
##' @param control Control argument parsed on to the optimization routine
##' @param messages Control amount of messages shown 
##' @param ... Additional arguments parsed on to lower-level functions
twinlm <- function(formula, data, id, zyg, DZ, group=NULL,
           group.equal=FALSE, strata=NULL, weights=NULL, type=c("ace"),
           twinnum="twinnum",
           binary=FALSE,ordinal=0,
           keep=weights,estimator=NULL,
           constrain=TRUE,control=list(),messages=1,...)
{
    
  cl <- match.call(expand.dots=TRUE)
  opt <- options(na.action="na.pass")
  mf <- model.frame(formula,data)
  mt <- attr(mf, "terms")
  y <- model.response(mf, "any")
    ## formula <- update(formula, ~ . + 1)
  yvar <- getoutcome(formula)
  if (missing(zyg)) stop("Zygosity variable not specified")
  if (!(zyg%in%colnames(data))) stop("'zyg' not found in data")
  if (!(id%in%colnames(data))) stop("'id' not found in data")
  if (missing(id)) stop("Twin-pair variable not specified")

  if (binary || ordinal || is.factor(mf[,yvar]) || is.character(mf[,yvar]) || is.logical(mf[,yvar])) {
      if (length(unique(mf[,yvar]))>2 || ordinal) {
          if (is.character(mf[,yvar])) {
              y <- as.factor(y)
          }
          if (ordinal<2) ordinal <- length(unique(mf[,yvar]))
          y <- as.numeric(y)-1
          
      } else {
          args <- as.list(cl)
          names(args)[which(names(args)=="formula")] <- "x"
          args[[1]] <- NULL
          return(do.call("bptwin",args,envir=parent.frame()))
      }
  }
  cl$ordinal <- ordinal
  
  formulaId <- unlist(Specials(formula,"cluster"))
  formulaStrata <- unlist(Specials(formula,"strata"))
  formulaSt <- paste("~.-cluster(",formulaId,")-strata(",paste(formulaStrata,collapse="+"),")")
  formula <- update(formula,formulaSt)
  if (!is.null(formulaId)) {
    id <- formulaId
    cl$id <- id
  }
  if (!is.null(formulaStrata)) strata <- formulaStrata
  cl$formula <- formula
 
  if (!is.null(strata)) {
    dd <- split(data,interaction(data[,strata]))
    nn <- unlist(lapply(dd,nrow))
    dd[which(nn==0)] <- NULL
    if (length(dd)>1) {
      fit <- lapply(seq(length(dd)),function(i) {
        if (messages>0) message("Strata '",names(dd)[i],"'")
        cl$data <- dd[[i]]
        eval(cl)
      })
      res <- list(model=fit)
      res$strata <- names(res$model) <- names(dd)
      class(res) <- c("twinlm.strata","twinlm")
      res$coef <- unlist(lapply(res$model,coef))
      res$vcov <- blockdiag(lapply(res$model,vcov))
      res$N <- length(dd)
      res$idx <- seq(length(coef(res$model[[1]])))
      rownames(res$vcov) <- colnames(res$vcov) <- names(res$coef)
      return(res)
    }
  }
  
  type <- tolower(type)
  if (type%in%c("u", "samemarg", "eqmarg", "2"))
    type <- c("u")
  if (type%in%c("flex", "zyg", "1"))
    type <- c("flex")
  if (type%in%c("sat", "0")) {
    type <- c("sat")
  }

  varnames <- all.vars(formula)
  latentnames <- c("a1","a2","c1","c2","d1","d2","e1","e2")
  if (any(latentnames%in%varnames))
    stop(paste(paste(latentnames,collapse=",")," reserved for names of latent variables.",sep=""))
  
  M <- model.matrix(formula,mf)
  options(opt)  
  covars <- colnames(M)
  hasIntercept <- FALSE
  if (attr(terms(formula),"intercept")==1) {
      hasIntercept <- TRUE
      covars <- covars[-1]
  }
  if(length(covars)<1) covars <- NULL
  
  zygstat <- data[,zyg]
  if(!is.factor(zygstat)) {
    zygstat <- as.factor(zygstat)
  }
  zyglev <- levels(zygstat)
  if (length(zyglev)>2) stop("More than two zygosity levels found. For opposite sex (OS) analysis use the 'group' argument (and regroup OS group as DZ, e.g. DZ=c('OS','DZ'))")

  if (tolower(type)=="cor") type <- "u"
  if (!is.null(group) && type%in%c("u","flex","sat")) stop("Only polygenic models are allowed with 'group' ('type' subset of 'acde'). See also the 'strata' argument.")      
  ## To wide format:
  num <- NULL; if (twinnum%in%colnames(data)) num <- twinnum
  if (!is.null(group)) data[,group] <- as.factor(data[,group])
  y <- data.frame(y); names(y) <- yvar
  #y <- cbind(as.vector(y)); colnames(y) <- yvar
  data <- cbind(y,data[,c(keep,num,zyg,id,group)],M)
  ddd <- fast.reshape(data,id=c(id),varying=c(yvar,keep,covars,group),keep=zyg,num=num,sep=".",labelnum=TRUE)
  groups <- paste(group,".",1:2,sep="")
  outcomes <- paste(yvar,".",1:2,sep="")  

  if (missing(DZ)) {
    warning("Using first level, `",zyglev[1],"', in status variable as indicator for 'dizygotic'", sep="")
    DZ <- zyglev[1]    
  }
  MZ <- setdiff(zyglev,DZ)
  wide1 <- ddd[which(ddd[,zyg]==MZ),,drop=FALSE]
  wide2 <- ddd[which(ddd[,zyg]%in%DZ),,drop=FALSE]

  mm <- nn <- c()
  dd <- list()
  levgrp <- NULL
  if (!is.null(group)) {
      levgrp <- levels(data[,group])
      for (i1 in levgrp) {
          for (i2 in levgrp) {
              idxMZ <- which(wide1[,groups[1]]==i1 & wide1[,groups[2]]==i2)
              dMZ <- wide1[idxMZ,,drop=FALSE]
              idxDZ <- which(wide2[,groups[1]]==i1 & wide2[,groups[2]]==i2)
              dDZ <- wide2[idxDZ,,drop=FALSE]
              m0 <- twinsem1(outcomes,c(i1,i2),
                            levels=levgrp,covars=covars,type=type,
                            data=list(dMZ,dDZ),constrain=constrain,
                            equal.marg=group.equal,intercept=hasIntercept,
                            ordinal=ordinal)$model
              if (length(idxMZ)>0) {
                  nn <- c(nn,paste("MZ:",i1,sep=""))
                  dd <- c(dd,list(dMZ))
                  mm <- c(mm,list(m0[[1]]))
              }
              if (length(idxDZ)>0) {
                  nn <- c(nn,paste("DZ:",i1," ",i2,sep=""))
                  dd <- c(dd,list(dDZ))
                  mm <- c(mm,list(m0[[2]]))
              }
          }
      }; names(mm) <- nn; names(dd) <- nn      
  } else {
      mm <- twinsem1(outcomes,NULL,
                     levels=NULL,covars=covars,type=type,
                     data=list(wide1,wide2),constrain=constrain,
                     intercept=hasIntercept,ordinal=ordinal)$model
      dd <- list(MZ=wide1,DZ=wide2)
  }
  
  newkeep <- unlist(sapply(keep, function(x) paste(x, 1:2, sep = ".")))
    if (!is.null(estimator) && (inherits(estimator,c("numeric","logical")) && !estimator)) return(multigroup(mm, dd, missing=TRUE,fix=FALSE,keep=newkeep,type=2))
  optim <- list()
  if (is.null(control$start)) {
    optim <- list(start=rep(0.1,length(coef(mm[[1]]))*length(mm)))
##    optim <- list(method="nlminb2",refit=FALSE,gamma=1,start=rep(0.1,length(coef(mm[[1]]))*length(mm)))

    optim$start <- twinlmStart(formula,na.omit(mf),type,hasIntercept,
                              surv=inherits(data[,yvar],"Surv"),ordinal=ordinal,
                              model=mm, group=levgrp, group.equal=group.equal)
  }
  if (length(control)>0) {
    optim[names(control)] <- control
  }
    
  if (inherits(data[,yvar],"Surv")) {
      ##if (!requireNamespace("lava.tobit",quietly=TRUE)) stop("lava.tobit required")
      if (!is.null(estimator) && estimator%in%c("gaussian","tobit")) optim$method <- "nlminb1"
      suppressWarnings(e <- estimate(mm,dd,control=optim,estimator=estimator,...))
  } else {
      suppressWarnings(e <- estimate(mm,dd,weights=weights,estimator=estimator,fix=FALSE,control=optim,...))
  }

  if (!is.null(optim$refit) && optim$refit) {
    optim$method <- "NR"
    optim$start <- pars(e)
    if (inherits(mf[,yvar],"Surv")) {
        suppressWarnings(e <- estimate(mm,dd,estimator=estimator,fix=FALSE,control=optim,...))
    } else {
        suppressWarnings(e <- estimate(mm,dd,weights=weights,estimator=estimator,fix=FALSE,control=optim,...))
    }
  }
  e$vcov <- Inverse(information(e,type="hessian"))
  
  counts <- function(dd) {
    dd0 <- apply(dd,2,function(x) !is.na(x))
    pairs <- sum(dd0[,1]*dd0[,2])
    singletons <- sum((!dd0[,1])*dd0[,2] + (!dd0[,2])*dd0[,1])
    return(c(pairs,singletons))
  }
  counts <- lapply(dd, function(x) counts(x[,outcomes]))
  ## mz  <- counts(object$data.mz[,object$outcomes])
  ## dz  <- counts(object$data.dz[,object$outcomes])
  if (!e$model$missing) {
      zygtab <- c("MZ-pairs"=counts[[1]][1],"DZ-pairs"=counts[[2]][1])
  } else {
      zygtab <- c(paste(counts[[1]],collapse="/"),paste(counts[[2]],collapse="/"))
      names(zygtab) <- c("MZ-pairs/singletons","DZ-pairs/singletons")
  }

  res <- list(coefficients=e$opt$estimate, vcov=e$vcov,
              estimate=e, model=mm, call=cl, data=data, zyg=zyg,
              id=id, twinnum=twinnum, type=type,  group=group,
              constrain=constrain, outcomes=outcomes, zygtab=zygtab,
              nam=nn, groups=levgrp, group.equal=group.equal,
              counts=counts,ordinal=ordinal)
  class(res) <- "twinlm"
  return(res)
}

###}}} twinlm

###{{{ twinsem1 (create lava model)

##outcomes <- c("y1","y2"); groups <- c("M","F"); covars <- NULL; type <- "ace"
##twinsem1(c("y1","y2"),c("M","F"))
twinsem1 <- function(outcomes,groups=NULL,levels=NULL,covars=NULL,type="ace",data,constrain=TRUE,equal.marg=FALSE,intercept=TRUE,ordinal=0,...) {
    isA <- length(grep("a",type))>0 & type!="sat"
    isC <- length(grep("c",type))>0
    isD <- length(grep("d",type))>0
    isE <- length(grep("e",type))>0 | type=="sat" | type=="u" 
    lambdas <- c("lambda[a]","lambda[c]","lambda[d]","lambda[e]")
    varidx <- which(c(isA,isC,isD,isE))
    vMZ1 <- c("a1","c1","d1","e1")
    vMZ2 <- c("a1","c1","d1","e2")
    vDZ2 <- c("a2","c1","d2","e2")
    rhoA <- rhoD <- zA <- zD <- NULL

    ##if (ordinal>0) {
    ##     mm <- lapply(mm, function(x) {
    ##         x
    ##     })
    ## }

    if (is.list(outcomes)) {
        if (!is.null(groups) & is.null(levels)) stop("missing levels")
        if (is.null(groups)) groups <- c("","")
        grp <- paste(sort(groups),collapse=" ")
        sameGroup <- groups[1]==groups[2]
        model1 <- outcomes[[1]]
        model2 <- outcomes[[2]]

        if (!is.null(levels)) {
            pars <- c()
            for (i in seq(length(levels)-1)) for (j in seq(i+1,length(levels)))
                pars <- c(pars,paste(sort(levels)[c(i,j)],collapse=" "))
            if (isA) {
                parameter(model1) <- paste("z(A):",pars,sep="")
                parameter(model2) <- paste("z(A):",pars,sep="")
            }
            if (isD) {
                parameter(model1) <- paste("z(D):",pars,sep="")
                parameter(model2) <- paste("z(D):",pars,sep="")
            }
        }
        outcomes <- endogenous(model1)
        if (!(type%in%c("u","flex","sat"))) {
            if (equal.marg) {
                regression(model1,to=outcomes[1],from=vMZ1[varidx]) <-
                    lambdas[varidx]
                regression(model1,to=outcomes[2],from=vMZ2[varidx]) <-
                    lambdas[varidx]            
                regression(model2,to=outcomes[1],from=vMZ1[varidx]) <-
                    lambdas[varidx]
                regression(model2,to=outcomes[2],from=vDZ2[varidx]) <-
                    lambdas[varidx]
            } else {
                regression(model1,to=outcomes[1],from=vMZ1[varidx]) <-
                    paste(lambdas,groups[1],sep="")[varidx]
                regression(model1,to=outcomes[2],from=vMZ2[varidx]) <-
                    paste(lambdas,groups[2],sep="")[varidx]            
                regression(model2,to=outcomes[1],from=vMZ1[varidx]) <-
                    paste(lambdas,groups[1],sep="")[varidx]
                regression(model2,to=outcomes[2],from=vDZ2[varidx]) <-
                    paste(lambdas,groups[2],sep="")[varidx]
            }
        }
        if (sameGroup) {
            if (isA) covariance(model2,a1~a2)  <- 0.5
            if (isD) covariance(model2,d1~d2)  <- 0.25
        } else {            
            if (isA) {
                rhoA <- paste("Kinship[A]:",grp,sep="")
                zA <- paste("z(A):",grp,sep="")
                covariance(model2, a1~a2) <- rhoA
                constrain(model2, rhoA,zA) <- function(x) tanh(x)
            }
            if (isD) {                
                rhoD <- paste("Kinship[D]:",grp,sep="")
                zD <- paste("z(D):",grp,sep="")
                covariance(model2, d1~d2) <- rhoD
                constrain(model2, rhoD,zD) <- function(x) tanh(x)
            }
        }
        if (ordinal>0) {            
            for (i in c("model1","model2")) {
                model <- get(i)
                yy <- lava::endogenous(model)
                if (type=="sat") {
                    lava::ordinal(model,K=ordinal) <- yy
                } else {
                    pname <- paste0("t",seq_len(ordinal-1))
                    if (type=="flex") pname <- paste0(pname,gsub("model","",i))
                    lava::ordinal(model,K=ordinal,pname) <- yy
                }
                if (!(type%in%c("u","flex","sat"))) {
                    lava::covariance(model,yy) <- 0
                    lava::regression(model,from="e1",to=yy[1]) <- 1
                    lava::regression(model,from="e2",to=yy[2]) <- 1
                }
                assign(i,model)
            }
        }
        return(list(model=list(MZ=model1,DZ=model2),
                    zA=zA, rhoA=rhoA, zD=zD, rhoD=rhoD))
    }
    
    ### Build model from scratch....
    model1<- lvm()    
    if (!(type%in%c("u","flex","sat"))) {
        model1 <- regression(model1,to=outcomes[1],from=vMZ1[varidx])
        model1 <- regression(model1,to=outcomes[2],from=vMZ2[varidx])
    } else {
        addvar(model1) <- outcomes
        covariance(model1) <- outcomes        
    }    
    latent(model1) <- union(vMZ1[varidx],vMZ2[varidx])
    intercept(model1,latent(model1)) <- 0

        if (!is.null(covars))
            for (i in 1:length(covars)) {
            regression(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta[",i,"]",sep="")
            regression(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta[",i,"]",sep="")
        }
        covariance(model1,outcomes) <- 0
        covariance(model1, latent(model1)) <- 1

        if (!type%in%c("sat","flex")) {    
            intercept(model1,outcomes) <- "mu"
        }
        if (type%in%c("u","flex","sat")) {
            kill(model1) <- ~e1+e2
            covariance(model1,outcomes) <- "v1"
        }
        model2 <- model1
        if (!(type%in%c("u","flex","sat"))) {
            model2 <- cancel(model2,c(outcomes[2],vMZ2[varidx]))
            model2 <- regression(model2,to=outcomes[2],from=vDZ2[varidx])
            latent(model2) <- vDZ2[varidx]
            intercept(model2, latent(model2)) <- 0
            covariance(model2, latent(model2)) <- 1
        }
    if (type=="flex") {
        varMZ <- paste("var(MZ)",groups,sep="")
        varDZ <- paste("var(DZ)",groups,sep="")
        intercept(model1,outcomes) <- "mu1"
        intercept(model2,outcomes) <- "mu2"
        covariance(model1,outcomes) <- varMZ
        covariance(model2,outcomes) <- varDZ
    }
    if (type=="sat") {
        varMZ <- c(paste("var(MZ)1",groups[1],""),
                   paste("var(MZ)2",groups[2],""))
        varDZ <- c(paste("var(DZ)1",groups[1],""),
                   paste("var(DZ)2",groups[2],""))        
        covariance(model1,outcomes) <- varMZ
        covariance(model2,outcomes) <- varDZ
    }
    if (type%in%c("u","flex","sat")) {
        if (constrain) {
            if (ordinal) {
                covariance(model1,outcomes,pairwise=TRUE) <- "covMZ"
                covariance(model2,outcomes,pairwise=TRUE) <- "covDZ"
                constrain(model1,"atanh(rhoMZ)","covMZ") <- tanh
                constrain(model2,"atanh(rhoDZ)","covDZ") <- tanh
            }            
            if (type=="sat") {
                if (!ordinal) {
                    model1 <- covariance(model1,outcomes,constrain=TRUE,rname="atanh(rhoMZ)",cname="covMZ",lname="log(var(MZ)).1",l2name="log(var(MZ)).2")
                    model2 <- covariance(model2,outcomes,constrain=TRUE,rname="atanh(rhoDZ)",cname="covDZ",lname="log(var(DZ)).1",l2name="log(var(DZ)).2")
                }
            } else {
                if (type=="flex") {
                    if (!ordinal) {
                        model1 <- covariance(model1,outcomes,constrain=TRUE,rname="atanh(rhoMZ)",cname="covMZ",lname="log(var(MZ))")
                        model2 <- covariance(model2,outcomes,constrain=TRUE,rname="atanh(rhoDZ)",cname="covDZ",lname="log(var(DZ))")
                    }
                } else {
                    if (!ordinal) {
                        model1 <- covariance(model1,outcomes,constrain=TRUE,rname="atanh(rhoMZ)",cname="covMZ",lname="log(var)")
                        model2 <- covariance(model2,outcomes,constrain=TRUE,rname="atanh(rhoDZ)",cname="covDZ",lname="log(var)")
                    }
                }        
            }     
        } else {
            covariance(model1,outcomes[1],outcomes[2]) <- "covMZ"
            covariance(model2,outcomes[1],outcomes[2]) <- "covDZ"
        }
    }

    if (!is.null(covars) & type%in%c("flex","sat")) {
        sta <- ""
        if (type=="sat") sta <- "b"
        for (i in 1:length(covars)) {
            regression(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta1[",i,"]",sep="")         
            regression(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta1",sta,"[",i,"]",sep="")
            regression(model2, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta2[",i,"]",sep="")
            regression(model2, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta2",sta,"[",i,"]",sep="")
        }
    }
    if (!intercept) {
        intercept(model1,outcomes) <- 0
        intercept(model2,outcomes) <- 0
    }
   
    ## Full rank covariate/design matrix?
    if (!missing(data))
    for (i in covars) {
        myvars <- paste(i,c(1,2),sep=".")
        dif <- data[[1]][,myvars[1]]-data[[1]][,myvars[2]]   
        mykeep <- myvars
        if (all(na.omit(dif)==00)) {
            mykeep <- mykeep[-2]
        }   
        trash <- setdiff(myvars,mykeep)
        if (length(mykeep)==1) {
            regression(model1, to=outcomes[2], from=mykeep) <- lava::regfix(model1)$label[trash,outcomes[2]]
            kill(model1) <- trash
        }
        dif <- data[[2]][,myvars[1]]-data[[2]][,myvars[2]]   
        mykeep <- myvars
        if (all(na.omit(dif)==00)) {
            mykeep <- mykeep[-2]
        }  
        trash <- setdiff(myvars,mykeep)
        if (length(mykeep)==1) {
            regression(model2, to=outcomes[2], from=mykeep) <- lava::regfix(model2)$label[trash,outcomes[2]]
            kill(model2) <- trash
        }
    }
    twinsem1(list(MZ=model1,DZ=model2),groups=groups,type=type,levels=levels,constrain=constrain,equal.marg=equal.marg,ordinal=ordinal)
}

###}}} twinsem1

###{{{ twinlmStart (starting values)

twinlmStart <- function(formula,mf,type,hasIntercept,surv=FALSE,ordinal=0,model,group=NULL,group.equal,...)  {
    if (surv) {
        l <- survival::survreg(formula,data=mf,dist="gaussian")
        beta <- coef(l)
        sigma <- l$scale
    } else {
        if (ordinal) {
            l <- lava::ordreg(formula,mf)
            beta <- coef(l)
            sigma <- 1
        } else {
            l <- lm.fit(model.matrix(formula,mf),mf[,1])
            beta <- l$coefficients
            sigma <- sd(l$residuals)
        }
    }
    intidx <- 1
    if (ordinal) intidx <- seq_len(ordinal-1)
    start <- rep(sigma/sqrt(nchar(type)),nchar(type))
    if (ordinal) start <- start[-length(start)]
    if (hasIntercept) {
        start <- c(beta[intidx],start)
        start <- c(start,beta[-intidx])
    } else start <- c(start,beta)
    varp <- 0.5
    if (type=="sat") {
        if (!ordinal) varp <- c(rep(log(sigma^2),2),varp)
        start <- c()
        if (hasIntercept) {
            start <- rep(beta[intidx],4)
            beta <- beta[-intidx]
        }
        start <- c(start,rep(c(rep(beta,2),varp),2))
    }
    if (type=="flex") {
        if (!ordinal) varp <- c(log(sigma^2),varp)
        start <- c()
        if (hasIntercept) {
            start <- c(beta[intidx],beta[intidx])
            beta <- beta[-intidx]
        }
        start <- c(start,rep(c(beta,varp),2))
    }
    if (type=="u") {
        start <- c(varp,varp)
        if (!ordinal) start <- c(log(sigma^2),start)
        start <- c(beta,start)
    }
    names(start) <- NULL
    if (!is.null(group)) {
        cc <- coef(model[[1]])        
        iA <- grep(c("z(A):"),cc,fixed=TRUE)
        iD <- grep(c("z(D):"),cc,fixed=TRUE)
        nplus <- length(iA) + length(iD) + max(length(iA),length(iD))
        start <- c(start,rep(.2,nplus))
        ii <- sort(c(iA,iD))
        start <- c(start[seq(ii[1]-1)],rep(0.3,length(ii)),start[seq(ii[1]+1,length(start))])
    }
    
    return(start)
}

###}}} twinlmStart

#' Simulation of Piecewise constant hazard model (Cox).
#' 
#' Simulates data from piecwise constant baseline hazard that can also be of
#' Cox type. Censor data at highest value of the break points.
#' 
#' For a piecewise linear cumulative hazard the inverse is easy to compute with 
#' and delayed entry x we compute 
#' \deqn{\Lambda^{-1}(\Lambda(x) + E/RR)}, 
#' where RR are the relative risks and E is exponential with mean 1.
#' This quantity has survival function 
#' \deqn{P(T > t | T>x) = exp(-RR (\Lambda(t) - \Lambda(x)))}. 
#' 
#' @param cumhazard cumulative hazard, or piece-constant rates for periods
#'   defined by first column of input.
#' @param rr relative risk for simulations, alternatively when rr=1 specify n
#' @param n number of simulation if rr not given
#' @param entry delayed entry time for simuations.
#' @param cum.hazard specifies wheter input is cumulative hazard or rates.
#' @param cause name of cause
#' @param extend to extend piecewise constant with constant rate. Default is
#'   average rate over time from cumulative (when TRUE), if numeric then uses
#'   given rate.
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' library(mets)
#' chaz <-  c(0,1,1.5,2,2.1)
#' breaks <- c(0,10,   20,  30,   40)
#' cumhaz <- cbind(breaks,chaz)
#' n <- 10
#' X <- rbinom(n,1,0.5)
#' beta <- 0.2
#' rrcox <- exp(X * beta)
#' 
#' pctime <- rchaz(cumhaz,n=10)
#' pctimecox <- rchaz(cumhaz,rrcox,entry=runif(n))
#' 
#' @export 
#' @aliases simrchaz addCums lin.approx simCens extendCums 
rchaz <- function(cumhazard,rr,n=NULL,entry=NULL,cum.hazard=TRUE,cause=1,extend=FALSE)
{# {{{
  if (!is.null(n)) rr <- rep(1,n)
  n <- length(rr)

  breaks <- cumhazard[,1]
  rates <- cumhazard[,2][-1]
  mm <- tail(breaks,1)
  if (cum.hazard==FALSE) {
        cumh <- cumsum(c(0,diff(breaks)*rates))
        cumhazard <- cbind(breaks,cumh)
  } else cumh <- cumhazard[,2] 
   ttt <- rexp(n)/rr
   if (cumhazard[1,2]>0)  { ## start cumulative hazard with a 0
###   warning("Safest to start with cumulative hazard 0 to avoid problems\n"); 
      cumhazard <- rbind(c(0,0),cumhazard)
      cumh <- c(0,cumh)
   }
   ###
   if (!is.null(entry)) {
	   if (length(entry)==1) entry <- rep(entry,n) else entry <- entry
	   cumentry <- lin.approx(entry,cumhazard,x=1)
   } else { entry <- cumentry <- rep(0,n) }
   ###
   ttte <- ttt+cumentry
   rrx <- lin.approx(ttte,cumhazard,x=-1)
   rrx <- ifelse(rrx>mm,mm,rrx)
   status <- rep(0,n)
   status <- ifelse(rrx<mm,cause,status)
   tcum <- tail(cumhazard,1)
   extend.rate <- NULL
   if (is.logical(extend))
	   if (extend) extend.rate <- tcum[2]/tcum[1]
   if (is.numeric(extend)) { extend.rate <- extend; extend <- TRUE;}
   if (extend) {
	   whoe <- which(status==0)
	   nn <- length(whoe)
	   if (nn>0) {
	     textend <- rexp(nn)/(rr[whoe]*extend.rate)
	     rrx[whoe] <- mm+textend
	     status[whoe] <- 1
	   }
   }
   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr)
   colnames(dt) <- c("entry","time","status","rr"); 
   attr(dt,"cumhaz") <- cumhazard
   attr(dt,"extend.rate") <- extend.rate
   return(dt)
}# }}}

#' @export 
rchazl <- function (cumhaz, rr, causes=NULL,...)
{# {{{
    l <- length(cumhaz)
    tall <- rchaz(cumhaz[[1]], rr[, 1], ...)
    if (l >= 2)
        for (i in 2:l) {
            tall2 <- rchaz(cumhaz[[i]], rr[, i], ...)
            tall$status <- ifelse(tall$time < tall2$time, tall$status, i * tall2$status)
            tall$time <- pmin(tall$time, tall2$time)
        }
 if (!is.null(causes)) {
      where <- which(tall$status!=0) 
      tall$status[where] <- causes[tall$status[where]]
 }
    return(tall)
}# }}}

#' @export
lin.approx <- function (x2, xfx, x = 1)
{ ## {{{ 
    breaks <- xfx[, x]
    fx <- xfx[, -x]
    ri <- fast.approx(breaks, x2, type = "left")
    maxindex <- which(ri == length(breaks))
    rip1 <- ri + 1
    rip1[maxindex] <- length(breaks)
    rrr <- (x2 - breaks[ri])/(breaks[rip1] - breaks[ri])
    rrr[maxindex] <- 0
    res <- rrr * (fx[rip1] - fx[ri]) + fx[ri]
    res[is.na(res)] <- tail(fx, 1)
    res[is.na(rrr)] <- fx[ri][is.na(rrr)]
    return(res)
} ## }}}

#' @export
addCums <- function(cumB,cumA,max=NULL)
{# {{{
 ## take times
 times <- sort(unique(c(cumB[,1],cumA[,1])))
 if (!is.null(max)) times <- times[times<max]
 cumBjx <- lin.approx(times,cumB,x=1)
 cumAjx <- lin.approx(times,cumA,x=1)
 cumBA <- cumBjx+cumAjx
 return(cbind(times,cumBA))
}# }}}

##' @export
extendCums <- function(cumA,cumB,extend=NULL)
{# {{{
## setup as list to run within loop
if (!is.null(cumB)) {cumA <- list(cumA,cumB); } 

## to also work with strata version where each list contains a list of cumHaz for strata
matrixlist <- any(unlist(lapply(cumA, function(x) is.data.frame(x) | is.matrix(x))))
## any stratified components 
basecumhaz <- any(unlist(lapply(cumA, function(x) inherits(x,"basecumhaz"))))
nn <- length(cumA)
nl <- lengths(cumA)
if (basecumhaz) cumA <- unlist(cumA, recursive = FALSE)

restore <- function(flat, lengths) {
  ends <- cumsum(lengths)
  starts <- c(1, head(ends + 1, -1))
  mapply(function(s, e) flat[s:e], starts, ends, SIMPLIFY = FALSE)
}

 maxx <- unlist(lapply(cumA,function(x) tail(x,1)[1]))
 mm <- which.max(maxx)
 haza <- NULL
 if (is.numeric(extend)) 
 if (length(extend)!=length(cumA)) haza <- rep(extend,length(cumA)) 

 ## extend all that are not at maxtime
for (i in seq(nn)[-mm]) {
  cumB <- as.matrix(cumA[[i]]); 
  cumB <- rbind(c(0,0),cumB); 

  ### linear extrapolation of mortality using given dHaz/dt or haza given rate
  if (tail(cumB[,1],1)<maxx[mm]) {
      tailB <- tail(cumB,1)
      cumlast <- tailB[2]
      timelast <- tailB[1]
      if (is.null(haza)) hazb <- cumlast/timelast else hazb <- haza[i]
      cumB <- rbind(cumB,c(maxx[mm],cumlast+hazb*(maxx[mm]-timelast))) 
  }
  cumA[[i]] <- cumB
}
 cumA[[mm]] <- rbind(c(0,0),cumA[[mm]])

 if (basecumhaz) cumA <- restore(cumA,nl)
 return( setNames(cumA,paste("cum",seq(nn),sep="")))
}# }}}

#' @export
simrchaz <- function(cumhazard,rr,n=NULL,cens=NULL,rrc=NULL,entry=NULL,...)
{# {{{
###   adds censoring to to rchaz call
   if (!is.null(n)) rr <- rep(1,n)
   n <- length(rr)  

   ptt <- rchaz(cumhazard,rr,entry=entry,...)

   if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
      dt <- data.frame(time=pmin(ptt$time,pct),status=ifelse(ptt$time<pct,ptt$status,0))
   } else dt <- ptt[,c("time","status")]

   return(dt)
}# }}}

#' @export
simCens <- function(cens,rrc=NULL,n=NULL,entry=NULL,...)
{# {{{
   if (is.null(rrc) & is.null(n)) stop("must give either rr or n\n"); 
   if (is.null(rrc)) rrc <- rep(1,n)
   n <- length(rrc)  
   if (is.matrix(cens)) {
	   pct <- rchaz(cens,rrc,entry=entry,...)
	   pct <- pct$time
   } else if (is.numeric(cens)) {
	   pct<- rexp(n)/(rrc*cens)  
           if (!is.null(entry)) pct <- entry+pct
   } else stop("cens must be cumulative hazard or constant rate\n"); 

   return(pct)
} ## }}}

#' Simulation of Piecewise constant hazard models with two causes (Cox).
#' 
#' Simulates data from piecwise constant baseline hazard that can also be of
#' Cox type. Censor data at highest value of the break points for either of the
#' cumulatives, see also sim.phregs  
#' 
#' @param cumA cumulative hazard of cause 1, or list of multiple cumulative hazards
#' @param cumB cumulative hazard of cause 2 or NULL when cumA is a list 
#' @param rr1 number of simulations or vector of relative risk for simuations, or matrix with columns equal to number of hazards in list
#' @param rr2 number of simulations or vector of relative risk for simuations.
#' @param n number of simulation if rr not given, must be given when rr is not given  
#' @param cens to censor further , rate or cumumlative hazard
#' @param rrc retlativ risk for censoring.
#' @param extend to extend the cumulative hazards to largest end-point 
#' @param causes to assign status values for each of the causes, vector of integers 
#' @param ... arguments for rchaz 
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' library(mets)
#' data(bmt); 
#' n <- 100
#' cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
#' cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)
#'
#' X1 <- bmt[,c("tcell","platelet")]
#' xid <- sample(1:nrow(X1),n,replace=TRUE)
#' Z1 <- X1[xid,]
#' Z2 <- X1[xid,]
#' rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
#' rr2 <- exp(as.matrix(Z2) %*% cox2$coef)
#'
#' d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=2/70)
#' dd <- cbind(d,Z1)
#'
#' d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=cbind(c(1,30,68),c(.01,1,3)))
#' dd <- cbind(d,Z1)
#'
#' par(mfrow=c(1,3))
#' scox0 <- phreg(Surv(time,status==0)~tcell+platelet,data=dd)
#' plot(scox0); lines(cbind(c(1,30,68),c(.01,1,3)),col=2)
#' ##
#' scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
#' scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
#' plot(cox1); plot(scox1,add=TRUE,col=2)
#' plot(cox2); plot(scox2,add=TRUE,col=2)
#' cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
#' 
#' # 3 causes and censoring 
#' d3 <-  rcrisk(list(cox1$cum,cox2$cum,cox1$cum),NULL,n=100,cens=cbind(c(1,30,68),c(.01,1,3)))
#' dtable(d3,~status)
#' 
#' @export 
#' @aliases  rchazl 
rcrisk <-function(cumA,cumB,rr1=NULL,rr2=NULL,n=NULL,
                  cens=NULL,rrc=NULL,extend=TRUE,causes=NULL,...)
{#'# {{{
  if (!is.null(cumB)) {
         cumA <- list(cumA,cumB);
        rr <- cbind(rr1,rr2);
 } else cumA <- c(cumA,cumB)

 if (!is.null(n)) {
   rr <- matrix(1,n,length(cumA));
  } else {
     n <- length(rr1);
     rr <- cbind(rr1,rr2)
  }

  if (!is.null(extend))  cumA <- extendCums(cumA,NULL,extend=extend)

 l <- length(cumA)
 ## simulate first 
 ptt <- rchaz(cumA[[1]],rr[,1],...)
 if (l>=2) for (i in 2:l) {
         ptt2 <- rchaz(cumA[[i]],rr[,i],...)
 ptt$status <- ifelse(ptt$time<ptt2$time,ptt$status,i*ptt2$status)
 ptt$time <- pmin(ptt$time,ptt2$time)
 }
 if (!is.null(causes)) {
         where <- which(ptt$status!=0)
      ptt$status[where] <- causes[ptt$status[where]]
 }

 ## add censoring 
 if (!is.null(cens)) {
   pct <- simCens(cens,rrc=rrc,n=n,...)
   ptt$time <- pmin(ptt$time,pct)
   ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

return(ptt)
}# }}}

#' Simulation of output from Cox model.
#' 
#' Simulates data that looks like fit from Cox model. Censor data automatically
#' for highest value of the event times by using cumulative hazard. 
#' 
#' @param cox output form coxph or cox.aalen model fitting cox model.
#' @param n number of simulations.
#' @param data to extract covariates for simulations (draws from observed covariates).
#' @param Z give design matrix instead of data
#' @param rr possible vector of relative risk for cox model.
#' @param strata possible vector of strata 
#' @param entry delayed entry variable for simulation.
#' @param extend to extend possible stratified baselines to largest end-point 
#' given then takes average rate of in simulated data from cox model.
#' @param cens specifies censoring model, if "is.matrix" then uses cumulative
#' hazard given, if "is.scalar" then uses rate for exponential, and if not
#' @param rrc possible vector of relative risk for cox-type censoring.
#' @param ... arguments for rchaz, for example entry-time.
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' library(mets)
#' data(sTRACE)
#' nsim <- 100
#' coxs <-  phreg(Surv(time,status==9)~strata(chf)+vf+wmi,data=sTRACE)
#' set.seed(100)
#' sim3 <- sim.phreg(coxs,nsim,data=sTRACE)
#' head(sim3)
#' cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim3)
#' cbind(coxs$coef,cc$coef)
#' plot(coxs,col=1); plot(cc,add=TRUE,col=2)
#' 
#' Z <- sim3[,c("vf","chf","wmi")]
#' strata <- sim3[,c("chf")]
#' rr <- exp(as.matrix(Z[,-2]) %*% coef(coxs))
#' sim4 <- sim.phreg(coxs,nsim,data=NULL,rr=rr,strata=strata)
#' sim4 <- cbind(sim4,Z)
#' cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim4)
#' cbind(coxs$coef,cc$coef)
#' plot(coxs,col=1); plot(cc,add=TRUE,col=2)
#' 
#' @aliases draw.phreg setup.phreg
#' @export sim.phreg 
sim.phreg <- function(cox,n,data=NULL,Z=NULL,rr=NULL,strata=NULL,
                      entry=NULL,extend=NULL,cens=NULL,rrc=NULL,...)
{# {{{

if  (!is.null(data)) {
   scox1 <- draw.phreg(cox,n,data=data,...)
   dat <- scox1$data
   dat$orig.id <- scox1$id

   if (is.null(strata))  strata <- scox1$strata 
   if (is.null(rr)) rr <- scox1$rr  
   n <- length(rr)
   id <- 1:length(rr)
} else {
	if (!is.null(Z)) {
	  if (is.data.frame(Z)) znames <- names(Z) else znames <- colnames(Z)
	} else znames <- NULL
	if (is.null(rr) & (!is.null(Z))) rr <- exp(as.matrix(Z) %*% cif$coef)
	if (is.null(rr) & is.null(Z)) rr <- rep(1,n)
	id <- 1:length(rr)
	n <- length(rr)
	dat <- NULL
}
if (is.null(strata)) strata <- rep(0,n)
   
   if (inherits(cox,c("phreg","cifreg"))) cumhaz <- basecumhaz(cox,only=1)
   else {
        if (!is.list(cox)) stop("must be phreg or list of hazards\n") else cumhaz <- cox
   }
   if (!is.null(extend))  cumhaz <- extendCums(cumhaz,NULL,extend=extend)
   ids <- 1:n
   lentry <- NULL

   ptt <- c()
   for (i in unique(strata)) {
      whichi <- which(strata==i)
      cumhazj <- rbind(0,cumhaz[[i+1]])
      if (!is.null(entry)) lentry <- entry[whichi]
      simj <- rchaz(cumhazj,rr[whichi],entry=lentry) 
      simj$id <- ids[whichi]
      ptt  <-  rbind(ptt,simj)
    }
    dsort(ptt) <- ~id
    if (!is.null(dat)) ptt <- cbind(ptt,dat)

 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

return(ptt)
}# }}}

#' @export draw.phreg
#' @usage draw.phreg(cox,n,data=NULL,Z=NULL,drawZ=TRUE,fixZ=FALSE,id=NULL)
draw.phreg <- function(cox,n,data=NULL,Z=NULL,strata=NULL,
                       drawZ=TRUE,fixZ=FALSE,id=NULL)
{# {{{
###if (!inherits(cox,"phreg")) stop("must be phreg object\n"); 

## give data so that design can be constructed based on model-formula
if (!is.null(data)) {
cid <- countID(data.frame(id=cox$id))
whereid <- which(cid$Countid==1)
if (drawZ==TRUE) xid <- sample(whereid,n,replace=TRUE) else xid <- id
## TMP vars <- all.vars(update(cox$formula,-1~.))
vars <- all.vars(cox$formula)
dataid <- data[xid,vars,drop=FALSE] 

desX <- readPhreg(cox,dataid)
Z <- desX$X
strata <- desX$strata
} else {  ## Z and strata
   xid <- 1:nrow(Z); 
   n <- nrow(Z); 
   dataid <- Z; 
   if (is.null(strata)) strata <- rep(0,n)
}

nz <- ncol(Z)
if (nz>0) rr <- exp(as.matrix(Z) %*% cox$coef) else rr <- rep(1,nrow(Z))
cumhaz <- rbind(c(0,0),cox$cumhaz)
   if (cox$nstrata>1) {
      stratname <-  substring(cox$strata.name,8,nchar(cox$strata.name)-1)
   } else stratname <- NULL
   model <-c(class(cox),is.null(cox$propodds))

out <- list(Z=Z,cumhaz=cumhaz,rr=rr,id=xid,model=model,
	    strata=strata,data=dataid,stratname=stratname)

return(out)
} ## }}}

#' @export
setup.phreg  <- function(cumhazard,coef,Znames=NULL,strata=NULL)
{# {{{
    cox <- list()
    cox$cumhaz <- cumhazard
    cox$coef <- coef
    if (is.null(strata)) { strata <- rep(0,nrow(cumhazard)); nstrata <- 1} else nstrata <- max(strata)+1
    cox$strata <- strata
    cox$nstrata <- nstrata
    cox$strata.name <- ""
    cox$jumps <- 1:nrow(cumhazard)
    class(cox) <- c("setup","phreg")
    attr(cox,"znames") <- Znames
    return(cox)
}# }}}

#' Simulation of cause specific from Cox models.
#' 
#' Simulates data that looks like fit from cause specific Cox models. 
#' Censor data automatically. When censoring is given in the  list of causes this
#' will give censoring that looks like the data.  Covariates are drawn from data-set
#' with replacement. This gives covariates like the data.  Calls sim.phregs
#' 
#' 
#' @param coxs list of cox models.
#' @param n number of simulations.
#' @param data to extract covariates for simulations (draws from observed covariates).
#' @param rr possible vector of relative risk for cox model.
#' @param strata possible vector of strata 
#' @param entry delayed entry variable for simulation.
#' @param extend to extend possible stratified baselines to largest end-point 
#' @param cens specifies censoring model, if NULL then only censoring for 
#'   	       each cause at end of last event of this type. 
#' 	       if "is.matrix" then uses cumulative. 
#'             hazard given, if "is.scalar" then uses rate for exponential, and if not
#'             given then takes average rate of in simulated data from cox model.
#'             But censoring can also be given as a cause.
#' @param rrc possible vector of relative risk for cox-type censoring.
#' @param ... arguments for rchaz, for example entry-time
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' library(mets)
#' data(bmt)
#' nsim <- 100; 
#' 
#' cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet+age,data=bmt)
#' cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
#' coxs <- list(cox1,cox2)
#' ## just calls sim.phregs !
#' dd <- sim.phregs(coxs,nsim,data=bmt,extend=c(0.001))
#' scox1 <- phreg(Surv(time,status==1)~strata(tcell)+platelet+age,data=dd)
#' scox2 <- phreg(Surv(time,status==2)~tcell+strata(platelet),data=dd)
#'
#' cbind(cox1$coef,scox1$coef)
#' cbind(cox2$coef,scox2$coef)
#' par(mfrow=c(1,2))
#' plot(cox1); plot(scox1,add=TRUE); 
#' plot(cox2); plot(scox2,add=TRUE); 
#' 
#' @export sim.phregs
sim.phregs <- function(coxs,n,data=NULL,rr=NULL,strata=NULL,
                       entry=NULL,extend=NULL,cens=NULL,rrc=NULL,...)
{# {{{
   scox1 <- draw.phreg(coxs[[1]],n,data=data)
   datas <- scox1$data
   stratam <-  scox1$strata
   rrm <- scox1$rr 

   cumhazl <- list()
   cumhazl[[1]] <- basecumhaz(coxs[[1]],only=1)
i=2
   if (length(coxs)>1) 
   for (i in 2:length(coxs)) {
      coxn <- draw.phreg(coxs[[i]],n,data=data,drawZ=FALSE,id=scox1$id)
      ind <-  match(names(datas),names(coxn$data),nomatch=0)
      ind <- ind[ind!=0]
      if (length(ind)>1)  datas <- cbind(datas,coxn$data[,-ind,drop=FALSE]) else datas <- cbind(datas,coxn$data)
      rrm <- cbind(rrm,coxn$rr)
      stratam <- cbind(stratam,coxn$strata)
      cumhazl[[i]] <- basecumhaz(coxs[[i]],only=1)
   }
   datas$orig.id <- scox1$id
   if (is.null(rr)) rr <- rrm
   if (is.null(strata)) strata <- stratam
   lentry <- NULL

   if (!is.null(extend)) cumhazl <- extendCums(cumhazl,NULL,extend=extend)

   ## simulate first  time
   simdata <- sim.phreg(cumhazl[[1]],n,rr=rr[,1],strata=strata[,1])
   l <- length(coxs)
   if (l>=2) 
   for (i in 2:l) {
      tall2 <- sim.phreg(cumhazl[[i]],n,rr=rr[,i],strata=strata[,i])
      simdata$status <- ifelse(simdata$time<tall2$time,simdata$status,i*tall2$status)
      simdata$time <- pmin(simdata$time,tall2$time)
   }
   ptt <- cbind(simdata,datas)

 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,entry=entry,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

return(ptt)
}# }}}

#' @export
simsubdist <- function(cumhazard,rr,n=NULL,entry=NULL,
                       type="cloglog",startcum=c(0,0),U=NULL,...)
{# {{{
  ## Fine-Gray model cloglog F1= 1-exp(-cum(t)*rr)
  ## logistic                F1= cum(t)*rr/(1+cum(t)*rr)
  ## rr                      F1= cum(t)*rr,  rr=exp(X^t beta) 
  if (!is.null(n)) rr <- rep(1,n)
  entry=NULL

  logit <- function(p) log(p/(1-p))

  if (cumhazard[1,2]>0)  cumhazard <- rbind(startcum,cumhazard)
  breaks <- cumhazard[,1]
  cumh <- cumhazard[,2] 
  mm <- tail(breaks,1)
  n <- length(rr)

  if (is.null(U)) U <- runif(n)

  if (type=="cloglog") {
      F1tau <- 1-exp(-tail(cumh,1)*rr)
      ttt <- -log(1-U*F1tau)/rr
  } else if (type=="logistic") {
     F1tau <- tail(cumh,1)*rr/(1+tail(cumh,1)*rr)
     v <- U*F1tau
     ttt <- exp(logit(v))/rr; 
  }  else if (type=="rr" | type=="cif") {
     F1tau <- tail(cumh,1)
     ttt <- U*F1tau
     ## rr only affects binomial draw 
     F1tau <- F1tau*rr 
  } else stop(" cloglog or logistic or give function (fun=) \n"); 
  ###
   entry <- cumentry <- rep(0,n)
   ttte <- ttt+cumentry
   rrx <- lin.approx(ttt,cumhazard,x=-1)
   timecause <- rrx
   ###
   rrx <- ifelse(rrx>mm,mm,rrx)
   status <- rbinom(n,1,F1tau) 
   rrx[status==0] <- mm
   dt <- data.frame(entry=entry,time=rrx,status=status,rr=rr,F1tau=F1tau,timecause=timecause)
   attr(dt,"cumhaz") <- cumhazard
   return(dt)
}# }}}

#' @export
invsubdist <- function(F1,u,entry=NULL,cond=1,ptrunc=NULL)
{# {{{
  ### computes inverse subdist F1^{-1}(u) 
  ### computes inverse subdist F1^{-1}(u)
  F1max <- tail(F1[,2],1)
  mmax <- tail(F1[,1],1)

  if (!is.null(entry))  F1entry <- subdist(F1,entry)[,2];  
  if (!is.null(entry)) if (is.null(ptrunc)) ptrunc <- 1- F1entry
  if (cond==0 & !is.null(entry))  u<-u*ptrunc+F1entry
  if (cond==1 & !is.null(entry))  u<-u*(F1max-F1entry)+F1entry 
  if (cond==1 & is.null(entry))   u<-u*F1max
  rrx  <- lin.approx(u,F1,x=-1)
  status <- rep(1,length(u))
  status[u>=F1max] <- 0
  dt <- data.frame(time=rrx,status=status,y=u)
  if (!is.null(entry)) dt <- cbind(dt,entry)
  attr(dt,"F1") <- F1
  attr(dt,"Fmax") <- F1max
  return(dt)
}# }}}

#' @export
subdist <- function(F1,times)
{# {{{
  rrx <-   lin.approx(times,F1,x=1)
  dt <- cbind(times,rrx)
  colnames(dt) <- c("time","subdist")
  return(dt)
}# }}}

#' Simulation of output from Cumulative incidence regression model 
#' 
#' Simulates data that looks like fit from fitted cumulative incidence model
#' 
#' @param cif output form prop.odds.subdist or ccr (cmprsk), can also call invsubdist with 
#'    with cumulative and linear predictor 
#' @param n number of simulations.
#' @param data to extract covariates for simulations (draws from observed
#' covariates).
#' @param Z to use these covariates for simulation rather than drawing new ones. 
#' @param rr possible vector of relative risk for cox model.
#' @param strata possible vector of strata 
#' @param drawZ to random sample from Z or not 
#' @param cens specifies censoring model, if "is.matrix" then uses cumulative
#' hazard given, if "is.scalar" then uses rate for exponential, and if not
#' given then takes average rate of in simulated data from cox model.
#' @param rrc possible vector of relative risk for cox-type censoring.
#' @param cumstart to start cumulatives at time 0 in 0. 
#' @param U uniforms to use for drawing of timing for cumulative incidence. 
#' @param pU uniforms to use for drawing event type (F1,F2,1-F1-F2). 
#' @param type of model logistic,cloglog,rr 
#' @param extend  to extend piecewise constant with constant rate. Default is average rate over time from cumulative (when TRUE), if numeric then uses given rate.
#' @param ... arguments for simsubdist (for example Uniform variable for realizations)
#' @author Thomas Scheike
#' @keywords survival
#' @examples
#' library(mets)
#' data(bmt)
#' nsim <- 100
#' 
#' ## logit cumulative incidence regression model 
#' cif <- cifreg(Event(time,cause)~platelet+age,data=bmt,cause=1)
#' estimate(cif)  
#' plot(cif,col=1)
#' simbmt <- sim.cif(cif,nsim,data=bmt)
#' dtable(simbmt,~status)
#' scif <- cifreg(Event(time,status)~platelet+age,data=simbmt,cause=1)
#' estimate(scif)
#' plot(scif,add=TRUE,col=2)
#' 
#' ## Fine-Gray cloglog cumulative incidence regression model 
#' cif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
#' estimate(cif)  
#' plot(cif,col=1)
#' simbmt <- sim.cif(cif,nsim,data=bmt)
#' scif <- cifregFG(Event(time,status)~strata(tcell)+age,data=simbmt,cause=1)
#' estimate(scif)
#' plot(scif,add=TRUE,col=2)
#' 
#' ################################################################
#' #  simulating several causes with specific cumulatives 
#' ################################################################
#' cif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
#' cif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=bmt,cause=2)
#' cifss <-  list(cif1,cif2)
#' simbmt <- sim.cifs(list(cif1,cif2),nsim,data=bmt,extend=0.005)
#' dtable(simbmt,~status)
#' scif1 <-  cifreg(Event(time,status)~strata(tcell)+age,data=simbmt,cause=1)
#' scif2 <-  cifreg(Event(time,status)~strata(platelet)+tcell+age,data=simbmt,cause=2)
#' cbind(cif1$coef,scif1$coef)   
#' ## can be off due to restriction F1+F2<= 1    
#' cbind(cif2$coef,scif2$coef)   
#'     
#' par(mfrow=c(1,2))   
#' ## Cause 1 follows the model    
#' plot(cif1); plot(scif1,add=TRUE,col=1:2,lwd=2)
#' # Cause 2:second cause is modified with restriction to satisfy F1+F2<= 1, so scaled down     
#' plot(cif2); plot(scif2,add=TRUE,col=1:2,lwd=2)
#'    
#' @aliases sim.cif sim.cifs simul.cifs setup.cif subdist simsubdist invsubdist
#' @export sim.cif
sim.cif <- function(cif,n,data=NULL,Z=NULL,rr=NULL,strata=NULL,
                    drawZ=TRUE,cens=NULL,rrc=NULL,
		    cumstart=c(0,0),U=NULL,pU=NULL,type=NULL,extend=NULL,...)
{# {{{
## also extracts coefficients and baseline from cifreg
if (!is.null(data))  {
   des <- 	draw.phreg(cif,n,data=data,Z=Z,drawZ=drawZ,...)
   dats <- des$data
   dats$orig.id <- des$id

   if (is.null(strata))  strata <- des$strata 
   if (is.null(rr)) rr <- des$rr  
   znames <- names(dats); 
    n <- nrow(dats)
} else {
	if (is.null(Z) & is.null(rr)) stop("must give Z or rr ")
	if (!is.null(Z)) {
	  if (is.data.frame(Z)) znames <- names(Z) else znames <- colnames(Z)
	} else znames <- NULL
	if (is.null(rr)) rr <- exp(as.matrix(Z) %*% cif$coef)
	n <- length(rr)
	orig.id <- 1:n
	dats <- NULL
}
if (is.null(strata)) strata <- rep(0,n)

if (inherits(cif,c("phreg","cifreg"))) cumhaz <- basecumhaz(cif,only=1,extend=extend)
else {
if (!is.list(cif)) stop("must be cifreg or list of hazards\n") else cumhaz <- cif
}
if (!is.null(extend))   cumhaz <- extendCums(cumhaz,NULL,extend=extend)

if (is.null(type)) {
if (is.null(cif$propodds)) type <- "cloglog" else type <- "logistic" 
}
ids <- 1:n

## {{{ simulation of cif 
    ptt <- c()
    for (i in unique(strata)) {
      whichi <- which(strata==i)
      cumhazj <- rbind(0,cumhaz[[i+1]])
      if (!is.null(U)) Ui <- U[whichi] else Ui <- NULL
      simj <- simsubdist(cumhazj,rr[whichi],type=type,U=Ui)
      simj$id <- ids[whichi]
      ptt  <-  rbind(ptt,simj)
    }
    dsort(ptt) <- ~id
### }# }}}
 if (!is.null(dats)) ptt <- cbind(ptt,dats)

 ### adds censoring 
 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

   attr(ptt,"type") <- type
   attr(ptt,"znames") <- znames
   attr(ptt,"cumhaz") <-  cumhaz
   return(ptt)
}# }}}

#' @export sim.cifs
sim.cifs <- function(cifs,n,data=NULL,rr=NULL,strata=NULL,Z=NULL,
                     cens=NULL,rrc=NULL,max.times=NULL,causes=c(1,2),
                     U=NULL,pU=NULL,extend=TRUE,type=NULL,restrict=TRUE,...)
{# {{{
if (!is.list(cifs)) stop("Cif models in list form\n"); 
if (length(cifs)!=2) stop("Only two models\n"); 

if (!is.null(type)) { 
   if (length(type)==1) type <- rep(type,2)
   model1 <- type[1]; model2 <- type[2];
} else {
   if (is.null(cifs[[1]]$propodds)) model1 <- "cloglog" else model1 <- "logistic" 
   if (is.null(cifs[[2]]$propodds)) model2 <- "cloglog" else model2 <- "logistic" 
}
###if (!is.list(cifs[[1]]$cumhaz)) cumhazl <- basecumhaz(cif,only=1) else cumhazl <- cif$cumhaz

   scox1 <- draw.phreg(cifs[[1]],n,data=data,Z=Z)
   datas <- as.data.frame(scox1$data)
   stratam <-  scox1$strata
   rrm <- scox1$rr 

   cumhazl <- list()
   cumhazl[[1]] <- basecumhaz(cifs[[1]],only=1)

   if (length(cifs)>1) 
   for (i in 2:length(cifs)) {
      coxn <- draw.phreg(cifs[[i]],n,data=data,drawZ=FALSE,id=scox1$id,Z=Z)
      ind <-  match(names(datas),names(coxn$data),nomatch=0)
      ind <- ind[ind!=0]
      if (length(ind)>1)  datas <- cbind(datas,coxn$data[,-ind,drop=FALSE]) else datas <- cbind(datas,coxn$data)
      rrm <- cbind(rrm,coxn$rr)
      stratam <- cbind(stratam,coxn$strata)
      cumhazl[[i]] <- basecumhaz(cifs[[i]],only=1)
   }
   datas$orig.id <- scox1$id
   if (is.null(rr)) rr <- rrm
   if (is.null(strata)) strata <- stratam
   lentry <- NULL

   if (!is.null(extend)) cumhazl <- extendCums(cumhazl,NULL,extend=extend)

  cifs[[1]]$cumhaz <- cumhazl[[1]]
  cifs[[2]]$cumhaz <- cumhazl[[2]]

  tau <- tail(cumhazl[[1]][[1]],1)[1]
  ## simulate first  time
  sim1 <- sim.cif(cumhazl[[1]],n,rr=rr[,1],strata=strata[,1],U=U,type=model1)
  sim2 <- sim.cif(cumhazl[[2]],n,rr=rr[,2],strata=strata[,2],U=U,type=model2)

  ## drawing which cause  that is seen if any 
  F1tau <- sim1$F1tau
  if (restrict) F2tau <- sim2$F1tau * (1-F1tau) else F2tau <- sim2$F1tau
  ptot <- F1tau+F2tau
  if (!is.null(pU)) {
      rt <- 1*(pU< pmin(ptot,1))
      rb <- 1*(pU< sim1$F1tau) 
  } else {
      rt <- rbinom(n,1,pmin(ptot,1))
      rb <- rbinom(n,1,sim1$F1tau/ptot)
  }
  cause=ifelse(rb==1,1,2)
  time=ifelse(cause==causes[1],sim1$timecause,sim2$timecause)
  cause <- rt*cause
  time[cause==0] <- tau

  ptt <- data.frame(time=time,status=cause,cause=cause,ptot=ptot)
  ptt <- cbind(ptt,datas)

### adds censoring 
 if (!is.null(cens)) {
      pct <- simCens(cens,rrc=rrc,n=n,...)
      ptt$time <- pmin(ptt$time,pct)
      ptt$status <- ifelse(ptt$time<pct,ptt$status,0)
 }

   return(ptt)
}# }}}

##' @export
simul.cifs <- function(n,rho1,rho2,beta,rc=0.5,depcens=0,rcZ=0.5,
                       bin=1,type=c("cloglog","logistic"),rate=1,
                       Z=NULL,U=NULL,pU=NULL) {# {{{
    p=length(beta)/2
    tt <- seq(0,6,by=0.1)
    if (length(rate)==1) rate <- rep(rate,2)
    Lam1 <- rho1*(1-exp(-tt/rate[1]))
    Lam2 <- rho2*(1-exp(-tt/rate[2]))

    if (length(bin)==1) bin <- rep(bin,2)
    if (length(rcZ)==1) rcZ <- c(rcZ,0)

    if (is.null(Z)) 
    Z=cbind((bin[1]==1)*(2*rbinom(n,1,1/2)-1)+(bin[1]==0)*rnorm(n),(bin[2]==1)*(rbinom(n,1,1/2))+(bin[2]==0)*rnorm(n))
    p <- ncol(Z)
    colnames(Z) <- paste("Z",1:p,sep="")

    cif1 <- setup.cif(cbind(tt,Lam1),beta[1:p],Znames=colnames(Z),type=type[1])
    cif2 <- setup.cif(cbind(tt,Lam2),beta[(p+1):(2*p)],Znames=colnames(Z),type=type[1])

    data <- sim.cifs(list(cif1,cif2),n,Z=Z,U=U,pU=pU,type=type[1],extend=NULL)

    if (!is.null(rc)) {
    if (depcens==0) censor=pmin(rexp(n,1)*(1/rc),6) else censor=pmin(rexp(n,1)*(1/(rc*exp(Z %*% rcZ))),6)
    } else censor <- 6 

    status=data$status*(data$time<=censor)
    time=pmin(data$time,censor)
    data <- data.frame(time=time,status=status)
    data <- cbind(data,Z)
    attr(data,"Lam1") <- cbind(tt,Lam1)
    attr(data,"Lam2") <- cbind(tt,Lam2)
    return(data)

}# }}}


simul.cifsRA <- function (n, rho1, rho2, beta, rc = 0.5, depcens.R = 0, rcZ = 0.5,pCA=0.5,pCR=0.5,
    bin = 1, type = c("cloglog", "logistic"), rate = 1, Z = NULL,rc2=0.3,depcens.Adm=0)
{# {{{
    p = length(beta)/2
    tt <- seq(0, 6, by = 0.1)
    if (length(rate) == 1)
        rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    if (length(bin) == 1)
        bin <- rep(bin, 2)
    if (length(rcZ) == 1)
        rcZ <- c(rcZ, 0)
    if (is.null(Z))
        Z = cbind((bin[1] == 1) * (2 * rbinom(n, 1, 1/2) - 1) +
            (bin[1] == 0) * rnorm(n), (bin[2] == 1) * (rbinom(n,
            1, 1/2)) + (bin[2] == 0) * rnorm(n))
    colnames(Z) <- paste("Z", 1:2, sep = "")
    p <- ncol(Z)
    cif1 <- setup.cif(cbind(tt, Lam1), beta[1:p], Znames = colnames(Z),
        type = type[1])
    cif2 <- setup.cif(cbind(tt, Lam2), beta[(p + 1):(2 * p)],
        Znames = colnames(Z), type = type[1])
    data <- sim.cifs(list(cif1, cif2), n, Z = Z,extend=NULL)

    admcens <- rbinom(n,1,pCA)
    if (depcens.Adm==1) rrA <- exp( Z %*% rcZ) else rrA <- 0
    censorA = admcens*(rrA+runif(n)*(6-rrA))+ 6*(admcens==0)

    statusA = data$status * (data$time <= censorA)
    timeA = pmin(data$time, censorA)
    statusA[statusA==0] <- 7

    Rcens <- rbinom(n,1,pCR)
    if (depcens.R == 0)  
        censorR = Rcens*runif(n)*6+(Rcens==0)*6
    else censorR = Rcens*pmin(rexp(n, 1) * (1/(rc * exp(Z %*% rcZ))),6) + (Rcens==0)*6
    censor <- ifelse( censorR < censorA, censorR,censorA)

    status = data$status * (data$time <= censor)
    time = pmin(data$time, censor)
    status[status==0 & (censorR > censorA)] <- 7
    ## extra censoring
    data <- data.frame(time=time,status=status,cens.time=censor,censorA=censorA,censorR=censorR,
		       timeA=timeA,statusA=statusA)
    return(cbind(data, Z))
}# }}}


#' @export
setup.cif  <- function(cumhazard,coef,Znames=NULL,type="logistic")
{# {{{
    cif <- list()
    cif$cumhaz <- cumhazard
    cif$se.cumhaz <- cumhazard
    cif$coef <- coef
    cif$model <- type
    if (type=="logistic") cif$propodds <- 1 else cif$propodds <- NULL
    cif$strata <- rep(0,nrow(cumhazard))
    cif$jumps <- 1:nrow(cumhazard)

    cif$nstrata <- 1
    class(cif) <- "defined"
    attr(cif,"znames") <- Znames
    return(cif)
}# }}}

##' @export
kumarsim <- function (n,rho1=0.71,rho2=0.40,rate = c(6.11,24.2),
	      beta=c(-0.67,0.59,-0.55,0.25,0.68,0.18,0.45,0.31),
	      labels= c("gp","dnr","preauto","ttt24(24,300]"),
	      depcens=0,type = c("logistic", "cloglog"),restrict=TRUE,...)
{# {{{
    p = length(beta)/2
    tt <- seq(0, 150, by = 1)
    if (length(rate) == 1)
    rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    
    ## fully saturated model for kumar covariates 
    Zdist <- c(0.21064815,0.02083333,0.05555556,0.01504630,
	       0.13888889,0.15393519, 0.02662037, 0.04398148, 
	       0.04745370, 0.02430556, 0.02199074, 0.03935185,
               0.02430556, 0.05555556, 0.01273148, 0.10879630)
    Zs <- expand.grid(gp=c(0,1),dnr=c(0,1),preauto=c(0,1),ttt24=c(0,1))
    Zs <- dsort(Zs,~ttt24+gp+dnr+preauto)

    samn <- sample(1:16,n,replace=TRUE,prob=c(Zdist))
    Z <- Zs[samn,]

    colnames(Z) <- labels
    cif1 <- setup.cif(cbind(tt, Lam1), beta[1:4], Znames = colnames(Z), type = type[1])
    cif2 <- setup.cif(cbind(tt, Lam2), beta[5:8], Znames = colnames(Z), type = type[1])
    data <- sim.cifs(list(cif2, cif1), n, Z = Z,extend=NULL,restrict=restrict,...)
    stat12 <- which(data$status %in% c(1,2))
    data$status[stat12] <- c(2,1)[data$status[stat12]]

    ## kumar censoring, cox model 
    c0 <- list()     
    c0$cumhaz <- cbind(c(0,20,60,90,160),
		       c(0, 0.07797931, 0.28512764, 0.76116180, 1.95720759))
    c0$coef <- c(1.8503113,-0.6976226,0.5828763,-0.2000003)

    if (depcens == 0)
        censor  <- rchaz(c0$cumhaz,n=n)
    else {
        rrc <- exp(as.matrix(Z) %*% c0$coef)
        censor  <- rchaz(c0$cumhaz,rrc)
    }
    status = data$status * (data$time <= censor$time)
    time = pmin(data$time, censor$time)
    data <- data.frame(time = time, status = status)
    return(cbind(data, Z))
}# }}}

##' @export
kumarsimRCT <- function (n,rho1=0.71,rho2=0.40,rate = c(6.11,24.2),
	      beta=c(-0.67,0.59,-0.55,0.25,0.68,0.18,0.45,0.31),
	      labels= c("gp","dnr","preauto","ttt24(24,300]"),
	      nocens=0,addcens=1,rct=1,
	      type = c("logistic", "cloglog"),restrict=TRUE,
	      censpar=c(1,1,1,1), F1par=c(1,1,1,1), F2par=c(1,1,1,1),
	      treatmodel=c(-0.18,-0.16,0.06,0.24),... )
{# {{{
    p = length(beta)/2
    tt <- seq(0, 150, by = 1)
    if (length(rate) == 1)
    rate <- rep(rate, 2)
    Lam1 <- rho1 * (1 - exp(-tt/rate[1]))
    Lam2 <- rho2 * (1 - exp(-tt/rate[2]))
    
    ## fully saturated model for kumar covariates 
    Zdist <- c(0.21064815,0.02083333,0.05555556,0.01504630,
	       0.13888889,0.15393519, 0.02662037, 0.04398148, 
	       0.04745370, 0.02430556, 0.02199074, 0.03935185,
               0.02430556, 0.05555556, 0.01273148, 0.10879630)
    Zs <- expand.grid(gp=c(0,1),dnr=c(0,1),preauto=c(0,1),ttt24=c(0,1))
    Zs <- dsort(Zs,~ttt24+gp+dnr+preauto)

    samn <- sample(1:16,n,replace=TRUE,prob=c(Zdist))
    Z <- Zs[samn,]
    ## randomized gp instead
    if (rct==1) Z[,1] <- rbinom(n,1,0.5)
    ## randomized gp given other covariates 
    if (rct==2) {
	    lp <- as.matrix(cbind(1,Z[,-1])) %*% treatmodel 
	    Z[,1] <- rbinom(n,1, expit(lp))
    }
    colnames(Z) <- labels
    cif1 <- setup.cif(cbind(tt, Lam1), F1par*beta[1:4], Znames = colnames(Z), type = type[1])
    cif2 <- setup.cif(cbind(tt, Lam2), F2par*beta[5:8], Znames = colnames(Z), type = type[1])
    data <- sim.cifs(list(cif2, cif1), n, Z = Z,extend=NULL,restrict=restrict,...)
    stat12 <- which(data$status %in% c(1,2))
    data$status[stat12] <- c(2,1)[data$status[stat12]]

   if (nocens==0) {
    ## kumar censoring, cox model 
    c0 <- list()     
    c0$cumhaz <- cbind(c(0,20,60,90,160),
		       c(0, 0.07797931, 0.28512764, 0.76116180, 1.95720759))
    c0$coef <-censpar* c(1.8503113,-0.6976226,0.5828763,-0.2000003)

	if (addcens)  {
	  ## draw from cox gp model  
         rrc <- exp(as.matrix(Z) %*% c(c(1,0,0,0)*c0$coef))
         censorgp  <- rchaz(c0$cumhaz,rrc)
	 ## draw from cox other covariates   
         rrc <- exp(as.matrix(Z) %*% c(c(0,1,1,1) * c0$coef))
         censoroc  <- rchaz(c0$cumhaz,rrc)
	 ## miniumum of the two censoring times 
	 censor <- censorgp
	 censor$time <- pmin(censoroc$time,censorgp$time)
	 censor$status <- pmax(censoroc$status,censorgp$status)
	} else {
        rrc <- exp(as.matrix(Z) %*% c0$coef)
        censor  <- rchaz(c0$cumhaz,rrc)
	}
    status = data$status * (data$time <= censor$time)
    time = pmin(data$time, censor$time)
    data <- data.frame(time = time, status = status)
    }
    return(cbind(data, Z))
}# }}}

######################## ##################### #####################
## illness death competing risks with two causes of death
## First simple model, then with covariates addedd 
## effect of transition into 2 being early for cause 3, and 4
######################## ##################### #####################
simMultistateII <- function(cumhaz,death.cumhaz,death.cumhaz2,n=NULL,
		    rr=NULL,rd=NULL,rd2=NULL,gamma23=0,gamma24=0,early2=10000,
		    gap.time=FALSE,max.recurrent=100,cens=NULL,rrc=NULL,extend=TRUE,...) 
{# {{{

  status <- NULL

  ## cumhaz is cumulative out of state 1 to state 2 
  if (!is.null(n)) rr <- matrix(1,n,1) 
  n <- nrow(rr); 
  ## covariate adjustment 
  if (is.null(rd))   rd  <- matrix(1,n,length(death.cumhaz))
  if (is.null(rd2))  rd2  <- matrix(1,n,length(death.cumhaz2))

  ####
  if (!is.null(cens)) cens <- list(cens)
  cumss  <-  c(list(cumhaz),death.cumhaz,death.cumhaz2,cens)
  ### extend of cumulatives
  for (i in seq_along(cumss))  cumss[[i]] <- rbind(0,cumss[[i]])
  ## extend range of all cumulatives to max range, by linear extension
  cumss <- extendCums(cumss,NULL,extend=extend)
  maxtime <- tail(cumss[[1]],1)[1]

 if (!is.null(cens)) { 
	  cens <- cumss[[1+length(death.cumhaz)+length(death.cumhaz2)+1]]
	  if (is.null(rrc)) rrc <- rep(1,n); 
	  ctime <- rchaz(cens,rrc)$time
  } else ctime <- rep(maxtime,n)

  ## hazards out of 1 and out of 2
  chaz1 <- cumss[1:3]; rr1 <- cbind(rr,rd)
  chaz2 <- cumss[4:5]; rr2 <- cbind(rd2)

  ## time out of state 1 
  tall <- rchazl(chaz1,rr1,causes=c(2,3,4))
  tall$id <- 1:n
  tall$status[tall$time>ctime] <- 0; 
  tall$time[tall$time>ctime] <- ctime[tall$time>ctime] 
  tall$from <- 1
  tall$to <- tall$status

  ## simulating out of 2 
  tall2 <- subset(tall,status==2)
  rr23t <- exp(gamma23*(tall2$time<early2))
  rr24t <- exp(gamma24*(tall2$time<early2))
  sim2 <- rchazl(chaz2,rr2[tall2$id,]*cbind(rr23t,rr24t),causes=c(3,4),entry=tall2$time)
  sim2$id <- tall2$id
  ctime2 <- ctime[tall2$id]
  sim2$status[sim2$time>ctime2] <- 0; 
  sim2$time[sim2$time>ctime2] <- ctime2[sim2$time>ctime2] 
  sim2$from <- 2
  sim2$to <- sim2$status
  tall <- rbind(tall,sim2)

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  return(tall)
  }# }}}

##' Simulation of illness-death model 
##'
##' Simulation of illness-death model 
##'
##' simMultistate with different death intensities from states 1 and 2 
##'
##' Must give cumulative hazards on some time-range 
##'
##' @param n number of id's
##' @param cumhaz cumulative hazard of going from state 1 to 2.
##' @param cumhaz2 cumulative hazard of going from state 2 to 1.
##' @param death.cumhaz cumulative hazard of death from state 1.
##' @param death.cumhaz2 cumulative hazard of death from state 2.
##' @param rr relative risk adjustment for cumhaz
##' @param rr2 relative risk adjustment for cumhaz2
##' @param rd relative risk adjustment for death.cumhaz
##' @param rd2 relative risk adjustment for death.cumhaz2
##' @param rrc relative risk adjustment for censoring
##' @param gap.time if true simulates gap-times with specified cumulative hazard
##' @param max.recurrent limits number recurrent events to 100
##' @param dependence 0:independence; 1:all share same random effect with
##'   variance var.z; 2:random effect exp(normal) with correlation structure
##'   from cor.mat; 3:additive gamma distributed random effects, z1= (z11+
##'   z12)/2 such that mean is 1 , z2= (z11^cor.mat(1,2)+ z13)/2, z3=
##'   (z12^(cor.mat(2,3)+z13^cor.mat(1,3))/2, with z11 z12 z13 are gamma with
##'   mean and variance 1 , first random effect is z1 and for N1 second random
##'   effect is z2 and for N2 third random effect is for death
##' @param var.z variance of random effects
##' @param cor.mat correlation matrix for var.z variance of random effects
##' @param cens rate of censoring exponential distribution
##' @param extend to extend hazards to max-time
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' ########################################
##' ## getting some rates to mimick 
##' ########################################
##' library(mets)
##' data(CPH_HPN_CRBSI)
##' dr <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi 
##' base4 <- CPH_HPN_CRBSI$mechanical
##' dr2 <- scalecumhaz(dr,1.5)
##' cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
##'
##' iddata <- simMultistate(100,base1,base1,dr,dr2,cens=cens)
##' dlist(iddata,.~id|id<3,n=0)
##'  
##' ### estimating rates from simulated data  
##' c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
##' c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
##' c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
##' c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
##' ###
##' par(mfrow=c(2,3))
##' plot(c0)
##' lines(cens,col=2) 
##' plot(c3,main="rates 1-> 3 , 2->3")
##' lines(dr,col=1,lwd=2)
##' lines(dr2,col=2,lwd=2)
##' ###
##' plot(c1,main="rate 1->2")
##' lines(base1,lwd=2)
##' ###
##' plot(c2,main="rate 2->1")
##' lines(base1,lwd=2)
##'  
##' @aliases simMultistateII
##' @export
simMultistate <- function(n,cumhaz,cumhaz2,death.cumhaz,death.cumhaz2,
		    rr=NULL,rr2=NULL,rd=NULL,rd2=NULL,rrc=NULL,
		    gap.time=FALSE,max.recurrent=100,
		    dependence=0,var.z=0.22,cor.mat=NULL,cens=NULL,extend=TRUE,...) 
{# {{{

  fdeath <- dtime <- NULL # to avoid R-check 
  status <- dhaz <- NULL; dhaz2 <- NULL

  if (dependence==0) { z <- z1 <- z2 <- zd  <- zd2 <-  rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
###	      z <- exp(rnorm(n,1)*var.z[1]^.5)
	      z1 <- z; z2 <- z; zd <- z
	      if (!is.null(cor.mat)) { zd <- rep(1,n); }
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
###	      print(summary(z))
###	      print(cor(z))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
###	      print(summary(z))
###	      print(cor(z))
      } else stop("dependence 0-3"); # }}}

  ## covariate adjustment 
  if (is.null(rr))  rr <- z1; 
  if (is.null(rr2)) rr2 <- z2; 
  if (is.null(rd))  rd  <- zd; 
  if (is.null(rd2)) rd2 <- zd2; 

  ll <- nrow(cumhaz)
  ### extend of cumulatives
  cumhaz <- rbind(c(0,0),cumhaz)
  cumhaz2 <- rbind(c(0,0),cumhaz2)
  death.cumhaz <- rbind(c(0,0),death.cumhaz)
  death.cumhaz2 <- rbind(c(0,0),death.cumhaz2)

  haz <- haz2 <- NULL
  ## range max of cumhaz and cumhaz2 

  if (!is.null(cens)) {
      if (is.matrix(cens))  {
      out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz,death.cumhaz2,cens),NULL,extend=extend)
      cens <- out$cum5
      }
  } else {
     out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz,death.cumhaz2),NULL,extend=extend)
  }
  cumhaz <- out$cum1
  cumhaz2 <- out$cum2
  cumhazd <- out$cum3
  cumhazd2 <- out$cum4
  max.time <- tail(cumhaz[,1],1)

  tall <- rcrisk(cumhaz,cumhazd,rr,rd,cens=cens,extend=NULL)

  tall$id <- 1:n
  ### fixing the first time to event
  tall$death <- 0
  ### cause 2 is death state 3, cause 1 is state 2
  tall <- dtransform(tall,status=3,status==2)
  tall <- dtransform(tall,death=1,status==3)
  tall <- dtransform(tall,status=2,status==1)
  ### dead or censored
  deadid <- (tall$status==3 | tall$status==0)
  tall$from <- 1
  tall$to <- tall$status
  ## id's that are dead: tall[deadid,]
  ## go furhter with those that are not yet dead  or censored
  tt <- tall[!deadid,,drop=FALSE]
  ## also check that we are before max.time
  tt <- subset(tt,tt$time<max.time)

  i <- 1; 
  while ( (nrow(tt)>0) & (i < max.recurrent)) {# {{{
	  i <- i+1
	  nn <- nrow(tt)

	  z1r <- rr[tt$id]
	  zdr <- rd[tt$id]
	  z2r <- rr2[tt$id]
	  zd2r <- rd2[tt$id]

	  if (i%%2==0) { ## in state 2
	  ## out of 2 for those in 2
          tt1 <- rcrisk(cumhaz2,cumhazd2,z2r,zd2r,entry=tt$time,cens=cens,extend=NULL)
          tt1$death <- 0
	  ### status 2 is death state 3, status 1 is state 1
	  tt1 <- dtransform(tt1,status=3,status==2)
	  tt1 <- dtransform(tt1,death=1,status==3)
	  tt1$from <- 2
	  tt1$to <- tt1$status
	  ## take id from tt
	  tt1$id <-  tt$id
	  ### add to data 
	  tall <- rbind(tall,tt1,row.names=NULL)

          deadid <- (tt1$status==3 | tt1$status==0)
	  ### those that are still under risk 
	  tt <- tt1[!deadid,,drop=FALSE]
	  ## also keep only those before max.time
          tt <- subset(tt,tt$time<max.time)
		  
	  } else { ## in state 1
	  ## out of 1 for those in 1
          tt1 <- rcrisk(cumhaz,cumhazd,z1r,zdr,entry=tt$time,cens=cens,extend=NULL)

          tt1$death <- 0
	  ### status 2 is death state 3, status 1 is state 2
	  tt1 <- dtransform(tt1,status=3,status==2)
	  tt1 <- dtransform(tt1,death=1,status==3)
	  tt1 <- dtransform(tt1,status=2,status==1)
	  tt1$from <- 1
	  tt1$to <- tt1$status
	  tt1$id <-  tt$id

	  ### add to data 
	  tall <- rbind(tall,tt1,row.names=NULL)

	  ## take id from tt
          deadid <- (tt1$status==3 | tt1$status==0)
	  ### those that are still under risk 
	  tt <- tt1[!deadid,,drop=FALSE]
	  ### also only keep those before max.time
          tt <- subset(tt,tt$time<max.time)
	  }

  }  # }}}

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"death.cumhaz2") <- cumhazd2
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"cens.cumhaz") <- cens
  attr(tall,"z") <- z

  return(tall)
  }# }}}

F1addfg<-function(t,lam0=0.13,beta=c(-0.5),x=0) # FG
{ ## {{{
baset <- lam0*pnorm((t-.70)/0.15)
return( 1 - exp(-baset*exp(c(x * beta)))) 
} ## }}}

##' @export 
corsim.prostate <- function(n,theta=1,thetaslope=0,censS=c(0,1),pcens=0.5,test=0,mt=1,same.cens=TRUE,country=TRUE,
   delayed=FALSE,ptrunc=0.5,lam0=0.13,truncS=c(0,1)) 
{ ## {{{
###n <- 10; theta <- 1; thetaslope <- 0; mt <- 1
if (country==TRUE) xl <- sample(1:4,n,replace=TRUE) else xl <- rep(1,n)
x <- (xl==1)
tt<-seq(0,1,length=100)
###
###n=100;theta=1;lam0=0.5;beta=0.3;crate=2
thetat <- exp(log(theta))
F11x<-F1addfg(1,x=x,lam0=lam0)
F12x<-F1addfg(1,x=x,lam0=lam0)
###
thetaslut <- exp(log(theta)+thetaslope*(1-1/2))
p11 <- thetaslut*F11x*F12x/((1-F11x)+thetaslut*F11x)
p12 <- F11x-p11
p21 <- F12x-p11
p22 <- 1-F12x-F11x+p11
###print(apply(cbind(p11,p12,p21),1,sum))
###print(cbind(p11,p12,p21,p22))
if (test==1) { ## {{{
for (i in 1:2) {
print(x[i,]); 
F11xt<-F1addfg(tt,x=x[i,])
F12xt<-F1addfg(tt,x=x[i,])
p11t <- thetat* F11xt*F12xt/((1-F11xt)+thetat*F11xt)
cortt <- ((p11t)/(F12xt-p11t))/(F11xt/(1-F11xt))
###plot(tt,log(cortt))
if (i==1) { 
plot(tt,p11t,type="l",ylim=c(0,0.1),xlim=c(0,mt))
###lines(tt,F11x[i]-p11t,col=2)
###lines(tt,F12x[i]-p11t,col=2)
} else lines(tt,p11t,col=2);
###if (sum(diff(p11t<0))>0) stop("dec\n"); 
###p11 <- max(p11t)
###p12 <- F11x[i]-p11
###p21 <- F12x[i]-p11
###p22 <- 1- F12x[i]-F11x[i]+p11
###pnn <- 1- F12x[i]-F11x[i]+p11
}
} ## }}}
###apply(cbind(p11,p12,p21,p22),1,sum)
###
print(table(F11x))
print(table(p11))

types <- rep(0,n)
causes <- matrix(0,n,2)
stime<-matrix(1+1,n,2); 
for (i in 1:n)
{ ## {{{ 
ptype <- runif(1)
if (ptype<=p11[i]) {
	types[i] <- 1
	myhazx<-F1addfg(tt,x=x[i,])/F12x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist\n"); 
	stime[i,2]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	f1<- F1addfg(tt,x=x[i,])
       	myhazx<- (F12x[i]/p11[i]) * (thetat*f1/((1-f1)+thetat*f1))
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist\n"); 
	stime[i,1]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(1,1)
}
if ((ptype>p11[i]) & (ptype<=p12[i]+p11[i])) {
	types[i] <- 2
	f1 <- F1addfg(tt,x=x[i,])
	myhazx<- ( f1 - thetat*F12x[i]*f1/((1-f1)+thetat*f1))/p12[i]; 
	myhazx <- f1/F11x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist 2 \n"); 
	stime[i,1]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(1,2)
	stime[i,2] <- runif(1)*1
}
if ((ptype>p11[i]+p12[i]) && (ptype<=p21[i]+p12[i]+p11[i])) {
	types[i] <- 3
        f2 <- F1addfg(tt,x=x[i,])
	myhazx <-  (f2 - (thetat*F11x[i]*f2/((1-F11x[i])+thetat*F11x[i])))/p21[i]; 
	myhazx <- f2/F12x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist3 \n"); 
	stime[i,2]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(2,1)
	stime[i,1] <- runif(1)*1
}
if (ptype>p11[i]+p12[i]+p21[i] ) {
	types[i] <- 4
	causes[i,] <- c(2,2)
	stime[i,1:2] <- runif(2)*1
}

} ## }}} 
stime <- c(stime)
cause <- c(causes)

###print(summary(stime))
###print(sum(stime==mt))

###same.cens=TRUE
if (same.cens==TRUE) {
	ctime <- rep(rbinom(n,1,pcens)*runif(n,censS),each=2)
        ctime[ctime==0] <- 1;
}
else {
	ctime<- rbinom(2*n,1,pcens)*runif(2*n,censS)
        ctime[ctime==0] <- 1;
}

cens <- (ctime< stime)
time <- ifelse(cens,ctime,stime)
cause <- ifelse(cens,0,cause)
id <- rep(1:n,rep(2,n))

country <- c()
country[xl==1] <- "SWE"
country[xl==2] <- "DK"
country[xl==3] <- "FIN"
country[xl==4] <- "NOR"


if (delayed) {
if (same.cens==TRUE) {
    etime <- rep(rbinom(n,1,ptrunc)*(runif(n,truncS)),each=2)
} else  etime<- rbinom(2*n,1,ptrunc)*(runif(2*n,truncS))
} else etime <- rep(0,2*n)


data<-data.frame(time=mt*time,cause=cause,xl=rep(xl,each=2),
		 country=rep(country,each=2),id=id,cens=cens,stime=mt*stime,type=rep(types,each=2),
		 f1=rep(F11x,each=2),p11=rep(p11,each=2),p12=rep(p12,each=2),p21=rep(p21,each=2),
		 p22=rep(p22,each=2),entry=mt*etime,truncated=(time<etime), stringsAsFactors=TRUE)
return(data)
} ## }}}

##' @export 
simnordic <- function(n,cordz=2,cormz=3,
		      censSmz=c(0,1),censSdz=c(0,1),
		      pcensmz=0.5,pcensdz=0.5,
		      ptrunc=0.5,
		      truncSmz=c(0,0.5),truncSdz=c(0,0.5),
		      country=TRUE,same.cens=TRUE,
                      delayed=FALSE,only.delayed=FALSE,lam0=0.13) 
{ ## {{{
outdz <- corsim.prostate(n,theta=cordz,censS=censSdz,pcens=pcensdz,mt=1,same.cens=same.cens,test=0,country=country,
	 delayed=delayed,ptrunc=ptrunc,lam0=lam0,truncS=truncSdz) 
outmz <- corsim.prostate(n,theta=cormz,censS=censSmz,pcens=pcensmz,mt=1,same.cens=same.cens,test=0,country=country,
	 delayed=delayed,ptrunc=ptrunc,lam0=lam0,truncS=truncSmz)
outdz$zyg <- "DZ" 
outmz$zyg <-  "MZ"
outmz$id <- outmz$id+nrow(outdz)
###
out <- rbind(outdz,outmz)
out$time <- out$time*100
out$entry <- out$entry*100
###table(out$type,out$country)
###table(out$type,out$cause)
if (country==TRUE) out$country <- relevel(factor(out$country),ref="SWE")
###table(out$country)
outk <- out[,c("country","cause","id","time","zyg","type","entry","truncated")]

if (only.delayed) outk <- outk[!out$truncated,]

return(outk)
} ## }}}

## {{{ simulation for gamma distributed cif model 

lap<-function(theta,t) { return( (1+t/theta)^(-theta)) }

##' @export 
ilap<-function(theta,t) {
	itheta<-1/theta; return((t^(-itheta)-1)/(itheta)) }

F1clust<-function(t,rtheta=1,theta=1,lam0=0.5,x=0) {
	return(1-exp(-rtheta*ilap(theta,1-F1addfg(t,lam0=lam0,x=x))))
}

F1<-function(t,lam0=0.5,beta=0.3,x=0) # additive version
{ return( 1 - exp(-t*lam0-t*x*beta)) }

F2<-function(t,lam0=0.5,beta=0.3,x=0) # additive version
{ return( 1 - exp(-t*lam0-t*x*beta)) }

sim.F1F2<-function(n,theta=1,lam0=0.1,beta=0.3,lam02=0.1,beta2=0,crate=3,cstart=0) 
{ ## {{{
	x<-rbinom(n,1,0.5); 
	tt<-seq(0,3,length=100)
	F1x<-F1(3,x=x,beta=beta,lam0=lam0)
	F2x<-F2(3,x=x,beta=beta2,lam0=lam02)

	okX <- (F1x+F2x<=1)*1
	n <- sum(okX)
	F1x <- F1x[okX]
	F2x <- F1x[okX]

	### death or alive
	cause12<-rbinom(n,1,F1x+F2x)
	### if death 1 or 2
	cause1e2<-rbinom(n,1,F1x/(F1x+F2x))
	cause <- rep(0,n)
	stime<-rep(3,n); 

	for (i in 1:n)
	{
		if (cause12[i]==1) {
			if (cause1e2[i]==1) {
				cause[i] <- 1
				myhazx<-F1(tt,x=x[i],beta=beta,lam0=lam0)/F1x[i]
				stime[i]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
			} else  {
				cause[i] <- 2
				myhazx<-F2(tt,x=x[i],beta=beta2,lam0=lam02)/F2x[i]
				stime[i]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
			}
		}
	}

	### censoring on top
	ctime<-(cstart+runif(n)*crate)
	time<-apply(cbind(ctime,stime),1,min)
	status<-(stime<ctime); 
	if (sum(status==0)>0) cause[status==0]<-0; 
	data<-data.frame(time=time,ctime=ctime,status=status,X=x,cause=cause)
	return(data)
} ## }}}

sim.F1<-function(n,lam0=0.5,beta=0.3,Cint=c(0,1)) 
{ ## {{{
	x<-runif(n);
	tt<-seq(0,1,length=100)
	F11x<-F1(1,x=x,beta=beta,lam0=lam0)
	cause1<-rbinom(n,1,F11x)
	###
	stime<-rep(2,n); 
	for (i in 1:n) {
		if (cause1[i]==1) {
			myhazx<-F1(tt,x=x[i],beta=beta,lam0=lam0)/F11x[i]
			stime[i]<-cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
		} 
	}
	ctime<-runif(n,Cint)
	time<-pmin(ctime,stime)
	status<-(stime<ctime); 
	cause1[status==0]<-0; 
	data<-data.frame(time=time,status=status,X=x,cause=cause1)
	return(data)
} ## }}}

sim.F1clust<-function(n,theta=1,lam0=0.5,Clims=c(0,1),
		      beta=-0.5,same.cens=FALSE,fix.cens=FALSE)
{ ## {{{ 
	k<-n/2; tt<-seq(0,1,length=100)
	theta <- 1/theta
	rtheta<-rgamma(k,theta,scale=1/theta)
	stime<-c();cause1<-c();id<-c();vtheta<-c(); X<-c()

	for (i in 1:k)
	{  ## {{{  
		x <- rbinom(1,1,0.25)
		x <- rep(x,2)
		X<-c(X,x); 
		F11x<-F1clust(1,rtheta=rtheta[i],theta=theta,x=x,lam0=lam0) 
		cause<-rbinom(2,1,F11x); 
		###cause1<-c(cause1,cause); 
		id<-c(id,rep(i,2)); vtheta<-c(vtheta,rep(rtheta[i],2))

		for (j in 1:2) {
			if (cause[j]==1) {
				myhazx<-F1clust(tt,x=x[j],rtheta=rtheta[i],theta=theta,lam0=lam0)/F11x[j]
				stime<-c(stime,cpred(cbind(myhazx,tt),runif(1))[1,2]+ runif(1,0,0.001))
				cause1 <- c(cause1,1);
			} else { stime<-c(stime,runif(1)); cause1 <- c(cause1,2);}
		}
	} ## }}} 

	if (same.cens) ctime <- rep(runif(k,min=Clims[1],max=Clims[2]),each=2) else ctime<-runif(n,min=Clims[1],max=Clims[2]);
	if (fix.cens) ctime <- rep(Clims[2],n)

	time<-apply(cbind(ctime,stime),1,min)
	status<-(stime<ctime); 
	cause1[status==0]<-0; 
	data<-data.frame(time=time,X=X,cause=cause1,stime=stime,ctime=ctime,id=id,theta=vtheta)
	return(data)
} ## }}}
## }}}

##' @export 
corsim.prostate.random <- function(n,theta=1,censS=c(0,1),
   pcens=0.5,test=0,mt=1,same.cens=TRUE,country=TRUE,
   delayed=FALSE,ptrunc=0.5,lam0=0.13,truncS=c(0.5,1)) 
{ ## {{{
### n <- 1000; theta <- 2; 
k<-n/2; 
theta <- 1/theta
tt<-seq(0,1,length=100)
rtheta<-rgamma(k,theta,scale=1/theta)
stime<-c();cause1<-c();id<-c();vtheta<-c(); X<-c()

for (i in 1:k)
{  ## {{{  
if (country==TRUE) x <- rbinom(1,1,0.25) else x <- 0
x <- rep(x,2)
X <- c(X,x); 
F11x <- F1clust(1,rtheta=rtheta[i],theta=theta,x=x,lam0=lam0) 
cause <- rbinom(2,1,F11x); 
###cause1<-c(cause1,cause); 
id<-c(id,rep(i,2)); vtheta<-c(vtheta,rep(rtheta[i],2))

for (j in 1:2) {
if (cause[j]==1) {
myhazx<-F1clust(tt,x=x[j],rtheta=rtheta[i],theta=theta,lam0=lam0)/F11x[j]
stime<-c(stime,cpred(cbind(myhazx,tt),runif(1))[1,2]+ runif(1,0,0.001))
cause1 <- c(cause1,1);
} else { stime<-c(stime,runif(1)); cause1 <- c(cause1,2);}
}
} ## }}} 

###same.cens=TRUE
if (same.cens==TRUE) {
    ctime <- rep(rbinom(n,1,pcens)*runif(n,min=censS[1],max=censS[2]),each=2)
    ctime[ctime==0] <- 1;
}
else {
     ctime<- rbinom(2*n,1,pcens)*runif(2*n,min=censS[1],max=censS[2])
     ctime[ctime==0] <- 1;
}

cens <- (ctime < stime)
time <- ifelse(cens,ctime,stime)
cause <- ifelse(cens,0,cause1)
id <- rep(1:n,each=2)

if (country==TRUE) {
country <- c()
country[X==1] <- "DK"
no <- sum(X==0)
country[X==0] <- rep(sample(c("SWE","FIN","NOR"),no/2,replace=TRUE),each=2)
country <- relevel(factor(country),ref="DK")
} else  country <- rep("same",2*n)

if (delayed) {
if (same.cens==TRUE) etime <- rep(rbinom(n,1,ptrunc)*runif(n,min=truncS[1],max=truncS[2]),each=2)
 else  etime<- rbinom(2*n,1,ptrunc)*(runif(2*n,min=truncS[1],max=truncS[2]))
} else etime <- rep(0,2*n)

data<-data.frame(time=time,cause=cause,x=X,
	 country=country,id=id,cens=cens,
	 stime=stime, entry=etime,truncated=(time<etime),stringsAsFactors=TRUE)
return(data)
} ## }}}

##' @export 
simnordic.random <- function(n,cordz=2,cormz=3,
     censSmz=c(0,1),censSdz=c(0,1),pcensmz=0.5,pcensdz=0.5,
     ptrunc=0.5,truncSmz=c(0.5,1),truncSdz=c(0.5,1),
     country=TRUE,same.cens=TRUE,
     delayed=FALSE,only.delayed=FALSE,lam0=0.13) 
{ ## {{{
outdz <- corsim.prostate.random(n,theta=cordz,censS=censSdz,
 pcens=pcensdz,mt=1,same.cens=same.cens,test=0,country=country,
 delayed=delayed,ptrunc=ptrunc,lam0=lam0,truncS=truncSdz) 
outmz <- corsim.prostate.random(n,theta=cormz,censS=censSmz,
 pcens=pcensmz,mt=1,same.cens=same.cens,test=0,country=country,
 delayed=delayed,ptrunc=ptrunc,lam0=lam0,truncS=truncSmz)
outdz$zyg <- "DZ" 
outmz$zyg <-  "MZ"
outmz$id <- outmz$id+nrow(outdz)
###
out <- rbind(outdz,outmz)
out$time <- out$time*100
out$entry <- out$entry*100
out$zyg <- relevel(factor(out$zyg),ref="MZ")
if (country==TRUE) out$country <- relevel(factor(out$country),ref="DK")
###
if (only.delayed) out <- out[!out$truncated,]

return(out)
} ## }}}

##' @export
drop.strata <- function(x) {
    drop.specials(x, c('strata', 'strataC'))
}

##' @export
drop.specials <- function(x, components, ...) {
    variables <- c()
    for (comp in components) {
        mm <- lava::Specials(x, comp, ...)
        vars <- unlist(lapply(mm, function(x) strsplit(x,',')))
        variables <- c(variables, list(vars))
        for (i in mm) {
            newf <- as.formula(paste0('.~.-', comp, '(',i,')'))
            x <- update(x, newf)
        }
    }
    names(variables) <- components
    return( structure(x, variables=variables) )
}

onerunfam <- function(i,n,alr=0,manual=1,time=0,simplealr=1,theta=1) { ## {{{ 
### n=200; beta=0.2; theta=1; time=0; i=1
    print(i)
    dd <- simBinFam(n,beta=0,theta) 
    ddl <- fast.reshape(dd,varying="y",keep="y")
    out2t <- system.time(
        marg  <-  glm(y~+1,data=ddl,family=binomial())
        )
    ps <- predict(marg,type="response")
    if (time==1) print(out2t)

    if (manual==1) {
        if (time==1) print(date())
        ddl$ps <- ps
        fam <- familycluster.index(ddl$id)
        prtfam <- ddl[fam$familypairindex,]
        prtfam$subfam <- fam$subfamilyindex
### lave afhængighedsdesign pba af wide format zyg1*zyg2  feks
        prtfamclust <- data.frame(fast.reshape(prtfam,id="subfam"))
###     des <- model.matrix(~-1+factor(num1):factor(num2),data=prtfamclust)
        mf <- with(prtfamclust,(num1=="m")*(num2=="f")*1)
        mb <- with(prtfamclust,(num1=="m" | num1=="f")*(num2=="b1" | num2=="b2")*1)
        bb <- with(prtfamclust,(num1=="b1" )*(num2=="b1" | num2=="b2")*1)
        des <- cbind(mf,mb,bb)*1
        mulig <- (apply(des,2,sum)>0)
        names <- colnames(des)
        prtfamclust <- cbind(prtfamclust,des)
        prtfam <- fast.reshape(prtfamclust,varying=c("y","ps","num"),keep=c("y","ps","num","subfam","id",names))
        prtfam$famclust <- prtfam$id1
        destheta <- prtfam[,names]
        if (time==1) print(date())
        udt <-  system.time(
            udf <- binomial.twostage(prtfam$y,data=prtfam,
                                     clusters=prtfam$subfam, detail=0,
###	   score.method="nlminb",
                                     score.method="fisher.scoring",
                                     theta.des=prtfam[,names],
                                     max.clust=1000,iid=1,
                                     Nit=60,marginal.p=prtfam$ps,se.clusters=prtfam$famclust)
            )
        if (time==1) print(udt)

        zfam <- rbind(c(1,0,0), ## m-f
                      c(0,1,0),  ## m-b1
                      c(0,1,0),  ## m-b2
                      c(1,0,0), ## f-m
                      c(0,1,0),  ## f-b1
                      c(0,1,0),  ## f-b2
                      c(0,1,0), ## b1-m
                      c(0,1,0), ## b1-f
                      c(0,0,1), ## b1-b2
                      c(0,1,0), ## b2-m
                      c(0,1,0), ## b2-f
                      c(0,0,1)) ## b2-b1


        if (alr==1) {
	    if (simplealr==0) {
###       cvec <- (ddl$num=="m"| ddl$num=="f")*1 +(ddl$num=="b1"| ddl$num=="b2")*2
###       k <- 2
###       dmat <- rbind(c(1,0,0),c(0,1,0),c(0,0,1))
###       udz <- class2z(cvec,ddl$id,k,dmat)
###
###    ZMAST <- rep(1,12)
###    ZMAST <- cbind(ZMAST,c(0,0,0,0,1,1,0,1,1,0,1,1))
###
###   outl <- alr(ddl$y~+1,id=ddl$id,depmodel="general",ainit=rep(0.01,3),z=udz$z,zmast=0)
                if (!require(alr)) stop("'alr' package required")
                out4t <-  system.time(
                    outl <- alr(ddl$y~+1,id=ddl$id,depmodel="general",zlocs=rep(1:4,n),ainit=rep(0.01,3),z=zfam,zmast=1)
                    )
                if (time==1) print(out4t)

                outl <- c(summary(outl)$alpha[,1],summary(outl)$alpha[,2])
                names(outl) <- c(rep("alr",3),rep("se-alr",3))
	    } else {
                outl <- alr(ddl$y~+1,id=ddl$id,depmodel="exchangeable",ainit=rep(0.01,1))
                outl <- c(summary(outl)$alpha[,1],summary(outl)$alpha[,2])
                names(outl) <- c(rep("alr",1),rep("se-alr",1))
	    }

        }

    } else { ### med design formula
        form <- ~factor(num1)*factor(num2)
        udbin <- easy.binomial.twostage(marg,data=ddl,
                                        response="y",id="id",theta.formula=form,
                                        marginal.p=ps,
                                        score.method="fisher.scoring")

    }

###   if (alr==1) { ### alr til simpelt design
###   outl <- alr(ddl$y~ddl$x,id=ddl$id,depm="exchangeable", ainit=0.01)
###   outl <- summary(outl)$alpha
###   ud <- c(udbin$theta,udbin$var.theta^.5,udbin$hessi^.5,c(outl)[1:2])
###   names(ud) <- c("TWO","se-two","se-twoR","alr","se-alr")
###   } 
    ud <- c(udf$theta,diag(udf$var.theta)^.5)
    if (alr==1)  ud <- c(ud,outl)
    return(ud)
} ## }}} 

onerunfam2 <- function(i,n,alr=0,manual=1,time=0,theta=1) { ## {{{ 
### n=1000; beta=0.2; theta=1; time=0; i=1
    print(i)
    dd <- simBinFam(n,beta=0,theta) 
    ddl <- fast.reshape(dd,varying="y",keep="y")

    desfs <- function(x,num1="num1",num2="num2")
        { ## {{{ 
            mf <- (x[num1]=="m")*(x[num2]=="f")*1
            mb <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
            bb <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
            c(mf,mb,bb)
        } ## }}} 

    ud <- easy.binomial.twostage(y~+1,data=ddl,
                                 response="y",id="id",
                                 score.method="fisher.scoring",deshelp=0,
                                 theta.formula=desfs,desnames=c("pp","pc","cc"))

    ud <- c(ud$theta[,1],diag(ud$var.theta)^.5)

    zfam <- rbind(c(1,0,0), ## m-f
	      	  c(0,1,0),  ## m-b1
	      	  c(0,1,0),  ## m-b2
                  c(1,0,0), ## f-m
	      	  c(0,1,0),  ## f-b1
	      	  c(0,1,0),  ## f-b2
	      	  c(0,1,0), ## b1-m
	      	  c(0,1,0), ## b1-f
	      	  c(0,0,1), ## b1-b2
	      	  c(0,1,0), ## b2-m
	      	  c(0,1,0), ## b2-f
	      	  c(0,0,1)) ## b2-b1

    if (alr==1) {
        if (!require(alr)) stop("'alr' package required")
        outl <- alr(ddl$y~+1,id=ddl$id,
                    depmodel="general",zlocs=rep(1:4,n),ainit=rep(0.01,3),z=zfam,zmast=1)
        outl <- c(summary(outl)$alpha[,1],summary(outl)$alpha[,2])
        names(outl) <- c(rep("alr",3),rep("se-alr",3))
    }

    if (alr==1)  ud <- c(ud,outl)
    return(ud)
} ## }}} 

##' Multinomial regression based on phreg regression
##'
##' Fits multinomial regression model 
##' \deqn{ P_i = \frac{ \exp( X^\beta_i ) }{ \sum_{j=1}^K \exp( X^\beta_j ) }} 
##' for \deqn{i=1,..,K}
##' where \deqn{\beta_1 = 0}, such that \deqn{\sum_j P_j = 1} using phreg function. 
##' Thefore the ratio \deqn{\frac{P_i}{P_1} = \exp( X^\beta_i )}
##'
##' Coefficients give log-Relative-Risk relative to baseline group (first level of factor, so that it can reset by relevel command).  
##' Standard errors computed based on sandwhich form \deqn{ DU^-1  \sum U_i^2 DU^-1}.  
##'
##' Can also get influence functions (possibly robust) via iid() function, response should be a factor. 
##'
##' Can fit cumulative odds model as a special case of interval.logitsurv.discrete
##'
##' @param formula formula with outcome (see \code{coxph})
##' @param data data frame
##' @param weights for score equations 
##' @param offset offsets for partial likelihood 
##' @param fix.X to have same coefficients for all categories
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' library(mets)
##' data(bmt)
##' bmt$id <- sample(200,408,replace=TRUE)
##' dfactor(bmt) <- cause1f~cause
##' drelevel(bmt,ref=3) <- cause3f~cause
##' dlevels(bmt)
##'
##' mreg <- mlogit(cause1f~+1+cluster(id),bmt)
##' summary(mreg)
##' head(iid(mreg))
##' dim(iid(mreg))
##'
##' mreg <- mlogit(cause1f~tcell+platelet,bmt)
##' summary(mreg)
##' head(iid(mreg))
##' dim(iid(mreg))
##' 
##' mreg3 <- mlogit(cause3f~tcell+platelet,bmt)
##' summary(mreg3)
##' 
##' ## inverse information standard errors 
##' lava::estimate(coef=mreg3$coef,vcov=mreg3$II)
##' 
##' ## predictions based on seen response or not 
##' ## all probabilities
##' head(predict(mreg,response=FALSE))
##' head(predict(mreg))
##' ## using newdata 
##' newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1),cause1f=c("2","2","0"))
##' ## only probability of seen response 
##' predict(mreg,newdata)
##' ## without response
##' predict(mreg,newdata,response=FALSE)
##' ## given indexx of P(Y=j)
##' predict(mreg,newdata,Y=c(1,2,3))
##' ##  reponse not given 
##' newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1))
##' predict(mreg,newdata)
##' @export
##' @aliases predict 
mlogit <- function(formula,data,offset=NULL,weights=NULL,fix.X=FALSE,...)
{# {{{
  cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,nrow(X)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,nrow(X)) 
  } else weights <- des.weights

   if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
   strata.name <- NULL

  res <- mlogit01(X,Y,id=id,strata=strata,offset=offset,weights=weights,strata.name=strata.name,
		  fix.X=fix.X,formula.call=formula,...) 
  res$design <- des
  return(res)
}# }}}

mlogit01 <- function(X,Y,id=NULL,strata=NULL,offset=NULL,weights=NULL,
       strata.name=NULL,cumhaz=FALSE,
       beta,stderr=TRUE,method="NR",no.opt=FALSE,Z=NULL,
       propodds=NULL,AddGam=NULL,case.weights=NULL,fix.X=FALSE,formula.call=NULL,
  X.call=NULL,Y.call=NULL,...) {# {{{
  p <- ncol(X)
  if (missing(beta)) beta <- rep(0,p)
  if (p==0) X <- cbind(rep(0,length(Y)))
  if (is.null(offset)) offset <- rep(0,length(Y)) 
  if (is.null(weights)) weights <- rep(1,length(Y)) 
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given 
  if (!is.null(Z)) Zcall <- Z
  if (is.null(case.weights)) case.weights <- rep(1,length(Y)) 

  call.id <- id
  if (is.null(id)) id <- 1:nrow(X)

  types <- unique(as.numeric(Y))
  nlev <- length(types)

  nX <- nrow(X)
  idrow <- rep(1:nX,each=nlev)
  X <- X[idrow,,drop=FALSE]
  px <- ncol(X)
  Y <- Y[idrow]
  id <- id[idrow]
  status <- rep(0,nrow(X))
  nY <- as.numeric(Y)

  refg <- 1  ### else refg <- match(ref,types)
  nrefs <- (1:nlev)[-refg]
  for (i in 1:nlev) status[nY==i] <- rep(((1:nlev)==i),sum(nY==i)/nlev)
  time <- id
  strat <- rep(1:nlev,nX)
  XX <- c()
  nn <- c()
  namesX <- colnames(X); 
  if (!fix.X) {
     for (i in nrefs) { XX <- cbind(XX,X*(strat==i)); 
                      nn<-c(nn, paste(namesX,i,sep=".")) 
     }  
     colnames(XX) <- nn; 
  } else {
     ### different intercepts
     for (i in nrefs) { XX <- cbind(XX,cbind(X[,1]*(strat==i))); 
                        nn<-c(nn, paste(namesX[1],i,sep=".")) 
     }  
     ### same covariate effects 
     XX<- cbind(XX,X[,-1]*(strat!=1))
     nn <- c(nn,colnames(X[,-1]))
     colnames(XX) <- nn; 
  }
  rownames(XX) <- NULL
  namesXX <- paste("names",1:ncol(XX),sep="")

  namesXXX <- gsub("\\)","", gsub("\\(","",nn))
  namesXXX <- gsub("\\.","", gsub("\\.","",namesXXX))
  namesXXX <- gsub("\\:","", gsub("\\:","",namesXXX))
  colnames(XX) <- namesXXX

  datph = cbind(
      data.frame(
          time = time, status = status, id = id, idrow = idrow
      ),
      XX
  )
  loffset <- offset[idrow]
  lweights<- weights[idrow]

  form <- "Surv(time,status)~strata(idrow)+cluster(id)"
  form <- paste(
    form, "+",
    paste(
     colnames(XX),
     collapse = "+")
  )


  res <- phreg(as.formula(form), datph, weights = lweights, offset = loffset, ...)
  res$formula <- formula.call
  res$px <- px
  res$nlev <- nlev
  class(res) <- rev(c("phreg","mlogit"))
  return(res)
}# }}}

##' @export
predict.mlogit <- function (object, newdata , se = TRUE, response=TRUE , Y=NULL,level=0.95,...)
{# {{{

  ylev <- levels(object$design$y)
  if (missing(newdata)) {
     X <- object$design$x
     ## take response from newdata if it is there and it is not given  
     if (response) Y <- as.numeric(object$design$y) 
  }  else {
     respindata <- length((grep(all.vars(object$formula)[1],names(newdata))))
     if (respindata>0 & response) x <- update_design(object$design,data = newdata,response=TRUE)
     else x <- update_design(object$design,data = newdata)
     X <- x$x
     ## take response from newdata if it is there and it is not given  
     if (is.null(Y))  { 
         if (response & !is.null(x$y))   
	   Y <- as.numeric(factor(x$y,levels=ylev))
     } else {
	     if (!is.numeric(Y)) 
		     Y <- as.numeric(factor(Y,levels=ylev))
     }
  }

  refg <- 1  ### else refg <- match(ref,types)
  nrefs <- (1:(object$nlev-1))
  px <- ncol(X)
  Xbeta <- c()
  for (i in nrefs) { Xbeta <- cbind(Xbeta,X %*% object$coef[(1:px)+px*(i-1)]);  }

  ppp <- cbind(1,exp(Xbeta))
  spp <- apply(ppp,1,sum)
  pp <- ppp/spp
  colnames(pp) <- ylev
  alpha <- 1-level
  crit <- -qnorm(1-alpha/2)

  if (!is.null(Y)) {
	  Yg2 <- 1*(Y>=2)
          pp <- p <- c(mdi(pp,1:length(Y),Y)) 
          pppy <- c(mdi(ppp,1:length(Y),Y)) 
     if (se) {
	     Dppy <-  (spp*Yg2-pppy) 
	     Dp <- c()
             for (i in nrefs) Dp <- cbind(Dp,X*ppp[,i+1]*Dppy/spp^2);  
             if (is.null(object$var)) covv <- vcov(object) else covv <- object$var
	     se <-  apply((Dp %*% covv) * Dp,1,sum)^.5
	     cmat <- data.frame(pred = p, se = se, lower = p - crit * se, upper = p + crit * se)
             names(cmat)[1:4] <- c("pred", "se", "lower", "upper")
             pp <- cmat
     }
 }

  return(pp)
}# }}}

##' @export
predict.biprobit <- function(object,newdata,X,Z,which=NULL,fun=NULL,type,...) {
    if (missing(newdata)) newdata <- data.frame(1)
    if (missing(X)) {
        ff <- object$formula; ff[2] <- NULL
        X <- model.matrix(ff,newdata)
    }
    if (missing(Z)) Z <- model.matrix(object$rho.formula,newdata)    
    p <- coef(object)

    h <- function(p) log(p/(1-p)) ## logit
    ih <- function(z) 1/(1+exp(-z)) ## expit
    if (!missing(type)) {
        h <- asin; ih <- sin      
        if (is.null(type)) {
            h <- ih <- identity
        }
        if (is.list(type)) {
            h <- type[[1]]; ih <- type[[2]]
        }
    }
        
    prob <- function(p,which=NULL,fun=NULL,...) {
        blen <- object$model$blen    
        beta1 <- p[seq(blen)]
        if (object$model$eqmarg) {
            beta2 <- beta1
        } else {
            beta2 <- beta1[-seq(blen/2)]
            beta1 <- beta1[seq(blen/2)]
        }    
        gamma <- p[-seq(blen)]
        m1 <- X%*%beta1
        m2 <- X%*%beta2
        r <- object$SigmaFun(gamma,cor=TRUE,Z=Z)
        pp <- data.frame(mu1=m1,mu2=m2,rho=r$rho)        
        p11 <- with(pp, pmvn(lower=c(0,0),upper=c(Inf,Inf),mu=cbind(mu1,mu2),sigma=cbind(rho),cor=TRUE))
        p10 <- with(pp, pmvn(lower=c(0,-Inf),upper=c(Inf,0),mu=cbind(mu1,mu2),sigma=cbind(rho),cor=TRUE))
        p01 <- with(pp, pmvn(lower=c(-Inf,0),upper=c(0,Inf),mu=cbind(mu1,mu2),sigma=cbind(rho),cor=TRUE))
        p00 <- with(pp, pmvn(lower=c(-Inf,-Inf),upper=c(0,0),mu=cbind(mu1,mu2),sigma=cbind(rho),cor=TRUE))
        p1 <- p11+p10
        p2 <- p11+p01
        res <- cbind(p11,p10,p01,p00,p1,p2,as.matrix(pp))
        if (!is.null(fun)) return(fun(res))
        if (!is.null(which)) {
            tr.idx <- base::which(which<7)
            res <- res[,which,drop=FALSE]
            if (length(tr.idx)>0) res[,tr.idx] <- h(res[,tr.idx])
            return(structure(res,tr.idx=tr.idx))
        }
        return(res)
    }
    pp <- prob(p,which=which,fun=fun)
    if (!is.null(fun)) {
        res <- estimate(object,prob,fun=fun)$coefmat
        if (!missing(newdata) && nrow(newdata)==nrow(res)) {
            suppressWarnings(res <- cbind(res,parameter=rownames(res),newdata))                   
        }
        return(res)
    }        
    if (!is.null(which)) {
        res <- estimate(object,prob,which=which)$coefmat[,c(1,3,4),drop=FALSE]
        tr.idx <- (attributes(pp)$tr.idx-1)*nrow(pp)
        if (length(tr.idx)>0) {
            for (ii in tr.idx) {
                idx <- ii+seq(nrow(pp))
                res[idx,] <- ih(res[idx,])
            }
        }
        rownames(res) <- rep(colnames(pp),each=nrow(pp))
        pp <- res
    }    
    if (!missing(newdata)) {
        suppressWarnings(pp <- cbind(pp,parameter=rownames(pp),newdata))
    }
    return(pp)
}

##' Fits Clayton-Oakes or bivariate Plackett (OR) models for binary data
##' using marginals that are on logistic form.
##' If clusters contain more than two times, the algoritm uses a compososite likelihood
##' based on all pairwise bivariate models.
##'
##' The pairwise pairwise odds ratio model provides an alternative to the alternating logistic
##' regression (ALR).
##'
##' The reported standard errors are based on a cluster corrected score equations from the
##' pairwise likelihoods assuming that the marginals are known. This gives correct standard errors
##' in the case of the Odds-Ratio model (Plackett distribution) for dependence, but incorrect standard
##' errors for the Clayton-Oakes types model (that is also called "gamma"-frailty). For the additive gamma version of the
##' standard errors
##' are adjusted for the uncertainty in the marginal models via an iid deomposition using the iid() function of
##' lava. For the clayton oakes model that is not speicifed via the random effects these can be
##' fixed subsequently using the iid influence functions for the marginal model, but typically this does not
##' change much.
##'
##' For the Clayton-Oakes version of the model, given the gamma distributed random effects it is
##' assumed that the probabilities are indpendent, and that the marginal survival functions are on logistic form
##' \deqn{
##' logit(P(Y=1|X)) = \alpha + x^T \beta
##' }
##' therefore conditional on the random effect the probability of the event is
##' \deqn{
##' logit(P(Y=1|X,Z)) = exp( -Z \cdot Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
##' }
##'
##' Can also fit a structured additive gamma random effects model, such
##' the ACE, ADE model for survival data:
##'
##' Now random.design specificies the random effects for each subject within a cluster. This is
##' a matrix of 1's and 0's with dimension n x d.  With d random effects.
##' For a cluster with two subjects, we let the random.design rows be
##'  \eqn{v_1} and \eqn{v_2}.
##' Such that the random effects for subject
##' 1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
##' has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
##' subjects 1's random effect are Gamma distributed with
##' mean \eqn{\lambda_j/v_1^T \lambda}
##' and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
##' \eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
##' It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed over all clusters
##' as it would be for the ACE model below.
##'
##' The DEFAULT parametrization uses the variances of the random effecs (var.par=1)
##' \deqn{
##' \theta_j  = \lambda_j/(v_1^T \lambda)^2
##' }
##'
##' For alternative parametrizations (var.par=0) one can specify how the parameters relate
##' to \eqn{\lambda_j} with the function
##'
##' Based on these parameters the relative contribution (the heritability, h) is
##' equivalent to  the expected values of the random effects  \eqn{\lambda_j/v_1^T \lambda}
##'
##' Given the random effects the probabilities  are independent and on the form
##' \deqn{
##' logit(P(Y=1|X)) = exp( - Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
##' }
##' with the inverse laplace of the gamma distribution with mean 1 and variance lamtot.
##'
##' The parameters \eqn{(\lambda_1,...,\lambda_d)}
##' are related to the parameters of the model
##' by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
##' \eqn{\lambda} parameters
##' with the (k) underlying \eqn{\theta} parameters
##' \deqn{
##' \lambda = theta.des  \theta
##' }
##' here using theta.des to specify these low-dimension association. Default is a diagonal matrix.
##'
##' @export
##' @aliases binomial.twostage binomial.twostage.time
##' @references
##' Two-stage binomial modelling
##' @examples
##' data(twinstut)
##' twinstut0 <- subset(twinstut, tvparnr<4000)
##' twinstut <- twinstut0
##' twinstut$binstut <- (twinstut$stutter=="yes")*1
##' theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
##' margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
##' bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
##'          clusters=twinstut$tvparnr,theta.des=theta.des,detail=0)
##' summary(bin)
##'
##' twinstut$cage <- scale(twinstut$age)
##' theta.des <- model.matrix( ~-1+factor(zyg)+cage,data=twinstut)
##' bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des)
##' summary(bina)
##'
##' theta.des <- model.matrix( ~-1+factor(zyg)+factor(zyg)*cage,data=twinstut)
##' bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des)
##' summary(bina)
##'
##' \donttest{ ## Reduce Ex.Timings
##' ## refers to zygosity of first subject in eash pair : zyg1
##' ## could also use zyg2 (since zyg2=zyg1 within twinpair's))
##' out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
##'                           response="binstut",id="tvparnr",var.link=1,
##' 	             	      theta.formula=~-1+factor(zyg1))
##' summary(out)
##'
##' ## refers to zygosity of first subject in eash pair : zyg1
##' ## could also use zyg2 (since zyg2=zyg1 within twinpair's))
##' desfs<-function(x,num1="zyg1",num2="zyg2")
##'     c(x[num1]=="dz",x[num1]=="mz",x[num1]=="os")*1
##'
##' out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
##'       data=twinstut,response="binstut",id="tvparnr",var.link=1,
##'       theta.formula=desfs,desnames=c("mz","dz","os"))
##' summary(out3)
##' }
##'
##' ### use of clayton oakes binomial additive gamma model
##' ###########################################################
##' \donttest{ ## Reduce Ex.Timings
##' data <- simbinClaytonOakes.family.ace(10000,2,1,beta=NULL,alpha=NULL)
##' margbin <- glm(ybin~x,data=data,family=binomial())
##' margbin
##'
##' head(data)
##' data$number <- c(1,2,3,4)
##' data$child <- 1*(data$number==3)
##'
##' ### make ace random effects design
##' out <- ace.family.design(data,member="type",id="cluster")
##' out$pardes
##' head(out$des.rv)
##'
##' bints <- binomial.twostage(margbin,data=data,
##'      clusters=data$cluster,detail=0,var.par=1,
##'      theta=c(2,1),var.link=0,
##'      random.design=out$des.rv,theta.des=out$pardes)
##' summary(bints)
##'
##' data <- simbinClaytonOakes.twin.ace(10000,2,1,beta=NULL,alpha=NULL)
##' out  <- twin.polygen.design(data,id="cluster",zygname="zygosity")
##' out$pardes
##' head(out$des.rv)
##' margbin <- glm(ybin~x,data=data,family=binomial())
##'
##' bintwin <- binomial.twostage(margbin,data=data,
##'      clusters=data$cluster,var.par=1,
##'      theta=c(2,1),random.design=out$des.rv,theta.des=out$pardes)
##' summary(bintwin)
##' concordanceTwinACE(bintwin)
##' }
##'
##' @keywords binomial regression
##' @author Thomas Scheike
##' @export
##' @param margbin Marginal binomial model
##' @param data data frame
##' @param method Scoring method "nr", for lava NR optimizer
##' @param detail Detail
##' @param clusters Cluster variable
##' @param silent Debug information
##' @param weights Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.
##' @param theta Starting values for variance components
##' @param theta.des design for dependence parameters, when pairs are given the indeces of the
##' theta-design for this pair, is given in pairs as column 5
##' @param var.link Link function for variance
##' @param var.par parametrization
##' @param var.func when alternative parametrizations are used this function can specify how the paramters are related to the \eqn{\lambda_j}'s.
##' @param iid Calculate i.i.d. decomposition when iid>=1, when iid=2 then avoids adding the uncertainty for marginal paramters for additive gamma model (default).
##' @param notaylor Taylor expansion
##' @param model model
##' @param marginal.p vector of marginal probabilities
##' @param beta.iid iid decomposition of marginal probability  estimates for each subject, if based on GLM model this is computed.
##' @param Dbeta.iid derivatives of marginal model wrt marginal parameters, if based on GLM model this is computed.
##' @param strata strata for fitting: considers only pairs where both are from same strata
##' @param max.clust max clusters
##' @param se.clusters clusters for iid decomposition for roubst standard errors
##' @param numDeriv uses Fisher scoring aprox of second derivative if 0, otherwise numerical derivatives
##' @param random.design random effect design for additive gamma model, when pairs are given the
##' indeces of the pairs random.design rows are given as columns 3:4
##' @param pairs matrix with rows of indeces (two-columns) for the pairs considered in the pairwise composite score, useful for case-control sampling when marginal is known.
##' @param dim.theta dimension of theta when pairs and pairs specific design is given. That is when pairs has 6 columns.
##' @param additive.gamma.sum this is specification of the lamtot in the models via a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number of theta parameters), when null then sums all parameters. Default is a matrix of 1's
##' @param pair.ascertained if pairs are sampled only when there are events in the pair i.e. Y1+Y2>=1.
##' @param case.control if data is case control data for pair call, and here 2nd column of pairs are probands (cases or controls)
##' @param twostage default twostage=1, to fit MLE use twostage=0
##' @param beta is starting value for beta for MLE version
##' @param no.opt for not optimizing 
##' @param ... for NR of lava 
binomial.twostage <- function(margbin,data=parent.frame(),
     method="nr",detail=0,clusters=NULL,silent=1,weights=NULL,
     theta=NULL,theta.des=NULL,var.link=0,var.par=1,var.func=NULL,
     iid=1,notaylor=1,model="plackett",marginal.p=NULL,beta.iid=NULL,Dbeta.iid=NULL,
     strata=NULL,max.clust=NULL,se.clusters=NULL,numDeriv=0,
     random.design=NULL,pairs=NULL,dim.theta=NULL,additive.gamma.sum=NULL,
     pair.ascertained=0,case.control=0,no.opt=FALSE,twostage=1,beta=NULL,...)
{ ## {{{
    ## {{{ seting up design and variables
    rate.sim <- 1; sym=1;
    if (model=="clayton.oakes"  || model=="gamma") dep.model <- 1 else if (model=="plackett" || model=="or") dep.model <- 2 else stop("Model must by either clayton.oakes or plackett \n");
    antpers <- NROW(data);
    if (!is.null(pairs)) nn <- NROW(pairs) else nn <- 1

### marginal prediction and binomial response, two types of calls ## {{{
    if (inherits(margbin,"glm")) {
        ps <- predict(margbin,newdata=data,type="response")
        if (margbin$family$family!="binomial") warning("not binomial family\n");
        ### takes data to extract response and predictions, these could be different for pairs call
        cause <- data[,all.vars(margbin$formula)[1]]

	if (!is.numeric(cause)) stop(paste("response in data",margbin$formula)[1],"not numeric\n");
	if (is.null(beta.iid))   beta.iid <- lava::iid(margbin,id=clusters)
	if (is.null(Dbeta.iid)) Dbeta.iid <- model.matrix(margbin$formula,data=data) * ps
	if (twostage==0)            Xbeta <- model.matrix(margbin$formula,data=data)
    }
    else if (inherits(margbin,"formula")) {
        margbin <- glm(margbin,data=data,family=binomial())
        ps <- predict(margbin,type="response")
        cause <- margbin$y
	if (twostage==0)            Xbeta <- model.matrix(margbin$formula,data=data)
	if (is.null(Dbeta.iid)) Dbeta.iid <- model.matrix(margbin$formula,data=data) * ps
	if (is.null(beta.iid))   beta.iid <- iid(margbin,id=clusters)
    }  else if (is.null(marginal.p))
        stop("without marginal model, marginal p's must be given\n");

    if (!is.null(marginal.p)) {
        if (length(margbin)!=antpers)
            stop("with marginal margbin is response \n")
        else cause <- margbin
        if (length(marginal.p)!=antpers)
            stop("length same as data dimension  \n")
        else ps <- marginal.p
    }
    ## }}}

    notaylor <- 1
    if (is.null(weights)==TRUE) weights <- rep(1,antpers);
    if (is.null(strata)==TRUE) strata<- rep(1,antpers);
    if (length(strata)!=antpers) stop("Strata must have length equal to number of data points \n");

# {{{ cluster setup
    out.clust <- cluster.index(clusters);
    clusters <- out.clust$clusters
    maxclust <- out.clust$maxclust
    antclust <- out.clust$cluster.size
    clusterindex <- out.clust$idclust
    clustsize <- out.clust$cluster.size
    call.secluster <- se.clusters

    if (is.null(se.clusters)) { se.clusters <- clusters; antiid <- nrow(clusterindex);} else  {
	iids <-  unique(se.clusters);
	antiid <- length(iids);
	if (is.numeric(se.clusters)) se.clusters <-  fast.approx(iids,se.clusters)-1
	 else se.clusters <- as.integer(factor(se.clusters, labels = seq(antiid)))-1
        }
    if (length(se.clusters)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

    if ((!is.null(max.clust))) if (max.clust< antiid) {
        coarse.clust <- TRUE
	qq <- unique(quantile(se.clusters, probs = seq(0, 1, by = 1/max.clust)))
	qqc <- cut(se.clusters, breaks = qq, include.lowest = TRUE)
	se.clusters <- as.integer(qqc)-1
	max.clusters <- length(unique(se.clusters))
	maxclust <- max.clust
	antiid <- max.clusters
    }
# }}}

    if (is.null(beta)==TRUE & twostage==0) beta <- coef(margbin) else beta <- 0
    ## }}}
  dimbeta <- length(beta);

  ### setting design for random variables, in particular with pairs are given
  ddd <- randomDes(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,var.link,clusterindex,dim.theta)

  random.design=ddd$random.design;clusterindex=ddd$clusterindex;
  antpairs=ddd$antpairs;
  theta=ddd$theta;ptheta=ddd$ptheta;theta.des=ddd$theta.des
  pair.structure=ddd$pair.structure; dep.model=ddd$dep.model
  dim.rv <- ddd$dim.rv; additive.gamma.sum=ddd$additive.gamma.sum

  if (dep.model==3) model <- "clayton.oakes"

if (pair.structure==1) {
    if (length(case.control)!=antpairs)         case.control <- rep(case.control[1],antpairs)
    if (length(pair.ascertained)!=antpairs) pair.ascertained <- rep(pair.ascertained[1],antpairs)
    if (any(case.control+pair.ascertained==2))  stop("Each pair is either case.control pair or pair.ascertained \n");
    }

    if (is.null(Dbeta.iid)) Dbeta.iid <- matrix(0,length(cause),1);
    ptrunc <- rep(1,antpers);

    theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);

    obj <- function(par)
    { ## {{{

      if (pair.structure==0 | dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);
      if (pair.structure==1 & dep.model==3) Xtheta <- matrix(0,antpers,1); ## not needed
      if (pair.structure==1 & dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par[seq(1,ptheta)]),nrow=ptheta,ncol=1);

      if (twostage==0) epar <- par[seq(1,ptheta)] else epar <- par
      if (twostage==0) { ### update, marginal.p og score for logistic model
	    beta <- par[seq(ptheta+1,ptheta+dimbeta)]
	    lp <- c(Xbeta %*%  beta)
            psu <- exp(lp)/(1+exp(lp))
            dpsu <- psu/(1+exp(lp))
	    Dbeta.iid <- Xbeta * dpsu
	    ps <- psu
      }

      if (var.link==1 & dep.model==3) epar <- c(exp(epar)) else epar <- c(epar)
      partheta <- epar

      if (var.par==1 & dep.model==3) {
       ## from variances to
       if (is.null(var.func)) {
	    sp <- sum(epar)
	    partheta <- epar/sp^2
         } else partheta <- epar; ## par.func(epar)
      }

      if (pair.structure==0)
            outl<-.Call("twostageloglikebin", ## {{{
            icause=cause,ipmargsurv=ps, itheta=c(partheta), ithetades=theta.des, icluster=clusters,
	    iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,iiid=iid,iweights=weights,
	    isilent=silent,idepmodel=dep.model, itrunkp=ptrunc,istrata=strata,iseclusters=se.clusters,
	    iantiid=antiid, irvdes=random.design,iags=additive.gamma.sum,ibetaiid=Dbeta.iid,pa=pair.ascertained,
	    twostage=twostage, PACKAGE="mets")
            ## }}}
      else outl<-.Call("twostageloglikebinpairs", ## {{{
            icause=cause,ipmargsurv=ps, itheta=c(partheta), ithetades=theta.des,
            icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,
	    iiid=iid,iweights=weights,isilent=silent,idepmodel=dep.model,
            itrunkp=ptrunc,istrata=strata,iseclusters=se.clusters,iantiid=antiid,
            irvdes=random.design,
            iags=additive.gamma.sum,ibetaiid=Dbeta.iid,pa=pair.ascertained,twostage=twostage,
	    icasecontrol=case.control,
            PACKAGE="mets")
             ## }}}

      if (detail==3) print(c(par,outl$loglike))

         ## variance parametrization, and inverse.link
	 if (dep.model==3) {# {{{
	    if (var.par==1) {
		 ## from variances to and with sum for all random effects
		 if (is.null(var.func)) {
		 if (var.link==0)  {
	###		 print(c(sp,epar))
		     mm <- matrix(-epar*2*sp,length(epar),length(epar))
		     diag(mm) <- sp^2-epar*2*sp
		 } else {
		    mm <- -c(epar) %o% c(epar)*2*sp
		    diag(mm) <- epar*sp^2-epar^2*2*sp
		 }
		    mm <- mm/sp^4
		 } else mm  <- numDeriv::hessian(var.func,par)
	      } else {
	          if (var.link==0) mm <- diag(length(epar)) else
			  mm <- diag(length(c(epar)))*c(epar)
	      }
              if (twostage==0) {  ### beta for logistic regression also part of model
		      mm0 <- diag(length(par))
		      mm0[1:nrow(mm),1:nrow(mm)] <- mm
		      mm <- mm0
	      }

	      }# }}}


	    if (dep.model==3) {# {{{
	       outl$score <-  t(mm) %*% outl$score
	       if (twostage==1) outl$DbetaDtheta <-  t(mm) %*% outl$DbetaDtheta
	       outl$Dscore <- t(mm) %*% outl$Dscore %*% mm
               if (iid==1) outl$theta.iid <- t(t(mm) %*% t(outl$theta.iid))
	    }# }}}

            attr(outl,"grad") <- attr(outl,"gradient") <-outl$score
	    attr(outl,"hessian") <- outl$Dscore
            outl$gradient <- outl$score 
            outl$hessian <- -outl$Dscore

###            if (oout==0) ret <- c(-1*outl$loglike) else if (oout==1) ret <- sum(outl$score^2) else if (oout==3) ret <- outl$score else ret <- outl

    if (oout==0) ret <- with(outl,structure(-outl$loglike,gradient=-gradient,hessian=-hessian))
    else if (oout==1) ret <- outl$gradient 
    else if (oout==2) ret <- outl

    return(ret)
    } ## }}}

  theta.iid <- NULL
  logl <- NULL
  p <- theta
  if (twostage==0) p <- c(p,beta);
  theta <- p
    
  oout <- 0
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(p,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  ## names(cc) <- colnames(X)
      oout <- 2
      val <- c(list(coef=cc),obj(opt$estimate))
 } else val <- c(list(coef=beta),obj(p))
 theta <- matrix(val$coef,ncol=1)

    if (numDeriv>=1) {
         oout <- 1
         if (detail==1 ) cat("starting numDeriv for second derivative \n");
         val$hessian <- numDeriv::jacobian(obj,p,method="simple")
         if (detail==1 ) cat("finished numDeriv for second derivative \n");
        }

  hess <- val$hessian
  if (!is.na(sum(hess))) hessi <- lava::Inverse(val$hessian) else hessi <- diag(nrow(val$hessian))


    ## {{{ handling output
    iid.tot <- NULL
    var.tot <- robvar.theta <- NULL
    beta <- NULL
    if (iid>=1) {
        theta.iid <- val$theta.iid %*% hessi
        if (dep.model==3 & iid!=2  & (!is.null(beta.iid)))
	if (nrow(beta.iid)==nrow(val$theta.iid) & twostage==1) {
	     theta.beta.iid <- (beta.iid %*% t(val$DbetaDtheta) ) %*% hessi
	     theta.iid  <- theta.iid+theta.beta.iid
	     iid.tot <- cbind(theta.iid,beta.iid)
	     var.tot <- crossprod(iid.tot)
	}
        var.theta <- robvar.theta  <- (t(theta.iid) %*% theta.iid)
	if (is.null(var.tot)) var.tot <- var.theta
    } else var.theta <- -1* hessi

  if (inherits(margbin,"glm")) beta <- coef(margbin);
  if (twostage==0) beta <- theta[seq(ptheta,ptheta+dimbeta)]

  if (iid==1) var.theta <- robvar.theta else var.theta <- -hessi
  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(theta),sep="")
   if (length(thetanames)==nrow(theta)) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }

    ud <- list(coef=val$coef,theta=theta,score=val$gradient,hess=hess,hessi=hessi,
	       var.theta=var.theta,model=model,robvar.theta=robvar.theta,
    theta.iid=theta.iid,thetanames=thetanames,loglike=val$loglike,Dscore=val$Dscore,
    margsurv=ps,iid.tot=iid.tot,var.tot=var.tot,beta=beta);
    class(ud)<-"mets.twostage"
    attr(ud, "binomial") <- TRUE
    attr(ud, "ptheta") <- ptheta
    attr(ud, "Formula") <- formula
    attr(ud, "Clusters") <- clusters
    attr(ud,"sym")<-sym;
    attr(ud,"var.link")<-var.link;
    attr(ud,"var.par")<-var.par;
    attr(ud,"var.func")<-var.func;
    attr(ud,"antpers")<-antpers;
    attr(ud,"antclust")<-antclust;
    attr(ud, "Type") <- model
    attr(ud,"DbetaDtheta")<-val$DbetaDtheta;
    attr(ud,"ags")<- additive.gamma.sum
    attr(ud,"twostage")<- twostage
    attr(ud,"pair.ascertained")<- pair.ascertained
    ### to be consistent with structure for survival twostage model
    attr(ud, "additive-gamma") <- (dep.model==3)*1
    attr(ud, "likepairs") <- c(val$likepairs)
    if (dep.model==3 & pair.structure==0) {
       attr(ud, "pardes") <- theta.des
       attr(ud, "theta.des") <- theta.des
       attr(ud, "rv1") <-    random.design[1,]
    }
    if (dep.model==3 & pair.structure==1) {
       nrv <- clusterindex[1,6]
       attr(ud, "theta.des") <- matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "pardes") <-    matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "rv1") <-    random.design[1,]
       attr(ud, "nrv") <- clusterindex[1,6]
    }

    attr(ud, "response") <- "binomial"
    return(ud);
    ## }}}
} ## }}}


##' @export
p11.binomial.twostage.RV <- function(theta,rv1,rv2,p1,p2,pardes,ags=NULL,link=0,i=1,j=1) { ## {{{
	## computes p11 pij for additive gamma binary random effects model
     if (is.null(ags)) ags <- matrix(1,dim(pardes))
     out <- .Call("claytonoakesbinRV",theta,i,j,p1,p2,rv1,rv2,pardes,ags,link,PACKAGE="mets")$like
 return(out)
} ## }}}

##' @export
concordanceTwostage<- function(theta,p,rv1,rv2,theta.des,additive.gamma.sum=NULL,link=0,var.par=0,...)
{# {{{

   ### takes dependence paramter from output
      ptheta <- length(theta)

   if (var.par==1) theta <- theta/sum(theta)^2

   nn <- nrow(as.matrix(rv1))
   if (is.matrix(p)==FALSE) { ll <- length(p); p <- matrix(p,ll,2); }
   if (ncol(p)!=2) p <- matrix(p,ncol=2)
   if (is.matrix(rv1)==FALSE) rv1 <- matrix(rv1,nn,length(rv1),byrow=TRUE)
   if (is.matrix(rv2)==FALSE) rv2 <- matrix(rv2,nn,length(rv1),byrow=TRUE)

   if (is.null(additive.gamma.sum)) ags <- matrix(1,ncol(rv1),ptheta) else ags <- additive.gamma.sum


   tabs <- list()
   for (i in 1:nn)
   {
        p1 <- p[i,1]
        p2 <- p[i,2]
	p11 <- p11.binomial.twostage.RV(theta,rv1[i,],rv2[i,],p1,p2,theta.des,ags=ags,link=link)
	p01 <- p[i,1]-p11
	p10 <- p[i,2]-p11
	p00 <- 1-p01-p10+p11
	tabs[[i]] <- list(pmat=matrix(c(p00,p10,p01,p11),2,2),casewise=c(p11/p1,p11/p2),marg=c(p1,p2))
   }

   return(tabs)
}	# }}}

##' @export
concordanceTwinACE<- function(object,rv1=NULL,rv2=NULL,xmarg=NULL,type="ace",...)
{# {{{

if (type=="ace" | type=="ade") {
if (is.null(rv1))  rv1 <- rbind(c(1,0,0,0,1),c(0,1,1,0,1))
if (is.null(rv2))  rv2 <- rbind(c(1,0,0,0,1),c(0,1,0,1,1))
}
if (type=="ae" | type=="de") {
if (is.null(rv1))  rv1 <- rbind(c(1,0,0,0),c(0,1,1,0))
if (is.null(rv2))  rv2 <- rbind(c(1,0,0,0),c(0,1,0,1))
}

var.par <- attr(object,"var.par")
var.link <- attr(object,"var.link")
theta.des <- attr(object,"theta.des")
twostage <- attr(object,"twostage")

if (twostage==0) {
	pt <- attr(object,"ptheta")
	theta <- object$theta[seq(1,pt)]
        beta <- object$theta[seq(pt+1,length(object$theta))]
	var.theta <- object$robvar.theta[seq(1,pt),seq(1,pt)]
	var.tot <- object$var.theta
} else {
	theta <- object$theta;
	beta <- object$beta
	var.theta <- object$var.theta;
	var.tot <- object$var.tot
}

if (var.link==1) theta <- exp(theta)
###print(beta)
ags <- attr(object,"ags");
if (is.null(xmarg)) {xmarg <- rep(0,length(beta)); xmarg[1] <- 1;}

nn <- nrow(rv1)
if (is.matrix(rv1)==FALSE) rv1 <- matrix(rv1,nn,length(rv1),byrow=TRUE)
if (is.matrix(rv2)==FALSE) rv2 <- matrix(rv2,nn,length(rv1),byrow=TRUE)
if (is.null(ags)) ags <- matrix(1,ncol(rv1),length(theta));

   fp <- function(par) {# {{{
       pp <- par[1:length(theta)];
       beta <- par[(length(theta)+1):length(par)];
       xp  <- sum(xmarg*beta);
       pm <- exp(xp)/(1+exp(xp));
       if (var.par==1) pp <- pp/sum(pp)^2
       p11 <- p11.binomial.twostage.RV(pp,rv1l,rv2l,pm,pm,theta.des,ags=ags,link=0)
       casewise <- p11/pm
       ret <- c(p11,casewise,pm)
       names(ret) <- c("concordance","casewise concordance","marginal")
       return(ret)
   }# }}}


   nnn<-c("MZ","DZ")
   tabs <- list()
   for (i in 1:nn)
   {
      rv1l <- rv1[i,]
      rv2l <- rv2[i,]
      tabs<-c(tabs,setNames(list(tabs[[i]] <- lava::estimate(coef=c(theta,beta),vcov=var.tot,f=function(p) fp(p))), nnn[i]))
   }

   return(tabs)
}	# }}}

##' @export
binomial.twostage.time <- function(formula,data,id,...,silent=1,fix.censweights=1,
breaks=Inf,pairsonly=TRUE,fix.marg=NULL,cens.formula,cens.model="aalen",weights="w") {
   ## {{{
    m <- match.call(expand.dots = FALSE)
    m <- m[match(c("","data"),names(m),nomatch = 0)]
    Terms <- terms(cens.formula,data=data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    M <- eval(m,envir=parent.frame())

    censtime <- model.extract(M, "response")
    status <- censtime[,2]
    time <- censtime[,1]
    timevar <- colnames(censtime)[1]
    outcome <- as.character(terms(formula)[[2]])
    if (is.null(breaks)) breaks <-  quantile(time,c(0.25,0.5,0.75,1))

    outcome0 <- paste(outcome,"_dummy")
    res <- list()
    logor <- cif <- conc <- c()
    k <- 0
    for (tau in rev(breaks)) {
        if ((length(breaks)>1) & (silent==0)) message(tau)
        ### construct min(T_i,tau) or T_i and related censoring variable,
        ### thus G_c(min(T_i,tau)) or G_c(T_i) as weights
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) {
        data0 <- data
        time0 <- time
        status0 <- status
        }
        cond0 <- (time>tau)
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) status0[cond0 & status==1] <- 3
        if (fix.censweights==0) status0[cond0 & status==1] <- 3
        data0[,outcome] <- data[,outcome]
        data0[cond0,outcome] <- FALSE
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) time0[cond0] <- tau
        if ((fix.censweights==1 & k==0) | (fix.censweights==0)) {
		data0$S <- survival::Surv(time0,status0==1)
	}
	if ((fix.censweights==1 & k==0) | (fix.censweights==0))
        dataw <- ipw(update(cens.formula,S~.),data=data0,cens.model=cens.model,
		     obsonly=TRUE)
        if ((fix.censweights==1))
		dataw[,outcome] <- (dataw[,outcome])*(dataw[,timevar]<tau)
	marg.bin <- glm(formula,data=dataw,weights=1/dataw[,weights],family="quasibinomial")
        pudz <- predict(marg.bin,newdata=dataw,type="response")
	dataw$pudz <- pudz
	datawdob <- fast.reshape(dataw,id=id)
        datawdob$minw <- pmin(datawdob$w1,datawdob$w2)
        dataw2 <- fast.reshape(datawdob)
	### removes second row of singletons
	dataw2  <- subset(dataw2,!is.na(dataw2$minw))
	k <- k+1
	if (!is.null(fix.marg)) dataw2$pudz <- fix.marg[k]
        suppressWarnings( b <- binomial.twostage(dataw2[,outcome],data=dataw2,clusters=dataw2[,id],marginal.p=dataw2$pudz,weights=1/dataw2$minw,...))
        theta0 <- b$theta[1,1]
        prev <- prev0 <- exp(coef(marg.bin)[1])/(1+exp(coef(marg.bin)[1]))
	if (!is.null(fix.marg)) prev <- fix.marg[k]
        concordance <- plack.cif2(prev,prev,theta0)
	conc <- c(conc,concordance)
	cif <- c(cif,prev0)
	logor <- rbind(logor,coef(b))
    }
    res <- list(varname="Time",var=breaks,concordance=rev(conc),cif=rev(cif),
		time=breaks,call=m,type="time",logor=logor[k:1,])
    return(res)
} ## }}}


##' Fits two-stage binomial for describing depdendence in binomial data
##' using marginals that are on logistic form using the binomial.twostage funcion, but
##' call is different and easier and the data manipulation is build into the function.
##' Useful in particular for family design data.
##'
##' If clusters contain more than two times, the algoritm uses a compososite likelihood
##' based on the pairwise bivariate models.
##'
##' The reported standard errors are based on the estimated information from the
##' likelihood assuming that the marginals are known. This gives correct standard errors
##' in the case of the plackett distribution (OR model for dependence), but incorrect for
##' the clayton-oakes types model. The OR model is often known as the ALR model.
##' Our fitting procedures gives correct standard errors due to the ortogonality and is
##' fast.
##'
##' @examples
##' data(twinstut)
##' twinstut0 <- subset(twinstut, tvparnr<4000)
##' twinstut <- twinstut0
##' twinstut$binstut <- (twinstut$stutter=="yes")*1
##' theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
##' margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
##' bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des,detail=0,
##' 	                 method="nr")
##' summary(bin)
##' lava::estimate(coef=bin$theta,vcov=bin$var.theta,f=function(p) exp(p))
##'
##' twinstut$cage <- scale(twinstut$age)
##' theta.des <- model.matrix( ~-1+factor(zyg)+cage,data=twinstut)
##' bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des,detail=0)
##' summary(bina)
##'
##' theta.des <- model.matrix( ~-1+factor(zyg)+factor(zyg)*cage,data=twinstut)
##' bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
##' 		         clusters=twinstut$tvparnr,theta.des=theta.des)
##' summary(bina)
##'
##' out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
##'                               response="binstut",id="tvparnr",var.link=1,
##' 			          theta.formula=~-1+factor(zyg1))
##' summary(out)
##'
##' ## refers to zygosity of first subject in eash pair : zyg1
##' ## could also use zyg2 (since zyg2=zyg1 within twinpair's))
##' ## do not run t save time
##' # desfs <- function(x,num1="zyg1",namesdes=c("mz","dz","os"))
##' #     c(x[num1]=="mz",x[num1]=="dz",x[num1]=="os")*1
##' #
##' #out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
##' #                               data=twinstut, response="binstut",id="tvparnr",
##' #                               var.link=1,theta.formula=desfs,
##' #                               desnames=c("mz","dz","os"))
##' #summary(out3)
##'
##' \donttest{ ## Reduce Ex.Timings
##' n <- 1000
##' set.seed(100)
##' dd <- simBinFam(n,beta=0.3)
##' binfam <- fast.reshape(dd,varying=c("age","x","y"))
##' ## mother, father, children  (ordered)
##' head(binfam)
##'
##' ########### ########### ########### ########### ########### ###########
##' ####  simple analyses of binomial family data
##' ########### ########### ########### ########### ########### ###########
##' desfs <- function(x,num1="num1",num2="num2")
##' {
##'      pp <- 1*(((x[num1]=="m")*(x[num2]=="f"))|(x[num1]=="f")*(x[num2]=="m"))
##'      pc <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
##'      cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
##'      c(pp,pc,cc)
##' }
##'
##' ud <- easy.binomial.twostage(y~+1,data=binfam,
##'      response="y",id="id",
##'      theta.formula=desfs,desnames=c("pp","pc","cc"))
##' summary(ud)
##'
##' udx <- easy.binomial.twostage(y~+x,data=binfam,
##'      response="y",id="id",
##'      theta.formula=desfs,desnames=c("pp","pc","cc"))
##' summary(udx)
##'
##' ########### ########### ########### ########### ########### ###########
##' ####  now allowing parent child POR to be different for mother and father
##' ########### ########### ########### ########### ########### ###########
##'
##' desfsi <- function(x,num1="num1",num2="num2")
##' {
##'     pp <- (x[num1]=="m")*(x[num2]=="f")*1
##'     mc <- (x[num1]=="m")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     fc <- (x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     c(pp,mc,fc,cc)
##' }
##'
##' udi <- easy.binomial.twostage(y~+1,data=binfam,
##'      response="y",id="id",
##'      theta.formula=desfsi,desnames=c("pp","mother-child","father-child","cc"))
##' summary(udi)
##'
##' ##now looking to see if interactions with age or age influences marginal models
##' ##converting factors to numeric to make all involved covariates numeric
##' ##to use desfai2 rather then desfai that works on binfam
##'
##' nbinfam <- binfam
##' nbinfam$num <- as.numeric(binfam$num)
##' head(nbinfam)
##'
##' desfsai <- function(x,num1="num1",num2="num2")
##' {
##'     pp <- (x[num1]=="m")*(x[num2]=="f")*1
##' ### av age for pp=1 i.e parent pairs
##'     agepp <- ((as.numeric(x["age1"])+as.numeric(x["age2"]))/2-30)*pp
##'     mc <- (x[num1]=="m")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     fc <- (x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
##'     agecc <- ((as.numeric(x["age1"])+as.numeric(x["age2"]))/2-12)*cc
##'     c(pp,agepp,mc,fc,cc,agecc)
##' }
##'
##' desfsai2 <- function(x,num1="num1",num2="num2")
##' {
##'     pp <- (x[num1]==1)*(x[num2]==2)*1
##'     agepp <- (((x["age1"]+x["age2"]))/2-30)*pp ### av age for pp=1 i.e parent pairs
##'     mc <- (x[num1]==1)*(x[num2]==3 | x[num2]==4)*1
##'     fc <- (x[num1]==2)*(x[num2]==3 | x[num2]==4)*1
##'     cc <- (x[num1]==3)*(x[num2]==3 | x[num2]==4)*1
##'     agecc <- ((x["age1"]+x["age2"])/2-12)*cc ### av age for children
##'     c(pp,agepp,mc,fc,cc,agecc)
##' }
##'
##' udxai2 <- easy.binomial.twostage(y~+x+age,data=binfam,
##'      response="y",id="id",
##'      theta.formula=desfsai,
##'      desnames=c("pp","pp-age","mother-child","father-child","cc","cc-age"))
##' summary(udxai2)
##' }
##' @keywords binomial regression
##' @export
##' @param margbin Marginal binomial model
##' @param data data frame
##' @param response name of response variable in data frame
##' @param id name of cluster variable in data frame
##' @param method Scoring method
##' @param Nit Number of iterations
##' @param detail Detail for more output for iterations
##' @param silent Debug information
##' @param weights Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.
##' @param control Optimization arguments
##' @param theta Starting values for variance components
##' @param theta.formula design for depedence, either formula or design function
##' @param desnames names for dependence parameters
##' @param deshelp if 1 then prints out some data sets that are used, on on which the design function operates
##' @param var.link Link function for variance
##' @param iid Calculate i.i.d. decomposition
##' @param step Step size
##' @param model model
##' @param marginal.p vector of marginal probabilities
##' @param strata strata for fitting
##' @param max.clust max clusters used for i.i.d. decompostion
##' @param se.clusters clusters for iid decomposition for roubst standard errors
easy.binomial.twostage <- function(margbin=NULL,data=parent.frame(),method="nr",
                                   response="response",id="id",
                                   Nit=60,detail=0, silent=1,weights=NULL,control=list(),
                                   theta=NULL,theta.formula=NULL,desnames=NULL,deshelp=0,var.link=1,iid=1,
                                   step=1.0,model="plackett",marginal.p=NULL,
				   strata=NULL,max.clust=NULL,se.clusters=NULL)
{ ## {{{
   if (inherits(margbin,"glm")) ps <- predict(margbin,type="response")
   else if (inherits(margbin,"formula")) {
       margbin <- glm(margbin,data=data,family=binomial())
       ps <- predict(margbin,type="response")
   }  else if (is.null(marginal.p))
	   stop("without marginal model, marginal p's must be given\n");

    if (!is.null(marginal.p)) ps <- marginal.p

    data <- cbind(data,ps)

### make all pairs in the families,
    fam <- familycluster.index(data[,id])
    data.fam <- data[c(fam$familypairindex),]
    data.fam$subfam <- c(fam$subfamilyindex)

### make dependency design using wide format for all pairs
    data.fam.clust <- fast.reshape(data.fam,id="subfam")
    if (is.function(theta.formula)) {
        desfunction <- compiler::cmpfun(theta.formula)
	if (deshelp==1){
            cat("These names appear in wide version of pairs for dependence \n")
            cat("design function must be defined in terms of these: \n")
            cat(names(data.fam.clust)); cat("\n")
            cat("Here is head of wide version with pairs\n")
            print(head(data.fam.clust)); cat("\n")
	}
###	des.theta <- Reduce("rbind",lapply(seq(nrow(data.fam.clust)),function(i) unlist(desfunction(data.fam.clust[i,] ))))
        des.theta <- t(apply(data.fam.clust,1, function(x) desfunction(x)))
	colnames(des.theta) <- desnames
	desnames <- desnames
    } else {
        if (is.null(theta.formula)) theta.formula <- ~+1
        des.theta <- model.matrix(theta.formula,data=data.fam.clust)
        desnames <- colnames(des.theta);
    }
    data.fam.clust <- cbind(data.fam.clust,des.theta)
    if (deshelp==1) {
        cat("These names appear in wide version of pairs for dependence \n")
        print(head(data.fam.clust))
    }

### back to long format keeping only needed variables
    data.fam <- fast.reshape(data.fam.clust,varying=c(response,id,"ps"))
    if (deshelp==1) {
	cat("Back to long format for binomial.twostage (head)\n");
        print(head(data.fam));
	cat("\n")
	cat(paste("binomial.twostage, called with reponse",response,"\n"));
	cat(paste("cluster=",id,",  subcluster (pairs)=subfam \n"));
	cat(paste("design variables ="));
	cat(desnames)
	cat("\n  theta design (head) \n")
        print(head(data.fam[,desnames]));
	cat("\n  response (head) \n")
        print(head(data.fam[,response]));
	cat("\n")
    }

    out <- binomial.twostage(data.fam[,response],data=data.fam,
                             clusters=data.fam$subfam,
                             theta.des=data.fam[,desnames],
                             detail=detail, method=method, Nit=Nit,step=step,
                             iid=iid,theta=theta,var.link=var.link,model=model,
                             max.clust=max.clust,
                             marginal.p=data.fam[,"ps"],se.clusters=data.fam[,id])

    return(out)
} ## }}}

##' @export
simBinPlack <- function(n,beta=0.3,theta=1,...) { ## {{{
    x1 <- rbinom(n,1,0.5)
    x2 <- rbinom(n,1,0.5)
###
    p1 <- exp(0.5+x1*beta)
    p2 <- exp(0.5+x2*beta)
    p1 <- p1/(1+p1)
    p2 <- p2/(1+p2)
###
    p11 <- plack.cif2(p1,p2,theta)
    p10 <- p1-p11
    p01 <- p2-p11
    p00 <- 1- p10-p01-p11
###
    y1 <- rbinom(n,1,p1)
    y2 <- (y1==1)*rbinom(n,1,p11/p1)+(y1==0)*rbinom(n,1,p01/(1-p1))
    list(x1=x1,x2=x2,y1=y1,y2=y2,id=1:n)
} ## }}}

##' @export
simBinFam <- function(n,beta=0.0,rhopp=0.1,rhomb=0.7,rhofb=0.1,rhobb=0.7) { ## {{{
    xc <- runif(n)*0.5
    xm <- rbinom(n,1,0.5+xc);
    xf <- rbinom(n,1,0.5+xc);
    xb1 <- rbinom(n,1,0.3+xc);
    xb2 <- rbinom(n,1,0.3+xc);
###
    rn <- matrix(rnorm(n*4),n,4)
    corm <- matrix( c(1,rhopp,rhomb,rhomb, rhopp,1,rhofb,rhofb, rhomb,rhofb,1,rhobb, rhomb,rhofb,rhobb,1),4,4)
    rnn <- t( corm %*% t(rn))
    zm <- exp(rnn[,1]); zf <- exp(rnn[,2]); zb1 <- exp(rnn[,3]); zb2 <- exp(rnn[,4]);
    pm <- exp(0.5+xm*beta+zm)
    pf <- exp(0.5+xf*beta+zf)
    pf <- pf/(1+pf)
    pm <- pm/(1+pm)
    pb1 <- exp(0.5+xb1*beta+zb1)
    pb1 <- pb1/(1+pb1)
    pb2 <- exp(0.5+xb2*beta+zb2)
    pb2 <- pb2/(1+pb2)
    ym <- rbinom(n,1,pm)
    yf <- rbinom(n,1,pf)
    yb1 <- rbinom(n,1,pb1)
    yb2 <- rbinom(n,1,pb2)
                                        #
    agem <- 20+runif(n)*10
    ageb1 <- 5+runif(n)*10
    data.frame(agem=agem,agef=agem+3+rnorm(n)*2,
               ageb1=ageb1,ageb2=ageb1+1+runif(n)*3,xm=xm,xf=xf,xb1=xb1,xb2=xb2,ym=ym,yf=yf,yb1=yb1,yb2=yb2,id=1:n)
} ## }}}

##' @export
simBinFam2 <- function(n,beta=0.0,alpha=0.5,lam1=1,lam2=1,...) { ## {{{
    x1 <- rbinom(n,1,0.5); x2 <- rbinom(n,1,0.5);
    x3 <- rbinom(n,1,0.5); x4 <- rbinom(n,1,0.5);
### random effects speicification of model via gamma distributions
    zf <- rgamma(n,shape=lam1); zb <- rgamma(n,shape=lam2);
    if (length(alpha)!=4) alpha <- rep(alpha[1],4)
    if (length(beta)!=4) beta <- rep(beta[1],4)
    pm <- exp(alpha[1]+x1*beta[1]+zf)
    pf <- exp(alpha[2]+x2*beta[2]+zf)
    pf <- pf/(1+pf)
    pm <- pm/(1+pm)
    pb1 <- exp(alpha[3]+x1*beta[3]+zf+zb)
    pb1 <- pb1/(1+pb1)
    ym <- rbinom(n,1,pm)
    yf <- rbinom(n,1,pf)
    yb1 <- rbinom(n,1,pb1)
    yb2 <- rbinom(n,1,pb1)
                                        #
    data.frame(x1=x1,x2=x2,ym=ym,yf=yf,yb1=yb1,yb2=yb2,id=1:n)
} ## }}}

##' @export
simbinClaytonOakes.family.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
###  K <- 10000
  n <- 4 # family members with ace structure
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2;
  varc <- varc/sumpar^2
  ## total variance 1/(varg+varc)
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc+varg
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  mother.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  father.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  env <- rgamma(K,varc)/eta
  mother <- apply(mother.g,1,sum)+env
  father <- apply(father.g,1,sum)+env
  child1 <- apply(cbind(mother.g[,c(1,2)],father.g[,c(1,2)]),1,sum) + env
  child2 <- apply(cbind(mother.g[,c(1,3)],father.g[,c(1,3)]),1,sum) + env
  Gam1 <- cbind(mother,father,child1,child2)
  ## }}}

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  xb1 <- rbinom(K,1,0.5)
  xb2 <- rbinom(K,1,0.5)
  xm <- rbinom(K,1,0.5)
  xf <- rbinom(K,1,0.5)
###
  if (is.null(beta)) beta <- rep(0.3,4)
  if (is.null(alpha)) alpha <- rep(0.5,4)
  pm <- exp(alpha[1]+xm*beta[1]); pf <- exp(alpha[2]+xf*beta[2])
  pb1 <- exp(alpha[3]+xb1*beta[3]); pb2 <- exp(alpha[4]+xb2*beta[4])
  p <- cbind(pm,pf,pb1,pb2)
  p <- p/(1+p)

  vartot <- eta
  pgivenZ <- ilap(vartot,p)
###  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)
  xs <- cbind(xm,xf,xb1,xb2)
  type <- rep(c("mother","father","child","child"),K)

  ud <- data.frame(ybin=c(Ybin),x=c(t(xs)),type=type,cluster=rep(1:K,each=n))

return(ud)
} ## }}}

##' @export
simbinClaytonOakes.twin.ace <- function(K,varg,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
  n <- 2 # twins with ace structure
  ## total variance 1/(varg+varc)
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2;
  varc <- varc/sumpar^2
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc+varg
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  dz.g <-  cbind(rgamma(K/2,varg*0.5)/eta, rgamma(K/2,varg*0.5)/eta, rgamma(K/2,varg*0.5)/eta)
  mz.g <-  rgamma(K/2,varg)/eta
  env1 <- rgamma(K/2,varc)/eta
  env2 <- rgamma(K/2,varc)/eta
  mz   <- mz.g+env1
  dz1 <- apply(dz.g[,c(1,2)],1,sum) + env2
  dz2 <- apply(dz.g[,c(1,3)],1,sum) + env2
  Gam1 <- rbind(cbind(mz,mz),cbind(dz1,dz2))
  ## }}}

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  xb1 <- rbinom(K,1,0.5)
  xb2 <- rbinom(K,1,0.5)
###
  if (is.null(beta)) beta <- rep(0.3,2)
  if (is.null(alpha)) alpha <- rep(0.5,2)
  pb1 <- exp(alpha[1]+xb1*beta[1]);
  pb2 <- exp(alpha[2]+xb2*beta[2])
  p <- cbind(pb1,pb2)
  p <- p/(1+p)

  vartot <- eta
  pgivenZ <- ilap(vartot,p)

###  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)
  xs <- cbind(xb1,xb2)
  type <- rep(c("twin1","twin2"),K)
  zygosity <- rep(c("MZ","DZ"),each=K)

  ud <- data.frame(ybin=c(Ybin),x=c(t(xs)),type=type,cluster=rep(1:K,each=n),zygosity=zygosity)

###names(ud)<-c("ybin","x","cluster","type")
return(ud)
} ## }}}

##' @export
simbinClaytonOakes.pairs <- function(K,varc,beta=NULL,alpha=NULL)  ## {{{
{
  ## K antal clustre (families), n=antal i clustre
###  K <- 10000
  n <- 2 # twins with ace structure
  ## total variance 1/(varg+varc)
  ## {{{  random effects
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc <- 1/varc
  ### mother and father share environment
  ### children share half the genes with mother and father and environment
  rans <- rgamma(K,varc)/eta
  Gam1 <- cbind(rans,rans)
  ## }}}
###  apply(Gam1,2,mean); apply(Gam1,2,var); cor(Gam1)

  ## {{{ marginals p's and conditional p's given random effects
  ### marginals p's for mother, father, children
  x1 <- rbinom(K,1,0.5)
  x2 <- rbinom(K,1,0.5)
###
###
  if (is.null(beta)) beta <- rep(0.3,2)
  if (is.null(alpha)) alpha <- rep(0.5,2)
  p1 <- exp(alpha[1]+x1*beta[1]);
  p2 <- exp(alpha[1]+x2*beta[2])
  p <- cbind(p1,p2)
  p <- p/(1+p)

  vartot <- eta
###  pgivenZ <- mets:::ilap(vartot,p)
  pgivenZ <- ilap(vartot,p)
  pgivenZ <- exp(- Gam1*pgivenZ)
  ## }}}

  Ybin <- matrix(rbinom(n*K,1,c(pgivenZ)),K,n)
  Ybin <- t(Ybin)

  xs <- cbind(x1,x2)
  ud <- cbind(c(Ybin),c(t(xs)),rep(1:K,each=n))

  ud <- cbind(ud)
  ud <- data.frame(ud)

names(ud)<-c("ybin","x","cluster")
return(ud)
} ## }}}

### pairwise POR model based on case-control data
##' @export
CCbinomial.twostage <- function(margbin=NULL,data=parent.frame(),method="nlminb",
    response="response",id="id",num="num",case.num=0,
    Nit=60,detail=0, silent=1,weights=NULL, control=list(),
    theta=NULL,theta.formula=NULL,desnames=NULL,
    deshelp=0,var.link=1,iid=1,
    step=0.5,model="plackett",marginal.p=NULL,
    strata=NULL,max.clust=NULL,se.clusters=NULL)
{ ## {{{
  ### under construction

    if (inherits(margbin,"glm")) ps <- predict(margbin,type="response")
    else if (inherits(margbin,"formula")) {
        margbin <- glm(margbin,data=data,family=binomial())
        ps <- predict(margbin,type="response")
    }  else if (is.null(marginal.p)) stop("without marginal model, marginal p's must be given\n");

    if (!is.null(marginal.p)) ps <- marginal.p

    data <- cbind(data,ps)

### make all pairs in the families,
    fam <- familycluster.index(data[,id])
    data.fam <- data[fam$familypairindex,]
    data.fam$subfam <- fam$subfamilyindex

### make dependency design using wide format for all pairs
    data.fam.clust <- fast.reshape(data.fam,id="subfam")
    if (is.function(theta.formula)) {
        desfunction <- compiler::cmpfun(theta.formula)
	if (deshelp==1){
            cat("These names appear in wide version of pairs for dependence \n")
            cat("design function must be defined in terms of these: \n")
            cat(names(data.fam.clust)); cat("\n")
            cat("Here is head of wide version with pairs\n")
            print(head(data.fam.clust)); cat("\n")
	}
###	des.theta <- Reduce("rbind",lapply(seq(nrow(data.fam.clust)),function(i) unlist(desfunction(data.fam.clust[i,] ))))
        des.theta <- t(apply(data.fam.clust,1, function(x) desfunction(x)))
	colnames(des.theta) <- desnames
	desnames <- desnames
    } else {
        if (is.null(theta.formula)) theta.formula <- ~+1
        des.theta <- model.matrix(theta.formula,data=data.fam.clust)
        desnames <- colnames(des.theta);
    }
    data.fam.clust <- cbind(data.fam.clust,des.theta)
    if (deshelp==1) {
        cat("These names appear in wide version of pairs for dependence \n")
        print(head(data.fam.clust))
    }

### back to long format keeping only needed variables
    data.fam <- fast.reshape(data.fam.clust,varying=c(response,id,"ps"))
    if (deshelp==1) {
	cat("Back to long format for binomial.twostage (head)\n");
        print(head(data.fam));
	cat("\n")
	cat(paste("binomial.twostage, called with reponse",response,"\n"));
	cat(paste("cluster=",id,",  subcluster (pairs)=subfam \n"));
	cat(paste("design variables ="));
	cat(desnames)
	cat("\n")
    }

    out <- binomial.twostage(data.fam[,response],data=data.fam,
                             clusters=data.fam$subfam,
                             theta.des=data.fam[,desnames],
                             detail=detail, method=method, Nit=Nit,step=step,
                             iid=iid,theta=theta, var.link=var.link,model=model,
                             max.clust=max.clust,
                             marginal.p=data.fam[,"ps"], se.clusters=data.fam[,id])
    return(out)
} ## }}}

Print <- function(x,n=NULL,nfirst=5,nlast=nfirst,digits=max(3,getOption("digits")-3),...) {
    mat <- !is.null(dim(x))
    if (!mat) {
        x <- cbind(x)
        colnames(x) <- ""
    }
    if (is.null(n)) {
        if (NROW(x)<=(nfirst+nlast)) n <- list(seq(NROW(x)))
        else {
            n <- c()
            if (nfirst>0)
                n <- c(n,list(seq(nfirst)))
            if (nlast>0)
                n <- c(n,list(-rev(seq(nlast))))
        }
    }
    if (is.null(n) || !is.list(n) && length(n)==1 && n==0) return(x)
    if (!is.list(n)) n <- list(n)
    d <- lapply(n,function(idx) {
        N <- NROW(x)
        idx <- idx[idx!=0 & abs(idx)<=N]
        idx[idx<0] <- N+idx[idx<0]+1
        base::format(x[idx,,drop=FALSE],digits=digits,...)
    })
    val <- c()
    sep <- rbind("---"=rep('',ncol(x)))
    for (i in seq_along(d)) {
        if (i>1) val <- rbind(val,sep)
        val <- rbind(val,base::as.matrix(d[[i]]))

    }
    return(structure(val,class=c("Print",class(val))))
}

##' @export
print.Print <- function(x,quote=FALSE,...) {
    class(x) <- class(x)[-1]
    print(x,quote=quote,...)
}


##' list, head, print, tail 
##'
##' listing for data frames
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param n Index of observations to print (default c(1:nfirst, n-nlast:nlast)
##' @param ... Optional additional arguments (nfirst,nlast, and print options)
##' @param x possible group variable
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##' m <- lava::lvm(letters)
##' d <- lava::sim(m, 20)
##'
##' dlist(d,~a+b+c)
##' dlist(d,~a+b+c|a<0 & b>0)
##' ## listing all : 
##' dlist(d,~a+b+c|a<0 & b>0,n=0)
##' dlist(d,a+b+c~I(d>0)|a<0 & b>0)
##' dlist(d,.~I(d>0)|a<0 & b>0)
##' dlist(d,~a+b+c|a<0 & b>0, nlast=0)
##' dlist(d,~a+b+c|a<0 & b>0, nfirst=3, nlast=3)
##' dlist(d,~a+b+c|a<0 & b>0, 1:5)
##' dlist(d,~a+b+c|a<0 & b>0, -(5:1))
##' dlist(d,~a+b+c|a<0 & b>0, list(1:5,50:55,-(5:1)))
##' dprint(d,a+b+c ~ I(d>0) |a<0 & b>0, list(1:5,50:55,-(5:1)))
##' @aliases dprint dlist dhead dtail 
##' @export
dprint <- function(data,y=NULL,n=0,...,x=NULL) daggregate(data,y,x,...,fun=function(z,...) Print(z,n=n,...),silent=FALSE)

##' @export
dlist <- function(data,y=NULL,n=NULL,...) dprint(data,y=y,n=n,...)

##' Calculate summary statistics grouped by variable
##'
##' Calculate summary statistics grouped by
##' @title Calculate summary statistics grouped by
##' @param data Data.frame
##' @param INPUT Input variables (character or formula)
##' @param ... functions
##' @param ID id variable
##' @param ORDER (optional) order variable
##' @param SUBSET (optional) subset expression
##' @param SORT sort order (id+order variable)
##' @param COMBINE If TRUE result is appended to data
##' @param NOCHECK No sorting or check for missing data
##' @param ARGS Optional list of arguments to functions (...)
##' @param NAMES Optional vector of column names
##' @param COLUMN If TRUE do the calculations for each column
##' @param REDUCE Reduce number of redundant rows
##' @param REGEX Allow regular expressions
##' @param ALL if FALSE only the subset will be returned
##' @export
##' @author Klaus K. Holst and Thomas Scheike
##' @details
##' dby2 for column-wise calculations
##' @aliases dby dby<- dby2 dby2<- dbyr
##' @examples
##' n <- 4
##' k <- c(3,rbinom(n-1,3,0.5)+1)
##' N <- sum(k)
##' d <- data.frame(y=rnorm(N),x=rnorm(N),id=rep(seq(n),k),num=unlist(sapply(k,seq)))
##' d2 <- d[sample(nrow(d)),]
##'
##' dby(d2, y~id, mean)
##' dby(d2, y~id + order(num), cumsum)
##'
##' dby(d,y ~ id + order(num), dlag)
##' dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2))
##' dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2), NAMES=c("l1","l2"))
##'
##' dby(d, y~id + order(num), mean=mean, csum=cumsum, n=length)
##' dby(d2, y~id + order(num), a=cumsum, b=mean, N=length, l1=function(x) c(NA,x)[-length(x)])
##'
##' dby(d, y~id + order(num), nn=seq_along, n=length)
##' dby(d, y~id + order(num), nn=seq_along, n=length)
##'
##' d <- d[,1:4]
##' dby(d, x<0) <- list(z=mean)
##' d <- dby(d, is.na(z), z=1)
##'
##' f <- function(x) apply(x,1,min)
##' dby(d, y+x~id, min=f)
##'
##' dby(d,y+x~id+order(num), function(x) x)
##'
##' f <- function(x) { cbind(cumsum(x[,1]),cumsum(x[,2]))/sum(x)}
##' dby(d, y+x~id, f)
##'
##' ## column-wise
##' a <- d
##' dby2(a, mean, median, REGEX=TRUE) <- '^[y|x]'~id
##' a
##' ## wildcards 
##' dby2(a,'y*'+'x*'~id,mean) 
##'
##'
##' ## subset
##' dby(d, x<0) <- list(z=NA)
##' d
##' dby(d, y~id|x>-1, v=mean,z=1)
##' dby(d, y+x~id|x>-1, mean, median, COLUMN=TRUE)
##'
##' dby2(d, y+x~id|x>0, mean, REDUCE=TRUE)
##'
##' dby(d,y~id|x<0,mean,ALL=FALSE)
##'
##' a <- iris
##' a <- dby(a,y=1)
##' dby(a,Species=="versicolor") <- list(y=2)
dby <- function(data,INPUT,...,ID=NULL,ORDER=NULL,SUBSET=NULL,SORT=0,COMBINE=!REDUCE,NOCHECK=FALSE,ARGS=NULL,NAMES,COLUMN=FALSE,REDUCE=FALSE,REGEX=mets.options()$regex,ALL=TRUE) {
    if (missing(INPUT)) INPUT <- .~1
    val <- substitute(INPUT)
    INPUT <- try(eval(val),silent=TRUE)
    funs <- substitute(list(...))[-1]
    if (inherits(INPUT,"try-error")) {
        INPUT <- as.formula(paste0(".~1|",deparse(val)))        
    }
    if (inherits(INPUT,c("formula","character"))) {
        INPUT <- procformdata(INPUT,sep="\\|",data=data,na.action=na.pass,do.filter=FALSE,regex=REGEX,specials="order")
        if (is.null(ID)) ID <- INPUT$predictor
        if (is.null(SUBSET)) {
            if (length(INPUT$group)!=0)
                SUBSET <- INPUT$group[[1]]
        }
        if (is.null(ORDER)) {
            if (length(INPUT$specials$order)!=0)
                ORDER <- INPUT$specials$order[[1]]
        }
        INPUT <- INPUT$response
    }

    if (inherits(ID,"formula")) ID <- model.frame(ID,data=data,na.action=na.pass)
    if (inherits(ORDER,"formula")) ORDER <- model.frame(ORDER,data=data,na.action=na.pass)
    if (inherits(SUBSET,"formula")) SUBSET <- model.frame(SUBSET,data=data,na.action=na.pass)
    if (is.null(INPUT)) {
        INPUT <- ID
        ID <- NULL
    }
    noID <- FALSE
    if (length(ID)==0) {
        noID <- TRUE
        ID = rep(1,NROW(INPUT))
    }
    group <- NULL
    if (!COMBINE || length(funs)==0) group <- ID
    if (NCOL(ID)>1) ID <- interaction(ID)
    if (is.data.frame(ID)) {
        ID <- as.numeric(ID[,1,drop=TRUE])
    } else {
        ID <- as.numeric(ID)
    }
    if (is.data.frame(ORDER)) {
        ORDER <- ORDER[,1,drop=TRUE]
    }
    if (!NOCHECK) {
        if (length(ORDER)>0) {
            ord <- order(ID,ORDER,decreasing=SORT,method="radix")
        } else {
            ord <- order(ID,decreasing=SORT,method="radix")
        }
        numerics <- unlist(lapply(INPUT[1,],is.numeric))
        INPUT <- as.matrix(INPUT[ord,which(numerics),drop=FALSE])
        ID <- ID[ord]
        if (is.null(group)) data <- data[ord,,drop=FALSE]
        else {
            if (is.vector(group)) group <- group[ord]
            else group <- group[ord,,drop=FALSE]
        }
        na.idx <- which(is.na(ID))
    } else {
        INPUT <- as.matrix(INPUT)
    }

    if (length(SUBSET)>0) {
        SUBSET <- which(as.matrix(SUBSET))
        INPUT <- INPUT[SUBSET,,drop=FALSE]
        ID <- ID[SUBSET]
    }
    if (length(funs)==0) {
        if (noID) return(INPUT)
        return(cbind(INPUT,group))
    }

    if (NROW(INPUT)>0) {
    resl <- lapply(funs, function(fun_) {
        env <- new.env()
        isfun <- tryCatch(is.function(eval(fun_)),error=function(...) FALSE)
        if (isfun) {
            env$fun_ <- eval(fun_)
            if (length(ARGS)>0) {
                fun_ <- eval(fun_)
                ff <- function(...) do.call(fun_,c(list(...),ARGS))
                expr <- quote(ff(x))
            } else {
                expr <- quote(fun_(x))
            }
        } else {
            expr <- fun_
        }
        .ApplyBy2(INPUT,ID,F=expr,Env=env,Argument="x",Columnwise=COLUMN)
    })
    res <- Reduce(cbind,resl)
    } else {
        resl <- NULL
        res <- NULL
    }

    dn <- NULL
    ## Setting column names
    if (missing(NAMES)) {
        a <- match.call(expand.dots=FALSE)
        ff <- lapply(a[["..."]],deparse)
        fn <- lapply(ff,deparse)
        dn <- names(ff)
        if (is.null(dn)) dn <- rep("",length(ff))
        idx <- which(dn=="")
        if (length(idx)>0) {
            newn <- unlist(ff[idx])
            dn[idx] <- newn[seq_along(idx)]
            fcall <- grepl("^function",dn)
            if (any(fcall))
                dn[which(fcall)] <- which(fcall)
        }
        numcolf <- unlist(lapply(resl, NCOL))
        nn <- c()
        if (COLUMN) {
            colnames(INPUT)
            dn <- unlist(lapply(dn, function(x) paste(x,colnames(INPUT),sep=".")))
        }
        for (i in seq_along(resl)) {
            nc <- numcolf[i]
            curnam <- dn[i]
            if (COLUMN) {
                nc <- nc/NCOL(INPUT)
                pos <- NCOL(INPUT)*(i-1)+1
                pos <- seq(pos,pos+NCOL(INPUT)-1)
                curnam <- dn[pos]
            }
            if (nc>1) {
                nn <- c(nn, unlist(lapply(curnam, function(x) paste0(x,seq(nc)))))
            }
            else nn <- c(nn,curnam)
        }
        NAMES <- nn
    }

    try(colnames(res) <- NAMES,silent=TRUE)
    numidx <- grep("^[0-9]",colnames(res)) ## column names starting with digit
    if (length(numidx)>0)
       colnames(res)[numidx] <- paste0("_",colnames(res)[numidx])

    if (!NOCHECK && length(na.idx)>0) {
        res[na.idx,] <- NA
    }
    cl <- attr(resl[[1L]],"clustersize")
    if (COMBINE) {
        if (is.null(NAMES) & is.null(res) & length(dn)>0) {
            res <- matrix(NA,nrow(data),length(dn))
            colnames(res) <- dn
        }        
        nn <- colnames(res)
        nc <- ifelse(is.null(res), 0, ncol(res))
        res0 <- matrix(NA,nrow(data),nc)
        colnames(res0) <- nn
        didx <- which(colnames(data)%in%nn)
        if (length(didx)>0) {
            ridx <- match(colnames(data)[didx],nn)
            res0[,ridx] <- as.matrix(data[,didx])
            data <- data[,-didx,drop=FALSE]
        }
        if (length(SUBSET)>0) {
            res0[SUBSET,] <- res
            res <- cbind(data,res0)
        } else {
            if (!is.null(res))
                res <- cbind(data,res)
            else
                res <- data
        }

    } else {
        if (!noID) {
            if (length(SUBSET)>0) group <- group[SUBSET,,drop=FALSE]
            res <- cbind(group,res)
        }
    }
    if (REDUCE!=0) {
        if (REDUCE<0) { # Grab the last observation
            idx <- cumsum(cl)
        } else { # Grab the first
            idx <- cumsum(c(1,cl[-length(cl)]))
        }
        res <- res[unique(idx),,drop=FALSE]
        if (NROW(res)==1) {
            rownames(res) <- ""
        } else {
            rownames(res) <- NULL
        }
    }
    if (!ALL) {
        return(res[SUBSET,,drop=FALSE])
    }
    return(res)
}


##' @export
"dby<-" <- function(data,INPUT,...,value) {
    cl <- match.call()
    cl[[1L]] <- substitute(dby)
    a <- substitute(value)
    if (inherits(value,"formula")) {
        cl["value"] <- NULL
        names(cl)[names(cl)=="INPUT"] <- ""
        cl[["INPUT"]] <- value
    } else {
        if (is.list(value)) {
            cl[which(names(cl)=="value")] <- NULL
            start <- length(cl)
            for (i in seq_along(value)) {
                cl[start+i] <- value[i]
            }
            if (length(names(value))>0)
                names(cl)[start+seq_along(value)] <- names(value)
        } else {
            names(cl)[which(names(cl)=="value")] <- ""
        }
    }
    eval.parent(cl)
}

##' @export
dby2 <- function(data,INPUT,...) {
    cl <- match.call()
    cl[[1L]] <- substitute(dby)
    cl[["COLUMN"]] <- TRUE
    eval.parent(cl)
}

##' @export
"dby2<-" <- function(data,INPUT,...,value) {
    cl <- match.call()
    cl[[1L]] <- substitute(`dby<-`)
    cl[["COLUMN"]] <- TRUE
    eval.parent(cl)
}

##' @export
dbyr <- function(data,INPUT,...,COLUMN=FALSE) {
    cl <- match.call()
    cl[[1L]] <- substitute(`dby`)
    cl[["REDUCE"]] <- TRUE
    cl[["COLUMN"]] <- COLUMN
    eval.parent(cl)
}

##' Derivatives of the bivariate normal cumulative distribution function
##'
##' @title Derivatives of the bivariate normal cumulative distribution function
##' @param p Parameter vector
##' @param design Design function with defines mean, derivative of mean, variance,
##' and derivative of variance with respect to the parameter p
##' @param Y column vector where the CDF is evaluated
##' @author Klaus K. Holst
##' @usage
##' Dbvn(p,design=function(p,...) {
##'      return(list(mu=cbind(p[1],p[1]),
##'                dmu=cbind(1,1),
##'                S=matrix(c(p[2],p[3],p[3],p[4]),ncol=2),
##'                dS=rbind(c(1,0,0,0),c(0,1,1,0),c(0,0,0,1)))  )},                 
##'      Y=cbind(0,0))
##' @export
Dbvn <- function(p,design=function(p,...) {
                 return(list(mu=cbind(p[1],p[1]),
                             dmu=cbind(1,1),
                             S=matrix(c(p[2],p[3],p[3],p[4]),ncol=2),
                             dS=rbind(c(1,0,0,0),c(0,1,1,0),c(0,0,0,1)))
                        )},                 
                        Y=cbind(0,0)) {
  mS <- design(p)
  U0 <- with(mS,.Call("biprobit0",
                     mu,
                     S,dS,Y,dmu,NULL,FALSE,FALSE,
                     PACKAGE="mets"));
  return(c(U0,mS))
}

##' @export
npc <- function(T,cause,same.cens=TRUE,sep=FALSE) {
  mtime <- apply(T[,1:2],1,max)
  ot <- order(mtime)
  mtime <- mtime[ot]
  T <- T[ot,]
  if (!sep) {
    time1 <- as.vector(T[,1:2]); status1 <- as.vector(T[,3:4])
    ud.cens1<-survival::survfit(Surv(time1,status1==0)~+1);
    Gfit1<-cbind(ud.cens1$time,ud.cens1$surv)
    Gfit2 <- Gfit1<-rbind(c(0,1),Gfit1);
  } else {
    time1 <- as.vector(T[,1]); status1 <- as.vector(T[,3])
    ud.cens1<-survival::survfit(Surv(time1,status1==0)~+1);
    time2 <- as.vector(T[,2]); status2 <- as.vector(T[,4])
    ud.cens2<-survival::survfit(Surv(time2,status2==0)~+1);
    Gfit1<-cbind(ud.cens1$time,ud.cens1$surv)
    Gfit1<-rbind(c(0,1),Gfit1);
    Gfit2<-cbind(ud.cens2$time,ud.cens2$surv)
    Gfit2<-rbind(c(0,1),Gfit2);
  }
  i1 <- fast.approx(Gfit1[,1],T[,1])
  cweights1<-fast.approx(,Gfit1[,2])[[1]]
  cweights2<-fast.approx(Gfit2[,1],T[,2],Gfit2[,2])[[1]];
  weight11 <- apply(cbind(cweights1,cweights2),1,min)

  if (same.cens) {
    conc <- (T[,3]==cause[1])*(T[,4]==cause[2])/weight11
  } else {
    conc <-(T[,3]==cause[1])*(T[,4]==cause[2])/(cweights1*cweights2);
  }
  mtime <- mtime[!is.na(conc)]
  conc <- conc[!is.na(conc)]
  cbind(mtime,cumsum(conc)/length(conc))
}

##' @export
nonparcuminc <- function(t,status,cens=0) {
  ord <- order(t); t <- t[ord]; status <- status[ord]
  ud.cens<-survival::survfit(Surv(t,status==cens)~1)
  Gfit<-cbind(ud.cens$time,ud.cens$surv)
  Gfit<-rbind(c(0,1),Gfit);
  causes <- setdiff(unique(status),cens)
  cweight<-fast.approx(Gfit[,1],t,Gfit[,2])[[1]];
  cc <- t
  for (i in 1:length(causes)) {
    c1 <- status==causes[i]
    cc <- cbind(cc,cumsum(c1/cweight)/length(c1))
  }
  return(cc)
}

##' Piecewise constant hazard distribution
##'
##' Piecewise constant hazard distribution
##' @aliases rpch ppch
##' @export
##' @param n sample size
##' @param lambda rate parameters
##' @param breaks time cut-points
##' @aliases rpch ppch
rpch <- function(n, lambda=1, breaks=c(0,Inf)) {
    res <- .Call("_mets_rpch",
                 n=n,
                 lambda=lambda,
                 time=breaks)
    return(res)
}

##' Piecewise constant hazard distribution
##'
##' Piecewise constant hazard distribution
##' @param base1 baseline 
##' @param rr relative risk terms 
##' @param entry entry times for left truncation 
##' @export
rchazC <- function(base1, rr, entry) {
    if (sum(abs(base1[1,]))>0) base1 <- rbind(0,base1)
    res <- .Call("_mets_rchazC",as.matrix(base1),rr,entry)
    return(res)
}

##' Finds subjects related to same cluster
##' 
##' @references
##' Cluster indeces 
##' @examples
##' i<-c(1,1,2,2,1,3)
##' d<- cluster.index(i)
##' print(d)
##'
##' type<-c("m","f","m","c","c","c")
##' d<- cluster.index(i,num=type,Rindex=1)
##' print(d)
##' @seealso familycluster.index familyclusterWithProbands.index
##' @author Klaus Holst, Thomas Scheike
##' @param clusters  list of indeces
##' @param index.type if TRUE then already list of integers of index.type
##' @param num to get numbering according to num-type in separate columns
##' @param Rindex index starts with 1, in C is it is 0
##' @param mat to return matrix of indeces
##' @param return.all return all arguments
##' @param code.na how to code missing values
##' @aliases countID pairRisk mystrata mystrata2index
##' @export
cluster.index <- function(clusters,index.type=FALSE,num=NULL,Rindex=0,mat=NULL,return.all=FALSE,code.na=NA)
{ ## {{{
  n <- length(clusters)

  if (index.type==FALSE)  {
    if (is.numeric(clusters)) clusters <-  fast.approx(unique(clusters),clusters)-1 else  {
      max.clust <- length(unique(clusters))
      clusters <- as.integer(factor(clusters, labels = seq(max.clust)))-1
    }
  }
  
  if ((!is.null(num))) { ### different types in different columns
    mednum <- 1
    if (is.numeric(num)) numnum <-  fast.approx(unique(num),num)-1
    else {
      numnum <- as.integer(factor(num, labels = seq(length(unique(clusters))))) -1
    }
  } else { numnum <- 0; mednum <- 0; }

  clustud <- .Call("clusterindexM",as.integer(clusters),as.integer(mednum), as.integer(numnum),mat,return.all,PACKAGE="mets")
  if (!is.null(mat) && !return.all) return(clustud)
  
  if (Rindex==1) clustud$idclust <- clustud$idclustmat+1
  if (Rindex==1) clustud$firstclustid <- clustud$firstclustid +1 
  ### avoid NA's for C call
  if (Rindex==0 & !is.na(code.na)) clustud$idclust[is.na(clustud$idclust)] <- code.na
  
  clustud
} ## }}}

##' @export
countID <- function(data,id="id",names.count="Count",total.count="Total",index.name="index",reverse=TRUE,sep="",addid=TRUE)
{# {{{

clusters <- data[,id]
if (is.numeric(clusters)) {
   ## integeres from 0 to max.clust
   uc <- unique(clusters)
   max.clust <- length(uc)
   clusters <- fast.approx(uc, clusters) - 1
}
else {
     max.clust <- length(unique(clusters))
     clusters <- as.integer(factor(clusters, labels = seq(max.clust))) - 1
}

###tabid <- table(clusters)
nclust <- table(clusters)

if (addid)  {
name1 <- paste(names.count,id,sep=sep)
name2 <- paste("index",id,sep=sep)
name3 <- paste(total.count,id,sep=sep)
name4 <- paste("reverse",names.count,id,sep=sep)
} else {
name1 <- names.count
name2 <- index.name
name3 <- total.count
name4 <- paste("reverse",names.count,sep=sep)
}

out <- data[,id,drop=FALSE]
out[,name1] <- c(cumsumstrata(rep(1,nrow(data)),clusters,max.clust))
out[,name2] <- clusters 
out[,name3] <- nclust[clusters+1]
out[,name4] <- 1+out[,name3]-out[,name1]

attr(out,"max.clust") <- max.clust

return(out)
}# }}}

##' @export
pairRisk <- function(start,stop,status,expo,clust,nsize=10,doublerisk=1)
{# {{{
    n <- length(start)
    id <- 1:n
    nclust <- length(unique(clust))
    sig <- c(rep(-1, each = n), rep(1, each = n))
    clust.seq <- c(as.numeric(as.factor(clust)), as.numeric(as.factor(clust)))
    clust <- c(clust, clust)
    expo <- c(expo, expo)
    id <- c(id, id)
    sstatus <- c(rep(0, length(start)), status)
    tts <- c(start, stop)
    ot <- order(clust.seq, tts, -rank(sstatus))
    tts <- tts[ot]
    sstatus <- sstatus[ot]
    sig <- sig[ot]
    expo <- expo[ot]
    clust <- clust[ot]
    clust.seq <- clust.seq[ot]
    id <- id[ot]
    cc <- c(mets::revcumsumstrata(sig * expo * nsize + sig * (expo == 0), clust.seq - 1, nclust))
    ### both under risk when cc>10
    if (doublerisk)
    pair.risk <- which(cc > nsize)
    else pair.risk <- which(cc>=1)
    clustpl <- clust[pair.risk]
    weightpl <- cc[pair.risk] - nsize
    caseweightpl <- rep(-1, length(weightpl))
    casepl <- expo[pair.risk]
    caseweightpl[casepl == 1] <- weightpl[casepl == 1]
    ttexit <- tts[pair.risk]
    ttentry <- tts[pair.risk - 1]
    ttid <- id[pair.risk]
    tstatus <- sstatus[pair.risk]
    timesout <- cbind(rep(ttentry, times = weightpl), rep(ttexit, times = weightpl))
    whichnotsame <- which(timesout[, 1] != timesout[, 2])
    caseweightrep <- rep(caseweightpl, times = weightpl) * (!duplicated(cbind(rep(ttexit,
        times = weightpl), rep(clustpl, time = weightpl)))) * 1
    weightstatusrep <- rep(tstatus, times = weightpl) * (caseweightrep != 0)
    idout <- rep(ttid, times = weightpl) * (caseweightrep != 0)
    clustplrep <- rep(clustpl, times = weightpl)
    out <- data.frame(timesout, weightstatusrep, caseweightrep,
        clustplrep, idout, stringsAsFactors = FALSE)[whichnotsame, ]
    out <- out[order(out[, 5]), ]
    return(out)
}# }}}

##' @export
mystrata <- function(ll,sort=TRUE) {# {{{
	for (j in seq(1,length(ll))) 
	if (!is.factor(ll[[j]])) ll[[j]] <- factor(ll[[j]])

	nll <- length(ll[[1]])
        ss <- rep(0,nll)
        nl <- unlist(lapply(ll,nlevels))
        poss <- c(exp(revcumsum(log(nl[-1]))))
	poss <- c(poss,1)
	for (j in seq(1,length(ll))) {
	     ss <- ss+as.numeric(ll[[j]])*poss[j]
	}
	uss <- unique(ss)
	nindex <- length(uss)
        sindex <- fast.approx(uss,ss)
        attr(sindex,"nlevel") <- nindex
        attr(sindex,"levels") <- nl 
	return(sindex)
} # }}}

##' @export
mystrata2index <- function(ll,sort=TRUE) {# {{{
	nll <- nrow(ll)
        ss <- rep(0,nll)
        nl <- apply(ll,2,max)
        poss <- c(exp(revcumsum(log(nl[-1]))))
	poss <- c(poss,1)
	for (j in seq(1,ncol(ll))) { ss <- ss+ll[,j]*poss[j] }
	uss <- unique(ss)
	nindex <- length(uss)
        sindex <- fast.approx(uss,ss)
        attr(sindex,"nlevel") <- nindex
        attr(sindex,"levels") <- nl 
	return(sindex)
} # }}}


##' Finds all pairs within a cluster (family)
##' 
##' @references
##' Cluster indeces 
##' @examples
##' i<-c(1,1,2,2,1,3)
##' d<- familycluster.index(i)
##' print(d)
##' @seealso cluster.index familyclusterWithProbands.index
##' @author Klaus Holst, Thomas Scheike
##' @param clusters  list of indeces 
##' @param index.type argument of cluster index 
##' @param num num 
##' @param Rindex index starts with 1 in R, and 0 in C
##' @export
familycluster.index <- function(clusters,index.type=FALSE,num=NULL,Rindex=1)
{ ## {{{
  clusters <- cluster.index(clusters,Rindex=Rindex)
  totpairs <- sum(clusters$cluster.size*(clusters$cluster.size-1)/2)
  clustud <- .Call("familypairindex",clusters$idclust,clusters$cluster.size,as.integer(2*totpairs),PACKAGE="mets")
  clustud$pairs <- matrix(clustud$familypairindex,ncol=2,byrow=TRUE)
  clustud$clusters <- clusters$clusters[clustud$pairs[,2]]+1

  invisible(clustud)
} ## }}}

##' Finds all pairs within a cluster (famly)  with the proband (case/control) 
##' 
##' second column of pairs are the probands and the first column the related subjects
##' 
##' @references
##' Cluster indeces 
##' @examples
##' i<-c(1,1,2,2,1,3)
##' p<-c(1,0,0,1,0,1)
##' d<- familyclusterWithProbands.index(i,p)
##' print(d)
##' @author Klaus Holst, Thomas Scheike
##' @seealso familycluster.index cluster.index
##' @param clusters list of indeces giving the clusters (families)
##' @param probands list of 0,1 where 1 specifices which of the subjects that are probands 
##' @param index.type argument passed to other functions
##' @param num argument passed to other functions
##' @param Rindex index starts with 1, in C is it is 0
##' @export
familyclusterWithProbands.index <- function(clusters,probands,index.type=FALSE,num=NULL,Rindex=1)
{ ## {{{
    famc <-familycluster.index(clusters,index.type=index.type,num=num,Rindex=Rindex)
    if (length(probands)!=length(clusters)) stop("clusters and probands not same length\n"); 
    index.probs <- (1:length(clusters))[probands==1]
    subfamsWprobands <-famc$subfamilyindex[ famc$familypairindex %in% index.probs ]
    indexWproband <- famc$subfamilyindex %in% subfamsWprobands 
    famc$subfamilyindex <- famc$subfamilyindex[indexWproband]
    famc$familypairindex <- famc$familypairindex[indexWproband]
    pairs <- matrix(famc$familypairindex,ncol=2,byrow=TRUE)
    ipi1 <- pairs[,1] %in% index.probs
    gem2 <- pairs[,2]
    pairs[ipi1,2] <- pairs[ipi1,1]
    pairs[ipi1,1] <- gem2[ipi1]
    famc$pairs <- pairs
    famc$clusters <-  famc$clusters[ipi1]

    famc$familypairindex <- c(t(pairs))

    invisible(famc)
} ## }}}

##' @export
coarse.clust <- function(clusters,max.clust=100)
{ ## {{{ 

if (is.numeric(clusters)) 
   clusters <-  fast.approx(unique(clusters),clusters)

  cluster.size <- length(unique(clusters))

qq <- unique(quantile(clusters, probs = seq(0, 1, by = 1/max.clust)))
qqc <- cut(clusters, breaks = qq, include.lowest = TRUE)    
cclusters <-  as.integer(qqc)-1

return(cclusters)
} ## }}} 

##' @export
faster.reshape <- function(data,clusters,index.type=FALSE,num=NULL,Rindex=1)
{ ## {{{
  if (NCOL(data)==1) data <- cbind(data)
 ### uses data.matrix 
  if (!is.matrix(data)) data <- data.matrix(data)
  if (is.character(clusters)) clusters <- data[,clusters]
  n <- length(clusters)

  if (nrow(data)!=n)  stop("nrow(data) and clusters of different lengths\n"); 

  if (index.type==FALSE)  {
    max.clust <- length(unique(clusters))
    if (is.numeric(clusters)) clusters <-  fast.approx(unique(clusters),clusters)-1 else 
    {
      max.clust <- length(unique(clusters))
      clusters <- as.integer(factor(clusters, labels = 1:max.clust))-1
    }
  }

  if ((!is.null(num))) { ### different types in different columns
    if (length(num)!=n)  stop("clusters and num of different lengths\n"); 
    mednum <- 1
    if (is.numeric(num)) num <-  fast.approx(unique(num),num)-1
    else num <- as.integer(factor(num, labels = seq(length(unique(clusters))))) -1
  } else { num <- 0; mednum <- 0; }

  clustud <- .Call("clusterindexdata",as.integer(clusters),as.integer(mednum), as.integer(num),iddata=data,PACKAGE="mets")

  if (Rindex==1) clustud$idclust  <- clustud$idclust+1
###  if(Rindex==1) idclust[idclust==0] <- NA 
  maxclust <- clustud$maxclust

  xny <- clustud$iddata
  xnames <- colnames(data); 
  missingname <- (colnames(data)=="")
  xnames[missingname] <- paste(seq_len(maxclust))[missingname]
  xny <- data.frame(xny)
  mm <- as.vector(outer(xnames,seq_len(maxclust),function(...) paste(...,sep=".")))
  names(xny) <- mm

  return(xny); 
} ## }}}

context("Reshaping data")

test_that("fast reshape I", {    
    m <- lvm()
    regression(m,c(y1,y2,y3)~x) <- c(1,10,100)
    distribution(m,~x) <- f <- function(n,...) rbinom(n,1,0.5)+1
    d <- sim(m,10); 
    dd <- fast.reshape(d,var="y")
    d1 <- fast.reshape(dd,id="id")
    expect_true(sum((d[,endogenous(m)]-d1[,endogenous(m)])^2)<1e-20)
    d2 <- fast.reshape(dd,id="id",var="y",num="num")
    expect_true(sum((d-d2[,colnames(d)])^2)<1e-20)
})


test_that("fast reshape II", {
    testdata <- data.frame(hour=c(12,13,14,11,12,14,15,16),id=c(1,1,1,2,2,3,3,3),y=round(rnorm(8),2))
    widetest <- reshape(testdata,v.names="y",idvar="id",direction="wide",timevar="hour")
    wide <- fast.reshape(testdata,varying="y",id="id",num="hour",sep=".")
    expect_equivalent(widetest,wide[,colnames(widetest)])
})


test_that("fast rehape: different data types", {
    d <- data.frame(time1=c(1:5),
                    time2=c(6.070311,
                        2.026996,
                        7.584480,
                        8.630120,
                        8.193392))
    dd <- mets::fast.reshape(d)
    expect_equivalent(dd[,1],as.vector(t(d)))

    d <- data.frame(time1=c(TRUE,FALSE,TRUE,FALSE,TRUE),
                    time2=c(6.070311,
                        2.026996,
                        7.584480,
                        8.630120,
                        8.193392))
    dd <- mets::fast.reshape(d)
    expect_equivalent(dd[,1],as.vector(t(d)))
})



## fast.reshape(fast.reshape(d,var=c("y","z","w")),id="id",var=c("y","z","w"))
## library(mets)
## x <- matrix(1:10,5,2)
## x[3,2] <- 8
## x[3,2] <- NA
## cluster <- c(1,1,2,2,3)
## x <- cbind(x,cluster)
## x
## ud <- fast.reshape(data.frame(x),"cluster")
## ud
## ###
## out=cluster.index(cluster)
## out
## ###
## ud <- faster.reshape(x,cluster)
## ud
## ud <- faster.reshape(data.frame(x),cluster)
## ud
## ###
## colnames(x) <- c("y1","y2","cluster")
## x
## ud <- fast.reshape(data.frame(x),"cluster")
## ud
## ###
## num <- c(2,1,1,2,2)
## x
## out <- faster.reshape(x,cluster)
## out
## out <- faster.reshape(x,cluster,num=num)
## out

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"  
)

## -----------------------------------------------------------------------------
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations

## -----------------------------------------------------------------------------
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)

## -----------------------------------------------------------------------------
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim.phreg(cox1,n,data=bmt)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)

 ## changing the parameters 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim.phreg(cox10,n,data=bmt)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)

## -----------------------------------------------------------------------------
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)

## -----------------------------------------------------------------------------
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim.phreg(coxs,nsim,data=NULL,strata=strata)

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim.phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)

## -----------------------------------------------------------------------------
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim.phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,status==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,status==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)

## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- simMultistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)
 

## -----------------------------------------------------------------------------
library(mets)
nsim <- 100
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul.cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul.cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
plot(ciff)
lines(attr(dats,"Lam1"))

## -----------------------------------------------------------------------------
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim.cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
 cbind(cif2$coef,scif2$coef)
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)

## -----------------------------------------------------------------------------
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 1000

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
 estimate(dr)

 simcoxcox <- sim.recurrent(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(start,stop,statusD==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(start,stop,statusD==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)


## -----------------------------------------------------------------------------
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
 estimate(dr)

 simglcox <- sim.recurrent(recGL,dr,n=n,data=hf)

 simcoxcox <- sim.recurrent(xr,dr,n=n,data=hf)
 dtable(simcoxcox,~statusD)

 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 simglcox <- sim.recurrent(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(start,stop,statusD)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(start,stop,statusD==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)


## -----------------------------------------------------------------------------
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 100
 rr <- simRecurrent(n,base1,death.cumhaz=dr)
 ###
 par(mfrow=c(1,3))
 showfitsim(causes=1,rr,dr,base1,base1,which=1:2)

 rr <- simRecurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
 showfitsim(causes=2,rr,dr,base1,base4,which=1:2)

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- simRecurrentList(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
 showfitsimList(rr,cumhaz,drl) 

## ----weibull1-----------------------------------------------------------------
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)

## ----weibull_sim--------------------------------------------------------------
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)

## -----------------------------------------------------------------------------
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
 library(mets)
 data(diabetes)
 set.seed(100)
 
 # Marginal Cox model  with treat as covariate
 margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
 summary(fitco1)
 
 # Clayton-Oakes
 fitco2 <- survival.twostage(margph,data=diabetes,theta=0.0,
                  clusters=diabetes$id,var.link=1,model="clayton.oakes")
 summary(fitco2)
 fitco3 <- survival.twostage(margph,data=diabetes,theta=1.0,
                  clusters=diabetes$id,var.link=0,model="clayton.oakes")
 summary(fitco3)

## -----------------------------------------------------------------------------
  # without covariates but marginal model stratified 
  marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
 fitco<-twostageMLE(marg,data=diabetes,theta=1.0)
 summary(fitco)

  fitcoa <- survival.twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
		   model="clayton.oakes",var.link=0)
  summary(fitcoa)

## -----------------------------------------------------------------------------
 d <- simClaytonOakes(200,2,0.5,0,3)
  margph <- phreg(Surv(time,status)~x+cluster(cluster),data=d)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=d)
 summary(fitco1)

## -----------------------------------------------------------------------------
 udp <- piecewise.twostage(c(0,0.5,2),data=d,id="cluster",timevar="time",status="status",model="clayton.oakes",silent=0)
 summary(udp)

## -----------------------------------------------------------------------------
 data <- simClaytonOakes.twin.ace(200,2,1,0,3)

 out <- twin.polygen.design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 pardes <- out$pardes
 pardes 

## -----------------------------------------------------------------------------
 des.rv <- out$des.rv
 # MZ
 head(des.rv,2)
 # DZ 
 tail(des.rv,2)

## -----------------------------------------------------------------------------
### data <- simClaytonOakes.twin.ace(2000,2,1,0,3)
### out <- twin.polygen.design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data)
 ts <- twostage(aa,data=data,clusters=data$cluster,
      theta=c(2,1),var.link=0,random.design=out$des.rv,theta.des=out$pardes)
 summary(ts)

## -----------------------------------------------------------------------------
run <- 0
if (run==1) {
 data <- simClaytonOakes.twin.ace(1000000,2,1,0,3)

 out <- twin.polygen.design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 pardes <- out$pardes
 aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data)
 system.time(
 ts <- twostage(aa,data=data,clusters=data$cluster,
      theta=c(2,1),var.link=0,random.design=out$des.rv,theta.des=out$pardes)
 )
 summary(ts)
}

## -----------------------------------------------------------------------------
kendall.ClaytonOakes.twin.ace(ts$theta[1],ts$theta[2],K=10000) 

## -----------------------------------------------------------------------------
library(mets)
set.seed(1000)
data <- simClaytonOakes.family.ace(200,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

## -----------------------------------------------------------------------------
out <- ace.family.design(data,member="type",id="cluster")
out$pardes
head(out$des.rv,4)

## -----------------------------------------------------------------------------
pa <- phreg(Surv(time,status)~+1+cluster(cluster),data=data)

# make ace random effects design
ts <- twostage(pa,data=data,clusters=data$cluster,var.par=1,var.link=0,theta=c(2,1),
        random.design=out$des.rv,theta.des=out$pardes)
summary(ts)

## -----------------------------------------------------------------------------
# now specify fitting via specific pairs 
# first all pairs 
mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=12)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
head(pairs,n=6)

## -----------------------------------------------------------------------------
ts <- twostage(pa,data=data,clusters=data$cluster, theta=c(2,1),var.link=0,step=1.0,
        random.design=out$des.rv, theta.des=out$pardes,pairs=pairs)
summary(ts)

## -----------------------------------------------------------------------------
ssid <- sort(sample(1:nrow(pairs),200))
tsd <- twostage(pa,data=data,clusters=data$cluster,
    theta=c(2,1)/10,var.link=0,random.design=out$des.rv,
   theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)

## -----------------------------------------------------------------------------
ids <- sort(unique(c(pairs[ssid,])))

pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)

# this requires that pair.new refers to id's in dataid (survival, status and so forth)
# random.design and theta.des are constructed to be the array 3 dims via individual specfication from ace.family.design
dataid <- dsort(data[ids,],"cluster")
outid <- ace.family.design(dataid,member="type",id="cluster")
outid$pardes
head(outid$des.rv)

## -----------------------------------------------------------------------------
tsdid <- twostage(pa,data=dataid,clusters=dataid$cluster,theta=c(2,1)/10,var.link=0,baseline.iid=0,
          random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)

paid <- phreg(Surv(time,status)~+1+cluster(cluster),data=dataid)
tsdidb <- twostage(paid,data=dataid,clusters=dataid$cluster,theta=c(2,1)/10,
  var.link=0,random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdidb)
coef(tsdid)

## -----------------------------------------------------------------------------
pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new)
head(pair.types)

theta.des  <- rbind( c(rbind(c(1,0),c(1,0),c(0,1),c(0,0))),
		c(rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))))
random.des <- rbind( 
        c(1,0,1,0),c(0,1,1,0),
        c(1,1,0,1),c(1,0,1,1))
mf <- 1*(pair.types[,1]=="mother" & pair.types[,2]=="father")
##          pair, rv related to pairs,  theta.des related to pair 
pairs.new <- cbind(pair.new,(mf==1)*1+(mf==0)*3,(mf==1)*2+(mf==0)*4,(mf==1)*1+(mf==0)*2,(mf==1)*3+(mf==0)*4)

## -----------------------------------------------------------------------------
head(pairs.new[1:3,])
head(dataid)

## -----------------------------------------------------------------------------
random.des[1,]
random.des[2,]
matrix(theta.des[1,],4,2)

## -----------------------------------------------------------------------------
head(dataid)
matrix(theta.des[2,],4,2)
random.des[3,]
random.des[4,]

## -----------------------------------------------------------------------------
tsdid2 <- twostage(pa,data=dataid,clusters=dataid$cluster,
       theta=c(2,1)/10,var.link=0,step=1.0,random.design=random.des,
       baseline.iid=0, theta.des=theta.des,pairs=pairs.new,dim.theta=2)
summary(tsdid2)
tsd$theta
tsdid2$theta
tsdid$theta

## -----------------------------------------------------------------------------
kinship  <- rep(0.5,nrow(pair.types))
kinship[pair.types[,1]=="mother" & pair.types[,2]=="father"] <- 0
head(kinship,n=10)

out <- make.pairwise.design(pair.new,kinship,type="ace") 

## -----------------------------------------------------------------------------
tsdid3 <- twostage(pa,data=dataid,clusters=dataid$cluster,
   theta=c(2,1)/10,var.link=0,step=1.0,random.design=out$random.design,
   baseline.iid=0,theta.des=out$theta.des,pairs=out$new.pairs,dim.theta=2)
summary(tsdid3)
tsdid2$theta
tsdid$theta

## -----------------------------------------------------------------------------
outae <- make.pairwise.design(pair.new,kinship,type="ae") 
tsdid4 <- twostage(pa,data=dataid,clusters=dataid$cluster,
   theta=c(2,1)/10,var.link=0,random.design=outae$random.design,
   baseline.iid=0,theta.des=outae$theta.des,pairs=outae$new.pairs,dim.theta=1)
summary(tsdid4)

## -----------------------------------------------------------------------------
library(mets)
set.seed(1000)
data <- simClaytonOakes.family.ace(200,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)

## -----------------------------------------------------------------------------
 dtypes <- interaction( data[pairs[,1],"type"], data[pairs[,2],"type"])
 dtypes <- droplevels(dtypes)
 table(dtypes)
 dm <- model.matrix(~-1+factor(dtypes))
 head(dm)

## -----------------------------------------------------------------------------
pa <- phreg(Surv(time,status)~cluster(cluster),data)

tsp <- twostage(pa,data=data,theta.des=dm,pairs=cbind(pairs,1:nrow(dm)),se.clusters=data$clust)
summary(tsp)

## -----------------------------------------------------------------------------
 library(mets)
 data(diabetes)
 
 # Marginal Cox model  with treat as covariate
 margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
 summary(fitco1)
 
 # Plackett model
 mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 fitp <- survival.twostage(mph,data=diabetes,theta=3.0,
                clusters=diabetes$id,var.link=1,model="plackett")
 summary(fitp)
 
 # without covariates but with stratafied 
 marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
 fitpa <- survival.twostage(marg,data=diabetes,theta=1.0,
                 clusters=diabetes$id)
 summary(fitpa)
 
 fitcoa <- survival.twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
                  model="clayton.oakes")
 summary(fitcoa)

## -----------------------------------------------------------------------------
 mm <- model.matrix(~-1+factor(adult),diabetes)
 fitp <- survival.twostage(mph,data=diabetes,theta=3.0,Nit=40,
                clusters=diabetes$id,var.link=1,model="plackett",
		theta.des=mm)
 summary(fitp)

## -----------------------------------------------------------------------------
 # Piecewise constant cross hazards ratio modelling

 d <- subset(simClaytonOakes(1000,2,0.5,0,stoptime=2,left=0),!truncated)
 udp <- piecewise.twostage(c(0,0.5,2),data=d,id="cluster",timevar="time",
                           status="status",model="plackett",silent=0)
 summary(udp)

## -----------------------------------------------------------------------------
sessionInfo()

##' @export
`lifetable` <- function(x,...) UseMethod("lifetable")

##' Create simple life table 
##' 
##' @title Life table
##' @param x time formula (Surv) or matrix/data.frame with columns time,status or entry,exit,status
##' @param strata strata
##' @param data data.frame
##' @param breaks time intervals
##' @param weights weights variable
##' @param confint if TRUE 95\% confidence limits are calculated
##' @param ... additional arguments to lower level functions
##' @author Klaus K. Holst
##' @aliases lifetable lifetable.matrix lifetable.formula
##' @usage
##'  \method{lifetable}{matrix}(x, strata = list(), breaks = c(),
##'    weights=NULL, confint = FALSE, ...)
##'
##'  \method{lifetable}{formula}(x, data=parent.frame(), breaks = c(),
##'    weights=NULL, confint = FALSE, ...)
##' @examples
##' library(timereg)
##' data(TRACE)
##' 
##' d <- with(TRACE,lifetable(Surv(time,status==9)~sex+vf,breaks=c(0,0.2,0.5,8.5)))
##' summary(glm(events ~ offset(log(atrisk))+factor(int.end)*vf + sex*vf,
##'             data=d,poisson))
##' @export
lifetable.matrix <- function(x,strata=list(),breaks=c(),weights=NULL,confint=FALSE,...) {
    if (ncol(x)==3) {
        status <- x[,3]
        entry <- x[,1]
        time <- x[,2]
    } else {
        status <- x[,2]
        time <- x[,1]
        entry <- rep(0,length(time))
    }
    LifeTable(time,status,entry,strata=strata,breaks=breaks,weights=weights,confint,...)
}

##' @export
lifetable.formula <- function(x,data=parent.frame(),breaks=c(),weights=NULL,confint=FALSE,...) {
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("x", "data", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)
    names(mf)[which(names(mf)=="x")] <- "formula"    
    mf <- eval(mf, parent.frame())
    weights <- as.vector(model.weights(mf))    
    ##mf <- model.frame(x,data)
    Y <- model.extract(mf, "response")
    Terms <- terms(x, data = data)
    if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    strata <- list()
    X <- model.matrix(Terms, data)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)>0) {
        strata <- as.list(model.frame(Terms,data)[,-1,drop=FALSE])
    }
    LifeTable(exit,status,entry,
              strata=strata,breaks=breaks,weights=weights,confint=confint,...)       
}

## lifetable.data.table <- function(x,entry,exit,status,strata,breaks,...) {
##     requireNamespace("data.table")
##     breaks <- sort(unique(breaks))
##     nbreaks <- length(breaks)
##     ## prs <- parse(text=paste0("pmin(x,",exit,")-pmax(x-1,",entry,")"))
##     ## dt[,eval(prs),by=strata]
##     system.time(dur1 <- x[,
##                            lapply(breaks[-1],function(x)
##                               eval(parse(text=paste0("pmin(x,",exit,")-pmax(x-1,",entry,")")))
##                                  ),by=strata])
##     dur1[dur1<0] <- NA
##     system.time(endur1 <- x[,
##                             lapply(breaks[-nbreaks],function(x)
##                                 eval(parse(text=paste0("x+1-pmax(x,",entry,")")))
##                                    ),by=strata])
##     enter <- dur1[,lapply(.SD,function(x) sum(!is.na(x))),by=strata]
##     atrisk <- dur1[,lapply(.SD,function(x) sum(x,na.rm=TRUE)),by=strata]
##     vidx <- seq(length(strata)+1,ncol(atrisk))
##     suppressWarnings(names(atrisk)[vidx] <- paste0("_R",seq_along(vidx)))   
##     system.time(eventcens <- x[,
##                                lapply(breaks[-1],function(x)
##                                    eval(parse(text=paste0("((pmin(x,",exit,")-pmax(x-1,",entry,"))<(x-pmax(x-1,",entry,")))*(1+",status,")")))
##                                            ),by=strata])
##     lost <- eventcens[,lapply(.SD,function(x) sum(x==1,na.rm=TRUE)),by=strata]
##     events <- eventcens[,lapply(.SD,function(x) sum(x==2,na.rm=TRUE)),by=strata]
##     suppressWarnings(names(events)[vidx] <- paste0("_E",seq_along(vidx)))
##     res <- fast.reshape(cbind(events,atrisk[,vidx,with=FALSE]))
##     return(res)
##     res <- subset(data.frame(enter=enter,
##                              atrisk=atrisk,
##                              lost=lost,
##                              events=events,
##                              ## int.start=c(-Inf,breaks),
##                              ## int.end=c(breaks,Inf),
##                              int.start=breaks[-length(breaks)],
##                              int.end=breaks[-1],
##                              surv=0,
##                              rate=events/atrisk))
## }



LifeTable <- function(time,status,entry=NULL,weights=NULL,strata=list(),breaks=c(),confint=FALSE,interval=TRUE,mesg=FALSE) {
    if (is.null(entry)) entry <- rep(0,NROW(time))
    if (mesg) message(dim(time))
    if ((is.matrix(time) || is.data.frame(time)) && ncol(time)>1) {
        if (ncol(time)>=3L) {
            if (ncol(time)==4L) weights <- time[,4]
            status <- time[,3]
            entry <- time[,1]
            time <- time[,2]            
        } else {
            status <- time[,2]
            time <- time[,1]
            entry <- rep(0,length(time))
        }
    }

    if (mesg) message(dim(time))
    if ((is.matrix(time) || is.data.frame(time)) && ncol(time)>1) {
        if (ncol(time)==3) {
            status <- time[,3]
            entry <- time[,1]
            time <- time[,2]
        } else {
            status <- time[,2]
            time <- time[,1]
            entry <- rep(0,length(time))
        }
    }
    if (length(strata)>0) {
        a <- by(cbind(entry,time,status,weights), strata,
                FUN=LifeTable, breaks=breaks, confint=confint)
        cl <- lapply(strata,class)
        nulls <- which(unlist(lapply(a,is.null)))
        nonnulls <- setdiff(seq_along(a),nulls)
        nn <- do.call("expand.grid",attributes(a)$dimnames)
        if (length(nulls)>0) nn <- nn[-nulls,,drop=FALSE]
        nam <- nn[rep(seq(NROW(nn)),each=NROW(a[[nonnulls[1]]])),,drop=FALSE]
        xx <- list()
        for (i in seq(ncol(nam))) {
            if (cl[i]%in%c("numeric","integer"))
                xx <- c(xx,list(as.numeric(as.character(nam[,i]))))
            else                    
                xx <- c(xx, list(do.call(paste("as.",as.character(cl[i]),sep=""),list(nam[,i]))))
        }
        xx <- as.data.frame(xx); colnames(xx) <- colnames(nam)
        res <- Reduce("rbind",a)
        res <- cbind(res,xx)
        return(res)
    }
    if (length(breaks)==0) breaks <- c(0,max(time,na.rm=TRUE))
    if (length(breaks)==1) breaks <- c(0,breaks)
    breaks <- sort(unique(breaks))
    ## en <- matrix(unlist(lapply(c(-Inf, breaks),function(x) pmax(x,entry))),
    ##               ncol=length(breaks)+1)
    ## ex <- matrix(unlist(lapply(c(breaks, Inf),function(x) pmin(x,time))),
    ##               ncol=length(breaks)+1)
    en <- matrix(unlist(lapply(breaks[-length(breaks)],function(x) pmax(x,entry))),
                  ncol=length(breaks)-1)
    ex <- matrix(unlist(lapply(breaks[-1],function(x) pmin(x,time))),
                  ncol=length(breaks)-1)
    dur <- ex-en
    dur[dur<=0] <- NA
    eventcens <- dur; eventcens[!is.na(dur)] <- 0
    lastobs <- apply(eventcens,1,function(x) tail(which(!is.na(x)),1))
    eventcens[cbind(seq(nrow(eventcens)),lastobs)] <- status+1
    if (!is.null(weights)) {
        W <- weights %x% rbind(rep(1,ncol(dur)))
        W[is.na(dur)] <- NA
        enter <- colSums(W,na.rm=TRUE)
        atrisk <- colSums(dur*W,na.rm=TRUE)
        Sum <- function(x) { res <- sum(x,na.rm=TRUE); ifelse(length(res)>0L, res, 0L) } 
        lost <- apply(eventcens, 2, function(x) res <- Sum(weights[which(x==1)]))
        events <- apply(eventcens, 2, function(x) res <- Sum(weights[which(x==2)]))
    } else {
        enter <- colSums(!is.na(dur))
        atrisk <- colSums(dur,na.rm=TRUE)
        lost <- colSums(eventcens==1,na.rm=TRUE)
        events <- colSums(eventcens==2,na.rm=TRUE)
    }
    rate <- events/atrisk; rate[is.nan(rate)] <- 0
    res <- subset(data.frame(enter=enter,
                             atrisk=atrisk,
                             lost=lost,
                             events=events,
                             ## int.start=c(-Inf,breaks),
                             ## int.end=c(breaks,Inf),
                             int.start=breaks[-length(breaks)],
                             int.end=breaks[-1],
                             rate=rate))
    if (interval) res$interval <- factor(paste0("[",res$int.start,";",res$int.end,")"))
    cumsum.na <- function(x,...) { x[is.na(x)] <- 0; cumsum(x) }
    if (length(strata)==0) res$surv <- with(res, exp(-cumsum.na(rate*(int.end-int.start))))    
    if (confint) {
        ff <- events ~ offset(log(atrisk))
        if (length(breaks)>2) ff <- update(ff,.~.+factor(int.end)-1)
        g <- glm(ff,data=res,poisson)
        suppressMessages(ci <- rbind(exp(stats::confint(g))))
        res[,"2.5%"] <- ci[,1]
        res[,"97.5%"] <- ci[,2]
    }
    res
}

eh <- function(formula,intervals,family=poisson(log),...) {
    if (missing(intervals)) stop("Please supply list of time-intervals")
    g <- glm(formula,family=family,...)
    structure(g,class=c("glm","eh"),intervals=intervals)
    
}

##' Summary for survival analyses via the 'lifetable' function
##'
##' Summary for survival analyses via the 'lifetable' function 
##' @title Extract survival estimates from lifetable analysis
##' @param object glm object (poisson regression)
##' @param ... Contrast arguments
##' @param timevar Name of time variable
##' @param time Time points (optional)
##' @param int.len Time interval length (optional)
##' @param confint If TRUE confidence limits are supplied
##' @param level Level of confidence limits
##' @param individual Individual predictions
##' @param length.out Length of time vector
##' @aliases eventpois pcif 
##' @export
##' @author Klaus K. Holst
eventpois <- function(object,...,timevar,time,int.len,confint=FALSE,level=0.95,individual=FALSE,length.out=25) {      
    if (missing(timevar)) {
        timevar <- names(attributes(object)$intervals)[1]
        times <- attributes(object)$intervals
        int.len <- diff(times)
    }
    nn <- names(coef(object))
    if (is.null(timevar)) {
        idx <- unlist(sapply(c("\\(Intercept\\)","bs\\(","ns\\("),function(x) grep(x,nn)))
        keep <- setdiff(seq_along(nn),idx)
        cc <- estimate(object,exp,keep=keep,labels=nn[keep])$coefmat
        colnames(cc)[1] <- "RR"
        cc[,5] <- coef(summary(object))[keep,4]
        return(cc)
    }
    timevar_re0 <- gsub("\\$|\\^","",glob2rx(timevar))
    timevar_re <- paste0(timevar_re0,"[0-9]+\\.*[0-9]*")
    idx <- regexpr(timevar_re,nn)
    dots <- list(...)
    if (all(idx<0)) {
        timevar_re0 <- gsub("\\$|\\^","",glob2rx(paste0("factor(",timevar,")")))
        timevar_re <- paste0(timevar_re0,"[0-9]+\\.*[0-9]*")
        idx <- regexpr(timevar_re,nn)
    }
    
    tvar <- unique(regmatches(nn,idx))
    if (missing(time)) {        
        time <- sort(unique(as.numeric(gsub(timevar_re0,"",tvar))))
        if (length(time)==0) {
            i0 <- seq(nrow(object$data))
            ## browser()
            ## if ("rate"%in%colnames(object$data)) {
            ##     ii0 <- which(object$data[,"rate"]>0)
            ##     i0 <- intersect(i0,ii0)
            ## }
            ## for (i in seq_along(dots)) {
            ##      i0 <- intersect(i0,which(object$data[,names(dots)[i]]==dots[[i]]))
            ##  }
            rg <- range(object$data[i0,timevar],na.rm=TRUE)        
            time <- seq(rg[1],rg[2],length.out=length.out)
        } else {
            ##time <- seq(1,rg[2])
        }
    }
    if (missing(int.len)) {
        int.len <- diff(c(0,time))
    } else if (int.len==1) int.len <- rep(int.len,nrow(res))
    tt <- terms(object)
    offsetvar <- NULL
    if (attr(tt,"offset")) {
        offsetvar <- all.vars(tt)[attr(tt,"offset")]
        dots[[offsetvar]] <- 1
    }
    responsevar <- getoutcome(tt)
    vv <- setdiff(all.vars(formula(tt)),c(offsetvar,responsevar))
    dots0 <- dots
    dots[[timevar]] <- time
    dotnames <- c()
    if (individual) {
        newdata <- as.data.frame(dots)
        for (v in vv) {
            if (v%ni%names(dots)) newdata[,v] <- object$data[1,v]
            else dotnames <- c(dotnames,v)
        }    
    } else {
        for (v in vv) {            
            if (v%ni%names(dots)) {
                dots[[v]] <- object$data[1,v]
            } else {
                dotnames <- c(dotnames,v)
            }
        }        
        args <- c(list(`_data`=model.frame(object)),dots)
        newdata <- do.call(Expand, args)
        newdata <- dsort(newdata,c(setdiff(dotnames,timevar),timevar))
    }

    Terms <- delete.response(tt)    
    m <- model.frame(Terms, newdata, xlev = object$xlevels)
    X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    ## NA-robust matrix product
    beta0 <- coef(object); beta0[is.na(beta0)] <- -.Machine$double.xmax    
    V <- vcov(object)    
    V0 <- structure(matrix(0,length(beta0),length(beta0)),dimnames=list(names(beta0),names(beta0)))
    idx <- which(rownames(V0)%in%rownames(V))
    V0[idx,idx] <- V
    coefs <- X%*%beta0
    res <- cbind(time-int.len,int.len,exp(coefs))
    colnames(res) <- c("time","int.len","rate")    
    if (nrow(coefs)<=length(time)) {
        if (!confint) {
            res <- cbind(res,cumsum(res[,3]*int.len))
            res <- cbind(res,exp(-res[,4]))
        } else {
            S0 <- X%*%V0%*%t(X)
            system.time(e0 <- estimate(NULL,coef=coefs,vcov=S0,function(x) cumsum(exp(x)*int.len))) ## Cumulative hazard
            system.time(e1 <- estimate(e0, function(x) exp(-x),level=level)) ## Survival
            res <- cbind(res,e0$coefmat[,1],e1$coefmat[,c(1,3:4)])
        }
        colnames(res)[4:5] <- c("chaz","surv") 
    }
    if (!is.null(offsetvar)) newdata[,offsetvar] <- NULL

    times <- list()
    if (!individual) {
        aa <- unique(newdata[,setdiff(dotnames,c(timevar,offsetvar)),drop=FALSE])
        vv <- colnames(aa)
        for (i in seq(nrow(aa))) {
            idx <- seq(nrow(object$data))
            for (j in seq_along(vv)) {
                val <- aa[i,j]
                if (is.factor(val)) val <- as.character(val)
                idx <- intersect(idx,which(object$data[,vv[j]]==val))
            }
            times <- c(times, list(sort(unique(object$data[idx,timevar]))))
        }
    }

    structure(as.data.frame(cbind(res,newdata)),
              args=dots0,
              variables=aa,
              times=times,
              individual=individual,
              class=c("eventpois","data.frame")
              )
}

##' @export
plot.eventpois <- function(x,var,confint=TRUE,cont=TRUE,length.out=200,type="surv",
                           line.type="l",lty=c(1:8),col=1,lwd=1,add=FALSE,
                           use.time=TRUE,
                           legend,                           
                           xlab="Time",ylab="Survival probability",xlim,ylim,...) {
    type <- agrep(type,c("clogclog","hazard","survival"))
    if (!add) {
        if (missing(xlim)) xlim <- c(0,max(x[,1]+x[,2]))
        if (missing(ylim)) {
            ylim <- switch(type,
                           '1'=c(-30,max(log(sum(x[,2]*x[,3])))),
                           '2'=c(0,max(x[,3])),
                           c(0,1))
        }
        plot(0,type="n",xlab=xlab,ylab=ylab,xlim=xlim,ylim=ylim,...)
    }
    ##args <- attr(x,"args")
    ##aa <- expand.grid(args)
    aa <- attr(x,"variables")
    vv <- colnames(aa)    
    if (attr(x,"individual")) aa <- aa[1,,drop=FALSE]
    if (missing(legend)) legend <- nrow(aa)>1
    if (length(lty)<nrow(aa)) lty <- rep(lty,nrow(aa))
    if (length(col)<nrow(aa)) col <- rep(col,nrow(aa))
    if (length(lwd)<nrow(aa)) lwd <- rep(lwd,nrow(aa))
    for (i in seq(nrow(aa))) {
        idx <- seq(nrow(x))
        
        if (!attr(x,"individual"))
        for (j in seq_along(vv)) {
            val <- aa[i,j]
            if (is.factor(val)) val <- as.character(val)
            idx <- intersect(idx,which(x[,vv[j]]==val))
        }
        idx <- intersect(idx,which(x[,3]>0))
        e0 <- x[idx,,drop=FALSE]
        dt <- tail(e0,1)        
        time0 <- c(e0[,1],dt[,1]+dt[,2])
        if (use.time) {
            ##browser()
            time0 <- attr(x,"time")[[i]]-1
            e0 <- e0[na.omit(match(time0,e0[,"time"])),,drop=FALSE]
            time0 <- time0[match(e0[,"time"],time0)]
            dt <- tail(e0,1)
            time0 <- c(time0,dt[,1]+dt[,2])
        }
        chaz0 <- cumsum(e0[,3]*e0[,2])
        tt <- seq(min(time0),max(time0),length.out=length.out)
        ff <- approxfun(time0,c(0,chaz0),method="linear")
        if (type==3) {
            lines(tt,exp(-ff(tt)),lty=lty[i],col=col[i],lwd=lwd[i],...)
        } else if (type==1) {
            lines(tt,log(ff(tt)),lty=lty[i],col=col[i],lwd=lwd[i],...)
        } else {
            lines(time0,e0[,3],lty=lty[i],col=col[i],lwd=lwd[i],...)
        }
    }
    if (legend) {
        lab <- apply(aa,1,function(x) paste(x,collapse=","))
        graphics::legend("bottomleft",lab,lty=lty,col=col,lwd=lwd,bg="white")
    }
    return(aa)    
    ## if (surv) lines(x[,c(1,5)],lty=lty[1],type=type,...)
    ## else lines(x[,c(1,3)],lty=lty[1],type=type,...)
    ## if (ncol(x)>4 && confint && surv) {
    ##     lines(x[,c(1,6)],lty=lty[2],type=type,...)
    ##     lines(x[,c(1,7)],lty=lty[2],type=type,...)
    ## }
}


##' @export
pcif <- function(models,timevar,survival,...,delta=0.01,stop.time,trunc=TRUE) {
    if (!is.list(models)) models <- list(models)
    ee <- lapply(models,eventpois,timevar=timevar,...)
    t0 <- c(0,ee[[1]][,timevar])
    int.len <- diff(t0)
    if (!missing(survival)) {
        G <- c(1,eventpois(survival,...)$surv)        
    } else {
        chaz <- c(0,Reduce("+",lapply(ee,function(x) x$chaz)))
        G <- exp(-chaz)
    }
    rates <- lapply(ee,function(x) x$rate)
    if (missing(stop.time)) stop.time <- max(ee[[1]][,1])
    tt <- seq(0,stop.time,by=delta)
    GG <- approx(x=t0,y=G,xout=tt,method="linear")
    rr <- lapply(rates,function(rate)
        approx(x=t0,y=c(rate,0),xout=tt,method="constant",f=0))    
    FF <- lapply(rr,function(x) {
        cif0 <- c(0,cumsum(x$y*GG$y*delta))
        cif0 <- cif0[-length(cif0)]
    })
    S <- Reduce("+",FF)
    idx <- which(S>1)
    if (trunc & length(idx)>0) {
        for (i in seq_along(FF)) {
            FF[[i]][idx] <- FF[[i]][idx]/S[idx]
        }
    }
    FF0 <- Reduce(cbind,FF); colnames(FF0) <- paste0("F",seq(ncol(FF0)))
    as.data.frame(cbind(time=tt,FF0,G=GG$y))
}

gsub2 <- function(pattern, replacement, x, ...) {
  if (length(pattern)!=length(replacement)) {
      pattern <- rep(pattern, length.out=length(replacement))
  }
  result <- x
  for (i in 1:length(pattern)) {
    result <- sub(pattern[i], replacement[i], result, ...)
  }
  result
}

##' @export
procform <- function(formula=NULL, sep="\\|", nsep=1, return.formula=FALSE, data=NULL,
             no.match=TRUE, regex=FALSE, return.list=TRUE, specials=NULL,...) {# {{{
    res <- NULL
    if (is.null(formula)) {
        res <- colnames(data)
    } else if (is.character(formula)) {
        if (is.null(data)) {
            res <- unique(formula)
        } else {
            yy <-c()
            for (y0 in formula) {
                y0orig <- y0
                if (!regex) y0 <- glob2rx(y0)
                npos <- grep(y0,names(data),perl=mets.options()$regex.perl)
                if (no.match && length(npos)==0) {
                    yy <- union(yy, y0orig)
                } else {
                    yy <- union(yy,names(data)[npos])
                }
            }
            res <- unique(yy)
        }
    }
    if (is.numeric(formula)) res <- colnames(data)[formula]
    if (is.character(res)) {
        if (!return.list) return(res)
        if (return.formula) return(as.formula(paste("~",paste(res,collapse="+"))))
        return(list(response=res,predictor=NULL,filter=NULL))
    }


    ## Add parantheses around quotes if it is not a function call
    if (inherits(formula,"formula")) {

        st <- Reduce(paste,deparse(formula))
        strsplit(st,"\"")
        quotepos <- gregexpr("[\"']",st)[[1]]
        if (quotepos[1]>0) {
            sts <- strsplit(st,"[\"']")[[1]]
            foundsep <- any(grepl("|", sts, fixed=TRUE))
            p <- length(quotepos)
            ##repl <- rep(c("(\"","\")"),p)
            for (i in seq(p/2)*2-1) {
                sts[i] <- paste0(sts[i],"(\"")
                sts[i+1] <- paste0(sts[i+1],"\")")
            }
            ## To handle regular expression entered as strings in the formula, we add a 'filter' expression at the end of the formula
            if (!foundsep) sts <- c(sts,"|1")
            formula <- as.formula(paste(sts,collapse=""))
        }
    }

    aa <- attributes(terms(formula,data=data,specials="regex"))
    if (aa$response == 0) {
        res <- NULL
    } else {
        res <- paste(deparse(formula[[2]]), collapse = "")
    }
    filter.expression <- NULL
    foundsep <- FALSE
    pred <- filter <- c()
    if (!is.null(sep) && length(aa$term.labels) > 0) {
        foundsep <- any(grepl(sep,aa$term.labels))
        if (foundsep) {
            if (nsep>1) {
                xc <- gsub(" ","",unlist(lapply(aa$term.labels, function(z) strsplit(z,sep)[[1]])))
                pred <- xc[1]
                filter <- xc[-1]
            } else {
                xc <- gsub(" ","",unlist(lapply(aa$term.labels, function(z) {
                    spl <- regexpr(sep,z) ## first appearance
                    pred <- substr(z,1,spl-1)
                    filter <- substr(z,spl+1,nchar(z))
                    return(c(pred,filter))
                })))
                pred <- xc[1]
                filter <- xc[2]
            }
            if (any(pred==".")) {
                f <- as.formula(paste0(paste0(c(res,filter),collapse="+"),"~."))
                x <- attributes(terms(f,data=data))$term.labels
                pred <- x
            }
            if (filter%in%c("0","-1")) {
                filter <- list()
                filter.expression <- NULL
            } else {
                filter.expression <- parse(text=filter)
                filter <- as.list(filter)
            }
        }
    }
    if (!foundsep) pred <- aa$term.labels

    expandst <- function(st) {
        st <- res <- unlist(strsplit(gsub(" ","",st),"\\+"))
        if (any(unlist(lapply(st, function(x) grepl("^\\(",x))))) {
            res <- c()
            for (x in st) {
                if (grepl("^\\(",x)) {
                    x <- gsub('\\"',"",x)
                    x <- gsub('^\\(',"",x)
                    x <- gsub('\\)$',"",x)
                    res <- c(res,unlist(procform(x,data=data,regex=regex, no.match=FALSE)$response))
                } else {
                    res <- c(res,x)
                }
                res <- unique(res)
            }
        }
        return(res)
    }
    res <- expandst(res)
    pred <- expandst(pred)
    if (any(res==".")) {
        diffset <- c(".",setdiff(pred,res))
        res <- setdiff(union(res,colnames(data)),diffset)
    }
    filter <- lapply(filter, expandst)
    if (!is.null(specials)) {
        foundspec <- replicate(length(specials),c())
        names(foundspec) <- specials
        rmidx <- c()
        spec <- paste0("^",specials,"\\(")
        val <- lapply(spec, function(x) which(grepl(x,pred)))
        for (i in seq_along(val)) {
            if (length(val[[i]])>0) { # special function found
                rmidx <- c(rmidx,val[[i]])
                cleanpred <- gsub("\\)$","",gsub(spec[i],"",pred[val[[i]]]))
                foundspec[[i]] <- c(foundspec[[i]],cleanpred)
            }
        }
        if (length(rmidx)>0)
            pred <- pred[-rmidx]
        if (length(pred)==0) pred <- NULL
        specials <- foundspec
        for (i in seq_along(specials)) if (is.null(specials[[i]])) specials[i] <- NULL
        if (length(specials)==0) specials <- NULL
    }
    if (return.formula) {
        if (foundsep && !is.null(filter)) {
            filter <- lapply(filter,
                             function(z) as.formula(paste0("~", paste0(z,collapse="+"))))
        }
        if (length(pred)>0)
            pred <- as.formula(paste0("~", paste0(pred,collapse="+"), collapse=""))
        if (length(res)>0)
            res <- as.formula(paste0("~", paste0(res,collapse="+"), collapse=""))
        if (!is.null(specials)) {
            specials <- lapply(specials,function(x)
                              as.formula(paste0("~", paste0(x,collapse="+"), collapse="")))
        }
    }
    res <- list(response=res,
                predictor=pred,
                filter=filter,
                filter.expression=filter.expression,
                specials=specials)
    if (!return.list) return(unlist(unique(res)))
    return(res)
}

##' @export
procformdata <- function(formula,data,sep="\\|", na.action=na.pass, do.filter=TRUE, ...) {
    res <- procform(formula,sep=sep,data=data,return.formula=TRUE,...)
    if (inherits(res,"formula")) {
        res <- list(response=res)
    }
    y <- x <- NULL
    filter <- res$filter.expression
    if (!do.filter) {
        filter <- NULL
    }
    ### when filter.expression is expression(1) then also no filter, ts
    if ((!missing(filter))) if (!is.null(filter)) if (as.character(filter)=="1") filter <- NULL

    if (length(res$response)>0) {
        if (is.null(filter)) y <- model.frame(res$response,data=data,na.action=na.action)
        else y <- model.frame(res$response,data=subset(data,eval(filter)),na.action=na.action)
    }
    if (length(res$predictor)>0) {
        if (is.null(filter)) x <- model.frame(res$predictor,data=data,na.action=na.action)
        else x <- model.frame(res$predictor,data=subset(data,eval(filter)),na.action=na.action)

    }

    specials <- NULL
    if (!is.null(res$specials)) {
        specials <- lapply(res$specials,
                      function(x) {
                          if (is.null(filter)) model.frame(x,data=data,na.action=na.action)
                          else model.frame(x,data=subset(data,eval(filter)),na.action=na.action)
                      })
    }

    if (!do.filter) {
        group <- lapply(res$filter, function(x) model.frame(x,data=data,na.action=na.action))
        return(list(response=y,predictor=x,group=group,specials=specials))
    }
    return(list(response=y,predictor=x,specials=specials))
}# }}}

procform2 <- function(y,x=NULL,z=NULL,...) {# {{{
    yx <- procform(y,return.formula=FALSE,...)
    y <- yx$response
    x0 <- yx$predictor
    z0 <- NULL
    if (length(yx$filter)>0) z0 <- yx$filter[[1]]
    if (is.null(x) && length(y)>0) x <- x0
    if (NCOL(x)==0) x <- NULL
    if (length(y)==0) y <- x0
    if (!is.null(x)) {
        x <- unlist(procform(x,return.formula=FALSE,...))
    }
    if (is.null(z)) z <- z0
    if (!is.null(z)) {
        zz <- unlist(procform(z,return.formula=FALSE,...))
    }
    if (is.null(z)) z <- z0
    return(list(y=y,x=x,z=z))
}# }}}

##' @export
procform3 <- function(y,x=NULL,z=NULL,...) {# {{{
    yx <- procform(y,return.formula=FALSE,...)
    x0 <- yx$predictor
    y  <- yx$response
    if (is.null(yx$predictor)) { x0 <- yx$response ; y <- NULL}

    if (is.null(y))
    if (!is.null(x)) {
        x <- procform(x,return.formula=FALSE,...)
        y <- c(x$predictor,x$response)
    }
    return(list(y=y,x=x0,z=z))
}# }}}


Specials <- function(f,spec,split1=",",split2=NULL,...) {# {{{
  tt <- terms(f,spec)
  pos <- attributes(tt)$specials[[spec]]
  if (is.null(pos)) return(NULL)
  x <- rownames(attributes(tt)$factors)[pos]
  st <- gsub(" ","",x) ## trim
  spec <- unlist(strsplit(st,"[()]"))[[1]]
  res <- substr(st,nchar(spec)+2,nchar(st)-1)
  if (!is.null(split1))
      res <- unlist(strsplit(res,split1))
  res <- as.list(res)
  for (i in seq_along(res)) {
      if (length(grep("~", res[[i]])) > 0) {
          res[[i]] <- as.formula(res[[i]])
      }
  }
  return(structure(res, "name.mf" = x))
}# }}}

decomp.specials <- function (x, pattern = "[()]", sep = ",", ...)
  {
    st <- gsub(" ", "", x)
    if (!is.null(pattern))
      st <- rev(unlist(strsplit(st, pattern, ...)))[1]
    unlist(strsplit(st, sep, ...))
  }


model.extract2 <- function(frame, component) {
  component <- as.character(substitute(component))
  if (component %in% c("response", "offset")) {
    return(do.call(
      model.extract,
      list(frame = frame, component = component)
    ))
  }
  vname <- paste0("(", component, ")")
  if (!(vname %in% names(frame))) {
    regex <- paste0("^", component, "\\(.*\\)$")
    if (any(grepl(regex, names(frame)))) {
      vname <- grep(regex, names(frame))
      if (length(vname) > 1) stop("model.extract2: non-unique component")
    }
  }
  rval <- frame[[vname]]

  if (!is.null(rval)) {
    if (length(rval) == nrow(frame)) {
      names(rval) <- attr(frame, "row.names")
    } else if (is.matrix(rval) && nrow(rval) == nrow(frame)) {
      t1 <- dimnames(rval)
      dimnames(rval) <- list(
        attr(frame, "row.names"),
        t1[[2L]]
      )
    }
  }
  return(rval)
}

# #' Extract design matrix from data.frame and formula
# #' @title Extract design matrix
# #' @param formula formula
# #' @param data data.frame
# #' @param intercept (logical) If FALSE an intercept is not included in the
# #'   design matrix
# #' @param response (logical) if FALSE the response variable is dropped
# #' @param rm_envir Remove environment
# #' @param ... additional arguments (e.g, specials such weights, offsets, ...)
# #' @param specials character vector specifying functions in the formula that
# #'   should be marked as special in the [terms] object
# #' @param specials.call (call) specials optionally defined as a call-type
# #' @param levels a named list of character vectors giving the full set of
# #'   levels
# #'   to be assumed for each factor
# #' @param design.matrix (logical) if FALSE then only response and specials are
# #'   returned. Otherwise, the design.matrix `x` is als part of the returned
# #'   object.
# #' @param na.action method to handle missing data
# #' @return An object of class 'mets.design'
# #' @examples
# #' n <- 1e3
# #' a <- rbinom(n, 1, 0.5)
# #' t <- rweibull(n, shape=0.5, exp(3 + a))
# #' d <- data.frame(time=t, status=TRUE, a=a, id=seq(n), x=rnorm(n))
# #' des <- proc_design(
# #' Surv(time, status) ~ x + strata(a) + cluster(id),
# #'   specials = c("strata", "cluster"),
# #'   data=d)
# #' new <- update_design(des, head(d))
# #' new$strata
# #' new$cluster
# #' new$y
# #' new$x
proc_design <- function(formula, data, ..., # nolint
                        intercept = FALSE,
                        response = TRUE,
                        rm_envir = FALSE,
                        specials = NULL,
                        specials.call = NULL,
                        levels = NULL,
                        design.matrix = TRUE,
                        na.action = na.omit
                        ) {
  if (inherits(data, c("data.table", "tbl_df"))) {
    data <- as.data.frame(data)
  }
  dots <- substitute(list(...))
  if ("subset" %in% names(dots)) {
    stop("subset is not an allowed specials argument for `proc_design`")
  }
  formulaenv <- environment(formula)
  tt <- terms(formula, data = data, specials = specials)
  term.labels <- attr(tt, "term.labels") # predictors

  if (response && inherits(
                    try(model.frame(update(tt, ~1),
                                    data = data,
                                    na.action = na.action),
                        silent = TRUE),
                    "try-error"
                  )) {
    response <- FALSE
  }
  # delete response to generate design matrix when making predictions
  if (!response) tt <- delete.response(tt)

  sterm.list <- c()
  if (length(specials) > 0) {
    des <- attr(tt, "factors")
    for (s in specials) {
      sterm <- rownames(des)[attr(tt, "specials")[[s]]]
      sterm.list <- c(sterm.list, sterm)
    }
    if (length(sterm.list) > 0) {
      # create formula without specials
      if ((nrow(attr(tt, "factors")) - attr(tt, "response")) ==
          length(sterm.list)) {
        # only specials on the rhs, remove everything
        formula <- update(formula, ~1)
      } else {
        # predictors without the specials
        term.labels <- setdiff(
          term.labels,
          unlist(sterm.list)
        )
        ## formula <- update(tt, reformulate(term.labels))
      }
      fst <- lava::trim(paste(deparse(formula), collapse = ""), all = TRUE)
      for (s in sterm.list) {
        fst <- gsub(trim(s, all = TRUE), "", fst, fixed = TRUE)
      }
      fst <- gsub("[\\+]*$", "", fst) # remove potential any trailing '+'
      formula <- as.formula(fst)
      environment(formula) <- formulaenv
    }
  }

  formula0 <- formula
  environment(formula0) <- formulaenv
  if (!response) formula0 <- formula(delete.response(terms(formula)))
  xlev <- levels
  xlev[["response_"]] <- NULL
  if (!design.matrix) { # only extract specials, response
    des <- attr(tt, "factors")
    fs <- update(formula0, ~1)
    if (length(sterm.list) > 0) {
      # formula with only special-terms
      fs <- reformulate(paste(sterm.list, collapse = " + "))
      fs <- update(formula0, fs)
    }
    environment(formula) <- formulaenv
    mf <- model.frame(fs, data=data, na.action = na.action, ...)
  } else { # also extract design matrix
    mf <- model.frame(tt,
                      data = data, ...,
                      xlev = xlev,
                      na.action = na.action,
                      drop.unused.levels = FALSE
                      )
    if (is.null(xlev)) {
      xlev <- .getXlevels(tt, mf)
    }
    xlev0 <- xlev
  }

  y <- NULL
  if (response) {
    y <- tryCatch(
      model.response(mf, type = "any"),
      error = function(...) NULL
    )
    if (is.factor(y) || is.character(y)) {
      ylev <- levels[["response_"]]
      if (!is.null(ylev)) {
        factor(y, levels = ylev)
      } else {
        ylev <- if (is.factor(y)) {
                  levels(y)
                } else if (is.character(y)) {
                  levels(as.factor(y))
                }
        levels[["response_"]] <- ylev
      }
    }
  }

  has_intercept <- attr(tt, "intercept") == 1L
  specials <- union(
    specials,
    names(dots)[-1] # removing "" at first position when calling dots, which
  ) # is a call object

  specials.list <- c()
  specials.var <- c()
  if (length(specials) > 0) {
    for (s in specials) {
      w <- eval(substitute(model.extract2(mf, s), list(s = s)))
      specials.list <- c(specials.list, list(w))
      spec <- Specials(tt, spec = s)
      if (!is.null(unlist(spec))) {
        spec <- structure(unlist(spec), "name.mf" = attr(spec, "name.mf"))
      }
      specials.var <- c(
        specials.var,
        list(spec)
      )
    }
    names(specials.var) <- specials
    names(specials.list) <- specials
    if (length(sterm.list) > 0) {
      if (design.matrix) {
        xlev0[sterm.list] <- NULL
        mf <- model.frame(formula0,
                          data = data, ...,
                          xlev = xlev0,
                          na.action = na.action,
                          drop.unused.levels = FALSE
                          )
      }
    }
  }

  if (!is.null(specials.call)) {
    specials.list2 <- eval(specials.call, data)
    for (n in names(specials.list2)) {
      if (is.null(specials.list[[n]])) {
        specials.list[[n]] <- specials.list2[[n]]
      }
    }
  }

  if (design.matrix) {
    x <- model.matrix(mf, data = structure(data, terms=NULL),
                      xlev = xlev0)
    if (!intercept && has_intercept) {
      has_intercept <- FALSE
      x <- x[, -1, drop = FALSE]
    }
  } else {
    term.labels <- NULL
    x <- NULL
  }

  if (rm_envir) attr(tt, ".Environment") <- NULL
  if (is.null(specials.call)) specials.call <- dots

  xlev[["response_"]] <- levels[["response_"]]
  res <- c(
    list(
      formula = formula, # formula without specials
      terms = tt,
      term.labels = term.labels,
      levels = xlev,
      x = x, y = y,
      design.matrix = design.matrix,
      intercept = has_intercept,
      data = mf,
      ## data[0, ], ## Empty data.frame to capture structure of data
      specials = specials,
      na.action = na.action,
      specials.var = specials.var,
      specials.call = specials.call
    ),
    specials.list
  )
  return(structure(res, class="mets.design"))
}


update_design <- function(object, data = NULL, response = TRUE, ...) {
  if (is.null(data)) data <- object$data
  missing_spec_var <- missing_spec <- c()
  levels <- object$levels
  for (s in names(object$specials.var)) {
    miss <- FALSE
    specials.var <- object$specials.var[[s]]
    if (!is.null(specials.var)) { ## special used in original design object
      for (v in specials.var) {
        # looping through variables used in special-term checking if missing
        if (!v %in% colnames(data)) {
          if (is.null(object$levels[[v]])) {
            data[[v]] <- 0
          } else { # if factor replace with first level
            data[[v]] <- object$levels[[v]][1]
          }
          miss <- TRUE
        }
      }
      if (miss) {
        missing_spec <- c(missing_spec, s)
        missing_spec_var <- c(missing_spec_var, attr(specials.var, "name.mf"))
      }
    }
  }
  if (length(missing_spec) > 0) object$levels[missing_spec_var] <- NULL
  res <- proc_design(object$terms,
    data = data,
    design.matrix = object$design.matrix,
    levels = object$levels,
    response = response,
    na.action = object$na.action,
    intercept = object$intercept,
    specials = object$specials,
    specials.call = object$specials.call
    )
  object$levels <- levels
  res[missing_spec] <- NULL
  return(res)
}


clean_design <- function(object, ...) {
  object$x <- object$x[0, , drop = FALSE]
  object$y <- NULL
  for (i in object$specials) object[[i]] <- NULL
  return(object)
}


##' @export
model.frame.mets.design <- function(formula, data = NULL, ...) {
  if (is.null(data)) return(formula$data)
  model.frame(formula$formula, data = data, ...)
}

## ---- include = FALSE, label=setup--------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
  )
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE") || length(list.files("data"))==0
saveobj <- function(obj, null_elements) {
  x <- get(obj, envir=parent.frame())
  if (length(null_elements)>0) {
    print(object.size(x))
    x[null_elements] <- NULL
    attributes(x)[null_elements] <- NULL
    print(object.size(x))
  }
  saveRDS(x, paste0("data/", obj, ".rds"), compress="xz")
}

library(mets)
 cols <- c("darkred","darkblue","black")
 ltys <- c(1,3,2)
 fig_w <- 5
 fig_h <- 5
 savefig <- TRUE

## ----results="hide", echo=FALSE, eval=!fullVignette---------------------------
## To save time building the vignettes on CRAN, we cache time consuming computations
fitco1 <- readRDS("data/fitco1.rds")
fitco2 <- readRDS("data/fitco2.rds")
fitco3 <- readRDS("data/fitco3.rds")
fitco4 <- readRDS("data/fitco4.rds")
fitace <- readRDS("data/fitace.rds")
fitde <- readRDS("data/fitde.rds")
cse <- readRDS("data/cse.rds")
slr <- readRDS("data/slr.rds")
outacem <- readRDS("data/outacem.rds")
b0 <- readRDS("data/b0.rds")
b1 <- readRDS("data/b1.rds")
b2 <- readRDS("data/b2.rds")
a1 <- readRDS("data/a1.rds")
h2 <- readRDS("data/h2.rds")
concMZ <- readRDS("data/concMZ.rds")
s_mz_country <- readRDS("data/s_mz_country.rds")
s_dz_country <- readRDS("data/s_dz_country.rds")

## ---- label=data-prt----------------------------------------------------------
 library(mets)
 set.seed(122)
 data(prt)
 
 dtable(prt,~status+cancer)
 dtable(prt,~zyg+country,level=1)

## ---- label=survival-marginal-------------------------------------------------
 # Marginal Cox model here stratified on country without covariates 
 margph <- phreg(Surv(time,cancer)~strata(country)+cluster(id),data=prt)
 plot(margph)

## ---- label=survival-pairwise1, eval=fullVignette-----------------------------
#  # Clayton-Oakes, MLE , overall variance
#  fitco1<-twostageMLE(margph,data=prt,theta=2.7)

## -----------------------------------------------------------------------------
 summary(fitco1)

## ---- label=survival-pairwise2, eval=fullVignette-----------------------------
#  fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)

## -----------------------------------------------------------------------------
 summary(fitco2)

## ----label=survival-pairwise3, eval=fullVignette------------------------------
#   mm <- model.matrix(~-1+factor(zyg),prt)
#   fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)

## -----------------------------------------------------------------------------
 summary(fitco3)

## ----label=survival-pairwise4, eval=fullVignette------------------------------
#  fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)

## -----------------------------------------------------------------------------
 summary(fitco4)
 round(estimate(coef=fitco4$coef,vcov=fitco4$var.theta)$coefmat[,c(1,3:4)],2)

 ## mz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[2],0,K=1000)$mz.kendall
 ## dz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[1],0,K=1000)$mz.kendall

## ---- label=survival-polygenic1, eval=fullVignette----------------------------
#   ### setting up design for random effects and parameters of random effects
#   desace <- twin.polygen.design(prt,type="ace")
#  
#   ### ace model
#   fitace <- survival.twostage(margph,data=prt,theta=1,
#         clusters=prt$id,var.link=0,model="clayton.oakes",
#         numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)

## -----------------------------------------------------------------------------
 summary(fitace)

## ----label=survival-polygenic2, eval=fullVignette-----------------------------
#   ### ace model with positive random effects variances
#   # fitacee <- survival.twostage(margph,data=prt,theta=1,
#   #      clusters=prt$id,var.link=1,model="clayton.oakes",
#   #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
#   #summary(fitacee)
#  
#   ### ae model
#   #desae <- twin.polygen.design(prt,type="ae")
#   #fitae <- survival.twostage(margph,data=prt,theta=1,
#   #      clusters=prt$id,var.link=0,model="clayton.oakes",
#   #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
#   #summary(fitae)
#  
#   ### de model
#   desde <- twin.polygen.design(prt,type="de")
#   fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
#  numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)
#  

## -----------------------------------------------------------------------------
summary(fitde)

## -----------------------------------------------------------------------------
prt <-  force.same.cens(prt,cause="status")

 dtable(prt,~status+cancer)
 dtable(prt,~status+country)
 dtable(prt,~zyg+country)

## ---- label=concordance-------------------------------------------------------
 ## cumulative incidence with cluster standard errors.
 cif1 <- cif(Event(time,status)~strata(country)+cluster(id),prt,cause=2)
 plot(cif1,se=1)

 cifa <- cif(Event(time,status)~+1,prt,cause=2)

 ### concordance estimator, ignoring country differences. 
 p11 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2))
p11mz <- p11$model$"MZ"
p11dz <- p11$model$"DZ" 

## -----------------------------------------------------------------------------
 par(mfrow=c(1,2))
 ## Concordance
 plot(p11mz,ylim=c(0,0.1));
 plot(p11dz,ylim=c(0,0.1));

## ---- label=concordance2------------------------------------------------------
 library(prodlim)
 outm <- prodlim(Hist(time,status)~+1,data=prt)

 cifzyg <- cif(Event(time,status)~+strata(zyg)+cluster(id),data=prt,cause=2)
 cifprt <- cif(Event(time,status)~country+cluster(id),data=prt,cause=2)
     
 times <- 70:100
 cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ")) ## cause is 2 (second cause) 
 cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))
    
 ### concordance for MZ and DZ twins<
 cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
 ccdz <- cc$model$"DZ"
 ccmz <- cc$model$"MZ"
     
 cdz <- casewise(ccdz,outm,cause.marg=2) 
 cmz <- casewise(ccmz,outm,cause.marg=2)

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,col=cols[2:1], lty=ltys[2:1], legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-MZ","concordance-DZ"),col=cols[1:2],lty=ltys[1:2])

 plot(cmz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,col=cols[c(1,3,3)],lty=ltys[c(1,3,3)],
      ylab="Casewise",xlab="Age")
  plot(cdz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(cols[2],NA,NA), lty=ltys[c(2,3,3)], add=TRUE)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[1]))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[2]))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=cols, lty=ltys, bg="white")

 summary(cdz)
 summary(cmz)

 Cpred(cmz$casewise,80)
 Cpred(cdz$casewise,80)

## ---- label=concordance3------------------------------------------------------

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-DZ","concordance-MZ"),col=c(1,2),lty=1)
 plot(cmz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,col=c(2,3,3),ylab="Casewise",xlab="Age",lty=c(1,3))
 plot(cdz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(1,3,3), add=TRUE, lty=c(2,3))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=c(2,1,3),lty=1)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=2))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=1))


## ---- label=concordance4, eval=fullVignette-----------------------------------
#   ### new version of Casewise for specific time-point based on binreg
#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
#   newdata <- data.frame(zyg=c("DZ","MZ"),id=1)
#  
#   ## concordance
#   bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
#   pconc <- predict(bcif1,newdata)
#  
#   ## marginal estimates
#   mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
#   mc <- predict(mbcif1,newdata)
#  
#   ### casewise with improved se's from log-scale
#   cse <- binregCasewise(bcif1,mbcif1)

## -----------------------------------------------------------------------------
 cse 

## ---- label=semiparconc, eval=fullVignette------------------------------------
#   ### semi-parametric modelling of concordance
#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
#   regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
#   regconc
#   ### interaction test
#   wald.test(regconc,coef.null=5:7)
#  
#   regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
#   regconc
#  
#   ## logistic link
#   logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
#   slr <- summary(logitregconc)

## -----------------------------------------------------------------------------
slr
### library(Publish)
### publish(round(slr$exp.coef[,-c(2,5)],2),latex=TRUE,digits=2)

## ---- label=additive_gamma, eval=fullVignette---------------------------------
#    times <- seq(50,90,length.out=5)
#    cif1 <- comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
#  		   cause=2,times=times,max.clust=NULL)
#  
#    mm <- model.matrix(~-1+factor(zyg),prt)
#    out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
#  		  theta.des=mm,same.cens=TRUE,step=0.5)
#    summary(out1)
#    round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)
#  
#    desace <- twin.polygen.design(prt,type="ace")
#  
#    outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
#    	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
#           step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
#    ##outacem$score

## -----------------------------------------------------------------------------
  summary(outacem)

 ###  variances
 estimate(coef=outacem$theta,vcov=outacem$var.theta,f=function(p) p/sum(p)^2)

 ## AE polygenic model
 # desae <- twin.polygen.design(prt,type="ae")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #    same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
 #        step=0.5,theta.des=desae$pardes,random.design=desae$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p)     p/sum(p)^2)

 ## AE polygenic model
 # desde <- twin.polygen.design(prt,type="de")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #   same.cens=TRUE,theta=c(0.35),var.link=0,
 #   step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

  times <- 90
  cif1 <- comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

 desde <- twin.polygen.design(prt,type="de")
 outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
	same.cens=TRUE,theta=c(0.35),var.link=0,
        step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 outaem$score
 summary(outaem)
 estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)


## ---- label=probit1-----------------------------------------------------------
rm(prt)
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in

## ----b0, eval=fullVignette----------------------------------------------------
#  b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)

## -----------------------------------------------------------------------------
summary(b0)

## ---- label=liability_ace1, eval=fullVignette---------------------------------
#  b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)

## -----------------------------------------------------------------------------
summary(b1)

## -----------------------------------------------------------------------------
AIC(b0, b1)

## ---- label=liability_ace_country, eval=fullVignette--------------------------
#  b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg+country, breaks=95)

## -----------------------------------------------------------------------------
summary(b2)

## ---- label=bptime1, eval=fullVignette----------------------------------------
#  bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
#                cens.formula=Surv(time,status==0)~zyg,
#                summary.function=function(x) x, breaks=tt)
#  h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
#  concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))
#  

## -----------------------------------------------------------------------------
par(mfrow=c(1,2))
plot(tt, h2[,1], type="s", lty=1, col=cols[3], xlab="Age", ylab="Heritability", ylim=c(0,1))
confband(tt, h2[,2], h2[,3],polygon=TRUE, step=TRUE, col=Col(cols[3], 0.1), border=NA)
plot(tt, concMZ[,1], type="s", lty=1, col=cols[1], xlab="Age", ylab="Concordance", ylim=c(0,.1))
confband(tt, concMZ[,2], concMZ[,3],polygon=TRUE, step=TRUE, col=Col(cols[1], 0.1), border=NA)

## ---- label=biprobittime1-----------------------------------------------------
system.time(a.mz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~1, pairs.only=TRUE,
                                breaks=tt))
system.time(a.dz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~1, pairs.only=TRUE,
                               breaks=tt))

#system.time(a.zyg <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, 
#                               cens.formula = Event(time,status==0)~1,
#                               eqmarg=FALSE, fix.cens.weight
#                               breaks=seq(75,100,by=10)))

a.mz
a.dz

plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance", ylim=c(0,0.07))
plot(a.mz, ylim=c(0,.07), col=cols[1], lty=ltys[1], legend=FALSE, add=TRUE)
plot(a.dz, col=cols[2], lty=ltys[2], add=TRUE)

## ---- label=biprobittime2, eval=fullVignette----------------------------------
#  a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
#                                 cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
#                                  breaks=tt)
#  system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
#                                 cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
#                                 breaks=tt))
#  
#  s_mz_country <- summary(a.mz_country)
#  s_dz_country <- summary(a.dz_country)

## -----------------------------------------------------------------------------
s_mz_country
s_dz_country

## ---- label=liability_ace_time1, eval=fullVignette----------------------------
#  ## ACE model (time-varying) with and without adjustment for country
#  a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
#                                zyg="zyg", DZ="DZ",
#                                cens.formula=Surv(time,status==0)~zyg,
#                                breaks=tt)
#  
#  #a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
#  #                              zyg="zyg", DZ="DZ",
#  #                              #cens.formula=Surv(time,status==0)~country+zyg,
#  #                              breaks=tt)

## -----------------------------------------------------------------------------
plot(a.mz, which=c(6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1], lty=ltys[1], legend=NULL, alpha=.1)
plot(a.dz, which=c(6), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, alpha=.1)
legend("topleft", c("MZ tetrachoric correlation", "DZ tetrachoric correlation"),
       col=cols, lty=ltys, lwd=2)

plot(a.mz, which=c(4), xlab="Age", ylab="Relative Recurrence Risk",
     ylim=c(1,20), col=cols[1], lty=ltys[1], legend=NULL, lwd=2, alpha=.1)
plot(a.dz, which=c(4), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, lwd=2, alpha=.1)
legend("topright", c("MZ relative recurrence risk", "DZ relative recurrence risk"),
       col=cols, lty=ltys, lwd=2)

plot(a1, which=c(5,6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1:2], lty=ltys[1:2], lwd=2, alpha=0.1,
     legend=c("MZ tetrachoric correlation", "DZ tetrachoric correlation"))

plot(a1, which=c(1), xlab="Age", ylim=c(0,1), col="black", lty=1, ylab="Heritability", legend=NULL, alpha=.1)

## -----------------------------------------------------------------------------
sessionInfo()

## ----saveobj, results="hide", echo=FALSE, eval=fullVignette-------------------
#  ## To save time building the vignettes on CRAN, we cache time consuming computations
#  
#  rms <- c('id','theta.iid','theta.des','marginal.trunc',
#    'loglikeiid','marginal.surv','theta.iid.naive',
#    'antclust','secluster','cluster.call','trunclikeiid',
#    'logl.iid','score.iid','clusters')
#  tmp <- lapply(as.list(paste0("fitco", 1:4)),
#                function(x) saveobj(x, rms))
#  
#  rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
#  saveobj("fitace", rms)
#  saveobj("fitde", rms)
#  
#  saveobj("cse", NULL)
#  saveobj("slr", NULL)
#  
#  rms <- c("theta.iid", "Clusters", "p11")
#  saveobj("outacem", rms)
#  
#  rms <- c("model.frame", "score", "id", "logLik")
#  saveobj("b0", rms)
#  saveobj("b1", rms)
#  saveobj("b2", rms)
#  saveobj("a1", rms)
#  saveobj("h2", NULL)
#  saveobj("concMZ", NULL)
#  saveobj("s_mz_country", NULL)
#  saveobj("s_dz_country", NULL)

##' Computes mediation weights 
##'
##' Computes mediation weights for either binary or multinomial mediators.
##' The important part is that the influence functions can be obtained to compute standard errors. 
##'
##' @param fit either glm-binomial or mlogit (mets package) 
##' @param data data frame with data
##' @param var is NULL reads mediator and exposure from formulae in the fit.
##' @param name.weight name of weights
##' @param id.name name of id variable, important for SE computations
##' @param ... Additional arguments to 
##' @author Thomas Scheike
##' @export
medweight <- function(fit,data=data,var=NULL,name.weight="weights",id.name="id",...)
{# {{{
vvars <- all.vars(fit$formula)
fexp <- data[,vvars[2]]
fmed <- data[,vvars[1]]

if (is.null(var))  { 
if (!is.factor(fexp)) stop("exposure variable must be a factor\n")
 var <- vvars[2] } else {
if (!is.factor(data[,var])) stop("first predictor of weightmodel is exposure variable and must be a factor\n")
}
if (!is.factor(fmed)) stop("mediator variable must be a factor\n")

nlev <- nlevels(fexp)
levexp <- levels(fexp)
ss <- as.numeric(fexp)
ind <- rep(1:nrow(data),each=nlev)
varn <- paste(var,0:1,sep="")
data[,varn[1]] <- data[,var]

wdata <- data[ind,]
mlev <- nlevels(fmed)
ovar <- c(1+t(outer(ss,(nlev-1):0,FUN="+") %%nlev))
wdata[,varn[2]]  <- factor(ovar,labels=levels(fexp))
## set original variable to a going through exposure levels to use predict
wdata[,var] <- wdata[,varn[2]]  

## predicting using predict function
if (inherits(fit,"glm")) { 
	pp1 <- predict(fit,wdata,type="response"); 
	y <- as.numeric(wdata[,vvars[1]])-1
	pp <- pp1*y + (1-pp1)*(1-y)
	wdata[,"basep1"] <- pp
} else if (inherits(fit,"mlogit")) pp <- predictmlogit(fit,wdata,se=0) 
else stop("Considers only binary or multinomial regression modelss glm mlogit \n");

countid <- rep(1:nlev,nrow(data))
weight0 <- rep(pp[countid==1],each=nlev)
weight1 <- pp/weight0
wdata[,name.weight] <- weight1
wdata[,id.name] <- rep(data[,id.name],each=nlev)

attr(wdata,"exposure") <- vvars[[2]]
attr(wdata,"mediator") <- vvars[[1]]
attr(wdata,"nlev-exposure") <- nlev 
attr(wdata,"countid") <- countid
return(wdata)
}# }}}

##' Mediation analysis in survival context 
##'
##' Mediation analysis in survival context  with robust standard errors taking the weights into account
##' via influence function computations. Mediator and exposure must be factors.  This is based on numerical
##' derivative wrt parameters for weighting.  See vignette for more examples. 
##'
##' @param survmodel with mediation model (binreg, aalenMets, phreg) 
##' @param weightmodel mediation model
##' @param data for computations
##' @param wdata weighted data expansion for computations
##' @param id name of id variable, important for SE computations
##' @param silent to be silent
##' @param ... Additional arguments to survival model 
##' @author Thomas Scheike
##' @examples
##' 
##' n <- 400
##' dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
##'           beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
##'     treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
##' dfactor(dat) <- dnr.f~dnr
##' dfactor(dat) <- gp.f~gp
##' drename(dat) <- ttt24~"ttt24*"
##' dat$id <- 1:n
##' dat$ftime <- 1
##' 
##' weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
##' wdata <- medweight(fit,data=dat)
##'
##' ### fitting models with and without mediator
##' aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
##' aaMss22 <- binreg(Event(time,status)~dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
##'
##' ### estimating direct and indirect effects (under strong strong assumptions) 
##' aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),
##'                 data=wdata,time=50,weights=wdata$weights,cause=2)
##' ## to compute standard errors , requires numDeriv
##' library(numDeriv)
##' ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
##' summary(ll)
##' ## not run bootstrap (to save time)
##' ## bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=500)
##'
##' @aliases  BootmediatorSurv 
##' @export
mediatorSurv <- function(survmodel,weightmodel,data=data,wdata=wdata,id="id",silent=TRUE,...)
{# {{{
if (is.null(weightmodel$iid)) {
idvar <- data[,id]
iidp <- iid(weightmodel,id=idvar)
ihessian <- attr(iidp,"bread")
} else { iidp <- weightmodel$iid; ihessian <- weightmodel$ihessian }

if (is.null(survmodel$iid)) iids <- iid(survmodel) else iids <- survmodel$iid

evar <- all.vars(weightmodel$formula)[2]
mvar <- all.vars(weightmodel$formula)[1]
fexp <- data[,evar]
nlev <- nlevels(fexp)
countid <- rep(1:nlev,nrow(data))

DPbetaScoreA <- function(p) {# {{{
if (inherits(weightmodel,"glm")) {
weightmodel$coefficients <- p
pp1 <- predict(weightmodel,wdata,type="response")
y <- as.numeric(wdata[,mvar])-1
pp <- pp1*y + (1-pp1)*(1-y)
} else if (inherits(weightmodel,"mlogit"))  {
weightmodel$coef <- p
pp <- predictmlogit(weightmodel,wdata,se=0)
} else stop("must be glm or mlogit weights\n") 

countid <- attr(wdata,"countid")
weight0 <- rep(pp[countid==1],each=nlev)
weight1 <- pp/weight0

model <- tail(class(survmodel),1)
## starting values coef(survmodel) and ... arguments + new weights 
ll <- c(list(formula=survmodel$formula),list(data=wdata),list(weights=weight1),list(...))
## add time variable for binreg
if (model=="binreg")  ll <- c(ll,list(time=survmodel$time),list(cause=survmodel$cause),list(cens.code=survmodel$cens.code))
## fine-gray or logit link
if (model=="cifreg")  ll <- c(ll,list(propodds=survmodel$propodds),list(cause=survmodel$cause),list(cens.code=survmodel$cens.code))
## do not optimize, just compute score for these weights at fixed survival parameter 
if (model!="aalenMets")  ll <- c(ll,list(no.opt=TRUE))
if (model!="aalenMets")  ll <- c(ll,list(beta=coef(survmodel)))

## evaluate score for the new weights 
mscore <- do.call(model,ll)
if (model=="aalenMets")  {
	## compute score for weight change  with fixed parameter
	score <- mscore$intZHdN - mscore$intZHZ %*% coef(survmodel)
} else score <- mscore$gradient

return(score)
}# }}}

###iiDD <- t(iids) %*% ( as.matrix(iidp)  %*% solve(ihessian))
###print(iiDD)
if (!silent) cat("start numerical derivative wrt weight-parameters\n")
DD <- numDeriv::jacobian(DPbetaScoreA,coef(weightmodel)) # /nrow(data)
iiDD <- survmodel$ihessian %*% DD
iid.w <-    iidp  %*% t(iiDD)
iid.tot <- iids +  iid.w
iid.tot2 <- iids -  iid.w
robvar <- crossprod(iid.tot)
robvar2 <- crossprod(iid.tot2)

out <- list(coef=survmodel$coef,iid=iid.tot,
	    var=robvar,se=diag(robvar)^.5,
	    var2=robvar2,se2=diag(robvar2)^.5,
       n=survmodel$n,ncluster=survmodel$ncluster,nevent=survmodel$nevent,iDPbeta=iiDD,
       iid.w=iid.w,iid.surv=iids)
   class(out) <- c("binreg","medSurv")
   return(out)
}# }}}

##' @export
BootmediatorSurv <- function(survmodel,weightmodel,data=data,id="id",silent=TRUE,k.boot=100,...)
{# {{{

p <- length(coef(survmodel))

mediatorAn <- function(bdata,...) 
{# {{{

if (inherits(weightmodel,"glm")) {
bfit <- suppressWarnings(glm(weightmodel$formula,data=bdata,family=binomial))
} else if (inherits(weightmodel,"mlogit"))  {
bfit <- mlogit(weightmodel$formula,data=bdata)
} else stop("must be glm or mlogit weights\n") 

wdatab <- medweight(bfit,bdata)

model <- tail(class(survmodel),1)
ll <- c(list(formula=survmodel$formula),list(data=wdatab),list(weights=wdatab$weights),list(...))
if (model=="binreg")  ll <- c(ll,list(time=survmodel$time),list(cause=survmodel$cause),list(cens.code=survmodel$cens.code))
if (model=="cifreg")  ll <- c(ll,list(propodds=survmodel$propodds),list(cause=survmodel$cause),list(cens.code=survmodel$cens.code))

coefb <- do.call(model,ll)$coef
return(coefb)
}# }}}

parb <- c()
if (k.boot>=1)  {
for (i in 1:k.boot) {
bdata <- blocksample(data,nrow(data),idvar=id)
parb <- rbind(parb,mediatorAn(bdata,...))
}
bvar <- cov(parb)
} else bvar <- matrix(0,p,p)

out <- list(coef=survmodel$coef,var=bvar,se.coef=diag(bvar)^.5,parb=parb,
       n=survmodel$n,ncluster=survmodel$ncluster,nevent=survmodel$nevent)
   class(out) <- c("binreg","medSurv")
   return(out)
}# }}}

##' GOF for Cox PH regression
##'
##' Cumulative score process residuals for Cox PH regression
##' p-values based on Lin, Wei, Ying resampling.
##' @param object is phreg object 
##' @param n.sim number of simulations for score processes
##' @param silent to show timing estimate will be produced for longer jobs
##' @param robust to control wether robust dM_i(t) or dN_i  are used for simulations
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike and Klaus K. Holst
##' @export
##' @aliases gof.phreg 
##' @examples
##' library(mets)
##' data(sTRACE)
##' 
##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes,data=sTRACE) 
##' gg <- gof(m1)
##' gg
##' par(mfrow=c(1,3))
##' plot(gg)
##' 
##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes,data=sTRACE) 
##' ## to get Martingale ~ dN based simulations
##' gg <- gof(m1)
##' gg
##' 
##' ## to get Martingale robust simulations, specify cluster in  call 
##' sTRACE$id <- 1:500
##' m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes+cluster(id),data=sTRACE) 
##' gg <- gof(m1)
##' gg
##' 
##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes+cluster(id),data=sTRACE) 
##' gg <- gof(m1)
##' gg
##' @export
gof.phreg  <- function(object,n.sim=1000,silent=1,robust=NULL,...)
{# {{{

### test for proportionality 
p <- length(object$coef)
nnames <- names(object$coef)
ii <- solve(object$hessian)
jumptimes <- object$jumptimes
Pt <- object$hessianttime
U <-  object$U
Pt <- apply(Pt,2,cumsum)
Ut <- apply(U,2,cumsum)

nd <- nrow(object$U)
Pt <-  .Call("XXMatFULL",Pt,p,PACKAGE="mets")$XXf
Pt <-  .Call("CubeMat",Pt,ii,PACKAGE="mets")$XXX
sup <- matrix(0,n.sim,nrow(ii))
hatti <- matrix(0,nd,nrow(ii))
obs <- apply(abs(Ut),2,max)

if (is.null(robust)) 
   if (!is.null(object$call.id)) robust <- TRUE else robust <- FALSE

   ### cluster call or robust \hat M_i(t) based  
if (robust) {# {{{
   xx <- object$cox.prep
   Z <- xx$X
   rrw <- c(xx$sign * exp(Z %*% coef(object) + xx$offset)*xx$weights)
   UdN <- xx$weights[xx$jumps+1]*object$U 
   ### also weights 
   nn <- nrow(Z)

   tt <- system.time(simcox1<- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
	     object$S0,object$E,
             10,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps))

   prt <- n.sim*tt[3]/(10*60)
   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
   simcox <- .Call("PropTestCoxClust",UdN,Pt,rrw,xx$X,
	     object$S0,object$E,
	     n.sim,obs,nn,xx$id,xx$strata,xx$nstrata,object$strata.jumps,xx$jumps)
} else {# }}}
###  or dN_i based  # {{{
   tt <- system.time(simcox1<-.Call("PropTestCox",U,Pt,10,obs,PACKAGE="mets"))
   prt <- n.sim*tt[3]/(10*60)
   if (prt>1 & silent==0) cat(paste("Predicted time minutes",signif(prt,2),"\n"))
   simcox <-  .Call("PropTestCox",U,Pt,n.sim,obs,PACKAGE="mets")
}# }}}

sup <-  simcox$supUsim
res <- cbind(obs,simcox$pval)
colnames(res) <- c("Sup|U(t)|","pval")
rownames(res) <- nnames 

if (silent==0) {
cat("Cumulative score process test for Proportionality:\n")
prmatrix(round(res,digits=2))
}

out <- list(jumptimes=object$jumptimes,supUsim=sup,res=res,supU=obs,
    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,type="prop",robust=robust)
class(out) <- "gof.phreg"
return(out)
}# }}}


##' GOF for Cox covariates in  PH regression
##'
##' Cumulative residuals after model matrix for Cox PH regression
##' p-values based on Lin, Wei, Ying resampling.
##'
##' That is, computes 
##' \deqn{
##'  U(t) = \int_0^t M^t d \hat M 
##' }
##' and resamples its asymptotic distribution. 
##'
##' This will show if the residuals are consistent with the model. Typically,
##' M will be a design matrix for the continous covariates that gives for example
##' the quartiles, and then the plot will show if for the different quartiles of the covariate the risk
##' prediction is consistent over time  (time x covariate interaction).
##'
##' @param formula formula for cox regression 
##' @param data data for model
##' @param offset offset 
##' @param weights weights 
##' @param modelmatrix  matrix for cumulating residuals
##' @param n.sim number of simulations for score processes
##' @param silent to keep it absolutely silent, otherwise timing estimate will be prduced for longer jobs.
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike and Klaus K. Holst
##' @export
##' @examples
##' library(mets)
##' data(TRACE)
##' set.seed(1)
##' TRACEsam <- blocksample(TRACE,idvar="id",replace=FALSE,100)
##' 
##' dcut(TRACEsam)  <- ~. 
##' mm <- model.matrix(~-1+factor(wmicat.4),data=TRACEsam)
##' m1 <- gofM.phreg(Surv(time,status==9)~vf+chf+wmi,data=TRACEsam,modelmatrix=mm)
##' summary(m1)
##' if (interactive()) {
##' par(mfrow=c(2,2))
##' plot(m1)
##' }
##' 
##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,modelmatrix=mm) 
##' summary(m1)
##' 
##' ## cumulative sums in covariates, via design matrix mm 
##' mm <- cumContr(TRACEsam$wmi,breaks=10,equi=TRUE)
##' m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,
##' 		  modelmatrix=mm,silent=0)
##' summary(m1)
##' 
##' @export
gofM.phreg  <- function(formula,data,offset=NULL,weights=NULL,modelmatrix=NULL,
			n.sim=1000,silent=1,...)
{# {{{

if (is.null(modelmatrix)) stop(" must give matrix for cumulating residuals\n"); 

cox1 <- phreg(formula,data,offset=NULL,weights=NULL,Z=modelmatrix,cumhaz=FALSE,...) 
offsets <- cox1$X %*% cox1$coef
if (!is.null(offset)) offsets <- offsets*offset
if (!is.null(cox1$strata.name)) 
     coxM <- phreg(cox1$model.frame[,1]~modelmatrix+strata(cox1$strata),data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,no.var=1,...)
else coxM <- phreg(cox1$model.frame[,1]~modelmatrix,data,offset=offsets,weights=weights,no.opt=TRUE,cumhaz=FALSE,no.var=1,...)
nnames <- colnames(modelmatrix)

Ut <- apply(coxM$U,2,cumsum)
jumptimes <- coxM$jumptimes
U <- coxM$U
Ubeta <- cox1$U
ii <- -solve(cox1$hessian)
EE <- .Call("vecMatMat",coxM$E,cox1$E,PACKAGE="mets")$vXZ; 
Pt <- cox1$ZX - EE
Pt <- apply(Pt,2,cumsum)
betaiid <- t(ii %*% t(Ubeta))
obs <- apply(abs(Ut),2,max)
simcox <-  .Call("ModelMatrixTestCox",U,Pt,betaiid,n.sim,obs,PACKAGE="mets")

sup <-  simcox$supUsim
res <- cbind(obs,simcox$pval)
colnames(res) <- c("Sup_t |U(t)|","pval")
rownames(res) <- nnames 

if (silent==0) {
   cat("Cumulative score process test for modelmatrix:\n")
   prmatrix(round(res,digits=2))
}

 ## pvals efter z i model.matrix sup_z | M(z,tau) | 
 Utlast <- max(abs(tail(Ut,1)))
 maxlast <- apply(abs(simcox$last),1,max)
 pval.last <- mean(maxlast>=Utlast)
 res.last <- matrix(c(Utlast,pval.last),1,2)
 colnames(res.last) <- c("Sup_z |U(tau,z)|","pval")
 rownames(res.last) <- "matrixZ"

out <- list(jumptimes=jumptimes,supUsim=simcox$supUsim,res=res,supU=obs,
	    pvals=simcox$pval,score=Ut,simUt=simcox$simUt,
	    simUtlast=simcox$last,Utlast=Utlast,pval.last=pval.last,
	    res.last=res.last, type="modelmatrix")
class(out) <- "gof.phreg"

return(out)
}# }}}

##' GOF for Cox covariates in  PH regression
##'
##' That is, computes 
##' \deqn{
##'  U(z,\tau) = \int_0^\tau M(z)^t d \hat M 
##' }
##' and resamples its asymptotic distribution. 
##'
##' This will show if the residuals are consistent with the model evaulated in the z covariate. 
##' M is here chosen based on a grid (z_1, ..., z_m) and the different columns are \eqn{I(Z_i \leq z_l)}.
##' for \eqn{l=1,...,m}. 
##' The process in z is resampled to find extreme values.  The time-points of evuluation is by default
##' 50 points, chosen as 2%,4%,..., percentiles of the covariates.
##'
##' The p-value is valid but depends on the chosen grid. When the number of break points are high
##' this will give the orginal test of Lin, Wei and Ying for linearity, that is also computed in 
##' the timereg package. 
##'
##' @param formula formula for cox regression 
##' @param data data for model
##' @param vars which variables to test for linearity 
##' @param offset offset 
##' @param weights weights 
##' @param breaks number of breaks for cumulatives in covarirate direction
##' @param equi equidistant breaks  or not 
##' @param n.sim number of simulations for score processes
##' @param silent to keep it absolutely silent, otherwise timing estimate will be prduced for longer jobs.
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike and Klaus K. Holst
##' @export
##' @examples
##' library(mets)
##' data(TRACE)
##' set.seed(1)
##' TRACEsam <- blocksample(TRACE,idvar="id",replace=FALSE,100)
##' 
##' ## cumulative sums in covariates, via design matrix mm
##' \donttest{ ## Reduce Ex.Timings
##' m1 <- gofZ.phreg(Surv(time,status==9)~strata(vf)+chf+wmi+age,data=TRACEsam)
##' summary(m1) 
##' plot(m1,type="z")
##' }
##' @aliases cumContr 
##' @export
gofZ.phreg  <- function(formula,data,vars=NULL,offset=NULL,weights=NULL,breaks=50,equi=FALSE,
			n.sim=1000,silent=1,...)
{# {{{
if (is.null(vars)) {
     vars <- NULL
      ## find strata var's 
      yxzf <- procform(formula,x=NULL,z=NULL,data=data,do.filter=FALSE)
      avars <- all.vars(formula[-2])
      svar <- grep("strata",yxzf$predictor)
    if (length(svar)>=1) {
	avars <- avars[-svar]
     }
     ## check that it is not a factor and that there are more than 2 levels
     for (vv in avars) {
	  if (length(unique(data[,vv]))>2 & !is.factor(data[,vv]))
	  vars  <-  c(vars,vv)
     }
 }

 gres <- list()
 res <- matrix(0,length(vars),2)
 colnames(res) <- c("Sup_z |U(tau,z)|","pval")
 rownames(res) <- vars

i <- 1
for (vv in vars) {
 modelmatrix <- cumContr(data[,vv],breaks=breaks,equi=equi)
 lres <- gofM.phreg(formula,data,modelmatrix=modelmatrix) 
 lres$xaxs <- attr(modelmatrix,"breaks")

 res[i,] <- c(lres$Utlast,lres$pval.last)
 i <- i+1

 lres <- list(lres)
 names(lres) <- vv
 gres <- c(gres,lres)
}

out <- list(res=res,Zres=gres,type="Zmodelmatrix")
class(out) <- c("gof.phreg")

return(out)
}# }}}


##' @export
cumContr <- function(data,breaks=4,probs=NULL,equi=TRUE,na.rm=TRUE,unique.breaks=TRUE,...)
 {# {{{
 if (is.vector(data)) {
        if (is.list(breaks))
            breaks <- unlist(breaks)
        if (length(breaks) == 1) {
            if (!is.null(probs)) {
                breaks <- quantile(data, probs, na.rm = na.rm,...)
	        breaks <- breaks[-1]
            }
            else {
                if (!equi) {
                  probs <- seq(0, 1, length.out = breaks + 1)
                  breaks <- quantile(data, probs, na.rm = na.rm, ...)
	          if (unique.breaks) breaks <- unique(breaks)
	          breaks <- breaks[-1]
                }
                if (equi) {
                  rr <- range(data, na.rm = na.rm)
                  breaks <- seq(rr[1], rr[2], length.out = breaks + 1)
	          breaks <- breaks[-1]
                }
            }
        }
        if (sum(duplicated(breaks)) == 0) {
		i <- 0; 
		gm <- matrix(0,length(data),length(breaks))
            for (bb in breaks)  {
		    i <- i+1 
		    gm[,i] <- (data <= bb)*1
	    }
	} else {
            wd <- which(duplicated(breaks))
            mb <- min(diff(breaks[-wd]))
            breaks[wd] <- breaks[wd] + (mb/2) * seq(length(wd))/length(wd)
            i <- 0; gm <- matrix(0,length(data),length(breaks))
            for (bb in breaks)  {
		    i <- i+1 
		    gm[,i] <- (data <= bb)*1
	    }
            warning(paste("breaks duplicated"))
        }
        colnames(gm) <- paste("<=",breaks,sep="")
	attr(gm,"breaks") <- breaks
        return(gm)
    }
 }# }}}

##' Stratified baseline graphical GOF test for Cox covariates in PH regression
##'
##' Looks at stratified baseline in Cox model and plots all baselines versus each
##' other to see if lines are straight, with 50 resample versions under the 
##' assumptiosn that the stratified Cox is correct 
##'
##' @param x phreg object
##' @param sim to simulate som variation from cox model to put on graph
##' @param silent to keep it absolutely silent 
##' @param lm addd line to plot, regressing the cumulatives on each other  
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike and Klaus K. Holst
##' @export
##' @examples
##' data(tTRACE)
##' 
##' m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=tTRACE) 
##' m2 <- phreg(Surv(time,status==9)~vf+strata(chf)+wmi,data=tTRACE) 
##' par(mfrow=c(2,2))
##' 
##' gofG.phreg(m1)
##' gofG.phreg(m2)
##' 
##' bplot(m1,log="y")
##' bplot(m2,log="y")
##' @export
gofG.phreg  <- function(x,sim=0,silent=1,lm=TRUE,...)
{# {{{

p <- length(x$coef)
nnames <- names(x$coef)
strata <- x$strata[x$jumps]
nstrata <- x$nstrata
jumptimes <- x$jumptimes
cumhaz <- x$cumhaz

ms <- match(x$strata.name,names(x$model.frame))
lstrata <- levels(x$model.frame[,ms])
stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
stratnames <- paste(stratn,lstrata,sep=":")

if (is.null(cumhaz)) stop("Must run phreg with cumhaz=TRUE (default)"); 
if (nstrata==1) stop("Stratified Cox to look at baselines");

if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0

for (i in 0:(nstrata-2))
for (j in (i+1):(nstrata-1)) { 
      iij <- which(strata %in% c(i,j))
      ii <- which(strata %in% i)
      ij <- which(strata %in% j)
      dijjumps  <- jumptimes[iij] 
      cumhazi <- cpred(cumhaz[ii,],dijjumps)
      cumhazj <- cpred(cumhaz[ij,],dijjumps)

      plot(cumhazj[,2],cumhazi[,2],type="s",lwd=2,xlab=stratnames[j+1],ylab=stratnames[i+1])
      graphics::title(paste("Stratified baselines for",stratn))
      if ((fixbeta==0 | sim==0) & lm ) 
      graphics::legend("topleft",c("Nonparametric","lm"),lty=1,col=1:2)
      ab <- lm(cumhazi[,2]~-1+cumhazj[,2])
      if (sim==1 & fixbeta==0) {
             Pt <- DLambeta.t <- apply(x$E/c(x$S0),2,cumsumstrata,strata,nstrata)
             II <- -solve(x$hessian)
             betaiid <- t(II %*% t(x$U))
	     simband <-  .Call("simBandCumHazCox",1/x$S0,Pt,betaiid,50,rep(1,nrow(Pt)),PACKAGE="mets")
	     simU <-simband$simUt
	     for (k in 1:50)
	     {
	      di <- cpred(cbind(jumptimes[ii],simU[ii,k]),dijjumps)[,2]
	      dj <- cpred(cbind(jumptimes[ij],simU[ij,k]),dijjumps)[,2]
	      lines(cumhazj[,2]+dj,cumhazi[,2]+di,type="s",lwd=0.1,col=3)
	     }
      }
      lines(cumhazj[,2],cumhazi[,2],type="s",lwd=2,col=1)
      if (lm==TRUE) abline(c(0,coef(ab)),col=2,lwd=2)
}

}# }}}

##' @export
plot.gof.phreg <-  function(x,col=3,type=NULL,...)
{# {{{

if (is.null(type)) {
	if (x$type=="prop") type <- "time"
	if (x$type=="modelmatrix" ) type <- "modelmatrix"
	if (x$type=="Zmodelmatrix") type <- "z"
}

if (type=="time" || type=="modelmatrix") {
p <- ncol(x$score)
for (i in 1:p)
{
simU <- x$simUt[,(0:49)*p+i]
rsU <- max(abs(simU))
rsU <- max(rsU,abs(x$score[,i]))
plot(x$jumptimes,x$score[,i],type="s",ylim=c(-rsU,rsU),xlab="",ylab="")
title(main=rownames(x$res)[i])
matlines(x$jumptimes,simU,type="s",lwd=0.3,col=col)
lines(x$jumptimes,x$score[,i],type="s",lwd=1.5)
}
} else {
	if (type=="modelmatrix") {
		obsz <- c(tail(x$score,1))
		times <- 1:length(obsz)
		rsU <- max(max(abs(obsz)),max(abs(x$simUtlast[1:50,])))
		plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
		matlines(times,t(x$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
	        ## redraw with thick to make observed clear 
	        lines(times,obsz,lwd=2,col=1)
	} else {
	   for (i in 1:length(x$Zres))
	   {
	    xr <- x$Zres[[i]]
	    obsz <- c(tail(xr$score,1))
	    times <- xr$xaxs
	    rsU <- max(max(abs(obsz)),max(abs(xr$simUtlast[1:50,])))
	    plot(times,obsz,type="l",ylim=c(-rsU,rsU),xlab="",ylab="")
	    title(main=rownames(x$res)[i])
	    matlines(times,t(xr$simUtlast[1:50,]),type="l",lwd=0.3,col=col)
	    ## redraw with thick to make observed clear 
	    lines(times,obsz,lwd=2,col=1)
           }
	}
}

}# }}}

##' @export
summary.gof.phreg <-  function(object,...)
{# {{{
if (object$type=="prop")
     cat("Cumulative score process test for Proportionality:\n")
else cat("Cumulative residuals versus modelmatrix :\n")
print(object$res)

if (!is.null(object$res.last)) {
   cat("\n")
   cat("Cumulative score process versus covariates (discrete z via model.matrix):\n")
   print(object$res.last)
}

} # }}}

##' @export
print.gof.phreg <-  function(x,...)
{# {{{
if (x$type=="prop")
     cat("Cumulative score process test for Proportionality:\n")
else cat("Cumulative residuals versus modelmatrix :\n")
print(x$res)

if (!is.null(x$res.last)) {
   cat("\n")
   cat("Cumulative score process versus covariates (discrete z via model.matrix):\n")
   print(x$res.last)
}

} # }}}

##' Binomial Regression with Stratified Pseudo-Values and Censoring Adjustment
##'
##' Fits a stratified binomial regression model for the cumulative incidence
##' function (CIF), restricted mean survival time (RMST), or restricted mean
##' time lost (RMTL), using inverse probability of censoring weighting (IPCW).
##' Each stratum may either be fit on its own observations or, when
##' \code{cv.fold = TRUE}, on the complement of its observations (i.e. a
##' leave-one-stratum-out / cross-validation style fit). Standard errors are
##' computed via an iid decomposition that accounts for the estimation of the
##' censoring distribution.
##'
##' @param formula A formula with a \code{\link[survival]{Surv}} or
##'   \code{Event} object on the left-hand side, and covariates, optional
##'   \code{cluster()}, \code{strata()}, \code{offset()}, and \code{weights()}
##'   terms on the right-hand side.
##' @param data A \code{data.frame} containing the variables in \code{formula}.
##' @param cause Value(s) of the status variable that identify the event of
##'   interest (cause 1 by default).
##' @param time The fixed time-point at which the CIF, RMST, or RMTL is
##'   evaluated. Required.
##' @param beta Optional starting values for the regression coefficients.
##'   Either a vector (recycled across strata) or a matrix with one row per
##'   stratum.
##' @param type Character; either \code{"II"} (default) or \code{"I"}. When
##'   \code{"II"}, an additional augmentation term and an extra component of
##'   the censoring-related iid decomposition are computed.
##' @param offset Optional offset term; can also be specified via the formula.
##' @param weights Optional case weights; can also be specified via the
##'   formula.
##' @param cens.weights Optional pre-computed inverse probability of censoring
##'   weights. If supplied, the internal censoring model fit is skipped and
##'   \code{se} is forced to \code{FALSE}.
##' @param cens.model A formula for the censoring model (right-hand side only,
##'   fitted via \code{\link{phreg}}). Defaults to the Kaplan-Meier
##'   (\code{~+1}).
##' @param se Logical. If \code{TRUE} (default), standard errors are computed,
##'   including the contribution from estimating the censoring distribution.
##' @param kaplan.meier Logical. If \code{TRUE} (default), uses the
##'   Kaplan-Meier estimator for censoring weights when the censoring model has
##'   no covariates; otherwise a Cox-type estimator is used internally when
##'   covariates are present.
##' @param cens.code The code in the status variable denoting censoring
##'   (default 0).
##' @param no.opt Logical. If \code{TRUE}, no optimization is performed and the
##'   object is evaluated at the supplied \code{beta}.
##' @param method Optimization method. Currently only \code{"nr"} (Newton-Raphson
##'   via \code{lava::NR}) is supported.
##' @param augmentation Optional augmentation term(s) added to the score
##'   equations, either a vector (recycled across strata) or a matrix with one
##'   row per stratum.
##' @param outcome Character; the outcome scale to model: \code{"cif"}
##'   (default), \code{"rmst"}, or \code{"rmtl"}.
##' @param model Character; the link function: \code{"default"} (chooses
##'   \code{"logit"} for CIF and \code{"exp"} for RMST/RMTL), \code{"logit"},
##'   \code{"exp"}, or \code{"lin"}.
##' @param Ydirect Optional, directly supplied response vector, overriding the
##'   response otherwise constructed from \code{outcome}.
##' @param strata Optional numeric vector defining strata, used when strata are
##'   not specified via \code{strata()} in the formula.
##' @param cv.fold Logical. If \code{TRUE}, for each stratum \code{s} the
##'   regression coefficients are estimated using all observations \emph{not}
##'   in stratum \code{s} (a cross-validation-style fit), rather than the
##'   observations within stratum \code{s} (the default, \code{FALSE}).
##' @param ... Additional arguments, currently passed as optimizer
##'   \code{control} settings (e.g. \code{tol}, \code{stepsize}) to the
##'   Newton-Raphson routine.
##'
##' @return An object of class \code{c("binregStrata","binreg")}, a list with
##'   components including:
##'   \item{coef}{Estimated regression coefficients (stacked across strata).}
##'   \item{iid}{Subject-level iid decomposition of the coefficients, adjusted
##'     for censoring estimation when \code{se = TRUE}.}
##'   \item{var, robvar}{Robust (sandwich) variance-covariance matrix.}
##'   \item{se.coef, se.robust}{Standard errors derived from \code{var}.}
##'   \item{hessian, ihessian}{Per-stratum Hessians and their (pseudo-)inverses.}
##'   \item{coef_list}{Per-stratum coefficient vectors.}
##'   \item{time, cause, cens.code, model, outcome}{Echoed arguments.}
##'   \item{Y, Yipcw, IPCW}{The constructed response, IPCW-weighted response,
##'     and IPCW weights.}
##'   \item{cens.weights, formC, cens.strata, cens.nstrata}{Censoring model
##'     weights and related information (when internally estimated).}
##'   \item{nstrata, strata_orig, strata_call, cv.fold}{Stratification
##'     information and whether the cross-validation fold scheme was used.}
##'   \item{n, nevent, ncluster, nid, id, name.id}{Sample size and
##'     identifier information.}
##'   \item{call, design}{The matched call and the design object.}
##'
##' @details
##' When \code{cv.fold = FALSE} (the standard case), the coefficients for
##' stratum \code{s} are estimated using only observations with
##' \code{strata == s}. When \code{cv.fold = TRUE}, the coefficients for
##' stratum \code{s} are instead estimated using observations with
##' \code{strata != s}, i.e. the complement; this also affects the
##' construction of the censoring-related iid terms (\code{MGCiid}) and the
##' per-stratum subject-level influence functions used for the robust
##' variance.
##'
##' Censoring weights are computed via a Cox model (\code{\link{phreg}}) on
##' \code{cens.model}, evaluated at \code{pmin(exit, time)}, unless
##' \code{cens.weights} is supplied directly. When \code{se = TRUE}, the
##' additional variability coming from the estimation of the censoring
##' distribution is incorporated into the standard errors via a martingale-type
##' iid decomposition (\code{MGCiid}).
##'
##' @seealso \code{\link{phreg}}, \code{\link{binreg}}
##'
##' @examples
##' \dontrun{
##' library(mets)
##' data(bmt)
##' out <- binregStrata(Event(time, cause) ~ tcell + strata(platelet),
##'                      data = bmt, cause = 1, time = 50)
##' summary(out)
##' }
##'
#' @export
binregStrata <- function(formula, data, cause=1, time=NULL, beta=NULL,
                          type=c("II","I"),
                          offset=NULL, weights=NULL, cens.weights=NULL,
                          cens.model=~+1, se=TRUE,
                          kaplan.meier=TRUE, cens.code=0, no.opt=FALSE,
                          method="nr", augmentation=NULL,
                          outcome=c("cif","rmst","rmtl"),
                          model=c("default","logit","exp","lin"),
                          Ydirect=NULL, strata=NULL, cv.fold=FALSE, ...)
{ # {{{
  monotone <- TRUE
  cl       <- match.call()

  ## ------------------------------------------------------------------
  ## Design
  ## ------------------------------------------------------------------
  des <- proc_design(
    formula, data = data,
    specials = c("offset","weights","cluster","strata"),
    intercept = TRUE
  )
  Y <- des$y
  if (!inherits(Y, c("Event","Surv")))
    stop("Expected a 'Surv' or 'Event'-object")

  if (ncol(Y) == 2) {
    exit  <- Y[,1]; entry <- rep(0, nrow(Y)); status <- Y[,2]
  } else {
    entry <- Y[,1]; exit  <- Y[,2];           status <- Y[,3]
  }

  X           <- des$x
  des.weights <- des$weights
  des.offset  <- des$offset
  id          <- des$cluster

  if (ncol(X) == 0) X <- matrix(nrow=0, ncol=0)

  call.id <- id
  conid   <- construct_id(id, nrow(X), namesX = rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  orig.id <- id

  if (is.null(des.offset))  offset  <- rep(0, length(exit)) else offset  <- des.offset
  if (is.null(des.weights)) weights <- rep(1, length(exit)) else weights <- des.weights

  ## ------------------------------------------------------------------
  ## Strata
  ## ------------------------------------------------------------------
  des.strata <- des$strata
  if (is.null(des.strata)) {
    if (is.null(strata)) {
      strata  <- rep(0, nrow(X)); nstrata <- 1
    } else {
      if (!is.numeric(strata)) stop("strata must be numeric\n")
      ustrata <- sort(unique(strata)); nstrata <- length(ustrata)
      strata  <- fast.approx(ustrata, strata) - 1
    }
  } else {
    strata  <- des.strata
    ustrata <- sort(unique(strata)); nstrata <- length(ustrata)
    strata  <- fast.approx(ustrata, strata) - 1
  }

  if (is.null(time)) stop("Must give time for logistic modelling\n")

  ## ------------------------------------------------------------------
  ## Event / censoring indicators
  ## ------------------------------------------------------------------
  statusC  <- 1*(status %in% cens.code)
  statusE  <- (status %in% cause) & (exit <= time)
  if (sum(statusE) == 0) warning("No events of type 1 before time\n")

  kmt     <- kaplan.meier
  ucauses <- sort(unique(status))
  ccc     <- which(ucauses %in% cens.code)
  Causes  <- if (length(ccc) == 0) ucauses else ucauses[-ccc]
  competing <- any(!(Causes %in% cause))

  data$id__    <- id
  data$exit    <- exit
  data$statusC <- statusC

  cens.strata <- cens.nstrata <- NULL
  nevent      <- sum((status %in% cause) * (exit <= time))
  obs         <- (exit <= time & statusC == 0) | (exit >= time)

  ## ------------------------------------------------------------------
  ## Censoring weights
  ## ------------------------------------------------------------------
  if (is.null(cens.weights)) { # {{{
    formC    <- update.formula(cens.model, Surv(exit,statusC) ~ . + cluster(id__))
    resC     <- phreg(formC, data)
    if (resC$p > 0) kmt <- FALSE
    exittime <- pmin(exit, time)
    cens.weights <- suppressWarnings(
      predict(resC, data, times = exittime, individual.time = TRUE,
              se = FALSE, km = kmt, tminus = TRUE)$surv)
    cens.strata  <- resC$strata[order(resC$ord)]
    cens.nstrata <- resC$nstrata
  } else {
    se <- FALSE; resC <- formC <- NULL
  } # }}}

  ## ------------------------------------------------------------------
  ## Response
  ## ------------------------------------------------------------------
  expit <- function(z) 1 / (1 + exp(-z))
  p     <- ncol(X)

  if (is.null(beta))         beta         <- rep(0, p)
  if (is.null(augmentation)) augmentation <- matrix(0, nstrata, p)
  if (!is.matrix(augmentation))
    augmentation <- matrix(augmentation, nstrata, p, byrow = TRUE)

  X  <- as.matrix(X)
  X2 <- .Call("vecCPMat", X)$XX

  if (!is.null(Ydirect)) {
    Y <- Ydirect
  } else {
    if (outcome[1] == "cif") {
      Y <- c((status %in% cause) * (exit <= time))
    } else {
      if (!competing) {
        if (outcome[1] == "rmst") Y <- c(pmin(exit, time))
        else                      Y <- c(time - pmin(exit, time))
      } else {
        Y <- c((status %in% cause) * (time - pmin(exit, time)))
      }
    }
  }
  Yresp <- c(Y)
  Y     <- Yipcw <- c(Y * obs / cens.weights)
  IPCW  <- c(obs / cens.weights)

  ## ------------------------------------------------------------------
  ## MGCiid: censoring IID influence
  ## cv.fold: block s populated with complement rows (strata != s)
  ## standard: block s populated with stratum-s rows
  ## ------------------------------------------------------------------
  MGCiid <- 0
  if (se & monotone) { # {{{
    ord     <- resC$ord
    X       <- X[ord,  , drop = FALSE]
    X2      <- X2[ord, , drop = FALSE]
    status  <- status[ord]
    exit    <- exit[ord]
    weights <- weights[ord]
    offset  <- offset[ord]
    Y       <- Y[ord]
    strata  <- strata[ord]

    xx   <- resC$cox.prep
    S0i2 <- S0i <- rep(0, length(xx$strata))
    S0i[xx$jumps  + 1] <- 1 / resC$S0
    S0i2[xx$jumps + 1] <- 1 / resC$S0^2
    btime <- 1*(exit < time)
    mid   <- max(id)

    ## Block-expanded design: n x (nstrata*p)
    ## cv.fold: block s gets complement rows; standard: block s gets stratum-s rows
    Xs <- matrix(0, nrow(X), nstrata * p)
    for (s in 0:(nstrata - 1)) {
      idx  <- if (cv.fold) which(strata != s) else which(strata == s)
      cols <- s * p + seq_len(p)
      Xs[idx, cols] <- X[idx, ]
    }

    h           <- Mrevcumsumstrata(Xs * Y, xx$strata, xx$nstrata)
    IhdLam0     <- Mcumsumstrata(h * S0i2 * btime, xx$strata, xx$nstrata)
    U           <- matrix(0, nrow(xx$X), ncol(Xs))
    U[xx$jumps + 1,] <- (resC$jumptimes < time) *
                         h[xx$jumps + 1,] / c(resC$S0)
    MGt    <- (U - IhdLam0) * c(xx$weights)
    MGCiid <- Msumstrata(MGt, xx$id, mid + 1)

    if (type[1] == "II") { # {{{
      hYt      <- revcumsumstrata(Y,   xx$strata, xx$nstrata)
      IhdLam0  <- cumsumstrata(hYt * S0i2 * btime, xx$strata, xx$nstrata)
      U2       <- rep(0, length(xx$strata))
      U2[xx$jumps + 1] <- (resC$jumptimes < time) *
                           hYt[xx$jumps + 1] / c(resC$S0)
      MGt2    <- Xs * c(U2 - IhdLam0) * c(xx$weights)
      MGtiid  <- Msumstrata(MGt2, xx$id, mid + 1)
      augmentation <- colSums(MGtiid) + augmentation
      augmentation <- matrix(augmentation, nstrata, p, byrow = TRUE)

      EXt          <- Mrevcumsumstrata(Xs, xx$strata, xx$nstrata)
      IEXhYtdLam0  <- Mcumsumstrata(EXt * c(hYt) * S0i * S0i2 * btime,
                                     xx$strata, xx$nstrata)
      U3 <- matrix(0, nrow(xx$X), ncol(Xs))
      U3[xx$jumps + 1,] <- (resC$jumptimes < time) *
                            hYt[xx$jumps + 1] *
                            EXt[xx$jumps + 1,] / c(resC$S0)^2
      MGt3    <- (U3 - IEXhYtdLam0) * c(xx$weights)
      MGCiid2 <- Msumstrata(MGt3, xx$id, mid + 1)
      MGCiid  <- MGCiid + (MGtiid - MGCiid2)
    } # }}}

    id <- xx$id
  } else {
    MGCiid <- 0
  } # }}}

  ## ------------------------------------------------------------------
  ## Model default
  ## ------------------------------------------------------------------
  if (model[1] == "default") {
    if (outcome[1] == "cif")        model <- "logit"
    if (outcome[1] == "rmst")       model <- "exp"
    if (outcome[1] == "rmtl")       model <- "exp"
    if (outcome[1] == "years-lost") model <- "exp"
  }

  ## ------------------------------------------------------------------
  ## obj(): full objective (used for final all=TRUE call and NR)
  ## ------------------------------------------------------------------
  obj <- function(pp, all = FALSE) { # {{{
    np <- length(pp[[1]])

    ## ----------------------------------------------------------------
    ## cv.fold path: for stratum s, beta_s estimated on complement rows
    ## ----------------------------------------------------------------
    if (cv.fold) {
      ploglik_list  <- vector("list", nstrata)
      gradient_list <- vector("list", nstrata)
      D2log_list    <- vector("list", nstrata)

      for (s in 0:(nstrata - 1)) {
        idx_c <- which(strata != s)          # complement indices
        X_c   <- X[idx_c,  , drop=FALSE]
        X2_c  <- X2[idx_c, , drop=FALSE]
        Y_c   <- Y[idx_c]
        w_c   <- weights[idx_c]
        off_c <- offset[idx_c]
        aug_s <- augmentation[s + 1, ]
        lp_c  <- as.vector(X_c %*% pp[[s + 1]]) + off_c

        if (model[1] == "exp") {
          pr_c  <- exp(lp_c)
          if (monotone) {
            Dl_c  <- w_c * X_c * c(Y_c - pr_c)
            D2l_c <- c(w_c * pr_c) * X2_c
          } else {
            Dl_c  <- w_c * X_c * c((Y_c - pr_c) * pr_c)
            D2l_c <- c(w_c * pr_c^2) * X2_c
          }
          pl_c <- 0
        } else if (model[1] == "lin") {
          pr_c  <- lp_c
          Dl_c  <- w_c * X_c * c(Y_c - pr_c)
          D2l_c <- c(w_c) * X2_c
          pl_c  <- sum(w_c * (Y_c - pr_c)^2)
        } else if (model[1] == "logit") {
          pr_c <- expit(lp_c)
          if (monotone) {
            Dl_c  <- w_c * X_c * c(Y_c - pr_c)
            D2l_c <- c(w_c * pr_c / (1 + exp(lp_c))) * X2_c
          } else {
            varp  <- pr_c / (1 + exp(lp_c))
            Dl_c  <- w_c * X_c * c((Y_c - pr_c) * varp)
            D2l_c <- c(w_c * varp^2) * X2_c
          }
          pl_c <- 0
        } else stop("link must be logit, exp, or lin\n")

        ploglik_list[[s + 1]]  <- pl_c
        gradient_list[[s + 1]] <- -(colSums(Dl_c) + aug_s)
        D2log_list[[s + 1]]    <- colSums(D2l_c)
      }

      sym <- function(x)
        matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                     PACKAGE="mets")$XXf, np, np)
      hessian <- lapply(D2log_list, sym)

      if (all) {
        ihess <- lapply(hessian, pinv)

        ## Per-stratum IID: recompute Dl_c at optimal pp[[s]] separately
        ## for each stratum so blocks are independent and correctly sized.
        ## Obs i in stratum r contributes to beta_s for all s != r;
        ## Msumstrata fills zeros for stratum-s obs (they don't contribute).
        beta.iid <- do.call(cbind, lapply(seq_len(nstrata), function(s) {
          idx_c <- which(strata != (s - 1))
          X_c   <- X[idx_c,  , drop=FALSE]
          Y_c   <- Y[idx_c]
          w_c   <- weights[idx_c]
          off_c <- offset[idx_c]
          id_c  <- id[idx_c]
          lp_c  <- as.vector(X_c %*% pp[[s]]) + off_c

          Dl_c <- switch(model[1],
            exp = {
              pr_c <- exp(lp_c)
              if (monotone) w_c * X_c * c(Y_c - pr_c)
              else          w_c * X_c * c((Y_c - pr_c) * pr_c)
            },
            lin = w_c * X_c * c(Y_c - lp_c),
            logit = {
              pr_c <- expit(lp_c)
              if (monotone) w_c * X_c * c(Y_c - pr_c)
              else {
                varp <- pr_c / (1 + exp(lp_c))
                w_c * X_c * c((Y_c - pr_c) * varp)
              }
            },
            stop("unknown model\n")
          )

          raw <- Dl_c %*% ihess[[s]]
          Msumstrata(raw, id_c, max(id) + 1)
        }))

        robvar <- crossprod(beta.iid)
        return(list(
          par      = pp,             coef      = unlist(pp),
          ploglik  = ploglik_list,   gradient  = gradient_list,
          hessian  = hessian,        ihessian  = ihess,
          id       = id,             Dlogl     = NULL,
          iid      = beta.iid,       robvar    = robvar,
          var      = robvar,         se.robust = diag(robvar)^.5
        ))
      }

      return(structure(ploglik_list,
                       gradient = gradient_list,
                       hessian  = hessian))
    }

# ----------------------------------------------------------------
# Standard non-CV
# ----------------------------------------------------------------
    loffset <- eta <- numeric(length(Y))
    for (s in 0:(nstrata - 1)) {
      idx          <- strata == s
      eta[idx]     <- as.vector(X[idx, , drop=FALSE] %*% pp[[s + 1]])
      loffset[idx] <- offset[idx]
    }
    lp <- eta + loffset

    if (model[1] == "exp") {
      pr     <- exp(lp)
      if (monotone) {
        Dlogl  <- weights * X * c(Y - pr)
        D2logl <- c(weights * pr) * X2
      } else {
        Dlogl  <- weights * X * c((Y - pr) * pr)
        D2logl <- c(weights * pr^2) * X2
      }
    } else if (model[1] == "lin") {
      pr     <- lp
      Dlogl  <- weights * X * c(Y - pr)
      D2logl <- c(weights) * X2
    } else if (model[1] == "logit") {
      pr <- expit(lp)
      if (monotone) {
        Dlogl  <- weights * X * c(Y - pr)
        D2logl <- c(weights * pr / (1 + exp(lp))) * X2
      } else {
        varp   <- pr / (1 + exp(lp))
        Dlogl  <- weights * X * c((Y - pr) * varp)
        D2logl <- c(weights * varp^2) * X2
      }
    } else stop("link functions must be logit, exp, lin\n")

    ploglik  <- sumstrata(weights * (Y - pr)^2, strata, nstrata)
    ploglik  <- matrix(ploglik, nstrata, 1)
    if (model[1] != "lin") ploglik <- 0 * ploglik
    D2log    <- Msumstrata(D2logl,  strata, nstrata)
    gradient <- Msumstrata(Dlogl,   strata, nstrata) + augmentation
    if (!is.matrix(D2log))    D2log    <- matrix(D2log,    1, length(D2log))
    if (!is.matrix(gradient)) gradient <- matrix(gradient, 1, np)

    gradient <- split(-gradient, seq_len(nrow(gradient)))
    ploglik  <- split(-ploglik,  seq_len(nrow(ploglik)))
    D2log    <- split(D2log,     seq_len(nrow(D2log)))

    sym <- function(x)
      matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                   PACKAGE="mets")$XXf, np, np)
    hessian <- lapply(D2log, sym)

    if (all) {
      ihess    <- lapply(hessian, pinv)
      iid_list <- lapply(seq_len(nstrata), function(s) {
        idx  <- strata == (s - 1)
        id_s <- id[idx]
        Dl_s <- Dlogl[idx, , drop=FALSE]
        raw  <- Dl_s %*% ihess[[s]]
        Msumstrata(raw, id_s, max(id) + 1)
      })
      beta.iid <- do.call(cbind, iid_list)
      robvar   <- crossprod(beta.iid)
      return(list(
        par      = pp,           coef      = unlist(pp),
        ploglik  = ploglik,      gradient  = gradient,
        hessian  = hessian,      ihessian  = ihess,
        id       = id,           Dlogl     = Dlogl,
        iid      = beta.iid,     robvar    = robvar,
        var      = robvar,       se.robust = diag(robvar)^.5
      ))
    }
    structure(ploglik, gradient = gradient, hessian = hessian)
  } # }}}

  ## ------------------------------------------------------------------
  ## make_stratum_obj: per-stratum closure for NR
  ## cv.fold: use complement rows; standard: use stratum rows
  ## ------------------------------------------------------------------
  make_stratum_obj <- function(s, pp) { # {{{
    idx       <- if (cv.fold) strata != s else strata == s
    X_s       <- X[idx,  , drop=FALSE]
    X_s2      <- X2[idx, , drop=FALSE]
    Y_s       <- Y[idx]
    weights_s <- weights[idx]
    offset_s  <- offset[idx]
    aug_s     <- augmentation[s + 1, ]

    sym <- function(x, np)
      matrix(.Call("XXMatFULL", matrix(x, nrow=1), np,
                   PACKAGE="mets")$XXf, np, np)

    function(theta) {
      lp <- as.vector(X_s %*% theta) + offset_s
      if (model[1] == "exp") {
        pr <- exp(lp)
        if (monotone) {
          Dlogl  <- weights_s * X_s * c(Y_s - pr)
          D2logl <- c(weights_s * pr) * X_s2
        } else {
          Dlogl  <- weights_s * X_s * c((Y_s - pr) * pr)
          D2logl <- c(weights_s * pr^2) * X_s2
        }
        ploglik <- 0
      } else if (model[1] == "lin") {
        pr      <- lp
        Dlogl   <- weights_s * X_s * c(Y_s - pr)
        D2logl  <- c(weights_s) * X_s2
        ploglik <- sum(weights_s * (Y_s - pr)^2)
      } else if (model[1] == "logit") {
        pr <- expit(lp)
        if (monotone) {
          Dlogl  <- weights_s * X_s * c(Y_s - pr)
          D2logl <- c(weights_s * pr / (1 + exp(lp))) * X_s2
        } else {
          varp   <- pr / (1 + exp(lp))
          Dlogl  <- weights_s * X_s * c((Y_s - pr) * varp)
          D2logl <- c(weights_s * varp^2) * X_s2
        }
        ploglik <- 0
      } else stop("link functions must be logit, exp, lin\n")

      np   <- length(theta)
      grad <- -(colSums(Dlogl) + aug_s)
      D2   <- colSums(D2logl)
      hess <- sym(D2, np)
      structure(-ploglik, gradient = grad, hessian = hess)
    }
  } # }}}

  ## ------------------------------------------------------------------
  ## Optimise
  ## ------------------------------------------------------------------
  dots    <- list(...)
  control <- if (length(dots) == 0) {
    if (model[1] == "exp") list(tol=1e-10, stepsize=0.5) else NULL
  } else dots[[1]]

  if (!is.matrix(beta)) beta <- matrix(beta, nstrata, p, byrow = TRUE)
  beta <- split(beta, seq_len(nrow(beta)))

  NR_list <- function() {
    Map(function(s) {
      lava::NR(start   = beta[[s + 1]],
                obj     = make_stratum_obj(s, beta),
                control = control)
    }, seq_len(nstrata) - 1L)
  }

  beta_opt <- beta   # fallback

  if (p > 0) {
    if (!no.opt) {
      if (tolower(method) == "nr") {
        tim      <- NR_list()
        beta_opt <- lapply(tim, function(r) r$par)
        cc       <- unlist(beta_opt)
        val      <- c(list(coef = cc), obj(beta_opt, all = TRUE))
        val$gradient <- lapply(tim, function(r) r$gradient)
      } else {
        stop("only NR of lava\n")
      }
    } else {
      val <- c(list(coef = unlist(beta)), obj(beta, all = TRUE))
    }
  } else {
    val <- obj(as.list(rep(0, nstrata)), all = TRUE)
  }

  ## ------------------------------------------------------------------
  ## Coefficient naming
  ## ------------------------------------------------------------------
  if (nstrata == 1 && length(val$coef) == length(colnames(X))) {
    names(val$coef) <- colnames(X)
  } else {
    cnames <- paste(
      rep(paste0("s", 1:nstrata), each = ncol(X)),
      rep(colnames(X), times = nstrata),
      sep = "."
    )
    names(val$coef)   <- cnames
    colnames(val$iid) <- cnames
  }

  ## ------------------------------------------------------------------
  ## Assemble output
  ## ------------------------------------------------------------------
  val <- c(val, list(
    time          = time,          formula      = formula,
    formC         = formC,         cens.weights = cens.weights,
    cens.strata   = cens.strata,   cens.nstrata = cens.nstrata,
    n             = length(exit),  nevent       = nevent,
    ncluster      = nid
  ))
  val$call      <- cl
  val$MGciid    <- MGCiid
  val$call.id   <- call.id
  val$id        <- orig.id
  val$name.id   <- name.id
  val$nid       <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL
  val$coef_list <- beta_opt
  val$des       <- des

  ## ------------------------------------------------------------------
  ## Censoring IID adjustment, per stratum block
  ## iid[, cols_s] += MGCiid[, cols_s] %*% H_s^{-1}
  ## For cv.fold, MGCiid[, cols_s] already encodes complement rows via Xs,
  ## and ihessian[[s]] is from the complement fit
  ## ------------------------------------------------------------------
  if (se) {
    val$iid <- do.call(cbind, lapply(seq_len(nstrata), function(s) {
      cols <- (s - 1) * p + seq_len(p)
      val$iid[, cols] + MGCiid[, cols] %*% val$ihessian[[s]]
    }))
  }

  if (!is.null(name.id)) val$iid <- nameme(val$iid, name.id)
  robvar        <- crossprod(val$iid)
  val$var       <- val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef   <- diag(val$var)^.5

  val$cause        <- cause
  val$cens.code    <- cens.code
  val$augmentation <- augmentation
  val$model        <- model[1]
  val$outcome      <- outcome[1]
  val$Yipcw        <- Yipcw
  val$Y            <- Yresp
  val$IPCW         <- IPCW
  val$Causes       <- Causes
  val$nevent       <- nevent
  val$design       <- des
  val$nstrata      <- nstrata
  val$strata_orig  <- strata
  val$strata_call  <- des.strata
  val$cv.fold      <- cv.fold

  class(val) <- c("binregStrata","binreg")
  return(val)
} # }}}

## {{{  summary, predict, 

## small helper: given flat coef vector, route predictions per stratum
strata_lp <- function(object, Z, strata_new, offset_new) {
  p         <- ncol(Z)
  beta_list <- split(object$coef, rep(seq_len(object$nstrata), each = p))
  eta       <- numeric(nrow(Z))
  for (s in seq_len(object$nstrata)) {
    idx       <- strata_new == (s - 1)
    eta[idx]  <- as.vector(Z[idx, , drop = FALSE] %*% beta_list[[s]]) + offset_new[idx]
  }
  eta
}

##' @export
predict.binregStrata <- function(object, newdata = NULL, se = TRUE,
                                  pred.iid = FALSE, level = 0.95,
                                  strata = NULL,offset=NULL, ...) {  ## {{{ 
  ## design matrix
  if (!is.null(newdata)) {
    x <- update_design(object$design, data = newdata, response = FALSE)
  } else {
    x <- object$design
  }
  Z          <- x$x

  if (!is.null(strata)) {
  strata_new <- as.numeric(strata) - 1
  } else if (!is.null(x$strata)) {
  strata_new <- as.numeric(x$strata) - 1
  } else {
  strata_new <- object$strata_orig
  }

  offset_new <- if (!is.null(offset)) {
  offset
  } else if (!is.null(x$offset)) {
  x$offset
  } else {
  rep(0, nrow(Z))
  }

  ## linear predictor routed per stratum
  lp <- strata_lp(object, Z, strata_new, offset_new)

  ## link
  p <- switch(object$model[1],
    logit = expit(lp),
    exp   = exp(lp),
    lin   = lp,
    stop("unknown model")
  )

## derivative for delta method: same shape as before, per-stratum coef block
  p_col <- ncol(Z)
  Dpv   <- matrix(0, nrow(Z), object$nstrata * p_col)
  for (s in seq_len(object$nstrata)) {
    idx  <- strata_new == (s - 1)
    cols <- (s - 1) * p_col + seq_len(p_col)
    if (object$model[1] == "logit") {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE] * exp(-lp[idx]) * p[idx]^2
    } else if (object$model[1] == "exp") {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE] * p[idx]
    } else {
      Dpv[idx, cols] <- Z[idx, , drop = FALSE]
    }
  }

  preds <- p
  if (se) {
    covv   <- object$var
    se_vec <- apply((Dpv %*% covv) * Dpv, 1, sum)^.5
    crit   <- qnorm(1 - (1 - level) / 2)
    preds  <- data.frame(pred  = p,
                         se    = se_vec,
                         lower = p - crit * se_vec,
                         upper = p + crit * se_vec)
  }
  if (pred.iid) {
    Piid  <- object$iid %*% t(Dpv)
    preds <- list(pred = preds, iid = Piid, id = x$cluster)
  }
  attr(preds, "id") <- x$cluster
  preds
} ## }}} 

##' @export
summary.binregStrata <- function(object, ...) {
  ## per-stratum coefficient tables
  p         <- length(object$coef) / object$nstrata
  beta_list <- split(object$coef, rep(seq_len(object$nstrata), each = p))
  nms       <- colnames(object$design$x)

  cat("Stratified binreg fit",object$nstrata,"strata\n")
  cat("n =", object$n, "  events =", object$nevent, "\n\n")

  for (s in seq_len(object$nstrata)) {
    cols  <- (s - 1) * p + seq_len(p)
    b_s   <- object$coef[cols]
    var_s <- object$var[cols, cols]
    cc    <- estimate(coef = b_s, vcov = var_s)$coefmat
    rownames(cc) <- nms
    cat("Stratum", s, ":\n")
    printCoefmat(cc, ...)
    if (object$model[1] %in% c("exp", "logit")) {
      cat("exp(coef):\n")
      printCoefmat(
        exp(estimate(coef = b_s, vcov = var_s)$coefmat[, c(1,3,4), drop=FALSE]),
        ...
      )
    }
    cat("\n")
  }
  invisible(object)
}

##' @export
coef.binregStrata <- function(object, ...) object$coef

##' @export
vcov.binregStrata <- function(object, ...) object$var

##' @export
IC.binregStrata <- function(x, ...) x$iid * NROW(x$iid)

## }}} 

## --------------------------------------------------------------------------
## summary.binregStrata
## --------------------------------------------------------------------------
##' @export
summary.binregStrata <- function(object, level = 0.95, ...) { ## {{{
  p       <- length(object$coef) / object$nstrata
  nms     <- colnames(object$design$x)
  n       <- nrow(object$iid)
  crit    <- qnorm(1 - (1 - level) / 2)

  exp_label <- switch(object$model[1],
    logit = "OR",
    exp   = "RR",
    lin   = NULL
  )

  strata_tables     <- vector("list", object$nstrata)
  strata_exp_tables <- vector("list", object$nstrata)

  for (s in seq_len(object$nstrata)) {
    cols  <- (s - 1L) * p + seq_len(p)
    b_s   <- object$coef[cols]
    iid_s <- object$iid[, cols, drop = FALSE]
    e_s   <- lava::estimate(NULL, coef = b_s, IC = iid_s * n)
    cm    <- e_s$coefmat
    rownames(cm) <- if (!is.null(nms)) nms else paste0("X", seq_len(p))
    strata_tables[[s]] <- cm

    if (!is.null(exp_label)) {
      e_exp <- lava::estimate(NULL, coef = b_s, IC = iid_s * n, f = exp)
      cm_exp <- e_exp$coefmat
      rownames(cm_exp) <- rownames(cm)
      strata_exp_tables[[s]] <- cm_exp
    }
  }
  names(strata_tables)     <- paste0("stratum", seq_len(object$nstrata))
  names(strata_exp_tables) <- paste0("stratum", seq_len(object$nstrata))

  structure(
    list(strata_tables     = strata_tables,
         strata_exp_tables = strata_exp_tables,
         exp_label         = exp_label,
         nstrata           = object$nstrata,
         p                 = p,
         model             = object$model[1],
         outcome           = object$outcome[1],
         n                 = object$n,
         nevent            = object$nevent,
         ncluster          = object$ncluster,
         cv.fold           = isTRUE(object$cv.fold),
         call              = object$call),
    class = "summary.binregStrata"
  )
} ## }}}

## --------------------------------------------------------------------------
## print.summary.binregStrata
## --------------------------------------------------------------------------
##' @export
print.summary.binregStrata <- function(x, digits = 3, ...) { ## {{{ 
  cat("\nCall:\n")
  if (!is.null(x$call)) print(x$call)
  cat("\nStratified binreg fit\n")
  cat("  Strata  :", x$nstrata,
      if (x$cv.fold) " [cv.fold -- leave-one-stratum-out CV]" else "", "\n")
  cat("  Model   :", x$model, "\n")
  cat("  Outcome :", x$outcome, "\n")
  cat("  n =", x$n,
      " | events =", x$nevent,
      " | clusters =", x$ncluster, "\n\n")

  for (s in seq_len(x$nstrata)) {
    cat(paste0("--- Stratum ", s,
               if (x$cv.fold)
                 paste0(" (fitted on strata != ", s, ")")
               else "",
               " ---\n"))
    printCoefmat(x$strata_tables[[s]], digits = digits, ...)

    if (!is.null(x$exp_label) && !is.null(x$strata_exp_tables[[s]])) {
      cat(x$exp_label, "= exp(coef):\n")
      em <- x$strata_exp_tables[[s]][, c("Estimate","2.5%","97.5%"), drop = FALSE]
      colnames(em)[1] <- x$exp_label
      print(round(em, digits))
    }
    cat("\n")
  }
  invisible(x)
} ## }}} 

## --------------------------------------------------------------------------
## print.binregStrata  -- brief one-liner per stratum
## --------------------------------------------------------------------------
##' @export
print.binregStrata <- function(x, digits = 3, ...) { ## {{{ 
  cat("\nStratified binreg fit\n")
  cat("  Model:", x$model[1], " | Outcome:", x$outcome[1],
      " | Strata:", x$nstrata,
      if (isTRUE(x$cv.fold)) " [cv.fold]" else "", "\n")
  cat("  n =", x$n,
      "| events =", x$nevent,
      "| clusters =", x$ncluster, "\n\n")

  p   <- length(x$coef) / x$nstrata
  nms <- colnames(x$design$x)
  n   <- nrow(x$iid)

  for (s in seq_len(x$nstrata)) {
    cols <- (s - 1L) * p + seq_len(p)
    b_s  <- x$coef[cols]
    se_s <- sqrt(diag(x$var)[cols])
    mat  <- cbind(Estimate = b_s, `Std.Err` = se_s,
                  `z`      = b_s / se_s,
                  `Pr(>|z|)` = 2 * pnorm(-abs(b_s / se_s)))
    if (!is.null(nms)) rownames(mat) <- nms
    cat(paste0("Stratum ", s,
               if (isTRUE(x$cv.fold))
                 paste0(" (train: strata != ", s, ")")
               else "", ":\n"))
    printCoefmat(mat, digits = digits, ...)
    cat("\n")
  }
  invisible(x)
} ## }}} 

##' @export
coef.binregStrata <- function(object, ...) object$coef

##' @export
vcov.binregStrata <- function(object, ...) object$var

##' @export
IC.binregStrata <- function(x, ...) x$iid * NROW(x$iid)


##' Cross-validated Brier score for competing risks, RMST and RMTL regression
##' using stratified leave-fold-out fits (\code{binregStrata})
##'
##' Fits candidate models via \code{\link[mets]{binregStrata}} with
##' \code{cv.fold = TRUE} and evaluates each using apparent (in-sample) and
##' \emph{k}-fold cross-validated IPCW Brier scores relative to a null model.
##' Supports all outcome types available in \code{binreg}/\code{binregStrata}:
##' cause-specific cumulative incidence (competing risks), restricted mean
##' survival time (RMST), and restricted mean time lost (RMTL).
##'
##' Cross-validation is delegated directly to
##' \code{\link[mets]{binregStrata}} via its \code{strata} and
##' \code{cv.fold = TRUE} arguments: each stratum (fold) gets its own
##' coefficient vector \eqn{\beta_s}, estimated on the complement of that
##' fold. Out-of-fold predictions are obtained by routing each observation to
##' the \eqn{\beta_s} corresponding to its own fold, which was trained on the
##' other folds.
##'
##' The fold membership is stored in an internal \code{fold} column (1-based)
##' in \code{data} and passed to \code{binregStrata} as a 0-based
##' \code{strata} vector. Censoring weights for the underlying competing-risks
##' model are estimated according to \code{cens.model} as usual; since strata
##' already define the folds, \code{cens.model} need not include
##' \code{strata(fold)} explicitly.
##'
##' Brier scores are computed on the \strong{linear scale} via
##' \code{resmeanIPCW(..., model = "lin")} and then log-transformed using
##' \code{lava::estimate(..., f = log)} with the delta method. All stored
##' coefficients, standard errors, and influence functions are therefore on
##' the \strong{log scale}. Use \code{summary(fit, transform = exp)} or
##' \code{estimate(fit, f = exp)} to back-transform to the original Brier
##' scale.
##'
##' The delta-method correction for the cross-validated influence function
##' accounts for the block structure of \code{binregStrata}'s influence
##' function (\eqn{n_{id} \times (\code{nstrata} \times p)}, one
##' \eqn{p}-column block per stratum). For each stratum \eqn{s}, only
##' observations belonging to fold \eqn{s} have a non-zero derivative of
##' their out-of-fold prediction with respect to \eqn{\beta_s}, since
##' \eqn{\beta_s} is exactly the coefficient used to predict fold \eqn{s}.
##' The full gradient is therefore block-sparse, with one non-zero
##' \eqn{p}-length block per stratum, and the correction reduces to a single
##' matrix product against \code{iid(outff_fit)}.
##'
##' Left-truncation (delayed entry) is not supported and will raise an error.
##' Specifically, \code{Event(entry, time, status)} is rejected; use
##' \code{Event(time, status)} or \code{Event(time, status, cause)}.
##'
##' @param formula Two-sided formula of the form
##'   \code{Event(time, status) ~ covariates + cluster(id)} or
##'   \code{Event(time, status, cause) ~ covariates + cluster(id)}.
##'   The \code{cluster(id)} term is optional; if absent an internal
##'   identifier \code{id__} is added automatically assuming independent
##'   observations.
##' @param data A \code{data.frame} containing all variables in
##'   \code{formula} and \code{rhs}.
##' @param time Scalar or numeric vector of evaluation time points. When a
##'   vector is supplied the function loops over time points reusing the same
##'   fold structure and returns a named list of \code{binregCV} objects with
##'   class \code{"binregCV_list"}.
##' @param fold Integer. Number of cross-validation folds / strata
##'   (default 5).
##' @param rhs Named list of one-sided formulas specifying candidate models,
##'   e.g. \code{list(small = ~Z1, full = ~Z1 + Z2)}. If \code{NULL} (default),
##'   the right-hand side of \code{formula} (minus any \code{cluster()} term)
##'   is used as a single candidate named \code{"model1"}.
##' @param rhs0 One-sided formula for the null (intercept-only) model
##'   (default \code{~+1}).
##' @param cens.model Censoring model passed to \code{binregStrata} for the
##'   CV fold fits (default \code{~+1}). Since folds are handled internally
##'   by \code{binregStrata}'s \code{strata}/\code{cv.fold} mechanism, this
##'   does \emph{not} need to include \code{strata(fold)}.
##' @param brier.cens.model Censoring model passed to \code{resmeanIPCW} when
##'   computing Brier scores. Default \code{~+1} gives Kaplan-Meier weights.
##'   Supply a stratified model, e.g. \code{~strata(Z1, Z2)}, for
##'   covariate-adjusted IPCW weights.
##' @param brier.args Named list of additional arguments forwarded to every
##'   \code{resmeanIPCW} call (beyond \code{formula}, \code{data},
##'   \code{time}, \code{Ydirect}, \code{cens.model}, and \code{model} which
##'   are set internally). Typically empty.
##' @param cv.split.index List from \code{folds(n, fold)} or a vector of fold
##'   indices (1:fold) for each observation, to fix the cross-validation
##'   folds. \code{NULL} (default) generates folds internally via
##'   \code{folds(n, fold)}.
##' @param \dots Additional arguments passed to every \code{binreg} /
##'   \code{binregStrata} call, e.g. \code{cause = 2}, \code{outcome = "rmst"},
##'   or \code{offset}.
##'
##' @return An object of class \code{"binregCV"} (single time point) or
##'   \code{"binregCV_list"} (multiple time points), with the following
##'   structure:
##' \describe{
##'   \item{\code{null}}{List with \code{coef}, \code{se}, and \code{iid} for
##'     the null model. \code{coef} and \code{se} are named vectors with
##'     entries \code{bbrier} (apparent, log scale) and \code{bbrierCV}
##'     (cross-validated, log scale). \code{iid} contains the
##'     influence-function matrix for \code{bbrierCV} with delta-method
##'     correction for the stratified CV prediction step and the log
##'     transform.}
##'   \item{\code{models}}{Named list, one element per entry in \code{rhs}.
##'     Each element contains \code{coef} (\code{brier}, \code{brierCV}, log
##'     scale), \code{se}, \code{dcoef} (differences from null: log Brier
##'     ratios), \code{sed}, and \code{iid} (influence-function matrix for
##'     \code{brierCV}, log scale, with CV delta-method correction).}
##'   \item{\code{iid_mat}}{Matrix of dimension \eqn{n \times (1 +
##'     \code{length(rhs)})} with columns \code{null} and one per entry of
##'     \code{rhs}, each the log-scale CV Brier influence function for the
##'     corresponding model. Used by \code{\link{estimate.binregCV}} for joint
##'     inference.}
##'   \item{\code{cv.split.index}}{List of fold index vectors used.}
##'   \item{\code{rhs}, \code{rhs0}}{Processed candidate and null formulas
##'     (cluster terms removed).}
##'   \item{\code{model}}{Link function read from the \code{binreg} fit:
##'     \code{"logit"}, \code{"exp"}, or \code{"lin"}.}
##'   \item{\code{call}, \code{time}, \code{fold}, \code{cens.model},
##'     \code{brier.cens.model}}{Call and settings for printing and
##'     downstream use.}
##' }
##'
##' @section Outcome types:
##' The function supports all outcome types provided by \code{binreg}/
##' \code{binregStrata}:
##' \describe{
##'   \item{Competing risks CIF}{Default; use
##'     \code{Event(time, status, cause)} and pass \code{cause} via \code{...}.
##'     The Brier score measures squared error for the cause-specific CIF.}
##'   \item{RMST}{Pass \code{outcome = "rmst"} via \code{...}. The Brier score
##'     measures squared error for the restricted mean survival time up to
##'     \code{time}.}
##'   \item{RMTL}{Pass \code{outcome = "rmtl"} via \code{...}. The Brier score
##'     measures squared error for the restricted mean time lost.}
##' }
##'
##' @section Scale and inference:
##' All output is on the log scale. To obtain Brier scores on the original
##' scale with confidence intervals, use:
##' \preformatted{
##'   summary(fit, transform = exp)
##'   estimate(fit, f = exp)
##' }
##' Differences on the log scale correspond to log Brier ratios; exponentiate
##' to obtain Brier ratios. For absolute differences on the original scale,
##' compute contrasts after \code{estimate(fit, f = exp)}.
##'
##' @seealso
##'   \code{\link{estimate.binregCV}} for joint inference across models and
##'   time points;
##'   \code{\link{summary.binregCV}} and \code{\link{print.summary_binregCV_multi}}
##'   for formatted output;
##'   \code{\link[mets]{binregStrata}} for the underlying stratified
##'   regression model;
##'   \code{\link[mets]{resmeanIPCW}} for IPCW mean estimation;
##'   \code{\link[lava]{estimate}} for delta-method inference.
##'
##' @examples
##' \dontrun{
##' library(mets)
##' data(bmt)
##' bmt$id <- seq_len(nrow(bmt))
##'
##' ## --- competing risks CIF/Survival (default) ---
##' fit <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs = list(small = ~age, full = ~tcell + platelet + age)
##' )
##' summary(fit)                        ## log scale
##' summary(fit, transform = exp)       ## Brier scale
##'
##' ## multiple time points
##' fitm <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = c(50, 60),
##'   rhs = list(small = ~age, full = ~tcell + platelet + age)
##' )
##' summary(fitm, transform = exp)
##'
##' ## joint inference across models via lava::estimate
##' e <- estimate(fitm)
##' e
##' estimate(e, f = exp)               ## back-transform
##'
##' ## stratified IPCW weights for Brier score
##' fit_strat <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs = list(small = ~age, full = ~tcell + platelet + age),
##'   brier.cens.model = ~strata(tcell)
##' )
##'
##' ## --- RMST --- survival case only
##' fit_rmst <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50, cause = 1:2,
##'   rhs    = list(small = ~age, full = ~tcell + platelet + age),
##'   outcome = "rmst"
##' )
##' summary(fit_rmst, transform = exp)
##'
##' ## --- RMTL --- competing risks or survival (RMTL)
##' fit_rmtl <- brier_binreg(
##'   Event(time, cause) ~ tcell + platelet + age + cluster(id),
##'   data = bmt, time = 50,
##'   rhs    = list(small = ~age, full = ~tcell + platelet + age),
##'   outcome = "rmtl"
##' )
##' summary(fit_rmtl, transform = exp)
##' }
##' @export
brier_binreg <- function(formula, data, time,
                                     fold             = 5,
                                     rhs              = NULL,
                                     rhs0             = ~+1,
                                     cens.model       = ~+1,
                                     brier.cens.model = ~+1,
                                     brier.args       = list(),
                                     cv.split.index   = NULL,
                                     ...) { ## {{{
  ## ------------------------------------------------------------------
  ## Small helpers (identical to brier_binregStrata)
  ## ------------------------------------------------------------------
  extract_cluster_id <- function(f) {
    tt     <- terms(f, specials = "cluster")
    cl_idx <- attr(tt, "specials")$cluster
    if (is.null(cl_idx)) return(NULL)
    cl_term <- attr(tt, "variables")[[cl_idx + 1L]]
    as.character(cl_term[[2]])
  }
  drop_cluster <- function(f) {
    tt     <- terms(f, specials = "cluster")
    cl_idx <- attr(tt, "specials")$cluster
    if (is.null(cl_idx)) return(f)
    cl_var <- attr(tt, "variables")[[cl_idx + 1L]]
    update(f, paste("~ . -", deparse(cl_var)))
  }
  ## ------------------------------------------------------------------
  ## Parse top-level formula
  ## ------------------------------------------------------------------
  lhs_str  <- deparse(formula[[2]])
  lhs_call <- formula[[2]]
  if (length(lhs_call) == 4L)
    stop("brier_binregStrata_loop does not support delayed entry (left-truncation).\n",
         "  Found: ", deparse(lhs_call))
  time_var   <- as.character(lhs_call[[2]])
  status_var <- as.character(lhs_call[[3]])
  id         <- extract_cluster_id(formula)
  if (is.null(id)) {
    id         <- "id__"
    data[[id]] <- seq_len(nrow(data))
    formula    <- update(formula, paste("~ . + cluster(id__)"))
  }
  base_resmean <- as.formula(paste(lhs_str, "~ +1 + cluster(", id, ")"))
  ## ------------------------------------------------------------------
  ## Parse rhs list
  ## ------------------------------------------------------------------
  formula_rhs_clean <- drop_cluster(
    reformulate(attr(terms(formula), "term.labels"), intercept = FALSE))
  if (is.null(rhs))        rhs <- list(model1 = formula_rhs_clean)
  if (!is.list(rhs))       rhs <- list(model1 = rhs)
  if (is.null(names(rhs)) || any(names(rhs) == ""))
    names(rhs) <- paste0("model", seq_along(rhs))
  rhs  <- lapply(rhs,  drop_cluster)
  rhs0 <- drop_cluster(rhs0)
  build_full <- function(f) {
    labs    <- attr(terms(f), "term.labels")
    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
    as.formula(paste(lhs_str, "~", rhs_str, "+ cluster(", id, ")"))
  }
  if (!id %in% names(data)) stop("id variable '", id, "' not found in data")
  ## ------------------------------------------------------------------
  ## CV fold index -- computed once, reused for all time points
  ## ------------------------------------------------------------------
  as_fold_list <- function(x) {
    if (is.list(x))                      return(x)
    if (is.numeric(x) || is.integer(x)) return(split(seq_along(x), x))
    stop("Unsupported cv.split.index format")
  }
  n  <- nrow(data)
  dd <- if (is.null(cv.split.index)) folds(n, fold) else as_fold_list(cv.split.index)
  data$fold <- 0L
  for (k in seq_len(fold)) data$fold[dd[[k]]] <- k
  strata_vec <- data$fold - 1L
  build_strata_formula <- function(f) {
    labs    <- attr(terms(f), "term.labels")
    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
    as.formula(paste(lhs_str, "~", rhs_str,
                     "+ strata(fold) + cluster(", id, ")"))
  }
  predict_cvS <- function(fit, newdata, se = FALSE) {
    as.numeric(predict(fit, newdata, se = se))
  }
  ## ------------------------------------------------------------------
  ## cv_iid_correctS -- block-sparse version (identical to brier_binregStrata)
  ## ------------------------------------------------------------------
  cv_iid_correctS <- function(brier_fit, outff_fit, newdata,
                               pred_vec, ipcw_wt_vec, outcome_vec,
                               binreg_model) {
    nstrata <- outff_fit$nstrata
    p       <- length(outff_fit$coef) / nstrata
    n       <- length(pred_vec)
    des        <- update_design(outff_fit$design, data = newdata,
                                       response = FALSE)
    X          <- des$x
    strata_idx <- if (!is.null(des$strata)) {
      as.integer(des$strata) - 1L
    } else if ("fold" %in% names(newdata)) {
      as.integer(newdata$fold) - 1L
    } else {
      stop("Cannot determine stratum for newdata: needs 'fold' column.")
    }
    Xs <- matrix(0.0, n, nstrata * p)
    for (s in seq_len(nstrata)) {
      idx  <- which(strata_idx == (s - 1L))
      cols <- (s - 1L) * p + seq_len(p)
      Xs[idx, cols] <- X[idx, , drop = FALSE]
    }
    mu_prime <- switch(binreg_model,
      logit = pred_vec * (1 - pred_vec),
      exp   = pred_vec,
      lin   = rep(1.0, n),
      stop("unrecognised binreg model '", binreg_model, "'")
    )
    resid       <- (outcome_vec - pred_vec) * ipcw_wt_vec
    Dbrier_full <- -2 * colSums(Xs * mu_prime * resid) / n
    iid(brier_fit) + iid(outff_fit) %*% Dbrier_full
  }
  ## ------------------------------------------------------------------
  ## resmeanIPCW wrapper / log-scale estimate helper
  ## ------------------------------------------------------------------
  do_resmean <- function(dat, Ydirect, tt) {
    do.call(resmeanIPCW,
            c(list(base_resmean, dat, time = tt,
                   Ydirect    = Ydirect,
                   cens.model = brier.cens.model,
                   model      = "lin"),
              brier.args))
  }
  log_est <- function(fit, iid_corrected = NULL) {
    if (!is.null(iid_corrected)) {
      nn <- nrow(iid_corrected)
      e <- lava::estimate(NULL,
                          coef = fit$coef,
                          IC   = iid_corrected * nn,
                          f    = log)
    } else {
      e <- estimate(fit, f = log)
    }
    list(coef = as.numeric(e$coef),
         se   = as.numeric(diag(vcov(e))^.5),
         iid  = iid(e))
  }

  ## ------------------------------------------------------------------
  ## Explicit nested for-loops: outer over time, inner over rhs models
  ## ------------------------------------------------------------------
  cl       <- match.call()
  ntime    <- length(time)
  nmodels  <- length(rhs)
  results  <- vector("list", ntime)
  names(results) <- as.character(time)

  for (ti in seq_len(ntime)) {
    tt <- time[ti]

    ## ---------------- null model: apparent ----------------
    outb         <- binreg(build_full(rhs0), data, time = tt, ...)
    binreg_model <- outb$model
    outcome      <- as.numeric(outb$Y)
    ipcw_wt_b    <- as.numeric(outb$IPCW)
    predb        <- predict(outb, data, se = FALSE)
    data$brier_  <- (outcome - predb)^2
    bbrier       <- do_resmean(data, data$brier_, tt)
    bbrier_log   <- log_est(bbrier)

    ## ---------------- null model: CV ----------------
    f0_strata  <- build_strata_formula(rhs0)
    outfb      <- binregStrata(f0_strata, data = data, time = tt,
                                strata     = strata_vec,
                                cens.model = cens.model,
                                cv.fold    = TRUE, ...)
    predbcv     <- predict_cvS(outfb, data)
    data$brier_ <- (outcome - predbcv)^2
    bbrierCV    <- do_resmean(data, data$brier_, tt)
    iid.bbrierCV <- cv_iid_correctS(bbrierCV, outfb, data,
                                     predbcv, ipcw_wt_b, outcome,
                                     binreg_model)
    bbrierCV_log <- log_est(bbrierCV, iid_corrected = iid.bbrierCV)

    null_res <- list(
      coef = c(bbrier   = bbrier_log$coef,
               bbrierCV = bbrierCV_log$coef),
      se   = c(bbrier   = bbrier_log$se,
               bbrierCV = bbrierCV_log$se),
      iid  = list(bbrierCV = bbrierCV_log$iid)
    )

    ## ---------------- candidate models: explicit for-loop ----------------
    model_res        <- vector("list", nmodels)
    names(model_res) <- names(rhs)

    for (mi in seq_len(nmodels)) {
      f <- rhs[[mi]]

      ## apparent
      out         <- binreg(build_full(f), data, time = tt, ...)
      outcome_m   <- as.numeric(out$Y)
      ipcw_wt_m   <- as.numeric(out$IPCW)
      pred        <- predict(out, data, se = FALSE)
      data$brier_ <- (outcome_m - pred)^2
      brier       <- do_resmean(data, data$brier_, tt)
      brier_log   <- log_est(brier)
      dbrier      <- brier_log$coef - bbrier_log$coef
      se.dbrier   <- as.numeric(crossprod(brier_log$iid - bbrier_log$iid)^.5)

      ## CV
      f_strata <- build_strata_formula(f)
      outff    <- binregStrata(f_strata, data = data, time = tt,
                                strata     = strata_vec,
                                cens.model = cens.model,
                                cv.fold    = TRUE, ...)
      pred        <- predict_cvS(outff, data)
      data$brier_ <- (outcome_m - pred)^2
      brierCV     <- do_resmean(data, data$brier_, tt)
      iid.brierCV <- cv_iid_correctS(brierCV, outff, data,
                                      pred, ipcw_wt_m, outcome_m,
                                      binreg_model)
      brierCV_log <- log_est(brierCV, iid_corrected = iid.brierCV)
      dbrierCV    <- brierCV_log$coef - bbrierCV_log$coef
      se.dbrierCV <- as.numeric(crossprod(brierCV_log$iid - bbrierCV_log$iid)^.5)

      model_res[[mi]] <- list(
        coef  = c(brier     = brier_log$coef,
                  brierCV   = brierCV_log$coef),
        se    = c(brier     = brier_log$se,
                  brierCV   = brierCV_log$se),
        dcoef = c(dbrier    = dbrier,
                  dbrierCV  = dbrierCV),
        sed   = c(se.dbrier   = se.dbrier,
                  se.dbrierCV = se.dbrierCV),
        iid   = list(brierCV = brierCV_log$iid)
      )
    } ## end inner for over models

    ## iid matrix: n x (1 + nmodels)
    iid_cols <- vector("list", nmodels + 1L)
    iid_cols[[1]] <- bbrierCV_log$iid
    for (mi in seq_len(nmodels)) iid_cols[[mi + 1L]] <- model_res[[mi]]$iid$brierCV
    iid_mat <- do.call(cbind, iid_cols)
    colnames(iid_mat) <- c("null", names(rhs))

    results[[ti]] <- structure(
      list(null             = null_res,
           models           = model_res,
           iid_mat          = iid_mat,
           cv.split.index   = dd,
           time             = tt,
           fold             = fold,
           rhs              = rhs,
           rhs0             = rhs0,
           model            = binreg_model,
           cens.model       = cens.model,
           brier.cens.model = brier.cens.model),
      class = "binregCV"
    )
  } ## end outer for over time

  ## ------------------------------------------------------------------
  ## Dispatch: single or multiple time points
  ## ------------------------------------------------------------------
  if (ntime == 1L) {
    out      <- results[[1]]
    out$call <- cl
    out
  } else {
    res        <- results
    res$call   <- cl
    res$rhs    <- rhs
    res$rhs0   <- rhs0
    class(res) <- "binregCV_list"
    res
  }
} ## }}}

###brier_binreg <- function(formula, data, time,
###                               fold             = 5,
###                               rhs              = NULL,
###                               rhs0             = ~+1,
###                               cens.model       = ~+1,
###                               brier.cens.model = ~+1,
###                               brier.args       = list(),
###                               cv.split.index   = NULL,
###                               ...) { ## {{{
###
###  ## ------------------------------------------------------------------
###  ## Small helpers
###  ## ------------------------------------------------------------------
###  extract_cluster_id <- function(f) {
###    tt     <- terms(f, specials = "cluster")
###    cl_idx <- attr(tt, "specials")$cluster
###    if (is.null(cl_idx)) return(NULL)
###    cl_term <- attr(tt, "variables")[[cl_idx + 1L]]
###    as.character(cl_term[[2]])
###  }
###  drop_cluster <- function(f) {
###    tt     <- terms(f, specials = "cluster")
###    cl_idx <- attr(tt, "specials")$cluster
###    if (is.null(cl_idx)) return(f)
###    cl_var <- attr(tt, "variables")[[cl_idx + 1L]]
###    update(f, paste("~ . -", deparse(cl_var)))
###  }
###
###  ## ------------------------------------------------------------------
###  ## Parse top-level formula
###  ## ------------------------------------------------------------------
###  lhs_str  <- deparse(formula[[2]])
###  lhs_call <- formula[[2]]
###  if (length(lhs_call) == 4L)
###    stop("brier_binreg does not support delayed entry (left-truncation).\n",
###         "  Found: ", deparse(lhs_call))
###
###  time_var   <- as.character(lhs_call[[2]])
###  status_var <- as.character(lhs_call[[3]])
###  id         <- extract_cluster_id(formula)
###
###  if (is.null(id)) {
###    id         <- "id__"
###    data[[id]] <- seq_len(nrow(data))
###    formula    <- update(formula, paste("~ . + cluster(id__)"))
###  }
###  base_resmean <- as.formula(paste(lhs_str, "~ +1 + cluster(", id, ")"))
###
###  ## ------------------------------------------------------------------
###  ## Parse rhs list
###  ## ------------------------------------------------------------------
###  formula_rhs_clean <- drop_cluster(
###    reformulate(attr(terms(formula), "term.labels"), intercept = FALSE))
###
###  if (is.null(rhs))        rhs <- list(model1 = formula_rhs_clean)
###  if (!is.list(rhs))       rhs <- list(model1 = rhs)
###  if (is.null(names(rhs)) || any(names(rhs) == ""))
###    names(rhs) <- paste0("model", seq_along(rhs))
###
###  rhs  <- lapply(rhs,  drop_cluster)
###  rhs0 <- drop_cluster(rhs0)
###
###  ## build_full: attach lhs and cluster() to a bare rhs formula
###  build_full <- function(f) {
###    labs    <- attr(terms(f), "term.labels")
###    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
###    as.formula(paste(lhs_str, "~", rhs_str, "+ cluster(", id, ")"))
###  }
###
###  if (!id %in% names(data)) stop("id variable '", id, "' not found in data")
###
###  ## ------------------------------------------------------------------
###  ## CV fold index -- computed once, reused for all time points
###  ## ------------------------------------------------------------------
###  as_fold_list <- function(x) {
###    if (is.list(x))                      return(x)
###    if (is.numeric(x) || is.integer(x)) return(split(seq_along(x), x))
###    stop("Unsupported cv.split.index format")
###  }
###
###  n  <- nrow(data)
###  dd <- if (is.null(cv.split.index)) folds(n, fold) else as_fold_list(cv.split.index)
###
###  ## Attach fold membership to data (0-based stratum index for binregStrata)
###  ## strata(fold) in cens.model will use the 1-based fold column.
###  data$fold <- 0L
###  for (k in seq_len(fold)) data$fold[dd[[k]]] <- k
###  ## 0-based stratum vector passed to binregStrata via strata argument
###  strata_vec <- data$fold - 1L   # 0 .. fold-1
###
###  ## ------------------------------------------------------------------
###  ## build_strata_formula: attach strata(fold) to the rhs for cens.model
###  ## and keep cluster() for binregStrata
###  ## ------------------------------------------------------------------
###  build_strata_formula <- function(f) {
###    ## f is already drop_cluster'd; we re-attach cluster and add strata(fold)
###    labs    <- attr(terms(f), "term.labels")
###    rhs_str <- if (length(labs) == 0) "1" else paste(labs, collapse = " + ")
###    as.formula(paste(lhs_str, "~", rhs_str,
###                     "+ strata(fold) + cluster(", id, ")"))
###  }
###
###  ## ------------------------------------------------------------------
###  ## predict_cvS: get OOF predictions from a binregStrata(cv.fold=TRUE) fit
###  ##
###  ## binregStrata with cv.fold=TRUE fits beta_s on complement of stratum s.
###  ## To predict for stratum s we simply call predict() on the full newdata:
###  ## the predict.binregStrata routes each obs to its own stratum's beta_s,
###  ## which was trained on the other strata -- exactly the OOF prediction.
###  ## ------------------------------------------------------------------
###  predict_cvS <- function(fit, newdata, se = FALSE) {
###    as.numeric(predict(fit, newdata, se = se))
###  }
###
###  ## ------------------------------------------------------------------
###  ## cv_iid_correctS: delta-method IID correction for binregStrata CV fits
###  ##
###  ## binregStrata iid has shape n_id x (nstrata * p), blocked as
###  ##   [ iid_beta_1 | iid_beta_2 | ... | iid_beta_K ]
###  ## each block is n_id x p.
###  ##
###  ## For stratum s, beta_s was fitted on complement rows (strata != s-1).
###  ## The Brier gradient w.r.t. beta_s is:
###  ##   D_s = -2 * mean_over_ALL_n( mu'_i * (outcome_i - pred_i) * ipcw_i * x_i )
###  ## but only obs in stratum s contribute through their OOF prediction
###  ## (obs not in s used a different beta), so in practice we restrict to
###  ## stratum-s obs for the gradient of the Brier w.r.t. beta_s.
###  ##
###  ## More precisely:
###  ##   pred_i = mu(x_i' beta_{s_i})   where s_i = stratum of obs i
###  ## so d(pred_i)/d(beta_s) = mu'_i * x_i  iff  s_i == s,  else 0.
###  ##
###  ## Therefore:
###  ##   D_s = -2/n * sum_{i: s_i = s} mu'_i * (outcome_i - pred_i) * ipcw_i * x_i
###  ##
###  ## and the IID correction is:
###  ##   iid(brier_fit) + sum_s  iid_beta_s  %*%  D_s
###  ## ------------------------------------------------------------------
###  cv_iid_correctS <- function(brier_fit, outff_fit, newdata,
###                               pred_vec, ipcw_wt_vec, outcome_vec,
###                               binreg_model) {
###    nstrata <- outff_fit$nstrata
###    p       <- length(outff_fit$coef) / nstrata
###    n       <- length(pred_vec)
###
###    ## design matrix for all obs (rows ordered as in data / newdata)
###    des  <- update_design(outff_fit$design, data = newdata, response = FALSE)
###    X    <- des$x                               # n x p
###    ## 0-based stratum index for each obs
###    if (!is.null(des$strata)) {
###      strata_idx <- as.integer(des$strata) - 1L
###    } else {
###      strata_idx <- outff_fit$strata_orig       # already 0-based after ord
###    }
###
###    mu_prime <- switch(binreg_model,
###      logit = pred_vec * (1 - pred_vec),
###      exp   = pred_vec,
###      lin   = rep(1.0, n),
###      stop("unrecognised binreg model '", binreg_model, "'")
###    )
###
###    iid_full <- iid(outff_fit)    # n_id x (nstrata * p)
###    n_id     <- nrow(iid_full)
###
###    iid_adj <- matrix(0.0, n_id, 1)
###
###    for (s in seq_len(nstrata)) {
###      ## obs whose OOF prediction came from beta_s (i.e. obs in stratum s-1)
###      idx_s   <- which(strata_idx == (s - 1L))
###      if (length(idx_s) == 0L) next
###
###      x_s       <- X[idx_s, , drop = FALSE]
###      mu_s      <- mu_prime[idx_s]
###      resid_s   <- (outcome_vec[idx_s] - pred_vec[idx_s]) * ipcw_wt_vec[idx_s]
###
###      ## gradient of Brier w.r.t. beta_s  (p-vector)
###      D_s <- -2 / n * colSums(x_s * mu_s * resid_s)
###
###      ## iid block for beta_s
###      cols    <- (s - 1L) * p + seq_len(p)
###      iid_s   <- iid_full[, cols, drop = FALSE]   # n_id x p
###
######    print(D_s)
######    print(head(iid_s))
###      iid_adj <- iid_adj + iid_s %*% D_s          # n_id x 1
###    }
######    print(head(iid_adj))
###
###    iid(brier_fit) + iid_adj
###  }
###
###
###  ## ------------------------------------------------------------------
##### cv_iid_correctS: delta-method IID correction for binregStrata CV fits
#####
##### Key alignment facts:
#####   iid(outff_fit)  : n_id x (nstrata*p), rows in REORDERED id order
#####                     (outff_fit$id is xx$id after Cox-prep reordering)
#####                     outff_fit$name.id maps row -> original cluster label
#####   iid(brier_fit)  : n x 1, rows in ORIGINAL data order,
#####                     indexed by original cluster id
#####
##### We must accumulate iid_adj in the same row space as iid(brier_fit).
#####
##### Strategy:
#####   1. Compute D_s (p-vector gradient) for each stratum s -- uses
#####      original-data-order vectors (pred_vec, ipcw_wt_vec, outcome_vec,
#####      and X from update_design on newdata which is in original order).
#####   2. For each stratum, iid_s = iid_full[, cols_s] is n_id x p in
#####      REORDERED order.  Map it back to original order via name.id.
#####   3. Add iid_s %*% D_s into iid_adj at the right rows.
#####   4. Return iid(brier_fit) + iid_adj, both in original order.
##### ------------------------------------------------------------------
###cv_iid_correctS <- function(brier_fit, outff_fit, newdata,
###                             pred_vec, ipcw_wt_vec, outcome_vec,
###                             binreg_model) { ## {{{ 
###  nstrata  <- outff_fit$nstrata
###  p        <- length(outff_fit$coef) / nstrata
###  n        <- length(pred_vec)                  # original data n
###  iid_b    <- iid(brier_fit)                    # n_id_brier x 1, original order
###  n_id_b   <- nrow(iid_b)
###
###  ## ------------------------------------------------------------------
###  ## Design matrix in ORIGINAL data order (newdata = original data)
###  ## ------------------------------------------------------------------
###  des      <- update_design(outff_fit$design, data = newdata,
###                                   response = FALSE)
###  X        <- des$x                             # n x p, original order
###
###  ## 0-based stratum index in original data order
###  strata_idx <- if (!is.null(des$strata)) {
###    as.integer(des$strata) - 1L
###  } else if ("fold" %in% names(newdata)) {
###    as.integer(newdata$fold) - 1L
###  } else {
###    ## strata_orig is in reordered frame -- use fold column as fallback
###    stop("Cannot determine stratum membership for newdata in cv_iid_correctS.\n",
###         "  Ensure newdata contains a 'fold' column (1-based).")
###  }
###
###  ## ------------------------------------------------------------------
###  ## mu' in original data order
###  ## ------------------------------------------------------------------
###  mu_prime <- switch(binreg_model,
###    logit = pred_vec * (1 - pred_vec),
###    exp   = pred_vec,
###    lin   = rep(1.0, n),
###    stop("unrecognised binreg model '", binreg_model, "'")
###  )
###
###  ## ------------------------------------------------------------------
###  ## iid of outff_fit: n_id x (nstrata*p), rows in REORDERED order.
###  ## Reorder to original cluster order using name.id / orig.id.
###  ##
###  ## outff_fit$name.id : character vector of length n_id giving cluster
###  ##                     labels in REORDERED frame order
###  ## We need to map these to the row indices of iid(brier_fit).
###  ## brier_fit is a resmeanIPCW fit whose iid rows correspond to the
###  ## unique cluster ids in the original data in the order they first
###  ## appear (standard mets convention).
###  ## ------------------------------------------------------------------
###  iid_full   <- iid(outff_fit)                  # n_id x (nstrata*p), reordered
###  n_id       <- nrow(iid_full)
###
###  ## Build a re-indexing vector: for each row r of iid_full (reordered),
###  ## which row of iid_b (original order) does it belong to?
###  ## outff_fit$name.id gives cluster labels in reordered frame.
###  ## We use match() against the cluster labels of brier_fit.
###  ## resmeanIPCW stores cluster id in $id (integer, 0-based or 1-based)
###  ## and $name.id.  Use name.id when available, else fall back to id.
###  if (!is.null(outff_fit$name.id) && !is.null(brier_fit$name.id)) {
###    ## both have string cluster labels -- align on those
###    reorder_idx <- match(outff_fit$name.id, brier_fit$name.id)
###  } else {
###    ## integer cluster ids -- outff_fit$id is in reordered frame,
###    ## values are 0-based cluster indices matching brier_fit$id
###    ## resmeanIPCW $id is 1-based; subtract 1 to get 0-based index
###    reorder_idx <- match(outff_fit$id + 1L,
###                         sort(unique(brier_fit$id)))
###    if (anyNA(reorder_idx)) {
###      ## last resort: assume same ordering
###      reorder_idx <- seq_len(n_id)
###    }
###  }
###
###  ## iid_full reordered to match brier_fit row space
###  ## If n_id == n_id_b and reorder_idx is a permutation, this is exact.
###  ## If some cluster ids appear in one but not the other (edge case),
###  ## we restrict to the intersection -- but normally they match.
###  iid_full_orig <- matrix(0.0, n_id_b, ncol(iid_full))
###  valid         <- !is.na(reorder_idx) & reorder_idx <= n_id_b
###  iid_full_orig[reorder_idx[valid], ] <- iid_full[valid, ]
###
###  ## ------------------------------------------------------------------
###  ## Accumulate iid adjustment stratum by stratum
###  ## ------------------------------------------------------------------
###  iid_adj <- matrix(0.0, n_id_b, 1)
###
###  for (s in seq_len(nstrata)) {
###    ## obs in stratum s (their OOF pred came from beta_s)
###    idx_s <- which(strata_idx == (s - 1L))
###    if (length(idx_s) == 0L) next
###
###    x_s     <- X[idx_s, , drop = FALSE]        # n_s x p, original order
###    mu_s    <- mu_prime[idx_s]
###    resid_s <- (outcome_vec[idx_s] - pred_vec[idx_s]) * ipcw_wt_vec[idx_s]
###
###    ## gradient of Brier w.r.t. beta_s (p-vector)
###    D_s <- -2 / n * colSums(x_s * mu_s * resid_s)
###
###    ## iid block for beta_s, now in original-data row order
###    cols    <- (s - 1L) * p + seq_len(p)
###    iid_s   <- iid_full_orig[, cols, drop = FALSE]   # n_id_b x p
###
###    iid_adj <- iid_adj + iid_s %*% D_s               # n_id_b x 1
###  }
###
###  iid_b + iid_adj
###} ## }}} 
###
###
###cv_iid_correctS <- function(brier_fit, outff_fit, newdata,
###                             pred_vec, ipcw_wt_vec, outcome_vec,
###                             binreg_model) { ## {{{ 
###  nstrata <- outff_fit$nstrata
###  p       <- length(outff_fit$coef) / nstrata
###  n       <- length(pred_vec)
###
###  ## ------------------------------------------------------------------
###  ## Design matrix in original data order
###  ## ------------------------------------------------------------------
###  des        <- update_design(outff_fit$design, data = newdata,
###                                     response = FALSE)
###  X          <- des$x                           # n x p, original order
###
###  ## 0-based stratum index, original order
###  strata_idx <- if (!is.null(des$strata)) {
###    as.integer(des$strata) - 1L
###  } else if ("fold" %in% names(newdata)) {
###    as.integer(newdata$fold) - 1L
###  } else {
###    stop("Cannot determine stratum for newdata: needs 'fold' column.")
###  }
###
###  ## ------------------------------------------------------------------
###  ## Build full block-diagonal Xs: n x (nstrata*p)
###  ## For cv.fold: obs i in stratum s_i contributes only to block s_i
###  ## (its OOF prediction came from beta_{s_i}).
###  ## This mirrors the Xs construction inside binregStrata for MGCiid,
###  ## but here each obs goes in its OWN stratum block (not complement),
###  ## because d(pred_i)/d(beta_s) is nonzero only when s == s_i.
###  ## ------------------------------------------------------------------
###  Xs <- matrix(0.0, n, nstrata * p)
###  for (s in seq_len(nstrata)) {
###    idx  <- which(strata_idx == (s - 1L))
###    cols <- (s - 1L) * p + seq_len(p)
###    Xs[idx, cols] <- X[idx, , drop = FALSE]
###  }
###
###  ## ------------------------------------------------------------------
###  ## mu' in original order
###  ## ------------------------------------------------------------------
###  mu_prime <- switch(binreg_model,
###    logit = pred_vec * (1 - pred_vec),
###    exp   = pred_vec,
###    lin   = rep(1.0, n),
###    stop("unrecognised binreg model '", binreg_model, "'")
###  )
###
###  ## ------------------------------------------------------------------
###  ## Full gradient vector: nstrata*p
###  ## Dbrier_full[cols_s] = -2/n * sum_{i: s_i=s} mu'_i*(Y_i-pred_i)*ipcw_i * x_i
###  ## All other blocks are zero because d(pred_i)/d(beta_s)=0 for s != s_i.
###  ## ------------------------------------------------------------------
###  resid      <- (outcome_vec - pred_vec) * ipcw_wt_vec   # n-vector
###  Dbrier_full <- -2 * colSums(Xs * mu_prime * resid)/n # nstrata*p vector
###
###  ## ------------------------------------------------------------------
###  ## iid(outff_fit): n_id x (nstrata*p)
###  ## Single matrix multiply -- structurally identical to brier_binreg's
###  ##   iid(outff_fit) %*% Dbrier
###  ## ------------------------------------------------------------------
######  print(head(iid(outff_fit))) 
######  print(Dbrier_full)
######  print(head( iid(outff_fit) %*% Dbrier_full))
###  iid(brier_fit) + iid(outff_fit) %*% Dbrier_full
###}
###
###
###  ## ------------------------------------------------------------------
###  ## resmeanIPCW wrapper
###  ## ------------------------------------------------------------------
###  do_resmean <- function(dat, Ydirect, tt) {
###    do.call(resmeanIPCW,
###            c(list(base_resmean, dat, time = tt,
###                   Ydirect    = Ydirect,
###                   cens.model = brier.cens.model,
###                   model      = "lin"),
###              brier.args))
###  } ## }}} 
###
###  ## ------------------------------------------------------------------
###  ## log-scale estimate helper
###  ## ------------------------------------------------------------------
###  log_est <- function(fit, iid_corrected = NULL) {
###    if (!is.null(iid_corrected)) {
###      n <- nrow(iid_corrected)
###      e <- lava::estimate(NULL,
###                          coef = fit$coef,
###                          IC   = iid_corrected * n,
###                          f    = log)
###    } else {
###      e <- estimate(fit, f = log)
###    }
###    list(coef = as.numeric(e$coef),
###         se   = as.numeric(diag(vcov(e))^.5),
###         iid  = iid(e))
###  }
###
###  ## ------------------------------------------------------------------
###  ## run_one: compute Brier scores at a single time point
###  ## ------------------------------------------------------------------
###  run_one <- function(tt) { ## {{{
###
###    ## ----------------------------------------------------------------
###    ## Apparent (no-CV) null model via plain binreg
###    ## ----------------------------------------------------------------
###    outb         <- binreg(build_full(rhs0), data, time = tt, ...)
###    binreg_model <- outb$model
###    outcome      <- as.numeric(outb$Y)
###    ipcw_wt_b    <- as.numeric(outb$IPCW)
###    predb        <- predict(outb, data, se = FALSE)
###    data$brier_  <- (outcome - predb)^2
###    bbrier       <- do_resmean(data, data$brier_, tt)
###    bbrier_log   <- log_est(bbrier)
###
###    ## ----------------------------------------------------------------
###    ## CV null model via binregStrata(cv.fold = TRUE)
###    ## The fold column acts as the stratum; cv.fold=TRUE means beta_s
###    ## is estimated on all obs NOT in fold s.
###    ## ----------------------------------------------------------------
###    f0_strata  <- build_strata_formula(rhs0)
###    outfb      <- binregStrata(f0_strata, data = data, time = tt,
###                                strata     = strata_vec,
###                                cens.model = cens.model,
###                                cv.fold    = TRUE, ...)
######    print(estimate(outfb))
###    predbcv    <- predict_cvS(outfb, data)
###    data$brier_ <- (outcome - predbcv)^2
###    bbrierCV    <- do_resmean(data, data$brier_, tt)
###
###    iid.bbrierCV <- cv_iid_correctS(bbrierCV, outfb, data,
###                                     predbcv, ipcw_wt_b, outcome,
###                                     binreg_model)
###    bbrierCV_log <- log_est(bbrierCV, iid_corrected = iid.bbrierCV)
###
###    null_res <- list(
###      coef = c(bbrier   = bbrier_log$coef,
###               bbrierCV = bbrierCV_log$coef),
###      se   = c(bbrier   = bbrier_log$se,
###               bbrierCV = bbrierCV_log$se),
###      iid  = list(bbrierCV = bbrierCV_log$iid)
###    )
###
###    ## ----------------------------------------------------------------
###    ## Loop over candidate models
###    ## ----------------------------------------------------------------
###    model_res        <- vector("list", length(rhs))
###    names(model_res) <- names(rhs)
###
###    for (m in seq_along(rhs)) {
###      f <- rhs[[m]]
###
###      ## --- apparent -------------------------------------------------
###      out        <- binreg(build_full(f), data, time = tt, ...)
###      outcome_m  <- as.numeric(out$Y)
###      ipcw_wt_m  <- as.numeric(out$IPCW)
###      pred       <- predict(out, data, se = FALSE)
###      data$brier_ <- (outcome_m - pred)^2
###      brier       <- do_resmean(data, data$brier_, tt)
###      brier_log   <- log_est(brier)
###      dbrier      <- brier_log$coef - bbrier_log$coef
###      se.dbrier   <- as.numeric(crossprod(brier_log$iid - bbrier_log$iid)^.5)
###
###      ## --- CV via binregStrata(cv.fold = TRUE) ----------------------
###      f_strata <- build_strata_formula(f)
###      outff    <- binregStrata(f_strata, data = data, time = tt,
###                                strata     = strata_vec,
###                                cens.model = cens.model,
###                                cv.fold    = TRUE, ...)
######      print(estimate(outff))
###      pred        <- predict_cvS(outff, data)
###      data$brier_ <- (outcome_m - pred)^2
###      brierCV     <- do_resmean(data, data$brier_, tt)
###
###      iid.brierCV <- cv_iid_correctS(brierCV, outff, data,
###                                      pred, ipcw_wt_m, outcome_m,
###                                      binreg_model)
###      brierCV_log <- log_est(brierCV, iid_corrected = iid.brierCV)
###      dbrierCV    <- brierCV_log$coef - bbrierCV_log$coef
###      se.dbrierCV <- as.numeric(crossprod(brierCV_log$iid - bbrierCV_log$iid)^.5)
###
###      model_res[[m]] <- list(
###        coef  = c(brier     = brier_log$coef,
###                  brierCV   = brierCV_log$coef),
###        se    = c(brier     = brier_log$se,
###                  brierCV   = brierCV_log$se),
###        dcoef = c(dbrier    = dbrier,
###                  dbrierCV  = dbrierCV),
###        sed   = c(se.dbrier   = se.dbrier,
###                  se.dbrierCV = se.dbrierCV),
###        iid   = list(brierCV = brierCV_log$iid)
###      )
###    }
###
###    ## iid matrix: n x (1 + nmodels)  null | model1 | model2 | ...
###    iid_mat <- cbind(
###      null = bbrierCV_log$iid,
###      do.call(cbind, setNames(
###        lapply(model_res, function(m) m$iid$brierCV),
###        names(rhs)
###      ))
###    )
###
###    structure(
###      list(null             = null_res,
###           models           = model_res,
###           iid_mat          = iid_mat,
###           cv.split.index   = dd,
###           time             = tt,
###           fold             = fold,
###           rhs              = rhs,
###           rhs0             = rhs0,
###           model            = binreg_model,
###           cens.model       = cens.model,
###           brier.cens.model = brier.cens.model),
###      class = "binregCV"
###    )
###  } ## }}}
###
###  ## ------------------------------------------------------------------
###  ## Dispatch: single or multiple time points
###  ## ------------------------------------------------------------------
###  cl <- match.call()
###  if (length(time) == 1) {
###    out        <- run_one(time)
###    out$call   <- cl
###    class(out) <- "binregCV"
###    out
###  } else {
###    res        <- setNames(lapply(time, run_one), as.character(time))
###    res$call   <- cl
###    res$rhs    <- rhs
###    res$rhs0   <- rhs0
###    class(res) <- "binregCV_list"
###    res
###  }
###} ## }}}
###
## {{{  infrastructure for brier_binreg
## =============================================================================
## estimate.binregCV  -- lava-style joint estimate  (null + all CV models)
## =============================================================================



##' Joint estimate across CV models for a binregCV object
##'
##' @param x A `binregCV` object.
##' @param f Optional transformation function passed to `lava::estimate`.
##' @param ... Not currently used.
##' @export
estimate.binregCV <- function(x , f = NULL, ...) {
  coefs <- c(
    null = x$null$coef["bbrierCV"],
    setNames(
      sapply(x$models, function(m) m$coef["brierCV"]),
      names(x$models)
    )
  )
  iid_mat <- x$iid_mat   # n x (1 + nmodels)
  n       <- nrow(iid_mat)
  e       <- lava::estimate(NULL, coef = coefs, IC = iid_mat * n)
  if (!is.null(f)) e <- estimate(e, f = f)
  e
}


## =============================================================================
## summarize_brier_binregCV  (works for binregCV and binregCV_list)
## =============================================================================
summarize_brier_binregCV <- function(fit,
                                      transform  = NULL,
                                      conf.level = 0.95,
                                      type       = c("cv", "both")) {
  type <- match.arg(type)

  fit_call <- fit$call
  fit_rhs  <- fit$rhs
  fit_rhs0 <- fit$rhs0

  if (inherits(fit, "binregCV")) {
    tname    <- as.character(fit$time)
    fit_call <- fit$call
    fit_rhs  <- fit$rhs
    fit_rhs0 <- fit$rhs0
    fit      <- setNames(list(fit), tname)
  }

  times <- names(fit)[!(names(fit) %in% c("call", "rhs", "rhs0"))]
  if (length(times) == 0L) stop("No time points found in fit")

  pdiff <- lava::pairwise.diff

  col_se <- function(cm) grep("^Std", colnames(cm), value = TRUE)[1]
  col_lo <- function(cm) grep("%",    colnames(cm), value = TRUE)[1]
  col_hi <- function(cm) grep("%",    colnames(cm), value = TRUE)[2]
  col_p  <- function(cm) grep("^P",   colnames(cm), value = TRUE)[1]

  parse_lava_contrast <- function(rn) {
    m    <- gregexpr("(?<=\\[)[^\\]]+(?=\\])", rn, perl = TRUE)
    hits <- regmatches(rn, m)[[1]]
    c(reference = hits[1], model = hits[2])
  }
  strip_t <- function(x) sub("^t[^.]+\\.", "", x)

  coefmat_df <- function(e, time_val) {
    cm <- e$coefmat
    data.frame(
      time  = as.numeric(time_val),
      model = names(coef(e)),
      Brier = as.numeric(cm[, "Estimate"]),
      SE    = as.numeric(cm[, col_se(cm)]),
      lower = as.numeric(cm[, col_lo(cm)]),
      upper = as.numeric(cm[, col_hi(cm)]),
      stringsAsFactors = FALSE
    )
  }

  apparent_coefmat_df <- function(obj, time_val, tfun, cl) {
    z <- qnorm(1 - (1 - cl) / 2)
    coefs <- c(
      null = obj$null$coef["bbrier"],
      setNames(
        sapply(names(obj$models), function(nm) obj$models[[nm]]$coef["brier"]),
        names(obj$models)
      )
    )
    ses <- c(
      null = obj$null$se["bbrier"],
      setNames(
        sapply(names(obj$models), function(nm) obj$models[[nm]]$se["brier"]),
        names(obj$models)
      )
    )
    lower_log <- coefs - z * ses
    upper_log <- coefs + z * ses
    if (!is.null(tfun)) {
      eps    <- sqrt(.Machine$double.eps)
      fprime <- (tfun(coefs + eps) - tfun(coefs - eps)) / (2 * eps)
      data.frame(time  = as.numeric(time_val),
                 model = names(coefs),
                 Brier = as.numeric(tfun(coefs)),
                 SE    = as.numeric(abs(fprime) * ses),
                 lower = as.numeric(tfun(lower_log)),
                 upper = as.numeric(tfun(upper_log)),
                 stringsAsFactors = FALSE)
    } else {
      data.frame(time  = as.numeric(time_val),
                 model = names(coefs),
                 Brier = as.numeric(coefs),
                 SE    = as.numeric(ses),
                 lower = as.numeric(lower_log),
                 upper = as.numeric(upper_log),
                 stringsAsFactors = FALSE)
    }
  }

  models_list          <- vector("list", length(times))
  comps_list           <- vector("list", length(times))
  apparent_models_list <- vector("list", length(times))
  names(models_list) <- names(comps_list) <-
    names(apparent_models_list) <- times

  for (tt in times) {
    obj <- fit[[tt]]

    e <- if (is.null(transform)) estimate(obj)
         else                    estimate(obj, f = transform)
    nmodels <- length(coef(e))
    models_list[[tt]] <- coefmat_df(e, tt)

    if (nmodels > 1L) {
      ec  <- estimate(e, -pdiff(nmodels))
      cmc <- ec$coefmat
      rn  <- rownames(cmc)
      parsed <- t(sapply(rn, parse_lava_contrast))
      comps_list[[tt]] <- data.frame(
        time        = as.numeric(tt),
        model       = strip_t(parsed[, "model"]),
        reference   = strip_t(parsed[, "reference"]),
        delta.Brier = as.numeric(cmc[, "Estimate"]),
        SE          = as.numeric(cmc[, col_se(cmc)]),
        lower       = as.numeric(cmc[, col_lo(cmc)]),
        upper       = as.numeric(cmc[, col_hi(cmc)]),
        P           = as.numeric(cmc[, col_p(cmc)]),
        stringsAsFactors = FALSE
      )
    }

    if (type == "both")
      apparent_models_list[[tt]] <- apparent_coefmat_df(obj, tt, transform, conf.level)
  }

  obj1       <- fit[[times[1]]]
  fit_fold   <- obj1$fold
  if (is.null(fit_rhs))  fit_rhs  <- obj1$rhs
  if (is.null(fit_rhs0)) fit_rhs0 <- obj1$rhs0
  model_list <- c(
    list(`null (rhs0)` = deparse(fit_rhs0)),
    setNames(lapply(fit_rhs, function(f) deparse(f[[length(f)]])),
             names(fit_rhs))
  )

  models_df <- do.call(rbind, models_list)
  comps_df  <- do.call(rbind, Filter(Negate(is.null), comps_list))
  if (is.null(comps_df)) comps_df <- data.frame()
  rownames(models_df) <- NULL
  if (nrow(comps_df) > 0) rownames(comps_df) <- NULL

  apparent_models_df <- if (type == "both")
    do.call(rbind, apparent_models_list) else NULL

  structure(
    list(models               = models_df,
         comparisons          = comps_df,
         apparent.models      = apparent_models_df,
         apparent.comparisons = NULL,
         type                 = type,
         transform            = transform,
         conf.level           = conf.level,
         times                = as.numeric(times),
         call                 = fit_call,
         fold                 = fit_fold,
         model_list           = model_list),
    class = "summary_binregCV_multi"
  )
}


## =============================================================================
## summary methods
## =============================================================================

##' Summary method for binregCV objects
##'
##' @param object A \code{binregCV} object, as returned by
##'   \code{\link{brier_binreg}} for a single evaluation time.
##' @param transform Optional back-transformation function applied to Brier
##'   scores and confidence limits, e.g. \code{exp} to undo the internal log
##'   scale.
##' @param conf.level Confidence level for the reported intervals (default
##'   0.95).
##' @param type Either \code{"cv"} (default) for cross-validated Brier scores
##'   only, or \code{"both"} to also include apparent (in-sample) scores.
##' @param ... Not currently used.
##' @export
summary.binregCV <- function(object, transform = NULL,
                              conf.level = 0.95,
                              type = c("cv", "both"), ...) {
  summarize_brier_binregCV(object,
                            transform  = transform,
                            conf.level = conf.level,
                            type       = match.arg(type))
}

##' Summary method for binregCV_list objects
##'
##' @param object A \code{binregCV_list} object, as returned by
##'   \code{\link{brier_binreg}} when \code{time} has length greater than
##'   one.
##' @param transform Optional back-transformation function applied to Brier
##'   scores and confidence limits, e.g. \code{exp} to undo the internal log
##'   scale.
##' @param conf.level Confidence level for the reported intervals (default
##'   0.95).
##' @param type Either \code{"cv"} (default) for cross-validated Brier scores
##'   only, or \code{"both"} to also include apparent (in-sample) scores.
##' @param ... Not currently used.
##' @export
summary.binregCV_list <- function(object, transform = NULL,
                                   conf.level = 0.95,
                                   type = c("cv", "both"), ...) {
  summarize_brier_binregCV(object,
                            transform  = transform,
                            conf.level = conf.level,
                            type       = match.arg(type))
}


## =============================================================================
## print.summary_binregCV_multi
## =============================================================================

##' Print method for summary_binregCV_multi objects
##'
##' @param x A `summary_binregCV_multi` object.
##' @param digits Number of digits to round to.
##' @param ... Not currently used.
##' @export
print.summary_binregCV_multi <- function(x, digits = 4, ...) {
  sep_line <- paste(rep("-", 56), collapse = "")

  rd <- function(df) {
    num_cols <- sapply(df, is.numeric)
    df[num_cols] <- lapply(df[num_cols], round, digits)
    df
  }

  scale_note <- if (is.null(x$transform)) "[log scale]" else "[transformed scale]"
  cat(sprintf("\nCross-validated Brier score summary  %s\n", scale_note))
  if (!is.null(x$call)) { cat("Call: "); print(x$call) }
  cat(sprintf("Evaluation time(s): %s   Folds: %d   CI level: %g%%\n",
              paste(x$times, collapse = ", "), x$fold, 100 * x$conf.level))
  cat("Estimator: binregStrata  |  IID: block-diagonal delta-method correction\n")

  cat("\nModels:\n")
  for (nm in names(x$model_list)) {
    rhs_str <- trimws(gsub("- 1", "", gsub("\\s+", " ", x$model_list[[nm]])))
    cat(sprintf("  %-18s : %s\n", nm, rhs_str))
  }
  cat("\n")

  if (!is.null(x$type) && x$type == "both" && !is.null(x$apparent.models)) {
    cat("Apparent (no CV) Brier scores:\n\n")
    disp_app <- rd(x$apparent.models)[, c("model", "time", "Brier", "SE",
                                           "lower", "upper")]
    for (tt in unique(disp_app$time)) {
      block      <- disp_app[disp_app$time == tt, , drop = FALSE]
      block$time <- NULL
      cat(sprintf("  time = %g\n", tt))
      print(block, row.names = FALSE, quote = FALSE)
      cat(sep_line, "\n")
    }
    cat("Note: apparent comparisons not shown (IID not stored for apparent fits).\n\n")
  }

  cat("CV Brier scores by model:\n\n")
  disp_mod <- rd(x$models)[, c("model", "time", "Brier", "SE", "lower", "upper")]
  for (tt in unique(disp_mod$time)) {
    block      <- disp_mod[disp_mod$time == tt, , drop = FALSE]
    block$time <- NULL
    cat(sprintf("  time = %g\n", tt))
    print(block, row.names = FALSE, quote = FALSE)
    cat(sep_line, "\n")
  }

  if (!is.null(x$comparisons) && nrow(x$comparisons) > 0L) {
    cat("\nModel comparisons (delta CV Brier):\n\n")
    comps <- rd(x$comparisons)
    for (tt in unique(comps$time)) {
      block <- comps[comps$time == tt, , drop = FALSE]
      cat(sprintf("  time = %g\n", tt))
      print(block, row.names = FALSE, quote = FALSE)
      cat(sep_line, "\n")
    }
  }

  if (is.null(x$transform))
    cat("\nNote: use summary(fit, transform = exp) for Brier scores on the original scale.\n")

  invisible(x)
}

## }}} 

## =============================================================================
## estimate.binregCV_list -- joint estimate across ALL time points and models
## =============================================================================
##' @export
estimate.binregCV_list <- function(x, f = NULL, ...) { ## {{{ 
  times <- names(x)[!(names(x) %in% c("call", "rhs", "rhs0"))]
  if (length(times) == 0L) stop("No time points found in object")

  ## Collect per-time coefficient vectors and iid matrices, with
  ## time-qualified names so e.g. "null" at t=50 and t=60 don't collide.
  coef_list <- vector("list", length(times))
  iid_list  <- vector("list", length(times))
  names(coef_list) <- names(iid_list) <- times

  n_ref <- NULL
  for (tt in times) {
    obj <- x[[tt]]
    coefs <- c(
      null = obj$null$coef["bbrierCV"],
      setNames(
        sapply(obj$models, function(m) m$coef["brierCV"]),
        names(obj$models)
      )
    )
    nm <- paste0("t", tt, ".", names(coefs))
    names(coefs) <- nm

    im <- obj$iid_mat                 ## n x (1+nmodels), cols: null, model1, ...
    colnames(im) <- nm

    if (is.null(n_ref)) {
      n_ref <- nrow(im)
    } else if (nrow(im) != n_ref) {
      stop("iid_mat row counts differ across time points (", n_ref,
           " vs ", nrow(im), " at time ", tt, "); cannot combine into a ",
           "single joint estimate. This typically means rows correspond ",
           "to different clusters/ids across time points.")
    }

    coef_list[[tt]] <- coefs
    iid_list[[tt]]  <- im
  }

  coefs_all   <- do.call(c, coef_list)
  iid_mat_all <- do.call(cbind, iid_list)   ## n x (ntimes * (1+nmodels))

  n <- nrow(iid_mat_all)
  e <- lava::estimate(NULL, coef = coefs_all, IC = iid_mat_all * n)
  if (!is.null(f)) e <- estimate(e, f = f)
  e
} ## }}} 

##' @export
or2prob <- function(OR,marg) {
    p1 <- marg[1]; p2 <- marg[2]
    if (length(marg)==1) p2 <- p1
    if (OR==1) {
        PP <- outer(c(p1,1-p1),c(p2,1-p2))
    } else {
        ## OR (p11*p00)/(p10*p01) = p11*(1+p11-p1-p2)/(p1-p11)*(p2-p11)
        ## (OR-1)p11^2-(OR(p1+p2)+(1-p1-p2))*p11+OR*p1*p2 = 0
        ## (1-OR)p11^2+(OR(p1+p2)+(1-p1-p2))*p11-OR*p1*p2 = 0
        a <- (1-OR)       
        b <- 1+(p1+p2)*(OR-1)
        ac <- -OR*p1*p2*a
        d <- sqrt(b^2-4*ac)
        ## Only one solution 
        p11 <- (-b+d)/(2*a)
        PP <- c(p11,p1-p11,p2-p11)
        PP <- c(PP,1-sum(PP))
    }
    structure(matrix(PP,2),marg=c(p1,p2))
}


##' Estimate parameters from odds-ratio
##'
##' Calculate tetrachoric correlation of probabilities from odds-ratio
##' @param P Joint probabilities or marginals (if OR is given)
##' @param OR Odds-ratio
##' @param approx If TRUE an approximation of the tetrachoric correlation is used
##' @param ... Additional arguments
##' @export
##' @aliases or2prob tetrachoric
##' @examples
##' tetrachoric(0.3,1.25) # Marginal p1=p2=0.3, OR=2
##' P <- matrix(c(0.1,0.2,0.2,0.5),2)
##' prod(diag(P))/prod(lava::revdiag(P))
##' ##mets:::assoc(P)
##' tetrachoric(P)
##' or2prob(2,0.1)
##' or2prob(2,c(0.1,0.2))
tetrachoric <- function(P,OR,approx=0,...) {
    if (!missing(OR)) {
        ## Assuming P[1],P[2] is the marginals
        P <- or2prob(OR,P)
        p1 <- attributes(P)$marg[1]
        p2 <- attributes(P)$marg[2]
    } else {
        ## Assuming P contains the joint probabilities
        if (is.vector(P)) {
            if (length(P)==3) P <- c(P,1-sum(P))
            P <- matrix(P,2)
        }
        if (!all.equal(sum(P),1)) stop("Not a probability matrix")
        p1 <- colSums(P)[1]
        p2 <- rowSums(P)[1]
    }
    if (approx>0) { ## Bonnet & Price 2005
        k <- (1-abs(p1-p2)/5 - (.5-min(p1,p2))^2)/2
        if (missing(OR)) OR <- prod(diag(P))/prod(revdiag(P))
        return(cos(pi/(1+OR^k)))
    }    
    q1 <- qnorm(p1)
    q2 <- qnorm(p2)
    lo <- rbind(c(0,0),c(0,-Inf),c(-Inf,0),c(-Inf,-Inf))
    hi <- rbind(c(Inf,Inf),c(Inf,0),c(0,Inf),c(0,0))
    mu <- cbind(q1,q2)%x%cbind(rep(1,4))
    obj <- function(r) {
        Pr <- pmvn(lower=lo,upper=hi,mu=mu,sigma=r,cor=TRUE)
        return(mean(abs(P-Pr)^2))
        ##(P[1,1]-pmvn(lower=c(0,0),mu=c(q1,q2),sigma=r,cor=TRUE))^2
    }
    optimize(obj,interval=c(-1,1))$minimum
}




assoc <- function(x,id,...) {
    N <- sum(x)
    P <- x
    if (N!=1) P <- P/N
    p1 <- colSums(P)[1]
    p2 <- rowSums(P)[1]
    OR <- prod(diag(P))/prod(revdiag(P))
    rho <- tetrachoric(P)
    list(P=P, OR=OR, rho=rho)
}

## library(vcd)
## data(prt)
## mosaic(cancer ~country*zyg,prt)
## (ftable(cancer ~ country, prt))
## grouptable(...)

##' Additive hazards model with (gamma) frailty
##'
##' Aalen frailty model
##' @title Aalen frailty model
##' @param time Time variable
##' @param status Status variable (0,1)
##' @param X Covariate design matrix
##' @param id cluster variable
##' @param theta list of thetas (returns score evaluated here), or
##' starting point for optimization (defaults to magic number 0.1)
##' @param B (optional) Cumulative coefficients (update theta by fixing B)
##' @param ... Additional arguments to lower level functions
##' @return Parameter estimates
##' @author Klaus K. Holst
##' @export
##' @examples
##' library("timereg")
##' dd <- simAalenFrailty(5000)
##' f <- ~1##+x
##' X <- model.matrix(f,dd) ## design matrix for non-parametric terms
##' system.time(out<-timereg::aalen(update(f,Surv(time,status)~.),dd,n.sim=0,robust=0))
##' dix <- which(dd$status==1)
##' t1 <- system.time(bb <- .Call("Bhat",as.integer(dd$status),
##'                               X,0.2,as.integer(dd$id),NULL,NULL,
##'                               PACKAGE="mets"))
##' spec <- 1
##' ##plot(out,spec=spec)
##' ## plot(dd$time[dix],bb$B2[,spec],col="red",type="s",
##' ##      ylim=c(0,max(dd$time)*c(beta0,beta)[spec]))
##' ## abline(a=0,b=c(beta0,beta)[spec])
##' ##'
##' 
##' \dontrun{
##' thetas <- seq(0.1,2,length.out=10)
##' Us <- unlist(aalenfrailty(dd$time,dd$status,X,dd$id,as.list(thetas)))
##' ##plot(thetas,Us,type="l",ylim=c(-.5,1)); abline(h=0,lty=2); abline(v=theta,lty=2)
##' op <- aalenfrailty(dd$time,dd$status,X,dd$id)
##' op
##' }
aalenfrailty <- function(time,status,X,id,theta,B=NULL,...) {  
  dix <- which(status==1)
  cc <- cluster.index(id)
  ncluster <- length(cc$clusters)
  U <- function(theta,indiv=FALSE,Bhat=FALSE) {
    if (is.null(B)) {
        BB <- .Call("Bhat",as.integer(status),X,theta,as.integer(cc$clusters),
                   cc$idclust,as.integer(cc$cluster.size),PACKAGE="mets")$B2
    } else {
        BB <- B*time[dix]
    }
    Hij0 <- apply(X[dix,,drop=FALSE]*BB,1,sum)
    Hij <- cpred(cbind(time[dix],Hij0),time)[,2,drop=FALSE]
##    if (is.na(Hij[1])) browser()
    res <- .Call("Uhat",as.integer(status),Hij,theta,
                cc$idclust,as.integer(cc$cluster.size),PACKAGE="mets")
    if (!indiv) res <- mean(res,na.rm=TRUE)
    if (Bhat) attributes(res)$B <- BB
    return(res)
  }
  if (missing(theta)) theta <- 0.1
  if (is.list(theta)) {
      cc <- lapply(theta,function(x) U(x,Bhat=TRUE,...))
      BB <- Reduce("cbind",lapply(cc,function(x) attributes(x)$B))
      UU <- unlist(lapply(cc,function(x) x[1]))
      res <- list(U=UU, B=BB, time=time, dix=dix, X=X, id=id, status=status)
      return(res)
  }
  op <- nlminb(theta,function(x) U(x)^2)
  uu <- U(op$par,TRUE)
  du <- numDeriv::grad(U,op$par)
  return(list(theta=op$par, sd=(mean(uu^2)/du^2/ncluster)^0.5))
}
                  

##' Simulate observations from Aalen Frailty model with Gamma
##' distributed frailty and constant intensity.
##'
##' @title Simulate from the Aalen Frailty model
##' @param n Number of observations in each cluster
##' @param theta Dependence paramter (variance of frailty)
##' @param K Number of clusters
##' @param beta0 Baseline (intercept)
##' @param beta Effect (log hazard ratio) of covariate
##' @param cens Censoring rate
##' @param cuts time cuts
##' @param ... Additional arguments
##' @author Klaus K. Holst
##' @export
simAalenFrailty <- function(n=5e3,theta=0.3,K=2,beta0=1.5,beta=1,cens=1.5,cuts=0,...) {
    ## beta0 (constant baseline intensity)
    ## beta (covariate effect)
    if (length(beta0)!=length(cuts)) 
        stop("Number of time-intervals (cuts) does not agree with number of rate parameters (beta0)")  
    cuts <- c(cuts,Inf)
    id <- rep(seq(n/K),each=K) ## Cluster indicator
    x <- rbinom(n,1,0.5) ## Binary covariate
    Z <- rep(rgamma(n/K,1/theta,1/theta),each=K) ## Frailty, mean 1, var theta
    Ai <- function() {
        vals <- matrix(0,ncol=length(beta0),nrow=n)
        ival <- numeric(n)
        for (i in seq(length(beta0))) {
            u <- -log(runif(n)) ##rexp(n,1)
            vals[,i] <-  cuts[i] + u/(beta0[i]+beta*x)/Z
            idx <- which(vals[,i]<=cuts[i+1] & ival==0)
            ival[idx] <- vals[idx,i]
        }
        ival
    }
    dat <- data.frame(time=Ai(), x=x, status=1, id=id, Z=Z)
    if (cens==0) cens <- Inf else cens <- -log(runif(n))/cens
    dat$status <- (dat$time<=cens)*1
    dat$time <- apply(cbind(dat$time,cens),1,min)
    dat <- dat[order(dat$time),] ## order after event/censoring time
    return(dat)
}

do.twinlm.strata <- function(x,fun,...) {
  res <- lapply(x$model,function(m) do.call(fun,c(list(m),list(...))))
  names(res) <- names(x$model)
  class(res) <- "do.twinlm.strata"
  newattr <- setdiff(names(attributes(x)),names(attributes(res)))
  attributes(res) <- c(attributes(res),attributes(x)[newattr])
  return(res)
}

##' @export
print.do.twinlm.strata <- function(x,...) {
  for (i in seq_len(length(x))) {    
    message(rep("-",60),sep="")
    message("Strata '",names(x)[i],"'",sep="")
    print(x[[i]])
  }
  if (!is.null(attributes(x)$time)) {
      message(rep("-",60),sep="")
      cat("\n")
      cat("Event of interest before time ", attributes(x)$time, "\n", sep="")      
  }
  return(invisible(x))
}


##' @export
plot.twinlm.strata <- function(x,...)
  suppressMessages(do.twinlm.strata(x,"plot",...))

##' @export
print.twinlm.strata <- function(x,...)
  print.do.twinlm.strata(x$model,...)

##' @export
summary.twinlm.strata <- function(object,...) 
  do.twinlm.strata(object,"summary",...)

##' @export
coef.twinlm.strata <- function(object,...) object$coef

##' @export
logLik.twinlm.strata <- function(object,indiv=FALSE,list=FALSE,...) {
  ll <- lapply(object$model,function(x) logLik(x,indiv=indiv,...))
  if (list) return(ll)
  if (!indiv) {
    res <- structure(sum(unlist(ll)),df=0,nall=0)
    for (i in seq(length(ll))) {
      attributes(res)$nall <- attributes(res)$nall+attributes(ll[[i]])$nall
      attributes(res)$df <- attributes(res)$df+attributes(ll[[i]])$df
    }
    ##  attributes(res)$nobs <- attributes(res)$nall-attributes(res)$df
    attributes(res)$nobs <- attributes(res)$nall
    class(res) <- "logLik"
    return(res)
  }
  return(unlist(ll))
}

##' @export
score.twinlm.strata <- function(x,...) {
  ss <- lapply(x$model,function(m) score(m,indiv=FALSE,...))
  return(unlist(ss))
}

##' @export
summary.biprobit <- function(object,level=0.05,transform,contrast,mean.contrast=NULL,mean.contrast2=NULL,cor.contrast=NULL,marg.idx=1,iid=FALSE,...) {
  alpha <- level/2
  varcomp <- object$coef[length(coef(object)),1:2]
  varcomp <- rbind(object$model$tr(c(varcomp[1],varcomp[1]%x%cbind(1,1) + qnorm(1-alpha)*varcomp[2]%x%cbind(-1,1))))
  colnames(varcomp)[2:3] <- paste(c(alpha*100,100*(1-alpha)),"%",sep="")
  rownames(varcomp) <- ifelse(is.null(object$model$varcompname),"Variance component",object$model$varcompname)
  if (!missing(contrast)) {
      contrast <- rbind(contrast)
      mean.contrast <- contrast[,seq(object$model$blen),drop=FALSE]
      cor.contrast <- contrast[,seq(object$model$zlen)+object$model$blen,drop=FALSE]
      if (!object$model$eqmarg) {          
          mean.contrast2 <- mean.contrast[,seq(ncol(mean.contrast)/2)+ncol(mean.contrast)/2,drop=FALSE]
          mean.contrast <- mean.contrast[,seq(ncol(mean.contrast)/2),drop=FALSE]
      }
  }
  
  h <- function(p) log(p/(1-p)) ## logit
  ih <- function(z) 1/(1+exp(-z)) ## expit
  ##dlogit <- function(p) 1/(p*(1-p))
  if (!missing(transform)) {
      h <- asin; ih <- sin      
      if (is.null(transform)) {
          h <- ih <- identity
      }
      if (is.list(transform)) {
          h <- transform[[1]]; ih <- transform[[2]]
      }     
  }
  convval <- function(val) {
      i1 <- which(val==1)
      i2 <- which(val==-1)
      val <- as.character(val)
      val[seq_len(length(val)-1)+1] <-
          paste("+ ", val[seq_len(length(val)-1)+1],sep="")
      val[i2] <- "- ";
      val[i1] <- "+ "
      val[intersect(1,i1)] <- ""
      return(val)
  }
  parfun <- function(p,ref=FALSE,mean.contrast,mean.contrast2,cor.contrast) {
      nn <- paste("[",gsub("r:","",rownames(object$coef),fixed=TRUE),"]",sep="")
      m <- rep(p[1],2)
      r <- p[object$model$blen+1]
      corref <- mref1 <- mref2 <- NULL
      if (ref) {
          corref <- nn[object$model$blen+1]
          mref1 <- mref2 <- nn[1]
      }
      if (!is.null(mean.contrast)) {
          m[1] <- sum(p[seq_along(mean.contrast)]*mean.contrast)
          if (ref) {
              idx1 <- which(mean.contrast!=0)
              mref <- nn[idx1]              
              mref1 <- mref2 <- paste(convval(mean.contrast[idx1]),mref,sep="")
          }
          if (is.null(mean.contrast2) && !object$model$eqmarg) {
              idx1 <- which(mean.contrast!=0)
              idx2 <- idx1+1
              if (length(object$npar$pred)>0) idx2 <- idx2+object$npar$pred/2
              mean.contrast2 <- rep(0,object$model$blen)
              mean.contrast2[idx2] <- mean.contrast[idx1]              
          }
      }
      if (!is.null(mean.contrast2)) {
          m[2] <- sum(p[seq_len(object$model$blen)]*mean.contrast2[seq_len(object$model$blen)])
          if (ref) {
              idx1 <- which(mean.contrast2!=0)
              mref <- nn[idx1]
              mref2 <- paste(convval(mean.contrast2[idx1]),mref,sep="")
          }          
      } else {
          if (object$model$eqmarg) { m <- rep(m[1],2) }          
      }
      if (!object$model$eqmarg & is.null(mean.contrast) & is.null(mean.contrast2)) {
          idx <- 2
          if (length(object$npar$pred)>0 && object$npar$pred!=0) idx <- object$npar$pred/2+1
          m[2] <- p[idx]
          mref2 <- nn[idx]
      }
      if (!is.null(cor.contrast)) {          
          p.idx <- seq_len(object$model$zlen)+object$model$blen
          if (length(cor.contrast)==length(p)) p.idx <- seq(length(p))          
          r <- sum(p[p.idx]*cor.contrast)
          if (ref) {
              idx1 <- which(cor.contrast!=0)
              corref <- nn[p.idx[idx1]]
              corref <- paste(convval(cor.contrast[idx1]),corref,sep="")
          }
      }
      return(list(m=m,r=r,mref1=mref1,mref2=mref2,corref=corref))
  }
  probs <- function(p,...) {
      pp <- parfun(p,...)
      m <- pp[[1]]
      r <- pp[[2]]
      S <- object$SigmaFun(r,cor=FALSE)
      ##mu.cond <- function(x) m[1]+S[1,2]/S[2,2]*(x-m[2])
      ##var.cond <- S[1,1]-S[1,2]^2/S[2,2]
      p11 <- pmvn(lower=c(0,0),mu=m,sigma=S) 
      p01 <- pmvn(lower=c(-Inf,0),upper=c(0,Inf),mu=m,sigma=S)
      p10 <- pmvn(lower=c(0,-Inf),upper=c(Inf,0),mu=m,sigma=S)
      p00 <- 1-p11-p10-p01
      marg1 <- p11+p10
      marg2 <- p11+p01
      cond1 <- p11/marg2
      lambda <- cond1/marg1
      discond1 <- p10/(1-marg2)
      logOR <- log(cond1)-log(1-cond1)-log(discond1)+log(1-discond1)
      ##rho <- S[1,2]/S[1,1]
      if (object$model$eqmarg) {
          return(c(h(c(p11,cond1,marg1)),lambda,logOR,r))
      }
      return(c(h(c(p11,p10,p01,p00,marg1,marg2)),logOR,r))
  }  
  alpha <- level/2
  CIlab <- paste(c(alpha*100,100*(1-alpha)),"%",sep="")
  mycoef <- coef(object)

  cor.contrast <- rbind(cor.contrast)
  mean.contrast <- rbind(mean.contrast)
  mean.contrast2 <- rbind(mean.contrast2)
  KK <- lapply(list(cor.contrast,mean.contrast,mean.contrast2),nrow)
  if (all(is.null(unlist(KK)))) K <- 1 else  K <- max(unlist(KK))
  IID <- res <- pa <- c()
  for (i in seq(K)) {
      prob <- probs(mycoef,cor.contrast=cor.contrast[i,],mean.contrast=mean.contrast[i,],mean.contrast2=mean.contrast2[i,])
      Dprob <- numDeriv::jacobian(probs,mycoef,cor.contrast=cor.contrast[i,],mean.contrast=mean.contrast[i,],mean.contrast2=mean.contrast2[i,])
      sprob <- diag((Dprob)%*%vcov(object)%*%t(Dprob))^0.5      
      pp <- cbind(prob,prob-qnorm(1-alpha)*sprob,prob+qnorm(1-alpha)*sprob)
      pp[nrow(pp),] <- object$model$tr(pp[nrow(pp),])
      pp[nrow(pp)-1,] <- exp(pp[nrow(pp)-1,])

      if (!object$model$eqmarg) {
          pp[1:6,] <- ih(pp[1:6,])      
          nn <- c("P(Y1=1,Y2=1)","P(Y1=1,Y2=0)","P(Y1=0,Y2=1)","P(Y1=0,Y2=0)","P(Y1=1)","P(Y2=1)","OR","Tetrachoric correlation")
      } else {
          pp[1:3,] <- ih(pp[1:3,])      
          nn <- c("Concordance","Casewise Concordance","Marginal","Rel.Recur.Risk","OR","Tetrachoric correlation")
      }
      
      if (K>1) nn <- paste("c",i,":",nn,sep="")
      if (nrow(pp)-length(nn)>0) nn <- c(nn,rep("",nrow(pp)-length(nn)))
      rownames(pp) <- nn
      colnames(pp) <- c("Estimate",CIlab)
      
      P <- nrow(pp)
      pa <- c(pa, list(parfun(object$coef[,1],ref=TRUE,cor.contrast=cor.contrast[i,],mean.contrast[i,],mean.contrast2[i,])))
      res <- rbind(res,pp)
      
      if (iid) {
          ff <- function(p) {
              res <- probs(p,cor.contrast=cor.contrast[i,],mean.contrast=mean.contrast[i,],mean.contrast2=mean.contrast2[i,])
              nn <- names(res)
              res[length(res)] <- object$model$tr(res[length(res)])
              res[length(res)-1] <- exp(res[length(res)-1])
              idx <- 1:6
              if (object$model$eqmarg) idx <- 1:3
              res[idx] <- ih(res[idx])
              res
          }
          ee <- lava::estimate(object,ff,labels=nn,id=object$id)
          IID <- c(IID,list(ee))
      }     
      
  }      
  contrast <- any(c(!is.null(cor.contrast),!is.null(mean.contrast),!is.null(mean.contrast2)))
  res <- list(all=res,varcomp=varcomp,prob=res,coef=object$coef,score=colSums(object$score),logLik=object$logLik,msg=object$msg,N=object$N,ncontrasts=K,nstat=P,
              par=pa,model=object$model,contrast=contrast, time=attributes(object)$time,
              estimate=IID)
  class(res) <- "summary.biprobit"
  res
}

##' @title Twostage survival model for multivariate survival data
##'
##' @description
##' Fits Clayton-Oakes or bivariate Plackett models for bivariate survival data
##' using marginals that are on Cox form. The dependence can be modelled via
##' \enumerate{
##' \item  Regression design on dependence parameter.
##' \item  Random effects, additive gamma model.
##' }
##'
##' If clusters contain more than two subjects, we use a composite likelihood
##' based on the pairwise bivariate models, for full MLE see twostageMLE.
##'
##' The two-stage model is constructed such that
##' given the gamma distributed random effects it is assumed that the survival functions
##' are indpendent, and that the marginal survival functions are on Cox form (or additive form)
##' \deqn{
##' P(T > t| x) = S(t|x)= exp( -exp(x^T \beta) A_0(t) )
##' }
##'
##' One possibility is to model the variance within clusters via a regression design, and
##' then one can specify a regression structure for the independent gamma distributed
##' random effect for each cluster, such that the variance is given by
##' \deqn{
##'  \theta = h( z_j^T \alpha)
##' }
##' where \eqn{z} is specified by theta.des, and a possible link function var.link=1 will
##' will use the exponential link \eqn{h(x)=exp(x)}, and var.link=0 the identity link \eqn{h(x)=x}.
##' The reported standard errors are based on the estimated information from the
##' likelihood assuming that the marginals are known (unlike the twostageMLE and for the
##' additive gamma model below).
##'
##' Can also fit a structured additive gamma random effects model, such
##' as the ACE, ADE model for survival data.  In this case the
##' random.design specificies the random effects for each subject within a cluster. This is
##' a matrix of 1's and 0's with dimension n x d.  With d random effects.
##' For a cluster with two subjects, we let the random.design rows be
##'  \eqn{v_1} and \eqn{v_2}.
##' Such that the random effects for subject
##' 1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
##' has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}.
##' By construction subjects 1's random effect are Gamma distributed with
##' mean \eqn{\lambda_j/v_1^T \lambda}
##' and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
##' \eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
##' It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed within clusters
##' as it would be for the ACE model below.
##'
##' Based on these parameters the relative contribution (the heritability, h) is
##' equivalent to  the expected values of the random effects: \eqn{\lambda_j/v_1^T \lambda}
##'
##' The DEFAULT parametrization (var.par=1) uses the variances of the random effecs
##' \deqn{
##' \theta_j  = \lambda_j/(v_1^T \lambda)^2
##' }
##' For alternative parametrizations one can specify how the parameters relate to \eqn{\lambda_j}
##' with the argument var.par=0.
##'
##' For both types of models the basic model assumptions are that
##' given the random effects of the clusters the survival distributions within a cluster
##' are independent and ' on the form
##' \deqn{
##' P(T > t| x,z) = exp( -Z \cdot Laplace^{-1}(lamtot,lamtot,S(t|x)) )
##' }
##' with the inverse laplace of the gamma distribution with mean 1 and variance 1/lamtot.
##'
##' The parameters \eqn{(\lambda_1,...,\lambda_d)} are related to the parameters of the model
##' by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
##' \eqn{\lambda} parameters
##' with the (k) underlying \eqn{\theta} parameters
##' \deqn{
##' \lambda = theta.des  \theta
##' }
##' here using theta.des to specify these low-dimension association. Default is a diagonal matrix.
##' This can be used to make structural assumptions about the variances of the random-effects
##' as is needed for the ACE model for example.
##'
##' The case.control option that can be used with the pair specification of the pairwise parts
##' of the estimating equations. Here it is assumed that the second subject of each pair is the proband.
##'
##' @references
##'
##' Twostage estimation of additive gamma frailty models for survival data.
##' Scheike (2019), work in progress
##'
##' Shih and Louis (1995) Inference on the association parameter in copula models for bivariate
##' survival data, Biometrics, (1995).
##'
##' Glidden (2000), A Two-Stage estimator of the dependence
##' parameter for the Clayton Oakes model, LIDA, (2000).
##'
##' Measuring early or late dependence for bivariate twin data
##' Scheike, Holst, Hjelmborg (2015), LIDA
##'
##' Estimating heritability for cause specific mortality based on twins studies
##' Scheike, Holst, Hjelmborg (2014), LIDA
##'
##' Additive Gamma frailty models for competing risks data, Biometrics (2015)
##' Eriksson and Scheike (2015),
##'
##' @examples
##' data(diabetes)
##'
##' # Marginal Cox model  with treat as covariate
##' margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
##' ### Clayton-Oakes, MLE
##' fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
##' summary(fitco1)
##'
##' ### Plackett model
##' mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
##' fitp <- survival.twostage(mph,data=diabetes,theta=3.0,Nit=40,
##'                clusters=diabetes$id,var.link=1,model="plackett")
##' summary(fitp)
##'
##' ### Clayton-Oakes
##' fitco2 <- survival.twostage(mph,data=diabetes,theta=0.0,detail=0,
##'                  clusters=diabetes$id,var.link=1,model="clayton.oakes")
##' summary(fitco2)
##' fitco3 <- survival.twostage(margph,data=diabetes,theta=1.0,detail=0,
##'                  clusters=diabetes$id,var.link=0,model="clayton.oakes")
##' summary(fitco3)
##'
##' ### without covariates but with stratafied
##' marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
##' fitpa <- survival.twostage(marg,data=diabetes,theta=1.0,
##'                 clusters=diabetes$id,model="clayton.oakes")
##' summary(fitpa)
##'
##' fitcoa <- survival.twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
##'                  model="clayton.oakes")
##' summary(fitcoa)
##'
##' ### Piecewise constant cross hazards ratio modelling
##' ########################################################
##'
##' d <- subset(simClaytonOakes(2000,2,0.5,0,stoptime=2,left=0),!truncated)
##' udp <- piecewise.twostage(c(0,0.5,2),data=d,method="optimize",
##'                           id="cluster",timevar="time",
##'                           status="status",model="clayton.oakes",silent=0)
##' summary(udp)
##'
##' \donttest{ ## Reduce Ex.Timings
##' ### Same model using the strata option, a bit slower
##' ########################################################
##' ## makes the survival pieces for different areas in the plane
##' ##ud1=surv.boxarea(c(0,0),c(0.5,0.5),data=d,id="cluster",timevar="time",status="status")
##' ##ud2=surv.boxarea(c(0,0.5),c(0.5,2),data=d,id="cluster",timevar="time",status="status")
##' ##ud3=surv.boxarea(c(0.5,0),c(2,0.5),data=d,id="cluster",timevar="time",status="status")
##' ##ud4=surv.boxarea(c(0.5,0.5),c(2,2),data=d,id="cluster",timevar="time",status="status")
##'
##' ## everything done in one call
##' ud <- piecewise.data(c(0,0.5,2),data=d,timevar="time",status="status",id="cluster")
##' ud$strata <- factor(ud$strata);
##' ud$intstrata <- factor(ud$intstrata)
##'
##' ## makes strata specific id variable to identify pairs within strata
##' ## se's computed based on the id variable across strata "cluster"
##' ud$idstrata <- ud$id+(as.numeric(ud$strata)-1)*2000
##'
##' marg2 <- timereg::aalen(Surv(boxtime,status)~-1+factor(num):factor(intstrata),
##'                data=ud,n.sim=0,robust=0)
##' tdes <- model.matrix(~-1+factor(strata),data=ud)
##' fitp2 <- survival.twostage(marg2,data=ud,se.clusters=ud$cluster,clusters=ud$idstrata,
##'                 model="clayton.oakes",theta.des=tdes,step=0.5)
##' summary(fitp2)
##'
##' ### now fitting the model with symmetry, i.e. strata 2 and 3 same effect
##' ud$stratas <- ud$strata;
##' ud$stratas[ud$strata=="0.5-2,0-0.5"] <- "0-0.5,0.5-2"
##' tdes2 <- model.matrix(~-1+factor(stratas),data=ud)
##' fitp3 <- survival.twostage(marg2,data=ud,clusters=ud$idstrata,se.cluster=ud$cluster,
##'                 model="clayton.oakes",theta.des=tdes2,step=0.5)
##' summary(fitp3)
##'
##' ### same model using strata option, a bit slower
##' fitp4 <- survival.twostage(marg2,data=ud,clusters=ud$cluster,se.cluster=ud$cluster,
##'                 model="clayton.oakes",theta.des=tdes2,step=0.5,strata=ud$strata)
##' summary(fitp4)
##' }
##'
##' \donttest{ ## Reduce Ex.Timings
##' ### structured random effects model additive gamma ACE
##' ### simulate structured two-stage additive gamma ACE model
##' data <- simClaytonOakes.twin.ace(4000,2,1,0,3)
##' out <- twin.polygen.design(data,id="cluster")
##' pardes <- out$pardes
##' pardes
##' des.rv <- out$des.rv
##' head(des.rv)
##' aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data,robust=0)
##' ts <- survival.twostage(aa,data=data,clusters=data$cluster,detail=0,
##' 	       theta=c(2,1),var.link=0,step=0.5,
##' 	       random.design=des.rv,theta.des=pardes)
##' summary(ts)
##' }
##'
##' @keywords survival
##' @author Thomas Scheike
##' @param margsurv Marginal model
##' @param data data frame
##' @param method Scoring method "nr", for lava NR optimizer
##' @param detail Detail
##' @param clusters Cluster variable
##' @param silent Debug information
##' @param weights Weights
##' @param theta Starting values for variance components
##' @param theta.des design for dependence parameters, when pairs are given the indeces of the
##' theta-design for this pair, is given in pairs as column 5
##' @param var.link Link function for variance:  exp-link.
##' @param baseline.iid to adjust for baseline estimation, using phreg function on same data.
##' @param model model
##' @param marginal.trunc marginal left truncation probabilities
##' @param marginal.survival optional vector of marginal survival probabilities
##' @param strata strata for fitting, see example
##' @param se.clusters for clusters for se calculation with iid
##' @param numDeriv to get numDeriv version of second derivative, otherwise uses sum of squared scores for each pair
##' @param random.design random effect design for additive gamma model, when pairs are given the
##' indeces of the pairs random.design rows are given as columns 3:4
##' @param pairs matrix with rows of indeces (two-columns) for the pairs considered in the pairwise
##' composite score, useful for case-control sampling when marginal is known.
##' @param dim.theta dimension of the theta parameter for pairs situation.
##' @param numDeriv.method uses simple to speed up things and second derivative not so important.
##' @param additive.gamma.sum for two.stage=0, this is specification of the lamtot in the models via
##' a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number
##' of theta parameters), when null then sums all parameters.
##' @param var.par is 1 for the default parametrization with the variances of the random effects,
##' var.par=0 specifies that the \eqn{\lambda_j}'s are used as parameters.
##' @param no.opt for not optimizng
##' @param ... Additional arguments to maximizer NR of lava.
##' and ascertained sampling
##' @aliases survival.twostage twostage.aalen twostage.cox.aalen twostage.coxph twostage.phreg randomDes readmargsurv 
##' @export survival.twostage
survival.twostage <- function(margsurv,data=parent.frame(),
    method="nr",detail=0,clusters=NULL,
    silent=1,weights=NULL,theta=NULL,theta.des=NULL,
    var.link=1,baseline.iid=1,model="clayton.oakes",
    marginal.trunc=NULL,marginal.survival=NULL,strata=NULL,
    se.clusters=NULL,numDeriv=1,random.design=NULL,pairs=NULL,dim.theta=NULL,
    numDeriv.method="simple",additive.gamma.sum=NULL,var.par=1,no.opt=FALSE,...)
{#{{{
  requireNamespace("timereg")
# seting up design and variables
iid <- 1
two.stage <- 1; rate.sim <- 1; sym=1; var.func <- NULL
if (model=="clayton.oakes" || model=="gamma") dep.model <- 1
else if (model=="plackett" || model=="or") dep.model <- 2
else stop("Model must by either clayton.oakes or plackett \n");
start.time <- NULL; ptrunc <- NULL; psurvmarg <- NULL; status <- NULL; score.iid <- NULL
fix.baseline <- 0; convergence.bp <- 1;  ### to control if baseline profiler converges

  if ((!is.null(margsurv)) | (!is.null(marginal.survival))) fix.baseline <- 1
  antpers <- nrow(data); RR <-  rep(1,antpers);


  if (!is.null(margsurv)) {
     rrr <-  readmargsurv(margsurv,data,clusters)
     psurvmarg <- rrr$psurvmarg; ptrunc <- rrr$ptrunc; start.time <- rrr$entry;
     time2 <- rrr$exit; status <- rrr$status; clusters <- rrr$clusters
  }

  if (is.null(psurvmarg)) psurvmarg <- rep(1,antpers);
  if (!is.null(marginal.survival)) psurvmarg <- marginal.survival
  if (!is.null(marginal.trunc)) ptrunc <- marginal.trunc
  if (is.null(ptrunc)) ptrunc <- rep(1,length(psurvmarg))
  if (is.null(weights)==TRUE) weights <- rep(1,antpers);
  if (is.null(strata)==TRUE) strata<- rep(1,antpers);
  if (length(strata)!=antpers) stop("Strata must have length equal to number of data points \n");

  # cluster set up
  cluster.call <- clusters
  out.clust <- cluster.index(clusters);
  clusters <- out.clust$clusters
  maxclust <- out.clust$maxclust
  antclust <- out.clust$cluster.size
  clusterindex <- out.clust$idclust
  clustsize <- out.clust$cluster.size
  call.secluster <- se.clusters

  if (is.null(se.clusters)) { se.clusters <- clusters; antiid <- nrow(clusterindex);} else  {
      iids <-  unique(se.clusters);
      antiid <- length(iids);
      if (is.numeric(se.clusters)) se.clusters <-  fast.approx(iids,se.clusters)-1
       else se.clusters <- as.integer(factor(se.clusters, labels = seq(antiid)))-1
  }
  if (length(se.clusters)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

  #

  ### setting design for random variables, in particular with pairs are given
  ddd <- randomDes(dep.model,random.design,theta.des,theta,antpers,additive.gamma.sum,pairs,var.link,clusterindex,dim.theta)
  random.design=ddd$random.design;clusterindex=ddd$clusterindex;
  antpairs=ddd$antpairs;
  theta=ddd$theta;ptheta=ddd$ptheta;theta.des=ddd$theta.des
  pair.structure=ddd$pair.structure; dep.model=ddd$dep.model
  dim.rv <- ddd$dim.rv; additive.gamma.sum=ddd$additive.gamma.sum

  theta.score<-rep(0,ptheta);
  Stheta<-var.theta<-matrix(0,ptheta,ptheta);
  #
  ascertained <- 0

  obj <- function(par)
  { #

     if (pair.structure==0 | dep.model!=3) Xtheta <- as.matrix(theta.des) %*% matrix(c(par),nrow=ptheta,ncol=1);
     if (pair.structure==1 & dep.model==3) Xtheta <- matrix(0,antpers,1); ## not needed

      if (var.link==1 & dep.model==3) epar <- c(exp(par)) else epar <- c(par)
      partheta <- epar

      if (var.par==1 & dep.model==3) {
	 ## from variances to
	 if (is.null(var.func)) {
	    sp <- sum(epar)
	    partheta <- epar/sp^2
         } else partheta <- epar ## par.func(epar)
      }

      if (pair.structure==0) {
	      outl<-.Call("twostageloglikeRV", # only two stage model for this option
	      icause=status,ipmargsurv=psurvmarg,
	      itheta=c(partheta), ithetades=theta.des,
	      icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex,
	      ivarlink=var.link,iid=iid,iweights=weights,isilent=silent,idepmodel=dep.model,
	      itrunkp=ptrunc,istrata=as.numeric(strata),iseclusters=se.clusters,iantiid=antiid,
	      irvdes=random.design,iags=additive.gamma.sum,iascertained=ascertained,
              PACKAGE="mets") #
      }
      else { # pair-structure
          outl<-.Call("twostageloglikeRVpairs", #
          icause=status,ipmargsurv=psurvmarg, itheta=c(partheta), ithetades=theta.des,
          icluster=clusters,iclustsize=clustsize,iclusterindex=clusterindex, ivarlink=var.link,
	  iiid=iid,iweights=weights,isilent=silent, idepmodel=dep.model,
	  itrunkp=ptrunc,istrata=as.numeric(strata), iseclusters=se.clusters,iantiid=antiid,
	  irvdes=random.design, iags=additive.gamma.sum, iascertained=ascertained,PACKAGE="mets")
	  #
      } #

    if (detail==3) print(c(partheta,outl$loglike))

    ## variance parametrization, and inverse.link
    if (dep.model==3) {
    if (var.par==1) {
         ## from variances to and with sum for all random effects
         if (is.null(var.func)) {
	 if (var.link==0)  {
             mm <- matrix(-epar*2*sp,length(epar),length(epar))
	     diag(mm) <- sp^2-epar*2*sp
	 } else {
            mm <- -c(epar) %o% c(epar)*2*sp
            diag(mm) <- epar*sp^2-epar^2*2*sp
	 }
	    mm <- mm/sp^4
	 } else mm  <- numDeriv::hessian(var.func,par)
      } else {
	   if (var.link==0) mm <- diag(length(epar)) else
			  mm <- diag(length(c(epar)))*c(epar)
      }
      }


    if (dep.model==3) {
       outl$score <-  t(mm) %*% outl$score
       outl$Dscore <- t(mm) %*% outl$Dscore %*% mm
       if (iid==1) { outl$score.iid <- t(t(mm) %*% t(outl$score.iid))
               if (inherits(margsurv,"phreg")) {
	          outl$D1thetal  <- t(t(mm) %*% t(outl$D1thetal))
	          outl$D2thetal  <- t(t(mm) %*% t(outl$D2thetal))
	       }
       }
    }


    outl$gradient <- outl$score 
    outl$hessian <- outl$Dscore

    if (oout==0) ret <- with(outl,structure(outl$loglike,gradient=-gradient,hessian=-hessian))
    else if (oout==1) ret <- outl$gradient 
    else if (oout==2) ret <- outl

    return(ret)
  } #

  score1 <- NULL
  theta.iid <- NULL
  p <- theta

  oout <- 0
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(p,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  ## names(cc) <- colnames(X)
      oout <- 2
      val <- c(list(coef=cc),obj(opt$estimate))
 } else {oout <- 2; val <- c(list(coef=p),obj(p))}

     if (numDeriv>=1) {
	 p <- val$coef
         oout <- 1
         if (detail==1 ) cat("starting numDeriv for second derivative \n");
         val$hessian <- numDeriv::jacobian(obj,p,method=numDeriv.method)
         if (detail==1 ) cat("finished numDeriv for second derivative \n");
        }


    ### update p, but note that score and derivative in fact related to previous p
    if (!is.nan(sum(p))) {
      if (detail==1 && iid==1) cat("iid decomposition\n");
         if (iid==1) {  #
            score.iid <- val$score.iid
	    val$theta.iid <- score.iid
	    theta.iid <- score.iid
	    ucc <-  unique(cluster.call)
	    if (length(ucc)== nrow(theta.iid))
		    rownames(theta.iid) <- unique(cluster.call)
                ### lets iid for theta be just score to start, correction for marginal for phreg call

                if (inherits(margsurv,"phreg") & baseline.iid==1) {  # adjust for baseline when phreg is used

                  if ((margsurv$no.opt) | is.null(margsurv$coef)) fixbeta<- 1 else fixbeta <- 0
		  xx <- margsurv$cox.prep
                  id  <-  xx$id+1
		  cumhazt <- c(rrr$cum)
		  rr <- c(rrr$RR)

		  ## these refers to id given in cluster.call
		  xx <- margsurv$cox.prep
		  D1thetal<- val$D1thetal
		  D2thetal<- val$D2thetal
		  Dlamthetal <- -(D1thetal+D2thetal)
		  ## ordered as original data
                  Fbeta <- t(Dlamthetal) %*% (margsurv$X *c(cumhazt*psurvmarg))

                  ### timeordered
		  ## t- not needed because using S(T-) for survival already
                  Gbasedt <- Dlamthetal[xx$ord+1,,drop=FALSE]*psurvmarg[xx$ord+1]*rr[xx$ord+1]
                  Gbase <- apply(Gbasedt,2,revcumsumstrata,xx$strata,xx$nstrata)

                  if (fixbeta==0) {  ### iid after beta of marginal model
		     Z <- xx$X
		     U <- E <- matrix(0,nrow(xx$X),margsurv$p)
		     E[xx$jumps+1,] <- margsurv$E
		     U[xx$jumps+1,] <- margsurv$U
		     invhess <- -solve(margsurv$hessian)
		     S0i <- rep(0,length(xx$strata))
		     S0i[xx$jumps+1] <- 1/margsurv$S0
		     cumhaz <- c(cumsumstrata(S0i,xx$strata,xx$nstrata))
		     EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	             rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset))
		     ### Martingale for all subjects
		     MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
		     UUbeta <- apply(MGt,2,sumstrata,id-1,max(id))
		     UUbeta <- UUbeta %*% invhess
		     GbaseEdLam0 <- t(Gbase) %*% (E*S0i)
		     Fbeta   <-  Fbeta -  GbaseEdLam0
		     Fbetaiid <- UUbeta %*% t(Fbeta)
	          }

	  ###  \int_0^\tau (GBase(s) / S_0(s)) dN_i(s) - dLamba_i(s)
	  xx <- margsurv$cox.prep
	  S0i  <- S0i2 <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/margsurv$S0
	  S0i2[xx$jumps+1] <- 1/margsurv$S0^2

	  GbasedN <- Gbase*S0i
	  GbasedLam0 <- apply(Gbase*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)

	  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset)) else rr <- c(xx$sign*exp( xx$offset))
	  MGt <- (GbasedN[,drop=FALSE]-GbasedLam0*rr)*c(xx$weights)
	  MGti <- apply(MGt,2,sumstrata,id-1,max(id))

	  if (fixbeta==0) MGti <-  MGti + Fbetaiid
	  theta.iid <-  -theta.iid + MGti
	  val$theta.iid <- theta.iid
	  } #
   } #
   }

  hess <- val$hessian
  if (!is.na(sum(hess))) hessi <- lava::Inverse(val$hessian) else hessi <- diag(nrow(val$hessian))

# handling output
  loglikeiid <- NULL; robvar.theta <- NULL; likepairs <- NULL; marginal.surv <- psurvmarg; 
  marginal.trunc <- ptrunc; 

  if (iid==1) {
  if (dep.model==3 & pair.structure==1) likepairs <- val$likepairs
  if (dep.model==3 & two.stage==0) {
	  hessi <- -1*hessi
	  all.likepairs <- val$all.likepairs
	  colnames(all.likepairs) <- c("surv","dt","ds","dtds","cause1","cause2")
  }
     theta.iid <- val$theta.iid %*% hessi
### rownames not set to make more robust
### if (is.null(call.secluster)) rownames(theta.iid) <- unique(cluster.call) else rownames(theta.iid) <- unique(se.clusters)
     robvar.theta  <- crossprod(theta.iid)
     loglikeiid <- val$loglikeiid
    } else { all.likepairs <- NULL}
  var.theta <- robvar.theta
  if (is.null(robvar.theta)) var.theta <- hessi

  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(theta),sep="")
  theta <- matrix(theta,length(c(theta)),1)
  if (length(thetanames)==nrow(theta)) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }


  ud <- list(theta=matrix(val$coef,ncol=1),coef=val$coef,score=val$gradient,hess=hess,hessi=hessi,var.theta=var.theta,
     model=model,robvar.theta=robvar.theta,
     loglike=val$loglike,loglikeiid=loglikeiid,likepairs=likepairs,
     theta.iid=theta.iid,thetanames=thetanames,
     score1=score1,Dscore=val$Dscore,
     marginal.surv=marginal.surv,marginal.trunc=marginal.trunc,
     se=diag(robvar.theta)^.5,score.iid=-score.iid,theta.des=theta.des,random.design=random.design)
  class(ud) <- "mets.twostage"
  attr(ud,"response") <- "survival"
  attr(ud,"Formula") <- formula
  attr(ud,"clusters") <- clusters
  attr(ud,"cluster.call") <- cluster.call
  attr(ud,"secluster") <- c(se.clusters)
  attr(ud,"sym")<-sym;
  attr(ud,"var.link")<-var.link;
  attr(ud,"var.par")<-var.par;
  attr(ud,"var.func")<-var.func;
  attr(ud,"ptheta")<-ptheta
  attr(ud,"antpers")<-antpers;
  attr(ud,"antclust")<-antclust;
  attr(ud,"Type") <- model
  attr(ud,"twostage") <- two.stage
  attr(ud,"additive-gamma") <- (dep.model==3)*1
  if (!is.null(marginal.trunc)) attr(ud,"trunclikeiid")<- val$trunclikeiid
  if (dep.model==3 & two.stage==0) attr(ud,"all.likepairs")<- all.likepairs
  if (dep.model==3 ) attr(ud,"additive.gamma.sum") <- additive.gamma.sum

if (dep.model==3 & pair.structure==0) {
       attr(ud, "pardes") <- theta.des
       attr(ud, "theta.des") <- theta.des
       attr(ud, "rv1") <-    random.design[1,]
    }
    if (dep.model==3 & pair.structure==1) {
       nrv <- clusterindex[1,6]
       attr(ud, "theta.des") <- matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "pardes") <-    matrix(theta.des[1,1:(nrv*ptheta)],nrv,ptheta)
       attr(ud, "rv1") <-    random.design[1,]
       attr(ud, "nrv") <- clusterindex[1,6]
    }


  return(ud);
  #

} # }}}

##' @export
randomDes <- function(dep.model,random.design,theta.des,theta,antpers,ags,pairs,var.link,clusterindex,dim.theta)
{
   additive.gamma.sum <- ags

  if (!is.null(random.design)) { ### different parameters for Additive random effects 
     dep.model <- 3
     dim.rv <- ncol(random.design);
     if (is.null(theta.des)) theta.des<-diag(dim.rv);
  } else {
      random.design <- matrix(0,1,1);  dim.rv <- 1;
      additive.gamma.sum <- matrix(1,1,1);
  }

  if (is.null(theta.des)) ptheta<-1;
  if (is.null(theta.des)) theta.des<-matrix(1,antpers,ptheta); ###  else theta.des<-as.matrix(theta.des);
  ptheta<-ncol(theta.des)

  if (is.matrix(pairs)) {
       if ( ncol(pairs)==6)  ptheta<- dim.theta else ptheta<-ncol(theta.des)
  }

  theta.des <- as.matrix(theta.des)
  if (is.null(theta)==TRUE) {
         if (var.link==1) theta<- rep(-0.7,ptheta);
         if (var.link==0) theta<- rep(exp(-0.7),ptheta);
  }

  if (length(theta)!=ptheta) {
         theta<-rep(theta[1],ptheta);
  }
  theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);
  antpairs <- 1; ### to define

  if (is.null(additive.gamma.sum)) additive.gamma.sum <- matrix(1,dim.rv,ptheta)
  if (!is.null(pairs)) { pair.structure <- 1;} else  pair.structure <- 0;

  if (pair.structure==1 & dep.model==3) { #
  antpairs <- nrow(pairs);

  if ( (ncol(pairs)==2))
  {
      pairs <- cbind(pairs,pairs,1,ncol(random.design))
      theta.des <- matrix(c(theta.des),1,length(c(theta.des)))
  }
   clusterindex <- cbind(pairs[,1:5]-1,ncol(random.design))
  } #

  if (pair.structure==1 & dep.model!=3) { #
       antpairs <- nrow(pairs);
       if ((ncol(pairs)==2)) { ## when only pairs are given we refer to rows of theta.des from 3 column: same as index of pair-1
       pairs <- cbind(pairs,pairs[,1])
       }
       if (ncol(pairs)!=3) stop("with pairstructure and theta.des, 3rd column of pairs must give relevant design from theta.des\n")
       ## index in c are Rindex -1
       clusterindex <- pairs-1;
  }

###  print(head(pairs)); print(head(theta.des))
###  print(dim(pairs)); print(dim(theta.des))

  return(list(random.design=random.design,clusterindex=clusterindex,
	 antpairs=antpairs,pair.structure=pair.structure,
	 dep.model=dep.model,dim.rv=dim.rv, additive.gamma.sum=additive.gamma.sum,
         theta=theta,ptheta=ptheta,theta.des=theta.des))

}



##' @export
readmargsurv <- function(margsurv,data,clusters)
{
start.time <- 0

if (inherits(margsurv,c("aalen","cox.aalen"))) {
	 formula<-attr(margsurv,"Formula");
	 beta.fixed <- attr(margsurv,"beta.fixed")
	 if (is.null(beta.fixed)) beta.fixed <- 1;
	 ldata <- timereg::aalen.des(formula,data=data,model="cox.aalen");
	 id <- attr(margsurv,"id");
	 mclusters <- attr(margsurv,"cluster.call")
	 X<-ldata$X;
	 time<-ldata$time2;
	 Z<-ldata$Z;
	 status<-ldata$status;
	 time2 <- attr(margsurv,"stop");
	 start.time <- attr(margsurv,"start")
	 antpers<-nrow(X);
         if (is.null(Z)==TRUE) {npar<-TRUE; semi<-0;}  else {
		     Z<-as.matrix(Z); npar<-FALSE; semi<-1;}
         if (npar==TRUE) {Z<-matrix(0,antpers,1); pz<-1; fixed<-0;} else {fixed<-1;pz<-ncol(Z);}
	 px<-ncol(X);

         if (is.null(clusters) && is.null(mclusters)) stop("No cluster variabel specified in marginal or twostage call\n");
         if (is.null(clusters)) clusters <- mclusters
	 if (nrow(X)!=length(clusters)) stop("Length of Marginal survival data not consistent with cluster length\n"); 
	 } else if (inherits(margsurv,"phreg")) { 
            ### setting up newdata with factors and strata
                antpers <- nrow(data)
		 rr <- readPhreg(margsurv,data,nr=FALSE)
		 time2 <- rr$exit
		 if (!is.null(rr$entry)) start.time <- rr$entry
		 else start.time <- rep(0,antpers);
		 status <- rr$status
	         if (is.null(clusters)) clusters <- rr$clusters
	
} else { ### coxph 
	  antpers <- margsurv$n
	  id <- 0:(antpers-1)
	  mt <- model.frame(margsurv)
	  Y <- model.extract(mt, "response")
	  if (!inherits(Y, "Surv")) stop("Response must be a survival object")
	  if (attr(Y, "type") == "right") {
	      time2 <- Y[, "time"];
	      status <- Y[, "status"]
		start.time <- rep(0,antpers);
		} else {
		 start.time <- Y[, 1];
		 time2 <- Y[, 2];
		 status <- Y[, 3];
		}
	   Z <- matrix(1,antpers,length(coef(margsurv)));

	   if (is.null(clusters)) stop("must give clusters for coxph\n");
	   cluster.call <- clusters;
	   X <- matrix(1,antpers,1); ### Z <- matrix(0,antpers,1); ### no use for these
	   px <- 1; pz <- ncol(Z);
	   start <- rep(0,antpers);
	   beta.fixed <- 0
	   semi <- 1
}

  if (!is.null(start.time)) {
      if (any(abs(start.time)>0)) lefttrunk <- 1  else lefttrunk <- 0;
  } else lefttrunk <- 0

if (!is.null(margsurv))  {
  if (inherits(margsurv,c("aalen","cox.aalen")))  { 
         resi <- timereg::residualsTimereg(margsurv,data=data)
         RR  <- resi$RR
	 cum <- resi$cumhaz/RR
	 psurvmarg <- exp(-resi$cumhaz);
         ptrunc <- rep(1,length(psurvmarg));
	 if (lefttrunk==1) ptrunc <- exp(-resi$cumhazleft); 
  } else if (inherits(margsurv,"coxph")) {  
    ### some problems here when data is different from data used in margsurv
       residuals <- residuals(margsurv)
       cum <- cumhaz <- status-residuals
       psurvmarg <- exp(-cumhaz);
       cumhazleft <- rep(0,antpers)
       ptrunc <- rep(1,length(psurvmarg));
       RR<- exp(margsurv$linear.predictors+sum(margsurv$means*coef(margsurv)))
       cum <- cum/RR
        if ((lefttrunk==1)) {
         stop("Use cox.aalen function for truncation case \n");
         baseout <- survival::basehaz(margsurv,centered=FALSE);
         cumt <- cbind(baseout$time,baseout$hazard)
	 cumt <- cpred(cumt,start.time)[,2]
	 ptrunc <- exp(-cumt * RR)
	}
  } else if (inherits(margsurv,"phreg")) {

	ppsurvmarg <- predict(margsurv,data,tminus=TRUE,times=time2,individual.time=TRUE,se=FALSE)
        psurvmarg <- ppsurvmarg$surv
        cum <- ppsurvmarg$cumhaz
	RR <- ppsurvmarg$RR
        ptrunc <- rep(1,length(psurvmarg));
        if ((lefttrunk==1)) {
	  ptrunc <- predict(margsurv,data,tminus=TRUE,times=start.time,individual.time=TRUE,se=FALSE)$surv
	}
  }
}

if (is.null(clusters) &  (!inherits(margsurv,"phreg"))) stop("must give clusters")

return(list(psurvmarg=psurvmarg,ptrunc=ptrunc,entry=start.time,exit=time2,
	    status=status,clusters=clusters,cum=cum,RR=RR))

} #

##' @title Twostage survival model fitted by pseudo MLE
##'
##' @description
##' Fits Clayton-Oakes clustered  survival data
##' using marginals that are on Cox form in the likelihood for the dependence parameter
##' as in Glidden (2000). The dependence can be modelled via  a
##' \enumerate{
##' \item  Regression design on dependence parameter.
##' }
##'
##' We allow a regression structure for the indenpendent gamma distributed
##' random effects  and their variances that may depend on cluster covariates. So
##' \deqn{
##'  \theta = h( z_j^T \alpha)
##' }
##' where \eqn{z} is specified by theta.des . The link function can be the exp when var.link=1
##' @references
##'
##' Measuring early or late dependence for bivariate twin data
##' Scheike, Holst, Hjelmborg (2015), LIDA
##'
##' Twostage modelling of additive gamma frailty models for survival data.
##' Scheike and Holst, working paper
##'
##' Shih and Louis (1995) Inference on the association parameter in copula models for bivariate
##' survival data, Biometrics, (1995).
##'
##' Glidden (2000), A Two-Stage estimator of the dependence
##' parameter for the Clayton Oakes model, LIDA, (2000).
##'
##' @examples
##' data(diabetes)
##' dd <- phreg(Surv(time,status==1)~treat+cluster(id),diabetes)
##' oo <- twostageMLE(dd,data=diabetes)
##' summary(oo)
##'
##' theta.des <- model.matrix(~-1+factor(adult),diabetes)
##'
##' oo <-twostageMLE(dd,data=diabetes,theta.des=theta.des)
##' summary(oo)
##' @keywords survival
##' @author Thomas Scheike
##' @param margsurv Marginal model from phreg
##' @param data data frame
##' @param theta Starting values for variance components
##' @param theta.des design for dependence parameters, when pairs are given this is could be a
##' (pairs) x (numer of parameters)  x (max number random effects) matrix
##' @param var.link Link function for variance  if 1 then uses exp link
##' @param method type of opitmizer, default is Newton-Raphson "NR"
##' @param no.opt to not optimize, for example to get score and iid for specific theta
##' @param weights cluster specific weights, but given with length equivalent to data-set, weights for score equations
##' @param se.cluster specifies how the influence functions are summed before squared when computing the variance. Note that the id from the marginal model is used to construct MLE, and then these scores can be summed with the se.cluster argument. 
##' @param ... arguments to be passed to  optimizer
##' @export
twostageMLE <-function(margsurv,data=parent.frame(),
theta=NULL,theta.des=NULL,var.link=0,method="NR",no.opt=FALSE,weights=NULL,se.cluster=NULL,...)
{
 if (!inherits(margsurv,"phreg"))  stop("Must use phreg for this \n");

 clusters <- margsurv$cox.prep$id
 n <- nrow(margsurv$cox.prep$X)


 if (is.null(theta.des)==TRUE) ptheta<-1;
 if (is.null(theta.des)==TRUE) theta.des<-matrix(1,n,ptheta) else theta.des<-as.matrix(theta.des);
  ptheta<-ncol(theta.des);
  if (nrow(theta.des)!=n) stop("Theta design does not have correct dim");

  if (is.null(theta)==TRUE) {
      if (var.link==1) theta<- rep(0.0,ptheta);
      if (var.link==0) theta<- rep(1.0,ptheta);
  }
  if (length(theta)!=ptheta) theta<-rep(theta[1],ptheta);
  theta.score<-rep(0,ptheta);Stheta<-var.theta<-matrix(0,ptheta,ptheta);

  max.clust <- length(unique(clusters))
  theta.iid <- matrix(0,max.clust,ptheta)

  xx <- margsurv$cox.prep
  nn <- length(xx$strata)

  if (is.null(weights)) weights <-  rep(1,nn)
  if (length(weights)!=nn) stop("Weights do not have right length")

  statusxx <- rep(0,length(xx$strata))
  statusxx[xx$jumps+1] <- 1
  xx$status <- statusxx
  mid <- max(xx$id)+1

  ## Ni.(t-), Ni.(t)
  Nsum <- cumsumstratasum(statusxx,xx$id,mid,type="all")
  ## Ni.(tau)
  Ni.tau <- sumstrata(statusxx,xx$id,mid)
  ## cumahz(T_i-)
  S0i2 <- S0i <- rep(0, length(xx$strata))
  S0i[xx$jumps + 1] <- 1/margsurv$S0
  cumhazD <- cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum
  if (!is.null(margsurv$coef)) RR <- exp(xx$X %*% margsurv$coef) else RR  <-  rep(1,nn)
  H <- c(cumhazD * RR)

  cc <- cluster.index(xx$id)
  firstid <- cc$firstclustid+1
  if (max(cc$cluster.size)==1) stop("No clusters !, maxclust size=1\n");

  ### order after time-sorted data
  theta.des <- theta.des[xx$ord+1,,drop=FALSE]
  weightsid <- weights <- weights[xx$ord+1]
  ### clusterspecific weights
  weights <- weights[firstid]

  thetaX<- as.matrix(theta.des[firstid,,drop=FALSE])

obj <- function(par,all=FALSE)
{

if (var.link==1) epar <- c(exp(c(par))) else epar <- c(par)

thetav <- c(as.matrix(theta.des) %*%  c(epar))
thetai <-thetav[firstid]

Hs <- sumstrata(H*exp(thetav*H),xx$id,mid)
R <- sumstrata((exp(thetav*H)-1),xx$id,mid) + 1
H2 <- sumstrata(H^2*exp(thetav*H),xx$id,mid)

l1 <- sumstrata(log(1+thetav*Nsum$lagsum)*statusxx,xx$id,mid)
l2 <- sumstrata(statusxx*H,xx$id,mid)
l3 <- -((thetai)^{-1}+Ni.tau) * log(R)
logliid <- (l1 + thetai* l2 + l3)*c(weights)
logl <- sum(logliid)
ploglik <- logl

## first derivative
l1s <- sumstrata(Nsum$lagsum/(1+thetav*Nsum$lagsum)*statusxx,xx$id,mid)
l2s <- (thetai^{-2}) * log(R)
l3s <- -(thetai^{-1}+Ni.tau) * Hs / R
Dltheta <- (l1s+l2s+l3s+l2)*c(weights)
scoreiid <- thetaX* c(Dltheta)

## second derivative
D2N <- -sumstrata(Nsum$lagsum^2/(1+thetav*Nsum$lagsum)^2*statusxx,xx$id,mid)
Dhes  <- c(D2N+(2/thetai^2)*Hs/R-(2/thetai^3)*log(R)-(1/thetai+Ni.tau)*(H2*R-Hs*Hs)/R^2)
Dhes  <-  Dhes * c(weights)

if (var.link==1) {
scoreiid  <-  scoreiid * c(thetai);  
Dhes<- Dhes* thetai^2 + thetai *  Dltheta
}

gradient <- apply(scoreiid,2,sum)
hessian <-  crossprod(thetaX,thetaX*c(Dhes))
hess2 <-    crossprod(scoreiid)

val <- list(id=xx$id,score.iid=scoreiid,logl.iid=logliid,ploglik=ploglik,
            gradient=-gradient,hessian=hessian,hess2=hess2)

if (all) return(val);

with(val, structure(-ploglik,gradient=-gradient,hessian=hessian))
}

  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          opt <- lava::NR(theta,obj,...)
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,theta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;
###   names(cc) <- colnames(theta.des)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=theta),obj(theta,all=TRUE))
  val$score <- val$gradient


  theta <- matrix(c(val$coef),length(c(val$coef)),1)
  if (!is.null(colnames(theta.des))) thetanames <- colnames(theta.des) else thetanames <- paste("dependence",1:length(c(theta)),sep="")
  if (length(thetanames)==length(c(theta))) { rownames(theta) <- thetanames; rownames(var.theta) <- colnames(var.theta) <- thetanames; }

  hessianI <- solve(val$hessian)
  val$theta.iid.naive  <-  val$score.iid %*% hessianI

  ### iid due to Marginal model
  biid <- 1
  if (biid==1) {

  if ((margsurv$no.opt) | is.null(margsurv$coef)) fixbeta<- 1 else fixbeta <- 0

   id  <-  xx$id+1
   if (var.link==1) epar <- c(exp(theta)) else epar <- c(theta)

   thetaX<- as.matrix(theta.des[firstid,,drop=FALSE])
   thetav <- c(as.matrix(theta.des) %*%  c(epar))

   Hs <- c(sumstrata(H*exp(thetav*H),xx$id,mid))
   R <- c(sumstrata((exp(thetav*H)-1),xx$id,mid)) + 1
   H2 <- c(sumstrata(H^2*exp(thetav*H),xx$id,mid) )

   Ft <- ((1/(thetav*R[id]))*exp(thetav*H)-(1/thetav+Ni.tau[id])*(1+thetav*H)*exp(thetav*H)/R[id]+statusxx+(1+thetav*Ni.tau[id])*exp(thetav*H)*Hs[id]/R[id]^2)

  if (var.link==1) {
	Ft  <-  Ft * c(exp(theta.des %*% val$coef));
  }
  Gt <- c(RR *Ft * xx$sign * weightsid)
  Ft  <- c( Ft * H * weightsid )
  Fbeta <-  t(theta.des) %*% ( xx$X * Ft )
  Gbase <- apply(Gt* theta.des,2,revcumsumstrata,xx$strata,xx$nstrata)

  ### beta part
  if (fixbeta==0) {
	  Z <- xx$X
	  U <- E <- matrix(0,nrow(xx$X),margsurv$p)
	  E[xx$jumps+1,] <- margsurv$E
	  U[xx$jumps+1,] <- margsurv$U
          invhess <- -solve(margsurv$hessian)
	  S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/margsurv$S0
	  cumhaz <- c(cumsumstrata(S0i,xx$strata,xx$nstrata))
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset))
	  ### Martingale  as a function of time and for all subjects to handle strata
	  MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
          UUbeta <- apply(MGt,2,sumstrata,id-1,max(id))
	  UUbeta <- UUbeta %*% invhess
	  GbaseEdLam0 <- t(Gbase) %*% (E*S0i)
	  Fbeta   <-  Fbeta -  GbaseEdLam0
          Fbetaiid <- UUbeta %*% t(Fbeta)
  }

  ###  \int_0^\tau (GBase(s) / S_0(s)) dN_i(s) - dLamba_i(s)
  xx <- margsurv$cox.prep
  S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <- 1/margsurv$S0
  S0i2[xx$jumps+1] <- 1/margsurv$S0^2

  GbasedN <- Gbase*S0i
  GbasedLam0 <- apply(Gbase*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)

  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(margsurv) + xx$offset)) else rr <- c(xx$sign*exp( xx$offset))
  MGt <- (GbasedN[,drop=FALSE]-GbasedLam0*rr)*c(xx$weights)
  MGti <- apply(MGt,2,sumstrata,id-1,max(id))

###  if (fixbeta==0) print(cbind(val$score.iid,MGti,Fbetaiid)) else print(cbind(val$score.iid,MGti))

  if (fixbeta==0) MGti <- MGti+Fbetaiid
  theta.iid <-  val$score.iid + MGti
  theta.iid  <-  theta.iid %*% hessianI
  ## take names from phreg call
  if (!is.null(margsurv$id)) rownames(theta.iid) <- unique(margsurv$id)
  val$theta.iid <- theta.iid

  }

  ### if se.cluster is there we cluster iid-decomp before squaring 
  if (!is.null(se.cluster)) 
	  if (length(se.cluster)!=length(clusters)) stop("Length of seclusters and clusters must be same\n");

  if (!is.null(se.cluster)) {
      iids <-  unique(se.cluster);
      nseclust  <- length(iids);
      if (is.numeric(se.cluster)) se.cluster <-  fast.approx(iids,se.cluster)-1
       else se.cluster <- as.integer(factor(se.cluster, labels = seq(nseclust)))-1

      val$theta.iid <-       apply(val$theta.iid,se.cluster,nseclust)
      val$theta.iid.naive <- apply(val$theta.iid.naive,se.cluster,nseclust)
  }

  var <- robvar.theta  <-  var.theta  <-  crossprod(val$theta.iid)
  naive.var  <-   crossprod(val$theta.iid.naive)

  val  <- c(val,list(theta=theta,var.theta=var.theta,robvar.theta=robvar.theta,
		     var=var,thetanames=thetanames,model="clayton.oakes",
	             se=diag(robvar.theta)^.5),
	             var.naive=naive.var)
  class(val) <- "mets.twostage"
  attr(val,"clusters") <- clusters
  attr(val,"secluster") <- c(se.cluster)
  attr(val,"var.link")<-var.link;
  attr(val,"ptheta")<-ptheta
  attr(val,"n")<-n ;
  attr(val,"response")<- "survival"
  attr(val,"additive-gamma")<-0
  attr(val,"twostage") <- "two.stage"

  return(val)
}

##' @export
summary.mets.twostage <- function(object,digits = 3,silent=0,theta.des=NULL,...)
{ #
  if (!(inherits(object,"mets.twostage"))) stop("Must be a Two-Stage object")

  var.link<-attr(object,"var.link");
  var.par  <- attr(object,"var.par");
  model <- object$model
  if ((model=="plackett" || model=="or") )         model <- "or"
  if ((model=="clayton.oakes" || model=="gamma") ) model <- "gamma"
  if ((attr(object,"additive-gamma")==1) & (silent==0)) addgam <- TRUE else addgam <- FALSE

  if ((model=="or") && (silent==0)) cat("Dependence parameter for Odds-Ratio (Plackett) model \n");
  if (attr(object,"response")=="binomial") response <- "binomial" else response <- "survival"
  if ((model=="gamma") && (silent==0)) {
	  cat("Dependence parameter for Clayton-Oakes model\n")
	  if (var.par==1 || !addgam) cat("Variance of Gamma distributed random effects \n");
	  if (var.par==0 && addgam) cat("Inverse of variance of Gamma distributed random effects \n");
  }
  if (var.link==1 && silent==0) cat("With log-link \n")
  if ((sum(abs(object$score))>0.0001) & (silent==0))  {
	  cat(" Variance parameters did not converge, allow more iterations.\n");
	  cat(paste("Score:",object$score,"  \n"));
  }
  theta <- object$theta
  if (is.null(rownames(theta)))  rownames(theta) <- paste("dependence",1:nrow(theta),sep="")

  coefs <- coef.mets.twostage(object,response=response,...);

  if (attr(object,"additive-gamma")==1 & (!is.null(object$robvar.theta))  ) {
      var.link <- attr(object,"var.link");
      var.par  <- attr(object,"var.par");
      rv1      <- attr(object,"rv1");
      if (is.matrix(rv1)) rv1 <- rv1[1,]
      theta.des <- attr(object,"pardes");
      ags <- attr(object,"additive.gamma.sum");
      ptheta <- attr(object,"ptheta")
      npar <- nrow(object$theta)
      theta <- object$theta[seq(1,ptheta),1,drop=FALSE]
      robvar.theta <- object$robvar.theta[seq(1,ptheta),seq(1,ptheta)]
      if (var.link==1) par <- theta.des %*% exp(theta) else  par <- theta.des %*% theta

      if (model=="or" || model=="plackett") var.par<-1 ## same as var.par=0 for this model

      if (attr(object,"twostage")==0) {
      }

      if (var.par==0) { #
	      if (var.link==1) {
		  fp <- function(p,d,t){ res <- exp(p*t)/(sum(rv1* c(theta.des %*% matrix(exp(p),ncol=1))))^d;
					     if (t==0) res <- res[1]; return(res); }
                  pare <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) exp(p),labels=rownames(theta))
	      } else {
		      fp <- function(p,d,t) {  res <- (p^t)/(sum(rv1* c(theta.des %*% matrix(p,ncol=1))))^d;
					     if (t==0) res <- res[1]; return(res); }
	          pare <- NULL
	      }

             e      <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
             vare <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,2,1),labels=rownames(theta))
             vartot <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,0))
      } #
      if (var.par==1) { #

	   if (var.link==1) { #
	     fp <- function(p,d,t){  res <- exp(p*t)/(sum(rv1* c(theta.des %*% matrix(exp(p),ncol=1))))^d;
                                     if (t==0) res <- res[1]; return(res); }
             e      <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
             vare   <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) exp(p),labels=rownames(theta))
             vartot <-   lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) sum(exp(p)))
      } else {
              fp <- function(p,d,t) {  res <- (p^t)/(sum(rv1* c(theta.des %*% matrix(p,ncol=1))))^d;
                                     if (t==0) res <- res[1]; return(res); }
              e     <-  lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,1,1),labels=rownames(theta))
              pare  <-  lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) fp(p,2,1),labels=rownames(theta))
              vartot <- lava::estimate(coef=theta,vcov=robvar.theta,f=function(p) sum(p))
	      vare <- NULL
	       } #
      } #
      res <- list(estimates=coefs, type=attr(object,"Type"),h=e,vare=vare,vartot=vartot)

  } else {
	 if (var.link==1) { #
	     if (model=="or") {
		or <-  lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p),labels=rownames(theta))
	        res <- list(estimates=coefs,or=or,type=attr(object,"Type"))
	     } else {
	        vargam <-  lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) exp(p),labels=rownames(theta))
                res <- list(estimates=coefs,vargam=vargam, type=attr(object,"Type"))
	     }
      } else {
	     if (model=="or") {
             lor <- lava::estimate(coef=object$theta,vcov=object$var.theta,f=function(p) log(p),labels=rownames(theta))
	     res <- list(estimates=coefs,log.or=lor,type=attr(object,"Type"))
	     } else  {
             res <- list(estimates=coefs,type=attr(object,"Type"))
	     }
      } #
  }

  class(res) <- "summary.mets.twostage"
  res
} #

##' @export
coef.mets.twostage <- function(object,var.link=NULL,response="survival",...)
{ #
  pt <- attr(object,"ptheta")
  theta <- object$theta[seq(pt),1]

  if (is.null(object$robvar.theta))
	  var.theta  <-  object$var.theta[seq(1,pt),seq(1,pt),drop=FALSE] else
	  var.theta  <-   object$robvar.theta[seq(1,pt),seq(1,pt),drop=FALSE]
  se <- diag(var.theta)^.5

  if (is.null(var.link))
     if (attr(object,"var.link")==1) vlink <- 1 else vlink <- 0
     else vlink <- var.link
  res <- cbind(theta, se )
  wald <- theta/se
  waldp <- (1 - pnorm(abs(wald))) * 2
  if (response=="survival") {
       if (object$model=="plackett") {
       spearman <- alpha2spear(theta,link=vlink)
       Dspear <- numDeriv::jacobian(alpha2spear,theta,link=vlink)
       var.spearman <- Dspear %*% var.theta %*%  Dspear
       se.spearman <- diag(var.spearman)^.5
       res <- as.matrix(cbind(res, wald, waldp,spearman,se.spearman))
       if (vlink==1) colnames(res) <- c("log-Coef.", "SE","z", "P-val","Spearman Corr.","SE")
	  else colnames(res) <- c("Coef.", "SE","z", "P-val","Spearman Corr.","SE")
	  if ((!is.null(object$thetanames)) & (nrow(res)==length(object$thetanames))) rownames(res)<-object$thetanames
       }
       if (object$model=="clayton.oakes") {
       kendall <- alpha2kendall(theta,link=vlink)
       Dken <- numDeriv::jacobian(alpha2kendall,theta,link=vlink)
       var.kendall<- Dken %*% var.theta %*%  Dken
       se.kendall <- diag(var.kendall)^.5
       res <- as.matrix(cbind(res, wald, waldp,kendall,se.kendall))
       if (vlink==1) colnames(res) <- c("log-Coef.", "SE","z", "P-val","Kendall tau","SE")
       else colnames(res) <- c("Coef.", "SE","z", "P-val","Kendall tau","SE")
       if ((!is.null(object$thetanames)) & (nrow(res)==length(object$thetanames))) rownames(res)<-object$thetanames
       }
  }
  return(res)
} #

##' @export
print.mets.twostage<-function(x,digits=3,...)
{ #
  cat("\n")
  print(summary(x,silent=0));
} #

##' @export
plot.mets.twostage<-function(x,pointwise.ci=1,robust=0,specific.comps=FALSE,
		level=0.05,
		start.time=0,stop.time=0,add.to.plot=FALSE,mains=TRUE,
                xlab="Time",ylab ="Cumulative regression function",...)
{ #
  if (!(inherits(x, 'two.stage'))) stop("Must be a Two-Stage object")
  object <- x; rm(x);

  B<-object$cum; V<-object$var.cum; p<-dim(B)[[2]];
  if (robust>=1) V<-object$robvar.cum;

  if (sum(specific.comps)==FALSE) comp<-2:p else comp<-specific.comps+1
  if (stop.time==0) stop.time<-max(B[,1]);

  med<-B[,1]<=stop.time & B[,1]>=start.time
  B<-B[med,]; Bs<-B[1,];  B<-t(t(B)-Bs); B[,1]<-B[,1]+Bs[1];
  V<-V[med,]; Vs<-V[1,]; V<-t( t(V)-Vs);
  Vrob<-object$robvar.cum;
  Vrob<-Vrob[med,]; Vrobs<-Vrob[1,]; Vrob<-t( t(Vrob)-Vrobs);

  c.alpha<- qnorm(1-level/2)
  for (v in comp) {
    c.alpha<- qnorm(1-level/2)
    est<-B[,v];ul<-B[,v]+c.alpha*V[,v]^.5;nl<-B[,v]-c.alpha*V[,v]^.5;
    if (add.to.plot==FALSE)
      {
        plot(B[,1],est,ylim=1.05*range(ul,nl),type="s",xlab=xlab,ylab=ylab)
        if (mains==TRUE) title(main=colnames(B)[v]); }
    else lines(B[,1],est,type="s");
    if (pointwise.ci>=1) {
      lines(B[,1],ul,lty=pointwise.ci,type="s");
      lines(B[,1],nl,lty=pointwise.ci,type="s"); }
    if (robust>=1) {
      lines(B[,1],ul,lty=robust,type="s");
      lines(B[,1],nl,lty=robust,type="s"); }
    abline(h=0);
  }
}  #

##' @export
matplot.mets.twostage <- function(object,...)
{ #
B <- object$baseline
matplot(B[,1],B[,-1],type="s",...)
} #

##' @export
predict.mets.twostage <- function(object,X=NULL,Z=NULL,times=NULL,times2=NULL,theta.des=NULL,diag=TRUE,...)
{ #
time.coef <- data.frame(object$cum)
if (!is.null(times)) {
cum <- cpred(object$cum,times);
cum2 <- cpred(object$cum,times);
} else { cum <- object$cum; cum2 <- object$cum }
if (!is.null(times2)) cum2 <- cpred(object$cum,times2);

if (is.null(X)) X <- 1;
if (is.null(X) & (!is.null(Z))) { Z <- as.matrix(Z);  X <- matrix(1,nrow(Z),1)}
if (is.null(Z) & (!is.null(X)))  {X <- as.matrix(X);  Z <- matrix(0,nrow(X),1); gamma <- 0}

if (diag==FALSE) {
   time.part <-  X %*% t(cum[,-1])
   time.part2 <-  X %*% t(cum2[,-1])
   if (!is.null(object$gamma)) { RR <- exp( Z %*% gamma );
       cumhaz <- t( t(time.part) * RR ); cumhaz2 <- t( t(time.part2) * RR )}
	    else { cumhaz <- time.part;  cumhaz2 <- time.part2;   }
} else {
	time.part <-  apply(as.matrix(X*cum[,-1]),1,sum)
	time.part2 <-  apply(as.matrix(X*cum2[,-1]),1,sum)
}

if (!is.null(object$gamma)) {
	RR<- exp(Z%*%gamma);
	cumhaz <- t( t(time.part) * RR );
	cumhaz2 <- t( t(time.part2) * RR )} else {
		cumhaz <- time.part;  cumhaz2 <- time.part2;
}
S1 <- exp(-cumhaz); S2 <- exp(-cumhaz2)

if (attr(object,"var.link")==1) theta  <- exp(object$theta) else theta <- object$theta
if (!is.null(theta.des)) theta <- c(theta.des %*% object$theta)

if (diag==FALSE) St1t2<- (outer(c(S1)^{-(theta)},c(S2)^{-(theta)},FUN="+") - 1)^(-(1/theta)) else
St1t2<- ((S1^{-(theta)}+S2^{-(theta)})-1)^(-(1/theta))

out=list(St1t2=St1t2,S1=S1,S2=S2,times=times,times2=times2,theta=theta)
return(out)
} #

##' @export ascertained.pairs
ascertained.pairs <-function (pairs,data,cr.models,bin=FALSE)
{
      timestatus <- all.vars(cr.models)
      ### let first event by second column and only
      ### use pairs where first is event
      apairs <- pairs
      if (bin==TRUE) fj <- ifelse(data[pairs[,1],timestatus[1]] > data[pairs[,2],timestatus[1]],1,2)
      else fj <- ifelse(data[pairs[,1],timestatus[1]] < data[pairs[,2],timestatus[1]],1,2)
      ### change order when 1st comes first
      apairs[fj==1,1] <- pairs[fj==1,2]
      apairs[fj==1,2] <- pairs[fj==1,1]
      ### only take pairs where first is a jump
      if (bin==FALSE) {
	      jmpf <- (data[apairs[,2],timestatus[2]]==1)
	      apairs <- apairs[data[apairs[,2],timestatus[2]]==1,]
              attr(pairs,"jump-first") <- jmpf
      }
      pairs <- apairs
      return(pairs)
} 

##' @export
alpha2spear <- function(theta,link=1) { #
   if (link==1) theta <- exp(theta)
if (length(theta)>1) {
   out <- c()
   for (thet in theta) {
   if (thet!=1) out <- c(out,( (thet+1)/(thet-1) -2* thet* log(thet)/ (thet-1)^2))
   else out <- c(out,0)
   }
} else { if (theta!=1) out <- ( (theta+1)/(theta-1) -2* theta* log(theta)/ (theta-1)^2) }

return(out)
} #

##' @export
alpha2kendall <- function(theta,link=0) {  #
   if (link==1) theta <- exp(theta)
   return(1/(1+2/theta))
} #

##' @export piecewise.twostage
piecewise.twostage <- function(cut1,cut2,data=parent.frame(),timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,
            method="optimize",Nit=100,detail=0,silent=1,weights=NULL,
            control=list(),theta=NULL,theta.des=NULL,var.link=1,
	    step=0.5,model="plackett",data.return=0)
{ #{{{
iid <- 1
ud <- list()
if (missing(cut2)) cut2 <- cut1;
nc1 <- length(cut1); nc2 <- length(cut2)
names1 <- names2 <- c()
theta.mat <- se.theta.mat <- cor.mat <- score.mat <- se.cor.mat <- matrix(0,nc1-1,nc2-1);
clusters <- data[,id]
cluster.call <- clusters
idi <- unique(data[,id]);

if (iid==1) { theta.iid <- matrix(0,length(idi),(nc1-1)*(nc2-1));
              rownames(theta.iid) <- idi
            } else theta.iid <- NULL

thetal <- c()
k <- 0;
for (i1 in 2:nc1)
for (i2 in 2:nc2)
{
k <-(i1-2)*(nc2-1)+(i2-1)
if (silent<=0) cat(paste("Data-set ",k,"out of ",(nc1-1)*(nc2-1)),"\n");
datalr <- surv.boxarea(c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),data,timevar=timevar,
status=status,id=id,covars=covars,covars.pairs=covars.pairs,num=num,silent=silent)
if (silent<=-1) print("back in piecewise.twostage");
if (silent<=-1) print(summary(datalr));
if (silent<=-1) print(head(datalr));
if (silent<=-1) print(summary(datalr[,id]));
 boxlr <- list(left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]))
datalr$tstime <- datalr[,timevar]
datalr$tsstatus <- datalr[,status]
datalr$tsid <- datalr[,id]
###

if (is.null(covars))
f <- as.formula(with(attributes(datalr),paste("Surv(",time,",",status,")~-1+factor(",num,")")))
else f <- as.formula(with(attributes(datalr),paste("Surv(",time,",",status,")~-1+factor(",num,"):",covars)))
marg1 <- timereg::aalen(f,data=datalr,n.sim=0,robust=0)

fitlr<-  survival.twostage(marg1,data=datalr,clusters=datalr$tsid,
,model=model, Nit=Nit,detail=detail,silent=silent,weights=weights,
theta=theta,theta.des=theta.des,var.link=var.link,step=step)
###fitlr<-  survival.twostageCC(marg1,data=datalr,clusters=datalr$tsid,model=model,method=method,
###Nit=Nit,detail=detail,silent=silent,weights=weights,
###baseline.iid=0,control=control,
###theta=theta,theta.des=theta.des,var.link=var.link,step=step)
####
coef <- coef(fitlr)
theta.mat[i1-1,i2-1] <- fitlr$theta
se.theta.mat[i1-1,i2-1] <- fitlr$var.theta^.5
cor.mat[i1-1,i2-1] <- coef[1,5]
se.cor.mat[i1-1,i2-1] <- coef[1,6]
score.mat[i1-1,i2-1] <- fitlr$score
if (data.return==0)
ud[[k]] <- list(index=c(i1,i2),left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]),fitlr=fitlr)
if (data.return==1)
ud[[k]] <- list(index=c(i1,i2),left=c(cut1[i1-1],cut2[i2-1]),right=c(cut1[i1],cut2[i2]),fitlr=fitlr,data=datalr)
if (i2==2) names1 <- c(names1, paste(cut1[i1-1],"-",cut1[i1]))
if (i1==2) names2 <- c(names2, paste(cut2[i2-1],"-",cut2[i2]))
thetal <- c(thetal,fitlr$theta)

if ((silent<=-1) & (iid==1)) print(head(fitlr$theta.iid));
if ((silent<=-1) & (iid==1)) {
print(idi) ; print(datalr$tsid)
print(dim(fitlr$theta.iid))
print(head(fitlr$theta.iid))
print(dim(theta.iid))
print(length( idi %in% unique(datalr$tsid)))
}
if (iid==1) theta.iid[idi %in% unique(datalr$tsid),k] <-c(fitlr$theta.iid)
###if (iid==1) theta.iid[rownames(fitlr$theta.iid),k] <-  fitlr$theta.iid
}

var.thetal <- NULL
if (iid==1)  var.thetal <- t(theta.iid) %*% theta.iid

colnames(score.mat) <- colnames(cor.mat) <-  colnames(se.cor.mat)  <- colnames(se.theta.mat) <- colnames(theta.mat) <- names1;
rownames(score.mat) <- rownames(cor.mat) <-  rownames(se.cor.mat) <-  rownames(se.theta.mat) <- rownames(theta.mat) <- names2;

ud <- list(model.fits=ud,theta=theta.mat,var.theta=se.theta.mat^2,
	   se.theta=se.theta.mat,thetal=thetal,thetal.iid=theta.iid,var.thetal=var.thetal,model=model,
	   cor=cor.mat,se.cor=se.cor.mat,score=score.mat);
class(ud)<-"pc.twostage"
attr(ud,"var.link")<-var.link;
attr(ud, "Type") <- model
return(ud);
} #}}}

##' @export piecewise.data
piecewise.data <- function(cut1,cut2,data=parent.frame(),timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1)
{ #
ud <- list()
if (missing(cut2)) cut2 <- cut1;
nc1 <- length(cut1); nc2 <- length(cut2)
dataud <- c()

k <- 0;
for (i1 in 2:nc1)
for (i2 in 2:nc2)
{
k <-(i1-2)*(nc2-1)+(i2-1)
if (silent<=0) cat(paste("Data-set ",k,"out of ",(nc1-1)*(nc2-1)),"\n");
 datalr <- surv.boxarea(c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),data,timevar=timevar,
			status=status,id=id,covars=covars,covars.pairs=covars.pairs,num=num,silent=silent)
if (silent<=-1) print(summary(datalr));
if (silent<=-1) print(head(datalr));
datalr$tstime <- datalr[,timevar]
datalr$tsstatus <- datalr[,status]
datalr$tsid <- datalr[,id]
datalr$strata <- paste( c(cut1[i1-1],cut2[i2-1]),c(cut1[i1],cut2[i2]),collapse=",",sep="-")
datalr$intstrata <-
c(paste(c(cut1[i1-1],cut1[i1]),collapse=",",sep="-"),paste( c(cut2[i2-1],cut2[i2]),collapse=",",sep="-"))

if (silent<=-1) print(head(datalr));
dataud <- rbind(dataud,datalr)
}

return(data.frame(dataud))
} #

##' @export
summary.pc.twostage <- function(object,var.link=NULL,...)
{ #
  if (!(inherits(object,"pc.twostage"))) stop("Must be a Piecewise constant two-Stage object")

  res <- list(estimates=object$theta,se=object$se.theta,cor=object$cor,se.cor=object$se.cor,
	      model=object$model,score=object$score)
  class(res) <- "summary.pc.twostage"
  attr(res,"var.link")<-attr(object,"var.link");
  attr(res, "Type") <- object$model
  res
} #

##' @export
print.pc.twostage <- function(x,var.link=NULL,...)
{ #
   if (!(inherits(x,"pc.twostage"))) stop("Must be a Piecewise constant two-Stage object")
   print( summary(x,var.link=var.link,...))
} #

##' @export
print.summary.pc.twostage <- function(x,var.link=NULL, digits=3,...)
{ #

  if (is.null(var.link)) { if (attr(x,"var.link")==1) vlink <- 1 else vlink <- 0; } else vlink <- var.link
  print(vlink)

  if (x$model=="plackett") cat("Dependence parameter for Plackett model \n");
  if (x$model=="clayton.oakes") cat("Dependence parameter for Clayton-Oakes model \n");

  if (max(x$score)>0.001) { cat("Score of log-likelihood for parameter estimates (too large?)\n"); print(x$score);cat("\n\n");}

  if (vlink==1) cat("log-coefficient for dependence parameter (SE) \n")  else cat("Dependence parameter (SE) \n");
  print(coefmat(x$estimate,x$se,digits=digits,...))
  cat("\n")

  if (x$model=="plackett") {cat("Spearman Correlation (SE) \n");cor.type <- "Spearman Correlation"; }
  if (x$model=="clayton.oakes") {cat("Kendall's tau (SE) \n"); cor.type <- "Kendall's tau";}

  print(coefmat(x$cor,x$se.cor,digits,...))
  cat("\n")
} #

##' @export
coefmat <- function(est,stderr,digits=3,...) { #
  myest <- round(10^digits*(est))/10^digits;
  myest <- paste(ifelse(myest<0,""," "),myest,sep="")
  mysd <- round(10^digits*(stderr))/10^digits;
  res <- matrix(paste(format(myest)," (",format(mysd),")",sep=""),ncol=ncol(est))
  dimnames(res) <- dimnames(est)
  colnames(res) <- paste("",colnames(res))
  noquote(res)
} #


##' @export
simSurvFam <- function(n,beta=0.0,theta=1,lam0=0.5,lam1=1,lam2=1,ctime=10,...) { #
xm <- rbinom(n,1,0.5); xf <- rbinom(n,1,0.5);
xb1 <- rbinom(n,1,0.5); xb2 <- rbinom(n,1,0.5);
###
zf <- rgamma(n,shape=lam1); zb <- rgamma(n,shape=lam2);
tm <- rexp(n)/(zf*exp(xm*beta)*lam0)
tf <- rexp(n)/(zf*exp(xf*beta)*lam0)
tb1 <- rexp(n)/((zf+zb)*exp(xb1*beta)*2*lam0)
tb2 <- rexp(n)/((zf+zb)*exp(xb2*beta)*2*lam0)
cm <- ifelse(tm<ctime,1,0); cf <- ifelse(tf<ctime,1,0);
cb1 <- ifelse(tb1<ctime,1,0); cb2 <- ifelse(tb2<ctime,1,0);
tm <- ifelse(tm<ctime,tm,ctime); tf <- ifelse(tf<ctime,tf,ctime)
tb1 <- ifelse(tb1<ctime,tb1,ctime); tb2 <- ifelse(tb2<ctime,tb2,ctime)
#
data.frame(xm=xm,xf=xf,xb1=xb1,xb2=xb2,timem=tm,timef=tf,timeb1=tb1,timeb2=tb2,statusm=cm,statusf=cf,
	   statusb1=cb1,statusb2=cb2,id=1:n)
} #

##' @export
object.defined <- function(object)
{
   exists(as.character(substitute(object)))
}

##' @export
twin.polygen.design <-function (data,id="id",zyg="DZ",zygname="zyg",type="ace",tv=NULL,...) { #
  ### twin case
  id <- data[,id]
  tv <- diff(c(NA,id))
  tv[tv!=0 | is.na(tv)] <- 1
  tv[tv==0] <- 2

  zygbin <- (data[,zygname]==zyg)*1
  zygdes=model.matrix(~-1+factor(zygbin),data)
  n <- length(zygbin)

  if (type=="ace") { ### ace #
  ### random effects for each cluster
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,1)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2","env")
  pard <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0), c(0,1))
  } #

  if (type=="ae") { ### ae #
  ###AE model
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2")
  pard <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0))[,1,drop=FALSE]
  } #

  if (type=="dce") { ### dce #
  ### DCE
  ### random effects for each cluster
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,1)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2","env")
  pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0), c(0,1))
  } #

  if (type=="ade") { ### ade #
  #ADE
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns,DZns,1)
  pard <- rbind(c(1,0),c(0.25,0),c(0.75,0),c(0.75,0),c(0,1),c(0,0.5),c(0,0.5),c(0,0.5) )
  pardes <- matrix(pard,n,16,byrow=TRUE)
  des.rv <- NULL
  } #

  if (type=="adce") { ### adce #
  } #

  if (type=="de") { ### ae #
  DZns <- cbind((zygbin==1)*(tv==1)*cbind(rep(1,n),rep(0,n))+
		(zygbin==1)*(tv==2)*cbind(rep(0,n),rep(1,n)))
  des.rv <- cbind(zygdes,DZns)
  colnames(des.rv) <- c("MZ","DZ","DZns1","DZns2")
  pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0))[,1,drop=FALSE]
  } #

  if (type=="un") { ### ae #
  des.rv <- cbind(zygdes)
  colnames(des.rv) <- c("MZ","DZ")
  pard <- rbind(c(1,0),c(0,1))
  } #

res <- list(pardes=pard,des.rv=des.rv)
return(res)
} #

##' @export
ace.family.design <-function (data,id="id",member="type",mother="mother",father="father",child="child",child1="child",type="ace",...) {
#
  ### standard family case
###  nid <- table(data[,id])
  id <- data[,id]

  if (type=="ace") { ### ace #
  ### random effects for each cluster
	  mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2,1)

	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4","env")
	  pard <- rbind( c(0.25,0), c(0.25,0),c(0.25,0), c(0.25,0),
			 c(0.25,0), c(0.25,0),c(0.25,0), c(0.25,0), c(0,1))
  } #

  if (type=="ae") { ### ae #
  ###AE model
          mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2)
	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4")
	  pard <- rbind( c(0.25), c(0.25),c(0.25), c(0.25),
			 c(0.25), c(0.25),c(0.25), c(0.25))
  } #

  if (type=="dce") { ### dce #
  ### DCE
  ### random effects for each cluster
	  stop("not done yet");
          mom <- 1*(data[,member]==mother)
	  mo <- cbind(mom*1,1,1,1)*(mom)
	  fa <- (data[,member]==father)
	  fad <- cbind(fa,1,1,1)*fa
	  ch1 <- (data[,member]==child)*(data[,child1]==1)
	  cc1 <- cbind(ch1,1,0,0,1,1,0,0)*ch1
	  ch2 <- (data[,member]==child)*(data[,child1]==0)
	  cc2 <- cbind(ch2,0,1,0,1,0,1,0)*ch2
	  des.rv <- cbind(cbind(mo,fad)+cc1+cc2)
	  colnames(des.rv) <- c("m1","m2","m3","m4","f1","f2","f3","f4")
	  pard <- rbind( c(0.25), c(0.25),c(0.25), c(0.25),
			 c(0.25), c(0.25),c(0.25), c(0.25))

  } #

  if (type=="ade") { ### ade #
  #ADE
    stop("not done yet");
###    pard <- rbind(c(1,0),c(0.25,0),c(0.75,0),c(0.75,0),c(0,1),c(0,0.5),c(0,0.5),c(0,0.5) )
###    pardes <- matrix(pard,n,16,byrow=TRUE)
  } #

  if (type=="adce") { ### adce #
	  stop("not done yet");
  } #

  if (type=="de") { ### ae #
    stop("not done yet");
    pard <- rbind(c(1,0), c(0.25,0),c(0.75,0), c(0.75,0))[,1,drop=FALSE]
  } #

  if (type=="un") { ### ae #
	 stop("not done yet");
         pard <- diag(4)
  } #

res <- list(pardes=pard,des.rv=des.rv)
return(res)
} #

##' @export
make.pairwise.design  <- function(pairs,kinship,type="ace")
{ #
### makes pairwise random effects design for shared and non-shared random effects
### kinship gives shared genes for each pair

uk <- sort(unique(kinship))
iuk <- fast.approx(uk,kinship)

if (type=="ace")  {
   theta.des <-  matrix(0,length(uk),8)
   i <- 0
   for (uuk in uk)  {
     i <- i+1
     theta.des[i,] <-c( rbind(c(uuk,0), c(1-uuk,0), c(1-uuk,0), c(0,1)))
   }
   random.des <- rbind(c(1,1,0,1),c(1,0,1,1))
   rvs <- rep(4,length(kinship))
   new.pairs <- cbind(pairs,1,2,iuk,4)
}

if (type=="ae") {
  i <- 0
  theta.des <-  matrix(0,length(uk),3)
  for (uuk in uk)  {
    i <- i+1
    theta.des[i,] <- rbind(c(uuk), c(1-uuk), c(1-uuk))
  }
  random.des <- rbind(c(1,1,0),c(1,0,1))
  rvs <- rep(3,length(kinship))
  new.pairs <- cbind(pairs,1,2,iuk,3)
}

if (type=="ade") {
   stop(" not specified yet \n")
   theta.des <-  matrix(0,length(uk),8)
   i <- 0
   for (uuk in uk)  {
      i <- i+1
      theta.des[i,] <- rbind(c((uuk==1)+(uuk!=1)*uuk*0.5,0), c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),
			c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),c(0,1))
   }
   random.des <- rbind(c(1,1,0,1),c(1,0,1,1))
   rvs <- rep(4,length(kinship))
   new.pairs <- cbind(pairs,1,2,iuk,4)
}

if (type=="ad") {
   stop(" not specified yet \n")
   i <- 0
   for (uuk in uk)  {
   i <- i+1
   theta.des[i,] <- rbind(c((uuk==1)+(uuk!=1)*uuk*0.5,0), c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),
			c(1-(uuk==1)+(uuk!=1)*uuk*0.5,0),c(0,1))
   random.des <- rbind(c(1,1,0),c(1,0,1))
   new.pairs <- cbind(pairs,1,2,iuk,2)
   }
}

return(list(new.pairs=new.pairs,theta.des=theta.des,random.design=random.des))
} #


##' Relative risk for additive gamma model
##'
##' Computes the relative risk for additive gamma model at time 0
##'
##' @references
##'
##' Eriksson and Scheike (2015), Additive Gamma frailty models for competing risks data, Biometrics (2015)
##'
##' @examples
##' lam0 <- c(0.5,0.3)
##' pars <- c(1,1,1,1,0,1)
##' ## genetic random effects, cause1, cause2 and overall
##' parg <- pars[c(1,3,5)]
##' ## environmental random effects, cause1, cause2 and overall
##' parc <- pars[c(2,4,6)]
##'
##' ## simulate competing risks with two causes with hazards 0.5 and 0.3
##' ## ace for each cause, and overall ace
##' out <- simCompete.twin.ace(10000,parg,parc,0,2,lam0=lam0,overall=1,all.sum=1)
##'
##' ## setting up design for running the model
##' mm <- familycluster.index(out$cluster)
##' head(mm$familypairindex,n=10)
##' pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
##' tail(pairs,n=12)
##' #
##' kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5
##'
##' # dout <- make.pairwise.design.competing(pairs,kinship,
##' #          type="ace",compete=length(lam0),overall=1)
##' # head(dout$ant.rvs)
##' ## MZ
##' # dim(dout$theta.des)
##' # dout$random.design[,,1]
##' ## DZ
##' # dout$theta.des[,,nrow(pairs)]
##' # dout$random.design[,,nrow(pairs)]
##' #
##' # thetades <- dout$theta.des[,,1]
##' # x <- dout$random.design[,,1]
##' # x
##' ##EVaddGam(rep(1,6),x[1,],x[3,],thetades,matrix(1,18,6))
##'
##' # thetades <- dout$theta.des[,,nrow(out)/2]
##' # x <- dout$random.design[,,nrow(out)/2]
##' ##EVaddGam(rep(1,6),x[1,],x[4,],thetades,matrix(1,18,6))
##' @author Thomas Scheike
##' @export
##' @param theta theta
##' @param x1 x1
##' @param x2 x2
##' @param thetades thetades
##' @param ags ags
EVaddGam <- function(theta,x1,x2,thetades,ags)
{ #
	pars <- thetades %*% theta
	lamtot <- ags %*% theta

	mvar <- pars/lamtot
	vvar <- pars/lamtot^2

	x1mvar <- sum(x1 * mvar)
	x2mvar <- sum(x2 * mvar)
	x1x2vvar <- sum(x1*x2*vvar)

	list(x1m=x1mvar,mx2=x2mvar,
	     dN=x1x2vvar/x2mvar)
} #


##' @export
twostage.aalen <- function(object,...) survival.twostage(object,...)

##' @export
twostage.cox.aalen <- function(object,...) survival.twostage(object,...)

##' @export
twostage.coxph <- function(object,...) survival.twostage(object,...)

##' @export
twostage.phreg <- function(object,...) survival.twostage(object,...)

##' @export
folds<- function (n, folds = 10) 
{ ## {{{ 
   split(sample(1:n), rep(1:folds, length = n))
} ## }}} 

##' Split a data set and run function 
##'
##' @title Split a data set and run function 
##' @param func called function
##' @param data data-frame
##' @param size size of splits
##' @param splits number of splits (ignored if size is given)
##' @param id optional cluster variable
##' @param ... Additional arguments to lower level functions
##' @author Thomas Scheike, Klaus K. Holst
##' @export
##' @examples
##' ## avoid dependency on timereg
##' ## library(timereg)
##' ## data(TRACE)
##' ## res <- divide.conquer(prop.odds,TRACE,
##' ## 	     formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=200)
divide.conquer <- function(func=NULL,data,size,splits,id=NULL,...)
{ ## {{{
    nn <- nrow(data)
    if (!is.null(id)) {
        if (is.character(id) && length(id)==1) id <- data[,id]
        if (length(id)!=nn) stop("Wrong length of id variable")
        cc <- cluster.index(id)
        if (!missing(size)) splits <- round(cc$uniqueclust/size)
        splits <- min(splits,cc$uniqueclust)
        all.folds <- folds(cc$uniqueclust,splits)
        res <- lapply(all.folds, function(i) {
            idx <- na.omit(as.vector(cc$idclustmat[i,]+1))
            do.call(func, c(list(data=data[idx,]),...))
        })        
        return(res)
    }    
    splits <- round(nn/size)
    all.folds <- folds(nn,splits)
    res <- lapply(all.folds, function(i) 
	do.call(func, c(list(data=data[i,]),...)))
    res
} ## }}} 

##' Split a data set and run function of cox-aalen type and aggregate results 
##'
##' @title Split a data set and run function from timereg and aggregate
##' @param func called function
##' @param data data-frame
##' @param size size of splits
##' @param ... Additional arguments to lower level functions
##' @author Thomas Scheike, Klaus K. Holst
##' @export
##' @examples
##' ## library(timereg)
##' ## data(TRACE)
##' ## a <- divide.conquer.timereg(prop.odds,TRACE,
##' ##                            formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=200)
##' ## coef(a)
##' ## a2 <- divide.conquer.timereg(prop.odds,TRACE,
##' ##                              formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=500)
##' ## coef(a2)
##' ## 
##' ##if (interactive()) {
##' ##par(mfrow=c(1,1))
##' ##plot(a,xlim=c(0,8),ylim=c(0,0.01))
##' ##par(new=TRUE)
##' ##plot(a2,xlim=c(0,8),ylim=c(0,0.01))
##' ##}
divide.conquer.timereg <- function(func=NULL,data,size,...)
{ ## {{{ 

res <- divide.conquer(func=func,data,size,...)
K <- length(res)

gamma <- Reduce("+", lapply(res,function(x) x$gamma))/K
gammas <- do.call("cbind", lapply(res,function(x) x$gamma))
var.gamma <-    Reduce("+", lapply(res,function(x) x$var.gamma))/K^2
robvar.gamma <-    Reduce("+", lapply(res,function(x) x$robvar.gamma))/K^2 

times <- c(0,sort(unlist(lapply(res,function(x) x$cum[-1,1]))))
cum <- Reduce("+",lapply(res,function(x) cpred(x$cum,times)))/K 
var.cum <-    Reduce("+", lapply(res,function(x)  cpred(x$var.cum,times)))/K^2
robvar.cum <-    Reduce("+", lapply(res,function(x) cpred(x$robvar.cum,times)))/K^2
robvar.cum[,1] <- var.cum[,1] <- times

res <- list(gamma=gamma, var.gamma=var.gamma, robvar.gamma=robvar.gamma,
	    gammas=gammas, sdgammas=apply(gammas,1,sd),
	    cum=cum,var.cum=var.cum,robvar.cum=robvar.cum,
            res=res,prop.odds=TRUE,score=0,D2linv=diag(nrow(gamma)))
class(res) <- "cox.aalen"

return(res)
} ## }}} 

library(mets)

set.seed(100)
n <- 40000

## {{{ competing risks ace model with profile of baseline 
lam0 <- c(0.3,0.2)
pars <- c(1,1,1,1,0.1,1)*0.25
## genetic random effects, cause1, cause2 and overall 
parg <- pars[c(1,3,5)]
## environmental random effects, cause1, cause2 and overall 
parc <- pars[c(2,4,6)]

## simulate competing risks with two causes with hazards 0.5 and 0.3
## ace for each cause, and overall ace 
out <- simCompete.twin.ace(n,parg,parc,0,2,
			   lam0=lam0,overall=1,all.sum=1)

## setting up design for running the model 
## {{{ setting pairs and random effects 
# 
mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

dout <- make.pairwise.design.competing(pairs,kinship,
	       type="ace",compete=length(lam0),overall=1)
head(dout$ant.rvs)
## MZ
dim(dout$theta.des)
dout$theta.des[,,1]
dout$random.design[,,1]
dout$theta.des[,,nrow(out)/2]
dout$random.design[,,nrow(out)/2]
###table(out$status)
## }}} 

## competing risks models, given as list 
cr.models=list(Surv(time,status==1)~+1,
	       Surv(time,status==2)~+1)
ms <- out$time %o% lam0

par(mfrow=c(1,1))
tsf <- twostage(NULL,data=out,clusters=out$cluster,
               theta=0.01+pars/sum(pars)^2,
	       var.link=0,step=1.0,Nit=20,detail=1,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       numDeriv=0,
               marginal.status=out$status,
               marginal.survival=ms,
	       two.stage=0,cr.models=cr.models)
coef(tsf)
pars/sum(pars)^2
summary(tsf$marginal.trunc)
summary(tsf$marginal.surv)
###tsf$score; tsf$score1

###
source("../R/twostage.R")
par(mfrow=c(1,1))
ts <- twostage(NULL,data=out,clusters=out$cluster,
               theta=tsf$theta,
	       step=1.0,Nit=20,detail=1,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       numDeriv=0,
               marginal.status=out$status,
	       two.stage=0,cr.models=cr.models)

coef(ts)
pars/sum(pars)^2

matplot.twostage(ts)
abline(c(0,lam0[1])); abline(c(0,lam0[2])); 

system.time(
out1 <- aalen(cr.models[[1]],data=out,robust=0)
)
system.time(
out1 <- aalen(Surv(time,status!=0)~+1,data=out,robust=0)
)


## }}} 

onec <- 1
if (onec==1) {
## {{{ one cause ACE survival
# 

lam0 <- c(0.5)
pars <- c(1,0.5); 
pars <- c(0.5,1); 
out <- simCompete.twin.ace(n,pars[1],pars[2],0,2,lam0=lam0,overall=0)

## {{{ 
mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

dout <- make.pairwise.design(pairs,kinship,type="ace")
head(dout$ant.rvs)
## MZ
dim(dout$theta.des)
dout$theta.des[,,1]
dout$random.design[,,1]
## DZ
dout$theta.des[,,nrow(out)/2]
dout$random.design[,,nrow(out)/2]
## }}} 

lams <- cbind(lam0[1]*out$time)
table(out$status)

ts <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars/sum(pars)^2,
	       var.link=0,step=1.0,Nit=10,detail=0,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       numDeriv=0,
               marginal.surv=lams,
	       marginal.status=out$status,
               two.stage=0)
summary(ts)

ts2 <- twostage(NULL,data=out,clusters=out$cluster,
               theta=ts$theta,
	       var.link=0,step=1.0,Nit=10,detail=1,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       numDeriv=0,
	       marginal.status=out$status,
	       cr.model=list(Surv(time,status)~+1),
               two.stage=0)
summary(ts2)
ts2$score

## }}} 
}

twoc <- 1
if (twoc==1) {
## {{{ two cause two independent ACE survival
# 

lam0 <- c(0.5,0.4)
pars <- c(0.5,1,0.5,1)*0.5; 
out <- simCompete.twin.ace(n,pars[c(1,3)],pars[c(2,4)],0,2,lam0=lam0,overall=0)
table(out$status)
out$status1 <- out$status==1
table(out$status1)

## {{{ 
mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

###kinship <- c(rep(1,5000),rep(0.5,5000))
###
###source("mets/R/twostage.R")
###dout <- make.pairwise.design(pairs,kinship,type="ace")
dout <- make.pairwise.design.competing(pairs,kinship,
	       type="ace",overall=0,compete=2)
head(dout$ant.rvs)
## MZ
dim(dout$theta.des)
dout$theta.des[,,1]
dout$random.design[,,1]
dout$theta.des[,,nrow(out)/2]
dout$random.design[,,nrow(out)/2]
###
###
###out$status[out$status==3] <- 2
# 
table(out$status)
## }}} 

###lams <- cbind(lam0[1]*out$time,lam0[2]*out$time)
lams <- cbind(out$time*lam0[1],out$time*lam0[2])

ts <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars/sum(pars)^2,
	       var.link=0,step=1.0,Nit=10,detail=1,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
               marginal.surv=lams,
	       marginal.status=out$status,
               two.stage=0)
summary(ts)
ts$score

ts2 <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars/sum(pars)^2,
	       var.link=0,step=1.0,Nit=20,detail=1,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       marginal.status=out$status,
               two.stage=0,
     cr.models=list( Surv(time,status==1)~+1,
		     Surv(time,status==2)~+1)
	       )
ts2$theta
pars/sum(pars)^2

matplot.twostage(ts2)
abline(c(0,lam0[1])); abline(c(0,lam0[2])); 

} ## }}} 


itwoc <- 1
if (itwoc==1) {
## {{{ cause specific analyses because independence 

lam0 <- c(0.5,0.4)
pars <- c(1,0.5); 
pars <- c(0.5,1,0.5,1); 
out <- simCompete.twin.ace(n,
	   pars[c(1,3)],pars[c(2,4)],
           0,2,lam0=lam0,overall=0,all.sum=1)
table(out$status)

## {{{ 
mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

###kinship <- c(rep(1,5000),rep(0.5,5000))
###
###source("mets/R/twostage.R")
###dout <- make.pairwise.design(pairs,kinship,type="ace")
dout <- make.pairwise.design.competing(pairs,kinship,
	       type="ace",overall=0)
head(dout$ant.rvs)
## MZ
dim(dout$theta.des)
dout$theta.des[,,1]
dout$random.design[,,1]
## DZ
dout$theta.des[,,nrow(out)/2]
dout$random.design[,,nrow(out)/2]
###
###
# 
table(out$status)
out$statusc1 <- 1*(out$status==1)
out$statusc2 <- 1*(out$status==2)
## }}} 


## design for only cause 1
dout1 <- make.pairwise.design(pairs,kinship,
	       type="ace")

## competing risks models, given as list 
cr.models=list(Surv(time,status==1)~+1,Surv(time,status==2)~+1)

ts <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars,
	       score.method="fisher.scoring",
	       var.link=0,step=1.0,Nit=10,detail=0,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
	       numDeriv=0,
               marginal.status=out$status,
       two.stage=0,cr.models=cr.models)
coef(ts)


### considering cause 1 alone 

cr.models1=list(Surv(time,statusc1==1)~+1)

## note due to parametrization ags=2 ! 
ts1 <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars[1:2],
	       var.link=0,step=1.0,Nit=10,detail=0,
               random.design=dout1$random.design,
               theta.des=dout1$theta.des,
	       pairs=pairs,
	       marginal.status=out$statusc1,
               two.stage=0,
	       cr.models=cr.models2)
summary(ts1)

### considering cause 2 alone 

cr.models2=list(Surv(time,statusc2==1)~+1)

ts2 <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars[1:2],
	       var.link=0,step=1.0,Nit=10,detail=0,
               random.design=dout1$random.design,
               theta.des=dout1$theta.des,pairs=pairs,
	       marginal.status=out$statusc2,
               two.stage=0,
	       cr.models=cr.models2)
summary(ts2)

## }}} 
}


## {{{ 2 compete+ overall ace, dependence test, cox 
### conditional cox model 
lam0 <- c(0.5,0.5)
pars <- rep(1,6)
out <- simCompete.twin.ace(n,c(1,1,1),c(1,1,1),0,2,
		  lam0=lam0,overall=1,all.sum=1)

table(out$status)
###
out2 <- fast.reshape(out,id="cluster")
###
out2$mintime <- pmin(out2$time1,out2$time2)
out2$whichmin <- ifelse(out2$time1<out2$time2,1,2)
out2$mincause <- ifelse(out2$time1<out2$time2,out2$status1,
			out2$status2)
out2$maxtime <- pmax(out2$time1,out2$time2)
out2$maxcause <- ifelse(out2$time1>out2$time2,out2$status1,
			out2$status2)
###
out1 <- event.split(out2,time="time1",status="status1",
	    cuts="time2")
out1$mstat <- out1$status2*out1$num
head(out1[out1$whichmin==2,])
head(out1[out1$whichmin==1,])
table(out1$status1)
table(out1$mstat)
###out1[out1$num==1,]
out1$mincause[out1$num==0] <- 0
###
coxph(Surv(start,time1,status1==1)~factor(mstat)*factor(zyg1),data=out1)
###
coxph(Surv(start,time1,status1==2)~factor(mstat)*factor(zyg1),data=out1)

mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

dout <- make.pairwise.design.competing(pairs,kinship,
	       type="ace",compete=length(lam0),overall=1)
head(dout$ant.rvs)
## MZ
dim(dout$theta.des)
dout$theta.des[,,1]
dout$random.design[,,1]
## DZ
dout$theta.des[,,nrow(out)/2]
dout$random.design[,,nrow(out)/2]
###
###

cr.models=list(Surv(time,status==1)~+1,Surv(time,status==2)~+1)

ts <- twostage(NULL,data=out,clusters=out$cluster,
               theta=pars,
	       score.method="fisher.scoring",
	       var.link=0,step=1.0,Nit=20,detail=0,
               random.design=dout$random.design,
               theta.des=dout$theta.des,pairs=pairs,
               marginal.status=out$status,
	       two.stage=0,cr.models=cr.models)

## }}} 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)

## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z
 
out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~X)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X,cens.model=~+1)

## -----------------------------------------------------------------------------
###library(speff2trial) 
library(mets)
data(ACTG175)
###
data <- ACTG175[ACTG175$arms==0 | ACTG175$arms==1, ]
data <- na.omit(data[,c("days","cens","arms","strat","cd40","cd80","age")])
data$days <- data$days+runif(nrow(data))*0.01
dfactor(data) <- arms.f~arms
notrun <- 1

if (notrun==0) { 
fit1 <- speffSurv(Surv(days,cens)~cd40+cd80+age,data=data,trt.id="arms",fixed=TRUE)
summary(fit1)
}
# 
# Treatment effect
#             Log HR       SE   LowerCI   UpperCI           p
# Prop Haz  -0.70375  0.12352  -0.94584  -0.46165  1.2162e-08
# Speff     -0.72430  0.12051  -0.96050  -0.48810  1.8533e-09

out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~cd40+cd80+age,augmentC=~cd40+cd80+age)
summary(out)

## -----------------------------------------------------------------------------
dtable(data,~strat+arms)
dfactor(data) <- strat.f~strat
out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~strat.f)
summary(out)

## -----------------------------------------------------------------------------
data(calgb8923)
calgt <- calgb8923

## tm <- At.f~factor(Count2)+age+sex+wbc
## tm <- At.f~factor(Count2)
tm <-  At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=1:10)
summary(pll0,times=1:10)

## -----------------------------------------------------------------------------
library(mets)
data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1

## making time-dependent indicators of going to B1/B2
calgt$A10t <- calgt$A11t <- 0
calgt <- dtransform(calgt,A10t=1,A1==0 & Count2==1)
calgt <- dtransform(calgt,A11t=1,A1==1 & Count2==1)
calgt0 <- subset(calgt,A0==0)

ss0 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==0),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss0)

ss1 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==1),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss1)

## -----------------------------------------------------------------------------
ssf <- phreg_rct(Event(start,time,status)~A0.f+A10t+A11t+cluster(id),
	 data=calgt,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR0=~age+wbc+sex,augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)

## -----------------------------------------------------------------------------
n <- 1000
beta <- 0.15; 
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- scalecumhaz(CPH_HPN_CRBSI$mechanical,0.5)

cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
ce <- 3; betao1 <- 0

varz <- 1; dep=4; X <- z <- rgamma(n,1/varz)*varz
Z0 <- NULL
px <- 0.5
if (betao1!=0) px <- lava::expit(betao1*X)      
A0 <- rbinom(n,1,px)
r1 <- exp(A0*beta[1])
rd <- exp( A0 * 0.15)
rc <- exp( A0 * 0 )
###
rr <-    mets:::simLUCox(n,base1,death.cumhaz=dr,r1=r1,Z0=X,dependence=dep,var.z=varz,cens=ce/5000)
rr$A0 <- A0[rr$id]
rr$z1 <- attr(rr,"z")[rr$id]
rr$lz1 <- log(rr$z1)
rr$X <- rr$lz1 
rr$lX <- rr$z1
rr$statusD <- rr$status
rr <- dtransform(rr,statusD=2,death==1)
rr <- count.history(rr)
rr$Z <- rr$A0
data <- rr
data$Z.f <- as.factor(data$Z)
data$treattime <- 0
data <- dtransform(data,treattime=1,lbnr__id==1)
dlist(data,start+stop+statusD+A0+z1+treattime+Count1~id|id %in% c(4,5))

## -----------------------------------------------------------------------------
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 treat.var="treattime",typesR=c("non","R0"),typesC=c("non","C","dynC"),
	 augmentR0=~z1,augmentC=~z1+Count1)
summary(fit2)

## -----------------------------------------------------------------------------
n <- 500
beta=c(0.3,0.3);betatr=0.3;betac=0;betao=0;betao1=0;ce=3;fixed=1;sim=1;dep=4;varz=1;ztr=0; ce <- 3
## take possible frailty 
Z0 <- rgamma(n,1/varz)*varz
px0 <- 0.5; if (betao!=0) px0 <- expit(betao*Z0)
A0 <- rbinom(n,1,px0)
r1 <- exp(A0*beta[1])
#
px1 <- 0.5; if (betao1!=0) px1 <- expit(betao1*Z0)
A1 <- rbinom(n,1,px1)
r2 <- exp(A1*beta[2])
rtr <- exp(A0*betatr[1])
rr <-  mets:::simLUCox(n,base1,death.cumhaz=dr,cumhaz2=base1,rtr=rtr,betatr=0.3,A0=A0,Z0=Z0,
		r1=r1,r2=r2,dependence=dep,var.z=varz,cens=ce/5000,ztr=ztr)
rr$z1 <- attr(rr,"z")[rr$id]
rr$A1 <- A1[rr$id]
rr$A0 <- A0[rr$id]
rr$lz1 <- log(rr$z1)
rr <- count.history(rr,types=1:2)
rr$A1t <- 0
rr <- dtransform(rr,A1t=A1,Count2==1) 
rr$At.f <- rr$A0
rr$A0.f <- factor(rr$A0)
rr$A1.f <- factor(rr$A1)
rr <- dtransform(rr, At.f = A1, Count2 == 1)
rr$At.f <- factor(rr$At.f)
dfactor(rr)  <-  A0.f~A0
rr$treattime <- 0
rr <- dtransform(rr,treattime=1,lbnr__id==1)
rr$lagCount2 <- dlag(rr$Count2)
rr <- dtransform(rr,treattime=1,Count2==1 & (Count2!=lagCount2))
dlist(rr,start+stop+statusD+A0+A1+A1t+At.f+Count2+z1+treattime+Count1~id|id %in% c(5,10))

## -----------------------------------------------------------------------------
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),treat.var="treattime",
	 treat.model=At.f~factor(Count2),
	 augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)

## -----------------------------------------------------------------------------
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 typesR=c("non","R0"),typesC=c("non","C","dynC"),
         RCT=FALSE,treat.model=Z.f~z1,augmentR0=~z1,augmentC=~z1+Count1,
	 treat.var="treattime")
summary(fit2)

## -----------------------------------------------------------------------------
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
         treat.var="treattime",
	 RCT=FALSE,treat.model=At.f~z1*factor(Count2),
         augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)

## -----------------------------------------------------------------------------
sessionInfo()

##' Double CIF Fine-Gray model with two causes
##'
##' Estimation based on derived hazards and recursive estimating equations.
##' fits two parametrizations
##' 1)
##'  \deqn{
##' F_1(t,X) = 1 - \exp( - \exp( X^T \beta ) \Lambda_1(t))
##' }
##' and
##'  \deqn{
##' F_2(t,X_2) = 1 - \exp( -  \exp( X_2^T \beta_2 ) \Lambda_2(t))
##' }
##' or restricted version
##' 2)
##'  \deqn{
##' F_1(t,X) = 1 - \exp( -  \exp( X^T \beta ) \Lambda_1(t))
##' }
##' and
##'  \deqn{
##' F_2(t,X_2,X) = ( 1 - \exp(  - \exp( X_2^T \beta_2 ) \Lambda_2(t)) ) (1 - F_1(\infty,X))
##' }
##'
##' @param formula formula with 'Event'
##' @param data data frame
##' @param offset offsets for cox model
##' @param weights weights for Cox score equations
##' @param X2 specifies the regression design for second CIF model
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' res <- 0
##' data(bmt)
##' bmt$age2 <- bmt$age
##' newdata <- bmt[1:19,]
##' if (interactive()) par(mfrow=c(5,3))
##'
##' ## same X1 and X2
##' pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,restrict=res)
##' if (interactive()) {
##'   bplotdFG(pr2,cause=1)
##'   bplotdFG(pr2,cause=2,add=TRUE)
##' }
##' pp21 <- predictdFG(pr2,newdata=newdata)
##' pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##' pp21 <- predictdFG(pr2)
##' pp22 <- predictdFG(pr2,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##'
##' pr2 <- doubleFGR(Event(time,cause)~strata(platelet),data=bmt,restrict=res)
##' if (interactive()) {
##'   bplotdFG(pr2,cause=1)
##'   bplotdFG(pr2,cause=2,add=TRUE)
##' }
##' pp21 <- predictdFG(pr2,newdata=newdata)
##' pp22 <- predictdFG(pr2,,newdata=newdata,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##' pp21 <- predictdFG(pr2)
##' pp22 <- predictdFG(pr2,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##'
##' ## different X1 and X2
##' pr2 <- doubleFGR(Event(time,cause)~age+platelet+age2,data=bmt,X2=3,restrict=res)
##' if (interactive()) {
##'   bplotdFG(pr2,cause=1)
##'   bplotdFG(pr2,cause=2,add=TRUE)
##' }
##' pp21 <- predictdFG(pr2,newdata=newdata)
##' pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##' pp21 <- predictdFG(pr2)
##' pp22 <- predictdFG(pr2,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##'
##' ### uden X1
##' pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,X2=1:2,restrict=res)
##' if (interactive()) {
##'   bplotdFG(pr2,cause=1)
##'   bplotdFG(pr2,cause=2,add=TRUE)
##' }
##' pp21 <- predictdFG(pr2,newdata=newdata)
##' pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##' pp21 <- predictdFG(pr2)
##' p22 <- predictdFG(pr2,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##'
##' ### without X2
##' pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,X2=0,restrict=res)
##' if (interactive()) {
##'   bplotdFG(pr2,cause=1)
##'   bplotdFG(pr2,cause=2,add=TRUE)
##' }
##' pp21 <- predictdFG(pr2,newdata=newdata)
##' pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##' pp21 <- predictdFG(pr2)
##' pp22 <- predictdFG(pr2,cause=2)
##' if (interactive()) {
##'   plot(pp21)
##'   plot(pp22,add=TRUE,col=2)
##' }
##'
##' @aliases bplotdFG predictdFG
##' @export
doubleFGR <- function(formula,data,offset=NULL,weights=NULL,X2=NULL,...) {# {{{
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}
  X <- model.matrix(Terms, m)
  if (!is.null(intpos  <- attributes(Terms)$intercept))
    X <- X[,-intpos,drop=FALSE]
  X2call <- X2;
  if (!is.null(X2)) {
	 if (X2[1]==0)  X2 <- matrix(nrow=0,ncol=0)
	 else { X2 <-  X[,X2call,drop=FALSE];
                X <- X[,-X2call,drop=FALSE];
	 }
  } else X2 <- X
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
  if (ncol(X2)==0) X2 <- matrix(nrow=0,ncol=0)
  res <- c(doubleFG01R(X,X2,entry,exit,status,id,strata,offset,weights,strata.name,X2call=X2call,...),
   list(call=cl,model.frame=m,formula=formula,strata.pos=pos.strata,cluster.pos=pos.cluster))
  class(res) <- c("doubleFG")

  res
}# }}}

doubleFG01R <- function(X,X2, entry,exit,status,id=NULL,strata=NULL,offset=NULL,weights=NULL,
             strata.name=NULL,cumhaz=TRUE,
             beta,stderr=TRUE,method="NR",no.opt=FALSE,Z=NULL,propodds=NULL,AddGam=NULL,
	     restrict=0,case.weights=NULL,X2call=NULL,...) {# {{{
  p1 <- ncol(X);
  p2 <- ncol(X2)
  p <- p1+p2
  if (missing(beta))  beta <- rep(0,p)
  if (p1==0) X <- cbind(rep(0,length(exit)))
  if (p2==0) X2 <- cbind(rep(0,length(exit)))
  if (is.null(strata)) { strata <- rep(0,length(exit)); nstrata <- 1; strata.level <- NULL; } else {
	  strata.level <- levels(strata)
	  ustrata <- sort(unique(strata))
	  nstrata <- length(ustrata)
	  strata.values <- ustrata
      if (is.numeric(strata)) strata <-  fast.approx(ustrata,strata)-1 else  {
      strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
    }
  }
  if (is.null(offset)) offset <- rep(0,length(exit))
  if (is.null(weights)) weights <- rep(1,length(exit))
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given
  if (!is.null(Z)) Zcall <- Z

  ## possible casewights to use for bootstrapping and other things
  if (is.null(case.weights)) case.weights <- rep(1,length(exit))

  trunc <- (!is.null(entry))
  if (!trunc) entry <- rep(0,length(exit))

  if (!is.null(id)) {
	  ids <- unique(id)
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
   } else id <- as.integer(seq_along(entry))-1;
   ## orginal id coding into integers
   id.orig <- id+1;

   dd <- .Call("FastCoxPrepStrata",entry,exit,status,X,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")
   if (!is.null(X2call))  {
   dd2 <- .Call("FastCoxPrepStrata",entry,exit,status,X2,id,trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets")
   dd$X2 <- dd2$X
   dd$XX2 <- dd2$XX
   } else {
      dd$X2 <- dd$X
      dd$XX2 <- dd$XX
   }
   if (p1==0)  dd$X <- matrix(nrow=0,ncol=0)
   if (p2==0)  dd$X2 <- matrix(nrow=0,ncol=0)

   cause <- status[dd$ord+1]
   dd$cause <- cause
   dd$nstrata <- nstrata

	obj <- function(pp,U=FALSE,all=FALSE) {# {{{
	  val <- with(dd, doubleFGstrataR(pp,X,XX,X2,XX2,sign,cause,jumps,strata,nstrata,weights,offset,ZX,caseweights,restrict))

	  if (all) {
	      val$time <- dd$time
	      val$cause <- dd$cause
	      val$ord <- dd$ord+1
	      val$jumps <- dd$jumps+1
	      val$jumptimes <- val$time[val$jumps]
	      val$weightsJ <- dd$weights[val$jumps]
	      val$case.weights <- dd$case.weights[val$jumps]
	      val$strata.jumps <- val$strata[val$jumps]
	      val$nevent <- length(val$S0)
	      val$nstrata <- dd$nstrata
	      val$strata <- dd$strata
	      return(val)
	  }
	 with(val,structure(-ploglik,gradient=-gradient,hessian=-hessian))
	}# }}}

  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(beta,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;
      names(cc) <- c(colnames(X),colnames(X2))
      if (!stderr) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(rep(0,2),all=TRUE)
  }

  se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL
  II <- NULL
  ### computes Breslow estimator
  if (cumhaz==TRUE) { # {{{
	 if (no.opt==FALSE & p!=0) {
               II <- - tryCatch(solve(val$hessian),error=
	              function(e) matrix(0,nrow(val$hessian),ncol(val$hessian)) )
	 } else II <- matrix(0,p,p)
	 strata <- val$strata[val$jumps]
	 nstrata <- val$nstrata
	 jumptimes <- val$jumptimes

	 ## Brewslow estimator
	 cumhaz <- cbind(jumptimes,val$base12$base)
	 se.cumhaz <- cumhaz
 } # }}}
 else {cumhaz <- se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL}

 II <- - tryCatch(solve(val$hessian),error=
        function(e) matrix(0,nrow(val$hessian),ncol(val$hessian)) )

  res <- c(val,
           list(cox.prep=dd,
		strata.call=strata.call, strata.level=strata.level,
                entry=entry,
                exit=exit,
                status=status,
                p=p,
                X=X,
                X2=X2,
		X2call=X2call,
		offsets=offset,
		weights=weights,
                id=id.orig,
		opt=opt,
		var=II,
		cumhaz=cumhaz, se.cumhaz=se.cumhaz,
		lcumhaz=lcumhaz, lse.cumhaz=lse.cumhaz,
		II=II,
		strata.name=strata.name,propodds=propodds,restrict=restrict))
  class(res) <- "doubleFG"
  res
}# }}}

## double FG in in R version
doubleFGstrataR <- function(beta, X, XX, X2, XX2, Sign, cause, Jumps, strata, nstrata, weights, offsets, ZX, caseweights,restrict)
{# {{{
	p=length(beta)
	p1  <-  ncol(X)
	p2 <- ncol(X2)
	if (nrow(X)==0) X <- matrix(1,length(strata),1)
	if (nrow(X2)==0) X2 <- matrix(1,length(strata),1)
	strata=c(strata)
	if (p1==0) beta1 <- 0 else beta1 <- beta[1:p1]
	if (p2==0) beta2 <- 0 else beta2 <- beta[(p1+1):p]
	Xb1 = c(X %*% beta1+offsets)
	Xb2 = c(X2 %*% beta2+offsets)
	eXb1 = c(exp(Xb1)*weights);
	eXb2 = c(exp(Xb2)*weights);
	if (nrow((Sign))==length(eXb1)) { ## Truncation
		eXb1 = c(Sign)*eXb1;
		eXb2 = c(Sign)*eXb2;
	}

	S01 = c(revcumsumstrata(eXb1,strata,nstrata))
	S02 = c(revcumsumstrata(eXb2,strata,nstrata))
	E1=apply(eXb1*as.matrix(X),2,revcumsumstrata,strata,nstrata)/S01;
	E2=apply(eXb2*as.matrix(X2),2,revcumsumstrata,strata,nstrata)/S02;
	Jumps=Jumps+1
	causeJ <- cause[Jumps]
	Jumps1 <- Jumps[causeJ==1]
	Jumps2 <- Jumps[causeJ==2]

	## both E's, SO's, X same
	S0J <- cbind(S01,S02)[Jumps,]
	eXbJ <- cbind(eXb1,eXb2)[Jumps,]
	EJ <- cbind(E1,E2)[Jumps,]
	XJ <- X[Jumps,]
	X2J <- X2[Jumps,]

	E1 = E1[Jumps1,,drop=FALSE];
	E2 = E2[Jumps2,,drop=FALSE];
        E21=.Call("vecMatMat",E1,E1)$vXZ;
        E22=.Call("vecMatMat",E2,E2)$vXZ;

	XX21=apply(XX*eXb1,2,revcumsumstrata,strata,nstrata)/S01;
	XX22=apply(XX2*eXb2,2,revcumsumstrata,strata,nstrata)/S02;
	XX21 = XX21[Jumps1,,drop=FALSE];
	XX22 = XX22[Jumps2,,drop=FALSE];
        ## back to full dimension
        XX21 <-  .Call("XXMatFULL",XX21,ncol(E1),PACKAGE="mets")$XXf
        XX22 <-  .Call("XXMatFULL",XX22,ncol(E2),PACKAGE="mets")$XXf

	weightsJ=weights[Jumps];
	caseweightsJ=caseweights[Jumps];

	## compute recursive weights given beta1, beta2
        base12 <- .Call("cumsumstrataDFGR",weightsJ,S0J,causeJ,strata[Jumps],nstrata,eXbJ)
	if (restrict>0) {
		for (i in 1:restrict) {
		   Lam1inf <- tailstrata(strata[Jumps],nstrata);
		   Lam1inf  <- base12$base[Lam1inf,1]
		   base12 <- .Call("cumsumstrataDFGRestrictR",weightsJ,S0J,causeJ,strata[Jumps],nstrata,eXbJ,Lam1inf);
	        }
	}

	grad1 = (X[Jumps1,,drop=FALSE]-E1);         ## Score
	grad2 = (X2[Jumps2,,drop=FALSE]-E2);        ## Score

	### weights for Fg1 an Fg2 models
	pow1 <- base12$pow1[causeJ==1]
	pow2 <- base12$pow2[causeJ==2]
	cw1 <- (caseweightsJ*weightsJ)[causeJ==1]
	cw2 <- (caseweightsJ*weightsJ)[causeJ==2]
	S012 = S01[Jumps1]/(pow1*cw1);        ## S0 with weights
	S022 = S02[Jumps2]/(pow2*cw2);        ## S0 with weights

	val =  sum(pow1*cw1*(Xb1[Jumps1]-log(S012)))+
	       sum(pow2*cw2*(Xb2[Jumps2]-log(S022))); ## Partial log-likelihood

	grad21= grad1*(pow1*cw1);                   ## score  with weights
	grad22= grad2*(pow2*cw2);                   ## score  with weights
	grad <- c(apply(grad21,2,sum),apply(grad22,2,sum))
	gradient <- grad
	if (p1==0) gradient <- grad[-1]
	if (p2==0) gradient  <-  grad[1:p1]


	## no weights
	val2 = val;                          ## Partial log-likelihood with weights

 	hesst1 = -(XX21-E21);               ## hessian contributions in jump times
	hesst2 = -(XX22-E22);               ## hessian contributions in jump times
	hesst12 = hesst1*(cw1)*pow1;        ## hessian over time with weights
	hesst22 = hesst2*(cw2)*pow2;        ## hessian over time with weights

	## missing some derivative terms for hessian (due to pow=w(beta,\Lam(beta,t-))

	## setup hessian matrix
	hess12 = matrix(apply(hesst12,2,sum),p1,p1);         ## hessian with weights
	hess22 = matrix(apply(hesst22,2,sum),p2,p2);         ## hessian with weights
	hess2 <- matrix(0,p1+p2,p1+p2)
	pd2 <- p1
	p <- length(beta)
	if (p1>0) hess2[1:p1,1:p1] <- hess12
	if (p2>0) hess2[(p1+1):p,(p1+1):p] <- hess22

	out=list(jumps=Jumps, ploglik=sum(val2),U=grad2,base12=base12,
		 gradient=matrix(gradient,1,p), hessian=hess2,
		 ##hessianttime=hesst2, S2S0=XX2,
		 E=EJ, S0=S0J
		 )
	return(out)
}# }}}

##' @export
coef.doubleFG <- function(object,...) object$coef

##' @export
vcov.doubleFG <- function(object,...) object$var

##' @export
summary.doubleFG <- function(object,...) estimate(object)

##' @export
print.doubleFG <- function(x,...) estimate(x)

##' @export
bplotdFG <- function(x,cause=1,...)  {# {{{
	x$cumhaz <- x$cumhaz[,c(1,cause+1)]
	basehazplot.phreg(x,...)
}# }}}

##' @export
predictdFG <- function(x,cause=1,se=FALSE,times=NULL,...)  {# {{{
	cumhaz <- x$cumhaz
	coef <- x$coef
	p <- x$p
        X2call <- x$X2call;

	### finding the right coefficients
	if (p>0) {
              if (!is.null(X2call))  {
		 if (X2call[1]==0)  {
			 p2 <- 0; p1 <- p;
		         cc1 <- 1:p; cc2 <- 0
		 } else {
	             p2 <- length(X2call);
	             p1 <- p-p2;
		     cc1 <- (1:p)[-X2call]
		     cc2 <- (1:p)[X2call]
		     if (p2==0) x$X <- x$X
		     if (p1==0)  x$X <- x$X2
		     if (p1!=0 & p2!=0) x$X <- cbind(x$X,x$X2)
		 }
         } else {
	         model.frame2 <- x$model.frame
		 p1 <- p/2
		 cc1 <- 1:p1
		 cc2 <- (p1+1):p
		 p <- p1
	 }
	if (cause==1) {
	      if (is.null(X2call)) x$coef <- x$coef[cc1]
	      else x$coef[cc2] <- 0
	}
	if (cause==2) {
	      if (is.null(X2call)) x$coef <- x$coef[cc2]
	      else x$coef[cc1] <- 0
	}
	}

	x$cumhaz <- x$cumhaz[,c(1,cause+1)]
	class(x) <- c("phreg","cifreg")
	pll <- predict(x,se=se,times=times,...)

	if (x$restrict>0 & cause==2) {
		x$cumhaz <- cumhaz[,1:2]
                if (is.null(X2call)) x$coef <- coef[cc1] else { x$coef <- coef;  x$coef[cc2] <- 0; }
		times <- max(cumhaz[,1])
		mm <- "individual.time" %in% names(list(...))
		if (mm) times <- rep(times,nrow(x$model.frame))
		p1ll <- predict(x,se=se,times=times,...)
		cif1 <- c(1-p1ll$cif)
		pll$cif <- pll$cif*cif1
	}
	return(pll)
}# }}}

##' Risk predictions to work with riskRegression package
##'
##' @title Risk predictions to work with riskRegression package
##' @param object phreg/binreg/cifreg object
##' @param ... additional arguments to lower level functions
##' @author Thomas Scheike
##' @aliases predictRisk 
##' @export
predictRisk <- function(object, ...) UseMethod("predictRisk")


##' @export
predictRisk.phreg <- function(object,newdata,times=NULL,...)
{# {{{
	pcif <- predict(object,newdata,times=times,se=0)
	return(c(1-pcif$surv))
}# }}}

##' @title Risk predictions to work with riskRegression package
##' @inheritParams predictRisk
##' @param newdata data.frame on which to make new predictions
##' @param times times for predictions
##' @param cause cause (cif) to predict
##' @aliases predictRisk.phreg predictRisk.binreg predictRisk.cifreg
##'   predictRisk.cifregFG predictRisk.recreg
##' @export
predictRisk.binreg <- function(object,newdata,cause,times=NULL,...)
{# {{{
	pcif <- predict(object,newdata,se=0)
	return(c(pcif))
}# }}}

##' @export
predictRisk.cifreg <- function(object,newdata,cause,times=NULL,...)
{# {{{b
	pcif <- predict(object,newdata,times=times,se=0)$cif
	return(c(pcif))
}# }}}

##' @export
predictRisk.cifregFG <- function(object,newdata,cause,times=NULL,...)
{# {{{
	pcif <- predict(object,newdata,times=times,se=0)$cif
	return(c(pcif))
}# }}}

##' @export
predictRisk.recreg <- function(object,newdata,cause,times=NULL,...)
{# {{{
	pcif <- predict(object,newdata,times=times,se=0)$cumhaz
	return(c(pcif))
}# }}}

#
# BMI data on twin pairs
#

library(mets)
data(twinbmi)

str(twinbmi)
head(twinbmi)

# restrict to data, where response is not missing
twinbmi <- twinbmi[!is.na(twinbmi$bmi),]

# install.packages("lattice")
library(lattice)
plot( histogram( ~ bmi| gender, type="density", col="red", xlab="kg/m^2", 
				main="Histogram of BMI", data=twinbmi) )

# BMI is often studied on log-scale.
# boxcox(bmi ~ age*gender, data = twinbmi)
twinbmi$logbmi <- log(twinbmi$bmi)

## Saturated model
a <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
            type="sat",control=list(refit=TRUE))
mean(score(a)^2)

aa <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
             type="sat",control=list(method="NR",start=coef(a)))
mean(score(aa)^2)
mean((coef(a)-coef(aa))^2)

## Ace model
ace <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ace")
mean(score(ace)^2) ## Convergence?

#
lnbmi.flex <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="flex")
lnbmi.flex$estimate$opt$message
mean(score(lnbmi.flex)^2)

compare(a,lnbmi.flex)

#
lnbmi.u <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="u")
lnbmi.u$estimate$opt$message
lnbmi.u

cl <- lnbmi.u$call
cl$control <- list(method="NR",start=coef(lnbmi.u))
aa <- eval(cl)

compare(lnbmi.u,lnbmi.flex)

#
lnbmi.ace <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ace")
mean(score(lnbmi.ace)^2) 
lnbmi.ace$estimate$opt

lnbmi.ace$estimate$opt$message
lnbmi.ace   

#
lnbmi.ade <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ade")
lnbmi.ade$estimate$opt

AIC(lnbmi.ace,lnbmi.ade)

#
lnbmi.ae <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ae",control=list(method="NR"))
lnbmi.ae$estimate$opt$message
lnbmi.ae  

compare(lnbmi.ace,lnbmi.ae)

#CE
lnbmi.ce <- twinlm(logbmi~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ce",control=list(method="NR"))
lnbmi.ce$estimate$opt$message
lnbmi.ce  

AIC(lnbmi.ace,lnbmi.ce)



twinbmi$y <- twinbmi$bmi>25
lnbmi.ae <- twinlm(y~age*gender, id="tvparnr", DZ="DZ", zyg="zyg",data=twinbmi,
			type="ace",control=list(trace=1))

# GOF-Table?
# mx and openmx for same data
# reshape wide -  twin-twin plot.

##' Sort data according to columns in data frame
##'
##' @title Sort data frame
##' @param data Data frame
##' @param x variable to order by
##' @param ... additional variables to order by
##' @param decreasing sort order (vector of length x)
##' @param return.order return order 
##' @return data.frame
##' @export
##' @examples
##' data(data="hubble",package="lava")
##' dsort(hubble, "sigma")
##' dsort(hubble, hubble$sigma,"v")
##' dsort(hubble,~sigma+v)
##' dsort(hubble,~sigma-v)
##'
##' ## with direct asignment
##' dsort(hubble) <- ~sigma-v
##' @aliases dsort2 dsort dsort<-
##' @export
dsort <- function(data,x,...,decreasing=FALSE,return.order=FALSE)
{# {{{
    if (missing(x)) return(data)
    if (inherits(x,"formula")) {
        xx <- lava::procformula(value=x)$res
        decr <- unlist(lapply(xx,function(x) substr(trim(x),1,1)=="-"))
        if (any(decr)) decreasing <- decr
        x <- all.vars(x)
    }
    if (is.character(x) && length(x)!=nrow(data)) x <- lapply(x,function(z) data[,z,drop=TRUE])
    dots <- list(...)
    args <- lapply(dots, function(x) {
        if (length(x)==1 && is.character(x)) x <- data[,x,drop=TRUE]
        x
    })
    if (!is.list(x)) x <- list(x)
    ord <- do.call("order",c(c(x,args),list(decreasing=decreasing,method="radix")))
    data[ord,]
}# }}}

##' @export
dsort2 <- function(data,x,...,decreasing=FALSE,return.order=FALSE,regex=FALSE)
{# {{{
    if (missing(x)) return(data)
    if (inherits(x,"formula")) {
        xx <- lava::procformula(value=x)$res
        yxzf <- procform(x,data=data,do.filter=FALSE,regex=regex)
        decr <- unlist(lapply(xx,function(x) substr(trim(x),1,1)=="-"))
        if (any(decr)) decreasing <- decr
        x <- all.vars(x)
	x <- yxzf$predictor
    }
    if (is.character(x) && length(x)!=nrow(data)) x <- lapply(x,function(z) data[,z,drop=TRUE])
    dots <- list(...)
    args <- lapply(dots, function(x) {
        if (length(x)==1 && is.character(x)) x <- data[,x,drop=TRUE]
        x
    })
    if (!is.list(x)) x <- list(x)
    ord <- do.call("order",c(c(x,args),list(decreasing=decreasing,method="radix")))
    data[ord,]
}# }}}

##' @export
"dsort<-" <- function(data,...,value)  dsort(data,value,...)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets)
set.seed(100)

data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001
dfactor(bmt) <- tcell~tcell
bmt$event <- (bmt$cause!=0)*1

fg1 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=1)
summary(survivalG(fg1,bmt,time=50))

fg2 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=2)
summary(survivalG(fg2,bmt,time=50))

cif1time <- survivalGtime(fg1,bmt)
plot(cif1time,type="risk"); 

## -----------------------------------------------------------------------------
ss <- phreg(Surv(time,event)~tcell+platelet+age,bmt)
sss <- survivalG(ss,bmt,time=50)
summary(sss)

Gtime <- survivalGtime(ss,bmt)
plot(Gtime)

## -----------------------------------------------------------------------------

## survival situation
sr1 <- binregATE(Event(time,event)~tcell+platelet+age,bmt,cause=1,
		 time=40, treat.model=tcell~platelet+age)
summary(sr1)

## relative risk effect 
estimate(coef=sr1$riskDR,vcov=sr1$var.riskDR,f=function(p) p[2]/p[1],null=1)

## competing risks 
br1 <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,cause=1,
		 time=40,treat.model=tcell~platelet+age)
summary(br1)

## -----------------------------------------------------------------------------
br1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=40)
Gbr1 <- binregG(br1,bmt,Avalues=NULL)
summary(Gbr1)

## contrasting average age to 1+2-sd age, Avalues
Gbr2 <- binregG(br1,bmt,varname="age",Avalues=c(0,1,2))
summary(Gbr2)

## -----------------------------------------------------------------------------
sessionInfo()

##' Clayton-Oakes frailty model
##'
##' @title Clayton-Oakes model with piece-wise constant hazards
##' @param formula formula specifying the marginal proportional (piecewise constant) hazard structure with the right-hand-side being a survival object (Surv) specifying the entry time (optional), the follow-up time, and event/censoring status at follow-up. The clustering can be specified using the special function \code{cluster} (see example below).
##' @param data Data frame
##' @param cluster Variable defining the clustering (if not given in the formula)
##' @param var.formula Formula specifying the variance component structure (if not given via the cluster special function in the formula) using a linear model with log-link.
##' @param cuts Cut points defining the piecewise constant hazard
##' @param type when equal to \code{two.stage}, the Clayton-Oakes-Glidden estimator will be calculated via the \code{timereg} package
##' @param start Optional starting values
##' @param control Control parameters to the optimization routine
##' @param var.invlink Inverse link function for variance structure model
##' @param ... Additional arguments
##' @author Klaus K. Holst
##' @examples
##' set.seed(1)
##' d <- subset(simClaytonOakes(500,4,2,1,stoptime=2,left=2),truncated)
##' e <- ClaytonOakes(survival::Surv(lefttime,time,status)~x+cluster(~1,cluster),
##'                   cuts=c(0,0.5,1,2),data=d)
##' e
##'
##' d2 <- simClaytonOakes(500,4,2,1,stoptime=2,left=0)
##' d2$z <- rep(1,nrow(d2)); d2$z[d2$cluster%in%sample(d2$cluster,100)] <- 0
##' ## Marginal=Cox Proportional Hazards model:
##' ts <- ClaytonOakes(survival::Surv(time,status)~timereg::prop(x)+cluster(~1,cluster),
##'                    data=d2,type="two.stage")
##' ## Marginal=Aalens additive model:
##' ts2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),
##'                     data=d2,type="two.stage")
##' ## Marginal=Piecewise constant:
##' e2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~-1+factor(z),cluster),
##'                    cuts=c(0,0.5,1,2),data=d2)
##' e2
##'
##'
##' e0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~-1+factor(z),cluster),
##'                    cuts=c(0,0.5,1,2),data=d2)
##' ts0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~1,cluster),
##'                    data=d2,type="two.stage")
##' plot(ts0)
##' plot(e0,add=TRUE)
##'
##' e3 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),cuts=c(0,0.5,1,2),
##'                    data=d,var.invlink=identity)
##' e3
##' @export
ClaytonOakes <- function(formula,data=parent.frame(),cluster,var.formula=~1,cuts=NULL,type="piecewise",start,control=list(),var.invlink=exp,...) {

  mycall <- match.call()
  dots <- list(...)
  formulaId <- Specials(formula,"cluster") 
  formulaStrata <- Specials(formula,"strata")
  formulaSt <- "~."
  formulaProp <- Specials(formula,"prop")
  if (!is.null(formulaId)) {
    var.formulaId <- ~1
    if (length(formulaId)>1) {
      var.formula <- as.formula(formulaId[[1]])
      formulaId <- formulaId[[2]]
    }
    cluster <- formulaId
    mycall$cluster <- cluster
    formulaSt <- paste(formulaSt,paste("-cluster(",paste(var.formula,collapse=""),
                                       ",",formulaId,")"))
  }
  formulaSt <- paste(formulaSt,paste("-strata(",paste(formulaStrata,collapse="+"),")"))
  formula <- update(formula,formulaSt)
  
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  if (missing(cluster)) stop("Missing 'cluster' variable")
  ngamma <- 0
  data <- data[order(data[,cluster]),]
  Z <- model.matrix(var.formula,data)
  ngamma <- ncol(Z)

  if (type!="piecewise") {
    timeregmod <- ifelse(length(formulaProp)>0,"cox.aalen","aalen")
    timeregmod <- get(timeregmod, asNamespace("timereg"))
    if (is.null(dots$robust)) dots$robust <- 0
    args <- c(list(formula=formula,data=data,max.clust=NULL,clusters=data[,cluster]),dots)
    marg <- do.call(timeregmod, args)    
    return(two.stage(marg,data=data,theta.des=Z,var.link=1,...))
  }
  
  
  timevar <- terms(formula)[[2]]
  if (is.call(timevar)) {
    delayedentry <- (length(timevar)==4)*1
    entry <- NULL
    if (delayedentry==1)
      entry <- as.character(timevar[[2]])
    causes <- timevar[[3+delayedentry]]
    timevar <- timevar[[2+delayedentry]]
  }  
  timevar <- as.character(timevar)
  causes <- as.character(causes)
  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  X <- NULL
  nbeta <- 0  
  if (length(covars)>0) {
##    X <- model.matrix(as.formula(paste("~-1+",paste(covars,collapse="+"))),data)
    X <- model.matrix(update(formula,.~.+1),data)[,-1,drop=FALSE]
    nbeta <- ncol(X)
  }
  
  if (is.data.frame(data)) {
    mydata <- data.frame(T=data[,timevar],status=data[,causes],cluster=data[,cluster],entry=0)
    if (!is.null(entry)) {
      mydata$entry <- data[,entry]
    }
  } else {
    mydata <- data.frame(T=get(timevar,envir=data),status=get(causes,envir=data),cluster=get(cluster,envir=data),entry=0)
    if (!is.null(entry))      
      mydata$entry <- get(entry,envir=data)
  }
  if (is.null(cuts)) {
    cuts <- c(0,max(mydata$T))
  }
  if (max(mydata$T)>tail(cuts,1)) stop("Interval does not embed time observations")
  if (any(with(mydata, T<entry))) stop("Entry time occuring after event")

  ucluster <- unique(mydata$cluster)
  
  npar <- length(cuts)-1
  if (!is.null(X)) npar <- npar+ncol(X)
  npar <- npar+ncol(Z)
  p0 <- rep(0.1,npar)
  if (!missing(start)) p0 <- c(start,rep(0,max(0,length(npar)-length(start))))
  if (!is.null(dots$theta)) p0[1] <- dots$theta

  invlinkname <- as.character(substitute(var.invlink))

  
  obj <- function(p) {
    varpar <- p[seq(ngamma)]
    p <- p[-seq(ngamma)]
    ##    theta0 <- rep(exp(varpar),length(ucluster));
    theta0 <- Z%*%varpar
    if (exists(invlinkname)) theta0 <- var.invlink(theta0)
    multhaz <- rep(1,nrow(mydata))
    if (!is.null(X)) {
      nbeta <- ncol(X)
      beta <- p[seq(nbeta)]
      p <- p[-seq(nbeta)]
      multhaz <- exp(X%*%beta)
    }
    if (!is.null(dots$theta)) theta0 <- cbind(rep(dots$theta,length(theta0)))
    res <- .Call("claytonoakes",
           ds=mydata$status,ts=mydata$T,es=mydata$entry,
           allcs=mydata$cluster,cs=ucluster, cuts=cuts,
           hs=exp(p),mulths=multhaz,
           var=theta0,PACKAGE="mets")$logLik    
    return(-res)
  }
  opt <- tryCatch(nlminb(p0,obj,control=control),error=function(x) NULL)
  if (is.null(opt)) stop("Critical optmization problem")
  if (any(is.na(opt$par)) | any(!is.finite(opt$par)) | any(is.nan(opt$par)) ) {
    V <- matrix(NA,length(p0),length(p0))
  } else {    
    I <- numDeriv::hessian(obj,opt$par)
    ee <- tryCatch(eigen(I),error=function(x) NULL); 
    if (!is.null(ee)) {
      threshold <- 1e-12
      idx <- ee$values>threshold
      ee$values[idx] <- 1/ee$values[idx];
      if (!all(idx))
        ee$values[!idx] <- 0
      V <- with(ee, vectors%*%diag(values)%*%t(vectors))
    } else {
      V <- matrix(NA,length(p0),length(p0))
    }
  }
  res <- list(coef=opt$par,vcov=V,cuts=cuts,nbeta=nbeta,ngamma=ngamma,betanames=colnames(X),gammanames=colnames(Z),opt=opt,invlink=var.invlink,invlinkname=invlinkname)
  class(res) <- "claytonoakes"
  return(res)
}

##################################################

##' @export
print.claytonoakes <- function(x,...) {
  print(summary(x))
}

##' @export
print.summary.claytonoakes <- function(x,...) {
  printCoefmat(x$coef[,c(1,3,4)],...)
  cat("\nDependence parameters:\n")
  printCoefmat(x$var,...)
  invisible(x)
}

##' @export
summary.claytonoakes <- function(object,...) {
  mycoef <- matrix(nrow=length(object$coef),ncol=4)
  mycoef[,1:2] <- cbind(object$coef,sqrt(diag(object$vcov)))
  mycoef[,3:4] <- cbind(mycoef[,1]-qnorm(0.975)*mycoef[,2],mycoef[,1]+qnorm(0.975)*mycoef[,2])
  colnames(mycoef) <- c("Estimate","Std.Err","2.5%","97.5%")
  if (length(object$cuts))
  cutnames <- levels(cut(0,breaks=object$cuts))
  varname <- switch(object$invlinkname,exp="log-Var:",identity="Var:",paste("inv",object$invlinkname,"-Var:",sep=""))
  rownames(mycoef) <- c(paste(varname,object$gammanames,sep=""),object$betanames,cutnames)
  mycoef[-seq(object$ngamma),] <- exp(mycoef[-seq(object$ngamma),])
  varcoef <- object$invlink(mycoef[seq(object$ngamma),c(1,3,4),drop=FALSE])
  rownames(varcoef) <- object$gammanames
  varcoef <- cbind(varcoef,1/(1+2/varcoef))
  colnames(varcoef)[c(1,4)] <- c("Variance","Kendall's tau")  
  res <- list(coef=mycoef,var=varcoef)
  class(res) <- "summary.claytonoakes"
  res
}

##' @export
plot.claytonoakes <- function(x,chaz=TRUE,add=!is.null(dev.list()),col="darkblue",...) {
  haz <- summary(x)$coef[-seq(x$nbeta+x$ngamma),,drop=FALSE]
  t <- x$cuts
  L <- approxfun(t,f=1,cumsum(c(0,haz[,1]*diff(t))),method="linear")
  if (add) {
    lines(t,L(t),col=col,...)
  } else {
    plot(t,L(t),type="l",col=col,...)
  }
  invisible(x)  
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="png",
  comment = "#>"  
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)

## -----------------------------------------------------------------------------
 library(mets)
 runb <- 0
 options(warn=-1)
 set.seed(1000) # to control output in simulatins for p-values below.

n <- 200; k.boot <- 10; 

dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
          beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
    treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
dfactor(dat) <- dnr.f~dnr
dfactor(dat) <- gp.f~gp
drename(dat) <- ttt24~"ttt24*"
dat$id <- 1:n
dat$ftime <- 1

## -----------------------------------------------------------------------------
weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
wdata <- medweight(fit,data=dat)

## -----------------------------------------------------------------------------
aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
summary(aaMss2)

## ----label=firstmodel---------------------------------------------------------
### binomial regression ###########################################################
aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cause=2)
summary(aaMss)

ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

## ----label=multiplemodels-----------------------------------------------------
### lin-ying model ################################################################
aaMss <- aalenMets(Surv(time/100,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		   weights=wdata$weights)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### cox model ###############################################################################
aaMss <- phreg(Surv(time,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
	       weights=wdata$weights)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### Fine-Gray #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,propodds=NULL,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### logit model  #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### binomial outcome  ############################
aaMss <- binreg(Event(ftime,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cens.weights=1,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

## ----label=multinom, cache=TRUE, eval=fullVignette----------------------------
#  data(tTRACE)
#  dcut(tTRACE) <- ~.
#  
#  weightmodel <- fit <- mlogit(wmicat.4 ~agecat.4+vf+chf,data=tTRACE,family=binomial)
#  wdata <- medweight(fit,data=tTRACE)
#  
#  aaMss <- binreg(Event(time,status)~agecat.40+ agecat.41+ vf+chf+cluster(id),data=wdata,
#  		time=7,weights=wdata$weights,cause=9)
#  summary(aaMss)
#  MultMed <- mediatorSurv(aaMss,fit,data=tTRACE,wdata=wdata)

## ----results="hide", echo=FALSE-----------------------------------------------
## To save time building the vignettes on CRAN, we cache time consuming computations
if (fullVignette) {
  MultMed[c('iid','iid.w','iid.surv')] <- NULL
  saveRDS(MultMed, "data/MultMed.rds")
} else {
  MultMed <- readRDS("data/MultMed.rds")
}

## -----------------------------------------------------------------------------
summary(MultMed)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
set.seed(100)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001
     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean.phreg(out1,times=30)
     summary(rm1)
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     ## same as integrated cumulative incidence 
     rmc1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)

## -----------------------------------------------------------------------------

 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,outcome="rmst-cause",
		    time=40,treat.model=tcell~platelet)
 summary(out1)

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,outcome="rmst-cause",
		    time=40,treat.model=tcell~platelet)
 summary(out2)

## -----------------------------------------------------------------------------
sessionInfo()

pred_weibull <- function(object, X, Z,
                         times,
                         individual.times = FALSE,
                         type = c("surv", "haz", "chaz", "lp"), ...) {
    p <- coef(object)
    V <- vcov(object)
    X <- rbind(X)
    Z <- rbind(Z)
    p1 <- X %*% p[seq_len(ncol(X))] # log-scale
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)] # log-shape
    if (type[1] == "lp") {
      return(cbind(lograte = p1, logshape = p2))
    }
    if (nrow(X) != nrow(Z)) stop("incompatible X and Z matrix")
    n <- nrow(X)
    np <- ncol(X) + ncol(Z)
    xz <- matrix(0, ncol = np, nrow = 2)
    var_arr <- array(dim = c(2, 2, n))
    for (i in seq_len(n)) {
      xz[1, seq_len(ncol(X))] <- X[i, ]
      xz[2, seq_len(ncol(Z)) + ncol(X)] <- Z[i, ]
      var_arr[, , i] <- xz %*% V %*% t(xz)
    }
    par <- cbind(p1, p2)
    nt <- length(times)
    if (individual.times && nt != n) {
        stop("For individual time predictions 'times', 'X' and 'Z' should agree") # nolint
    }
    ## time.fun optional smooth function specifying transformation of
    ##   time-variable. Here the cumulative hazard is assumed to be
    ##   \eqn{H(t)=\lambda g(t)^s}
    ## if (is.null(time.fun)) {
    ##   time.fun <- structure(identity, grad = identity)
    ## }
    ## chaz <- function(par, ...) exp(par[1]) * (time.fun(newtime)**exp(par[2]))
    chaz <- function(par, ...) exp(par[1]) * (newtime**exp(par[2]))
    surv <- function(par, ...) exp(-chaz(par, ...))
    haz <- function(par) {
      res <- exp(par[1] + par[2] +
          (exp(par[2]) - 1) * log(newtime))
                 ## (exp(par[2]) - 1) * log(time.fun(newtime))
                 ## + log(attr(time.fun, "grad")(newtime)))
      res[newtime == 0] <- 0
      return(res)
    }
    if (individual.times) {
      est_arr <- array(dim = c(nt, 4))
    } else {
      est_arr <- array(dim = c(nt, 4, n))
      newtime <- times
    }
    for (i in seq_len(n)) {
      if (individual.times) {
        newtime <- times[i]
      }
      pr <- estimate(
        coef = par[i, ],
        vcov = var_arr[, , i], f = get(type[1]), ...
        )$coefmat
      if (individual.times) {
        est_arr[i, ] <- pr[1:4]
      } else {
        est_arr[, , i] <- pr[, 1:4]
      }
    }
    if (individual.times) {
      dimnames(est_arr) <- list(seq_len(nt), colnames(pr)[1:4])
    } else {
      dimnames(est_arr) <- list(
        paste0("t", seq_len(nt)),
        colnames(pr)[1:4], seq_len(n)
      )
    }
  return(est_arr)
}

logl_weibull <- function(p, entry, exit, status,
                         X = NULL, Z = NULL,
                         time.fun = NULL) {
    if (is.null(X)) X <- cbind(rep(1, length(exit)))
    if (is.null(Z)) Z <- cbind(rep(1, length(exit)))
    dt <- 1
    if (!is.null(time.fun)) {
      if (is.null(attr(time.fun, "grad"))) {
          # complex-step derivative
          dt <- numDeriv::grad(time.fun, exit, method = "complex")
      } else {
          # numerical derivative
          dt <- attr(time.fun, "grad")(exit)
      }
      exit <- time.fun(exit)
      entry <- time.fun(entry)
    }
    if (length(p) != ncol(X) + ncol(Z)) stop("wrong parameter length")
    p1 <- X %*% p[seq_len(ncol(X))] # log-scale
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)] # log-shape
    theta <- cbind(exp(p1), exp(p2))
    loghaz <- log(theta[, 1]) + log(theta[, 2]) +
      (theta[, 2] - 1) * log(exit) + log(dt)
    chaz <- function(t) theta[, 1] * t**theta[, 2]
    logl <- status * loghaz - chaz(exit) + chaz(entry)
    logl
}

score_weibull <- function(p, entry, exit, status,
                          X = NULL, Z = NULL,
                          time.fun = NULL) {
    if (is.null(X)) X <- cbind(rep(1, length(exit)))
    if (is.null(Z)) Z <- cbind(rep(1, length(exit)))
    if (!is.null(time.fun)) {
        entry = time.fun(entry)
        exit = time.fun(exit)
    }
    if (length(p) != ncol(X) + ncol(Z)) stop("wrong parameter length")
    p1 <- X %*% p[seq_len(ncol(X))]
    p2 <- Z %*% p[seq_len(ncol(Z)) + ncol(X)]
    theta <- exp(cbind(p1, p2))
    dloghaz.dtheta1 <- status / theta[, 1]
    dchaz.dtheta1 <- function(t) {
        idx0 <- which(t == 0)
        res <- t**theta[, 2]
        res[idx0] <- 0 # conv. log(x)*x =0 when x=0
        return(res)
    }
    dloghaz.dtheta2 <- status / theta[, 2] + status * log(exit)
    dchaz.dtheta2 <- function(t) {
        idx0 <- which(t == 0)
        res <- theta[, 1] * log(t) * t**theta[, 2]
        res[idx0] <- 0 # conv. log(x)*x =0 when x=0
        return(res)
    }
    res <- cbind(
        dloghaz.dtheta1 - dchaz.dtheta1(exit) + dchaz.dtheta1(entry),
        dloghaz.dtheta2 - dchaz.dtheta2(exit) + dchaz.dtheta2(entry)
    )
    for (i in seq_len(ncol(X))) X[, i] <- X[, i] * res[, 1] * theta[, 1]
    for (i in seq_len(ncol(Z))) Z[, i] <- Z[, i] * res[, 2] * theta[, 2]
    return(cbind(X, Z))
}

##' @description Fits a Cox-Weibull with cumulative hazard given by \deqn{
##'   \Lambda(t) = \lambda \cdot t^s } where \eqn{s} is the shape parameter, and
##'   \eqn{\lambda} the rate parameter. We here allow a regression model for
##'   both parameters \deqn{\lambda := \exp(\beta^\top X)} \deqn{s :=e
##'   \exp(\gamma^\top Z)} as defined by `formula` and `shape.formula`
##'   respectively.
##' @details The parametrization
##' @title Weibull-Cox regression
##' @param formula Formula for proportional hazards. The right-handside must be
##'   an [Event] or [Surv] object (with right-censoring and possibly delayed
##'   entry).
##' @param shape.formula Formula for shape parameter
##' @param data data.frame
##' @param save.data if TRUE the data.frame is stored in the model object (for
##'   predictions and simulations)
##' @param control control arguments to optimization routine [stats::nlmbin]
##' @seealso [mets::phreg()]
##' @aliases phreg_weibull phreg.par
##' @author Klaus Kähler Holst, Thomas Scheike
##' @examples
##' data(sTRACE, package="mets")
##' sTRACE$entry <- 0
##' fit1 <- phreg_weibull(Event(entry, time, status == 9) ~ age,
##'              shape.formula = ~age, data = sTRACE)
##' tt <- seq(0,10, length.out=100)
##' pr1 <- predict(fit1, newdata = sTRACE[1, ], times = tt)
##' fit2 <- phreg(Event(time, status == 9) ~ age, data = sTRACE)
##' pr2 <- predict(fit2, newdata = sTRACE[1, ], se = FALSE)
##' if (interactive()) {
##'    plot(pr2$times, pr2$surv, type="s")
##'    lines(tt, pr1[,1,1], col="red", lwd=2)
##' }
##' @export
##' @return `phreg.par` object
phreg_weibull <- function(formula,
                          shape.formula = ~1,
                          data,
                          save.data = TRUE,
                          control = list()) {
    cl <- match.call()
    des <- proc_design(
      formula,
      data = data,
      specials = c("offset", "weights", "cluster"),
      intercept = TRUE
    )
    des2 <- proc_design(shape.formula, data = data, intercept = TRUE)
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    Z <- des2$x
    obj <- function(p) {
        -.logl_weibull(p, entry, exit, status,
            X = X, Z = Z)
    }
    grad <- function(p, indiv = FALSE) {
      -.score_weibull(p, entry, exit, status,
                      X = X, Z = Z, indiv = indiv)
    }
    if (!is.null(control$start)) {
        p0 <- control$start
        control$init <- NULL
    } else {
        p0 <- rep(0, ncol(X) + ncol(Z))
    }
    op <- stats::nlminb(p0, obj, grad, control = control)
    p <- op$par
    H <- numDeriv::jacobian(grad, p)
    U <- -grad(op$par, indiv=TRUE)
    n <- nrow(U)
    loglik <- structure(-obj(p),
        df = length(p), nobs = n, nall = n,
        class = "logLik"
        )
    ic <- t(solve(H) %*% t(U)) * n
    names(p) <- c(
      colnames(X),
      paste0("s:", colnames(Z))
    )
    est <- lava::estimate(coef = p, IC = ic)
    est$model.index <- list(ncol(X), ncol(X) + ncol(Z))
    res <- list(
        loglik = loglik,
        response = list(entry = entry, exit = exit, status = status),
        call = cl, opt = op, coef = p,
        hessian = H,
        rate.design = clean_design(des),
        shape.design = clean_design(des2),
        data = NULL,
        estimate = est
    )
    if (save.data) res$data <- data
    return(structure(res, class = "phreg.par"))
}

##' @export
phreg.par <- function(...) {
  warning("`phreg.par` will be deprecated in the next release of mets and replaced by `phreg_weibull`") # nolint
  res <- phreg_weibull(...)
  if (deparse(res$shape.design$terms)!="~1") {
    stop("this function only works with a single shape")
  }
  if (attr(res$rate.design$terms, "intercept") == 0L) {
    stop("model needs intercept")
  }
  np <- length(coef(res))
  tr <- function(p) c(p[1] / exp(p[np]), p[np], p[-c(1, np)])
  estimate(res, tr)
}

##' @export
print.phreg.par <- function(x, ...) {
    cat("\n- Weibull-Cox model -\n\n")
    cat("Call:\n")
    dput(x$call)
    cat("\n")
    cat(sprintf("log-Likelihood: %f", x$loglik), "\n\n")
    dur <- with(x$response, exit - entry)
    n <- length(x$response$status)
    events <- sum(x$response$status)
    nn <- rbind(c(n, events, sum(dur)))
    colnames(nn) <- c("n","events","obs.time")
    rownames(nn) <- ""
    print(nn, quote=FALSE)
    cat("\n")
    print(x$estimate, ...)
    cat("\n")
}

##' @export
vcov.phreg.par <- function(object, ...) {
    vcov(object$estimate)
}

##' @export
coef.phreg.par <- function(object, ...) {
    coef(object$estimate)
}

##' @export
simulate.phreg.par <- function(object,
                               nsim = nrow(data),
                               seed = NULL,
                               data = object$data,
                               cens.model,
                               var.names = c("time", "status"),
                               ...) {
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    runif(1)
  if (is.null(seed))
    RNGstate <- get(".Random.seed", envir = .GlobalEnv)
  else {
    R.seed <- get(".Random.seed", envir = .GlobalEnv)
    set.seed(seed)
    RNGstate <- structure(seed, kind = as.list(RNGkind()))
    on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
  }
  # bootstrap covariates
  newd <- mets::dsample(size = nsim, data)
  # linear-predictors
  lp <- predict(object, newdata = newd, type = "lp")
  ## simulate event times
  time <- rweibullcox(nrow(lp),
                      rate = exp(lp[, 1]),
                      shape = exp(lp[, 2])
                      )
  if (missing(cens.model)) {
      y <- update_design(object$rate.design, data = data, response = TRUE)$y
      data$cens_ <- !y[, 2]
      data$time_ <- y[, 1]
      cens.model <- phreg_weibull(Surv(time_, cens_) ~ 1, data = data)
  }
  if (is.null(cens.model)) {
    cens <- rep(TRUE, nrow(newd))
  } else {
    cens.par <- exp(predict(cens.model, type = "lp", newdata = newd))
    cens <- rweibullcox(nsim, cens.par[, 1], cens.par[, 2])
  }
  newd[[var.names[1]]] <- pmin(time, cens)
  newd[[var.names[2]]] <- (time <= cens)
  return(newd)
}

##' @export
IC.phreg.par <- function(x, p=coef(x), ...) {
  IC(x$estimate)
}

##' @export
logLik.phreg.par <- function(object, ...) {
  object$loglik
}

##' @export
predict.phreg.par <- function(object,
                              newdata = object$data,
                              times,
                              individual.times = FALSE,
                              type = c("surv", "haz", "chaz", "lp"),
                              level = 0.05,
                              ...) {
  xy <- update_design(object$rate.design, data = newdata, response = TRUE)
  x <- xy$x
  z <- update_design(object$shape.design, data = newdata)$x
  if (type[1] != "lp" && missing(times)) {
    times <- xy$y[, 1]
  }
  pr <- pred_weibull(object,
                     X = x, Z = z,
                     times = times,
                     individual.times = individual.times,
                     level = level,
                     type = type, ...
                     )
  return(pr)
}


##' @export
##' @description Simulate observations from the model with cumulative hazard
##'   given by \deqn{\Lambda(t) = \lambda\cdot t^s} where \eqn{\lambda} is the
##'   \emph{rate parameter} and \eqn{s} is the \emph{shape parameter}.
##' @details [stats::rweibull()] uses a different parametrization with
##'   cumulative hazard given by
##' \deqn{H(t) = (t/b)^a,}
##' i.e., the shape is the same \eqn{a:=s} but the scale paramter \eqn{b}
##' is related to rate paramter \eqn{r} by
##' \deqn{r := b^{-a}}
##' @seealso [stats::rweibull()]
##' @title Simulate observations from a Weibull distribution
##' @param n (integer) number of observations
##' @param rate (numeric) rate parameter (can be a vector of size n)
##' @param shape (numeric) shape parameter (can be a vector of size n)
rweibullcox <- function(n, rate, shape) {
  # F(t) = 1-exp(-H(t)) = 1-exp(-lambda t^s). Inversion=>
  (-log(1-runif(n))/rate)^(1/shape)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## ----while-alive--------------------------------------------------------------
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")


## ----wa-composite-------------------------------------------------------------
hfactioncpx12$marks <- runif(nrow(hfactioncpx12))

##ddmg <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
##cause=1:2,death.code=2,marks=hfactioncpx12$marks)
##summary(ddmg)

ddm <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
cause=1:2,death.code=2,marks=hfactioncpx12$status)


## -----------------------------------------------------------------------------
sessionInfo()

##' Discrete Time-to-Event Haplotype Analysis
##'
##' Performs cycle-specific logistic regression to estimate haplotype effects on discrete 
##' time-to-event data, accounting for phase ambiguity. Given observed genotypes \eqn{G} 
##' and unobserved haplotypes \eqn{H}, the method integrates (mixes out) over the possible 
##' haplotype configurations using the conditional probabilities \eqn{P(H|G)}.
##' 
##' The survival function is computed by averaging over the possible haplotypes:
##' \deqn{ S(t|x,G) = E[ S(t|x,H) | G ] = \sum_{h \in G} P(h|G) S(t|x,h) }
##' 
##' The discrete hazard function is modeled using logistic regression:
##' \deqn{ \text{logit}(P(T=t | T \geq t, x, h)) = \alpha_t + x(h) \beta }
##' where \eqn{\alpha_t} are time-specific intercepts (baseline hazards), \eqn{x(h)} is the 
##' regression design constructed from covariates and haplotypes \eqn{h=(h_1, h_2)}, and 
##' \eqn{\beta} are the regression coefficients.
##' 
##' The likelihood is maximized numerically. Standard errors are computed assuming that 
##' \eqn{P(H|G)} is known (i.e., ignoring the uncertainty in haplotype estimation).
##' 
##' The design matrix over possible haplotypes is constructed by merging the covariate 
##' data \eqn{X} with the haplotype probabilities \eqn{Haplos} and applying a user-defined 
##' \code{designfunc}.
##'
##' @param X Design matrix data frame (must be sorted by \code{id} and \code{time}) containing 
##'   the ID, time variable, binary response, and covariates.
##' @param y Name of the response variable (binary, 0/1) in \code{X}.
##' @param time.name Name of the time variable in \code{X} used for sorting and cycle definition.
##' @param Haplos Data frame containing \code{id}, \code{haplo1}, \code{haplo2} (haplotypes as factors), 
##'   and \code{p} (probability \eqn{P(H|G)}).
##' @param id Name of the ID variable in \code{X} and \code{Haplos}.
##' @param desnames Names of the covariate columns in \code{X} to be used in the design matrix.
##' @param designfunc Function that computes the design vector given haplotypes \eqn{h=(h_1, h_2)} 
##'   and covariates. Must return a vector or matrix compatible with the model.
##' @param beta Starting values for the optimization (vector of length \eqn{p + k}, where \eqn{p} 
##'   is the number of covariate effects and \eqn{k} is the number of time cycles).
##' @param no.opt Logical; if TRUE, skips optimization and returns estimates based on the 
##'   provided \code{beta} (useful for initialization or diagnostics).
##' @param method Optimization method: \code{"NR"} (Newton-Raphson, default) or \code{"nlm"}.
##' @param stderr Logical; if FALSE, returns only the coefficient estimates.
##' @param designMatrix Alternative to \code{X} and \code{Haplos}: provides the response and 
##'   design matrix directly (not fully implemented).
##' @param response Alternative to \code{X}: provides the response and design directly (not fully implemented).
##' @param idhap Name of the ID-haplotype variable to specify different haplotypes for different IDs.
##' @param design.only Logical; if TRUE, returns only the design matrices constructed for the analysis.
##' @param covnames Names of covariates to extract from the object for regression output.
##' @param fam Family of the model (default \code{binomial}, currently the only option).
##' @param weights Weights following ID for the GLM component.
##' @param offsets Offsets following ID for the GLM component.
##' @param idhapweights Weights following ID-haplotype for the GLM component (Work in Progress).
##' @param ... Additional arguments passed to the optimizer (\code{lava::NR} or \code{nlm}).
##' @return An object of class \code{"haplosurvd"} containing:
##'   \item{coef}{Estimated coefficients (baseline time effects and haplotype/covariate effects).}
##'   \item{se}{Standard errors of the coefficients.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{se.robust}{Robust standard errors (if available).}
##'   \item{iid}{Influence function (IID) decomposition.}
##'   \item{ploglik}{Log-likelihood at convergence.}
##'   \item{gradient, hessian}{Optimization results.}
##'   \item{Xhap, X, Haplos}{Data and design matrices used.}
##'   \item{nid, nidhap}{Number of IDs and ID-haplotype combinations.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. H. (2024). Discrete time survival analysis with haplotype effects. mets package documentation.
##' @examples
##' ## Some haplotypes of interest
##' types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")
##' 
##' ## Some haplotype frequencies for simulations 
##' data(haplo)
##' hapfreqs <- haplo$hapfreqs 
##'
##' www <- which(hapfreqs$haplotype %in% types)
##' hapfreqs$freq[www]
##'
##' baseline <- hapfreqs$haplotype[9]
##' baseline
##'
##' ## Design function: indicator for presence of any 'types' haplotype
##' designftypes <- function(x, sm=0) {
##'   hap1 <- x[1]
##'   hap2 <- x[2]
##'   if (sm == 0) y <- 1 * ((hap1 == types) | (hap2 == types))
##'   if (sm == 1) y <- 1 * (hap1 == types) + 1 * (hap2 == types)
##'   return(y)
##' }
##'
##' tcoef <- c(-1.93110204, -0.47531630, -0.04118204, -1.57872602, -0.22176426, -0.13836416,
##'            0.88830288, 0.60756224, 0.39802821, 0.32706859)
##' 
##' ghaplos <- haplo$ghaplos
##' haploX  <- haplo$haploX
##' 
##' haploX$time <- haploX$times
##' Xdes <- model.matrix(~ factor(time), haploX)
##' colnames(Xdes) <- paste("X", 1:ncol(Xdes), sep="")
##' X <- dkeep(haploX, ~ id + y + time)
##' X <- cbind(X, Xdes)
##' Haplos <- dkeep(ghaplos, ~ id + "haplo*" + p)
##' desnames <- paste("X", 1:6, sep="")   # Six X's related to 6 cycles 
##' out <- haplo_surv_discrete(X=X, y="y", time.name="time",
##'          Haplos=Haplos, desnames=desnames, designfunc=designftypes) 
##' names(out$coef) <- c(desnames, types)
##' out$coef
##' summary(out)
##' @export
haplo_surv_discrete <- function (X=NULL,y="y",time.name="time",Haplos=NULL,id="id",desnames=NULL,designfunc=NULL,
    beta=NULL,no.opt=FALSE,method="NR",stderr=TRUE,designMatrix=NULL,response=NULL,idhap=NULL,design.only=FALSE,
    covnames=NULL,fam=binomial,weights=NULL,offsets=NULL,idhapweights=NULL,...)
{ ## {{{ 
  cond=NULL

if (is.null(designMatrix)) { 
if (!is.null(Haplos)) { ## with haplo-types {{{
  ## X: y, Xdes, id
  ## Haplos, haplo1, haplo2, id, p (HgivenG)
  bothid <- intersect(X$id,Haplos$id)
  X <- subset(X,id %in% bothid)
  Haplos <- subset(Haplos,id %in% bothid)
  ## new iid starts at 1 
  Haplos$id <- fast.approx(bothid,Haplos$id) 
  iiid <- Haplos$id-1
  X$id <- fast.approx(bothid,X$id) 
  Xo <- X

  Xhap <- merge(X,Haplos,by.x="id",by.y="id")
  Xhap <- dsort2(Xhap,~id+"haplo*"+time)
  Haplos <- dsort2(Haplos,~id+"haplo*")

  time <- Xhap[,time.name]
  tn <- match(time.name,names(Xhap))
  Xhap <- Xhap[,-tn]

  response <- Xhap[,y]
  yn <- match(y,names(Xhap))
  Xhap <- Xhap[,-yn]

  mm <- grep("^haplo", names(Xhap))
  Xhaps <- Xhap[,mm]
  if (!is.null(designfunc)) {
	  Xo <- X <- as.matrix(apply(Xhap[,mm],1,designfunc))
	  if (ncol(X)==nrow(Xhap))  X <- t(X)
	  colnames(X) <- paste("haplo",1:ncol(X),sep="")
	  X <- as.matrix(cbind(Xhap[,desnames],X))
  } else Xo <- X <- as.matrix(Xhap[,desnames])

  ## X(i)^T %*% X(i) for each row 
  X2  <- .Call("vecMatMat",X,X)$vXZ

## creates sub-index for haplo-types within each id
   nmm <- names(Xhap)[mm]
   ms <- mystrata(Xhap[,c("id",nmm)])
	   stratidhap <- ms
   nidhap <- attr(ms,"nlevel")
   nid <- length(unique(Haplos$id))

  ## weights will follow id 
  if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
  ## idhap weights will follow id-hap
  if (is.null(idhapweights))  whapiid <- rep(1,nidhap)  else whapiid <- idhapweights
  ## offsets can follow both Haplo or X design and will then appear in mixed design
  if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

  ### to make the optimizer more flexible and use for interval censored data 
  if (is.null(cond)) cond <- rep(0,nidhap)

   hgiveng <- Haplos$p
# }}}
} else {  ## standard glm {{{ 
  ## X: y, Xdes, id

  id.name <- id
  ## id going from 1 to #id's
  id <- X$id <- fast.approx(unique(X[,id]),X[,id]) 
  Xhap <- Xo <- X
  iiid <- unique(X$id)-1
  nid <- length(iiid)
  stratidhap <- id 
  nidhap <- length(unique(stratidhap))

  ###
  response <- Xhap[,y]
  yn <- match(y,names(Xhap))
  Xhap <- Xhap[,-yn]

  X <- as.matrix(Xhap[,desnames])
  ## X(i)^T %*% X(i) for each row 
  X2  <- .Call("vecMatMat",X,X)$vXZ

  ## weights will follow id 
  if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
  ## idhap weights will follow id-hap
  if (is.null(idhapweights))  wph <- rep(1,nidhap)  else wph <- idhapweights
  ## offsets can follow both Haplo or X design and will then appear in mixed design
  if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

  if (is.null(cond)) cond <- rep(0,nidhap)
  hgiveng <- rep(1,nid)

}# }}}
} else {# {{{
	hgiveng <- designMatrix$p
	X <- designMatrix[,desnames]
	id <- designMatrix[,id]
        iiid <- unique(id)-1

	if (is.null(idhap)) stop("'idhap' must be supplied when 'designMatrix' is provided.")
        stratidhap <- designMatrix[,idhap]
        nidhap <- length(unique(stratidhap))
        nid <- length(iiid)
	design.only <- FALSE

        ## weights will follow id 
        if (is.null(weights))  wiid <- rep(1,nid)  else wiid <- weights
        ## idhap weights will follow id-hap
        if (is.null(idhapweights))  wph <- rep(1,nidhap)  else wph <- idhapweights
        ## offsets can follow both Haplo or X design and will then appear in mixed design
        if (is.null(offsets))  offiid <- rep(0,nrow(X)) else offiid <- offsets

        if (is.null(cond)) cond <- rep(0,nidhap)

}# }}}

if (!design.only) {

	if (is.null(beta)) beta <- rep(0,ncol(X))

	obj <- function(pp,all=FALSE)
	{ # {{{

	lp <- X %*% pp 
	## plp <- family$linkinv(lp)
	plp <- expit(lp+ offiid)
	nplp <- 1-plp

	logpht <- log(plp)*response+log(nplp)*(1-response)
	pht <-c(exp(logpht))
	Dlogpht <-  X* c(response-plp)
	D2logpht <- c(plp/(1+exp(lp)))*X2

	ph <- c(exp(sumstrata(logpht,stratidhap-1,nidhap)))
	pg <- c(sumstrata(ph*hgiveng,iiid,nid))
	logl <- wiid*log(pg)

	## Derivative 
	Dlogph  <- apply(Dlogpht,2,sumstrata,stratidhap-1,nidhap)
	Dph  <- c(ph)*Dlogph
	Dpg <- apply(Dph*hgiveng,2,sumstrata,iiid,nid)# {{{}}}
	Dlogl    <- wiid*Dpg/pg
	DpgDpg  <- .Call("vecMatMat",Dpg,Dpg)$vXZ

	## 2nd Derivative 
	D2logph <- apply(D2logpht,2,sumstrata,stratidhap-1,nidhap)
	DphDlogph <- .Call("vecMatMat",Dph,Dlogph)$vXZ
	D2ph <- ph*D2logph+DphDlogph
	D2pg <-apply(D2ph*hgiveng,2,sumstrata,iiid,nid)
	D2logi <- wiid*(pg*D2pg-DpgDpg)/pg^2
	D2log <- apply(D2logi,2,sum)
	D2log <- matrix(D2log,ncol(X),ncol(X))

	ploglik <- sum(logl)
	gradient <- apply(Dlogl,2,sum)
	hessian <- D2log

	  if (all) {
	      ihess <- pinv(hessian)
	      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
	      robvar <- crossprod(beta.iid)
	      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
			  iid=beta.iid,robvar=robvar,var=ihess,
			  id=iiid,
			  se=diag(ihess)^.5,se.robust=diag(robvar)^.5)
	      return(val)
	  }  
	 structure(-ploglik,gradient=-gradient,hessian=hessian)
	}# }}}

	  p <- ncol(X)
	  opt <- NULL
	  if (p>0) {
	  if (no.opt==FALSE) {
	      if (tolower(method)=="nr") {
		  opt <- lava::NR(beta,obj,...)
		  opt$estimate <- opt$par
	      } else {
		  opt <- nlm(obj,beta,...)
		  opt$method <- "nlm"
	      }
	      cc <- opt$estimate; 
	      if (!stderr) return(cc)
	      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
	      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
	  } else {
	      val <- obj(0,all=TRUE)
	  }


}  else val <- NULL

  val <- c(list(Xhap=Xhap,X=X,Haplos=Haplos),val)
  class(val) <- "haplosurvd"
  return(val)
} ## }}} 

## uses HaploSurvival package of github install via devtools
## devtools::install_github("scheike/HaploSurvival")
## this is only used for simulations 
## out <- simHaplo(1,100,tcoef,hapfreqs)

##' @export
summary.haplosurvd <- function(object,...) { ## {{{ 
out <- lava::estimate(object,...)
return(out)
} ## }}} 

##' @export
print.haplosurvd <- function(x,...) summary(x,...)

##' @export
vcov.haplosurvd <- function(object,...) return(object$var) 

##' @export
coef.haplosurvd <- function(object,...) return(object$coef)

##' @export
fast.cluster <- function(x,...) {
    arglist <- list("FastCluster",
                    time=as.integer(x),
                    PACKAGE="mets")
    res <- do.call(".Call",arglist)
    return(as.vector(res))
}

F1addfg<-function(t,lam0=0.5,beta=c(-0.5,-0.005,-0.004),x=rep(0,3)) # FG
{ ## {{{
baset <- 0.13*pnorm((t-.70)/.13)
xm <- matrix(x,ncol=3) 
return( 1 - exp(-baset*exp(xm %*% matrix(beta,3,1)))) 
} ## }}}

##' @export 
corsim.prostate <- function(n,theta=1,thetaslope=0,crate=2,test=0,pcens=0,mt=1,same.cens=TRUE) 
{ ## {{{
###n <- 10; theta <- 1; thetaslope <- 0; mt <- 1
xl <- sample(1:4,n,replace=TRUE)
###xl <- rep(xl,each=2)
x<-cbind(xl==2,xl==3,xl==4)*1
tt<-seq(0,mt,length=mt*100)
###
###n=100;theta=1;lam0=0.5;beta=0.3;crate=2
thetat <- exp(log(theta))
F11x<-F1addfg(mt,x=x)
F12x<-F1addfg(mt,x=x)
###
thetaslut <- exp(log(theta)+thetaslope*(mt-mt/2))
p11 <- thetaslut*F11x*F12x/((1-F11x)+thetaslut*F11x)
p12 <- F11x-p11
p21 <- F12x-p11
p22 <- 1-F12x-F11x+p11
###apply(cbind(p11,p12,p21),1,sum)
if (test==1) { ## {{{
for (i in 1:2) {
print(x[i,]); 
F11xt<-F1addfg(tt,x=x[i,])
F12xt<-F1addfg(tt,x=x[i,])
p11t <- thetat* F11xt*F12xt/((1-F11xt)+thetat*F11xt)
cortt <- ((p11t)/(F12xt-p11t))/(F11xt/(1-F11xt))
###plot(tt,log(cortt))
if (i==1) { 
plot(tt,p11t,type="l",ylim=c(0,0.1),xlim=c(0,mt))
###lines(tt,F11x[i]-p11t,col=2)
###lines(tt,F12x[i]-p11t,col=2)
} else lines(tt,p11t,col=2);
###if (sum(diff(p11t<0))>0) stop("dec\n"); 
###p11 <- max(p11t)
###p12 <- F11x[i]-p11
###p21 <- F12x[i]-p11
###p22 <- 1- F12x[i]-F11x[i]+p11
###pnn <- 1- F12x[i]-F11x[i]+p11
}
} ## }}}
###apply(cbind(p11,p12,p21,p22),1,sum)
###
types <- rep(0,n)
causes <- matrix(0,n,2)
stime<-matrix(mt+1,n,2); 
for (i in 1:n)
{
ptype <- runif(1)
if (ptype<=p11[i]) {
	types[i] <- 1
	myhazx<-F1addfg(tt,x=x[i,])/F12x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist\n"); 
	stime[i,2]<-Cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	f1<- F1addfg(tt,x=x[i,])
       	myhazx<- (F12x[i]/p11[i]) * (thetat*f1/((1-f1)+thetat*f1))
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist\n"); 
	stime[i,1]<-Cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(1,1)
}
if ((ptype>p11[i]) & (ptype<=p12[i]+p11[i])) {
	types[i] <- 2
	f1 <- F1addfg(tt,x=x[i,])
	myhazx<- ( f1 - thetat*F12x[i]*f1/((1-f1)+thetat*f1))/p12[i]; 
	myhazx <- f1/F11x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist 2 \n"); 
	stime[i,1]<-Cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(1,2)
	stime[i,2] <- runif(1)*mt
}
if ((ptype>p11[i]+p12[i]) && (ptype<=p21[i]+p12[i]+p11[i])) {
	types[i] <- 3
        f2 <- F1addfg(tt,x=x[i,])
	myhazx <-  (f2 - (thetat*F11x[i]*f2/((1-F11x[i])+thetat*F11x[i])))/p21[i]; 
	myhazx <- f2/F12x[i]
###	if (abs(max(myhazx)-1)> 0.001) stop("not dist3 \n"); 
	stime[i,2]<-Cpred(cbind(myhazx,tt),runif(1))[1,2]+runif(1,0,0.001)
	causes[i,] <- c(2,1)
	stime[i,1] <- runif(1)*mt
}
if (ptype>p11[i]+p12[i]+p21[i] ) {
	types[i] <- 4
	causes[i,] <- c(2,2)
	stime[i,1:2] <- runif(2)*mt
}

}
###stime
###causes
stime <- c(t(stime))
cause <- c(t(causes))

###same.cens=TRUE
if (same.cens==TRUE) {
	ctime <- rep(rbinom(n,1,pcens),each=2) 
        ctime[ctime==1] <- rep(runif(sum(ctime==1)/2),each=2)*crate
}
else {
	ctime<- rbinom(n,1,pcens)
        ctime[ctime==1] <- runif(sum(ctime==1))*crate
}

ctime[ctime==0] <- mt;

cens <- (ctime< stime)
time <- ifelse(cens,ctime,stime)
cause <- ifelse(cens,0,cause)
id <- rep(1:n,rep(2,n))

country <- c()
country[xl==1] <- "SWE"
country[xl==2] <- "DK"
country[xl==3] <- "FIN"
country[xl==4] <- "NOR"

data<-data.frame(time=time,cause=cause,xl=rep(xl,each=2),
		 country=rep(country,each=2),id=id,cens=cens,stime=stime,type=rep(types,each=2),
		 f1=rep(F11x,each=2),p11=rep(p11,each=2),p12=rep(p12,each=2),p21=rep(p21,each=2),
		 p22=rep(p22,each=2))
return(data)
} ## }}}

##' @export 
simnordic2 <- function(n,cordz=2,cormz=3,cratemz=2,cratedz=2,pcensmz=0.8,pcensdz=0.8) 
{ ## {{{
outdz <- corsim.prostate(n,theta=cordz,crate=cratedz,pcens=pcensmz,mt=1,same.cens=TRUE,test=0) 
outmz <- corsim.prostate(n,theta=cormz,crate=cratemz,pcens=pcensdz,mt=1,same.cens=TRUE,test=0) 
outdz$zyg <- "DZ" 
outmz$zyg <-  "MZ"
outmz$id <- outmz$id+nrow(outdz)
###
out <- rbind(outdz,outmz)
out$time <- out$time*100
table(out$type,out$country)
table(out$type,out$cause)
out$country <- relevel(factor(out$country),ref="SWE")
table(out$country)
outk <- out[,c("country","cause","id","time","zyg","type")]
table(outk$cause)
table(outk$type,outk$country)
table(outk$cause,outk$country)
###

return(outk)
} ## }}}

###outk <- simnordic(2000)

trMean <- function(b,blen) {
##  mytr <- function(x) x^2; dmytr <- function(x) 2*x
  mytr <- dmytr <- exp
  if (blen==0) return(b) 
  k <- length(b)
  Bidx <- seq_len(blen)+(k-blen)
  b[Bidx[1]] <- mytr(b[Bidx[1]])
  D <- diag(nrow=k)
  D[Bidx[1]:k,Bidx[1]] <- b[Bidx[1]]
  for (i in Bidx[-1]) {
    D[i:k,i] <- dmytr(b[i])
    b[i] <- b[i-1]+mytr(b[i])
  }
  attributes(b)$D <- D
  attributes(b)$idx <- Bidx
  return(b)
}


##' @export
plot.bptwin <- function(x,n=50,rg=range(x$B[,1]),xlab="Time",ylab="Concordance",...) {
  if (x$Blen>0) {
    ##    rg <- range(x$B[,1])
    t <- seq(rg[1],rg[2],length.out=n)
    B0 <- bs(t,degree=x$Blen)
    b0. <- coef(x)[x$midx0]
    b1. <- coef(x)[x$midx1]
    b0 <- trMean(b0.,x$Blen)
    b1 <- trMean(b1.,x$Blen)
    b00 <- tail(b0,x$Blen)
    b11 <- tail(b1,x$Blen)
    pr0 <- sapply(as.numeric(B0%*%b00+b0[1]), function(z)
                  pbvn(upper=rep(z,2),sigma=x$Sigma0))
    pr1 <- sapply(as.numeric(B0%*%b11+b1[1]), function(z)
                  pbvn(upper=rep(z,2),sigma=x$Sigma1))
    plot(pr0~t,type="l", xlab=xlab, ylab=ylab,...)
    lines(pr1~t,type="l",lty=2)
  }
  return(invisible(x))
}

##' Estimates the casewise concordance based on Concordance and marginal estimate using timereg and performs test for independence
##'
##' @title Estimates the casewise concordance based on Concordance and marginal estimate using timereg and performs test for independence
##' @details Uses cluster based conservative standard errors for marginal and sometimes only the uncertainty of the concordance estimates. This works prettey well, alternatively one can use also the 
##'          funcions Casewise for a specific time point 
##' @param conc Concordance 
##' @param marg Marginal estimate
##' @param test Type of test for independence assumption. "conc" makes test on concordance scale and "case" means a test on the casewise concordance
##' @param p check that marginal probability is greater at some point than p
##' @author Thomas Scheike
##' @aliases casewise.test slope.process casewise.bin
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##' library("timereg")
##' data("prt",package="mets");
##' prt <- force.same.cens(prt,cause="status")
##' 
##' prt <- prt[which(prt$id %in% sample(unique(prt$id),7500)),]
##' ### marginal cumulative incidence of prostate cancer
##' times <- seq(60,100,by=2)
##' outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,cause=2,times=times)
##' 
##' cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
##' cifdz <- predict(outm,X=1,uniform=0,resample.iid=1)
##' 
##' ### concordance for MZ and DZ twins
##' cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
##'                  data=prt,cause=c(2,2))
##' cdz <- cc$model$"DZ"
##' cmz <- cc$model$"MZ"
##' 
##' ### To compute casewise cluster argument must be passed on,
##' ###  here with a max of 100 to limit comp-time
##' outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,
##'                  cause=2,times=times,max.clust=100)
##' cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
##' cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,
##'                 cause=c(2,2),se.clusters=outm$clusters)
##' cdz <- cc$model$"DZ"
##' cmz <- cc$model$"MZ"
##' 
##' cdz <- casewise.test(cdz,cifmz,test="case") ## test based on casewise
##' cmz <- casewise.test(cmz,cifmz,test="conc") ## based on concordance
##' 
##' plot(cmz,ylim=c(0,0.7),xlim=c(60,100))
##' par(new=TRUE)
##' plot(cdz,ylim=c(0,0.7),xlim=c(60,100))
##' 
##' slope.process(cdz$casewise[,1],cdz$casewise[,2],iid=cdz$casewise.iid)
##' 
##' slope.process(cmz$casewise[,1],cmz$casewise[,2],iid=cmz$casewise.iid)
##'
##' }
##' @export
casewise.test <- function(conc,marg,test="no-test",p=0.01)
{ ## {{{
  if (sum(marg$P1>p)==0) stop("No timepoints where marginal > ",p,"\n"); 
  time1 <- conc$time; time2 <- marg$time[marg$P1>0.01]
  mintime <- max(time1[1],time2[1])
  maxtime <- min(max(time1),max(time2))
  timer <- seq(mintime, maxtime,length=100)
  dtimer <- timer[2]-timer[1]

  margtime <- cpred(cbind(marg$time,c(marg$P1)),timer)[,2]
  concP1 <-   cpred(cbind(conc$time,c(conc$P1)),timer)[,2]
  se.margtime <- cpred(cbind(marg$time,c(marg$se.P1)),timer)
  se.concP1 <-   cpred(cbind(conc$time,c(conc$se.P1)),timer)
  outtest <- NULL
  casewise.iid <- NULL
  casewise <- concP1/margtime

  se.casewise2 <- as.matrix(se.concP1[,2]/margtime,ncol=100)
  se.margtime <- as.matrix(se.margtime[,2],ncol=100)
  se.casewise <- NULL;
  outtest <- NULL; 

  if (is.null(conc$P1.iid) || is.null(marg$P1.iid))   {
    cat("Warning, need iid decomposition for correct se's for Concordance \n");
  } else {
   conciid <- cpred(cbind(conc$time,conc$P1.iid),timer)[,-1]
   nconc <- colnames(conc$P1.iid)
   P1iid <- cpred(cbind(marg$time,marg$P1.iid),timer)[,-1]
   P1iid2  <- 2*P1iid*margtime;
   se.p2 <- apply(P1iid2^2,1,sum)^.5
   conciid <- (P1iid2[,nconc]-conciid) ### iid of conc-p1^2 test
   c.iid <- casewise.iid <- conciid/margtime - P1iid[,nconc]*casewise
   se.casewise <- apply(casewise.iid^2,1,sum)^.5
   dif.casewise.iid <- conciid/margtime  ### iid of casewise test

   if (test=="case") 
   {
        diff.iidcase <- apply(dif.casewise.iid,2,sum)*dtimer;
        sd.pepem <- sum(diff.iidcase^2)^.5
        diff.pepem <- sum((casewise-margtime))*dtimer
        z.pepem <- diff.pepem/sd.pepem
        pval.pepem <- 2*pnorm(-abs(z.pepem))
        outtest <- cbind(diff.pepem,sd.pepem,z.pepem,pval.pepem)

	### test for constant casewise concordance
	diff.const <- (casewise-mean(casewise))
	iid.constant <-  (c.iid - matrix(apply(c.iid,2,mean),nrow(c.iid),ncol(c.iid),byrow=T))
	se.const <- apply(iid.constant^2,1,sum)^.5
        test.constant <- max(abs(diff.const/se.const))
	sim.maxs <- apply(abs(iid.constant/se.const),1,max)
	pval.const <- timereg::pval(sim.maxs,test.constant)
        outtest <- cbind(diff.pepem,sd.pepem,z.pepem,pval.pepem,test.constant,pval.const)

        colnames(outtest) <- c("cum dif.","sd","z","pval","constant-case","pval") 			 
        rownames(outtest) <- "pepe-mori"

    } else if (test=="conc") {
        diff.iid <- apply(conciid,2,sum)*dtimer
        sd.pepem <- sum(diff.iid^2)^.5
        diff.pepem <- sum((concP1-margtime^2))*dtimer
        z.pepem <- diff.pepem/sd.pepem
        pval.pepem <- 2*pnorm(-abs(z.pepem))
        outtest <- cbind(diff.pepem,sd.pepem,z.pepem,pval.pepem)
        colnames(outtest) <- c("cum dif.","sd","z","pval") 			 
        rownames(outtest) <- "pepe-mori"
    }
  }

   concout <- cbind(timer,concP1,se.concP1[,2])
   colnames(concout) <- c("time","concordance","se")
   margout <- cbind(timer,margtime,se.margtime)
   colnames(margout) <- c("time","cif","se")
   casewiseout <- cbind(timer,casewise,se.casewise,se.casewise2)
   colnames(casewiseout) <- c("time","casewise","se","se2")
   difout <- cbind(timer,concP1-margtime^2,apply(conciid^2,1,sum)^.5)

  out <- list(casewise=casewiseout,marg=margout,conc=concout,casewise.iid=casewise.iid, 
	      test=outtest,mintime=mintime,maxtime=maxtime,same.cluster=TRUE,testtype=test)
  class(out) <- "casewise"
  return(out)
} ## }}}

##' @export
slope.process  <-  function(time,y,iid=NULL)
{ ## {{{ 
ctime <- scale(time)
caselm <- lm(y ~ ctime)
slope <- coef(caselm)[2]

if (!is.null(iid)) {
diff.iid <- iid 
lm.iid <- c()
for (i in 1:ncol(iid))
{
lmiid <- lm(iid[,i] ~ ctime)
lm.iid <- rbind(lm.iid,coef(lmiid))
}
se.slope <- apply(lm.iid,2,sd)^.5
} else {se.slop <- NULL}

z.slope <- slope/se.slope[2]
pval <- 2*(1-pnorm(abs(z.slope)))

out <- list(intercept=coef(caselm)[1],slope=slope,se.slope=se.slope,pval.slope=pval)
return(out)
} ## }}} 

##' .. content for description (no empty lines) ..
##'
##' @title Estimates the casewise concordance based on Concordance and marginal estimate using prodlim but no testing
##' @param conc Concordance 
##' @param marg Marginal estimate
##' @param cause.marg specififes which cause that should be used for marginal cif based on prodlim
##' @author Thomas Scheike
##' @examples
##' \donttest{ ## Reduce Ex.Timings
##' library(prodlim)
##' data(prt);
##' prt <- force.same.cens(prt,cause="status")
##' 
##' ### marginal cumulative incidence of prostate cancer##' 
##' outm <- prodlim(Hist(time,status)~+1,data=prt)
##' 
##' times <- 60:100
##' cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ")) ## cause is 2 (second cause) 
##' cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))
##' 
##' ### concordance for MZ and DZ twins
##' cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
##' cdz <- cc$model$"DZ"
##' cmz <- cc$model$"MZ"
##' 
##' cdz <- casewise(cdz,outm,cause.marg=2) 
##' cmz <- casewise(cmz,outm,cause.marg=2)
##' 
##' plot(cmz,ci=NULL,ylim=c(0,0.5),xlim=c(60,100),legend=TRUE,col=c(3,2,1))
##' par(new=TRUE)
##' plot(cdz,ci=NULL,ylim=c(0,0.5),xlim=c(60,100),legend=TRUE)
##' summary(cdz)
##' summary(cmz)
##' }
##' @export
casewise <- function(conc,marg,cause.marg)
{ ## {{{
  if (missing(cause.marg)) stop("Please specify cause of marginal (as given in Event object)")
  if ((!inherits(conc,"prodlim"))  || (!inherits(marg,"prodlim"))) stop("Assumes that both models are based on prodlim function \n"); 
  time1 <- conc$time
  time2 <- marg$time

  cause.prodlim <- match(as.character(cause.marg),levels(prodlim::getEvent(marg$model.response)))
  if (is.na(cause.prodlim)) stop("Cause did not match marginal model")
  
  mintime <- max(time1[1],time2[1])
  maxtime <- min(max(time1),max(time2))
  timer <- seq(mintime, maxtime,length=100)
  dtimer <- timer[2]-timer[1]
 
  out <- conc
  out$time <- timer
  if (inherits(marg,"comp.risk")) margtime <- cpred(cbind(marg$time,c(marg$P1)),timer)[,2] else if (inherits(marg,"prodlim")) {
	  cuminc <- data.frame(marg$cuminc)[,cause.prodlim]; 
	  se.cuminc <- data.frame(marg$se.cuminc)[,cause.prodlim]; 
	  margtime <- cpred(cbind(marg$time,c(cuminc)),timer)[,2]; 
	  se.margtime <- cpred(cbind(marg$time,c(se.cuminc)),timer)[,2]; 
  } else stop("marginal cumulative incidence comp.risk or prodlim output\n"); 

  if (inherits(conc,"comprisk")) concP1 <-  cpred(cbind(conc$time,c(conc$P1)),timer)[,2]
  else if (inherits(conc,"prodlim"))  {
	  conc.cuminc <- data.frame(conc$cuminc)[,1]
	  conc.se.cuminc <- data.frame(conc$se.cuminc)[,1]
          se.P1 <-  cpred(cbind(conc$time,conc.se.cuminc),timer)[,2]
	  concP1 <- cpred(cbind(conc$time,conc.cuminc),timer)[,2]
  }
  concordance<- cbind(timer,concP1,se.P1)
  colnames(concordance) <- c("time","cif","se.cif")

  P1 <- concP1/margtime
  se.P1 <- se.P1/margtime
  med <- (margtime>0)  & (concP1 > 0) 
  out$P1 <- P1[med]
  out$se.P1 <- se.P1[med]
  out$timer <- timer[med]


  margout <- cbind(timer,margtime,se.margtime)
  colnames(margout) <- c("time","cif","se.cif")

  probout <- cbind(out$timer,out$P1,out$se.P1)
  colnames(probout) <- c("time","casewise conc","se casewise")

  out <- list(casewise=probout,marg=margout,concordance=concordance,test=NULL)
  class(out) <- "casewise"
  return(out)
} ## }}}

##' @export
casewise.bin <- function(nc,nd)
{ ## {{{
ud <- 	glm(cbind(nc,round(0.5*nd+nc))~ +1,family=binomial())
udci <- confint(ud)
pud <- predict(ud,se.fit=TRUE,type="response")
return(list(p.casewise=pud$fit,ci.casewise=exp(udci)))
} ## }}}


##' @export
plot.casewise <- function(x,ci=NULL,lty=NULL,ylim=NULL,col=NULL,xlab="time",ylab="concordance",
                          legend=FALSE,add=FALSE, ...)
{ ## {{{
  if (is.null(col)) col <- 1:3
  if (is.null(lty)) lty <- 1:3
  if (is.null(ylim)) ylim=range(c(x$casewise[,2],x$marg[,2]))

  if (add) {
    lines(x$casewise[,1],x$casewise[,2],type="s",lty=lty[1],col=col[1],...)
  } else {
   plot(x$casewise[,1],x$casewise[,2],type="s",ylim=ylim,lty=lty[1],col=col[1],xlab=xlab,ylab=ylab,...)
  }
  if (!is.null(ci)) {
     ul <- x$casewise[,2]+qnorm(1-(1-ci)/2)* x$casewise[,3]
     nl <- x$casewise[,2]-qnorm(1-(1-ci)/2)* x$casewise[,3]
     lines(x$casewise[,1],ul,type="s",ylim=ylim,lty=lty[3],col=col[3])
     lines(x$casewise[,1],nl,type="s",ylim=ylim,lty=lty[3],col=col[3])
  }
  lines(x$marg[,1],x$marg[,2],lty=lty[2],col=col[2],type="s")

  if (legend==TRUE) legend("topleft",lty=lty[1:2],col=col[1:2],c("Casewise concordance","Marginal estimate"))
} ## }}}

##' @export
summary.casewise <- function(object,marg=FALSE,...)
{ ## {{{
   cat("Casewise concordance and standard errors \n"); 
   print(signif(cbind(object$casewise),3))
   cat("\n"); 

   if (marg==TRUE) {
      cat("Marginal cumulative incidence and standard errors \n"); 
      print(signif(cbind(object$marg),3))
   }
   print(object,...)
} ## }}}

##' prints Concordance test 
##'
##' @title prints Concordance test 
##' @param x output from casewise.test 
##' @param digits number of digits
##' @param \dots Additional arguments to lower level functions
##' @author Thomas Scheike
##' @export
print.casewise <- function(x,digits=3,...)
{ ## {{{
    cat("\n")
    if (!is.null(x$test)) {
        cat("Pepe-Mori type test for H_0: conc_1(t)= conc_2(t)\n")
        if (x$same.cluster==TRUE) cat("Assuming same clusters for the two functions\n") else 
                                                                                           cat("Assuming independence for estimators\n");
        cat(paste("Time.range =",signif(x$mintime,3),"--",signif(x$maxtime,3),"\n\n"));
        prmatrix(signif(x$test,digits))
        invisible(x)
    }    
} ## }}}

##' .. content for description (no empty lines) ..
##'
##' @title Concordance test Compares two concordance estimates
##' @param conc1 Concordance estimate of group 1
##' @param conc2 Concordance estimate of group 2
##' @param same.cluster if FALSE then groups are independent, otherwise estimates are based on same data. 
##' @author Thomas Scheike
##' @export
test.conc <- function(conc1,conc2,same.cluster=FALSE)
{ ## {{{
  time  <- time1 <- conc1$time
  time2 <- conc2$time
  mintime <- max(time1[1],time2[1])
  maxtime <- min(max(time1),max(time2))
  timer <- seq(mintime, maxtime,length=100)
  dtimer <- timer[2]-timer[1]

  conc2timer <- cpred(cbind(conc2$time,c(conc2$P1)),timer)[,2]
  conc1timer <- cpred(cbind(conc1$time,c(conc1$P1)),timer)[,2]
  outtest <- NULL

  if (is.null(conc1$P1.iid) || is.null(conc2$P1.iid)) 
	  stop("Must give iid represenation for both estimators\n");  

    if (!is.null(conc1$P1.iid) && !is.null(conc2$P1.iid))  {
    if ( ((ncol(conc1$P1.iid[,])-ncol(conc2$P1.iid[,]))!=0) && same.cluster==TRUE)
       cat("Warning, not same number of iid residuals for concordance and marginal estimate\n"); 
    }

    if (!is.null(conc1$P1.iid)) if (!is.null(conc2$P1.iid)) {
    ### iid version af integraler
      conc2P1.iid  <- cpred(cbind(conc2$time,conc2$P1.iid[,]),timer)[,-1]
      conc1P1.iid  <- cpred(cbind(conc1$time,conc1$P1.iid[,]),timer)[,-1]
    if ( (ncol(conc1$P1.iid)==ncol(conc2$P1.iid)) && same.cluster==TRUE) {
	diff.iid <- conc1P1.iid-conc2P1.iid
	sdiff.iid <- apply(diff.iid,2,sum)*dtimer
	sd.pepem <- sum(sdiff.iid^2)^.5
    } else {
	diff2.iid <- conc2P1.iid
	sdiff2.iid <- apply(diff2.iid,2,sum)*dtimer
	var2.pepem <- sum(sdiff2.iid^2)
	diff1.iid <- conc1P1.iid
	sdiff1.iid <- apply(diff1.iid,2,sum)*dtimer
	var1.pepem <- sum(sdiff1.iid^2)
	sd.pepem <- (var1.pepem+var2.pepem)^.5
    }
      diff.pepem <- sum(conc2timer-conc1timer)*dtimer
###	print(cbind(conc2timer,conc1timer))
      z.pepem <- diff.pepem/sd.pepem
      pval.pepem <- 2*pnorm(-abs(z.pepem))

      outtest <- cbind(diff.pepem,sd.pepem,z.pepem,pval.pepem)
      colnames(outtest) <- c("cum dif.","sd","z","pval") 			 
      rownames(outtest) <- "pepe-mori"
###  print(outtest,4)
###  prmatrix(outtest,3)			
    } 

  outtest <- list(test=outtest,mintime=mintime,maxtime=maxtime,same.cluster=same.cluster)
###attr(out,"class") <- rev(attr(out,"class")) 
  class(outtest) <- "testconc"
  return(outtest)
} ## }}}

##' convert to timereg object
##'
##' @title Convert to timereg object
##' @param obj no use
##' @author Thomas Scheike
##' @export
back2timereg <- function(obj)
{ ## {{{
  out <- obj
  attr(out,"class") <- rev(attr(out,"class")) 
  return(out)
} ## }}}


##' Estimates the casewise concordance based on Concordance and marginal estimate using binreg 
##'
##' @title Estimates the casewise concordance based on Concordance and marginal estimate using binreg 
##' @details Uses cluster iid for the two binomial-regression estimates  standard errors better than those of casewise that are often conservative.
##' @param concbreg Concordance 
##' @param margbreg Marginal estimate
##' @param zygs order of zygosity for estimation of concordance and casewise.
##' @param newdata to give instead of zygs.
##' @param ... to pass to estimate function
##' @author Thomas Scheike
##' @examples
##' data(prt)
##' prt <- force.same.cens(prt,cause="status")
##' 
##' dd <- bicompriskData(Event(time, status)~strata(zyg)+id(id), data=prt, cause=c(2, 2))
##' newdata <- data.frame(zyg=c("DZ","MZ"),id=1)
##' 
##' ## concordance 
##' bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id), data=dd,
##'                 time=80, cause=1, cens.model=~strata(zyg))
##' pconc <- predict(bcif1,newdata)
##' 
##' ## marginal estimates 
##' mbcif1 <- binreg(Event(time,status)~cluster(id), data=prt, time=80, cause=2)
##' mc <- predict(mbcif1,newdata)
##' mc
##' 
##' cse <- binregCasewise(bcif1,mbcif1)
##' cse
##' @export
binregCasewise <- function(concbreg,margbreg,zygs=c("DZ","MZ"),newdata=NULL,...)
{# {{{
  if (is.null(newdata)) newdata <- data.frame(zyg=zygs,id=1)

  pconc <- predict(concbreg,newdata)
  pmarg <- predict(margbreg,newdata)

  f <- function(p) {concbreg$coef <- p; return(log(predict(concbreg,newdata)[,1]))} 
  fcase <- function(p) {
      concbreg$coef <- p[1:2];  
      margbreg$coef <- p[3]; 
     res <- (log(predict(concbreg,newdata)[,1]) - log(predict(margbreg,newdata)[,1]))
     return(res)
  } 

  margiid <- margbreg$iid
  conciid <- matrix(0,nrow(margiid),nrow(pconc))
  wiid <- which(concbreg$iid.origid %in% margbreg$iid.origid)
  conciid[wiid,] <- concbreg$iid
  iids <- cbind(conciid,margiid)
  vcov <- crossprod(iids)

  dd <- estimate(coef=c(concbreg$coef,margbreg$coef),vcov=vcov,f=fcase,...)
  expcoef <- exp(dd$coefma[,c(1,3:4)])

  res <- list(coef=expcoef,logcoef=dd)
  return(res)
}# }}}

by2mat <- function(x,nam,...) {
    nulls <- which(unlist(lapply(x,is.null)))
    nonnulls <- setdiff(seq_along(x),nulls)
    nn <- do.call("expand.grid",attributes(x)$dimnames)
    if (length(nulls)>0) nn <- nn[-nulls,,drop=FALSE]
    res <- Reduce("rbind",x)
    if (is.null(colnames(res)) && !missing(nam)) {
        colnames(res) <- nam[seq(length(ncol(res)))]
    }
    suppressWarnings(res <- cbind(nn,res)) ## no warnings on row-names
    for (i in seq(ncol(res)-1)+1) {
        if (is.list(res[,i])) {
            if (!is.null(nn <- names(res[,i][[1]])))
                colnames(res)[i] <- paste0(colnames(res)[i],"(",paste0(nn,collapse=","),")")
        }
    }
    a <- rownames(x[[1]])
    res$"_var" <- a
    rownames(res) <- seq(nrow(res))
    return(res)
}


##' aggregating for for data frames
##'
##' aggregating for for data frames
##' @examples
##' data("sTRACE",package="timereg")
##' daggregate(iris, "^.e.al", x="Species", fun=cor, regex=TRUE)
##' daggregate(iris, Sepal.Length+Petal.Length ~Species, fun=summary)
##' daggregate(iris, log(Sepal.Length)+I(Petal.Length>1.5) ~ Species,
##'                  fun=summary)
##' daggregate(iris, "*Length*", x="Species", fun=head)
##' daggregate(iris, "^.e.al", x="Species", fun=tail, regex=TRUE)
##' daggregate(sTRACE, status~ diabetes, fun=table)
##' daggregate(sTRACE, status~ diabetes+sex, fun=table)
##' daggregate(sTRACE, status + diabetes+sex ~ vf+I(wmi>1.4), fun=table)
##' daggregate(iris, "^.e.al", x="Species",regex=TRUE)
##' dlist(iris,Petal.Length+Sepal.Length ~ Species |Petal.Length>1.3 & Sepal.Length>5,
##'             n=list(1:3,-(3:1)))
##' daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5))
##' daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5),
##'                  fun=table)
##'
##' dsum(iris, .~Species, matrix=TRUE, missing=TRUE)
##'
##' par(mfrow=c(1,2))
##' data(iris)
##' drename(iris) <- ~.
##' daggregate(iris,'sepal*'~species|species!="virginica",fun=plot)
##' daggregate(iris,'sepal*'~I(as.numeric(species))|I(as.numeric(species))!=1,fun=summary)
##'
##' dnumeric(iris) <- ~species
##' daggregate(iris,'sepal*'~species.n|species.n!=1,fun=summary)
##'
##' @export
##' @param data data.frame
##' @param y name of variable, or formula, or names of variables on data frame.
##' @param x name of variable, or formula, or names of variables on data frame.
##' @param subset subset expression
##' @param ... additional arguments to lower level functions
##' @param fun function defining aggregation
##' @param regex interpret x,y as regular expressions
##' @param missing Missing used in groups (x)
##' @param remove.empty remove empty groups from output
##' @param matrix if TRUE a matrix is returned instead of an array
##' @param silent suppress messages
##' @param na.action How model.frame deals with 'NA's
##' @param convert if TRUE try to coerce result into matrix. Can also be a user-defined function
##' @aliases daggr
daggregate <- function(data,y=NULL,x=NULL,subset,...,fun="summary",regex=mets.options()$regex, missing=FALSE, remove.empty=FALSE, matrix=FALSE, silent=FALSE, na.action=na.pass, convert=NULL)
{# {{{
    if (is.vector(data)) data <- data.frame(data)
    subs <- substitute(subset)
    if (!base::missing(subs)) {
        expr <- suppressWarnings(inherits(try(subset,silent=TRUE),"try-error"))
        if (expr) data <- data[which(eval(subs,envir=data)),,drop=FALSE]
        else data[subset,,drop=FALSE]
    }
    if (is.null(y)) y <- colnames(data)
    if (inherits(y,"formula")) {

        yx <- procformdata(y,sep="\\|",data=data,na.action=na.action,regex=regex,...)
        y <- yx$response
        x0 <- yx$predictor
        if (is.null(x) && length(y)>0) x <- x0
        if (NCOL(x)==0) x <- NULL
        if (length(y)==0) {
            y <- x0
        }
    } else {
        yy <- c()
        for (y0 in y) {
            if (!regex) y0 <- glob2rx(y0)
            n <- grep(y0,names(data),perl=mets.options()$regex.perl)
            yy <- union(yy,names(data)[n])
        }
        y <- data[,yy,drop=FALSE]
    }
    if (is.character(x) && length(x)<NROW(data)) {
        xx <- c()
        for (x0 in x) {
            if (!regex) x0 <- glob2rx(x0)
            n <- grep(x0,names(data),perl=mets.options()$regex.perl)
            xx <- union(xx,names(data)[n])
        }
        x <- data[,xx,drop=FALSE]
    }
    if (inherits(x,"formula")) {
        x <- model.frame(x,data=data,na.action=na.action)
    }
    if (!is.null(x)) {
        xidx <- na.omit(match(colnames(x),colnames(y)))
        if (length(xidx)>0) y <- y[,-xidx,drop=FALSE]
    }
    if (is.character(fun)) fun <- get(fun)
    if (!is.null(convert) && is.logical(convert)) {
        if (convert) convert <- as.matrix
        else convert <- NULL
    }
    if (!is.null(convert)) {
        fun_ <- fun
        fun <- function(x,...) fun_(convert(x,...))
    }
    if (!is.null(x)) {
        if (missing) {
            x[is.na(x)] <- 'NA'
        }
        if (silent) {
                capture.output(res <- by(y,x,fun,...))
        } else {
            res <- by(y,x,fun,...)
        }
        if (remove.empty) {
            # ... need to abandon 'by'?
        }
        if (matrix) {
            res <- by2mat(res,colnames(y))
        }
        return(structure(res,ngroupvar=NCOL(x),class=c("daggregate",class(res))))
    }
    if (silent)
        capture.output(res <- do.call(fun, c(list(y),list(...))))
    else
        res <- do.call(fun, c(list(y),list(...)))
    res
    structure(res, ngroupvar=0, class=c("daggregate",class(res)))
}# }}}

##' @export
daggr <- function(data,...,convert=as.matrix) daggregate(data,...,convert=convert)

##' @export
print.daggregate <- function(x,quote=FALSE,...) {
    attr(x,c("ngroupvar")) <- NULL
    class(x) <- class(x)[-1]
    print(x,quote=quote,...)
}


##' @export
dhead <- function(data,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) utils::head(z,...))

##' @export
dtail <- function(data,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) utils::tail(z,...))

##' @export
dsummary <- function(data=NULL,y=NULL,x=NULL,...) daggregate(data,y,x,fun=function(z) base::summary(z,...))

##' @export
dstr <- function(data,y=NULL,x=NULL,...) invisible(daggregate(data,y,x,fun=function(z) utils::str(z,...)))

##' @export
dunique <- function(data,y=NULL,x=NULL,...) invisible(daggregate(data,y,x,fun=function(z) base::unique(z,...)))


##' summary, tables, and correlations for data frames
##'
##' summary, tables, and correlations for data frames
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x possible group variable
##' @param use how to handle missing values
##' @param ... Optional additional arguments
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##' data("sTRACE",package="timereg")
##' dt<- sTRACE
##' dt$time2 <- dt$time^2
##' dt$wmi2 <- dt$wmi^2
##' head(dt)
##'
##' dcor(dt)
##'
##' dcor(dt,~time+wmi)
##' dcor(dt,~time+wmi,~vf+chf)
##' dcor(dt,time+wmi~vf+chf)
##'
##' dcor(dt,c("time*","wmi*"),~vf+chf)
##' @aliases dsummary dstr dcor dsubset dquantile dcount dmean dmeansd dscalar deval deval2 dsum dsd
##' @export
dcor <- function(data,y=NULL,x=NULL,use="pairwise.complete.obs",...) daggregate(data,y,x,...,fun=function(z,...) stats::cor(z,use=use,...))

##' @export
dscalar <- function(data,y=NULL,x=NULL,...,na.rm=TRUE,matrix=TRUE,fun=base::mean) {
    daggregate(data,y,x,matrix=matrix,...,
               fun=function(z,...) {
                   if (is.matrix(z)) {
                       apply(z,2,function(x)
                           suppressWarnings(tryCatch(fun(x,na.rm=na.rm,...),error=function(e) return(NA))))
                   } else {
                       unlist(lapply(z,function(x) {
                           suppressWarnings(tryCatch(fun(x,na.rm=na.rm,...),error=function(e) return(NA)))
                       }))
                   }
               })
}


Summary <- function(object,na.rm=TRUE,...) {
    if (is.numeric(object)) {
        x <- c(summary(object,...),sd=sd(object,na.rm=TRUE))
    } else {
        x <- summary(object,...)
    }
    ## Formatting
    xx <- x
    if (is.numeric(x) || is.complex(x)) {
        finite <- is.finite(x)
        xx[finite] <- zapsmall(x[finite])
      }
    m <- match("NA's", names(xx), 0)
    if (inherits(x, "Date") || inherits(x, "POSIXct")) {
        xx <- if (length(a <- attr(x, "NAs")))
                 c(format(xx), `NA's` = as.character(a))
             else format(xx)
    }
    else if (m && !is.character(x))
        xx <- c(format(xx[-m]), `NA's` = as.character(xx[m]))
    xx
}

##' @export
deval2 <- function(data,...,matrix=simplify,simplify=TRUE)  deval(data,matrix=TRUE,simplify=TRUE,...)

##' @export
deval <- function(data,y=NULL,x=NULL,...,matrix=FALSE,fun=Summary,simplify=FALSE) {
    if (is.list(fun)) {
        newf <- function(x,...) {
            unlist(lapply(fun,function(f) f(x,...), ...))
        }
    }
    else newf <- fun
    res <- daggregate(data,y,x,matrix=matrix,...,
                     fun=function(z) lapply(z,function(x) {
                         suppressWarnings(tryCatch(newf(x,...),error=function(e) return(NA)))
                     }))
    if (simplify) {
        for (i in seq_len(ncol(res))) {
            if (is.list(res[,i])) res[,i] <- unlist(res[,i])
        }
        ## }
    }
    res
}


##' @export
dmeansd <- function(data,...)  {
	mm <- dscalar(data,fun=base::mean,...)
	vv <- dscalar(data,fun=stats::sd,...)
	colnames(vv) <- paste("sd.",colnames(vv),sep="")
	cbind(mm,vv)
}


##' @export
dmean <- function(data,...) dscalar(data,fun=base::mean,...)

##' @export
dsum <- function(data,...) dscalar(data,fun=base::sum,...)

##' @export
dsd <- function(data,...) dscalar(data,fun=stats::sd,...)


##' @export
dcount <- function(data,x=NULL,...,na.rm=TRUE) {
    res <- rbind(daggregate(data,x=x,matrix=TRUE,...,fun=function(z,...) NROW(z)))
    res[is.na(res)] <- 0
    rownames(res) <- seq(nrow(res))
    colnames(res)[ncol(res)] <- "count"
    res
}


##' @export
dsubset <- function(data,...) {
    daggregate(data,...,fun=function(z) z)
}


##' @export
dquantile <- function(data,y=NULL,x=NULL,probs=seq(0,1,by=1/breaks),breaks=4,matrix=TRUE,reshape=FALSE,...,na.rm=TRUE) {
    a <- daggregate(data,y,x,matrix=FALSE,...,fun=function(z,...) apply(z,2,function(x,...) quantile(x,probs=probs,na.rm=na.rm,...)))
    if (matrix) {
        res <- by2mat(a)
        xidx <- seq_len(attr(a, "ngroupvar"))
        if (!reshape || is.null(res[,"_var"]) || length(xidx)==0) return(res)
        res <- dreshape(res, id=colnames(res)[xidx], num="_var",sep="_")
        return(res)
    }
    return(a)
}

##' @export
dreshape <- function(data,...) fast.reshape(data,...)

##' Fast reshape
##'
##' Fast reshape/tranpose of data
##' @param data data.frame or matrix
##' @param varying Vector of prefix-names of the time varying variables. Optional for Long->Wide reshaping.
##' @param id id-variable. If omitted then reshape Wide->Long. 
##' @param num Optional number/time variable
##' @param sep String seperating prefix-name with number/time
##' @param keep Vector of column names to keep
##' @param idname Name of id-variable (Wide->Long)
##' @param numname Name of number-variable (Wide->Long)
##' @param factor If true all factors are kept (otherwise treated as character)
##' @param idcombine If TRUE and \code{id} is vector of several variables, the unique id is combined from all the variables.
##' Otherwise the first variable is only used as identifier.
##' @param labelnum If TRUE varying variables in wide format (going from long->wide) are labeled 1,2,3,... otherwise use 'num' variable. In long-format (going from wide->long) varying variables matching 'varying' prefix are only selected if their postfix is a number.
##' @param labels Optional labels for the number variable
##' @param regex Use regular expressions
##' @param dropid Drop id in long format (default FALSE)
##' @param ... Optional additional arguments
##' @author Thomas Scheike, Klaus K. Holst
##' @aliases fast.reshape dreshape
##' @export
##' @examples
##' m <- lava::lvm(c(y1,y2,y3,y4)~x)
##' d <- lava::sim(m,5)
##' d
##' fast.reshape(d,"y")
##' fast.reshape(fast.reshape(d,"y"),id="id")
##' 
##' ##### From wide-format
##' (dd <- fast.reshape(d,"y"))
##' ## Same with explicit setting new id and number variable/column names
##' ## and seperator "" (default) and dropping x
##' fast.reshape(d,"y",idname="a",timevar="b",sep="",keep=c())
##' ## Same with 'reshape' list-syntax
##' fast.reshape(d,list(c("y1","y2","y3","y4")),labelnum=TRUE)
##' 
##' ##### From long-format
##' fast.reshape(dd,id="id")
##' ## Restrict set up within-cluster varying variables
##' fast.reshape(dd,"y",id="id")
##' fast.reshape(dd,"y",id="id",keep="x",sep=".")
##' 
##' #####
##' x <- data.frame(id=c(5,5,6,6,7),y=1:5,x=1:5,tv=c(1,2,2,1,2))
##' x
##' (xw <- fast.reshape(x,id="id"))
##' (xl <- fast.reshape(xw,c("y","x"),idname="id2",keep=c()))
##' (xl <- fast.reshape(xw,c("y","x","tv")))
##' (xw2 <- fast.reshape(xl,id="id",num="num"))
##' fast.reshape(xw2,c("y","x"),idname="id")
##' 
##' ### more generally:
##' ### varying=list(c("ym","yf","yb1","yb2"), c("zm","zf","zb1","zb2"))
##' ### varying=list(c("ym","yf","yb1","yb2")))
##' 
##' ##### Family cluster example
##' d <- mets:::simBinFam(3)
##' d
##' fast.reshape(d,var="y")
##' fast.reshape(d,varying=list(c("ym","yf","yb1","yb2")))
##' 
##' d <- lava::sim(lava::lvm(~y1+y2+ya),10)
##' d
##' (dd <- fast.reshape(d,"y"))
##' fast.reshape(d,"y",labelnum=TRUE)
##' fast.reshape(dd,id="id",num="num")
##' fast.reshape(dd,id="id",num="num",labelnum=TRUE)
##' fast.reshape(d,c(a="y"),labelnum=TRUE) ## New column name
##' 
##' 
##' ##### Unbalanced data
##' m <- lava::lvm(c(y1,y2,y3,y4)~ x+z1+z3+z5)
##' d <- lava::sim(m,3)
##' d
##' fast.reshape(d,c("y","z"))
##' 
##' ##### not-varying syntax:
##' fast.reshape(d,-c("x"))
##' 
##' ##### Automatically define varying variables from trailing digits
##' fast.reshape(d)
##' 
##' ##### Prostate cancer example
##' data(prt)
##' head(prtw <- fast.reshape(prt,"cancer",id="id"))
##' ftable(cancer1~cancer2,data=prtw)
##' rm(prtw)
fast.reshape <- function(data,varying,id,num,sep="",keep,
                 idname="id",numname="num",factor=FALSE,
                 idcombine=TRUE,labelnum=FALSE,labels,
                 regex=mets.options()$regex,
                 dropid=FALSE, ...) {
    if (!is.data.frame(data) & is.list(data)) {
        data <- as.data.frame(data)
    } else {
        if (NCOL(data)==1) data <- cbind(data)
    }
    nn <- colnames(data)
    if (!missing(varying)) {
        varsubst <- substitute(varying)
        if (as.character(varsubst)[1]=="-") {
            notvarying <- varsubst[[-1]]
            vars0 <- setdiff(nn,eval(notvarying,parent.frame()))
            ##numstr <- gsub("([1-9]\\d+)$","",vars0)
            ##numstr_sanspre0 <- gsub("(^0+)",num)
            varying <- unique(gsub("([1-9]|[1-9]\\d+)$","",vars0))
        }
        if (!missing(id)) varying <- setdiff(varying,id)
        if (!missing(num)) varying <- setdiff(varying,num)
    }
        
    if (missing(id)) {
        ## reshape from wide to long format. 
        nsep <- nchar(sep)
        if (missing(varying)) {#stop("Prefix of time-varying variables needed")
            ## Find all variable names with trailing digits (and leading zeros)
            vars0 <- grep("([1-9]|[1-9]\\d+)$",nn);
            varying <- unique(gsub("([1-9]|[1-9]\\d+)$","",nn[vars0]))
        }
        if (is.list(varying)) {
            orig <- varying
            for (i in seq_along(varying)) {
                elem <- varying[[i]]
                if (is.numeric(elem[i])) {
                    varying[[i]] <- colnames(data)[elem]
                }
                if (is.character(elem[i])) {
                    ii <- elem%in%colnames(data)
                    if (!all(ii)) {
                        newelem <- c()
                        for (j in seq_along(elem)) {
                            if (ii[j]) newelem <- c(newelem,elem[j])
                            else {
                                if (!regex) elem[j] <- glob2rx(elem[j])
                                newelem <- c(newelem,
                                            grep(elem[j],colnames(data),value=TRUE))
                            }
                        }
                        varying[[i]] <- newelem
                    }
                }
            }
        }
        ## nl <- as.list(seq_along(data)); names(nl) <- nn
        ## varying <- eval(substitute(varying),nl,parent.frame())
        vnames <- NULL
        ncvar <- sapply(varying,nchar)
        newlist <- c()
        numlev <- TRUE
        all_levels <- c()
        thelevels <- c()
        if (!is.list(varying)) {
            for (i in seq_len(length(varying))) {
                ii <- which(varying[i]==substr(nn,1,ncvar[i]))                
                thelevel <- substring(nn[ii],ncvar[i]+1+nsep)
                if (labelnum) {
                    ii0 <- suppressWarnings(which(!is.na(as.numeric(thelevel))))
                    ii <- ii[ii0]
                    thelevel <- thelevel[ii0]
                }
                all_levels <- union(all_levels,thelevel)
                thelevels <- c(thelevels,list(thelevel))
                suppressWarnings(tt <- as.numeric(thelevel))
                newlist <- c(newlist,list(nn[ii[order(tt)]]))
            }
            len <- unlist(lapply(newlist,length))
            for (i in seq_len(length(varying))) {
                if (len[i]<length(all_levels)) {
                    pp <- setdiff(all_levels,thelevels[[i]])
                    vv <- paste(varying[i],pp,sep=sep)
                    data[,vv] <- NA
                    newlist[[i]] <- paste(varying[i],all_levels,sep=sep)
                }                    
            }
            if (any(is.na(suppressWarnings(as.numeric(all_levels))))) {
                numlev <- FALSE
            } else {
                all_levels <- as.numeric(all_levels)
            }
            thelevels <- all_levels
            vnames0 <- names(varying) 
            vnames <- varying
            if (!is.null(vnames0)) {
                vidx <- which(vnames0!="")
                vnames[vidx] <- vnames0[vidx]
            }
            varying <- newlist
        } else {
            if (labelnum) {
                thelevels <- seq(length(varying[[1]]))
            } else {
                thelevels <- varying[[1]]
            }
        }
        if (!missing(labels)) thelevels <- labels
        is_df <- is.data.frame(data)
        oldreshape <- FALSE
        if (is_df) {
            ## D0 <- droplevels(data)[1,,drop=FALSE]
            D0 <- data[1,,drop=FALSE]
            classes <- unlist(lapply(D0,class))
            dim <- unlist(lapply(D0,NCOL))
            if (any(dim>1) || !all(classes%in%c("numeric","logical","integer","matrix","factor","character"))) { ## e.g. Surv columns 
                oldreshape <- TRUE
            } ## else {
            ##     chars <- which(classes%in%c("character"))
            ##     factors <- which(classes%in%c("factor"))
            ##     for (j in chars) data[,j] <- as.factor(data[,j])
            ##     if (length(c(chars,factors))>0) {
            ##         for (k in varying) {
            ##             if (any(nn[c(chars,factors)]%in%k)) {
            ##                 lev <- lapply(data[1,k],levels)
            ##                 allsame <- unlist(lapply(lev,function(x)
            ##                                          identical(x,lev[[1]])))
            ##                 if (!all(allsame))
            ##                     for (j in k) data[,j] <- factor(data[,j],levels=lev)
            ##             }
            ##         }
            ##         classes[chars] <- "factor"
            ##         D0 <- data[1,,drop=FALSE]
            ##     }            
            ##     data <- data.matrix(data)
            ## }
        }
        if (is.null(vnames)) {
            vnames <- unlist(lapply(varying,function(x) x[1]))
            if (!is.null(names(vnames))) vnames <- names(vnames)
        }

        if (oldreshape) {
            ## Fall-back to stats::reshape
            return(
                structure(reshape(as.data.frame(data),varying=varying,direction="long",v.names=vnames,timevar=numname,idvar=idname,...),
                          class=c("fast.reshape","data.frame"),
                          direction="wide",
                          varying=varying))
        }

        fixed <- setdiff(nn,unlist(c(varying,numname)))
        if (!missing(keep)) fixed <- intersect(fixed,c(keep,idname,numname))
        nfixed <- length(fixed)
        nvarying <- length(varying)
        nclusts <- unlist(lapply(varying,length))
        ##        nclust <- length(varying[[1]])
        nclust <- max(nclusts)
        
        if (any(nclusts!=nclust)) stop("Different length of varying vectors!")
        data <- data[,c(fixed,unlist(varying)),drop=FALSE]
        long <- as.data.frame(.Call("FastLong2",
                                    idata=data,
                                    inclust=as.integer(nclust),
                                    as.integer(nfixed),
                                    as.integer(nvarying),PACKAGE="mets"
                                    ));


        if (numname%in%fixed) {
            while (numname%in%c(fixed)) numname <- paste(numname,"_",sep="")
        }
        if (idname%in%fixed) {
            long <- long[,-(ncol(long)-1)]
            cnames <- c(fixed,vnames,numname)
        } else {
            cnames <- c(fixed,vnames,idname,numname)
        }
        ##  while (idname%in%c(fixed,vnames,numname)) idname <- paste(idname,"_",sep="")
        ##  while (numname%in%c(fixed,vnames)) numname <- paste(numname,"_",sep="")
        colnames(long) <- cnames
        if (!numlev) {
            long[,numname] <- base::factor(long[,numname],labels=thelevels)
        } else {
            if (!identical(order(thelevels),thelevels))
                long[,numname] <- thelevels[long[,numname]]
        } 

        if (is_df && factor) { ## Recreate classes            
            vars.orig <- c(fixed,unlist(lapply(varying,function(x) x[1])))
            vars.new <- c(fixed,vnames)
            factors <- which("factor"==classes[vars.orig])
            lev <- lapply(data[1,factors],levels)
            count <- 0
            for (i in factors) {
                count <- count+1
                long[,vars.new[i]] <- base::factor(long[,vars.new[i]],levels=lev[[count]])
            }
        }
        if (dropid) {
            ii <- which(colnames(long)%in%c(idname)) ##,numname
            long <- long[,-ii,drop=FALSE]
        }
        return(
            structure(long,
                      class=c("fast.reshape","data.frame"),
                      type="wide",
                      varying=varying))
    }


##################################################
### Long to wide format:
##################################################
    numvar <- idvar <- NULL 
    if (is.character(id)) {
        idvar <- id
        if (length(id)==1) {
            id <- data[,idvar,drop=TRUE]
        } else {
            if (idcombine)
                id <- interaction(as.data.frame(data[,idvar,drop=FALSE]),drop=TRUE)
            else
                id <- data[,idvar[1],drop=TRUE]
        } 
    } else {
        if (length(id)!=nrow(data)) stop("Length of ids and data-set does not agree")
    }
    
    unum <- NULL
    if (!missing(num) && !is.null(num)) {
        if (is.character(num)) {
            numvar <- num
            if (is.character(data[1,num,drop=TRUE])) {
                data[,num] <- as.factor(data[,num,drop=TRUE])
            }
            num <- as.integer(data[,num,drop=TRUE])
            if (!labelnum) unum <- sort(unique(data[,numvar,drop=TRUE]))
        } else {
            if (length(num)!=nrow(data)) stop("Length of time and data-set does not agree")
            if (!labelnum) unum <- unique(num)
        }
    } else {
        num <- NULL
    }
    
    if (any(nn=="")) data <- data.frame(data)

    clustud <- cluster.index(id,num=num)
    maxclust <- clustud$maxclust
    idclust <- clustud$idclust  
    obs1 <- clustud$firstclustid+1 ## as.vector(apply(idclust,1,function(x) na.omit(x)[1]))+1

    if (!is.null(numvar)) {
        ii <- which(colnames(data)==numvar)
        data <- data[,-ii,drop=FALSE]
    }

    if (!missing(keep)) {
        keepers <- c(keep,idvar)
        if (!missing(varying)) keepers <- c(keepers,varying)
        ii <- which(colnames(data)%in%keepers)
        data <- data[,ii,drop=FALSE]
    }

    if (missing(varying)) varying <- setdiff(colnames(data),c(idvar))
    vidx <- match(varying,colnames(data))
    N <- nrow(idclust)
    p <- length(varying)
    P <- NCOL(data)  
    fixidx <- setdiff(seq(P),vidx)
    if (is.matrix(data) || (all(apply(data[1,,drop=FALSE],2,is.numeric)) & length(unlist(data[1,]))==length(data[1,]) )) {
        ## Everything numeric - we can work with matrices
        dataw <- matrix(NA, nrow = N, ncol = p * (maxclust-1) + ncol(data))
        dataw[,fixidx] <- as.matrix(data[obs1,fixidx,drop=FALSE])
        mnames <- colnames(data)
        if (!is.null(unum)) {
            mnames[vidx] <- paste(mnames[vidx],unum[1],sep=sep)
        } else {
            mnames[vidx] <- paste(mnames[vidx],1,sep=sep)
        }
        if (p>0) {
            for (i in seq_len(maxclust)) {
                idx <- idclust[, i] + 1
                pos <- vidx
                if (i>1) {
                    pos <- P+seq(p)+p*(i-2)
                }
                dataw[which(!is.na(idx)), pos] <-
                    as.matrix(data[na.omit(idx),vidx,drop=FALSE])      
            }
            if (!is.null(unum)) {
                postn <- unum[-1]
            } else {
                postn <- seq_len(maxclust-1)+1
            }
            ##if (is.null(numname)) postn <- idlev[postn]
            mnames <- c(mnames,
                        as.vector(t(outer(postn,varying,function(x,y) paste(y,x,sep=sep)))))      
        }
        colnames(dataw) <- mnames
        return(structure(as.data.frame(dataw),class=c("fast.reshape","data.frame"),
                         varying=varying,direction="long"))
    }

    ## Potentially slower with data.frame where we use cbind
    for (i in seq_len(maxclust)) {
        if (i==1) {
            dataw <- data[obs1,,drop=FALSE]
            mnames <- names(data);
            dataw[,vidx] <- data[idclust[,i]+1,vidx,drop=FALSE]
            if (!is.null(unum)) 
                mnames[vidx] <- paste(varying,sep,unum[i],sep="")
            else 
                mnames[vidx] <- paste(mnames[vidx],sep,i,sep="")
        } else {
            dataw <- cbind(dataw,data[idclust[,i]+1,varying,drop=FALSE])
            if (!is.null(unum)) 
                mnames <- c(mnames,paste(varying,sep,unum[i],sep=""))
            else
                mnames <- c(mnames,paste(varying,sep,i,sep=""))
        }
    }
    names(dataw) <- mnames
    return(structure(dataw,class=c("fast.reshape","data.frame"),
                     varying=varying,type="long"))
} 

simple.reshape <- function (data, id = "id", num = NULL) {
    cud <- cluster.index(data[, c(id)], num = num, Rindex = 1)
    N <- nrow(cud$idclust)
    p <- ncol(data)
    dataw <- matrix(NA, nrow = N, ncol = p * cud$maxclust)
    for (i in seq_len(cud$maxclust)) {
        dataw[, seq(p) + (i - 1) * p] <- as.matrix(data[cud$idclust[, i] + 1, ])
    }
    colnames(dataw) <- paste(names(data), rep(seq_len(cud$maxclust), each = p), sep = ".")
    return(dataw)
}

##' Estimation of twostage model with cluster truncation in bivariate situation
##'
##' @title Estimation of twostage model with cluster truncation in bivariate situation
##' @param survformula Formula with survival model aalen or cox.aalen, some limitiation on model specification due to call of fast.reshape (so for example interactions and * and : do not work here, expand prior to call)
##' @param data Data frame
##' @param theta.des design for dependence parameters in two-stage model
##' @param clusters clustering variable for twins
##' @param var.link exp link for theta
##' @param Nit number of iteration
##' @param final.fitting TRUE to do final estimation with SE and ... arguments for marginal models
##' @param ... Additional arguments to lower level functions
##' @author Thomas Scheike
##' @export
##' @examples
##' library("timereg")
##' data(diabetes)
##' v <- diabetes$time*runif(nrow(diabetes))*rbinom(nrow(diabetes),1,0.5)
##' diabetes$v <- v
##'
##' aout <- twin.clustertrunc(Surv(v,time,status)~1+treat+adult,
##'			 data=diabetes,clusters="id")
##' aout$two        ## twostage output
##' par(mfrow=c(2,2))
##' plot(aout$marg) ## marginal model output
##'
##' out <- twin.clustertrunc(Surv(v,time,status)~1+prop(treat)+prop(adult),
##'			 data=diabetes,clusters="id")
##' out$two        ## twostage output
##' plot(out$marg) ## marginal model output
twin.clustertrunc <- function(survformula,data=parent.frame(),theta.des=NULL,clusters=NULL,var.link=1,
		      Nit=10,final.fitting=FALSE,...)
{ ## {{{
## {{{  adding names of covariates from survival model to data frame if needed
## adds names that are not in data (typically intercept from additive) or
### expansion of factors,
## also reducing only to needed covariates

survnames <-  all.vars(update(survformula, .~0))
if (length(survnames)!=3) stop("Must give entry, exit and status")
entry <- survnames[1]
exit <- survnames[2]
status <- survnames[3]

tss <- terms(survformula)
Znames <- attr(tss,"term.labels")
ZnamesO <- Znames
### checks if cluster is given in survformula then removes
clustervar <- grep("^cluster[(][A-z0-9._:]*",Znames,perl = TRUE)
if (length(clustervar)>=1) Znames <- Znames[-clustervar]
propvar <- grep("^prop[(][A-z0-9._:]*",Znames,perl = TRUE)
if (length(propvar)>=1) model <- "cox.aalen" else model <- "aalen"

### removes prop from names
if (model=="cox.aalen") {
   Zn <- c()
   nn <- length(Znames)
   ### droppe prop() for navne
   for (i in 1:nn) Zn <- c(Zn,substr(Znames[i],6,nchar(Znames[i])-1))
   Znames <- Zn
}

clust.orig <- data[,clusters]
d0 <- data[,c(entry,exit,status,clusters,Znames)]
data <- d0
data$dataid <- 1:nrow(data)

d2 <- fast.reshape(data,id=clusters)
d2 <- na.omit(d2)
### only double entry people
data <- fast.reshape(d2,labelnum=TRUE)

des <- timereg::aalen.des(survformula,data=data,model=model)
factornamesX  <-  !(des$covnamesX %in% Znames)
colnames(des$X) <- des$covnamesX
if (sum(factornamesX)>=1) data <- cbind(data,des$X[,factornamesX,drop=FALSE])
if (model=="cox.aalen") {
	factornamesZ   <-  !(des$covnamesZ %in% Znames)
        colnames(des$Z) <- des$covnamesZ
        if (sum(factornamesZ)>=1) data <- cbind(data,des$Z[,factornamesZ,drop=FALSE])
}
namesX <- des$covnamesX
namesZ <- des$covnamesZ

pweight <- rep(1,nrow(data))
if (!is.null(clusters)) nclusters <- data[,clusters] else stop("must give clusters\n");
if (is.null(theta.des)) ptheta <- 0 else ptheta <- rep(0,ncol(theta.des))

###singletons might be dropped so same for theta.des
theta.des <- theta.des[data$dataid,]
## }}}

  assign("pweight",pweight,envir=environment(survformula))

for (i in 1:Nit)
{ ## {{{
  if (model=="cox.aalen") { aout <- timereg::cox.aalen(survformula,data=data,weights=1/pweight,robust=0,n.sim=0,beta=0);
                            beta <- c(aout$gamma,aout$cum[,-1])
  }  else  {
           aout <- timereg::aalen(survformula,data=data,weights=1/pweight,robust=0,n.sim=0);
           beta <- aout$cum[,-1]
  }
  if (i==1) {
   if (model=="cox.aalen") pbeta <- 0*c(aout$gamma,aout$cum[,-1]) else pbeta <- 0*aout$cum[,-1]
  }
  if (i>=2) tout <- twostage(aout,data=data,clusters=nclusters,theta.des=theta.des,theta=ptheta,var.link=var.link)
  else tout <- twostage(aout,data=data,clusters=nclusters,theta.des=theta.des,var.link=var.link)
  if (!is.null(theta.des)) theta <- c(theta.des %*% tout$theta) else theta <- tout$theta
###  if (attr(tout, "var.link") == 1) theta <- exp(tout$theta)
  data$thetades  <- c(theta)
  data$delay <- tout$marginal.trunc
  data$surv <-  tout$marginal.surv

  dd <- fast.reshape(data,id=clusters)

  v1 <- dd[,paste(entry,"1",sep="")];
  v2 <- dd[,paste(entry,"2",sep="")]
  time1 <- dd[,paste(exit,"1",sep="")];
  time2 <- dd[,paste(exit,"2",sep="")]
  status1 <- dd[,paste(status,"1",sep="")];
  status2 <- dd[,paste(status,"2",sep="")]
  nn <- nrow(dd)

  ppv1t2 <- .Call("claytonoakesR",dd$thetades1,rep(0,nn),status2,dd$delay1,dd$surv2,var.link,PACKAGE="mets")$like
  ppv1t2[status2==0] <- ppv1t2[dd$status2==0]/dd$surv2[status2==0]
  ppt1v2 <- .Call("claytonoakesR",dd$thetades1,dd$status1,rep(0,nn),dd$surv1,dd$delay2,var.link,PACKAGE="mets")$like
  ppt1v2[status1==0] <- ppt1v2[status1==0]/dd$surv1[status1==0]
  dd$weight1 <- c(ppt1v2)
  dd$weight2 <- c(ppv1t2)
  nn <- nrow(dd)

  dd2 <- fast.reshape(dd,labelnum=TRUE)
  pweight <- dd2$weight
  dtheta <- sum(abs(tout$theta-ptheta))
  dmarg <- sum(abs(beta-pbeta))
  if ((dtheta+dmarg) < 0.001) break;
  ptheta <- tout$theta
  if (model=="aalen") pbeta <- aout$cum[,-1] else  pbeta <- c(aout$gamma,aout$cum[,-1])
} ## }}}

if (final.fitting==TRUE) {  ## {{{
  if (model=="cox.aalen") aout <- timereg::cox.aalen(survformula,data=data,weights=1/pweight,n.sim=0,...) else
           aout <- timereg::aalen(survformula,data=data,weights=1/pweight,n.sim=0,...)
  tout <- twostage(aout,data=data,clusters=nclusters,theta.des=theta.des,var.link=var.link)
} ## }}}

res <- list(marg=aout,two=tout,marg.weights=pweight,dtheta=dtheta,dmarg=dmarg,model=model)

return(res)
} ## }}}

###twin.dobdata <- function(survformula,data=data,clusters=NULL,
###                         entry="v",exit="time",status="status")
###{ ## {{{
##### {{{  adding names of covariates from survival model to data frame if needed
##### adds names that are not in data (typically intercept from additive) model
##### also reducing only to needed covariates
###
###d0 <- data[,c(entry,exit,status,clusters)]
###
###model <- "aalen"
###des <- timereg::aalen.des(survformula,data=data,model=model)
###X <- des$X
######med <- des$covnamesX %in% names(data)
###   colnames(X) <- des$covnamesX
###   d0 <- cbind(d0,X)
###
###data <- d0
###
###d2 <- fast.reshape(data,id=clusters)
###d2 <- na.omit(d2)
###data <- fast.reshape(d2,numlabel=TRUE)
###
######theta.des <- data.frame(theta.des)
######theta.des$clusters <- data[,clusters]
######
######t2 <- fast.reshape(theta.des,id=clusters)
######t2 <- na.omit(t2)
######theta.des <- fast.reshape(t2,numlabel=TRUE)
###
###return(data)
###}
###

##' @export
pbvn <- function(upper,rho,sigma) {
    if (!missing(sigma)) {
        rho <- cov2cor(sigma)[1,2]
        upper <- upper/diag(sigma)^0.5
    }
    arglist <- list("bvncdf",
                    a=upper[1],
                    b=upper[2],
                    r=rho,
                    PACKAGE="mets")
    res <- do.call(".Call",arglist)
    return(res)
}


##' Multivariate normal distribution function
##'
##' Multivariate normal distribution function
##' @aliases pmvn pbvn loglikMVN scoreMVN dmvn rmvn
##' @export
##' @examples
##' lower <- rbind(c(0,-Inf),c(-Inf,0))
##' upper <- rbind(c(Inf,0),c(0,Inf))
##' mu <- rbind(c(1,1),c(-1,1))
##' sigma <- diag(2)+1
##' pmvn(lower=lower,upper=upper,mu=mu,sigma=sigma)
##' @param lower lower limits
##' @param upper upper limits
##' @param mu mean vector
##' @param sigma variance matrix or vector of correlation coefficients
##' @param cor if TRUE sigma is treated as standardized (correlation matrix)
pmvn <- function(lower,upper,mu,sigma,cor=FALSE) {
    if (missing(sigma)) stop("Specify variance matrix 'sigma'")
    if (missing(lower)) {
        if (missing(upper)) stop("Lower or upper integration bounds needed")
        lower <- upper; lower[] <- -Inf
    }
    p <- ncol(rbind(lower))
    if (missing(upper)) {
        upper <- lower; upper[] <- Inf
    }
    if (missing(mu)) mu <- rep(0,p)
    sigma <- rbind(sigma)
    ncor <- p*(p-1)/2
    if (ncol(sigma)!=p && ncol(sigma)!=ncor)
        stop("Incompatible dimensions of mean and variance")    
    if (ncol(rbind(lower))!=p || ncol(rbind(upper))!=p)
        stop("Incompatible integration bounds")    
    arglist <- list("pmvn0",
                   lower=rbind(lower),
                   upper=rbind(upper),
                   mu=rbind(mu),
                   sigma=rbind(sigma),
                   cor=as.logical(cor[1]),
                   PACKAGE="mets")
    res <- do.call(".Call",arglist)
    return(as.vector(res))
}


introotpn <- function(p) {
    ## Find integer root of x^2-x-2*p=0
    n <- 0.5*(1+sqrt(1+8*p))
    if (floor(n)!=n) n <- NA
    return(n)
}

##' @export
rmvn <- function(n,mu,sigma,rho,...) {
    if (!missing(rho)) {
        if (is.vector(rho)) rho <- rbind(rho)
        if (missing(mu)) {
            p <- introotpn(NCOL(rho))
            mu <- rep(0,p)
        }
        return (.Call("_mets_rmvn",
                 n=as.integer(n),
                 mu=rbind(mu),
                 rho=rbind(rho)))
    }
    if (!missing(mu) && missing(sigma)) sigma <- diag(nrow=length(mu))
    if (missing(sigma)) sigma <- matrix(1)
    if (is.vector(sigma)) sigma <- diag(sigma,ncol=length(sigma))
    if (missing(mu)) mu <- rep(0,ncol(sigma))    
    PP <- with(svd(sigma), v%*%diag(sqrt(d),ncol=length(d))%*%t(u))
    res <- matrix(rnorm(ncol(sigma)*n),ncol=ncol(sigma))%*%PP
    if (NROW(mu)==nrow(res) && NCOL(mu)==ncol(res)) return(res+mu)
    return(res+cbind(rep(1,n))%*%mu)
}

##' @export
dmvn <- function(x,mu,sigma,rho,log=FALSE,nan.zero=TRUE,...) {    
    if (!missing(rho)) {
        if (is.vector(rho)) rho <- rbind(rho)
        if (is.vector(x)) x <- rbind(x)
        if (missing(mu)) {
            p <- NCOL(x)
            mu <- rep(0,p)
        }
        res <- .Call("_mets_dmvn",
                    u=x,
                    mu=rbind(mu),
                    rho=rho)
        if (!log) res <- exp(res)
        return(res)
    }
    if (!missing(mu) && missing(sigma)) sigma <- diag(nrow=length(mu))
    if (missing(sigma)) sigma <- matrix(1)
    if (is.vector(sigma)) sigma <- diag(sigma,ncol=length(sigma))
    if (missing(mu)) mu <- rep(0,ncol(sigma))
    
    if (length(sigma)==1) {
        k <- 1
        isigma <- structure(cbind(1/sigma),det=as.vector(sigma))

    } else {
        k <- ncol(sigma)
        isigma <- Inverse(sigma)
    }
    if (!missing(mu)) {
        if (NROW(mu)==NROW(x) && NCOL(mu)==NCOL(x)) {
            x <- x-mu
        } else {
            x <- t(t(x)-mu)
        }
    }
    logval <- -0.5*(base::log(2*base::pi)*k+
                    base::log(attributes(isigma)$det)+
                    rowSums((x%*%isigma)*x))
    if (nan.zero) logval[is.nan(logval)] <- -Inf
    if (log) return(logval)
    return(exp(logval))
}

## Censored regression (loglikMVN)
library("tinytest")

cens1 <- function(threshold, type='right') {
  function(x) {
    x <- unlist(x)
    if (type=='left')
      return( survival::Surv(pmax(x,threshold), x>=threshold, type='left') )
      return ( survival::Surv(pmin(x,threshold), x<=threshold) )
  }
}

m0 <- lava::lvm() |>
  lava::covariance(y1 ~ y2, value='r') |>
  lava::regression(y1 + y2 ~ x)
m0 <-  transform(m0, s1 ~ y1, cens1(-2, 'left')) |>
  transform(s2 ~ y2, cens1(2,  'right'))

test_estimatenormal <- function() {
  d <- lava::sim(m0, 500, p=c(r=0.9), seed=1)

  m3 <- lava::lvm() |>
    lava::regression(y1 + s2 ~ x) |>
    lava::covariance(y1 ~ s2, constrain=TRUE, rname='z')

  e3 <- lava::estimate(m3, d)
  expect_true(mean(lava::score(e3)**2)<1e-2)
}
test_estimatenormal()


## pmvn

test_mvn <- function() {
  S <- diag(2)+0.5
  p2 <- mvtnorm::pmvnorm(lower = c(1, 1), upper = c(2, 2), sigma = S)
  p1 <- mvtnorm::pmvnorm(lower = 1, upper = 2, sigma = S[1])
  z2 <- pmvn(lower = c(1, 1), upper = c(2, 2), sigma = S)
  z1 <- pmvn(lower = 1, upper = 2, sigma = S[1])
  # check bivariate
  expect_equivalent(p2, z2)
  # check univariate
  expect_equivalent(p1, z1)
  # check vectorized versions
  z <- pmvn(upper=cbind(c(-3:3)), sigma = 1)
  expect_equivalent(z, pnorm(-3:3))
}

##' Fast approximation
##' 
##' Fast approximation
##' @param time Original ordered time points
##' @param new.time New time points
##' @param equal If TRUE a list is returned with additional element
##' @param type Type of matching, nearest index, nearest greater than
##'     or equal (right), number of elements smaller than y otherwise
##'     the closest value above new.time is returned.
##' @param sorted Set to true if new.time is already sorted
##' @param ... Optional additional arguments
##' @author Klaus K. Holst
##' @examples
##' id <- c(1,1,2,2,7,7,10,10)
##' fast.approx(unique(id),id)
##' 
##' t <- 0:6
##' n <- c(-1,0,0.1,0.9,1,1.1,1.2,6,6.5)
##' fast.approx(t,n,type="left")
##' @aliases  indexstrata predictCumhaz cpred
##' @export
fast.approx <- function(time,new.time,equal=FALSE,type=c("nearest","right","left"),sorted=FALSE,...) {# {{{
    if (!sorted) {
        ord <- order(new.time,decreasing=FALSE)
        new.time <- new.time[ord]
    }
    A <- NULL
    if (NCOL(time)>1) {
        A <- time
        time <- A[,1,drop=TRUE]
    }
    if (is.unsorted(time)) warnings("'time' will be sorted")
    type <- agrep(type[1],c("nearest","right","left"))-1
    arglist <- list("FastApprox",
                    time=sort(time),
                    newtime=new.time,
                    equal=equal,
                    type=type,
                    PACKAGE="mets")
    res <- do.call(".Call",arglist)
    if (!sorted) {
        oord <- order(ord)
        if (!equal) return(res[oord])
        res <- lapply(res,function(x) x[oord])
    }
    if (!is.null(A)) {
        A[res,,drop=FALSE]
    }
    return(res)
}# }}}

##' @export
predictCumhaz <- function (cum, new.time, type = c("left", "right", "nearest"),tminus=TRUE,tplus=TRUE,...)
{# {{{
   if (NCOL(cum)>1) { cumh <- cum[,-1,drop=FALSE]; time <- cum[,1]} else time <- cum
   index <- fast.approx(time,new.time, type = type[1],equal=TRUE,...)
   equali <- which(index$eq!=0)
   index <- index$idx
   if (length(equali)>=1 & tminus & type[1]=="left") {
	   index[equali] <- index[equali] -1
   }
   if (length(equali)>=1 & tplus & type[1]=="right") {
	   index[equali] <- pmin(length(time),index[equali] +1)
   }
   if (any(index==0))  { 
      index[index==0] <- 1 
   }
   if (NCOL(cum)>1) {
	   res <- cbind(new.time, cumh[index,])
   } else res <-  index
   return(res)
}# }}}

##' @export
cpred <- function(...) predictCumhaz(...)

##' @export
indexstrata <- function(jump.times,jump.strata,eval.times,eval.strata,nstrata,equal=FALSE,
			type=c("nearest","right","left"),sorted=FALSE,start.time=NULL)
{   # {{{
	stopifnot(is.numeric(jump.times))
	stopifnot(is.numeric(eval.times))
	if (any(eval.strata<0) | any(eval.strata>nstrata-1)) stop("eval.strata index not ok\n"); 
	if (any(jump.strata<0) | any(jump.strata>nstrata-1)) stop("jump.strata index not ok\n"); 
	N <- length(jump.times)
	index <- rep(0,length(eval.times))

	for (i in unique(eval.strata)) {
	   wherej <- which(eval.strata==i)
	   whereJ <- which(jump.strata==i)
	   if (!is.null(start.time)) iindex <- fast.approx(c(start.time,jump.times[whereJ]),eval.times[wherej],equal=equal,type=type,sorted=sorted) 
	   else iindex <- fast.approx(jump.times[whereJ],eval.times[wherej],equal=equal,type=type,sorted=sorted) 
	iindexn0 <- which(iindex!=0)
        if (length(iindexn0)>0)
	   index[wherej[iindexn0]] <- whereJ[iindex[iindexn0]]
	}
	return(index)
}# }}}

##' Event split with two time-scales, time and gaptime 
##'
##' splits after cut times for the two time-scales. 
##'
##' @param data data to be split
##' @param time time variable.
##' @param status status variable.
##' @param entry name of entry variable.
##' @param cuts cuts variable or numeric cut (only one value)
##' @param name.id name of id variable.
##' @param gaptime gaptime variable.
##' @param gaptime.entry name of entry variable for gaptime.
##' @param cuttime to cut after time or gaptime
##' @param cens.code code for the censoring.
##' @param order.id order data after id and start.
##' @author Thomas Scheike
##' @keywords survival
##' @examples
##' rr  <- data.frame(time=c(500,1000),start=c(0,500),status=c(1,1),id=c(1,1))
##' rr$gaptime <-  rr$time-rr$start
##' rr$gapstart <- 0
##'
##' rr1 <- EventSplit(rr,cuts=600,cuttime="time",   gaptime="gaptime",gaptime.entry="gapstart")
##' rr2 <- EventSplit(rr1,cuts=100,cuttime="gaptime",gaptime="gaptime",gaptime.entry="gapstart")
##'
##' dlist(rr1,start-time+status+gapstart+gaptime~id)
##' dlist(rr2,start-time+status+gapstart+gaptime~id)
##'
##' @export 
EventSplit <- function(data,
		time="time",status="status",entry="start",cuts="cuts",name.id="id",
		gaptime=NULL,gaptime.entry=NULL,cuttime=c("time","gaptime"),
		cens.code=0,order.id=TRUE)
{
## {{{ 
    n <- nrow(data)
    new.time <- data[,time]
    new.status <- data[,status]

    if (!is.null(gaptime)) { new.gaptime <- data[,gaptime]; nngap <- new.gaptime;}
    if (!is.null(gaptime.entry)) new.gapstart <- data[,gaptime.entry]

    if (is.numeric(cuts)) {
	    new.cuts <- rep(cuts,nrow(data))
    } else { new.cuts  <-  data[,cuts] }

    if (is.numeric(entry)) {
	    start0 <- entry
	    name.entry <- paste("start",entry,sep=".")
            data[,name.entry] <- start0
    }  else name.entry <- entry

    if ((name.entry %in% names(data))) {
      new.start <- data[,name.entry]
    } else new.start <- rep(0,n)
    nnstart <- new.start

    if (any(new.start>= new.time)) cat("any(new.start>= new.time) is TRUE\n"); 

    if ((name.id %in% names(data))) idl <- data[,name.id] else {
	    idl <- 1:n
	    data[,name.id] <- idl 
    }

    if (cuttime[1]=="time") {# {{{
    splits <- which(new.cuts<new.time & new.start<new.cuts)

    if (length(splits)) {
	    rows  <- c(1:n,splits)
	    new.time <-   c(new.time,new.time[splits])
	    new.start <-  c(new.start,new.cuts[splits])
	    new.time[splits] <- new.cuts[splits]
	    new.status <- c(new.status,new.status[splits])
	    new.status[splits] <- cens.code
	    idl <- c(idl,idl[splits]) 

	    if (!is.null(gaptime)) {
	       new.gapstart <- c(new.gapstart,new.cuts[splits]-nnstart[splits]+new.gapstart[splits])
               new.gaptime <-   c(new.gaptime,new.gaptime[splits])
	       new.gaptime[splits] <- new.cuts[splits]- nnstart[splits]+new.gapstart[splits]
            }

	    data <- data[rows,]
	    data[,time] <- new.time
	    data[,status] <- new.status
	    data[,name.entry] <- new.start
	    data[,name.id] <- idl

	    if (!is.null(gaptime)) data[,gaptime]  <-  new.gaptime 
	    if (!is.null(gaptime)) data[,gaptime.entry] <- new.gapstart
    }

    if (order.id) data <- data[order(idl,new.start),] # }}}
    } else {# {{{
    splits <- which(new.cuts<new.gaptime & new.gapstart<new.cuts)

    if (length(splits)) {
        nord <- c(1:n,(1:n)[splits]+0.1)
	rows  <- c(1:n,splits)
	new.gaptime <-   c(new.gaptime,new.gaptime[splits])
        new.gapstart <-  c(new.gapstart,new.cuts[splits])
        new.gaptime[splits] <- new.cuts[splits]
        new.status <- c(new.status,new.status[splits])
        new.status[splits] <- cens.code
        idl <- c(idl,idl[splits]) 

        new.time  <- c(new.time,new.time[splits])
        new.time[splits] <- new.start[splits]+(new.gaptime[splits]-new.gapstart[splits])
        new.start  <-   c(new.start,new.start[splits]+(new.gaptime[splits]-new.gapstart[splits]))

        data <- data[rows,]
        data[,time] <- new.time
        data[,status] <- new.status
        data[,name.entry] <- new.start
        data[,name.id] <- idl
        ###
        data[,gaptime]  <-  new.gaptime 
        data[,gaptime.entry] <- new.gapstart
        if (order.id) data <- data[order(idl,new.start),] 
       }

    }# }}}

    return(data)
    ## }}} 
} 

##' Cutting, sorting, rm (removing), rename for data frames
##'
##' Cut variables, if breaks are given these are used, otherwise cuts into 
##' using group size given by probs, or equispace groups on range. Default 
##' is equally sized groups if possible
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x name of variable, or fomula, or names of variables on data frame.
##' @param probs groups defined from quantiles
##' @param breaks  number of breaks, for variables or vector of break points,
##' @param equi for equi-spaced breaks  
##' @param regex for regular expressions.
##' @param sep seperator for naming of cut names.
##' @param na.rm to remove NA for grouping variables.
##' @param labels to use for cut groups 
##' @param all to do all variables, even when breaks are not unique 
##' @param ... Optional additional arguments
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##' data("sTRACE",package="timereg")
##' sTRACE$age2 <- sTRACE$age^2
##' sTRACE$age3 <- sTRACE$age^3
##'
##' mm <- dcut(sTRACE,~age+wmi)
##' head(mm)
##'
##' mm <- dcut(sTRACE,catage4+wmi4~age+wmi)
##' head(mm)
##'
##' mm <- dcut(sTRACE,~age+wmi,breaks=c(2,4))
##' head(mm)
##'
##' mm <- dcut(sTRACE,c("age","wmi"))
##' head(mm)
##'
##' mm <- dcut(sTRACE,~.)
##' head(mm)
##'
##' mm <- dcut(sTRACE,c("age","wmi"),breaks=c(2,4))
##' head(mm)
##'
##' gx <- dcut(sTRACE$age)
##' head(gx)
##'
##'
##' ## Removes all cuts variables with these names wildcards
##' mm1 <- drm(mm,c("*.2","*.4"))
##' head(mm1)
##'
##' ## wildcards, for age, age2, age4 and wmi
##' head(dcut(mm,c("a*","?m*")))
##'
##' ## with direct asignment
##' drm(mm) <- c("*.2","*.4")
##' head(mm)
##'
##' dcut(mm) <- c("age","*m*")
##' dcut(mm) <- ageg1+wmig1~age+wmi
##' head(mm)
##'
##' ############################
##' ## renaming
##' ############################
##'
##' head(mm)
##' drename(mm, ~Age+Wmi) <- c("wmi","age")
##' head(mm)
##' mm1 <- mm
##'
##' ## all names to lower
##' drename(mm1) <- ~.
##' head(mm1)
##'
##' ## A* to lower
##' mm2 <-  drename(mm,c("A*","W*"))
##' head(mm2)
##' drename(mm) <- "A*"
##' head(mm)
##'
##' dd <- data.frame(A_1=1:2,B_1=1:2)
##' funn <- function(x) gsub("_",".",x)
##' drename(dd) <- ~.
##' drename(dd,fun=funn) <- ~.
##' names(dd)
##' @aliases dcut dcut<- dunique drm drm<- dnames dnames<- drename drename<- dkeep dkeep<- ddrop ddrop<- 
##' @export
dcut <- function(data,y=NULL,x=NULL,breaks=4,probs=NULL,equi=FALSE,regex=mets.options()$regex,sep=NULL,na.rm=TRUE,labels=NULL,all=FALSE,...)
{# {{{
    if (is.vector(data)) {# {{{
	if (is.list(breaks)) breaks <- unlist(breaks)

        if (length(breaks)==1) { 
            if (!is.null(probs))
            {
                breaks <- quantile(data, probs, na.rm=na.rm, ...)
            } else {
	   	if (!equi) { 
                    probs <- seq(0, 1, length.out = breaks + 1)
                    breaks <- quantile(data, probs,na.rm=na.rm, ...)
		} 
		if (equi) { 
                    rr <- range(data,na.rm=na.rm)
                    breaks <-  seq(rr[1],rr[2],length.out=breaks+1)
		}
            }
	}

        if (sum(duplicated(breaks))==0)
            gx <- cut(data, breaks = breaks, include.lowest = TRUE,labels=labels,...)
        else {
            wd <- which(duplicated(breaks))
            mb <- min(diff(breaks[-wd]))
            breaks[wd] <- breaks[wd] +  (mb/2)*seq(length(wd))/length(wd)
            gx  <- cut(data,breaks=breaks,include.lowest=TRUE,labels=labels,...)
            warning(paste("breaks duplicated"))
        }
        return(gx)
    }# }}}

    if (is.data.frame(data)) {# {{{

        if (is.null(sep)) sep <- "cat."

        usernames <- FALSE# {{{

        vars <-mets::procform3(y,x,data=data,regex=regex,...)
        x <-  xnames <- vars$x

        if (!is.null(vars$y)) {
            usernames<-TRUE
            newnames <- vars$y
            if (length(vars$y)!=length(vars$x)) { 
                warning("length of new names not consistent with length of cut variables, uses default naming\n"); 
                usernames <- FALSE
            }
        }
                                        # }}}

        if (is.character(x) && length(x)<nrow(data)) x <- lapply(xnames,function(z) data[,z])
        dots <- list()
        args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })

        if (!is.list(x)) x <- list(x)
        ll <- length(x)
        if (ll==1 & !is.list(breaks) & length(breaks)>1) breaks <- list(breaks)

        break.points <- FALSE
        if (is.list(breaks)) {
            break.points <- TRUE
            if (length(x)!=length(breaks) & length(breaks)!=1) 
                warning("length of variables not consistent with list of breaks"); 
            if (length(breaks)!=ll) breaks <- rep(list(breaks[[1]]),ll)
        }

        if (!break.points) {
            if (length(x)!=length(breaks) & length(breaks)!=1) 
                warning("length of variables not consistent with breaks"); 
            if (length(breaks)!=ll) breaks<- rep(breaks[1],ll)
        }

        if (ll==1 & !is.list(labels)) labels <- list(labels)
        if (!is.list(labels)) labels <- list(labels); 
        if (length(labels)!=ll ) labels <- rep(list(labels[[1]]),ll)
        if (!is.list(labels)) stop("labels should be given as list"); 


        for (k in 1:ll)
        {
            xx <- x[[k]]
            if (is.numeric(xx)) {

                if (!is.list(breaks))
                {
                    if (!is.null(probs))
                    {
                        bb <- quantile(xx, probs,na.rm=na.rm, ...)
                    } else {
                        if (!equi) { 
                            probs <- seq(0, 1, length.out = breaks[k] + 1)
                            bb <- quantile(xx, probs, na.rm=na.rm,...)
                        } 
                        if (equi) { 
                            rr <- range(xx,na.rm=na.rm)
                            bb <-  seq(rr[1],rr[2],length.out=breaks[k]+1)
                        }
                    }
                    name<-paste(xnames[k],breaks[k],sep=sep)
                } else { bb <- breaks[[k]]; name<-paste(xnames[k],breaks[[k]][1],sep=sep) }

                if (usernames) name <- newnames[k]

                if (sum(duplicated(bb))==0)
                    data[,name] <- cut(xx,breaks=bb,include.lowest=TRUE,labels=labels[[k]],...)
                else { 
                    if (all==TRUE) {
                        wd <- which(duplicated(bb))
                        mb <- min(diff(bb[-wd]))
                        bb[wd] <- bb[wd] +  (mb/2)*seq(length(wd))/length(wd)
                        data[,name] <- cut(xx,breaks=bb,include.lowest=TRUE,labels=labels[[k]],...)
                        warning(paste("breaks duplicated for=",xnames[k]))
                    }
                }
            }
        }

        return(data)
    }# }}}

}# }}}

##' @export
"dcut<-" <- function(data,...,value) dcut(data,y=value,...)

##' relev levels for data frames
##'
##' levels shows levels for variables in data frame, relevel relevels a factor in data.frame 
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x name of variable, or fomula, or names of variables on data frame.
##' @param ref new reference variable 
##' @param newlevels to combine levels of factor in data frame
##' @param regex for regular expressions.
##' @param sep seperator for naming of cut names.
##' @param overwrite to overwrite variable 
##' @param ... Optional additional arguments
##' @author Klaus K. Holst and Thomas Scheike
##' @examples
##'
##' data(mena)
##' dstr(mena)
##' dfactor(mena)  <- ~twinnum
##' dnumeric(mena) <- ~twinnum.f
##' 
##' dstr(mena)
##' 
##' mena2 <- drelevel(mena,"cohort",ref="(1980,1982]")
##' mena2 <- drelevel(mena,~cohort,ref="(1980,1982]")
##' mena2 <- drelevel(mena,cohortII~cohort,ref="(1980,1982]")
##' dlevels(mena)
##' dlevels(mena2)
##' drelevel(mena,ref="(1975,1977]")  <-  ~cohort
##' drelevel(mena,ref="(1980,1982]")  <-  ~cohort
##' dlevels(mena,"coh*")
##' dtable(mena,"coh*",level=1)
##' 
##' ### level 1 of zyg as baseline for new variable
##' drelevel(mena,ref=1) <- ~zyg
##' drelevel(mena,ref=c("DZ","[1973,1975]")) <- ~ zyg+cohort
##' drelevel(mena,ref=c("DZ","[1973,1975]")) <- zygdz+cohort.early~ zyg+cohort
##' ### level 2 of zyg and cohort as baseline for new variables
##' drelevel(mena,ref=2) <- ~ zyg+cohort
##' dlevels(mena)
##' 
##' ##################### combining factor levels with newlevels argument
##' 
##' dcut(mena,labels=c("I","II","III","IV")) <- cat4~agemena
##' dlevels(drelevel(mena,~cat4,newlevels=1:3))
##' dlevels(drelevel(mena,ncat4~cat4,newlevels=3:2))
##' drelevel(mena,newlevels=3:2) <- ncat4~cat4
##' dlevels(mena)
##' 
##' dlevels(drelevel(mena,nca4~cat4,newlevels=list(c(1,4),2:3)))
##' 
##' drelevel(mena,newlevels=list(c(1,4),2:3)) <- nca4..2 ~ cat4
##' dlevels(mena)
##' 
##' drelevel(mena,newlevels=list("I-III"=c("I","II","III"),"IV"="IV")) <- nca4..3 ~ cat4
##' dlevels(mena)
##' 
##' drelevel(mena,newlevels=list("I-III"=c("I","II","III"))) <- nca4..4 ~ cat4
##' dlevels(mena)
##' 
##' drelevel(mena,newlevels=list(group1=c("I","II","III"))) <- nca4..5 ~ cat4
##' dlevels(mena)
##' 
##' drelevel(mena,newlevels=list(g1=c("I","II","III"),g2="IV")) <- nca4..6 ~ cat4
##' dlevels(mena)
##' 
##' @aliases dlevels dlevel dlev drelevel drelev dlev<- dlevel<- drelev<- drelevel<- dfactor dfactor<- dnumeric dnumeric<-
##' @export
drelevel <- function(data,y=NULL,x=NULL,ref=NULL,newlevels=NULL,regex=mets.options()$regex,sep=NULL,overwrite=FALSE,...)
{# {{{

    if (is.null(ref) && is.null(newlevels)) stop("specify baseline-reference level or new levels \n")

    if (!is.null(ref) & !is.null(newlevels)) { 
        warning("can only either change ref or combine old levels, will change ref")
        newlevels <- NULL
    }

    if (is.null(sep))  sep <- "."

    if (is.vector(data) | inherits(data,"factor")) {# {{{

        if (is.vector(data)) data <- factor(data)
        if (!is.null(ref)) {
            if (is.numeric(ref)) ref <-  levels(data)[ref]
            gx <- relevel(data,ref=ref,...)
            return(gx)
        }
        if (!is.null(newlevels)) {
            pnewlevels <- levlev(data,newlevels)
            levels(data,...) <- pnewlevels
            return(data)
        }
    } # }}}

    if (is.data.frame(data)) {# {{{

        usernames <- FALSE# {{{

        vars <-mets::procform3(y,x,data=data,regex=regex,...)
        x <-  xnames <- vars$x

        if (!is.null(vars$y)) {
            usernames<-TRUE
            newnames <- vars$y
            if (length(vars$y)!=length(vars$x)) { 
                warning("length of new names not consistent with length of cut variables, uses default naming\n"); 
                usernames <- FALSE
            }
        }
                                        # }}}

        if (is.character(x) && length(x)<nrow(data)) x <- lapply(xnames,function(z) data[,z])
        dots <- list()
        args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })

        if (!is.list(x)) x <- list(x)
        ll <- length(x)

        if (!is.null(ref)) 
        {  if (ll>1 & length(ref)==1) ref <- rep(ref,ll)
            if (length(x)!=length(ref)) stop("length of baseline reference 'ref' not consistent with variables")
        }

        if (!is.null(newlevels)) {
            if (ll==1 & !is.list(newlevels)) newlevels <- list(newlevels)
            if (is.list(newlevels) && !is.list(newlevels[[1]])) newlevels <- list(newlevels)
            if (length(x)!=length(newlevels)) 
                warning("length of variables not consistent with list of breaks"); 
            if (length(newlevels)!=ll) newlevels <- rep(list(newlevels[[1]]),ll)
        }


        for (k in 1:ll)
        {
            xx <- x[[k]]
            if (!is.factor(xx)) xx <- factor(xx)

            if (usernames) name <- newnames[k]
            if (!is.null(ref)) {
                name<- paste(xnames[k],ref[k],sep=sep)
                if (usernames) name <- newnames[k]
                if (overwrite) name<-xnames[k]
                if (is.numeric(ref[k])) refk <-  levels(xx)[ref[k]] else refk <- ref[k]
                gx <- relevel(xx,ref=refk,...)
                data[,name] <- gx
            }
            if (!is.null(newlevels)) {
                name<- paste(xnames[k],newlevels[[k]][1],sep=sep)
                if (usernames) name <- newnames[k]
                if (overwrite) name<-xnames[k]
                pnewlevels <- levlev(xx,newlevels[[k]])
                levels(xx,...) <- pnewlevels
                data[,name] <- xx
            }
        }

        return(data)
    }# }}}

}# }}}

##' @export
"drelev<-" <- function(data,x=NULL,...,value) drelevel(data,y=value,x=x,...)

##' @export
drelev <- function(data,y=NULL,x=NULL,...) drelevel(data,y=y,x=x,...)

##' @export
"drelevel<-" <- function(data,x=NULL,...,value) drelevel(data,y=value,x=x,...)

tsglob2rx <- function(x) {
    glob2rx(gsub("\\+","\\\\+",x))
}

levlev <- function(fac,ref,regex=FALSE)
{# {{{
    if (!is.list(ref)) ref <- list(ref)
    lf <- levels(fac)
    lfr <- lf
    listnames <- names(ref)

    newreflist <- list()
    for (k in 1:length(ref)) {
	if (!is.null(listnames)) nn <- listnames[k] else nn <- NULL
	ln <- length(ref[[k]])
	if (is.numeric(ref[[k]])) refs <- lf[ref[[k]]] else refs <- ref[[k]]
	xxx <- c()
	for (xx in refs)
	{
            if (!regex) xx <- tsglob2rx(xx)
            n <- grep(xx,lf)
            xxx <- c(xxx,lf[n])
	}
	xxx<- xxx[!duplicated(xxx)]
	refs <- xxx

	ln <- length(refs)
	if (is.null(nn) || nn=="") 
	{
            if (length(refs)>1) nn <- paste(refs[1],refs[ln],sep="-") else nn <- refs[1]
	}
	newreflist <- c(newreflist,setNames(list(refs),nn))
	mm <- match(refs,lfr)

	lfr <- lfr[-mm]
    }


    if (length(lfr)>=1)
    {
        for (k in 1:length(lfr)) 
        {
            nn <- paste(lfr[k])
            newreflist <- c(newreflist,setNames(list(lfr[k]),nn))
        }
    }

    return(newreflist)
}# }}}

##' @export
dlevels <- function(data,y=NULL,x=NULL,regex=mets.options()$regex,max.levels=20,cols=FALSE,...)
{# {{{

    if (is.factor(data)) {
        return(base::levels(data,...)) 
    } 

    if (is.data.frame(data)) {
        usernames <- FALSE# {{{

        vars <-mets::procform3(y,x,data=data,regex=regex,...)
        x <-  xnames <- vars$x

        if (!is.null(vars$y)) {
            usernames<-TRUE
            newnames <- vars$y
            if (length(vars$y)!=length(vars$x)) { 
                warning("length of new names not consistent with length of cut variables, uses default naming\n"); 
                usernames <- FALSE
            }
        }
                                        # }}}

        if (is.character(x) && length(x)<nrow(data)) x <- lapply(xnames,function(z) data[,z])
        dots <- list()
        args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })
        if (!is.list(x)) x <- list(x)
        ll <- length(x)

        if (cols==TRUE) {
            antfactor <- 0
            namesfac <- c()
            nlev     <- c()
            lll <- list()
            maxl <- 0
            m <- 0
        }
        
        for (k in 1:ll)
        {
            xx <- x[[k]]
            if (is.factor(xx))  {
                cat(paste(xnames[k],":",sep=" #levels="));
                nxx <- nlevels(xx) 
                cat(paste(nxx,"\n")); 
                if (is.null(max.levels) || ((!is.null(max.levels)) & (nxx<max.levels))) {
                    if (cols==FALSE)   print(base::levels(xx)) 
                    if (cols==TRUE)  { maxl <- ifelse(base::nlevels(xx) > maxl, base::nlevels(xx), maxl); 
                        antfactor <- antfactor+1; 
                        namesfac <- c(namesfac,xnames[k])
                        nlev <- c(nlev,base::nlevels(xx))
                        m <- m+1
                        lll[[m]] <- base::levels(xx)
                    }

                }
                if (cols==FALSE)   cat("-----------------------------------------\n")
            }
        }

        if (cols==TRUE) { 
            mout <- matrix("",maxl,antfactor)
            for (k in 1:antfactor) {
		mout[1:nlev[k],k] <- lll[[k]]
            }
            colnames(mout) <- namesfac
            rownames(mout) <- rep("  ",nrow(mout))
            prmatrix(mout,quote=FALSE)
        }

    }

}# }}}

##' @export
dlevel <- function(data,y=NULL,x=NULL,...) dlevels(data,y=y,x=x,...)

##' @export
dlev <- function(data,y=NULL,x=NULL,...) dlevels(data,y=y,x=x,...)

##' @export
drename <- function(data,y=NULL,x=NULL,fun=base::tolower,...)
{  # {{{

    vars <-mets::procform3(y,x,data=data,...)
    x <-  xnames <- vars$x

    if (!is.null(vars$y)) {
        newnames <- vars$y
        if (length(vars$y)!=length(vars$x)) { 
            stop("length of new names not consistent with length of cut variables, uses default naming\n"); 
        }
    } else { ## if newnames not given then use fun 
        newnames <- do.call(fun,list(x))
    }

    varpos <- match(x,colnames(data))

    if (length(varpos)!= length(newnames)) stop("length of old and new variables must match")
    colnames(data)[varpos] <- newnames
    return(data)
} # }}}

##' @export
"drename<-" <- function(data,x=NULL,...,value) drename(data,y=value,x=x,...)

##' @export
dfactor <- function(data,y=NULL,x=NULL,regex=mets.options()$regex,sep=NULL,usernames=NULL,levels,labels,...)
{# {{{

    if (is.null(sep))  sep <- ".f"

    if (!is.data.frame(data)) {
        if (!is.factor(data) || !missing(levels) || !missing(labels)) {
            args <- list(data)
            if (!missing(levels))  {
                if (is.numeric(levels) & is.factor(data)) 
                    levels <-  levels(data)[levels]
                args <- c(args,list(levels=levels,...))
            }
            if (!missing(labels)) {
                args <- c(args,list(labels=labels,...))
            }
            gx <- do.call(factor,args)
            return(gx)
        } 
    }

    if (is.data.frame(data)) {

        usernames <- FALSE# {{{

        vars <-mets::procform3(y,x,data=data,regex=regex,...)
        x <-  xnames <- vars$x

        if (!is.null(vars$y)) {
            usernames<-TRUE
            newnames <- vars$y
            if (length(vars$y)!=length(vars$x)) { 
		usernames <- FALSE
            }
        }
                                        # }}}

        if (is.character(x) && length(x)<=ncol(data)) {
            x <- lapply(xnames,function(z) data[,z])
        }
        dots <- list()
        args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })
        if (!is.list(x)) x <- list(x)
        ll <- length(x)

        if (!missing(levels)) if (!is.list(levels))   levels <- list(levels)
        if (!missing(labels)) if (!is.list(labels) )   labels <- list(labels)


        misslabel <- TRUE
        if (!missing(labels))  {
            misslabel <- FALSE
            if ((length(x)!=length(labels))) {
                warning("length of label list not consistent with variables")
                labels <- rep(list(labels[[1]]),ll)
            }
        }


        misslevel <- TRUE
        if (!missing(levels))  { 
            misslevel <- FALSE
            if ((length(x)!=length(levels))) {
                warning("length of levels list not consistent with variables")
                levels <- rep(list(levels[[1]]),ll)
            }
        }
        

        for (k in 1:ll)
        {
            xx <- x[[k]]
            name<- paste(xnames[k],sep,sep="")

            if (usernames) name <- newnames[k]

            if (!is.factor(xx) || !missing(levels) || !missing(labels)) {
                args <- list(xx)
                if (!misslevel) {
                    if (is.numeric(levels[[k]]) & is.factor(xx)) 
                        llevels <-  levels(xx)[levels[[k]]]
                    else llevels <- levels[[k]]
                    args <- c(args,list(levels=llevels,...))
                }
                if (!misslabel) args <- c(args,list(labels=labels[[k]],...))
                gx <- do.call(factor,args)
                data[,name] <- gx  
            }
        }

        return(data)
    }

}# }}}

##' @export
"dfactor<-" <- function(data,x=NULL,...,value) dfactor(data,y=value,x=x,...)


##' @export
dnumeric <- function(data,y=NULL,x=NULL,regex=mets.options()$regex,sep=NULL,all=FALSE,...)
{# {{{

    if (is.null(sep))  sep <- ".n"

    if (is.factor(data)) {
        gx <- as.numeric(data) 
        return(gx)
    } 

    if (is.data.frame(data)) {

        usernames <- FALSE# {{{

        vars <-mets::procform3(y,x,data=data,regex=regex,...)
        x <-  xnames <- vars$x

        if (!is.null(vars$y)) {
            usernames<-TRUE
            newnames <- vars$y
            if (length(vars$y)!=length(vars$x)) { 
                warning("length of new names not consistent with length of cut variables, uses default naming\n"); 
                usernames <- FALSE
            }
        }
                                        # }}}

        if (is.character(x) && length(x)<nrow(data)) x <- lapply(xnames,function(z) data[,z])
        dots <- list()
        args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })
        if (!is.list(x)) x <- list(x)
        ll <- length(x)


        for (k in 1:ll)
        {
            xx <- x[[k]]
            name<- paste(xnames[k],sep,sep="")
            if (usernames) name <- newnames[k]
            if (!is.numeric(xx) || all==TRUE) { 
                gx <- as.numeric(xx,...); 
                data[,name] <- gx 
            } 
        }

        return(data)
    }

}# }}}

##' @export
"dnumeric<-" <- function(data,x=NULL,...,value) dnumeric(data,y=value,x=x,...)

##' @export
drm <- function(data,x=NULL,regex=mets.options()$regex,...)
{# {{{
    vars <-mets::procform(x,data=data,no.match=FALSE,regex=regex,...)
    xnames <- c(vars$predictor,vars$response)

    data[,xnames] <- NULL
    return(data)
}# }}}

##' @export
"drm<-" <- function(data,...,value) drm(data,x=value,...)

##' @export
dkeep <- function(data,x=NULL,keep=TRUE,regex=mets.options()$regex,...)
{  # {{{
    vars <-mets::procform(x,data=data,no.match=FALSE,regex=regex,...)
    xnames <- c(vars$predictor,vars$response)
    nnames <- match(xnames,names(data))

    if (keep) data <- data[,nnames] else data <- data[,-1*nnames]
    return(data)
} # }}}

##' @export
"dkeep<-" <- function(data,...,value) dkeep(data,x=value,...)

##' @export
ddrop <- function(data,x=NULL,keep=FALSE,...) dkeep(data,x,keep=FALSE,...)

##' @export
"ddrop<-" <- function(data,...,value) dkeep(data,x=value,keep=FALSE,...)

##' @export
dnames <- function(data,...) drename(data,...)

##' @export
"dnames<-" <- function(data,...,value) drename(data,value=value,...)


##' Lag operator
##'
##' Lag operator
##' @examples
##' d <- data.frame(y=1:10,x=c(10:1))
##' dlag(d,k=1:2)
##' dlag(d,~x,k=0:1)
##' dlag(d$x,k=1)
##' dlag(d$x,k=-1:2, names=letters[1:4])
##' @export
##' @param data data.frame or vector
##' @param x optional column names or formula
##' @param k lag (vector of integers)
##' @param combine combine results with original data.frame
##' @param simplify Return vector if possible
##' @param names optional new column names
##' @param ... additional arguments to lower level functions
##' @aliases dlag dlag<-
dlag <- function(data,x,k=1,combine=TRUE,simplify=TRUE,names,...) {# {{{
    isvec <- FALSE
    if (!is.data.frame(data)) {
        isvec <- is.vector(data)
        data <- as.data.frame(data)
        combine <- FALSE
    }
    if (missing(x)) x <- base::names(data)
    if (is.character(x)) x <- data[,x,drop=FALSE]
    if (inherits(x,"formula")) {
        ##x <- as.data.frame(model.matrix(update(x,~.-1), model.frame(~.,data=data, na.action=na.pass)))
        environment(x) <- environment()
        x <- model.frame(x,data=data, na.action=na.pass)
    }
    kmin0 <- abs(min(k,0))
    kmax0 <- max(k,0)
    kmax <- kmin0+kmax0
    pad <- function(x) c(rep(NA,kmax0),x,rep(NA,kmin0))
    kidx <- match(k,seq(min(k,0),max(k,0)))
    val <- lapply(x,function(y) embed(pad(y),dimension=kmax+1)[,kidx,drop=FALSE])
    dval <- as.data.frame(val)
    if (!missing(names)) {
        base::names(dval) <- names
    } else {
        nn <- as.vector(sapply(colnames(x),function(x) paste0(x,paste0(".",k))))        
        if (length(nn)==ncol(dval)) {
            nn <- gsub("-","_",nn)
            base::names(dval) <- nn
        }
    }
    if (combine) {        
        res <- cbind(data,dval)
        names(res) <- make.unique(base::names(res))
        return(res)
    }
    if (length(k)==1 && simplify && isvec) return(as.vector(val[[1]]))
    names(dval) <- base::make.unique(base::names(dval))
    return(as.matrix(dval))
}# }}}

##' @export
"dlag<-" <- function(data,k=1,combine=TRUE,...,value) {
    dlag(data,value,k=k,combine=combine,...)
}

##' Simple linear spline 
##' 
##' Constructs simple linear spline  on a data frame using the formula syntax of dutils
##' that is adds (x-cuti)* (x>cuti) to the data-set for each knot of the spline. 
##' The full spline is thus given by x and spline variables added to the data-set. 
##' 
##' @param data if x is formula or names for data frame then data frame is needed.
##' @param y name of variable, or fomula, or names of variables on data frame.
##' @param x name of variable, or fomula, or names of variables on data frame.
##' @param probs groups defined from quantiles
##' @param breaks  number of breaks, for variables or vector of break points,
##' @param equi for equi-spaced breaks  
##' @param regex for regular expressions.
##' @param sep seperator for naming of cut names.
##' @param na.rm to remove NA for grouping variables.
##' @param labels to use for cut groups 
##' @param all to do all variables, even when breaks are not unique 
##' @param ... Optional additional arguments
##' @author Thomas Scheike
##' @examples
##' data(TRACE)
##' TRACE <- dspline(TRACE,~wmi,breaks=c(1,1.3,1.7))
##' cca <- coxph(Surv(time,status==9)~age+vf+chf+wmi,data=TRACE)
##' cca2 <- coxph(Surv(time,status==9)~age+wmi+vf+chf+wmi.spline1+wmi.spline2+wmi.spline3,data=TRACE)
##' anova(cca,cca2)
##' 
##' nd=data.frame(age=50,vf=0,chf=0,wmi=seq(0.4,3,by=0.01))
##' nd <- dspline(nd,~wmi,breaks=c(1,1.3,1.7))
##' pl <- predict(cca2,newdata=nd)
##' plot(nd$wmi,pl,type="l")
##'
##' @export
##' @aliases dspline<-
dspline <- function(data,y=NULL,x=NULL,breaks=4,probs=NULL,equi=FALSE,regex=mets.options()$regex,sep=NULL,na.rm=TRUE,labels=NULL,all=FALSE,...)
{# {{{
    if (is.vector(data)) {# {{{
	if (is.list(breaks)) breaks <- unlist(breaks)

        if (length(breaks)==1) { 
             if (!is.null(probs))
	     {
                breaks <- quantile(data, probs, na.rm=na.rm, ...)
	        breaks <- breaks[-c(1,length(breaks))]
	     } else {
	   	if (!equi) { 
			probs <- seq(0, 1, length.out = breaks + 1)
			breaks <- quantile(data, probs,na.rm=na.rm, ...)
	                breaks <- breaks[-c(1,length(breaks))]
		} 
		if (equi) { 
			rr <- range(data,na.rm=na.rm)
			breaks <-  seq(rr[1],rr[2],length.out=breaks+1)
	                breaks <- breaks[-c(1,length(breaks))]
		}
	     }
	}

        if (sum(duplicated(breaks))==0) {
             gx <- LinSpline(data, breaks, ...)
	     attr(gx,"breaks") <- breaks
	} else {
	      wd <- which(duplicated(breaks))
              mb <- min(diff(breaks[-wd]))
	      breaks[wd] <- breaks[wd] +  (mb/2)*seq(length(wd))/length(wd)
              gx <- LinSpline(data, breaks,...)
	      attr(gx,"breaks") <- breaks
              warning(paste("breaks duplicated"))
        }
        return(gx)
    }# }}}

if (is.data.frame(data)) {# {{{

 if (is.null(sep)) sep <- "."

 usernames <- FALSE# {{{

     vars <-mets::procform3(y,x,data=data,regex=regex,...)
     x <-  xnames <- vars$x

     if (!is.null(vars$y)) {
         usernames<-TRUE
         newnames <- vars$y
	 if (length(vars$y)!=length(vars$x)) { 
	   warning("length of new names not consistent with length of cut variables, uses default naming\n"); 
	   usernames <- FALSE
      }
     }
# }}}

  if (is.character(x) && length(x)<nrow(data)) x <- lapply(xnames,function(z) data[,z])
  dots <- list()
  args <- lapply(dots, function(x) { if (length(x)==1 && is.character(x)) x <- data[,x]; x })

  if (!is.list(x)) x <- list(x)
  ll <- length(x)
  if (ll==1 & !is.list(breaks) & length(breaks)>1) breaks <- list(breaks)

  break.points <- FALSE
  if (is.list(breaks)) {
     break.points <- TRUE
     if (length(x)!=length(breaks) & length(breaks)!=1) 
	     warning("length of variables not consistent with list of breaks"); 
     if (length(breaks)!=ll) breaks <- rep(list(breaks[[1]]),ll)
  }

  if (!break.points) {
     if (length(x)!=length(breaks) & length(breaks)!=1) 
	     warning("length of variables not consistent with breaks"); 
     if (length(breaks)!=ll) breaks<- rep(breaks[1],ll)
  }

  if (ll==1 & !is.list(labels)) labels <- list(labels)
  if (!is.list(labels)) labels <- list(labels); 
  if (length(labels)!=ll ) labels <- rep(list(labels[[1]]),ll)
  if (!is.list(labels)) stop("labels should be given as list"); 


for (k in 1:ll)
{
  xx <- x[[k]]
  if (is.numeric(xx)) {

      if (!is.list(breaks))
      {
          if (!is.null(probs))
	  {
                bb <- quantile(xx, probs,na.rm=na.rm, ...)
	        bb <- bb[-c(1,length(bb))]
	  } else {
	   	if (!equi) { 
			probs <- seq(0, 1, length.out = breaks[k] + 1)
			bb <- quantile(xx, probs, na.rm=na.rm,...)
	                bb <- bb[-c(1,length(bb))]
		} 
		if (equi) { 
			rr <- range(xx,na.rm=na.rm)
			bb <-  seq(rr[1],rr[2],length.out=breaks[k]+1)
	                bb <- bb[-c(1,length(bb))]
		}
	     }
          name<-xnames[k]
      } else { bb <- breaks[[k]]; 
          name<-xnames[k]
      }

      if (usernames) name <- newnames[k]

      if (sum(duplicated(bb))==0) {
	attr(data,paste(name,"spline.breaks",sep="")) <- bb
	for (i in seq_along(c(bb)))
	{
           namei <- paste(name,".spline",i,sep="")
	   data[,namei] <-  (xx-bb[i])*(xx>bb[i])
	}
      }
      else { 
	   if (all==TRUE) {
	      wd <- which(duplicated(bb))
              mb <- min(diff(bb[-wd]))
	      bb[wd] <- bb[wd] +  (mb/2)*seq(length(wd))/length(wd)
	      attr(data,paste(name,"spline.breaks",sep="")) <- bb
              for (i in seq_along(c(breaks)))
              {
		     namei <- paste(name,".spline",i,sep="")
		     data[,namei] <-  (xx-bb[i])*(xx>bb[i])
	      }
             warning(paste("breaks duplicated for=",xnames[k]))
	   }
      }
   }
}

return(data)
}# }}}

}# }}}

##' @export
"dspline<-" <- function(data,...,value) dspline(data,y=value,...)

##' Simple linear spline 
##' 
##' Simple linear spline 
##' 
##' @param x variable to make into spline
##' @param knots cut points 
##' @param num to give names x1 x2 and so forth
##' @param name name of spline expansion name.1 name.2 and so forth
##' @author Thomas Scheike
##' @keywords survival
##' @export
LinSpline <- function(x,knots,num=TRUE,name="Spline")
{# {{{

lspline <- matrix(0,length(c(x)),length(c(knots)))
for (i in seq_along(c(knots)))
{
    lspline[,i] <- (x-knots[i])*(x>knots[i])
}

lspline <- as.data.frame(lspline)
if (num==TRUE) names(lspline) <- paste(name,seq_along(c(knots)),sep="")
else if (!is.null(signif)) names(lspline) <- paste(name,round(c(knots),signif),sep="")

    return(lspline)
}# }}}

##' @export
predictGLM <- function(object,newdata,id=NULL,fun=NULL,link.conf=TRUE,...) {# {{{
    tt <- terms(object)
    if (missing(newdata) || is.null(newdata)) {
        mm <- X <- model.matrix(object)
        mmDone <- TRUE
        offset <- object$offset
    }
    else {
        Terms <- delete.response(tt)
        m <- model.frame(Terms, newdata, xlev = object$xlevels)
        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    }
    offsets <- rep(0,nrow(X))
    if (!is.null(object$offset)) offsets <- m[, grep("offset", names(m))] 

if (!is.null(object$family$linkinv)) linkinv <- object$family$linkinv  else linkinv <- function(x) x
f <- function(p) { pp <- X %*% p+offsets; return(linkinv(pp)); }
fl <- function(p) { pp <- X %*% p+offsets; return(pp); }
if (!is.null(fun))  f <- fun

if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)

if (link.conf) { 
     if (!is.null(id)) resl <- estimate(object,f=fl,id=id,...) else resl <- estimate(object,f=fl,...)
     res <- linkinv(resl$coefmat[,c(1,3,4)]) 
} else {
if (!is.null(id)) res <- estimate(object,f=f,id=id,...) else res <- estimate(object,f=f,...)
res <- res$coefmat
}

return(list(coef=coef,pred=res))
}
# }}}

##' Reporting OR (exp(coef)) from glm with binomial link and glm predictions
##' 
##' Reporting OR from glm with binomial link  and glm predictions
##' 
##' @param object glm output 
##' @param id possible id for cluster corrected standard errors
##' @param fun possible function for non-standard predictions based on object
##' @param ... arguments of estimate of lava for example level=0.95 
##' @author Thomas Scheike
##' @export
##' @examples
##' data(sTRACE)
##' sTRACE$id <- sample(1:100,nrow(sTRACE),replace=TRUE)
##'
##' model <- glm(I(status==9)~sex+factor(diabetes)+age,data=sTRACE,family=binomial)
##' summaryGLM(model)
##' summaryGLM(model,id=sTRACE$id)
##'
##' nd <- data.frame(sex=c(0,1),age=67,diabetes=1)
##' predictGLM(model,nd)
##' @aliases predictGLM 
summaryGLM <- function(object,id=NULL,fun=NULL,...) {# {{{

f <- function(p) { pp <- exp(pp); return(pp); }

if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)

if (!is.null(id)) resl <- estimate(object,id=id,...) else resl <- estimate(object,...)
res <- exp(resl$coefmat[,c(1,3,4)]) 

if (!is.null(fun))  {
if (!is.null(id)) resl <- estimate(object,id=id,f=fun,...) else resl <- estimate(object,f=fun,...)
fout <- resl$coefmat
} else fout <- NULL

return(list(coef=coef,or=res,fout=NULL))
}
# }}}

##' Wald test for model (type III test)
##' 
##' Wald test for model (type III test)
##' 
##' @param object, for example glm object that can be used with estimate
##' @param ... arguments for estimate of lava for example id=data$id for cluster correction
##' @author Thomas Scheike
##' @export
waldTest <- function(object,...)
{# {{{
Vars <-attr(terms(formula(object)),"term.labels")
## remove possible cluster term
clusterTerm<- grep("^cluster[(][A-z0-9._:]*",Vars,perl=TRUE)
if (length(clusterTerm)==1) Vars <- Vars[-clusterTerm]
Varf <- names(attr(attr(object$model,"terms"),"dataClasses"))[grep("factor",attr(attr(object$model,"terms"),"dataClasses"))]
txtVar<-names(coef(object))
###
test.matrix <- matrix(0,length(Vars),3)
colnames(test.matrix) <- c("Chisq","df","p.value")
rownames(test.matrix) <- Vars
k <- 1
for (ff in Vars) {
	pos <- grep(ff,txtVar,fixed=TRUE)
        wt <- estimate(object,as.list(pos),...)$compare
###	wt <- estimate(object,as.list(3:5))
	test.matrix[k,] <- unlist(wt[1:3])
	k <- k+1
}
return(test.matrix)
}# }}}

## library(lava)
## m <- lvm(y~x)
## distribution(m,~y) <- coxWeibull.lvm(shape=3,scale=5)
## transform(m,~status) <- function(...) TRUE
## d <- lava::sim(m,2e4,p=c("y~x"=2))
## library(eha)
## with(d,phreg.par(y,status,cbind(x)))
## weibreg(Surv(y,status)~x,data=d)

## ## Note in simulation A(t) = lambda*t^scale
## ## but here A(t) (scale*t)^shape, hence
## ## lambda := scale^(1/shape)
## tt <- seq(0,100,length.out=100)
## plot(tt,exp(-(exp(-4)*tt)^exp(.5))
## y <- runif(2e4,0,100)
## (op <- phreg.par(y,TRUE))

## tt <- seq(0,100,length.out=100)
## cc <- coxph(Surv(y,rep(TRUE,length(y)))~1)
## plot(survfit(cc),mark.time=FALSE)
## lines(tt,exp(-(exp(-4)*tt)^exp(.484)),col="red") 
##(op <- phreg.weibull(d$y,TRUE,cbind(d$x)))
## (a <- survival::survreg(Surv(y,status)~1+x,dist="weibull",data=d))
## (e <- eha::weibreg(Surv(y,status)~x,data=d))

###{{{ Weibull

info_phreg_weibull <- function(...) {
    list(npar=2,
         start=c(-1,-1),
         name="weibull",
         partrans=function(theta) {
             exp(theta)
         },
         cumhaz=function(t,theta) {
             (theta[1]*t)^theta[2]
         }
         )
}

logl_phreg_weibull <- function(theta, time, status,
                               X=NULL, theta.idx=NULL,
                              indiv=FALSE) {
  if (!is.null(theta.idx)) {
    offsets <- which(is.na(theta.idx))
    theta <- theta[theta.idx]
    theta[offsets] <- 1
  }
  lambda <- exp(theta[1])
  p <- exp(theta[2])
  if (is.null(X)) {
    eta <- 0
  } else {
    beta <- theta[-c(1:2)]
    eta <- X%*%beta
  }
  
  val <- status*log(lambda*p) + status*(p-1)*log(lambda*time) + status*eta - (lambda*time)^p*exp(eta)
  if (indiv)
    return(val)
  sum(val)
}

obj_phreg_weibull <- function(...)
  -logl_phreg_weibull(...)

score_phreg_weibull <- function(theta, time, status,
                                X=NULL, theta.idx=NULL, indiv=FALSE) {
  if (!is.null(theta.idx)) {
    offsets <- which(is.na(theta.idx))
    theta <- theta[theta.idx]
    theta[offsets] <- 1
  }
  lambda <- exp(theta[1])
  p <- exp(theta[2])
  lambdaT <- lambda*time
  loglambdaT <- log(lambdaT)
  lambdaTp <- exp(loglambdaT*p)
  
  if (is.null(X)) {
    eta <- 0; expeta <- 1
    dbeta <- NULL
  } else {
    beta <- theta[-c(1:2)]
    eta <- X%*%beta
    expeta <- exp(eta)
    dbeta <- ((status-expeta*lambdaTp)%x%rbind(rep(1,NCOL(X))))*X
  }
  dp <- status*(1/p + loglambdaT) - loglambdaT*lambdaTp*expeta
  dlogp <- p*dp
  dlambda <- status*(p/lambda) - p*lambdaTp/lambda*expeta
  dloglambda <- lambda*dlambda
  S <- cbind(dloglambda,dlogp,dbeta)
  if (!is.null(theta.idx)) {
    u.idx <- na.omit(unique(theta.idx))
    newS <- matrix(0,ncol=length(u.idx),nrow=nrow(S))
    for (i in u.idx) {
      newS[,i] <- cbind(rowSums(S[,which(theta.idx==i),drop=FALSE]))
    }
    S <- newS
  }
  if (indiv)
    return(S)
  colSums(S)
}

hessian_phreg_weibull <- function(theta, time, status,
                                  X=NULL, theta.idx=NULL,
                                  all=FALSE) {
  if (!is.null(theta.idx)) {
    offsets <- which(is.na(theta.idx))
    theta <- theta[theta.idx]
    theta[offsets] <- 1
  }
  lambda <- exp(theta[1])
  p <- exp(theta[2])
  lambdaT <- lambda*time
  loglambdaT <- log(lambdaT)
  Tp <- time^p
  lambdaTp <- lambda^p*Tp
    if (is.null(X)) {
    eta <- 0; expeta <- 1
    d2dlogpdbeta <- d2dlogpdbeta <- d2beta <- NULL
  } else {
    beta <- theta[-c(1:2)]
    eta <- X%*%beta
    expeta <- exp(eta)
    ## D(beta,beta)
    U <- ((expeta*Tp)%x%rbind(rep(1,NCOL(X))))*X
    d2beta <- -t(lambda^p*U)%*%X
    ## D(p,beta)
    d2dpdbeta <-  colSums(-((loglambdaT*lambdaTp*expeta)%x%rbind(rep(1,NCOL(X))))*X)
    d2dlogpdbeta <- d2dpdbeta*p  
    ## D(lambda,beta)
    d2dlambdadbeta <- -U*p*lambda^(p-1)
    d2dloglambdadbeta <- colSums(d2dlambdadbeta)*lambda  
  }
  ## D(p,p)
  dp <- status*(1/p + loglambdaT) - loglambdaT*lambdaTp*expeta
  d2p <- -sum(status/(p^2) + loglambdaT^2*expeta*lambdaTp)
  dlogp <- p*dp
  d2logp <- sum(dlogp)+p^2*d2p
  ## D(lambda,lambda)
  dlambda <- status*(p/lambda) - p*lambdaTp/lambda*expeta
  d2lambda <- -sum(status*(p/lambda^2) + p*(p-1)*lambdaTp/(lambda^2)*expeta)
  dloglambda <- lambda*dlambda
  d2loglambda <- sum(dloglambda)+lambda^2*d2lambda
  ## D(p,lambda)
  d2dpdlambda <- -status/(p^2) - loglambdaT^2*expeta*lambdaTp
  d2dpdlambda <- status/lambda - lambdaTp/lambda*expeta - p*loglambdaT*lambdaTp/lambda*expeta 
  d2dlogpdloglambda <- sum(d2dpdlambda)*p*lambda  
  ## Hessian:
  H <- matrix(0,length(theta),length(theta))
  H[1,1] <- d2loglambda
  H[2,2] <- d2logp
  H[1,2] <- H[2,1] <- d2dlogpdloglambda
  if (!is.null(X)) {
    H[3:length(theta),3:length(theta)] <- d2beta
    H[2,3:length(theta)] <- H[3:length(theta),2] <- d2dlogpdbeta
    H[1,3:length(theta)] <- H[3:length(theta),1] <- d2dloglambdadbeta
  }
  if (!is.null(theta.idx)) {
    u.idx <- na.omit(unique(theta.idx))
    newH <- matrix(0,length(u.idx),length(u.idx))
    for (i in u.idx) {
      for (j in u.idx) {
        newH[i,j] <- sum(H[which(theta.idx==i),which(theta.idx==j)])
      }
    }
    H <- newH
  }
  if (all) {
    ## Score:
    if (is.null(X)) dbeta <- NULL else dbeta <- status*X-lambda^p*U
    S <- cbind(dloglambda,dlogp,dbeta)
    if (!is.null(theta.idx)) {
      u.idx <- na.omit(unique(theta.idx))
      newS <- matrix(0,ncol=length(u.idx),nrow=nrow(S))
      for (i in u.idx) {
        newS[,i] <- cbind(rowSums(S[,which(theta.idx==i),drop=FALSE]))
      }
      S <- newS
    }
    attributes(H)$grad <- colSums(S)
    attributes(H)$score <- S
    ## LogLik
    attributes(H)$logL <- sum(status*log(lambda*p) + status*(p-1)*loglambdaT + status*eta - lambdaTp*expeta)
  }
  return(H)
}

###}}} Weibull

###{{{ Generalized-Gamma


## http://www.stanford.edu/~lutian/coursepdf/unit1.pdf

gengamma_phreg_f <- function(t,p,lambda,alpha,...) {
    p*lambda*(lambda*t)^(alpha-1)*exp(-(lambda*t)^p)/gamma(alpha/p)
}

## incomplete gamma gamma(s,x) = pgamma(x,s)
gengamma_phreg_F <- function(t,p,lambda,alpha,...) {
    pgamma((lambda*t)^p,alpha/p)/gamma(alpha/p)
}

## incomplete gamma gamma(s,x) = pgamma(x,s)
gengamma_phreg_h <- function(t,p,lambda,alpha,...) {
    p*lambda*(lambda*t)^(alpha-1)*exp(-(lambda*t)^p)/pgamma((lambda*t)^p,alpha/p)
}

logl_phreg_gengamma <- function(theta,time,status,X=NULL,indiv=FALSE,...) {
    ## suppressMessages(browser())
    p <- exp(theta[1])
    lambda <- exp(theta[2])
    alpha <- exp(theta[3])
    eta <- 0
    if (!is.null(X)) {
        beta <- theta[seq(length(theta)-3)+3]
        eta <-X%*%beta
    }
    res <- log(gengamma_phreg_f(time,p,lambda,alpha))*status + status*eta +
        log(1-gengamma_phreg_F(time,p,lambda,alpha)*exp(eta))
    if (indiv) return(res)
    return(sum(res))
}

score_phreg_gengamma <- function(theta,...) {
    numDeriv::jacobian(logl_phreg_gengamma, theta,...)
}

hessian_phreg_gengamma <- function(theta,...) {
    numDeriv::hessian(logl_phreg_gengamma, theta,...)
}

info_phreg_gengamma <- function(...) {
    list(npar=3, start=c(-1,-1,-1), name="gengamma")
}

###}}} Generalized-Gamma

##' @export
predict.phreg.par <- function(object, p=coef(object),
                              X=object$X, time=object$time, ...) {
    info <- do.call(paste("info_phreg",object$model,sep="_"),list())
    cc <- coef(object)
    eta <- 0
    if (length(cc)>info$npar) {
        eta <- X%*%p[-seq(info$npar)]
    }
    exp(-(info$cumhaz(time,info$partrans(p))*exp(eta)))
}



###{{{ phreg.par + methods

##' @export
phreg.par <- function(formula,data=parent.frame(), formula2,
                      time, status, X=NULL, model="weibull",
                      theta.idx=NULL, theta0,
                      niter=100, tol1=1e-9, tol2=1e-9,
                      lambda1=0.5, lambda2=1, trace=0,...) {

    if (!missing(formula)) {
        cl <- match.call()
        m <- match.call(expand.dots = TRUE)[1:3]
        special <- c("strata", "cluster")
        Terms <- terms(formula, special, data = data)
        m$formula <- Terms
        m[[1]] <- as.name("model.frame")
        m <- eval(m, parent.frame())
        Y <- model.extract(m, "response")
        if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
        if (ncol(Y)==2) {
            exit <- eval(Y[,1],data)
            entry <- NULL ## rep(0,nrow(Y))
            status <- Y[,2]
        } else {
            entry <- Y[,1]
            exit <- Y[,2]
            status <- Y[,3]
        }
        id <- strata <- NULL
        if (!is.null(attributes(Terms)$specials$cluster)) {
            ts <- survival::untangle.specials(Terms, "cluster")
            Terms  <- Terms[-ts$terms]
            id <- m[[ts$vars]]
        }
        if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
            ts <- survival::untangle.specials(Terms, "strata")
            Terms  <- Terms[-ts$terms]
            strata <- m[[ts$vars]]
        }

        X <- model.matrix(Terms, m)
        if (!is.null(intpos  <- attributes(Terms)$intercept))
            X <- X[,-intpos,drop=FALSE]
        if (ncol(X)==0) X <- NULL
        time <- exit        
    }

    myinfo <- do.call(paste("info_phreg",model,sep="_"),list())
    if (is.null(X)) {
        theta0 <- myinfo$start
    } else {
        if (missing(theta0))
            theta0 <- rep(0,ifelse(is.null(theta.idx),
                                   myinfo$npar+NCOL(X),length(unique(na.omit(theta.idx)))))
    }

  hess <- paste("hessian_phreg",model,sep="_")
  scor <- paste("score_phreg",model,sep="_")
  logl <- paste("logl_phreg",model,sep="_")
  thetas <- theta0; logL <- c()
  for (i in 1:niter) {
      H <- do.call(hess, list(theta=theta0,all=TRUE,time=time,status=status,X=X,theta.idx=theta.idx))
      if (!is.null(attributes(H)$score)) {
          S <- colSums(attributes(H)$score)        
      } else {
          S <- do.call(scor, list(theta=theta0,time=time,status=status,X=X))
      }
      gamma <- 0
      if (lambda2>0) gamma <- lambda2*sqrt((t(S)%*%S)[1])*diag(NROW(H))
      theta0 <- theta0 - lambda1*solve(H-gamma)%*%S
      thetas <- rbind(thetas,as.vector(theta0))
      if (!is.null(attributes(H)$logL))  {  
          logL <- c(logL, attributes(H)$logL)
      } else {
          logL <- c(logL,do.call(logl, list(theta=theta0,time=time,status=status,X=X)))
      }
      if(trace>0)
          if (i%%trace==0) {
              cat("Iter=",i, ", logLik=",tail(logL,1),"\n",sep="")
              cat("theta=(",paste(formatC(theta0),collapse=";"),")\n",sep="")
          }
      if (i>1)
          if (sum(abs(S^2))<tol1 & abs(logL[i]-logL[i-1])<tol2) break;
  }

  V <- solve(-H)
  coefs <- cbind(theta0,diag(V)^0.5); colnames(coefs) <- c("Estimate","Std.Err");
  attributes(coefs)$score <- S
  attributes(coefs)$logLik <- tail(logL,1)
  mynames <- c("log(scale)","log(shape)")
  if (!is.null(X)) {
    xnames <- colnames(X)
    if (is.null(xnames))
      xnames <- paste("x",seq(NCOL(X)),sep="")
    mynames <- c(mynames,xnames)
  }
  if (!is.null(theta.idx)) {
    mynames <- mynames[na.omit(unique(theta.idx))]
  }
  rownames(coefs) <- mynames
  structure(list(call=match.call(), coef=coefs[,1],
                 coefmat=coefs, vcov=V,
                 formula=eval(formula),
                 nevent=sum(status),
                 model=model,
                 time=time, status=status, X=X),
            class=c("phreg.par", "phreg"))
}


##' @export
print.phreg.par <- function(x, ...) {
    cat("Call:\n")
    dput(x$call)
    printCoefmat(x$coefmat, ...)
}

##' @export
vcov.phreg.par <- function(object, ...) {
    object$vcov
}

##' @export
coef.phreg.par <- function(object, ...) {
    object$coef
}

##' @export
IC.phreg.par <- function(x, p=coef(x), ...) {
    iI <- vcov(x)
    U <- do.call(paste("score_phreg", x$model, sep="_"),
                 list(theta=p, time=x$time, status=x$status,
                      X=x$X, indiv=TRUE))
    res <- (U)%*%iI*NROW(U)
    colnames(res) <- names(coef(x))
    structure(res, iI=iI)
}

##' @export
logLik.phreg.par <- function(object, p=coef(object), ...) {
  do.call(paste("logl_phreg", object$model, sep="_"),
          list(theta=p, object$time, object$status,
               object$X, ...))
}

##' @export
model.frame.phreg.par <- function(formula, ...) {
    formula$X
}

##' @export
predict.phreg.par <- function(object,p=coef(object),X=object$X,time=object$time,...) { 	 
  info <- do.call(paste("info_phreg",
                        object$model, sep="_"), list())
    cc <- coef(object)
    eta <- 0
    if (length(cc)>info$npar) {
        eta <- X%*%p[-seq(info$npar)]
    }
    exp(-(info$cumhaz(time,info$partrans(p))*exp(eta)))
}

##' Binomial Regression for Censored Competing Risks Data
##'
##' Fits a binomial regression model for a specific time point in the presence of 
##' right-censored data and competing risks. This function implements the 
##' Inverse Probability of Censoring Weighted (IPCW) estimating equation approach.
##'
##' The model estimates the probability:
##' \deqn{P(T \leq t, \epsilon=1 | X ) = \text{expit}( X^T \beta) }
##'
##' Based on binomial regresion IPCW response estimating equation: 
##' \deqn{ X ( \Delta^{ipcw}(t) I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 }
##' where \deqn{\Delta^{ipcw}(t) = I((min(t,T)< C)/G_c(min(t,T)-)} is 
##' IPCW adjustment of the response \deqn{Y(t)= I(T \leq t, \epsilon=1 )}.  
##' Two types of estimators are available:
##' \itemize{
##'   \item \code{type="I"}: Solves the standard IPCW estimating equation.
##'   \item \code{type="II"}: Adds a censoring augmentation term for efficiency gains, 
##'   solving \deqn{X \int E(Y(t)| T>s)/G_c(s) d \hat M_c}.
##' }
##'
##' Alternatively, \code{logitIPCW} performs a standard logistic regression with 
##' IPCW weights applied directly to the likelihood. Thus solving 
##' \deqn{ X  I(min(T_i,t) < G_i)/G_c(min(T_i ,t)) ( I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 } 
##' a standard logistic regression with IPCW weights.
##'
##' The variance estimation is based on squared influence functions, with options 
##' for naive variance (assuming known censoring) and robust variance (accounting 
##' for censoring model estimation).
##'
##' Censoring model may depend on strata (cens.model=~strata(gX)). 
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime data analysis}, 29, 441–482.
##'   \item Scheike TH, Zhang MJ, Gerds TA (2008). "Predicting cumulative incidence probability by direct binomial regression." \emph{Biometrika}, 95(1), 205–220.
##' }
##'
##' @param formula A formula object specifying the outcome and covariates. 
##'   The outcome must be an \code{Event} object (\code{Event(time, cause)}).
##' @param data A data frame containing the variables in the formula.
##' @param cause Numeric vector or scalar indicating the cause of interest for the competing risks.
##' @param time Numeric scalar indicating the time point of interest for the cumulative incidence.
##' @param beta Optional numeric vector of starting values for the coefficients. Defaults to zeros.
##' @param type Character string. Either \code{"I"} (classic binomial regression) or \code{"II"} (adds augmentation term).
##' @param offset Optional numeric vector of offsets for the linear predictor.
##' @param weights Optional numeric vector of weights for the score equations.
##' @param cens.weights Optional numeric vector of pre-calculated censoring weights. If NULL, they are estimated internally.
##' @param cens.model A formula specifying the censoring model. Defaults to \code{~+1}. Can include strata (e.g., \code{~strata(group)}).
##' @param se Logical. If TRUE, computes standard errors based on IPCW influence functions.
##' @param kaplan.meier Logical. If TRUE, uses Kaplan-Meier estimator for IPCW weights; otherwise uses exponential baseline.
##' @param cens.code Numeric code representing censored observations in the status variable. Defaults to 0.
##' @param no.opt Logical. If TRUE, optimization is skipped and starting values are used.
##' @param method Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.
##' @param augmentation Optional numeric vector for additional augmentation terms.
##' @param outcome Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function), \code{"rmst"} (Restricted Mean Survival Time), or \code{"rmtl"} (Restricted Mean Time Lost).
##' @param model Character string. Link function: \code{"default"} (auto-selects based on outcome), \code{"logit"}, \code{"exp"}, or \code{"lin"} (identity).
##' @param Ydirect Optional numeric vector. If provided, this outcome is used instead of constructing one from \code{outcome}. Useful for custom IPCW adjustments.
##'
##' @section Outcome definition:
##' The observed outcome \eqn{Y_i} constructed from \code{outcome} depends on whether
##' competing risks are present. Competing risks are detected automatically: the data are
##' considered to have competing risks when causes other than those specified in \code{cause}
##' are observed (i.e. \code{any(!(Causes \%in\% cause))}, where \code{Causes} are all
##' non-censoring event codes found in the data).
##'
##' \strong{No competing risks} (all observed events belong to \code{cause}):
##' \itemize{
##'   \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i = 1)}. Cumulative incidence.
##'   \item \code{"rmst"}: \eqn{Y_i = \min(T_i, t)}. Restricted mean survival time.
##'   \item \code{"rmtl"}: \eqn{Y_i = t - \min(T_i, t)}. Restricted mean time lost.
##' }
##'
##' \strong{Competing risks} (causes beyond \code{cause} are observed in the data):
##' \itemize{
##'   \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i \in \code{cause})}. Cumulative
##'     incidence for the cause(s) of interest.
##'   \item \code{"rmst"} or \code{"rmtl"}: \eqn{Y_i = I(\epsilon_i \in \code{cause})(t - \min(T_i, t))}.
##'     Cause-specific years lost, accumulated only for subjects who experience an event in \code{cause}.
##' }
##'
##' The default link function (\code{model = "default"}) is \code{"logit"} for \code{"cif"} and
##' \code{"exp"} for \code{"rmst"} and \code{"rmtl"}.
##' If \code{Ydirect} is supplied, outcome construction is bypassed entirely.
##' @param ... Additional arguments passed to lower-level optimization functions.
##'
##' @return An object of class \code{"binreg"} containing coefficients, variance-covariance matrix, 
##'   influence functions (\code{iid}), and model details.
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' # logistic regresion with IPCW binomial regression 
##' out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
##' summary(out)
##' head(iid(out))
##'
##' predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
##'
##' outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
##' summary(outs)
##'
##' ## glm with IPCW weights 
##' outl <- logitIPCW(Event(time,cause)~tcell+platelet,bmt,time=50)
##' summary(outl)
##'
##' ##########################################
##' ### risk-ratio of different causes #######
##' ##########################################
##' data(bmt)
##' bmt$id <- 1:nrow(bmt)
##' bmt$status <- bmt$cause
##' bmt$strata <- 1
##' bmtdob <- bmt
##' bmtdob$strata <-2
##' bmtdob <- dtransform(bmtdob,status=1,cause==2)
##' bmtdob <- dtransform(bmtdob,status=2,cause==1)
##' ###
##' bmtdob <- rbind(bmt,bmtdob)
##' dtable(bmtdob,cause+status~strata)
##' 
##' cif1 <- cif(Event(time,cause)~+1,bmt,cause=1)
##' cif2 <- cif(Event(time,cause)~+1,bmt,cause=2)
##' plot(cif1)
##' plot(cif2,add=TRUE,col=2)
##' 
##' cifs1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=50)
##' cifs2 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=2,time=50)
##' summary(cifs1)
##' summary(cifs2)
##' 
##' cifdob <- binreg(Event(time,status)~-1+factor(strata)+
##' 	 tcell*factor(strata)+platelet*factor(strata)+age*factor(strata)
##' 	 +cluster(id),bmtdob,cause=1,time=50,cens.model=~strata(strata))
##' summary(cifdob)
##' head(iid(cifdob)) 
##' 
##' newdata <- data.frame(tcell=1,platelet=1,age=0,strata=1:2,id=1)
##' riskratio <- function(p) {
##'   cifdob$coef <- p
##'   p <- predict(cifdob,newdata,se=0)
##'   return(p[1]/p[2])
##' }
##' lava::estimate(cifdob,f=riskratio)
##' 
##' predict(cifdob,newdata)
##' (p1 <- predict(cifs1,newdata))
##' (p2 <- predict(cifs2,newdata))
##' p1[1,1]/p2[1,1]
##' 
##' @aliases logitIPCW binregt
##' @export
binreg <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("II","I"),
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),
	   Ydirect=NULL,...)
{# {{{
	monotone <- TRUE
cl <- match.call()# {{{
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting later 
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- 1*(status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing <- any(!(Causes %in% cause))
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

 nevent <- sum((status %in% cause)*(exit<=time))
 ## if event before time or alive, then uncensored, equality for both censored and events  
 obs <- (exit<=time & (statusC==0)) | (exit>=time)

  if (is.null(cens.weights))  { ## {{{
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else { se <- FALSE; resC <- formC <- NULL} ## }}}
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  if (is.null(augmentation))  augmentation=rep(0,p)
  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX

 if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst")
	     Y <-  c(pmin(exit,time))
             else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
  Yresp <- c(Y)
  Y <- Yipcw <- c(Y*obs/cens.weights)
  IPCW <- c(obs/cens.weights)

 if (se & monotone) {## {{{ censoring adjustment of variance  and infinitissimal jacknife pseudo augmentation
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    X2 <-  X2[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    Y <- Y[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    h  <-  Mrevcumsumstrata(X*Y,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    btime <- 1*(exit<time)
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    IhdLam0 <- Mcumsumstrata(h*S0i2*btime,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)
###    MGCiid <- Msumstrata(MGt,2,sumstrata,xx$id,mid+1)
    MGCiid <- Msumstrata(MGt,xx$id,mid+1)

   if (type[1]=="II") { ##  pseudo-value type augmentation
    hYt  <-  revcumsumstrata(Y,xx$strata,xx$nstrata)
    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
    U <- rep(0,length(xx$strata))
    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
    MGt <- X*c(U-IhdLam0)*c(xx$weights)
###    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
    MGtiid <- Msumstrata(MGt,xx$id,mid+1)
    augmentation  <-  colSums(MGtiid) + augmentation
    ###
    EXt  <-  Mrevcumsumstrata(X,xx$strata,xx$nstrata)
    IEXhYtdLam0 <- Mcumsumstrata(EXt*c(hYt)*S0i*S0i2*btime,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
    ###
    MGCiid2 <- Msumstrata(MGt2,xx$id,mid+1)
    ### Censoring Variance Adjustment 
    MGCiid <- MGCiid+(MGtiid-MGCiid2)
   }
   ## use data ordered by time (keeping track of id also)
   id <- xx$id
   }  else {
	  MGCiid <- 0
  }## }}}

 ## default links
 if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
	 if (outcome[1]=="years-lost") model <- "exp"
 }

obj <- function(pp,all=FALSE)
{ # {{{
lp <- c(X %*% pp+offset)

    if (model[1]=="exp") {
	 p <- exp(lp) 
         D2logl <- c(weights*p)*X2 
	 if (monotone) {
	    Dlogl <- weights*X*c(Y-p) 
            D2logl <- c(weights*p)*X2 
	 } else { 
	    Dlogl <- weights*X*c((Y-p)*p)
            D2logl <- c(weights*p^2)*X2 
	 }
     } else if (model[1]=="lin") {
	 p <- lp
         D2logl <- c(weights)*X2
         Dlogl <- weights*X*c(Y-p)
    } else if (model[1]=="logit") {
	p <- expit(lp)
        if (monotone) {
           D2logl <- c(weights*p/(1+exp(lp)))*X2
           Dlogl <- weights*X*c(Y-p)
	} else {
           varp <- p/(1+exp(lp))
           Dlogl <- weights*X*c((Y-p)*varp)
           D2logl <- c(weights*varp^2)*X2
	}
} else stop("link functions must be logit,exp,lin\n") 
ploglik <- sum(weights*(Y-p)^2)
## to avoid using ploglik for fitting, with exp,logit only solve estimating equation
if (model[1]!="lin") ploglik <- 0

###D2log <- apply(D2logl,2,sum)
###gradient <- apply(Dlogl,2,sum)+augmentation
D2log <- colSums(D2logl)
gradient <- colSums(Dlogl)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ploglik <- sum(weights*(Y-p)^2)
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
###      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      beta.iid <-  Msumstrata(beta.iid,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

## setting default for NR 
dots <- list(...)
if (length(dots)==0) {
   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
   else control <- NULL
} else control <- dots[[1]]

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
###	      if (!se) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else {
	      val <- c(list(coef=beta),obj(beta,all=TRUE))
  }
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid))

  if ( (!monotone) & se) { ## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    lp <- c(X %*% val$coef+offset)
    Y <- Y[ord]

   if (model[1]=="exp") {
	 p <- exp(lp) 
         Yglm <- weights*X*c(Y*p)
    } else if (model[1]=="lin") {
	 p <- lp
         Yglm <- weights*X*c(Y)
    } else if (model[1]=="logit") {
	 p <- expit(lp) 
         varp <- p/(1+exp(lp))
         Yglm <- weights*X*c(Y*varp)
    }

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
  }## }}}

  val$call <- cl
  val$MGciid <- MGCiid 
  val$call.id <- call.id
  val$id <- orig.id
  val$name.id <- name.id
  val$nid <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL 
  if (se)  val$iid  <- val$iid+(MGCiid %*% val$ihessian)
  if (!is.null(name.id)) val$iid <- nameme(val$iid,name.id)
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$augmentation <- augmentation
  val$model <- model[1]
  val$outcome <- outcome[1]
  val$Yipcw <- Yipcw
  val$Y     <- Yresp
  val$IPCW   <- IPCW
  val$Causes <- Causes
  val$nevent <- nevent
  val$design <- des

  class(val) <- "binreg"
  return(val)
}# }}}

###binreg <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("II","I"),
###	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
###	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
###	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),
###	   Ydirect=NULL,...)
###{# {{{
###	monotone <- TRUE
###cl <- match.call()# {{{
######    m <- match.call(expand.dots = TRUE)[1:3]
###    des <- proc_design(
###        formula,
###        data = data,
###        specials = c("offset", "weights", "cluster"),
###        intercept = TRUE
###    )
###    Y <- des$y
###    if (!inherits(Y, c("Event", "Surv"))) {
###        stop("Expected a 'Surv' or 'Event'-object")
###    }
###    if (ncol(Y) == 2) {
###        exit <- Y[, 1]
###        entry <- rep(0, nrow(Y))
###        status <- Y[, 2]
###    } else {
###        entry <- Y[, 1]
###        exit <- Y[, 2]
###        status <- Y[, 3]
###    }
###    X <- des$x
###    des.weights <- des$weights
###    des.offset  <- des$offset
###    id      <- des$cluster
###  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
###
###  ### possible handling of id to code from 0:(antid-1)
###  call.id <- id 
###  conid <- construct_id(id,nrow(X),namesX=rownames(X))
###  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
###  ## id before time-sorting later 
###  orig.id <- id
###
###  ## take offset and weight first from formula, but then from arguments
###  if (is.null(des.offset)) {
###	  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  } else offset <- des.offset
###  if (is.null(des.weights)) {
###	  if (is.null(weights)) weights <- rep(1,length(exit)) 
###  } else weights <- des.weights
#### }}}
###
###  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
###  statusC <- 1*(status %in% cens.code) 
###  statusE <- (status %in% cause) & (exit<= time) 
###  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
###  kmt <- kaplan.meier
###
###  ucauses  <-  sort(unique(status))
###  ccc <- which(ucauses %in% cens.code)
###  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
###  competing <- any(!(Causes %in% cause))
###  data$id__ <- id
###  data$exit <- exit
###  data$statusC <- statusC 
###  cens.strata <- cens.nstrata <- NULL 
###
### nevent <- sum((status %in% cause)*(exit<=time))
### ## if event before time or alive, then uncensored, equality for both censored and events  
### obs <- (exit<=time & (statusC==0)) | (exit>=time)
###
###  if (is.null(cens.weights))  { ## {{{
###      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
###      resC <- phreg(formC,data)
###      if (resC$p>0) kmt <- FALSE
###      exittime <- pmin(exit,time)
###      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
###      ## strata from original data 
###      cens.strata <- resC$strata[order(resC$ord)]
###      cens.nstrata <- resC$nstrata
###  } else { se <- FALSE; resC <- formC <- NULL} ## }}}
###  expit  <- function(z) 1/(1+exp(-z)) ## expit
###
###  p <- ncol(X)
###  if (is.null(beta)) beta <- rep(0,p)
###  if (is.null(augmentation))  augmentation=rep(0,p)
###  X <-  as.matrix(X)
###  X2  <- .Call("vecCPMat",X)$XX
###
### if (!is.null(Ydirect)) Y <-  Ydirect else {
###     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
###     else { if (!competing) {
###	     if (outcome[1]=="rmst")
###	     Y <-  c(pmin(exit,time))
###             else Y <-  c((time-pmin(exit,time)))
###            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
###     }
###  }
###  Yresp <- c(Y)
###  Y <- Yipcw <- c(Y*obs/cens.weights)
###  IPCW <- c(obs/cens.weights)
###
### if (se & monotone) {## {{{ censoring adjustment of variance  and infinitissimal jacknife pseudo augmentation
###    ### order of sorted times
###    ord <- resC$ord
###    X <-  X[ord,,drop=FALSE]
###    X2 <-  X2[ord,,drop=FALSE]
###    status <- status[ord]
###    exit <- exit[ord]
###    weights <- weights[ord]
###    offset <- offset[ord]
###    Y <- Y[ord]
###
###    xx <- resC$cox.prep
###    S0i2 <- S0i <- rep(0,length(xx$strata))
###    S0i[xx$jumps+1]  <- 1/resC$S0
###    S0i2[xx$jumps+1] <- 1/resC$S0^2
###    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
###    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
###    ### Cens-Martingale as a function of time and for all subjects to handle strata 
###    btime <- 1*(exit<time)
###    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
###    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
###
###    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
###    mid <- max(id)
###    MGCiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
###
###   if (type[1]=="II") { ##  pseudo-value type augmentation
###    hYt  <-  revcumsumstrata(Y,xx$strata,xx$nstrata)
###    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
###    U <- rep(0,length(xx$strata))
###    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
###    MGt <- X*c(U-IhdLam0)*c(xx$weights)
###    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
###    augmentation  <-  apply(MGtiid,2,sum) + augmentation
###    ###
###    EXt  <-  apply(X,2,revcumsumstrata,xx$strata,xx$nstrata)
###    IEXhYtdLam0 <- apply(EXt*c(hYt)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
###    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
###    ###
###    MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
###    ### Censoring Variance Adjustment 
###    MGCiid <- MGCiid+(MGtiid-MGCiid2)
###   }
###   ## use data ordered by time (keeping track of id also)
###   id <- xx$id
###   }  else {
###	  MGCiid <- 0
###  }## }}}
###
### ## default links
### if (model[1]=="default") {
###	 if (outcome[1]=="cif") model <- "logit"
###	 if (outcome[1]=="rmst") model <- "exp"
###	 if (outcome[1]=="rmtl") model <- "exp"
###	 if (outcome[1]=="years-lost") model <- "exp"
### }
###
###obj <- function(pp,all=FALSE)
###{ # {{{
###lp <- c(X %*% pp+offset)
###
###    if (model[1]=="exp") {
###	 p <- exp(lp) 
###         D2logl <- c(weights*p)*X2 
###	 if (monotone) {
###	    Dlogl <- weights*X*c(Y-p) 
###            D2logl <- c(weights*p)*X2 
###	 } else { 
###	    Dlogl <- weights*X*c((Y-p)*p)
###            D2logl <- c(weights*p^2)*X2 
###	 }
###     } else if (model[1]=="lin") {
###	 p <- lp
###         D2logl <- c(weights)*X2
###         Dlogl <- weights*X*c(Y-p)
###    } else if (model[1]=="logit") {
###	p <- expit(lp)
###        if (monotone) {
###           D2logl <- c(weights*p/(1+exp(lp)))*X2
###           Dlogl <- weights*X*c(Y-p)
###	} else {
###           varp <- p/(1+exp(lp))
###           Dlogl <- weights*X*c((Y-p)*varp)
###           D2logl <- c(weights*varp^2)*X2
###	}
###} else stop("link functions must be logit,exp,lin\n") 
###ploglik <- sum(weights*(Y-p)^2)
##### to avoid using ploglik for fitting, with exp,logit only solve estimating equation
###if (model[1]!="lin") ploglik <- 0
###
###D2log <- apply(D2logl,2,sum)
###gradient <- apply(Dlogl,2,sum)+augmentation
###np <- length(pp)
###hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)
###
###  if (all) {
###      ploglik <- sum(weights*(Y-p)^2)
###      ihess <- pinv(hessian)
###      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
###      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
###      robvar <- crossprod(beta.iid)
###      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
###	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
###      return(val)
###  }  
### structure(-ploglik,gradient=-gradient,hessian=hessian)
###}# }}}
###
##### setting default for NR 
###dots <- list(...)
###if (length(dots)==0) {
###   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
###   else control <- NULL
###} else control <- dots[[1]]
###
###  p <- ncol(X)
###  opt <- NULL
###  if (p>0) {
###  if (no.opt==FALSE) {
###      if (tolower(method)=="nr") {
###          tim <- system.time(opt <- lava::NR(beta,obj,control=control))
###	  opt$timing <- tim
###	  opt$estimate <- opt$par
###      } else {
###	  opt <- nlm(obj,beta,...)
###	  opt$method <- "nlm"
###      }
###      cc <- opt$estimate; 
######	      if (!se) return(cc)
###      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
###      } else {
###	      val <- c(list(coef=beta),obj(beta,all=TRUE))
###  }
###  } else {
###      val <- obj(0,all=TRUE)
###  }
###
###  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
###  val <- c(val,list(time=time,formula=formula,formC=formC,
###    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
###    n=length(exit),nevent=nevent,ncluster=nid))
###
###  if ( (!monotone) & se) { ## {{{ censoring adjustment of variance 
###    ### order of sorted times
###    ord <- resC$ord
###    X <-  X[ord,,drop=FALSE]
###    status <- status[ord]
###    exit <- exit[ord]
###    weights <- weights[ord]
###    offset <- offset[ord]
###    lp <- c(X %*% val$coef+offset)
###    Y <- Y[ord]
###
###   if (model[1]=="exp") {
###	 p <- exp(lp) 
###         Yglm <- weights*X*c(Y*p)
###    } else if (model[1]=="lin") {
###	 p <- lp
###         Yglm <- weights*X*c(Y)
###    } else if (model[1]=="logit") {
###	 p <- expit(lp) 
###         varp <- p/(1+exp(lp))
###         Yglm <- weights*X*c(Y*varp)
###    }
###
###    xx <- resC$cox.prep
###    S0i2 <- S0i <- rep(0,length(xx$strata))
###    S0i[xx$jumps+1]  <- 1/resC$S0
###    S0i2[xx$jumps+1] <- 1/resC$S0^2
###    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
###    ## to make \int h(s)/Ys  dM_i^C(s) 
###    h  <-  apply(Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
###    ### Cens-Martingale as a function of time and for all subjects to handle strata 
###    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
###    U <- matrix(0,nrow(xx$X),ncol(X))
###    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
###    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
###    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
###  }## }}}
###
###  val$call <- cl
###  val$MGciid <- MGCiid 
###  val$call.id <- call.id
###  val$id <- orig.id
###  val$name.id <- name.id
###  val$nid <- nid
###  val$iid.naive <- val$iid
###  val$naive.var <- NULL 
###  if (se)  val$iid  <- val$iid+(MGCiid %*% val$ihessian)
###  if (!is.null(name.id)) val$iid <- nameme(val$iid,name.id)
###  robvar <- crossprod(val$iid)
###  val$var <-  val$robvar <- robvar
###  val$se.robust <- diag(robvar)^.5
###  val$se.coef <- diag(val$var)^.5
###  val$cause <- cause
###  val$cens.code <- cens.code 
###  val$augmentation <- augmentation
###  val$model <- model[1]
###  val$outcome <- outcome[1]
###  val$Yipcw <- Yipcw
###  val$Y     <- Yresp
###  val$IPCW   <- IPCW
###  val$Causes <- Causes
###  val$nevent <- nevent
###  val$design <- des
###
###  class(val) <- "binreg"
###  return(val)
###}# }}}


#' Influence curve components for binomial regression ATE
#'
#' Extracts and scales influence function (IC) components from a fitted
#' binomial regression ATE object or raw IC matrix. Depending on the object
#' class and requested `type`, returns one or more components combined by
#' column-binding.
#'
#' @param x An object of class \code{ATE}, or an object containing an
#'   \code{iid} component.
#' @param type Character vector specifying which influence curve components
#'   to return. Possible values are:
#'   \describe{
#'     \item{coef}{Influence curve for regression coefficients}
#'     \item{DR}{Doubly robust ATE influence curve}
#'     \item{G}{G-computation ATE influence curve}
#'   }
#'   If \code{x} inherits from \code{ATE} and \code{type} is missing,
#'   the default is \code{c("DR", "G")}.
#' @param ... Currently unused.
#'
#' @return A numeric matrix obtained by column-binding the selected
#' influence curve components. Each column corresponds to one element
#' of \code{type}, with column names preserved.
#'
#' @details
#' For objects of class \code{ATE}, the function extracts and rescales
#' the stored influence curve components:
#'
#' \itemize{
#'   \item \code{x$iid} for coefficient ICs
#'   \item \code{x$riskDR.iid} for doubly robust ATE ICs
#'   \item \code{x$riskG.iid} for G-computation ICs
#' }
#'
#' Each component is multiplied by its sample size (\code{NROW(.)}).
#'
#' For non-\code{ATE} objects, the function returns
#' \code{NROW(x$iid) * x$iid}.
#'
#' @examples
#' \dontrun{
#' # default ATE behavior (DR + G)
#' IC.binreg(fit)
#'
#' # only DR component
#' IC.binreg(fit, "DR")
#'
#' # multiple components
#' IC.binreg(fit, c("coef", "DR"))
#' }
#'
#' @export
IC.binreg <- function(x, type = c("coef", "DR", "G"), ...) { ## {{{ 

  # Default override for ATE objects
  if (inherits(x, "ATE") && missing(type)) {
    type <- c("DR", "G")
  }

  type <- match.arg(type, several.ok = TRUE)

  if (!inherits(x, "ATE")) {
    return(x$iid * NROW(x$iid))
  }

  comps <- list()

  for (t in type) {

    ic_t <- switch(t,
      "coef" = NROW(x$iid) * x$iid,
      "DR"   = NROW(x$riskDR.iid) * x$riskDR.iid,
      "G"    = NROW(x$riskG.iid) * x$riskG.iid,
      stop("Unknown type: ", t)
    )

    comps[[t]] <- ic_t
  }

  out <- do.call(cbind, comps)

  return(out)
}  ## }}} 

##' @export
print.binreg  <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.binreg <- function(object,...) {# {{{

if (!is.null(object$gradient)) { ## write warning if gradient not small 
gradient <- max(abs(object$gradient))
if (gradient > 0.000001) { cat("gradient:\n"); print(object$gradient) }
}

cc  <- estimate(coef=object$coef,vcov=object$var)$coefmat
V=object$var

res <- list(coef=cc,n=object$n,nevent=object$nevent,strata=NULL,
	    ncluster=object$ncluster,var=V,model=object$model[1])

if (object$model[1]=="exp" | object$model[1]=="logit") {
expC <- exp(lava::estimate(coef=coef(object),vcov=object$var)$coefmat[,c(1,3,4),drop=FALSE])
res <- c(res,list(exp.coef=expC))
}

## to add marginal estimates for binregATE 
if (!is.null(object$riskDR))  {
    marginalDR <- estimate(coef=object$riskDR,vcov=object$var.riskDR)$coefmat
    difmarginalDR <- estimate(coef=object$difriskDR,vcov=as.matrix(object$var.difriskDR))$coefmat
###    rownames(difmarginalDR) <- "differenceDR"
    marginalDR <- rbind(marginalDR,difmarginalDR)

    marginalG <- estimate(coef=object$riskG,vcov=object$var.riskG)$coefmat
    difG <- estimate(coef=object$difriskG,vcov=as.matrix(object$var.difriskG))$coefmat
###    rownames(difG) <- "differenceG"
    marginalG <- rbind(marginalG,difG) 
    res <- c(res,list(ateG=marginalG,ateDR=marginalDR))

    if (!is.null(object$attc)) {
    attc <- estimate(coef=object$attc,vcov=object$var.attc)$coefmat
    res <- c(res,list(attc=attc))
    }
}

class(res) <- "summary.binreg"
return(res)
}# }}}

##' @export
print.summary.binreg <- function(x,max.strata=5,...) { ## {{{

  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- levels(x$strata); colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  if (length(x$strata)>max.strata) {
      nn <- rbind(c(colSums(nn),length(x$strata)));
      colnames(nn) <- c("n","events","stratas")
      rownames(nn) <- ""
  } 
  print(nn,quote=FALSE)  
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coeffients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    if (x$model[1]=="exp" | x$model[1]=="logit") {
    cat("exp(coeffients):\n")
    printCoefmat(x$exp.coef,...)
    }
  }
  cat("\n")

 ## for binreg ATE
 if (!is.null(x$ateDR)) {
    cat("Average Treatment effects (G-formula) :\n")
    printCoefmat(x$ateG,...)
    cat("\n")

    cat("Average Treatment effects (double robust) :\n")
    printCoefmat(x$ateDR,...)
    cat("\n")

###    if (!is.null(x$attc)) {
###    cat("Average Treatment effects on Treated/Non-Treated (DR) :\n")
###    printCoefmat(x$attc,...)
###    cat("\n")
###    }

  }
  cat("\n")

} ## }}}

###}}} 


##' @export
vcov.binreg <- function(object,...) {# {{{
	return(object$var)
}# }}}

##' @export
coef.binreg <- function(object,...) {# {{{
	return(object$coef)
}# }}}

##' @export
predict.binreg <- function(object,newdata=NULL,se=TRUE,pred.iid=FALSE,level=0.95,design=FALSE,...)
{# {{{
   if (!is.null(newdata)) x <- update_design(object$design,data = newdata,response=FALSE) else x <- object$design
  Z <- x$x

  if (object$model[1]=="logit") {
  lp <- c(Z %*% object$coef)
  p <- expit(lp)
  preds <- p
  alpha <- 1-level

  if (se) {
     if (is.null(object$var)) covv <- vcov(object)  else covv <- object$var
     Dpv <- Z*exp(-lp)*p^2
     se <- apply((Dpv %*% covv)* Dpv,1,sum)^.5
     crit <- qnorm(1-alpha/2)
     cmat <- data.frame(pred=p,se=se,lower=p-1.96*se,upper=p+1.96*se)
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- cmat
  }
  if (pred.iid) {
      Piid <- object$iid  %*% t(Dpv)
      preds <- list(pred=preds,iid=Piid)
  }

  } else {

  lp <- c(Z %*% object$coef)
  if (object$model[1]=="exp") p <- exp(lp) else p <- lp
  preds <- p

  level <- 0.95
  alpha <- 1-level
  crit <- qnorm(1-alpha/2)

  if (se) {
     if (is.null(object$var)) covv <- vcov(object)  else covv <- object$var
###     if (object$model[1]=="dexp") Dpv <- Z*p^2 else 
     if (object$model[1]=="exp") Dpv <- Z*p else Dpv <- Z 
     se <- apply((Dpv %*% covv)* Dpv,1,sum)^.5
     cmat <- data.frame(pred=p,se=se,lower=p-crit*se,upper=p+crit*se)
     names(cmat)[1:4] <- c("pred","se","lower","upper")
     preds <- cmat
  }
  if (pred.iid) {
      Piid <- object$iid  %*% t(Dpv)
      preds <- list(pred=preds,iid=Piid,id=x$cluster)
  }

  }

  attr(preds,"id") <- x$cluster
  if (design) attr(preds,"x") <- Z

return(preds)
} # }}}

##' @export
binregt <- function(formula,data,cause=1,time=NULL,beta=NULL,
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,...)
{# {{{
  cl <- match.call()# {{{
###    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

  call.id <- id
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  ## id before time-sorting
  orig.id <- id

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

# }}}




  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in% cens.code) 
  statusE <- (status %in% cause)*outer(exit,time,"<=") 
  if (any(apply(statusE,2,sum))==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      cens.weights <- c()
      for (tt in time)  {
      exittime <- pmin(exit,tt)
      cens.weights1 <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      cens.weights <- cbind(cens.weights,cens.weights1)
      }
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  if (length(time)>1) {
      Yt <- outer(exit,time,"<=") 
      nevent <- apply(c(status %in% cause)*Yt,2,sum)
      Y <- c((status %in%cause)/cens.weights)*Yt
  } else {
      Y <- c((status %in%cause)*(exit<=time)/cens.weights)
      nevent <- sum((status %in%cause)*(exit<=time))
  }

  ppt <- length(time)
  p <- pd <- ncol(X)+(length(time)-1)
  if (is.null(beta)) beta <- rep(0,ncol(X)+ppt-1)
  X <-  as.matrix(X)
  X2o  <- .Call("vecMatMat",X[,-1,drop=FALSE],X[,-1,drop=FALSE])$vXZ
###  X2  <- .Call("vecMatMat",X,X)$vXZ

  if (is.null(augmentation))  augmentation=rep(0,p)

obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X[,-1,drop=FALSE] %*% pp[-seq(1,ppt)])
lp <- outer(lp,pp[1:ppt],"+")
p <- expit(lp)
ploglik <- sum(weights*(Y-p)^2)

Dloglt <- weights*(Y-p)
Dloglo <- weights*X[,-1]*apply((Y-p),1,sum)
Dlogl <- cbind(Dloglt,Dloglo)

wwp <- (weights*p/(1+exp(lp)))
wwpt <- apply(wwp,2,sum)
wwpo <- apply(wwp,1,sum)
D2loglo <- c(wwpo)*X2o
hessian <- matrix(0,pd,pd)
hessian[seq(1,ppt),seq(1,ppt)] <- diag(wwpt,ppt,ppt)
for (i in 1:ppt) hessian[i,(ppt+1):pd] <- apply(wwp[,i]*X[,-1,drop=FALSE],2,sum)
hessian[(ppt+1):pd,1:ppt]  <- t(hessian[1:ppt,(ppt+1):pd])
hessian[(ppt+1):pd,(ppt+1):pd] <- c(apply(D2loglo,2,sum))

gradient <- apply(Dlogl,2,sum)+augmentation

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik/nid,gradient=-gradient/nid,hessian=hessian/nid)
}# }}}

	  p <- pd 
	  opt <- NULL
	  if (p>0) {
	  if (no.opt==FALSE) {
	      if (tolower(method)=="nr") {
		  tim <- system.time(opt <- lava::NR(beta,obj,...))
		  opt$timing <- tim
		  opt$estimate <- opt$par
	      } else {
		  opt <- nlm(obj,beta,...)
		  opt$method <- "nlm"
	      }
	      cc <- opt$estimate; 
	      if (!se) return(cc)
	      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
	      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
	  } else {
	      val <- obj(0,all=TRUE)
	  }

	  if (length(time)==1) {
	  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X) }
	  else names(val$coef) <- c(paste("Intercept",time,sep=""),colnames(X)[-1])
	  val <- c(val,list(time=time,formula=formula,formC=formC,
	    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
	    n=length(exit),nevent=nevent,ncluster=nid))
	  

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    if (length(time)>1) {
         Yt <- outer(exit,time,"<=") 
         Y <- c((status%in% cause)/cens.weights)*Yt
	 Ys <- apply(Y,1,sum)
     } else {
         Ys <- Y <- c((status%in%cause)*(exit<=time)/cens.weights)
     }

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(cbind(Y,X[,-1]*Ys),2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
###    btime <- 1*outer(exit,time,"<") 
    IhdLam0 <- apply(h*c(S0i2),2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(X),pd)
    jtime <- outer(resC$jumptimes,time,"<")*1
    U[xx$jumps+1,] <- (h[xx$jumps+1,])/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
 
   }  else {
	  MGCiid <- 0
  }## }}}

  val$call <- cl
  val$MGciid <- MGCiid
  val$call.id <- call.id
  val$name.id <- name.id
  val$id <- orig.id
  val$nid <- nid
  val$iid.naive <- val$iid 
  if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian) else val$iid  <- val$iid
  val$naive.var <- val$var
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$model <- "logit"
  val$design <- des


  class(val) <- "binreg"
  return(val)
}# }}}

##' @export
logitIPCW <- function(formula,data,cause=1,time=NULL,beta=NULL,
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
###  m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

  ### possible handling of id to code from 0:(antid-1)
  call.id <- id 
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
  orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status%in% cens.code) 
  statusE <- (status%in%cause) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  statusC <- (status %in%cens.code) 
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  if (is.null(beta)) beta <- rep(0,ncol(X))
  p <- ncol(X)

  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX
  obs <- (exit<=time & status!=cens.code) | (exit>=time)
  weights <- obs*weights/c(cens.weights)
  cens.weights <- c(cens.weights)

  if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst")
	     Y <-  c(pmin(exit,time)) 
             else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
  Yipcw <- Y
 
 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum(Y)

 ## default links
 if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
	 if (outcome[1]=="years-lost") model <- "exp"
 }


obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X %*% pp+offset)

    if (model[1]=="exp") {
	 p <- exp(lp) 
         D2logl <- c(weights*p)*X2 
     } else if (model[1]=="lin") {
	 p <- lp
         D2logl <- c(weights)*X2
     } else if (model[1]=="logit") {
	p <- expit(lp)
        D2logl <- c(weights*p/(1+exp(lp)))*X2
     } else stop("link functions must be logit,exp,lin\n") 
ploglik <- sum(weights*(Y-p)^2)

if (model[1]=="exp") ploglik <- 0
Dlogl <- weights*X*c(Y-p)
D2log <- apply(D2logl,2,sum)
gradient <- apply(Dlogl,2,sum)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

## setting default for NR 
dots <- list(...)
if (length(dots)==0) {
   if (model[1]=="exp") control <- list(tol=1e-10,stepsize=0.5)  
   else control <- NULL
} else control <- dots[[1]]

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid,weights=weights))
  
 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    lp <- c(X %*% val$coef+offset)
    p <- expit(lp)
    Yglm <- weights*c(Y[ord]-p) # *(exit<=time)

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
  }  else {
	  MGCiid <- 0
  }## }}}

    val$Y <- Yipcw
    val$model <- model[1]
    val$outcome <- outcome[1]
    val$call <- cl
    val$MGciid <- MGCiid
    val$id <- orig.id
    val$call.id <- call.id
    val$name.id <- name.id
    val$nid <- nid
    val$iid.naive <- val$iid 
    if (se)
    val$iid  <- val$iid+(MGCiid %*% val$ihessian)
    if (!is.null(call.id)) val$iid <- nameme(val$iid,name.id)
    val$naive.var <- val$var
    robvar <- crossprod(val$iid)
    val$var <-  val$robvar <- robvar
    val$se.robust <- diag(robvar)^.5
    val$se.coef <- diag(val$var)^.5
    val$cause <- cause
    val$cens.code <- cens.code 
    val$augmentation <- augmentation
    val$design <- des

  class(val) <- "binreg"
  return(val)
}# }}}


##' Average Treatment Effect for Censored Competing Risks Data using Binomial Regression
##'
##' Estimates the average treatment effect (ATE) \eqn{E(Y(1) - Y(0))} for censored 
##' competing risks data using binomial regression with Inverse Probability of 
##' Censoring Weighting (IPCW).
##'
##' Under standard causal assumptions, the ATE can be estimated. These assumptions include:
##' \itemize{
##'   \item \strong{Consistency}: The observed outcome equals the potential outcome under the observed treatment.
##'   \item \strong{Ignorability}: \eqn{(Y(1), Y(0)) \perp A | X} (treatment assignment is independent of potential outcomes given covariates).
##'   \item \strong{Positivity}: All treatment levels have non-zero probability given covariates.
##' }
##'
##' The first covariate in the competing risks regression model must be the treatment variable, 
##' which should be coded as a factor. If the factor has more than two levels, multinomial 
##' logistic regression (\code{mlogit}) is used for propensity score modeling. In the absence 
##' of censoring, this reduces to ordinary logistic regression.
##'
##' The ATE is estimated using standard doubly robust estimating equations that are 
##' IPCW-censoring adjusted. As an alternative to binomial regression, 
##' \code{logitIPCWATE} provides an IPCW-weighted version of standard logistic regression.
##'
##' When \code{typeATE = "II"}, the estimating equation is augmented with:
##' \deqn{ (A/\pi(X)) \int E( O(t) | T \geq t, S(X))/ G_c(t,S(X)) d \hat M_c(s) }
##' when estimating the mean outcome for the treated group.
##'
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
##' }
##'
##' @param formula A formula object specifying the outcome and covariates (see \code{coxph}). 
##'   The first covariate should be the treatment variable coded as a factor.
##' @param data A data frame containing the variables in the formula.
##' @param cause Numeric scalar indicating the cause of interest for competing risks.
##' @param time Numeric scalar indicating the time point of interest.
##' @param beta Optional numeric vector of starting values for the coefficients.
##' @param treat.model A formula specifying the logistic treatment model given covariates 
##'   (e.g., \code{treatment ~ covariate1 + covariate2}).
##' @param cens.model A formula specifying the censoring model. Only stratified Cox models 
##'   without covariates are supported (e.g., \code{~ strata(group)}).
##' @param offset Optional numeric vector of offsets for the partial likelihood.
##' @param weights Optional numeric vector of weights for the score equations.
##' @param cens.weights Optional numeric vector of pre-calculated censoring weights. 
##'   If \code{NULL}, weights are estimated internally.
##' @param se Logical. If \code{TRUE}, computes standard errors with IPCW adjustment. 
##'   If \code{FALSE}, assumes IPCW weights are known.
##' @param type Character string. Either \code{"I"} (classic binomial regression) or 
##'   \code{"II"} (adds augmentation term for efficiency).
##' @param kaplan.meier Logical. If \code{TRUE}, uses Kaplan-Meier for IPCW weights; 
##'   otherwise uses \eqn{\exp(-\text{Baseline})}.
##' @param cens.code Numeric code representing censored observations in the status variable.
##' @param no.opt Logical. If \code{TRUE}, optimization is skipped and starting values are used.
##' @param method Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.
##' @param augmentation Optional numeric vector for augmenting binomial regression.
##' @param outcome Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function, 
##'   \eqn{F(t|X)}), \code{"rmst"} (Restricted Mean Survival Time, \eqn{E(\min(T, t) | X)}), 
##'   or \code{"rmtl"} (Restricted Mean Time Lost, \eqn{E(I(\epsilon=\text{cause})(t - \min(T,t)) | X)}).
##' @param model Character string. Link function for the outcome model: \code{"exp"} or 
##'   \code{"lin"} (identity). For \code{"cif"}, \code{"logit"} is typically used.
##' @param Ydirect Optional numeric vector. Use this outcome Y with IPCW version instead of 
##'   constructing one from \code{outcome}.
##' @param typeATE Character string. Either \code{"II"} (censoring augmentation of the 
##'   estimating equation) or \code{"I"} (standard).
##' @param ... Additional arguments passed to lower-level functions (e.g., \code{binreg} 
##'   that fits the outcome model).
##'
##' @return An object of class \code{c("binreg", "ATE")} containing:
##' \item{coef}{Estimated coefficients from the outcome model.}
##' \item{riskDR}{Double-robust marginal risk estimates for each treatment level.}
##' \item{riskG}{G-formula marginal risk estimates for each treatment level.}
##' \item{difriskDR}{Difference in risks (ATE) using double-robust estimator.}
##' \item{difriskG}{Difference in risks (ATE) using G-formula estimator.}
##' \item{riskDR.iid, riskG.iid}{Influence functions for marginal risk estimates.}
##' \item{var, var.riskDR, var.riskG}{Variance-covariance matrices.}
##' \item{se.coef, se.riskDR, se.riskG}{Standard errors.}
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{binreg}}, \code{\link{logitIPCWATE}}, \code{\link{logitATE}}, 
##'   \code{\link{binregG}}
##' @examples
##' data(bmt)
##' dfactor(bmt)  <-  ~.
##'
##' brs <- binregATE(Event(time,cause)~tcell.f+platelet+age,bmt,time=50,cause=1,
##'	  treat.model=tcell.f~platelet+age)
##' summary(brs)
##' head(brs$riskDR.iid)
##' head(brs$riskG.iid)
##'
##' brsi <- binregATE(Event(time,cause)~tcell.f+tcell.f*platelet+tcell.f*age,bmt,time=50,cause=1,
##'	  treat.model=tcell.f~platelet+age)
##' summary(brsi)
##' head(brs$riskDR.iid)
##' head(brs$riskG.iid)
##' @seealso [kumarsim()] [kumarsimRCT()]
##' @aliases logitIPCWATE logitATE normalATE
##' @export
binregATE <- function(formula,data,cause=1,time=NULL,beta=NULL,treat.model=~+1,cens.model=~+1,
   offset=NULL,weights=NULL,cens.weights=NULL,se=TRUE,type=c("II","I"),
   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
   outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,typeATE="II",...)
{# {{{
  cl <- match.call()# {{{
###  m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

 call.id <- id;
 conid <- construct_id(id,nrow(X),namesX=rownames(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 idclust <- id; nclust <- nid

 ## take each data-record with its own iid or organize after idclust later 
 nid <- length(id); 
 id <- 0:(nid-1); 

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  ## {{{ setting up status, response, censoring model 
  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in%cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  statusC <- (status %in%cens.code) 
  data$id__ <- id 
  data$exit <- exit
  data$statusC <- statusC 
  n <- length(exit)
  cens.strata <- cens.nstrata <- NULL 

  call.cw <- cens.weights
  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  X <-  as.matrix(X)
  ###  X2  <- .Call("vecMatMat",X,X)$vXZ
  X2  <- .Call("vecCPMat",X)$XX
  ###
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)>=1) Causes <- ucauses[-ccc] else Causes <- ucauses
  obs <- (exit<=time & (status %in% Causes)) | (exit>=time)

 if (!is.null(Ydirect)) Y <-  Ydirect*obs/cens.weights else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time)/cens.weights)
     else { if (!competing) {
	     if (outcome[1]=="rmst") Y <-  c(pmin(exit,time)*obs)/cens.weights 
	     else Y <-  c((time-pmin(exit,time))*obs)/cens.weights 
            } else Y <- c((status %in% cause)*(time-pmin(exit,time))*obs)/cens.weights
     }
  }
 nevent <- sum((status %in% cause)*(exit<=time))

 ## }}}

## change id from call to id__ to get ordering as data
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))

  val <- binreg(formula,data,cause=cause,time=time,beta=beta,type=type,
	cens.model=cens.model,se=se,kaplan.meier=kaplan.meier,
        offset=offset,weights=weights,cens.weights=call.cw,
        cens.code=cens.code,no.opt=no.opt,method=method,augmentation=augmentation,
        outcome=outcome[1],model=model[1],Ydirect=Ydirect,...)

model <- val$model[1]

# {{{ computation of ate, att, atc and their influence functions

### treatment is rhs of treat.model 
if (length(all.vars(treat.model))==0) 
{ respformula  <-  as.formula( paste(all.vars(formula)[3],"~."))
  treat.model  <- update.formula(treat.model,respformula)
###  cat("Using propensity score model :\n "); print(treat.model)
}
treat.name <-  all.vars(treat.model)[1]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 
## treatvar, 1,2,...,nlev or 1,2
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
ytreat <- ntreatvar-1

## dropping cluster here 
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(treatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(treatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2
   p1lp <-   X %*% val$coef+offset
   if (model[1]=="logit") { p1 <- expit(p1lp) } else {
    if (model[1]=="exp") { p1 <- exp(p1lp); } else { p1 <- p1lp;}
   }


k <- 0
DePsia <- DariskG <- DaPsia <- list(); 
pia <- Ya <- riskG <- riska <- c(); 
datA <- dkeep(data,x=all.vars(formula))
xlev <- lapply(datA,levels)
formulanc <- drop.specials(formula,"cluster")
a <- nlevs[1]
for (a in nlevs) {# {{{
	k <- k+1
	datA[,treat.name] <- a
	Xa <- model.matrix(formulanc[-2],datA,xlev=xlev)
        lpa <- Xa %*% val$coef+offset
	if (model[1]=="logit") {
	   ma <- expit(lpa); Dma  <-  Xa*c(ma/(1+exp(lpa)))
	} else {
	    if (model[1]=="exp") { ma <- exp(lpa);  Dma<-c(ma)*Xa; } else { ma <- lpa; Dma <- Xa }
	}
	paka <- pal[,k]
	riska <- cbind(riska,((treatvar==a)/paka)*(Y-ma)+ma)
	riskG <- cbind(riskG,ma)
	Ya <- cbind(Ya,Y*((treatvar==a)/paka))
	pia <- cbind(pia,((treatvar==a)/paka))
        DariskG[[k]] <- apply(Dma,2,sum)
        DePsia[[k]] <-  apply(Dma*(1-(treatvar==a)/paka),2,sum)
        DaPsia[[k]] <-  apply(DPai*(treatvar==a)*c(Y-p1),2,sum)
}# }}}

augmentationATE <- 0
 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    Xtreat <- Xtreat[ord,,drop=FALSE]
    ytreat <- ytreat[ord]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    Ya <- Ya[ord,]
    pal <- pal[ord]
    piaC <- pia[ord,]
    cens.weights <- cens.weights[ord]
    lp <- c(X %*% val$coef+offset)
    obs <- obs[ord]
    p <- expit(lp)

  Y <- Y[ord]
  Yc <- matrix(Y,ncol=1)

 Y <- Y*weights 
 xx <- resC$cox.prep
 S0i2 <- S0i <- rep(0,length(xx$strata))
 S0i[xx$jumps+1]  <- 1/resC$S0
 S0i2[xx$jumps+1] <- 1/resC$S0^2
 ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
 ## to make \int h(s)/Ys  dM_i^C(s) 
 has  <-  apply(Ya,2,revcumsumstrata,xx$strata,xx$nstrata)
###    hattc  <-  apply(cbind(ytreat-pal*(1-ytreat)/(1-pal),-(1-ytreat)+(1-pal)*ytreat/pal)*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    btime <- 1*(exit<time)
    IhdLamhas <- apply(has*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    Uas <- matrix(0,nrow(xx$X),ncol(has))
    Uas[xx$jumps+1,] <- (resC$jumptimes<time)*has[xx$jumps+1,] /c(resC$S0)
    MGtas <- (Uas[,drop=FALSE]-IhdLamhas)*c(xx$weights)

    if (typeATE=="II") {
            h  <-  apply(Yc,2,revcumsumstrata,xx$strata,xx$nstrata)
	    IhdLam <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
	    U <- matrix(0,nrow(xx$X),1)
	    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
	    MGateII <- piaC*c(U[,drop=FALSE]-IhdLam)*c(xx$weights)
	    augmentationATE <- apply(MGateII,2,sum)

	    hapi  <-  apply(piaC,2,revcumsumstrata,xx$strata,xx$nstrata)
	    IhaidLam <- apply(hapi*c(h)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
	    Uai <- matrix(0,nrow(xx$X),ncol(hapi))
	    Uai[xx$jumps+1,] <- (resC$jumptimes<time)*c(h[xx$jumps+1,])*hapi[xx$jumps+1,]/c(resC$S0)^2
	    MGait <- (Uai[,drop=FALSE]-IhaidLam)*c(xx$weights)
	    MGtII <- MGateII - MGait
    }
    ### Censoring Variance Adjustment \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(xx$id)+1
    MGCiidas <- apply(MGtas,2,sumstrata,xx$id,mid)
    if (typeATE=="II") {
        MGtII <-  apply(MGtII,2,sumstrata,xx$id,mid)
        MGCiidas  <-  MGCiidas+MGtII
    } else MGtII <- 0
 
  }  else { MGCiid <- MGCiidas <- 0 }
## }}}

################################
### estimates risk, att, atc
################################
###val$riskDR <- apply(riska,2,mean)
val$riskDR <- apply(riska,2,mean)+augmentationATE/nid
val$riskG<-  apply(riskG,2,mean)
names(val$riskDR) <- paste("treat",nlevs,sep="")
names(val$riskG) <- paste("treat",nlevs,sep="")

################################
## iid's of marginal risk estimates 
################################

k <- 1
iidrisk <- c()
riskG.iid <- c()
for (a in nlevs) {
	iidbasea <- c(sumstrata(riska[,k]-val$riskDR[k],id,nid))
	iidcifa <- c(DePsia[[k]] %*% t(val$iid))
	iidpala <- c(DaPsia[[k]] %*% t(iidalpha))
	if (se)  iidGca <- MGCiidas[,k] else iidGca<-0 
        ###
	iidriskDR <- (iidbasea+iidcifa+iidpala+iidGca)/nid
        iidrisk <- cbind(iidrisk,iidriskDR)
	iidriskG <- c(sumstrata(riskG[,k]-val$riskG[k],id,nid))
	iidriskG <- c(iidriskG)/nid+c(DariskG[[k]] %*% t(val$iid))/nid
        riskG.iid <- cbind(riskG.iid,iidriskG)
	k <- k+1
}
# }}}

val$call.id <- call.id
val$name.id  <- name.id
val$id <- idclust
val$nid  <- nclust

# {{{ output variances and se for ate; cluster correction
val$iid <- apply(val$iid,2,sumstrata,idclust,nclust)
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$MGciid <- NULL
val$iid.naive <- NULL

## outcome model from binreg
iidrisk <- apply(iidrisk,2,sumstrata,idclust,nclust)
val$var.riskDR <- crossprod(iidrisk); 
val$se.riskDR <- diag(val$var.riskDR)^.5
val$riskDR.iid <- iidrisk

pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
contrast <- -pdiff(length(nlevs))
nncont <- c()
for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
rownames(contrast) <- nncont

mm <- estimate(coef=val$riskDR,vcov=val$var.riskDR,f=contrast)
val$difriskDR <- mm$coef 
names(val$difriskDR) <-  rownames(contrast) 
val$var.difriskDR <- mm$vcov 
val$se.difriskDR <- diag(val$var.difriskDR)^.5

riskG.iid <- apply(riskG.iid,2,sumstrata,idclust,nclust)
###
val$riskG.iid <- riskG.iid
val$var.riskG <- crossprod(val$riskG.iid)
val$se.riskG <- diag(val$var.riskG)^.5
###
mm <- estimate(coef=val$riskG,vcov=val$var.riskG,f=contrast)
val$difriskG <- mm$coef 
names(val$difriskG) <-  rownames(contrast) 
val$var.difriskG <- mm$vcov 
val$se.difriskG <- diag(val$var.difriskG)^.5

###### DR-estimator, G add names to iid 
if (!is.null(call.id)) {
    val$riskDR.iid <- nameme(iidrisk,name.id)
    val$riskG.iid  <- nameme(riskG.iid,name.id)
    val$iid        <- nameme(val$iid,name.id)
}
## }}}

  class(val) <- c("binreg","ATE")
  return(val)
}# }}}

##' @export
estimate.ATE  <- function(x,type=c("all","DR","G"),...)
{
if (type[1]=="all") {
iidGDR <- cbind(x$riskG.iid,x$riskDR.iid)
coefs <- c(x$riskG,x$riskDR)
names(coefs)  <- c(paste("G",names(x$riskG),sep="-"), paste("DR",names(x$riskG),sep="-"))
} else if (type[1]=="DR") {
iidGDR <- x$riskDR.iid
coefs <- x$riskDR
names(coefs)  <- paste("DR",names(x$riskG),sep="-")
} else {
iidGDR <- x$riskG.iid
coefs <- x$riskG
names(coefs)  <- paste("G",names(x$riskG),sep="-")
}
out <-estimate(coef=coefs,IC=iidGDR*nrow(iidGDR),...)
return(out)
}


##' G-Estimator for Binomial Regression Model (Standardized Estimates)
##'
##' Computes the G-estimator (G-formula) for standardized risk estimates based on a 
##' fitted \code{binreg} object. The G-estimator standardizes predictions over the 
##' covariate distribution in the data:
##' \deqn{ \hat F(t, A=a) = n^{-1} \sum_{i=1}^n \hat F(t, A=a, Z_i) }
##'
##' This function assumes that the first covariate in the original model formula 
##' represents the treatment or exposure variable (\eqn{A}). It calculates the 
##' marginal risk for specified values of \eqn{A} by averaging the conditional 
##' predictions over the observed covariate distribution \eqn{Z}.
##'
##' The function returns influence functions for these risk estimates, allowing 
##' for the computation of standard errors and confidence intervals.
##'
##' If the first covariate is a factor, contrasts between all levels are computed 
##' automatically. If it is continuous, specific values must be provided via 
##' \code{Avalues}.
##'
##' @section References:
##' \itemize{
##'   \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
##' }
##'
##' @param x An object of class \code{"binreg"} obtained from \code{binreg()} or \code{logitIPCW()}.
##' @param data A data frame containing the covariates to be used for averaging the risk 
##'   estimates. This should ideally be the same data used to fit the model, or a 
##'   representative sample.
##' @param Avalues Numeric or factor vector specifying the values of the first covariate 
##'   (\eqn{A}) for which to compute standardized risks. 
##'   \itemize{
##'     \item If the first covariate is a factor and \code{Avalues} is \code{NULL}, 
##'       all levels of the factor are used.
##'     \item If the first covariate is continuous, \code{Avalues} must be provided.
##'   }
##' @param varname Optional character string specifying the name of the variable to be 
##'   treated as the treatment/exposure variable. If \code{NULL}, the first variable 
##'   in the model formula is used.
##'
##' @return An object of class \code{"survivalG"} containing:
##' \item{risk}{A table of standardized risk estimates for each value of \code{Avalues}.}
##' \item{risk.iid}{Influence functions for the standardized risk estimates.}
##' \item{difference}{Pairwise differences in risks between levels of \code{A}.}
##' \item{ratio}{Risk ratios between levels of \code{A}.}
##' \item{vcov}{Variance-covariance matrix of the risk estimates.}
##' \item{model}{The link function used in the original model.}
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{binreg}}, \code{\link{binregATE}}
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' bmt$event <- (bmt$cause!=0)*1
##' 
##' b1 <- binreg(Event(time,cause)~age+tcell+platelet,bmt,cause=1,time=50)
##' sb1 <- binregG(b1,bmt,Avalues=c(0,1,2))
##' summary(sb1)
##' @export
binregG <- function(x,data,Avalues=NULL,varname=NULL)
{# {{{

### dealing with first variable 
if (is.null(varname))  {
   treat.name <- all.vars(update.formula(x$formula,1~.))[1]
} else treat.name <- varname
treatvar <- data[,treat.name]

if (is.factor(treatvar)) {
   ## treatvar, levels to look at 
   if (is.null(Avalues)) nlevels <- levels(treatvar) else nlevels <- Avalues
} else {
   nlevels <- Avalues
}
if (is.null(nlevels)) stop("must specify values")

desx <- update_design(x$design,data=data)
if (is.null(desx$cluster)) { ## data must be same as data used for x
   id <- x$id
} else {
  id <- desx$cluster-1
}
datA <- data

DariskG <- list()
risks <- c()
k <- 0
for (a in nlevels) {# {{{
k <- k+1
datA[,treat.name] <- a
pa <- predict.binreg(x,datA,se=0,design=TRUE)
Xa <- attr(pa,"x")
lpa <- Xa %*% x$coef
if (x$model[1]=="logit") {
###   pa <- expit(lpa)
   Dma  <-  Xa*c(pa/(1+exp(lpa)))
} else if (x$model[1]=="exp") {
###   pa <- exp(lpa)
   Dma  <-  Xa*c(pa) 
} else {
###   pa <- lpa
   Dma  <-  Xa
}
risks <- cbind(risks,pa)
DariskG[[k]] <- apply(Dma,2,sum)
}# }}}

Grisk <- apply(risks,2,mean)
Gest <- list(Gest=Grisk,iid=t(t(risks)-Grisk))

nid <- max(x$id)+1
n <-  ndata <- length(unique(id))
risk.iid <- apply(Gest$iid,2,sumstrata,x$id,nid)/n 
DGiid <- risk.iid

for (a in seq_along(nlevels)) {
	DGiid[,1] <- c(x$iid  %*% DariskG[[a]])/n
	risk.iid[,a] <- risk.iid[,a]+ c(x$iid  %*% DariskG[[a]])/n
}
vv <- crossprod(risk.iid)

Gout <- estimate(coef=Gest$Gest,vcov=vv,labels=paste("risk",nlevels,sep=""))
ed <-  estimate(coef=Gest$Gest,vcov=vv,f=function(p) p[-1]-p[1])
rd <- estimate(coef=Gest$Gest,vcov=vv,f=function(p) log(p[-1]/p[1]),null=0)
out <- list(risk.iid=risk.iid,risk=Gout,difference=ed,ratio=rd,vcov=vv,model=x$model[1])
class(out) <- "survivalG"
return(out)
} ## }}}

##' @export
logitIPCWATE <- function(formula,data,cause=1,time=NULL,beta=NULL,treat.model=~+1, cens.model=~+1,
	   offset=NULL,weights=NULL,cens.weights=NULL,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
           outcome=c("cif","rmst","rmtl"),model=c("default","logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster


 call.id <- id;
 conid <- construct_id(id,nrow(X),namesX=rownames(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 idclust <- id; nclust <- nid

 ## make all computations for each subject, later sum influence functions and sort after idclust 
 nid <- length(id); id <- 0:(nid-1); 

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights

###  if (is.null(offset)) offset <- rep(0,length(exit)) 
###  if (is.null(weights)) weights <- rep(1,length(exit)) 
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- (status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) stop("No events of type 1 before time \n"); 
  kmt <- kaplan.meier
  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 

  statusC <- (status %in% cens.code) 
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  n <- length(exit)

  cens.strata <- cens.nstrata <- NULL 

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
###      cens.weights <- predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt)$surv
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else formC <- NULL

  if (is.null(beta)) beta <- rep(0,ncol(X))
  p <- ncol(X)

  X <-  as.matrix(X)
  X2  <- .Call("vecMatMat",X,X)$vXZ
  obs <- (exit<=time & status!=cens.code) | (exit>=time)
  weights <- obs*weights/c(cens.weights)
  cens.weights <- c(cens.weights)

  Y <- c((status==cause)*(exit<=time))

  if (!is.null(Ydirect)) Y <-  Ydirect else {
     if (outcome[1]=="cif") Y <- c((status %in% cause)*(exit<=time))
     else { if (!competing) {
	     if (outcome[1]=="rmst") Y <-  c(pmin(exit,time))
	     else Y <-  c((time-pmin(exit,time)))
            } else Y <- c((status %in% cause)*(time-pmin(exit,time)))
     }
  }
 Yout <- Y
 if (is.null(augmentation))  augmentation=rep(0,p)
 nevent <- sum(Y)

 ## default links
if (model[1]=="default") {
	 if (outcome[1]=="cif") model <- "logit"
	 if (outcome[1]=="rmst") model <- "exp"
	 if (outcome[1]=="rmtl") model <- "exp"
 }

obj <- function(pp,all=FALSE)
{ # {{{

lp <- c(X %*% pp+offset)
if (model[1]=="exp") {
    p <- exp(lp) 
    D2logl <- c(weights*p)*X2 
} else if (model[1]=="lin") {
    p <- lp
    D2logl <- c(weights)*X2
} else if (model[1]=="logit") {
   p <- expit(lp)
   D2logl <- c(weights*p/(1+exp(lp)))*X2
} 

ploglik <- sum(weights*(Y-p)^2)
Dlogl <- weights*X*c(Y-p)
D2log <- apply(D2logl,2,sum)
gradient <- apply(Dlogl,2,sum)+augmentation
hessian <- matrix(D2log,length(pp),length(pp))

  if (all) {
      ihess <- pinv(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,
	 iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(-ploglik,gradient=-gradient,hessian=hessian)
}# }}}

  p <- ncol(X)
  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid,weights=weights))
  
# {{{ computation of ate, att, atc and their influence functions
if (length(all.vars(treat.model))==0) 
{ respformula  <-  as.formula( paste(all.vars(formula)[3],"~."))
  treat.model  <- update.formula(treat.model,respformula)
###  cat("Using propensity score model :\n "); print(treat.model)
}

## dropping cluster here 
treat.model <- drop.specials(treat.model,"cluster")
treat <- glm(treat.model,data,family="binomial")
Xtreat <- model.matrix(treat$formula,data)
ytreat <- treat$y

lpa <- treat$linear.predictors 
pal <- expit(lpa)
iidalpha <- iid(treat,id=id)

### treatment is rhs of treat.model 
treat.name <-  all.vars(treat.model)[1]
dat0 <- data
dat0[,treat.name] <- 0 ## treat.contrast[1]
dat1 <- data
dat1[,treat.name] <- 1 ## treat.contrast[2]

formulanc <- drop.specials(formula,"cluster")

X1 <- model.matrix(formulanc[-2],dat1)
X0 <- model.matrix(formulanc[-2],dat0)

p11lp <- X1 %*% val$coef+offset
p10lp <- X0 %*% val$coef+offset
p1lp <-   X %*% val$coef+offset

if (model[1]=="exp") {
   p1 <- exp(p1lp); p10 <- exp(p10lp); p11 <- exp(p11lp);
} else if (model[1]=="lin") {
   p1 <- p1lp; p10 <- p10lp; p11 <- p11lp; 
} else if (model[1]=="logit") {
    p1 <- expit(p1lp); p10 <- expit(p10lp); p11 <- expit(p11lp);
} else stop("link functions must be logit,exp,lin\n") 

Yorig <- Y
Y <- c(Y*obs/cens.weights)
risk1 <- ytreat*(Y-p11)/pal+p11
risk0 <- (1-ytreat)*(Y-p10)/(1-pal)+p10
riskG1 <- p11
riskG0 <- p10

ntreat <- sum(ytreat)
###att <- ytreat*Y-(pal*(1-ytreat)*Y + (ytreat - pal)* p10)/(1-pal)
###atc <- ((1-pal)*ytreat*Y - ((ytreat - pal)* p11)/pal)-(1-ytreat)*Y

if (model[1]=="logit") {
	Dp1 <- X * c(p1/(1+exp(p1lp)))
	Dp11 <- X1 * c(p11/(1+exp(p11lp)))
	Dp10 <- X0 * c(p10/(1+exp(p10lp)))
} else if (model[1]=="lin") {
      Dp1 <- X; Dp11 <- X1; Dp10 <- X0;
} else {
      Dp1 <- c(p1)*X; Dp11 <- c(p11)*X1; Dp10 <- c(p10)*X0;
}

Dpai <-  - Xtreat * exp(-lpa)
D1mpai <-   Xtreat * exp(lpa)

DaPsi1 <-  apply( Dpai * ytreat * c( Y - p1),2,sum)
DaPsi0 <-  apply( D1mpai * (1-ytreat) * c( Y - p1),2,sum)
DePsi1 <-  apply( Dp11 * ( 1- ytreat/pal),2,sum)
DePsi0 <-  apply( Dp10 * ( 1- (1-ytreat)/(1-pal)),2,sum)

###DePsiatt <- - apply( Dp10* (ytreat-pal)/(1-pal),2,sum)
###DaPsiatt <- apply(c((1-ytreat)*(Y-p10))*D1mpai,2,sum)
###
###DePsiatc <- -apply( Dp11* (ytreat-pal)/pal,2,sum)
###DaPsiatc <- apply(c(ytreat*(Y-p11))*Dpai,2,sum)

DriskG1 <- apply(Dp11,2,sum)
DriskG0 <- apply(Dp10,2,sum)
DdifriskG <- DriskG1-DriskG0

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    Xtreat <- Xtreat[ord,,drop=FALSE]
    ytreat <- ytreat[ord]
    pal <-   pal[ord]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    lp <- c(X %*% val$coef+offset)
    if (model[1]=="exp") { p <- exp(lp) } else if (model[1]=="lin") { p <- lp } else if (model[1]=="logit") {p <- expit(lp) } 
    Yglm <- weights*(Yout[ord]-p)
    Y <- Yout[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yglm,2,revcumsumstrata,xx$strata,xx$nstrata)
    h10  <-  apply(cbind((ytreat/pal)*Y,I(ytreat==0)/(1-pal))*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
###  hattc  <-  apply(cbind(ytreat-pal*(1-ytreat)/(1-pal),-(1-ytreat)+(1-pal)*ytreat/pal)*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply((exit<=time)*h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<=time)*h[xx$jumps+1,] /c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    IhdLamh10 <- apply(h10*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
    U10 <- matrix(0,nrow(xx$X),2)
    U10[xx$jumps+1,] <- h10[xx$jumps+1,] /c(resC$S0)
    MGt10 <- (U10[,drop=FALSE]-IhdLamh10)*c(xx$weights)

###    IhdLamhattc <- apply(hattc*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
###    Uattc <- matrix(0,nrow(xx$X),2)
###    Uattc[xx$jumps+1,] <- hattc[xx$jumps+1,]/c(resC$S0)
###    MGtattc <- (Uattc[,drop=FALSE]-IhdLamhattc)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)+1
    MGCiid <- apply(MGt,2,sumstrata,xx$id,mid)
    MGCiid10 <- apply(MGt10,2,sumstrata,xx$id,mid)
###    MGCiidattc <- apply(MGtattc,2,sumstrata,xx$id,mid)
  }  else { MGCiidattc <- MGCiid <- 0; MGCiid10 <- 0 }
## }}}

val$MGciid <- MGCiid
val$MGciid10 <- MGCiid10
val$id <- id
val$call.id <- call.id
val$name.id <- name.id
val$nid <- nid
val$iid.naive <- val$iid 
if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian)
val$naive.var <- val$var
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$cause <- cause
val$cens.code <- cens.code 

### estimates risk, att, atc
val$riskDR <- c(mean(risk1),mean(risk0))
val$riskG<- c(mean(riskG1),mean(riskG0))

###val$att <- sum(att)/ntreat
###val$atc <- sum(atc)/(n-ntreat)
###val$attc <- c(val$att,val$atc)
###names(val$attc) <- c("ATT","ATC")

names(val$riskDR) <- paste("treat",1:0,sep="-")
names(val$riskG) <- paste("treat",1:0,sep="-")

## iid's of marginal risk estimates 
iidbase1 <- c(risk1-val$riskDR[1])
iidcif1 <- c(c(DePsi1) %*% t(val$iid))
iidpal1 <- c(c(DaPsi1) %*% t(iidalpha))
if (se)  {
iidGc1 <- MGCiid10[,1]; iidGc0 <- MGCiid10[,2]
###iidGatt <-  MGCiidattc[,1]; iidGatc <-  MGCiidattc[,2]
}  else { iidGc1 <- iidGatt  <- iidGatc  <- iidGc0  <- 0 } 

iidbase0 <- c(risk0-val$riskDR[2])
iidcif0 <- c(c(DePsi0) %*% t(val$iid))
iidpal0 <- c(c(DaPsi0) %*% t(iidalpha))

iidrisk1 <- iidbase1+iidcif1+iidpal1+iidGc1
iidrisk0 <- iidbase0+iidcif0+iidpal0+iidGc0
difriskiid <- (iidrisk1-iidrisk0)/n

iidrisk <- cbind(iidrisk1,iidrisk0)/n

###iidatt <- att-ytreat*val$att
###iidatc <- atc-(1-ytreat)*val$atc
###iidcifatt <- c(c(DePsiatt) %*% t(val$iid))
###iidpalatt <- c(c(DaPsiatt) %*% t(iidalpha))  
###iidcifatc <- c(c(DePsiatc) %*% t(val$iid))
###iidpalatc <- c(c(DaPsiatc) %*% t(iidalpha)) 
###iidatt <- iidatt+iidcifatt+iidpalatt+iidGatt
###iidatc <- iidatc+iidcifatc+iidpalatc+iidGatc

# }}}


val$call.id <- call.id
val$name.id  <- name.id

# {{{ output variances and se for ate; cluster correction
val$id <- idclust
val$nid  <- nclust

## make iid form before combining into clusters 
riskG.iid <- cbind(c(p11-val$riskG[1]) + c(DriskG1 %*% t(val$iid)),
	          c(p10-val$riskG[2]) + c(DriskG0 %*% t(val$iid)))/n

## outcome model from IPCW logistic regression 
val$iid <- apply(val$iid,2,sumstrata,idclust,nclust)
robvar <- crossprod(val$iid)
val$var <-  val$robvar <- robvar
val$se.robust <- diag(robvar)^.5
val$se.coef <- diag(val$var)^.5
val$MGciid <- NULL
val$iid.naive <- NULL

## DR estimates 
iidrisk <- apply(iidrisk,2,sumstrata,idclust,nclust)
val$var.riskDR <- crossprod(iidrisk); 
val$se.riskDR <- diag(val$var.riskDR)^.5
val$riskDR.iid <- iidrisk

###nlevs <- 2
###pdiff <- function(x) lava::contr(lapply(seq(x-1), function(z) seq(z,x)))
###contrast <- -pdiff(length(nlevs))
###nncont <- c()
###for (k in seq_along(nlevs[-length(nlevs)])) nncont <-c(nncont, paste("treat:",nlevs[-seq(k)],"-",nlevs[k],sep="")) 
###rownames(contrast) <- nncont
contrast <- rbind(c(1,-1))

mm <- estimate(coef=val$riskDR,vcov=val$var.riskDR,f=rbind(c(1,-1)))
val$difriskDR <- mm$coef 
names(val$difriskDR) <-  rownames(contrast) 
val$var.difriskDR <- mm$vcov 
val$se.difriskDR <- diag(val$var.difriskDR)^.5

riskG.iid <- apply(riskG.iid,2,sumstrata,idclust,nclust)
###
val$riskG.iid <- riskG.iid
val$var.riskG <- crossprod(riskG.iid)
val$se.riskG <- diag(val$var.riskG)^.5
###
mm <- estimate(coef=val$riskG,vcov=val$var.riskG,f=contrast)
val$difriskG <- mm$coef 
names(val$difriskG) <-  rownames(contrast) 
val$var.difriskG <- mm$vcov 
val$se.difriskG <- diag(val$var.difriskG)^.5

### DR-estimator ; G -estimator sort after namid; also outcome model
if (!is.null(call.id)) {
    val$riskDR.iid <- nameme(iidrisk,name.id)
    val$riskG.iid  <- nameme(riskG.iid,name.id)
    val$iid <-       nameme(val$iid,name.id)
}
## }}}

  val$model <- model[1]
  val$outcome <- outcome[1]
  val$call.id <- call.id
  class(val) <- c("binreg","ATE")
  return(val)
}# }}}

##' @export
logitATE <- function(formula,data,binreg=TRUE,...)
{# {{{
   ## use IPCW machine in no-censoring case
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster", "offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (inherits(Y,"Event")) {
    if (binreg) out <- binregATE(formula,data,...) else out <- logitIPCWATE(formula,data,...)
    } else {
      response <- all.vars(formula)[1]
      Ydirect <-  as.numeric(data[,response]) 
      data$time <- 2
      data$event <- 1
      time <- 2
      Survform <-  update.formula(formula,Event(time,event)~.)
      n <- nrow(data)
      if (binreg)
      out <- binregATE(Survform,data,se=0,cens.weights=rep(1,n),time=time,Ydirect=Ydirect,...)
      else
      out <- logitIPCWATE(Survform,data,se=0,cens.weights=rep(1,n),time=time,Ydirect=Ydirect,...)
    }

   return(out)
}# }}}

##' @export
normalATE <- function(formula,data,binreg=TRUE,model="lin",...)
{# {{{
   if (binreg) 
   out <- logitATE(formula,data,binreg=binreg,model=model,outcome="rmst",...)
   else 
   out <- logitATE(formula,data,binreg=binreg,...)
   return(out)
}# }}}

##' Restricted IPCW Mean for Censored Survival Data
##'
##' Provides a fast implementation of Inverse Probability of Censoring Weighting (IPCW) 
##' regression for a single time point. It fits the model:
##' \deqn{ E( \min(T, t) | X ) = \exp( X^T \beta) }
##' or, in the case of competing risks data:
##' \deqn{ E( I(\epsilon=1) (t - \min(T, t)) | X ) = \exp( X^T \beta) }
##' which represents the "Years Lost Due to Cause" (RMTL).
##'
##' The method solves the binomial regression IPCW response estimating equation:
##' \deqn{ X \left( \frac{\Delta(\min(T,t)) Y}{G_c(\min(T,t))} - \exp( X^T \beta) \right) = 0 }
##' where \eqn{\Delta(\min(T,t)) = I(\min(T,t) \leq C)} is the indicator of being uncensored 
##' at the time of interest.
##'
##' When the status variable is binary, the outcome is assumed to be \eqn{Y = \min(T,t)} (RMST).
##' If the status has more than two levels (competing risks), the outcome is 
##' \eqn{Y = (t - \min(T,t)) I(\text{status}=\text{cause})} (RMTL for a specific cause).
##'
##' The function supports:
##' \itemize{
##'   \item \strong{IPCW Adjustment}: Weights by the inverse of the censoring survival probability.
##'   \item \strong{Augmentation}: Can include an augmentation term (type="II" or "III") to improve efficiency 
##'     and robustness (Double Robust estimation).
##'   \item \strong{Variance Estimation}: Based on the influence function, including adjustments for 
##'     the estimation of the censoring model.
##' }
##'
##' @param formula Formula with an \code{Event} outcome (e.g., \code{Event(time, cause)}).
##' @param data Data frame containing the variables.
##' @param outcome Outcome type: \code{"rmst"} (Restricted Mean Survival Time) or 
##'   \code{"rmtl"} (Restricted Mean Time Lost).
##' @param ... Additional arguments passed to \code{binreg}, such as \code{time}, \code{cause}, 
##'   \code{cens.model}, \code{model}, \code{type}, etc.
##' @return An object of class \code{"binreg"} containing:
##'   \item{coef}{Coefficient estimates.}
##'   \item{se}{Standard errors.}
##'   \item{var}{Variance-covariance matrix.}
##'   \item{iid}{Influence function decomposition.}
##'   \item{naive.var}{Variance under known censoring model (if applicable).}
##'   \item{time}{Time point used.}
##'   \item{outcome}{Type of outcome analyzed.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.
##' @seealso \code{\link{binreg}}, \code{\link{resmeanATE}}, \code{\link{rmstIPCW}}
##' @examples
##' data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001
##' 
##' # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
##' out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'                 time=50, cens.model=~strata(platelet), model="exp")
##' summary(out)
##' 
##' \donttest{ ## Reduce Ex.Timings
##' ## Weighted GLM version (RMST)
##' out2 <- logitIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'             time=50, cens.model=~strata(platelet), model="exp", outcome="rmst")
##' summary(out2)
##' 
##' ### Time-lost (RMTL)
##' outtl <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
##'                 time=50, cens.model=~strata(platelet), model="exp", outcome="rmtl")
##' summary(outtl)
##' 
##' ### Same as Kaplan-Meier for full censoring model 
##' bmt$int <- with(bmt, strata(tcell, platelet))
##' out <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30,
##'                              cens.model=~strata(platelet, tcell), model="lin")
##' estimate(out)
##' out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet), data=bmt)
##' rm1 <- resmean_phreg(out1, times=30)
##' summary(rm1)
##' 
##' ### Years lost regression
##' outl <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30, outcome="years-lost",
##'                              cens.model=~strata(platelet, tcell), model="lin")
##' estimate(outl)
##' 
##' ## Competing risks years-lost for cause 1  
##' out <- resmeanIPCW(Event(time,cause)~-1+int, bmt, time=30, cause=1,
##'                             cens.model=~strata(platelet, tcell), model="lin")
##' estimate(out)
##' ## Same as integrated cumulative incidence 
##' rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet), data=bmt, times=30)
##' summary(rmc1)
##' }
##' @aliases rmstIPCW 
##' @export
resmeanIPCW  <- function(formula,data,outcome=c("rmst","rmtl"),...)
{# {{{
   out <- binreg(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

##' @export
rmstIPCW <- function(formula,data,outcome=c("rmst","rmtl"),...)
{# {{{
   out <- binreg(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

preprrm <- function(cs,ss,X,times,data,model="exp") 
{# {{{

ttimes <- ss$cumhaz[,1]
ttimes <- ttimes[(ttimes< times)]
ttimes <- c(ttimes,times)
## cens model 
pcs <- predict(cs,data,se=FALSE,times=ttimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
## survival model 
ps <- predict(ss,data,se=FALSE,times=ttimes)
###dLamt <- apply(cbind(0,ps$cumhaz),1,diff)
pst0 <- c(tail(t(ps$surv),n=1))
dtime <- diff(c(0,ttimes))
RRM <-  apply(dtime * t(ps$surv),2,cumsum)
RRMt0 <- c(tail(RRM,n=1))
dF <- -apply(cbind(1,ps$surv),1,diff)
I <- apply(ttimes^2/t(pcs$surv)*dF,2,sum)+times^2*pst0/pcst0 -RRMt0*apply(ttimes/t(pcs$surv)*dF,2,sum)-RRMt0*times*pst0/pcst0 
varY <- apply(ttimes^2*dF,2,sum)+times^2*pst0 - RRMt0^2

ctimes <- cs$cumhaz[,1]
ctimes <- ctimes[(ctimes< times)]
## cens model 
pcs <- predict(cs,data,se=FALSE,times=ctimes)
pcst0 <- c(tail(t(pcs$surv),n=1))
dLamc <- apply(cbind(0,pcs$cumhaz),1,diff)
## survival model 
ps <- predict(ss,data,se=FALSE,times=ctimes)
pst0 <- c(tail(t(ps$surv),n=1))
Fsst0 <- 1-pst0
dtime <- diff(c(0,ctimes))
###dS <- -apply(cbind(1,ps$surv),1,diff)
RRM <-  apply(dtime * t(ps$surv),2,cumsum)
###
RRMt0 <- c(tail(RRM,n=1))
dF <- dRRM <- t(RRMt0-t(RRM)) + ctimes*t(ps$surv)
II <- apply((dF^2/t(ps$surv*pcs$surv))*dLamc,2,sum)-RRMt0*apply((dF/t(pcs$surv))*dLamc,2,sum) 
if (model=="exp") DbetaF <- RRMt0 else DbetaF <- 1
h <- DbetaF/(I-II) 
###varYII <- 2*apply(ctimes*dtime*ps$surv,1,sum)-RRMt0^2

cmtimes <- matrix(ctimes,nrow(dLamc),ncol(dLamc))
tttimes <- matrix(pmin(data[,"time"],times),nrow(dLamc),ncol(dLamc),byrow=TRUE)
Augmentf <- (dF/t(ps$surv*pcs$surv))
AugmentC <- apply(Augmentf*dLamc*(cmtimes<=tttimes),2,sum)
###
n <- nrow(data)
AdN <- rep(0,n)
jc <- (1:nrow(data))[cs$ord][cs$jumps]
jc <-jc[1:length(ctimes)] 
AdN[jc] <- mdi(Augmentf,1:length(ctimes),jc)
Mc <- AdN- AugmentC

ph <- 1
if (model=="exp") ph <- RRMt0
Xaugment <- apply(X*Mc*ph,2,sum)
augment <- apply(X*Mc*h,2,sum)
hh <- (DbetaF/varY)
Faugment <- apply(X*hh*Mc,2,sum)
return(list(Mc=Mc,Xaugment=Xaugment,Faugment=Faugment,hXaugment=augment,h=h,hh=hh,varY=varY,RRMt0=RRMt0))
}# }}}


##' Average Treatment Effect for Restricted Mean Time
##'
##' Estimates the Average Treatment Effect (ATE) for Restricted Mean Survival Time (RMST) 
##' or Restricted Mean Time Lost (RMTL) in censored competing risks data using IPCW.
##' 
##' Under standard causal assumptions (Consistency, Ignorability, Positivity), the ATE is 
##' estimated as \eqn{E(Y(1) - Y(0))}, where \eqn{Y(a)} is the potential outcome under treatment \eqn{a}.
##' The method uses double robust estimating equations that are IPCW-adjusted for censoring.
##'
##' The first covariate in the formula must be the treatment effect (a factor). If the factor 
##' has more than two levels, multinomial logistic regression (mlogit) is used for propensity 
##' score modeling.
##'
##' @param formula Formula with an \code{Event} outcome. The first covariate must be the treatment factor.
##' @param data Data frame.
##' @param model Link function: \code{"exp"} (exponential) or \code{"lin"} (identity).
##' @param outcome Outcome type: \code{"rmst"} or \code{"rmtl"}.
##' @param ... Additional arguments passed to \code{binregATE}, such as \code{time}, \code{treat.model}, 
##'   \code{augmentR0}, \code{augmentC}, etc.
##' @return An object of class \code{"binregATE"} containing:
##'   \item{riskG}{Simple IPCW estimator results.}
##'   \item{riskDR}{Double Robust estimator results.}
##'   \item{riskG.iid, riskDR.iid}{Influence functions.}
##'   \item{coef}{Treatment effect estimates.}
##'   \item{se}{Standard errors.}
##' @author Thomas Scheike
##' @references 
##' Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.
##' 
##' Scheike, T. and Tanaka, S. (2025). Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause. WIP.
##' @seealso \code{\link{binregATE}}, \code{\link{ratioATE}}
##' @examples
##' data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
##' 
##' out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, 
##'                   treat.model=tcell~platelet, outcome="rmtl")
##' summary(out)
##' 
##' out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40,
##'                    treat.model=tcell~platelet, outcome="rmtl")
##' summary(out1)
##' 
##' @aliases rmstATE 
##' @export
resmeanATE <- function(formula,data,model="exp",outcome=c("rmst","rmtl"),...)
{# {{{
out <- 	binregATE(formula,data,outcome=outcome[1],model=model,...) 
return(out)
}# }}}

##' @export
rmstATE <- function(formula,data,model="exp",outcome=c("rmst","rmtl"),...)
{# {{{
out <- 	binregATE(formula,data,outcome=outcome[1],model=model,...) 
return(out)
}# }}}

##' Ratio of Average Treatment Effects
##'
##' Computes the ratio of two Average Treatment Effects (ATEs), typically comparing the 
##' ATE for a specific cause (e.g., RMTL due to cause 1) to the ATE for the total RMTL.
##' 
##' The function transforms the estimates (e.g., using log) to compute the ratio and 
##' its standard error using the delta method and the joint influence functions.
##'
##' @param rmtl Object containing the ATE for the total RMTL (from \code{resmeanATE}).
##' @param rmtl1 Object containing the ATE for the specific cause RMTL (from \code{resmeanATE}).
##' @param h Transformation function (e.g., \code{log}) applied to the ratio. Default is identity.
##' @param null Value under the null hypothesis for the ratio (default 1).
##' @return A list containing:
##'   \item{ratioG}{Ratio based on the simple IPCW estimator.}
##'   \item{ratioDR}{Ratio based on the double robust estimator.}
##' @author Thomas Scheike
##' @seealso \code{\link{resmeanATE}}
##' @examples
##' data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
##' out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, outcome="rmtl")
##' out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40, outcome="rmtl")
##' ratioATE(out, out1, h=log)
##' @export
ratioATE <- function(rmtl, rmtl1, h=NULL, null=1)
##' @export
ratioATE <- function(rmtl,rmtl1,h=NULL,null=1) { ## {{{

if (is.null(h)) h <- function(x) x

coefG <- c(rmtl$riskG,rmtl1$riskG)
iidG <- cbind( rmtl$riskG.iid, rmtl1$riskG.iid)
covG <- crossprod(iidG)
###
coefDR <- c(rmtl$riskDR,rmtl1$riskDR)
iidDR <- cbind(rmtl$riskDR.iid, rmtl1$riskDR.iid)
covDR <- crossprod(iidDR)

ratioG <- estimate(coef=coefG,vcov=covG,f=function(p) h(c(p[3:4]/p[1:2],p[1]*p[4]/(p[2]*p[3]),p[2]*p[3]/(p[1]*p[4]))),null=null)
ratioDR <- estimate(coef=coefDR,vcov=covDR,f=function(p) h(c(p[3:4]/p[1:2],p[1]*p[4]/(p[2]*p[3]),p[2]*p[3]/(p[1]*p[4]))),null=null)
out <- list(ratioG=ratioG,ratioDR=ratioDR)
return(out)

} ## }}}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets)

data(ttpd) 
dtable(ttpd,~entry+time2)
out <- interval.logitsurv.discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
summary(out)

dfactor(ttpd) <- entry.f~entry
out <- cumoddsreg(entry.f~X1+X2+X3+X4,ttpd)
summary(out)

## -----------------------------------------------------------------------------
set.seed(1000) # to control output in simulatins for p-values below.
n <- 200
Z <- matrix(rbinom(n*4,1,0.5),n,4)
outsim <- simlogitSurvd(out$coef,Z)
outsim <- transform(outsim,left=time,right=time+1)
outsim <- dtransform(outsim,right=Inf,status==0)
outss <- interval.logitsurv.discrete(Interval(left,right)~+X1+X2+X3+X4,outsim)
summary(outss)

pred <- predictlogitSurvd(out,se=TRUE)
plotSurvd(pred,se=TRUE)

## -----------------------------------------------------------------------------
test <- 0 
if (test==1) {

require(icenReg)
data(IR_diabetes)
IRdia <- IR_diabetes
## removing fully observed data in continuous version, here making it a discrete observation 
IRdia <- dtransform(IRdia,left=left-1,left==right)
dtable(IRdia,~left+right,level=1)

ints <- with(IRdia,dInterval(left,right,cuts=c(0,5,10,20,30,40,Inf),show=TRUE) )
}

## -----------------------------------------------------------------------------
if (test==1) {
ints$Ileft <- ints$left
ints$Iright <- ints$right
IRdia <- cbind(IRdia,data.frame(Ileft=ints$Ileft,Iright=ints$Iright))
dtable(IRdia,~Ileft+Iright)
# 
#       Iright   1   2   3   4   5 Inf
# Ileft                               
# 0             10   1  34  25   4   0
# 1              0  55  19  17   1   1
# 2              0   0 393  16   4   0
# 3              0   0   0 127   1   0
# 4              0   0   0   0  21   0
# 5              0   0   0   0   0   2

outss <- interval.logitsurv.discrete(Interval(Ileft,Iright)~+gender,IRdia)
#            Estimate Std.Err    2.5%    97.5%   P-value
# time1        -3.934  0.3316 -4.5842 -3.28418 1.846e-32
# time2        -2.042  0.1693 -2.3742 -1.71038 1.710e-33
# time3         1.443  0.1481  1.1530  1.73340 1.911e-22
# time4         3.545  0.2629  3.0295  4.06008 1.976e-41
# time5         6.067  0.7757  4.5470  7.58784 5.217e-15
# gendermale   -0.385  0.1691 -0.7165 -0.05351 2.283e-02
summary(outss)
outss$ploglik
# [1] -646.1946

fit <- ic_sp(cbind(Ileft, Iright) ~ gender, data = IRdia, model = "po")
# 
# Model:  Proportional Odds
# Dependency structure assumed: Independence
# Baseline:  semi-parametric 
# Call: ic_sp(formula = cbind(Ileft, Iright) ~ gender, data = IRdia, 
#     model = "po")
# 
#            Estimate Exp(Est)
# gendermale    0.385     1.47
# 
# final llk =  -646.1946 
# Iterations =  6 
# Bootstrap Samples =  0 
# WARNING: only  0  bootstrap samples used for standard errors. 
# Suggest using more bootstrap samples for inference
summary(fit)

## sometimes NR-algorithm needs modifications of stepsize to run 
## outss <- interval.logitsurv.discrete(Interval(Ileft,Iright)~+gender,IRdia,control=list(trace=TRUE,stepsize=1.0))
}


## -----------------------------------------------------------------------------

data(ttpd) 
dtable(ttpd,~entry+time2)
ttpd <- dfactor(ttpd,fentry~entry)
out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
summary(out)

out$ploglik

if (test==1) {
### library(ordinal)
### out1 <- clm(fentry~X1+X2+X3+X4,data=ttpd)
### summary(out1)

# formula: fentry ~ X1 + X2 + X3 + X4
# data:    ttpd
# 
#  link  threshold nobs logLik   AIC     niter max.grad cond.H 
#  logit flexible  1000 -1676.46 3372.91 6(2)  1.17e-12 5.3e+02
# 
# Coefficients:
#    Estimate Std. Error z value Pr(>|z|)    
# X1  -0.9913     0.1171  -8.465  < 2e-16 ***
# X2  -0.6962     0.1156  -6.021 1.74e-09 ***
# X3  -0.3466     0.1150  -3.013  0.00259 ** 
# X4  -0.3223     0.1147  -2.810  0.00495 ** 
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Threshold coefficients:
#     Estimate Std. Error z value
# 0|1  -2.0064     0.1461 -13.733
# 1|2  -1.3940     0.1396  -9.984
# 2|3  -0.7324     0.1347  -5.435
# 3|4  -0.6266     0.1343  -4.667
# 4|5  -0.1814     0.1333  -1.361
# 5|6   0.2123     0.1342   1.582
}


## -----------------------------------------------------------------------------
sessionInfo()

##' Internal function.
##' Calculates Inverse Probability of Censoring
##' Weights (IPCW) and adds them to a data.frame
##'
##' @title Inverse Probability of Censoring Weights
##' @param formula Formula specifying the censoring model 
##' @param data data frame
##' @param cluster clustering variable
##' @param same.cens For clustered data, should same censoring be assumed (bivariate probability calculated as mininum of the marginal probabilities)
##' @param obs.only Return data with uncensored observations only
##' @param weight.name Name of weight variable in the new data.frame
##' @param indi.weight Name of individual censoring weight  in the new data.frame
##' @param trunc.prob If TRUE truncation probabilities are also calculated and stored in 'weight.name2' (based on Clayton-Oakes gamma frailty model)
##' @param weight.name2 Name of truncation probabilities
##' @param cens.model Censoring model (default Aalens additive model)
##' @param pairs For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)
##' @param theta.formula Model for the dependence parameter in the Clayton-Oakes model (truncation only)
##' @param ... Additional arguments to censoring model 
##' @author Klaus K. Holst
##' @examples
##' \dontrun{
##' data("prt",package="mets")
##' prtw <- ipw(Surv(time,status==0)~country, data=prt[sample(nrow(prt),5000),],
##'             cluster="id",weight.name="w")
##' plot(0,type="n",xlim=range(prtw$time),ylim=c(0,1),xlab="Age",ylab="Probability")
##' count <- 0
##' for (l in unique(prtw$country)) {
##'     count <- count+1
##'     prtw <- prtw[order(prtw$time),]
##'     with(subset(prtw,country==l),
##'          lines(time,w,col=count,lwd=2))
##' }
##' legend("topright",legend=unique(prtw$country),col=1:4,pch=-1,lty=1)
##' }
##' @export
ipw <- function(formula,data,cluster,
                same.cens=FALSE,obs.only=FALSE,
                weight.name="w",
                trunc.prob=FALSE,weight.name2="wt",indi.weight="pr",
                cens.model="aalen", pairs=FALSE,
                theta.formula=~1, ...) { 
## {{{
    ##iid=TRUE,

    ##cens.args <- c(list(formula,n.sim=0,robust=0,data=eval(data)),list(...))
    if (tolower(cens.model)%in%c("weibull","phreg.par","phreg.weibull")) {
	ud.cens <- phreg.par(formula,data,...)
        pr <- predict(ud.cens)
        noncens <- which(!ud.cens$status)        
    } else { ## {{{ 
        m <- match.call(expand.dots = FALSE)
        m <- m[match(c("", "formula", "data", "subset", "na.action"), 
                     names(m), nomatch = 0)]
        special <- c("strata", "factor", "NN", "cluster", "dummy")
        if (missing(data)) 
            Terms <- terms(formula)
        else Terms <- terms(formula, data = data)
        m$formula <- Terms
        m[[1]] <- as.name("model.frame")
        M <- eval(m, parent.frame())
        censtime <- model.extract(M, "response")
        if (ncol(censtime)==3) {
            status <- censtime[,3]
            otimes <- censtime[,2]
            ltimes <- censtime[,1]
        } else {
            status <- censtime[,2]
            otimes <- censtime[,1]
        }
        noncens <- !status
        if (is.null(attr(terms(formula,"prop"),"specials")$prop)) {
            ud.cens <- timereg::aalen(formula,n.sim=0,robust=0,data=data,...)
            XZ <- model.matrix(formula,data)
###         Gcx <- ud.cens$cum[prodlim::sindex(ud.cens$cum[,1],otimes),-1]
            Gcx<-cpred(ud.cens$cum,otimes)[,-1];            
            Gcx<-exp(-apply(Gcx*XZ,1,sum))            
        } else {
            ud.cens <- timereg::cox.aalen(formula,n.sim=0,robust=0,data=data,...)
            XZ <- model.matrix(formula,data)
            Gcx<-cpred(ud.cens$cum,otimes)[,-1];
            Gcx<-exp(-apply(Gcx*XZ,1,sum))            
        }
        ##ud.cens <- do.call(cens.model,cens.args)
        Gcx[Gcx>1]<-1; Gcx[Gcx<0]<-0
        pr <- Gcx
	data[,indi.weight] <- Gcx
    } ## }}} 

     if (trunc.prob) stop("Under development\n"); 
    if (trunc.prob & ncol(censtime)==3) { ## truncation ## {{{ 
        data$truncsurv <- Surv(ltimes,otimes,noncens)
        trunc.formula <- update(formula,truncsurv~.)        
        ud.trunc <- timereg::aalen(trunc.formula,data=data,robust=0,n.sim=0,residuals=0,silent=1)
        dependX0 <- model.matrix(theta.formula,data)        
        twostage.fit <- timereg::two.stage(ud.trunc,data=data,robust=0,detail=0,
			         clusters=data[,cluster],
				 theta.des=dependX0)#,Nit=20,step=1.0,notaylor=1)
	pre.theta <- dependX0 %*% twostage.fit$theta
        X <- model.matrix(formula,data)
        Xnam <- colnames(X)
	X <- cbind(X,pre.theta)
	colnames(X)[ncol(X)] <- "pre.theta"
        ww <- fast.reshape(cbind(X,".num"=seq(nrow(X)),".lefttime"=ltimes),varying=c(".num",".lefttime"),id=data[,cluster])
	if (anyNA(ww)) stop("not balanced for predict two stage\n");  
        Prob <- timereg::predict.two.stage(twostage.fit,X=ww[,Xnam],
                                  times=ww[,".lefttime1"],times2=ww[,".lefttime2"],
                                  theta=ww$pre.theta)
        P0 <- numeric(nrow(X))
        P0[ww[,".num1"]] <- Prob$St1t2
        P0[ww[,".num2"]] <- Prob$St1t2
        data[,weight.name2] <- P0
	print(summary(P0))
    }
    data[,weight.name] <- pr
    ## }}} 
        
    if (same.cens & missing(cluster)) message("no cluster for same-cens given \n"); 

    if (same.cens & !missing(cluster)) { ## {{{         
###        message("Minimum weights...")
        myord <- order(data[,cluster])
        data <- data[myord,,drop=FALSE]
        id <-  table(data[,cluster])
        if (pairs) {
            gem <- data[,cluster]%in%(names(id)[id==2])
            id <- id[id==2]
            data <- data[gem,]
        }
        d0 <- subset(data,select=c(cluster,weight.name))
        noncens <- with(data,!eval(terms(formula)[[2]][[3]]))
        d0[,"observed."] <- noncens
        timevar <- paste("_",cluster,weight.name,sep="")
        d0[,timevar] <- unlist(lapply(id,seq))
        Wide <- reshape(d0,direction="wide",timevar=timevar,idvar=cluster)
        W <- apply(Wide[,paste(weight.name,1:2,sep=".")],1,
                   function(x) min(x,na.rm=TRUE))
        Wmarg <- d0[,weight.name]
        data[,weight.name] <- 1/Wmarg
        Wmin <- rep(W,id)
        
        obs1only <- rep(with(Wide, observed..1 & (is.na(observed..2) | !observed..2)),id)
        obs2only <- rep(with(Wide, observed..2 & (is.na(observed..1) | !observed..1)),id)
        obsOne <- which(na.omit(obs1only|obs2only))
        obsBoth <- rep(with(Wide, !is.na(observed..1) & !is.na(observed..2) & observed..2 & observed..1),id)

        data[obsBoth,weight.name] <-
            ifelse(noncens[obsBoth],1/Wmin[obsBoth],0)    
        data[obsOne,weight.name] <-
            ifelse(noncens[obsOne],1/Wmarg[obsOne],0)
    } ## }}} 

    if (obs.only)
        data <- data[noncens,,drop=FALSE]
    
    return(data)    
} ## }}} 

##' Internal function.
##' Calculates Inverse Probability of Censoring and Truncation 
##' Weights and adds them to a data.frame
##'
##' @title Inverse Probability of Censoring Weights
##' @param data data frame
##' @param times possible time argument for speciying a maximum value of time tau=max(times), to specify when things are considered censored or not.
##' @param entrytime nam of entry-time for truncation. 
##' @param time name of time variable on data frame.  
##' @param cause name of cause indicator on data frame.
##' @param same.cens For clustered data, should same censoring be assumed and same truncation (bivariate probability calculated as mininum of the marginal probabilities)
##' @param cluster name of clustering variable
##' @param pairs For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)
##' @param strata  name of strata variable to get weights stratified.
##' @param obs.only Return data with uncensored observations only
##' @param cens.formula model for Cox models for truncation and right censoring times.
##' @param cens.code censoring.code 
##' @param pair.cweight Name of weight variable in the new data.frame for right censorig of pairs
##' @param pair.tweight Name of weight variable in the new data.frame for left truncation of pairs
##' @param pair.weight Name of weight variable in the new data.frame for right censoring and left truncation of pairs
##' @param cname Name of weight variable in the new data.frame for right censoring of individuals
##' @param tname Name of weight variable in the new data.frame for left truncation of individuals
##' @param weight.name Name of weight variable in the new data.frame for right censoring and left truncation of individuals
##' @param prec.factor To let tied censoring and truncation times come after the death times. 
##' @param ... Additional arguments to censoring model 
##' @author Thomas Scheike 
##' @examples
##' library("timereg")
##' set.seed(1)
##' d <- simnordic.random(5000,delayed=TRUE,ptrunc=0.7,
##'       cordz=0.5,cormz=2,lam0=0.3,country=FALSE)
##' d$strata <- as.numeric(d$country)+(d$zyg=="MZ")*4
##' times <- seq(60,100,by=10)
##' c1 <- timereg::comp.risk(Event(time,cause)~1+cluster(id),data=d,cause=1,
##' 	model="fg",times=times,max.clust=NULL,n.sim=0)
##' mm=model.matrix(~-1+zyg,data=d)
##' out1<-random.cif(c1,data=d,cause1=1,cause2=1,same.cens=TRUE,theta.des=mm)
##' summary(out1)
##' pc1 <- predict(c1,X=1,se=0)
##' plot(pc1)
##' 
##' dl <- d[!d$truncated,]
##' dl <- ipw2(dl,cluster="id",same.cens=TRUE,time="time",entrytime="entry",cause="cause",
##'            strata="strata",prec.factor=100)
##' cl <- timereg::comp.risk(Event(time,cause)~+1+
##' 		cluster(id),
##' 		data=dl,cause=1,model="fg",
##' 		weights=dl$indi.weights,cens.weights=rep(1,nrow(dl)),
##'             times=times,max.clust=NULL,n.sim=0)
##' pcl <- predict(cl,X=1,se=0)
##' lines(pcl$time,pcl$P1,col=2)
##' mm=model.matrix(~-1+factor(zyg),data=dl)
##' out2<-random.cif(cl,data=dl,cause1=1,cause2=1,theta.des=mm,
##'                  weights=dl$weights,censoring.weights=rep(1,nrow(dl)))
##' summary(out2)
##' @export
ipw2 <- function(data,times=NULL,entrytime=NULL,time="time",cause="cause",
     same.cens=FALSE,cluster=NULL,pairs=FALSE,
     strata=NULL,obs.only=TRUE,cens.formula=NULL,cens.code=0,
     pair.cweight="pcw",pair.tweight="ptw",pair.weight="weights",
     cname="cweights",tname="tweights",weight.name="indi.weights",
     prec.factor=100,...)
{ ## {{{ 
   ### first calculates weights based on marginal
   ### estimators  of censoring and truncation

## {{{  weights, up at T_i or min(T_i,max(times))
   if (is.null(times)) times <- max(data[,time])
   if (is.null(entrytime)) entry <- rep(0,nrow(data)) else entry <- data[,entrytime]
   mtt <- max(times)
   prec <- .Machine$double.eps * prec.factor
   trunc.model <- cens.model <- NULL ## output of Cox models for entry cens

   if (is.null(cens.formula)) { 
   if (is.null(strata)) { ## {{{ 
	   if (!is.null(entrytime)) {
		   surv.trunc <- survival::survfit(Surv(-data[,time],-entry+prec,rep(1,nrow(data))) ~ 1) 
		   trunc.dist <- summary(surv.trunc)
		   trunc.dist$time <- rev(-trunc.dist$time)
		   trunc.dist$surv <- c(rev(trunc.dist$surv)[-1], 1)
		   Lfit <-cpred(cbind(trunc.dist$time,trunc.dist$surv),data[,time],strict=TRUE)
		   Lw <- Lfit[,2]
	   } else { Lw <- 1; }
	   if (!is.null(entrytime)) 
	   ud.cens<- survival::survfit(Surv(entry,data[,time],data[,cause]==0)~+1) 
           else ud.cens<- survival::survfit(Surv(data[,time],data[,cause]==0)~+1) 
	   Gfit<-cbind(ud.cens$time,ud.cens$surv)
	   Gfit<-rbind(c(0,1),Gfit); 
	   Gcx<-cpred(Gfit,pmin(mtt,data[,time]),strict=TRUE)[,2];
           weights <- 1/(Lw*Gcx); 
	   cweights <-  Gcx; 
	   tweights <-  Lw; 
   ### ## }}} 
   } else { ## {{{ 
	   ### compute for each strata and combine 
	  vstrata <- as.numeric(factor(data[,strata]))
          weights <- rep(1,nrow(data))
          cweights <- rep(1,nrow(data))
          tweights <- rep(1,nrow(data))
	  for (i in unique(vstrata)) { ## {{{ for each strata
	       who <- (vstrata == i)
	       if (sum(who) <= 1) stop(paste("strata",i,"less than 1 observation\n")); 
	   datas <- subset(data,who)
	   entrytimes <- entry[who]
	   if (!is.null(entrytime)) {
		   surv.trunc <- survival::survfit(Surv(-datas[,time],-entrytimes+prec,rep(1,nrow(datas))) ~ +1) 
		   trunc.dist <- summary(surv.trunc)
		   trunc.dist$time <- rev(-trunc.dist$time)
		   trunc.dist$surv <- c(rev(trunc.dist$surv)[-1], 1)
		   Lfit <-cpred(cbind(trunc.dist$time,trunc.dist$surv),datas[,time],strict=TRUE)
		   Lw <- Lfit[,2]
	   } else {Lw <- 1; }
	   if (!is.null(entrytime)) 
	   ud.cens<- survival::survfit(Surv(entrytimes,datas[,time],datas[,cause]==0)~+1) 
           else ud.cens<- survival::survfit(Surv(entrytimes,datas[,time],datas[,cause]==0)~+1) 
	   Gfit<-cbind(ud.cens$time,ud.cens$surv)
	   Gfit<-rbind(c(0,1),Gfit); 
	   Gcx<-cpred(Gfit,pmin(mtt,datas[,time]),strict=TRUE)[,2];
	   weights[who]<-  1/(Lw*Gcx); 
	   cweights[who] <-  Gcx; 
	   tweights[who] <-  Lw; 
          } ## }}} 
   } ## }}} 
   } else { ### cens.formula Cox models  ## {{{
        X <- model.matrix(cens.formula,data=data)[,-1,drop=FALSE]; 

	if (!is.null(entrytime)) {
###		trunc.model <- timereg::cox.aalen(Surv(-data[,time],-entrytime+prec,rep(1,nrow(data))) ~ prop(X))
		trunc.model <- survival::coxph(Surv(-data[,time],-entrytime+prec,rep(1,nrow(data))) ~ X) 
###                baseout <- cens.model$cum 
		baseout <- survival::basehaz(trunc.model,centered=FALSE); 
		baseout <- cbind(rev(-baseout$time),rev(baseout$hazard))
	###
		Lfit <-cpred(baseout,data[,time],strict=TRUE)[,-1]
		RR<-exp(as.matrix(X) %*% coef(trunc.model))
		Lfit<-exp(-Lfit*RR)
		Lw <- Lfit
        } else {Lw <- 1; }
###
	if (!is.null(entrytime)) 
	cens.model <- survival::coxph(Surv(entrytime,data[,time],data[,cause]==0)~+X) 
        else cens.model <- survival::coxph(Surv(data[,time],data[,cause]==0)~+X) 
        baseout <- survival::basehaz(cens.model,centered=FALSE); 
###        baseout <- cens.model$cum 
	baseout <- cbind(baseout$time,baseout$hazard)
	Gfit<-cpred(baseout,pmin(mtt,data[,time]),strict=TRUE)[,2];
	RR<-exp(as.matrix(X) %*% coef(cens.model))
	Gfit<-exp(-Gfit*RR)
        weights <- 1/(Lw*Gfit); 
        cweights <- Gfit
        tweights <- Lw
   } ## }}} 

   data[,weight.name] <- weights
   data[,"cw"] <- 1

   if (!is.null(entrytime)) {
   mint <- min(tweights); maxt <- min(tweights) 
   if (mint<0 | mint>1) warning("min(truncation weights) strange, maybe prec.factor should be different\n")
   if (maxt<0 | maxt>1) warning("max(truncation weights) strange, maybe prec.factor should be different\n")
   }

   if (!is.null(entrytime)) {
   attr(data,"trunc.model") <- trunc.model
   attr(data,"cens.model") <- cens.model 
   }
   data[,cname] <- cweights
   data[,tname] <- tweights
## }}} 

  observed <- ((data[,time]>mtt & data[,cause]==cens.code)) | (data[,cause]!=cens.code)
  data[,"observed."] <- observed

  if (same.cens & is.null(cluster)) message("no cluster for same-cens given \n"); 
    if (same.cens & !is.null(cluster)) { ## {{{         
###        message("Minimum weights.cens..")
###        message("Maximum weights.trunc..")
       myord <- order(data[,cluster])
       data <- data[myord,,drop=FALSE]
       id <-  table(data[,cluster])
       observed <- ((data[,time]>mtt & data[,cause]==cens.code)) | (data[,cause]!=cens.code)
       d0 <- data[,c(cluster,cname,tname,time,cause,strata)]
       noncens <- observed
       d0[,"observed."] <-observed
       timevar <- paste("_",cluster,cname,sep="")
       d0[,timevar] <- unlist(lapply(id,seq))
###    print(head(d0))
###    Wide <- reshape(d0,direction="wide",timevar=timevar,idvar=cluster)
       Wide <- fast.reshape(d0,id=cluster)
       ### censoring same cens

       cause1 <- paste(cause,"1",sep=""); cause2 <- paste(cause,"2",sep="")
       obs1 <- "observed.1"; obs2 <- "observed.2"
       time1 <- paste(time,"1",sep=""); time2 <- paste(time,"1",sep="")
###       print(c(time1,time2,cause1,cause2))
###       print(head(Wide))

###       tmax <- apply(Wide[,paste(time,1:2,sep="")],1,
###                   function(x) max(x)) ## NA when there is just one
###       maxstat <- ifelse(Wide[,time1]> Wide[,time2],Wide[,cause1],Wide[,cause2])
###       obsstat <- ifelse(Wide[,time1]> Wide[,time2],Wide[,obs1],Wide[,obs2])*1
###       print(head(tmax,10)); print(head(maxstat,10))
###       print(names(Wide))
###       datp <- data.frame(time=tmax,cause=obsstat,strata=Wide[,paste(strata,"1",sep="")])
###       print(head(datp))
###       datp <- pred.stratKM(datp)
###       print(head(tmax,datp))
###
###       ud.cens<- survival::survfit(Surv(tmax,obsstat==0)~+1) 
###       Gfit<-cbind(ud.cens$time,ud.cens$surv)
###       Gfit<-rbind(c(0,1),Gfit); 
###       tmaxna <- tmax; tmaxna[is.na(tmax)] <- 0
###       Gcx<-cpred(Gfit,pmin(mtt,tmaxna),strict=TRUE)[,2];
###       Wd <- Gcx
###
###      data[,"tmax"] <- rep(tmax,id)
###	data[,"stattmax"] <- rep(maxstat,id)

       W <- apply(Wide[,paste(cname,1:2,sep="")],1,
                   function(x) min(x)) ## NA when there is just one
###                   function(x) min(x,na.rm=TRUE))
        Wmin <- rep(W,id)
	data[,pair.cweight] <- 1/Wmin
###	data[,paste(pair.cweight,"max",sep="")] <- 1/rep(Wd,id)

	### when pair-weight is NA takes individual weight
	naW <- is.na(Wmin)
	data[naW,pair.cweight] <- 1/data[naW,cname]
	data[naW,paste(pair.cweight,"max",sep="")] <- 1/data[naW,cname] 

   ### truncation same truncation  
   if (!is.null(entrytime)) { 
        Wt <- apply(Wide[,paste(tname,1:2,sep="")],1,
                   function(x) min(x)) ## NA when there is just one
###        Wtmarg <- d0[,tname]
###        data[,pair.tweight] <- 0; ##1/Wtmarg
        Wtmax <- rep(Wt,id)
	data[,pair.tweight] <- 1/Wtmax

	### when pair-weight is NA takes individual weight
	naW <- is.na(Wtmax)
	data[naW,pair.tweight] <- 1/data[naW,tname]
   }

    if (!is.null(entrytime)) { 
    data[,pair.weight] <- data[,pair.cweight]*data[,pair.tweight]
    } else data[,pair.weight] <- data[,pair.cweight]
    } ## }}} 

    if (obs.only) { data <- data[observed,] } 

    id <-  table(data[,cluster])
    if (pairs) {
          gem <- data[,cluster]%in%(names(id)[id==2])
          id <- id[id==2]
          data <- data[gem,]
    } else data[,"pairs."] <- data[,cluster]%in%(names(id)[id==2])

   return(data)
} ## }}} 

##' Simulate observations from the Clayton-Oakes copula model with
##' piecewise constant marginals.
##'
##' @title Simulate from the Clayton-Oakes frailty model
##' @param K Number of clusters
##' @param n Number of observations in each cluster
##' @param eta variance
##' @param beta Effect (log hazard ratio) of covariate
##' @param stoptime Stopping time
##' @param lam constant hazard 
##' @param left Left truncation
##' @param pairleft pairwise (1) left truncation or individual (0)
##' @param trunc.prob Truncation probability 
##' @param same if 1 then left-truncation is same also for univariate truncation
##' @author Thomas Scheike and Klaus K. Holst
##' @aliases simClaytonOakes simClaytonOakesLam
##' @export
simClaytonOakes <- function(K,n,eta,beta,stoptime,lam=1,left=0,pairleft=0,trunc.prob=0.5,same=0)  ## {{{ 
{
  ## K antal clustre, n=antal i clustre
  ###	K <- 100; n=2; stoptime=2; eta=1/2; beta=0; lam=0.5;left=0.5; trunc.prob=0.5; pairleft=0; same=0
  ### change such that eta is variance 
  x<-array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  C<-matrix(stoptime,K,n);
  Gam1 <-matrix(rgamma(K,eta)/eta,K,n)
  temp<-eta*log(-log(1-x[,,1])/(eta*Gam1*lam)+1)*exp(-beta*x[,,3])
  x[,,2]<-ifelse(temp<=C,1,0);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  

if (left>0) {
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       if (same==0) lefttime <- rexp(n*K)*left
       if (same==1) lefttime <- rep(rexp(K)*left,each=n)
       if (same==0) left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       if (same==1) left <- rep(rbinom(K,1,trunc.prob),each=n)
       lefttime <- lefttime*left
       trunk <- ud[,1] > lefttime
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);}
  if (pairleft==1) ud <- cbind(ud,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,rep(minstime,each=n),lefttime,trunk)

  names(ud)<-c("time","status","x","cluster","mintime","lefttime","truncated")
  return(ud)
} ## }}} 

##' @export
simClaytonOakesLam <- function(n,k,cumhaz,vargam,entry=NULL)  
{ ## {{{ 
base1 <- cumhaz
dtt <- diff(c(0,base1[,1]))
lams <- (diff(c(0,base1[,2]))/dtt)*exp(vargam*base1[,2])
Lams <- cbind(base1[,1],cumsum(dtt*lams))

if (is.null(entry)) entry <- rep(0,n*k)
gamma <- rep(rgamma(n,1/vargam)*vargam,each=k)
ddd <- rchaz(rbind(c(0,0),Lams),rr=gamma,entry=entry)
ddd$cluster <- rep(1:n,each=k)
ddd$gamma <- gamma

attr(ddd,"cumhaz") <- Lams
return(ddd)
} ## }}} 

##' Simulate observations from the Clayton-Oakes copula model with
##' Weibull type baseline and Cox marginals.
##'
##' @title Simulate from the Clayton-Oakes frailty model
##' @param K Number of clusters
##' @param n Number of observations in each cluster
##' @param eta 1/variance
##' @param beta Effect (log hazard ratio) of covariate
##' @param stoptime Stopping time
##' @param weiscale weibull scale parameter 
##' @param weishape weibull shape parameter 
##' @param left Left truncation
##' @param pairleft pairwise (1) left truncation or individual (0)
##' @author Klaus K. Holst 
##' @export
simClaytonOakesWei <- function(K,n,eta,beta,stoptime,weiscale=1,weishape=2,left=0,pairleft=0)
{ ## {{{ 
###cat(" not quite \n"); 
## K antal clustre, n=antal i clustre
### K=10; n=2; eta=1; beta=0.3; stoptime=3; lam=0.5; 
### weigamma=2; left=0; pairleft=0
 X <- rbinom(n*K,1,0.5)
 C<-rep(stoptime,n*K);
 Gam1 <-rep(rgamma(K,eta),each=n)
 temp <- rexp(K*n)
 temp<- (eta*log(eta*temp/(eta*Gam1)+1)/(exp(beta*X)*weiscale^weishape))^{1/weishape}
 status<- ifelse(temp<=C,1,0);
 temp <-   pmin(temp,C)
 xt <- matrix(temp,n,K)
 minstime <- apply(xt,2,min)  
 id=rep(1:K,each=n)
  ud <- cbind(temp,status,X,id)
if (left>0) {
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,0.5) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,0.5) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);}
  if (pairleft==1) ud <- cbind(ud,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,rep(minstime,each=n),lefttime,trunk)

  colnames(ud)<-c("time","status","x","cluster","mintime","lefttime","truncated")
  ud <- data.frame(ud)
  return(ud)
} ## }}} 

##' @export
simClaytonOakes.twin.ace <- function(K,varg,varc,beta,stoptime,Cvar=0,left=0,pairleft=0,trunc.prob=0.5,lam0=1)  ## {{{ 
{
  ## K antal clustre, n=antal i clustre
  n <- 2 # twins with ace structure
  #change parametrization 
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2; 
  varc <- varc/sumpar^2
  x<-array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  if (Cvar==0) C<-matrix(stoptime,K,n) else C<-matrix(Cvar*runif(K*n)*stoptime,K,n) 
  ### total variance of gene and env. 
  ###  random effects with 
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  etao <- eta <- varc+varg
  if (etao==0) eta <- 1
  Gams1 <-cbind( rgamma(K,varg)/eta, rgamma(K,varg*0.5)/eta, rgamma(K,varg*0.5)/eta, rgamma(K,varg*0.5)/eta, rgamma(K,varc)/eta )
  mz <- c(rep(1,K/2),rep(0,K/2)); dz <- 1-mz;
  mzrv <-  Gams1[,1]+Gams1[,5]           ### shared gene + env 
  dzrv1 <- Gams1[,2]+Gams1[,3]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  dzrv2 <- Gams1[,2]+Gams1[,4]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  Gam1 <- cbind(mz*mzrv+dz*dzrv1,mz*mzrv+dz*dzrv2)
  Gam1[Gam1==0] <- 1 ## to work also under independence 
  temp<-eta*log(-log(1-x[,,1])/(eta*Gam1)+1)*exp(-beta*x[,,3])/lam0
  if (etao==0) temp <- matrix(rexp(n*K),K,n)*exp(-beta*x[,,3])/lam0
  x[,,2]<-ifelse(temp<=C,1,0);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  
  zyg <- c(rep("MZ",K),rep("DZ",K))

if (left>0) { ## {{{ 
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);} ## }}} 
  if (pairleft==1) ud <- cbind(ud,zyg,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,zyg,rep(minstime,each=n),lefttime,trunk)

names(ud)<-c("time","status","x","cluster","zyg","mintime","lefttime","truncated")
return(ud)
} ## }}} 

##' @export
simClaytonOakes.family.ace <- function(K,varg,varc,beta,stoptime,lam0=0.5,Cvar=0,left=0,pairleft=0,trunc.prob=0.5)  ## {{{ 
{
  ## K antal clustre (families), n=antal i clustre
  n <- 4 # twins with ace structure
  x<- array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  if (Cvar==0) C<-matrix(stoptime,K,n) else C<-matrix(Cvar*runif(K*n)*stoptime,K,n) 
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2; 
  varc <- varc/sumpar^2
  ### total variance of gene and env. 
  ###  random effects with 
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  eta <- varc+varg
  ### mother and father share environment
  ### children share half the genes with mother and father and environment 
  mother.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  father.g <-  cbind(rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta, rgamma(K,varg*0.25)/eta)
  env <- rgamma(K,varc)/eta 
  mother <- apply(mother.g,1,sum)+env
  father <- apply(father.g,1,sum)+env
  child1 <- apply(cbind(mother.g[,c(1,2)],father.g[,c(1,2)]),1,sum) + env
  child2 <- apply(cbind(mother.g[,c(1,3)],father.g[,c(1,3)]),1,sum) + env
  Gam1 <- cbind(mother,father,child1,child2)
  temp<-eta*log(-log(1-x[,,1])/(eta*Gam1)+1)*exp(-beta*x[,,3])/lam0
  x[,,2]<-ifelse(temp<=C,1,0);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  
  type <- rep(c("mother","father","child","child"),K)

if (left>0) { ## {{{ 
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);} ## }}} 
  if (pairleft==1) ud <- cbind(ud,type,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,type,rep(minstime,each=n),lefttime,trunk)

names(ud)<-c("time","status","x","cluster","type","mintime","lefttime","truncated")
return(ud)
} ## }}} 

##' @export
simCompete.twin.ace <- function(K,varg,varc,beta,stoptime,lam0=c(0.2,0.3),
		Cvar=0,left=0,pairleft=0,trunc.prob=0.5,overall=1,all.sum=1)  ## {{{ 
{
  ## K antal clustre, n=antal i clustre
  n=2 # twins with ace structure
  sumpar <- sum(varg+varc)
  varg <- varg/sumpar^2; 
  varc <- varc/sumpar^2
  ## length(lam0) competing risk with constant hazards lam0
  x<-array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  if (Cvar==0) C<-matrix(stoptime,K,n) else C<-matrix(Cvar*runif(K*n)*stoptime,K,n) 
  ### total variance of gene and env. 
  ### one for each cause and one shared (across causes)
  ###  random effects with 
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  if (length(varc)==1) varc  <- rep(varc,length(lam0)+overall)
  if (length(varg)==1) varg  <- rep(varg,length(lam0)+overall)
  eta <- varc+varg
  etat <- sum(eta)
  ### total variance for each cause + overall
  nc <- length(lam0); 

  mz <- c(rep(1,K/2),rep(0,K/2)); dz <- 1-mz;
  ### ace overall 
  if (overall==1) {
    varcl <- varc[nc+1]; vargl <- varg[nc+1]
    if (all.sum==1) etal <-  etat else etal <- vargl+varcl
    Gams1 <-cbind(
       rgamma(K,vargl)/etal, rgamma(K,vargl*0.5)/etal, rgamma(K,vargl*0.5)/etal, rgamma(K,vargl*0.5)/etal,
       rgamma(K,varcl)/etal )
  mzrv <-  Gams1[,1]+          Gams1[,5] ### shared gene + env 
  dzrv1 <- Gams1[,2]+Gams1[,3]+Gams1[,5] 
  dzrv2 <- Gams1[,2]+Gams1[,4]+Gams1[,5]
  Gamoa <- cbind(mz*mzrv+dz*dzrv1,mz*mzrv+dz*dzrv2)
  } else Gamoa <- 0

  temp1 <- matrix(0,K,length(lam0))
  temp2 <- matrix(0,K,length(lam0))
  Gamm <- c()
  for (i in 1:nc)
  {
  varcl <- varc[i]; vargl <- varg[i]
  if (all.sum==1) etal <-  etat else etal <- vargl+varcl
  Gams1 <-cbind(
      rgamma(K,vargl)/etal, 
      rgamma(K,vargl*0.5)/etal, rgamma(K,vargl*0.5)/etal, rgamma(K,vargl*0.5)/etal,
      rgamma(K,varcl)/etal )
  ###  
  mzrv <-  Gams1[,1]+Gams1[,5]     ### shared gene + env 
  dzrv1 <- Gams1[,2]+Gams1[,3]+Gams1[,5] 
  dzrv2 <- Gams1[,2]+Gams1[,4]+Gams1[,5]
  Gam1 <- cbind(mz*mzrv+dz*dzrv1,mz*mzrv+dz*dzrv2)
  Gam1 <- Gam1+Gamoa
  Gamm <- cbind(Gamm,Gam1)
  ttemp<-matrix(rexp(2*K),K,2)/(Gam1*exp(beta*x[,,3])*lam0[i])
  temp1[,i] <- ttemp[,1]
  temp2[,i] <- ttemp[,2]
  }
  temp <- cbind( apply(temp1,1,min), apply(temp2,1,min))
  cause1 <- apply(temp1,1,which.min)
  cause2 <- apply(temp2,1,which.min)
  x[,,2]<- ifelse(temp<=C,1,0)*cbind(cause1,cause2);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  
  zyg <- c(rep("MZ",K),rep("DZ",K))

if (left>0) { ## {{{ 
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);} ## }}} 
  if (pairleft==1) ud <- cbind(ud,zyg,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,zyg,rep(minstime,each=n),lefttime,trunk)

names(ud)<-c("time","status","x","cluster","zyg","mintime","lefttime","truncated")
return(ud)
} ## }}} 

##' @export
simCompete.simple <- function(K,varr,beta,stoptime,lam0=c(0.2,0.3),
	Cvar=0,left=0,pairleft=0,trunc.prob=0.5,overall=1,all.sum=1)  ## {{{ 
{
  ## K antal clustre, n=antal i clustre
  n=2 # twins with ace structure
  ## length(lam0) competing risk with constant hazards lam0
  x<-array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  if (Cvar==0) C<-matrix(stoptime,K,n) else C<-matrix(Cvar*runif(K*n)*stoptime,K,n) 
  ## variance and mean of additve gamma via paramenters 
  sp <- sum(varr)
  partheta <- varr/sp^2 

  eta <- partheta
  etat <- sum(eta)
  ### total variance for each cause + overall
  nc <- length(lam0); 
  print(eta)
  print(etat)

  mz <- c(rep(1,K/2),rep(0,K/2)); dz <- 1-mz;
  ### ace overall 
  if (overall==1) {
    etal <- etat
    Gams1 <-cbind(rgamma(K,eta[nc+1])/etal)
  Gamoa <- Gams1
  } else Gamoa <- 0


  temp1 <- matrix(0,K,length(lam0))
  temp2 <- matrix(0,K,length(lam0))
  for (i in 1:nc)
  {
	  etal <- etat
	  Gams1 <-cbind(rgamma(K,eta[i])/etal)
	  Gam1 <- Gams1+Gamoa
	  Gam1 <- cbind(Gam1,Gam1)
	  occ <- (1:nc)[-i]
	  for (j in  occ) {
		  print(eta[j])
	     Gamo <- cbind(rgamma(K,eta[j])/etal,rgamma(K,eta[j])/etal)
	     Gam1 <- Gam1+Gamo
	  }
  print(apply(Gam1,2,mean))
  print(apply(Gam1,2,var))
	  ttemp<-matrix(rexp(2*K),K,2)/(Gam1*exp(beta*x[,,3])*lam0[i])
	  temp1[,i] <- ttemp[,1]
	  temp2[,i] <- ttemp[,2]
  }
  temp <- cbind( apply(temp1,1,min), apply(temp2,1,min))
  cause1 <- apply(temp1,1,which.min)
  cause2 <- apply(temp2,1,which.min)
  x[,,2]<- ifelse(temp<=C,1,0)*cbind(cause1,cause2);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  
  zyg <- c(rep("MZ",K),rep("DZ",K))

if (left>0) { ## {{{ 
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);} ## }}} 
  if (pairleft==1) ud <- cbind(ud,zyg,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,zyg,rep(minstime,each=n),lefttime,trunk)

names(ud)<-c("time","status","x","cluster","zyg","mintime","lefttime","truncated")
return(ud)
} ## }}} 

##' @export
simFrailty.simple <- function(K,varr,beta,stoptime,lam0=c(0.2),
		Cvar=0,left=0,pairleft=0,trunc.prob=0.5,overall=1,all.sum=NULL)  ## {{{ 
{
  n=2 
  ## length(lam0) competing risk with constant hazards lam0
  x<-array(c(runif(n*K),rep(0,n*K),rbinom(n*K,1,0.5)),dim=c(K,n,3))
  if (Cvar==0) C<-matrix(stoptime,K,n) else C<-matrix(Cvar*runif(K*n)*stoptime,K,n) 
  if (length(varr)==1) varr  <- rep(varr,length(lam0)+overall)
  eta <- varr
  etat <- sum(varr)
  if (!is.null(all.sum)) etat <- all.sum 
  varr <- varr/etat 
  ### total variance for each cause + overall
  nc <- length(lam0); 

  mz <- c(rep(1,K/2),rep(0,K/2)); dz <- 1-mz;
  if (overall==1) {
    etal <- etat
    Gams1 <-cbind(rgamma(K,varr[nc+1])/etal)
    Gamoa <- Gams1
  } else Gamoa <- 0

  print(etat); print(varr)
  temp1 <- matrix(0,K,length(lam0))
  temp2 <- matrix(0,K,length(lam0))
  for (i in 1:nc)
  {
  etal <- etat
  Gams1 <-cbind(rgamma(K,varr[i])/etal)
  Gam1 <- Gams1+Gamoa
  Gam1 <- cbind(Gam1,Gam1)
  print(i); 
  print(apply(Gam1,2,mean)); print(apply(Gam1,2,var)); 
  print(apply(Gams1,2,mean)); print(apply(Gams1,2,var)); 
  print(apply(as.matrix(Gamoa),2,mean)); print(apply(as.matrix(Gamoa),2,var)); 
  ttemp<-matrix(rexp(2*K),K,2)/(Gam1*exp(beta*x[,,3])*lam0[i])
  temp1[,i] <- ttemp[,1]
  temp2[,i] <- ttemp[,2]
  }
  temp <- cbind( apply(temp1,1,min), apply(temp2,1,min))
  cause1 <- apply(temp1,1,which.min)
  cause2 <- apply(temp2,1,which.min)
  x[,,2]<- ifelse(temp<=C,1,0)*cbind(cause1,cause2);
  x[,,1]<-pmin(temp,C)
  minstime <- apply(x[,,1],1,min)  
  ud <- as.data.frame(cbind(apply(x,3,t),rep(1:K,each=n)))  
  zyg <- c(rep("MZ",K),rep("DZ",K))

if (left>0) { ## {{{ 
     if (pairleft==1) {
     lefttime <- runif(K)*(stoptime-left)
     left <- rbinom(K,1,trunc.prob) ## not trunation times!
     lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > minstime)
       medleft <- rep(trunk,each=n)
     } else {
       lefttime <- runif(K)*(stoptime-left)
       left <- rbinom(n*K,1,trunc.prob) ## not trunation times!
       lefttime <- apply(cbind(lefttime*left,3),1,min)
       trunk <- (lefttime > ud[,1])
       medleft <- trunk
     }
  } else { lefttime <- trunk <- rep(0,K);} ## }}} 
  if (pairleft==1) ud <- cbind(ud,zyg,rep(minstime,each=n),rep(lefttime,each=n),rep(trunk,each=n))
  else ud <- cbind(ud,zyg,rep(minstime,each=n),lefttime,trunk)

names(ud)<-c("time","status","x","cluster","zyg","mintime","lefttime","truncated")
return(ud)
} ## }}} 

##' @export
kendall.ClaytonOakes.twin.ace <- function(parg,parc,K=10000,test=0)  ## {{{ 
{ 
  ## K antal clustre, n=antal i clustre
  ### total variance of gene and env. 
  ###  K <- 10; varg <- 1; varc <- 1; 
  ###  random effects with 
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  sumpar <- sum(parg+parc)
  parg <- parg/sumpar^2; 
  parc <- parc/sumpar^2
  K <- K*2
  eta <- parc+parg
  Gams1 <-cbind( rgamma(K,parg)/eta, rgamma(K,parg*0.5)/eta, 
                 rgamma(K,parg*0.5)/eta, rgamma(K,parg*0.5)/eta, 
		 rgamma(K,parc)/eta )
  mz <- c(rep(1,K/2),rep(0,K/2)); 
  dz <- 1-mz;
  id <- rep(1:(K/2),each=2)
  mzrv <-  Gams1[,1]+Gams1[,5]           ### shared gene + env 
  dzrv1 <- Gams1[,2]+Gams1[,3]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  dzrv2 <- Gams1[,2]+Gams1[,4]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  Gam1 <- cbind(mz*mzrv+dz*dzrv1,mz*mzrv+dz*dzrv2)
  if (test==1) {
	  cat("mz cor")
	  print(apply(Gam1[mz==1,1:2],2,var))
	  print(cor(Gam1[mz==1,1:2]))
	  cat("dz cor")
	  print(apply(Gam1[mz==0,1:2],2,var))
	  print(cor(Gam1[mz==0,1:2]))
  }
  Gam1 <- data.frame(cbind(Gam1,mz,id))

  ## Silence false R CMD CHECK warnings:
  V11 <- V12 <- V21 <- V22 <- NULL
    
  gams.pair <- fast.reshape(Gam1,id="id")
  gams.pair <-  transform(gams.pair,
              kendall = ((V11-V12)*(V21-V22))/((V11+V12)*(V21+V22))
  )

  kendall <- gams.pair$kendall
  mz.kendall <- mean(kendall[gams.pair$mz1==1])
  dz.kendall <- mean(kendall[gams.pair$mz1==0])

  return(list(mz.kendall=mz.kendall,dz.kendall=dz.kendall))
} ## }}} 

##' @export
kendall.normal.twin.ace <- function(parg,parc,K=10000)  ## {{{ 
{
  ## K antal clustre, n=antal i clustre
  ### total variance of gene and env. 
  ###  K <- 10; varg <- 1; varc <- 1; 
  ###  random effects with 
  ###  means varg/(varg+varc) and variances varg/(varg+varc)^2
  K <- K*2
  Gams1 <-cbind( parg^.5*rnorm(K,parg), (parg*0.5)^.5*rnorm(K), 
                 (parg*0.5)^.5*rnorm(K), (parg*0.5)^.5*rnorm(K), parc^.5*rnorm(K) )
  mz <- c(rep(1,K/2),rep(0,K/2)); 
  dz <- 1-mz;
  id <- rep(1:(K/2),each=2)
  mzrv <-  Gams1[,1]+Gams1[,5]           ### shared gene + env 
  dzrv1 <- Gams1[,2]+Gams1[,3]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  dzrv2 <- Gams1[,2]+Gams1[,4]+Gams1[,5] ### 0.5 shared gene + 0.5 non-shared + env 
  Gam1 <- cbind(mz*mzrv+dz*dzrv1,mz*mzrv+dz*dzrv2)
  Gam1 <- data.frame(cbind(exp(Gam1),mz,id))

  ## Silence false R CMD CHECK warnings:
  V11 <- V12 <- V21 <- V22 <- NULL

  gams.pair <- fast.reshape(Gam1,id="id")
  gams.pair <-  transform(gams.pair,
              kendall = ((V11-V12)*(V21-V22))/((V11+V12)*(V21+V22))
  )

  kendall <- gams.pair$kendall
  mz <- gams.pair$mz1
  mz.kendall <- mean(kendall[mz==1])
  dz.kendall <- mean(kendall[mz==0])

  return(list(mz.kendall=mz.kendall,dz.kendall=dz.kendall))
} ## }}} 

##' While-Alive estimands for recurrent events 
##'
##' Considers the ratio of means \deqn{E(N(min(D,t)))/E(min(D,t))} and the
##' the mean of the events per time unit \deqn{E(N(min(D,t))/min(D,t))} both based on
##' IPCW etimation. RMST estimator equivalent to Kaplan-Meier based estimator.
##'
##' @param formula Event formula first covariate on rhs must be a factor giving the treatment
##' @param data data frame 
##' @param time for estimation 
##' @param cens.code of censorings 
##' @param cause of events 
##' @param death.code of terminal events 
##' @param trans possible power for mean of events per time-unit
##' @param cens.formula censoring model, default is to use strata(treatment)
##' @param augmentR covariates for model of mean ratio
##' @param augmentC covariates for censoring augmentation
##' @param type augmentation for call of binreg, when augmentC is given default is "I" and otherwise "II"
##' @param marks possible marks for composite outcome situation for model for counts with marks
##' @param ...  arguments for binregATE 
##' @author Thomas Scheike
##' @examples
##' library(mets)
##' data(hfactioncpx12)
##' 
##' dtable(hfactioncpx12,~status)
##' dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
##' summary(dd)
##' @references 
##' Nonparametric estimation of the Patient Weighted While-Alive Estimand arXiv preprint by A. Ragni, T. Martinussen, T. Scheike
##' Mao, L. (2023). Nonparametric inference of general while-alive estimands for recurrent events. Biometrics, 79(3):1749–1760.
##' Schmidli, H., Roger, J. H., and Akacha, M. (2023). Estimands for recurrent event endpoints in the presence of a terminal event. Statistics in Biopharmaceutical Research, 15(2):238–248.
##' @export
WA_recurrent <- function(formula,data,time=NULL,cens.code=0,cause=1,death.code=2,
	 trans=NULL,cens.formula=NULL,augmentR=NULL,augmentC=NULL,type=NULL,marks=NULL,...)
{ ## {{{
  cl <- match.call() ## {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","marks"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

   call.id <- id
   conid <- construct_id(id, nrow(X), namesX = rownames(X))
   name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
   orig.id <- id
   data$id__ <- id 
    if (is.null(des.offset)) {
        if (is.null(offset))
            offset <- rep(0, length(exit))
    } else offset <- des.offset
    if (is.null(des.weights)) {
        if (is.null(weights))
            weights <- rep(1, length(exit))
    } else weights <- des.weights
    if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  ## }}}

## use sorted id for all things  and here indentify last record of each subject
cid <- countID(data,"id__",sorted=TRUE)
rrR <- subset(data,cid$reverseCountid==1)

## first var on rhs of formula
vars <- all.vars(formula)
treat.name <- vars[4]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
ntreatvar <- as.numeric(treatvar) - 1
statusD <- rrR[, vars[3]] %in% death.code
rrR[["statusD__"]] <- statusD
treat.formula <- treat.model <- as.formula(paste(treat.name,"~+1",sep=""))
if (is.null(cens.formula)) cens.formula <- as.formula( paste("~strata(",treat.name,")",collapse=""))
formC <- as.formula( paste("Event(",vars[1],",",vars[2],",",vars[3],"%in%", cens.code,")~+cluster(id__)",collapse=""))
formD <- as.formula( paste("Event(",vars[2], ",statusD__)~-1+",treat.name,"+cluster(id__)",collapse=""))
form1 <- as.formula( paste("Event(",vars[2],",",vars[3],")~-1+",treat.name,"+cluster(id__)",collapse=""))

  ## drop id as cluster variable and use id__
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))
## take out intercept, to get mean in treated/non-treated
formrec <- update(formula, reformulate(c(".", "-1")))

if (!is.null(augmentR)) {
   varsR <- c(attr(terms(augmentR), "term.labels"))
   form1X <- update(form1, reformulate(c(".", varsR)))
} else form1X <- form1

## ratio of means ## {{{
dd <- resmeanIPCW(formD,data=rrR,cause=1,cens.code=0,cens.model=cens.formula,time=time, model="lin")
ddN <- recregIPCW(formrec,data=data,cause=cause,death.code=death.code,cens.code=cens.code, cens.model=cens.formula,times=time,model="lin",marks=marks)

treatdata <- data.frame(treatment=nlevs,id__=1)

f <- function(p,logg=0) {
ddN$coef <- p[1:2]
dd$coef <- p[3:4]
prmst <- predict(dd,treatdata,se=0)
prec <- predict(ddN,treatdata,se=0)
out <- prec/prmst
if (logg==1) out <- log(out)
return(out)
}

cc <- c(ddN$coef,dd$coef)
cciid <- cbind(ddN$iid,dd$iid)
ratio.means  <- estimate(coef=cc,vcov=crossprod(cciid),f=f)
###			 f=function(p) (p[1:2]/p[3:4]))
ratio.means.log <- estimate(coef=cc,vcov=crossprod(cciid),f=f,logg=1)
RAW <- list(iid=cciid,coef=cc,time=time,rmst=dd,meanN=ddN,ratio.means=ratio.means,ratio.means.log=ratio.means.log)
## }}}

data <- count.history(data,id="id__",lag=TRUE,types=cause,status=vars[3],marks=marks,multitype=TRUE)

nameCount <- paste("Count",cause[1],sep="")
formulaCount <- update.formula(formula,.~+1)
cform <- as.formula(paste("~",nameCount,"+cluster(id__)",sep=""))
formulaCount <- update.formula(formulaCount,cform)

## While-Alive mean of events per time-unit 
## with possible marks for death.codes 
if (any(cause %in% death.code)) {
	wd <- match(cause,death.code,nomatch=0)
	mark.codes <- death.code[wd]
} else mark.codes <- NULL
dataDmin <- evalTerminal(formulaCount,data=data,time=time,death.code=death.code, mark.codes=mark.codes,marks=marks)

### setting new response , Ratio of composite outcome
rrR[,"ratio__"] <- dataDmin[cid$reverseCountid==1,"ratio"]
if (!is.null(trans)) {
     rrR[,"ratio__"] <- rrR[,"ratio__"]^trans
}
Yr <- rrR[,"ratio__"]

if (is.null(type)) {
    if (is.null(augmentC)) type <- "II" else type <- "I"
}

outae <- binregATE(form1X,rrR,cause=death.code,time=time,treat.model=treat.formula,
   cens.code=cens.code,Ydirect=Yr,outcome="rmst",model="lin",cens.model=cens.formula,
   type=type[1],...) 
ET <- list(riskDR=outae)

##  Yipcw as it comes in data frame 
data[,"ratio__"] <- outae$Yipcw[cid$indexid+1]

if (!is.null(augmentC)) { ## {{{
dc0 <- dynCensAug(formC,subset(data,ntreatvar==0),augmentC=augmentC,response="ratio__",time=time)
dc1 <- dynCensAug(formC,subset(data,ntreatvar==1),augmentC=augmentC,response="ratio__",time=time)
###nn <- table(rr$treat)
nid <- nrow(rrR)

treats <- function(treatvar) { treatvar <- droplevels(treatvar)# {{{
        nlev <- nlevels(treatvar)
        nlevs <- levels(treatvar)
        ntreatvar <- as.numeric(treatvar)
        return(list(nlev = nlev, nlevs = nlevs, ntreatvar = ntreatvar))
}  

fittreat <- function(treat.model, data, id, ntreatvar, nlev) {
if (nlev == 2) {
    treat.model <- drop.specials(treat.model, "cluster")
    treat <- glm(treat.model, data, family = "binomial")
    iidalpha <- lava::iid(treat, id =id)
    lpa <- treat$linear.predictors
    pal <- expit(lpa)
    pal <- cbind(1 - pal, pal)
    ppp <- (pal/pal[, 1])
    spp <- 1/pal[, 1]
} else {
    treat.modelid <- update.formula(treat.model, . ~ . + cluster(id__))
    treat <- mlogit(treat.modelid, data)
    iidalpha <- lava::iid(treat)
    pal <- predict(treat, data, se = 0, response = FALSE)
    ppp <- (pal/pal[, 1])
    spp <- 1/pal[, 1]
}
Xtreat <- model.matrix(treat.model, data)
tvg2 <- 1 * (ntreatvar >= 2)
pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
pppy <- c(mdi(ppp, 1:length(ntreatvar), ntreatvar))
Dppy <- (spp * tvg2 - pppy)
Dp <- c()
for (i in seq(nlev - 1)) Dp <- cbind(Dp, Xtreat * ppp[,
    i + 1] * Dppy/spp^2)
DPai <- -1 * Dp/pA^2
out <- list(iidalpha = iidalpha, pA = pA, Dp = Dp, pal = pal,
    ppp = ppp, spp = spp, id = id, DPai = DPai)
return(out)
} # }}}

expit <- function(x) 1/(1 + exp(-x))
idW <- rrR[,"id__"]
treatsvar <- rrR[,treat.name]
treats <- treats(treatsvar)

fitt <- fittreat(treat.model, rrR, idW, treats$ntreatvar, treats$nlev)
iidalpha0 <- fitt$iidalpha
wPA <- c(fitt$pA)
## ordered after idW
wPA <- wPA[order(idW)]

riskDRC <- outae$riskDR+c(dc0$augment,dc1$augment)/nid
MGC0 <- sumstrata(dc0$MGCiid/c(wPA[dc0$id+1]),dc0$id,nid)*dc0$n
MGC1 <- sumstrata(dc1$MGCiid/c(wPA[dc1$id+1]),dc1$id,nid)*dc1$n
MGC <- cbind(MGC0,MGC1)
ccaugment <- apply(MGC,2,sum)

riskDRC <- outae$riskDR+ccaugment/nid
iidDRC <- outae$riskDR.iid+MGC/nid
varDRC <- crossprod(iidDRC)
se.riskDRC <- diag(varDRC)^.5

riskDRC <- list(riskDRC=riskDRC,iid=iidDRC,var=varDRC,coef=riskDRC,se=se.riskDRC)
ET <- list(riskDRC=riskDRC,riskDR=outae)
###	   MGC=MGC,Y=data[,"ratio__"],dc0=dc0,dc1=dc1,wPA=wPA)
} ## }}}

###### sort iid after name.id and put as rownames
if (!is.null(call.id)) {
	ET$riskDRC$iid <- nameme(ET$riskDRC$iid,name.id)
	ET$riskDR$riskG.iid <- nameme(outae$riskG.iid,name.id)
	ET$riskDR$riskDR.iid <- nameme(outae$riskG.iid,name.id)
	ET$riskDR$iid <- nameme(ET$riskDR$iid,name.id)
}

out <- list(time=time,id=id,call.id=call.id,name.id=name.id,nid=nid,
	    trans=trans,cause=cause,cens.code=cens.code,death.code,
	    RAW=RAW,ET=ET,augmentR=augmentR,augmentC=augmentC)
class(out) <- "WA"
return(out)
} ## }}}

##' @export
print.WA  <- function(x,type="log",...) {# {{{
  print(summary(x),type=type,...)
}# }}}

##' @export
summary.WA <- function(object,type="p",augtype=NULL,...) {# {{{

rmst <- estimate(object$RAW$rmst)
rmst.test <- estimate(rmst,contrast=rbind(c(1,-1)))
rmst.log <- estimate(rmst,function(p) log(p))
rmst.test.log <- estimate(rmst.log,contrast=rbind(c(1,-1)))

meanNtD <- estimate(object$RAW$meanN)
meanNtD.test <- estimate(meanNtD,contrast=rbind(c(1,-1)))
meanNtD.log <- estimate(meanNtD,function(p) log(p))
meanNtD.test.log <- estimate(meanNtD.log,contrast=rbind(c(1,-1)))

eer <- estimate(object$RAW$ratio.means)
eedr <- estimate(eer,contrast=rbind(c(1,-1)))

if (is.null(augtype)) {
   augtype <- "riskDR"
   if (!is.null(object$ET$riskDRC)) augtype <- "riskDRC" 
}
if (augtype=="riskDR")
ee <- estimate(coef=object$ET$riskDR$riskDR,vcov=object$ET$riskDR$var.riskDR)
if (augtype=="riskDRC")
ee <- estimate(coef=object$ET$riskDRC$coef,vcov=object$ET$riskDRC$var)
eed <- estimate(ee,contrast=rbind(c(1,-1)))
eer.log <- object$RAW$ratio.means.log
eedr.log <- estimate(eer.log,contrast=rbind(c(1,-1)))
eelog <-  estimate(ee,function(p) log(p))
eedlog <- estimate(eelog,contrast=rbind(c(1,-1)))

res <- list(rmst=rmst,rmst.test=rmst.test,meanNtD=meanNtD,meanNtD.test=meanNtD.test,
            ratio=eer,test.ratio=eedr,meanpt=ee,test.meanpt=eed)
reslog <- list(rmst=rmst.log,rmst.test=rmst.test.log,
               meanNtD=meanNtD.log,meanNtD.test=meanNtD.test.log,
    ratio=eer.log,test.ratio=eedr.log,meanpt=eelog,test.meanpt=eedlog)
class(res) <- "summary.WA"
class(reslog) <- "summary.WA"
attr(res,"log") <- (type!="p")
attr(reslog,"log") <- (type!="p")

if (type=="p") return(res) else return(reslog)
}# }}}

##' @export
print.summary.WA <- function(x,...)
{# {{{

if (attr(x,"log")) 
cat(paste("While-Alive summaries, log-scale:",x$time,"\n\n"))
else
cat(paste("While-Alive summaries:",x$time,"\n\n"))

cat("RMST,  E(min(D,t)) \n")
print(x$rmst)
cat(" \n")
print(x$rmst.test)

cat("mean events, E(N(min(D,t))): \n")
print(x$meanNtD)
cat(" \n")
print(x$meanNtD.test)
cat("_______________________________________________________ \n")

cat("Ratio of means E(N(min(D,t)))/E(min(D,t)) \n")
print(x$ratio)
cat(" \n")
print(x$test.ratio)
cat("_______________________________________________________ \n")

cat("Mean of Events per time-unit E(N(min(D,t))/min(D,t)) \n")
print(x$meanpt)
cat(" \n")
print(x$test.meanpt)
} # }}}

dynCensAug <- function(formC,data,augmentC=~+1,response="Yipcw",time=NULL,Z=NULL) { ## {{{

if (is.null(time)) stop("must give time of response \n")
   data$Y__ <- data[,response]
   varsC <- c("Y__",attr(terms(augmentC), "term.labels"))
   formCC <- update(formC, reformulate(c(".", varsC)))
   cr2 <- phreg(formCC, data = data, no.opt = TRUE, no.var = 1,Z=Z)
   xx <- cr2$cox.prep
   icoxS0 <- rep(0,length(cr2$S0))
   icoxS0[cr2$S0>0] <- 1/cr2$S0[cr2$S0>0]
   S0i <- rep(0, length(xx$strata))
   S0i[xx$jumps + 1] <- icoxS0
   km <- TRUE
   if (!km) {
        cumhazD <- c(cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum)
        St <- exp(-cumhazD)
   } else St <- c(exp(cumsumstratasum(log(1 - S0i), xx$strata, xx$nstrata)$lagsum))

   nterms <- cr2$p-1
   dhessian <- cr2$hessianttime
   dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
   ###  matrix(apply(dhessian,2,sum),3,3)
   timeb <- which(cr2$cumhaz[,1]<time)
   ### take relevant \sum H_i(s,t) (e_i - \bar e)
   covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
   ### construct relevant \sum (e_i - \bar e)^2
   Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
   ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
   gammatt <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
   S0 <- cr2$S0[timeb]
   gammatt[is.na(gammatt)] <- 0
   gammatt[gammatt==Inf] <- 0
   Gctb <- St[cr2$cox.prep$jumps+1][timeb]
   augmentt.times <- apply(gammatt*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum)
   augment.times <- sum(augmentt.times)
   if (!is.null(Z)) {
             Zj <- cr2$cox.prep$Z[cr2$cox.prep$jumps+1,][timeb]
             Xaugment.times <- apply( augmentt.times*Zj,2,sum)
   }

   p <- 1
   #### iid magic  for censoring augmentation martingale #
   ### int_0^infty gamma(t) (e_i - ebar(s)) 1/G_c(s) dM_i^c
   xx <- cr2$cox.prep
   nid <- max(xx$id)+1
   jumpsC <- xx$jumps+1
   rr0 <- xx$sign
   S0i <- rep(0,length(xx$strata))
   S0i[jumpsC] <- c(1/(icoxS0*St[jumpsC]))
   S0i[jumpsC] <- icoxS0
   xxtime <- 1*c(xx$time<time)

   pXXA <- ncol(cr2$E)-1
   EA <- cr2$E[timeb,-1,drop=FALSE]
   gammasEs <- .Call("CubeMattime",gammatt,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   gammasE <- matrix(0,length(xx$strata),1)
   gammattt  <-    matrix(0,length(xx$strata),pXXA*1)
   jumpsCt <- jumpsC[timeb]
   gammasE[jumpsCt,] <- gammasEs
   gammattt[jumpsCt,] <- gammatt
   gammaEsdLam0 <- apply(gammasE*S0i*xxtime,2,cumsumstrata,xx$strata,xx$nstrata)
   gammadLam0 <-   apply(gammattt*S0i*xxtime,2,cumsumstrata,xx$strata,xx$nstrata)
   XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X[,-1,drop=FALSE],pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
   Ut <- Et <- matrix(0,length(xx$strata),1)
   Ut[jumpsCt,] <- augmentt.times
   MGCt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
   MGCiid <- apply(MGCt,2,sumstrata,xx$id,nid)
   MGCiid <- MGCiid/nid

   nid <- max(cr2$id)+1
   res <- list(MGCiid=MGCiid,gammat=gammatt,augment=augment.times,
	       id=cr2$name.id,n=nid)
} ## }}}

##' Evaluates piece constant covariates at min(D,t) where D is a terminal event
##'
##' returns X(min(D,t)) and min(D,t) and their ratio. for censored observation 0. 
##' to use with the IPCW models implemented. 
##'
##' @param formula formula with 'Event' outcome and X to evaluate at min(D,t)
##' @param data data frame
##' @param death.code codes for death (terminating event, 2 default)
##' @param time for evaluation 
##' @param marks for terminal events to add marks*I(D <=t ,epsilon "in" mark.codes)  to X(min(D,t))
##' @param mark.codes gives death codes for which to add mark value
##' @author Thomas Scheike
##' @export
evalTerminal <- function(formula,data=data,death.code=2,time=NULL,marks=NULL,mark.codes=NULL)
{# {{{
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    id <- strata <- NULL
    if (!is.null(attributes(Terms)$specials$cluster)) {
        ts <- survival::untangle.specials(Terms, "cluster")
        pos.cluster <- ts$terms
        Terms  <- Terms[-ts$terms]
        id <- m[[ts$vars]]
    } else pos.cluster <- NULL
###    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
###        ts <- survival::untangle.specials(Terms, "strata")
###        pos.strata <- ts$terms
###        Terms  <- Terms[-ts$terms]
###        strata <- m[[ts$vars]]
###        strata.name <- ts$vars
###    }  else { strata.name <- NULL; pos.strata <- NULL}
###    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
###        ts <- survival::untangle.specials(Terms, "offset")
###        Terms  <- Terms[-ts$terms]
###        offset <- m[[ts$vars]]
###    }
    X <- model.matrix(Terms, m)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## }}}

   if (is.null(time)) time <- max(exit)+1

###   if (!is.null(id)) {
###	   call.id <- id
###        ids <- unique(id)
###        nid <- length(ids)
###        if (is.numeric(id))
###            id <-  fast.approx(ids,id)-1
###        else  {
###            id <- as.integer(factor(id,labels=seq(nid)))-1
###        }
###    } else { call.id <- id <- as.integer(seq_along(entry))-1;  nid <- nrow(X); }
###    ## orginal id coding into integers 1:...
###   id <- id+1

###   ## new id 1,2,.... and so on, referring to rows of data
   call.id <- id
   conid <- construct_id(id,length(exit))
   id <- conid$id+1; nid <- conid$nid; 

   call.marks <- marks
   if (is.null(marks)) marks <- rep(1,length(id))

###   dd <- data.frame(id=id)
###   dd <- countID(dd,sorted=TRUE)
###   ## new id 1,2,.... and so on, referring to rows of data
###   id <- dd$indexid+1

 ###
 indexD  <- which(exit <= time & (status %in% death.code))
 indexDM  <- which(exit <= time & (status %in% mark.codes))
 idD <- id[indexD]
 idDM <- id[indexDM]
 obsid <- MarkDid <- Dmintid <- rep(0,nid)
 Dmintid[idD] <- exit[indexD]
 MarkDid[idDM] <- marks[idDM]
 ### 
 indexA<- which(entry <= time & time <= exit)
 idA <- id[indexA]
 Dmintid[idA] <- time
 obsid[idA] <- 1
 Dmint <- Dmintid[id]
 obsid[idA] <- 1
 obsid[idD] <- 1
 ###
 XminDtid <- matrix(0,nid,ncol(X))
 colnames(XminDtid) <- colnames(X)
 XminDtid[idD,] <- X[indexD,]
 XminDtid[idA,]  <- X[indexA,]
 if (length(idDM)>0) XminDtid  <- XminDtid+MarkDid
 XminDt <- XminDtid[id,,drop=FALSE]
 ratio <- rep(0,length(exit))
 obs <- obsid[id]
 ratio[obs==1] <- XminDt[obs==1]/Dmint[obs==1]

 nX <- colnames(XminDt) <- paste(colnames(X),"minDt",sep="") 
 if (ncol(X)==1) nR <- "ratio" else nR <- paste(colnames(X),"ratio",sep="") 
 dd <- data.frame(cbind(XminDt,Dmint,id,call.id,obs,ratio))
 colnames(dd) <- c(nX,"minDt","nid","call.id","uncensored",nR)

 return(dd)
} # }}}

##' Wild bootstrap for Cox PH regression
##'
##' wild bootstrap for uniform bands for Cox models
##' @param formula formula with 'Surv' outcome (see \code{coxph})
##' @param data data frame
##' @param offset offsets for cox model
##' @param weights weights for Cox score equations
##' @param B bootstraps 
##' @param type distribution for multiplier
##' @param ... Additional arguments to lower level funtions
##' @author Klaus K. Holst, Thomas Scheike
##' @examples
##' 
##'  n <- 100
##'  x <- 4*rnorm(n)
##'  time1 <- 2*rexp(n)/exp(x*0.3)
##'  time2 <- 2*rexp(n)/exp(x*(-0.3))
##'  status <- ifelse(time1<time2,1,2)
##'  time <- pmin(time1,time2)
##'  rbin <- rbinom(n,1,0.5)
##'  cc <-rexp(n)*(rbin==1)+(rbin==0)*rep(3,n)
##'  status <- ifelse(time < cc,status,0)
##'  time  <- ifelse(time < cc,time,cc)
##'  data <- data.frame(time=time,status=status,x=x)
##' 
##'  b1 <- Bootphreg(Surv(time,status==1)~x,data,B=1000)
##'  b2 <- Bootphreg(Surv(time,status==2)~x,data,B=1000)
##'  c1 <- phreg(Surv(time,status==1)~x,data)
##'  c2 <- phreg(Surv(time,status==2)~x,data)
##' 
##'  ### exp to make all bootstraps positive
##'  out <- pred.cif.boot(b1,b2,c1,c2,gplot=0)
##' 
##'  cif.true <- (1-exp(-out$time))*.5
##'  with(out,plot(time,cif,ylim=c(0,1),type="l"))
##'  lines(out$time,cif.true,col=3)
##'  with(out,plotConfRegion(time,band.EE,col=1))
##'  with(out,plotConfRegion(time,band.EE.log,col=3))
##'  with(out,plotConfRegion(time,band.EE.log.o,col=2))
##' 
##' @references
##' 
##' Wild bootstrap based confidence intervals for multiplicative hazards models, 
##' Dobler, Pauly, and Scheike (2018), 
##' 
##' @aliases pred.cif.boot 
##' @export
Bootphreg <- function(formula,data,offset=NULL,weights=NULL,B=1000,type=c("exp","poisson","normal"),...) {# {{{
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL
  if (!is.null(attributes(Terms)$specials$strata)) {
    ts <- survival::untangle.specials(Terms, "strata")
    pos.strata <- ts$terms
    Terms  <- Terms[-ts$terms]
    strata <- m[[ts$vars]]
    strata.name <- ts$vars
  }  else { strata.name <- NULL; pos.strata <- NULL}
  X <- model.matrix(Terms, m)
  if (!is.null(intpos  <- attributes(Terms)$intercept))
    X <- X[,-intpos,drop=FALSE]
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  res <- Bootphreg01(X,entry,exit,status,id,strata,offset,weights,strata.name,B=B,
		     type=type,...)
  class(res) <- "boot-phreg"
  
  res
}# }}}

###{{{ Bootpreg01

Bootphreg01 <- function(X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,weights=NULL,
             strata.name=NULL,B=1000,type=c("normal","poisson","exp"),
	     cumhaz=TRUE,
             beta,stderr=TRUE,
	     method="NR",no.opt=FALSE,Z=NULL,propodds=NULL,AddGam=NULL,
	     case.weights=NULL,...) {
  p <- ncol(X)
  if (missing(beta)) beta <- rep(0,p)
  if (p==0) X <- cbind(rep(0,length(exit)))
  if (is.null(strata)) { strata <- rep(0,length(exit)); nstrata <- 1; strata.level <- NULL; } else {
	  strata.level <- levels(strata)
	  ustrata <- sort(unique(strata))
	  nstrata <- length(ustrata)
	  strata.values <- ustrata
      if (is.numeric(strata)) strata <-  fast.approx(ustrata,strata)-1 else  {
      strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
    }
  }
  if (is.null(offset)) offset <- rep(0,length(exit)) 
  if (is.null(weights)) weights <- rep(1,length(exit)) 
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given 
  if (!is.null(Z)) Zcall <- Z

  ## possible casewights to use for bootstrapping and other things
  if (is.null(case.weights)) case.weights <- rep(1,length(exit)) 

  trunc <- (!is.null(entry))
  if (!trunc) entry <- rep(0,length(exit))

  id.orig <- id; 
  if (!is.null(id)) {
	  ids <- unique(id)
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
   } else id <- as.integer(seq_along(entry))-1; 

   system.time(dd <- .Call("FastCoxPrepStrata",
		     entry,exit,status,X, id, ### as.integer(seq_along(entry)),
		     trunc,strata,weights,offset,Zcall,case.weights,PACKAGE="mets"))

   dd$nstrata <- nstrata
   nj <- length(exit)

   res <- list()
   for (i in 1:B) {

      g <- switch(type[1],
                    "normal" =  {rnorm(nj)+1},
                    "exp"     = {rexp(nj)},
                    "poisson" = {rpois(nj,1)}
      )
      ## follows id that may be given in call with cluster(id)
      dd$caseweights[dd$id+1] <- g

   obj <- function(pp,U=FALSE,all=FALSE) {# {{{
		if (is.null(propodds) & is.null(AddGam)) 
	  val <- with(dd,
		   .Call("FastCoxPLstrata",pp,X,XX,sign,jumps,
		    strata,nstrata,weights,offset,ZX,caseweights,PACKAGE="mets"))
         else if (is.null(AddGam)) 
		 val <- with(dd,
		   .Call("FastCoxPLstrataPO",pp,X,XX,sign,jumps,
		    strata,nstrata,weights,offset,ZX,propodds,PACKAGE="mets"))
	 else val <- with(dd,
		   .Call("FastCoxPLstrataAddGam",pp,X,XX,sign,jumps,
		    strata,nstrata,weights,offset,ZX,
		    AddGam$theta,AddGam$dimthetades,AddGam$thetades,AddGam$ags,AddGam$varlink,AddGam$dimjumprv,AddGam$jumprv,AddGam$JumpsCauses,PACKAGE="mets"))
	 

	  if (all) {
	      val$time <- dd$time
	      val$ord <- dd$ord+1
	      val$jumps <- dd$jumps+1
	      val$jumptimes <- val$time[val$jumps]
	      val$strata.jumps <- val$strata[val$jumps]
	      val$nevent <- length(val$S0)
	      val$nstrata <- dd$nstrata
	      val$strata <- dd$strata
	      return(val)
	  }
	  with(val, structure(-ploglik,gradient=-gradient,hessian=-hessian))
	}# }}}

  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          opt <- lava::NR(beta,obj,...)
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  names(cc) <- colnames(X)
      if (!stderr) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
  } else {
      val <- obj(beta,all=TRUE)
###      val[c("ploglik","gradient","hessian","U")] <- NULL
  }

  se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL
  II <- NULL
  ### computes Breslow estimator 
 strata <- val$strata[val$jumps]
 nstrata <- val$nstrata
 jumptimes <- val$jumptimes

 ## Brewslow estimator
 cumhaz <- cbind(jumptimes,cumsumstrata(1/val$S0,strata,nstrata))

 colnames(cumhaz)    <- c("time","cumhaz")

 res[[i]] <- list(coef=cc,cumhaz=cumhaz[,2])
 }
 names(res) <- 1:B

 return(res)
}

###}}} phreg0

##' @export
pred.cif.boot <- function(b1,b2,c1,c2,gplot=1)
{# {{{
B <- length(b1)

times1 <- c1$cumhaz[,1]
times2 <- c2$cumhaz[,1]
coef1 <- do.call("rbind",lapply(b1,function(x) x$coef))
coef2 <- do.call("rbind",lapply(b2,function(x) x$coef))
###
bcums1 <- do.call("cbind",lapply(b1,function(x) x$cumhaz))
bcums2 <- do.call("cbind",lapply(b2,function(x) x$cumhaz))

where2 <- fast.approx(c(0,times2),times1,tminus=TRUE)
cums2 <- c(0,c2$cumhaz[,2])
cums2 <- cums2[where2]
cums1 <- c1$cumhaz[,2]

bcums2 <- rbind(0,bcums2)[where2,]

n <- length(times1)
cif1 <- cumsum( exp(-c(0,cums1[-n])-cums2)*diff(c(0,cums1)))

ccoef1 <- c1$coef
ccoef2 <- c2$coef
###

bcifs <- apply(exp(-rbind(0,bcums1[-n,])-bcums2)*apply(rbind(0,bcums1),2,diff),2,cumsum)

if (gplot==1) {
   matplot(times1,bcifs,type="s",lwd=0.2)
   lines(times1,cif1,type="s",lwd=2)
}

    cumx <- cif1
    ccums <- bcifs-cif1
    sdcumb <- apply(ccums,1,sd)
    zcums   <- ccums/sdcumb

    ### log-scale 
    lcum <- log(cif1)
    lccums <- log(bcifs)-lcum
    sdlogcumb <- apply(lccums,1,sd)
    zlogcums   <- lccums/sdlogcumb
  
    cumx.inv <- 1/cumx
    # in order to not divide by 0
    cumx.inv[cumx.inv == 0] <- 1
    
    # In fact: here we use the same quantiles independent of log or not log. 
    # Therefore: A division by cumx is required in the definition of the log bands.
    pcumsdb.EE <- timereg::percen(c(apply(abs(zcums),2,max, na.rm=TRUE)),0.95)
    pcumsdb.EE.log <- timereg::percen(apply(abs(zcums),2,max, na.rm=TRUE),0.95)
    pcumsdb.EE.log.o <- timereg::percen(apply(abs(zlogcums),2,max, na.rm=TRUE),0.95)

    band.EE <-     cbind( cumx - sdcumb * pcumsdb.EE , cumx + sdcumb * pcumsdb.EE)
    band.EE.log <- cbind( cumx*exp(- pcumsdb.EE.log * sdcumb * cumx.inv)  ,cumx*exp( pcumsdb.EE.log * sdcumb * cumx.inv ))
    band.EE.log.o <- cbind(exp(lcum - sdlogcumb* pcumsdb.EE.log.o), 
			   exp(lcum + sdlogcumb* pcumsdb.EE.log.o))

 return(list(time=times1,cif=cif1,
	     sdcif=sdcumb,sdlogcif=sdlogcumb,bcifs=bcifs,
	     band.EE=band.EE,band.EE.log=band.EE.log,
	     band.EE.log.o=band.EE.log.o))

}# }}}

##' Marginal mean estimation for recurrent events with a terminal event
##'
##' Estimates the marginal mean number of recurrent events over time in the
##' presence of a competing terminal event (e.g. death), using the nonparametric
##' estimator of Ghosh and Lin (2000). Two proportional hazards models are fitted
##' internally—one for the recurrent event rate and one for the terminal
##' event—and combined to form the estimator
##' \deqn{\mu(t) = \int_0^t S(u-)\,dR(u),}
##' where \eqn{S(u)} is the marginal survival probability at the baseline covariate
##' level and \eqn{dR(u)} is the baseline recurrent event rate among survivors.
##' Robust (sandwich) standard errors are computed via the influence-function
##' approach of Ghosh and Lin (2000).
##'
##' Jump times must be unique within each stratum. If ties are present, use
##' \code{\link{tie_breaker}} to resolve them before calling this function.
##'
##' @param formula A formula with an \code{\link{Event}} response on the left-hand
##'   side, specifying entry time, exit time, and event status. The right-hand side
##'   may include \code{cluster()} to identify subjects and \code{strata()} for a
##'   stratified analysis. A \code{cluster()} term is required.
##' @param data A data frame containing all variables in \code{formula}.
##' @param cause Integer code(s) for the recurrent event of interest. Default is
##'   \code{1}.
##' @param death.code Integer code(s) for the terminal event. Default is \code{2}.
##' @param test Logical. If \code{TRUE}, a logrank-type test comparing strata is
##'   computed and stored as an attribute of the result. Default is \code{FALSE}.
##' @param ... Further arguments passed to \code{\link{phreg}}.
##'
##' @return An object of class \code{"recurrent"} with the following components:
##'   \item{mu}{Estimated marginal mean \eqn{\mu(t)} at each jump time.}
##'   \item{se.mu}{Robust standard error of \code{mu}.}
##'   \item{times}{Jump times at which estimates are computed.}
##'   \item{St}{Marginal survival estimate \eqn{S(t)} at each jump time.}
##'   \item{cumhaz}{Two-column matrix of \code{(time, mu)}, suitable for plotting.}
##'   \item{se.cumhaz}{Two-column matrix of \code{(time, se.mu)}.}
##'   The object carries three attributes: \code{"logrank"} (the test result when
##'   \code{test = TRUE}, otherwise \code{NULL}), \code{"cause"}, and
##'   \code{"death.code"}.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{test_logrankRecurrent}}, \code{\link{tie_breaker}},
##'   \code{\link{prob_exceed_recurrent}}
##'
##' @references
##' Cook, R. J. and Lawless, J. F. (1997). Marginal analysis of recurrent events
##' and a terminating event. \emph{Statistics in Medicine}, 16, 911--924.
##'
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
##' and death. \emph{Biometrics}, 56, 554--562.
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' hf$x <- as.numeric(hf$treatment)
##'
##' ## Fit nonparametric baseline models for recurrent events and death
##' xr <- phreg(Surv(entry, time, status == 1) ~ cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ cluster(id), data = hf)
##'
##' par(mfrow = c(1, 3))
##' plot(dr, se = TRUE); title(main = "Death")
##' plot(xr, se = TRUE); title(main = "Recurrent events")
##'
##' ## Compare naive and robust standard errors for the recurrent event rate
##' rxr <- robust_phreg(xr, fixbeta = 1)
##' plot(rxr, se = TRUE, robust = TRUE, add = TRUE, col = 4)
##'
##' ## Marginal mean via formula interface
##' outN <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
##'                            data = hf, cause = 1, death.code = 2)
##' plot(outN, se = TRUE, col = 2, add = TRUE)
##' summary(outN, times = 1:5)
##'
##' ## Stratified analysis with logrank test
##' out <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
##'                           data = hf, cause = 1, death.code = 2, test = TRUE)
##' plot(out, se = TRUE, ylab = "Marginal mean", col = 1:2)
##' attr(out, "logrank")
##' summary(out, times = 1:5)
##'
##' ## Influence-function (iid) decomposition at a fixed time point
##' head(iid(outN, time = 3))
##'
##' @aliases recurrent_marginalAIPCW recurrentMarginalPhreg iidRecurrent
##' @export
recurrent_marginal <- function(formula,data,cause=1,...,death.code=2,test=FALSE)
{# {{{
  if (missing(formula)) { # Fall-back to recurrentMarginalPhreg for backward compatibility <= 1.3.5
    return(recurrentMarginalPhreg(...))
  }
  if (inherits(formula,"phreg")) { # Fall-back to recurrentMarginalPhreg
    if (inherits(data, "phreg")) {
        return(recurrentMarginalPhreg(formula, data, ...))
    } else {
        stop("You can give 2 phreg objects \n")
    }
  }

  ## {{{
  cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    specials = c("offset", "weights", "cluster","strata")
    Terms <- terms(formula, specials, data = data)
    ts <- survival::untangle.specials(Terms, "strata")
    if (!is.null(strata)) strata.name <- ts$vars else strata.name <- NULL
###    if (!is.null(strata)) strata.name <- names(des$xlevels) 
###    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of 
    pos.cluster <- NULL
 ## }}}
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

 call.id <- id;
 conid <- construct_id(id,nrow(X),as.data=TRUE)
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid

  statusE <- 1*(status %in% cause)
###  if (is.null(death.code)) statusD <- 1*(!(status %in% cens.code)) else 
  statusD <- 1*(status %in% death.code)
  data$statusE__ <- statusE
  data$statusD__ <- statusD
  data$strata__  <- strata

  if (is.null(call.id))  stop("must specify 'cluster()'\n"); 
  ### setting up formulae for the two phreg (cause of interest and death)
###  if (is.null(call.id)) { 
###     stop("must give id\n"); 
###     formid <- update.formula(formula,~.+cluster(id)) 
###     data$id <- id
###     tt <- terms(formid)
###     tt <- delete.response(tt)
###     formid <- formula(tt)
###  }  else 
  formid <- formula
  tt <- terms(formid)
  tt <- delete.response(tt)
  formid <- formula(tt)

  data$exit__ <- exit
  if (ncol(Y)==3) {
     data$entry__ <- entry
     formE <- as.formula(paste("Surv(entry__,exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(entry__,exit__,statusD__)~+1"))
  } else {
     formE <- as.formula(paste("Surv(exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(exit__,statusD__)~+1"))
  }
 formE <- update.formula(formE,formid)
 formD <- update.formula(formD,formid)
 ## drop marks from terminal event formula
 formD <- drop.specials(formD,"marks")

  if (sum(statusE)==0) warning("No events of type 1\n"); 
  coxE <- phreg(formE,data=data,...)
  coxS <- phreg(formD,data=data,...)

  ### cif 
  if (sum(statusE)>0) meano <- recurrentMarginalPhreg(coxE,coxS) else meano <- coxE
  ## to work with predict function
  meano$no.opt <- TRUE

  if (test) {
	  logrank <- logrankRecurrentBase(coxE,coxS)
  } else logrank <- NULL

  attr(meano,"logrank") <- logrank
  attr(meano,"cause") <- cause
  attr(meano,"death.code") <- death.code
  return(meano)
}# }}}

##' @rdname recurrent_marginal
##' @export
recurrentMarginal <- function(formula,data,...) recurrent_marginal(formula,data,...)

##' @export
recurrentMarginalPhreg <- function(recurrent,death,fixbeta=NULL,km=TRUE)
{# {{{
  xr <- recurrent
  dr <- death 

  ### sets fixbeta based on whether xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) 
  if ((xr$no.opt) | is.null(xr$coef)) fixbeta<- 1 else fixbeta <- 0

  ### marginal expected events  int_0^t G(s) \lambda_r(s) ds 
  # {{{
  strat <- dr$strata[dr$jumps]
  ###
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## survival at t- to also work in competing risks situation
  if (!km) { 
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  x <- xr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  cumhazR <-  cbind(xx$time,cumsumstrata(S0i,xx$strata,xx$nstrata))
  cumhazDR <- cbind(xx$time,cumsumstrata(St*S0i,xx$strata,xx$nstrata))
  mu <- cumhazDR[,2]
# }}}

  ### robust standard errors 
  ### 1. sum_k ( int_0^t S(s)/S_0^r(s) dM_k.^r(s) )^2
 resIM1 <-  squareintHdM(xr,ft=St,fixbeta=fixbeta)
 ### 2. mu(t)^2 * sum_k ( int_0^t 1/S_0^d(s) dM_k.^d(s) )^2
 resIM2 <-  squareintHdM(dr,ft=NULL,fixbeta=fixbeta)
  ### 3. sum_k( int_0^t mu(s) /S_0^d(s) dM_k.^d(s))^2
 resIM3 <-  squareintHdM(dr,ft=mu,fixbeta=fixbeta)

 varA <-  resIM1$varInt+mu^2*resIM2$varInt+resIM3$varInt 

## covariances between different terms  13 23  12 12
## to allow different strata for xr and dr, but still nested strata
 if ((xr$nstrata>1 & dr$nstrata==1)) {
    cM1M3 <- covIntH1dM1IntH2dM2(resIM1,resIM3,fixbeta=fixbeta,mu=NULL)
    cM1M2 <- covIntH1dM1IntH2dM2(resIM1,resIM2,fixbeta=fixbeta,mu=mu)
 } else  {
    cM1M3 <- covIntH1dM1IntH2dM2(resIM3,resIM1,fixbeta=fixbeta,mu=NULL)
    cM1M2 <- covIntH1dM1IntH2dM2(resIM2,resIM1,fixbeta=fixbeta,mu=mu)
 }
 cM2M3 <- covIntH1dM1IntH2dM2(resIM2,resIM3,fixbeta=fixbeta,mu=mu)

 varA <- varA+2*cM1M3$cov12A-2*cM1M2$cov12A-2*cM2M3$cov12A 
### varA <- varA-2*cM1M3$cov12A+2*cM1M2$cov12A+2*cM2M3$cov12A 

 cov12aa <- cov13aa <- cov23aa <- 0

 if (fixbeta==0) {
    varA <-varA + cM2M3$covbeta - cM1M3$covbeta + cM1M2$covbeta 
 }

 varrs <- data.frame(mu=mu,cumhaz=mu,se.mu=varA^.5,time=xr$time,
		     se.cumhaz=varA^.5,strata=xr$strata,St=St)
 varrs <- varrs[c(xr$cox.prep$jumps)+1,]

 ### to use basehazplot.phreg
 ### making output such that basehazplot can work also
 out <- list(mu=varrs$mu,se.mu=varrs$se.mu,times=varrs$time,
     St=varrs$St,
     cumhaz=cbind(varrs$time,varrs$mu),se.cumhaz=cbind(varrs$time,varrs$se.mu),
     strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),
     strata.name=xr$strata.name,strata.level=recurrent$strata.level,call=xr$call,
     formula=xr$formula,no.opt=TRUE,exit=varrs$time,p=0)
 attr(out,"recurrent") <-  recurrent
 attr(out,"death") <- death
 class(out) <- rep("recurrent",2)
 return(out)
}# }}}

##' @export
plot.recurrent <- function(x,ylab=NULL,test=TRUE,...) {# {{{
 if (inherits(x,"recurrent") & is.null(ylab)) ylab <- "Mean events"
 baseplot(x,ylab=ylab,...)
 logrank <- attr(x,"logrank")
 if (!is.null(logrank) & test)
 legend("bottomright",paste("Logrank test, p=",round(logrank$compare$p.value,3)))
}# }}}

##' @export
summary.recurrent <- function(object,...) {# {{{
out <- summaryRecurrentobject(object,...)
return(out)
}# }}}

summaryRecurrentobject <- function(object,times=NULL,strata=NULL,estimates=FALSE,
			      name="mean",cumhaz="cumhaz",se.cumhaz="se.cumhaz",robust=FALSE,
	      conf.type=c("log","log-log","plain"),...) {# {{{
base <- basecumhaz(object,joint=1,robust=robust,cumhaz=cumhaz,se.cumhaz=se.cumhaz)
nstrata <- object$nstrata
stratobs <- attr(base,"stratobs")

baseci <- rep(list(NULL),nstrata)
if (length(stratobs)>0) 
 for (i in stratobs) {
   cumhaz <- base[[i+1]]$cumhaz
   if (nrow(cumhaz)>=1) {
     mu <- base[[i+1]]$cumhaz[,2]
     se.mu <- base[[i+1]]$cumhaz[,3]
	   if (length(mu)>=1) {
	   conf <- conftype(mu,se.mu,conf.type=conf.type[1],...)
	   out <- data.frame(times=cumhaz[,1],mu=mu,se.mu=se.mu,lower=conf$lower,upper=conf$upper,strata=i)
	   names(out) <- c("times",name,"se","CI-2.5%","CI-97.5%","strata")
	   baseci[[i+1]] <- out
	   } 
   } 
   }

i <- stratobs
pbaseci <- NULL
if (!is.null(times)) {
 if (length(stratobs)>0) 
 for (i in stratobs) 
if (!is.null(baseci[[i+1]])) pbaseci[[i+1]] <- predictCumhaz(rbind(0,baseci[[i+1]]),times) 
}
out <- list(baseci=baseci,pbaseci=pbaseci,times=times)

class(out) <- "summary.recurrent"
return(out)
}# }}}

##' @export
print.summary.recurrent  <- function(x,...) {# {{{
if (is.null(x$times)) print(x$baseci) else print(x$pbaseci)
} # }}}

iidRecurrent <- function(recurrent,death,wt=NULL,km=TRUE,start=0,time=NULL) { ## {{{ 
xr <- recurrent
dr <- death 
if (is.null(time)) time <- max(xr$jumptimes)
### marginal expected events  int_0^t w(s) S(s) \lambda_r(s) ds 
### weight is considered known 
## {{{
###
x <- dr
xx <- x$cox.prep
xstrata <- xx$strata
nstrata <- xx$nstrata

S0i2 <- S0i <- rep(0,length(xstrata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2
## survival at t- to also work in competing risks situation
if (!km) { 
cumhazD <- c(cumsumstratasum(S0i,xstrata,nstrata)$lagsum)
St      <- exp(-cumhazD)
} else St <- c(exp(cumsumstratasum(log(1-S0i),xstrata,nstrata)$lagsum))
x <- xr
xx <- x$cox.prep
S0i2 <- S0i <- rep(0,length(xstrata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2

btime <- (xx$time<=time  & xx$time>start)*1
wt.call <- wt
if (is.null(wt)) wt <- rep(1,length(xx$time))

cumhazR <-  cbind(xx$time,cumsumstrata(S0i*btime,xstrata,nstrata))
meanN <- cumhazDR <- cbind(xx$time,cumsumstrata(wt*St*S0i*btime,xstrata,nstrata))
mu <- cumhazDR[,2]
meanN <- meanN[xx$jumps+1,]

cumhazSR <- cumsumstrata(wt*St*S0i2*btime, xstrata, nstrata)
rr <- c(xx$sign * exp( xx$offset))
MSGt <- wt*St*S0i*btime - cumhazSR*rr*c(xx$weights)
id <- xx$id
MSGtiid <- apply(MSGt,2,sumstrata,id,max(id)+1)

x <- dr
xx <- x$cox.prep
rr <- c(xx$sign * exp( xx$offset))
S0i2 <- S0i <- rep(0,length(xx$strata))
S0i[xx$jumps+1] <-  1/x$S0
S0i2[xx$jumps+1] <- 1/x$S0^2

cumhazD <-  cumsumstrata(S0i2*btime,xstrata,nstrata)
cumhazmuD <-  cumsumstrata(mu*S0i2*btime,xstrata,nstrata)
tails <- tailstrata(xstrata,nstrata)
mut <- mu[tails][xstrata+1]

mutMDt <- mut*S0i*btime-mut*cumhazD*rr*c(xx$weights)
mutMDtiid <- apply(mutMDt,2,sumstrata,id,max(id)+1)

MmuDt <- mu*S0i*btime-cumhazmuD*rr*c(xx$weights)
MmuDtiid <- apply(MmuDt,2,sumstrata,id,max(id)+1)

MGAiid <- MSGtiid-mutMDtiid+MmuDtiid
mid <- max(id)+1

strataid <- xstrata[headstrata(xx$id,mid)]
id <- 1:nrow(MGAiid)

MGAiids <- matrix(0,nrow(MGAiid),nstrata)
cumhaz.time <- c()
sus <- sort(unique(strataid))
for (i in sus)  { 
 wi <- which(strataid==i)
 MGAiids[wi,i+1] <- MGAiid[wi,]
}
MGAiid <- MGAiids
if (is.matrix(MGAiid)) {
colnames(MGAiid) <- paste("strata",sus,sep="")
if (!is.null(x$call.id)) MGAiid <- nameme(MGAiid,x$name.id) 
}

var.iid <- crossprod(MGAiid)
se <- diag(var.iid)^.5
## }}} 

out <- list(iid=MGAiid,wt=wt.call,time=time,start=start,meanN=meanN,var=var.iid,se=diag(var.iid)^.5,mut=mu[tails])
return(out)
} ## }}}

##' @export
estimate.recurrent <- function(x,time=NULL,...) { ## {{{ 
   if (is.null(time)) stop("must give time for iid decomposition \n")
   ic <- IC(x,time=time)
   out <- estimate(coef=attr(ic,"coefs"),IC=ic,...)
return(out)
} ## }}}

##' @export
iid.recurrent <- function(x,...) { ## {{{ 
iid <- iidRecurrent(attr(x,"recurrent"),attr(x,"death"),...)
out <- iid$iid
attr(out,"coefs") <- iid$mut 
attr(out,"time") <-  iid$time
return(out)
} ## }}}

##' @export
IC.recurrent <- function(x,...) { ## {{{ 
iid <- iid(x,...)
ic <- iid*nrow(iid)
return(ic)
} ## }}}

##' Logrank-type test for comparing recurrent event marginal means between groups
##'
##' Tests whether the marginal mean number of recurrent events differs across
##' groups (strata), extending the classical logrank test to the setting of
##' recurrent events with a competing terminal event. The test statistic is
##' \deqn{z = \int_0^\tau w(s)\bigl[d\hat\mu_1(s) - d\hat\mu_2(s)\bigr],}
##' where \eqn{w(s)} is a weight function and \eqn{\hat\mu_j(s)} is the estimated
##' marginal mean for group \eqn{j}. Variance is estimated robustly via the
##' influence functions of Ghosh and Lin (2000).
##'
##' Three weight schemes are available:
##' \describe{
##'   \item{\code{"I"}}{(Default) \eqn{w(t) = R_1(t) R_2(t) / (R_1(t) + R_2(t))},
##'     where \eqn{R_j(t) = Y_j(t) / \hat S_j(t-)}. Analogous to the standard
##'     logrank weight.}
##'   \item{\code{"II"}}{\eqn{w(t) = Y_j(t)}, the raw risk-set size. Equivalent
##'     to using observed counts without survival adjustment.}
##'   \item{\code{"III"}}{A modified weight incorporating the cumulative incidence,
##'     analogous to Gray's test for competing risks.}
##' }
##'
##' @param recurrent Either a \code{"recurrent"} object returned by
##'   \code{\link{recurrent_marginal}}, or a \code{"phreg"} object for the
##'   recurrent event model (in which case \code{death} must also be supplied).
##' @param death A \code{"phreg"} object for the terminal event model. Required
##'   when \code{recurrent} is a \code{"phreg"} object; ignored otherwise.
##' @param weight Character string specifying the weight scheme: \code{"I"},
##'   \code{"II"}, or \code{"III"}. Default is \code{"I"}. For competing risks \code{"III"} 
##'    gives Gray's test with robust standard errors.
##' @param km Logical. If \code{TRUE} (default), the Kaplan-Meier estimator is
##'   used for the survival probability \eqn{S(t)}; otherwise the Nelson-Aalen
##'   estimator is used.
##' @param start Left truncation time for the integration. Default is \code{0}.
##' @param stop Right truncation time for the integration. Defaults to the last
##'   observed jump time.
##' @param at.risk Minimum combined risk-set size below which the weight is set
##'   to zero. Default is \code{5}.
##' @param cluster.id Optional vector of cluster identifiers for aggregating
##'   influence functions across clusters before forming the test statistic.
##' @param ... Currently unused.
##'
##' @return An object of class \code{"estimate"} (from the \pkg{lava} package)
##'   with the following components:
##'   \item{coef}{The weighted difference in marginal means between groups.}
##'   \item{se}{Robust standard error of the test statistic.}
##'   \item{lower, upper}{95\% confidence interval bounds.}
##'   \item{p.value}{Two-sided p-value for the null hypothesis of no difference.}
##'   The object also carries an \code{iid} attribute containing the
##'   subject-level influence function decomposition of the test statistic,
##'   which can be used for further inference or combination with other estimators.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{logrankRecurrentBase}}
##'
##' @references
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
##' and death. \emph{Biometrics}, 56, 554--562.
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##'
##' ## Test using two separate phreg models
##' xr <- phreg(Surv(entry, time, status == 1) ~ strata(treatment) + cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ strata(treatment) + cluster(id), data = hf)
##' out <- test_logrankRecurrent(xr, dr, stop = 5)
##' summary(out)
##'
##' ## Equivalently, using a recurrent_marginal object directly
##' outN <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
##'                            data = hf, cause = 1, death.code = 2)
##' test_logrankRecurrent(outN)
##'
##' @aliases logrankRecurrentBase
##' @export
test_logrankRecurrent <- function(recurrent,death,
              weight=c("I","II","III"),km=TRUE,start=0,stop=NULL,at.risk=5,cluster.id=NULL,...) { ## {{{ 
  if (inherits(recurrent,"phreg")) { # Fall-back to recurrentMarginalPhreg
    if (inherits(death, "phreg")) {
        return(logrankRecurrentBase(recurrent,death,
		weight=weight[1],km=km,start=start,stop=stop,at.risk=at.risk,
		cluster.id=cluster.id))
     } 
     return(NULL)
  } else if (inherits(recurrent,"recurrent")) { # Fall-back to recurrentMarginalPhreg
        return(logrankRecurrentBase(attr(recurrent,"recurrent"),attr(recurrent,"death"),
		weight=weight[1],km=km,start=start,stop=stop,at.risk=at.risk,cluster.id=cluster.id))
 } else stop("input either output from recurrentMarginal or two phreg's\n")

} ## }}}

##' @export
logrankRecurrentBase <- function(recurrent,death,weight=c("I","II","III"),km=TRUE,
       start=0,stop=NULL,at.risk=0,cluster.id=NULL) { ## {{{ 
  xr <- recurrent
  dr <- death 
  if (is.null(stop)) stop <- max(xr$jumptimes)

  nlev <- xr$nstrata
  ###
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  n <- dr$n

  Yrr <- revcumsumstrata(xx$sign,xx$strata,xx$nstrata)
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/Yrr[xx$jumps+1]

  Sta <- cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)
  St <- c(exp(Sta$lagsum))
  kmss <- exp(Sta$alllagsum)
  Yss <- .Call("_mets_riskstrataR",xx$sign,xx$strata,xx$nstrata)$risk

  if (weight[1]=="I") {
	  Rss <-  Yss/kmss
	  Rss[Yss==0] <- 0 
  }
  if (weight[1]=="III") {
     xx <- xr$cox.prep
     S0ri <- rep(0,length(xx$strata))
     S0ri[xx$jumps+1] <-  1/Yrr[xx$jumps+1]
     cifsm <- cumsumstratasum(St*S0ri,xx$strata,xx$nstrata)
     F1c <-  1-cifsm$alllagsum 
     Rss <-  F1c*Yss/kmss
     Rss[Yss==0] <- 0 
     F1ci <- rep(0,length(xx$strata))
     F1ci[xx$jumps+1] <-  1/(1-cifsm$lagsum[xx$jumps+1])
  }
  if (weight[1]=="II") {
     Rss <- Yss 
  }
  Rp <- apply(Rss,1,sum,na.rm=TRUE)
  Rssstrat <- mdi(Rss,1:length(xx$strata),xx$strata+1)

  contr <- contr.iid <- c()
  i <- 1
  for (i in 1:(nlev-1)) {
     wt <- rep(0,length(xx$strata))
     strati  <- which(xx$strata==(i-1))
     wt <- Rss[,i]*Rssstrat/Rp
     wt[strati] <- Rss[strati,i]*(Rp[strati]-Rss[strati,i])/Rp[strati]
     wt[is.na(wt)] <- 0
      if (at.risk>0) {
        minrisk <- pmin(Yss[,i],apply(Yss[,-i,drop=FALSE],1,sum))
        wt[minrisk<=at.risk] <- 0
      }
     ## adjust weight to get Grays test with [1-F_1(s-)]^-1
     if (weight[1]=="III") wt <- wt*F1ci

      lrt <- iidRecurrent(xr,dr,wt=wt)
      diff(lrt$mut)
      if (!is.null(cluster.id)) {
	      nid <- length(unique(cluster.id))
	      conid <- construct_id(cluster.id,nid)
	      lrt$iid <- apply(lrt$iid,2,sumstrata,conid$id,conid$nid)
      }
      diffi <- lrt$mut[i]-sum(lrt$mut[-i])
      contr <- c(contr,diffi)
      contr.iid <- cbind(contr.iid,lrt$iid[,i]-apply(lrt$iid[,-i,drop=FALSE],1,sum))
   }
   logrank <- estimate(coef=contr,IC=contr.iid*nrow(contr.iid),null=0)
   p <- length(contr)

   return(logrank)
} ## }}} 

##' @export
recurrent_marginalAIPCW <- function(formula,data=NULL,cause=1,cens.code=0,death.code=2,
	  cens.model=~1,km=TRUE,times=NULL,augment.model=~Nt,...)
{# {{{
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    id <- strata <- NULL
    if (!is.null(attributes(Terms)$specials$cluster)) {
        ts <- survival::untangle.specials(Terms, "cluster")
        pos.cluster <- ts$terms
        Terms  <- Terms[-ts$terms]
        id <- m[[ts$vars]]
    } else pos.cluster <- NULL
    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
        ts <- survival::untangle.specials(Terms, "strata")
        pos.strata <- ts$terms
        Terms  <- Terms[-ts$terms]
        strata <- m[[ts$vars]]
        strata.name <- ts$vars
    }  else { strata.name <- NULL; pos.strata <- NULL}
    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
        ts <- survival::untangle.specials(Terms, "offset")
        Terms  <- Terms[-ts$terms]
        offset <- m[[ts$vars]]
    }
    X <- model.matrix(Terms, m)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    ## }}}

   if (!is.null(id)) {
        ids <- unique(id)
        nid <- length(ids)
        if (is.numeric(id))
            id <-  fast.approx(ids,id)-1
        else  {
            id <- as.integer(factor(id,labels=seq(nid)))-1
        }
    } else { id <- as.integer(seq_along(entry))-1;  nid <- nrow(X); }
    ## orginal id coding into integers 1:...
    id.orig <- id+1;

   ### setting up with artificial names
 data$status__ <-  status 
 data$id__ <-  id
 ## lave Countcause
 data <- count_history(data,status="status__",id="id__",types=cause,multitype=TRUE)
 data$Count1__ <- data[,paste("Count",cause[1],sep="")]
 data$death__ <- (status %in% death.code)*1
 data$entry__ <- entry 
 data$exit__ <- exit 
 data$statusC__ <- (status %in% cens.code)*1
 data$status__cause <- (status %in% cause)*1

  xr <- phreg(Surv(entry__,exit__,status__cause)~Count1__+death__+cluster(id__),data=data,no.opt=TRUE,no.var=1)

  formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
  cr <- phreg(formC,data=data)
  whereC <- which(status %in% cens.code)

  if (length(whereC)>0) {# {{{
  ### censoring weights
  strat <- cr$strata[cr$jumps]
  x <- cr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## survival at t- to also work in competing risks situation
  if (!km) { 
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  } else  St <- rep(1,length(xx$strata))
  Gc <- St
  ## }}}

  
formD <- as.formula(Surv(entry__,exit__,death__)~cluster(id__))
form1L <- as.formula(Surv(entry__,exit__,status__cause)~Count1__+death__+statusC__+cluster(id__))
form1 <- as.formula(Surv(entry__,exit__,status__cause)~cluster(id__))

 xr <- phreg(form1L,data=data,no.opt=TRUE,no.var=1)
 dr <- phreg(formD,data=data,no.opt=TRUE,no.var=1)

 ### augmenting partioned estimator computing \hat H_i(s,t) for fixed t
 data$Gctrr <- exp(-cpred(rbind(0,cr$cumhaz),exit)[,2])

 ### cook-lawless-ghosh-lin
 xr0 <- phreg(form1,data=data,no.opt=TRUE)
 clgl  <- recurrentMarginalPhreg(xr0,dr)
### bplot(clgl,se=1); print(cpred(clgl$cumhaz,times)); print(cpred(clgl$se.cumhaz,times)); 

  ####  First \mu_ipcw(t) \sum_i I(T_i /\ t \leq C_i)/G_c(T_i /\ t ) N_(T_i /\ t) {{{
  x <- xr
  xx <- xr$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  ## stay with N(D_i) when t is large so no -1 when death
  ## xx$X[,1] er Count1 dvs N(t-)
  Nt <- revcumsumstrata(xx$X[,1]*xx$sign,xx$strata,xx$nstrata)
  Nt <- Nt/Gc
  ## counting N(D) forward in time skal ikke checke ud når man dør N_(D_i) er i spil efter D_i
  NtD <- cumsumstrata(xx$X[,1]*(xx$X[,2]==1)*(xx$sign==1)/Gc,xx$strata,xx$nstrata)
  jump1 <- xx$jumps+1
  timeJ <- xx$time[jump1]
  avNtD <- (NtD+Nt)[jump1]/nid
  strataN1J <- xx$strata[jump1]

  varIPCW1 <- NULL
  seIPCW1 <- NULL

  ### IPCW estimator 
  cumhaz <- cbind(timeJ,avNtD)
# }}}

  ### Partitioned estimator , same as Lin, Lawless & Cook estimator {{{
  cumhazP <- c(cumsumstrata(1/Gc[jump1],strataN1J,xx$nstrata)/nid)
  cumhazP <- cbind(timeJ,cumhazP)

  ### variance of partitioned estimator 

  ### calculate E(H,s,t) = E(H,t) - E(H,s) 
  ### E(H,t) = 1/(S(t)*n) \sum_ \int Y_i(s)/G_c(s) dN_{1i} = 1/(S(t)) (\mu_p(t) - \mu_p(s)) 
  ### \hat H_i for all subjects, and look at together with Hst, eHst 
  ### for censoring martingale 
  data$Nt <- data$Count1__
  data$Nt2 <- data$Nt^2
  data$expNt <- exp(-data$Nt)
  data$NtexpNt <- data$Nt*exp(-data$Nt)

  Gcdata <- exp(-cpred(rbind(0,cr$cumhaz),exit)[,2])

  form <- as.formula(paste("Surv(entry__,exit__,statusC__)~+1"))
  desform <- update.formula(augment.model,~Hst + . + cluster(id__))
  form[[3]] <- desform[[2]]
  nterms <- length(all.vars(form[[3]]))-1

  if (!is.null(times)) {
       semuPA <-  muPA <- semuPA.times <-  muPA.times <- rep(0,length(times))
       ww <- fast.approx(timeJ,times,type="left")
       muP.times <- cumhazP[ww,2]
       semuP.times <- clgl$se.mu[ww]

  for (i in seq_along(times)) {
     timel <- times[i]
     data$Hst <- revcumsumstrata((exit<timel)*(status %in% cause)/Gcdata,id,nid)
     cr2 <- phreg(form,data=data,no.opt=TRUE,no.var=1)
     nterms <- cr2$p-1

     dhessian <- cr2$hessianttime
     dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
     ###  matrix(apply(dhessian,2,sum),3,3)
     timeb <- which(cr$cumhaz[,1]<timel)
     ### take relevant \sum H_i(s,t) (e_i - \bar e)
     covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
     ### construct relevant \sum (e_i - \bar e)^2
     Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
     ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
     gammahat <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
     S0 <- cr$S0[timeb]
     gammahat[is.na(gammahat)] <- 0
     gammahat[gammahat==Inf] <- 0
     Gctb <- Gc[cr$cox.prep$jumps+1][timeb]
     augment.times <- sum(apply(gammahat*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum))/nid
     mterms <- length(terms)
     mterms <- nterms
     ###
     varZ <- matrix(apply(Pt/Gctb^2,2,sum),mterms,mterms)
     gamma <- .Call("CubeVec",matrix(c(varZ),nrow=1),matrix(apply(covts/Gctb,2,sum),nrow=1),1,PACKAGE="mets")$XXbeta
     gamma <- c(gamma)
     gamma[is.na(gamma)] <- 0
     gamma[gamma==Inf] <- 0
     augment <- sum(apply(gamma*t(cr2$U[timeb,1+1:nterms,drop=FALSE])/Gctb,2,sum))/nid
     ###
     muPA[i] <- muP.times[i]+augment
     semuPA[i] <- (semuP.times[i]^2 +(gamma %*% varZ %*% gamma)/nid^2)^.5
     muPA.times[i] <- muP.times[i]+augment.times
     semuPA.times[i] <- (semuP.times[i]^2+sum(gammahat*.Call("CubeVec",Pt,gammahat,0,PACKAGE="mets")$XXbeta)/(nid^2))^.5
  }
  } else { Gctb <- NULL;gammahat <- NULL; 
  muPA.times <-semuPA.times <- muPA  <- semuPA <- NULL }
# }}}

  return(list(censoring.weights=Gctb,muP.all=cumhazP,Gcjump=Gc[jump1],gamma=gamma,gamma.time=gammahat,times=times,
  muP=muP.times,semuP=semuP.times, muPAt=muPA.times,semuPAt=semuPA.times, muPA=muPA,semuPA=semuPA))
}# }}}

##' Summarize a Time-Varying Estimate with Confidence Bands
##'
##' Extracts estimates at specified time points with confidence intervals.
##'
##' @param mutimes vector of estimation time points.
##' @param mu vector or matrix of estimates.
##' @param se.mu vector or matrix of standard errors (optional).
##' @param times time points at which to evaluate (default: \code{mutimes}).
##' @param type confidence interval type: \code{"log"} or \code{"plain"}.
##' @param level confidence level (default 0.95).
##' @param ... additional arguments.
##' @return A data.frame with columns: times, mean, se-mean, CI-2.5\%, CI-97.5\%.
##' @export
summaryTimeobject <-function(mutimes,mu,se.mu=NULL,times=NULL,type="log",level=0.95,...) {# {{{
 if (is.null(times)) times <- mutimes

 where <- fast.approx(c(0,mutimes),times,type="left")

 crit <- qnorm(1-(1-level)/2)
 ##  see if object is vector or matrix
 if (is.matrix(mu)) mu <- rbind(0,mu)[where,] else mu <- c(0,mu)[where]
 if (!is.null(se.mu)) {
     if (is.matrix(se.mu)) se.mu <- rbind(0,se.mu)[where,] else se.mu <- c(0,se.mu)[where]
 se.logmu=se.mu/mu
 if (type=="log") {
 lower <- exp(log(mu) - crit*se.logmu)
 upper <- exp(log(mu) + crit*se.logmu)
 } else {
 lower <- mu - crit*se.mu
 upper <- mu + crit*se.mu
 }
 } else {se.mu <- se.logmu <- lower <- upper <- NA}

 out <- data.frame(times=times,mu=mu,se.mu=se.mu,lower=lower,upper=upper)
 names(out) <- c("times","mean","se-mean","CI-2.5%","CI-97.5%")
 return(out)
}# }}}

recmarg <- function(recurrent,death,Xr=NULL,Xd=NULL,km=TRUE,...)
{# {{{
  xr <- recurrent
  dr <- death 

  if (!is.null(Xr)) rr <- exp(sum(xr$coef * Xr)) else rr <- 1
  if (!is.null(Xd)) rrd <- exp(sum(dr$coef * Xd)) else rrd <- 1

  ### marginal expected events  int_0^t G(s) \lambda_r(s) ds 
  # {{{
  strat <- dr$strata[dr$jumps]
  x <- dr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  if (!km) { 
     cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
     St      <- exp(-cumhazD*rrd)
  } else St <- exp(rrd*c(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  ###
  x <- xr
  xx <- x$cox.prep
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/x$S0^2
  cumhazDR <- cbind(xx$time,cumsumstrata(St*S0i,xx$strata,xx$nstrata))
  mu <- rr*cumhazDR[,2]
# }}}

 varrs <- data.frame(mu=mu,time=xr$time,strata=xr$strata,St=St)
 varrs  <-  varrs[c(xr$cox.prep$jumps)+1,]

 ### to use basehazplot.phreg
 ### making output such that basehazplot can work also
 out <- list(mu=varrs$mu,time=varrs$time,
	     St=varrs$St,cumhaz=cbind(varrs$time,varrs$mu),
             strata=varrs$strata,nstrata=xr$nstrata,jumps=1:nrow(varrs),
	     strata.name=xr$strata.name,strata.level=recurrent$strata.level)
 return(out)
}# }}}

squareintHdM <- function(phreg,ft=NULL,fixbeta=NULL,beta.iid=NULL,...)
{# {{{
###  sum_k ( int_0^t f(s)/S_0^r(s) dM_k.^r(s) )^2
###  strata "r" from object and "k" id from cluster 
  if (!inherits(phreg,"phreg")) stop("Must be phreg object\n"); 

  ### sets fixbeta based on whether xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) 
  if ((phreg$no.opt) | is.null(phreg$coef)) fixbeta<- 1 else fixbeta <- 0

  x <- phreg
  xx <- x$cox.prep
  ww <- xx$caseweights*xx$weights
 
  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <-  1/x$S0
  S0i2[xx$jumps+1] <- 1/(x$S0^2*ww[xx$jumps+1])
  Z <- xx$X
  U <- E <- matrix(0,nrow(xx$X),x$p)
  E[xx$jumps+1,] <- x$E
  U[xx$jumps+1,] <- x$U
  cumhaz <- cbind(xx$time,cumsumstrata(S0i,xx$strata,xx$nstrata))
  if (is.null(ft))  ft <- rep(1,length(xx$time))
  cumS0i2 <- c(cumsumstrata(ft*S0i2,xx$strata,xx$nstrata))
  if (fixbeta==0) {
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Ht <- apply(ft*E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
  } else Ht <- NULL
  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
  else rr <- c(xx$sign*exp(xx$offset))
  id <-   xx$id
  mid <- max(id)+1
  ### also weights 
  w <- c(xx$weights)
  xxx <- (ft*S0i-rr*cumS0i2*w)
  ssf <- cumsumidstratasum(xxx,id,mid,xx$strata,xx$nstrata)$sumsquare
  ss <-  revcumsumidstratasum(w*rr,id,mid,xx$strata,xx$nstrata)$lagsumsquare*cumS0i2^2
  covv <- covfridstrata(xxx,w*rr,id,mid,xx$strata,xx$nstrata)$covs*cumS0i2
  varA1 <- c(ssf+ss-2*covv)

  vbeta <- betaiidR <- NULL
  if (fixbeta==0) {# {{{
    if (!is.null(beta.iid))  betaiidR <- beta.iid else {
	     invhess <- -pinv(x$hessian)
	     MGt <- U[,drop=FALSE]-(Z*cumhaz[,2]-EdLam0)*rr*c(xx$weights)
	     UU <- apply(MGt,2,sumstrata,id,mid)
	     betaiidR <- UU %*% invhess
     }
     vbeta <- crossprod(betaiidR)
     varbetat <-   rowSums((Ht %*% vbeta)*Ht)
     ### writing each beta for all individuals 
     betakt <- betaiidR[id+1,,drop=FALSE]
     covk1 <- apply(xxx*betakt,2,cumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="sum")
     covk2 <- apply(w*rr*betakt,2,revcumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="lagsum")
     covk2 <- covk2*cumS0i2
     covv <- covk1-covk2
     varA1 <- varA1+varbetat-2*apply(covv*Ht,1,sum)
  }# }}}

  return(list(xx=xx,Ht=Ht,varInt=varA1,xxx=xxx,rr=rr,
	      cumhaz=cumhaz,cumS0i2=cumS0i2,mid=mid,id=id,
	      betaiid=betaiidR,vbeta=vbeta,covv=covv))
} # }}}

covIntH1dM1IntH2dM2 <- function(square1,square2,fixbeta=1,mu=NULL)
{# {{{

 ### strata and id same for two objects 
 xx <- square1$xx; xx2 <- square2$xx
 xxxR <- square1$xxx;     xxxD1 <- square2$xxx
 rrR  <- square1$rr;       rrD1 <- square2$rr
 id   <- id1 <- square1$id; id2 <- square2$id
 mid  <- square1$mid; w <- c(xx$weights)

 if (is.null(mu)) mu <- rep(1,length(xx$strata))

 cov12 <-  c(cumsumidstratasumCov(xxxR,xxxD1,id,mid,xx$strata,xx$nstrata)$sumsquare)
 cov122 <- c(revcumsumidstratasumCov(w*rrR,w*rrD1,id,mid,xx$strata,xx$nstrata)$lagsumsquare)*c(square1$cumS0i2*square2$cumS0i2)
 cov123 <- covfridstrataCov(xxxR,w*rrR,xxxD1,w*rrD1,id,mid,xx$strata,xx$nstrata)$covs*c(square2$cumS0i2)
 cov124 <- covfridstrataCov(xxxD1,w*rrD1,xxxR,w*rrR,id,mid,xx$strata,xx$nstrata)$covs*c(square1$cumS0i2)
 cov12A <- c(cov12+cov122+cov123+cov124)
 
 test <- 0
 if (test==1) {# {{{
	 print("________cov cov ___________________________"); 
	 print(summary(c(xxxR))); print(summary(c(xxxD1))); 
	 print(summary(c(rrD1))); print(summary(c(rrR)))
	 print(summary(c(square1$cumS0i2))); print(summary(c(square2$cumS0i2)));
	 print("-----------"); 
	 print(summary(cov12)); 
	 print(summary(cov122)); 
	 print(summary(c(cov123))); 
	 print(summary(c(cov124))); 
	 print("______________________________________"); 
	 jumps <- c(square1$xx$jumps,square2$xx$jumps)+1
	 print(summary(jumps))
         print(summary(cov12[jumps])); 
	 print(summary(cov122[jumps])); 
	 print(summary(c(cov123[jumps]))); 
	 print(summary(c(cov124[jumps]))); 
 }# }}}

 cov12aa <- 0
 if (fixbeta==0) {
 ### covariances between different terms and  beta's 
 # {{{
	 betaiidR <- square1$betaiid; betaiidD <- square2$betaiid
	 HtR <- square1$Ht; HtD <- square2$Ht
	 covbetaRD <- t(betaiidR) %*% betaiidD
	 covbeta <-   -1*rowSums((HtR %*% covbetaRD)*HtD)

	 ### cov12 wrt betaD and betaR
	 betakt <- betaiidD[id1+1,,drop=FALSE]
	 covk1 <-apply(xxxR*betakt,2,cumsumidstratasum,id1,mid,xx$strata,xx$nstrata,type="sum")
	 covk2 <-apply(w*rrR*betakt,2,revcumsumidstratasum,id1,mid,xx$strata,xx$nstrata,type="lagsum")
	 covk2 <- c(covk2)*c(square1$cumS0i2)
	 covRD12 <- apply((covk1-covk2)*HtD,1,sum)
	 betakt <- betaiidR[id2+1,,drop=FALSE]
	 covk1 <-apply(xxxD1*betakt,2,cumsumidstratasum,id2,mid,xx2$strata,xx$nstrata,type="sum")
	 covk2 <-apply(w*rrD1*betakt,2,revcumsumidstratasum,id2,mid,xx2$strata,xx$nstrata,type="lagsum")
	 covk2 <- c(covk2)*c(square2$cumS0i2)
	 covRD21 <- apply((covk1-covk2)*HtR,1,sum)
	 cov12aa <- 2*(covbeta + covRD12+covRD21)
	 test <- 0
	 if (test==1) {
		 print("--------------")
		 print(summary(2*mu*covbeta))
		 print(summary(2*mu*covRD12))
		 print(summary(2*mu*covRD21))
		 print("--------------")
	 }
 } # }}}

 cov12 <- (cov12A-cov12aa)*mu

 return(list(cov=cov12,cov12A=cov12A*mu,covbeta=cov12aa*mu))
} # }}}

##' Break ties in event times for recurrent event data
##'
##' Resolves tied event times in a counting-process dataset by adding a small
##' random perturbation to duplicated exit times of event rows. This is a
##' preprocessing step required by \code{\link{recurrent_marginal}} and related
##' functions, which assume unique jump times within each stratum.
##'
##' A tie is defined as an event exit time (rows where \code{status} is a cause
##' code) that coincides with another exit time in the dataset. When
##' \code{exit.unique = TRUE} (default), a tie is flagged whenever an event time
##' also appears among any other exit times (censored or event). When
##' \code{exit.unique = FALSE}, only exact ties between two event rows are
##' resolved.
##'
##' Tied event times are perturbed by adding \eqn{U \cdot \delta} where
##' \eqn{U \sim \text{Uniform}(0, 1)} and \eqn{\delta} is \code{ddt} (defaulting
##' to half the smallest observed positive gap between consecutive exit times).
##' When subject IDs are provided via \code{id}, the corresponding interval start
##' time of the immediately following row for the same subject is updated to
##' maintain a valid counting-process structure, and a logical \code{tiebreaker}
##' column is added to flag affected rows.
##'
##' @param data A data frame in counting-process format, sorted by subject and
##'   time.
##' @param stop Name of the column containing interval exit (stop) times.
##'   Default is \code{"time"}.
##' @param start Name of the column containing interval entry (start) times,
##'   used to update the following row when \code{id} is supplied. Default is
##'   \code{"entry"}.
##' @param status Name of the column containing event status codes. Default is
##'   \code{"status"}.
##' @param id Name of the column containing subject identifiers. If supplied,
##'   the start time of the next interval for the same subject is adjusted to
##'   match the perturbed stop time, preserving interval continuity. Default is
##'   \code{NULL} (no adjustment made).
##' @param ddt Maximum perturbation size. Tied event times are shifted by a
##'   uniform draw on \eqn{[0, \text{ddt}]}. If \code{NULL} (default), \code{ddt}
##'   is set to half the smallest positive gap between any two consecutive exit
##'   times in the data.
##' @param exit.unique Logical. If \code{TRUE} (default), an event time is
##'   considered tied whenever it coincides with \emph{any} exit time in the
##'   data (including censored rows). If \code{FALSE}, only ties among event rows
##'   are resolved.
##' @param cause Integer vector of status codes that identify events (non-censored
##'   rows). If \code{NULL} (default), all non-censoring status values are treated
##'   as events.
##' @param cens.code Integer code(s) for censoring. Rows with this status are
##'   never perturbed. Default is \code{0}.
##' @param seed Optional integer passed to \code{set.seed()} before drawing the
##'   perturbations, making results reproducible. Default is \code{NULL} (no seed
##'   set).
##'
##' @return The input data frame \code{data} with tied event exit times
##'   perturbed. If \code{id} is supplied, an additional logical column
##'   \code{tiebreaker} marks rows whose start time was adjusted as a consequence
##'   of a perturbation in the preceding row.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{test_logrankRecurrent}}
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##'
##' ## Check for ties in event exit times
##' ev <- hf[hf$status == 1, ]
##' any(duplicated(ev$time))
##'
##' ## Resolve ties before fitting the marginal mean model
##' hf_clean <- tie_breaker(hf, stop = "time", start = "entry",
##'                          status = "status", id = "id",
##'                          cause = 1, cens.code = 0)
##'
##' out <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
##'                            data = hf_clean, cause = 1, death.code = 2)
##' summary(out, times = 1:5)
##'
##' @export
tie_breaker <- function(data, stop = "time", start = "entry", status = "status",
                        id = NULL, cause = NULL, cens.code = 0,
                        exit.unique = TRUE, ddt = NULL, seed = NULL)
{# {{{
   id.col <- id
   if (!is.null(id)) id <- data[, id]
   ord  <- 1:nrow(data)
   stat <- data[, status]
   time <- data[, stop]

   ## determine event codes, excluding censoring
   if (is.null(cause)) cause <- unique(stat)
   type0 <- which(cause %in% cens.code)
   if (length(type0) > 0) cause <- cause[-type0]
   jumps <- stat %in% cause

   ## detect tied event times
   time1 <- data[jumps,  stop]
   time0 <- data[!jumps, stop]
   lt0   <- length(time0)
   ddp   <- duplicated(c(time0, time1))
   if (exit.unique)
     ties <- ddp[(lt0 + 1):nrow(data)]
   else
     ties <- duplicated(c(time1))

   nties <- sum(ties)

   ## initialise tiebreaker column unconditionally
   data[, "tiebreaker"] <- FALSE

   if (nties >= 1) {
     message(nties, " tied event time(s) found and perturbed.")

     ordties <- ord[jumps][ties]

     ## compute ddt from sorted unique event-time gaps, not row differences
     if (is.null(ddt)) {
       ut  <- sort(unique(time[jumps]))
       gaps <- diff(ut)
       ddt <- min(gaps[gaps > 0]) * 0.5
     }

     ## apply reproducible perturbation
     if (!is.null(seed)) set.seed(seed)
     time[ordties] <- time[ordties] + runif(nties) * ddt
     data[ordties, stop] <- time[ordties]

     ## mark perturbed rows
     data[ordties, "tiebreaker"] <- TRUE

     ## when id is available, propagate stop -> start for the next interval
     ## of the same subject to maintain a valid counting-process structure
     if (!is.null(id.col)) {
       ties_flag  <- ord %in% ordties
       lagties    <- dlag(ties_flag)
       change.start <- lagties == TRUE & id == dlag(id)
       change.start[is.na(change.start)] <- FALSE
       ocs <- ord[change.start]
       data[ocs, start] <- data[ocs - 1, stop]
     }
   }

   return(data)
 } # }}}


##' Simulate recurrent events with two event types and a terminal event
##'
##' Simulates recurrent event data with up to two distinct event types and an
##' optional terminal event (death), based on user-supplied cumulative hazard
##' functions. Dependence between processes can be introduced via shared or
##' correlated gamma-distributed frailties.
##'
##' The simulation proceeds by sequentially drawing the next event time from the
##' specified cumulative hazards, taking the minimum of the two recurrent event
##' times, and stopping each subject at death or administrative censoring.
##'
##' Dependence between processes is controlled by \code{dependence}:
##' \describe{
##'   \item{\code{0}}{Independence: all subjects have frailty fixed at 1.}
##'   \item{\code{1}}{Shared frailty: all processes share a single
##'     gamma-distributed random effect with mean 1 and variance \code{var.z}.}
##'   \item{\code{4}}{Recurrent-event frailty only: the two recurrent event
##'     processes share a gamma frailty but the terminal event is independent.}
##' }
##' For more complex correlation structures across two event types and death, use
##' \code{\link{sim_recurrentTS}}.
##'
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the first type of recurrent event.
##' @param cumhaz2 Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the second type of recurrent event.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event. If \code{NULL}, no terminal event
##'   is simulated and follow-up ends at the end of \code{cumhaz}.
##' @param r1 Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the first event type.
##' @param r2 Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the second event type.
##' @param rd Optional numeric vector of length \code{n} with subject-specific
##'   relative risk multipliers for the terminal event.
##' @param rc Optional numeric vector of length \code{n} with subject-specific
##'   multipliers for the exponential censoring rate.
##' @param gap.time Logical. If \code{TRUE}, event times are drawn as gap times
##'   (time since the last event) rather than calendar times. Default is
##'   \code{FALSE}.
##' @param max.recurrent Maximum number of recurrent events allowed per subject.
##'   Default is \code{100}.
##' @param dependence Integer specifying the frailty structure. One of \code{0}
##'   (independence), \code{1} (shared gamma frailty), or \code{4} (shared frailty
##'   for recurrent events only). See Details.
##' @param var.z Variance of the gamma-distributed frailty. Default is \code{1}.
##' @param cor.mat Correlation matrix for the random effects. Used when
##'   \code{dependence = 2} (in \code{\link{sim_recurrent_list}}).
##' @param cens Rate of exponential censoring. If \code{NULL} (default), no
##'   additional censoring is applied.
##' @param ... Further arguments passed to \code{\link{sim_recurrent_list}}.
##'
##' @return A data frame in counting-process format (one row per event interval
##'   per subject) with columns:
##'   \item{id}{Subject identifier.}
##'   \item{start, entry}{Interval start time.}
##'   \item{stop, time}{Interval end time (event or censoring time).}
##'   \item{status}{Event type at \code{stop}: \code{1} or \code{2} for a
##'     recurrent event of the corresponding type, \code{0} for censoring.}
##'   \item{death}{Indicator for a terminal event (\code{1}) or censoring/survival
##'     (\code{0}).}
##'   Attributes \code{"cumhaz"}, \code{"death.cumhaz"}, \code{"rr"}, and
##'   \code{"rd"} store the inputs used for simulation.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{sim_recurrent}}, \code{\link{sim_recurrent_list}},
##'   \code{\link{sim_recurrentTS}}
##'
##' @examples
##' data(CPH_HPN_CRBSI)
##' dr    <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi
##' base4 <- CPH_HPN_CRBSI$mechanical
##'
##' ## Single recurrent event type, with and without terminal event
##' rr <- sim_recurrent(5, base1)
##' dlist(rr, . ~ id, n = 0)
##'
##' rr <- sim_recurrent(5, base1, death.cumhaz = dr)
##' dlist(rr, . ~ id, n = 0)
##'
##' ## Verify that estimated rates recover the true baselines (increase n for precision)
##' rr <- sim_recurrent(100, base1, death.cumhaz = dr)
##' par(mfrow = c(1, 3))
##' mets:::showfitsim(causes = 1, rr, dr, base1, base1)
##'
##' ## Shared frailty across all processes
##' rr <- sim_recurrent(100, base1, death.cumhaz = dr, dependence = 1, var.z = 0.4)
##' dtable(rr, ~death + status)
##'
##' ## Two event types; second type uses the mechanical complication rate
##' set.seed(100)
##' rr <- sim_recurrentII(100, base1, base4, death.cumhaz = dr)
##' dtable(rr, ~death + status)
##' par(mfrow = c(2, 2))
##' mets:::showfitsim(causes = 2, rr, dr, base1, base4)
##'
##' ## Three event types and two causes of death via sim_recurrent_list
##' set.seed(100)
##' cumhaz <- list(base1, base1, base4)
##' drl    <- list(dr, base4)
##' rr     <- sim_recurrent_list(100, cumhaz, death.cumhaz = drl, dependence = 0)
##' dtable(rr, ~death + status)
##' mets:::showfitsimList(rr, cumhaz, drl)
##'
##' @export
##' @aliases sim_recurrentII sim_recurrent_list
sim_recurrentII <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,r1=NULL,r2=NULL,rd=NULL,rc=NULL,dependence=0,var.z=1,
			   cor.mat=NULL,cens=NULL,gap.time=FALSE,max.recurrent=100,...) 
{# {{{
cumhazL <- list(cumhaz,cumhaz2)
rr <- cbind(r1,r2)
if (!is.null(death.cumhaz)) death.cumhaz <- list(death.cumhaz)
if (!is.null(r1)) {
	if (!is.null(r2)) r2 <- rep(1,length(r1))
	rr <- cbind(r1,r2)
}
data <-     sim_recurrent_list(n,cumhazL,death.cumhaz=death.cumhaz,rr=rr,
		     rd=rd,rc=rc,dependence=dependence,var.z=var.z,
		     cor.mat=cor.mat,cens=cens,gap.time=gap.time,
		     max.recurrent=max.recurrent,...)
return(data)
}# }}}

##' Simulate recurrent events with a single event type and a terminal event
##'
##' A convenience wrapper around \code{\link{sim_recurrentII}} for the common
##' case of a single recurrent event type. Frailty and censoring options are
##' passed through to \code{\link{sim_recurrent_list}}.
##'
##' @inherit sim_recurrentII examples author
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the cumulative
##'   hazard of the recurrent event.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event. If \code{NULL}, no terminal event
##'   is included.
##' @param r1 Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the recurrent event.
##' @param rd Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the terminal event.
##' @param rc Optional numeric vector of length \code{n} of subject-specific
##'   multipliers for the exponential censoring rate.
##' @param ... Further arguments passed to \code{\link{sim_recurrent_list}},
##'   including \code{dependence}, \code{var.z}, \code{gap.time}, and
##'   \code{max.recurrent}.
##' @export
sim_recurrent <- function(n,cumhaz,death.cumhaz=NULL,r1=NULL,rd=NULL,rc=NULL,...) 
{# {{{
## wrapper for sim_recurrentII without type-2 events
if (!is.null(death.cumhaz)) death.cumhaz <- list(death.cumhaz)
if (!is.null(r1)) r1 <- as.matrix(r1,ncol=1)
if (!is.null(rd)) rd <- as.matrix(rd,ncol=1)

rr <- sim_recurrent_list(n,list(cumhaz),death.cumhaz=death.cumhaz,rr=r1,rd=rd,rc=rc,...)
return(rr)
}# }}}

##' @export
sim_recurrent_list <- function(n,cumhaz,death.cumhaz=NULL,rr=NULL,rd=NULL,rc=NULL,zzr=NULL,zzd=NULL,
  gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,dependence=0,var.z=1,cor.mat=NULL,cens=NULL,extend=TRUE,...) 
{# {{{
  status <- fdeath <- dtime <- NULL # to avoid R-check 

  if (dependence==0) { # {{{
	  z <- z1 <- z2 <- zd <- rep(1,n) 
	  if (is.null(zzr)) zzr <- matrix(z,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     } else if (dependence==1) {
	  z <- rgamma(n,1/var.z[1])*var.z[1]
	  z1 <- z; z2 <- z; zd <- z
	  if (is.null(zzr)) zzr <- matrix(z,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; zd <- rep(1,n) 
	  if (is.null(zzr)) zzr <- matrix(zz,n,length(cumhaz)) 
	  if (!is.null(death.cumhaz))
	  if (is.null(zzd)) zzd <- matrix(zd,n,length(death.cumhaz))
     }    else stop("dependence 0,1,4"); # }}}

   if (is.null(rr)) rr <- matrix(1,n,length(cumhaz))
   if (!is.null(death.cumhaz))
   if (is.null(rd)) rd <- matrix(1,n,length(death.cumhaz))
   if (is.null(rc)) rc <- rep(1,n)

  ## extend cumulative for death to full range  of cause 1
  if (!is.null(death.cumhaz)) {
     out <- extendCums(c(cumhaz,death.cumhaz),NULL,extend=extend)
     l <- length(cumhaz)
     ld <- length(death.cumhaz)
     cumhaz <- out[1:l]
     cumhazd <- out[(l+1):(l+ld)]
  } else {
     if (length(cumhaz)>1) {
	     out <- extendCums(cumhaz,NULL,extend=extend)
	     l <- length(cumhaz)
	     cumhaz <- out[1:l]
      }
  }
  max.time <- tail(cumhaz[[1]][,1],1)

  rrz <- rr*zzr
  tall <- rchazl(cumhaz,rrz) 
  tall$id <- 1:n

### death time simulated
  if (!is.null(death.cumhaz)) {# {{{
          rrzd <- rd*zzd
	  timed   <- rchazl(cumhazd,rrzd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }
# }}}


### fixing the first time to event
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  i <- 0; 
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i <= max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
	  tt <- rchazl(cumhaz,rrz[still$id,,drop=FALSE],entry=(1-gap.time)*still$time) 
	  if (i==max.recurrent) {
		  tt$time <- max.time; tt$status <- 0
	  }
	  if (gap.time) {
		  tt$entry <- still$time
		  tt$time  <- tt$time+still$time
		  if (i==max.recurrent) { tt$time <- max.time; tt$status <- 0}
	  }
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  } 

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time
  tall$rr <- NULL

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"rr") <- rr
  attr(tall,"rd") <- rd
  attr(tall,"z") <- zzr
  attr(tall,"zd") <- zzd

  return(tall)
  }# }}}

showfitsimList <- function(rr,cumhaz,dr,...)  {# {{{
 colp <- max(length(cumhaz),length(dr))
 par(mfrow=c(2,colp))
 if (!is.null(cumhaz)) {
 for (i in 1:length(cumhaz)) {
	 pp <- phreg(Surv(entry,time,status==i)~+1,rr)
	 plot(pp,main=paste("status=",i),...); 
	 lines(cumhaz[[i]],col=2,lwd=2)
 }
 }
 if (!is.null(dr)) {
 for (i in 1:length(dr)) {
	 pp <- phreg(Surv(entry,time,death==i)~+1,rr)
	 plot(pp,main=paste("death=",i),...); 
	 lines(dr[[i]],col=2,lwd=2)
 }
 }
}
# }}}

showfitsim <- function(causes=2,rr,dr,base1,base4,which=1:3) 
{# {{{
if (1 %in% which) {
  drr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
  basehazplot.phreg(drr,ylim=c(0,8))
  lines(dr,col=2)
}
###
if (2 %in% which) {
  xrr <- phreg(Surv(entry,time,status==1)~cluster(id),data=rr)
  basehazplot.phreg(xrr,add=TRUE)
###  basehazplot.phreg(xrr)
  lines(base1,col=2)
  if (causes>=2) {
	  xrr2 <- phreg(Surv(entry,time,status==2)~cluster(id),data=rr)
	  basehazplot.phreg(xrr2,add=TRUE)
	  lines(base4,col=2)
  }
  }
if (3 %in% which) {
  meanr1 <-   recurrentMarginalPhreg(xrr,drr)
  basehazplot.phreg(meanr1,se=TRUE)
  if (causes>=2) {
	  meanr2 <-   recurrentMarginalPhreg(xrr2,drr)
	  basehazplot.phreg(meanr2,se=TRUE,add=TRUE,col=2)
  }
}
}# }}}

##' Simulate recurrent events from a two-stage Cox or Ghosh-Lin model
##'
##' Simulates recurrent event data from a fitted two-stage model, where the
##' recurrent event process and the terminal event are each described by a
##' separate fitted model. The recurrent event model may be either a Cox
##' proportional hazards model (\code{phreg}) or a Ghosh-Lin marginal rate model
##' (\code{recreg}); the terminal event model must be a Cox model (\code{phreg}).
##'
##' Covariates are drawn by bootstrap from \code{data} (if supplied), and
##' subject-specific relative risks and strata are derived from the fitted
##' model objects. Stratified baselines are fully supported. When
##' \code{dependence} is \code{NULL} (default), the simulation uses the
##' two-stage structure from \code{\link{sim_GLcox}}; setting \code{dependence}
##' to an integer falls back to \code{\link{sim_recurrent_list}} with the
##' corresponding frailty model.
##'
##' @param cox1 A fitted \code{phreg} object for the recurrent event rate, or a
##'   fitted \code{recreg} (Ghosh-Lin) object. The model type is detected
##'   automatically from the class.
##' @param coxd A fitted \code{phreg} object for the terminal event. May be
##'   \code{NULL} if no terminal event is modelled.
##' @param n Number of subjects to simulate. Default is \code{1}.
##' @param data The data frame on which \code{cox1} and \code{coxd} were fitted,
##'   used to draw covariate values for the simulated subjects. If \code{NULL},
##'   covariates must be supplied via \code{r1}, \code{rd}, \code{strata1}, and
##'   \code{stratad}.
##' @param type Simulation type: \code{"default"} (auto-detected from class of
##'   \code{cox1}), \code{"cox-cox"}, or \code{"gl-cox"}.
##' @param id Name of the subject identifier variable in \code{data}. Default is
##'   \code{"id"}.
##' @param varz Variance of the frailty distribution in the two-stage model.
##'   Default is \code{1}.
##' @param share Proportion of the shared frailty assigned to the recurrent event
##'   process in the partial-sharing model. Default is \code{1}.
##' @param cens Rate of exponential censoring. Default is \code{0.001}.
##' @param scale1 Scalar multiplier for the baseline cumulative hazard of the
##'   recurrent event process. Default is \code{1}.
##' @param scaled Scalar multiplier for the baseline cumulative hazard of the
##'   terminal event. Default is \code{1}.
##' @param dependence If non-\code{NULL}, falls back to
##'   \code{\link{sim_recurrent_list}} with this frailty structure (see
##'   \code{\link{sim_recurrentII}} for valid values). Default is \code{NULL}.
##' @param r1 Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the recurrent event, used when \code{data = NULL}.
##' @param rd Optional numeric vector of length \code{n} of subject-specific
##'   relative risks for the terminal event, used when \code{data = NULL}.
##' @param rc Optional numeric vector of length \code{n} of subject-specific
##'   censoring rate multipliers.
##' @param strata1 Optional integer vector of length \code{n} specifying the
##'   stratum index (0-based) for the recurrent event model, used when
##'   \code{data = NULL}.
##' @param stratad Optional integer vector of length \code{n} specifying the
##'   stratum index (0-based) for the terminal event model, used when
##'   \code{data = NULL}.
##' @param death.code Integer status code used for the terminal event in the
##'   output \code{status} column. Default is \code{3}.
##' @param ... Further arguments passed to \code{\link{sim_GLcox}}, including
##'   \code{nmin} and \code{nmax} for the linear approximation grid.
##'
##' @return A data frame in counting-process format with one row per event
##'   interval per subject. Column names match those in the original model
##'   formula (entry, exit, and status variables). Additional columns include
##'   \code{id} and the covariates drawn from \code{data} (if supplied). The
##'   terminal event is coded as \code{death.code} in the status variable;
##'   recurrent events are coded as \code{1}.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{sim_recurrent_list}},
##'   \code{\link{sim_GLcox}}
##'
##' @references
##' Scheike, T. H. (2026). Two-stage recurrent events random effects models.
##' \emph{Lifetime Data Analysis}.
##'
##' @examples
##' data(hfactioncpx12)
##' hf    <- hfactioncpx12
##' hf$x  <- as.numeric(hf$treatment)
##' n     <- 100
##'
##' ## Cox-Cox two-stage model
##' xr <- phreg(Surv(entry, time, status == 1) ~ x + cluster(id), data = hf)
##' dr <- phreg(Surv(entry, time, status == 2) ~ x + cluster(id), data = hf)
##' simcoxcox <- sim_recurrent_ts(xr, dr, n = n, data = hf, death.code = 2)
##'
##' ## Ghosh-Lin/Cox two-stage model
##' recGL  <- recreg(Event(entry, time, status) ~ x + cluster(id), hf, death.code = 2)
##' simglcox <- sim_recurrent_ts(recGL, dr, n = n, data = hf, death.code = 2)
##'
#' @export sim_recurrent_ts
sim_recurrent_ts <- function(cox1,coxd=NULL,
             n=1, data=NULL,type=c("default","cox-cox","gl-cox"),
             id="id",varz=1,share=1,cens=0.001,
      scale1=1,scaled=1,dependence=NULL,
 r1=NULL,rd=NULL,rc=NULL,strata1=NULL,stratad=NULL,death.code=3,
                          ...) {# {{{
## exp censoring default
statusD <- death <- NULL

if (type[1]=="default" & inherits(cox1,"recreg")) type <- "gl-cox" 
if (type[1]=="default" & inherits(cox1,"phreg")) type <- "cox-cox" 
if (type[1]=="cox-cox") type <- 3 else type <- 2

if (!is.null(data)) {
   coxs <- list(cox1,coxd)
   rrdata <- draw_phregs(coxs,n,data=data)
   rr1 <- rrdata$rr[,1]
   rstrata1 <- rrdata$strata[,1]

   if (!is.null(coxd)) {
      rrd <- rrdata$rr[,2]
      rstratad <- rrdata$strata[,2]
   }
} else { 
	data <- c()
	rr1 <- rep(1,n)
	rrd <- rep(1,n)
	rstrata1 <- rep(0,n)
	rstratad <- rep(0,n)
}

if (is.null(rc)) rc <- rep(1,n)

if (!is.null(coxd))    {
    coxd$cumhaz <- scalecumhaz(coxd$cumhaz,scaled); 
    LamD <- basecumhaz(coxd,only=1)
} else LamD <- NULL
cox1$cumhaz <-   scalecumhaz(cox1$cumhaz,scale1); 
Lam1 <- basecumhaz(cox1,only=1)

### when data not given then use r1,rd, strata1,stratad
if (is.null(r1)) r1 <- rr1
if (is.null(rd)) rd <- rrd
if (is.null(strata1)) strata1 <- rstrata1
if (is.null(stratad)) stratad <- rstratad


if (is.null(dependence) & (!is.null(LamD))) {
 ## go through combined strata for the two models 
strat1d <- mystrata(data.frame(strata1,stratad))
rrs <- c()
for (j in 1:attr(strat1d,"nlevel")) {
     r1i <- which(strat1d==j)
     strata1ss <- strata1[r1i[1]]+1
     stratadss <- stratad[r1i[1]]+1
     Lam1s <- Lam1[[strata1ss]]
     LamDs <- LamD[[stratadss]]
     ns <- length(r1i)
     rrss <- sim_GLcox(ns,Lam1s,LamDs,var.z=varz,r1=r1[r1i],rd=rd[r1i],rc=rc[r1i],
		model="twostage",cens=cens,type=type,share=share,...)
    rrss$ids <- rrss$id
    rrss$id <- r1i[rrss$id+1]
    rrs <- rbind(rrs,rrss)
}			  
rrs <- dtransform(rrs,statusD=death.code,statusD==3)
} else { 
if (is.null(dependence)) dependence <- 0
if (!is.null(LamD)) 
rrs <- sim_recurrent_list(n,Lam1,death.cumhaz=LamD,rr=matrix(r1,ncol=1),rd=matrix(rd,ncol=1),rc=rc,cens=cens,var.z=varz,dependence=dependence)
else rrs <- sim_recurrent_list(n,list(Lam1),rr=matrix(r1,ncol=1),rc=rc,cens=cens,var.z=varz,dependence=dependence)
rrs$Z <- attr(rrs,"z")[rrs$id+1]
rrs$statusD <- rrs$status
if (!is.null(LamD))  {
   rrs <- dtransform(rrs,statusD=death.code,death==1)
}
}

## add correct names to entry,time,status if possible
if (inherits(cox1,"phreg")) {
   varsY <- all.vars(update(drop.specials(cox1$formula,"cluster"),.~1)) 
   rrs[,varsY] <- cbind(rrs$start,rrs$stop,rrs$statusD)
   rrs <- dkeep(rrs,c(varsY,"id"))
}

## add covariates, 
if (!is.null(data)) rrs <- cbind(rrs,rrdata$data[rrs$id,])

return(rrs)
}
# }}}

sim_RecurrentIIHist <- function(n,cumhaz,death.cumhaz,cens=NULL,rr=NULL,rc=NULL,rd=NULL,
	    max.recurrent=100,dependence=0,var.z=0.22,cor.mat=NULL,
	    HistN1=~I(Nt^.5),HistD=~I(Nt^.5),HistN1.beta=c(1.0),HistD.beta=c(1.0),...) 
{# {{{

  ctime <- fdeath <- dtime <- NULL # to avoid R-check 
  status <- dhaz <- NULL; dhaz2 <- NULL

  if (dependence==0) { z <- z1 <- zc <- zd  <-  rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
###	      z <- exp(rnorm(n,1)*var.z[1]^.5)
	      z1 <- z; z2 <- z; zd <- z
	      if (!is.null(cor.mat)) { zd <- rep(1,n); }
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
###	      print(summary(z))
###	      print(cor(z))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
###	      print(summary(z))
###	      print(cor(z))
      } else stop("dependence 0-3"); # }}}

  ## covariate adjustment 
  if (is.null(rr))  rr <- z1; 
  if (is.null(rc))  rc <- zc; 
  if (is.null(rd))  rd <- zd; 
  if (length(rr)!=n) rr <- rep(rr[1],n)
  if (length(rc)!=n) rc <- rep(rc[1],n)
  if (length(rd)!=n) rd <- rep(rd[1],n)

  ll <- nrow(cumhaz)
  ### extend of cumulatives
  cumhaz <- rbind(c(0,0),cumhaz)
  death.cumhaz <- rbind(c(0,0),death.cumhaz)

  if (!is.null(cens)) {
	  if (is.matrix(cens))  {
             out <- extendCums(list(cumhaz,death.cumhaz,cens),NULL)
   	     cumcens <- out$cum3
	  } else {
             out <- extendCums(list(cumhaz,death.cumhaz),NULL)
	  }
  } else {
     out <- extendCums(list(cumhaz,death.cumhaz),NULL)
  }
  cumhaz <- out$cum1
  cumhazd <- out$cum2
  max.time <- tail(cumhaz[,1],1)

  ### draw censorting times 
  if (!is.null(cens)) { 
	  if (is.matrix(cens)) sdata <- rchaz(cens,rc) else 
          sdata <- data.frame(entry=0,time=pmin(max.time,rexp(n)/(c(rc)*cens)),
			      status=0,id=1:n)
  } else sdata <- data.frame(entry=0,time=rep(max.time,n),status=0,id=1:n)
  sdata$ctime <- sdata$time
  sdata$Nt <- 0

  i <- 0; 
  tall <- c()
  still <- tt1 <-  sdata
  ## start at 0
  tt1$time <- still$time <- 0
  while (any((still$time<still$ctime) &  (i < max.recurrent))) { ## {{{
	  still$Nt <- i
	  nn <- nrow(still)
	  z1r <- rr[still$id]
	  zdr <- rd[still$id]

	  m1 <- model.matrix(HistN1,still)[,-1,drop=FALSE]
	  md <- model.matrix(HistD,still)[,-1,drop=FALSE]
	  r1h <- c(exp(m1 %*% HistN1.beta))
	  rdh <- c(exp(md %*% HistN1.beta))

          tt1 <- rcrisk(cumhaz,cumhazd,z1r*r1h,zdr*rdh,entry=still$time)
	  tt1 <- cbind(tt1,dkeep(still,~id+ctime),row.names=NULL)
	  tt1 <- dtransform(tt1,status=0,time>ctime)
	  tt1 <- dtransform(tt1,time=ctime,time>ctime)
	  tt1$Nt <- i
          ## remove dead from tt1 to get those still there 
          deadid <- (tt1$status %in% c(0,2))
	  ### those that are still under risk 
	  still <- tt1[!deadid,,drop=FALSE]
	  ## also keep only those before max.time
          still <- subset(still,still$time<max.time)
	  tall <- rbind(tall,tt1,row.names=NULL)
	  i <- i+1
  }  # }}}

  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time
  tall <- dkeep(tall,~id+entry+time+status+ctime+Nt+start+stop)

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cens.cumhaz") <- cens

  return(tall)
  }# }}}

##' Simulate recurrent events from a two-stage model with structured gamma frailties
##'
##' Simulates recurrent event data with two event types and a terminal event,
##' using a parametric two-stage frailty model. The construction ensures that the
##' marginal rates are approximately correct: conditional on survival,
##' \eqn{E(dN_j \mid D > t) \approx} \code{cumhazj}, and the hazard of death
##' equals \code{death.cumhaz}.
##'
##' The frailty structure uses three gamma random variables \eqn{Z_{d1}},
##' \eqn{Z_{d2}}, \eqn{Z_{12}} to induce dependence:
##' \deqn{Z_\text{death} = Z_{d1} + Z_{d2}, \quad
##'       Z_1 = Z_{d1}^{\nu_1} Z_{12}, \quad
##'       Z_2 = Z_{d2}^{\nu_2} Z_{12}^{\nu_3}.}
##' The parameters \code{share1} and \code{vargamD} control how the death frailty
##' splits between the two components, and \code{vargam12} controls the shared
##' recurrent-event frailty. Setting \eqn{\nu = (1,1,1)} with \code{share1 = 0.5}
##' gives a symmetric structure; varying \eqn{\nu} allows asymmetric dependence.
##'
##' @param n Number of subjects to simulate.
##' @param cumhaz Two-column matrix \code{(time, cumhaz)} giving the target
##'   marginal cumulative rate of the first recurrent event type.
##' @param cumhaz2 Two-column matrix \code{(time, cumhaz)} giving the target
##'   marginal cumulative rate of the second recurrent event type.
##' @param death.cumhaz Two-column matrix \code{(time, cumhaz)} giving the
##'   cumulative hazard of the terminal event.
##' @param nu Numeric vector of length 3: the powers \eqn{(\nu_1, \nu_2, \nu_3)}
##'   applied to the frailty components (see Details). Must satisfy
##'   \eqn{\nu_j > -1/\text{shape}}. Default is \code{rep(1, 3)}.
##' @param share1 Proportion of the total death frailty variance assigned to the
##'   first component \eqn{Z_{d1}}. The remainder goes to \eqn{Z_{d2}}. Must be
##'   in \eqn{(0, 1)}. Default is \code{0.3}.
##' @param vargamD Total variance of the death frailty \eqn{Z_\text{death}}.
##'   Default is \code{2}.
##' @param vargam12 Variance of the shared recurrent-event frailty \eqn{Z_{12}}.
##'   Default is \code{0.5}.
##' @param gap.time Logical. If \code{TRUE}, event times are drawn as gap times
##'   rather than calendar times. Default is \code{FALSE}.
##' @param max.recurrent Maximum number of recurrent events per subject. Default
##'   is \code{100}.
##' @param cens Rate of exponential censoring. If \code{NULL} (default), no
##'   administrative censoring is applied.
##' @param ... Further arguments passed to lower-level simulation functions.
##'
##' @return A data frame in counting-process format (one row per event interval
##'   per subject) with columns \code{id}, \code{start}, \code{stop},
##'   \code{entry}, \code{time}, \code{status}, and \code{death}. Attributes
##'   store the (possibly adjusted) cumulative hazards used in simulation and
##'   the frailty parameters.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{sim_recurrentII}}, \code{\link{sim_recurrent_ts}}
##'
##' @examples
##' data(CPH_HPN_CRBSI)
##' dr    <- CPH_HPN_CRBSI$terminal
##' base1 <- CPH_HPN_CRBSI$crbsi
##' base4 <- CPH_HPN_CRBSI$mechanical
##'
##' rr <- sim_recurrentTS(1000, base1, base4, death.cumhaz = dr)
##' dtable(rr, ~death + status)
##' mets:::showfitsim(causes = 2, rr, dr, base1, base4)
##'
##' @export
sim_recurrentTS <- function(n,cumhaz,cumhaz2,death.cumhaz=NULL,
		    nu=rep(1,3),share1=0.3,vargamD=2,vargam12=0.5,
		    gap.time=FALSE,max.recurrent=100,cens=NULL,...) 
{# {{{

k <- 1
nu1 <- nu[1]; nu2 <- nu[2]; nu3 <- nu[3]
###nu1 <- 1; nu2 <- 1; nu3 <- 0.4
share2 <- (1-share1)
vargam <- vargamD
vargam12 <- 0.5
agam1 <- share1/vargam
agam2 <- share2/vargam
betagam=1/vargam
gamma1  <- rep(rgamma(n,agam1)*vargam,each=k)
gamma2  <- rep(rgamma(n,agam2)*vargam,each=k)
agam12 <- 1/vargam12
betagam12 <- 1/vargam12
gamma12 <- rep(rgamma(n,agam12)*vargam12,each=k)
agamD <- agam1+agam2
z1 <- (gamma1^nu1)*gamma12
z2 <- (gamma2^nu2)*gamma12^nu3
gamD <- gamma1+gamma2
zd <- gamD
egamma12nu3 <- (gamma(agam12+nu3)/gamma(agam12))*1/(betagam12)^nu3
zs <- cbind(z1,z2,zd)

 status <- fdeath <- dtime <- NULL # to avoid R-check 
 dhaz <- haz2 <- dhaz <- NULL

 ll <- nrow(cumhaz)
 max.time <- tail(cumhaz[,1],1)

 ################################################################
 ### approximate hazards to make marginals fit (approximately)
 ################################################################
 ## step-size set to one and range to that of base1
 base1 <- predictCumhaz(rbind(0,as.matrix(cumhaz)),1:round(tail(cumhaz[,1],1)) )
 if (!is.null(death.cumhaz)) 
     death.cumhaz <- predictCumhaz(rbind(0,as.matrix(death.cumhaz)),base1[,1])

 orig.death <- death.cumhaz
 dbase1 <- death.cumhaz
 gt <- exp(vargam*dbase1[,2]) 
 dtt <- diff(c(0,dbase1[,1]))
 lams <- (diff(c(0,dbase1[,2]))/dtt)*gt
 death.cumhaz <- cbind(dbase1[,1],cumsum(dtt*lams))

 dbase1 <- cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2]
 dtt <- diff(c(0,base1[,1]))
 gt <- (gamma(agam1+nu1)/gamma(agam1))*(1/(betagam+dbase1))^nu1
 lams <- (diff(c(0,base1[,2]))/dtt)*(1/gt)
 cumhaz <- cbind(base1[,1],cumsum(dtt*lams))

 base1 <- cumhaz2
 dbase1 <- cpred(rbind(c(0,0),death.cumhaz),base1[,1])[,2]
 dtt <- diff(c(0,base1[,1]))
 gt <- (gamma(agam2+nu2)/gamma(agam2))*(1/(betagam+dbase1))^nu2
 lams <-(1/egamma12nu3)*(diff(c(0,base1[,2]))/dtt)*(1/gt)
 cumhaz2 <- cbind(base1[,1],cumsum(dtt*lams))

 cumhaz <- rbind(c(0,0),cumhaz)
 cumhaz2 <- rbind(c(0,0),cumhaz2)
 death.cumhaz <- rbind(c(0,0),death.cumhaz)

## range max of cumhaz and cumhaz2 
  out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz),NULL)
  cumhaz <- out$cum1
  cumhaz2 <- out$cum2
  cumhazd <- out$cum3
  max.time <- tail(cumhaz[,1],1)

### recurrent first time
  tall1 <- rchaz(cumhaz,rr=z1)
  tall2 <- rchaz(cumhaz2,rr=z2)
  tall <- tall1 
  tall$status <- ifelse(tall1$time<tall2$time,tall1$status,2*tall2$status)
  tall$time <- ifelse(tall1$time<tall2$time,tall1$time,tall2$time)
  tall$id <- 1:n
  tall$rr2 <- tall2$rr
### death time simulated
  if (!is.null(death.cumhaz)) {# {{{
	  timed   <- rchaz(cumhazd,rr=zd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/cens
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/cens
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }# }}}

  ### fixing the first time to event
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  ### setting aside memory 
  tt1 <- tt2 <- tt
  i <- 1; 
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i < max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
          tt1 <- rchaz(cumhaz,rr=z1[still$id],entry=still$time)
          tt2 <- rchaz(cumhaz2,rr=z2[still$id],entry=still$time)
	  tt <- tt1
          tt$status <- ifelse(tt1$time<=tt2$time,tt1$status,2*tt2$status)
          tt$time <-   ifelse(tt1$time<=tt2$time,tt1$time,tt2$time)
	  tt$rr2 <- tt2$rr
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  }
  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"zs") <- zs

  attr(tall,"gamma.death") <- c(agam1,agam2,betagam,vargamD)
  attr(tall,"gamma.N12") <-   c(agam12,betagam12,vargam12)

  return(tall)
  }# }}}

##' Compute cumulative event counts as time-dependent covariates
##'
##' For each subject and each row of \code{data}, counts the number of prior
##' events of specified types in the recurrent event history. The resulting
##' count columns can be used as time-dependent covariates in subsequent models,
##' e.g. to capture event-history dependence in the recurrent event rate.
##'
##' When \code{lag = TRUE} (default), the count at each row reflects events that
##' occurred strictly before the current time point (i.e. \eqn{N(t-)}), making
##' it suitable as a left-continuous covariate in counting-process models.
##'
##' @param data A data frame in counting-process format, with one row per event
##'   interval per subject.
##' @param status Name of the column containing event status codes. Default is
##'   \code{"status"}.
##' @param id Name of the column containing subject identifiers. Default is
##'   \code{"id"}.
##' @param types Integer vector of status codes to count. Each value in
##'   \code{types} generates one new count column (when \code{multitype = FALSE})
##'   or contributes to a single combined count (when \code{multitype = TRUE}).
##'   Default is \code{1}.
##' @param names.count Prefix for the names of the new count columns. The
##'   status code is appended, e.g. \code{"Count1"}, \code{"Count2"}.
##'   Default is \code{"Count"}.
##' @param lag Logical. If \code{TRUE} (default), the count at each row is the
##'   number of events strictly before the current time (\eqn{N(t-)}). If
##'   \code{FALSE}, events at the current time are included (\eqn{N(t)}).
##' @param multitype Logical. If \code{TRUE}, events with status in \code{types}
##'   are aggregated into a single count column, optionally weighted by
##'   \code{marks}. If \code{FALSE} (default), a separate count column is created
##'   for each value in \code{types}.
##' @param marks Optional numeric vector of weights applied to events when
##'   \code{multitype = TRUE}. If \code{NULL} (default), each event has weight 1.
##'
##' @return The input data frame \code{data} with one or more new integer columns
##'   appended. With \code{multitype = FALSE}, columns are named
##'   \code{paste0(names.count, k)} for each \code{k} in \code{types}; with
##'   \code{multitype = TRUE}, a single column named
##'   \code{paste0(names.count, types[1])} is added. An internal bookkeeping
##'   column \code{lbnr__id} is also added.
##'
##' @author Thomas Scheike
##'
##' @examples
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' dtable(hf, ~status)
##'
##' ## Separate counts for event types 1 and 2
##' rr <- count_history(hf, types = 1:2, id = "id", status = "status")
##' dtable(rr, ~"Count*" + status, level = 1)
##'
##' @export
count_history <- function(data,status="status",id="id",types=1,names.count="Count",lag=TRUE,multitype=FALSE,marks=NULL)
{# {{{
stat <- data[,status]

## also allowing marks  when multitype=TRUE
if (multitype & is.null(marks))  marks <- rep(1,nrow(data))

clusters <- data[,id]
if (is.numeric(clusters)) {
      clusters <- fast.approx(unique(clusters), clusters) - 1
      max.clust <- max(clusters)
}
else {
     max.clust <- length(unique(clusters))
     clusters <- as.integer(factor(clusters, labels = seq(max.clust))) - 1
}

data[,"lbnr__id"] <- cumsumstrata(rep(1,nrow(data)),clusters,max.clust+1) 
if (!multitype) {
for (i in types)  {
if (lag==TRUE)
data[,paste(names.count,i,sep="")] <- 
   cumsumidstratasum((stat==i),rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,paste(names.count,i,sep="")] <- 
   cumsumidstratasum((stat==i),rep(0,nrow(data)),1,clusters,max.clust+1)$sum 
}
} else {
if (lag==TRUE)
data[,paste(names.count,types[1],sep="")] <- 
   cumsumidstratasum((stat %in% types)*marks,rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,paste(names.count,types[1],sep="")] <- 
   cumsumidstratasum((stat %in% types)*marks,rep(0,nrow(data)),1,clusters,max.clust+1)$sum 
}

return(data)
}# }}}

count_historyVar <- function(data,var="status",id="id",names.count="Count",lag=TRUE)
{# {{{
vvar <- data[,var]

clusters <- data[,id]
if (is.numeric(clusters)) {
      clusters <- fast.approx(unique(clusters), clusters) - 1
      max.clust <- max(clusters)
}
else {
     max.clust <- length(unique(clusters))
     clusters <- as.integer(factor(clusters, labels = seq(max.clust))) - 1
}

data[,"lbnr__id"] <- cumsumstrata(rep(1,nrow(data)),clusters,max.clust+1) 
if (lag==TRUE)
data[,names.count] <- 
   cumsumidstratasum(vvar,rep(0,nrow(data)),1,clusters,max.clust+1)$lagsum 
   else 
data[,names.count] <- 
   cumsumidstratasum(vvar,rep(0,nrow(data)),1,clusters,max.clust+1)$sum 

return(data)
}# }}}

##' Estimate the probability of exceeding k recurrent events by time t
##'
##' Estimates \eqn{P(N(t) \geq k)} as a function of time \eqn{t}, for a range of
##' thresholds \eqn{k}, in the presence of a terminal event (death). The estimator
##' is based on the cumulative incidence of "reaching \eqn{k} events", treating
##' death as a competing risk. Confidence intervals are computed on the log or
##' plain scale.
##'
##' For each threshold \eqn{k} in \code{exceed}, the function identifies the
##' first time each subject reaches \eqn{k} events, then fits a competing risks
##' model (\code{\link{cif}}) with "reaching \eqn{k} events" as the event of
##' interest and death as the competing event. Strata are supported. When
##' \code{marks} is \code{NULL}, each event contributes equally; otherwise events
##' are weighted by their mark values before cumulative counts are formed.
##'
##' @param formula A formula with an \code{\link{Event}} response giving the
##'   exit time and status (and optionally entry time). The right-hand side may
##'   include \code{cluster()} and \code{strata()}.
##' @param data A data frame containing all variables in \code{formula}.
##' @param cause Integer code for the recurrent event of interest. Default is
##'   \code{1}.
##' @param death.code Integer code for the terminal event. Default is \code{2}.
##' @param cens.code Integer code for censoring. Default is \code{0}.
##' @param exceed Integer vector of thresholds \eqn{k} to evaluate. If
##'   \code{NULL} (default), all observed cumulative counts are used.
##' @param marks Optional numeric vector of event weights. If non-\code{NULL},
##'   cumulative counts are formed as weighted sums of events rather than simple
##'   counts. Must have the same length as \code{nrow(data)}.
##' @param all.cifs Logical. If \code{TRUE}, the fitted \code{cif} object for
##'   each threshold is returned in \code{cif.exceed}. Default is \code{FALSE}.
##' @param return.data Logical. If \code{TRUE}, the constructed dataset for each
##'   threshold is returned in \code{dataList}. Default is \code{FALSE}.
##' @param conf.type Type of confidence interval transformation: \code{"log"}
##'   (default) or \code{"plain"}.
##' @param level Confidence level. Default is \code{0.95}.
##' @param ... Further arguments passed to \code{\link{cif}}.
##'
##' @return An object of class \code{"exceed"} with the following components:
##'   \item{time}{Vector of evaluation time points.}
##'   \item{prob}{Array of dimension \code{(length(time), length(exceed) + 1,
##'     nstrata)} containing \eqn{P(N(t) \geq k)} for each threshold and stratum.
##'     The first column gives \eqn{P(N(t) < \text{exceed}[1])}.}
##'   \item{se.prob}{Standard errors of \code{prob}.}
##'   \item{lower, upper}{Pointwise confidence interval bounds.}
##'   \item{meanN}{Estimated mean number of events \eqn{E(N(t))} (single stratum
##'     only; \code{NULL} for stratified analyses).}
##'   \item{meanN2, varN}{Second moment and variance of \eqn{N(t)} (single
##'     stratum only).}
##'   \item{exceed}{Thresholds evaluated (excluding zero).}
##'   \item{cif.exceed}{List of fitted \code{cif} objects (if
##'     \code{all.cifs = TRUE}).}
##'   \item{dataList}{List of datasets for each threshold (if
##'     \code{return.data = TRUE}).}
##'   \item{nstrata, strata.levels, strata.name}{Stratification information.}
##'   Use \code{plot()} and \code{summary()} methods for visualisation and
##'   tabulation.
##'
##' @author Thomas Scheike
##'
##' @seealso \code{\link{recurrent_marginal}}, \code{\link{cif}}
##'
##' @references
##' Scheike, T. H., Eriksson, L., and Tribler, P. (2019). The mean, variance and
##' correlation for bivariate recurrent events with a terminal event.
##' \emph{Journal of the Royal Statistical Society, Series C}, 68(5).
##'
##' @examples
##' data(hfactioncpx12)
##' dtable(hfactioncpx12, ~status)
##'
##' oo <- prob_exceed_recurrent(Event(entry, time, status) ~ cluster(id),
##'                              hfactioncpx12, cause = 1, death.code = 2)
##' plot(oo)
##' summary(oo, times = c(1, 2, 5))
##' @export
prob_exceed_recurrent <- function(formula,data,cause=1,death.code=2,cens.code=0, exceed=NULL,marks=NULL,all.cifs=FALSE,
				  return.data=FALSE,conf.type=c("log","plain"),level=0.95,...)
{# {{{
    cifmets <- TRUE
    cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("marks","strata","offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    des.marks <- des$marks
    id      <- des$cluster
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    strata <- des$strata
    if (!is.null(strata))  {
      ns <- grep("strata",names(des$levels))
      strata.name  <-  names(des$levels)[ns]
    } else strata.name <- NULL
    ## no use of 
    pos.cluster <- pos.strata <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
  if (!is.null(des.marks) & is.null(marks))  marks <- des.marks
  marks.call <- marks
  if (is.null(marks)) marks <- rep(1,length(exit))
  ## }}}
  if (!is.null(strata)) {
	  nstrata <- nlevels(strata) 
          strata.levels <- levels(strata)
  } else { strata.levels <- NULL; nstrata <- 1}

 call.id <- id;
 conid <- construct_id(id,nrow(X),as.data=TRUE)
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 data$id__  <- id

 times <- NULL
 statusD <- (status %in% death.code)*1
 statusE <- (status %in% cause)*1
 if (sum(statusE)==0) stop("none of type events")
 if (!is.null(strata) & !cifmets) stop("strata only for cifmets=TRUE\n")
 allvars <- all.vars(formula)
 if (is.null(times)) times <- sort(unique(exit[statusE==1]))

 countE <- cumsumstrata(statusE*marks,id,nid)
 if (is.null(exceed)) exceed <- sort(unique(countE))
 w0 <- which(exceed==0)
 if (length(w0)>=1) exceed <- exceed[-w0]

 if (is.null(marks.call)) {
    mc <- max(countE)+1
    idcount <- id*mc+countE
    idcount <- cumsumstrata(rep(1,length(idcount)),idcount,mc*(nid+1))
 } else {
    ex <- outer(countE,exceed,">=")[,1,]
    exC <- apply(ex,2,cumsumstrata,id,nid)
    idcount <- exC
 }

if (!cifmets) {
  ## can not handle start stop and id, so uses MG standard errors 
  pp <- as.formula(paste("Hist(entry=",allvars[1],",",allvars[2],",statN)~+1",sep=""))
  ###rhs <- update(formula,-1~.)
  ###form <- as.formula(update.formula(rhs,pp))
  form <- as.formula(pp)
} else {
###  pp <- as.formula(paste("Event(",allvars[1],",",allvars[2],",statN)~.",sep=""))
  pp <- as.formula(paste("Event(",allvars[2],",statN)~.",sep=""))
  rhs <- update(formula,-1~.)
  form <- as.formula(update.formula(rhs,pp))
}

cif.exceed <- NULL
dataList <- NULL
if (all.cifs) cif.exceed <- list() 
if (return.data) dataList <- list() 
se.probs <- probs <- lower <- upper <- array(0,c(length(times),length(exceed)+1,nstrata))
i <- 1
for (n1 in exceed) {# {{{
	i <- i+1
	### first time that get to n1
        if (is.null(marks.call)) keep <- (countE<n1 ) | (countE==n1 & idcount==1) else keep <- exC[,i-1]<=1 
###	keep <- (countE<n1 ) | (firsttoE==n1)

	### status, censoring, get to n1, or die
        statN <- rep(0,nrow(data))
        if (is.null(marks.call)) statN[countE==n1] <- 1 else statN[exC[,i-1]==1] <- 1
	statN[statusD==1] <- 2
	statN <- statN[keep]
	dataN <- data[keep,]
	dataN$statN <- statN

        idN <- revcumsumstrata(rep(1,nrow(dataN)),dataN$id__,nid)
        dataN <- subset(dataN,idN==1)
	if (return.data) dataList[[i-1]] <- dataN

   pN1 <-  suppressWarnings(cif(form,data=dataN))
   if (all.cifs) cif.exceed[[i-1]] <- pN1

###   lpN1 <- basecumhaz(pN1)
   where <- predictCumhaz(c(0,pN1$times),times)
###  where <- fast.approx(c(0,pN1$times),times,type="left")
###   print(c(length(pN1$mu),length(pN1$strata),pN1$nstrata))
   if (length(pN1$mu)>=1) {
   cifs <- vecAllStrata(pN1$mu,pN1$strata,pN1$nstrata)
   se.cifs <- vecAllStrata(pN1$se.mu,pN1$strata,pN1$nstrata)
   probs[,i,] <- rbind(0,cifs)[where,]
   se.probs[,i,] <- rbind(0,se.cifs)[where,]

   lu <- conftype(probs[,i,],se.probs[,i,],conf.type=conf.type[1],
         	 restrict=c("prob"),conf.int=level)
   lower[,i,] <- lu$lower
   upper[,i,] <- lu$upper
   } else {
   probs[,i,] <- se.probs[,i,] <- lower[,i,] <- upper[,i,] <- 0
   }

   ## surviving first level
   if (i==2) { probs[,1,]  <- 1-probs[,2,]; 
	    se.probs[,1,] <- se.probs[,2,]; 
	    lower[,1,] <- 1-lower[,2,]; 
	    upper[,1,] <- 1-upper[,2,]; 
   }

}# }}}

if (is.null(marks.call) & nstrata==1) {
dp <- -t(apply(cbind(probs[,-1,1],0),1,diff))
meanN <- apply(probs[,-1,1,drop=FALSE],1,sum)
meanN2 <- apply(t(exceed^2 * t(dp)),1,sum)
varN <- meanN2-meanN^2
} else dp <- meanN <- meanN2 <- varN <- NULL
 
colnames(probs) <- c(paste("N<",exceed[1],sep=""),paste("exceed>=",exceed,sep=""))
colnames(se.probs) <- c(paste("N<",exceed[1],sep=""),paste("exceed>=",exceed,sep=""))

out <- list(time=times,times=times,prob=probs,se.prob=se.probs,meanN=meanN,
	  lower=lower,upper=upper,meanN2=meanN2,varN=varN,exceed=exceed[-1],formula=form,
 cif.exceed=cif.exceed,dataList=dataList,
 nstrata=nstrata,strata.levels=strata.levels,strata.name=strata.name)
class(out) <- "exceed"
return(out)
}# }}}

##' @export
plot.exceed <- function(x,types=NULL,se=1,where=0.6,legend=NULL,strata=NULL,...) { ## {{{ 
if (is.null(types)) types <- 1:ncol(x$prob)
if (is.null(strata)) stratas <- seq(x$nstrata) else stratas <- strata
for (j in stratas) {
matplot(x$time,x$prob[,types,j],type="s",xlab="time",ylab="probabilty",xlim=range(c(0,x$times)),...)
if (x$nstrata>1) title(main=paste(x$strata.name,"=",x$strata.levels[j],sep=""))
if (se==1) 
for (i in types) plotConfRegion(x$time, cbind(x$lower[,i,j],x$upper[,i,j]) ,col=i)
if (!is.null(where)) {
if (is.null(legend)) legend <- colnames(x$prob)[types]
legend(0,0.6,legend=legend,lty=types,col=types)
}
}
} ## }}}

##' @export
summary.exceed <- function(object,times=NULL,types=NULL,strata=NULL,...) { ## {{{ 

if (is.null(types)) types <- 1:ncol(object$prob)
if (is.null(strata)) stratas <- seq(object$nstrata) else stratas <- strata
out <- list()

for (j in stratas) {
prob <- cbind(object$time,object$prob[,types,j])
se <- cbind(object$time,object$se.prob[,types,j])
lower <- cbind(object$time,object$lower[,types,j])
upper <- cbind(object$time,object$upper[,types,j])
if (is.null(times)) 
outl <- list(prob=prob,se=se,lower=lower,upper=upper)
else {
   rows <- fast.approx(object$time,times,type="left")
   probt <- cbind(times,prob[rows,])
   set <-   cbind(times,se[rows,])
   lowert <- cbind(times,lower[rows,])
   uppert <- cbind(times,upper[rows,])
   outl <- list(prob=probt,se=set,lower=lowert,upper=uppert)
}
if (length(stratas)>1) out[[j]] <- outl else out <- outl 
}
if (length(stratas)>1) names(out) <- object$strata.levels[stratas]
return(out)
} ## }}}

##' @export
print.exceed  <- function(x,...) summary.exceed(x,...)

##' @export
biprobit.vector <- function(x,id,X=NULL,Z=NULL,
                            weights=NULL,
                            weights.fun=function(x) ifelse(any(x<=0), 0, max(x)),
                    ## weights.fun=function(x) { u <- min(x); ifelse(u==0,u,1/u) },
                    eqmarg=TRUE,add=NULL,control=list(),p=NULL,
                    pair.ascertained=FALSE,
                    marg=NULL,
                    cells,
                    ...) {

    if (is.factor(x)) x <- factor(x,labels=c(0,1))
    else x <- factor(x*1,levels=c(0,1))
    x <- factor(x,levels=c(0,1),labels=c(0,1))
    namX <- colnames(X)
    namZ <- colnames(Z)
    if (is.null(namX)) namX <- "(Intercept)"
    if (is.null(namZ)) namZ <- "(Intercept)"
    namZ <- paste("r:",namZ,sep="")
    resh <- function(x,...,nam,onecol=0,df=1) {
        x <- c(list(x),list(...))
        res <- mis <- c()
        for (i in seq_along(x)) {
            if (length(x[[i]])==0) {
                res <- c(res,list(NULL))
            } else {                
                 if (i%in%df) {
                    xx <- fast.reshape(x[[i]],id=id)
                } else {
                    xx <- as.matrix(fast.reshape(x[[i]],id=id))
                }
                if (i%in%onecol) {
                    xx <- xx[,seq(ncol(x[[i]])),drop=FALSE]
                    colnames(xx) <- colnames(x[[i]])
                }
                idx <- which(is.na(xx),arr.ind=TRUE)                
                if (length(idx)>0) mis <- c(mis,idx[,1])
                res <- c(res,list(xx))
            }
        }
        mis <- unique(mis)
        if (length(mis)>0) {
            for (i in seq_along(res)) {
                if (length(res[[i]])>0) res[[i]] <- res[[i]][-mis,,drop=FALSE]
            }
        }
        if (!missing(nam)) names(res) <- nam
        return(res)
    }
    DD <- resh(data.frame(x),X,Z,weights,nam=c("y","x","z","w"),onecol=3)
    Y00 <- matrix(c(0,0, 1,0, 0,1, 1,1),ncol=2,byrow=TRUE)
  if (is.null(X) && is.null(Z)) {
        pos <- factor(interaction(DD$y))
        ipos <- unique(as.numeric(pos))
        Tab <- rbind(as.vector(table(DD$y))); colnames(Tab) <- c("00","10","01","11")
        Y0 <- Y00
        NN <- as.vector(Tab)
        midx1 <- 1; midx2 <- 2
        X0 <- matrix(1,ncol=2,nrow=4)        
        Z0 <- matrix(1,ncol=1,nrow=4)
        namX <- "(Intercept)"
        namZ <- "r:(Intercept)"
    } else {
        pos2 <- fast.pattern(cbind(DD$x,DD$z))
        pos <- interaction(interaction(DD$y),pos2$group)
        XZ0 <- pos2$pattern; colnames(XZ0) <- c(colnames(DD$x),colnames(DD$z))
        NN2 <- unlist(by(DD$y,pos,nrow))            
        NN <- rep(0,4*nrow(XZ0))
        ipos <- unique(as.numeric(pos))
        NN[ipos] <- NN2[ipos]

        ## tt <- with(DD, by(y,as.list(as.data.frame(cbind(x,z))),FUN=function(x) as.vector(table(x[,1:2])),simplify=FALSE))
        ## XZ0 <- do.call("expand.grid",lapply(attributes(tt)$dimnames,as.numeric))
        ## rem <- which(unlist(lapply(tt,is.null)))
        ## if (length(rem)>0) {
        ##     tt <- tt[-rem]
        ##     XZ0 <- XZ0[-rem,,drop=FALSE]
        ## }
        ## NN <- Reduce("c",tt);
        ## Reduce("cbind",tt)
        suppressWarnings(Tab <- cbind(matrix(NN,ncol=4,byrow=TRUE),XZ0)); colnames(Tab)[1:4] <- c("00","10","01","11")
        
        Y0 <- matrix(rep(t(Y00),length(NN)/4),ncol=2,byrow=TRUE)
        if (is.null(X)) X0 <- matrix(1,ncol=2,nrow=nrow(Y0)) else {
            X0 <- as.matrix(XZ0[rep(seq(nrow(XZ0)),each=4),seq(ncol(X)*2),drop=FALSE])
        }
        if (is.null(Z)) Z0 <- matrix(1,ncol=1,nrow=nrow(Y0)) else {
            Z1 <- as.matrix(XZ0[,ncol(XZ0)-ncol(Z)+seq(ncol(Z)),drop=FALSE])
            Z0 <- Z1[rep(seq(nrow(XZ0)),each=4),,drop=FALSE]
            colnames(Z0) <- colnames(Z1)
        }
    }

    nx <- ncol(X0)/2
    midx1 <- seq(nx)
    midx2 <- midx1+nx
    midx <- seq(2*nx)
    blen <- ifelse(eqmarg,nx,2*nx)
    zlen <- ncol(Z0)      
    plen <- blen+zlen
    
    MyData <- ExMarg(Y0,X0,W0=NULL,NULL,
                     midx1=1,midx2=2,eqmarg=eqmarg,allmarg=FALSE,Z0)
 
    datanh <- function(r) 1/(1-r^2)
    dtanh <- function(z) 4*exp(2*z)/(exp(2*z)+1)^2
    vartr <- tanh
    dvartr <- dtanh; varitr <- atanh
    trname <- "tanh"; itrname <- "atanh"    
    varcompname <- "Tetrachoric correlation"    
    ##msg <- "Variance of latent residual sterm = 1 (standard probit link)"
    msg <- NULL
    model <- list(tr=vartr,name=trname,inv=itrname,invname=itrname,deriv=dvartr,varcompname=varcompname,eqmarg=eqmarg,blen=blen,zlen=zlen,...)
    SigmaFun <- function(p,Z=MyData$Z0,cor=TRUE,...) {
        if (!cor) {
            r <- vartr(p[1])
            Sigma <- matrix(c(1,r,r,1),2)
            attributes(Sigma)$dvartr <- dvartr
            return(Sigma)
        }
        val <- Z%*%p
        dr <- apply(Z,2,function(x) x*dvartr(val))
        structure(list(rho=vartr(val),lp=val,drho=dr),dvartr=dvartr,vartr=vartr)
    }


    if (length(weights)<2) {
        w0 <- NN
    } else {
        w1 <- apply(DD$w,1, weights.fun)
        w2 <- unlist(by(w1,pos,sum))
        w0 <- rep(0,4*nrow(Tab))
        w0[ipos] <- w2[ipos]
    }

    if (!missing(cells)) {
        idx <- unlist(apply(MyData$Y0,1,function(x) all(x==cells)))
        w0[!idx] <- 0
    }
    if (!is.null(control$start)) {
        p0 <- control$start
        control$start <- NULL
    } else {
        p0 <- rep(0,plen)
        events <- Tab[,2]+Tab[,3]+2*Tab[,4]
        totals <- rowSums(Tab[,1:4,drop=FALSE])
        if (is.null(X)) xx1 <- rep(1,length(totals)) else  xx1 <- Tab[,midx1+4,drop=FALSE]
        b1 <- glm(cbind(events,totals)~-1+xx1,family=binomial("probit"))
        p0[midx1] <- coef(b1)
        if (!eqmarg) {
            if (is.null(X)) xx2 <- rep(1,length(totals)) else  xx2 <- Tab[,midx2+4,drop=FALSE]
            b2 <- glm(cbind(events,totals)~-1+xx2,family=binomial("probit"))
            p0[midx2] <- coef(b2)
        }     
    }

    U0 <- function(p) {
        MyData0 <- MyData
        MyData0$Y0[,] <- 0
        val0 <- Ubiprobit(p,SigmaFun,eqmarg,nx,MyData0,indiv=TRUE)
        P <- exp(attr(val0,"logLik"))
        res <- log(1-P)
        dlogP <- val0
        dres <- (-P/(1-P)) %x% rbind(rep(1,ncol(val0)))
        structure(res,score=dres)
    }
       
    U <- function(p,w0) {
        val <- Ubiprobit(p,SigmaFun,eqmarg,nx,MyData,indiv=TRUE)
        if (pair.ascertained) {
            ## log Pij - log (1-P00)
            ## U := Uij + dP00/(1-P00) = Uij+ DlogP00*P00/(1-P00)
            MyData0 <- MyData
            MyData0$Y0[,] <- 0
            dlogP00 <- Ubiprobit(p,SigmaFun,eqmarg,nx,MyData0,indiv=TRUE)
            logP00 <- attr(dlogP00,"logLik")
            P00 <- exp(logP00)
            ll00 <- log(1-P00)
            dll00 <- -(P00/(1-P00)) %x% rbind(rep(1,ncol(dlogP00)))
            dll00 <- dll00 * dlogP00
            val1 <- val
            attr(val,"logLik") <- attr(val,"logLik") - ll00
            val <- val-dll00
        }
        logl <- w0*as.vector(attributes(val)$logLik)
        score <- apply(val,2,function(x) w0*x)
        return(structure(score,logLik=logl))
    }

    f0 <- function(p) -sum(attributes(U(p,w0))$logLik)
    ## f00 <- function(p) -sum(attributes(U(c(0,p),w0))$logLik)
    g0 <- function(p) -as.numeric(colSums(U(p,w0)))

    if (!is.null(p)) op <- list(par=p)
    else {
        suppressWarnings(op <- nlminb(p0,f0,gradient=g0,control=control))
    }
    
    iI <- Inverse(numDeriv::jacobian(g0,op$par))
    V <- iI
    UU <- U(op$par,w0)
    logLik <- sum(attributes(UU)$logLik)
    UU <- U(op$par,1)    
    if (length(weights)>1) {
        UU <- apply(UU[pos,],2,function(x) w1*x)
    } else {
        UU <- UU[pos,]
    }
    meat <- crossprod(UU)
    if (length(weights>1)) V <- iI%*%meat%*%iI
    
    mycall <- match.call()    
    cc <- cbind(op$par,sqrt(diag(V)))
    cc <- cbind(cc,cc[,1]/cc[,2],2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
    colnames(cc) <- c("Estimate","Std.Err","Z","p-value")
    if (!eqmarg)
        rownames(cc) <- c(paste(namX,rep(c(1,2),each=length(namX)),sep="."),
                          namZ)
    else
    rownames(cc) <- c(namX,namZ)
    rownames(V) <- colnames(V) <- rownames(cc)

    npar <- list(intercept=1,
                 pred=blen-2+eqmarg)
    if (!eqmarg) npar <- lapply(npar,function(x) x*2)
    npar$var <- zlen
    N <- with(MyData, c(pairs=sum(NN)))
    
    val <- c(list(coef=cc,
                  N=N,
                  vcov=V,
                  bread=iI,
                  score=rbind(UU),
                  logLik=logLik,
                  opt=op,
                  call=mycall,
                  model=model,
                  msg=msg,
                  table=Tab,
                  npar=npar,
                  SigmaFun=SigmaFun),add)
    class(val) <- "biprobit"
    return(val)
}


##' Bivariate Probit model
##' 
##' @export
##' @aliases biprobit biprobit.vector biprobit.time
##' @param x formula (or vector)
##' @param data data.frame
##' @param id The name of the column in the dataset containing the cluster id-variable.
##' @param rho Formula specifying the regression model for the dependence parameter
##' @param num Optional name of order variable
##' @param strata Strata
##' @param eqmarg If TRUE same marginals are assumed (exchangeable)
##' @param indep Independence
##' @param weights Weights
##' @param weights.fun Function defining the bivariate weight in each cluster
##' @param randomeffect If TRUE a random effect model is used (otherwise correlation parameter is estimated allowing for both negative and positive dependence)
##' @param vcov Type of standard errors to be calculated
##' @param pairs.only Include complete pairs only?
##' @param allmarg Should all marginal terms be included
##' @param control Control argument parsed on to the optimization routine. Starting values may be parsed as '\code{start}'.
##' @param messages Control amount of messages shown 
##' @param constrain Vector of parameter constraints (NA where free). Use this to set an offset.
##' @param table Type of estimation procedure
##' @param p Parameter vector p in which to evaluate log-Likelihood and score function
##' @param ... Optional arguments
##' @examples
##' data(prt)
##' prt0 <- subset(prt,country=="Denmark")
##' a <- biprobit(cancer~1+zyg, ~1+zyg, data=prt0, id="id")
##' b <- biprobit(cancer~1+zyg, ~1+zyg, data=prt0, id="id",pairs.only=TRUE)
##' predict(b,newdata=lava::Expand(prt,zyg=c("MZ")))
##' predict(b,newdata=lava::Expand(prt,zyg=c("MZ","DZ")))
##' 
##' \donttest{ ## Reduce Ex.Timings
##' n <- 2e3
##' x <- sort(runif(n, -1, 1))
##' y <- rmvn(n, c(0,0), rho=cbind(tanh(x)))>0
##' d <- data.frame(y1=y[,1], y2=y[,2], x=x)
##' dd <- fast.reshape(d)
##' 
##' a <- biprobit(y~1+x,rho=~1+x,data=dd,id="id")
##' summary(a, mean.contrast=c(1,.5), cor.contrast=c(1,.5))
##' with(predict(a,data.frame(x=seq(-1,1,by=.1))), plot(p00~x,type="l"))
##' 
##' pp <- predict(a,data.frame(x=seq(-1,1,by=.1)),which=c(1))
##' plot(pp[,1]~pp$x, type="l", xlab="x", ylab="Concordance", lwd=2, xaxs="i")
##' lava::confband(pp$x,pp[,2],pp[,3],polygon=TRUE,lty=0,col=lava::Col(1))
##' 
##' pp <- predict(a,data.frame(x=seq(-1,1,by=.1)),which=c(9)) ## rho
##' plot(pp[,1]~pp$x, type="l", xlab="x", ylab="Correlation", lwd=2, xaxs="i")
##' lava::confband(pp$x,pp[,2],pp[,3],polygon=TRUE,lty=0,col=lava::Col(1))
##' with(pp, lines(x,tanh(-x),lwd=2,lty=2))
##' 
##' xp <- seq(-1,1,length.out=6); delta <- mean(diff(xp))
##' a2 <- biprobit(y~1+x,rho=~1+I(cut(x,breaks=xp)),data=dd,id="id")
##' pp2 <- predict(a2,data.frame(x=xp[-1]-delta/2),which=c(9)) ## rho
##' lava::confband(pp2$x,pp2[,2],pp2[,3],center=pp2[,1])
##' 
##' 
##' }
##' 
##' ## Time
##' \dontrun{
##'     a <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, eqmarg=TRUE,
##'                        cens.formula=Surv(time,status==0)~1,
##'                        breaks=seq(75,100,by=3),fix.censweights=TRUE)
##' 
##'     a <- biprobit.time2(cancer~1+zyg, rho=~1+zyg, id="id", data=prt0, eqmarg=TRUE,
##'                        cens.formula=Surv(time,status==0)~zyg,
##'                        breaks=100)
##' 
##'     #a1 <- biprobit.time2(cancer~1, rho=~1, id="id", data=subset(prt0,zyg=="MZ"), eqmarg=TRUE,
##'     #                   cens.formula=Surv(time,status==0)~1,
##'     #                   breaks=100,pairs.only=TRUE)
##' 
##'     #a2 <- biprobit.time2(cancer~1, rho=~1, id="id", data=subset(prt0,zyg=="DZ"), eqmarg=TRUE,
##'     #                    cens.formula=Surv(time,status==0)~1,
##'     #                    breaks=100,pairs.only=TRUE)
##'}
biprobit <- function(x, data, id, rho=~1, num=NULL, strata=NULL, eqmarg=TRUE,
                     indep=FALSE, weights=NULL, 
                     weights.fun=function(x) ifelse(any(x<=0), 0, max(x)),
                     randomeffect=FALSE, vcov="robust",
                     pairs.only=FALSE,                             
                     allmarg=!is.null(weights),
                     control=list(trace=0),
                     messages=1, constrain=NULL,
                     table=pairs.only,
                     p=NULL,
                     ...) {

  mycall <- match.call()

  formulaId <- unlist(Specials(x,"cluster"))
  if (is.null(formulaId)) {
    formulaId <- unlist(Specials(x,"id"))
  }
  ##  formulaOffset <- unlist(Specials(x,"offset"))
  formulaStrata <- unlist(Specials(x,"strata"))
  formulaSt <- paste("~.-cluster(",formulaId,")",
                     "-id(",formulaId,")",
                     "-strata(",paste(formulaStrata,collapse="+"),")",sep="")
  formula <- update(x,formulaSt)

  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) strata <- formulaStrata
  mycall$x <- formula
  if (!is.null(strata)) {
    dd <- split(data,interaction(data[,strata]))
    fit <- lapply(seq(length(dd)),function(i) {
      if (messages>0) message("Strata '",names(dd)[i],"'")
      mycall$data <- dd[[i]]
      eval(mycall)
    })
    res <- list(model=fit)
    res$strata <- names(res$model) <- names(dd)
    class(res) <- c("twinlm.strata","biprobit")
    res$coef <- unlist(lapply(res$model,coef))
    res$vcov <- blockdiag(lapply(res$model,vcov.biprobit))
    res$N <- length(dd)
    res$idx <- seq(length(coef(res$model[[1]])))
    rownames(res$vcov) <- colnames(res$vcov) <- names(res$coef)
    return(res)
  }

  if (missing(id)) {    
    if (!is.null(weights)) {
      weights <- data[,weights]
      return(glm(formula,data=data,family=binomial(probit),weights=weights,...))
    }
    return(glm(formula,data=data,family=binomial(probit),...))    
  }

  yx <- getoutcome(formula)

  ids <- table(data[,id])
  if (all(ids==2)) pairs.only <- TRUE
  if (pairs.only) {
      X <- Z <- NULL
      zf <- getoutcome(rho); if (length(attr(zf,"x"))>0) Z <- model.matrix(rho,data);
      if (table && NCOL(Z)<10 && length(unique(sample(Z,min(1000,length(Z)))))<10) { ## Not quantitative?
          if (!is.null(weights)) weights <- data[,weights]
          if (length(attr(yx,"x")>0)) X <- model.matrix(x,data);
          return(biprobit.vector(data[,yx],X=X,Z=Z,id=data[,id],
                                 weights,weights.fun=weights.fun,eqmarg=eqmarg,
                                 add=list(formula=formula,rho.formula=rho),
                                 control=control,p=p,...))
      }
  }
  
  mycall <- match.call()
  DD <- procdatabiprobit(formula,data,id,num=num,weights=weights,pairs.only=pairs.only,rho,...)
  rnames1 <- DD$rnames1

  nx <- length(rnames1)
  ## if (nx==0) stop("Zero design not allowed")  
  midx1 <- seq(nx)
  midx2 <- midx1+nx
  midx <- seq(2*nx)
  blen <- ifelse(eqmarg,nx,2*nx)
  zlen <- ncol(DD$Z0)      
  plen <- blen+zlen
  
  datanh <- function(r) 1/(1-r^2)
  dtanh <- function(z) 4*exp(2*z)/(exp(2*z)+1)^2
  vartr <- tanh
  dvartr <- dtanh; varitr <- atanh
  trname <- "tanh"; itrname <- "atanh"    
  Sigma1 <- diag(2)  
  Sigma2 <- matrix(c(0,1,1,0),2,2)
  dS0 <- rbind(c(0,1,1,0))
  
  varcompname <- "Tetrachoric correlation"
  msg <- NULL
  if (randomeffect) msg <- "Variance of latent residual term = 1 (standard probit link)"
  if (randomeffect) {
      dS0 <- rbind(rep(1,4))
      vartr <- dvartr <- exp; inv <- log
      trname <- "exp"; itrname <- "log"
      Sigma2 <- 1
      varcompname <- NULL
  }
  model <- list(tr=vartr,name=trname,inv=itrname,invname=itrname,deriv=dvartr,varcompname=varcompname,dS=dS0,eqmarg=eqmarg,randomeffect=randomeffect,blen=blen,zlen=zlen)

  MyData <- with(DD,ExMarg(Y0,XX0,W0,dS0,midx1,midx2,eqmarg=eqmarg,allmarg=allmarg,Z0,id=id))
  if (pairs.only) {
    MyData$Y0_marg <- NULL
  }
  if (!is.null(weights)) {
      MyData$W0 <- cbind(apply(MyData$W0,1,weights.fun))
      if (!is.null(MyData$Y0_marg)) {
          MyData$W0_marg <- cbind(apply(MyData$W0_marg,1,weights.fun))
      }
  }

  SigmaFun <- function(p,Z=MyData$Z0,cor=!randomeffect,...) {
      if (!cor) {
          r <- vartr(p[1])
          Sigma <- matrix(c(1,r,r,1),2)
          if (indep) Sigma <- diag(2)
          attributes(Sigma)$dvartr <- dvartr
          return(Sigma)
      }
      val <- Z%*%p
      dr <- apply(Z,2,function(x) x*dvartr(val))
      structure(list(rho=vartr(val),lp=val,drho=dr),dvartr=dvartr,vartr=vartr)
  }

  U <- function(p,indiv=FALSE) {
      gamma <- p[seq(zlen)+blen]
      ##if (bound) gamma <- min(gamma,20)
      Sigma <- SigmaFun(gamma)
      if (randomeffect) {
          lambda <- eigen(Sigma)$values
          if (any(lambda<1e-12 | lambda>1e9)) stop("Variance matrix out of bounds")
      }
      Mu_marg <- NULL
      if (eqmarg) {
          B <- cbind(p[midx1])
          Mu <- with(MyData,
                     cbind(XX0[,midx1,drop=FALSE]%*%B,XX0[,midx2,drop=FALSE]%*%B))     
          if (!is.null(MyData$Y0_marg)) 
              Mu_marg <- with(MyData, XX0_marg%*%B)
      } else {
          B1 <- cbind(p[midx1])
          B2 <- cbind(p[midx2])
          Mu <- with(MyData,
                     cbind(XX0[,midx1,drop=FALSE]%*%B1,XX0[,midx2,drop=FALSE]%*%B2))
          if (!is.null(MyData$Y0_marg))
              Mu_marg <- with(MyData, rbind(X0_marg1%*%B1,X0_marg2%*%B2))
      }

      if (randomeffect) {
          U <- with(MyData, .Call("biprobit2",
                                 Mu,XX0,
                                 Sigma,dS0*attributes(Sigma)$dvartr(p[plen]),Y0,W0,
                                 !is.null(W0),TRUE,eqmarg,FALSE,
                                 PACKAGE="mets"))
      } else {
          U <- with(MyData, .Call("biprobit2",
                                 Mu,XX0,
                                 Sigma$rho,Sigma$drho,
                                 Y0,W0,
                                 !is.null(W0),TRUE,eqmarg,TRUE,
                                 PACKAGE="mets"))
      }
      
      if (!is.null(MyData$Y0_marg)) {
          if (randomeffect) {
              U_marg <- with(MyData, .Call("uniprobit",
                                          Mu_marg,XX0_marg,
                                          Sigma[1,1],dS0_marg*attributes(Sigma)$dvartr(p[plen]),Y0_marg,
                                          W0_marg,!is.null(W0_marg),TRUE,
                                          PACKAGE="mets"))
          } else {
              U_marg0 <- matrix(0,length(MyData$Y0_marg),ncol=plen)
              U_marg <- with(MyData, .Call("uniprobit",
                                          Mu_marg,XX0_marg,
                                          1,matrix(ncol=0,nrow=0),Y0_marg,
                                          W0_marg,!is.null(W0_marg),TRUE,
                                          PACKAGE="mets"))
              U_marg0[,seq(blen)] <-  U_marg[[1]]
              U_marg[[1]] <- U_marg0
          }
          U$score <- rbind(U$score,U_marg$score)
          U$loglik <- c(U$loglik,U_marg$loglik)
      }
      
      if (indiv) {
          val <- U$score
          if (!is.null(MyData$idmarg) && !pairs.only) {
              val <- with(MyData, cluster.index(c(id,idmarg),mat=U$score))
          }          
          ## val <- U$score[MyData$id,,drop=FALSE]
          ## N <- length(MyData$id)
          ## idxs <- seq_len(N)
          ## for (i in seq_len(N)) {
          ##     idx <- which((MyData$idmarg)==(MyData$id[i]))+N
          ##     idxs <- c(idxs,idx)
          ##     val[i,] <- val[i,]+colSums(U$score[idx,,drop=FALSE])
          ## }      
          ## val <- rbind(val, U$score[-idxs,,drop=FALSE])
          attributes(val)$logLik <- U$loglik
          return(val)
      }
      val <- colSums(U$score)
      attributes(val)$logLik <- sum(U$loglik)
      return(val)
  }  

  p0 <- rep(0,plen)  
  if (!is.null(control$start)) {
    p0 <- control$start
    control$start <- NULL
  } else {
      g <- suppressWarnings(glm(formula,data,family=binomial(probit)))
      p0[midx1] <- coef(g)
      if (!eqmarg) p0[midx2] <- coef(g)    
  }

  f <- function(p) crossprod(U(p))[1]
  f0 <- function(p) -sum(attributes(U(p))$logLik)
  g0 <- function(p) -as.numeric(U(p))
  h0 <- function(p) crossprod(U(p,indiv=TRUE))

  if (!is.null(constrain)) {
      if (length(constrain)!=length(p0)) stop("Wrong length of constraints (should be NA at positions not to be fixed)")
      fix <- which(!is.na(constrain))
      free <- which(is.na(constrain))
      p0 <- p0[free]
      U0 <- U
      U <- function(p,indiv=FALSE) {
          p1 <- constrain
          p1[free] <- p
          res <- U0(p1,indiv)
          if (is.matrix(res)) {              
              return(structure(res[,free,drop=FALSE],logLik=attributes(res)$logLik))
          } 
          return(structure(res[free],logLik=attributes(res)$logLik))
      }
  }

  if (is.null(control$method)) {
    control$method <- "quasi"
  }
  control$method <- tolower(control$method)
  
  if (is.null(p)) {
      if (control$method=="score") {
          control$method <- NULL
          op <- nlminb(p0,f,control=control,...)
      } else if (control$method=="quasi") {
          control$method <- NULL
          suppressWarnings(op <- nlminb(p0,f0,gradient=g0,control=control))
          ## }
  ## else if (control$method=="bhhh") {
  ##   controlnr <- list(stabil=FALSE,
  ##                     gamma=0.1,
  ##                     gamma2=1,
  ##                     ngamma=5,
  ##                     iter.max=200,
  ##                     epsilon=1e-12,
  ##                     tol=1e-9,
  ##                     trace=1,
  ##                     stabil=FALSE)
  ##   controlnr[names(control)] <- control
  ##   op <- lava:::NR(start=p0,NULL,g0, h0,control=controlnr)
      } else {
          control$method <- NULL
          op <- nlminb(p0,f0,control=control)
      }
  } else op <- list(par=p)

    UU <- U(op$par,indiv=TRUE)
    idx <- seq(nrow(UU))
    if (!is.null(MyData$idmarg))
        idx <- with(MyData,cluster.index(c(id,idmarg)))$firstclustid+1
    idvar <- with(MyData, c(id0,idmarg0))[idx] 
       
  J <- crossprod(UU)
  ##  iJ <- Inverse(J)
  iI <- Inverse(-numDeriv::jacobian(U,op$par))
  V <- switch(vcov,
              robust=,
              sandwich=iI%*%J%*%iI,##iJ%*%I%*%iJ,
              score=,
              outer=Inverse(J),
              hessian=iI              
              )

  cc <- cbind(op$par,sqrt(diag(V)))
  cc <- cbind(cc,cc[,1]/cc[,2],2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
  if (!is.null(constrain)) {
      cc0 <- matrix(NA,nrow=length(constrain),ncol=4)
      cc0[free,] <- cc
      cc0[fix,1] <- constrain[fix]
      cc <- cc0
      V0 <- matrix(0,nrow=length(constrain),ncol=length(constrain))
      V0[free,free] <- V
      V <- V0
  }
  colnames(cc) <- c("Estimate","Std.Err","Z","p-value")

  p1 <- "("; p2 <- ")"
  if (itrname=="log") rhonam <- "U" else {
      rhonam <- DD$znames
      p1 <- p2 <- ""; itrname <- "r:" 
  }
  if (!eqmarg)
    rownames(cc) <- c(paste(rnames1,rep(c(1,2),each=length(rnames1)),sep="."),
                      paste(itrname,p1,rhonam,p2,sep=""))
  else
    rownames(cc) <- c(rnames1,paste(itrname,p1,rhonam,p2,sep=""))
  rownames(V) <- colnames(V) <- rownames(cc)

  npar <- list(intercept=attributes(terms(formula))$intercept,
              pred=nrow(attributes(terms(formula))$factor)-1)
  if (!eqmarg) npar <- lapply(npar,function(x) x*2)
    npar$var <- 1##nrow(cc)-sum(unlist(npar))
  N <- with(MyData, c(n=nrow(XX0)*2+length(margidx), pairs=nrow(XX0)))
  val <- list(coef=cc,N=N,vcov=V,bread=iI,score=UU,logLik=attributes(UU)$logLik,opt=op, call=mycall, model=model,msg=msg,npar=npar,
              SigmaFun=SigmaFun,rho.formula=rho,formula=formula,constrain=constrain,
              id=idvar)
  class(val) <- "biprobit"
  return(val)
}



procdatabiprobit <- function(formula,data,id,num=NULL,weights=NULL,pairs.only=FALSE,rho=~1,...) {

    data <- data[order(data[,id]),]
    idtab <- table(data[,id])
    if (pairs.only) {
        data <- data[which(as.character(data[,id])%in%names(idtab)[idtab==2]),]
        idtab <- table(data[,id])
    }

    ff <- paste(as.character(formula)[3],"+",
                paste(c(id,num),collapse="+"))
    yvar <- paste(deparse(formula[[2]]),collapse="")
    if (!is.null(weights))
        ff <- paste(weights,"+",ff)
    ff <- paste("~",yvar,"+",ff)

    if (is.logical(data[,yvar])) data[,yvar] <- data[,yvar]*1
    if (is.factor(data[,yvar])) data[,yvar] <- as.numeric(data[,yvar])-1  

    formula0 <- as.formula(ff)  
    opt <- options(na.action="na.pass")
    Data <- model.matrix(formula0,data)
    options(opt)
    rnames1 <- setdiff(colnames(Data),c(yvar,num,id,weights))
    X0 <- as.matrix(Data[,rnames1])

    ex <- 1+!is.null(num)
    rho <- update(rho,paste("~.+",
                           paste(c(id,num),collapse="+")))
    Z0 <- model.matrix(rho,data);
    znames <- setdiff(colnames(Z0),c(id,num))
    znames1 <- paste(znames,1,sep="")
    Z0 <- as.matrix(subset(fast.reshape(Z0,id=id),select=znames1))
    colnames(Z0) <- znames1
    Wide <- fast.reshape(as.data.frame(Data),id=id,num=num,sep=".",labelnum=TRUE)
    
    W0 <- NULL
    yidx <- paste(yvar,1:2,sep=".")
    rmidx <- c(id,yidx)
    if (!is.null(weights)) {
        W <- cbind(data[,weights])
        widx <- paste(weights,1:2,sep=".")
        W0 <- as.matrix(Wide[,widx])
        rmidx <- c(rmidx,widx)
    }   
    Y0 <- as.matrix(Wide[,yidx])
    XX0 <- as.matrix(Wide[,setdiff(colnames(Wide),rmidx)])
    XX0[is.na(XX0)] <- 0
    list(Y0=Y0,XX0=XX0,W0=W0,Z0=Z0,znames=znames,rnames1=rnames1,id=Wide[,id])
}



Ubiprobit <- function(p,Rho,eqmarg,nx,MyData,indiv=FALSE) {
    midx1 <- seq(nx)
    midx2 <- midx1+nx
    midx <- seq(2*nx)
    blen <- ifelse(eqmarg,nx,2*nx)
    zlen <- length(p)-blen
    plen <- blen+zlen
    gamma <- p[blen+seq(zlen)]
    Sigma <- Rho(gamma)
    ## lambda <- eigen(Sigma)$values
    ## if (any(lambda<1e-12 | lambda>1e9)) stop("Variance matrix out of bounds")
    Mu_marg <- NULL
    if (eqmarg) {
        B <- cbind(p[midx1])
        Mu <- with(MyData,
                   cbind(XX0[,midx1,drop=FALSE]%*%B,XX0[,midx2,drop=FALSE]%*%B))     
        ##      Mu <- with(MyData, matrix(X0%*%B,ncol=2,byrow=TRUE))
        if (!is.null(MyData$Y0_marg))
            Mu_marg <- with(MyData, XX0_marg%*%B)
    } else {
        B1 <- cbind(p[midx1])
        B2 <- cbind(p[midx2])
        Mu <- with(MyData,
                   cbind(XX0[,midx1,drop=FALSE]%*%B1,XX0[,midx2,drop=FALSE]%*%B2))
        if (!is.null(MyData$Y0_marg))
            Mu_marg <- with(MyData, rbind(X0_marg1%*%B1,X0_marg2%*%B2))
    }

    U <- with(MyData, .Call("biprobit2",
                           Mu,XX0,
                           Sigma$rho,Sigma$drho,Y0,W0,
                           !is.null(W0),TRUE,eqmarg,TRUE,
                           PACKAGE="mets"))
    
    if (!is.null(MyData$Y0_marg)) {
        U_marg0 <- matrix(0,length(MyData$Y0_marg),ncol=plen)
        U_marg <- with(MyData, .Call("uniprobit",
                                    Mu_marg,XX0_marg,
                                    1,matrix(ncol=0,nrow=0),Y0_marg,
                                    W0_marg,!is.null(W0_marg),TRUE,
                                    PACKAGE="mets"))
        U_marg0[,seq(blen)] <-  U_marg[[1]]
        U_marg[[1]] <- U_marg0
        
        U$score <- rbind(U$score,U_marg$score)
        U$loglik <- c(U$loglik,U_marg$loglik)
    }

    if (indiv) {
        val <- with(MyData, cluster.index(c(id,idmarg),mat=U$score))
        attributes(val)$logLik <- U$loglik
        return(val)
    }
    val <- colSums(U$score)
    attributes(val)$logLik <- sum(U$loglik)
    return(val)
}

##' IPTW GLM, Inverse Probaibilty of Treatment Weighted GLM  
##'
##' Fits GLM model with treatment weights \deqn{ w(A)= \sum_a I(A=a)/P(A=a|X) }, computes
##' standard errors via influence functions that are returned as the IID argument. 
##' Propensity scores are fitted using either logistic regression (glm) or the multinomial model (mlogit) when more
##' than two categories for treatment. The treatment needs to be a factor and is identified on the rhs
##' of the "treat.model". 
##'
##' Also works with cluster argument. 
##'
##' @param formula for glm 
##' @param data data frame for risk averaging
##' @param treat.model propensity score model (binary or multinomial) 
##' @param family of glm (logistic regression)
##' @param id cluster id for standard errors 
##' @param weights may be given, and then uses weights*w(A) as the weights
##' @param estpr to estimate propensity scores and get infuence function contribution to uncertainty
##' @param pi0 fixed simple weights 
##' @param ...  arguments for glm call
##' @author Thomas Scheike
##' @export
glm_IPTW <- function(formula,data,treat.model=NULL,family=binomial(),id=NULL,weights=NULL,estpr=1,pi0=0.5,...) {# {{{
  if (!is.null(id)) {
          orig.id <- id
	  ids <- sort(unique(id))
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
  } else { orig.id <- NULL; nid <- nrow(data); id <- 0:(nid-1); ids <- NULL}
  ### id from call coded as numeric 1 -> 
  id <- id+1
  nid <- length(unique(id))
  data$id__  <-  id

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

fittreat <- function(treat.model,data,id,ntreatvar,nlev) {# {{{
if (nlev==2) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   Dp <- c()
   for (i in seq(nlev-1)) Dp <- cbind(Dp,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*Dp/pA^2

out <- list(iidalpha=iidalpha,pA=pA,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

expit <- function(x) 1/(1+exp(-x))

treat.name <-  all.vars(treat.model)[1]
treatvar <- data[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be coded as factor \n",sep="")); 

treats <- treats(treatvar)
if (estpr[1]==1) {
   fitt <- fittreat(treat.model,data,id,treats$ntreatvar,treats$nlev)
   iidalpha0 <- fitt$iidalpha
} else {
   ## assumes constant fixed prob over groups
   pi0 <- rep(pi0,treats$nlev)
}

wPA <- c(1/fitt$pA)
if (is.null(weights)) ww <- wPA else ww <- weights*wPA
data$ww <- ww

## fit the weighted model and bring the derivatives to bring them along
glmw <- suppressWarnings(glm(formula,data,weights=ww,family=family,...))
glm.iid <- lava::iid(glmw,id=id)
ihess <- attr(glm.iid,"bread")

check.derivative <- 0
### for checking derivative 
if (check.derivative==1) {
###fpar <- glm(treat.model,data,family=binomial)
###mm <- as.matrix(model.matrix(treat.model,data))
###cpar <- coef(fpar)
###library(numDeriv)
###X <- model.matrix(formula,data)
###formulaS <- Event(tt,status)~tcell+platelet+age.f
###ff <- function(par) {
###pa <- 	lava::expit(mm %*% par)
###www <- 1/ifelse(data$tcell== 1, pa, 1 - pa)
######print(summary(www))
###
###pp <- binreg(Event(tt,cause)~tcell+platelet+age.f,bmt,cause=1,time=5,cens.code=5,no.opt=TRUE,
###	     weights=www, beta=coef(glmw))
###gradient  <- pp$gradient
###return(gradient)
###}
###print(ff(cpar))
###gf <- jacobian(ff,cpar)
###print(gf)
}

### iid after propensity model 
### computing  derivatives 
if (estpr[1]==1) {
X <- model.matrix(formula,data)
res <- glmw$y -predict(glmw,data,type="response") 
XY <- X*res 
###XWY <- X*ww*res
###print( apply(XWY,2,sum))
DUa  <-  t(fitt$DPai) %*% XY  
###print(DUa)
iidpal <- iidalpha0 %*% DUa ## /nid
glmw$naive.var  <- crossprod(glm.iid)
glm.iid <-   glm.iid + iidpal %*% ihess
glmw$DUa <- DUa
} 

glmw$iid <- glm.iid
glmw$var  <-  crossprod(glm.iid)
## add some arguments to comply with summary.binreg
glmw$n <- nrow(data)
glmw$ncluster <- nid
glmw$nevent <- sum(glmw$y)
class(glmw) <- c("binreg","glm")

return(glmw)
}# }}}

##' @export
predictGLM <- function(object,newdata,id=NULL,fun=NULL,link.conf=TRUE,...) {# {{{
    tt <- terms(object)
    if (missing(newdata) || is.null(newdata)) {
        mm <- X <- model.matrix(object)
        mmDone <- TRUE
        offset <- object$offset
    }
    else {
        Terms <- delete.response(tt)
        m <- model.frame(Terms, newdata, xlev = object$xlevels)
        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
    }
    offsets <- rep(0,nrow(X))
    if (!is.null(object$offset)) offsets <- m[, grep("offset", names(m))] 

if (!is.null(object$family$linkinv)) linkinv <- object$family$linkinv  else linkinv <- function(x) x
f <- function(p) { pp <- X %*% p+offsets; return(linkinv(pp)); }
fl <- function(p) { pp <- X %*% p+offsets; return(pp); }
if (!is.null(fun))  f <- fun

if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)

if (link.conf) { 
     if (!is.null(id)) resl <- estimate(object,f=fl,id=id,...) else resl <- estimate(object,f=fl,...)
     res <- linkinv(resl$coefmat[,c(1,3,4)]) 
} else {
if (!is.null(id)) res <- estimate(object,f=f,id=id,...) else res <- estimate(object,f=f,...)
res <- res$coefmat
}

return(list(coef=coef,pred=res))
}
# }}}

##' Reporting OR (exp(coef)) from glm with binomial link and glm predictions
##' 
##' Reporting OR from glm with binomial link  and glm predictions
##' 
##' @param object glm output 
##' @param id possible id for cluster corrected standard errors
##' @param fun possible function for non-standard predictions based on object
##' @param ... arguments of estimate of lava for example level=0.95 
##' @author Thomas Scheike
##' @export
##' @examples
##' data(sTRACE)
##' sTRACE$id <- sample(1:100,nrow(sTRACE),replace=TRUE)
##'
##' model <- glm(I(status==9)~sex+factor(diabetes)+age,data=sTRACE,family=binomial)
##' summaryGLM(model)
##' summaryGLM(model,id=sTRACE$id)
##'
##' nd <- data.frame(sex=c(0,1),age=67,diabetes=1)
##' predictGLM(model,nd)
##' @aliases predictGLM 
summaryGLM <- function(object,id=NULL,fun=NULL,...) {# {{{

f <- function(p) { pp <- exp(pp); return(pp); }

if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)

if (!is.null(id)) resl <- estimate(object,id=id,...) else resl <- estimate(object,...)
res <- exp(resl$coefmat[,c(1,3,4)]) 

if (!is.null(fun))  {
if (!is.null(id)) resl <- estimate(object,id=id,f=fun,...) else resl <- estimate(object,f=fun,...)
fout <- resl$coefmat
} else fout <- NULL

return(list(coef=coef,or=res,fout=NULL))
}
# }}}

waldTest <- function(object,...)
{# {{{
Vars <-attr(terms(formula(object)),"term.labels")
## remove possible cluster term
clusterTerm<- grep("^cluster[(][A-z0-9._:]*",Vars,perl=TRUE)
if (length(clusterTerm)==1) Vars <- Vars[-clusterTerm]
Varf <- names(attr(attr(object$model,"terms"),"dataClasses"))[grep("factor",attr(attr(object$model,"terms"),"dataClasses"))]
txtVar<-names(coef(object))
###
test.matrix <- matrix(0,length(Vars),3)
colnames(test.matrix) <- c("Chisq","df","p.value")
rownames(test.matrix) <- Vars
k <- 1
for (ff in Vars) {
	pos <- grep(ff,txtVar,fixed=TRUE)
        wt <- estimate(object,as.list(pos),...)$compare
###	wt <- estimate(object,as.list(3:5))
	test.matrix[k,] <- unlist(wt[1:3])
	k <- k+1
}
return(test.matrix)
}# }}}

#' @keywords internal
"_PACKAGE"

##' Analysis of Multivariate Events
##'
##' Implementation of various statistical models for multivariate
##' event history data. Including multivariate cumulative incidence models,
##' and bivariate random effects probit models (Liability models)
##'
##' @name mets-package
##' @author Klaus K. Holst and Thomas Scheike
##' @useDynLib mets, .registration=TRUE
##' @import stats splines Rcpp mvtnorm
##' @importFrom lava iid estimate bootstrap compare score information
##'   twostage %++% %ni% addvar<- blockdiag cancel Col confband constrain<-
##'   constraints covariance covariance<- coxWeibull.lvm devcoords
##'   distribution<- endogenous eventTime Expand getoutcome gof intercept<-
##'   Inverse kill<- latent latent<- lava.options lvm Model multigroup
##'   parameter<- pars regression regression<- revdiag trim IC expit logit
##' @importFrom survival Surv is.Surv concordance strata cluster finegray
##' @importFrom timereg two.stage predict.two.stage
##' @importFrom utils head tail getS3method glob2rx capture.output
##' @importFrom graphics matplot lines plot polygon par points abline title
##'   matlines legend mtext layout axis barplot mosaicplot
##' @importFrom methods hasArg
##' @importFrom RcppArmadillo armadillo_set_number_of_omp_threads
##' @importFrom grDevices dev.list devAskNewPage dev.interactive
##' @aliases mets mets-package
##' @examples
##'
##' ## To appear
##'
NULL

##' @export
lava::iid

##' @export
lava::IC

##' @export
lava::twostage

##' @export
lava::estimate

##' @export
lava::gof

##' @export
survival::strata

##' @export
survival::Surv

##' @export
survival::cluster

##' np data set
##'
##' @name np
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Migraine data
##'
##' @name migr
##' @docType data
##' @keywords data
NULL

##' Dermal ridges data (families)
##'
##' Data on dermal ridge counts in left and right hand in (nuclear) families
##' @name dermalridges
##' @docType data
##' @keywords data
##' @format Data on 50 families with ridge counts in left and right
##' hand for moter, father and each child. Family id in 'family' and
##' gender and child number in 'sex' and 'child'.
##' @source Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
##' asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
##' pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
##' @examples
##' data(dermalridges)
##' fast.reshape(dermalridges,id="family",varying=c("child.left","child.right","sex"))
NULL

##' Dermal ridges data (monozygotic twins)
##'
##' Data on dermal ridge counts in left and right hand in (nuclear) families
##' @name dermalridgesMZ
##' @docType data
##' @keywords data
##' @format Data on dermal ridge counts (left and right hand) in 18
##' monozygotic twin pairs.
##' @source Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
##' asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
##' pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
##' @examples
##' data(dermalridgesMZ)
##' fast.reshape(dermalridgesMZ,id="id",varying=c("left","right"))
NULL

##' CALGB 8923, twostage randomization SMART design
##'
##' Data from CALGB 8923 
##' @name calgb8923 
##' @docType data
##' @keywords data
##' @format Data from smart design
##' id: id of subject
##' status : 1-death, 2-response for second randomization, 0-censoring
##' A0 : treament at first randomization
##' A1 : treament at second randomization
##' At.f : treament given at record (A0 or A1)
##' TR : time of response 
##' sex : 0-males, 1-females 
##' consent: 1 if agrees to 2nd randomization, censored if not
##' R: 1 if response 
##' trt1: A0
##' trt2: A1
##' @source  https://github.com/ycchao/code_Joint_model_SMART
##' @examples
##' data(calgb8923)
NULL

##' ACTG175, block randmized study from speff2trial package
##'
##' Data from speff2trial
##' @name ACTG175
##' @docType data
##' @keywords data
##' @format Randomized study 
##' @source  Hammer et al. 1996, speff2trial package.
##' @examples
##' data(ACTG175)
NULL

##' hfaction, subset of block randmized study HF-ACtion from WA package  
##'
##' Data from HF-action trial slightly modified from WA package,
##' consisting of 741 nonischemic patients with baseline
##' cardiopulmonary test duration less than or equal to 12 minutes.
##' @name hfactioncpx12
##' @docType data
##' @keywords data
##' @format Randomized study 
##' status : 1-event, 2-death, 0-censoring
##' treatment : 1/0
##' @source  WA package, Connor et al. 2009
##' @examples
##' data(hfactioncpx12)
NULL


##' Menarche data set
##'
##' @name mena
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Multivariate Cumulative Incidence Function example data set
##'
##' @name multcif
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' Stutter data set
##'
##' Based on nation-wide questionnaire answers from 33,317 Danish twins
##' @format
##' tvparnr: twin-pair id
##' zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
##' stutter: stutter status (yes/no)
##' age: age
##' nr: number within twin-pair
##' @name twinstut
##' @docType data
##' @keywords data
NULL

##' BMI data set
##'
##' @format
##' Self-reported BMI-values on 11,411 subjects
##'
##' tvparnr: twin id
##' bmi: BMI (m/kg^2)
##' age: Age
##' gender: (male/female)
##' zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
##' @name twinbmi
##' @docType data
##' @keywords data
NULL

##' Prostate data set
##'
##' @name prt
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

##' For internal use
##'
##' @title For internal use
##' @name npc
##' @rdname internal
##' @author Klaus K. Holst
##' @keywords utilities
##' @export
##' @aliases plotcr npc nonparcuminc simnordic corsim.prostate
##'  alpha2kendall alpha2spear coefmat piecewise.twostage surv.boxarea
##'  faster.reshape piecewise.data
##'  simBinPlack simBinFam simBinFam2 simSurvFam corsim.prostate.random
##'  simnordic.random simCox sim
##'  grouptable jumptimes folds
##'  ace.family.design ascertained.pairs CCbinomial.twostage
##'  coarse.clust concordanceTwinACE concordanceTwostage
##'  fast.cluster force.same.cens ilap
##'  kendall.ClaytonOakes.twin.ace kendall.normal.twin.ace
##'  make.pairwise.design make.pairwise.design.competing
##'  matplot.mets.twostage object.defined p11.binomial.twostage.RV
##'  predictPairPlack simbinClaytonOakes.family.ace
##'  simbinClaytonOakes.pairs simbinClaytonOakes.twin.ace
##'  simClaytonOakes.family.ace simClaytonOakes.twin.ace simFrailty.simple
##'  simCompete.simple simCompete.twin.ace twin.polygen.design
##'  procform procform3 procformdata drop.specials
NULL


##' Rates for HPN program for patients of Copenhagen Cohort
##'
##' @name CPH_HPN_CRBSI 
##' @format
##'  crbsi: cumulative rate of catheter related bloodstream infection in HPN patients of Copenhagen
##'  mechanical: cumulative rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen
##' trombo: cumulative rate of Occlusion/Thrombosis complication for catheter of HPN patients of Copenhagen
##' terminal: rate of terminal event, patients leaving the HPN program
##' @docType data
##' @keywords data
##' @source Estimated data
NULL

##' haplo fun data 
##'
##' @name haplo 
##' @format
##' hapfreqs : haplo frequencies
##' haploX:  covariates and response for haplo survival discrete survival
##' ghaplos:  haplo-types for subjects of haploX data
##' @docType data
##' @keywords data
##' @source Estimated data
NULL

##' ttpd discrete survival data on interval form
##'
##' @name ttpd
##' @docType data
##' @keywords data
##' @source Simulated data
NULL

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)

## -----------------------------------------------------------------------------
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

proc_design <- mets:::proc_design
## marginal mean using formula  
outNZ <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 

outN <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
summary(outN,times=3) 

## -----------------------------------------------------------------------------
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
head(iid(outNZ3))

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
head(iid(outN3))

## -----------------------------------------------------------------------------
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
plot(GL,add=TRUE,col=2)

## -----------------------------------------------------------------------------
ooNZ <- prob.exceed.recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
summary(ooNZ,times=3)

## -----------------------------------------------------------------------------
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
head(iid(RecLost))

## -----------------------------------------------------------------------------
sessionInfo()

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.ApplyBy2 <- function(idata, icluster, F, Env, Argument = "x", Columnwise = 0L, Reduce = 0L, epsilon = 1.0e-16) {
    .Call(`_mets_ApplyBy2`, idata, icluster, F, Env, Argument, Columnwise, Reduce, epsilon)
}

.ApplyBy <- function(idata, icluster, f) {
    .Call(`_mets_ApplyBy`, idata, icluster, f)
}

S0FG <- function(ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart) {
    .Call(`_mets_S0_FG_GcR`, ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart)
}

Gcjumps <- function(iGc, istatus, istrata2, instrata2, iGcstart, injumps) {
    .Call(`_mets_GcjumpsR`, iGc, istatus, istrata2, instrata2, iGcstart, injumps)
}

S0FGN <- function(ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps) {
    .Call(`_mets_S0_FGRN`, ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps)
}

.scoreMVN <- function(Y, Mu, dMu, S, dS, itol = 0.0) {
    .Call(`_mets_scoreMVN`, Y, Mu, dMu, S, dS, itol)
}

.loglikMVN <- function(Yl, Yu, Status, Mu, S, Threshold, z, su, itol) {
    .Call(`_mets_loglikMVN`, Yl, Yu, Status, Mu, S, Threshold, z, su, itol)
}

.dmvn <- function(u, mu, rho) {
    .Call(`_mets_dmvn`, u, mu, rho)
}

.rmvn <- function(n, mu, rho) {
    .Call(`_mets_rmvn`, n, mu, rho)
}

.logl_weibull <- function(par, entry, exit, status, X, Z) {
    .Call(`_mets_logl_weibull`, par, entry, exit, status, X, Z)
}

.score_weibull <- function(par, entry, exit, status, X, Z, indiv = TRUE) {
    .Call(`_mets_score_weibull`, par, entry, exit, status, X, Z, indiv)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_mets_RcppExport_registerCCallable`)
})

##' Estimation of concordance in bivariate competing risks data
##'
##' @title Estimation of concordance in bivariate competing risks data
##' @param formula Formula with left-hand-side being a \code{Event} object (see example below) and the left-hand-side specying the covariate structure
##' @param data Data frame
##' @param cause Causes (default (1,1)) for which to estimate the bivariate cumulative incidence
##' @param cens The censoring code
##' @param causes causes
##' @param indiv indiv
##' @param strata Strata
##' @param id Clustering variable
##' @param num num
##' @param max.clust max number of clusters in timereg::comp.risk call for iid decompostion, max.clust=NULL uses all clusters otherwise rougher grouping.
##' @param marg marginal cumulative incidence to make stanard errors for same clusters for subsequent use in casewise.test()
##' @param se.clusters to specify clusters for standard errors. Either a vector of cluster indices or a column name in \code{data}. Defaults to the \code{id} variable.
##' @param wname name of additonal weight used for paired competing risks data.
##' @param prodlim prodlim to use prodlim estimator (Aalen-Johansen) rather than IPCW weighted estimator based on comp.risk function.These are equivalent in the case of no covariates. These esimators are the same in the case of stratified fitting.
##' @param messages Control amount of output
##' @param model Type of competing risk model (default is Fine-Gray model "fg", see comp.risk).
##' @param return.data Should data be returned (skipping modeling)
##' @param uniform to compute uniform standard errors for concordance estimates based on resampling.
##' @param conservative for conservative standard errors, recommended for larger data-sets.
##' @param resample.iid to return iid residual processes for further computations such as tests.
##' @param ... Additional arguments to timereg::comp.risk function
##' @author Thomas Scheike, Klaus K. Holst
##' @aliases bicompriskData
##' @export
##' @references
##' Scheike, T. H.; Holst, K. K. & Hjelmborg, J. B.
##' Estimating twin concordance for bivariate competing risks twin data
##' Statistics in Medicine, Wiley Online Library, 2014 , 33 , 1193-204
##'
##' @examples
##' library("timereg")
##'
##' ## Simulated data example
##' prt <- simnordic.random(2000,delayed=TRUE,ptrunc=0.7,
##'		      cordz=0.5,cormz=2,lam0=0.3)
##' ## Bivariate competing risk, concordance estimates
##' p11 <- bicomprisk(Event(time,cause)~strata(zyg)+id(id),data=prt,cause=c(1,1))
##'
##' p11mz <- p11$model$"MZ"
##' p11dz <- p11$model$"DZ"
##' par(mfrow=c(1,2))
##' ## Concordance
##' plot(p11mz,ylim=c(0,0.1));
##' plot(p11dz,ylim=c(0,0.1));
##'
##' ## entry time, truncation weighting
##' ### other weighting procedure
##' prtl <-  prt[!prt$truncated,]
##' prt2 <- ipw2(prtl,cluster="id",same.cens=TRUE,
##'      time="time",cause="cause",entrytime="entry",
##'      pairs=TRUE,strata="zyg",obs.only=TRUE)
##'
##' prt22 <- fast.reshape(prt2,id="id")
##'
##' prt22$event <- (prt22$cause1==1)*(prt22$cause2==1)*1
##' prt22$timel <- pmax(prt22$time1,prt22$time2)
##' ipwc <- timereg::comp.risk(Event(timel,event)~-1+factor(zyg1),
##'   data=prt22,cause=1,n.sim=0,model="rcif2",times=50:90,
##'   weights=prt22$weights1,cens.weights=rep(1,nrow(prt22)))
##'
##' p11wmz <- ipwc$cum[,2]
##' p11wdz <- ipwc$cum[,3]
##' lines(ipwc$cum[,1],p11wmz,col=3)
##' lines(ipwc$cum[,1],p11wdz,col=3)
##'
bicomprisk <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
 strata=NULL, id,num, max.clust=1000, marg=NULL,se.clusters=NULL,wname=NULL,
 prodlim=FALSE,messages=TRUE,model,return.data=0,uniform=0,conservative=1,resample.iid=1,...) {
## {{{

  mycall <- match.call()
  formulaId <- unlist(Specials(formula,"id"))
  formulaIndiv <- Specials(formula,"indiv")
  formulaStrata <- unlist(Specials(formula,"strata"))
  formulaSt <- paste("~.-id(",formulaId,")",
                     "-strata(",paste(formulaStrata,collapse="+"),")",
                     "-indiv(",paste(formulaIndiv,collapse="+"),")")
  formula <- update(formula,formulaSt)

  ### ts  11/10
  ## if (substr(as.character(formula)[2],1,4)=="Hist") {
  ##     stop("Since version : The left hand side of the formula must be specified as
  ##     Event(time, event) or with non default censoring codes Event(time, event, cens.code=0).")
  ## }


  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  indiv <- formulaIndiv
  if (!is.null(formulaIndiv)) {
    mycall$indiv <- indiv
  }
  if (missing(id)) stop("Missing 'id' variable")

  ### setting up cluster level for iid decomposition
  if (is.null(se.clusters) & is.null(marg))  lse.clusters <- data[,c(id)]
  else {
    if (is.null(se.clusters)) {
      lse.clusters <- marg$clusters
###      if (!is.null(max.clust)) {   }
    } else {
      if (is.character(se.clusters)) {
        lse.clusters <- data[,se.clusters]
      } else {
        lse.clusters <- se.clusters
      }
    }
  }
  if (length(lse.clusters)!=nrow(data)) stop("'se.clusters' and 'data' does not match!")
  se.clusters.call <- se.clusters

  data <- data.frame(cbind(data,lse.clusters))

  timevar <- terms(formula)[[2]]
  ##  hh <- with(data,eval(timevar))
  if (is.call(timevar)) {
    causes <- timevar[[3]]
    timevar <- timevar[[2]]
  }
  timevar <- as.character(timevar)
  causes <- as.character(causes)

  if (!is.null(strata)) {
    fac <- interaction(data[,strata])
    dd <- split(data,fac)
    nn <- unlist(lapply(dd,nrow))
    dd[which(nn==0)] <- NULL
    if (length(dd)>1) {
      fit <- lapply(seq_len(length(dd)),function(i) {
        if (messages>0) message("Strata '",names(dd)[i],"'")
        idx <- which(fac==names(dd)[i])
        mycall$se.clusters <- lse.clusters[idx]
        mycall$formula <- formula
        mycall$data <- dd[[i]]
        eval(mycall)
      })
      res <- list(model=fit)
      res$strata <- names(res$model) <- names(dd)
      class(res) <- c("bicomprisk.strata","twinlm.strata")
      res$N <- length(dd)
      return(res)
    }
  }

  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  ### adds weights,
  if (!is.null(wname)) covars <- c(covars,wname)

  indiv2 <- covars2 <- NULL

  data <- data[order(data[,id]),]
  idtab <- table(data[,id])
  ##which(data[,id]%in%names(idtab==2))
  data <- data[which(data[,id]%in%names(idtab==2)),]
  if (missing(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,num] <- unlist(lapply(idtab,seq_len))
  }

  oldreshape <- 0
  if (oldreshape==1) sep="." else sep=""
  timevar2 <- paste(timevar,1:2,sep=sep)
  causes2 <- paste(causes,1:2,sep=sep)
  if (length(covars)>0)
    covars2 <- paste(covars,1,sep=sep)
  for (i in seq_len(length(indiv)))
  indiv2 <- c(indiv2, paste(indiv[i],1:2,sep=sep))

  if (oldreshape==1)
  ww0 <- reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],
         direction="wide",idvar=id,timevar=num)[,c(id,"lse.clusters.1",timevar2,causes2,indiv2,covars2)]
  else
  ww0 <- fast.reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],id=id,num=data$num,labelnum=TRUE)[,c(id,"lse.clusters1",timevar2,causes2,indiv2,covars2)]
  ww0 <- na.omit(ww0)

  status <- rep(0,nrow(ww0))
  time <- ww0[,timevar2[1]]

  ## {{{ (i,j) causes
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cause[2])
  if (length(idx2)>0) {
      status[idx2] <- 1
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,max)
  }

  ##(0,0), (0,j)
  idx2 <- which(ww0[,causes2[1]]==cens & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(ic,0), (ic,j)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(i,0)
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cens)
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  ##(ic,jc)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,min)
  }

  ##(0,jc),(i,jc)
  idx2 <- which((ww0[,causes2[1]]==cens | ww0[,causes2[1]]==cause[1]) & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  mydata0 <- mydata <- data.frame(time,status,ww0[,covars2],ww0[,indiv2])
  names(mydata) <- c(timevar,causes,covars,indiv2)
  ## }}}

  if (return.data==2) return(list(data=mydata)) else {
  if (!prodlim) {
    ff <- paste("Event(",timevar,",",causes,",cens.code=",cens,") ~ 1",sep="")
    if (!is.null(wname)) covars <- covars[-which(covars %in% c(wname))]
    if (length(c(covars,indiv))>0) {
     xx <- c(covars,indiv2)
     for (i in seq_len(length(xx)))
	     xx[i] <- paste("const(",xx[i],")",sep="")
      ff <- paste(c(ff,xx),collapse="+")
      if (missing(model)) model <- "fg"
    }
    if (missing(model)) model <- "fg"
    ### clusters for iid construction
    lse.clusters <- NULL
    if (!is.null(se.clusters.call)) {
        lse.clusters <- ww0[,"lse.clusters1"]
    }

###    if (!(is.null(wname))) mydata <- ipw2(mydata,time=timevar,cause=causes) #,cens.code=cens)

    if (is.null(wname)) {
	    add<- timereg::comp.risk(as.formula(ff),data=mydata,
	    cause=1,n.sim=0,resample.iid=resample.iid,model=model,conservative=conservative,
	    clusters=lse.clusters, max.clust=max.clust,...)
    } else {
	    add<-timereg::comp.risk(as.formula(ff),data=mydata,
	    cause=1,n.sim=0,resample.iid=resample.iid,model=model,conservative=conservative,
	    clusters=lse.clusters, max.clust=max.clust,
	    weights=mydata[,wname]*mydata$indi.weights,cens.weights=rep(1,nrow(mydata)),...)
    }

    padd <- predict(add,X=1,se=1,uniform=uniform,resample.iid=resample.iid)
    padd$cluster.names <- lse.clusters
  } else {
      if (!requireNamespace("prodlim",quietly=TRUE)) stop("prodlim requested but not installed")
    ff <- as.formula(paste("prodlim::Hist(",timevar,",",causes,")~",paste(c("1",covars,indiv2),collapse="+")))
    padd <- prodlim::prodlim(ff,data=mydata)
  }
###  class(padd) <- c("bicomprisk",class(padd))
 if (return.data==1) return(list(comp.risk=padd,data=mydata)) else return(padd)
  }
## }}}
}


##' @export
bicompriskData <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
 strata=NULL, id,num, se.clusters=NULL,wname=NULL, ...) {
## {{{

  mycall <- match.call()
  formulaId <- unlist(Specials(formula,"id"))
  formulaIndiv <- Specials(formula,"indiv")
  formulaStrata <- unlist(Specials(formula,"strata"))
  formulaSt <- paste("~.-id(",formulaId,")",
                     "-strata(",paste(formulaStrata,collapse="+"),")",
                     "-indiv(",paste(formulaIndiv,collapse="+"),")")
  formula <- update(formula,formulaSt)

  if (!is.null(formulaId)) {
    id <- formulaId
    mycall$id <- id
  }
  if (!is.null(formulaStrata)) {
    strata <- formulaStrata
    mycall$strata <- strata
  }
  indiv <- formulaIndiv
  if (!is.null(formulaIndiv)) {
    mycall$indiv <- indiv
  }
  if (missing(id)) stop("Missing 'id' variable")

  ### setting up cluster level for iid decomposition
  if (is.null(se.clusters))  lse.clusters <- data[,c(id)]
  if (length(lse.clusters)!=nrow(data)) stop("'se.clusters' and 'data' does not match!")
  se.clusters.call <- se.clusters


  data <- data.frame(cbind(data,lse.clusters))

  timevar <- terms(formula)[[2]]
  ##  hh <- with(data,eval(timevar))
  if (is.call(timevar)) {
    causes <- timevar[[3]]
    timevar <- timevar[[2]]
  }
  timevar <- as.character(timevar)
  causes <- as.character(causes)

  covars <- as.character(attributes(terms(formula))$variables)[-(1:2)]
  covars <- c(covars,strata)
  ### adds weights,
  if (!is.null(wname)) covars <- c(covars,wname,strata)

  indiv2 <- covars2 <- NULL

  data <- data[order(data[,id]),]
  idtab <- table(data[,id])
  ##which(data[,id]%in%names(idtab==2))
  data <- data[which(data[,id]%in%names(idtab==2)),]
  if (missing(num)) {
    idtab <- table(data[,id])
    num <- "num"
    while (num%in%names(data)) num <- paste(num,"_",sep="")
    data[,num] <- unlist(lapply(idtab,seq_len))
  }

  sep <- ""
  timevar2 <- paste(timevar,1:2,sep=sep)
  causes2 <- paste(causes,1:2,sep=sep)
  if (length(covars)>0)
    covars2 <- paste(covars,1,sep=sep)
  for (i in seq_len(length(indiv)))
  indiv2 <- c(indiv2, paste(indiv[i],1:2,sep=sep))

  ww0 <- fast.reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],id=id,num=data$num,labelnum=TRUE)[,c(id,"lse.clusters1",timevar2,causes2,indiv2,covars2)]
  ww0 <- na.omit(ww0)

  status <- rep(0,nrow(ww0))
  time <- ww0[,timevar2[1]]

  ## {{{ (i,j) causes
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cause[2])
  if (length(idx2)>0) {
      status[idx2] <- 1
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,max)
  }

  ##(0,0), (0,j)
  idx2 <- which(ww0[,causes2[1]]==cens & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(ic,0), (ic,j)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]==cens | ww0[,causes2[2]]==cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[1]]
  }

  ##(i,0)
  idx2 <- which(ww0[,causes2[1]]==cause[1] & ww0[,causes2[2]]==cens)
  if (length(idx2)>0) {
      status[idx2] <- 0
      time[idx2] <- ww0[idx2,timevar2[2]]
  }

  ##(ic,jc)
  idx2 <- which(ww0[,causes2[1]]!=cens & ww0[,causes2[1]]!=cause[1] & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- apply(ww0[idx2,timevar2[1:2],drop=FALSE],1,min)
  }

  ##(0,jc),(i,jc)
  idx2 <- which((ww0[,causes2[1]]==cens | ww0[,causes2[1]]==cause[1]) & (ww0[,causes2[2]]!=cens & ww0[,causes2[2]]!=cause[2]))
  if (length(idx2)>0) {
      status[idx2] <- 2
      time[idx2] <- ww0[idx2,timevar2[2]]
  }


  mydata0 <- mydata <- data.frame(time,status,ww0[,covars2],ww0[,indiv2],ww0[,id])

  names(mydata) <- c(timevar,causes,covars,indiv2,id)
  ## }}}

  return(mydata)
} ## }}}



## plot.bicomprisk <- function(x,add=FALSE,...) {
##   if ("predict.timereg"%in%class(a)) {
##     if (!add) { plot.predict.timereg(x,...) }
##     else {
##       with(x,lines(time,P1,...))
##     }
##   } else {
##     plot(x,...)
##   }
##   return(invisible(x))
##}

nameme <- function(iid,name.id,sort=TRUE) { ## {{{
if (is.matrix(iid))
	if (nrow(iid)==length(name.id)) {
		rownames(iid) <- name.id
###		if (sort)  {
###		oid <- order(name.id);
###		iid <- iid[oid,,drop=FALSE];
###		}
}
return(iid)
} ## }}}

construct_id <- function(id,nid,namesX=NULL,as.data=FALSE) { ## {{{
  call.id <- id

  if (!is.null(id)) {
	  ids <- unique(id)
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
     name.id <- sort(ids)
   } else {
	   id <- 1:nid-1;
	   ids <- id+1
	   order.ids <- ids
	   name.id <- NULL
	   ## Use rownames of design matrix as labels when id was not provided
           name.id <- if (!is.null(namesX)) namesX else NULL 
   }
   ## orginal id coding into integers
   ## id from 0,1,...,nid-1

    if (as.data) {
        id  <- (0:(nid - 1))[order(ids)][id +1]
        name.id <- ids
    }

  return(list(call.id=call.id,id=id,nid=nid,unique.id=ids,name.id=name.id))
} ## }}}

###{{{ phreg01
phreg01 <- function(X,entry,exit,status,id=NULL,strata=NULL, offset=NULL,weights=NULL,strata.name=NULL,cumhaz=TRUE,
    beta,stderr=TRUE,method="NR", no.opt=FALSE,Z=NULL,propodds=NULL,AddGam=NULL,
    marks=NULL,no.var=0,augmentation=0,...) {
  p <- ncol(X)
  if (missing(beta)) beta <- rep(0,p)
  if (p==0) X <- cbind(rep(0,length(exit)))

  if (is.null(strata)) {
	  strata <- rep(0,length(exit)); nstrata <- 1; strata.level <- NULL;
  } else {
	  strata.level <- levels(strata)
	  ustrata <- sort(unique(strata))
	  nstrata <- length(ustrata)
	  strata.values <- ustrata
      if (is.numeric(strata)) strata <-  fast.approx(ustrata,strata)-1 else  {
      strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
    }
  }
  strata.call <- strata
  Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given
  if (!is.null(Z)) Zcall <- Z

  ## possible casewights to use for bootstrapping and other things
  if (is.null(marks)) marks <- rep(1,length(exit))
  trunc <- (any(entry>0))
###  if (!trunc) entry <- rep(0,length(exit))

  call.id <- id;
  conid <- construct_id(id,nrow(X),namesX=rownames(X))
  id <- conid$id; nid <- conid$nid; name.id <- conid$name.id

  dd <- .Call("FastCoxPrepStrata", entry,exit,status,X,id,trunc,strata,weights,offset,Zcall,marks,PACKAGE="mets")

  dd$nstrata <- nstrata
   obj <- function(pp,U=FALSE,all=FALSE) {# {{{
      if (length(dd$jumps) >0) {
	      if (is.null(propodds) & is.null(AddGam)) {
		val <- with(dd, .Call("FastCoxPLstrata",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,caseweights,PACKAGE="mets"))
	      } else if (is.null(AddGam))
		val <- with(dd, .Call("FastCoxPLstrataPO",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,propodds,PACKAGE="mets"))
	      else val <- with(dd, .Call("FastCoxPLstrataAddGam",pp,X,XX,sign,jumps, strata,nstrata,weights,offset,ZX,
					 AddGam$theta,AddGam$dimthetades,AddGam$thetades,AddGam$ags,AddGam$varlink,AddGam$dimjumprv,AddGam$jumprv,
					 AddGam$JumpsCauses,PACKAGE="mets"))
	      val$gradient <- val$gradient+augmentation
      } else {
	      val <- list(jumps=NULL,ploglik=NULL,U=NULL,gradient=NULL,hessian=NULL,hessiantime=NULL,S2S0=NULL,E=NULL,S0=NULL)
      }

	  if (all) {
	      val$time <- dd$time
	      ### plus 1 for R index
	      val$ord <- dd$ord+1
	      val$jumps <- dd$jumps+1
	      val$jumptimes <- val$time[val$jumps]
	      val$weightsJ <- dd$weights[val$jumps]
	      val$marks <- dd$marks[val$jumps]
	      val$nevent <- length(val$S0)
	      val$nstrata <- dd$nstrata
	      val$strata <- dd$strata
	      val$strata.jumps <- val$strata[val$jumps]
	      return(val)
	  }
          n <- length(dd$time)
	 with(val,structure(-ploglik/n,gradient=-gradient/n,hessian=-hessian/n))
	}# }}}

  opt <- NULL
  if (p>0) {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
          tim <- system.time(opt <- lava::NR(beta,obj,...))
          opt$timing <- tim
          opt$estimate <- opt$par
      } else {
          opt <- nlm(obj,beta,...)
          opt$method <- "nlm"
      }
      cc <- opt$estimate;  names(cc) <- colnames(X)
      if (!stderr) return(cc)
      val <- c(list(coef=cc), obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta), obj(beta,all=TRUE))
  } else {
      val <- obj(0,all=TRUE)
  }

  se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL
  II <- NULL
  if (no.opt==FALSE & p!=0) {
         II <- - tryCatch(pinv(val$hessian),error=function(e) matrix(0,nrow(val$hessian),ncol(val$hessian)) )
  } else II <- matrix(0,p,p)

  ## Brewslow estimator, to handle also possible weights, caseweights that are 0
  ww <- val$caseweightsJ * val$weightsJ
  val$S0[abs(ww)<.00000001] <- 0

  ### computes Breslow estimator
  if (cumhaz==TRUE & (length(val$jumps)>0)) { # {{{
	 strata <- val$strata[val$jumps]
	 nstrata <- val$nstrata
	 jumptimes <- val$jumptimes

	 ## Brewslow estimator, to handle also possible weights/case-weights, and caseweights that are 0
	 S0i2 <- S0i <- rep(0,length(val$S0))
         wwJ <- val$caseweightsJ*val$weightsJ
	 S0i[val$S0>0] <- 1/val$S0[val$S0>0]
	 S0i2[val$S0>0] <- 1/(val$S0[val$S0>0]^2*wwJ[val$S0>0])
	 cumhaz <- cbind(jumptimes,cumsumstrata(S0i,strata,nstrata))
	 if ((no.opt==FALSE & p!=0)) {
	     DLambeta.t <- apply(val$E*S0i,2,cumsumstrata,strata,nstrata)
	     varbetat <-   rowSums((DLambeta.t %*% II)*DLambeta.t)
	 ### covv <-  apply(covv*DLambeta.t,1,sum) Covariance is "0" by construction
	 } else varbetat <- 0
	 var.cumhaz <- cumsumstrata(S0i2,strata,nstrata)+varbetat
	 se.cumhaz <- cbind(jumptimes,(var.cumhaz)^.5)

	 colnames(cumhaz)    <- c("time","cumhaz")
	 colnames(se.cumhaz) <- c("time","se.cumhaz")
 } # }}}
 else {cumhaz <- se.cumhaz <- lcumhaz <- lse.cumhaz <- NULL}

  res <- c(val,
           list(cox.prep=dd,
                strata.call=strata.call, strata.level=strata.level,
                entry=entry, exit=exit, status=status, X=X,
                p=p, offsets=offset, weights=weights,
                id=id,call.id=call.id,nid=nid,name.id=name.id,
                opt=opt, no.opt=no.opt, cumhaz=cumhaz, se.cumhaz=se.cumhaz,
                lcumhaz=lcumhaz, lse.cumhaz=lse.cumhaz, ihessian=II,
                II=II,strata.name=strata.name,propodds=propodds))
  class(res) <- "phreg"

  ## also computing robust variance
  if (p>0 & no.var==0) {
  beta.iid <- iid(res)
  phvar <- crossprod(beta.iid)
###  estimate.beta <- estimate(coef=val$coef,IC=nrow(beta.iid)*beta.iid,id=name.id)
###  res$estimate.beta <- estimate.beta
  colnames(phvar) <- rownames(phvar) <- names(res$coef)
  res$var <- phvar
  res$beta.iid <- beta.iid
  } else res$var <- NULL

  return(res)
}

###}}} phreg0

###{{{ phreg

##' Fast Cox Proportional Hazards Regression
##'
##' Fits a Cox proportional hazards model using a fast C++ backend.
##' Robust variance (sandwich estimator) is the default variance estimate in the summary.
##'
##' The influence functions (IID decomposition) follow the numerical order of the given cluster variable.
##' Ordering the results by `$id` will align the IID terms with the original dataset order.
##'
##' @param formula Formula with a 'Surv' or 'Event' outcome (similar to \code{coxph}).
##' @param data Data frame containing the variables.
##' @param offset Offsets for the Cox model linear predictor.
##' @param weights Weights for the Cox score equations.
##' @param ... Additional arguments passed to lower-level functions (e.g., optimization controls).
##'
##' @return An object of class \code{"phreg"} containing:
##' \item{coef}{Vector of estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{beta.iid}{Influence functions for the regression coefficients.}
##' \item{cumhaz}{Matrix of cumulative hazard estimates (time, cumhaz).}
##' \item{se.cumhaz}{Matrix of standard errors for the cumulative hazard.}
##' \item{cox.prep}{List containing preprocessed data for the Cox model.}
##' \item{opt}{Optimization results (if optimization was performed).}
##' \item{call}{The matched call.}
##'
##' @author Klaus K. Holst, Thomas Scheike
##' @seealso \code{\link{plot.phreg}}, \code{\link{predict.phreg}}, \code{\link{robust_phreg}}
##' @aliases phreg robust_phreg
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' # Fit model with clustering
##' out1 <- phreg(Surv(time, status == 9) ~ wmi + age + strata(vf, chf) + cluster(id), data = TRACE)
##' summary(out1)
##'
##' # Plotting baselines
##' par(mfrow = c(1, 2))
##' plot(out1)
##'
##' # Computing robust variance for baseline
##' rob1 <- robust_phreg(out1)
##' plot(rob1, se = TRUE, robust = TRUE)
##'
##' # IID decomposition for regression parameters
##' head(iid(out1))
##'
##' # IID decomposition for baseline at a specific time-point
##' Aiiid <- iid(out1, time = 30)
##' head(Aiiid)
##'
##' # Combined IID decomposition (beta and baseline)
##' dd <- iidBaseline(out1, time = 30)
##' head(dd$beta.iid)
##' head(dd$base.iid)
##'
##' # Stratified model
##' outs <- phreg(Surv(time, status == 9) ~ strata(vf, wmicat.4) + cluster(id), data = TRACE)
##' summary(outs)
##' par(mfrow = c(1, 2))
##' plot(outs)
##' @export
phreg <- function(formula,data,offset=NULL,weights=NULL,...) {# {{{
  cl <- match.call() ## {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata","marks"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    marks <- des$marks
    if (!is.null(strata))  {
      strata.name <- attr(des$specials.var$strata, "name.mf")
    } else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL

 ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit))
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit))
  } else weights <- des.weights
 ## }}}

 res <- c(
     phreg01(X, entry, exit, status,
         id = id, strata = strata, offset = offset,
         weights = weights, strata.name = strata.name, marks = marks, ...
     ),
     list(
         call = cl, formula = formula, strata.pos = pos.strata,
         cluster.pos = pos.cluster, n = length(exit)
     )
 )
  res$model.frame <- proc_design(des$terms, data)$data #, specials="cluster")$data
  des$data <- data[0,]
  res$design <- des
  class(res) <- "phreg"

  res
}# }}}

readPhreg <- function (object, newdata, nr=TRUE,data=TRUE, ...)
{# {{{
   respindata <- length((grep(all.vars(object$formula)[1],names(newdata))))
   if (respindata>0) des <- update_design(object$design,data = newdata,response=TRUE)
   else des <- update_design(object$design,data = newdata)

   clusters <- des$cluster
   X <- des$x
   if (data) {
       xxr <- drop.specials(object$formula,"cluster")
       vars <- all.vars( update.formula(xxr,1~.))
       data <- newdata[,vars,drop=FALSE]
   } else data <- NULL

   if (!is.null(des$strata)) strataNew <- as.numeric(des$strata)-1 else strataNew <- rep(0,nrow(X))

    if (!is.null(des$y)) {
	    Y <- des$y
	    if (!inherits(Y, c("Event", "Surv"))) {
		stop("Expected a 'Surv' or 'Event'-object")
	    }
	    if (ncol(Y) == 2) {
		exit <- Y[, 1]
		entry <- rep(0, nrow(Y))
		status <- Y[, 2]
	    } else {
		entry <- Y[, 1]
		exit <- Y[, 2]
		status <- Y[, 3]
	    }
    } else {status <- exit <- entry <- NULL; }

return(list(data=data,X=X,strata=strataNew,entry=entry,exit=exit,status=status,clusters=clusters))
}# }}}


### }}} phreg

### {{{ iid & Robust variances

transformation_phreg <- function(type, lp = TRUE) {
  # p[1]: linear predictor
  # p[2..]: baseline cumulative hazard in different strata
  if (type == "risk") {
    f <- ifelse(lp,
      function(p) {
        res <- numeric(length(p) - 1)
        for (i in seq(2, length(p))) {
          res[i - 1] <- 1 - exp(-p[i])**exp(p[1])
        }
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)[-1]
        }
        return(res)
      },
      function(p) {
        res <- 1 - exp(-p)
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)
        }
        return(res)
      }
    )
    return(f)
  }
  if (type == "surv") {
    f <- ifelse(lp,
      function(p) {
        res <- numeric(length(p) - 1)
        for (i in seq(2, length(p))) {
          res[i - 1] <- exp(-p[i])**exp(p[1])
        }
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)[-1]
        }
        return(res)
      },
      function(p) {
        res <- exp(-p)
        if (length(res) == 1L) {
          names(res) <- type
        } else {
          names(res) <- names(p)
        }
        return(res)
      }
    )
    return(f)
  }
  ## type == "chaz"
  f <- ifelse(lp,
    function(p) {
      res <- numeric(length(p) - 1)
      for (i in seq(2, length(p))) {
        res[i - 1] <- p[i]*exp(p[1])
      }
      if (length(res) == 1L) {
        names(res) <- type
      } else {
        names(res) <- names(p)[-1]
      }
      return(res)
    },
    function(p) {
      if (length(p) == 1L) {
        names(p) <- type
      }
      return(p)
    }
  )
  return(f)
}

##' @export
estimate.phreg <- function(x, ..., time = NULL,
			   newdata = NULL, X = NULL,all=FALSE,
                           type = c("chaz", "surv", "risk"),
                           baseline.args = list()) { ## {{{ 
  if (NCOL(model.matrix(x))==0L & is.null(time)) stop("Non-parametric model; need `time` argument")
  if (!is.null(newdata) || !is.null(X)) {
    if (is.null(X)) {
      X <- model.matrix(x, data = newdata)
    }
    if (NROW(X) > 1) {
      X <- X[1, ]
      warning("Only using first row of the data")
    }
    if (all(X == 0L)) X <- NULL
  }
  if (!is.null(X)) {
    lp <- estimate(x, rbind(X), labels = "lp")
    if (is.null(time)) {
      return(estimate(lp, ...))
    }
    base <- estimate(x,
      time = time,
      type = "chaz",
      baseline.args = baseline.args
    )
    res <- transform(merge(lp, base,
                           paired = TRUE
                           ),
                     transformation_phreg(tolower(type[1]))
                     )
    return(estimate(res, ...))
  }
  ic <- do.call(IC, c(list(x, time = time,all=all), baseline.args))
  cc <- attr(ic, "coef")
  if (is.null(cc)) cc <- coef(x)
  lab <- names(cc)
###  if (!is.null(time) && length(cc) > 1 || is.null(lab)) lab <- x$strata.level
###  b <- lava::estimate(coef = cc, IC = ic, labels = lab)
  b <- lava::estimate(coef = cc, IC = ic)
  if (!is.null(time)) {
    b <- transform(b, transformation_phreg(tolower(type[1]), FALSE))
  }
  return(estimate(b, ...))
} ## }}} 

##' Influence Functions for phreg objects
##'
##' Computes the influence functions (IID decomposition) for the regression coefficients
##' and/or the baseline cumulative hazard at a specific time point.
##'
##' @param x Object of class \code{"phreg"}.
##' @param type Type of influence function: \code{"robust"} (default) or \code{"martingale"}.
##' @param all Logical; if \code{TRUE}, returns both beta and baseline influence functions.
##' @param time Time point for baseline influence function (required if baseline is requested).
##' @param baseline Arguments for baseline estimation.
##' @param ... Additional arguments.
##'
##' @return A matrix of influence functions. If \code{all=TRUE}, columns correspond to
##' regression coefficients and baseline cumulative hazard. Attributes include \code{coef}
##' and \code{time}.
##'
##' @author Thomas Scheike
##' @seealso \code{\link{phreg}}, \code{\link{iidBaseline}}
##' @export
IC.phreg  <- function(x,type="robust",all=FALSE,time=NULL,baseline=NULL,...) {# {{{
  if (x$p==0 & is.null(time)) stop("Non-parametric model; need `time` argument")
  if (!is.null(time)) {
    iid <- iidBaseline(x, time = time, ...)
    if (all)  {
            res <- with(iid, cbind(beta.iid,base.iid) * NROW(base.iid))
	    coefs <- c(iid$coef,iid$cumhaz.time) 
	    colnames(res) <- names(coefs)
            attr(res, "coef") <- coefs
    } else { 
            res <- with(iid, base.iid * NROW(base.iid))
	    attr(res, "coef") <- c(iid$cumhaz.time)
    }
    attr(res, "strata.level") <- iid$strata.level
    attr(res, "time") <- time
    return(res)
  }
    classes1 <- "mlogit"
    if ((length(class(x)) == 1) || inherits(x, classes1)) {
        invhess <- -pinv(x$hessian)
        orig.order <- FALSE
        if (is.null(x$propodds)) {
            if (type == "robust") {
                xx <- x$cox.prep
                ii <- invhess
                S0i <- rep(0, length(xx$strata))
                S0i[xx$jumps + 1] <- 1/x$S0
                Z <- xx$X
                U <- E <- matrix(0, nrow(xx$X), x$p)
                E[xx$jumps + 1, ] <- x$E
                U[xx$jumps + 1, ] <- x$U
                cumhaz <- cbind(xx$time, cumsumstrata(S0i, xx$strata,
                  xx$nstrata))
                EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata,
                  xx$nstrata)
                rr <- c(xx$sign * exp(Z %*% coef(x) + xx$offset))
                MGt <- U[, drop = FALSE] - (Z * cumhaz[, 2] -
                  EdLam0) * rr * c(xx$weights)
                if (orig.order) {
                  oo <- (1:nrow(xx$X))[xx$ord + 1]
                  oo <- order(oo)
                  MGt <- MGt[oo, , drop = FALSE]
                  id <- xx$id[oo]
                }
                else id <- xx$id
            }
            else {
                MGt <- x$U
                MG.base <- 1/x$S0
            }
        } else {
            xx <- x$cox.prep
            ii <- invhess
            S0i <- rep(0, length(xx$strata))
            S0i[xx$jumps + 1] <- 1/x$S0
            Z <- xx$X
            U <- E <- matrix(0, nrow(xx$X), x$p)
            E[xx$jumps + 1, ] <- x$E
            U[xx$jumps + 1, ] <- x$U
            cumhazA <- cumsumstratasum(S0i, xx$strata, xx$nstrata,
                type = "all")
            cumhaz <- c(cumhazA$sum)
            cumhazm <- cumhazA$lagsum
            EdLam0 <- apply(E * S0i, 2, cumsumstrata, xx$strata,
                xx$nstrata)
            rr <- c(xx$sign * exp(Z %*% coef(x) + xx$offset))
            rro <- c(exp(Z %*% coef(x) + xx$offset))
            S0star <- revcumsumstrata(rr/(1 + rro * cumhazm),
                xx$strata, xx$nstrata)
            S0 <- revcumsumstrata(rr, xx$strata, xx$nstrata)
            S1 <- apply(Z * rr, 2, revcumsumstrata, xx$strata,
                xx$nstrata)
            Et <- S1/c(S0)
            lt <- apply((Z - Et) * c(rr * rro/(1 + rro * cumhazm)),
                2, revcumsumstrata, xx$strata, xx$nstrata)
            Estar <- S0star/S0
            EstardLam <- cumsumstrata(Estar * S0i, xx$strata,
                xx$nstrata)
            k <- exp(-EstardLam)
            basecor <- apply(lt * c(k * S0i), 2, revcumsumstrata,
                xx$strata, xx$nstrata)
            basecor <- basecor/c(k * S0)
            www <- x$propoddsW * x$weightsJ
            U[xx$jumps + 1, ] <- U[xx$jumps + 1, ] - c(www) *
                basecor[xx$jumps + 1, ]
            MGt <- U
            if (type == "robust") {
                baseDLam0 <- apply(basecor * S0i, 2, cumsumstrata,
                  xx$strata, xx$nstrata)
                MGt <- U[, drop = FALSE] - (Z * cumhaz - EdLam0 -
                  baseDLam0) * rr * c(xx$weights)
            }
            else MGt <- MGt[xx$jumps + 1, ]
            MG.base <- 1/x$S0
            id <- xx$id
        }
        ncluster <- NULL
        if (type == "robust" & (!is.null(x$id) | any(x$entry >
            0))) {
            if (type == "martingale")
                id <- x$id[x$jumps]
            UU <- apply(MGt, 2, sumstrata, id, max(id) + 1)
            ncluster <- nrow(UU)
        }
        else {
            UU <- MGt
        }
        res <- structure(UU %*% invhess, invhess = invhess, ncluster = ncluster)
        if (is.null(colnames(res))) colnames(res) <- names(coef(x))
	if (!is.null(x$name.id)) res <- nameme(res, x$name.id)
        res <- res * NROW(res)
        return(res)
    }
    else if (inherits(x, c("cifreg", "recreg", "IPTW"))) {
        if (inherits(x, c("IPTW")))
            res <- x$IID
        else res <- x$iid
        if (is.null(colnames(res)))
            colnames(res) <- names(coef(x))
        res <- res * NROW(res)
        return(res)
    }
} ## }}}

##' Influence Functions or IID Decomposition of Baseline
##'
##' Computes the influence functions for the baseline cumulative hazard (and optionally
##' regression coefficients) for \code{phreg}, \code{recreg}, or \code{cifregFG} objects.
##'
##' The decomposition is based on the formula:
##' \deqn{ \sum_i \int_0^t \frac{f(s)}{S_0(s)} dM_{ki}(s) - P(t) \beta_k }
##' where \eqn{k} denotes the stratum and \eqn{i} the cluster.
##'
##' @param object Object of class \code{"phreg"}, \code{"recreg"}, or \code{"cifregFG"}.
##' @param time Time point for baseline IID (required).
##' @param ft Function to compute IID of baseline integrated against \eqn{f(t)}.
##' @param fixbeta Logical; if \code{TRUE}, fixes the coefficients (useful for specific tests).
##' @param beta.iid Optional matrix of beta influence functions to use.
##' @param tminus Logical; if \code{TRUE}, computes predictions at \eqn{t-} (strictly before \eqn{t}), useful for IPCW techniques.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"iidBaseline"} containing:
##' \item{time}{Time point.}
##' \item{base.iid}{Influence functions for the baseline.}
##' \item{beta.iid}{Influence functions for the regression coefficients.}
##' \item{cumhaz.time}{Cumulative hazard at the specified time.}
##' \item{strata}{Strata indices.}
##'
##' @author Thomas Scheike
##' @seealso \code{\link{phreg}}, \code{\link{recreg}}, \code{\link{cifreg}}
##' @export
iidBaseline <- function(object, time = NULL, ft = NULL, fixbeta = NULL, beta.iid = NULL, tminus = FALSE, ...) UseMethod("iidBaseline")

##' @export
iidBaseline.phreg <- function(object,time=NULL,ft=NULL,fixbeta=NULL,beta.iid=NULL,tminus=FALSE,...)
{# {{{
###  sum_i int_0^t f(s)/S_0(s) dM_{ki}(s) - P(t) \beta_k
###  with possible strata and cluster "k", and i in clusters
  x <- object
  if (!inherits(x,"phreg")) stop("Must be phreg object\n");
  if (is.null(time)) stop("Must give time for iid of baseline")

  if (!is.null(x$propodds))  stop("Only for Cox model")
  ### sets fixbeta based on  wheter xr has been optimized in beta (so cox case)
  if (is.null(fixbeta)) {
    fixbeta <- if ((x$no.opt) | is.null(x$coef)) 1 else 0
 }


  xx <- x$cox.prep
  btimexx <- c(1*(xx$time < time))

  S0i2 <- S0i <- rep(0,length(xx$strata))
  S0i[xx$jumps+1] <- 1/x$S0
  ww <- xx$caseweights*xx$weights
  S0i2[xx$jumps+1] <- 1/(x$S0^2*ww[xx$jumps+1])
  Z <- xx$X
  U <- E <- matrix(0,nrow(xx$X),x$p)
  E[xx$jumps+1,] <- x$E
  U[xx$jumps+1,] <- x$U
  ### only up to time t for A
  if (is.null(ft))  ft <- rep(1,length(xx$time))
  if (is.matrix(ft)) ft <- c(ft)
  cumhaz <- c(cumsumstrata(ft*S0i,xx$strata,xx$nstrata))
  cumS0i2 <- c(cumsumstrata(ft*S0i2*btimexx,xx$strata,xx$nstrata))
  if (fixbeta==0) {
	  EdLam0 <- apply(ft*E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Ht <- apply(ft*E*S0i*btimexx,2,cumsumstrata,xx$strata,xx$nstrata)
  } else Ht <- NULL
  if (fixbeta==0) rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset)) else rr <- c(xx$sign*exp(xx$offset))
  id <-   xx$id
  mid <- max(id)+1
  MGAiid <- ft*S0i*btimexx-cumS0i2*rr*c(xx$weights)

  MGtiid <- NULL
  if (fixbeta==0) {# {{{
     if (!is.null(beta.iid)) MGtiid <- beta.iid else {
     invhess <- -pinv(x$hessian)
     MGt <- ft*U[,drop=FALSE]-(Z*cumhaz-EdLam0)*rr*c(xx$weights)
     MGt <- MGt %*% invhess
     MGtiid <- apply(MGt,2,sumstrata,id,mid)
    }
     ## Ht efter strata
     Htlast <- tailstrata(xx$strata,xx$nstrata)
     HtS <- Ht[Htlast,,drop=FALSE]
     if (!is.null(x$call.id)) MGtiid <- nameme(MGtiid,x$name.id)
  }# }}}

 ### \hat beta - \beta = \sum_i \beta_i  (iid)
 ### iid after baseline:
 ### \hat A_s-A_s=\sum_{i clusters} \sum_{j: i(j)=i, s(j)=s} \int_0^t 1/S_0 dM^s_{i,j} - P^s(t) \sum_i \beta_i
 ### = \sum_{i clusters} ( \sum_{j \in i(j)=i, s(j)=s} \int_0^t 1/S_0 dM^s_j - P^s(t) \beta_i )

 ## sum after id's within strata and order
 MGAiids <- c()
 cumhaz.time <- c()
 sus <- sort(unique(xx$strata))
 for (i in sus)  {
	 wi <- which(xx$strata==i)
         MGAiidl <- sumstrata(MGAiid[xx$strata==i],xx$id[xx$strata==i],mid)
	 cumhazs <- rbind(0,x$cumhaz[x$strata[x$jumps]==i,])
	 cumhaz.time <- c(cumhaz.time,cpred(cumhazs,time,tminus=tminus)[,2])

        if (fixbeta==0) {
           UU <-  apply(HtS[i+1,]*t(MGtiid),2,sum)
           MGAiidl <- MGAiidl - UU
         }
         MGAiids <- cbind(MGAiids,MGAiidl)
 }
 names(cumhaz.time) <- paste("strata",sus,sep="")
 MGAiid <- MGAiids
 if (is.matrix(MGAiid)) {
    colnames(MGAiid) <- paste("strata",sus,sep="")
   if (!is.null(x$name.id)) MGAiid <- nameme(MGAiid,x$name.id)
    ###    if (length(x$name.id)==nrow(MGAiid)) rownames(MGAiid) <- x$name.id
 }

 cumhaz <- x$cumhaz
 out <- list(time=time,base.iid=MGAiid,strata=xx$strata,nstrata=xx$nstrata,coef=x$coef,
	     cumhaz=cumhaz,cumhaz.time=cumhaz.time,
	     id=id,beta.iid=MGtiid,formula=x$formula,
	     design=object$design)
 class(out) <- c("iidBaseline","phreg")
 return(out)
} # }}}

##' @export
summary.iidBaseline  <- function(object,...) { ## {{{
cat("iid decomposition of object phreg/cifregFG/recreg \n")
} # }}}

##' @export
print.iidBaseline <- function(x,...) { ## {{{
	x$base.iid
} # }}}


##' @export
residuals.phreg  <- function(object,cumsum=FALSE,...) {# {{{
  orig.order <- FALSE
  x <- object

if (is.null(x$propodds)) { # {{{ cox model
	  xx <- x$cox.prep
	  dN <- S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  dN[xx$jumps+1] <- 1
	  cumhaz <- cumsumstrata(S0i,xx$strata,xx$nstrata)
	  Z <- xx$X
	  if (is.null(coef(x)))
		  rr <- c(xx$sign*exp(xx$offset))
          else
	  rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
	  ### dMartingale  as a function of time and for all subjects to handle strata
	  Lamt <- (cumhaz)*rr*c(xx$weights)
	  dMGt <- dN-Lamt
	  if (orig.order) {
	     oo <- (1:nrow(xx$X))[xx$ord+1]
	     oo <- order(oo)
	     ### back to order of iid variable
	     dMGt <- dMGt[oo,,drop=FALSE]   ## sum after id later so not needed
	     id <- xx$id[oo]
	  } else id <-  xx$id
} else { ## }}}
	# {{{  prop-odds model logitSurv
	  xx <- x$cox.prep
	  dN <- S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  dN[xx$jumps+1] <- 1
	  U <- E <- matrix(0,nrow(xx$X),x$p)
	  E[xx$jumps+1,] <- x$E
	  U[xx$jumps+1,] <- x$U
	  S0i <- rep(0,length(xx$strata))
	  S0i[xx$jumps+1] <- 1/x$S0
	  cumhazA <- cumsumstratasum(S0i,xx$strata,xx$nstrata,type="all")
	  cumhaz <- c(cumhazA$sum)
	  cumhazm <- cumhazA$lagsum
          ###
	  EdLam0 <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
	  Z <- xx$X
	  rr <- c(xx$sign*exp(Z %*% coef(x) + xx$offset))
	  rro <- c(exp(Z %*% coef(x) + xx$offset))
          S0star <- revcumsumstrata(rr/(1+rro*cumhazm),xx$strata,xx$nstrata)
          S0 <- revcumsumstrata(rr,xx$strata,xx$nstrata)
          S1 <- apply(Z*rr,2,revcumsumstrata,xx$strata,xx$nstrata)
	  Et <- S1/c(S0)
          lt <- apply((Z-Et)*c(rr*rro/(1+rro*cumhazm)),2,revcumsumstrata,xx$strata,xx$nstrata)
	  Estar <- S0star/S0
	  EstardLam <- cumsumstrata(Estar*S0i,xx$strata,xx$nstrata)
	  k <- exp(-EstardLam)
	  basecor <- apply(lt*c(k*S0i),2,revcumsumstrata,xx$strata,xx$nstrata)
	  basecor <- basecor/c(k*S0)
	  www <- x$propoddsW*x$weightsJ
	  U[xx$jumps+1,] <- U[xx$jumps+1,] - c(www)* basecor[xx$jumps+1,]
	  baseDLam0 <- apply(basecor*S0i,2,cumsumstrata,xx$strata,xx$nstrata)

	  ### Martingale  as a function of time and for all subjects to handle strata
	  MGt <- U[,drop=FALSE]-(Z*cumhaz-EdLam0-baseDLam0)*rr*c(xx$weights)

	  if (orig.order) {
	     oo <- (1:nrow(xx$X))[xx$ord+1]
	     oo <- order(oo)
	     ### back to order of data-set
	     MGt <- MGt[oo,,drop=FALSE]
	     id <- xx$id[oo]
	  } else id <-  xx$id
  }# }}}

  ncluster <- NULL
  if (!cumsum) {
	  mid <- max(id)+1
    Mt <- sumstrata(dMGt,id,mid)
    cumhaz <- sumstrata(Lamt,id,mid)
    out <- list(residuals=c(Mt),cumhaz=c(cumhaz))
  } else {
     mid <- max(id)+1
     out <- data.frame(dMGt=c(dMGt),Lamt=Lamt,status=dN,
		time=c(xx$time),id=c(xx$id)+1,sign=xx$sign)
     dsort(out) <- ~time+status+id-sign
     Mt <- cumsumstrata(out$dMGt,out$id-1,mid)
     cumhaz <- cumsumstrata(out$Lamt,out$id-1,mid)
    out <- cbind(out,Mt,cumhaz)
    out <- subset(out,sign==1)
    ddrop(out)  <- ~sign+dMGt+Lamt
  }

  return(out)
} # }}}

##' Robust Baseline Hazard Standard Errors
##'
##' Computes robust (sandwich) standard errors for the cumulative baseline
##' hazard from a \code{phreg} object.
##'
##' @param x a \code{phreg} object.
##' @param type type of standard error (default \code{"robust"}).
##' @param fixbeta if non-NULL, fixes beta at given value.
##' @param ... additional arguments passed to \code{squareintHdM}.
##' @return A list with \code{cumhaz}, \code{se.cumhaz}, and \code{strata}.
##' @rdname phreg-helpers
##' @export
robust.basehaz.phreg  <- function(x,type="robust",fixbeta=NULL,...) {# {{{

  IsdM <- squareintHdM(x,ft=NULL,fixbeta=fixbeta,...)
  varA <-   IsdM$varInt[x$jumps]
  strata <- x$strata[x$jumps]
  cumhaz <- x$cumhaz
  se.cumhaz <- cbind(cumhaz[,1],varA^.5)
  colnames(se.cumhaz) <- c("time","se.cumhaz")

  return(list(cumhaz=cumhaz,se.cumhaz=se.cumhaz,strata=strata))
} # }}}

##' @export robust_phreg
robust_phreg  <- function(x,fixbeta=NULL,...) {

  if (is.null(fixbeta))
  if ((x$no.opt) | is.null(x$coef)) fixbeta<- 1 else fixbeta <- 0

 if (fixbeta==0)  {
    beta.iid <- iid(x)
    robvar <- crossprod(beta.iid)
 } else robvar <- beta.iid <- NULL
 baseline <- robust.basehaz.phreg(x,fixbeta=fixbeta,...);
 ## add arguments so that we can call basehazplot.phreg
 res <- c(x,list(beta.iid=beta.iid,robvar=robvar,robse.cumhaz=baseline$se.cumhaz))
 class(res) <- "phreg"
 return(res)
}

###}}}

###{{{ phreg: coef vcov print summary print.summary  plot lines

##' @export
coef.phreg  <- function(object,...) { ## {{{
  object$coef
} ## }}}

##' @export
vcov.phreg  <- function(object,...) {     ## {{{
 if (is.null(coef(object))) return(NULL)
 if ((length(class(object))==1) & inherits(object,"phreg")) {
  res <- object$var
  if (!is.null(res)) colnames(res) <- rownames(res) <- names(coef(object))
} else {
  res <- object$var
  if (!is.null(res)) colnames(res) <- rownames(res) <- names(coef(object))
}
  res
} ## }}}

##' @export
summary.phreg <- function(object,type=c("robust","martingale"),augment.type=c("var.augment.times","var.augmenti"),...) { ## {{{
  expC <- cc <- ncluster <- V <- NULL

   if (length(object$p)>0 & object$p>0 & (!object$no.opt)) {
    I <- -pinv(object$hessian)
    if ( (length(class(object))==2) && ( inherits(object,c("cifreg","recreg")))) {
	    V <- object$var
	    ncluster <- object$ncluster ## nrow(object$Uiid)
            if (!is.null(object$augmentation)) { V <- object[[augment.type[1]]]; }
    } else  {  ## phreg
	    V <- vcov(object,type=type[1])
            ncluster <- object$n
    }
    cc <- cbind(coef(object),diag(V)^0.5,diag(I)^0.5)
    cc  <- cbind(cc,2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
    colnames(cc) <- c("Estimate","S.E.","dU^-1/2","P-value")
    if (length(class(object))==1) if (!is.null(ncluster <- attributes(V)$ncluster))
    rownames(cc) <- names(coef(object))
    expC <- exp(lava::estimate(coef=coef(object),vcov=V)$coefmat[,c(1,3,4),drop=FALSE])
  }
  Strata <- levels(object$strata)
  n <- object$n
  res <- list(coef=cc,n=n,nevent=object$nevent,strata=Strata,ncluster=ncluster,var=V,exp.coef=expC)
  class(res) <- "summary.phreg"
  res
} ## }}}

##' Summarize Baseline Hazard from phreg
##'
##' Summarizes cumulative baseline hazard estimates from a phreg object,
##' optionally with robust standard errors.
##'
##' @param object a \code{phreg} object.
##' @param robust logical; if TRUE, uses robust standard errors.
##' @param ... additional arguments.
##' @return An object of class \code{"summary.recurrent"}.
##' @rdname phreg-helpers
##' @export
summarybase.phreg <- function(object,robust=FALSE,...) { ## {{{
  out <- summaryRecurrentobject(object,robust=robust,...)
  class(out) <- "summary.recurrent"
  return(out)
}# }}}


##' @export
print.phreg  <- function(x,...) { ## {{{
  cat("Call:\n")
  dput(x$call)
  cat("\n")

  Strata <- levels(x$strata)
  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- Strata; colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  print(nn, quote=FALSE)

  if (!is.null(x$coef) && length(x$coef) > 0) {
    cat("\ncoefficients:\n")
    ## V <- x$var
    ## if (!is.null(V)) {
    ##   cc <- cbind(coef(x), diag(V)^0.5)
    ##   cc <- cbind(cc, 2*(pnorm(abs(cc[,1]/cc[,2]), lower.tail=FALSE)))
    ##   colnames(cc) <- c("Estimate", "S.E.", "P-value")
    ##   rownames(cc) <- names(coef(x))
    ##   printCoefmat(cc, P.values=TRUE, has.Pvalue=TRUE, ...)
    ## } else {
    print(coef(x))
    ## }
  }
  cat("\n")
  invisible(x)
}  ## }}}

##' Plotting the baselines of stratified Cox
##'
##' Plotting the baselines of stratified Cox
##' @param x phreg object
##' @param ... Additional arguments to baseplot funtion
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases baseplot bplot  basecumhaz plotConfRegion  plotConfRegionSE plotstrata kmplot plotConfregion
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' out1 <- phreg(Surv(time,status==9)~vf+chf+strata(wmicat.4),data=TRACE)
##'
##' par(mfrow=c(2,2))
##' plot(out1)
##' plot(out1,stratas=c(0,3))
##' plot(out1,stratas=c(0,3),col=2:3,lty=1:2,se=TRUE)
##' plot(out1,stratas=c(0),col=2,lty=2,se=TRUE,polygon=FALSE)
##' plot(out1,stratas=c(0),col=matrix(c(2,1,3),1,3),lty=matrix(c(1,2,3),1,3),se=TRUE,polygon=FALSE)
##' @aliases plotConfRegion  plotConfRegionSE plotstrata kmplot plotConfregion
##' @export
plot.phreg  <- function(x,...)  baseplot(x,...)

##' @export
lines.phreg <- function(x,...,add=TRUE) plot(x,...,add=add)

##' @export
bplot <- function(x,...,add=TRUE) plot(x,...,add=add)

##' @export
print.summary.phreg  <- function(x,max.strata=5,...) { ## {{{

  if (length(class(x))==2 & inherits(x,"cifreg")) cat("Competing risks regression \n");
  if (!is.null(x$propodds)) {
       cat("Proportional odds model, log-OR regression \n");
  } else cat("\n")

  nn <- cbind(x$n, x$nevent)
  rownames(nn) <- levels(x$strata); colnames(nn) <- c("n","events")
  if (is.null(rownames(nn))) rownames(nn) <- rep("",NROW(nn))
  if (length(x$strata)>max.strata) {
      nn <- rbind(c(colSums(nn),length(x$strata)));
      colnames(nn) <- c("n","events","stratas")
      rownames(nn) <- ""
  }
  print(nn,quote=FALSE)
  if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
  if (!is.null(x$coef)) {
    cat("coefficients:\n")
    printCoefmat(x$coef,...)
    cat("\n")
    cat("exp(coefficients):\n")
    printCoefmat(x$exp.coef,...)
  }
  cat("\n")

} ## }}}

###}}}

##' Confidence Intervals with Transformations
##'
##' Computes confidence intervals using log or plain transformations,
##' with optional restrictions to positive values or probability scale.
##'
##' @param x point estimate(s).
##' @param std.err standard error(s).
##' @param conf.type type of transformation: \code{"log"} or \code{"plain"}.
##' @param restrict restriction: \code{"positive"}, \code{"prob"}, or \code{"none"}.
##' @param conf.int confidence level (default 0.95).
##' @return A list with \code{upper}, \code{lower}, \code{conf.type}, and \code{conf.int}.
##' @rdname phreg-helpers
##' @aliases conftype summarybase.phreg robust.basehaz.phreg
##' @export
conftype <- function(x,std.err,conf.type=c("log","plain"),restrict=c("positive","prob","none"),conf.int=0.95)
{ ## {{{
zval <- qnorm(1 - (1 - conf.int)/2, 0, 1)
 if (conf.type[1] == "plain") {# {{{
    temp1 <- x + zval * std.err
    temp2 <- x - zval * std.err
    if (restrict[1]=="prob") cix <- list(upper = pmin(temp1, 1), lower = pmax(temp2, 0), conf.type = "plain", conf.int = conf.int)
    if (restrict[1]=="positive") cix <- list(upper = pmax(temp1, 0), lower = pmax(temp2, 0), conf.type = "plain", conf.int = conf.int)
    if (restrict[1]=="none") cix <- list(upper = temp1, lower = temp2, conf.type = "plain", conf.int = conf.int)
 }
 if (conf.type[1] == "log") {
    xx <- ifelse(x <= 0, NA, x)
    temp1 <- ifelse(x <= 0, NA, exp(log(xx) + zval * std.err/xx))
    temp2 <- ifelse(x <= 0, NA, exp(log(xx) - zval * std.err/xx))
    if (restrict[1]=="prob") cix <- list(upper = pmin(temp1, 1), lower = temp2, conf.type = "log", conf.int = conf.int)
    else cix <- list(upper = temp1, lower = temp2, conf.type = "log", conf.int = conf.int)
 }
### if (conf.type[1] == "log-log") {
###    who <- (x == 0 | x == 1)
###    xx <- ifelse(who, NA, x)
###    temp1 <- exp(-exp(log(-log(xx)) + zval * std.err/log(xx)))
###    temp2 <- exp(-exp(log(-log(xx)) - zval * std.err/log(xx)))
###    if (restrict[1]=="prob") {
###	    temp3 <- ifelse(x == 0, NA, 1)
###	    temp1 <- ifelse(who, temp3, temp1)
###	    temp2 <- ifelse(who, temp3, temp2)
###    } else {
###    cix <- list(upper = temp1, lower = temp2, conf.type = "log-log", conf.int = conf.int)
###    }
### }# }}}
 return(cix)
}# }}}

basecumhaz <- function(x,type=c("list"),only=0,joint=0,cumhaz="cumhaz",se.cumhaz="se.cumhaz",robust=FALSE,columns=NULL,...) {# {{{
   ## all strata
   stratobs <- x$strata[x$jumps]
   ustratobs <- sort(unique(stratobs))
   stratas <- 0:(x$nstrata-1)
   ## default 
   if (is.null(columns)) columns <- 1:2

   se.cum <- cum <- c()
   se.cum <- cum <- x[[cumhaz]]
   if (robust==TRUE) {
	   ## take robse if there otherwise stay with se.cumhaz
	   secum <- x$robse.cumhaz
	   if (is.null(secum)) secum <- x[[se.cumhaz]]
   } else secum  <- x[[se.cumhaz]]
   if (is.null(secum)) nose <- TRUE else nose <- FALSE

   out <- rep(list(NULL),x$nstrata)
   if (length(ustratobs)>0)
   for (i in ustratobs) {
	   if (!is.null(x[[cumhaz]])) {
	   cumhazard <- x[[cumhaz]][stratobs==i,columns,drop=FALSE]
           nr <- nrow(cumhazard)
	   if (nr>=1) {
		   if (!nose) se.cum <- secum[stratobs==i,columns,drop=FALSE] else se.cum <- NULL
		   if (only==0) {
			  if (joint==0) out[[i+1]] <- list(cumhaz=cumhazard,se.cumhaz=se.cum,strata=i) else
		                        out[[i+1]] <- list(cumhaz=cbind(cumhazard,se.cum[,2],i))
	           } else out[[i+1]] <- cumhazard
         }
	 }
   }

   class(out) <- "basecumhaz"
   attr(out,"stratobs") <- ustratobs
   return(out)
}# }}}

baseplot  <- function(x,se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,
     lty=NULL,col=NULL,lwd=NULL,legend=TRUE,legend.args=list(),
     ylab="Cumulative hazard",xlab="time",
     polygon=TRUE,level=0.95,stratas=NULL,robust=FALSE,cumhaz="cumhaz",
     se.cumhaz="se.cumhaz",
     conf.type=c("log","plain"),restrict = c("positive","prob", "none"),...) {# {{{

###   if (!inherits(x,"phreg")) stop("must be phreg/recreg/ \n")

   base <- basecumhaz(x,joint=1,robust=robust,cumhaz=cumhaz,se.cumhaz=se.cumhaz)
   nstrata <- x$nstrata
   stratobs <- attr(base,"stratobs")
   ###
   if (is.null(stratas)) stratas <- stratobs else {
         wm <-   match(stratas,stratobs)
         wm <- wm[!is.na(wm)]
         if (length(wm)==0) stratas <- rep(0,0) else stratas <- stratobs[wm]
   }

   ## all confidence intervals for relevant strata  to make ylim
   if (length(stratobs)>0) {
   xx <- conftype(x$cumhaz[,2],x$se.cumhaz[,2],conf.type=conf.type[1],restrict=restrict[1],conf.int=level)

   if (is.null(xlim)) {
	   xlim <-  range(x$cumhaz[,1],na.rm=TRUE)
   }
   if (se) {
       if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz)) stop("phreg must be with cumhazard=TRUE\n");
       rrse <- range(c(xx$lower,xx$upper),na.rm=TRUE)
       if (is.null(ylim))
       {
        if (Inf %in% abs(rrse)) ylim <- range(x$cumhaz[,2],na.rm=TRUE) else ylim <- rrse
       }
   } else {
       if (is.null(ylim)) ylim <- range(x$cumhaz[,2],na.rm=TRUE)
   }

   ## all strata
   ## {{{
   ltys <- lty; cols <- col; lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   stratnames <- paste(stratn,lstrata,sep=":")

      if (!is.matrix(lty)) {
         if (is.null(lty)) lty <- ltys <- 1:length(stratas)
         if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas)) else ltys <- lty
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) col <- cols <- 1:length(stratas)
	 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas)) else cols <- col
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwd <- lwds <- rep(1,length(stratas))
	 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas)) else lwds <- lwd
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)
  ## }}}

  first <- 1
  j <- 1
  for (i in stratas)  {
	if (!is.null(base[[i+1]])) {
	cumhazard <- base[[i+1]]$cumhaz
	if (nrow(cumhazard)>1) {
        if (add | first==0) lines(cumhazard,type="s",lty=ltys[j,1],col=cols[j,1],lwd=lwds[j,1])
        else {
	  first <- 0
          plot(cumhazard,type="s",lty=ltys[j,1],col=cols[j,1],lwd=lwds[j,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
        }
        if (se==TRUE) {
            xx <- conftype(cumhazard[,2],cumhazard[,3],conf.type=conf.type[1],restrict=restrict[1],conf.int=level)
            nl <- cbind(cumhazard[,1],xx$lower); ul <- cbind(cumhazard[,1],xx$upper)
	      if (!polygon) {
		  lines(nl,type="s",lty=ltys[j,2],col=cols[j,2],lwd=lwds[j,2],...)
		  lines(ul,type="s",lty=ltys[j,3],col=cols[j,3],lwd=lwds[j,3],...)
	      } else plotConfRegion(nl[,1],cbind(nl[,2],ul[,2]),col=cols[j,1])
        }
     }
     }
    j <- j+1
  }

     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }
   } else {
	   if (!add) plot(0,0,ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
   }
}# }}}

###{{{ phreg: basehazplot.phreg predictphreg 

basehazplot.phreg  <- function(x,se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,
			       lwd=NULL,legend=TRUE,legend.args=list(),
                   ylab=NULL,xlab=NULL,polygon=TRUE,level=0.95,
			       stratas=NULL,robust=FALSE,conf.type=c("plain","log"),...) {# {{{
   if (inherits(x,"phreg") & is.null(ylab)) ylab <- "Cumulative hazard"
   if (is.null(xlab)) xlab <- "time"
   level <- -qnorm((1-level)/2)
   rr <- range(x$cumhaz[,-1])
   strat <- x$strata[x$jumps]
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$cumhaz[,1])
   if (se==TRUE) {
	   if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz)) stop("phreg must be with cumhazard=TRUE\n");
       if (conf.type[1]=="plain")
       rrse <- range(c(x$cumhaz[,-1]+level*x$se.cumhaz[,-1]))
       else { ## type="log"
	       relse <- exp(level*x$se.cumhaz[,-1]/x$cumhaz[,-1])
	       rrse <- range(x$cumhaz[,-1]/relse,x$cumhaz[,-1]*relse)
       }
       if (inherits(x,"km")) rrse <- c(min(x$lower,na.rm=TRUE),1)
       if (is.null(ylimo)) ylim <- rrse
   }

   ## all strata
   if (is.null(stratas)) stratas <- 0:(x$nstrata-1)
   ltys <- lty
   cols <- col
   lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   stratnames <- paste(stratn,lstrata,sep=":")

      if (!is.matrix(lty)) {
         if (is.null(lty)) lty <- ltys <- 1:length(stratas)
         if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas)) else ltys <- lty
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) col <- cols <- 1:length(stratas)
	 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas)) else cols <- col
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwd <- lwds <- rep(1,length(stratas))
	 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas)) else lwds <- lwd
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)


  first <- 0
  for (i in seq(stratas)) {
      j <- stratas[i]
        cumhazard <- x$cumhaz[strat==j,,drop=FALSE]
        if (!is.null(cumhazard)) {
	if (nrow(cumhazard)>1) {
        if (add | first==1)
        lines(cumhazard,type="s",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1])
       else {
	  first <- 1
          plot(cumhazard,type="s",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
       }
       if (se==TRUE) {
	    if (robust==TRUE) secumhazard  <- x$robse.cumhaz[strat==j,,drop=FALSE]
	    else secumhazard <- x$se.cumhaz[strat==j,,drop=FALSE]
	    xx <- cumhazard[,2]
	    std.err <- secumhazard[,2]
	 if (conf.type[1] == "log") {
	    temp1 <-  exp(log(xx) + level * std.err/xx)
	    temp2 <-  exp(log(xx) - level * std.err/xx)
	    ul = cbind(cumhazard[,1],temp1);
	    nl <- cbind(cumhazard[,1],temp2);
	 }
	 if (conf.type[1] == "plain") {
		 ul <-cbind(cumhazard[,1],cumhazard[,2]+level*secumhazard[,2])
		 nl <-cbind(cumhazard[,1],cumhazard[,2]-level*secumhazard[,2])
	 }
	 if (inherits(x,"km")) { ul[,2] <- x$upper[x$strata==j];
	                          nl[,2] <- x$lower[x$strata==j];
	                          wna <- which(is.na(ul[,2]))
	                          ul[wna,2] <- 0
	                          nl[wna,2] <- 0

	 }
	      if (!polygon) {
		  lines(nl,type="s",lty=ltys[i,2],col=cols[i,2],lwd=lwds[i,2],...)
		  lines(ul,type="s",lty=ltys[i,3],col=cols[i,3],lwd=lwds[i,3],...)
	      } else plotConfRegion(nl[,1],cbind(nl[,2],ul[,2]),col=cols[j+1])
        }
     }
     }
    }

     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }

}# }}}

###}}} plot

plotpredictphreg <- function(x,se=FALSE,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,
       type=c("surv","cumhaz","cif"),ylab=NULL,xlab=NULL,
       polygon=TRUE,whichx=NULL,robust=FALSE,...) {# {{{
   if (type[1]=="surv" & is.null(ylab)) ylab <- "Surival probability"
   if (type[1]=="cumhaz" & is.null(ylab)) ylab <- "Cumulative hazard"
   if (type[1]=="cif" & is.null(ylab)) ylab <- "Cumulative probability"

   if (is.null(xlim)) xlim <- range(x$times)
   if (is.null(xlab)) xlab <- "time"
   if (is.null(x$se.cumhaz) & se==TRUE)  {
	  warning("predict.phreg must be with se=TRUE\n");
          se <- FALSE
   }
   ylimo <- ylim
   nlower <- paste(type[1],".lower",sep="")
   nupper <- paste(type[1],".upper",sep="")
   xx <- x[[type[1]]]

   if (se==TRUE) {
      upper <- x[[nupper]]
      lower <- x[[nlower]]
      if (is.null(x$se.cumhaz)) stop("predict.phreg/cifreg/recreg must be with se=TRUE\n");
      rrse <- range(c(0,x[[nlower]],x[[nupper]]),na.rm=TRUE)
      if (is.null(ylimo)) ylim <- rrse
   } else {
      rrse <- range(c(xx),na.rm=TRUE)
      if (is.null(ylimo)) ylim <- rrse
   }

   ## all covriates
   nx <- nrow(x$surv)
   if (is.null(whichx)) whichx <- 1:nx
   stratas <- whichx
   ltys <- lty
   cols <- col

   if (length(whichx)>0 ) { ## with X

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(whichx) else
		 if (length(lty)!=length(whichx)) ltys <- rep(lty[1],length(whichx)) else ltys <- lty
      } else ltys <- lty

      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
   } else {
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)

  i <- 1
  j <- whichx[i]

   if (!add)
  plot(x$times,xx[j,],type="s",lty=ltys[i,1],col=cols[i,1],ylim=ylim,ylab=ylab,xlim=xlim,xlab=xlab,...)
  else lines(x$times,xx[j,],type="s", lty=ltys[i,1],col=cols[i,1],...)
  if (length(whichx)>1)
  for (i in seq(2,length(whichx))) lines(x$times,xx[whichx[i],],type="s",lty=ltys[i,1],col=cols[i,1],...)

    if (se==TRUE) {
    for (i in seq(1,length(whichx))) {
      j <- whichx[i]
      ul <- upper[j,]; nl <- lower[j,];
      if (!polygon) {
      lines(x$times,nl,type="s",lty=ltys[i,2],col=cols[i,2])
      lines(x$times,ul,type="s",lty=ltys[i,3],col=cols[i,3])
      } else plotConfRegion(x$times,cbind(nl,ul),col=cols[i,1])
    }
  }

  where <- "topleft";
  if (type[1]=="surv") where <-  "topright"

}# }}}

###{{{ predict.phreg print.predict plot.predict.phreg

##' Predictions from Proportional Hazards Model
##'
##' Computes predictions for survival probability, cumulative hazard, or risk at specified
##' time points for new data or existing data. Includes standard errors and confidence intervals.
##'
##' @param object Object of class \code{"phreg"}.
##' @param newdata Data frame for prediction. If \code{NULL}, predictions are made for the original data.
##' @param times Time points for prediction. Defaults to all unique event times in the model.
##' @param individual.time Logical; if \code{TRUE}, uses one time per subject (requires \code{newdata} and \code{times} to be same length).
##' @param tminus Logical; if \code{TRUE}, predicts at \eqn{t-} (strictly before \eqn{t}).
##' @param se Logical; if \code{TRUE}, computes standard errors and confidence intervals.
##' @param robust Logical; if \code{TRUE}, uses robust standard errors (default for most functions).
##' @param conf.type Transformation for survival estimates: \code{"log"} (default) or \code{"plain"}.
##' @param conf.int Confidence level (default 0.95).
##' @param km Logical; if \code{TRUE}, uses Kaplan-Meier product-limit for baseline; otherwise uses exponential of cumulative baseline.
##' @param ... Additional arguments for plotting functions.
##'
##' @return An object of class \code{"predictphreg"} containing:
##' \item{surr}{Matrix of survival probabilities.}
##' \item{cumhaz}{Matrix of cumulative hazards.}
##' \item{cif}{Matrix of cumulative incidence functions (if applicable).}
##' \item{times}{Vector of time points.}
##' \item{surv.upper, surv.lower}{Confidence bounds for survival.}
##' \item{RR}{Relative risks.}
##'
##' @author Thomas Scheike
##' @export
predict.phreg <- function(object,newdata,times=NULL,individual.time=FALSE,tminus=FALSE,se=TRUE,robust=FALSE,conf.type="log",conf.int=0.95,km=FALSE,...)
{# {{{ default is all time-points from the object
### take baseline and strata from object# {{{
ocumhaz <- object$cumhaz
jumptimes <- ocumhaz[,1]
chaz <- ocumhaz[,2]
if (is.null(object$nstrata)) {  ## try to make more robust
nstrata <- 1;
strata <- rep(1,length(jumptimes))
} else {
nstrata <- object$nstrata
strata <- object$strata[object$jumps]
}
if (length(jumptimes)==0) se <- FALSE
if (se) {
if (!robust) {
   se.chaz <- object$se.cumhaz[,2]
   varbeta <- object$ihessian
   if (!object$no.opt) Pt <- apply(object$E/c(object$S0),2,cumsumstrata,strata,nstrata)
   else Pt <- 0
} else { ## only relevant for phreg objects
  if ((object$no.opt) | is.null(object$coef)) fixbeta<- 1 else fixbeta <- 0
  IsdM <- squareintHdM(object,ft=NULL,fixbeta=fixbeta,...)
  ###
  se.chaz <-   IsdM$varInt[object$jumps]^.5
  covv <- IsdM$covv[object$jumps,,drop=FALSE]
  varbeta <- IsdM$vbeta
  Pt <- IsdM$Ht[object$jumps,,drop=FALSE]
}
} # }}}

### setting up newdata with factors and strata
if (!is.null(newdata)) 
x <- update_design(object$design,data = newdata,response=FALSE)
else x <- object$design
X <- x$x
if (!is.null(x$strata)) strataNew <- as.numeric(x$strata)-1 else strataNew <- rep(0,nrow(X))

if (is.null(times)) times <- sort(unique(c(object$exit)))
if (individual.time & is.null(times)) times <- c(object$exit)
if (individual.time & length(times)==1) times <- rep(times,length(object$exit))

    se.cumhaz <- NULL
    if (!individual.time) {
       pcumhaz <- surv <- matrix(0,nrow(X),length(times))
       if (se) se.cumhaz <- matrix(0,nrow(X),length(times))
    } else {
        pcumhaz <- surv <- matrix(0,nrow(X),1)
        if (se) se.cumhaz <- matrix(0,nrow(X),1)
    }
    hazt <- length(times)

    for (j in unique(strataNew)) {
        where <- predictCumhaz(c(0,jumptimes[strata==j]),times,type="left",tminus=tminus)
	plhazt <- hazt <- c(0,chaz[strata==j])
	if (km) { plhazt <- suppressWarnings(c(1,exp(cumsum(log(1-diff(hazt))))));  plhazt[is.na(hazt)] <- 0 }
	if (se) se.hazt <- c(0,se.chaz[strata==j])
	Xs <- X[strataNew==j,,drop=FALSE]
        if (object$p==0) RR <- rep(1,nrow(Xs)) else RR <- c(exp( Xs %*% coef(object)))
	if (se)  { # {{{ based on Hazard's
		if (object$p>0) {
			Ps <- Pt[strata==j,,drop=FALSE]
			Ps <- rbind(0,Ps)[where,,drop=FALSE]
			Xbeta <- Xs %*% varbeta
			seXbeta <- rowSums(Xbeta*Xs)^.5
			cov2 <- cov1 <- Xbeta %*% t(Ps*hazt[where])
		        if (robust)	{
			   covvs <- covv[strata==j,,drop=FALSE]
			   covvs <- rbind(0,covvs)[where,,drop=FALSE]
                           covv1 <- Xs %*% t((covvs*hazt[where]))
			   cov1 <- cov1-covv1
			}
		} else cov1 <- 0
	}# }}}
   haztw <- hazt[where]
   if (se) se.haztw <- se.hazt[where]
	if (is.null(object$propodds)) {
           plhaztw <- plhazt[where]
 	   if (!individual.time) pcumhaz[strataNew==j,]  <- RR%o%haztw else pcumhaz[strataNew==j,] <- RR*haztw[strataNew==j]
           if (!km) {
	     if (!individual.time) surv[strataNew==j,]  <- exp(- RR%o%haztw)
	     else surv[strataNew==j,]  <- exp(-RR*haztw[strataNew==j])
	   } else {
             if (!individual.time) surv[strataNew==j,]  <- exp( RR %o% log(plhaztw))
	     else surv[strataNew==j,]  <- plhaztw[strataNew==j]^RR
	   }
	} else {
	  if (!individual.time) surv[strataNew==j,]  <- 1/(1+RR%o%haztw)
          else surv[strataNew==j,]  <- 1/(1+RR*haztw[strataNew==j])
	}
	if (se) {# {{{
	    if (object$p>0)  {
	       if (!individual.time) se.cumhaz[strataNew==j,]  <-
		     ((RR %o% se.haztw)^2+(c(RR*seXbeta) %o% haztw)^2-2*RR^2*cov1)^.5
	        else se.cumhaz[strataNew==j,]  <- RR* (se.haztw^2+(c(seXbeta)*haztw)^2-2*diag(cov1))^.5
	    } else {
	       if (!individual.time) se.cumhaz[strataNew==j,]  <- RR %o% (se.haztw)
	        else se.cumhaz[strataNew==j,]  <- RR* se.haztw[strataNew==j]
	    }
	}# }}}
    }

  if (se)  {
     cibase <- conftype(pcumhaz,se.cumhaz,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
     cisurv <- conftype(surv,surv*se.cumhaz,conf.type=conf.type[1],restrict="prob",conf.int=conf.int)
  } else {
     cibase <- list(upper=NULL,lower=NULL,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
     cisurv <- list(upper=NULL,lower=NULL,conf.type=conf.type[1],restrict="positive",conf.int=conf.int)
  }

 if (object$p>0) RR <-  exp(X %*% coef(object)) else RR <- rep(1,nrow(X))

### non-cox setting
if (!is.null(object$propodds)) pcumhaz <- -log(surv)

 out <- list(surv=surv,times=times,
    surv.upper=cisurv$upper,surv.lower=cisurv$lower,cumhaz=pcumhaz,se.cumhaz=se.cumhaz,
    se.surv=surv*se.cumhaz,se.cif=surv*se.cumhaz,
    cif=1-surv,cif.lower=1-cisurv$upper,cif.upper=1-cisurv$lower,
    cumhaz.upper=cibase$upper,cumhaz.lower=cibase$lower,strata=strataNew,X=X,RR=RR,id=x$cluster)

class(out) <- c("predictphreg",class(object)[1])
return(out)
}# }}}

##' @export
model.frame.phreg <- function(formula, data = NULL, ...) {
    if (is.null(data)) {
        return(formula$model.frame)
    }
    model.frame(formula$design$formula, data = data, ...)
}

##' @export
model.matrix.phreg <- function(object, data=NULL, ...) {
  if (is.null(data)) return(object$design$x)
  update_design(object$design, data=data, ...)$x
}

##' @export
summary.predictphreg <- function(object, type = c("surv", "cumhaz", "cif")[1],
                                 times = NULL, np = 10, extend = FALSE,
                                 digits = 4, ...) { ## {{{ 
  # ---- resolve time indices ----------------------------------------
  call.times <- times
  if (is.null(times)) {
    indexcol <- seq(ncol(object$surv))
    times    <- object$times
  } else {
    if (!is.numeric(times))
      stop("'times' must be numeric or NULL\n")
    if (extend) {
      indexcol <- predictCumhaz(c(0, object$times), times, return.index = TRUE)
    } else {
      indexcol <- predictCumhaz(object$times, times, return.index = TRUE)
      times[indexcol == 0] <- NA
      if (any(indexcol == 0))
        warning("Some requested times are before the first event time, returning NA\n")
    }
  }

  # ---- resolve subject (row) indices --------------------------------
  n_subj <- nrow(object[[type[1]]])
  if (is.null(np)) {
    ids <- seq(n_subj)
  } else {
    if (!is.numeric(np))
      stop("'np' must be row ids, a count, or NULL\n")
    if (length(np) > 1)       ids <- np           # explicit ids
    else if (np >= n_subj)    ids <- seq(n_subj)  # np >= data size
    else                      ids <- seq(np)       # first np rows
  }

  # ---- extract the requested quantity --------------------------------
  qty <- type[1]
  if (is.null(object[[qty]]))
    stop(sprintf("Type '%s' not found in predict object\n", qty))

  mat      <- object[[qty]]
  lower    <- object[[paste0(qty, ".lower")]]
  upper    <- object[[paste0(qty, ".upper")]]
  se.out   <- object[[paste0("se.", qty)]]

  # ---- extend with time-0 boundary if requested ---------------------
  if (extend) {
    boundary <- if (qty == "surv") 1 else 0
    mat    <- cbind(boundary, mat)
    se.out <- if (!is.null(se.out)) cbind(0, se.out) else NULL
    lower  <- if (!is.null(lower))  cbind(boundary, lower)  else NULL
    upper  <- if (!is.null(upper))  cbind(boundary, upper)  else NULL
  }

  # ---- slice and return ---------------------------------------------
  has_ci <- length(lower) > 1
  if (has_ci) {
    out <- list(
      pred    = mat[ids, indexcol, drop = FALSE],
      se.pred = se.out[ids, indexcol, drop = FALSE],
      lower   = lower[ids, indexcol, drop = FALSE],
      upper   = upper[ids, indexcol, drop = FALSE],
      times   = times,
      rows    = ids
    )
  } else {
    out <- list(
      pred  = mat[ids, indexcol, drop = FALSE],
      times = times,
      rows  = ids
    )
  }

  out$call.times <- call.times
  out$type       <- qty
  class(out)     <- "summary.predictphreg"
  out
} ## }}} 

##' @export
print.summary.predictphreg <- function(x, digits = 4, ...) { ## {{{
  has_ci <- !is.null(x$lower)
  cat(sprintf("Predictions of type '%s'\n", x$type))
  cat(sprintf("  Showing subjects: %s\n",
              paste(x$rows, collapse = ", ")))
  cat(sprintf("  Showing times:    %s\n\n",
              paste(round(x$times, digits), collapse = ", ")))

  for (i in seq_along(x$rows)) {
    cat(sprintf("-- Subject %d --\n", x$rows[i]))
    entry <- data.frame(time = round(x$times, digits),
                        pred = round(x$pred[i, ], digits))
    names(entry)[2] <- x$type
    if (has_ci) {
      entry$se    <- round(x$se.pred[i, ], digits)
      entry$lower <- round(x$lower[i, ],   digits)
      entry$upper <- round(x$upper[i, ],   digits)
    }
    print(entry, row.names = FALSE)
    cat("\n")
  }
  invisible(x)
} ## }}} 

##' @export
print.predictphreg <- function(x, type = c("surv", "cumhaz", "cif")[1],
                               times = NULL, np = 10, digits = 4, ...) { ## {{{ 
  # Run summary to get the sliced data
  s <- summary(x, type = type[1], times = times, np = np, ...)

  n_subj  <- nrow(x[[type[1]]])
  n_times <- ncol(x[[type[1]]])
  has_ci  <- !is.null(s$lower)

  cat(sprintf("Predictions of type '%s'\n", type[1]))
  cat(sprintf("  Subjects: %d total, showing %d\n", n_subj,  length(s$rows)))
  cat(sprintf("  Times:    %d total, showing %d\n", n_times, length(s$times)))
  if (has_ci) cat("  Confidence intervals available\n")
  cat("\n")

  # Build a long-format data frame: one row per (subject, time)
  rows <- s$rows
  tms  <- round(s$times, digits)
  pred <- s$pred

  df_list <- vector("list", length(rows))
  for (i in seq_along(rows)) {
    entry <- data.frame(
      subject = rows[i],
      time    = tms,
      pred    = round(pred[i, ], digits)
    )
    names(entry)[3] <- type[1]
    if (has_ci) {
      entry$lower <- round(s$lower[i, ], digits)
      entry$upper <- round(s$upper[i, ], digits)
      entry$se    <- round(s$se.pred[i, ], digits)
    }
    df_list[[i]] <- entry
  }
  out_df <- do.call(rbind, df_list)
  rownames(out_df) <- NULL

  print(out_df, row.names = FALSE)
  invisible(x)
} ## }}} 


##' @export
plot.predictphreg  <- function(x,se=FALSE,add=FALSE,ylim=NULL,xlim=NULL,lty=NULL,col=NULL,type=c("surv","cumhaz","cif"),ylab=NULL,xlab=NULL,
    polygon=TRUE,level=0.95,whichx=NULL,robust=FALSE,...) {# {{{
   if (type[1]=="cumhaz" & is.null(ylab)) ylab <- "Cumulative hazard"
   if (type[1]=="cif" & is.null(ylab)) ylab <- "Cumulative probability"
   if (type[1]=="surv" & is.null(ylab)) ylab <- "Surival probability"

   if (is.null(xlab)) xlab <- "time"
   if (type[1]=="cif") rr <- c(0,1)
   if (type[1]=="surv") rr <- c(0,1)
   if (type[1]=="cumhaz") rr <- range(c(0,x$cumhaz))
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$times)
   if (is.null(x$se.cumhaz) & se==TRUE)  {
	  warning("predict.phreg must be with se=TRUE\n");
          se <- FALSE
   }
   if (se==TRUE) {
      if (is.null(x$se.cumhaz)) stop("predict.phreg must be with se=TRUE\n");
   if (type[1]=="surv") rrse <- range(c(x$surv.upper,x$surv.lower,na.rm=TRUE))
   if (type[1]=="surv") rrse <- range(c(x$surv.upper,x$surv.lower,na.rm=TRUE))
   if (type[1]=="cumhaz") {
	   cumhaz.upper <- x$cumhaz+level*x$se.cumhaz
	   cumhaz.lower <- x$cumhaz-level*x$se.cumhaz
       rrse <- range(c(cumhaz.upper,cumhaz.lower),na.rm=TRUE)
   }
   if (type[1]=="surv") rrse <- c(0,1)
   if (type[1]=="cif") rrse <- c(0,1)
   if (type[1]=="cumhaz") rrse <- c(max(0,rrse[1]),rrse[2])
   if (is.null(ylimo)) ylim <- rrse
   }

   ## all covriates
   nx <- nrow(x$surv)
   if (is.null(whichx)) whichx <- 1:nx
   stratas <- whichx

   ltys <- lty
   cols <- col

   if (length(whichx)>0 ) { ## with X

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(whichx) else
		 if (length(lty)!=length(whichx)) ltys <- rep(lty[1],length(whichx)) else ltys <- lty
      } else ltys <- lty

      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
   } else {
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)

  i <- 1
  j <- whichx[i]

  recreg <- cifreg <- FALSE
  if ((length(class(x))==2) && (substr(class(x)[2],1,3)=="cif")) cifreg <- TRUE
###  if ((length(class(x))==2) && (substr(class(x)[2],1,3)=="rec")) recreg <- TRUE
  if (type[1]=="surv") {
	  xx <- x$surv
	  if (cifreg)  xx <- x$cif
	  if (recreg)  xx <- x$mean
  } else if (type[1]=="cif") {
	  xx <- 1-x$surv
  } else xx <- x$cumhaz
  if (se) {
     if (type[1]=="surv") {
	     upper <- x$surv.upper; lower <- x$surv.lower
	     if (cifreg) { upper <- x$cif.upper; lower <- x$cif.lower}
	     if (recreg) { upper <- x$mean.upper; lower <- x$mean.lower}
     } else if (type[1]=="cif") {
	     upper <- 1-x$surv.lower; lower <- 1-x$surv.upper
     } else { upper <- cumhaz.upper; lower <- cumhaz.lower}
  }

  if (!add)
  plot(x$times,xx[j,],type="s",lty=ltys[i,1],col=cols[i,1],ylim=ylim,ylab=ylab,xlim=xlim,xlab=xlab,...)
  else lines(x$times,xx[j,],type="s", lty=ltys[i,1],col=cols[i,1],...)
  if (length(whichx)>1)
  for (i in seq(2,length(whichx))) lines(x$times,xx[whichx[i],],type="s",lty=ltys[i,1],col=cols[i,1],...)

    if (se==TRUE) {
    for (i in seq(1,length(whichx))) {
      j <- whichx[i]
      ul <- upper[j,]; nl <- lower[j,];
      if (!polygon) {
      lines(x$times,nl,type="s",lty=ltys[i,2],col=cols[i,2])
      lines(x$times,ul,type="s",lty=ltys[i,3],col=cols[i,3])
      } else plotConfRegion(x$times,cbind(nl,ul),col=cols[i,1])
    }
  }

  where <- "topleft";
  if (type[1]=="surv") where <-  "topright"

}# }}}

###}}} predict

# {{{ Restricted mean for stratified Kaplan-Meier with martingale standard errors

##' Restricted Mean for Stratified Kaplan-Meier or Cox Model
##'
##' Computes the Restricted Mean Survival Time (RMST) for stratified Kaplan-Meier or
##' stratified Cox models with martingale standard errors.
##'
##' The standard error is computed using linear interpolation between standard errors at jump-times.
##' This allows plotting the restricted mean as a function of time.
##'
##' Years lost can be computed based on this and decomposed into years lost for different causes
##' using the \code{cif_yearslost} function.
##'
##' @param x Object of class \code{"phreg"}.
##' @param times Possible times for which to report restricted mean. If \code{NULL}, reports for all event times.
##' @param covs Possible covariates for Cox model adjustment.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"resmean_phreg"} containing:
##' \item{rmst}{Matrix of restricted mean survival times.}
##' \item{se.rmst}{Standard errors for RMST.}
##' \item{intkmtimes}{Restricted mean at specified times.}
##' \item{years.lost}{Years lost (if applicable).}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' out1 <- phreg(Surv(time, cause != 0) ~ strata(tcell, platelet), data = bmt)
##'
##' rm1 <- resmean_phreg(out1, times = 10 * (1:6))
##' summary(rm1)
##' e1 <- estimate(rm1)
##' par(mfrow = c(1, 2))
##' plot(rm1, se = 1)
##' plot(rm1, years.lost = TRUE, se = 1)
##'
##' ## Comparing populations
##' rm1 <- resmean_phreg(out1, times = 40)
##' e1 <- estimate(rm1)
##' estimate(e1, rbind(c(1, -1, 0, 0)))
##' @aliases rmst_phreg resmean_phreg
##' @export
resmean_phreg <- function(x,times=NULL,covs=NULL,...)
{# {{{
  ii <- invhess <- x$II

 if (!is.null(times)) {
	   tt <- expand.grid(times,0:(x$nstrata-1))
           mm <- cbind(x$jumptimes,x$strata.jumps,1)
           mm <- rbind(mm,cbind(tt[,1],tt[,2],0))
           ord <- order(mm[,1])
           mm <- mm[ord,]
	   phd <- which(mm[,3]==1)
	   newtimes <- which(mm[,3]==0)
	   S0i <- S0i2 <- rep(0,nrow(mm))
	   S0i[phd] <- c(1/x$S0)
	   S0ii2 <- c(1/(x$S0*(x$S0-1)))
	   S0ii2[x$S0==1] <- 0
	   S0i2[phd] <- S0ii2
           strata.jumps <- mm[,2]
	   nstrata <- x$nstrata;
	   jumptimes <- mm[,1]
	   E <- matrix(0,nrow(mm),x$p)
	   E[phd,] <- x$E
    }   else {
	   strata.jumps <- x$strata.jumps;
	   nstrata <- x$nstrata;
	   jumptimes <- x$jumptimes;
	   S0i <- c(1/x$S0)
	   S0i2 <- c(1/(x$S0*(x$S0-1)))
	   S0i2[x$S0==1] <- 0
	   E <- x$E
 }

  cumhaz <- cumsumstrata(S0i,strata.jumps,nstrata)
  var.cumhazMG <- cumsumstrata(S0i2,strata.jumps,nstrata)

  ## baseline survival
  km <- exp(cumsumstratasum(log(1-S0i),strata.jumps,nstrata,type="lagsum"))

  ## If covariates compute rr = exp( X beta)  and adjust km
  if (!is.null(covs)) { rr <- exp(sum(covs * coef(x))) } else rr <- 1
  km <- km^rr

  ## start integral in 0
  dtime <- c(diffstrata(jumptimes,strata.jumps,nstrata))
  intkm <-  cumsumstrata( c(km)*dtime,strata.jumps,nstrata)

  ### variance of baseline term
  var.intkmcumhaz <- cumsumstrata(intkm*S0i2,strata.jumps,nstrata)
  var.intkm2cumhaz <- cumsumstrata(intkm^2*S0i2,strata.jumps,nstrata)
  var.resmean <- intkm^2*var.cumhazMG+var.intkm2cumhaz-2*intkm*var.intkmcumhaz

   if (!is.null(x$coef)) {
      intp <- apply(E*S0i,2,cumsumstratasum,strata.jumps,nstrata,type="lagsum")
      intpS <-  apply(intp*c(km)*dtime,2,cumsumstrata,strata.jumps,nstrata)
      Dbeta <- -intpS
      if (!is.null(covs)) {
	      intLam <- cumsumstratasum(S0i,strata.jumps,nstrata,type="lagsum")
	      intLamS <-  cumsumstrata( c(intLam)*km*dtime,strata.jumps,nstrata)
	      Dbeta <- Dbeta + matrix(covs,nrow=nrow(intLam),ncol=length(covs),byrow=TRUE)*c(intLamS)
      }
      varbetat <-   rowSums((Dbeta %*% x$II)*Dbeta)
      var.resmean <- rr^2*(var.resmean+varbetat)
  } else {
      varbetat <- 0
  }

  time<- jumptimes
  meanm <- cbind(time,intkm)
  se.resmean <- var.resmean^.5
  se.resmean[var.resmean<0] <- 0
  se.mm <- cbind(time,se.resmean)

  ### make output at specified times
  if (!is.null(times)) {
    intkmtimes <- meanm[newtimes,,drop=FALSE]
    se.intkmtimes <- se.resmean[newtimes]
    skmtimes <- mm[newtimes,2]
    years.lost <- intkmtimes[,1]-intkmtimes[,2]
    intkmtimes <- cbind(skmtimes,intkmtimes,se.intkmtimes,years.lost)
    colnames(intkmtimes) <- c("strata","times","rmean","se.rmean","years.lost")
    rownames(intkmtimes) <- rep(x$strata.level,length(times));
    intkmtimes=data.frame(intkmtimes)
###    logintkmtimes=cbind(intkmtimes[,1:2],log(intkmtimes[,3]),se.intkmtimes/intkmtimes[,3])
###    colnames(logintkmtimes) <- c("strata","times","log-rmean","log-se.rmean")
     coef <- intkmtimes[,3];
     var  <- diag(intkmtimes[,4]^2)
  } else {
	  coef <- var <- intkmtimes <- se.intkmtimes <- NULL
  }

  rmst <- cbind(meanm[,1],vecAllStrata(meanm[,2],strata.jumps,nstrata))
  se.rmst <- cbind(se.mm[,1],vecAllStrata(se.mm[,2],strata.jumps,nstrata))

 out <- list(cumhaz=meanm,se.cumhaz=se.mm,covs=covs,
       time=time,strata=strata.jumps,nstrata=nstrata,
       jumps=1:length(km),strata.name=x$strata.name,
       strata.level=x$strata.level,
       intkmtimes=intkmtimes, rmst=rmst,se.rmst=se.rmst, coef=coef,var=var)
class(out) <- c("resmean_phreg")
return(out)
}# }}}

##' @export
vcov.resmean_phreg <- function(object,cause=1,...)
{# {{{
  if (is.null(object$intkmtimes)) stop("Only for the times given in the call 'times' \n");
  if (is.na(match("rmean",names(object$intkmtimes)))) name <- paste("se.intF_",cause,sep="") else name <- "se.rmean"
  if (!(name %in% colnames(object$intkmtimes))) stop( paste("Cause not consistent with names, looks for ",name,"\n"))

  rest <- object$intkmtimes[,name]
  inttimes <- object$intkmtimes[,2]
  times <- unique(object$intkmtimes[,2])

  if (length(times)==1)   {
   res <- rest 
   out <- diag(res^2, nrow=length(res))
   if (length(res) == length(object$strata.level)) rownames(out) <- colnames(out) <- object$strata.level
  } else {
	  out <- list()
	  k <- 1
          for (tt in times)  {
		  res <- rest[tt==inttimes]
                  if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
                 res <- diag(res^2, nrow=length(res))
                 if (nrow(res) == length(object$strata.level)) rownames(res) <- colnames(res) <- object$strata.level
		  out[[k]] <- res
		  k <- k+1
           }
  }

  return(out)
}# }}}

##' @export
coef.resmean_phreg <- function(object,cause=1,...) 
{# {{{
  if (is.null(object$intkmtimes)) stop("Only for the times given in the call 'times' \n");
  if (is.na(match("rmean",names(object$intkmtimes)))) name <- paste("intF_",cause,sep="") else name <- "rmean"
  if (!(name %in% colnames(object$intkmtimes))) stop( paste("Cause not consistent with names, looks for ",name,"\n"))
  rest <- object$intkmtimes[,name]
  inttimes <- object$intkmtimes[,2]
  times <- unique(object$intkmtimes[,2])

  if (length(times)==1)   {
   res <- rest 
   if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
   out <- res
  } else {
	  out <- list()
	  k <- 1
          for (tt in times)  {
		  res <- rest[tt==inttimes]
                  if (length(res) == length(object$strata.level)) names(res) <- object$strata.level
		  out[[k]] <- res
		  k <- k+1
           }
  }

  return(out)
}# }}}

##' @export
estimate.resmean_phreg <- function(x,cause=1,...)
{# {{{
  if (is.null(x$intkmtimes)) stop("Only for the times given in the call 'times' \n");

  if (is.na(match("rmean",names(x$intkmtimes)))) name <- paste("se.intF_",cause,sep="") else name <- "se.rmean"

  coeft <- coef(x,cause=cause,...)
  vcovt <- vcov(x,cause=cause,...)
  times <- unique(x$intkmtimes[,2])
  if (length(times)==1)  
   out <- estimate(coef=coeft,vcov=vcovt,...)
  else {
	  out <- list()
          for (k in seq_along(times)) {
	  out[[k]] <- estimate(coef=coeft[[k]],vcov=vcovt[[k]],...)
	  }
  }

  return(out)
}# }}}



##' @export
rmst_phreg <- function(x,times=NULL,covs=NULL,...)
{# {{{
out <- resmean_phreg(x,times=times,covs=covs,...)
return(out)
}# }}}

##' Restricted Mean Time Lost for Competing Risks
##'
##' Computes the Restricted Mean Time Lost (RMTL) for competing risks based on the
##' integrated Aalen-Johansen estimator.
##'
##' A set of time points can be given to be returned in the summary, but the function
##' computes years-lost for all event times and can be plotted/viewed.
##' The RMTL for a specific time-point can also be computed using the \code{rmstIPCW} function.
##'
##' @param formula Formula for \code{phreg} object with \code{strata} to indicate strata, or \code{+1} if no strata.
##' @param data Data frame for calculations.
##' @param times Possible times for which to report restricted mean. Summary displays estimates for these times.
##' @param cens.code Censoring code (needed to separate event codes from censorings).
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"resmean_phreg"} containing:
##' \item{cumhaz}{Matrix of cumulative hazards (years lost).}
##' \item{se.cumhaz}{Standard errors.}
##' \item{intF1times}{Years lost at specified times.}
##' \item{causes}{Vector of cause codes.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##'
##' ## Years lost decomposed into causes
##' drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = c(40, 50))
##' par(mfrow = c(1, 2))
##' plot(drm1, cause = 1, se = 1)
##' plot(drm1, cause = 2, se = 1)
##' summary(drm1)
##' estimate(drm1, cause = 1)
##' estimate(drm1, cause = 2)
##'
##' ## Comparing populations
##' drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = 40)
##' summary(drm1, contrast = list(1:4))
##' e1 <- estimate(drm1)
##' estimate(e1, rbind(c(1, -1, 0, 0)))
##' @export
cif_yearslost <- function(formula,data=data,cens.code=0,times=NULL,...)
{# {{{
  cl <- match.call()
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
###  if (!is.Surv(Y)) stop("Expected a 'Surv'-object")
  if (!inherits(Y,c("Event","Surv"))) stop("Expected a 'Surv' or 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
  }

  x <- phreg(formula,data=data,no.opt=TRUE,no.var=1,Z=as.matrix(status,ncol=1),...)
  causes <- sort(unique(x$cox.prep$Z[,1]))
  ccc <- which(causes %in% cens.code)
  if (length(ccc)>=1) causes <- causes[-ccc]

 if (!is.null(times)) {# {{{
	   tt <- expand.grid(times,0:(x$nstrata-1))
###           cause.jumps <- x$U+x$E
           cause.jumps <- x$cox.prep$Z[x$cox.prep$jumps+1,1]
           mm <- cbind(x$jumptimes,x$strata.jumps,1,cause.jumps)
           mm <- rbind(mm,cbind(tt[,1],tt[,2],0,cens.code))
           ord <- order(mm[,1])
           mm <- mm[ord,]
	   phd <- which(mm[,3]==1)
	   newtimes <- which(mm[,3]==0)
	   S0i <- S0i2 <- rep(0,nrow(mm))
	   S0i[phd] <- c(1/x$S0)
	   S0ii2 <- c(1/(x$S0*(x$S0-1)))
	   S0ii2[x$S0==1] <- 0
	   S0i2[phd] <- S0ii2
           strata.jumps <- mm[,2]
	   nstrata <- x$nstrata;
	   jumptimes <- mm[,1]
	   cause.jumps <- mm[,4]
    }   else {
	   strata.jumps <- x$strata.jumps;
	   nstrata <- x$nstrata;
	   jumptimes <- x$jumptimes;
           cause.jumps <- x$cox.prep$Z[x$cox.prep$jumps+1]
	   S0i <- c(1/x$S0)
	   S0i2 <- c(1/(x$S0*(x$S0-1)))
	   S0i2[x$S0==1] <- 0
 }# }}}

 ### formula from Pepe-Mori: SIM 93, 737-
 pepemori <- 0
 ### formula from PKA SIM 2013, not corect, missing some terms
 pka <- 0

  years.lostF1 <- se.years.lostF1m  <- se.years.lostF1 <- se.years.lostF1pm <- c()
  for (i in seq_along(causes)) {
	  jumpsi <- (cause.jumps==causes[i])*1
	  jumpsni <- (cause.jumps %in% causes[-i])*1
	  cumhaz <- cumsumstrata(S0i*jumpsi,strata.jumps,nstrata)
	  var.cumhazMG <- cumsumstrata(S0i2*jumpsi,strata.jumps,nstrata)
	  var.cumhazniMG <- cumsumstrata(S0i2*jumpsni,strata.jumps,nstrata)

	  ## baseline survival
	  km <- exp(cumsumstratasum(log(1-S0i),strata.jumps,nstrata,type="lagsum"))
	  F1 <- cumsumstratasum(jumpsi*km*S0i,strata.jumps,nstrata,type="lagsum")
	  ### 1 - F2  = S + F1
	  F1n1 <- km+F1

	  ## start integral in 0
	  dtime <- c(diffstrata(jumptimes,strata.jumps,nstrata))
	  intkm <-  cumsumstrata( c(km)*dtime,strata.jumps,nstrata)
	  intF1 <- cumsumstrata( c(F1)*dtime,strata.jumps,nstrata)

	  years.lostF1 <- cbind(years.lostF1,intF1)

	  if (pka==1) {
	  ### variance of baseline term  \int_0^t 1/Y^2 \int_s^t S(u) du  dN^i{{{
	  var.intkmcumhaz <- cumsumstrata(jumpsi*intkm*S0i2,strata.jumps,nstrata)
	  var.intkm2cumhaz <- cumsumstrata(jumpsi*intkm^2*S0i2,strata.jumps,nstrata)
	  var.resmean1 <- intkm^2*var.cumhazMG+var.intkm2cumhaz-2*intkm*var.intkmcumhaz

	  ### variance of baseline term  \int_0^t 1/Y^2 \int_s^t F_i(u) du  dN^-i
	  var.intF1cumhaz <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  var.intF12cumhaz <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  var.resmean2 <- intF1^2*var.cumhazniMG+var.intF12cumhaz-2*intF1*var.intF1cumhaz

	  var.intF1 <- (var.resmean1+var.resmean2)

          ## possibly negative due to rounding errors
	  var.intF1[var.intF1<0] <- 0
	  se.intF1 <- var.intF1^.5
	  se.years.lostF1 <- cbind(se.years.lostF1,se.intF1) # }}}
	  }

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s) - (t-s) (1-Fn1(s))) dN_1^i{{{
	  vars.1 <- cumsumstrata(jumpsi*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazMG+vars.1-2*intF1*vars.2

	  vars.3 <- cumsumstrata(jumpsi*F1n1^2*S0i2,strata.jumps,nstrata)
	  vars.4 <- cumsumstrata(jumpsi*F1n1^2*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsi*F1n1^2*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- jumptimes^2*vars.3+vars.4-2*jumptimes*vars.5

          vars.6 <- cumsumstrata(jumpsi*F1n1*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsi*F1n1*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsi*F1n1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsi*F1n1*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7
 	  varsj1 <- vars.11+vars.21-2*vars.31

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s) - (t-s) F1(s)) dN_n1^i
	  vars.1 <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazniMG+vars.1-2*intF1*vars.2

	  vars.3 <- cumsumstrata(jumpsni*F1^2*S0i2,strata.jumps,nstrata)
	  vars.4 <- cumsumstrata(jumpsni*F1^2*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsni*F1^2*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- jumptimes^2*vars.3+vars.4-2*jumptimes*vars.5

          vars.6 <- cumsumstrata(jumpsni*F1*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsni*F1*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsni*F1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsni*F1*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7
	  varsnj1 <- vars.11+vars.21-2*vars.31

	  varss <-  varsj1+varsnj1
          varss[varss<0] <- 0
	  se.intF1m <- varss^.5
	  se.years.lostF1m <- cbind(se.years.lostF1m,se.intF1m)
# }}}

	  if (pepemori==1) {
          ### variance Pepe-Mori {{{
          ### variance of dN_1 term  \int_0^t 1/Y^2   dN_n1^i
	  vars.1 <- cumsumstrata(jumpsi*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazMG+vars.1-2*intF1*vars.2

	  vars.4 <- cumsumstrata(jumpsi*jumptimes^2*S0i2,strata.jumps,nstrata)
	  vars.5 <- cumsumstrata(jumpsi*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.21 <- F1n1^2*(jumptimes^2*var.cumhazMG+vars.4-2*jumptimes*vars.5)

          vars.6 <- cumsumstrata(jumpsi*S0i2,strata.jumps,nstrata)
	  vars.7 <- cumsumstrata(jumpsi*intF1*S0i2,strata.jumps,nstrata)
	  vars.8 <- cumsumstrata(jumpsi*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.9 <- cumsumstrata(jumpsi*intF1*jumptimes*S0i2,strata.jumps,nstrata)
	  vars.31 <- F1n1*(jumptimes*intF1*vars.6+vars.9-intF1*vars.8-jumptimes*vars.7)
 	  varsj1 <- vars.11+vars.21-2*vars.31

          ### variance of baseline term  \int_0^t 1/Y^2 ( IF(t)-IF(s)) dN_n1^i
	  vars.1 <- cumsumstrata(jumpsni*intF1^2*S0i2,strata.jumps,nstrata)
	  vars.2 <- cumsumstrata(jumpsni*intF1*S0i2,strata.jumps,nstrata)
	  vars.11 <- intF1^2*var.cumhazniMG+vars.1-2*intF1*vars.2

	  varspm <- vars.11+varsj1

          varspm[varspm<0] <- 0
	  se.intF1pm <- varspm^.5
	  se.years.lostF1pm <- cbind(se.years.lostF1pm,se.intF1pm)
# }}}
	  }
 }
 years.lostF1 <- cbind(jumptimes,years.lostF1)
 colnames(years.lostF1) <- c("time",paste("intF_",causes,sep=""))

  ### make output at specified times
  if (!is.null(times)) {
    intF1times <- years.lostF1[newtimes,]
    years.lost <- apply(intF1times[,-1,drop=FALSE],1,sum)
    if (pka==1) se.intF1times <- se.years.lostF1[newtimes,-1]
    se.intF1mtimes <- se.years.lostF1m[newtimes,]
    stratatimes <- strata.jumps[newtimes]
    intF1mtimes <- cbind(stratatimes,intF1times,se.intF1mtimes,years.lost)
    colnames(intF1mtimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes),"total-years-lost")
    rownames(intF1mtimes) <- rep(x$strata.level,length(times));
    intF1mtimes=data.frame(intF1mtimes)
    if (pka==1) {
	    intF1pkatimes <- cbind(stratatimes,intF1times,se.intF1times)
            colnames(intF1pkatimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes))
            intF1pkatimes=data.frame(intF1times)
    } else intF1pkatimes <- NULL
    if (pepemori==1) {
          se.intF1pmtimes <- se.years.lostF1pm[newtimes,]
          intF1pmtimes <- cbind(stratatimes,intF1times,se.intF1pmtimes)
          colnames(intF1pmtimes) <- c("strata","times",paste0("intF_",causes),paste0("se.intF_",causes))
         intF1pmtimes=data.frame(intF1pmtimes)
    } else intF1pmtimes <- NULL
  } else coef <- var <- intF1pmtimes <-intF1mtimes <- intF1pkatimes <- NULL

 se.years.lostF1 <- cbind(jumptimes,se.years.lostF1m)
 colnames(se.years.lostF1) <- c("time",paste("intF_",causes,sep=""))

## name things to make use of other programs
 out <- list(cumhaz=years.lostF1,se.cumhaz=se.years.lostF1,
       time=jumptimes, strata=strata.jumps,nstrata=nstrata,
       jumps=1:length(km),strata.name=x$strata.name,
       strata.level=x$strata.level,
       intkmtimes=intF1mtimes, intF1times=intF1mtimes,
       intF1pmtimes=intF1pmtimes, intF1pkatimes=intF1pkatimes,causes=causes
       )
class(out) <- c("resmean_phreg")
return(out)
}# }}}

##' @export
summary.resmean_phreg <- function(object,level=0.95,contrast=NULL,...)
{# {{{

if (is.null(object$intkmtimes)) {
	p <- ncol(object$cumhaz)
	outl <- basecumhaz(object,columns=1:p) 
	outl$strata.name  <- object$strata.name
	outl$strata.level <- object$strata.level
	outl$causes       <- object$causes
} else  {

out <- object$intkmtimes
if (ncol(out)==5) {
   mu <- out[,3]; se <- out[,4];
   xx <- conftype(mu,se,conf.int=level,...)
   lower <- xx$lower
   upper <- xx$upper
   years.lost <- out[,5]
   out <- cbind(out[,-5],lower,upper,years.lost)
   if (!is.null(contrast)) {
	   test <- estimate(object,f=contrast)
	   out <- list(estimates=out,test=test)
   }
   outl <- out
} else {
	p <- ncol(out)
        nn <- names(out)[1:2]
        nl <- names(out)[p]
        outl <- list()
        oute <- list()
	k <- 1
	for (i in object$causes) {
                   name <- paste("intF_",i,sep="")
                   sename <- paste("se.intF_",i,sep="")
		   mu <- out[,name];
		   se <- out[,sename];
                   xx <- conftype(mu,se,conf.int=level,...)
		   loweri <- paste0("lower_",name)
		   upperi <- paste0("upper_",name)
		   out[,loweri] <- xx$lower
		   out[,upperi] <- xx$upper
		   xo <- out[,c(nn,name,sename)]
		   xo[,loweri] <- xx$lower
		   xo[,upperi] <- xx$upper
		   oute[[name]] <- xo
		   if (!is.null(contrast)) {
             test <- estimate(coef=mu,vcov=diag(se^2),f=contrast)
		   testname <- paste0("test",name)
		   outl[[testname]] <- test
		   }
	}
###        outl$estimate <- out
	outl$estimate <- oute
	outl$total.years.lost <- out[,nl]
}
}

return(outl)
}# }}}


##' @export
print.resmean_phreg <- function(x,...)
{# {{{
print(summary.resmean_phreg(x,...))
}# }}}

##' @export
plot.resmean_phreg <- function(x, se=FALSE,time=NULL,add=FALSE,ylim=NULL,xlim=NULL,
                               lty=NULL,col=NULL,lwd=NULL,legend=TRUE,legend.args=list(),
                               ylab=NULL,xlab=NULL,
                               polygon=TRUE,level=0.95,stratas=NULL,robust=FALSE,years.lost=FALSE,cause=1,
                               conf.type=c("log","plain"),...) {# {{{

	if (inherits(x,"phreg") & is.null(ylab)) ylab <- "Cumulative hazard"
	if (inherits(x,"km") & is.null(ylab)) ylab <- "Survival probability"
	if (inherits(x,"cif") & is.null(ylab)) ylab <- "Probability"
	if (inherits(x,"resmean_phreg") & is.null(ylab)) ylab <- "Restricted residual mean life"
	if (inherits(x,"resmean_phreg") & !is.null(x$intF1times)) ylab <- "Years lost up to t: t - E(min(T,t))"
	if (years.lost) ylab <- "Years lost up to t: t - E(min(T,t))"
	if (is.null(xlab)) xlab <- "time"
	ci.level <- level
   level <- -qnorm((1-level)/2)
   if (years.lost) rr <- range(x$cumhaz[,1]-x$cumhaz[,cause+1])  else rr <- range(x$cumhaz[,cause+1])
   strat <- x$strata[x$jumps]
   ylimo <- ylim
   if (is.null(ylim)) ylim <- rr
   if (is.null(xlim)) xlim <- range(x$cumhaz[,1])
   if (se==TRUE) {
      if (is.null(x$se.cumhaz) & is.null(x$robse.cumhaz) )
		   stop("phreg must be with cumhazard=TRUE\n");
   if (years.lost) rrse <- range(c(x$cumhaz[,1]-x$cumhaz[,cause+1]+level*x$se.cumhaz[,cause+1])) else
       rrse <- range(c(x$cumhaz[,cause+1]+level*x$se.cumhaz[,cause+1]))
       if (is.null(ylimo)) ylim <- rrse
   }

   ## all strata
   if (is.null(stratas)) stratas <- 0:(x$nstrata-1)
   ltys <- lty
   cols <- col
   lwds <- lwd

   if (length(stratas)>0 & x$nstrata>1) { ## with strata
   lstrata <- x$strata.level[(stratas+1)]
   ## stratn <-  substring(x$strata.name,8,nchar(x$strata.name)-1)
   ## stratnames <- paste(stratn,lstrata,sep=":")
   stratnames <- lstrata

      if (!is.matrix(lty)) {
         if (is.null(lty)) ltys <- 1:length(stratas) else if (length(lty)!=length(stratas)) ltys <- rep(lty[1],length(stratas))
      } else ltys <- lty
      if (!is.matrix(col)) {
         if (is.null(col)) cols <- 1:length(stratas) else
		 if (length(col)!=length(stratas)) cols <- rep(col[1],length(stratas))
      } else cols <- col
      if (!is.matrix(lwd)) {
         if (is.null(lwd)) lwds <- rep(1,length(stratas)) else
		 if (length(lwd)!=length(stratas)) lwds <- rep(lwd[1],length(stratas))
      } else lwds <- lwd
   } else {
     stratnames <- "Baseline"
     if (is.matrix(col))  cols <- col
     if (is.null(col)) cols <- 1  else cols <- col[1]
     if (is.matrix(lty))  ltys <- lty
     if (is.null(lty)) ltys <- 1  else ltys <- lty[1]
     if (is.matrix(lwd))  lwds <- lwd
     if (is.null(lwd)) lwds <- 1  else lwds <- lwd[1]
   }

  if (!is.matrix(ltys))  ltys <- cbind(ltys,ltys,ltys)
  if (!is.matrix(cols))  cols <- cbind(cols,cols,cols)
  if (!is.matrix(lwds))  lwds <- cbind(lwds,lwds,lwds)

  if (years.lost) x$cumhaz[,cause+1] <- x$cumhaz[,1]-x$cumhaz[,cause+1]
  first <- 0
  for (i in seq(stratas)) {
      j <- stratas[i]
        cumhazard <- x$cumhaz[strat==j,c(1,cause+1),drop=FALSE]
        if (!is.null(cumhazard)) {
	if (nrow(cumhazard)>1) {
        if (add | first==1)
        lines(cumhazard,type="l",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1])
       else {
	  first <- 1
          plot(cumhazard,type="l",lty=ltys[i,1],col=cols[i,1],lwd=lwds[i,1],ylim=ylim,ylab=ylab,xlab=xlab,xlim=xlim,...)
       }
       if (se==TRUE) {
	    if (robust==TRUE) secumhazard  <- x$robse.cumhaz[strat==j,c(1,cause+1),drop=FALSE]
	    else secumhazard <- x$se.cumhaz[strat==j,c(1,cause+1),drop=FALSE]
            xx <- conftype(cumhazard[,2],secumhazard[,2],conf.type=conf.type[1],restrict="positive",conf.int=ci.level)
	    ul <-cbind(cumhazard[,1],xx$upper)
	    nl <-cbind(cumhazard[,1],xx$lower)
	    ul[is.na(ul)] <- 0
	    nl[is.na(nl)] <- 0
		 if (inherits(x,"km")) { ul[,2] <- x$upper[x$strata==j];
		                          nl[,2] <- x$lower[x$strata==j];
		                          wna <- which(is.na(ul[,2]))
		                          ul[wna,2] <- 0
		                          nl[wna,2] <- 0

		 }
	      if (!polygon) {
		  lines(nl,type="l",lty=ltys[i,2],col=cols[i,2],lwd=lwds[i,2])
		  lines(ul,type="l",lty=ltys[i,3],col=cols[i,3],lwd=lwds[i,3])
	      } else {
                 ## type="l" confidence regions
		 ll <- length(nl[,1])
		 timess <- nl[,1]
		 tt <- c(timess,rev(timess))
		 yy <- c(nl[,2],rev(ul[,2]))
		 col.alpha<-0.1
		 col.ci<-cols[i,]
		 col.trans <- sapply(col.ci, FUN=function(x)
		   do.call(grDevices::rgb,as.list(c(grDevices::col2rgb(x)/255,col.alpha))))
		 polygon(tt,yy,lty=0,col=col.trans)
	      }
        }
     }
     }
    }
     if (legend & (!add))  {
       if (is.null(legend.args$x)) {
         legend.args$x <- ifelse (inherits(x,"km"), "topright", "topleft")
       }
       if (is.null(legend.args$col)) legend.args$col <- cols[,1]
       if (is.null(legend.args$lty)) legend.args$lty <- ltys[,1]
       if (is.null(legend.args$legend)) legend.args$legend <- stratnames
       do.call(graphics::legend, legend.args)
     }

}# }}}


# }}}

##' IPTW Cox Regression (Inverse Probability of Treatment Weighted)
##'
##' Fits a Cox model with treatment weights \deqn{w(A) = \sum_a I(A=a)/\pi(a|X)}, where
##' \deqn{\pi(a|X) = P(A=a|X)}.
##'
##' Standard errors are computed via influence functions that are returned as the IID argument.
##' Propensity scores are fitted using either logistic regression (\code{glm}) or the multinomial
##' model (\code{mlogit}) when there are more than two treatment categories.
##'
##' The treatment variable must be a factor and is identified on the RHS of the \code{treat.model}.
##' Recurrent events can be considered with a start-stop structure, requiring \code{cluster(id)}.
##' Robust standard errors are computed in all cases.
##'
##' Time-dependent propensity score weights can be computed when \code{treat.var} is used. This weight
##' be 1 at the time of first (A_0) and 2nd treatment (A_1), then uses weights \deqn{w_0(A_0) * w_1(A_1)^{t>T_r}} where \deqn{T_r} is
##' time of 2nd randomization. The weights are constructed using a \code{glm} or \code{mlogit} model based on the data where 
##' \code{treat.var=1}. The propensity score can be constructed for any number of treatments in a similar manner. 
##'
##' @param formula Formula for \code{phreg}.
##' @param data Data frame for risk averaging.
##' @param treat.model Propensity score model (binary or multinomial).
##' @param treat.var A 1/0 variable indicating when treatment is given (for time-dependent weights).
##' @param weights Optional weights to multiply with the IPTW weights.
##' @param estpr (=1, default) to estimate propensity scores and include their uncertainty in the influence function.
##' @param pi0 Fixed simple weights (if \code{estpr=0}).
##' @param se.cluster To compute GEE-type standard errors when additional cluster structure is present.
##' @param ... Arguments for \code{phreg} call.
##'
##' @return An object of class \code{"phreg"} with additional IPTW components:
##' \item{IID}{Influence functions including propensity score uncertainty.}
##' \item{iptw}{IPTW weights used.}
##' \item{naive.var}{Naive variance ignoring propensity score uncertainty.}
##'
##' @author Thomas Scheike
##' @examples
##' data <- mets:::simLT(0.7, 100, beta = 0.3, betac = 0, ce = 1, betao = 0.3)
##' dfactor(data) <- Z.f ~ Z
##' out <- phreg_IPTW(Surv(time, status) ~ Z.f, data = data, treat.model = Z.f ~ X)
##' summary(out)
##' @export
phreg_IPTW <- function (formula, data,treat.model = NULL, treat.var = NULL,weights = NULL, estpr = 1, pi0 = 0.5,se.cluster=NULL,...)
{# {{{
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata)) strata.name <- names(des$xlevels)
    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL

 call.id <- id;
 conid <- construct_id(id,length(exit))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid

 ## take offset and weight first from formula, but then from arguments
 offset <- NULL
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit))
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit))
  } else weights <- des.weights


    if (!is.null(se.cluster)) {
        sds <- sort(unique(se.cluster))
        sid <- length(sds)
        if (is.numeric(se.cluster))
            se.cluster <- fast.approx(sds, se.cluster) - 1
        else se.cluster <- as.integer(factor(se.cluster, labels = seq(sid))) - 1
    }

    id <- id + 1
    nid <- length(unique(id))
    data$id__ <- id
    data$cid__ <- cumsumstrata(rep(1, length(id)), id - 1, nid)
    treats <- function(treatvar) {
        treatvar <- droplevels(treatvar)
        nlev <- nlevels(treatvar)
        nlevs <- levels(treatvar)
        ntreatvar <- as.numeric(treatvar)
        return(list(nlev = nlev, nlevs = nlevs, ntreatvar = ntreatvar))
    }
    fittreat <- function(treat.model, data, id, ntreatvar, nlev) {
        if (nlev == 2) {
            treat.model <- drop.specials(treat.model, "cluster")
            treat <- glm(treat.model, data, family = "binomial")
            iidalpha <- lava::iid(treat, id = id)
            lpa <- treat$linear.predictors
            pal <- expit(lpa)
            pal <- cbind(1 - pal, pal)
            ppp <- (pal/pal[, 1])
            spp <- 1/pal[, 1]
        }
        else {
            treat.modelid <- update.formula(treat.model, . ~ . + cluster(id__))
            treat <- mlogit(treat.modelid, data)
            iidalpha <- lava::iid(treat)
            pal <- predict(treat, data, se = 0, response = FALSE)
            ppp <- (pal/pal[, 1])
            spp <- 1/pal[, 1]
        }
        Xtreat <- model.matrix(treat.model, data)
        tvg2 <- 1 * (ntreatvar >= 2)
        pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
        pppy <- c(mdi(ppp, 1:length(ntreatvar), ntreatvar))
        Dppy <- (spp * tvg2 - pppy)
        Dp <- c()
        for (i in seq(nlev - 1)) Dp <- cbind(Dp, Xtreat * ppp[, i + 1] * Dppy/spp^2)
        DPai <- -1 * Dp/pA^2
        out <- list(iidalpha = iidalpha, pA = pA, Dp = Dp, pal = pal,
            ppp = ppp, spp = spp, id = id, DPai = DPai)
        return(out)
    }
    expit <- function(x) 1/(1 + exp(-x))
    if (!is.null(treat.var)) {
        weightWT <- data[, treat.var]
        whereW <- which(weightWT == 1)
        CountW <- cumsumstrata(weightWT, id - 1, nid)
        dataW <- data[whereW, ]
        idW <- id[whereW]
    } else {
        whereW <- 1:nrow(data)
        dataW <- data
        idW <- id
        CountW <- cumsumstrata(rep(1,nrow(data)), id-1,nid)
    }
    treat.name <- all.vars(treat.model)[1]
    treatvar <- dataW[, treat.name]
    if (!is.factor(treatvar))
        stop(paste("treatment=", treat.name, " must be coded as factor \n",
            sep = ""))
    treats <- treats(treatvar)
    wlPA <- ww <- rep(1, nrow(data))
    idWW <- mystrata2index(cbind(id, CountW))
    if (estpr[1] == 1) {
        fitt <- fittreat(treat.model, dataW, idW, treats$ntreatvar, treats$nlev)
        iidalpha0 <- fitt$iidalpha
        wPA <- c(fitt$pA)
        DPai <- fitt$DPai
    } else {
        if (length(pi0)==length(treats$ntreatvar)) wPA <- pi0
	else wPA <- ifelse(treats$ntreatvar==2,pi0[1],1-pi0[1])
        ## pi0 <- rep(pi0, treats$nlev)
        DPai <- matrix(0, nrow(data), 1)
    }
    ww <- rep(1, nrow(data))
    ww[whereW] <- wPA
    wlPA <- exp(cumsumstrata(log(ww), idWW - 1, attr(idWW, "nlevel")))
    wwt <- c(exp(cumsumstrata(log(ww), id - 1, nid)))
    ## P(t) = P_0 * P_1^(I(t>T1)), time-dependent weights
    ## DP = P(t) \sum_j P_j I(t> TJ) (-DP_j/P_j^2)
    if (estpr[1] == 1) {
        DPait <- matrix(0, nrow(data), ncol(DPai))
        DPait[whereW, ] <- DPai
        DPait <- apply(DPait * c(wlPA), 2, cumsumstrata, id - 1, nid)/wwt
    } else DPait <- NULL
    if (is.null(weights)) ww <- 1/wwt else ww <- weights/wwt
    phw <- phreg(formula, data, weights = ww, Z = DPait, ...)

###    check.derivative <- 0
###	if (check.derivative == 1) { ## {{{
###	### for checking derivative
###	fpar <- glm(treat.model,dataw,family=binomial)
###	mm <- model.matrix(treat.model,dataw)
###	cpar <- coef(fpar)
###
###	ff <- function(par,base=0) {
###	pa <-        expit(mm %*% par)
###	www <- ifelse(dataw[,treat.name] == "1", pa, 1 - pa)
###	ww <- rep(1, nrow(data))
###	ww[wherew] <- www
###	wlpa <- exp(cumsumstrata(log(ww), idww - 1, attr(idww, "nlevel")))
###	wwwt <- exp(cumsumstrata(log(ww), id - 1, nid))
###
###	pp <- phreg(formula,data,weights=1/wwwt,no.opt=true,beta=coef(phw))
###	if (base==1) po <- c(pp$cumhaz[,2]) else po <- pp$gradient
###	return(po)
###	}
###
###	print(ff(cpar))
###	gf <- jacobian(ff,cpar)
###	print(t(gf))
###        ###
###        print(ff(cpar,base=1))
###	gf <- jacobian(ff,cpar,base=1)
###	print(gf)
###	print("___________________________________");
###	} ## }}}

if (estpr[1] == 1) {
	xx <- phw$cox.prep
	nid <- max(xx$id)
	S0i <- rep(0, length(xx$strata))
	wPAJ <- xx$weights[xx$jumps+1]*xx$caseweights[xx$jumps+1]
	Xt <- xx$X
	S0 <- phw$S0 * wPAJ
	S0i[xx$jumps + 1] <- 1/S0
	U <- E <- matrix(0, nrow(xx$X), phw$p)
	U[xx$jumps + 1, ] <- phw$U/wPAJ
	E[xx$jumps + 1, ] <- phw$E


        if (phw$p>0) {
	rr <- c(xx$sign * exp(Xt %*% coef(phw) + xx$offset) * xx$weights)
	rrnw <- c(xx$sign * exp(Xt %*% coef(phw) + xx$offset))
        } else {
	rr <- c(xx$sign * exp(xx$offset) * xx$weights)
	rrnw <- c(xx$sign * exp(xx$offset))
        }
	DWX = .Call("vecMatMat", xx$Z, Xt)$vXZ
	S1 = apply(Xt * rr, 2, revcumsumstrata, xx$strata, xx$nstrata)
	###S00 = revcumsumstrata( rr, xx$strata, xx$nstrata)
	DS1 = apply(DWX * rrnw, 2, revcumsumstrata, xx$strata, xx$nstrata)
	DS0 = apply(xx$Z * rrnw, 2, revcumsumstrata, xx$strata, xx$nstrata)
	if (phw$p>0) {
		DS0S1 = .Call("vecMatMat", DS0[xx$jumps + 1, , drop = FALSE],S1[xx$jumps + 1, , drop = FALSE])$vXZ
		DUa2 <- apply(wPAJ * DS1[xx$jumps + 1, , drop = FALSE]/c(S0),2, sum) - apply(wPAJ * DS0S1/c(S0^2), 2, sum)
		DUa2 <- matrix(DUa2, ncol(fitt$DPai), phw$p)
		DUa1 <- t(xx$Z[xx$jumps + 1, ]) %*% (phw$U/wPAJ)
		DUa <- DUa1 - DUa2
		iidpal <- iidalpha0 %*% DUa
		iidbeta <- lava::iid(phw) + iidpal %*% phw$ihess
		phw$DUa <- DUa
		phw$IID <- iidbeta
		phw$naive.var <- phw$var
		phw$var <- crossprod(iidbeta)
	} else iidbeta <- NULL

        ###
	DAw <- apply(xx$caseweights[xx$jumps+1]*xx$Z[xx$jumps + 1, ,drop=FALSE]/c(S0),2,cumsumstrata,phw$strata.jumps,xx$nstrata)
	DA2 <- apply(wPAJ*DS0[xx$jumps + 1, , drop = FALSE]/c(S0)^2,2,cumsumstrata,phw$strata.jumps,xx$nstrata)
	DAt <- 1*(DAw-DA2)
	phw <- robust_phreg(phw,beta.iid=iidbeta)
	varA <- phw$robse.cumhaz[,2]^2
	phw$naive.se.cumhaz <- phw$robse.cumhaz
	vtheta <- crossprod(iidalpha0)
        varthetat <-   rowSums((DAt %*% vtheta)*DAt)
        ###
        ww <- xx$caseweights*xx$weights
	S0i2 <- rep(0,length(xx$strata))
        S0i2[xx$jumps+1] <- 1/(phw$S0^2*ww[xx$jumps+1])
        cumS0i2 <- c(cumsumstrata(S0i2,xx$strata,xx$nstrata))
        xxx <- (S0i-rr*cumS0i2)
	id <- xx$id
	mid <- max(xx$id)+1
        thetat <- iidalpha0[id+1,,drop=FALSE]
        covk1 <- apply(S0i*thetat,2,cumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="sum")
        covk2 <- apply(rr*thetat,2,revcumsumidstratasum,id,mid,xx$strata,xx$nstrata,type="lagsum")
        ###
        covk1 <- apply(S0i*thetat,2,cumsumstrata,xx$strata,xx$nstrata)
        covk2 <- apply(rr*thetat,2,revcumsumstrata,xx$strata,xx$nstrata)
        ###
        covk2 <- covk2*cumS0i2
        covv <- (covk1-covk2)[xx$jumps+1,,drop=FALSE]
        covvt <- 2*apply(covv*DAt,1,sum)
        varA <- varA+varthetat+2*apply(covv*DAt,1,sum)

	Ht <- NULL
	if (phw$p>0) {
		covbetatheta <- t(iidalpha0) %*%  iidbeta
		Ht <- apply(E*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
		Ht <- Ht[xx$jumps+1,,drop=FALSE]
		covvBT <- 2* rowSums((Ht %*% t(covbetatheta))*DAt)
		varA <- varA + covvBT
	}
	phw$robse.cumhaz[,2] <- varA^.5
	phw$se.cumhaz[,2] <- varA^.5
	phw$var.cumhaz <- cbind(phw$se.cumhaz[,1],varA)
	resAiid <- list(DAt=DAt,Ht=Ht,iidalpha0=iidalpha0,iidbeta=phw$IID)
	phw$resAiid <- resAiid
	phw$iptw <- ww
} else {
   phw$iptw <- ww
   phw <- robust_phreg(phw)
   phw$naive.se.cumhaz <- phw$se.cumhaz
   phw$se.cumhaz <- phw$robse.cumhaz
   phw$naive.var <- phw$var
}


if (is.matrix(phw$IID))
	if (nrow(phw$IID)==length(name.id)) rownames(phw$IID) <- name.id

if (!is.null(se.cluster)) {
	phw$IID.simple <- phw$IID
	phw$var.simple <- phw$var
	phw$IID <- apply(phw$IID,2,sumstrata,se.cluster,sid)
	phw$var <- crossprod(phw$IID)
	phw$se.cluster <- se.cluster
}
class(phw) <- c("phreg","IPTW")

return(phw)
}# }}}


##' G-Estimator for Cox and Fine-Gray Models
##'
##' Computes the G-estimator (G-formula) for standardized survival or cumulative incidence estimates:
##' \deqn{ \hat S(t, A=a) = n^{-1} \sum_i \hat S(t, A=a, Z_i) }
##'
##' Based on a \code{phreg} or \code{cifreg} object. Provides influence functions for these risk estimates,
##' allowing for standard error computation.
##'
##' If the first covariate is a factor, contrasts between all levels are computed automatically.
##' If it is continuous, specific values must be provided via \code{Avalues}.
##'
##' @param x Object of class \code{"phreg"} or \code{"cifreg"}.
##' @param data Data frame for risk averaging. Must be part of the data used for fitting unless \code{same.data=FALSE}.
##' @param time Time point for estimation.
##' @param Avalues Values to compare for the first covariate \eqn{A}.
##' @param varname Name of the variable to be treated as the treatment/exposure variable (default is the first variable).
##' @param same.data Logical; assumes the same data is used for fitting and averaging.
##' @param First Logical; if \code{TRUE}, uses only the first record for G-averaging (useful for start-stop structures).
##'
##' @return An object of class \code{"survivalG"} containing:
##' \item{risk}{Standardized risk estimates.}
##' \item{risk.iid}{Influence functions for the risk estimates.}
##' \item{difference}{Pairwise differences in risks.}
##' \item{ratio}{Risk ratios.}
##' \item{survival.ratio}{Survival ratios (for \code{phreg}).}
##' \item{survival.difference}{Survival differences (for \code{phreg}).}
##'
##' @author Thomas Scheike
##' @aliases survivalGtime
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' bmt$event <- (bmt$cause != 0) * 1
##' bmt$id <- 1:408
##' dfactor(bmt) <- tcell.f ~ tcell
##'
##' # Fine-Gray model
##' fg1 <- cifreg(Event(time, cause) ~ tcell.f + platelet + age, bmt,
##'               cause = 1, cox.prep = TRUE, propodds = NULL)
##' summary(survivalG(fg1, bmt, 50))
##'
##' \donttest{ ## Reduce Ex.Timings
##' # Cox model
##' ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age, bmt)
##' summary(survivalG(ss, bmt, 50))
##'
##' # Stratified Cox model
##' ss <- phreg(Surv(time, event) ~ strata(tcell.f) + platelet + age, bmt)
##' summary(survivalG(ss, bmt, 50))
##'
##' # Time-varying G-estimates
##' sst <- survivalGtime(ss, bmt, n = 50)
##' plot(sst)
##'
##' # Among treated (specify id to link influence functions)
##' ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age + cluster(id), bmt)
##' summary(survivalG(ss, subset(bmt, tcell == 1), 50))
##' }
##' @aliases survivalGtime
##' @export
survivalG <- function(x,data,time=NULL,Avalues=NULL,varname=NULL,same.data=TRUE,First=FALSE)
{# {{{
if (is.null(time)) stop("Give time for estimation of survival/cumulative incidence\n")
if (inherits(x,c("cifreg","phreg","recreg")))
Aiid <- iidBaseline(x,time=time) else stop("Must be cifregFG/phreg/recreg object \n");

### dealing with first variable
if (is.null(varname))  {
   treat.name <- all.vars(update.formula(x$formula,1~.))[1]
} else treat.name <- varname
treatvar <- data[,treat.name]

Alevels <- NULL
if (is.factor(treatvar)) {
   ## treatvar, levels to look at
   if (is.null(Avalues)) Alevels <- levels(treatvar) else Alevels <- Avalues
} else {
   Alevels <- Avalues
}
if (is.null(Alevels)) stop("must either have a factor as first variable or give Avalues for risk average")

## for recreg take first record only of each subject
cid <- countID(data.frame(id=x$id))
if (inherits(x,"recreg") | First) {
    FirstId <- which(cid$Countid==1)
    data <- data[FirstId,]
}

desx <- update_design(x$design,data=data)
if (is.null(desx$cluster)) { ## data must be same as data used for x
   id <- x$id
} else {
  id <- desx$cluster-1
}
datA <- data


k <- 1; risks <- c(); DariskG <- list()
for (a in Alevels) { ## {{{
   datA[,treat.name] <- a
   pp <- predict(x,datA,time,se=0)
   Xbase <- 1*outer(pp$strata,0:(x$nstrata-1),"==")

 if (inherits(x,"cifreg")) {
    ps0 <- c(pp$cif);
    Dma  <- cbind(c(pp$RR)*c(1-ps0)*Xbase,c((1-ps0)*pp$cumhaz)*pp$X);
 } else if (inherits(x,"recreg")) {
    ps0 <- pp$cumhaz;
    Dma <-  cbind(c(pp$RR)*Xbase,c(pp$cumhaz)*pp$X)
 } else if (inherits(x,"phreg"))  {
    ps0 <- c(pp$surv);
    Dma  <- -cbind(c(pp$RR)*ps0*Xbase,c(ps0*pp$cumhaz)*pp$X);
 }
 risks <- cbind(risks,ps0)
 DariskG[[k]] <- apply(Dma,2,sum)
 k <- k+1
} ## }}}

predictAiid <- NULL
###
Grisk <- apply(risks,2,mean)
risk.iid  <- t(t(risks)-Grisk)
### order after all id in data
nid <- max(x$id)+1
ndata <- length(unique(id))

## after id.data, if sub-data then still ok sorted after id.data=0:nid
risk.iid <- apply(risk.iid,2,sumstrata,id,nid)/ndata

## sorted after x$id, so is risk.iid
coxiid <- cbind(Aiid$base.iid,Aiid$beta.iid)

if (same.data) {
   for (a in seq_along(Alevels)) risk.iid[,a] <- risk.iid[,a]+ coxiid %*% DariskG[[a]]/ndata
   vv <- crossprod(risk.iid)
} else {
   predictAiid <- matrix(0,ndata,ncol(risks))
   for (a in seq_along(Alevels))  {
###	risk.iid[,a] <- risk.iid[,a]
	predictAiid[,a] <- coxiid %*% DariskG[[a]]/ndata
   }
   risk.iid <- rbind(risk.iid,predictAiid)
   vv <- crossprod(risk.iid)
}

if (inherits(x,"cifreg") | inherits(x,"recreg")) {
out <- estimate(coef=Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
ed <- estimate(coef=Grisk,vcov=vv,f=function(p) (p[-1])-(p[1]))
rd <- estimate(coef=Grisk,vcov=vv,f=function(p) log(p[-1]/p[1]),null=0)
out <- list(risk.iid=risk.iid,risk=out,difference=ed,ratio=rd,vcov=vv)
} else if (inherits(x,"phreg"))  {
out <- estimate(coef=1-Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
sout <- estimate(coef=Grisk,vcov=vv,labels=paste("risk",Alevels,sep=""))
ed <- estimate(coef=Grisk,vcov=vv,f=function(p) (1-p[-1])-(1-p[1]))
rd <- estimate(coef=Grisk,vcov=vv,f=function(p) log((1-p[-1])/(1-p[1])),null=0)
srd <- estimate(coef=Grisk,vcov=vv,f=function(p) log(p[-1]/p[1]),null=0)
ssd <- estimate(coef=Grisk,vcov=vv,f=function(p) p[-1]-p[1])
out <- list(risk.iid=risk.iid,survivalG=sout,risk=out,difference=ed,ratio=rd,
	    survival.ratio=srd, survival.difference=ssd,vcov=vv)
}

class(out) <- "survivalG"
attr(out,"levels") <- Alevels
return(out)
} ## }}}

##' @export
estimate.survivalG  <- function(x,...)
{
out <-estimate(coef=x$risk$coefmat[,1],IC=x$risk.iid*nrow(x$risk.iid),...)
return(out)
}

###{{{ summary  survivalG

##' @export
print.survivalG <- function(x,...) {
  print(summary(x,...))
}

##' @export
summary.survivalG <- function(object,...) {
  res <- list(risk=object$risk,difference=object$difference,ratio=object$ratio,
	      survival.ratio=object$survival.ratio,survival.difference=object$survival.difference)
  class(res) <- "summary.survivalG"
  res
}

##' @export
print.summary.survivalG  <- function(x,...) {
    cat("G-estimator :\n")
    print(x[["risk"]],...)
    cat("\n")

    cat("Average Treatment effect: difference (G-estimator) :\n")
    print(x$difference,...)
    cat("\n")

    cat("Average Treatment effect: ratio (G-estimator) :\n")
    cat("log-ratio: \n")
    print(x$ratio$coefmat,...)
    cat("ratio: \n")
    ratio <- exp(x$ratio$coefmat[,c(1,3:4)])
    print(ratio,...)
    cat("\n")

    if (!is.null(x$survival.difference)) {
    cat("Average Treatment effect:  survival-difference (G-estimator) :\n")
    print(x$survival.difference$coefmat,...)
    cat("\n")
    }

    if (!is.null(x$survival.ratio)) {
    cat("Average Treatment effect: 1-G (survival)-ratio (G-estimator) :\n")
    cat("log-ratio: \n")
    print(x$survival.ratio$coefmat,...)
    cat("ratio: \n")
    ratio <- exp(x$survival.ratio$coefmat[,c(1,3:4)])
    print(ratio,...)
    cat("\n")
    }

}

###}}} summary

##' @export
survivalGtime <- function(x,data,time=NULL,n=100,...)
{# {{{

if (is.null(time)) {
       rr <- range(x$cumhaz[,1])
       if (!is.null(n)) time <- seq(rr[1],rr[2],length=n) else time <- x$cumhaz[,1]
}

survivalGt <- risk <- difference <- ratio <- survival.ratio <- c()

for (tt in time) {
  Gt <- survivalG(x,data,time=tt,...)
  strata <- strata(rownames(Gt$risk$coefmat))
  survivalGt <- rbind(survivalGt,cbind(tt,Gt$survivalG$coefmat))
  risk <- rbind(risk,cbind(tt,Gt$risk$coefmat))
  difference <- rbind(difference,cbind(tt,Gt$difference$coefmat))
  ratio <- rbind(ratio,cbind(tt,exp(Gt$ratio$coefmat)))
  if (!is.null(Gt$survival.ratio$coefmat))
  survival.ratio <- rbind(survival.ratio,cbind(tt,exp(Gt$survival.ratio$coefmat)))
}
  colnames(survivalGt)[1] <- "time"
  colnames(risk)[1] <- "time"
  colnames(difference)[1] <- "time"
  colnames(ratio)[1] <- "time"
  if (!is.null(survival.ratio))
  colnames(survival.ratio)[1] <- "time"
  strata <- strata(rownames(risk))
out <- list(time=time,survivalG=survivalGt,risk=risk,difference=difference,
	    ratio=ratio,survival.ratio=survival.ratio,strata=strata)

class(out) <- "survivalGtime"
return(out)
} ## }}}

##' @export
plot.survivalGtime <- function(x,type=c("survival","risk","survival.ratio","difference","ratio"),ylim=NULL,legend=NULL,...) {# {{{

  ## to deal with fine-gray based things, that do not contain survival estimates
  if ((ncol(x$survivalG)==1) & type[1]=="survival") type <- "risk"
  us <- unique(x$strata)
  cols <- 1:length(us)
  ltys <- cols
  ss0 <- x$strata==us[1]

  if (type[1]=="survival") {
  if (is.null(ylim))  ylim  <- range(x$survivalG[,c(4,5)])
  plot(x$time,x$survivalG[ss0,2],type="s",xlab="time",ylab="Survival",col=cols[1],lty=ltys[1],ylim=ylim,...)
  plotConfRegion(x$time,x$survivalG[ss0,c(4,5)],col=cols[1])

  k <- 2
for (ss in us[-1]) {
   ss0 <- x$strata==ss
   lines(x$time,x$survivalG[ss0,2],type="s",ylim=c(0,1),col=cols[k],lty=ltys[k])
   plotConfRegion(x$time,x$survivalG[ss0,c(4,5)],col=cols[k])
   k <- k+1
}
  }

  if (type[1]=="risk") {
   if (is.null(ylim))  ylim  <- range(x$risk[,c(4,5)])
   plot(x$time,x$risk[ss0,2],type="s",xlab="time",ylab="risk",col=cols[1],lty=ltys[1],ylim=ylim,...)
   plotConfRegion(x$time,x$risk[ss0,c(4,5)],col=cols[1])
   k <- 2
   for (ss in us[-1]) {
   ss0 <- x$strata==ss
   lines(x$time,x$risk[ss0,2],type="s",ylim=c(0,1),col=cols[k],lty=ltys[k])
   plotConfRegion(x$time,x$risk[ss0,c(4,5)],col=cols[k])
   k <- k+1
   }
  }

  if (type[1]=="difference")  {
	  if (is.null(ylim))  ylim  <- range(x$risk[,c(4,5)])
  plot(x$time,x$difference[,2],type="s",xlab="time",ylab="difference in risk",ylim=range(x$difference[,2]),col=cols[1],lty=ltys[1],...)
  plotConfRegion(x$time,x$difference[,c(4,5)],col=cols[1])
  }
  if (type[1]=="ratio")  {
  if (is.null(ylim))  ylim  <- range(x$ratio[,c(4,5)])
  plot(x$time,x$ratio[,2],type="s",ylim=ylim,xlab="time",ylab="ratio",col=cols[1],lty=ltys[1],...)
  plotConfRegion(x$time,x$ratio[,c(4,5)],col=cols[1])
}
 if (type[1]=="survival.ratio")  {
   if (is.null(ylim))  ylim  <- range(x$survival.ratio[,c(4,5)])
  plot(x$time,x$survival.ratio[,2],type="s",xlab="time",ylab="risk",col=cols[1],lty=ltys[1],ylim=ylim,...)
  plotConfRegion(x$time,x$survival.ratio[,c(4,5)],col=cols[1])
}

 if (is.null(legend)) legend("topleft",levels(x$strata),col=cols,lty=ltys)

}
# }}}

##' Fast Additive Hazards Model with Robust Standard Errors
##'
##' Fits a fast Lin-Ying additive hazards model with a possibly stratified baseline.
##' Robust variance is the default variance estimate in the summary.
##'
##' Influence functions (IID) follow the numerical order of the given cluster variable.
##' Ordering by \code{$id} aligns the IID terms with the dataset order.
##'
##' @param formula Formula with a 'Surv' outcome (similar to \code{coxph}).
##' @param data Data frame.
##' @param no.baseline Logical; if \code{TRUE}, fits the model without baseline hazard estimation.
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"aalenMets"} (extends \code{"phreg"}) containing:
##' \item{coef}{Estimated coefficients.}
##' \item{var}{Robust variance-covariance matrix.}
##' \item{iid}{Influence functions.}
##' \item{intZHZ}{Integrated ZHZ matrix.}
##' \item{gamma}{Coefficient estimates.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$time <- bmt$time + runif(408) * 0.001
##' out <- aalenMets(Surv(time, cause == 1) ~ tcell + platelet + age, data = bmt)
##' summary(out)
##'
##' ## Comparison with timereg::aalen
##' ## out2 <- timereg::aalen(
##' ##   Surv(time, cause == 1) ~ const(tcell) + const(platelet) + const(age),
##' ##   data = bmt
##' ## )
##' ## summary(out2)
##' @export
aalenMets <- function(formula,data=data,no.baseline=FALSE,...)
{# {{{
formula.call <- formula
x <- phreg(formula,data=data,no.opt=TRUE,...)
xx <- x$cox.prep

### computation of intZHZ matrix, and gamma and baseline ##################
# {{{
eXb <- c(xx$sign) * c(xx$weights)
S0 = c(revcumsumstrata(eXb,xx$strata,xx$nstrata))
S0[S0==0] <- 1
E=apply(eXb*xx$X,2,revcumsumstrata,xx$strata,xx$nstrata)/S0;
if (no.baseline) E <- 0*E
###if (!no.int) E <- cbind(1,E)
###if (no.int) X <-  xx$X else X <- cbind(1,xx$X)
###if (no.int) XX <-  xx$XX else { XX <- .Call("vecCPMat",X)$XX }
X <- xx$X; XX <- xx$XX
###
S2=apply(eXb*XX,2,revcumsumstrata,xx$strata,xx$nstrata)
E2  <- .Call("vecCPMat",E)$XX
###
dts <- c(diffstrata(xx$time,xx$strata,xx$nstrata))
dt <- diff(c(0,xx$time))
###
intZbar <- apply(E*dts,2,cumsumstrata,xx$strata,xx$nstrata)
intZHZt <- apply((S2-S0*E2)*dts,2,cumsum)
p <- ncol(E)
intZHZ <-  matrix(.Call("XXMatFULL",tail(intZHZt,1),p,PACKAGE="mets")$XXf,p,p)

IintZHZ  <-  pinv(intZHZ)
intZHdN <- matrix(x$gradient,ncol(E),1)
XJ <- X[xx$jumps+1,]
if (no.baseline) intZHdN <- matrix(apply(XJ,2,sum),ncol(E),1)
gamma <- IintZHZ %*% intZHdN
###if (!no.int) nn <- c("int",colnames(x$X)) else
nn <- colnames(x$X)
rownames(gamma)  <-  nn
x$cumhaz[,2] <- x$cumhaz[,2]- intZbar[xx$jumps+1,] %*% gamma
# }}}

### iid gamma #########################################
# {{{
id <- xx$id
if (no.baseline==FALSE) {
  mm <-  X * c(X %*% gamma) * c(xx$time)+ apply( E* c(E %*% gamma)*dts,2,cumsumstrata,xx$strata,xx$nstrata) -  X* c(intZbar %*% gamma) - c(X %*% gamma)* intZbar
  mm <- c(xx$weights*xx$sign) * mm
  mm <- apply(mm,2,sumstrata,id,max(id)+1)
  ###
  MGt <- t(x$hessian %*% t(lava::iid(x))) + mm
  XJ2l  <- .Call("vecCPMat",x$U)$XX
  XJ2l <- matrix(apply(XJ2l,2,sum),nrow=1)
  varmg <-  matrix(.Call("XXMatFULL",XJ2l,p,PACKAGE="mets")$XXf,p,p)
  varmg <- IintZHZ %*% varmg %*% IintZHZ
} else {
  mm <-  X * c(X %*% gamma) * c(xx$time)
  mm <- c(xx$weights*xx$sign) * mm
  mm <- apply(mm,2,sumstrata,id,max(id)+1)
  XJ2l  <- .Call("vecCPMat",XJ)$XX
  XJ2l <- matrix(apply(XJ2l,2,sum),nrow=1)
  varmg <-  matrix(.Call("XXMatFULL",XJ2l,p,PACKAGE="mets")$XXf,p,p)
  varmg <- IintZHZ %*% varmg %*% IintZHZ
  XdN  <- apply(XJ,2,sumstrata,id[xx$jumps+1],max(id)+1)
  MGt <-  XdN-mm
}
iid <-  MGt %*% IintZHZ
# }}}

### output  #########################################
# {{{
coef <- c(gamma)
names(coef) <- nn
x$coef <- coef
x$var <- crossprod(iid)
x$varmg <- varmg
x$iid <- iid
x$intZHdN <- intZHdN
x$intZHZ  <-  intZHZ
x$formula <- formula.call
x$ihessian <- IintZHZ
## score of gamma
x$gradient <- c(intZHdN-intZHZ %*% gamma)

x$no.opt <- FALSE
class(x) <- c(class(x),"aalenMets")
# }}}

return(x)
}# }}}

##' Kaplan-Meier with Robust Standard Errors
##'
##' Computes Kaplan-Meier estimates with robust standard errors.
##' Robust variance is the default and is obtained from the \code{predict} call.
##'
##' @param formula Formula with 'Surv' or 'Event' outcome.
##' @param data Data frame.
##' @param km Logical; if \code{TRUE}, returns Kaplan-Meier estimates; otherwise returns Nelson-Aalen based estimates.
##' @param ... Additional arguments passed to \code{phreg}.
##'
##' @return An object of class \code{"km"} (extends \code{"predictphreg"}) containing:
##' \item{surr}{Survival probabilities.}
##' \item{se.surv}{Standard errors.}
##' \item{lower, upper}{Confidence intervals.}
##'
##' @author Thomas Scheike
##' @examples
##' data(sTRACE)
##' sTRACE$cluster <- sample(1:100, 500, replace = TRUE)
##' out1 <- km(Surv(time, status == 9) ~ strata(vf, chf), data = sTRACE)
##' out2 <- km(Surv(time, status == 9) ~ strata(vf, chf) + cluster(cluster), data = sTRACE)
##'
##' summary(out1, times = 1:3)
##' summary(out2, times = 1:3)
##'
##' par(mfrow = c(1, 2))
##' plot(out1, se = TRUE)
##' plot(out2, se = TRUE)
##' @export
km <- function(formula,data=data,km=TRUE,...)
{# {{{
res <- phreg(formula,data=data,...)

rhs <- update(formula,-1~.)
varss <- all.vars(formula)
## find all strata
first <- c(headstrata(res$strata.call,res$nstrata))

ddf <- data[first,varss]
pres <- predict(res,ddf,robust=TRUE,km=km,...)
pres$formula <- formula
pres$call <- match.call()

attr(pres,"data") <- ddf
class(pres) <- c("km","predictphreg")
return(pres)
}# }}}

##' @export
summary.km <- function(object,times=NULL,type=c("cif","cumhaz","surv")[3],...) { ## {{{
   out <- summary.predictphreg(object,times=times,type=type[1],...)
   return(out)
} ## }}}

##' @export
plot.km <- function(x,...) { ## {{{
   plot.predictphreg(x,...)
}# }}}

##' Cumulative Incidence with Robust Standard Errors
##'
##' Computes cumulative incidence functions with robust standard errors.
##'
##' @param formula Formula with 'Event' outcome and \code{strata} (only!).
##' @param data Data frame.
##' @param cause Cause of interest (default is \code{NULL}, which looks at all causes).
##' @param cens.code Censoring code (default is \code{"0"}).
##' @param death.code Alternative to \code{cens.code}; specifies codes of death.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"cif"} (extends \code{"phreg"}) containing:
##' \item{cumhaz}{Cumulative incidence estimates.}
##' \item{se.cumhaz}{Standard errors.}
##' \item{cause}{Cause of interest.}
##'
##' @author Thomas Scheike
##' @examples
##' data(bmt)
##' bmt$cluster <- sample(1:100, 408, replace = TRUE)
##' out1 <- cif(Event(time, cause) ~ +1, data = bmt, cause = 1)
##' out2 <- cif(Event(time, cause) ~ +1 + cluster(cluster), data = bmt, cause = 1)
##'
##' par(mfrow = c(1, 2))
##' plot(out1, se = TRUE)
##' plot(out2, se = TRUE)
##' @export
cif <- function(formula,data=data,cause=1,cens.code=0,death.code=NULL,...)
{# {{{
  cl <- match.call()
    des <- proc_design(
        formula,
        data = data,
        specials = c("offset", "weights", "cluster","strata"),
        intercept = FALSE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    strata <- des$strata
    if (!is.null(strata)) strata.name <- names(des$xlevels)
    else strata.name <- NULL
    pos.strata <- NULL
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster
    ## no use of
    pos.cluster <- NULL
  if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)

  id.orig <- id;
  if (!is.null(id)) {
	  ids <- sort(unique(id))
	  nid <- length(ids)
      if (is.numeric(id)) id <-  fast.approx(ids,id)-1 else  {
      id <- as.integer(factor(id,labels=seq(nid)))-1
     }
   } else id <- as.integer(seq_along(exit))-1;

  statusE <- 1*(status %in% cause)
  if (is.null(death.code)) statusD <- 1*(!(status %in% cens.code)) else statusD <- 1*(status %in% death.code)
  data$statusE__ <- statusE
  data$statusD__ <- statusD
  data$strata__ <- strata

  ### setting up formulae for the two phreg (cause of interest and death)
  if (is.null(id.orig)) {
     formid <- update.formula(formula,~.+cluster(id))
     data$id <- id
  } else formid <- formula
  tt <- terms(formid)
  tt <- delete.response(tt)
  formid <- formula(tt)

  data$exit__ <- exit
  if (ncol(Y)==3) {
     data$entry__ <- entry
     formE <- as.formula(paste("Surv(entry__,exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(entry__,exit__,statusD__)~+1"))
  } else {
     formE <- as.formula(paste("Surv(exit__,statusE__)~+1"))
     formD <- as.formula(paste("Surv(exit__,statusD__)~+1"))
  }
 formE <- update.formula(formE,formid)
 formD <- update.formula(formD,formid)

  if (sum(statusE)==0) warning("No events of type 1\n");
  coxE <- phreg(formE,data=data,...)
  coxS <- phreg(formD,data=data,...)

  ### cif
  if (sum(statusE)>0) cifo <- recurrentMarginalPhreg(coxE,coxS) else cifo <- coxE
  ## to work with predict function
  ##  cifo$no.opt <- TRUE

  ### to use basehazplot.phreg
  class(cifo) <- c("cif","phreg")
  attr(cifo,"cause") <- cause
  attr(cifo,"cens.code") <- cens.code
  attr(cifo,"death.code") <- death.code
  return(cifo)
}# }}}

##' @export
plot.cif <- function(x,se=FALSE,ylab=NULL,ylim=c(0,1),conf.type=c("log","plain"),...) { ## {{{
   if (inherits(x,"cif") & is.null(ylab)) ylab <- "Probability"
   baseplot(x,se=se,ylab=ylab,ylim=ylim,restrict="prob",conf.type=conf.type[1],...)
} ## }}}

##' @export
summary.cif <- function(object,se=FALSE,ylab=NULL,times=NULL,conf.type=c("log","plain"),...) { ## {{{
   if (inherits(object,"cif") & is.null(ylab)) ylab <- "Probability"
   out <- summaryRecurrentobject(object,name="probability",times=times,conf.type=conf.type[1],restrict="prob",...)
   return(out)
} ## }}}

##' Proportional Odds Survival Model
##'
##' Fits a semiparametric proportional odds model where:
##' \deqn{ \mbox{logit}(S(t|x)) = \log(\Lambda(t)) + x \beta }
##' Thus, covariate effects represent the odds ratio (OR) of survival.
##'
##' This is equivalent to using a hazards model:
##' \deqn{ Z \lambda(t) \exp(x \beta) }
##' where \eqn{Z} is gamma distributed with mean and variance 1.
##'
##' @param formula Formula with 'Surv' outcome (similar to \code{coxph}).
##' @param data Data frame.
##' @param offset Offsets for \eqn{\exp(x \beta)} terms.
##' @param weights Weights for score equations.
##' @param ... Additional arguments passed to lower-level functions.
##'
##' @return An object of class \code{"phreg"} with \code{propodds=1}.
##'
##' @author Thomas Scheike
##' @references
##' Eriksson, Frank, Li, Jianing, Scheike, Thomas, and Zhang, Mei-Jie (2015).
##' "The proportional odds cumulative incidence model for competing risks."
##' \emph{Biometrics}, 71(3), 687--695.
##'
##' @examples
##' data(TRACE)
##' dcut(TRACE) <- ~.
##' out1 <- logitSurv(Surv(time, status == 9) ~ vf + chf + strata(wmicat.4), data = TRACE)
##' summary(out1)
##' gof(out1)
##' plot(out1)
##' @export
logitSurv <- function(formula,data,offset=NULL,weights=NULL,...)
{# {{{
   out <- phreg(formula,data,offset=offset,weights=weights,propodds=1,...)
   out$se.cumhaz <- NULL
   return(out)
}# }}}

## {{{ Utility functions, cumsumstrata ....

tailstrata <- function(strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("tailstrataR",length(strata),strata,nstrata,PACKAGE="mets")
if (any(res$found<0.5))  { warning("Not all strata found");  cat((1:nstrata)[res$found>0.5]); }
return(res$where)
}# }}}

headstrata <- function(strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("headstrataR",length(strata),strata,nstrata,PACKAGE="mets")
if (any(res$found<0.5))  { warning("Not all strata found");  cat((1:nstrata)[res$found>0.5]); }
return(res$where)
}# }}}

##' Stratified Cumulative and Summary Operations
##'
##' Low-level helper functions for computing sums, cumulative sums, and
##' reverse cumulative sums within strata groups, as well as matrix
##' double-indexing.
##'
##' @name strata-numeric
##' @param x numeric vector (or matrix for \code{matdoubleindex}).
##' @param strata integer vector of strata indices (0-based).
##' @param nstrata number of distinct strata.
##' @param rows row indices for \code{matdoubleindex}.
##' @param cols column indices for \code{matdoubleindex}.
##' @param xvec optional values to assign at indexed positions.
##' @param ... additional arguments (for \code{mdi}).
##' @return Numeric vector of the same length as \code{x} (or modified matrix).
##' @author Klaus K. Holst, Thomas Scheike
##' @aliases sumstrata cumsumstrata revcumsumstrata revcumsum matdoubleindex mdi
NULL

##' @rdname strata-numeric
##' @export
sumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("sumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
cumsumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("cumsumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @export
diffstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("diffstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
revcumsumstrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
res <- .Call("revcumsumstrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

revcumsum2strata <- function(x,strata,nstrata,strata2,nstrata2,lag=FALSE)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
res <- .Call("revcumsum2strataR",as.double(x),strata,nstrata,strata2,nstrata2,PACKAGE="mets")
return(res)
}# }}}

revcumsum2stratafdN <- function(x,y,strata,nstrata,strata2,nstrata2,startx)
{# {{{
if (any(strata<0) | any(strata>nstrata-1))    stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
if (length(x)!=length(y)) stop("length of x and y must be same\n");
res <- .Call("revcumsum2stratafdNR",as.double(x),as.double(y),strata,nstrata,strata2,nstrata2,as.double(startx),PACKAGE="mets")
return(res)
}# }}}


cumsum2strata <- function(x,y,strata,nstrata,strata2,nstrata2,startx)
{# {{{
if (any(strata<0) | any(strata>nstrata-1))    stop("strata index not ok\n");
if (any(strata2<0) | any(strata2>nstrata2-1)) stop("strata2 index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
if (length(x)!=length(strata2)) stop("length of x and strata2 must be same\n");
res <- .Call("cumsum2strataR",as.double(x),as.double(y),strata,nstrata,strata2,nstrata2,as.double(startx),PACKAGE="mets")
return(res)
}# }}}


##' @export
vecAllStrata <- function(x,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata))  stop("length of x and strata must be same\n");
res <- .Call("vecAllStrataR",x,strata,nstrata,PACKAGE="mets")$res
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
revcumsum <- function(x)
{# {{{
res <- .Call("revcumsumR",x,PACKAGE="mets")$res
return(res)
}# }}}

revcumsumstratasum <- function(x,strata,nstrata,type="all")
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n");
if (type=="sum")    res <- .Call("revcumsumstratasumR",x,strata,nstrata)$sum
if (type=="lagsum") res <- .Call("revcumsumstratasumR",x,strata,nstrata)$lagsum
if (type=="all")    res <- .Call("revcumsumstratasumR",x,strata,nstrata)
return(res)
}# }}}

cumsumstratasum <- function(x,strata,nstrata,type="all")
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (length(x)!=length(strata)) stop("length of x and strata must be same\n"); 
if (type=="sum")    res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,0)$sum
if (type=="lagsum") res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,0)$lagsum
if (type=="all")    res <- .Call("_mets_cumsumstratasumR",x,strata,nstrata,1)
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
matdoubleindex <- function(x,rows,cols,xvec=NULL)
{# {{{
if (!is.matrix(x)) stop("x must be matrix")
ncols <- ncol(x); nrows <- nrow(x)
## to avoid warnings when going to C, get rid of Inf
cols[cols==Inf] <- ncol(x)+1;
rows[rows==Inf] <- nrow(x)+1;
if (length(rows)==1) rows <- rep(rows,length(cols))
if (length(cols)==1) cols <- rep(cols,length(rows))
if (length(cols)!=length(rows)) stop("rows and cols different lengths\n");
if (is.null(xvec)) { assign <- 0; xvec <- 1} else {
	assign <- 1;
        if (length(cols)!=length(xvec)) stop("rows and cols and xvec differ \n");
}
res <- .Call("Matdoubleindex",x,rows-1,cols-1,length(cols),assign,xvec)$mat
return(res)
}# }}}

##' @rdname strata-numeric
##' @export
mdi <- function(x,...) matdoubleindex(x,...)

covfr  <- function(x,y,strata,nstrata)
{# {{{
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfR",x,y,strata,nstrata)
return(res)
}# }}}

revcumsumidstratasum <- function(x,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$sum
if (type=="lagsum")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$lagsum
if (type=="lagsumsquare") res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)$lagsumsquare
if (type=="all")    res <- .Call("revcumsumidstratasumR",x,id,nid,strata,nstrata)
return(res)
}# }}}

revcumsumidstratasumCov <- function(x,y,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$sum
if (type=="lagsum")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$lagsum
if (type=="lagsumsquare") res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$lagsumsquare
if (type=="all")    res <- .Call("revcumsumidstratasumCovR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

cumsumidstratasumCov <- function(x,y,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")   res <- .Call("cumsumidstratasumCovR",x,y,id,nid,strata,nstrata)$sum
else res <- .Call("cumsumidstratasumCovR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

cumsumidstratasum <- function(x,id,nid,strata,nstrata,type="all")
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
if (type=="sum")   res <- .Call("cumsumidstratasumR",x,id,nid,strata,nstrata)$sum
else res <- .Call("cumsumidstratasumR",x,id,nid,strata,nstrata)
return(res)
}# }}}

covfridstrata  <- function(x,y,id,nid,strata,nstrata)
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfstrataR",x,y,id,nid,strata,nstrata)
return(res)
}# }}}

covfridstrataCov  <- function(x,y,x1,y1,id,nid,strata,nstrata)
{# {{{
if (any(id<0) | any(id>nid-1)) stop("id index not ok\n");
if (any(strata<0) | any(strata>nstrata-1)) stop("strata index not ok\n");
res <- .Call("covrfstrataCovR",x,y,x1,y1,id,nid,strata,nstrata)
return(res)
}# }}}


## }}}


Mcumsumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("McumsumstrataR", A, strata, as.integer(nstrata), FALSE)$res
}

Mrevcumsumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("McumsumstrataR", A, strata, as.integer(nstrata), TRUE)$res
}


Msumstrata <- function(A, strata, nstrata) {
    if (!is.matrix(A))
        stop("A must be a matrix")
    if (nrow(A) != length(strata))
        stop("nrow(A) must equal length(strata)")
    if (min(strata) < 0 || max(strata) >= nstrata)
        stop("strata values must be in 0:(nstrata-1)")
    .Call("MsumstrataR", A, strata, as.integer(nstrata))$res
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets) 
set.seed(100)

n <- 200
ddf <- mets:::gsim(n,covs=1,null=0,cens=1,ce=1,betac=c(0.3,1))
true <- apply(ddf$TTt<2,2,mean)
true
datat <- ddf$datat
## set-random response on data, only relevant after status==2 
response <- rbinom(n,1,0.5)
datat$response <- as.factor(response[datat$id]*datat$Count2)
datat$A000 <- as.factor(1)
datat$A111 <- as.factor(1)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f,
		augmentR1=~X11+X12+TR, augmentR0=~X01+X02,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f))
bb

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01))

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]/p[2],p[3]/p[4]))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]-p[2],p[3]-p[4]))


## -----------------------------------------------------------------------------
## 2 levels for each response , fixed weights 
datat$response.f <- as.factor(datat$response)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(0,0),pi0=0.5,pi1=0.5)
bb

## 2 levels for each response ,  estimated treat probabilities
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb


## 2 and 3 levels for each response , fixed weights 
datat$A1.23.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.23.f+response)
datat <- dtransform(datat,A1.23.f=2+rbinom(nrow(datat),1,0.5),
		    Count2==1 & A1.23.f==2 & response==0)
dtable(datat,~A1.23.f+response)
datat$A1.23.f <- as.factor(datat$A1.23.f)
dtable(datat,~A1.23.f+response|Count2==1)
###
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(1,0),pi1=c(0.3,0.5))
bb

## 2 and 3 levels for each response , estimated 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb

## 2 and 1 level for each response 
datat$A1.21.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.21.f+response|Count2==1)
datat <- dtransform(datat,A1.21.f=1,Count2==1 & response==1)
dtable(datat,~A1.21.f+response|Count2==1)
datat$A1.21.f <- as.factor(datat$A1.21.f)
dtable(datat,~A1.21.f+response|Count2==1)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb

## known weights 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,0),pi1=c(0.5,1))
bb

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)

## -----------------------------------------------------------------------------
library(mets)
data(twinstut)
twinstut$binstut <- 1*(twinstut$stutter=="yes")
twinsall <- twinstut
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
head(twinstut)
twinstut <- subset(twinstut,tvparnr < 3000)

## -----------------------------------------------------------------------------
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
summary(margbin)

## ----twostage1----------------------------------------------------------------
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
                       clusters=twinstut$tvparnr,detail=0)
summary(bina)

## ----twostage2----------------------------------------------------------------
# design for OR dependence 
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bin)

## ----twostage3----------------------------------------------------------------
twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+factor(sex),data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)

## ----altsyntax1---------------------------------------------------------------
 # refers to zygosity of first subject in eash pair : zyg1
 # could also use zyg2 (since zyg2=zyg1 within twinpair's)
 out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
                response="binstut",id="tvparnr",var.link=1,
                theta.formula=~-1+factor(zyg1))
summary(out)

## ----osdesign-----------------------------------------------------------------
 # refers to zygosity of first subject in eash pair : zyg1
 # could also use zyg2 (since zyg2=zyg1 within twinpair's))
 
 desfs<-function(x,num1="zyg1",num2="zyg2")
         c(x[num1]=="dz",x[num1]=="mz",x[num1]=="os")*1
     
 margbinall <- glm(binstut~factor(sex)+age,data=twinsall,family=binomial())
 out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
       data=twinsall,response="binstut",id="tvparnr",var.link=1,
       theta.formula=desfs,desnames=c("dz","mz","os"))
 summary(out3)

## -----------------------------------------------------------------------------
library(mets)
data(twinstut)
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
twinstut$binstut <- 1*(twinstut$stutter=="yes")
head(twinstut)
twinstut <- subset(twinstut,tvparnr < 2000)

## ----biprobit1----------------------------------------------------------------
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="un")
summary(b1)

## ----bptwin1------------------------------------------------------------------
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ace")
summary(b1)

## ----bptwin2------------------------------------------------------------------
b0 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ae")
summary(b0)

## ----addgamma1----------------------------------------------------------------
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial.twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)

# test for same dependence in MZ and DZ 
theta.des <- model.matrix( ~factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial.twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)

## ----polygenic1---------------------------------------------------------------
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ace")
head(cbind(out$des.rv,twinstut$tvparnr),10)
out$pardes

## ----polygenic2---------------------------------------------------------------
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin1 <- binomial.twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin1)

## -----------------------------------------------------------------------------
concordanceTwinACE(bintwin1,type="ace")

## ----polygenic_ae-------------------------------------------------------------
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ae")

bintwin <- binomial.twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin)

## -----------------------------------------------------------------------------
concordanceTwinACE(bintwin,type="ae")

## -----------------------------------------------------------------------------
sessionInfo()

##' Lu-Tsiatis More Efficient Log-Rank for Randomized studies with baseline covariates
##'
##' Efficient implementation of the Lu-Tsiatis improvement using baseline covariates, extended to competing risks and recurrent events. Results
##' almost equivalent with the speffSurv function of the speff2trial function in the survival case. A dynamic 
##' censoring augmentation regression is also computed to gain even more from the censoring augmentation. Furhter, we also deal with twostage
##' randomizations. The function was implemented to deal with recurrent events (start,stop) + cluster, and  more examples in vignette. 
##'
##' @param formula formula with 'Surv' or 'Event' outcome (see \code{coxph}) and
##'   treatment (randomization 0/1)
##' @param data data frame
##' @param cause to use for competing risks, recurrent events data
##' @param cens.code to use for competing risks, recurrent events data
##' @param weights to be used for phreg
##' @param typesR augmentations used for randomization
##' @param typesC augmentations used for censoring
##' @param weights weights for score equation
##' @param augmentR0 formula for the randomization augmentation (~age+sex)
##' @param augmentR1 formula for the randomization augmentation (~age+sex)
##' @param augmentC formula for the censoring augmentation (~age+sex)
##' @param treat.model propensity score model, default is ~+1, assuming an RCT study
##' @param RCT if false will use propensity score adjustment for marginal model
##' @param treat.var in case of twostage randomization, this variable is 1 for
##'   the treatment times, if start,stop then default assumes that only one
##'   treatment at first record
##' @param km use Kaplan-Meier for the censoring weights (stratified on
##'   treatment)
##' @param level of confidence intervals
##' @param cens.model default is censoring model ~strata(treatment) but any
##'   model can be used to make censoring martingales
##' @param estpr estimates propensity scores
##' @param pi0 possible fixed propensity scores for randomizations
##' @param base.augment TRUE to covariate augment baselines (only for R0
##'   augmentation)
##' @param return.augmentR0 to return augmentation data
##' @param mlogit if TRUE then forces use of this function for propensity
##'   scores, default for binary treatment is glm
##' @param ... Additional arguments to phreg function
##' @author Thomas Scheike
##' @references 
##' Lu, Tsiatis (2008), Improving the efficiency of the log-rank test using auxiliary covariates, Biometrika, 679--694
##' 
##' Scheike, Nerstroem and Martinussen (2025), Randomized clinical trials and the proportional hazards model for recurrent events.
##' @examples
##' ## Lu, Tsiatis simulation
##' data <- mets:::simLT(0.7,100)
##' dfactor(data) <- Z.f~Z
##' 
##' out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
##' summary(out)
##' @export
phreg_rct <- function(formula,data,cause=1,cens.code=0,
     typesR=c("R0","R1","R01"),typesC=c("C","dynC"),weights=NULL,
     augmentR0=NULL,augmentR1=NULL,augmentC=NULL,treat.model=~+1,RCT=TRUE,
     treat.var=NULL,km=TRUE,level=0.95,cens.model=NULL,estpr=1,pi0=0.5,
     base.augment=FALSE,return.augmentR0=FALSE,mlogit=FALSE,...) {# {{{
  Z <- typeII <- NULL
  cl <- match.call()# {{{
  m <- match.call(expand.dots = TRUE)[1:3]
  special <- c("strata", "cluster","offset")
  Terms <- terms(formula, special, data = data)
  m$formula <- Terms
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  Y <- model.extract(m, "response")
  if (!inherits(Y,c("Event","Surv"))) stop("Expected a 'Surv' or 'Event'-object")
  if (ncol(Y)==2) {
    exit <- Y[,1]
    entry <- NULL ## rep(0,nrow(Y))
    status <- Y[,2]
    mstatus <- matrix(status,ncol=1)
  } else {
    entry <- Y[,1]
    exit <- Y[,2]
    status <- Y[,3]
    mstatus <- matrix(status,ncol=1)
  }
  id <- strata <- NULL
  if (!is.null(attributes(Terms)$specials$cluster)) {
    ts <- survival::untangle.specials(Terms, "cluster")
    pos.cluster <- ts$terms
    Terms  <- Terms[-ts$terms]
    id <- m[[ts$vars]]
  } else pos.cluster <- NULL

 call.id <- id;
 conid <- construct_id(id,length(exit))
 name.id <- conid$name.id; id <- conid$id+1; nid <- conid$nid

 data$id__  <-  id
 data$cid__ <- cumsumstrata(rep(1,length(id)),id-1,nid)
 expit <- lava::expit

sides <- function(formula,vars) {# {{{
lhs <- update(formula,.~+1)
rhs <- update(formula,-1~.)
if (all.vars(lhs)[1]==".") 
   formula <- update.formula(formula,as.formula(paste(vars,"~.")))
res <- list(formula=formula,lhs=lhs,rhs=rhs)
}
# }}}

ssform <- sides(formula,"")
varss <- all.vars(ssform$lhs)

## first varaible on rhs of formula
## also candidate for treat variable
streat.name <-  all.vars(ssform$rhs)[1]

treatform <- sides(treat.model,streat.name)
treat.formula <- treatform$formula
treat.name <-  all.vars(treat.formula)[1]
## }}}

treats <- function(treatvar) {# {{{
treatvar <- droplevels(treatvar)
nlev <- nlevels(treatvar)
nlevs <- levels(treatvar)
###treatvar <- as.numeric(treatvar)
ntreatvar <- as.numeric(treatvar)
return(list(nlev=nlev,nlevs=nlevs,ntreatvar=ntreatvar))
}
# }}}

fittreat <- function(treat.model,data,id,ntreatvar,nlev,mlogit=FALSE) {# {{{
if (nlev==2 & !mlogit) {
   treat.model <- drop.specials(treat.model,"cluster")
   treat <- glm(treat.model,data,family="binomial")
   iidalpha <- lava::iid(treat,id=id)
   lpa <- treat$linear.predictors 
   pal <- expit(lpa)
   pal <-cbind(1-pal,pal)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
} else {  
   treat.modelid <- update.formula(treat.model,.~.+cluster(id__))
   treat <- mlogit(treat.modelid,data)
   iidalpha <- lava::iid(treat)
   pal <- predict(treat,data,se=0,response=FALSE)
   ppp <- (pal/pal[,1])
   spp <- 1/pal[,1]
}

   ###########################################################
   ### computes derivative of D (1/Pa) propensity score 
   ###########################################################
   Xtreat <- model.matrix(treat.model,data)
   tvg2 <- 1*(ntreatvar>=2)
   pA <- c(mdi(pal, 1:length(ntreatvar), ntreatvar))
   pppy <- c(mdi(ppp,1:length(ntreatvar), ntreatvar))
   Dppy <-  (spp*tvg2-pppy) 
   DpA <- c()
   for (i in seq(nlev-1)) DpA <- cbind(DpA,Xtreat*ppp[,i+1]*Dppy/spp^2);  
   DPai <- -1*DpA/pA^2

   ## Dp binomial
   Dp <- Xtreat*(pal[,1]*(1-pal[,1]))

out <- list(iidalpha=iidalpha,pA=pA,Dp=Dp,DpA=Dp,pal=pal,ppp=ppp,spp=spp,id=id,DPai=DPai)
return(out)
} # }}}

if (!is.null(treat.var)) { # {{{
	## time-changing weights
	weightWT <- data[,treat.var]
	whereW <- which(weightWT==1)
	CountW <- cumsumstrata(weightWT,id-1,nid)
        dataW <- data[whereW,]; 
	CountWW <- CountW[whereW]
        idW <- id[whereW]; } 
else { ## only first record is associated with treatment  
	weightWT <- rep(1,nrow(data))
	cid <- cumsumstrata(weightWT,id-1,nid)
	## first record of each subject is treatment 
	whereW <- which(cid==1)
	CountW <- cumsumstrata((cid==1)*1,id-1,nid)
	## constant weights 
	CountWW <- CountW[whereW]
	dataW <- data[whereW,]
        idW <- id[whereW]
	if (!is.null(augmentR1)) warning("must specify treat.var to indicate where treatment is given\nassumes now that treatment is given only at start\n")
} 
# }}}

treatvar <- dataW[,treat.name]
if (!is.factor(treatvar)) stop(paste("treatment=",treat.name," must be factor \n",sep="")); 
treats <- treats(treatvar)

if (estpr[1]==1 ) {
   fitt <- fittreat(treat.formula,dataW,idW,treats$ntreatvar,treats$nlev,mlogit=mlogit)
   pi0 <- fitt$pal[,-1]
   ## p(A) 
   wPA <- c(fitt$pA)
   DPai <- fitt$DPai
} else {
   ## assumes constant fixed prob over groups
   wPA <- ifelse((treats$ntreatvar)[idW]==2,pi0[1],1-pi0[1])         
   pi0 <- rep(pi0[1],length(idW))
}

## construct multiplicative weights, with possible start stop structure
## put propensity score weights at time of weight change, only when RCT=FALSE
ww <-  rep(1,nrow(data))
ww[whereW] <- wPA
wwt <- exp(cumsumstrata(log(ww),id-1,nid))

## set propensity score weights for Cox model's below 
if (!RCT) ww <- 1/wwt  else ww <- rep(1,nrow(data))

if (!is.null(weights)) ww <- weights*ww

rsss <- all.vars(formula)
if (ncol(Y)==2) 
rformulaS <-as.formula( paste("Surv(",rsss[1],",",rsss[2],"==",cause,")~."))
else 
rformulaS <-as.formula( paste("Surv(",rsss[1],",",rsss[2],",",rsss[3],"==",cause,")~."))
formula <- update(formula,rformulaS)

## change id from call to id__
formula <- drop.specials(formula,"cluster")
formula <- update(formula, .~.+cluster(id__))

if (RCT) {
### ... for phreg
fit0 <- phreg(formula,data=data,...)
if (fit0$p>0) eaM <- ea <- ea.iid <- (lava::iid(fit0) %*% fit0$hessian)
else ea <- eaM  <- ea.iid <- matrix(0,max(fit0$id),1)
} else {
fit0 <- phreg_IPTW(formula,data=data,treat.model=treat.formula,treat.var=treat.var,estpr=estpr,pi0=pi0,...)
if (fit0$p>0) {
ea <- ea.iid <- fit0$IID %*% fit0$hessian
## iid without Taylor expansion in weights, to use for censoring augmentation
eaM <- fit0$beta.iid %*% fit0$hessian
} else  ea <- eaM  <- ea.iid <- matrix(0,max(fit0$id),1)
}


data.augR0 <- NULL
AugR0 <- AugR1 <- AugR01 <- rep(0,ncol(ea))
AugR0.iid <-  AugR1.iid <-  AugR01.iid <- matrix(0,nrow(ea),ncol(ea))
if (!is.null(augmentR0)) {# {{{
   ## design 
   ff0 <- sides(augmentR0,all.vars(ssform$rhs)[1])
   dataW0 <- subset(dataW,CountWW==1)
   idW0 <- idW[CountWW==1]
   XR <- model.matrix(augmentR0,dataW0) 
   Z0 <- dataW0[,all.vars(ff0$formula)[1]]
   if (is.factor(Z0)) Z0 <- as.numeric(Z0)-1
   ## order after idW0
   XR[idW0, ] <-  XR 
   Z0[idW0] <- Z0
   strata0 <- piW0 <- rep(0,length(idW0))
   piW0[idW0] <- pi0[CountWW==1]
   ## strata0 0 ??  
   strata0 <- fit0$strata.call[idW0]

   XRpi <- (Z0-piW0)*XR
   XR0pi <- XRpi 
   if (estpr[1]==1) {
       Dp0 <- matrix(0,nid,ncol(fitt$Dp))
       Dp0[idW0,]  <- fitt$Dp[CountWW==1,]
    }
   if (return.augmentR0)  data.augR0 <- list(ea=ea,XRpi=XRpi,XR=XR,A=Z0,p0=piW0)

   augR0fit <- lm(ea~-1+XRpi)
   XRgamma <- augR0fit$fitted.values/(Z0-piW0)
   AugR0.iid <- matrix(augR0fit$fitted.values,ncol=ncol(ea))
   AugR0 <- apply(AugR0.iid,2,sum)


   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
	 Dp0f <- crossprod(Dp0,XRgamma) 
         iid.treat <- fitt$iidalpha %*% Dp0f
	 ### same for RCT \hat \pi(,X)
         AugR0.iid <- AugR0.iid - iid.treat
      } 
      ## oucome model iid 
      if (!RCT) {
         DpO <- apply(XRpi,2,sum)
         iid.outcome <- iid(augR0fit)
###         iid.outcome <- iid.outcome %*% DpO
###         AugR0.iid <- AugR0.iid-iid.outcome
      }

} # }}}

if (!is.null(augmentR1)) {# {{{
   ff1 <- sides(augmentR1,all.vars(ssform$rhs)[2])
   dataW1 <- subset(dataW,CountWW==2)
   idW1 <- idW[CountWW==2]
   XR11 <- model.matrix(augmentR1,dataW1) #[,-1,drop=FALSE]
   XR1 <- matrix(0,nid,ncol(XR11))
   XR1[idW1,] <- XR11
   Z1 <- dataW1[,all.vars(ff1$formula)[1]]
   if (is.factor(Z1)) Z1 <- as.numeric(Z1)-1
   piW1 <- pi0[CountWW==2]
   Z1p <- (Z1-piW1)
   Z1p1  <-  rep(0,nid)
   Z1p1[idW1] <- Z1p
   XR1pi <- Z1p1*XR1

   if (estpr[1]==1) {
	 Dp1 <- matrix(0,nid,ncol(fitt$Dp))
	 Dp1[idW1,]  <- fitt$Dp[CountWW==2,]
   }

   augR1fit <- lm(ea~-1+XR1pi)
   XRgamma <- XR1 %*%  coef(augR1fit)
   AugR1.iid <- augR1fit$fitted.values 
   AugR1 <- apply(AugR1.iid,2,sum)

   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
	 Dp1f <- crossprod(Dp1,XRgamma) 
         iid.treat <- fitt$iidalpha %*% Dp1f
	 ### same for RCT \hat \pi(,X)
         AugR1.iid <- AugR1.iid - iid.treat
      } 

      ## oucome model iid 
      if (!RCT) {
###         DpO <- apply(XR1pi,2,sum)
###         iid.outcome <- iid(augR1fit)
###         iid.outcome <- iid.outcome %*% rep(DpO,ncol(ea))
###         AugR1.iid <- AugR1.iid-iid.outcome
      }

} # }}}

if (!is.null(augmentR0) & !is.null(augmentR1)) {# {{{
   XRbpi <- cbind(XRpi,XR1pi)
   XRb <- cbind(XR,XR1)
   xxi <- solve(crossprod(XRbpi)) 

   augR01fit <- lm(ea~-1+XRbpi)
   XRgamma <- XRb %*%  coef(augR01fit)
   AugR01.iid <- augR01fit$fitted.values 
   AugR01 <- apply(AugR01.iid,2,sum)

   ## iid term for predicted P(treat=1)
      if (estpr[1]==1) {
         XRgamma0 <- XRpi %*% coef(augR01fit)[1:ncol(XRpi),]
         XRgamma1 <- XR1pi %*% coef(augR01fit)[-(1:ncol(XRpi)),]
         Dp01f <- crossprod(Dp0,XRgamma0)+crossprod(Dp1,XRgamma1)
         iid.treat <- fitt$iidalpha %*% Dp01f
         AugR01.iid <- AugR01.iid - iid.treat
      } 
      ## oucome model iid 
      if (!RCT) {
         DpO <- apply(XRbpi,2,sum)
###         iid.outcome <- iid(augR01fit)
###         iid.outcome <- iid.outcome %*% rep(DpO,ncol(ea))
###         AugR01.iid <- AugR1.iid-iid.outcome
      }

} else {AugR01.iid <- AugR0.iid+AugR1.iid; AugR01 <- AugR0+AugR1;  }
# }}}

varC.improve <- 0; formulaC <- NULL
AugC <- AugC.times <- AugClt <- rep(0,ncol(ea)) 
AugC.iid <- AugClt.iid <- matrix(0,nrow(ea),ncol(ea))

## compute regression augmentation for censoring martingale 
if ((!is.null(augmentC))) {## {{{

xxx <- fit0$cox.prep
### X fra GL + tail-death 
rr <- c(exp(xxx$X %*% coef(fit0)+ xxx$offset)*xxx$weights)
Zrr <- xxx$X*rr
S0i <- rep(0,nrow(xxx$X))
jumps <- xxx$jumps+1
S0i[jumps] <- 1/fit0$S0
E <- U <- matrix(0,nrow(Zrr),ncol(Zrr))
U[jumps,] <- fit0$U
E[jumps,] <- fit0$E
ZEdN <- apply(U,2,revcumsumstrata,xxx$id,nid)
cumhaz <- cumsumstrata(S0i,xxx$strata,xxx$nstrata)
EdLam0 <- apply(E*S0i,2,cumsumstrata,xxx$strata,xxx$nstrata)

mmA <- cbind(status,weightWT,CountW)

## formulac with or without start,stop formulation
if (is.null(cens.model)) 
  cens.model <- as.formula(paste("~strata(",treat.name,")+cluster(id__)"))
else cens.model <- update.formula(cens.model,.~.+cluster(id__))
rrss <- all.vars(ssform$lhs)
if (length(rrss)==2) 
formulaC <-as.formula( paste("Surv(",rrss[1],",",rrss[2],"==",cens.code,")~."))
else 
formulaC <-as.formula( paste("Surv(",rrss[1],",",rrss[2],",",rrss[3],"==",cens.code,")~."))
formulaC <- update.formula(formulaC,cens.model)
###
varsC <- attr(terms(augmentC),"term.labels")
formCC <- update(formulaC, reformulate(c(".", varsC)))

Cfit0 <- phreg(formCC,data=data,no.opt=TRUE,no.var=1,Z=mmA,...)

  ### computing weights for censoring terms
  x <- Cfit0
  xx <- x$cox.prep
  S0i <- rep(0,length(xx$strata))
  jumpsC <- xx$jumps+1
  S0i[jumpsC] <-  1/x$S0
  ## G_c survival at t- 
  if (!km) {
    cumhazD <- c(cumsumstratasum(S0i,xx$strata,xx$nstrata)$lagsum)
    St      <- exp(-cumhazD)
  } else St <- c(exp(cumsumstratasum(log(1-S0i),xx$strata,xx$nstrata)$lagsum))
  ###
  rr0 <- c(xx$sign)
  XXA <- xx$X
  EA <- Cfit0$E
  UA <- Cfit0$U
  dhessian <- Cfit0$hessianttime
  hesst <-  .Call("XXMatFULL",dhessian,Cfit0$p,PACKAGE="mets")$XXf

CovZXstrata <- function(X,Ej,Z,Sign,strata,nstrata,jumps) 
{# {{{
		strata  <- c(strata); Sign <- c(Sign)
         ###	Ej <- Ej[jumps,,drop=FALSE]; Ej <- Ej
		ZE <- apply(Z*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
		XZ  <- .Call("vecMatMat",X,Z)$vXZ;  
		XZ <- apply(XZ*Sign,2,revcumsumstrata,strata,nstrata)[jumps,,drop=FALSE]; 
		EXZ  <- .Call("vecMatMat",Ej,ZE)$vXZ;  
		out <- XZ-EXZ
		return(out)
}# }}}

fid <- headstrata(xxx$id,nid)

  ## {{{
  if (any(CountWW==2)) {
  rrd <- rr-rr[fid][xxx$id+1]
  Zrrd <- Zrr-Zrr[fid,,drop=FALSE][xxx$id+1,]

  ## place to start with Lam_o(T_R) 
  jumpsW <- which((xx$Z[,2]==1)*(xx$Z[,3]==2)*(xx$sign==-1)==1)
  EdLamTR <- XcumTR <- matrix(0,length(xxx$strata),ncol(xxx$X))
  XcumTR[jumpsW,] <-  Zrrd[jumpsW,]*cumhaz[jumpsW]
  XcumTR <- apply(XcumTR,2,cumsumstrata,xx$id,nid)

  ###  dd <-   cbind(XcumTR,xx$id,xxx$id,xx$status,xx$sign,xxx$sign,xx$Z,xx$time,rr)
  ###  jumpsW; dd[dd[,4]==61,]; dd[dd[,4]==20,]; dd[dd[,4]==95,]; cumhaz[jumpsW]; 

  EdLamTR[jumpsW,] <- EdLam0[jumpsW,]*rrd[jumpsW]
  EdLamTR <- apply(EdLamTR,2,cumsumstrata,xx$id,nid)

  ###  XcumTR <- xx$X*c(cumTR)
  covXTRsZ <-   CovZXstrata(XXA,EA,XcumTR,rr0,xx$strata,xx$nstrata,jumpsC) 
  covELTRsZ <-  CovZXstrata(XXA,EA,EdLamTR,rr0,xx$strata,xx$nstrata,jumpsC) 
  covttt <- covXTRsZ-covELTRsZ
  } else  covttt <- 0
  ## }}}

###  dd <-   cbind(cumTR,xxx$id,xx$sign,xx$Z,xx$time) 

covXsZ <-   CovZXstrata(XXA,EA,Zrr,rr0,xx$strata,xx$nstrata,jumpsC) 
covXsrr <-  CovZXstrata(XXA,EA,as.matrix(rr,ncol=1),rr0,xx$strata,xx$nstrata,jumpsC) 
covXsUs3 <- .Call("vecMatMat",covXsrr,EdLam0[jumpsC,,drop=FALSE])$vXZ;  
covXsUs2 <- covXsZ*cumhaz[jumpsC]-covXsUs3 
### U(infty)= UU
Uinfiid <- -eaM[xx$id+1,,drop=FALSE]
cZEdN <- ZEdN[fid,,drop=FALSE][xx$id+1,,drop=FALSE]-ZEdN
Us1 <- Uinfiid-cZEdN
covXsUs1 <- CovZXstrata(XXA,EA,Us1,rr0,xx$strata,xx$nstrata,jumpsC) 
## scale with Y_(s) because hessiantime is also scaled with this 
covXsYs <- (covXsUs1+covXsUs2-1*covttt)/c(Cfit0$S0); 

p <- ncol(ea)
pXXA <- ncol(XXA)
gammat <-  -.Call("CubeMattime",hesst,covXsYs,pXXA,pXXA,pXXA,p,1,0,0,PACKAGE="mets")$XXX
### solve(matrix(hesst[1,],3,3)) %*% matrix(covXsYs[1,],3,2)
gammat[is.na(gammat)] <- 0
gammat[gammat==Inf] <- 0
augmentt <- .Call("CubeMattime",gammat,UA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
AugCdyn <-  apply(augmentt,2,sum)
gain.times <- .Call("CubeMattime",covXsYs,gammat,pXXA,p,pXXA,p,0,1,0,PACKAGE="mets")$XXX
vardynC.improve  <- matrix(apply(gain.times,2,sum),p,p)

## regress U(s)=\int_s^\infty (Z-E) w(s) dM(s) on agument-model among survivors 
## U(s) = U(\infty) - \int_0^s (Z-E) w(s)  dM(s)
## sum (e_i - \bar e) U(s) Y_i(s)
  ## Lu-Tsiatis augmentation 
  out1 <- iidBaseline(Cfit0,ft=1/St,time=0,fixbeta=0)
  Hiid <- (out1$beta.iid %*% Cfit0$hessian)
  xxi <- solve(crossprod(Hiid))
  ###
  for (i in 1:ncol(ea)) {
     gamma <- xxi %*% crossprod(Hiid, eaM[,i])
     AugClt.iid[,i] <- Hiid %*%  gamma
     AugClt[i] <- sum(AugClt.iid[,i])
  }

Gcj <- St[jumpsC]
varZdN <- matrix(apply(hesst/c(Gcj^2),2,sum),pXXA,pXXA)
covXYdN <- matrix(apply(covXsYs/c(Gcj),2,sum),p,pXXA,byrow=TRUE) 
gamma <- -1*.Call("CubeMattime",matrix(varZdN,nrow=1),matrix(covXYdN,nrow=1),pXXA,pXXA,p,pXXA,1,0,1,PACKAGE="mets")$XXX
gamma <- matrix(gamma,p,pXXA,byrow=TRUE)
gamma[is.na(gamma)] <- 0; gamma[gamma==Inf] <- 0
augment <- c(gamma %*% apply(UA/c(Gcj),2,sum))
var.Clt.improve <-  gamma %*% t(covXYdN) ###  /(nid^2)

#### iid magic  for censoring augmentation martingale{{{
### int_0^infty gamma (e_i - ebar(s)) 1/G_c(s) dM_i^c
S0iG <- S0i <- rep(0,length(xx$strata))
S0iG[jumpsC] <- 1/c(x$S0*Gcj)
S0i[jumpsC] <-  1/x$S0
U <- E <- matrix(0,nrow(xx$X),pXXA)
E[jumpsC,] <- EA; 
U[jumpsC,] <- UA/c(Gcj)
cumhaz <- cumsumstrata(S0iG,xx$strata,xx$nstrata)
EdLam0 <- apply(E*S0iG,2,cumsumstrata,xx$strata,xx$nstrata)

gammasEs <- .Call("CubeMattime",gammat,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
gammasE <- matrix(0,nrow(XXA),p)
gammatt  <-    matrix(0,nrow(XXA),pXXA*p)
gammasE[jumpsC,] <- gammasEs
gammatt[jumpsC,] <- gammat
gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
gammadLam0 <-   apply(gammatt*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
Ut <- Et <- matrix(0,nrow(XXA),p)
Ut[jumpsC,] <- augmentt
MGCtt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
MGCttiid <- apply(MGCtt,2,sumstrata,xx$id,nid)
AugCdyn.iid <-  MGCttiid ## %*% iH
# }}}

} else {
iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
var.augment.times <- var.augment <- NULL
var.augment.times.iid <- var.augment.iid <- NULL
Uiid.augment.times <- Uiid.augment <- NULL
time.gammat <- gamma <- gammat <- NULL
ftime.gamma <- NULL
Gcj <- NULL
} ## }}}

## Fitting all models with augmentation terms 
if (fit0$p>0) {
coefMarg <- estimate(coef = coef(fit0), vcov = fit0$var, level = level)$coefmat
var.names <- rownames(coefMarg)
rownames(coefMarg) <- paste("Marginal",rownames(coefMarg),sep="-")
coefs <- coefMarg
} else coefs <- NULL

if (length(typesR)!=3) typesRR <- typesR else 
typesRR <- typesR[c(!is.null(augmentR0),!is.null(augmentR1),!is.null(augmentR0) & !is.null(augmentR1)) ]

if (length(typesC)!=2) typesCC <- typesC else 
typesCC <- typesC[rep(!is.null(augmentC),length(typesC))]
if (is.null(augmentC)) {
	AugCdyn <- AugClt <- AugCdyn.iid <- AugClt.iid <-  0
	typesCC <- "none"
}
if (length(typesRR)==0) { typesRR <- "none" }


baselinecox <- cumhazR0 <- cumhaz <-  se.cumhaz <- se.R0cumhaz <- NULL
iidn <- c()
varbeta <- iid <- fitt <- list(); j <- 0
for (typeR in typesRR) 
for (typeC in typesCC) {# {{{
if (typeR!=typeC) {
   j <- j+1
   AugR <- (typeR=="R0")*AugR0+ (typeR=="R1")*AugR1+(typeR=="R01")*AugR01 +0 
   AugR.iid <- 0+(typeR=="R0")*AugR0.iid+(typeR=="R1")*AugR1.iid + (typeR=="R01")*AugR01.iid
   AugC <- (typeC=="C")*AugClt+(typeC=="dynC")*AugCdyn+0
   Aug <- AugR+AugC

   if (fit0$p>0) iid[[j]] <- -1*(ea.iid-AugR.iid ) %*% fit0$ihessian else iid[[j]] <- 0
   var.beta <- crossprod(iid[[j]])
   AugC.iid <- 0+(typeC=="C")*AugClt.iid+ (typeC=="dynC")*AugCdyn.iid
   if (typeC=="dynC") {
       var.beta <- var.beta - fit0$ihessian %*% vardynC.improve%*% fit0$ihessian
   }
   if (typeC=="C") {
       var.beta <- var.beta - fit0$ihessian %*% var.Clt.improve%*% fit0$ihessian
   }
   if (fit0$p>0) varbeta[[j]] <- var.beta

   if (fit0$p>0) 
   fitts <- phreg(formula,data=data,augmentation=Aug,no.var=1,weights=ww,...)
   else fitts <- fit0

   ## only baseline augment with R0 augmentation
   if (base.augment & typeR=="R0" & (typeC!="dync" | typeC!="C")) { ## {{{
      xxx <- fitts$cox.prep
      xx <- crossprod(XR0pi)
      xxi <- solve(xx)
      XRpit <- XR0pi[xxx$id+1,]
      if (fit0$p>0) rr <- c(exp(xxx$X %*% coef(fitts)+ xxx$offset)*xxx$weights)
      else rr <- c(exp(xxx$offset)*xxx$weights)
      rrs <- rr*c(xxx$sign)
      XRE <- apply(XRpit*rrs,2,revcumsumstrata,xxx$strata,xxx$nstrata)
      S0i2 <- S0i <- rep(0,nrow(xxx$X))
      jumps <- xxx$jumps+1
      S0i[jumps] <- 1/fitts$S0
      ww <- xxx$caseweights*xxx$weights
      S0i2[jumps] <- 1/(fitts$S0^2*ww[xxx$jumps+1])
      ###
      U <- matrix(0,nrow(XRE),ncol(XRE))
      U[jumps,] <- XRpit[jumps,]*S0i[jumps]
      NXRE <- apply(U,2,cumsumstrata,xxx$strata,xxx$nstrata)[jumps,]
      XREdLam0 <- apply(XRE*S0i2,2,cumsumstrata,xxx$strata,xxx$nstrata)[jumps,]
      ns <- c(sumstrata(rep(1,nid),xxx$strata[headstrata(xxx$id,nid)],xxx$nstrata))[xxx$strata[jumps,]+1]
      if (RCT) covBase <- (NXRE-XREdLam0) else covBase <- (NXRE-XREdLam0)

      gamR0Base <- (covBase) %*% xxi
      XRs <- apply(XR0pi,2,sum)
      R0baseline.augment <-  c(XRs %*% t(gamR0Base))
      R0baseline.reduction<- apply(covBase*gamR0Base,1,sum)
      ## using augmented estimator of beta 
      if (fit0$p>0) fitr0 <- robust.phreg(fitts,beta.iid=iid[[j]])
      else fitr0 <- robust.phreg(fit0)
      cumhazR0 <- cumhaz <- fitts$cumhaz
      cumhazR0[,2] <- cumhaz[,2]-R0baseline.augment
      se.R0cumhaz <- se.cumhaz <- fitr0$robse.cumhaz
      se.R0cumhaz[,2] <- (se.cumhaz[,2]^2-R0baseline.reduction)^.5
      baselinecox <- list(phreg=fitts,beta.iid=iid[[j]],XR0pi=XR0pi,gamR0Base=gamR0Base)
   } else baselinecox <- cumhazR0 <- cumhaz <-  se.cumhaz <- se.R0cumhaz <- NULL  ## }}}

   if (fit0$p>0)  {
     coeffitt <- estimate(coef=coef(fitts),vcov=var.beta,level=level)$coefmat
     nnn <- paste(typeR,typeC,sep="_")
     iidn <- c(iidn,nnn)
     rownames(coeffitt) <- paste(nnn,var.names,sep=":")
     coefs <- rbind(coefs,coeffitt)
     if (!is.null(augmentC)) iid[[j]] <- iid[[j]] - AugC.iid%*% fit0$ihessian
   }
   } 
}
names(iid) <- iidn
# }}}

namesortme <- function(iid,name.id) { ## {{{
if (is.matrix(iid))  
	if (nrow(iid)==length(name.id)) {
		rownames(iid) <- name.id
		oid <- order(name.id)
		iid <- iid[oid,]
}
return(iid)
} ## }}}

### sort iid after name.id and put as rownames
if (!is.null(call.id)) {
ea.iid <- nameme(ea.iid,name.id)
for (l in 1:length(iid)) iid[[l]] <- nameme(iid[[l]],name.id)
AugC.iid <- nameme(AugC.iid,name.id)
AugClt.iid <- nameme(AugClt.iid,name.id)
}

out <- list(marginal=fit0,AugR0=AugR0,AugR1=AugR1,AugR01=AugR01,AugCdyn=AugCdyn,
    AugClt=AugClt,
    coefs=coefs,iid=iid,AugC.iid=AugC.iid,AugClt.iid=AugClt.iid,Cox.iid=ea.iid,
    formula=formula,formulaC=formulaC,treat.model=treat.model,
    id=id,call.id=call.id,name.id=name.id,
    cumhaz=cumhazR0,se.cumhaz=se.R0cumhaz,
    cumhaz.noAug=cumhaz,se.cumhaz.noAug=se.cumhaz, 
    strata=fit0$strata.jumps,nstrata=fit0$nstrata,jumps=seq_along(fit0$strata.jumps),
    strata.name=fit0$strata.name,strata.level=fit0$strata.level,
    baselinecox=baselinecox,data.augR0=data.augR0,var.beta=varbeta)
class(out) <- "phreg_rct"
return(out)
} ## }}} 


##' @export
plot.phreg_rct  <- function(x,...)  { ## {{{
	if (!is.null(x$cumhaz)) baseplot(x,...) 
} ## }}}

##' @export
print.phreg_rct <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.phreg_rct <- function(object,...) {# {{{
if (!is.null(object$coefs)) res  <- object$coefs else res <- "only baseline"
class(res) <- "summary.phreg_rct"
return(res)
}# }}}

simLT <- function(rho,n,beta=0,betac=0,ce=1,betao=0)
{# {{{
Sigma <- matrix(c(1,rho,rho,1),2,2)
M <- t(chol(Sigma))
# M %*% t(M)
Z <- matrix(rnorm(2*n),2,n) # 2 rows, N/2 columns
XY <- t(M %*% Z) 
###
X <- XY[,1]
Y <- XY[,2]
if (betao!=0) px <- expit(X*betao) else px <- 0.5
Z <- rbinom(n,1,px)
TT <- -exp(Z*beta)*log(1-pnorm(Y))
C <- exp(Z*betac)*rexp(n)*ce
status <- (TT<C)
mean(status)
time <- pmin(TT,C)
data <- data.frame(time=time,status=status,X=X,Z=Z)
return(data)
}# }}}

simLTTS <- function(rho,n,beta=c(0,0),betac=0,ce=1,cr=0.5,betao=0.4)
{# {{{

## to avoid R-check
TR <- Z1.f <- count2 <- X1 <- NULL

sigma <- matrix(rho,4,4)
diag(sigma) <- 1
m <- t(chol(sigma))
# m %*% t(m)
z <- matrix(rnorm(4*n),4,n) # 2 rows, n/2 columns
xy <- t(m %*% z) 
zz <- matrix(rnorm(4*n),4,n) # 2 rows, n/2 columns
xyz <- t(m %*% zz) 
###
x <- xy[,1]
y <- xy[,2]
tr <- xy[,3]
x1 <- xyz[,1]
y1 <- xyz[,2]
if (betao!=0) px <- lava::expit(x*betao) else px <- 0.5
if (betao!=0) px1 <- lava::expit(x1*betao+tr) else px1 <- 0.5
z0 <- rbinom(n,1,px)
z1 <- rbinom(n,1,px1)
tt0 <- -exp(z0*beta[1])*log(1-pnorm(y))
tt1 <- -exp(z0*beta[1]+z1*beta[2])*log(1-pnorm(y1))  ## log(1-pnorm(y1))
tr <- exp(z0*beta[1])*rexp(n)*cr 
tt <- ifelse(tt0<tr,tt0,tr)+(tt0>tr)*(tt1)
c <- exp(z0*betac)*rexp(n)*ce
status <- (tt<c)
time <- pmin(tt,c)
data <- data.frame(time=time,status=status,X=x,X1=x1,Z0=z0,Z1=z1,TR=tr)
data <- event.split(data,cuts="TR")
###data <- EventSplit(data,cuts="TR")
data <- dtransform(data,status=2,TR==time)
data <- dtransform(data,Z1=0,start<TR)
data$count2 <- 1*(data$start==data$TR)
data$Count2 <- 1*(data$start==data$TR)
data$cw <- 1
data$Zt.f <- factor(data$Z0)
data$Z0.f <- factor(data$Z0)
data$Z1.f <- factor(data$Z1)
data$Xt <- data$X
data <- dtransform(data,Zt.f=Z1.f,count2==1)
data <- dtransform(data,Xt=X1,count2==1)
data$X0 <- data$X
data$X1t <- 0
data <- dtransform(data,X1t=X1,count2==1)

return(data)
}# }}}

simLUCox <- function(n,cumhaz,death.cumhaz=NULL,cumhaz2=NULL,r1=NULL,r2=NULL,rd=NULL,
		     rc=NULL,rtr=NULL,ztr=0,betatr=0.3,A0=NULL,Z0=NULL,efTR=0,
   gap.time=FALSE,max.recurrent=100,dhaz=NULL,haz2=NULL,dependence=4,var.z=1,cor.mat=NULL,cens=NULL,...) 
{# {{{
  status <- death <- fdeath <- dtime <- NULL # to avoid R-check 

  if (dependence==0) { z <- z1 <- z2 <- zd <- rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- z; z2 <- z; zd <- z 
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
      } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; 
	      zd <- rep(1,n) 
	      z <- z1
      }      else stop("dependence 0-4"); # }}}

      if (!is.null(Z0)) {
	      if (dependence==1) zd <- Z0
	      z <- z1 <- z2 <- Z0
      }

   if (is.null(r1)) r1 <- rep(1,n)
   if (is.null(r2)) r2 <- rep(1,n)
   if (is.null(rd)) rd <- rep(1,n)
   if (is.null(rtr)) rtr <- rep(1,n)
   if (is.null(rc)) rc <- rep(1,n)

  ## extend cumulative for death to full range  of cause 1
  if (!is.null(death.cumhaz)) {
	  if (!is.null(cumhaz2))  {
	     out <- extendCums(list(cumhaz,cumhaz2,death.cumhaz),NULL)
	     cumhaz <- out$cum1
	     cumhaz2 <- out$cum2
	     cumhazd <- out$cum3
	  } else  {
	     out <- extendCums(list(cumhaz,death.cumhaz),NULL)
	     cumhaz <- out$cum1
	     cumhazd <- out$cum2
	     cumhaz2 <- NULL
     }
  }  else cumhaz <- rbind(c(0,0),cumhaz)

  ll <- nrow(cumhaz)
  max.time <- tail(cumhaz[,1],1)

### recurrent first time
  tall1 <- rchaz(cumhaz,r1*z1)
  if (ztr==1)  {
  ## should deal with selection, we know that TR has not happened, when A1(t)=0
  cum1A00 <- cbind(cumhaz[,1],cumsum(c(0,diff(cumhaz[,2]))*(1+var.z*cumhaz2[,2])))
  rtr1 <- exp(betatr)
  cum1A01 <- cbind(cumhaz[,1],cumsum(c(0,diff(cumhaz[,2]))*(1+var.z*cumhaz2[,2]*rtr1)))
  tallA0 <- rchaz(cum1A00,z1[A0==0]*r1[A0==0])
  tallA1 <- rchaz(cum1A01,z1[A0==1]*r1[A0==1])
  tall1[A0==0,] <- tallA0
  tall1[A0==1,] <- tallA1
  } else { cum1A00 <-cum1A01 <-  cumhaz; }
  tall <- tall1
  tall$id <- 1:n
  tall$tr <- 0
  tall$sel <- 1
  tall$timetr <- 0


  ## if tr occurs put tr=1
  if (!is.null(cumhaz2))  {
if (ztr==1) tall2 <- rchaz(cumhaz2,z1*rtr)
if (ztr==0) tall2 <- rchaz(cumhaz2,rtr)
       tall$status <- ifelse(tall1$time<tall2$time,tall1$status,2*tall2$status)
       tall$time <- ifelse(tall1$time<tall2$time,tall1$time,tall2$time)
       tall$tr <- ifelse(tall2$time<tall1$time,1,0)
       tall$timetr <- ifelse(tall2$time<tall1$time,tall2$time,0)
if (ztr==1) {
       ww <- (tall2$time<=tall1$time)
       baseTR <- cpred(cumhaz2,tall2$time[ww])
       basesel <- rtr[ww]*baseTR[,2]
       tall$sel[ww] <- (1+var.z*basesel)/(1+var.z)
  }
  }

### death time simulated
  if (!is.null(death.cumhaz)) {
	  timed   <- rchaz(cumhazd,zd*rd)
	  tall$dtime <- timed$time
	  tall$fdeath <- timed$status
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  } else { 
	  tall$dtime <- max.time; 
	  tall$fdeath <- 0; 
	  cumhazd <- NULL 
	  if (!is.null(cens)) { 
             ctime <- rexp(n)/(rc*cens)
	     tall$fdeath[tall$dtime>ctime] <- 0; 
	     tall$dtime[tall$dtime>ctime] <- ctime[tall$dtime>ctime] 
	  }
  }

### fixing the first time to event, if death or censoring occurs before
  tall$death <- 0
  tall <- dtransform(tall,death=fdeath,time>dtime)
  tall <- dtransform(tall,status=0,time>dtime)
  tall <- dtransform(tall,time=dtime,time>dtime)
  tt <- tall
  i <- 1
  while (any((tt$time<tt$dtime) & (tt$status!=0) & (i < max.recurrent))) {
	  i <- i+1
	  still <- subset(tt,time<dtime & status!=0)
	  nn <- nrow(still)
	  tr <- still$tr
 	  sel <- still$sel
 	  timetr <- still$timetr
	  r1tr <- rep(1,nn)
###	  if (efTR!=0) r1tr <- exp(efTR*timetr); 
	  rrt <- r1tr*r1[still$id]*(1-tr)+r1tr*r1[still$id]*r2[still$id]*tr
        if (ztr==0) tt1 <- rchaz(cumhaz,rrt*z1[still$id],entry=(1-gap.time)*still$time)
	if (ztr==1)  {

             ## sets up dataframe, overwrites below 
             tt1 <- rchaz(cumhaz,rrt*z1[still$id],entry=(1-gap.time)*still$time)
		if (any(tr==0)) {
			rz <- r1[still$id]*z1[still$id]
			A0s <- A0[still$id]
			if (any(A0s[tr==0]==0)) {
                          ww0 <-  A0s[tr==0]==0
			  tallA0 <- rchaz(cum1A00,rz[ww0],entry=still$time[ww0])
			  tt1[A0s[tr==0]==0,] <- tallA0
			}
			if (any(A0s[tr==0]==1)) {
                          ww1 <-  A0s[tr==0]==1
			  tallA0 <- rchaz(cum1A00,rz[ww1],entry=still$time[ww1])
			  tt1[A0s[tr==0]==1,] <- tallA0
			}
		}
		if (any(tr==1)) {
		    rrt <- r1[still$id]*r2[still$id]*sel[still$id]*z1[still$id]
		    rrt <- rrt[tr==1]
		    tt11 <- rchaz(cumhaz,rrt,entry=(1-gap.time)*still$time[tr==1])
		    tt1[tr==1,] <- tt11
		}
	}
	tt <- tt1
	tt$tr <- tr; tt$sel <- sel; tt$timetr <- timetr; 
	  if (!is.null(cumhaz2))  {
              if (ztr==0)  tt2 <- rchaz(cumhaz2,rtr[still$id],entry=(1-gap.time)*still$time)
              if (ztr==1)  tt2 <- rchaz(cumhaz2,rtr[still$id]*z1[still$id],entry=(1-gap.time)*still$time)
	  ## modify to t2=tr  only when tr==0
          tt$status <- ifelse(tt2$time<=tt1$time & tr==0,2*tt2$status,tt1$status)
          tt$time <-   ifelse(tt2$time<=tt1$time & tr==0,tt2$time,tt1$time)
          tt$timetr <-   ifelse(tt2$time<=tt1$time & tr==0,tt2$time,timetr)
	  if (ztr==1) {
             ww <- (tt2$time<=tt1$time & tr==0)
             Base2TR <- cpred(cumhaz2,tt$time[ww])
             basesel <- r1[still$id][ww]*Base2TR[,2]
             tt$sel[ww] <- (1+var.z*basesel)/(1+var.z)
	  }
          tt$tr[tt2$time<=tt1$time & tr==0] <- 1
	  }
	  if (gap.time) {
		  tt$entry <- still$time
		  tt$time  <- tt$time+still$time
	  }
          ###
	  tt <- cbind(tt,dkeep(still,~id+dtime+death+fdeath),row.names=NULL)
	  tt <- dtransform(tt,death=fdeath,time>dtime)
	  tt <- dtransform(tt,status=0,time>dtime)
	  tt <- dtransform(tt,time=dtime,time>dtime)
	  nt <- nrow(tt)
	  tt <- tt[,colnames(tall)]
	  tall <- rbind(tall,tt[1:nn,],row.names=NULL)
  } 
  dsort(tall) <- ~id+entry+time
  tall$start <- tall$entry
  tall$stop  <- tall$time

  tall$statusD <- tall$status
  tall <- dtransform(tall,statusD=3,death==1)

  attr(tall,"death.cumhaz") <- cumhazd
  attr(tall,"cumhaz") <- cumhaz
  attr(tall,"cumhaz2") <- cumhaz2
  attr(tall,"z") <- z1

  return(tall)
  }# }}}

simFrail <- function(n,cumhaz,r1=NULL,rc=NULL,dependence=1,var.z=1,cens=NULL,...) 
{# {{{

 cor.mat <- death <- NULL

  if (dependence==0) { z <- z1 <- z2 <- zd <- rep(1,n) # {{{
     } else if (dependence==1) {
	      z <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- z; z2 <- z; zd <- z 
      } else if (dependence==2) {
              stdevs <- var.z^.5
              b <- stdevs %*% t(stdevs)  
              covv  <- b * cor.mat  
	      z <- matrix(rnorm(3*n),n,3)
	      z <- exp(z%*% chol(covv))
	      z1 <- z[,1]; z2 <- z[,2]; zd <- z[,3]; 
      } else if (dependence==3) {
	      z <- matrix(rgamma(3*n,1),n,3)
              z1 <- (z[,1]^cor.mat[1,1]+z[,2]^cor.mat[1,2]+z[,3]^cor.mat[1,3])
              z2 <- (z[,1]^cor.mat[2,1]+z[,2]^cor.mat[2,2]+z[,3]^cor.mat[2,3])
              zd <- (z[,1]^cor.mat[3,1]+z[,2]^cor.mat[3,2]+z[,3]^cor.mat[3,3])
	      z <- cbind(z1,z2,zd)
      } else if (dependence==4) {
	      zz <- rgamma(n,1/var.z[1])*var.z[1]
	      z1 <- zz; z2 <- zz; zd <- rep(1,n) 
	      z <- z1
      }      else stop("dependence 0-4"); # }}}

   if (is.null(r1)) r1 <- rep(1,n)
   if (is.null(rc)) rc <- rep(1,n)

  ll <- nrow(cumhaz)

 tall   <- rchaz(cumhaz,z1*r1)
  if (!is.null(cens)) { 
     ctime <- rexp(n)/(rc*cens)
     tall$status[tall$time>ctime] <- 0; 
     tall$time[tall$time>ctime] <- ctime[tall$time>ctime] 
  }
  tall$start <- tall$entry
  tall$stop  <- tall$time

  attr(tall,"z") <- z1

  return(tall)
  }# }}}

##' Pepe-Mori Test for Marginal Mean Comparison
##'
##' Performs score-test type tests for proportionality of marginal means in competing 
##' risks data and recurrent events, as presented in Ghosh and Lin (2000). The test 
##' is based on an IPCW (Inverse Probability of Censoring Weighting) formulation.
##' 
##' The function computes several tests:
##' \enumerate{
##'   \item \strong{Pepe-Mori Test}: Tests for equality of marginal mean functions between groups.
##'   \item \strong{Ratio of AUC}: Compares the area under the curve of marginal means.
##'   \item \strong{Difference of AUC}: Tests for difference in areas under the curve.
##'   \item \strong{Score Test}: Tests for proportionality (equivalent to Gray's test for CIF).
##'   \item \strong{Proportionality Test}: Tests the proportional hazards assumption.
##' }
##' 
##' The Pepe-Mori test uses weights based on the number at risk in each group to 
##' construct a weighted integral of the difference in marginal means.
##'
##' @param formula Formula with an \code{Event} object on the left-hand side and 
##'   covariates (typically with \code{strata()} for group comparison) on the right.
##'   Can include \code{cluster(id)} for correlated data.
##' @param data Data frame containing all variables referenced in the formula.
##' @param cause Cause of interest (default 1).
##' @param cens.code Censoring code (default 0).
##' @param death.code Code for death (terminating event, default 2).
##' @param death.code.prop Code for other causes of death for Fine-Gray regression model.
##' @param time Upper limit for Pepe-Mori and AUC integrals. If NULL, defaults to 
##'   the maximum event time for the cause of interest.
##' @param beta Starting values for the score test (default NULL, uses zeros).
##' @param ... Additional arguments passed to lower-level functions.
##' @return An object of class \code{"marginalTest"} containing:
##'   \item{pepe.mori}{Pepe-Mori test results with compare p-value.}
##'   \item{RatioAUC}{Ratio of AUC test results with compare p-value.}
##'   \item{difAUC}{Difference of AUC test results with compare p-value.}
##'   \item{prop.test}{Proportionality test results.}
##'   \item{score.test}{Score test results (equivalent to Gray's test).}
##'   \item{score.iid}{Influence function for the score test.}
##'   \item{time}{Upper time limit used.}
##'   \item{RAUCl, RAUCe}{Raw and transformed AUC estimates.}
##' @author Thomas Scheike
##' @references 
##' Ghosh, D. and Lin, D. Y. (2000). Nonparametric Analysis of Recurrent Events and Death. Biometrics, 56, 554--562.
##' @seealso \code{\link{test_logrankRecurrent}}, \code{\link{test_conc}}
##' @examples
##' data(bmt,package="mets")
##' bmt$time <- bmt$time+runif(nrow(bmt))*0.01
##' bmt$id <- 1:nrow(bmt)
##' dcut(bmt) <- age.f~age
##'      
##' fg=cifregFG(Event(time,cause)~tcell,data=bmt,cause=1)
##'  
##' ## computing tests for difference  for CIF
##' pmt <- test_marginalMean(Event(time,cause)~strata(tcell)+cluster(id),data=bmt,cause=1,
##' 			 death.code=1:2,death.code.prop=2,cens.code=0,time=40)
##' summary(pmt) 
##'  
##' pmt$pepe.mori
##' pmt$RatioAUC
##' pmt$prop.test
##' ## score test equialent to Gray's test but variance estimated differently 
##' pmt$score.test
##'  
##' ### age-groups  
##' pmt <- test_marginalMean(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1,
##' 			 death.code=1:2,death.code.prop=2,cens.code=0)
##' summary(pmt) 
##'  
##' ## having a look at the cumulative incidences 
##' cifs <- cif(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1)
##' plot(cifs) 
##'  
##' ## recurrent events   
##' data(hfactioncpx12)
##' hf <- hfactioncpx12
##' pmt <- test_marginalMean(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
##' 			 cause=1,death.code=2,cens.code=0)
##' summary(pmt) 
##'  
##' @export
test_marginalMean <- function(formula,data,cause=1,cens.code=0,...,death.code=2,death.code.prop=NULL,time=NULL,beta=NULL) { ## {{{ 
cl <- match.call()
###m <- match.call(expand.dots = TRUE)[1:3]
des <- proc_design(formula, data = data, specials = c("offset","weights","cluster","strata","marks"), intercept = FALSE)
Y <- des$y
if (!inherits(Y, c("Event", "Surv"))) {
stop("Expected a 'Surv' or 'Event'-object")
}
if (ncol(Y) == 2) {
exit <- Y[, 1]
entry <- rep(0, nrow(Y))
status <- Y[, 2]
}
else {
entry <- Y[, 1]
exit <- Y[, 2]
status <- Y[, 3]
}
X <- des$x
strata <- des$strata
specials = c("offset", "weights", "cluster", "strata")
Terms <- terms(formula, specials, data = data)
ts <- survival::untangle.specials(Terms, "strata")
if (!is.null(strata))
strata.name <- ts$vars
else strata.name <- NULL
pos.strata <- NULL
des.weights <- des$weights
des.offset <- des$offset
id <- des$cluster
pos.cluster <- NULL

if (is.null(death.code.prop)) death.code.prop <- death.code

data$strata__ <- strata
data$entry__ <- entry
data$exit__ <- exit
nstrata <- as.numeric(strata)

###drop.specials(formula,"strata")
formR <- as.formula(paste("Event(entry__,exit__,status__)~factor(strata__)+cluster(id__)"))
data$status__  <- status
data$id__  <- id

## proportionality test  
proptest <- recreg(formR,data,cause=cause,death.code=death.code.prop,cens.model=~strata(strata__))

nlev <- nlevels(strata)
if (is.null(beta)) beta <- rep(0,nlev-1)
## score test  for prop-factor 
proptest0 <- recreg(formR,data,cause=cause,death.code=death.code.prop,
		   cens.model=~strata(strata__),beta=beta,no.opt=TRUE)
gradient <- matrix(proptest0$gradient,length(proptest0$gradient),1)
iidbeta <-  suppressWarnings(IC(proptest0))
n <- nrow(iidbeta)
iidU0 <- iidbeta %*% proptest0$hessian
score.test <- suppressWarnings(
  summary(estimate(coef=gradient,IC=iidU0), null=0)
)

### varU0 <- crossprod(iidU0)
### logrank.robust <- t(gradient) %*% solve(varU0) %*% gradient
### p.logrank.robust <- 1-pchisq(logrank.robust,nrow(gradient))
### score.test <- list(logrank.robust=logrank.robust,p.logrank.robust=p.logrank.robust,
###		    test.statistic=gradient,iid=iidU0)

ev_times <- exit[status %in% cause]
if (length(ev_times) == 0) stop("No events of the specified cause found; cannot determine time.")
if (is.null(time)) time <- max(ev_times)

## {{{ pepe-mori test 

ddataid <- data.frame(id=id)
cid <- countID(ddataid)
###ns <- c(table(strata[cid$reverse==1]))

data$statusC__ <- 1*(status %in% cens.code) 
data$statusD__ <- 1*(status %in% death.code)
formD <- as.formula(paste("Event(entry__,exit__,statusD__)~+1"))

if (nlev==2) {
	formC <- as.formula(paste("Event(entry__,exit__,statusC__)~strata(strata__)"))
	kms <- km(formC,data)
	kms0 <- phreg(formC,data,no.opt=TRUE)
	kms0$jumptimes
	kms0$strata.jumps
        Yr <- vecAllStrata(kms0$S0,kms0$strata.jumps,kms0$nstrata)
	minRisk <- apply(Yr,1,min)

        if (is.null(time)) {
               kms0 <- phreg(formC,data,no.opt=TRUE)
	       kms0$jumptimes
	       kms0$strata.jumps
               Yr <- vecAllStrata(kms0$S0,kms0$strata.jumps,kms0$nstrata)
	       minRisk <- apply(Yr,1,min)
               time <- max(kms0$jumptimes[minRisk>=5])
        }

        ns <- c(table(strata[cid$reverse==1]))
	n1 <- ns[1]
	n2 <- ns[2]
	## using number at risk for weighting 
	kmss <- cbind(kms$time,t(kms$surv))
	wt <- (n1+n2)*kmss[,2]*kmss[,3]/(n1*kmss[,2]+n2*kmss[,3])
	wt[is.na(wt)] <- 0
	Wt <- cumsum(diff(c(0,kms$time))*wt)

	Wtmark <- lin_approx(pmin(exit,time),rbind(0,cbind(kms$time,Wt)))
	Wfinal <- tail(Wt,1)
	###
	data$pmmark__ <- Wfinal-Wtmark

	pepe.mori <- recregIPCW(formR,data,cause=cause,death.code=death.code,
	    times=time,marks=data$pmmark__,cens.model=~strata(strata__),model="lin")
	pepe.mori <- suppressWarnings(
      summary(estimate(pepe.mori), contrast=lava::contr(2))
    )
} else {

###  formRC <- as.formula(paste("Event(entry__,exit__,status__)~factor(cstrata__)+cluster(id__)"))
###	contr <- contr.iid <- c()
###        for (i in 1:(nlev-1)) {
###
###   	   data$cstrata__  <-  1*(nstrata==i)
###           formC <- as.formula(paste("Event(entry__,exit__,statusC__)~strata(cstrata__)"))
###
###           kms <- km(formC,data)
###           kmss <- cbind(kms$time,t(kms$surv))
###
###           ns <- c(table(data$cstrata__[cid$reverse==1]))
###	   n1 <- ns[1]
###	   n2 <- ns[2]
###	   ## using number at risk for weighting 
###	   kmss <- cbind(kms$time,t(kms$surv))
###	   wt <- (n1+n2)*kmss[,2]*kmss[,3]/(n1*kmss[,2]+n2*kmss[,3])
###	   wt[is.na(wt)] <- 0
###	   Wt <- cumsum(diff(c(0,kms$time))*wt)
###	   Wtmark <- lin_approx(pmin(exit,time),rbind(0,cbind(kms$time,Wt)))
###	   Wfinal <- tail(Wt,1)
###	   ###
###	   data$pmmark__ <- Wfinal-Wtmark
###
###	   pmOut <- recregIPCW(formRC,data,cause=cause,death.code=death.code,
###	       times=time,marks=data$pmmark__,cens.model=~strata(cstrata__),model="lin")
###
###        contr <- c(contr,coef(pmOut)[2])
###	contr.iid <- cbind(contr.iid,pmOut$iid[,2])
###   }
###   var.contr <- crossprod(contr.iid)
###   ###  pepe.mori <- estimate(coef=contr,vcov=var.contr,null=0)
###   pepe.mori <- estimate(coef=contr,IC=contr.iid*nrow(contr.iid),null=0)
###   p <- length(contr)
###   pepe.mori <- estimate(pepe.mori,lava::contr(1:p))
	pepe.mori <- list()
	pepe.mori$error <- "only for two levels"
        pepe.mori$compare$p.value  <-  NA
}

## }}}

## Ratio of AUC
data$pmmarkAUC__ <- time-exit
## linear model for stability when fitting
RAUCl<- recregIPCW(formR,data,cause=cause,death.code=death.code,
	  times=time,cens.model=~strata(strata__),marks=data$pmmarkAUC__,
	  model="lin")
f <- function(p) p[-1]/p[1]
## reparametrize as baseline on log-scale, and log-ratio contrasts 
RAUCe <- estimate(RAUCl,function(p) c(log(p[1]),log((p[1]+p[-1])/p[1])))
p <- length(coef(RAUCl))
RAUCet <- summary(estimate(RAUCe),contrast=as.list(2:p))
RAUClt <- summary(RAUCl, contrast=as.list(2:p))

proptest <- suppressWarnings(
  summary(estimate(proptest),null=0)
)

out <- list(pepe.mori=pepe.mori,RatioAUC=RAUCet,difAUC=RAUClt,
	    RAUCl=RAUCl,RAUCe=RAUCe,
	    score.test=score.test,score.iid=iidU0,prop.test=proptest,time=time)
class(out) <- "marginalTest"
return(out)
} ## }}} 

##' @export
print.marginalTest  <- function(x,...) {# {{{
  print(summary(x),...)
}# }}}

##' @export
summary.marginalTest <- function(object,...) {# {{{

p.values <- c(object$time,
        object$pepe.mori$compare$p.value,
        object$RatioAUC$compare$p.value,
        object$prop.test$compare$p.value,
        object$score.test$compare$p.value)
p.values <- matrix(p.values,ncol=1)
colnames(p.values) <- "p-value"
rownames(p.values) <- c("time","Pepe-Mori","Ratio-AUC","Proportionality","Proportionality-score-test")

res <- p.values
class(res) <- "summary.marginalTest"
return(res)
}# }}}

##' @export
print.summary.marginalTest <- function(x,...) { ## {{{
  cat("coeffients:\n")
  printCoefmat(x,...)
  cat("\n")
} ## }}}


## proc_design <- mets:::proc_design

##' CIF regression
##'
##' CIF logistic-link for propodds=1 default and CIF Fine-Gray (cloglog) regression for propodds=NULL. The
##' FG model can also be called using the cifregFG function that has propodds=NULL.
##'
##' For FG model:
##' \deqn{
##' \int (X - E ) Y_1(t) w(t) dM_1
##' }
##' is computed and summed over clusters and returned multiplied with inverse
##' of second derivative as iid.naive. Here \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))} and
##' \deqn{E(t) = S_1(t)/S_0(t)} and \deqn{S_j(t) = \sum X_i^j Y_{i1}(t) w_i(t) \exp(X_i^T \beta)}.
##'
##' The iid decomposition of the beta's, however, also have a censoring term that is also
##' is computed and added (still scaled with inverse second derivative)
##' \deqn{
##' \int (X - E ) Y_1(t) w(t) dM_1 + \int q(s)/p(s) dM_c
##' }
##' and returned as the iid 
##'
##' For logistic link standard errors are slightly to small since uncertainty from recursive baseline is not considered, so for smaller
##' data-sets it is recommended to use the prop.odds.subdist of timereg that is also more efficient due to use of different weights for
##' the estimating equations. Alternatively, one can also bootstrap the standard errors.
##'
##' @param formula formula with 'Event' outcome
##' @param data data frame
##' @param propodds to fit logit link model, and propodds=NULL to fit Fine-Gray model
##' @param cause of interest
##' @param cens.code code of censoring
##' @param no.codes certain event codes to be ignored when finding competing causes, can be used with administrative censoring.
##' @param death.code can also specify death.code (in addition to cause) to overrule default which takes all remaining codes (minus cause,cens.code,no.codes)
##' @param ... Additional arguments to recreg 
##' @author Thomas Scheike
##' @examples
##' ## data with no ties
##' library(mets)
##' data(bmt,package="mets")
##' bmt$time <- bmt$time+runif(nrow(bmt))*0.01
##' bmt$id <- 1:nrow(bmt)
##'
##' ## logistic link  OR interpretation
##' or=cifreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' summary(or)
##' par(mfrow=c(1,2))
##' plot(or)
##' nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
##' por <- predict(or,nd)
##' plot(por)
##'
##' ## approximate standard errors 
##' por <-mets:::predict.phreg(or,nd)
##' plot(por,se=1)
##'
##' ## Fine-Gray model
##' fg=cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' summary(fg)
##' ##fg=recreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1,death.code=2)
##' ##summary(fg)
##' plot(fg)
##' nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
##' pfg <- predict(fg,nd,se=1)
##' plot(pfg,se=1)
##' 
##' ## bt <- iidBaseline(fg,time=30)
##' ## bt <- IIDrecreg(fg$cox.prep,fg,time=30)
##' 
##' ## not run to avoid timing issues
##' ## gofFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' 
##' sfg <- cifregFG(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
##' summary(sfg)
##' plot(sfg)
##'
##' ### predictions with CI based on iid decomposition of baseline and beta
##' ### these are used in the predict function above
##' fg <- cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
##' Biid <- iidBaseline(fg,time=20)
##' pfg1 <- FGprediid(Biid,nd)
##' pfg1
##' @aliases vecAllStrata diffstrata FGprediid indexstratarightR gofFG cifregFG
##' @export
cifreg  <- function(formula,data,propodds=1,cause=1,cens.code=0,no.codes=NULL,death.code=NULL,...)
{# {{{
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset","strataAugment")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y, "Event")) stop("Expected a 'Event'-object, with codes for terminal events (death.code if any), censoring (cens.code), and event of interest (cause)")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }

    ## default version of codes, otherwise call recregN directly
    all.codes <-  unique(status)
    codes <- c(cause,cens.code) 
    if (!is.null(no.codes)) codes <- c(codes,no.codes) 
    mcodes <- match(codes,all.codes,nomatch=0)
    ## default death.code are all other codes 
    if (is.null(death.code)) death.code <- all.codes[-mcodes]

    cif <- recreg(formula,data,propodds=propodds,cause=cause,cens.code=cens.code,death.code=death.code,...)
    class(cif) <- c("cifreg","phreg")
    cif$call <- cl
return(cif)
} # }}}

##' @export
cifregFG  <- function(formula,data,propodds=NULL,...)
{# {{{
cif <- cifreg(formula,data,propodds=propodds,...)
return(cif)
} # }}}

##' @export
iidBaseline.cifreg <- function(object,time=NULL,...)
{# {{{
  out <- iidBaseline.recreg(object,time=time,...)
  return(out)
} # }}}

iidBaseline.recreg <- function(object,time=NULL,ft=NULL,fixbeta=NULL,beta.iid=object$iid,tminus=FALSE,...)
{ ## {{{
	if (is.null(object$cox.prep)) stop("must call cifreg/recreg with cox.prep=TRUE\n")
	out <- IIDrecreg(object$cox.prep,object,time=time,fixbeta=fixbeta,beta.iid=beta.iid,adm.cens=object$adm.cens,tminus=tminus,...)
	out$design <- object$design
	return(out)
}  ## }}}

##' @export
IC.cifreg <- function(x, time=NULL, ...) {# {{{
  if (!is.null(time)) {
    iid <- iidBaseline.recreg(x, time=time,...)
    res <- iid$base.iid
    attr(res, "strata.level") <- iid$strata.level
    attr(res, "coef") <- iid$cumhaz.time
    attr(res, "time") <- time
    return(res * NROW(res))
  }
  res <- with(x, iid * NROW(iid))
  return(res)
}
# }}}

##' @export
estimate.cifreg <- function(x, ..., time = NULL, baseline.args = list()) {
  if (NCOL(model.matrix(x))==0L & is.null(time)) stop("Non-parametric model; need `time` argument")
  ic <- do.call(IC, c(list(x, time = time), baseline.args))
  cc <- attr(ic, "coef")
  if (is.null(cc)) cc <- coef(x)
  lab <- names(cc)
  if (!is.null(time)) {
    lab <- x$strata.level
    if (is.null(lab)) lab <- "cif"
  }
  b <- lava::estimate(
      coef = cc, IC = ic, labels = lab,
      function(x) 1 - exp(-x)
  )
  return(lava::estimate(b, ...))
}

##' @export
plot.cifreg <- function(x,se=FALSE,ylab=NULL,...) { ## {{{
   if (inherits(x,"cifreg") & is.null(ylab)) ylab <- "Cumulative Hazard"
if (!se) baseplot(x,se=se,ylab=ylab,...)
else {
    warning("Standard errors approximative (but too small), use predict and type='cumhaz' \n")
    baseplot(x,se=se,ylab=ylab,...)
}
} ## }}}

##' @export
predict.cifreg <- function(object,newdata,se=FALSE,times=NULL,np=50,...) { ## {{{
if (!is.null(object$propodds) & se) {se <- FALSE; warning("standard errors not computed for logit link\n"); }
if (!se) out <- predict.phreg(object,newdata,se=se,times=times,...)
else {
  out <- predictrecreg(object,newdata,times=times,np=np,...)
}
class(out) <- c("predictcifreg",class(object)[1])
return(out)
} ## }}}

##' @export
summary.predictcifreg <- function(object,times=NULL,type=c("cif","cumhaz","surv")[1],...) {# {{{
ret <- summary.predictrecreg(object,type=type[1],times=times,...)
return(ret)
}# }}}

##' @export
plot.predictcifreg <- function(x,se=FALSE,ylab=NULL,type="cif",...)
{ ## {{{
  if (inherits(x,"predictcifreg") & is.null(ylab)) ylab <- "Probability"
  plotpredictphreg(x,se=se,ylab=ylab,type=type[1],...)
} ## }}}

##' @export
gofFG <- function(formula,data,cause=1,cens.code=0,cens.model=NULL,...)
{# {{{
fgform <- update(formula, paste("Surv(fgstart, fgstop, fgstatus) ~ .+cluster(id)"))
## assumes simple Surv(time,status) is given 
vars <- all.vars(formula)
data$id <- 1:nrow(data)
formid <- update.formula(formula,paste(".~.+id"))

## process types to get type of interst and other types 
status <- data[,vars[2]]
types <- unique(status)
mm <- match(c(cens.code,cause),types)
mm <- mm[!is.na(mm)]
statusS <- status
statusS[!(status %in% c(cens.code,cause))] <- types[-mm][1]
typesF <- c(cens.code,cause,types[-mm][1])
###
###data[,vars[2]] <- factor(statusS,typesF,c("censoring","cause","ocause"))
data[,vars[2]] <- factor(statusS,typesF,typesF)

Xs <- vars[-(1:2)]
modP <- paste(Xs,collapse="+") 
formid <- as.formula(paste("Surv(",vars[1],",",vars[2],")~",modP,"+id"))
if (!is.null(cens.model)) {
Cstrata <- as.character(cens.model)
formid <- as.formula(paste("Surv(",vars[1],",",vars[2],")~",modP,"+",Cstrata[2],"+id"))
}

fgdata <- finegray(formid,data=data)
fgcph <- phreg(fgform,data=fgdata,weights=fgdata$fgwt,...)
ggmg <- gof(fgcph)
return(ggmg)
}# }}}

##' @export
indexstratarightR <- function(timeo,stratao,jump,js,nstrata,type="right")# {{{
{
###    if (any(stratao < 0 | stratao >= nstrata))
###        stop("time-strata strata not between 0-(nstrata-1)\n")
###    if (any(js < 0 | js >= nstrata))
###        stop("jump-strata strata not between 0-(nstrata-1)\n")
    mm <- cbind(timeo, stratao, 1:length(timeo), 0)
    mm <- rbind(mm, cbind(jump, js, 1:length(jump), 1))
    ord <- order(mm[, 1], mm[, 4])
    mm <- mm[ord, ]
    if (type == "right") right <- 1 else right <- 0
    ires <- .Call("indexstrataR", mm[, 2], mm[, 3], mm[, 4], nstrata,right)
    res <- ires$res
    or2 <- order(res[,2])
    res <- res[or2,1]
    reso <- which(res==0) 
    if (length(reso)>0) {
       jso <- js[reso]
       for (s in 1:nstrata) if (length(jso)>0) res[reso[jso==s-1]] <- ires$maxmin[s] 
    }
    return(res)
} ## }}}

##' @export
FGprediid <- function(iidBase,newdata,conf.type=c("log","cloglog","plain"),model="FG")
{# {{{
  des <- readPhreg(iidBase,newdata)
  strata <- des$strata
  if (!is.null(iidBase$beta.iid))  { 
	  fixbeta <- 0; beta.iid <- iidBase$beta.iid; X <- des$X; p <- ncol(beta.iid); 
  } else { fixbeta <- 1; beta.iid <- 0; X <- matrix(0,1,1); p <- 1 }

  sus <- sort(unique(strata))
  At <- iidBase$cumhaz.time[match(sus,iidBase$strata.time)]

   if (missing(X)) X <- matrix(0,1,p)
   if (ncol(X)!=p) stop("X and coef does not match \n"); 

   Ft <- function(coef,Xi=rep(0,length(p)),type="log") {
	   base <- coef[1]
	   p <- coef[-1]
   if (model=="FG") {
       if (type=="log")     y <- log(1-exp(-base*exp(Xi %*% p)))
       if (type=="plain")   y <- 1-exp(-base*exp(Xi %*% p))
       if (type=="cloglog") y <- log(-log(1-exp(-base*exp(Xi %*% p))))
   } else { ## Ghosh-Lin model
       if (type=="log")     y <- log(base*exp(Xi %*% p))
       if (type=="plain")   y <- base*exp(Xi %*% p)
   }
       return(y)
   }
   Ftback <- function(p,type="log")
   {
       if (type=="log")     y <- exp(p)
       if (type=="plain")   y <- p
       if (type=="cloglog") y <- exp(-exp(p)) 
       return(y)
   }

   preds <- matrix(0,length(strata),4)

   k <- 0
   for (i in sus) {
      wheres <- which(strata==i) 
      wi <- match(i,iidBase$strata.time)
      At <- iidBase$cumhaz.time[wi]
	if (fixbeta==0)  {
	   iidAB <- cbind(iidBase$base.iid[,wi],iidBase$beta.iid)
	   covv <- crossprod(iidAB)
	   coeff <- c(At,iidBase$coef)
	   for (j in wheres)  {
	      Xj <- X[j,]
	      eud <- estimate(coef=coeff,vcov=covv,f=function(p) Ft(p,Xi=Xj,type=conf.type[1]))
	      cmat <- eud$coefmat
	      cmat <- c(cmat[,-5])
	      cicmat <- Ftback(cmat[c(1,3:4)],type=conf.type[1])
	      cmat[c(1,3:4)] <- cicmat
	      preds[j,] <- c(cmat)
	   } 
	} else {
	      iidAB <- cbind(iidBase$base.iid[,wi])
	      covv <- crossprod(iidAB)
	      coeff <- c(At)
	      eud <- estimate(coef=coeff,vcov=covv,f=function(p) Ft(p,Xi=1,type=conf.type[1]))
	      cmat <- eud$coefmat
	      cmat <- c(cmat[,-5])
	      cicmat <- Ftback(cmat[c(1,3:4)],type=conf.type[1])
	      cmat[c(1,3:4)] <- cicmat
	      preds[wheres,] <- matrix(c(cmat),length(wheres),4,byrow=TRUE)
	}
}

colnames(preds) <- c("pred",paste("se",conf.type[1],sep="-"),"lower","upper")

return(preds)
}# }}}

##' @export
strataC <- survival::strata

##' Augmentation for Fine-Gray model based on stratified NPMLE Cif (Aalen-Johansen)
##'
##' Computes  the augmentation term for each individual as well as the sum
##' \deqn{
##' A(\beta) = \int H(t,X,\beta) \frac{F_2^*(t,s)}{S^*(t,s)} \frac{1}{G_c(t)} dM_c
##' }
##' with
##' \deqn{
##' H(t,X,\beta) = \int_t^\infty (X - E(\beta,t) ) G_c(t) d\Lambda_1^*i(t,s)
##' }
##' using a KM for \deqn{G_c(t)} and a working model for cumulative baseline
##' related to \deqn{F_1^*(t,s)} and \deqn{s} is strata,
##' \deqn{S^*(t,s) = 1 - F_1^*(t,s) - F_2^*(t,s)}, and
##' \deqn{E(\beta^p,t)} is given. Assumes that no strata for baseline of ine-Gay model that is augmented. 
##'
##' After a couple of iterations we end up with a solution of
##' \deqn{
##' \int (X - E(\beta) ) Y_1(t) w(t) dM_1 + A(\beta)
##' }
##' the augmented FG-score.
##'
##' Standard errors computed under assumption of correct \deqn{G_c} model.
##'
##' @param formula formula with 'Event', strata model for CIF given by strata, and strataC specifies censoring strata
##' @param data data frame
##' @param offset offsets for cox model
##' @param data data frame
##' @param E from FG-model
##' @param cause of interest
##' @param cens.code code of censoring
##' @param km to use Kaplan-Meier
##' @param case.weights weights for FG score equations (that follow dN_1)
##' @param weights weights for FG score equations
##' @param offset offsets for FG   model
##' @param ... Additional arguments to lower level funtions
##' @author Thomas Scheike
##' @examples
##' library(mets)
##' set.seed(100)
##' rho1 <- 0.2; rho2 <- 10
##' n <- 100
##' beta=c(0.0,-0.1,-0.5,0.3)
##' dats <- simul.cifs(n,rho1,rho2,beta,rc=0.2)
##' dtable(dats,~status)
##' dsort(dats) <- ~time
##' fg <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1,propodds=NULL)
##' summary(fg)
##' plot(fg);  lines(attr(dats,"Lam1"),col=2)
##'
##' fgaugS <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fg$E)
##' summary(fgaugS)
##' fgaugS2 <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fgaugS$E)
##' summary(fgaugS2)
##'
##' @aliases strataC drop.strata
##' @export
FG_AugmentCifstrata <- function(formula,data=data,E=NULL,cause=NULL,cens.code=0,km=TRUE,case.weights=NULL,weights=NULL,offset=NULL,...)
{# {{{
    cl <- match.call()
    m <- match.call(expand.dots = TRUE)[1:3]
    special <- c("strata", "cluster","offset","strataC")
    Terms <- terms(formula, special, data = data)
    m$formula <- Terms
    m[[1]] <- as.name("model.frame")
    m <- eval(m, parent.frame())
    Y <- model.extract(m, "response")
    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
    if (ncol(Y)==2) {
        exit <- Y[,1]
        entry <- NULL ## rep(0,nrow(Y))
        status <- Y[,2]
    } else {
        entry <- Y[,1]
        exit <- Y[,2]
        status <- Y[,3]
    }
    id <- strata <- NULL
    if (!is.null(attributes(Terms)$specials$cluster)) {
        ts <- survival::untangle.specials(Terms, "cluster")
        Terms  <- Terms[-ts$terms]
        id <- m[[ts$vars]]
    }
    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
        ts <- survival::untangle.specials(Terms, "strata")
        Terms  <- Terms[-ts$terms]
        strata <- m[[ts$vars]]
        strata.name <- ts$vars
    }  else strata.name <- NULL
    if (!is.null(stratapos <- attributes(Terms)$specials$strataC)) {
        ts <- survival::untangle.specials(Terms, "strataC")
        Terms  <- Terms[-ts$terms]
        strataC <- as.numeric(m[[ts$vars]])-1
        strataC.name <- ts$vars
    }  else { strataC <- NULL; strataC.name <- NULL}

    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
        ts <- survival::untangle.specials(Terms, "offset")
        Terms  <- Terms[-ts$terms]
        offset <- m[[ts$vars]]
    }
    X <- model.matrix(Terms, m)
    if (!is.null(intpos  <- attributes(Terms)$intercept))
        X <- X[,-intpos,drop=FALSE]
    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
    id.orig <- id;
    if (!is.null(id)) {
        ids <- sort(unique(id))
        nid <- length(ids)
        if (is.numeric(id))
            id <-  fast.approx(ids,id)-1
        else  {
            id <- as.integer(factor(id,labels=seq(nid)))-1
        }
    } else id <- as.integer(seq_along(exit))-1;

    p <- ncol(X)
    beta <- NULL
    if (is.null(beta)) beta <- rep(0,p)
    if (p==0) X <- cbind(rep(0,length(exit)))
    if (is.null(strata)) {
        strata <- rep(0,length(exit));
        nstrata <- 1;
        strata.level <- NULL; }
    else {
            strata.level <- levels(strata)
            ustrata <- sort(unique(strata))
            nstrata <- length(ustrata)
            strata.values <- ustrata
            if (is.numeric(strata))
                strata <-  fast.approx(ustrata,strata)-1
            else  {
                strata <- as.integer(factor(strata,labels=seq(nstrata)))-1
            }
        }

    if (is.null(strataC)) {
        strataC <- rep(0,length(exit))
        nstrataC <- 1
        strataC.level <- NULL
    } else {
        strataC.level <- levels(strataC)
        ustrataC <- sort(unique(strataC))
        nstrataC <- length(ustrataC)
        strataC.values <- ustrataC
        if (is.numeric(strataC))
            strataC <-  fast.approx(ustrataC,strataC)-1
        else  {
            strataC <- as.integer(factor(strataC,labels=seq(nstrataC)))-1
        }
    }

    cens.strata <- strataC
    cens.nstrata <- nstrataC

    if (is.null(offset)) offset <- rep(0,length(exit))
    if (is.null(weights)) weights <- rep(1,length(exit))
    strata.call <- strata
    Z <- NULL
    Zcall <- matrix(1,1,1) ## to not use for ZX products when Z is not given
    if (!is.null(Z)) Zcall <- Z

    ## possible casewights to use for bootstrapping and other things
    if (is.null(case.weights)) case.weights <- rep(1,length(exit))

    trunc <- (!is.null(entry))
    if (!trunc) entry <- rep(0,length(exit))

  call.id <- id;
  conid <- construct_id(id,nrow(X),as.data=TRUE)
  name.id <- conid$name.id; id <- conid$id; nid <- conid$nid

    statusC <- (status==cens.code)
    statusE <- (status==cause)
    if (sum(statusE)==0) stop("No events of type 1 before time \n");


    ## sorting after time and statusC, but event times unique after order
    Zcall <- cbind(status,strata)
    dd <- .Call("FastCoxPrepStrata",entry,exit,statusC,X,id,
                trunc,strataC,weights,offset,Zcall,case.weights,PACKAGE="mets")

    Z <- dd$X
    jumps <- dd$jumps+1
    xxstrataC <- c(dd$strata)
    xxstatus  <- dd$Z[,1]
    xxstrata  <- dd$Z[,2]
    other <- which((!(xxstatus %in% c(cens.code,cause)) ) )
    jumps1 <- which(xxstatus %in% cause)
    jumpsD <- which(!(xxstatus %in% cens.code))
    rr <- c(dd$sign*exp(dd$offset))
    ## S0 after strata
    S0 = c(revcumsumstrata(rr,xxstrata,nstrata))
    ## S0 after strataC
    S00C = c(revcumsumstrata(rr,xxstrataC,nstrataC))

    ## censoring MG, strataC
    stratJumps <- dd$strata[jumps]
    S00i <- rep(0,length(dd$strata))
    S00i[jumps] <-  1/S00C[jumps]

    ## cif calculation, uses strata {{{
    S0Di <- S02i <- S01i <- rep(0,length(dd$strata))
    S01i[jumps1] <-  1/S0[jumps1]
    S02i[other] <-  1/S0[other]
    S0Di[jumpsD] <-  1/S0[jumpsD]

    ## strata-Cif G_T(t)
    if (!km) {
        cumhazD <- cumsumstratasum(S0Di,xxstrata,nstrata)
        Stm <- exp(-cumhazD$lagsum)
        St <- exp(-cumhazD$sum)
    } else {
        StA <- cumsumstratasum(log(1-S0Di),xxstrata,nstrata)
        Stm <- exp(StA$lagsum)
        St <- exp(StA$sum)
    }

    ## G_c(t-)
    if (!km) {
        cumhazD <- cumsumstratasum(S00i,xxstrataC,nstrataC)
        Gc      <- exp(-c(cumhazD$sum))
        Gcm      <- exp(-c(cumhazD$lagsum))
    } else { 
        logGc <- cumsumstratasum(log(1-S00i),xxstrataC,nstrataC)
	Gcm <- c(exp(logGc$lagsum))
	Gc  <- c(exp(logGc$sum))
    }

    cif1 <- cumsumstrata(Stm*S01i,xxstrata,nstrata)
    cif2 <- cumsumstrata(Stm*S02i,xxstrata,nstrata)
    ## }}}

    Et <- matrix(0,nrow(Z),ncol(Z))
    Et[jumps1,] <- E

    Lam1fg <- -log(1-cif1)
    ## to deal with cif1=1 in which case cif2=0
    Lam1fg[is.na(Lam1fg)] <- 0
    laststrata <- tailstrata(xxstrata,nstrata)
    gtstart <- Lam1fg[laststrata]
    dLam1fg <- c(diffstrata(Lam1fg,xxstrata,nstrata))

    ## Gc~strataC, dLam1fg~strata
    tailcstrata <- tailstrata(xxstrataC,nstrataC)
    Gcstart <- Gc[tailcstrata]

    dstrata <- mystrata2index(cbind(xxstrataC,xxstrata))
    ndstrata <- attr(dstrata,"nlevel")
    lastt <- tailstrata(dstrata-1,ndstrata)

    ### ## \int_t^\infty G_c^j(t) d\Lambda_1^k(t)
    G0start <- rep(1,nstrataC)
    cLam1fg  <- cumsum2strata(Gc,dLam1fg,xxstrataC,nstrataC,xxstrata,nstrata,G0start)
    RLam1fg <- cLam1fg$res[lastt][dstrata]-cLam1fg$res

    ## E(s) from FG without strata
    ## \int_0^t  G_c^j(s) E(s) d\Lambda_1^k(s)
    fff <- function(x) {
        cx  <- cumsum2strata(Gc,x*dLam1fg,xxstrataC,nstrataC,xxstrata,nstrata,G0start)
        return(cx$res[lastt][dstrata]-cx$res)
    }
    ERLam1fg0  <- apply(Et,2,fff)

    cif2GS <-  c(cif2/(Gc*St))
    cif2GS[St<0.00001] <- 0
    cif2GS[Gc<0.00001] <- 0
    gt <-  RLam1fg*cif2GS
    ERLam1fg  <- ERLam1fg0*cif2GS

    sss <- headstrata(dstrata-1,ndstrata)
    gtstart <- gt[sss]
    E1dLam0 <- cumsum2strata(gt,S00i,dstrata-1,ndstrata,xxstrataC,nstrataC,gtstart)$res

    fff <- function(x) {
        gtstart <- x[sss]
        cx  <- cumsum2strata(x,S00i,dstrata-1,ndstrata,xxstrataC,nstrataC,gtstart)$res
        return(cx)
    }
    E2dLam0 <- apply(ERLam1fg,2,fff)

    U1 <- matrix(0,nrow(Z),1)
    U2 <- matrix(0,nrow(Z),ncol(Z))
    U1[jumps,] <- gt[jumps]
    U2[jumps,] <- ERLam1fg[jumps,]

    ### Martingale  as a function of time and for all subjects to handle strata
    MG1t <- Z*c(U1[,,drop=FALSE]-E1dLam0)*rr*c(dd$weights)
    MG2t <- (U2[,,drop=FALSE]-E2dLam0)*rr*c(dd$weights)
    MGt <- MG1t-MG2t
    MGiid <- apply(MGt,2,sumstrata,dd$id,max(dd$id)+1)
    augment <- apply(MGt,2,sum)

    augment <- list(MGiid=MGiid,augment=augment,id=id,id.orig=id.orig,
                    jumps1=jumps1,jumps=jumps,other=other,
                    nstrata=nstrata,nstrataC=nstrataC,dstrata=dstrata,ndstrata=ndstrata,
                    cif1=cif1,cif2=cif2,St=St,Gc=Gc,strata=xxstrata,strataC=xxstrataC,time=dd$time)

    ## drop strata's from formula and run wiht augmention term
    formulans <- drop.strata(formula)

    if (nstrataC==1) cens.model <- ~+1 else cens.model <- ~strata(strataCC)
    data$strataCC <- cens.strata

    fga <- tryCatch(cifreg(formulans,data=data,cause=cause,
                  propodds=NULL,augmentation=augment$augment,cens.model=cens.model,...),error=function(x) NULL) 

    if (!is.null(fga)) {
    ## adjust SE and var based on augmentation term
    fga$var.orig <- fga$var
    fga$augment <- augment$augment
    fga$iid <- fga$iid.naive + MGiid %*% fga$ihessian
    fga$var <- crossprod(fga$iid)
    fga$se.coef <-  diag(fga$var)^.5
    fga$MGciid <- MGiid
    } else  {
    fga$augment <- augment$augment
    fga$MGciid <- MGiid
    }

    return(fga)
}# }}})

#' Event history object
#'
#' Constructur for Event History objects
#'
#' ... content for details
#'
#' @aliases Event as.character.Event as.matrix.Event [.Event format.Event
#' print.Event rbind.Event summary.Event
#' @param time Time
#' @param time2 Time 2
#' @param cause Cause
#' @param cens.code Censoring code (default 0)
#' @param ... Additional arguments
#' @return Object of class Event (a matrix)
#' @author Klaus K. Holst and Thomas Scheike
#' @examples
#'
#' 	t1 <- 1:10
#' 	t2 <- t1+runif(10)
#' 	ca <- rbinom(10,2,0.4)
#' 	(x <- Event(t1,t2,ca))
#' @export
Event <- function(time,time2=TRUE,cause=NULL,cens.code=0,...) {# {{{
  if (missing(cause)) {
    if (is.factor(time2) | is.character(time2)) {
      stop("cause should be numeric\n")
    }
  }
  else { if (is.factor(cause) | is.character(cause))
           stop("cause should be numeric\n");  }
  out <- cbind(time, time2, cause)
  if (any(is.na(out))) warning("missing values in Event object\n")
  if (!missing(cause)) {
    colnames(out) <- c("entry", "exit", "cause")
    tmp <- (out[,1]>out[,2])
    if (any(tmp)) warning("entry time later than exit time\n")
  } else {
    colnames(out) <- c("exit", "cause")
  }
  class(out) <- "Event"
  attr(out, "cens.code") <- cens.code
  return(out)
}
# }}}

# exported in zzz.R
#' @exportS3Method NULL
as.matrix.Event <- function(x, ...) structure(x, class="matrix")

# exported in zzz.R
#' @exportS3Method NULL
as.data.frame.Event <- function(x, ...) {
  as.data.frame.model.matrix(x, ...)
}

# exported in zzz.R
#' @exportS3Method NULL
as.character.Event <- function(x, ...) {
  if (ncol(x) == 3) {
    res <- paste(
        "(", format(x[, 1], ...), ";",
        format(x[, 2], ...), ":",
        format(x[, 3], ...), "]",
        sep = ""
    )
  } else {
    res <- paste(format(x[, 1], ...), ":", format(x[, 2], ...), sep = "")
  }
  return(res)
}

# exported in zzz.R
#' @exportS3Method NULL
format.Event <- function(x, ...) format(as.character.Event(x), ...)

# exported in zzz.R
#' @exportS3Method NULL
is.na.Event <- function(x) {
    apply(x, 1, function(r) any(is.na(r)))
}

# exported in zzz.R
#' @exportS3Method NULL
print.Event <- function(x, ...) {
    print(as.character(x), ..., quote=FALSE)
}

# exported in zzz.R
#' @exportS3Method NULL
summary.Event <- function(object,...) {
  cat(paste("cens.code=",attr(object,"cens.code"),"\n"))
  cat("causes:\n")
  print(table(object[,"cause"]))
  cat("exit:\n")
  print(summary(object[,"exit"]))
  if (ncol(object)==3) {
    cat("entry:\n")
    print(summary(object[,"entry"]))
    cat("exit-entry:\n")
    print(summary(object[,"exit"]- object[,"entry"]))
  }
}

# exported in zzz.R
#' @exportS3Method NULL
"[.Event" <- function (x, i, j, drop = FALSE) {
  if (missing(j)) {
    atr <- attributes(x)
    class(x) <- "matrix"
    x <- x[i, , drop = FALSE]
    class(x) <- "Event"
    atr.keep <- c("cens.code", "entry")
    attributes(x)[atr.keep] <- atr[atr.keep]
    x
  }
  else {
    class(x) <- "matrix"
    NextMethod("[")
  }
}

# exported in zzz.R
#' @exportS3Method NULL
rbind.Event <- function(...) {
    dots <- list(...)
    cens.code <- attributes(dots[[1]])$cens.code
    type <- attributes(dots[[1]])$type
    ncol <- dim(dots[[1]])[2]
    nrow <- unlist(lapply(dots, nrow))
    cnrow <- c(0, cumsum(nrow))
    M <- matrix(ncol = ncol, nrow = sum(nrow))
    for (i in seq_len(length(dots))) {
        M[(cnrow[i] + 1):cnrow[i + 1], ] <- dots[[i]]
    }
    x <- c()
    for (i in seq_len(ncol(M))) x <- c(x, list(M[, i]))
    x <- c(x, list(cens.code = cens.code))
    do.call("Event", x)
}

# exported in zzz.R
#' @exportS3Method NULL
length.Event <- function(x) nrow(x)

# exported in zzz.R
#' @exportS3Method NULL
c.Event <- function(...) {
    objects <- list(...)
    if (!all(unlist(lapply(objects, function(x) inherits(x, "Event")))) &&
        all.equal(unlist(lapply(objects, NCOL)))) {
        stop("All elements should be `Event` of the same type")
    }
    Reduce(rbind, objects)
}

##' Percentage of years lost due to cause regression 
##'
##' Estimates the percentage of the years lost that is due to a cause and how covariates affects this percentage by doing ICPW regression.
##'
##' Let the years lost be  \deqn{Y1= t- min(T ,) } and the years lost due to cause 1 \deqn{Y2= I(epsilon==1) ( t- min(T ,t) } , then
##' we model the ratio \deqn{logit( E(Y2 | X)/E(Y1 | X))  = X^T \beta }. Estimation is based on 
##' on binomial regresion IPCW response estimating equation: 
##' \deqn{ X ( \Delta^{ipcw}(t) Y2 expit(X^T \beta) -  Y1 ) = 0 }
##' where \deqn{\Delta^{ipcw}(t) = I((min(t,T)< C)/G_c(min(t,T)-)} is 
##' IPCW adjustment of the response \deqn{Y(t)= I(T \leq t, \epsilon=1 )}.  
##'
##' (type="I") sovlves this estimating equation using a stratified Kaplan-Meier for the
##' censoring distribution. For (type="II") the default an additional 
##' censoring augmentation term \deqn{X \int E(Y(t)| T>s)/G_c(s) d \hat M_c} is added.
##'
##' The variance is based on the squared influence functions that are also returned as the iid component. naive.var is variance 
##' under known censoring model. 
##'
##' Censoring model may depend on strata (cens.model=~strata(gX)). 
##'
##' @param formula formula with outcome (see \code{coxph})
##' @param data data frame
##' @param cause cause of interest (numeric variable)
##' @param time  time of interest 
##' @param beta starting values 
##' @param type "II" adds augmentation term, and "I" classical outcome IPCW regression 
##' @param offset offsets for partial likelihood 
##' @param weights for score equations 
##' @param cens.weights censoring weights 
##' @param cens.model only stratified cox model without covariates
##' @param se to compute se's  based on IPCW 
##' @param kaplan.meier uses Kaplan-Meier for IPCW in contrast to exp(-Baseline)
##' @param cens.code gives censoring code
##' @param no.opt to not optimize 
##' @param method for optimization 
##' @param augmentation to augment binomial regression 
##' @param outcome  can do CIF regression "cif"=F(t|X), "rmtl"=E( t- min(T, t) | X)"
##' @param model  logit, exp or lin(ear) 
##' @param Ydirect use this Y instead of outcome constructed inside the program, should be a matrix with two column for numerator and denominator.
##' @param ... Additional arguments to lower level funtions
##' @references 
##' Scheike & Tanaka (2025), Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause, WIP
##' @author Thomas Scheike
##' @examples
##' library(mets)
##' data(bmt); bmt$time <- bmt$time+runif(408)*0.001
##' 
##' rmst30 <- rmstIPCW(Event(time,cause!=0)~platelet+tcell+age,bmt,time=30,cause=1)
##' rmst301 <- rmstIPCW(Event(time,cause)~platelet+tcell+age,bmt,time=30,cause=1)
##' rmst302 <- rmstIPCW(Event(time,cause)~platelet+tcell+age,bmt,time=30,cause=2)
##' 
##' estimate(rmst30)
##' estimate(rmst301)
##' estimate(rmst302)
##' 
##' ## percentage of total cumulative incidence due to cause 1
##' rmtlratioI <- rmtlRatio(Event(time,cause)~platelet+tcell+age,bmt,time=30,cause=1)
##' summary(rmtlratioI)
##' 
##' pp <- predict(rmtlratioI,bmt[1:5,])
##' pp
##' 
##' newdata <- data.frame(platelet=1,tcell=1,age=1)
##' ## percentage of total cumulative incidence due to cause 1
##' cifratio <- binregRatio(Event(time,cause)~platelet+tcell+age,bmt,time=30,cause=1)
##' summary(cifratio)
##' pp <- predict(cifratio,newdata)
##' pp
##' 
##' rmtlratioI <- binregRatio(Event(time,cause)~platelet+tcell+age,bmt,
##'                                time=30,cause=1,outcome="rmtl")
##' summary(rmtlratioI)
##' 
##' pp <- predict(rmtlratioI,newdata)
##' pp
##' @aliases rmtlRatio
##' @export
binregRatio <- function(formula,data,cause=1,time=NULL,beta=NULL,type=c("II","I"),
	   offset=NULL,weights=NULL,cens.weights=NULL,cens.model=~+1,se=TRUE,
	   kaplan.meier=TRUE,cens.code=0,no.opt=FALSE,method="nr",augmentation=NULL,
	   outcome=c("cif","rmtl"),model=c("logit","exp","lin"),Ydirect=NULL,...)
{# {{{
  cl <- match.call()# {{{
    m <- match.call(expand.dots = TRUE)[1:3]
    des <- proc_design(
        formula, data = data, specials = c("offset", "weights", "cluster"),
        intercept = TRUE
    )
    Y <- des$y
    if (!inherits(Y, c("Event", "Surv"))) {
        stop("Expected a 'Surv' or 'Event'-object")
    }
    if (ncol(Y) == 2) {
        exit <- Y[, 1]
        entry <- rep(0, nrow(Y))
        status <- Y[, 2]
    } else {
        entry <- Y[, 1]
        exit <- Y[, 2]
        status <- Y[, 3]
    }
    X <- des$x
    des.weights <- des$weights
    des.offset  <- des$offset
    id      <- des$cluster

 call.id <- id;
 conid <- construct_id(id,nrow(X))
 name.id <- conid$name.id; id <- conid$id; nid <- conid$nid
 orig.id <- id

  ## take offset and weight first from formula, but then from arguments
  if (is.null(des.offset)) {
	  if (is.null(offset)) offset <- rep(0,length(exit)) 
  } else offset <- des.offset
  if (is.null(des.weights)) {
	  if (is.null(weights)) weights <- rep(1,length(exit)) 
  } else weights <- des.weights
# }}}

  if (is.null(time)) stop("Must give time for logistic modelling \n"); 
  statusC <- 1*(status %in% cens.code) 
  statusE <- (status %in% cause) & (exit<= time) 
  if (sum(statusE)==0) warning("No events of type 1 before time \n"); 
  kmt <- kaplan.meier

  ucauses  <-  sort(unique(status))
  ccc <- which(ucauses %in% cens.code)
  if (length(ccc)==0) Causes <- ucauses else Causes <- ucauses[-ccc]
  competing  <-  (length(Causes)>1) 
  data$id__ <- id
  data$exit <- exit
  data$statusC <- statusC 
  cens.strata <- cens.nstrata <- NULL 

 nevent <- sum((status %in% cause)*(exit<=time))
 ## if event before time or alive, then uncensored, equality for both censored and events  
 obs <- (exit<=time & (!statusC)) | (exit>=time)

  if (is.null(cens.weights))  {
      formC <- update.formula(cens.model,Surv(exit,statusC)~ . +cluster(id__))
      resC <- phreg(formC,data)
      if (resC$p>0) kmt <- FALSE
      exittime <- pmin(exit,time)
      cens.weights <- suppressWarnings(predict(resC,data,times=exittime,individual.time=TRUE,se=FALSE,km=kmt,tminus=TRUE)$surv)
      ## strata from original data 
      cens.strata <- resC$strata[order(resC$ord)]
      cens.nstrata <- resC$nstrata
  } else { se <- FALSE; resC <- formC <- NULL}
  expit  <- function(z) 1/(1+exp(-z)) ## expit

  p <- ncol(X)
  if (is.null(beta)) beta <- rep(0,p)
  if (is.null(augmentation))  augmentation=rep(0,p)
  X <-  as.matrix(X)
  X2  <- .Call("vecCPMat",X)$XX

 if (!is.null(Ydirect)) Y <-  Ydirect*obs/cens.weights else {
     if (outcome[1]=="cif") Y <- cbind( c((status %in% Causes)*(exit<=time)/cens.weights) , c((status %in% cause)*(exit<=time)/cens.weights) )
     else { 
            Y <- cbind(c((time-pmin(exit,time))*obs)/cens.weights, c((status %in% cause)*(time-pmin(exit,time))*obs)/cens.weights)
     }
  }
  Yipcw <- Y

obj <- function(pp,all=FALSE)
{ # {{{
lp <- c(X %*% pp+offset)

if (model[1]=="logit") {
	p <- expit(lp)
        D2logl <- c(Y[,1]*weights*p/(1+exp(lp)))*X2
} else if (model[1]=="exp") {
	p <- exp(lp)
        D2logl <- c(Y[,1]*p*weights)*X2
} else {
	p <- lp
        D2logl <- c(Y[,1]*weights)*X2
}
Dlogl <- weights*X*c(Y[,1]*p-Y[,2])
D2log <- apply(D2logl,2,sum)
ploglik <- 0   ###sum(weights*(Y[,1]*p-Y[,2])^2)

gradient <- apply(Dlogl,2,sum)+augmentation
np <- length(pp)
hessian <- matrix(.Call("XXMatFULL",matrix(D2log,nrow=1),np,PACKAGE="mets")$XXf,np,np)

  if (all) {
      ploglik <- sum(weights*(Y[,1]*p-Y[,2])^2)
      ihess <- solve(hessian)
      beta.iid <- Dlogl %*% ihess ## %*% t(Dlogl) 
      beta.iid <-  apply(beta.iid,2,sumstrata,id,max(id)+1)
      robvar <- crossprod(beta.iid)
      val <- list(par=pp,ploglik=ploglik,gradient=gradient,hessian=hessian,ihessian=ihess,
	 id=id,Dlogl=Dlogl,iid=beta.iid,robvar=robvar,var=robvar,se.robust=diag(robvar)^.5)
      return(val)
  }  
 structure(0,gradient=-gradient/nid,hessian=-hessian/nid)
}# }}}

 if (model[1]=="exp") control <- list(stepsize=0.5)  else control <- NULL

  

  ## first run without pseudo-value augmentation
  p <- ncol(X)
  opt <- NULL
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(beta,obj,control=control))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
  } else val <- c(list(coef=beta),obj(beta,all=TRUE))

 coefI <- val$coef
 gradientI <- val$gradient
 hessianI <- val$hessian
 ihessianI <- val$ihessian
 iidI <- val$iid

 if (se) {## {{{ censoring adjustment of variance 
    ### order of sorted times
    ord <- resC$ord
    X <-  X[ord,,drop=FALSE]
    X2 <-  X2[ord,,drop=FALSE]
    status <- status[ord]
    exit <- exit[ord]
    weights <- weights[ord]
    offset <- offset[ord]
    cens.weights <- cens.weights[ord]
    Y <- Y[ord,]
    lp <- c(X %*% val$coef+offset)
    p <- expit(lp)
    Yo <- Y[,1]*p-Y[,2]
    id <- id[ord]

    xx <- resC$cox.prep
    S0i2 <- S0i <- rep(0,length(xx$strata))
    S0i[xx$jumps+1]  <- 1/resC$S0
    S0i2[xx$jumps+1] <- 1/resC$S0^2
    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
    ## to make \int h(s)/Ys  dM_i^C(s) 
    h  <-  apply(X*Yo,2,revcumsumstrata,xx$strata,xx$nstrata)
    ### Cens-Martingale as a function of time and for all subjects to handle strata 
    btime <- 1*(exit<time)
    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
    IhdLam0 <- apply(h*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*h[xx$jumps+1,]/c(resC$S0)
    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)

    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
    mid <- max(id)
    MGCiidI <- MGCiid <- apply(MGt,2,sumstrata,xx$id,mid+1)

   if (type[1]=="II") { ##  pseudo-value type augmentation
    hYt  <-  revcumsumstrata(Yo,xx$strata,xx$nstrata)
    IhdLam0 <- cumsumstrata(hYt*S0i2*btime,xx$strata,xx$nstrata)
    U <- rep(0,length(xx$strata))
    U[xx$jumps+1] <- (resC$jumptimes<time)*hYt[xx$jumps+1]/c(resC$S0)
    MGt <- X*c(U-IhdLam0)*c(xx$weights)
    MGtiid <- apply(MGt,2,sumstrata,xx$id,mid+1)
    augmentation  <-  apply(MGtiid,2,sum) + augmentation
    ###
    EXt  <-  apply(X,2,revcumsumstrata,xx$strata,xx$nstrata)
    IEXhYtdLam0 <- apply(EXt*c(hYt)*S0i*S0i2*btime,2,cumsumstrata,xx$strata,xx$nstrata)
    U <- matrix(0,nrow(xx$X),ncol(X))
    U[xx$jumps+1,] <- (resC$jumptimes<time)*hYt[xx$jumps+1]*EXt[xx$jumps+1,]/c(resC$S0)^2
    MGt2 <- (U[,drop=FALSE]-IEXhYtdLam0)*c(xx$weights)
    ###
    MGCiid2 <- apply(MGt2,2,sumstrata,xx$id,mid+1)
    ### Censoring Variance Adjustment 
    MGCiid <- MGCiid+(MGtiid-MGCiid2)
   }
   ## use data ordered by time (keeping track of id also)
   ## since X; Y and so forth are ordered in time
###   id <- xx$id
   }  else {
	 MGCiidI <-  MGCiid <- 0
  }## }}}

  ## first run without pseudo-value augmentation, then run with augmentation, if type="II"
 if (type[1]=="II") {
  if (no.opt==FALSE) {
      if (tolower(method)=="nr") {
	  tim <- system.time(opt <- lava::NR(coefI,obj,...))
	  opt$timing <- tim
	  opt$estimate <- opt$par
      } else {
	  opt <- nlm(obj,beta,...)
	  opt$method <- "nlm"
      }
      cc <- opt$estimate; 
###	      if (!se) return(cc)
      val <- c(list(coef=cc),obj(opt$estimate,all=TRUE))
      } else val <- c(list(coef=beta),obj(beta,all=TRUE))
 }

 if (length(val$coef)==length(colnames(X))) names(val$coef) <- colnames(X)
  val <- c(val,list(time=time,formula=formula,formC=formC,
    exit=exit, cens.weights=cens.weights, cens.strata=cens.strata, cens.nstrata=cens.nstrata, 
    n=length(exit),nevent=nevent,ncluster=nid))


  val$call <- cl
  val$MGciid <- MGCiid
  val$id <- orig.id
  val$call.id <- call.id
  val$name.id <- name.id
  val$nid <- nid
  val$iid.naive <- val$iid
  val$naive.var <- NULL 
  val$iidI <- iidI
  if (se) val$iid  <- val$iid+(MGCiid %*% val$ihessian)
  if (se) val$iidI  <- iidI+(MGCiidI %*% ihessianI)
  if (!is.null(call.id)) {
  val$iid <- nameme(val$iid,name.id)
  val$iidI <- nameme(val$iidI,name.id)
  }
  robvar <- crossprod(val$iid)
  val$var <-  val$robvar <- robvar
  val$se.robust <- diag(robvar)^.5
  val$se.coef <- diag(val$var)^.5
  val$cause <- cause
  val$cens.code <- cens.code 
  val$augmentation <- augmentation
  val$model <- model[1]
  val$outcome <- outcome[1]
  val$Yipcw <- Yipcw
  val$Causes <- Causes
  val$nevent <- nevent

  val$coefI <- coefI
  val$varI <- crossprod(val$iidI)
  val$se.coefI <- diag(val$varI)^.5
  val$gradientI <-  gradientI
  val$hessianI <- hessianI
  val$ihessianI <- ihessianI
  val$design <- des

  class(val) <- "binreg"
  return(val)
}# }}}

##' @export
rmtlRatio <- function(formula,data,outcome=c("rmtl"),...)
{# {{{
   out <- binregRatio(formula,data,outcome=outcome[1],...)
   return(out)
}# }}}

context("Censored regression (loglikMVN)")

cens1 <- function(threshold,type='right') {
  function(x) {
    x <- unlist(x)
    if (type=='left')
      return( survival::Surv(pmax(x,threshold), x>=threshold, type='left') )
      return ( survival::Surv(pmin(x,threshold), x<=threshold) )
  }
}

m0 <- lvm() |>
  covariance(y1 ~ y2, value='r') |>
  regression(y1 + y2 ~ x)
m0 <-  transform(m0, s1 ~ y1, cens1(-2, 'left')) |>
  transform(s2 ~ y2, cens1(2,  'right'))

test_that("phreg.iid", {
  d <- sim(m0, 500, p=c(r=0.9), seed=1)

  m3 <- lvm() |>
    regression(y1 + s2 ~ x) |>
    covariance(y1 ~ s2, constrain=TRUE, rname='z')

  e3 <- estimate(m3, d)
  expect_true(mean(score(e3)**2)<1e-2)
})

---
title: "GEE cluster standard errors and predictions for glm objects" 
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{GEE cluster standard errors and predictions for glm objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Utility functions for GLM objects
=================================

Computing odds ratios with confidence intervals using GEE (sandwich)
standard errors:


``` r
set.seed(100)

library(mets)
data(bmt); 
bmt$id <- sample(1:100,408,replace=TRUE)

glm1 <- glm(tcell~platelet+age,bmt,family=binomial)
summaryGLM(glm1)
#> $coef
#>             Estimate Std.Err    2.5%   97.5%   P-value
#> (Intercept)  -2.4371  0.2225 -2.8732 -2.0009 6.481e-28
#> platelet      1.1368  0.3076  0.5340  1.7397 2.189e-04
#> age           0.5927  0.1551  0.2888  0.8966 1.319e-04
#> 
#> $or
#>               Estimate       2.5%     97.5%
#> (Intercept) 0.08741654 0.05651794 0.1352076
#> platelet    3.11688928 1.70573194 5.6955015
#> age         1.80895115 1.33489115 2.4513641
#> 
#> $fout
#> NULL

## GEE robust standard errors
summaryGLM(glm1,id=bmt$id)
#> $coef
#>             Estimate Std.Err    2.5%   97.5%   P-value
#> (Intercept)  -2.4371  0.2157 -2.8599 -2.0142 1.361e-29
#> platelet      1.1368  0.2830  0.5822  1.6914 5.877e-05
#> age           0.5927  0.1434  0.3117  0.8738 3.568e-05
#> 
#> $or
#>               Estimate       2.5%     97.5%
#> (Intercept) 0.08741654 0.05727471 0.1334211
#> platelet    3.11688928 1.79006045 5.4271903
#> age         1.80895115 1.36575550 2.3959664
#> 
#> $fout
#> NULL
```

Predictions are equally straightforward:


``` r
age <- seq(-2,2,by=0.1)
nd <- data.frame(platelet=0,age=seq(-2,2,by=0.1))
pnd <- predictGLM(glm1,nd)
head(pnd$pred)
#>      Estimate       2.5%      97.5%
#> p1 0.02601899 0.01115243 0.05951051
#> p2 0.02756409 0.01214068 0.06136414
#> p3 0.02919819 0.01321187 0.06328733
#> p4 0.03092608 0.01437206 0.06528441
#> p5 0.03275278 0.01562757 0.06736019
#> p6 0.03468351 0.01698493 0.06952008
plot(age,pnd$pred[,1],type="l",ylab="predictions",xlab="age",ylim=c(0,0.3))
plotConfRegion(age,pnd$pred[,2:3],col=2)
```

<img src="figure/glm-utility-unnamed-chunk-3-1.png" alt="GLM predicted probabilities with confidence region."  />

``` r
###matlines(age,pnd$pred[,-1],col=2,lty=1)
```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Cooking Survival Data: 5-Minute Recipes"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
    toc: true
header-includes: 
  - \usepackage{tikz}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Cooking Survival Data: 5-Minute Recipes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



# Overview 

Simulation of survival data is important for both
theoretical and practical work. In a practical setting we might wish to
validate that standard errors are valid even in a rather small sample,
or validate that a complicated procedure is doing as intended.
Therefore it is useful to have simple tools for generating survival data
that resembles a particular dataset as closely as possible. In a theoretical
setting we are often interested in evaluating the finite-sample
properties of a new procedure in different settings that are
motivated by a specific practical problem. The aim is to provide such tools.

Bender et al. discussed how to generate survival data based on the Cox model,
and considered only a subset of the available parametric survival models (Weibull, exponential).
We start by restricting attention to piecewise linear baseline functions, which make it easy to simulate
data that follows closely the baseline estimated from the data using semi- or
nonparametric models. This makes it straightforward to capture important aspects of the
data. We later return to extended Weibull baseline hazard models.

Different survival models can be cooked, and we here give recipes for
hazard and cumulative incidence based simulations. More recipes are
given in vignette about recurrent events. 

  - hazard based. 
    - Cox Regression  models.
       - piecewise constant baselines 
       - parametric Weibull models
  - cumulative incidence. 
    - Regression models with cloglog or logistic link. 
  - recurrent events (see recurrent events vignette).
    - Regression models with exp link, for rates or Ghosh-Lin type.



``` r
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations
```

# Hazard based, Cox models  


Given a survival time $T$ with cumulative hazard $\Lambda(t)=\int_0^t \lambda(s) ds$, it
follows that, with $E \sim Exp(1)$ (exponential with rate 1), $\Lambda^{-1}(E)$ will have the
same distribution as $T$.

This provides the basis for simulations of survival times with a given hazard and is
a consequence of this simple calculation
$$
  P(\Lambda^{-1}(E) > t) = P(E > \Lambda(t)) = \exp( - \Lambda(t)) = P(T > t).
$$

Similarly if $T$ given $X$ has hazard on Cox form
$$
  \lambda_0(t) \exp( X^T \beta)
$$
where $\beta$ is a $p$-dimensional regression coefficient and $\lambda_0(t)$ a baseline
hazard function,
then it is useful to observe also that
$\Lambda^{-1}(E/HR)$ with $HR=\exp(X^T \beta)$ has the same distribution as $T$ given $X$.

Therefore, if the inverse of the cumulative hazard can be computed, we can generate survival times with
a specified hazard function. One useful observation is that for a piecewise linear continuous
cumulative hazard on an interval $[0,\tau]$, $\Lambda_l(t)$, it is easy to compute the inverse.

Further, any cumulative hazard can be approximated by a piecewise linear continuous
cumulative hazard, enabling simulation from this approximation. Recall that fitting
the Cox model to data yields a piecewise constant cumulative hazard and regression coefficients;
with these in hand we can approximate the piecewise constant Breslow estimator
by simply connecting the jump points with straight lines.

The engine is to simulate data with a given linear cumulative hazard. First generating survival data based on
the cumulative hazard `cumhaz`:


``` r
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)
```

Now we fit a simple Cox model


``` r
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim_phreg(cox1,n,data=bmt)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1 
#> 529 471
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
#>                [,1]       [,2]
#> tcell    -0.6517920 -0.4564152
#> platelet -0.5207454 -0.5113844
#> age       0.4083098  0.3860139
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-4-1.png" alt="Cumulative hazard: simulated vs fitted Cox model."  />

``` r

 ## modify regression coefficients 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim_phreg(cox10,n,data=bmt)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1 
#> 427 573
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
#>          [,1]       [,2]
#> tcell     0.0 0.05615982
#> platelet  0.4 0.34930321
#> age       0.3 0.42496872
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-4-2.png" alt="Cumulative hazard: simulated vs fitted Cox model."  />

Multiple Cox models for cause-specific hazards can be combined. Below we draw the
covariates manually; alternatively, `sim_phregs` draws covariates directly from
the data.


``` r
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-5-1.png" alt="Cumulative hazard: competing risks Cox models, two causes."  />

``` r
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
#>                [,1]       [,2]       [,3]       [,4]
#> tcell    -0.4232606 -0.3727007  0.3991068  0.8167564
#> platelet -0.5654438 -0.5834273 -0.2461474 -0.3190683
```

We now consider a fully nonparametric model with stratified baselines.


``` r
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
#> 
#>     diabetes   0   1
#> chf                 
#> 0            223  16
#> 1            230  31
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim_phreg(coxs,nsim,data=NULL,strata=strata)
 simb$strata  <- strata

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim_phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)
```

<img src="figure/cooking-survival-data-unnamed-chunk-6-1.png" alt="Cumulative hazard: stratified nonparametric model."  />

We now fit cause-specific hazard models with three causes (treating censoring as one of them)
and generate competing risks data with hazards taken from the fitted Cox models.
The following example includes stratified baselines for some of the models:


``` r
 ## competing risks with phreg 
 cox0 <- phreg(Surv(time,cause==0)~tcell+platelet,data=bmt)
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
 coxs <- list(cox0,cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)

 ## verify cause-specific hazards match fitted model; increase n for better agreement
 scox0 <- phreg(Surv(time,cause==1)~tcell+platelet,data=dd)
 scox1 <- phreg(Surv(time,cause==2)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==3)~strata(tcell)+platelet,data=dd)
 cbind(cox0$coef,scox0$coef)
#>               [,1]      [,2]
#> tcell    0.1912407 0.3053846
#> platelet 0.1563789 0.2586927
 cbind(cox1$coef,scox1$coef)
#>                [,1]       [,2]
#> tcell    -0.4232606 -0.6611783
#> platelet -0.5654438 -0.4566101
 cbind(cox2$coef,scox2$coef)
#>                [,1]       [,2]
#> platelet -0.2271912 -0.1857664
 par(mfrow=c(1,3))
 plot(cox0); plot(scox0,add=TRUE,col=2); 
 plot(cox1); plot(scox1,add=TRUE,col=2); 
 plot(cox2); plot(scox2,add=TRUE,col=2); 
```

<img src="figure/cooking-survival-data-stratified-1.png" alt="Cumulative hazard: stratified competing risks models."  />

``` r
 
 ########################################
 ## second example 
 ########################################

 cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
 coxs <- list(cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)
 cbind(cox1$coef,scox1$coef)
#>                [,1]       [,2]
#> platelet -0.5658612 -0.6949669
 cbind(cox2$coef,scox2$coef)
#>            [,1]      [,2]
#> tcell 0.4153706 0.2554993
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE); 
 plot(cox2); plot(scox2,add=TRUE); 
```

<img src="figure/cooking-survival-data-stratified-2.png" alt="Cumulative hazard: stratified competing risks models."  />

One further example, again fully nonparametric:


``` r
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim_phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1   2   3 
#>   1 397 241 361
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,cause==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
#>           [,1]        [,2]
#> bmi -0.1353399 -0.04444255
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-7-1.png" alt="Cumulative hazard: fully nonparametric competing risks."  />

**Summary of simulation functions**

| Function      | Purpose                                                     |
|---------------|-------------------------------------------------------------|
| `sim_phreg`   | Single `phreg` model; supports stratified baselines         |
| `sim_phregs`  | List of cause-specific `phreg` models; draws covariates automatically |


# Delayed entry

If $T$ given $X$ have hazard  on Cox form 
$$
  \lambda_0(t) \exp( X^T \beta)
$$
and we wish to generate data according to this hazard for those that are alive at time $s$, that is 
draw from the distribution of $T$ given $T>s$ (all given $X$ ), then we note that  
$$
\Lambda_0^{-1}( \Lambda_0(s) + E/HR)) 
$$
with $HR=\exp(X^T \beta))$ and with $E \sim Exp(1)$ has the distribution we are after. 

This is again a consequence of a simple calculation
$$
P_X\!\left(\Lambda^{-1}\!\left(\Lambda(s) + E/HR\right) > t\right)
= P_X\!\left(E > HR\!\left(\Lambda(t) - \Lambda(s)\right)\right)
= P_X(T > t \mid T > s).
$$

We here illustrate how to simulate from a competing risks model based on 
Cox hazards. 


``` r
data(bmt)
cox0 <- phreg(Surv(time, cause == 0) ~ tcell + platelet+age,          data = bmt)
cox1 <- phreg(Surv(time, cause == 1) ~ tcell + platelet+age,          data = bmt)
cox2 <- phreg(Surv(time, cause == 2) ~ strata(tcell) + platelet+age,  data = bmt)

nsim <- 800
entry <- rbinom(nsim, 1, 0.5) * runif(nsim) * 60

dd <- sim_phreg(cox0,nsim, data = bmt,entry=entry)
scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
cbind(cox0$coef, scox0$coef)
#>                [,1]       [,2]
#> tcell    0.09091888 0.11554247
#> platelet 0.15733517 0.09307867
#> age      0.10230287 0.11156111
par(mfrow = c(2, 2))
plot(cox0); plot(scox0, add = TRUE, col = 2)

dd <- sim_phregs(list(cox0, cox1, cox2), nsim, data = bmt,entry=entry)

scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
scox1 <- phreg(Surv(entry,time, cause == 2) ~ tcell + platelet+age,         data = dd)
scox2 <- phreg(Surv(entry,time, cause == 3) ~ strata(tcell) + platelet+age, data = dd)

cbind(cox0$coef, scox0$coef)
#>                [,1]         [,2]
#> tcell    0.09091888  0.006447759
#> platelet 0.15733517  0.074976667
#> age      0.10230287 -0.042215864
cbind(cox1$coef, scox1$coef)
#>                [,1]       [,2]
#> tcell    -0.6517920 -0.6996524
#> platelet -0.5207454 -0.3419084
#> age       0.4083098  0.5513612
cbind(cox2$coef, scox2$coef)
#>                [,1]        [,2]
#> platelet -0.2178543 -0.20179035
#> age       0.1306184  0.02950711

plot(cox0); plot(scox0, add = TRUE, col = 2)
plot(cox1); plot(scox1, add = TRUE, col = 2)
plot(cox2); plot(scox2, add = TRUE, col = 2)
```

<img src="figure/cooking-survival-data-entry-stratified-1.png" alt="Cumulative hazard: delayed entry competing risks."  />

# Parametric hazard models
While the semi‑parametric Cox model provides substantial flexibility for
simulating survival data, there are situations where a fully parametric
simulation model is convenient or preferable. Here we consider a Weibull model
 parametrized so that the cumulative hazard is given by $$\Lambda(t) = \lambda
 \cdot t^s$$ where $s$ is the **shape parameter**, and $\lambda$ the **rate
 parameter**. We allow regression on both parameters
 \begin{align*} \lambda :=
 \exp(\beta^\top X), \quad s := \exp(\gamma^\top Z) \end{align*}
  where $X$ and
 $Z$ are covariate vectors. Specifically, this opens up for exploring
 non‑proportional hazards when $s$ depends on covariates.

Revisiting the TRACE data example we can compare the predictions from the Cox
and the Weibull-Cox model stratified by `chf` and with a proportional hazard
effect of `age` 

``` r
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw
#> 
#> - Weibull-Cox model -
#> 
#> Call:
#> phreg_weibull(formula = Surv(time, status > 0) ~ chf + age, shape.formula = ~chf, 
#>     data = sTRACE)
#> 
#> log-Likelihood: -684.750499 
#> 
#>    n events obs.time
#>  500    264 2228.481
#> 
#>               Estimate  Std.Err     2.5%   97.5%   P-value
#> (Intercept)   -5.59626 0.465886 -6.50938 -4.6831 3.070e-33
#> chf            0.83250 0.197629  0.44516  1.2198 2.526e-05
#> age            0.05331 0.006165  0.04123  0.0654 5.241e-18
#> ─────────────                                             
#> s:(Intercept) -0.44096 0.116740 -0.66977 -0.2122 1.585e-04
#> s:chf         -0.11794 0.133078 -0.37877  0.1429 3.755e-01

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)
```

<img src="figure/cooking-survival-data-weibull1-1.png" alt="Cumulative hazard: Weibull vs Cox stratified model."  />

To simulate data we can use the `rweibullcox()` function. Note that the
`stats::rweibull()` function gives a different parametrization where the
cumulative hazard is given by $H(t) = (t/b)^s$, i.e., with the same scale
parameter but where the scale parameter $b$ is related to the rate parameter we consider by $r := b^{-s}$.



``` r
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)
#>            [,1]       [,2]
#> X4522 -1.641818 -0.4409608
#> X1337 -1.576030 -0.4409608
#> X4490 -3.052267 -0.4409608
#> X5363 -2.003547 -0.4409608
#> X4518 -2.093627 -0.5589006
#> X1354 -1.154411 -0.5589006

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)
#>       chf    age     time status
#> X4522   0 74.174 4.104843   TRUE
#> X1337   0 75.408 4.239002   TRUE
#> X4490   0 47.718 8.000000  FALSE
#> X5363   0 67.389 1.402283   TRUE
#> X4518   1 50.084 1.246696   TRUE
#> X1354   1 67.701 6.829409  FALSE

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)
#> 
#> - Weibull-Cox model -
#> 
#> Call:
#> phreg_weibull(formula = Surv(time, status) ~ chf + age, shape.formula = ~chf, 
#>     data = newd)
#> 
#> log-Likelihood: -6692.027131 
#> 
#>     n events obs.time
#>  5000   2613 21578.83
#> 
#>               Estimate  Std.Err     2.5%    97.5%    P-value
#> (Intercept)   -5.70070 0.157299 -6.00900 -5.39240 1.365e-287
#> chf            0.86612 0.059267  0.74996  0.98229  2.290e-48
#> age            0.05453 0.002144  0.05033  0.05873 9.532e-143
#> ─────────────                                               
#> s:(Intercept) -0.42290 0.033445 -0.48845 -0.35735  1.199e-36
#> s:chf         -0.13715 0.039317 -0.21421 -0.06009  4.859e-04
```

All these steps are wrapped in the `simulate` method:

``` r
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)
#>         no wmi status chf    age sex diabetes     time vf
#> X6167 6167 1.8   TRUE   0 54.418   1        0 1.329640  0
#> X3847 3847 1.6  FALSE   1 66.119   0        0 7.635526  1
#> X695   695 0.8   TRUE   1 74.377   1        1 4.424567  0
#> X1599 1599 2.0   TRUE   0 74.986   0        0 3.600678  0
#> X6045 6045 2.0   TRUE   0 79.997   0        0 3.737537  0
```


# Multistate models: The Illness Death model 

Using hazard-based simulation with delayed entry we can simulate data
from the general illness-death model. The cumulative hazards for each transition
must be specified.

We specify

 - $\Lambda_{12}(t)$ the cumulative hazard for $1 \rightarrow 2$ transitions
 - $\Lambda_{21}(t)$ the cumulative hazard for $2 \rightarrow 1$ transitions
 - $\Lambda_{13}(t)$ the cumulative hazard for $1 \rightarrow 3$ transitions
 - $\Lambda_{23}(t)$ the cumulative hazard for $2 \rightarrow 3$ transitions

Transitions are then generated using these hazards. Covariate effects can be included
via proportional hazards models by supplying hazard ratios for all components,
as well as an exponential censoring model. Dependence between transitions can be introduced via:

 - `dependence=0`: independence
 - `dependence=1`: all hazards share a common gamma-distributed frailty (variance `var.z`)


\begin{tikzpicture}[
    >=Stealth,
    node distance=4cm,
    state/.style={
        rectangle,
        draw=black,
        thick,
        minimum width=3cm,
        minimum height=1cm,
        align=center
    }
]

  % Top states
  \node[state] (H) {Healthy \\ (1)};
  \node[state] (I) [right=of H] {Ill \\ (2)};

  % Dead centered below
  \node[state] (D) at ($(H)!0.5!(I) + (0,-3)$) {Dead \\ (3)};

  % Two straight arrows between Healthy and Ill
  \draw[->, thick] 
    ($(H.east)+(0,0.15)$) -- ($(I.west)+(0,0.15)$)
    node[midway, above] {$\lambda_{12}$};

  \draw[->, thick] 
    ($(I.west)+(0,-0.15)$) -- ($(H.east)+(0,-0.15)$)
    node[midway, below] {$\lambda_{21}$};

  % Death transitions
  \draw[->, thick] (H) -- node[left] {$\lambda_{13}$} (D);
  \draw[->, thick] (I) -- node[right] {$\lambda_{23}$} (D);

\end{tikzpicture}

We pass the cumulative hazards for each transition to `simMultistate`
to simulate data from the model, then re-estimate the parameters on the simulated
data to validate the procedure.



``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- sim_multistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
#> id: 1
#>   entry     time status rr death from to start     stop
#> 1     0 201.9469      3  1     1    1  3     0 201.9469
#> ------------------------------------------------------------ 
#> id: 2
#>        entry     time status rr death from to    start     stop
#> 2     0.0000 395.3879      2  1     0    1  2   0.0000 395.3879
#> 801 395.3879 555.8022      3  1     1    2  3 395.3879 555.8022
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-9-1.png" alt="Multistate model: estimated vs true transition rates."  />


# Cumulative incidence 

In this section we discuss how to simulate competing risks data with a specified cumulative
incidence function. We consider for simplicity a competing risks model with two causes and denote the
cumulative incidence functions as $F_1(t,X) = P(T < t, \epsilon=1|X)$ and $F_2(t,X) = P(T < t, \epsilon=2|X)$,
given covariate $X$.

To generate data with the required cumulative incidence functions, a simple approach is to first determine
whether the subject experiences an event and, if so, from which cause; then draw the event time according to the
conditional distribution.

For simplicity we consider survival times in a fixed interval $[0,\tau]$. Given $X$:

  - first, flip a three-sided coin with probabilities $F_1(\tau,X)$, $F_2(\tau,X)$, $1-F_1(\tau,X)-F_2(\tau,X)$ to decide whether the subject survives or
     experiences one of the two causes.
  - second, draw the event time using the cumulative incidence distribution. The timing of a cause $j$ event is
       $T = \tilde F_j^{-1}(U,X)$ with $\tilde F_j(s,X) = F_j(s,X)/F_j(\tau,X)$ and $U$ uniform.

Then indeed $P(T \leq t, \epsilon=j|X) = F_j(t,X)$ for $j=1,2$.

We again note and use that if $\tilde F_j(s)$ and $F_j(s)$ are piecewise linear
continuous functions then the inverse is easy to compute. 

A couple of details worth noting:

 - the coin flip to determine cause is based on an underlying uniform, $pU$, which can be supplied and shared across subjects
     to generate dependence in the risk.
 - the uniform used for generating the timing, $U$, can also be supplied and shared across subjects to
    generate dependence in the timing.


## Cumulative incidence I

Here we simulate two causes of death with two binary covariates using a logistic link
\begin{align*}
F_1(t,X)  &= \frac{ \Lambda_1(t,\rho_1) exp(X^T  \beta_1)}{1+\Lambda_1(t,\rho_1) exp(X^T  \beta_1)}
\end{align*}
and $F_2$ here enforcing the sum condition $F_1+F_2 \leq 1$
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)} [ 1- F_1(\tau,X) ]
\end{align*}
or without the constraint
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)}.
\end{align*}
When the sum condition is not enforced through the construction, then 
it is enforced ad-hoc when drawing the cause of death. 

The baselines are given as $\Lambda_j(t) = \rho_1 (1- exp(-t/r_j))$ where $\rho_j$ 
and $r_j$ are positive constants, and here $\tau=6$. 

To simulate the survival time we use a piecewise linear approximation of the cumulative
incidence functions and will thus depends on some grid for linear approximation. Our linear 
approximation can be made arbitrarily close to any specific smooth cumulative incidence function.

The function `simul_cifs`

 - uses a time-scale $[0,6]$, which can obviously be rescaled.
 - takes regression coefficients as a single vector, with $\beta_1$ followed by $\beta_2$.
 - defaults to two binary covariates, but a covariate matrix $Z$ can be supplied (dimensions must match the coefficient vector).
 - uses exponential censoring with rate `rc=0.5`, which can be made covariate-dependent (`dependence=1`) with covariate effects given by
   `rcZ`, giving rate $rc \cdot \exp(Z^T rcZ)$ (dimensions must match). Censoring can be disabled by setting `rc=NULL`.


The function `sim_cifs` takes the output from `cifregFG` or `cifreg` and simulates using the
baselines and covariate effects stored in those objects.



``` r
library(mets)
nsim <- 400
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
#> 
#>    n events
#>  400     74
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.45731  0.13528  0.12180  0.0007
#> Z2 -0.21636  0.26451  0.23259  0.4134
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.57982 1.21187 2.0595
#> Z2  0.80545 0.47961 1.3527
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
#> 
#>    n events
#>  400     83
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.27811  0.11066  0.11175  0.0120
#> Z2 -0.57140  0.22567  0.22874  0.0113
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.32063 1.06314 1.6405
#> Z2  0.56474 0.36288 0.8789
plot(ciff)
lines(attr(dats,"Lam1"))
```

<img src="figure/cooking-survival-data-unnamed-chunk-10-1.png" alt="Cumulative incidence: logistic and cloglog link models."  />

We can also use the parameters based on fitted models 


``` r
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
#>             [,1]       [,2]
#> tcell -0.7966259 -0.9373498
#> age    0.4164286  0.4779627
 cbind(cif2$coef,scif2$coef)
#>              [,1]       [,2]
#> tcell  0.66687029  0.7791722
#> age   -0.03248846 -0.3603559
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-11-1.png" alt="Cumulative incidence: fitted model vs simulated data."  />


## CIF Delayed entry

Now assume that given covariates $F_1(t;X) = P(T < t, \epsilon=1|X)$ and  $F_2(t;X) = P(T < t, \epsilon=2|X)$ are two 
cumulative incidence functions that satisfies the needed constraints. We wish to generate data that follows these two
piecewise linear cumulative incidence functions with delayed entry at time $s$.  Given delayed entry at time $s$ we 
should thus generate data that follows the cumulative incidence functions
$$
\tilde F_1(t,s;X)=   \frac{F_1(t;X) - F_1(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
and 
$$
\tilde F_2(t,s;X)=   \frac{F_2(t;X) - F_2(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
This can be done according to the recipe in the previous section.

 - First draw event type with conditional probabilities $\tilde F_j(t,sX)$ for $j=1,2$ and the remaining survivors
 - Second draw event time (timing) of chosen type with distribution  (still conditional on being a survivor at entry)
   \begin{align*}
  \frac{\tilde F_j(t,sX)}{\tilde F_j(\tau,X)} & = \frac{F_j(t,X)- F_j(s,X)}{F_j(\tau,X)-F_j(s,X)} \mbox{for} j=1,2.
   \end{align*}

If only $F_1$ is specified, the function assumes a pure survival setting with $F_2 \equiv 0$. Note also that 
given event type the timing is unaffected by the truncation probability.

For the cloglog link the Fine-Gray model the timing can be drawn as 
\begin{align*}
 \Lambda_0^{-1}[ -\log(1-U ( F_j(\tau,X)-F_j(s,X)) - F_j(s,X) ] \exp(- X^T \beta)
\end{align*}
and for the logit-link as 
\begin{align*}
 \Lambda_0^{-1}[ \exp(logit( U ( F_j(\tau,X)-F_j(s,X)) + F_j(s,X)))] \exp(- X^T \beta)
\end{align*}



``` r
 data(bmt)
 ## two cloglog 
 cif1 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=1)
 cif2 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=2)

 nsim <- 800
 entry <- rbinom(nsim,1,0.5)*runif(nsim)*60
 dd <- sim_cif(cif1,nsim,data=bmt,entry=entry)

 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-12-1.png" alt="Cumulative incidence: single cause with delayed entry."  />

Combining two causes drawn with restriction $F_1+\tilde F_2 \leq 1$, thus modifying $\tilde F_2= F_2 (1-F_1(\tau))$ where 
$F_1$ and $F_2$ are the two input cumulative incidence models 


``` r
 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt,entry=entry)

 ## logistic link; nonparametric Aalen-Johansen with delayed entry
 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-13-1.png" alt="Cumulative incidence: two causes with delayed entry."  />

Here we again combine two causes, now using parametric baselines via `simul_cifs`.
The baselines for $F_1$ and $F_2$ are returned as attributes; note that $F_2$ is modified
to satisfy the constraint $F_1 + F_2 \leq 1$ (remember that due to the restriction the $F_2$ model is modified).



``` r
rho1 <- 0.3; rho2 <- 4
set.seed(100)
beta=c(0.3,-0.3,-0.3,0.3)
dep=0
rc <- 0.9

n <- nsim
entry <- rbinom(n,1,0.5)*runif(n)*6
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7),entry=entry) 
###
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,ylim=c(0,0.4))
###

## and without delayed entry for comparison
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7)) 
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,add=TRUE)

## true baseline cif, cloglog link 
baset <- attr(data,"Lam1")[,2]
timet <- attr(data,"Lam1")[,1]
F1base <- 1-exp(-baset)
lines(timet,F1base,lwd=3)
```

<img src="figure/cooking-survival-data-unnamed-chunk-14-1.png" alt="Cumulative incidence: parametric baselines with delayed entry."  />


# Recurrent events

See also recurrent events vignette

 - `sim_recurrent_ts` simulates from the Two-Stage model where:
    - Rate of the terminal event among survivors is on Cox form (`phreg`)
    - Rate of recurrent events among survivors is on Cox form (`phreg`)
    - Marginal rate of recurrent events follows a Ghosh-Lin model (`recreg`)
    - Simulations are based on piecewise linear approximations on a grid
    - Events can be dependent via a gamma-distributed frailty
  - `sim_recurrentII`, `sim_recurrent`, `sim_recurrent_list`:
    - Frailty gamma models where the rates of recurrent events and the terminal event are specified via
       cumulative baselines and relative risk covariate effects, yielding a Cox model given the frailty and covariates
    - `simRecurrentList` supports multiple recurrent event types and multiple causes of death


Two-stage models

The following example fits Cox models for recurrent events and the terminal event on the `hfactioncpx12` dataset,
then simulates data from the estimated two-stage model and re-estimates to verify recovery of the parameters.


``` r
library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 200

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
#>            Estimate Std.Err   2.5%    97.5% P-value
#> treatment1  -0.1534 0.08145 -0.313 0.006286 0.05973
 estimate(dr)
#>            Estimate Std.Err    2.5%    97.5% P-value
#> treatment1  -0.4301  0.1831 -0.7889 -0.07132  0.0188

 simcoxcox <- sim_recurrent_ts(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
#>            Estimate Std.Err    2.5%     97.5% P-value
#> treatment1  -0.3901  0.1969 -0.7762 -0.004138 0.04759
 estimate(drs)
#>            Estimate Std.Err    2.5%  97.5% P-value
#> treatment1  -0.2158  0.2827 -0.7698 0.3382  0.4453

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-15-1.png" alt="Recurrent events: two-stage Cox model."  />

Now with Ghosh-Lin and Cox marginals:


``` r
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment1  -0.1104 0.07866 -0.2646 0.04376  0.1604
 estimate(dr)
#>            Estimate Std.Err    2.5%    97.5% P-value
#> treatment1  -0.4301  0.1831 -0.7889 -0.07132  0.0188

 simglcox <- sim_recurrent_ts(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(entry,time,status)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
#>            Estimate Std.Err    2.5%  97.5% P-value
#> treatment1 -0.05645  0.1593 -0.3687 0.2558   0.723
 estimate(drs)
#>            Estimate Std.Err   2.5%   97.5% P-value
#> treatment1  -0.7905  0.3202 -1.418 -0.1629 0.01355

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
 plot(drs)
 plot(dr,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-16-1.png" alt="Recurrent events: Ghosh-Lin and Cox model."  />

We can also fit and simulate from stratified models:


``` r
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)
hf$age <- rnorm(741)[hf$id]
hf$Z1 <- rbinom(741,1,0.5)[hf$id]

xr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400 
rr <- sim_recurrent_ts(xr,dr,n=n,data=hf)
rxr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=rr)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age   0.0273 0.03922 -0.04958 0.1042  0.4864
estimate(rxr)
#>      Estimate Std.Err   2.5%  97.5% P-value
#> age 0.0002297 0.05522 -0.108 0.1085  0.9967
estimate(dr)
#>     Estimate Std.Err     2.5%    97.5% P-value
#> x   -0.42738 0.18252 -0.78510 -0.06965  0.0192
#> age  0.09527 0.08586 -0.07301  0.26354  0.2672
estimate(rdr)
#>     Estimate Std.Err    2.5%    97.5% P-value
#> x   -0.48230 0.20942 -0.8928 -0.07185 0.02128
#> age  0.04296 0.09506 -0.1433  0.22927 0.65130
plot(xr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-17-1.png" alt="Recurrent events: stratified two-stage model."  />

``` r
###

glr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=hf,death.code=2)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400
rr <- sim_recurrent_ts(glr,dr,n=n,data=hf)
rxr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=rr,death.code=3)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age   0.0273 0.03922 -0.04958 0.1042  0.4864
estimate(rxr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age  0.06015 0.06414 -0.06556 0.1859  0.3483
estimate(dr)
#>     Estimate Std.Err     2.5%    97.5% P-value
#> x   -0.42738 0.18252 -0.78510 -0.06965  0.0192
#> age  0.09527 0.08586 -0.07301  0.26354  0.2672
estimate(rdr)
#>     Estimate Std.Err    2.5%    97.5% P-value
#> x   -0.48433  0.2076 -0.8913 -0.07738 0.01967
#> age  0.05767  0.1067 -0.1515  0.26682 0.58887
plot(glr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-17-2.png" alt="Recurrent events: stratified two-stage model."  />


Frailty models (simulations based on the rates/intensities) 


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 400
 rr <- sim_recurrent(n,base1,death.cumhaz=dr)
 ###
 mets:::showfitsim(causes=1,rr,dr,base1,base1,which=1:2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-1.png" alt="Recurrent events: frailty model simulations."  />

``` r

 rr <- sim_recurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
#> 
#>       status    0    1    2
#> death                      
#> 0              41 1123  161
#> 1             359    0    0
 mets:::showfitsim(causes=2,rr,dr,base1,base4,which=1:2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-2.png" alt="Recurrent events: frailty model simulations."  />

``` r

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- sim_recurrent_list(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
#> 
#>       status   0   1   2   3
#> death                       
#> 0             10 832 847 111
#> 1            277   0   0   0
#> 2            113   0   0   0
 mets:::showfitsimList(rr,cumhaz,drl) 
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-3.png" alt="Recurrent events: frailty model simulations."  />


# SessionInfo



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Average treatment effect (ATE) for Competing risks and binary outcomes"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Competing risks and binary outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

The `binregATE` function estimates the average treatment effect for binary outcomes with
IPCW adjustment at a specific time-point, and can thus be used for survival or competing risks data.
Computation is linear in data size and influence functions are computed and available.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Average treatment effect 
=========================

First we simulate some data that mimics that of Kumar et al. (2012).
This is data from multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR). The data consisted of patients transplanted from 1995 to
2005, comparing outcomes between transplant periods: 2001--2005 ($N=488$)
versus 1995--2000 ($N=375$). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1) defined as death without relapse.
Kumar et al. (2012) considered the following risk covariates:
transplant time period (`gp`, main interest of the study: 1 for transplanted in
2001--2005 versus 0 for transplanted in 1995--2000), donor type (`dnr`: 1 for unrelated or
other related donor ($N=280$) versus 0 for HLA-identical sibling ($N=584$)), prior
autologous transplant (`preauto`: 1 for Auto+Allo transplant ($N=399$) versus 0 for
allogeneic transplant alone ($N=465$)) and time to transplant (`ttt24`: 1 for more than 24 months ($N=289$) versus 0 for less than or
equal to 24 months ($N=575$)).


We generate similar data by assuming that the two cumulative incidence curves are logistic and that
censoring depends on covariates via a Cox model, all wrapped in the `kumarsim` function.
The simulation does not enforce the constraint $F_1+F_2 < 1$, but as we increase the
sample size we still recover the parameters of cause 2.

We start by computing the average treatment effects for the transplant periods (`gp`), then consider
the possible interaction between `gp` and donor type (`dnr`) and estimate all contrasts between the risk estimates for all combinations
of `gp` and `dnr`. The key components are:

  - an outcome model (fitted by IPCW): `Event(...) ~ .....`
  - a propensity score model (using logistic regression): `treat.model=...`
  - an IPCW weight estimated using a stratified Kaplan-Meier model: `cens.model=~strata(..)`


```{r}
library(mets) 
set.seed(100)
###
n <- 400
kumar <- kumarsim(n,depcens=1)
kumar$cause <- kumar$status
kumar$ttt24 <- kumar[,6]
dtable(kumar,~cause)
dfactor(kumar) <- gp.f~gp
kumar$id <- 1:n
kumar$idc <- sample(100,n,TRUE)
kumar$ids <- sample(n,n)
kumar$id2 <- sample(n,n)
kumar2 <- kumar[order(kumar$id2),]
kumar$int <- interaction(kumar$gp,kumar$dnr)
kumar2$int <- interaction(kumar2$gp,kumar2$dnr)
clust <- 0

b2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(b2)

b5 <- binregATE(Event(time,cause)~int+preauto+ttt24,kumar,cause=2,
		treat.model=int~preauto+ttt24,cens.code=0,time=60)
summary(b5)
```

We note that the estimates found using the large censoring model are very different from those
using the simple Kaplan-Meier weights that are severely biased for these data. This is due to a strong 
censoring dependence. 

The average treatment is around $0.17 = E(Y(1) - Y(0))$ at time 60 for the 
transplant period, under the standard causal assumptions. 
The 1/0 treatment variable used for the causal computation is found as the right hand side (rhs) of the treat.model 
or as the first argument on the rhs of the response model. 

The binregATE default uses binreg with its default to fit the working model and is 
recommended, but the logitIPCW and logitIPCWATE can also be used and are GLM-type 
IPCW weighted models (see binreg help page/vignette).  We can compare the logitIPCWATE/logitIPCW
with that of wglm and its ate function in the riskRegression package, and they agree. 

```{r}
ib2 <- logitIPCWATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(ib2)

ib5 <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,
	 time=60,cens.model=~strata(gp,dnr))
summary(ib5)

ibs <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,time=60)
summary(ibs)

check <- 0
if (check==1) {
require(riskRegression)
e.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, formula.censor = Surv(time,cause==0)~+1, times = 60, data = kumar, product.limit=TRUE,cause=2)
summary(e.wglm)$coef
estimate(ibs)

es.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, 
formula.censor = Surv(time,cause==0)~strata(gp,dnr), times = 60, 
data = kumar, product.limit=TRUE,cause=2)
summary(es.wglm)$coef
estimate(ib5)
}
```

The cluster argument should not be used for the logitIPCWATE, but works for binregATE.


Average treatment for Competing risks data 
==========================================

The binreg function does direct binomial regression for one time-point, $t$, 
fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta),
\end{align*}
for possible right censored data. The estimation procedure 
is based on IPCW adjusted estimating equation (EE)
\begin{align*}
 U(\beta) = &  X \left( \Delta(t) I(T \leq t, \epsilon=1 )/G_c(T \wedge t) - \mbox{expit}( X^T beta) \right) = 0 
\end{align*}
where $G_c(t)=P(C > t)$, the censoring survival distribution, and with $\Delta(t) = I( C > T \wedge t)$ the indicator
of being uncensored at time $t$. 

The function logitIPCW instead considers the EE
\begin{align*}
 U(\beta) = &  X  \frac{\Delta(t)}{G_c(T \wedge t)} \left( I(T \leq t, \epsilon=1 ) - \mbox{expit}( X^T beta) \right) = 0.
\end{align*}
The two score equations are quite similar, and exactly the same when the censoring model is fully-nonparametric given $X$.

Additional functions logitATE, and binregATE computes the average treatment effect. We demonstrate their use below. 

The functions binregATE (recommended) and logitATE also works when there is no censoring and we thus have simple binary outcome. 

Variance is based on  sandwich formula with IPCW adjustment, 
and naive.var is variance 
under a known censoring model. The influence functions are stored in the 
output. Further, the standard errors can be
cluster corrected by specifying the relevant cluster for the working outcome model.

 - We estimate the average treatment effect of our binary response $I(T \leq t, \epsilon=1)$
   - Using a working logistic model for the response (possibly with a cluster specification)
   - Using a working logistic model for treatment given covariates
     - The binregATE can also handle a factor with more than two levels and then uses
        the mlogit multinomial regression function (of mets). 
   - Using a working model for censoring given covariates, this must be a stratified Kaplan-Meier.

If there are no censoring then the censoring weights are simply set to 1.

The average treatment effect is 
\begin{align*}
 E(Y(1) - Y(0)) 
\end{align*}
using counterfactual outcomes. 

We compute the simple G-estimator 
\begin{align*}
\sum  m_a(X_i) 
\end{align*}
to estimate the risk $E(Y(a))$. 

The DR-estimator instead uses the estimating equations that are double robust wrt 

   - A working logistic model for the response
   - A working logistic model for treatment given covariates

This is estimated using the estimator 
\begin{align*}
\sum \left[ \frac{A_i Y_i}{\pi_A(X_i)}-\frac{A_i - \pi_A(X_i)}{\pi_A(X_i)} m_1(X_i) \right]
   - \left[ \frac{(1-A_i) Y_i}{1-\pi_A(X_i)}+\frac{A_i - \pi_A(X_i)}{1-\pi_A(X_i)} m_0(X_i) \right]
\end{align*}
where

 - $A_i$ is treatment indicator
 - $\pi_A(X_i) = P(A_i=1|X_i)$ is treatment model 
 - $Y_i$ outcome, that in case of censoring is censoring adjusted $\tilde Y_i \Delta(t) /G_c(T_i- \wedge t)$
 - $\tilde Y_i = I(T_i \leq t, \epsilon_i=1)$ outcome before censoring.
 - $m_j(X_i)=P(Y_i=1| A_i=j,X_i)$ is outcome model, using binomial regression.

 The standard errors are then based on an iid decomposition using taylor-expansions 
 for the
 parameters of the treatment-model and the outcome-model, and the 
 censoring probability. 

 We need that the censoring model is correct, so it can be important to use a sufficiently 
 large censoring model as we also illustrate below. 

   - The censoring model can be specified by strata (used for `phreg`).

We also compute standard marginalization for average treatment effect  (called differenceG) 
\begin{align*}
\sum \left[  m_1(X_i) - m_0(X_i) \right]
\end{align*}
and again standard errors are based on the related influence functions and are also returned. 

For large data where there are more than 2 treatment groups the 
computations can be memory extensive when
there are many covariates due to the multinomial-regression model used for the 
propensity scores. Otherwise
the function (binregATE) will run for large data.


The ATE functions require that the treatment variable, given as the first variable on
the right-hand side of the outcome model, is a factor. The variable is also
identified from the left-hand side of the
treatment model (`treat.model`), which by default
assumes that treatment does not depend on any covariates.


Average treatment effect for binary or continuous responses
===========================================================

In the binary case a binary outcome is specified instead of the survival outcome, and as a
consequence no censoring adjustment is done.

 - the binary/numeric outcome must be a variable in the data frame

The following code runs the analysis (one can also use `binregATE` by coding `cause` without censoring values, i.e. setting `cens.code=2` and `time` large):

```{r}
kumar$cause2 <- 1*(kumar$cause==2)

b3 <- logitATE(cause2~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)

###library(targeted)
###b3a <- ate(cause2~gp.f|dnr+preauto+ttt24| dnr+preauto+ttt24,kumar,family=binomial)
###summary(b3a)

## calculate also relative risk
estimate(coef=b3$riskDR,vcov=b3$var.riskDR,f=function(p) p[1]/p[2])
```

Or with continuous response using normal estimating equations

```{r}
b3 <- normalATE(time~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "While Alive estimands for Recurrent Events"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{While Alive estimands for Recurrent Events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```


While Alive estimands for Recurrent Events
==========================================

We consider two while-alive estimands for recurrent events data
\begin{align*}
 \frac{E(N(D \wedge t))}{E(D \wedge t)}
\end{align*}
and the mean of the subject specific events per time-unit 
\begin{align*}
 E( \frac{N(D \wedge t)}{D \wedge t} ) 
\end{align*}
for two treatment-groups in the case of an RCT. 
For the mean of events per time-unit it has been seen that when the sample size is small 
one can improve the finite sample properties by employing a transformation such as square or cube-root, and
thus consider 
\begin{align*}
 E( (\frac{N(D \wedge t)}{D \wedge t})^.33 ) 
\end{align*}

```{r while-alive}
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")

```

We see that the ratio of means is not very different between groups, but that the subject-specific
mean of events per time-unit shows that those on the active treatment have fewer events per time-unit on average.

We can also fit a regression model for the mean of the subject-specific events per time-unit, here using the
exponential link.

```{r while-alive-reg}
hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id]
hfactioncpx12$sex <- rbinom(741,1,0.5)[hfactioncpx12$id]

dd <- WA_reg(Event(entry,time,status)~treatment+age+sex+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
```

Composite outcomes involving death and marks
============================================

The event count can be generalised in various ways by using outcomes other than $N(D \wedge t)$, for example,
\begin{align*}
 \tilde N(D \wedge t) = \int_0^t I(D \geq s) M(s) dN(s)  +  \sum_j  M_j I(D \leq t,\epsilon=j) )
\end{align*}
where $M(s)$ are the marks associated with $N(s)$ and $M_j$ are marks associated with the different
causes of the terminal event. This provides an extension of the weighted composite outcomes measure of Mao \& Lin (2022).

The marks (or weights) can be stochastic, for example when counting hospital expenses, and are
stored as a vector in the data frame. The marks for the event times (defined through the causes) are then used.

Here we weight death by 2 and otherwise count recurrent events as before (with weight 1):

```{r wa-composite}
hfactioncpx12$marks <- runif(nrow(hfactioncpx12))

##ddmg <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
##cause=1:2,death.code=2,marks=hfactioncpx12$marks)
##summary(ddmg)

ddm <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
cause=1:2,death.code=2,marks=hfactioncpx12$status)

```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "GEE cluster standard errors and predictions for glm objects" 
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{GEE cluster standard errors and predictions for glm objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Utility functions for GLM objects
=================================

Computing odds ratios with confidence intervals using GEE (sandwich)
standard errors:

```{r}
set.seed(100)

library(mets)
data(bmt); 
bmt$id <- sample(1:100,408,replace=TRUE)

glm1 <- glm(tcell~platelet+age,bmt,family=binomial)
summaryGLM(glm1)

## GEE robust standard errors
summaryGLM(glm1,id=bmt$id)
```

Predictions are equally straightforward:

```{r}
age <- seq(-2,2,by=0.1)
nd <- data.frame(platelet=0,age=seq(-2,2,by=0.1))
pnd <- predictGLM(glm1,nd)
head(pnd$pred)
plot(age,pnd$pred[,1],type="l",ylab="predictions",xlab="age",ylim=c(0,0.3))
plotConfRegion(age,pnd$pred[,2:3],col=2)
###matlines(age,pnd$pred[,-1],col=2,lty=1)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "IPCW Cumulative Cost"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
header-includes: 
  - \usepackage{tikz}
vignette: >
  %\VignetteIndexEntry{IPCW Cumulative Cost}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview
========

We describe how to perform regression modelling for cumulative cost
\begin{align*}
{\cal U}(t) & = \int_0^t Z(s) dN(s)
\end{align*}
where $N(s)$ is a counting process that registers the times at which costs are realised and accumulated,
and $Z(t)$ is the cost (or mark) at the event times. The counting process can be
a mix of random and fixed times, and the data are represented in counting process format with
the marks/costs attached to the event times. There are many additional uses of
such cumulative processes; for example, when considering time lost in a recurrent events setting, which we return to
below.

We can estimate the marginal mean of the cumulative process
\begin{align*}
\nu(t) & = E ( {\cal U}(t) )  
\end{align*}
possibly for strata with standard errors based on the derived influence function. 

We provide semi-parametric regression modelling using the  proportional model 
\begin{align*}
E ( {\cal U}(t) | X) & = \Lambda_0(t) \exp( X^T \beta).
\end{align*}

In addition for a fixed time-point $t \in [0,\tau]$ we can estimate the mean given covariates
\begin{align*}
E ( {\cal U}(t) | X) & = \exp( X^T \beta) 
\end{align*}
where $\tau$ is some maximum follow-up time. 

 - These quantities are estimated under independent right-censoring given $X$, using IPCW-adjusted estimating equations,
    - similarly to the Ghosh-Lin model for recurrent events.
 - A terminal event can be specified.

We also estimate the probability of exceeding thresholds over time
\begin{align*}
 P ( {\cal U}(t)  > k ) & = \mu_k(t),
\end{align*}
and in the setting with a terminal event this is based on a derived competing risks data structure that
keeps track of the competing terminal event.

\begin{tikzpicture}[
    node distance=3.5cm,
    box/.style={draw, rectangle, minimum width=2.8cm, minimum height=1cm, align=center}
]
% Nodes
\node[box] (start) {At Risk};
\node[box, right of=start, yshift=2.4cm] (exceed) {Exceed-k};
\node[box, right of=start, yshift=-2.4cm] (death) {D};

% Arrows with hazard labels
\draw[->, thick] (start) -- node[below] {\hspace{2 cm} $F_{exceed-k}(t)$} (exceed);
\draw[->, thick] (start) -- node[below] {$F_{D}(t)$} (death);
\end{tikzpicture}

Regression modelling of this quantity is also possible using competing risks
regression models, for example via the `cifreg` function in mets.


HF-action data
==============

Using the HF-action data, we simulate a severity score for each event.



``` r
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
set.seed(1)
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

## marginal mean using formula  
outNZ <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 
#> [[1]]
#>     new.time    mean        se CI-2.5% CI-97.5% strata
#> 682        3 10.3249 0.6044746  9.2056  11.5803      0
#> 
#> [[2]]
#>     new.time    mean        se  CI-2.5% CI-97.5% strata
#> 601        3 9.33613 0.6429995 8.157232 10.68541      1

outN <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
```

<img src="figure/cumulative-cost-unnamed-chunk-2-1.png" alt="Marginal mean cumulative cost by treatment."  />

``` r
summary(outN,times=3) 
#> [[1]]
#>     new.time     mean        se  CI-2.5% CI-97.5% strata
#> 682        3 2.118496 0.1138572 1.906692 2.353829      0
#> 
#> [[2]]
#>     new.time     mean        se  CI-2.5% CI-97.5% strata
#> 601        3 1.924062 0.1216577 1.699801 2.177912      1
```

For comparison we also compute the IPCW estimates with and without marks at time 3 using the linear model,
and note that they are identical. Standard errors are based on different formulae that are asymptotically
equivalent, and we note that they are very similar.


``` r
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
#>    n events
#>  741   1281
#> 
#>  741 clusters
#> coeffients:
#>            Estimate  Std.Err     2.5%    97.5% P-value
#> treatment0 10.32490  0.60444  9.14022 11.50959       0
#> treatment1  9.33613  0.64295  8.07597 10.59629       0
head(iid(outNZ3))
#>            [,1]      [,2]
#> 1 -0.0013037837 0.0000000
#> 2  0.0089249278 0.0000000
#> 3  0.0000000000 0.0211858
#> 4 -0.0056613816 0.0000000
#> 5 -0.0005288663 0.0000000
#> 6 -0.0342554114 0.0000000

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
#>    n events
#>  741   1281
#> 
#>  741 clusters
#> coeffients:
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment0  2.11850 0.11385 1.89535 2.34164       0
#> treatment1  1.92406 0.12165 1.68564 2.16248       0
head(iid(outN3))
#>            [,1]        [,2]
#> 1  0.0004542472 0.000000000
#> 2  0.0009756994 0.000000000
#> 3  0.0000000000 0.009301496
#> 4 -0.0029668336 0.000000000
#> 5 -0.0001120764 0.000000000
#> 6 -0.0070693971 0.000000000
```

We also apply the semiparametric proportional cost model with IPCW adjustment: 


``` r
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
#> 
#>     n events
#>  2132   1391
#> 
#>  741 clusters
#> coefficients:
#>             Estimate      S.E.   dU^-1/2 P-value
#> treatment1 -0.101014  0.089005  0.024304  0.2564
#> 
#> exp(coefficients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.90392 0.75922 1.0762
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
#> 
#>     n events
#>  2132   1391
#> 
#>  741 clusters
#> coefficients:
#>             Estimate      S.E.   dU^-1/2 P-value
#> treatment1 -0.110404  0.078656  0.053776  0.1604
#> 
#> exp(coefficients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.89547 0.76754 1.0447
plot(GL,add=TRUE,col=2)
```

<img src="figure/cumulative-cost-unnamed-chunk-4-1.png" alt="Proportional cost model baseline estimates."  />
Those treated have 14\% lower cumulative severity and 11\% lower expected number of events.


Exceed threshold
================

Finally, we estimate the probability of exceeding cumulative severity thresholds of 1, 5, and 10:


``` r
ooNZ <- prob_exceed_recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
```

<img src="figure/cumulative-cost-unnamed-chunk-5-1.png" alt="Probability of exceeding cumulative cost thresholds."  />

``` r
summary(ooNZ,times=3)
#> $`0`
#> $`0`$prob
#>            times           
#>                3 2.99865085
#> N<1            3 0.06474819
#> exceed>=1      3 0.93525181
#> exceed>=5      3 0.90118229
#> exceed>=10     3 0.74537955
#> exceed>=20     3 0.49282394
#> 
#> $`0`$se
#>            times           
#>                3 2.99865085
#> N<1            3 0.02657120
#> exceed>=1      3 0.02657120
#> exceed>=5      3 0.03013468
#> exceed>=10     3 0.04466101
#> exceed>=20     3 0.05149605
#> 
#> $`0`$lower
#>      times          
#> [1,]     3 2.9986509
#> [2,]     3 0.1154033
#> [3,]     3 0.8845967
#> [4,]     3 0.8440133
#> [5,]     3 0.6627899
#> [6,]     3 0.4015580
#> 
#> $`0`$upper
#>      times           
#> [1,]     3 2.99865085
#> [2,]     3 0.01119234
#> [3,]     3 0.98880766
#> [4,]     3 0.96222364
#> [5,]     3 0.83826056
#> [6,]     3 0.60483284
#> 
#> 
#> $`1`
#> $`1`$prob
#>            times           
#>                3 2.99865085
#> N<1            3 0.01770732
#> exceed>=1      3 0.98229268
#> exceed>=5      3 0.94934557
#> exceed>=10     3 0.79560979
#> exceed>=20     3 0.53319063
#> 
#> $`1`$se
#>            times           
#>                3 2.99865085
#> N<1            3 0.01097663
#> exceed>=1      3 0.01097663
#> exceed>=5      3 0.01645312
#> exceed>=10     3 0.03727391
#> exceed>=20     3 0.05545584
#> 
#> $`1`$lower
#>      times           
#> [1,]     3 2.99865085
#> [2,]     3 0.03898725
#> [3,]     3 0.96101275
#> [4,]     3 0.91763959
#> [5,]     3 0.72580801
#> [6,]     3 0.43486168
#> 
#> $`1`$upper
#>      times          
#> [1,]     3 2.9986509
#> [2,]     3 0.0000000
#> [3,]     3 1.0000000
#> [4,]     3 0.9821470
#> [5,]     3 0.8721245
#> [6,]     3 0.6537533
```

Cumulative time lost for recurrent events
=========================================

The cumulative time lost for recurrent events is defined as
\begin{align*}
  {\cal M}(t) = E\left[ \int_0^\tau (\tau-s) dN(s) \right] = \int_0^\tau \mu(s) ds
\end{align*}
where $\mu(t) = E( N(t) )$ is the marginal mean of the recurrent events at time $t$.



``` r
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
#>    n events
#>  741   1391
#> 
#>  741 clusters
#> coeffients:
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment0  8.58300 0.42951 7.74118 9.42482       0
#> treatment1  7.66234 0.46400 6.75292 8.57177       0
head(iid(RecLost))
#>            [,1]       [,2]
#> 1  0.0016920221 0.00000000
#> 2  0.0073388996 0.00000000
#> 3  0.0000000000 0.02120478
#> 4 -0.0095548150 0.00000000
#> 5 -0.0005696809 0.00000000
#> 6 -0.0201750011 0.00000000
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "While Alive estimands for Recurrent Events"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{While Alive estimands for Recurrent Events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




While Alive estimands for Recurrent Events
==========================================

We consider two while-alive estimands for recurrent events data
\begin{align*}
 \frac{E(N(D \wedge t))}{E(D \wedge t)}
\end{align*}
and the mean of the subject specific events per time-unit 
\begin{align*}
 E( \frac{N(D \wedge t)}{D \wedge t} ) 
\end{align*}
for two treatment-groups in the case of an RCT. 
For the mean of events per time-unit it has been seen that when the sample size is small 
one can improve the finite sample properties by employing a transformation such as square or cube-root, and
thus consider 
\begin{align*}
 E( (\frac{N(D \wedge t)}{D \wedge t})^.33 ) 
\end{align*}


``` r
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
#> 
#> status
#>    0    1    2 
#>  617 1391  124
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
#> While-Alive summaries:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate Std.Err  2.5% 97.5% P-value
#> treatment0    1.859 0.02108 1.817 1.900       0
#> treatment1    1.924 0.01502 1.894 1.953       0
#>  
#>                           Estimate Std.Err    2.5%    97.5% P-value
#> [treatment0] - [treat.... -0.06517 0.02588 -0.1159 -0.01444  0.0118
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err  2.5% 97.5%   P-value
#> treatment0    1.572 0.09573 1.384 1.759 1.375e-60
#> treatment1    1.453 0.10315 1.251 1.656 4.376e-45
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....   0.1185  0.1407 -0.1574 0.3943     0.4
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>    Estimate Std.Err   2.5%  97.5%   P-value
#> p1   0.8457 0.05264 0.7425 0.9488 4.411e-58
#> p2   0.7555 0.05433 0.6490 0.8619 5.963e-44
#>  
#>             Estimate Std.Err     2.5%  97.5% P-value
#> [p1] - [p2]  0.09022 0.07565 -0.05805 0.2385   0.233
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err   2.5%  97.5%   P-value
#> treat0   1.0725  0.1222 0.8331 1.3119 1.645e-18
#> treat1   0.7552  0.0643 0.6291 0.8812 7.508e-32
#>  
#>                     Estimate Std.Err    2.5%  97.5% P-value
#> [treat0] - [treat1]   0.3173  0.1381 0.04675 0.5879 0.02153

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")
#> While-Alive summaries, log-scale:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate  Std.Err   2.5%  97.5% P-value
#> treatment0   0.6199 0.011340 0.5977 0.6421       0
#> treatment1   0.6543 0.007807 0.6390 0.6696       0
#>  
#>                           Estimate Std.Err     2.5%     97.5% P-value
#> [treatment0] - [treat.... -0.03446 0.01377 -0.06145 -0.007478 0.01231
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err   2.5%  97.5%   P-value
#> treatment0   0.4523 0.06090 0.3329 0.5716 1.119e-13
#> treatment1   0.3739 0.07097 0.2348 0.5130 1.376e-07
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....  0.07835 0.09352 -0.1049 0.2616  0.4022
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>            Estimate Std.Err    2.5%    97.5%   P-value
#> treatment0  -0.1676 0.06224 -0.2896 -0.04563 7.081e-03
#> treatment1  -0.2804 0.07192 -0.4214 -0.13947 9.651e-05
#>  
#>                           Estimate Std.Err     2.5%  97.5% P-value
#> [treatment0] - [treat....   0.1128 0.09511 -0.07361 0.2992  0.2356
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err    2.5%   97.5%   P-value
#> treat0  -0.3833 0.04939 -0.4801 -0.2865 8.487e-15
#> treat1  -0.5380 0.05666 -0.6491 -0.4270 2.191e-21
#>  
#>                     Estimate Std.Err     2.5%  97.5% P-value
#> [treat0] - [treat1]   0.1548 0.07517 0.007459 0.3021 0.03948
```

We see that the ratio of means is not very different between groups, but that the subject-specific
mean of events per time-unit shows that those on the active treatment have fewer events per time-unit on average.

We can also fit a regression model for the mean of the subject-specific events per time-unit, here using the
exponential link.


``` r
set.seed(10)
hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id]
hfactioncpx12$sex <- rbinom(741,1,0.5)[hfactioncpx12$id]

dd <- WA_reg(Event(entry,time,status)~treatment+age+sex+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
#>    n events
#>  741     86
#> 
#>  741 clusters
#> coeffients:
#>                Estimate     Std.Err        2.5%       97.5% P-value
#> (Intercept) -1.6366e-02  1.1187e-01 -2.3564e-01  2.0290e-01  0.8837
#> treatment1  -3.5688e-01  1.4242e-01 -6.3602e-01 -7.7743e-02  0.0122
#> age         -1.0527e-01  5.3727e-02 -2.1058e-01  2.9638e-05  0.0501
#> sex          1.5723e-01  1.4784e-01 -1.3253e-01  4.4700e-01  0.2876
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.98377 0.79007 1.2250
#> treatment1   0.69986 0.52940 0.9252
#> age          0.90008 0.81012 1.0000
#> sex          1.17027 0.87587 1.5636
```

Composite outcomes involving death and marks
============================================

The event count can be generalised in various ways by using outcomes other than $N(D \wedge t)$, for example,
\begin{align*}
 \tilde N(D \wedge t) = \int_0^t I(D \geq s) M(s) dN(s)  +  \sum_j  M_j I(D \leq t,\epsilon=j) )
\end{align*}
where $M(s)$ are the marks associated with $N(s)$ and $M_j$ are marks associated with the different
causes of the terminal event. This provides an extension of the weighted composite outcomes measure of Mao \& Lin (2022).

The marks (or weights) can be stochastic, for example when counting hospital expenses, and are
stored as a vector in the data frame. The marks for the event times (defined through the causes) are then used.

Here we weight death by 2 and otherwise count recurrent events as before (with weight 1):


``` r
hfactioncpx12$marks <- runif(nrow(hfactioncpx12))

##ddmg <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
##cause=1:2,death.code=2,marks=hfactioncpx12$marks)
##summary(ddmg)

ddm <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
cause=1:2,death.code=2,marks=hfactioncpx12$status)

```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] prodlim_2026.03.11 timereg_2.0.7      survival_3.8-6     mets_1.3.12       
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              KernSmooth_2.23-26     otel_0.2.0            
#>  [7] data.table_1.18.4      future.apply_1.20.2    listenv_0.10.1        
#> [10] lava_1.9.2.1           stats4_4.6.0           grid_4.6.0            
#> [13] evaluate_1.0.5         yaml_2.3.12            mvtnorm_1.3-7         
#> [16] numDeriv_2016.8-1.1    compiler_4.6.0         codetools_0.2-20      
#> [19] Rcpp_1.1.1-1.1         ucminf_1.2.3           future_1.70.0         
#> [22] lattice_0.22-9         digest_0.6.39          parallelly_1.47.0     
#> [25] parallel_4.6.0         Matrix_1.7-5           tools_4.6.0           
#> [28] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Cumulative Incidence Regression"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5    
vignette: >
  %\VignetteIndexEntry{Cumulative Incidence Regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

The `cifreg` function fits the Fine-Gray model and the logit-link cumulative incidence
model for the cause of interest in competing risks settings. Computation is linear in
data size, making it suitable for large datasets.
For the Fine-Gray model, predictions with standard errors can be provided for specific
time-points based on influence functions for the baseline and the regression coefficients.

Key features:

  - the baseline can be stratified
  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors (Fine-Gray only)
  - computation time is linear in data size, including standard errors
  - Fine-Gray only: influence functions of baseline and regression coefficients are computed and available via `IC`, `iid`, and `iidBaseline`
  - cluster-corrected standard errors are available via the `clusters` argument


Fine-Gray model
================

Fine and Gray (1999) considered a cumulative incidence of the form
\begin{align*}
 F_1(t,X) & = P(T \leq t, \epsilon=1) = 1 - \exp( - \Lambda_0(t) \exp(X^T \beta)).
\end{align*}

In the case of independent right-censoring with the censoring distribution $G_c(t,X) = P(C > t | S(X))$ where
$S(X)$ is a set of strata defined from $X$, then an unbiased estimating equation is given by
\begin{align*}
  U^{FG}_{n}(\beta) = \sum_{i=0}^{n} \int_0^{+\infty} \left( X_i- E_n(t,\beta) \right) w_i(t,X_i) dN_{1,i}(t)  \text{ where }
	E_n(t,\beta)=\frac{\tilde S_1(t,\beta) }{\tilde S_0(t,\beta)},
\end{align*}
with \(w_i(t,X_i) = \frac{G_c(t,X_i)}{G_c(T_i \wedge t,X_i)} I( C_i > T_i \wedge t ) \)
,$\tilde S_k(t,\beta) = \sum_{j=1}^n X_j^k \exp(X_j^T\beta) Y_{1,j}(t)$
for $k=0,1$, and with $\tilde Y_{1,i}(t) = Y_{1,i}(t) w_i(t,X_i)$ for $i=1,...,n$.
$w_i(t)$ needs to be replaced by an estimator of the
censoring distribution; since it does not depend on $X$, we use
$\hat w_i(t) = \frac{\hat G_c(t,X_i)}{\hat G_c(T_i \wedge t,X_i)} I(C_i > T_i \wedge t)$
where $\hat G_c$ is the Kaplan-Meier estimator of the censoring distribution.

First we simulate some competing risks data using some utility functions.

We simulate data with two causes based on the Fine-Gray model:
\begin{align}
F_1(t,X) &  = P(T\leq t, \epsilon=1|X)=( 1 - exp(-\Lambda_1(t) \exp(X^T \beta_1))) \\
F_2(t,X) & = P(T\leq t, \epsilon=2|X)= ( 1 - exp(-\Lambda_2(t) \exp(X^T \beta_2))) \cdot (1 - F_1(\infty,X))  
\end{align}
where the baselines are given as $\Lambda_j(t) = \rho_j (1- exp(-t/\nu_j))$ for $j=1,2$, and the 
$X$ being two independent binomials. 
Alternatively, one can also replace the FG-model with a logistic link 
$\mbox{expit}( \Lambda_j(t) + \exp(X^T \beta_j))$.

The advantage of the Fine-Gray model is that it is easy to fit, easy to obtain
standard errors for, and quite flexible. On the downside,
the coefficients must be interpreted on the $\mbox{cloglog}$ scale.
Specifically, 
\begin{align}
\log(-\log( 1-F_1(t,X_1+1,X_2))) - \log(-\log( 1-F_1(t,X_1,X_2))) & =  \beta_1,
\end{align}
so the effect of an increase in $X_1$ is $\beta_1$ and leads to $1-F_1(t,X)$ on the $cloglog$ scale. 


```{r}
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in simulations for p-values below.

 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 ## beta1=c(0.0,-0.1); beta2=c(-0.5,0.3)
 dats <- simul_cifs(n,rho1,rho2,beta,rc=0.5,rate=7)
 dtable(dats,~status)
 dsort(dats) <- ~time
```

We have a look at the non-parametric cumulative incidence curves

```{r}
 par(mfrow=c(1,2))
 cifs1 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=1)
 plot(cifs1)

 cifs2 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=2)
 plot(cifs2)
```

Now fitting the Fine-Gray model 

```{r}
 fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(fg)

 dd <- expand.grid(Z1=c(-1,1),Z2=0:1)
 pfg <- predict(fg,dd)
 plot(pfg,ylim=c(0,0.2))
```

and GOF based on cumulative residuals (Li et al. 2015) 

```{r}
gofFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
```

showing no problem with the proportionality of the model. 


SE's for the baseline and predictions of FG
===========================================

The standard errors reported for the Fine-Gray estimator are based on the i.i.d.
decomposition (influence functions) of the estimator. A similar decomposition
exists for the baseline and is needed when standard errors of predictions are computed. These
are somewhat harder to compute for all time-points simultaneously, but they can be
obtained for specific time-points jointly with the i.i.d. decomposition of the regression coefficients,
and then used to obtain standard errors for predictions.

We plot the predictions with confidence intervals for predictions at time point 5:

```{r,pred-cif}
### predictions with CI based on iid decomposition of baseline and beta
fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
Biid <- iidBaseline(fg,time=5)
pfgse <- FGprediid(Biid,dd)
pfgse
plot(pfg,ylim=c(0,0.2))
for (i in 1:4) lines(c(5,5)+i/10,pfgse[i,3:4],col=i,lwd=2)
```

The i.i.d. decompositions are stored inside `Biid`; the
i.i.d. decomposition for $\hat \beta - \beta_0$ is obtained via the `iid()` function.

# Comparison 

We compare with the `cmprsk` function, which gives exactly the same results, but omit the code to avoid dependencies:

```{r,cmprsk}
run <- 0
if (run==1) {
library(cmprsk)
mm <- model.matrix(~Z1+Z2,dats)[,-1]
cr <- with(dats,crr(time,status,mm))
cbind(cr$coef,diag(cr$var)^.5,fg$coef,fg$se.coef,cr$coef-fg$coef,diag(cr$var)^.5-fg$se.coef)
#          [,1]      [,2]       [,3]      [,4]          [,5]          [,6]
# Z1  0.6968603 0.3876029  0.6968603 0.3876029 -2.442491e-15 -2.553513e-15
# Z2 -0.8592892 0.6245258 -0.8592892 0.6245258 -2.997602e-15  1.776357e-15
}
```

When comparing with the results from `coxph` based on setting up the data using
the `finegray` function, we get the same estimates but note that the standard errors
from `coxph` are missing a term and therefore slightly different. When comparing to
the estimates from `coxph` without the additional censoring term, we also get
the same standard errors.

```{r,coxph}
if (run==1) {
 library(survival)
 dats$id <- 1:nrow(dats)
 dats$event <- factor(dats$status,0:2, labels=c("censor", "death", "other"))
 fgdats <- finegray(Surv(time,event)~.,data=dats)
 coxfg <- survival::coxph(Surv(fgstart, fgstop, fgstatus) ~ Z1+Z2 + cluster(id), weight=fgwt, data=fgdats)

 fg0 <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1,propodds=NULL)
 cbind( coxfg$coef,fg0$coef, coxfg$coef-fg0$coef)
#          [,1]       [,2]          [,3]
# Z1  0.6968603  0.6968603 -1.110223e-16
# Z2 -0.8592892 -0.8592892 -1.110223e-15
 cbind(diag(coxfg$var)^.5,fg0$se.coef,diag(coxfg$var)^.5-fg0$se.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3876029  0.0013099915
# [2,] 0.6241225 0.6245258 -0.0004033148
 cbind(diag(coxfg$var)^.5,fg0$se1.coef,diag(coxfg$var)^.5-fg0$se1.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3889129 -2.331468e-15
# [2,] 0.6241225 0.6241225  2.553513e-15
}
```

We also remove all censorings from the data to compare the estimates with those
based on `coxph`, and observe that both the estimates and the standard errors agree.

```{r,no-cens}
datsnc <- dtransform(dats,status=2,status==0)
dtable(datsnc,~status)
datsnc$id <- 1:n
datsnc$entry <- 0
max <- max(dats$time)+1
## for cause 2 add risk interaval 
datsnc2 <- subset(datsnc,status==2)
datsnc2 <- transform(datsnc2,entry=time)
datsnc2 <- transform(datsnc2,time=max)
datsncf <- rbind(datsnc,datsnc2)
#
cifnc <- cifreg(Event(time,status)~Z1+Z2,data=datsnc,cause=1,propodds=NULL)
cc <- phreg(Surv(entry,time,status==1)~Z1+Z2+cluster(id),datsncf)
cbind(cc$coef-cifnc$coef, diag(cc$var)^.5-diag(cifnc$var)^.5)
#            [,1]          [,2]
# Z1 1.332268e-15 -4.440892e-16
# Z2 4.218847e-15  2.220446e-16
```

the cmprsk also gives the same 

```{r}
if (run==1) {
 library(cmprsk)
 mm <- model.matrix(~Z1+Z2,datsnc)[,-1]
 cr <- with(datsnc,crr(time,status,mm))
 cbind(cc$coef-cr$coef, diag(cr$var)^.5-diag(cc$var)^.5)
#             [,1]         [,2]
# Z1 -4.218847e-15 1.443290e-15
# Z2  7.549517e-15 1.110223e-16
}
```


# Strata dependent Censoring weights 

We can improve efficiency and reduce bias by allowing the censoring weights to depend on
the covariates.

```{r}
 fgcm <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1,cens.model=~strata(Z1,Z2))
 summary(fgcm)
 summary(fg)
```

We note that the standard errors are slightly smaller for the more efficient estimator.

The influence functions of the Fine-Gray estimator are given by Fine and Gray (1999),
\begin{align*}
\phi_i^{FG}  & = \int (X_i- e(t))  \tilde w_i(t) dM_{i1}(t,X_i)  + \int \frac{q(t)}{\pi(t)} dM_{ic}(t),  \\
             & = \phi_i^{FG,1}  + \phi_i^{FG,2},
\end{align*}
where the first term is what would be achieved for a known censoring distribution, and the second term is
due to the variability from the Kaplan-Meier estimator. 
Where $M_{ic}(t) = N_{ic}(t) - \int_0^t Y_i(s) d\Lambda_c (s)$
with $M_{ic}$ the standard censoring martingale. 

The function $q(t)$ that reflects that the censoring only affects the terms related 
to cause "2" jumps, can be written as 
\begin{align*} 
	q(t) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(C > T)/G_c(T)) = E( H(t,X) F_2(t,X) ), 
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s) d \Lambda_1(s,X)$ and since $\pi(t)=E(Y(t))=S(t) G_c(t)$.

In the case where the censoring weights are stratified (based on $X$)
we get the influence functions related to the censoring term with 
\begin{align*} 
	q(t,X) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(T < C)/G_c(T,X) | X) = H(t,X) F_2(t,X), 
\end{align*}
so that the influence function becomes 
\begin{align*}
\int (X-e(t)) w(t) dM_1(t,X) +  \int H(t,X) \frac{F_2(t,X)}{S(t,X)} \frac{1}{G_c(t,X)} dM_c(t,X).
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s,X) d \Lambda_1(s,X)$.


Logistic-link 
================

```{r}
 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 dats <- simul_cifs(n,rho1,rho2,beta,rc=0.5,rate=7,type="logistic")
 dtable(dats,~status)
 dsort(dats) <- ~time
```

The model
\begin{align*}
 \mbox{logit}(F_1(t,X)) & =  \alpha(t) + X^T \beta
\end{align*}
leads to an odds-ratio interpretation of $F_1$ and can be fitted easily; however, the standard errors
are harder to compute and only approximate
(assuming that the censoring weights are known), though this typically introduces only a small error.
In the `timereg` package the model can be
fitted using different estimators that are more efficient but considerably slower.

Fitting the model and getting OR's 

```{r}
 or <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(or)
```


Administrative Censoring
===========================

In the case with administrative censoring we can give the risk-set defined by the administrative censoring 
times for the Fine-Gray or logistic link cumulative incidence regression models. 

 - when an administrative censoring time is given, the risk-set is extended for all subjects experiencing the alternative event
 - if there are also additional random censorings, these can be adjusted for via IPCW weights, specified using `cens.code`
 - alternative causes are found by considering `cause`, `cens.code`, and the total number of values for the status variable; it can therefore be useful to also specify the other `death.code`(s)


```{r}
library(mets)
rho1 <- 0.3; rho2 <- 5.9
set.seed(100)
n <- 100
beta=c(0.3,-0.3,-0.5,0.3)
rc <- 0.5
###
dats <- mets:::simul_cifsRA(n,rho1,rho2,beta,bin=1,rc=rc,rate=c(3,7))
dats$status07 <- dats$status
dats$status07[dats$status %in% c(0,7)] <- 0
tt <- seq(0,6,by=0.1)
base1 <- rho1*(1-exp(-tt/3))
dtable(dats,~status+statusA,level=1)

## only admin censoring 
ccA  <-  cifregFG(Event(timeA,statusA)~Z1+Z2,dats,
		  adm.cens.time=dats$censorA,death.code=2)
estimate(ccA)

## admin and random censoring  via IPCW for C=min(C_A,C_R) 
ccAR_ipcw1  <-  cifregFG(Event(time,status)~Z1+Z2,dats,cens.code=c(0,7))
estimate(ccAR_ipcw1)

## admin and random censoring  via IPCW for C_R 
ccAR_ipcw2  <-  cifregFG(Event(time,status)~Z1+Z2,dats,cens.code=0,
		  adm.cens.time=dats$censorA,no.codes=7)
estimate(ccAR_ipcw2)
```

When there is only administrative censoring, the Fine-Gray model
can similarly be estimated using the modified risk-set and the
`phreg`/`coxph` function (equivalent to `ccA` above).

```{r}
dats$entry <- 0
dats$id <- 1:n
datA <- dats
datA2 <- subset(datA,statusA==2)
datA2$entry <- datA2$timeA
datA2$timeA <- datA2$censorA
datA2$statusA <- 0
datA <- rbind(datA,datA2)
ddA <- phreg(Event(entry,timeA,statusA==1)~Z1+Z2+cluster(id),datA)
estimate(ddA)

## also checking the cumulative baseline 
###plotl(tt,base1) 
###plot(ccA,add=TRUE,col=3)
###plot(ddA,col=2,add=TRUE)
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Recurrent events"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
header-includes: 
  - \usepackage{tikz}
  - \usepackage{adjustbox}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Recurrent events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview 
========

For recurrent events data it is often of interest to compute basic descriptive
quantities to gain some understanding of the phenomenon being studied.
We demonstrate how to compute:

 * the marginal mean 
   * AUC and logrank test for comparison of marginal means (also for special case with cumulative incidence)
   * efficient marginal mean estimation with fast computation of standard errors
 * the Ghosh-Lin Cox type regression for the marginal mean, possibly with composite outcomes. 
   * efficient regression augmentation of the Ghosh-Lin model
   * clusters can be specified
   * allows a stratified baseline 
 * the variance of a recurrent events process
 * the probability of exceeding k events
 * the two-stage recurrent events random effects model

In addition several tools can be used for simulating recurrent events and
bivariate recurrent events data, also with a possible terminating event: 

 * recurrent events with multiple 
   * event types 
   * Cox type rates 
 * with a terminal event with possibly multiple causes of death
   * Cox type rates 
 * frailty extensions
 * the Ghosh-Lin model when the survival rate is on Cox form.
   * frailty extensions
 * The general illness death model with  Cox models for all hazards. 

\begin{tikzpicture}[
    sharp corners=2pt,
    inner sep=4pt,
    node distance=0.9cm,
    >=latex,
    mynode/.style={draw,minimum height=0.9cm,minimum width=2.6cm}
]

\node[mynode] (healthy) {healthy $(h)$};
\node[mynode, right=of healthy] (e1) {1st event};
\node[mynode, right=of e1] (e2) {2nd event};

% Dots indicating continuation
\node[right=0.6cm of e2] (dots) {$\cdots$};

% Dead
\node[mynode, below=of e1] (dead) {dead $(d)$};

% Arrows along the event chain
\draw[->] (healthy) -- (e1);
\draw[->] (e1) -- (e2);
\draw[->] (e2) -- (dots);

% Arrows to 'dead'
\draw[->] (healthy) -- (dead);
\draw[->] (e1) -- (dead);
\draw[->] (e2) -- (dead);

\end{tikzpicture}


Simulation of recurrent events
==============================

We start by simulating some recurrent events data with two event types having cumulative hazards

 * $\Lambda_1(t)$ (rate among survivors)
 * $\Lambda_2(t)$ (rate among survivors)
 * $\Lambda_D(t)$

 where types 1 and 2 are considered and the terminal event rate is given by
 $\Lambda_D(t)$. The events are independent by default, but a random effects
 structure can also be specified to generate dependence.

 When simulating data, various random effects structures can be imposed to generate dependence:

* Dependence=0: The intensities are independent.

* Dependence=1: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $Z \lambda_D(t)$

* Dependence=4: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $\lambda_D(t)$

* Dependence=2: Normally distributed random effects $Z_1,Z_2,Z_d$ with variance (`var.z`) 
  and correlation (`cor.mat`) that can be specified. The intensities are
    + $\exp(Z_1) \lambda_1(t)$
    + $\exp(Z_2) \lambda_2(t)$
    + $\exp(Z_3) \lambda_D(t)$

* Dependence=3: Gamma-distributed random effects $Z_1,Z_2,Z_d$ where the sum structure can be specified via a matrix
   `cor.mat`. We compute $\tilde Z_j = \sum_k  Z_k^{cor.mat(j,k)}$ for $j=1,2,3$.
   The intensities are
    + $\tilde Z_1 \lambda_1(t)$
    + $\tilde Z_2 \lambda_2(t)$
    + $\tilde Z_3 \lambda_D(t)$

We return to how to run the different set-ups later and start by simulating independent processes. 

The key simulation functions are:

 * `sim_recurrent`: simple simulation with one event type and death
 * `sim_recurrentII`: extended version with possibly multiple types of recurrent events (rates can be zero);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent_list`: lists allowed for multiple event types and causes of death (competing risks);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent`: simulates from Cox-Cox (marginals) or Ghosh-Lin-Cox models

Data can also be simulated from the Ghosh-Lin model where marginal rates among survivors are on Cox form:

 * `simGLcox` (also `simRecurrentCox`): simulates from the Ghosh-Lin model
   * with frailties, where the survival model for the terminal event is on Cox form
   * simulates data where rates among survivors are on Cox form, with or without frailties

see examples below for specific models. 


Utility functions
=================

Two utility functions worth noting:

 * `tie.breaker`: breaks ties among jump times, which is expected by the functions below.
 * `count_history`: counts the number of previous jumps for each subject, i.e. $N_1(t-)$ and $N_2(t-)$.

Marginal Mean 
=============

We start by estimating the marginal mean $E(N_1(t \wedge D))$ where $D$ is the time of the terminal event.
The marginal mean is the average number of events observed before time $t$.

This is based on a two-rate model for

 * the type 1 events: $E(dN_1(t) | D > t)$
 * the terminal event: $E(dN_d(t) | D > t)$

and is defined as $\mu_1(t)=E(N_1(t))$ 
\begin{align}
   \int_0^t S(u) d R_1(u)	
\end{align}
where $S(t)=P(D \geq t)$ and $dR_1(t) = E(dN_1(t) | D > t)$ 

and can therefore be estimated by a 

 * Kaplan-Meier estimator, $\hat S(u)$ 
 * Nelson-Aalen estimator for $R_1(t)$

\begin{align}
  \hat R_1(t) & =   \sum_i \int_0^t  \frac{1}{Y_\bullet (s)}  dN_{1i}(s)
\end{align}
where $Y_{\bullet}(t)= \sum_i Y_i(t)$ such that the estimator is 
\begin{align}
  \hat \mu_1(t) & =    \int_0^t \hat S(u) d\hat R_1(u),
\end{align}
see Cook & Lawless (1997) and Ghosh & Lin (2000).

The variance can be estimated based on the asymptotic expansion
of $\hat \mu_1(t) - \mu_1(t)$
\begin{align*}
  & \sum_i \int_0^t \frac{S(s)}{\pi(s)} dM_{i1}  - \mu_1(t) \int_0^t  \frac{1}{\pi(s)} dM_i^d +  \int_0^t \frac{\mu_1(s) }{\pi(s)} dM_i^d,
\end{align*}

with mean-zero processes 

 * $M_i^d(t) = N_i^D(t)- \int_0^t Y_i(s) d \Lambda^D(s)$, 
 * $M_{i1}(t) = N_{i1}(t) - \int_0^t Y_{i}(s) dR_1(s)$. 

as described in Ghosh & Lin (2000).


Generating data
==================

We start by generating some data to illustrate the computation of the marginal mean 


``` r
library(mets)
set.seed(1000) # to control output in simulations for p-values below.
```


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- sim_recurrent(400,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))
#> id: 1
#>      entry  time   status dtime fdeath death start  stop   x        strata
#> 1       0.0  451.1 1      3291  1      0        0.0  451.1 -0.39707 0     
#> 401   451.1  485.6 1      3291  1      0      451.1  485.6 -1.04577 0     
#> 679   485.6 1276.1 1      3291  1      0      485.6 1276.1  0.09424 0     
#> 867  1276.1 1278.8 1      3291  1      0     1276.1 1278.8  2.58113 0     
#> 1019 1278.8 1618.3 1      3291  1      0     1278.8 1618.3  0.00696 0     
#> 1148 1618.3 1752.9 1      3291  1      0     1618.3 1752.9 -0.81183 0     
#> 1256 1752.9 2466.3 1      3291  1      0     1752.9 2466.3 -0.52972 0     
#> 1344 2466.3 2550.8 1      3291  1      0     2466.3 2550.8  0.03179 0     
#> 1411 2550.8 3290.8 0      3291  1      1     2550.8 3290.8 -0.72717 0     
#> ------------------------------------------------------------ 
#> id: 7
#>   entry time  status dtime fdeath death start stop  x       strata
#> 7 0     658.3 0      658.3 1      1     0     658.3 -0.2209 0     
#> ------------------------------------------------------------ 
#> id: 9
#>     entry time  status dtime fdeath death start stop  x      strata
#> 9     0.0 433.5 1      505.3 1      0       0.0 433.5 0.3552 0     
#> 405 433.5 505.3 0      505.3 1      1     433.5 505.3 0.2355 0
```

The status variable keeps track of the recurrent events and their type, and death the timing of 
death.

To compute the marginal mean we simply estimate the two rates functions of the 
number of events of interest and death by using the phreg function 
(to start without covariates). Then the estimates are combined with standard 
error computation in the recurrent_marginal function


``` r
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)
```

<img src="figure/recurrent-events-unnamed-chunk-4-1.png" alt="Death rate, event rate, and marginal mean."  />

We can also extract the estimate in different time-points 


``` r
summary(out,times=c(1000,2000))
#> [[1]]
#>     new.time     mean         se  CI-2.5% CI-97.5% strata
#> 495     1000 1.857035 0.08767256 1.692911 2.037072      0
#> 779     2000 3.802964 0.14690314 3.525668 4.102068      0
```

The marginal mean can also be estimated in the stratified case:


``` r
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrent_marginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

<img src="figure/recurrent-events-unnamed-chunk-6-1.png" alt="Stratified death rate, event rate, and marginal mean."  />

We can compare different marginal mean test (IPCW based) with a log-rank test  


``` r
test_logrankRecurrent(out)
#> Call: estimate.default(coef = contr, IC = contr.iid * nrow(contr.iid))
#> ────────────────────────────────────────────────────────────
#>    Estimate Std.Err   2.5%  97.5% P-value
#> p1   10.368   14.44 -17.93 38.661 0.47264
#> p2  -22.069   12.43 -46.43  2.288 0.07575
#> p3    2.019   15.72 -28.80 32.838 0.89785
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] = 0
#>   [p2] = 0
#>   [p3] = 0 
#>  
#> chisq = 3.3471, df = 3, p-value = 0.3411

dd <- test_marginalMean(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
dd
#> coeffients:
#>                              p-value
#> time                       5079.7045
#> Pepe-Mori                         NA
#> Ratio-AUC                     0.2147
#> Proportionality               0.3865
#> Proportionality-score-test    0.3920
summary(dd)
#> coeffients:
#>                              p-value
#> time                       5079.7045
#> Pepe-Mori                         NA
#> Ratio-AUC                     0.2147
#> Proportionality               0.3865
#> Proportionality-score-test    0.3920
dd$RAUCl
#>    n events
#>  400   1184
#> 
#>  400 clusters
#> coeffients:
#>                          Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept)              24707.72  1498.20 21771.30 27644.15  0.0000
#> factor(strata__)strata=1 -3840.23  2028.83 -7816.67   136.20  0.0584
#> factor(strata__)strata=2 -1015.76  2026.74 -4988.10  2956.57  0.6162
#> factor(strata__)strata=3  -351.09  1948.38 -4169.84  3467.66  0.8570
```

The AUC suggest that strata 2 looses 3840 days less than strata 0. 


If we adjust for covariates for the two rates we can still do
predictions of marginal mean, what can be plotted is the baseline marginal mean, 
that is for the covariates equal to 0 for both models. Predictions for specific 
covariates can also be obtained with the recmarg (recurren marginal mean used 
solely for predictions without standard error computation). 


``` r
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust_phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

<img src="figure/recurrent-events-unnamed-chunk-8-1.png" alt="Cox-adjusted death rate, event rate, and marginal mean."  />

``` r

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)
```

We here simulate multiple recurrent events processes with two causes of death causes  and exponential censoring with
rate $3/5000$, all processes are assumed independent (dependence=0)


``` r
rr <- sim_recurrent_list(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
#> 
#>      status
#> death   0   1   2   3
#>     0  37 135 165  12
#>     1  54   0   0   0
#>     2   9   0   0   0
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))
```

<img src="figure/recurrent-events-unnamed-chunk-9-1.png" alt="Simulated vs true rates for multiple recurrent event types."  />

Improving efficiency 
======================

To illustrate how the efficiency can be improved using heterogenity in the data, we
now simulate some data with strong heterogenity. The dynamic augmentation is 
a regression on the history  for each subject consisting of the specified terms 
terms:  Nt, Nt2 (Nt squared), expNt (exp(-Nt)), NtexpNt (Nt*exp(-Nt)) or by simply
specifying these directly.  This was developed in Cortese and Scheike (2022).


``` r
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count_history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)
#> 
#>       statusD
#> status   0   1   2   3
#>      0  83   0   0 117
#>      1   0 375   0   0
#>      2   0   0  44   0
#> 
#>      statusD
#> death   0   1   2   3
#>     0  83 375  44   0
#>     1   0   0   0 117

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
#>       times       muP     semuP     muPAt   semuPAt          
#>  [1,]   500 0.8744004 0.1059068 0.8670372 0.1055614 0.9967382
#>  [2,]  1000 1.2696069 0.1458220 1.2621656 0.1447864 0.9928981
#>  [3,]  1500 1.7921498 0.2291966 1.7957473 0.2240253 0.9774373
#>  [4,]  2000 2.1216192 0.3047378 2.1595208 0.2937613 0.9639804
#>  [5,]  2500 2.4775306 0.3934784 2.5544338 0.3713749 0.9438254
#>  [6,]  3000 2.7947412 0.5183678 2.9101041 0.4625630 0.8923451
#>  [7,]  3500 3.0289847 0.6033927 3.1307552 0.5193403 0.8607004
#>  [8,]  4000 3.3163659 0.7183096 3.3392837 0.5769952 0.8032681
#>  [9,]  4500 3.6229966 0.8415333 3.5349425 0.6152266 0.7310781
#> [10,]  5000 3.8810150 0.9255727 3.6777784 0.6159133 0.6654402

times <- 500*(1:10)
###recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
#>       times       muP     semuP     muPAt   semuPAt          
#>  [1,]   500 0.8744004 0.1059068 0.8666135 0.1054672 0.9958484
#>  [2,]  1000 1.2696069 0.1458220 1.2643398 0.1444958 0.9909053
#>  [3,]  1500 1.7921498 0.2291966 1.8063914 0.2218548 0.9679670
#>  [4,]  2000 2.1216192 0.3047378 2.1433122 0.2888682 0.9479237
#>  [5,]  2500 2.4775306 0.3934784 2.4921659 0.3607250 0.9167593
#>  [6,]  3000 2.7947412 0.5183678 2.6962370 0.4264556 0.8226892
#>  [7,]  3500 3.0289847 0.6033927 2.7724842 0.4558824 0.7555319
#>  [8,]  4000 3.3163659 0.7183096 2.8716708 0.4752409 0.6616101
#>  [9,]  4500 3.6229966 0.8415333 2.9362144 0.4519118 0.5370100
#> [10,]  5000 3.8810150 0.9255727 2.9717225 0.3831618 0.4139726

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))
```

<img src="figure/recurrent-events-unnamed-chunk-10-1.png" alt="Augmented marginal mean estimates."  />

In the case where covariates might be important but we are still interested in the marginal mean 
we can also augment wrt these covariates 


``` r
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)
#> 
#>       statusD
#> status   0   1   3
#>      0  92   0 108
#>      1   0 529   0
#> 
#>      statusD
#> death   0   1   3
#>     0  92 314   0
#>     1   0 215 108

times <- seq(500,5000,by=500)
recEFF1x <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))
#>            muP     muPA    muPAt      semuP     semuPA    semuPAt          
#>  [1,] 1.050317 1.052415 1.048871 0.08835005 0.08801111 0.08789822 0.9948859
#>  [2,] 1.843830 1.863996 1.838716 0.14730045 0.14606950 0.14556961 0.9882496
#>  [3,] 2.714728 2.768829 2.724566 0.24272224 0.23810724 0.23756192 0.9787398
#>  [4,] 3.546522 3.636583 3.591319 0.36401457 0.35109326 0.34874650 0.9580564
#>  [5,] 4.492684 4.658458 4.537357 0.51416355 0.50043401 0.49610099 0.9648700
#>  [6,] 5.080889 5.209681 5.047836 0.60551061 0.58818767 0.58285931 0.9625914
#>  [7,] 5.638012 5.701306 5.495962 0.70065049 0.67382134 0.66728557 0.9523801
#>  [8,] 6.895109 6.830233 6.447130 0.95745240 0.89622952 0.87919085 0.9182606
#>  [9,] 7.723651 6.794796 6.727887 1.15096828 0.99740795 0.96198463 0.8358046
#> [10,] 8.923607 7.170763 7.179372 1.45783461 1.07670415 1.01334938 0.6951059

out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)
#> [[1]]
#>     new.time     mean         se   CI-2.5%  CI-97.5% strata
#> 181      500 1.050317 0.08835005 0.8906755  1.238572      0
#> 279     1000 1.843830 0.14730045 1.5765935  2.156363      0
#> 355     1500 2.714728 0.24272224 2.2783527  3.234684      0
#> 413     2000 3.546522 0.36401457 2.9002498  4.336805      0
#> 462     2500 4.492684 0.51416355 3.5899669  5.622395      0
#> 480     3000 5.080889 0.60551061 4.0225230  6.417723      0
#> 492     3500 5.638012 0.70065049 4.4192133  7.192950      0
#> 514     4000 6.895109 0.95745240 5.2522287  9.051878      0
#> 523     4500 7.723651 1.15096828 5.7673685 10.343501      0
#> 530     5000 8.923607 1.45783461 6.4785990 12.291356      0
```

Regression models for the marginal mean 
========================================

One can also do regression modelling , using the model
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
then Ghosh-Lin suggested IPCW score equations that are implemented in the recreg function of mets. 

First we generate data that from a Ghosh-Lin model with regression coefficients 
$\beta=(-0.3,0.3)$ and the baseline given by base1, this is done under the assumption that the death 
rate given covariates is on Cox form with baseline dr: 


``` r
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.39075785 0.3162028 0.10070725 0.2165394
#> X2 -0.09651179 0.3581337 0.09682266 0.7875562
summary(outs)$coef
#>      Estimate      S.E.    dU^-1/2   P-value
#> X1  0.2122279 0.2946370 0.10162550 0.4713386
#> X2 -0.1576775 0.3523691 0.09699948 0.6545299

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)
```

<img src="figure/recurrent-events-unnamed-chunk-12-1.png" alt="Ghosh-Lin regression baseline estimates."  />

We note that for the extended censoring model we gain a little efficiency and that the estimates are close to the true values. 

Also possible to do IPCW regression at fixed time-point


``` r
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
#>               Estimate   Std.Err       2.5%     97.5%      P-value
#> (Intercept)  1.1590100 0.2775235  0.6150739 1.7029462 2.963428e-05
#> X1           0.2337768 0.2783519 -0.3117830 0.7793365 4.009866e-01
#> X2          -0.2114258 0.2758703 -0.7521217 0.3292701 4.434409e-01
summary(outipcws)$coef
#>               Estimate   Std.Err       2.5%     97.5%      P-value
#> (Intercept)  1.1681460 0.2766323  0.6259567 1.7103353 2.413511e-05
#> X1           0.2312122 0.2785043 -0.3146462 0.7770705 4.064300e-01
#> X2          -0.2321638 0.2747766 -0.7707160 0.3063885 3.981565e-01
```

We can also do the Mao-Lin type composite outcome where we both count the recurrent events (cause 1) and deaths (cause 3)
for example 
\begin{align*}
E(N_1(t) + I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}






``` r
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef
#>      Estimate      S.E.    dU^-1/2   P-value
#> X1  0.3677911 0.2642911 0.09289221 0.1640393
#> X2 -0.1056515 0.2995438 0.08962337 0.7243074
```

This can also be done with competing risks death 
\begin{align*}
E(w_1 N_1(t) + w_2 I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
and with weights  $w_1,w_2$ that follow the causes, here 1 and 3. 
We modify the data by changing some of the cause 3 deaths to cause 4


``` r
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.40121614 0.2943567 0.09755943 0.1728740
#> X2 -0.09714896 0.3323313 0.09368958 0.7700377

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.41043629 0.2765368 0.09469049 0.1377555
#> X2 -0.09770279 0.3106702 0.09084228 0.7531486

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)
```

<img src="figure/recurrent-events-unnamed-chunk-15-1.png" alt="Composite outcome baseline estimates."  />

Predictions and standard errors can be computed via the iid decompositions of the baseline and the regression coefficients. We illustrate this
for the standard Ghosh-Lin model 


``` r
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
#> 
#>    n events
#>  541    441
#> 
#>  100 clusters
#> coefficients:
#>     Estimate      S.E.   dU^-1/2 P-value
#> X1  0.390758  0.316203  0.100707  0.2165
#> X2 -0.096512  0.358134  0.096823  0.7876
#> 
#> exp(coefficients):
#>    Estimate    2.5% 97.5%
#> X1  1.47810 0.79534 2.747
#> X2  0.90800 0.45003 1.832
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])
#>          pred    se-log    lower    upper
#> [1,] 3.303778 0.2373066 2.074985 5.260253
#> [2,] 3.303778 0.2373066 2.074985 5.260253
#> [3,] 3.303778 0.2373066 2.074985 5.260253
#> [4,] 4.883316 0.2753957 2.846410 8.377841
#> [5,] 4.883316 0.2753957 2.846410 8.377841
```

The Ghosh-Lin model can be made more efficient by the regression augmentation method.
First computing the augmentation and then in a second step the augmented estimator (Cortese and Scheike (2023)): 


``` r
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.16625713 0.2854224 0.09827540 0.5602333
#> X2 -0.06801445 0.3288907 0.09702398 0.8361664
```

We note that the simple augmentation improves the standard errors as expected. The data was generated assuming independence with
previous number of events so it would suffice to augment only with the covariates. 



Administrative censoring for the Ghosh-Lin model
===============================================

In the case of administative censoring with possible additional random censorering we can fit the Ghosh-Lin model using 
risk-set adjustment for the administrative censorering and IPCW adjustment for the  random cenouring. We illustrate it using 
simulated data from the two-stage model described below. 
The advantage of this procedure is that we do not rely on any modelling assumptions for the administrative censoring that will
often be the important part of the total censoring. 



``` r
library(mets)
rho1 <- 1; rho2 <- 0.5
rate <- c(1,1)
tt <- seq(0, 6, by = 0.01)
base1 <- cbind(tt,rho1 * (1 - exp(-tt/rate[1])))
drcumhaz <- cbind(tt,rho2 * (1 - exp(-tt/rate[2])))
base13 <- cpred(base1,c(1,3))[,2]

dats <- mets:::sim_GLRA(100,base1,drcumhaz,varz=1)
datsA <- dats[[1]]
dtable(datsA,~statusD)
#> 
#> statusD
#>  1  3  7 
#> 88 35 65
datsRA <- dats[[2]]
dtable(datsRA,~statusD)
#> 
#> statusD
#>  0  1  3  7 
#> 23 73 32 45
```

We get two data-sets, one with only administrative censoring datsA 
and one with additional random censoring datsRA. 
 
  - datsA: statusD has events (1) death (3) and administrative censorings at the  time censorA
  - datsRA: statusD has events (1) death (3),  administrative censorings (7) and random censorings (0). The time 
     of the administriative censoring is given by censorA

First we deal only  with the administriative censoring. 


``` r
   ## handling admin censoring with IPCW 
    outRR  <- recreg(Event(start,stop,statusD)~Z1+Z2+cluster(id), datsA, cause = 1, 
		       cens.code = c(7),death.code=3)
    estimate(outRR)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1   0.2244  0.1918 -0.1515 0.6004  0.2420
#> Z2  -0.2565  0.3637 -0.9694 0.4565  0.4808

    ## Full Adm-censoring statusA, timeA
    out0  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsA, cause = 1, 
		       cens.code = 9, death.code=3, adm.cens.time=datsA$censorA)
    estimate(out0)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1   0.2501  0.1930 -0.1283 0.6284  0.1952
#> Z2  -0.1517  0.3636 -0.8642 0.5609  0.6765
```

The additional random censoring can be handled by combining the two censoring times, or by handling the 
administrative censoring (via risk-set adjustment) and random censoring (via IPCW) separately


``` r
    ## std Right censoring  on combined censoring time 
    outR  <- recreg(Event(start,stop,statusD)~Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = c(0,7),death.code=3)
    estimate(outR)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3036  0.1724 -0.03434 0.6415 0.07827
#> Z2  -0.2023  0.3457 -0.87997 0.4753 0.55839

    ## Combined R-IPCW + Adm-censoring status, time
    out1  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA)
    estimate(out1)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3284  0.1732 -0.01105 0.6679 0.05794
#> Z2  -0.1040  0.3418 -0.77397 0.5660 0.76105

    ## Combined R-IPCW + Adm-censoring status + censoring modelling time
    out1c  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		     cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA,cens.model=~strata(Z1,Z2))
     estimate(out1c)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3468  0.1741  0.00545 0.6881 0.04645
#> Z2  -0.1024  0.3417 -0.77220 0.5674 0.76440
```

We have influence functions of all parameters and can thus also make predictions with confindence intervals

 - this is based on going through a grid of time-points if se=1
   - when se=0 the predictions are done for all jump-times 


``` r
predR0 <- predict(outR,data.frame(Z1=0:1,Z2=0))
plot(predR0,ylim=c(0,1.5))

predR <- predict(outR,data.frame(Z1=0:1,Z2=0),se=1)
plot(predR,se=1,add=TRUE)

pred1c <- predict(out1c,data.frame(Z1=0:1,Z2=0),se=1)
plot(pred1c,se=1,add=TRUE)
```

<img src="figure/recurrent-events-unnamed-chunk-21-1.png" alt="Ghosh-Lin predictions with confidence intervals."  />

The censoring weights for the random censoring and the combined censoring


``` r
km07 <- km(Event(start,stop,statusD %in% c(0,7))~strata(Z1,Z2),datsRA)
km0 <- km(Event(start,stop,statusD==0)~strata(Z1,Z2),datsRA)
plot(km07,col=1)
plot(km0,add=TRUE,col=2,lwd=2)
```

<img src="figure/recurrent-events-unnamed-chunk-22-1.png" alt="Censoring weight estimates by strata."  />


Two-stage modelling 
====================

We simulate data with a terminal event on Cox form and recurrent events
satisfying the Ghosh-Lin model or having a rate on Cox form.

 - type=3 is Ghosh-Lin model for recurrent events and Cox for terminal event.
 - type=2 is Cox model for recurrent events among survivors and Cox for terminal event.
 - simulations based on time-grid to make linear approximations of cumulative hazards

Now we fit the two-stage model (the recreg must be called with twostage=TRUE)


``` r
set.seed(100)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,cens=1/5000,type=3) 
dtable(rr,~statusD)
#> 
#> statusD
#>   0   1   3 
#>  55 375 145
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
#> Cox(recurrent)-Cox(terminal) intensity model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.14651 0.14126 0.86964 1.42339       0
#> 
#> var,shared:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.14651 0.14126 0.86964 1.42339       0
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
#> Cox(recurrent)-Cox(terminal) intensity model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.07333 0.14096 0.79707 1.34960       0
#> share1       0.67344 0.12832 0.42193 0.92495       0
#> 
#> var,shared:
#>             Estimate    2.5%  97.5%
#> dependence1  1.07333 0.79707 1.3496
#> share1       0.67344 0.42193 0.9250
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
#> Ghosh-Lin(recurrent)-Cox(terminal) mean model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.04401 0.08988 0.86785 1.22017       0
#> 
#> var,shared:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.04401 0.08988 0.86785 1.22017       0
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
#> Ghosh-Lin(recurrent)-Cox(terminal) mean model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> dependence1 1.192629 0.094129 1.008140 1.377117       0
#> share1      1.643844 0.176676 1.297566 1.990121       0
#> 
#> var,shared:
#>             Estimate   2.5%  97.5%
#> dependence1   1.1926 1.0081 1.3771
#> share1        1.6438 1.2976 1.9901
glout$gradient
#> [1] -1.823718e-08  2.506173e-08
```

Standard errors are computed assuming that the parameters of `out` and `outs` are
both known, and are therefore probably a little too small. A bootstrap could be used to
obtain more reliable standard errors.


Simulations with specific structure
===================================

The function `simGLcox` can simulate data where the recurrent process has a mean on Ghosh-Lin form. The key identity is that
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta) = \int_0^t S(t|X,Z) dR(t|X,Z)
\end{align*}
where $Z$ is a possible frailty, so that
\begin{align*}
 R(t|X,Z) & = \frac{Z \Lambda_0(t)  \exp(X^T \beta) }{S(t|X,Z)}
\end{align*}
leads to a Ghosh-Lin model. The survival model can be specified to have Cox form among survivors via
`model="twostage"`; otherwise `model="frailty"` uses a survival model with rate $Z \lambda_d(t) r_d$.
The frailty $Z$ is gamma distributed with a variance that can be specified. Simulations are based on
a piecewise linear approximation of the hazard functions for $S(t|X,Z)$ and $R(t|X,Z)$.



``` r
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
```

We can also simulate from models where the terminal event is on Cox form and the rate among survivors is on Cox form. 

 * $E(dN_1 | D>t, X) = \lambda_1(t) r_1$
 * $E(dN_d | D>t, X) = \lambda_d(t) r_d$

underlying these models we have a shared frailty model


``` r
rr <- mets:::sim_GLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> X1   0.2535  0.2553 -0.2469 0.7538  0.3208
#> X2  -0.2595  0.2637 -0.7764 0.2574  0.3252
estimate(recurrent)
#>    Estimate Std.Err     2.5%   97.5% P-value
#> X1   0.6675  0.3019  0.07583 1.25921 0.02702
#> X2  -0.4356  0.2637 -0.95247 0.08135 0.09864
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)
```

<img src="figure/recurrent-events-unnamed-chunk-25-1.png" alt="Cox rates for survivors vs true baselines."  />

We can also simulate data with underlying dependence from the two-stage model (`simGLcox`) or using
`simRecurrent` random effects models, for Cox-Cox or Ghosh-Lin-Cox models.

Here with marginals on Cox-Cox and Ghosh-Lin-Cox form, drawing covariates from data:


``` r
simcoxcox <- sim_recurrent_ts(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim_recurrent_ts(recurrentGL,margsurv,n=10,data=rr)
```


Other marginal properties
=========================

The mean is a useful summary measure, but it is also easy and informative to examine other
simple summary measures such as the probability of exceeding $k$ events,

 * $P(N_1^*(t) \ge k)$
   * cumulative incidence of $T_{k} = \inf \{ t: N_1^*(t)=k \}$ with competing event $D$.

This is equivalent to a cumulative incidence of $T_k$ occurring before $D$, denoted $\hat F_k(t)$.

We note also that $N_1^*(t)^2$ can be written as
\begin{align*}
   \sum_{k=0}^K  \int_0^t I(D > s) I(N_1^*(s-)=k) f(k) dN_1^*(s)
\end{align*}
with $f(k)=(k+1)^2 - k^2$, such that its mean can be written as 
\begin{align*}
	\sum_{k=0}^K \int_0^t S(s) f(k) P(N_1^*(s-)= k  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k, D> s) 
\end{align*}
and estimated by
\begin{align*}
\tilde \mu_{1,2}(t) & = 
	\sum_{k=0}^K \int_0^t \hat S(s) f(k) 
	\frac{Y_{1\bullet}^k(s)}{Y_\bullet (s)} \frac{1}{Y_{1\bullet}^k(s)} d N_{1\bullet}^k(s)= \sum_{i=1}^n \int_0^t \hat S(s) f(N_{i1}(s-)) \frac{1}{Y_\bullet (s)} d N_{i1}(s),
\end{align*}
That is very similar to the "product-limit" estimator for $E( (N_1^*(t))^2 )$ 
\begin{align}
  \hat \mu_{1,2}(t) & =    \sum_{k=0}^K k^2 ( \hat F_{k}(t) - \hat F_{k+1}(t) ).
\end{align}

We use the estimator of the probability of exceeding $k$ events based on the fact that
$I(N_1^*(t) \geq k)$ is  equivalent to 
\begin{align*}
	\int_0^t I(D > s) I(N_1^*(s-)=k-1) dN_1^*(s),
\end{align*}
suggesting that its mean can be computed as
\begin{align*}
\int_0^t S(s) P(N_1^*(s-)= k-1  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k-1, D> s) 
\end{align*}
and estimated by 
\begin{align*}
\tilde F_k(t) = \int_0^t \hat S(s)  \frac{Y_{1\bullet}^{k-1}(s)}{Y_\bullet (s)} 
          	\frac{1}{Y_{1\bullet}^{k-1}(s)} d N_{1\bullet}^{k-1}(s).
\end{align*}

To compute these estimators we use the prob.exceed.recurrent function


``` r
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count_history(rr)
dtable(rr,~statusD)
#> 
#> statusD
#>   0   1   2   3 
#>  93 287  32 107

oo <- prob_exceed_recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)
```

<img src="figure/recurrent-events-unnamed-chunk-27-1.png" alt="Probability of exceeding k events over time."  />

We can also look at the mean and variance based on the estimators just described 


``` r
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))
```

<img src="figure/recurrent-events-unnamed-chunk-28-1.png" alt="Mean and variance of recurrent events process."  />


Multiple events
================

We now generate recurrent events data with two event types, starting with
independent events.


``` r
rr <- sim_recurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count_history(rr)
dtable(rr,~death+status)
#> 
#>       status   0   1   2
#> death                   
#> 0             32 651  84
#> 1            168   0   0
```

Based on this we can estimate also the joint distribution function, that is
the probability that $(N_1(t) \geq k_1, N_2(t) \geq k_2)$


``` r
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)
```

Looking at simulations with dependence
=============================================

Using normally distributed random effects we illustrate four settings. We set variance $0.5$ for all
random effects and vary the correlation. We denote the correlation between the random effect associated with
$N_1$ and $N_2$ as $\rho_{12}$, and the correlation between the random effects
associated with $N_j$ and $D$ (the terminal event) as $\rho_{j3}$, organised in the vector $\rho=(\rho_{12},\rho_{13},\rho_{23})$.

 * Scenario I: $\rho=(0,0.0,0.0)$ — independence among all effects.


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

  par(mfrow=c(1,3))
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  cor(attr(rr,"z"))
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  plot(coo,main ="Scenario I")
```
 * Scenario II: $\rho=(0,0.5,0.5)$ — independence among survivors but dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main ="Scenario II")
```

 * Scenario III: $\rho=(0.5,0.5,0.5)$ — positive dependence among survivors and dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # positive dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario III")
```

 * Scenario IV: $\rho=(-0.4,0.5,0.5)$ — negative dependence among survivors and positive dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # negative dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario IV")
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              KernSmooth_2.23-26     otel_0.2.0            
#>  [7] future.apply_1.20.2    listenv_0.10.1         lava_1.9.2.1          
#> [10] stats4_4.6.0           grid_4.6.0             evaluate_1.0.5        
#> [13] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [16] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [19] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [22] digest_0.6.39          parallelly_1.47.0      parallel_4.6.0        
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Two-Stage Randomization for Competing risks and Survival outcomes"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Two-Stage Randomization for Competing risks and Survival outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Two-Stage Randomization for Competing risks and Survival outcomes
=====================================================================

Under two-stage randomization we can estimate the average treatment effect $E(Y(i,\bar k))$ of treatment regime $(i,\bar k)$.

   - treatment $A_0=i$
and
     - for all responses, randomization $A_1 = (k_1)$, so treatment $k_1$
     - response$\times A_1 = (k_1, k_2)$, so treatment $k_1$ if response 1, and treatment $k_2$ if response 2.

The estimator can be augmented in different ways: using the two randomizations and the dynamic censoring augmentation.

Estimating  $\mu_{i,\bar k} = P(Y(i,\bar k,\epsilon=v) <= t)$, restricted mean
$E( \min(Y(i,\bar k),\tau))$  or years lost $E( I(\epsilon=v) \cdot (\tau  -  \min(Y(i,\bar k),\tau)))$ 
using IPCW weighted estimating equations : \\

 The solved estimating equation is 
 \begin{align*}
 \sum_i  \frac{I(min(T_i,t) < G_i)}{G_c(min(T_i ,t))} I(T \leq t, \epsilon=1 ) - AUG_0 - AUG_1 + AUG_C  -  p(i,j)) = 0 
 \end{align*}
 using the covariates from augmentR0 to augment with 
 \begin{align*}
 AUG_0 = \frac{A_0(i) - \pi_0(i)}{ \pi_0(i)} X_0 \gamma_0
 \end{align*}
 and  using the covariates from augmentR1 to augment with 
 \begin{align*}
 AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{ \pi_1(j)} X_1 \gamma_1
 \end{align*}
 and  censoring augmenting  with 
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s) 
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimize the variance given the dynamic covariates specified by augmentC.


  - Treatments must be given as factors.
  - Treatment for the 2nd randomization may depend on response.
      - Treatment probabilities are estimated by default and uncertainty from this is adjusted for.
  - Randomization augmentation for the 1st and 2nd randomization is possible.
  - Censoring model can be stratified on observed covariates (at time 0).
  - Censoring augmentation is done dynamically over time with time-dependent covariates.

Standard errors are estimated using the influence functions of all estimators; tests of differences can therefore be computed subsequently.

Data must be in start-stop-status survival format with

 - one code of `status` indicating response, i.e. 2nd randomization
 - other codes defining the outcome of interest


```{r}
library(mets) 
set.seed(100)

n <- 200
ddf <- mets:::gsim(n,covs=1,null=0,cens=1,ce=1,betac=c(0.3,1))
true <- apply(ddf$TTt<2,2,mean)
true
datat <- ddf$datat
## set-random response on data, only relevant after status==2 
response <- rbinom(n,1,0.5)
datat$response <- as.factor(response[datat$id]*datat$Count2)
datat$A000 <- as.factor(1)
datat$A111 <- as.factor(1)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f,
		augmentR1=~X11+X12+TR, augmentR0=~X01+X02,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f))
bb

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]/p[2],p[3]/p[4]))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]-p[2],p[3]-p[4]))

```


```{r}

## 2 levels for each response , fixed weights 
datat$response.f <- as.factor(datat$response)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(0,0),pi0=0.5,pi1=0.5)
bb

## 2 levels for each response ,  estimated treat probabilities
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb


## 2 and 3 levels for each response , fixed weights 
datat$A1.23.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.23.f+response)
datat <- dtransform(datat,A1.23.f=2+rbinom(nrow(datat),1,0.5),
		    Count2==1 & A1.23.f==2 & response==0)
dtable(datat,~A1.23.f+response)
datat$A1.23.f <- as.factor(datat$A1.23.f)
dtable(datat,~A1.23.f+response|Count2==1)
###
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1,treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
        estpr=c(1,0),pi1=c(0.3,0.5))
bb


## 2 and 3 levels for each response , estimated 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb


## 2 and 1 level for each response 
datat$A1.21.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.21.f+response|Count2==1)
datat <- dtransform(datat,A1.21.f=1,Count2==1 & response==1)
dtable(datat,~A1.21.f+response|Count2==1)
datat$A1.21.f <- as.factor(datat$A1.21.f)
dtable(datat,~A1.21.f+response|Count2==1)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb

## known weights 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,
	cens.model=~strata(A0.f),estpr=c(1,0),pi1=c(0.5,1))
bb
```

Two-Stage Randomization CALGB-9823
==================================

We illustrate an analysis of one SMART conducted by Cancer and
Leukemia Group B Protocol 8923 (CALGB 8923), Stone et al. (2001).
388 patients were randomized to an initial treatment of GM-CSF ($A_1$) or standard chemotherapy
($A_2$). Patients with complete remission and informed consent to the second stage were then re-randomized to
cytarabine only ($B_1$) or cytarabine plus mitoxantrone ($B_2$).

We first compute the weighted risk-set estimator based on estimated weights
\begin{align*}
\Lambda_{A1,B1}(t) & = \sum_i \int_0^t \frac{w_i(s)}{Y^w(s)} dN_i(s)
\end{align*}
where $w_i(s) = I(A0_i=A1) + I(t>T_R) I(A1_i=B1)/\pi_1(X_i)$, that is
1 when starting on treatment $A1$, and then scaled up
by the proportion switching to $B1$ at time $T_R$ for those that do so. This is equivalent to
the IPTW (inverse probability of treatment weighted) estimator. We estimate
the treatment regimes $A1, B1$ and $A2, B1$ by letting $A10$ indicate those consistent with ending on $B1$.
$A10$ starts at 1 and becomes 0 if the subject is treated with $B2$, but stays 1 if treated with $B1$.
We can then look at the two strata where $A0=0, A10=1$ and $A0=1, A10=1$. Similarly, we define $A11$ to start at 1,
remain 1 if $B2$ is taken, and become 0 if the second randomization is $B1$.

  - the treatment models apply to all time-points, unless the `weight.var` variable is given (1 for treatments, 0 otherwise) to accommodate a general start-stop format
  - the treatment model may also depend on a response value
  - standard errors are based on influence functions and are also computed for the baseline

We here use the propensity score model $P(A1=B1|A0)$ that uses the 
observed frequencies on arm $B1$ among those starting out on either $A1$ or $A2$.

```{r}
data(calgb8923)
calgt <- calgb8923

tm=At.f~factor(Count2)+age+sex+wbc
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=12)
summary(pll0,times=12)
```

The propensity score model can be extended to use covariates to get increased efficiency.
Note also that the propensity scores for $A0$ will cancel out in the different strata. 



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Haplotype Discrete Survival Models"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Haplotype Discrete Survival Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




Haplotype Analysis for discrete TTP
===================================

Cycle-specific logistic regression of haplotype effects with
known haplotype probabilities. Given observed genotype $G$ and
unobserved haplotypes $H$, we marginalise over the possible
haplotypes using $P(H|G)$ supplied as input.

\begin{align*}
   S(t|x,G) & = E( S(t|x,H) | G)  = \sum_{h \in G} P(h|G) S(t|z,h)
\end{align*}
so survival can be computed by marginalising over possible $h$ given $g$.

Survival is based on logistic regression for the discrete hazard
function of the form
\begin{align*}
      \mbox{logit}(P(T=t \mid T \geq t, x, h)) & = \alpha_t + x(h)^T \beta
\end{align*}
where $x(h)$ is a regression design of $x$ and haplotypes $h=(h_1,h_2)$.

Simple binomial data can also be fitted using this function.

Standard errors are computed assuming that the haplotype probabilities are known.

	
We are particularly interested in the types haplotypes: 

``` r
types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")

## some haplotypes frequencies for simulations 
data(haplo)
hapfreqs <- haplo$hapfreqs 
print(hapfreqs)
#>             index   haplotype     freq
#> DCGAGCTCACG     1 DCGAGCTCACG 0.010681
#> DCGCGCTCACG     2 DCGCGCTCACG 0.138387
#> DTGAGCTCACG     3 DTGAGCTCACG 0.000310
#> DTGAGCTCACA     4 DTGAGCTCACA 0.006800
#> DTGAGCTCGCG     5 DTGAGCTCGCG 0.034517
#> DTGACCTCACG     6 DTGACCTCACG 0.001336
#> DTGCGCTCACG     7 DTGCGCTCACG 0.009969
#> DTGCGCTCACA     8 DTGCGCTCACA 0.011833
#> DTGCGCTCGCG     9 DTGCGCTCGCG 0.302389
#> DTGCGCCCGCG    10 DTGCGCCCGCG 0.001604
#> DTGCCCTCACG    11 DTGCCCTCACG 0.003912
#> DTCAGCTGACG    12 DTCAGCTGACG 0.001855
#> DTCCGCTGACG    13 DTCCGCTGACG 0.103394
#> DTCCCCTGACG    14 DTCCCCTGACG 0.000310
#> ITCAGTTGACG    15 ITCAGTTGACG 0.048124
#> ITCCGCTGAGG    16 ITCCGCTGAGG 0.291273
#> ITCCGTTGACG    17 ITCCGTTGACG 0.031089
#> ITCCGTCGACG    18 ITCCGTCGACG 0.001502
#> ITCCCCTGAGG    19 ITCCCCTGAGG 0.000653
```

Among the haplotypes of interest we look up the frequencies and choose a baseline:


``` r
www <-which(hapfreqs$haplotype %in% types)
hapfreqs$freq[www]
#> [1] 0.138387 0.103394 0.048124 0.291273

baseline=hapfreqs$haplotype[9]
baseline
#> [1] "DTGCGCTCGCG"
```


We have cycle-specific data with $id$ and outcome $y$:


``` r
haploX  <- haplo$haploX
dlist(haploX,.~id|id %in% c(1,4,7))
#> id: 1
#>   y X1 X2 X3 X4 times lbnr__id Count1
#> 1 0 0  0  0  0  1     1        0     
#> 2 0 0  0  0  0  2     2        0     
#> 3 0 0  0  0  0  3     3        0     
#> 4 0 0  0  0  0  4     4        0     
#> 5 0 0  0  0  0  5     5        0     
#> 6 0 0  0  0  0  6     6        0     
#> ------------------------------------------------------------ 
#> id: 4
#>    y X1 X2 X3 X4 times lbnr__id Count1
#> 19 1 0  0  0  0  1     1        0     
#> ------------------------------------------------------------ 
#> id: 7
#>    y X1 X2 X3 X4 times lbnr__id Count1
#> 37 0 1  0  0  0  1     1        0     
#> 38 0 1  0  0  0  2     2        0     
#> 39 1 1  0  0  0  3     3        0
```
and a list of possible haplotypes for each id and their probabilities $p$
(the probabilities within each id sum to 1):


``` r
ghaplos <- haplo$ghaplos
head(ghaplos)
#>    id      haplo1      haplo2          p
#> 1   1 DTGCGCTCGCG DTGAGCTCGCG 1.00000000
#> 19  2 ITCCGTTGACG DTGAGCTCGCG 0.06867716
#> 21  2 ITCAGTTGACG DTGCGCTCGCG 0.93132284
#> 51  3 ITCCGTTGACG DTGAGCTCGCG 0.06867716
#> 53  3 ITCAGTTGACG DTGCGCTCGCG 0.93132284
#> 66  4 DTGCGCTCGCG DTGCGCTCGCG 1.00000000
```

The first id=1 has the haplotype fully observed, but id=2 has two possible haplotypes consistent with the
observed genotype for this id, the probabilities are 7\% and 93\%, respectively. 

With the baseline given above we can specify a regression design that gives an effect if a haplotype of the specified type
is present (`sm=0`), or an additive effect of haplotypes (`sm=1`):


``` r
designftypes <- function(x,sm=0) {
hap1=x[1]
hap2=x[2]
if (sm==0) y <- 1*( (hap1==types) | (hap2==types))
if (sm==1) y <- 1*(hap1==types) + 1*(hap2==types)
return(y)
}

```

To fit the model we first construct a time design matrix (named `X`)
and supply the haplotype distributions for each id:


``` r
haploX$time <- haploX$times
Xdes <- model.matrix(~factor(time),haploX)
colnames(Xdes) <- paste("X",1:ncol(Xdes),sep="")
X <- dkeep(haploX,~id+y+time)
X <- cbind(X,Xdes)
Haplos <- dkeep(ghaplos,~id+"haplo*"+p)
desnames=paste("X",1:6,sep="")   # six X's related to 6 cycles 
head(X)
#>   id y time X1 X2 X3 X4 X5 X6
#> 1  1 0    1  1  0  0  0  0  0
#> 2  1 0    2  1  1  0  0  0  0
#> 3  1 0    3  1  0  1  0  0  0
#> 4  1 0    4  1  0  0  1  0  0
#> 5  1 0    5  1  0  0  0  1  0
#> 6  1 0    6  1  0  0  0  0  1
```

We can now fit the model with the design given by the design function:

``` r
out <- haplo_surv_discrete(X=X,y="y",time.name="time",
      Haplos=Haplos,desnames=desnames,designfunc=designftypes) 
names(out$coef) <- c(desnames,types)
out$coef
#>          X1          X2          X3          X4          X5          X6 
#> -1.82153345 -0.61608261 -0.17143057 -1.27152045 -0.28635976 -0.19349091 
#> DCGCGCTCACG DTCCGCTGACG ITCAGTTGACG ITCCGCTGAGG 
#>  0.79753613  0.65747412  0.06119231  0.31666905
summary(out)
#>             Estimate Std.Err     2.5%   97.5%   P-value
#> X1          -1.82153  0.1619 -2.13892 -1.5041 2.355e-29
#> X2          -0.61608  0.1895 -0.98748 -0.2447 1.149e-03
#> X3          -0.17143  0.1799 -0.52398  0.1811 3.406e-01
#> X4          -1.27152  0.2631 -1.78719 -0.7559 1.346e-06
#> X5          -0.28636  0.2030 -0.68425  0.1115 1.584e-01
#> X6          -0.19349  0.2134 -0.61184  0.2249 3.647e-01
#> DCGCGCTCACG  0.79754  0.1494  0.50465  1.0904 9.445e-08
#> DTCCGCTGACG  0.65747  0.1621  0.33971  0.9752 5.007e-05
#> ITCAGTTGACG  0.06119  0.2145 -0.35931  0.4817 7.755e-01
#> ITCCGCTGAGG  0.31667  0.1361  0.04989  0.5834 1.999e-02
```

Haplotypes `"DCGCGCTCACG"` and `"DTCCGCTGACG"` give an increased hazard of pregnancy.

The data were generated with these true coefficients:


``` r
tcoef=c(-1.93110204,-0.47531630,-0.04118204,-1.57872602,-0.22176426,-0.13836416,
0.88830288,0.60756224,0.39802821,0.32706859)

cbind(out$coef,tcoef)
#>                               tcoef
#> X1          -1.82153345 -1.93110204
#> X2          -0.61608261 -0.47531630
#> X3          -0.17143057 -0.04118204
#> X4          -1.27152045 -1.57872602
#> X5          -0.28635976 -0.22176426
#> X6          -0.19349091 -0.13836416
#> DCGCGCTCACG  0.79753613  0.88830288
#> DTCCGCTGACG  0.65747412  0.60756224
#> ITCAGTTGACG  0.06119231  0.39802821
#> ITCCGCTGAGG  0.31666905  0.32706859
```

The fitted design matrix can be found in the output:

``` r
head(out$X,10)
#>    X1 X2 X3 X4 X5 X6 haplo1 haplo2 haplo3 haplo4
#> 1   1  0  0  0  0  0      0      0      0      0
#> 2   1  1  0  0  0  0      0      0      0      0
#> 3   1  0  1  0  0  0      0      0      0      0
#> 4   1  0  0  1  0  0      0      0      0      0
#> 5   1  0  0  0  1  0      0      0      0      0
#> 6   1  0  0  0  0  1      0      0      0      0
#> 8   1  0  0  0  0  0      0      0      1      0
#> 10  1  1  0  0  0  0      0      0      1      0
#> 12  1  0  1  0  0  0      0      0      1      0
#> 14  1  0  0  1  0  0      0      0      1      0
```


SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
# YAML header created by ox-ravel
title: Twin models
author: Klaus Holst & Thomas Scheike
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Twin models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->

This document provides a brief tutorial to analyzing twin data using the
**`mets`** package:



$$
\newcommand{\cov}{\mathbb{C}\text{ov}}
\newcommand{\cor}{\mathbb{C}\text{or}}
\newcommand{\var}{\mathbb{V}\text{ar}}
\newcommand{\E}{\mathbb{E}}
\newcommand{\unitfrac}[2]{#1/#2}
\newcommand{\n}{}
$$

 <!-- 
 The development version may be installed from *github*: 
 -->



# Twin analysis, continuous traits

  In the following we examine the heritability of Body Mass
Index\n{}<sup id="b71edfd9bc946c317f4a732845bcaf93"><a href="#korkeila_bmi_1991" title="Korkeila, Kaprio, Rissanen \&amp; Koskenvuo, {{E}ffects of gender and age on the heritability of body mass index}, {Int J Obes}, v(10), 647--654 (1991).">korkeila_bmi_1991</a></sup> <sup id="718839fcb6ade82ebb2d7de853582b80"><a href="#hjelmborg_bmi_2008" title="Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen \&amp; Kaprio, {{G}enetic influences on growth traits of {B}{M}{I}: a longitudinal study of adult twins}, {Obesity (Silver Spring)}, v(4), 847--852 (2008).">hjelmborg_bmi_2008</a></sup>, based on data
on self-reported BMI-values from a random sample of 11,411 same-sex
twins. First, we will load data


``` r
library(mets)
data("twinbmi")
head(twinbmi)
#>   tvparnr      bmi      age gender zyg id num
#> 1       1 26.33289 57.51212   male  DZ  1   1
#> 2       1 25.46939 57.51212   male  DZ  1   2
#> 3       2 28.65014 56.62696   male  MZ  2   1
#> 5       3 28.40909 57.73097   male  DZ  3   1
#> 7       4 27.25089 53.68683   male  DZ  4   1
#> 8       4 28.07504 53.68683   male  DZ  4   2
```

The data is on *long* format with one subject per row.

-   **`tvparnr`:** twin id
-   **`bmi`:** Body Mass Index (\(\mathrm{kg}/{\mathrm{m}^2}\))
-   **`age`:** Age (years)
-   **`gender`:** Gender factor (male,female)
-   **`zyg`:** zygosity (MZ, DZ)

We transpose the data allowing us to do pairwise analyses


``` r
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
head(twinwide)
#>    tvparnr     bmi1      age gender zyg id num     bmi2
#> 1        1 26.33289 57.51212   male  DZ  1   1 25.46939
#> 3        2 28.65014 56.62696   male  MZ  2   1       NA
#> 5        3 28.40909 57.73097   male  DZ  3   1       NA
#> 7        4 27.25089 53.68683   male  DZ  4   1 28.07504
#> 9        5 27.77778 52.55838   male  DZ  5   1       NA
#> 11       6 28.04282 52.52231   male  DZ  6   1 22.30936
```

Next we plot the association within each zygosity group

We show the log-transformed data here, which is slightly more symmetric
and more appropriate for the twin analysis (see Figure \@ref(fig:scatter1) and \@ref(fig:scatter2)).


``` r
mz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
plot_twin(mz)
```

![Scatter plot of logarithmic BMI measurements in MZ twins](figure/quantitative-twin-scatter1-1.png)


``` r
dz <- log(subset(twinwide, zyg=="DZ")[,c("bmi1","bmi2")])
plot_twin(dz)
```

![Scatter plot of logarithmic BMI measurements in DZ twins](figure/quantitative-twin-scatter2-1.png)

The plots and raw association measures show considerably stronger
dependence in the MZ twins, indicating genetic influence on the trait.


``` r
cor.test(mz[,1],mz[,2], method="spearman")
#> 
#> 	Spearman's rank correlation rho
#> 
#> data:  mz[, 1] and mz[, 2]
#> S = 165457624, p-value < 2.2e-16
#> alternative hypothesis: true rho is not equal to 0
#> sample estimates:
#>       rho 
#> 0.6956209
```


``` r
cor.test(dz[,1],dz[,2], method="spearman")
#> 
#> 	Spearman's rank correlation rho
#> 
#> data:  dz[, 1] and dz[, 2]
#> S = 2162514570, p-value < 2.2e-16
#> alternative hypothesis: true rho is not equal to 0
#> sample estimates:
#>       rho 
#> 0.4012686
```

Next we examine the marginal distribution (GEE model with working independence):


``` r
l0 <- lm(bmi ~ gender + I(age-40), data=twinbmi)
estimate(l0, id=twinbmi$tvparnr)
#>             Estimate  Std.Err    2.5%   97.5%    P-value
#> (Intercept)  23.3687 0.054534 23.2618 23.4756  0.000e+00
#> gendermale    1.4077 0.073216  1.2642  1.5512  2.230e-82
#> I(age - 40)   0.1177 0.004787  0.1083  0.1271 1.499e-133
```


``` r
library("splines")
l1 <- lm(bmi ~ gender*ns(age,3), data=twinbmi)
marg1 <- estimate(l1, id=twinbmi$tvparnr)
```


``` r
dm <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("male"),
	    age=seq(33,61,length.out=50))
df <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("female"),
	    age=seq(33,61,length.out=50))

plot(marg1, function(p) model.matrix(l1,data=dm)%*%p,
     data=dm["age"], ylab="BMI", xlab="Age",
     ylim=c(22,26.5))
plot(marg1, function(p) model.matrix(l1,data=df)%*%p,
     data=df["age"], col="red", add=TRUE)
legend("bottomright", c("Male","Female"),
       col=c("black","red"), lty=1, bty="n")
```

![Marginal association between BMI and Age for males and females.](figure/quantitative-twin-marg1-1.png)


## Polygenic model

We can decompose the trait into the following variance components

\begin{align*}
Y_i = A_i + D_i + C + E_i, \quad i=1,2
 \end{align*}

-   **\(A\):** Additive genetic effects of alleles
-   **\(D\):** Dominant genetic effects of alleles
-   **\(C\):** Shared environmental effects
-   **\(E\):** Unique environmental effects

Dissimilarity of MZ twins arises from unshared environmental effects
only, \(\cor(E_1,E_2)=0\) and

\begin{align*}
\cor(A_1^{MZ},A_2^{MZ}) = 1, \quad
\cor(D_1^{MZ},D_2^{MZ}) = 1,
\end{align*}

\begin{align*}
\cor(A_1^{DZ},A_2^{DZ}) = 0.5, \quad
\cor(D_1^{DZ},D_2^{DZ}) = 0.25,
\end{align*}

\begin{align*}
Y_i = A_i + C_i + D_i + E_i
\end{align*}

\begin{align*}
A_i \sim\mathcal{N}(0,\sigma_A^2), C_i
\sim\mathcal{N}(0,\sigma_C^2), D_i
\sim\mathcal{N}(0,\sigma_D^2),
E_i \sim\mathcal{N}(0,\sigma_E^2)
\end{align*}

\begin{gather*}
    \cov(Y_{1},Y_{2}) = \\
    \begin{pmatrix}
      \sigma_A^2 & 2\Phi\sigma_A^2 \\
      2\Phi\sigma_A^2 & \sigma_A^2
    \end{pmatrix} +
    \begin{pmatrix}
      \sigma_C^2 & \sigma_C^2 \\
      \sigma_C^2 & \sigma_C^2
  \end{pmatrix} +
    \begin{pmatrix}
      \sigma_D^2 & \Delta_{7}\sigma_D^2 \\
      \Delta_{7}\sigma_D^2 & \sigma_D^2
  \end{pmatrix} +
  \begin{pmatrix}
    \sigma_E^2 & 0 \\
    0 & \sigma_E^2
  \end{pmatrix}
\end{gather*}



``` r
dd <- na.omit(twinbmi)
```

Saturated model (different marginals in MZ and DZ twins and different marginals
for twin 1 and twin 2):

``` r
l0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="sat")
```

Different marginals for MZ and DZ twins (but same marginals within a pair)

``` r
lf <- twinlm(bmi ~ age+gender, data=dd,DZ="DZ", zyg="zyg", id="tvparnr", type="flex")
```
 
Same marginals but free correlation with MZ, DZ

``` r
lu <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="eqmarg")
estimate(lu)
#>                      Estimate  Std.Err    2.5%   97.5%    P-value
#> bmi.1@1               18.6037 0.251036 18.1116 19.0957  0.000e+00
#> bmi.1~age.1@1          0.1189 0.005635  0.1078  0.1299  9.177e-99
#> bmi.1~gendermale.1@1   1.3848 0.086573  1.2151  1.5544  1.376e-57
#> log(var)@1             2.4424 0.022095  2.3991  2.4857  0.000e+00
#> atanh(rhoMZ)@1         0.7803 0.036249  0.7092  0.8513 9.008e-103
#> atanh(rhoDZ)@2         0.2987 0.020953  0.2576  0.3397  4.288e-46
```

A formal test of genetic effects can be obtained by comparing the MZ and DZ correlation:

``` r
estimate(lu,lava::contr(5:6,6))
#>                           Estimate Std.Err   2.5%  97.5%   P-value
#> [atanh(rhoMZ)@1] - [a....   0.4816 0.04177 0.3997 0.5635 9.431e-31
```

We also consider the ACE model

``` r
ace0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="ace")
summary(ace0)
#>                  Estimate Std. Error Z value Pr(>|z|)
#> bmi            1.8599e+01 2.5576e-01  72.720   <2e-16
#> sd(A)          2.7270e+00 4.2658e-02  63.927   <2e-16
#> sd(C)          1.7123e-06 3.1064e-01   0.000        1
#> sd(E)          2.0276e+00 3.4787e-02  58.286   <2e-16
#> bmi~age        1.1892e-01 5.6246e-03  21.142   <2e-16
#> bmi~gendermale 1.3846e+00 8.8748e-02  15.601   <2e-16
#> 
#> MZ-pairs DZ-pairs 
#>     1483     2788 
#> 
#> Variance decomposition:
#>   Estimate 2.5%    97.5%  
#> A 0.64399  0.61793 0.67005
#> C 0.00000  0.00000 0.00000
#> E 0.35601  0.32995 0.38207
#> 
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.64399  0.61793 0.67005
#> 
#>                        Estimate 2.5%    97.5%  
#> Correlation within MZ: 0.64399  0.61718 0.66931
#> Correlation within DZ: 0.32200  0.30890 0.33497
#> 
#> 'log Lik.' -22019.66 (df=6)
#> AIC: 44051.32 
#> BIC: 44089.47
```


# Bibliography
<a id="korkeila_bmi_1991"></a>[korkeila_bmi_1991] Korkeila, Kaprio, Rissanen & Koskenvuo, Effects of gender and age on the heritability of body mass index, <i>Int J Obes</i>, <b>15(10)</b>, 647-654 (1991). [↩](#b71edfd9bc946c317f4a732845bcaf93)

<a id="hjelmborg_bmi_2008"></a>[hjelmborg_bmi_2008] Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen & Kaprio, Genetic influences on growth traits of BMI: a longitudinal study of adult twins, <i>Obesity (Silver Spring)</i>, <b>16(4)</b>, 847-852 (2008). [↩](#718839fcb6ade82ebb2d7de853582b80)

---
title: "IPCW Cumulative Cost"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    # fig_width: 7.15
    # fig_height: 5.5 
header-includes: 
  - \usepackage{tikz}
vignette: >
  %\VignetteIndexEntry{IPCW Cumulative Cost}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview
========

We describe how to perform regression modelling for cumulative cost
\begin{align*}
{\cal U}(t) & = \int_0^t Z(s) dN(s)
\end{align*}
where $N(s)$ is a counting process that registers the times at which costs are realised and accumulated,
and $Z(t)$ is the cost (or mark) at the event times. The counting process can be
a mix of random and fixed times, and the data are represented in counting process format with
the marks/costs attached to the event times. There are many additional uses of
such cumulative processes; for example, when considering time lost in a recurrent events setting, which we return to
below.

We can estimate the marginal mean of the cumulative process
\begin{align*}
\nu(t) & = E ( {\cal U}(t) )  
\end{align*}
possibly for strata with standard errors based on the derived influence function. 

We provide semi-parametric regression modelling using the  proportional model 
\begin{align*}
E ( {\cal U}(t) | X) & = \Lambda_0(t) \exp( X^T \beta).
\end{align*}

In addition for a fixed time-point $t \in [0,\tau]$ we can estimate the mean given covariates
\begin{align*}
E ( {\cal U}(t) | X) & = \exp( X^T \beta) 
\end{align*}
where $\tau$ is some maximum follow-up time. 

 - These quantities are estimated under independent right-censoring given $X$, using IPCW-adjusted estimating equations,
    - similarly to the Ghosh-Lin model for recurrent events.
 - A terminal event can be specified.

We also estimate the probability of exceeding thresholds over time
\begin{align*}
 P ( {\cal U}(t)  > k ) & = \mu_k(t),
\end{align*}
and in the setting with a terminal event this is based on a derived competing risks data structure that
keeps track of the competing terminal event.

\begin{tikzpicture}[
    node distance=3.5cm,
    box/.style={draw, rectangle, minimum width=2.8cm, minimum height=1cm, align=center}
]
% Nodes
\node[box] (start) {At Risk};
\node[box, right of=start, yshift=2.4cm] (exceed) {Exceed-k};
\node[box, right of=start, yshift=-2.4cm] (death) {D};

% Arrows with hazard labels
\draw[->, thick] (start) -- node[below] {\hspace{2 cm} $F_{exceed-k}(t)$} (exceed);
\draw[->, thick] (start) -- node[below] {$F_{D}(t)$} (death);
\end{tikzpicture}

Regression modelling of this quantity is also possible using competing risks
regression models, for example via the `cifreg` function in mets.


HF-action data
==============

Using the HF-action data, we simulate a severity score for each event.


```{r}
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

## marginal mean using formula  
outNZ <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 

outN <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
summary(outN,times=3) 
```

For comparison we also compute the IPCW estimates with and without marks at time 3 using the linear model,
and note that they are identical. Standard errors are based on different formulae that are asymptotically
equivalent, and we note that they are very similar.

```{r}
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
head(iid(outNZ3))

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
head(iid(outN3))
```

We also apply the semiparametric proportional cost model with IPCW adjustment: 

```{r}
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
plot(GL,add=TRUE,col=2)
```
Those treated have 14\% lower cumulative severity and 11\% lower expected number of events.


Exceed threshold
================

Finally, we estimate the probability of exceeding cumulative severity thresholds of 1, 5, and 10:

```{r}
ooNZ <- prob_exceed_recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
summary(ooNZ,times=3)
```

Cumulative time lost for recurrent events
=========================================

The cumulative time lost for recurrent events is defined as
\begin{align*}
  {\cal M}(t) = E\left[ \int_0^\tau (\tau-s) dN(s) \right] = \int_0^\tau \mu(s) ds
\end{align*}
where $\mu(t) = E( N(t) )$ is the marginal mean of the recurrent events at time $t$.


```{r}
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
head(iid(RecLost))
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Randomization for Cox Type rate models "
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Randomization for Cox Type rate models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Two-Stage Randomization for counting process outcomes
========================================================

Rate models are specified for $N_1(t)$, covering:

   - survival data
   - competing risks, cause-specific hazards
   - recurrent events data


Under simple randomization we can estimate the rate Cox model

  - \(  \lambda_0(t) \exp(A_0 \beta_0) \)

Under two-stage randomization we can estimate the rate Cox model

  - \( \lambda_0(t) \exp(A_0 \beta_0 + A_1(t) \beta_1 ) \)


The starting point is that Cox's partial likelihood score can be used for estimating parameters:
 \begin{align*}
   U(\beta) & = \int (A(t) - e(t)) dN_1(t)
 \end{align*}
 where $A(t)$ is the combined treatment vector over time.

  - The solution will converge to $\beta^*$, the Struthers-Kalbfleisch solution of the score, with robust Lin-Wei standard errors.

 The estimator can be augmented in different ways using additional covariates at the time of randomization and a censoring augmentation.
 The solved estimating equation is
 \begin{align*}
 \sum_i   U_i - AUG_0 - AUG_1 + AUG_C   = 0
 \end{align*}
 using covariates from `augmentR0` to augment with
 \begin{align*}
 AUG_0 = ( A_0 - \pi_0(X_0) ) X_0 \gamma_0
 \end{align*}
 where $P(A_0=1|X_0)=\pi_0(X_0)$ (which does not depend on covariates under randomization),
 and using covariates from `augmentR1` to augment with $R$ indicating whether the
 second randomization takes place:
 \begin{align*}
 AUG_1 = R ( A_1 - \pi_1(X_1))  X_1 \gamma_1
 \end{align*}
 and the dynamic censoring augmentation
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s)
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimise the variance given the dynamic covariates specified by `augmentC`.

Propensity score models are always estimated unless a fixed value such as $\pi_0=1/2$ is requested, though
it is generally better to estimate $\pi_0$ adaptively. Similarly, $\gamma_0$ and $\gamma_1$ are estimated to reduce the variance of $U_i$.


  - Treatments must be given as factors.
  - The treatment for the 2nd randomization may depend on the response.
      - Treatment probabilities are estimated by default and uncertainty from this is adjusted for.
      - `treat.model` must typically allow for interaction between treatment number and covariates.
  - Randomization augmentation for the 1st and 2nd randomization is possible.
       - `typesR=c("R0","R1","R01")`
  - The censoring model can be stratified on observed baseline covariates.
     - Default model stratifies on randomization `R0`.
     - `cens.model` can be specified.
  - Censoring augmentation is done dynamically over time with time-dependent covariates.
      - `typesC=c("C","dynC")`: `C` for fixed coefficients and `dynC` for dynamic.
      - Applied separately within each stratum of the censoring model.

 Standard errors are estimated using the influence functions of all estimators, so tests of differences
 can be computed subsequently.

  - Variance adjustment for censoring augmentation is computed by subtracting the variance gain.
  - Influence functions are given for the case with `R0` only.


The times of randomization are specified by:

   - `treat.var` is `"1"` when a randomization occurs.
     - Default assumes all time points correspond to a treatment (the survival case without time-dependent covariates).
     - For recurrent events, this must be specified as the first record for each subject; see example below.


Data must be provided in start-stop-status survival format with:

 - one status code indicating events of interest
 - one code for censorings


The phreg_rct can be used for counting process style data, and thus covers situations with 


  - recurrent events 
  - survival data
  - cause-specific hazards (competing risks)

and will in all cases compute augmentations  

  - dynamic censoring augmentation
     - dynC
  - RCT augmentation 
     - R0, R1 and R01



Simple Randomization: Lu-Tsiatis marginal Cox model
===================================================


```{r}
library(mets) 
set.seed(100)

## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z
 
out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~X)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X,cens.model=~+1)
```

Results consistent with speff of  library(speff2trial) 

```{r}
###library(speff2trial) 
library(mets)
data(ACTG175)
###
data <- ACTG175[ACTG175$arms==0 | ACTG175$arms==1, ]
data <- na.omit(data[,c("days","cens","arms","strat","cd40","cd80","age")])
data$days <- data$days+runif(nrow(data))*0.01
dfactor(data) <- arms.f~arms
notrun <- 1

if (notrun==0) { 
fit1 <- speffSurv(Surv(days,cens)~cd40+cd80+age,data=data,trt.id="arms",fixed=TRUE)
summary(fit1)
}
# 
# Treatment effect
#             Log HR       SE   LowerCI   UpperCI           p
# Prop Haz  -0.70375  0.12352  -0.94584  -0.46165  1.2162e-08
# Speff     -0.72430  0.12051  -0.96050  -0.48810  1.8533e-09

out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~cd40+cd80+age,augmentC=~cd40+cd80+age)
summary(out)
```

The study is actually block-randomized,
so the standard errors should be computed with an adjustment equivalent to augmenting with the block as a factor:

```{r}
dtable(data,~strat+arms)
dfactor(data) <- strat.f~strat
out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~strat.f)
summary(out)
```

Two-Stage Randomization CALGB-9823 for survival outcomes
=========================================================

We illustrate an analysis of one SMART (Sequential Multiple Assignment Randomised Trial) conducted by Cancer and
Leukemia Group B Protocol 8923 (CALGB 8923), Stone and others (2001).
388 patients were randomised to an initial treatment of GM-CSF ($A_1$) or standard chemotherapy
($A_2$). Patients with complete remission and informed consent were then re-randomised to
cytarabine only ($B_1$) or cytarabine plus mitoxantrone ($B_2$).

We first compute the weighted risk-set  estimator based on estimated weights
\begin{align*}
\Lambda_{A1,B1}(t) & = \sum_i \int_0^t \frac{w_i(s)}{Y^w(s)} dN_i(s)
\end{align*}
where $w_i(s) = I(A0_i=A1) + (t>T_R) I(A1_i=B1)/\pi_1(X_i)$, that is 
1 when you start on treatment $A1$ and
then for those that changes to $B1$ at time $T_R$ then is scaled up 
with the proportion doing this. This is equivalent to
the IPTW (inverse probability of treatment weighted estimator). We estimate 
the treatment regimes $A1, B1$ and $A2, B1$ by letting $A10$ indicate those that are consistent with ending on $B1$. 
$A10$ then starts being $1$ and becomes $0$ if the subject is treated with $B2$, but stays $1$ if the subject is treated with $B1$. 
We can then look at the two strata where $A0=0,A10=1$ and $A0=1,A10=1$. Similarly, for those that end being consistent with $B2$.
Thus defining $A11$ to start being $1$, then stays $1$ if $B2$ is taken, and becomes $0$ if the second randomization is $B1$.

  - the treatment models are for all time-points, unless the `weight.var` variable is given (1 for treatments, 0 otherwise) to accommodate a general start-stop format
  - the treatment model may also depend on a response value
  - standard errors are based on influence functions and is also computed for the baseline

We here use the propensity score model $P(A1=B1|A0)$ that uses the 
observed frequencies on arm $B1$ among those starting out on either $A1$ or $A2$.

```{r}
data(calgb8923)
calgt <- calgb8923

## tm <- At.f~factor(Count2)+age+sex+wbc
## tm <- At.f~factor(Count2)
tm <-  At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=1:10)
summary(pll0,times=1:10)
```

The propensity score model can be extended to use covariates to get increased efficiency.
Note also that the propensity scores for $A0$ will cancel out in the different strata. 

We now illustrate how to fit a  Cox model of the form
\begin{align*}
  & \lambda_{A0}(t) \exp( B1(t) \beta_1 + B2(t) \beta_2)
\end{align*}
where $\beta_0$ is the effect of treatment $A2$ and the effect of $B1$ 

Now  comparing only those starting on A1/A2 to compare the effect of B1 versus B2

```{r}
library(mets)
data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1

## making time-dependent indicators of going to B1/B2
calgt$A10t <- calgt$A11t <- 0
calgt <- dtransform(calgt,A10t=1,A1==0 & Count2==1)
calgt <- dtransform(calgt,A11t=1,A1==1 & Count2==1)
calgt0 <- subset(calgt,A0==0)

ss0 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==0),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss0)

ss1 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==1),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss1)
```

and a more structured model with both A0 and A1, that does not seem very reasonable based on 
the above,

```{r}
ssf <- phreg_rct(Event(start,time,status)~A0.f+A10t+A11t+cluster(id),
	 data=calgt,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR0=~age+wbc+sex,augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
```



Recurrent events: Simple Randomization
======================================

Recurrent events simulation with death and censoring.


```{r}
n <- 1000
beta <- 0.15; 
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- scalecumhaz(CPH_HPN_CRBSI$mechanical,0.5)

cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
ce <- 3; betao1 <- 0

varz <- 1; dep=4; X <- z <- rgamma(n,1/varz)*varz
Z0 <- NULL
px <- 0.5
if (betao1!=0) px <- lava::expit(betao1*X)      
A0 <- rbinom(n,1,px)
r1 <- exp(A0*beta[1])
rd <- exp( A0 * 0.15)
rc <- exp( A0 * 0 )
###
rr <-    mets:::simLUCox(n,base1,death.cumhaz=dr,r1=r1,Z0=X,dependence=dep,var.z=varz,cens=ce/5000)
rr$A0 <- A0[rr$id]
rr$z1 <- attr(rr,"z")[rr$id]
rr$lz1 <- log(rr$z1)
rr$X <- rr$lz1 
rr$lX <- rr$z1
rr$statusD <- rr$status
rr <- dtransform(rr,statusD=2,death==1)
rr <- count_history(rr)
rr$Z <- rr$A0
data <- rr
data$Z.f <- as.factor(data$Z)
data$treattime <- 0
data <- dtransform(data,treattime=1,lbnr__id==1)
dlist(data,start+stop+statusD+A0+z1+treattime+Count1~id|id %in% c(4,5))
```

Now we fit the model 

```{r}
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 treat.var="treattime",typesR=c("non","R0"),typesC=c("non","C","dynC"),
	 augmentR0=~z1,augmentC=~z1+Count1)
summary(fit2)
```

 - Censoring model was stratified on Z.f
 - treatment probabilities were estimated using the data


Twostage Randomization: Recurrent events 
=========================================



```{r}
n <- 500
beta=c(0.3,0.3);betatr=0.3;betac=0;betao=0;betao1=0;ce=3;fixed=1;sim=1;dep=4;varz=1;ztr=0; ce <- 3
## take possible frailty 
Z0 <- rgamma(n,1/varz)*varz
px0 <- 0.5; if (betao!=0) px0 <- expit(betao*Z0)
A0 <- rbinom(n,1,px0)
r1 <- exp(A0*beta[1])
#
px1 <- 0.5; if (betao1!=0) px1 <- expit(betao1*Z0)
A1 <- rbinom(n,1,px1)
r2 <- exp(A1*beta[2])
rtr <- exp(A0*betatr[1])
rr <-  mets:::simLUCox(n,base1,death.cumhaz=dr,cumhaz2=base1,rtr=rtr,betatr=0.3,A0=A0,Z0=Z0,
		r1=r1,r2=r2,dependence=dep,var.z=varz,cens=ce/5000,ztr=ztr)
rr$z1 <- attr(rr,"z")[rr$id]
rr$A1 <- A1[rr$id]
rr$A0 <- A0[rr$id]
rr$lz1 <- log(rr$z1)
rr <- count_history(rr,types=1:2)
rr$A1t <- 0
rr <- dtransform(rr,A1t=A1,Count2==1) 
rr$At.f <- rr$A0
rr$A0.f <- factor(rr$A0)
rr$A1.f <- factor(rr$A1)
rr <- dtransform(rr, At.f = A1, Count2 == 1)
rr$At.f <- factor(rr$At.f)
dfactor(rr)  <-  A0.f~A0
rr$treattime <- 0
rr <- dtransform(rr,treattime=1,lbnr__id==1)
rr$lagCount2 <- dlag(rr$Count2)
rr <- dtransform(rr,treattime=1,Count2==1 & (Count2!=lagCount2))
dlist(rr,start+stop+statusD+A0+A1+A1t+At.f+Count2+z1+treattime+Count1~id|id %in% c(5,10))
```

Now fitting the model and computing different augmentations (true values 0.3 and 0.3)

```{r}
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),treat.var="treattime",
	 treat.model=At.f~factor(Count2),
	 augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
```

  - `treat.model` codes $A_0$ and $A_1$ as `At.f`; here we allow a model that depends on the randomization to
    be as adaptive as possible.
  - For the observational case one can also adjust for covariates here.
  - The censoring model was stratified on `A0.f`.


Causal assumptions for Two-Stage Randomization: Recurrent events
===============================================================

We are interested in $N_1$ and also have death $N_d$.

Given $X_0$, we require:

  - \( A_0 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | X_0 \)
  - positivity: \(  1 > \pi_0(X_0) > 0 \)

Given $\bar X_1$, the history accumulated up to the time $T_R$ of the second randomization:

  - \( A_1 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | \bar X_1 \)
  - positivity: \(  1 > \pi_1(X_1) > 0 \)

And consistency, to link counterfactual quantities to observed data.

We must use an IPTW-weighted Cox score and augment as before.

In addition we require that censoring is independent given, for example, $A_0$:

  - Independent censoring.

To use `phreg_rct` in this setting:

 - Set `RCT=FALSE`.
 - Propensity score models must be specified.
 - The marginal estimate is based on the IPTW Cox model `phreg_IPTW`.
    - When the same model is used for both the propensity scores and the augmentation models,
      the augmentation models are not needed due to the adaptive nature of fitting the
      propensity score models.


```{r}
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 typesR=c("non","R0"),typesC=c("non","C","dynC"),
         RCT=FALSE,treat.model=Z.f~z1,augmentR0=~z1,augmentC=~z1+Count1,
	 treat.var="treattime")
summary(fit2)
```

and for twostage randomization

```{r}
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
         treat.var="treattime",
	 RCT=FALSE,treat.model=At.f~z1*factor(Count2),
         augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "GEE cluster standard errors and predictions for glm objects" 
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{GEE cluster standard errors and predictions for glm objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Utility functions for GLM objects
=================================

Getting the OR with confidence intervals  using the GEE (sandwhich) 
standard errors

```{r}
set.seed(100)

library(mets)
data(bmt); 
bmt$id <- sample(1:100,408,replace=TRUE)

glm1 <- glm(tcell~platelet+age,bmt,family=binomial)
summaryGLM(glm1)

## GEE robust standard errors
summaryGLM(glm1,id=bmt$id)
```

Predictions also simple 

```{r}
age <- seq(-2,2,by=0.1)
nd <- data.frame(platelet=0,age=seq(-2,2,by=0.1))
pnd <- predictGLM(glm1,nd)
head(pnd$pred)
plot(age,pnd$pred[,1],type="l",ylab="predictions",xlab="age",ylim=c(0,0.3))
matlines(age,pnd$pred[,-1],col=2)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




The `resmeanATE` or `resmeanIPCW`/`rmstIPCW` functions fit an exponential or linear link model with
IPCW adjustment at a specific time point for the restricted mean survival or years lost in
competing risks. The computation is linear in the data size, making it suitable for large datasets.
Influence functions are computed and available for downstream inference.

Key features:

  - censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument



RMST 
====

Regression for rmst outcome $E(T \wedge t | X) = exp(X^T \beta)$ based on IPCW adjustment for censoring, thus solving
the estimating equation 
\begin{align*}
 & X^T [ (T \wedge t) \frac{I(C > T \wedge t)}{G_C(T \wedge t,X)} - exp(X^T \beta) ] = 0 .
\end{align*}
This is done with the resmeanIPCW function. 
For a fully saturated model with a complete censoring model this is equal to the integral of the Kaplan-Meier estimator, as
illustrated below.

We can also compute the integral of the Kaplan-Meier or Cox-based survival estimator to
get the RMST (using the `resmean_phreg` function):
\[ \int_0^t S(s|X) ds. \]

For competing risks, the years lost can be computed via cumulative incidence functions (`cif_yearslost`)
or as an IPCW estimator, since
\[ E( I(\epsilon=1) ( t - T \wedge t ) | X) = \int_0^t F_1(s) ds. \]
For a fully saturated model with a complete censoring model these estimators are equivalent, as illustrated below.



``` r
set.seed(101)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
#>    n events
#>  408    245
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  3.014321  0.065114  2.886700  3.141941  0.0000
#> tcell        0.106526  0.138268 -0.164473  0.377525  0.4410
#> platelet     0.247103  0.097337  0.056325  0.437880  0.0111
#> age         -0.185936  0.043566 -0.271324 -0.100548  0.0000
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 20.37524 17.93403 23.1488
#> tcell        1.11241  0.84834  1.4587
#> platelet     1.28031  1.05794  1.5494
#> age          0.83033  0.76237  0.9043
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
#>                        Estimate Std.Err  2.5% 97.5%   P-value
#> inttcell=0, platelet=0    13.60  0.8316 11.97 15.23 3.826e-60
#> inttcell=0, platelet=1    18.90  1.2696 16.41 21.39 3.997e-50
#> inttcell=1, platelet=0    16.19  2.4061 11.48 20.91 1.705e-11
#> inttcell=1, platelet=1    17.77  2.4536 12.96 22.58 4.463e-13
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean_phreg(out1,times=30)
     summary(rm1)
#>                     strata times    rmean  se.rmean    lower    upper
#> tcell=0, platelet=0      0    30 13.60295 0.8315418 12.06700 15.33439
#> tcell=0, platelet=1      1    30 18.90127 1.2693263 16.57021 21.56026
#> tcell=1, platelet=0      2    30 16.19121 2.4006185 12.10806 21.65131
#> tcell=1, platelet=1      3    30 17.76610 2.4421987 13.57008 23.25956
#>                     years.lost
#> tcell=0, platelet=0   16.39705
#> tcell=0, platelet=1   11.09873
#> tcell=1, platelet=0   13.80879
#> tcell=1, platelet=1   12.23390
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
#>                        Estimate Std.Err   2.5%  97.5%   P-value
#> inttcell=0, platelet=0   12.105  0.8508 10.438 13.773 6.162e-46
#> inttcell=0, platelet=1    6.884  1.1741  4.583  9.185 4.536e-09
#> inttcell=1, platelet=0    7.261  2.3533  2.648 11.873 2.033e-03
#> inttcell=1, platelet=1    5.780  2.0925  1.679  9.882 5.737e-03
     ## same as integrated cumulative incidence 
     rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)
#> $estimate
#> $estimate$intF_1
#>                     strata times    intF_1 se.intF_1 lower_intF_1 upper_intF_1
#> tcell=0, platelet=0      0    30 12.105125 0.8508102    10.547328    13.893001
#> tcell=0, platelet=1      1    30  6.884171 1.1740988     4.928092     9.616665
#> tcell=1, platelet=0      2    30  7.260755 2.3532867     3.846790    13.704561
#> tcell=1, platelet=1      3    30  5.780369 2.0924946     2.843285    11.751429
#> 
#> $estimate$intF_2
#>                     strata times   intF_2 se.intF_2 lower_intF_2 upper_intF_2
#> tcell=0, platelet=0      0    30 4.291930 0.6161439     3.239330     5.686565
#> tcell=0, platelet=1      1    30 4.214556 0.9057028     2.765857     6.422055
#> tcell=1, platelet=0      2    30 6.548034 1.9703340     3.630616    11.809770
#> tcell=1, platelet=1      3    30 6.453535 2.0815225     3.429661    12.143507
#> 
#> 
#> $total.years.lost
#> [1] 16.39705 11.09873 13.80879 12.23390

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)
```

<img src="figure/rmst-ate-unnamed-chunk-2-1.png" alt="Restricted mean survival and years lost by cause."  />

Based on the output from the IPCW estimators we can derive any measure of interest


``` r
estimate(out)
#>                        Estimate Std.Err   2.5%  97.5%   P-value
#> inttcell=0, platelet=0   12.105  0.8508 10.438 13.773 6.162e-46
#> inttcell=0, platelet=1    6.884  1.1741  4.583  9.185 4.536e-09
#> inttcell=1, platelet=0    7.261  2.3533  2.648 11.873 2.033e-03
#> inttcell=1, platelet=1    5.780  2.0925  1.679  9.882 5.737e-03

measures <- function(p) {
 ratio1 <- p[1]/p[2]; ratio2 <- p[2]/p[1]; dif1 <- p[4]-p[1]; dif2 <- p[3]-p[1]
 m <- c(dif1,dif2,ratio1,ratio2)
 return(m)
}

 labs <- c("dif4-1","dif3-1","ratio 1/2","ratio 2/1")
 estimate(out,f=measures,labels=labs)
#>           Estimate Std.Err     2.5%    97.5%   P-value
#> dif4-1     -6.3248  2.2589 -10.7520 -1.89748 5.111e-03
#> dif3-1     -4.8444  2.5024  -9.7489  0.06018 5.288e-02
#> ratio 1/2   1.7584  0.3244   1.1227  2.39414 5.924e-08
#> ratio 2/1   0.5687  0.1049   0.3631  0.77431 5.924e-08
```

Average treatment effect
=========================

This section computes the average treatment effect for restricted mean survival and years lost in the competing risks setting.


``` r
 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
#>    n events
#>  408    241
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.852670  0.062493  2.730187  2.975153  0.0000
#> tcell1       0.021401  0.122907 -0.219493  0.262295  0.8618
#> platelet     0.303489  0.090758  0.125606  0.481372  0.0008
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 17.33400 15.33575 19.5926
#> tcell1       1.02163  0.80293  1.2999
#> platelet     1.35458  1.13383  1.6183
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.26223  0.95926 17.38212 21.14234  0.0000
#> treat1    19.67890  2.22777 15.31255 24.04526  0.0000
#> treat:1-0  0.41667  2.41067 -4.30815  5.14150  0.8628
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.28131  0.95807 17.40352 21.15910  0.0000
#> treat1    20.34466  2.54101 15.36438 25.32494  0.0000
#> treat:1-0  1.06335  2.70979 -4.24773  6.37443  0.6948
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
		    treat.model=tcell~platelet)
 summary(out1)
#>    n events
#>  408    157
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.806295  0.069619  2.669845  2.942745  0.0000
#> tcell1      -0.374065  0.247669 -0.859488  0.111358  0.1310
#> platelet    -0.491736  0.164946 -0.815025 -0.168447  0.0029
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 16.54849 14.43773 18.9678
#> tcell1       0.68793  0.42338  1.1178
#> platelet     0.61156  0.44263  0.8450
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    14.53185  0.95707 12.65604 16.40767  0.0000
#> treat1     9.99693  2.37814  5.33587 14.65800  0.0000
#> treat:1-0 -4.53492  2.57516 -9.58213  0.51229  0.0782
#> 
#> Average Treatment effects (double robust) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     14.513742   0.958025  12.636048  16.391436  0.0000
#> treat1      9.365012   2.417032   4.627717  14.102307  0.0001
#> treat:1-0  -5.148730   2.597947 -10.240612  -0.056848  0.0475

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,time=40,
		    treat.model=tcell~platelet)
 summary(out2)
#>    n events
#>  408     84
#> 
#>  408 clusters
#> coeffients:
#>               Estimate    Std.Err       2.5%      97.5% P-value
#> (Intercept)  1.8266090  0.1312181  1.5694263  2.0837918  0.0000
#> tcell1       0.4751558  0.2403839  0.0040121  0.9462996  0.0481
#> platelet    -0.0090724  0.2168469 -0.4340845  0.4159397  0.9666
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  6.21278 4.80389 8.0349
#> tcell1       1.60826 1.00402 2.5762
#> platelet     0.99097 0.64786 1.5158
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     6.19518  0.71372  4.79631  7.59405  0.0000
#> treat1     9.96349  2.09256  5.86216 14.06482  0.0000
#> treat:1-0  3.76831  2.21654 -0.57602  8.11264  0.0891
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     6.20484  0.71392  4.80559  7.60410  0.0000
#> treat1    10.30065  2.21700  5.95542 14.64588  0.0000
#> treat:1-0  4.09581  2.32897 -0.46889  8.66050  0.0786
```



SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              KernSmooth_2.23-26     otel_0.2.0            
#>  [7] future.apply_1.20.2    listenv_0.10.1         lava_1.9.2.1          
#> [10] stats4_4.6.0           grid_4.6.0             evaluate_1.0.5        
#> [13] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [16] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [19] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [22] digest_0.6.39          parallelly_1.47.0      parallel_4.6.0        
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Haplotype Discrete Survival Models"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Haplotype Discrete Survival Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




Haplotype Analysis for discrete TTP
===================================

Cycle-specific logistic regression of haplotype effects with
known haplotype probabilities. Given observed genotype $G$ and
unobserved haplotypes $H$, we marginalise over the possible
haplotypes using $P(H|G)$ supplied as input.

\begin{align*}
   S(t|x,G) & = E( S(t|x,H) | G)  = \sum_{h \in G} P(h|G) S(t|z,h)
\end{align*}
so survival can be computed by marginalising over possible $h$ given $g$.

Survival is based on logistic regression for the discrete hazard
function of the form
\begin{align*}
      \mbox{logit}(P(T=t \mid T \geq t, x, h)) & = \alpha_t + x(h)^T \beta
\end{align*}
where $x(h)$ is a regression design of $x$ and haplotypes $h=(h_1,h_2)$.

Simple binomial data can also be fitted using this function.

Standard errors are computed assuming that the haplotype probabilities are known.

	
We are particularly interested in the types haplotypes: 

``` r
types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")

## some haplotypes frequencies for simulations 
data(haplo)
hapfreqs <- haplo$hapfreqs 
print(hapfreqs)
#>             index   haplotype     freq
#> DCGAGCTCACG     1 DCGAGCTCACG 0.010681
#> DCGCGCTCACG     2 DCGCGCTCACG 0.138387
#> DTGAGCTCACG     3 DTGAGCTCACG 0.000310
#> DTGAGCTCACA     4 DTGAGCTCACA 0.006800
#> DTGAGCTCGCG     5 DTGAGCTCGCG 0.034517
#> DTGACCTCACG     6 DTGACCTCACG 0.001336
#> DTGCGCTCACG     7 DTGCGCTCACG 0.009969
#> DTGCGCTCACA     8 DTGCGCTCACA 0.011833
#> DTGCGCTCGCG     9 DTGCGCTCGCG 0.302389
#> DTGCGCCCGCG    10 DTGCGCCCGCG 0.001604
#> DTGCCCTCACG    11 DTGCCCTCACG 0.003912
#> DTCAGCTGACG    12 DTCAGCTGACG 0.001855
#> DTCCGCTGACG    13 DTCCGCTGACG 0.103394
#> DTCCCCTGACG    14 DTCCCCTGACG 0.000310
#> ITCAGTTGACG    15 ITCAGTTGACG 0.048124
#> ITCCGCTGAGG    16 ITCCGCTGAGG 0.291273
#> ITCCGTTGACG    17 ITCCGTTGACG 0.031089
#> ITCCGTCGACG    18 ITCCGTCGACG 0.001502
#> ITCCCCTGAGG    19 ITCCCCTGAGG 0.000653
```

Among the haplotypes of interest we look up the frequencies and choose a baseline:


``` r
www <-which(hapfreqs$haplotype %in% types)
hapfreqs$freq[www]
#> [1] 0.138387 0.103394 0.048124 0.291273

baseline=hapfreqs$haplotype[9]
baseline
#> [1] "DTGCGCTCGCG"
```


We have cycle-specific data with $id$ and outcome $y$:


``` r
haploX  <- haplo$haploX
dlist(haploX,.~id|id %in% c(1,4,7))
#> id: 1
#>   y X1 X2 X3 X4 times lbnr__id Count1
#> 1 0 0  0  0  0  1     1        0     
#> 2 0 0  0  0  0  2     2        0     
#> 3 0 0  0  0  0  3     3        0     
#> 4 0 0  0  0  0  4     4        0     
#> 5 0 0  0  0  0  5     5        0     
#> 6 0 0  0  0  0  6     6        0     
#> ------------------------------------------------------------ 
#> id: 4
#>    y X1 X2 X3 X4 times lbnr__id Count1
#> 19 1 0  0  0  0  1     1        0     
#> ------------------------------------------------------------ 
#> id: 7
#>    y X1 X2 X3 X4 times lbnr__id Count1
#> 37 0 1  0  0  0  1     1        0     
#> 38 0 1  0  0  0  2     2        0     
#> 39 1 1  0  0  0  3     3        0
```
and a list of possible haplotypes for each id and their probabilities $p$
(the probabilities within each id sum to 1):


``` r
ghaplos <- haplo$ghaplos
head(ghaplos)
#>    id      haplo1      haplo2          p
#> 1   1 DTGCGCTCGCG DTGAGCTCGCG 1.00000000
#> 19  2 ITCCGTTGACG DTGAGCTCGCG 0.06867716
#> 21  2 ITCAGTTGACG DTGCGCTCGCG 0.93132284
#> 51  3 ITCCGTTGACG DTGAGCTCGCG 0.06867716
#> 53  3 ITCAGTTGACG DTGCGCTCGCG 0.93132284
#> 66  4 DTGCGCTCGCG DTGCGCTCGCG 1.00000000
```

The first id=1 has the haplotype fully observed, but id=2 has two possible haplotypes consistent with the
observed genotype for this id, the probabilities are 7\% and 93\%, respectively. 

With the baseline given above we can specify a regression design that gives an effect if a haplotype of the specified type
is present (`sm=0`), or an additive effect of haplotypes (`sm=1`):


``` r
designftypes <- function(x,sm=0) {
hap1=x[1]
hap2=x[2]
if (sm==0) y <- 1*( (hap1==types) | (hap2==types))
if (sm==1) y <- 1*(hap1==types) + 1*(hap2==types)
return(y)
}

```

To fit the model we first construct a time design matrix (named `X`)
and supply the haplotype distributions for each id:


``` r
haploX$time <- haploX$times
Xdes <- model.matrix(~factor(time),haploX)
colnames(Xdes) <- paste("X",1:ncol(Xdes),sep="")
X <- dkeep(haploX,~id+y+time)
X <- cbind(X,Xdes)
Haplos <- dkeep(ghaplos,~id+"haplo*"+p)
desnames=paste("X",1:6,sep="")   # six X's related to 6 cycles 
head(X)
#>   id y time X1 X2 X3 X4 X5 X6
#> 1  1 0    1  1  0  0  0  0  0
#> 2  1 0    2  1  1  0  0  0  0
#> 3  1 0    3  1  0  1  0  0  0
#> 4  1 0    4  1  0  0  1  0  0
#> 5  1 0    5  1  0  0  0  1  0
#> 6  1 0    6  1  0  0  0  0  1
```

We can now fit the model with the design given by the design function:

``` r
out <- haplo_surv_discrete(X=X,y="y",time.name="time",
      Haplos=Haplos,desnames=desnames,designfunc=designftypes) 
names(out$coef) <- c(desnames,types)
out$coef
#>          X1          X2          X3          X4          X5          X6 
#> -1.82153345 -0.61608261 -0.17143057 -1.27152045 -0.28635976 -0.19349091 
#> DCGCGCTCACG DTCCGCTGACG ITCAGTTGACG ITCCGCTGAGG 
#>  0.79753613  0.65747412  0.06119231  0.31666905
summary(out)
#>             Estimate Std.Err     2.5%   97.5%   P-value
#> X1          -1.82153  0.1619 -2.13892 -1.5041 2.355e-29
#> X2          -0.61608  0.1895 -0.98748 -0.2447 1.149e-03
#> X3          -0.17143  0.1799 -0.52398  0.1811 3.406e-01
#> X4          -1.27152  0.2631 -1.78719 -0.7559 1.346e-06
#> X5          -0.28636  0.2030 -0.68425  0.1115 1.584e-01
#> X6          -0.19349  0.2134 -0.61184  0.2249 3.647e-01
#> DCGCGCTCACG  0.79754  0.1494  0.50465  1.0904 9.445e-08
#> DTCCGCTGACG  0.65747  0.1621  0.33971  0.9752 5.007e-05
#> ITCAGTTGACG  0.06119  0.2145 -0.35931  0.4817 7.755e-01
#> ITCCGCTGAGG  0.31667  0.1361  0.04989  0.5834 1.999e-02
```

Haplotypes `"DCGCGCTCACG"` and `"DTCCGCTGACG"` give an increased hazard of pregnancy.

The data were generated with these true coefficients:


``` r
tcoef=c(-1.93110204,-0.47531630,-0.04118204,-1.57872602,-0.22176426,-0.13836416,
0.88830288,0.60756224,0.39802821,0.32706859)

cbind(out$coef,tcoef)
#>                               tcoef
#> X1          -1.82153345 -1.93110204
#> X2          -0.61608261 -0.47531630
#> X3          -0.17143057 -0.04118204
#> X4          -1.27152045 -1.57872602
#> X5          -0.28635976 -0.22176426
#> X6          -0.19349091 -0.13836416
#> DCGCGCTCACG  0.79753613  0.88830288
#> DTCCGCTGACG  0.65747412  0.60756224
#> ITCAGTTGACG  0.06119231  0.39802821
#> ITCCGCTGAGG  0.31666905  0.32706859
```

The fitted design matrix can be found in the output:

``` r
head(out$X,10)
#>    X1 X2 X3 X4 X5 X6 haplo1 haplo2 haplo3 haplo4
#> 1   1  0  0  0  0  0      0      0      0      0
#> 2   1  1  0  0  0  0      0      0      0      0
#> 3   1  0  1  0  0  0      0      0      0      0
#> 4   1  0  0  1  0  0      0      0      0      0
#> 5   1  0  0  0  1  0      0      0      0      0
#> 6   1  0  0  0  0  1      0      0      0      0
#> 8   1  0  0  0  0  0      0      0      1      0
#> 10  1  1  0  0  0  0      0      0      1      0
#> 12  1  0  1  0  0  0      0      0      1      0
#> 14  1  0  0  1  0  0      0      0      1      0
```


SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Average treatment effect (ATE) for Competing risks and binary outcomes"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Competing risks and binary outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



The `binregATE` function estimates the average treatment effect for binary outcomes with
IPCW adjustment at a specific time-point, and can thus be used for survival or competing risks data.
Computation is linear in data size and influence functions are computed and available.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Average treatment effect 
=========================

First we simulate some data that mimics that of Kumar et al. (2012).
This is data from multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR). The data consisted of patients transplanted from 1995 to
2005, comparing outcomes between transplant periods: 2001--2005 ($N=488$)
versus 1995--2000 ($N=375$). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1) defined as death without relapse.
Kumar et al. (2012) considered the following risk covariates:
transplant time period (`gp`, main interest of the study: 1 for transplanted in
2001--2005 versus 0 for transplanted in 1995--2000), donor type (`dnr`: 1 for unrelated or
other related donor ($N=280$) versus 0 for HLA-identical sibling ($N=584$)), prior
autologous transplant (`preauto`: 1 for Auto+Allo transplant ($N=399$) versus 0 for
allogeneic transplant alone ($N=465$)) and time to transplant (`ttt24`: 1 for more than 24 months ($N=289$) versus 0 for less than or
equal to 24 months ($N=575$)).


We generate similar data by assuming that the two cumulative incidence curves are logistic and that
censoring depends on covariates via a Cox model, all wrapped in the `kumarsim` function.
The simulation does not enforce the constraint $F_1+F_2 < 1$, but as we increase the
sample size we still recover the parameters of cause 2.

We start by computing the average treatment effects for the transplant periods (`gp`), then consider
the possible interaction between `gp` and donor type (`dnr`) and estimate all contrasts between the risk estimates for all combinations
of `gp` and `dnr`. The key components are:

  - an outcome model (fitted by IPCW): `Event(...) ~ .....`
  - a propensity score model (using logistic regression): `treat.model=...`
  - an IPCW weight estimated using a stratified Kaplan-Meier model: `cens.model=~strata(..)`



``` r
library(mets) 
set.seed(100)
###
n <- 400
kumar <- kumarsim(n,depcens=1)
kumar$cause <- kumar$status
kumar$ttt24 <- kumar[,6]
dtable(kumar,~cause)
#> 
#> cause
#>   0   1   2 
#> 182  72 146
dfactor(kumar) <- gp.f~gp
kumar$id <- 1:n
kumar$idc <- sample(100,n,TRUE)
kumar$ids <- sample(n,n)
kumar$id2 <- sample(n,n)
kumar2 <- kumar[order(kumar$id2),]
kumar$int <- interaction(kumar$gp,kumar$dnr)
kumar2$int <- interaction(kumar2$gp,kumar2$dnr)
clust <- 0

b2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(b2)
#>    n events
#>  400    137
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.310227  0.221121 -1.743616 -0.876838  0.0000
#> gp.f1        0.898030  0.271691  0.365526  1.430535  0.0009
#> dnr          0.323059  0.271545 -0.209160  0.855278  0.2342
#> preauto      0.269177  0.278181 -0.276049  0.814402  0.3332
#> ttt24        0.496202  0.276045 -0.044836  1.037240  0.0722
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.26976 0.17489 0.4161
#> gp.f1        2.45476 1.44127 4.1809
#> dnr          1.38135 0.81127 2.3520
#> preauto      1.30889 0.75878 2.2578
#> ttt24        1.64247 0.95615 2.8214
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    0.293021 0.039148 0.216292 0.369750   0e+00
#> treat1    0.496481 0.041744 0.414665 0.578297   0e+00
#> treat:1-0 0.203460 0.060294 0.085285 0.321635   7e-04
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    0.311984 0.042914 0.227875 0.396094   0e+00
#> treat1    0.512310 0.042454 0.429103 0.595518   0e+00
#> treat:1-0 0.200326 0.060585 0.081582 0.319070   9e-04

b5 <- binregATE(Event(time,cause)~int+preauto+ttt24,kumar,cause=2,
		treat.model=int~preauto+ttt24,cens.code=0,time=60)
summary(b5)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.115768  0.285147 -1.674646 -0.556890  0.0001
#> int1.0       0.674582  0.333967  0.020018  1.329146  0.0434
#> int0.1       0.034751  0.494229 -0.933920  1.003421  0.9439
#> int1.1       1.340278  0.431989  0.493594  2.186961  0.0019
#> preauto      0.298967  0.272324 -0.234779  0.832713  0.2723
#> ttt24        0.476755  0.286955 -0.085667  1.039177  0.0966
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.32766 0.18737 0.5730
#> int1.0       1.96321 1.02022 3.7778
#> int0.1       1.03536 0.39301 2.7276
#> int1.1       3.82010 1.63819 8.9081
#> preauto      1.34846 0.79075 2.2995
#> ttt24        1.61084 0.91790 2.8269
#> 
#> Average Treatment effects (G-formula) :
#>                 Estimate    Std.Err       2.5%      97.5% P-value
#> treat0.0       0.3124362  0.0586931  0.1973999  0.4274724  0.0000
#> treat1.0       0.4676985  0.0422018  0.3849846  0.5504125  0.0000
#> treat0.1       0.3197899  0.0896402  0.1440983  0.4954815  0.0004
#> treat1.1       0.6274521  0.0726051  0.4851488  0.7697554  0.0000
#> treat:1.0-0.0  0.1552624  0.0738341  0.0105502  0.2999745  0.0355
#> treat:0.1-0.0  0.0073537  0.1048541 -0.1981565  0.2128640  0.9441
#> treat:1.1-0.0  0.3150160  0.0978046  0.1233225  0.5067094  0.0013
#> treat:0.1-1.0 -0.1479086  0.1003040 -0.3445009  0.0486837  0.1403
#> treat:1.1-1.0  0.1597536  0.0845518 -0.0059649  0.3254721  0.0588
#> treat:1.1-0.1  0.3076622  0.1178093  0.0767602  0.5385643  0.0090
#> 
#> Average Treatment effects (double robust) :
#>                 Estimate    Std.Err       2.5%      97.5% P-value
#> treat0.0       0.3580588  0.0674640  0.2258318  0.4902858  0.0000
#> treat1.0       0.4871368  0.0501492  0.3888461  0.5854274  0.0000
#> treat0.1       0.3102783  0.1210473  0.0730299  0.5475266  0.0104
#> treat1.1       0.7645588  0.1887893  0.3945387  1.1345790  0.0001
#> treat:1.0-0.0  0.1290779  0.0831680 -0.0339284  0.2920843  0.1207
#> treat:0.1-0.0 -0.0477806  0.1400295 -0.3222333  0.2266722  0.7329
#> treat:1.1-0.0  0.4065000  0.2040550  0.0065596  0.8064404  0.0464
#> treat:0.1-1.0 -0.1768585  0.1315359 -0.4346641  0.0809471  0.1788
#> treat:1.1-1.0  0.2774221  0.1937486 -0.1023182  0.6571624  0.1522
#> treat:1.1-0.1  0.4542806  0.2102654  0.0421680  0.8663931  0.0307
```

We note that the estimates found using the large censoring model are very different from those
using the simple Kaplan-Meier weights that are severely biased for these data. This is due to a strong 
censoring dependence. 

The average treatment is around $0.17 = E(Y(1) - Y(0))$ at time 60 for the 
transplant period, under the standard causal assumptions. 
The 1/0 treatment variable used for the causal computation is found as the right hand side (rhs) of the treat.model 
or as the first argument on the rhs of the response model. 

The binregATE default uses binreg with its default to fit the working model and is 
recommended, but the logitIPCW and logitIPCWATE can also be used and are GLM-type 
IPCW weighted models (see binreg help page/vignette).  We can compare the logitIPCWATE/logitIPCW
with that of wglm and its ate function in the riskRegression package, and they agree. 


``` r
ib2 <- logitIPCWATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(ib2)
#>    n events
#>  400    137
#> 
#>  400 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -1.27557  0.21917 -1.70514 -0.84601  0.0000
#> gp.f1        0.83337  0.25904  0.32566  1.34108  0.0013
#> dnr          0.36815  0.27656 -0.17390  0.91020  0.1831
#> preauto      0.48521  0.30772 -0.11791  1.08832  0.1148
#> ttt24        0.18958  0.30947 -0.41698  0.79614  0.5401
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.27927 0.18175 0.4291
#> gp.f1        2.30105 1.38494 3.8232
#> dnr          1.44506 0.84038 2.4848
#> preauto      1.62452 0.88878 2.9693
#> ttt24        1.20874 0.65904 2.2170
#> 
#> Average Treatment effects (G-formula) :
#>         Estimate  Std.Err     2.5%    97.5% P-value
#> treat-1 0.493059 0.040018 0.414625 0.571494  0.0000
#> treat-0 0.302695 0.040274 0.223759 0.381632  0.0000
#> p1      0.190364 0.058248 0.076200 0.304527  0.0011
#> 
#> Average Treatment effects (double robust) :
#>         Estimate  Std.Err     2.5%    97.5% P-value
#> treat-1 0.524459 0.043164 0.439860 0.609058   0e+00
#> treat-0 0.309211 0.043507 0.223938 0.394483   0e+00
#> p1      0.215248 0.061268 0.095165 0.335331   4e-04

ib5 <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,
	 time=60,cens.model=~strata(gp,dnr))
summary(ib5)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.142446  0.225267 -1.583960 -0.700931  0.0000
#> gp.f1        0.716372  0.267369  0.192339  1.240405  0.0074
#> dnr          0.551548  0.325499 -0.086418  1.189515  0.0902
#> preauto      0.748078  0.343802  0.074239  1.421917  0.0296
#> ttt24       -0.184844  0.373708 -0.917298  0.547610  0.6209
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.31904 0.20516 0.4961
#> gp.f1        2.04699 1.21208 3.4570
#> dnr          1.73594 0.91721 3.2855
#> preauto      2.11294 1.07706 4.1451
#> ttt24        0.83123 0.39960 1.7291

ibs <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,time=60)
summary(ibs)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.294250  0.220215 -1.725864 -0.862636  0.0000
#> gp.f1        1.633772  0.294651  1.056267  2.211277  0.0000
#> dnr          0.020551  0.309235 -0.585538  0.626639  0.9470
#> preauto      0.547275  0.305442 -0.051381  1.145931  0.0732
#> ttt24        0.288357  0.309124 -0.317515  0.894228  0.3509
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.27410 0.17802 0.4220
#> gp.f1        5.12316 2.87562 9.1274
#> dnr          1.02076 0.55681 1.8713
#> preauto      1.72854 0.94992 3.1454
#> ttt24        1.33423 0.72796 2.4454

check <- 0
if (check==1) {
require(riskRegression)
e.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, formula.censor = Surv(time,cause==0)~+1, times = 60, data = kumar, product.limit=TRUE,cause=2)
summary(e.wglm)$coef
estimate(ibs)

es.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, 
formula.censor = Surv(time,cause==0)~strata(gp,dnr), times = 60, 
data = kumar, product.limit=TRUE,cause=2)
summary(es.wglm)$coef
estimate(ib5)
}
```

The cluster argument should not be used for the logitIPCWATE, but works for binregATE.


Average treatment for Competing risks data 
==========================================

The binreg function does direct binomial regression for one time-point, $t$, 
fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta),
\end{align*}
for possible right censored data. The estimation procedure 
is based on IPCW adjusted estimating equation (EE)
\begin{align*}
 U(\beta) = &  X \left( \Delta(t) I(T \leq t, \epsilon=1 )/G_c(T \wedge t) - \mbox{expit}( X^T beta) \right) = 0 
\end{align*}
where $G_c(t)=P(C > t)$, the censoring survival distribution, and with $\Delta(t) = I( C > T \wedge t)$ the indicator
of being uncensored at time $t$. 

The function logitIPCW instead considers the EE
\begin{align*}
 U(\beta) = &  X  \frac{\Delta(t)}{G_c(T \wedge t)} \left( I(T \leq t, \epsilon=1 ) - \mbox{expit}( X^T beta) \right) = 0.
\end{align*}
The two score equations are quite similar, and exactly the same when the censoring model is fully-nonparametric given $X$.

Additional functions logitATE, and binregATE computes the average treatment effect. We demonstrate their use below. 

The functions binregATE (recommended) and logitATE also works when there is no censoring and we thus have simple binary outcome. 

Variance is based on  sandwich formula with IPCW adjustment, 
and naive.var is variance 
under a known censoring model. The influence functions are stored in the 
output. Further, the standard errors can be
cluster corrected by specifying the relevant cluster for the working outcome model.

 - We estimate the average treatment effect of our binary response $I(T \leq t, \epsilon=1)$
   - Using a working logistic model for the response (possibly with a cluster specification)
   - Using a working logistic model for treatment given covariates
     - The binregATE can also handle a factor with more than two levels and then uses
        the mlogit multinomial regression function (of mets). 
   - Using a working model for censoring given covariates, this must be a stratified Kaplan-Meier.

If there are no censoring then the censoring weights are simply set to 1.

The average treatment effect is 
\begin{align*}
 E(Y(1) - Y(0)) 
\end{align*}
using counterfactual outcomes. 

We compute the simple G-estimator 
\begin{align*}
\sum  m_a(X_i) 
\end{align*}
to estimate the risk $E(Y(a))$. 

The DR-estimator instead uses the estimating equations that are double robust wrt 

   - A working logistic model for the response
   - A working logistic model for treatment given covariates

This is estimated using the estimator 
\begin{align*}
\sum \left[ \frac{A_i Y_i}{\pi_A(X_i)}-\frac{A_i - \pi_A(X_i)}{\pi_A(X_i)} m_1(X_i) \right]
   - \left[ \frac{(1-A_i) Y_i}{1-\pi_A(X_i)}+\frac{A_i - \pi_A(X_i)}{1-\pi_A(X_i)} m_0(X_i) \right]
\end{align*}
where

 - $A_i$ is treatment indicator
 - $\pi_A(X_i) = P(A_i=1|X_i)$ is treatment model 
 - $Y_i$ outcome, that in case of censoring is censoring adjusted $\tilde Y_i \Delta(t) /G_c(T_i- \wedge t)$
 - $\tilde Y_i = I(T_i \leq t, \epsilon_i=1)$ outcome before censoring.
 - $m_j(X_i)=P(Y_i=1| A_i=j,X_i)$ is outcome model, using binomial regression.

 The standard errors are then based on an iid decomposition using taylor-expansions 
 for the
 parameters of the treatment-model and the outcome-model, and the 
 censoring probability. 

 We need that the censoring model is correct, so it can be important to use a sufficiently 
 large censoring model as we also illustrate below. 

   - The censoring model can be specified by strata (used for `phreg`).

We also compute standard marginalization for average treatment effect  (called differenceG) 
\begin{align*}
\sum \left[  m_1(X_i) - m_0(X_i) \right]
\end{align*}
and again standard errors are based on the related influence functions and are also returned. 

For large data where there are more than 2 treatment groups the 
computations can be memory extensive when
there are many covariates due to the multinomial-regression model used for the 
propensity scores. Otherwise
the function (binregATE) will run for large data.


The ATE functions require that the treatment variable, given as the first variable on
the right-hand side of the outcome model, is a factor. The variable is also
identified from the left-hand side of the
treatment model (`treat.model`), which by default
assumes that treatment does not depend on any covariates.


Average treatment effect for binary or continuous responses
===========================================================

In the binary case a binary outcome is specified instead of the survival outcome, and as a
consequence no censoring adjustment is done.

 - the binary/numeric outcome must be a variable in the data frame

The following code runs the analysis (one can also use `binregATE` by coding `cause` without censoring values, i.e. setting `cens.code=2` and `time` large):


``` r
kumar$cause2 <- 1*(kumar$cause==2)

b3 <- logitATE(cause2~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)
#>    n events
#>  400    400
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.219960  0.200830 -1.613580 -0.826341  0.0000
#> gp.f1        0.387595  0.243738 -0.090123  0.865314  0.1118
#> dnr          0.633992  0.241410  0.160837  1.107147  0.0086
#> preauto      0.139356  0.248680 -0.348049  0.626761  0.5752
#> ttt24        0.449527  0.243475 -0.027675  0.926730  0.0648
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.29524 0.19917 0.4376
#> gp.f1        1.47343 0.91382 2.3758
#> dnr          1.88512 1.17449 3.0257
#> preauto      1.14953 0.70606 1.8715
#> ttt24        1.56757 0.97270 2.5262
#> 
#> Average Treatment effects (G-formula) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.316084  0.037954  0.241695  0.390472  0.0000
#> treat1     0.400809  0.033395  0.335356  0.466262  0.0000
#> treat:1-0  0.084726  0.052651 -0.018468  0.187919  0.1076
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.343451  0.042284  0.260577  0.426325  0.0000
#> treat1     0.421615  0.033616  0.355729  0.487500  0.0000
#> treat:1-0  0.078164  0.053961 -0.027599  0.183926  0.1475

###library(targeted)
###b3a <- ate(cause2~gp.f|dnr+preauto+ttt24| dnr+preauto+ttt24,kumar,family=binomial)
###summary(b3a)

## calculate also relative risk
estimate(coef=b3$riskDR,vcov=b3$var.riskDR,f=function(p) p[1]/p[2])
#>        Estimate Std.Err   2.5% 97.5%   P-value
#> treat0   0.8146  0.1194 0.5807 1.049 8.831e-12
```

Or with continuous response using normal estimating equations


``` r
b3 <- normalATE(time~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)
#>    n events
#>  400    400
#> 
#>  400 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept)  43.4758   4.0796  35.4800  51.4716  0.0000
#> gp.f1       -27.2211   4.1379 -35.3313 -19.1109  0.0000
#> dnr           2.9485   4.4768  -5.8259  11.7228  0.5101
#> preauto      -2.8172   3.6191  -9.9105   4.2760  0.4363
#> ttt24        -1.9787   4.1285 -10.0704   6.1131  0.6318
#> 
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     42.3170   3.8362  34.7982  49.8357       0
#> treat1     15.0958   1.3284  12.4922  17.6994       0
#> treat:1-0 -27.2211   4.1379 -35.3313 -19.1109       0
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     42.3200   4.0417  34.3985  50.2415       0
#> treat1     15.1545   1.2580  12.6888  17.6201       0
#> treat:1-0 -27.1655   4.2405 -35.4768 -18.8543       0
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         mvtnorm_1.3-7         
#> [13] numDeriv_2016.8-1.1    compiler_4.6.0         codetools_0.2-20      
#> [16] Rcpp_1.1.1-1.1         ucminf_1.2.3           future_1.70.0         
#> [19] lattice_0.22-9         digest_0.6.39          parallelly_1.47.0     
#> [22] parallel_4.6.0         splines_4.6.0          Matrix_1.7-5          
#> [25] tools_4.6.0            RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
# YAML header created by ox-ravel
title: Twin models
author: Klaus Holst & Thomas Scheike
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Twin models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This document provides a brief tutorial to analyzing twin data using the
**`mets`** package:

```{r  include=FALSE,echo=FALSE,message=FALSE,warning=FALSE }
options(warn=-1, family="Times")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1
  ##dev="png",
  ##dpi=72,
  ## out.width = "70%")
)
library("mets")
```

$$
\newcommand{\cov}{\mathbb{C}\text{ov}}
\newcommand{\cor}{\mathbb{C}\text{or}}
\newcommand{\var}{\mathbb{V}\text{ar}}
\newcommand{\E}{\mathbb{E}}
\newcommand{\unitfrac}[2]{#1/#2}
\newcommand{\n}{}
$$

 <!-- 
 The development version may be installed from *github*: 
 -->
```{r  install, eval=FALSE, echo=FALSE}
# install.packages("remotes")
remotes::install_github("kkholst/mets", dependencies="Suggests")
```


# Twin analysis, continuous traits

  In the following we examine the heritability of Body Mass
Index\n{}<sup id="b71edfd9bc946c317f4a732845bcaf93"><a href="#korkeila_bmi_1991" title="Korkeila, Kaprio, Rissanen \&amp; Koskenvuo, {{E}ffects of gender and age on the heritability of body mass index}, {Int J Obes}, v(10), 647--654 (1991).">korkeila_bmi_1991</a></sup> <sup id="718839fcb6ade82ebb2d7de853582b80"><a href="#hjelmborg_bmi_2008" title="Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen \&amp; Kaprio, {{G}enetic influences on growth traits of {B}{M}{I}: a longitudinal study of adult twins}, {Obesity (Silver Spring)}, v(4), 847--852 (2008).">hjelmborg_bmi_2008</a></sup>, based on data
on self-reported BMI-values from a random sample of 11,411 same-sex
twins. First, we will load data

```{r  twinbmi }
library(mets)
data("twinbmi")
head(twinbmi)
```

The data is on *long* format with one subject per row.

-   **`tvparnr`:** twin id
-   **`bmi`:** Body Mass Index (\(\mathrm{kg}/{\mathrm{m}^2}\))
-   **`age`:** Age (years)
-   **`gender`:** Gender factor (male,female)
-   **`zyg`:** zygosity (MZ, DZ)

We transpose the data allowing us to do pairwise analyses

```{r  twinwide }
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
head(twinwide)
```

Next we plot the association within each zygosity group

We show the log-transformed data here, which is slightly more symmetric
and more appropriate for the twin analysis (see Figure \@ref(fig:scatter1) and \@ref(fig:scatter2)).

```{r  scatter1, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in MZ twins" }
mz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
plot_twin(mz)
```

```{r  scatter2, warning=FALSE,message=FALSE,fig.cap="Scatter plot of logarithmic BMI measurements in DZ twins" }
dz <- log(subset(twinwide, zyg=="DZ")[,c("bmi1","bmi2")])
plot_twin(dz)
```

The plots and raw association measures show considerably stronger
dependence in the MZ twins, indicating genetic influence on the trait.

```{r   }
cor.test(mz[,1],mz[,2], method="spearman")
```

```{r   }
cor.test(dz[,1],dz[,2], method="spearman")
```

Next we examine the marginal distribution (GEE model with working independence):

```{r gee  }
l0 <- lm(bmi ~ gender + I(age-40), data=twinbmi)
estimate(l0, id=twinbmi$tvparnr)
```

```{r  }
library("splines")
l1 <- lm(bmi ~ gender*ns(age,3), data=twinbmi)
marg1 <- estimate(l1, id=twinbmi$tvparnr)
```

```{r  marg1, warning=FALSE,message=FALSE,fig.cap="Marginal association between BMI and Age for males and females."}
dm <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("male"),
	    age=seq(33,61,length.out=50))
df <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("female"),
	    age=seq(33,61,length.out=50))

plot(marg1, function(p) model.matrix(l1,data=dm)%*%p,
     data=dm["age"], ylab="BMI", xlab="Age",
     ylim=c(22,26.5))
plot(marg1, function(p) model.matrix(l1,data=df)%*%p,
     data=df["age"], col="red", add=TRUE)
legend("bottomright", c("Male","Female"),
       col=c("black","red"), lty=1, bty="n")
```


## Polygenic model

We can decompose the trait into the following variance components

\begin{align*}
Y_i = A_i + D_i + C + E_i, \quad i=1,2
 \end{align*}

-   **\(A\):** Additive genetic effects of alleles
-   **\(D\):** Dominant genetic effects of alleles
-   **\(C\):** Shared environmental effects
-   **\(E\):** Unique environmental effects

Dissimilarity of MZ twins arises from unshared environmental effects
only, \(\cor(E_1,E_2)=0\) and

\begin{align*}
\cor(A_1^{MZ},A_2^{MZ}) = 1, \quad
\cor(D_1^{MZ},D_2^{MZ}) = 1,
\end{align*}

\begin{align*}
\cor(A_1^{DZ},A_2^{DZ}) = 0.5, \quad
\cor(D_1^{DZ},D_2^{DZ}) = 0.25,
\end{align*}

\begin{align*}
Y_i = A_i + C_i + D_i + E_i
\end{align*}

\begin{align*}
A_i \sim\mathcal{N}(0,\sigma_A^2), C_i
\sim\mathcal{N}(0,\sigma_C^2), D_i
\sim\mathcal{N}(0,\sigma_D^2),
E_i \sim\mathcal{N}(0,\sigma_E^2)
\end{align*}

\begin{gather*}
    \cov(Y_{1},Y_{2}) = \\
    \begin{pmatrix}
      \sigma_A^2 & 2\Phi\sigma_A^2 \\
      2\Phi\sigma_A^2 & \sigma_A^2
    \end{pmatrix} +
    \begin{pmatrix}
      \sigma_C^2 & \sigma_C^2 \\
      \sigma_C^2 & \sigma_C^2
  \end{pmatrix} +
    \begin{pmatrix}
      \sigma_D^2 & \Delta_{7}\sigma_D^2 \\
      \Delta_{7}\sigma_D^2 & \sigma_D^2
  \end{pmatrix} +
  \begin{pmatrix}
    \sigma_E^2 & 0 \\
    0 & \sigma_E^2
  \end{pmatrix}
\end{gather*}


```{r}
dd <- na.omit(twinbmi)
```

Saturated model (different marginals in MZ and DZ twins and different marginals
for twin 1 and twin 2):
```{r lmsat, eval=FALSE}
l0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="sat")
```

Different marginals for MZ and DZ twins (but same marginals within a pair)
```{r lmflex, eval=FALSE}
lf <- twinlm(bmi ~ age+gender, data=dd,DZ="DZ", zyg="zyg", id="tvparnr", type="flex")
```
 
Same marginals but free correlation with MZ, DZ
```{r lmeqmarg}
lu <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="eqmarg")
estimate(lu)
```

A formal test of genetic effects can be obtained by comparing the MZ and DZ correlation:
```{r   }
estimate(lu,lava::contr(5:6,6))
```

We also consider the ACE model
```{r  ace}
ace0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="ace")
summary(ace0)
```


# Bibliography
<a id="korkeila_bmi_1991"></a>[korkeila_bmi_1991] Korkeila, Kaprio, Rissanen & Koskenvuo, Effects of gender and age on the heritability of body mass index, <i>Int J Obes</i>, <b>15(10)</b>, 647-654 (1991). [↩](#b71edfd9bc946c317f4a732845bcaf93)

<a id="hjelmborg_bmi_2008"></a>[hjelmborg_bmi_2008] Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen & Kaprio, Genetic influences on growth traits of BMI: a longitudinal study of adult twins, <i>Obesity (Silver Spring)</i>, <b>16(4)</b>, 847-852 (2008). [↩](#718839fcb6ade82ebb2d7de853582b80)

---
title: "G-Computation or standardization for the Cox, Fine-Gray and binomial regression models for survival data" 
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) based on the Cox and Fine-Gray model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```


G-computation for the Cox and Fine-Gray models 
==============================================

We compute the standardized estimator (G-estimation) based on the Cox or the Fine-Gray model:
\[
\hat S(t,A=a) = n^{-1} \sum_i S(t,A=a,X_i) 
\]
and this estimator has influence function
\[
S(t,A=a,X_i) -  S(t,A=a)  + E( D_{A_0(t), \beta} S(t,A=a,X_i)  ) \epsilon_i(t)
\]
where $\epsilon_i(t)$ is the iid decomposition 
of the baseline and regression coefficients $(\hat A(t) - A(t), \hat \beta- \beta)$.

These estimates have a causal interpretation under the assumption
of no unmeasured confounders; even without the
causal assumption, standardisation can still be a useful summary measure.

We begin by looking at the cumulative incidence via a Fine-Gray model for both causes and plotting the
standardised cumulative incidence for cause 1.

```{r}
library(mets)
set.seed(100)

data(bmt); 
bmt$time <- bmt$time+runif(nrow(bmt))*0.001
dfactor(bmt) <- tcell~tcell
bmt$event <- (bmt$cause!=0)*1

fg1 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=1)
Gfg1 <- survivalG(fg1,bmt,time=50)
summary(Gfg1)

fg2 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=2)
summary(survivalG(fg2,bmt,time=50))

cif1time <- survivalGtime(fg1,bmt)
plot(cif1time,type="risk"); 

```

Now looking at the survival probability 

```{r}
ss <- phreg(Surv(time,event)~tcell+platelet+age,bmt)
sss <- survivalG(ss,bmt,time=50)
summary(sss)

Gtime <- survivalGtime(ss,bmt)
plot(Gtime)
```

G-computation for the binomial regression
==============================================

We compare with similar estimates using doubly robust
estimating equations via `binregATE`. The G-computation standardisation can also be computed using a specialised function that
uses less memory and is faster for large data: `binregG`.

```{r}

## survival situation
sr1 <- binregATE(Event(time,event)~tcell+platelet+age,bmt,cause=1,
		 time=40, treat.model=tcell~platelet+age)
summary(sr1)

## relative risk effect 
estimate(coef=sr1$riskDR,vcov=sr1$var.riskDR,f=function(p) p[2]/p[1],null=1)

## competing risks 
br1 <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,cause=1,
		 time=40,treat.model=tcell~platelet+age)
summary(br1)
```

and using the specialized function 

```{r}
br1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=40)
Gbr1 <- binregG(br1,bmt,Avalues=NULL)
summary(Gbr1)

## contrasting average age to 1+2-sd age, Avalues
Gbr2 <- binregG(br1,bmt,varname="age",Avalues=c(0,1,2))
summary(Gbr2)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "G-Computation or standardization for the Cox, Fine-Gray and binomial regression models for survival data" 
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{G-Computation or standardization for the Cox, Fine-Gray and binomial regression models for survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




G-computation for the Cox and Fine-Gray models 
==============================================

We compute the standardized estimator (G-estimation) based on the Cox or the Fine-Gray model:
\[
\hat S(t,A=a) = n^{-1} \sum_i S(t,A=a,X_i) 
\]
and this estimator has influence function
\[
S(t,A=a,X_i) -  S(t,A=a)  + E( D_{A_0(t), \beta} S(t,A=a,X_i)  ) \epsilon_i(t)
\]
where $\epsilon_i(t)$ is the iid decomposition 
of the baseline and regression coefficients $(\hat A(t) - A(t), \hat \beta- \beta)$.

These estimates have a causal interpretation under the assumption
of no unmeasured confounders; even without the
causal assumption, standardisation can still be a useful summary measure.

We begin by looking at the cumulative incidence via a Fine-Gray model for both causes and plotting the
standardised cumulative incidence for cause 1.


``` r
library(mets)
set.seed(100)

data(bmt); 
bmt$time <- bmt$time+runif(nrow(bmt))*0.001
dfactor(bmt) <- tcell~tcell
bmt$event <- (bmt$cause!=0)*1

fg1 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=1)
Gfg1 <- survivalG(fg1,bmt,time=50)
summary(Gfg1)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.4331 0.02749 0.3793 0.4870 6.321e-56
#> risk1   0.2727 0.05863 0.1577 0.3876 3.313e-06
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err   2.5%    97.5% P-value
#> ps0  -0.1605 0.06353 -0.285 -0.03597 0.01153
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>       Estimate   Std.Err       2.5%       97.5%    P-value
#> ps0 -0.4628288 0.2212039 -0.8963806 -0.02927703 0.03641016
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.6295004 0.4080439 0.9711474

fg2 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=2)
summary(survivalG(fg2,bmt,time=50))
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.2127 0.02314 0.1674 0.2581 3.757e-20
#> risk1   0.3336 0.06799 0.2003 0.4668 9.281e-07
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err     2.5%  97.5% P-value
#> ps0   0.1208 0.07189 -0.02009 0.2617 0.09285
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>      Estimate   Std.Err         2.5%     97.5%   P-value
#> ps0 0.4497465 0.2313601 -0.003710973 0.9032039 0.0519046
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 1.5679146 0.9962959 2.4674960

cif1time <- survivalGtime(fg1,bmt)
plot(cif1time,type="risk"); 
```

<img src="figure/survival-ate-unnamed-chunk-2-1.png" alt="Standardized cumulative incidence for cause 1."  />

Now looking at the survival probability 


``` r
ss <- phreg(Surv(time,event)~tcell+platelet+age,bmt)
sss <- survivalG(ss,bmt,time=50)
summary(sss)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%    P-value
#> risk0   0.6539 0.02709 0.6008 0.7070 9.218e-129
#> risk1   0.5640 0.05971 0.4470 0.6811  3.531e-21
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err    2.5%   97.5% P-value
#> ps0 -0.08992  0.0629 -0.2132 0.03337  0.1529
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>       Estimate   Std.Err       2.5%      97.5%   P-value
#> ps0 -0.1479231 0.1095247 -0.3625876 0.06674132 0.1768263
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.8624974 0.6958733 1.0690189 
#> 
#> Average Treatment effect:  survival-difference (G-estimator) :
#>       Estimate    Std.Err        2.5%     97.5%   P-value
#> ps0 0.08991862 0.06290398 -0.03337092 0.2132082 0.1528725
#> 
#> Average Treatment effect: 1-G (survival)-ratio (G-estimator) :
#> log-ratio: 
#>      Estimate   Std.Err       2.5%     97.5%   P-value
#> ps0 0.2309818 0.1503867 -0.0637708 0.5257343 0.1245583
#> ratio: 
#> Estimate     2.5%    97.5% 
#> 1.259836 0.938220 1.691701

Gtime <- survivalGtime(ss,bmt)
plot(Gtime)
```

<img src="figure/survival-ate-unnamed-chunk-3-1.png" alt="Standardized survival probability over time."  />

G-computation for the binomial regression
==============================================

We compare with similar estimates using doubly robust
estimating equations via `binregATE`. The G-computation standardisation can also be computed using a specialised function that
uses less memory and is faster for large data: `binregG`.


``` r

## survival situation
sr1 <- binregATE(Event(time,event)~tcell+platelet+age,bmt,cause=1,
		 time=40, treat.model=tcell~platelet+age)
summary(sr1)
#>    n events
#>  408    241
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  0.676409  0.137007  0.407880  0.944939  0.0000
#> tcell1      -0.023675  0.346994 -0.703770  0.656420  0.9456
#> platelet    -0.492952  0.246158 -0.975412 -0.010492  0.0452
#> age          0.343939  0.115561  0.117444  0.570434  0.0029
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  1.96680 1.50363 2.5727
#> tcell1       0.97660 0.49472 1.9279
#> platelet     0.61082 0.37704 0.9896
#> age          1.41049 1.12462 1.7690
#> 
#> Average Treatment effects (G-formula) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     0.6230976  0.0273827  0.5694284  0.6767667  0.0000
#> treat1     0.6177595  0.0731712  0.4743466  0.7611723  0.0000
#> treat:1-0 -0.0053381  0.0783973 -0.1589940  0.1483179  0.9457
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.622698  0.027460  0.568878  0.676518   0.000
#> treat1     0.637785  0.085242  0.470714  0.804857   0.000
#> treat:1-0  0.015087  0.089442 -0.160215  0.190389   0.866

## relative risk effect 
summary(estimate(coef=sr1$riskDR,vcov=sr1$var.riskDR,f=function(p) p[2]/p[1]),null=1)
#> Call: estimate.default(f = function(p) p[2]/p[1], coef = sr1$riskDR, 
#>     vcov = sr1$var.riskDR)
#> ────────────────────────────────────────────────────────────
#>        Estimate Std.Err   2.5% 97.5% P-value
#> treat1    1.024   0.144 0.7421 1.306  0.8664
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treat1] = 1 
#>  
#> chisq = 0.0283, df = 1, p-value = 0.8664

## competing risks 
br1 <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,cause=1,
		 time=40,treat.model=tcell~platelet+age)
summary(br1)
#>    n events
#>  408    157
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.191519  0.130883 -0.448044  0.065007  0.1434
#> tcell1      -0.712880  0.351489 -1.401786 -0.023974  0.0425
#> platelet    -0.531919  0.244495 -1.011119 -0.052718  0.0296
#> age          0.432939  0.107314  0.222607  0.643271  0.0001
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.82570 0.63888 1.0672
#> tcell1       0.49023 0.24616 0.9763
#> platelet     0.58748 0.36381 0.9486
#> age          1.54178 1.24933 1.9027
#> 
#> Average Treatment effects (G-formula) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.417746  0.027030  0.364768  0.470724  0.0000
#> treat1     0.267097  0.061849  0.145874  0.388319  0.0000
#> treat:1-0 -0.150649  0.067578 -0.283100 -0.018199  0.0258
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.417121  0.027112  0.363983  0.470259  0.0000
#> treat1     0.227776  0.061240  0.107748  0.347803  0.0002
#> treat:1-0 -0.189345  0.066600 -0.319878 -0.058812  0.0045
```

and using the specialized function 


``` r
br1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=40)
Gbr1 <- binregG(br1,bmt,Avalues=NULL)
summary(Gbr1)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.4177 0.02703 0.3648 0.4707 6.988e-54
#> risk1   0.2671 0.06185 0.1459 0.3883 1.571e-05
#> 
#> Average Treatment effect: difference (G-estimator) :
#> Call: estimate.default(coef = Gest$Gest, vcov = vv)
#> ────────────────────────────────────────────────────────────
#>      Estimate Std.Err   2.5%  97.5%   P-value
#> pa     0.4177 0.02703 0.3648 0.4707 6.988e-54
#> pa.1   0.2671 0.06185 0.1459 0.3883 1.571e-05
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [pa] = 0
#>   [pa] = 0 
#>  
#> chisq = 257.9401, df = 2, p-value < 2.2e-16
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>     Estimate    Std.Err      2.5%     97.5%      P-value
#> pa 0.4177459 0.02702993 0.3647682 0.4707236 6.987948e-54
#> pa 0.2670966 0.06184921 0.1458744 0.3883188 1.570838e-05
#> ratio: 
#>    Estimate     2.5%    97.5%
#> pa 1.518535 1.440180 1.601152
#> pa 1.306167 1.157051 1.474500

## contrasting average age to 1+2-sd age, Avalues
Gbr2 <- binregG(br1,bmt,varname="age",Avalues=c(0,1,2))
summary(Gbr2)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.3932 0.02537 0.3434 0.4429 3.738e-54
#> risk1   0.4964 0.03655 0.4248 0.5681 5.044e-42
#> risk2   0.5997 0.05531 0.4913 0.7081 2.136e-27
#> 
#> Average Treatment effect: difference (G-estimator) :
#> Call: estimate.default(coef = Gest$Gest, vcov = vv)
#> ────────────────────────────────────────────────────────────
#>      Estimate Std.Err   2.5%  97.5%   P-value
#> pa     0.3932 0.02537 0.3434 0.4429 3.738e-54
#> pa.1   0.4964 0.03655 0.4248 0.5681 5.044e-42
#> pa.2   0.5997 0.05531 0.4913 0.7081 2.136e-27
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [pa] = 0
#>   [pa] = 0
#>   [pa] = 0 
#>  
#> chisq = 252.683, df = 3, p-value < 2.2e-16
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>     Estimate    Std.Err      2.5%     97.5%      P-value
#> pa 0.3931626 0.02537319 0.3434321 0.4428932 3.738061e-54
#> pa 0.4964392 0.03654828 0.4248059 0.5680725 5.044045e-42
#> pa 0.5997207 0.05530589 0.4913231 0.7081182 2.136422e-27
#> ratio: 
#>    Estimate     2.5%    97.5%
#> pa 1.481659 1.409778 1.557206
#> pa 1.642861 1.529294 1.764862
#> pa 1.821610 1.634477 2.030167
```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              KernSmooth_2.23-26     otel_0.2.0            
#>  [7] future.apply_1.20.2    listenv_0.10.1         lava_1.9.2.1          
#> [10] stats4_4.6.0           grid_4.6.0             evaluate_1.0.5        
#> [13] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [16] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [19] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [22] digest_0.6.39          parallelly_1.47.0      parallel_4.6.0        
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "dUtility data-frame manipulations"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes 
vignette: >
  %\VignetteIndexEntry{dUtility data-frame manipulations} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->





Simple data manipulation for data-frames 
==========================================

   *  Renaming variables, Deleting variables
   *  Looking at the data
   *  Making new variables for the analysis
   *  Making factors (groupings)
   *  Working with factors
   *  Making a factor from existing numeric variable and vice versa


Here are some key data-manipulation steps on a data frame, which is how we typically
organise our data in R.
After reading the data into R it will typically be a data frame; if not, we
can force it to be one. The basic idea of the utility functions is
to provide a simple and easy-to-type way of performing common data manipulations on a
data frame, much like what is possible in SAS or Stata.

The functions `dcut`, `dfactor`, and so on do essentially what
the base R `cut` and `factor` functions do, but are easier to use in the context of data frames
and have additional functionality.


``` r
library(mets)
data(melanoma)
```


``` r
is.data.frame(melanoma)
#> [1] TRUE
```

Here we work on the melanoma data, which has already been read into R and is
a data frame.


dUtility functions 
==================

The structure for all functions is

*  `dfunction(dataframe, y~x|ifcond, ...)`

to apply the function to `y` in a data frame, grouped by `x`, and only when
condition `ifcond` is satisfied. The basic functions are:

Data processing
    * dsort
    * dreshape
    * dcut
    * drm, drename, ddrop, dkeep, dsubset
    * drelevel
    * dlag
    * dfactor, dnumeric

Data aggregation
    * dby, dby2
    * dscalar, deval, daggregate
    * dmean, dsd, dsum, dquantile, dcor
    * dtable, dcount

Data summaries
    * dhead, dtail, 
    * dsummary, 
    * dprint, dlist, dlevels, dunique


A generic function daggregate, daggr,  can be called with a function as the 
argument:

*  `daggregate(dataframe, y~x|ifcond, fun=function, ...)`

or without the grouping variable (`x`):

*  `daggregate(dataframe, ~y|ifcond, fun=function, ...)`

A useful feature is that `y` and `x` as well as the subset condition
can be specified using regular expressions or wildcards (default).
Here we compute the means of certain variables.

First, overall:

``` r
dmean(melanoma,~thick+I(log(thick)))
#>         thick I(log(thick)) 
#>    291.985366      5.223341
```

now only when days>500

``` r
dmean(melanoma,~thick+I(log(thick))|I(days>500))
#>         thick I(log(thick)) 
#>    271.582011      5.168691
```

and now after sex but only when days>500


``` r
dmean(melanoma,thick+I(log(thick))~sex|I(days>500))
#>   sex    thick I(log(thick))
#> 1   0 242.9580      5.060086
#> 2   1 320.2429      5.353321
```

and finally after quartiles of days (via the dcut function)


``` r
dmean(melanoma,thick+I(log(thick))~I(dcut(days)))
#>         I(dcut(days))    thick I(log(thick))
#> 1       [10,1.52e+03] 482.1731      5.799525
#> 2    (1.52e+03,2e+03] 208.5490      4.987652
#> 3    (2e+03,3.04e+03] 223.2941      4.974759
#> 4 (3.04e+03,5.56e+03] 250.1961      5.120129
```

or summary of all variables starting with "s" and that contains "a"


``` r
dmean(melanoma,"s*"+"*a*"~sex|I(days>500))
#>   sex   status     days
#> 1   0 1.831933 2399.143
#> 2   1 1.714286 2169.800
```


Renaming, deleting, keeping, dropping  variables
===============================================


``` r
melanoma=drename(melanoma,tykkelse~thick)
names(melanoma)
#> [1] "no"       "status"   "days"     "ulc"      "tykkelse" "sex"
```

Deleting variables


``` r
data(melanoma)
melanoma=drm(melanoma,~thick+sex)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

or SAS style:


``` r
data(melanoma)
melanoma=ddrop(melanoma,~thick+sex)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

alternatively, we can keep certain variables:


``` r
data(melanoma)
melanoma=dkeep(melanoma,~thick+sex+status+days)
names(melanoma)
#> [1] "thick"  "sex"    "status" "days"
```

This can also be done with direct assignment 


``` r
data(melanoma)
ddrop(melanoma) <- ~thick+sex
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

The dkeep function can also be used to re-ordering the variables in the data-frame


``` r
data(melanoma)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"    "thick"  "sex"
melanoma=dkeep(melanoma,~days+status+.)
names(melanoma)
#> [1] "days"   "status" "no"     "ulc"    "thick"  "sex"
```


Looking at the data 
===================


``` r
data(melanoma)
dstr(melanoma)
#> 'data.frame':	205 obs. of  6 variables:
#>  $ no    : int  789 13 97 16 21 469 685 7 932 944 ...
#>  $ status: int  3 3 2 3 1 1 1 1 3 1 ...
#>  $ days  : int  10 30 35 99 185 204 210 232 232 279 ...
#>  $ ulc   : int  1 0 0 0 1 1 1 1 1 1 ...
#>  $ thick : int  676 65 134 290 1208 484 516 1288 322 741 ...
#>  $ sex   : int  1 1 1 0 1 1 1 1 0 0 ...
```


The data can be viewed as a data table in RStudio, but to list certain parts of
the data in the output window:


``` r
dlist(melanoma)
#>     no  status days ulc thick sex
#> 1   789 3       10  1    676  1  
#> 2    13 3       30  0     65  1  
#> 3    97 2       35  0    134  1  
#> 4    16 3       99  0    290  0  
#> 5    21 1      185  1   1208  1  
#> ---                              
#> 201 317 2      4492 1   706   1  
#> 202 798 2      4668 0   612   0  
#> 203 806 2      4688 0    48   0  
#> 204 606 2      4926 0   226   0  
#> 205 328 2      5565 0   290   0
```


``` r
dlist(melanoma, ~.|sex==1)
#>     no  status days ulc thick
#> 1   789 3       10  1    676 
#> 2    13 3       30  0     65 
#> 3    97 2       35  0    134 
#> 5    21 1      185  1   1208 
#> 6   469 1      204  1    484 
#> ---                          
#> 191 445 2      3909 1   806  
#> 195 415 2      4119 0    65  
#> 197 175 2      4207 0    65  
#> 198 493 2      4310 0   210  
#> 201 317 2      4492 1   706
```


``` r
dlist(melanoma, ~ulc+days+thick+sex|sex==1)
#>     ulc days thick sex
#> 1   1    10   676  1  
#> 2   0    30    65  1  
#> 3   0    35   134  1  
#> 5   1   185  1208  1  
#> 6   1   204   484  1  
#> ---                   
#> 191 1   3909 806   1  
#> 195 0   4119  65   1  
#> 197 0   4207  65   1  
#> 198 0   4310 210   1  
#> 201 1   4492 706   1
```

Getting summaries


``` r
dsummary(melanoma)
#>        no            status          days           ulc            thick     
#>  Min.   :  2.0   Min.   :1.00   Min.   :  10   Min.   :0.000   Min.   :  10  
#>  1st Qu.:222.0   1st Qu.:1.00   1st Qu.:1525   1st Qu.:0.000   1st Qu.:  97  
#>  Median :469.0   Median :2.00   Median :2005   Median :0.000   Median : 194  
#>  Mean   :463.9   Mean   :1.79   Mean   :2153   Mean   :0.439   Mean   : 292  
#>  3rd Qu.:731.0   3rd Qu.:2.00   3rd Qu.:3042   3rd Qu.:1.000   3rd Qu.: 356  
#>  Max.   :992.0   Max.   :3.00   Max.   :5565   Max.   :1.000   Max.   :1742  
#>       sex        
#>  Min.   :0.0000  
#>  1st Qu.:0.0000  
#>  Median :0.0000  
#>  Mean   :0.3854  
#>  3rd Qu.:1.0000  
#>  Max.   :1.0000
```

or for specific variables


``` r
dsummary(melanoma,~thick+status+sex)
#>      thick          status          sex        
#>  Min.   :  10   Min.   :1.00   Min.   :0.0000  
#>  1st Qu.:  97   1st Qu.:1.00   1st Qu.:0.0000  
#>  Median : 194   Median :2.00   Median :0.0000  
#>  Mean   : 292   Mean   :1.79   Mean   :0.3854  
#>  3rd Qu.: 356   3rd Qu.:2.00   3rd Qu.:1.0000  
#>  Max.   :1742   Max.   :3.00   Max.   :1.0000
```


Summaries in different groups (sex)


``` r
dsummary(melanoma,thick+days+status~sex)
#> sex: 0
#>      thick             days          status     
#>  Min.   :  10.0   Min.   :  99   Min.   :1.000  
#>  1st Qu.:  97.0   1st Qu.:1636   1st Qu.:2.000  
#>  Median : 162.0   Median :2059   Median :2.000  
#>  Mean   : 248.6   Mean   :2283   Mean   :1.833  
#>  3rd Qu.: 306.0   3rd Qu.:3131   3rd Qu.:2.000  
#>  Max.   :1742.0   Max.   :5565   Max.   :3.000  
#> ------------------------------------------------------------ 
#> sex: 1
#>      thick             days          status     
#>  Min.   :  16.0   Min.   :  10   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1052   1st Qu.:1.000  
#>  Median : 258.0   Median :1860   Median :2.000  
#>  Mean   : 361.1   Mean   :1946   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2784   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :4492   Max.   :3.000
```


and only among those with thin-tumours or only females (sex==1)


``` r
dsummary(melanoma,thick+days+status~sex|thick<97)
#> sex: 0
#>      thick            days          status     
#>  Min.   :10.00   Min.   : 355   Min.   :1.000  
#>  1st Qu.:32.00   1st Qu.:1762   1st Qu.:2.000  
#>  Median :64.00   Median :2227   Median :2.000  
#>  Mean   :51.48   Mean   :2425   Mean   :2.034  
#>  3rd Qu.:65.00   3rd Qu.:3185   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4688   Max.   :3.000  
#> ------------------------------------------------------------ 
#> sex: 1
#>      thick            days          status     
#>  Min.   :16.00   Min.   :  30   Min.   :1.000  
#>  1st Qu.:30.00   1st Qu.:1820   1st Qu.:2.000  
#>  Median :65.00   Median :2886   Median :2.000  
#>  Mean   :55.75   Mean   :2632   Mean   :1.875  
#>  3rd Qu.:81.00   3rd Qu.:3328   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4207   Max.   :3.000
```


``` r
dsummary(melanoma,thick+status~+1|sex==1)
#>      thick            status     
#>  Min.   :  16.0   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1.000  
#>  Median : 258.0   Median :2.000  
#>  Mean   : 361.1   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :3.000
```

or 


``` r
dsummary(melanoma,~thick+status|sex==1)
#>      thick            status     
#>  Min.   :  16.0   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1.000  
#>  Median : 258.0   Median :2.000  
#>  Mean   : 361.1   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :3.000
```


For more complex conditions, use `I()`:


``` r
dsummary(melanoma,thick+days+status~sex|I(thick<97 & sex==1))
#> sex: 1
#>      thick            days          status     
#>  Min.   :16.00   Min.   :  30   Min.   :1.000  
#>  1st Qu.:30.00   1st Qu.:1820   1st Qu.:2.000  
#>  Median :65.00   Median :2886   Median :2.000  
#>  Mean   :55.75   Mean   :2632   Mean   :1.875  
#>  3rd Qu.:81.00   3rd Qu.:3328   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4207   Max.   :3.000
```

Tables between variables


``` r
dtable(melanoma,~status+sex)
#> 
#>        sex  0  1
#> status          
#> 1          28 29
#> 2          91 43
#> 3           7  7
```

All bivariate tables


``` r
dtable(melanoma,~status+sex+ulc,level=2)
#> 
#>    status
#> sex  1  2  3
#>   0 28 91  7
#>   1 29 43  7
#> 
#>    status
#> ulc  1  2  3
#>   0 16 92  7
#>   1 41 42  7
#> 
#>    sex
#> ulc  0  1
#>   0 79 36
#>   1 47 43
```

All univariate tables


``` r
dtable(melanoma,~status+sex+ulc,level=1)
#> 
#> status
#>   1   2   3 
#>  57 134  14 
#> 
#> sex
#>   0   1 
#> 126  79 
#> 
#> ulc
#>   0   1 
#> 115  90
```

and with new variables 


``` r
dtable(melanoma,~status+sex+ulc+dcut(days)+I(days>300),level=1)
#> 
#> status
#>   1   2   3 
#>  57 134  14 
#> 
#> sex
#>   0   1 
#> 126  79 
#> 
#> ulc
#>   0   1 
#> 115  90 
#> 
#> dcut(days)
#>       [10,1.52e+03]    (1.52e+03,2e+03]    (2e+03,3.04e+03] (3.04e+03,5.56e+03] 
#>                  52                  51                  51                  51 
#> 
#> I(days > 300)
#> FALSE  TRUE 
#>    11   194
```


Sorting the data
===============

To sort the data 


``` r
data(melanoma)
mel= dsort(melanoma,~days)
dsort(melanoma) <- ~days
head(mel)
#>    no status days ulc thick sex
#> 1 789      3   10   1   676   1
#> 2  13      3   30   0    65   1
#> 3  97      2   35   0   134   1
#> 4  16      3   99   0   290   0
#> 5  21      1  185   1  1208   1
#> 6 469      1  204   1   484   1
```


and to sort after multiple variables increasing and decreasing 


``` r
dsort(melanoma) <- ~days-status
head(melanoma)
#>    no status days ulc thick sex
#> 1 789      3   10   1   676   1
#> 2  13      3   30   0    65   1
#> 3  97      2   35   0   134   1
#> 4  16      3   99   0   290   0
#> 5  21      1  185   1  1208   1
#> 6 469      1  204   1   484   1
```


Making new variables for the analysis
=====================================

To define a bunch of new covariates within a data-frame


``` r
data(melanoma)
melanoma= transform(melanoma, thick2=thick^2, lthick=log(thick) ) 
dhead(melanoma)
#>    no status days ulc thick sex  thick2   lthick
#> 1 789      3   10   1   676   1  456976 6.516193
#> 2  13      3   30   0    65   1    4225 4.174387
#> 3  97      2   35   0   134   1   17956 4.897840
#> 4  16      3   99   0   290   0   84100 5.669881
#> 5  21      1  185   1  1208   1 1459264 7.096721
#> 6 469      1  204   1   484   1  234256 6.182085
```

When the above definitions are done using a condition this can be achieved 
using the dtransform function that extends transform with a possible condition  


``` r
 melanoma=dtransform(melanoma,ll=thick*1.05^ulc,sex==1)  
 melanoma=dtransform(melanoma,ll=thick,sex!=1)  
 dmean(melanoma,ll~sex+ulc)
#>   sex ulc       ll
#> 1   0   0 173.7342
#> 2   1   0 197.3611
#> 3   0   1 374.5532
#> 4   1   1 523.1198
```


Making factors (groupings)
=============================

On the melanoma data, the variable `thick` gives the thickness of the melanoma tumour. For some analyses we would like to create a factor depending on the thickness. This can be done in several different ways:


``` r
melanoma=dcut(melanoma,~thick,breaks=c(0,200,500,800,2000))
```


New variable is named `thickcat.0` by default.

To see levels of factors in data-frame


``` r
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
```


Checking group sizes 


``` r
dtable(melanoma,~thickcat.0)
#> 
#> thickcat.0
#>     [0,200]   (200,500]   (500,800] (800,2e+03] 
#>         109          64          20          12
```


With adding to the data-frame directly 


``` r
dcut(melanoma,breaks=c(0,200,500,800,2000)) <- gr.thick1~thick
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> gr.thick1 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
```

new variable is named thickcat.0 (after first cut-point), 
or to get quartiles with default names thick.cat.4 


``` r
dcut(melanoma) <- ~ thick  # new variable is thickcat.4
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> gr.thick1 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> thickcat.4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

or median groups, here starting again with the original data, 


``` r
data(melanoma)
dcut(melanoma,breaks=2) <- ~ thick  # new variable is thick.2
dlevels(melanoma)
#> thickcat.2 #levels=:2 
#> [1] "[10,194]"       "(194,1.74e+03]"
#> -----------------------------------------
```


to control new names


``` r
data(melanoma)
mela= dcut(melanoma,thickcat4+dayscat4~thick+days,breaks=4)
dlevels(mela)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> dayscat4 #levels=:4 
#> [1] "[10,1.52e+03]"       "(1.52e+03,2e+03]"    "(2e+03,3.04e+03]"   
#> [4] "(3.04e+03,5.56e+03]"
#> -----------------------------------------
```

or


``` r
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4+dayscat4~thick+days
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> dayscat4 #levels=:4 
#> [1] "[10,1.52e+03]"       "(1.52e+03,2e+03]"    "(2e+03,3.04e+03]"   
#> [4] "(3.04e+03,5.56e+03]"
#> -----------------------------------------
```

This can also be typed out more specifically


``` r
melanoma$gthick = cut(melanoma$thick,breaks=c(0,200,500,800,2000))
melanoma$gthick = cut(melanoma$thick,breaks=quantile(melanoma$thick),include.lowest=TRUE)
```


Working with factors
====================

To see levels of covariates in data-frame


``` r
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4~thick
dlevels(melanoma) 
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

To relevel the factor


``` r
dtable(melanoma,~thickcat4)
#> 
#> thickcat4
#>        [10,97]       (97,194]      (194,356] (356,1.74e+03] 
#>             56             53             45             51
melanoma = drelevel(melanoma,~thickcat4,ref="(194,356]")
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
```


or to take the third level in the list of levels, same as above, 


``` r
melanoma = drelevel(melanoma,~thickcat4,ref=2)
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(97,194]"       "[10,97]"        "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

To combine levels of a factor (first combining the first 3 groups into one):


``` r
melanoma = drelevel(melanoma,~thickcat4,newlevels=1:3)
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(97,194]"       "[10,97]"        "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.1:3 #levels=:2 
#> [1] "[10,97]-(194,356]" "(356,1.74e+03]"   
#> -----------------------------------------
```

or to combine groups 1 and 2 into one group and 3 and 4 into another


``` r
dkeep(melanoma) <- ~thick+thickcat4
melanoma = drelevel(melanoma,gthick2~thickcat4,newlevels=list(1:2,3:4))
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> gthick2 #levels=:2 
#> [1] "[10,97]-(97,194]"         "(194,356]-(356,1.74e+03]"
#> -----------------------------------------
```


Changing order of factor levels 


``` r
dfactor(melanoma,levels=c(3,1,2,4)) <-  thickcat4.2~thickcat4
dlevel(melanoma,~ "thickcat4*")
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
dtable(melanoma,~thickcat4+thickcat4.2)
#> 
#>                thickcat4.2 (194,356] [10,97] (97,194] (356,1.74e+03]
#> thickcat4                                                           
#> [10,97]                            0      56        0              0
#> (97,194]                           0       0       53              0
#> (194,356]                         45       0        0              0
#> (356,1.74e+03]                     0       0        0             51
```

Combine levels but now control factor-level names 


``` r
melanoma=drelevel(melanoma,gthick3~thickcat4,newlevels=list(group1.2=1:2,group3.4=3:4))
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> gthick2 #levels=:2 
#> [1] "[10,97]-(97,194]"         "(194,356]-(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> gthick3 #levels=:2 
#> [1] "group1.2" "group3.4"
#> -----------------------------------------
```

Making a factor from existing numeric variable and vice versa
==============================================================

A numeric variable `status` with values 1, 2, 3 can be converted to a factor by:


``` r
data(melanoma)
melanoma = dfactor(melanoma,~status, labels=c("malignant-melanoma","censoring","dead-other"))
melanoma = dfactor(melanoma,sexl~sex,labels=c("females","males"))
dtable(melanoma,~sexl+status.f)
#> 
#>         status.f malignant-melanoma censoring dead-other
#> sexl                                                    
#> females                          28        91          7
#> males                            29        43          7
```

A gender factor with values `"M"`, `"F"` can be converted to numerics by:


``` r
melanoma = dnumeric(melanoma,~sexl)
dstr(melanoma,"sex*")
#> 'data.frame':	205 obs. of  3 variables:
#>  $ sex   : int  1 1 1 0 1 1 1 1 0 0 ...
#>  $ sexl  : Factor w/ 2 levels "females","males": 2 2 2 1 2 2 2 2 1 1 ...
#>  $ sexl.n: num  2 2 2 1 2 2 2 2 1 1 ...
dtable(melanoma,~'sex*',level=2)
#> 
#>          sex
#> sexl        0   1
#>   females 126   0
#>   males     0  79
#> 
#>       sex
#> sexl.n   0   1
#>      1 126   0
#>      2   0  79
#> 
#>       sexl
#> sexl.n females males
#>      1     126     0
#>      2       0    79
```

SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] mets_1.3.12
#> 
#> loaded via a namespace (and not attached):
#>  [1] digest_0.6.39          numDeriv_2016.8-1.1    codetools_0.2-20      
#>  [4] RcppArmadillo_15.2.6-1 Matrix_1.7-5           xfun_0.57             
#>  [7] lattice_0.22-9         timereg_2.0.7          splines_4.6.0         
#> [10] lava_1.9.2.1           knitr_1.51             parallel_4.6.0        
#> [13] mvtnorm_1.3-7          cli_3.6.6              parallelly_1.47.0     
#> [16] grid_4.6.0             future_1.70.0          compiler_4.6.0        
#> [19] globals_0.19.1         tools_4.6.0            listenv_0.10.1        
#> [22] future.apply_1.20.2    evaluate_1.0.5         survival_3.8-6        
#> [25] Rcpp_1.1.1-1.1         otel_0.2.0             rlang_1.2.0
```

---
title: "dUtility data-frame manipulations"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes 
    fig_width: 7.15
    fig_height: 5.5   
vignette: >
  %\VignetteIndexEntry{dUtility data-frame manipulations} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```



Simple data manipulation for data-frames 
==========================================

   *  Renaming variables, Deleting variables
   *  Looking at the data
   *  Making new variables for the analysis
   *  Making factors (groupings)
   *  Working with factors
   *  Making a factor from existing numeric variable and vice versa


Here are some key data-manipulation steps on a data frame, which is how we typically
organise our data in R.
After reading the data into R it will typically be a data frame; if not, we
can force it to be one. The basic idea of the utility functions is
to provide a simple and easy-to-type way of performing common data manipulations on a
data frame, much like what is possible in SAS or Stata.

The functions `dcut`, `dfactor`, and so on do essentially what
the base R `cut` and `factor` functions do, but are easier to use in the context of data frames
and have additional functionality.

```{r}
library(mets)
data(melanoma)
```

```{r}
is.data.frame(melanoma)
```

Here we work on the melanoma data, which has already been read into R and is
a data frame.


dUtility functions 
==================

The structure for all functions is

*  `dfunction(dataframe, y~x|ifcond, ...)`

to apply the function to `y` in a data frame, grouped by `x`, and only when
condition `ifcond` is satisfied. The basic functions are:

Data processing
    * dsort
    * dreshape
    * dcut
    * drm, drename, ddrop, dkeep, dsubset
    * drelevel
    * dlag
    * dfactor, dnumeric

Data aggregation
    * dby, dby2
    * dscalar, deval, daggregate
    * dmean, dsd, dsum, dquantile, dcor
    * dtable, dcount

Data summaries
    * dhead, dtail, 
    * dsummary, 
    * dprint, dlist, dlevels, dunique


A generic function daggregate, daggr,  can be called with a function as the 
argument:

*  `daggregate(dataframe, y~x|ifcond, fun=function, ...)`

or without the grouping variable (`x`):

*  `daggregate(dataframe, ~y|ifcond, fun=function, ...)`

A useful feature is that `y` and `x` as well as the subset condition
can be specified using regular expressions or wildcards (default).
Here we compute the means of certain variables.

First, overall:
```{r}
dmean(melanoma,~thick+I(log(thick)))
```

now only when days>500
```{r}
dmean(melanoma,~thick+I(log(thick))|I(days>500))
```

and now after sex but only when days>500

```{r}
dmean(melanoma,thick+I(log(thick))~sex|I(days>500))
```

and finally after quartiles of days (via the dcut function)

```{r}
dmean(melanoma,thick+I(log(thick))~I(dcut(days)))
```

or summary of all variables starting with "s" and that contains "a"

```{r}
dmean(melanoma,"s*"+"*a*"~sex|I(days>500))
```


Renaming, deleting, keeping, dropping  variables
===============================================

```{r}
melanoma=drename(melanoma,tykkelse~thick)
names(melanoma)
```

Deleting variables

```{r}
data(melanoma)
melanoma=drm(melanoma,~thick+sex)
names(melanoma)
```

or SAS style:

```{r}
data(melanoma)
melanoma=ddrop(melanoma,~thick+sex)
names(melanoma)
```

alternatively, we can keep certain variables:

```{r}
data(melanoma)
melanoma=dkeep(melanoma,~thick+sex+status+days)
names(melanoma)
```

This can also be done with direct assignment 

```{r}
data(melanoma)
ddrop(melanoma) <- ~thick+sex
names(melanoma)
```

The dkeep function can also be used to re-ordering the variables in the data-frame

```{r}
data(melanoma)
names(melanoma)
melanoma=dkeep(melanoma,~days+status+.)
names(melanoma)
```


Looking at the data 
===================

```{r}
data(melanoma)
dstr(melanoma)
```


The data can be viewed as a data table in RStudio, but to list certain parts of
the data in the output window:

```{r}
dlist(melanoma)
```

```{r}
dlist(melanoma, ~.|sex==1)
```

```{r}
dlist(melanoma, ~ulc+days+thick+sex|sex==1)
```

Getting summaries

```{r}
dsummary(melanoma)
```

or for specific variables

```{r}
dsummary(melanoma,~thick+status+sex)
```


Summaries in different groups (sex)

```{r}
dsummary(melanoma,thick+days+status~sex)
```


and only among those with thin-tumours or only females (sex==1)

```{r}
dsummary(melanoma,thick+days+status~sex|thick<97)
```

```{r}
dsummary(melanoma,thick+status~+1|sex==1)
```

or 

```{r}
dsummary(melanoma,~thick+status|sex==1)
```


For more complex conditions, use `I()`:

```{r}
dsummary(melanoma,thick+days+status~sex|I(thick<97 & sex==1))
```

Tables between variables

```{r}
dtable(melanoma,~status+sex)
```

All bivariate tables

```{r}
dtable(melanoma,~status+sex+ulc,level=2)
```

All univariate tables

```{r}
dtable(melanoma,~status+sex+ulc,level=1)
```

and with new variables 

```{r}
dtable(melanoma,~status+sex+ulc+dcut(days)+I(days>300),level=1)
```


Sorting the data
===============

To sort the data 

```{r}
data(melanoma)
mel= dsort(melanoma,~days)
dsort(melanoma) <- ~days
head(mel)
```


and to sort after multiple variables increasing and decreasing 

```{r}
dsort(melanoma) <- ~days-status
head(melanoma)
```


Making new variables for the analysis
=====================================

To define a bunch of new covariates within a data-frame

```{r}
data(melanoma)
melanoma= transform(melanoma, thick2=thick^2, lthick=log(thick) ) 
dhead(melanoma)
```

When the above definitions are done using a condition this can be achieved 
using the dtransform function that extends transform with a possible condition  

```{r}
 melanoma=dtransform(melanoma,ll=thick*1.05^ulc,sex==1)  
 melanoma=dtransform(melanoma,ll=thick,sex!=1)  
 dmean(melanoma,ll~sex+ulc)
```


Making factors (groupings)
=============================

On the melanoma data, the variable `thick` gives the thickness of the melanoma tumour. For some analyses we would like to create a factor depending on the thickness. This can be done in several different ways:

```{r}
melanoma=dcut(melanoma,~thick,breaks=c(0,200,500,800,2000))
```


New variable is named `thickcat.0` by default.

To see levels of factors in data-frame

```{r}
dlevels(melanoma)
```


Checking group sizes 

```{r}
dtable(melanoma,~thickcat.0)
```


With adding to the data-frame directly 

```{r}
dcut(melanoma,breaks=c(0,200,500,800,2000)) <- gr.thick1~thick
dlevels(melanoma)
```

new variable is named thickcat.0 (after first cut-point), 
or to get quartiles with default names thick.cat.4 

```{r}
dcut(melanoma) <- ~ thick  # new variable is thickcat.4
dlevels(melanoma)
```

or median groups, here starting again with the original data, 

```{r}
data(melanoma)
dcut(melanoma,breaks=2) <- ~ thick  # new variable is thick.2
dlevels(melanoma)
```


to control new names

```{r}
data(melanoma)
mela= dcut(melanoma,thickcat4+dayscat4~thick+days,breaks=4)
dlevels(mela)
```

or

```{r}
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4+dayscat4~thick+days
dlevels(melanoma)
```

This can also be typed out more specifically

```{r}
melanoma$gthick = cut(melanoma$thick,breaks=c(0,200,500,800,2000))
melanoma$gthick = cut(melanoma$thick,breaks=quantile(melanoma$thick),include.lowest=TRUE)
```


Working with factors
====================

To see levels of covariates in data-frame

```{r}
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4~thick
dlevels(melanoma) 
```

To relevel the factor

```{r}
dtable(melanoma,~thickcat4)
melanoma = drelevel(melanoma,~thickcat4,ref="(194,356]")
dlevels(melanoma)
```


or to take the third level in the list of levels, same as above, 

```{r}
melanoma = drelevel(melanoma,~thickcat4,ref=2)
dlevels(melanoma)
```

To combine levels of a factor (first combining the first 3 groups into one):

```{r}
melanoma = drelevel(melanoma,~thickcat4,newlevels=1:3)
dlevels(melanoma)
```

or to combine groups 1 and 2 into one group and 3 and 4 into another

```{r}
dkeep(melanoma) <- ~thick+thickcat4
melanoma = drelevel(melanoma,gthick2~thickcat4,newlevels=list(1:2,3:4))
dlevels(melanoma)
```


Changing order of factor levels 

```{r}
dfactor(melanoma,levels=c(3,1,2,4)) <-  thickcat4.2~thickcat4
dlevel(melanoma,~ "thickcat4*")
dtable(melanoma,~thickcat4+thickcat4.2)
```

Combine levels but now control factor-level names 

```{r}
melanoma=drelevel(melanoma,gthick3~thickcat4,newlevels=list(group1.2=1:2,group3.4=3:4))
dlevels(melanoma)
```

Making a factor from existing numeric variable and vice versa
==============================================================

A numeric variable `status` with values 1, 2, 3 can be converted to a factor by:

```{r}
data(melanoma)
melanoma = dfactor(melanoma,~status, labels=c("malignant-melanoma","censoring","dead-other"))
melanoma = dfactor(melanoma,sexl~sex,labels=c("females","males"))
dtable(melanoma,~sexl+status.f)
```

A gender factor with values `"M"`, `"F"` can be converted to numerics by:

```{r}
melanoma = dnumeric(melanoma,~sexl)
dstr(melanoma,"sex*")
dtable(melanoma,~'sex*',level=2)
```

SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Recurrent events"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    # fig_width: 7.15
    # fig_height: 5.5 
header-includes: 
  - \usepackage{tikz}
  - \usepackage{adjustbox}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Recurrent events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview 
========

For recurrent events data it is often of interest to compute basic descriptive
quantities to gain some understanding of the phenomenon being studied.
We demonstrate how to compute:

 * the marginal mean 
   * AUC and logrank test for comparison of marginal means (also for special case with cumulative incidence)
   * efficient marginal mean estimation with fast computation of standard errors
 * the Ghosh-Lin Cox type regression for the marginal mean, possibly with composite outcomes. 
   * efficient regression augmentation of the Ghosh-Lin model
   * clusters can be specified
   * allows a stratified baseline 
 * the variance of a recurrent events process
 * the probability of exceeding k events
 * the two-stage recurrent events random effects model

In addition several tools can be used for simulating recurrent events and
bivariate recurrent events data, also with a possible terminating event: 

 * recurrent events with multiple 
   * event types 
   * Cox type rates 
 * with a terminal event with possibly multiple causes of death
   * Cox type rates 
 * frailty extensions
 * the Ghosh-Lin model when the survival rate is on Cox form.
   * frailty extensions
 * The general illness death model with  Cox models for all hazards. 

\begin{tikzpicture}[
    sharp corners=2pt,
    inner sep=4pt,
    node distance=0.9cm,
    >=latex,
    mynode/.style={draw,minimum height=0.9cm,minimum width=2.6cm}
]

\node[mynode] (healthy) {healthy $(h)$};
\node[mynode, right=of healthy] (e1) {1st event};
\node[mynode, right=of e1] (e2) {2nd event};

% Dots indicating continuation
\node[right=0.6cm of e2] (dots) {$\cdots$};

% Dead
\node[mynode, below=of e1] (dead) {dead $(d)$};

% Arrows along the event chain
\draw[->] (healthy) -- (e1);
\draw[->] (e1) -- (e2);
\draw[->] (e2) -- (dots);

% Arrows to 'dead'
\draw[->] (healthy) -- (dead);
\draw[->] (e1) -- (dead);
\draw[->] (e2) -- (dead);

\end{tikzpicture}


Simulation of recurrent events
==============================

We start by simulating some recurrent events data with two event types having cumulative hazards

 * $\Lambda_1(t)$ (rate among survivors)
 * $\Lambda_2(t)$ (rate among survivors)
 * $\Lambda_D(t)$

 where types 1 and 2 are considered and the terminal event rate is given by
 $\Lambda_D(t)$. The events are independent by default, but a random effects
 structure can also be specified to generate dependence.

 When simulating data, various random effects structures can be imposed to generate dependence:

* Dependence=0: The intensities are independent.

* Dependence=1: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $Z \lambda_D(t)$

* Dependence=4: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $\lambda_D(t)$

* Dependence=2: Normally distributed random effects $Z_1,Z_2,Z_d$ with variance (`var.z`) 
  and correlation (`cor.mat`) that can be specified. The intensities are
    + $\exp(Z_1) \lambda_1(t)$
    + $\exp(Z_2) \lambda_2(t)$
    + $\exp(Z_3) \lambda_D(t)$

* Dependence=3: Gamma-distributed random effects $Z_1,Z_2,Z_d$ where the sum structure can be specified via a matrix
   `cor.mat`. We compute $\tilde Z_j = \sum_k  Z_k^{cor.mat(j,k)}$ for $j=1,2,3$.
   The intensities are
    + $\tilde Z_1 \lambda_1(t)$
    + $\tilde Z_2 \lambda_2(t)$
    + $\tilde Z_3 \lambda_D(t)$

We return to how to run the different set-ups later and start by simulating independent processes. 

The key simulation functions are:

 * `sim_recurrent`: simple simulation with one event type and death
 * `sim_recurrentII`: extended version with possibly multiple types of recurrent events (rates can be zero);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent_list`: lists allowed for multiple event types and causes of death (competing risks);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent`: simulates from Cox-Cox (marginals) or Ghosh-Lin-Cox models

Data can also be simulated from the Ghosh-Lin model where marginal rates among survivors are on Cox form:

 * `simGLcox` (also `simRecurrentCox`): simulates from the Ghosh-Lin model
   * with frailties, where the survival model for the terminal event is on Cox form
   * simulates data where rates among survivors are on Cox form, with or without frailties

see examples below for specific models. 


Utility functions
=================

Two utility functions worth noting:

 * `tie.breaker`: breaks ties among jump times, which is expected by the functions below.
 * `count_history`: counts the number of previous jumps for each subject, i.e. $N_1(t-)$ and $N_2(t-)$.

Marginal Mean 
=============

We start by estimating the marginal mean $E(N_1(t \wedge D))$ where $D$ is the time of the terminal event.
The marginal mean is the average number of events observed before time $t$.

This is based on a two-rate model for

 * the type 1 events: $E(dN_1(t) | D > t)$
 * the terminal event: $E(dN_d(t) | D > t)$

and is defined as $\mu_1(t)=E(N_1(t))$ 
\begin{align}
   \int_0^t S(u) d R_1(u)	
\end{align}
where $S(t)=P(D \geq t)$ and $dR_1(t) = E(dN_1(t) | D > t)$ 

and can therefore be estimated by a 

 * Kaplan-Meier estimator, $\hat S(u)$ 
 * Nelson-Aalen estimator for $R_1(t)$

\begin{align}
  \hat R_1(t) & =   \sum_i \int_0^t  \frac{1}{Y_\bullet (s)}  dN_{1i}(s)
\end{align}
where $Y_{\bullet}(t)= \sum_i Y_i(t)$ such that the estimator is 
\begin{align}
  \hat \mu_1(t) & =    \int_0^t \hat S(u) d\hat R_1(u),
\end{align}
see Cook & Lawless (1997) and Ghosh & Lin (2000).

The variance can be estimated based on the asymptotic expansion
of $\hat \mu_1(t) - \mu_1(t)$
\begin{align*}
  & \sum_i \int_0^t \frac{S(s)}{\pi(s)} dM_{i1}  - \mu_1(t) \int_0^t  \frac{1}{\pi(s)} dM_i^d +  \int_0^t \frac{\mu_1(s) }{\pi(s)} dM_i^d,
\end{align*}

with mean-zero processes 

 * $M_i^d(t) = N_i^D(t)- \int_0^t Y_i(s) d \Lambda^D(s)$, 
 * $M_{i1}(t) = N_{i1}(t) - \int_0^t Y_{i}(s) dR_1(s)$. 

as described in Ghosh & Lin (2000).


Generating data
==================

We start by generating some data to illustrate the computation of the marginal mean 

```{r}
library(mets)
set.seed(1000) # to control output in simulations for p-values below.
```

```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- sim_recurrent(400,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))
```

The status variable keeps track of the recurrent events and their type, and death the timing of 
death.

To compute the marginal mean we simply estimate the two rates functions of the 
number of events of interest and death by using the phreg function 
(to start without covariates). Then the estimates are combined with standard 
error computation in the recurrent_marginal function

```{r}
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)
```

We can also extract the estimate in different time-points 

```{r}
summary(out,times=c(1000,2000))
```

The marginal mean can also be estimated in the stratified case:

```{r}
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrent_marginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

We can compare different marginal mean test (IPCW based) with a log-rank test  

```{r}
test_logrankRecurrent(out)

dd <- test_marginalMean(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
dd
summary(dd)
dd$RAUCl
```

The AUC suggest that strata 2 looses 3840 days less than strata 0. 


If we adjust for covariates for the two rates we can still do
predictions of marginal mean, what can be plotted is the baseline marginal mean, 
that is for the covariates equal to 0 for both models. Predictions for specific 
covariates can also be obtained with the recmarg (recurren marginal mean used 
solely for predictions without standard error computation). 

```{r}
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust_phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)
```

We here simulate multiple recurrent events processes with two causes of death causes  and exponential censoring with
rate $3/5000$, all processes are assumed independent (dependence=0)

```{r}
rr <- sim_recurrent_list(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))
```

Improving efficiency 
======================

To illustrate how the efficiency can be improved using heterogenity in the data, we
now simulate some data with strong heterogenity. The dynamic augmentation is 
a regression on the history  for each subject consisting of the specified terms 
terms:  Nt, Nt2 (Nt squared), expNt (exp(-Nt)), NtexpNt (Nt*exp(-Nt)) or by simply
specifying these directly.  This was developed in Cortese and Scheike (2022).

```{r}
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count_history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

times <- 500*(1:10)
###recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))
```

In the case where covariates might be important but we are still interested in the marginal mean 
we can also augment wrt these covariates 

```{r}
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)

times <- seq(500,5000,by=500)
recEFF1x <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))

out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)
```

Regression models for the marginal mean 
========================================

One can also do regression modelling , using the model
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
then Ghost-Lin suggested IPCW score equations that are implemented in the recreg function of mets. 

First we generate data that from a Ghosh-Lin model with regression coefficients 
$\beta=(-0.3,0.3)$ and the baseline given by base1, this is done under the assumption that the death 
rate given covariates is on Cox form with baseline dr: 

```{r}
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
summary(outs)$coef

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)
```

We note that for the extended censoring model we gain a little efficiency and that the estimates are close to the true values. 

Also possible to do IPCW regression at fixed time-point

```{r}
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
summary(outipcws)$coef
```

We can also do the Mao-Lin type composite outcome where we both count the recurrent events (cause 1) and deaths (cause 3)
for example 
\begin{align*}
E(N_1(t) + I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}





```{r}
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef
```

This can also be done with competing risks death 
\begin{align*}
E(w_1 N_1(t) + w_2 I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
and with weights  $w_1,w_2$ that follow the causes, here 1 and 3. 
We modify the data by changing some of the cause 3 deaths to cause 4

```{r}
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)
```

Predictions and standard errors can be computed via the iid decompositions of the baseline and the regression coefficients. We illustrate this
for the standard Ghosh-Lin model 

```{r}
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])
```

The Ghosh-Lin model can be made more efficient by the regression augmentation method.
First computing the augmentation and then in a second step the augmented estimator (Cortese and Scheike (2023)): 

```{r}
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef
```

We note that the simple augmentation improves the standard errors as expected. The data was generated assuming independence with
previous number of events so it would suffice to augment only with the covariates. 



Administrative censoring for the Ghosh-Lin model
===============================================

In the case of administative censoring with possible additional random censorering we can fit the Ghosh-Lin model using 
risk-set adjustment for the administrative censorering and IPCW adjustment for the  random cenouring. We illustrate it using 
simulated data from the two-stage model described below. 
The advantage of this procedure is that we do not rely on any modelling assumptions for the administrative censoring that will
often be the important part of the total censoring. 


```{r}
library(mets)
rho1 <- 1; rho2 <- 0.5
rate <- c(1,1)
tt <- seq(0, 6, by = 0.01)
base1 <- cbind(tt,rho1 * (1 - exp(-tt/rate[1])))
drcumhaz <- cbind(tt,rho2 * (1 - exp(-tt/rate[2])))
base13 <- cpred(base1,c(1,3))[,2]

dats <- mets:::sim_GLRA(100,base1,drcumhaz,varz=1)
datsA <- dats[[1]]
dtable(datsA,~statusD)
datsRA <- dats[[2]]
dtable(datsRA,~statusD)

```

We get two data-sets, one with only administrative censoring datsA 
and one with additional random censoring datsRA. 
 
  - datsA: statusD has events (1) death (3) and administrative censorings at the  time censorA
  - datsRA: statusD has events (1) death (3),  administrative censorings (7) and random censorings (0). The time 
     of the administriative censoring is given by censorA

First we deal only  with the administriative censoring. 

```{r}
   ## handling admin censoring with IPCW 
    outRR  <- recreg(Event(start,stop,statusD)~Z1+Z2+cluster(id), datsA, cause = 1, 
		       cens.code = c(7),death.code=3)
    estimate(outRR)

    ## Full Adm-censoring statusA, timeA
    out0  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsA, cause = 1, 
		       cens.code = 9, death.code=3, adm.cens.time=datsA$censorA)
    estimate(out0)
```

The additional random censoring can be handled by combining the two censoring times, or by handling the 
administrative censoring (via risk-set adjustment) and random censoring (via IPCW) separately

```{r}
    ## std Right censoring  on combined censoring time 
    outR  <- recreg(Event(start,stop,statusD)~Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = c(0,7),death.code=3)
    estimate(outR)

    ## Combined R-IPCW + Adm-censoring status, time
    out1  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA)
    estimate(out1)

    ## Combined R-IPCW + Adm-censoring status + censoring modelling time
    out1c  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		     cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA,cens.model=~strata(Z1,Z2))
     estimate(out1c)
```

We have influence functions of all parameters and can thus also make predictions with confindence intervals

 - this is based on going through a grid of time-points if se=1
   - when se=0 the predictions are done for all jump-times 

```{r}
predR0 <- predict(outR,data.frame(Z1=0:1,Z2=0))
plot(predR0,ylim=c(0,1.5))

predR <- predict(outR,data.frame(Z1=0:1,Z2=0),se=1)
plot(predR,se=1,add=TRUE)

pred1c <- predict(out1c,data.frame(Z1=0:1,Z2=0),se=1)
plot(pred1c,se=1,add=TRUE)
```

The censoring weights for the random censoring and the combined censoring

```{r}
km07 <- km(Event(start,stop,statusD %in% c(0,7))~strata(Z1,Z2),datsRA)
km0 <- km(Event(start,stop,statusD==0)~strata(Z1,Z2),datsRA)
plot(km07,col=1)
plot(km0,add=TRUE,col=2,lwd=2)
```


Two-stage modelling 
====================

We simulate data with a terminal event on Cox form and recurrent events
satisfying the Ghosh-Lin model or having a rate on Cox form.

 - type=3 is Ghosh-Lin model for recurrent events and Cox for terminal event.
 - type=2 is Cox model for recurrent events among survivors and Cox for terminal event.
 - simulations based on time-grid to make linear approximations of cumulative hazards

Now we fit the two-stage model (the recreg must be called with twostage=TRUE)

```{r}
set.seed(100)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,cens=1/5000,type=3) 
dtable(rr,~statusD)
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
glout$gradient
```

Standard errors are computed assuming that the parameters of `out` and `outs` are
both known, and are therefore probably a little too small. A bootstrap could be used to
obtain more reliable standard errors.


Simulations with specific structure
===================================

The function `simGLcox` can simulate data where the recurrent process has a mean on Ghosh-Lin form. The key identity is that
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta) = \int_0^t S(t|X,Z) dR(t|X,Z)
\end{align*}
where $Z$ is a possible frailty, so that
\begin{align*}
 R(t|X,Z) & = \frac{Z \Lambda_0(t)  \exp(X^T \beta) }{S(t|X,Z)}
\end{align*}
leads to a Ghosh-Lin model. The survival model can be specified to have Cox form among survivors via
`model="twostage"`; otherwise `model="frailty"` uses a survival model with rate $Z \lambda_d(t) r_d$.
The frailty $Z$ is gamma distributed with a variance that can be specified. Simulations are based on
a piecewise linear approximation of the hazard functions for $S(t|X,Z)$ and $R(t|X,Z)$.


```{r}
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
```

We can also simulate from models where the terminal event is on Cox form and the rate among survivors is on Cox form. 

 * $E(dN_1 | D>t, X) = \lambda_1(t) r_1$
 * $E(dN_d | D>t, X) = \lambda_d(t) r_d$

underlying these models we have a shared frailty model

```{r}
rr <- mets:::sim_GLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
estimate(recurrent)
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)
```

We can also simulate data with underlying dependence from the two-stage model (`simGLcox`) or using
`simRecurrent` random effects models, for Cox-Cox or Ghosh-Lin-Cox models.

Here with marginals on Cox-Cox and Ghosh-Lin-Cox form, drawing covariates from data:

```{r}
simcoxcox <- sim_recurrent_ts(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim_recurrent_ts(recurrentGL,margsurv,n=10,data=rr)
```


Other marginal properties
=========================

The mean is a useful summary measure, but it is also easy and informative to examine other
simple summary measures such as the probability of exceeding $k$ events,

 * $P(N_1^*(t) \ge k)$
   * cumulative incidence of $T_{k} = \inf \{ t: N_1^*(t)=k \}$ with competing event $D$.

This is equivalent to a cumulative incidence of $T_k$ occurring before $D$, denoted $\hat F_k(t)$.

We note also that $N_1^*(t)^2$ can be written as
\begin{align*}
   \sum_{k=0}^K  \int_0^t I(D > s) I(N_1^*(s-)=k) f(k) dN_1^*(s)
\end{align*}
with $f(k)=(k+1)^2 - k^2$, such that its mean can be written as 
\begin{align*}
	\sum_{k=0}^K \int_0^t S(s) f(k) P(N_1^*(s-)= k  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k, D> s) 
\end{align*}
and estimated by
\begin{align*}
\tilde \mu_{1,2}(t) & = 
	\sum_{k=0}^K \int_0^t \hat S(s) f(k) 
	\frac{Y_{1\bullet}^k(s)}{Y_\bullet (s)} \frac{1}{Y_{1\bullet}^k(s)} d N_{1\bullet}^k(s)= \sum_{i=1}^n \int_0^t \hat S(s) f(N_{i1}(s-)) \frac{1}{Y_\bullet (s)} d N_{i1}(s),
\end{align*}
That is very similar to the "product-limit" estimator for $E( (N_1^*(t))^2 )$ 
\begin{align}
  \hat \mu_{1,2}(t) & =    \sum_{k=0}^K k^2 ( \hat F_{k}(t) - \hat F_{k+1}(t) ).
\end{align}

We use the estimator of the probability of exceeding $k$ events based on the fact that
$I(N_1^*(t) \geq k)$ is  equivalent to 
\begin{align*}
	\int_0^t I(D > s) I(N_1^*(s-)=k-1) dN_1^*(s),
\end{align*}
suggesting that its mean can be computed as
\begin{align*}
\int_0^t S(s) P(N_1^*(s-)= k-1  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k-1, D> s) 
\end{align*}
and estimated by 
\begin{align*}
\tilde F_k(t) = \int_0^t \hat S(s)  \frac{Y_{1\bullet}^{k-1}(s)}{Y_\bullet (s)} 
          	\frac{1}{Y_{1\bullet}^{k-1}(s)} d N_{1\bullet}^{k-1}(s).
\end{align*}

To compute these estimators we use the prob.exceed.recurrent function

```{r}
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count_history(rr)
dtable(rr,~statusD)

oo <- prob_exceed_recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)
```

We can also look at the mean and variance based on the estimators just described 

```{r}
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))
```


Multiple events
================

We now generate recurrent events data with two event types, starting with
independent events.

```{r}
rr <- sim_recurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count_history(rr)
dtable(rr,~death+status)
```

Based on this we can estimate also the joint distribution function, that is
the probability that $(N_1(t) \geq k_1, N_2(t) \geq k_2)$

```{r}
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)
```

Looking at simulations with dependence
=============================================

Using normally distributed random effects we illustrate four settings. We set variance $0.5$ for all
random effects and vary the correlation. We denote the correlation between the random effect associated with
$N_1$ and $N_2$ as $\rho_{12}$, and the correlation between the random effects
associated with $N_j$ and $D$ (the terminal event) as $\rho_{j3}$, organised in the vector $\rho=(\rho_{12},\rho_{13},\rho_{23})$.

 * Scenario I: $\rho=(0,0.0,0.0)$ — independence among all effects.

```{r, eval=FALSE}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

  par(mfrow=c(1,3))
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  cor(attr(rr,"z"))
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  plot(coo,main ="Scenario I")
```
 * Scenario II: $\rho=(0,0.5,0.5)$ — independence among survivors but dependence on terminal event.

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main ="Scenario II")
```

 * Scenario III: $\rho=(0.5,0.5,0.5)$ — positive dependence among survivors and dependence on terminal event.

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # positive dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario III")
```

 * Scenario IV: $\rho=(-0.4,0.5,0.5)$ — negative dependence among survivors and positive dependence on terminal event.

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # negative dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario IV")
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Binomial Regression for Survival and Competing Risks Data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5 
vignette: >
  %\VignetteIndexEntry{Binomial Regression for Survival and Competing Risks Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Binomial Regression for censored data 
=====================================

The `binreg` function fits a logistic link model with IPCW adjustment for a specific time-point,
and can thus be used for both survival and competing risks data.
Computation is linear in data size, and influence functions are computed and available
for downstream calculations.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Details
=======

The binreg function does direct binomial regression for one time-point, $t$, fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta)  = F_1(t,X,\beta)
\end{align*}
based on an IPCW adjusted estimating equation (EE) with response  $Y(t)=I(T \leq t, \epsilon=1 )$
\begin{align*}
 U(\beta,\hat G_c) = &  X ( Y(t) \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)} - \mbox{expit}( X^T \beta)) = 0,
\end{align*}
with $G_c(t)=P(C>t)$, the  censoring survival distribution, and with $\Delta(t) = I( C_i > T_i \wedge t)$
the indicator of being uncensored at time $t$ (type="I"). 

The default type="II" is to augment with a censoring term, that is solve 
\begin{align*}
  &  U(\beta,\hat G_c) + \int_0^t X \frac{\hat E(Y(t)| T>u)}{\hat G_c(u)} d\hat M_c(u) =0 
\end{align*}
where $M_c(u)$ is the censoring martingale, this typically improves the performance. This is equivalent to the 
pseudo-value approach (see Overgaard (2025)).

The influence function for the type="II" estimator is 
\begin{align*}
    U(\beta,G_c) + \int_0^t X \frac{E(Y| T>u)}{G_c(u)} d M_c(u)  
    - \int_0^t  \frac{E(X| T>u) E(Y| T>u)}{G_c(u)} d M_c(u) - \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u)  
\end{align*}
and for type="I"
\begin{align*}
  &  U(\beta) + \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u).
\end{align*}
The means $E(X Y(t) | T>u)$ and $E(Y(t)| T>u)$ are estimated by IPCW estimators among survivors to get 
estimates of the influence functions.

The function logitIPCW instead considers 
\begin{align*}
 U^{glm}(\beta,\hat G_c) = & \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)}  X  ( Y(t) - \mbox{expit}( X^T \beta)) = 0.
\end{align*}
This score equation is quite similar to those of the binreg, and exactly the same when the censoring model is fully-nonparametric.

The logitIPCW has influence function 
\begin{align*}
  &  U^{glm}(\beta,G_c) + \int_0^t \frac{E( X ( Y - F_1(t,\beta)) | T>u)}{G_c(u)} d M_c(u)  
\end{align*}

Which estimator performs the best depends on the censoring distribution and it seems that the binreg with type="II" performs 
overall quite nicely (see Blanche et al (2023) and Overgaard (2025)).  For the full estimated censoring model all estimators have the
same influence function (see Blanche et al (2023)).

Additional functions logitATE, and binregATE computes the average treatment effect based on propensity and outcome modelling. 
We demonstrate this in another vignette. 

The functions logitATE/binregATE can be used when there is no censoring and we thus have simple binary outcome. 

The variance is based on a sandwich formula with IPCW adjustment (using the influence functions),
and `naive.var` is the variance under a known censoring model. The influence functions are stored in the output.
Clusters can be specified to obtain cluster-corrected standard errors.

Examples
========


```{r}
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in random noise just below.
 data(bmt)
 bmt$time <- bmt$time+runif(nrow(bmt))*0.01
 bmt$id <- 1:408

 # logistic regression with IPCW binomial regression 
 out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
 summary(out)
```

We can also compute predictions using the estimates 
```{r}
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
```

Further the censoring model can depend on strata 

```{r}
 outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
 summary(outs)
```

Absolute risk differences and ratio
===================================

Now for illustrations we consider the absolute risk difference depending on tcell 

```{r}
 outs <- binreg(Event(time,cause)~tcell,bmt,time=50,cens.model=~strata(tcell))
 summary(outs)
```

the risk difference is 

```{r}
ps <-  predict(outs,data.frame(tcell=c(0,1)),se=TRUE)
ps
sum( c(1,-1) * ps[,1])
```

Getting the standard errors is straightforward since the two groups are
independent. If we had additionally adjusted for other covariates, however, we would need
to apply the delta theorem using the relevant covariances, along the lines of:

```{r}
dd <- data.frame(tcell=c(0,1))
p <- predict(outs,dd)

riskdifratio <- function(p,contrast=c(1,-1)) {
   outs$coef <- p
   p <- predict(outs,dd)[,1]
   pd <- sum(contrast*p)
   r1 <- p[1]/p[2]
   r2 <- p[2]/p[1]
   return(c(pd,r1,r2))
}
     
estimate(outs,f=riskdifratio,dd,null=c(0,1,1))
```

same as 

```{r}
run <- 0
if (run==1) {
library(prodlim)
pl <- prodlim(Hist(time,cause)~tcell,bmt)
spl <- summary(pl,times=50,asMatrix=TRUE)
spl
}
```



SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Analysis of multivariate survival data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5 
vignette: >
  %\VignetteIndexEntry{Analysis of multivariate survival data} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
library(mets)
```

Overview 
==========

When analysing multivariate survival data with the aim of learning about the
dependence present, possibly after adjusting for covariates,
several approaches are available in the mets package:

   *  Binary models and adjust for censoring with inverse probabilty of  censoring weighting
      - biprobit  model

   *  Bivariate survival models of Clayton-Oakes type
      + With regression structure on dependence parameter 
      + With additive gamma distributed random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Plackett OR model
      + With regression structure on OR dependence parameter 

   * Cluster stratified Cox 


Typically it can be hard or impossible to specify random effects models with special 
structure among the parameters of the random effects. This is possible for 
our specification of the random effects models.

To be concrete about the model structure assume that we have paired survival 
data $(T_1, \delta_1, T_2, \delta_2,  X_1, X_2)$ where the censored 
survival responses are $(T_1, \delta_1, T_2, \delta_2)$ and the 
covariates are  $(X_1, X_2)$.

The basic model assumes that each subject has a marginal on Cox form
$$ 
\lambda_{s(k,i)}(t) \exp( X_{ki}^T \beta)
$$ 
where $s(k,i)$ is a strata variable. 


The constructed likelihood is a composite likelihood based on 

  * all pairs within each cluster (when the pairs argument is not used) 

  * the specified pairs when the pairs argument is used. 


In addition to the clusters specified for constructing the composite likelihood, these
can be further summed using the `se.clusters` argument, which sums the influence functions over
the `se.clusters`. When `se.clusters` is not specified it defaults to the `cluster` argument.

When the clusters of the dependence parameters are the same as those of the marginal model and the
`phreg` function is used, the standard errors are corrected for the uncertainty from the marginal models; otherwise
the returned standard errors are computed as if the marginals are known.




Gamma distributed frailties 
==========================

The focus of this vignette is to describe how to work with bivariate survival data using
additive gamma random effects models. We present two different ways of specifying
different dependence structures.

* Univariate models with a single random effect for each cluster and with 
  a regression design on the variance. 

* Multivariate models with multiple random effects for each cluster. 

The univariate models are given a cluster random effect $Z_k$ with
parameter $\theta$ the joint survival function is given by the Clayton copula 
and on the form
$$ 
  \psi(\theta, \psi^{-1}(\theta,S_1(t,X_{k1}) ) + \psi^{-1}(\theta, S_1(t,X_{k1}) ) 
$$
where $\psi$ is the Laplace transform of a gamma distributed random
variable with mean 1 and variance $\theta$.

We then model the variance within clusters by a cluster specific 
regression design  such that 
$$ 
  \theta = h(z_j^T \alpha)
$$
where $z$ is the regression design (specified by theta.des  in the software), 
and $h$ is link function, that is either $exp$ or the identity. 

This model  can be fitted using a pairwise likelihood or the pseudo-likelihood
using either

 * twostage

 * twostageMLE

 To make the twostage approach possible we need a model with specific structure for the
 marginals.  Therefore given the 
random effect of the clusters the survival distributions within a cluster 
are independent and on the form 
$$ 
 P(T_j > t| X_j,Z) = exp( -Z \cdot \Psi^{-1}(\nu^{-1},S(t|X_j)) )  
$$
with $\Psi$ the laplace of the gamma distribution with mean 1 and variance $1/\nu$.


Additive Gamma frailties 
========================

For the multivariate models we are given a multivariate random effect for each cluster,
$Z=(Z_1,...,Z_d)$ with $d$ random effects.
The total random effect for each subject $j$ in a cluster is specified using a
regression design on these random effects, with a regression vector
$V_j$ such that the total random effect is
  $V_j^T (Z_1,...,Z_d)$.  The elements of $V_j$ are 1/0.
The
random effects $(Z_1,...,Z_d)$ have associated parameters $(\lambda_1,...,\lambda_d)$
and  $Z_j$ is Gamma distributed with

  * mean $\lambda_j/V_1^T \lambda$

  * variance $\lambda_j/(V_1^T \lambda)^2$
	       
The key assumption to make the two-stage fitting possible is that
\begin{align*}
   \nu =V_j^T \lambda
\end{align*}
is constant within clusters.  The consequence of this is that 
the total random effect for each subject within a cluster, 
$V_j^T (Z_1,...,Z_d)$, is gamma distributed with variance $1/\nu$. 

The DEFAULT parametrization (var.par=1) uses the variances of the random effects 
\begin{align*}
 \theta_j  = \lambda_j/\nu^2
\end{align*}
 For alternative parametrizations one can specify that the parameters are  $\theta_j=\lambda_j$ with the argument var.par=0.

 Finally the parameters $(\theta_1,...,\theta_d)$ are related to the parameters 
 of the model by a regression construction $M$ (d x k), that links the $d$
 $\theta$  parameters with the $k$ underlying $\alpha$ parameters 
\begin{align*}
 \theta & = M  \alpha. 
\end{align*}
 The default is a diagonal matrix for $M$. 
 This can be used to make structural assumptions about the variances of the random-effects 
 as is needed for the ACE model for example. In the software $ M $ is called theta.des


Assume that the marginal survival distribution for subject $i$ within cluster $k$ is given
by $S_{X_{k,i}}(t)$ given covariates $X_{k,i}$. 

Now given the random effects of the cluster $Z_k$ and the covariates $X_{k,i}$ $i=1,\dots,n_k$ 
we assume that subjects within the cluster are independent with survival distributions 
\begin{align*}
  \exp(-  ( V_{k,i} Z_k)  \Psi^{-1} (\nu,S_{X_{k,i}}(t)) ).
\end{align*}

A consequence of this is that the hazards given the covariates $X_{k,i}$ and the random effects $Z_k$
are given by
\begin{align}
  \lambda_{k,i}(t;X_{k,i},Z_{k,i}) = ( V_{k,i} V_k) D_3 \Psi^{-1} (\nu,S_{X_{k,i}}(t))  D_t S_{X_{k,i}}(t) 
  \label{eq-cond-haz}
\end{align}
where $D_t$ and $D_3$ denotes the partial derivatives with respect to $t$ and the third argument, respectively. 

Further, we can express the multivariate survival distribution as 
\begin{align}
  S(t_1,\dots,t_m) & =  \exp( -\sum_{i=1}^m (V_i Z) \Psi^{-1}(\eta_l,\nu_l,S_{X_{k,i}}(t_i)) )  \nonumber \\
  & =  \prod_{l=1}^p  \Psi(\eta_l,\eta , \sum_{i=1}^m Q_{k,i} \Psi^{-1}(\eta,\eta,S_{X_{k,i}}(t_i))).
  \label{eq-multivariate-surv}
\end{align}
In the case of considering just pairs, we write this function as $C(S_{k,i}(t),S_{k,j}(t))$.

In addition to survival times from this model, we assume that we independent right censoring present 
$U_{k,i}$ such that the given $V_k$ and the covariates $X_{k,i}$ $i=1,\dots,n_k$ $(U_{k,1},\dots,U_{k,n_k})$
of $(T_{k,1},\dots,T_{k,n_k})$, and the conditional censoring distribution do not depend on $V_k$.

One consequence of the model structure is that Kendall's tau can be computed for
two-subjects $(i,j)$ across two clusters ``1'' and ``2'' as
\begin{align}
E( \frac{( V_{1i} Z_1-  V_{1j}Z_2)( V_{2i}Z_1 -  V_{2j}Z_2 )}{( V_{1i}Z_1 + V_{2i}Z_2 ) ( V_{1j}Z_1 + V_{2j}Z_2 )} ) 
\end{align}
under the assumption that we compare pairs with equivalent marginals, 
$S_{X_{1,i}}(t)= S_{X_{2,i}}(t)$ and $S_{X_{1,j}}(t)= S_{X_{2,j}}(t)$, 
and that $S_{X_{1,i}}(\infty)= S_{X_{1,j}}(\infty)=0$. 
Here we also use that $\eta$ is the same across clusters.
The Kendall's tau would be the same for \eqref{frailty-model} due to the same additive structure for the 
frailty terms, and the random effects thus have the same interpretation in terms of Kendall's tau.  


Univariate gamma (clayton-oakes) model twostage models 
=======================================================

We start by fitting simple Clayton-Oakes models for the data, that is
with an overall random effect that is Gamma distributed with variance
$\theta$. We can fit the model by a pseudo-MLE (twostageMLE)
and a pairwise composite likelihood approach (twostage). 

The pseudo-likelihood and the composite pairwise likelihood should give 
the same for this model since we have paired data. 
In addition the log-parametrization is illustrated with the 
var.link=1 option. 
In addition it is specified that we want a "clayton.oakes" model. 
We note that the standard errors differs because the twostage does not
include the variance due to the baseline parameters for this type of
modelling, so here it is better to use the twostageMLE. 


```{r}
 library(mets)
 data(diabetes)
 set.seed(100)
 
 # Marginal Cox model  with treat as covariate
 margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
 summary(fitco1)
 
 # Clayton-Oakes
 fitco2 <- survival_twostage(margph,data=diabetes,theta=0.0,
                  clusters=diabetes$id,var.link=1,model="clayton.oakes")
 summary(fitco2)
 fitco3 <- survival_twostage(margph,data=diabetes,theta=1.0,
                  clusters=diabetes$id,var.link=0,model="clayton.oakes")
 summary(fitco3)
```

Note, the standard errors are slightly different when comparing 
fitco1 with fitco3 since the survival_twostage uses numerical 
derivatives for the hessian and the derivative in the 
direction of the marginal model. 

The marginal models can be either structured Cox model or as here 
with a baseline for each strata. This gives quite similar results to those
before. 

```{r}
  # without covariates but marginal model stratified 
  marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
 fitco<-twostageMLE(marg,data=diabetes,theta=1.0)
 summary(fitco)

  fitcoa <- survival_twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
		   model="clayton.oakes",var.link=0)
  summary(fitcoa)
```

Piecewise constant Clayton-Oakes model 
=========================================

Let the cross-hazard ratio (CHR) be defined as 
\begin{align}
  \eta(t_1,t_2) =  \frac{ \lambda_1(t_1| T_2=t_2)}{ \lambda_1(t_1| T_2 \ge t_2)}
  =  \frac{ \lambda_2(t_2| T_1=t_1)}{ \lambda_2(t_2| T_1 \ge t_1)}
\end{align}
where $\lambda_1$ and $\lambda_2$ are the conditional hazard functions
of $T_1$ and $T_2$ given covariates.  For the Clayton-Oakes model this
ratio is $\eta(t_1,t_2) = 1+\theta$, and as a consequence we see that
if the co-twin is dead at any time we would increase our risk
assessment on the hazard scale with the constant $\eta(t_1,t_2)$. The
Clayton-Oakes model also has the nice property that Kendall's tau is
linked directly to the dependence parameter $\theta$ and is $1/(1+2/\theta)$.

A very useful extension of the constant
cross-hazard ratio (CHR) model is the
piecewise constant cross-hazard ratio (PCHR) model for bivariate survival
data (Nan et al., 2006), extended to competing
risks by Shih and Lu (2010).

In the survival setting we let the CHR
\begin{align}
  \eta(t_1,t_2) & = \sum \eta_{i,j} I(t_1 \in I_i, t_2 \in I_j)
\end{align}

The model allows the CHR to be constant in different regions of the plane, specified here by
the cut-points $c(0,0.5,2)$ thus defining 9 regions. This can also be thought of as fitting a separate
Clayton-Oakes model for each region.

This provides a constructive goodness-of-fit test for whether the Clayton-Oakes model
is valid: if valid, the parameter should be the same across all regions.

First we generate some data from the Clayton-Oakes model with variance $0.5$ and 2000 pairs.
And fit the related model. 

```{r}
 d <- sim_ClaytonOakes(200,2,0.5,0,3)
  margph <- phreg(Surv(time,status)~x+cluster(cluster),data=d)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=d)
 summary(fitco1)
```

 Now we cut the region at the cut-points 
$c(0,0.5,2)$ thus defining 9 regions and fit a separate model for each region.  
We see that the parameter is indeed rather constant over the 9 regions. A formal 
test can be constructed. 

```{r}
 udp <- piecewise_twostage(c(0,0.5,2),data=d,id="cluster",timevar="time",status="status",model="clayton.oakes",silent=0)
 summary(udp)
```

Multivariate gamma twostage models 
===================================

To illustrate how the multivariate models can be used, we first set 
up some twin data with ACE structure. That is two shared random effects,
one being the genes $\sigma_g^2$ and one the environmental effect
$\sigma_e^2$. Monozygotic twins share all genes whereas the dizygotic 
twins only share half the genes. This can be expressed via 5 random effect for
each twin pair (for example). We start by setting this up. 


 The pardes matrix tells how the the parameters of the 5 random effects are 
 related, and the matrix her first has one random effect with parameter
 $\theta_1$ (here the $\sigma_g^2$ ), then the next 3 random effects have
 parameters $0.5 \theta_1$ (here $0.5 \sigma_g^2$ ), and the last 
random effect that is given by its own parameter $\theta_2$ (here $\sigma_e^2$ ).

```{r}
 data <- sim_ClaytonOakes_twin_ace(200,2,1,0,3)

 out <- twin_polygen_design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 pardes <- out$pardes
 pardes 
```

 The last part of the model structure is to decide how the random effects
 are shared for the different pairs (MZ and DZ), this is specified by
 the random effects design ($V_1$ and $V_2$) for each pair. 
 This is here specified by an overall designmatrix for each subject 
 (since they enter all pairs with the same random effects design).

 For an MZ pair the two share the full gene random effect and the full
 environmental random effect. In contrast the DZ pairs share the 
 2nd random effect with half the gene-variance and have both a non-shared
 gene-random effect with half the variance, and finally a fully shared 
 environmental random effect. 

```{r}
 des.rv <- out$des.rv
 # MZ
 head(des.rv,2)
 # DZ 
 tail(des.rv,2)
```

 Now we call the twostage function. We see that we essentially recover the true values, and
 note that the output also compares the sizes of the genetic and environmental random effect. 
 This number is sometimes called the heritability.  In addition the total variance for each
 subject is also computed and is here around $3$, as we indeed constructed.

```{r}
### data <- sim_ClaytonOakes_twin_ace(2000,2,1,0,3)
### out <- twin_polygen_design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data)
 ts <- twostage(aa,data=data,clusters=data$cluster,
      theta=c(2,1),var.link=0,random.design=out$des.rv,theta.des=out$pardes)
 summary(ts)
```

 - A nice feature of the procedure is that it scales linearly in the number of observations
    - 1 mill pairs had a running time of around 100 seconds. 

```{r}
run <- 0
if (run==1) {
 data <- sim_ClaytonOakes_twin_ace(1000000,2,1,0,3)

 out <- twin_polygen_design(data,id="cluster",zyg="DZ",zygname="zyg",type="ace")
 pardes <- out$pardes
 aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data)
 system.time(
 ts <- twostage(aa,data=data,clusters=data$cluster,
      theta=c(2,1),var.link=0,random.design=out$des.rv,theta.des=out$pardes)
 )
 summary(ts)
}
```

The estimates can be transformed into Kendall's tau estimates for MZ and DZ twins. The Kendall's tau
in the above output reflects how  a gamma distributed random effect in the normal Clayton-Oakes model
is related to the Kendall's tau. In this setting the Kendall's of MZ and DZ, however, should reflect 
both random effects. 

We do this based on simulations. The Kendall's tau of the MZ is around 0.60, and for 
DZ around 0.33. Both are quite high and this is due to a large shared environmental 
effect and large genetic effect. 

```{r}
kendall_ClaytonOakes_twin_ace(ts$theta[1],ts$theta[2],K=10000) 
```

Family data 
=============

For family data, things are quite similar since we use only the pairwise structure.
We show how the designs are specified.

First we simulate data from an ACE model: 200 families each with two parents who share
only the environment, and two children who share genes with their parents.

```{r}
library(mets)
set.seed(1000)
data <- sim_ClaytonOakes_family_ace(200,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)
```

To set up the random effects some functions can be used. We here set up the ACE model
that has 9 random effects with one shared environmental effect (the last random effect) and
4 genetic random effects for each parent, with variance $\sigma_g^2/4$. 

The random effect is again set-up with an overall designmatrix because it is again the same for
each subject for all comparisons across family members.  We below demonstrate how the model can 
be specified in various other ways. 

Each child share 2 genetic random effects with each parent, and also share 2 genetic random effects
with his/her sibling.

```{r}
out <- ace_family_design(data,member="type",id="cluster")
out$pardes
head(out$des.rv,4)
```

Then we fit the model 

```{r}
pa <- phreg(Surv(time,status)~+1+cluster(cluster),data=data)

# make ace random effects design
ts <- twostage(pa,data=data,clusters=data$cluster,var.par=1,var.link=0,theta=c(2,1),
        random.design=out$des.rv,theta.des=out$pardes)
summary(ts)
```

The model can also be fitted by specifying the pairs that 
one wants for the pairwise likelihood. This is done by
specifying the pairs argument.  We start by considering all pairs as we also did before. 

All pairs can be written up by calling the familycluster_index
function. 

There are <!-- TODO: insert pair count --> pairs to consider, and the first 6 pairs for the first family 
is written out here. 

```{r}
# now specify fitting via specific pairs 
# first all pairs 
mm <- familycluster_index(data$cluster)
head(mm$familypairindex,n=12)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
head(pairs,n=6)
```

Then fitting the model using only specified pairs

```{r}
ts <- twostage(pa,data=data,clusters=data$cluster, theta=c(2,1),var.link=0,step=1.0,
        random.design=out$des.rv, theta.des=out$pardes,pairs=pairs)
summary(ts)
```

Now we only use a random sample of the pairs by sampling these.  The pairs
picked still refers to the data given in the data argument, and clusters (families)
are also specified as before. 

```{r}
ssid <- sort(sample(1:nrow(pairs),200))
tsd <- twostage(pa,data=data,clusters=data$cluster,
    theta=c(2,1)/10,var.link=0,random.design=out$des.rv,
   theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)
```

Sometimes one only has data from the pairs in addition to, for example,
a cohort estimate of the marginal survival models. We now demonstrate how this
is handled. Everything is essentially as before, except we need to organise the
design differently compared to before, when we specified the design
for everybody in the cohort. In addition, we do not bring in the
uncertainty from the baseline in the estimates here; even though this is
formally possible, when the data for the marginal model and the twostage model
are not the same, we must specify that we do not want the
decomposition for uncertainty due to the baseline (`baseline.iid=0`).

```{r}
ids <- sort(unique(c(pairs[ssid,])))

pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)

# this requires that pair.new refers to id's in dataid (survival, status and so forth)
# random.design and theta.des are constructed to be the array 3 dims via individual specfication from ace.family.design
dataid <- dsort(data[ids,],"cluster")
outid <- ace_family_design(dataid,member="type",id="cluster")
outid$pardes
head(outid$des.rv)
```

Now fitting the model using only the pair data. 

```{r}
tsdid <- twostage(pa,data=dataid,clusters=dataid$cluster,theta=c(2,1)/10,var.link=0,baseline.iid=0,
          random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)

paid <- phreg(Surv(time,status)~+1+cluster(cluster),data=dataid)
tsdidb <- twostage(paid,data=dataid,clusters=dataid$cluster,theta=c(2,1)/10,
  var.link=0,random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdidb)
coef(tsdid)
```

Estimates changed because we used either the marginal from the full-data, in which case 
the standard errors did not reflect the uncertainty from the baseline, or the marginal estimated 
from only the sub-sample in which case the marginals were slightly different.  


Pairwise specification of random effects and variances 
--------------------------------------------------------

Now we illustrate how one can also directly specify the random.design and
theta.design for each pair, rather than taking an overall specification that
can be used for the whole family via the rows of the des.rv for the
relevant pairs.  This can be much simpler in some situations. 

```{r}
pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new)
head(pair.types)

theta.des  <- rbind( c(rbind(c(1,0),c(1,0),c(0,1),c(0,0))),
		c(rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))))
random.des <- rbind( 
        c(1,0,1,0),c(0,1,1,0),
        c(1,1,0,1),c(1,0,1,1))
mf <- 1*(pair.types[,1]=="mother" & pair.types[,2]=="father")
##          pair, rv related to pairs,  theta.des related to pair 
pairs.new <- cbind(pair.new,(mf==1)*1+(mf==0)*3,(mf==1)*2+(mf==0)*4,(mf==1)*1+(mf==0)*2,(mf==1)*3+(mf==0)*4)
```

pairs.new is a matrix with 

 * columns 1:2 giving the indices of the  data points

 * columns 3:4 giving the indices of the random.design for the different pairs

 * columns 5 giving the indices of the theta.des written as rows 

 * columns 6 giving the number of random variables for this pair


Looking at the first three rows. We see that the composite likehood is based on data-points (1,2),
(3,4) and (5,6), these are  (mother, father), (mother, child), and (father, child), respectively. 


```{r}
head(pairs.new[1:3,])
head(dataid)
```

The random effects for these are specified from random effects with design read from the random.design, using
the rows (1,2), (3,4) and (3,4), respectively, and with random effects that have variances given by 
theta.des rows, 1,2, and 2 respectively in the three cases. 
For the first pair (1,2), the random vectors and their variances are given by, 
(mother, father) pair, 

```{r}
random.des[1,]
random.des[2,]
matrix(theta.des[1,],4,2)
```
thus sharing only the third random effect with variance $\sigma_e^2$ and
having two non-shared random effects with variances $\sigma_g^2$, and finally
a last 4th random effect with variance $0$ that thus could have been omitted. 


The length of all rows of theta.des
are the maximum number of random effects $\times$ the number of parameters. 
These two numbers are given in the call. In this case 4 $\times$ 2. 
So theta.des has rows of length $8$, possibly including some 0's for rows
not relevant due to fewer random effects, as is the case here for pairs 
that do not share genetic effects.


Now considering the parent and their child, they are 
thus sharing the first random effect with variance $0.5 \sigma_g^2$ then there are 
two non-shared random effects with variances $0.5 \sigma_g^2$, and finally a shared 
environment with variance $\sigma_e^2$. 

```{r}
head(dataid)
matrix(theta.des[2,],4,2)
random.des[3,]
random.des[4,]
```

And fitting again the same model as before 

```{r}
tsdid2 <- twostage(pa,data=dataid,clusters=dataid$cluster,
       theta=c(2,1)/10,var.link=0,step=1.0,random.design=random.des,
       baseline.iid=0, theta.des=theta.des,pairs=pairs.new,dim.theta=2)
summary(tsdid2)
tsd$theta
tsdid2$theta
tsdid$theta
```

Finally the same model structure can be setup based on a Kinship coefficient. 

```{r}
kinship  <- rep(0.5,nrow(pair.types))
kinship[pair.types[,1]=="mother" & pair.types[,2]=="father"] <- 0
head(kinship,n=10)

out <- make_pairwise_design(pair.new,kinship,type="ace") 
```

Same output as before 

```{r}
tsdid3 <- twostage(pa,data=dataid,clusters=dataid$cluster,
   theta=c(2,1)/10,var.link=0,step=1.0,random.design=out$random.design,
   baseline.iid=0,theta.des=out$theta.des,pairs=out$new.pairs,dim.theta=2)
summary(tsdid3)
tsdid2$theta
tsdid$theta
```

Now fitting the AE model without the "C" component for shared environment: 

```{r}
outae <- make_pairwise_design(pair.new,kinship,type="ae") 
tsdid4 <- twostage(pa,data=dataid,clusters=dataid$cluster,
   theta=c(2,1)/10,var.link=0,random.design=outae$random.design,
   baseline.iid=0,theta.des=outae$theta.des,pairs=outae$new.pairs,dim.theta=1)
summary(tsdid4)
```


Pairwise dependence modelling 
------------------------------

We now illustrate how to estimate all pairwise associations between different
family-members using the twostage function. They key is specify the pairs for
the composite likelihood directly  and then the associated design-matrix
design therefore follows the pairs and is a matrix where its row is given as
the third column in the pairs argument. 

First we get the pairs to be considered


```{r}
library(mets)
set.seed(1000)
data <- sim_ClaytonOakes_family_ace(200,2,1,0,3)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

mm <- familycluster_index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)
```

Now we construct the design matrix related to the pairs 

```{r}
 dtypes <- interaction( data[pairs[,1],"type"], data[pairs[,2],"type"])
 dtypes <- droplevels(dtypes)
 table(dtypes)
 dm <- model.matrix(~-1+factor(dtypes))
 head(dm)
```

Then we fit the model: 

```{r}
pa <- phreg(Surv(time,status)~cluster(cluster),data)

tsp <- twostage(pa,data=data,theta.des=dm,pairs=cbind(pairs,1:nrow(dm)),se.clusters=data$clust)
summary(tsp)
```

We note that mother-father pairs have the smallest correlation since they share only the environmental random effect, whereas
other pairs have a similar correlation (in fact the same, due to how the data were simulated), consisting of
half the genetic effect plus the shared environmental effect.


Univariate plackett model twostage models 
============================================

The copula known as the Plackett distribution (Plackett, 1965; Anderson et al., 1992; Ghosh et al., 2006), is on the form
\begin{align}
  C(u,v; \theta) =
  \begin{cases}
    \frac{ S - (S^2 - 4 u v \theta (\theta-a))}{2 (\theta -1)} & \mbox{ if } \theta \ne 1 \\
    u v & \mbox{ if } \theta = 1
  \end{cases}
\end{align}
with $S=1+(\theta-1) (u + v)$.  With marginals $S_i$ we now define the
bivariate survival function as $C(u_1,u_2)=H(S_1(t_1),S_2(t_2))$ with
$u_i=S_i(t_i)$.

The dependence parameter $\theta$ has the nice interpretation that the
it is equivalent to the odds-ratio of all $2 \times 2$ tables for
surviving past any cut of the plane $(t_1,t_2)$, that is
$$
\theta = \frac{ P(T_1 > t_1 | T_2 >t_2) P(T_1 \leq t_1 | T_2>t_2) }{P(T_1 > t_1 | T_2 \leq t_2) P(T_1 \leq t_1 | T_2 \leq t_2 ) }.
$$

One additional nice feature of the odds-ratio measure it that it is
directly linked to the Spearman correlation, $\rho$, that can be
computed as
\begin{align}
  \frac{\theta+1}{\theta -1} - \frac{2 \theta}{(\theta-1)^2} \log(\theta)
\end{align}
when $\theta \ne 1$, if $\theta=1$ then $\rho=0$.


This model has one more free parameter than the Clayton-Oakes model.

```{r}
 library(mets)
 data(diabetes)
 
 # Marginal Cox model  with treat as covariate
 margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 # Clayton-Oakes, MLE 
 fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
 summary(fitco1)
 
 # Plackett model
 mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
 fitp <- survival_twostage(mph,data=diabetes,theta=3.0,
                clusters=diabetes$id,var.link=1,model="plackett")
 summary(fitp)
 
 # without covariates but with stratafied 
 marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
 fitpa <- survival_twostage(marg,data=diabetes,theta=1.0,
                 clusters=diabetes$id)
 summary(fitpa)
 
 fitcoa <- survival_twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
                  model="clayton.oakes")
 summary(fitcoa)
```
 
 With a regression design 

```{r}
 mm <- model.matrix(~-1+factor(adult),diabetes)
 fitp <- survival_twostage(mph,data=diabetes,theta=3.0,Nit=40,
                clusters=diabetes$id,var.link=1,model="plackett",
		theta.des=mm)
 summary(fitp)
```
 
```{r}
 # Piecewise constant cross hazards ratio modelling

 d <- subset(sim_ClaytonOakes(1000,2,0.5,0,stoptime=2,left=0),!truncated)
 udp <- piecewise_twostage(c(0,0.5,2),data=d,id="cluster",timevar="time",
                           status="status",model="plackett",silent=0)
 summary(udp)
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Cooking Survival Data: 5-Minute Recipes"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
    toc: true
header-includes: 
  - \usepackage{tikz}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Cooking Survival Data: 5-Minute Recipes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



# Overview 

Simulation of survival data is important for both
theoretical and practical work. In a practical setting we might wish to
validate that standard errors are valid even in a rather small sample,
or validate that a complicated procedure is doing as intended.
Therefore it is useful to have simple tools for generating survival data
that resembles a particular dataset as closely as possible. In a theoretical
setting we are often interested in evaluating the finite-sample
properties of a new procedure in different settings that are
motivated by a specific practical problem. The aim is to provide such tools.

Bender et al. discussed how to generate survival data based on the Cox model,
and considered only a subset of the available parametric survival models (Weibull, exponential).
We start by restricting attention to piecewise linear baseline functions, which make it easy to simulate
data that follows closely the baseline estimated from the data using semi- or
nonparametric models. This makes it straightforward to capture important aspects of the
data. We later return to extended Weibull baseline hazard models.

Different survival models can be cooked, and we here give recipes for
hazard and cumulative incidence based simulations. More recipes are
given in vignette about recurrent events. 

  - hazard based. 
    - Cox Regression  models.
       - piecewise constant baselines 
       - parametric Weibull models
  - cumulative incidence. 
    - Regression models with cloglog or logistic link. 
  - recurrent events (see recurrent events vignette).
    - Regression models with exp link, for rates or Ghosh-Lin type.



``` r
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations
```

# Hazard based, Cox models  


Given a survival time $T$ with cumulative hazard $\Lambda(t)=\int_0^t \lambda(s) ds$, it
follows that, with $E \sim Exp(1)$ (exponential with rate 1), $\Lambda^{-1}(E)$ will have the
same distribution as $T$.

This provides the basis for simulations of survival times with a given hazard and is
a consequence of this simple calculation
$$
  P(\Lambda^{-1}(E) > t) = P(E > \Lambda(t)) = \exp( - \Lambda(t)) = P(T > t).
$$

Similarly if $T$ given $X$ has hazard on Cox form
$$
  \lambda_0(t) \exp( X^T \beta)
$$
where $\beta$ is a $p$-dimensional regression coefficient and $\lambda_0(t)$ a baseline
hazard function,
then it is useful to observe also that
$\Lambda^{-1}(E/HR)$ with $HR=\exp(X^T \beta)$ has the same distribution as $T$ given $X$.

Therefore, if the inverse of the cumulative hazard can be computed, we can generate survival times with
a specified hazard function. One useful observation is that for a piecewise linear continuous
cumulative hazard on an interval $[0,\tau]$, $\Lambda_l(t)$, it is easy to compute the inverse.

Further, any cumulative hazard can be approximated by a piecewise linear continuous
cumulative hazard, enabling simulation from this approximation. Recall that fitting
the Cox model to data yields a piecewise constant cumulative hazard and regression coefficients;
with these in hand we can approximate the piecewise constant Breslow estimator
by simply connecting the jump points with straight lines.

The engine is to simulate data with a given linear cumulative hazard. First generating survival data based on
the cumulative hazard `cumhaz`:


``` r
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)
```

Now we fit a simple Cox model


``` r
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim_phreg(cox1,n,data=bmt)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1 
#> 529 471
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
#>                [,1]       [,2]
#> tcell    -0.6517920 -0.4564152
#> platelet -0.5207454 -0.5113844
#> age       0.4083098  0.3860139
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-4-1.png" alt="Cumulative hazard: simulated vs fitted Cox model."  />

``` r

 ## modify regression coefficients 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim_phreg(cox10,n,data=bmt)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1 
#> 427 573
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
#>          [,1]       [,2]
#> tcell     0.0 0.05615982
#> platelet  0.4 0.34930321
#> age       0.3 0.42496872
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-4-2.png" alt="Cumulative hazard: simulated vs fitted Cox model."  />

Multiple Cox models for cause-specific hazards can be combined. Below we draw the
covariates manually; alternatively, `sim_phregs` draws covariates directly from
the data.


``` r
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-5-1.png" alt="Cumulative hazard: competing risks Cox models, two causes."  />

``` r
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
#>                [,1]       [,2]       [,3]       [,4]
#> tcell    -0.4232606 -0.3727007  0.3991068  0.8167564
#> platelet -0.5654438 -0.5834273 -0.2461474 -0.3190683
```

We now consider a fully nonparametric model with stratified baselines.


``` r
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
#> 
#>     diabetes   0   1
#> chf                 
#> 0            223  16
#> 1            230  31
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim_phreg(coxs,nsim,data=NULL,strata=strata)

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim_phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)
```

<img src="figure/cooking-survival-data-unnamed-chunk-6-1.png" alt="Cumulative hazard: stratified nonparametric model."  />

We now fit cause-specific hazard models with three causes (treating censoring as one of them)
and generate competing risks data with hazards taken from the fitted Cox models.
The following example includes stratified baselines for some of the models:


``` r
 ## competing risks with phreg 
 cox0 <- phreg(Surv(time,cause==0)~tcell+platelet,data=bmt)
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
 coxs <- list(cox0,cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)

 ## verify cause-specific hazards match fitted model; increase n for better agreement
 scox0 <- phreg(Surv(time,cause==1)~tcell+platelet,data=dd)
 scox1 <- phreg(Surv(time,cause==2)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==3)~strata(tcell)+platelet,data=dd)
 cbind(cox0$coef,scox0$coef)
#>               [,1]      [,2]
#> tcell    0.1912407 0.3053846
#> platelet 0.1563789 0.2586927
 cbind(cox1$coef,scox1$coef)
#>                [,1]       [,2]
#> tcell    -0.4232606 -0.6611783
#> platelet -0.5654438 -0.4566101
 cbind(cox2$coef,scox2$coef)
#>                [,1]       [,2]
#> platelet -0.2271912 -0.1857664
 par(mfrow=c(1,3))
 plot(cox0); plot(scox0,add=TRUE,col=2); 
 plot(cox1); plot(scox1,add=TRUE,col=2); 
 plot(cox2); plot(scox2,add=TRUE,col=2); 
```

<img src="figure/cooking-survival-data-stratified-1.png" alt="Cumulative hazard: stratified competing risks models."  />

``` r
 
 ########################################
 ## second example 
 ########################################

 cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
 coxs <- list(cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)
 cbind(cox1$coef,scox1$coef)
#>                [,1]       [,2]
#> platelet -0.5658612 -0.6949669
 cbind(cox2$coef,scox2$coef)
#>            [,1]      [,2]
#> tcell 0.4153706 0.2554993
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE); 
 plot(cox2); plot(scox2,add=TRUE); 
```

<img src="figure/cooking-survival-data-stratified-2.png" alt="Cumulative hazard: stratified competing risks models."  />

One further example, again fully nonparametric:


``` r
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim_phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~cause)
#> 
#> cause
#>   0   1   2   3 
#>   1 397 241 361
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,cause==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
#>           [,1]        [,2]
#> bmi -0.1353399 -0.04444255
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)
```

<img src="figure/cooking-survival-data-unnamed-chunk-7-1.png" alt="Cumulative hazard: fully nonparametric competing risks."  />

**Summary of simulation functions**

| Function      | Purpose                                                     |
|---------------|-------------------------------------------------------------|
| `sim_phreg`   | Single `phreg` model; supports stratified baselines         |
| `sim_phregs`  | List of cause-specific `phreg` models; draws covariates automatically |


# Delayed entry

If $T$ given $X$ have hazard  on Cox form 
$$
  \lambda_0(t) \exp( X^T \beta)
$$
and we wish to generate data according to this hazard for those that are alive at time $s$, that is 
draw from the distribution of $T$ given $T>s$ (all given $X$ ), then we note that  
$$
\Lambda_0^{-1}( \Lambda_0(s) + E/HR)) 
$$
with $HR=\exp(X^T \beta))$ and with $E \sim Exp(1)$ has the distribution we are after. 

This is again a consequence of a simple calculation
$$
P_X\!\left(\Lambda^{-1}\!\left(\Lambda(s) + E/HR\right) > t\right)
= P_X\!\left(E > HR\!\left(\Lambda(t) - \Lambda(s)\right)\right)
= P_X(T > t \mid T > s).
$$

We here illustrate how to simulate from a competing risks model based on 
Cox hazards. 


``` r
data(bmt)
cox0 <- phreg(Surv(time, cause == 0) ~ tcell + platelet+age,          data = bmt)
cox1 <- phreg(Surv(time, cause == 1) ~ tcell + platelet+age,          data = bmt)
cox2 <- phreg(Surv(time, cause == 2) ~ strata(tcell) + platelet+age,  data = bmt)

nsim <- 800
entry <- rbinom(nsim, 1, 0.5) * runif(nsim) * 60

dd <- sim_phreg(cox0,nsim, data = bmt,entry=entry)
scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
cbind(cox0$coef, scox0$coef)
#>                [,1]       [,2]
#> tcell    0.09091888 0.11554247
#> platelet 0.15733517 0.09307867
#> age      0.10230287 0.11156111
par(mfrow = c(2, 2))
plot(cox0); plot(scox0, add = TRUE, col = 2)

dd <- sim_phregs(list(cox0, cox1, cox2), nsim, data = bmt,entry=entry)

scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
scox1 <- phreg(Surv(entry,time, cause == 2) ~ tcell + platelet+age,         data = dd)
scox2 <- phreg(Surv(entry,time, cause == 3) ~ strata(tcell) + platelet+age, data = dd)

cbind(cox0$coef, scox0$coef)
#>                [,1]         [,2]
#> tcell    0.09091888  0.006447759
#> platelet 0.15733517  0.074976667
#> age      0.10230287 -0.042215864
cbind(cox1$coef, scox1$coef)
#>                [,1]       [,2]
#> tcell    -0.6517920 -0.6996524
#> platelet -0.5207454 -0.3419084
#> age       0.4083098  0.5513612
cbind(cox2$coef, scox2$coef)
#>                [,1]        [,2]
#> platelet -0.2178543 -0.20179035
#> age       0.1306184  0.02950711

plot(cox0); plot(scox0, add = TRUE, col = 2)
plot(cox1); plot(scox1, add = TRUE, col = 2)
plot(cox2); plot(scox2, add = TRUE, col = 2)
```

<img src="figure/cooking-survival-data-entry-stratified-1.png" alt="Cumulative hazard: delayed entry competing risks."  />

# Parametric hazard models
While the semi‑parametric Cox model provides substantial flexibility for
simulating survival data, there are situations where a fully parametric
simulation model is convenient or preferable. Here we consider a Weibull model
 parametrized so that the cumulative hazard is given by $$\Lambda(t) = \lambda
 \cdot t^s$$ where $s$ is the **shape parameter**, and $\lambda$ the **rate
 parameter**. We allow regression on both parameters
 \begin{align*} \lambda :=
 \exp(\beta^\top X), \quad s := \exp(\gamma^\top Z) \end{align*}
  where $X$ and
 $Z$ are covariate vectors. Specifically, this opens up for exploring
 non‑proportional hazards when $s$ depends on covariates.

Revisiting the TRACE data example we can compare the predictions from the Cox
and the Weibull-Cox model stratified by `chf` and with a proportional hazard
effect of `age` 

``` r
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw
#> 
#> - Weibull-Cox model -
#> 
#> Call:
#> phreg_weibull(formula = Surv(time, status > 0) ~ chf + age, shape.formula = ~chf, 
#>     data = sTRACE)
#> 
#> log-Likelihood: -684.750499 
#> 
#>    n events obs.time
#>  500    264 2228.481
#> 
#>               Estimate  Std.Err     2.5%   97.5%   P-value
#> (Intercept)   -5.59626 0.465886 -6.50938 -4.6831 3.070e-33
#> chf            0.83250 0.197629  0.44516  1.2198 2.526e-05
#> age            0.05331 0.006165  0.04123  0.0654 5.241e-18
#> ─────────────                                             
#> s:(Intercept) -0.44096 0.116740 -0.66977 -0.2122 1.585e-04
#> s:chf         -0.11794 0.133078 -0.37877  0.1429 3.755e-01

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)
```

<img src="figure/cooking-survival-data-weibull1-1.png" alt="Cumulative hazard: Weibull vs Cox stratified model."  />

To simulate data we can use the `rweibullcox()` function. Note that the
`stats::rweibull()` function gives a different parametrization where the
cumulative hazard is given by $H(t) = (t/b)^s$, i.e., with the same scale
parameter but where the scale parameter $b$ is related to the rate parameter we consider by $r := b^{-s}$.



``` r
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)
#>            [,1]       [,2]
#> X4522 -1.641818 -0.4409608
#> X1337 -1.576030 -0.4409608
#> X4490 -3.052267 -0.4409608
#> X5363 -2.003547 -0.4409608
#> X4518 -2.093627 -0.5589006
#> X1354 -1.154411 -0.5589006

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)
#>       chf    age     time status
#> X4522   0 74.174 4.104843   TRUE
#> X1337   0 75.408 4.239002   TRUE
#> X4490   0 47.718 8.000000  FALSE
#> X5363   0 67.389 1.402283   TRUE
#> X4518   1 50.084 1.246696   TRUE
#> X1354   1 67.701 6.829409  FALSE

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)
#> 
#> - Weibull-Cox model -
#> 
#> Call:
#> phreg_weibull(formula = Surv(time, status) ~ chf + age, shape.formula = ~chf, 
#>     data = newd)
#> 
#> log-Likelihood: -6692.027131 
#> 
#>     n events obs.time
#>  5000   2613 21578.83
#> 
#>               Estimate  Std.Err     2.5%    97.5%    P-value
#> (Intercept)   -5.70070 0.157299 -6.00900 -5.39240 1.365e-287
#> chf            0.86612 0.059267  0.74996  0.98229  2.290e-48
#> age            0.05453 0.002144  0.05033  0.05873 9.532e-143
#> ─────────────                                               
#> s:(Intercept) -0.42290 0.033445 -0.48845 -0.35735  1.199e-36
#> s:chf         -0.13715 0.039317 -0.21421 -0.06009  4.859e-04
```

All these steps are wrapped in the `simulate` method:

``` r
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)
#>         no wmi status chf    age sex diabetes     time vf
#> X6167 6167 1.8   TRUE   0 54.418   1        0 1.329640  0
#> X3847 3847 1.6  FALSE   1 66.119   0        0 7.635526  1
#> X695   695 0.8   TRUE   1 74.377   1        1 4.424567  0
#> X1599 1599 2.0   TRUE   0 74.986   0        0 3.600678  0
#> X6045 6045 2.0   TRUE   0 79.997   0        0 3.737537  0
```


# Multistate models: The Illness Death model 

Using hazard-based simulation with delayed entry we can simulate data
from the general illness-death model. The cumulative hazards for each transition
must be specified.

We specify

 - $\Lambda_{12}(t)$ the cumulative hazard for $1 \rightarrow 2$ transitions
 - $\Lambda_{21}(t)$ the cumulative hazard for $2 \rightarrow 1$ transitions
 - $\Lambda_{13}(t)$ the cumulative hazard for $1 \rightarrow 3$ transitions
 - $\Lambda_{23}(t)$ the cumulative hazard for $2 \rightarrow 3$ transitions

Transitions are then generated using these hazards. Covariate effects can be included
via proportional hazards models by supplying hazard ratios for all components,
as well as an exponential censoring model. Dependence between transitions can be introduced via:

 - `dependence=0`: independence
 - `dependence=1`: all hazards share a common gamma-distributed frailty (variance `var.z`)


\begin{tikzpicture}[
    >=Stealth,
    node distance=4cm,
    state/.style={
        rectangle,
        draw=black,
        thick,
        minimum width=3cm,
        minimum height=1cm,
        align=center
    }
]

  % Top states
  \node[state] (H) {Healthy \\ (1)};
  \node[state] (I) [right=of H] {Ill \\ (2)};

  % Dead centered below
  \node[state] (D) at ($(H)!0.5!(I) + (0,-3)$) {Dead \\ (3)};

  % Two straight arrows between Healthy and Ill
  \draw[->, thick] 
    ($(H.east)+(0,0.15)$) -- ($(I.west)+(0,0.15)$)
    node[midway, above] {$\lambda_{12}$};

  \draw[->, thick] 
    ($(I.west)+(0,-0.15)$) -- ($(H.east)+(0,-0.15)$)
    node[midway, below] {$\lambda_{21}$};

  % Death transitions
  \draw[->, thick] (H) -- node[left] {$\lambda_{13}$} (D);
  \draw[->, thick] (I) -- node[right] {$\lambda_{23}$} (D);

\end{tikzpicture}

We pass the cumulative hazards for each transition to `simMultistate`
to simulate data from the model, then re-estimate the parameters on the simulated
data to validate the procedure.



``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- sim_multistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
#> id: 1
#>   entry     time status rr death from to start     stop
#> 1     0 201.9469      3  1     1    1  3     0 201.9469
#> ------------------------------------------------------------ 
#> id: 2
#>        entry     time status rr death from to    start     stop
#> 2     0.0000 395.3879      2  1     0    1  2   0.0000 395.3879
#> 801 395.3879 555.8022      3  1     1    2  3 395.3879 555.8022
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-9-1.png" alt="Multistate model: estimated vs true transition rates."  />


# Cumulative incidence 

In this section we discuss how to simulate competing risks data with a specified cumulative
incidence function. We consider for simplicity a competing risks model with two causes and denote the
cumulative incidence functions as $F_1(t,X) = P(T < t, \epsilon=1|X)$ and $F_2(t,X) = P(T < t, \epsilon=2|X)$,
given covariate $X$.

To generate data with the required cumulative incidence functions, a simple approach is to first determine
whether the subject experiences an event and, if so, from which cause; then draw the event time according to the
conditional distribution.

For simplicity we consider survival times in a fixed interval $[0,\tau]$. Given $X$:

  - first, flip a three-sided coin with probabilities $F_1(\tau,X)$, $F_2(\tau,X)$, $1-F_1(\tau,X)-F_2(\tau,X)$ to decide whether the subject survives or
     experiences one of the two causes.
  - second, draw the event time using the cumulative incidence distribution. The timing of a cause $j$ event is
       $T = \tilde F_j^{-1}(U,X)$ with $\tilde F_j(s,X) = F_j(s,X)/F_j(\tau,X)$ and $U$ uniform.

Then indeed $P(T \leq t, \epsilon=j|X) = F_j(t,X)$ for $j=1,2$.

We again note and use that if $\tilde F_j(s)$ and $F_j(s)$ are piecewise linear
continuous functions then the inverse is easy to compute. 

A couple of details worth noting:

 - the coin flip to determine cause is based on an underlying uniform, $pU$, which can be supplied and shared across subjects
     to generate dependence in the risk.
 - the uniform used for generating the timing, $U$, can also be supplied and shared across subjects to
    generate dependence in the timing.


## Cumulative incidence I

Here we simulate two causes of death with two binary covariates using a logistic link
\begin{align*}
F_1(t,X)  &= \frac{ \Lambda_1(t,\rho_1) exp(X^T  \beta_1)}{1+\Lambda_1(t,\rho_1) exp(X^T  \beta_1)}
\end{align*}
and $F_2$ here enforcing the sum condition $F_1+F_2 \leq 1$
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)} [ 1- F_1(\tau,X) ]
\end{align*}
or without the constraint
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)}.
\end{align*}
When the sum condition is not enforced through the construction, then 
it is enforced ad-hoc when drawing the cause of death. 

The baselines are given as $\Lambda_j(t) = \rho_1 (1- exp(-t/r_j))$ where $\rho_j$ 
and $r_j$ are positive constants, and here $\tau=6$. 

To simulate the survival time we use a piecewise linear approximation of the cumulative
incidence functions and will thus depends on some grid for linear approximation. Our linear 
approximation can be made arbitrarily close to any specific smooth cumulative incidence function.

The function `simul_cifs`

 - uses a time-scale $[0,6]$, which can obviously be rescaled.
 - takes regression coefficients as a single vector, with $\beta_1$ followed by $\beta_2$.
 - defaults to two binary covariates, but a covariate matrix $Z$ can be supplied (dimensions must match the coefficient vector).
 - uses exponential censoring with rate `rc=0.5`, which can be made covariate-dependent (`dependence=1`) with covariate effects given by
   `rcZ`, giving rate $rc \cdot \exp(Z^T rcZ)$ (dimensions must match). Censoring can be disabled by setting `rc=NULL`.


The function `sim_cifs` takes the output from `cifregFG` or `cifreg` and simulates using the
baselines and covariate effects stored in those objects.



``` r
library(mets)
nsim <- 400
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
#> 
#>    n events
#>  400     74
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.45731  0.13528  0.12180  0.0007
#> Z2 -0.21636  0.26451  0.23259  0.4134
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.57982 1.21187 2.0595
#> Z2  0.80545 0.47961 1.3527
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
#> 
#>    n events
#>  400     83
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.27811  0.11066  0.11175  0.0120
#> Z2 -0.57140  0.22567  0.22874  0.0113
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.32063 1.06314 1.6405
#> Z2  0.56474 0.36288 0.8789
plot(ciff)
lines(attr(dats,"Lam1"))
```

<img src="figure/cooking-survival-data-unnamed-chunk-10-1.png" alt="Cumulative incidence: logistic and cloglog link models."  />

We can also use the parameters based on fitted models 


``` r
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
#>             [,1]       [,2]
#> tcell -0.7966259 -0.9373498
#> age    0.4164286  0.4779627
 cbind(cif2$coef,scif2$coef)
#>              [,1]       [,2]
#> tcell  0.66687029  0.7791722
#> age   -0.03248846 -0.3603559
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-11-1.png" alt="Cumulative incidence: fitted model vs simulated data."  />


## CIF Delayed entry

Now assume that given covariates $F_1(t;X) = P(T < t, \epsilon=1|X)$ and  $F_2(t;X) = P(T < t, \epsilon=2|X)$ are two 
cumulative incidence functions that satisfies the needed constraints. We wish to generate data that follows these two
piecewise linear cumulative incidence functions with delayed entry at time $s$.  Given delayed entry at time $s$ we 
should thus generate data that follows the cumulative incidence functions
$$
\tilde F_1(t,s;X)=   \frac{F_1(t;X) - F_1(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
and 
$$
\tilde F_2(t,s;X)=   \frac{F_2(t;X) - F_2(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
This can be done according to the recipe in the previous section.

 - First draw event type with conditional probabilities $\tilde F_j(t,sX)$ for $j=1,2$ and the remaining survivors
 - Second draw event time (timing) of chosen type with distribution  (still conditional on being a survivor at entry)
   \begin{align*}
  \frac{\tilde F_j(t,sX)}{\tilde F_j(\tau,X)} & = \frac{F_j(t,X)- F_j(s,X)}{F_j(\tau,X)-F_j(s,X)} \mbox{for} j=1,2.
   \end{align*}

If only $F_1$ is specified, the function assumes a pure survival setting with $F_2 \equiv 0$. Note also that 
given event type the timing is unaffected by the truncation probability.

For the cloglog link the Fine-Gray model the timing can be drawn as 
\begin{align*}
 \Lambda_0^{-1}[ -\log(1-U ( F_j(\tau,X)-F_j(s,X)) - F_j(s,X) ] \exp(- X^T \beta)
\end{align*}
and for the logit-link as 
\begin{align*}
 \Lambda_0^{-1}[ \exp(logit( U ( F_j(\tau,X)-F_j(s,X)) + F_j(s,X)))] \exp(- X^T \beta)
\end{align*}



``` r
 data(bmt)
 ## two cloglog 
 cif1 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=1)
 cif2 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=2)

 nsim <- 800
 entry <- rbinom(nsim,1,0.5)*runif(nsim)*60
 dd <- sim_cif(cif1,nsim,data=bmt,entry=entry)

 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-12-1.png" alt="Cumulative incidence: single cause with delayed entry."  />

Combining two causes drawn with restriction $F_1+\tilde F_2 \leq 1$, thus modifying $\tilde F_2= F_2 (1-F_1(\tau))$ where 
$F_1$ and $F_2$ are the two input cumulative incidence models 


``` r
 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt,entry=entry)

 ## logistic link; nonparametric Aalen-Johansen with delayed entry
 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-13-1.png" alt="Cumulative incidence: two causes with delayed entry."  />

Here we again combine two causes, now using parametric baselines via `simul_cifs`.
The baselines for $F_1$ and $F_2$ are returned as attributes; note that $F_2$ is modified
to satisfy the constraint $F_1 + F_2 \leq 1$ (remember that due to the restriction the $F_2$ model is modified).



``` r
rho1 <- 0.3; rho2 <- 4
set.seed(100)
beta=c(0.3,-0.3,-0.3,0.3)
dep=0
rc <- 0.9

n <- nsim
entry <- rbinom(n,1,0.5)*runif(n)*6
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7),entry=entry) 
###
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,ylim=c(0,0.4))
###

## and without delayed entry for comparison
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7)) 
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,add=TRUE)

## true baseline cif, cloglog link 
baset <- attr(data,"Lam1")[,2]
timet <- attr(data,"Lam1")[,1]
F1base <- 1-exp(-baset)
lines(timet,F1base,lwd=3)
```

<img src="figure/cooking-survival-data-unnamed-chunk-14-1.png" alt="Cumulative incidence: parametric baselines with delayed entry."  />


# Recurrent events

See also recurrent events vignette

 - `sim_recurrent_ts` simulates from the Two-Stage model where:
    - Rate of the terminal event among survivors is on Cox form (`phreg`)
    - Rate of recurrent events among survivors is on Cox form (`phreg`)
    - Marginal rate of recurrent events follows a Ghosh-Lin model (`recreg`)
    - Simulations are based on piecewise linear approximations on a grid
    - Events can be dependent via a gamma-distributed frailty
  - `sim_recurrentII`, `sim_recurrent`, `sim_recurrent_list`:
    - Frailty gamma models where the rates of recurrent events and the terminal event are specified via
       cumulative baselines and relative risk covariate effects, yielding a Cox model given the frailty and covariates
    - `simRecurrentList` supports multiple recurrent event types and multiple causes of death


Two-stage models

The following example fits Cox models for recurrent events and the terminal event on the `hfactioncpx12` dataset,
then simulates data from the estimated two-stage model and re-estimates to verify recovery of the parameters.


``` r
library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 200

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
#>            Estimate Std.Err   2.5%    97.5% P-value
#> treatment1  -0.1534 0.08145 -0.313 0.006286 0.05973
 estimate(dr)
#>            Estimate Std.Err    2.5%    97.5% P-value
#> treatment1  -0.4301  0.1831 -0.7889 -0.07132  0.0188

 simcoxcox <- sim_recurrent_ts(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
#>            Estimate Std.Err    2.5%     97.5% P-value
#> treatment1  -0.3901  0.1969 -0.7762 -0.004138 0.04759
 estimate(drs)
#>            Estimate Std.Err    2.5%  97.5% P-value
#> treatment1  -0.2158  0.2827 -0.7698 0.3382  0.4453

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-15-1.png" alt="Recurrent events: two-stage Cox model."  />

Now with Ghosh-Lin and Cox marginals:


``` r
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment1  -0.1104 0.07866 -0.2646 0.04376  0.1604
 estimate(dr)
#>            Estimate Std.Err    2.5%    97.5% P-value
#> treatment1  -0.4301  0.1831 -0.7889 -0.07132  0.0188

 simglcox <- sim_recurrent_ts(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(entry,time,status)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
#>            Estimate Std.Err    2.5%  97.5% P-value
#> treatment1 -0.05645  0.1593 -0.3687 0.2558   0.723
 estimate(drs)
#>            Estimate Std.Err   2.5%   97.5% P-value
#> treatment1  -0.7905  0.3202 -1.418 -0.1629 0.01355

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
 plot(drs)
 plot(dr,add=TRUE)
```

<img src="figure/cooking-survival-data-unnamed-chunk-16-1.png" alt="Recurrent events: Ghosh-Lin and Cox model."  />

We can also fit and simulate from stratified models:


``` r
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)
hf$age <- rnorm(741)[hf$id]
hf$Z1 <- rbinom(741,1,0.5)[hf$id]

xr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400 
rr <- sim_recurrent_ts(xr,dr,n=n,data=hf)
rxr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=rr)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age   0.0273 0.03922 -0.04958 0.1042  0.4864
estimate(rxr)
#>      Estimate Std.Err   2.5%  97.5% P-value
#> age 0.0002297 0.05522 -0.108 0.1085  0.9967
estimate(dr)
#>     Estimate Std.Err     2.5%    97.5% P-value
#> x   -0.42738 0.18252 -0.78510 -0.06965  0.0192
#> age  0.09527 0.08586 -0.07301  0.26354  0.2672
estimate(rdr)
#>     Estimate Std.Err    2.5%    97.5% P-value
#> x   -0.48230 0.20942 -0.8928 -0.07185 0.02128
#> age  0.04296 0.09506 -0.1433  0.22927 0.65130
plot(xr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-17-1.png" alt="Recurrent events: stratified two-stage model."  />

``` r
###

glr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=hf,death.code=2)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400
rr <- sim_recurrent_ts(glr,dr,n=n,data=hf)
rxr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=rr,death.code=3)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age   0.0273 0.03922 -0.04958 0.1042  0.4864
estimate(rxr)
#>     Estimate Std.Err     2.5%  97.5% P-value
#> age  0.06015 0.06414 -0.06556 0.1859  0.3483
estimate(dr)
#>     Estimate Std.Err     2.5%    97.5% P-value
#> x   -0.42738 0.18252 -0.78510 -0.06965  0.0192
#> age  0.09527 0.08586 -0.07301  0.26354  0.2672
estimate(rdr)
#>     Estimate Std.Err    2.5%    97.5% P-value
#> x   -0.48433  0.2076 -0.8913 -0.07738 0.01967
#> age  0.05767  0.1067 -0.1515  0.26682 0.58887
plot(glr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-17-2.png" alt="Recurrent events: stratified two-stage model."  />


Frailty models (simulations based on the rates/intensities) 


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 400
 rr <- sim_recurrent(n,base1,death.cumhaz=dr)
 ###
 mets:::showfitsim(causes=1,rr,dr,base1,base1,which=1:2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-1.png" alt="Recurrent events: frailty model simulations."  />

``` r

 rr <- sim_recurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
#> 
#>       status    0    1    2
#> death                      
#> 0              41 1123  161
#> 1             359    0    0
 mets:::showfitsim(causes=2,rr,dr,base1,base4,which=1:2)
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-2.png" alt="Recurrent events: frailty model simulations."  />

``` r

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- sim_recurrent_list(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
#> 
#>       status   0   1   2   3
#> death                       
#> 0             10 832 847 111
#> 1            277   0   0   0
#> 2            113   0   0   0
 mets:::showfitsimList(rr,cumhaz,drl) 
```

<img src="figure/cooking-survival-data-unnamed-chunk-18-3.png" alt="Recurrent events: frailty model simulations."  />


# SessionInfo



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Analysis of multivariate binomial data: family analysis"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5    
vignette: >
  %\VignetteIndexEntry{Analysis of multivariate binomial data: family analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Overview 
=============

When looking at multivariate binomial data with the aim of learning about the 
dependence that is present, possibly after correcting for some covariates many
models are available. 

   *  Random-effects models logistic regression covered elsewhere (glmer in lme4).

in the mets package you can fit the 

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

These last three models are all fitted in the mets package using composite 
likelihoods for pairs of data.  The models can be fitted specifically based 
on specifying which pairs one wants to use for the composite score. 

The models are described in futher details in the binomial-twin vignette. 


Simulated family data
====================

We start by simulating family data with and additive gamma structure
on ACE form.  Here 10000 families consisting of two parents and
two children. The response is ybin and there is one covariate x. 

```{r}
 library(mets)
 set.seed(100)
 data <- simbinClaytonOakes.family.ace(1000,2,1,beta=NULL,alpha=NULL)
 data$number <- c(1,2,3,4)
 data$child <- 1*(data$number==3)
 head(data)
```

We fit the marginal models, and here find a covariate
effect at 0.3 for x. The marginals can be specified excatly as one
wants. 

```{r}
 aa <- margbin <- glm(ybin~x,data=data,family=binomial())
 summary(aa)
```

Additive gamma model 
====================

For the additive gamma of this type we set-up the random effects
included in such a family to make the ACE valid using some
special functions for this. 

The model is constructe with one enviromental effect shared by
all in the family and 8 genetic random effects with size (1/4) genetic
variance. Looking at the first family we see that the mother and
father both share half the genes with the children and that the
two children also share half their genes with this specification.
Below we also show an alternative specification of this model
using all pairs. 

```{r}
# make ace random effects design
out <- ace.family.design(data,member="type",id="cluster")
out$pardes
head(out$des.rv,4)
```


We can now fit the model calling the two-stage function

```{r}
# fitting ace model for family structure
ts <- binomial.twostage(margbin,data=data,clusters=data$cluster,
theta=c(2,1),
random.design=out$des.rv,theta.des=out$pardes)
summary(ts)
# true variance parameters
c(2,1)
# total variance 
3
```


Pairwise fitting 
=================

We now specify the same model via extracting all pairs.  
The random
effecs structure is typically simpler when just looking at pairs, but we start by
specifying the random effects jointly for whole family. 
A special function writes up all combinations of pairs. 
There are 6 pairs within each family, and we keep track of
who belongs to the different families. We first simply give the
pairs and we then should get the same result as before. 



```{r}
mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)
```

Now with the pairs we fit the model 

```{r}
tsp <- binomial.twostage(margbin,data=data,clusters=data$cluster,theta=c(2,1),detail=0,
        random.design=out$des.rv,theta.des=out$pardes,pairs=pairs)
summary(tsp)
```

Here a random sample of pairs are given instead and we get other estimates. 

```{r}
set.seed(100)
ssid <- sort(sample(1:nrow(pairs),nrow(pairs)/2))
tsd <- binomial.twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1),step=1.0,
               random.design=out$des.rv,iid=1,Nit=10,
  	           theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)
```

To specify such a model when only the pairs are availble we show
how to specify the model. We here use the same marginal "aa"
to make the results comparable. 
The marginal can also be fitted based on available data. 

We start by selecting the data related to the pairs, and sets up new id's and 
to start we specify the model using the full design with 9 random effects. 
Below we show how one can use with only the random effects needed for each 
pair, which is typically simpler.


```{r}
head(pairs[ssid,])
ids <- sort(unique(c(pairs[ssid,])))

pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)

dataid <- dsort(data[ids,],"cluster")
outid <- ace.family.design(dataid,member="type",id="cluster")
outid$pardes
head(outid$des.rv)
```

Now fitting the model with the data set up 

```{r}
tsdid <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
         theta=c(2,1),random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)
```

We now specify the design specifically using the pairs. 
The random.design and design on the parameters
are now given for each pair, as a 3 dimensional matrix. 
with a direct specification of random.design and the 
design on the parameters theta.design.
In addition we need also to give the number of random effects for
each pair. These basic things are constructed by certain functions
for the ACE design. 


```{r}
pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new,7)
head(pair.types,7)
###
theta.des  <- rbind( c(rbind(c(1,0),  c(1,0),  c(0,1),  c(0,0))),
		     c(rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))))
random.des <- rbind( 
        c(1,0,1,0),c(0,1,1,0),
        c(1,1,0,1),c(1,0,1,1))
mf <- 1*(pair.types[,1]=="mother" & pair.types[,2]=="father")
##          pair, rv related to pairs,  theta.des related to pair 
pairs.new <- cbind(pair.new,(mf==1)*1+(mf==0)*3,(mf==1)*2+(mf==0)*4,(mf==1)*1+(mf==0)*2,(mf==1)*3+(mf==0)*4)
```

pairs.new is matrix with 

 * columns 1:2 giving the indeces of the  data points

 * columns 3:4 giving the indeces of the random.design for the different pairs

 * columns 5 giving the indeces of the theta.des written as rows 

 * columns 6 giving the number of random variables for this pair


The length of all rows of theta.des
are the maximum number of random effects $\times$ the number of parameters. 
These two numbers are given in the call. In this case 4 $\times$ 2. 
So theta.des has rows of length $8$, possibly including some 0's for rows
not relevant due to fewer random effects, as is the case here for pairs 
that do not share genetic effects.


For pair 1 that is a mother/farther pair, we see that they share 
1 environmental random effect of size 1. There are also two genetic
effects that are unshared between the two. 
So a total of 3 random effects are needed here. The theta.des relates the 3 
random effects to possible relationships in the parameters. Here the genetic
effects are full and so is the environmental effect. 
In contrast we also consider a mother/child pair that share half the genes, now
with random effects with (1/2) gene variance.  We there need 4 random effects,
2 non-shared half-gene, 1 shared half-gene, and one shared full environmental 
effect. 




```{r}
# 3 rvs here 
random.des
theta.des

head(pairs.new)
```

Now fitting the model, and we see that it is a lot quicker due to the
fewer random effects needed for pairs.  We need to also specify the number of
parameters in this case. 

```{r}
tsdid2 <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster, theta=c(2,1),
           random.design=random.des,theta.des=theta.des,pairs=pairs.new,dim.theta=2)
summary(tsdid2)
```

The same model can be specifed even simpler via the kinship
coefficient.  For this speicification there are 4 random effects for
each pair, but some have variance 0. The mother-father pair, here
shares a random effect with variance 0, and have two non-shared genetic 
effects with full variance, in addition to a fully shared environmental 
effect.

```{r}
kinship  <- rep(0.5,nrow(pair.types))
kinship[pair.types[,1]=="mother" & pair.types[,2]=="father"] <- 0
head(kinship,n=10)

out <- make.pairwise.design(pair.new,kinship,type="ace") 
```

```{r}
tsdid3 <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
             theta=c(2,1)/9,random.design=out$random.design,
             theta.des=out$theta.des,pairs=out$new.pairs,dim.theta=2)
summary(tsdid3)
```



Pairwise dependence modelling 
------------------------------


```{r}
library(mets)
set.seed(1000)
data <- simbinClaytonOakes.family.ace(10000,2,1,beta=NULL,alpha=NULL)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)
```


```{r}
 dtypes <- interaction( data[pairs[,1],"type"], data[pairs[,2],"type"])
 dtypes <- droplevels(dtypes)
 table(dtypes)
 dm <- model.matrix(~-1+factor(dtypes))
```

Now with the pairs we fit the model 

```{r}
aa <- margbin <- glm(ybin~x,data=data,family=binomial())

tsp <- binomial.twostage(aa,data=data, clusters=data$cluster,
		 theta.des=dm,pairs=cbind(pairs,1:nrow(dm)))
summary(tsp)
```


Pairwise odds ratio model 
=========================

To fit the pairwise odds-ratio model in the case of a pair-specification there 
are two options for fitting the model. 

1. One option is to set up some artificial data similar to twin data with
  + a pair-cluster-id  (clusters) 
  + with a cluster-id to get GEE type standard errors (se.cluster) 
2. We can also use the specify the design via the theta.des that is also a 
   matrix of dimension pairs x design with the design for POR model.  


Starting by the second option. We need to start by specify the design of
the odds-ratio of each pair. We set up the data and find all combinations 
within the pairs. Subsequently, we remove all the empty groups, by grouping
together the factor levels 4:9, and then we construct the design. 

```{r}
tdp <-cbind( dataid[pair.new[,1],],dataid[pair.new[,2],])
names(tdp) <- c(paste(names(dataid),"1",sep=""),
		paste(names(dataid),"2",sep=""))
tdp <-transform(tdp,tt=interaction(type1,type2))
dlevel(tdp)
drelevel(tdp,newlevels=list(mother.father=4:9)) <-  obs.types~tt
dtable(tdp,~tt+obs.types)
tdp <- model.matrix(~-1+factor(obs.types),tdp)
```

We then can fit the pairwise model using the pairs and the pair-design for
descrbing the OR. The results are consistent with the the ACE model as
the mother-father have a lower dependence as is due only the environmental
effects. All other combinations should have the same dependence as also seem
to be the case. 

To fit the OR model it is generally recommended to use the var.link to
use the parmetrization with log-odd-ratio regression.


```{r}
###porpair <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
###           theta.des=tdp,pairs=pair.new,model="or",var.link=1)
###summary(porpair)
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Analysis of multivariate binomial data: family analysis"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Analysis of multivariate binomial data: family analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview 
=============

When analysing multivariate binomial data with the aim of learning about the
dependence present, possibly after adjusting for covariates, many
models and methods are available:

   *  Random-effects logistic regression models covered elsewhere (`glmer` in `lme4`).

In the mets package you can fit the

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

These last three models are all fitted in the mets package using composite 
likelihoods based on pairs within clusters. The models can be fitted specifically based 
on specifying which pairs one wants to use for the composite score. 

The models are described in further details in the binomial-twin vignette. 


Simulated family data
====================

We start by simulating family data with an additive gamma structure
on ACE form.  Here 10000 families consisting of two parents and
two children. The response is ybin and there is one covariate x. 


``` r
 library(mets)
 library(timereg)
#> Loading required package: survival
#> 
#> Attaching package: 'timereg'
#> The following objects are masked from 'package:mets':
#> 
#>     Event, kmplot, plotConfregion
 set.seed(100)
 data <- sim_binClaytonOakes_family_ace(500,2,1,beta=NULL,alpha=NULL)
 data$number <- c(1,2,3,4)
 data$child <- 1*(data$number==3)
 head(data)
#>   ybin x   type cluster number child
#> 1    1 1 mother       1      1     0
#> 2    1 0 father       1      2     0
#> 3    1 0  child       1      3     1
#> 4    1 1  child       1      4     0
#> 5    1 1 mother       2      1     0
#> 6    1 0 father       2      2     0
```

We fit the marginal models, and here find a covariate effect at 0.3 for x. The marginals can be 
specified as desired.


``` r
 aa <- margbin <- glm(ybin~x,data=data,family=binomial())
 summary(aa)
#> 
#> Call:
#> glm(formula = ybin ~ x, family = binomial(), data = data)
#> 
#> Coefficients:
#>             Estimate Std. Error z value Pr(>|z|)    
#> (Intercept)  0.49908    0.06387   7.814 5.52e-15 ***
#> x            0.17531    0.09355   1.874   0.0609 .  
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> (Dispersion parameter for binomial family taken to be 1)
#> 
#>     Null deviance: 2610.2  on 1999  degrees of freedom
#> Residual deviance: 2606.7  on 1998  degrees of freedom
#> AIC: 2610.7
#> 
#> Number of Fisher Scoring iterations: 4
```

Additive Gamma model 
====================

For the Additive Gamma model we set-up the random effects
included in such a family to make the ACE valid using some
special functions for this. 

The model is constructed with one environmental effect shared by
all in the family and 8 genetic random effects with size 1/4 genetic
variance. Looking at the first family we see that the mother and
father both share half the genes with the children and that the
two children also share half their genes with this specification.
Below we also show an alternative specification of this model
using all pairs. 


``` r
# make ace random effects design
out <- ace_family_design(data,member="type",id="cluster")
out$pardes
#>       [,1] [,2]
#>  [1,] 0.25    0
#>  [2,] 0.25    0
#>  [3,] 0.25    0
#>  [4,] 0.25    0
#>  [5,] 0.25    0
#>  [6,] 0.25    0
#>  [7,] 0.25    0
#>  [8,] 0.25    0
#>  [9,] 0.00    1
head(out$des.rv,4)
#>      m1 m2 m3 m4 f1 f2 f3 f4 env
#> [1,]  1  1  1  1  0  0  0  0   1
#> [2,]  0  0  0  0  1  1  1  1   1
#> [3,]  1  1  0  0  1  1  0  0   1
#> [4,]  1  0  1  0  1  0  1  0   1
```


We can now fit the model calling the two-stage function


``` r
# fitting ace model for family structure
ts <- binomial_twostage(margbin,data=data,clusters=data$cluster,
theta=c(2,1),random.design=out$des.rv,theta.des=out$pardes)
summary(ts)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                theta        se
#> dependence1 2.817486 0.8981749
#> dependence2 1.137637 0.2806719
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err   2.5%  97.5%   P-value
#> dependence1   0.7124 0.09262 0.5308 0.8939 1.457e-14
#> dependence2   0.2876 0.09262 0.1061 0.4692 1.899e-03
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5%   P-value
#> p1    3.955  0.8668 2.256 5.654 5.051e-06
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
# true variance parameters
c(2,1)
#> [1] 2 1
# total variance 
3
#> [1] 3
```


Pairwise fitting 
=================

We now specify the same model by extracting all pairs.
The random effects structure is typically simpler when looking at pairs only, but we start by
specifying the random effects jointly for the whole family.
A special function writes up all combinations of pairs.
There are 6 pairs within each family, and we keep track of
which family each belongs to. We first simply supply the
pairs and should get the same result as before.



``` r
mm <- familycluster_index(data$cluster)
head(mm$familypairindex,n=20)
#>  [1] 1 2 1 3 1 4 2 3 2 4 3 4 5 6 5 7 5 8 6 7
pairs <- mm$pairs
dim(pairs)
#> [1] 3000    2
head(pairs,12)
#>       [,1] [,2]
#>  [1,]    1    2
#>  [2,]    1    3
#>  [3,]    1    4
#>  [4,]    2    3
#>  [5,]    2    4
#>  [6,]    3    4
#>  [7,]    5    6
#>  [8,]    5    7
#>  [9,]    5    8
#> [10,]    6    7
#> [11,]    6    8
#> [12,]    7    8
```

Now with the pairs we fit the model 


``` r
tsp <- binomial_twostage(margbin,data=data,clusters=data$cluster,theta=c(2,1),detail=0,
        random.design=out$des.rv,theta.des=out$pardes,pairs=pairs)
summary(tsp)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                theta        se
#> dependence1 2.817486 0.8981749
#> dependence2 1.137637 0.2806719
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err   2.5%  97.5%   P-value
#> dependence1   0.7124 0.09262 0.5308 0.8939 1.457e-14
#> dependence2   0.2876 0.09262 0.1061 0.4692 1.899e-03
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5%   P-value
#> p1    3.955  0.8668 2.256 5.654 5.051e-06
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

Here a random sample of pairs are given instead and we get other estimates. 


``` r
set.seed(100)
ssid <- sort(sample(1:nrow(pairs),nrow(pairs)/2))
tsd <- binomial_twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1),step=1.0,
               random.design=out$des.rv,iid=1,Nit=10,
  	           theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                 theta        se
#> dependence1 3.0275402 1.6045129
#> dependence2 0.8135103 0.3895686
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err     2.5%  97.5%   P-value
#> dependence1   0.7882  0.1511  0.49210 1.0843 1.816e-07
#> dependence2   0.2118  0.1511 -0.08431 0.5079 1.609e-01
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5% P-value
#> p1    3.841   1.402 1.093 6.589 0.00615
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

To specify such a model when only the pairs are available we show
how to specify the model. We here use the same marginal "aa"
to make the results comparable. 
The marginal can also be fitted based on available data. 

We start by selecting the data related to the pairs, setting up new ids, and
specifying the model using the full design with 9 random effects.
Below we also show how to use only the random effects needed for each
pair, which is typically simpler.



``` r
head(pairs[ssid,])
#>      [,1] [,2]
#> [1,]    1    2
#> [2,]    1    3
#> [3,]    2    3
#> [4,]    2    4
#> [5,]    3    4
#> [6,]    6    8
ids <- sort(unique(c(pairs[ssid,])))

pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)
#>      [,1] [,2]
#> [1,]    1    2
#> [2,]    1    3
#> [3,]    2    3
#> [4,]    2    4
#> [5,]    3    4
#> [6,]    5    7

dataid <- dsort(data[ids,],"cluster")
outid <- ace_family_design(dataid,member="type",id="cluster")
outid$pardes
#>       [,1] [,2]
#>  [1,] 0.25    0
#>  [2,] 0.25    0
#>  [3,] 0.25    0
#>  [4,] 0.25    0
#>  [5,] 0.25    0
#>  [6,] 0.25    0
#>  [7,] 0.25    0
#>  [8,] 0.25    0
#>  [9,] 0.00    1
head(outid$des.rv)
#>      m1 m2 m3 m4 f1 f2 f3 f4 env
#> [1,]  1  1  1  1  0  0  0  0   1
#> [2,]  0  0  0  0  1  1  1  1   1
#> [3,]  1  1  0  0  1  1  0  0   1
#> [4,]  1  0  1  0  1  0  1  0   1
#> [5,]  0  0  0  0  1  1  1  1   1
#> [6,]  1  1  0  0  1  1  0  0   1
```

Now fitting the model with the data set up 


``` r
aa <- glm(ybin~x,data=dataid,family=binomial())
tsdid <- binomial_twostage(aa,data=dataid,clusters=dataid$cluster,
         theta=c(2,1),random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                 theta       se
#> dependence1 3.1389859 1.649758
#> dependence2 0.8010544 0.399870
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err     2.5%  97.5%   P-value
#> dependence1   0.7967  0.1492  0.50421 1.0892 9.358e-08
#> dependence2   0.2033  0.1492 -0.08917 0.4958 1.731e-01
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5%  P-value
#> p1     3.94   1.438 1.121 6.759 0.006153
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

We now specify the design specifically using the pairs. 
The random.design and design on the parameters
are now given for each pair, as a 3 dimensional matrix. 
with a direct specification of random.design and the 
design on the parameters theta.design.
In addition we need also to give the number of random effects for
each pair. These basic things are constructed by certain functions
for the ACE design. 



``` r
pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new,7)
#>      [,1] [,2]
#> [1,]    1    2
#> [2,]    1    3
#> [3,]    2    3
#> [4,]    2    4
#> [5,]    3    4
#> [6,]    5    7
#> [7,]    6    7
head(pair.types,7)
#>      [,1]     [,2]    
#> [1,] "mother" "father"
#> [2,] "mother" "child" 
#> [3,] "father" "child" 
#> [4,] "father" "child" 
#> [5,] "child"  "child" 
#> [6,] "father" "child" 
#> [7,] "child"  "child"
###
theta.des  <- rbind( c(rbind(c(1,0),  c(1,0),  c(0,1),  c(0,0))),
		     c(rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))))
random.des <- rbind( 
        c(1,0,1,0),c(0,1,1,0),
        c(1,1,0,1),c(1,0,1,1))
mf <- 1*(pair.types[,1]=="mother" & pair.types[,2]=="father")
##          pair, rv related to pairs,  theta.des related to pair 
pairs.new <- cbind(pair.new,(mf==1)*1+(mf==0)*3,(mf==1)*2+(mf==0)*4,(mf==1)*1+(mf==0)*2,(mf==1)*3+(mf==0)*4)
```

`pairs.new` is a matrix with

 * columns 1:2 giving the indices of the data points

 * columns 3:4 giving the indices of the `random.design` for the different pairs

 * column 5 giving the indices of `theta.des` written as rows

 * column 6 giving the number of random variables for this pair


The length of all rows of theta.des
are the maximum number of random effects $\times$ the number of parameters. 
These two numbers are given in the call. In this case 4 $\times$ 2. 
So theta.des has rows of length $8$, possibly including some 0's for rows
not relevant due to fewer random effects, as is the case here for pairs 
that do not share genetic effects.


For pair 1 that is a mother/father pair, we see that they share 
1 environmental random effect of size 1. There are also two genetic
effects that are unshared between the two. 
So a total of 3 random effects are needed here. The theta.des relates the 3 
random effects to possible relationships in the parameters. Here the genetic
effects are full and so is the environmental effect. 
In contrast we also consider a mother/child pair that share half the genes, now
with random effects with (1/2) gene variance.  We there need 4 random effects,
2 non-shared half-gene, 1 shared half-gene, and one shared full environmental 
effect. 





``` r
# 3 rvs here 
random.des
#>      [,1] [,2] [,3] [,4]
#> [1,]    1    0    1    0
#> [2,]    0    1    1    0
#> [3,]    1    1    0    1
#> [4,]    1    0    1    1
theta.des
#>      [,1] [,2] [,3] [,4] [,5] [,6] [,7] [,8]
#> [1,]  1.0  1.0  0.0    0    0    0    1    0
#> [2,]  0.5  0.5  0.5    0    0    0    0    1

head(pairs.new)
#>      [,1] [,2] [,3] [,4] [,5] [,6]
#> [1,]    1    2    1    2    1    3
#> [2,]    1    3    3    4    2    4
#> [3,]    2    3    3    4    2    4
#> [4,]    2    4    3    4    2    4
#> [5,]    3    4    3    4    2    4
#> [6,]    5    7    3    4    2    4
```

Now fitting the model, and we see that it is a lot quicker due to the
fewer random effects needed for pairs.  We need to also specify the number of
parameters in this case. 


``` r
tsdid2 <- binomial_twostage(aa,data=dataid,clusters=dataid$cluster, theta=c(2,1),
           random.design=random.des,theta.des=theta.des,pairs=pairs.new,dim.theta=2)
summary(tsdid2)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                 theta       se
#> dependence1 3.1389859 1.649758
#> dependence2 0.8010544 0.399870
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err     2.5%  97.5%   P-value
#> dependence1   0.7967  0.1492  0.50421 1.0892 9.358e-08
#> dependence2   0.2033  0.1492 -0.08917 0.4958 1.731e-01
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5%  P-value
#> p1     3.94   1.438 1.121 6.759 0.006153
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

The same model can be specifed even simpler via the kinship
coefficient.  For this specification there are 4 random effects for
each pair, but some have variance 0. The mother-father pair, here
shares a random effect with variance 0, and have two non-shared genetic 
effects with full variance, in addition to a fully shared environmental 
effect.


``` r
kinship  <- rep(0.5,nrow(pair.types))
kinship[pair.types[,1]=="mother" & pair.types[,2]=="father"] <- 0
head(kinship,n=10)
#>  [1] 0.0 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5

out <- make_pairwise_design(pair.new,kinship,type="ace") 
```


``` r
tsdid3 <- binomial_twostage(aa,data=dataid,clusters=dataid$cluster,
             theta=c(2,1)/9,random.design=out$random.design,
             theta.des=out$theta.des,pairs=out$new.pairs,dim.theta=2)
summary(tsdid3)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                 theta       se
#> dependence1 3.1389859 1.649758
#> dependence2 0.8010544 0.399870
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err     2.5%  97.5%   P-value
#> dependence1   0.7967  0.1492  0.50421 1.0892 9.358e-08
#> dependence2   0.2033  0.1492 -0.08917 0.4958 1.731e-01
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err  2.5% 97.5%  P-value
#> p1     3.94   1.438 1.121 6.759 0.006153
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```



Pairwise dependence modelling 
------------------------------



``` r
library(mets)
set.seed(1000)
data <- sim_binClaytonOakes_family_ace(500,2,1,beta=NULL,alpha=NULL)
head(data)
#>   ybin x   type cluster
#> 1    0 1 mother       1
#> 2    0 0 father       1
#> 3    0 1  child       1
#> 4    0 0  child       1
#> 5    1 0 mother       2
#> 6    1 0 father       2
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

mm <- familycluster_index(data$cluster)
head(mm$familypairindex,n=20)
#>  [1] 1 2 1 3 1 4 2 3 2 4 3 4 5 6 5 7 5 8 6 7
pairs <- mm$pairs
dim(pairs)
#> [1] 3000    2
head(pairs,12)
#>       [,1] [,2]
#>  [1,]    1    2
#>  [2,]    1    3
#>  [3,]    1    4
#>  [4,]    2    3
#>  [5,]    2    4
#>  [6,]    3    4
#>  [7,]    5    6
#>  [8,]    5    7
#>  [9,]    5    8
#> [10,]    6    7
#> [11,]    6    8
#> [12,]    7    8
```



``` r
 dtypes <- interaction( data[pairs[,1],"type"], data[pairs[,2],"type"])
 dtypes <- droplevels(dtypes)
 table(dtypes)
#> dtypes
#>   child.child  father.child  mother.child mother.father 
#>           500          1000          1000           500
 dm <- model.matrix(~-1+factor(dtypes))
```

Now with the pairs we fit the model 


``` r
aa <- glm(ybin~x,data=data,family=binomial())

tsp <- binomial_twostage(aa,data=data, clusters=data$cluster,
		 theta.des=dm,pairs=cbind(pairs,1:nrow(dm)))
summary(tsp)
#> Dependence parameter for Odds-Ratio (Plackett) model
#> $estimates
#>                                theta        se
#> factor(dtypes)child.child   3.520109 0.7143594
#> factor(dtypes)father.child  3.871698 0.6055928
#> factor(dtypes)mother.child  3.922180 0.6233304
#> factor(dtypes)mother.father 1.328730 0.2657202
#> 
#> $log.or
#>                             Estimate Std.Err    2.5%  97.5%   P-value
#> factor(dtypes)child.child     1.2585  0.2029  0.8607 1.6562 5.596e-10
#> factor(dtypes)father.....     1.3537  0.1564  1.0471 1.6603 4.952e-18
#> factor(dtypes)mother.....     1.3666  0.1589  1.0552 1.6781 8.017e-18
#> factor(dtypes)mother......1   0.2842  0.2000 -0.1077 0.6762 1.552e-01
#> 
#> $type
#> [1] "plackett"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```


Pairwise odds ratio model 
=========================

To fit the pairwise odds-ratio model in the case of a pair-specification there 
are two options for fitting the model. 

1. One option is to set up some artificial data similar to twin data with
  + a pair-cluster-id  (clusters) 
  + with a cluster-id to get GEE type standard errors (se.cluster) 
2. We can also use the specify the design via the theta.des that is also a 
   matrix of dimension pairs x design with the design for POR model.  


Starting by the second option. We need to start by specify the design of
the odds-ratio of each pair. We set up the data and find all combinations 
within the pairs. Subsequently, we remove all the empty groups, by grouping
together the factor levels 4:9, and then we construct the design. 


``` r
tdp <-cbind( dataid[pair.new[,1],],dataid[pair.new[,2],])
names(tdp) <- c(paste(names(dataid),"1",sep=""),
		paste(names(dataid),"2",sep=""))
tdp <-transform(tdp,tt=interaction(type1,type2))
dlevel(tdp)
#> tt #levels=:6 
#> [1] "child.child"   "father.child"  "mother.child"  "child.father" 
#> [5] "father.father" "mother.father"
#> -----------------------------------------
drelevel(tdp,newlevels=list(mother.father=4:9)) <-  obs.types~tt
dtable(tdp,~tt+obs.types)
#> 
#>               obs.types mother.father child.child father.child mother.child
#> tt                                                                         
#> child.child                         0         232            0            0
#> father.child                        0           0          525            0
#> mother.child                        0           0            0          510
#> child.father                        0           0            0            0
#> father.father                       0           0            0            0
#> mother.father                     233           0            0            0
tdp <- model.matrix(~-1+factor(obs.types),tdp)
```

We can then fit the pairwise model using the pairs and the pair-design for
describing the OR. The results are consistent with the ACE model:
mother-father pairs have a lower dependence since it is driven only by environmental
effects. All other combinations appear to have the same dependence, as expected.

To fit the OR model it is generally recommended to use the var.link to
use the parametrization with log-odd-ratio regression.



``` r
###porpair <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
###           theta.des=tdp,pairs=pair.new,model="or",var.link=1)
###summary(porpair)
```


SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] digest_0.6.39          numDeriv_2016.8-1.1    codetools_0.2-20      
#>  [4] RcppArmadillo_15.2.6-1 Matrix_1.7-5           xfun_0.57             
#>  [7] lattice_0.22-9         splines_4.6.0          lava_1.9.2.1          
#> [10] knitr_1.51             parallel_4.6.0         mvtnorm_1.3-7         
#> [13] cli_3.6.6              parallelly_1.47.0      grid_4.6.0            
#> [16] future_1.70.0          compiler_4.6.0         globals_0.19.1        
#> [19] tools_4.6.0            listenv_0.10.1         future.apply_1.20.2   
#> [22] evaluate_1.0.5         Rcpp_1.1.1-1.1         otel_0.2.0            
#> [25] rlang_1.2.0
```

---
title: "Binomial Regression for Survival and Competing Risks Data"
author: Klaus Holst & Thomas Scheike
date: "2026-07-29"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Binomial Regression for Survival and Competing Risks Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Binomial Regression for censored data 
=====================================

The `binreg` function fits a logistic link model with IPCW adjustment for a specific time-point,
and can thus be used for both survival and competing risks data.
Computation is linear in data size, and influence functions are computed and available
for downstream calculations.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Details
=======

The binreg function does direct binomial regression for one time-point, $t$, fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta)  = F_1(t,X,\beta)
\end{align*}
based on an IPCW adjusted estimating equation (EE) with response  $Y(t)=I(T \leq t, \epsilon=1 )$
\begin{align*}
 U(\beta,\hat G_c) = &  X ( Y(t) \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)} - \mbox{expit}( X^T \beta)) = 0,
\end{align*}
with $G_c(t)=P(C>t)$, the  censoring survival distribution, and with $\Delta(t) = I( C_i > T_i \wedge t)$
the indicator of being uncensored at time $t$ (type="I"). 

The default type="II" is to augment with a censoring term, that is solve 
\begin{align*}
  &  U(\beta,\hat G_c) + \int_0^t X \frac{\hat E(Y(t)| T>u)}{\hat G_c(u)} d\hat M_c(u) =0 
\end{align*}
where $M_c(u)$ is the censoring martingale, this typically improves the performance. This is equivalent to the 
pseudo-value approach (see Overgaard (2025)).

The influence function for the type="II" estimator is 
\begin{align*}
    U(\beta,G_c) + \int_0^t X \frac{E(Y| T>u)}{G_c(u)} d M_c(u)  
    - \int_0^t  \frac{E(X| T>u) E(Y| T>u)}{G_c(u)} d M_c(u) - \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u)  
\end{align*}
and for type="I"
\begin{align*}
  &  U(\beta) + \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u).
\end{align*}
The means $E(X Y(t) | T>u)$ and $E(Y(t)| T>u)$ are estimated by IPCW estimators among survivors to get 
estimates of the influence functions.

The function logitIPCW instead considers 
\begin{align*}
 U^{glm}(\beta,\hat G_c) = & \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)}  X  ( Y(t) - \mbox{expit}( X^T \beta)) = 0.
\end{align*}
This score equation is quite similar to those of the binreg, and exactly the same when the censoring model is fully-nonparametric.

The logitIPCW has influence function 
\begin{align*}
  &  U^{glm}(\beta,G_c) + \int_0^t \frac{E( X ( Y - F_1(t,\beta)) | T>u)}{G_c(u)} d M_c(u)  
\end{align*}

Which estimator performs the best depends on the censoring distribution and it seems that the binreg with type="II" performs 
overall quite nicely (see Blanche et al (2023) and Overgaard (2025)).  For the full estimated censoring model all estimators have the
same influence function (see Blanche et al (2023)).

Additional functions logitATE, and binregATE computes the average treatment effect based on propensity and outcome modelling. 
We demonstrate this in another vignette. 

The functions logitATE/binregATE can be used when there is no censoring and we thus have simple binary outcome. 

The variance is based on a sandwich formula with IPCW adjustment (using the influence functions),
and `naive.var` is the variance under a known censoring model. The influence functions are stored in the output.
Clusters can be specified to obtain cluster-corrected standard errors.

Examples
========



``` r
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in random noise just below.
 data(bmt)
 bmt$time <- bmt$time+runif(nrow(bmt))*0.01
 bmt$id <- 1:408

 # logistic regression with IPCW binomial regression 
 out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
 summary(out)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.180338  0.126748 -0.428760  0.068084  0.1548
#> tcell       -0.418545  0.345480 -1.095675  0.258584  0.2257
#> platelet    -0.437644  0.240978 -0.909952  0.034665  0.0694
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.83499 0.65132 1.0705
#> tcell        0.65800 0.33431 1.2951
#> platelet     0.64556 0.40254 1.0353
```

We can also compute predictions using the estimates 

``` r
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
#>        pred         se     lower     upper
#> 1 0.3502406 0.04847582 0.2552280 0.4452533
#> 2 0.2618207 0.06969334 0.1252217 0.3984196
```

Further the censoring model can depend on strata 


``` r
 outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
 summary(outs)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.180697  0.127414 -0.430424  0.069030  0.1561
#> tcell       -0.365928  0.350632 -1.053154  0.321299  0.2967
#> platelet    -0.433494  0.240270 -0.904415  0.037428  0.0712
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.83469 0.65023 1.0715
#> tcell        0.69355 0.34884 1.3789
#> platelet     0.64824 0.40478 1.0381
```

Absolute risk differences and ratio
===================================

Now for illustrations we consider the absolute risk difference depending on tcell 


``` r
 outs <- binreg(Event(time,cause)~tcell,bmt,time=50,cens.model=~strata(tcell))
 summary(outs)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -0.30054  0.11153 -0.51914 -0.08194  0.0070
#> tcell       -0.51741  0.33981 -1.18342  0.14860  0.1278
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.74042 0.59503 0.9213
#> tcell        0.59606 0.30623 1.1602
```

the risk difference is 


``` r
ps <-  predict(outs,data.frame(tcell=c(0,1)),se=TRUE)
ps
#>        pred         se     lower     upper
#> 1 0.4254253 0.02726306 0.3719897 0.4788609
#> 2 0.3061988 0.06819019 0.1725461 0.4398516
sum( c(1,-1) * ps[,1])
#> [1] 0.1192264
```

Getting the standard errors is straightforward since the two groups are
independent. If we had additionally adjusted for other covariates, however, we would need
to apply the delta theorem using the relevant covariances, along the lines of:


``` r
dd <- data.frame(tcell=c(0,1))
p <- predict(outs,dd)

riskdifratio <- function(p,contrast=c(1,-1)) {
   outs$coef <- p
   p <- predict(outs,dd)[,1]
   pd <- sum(contrast*p)
   r1 <- p[1]/p[2]
   r2 <- p[2]/p[1]
   return(c(pd,r1,r2))
}
     
estimate(outs,f=riskdifratio,dd,null=c(0,1,1))
#> Call: estimate.default(x = outs, f = riskdifratio, dd, null = c(0, 
#>     1, 1))
#> ────────────────────────────────────────────────────────────
#>    Estimate Std.Err     2.5%  97.5% P-value
#> p1   0.1192 0.07344 -0.02471 0.2632 0.10448
#> p2   1.3894 0.32197  0.75833 2.0204 0.22652
#> p3   0.7197 0.16679  0.39284 1.0467 0.09291
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] = 0
#>   [p2] = 1
#>   [p3] = 1 
#>  
#> chisq = 12.0249, df = 2, p-value = 0.002448
```

same as 


``` r
run <- 0
if (run==1) {
library(prodlim)
pl <- prodlim(Hist(time,cause)~tcell,bmt)
spl <- summary(pl,times=50,asMatrix=TRUE)
spl
}
```



SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.12   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.2.1           stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Cooking Survival Data: 5-Minute Recipes"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    toc: true
    # fig_width: 7.15
    # fig_height: 5.5        
header-includes: 
  - \usepackage{tikz}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Cooking survival data, 5 minutes recipes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"  
)
```

# Overview 

Simulation of survival data is important for both
theoretical and practical work. In a practical setting we might wish to
validate that standard errors are valid even in a rather small sample,
or validate that a complicated procedure is doing as intended.
Therefore it is useful to have simple tools for generating survival data
that resembles a particular dataset as closely as possible. In a theoretical
setting we are often interested in evaluating the finite-sample
properties of a new procedure in different settings that are
motivated by a specific practical problem. The aim is to provide such tools.

Bender et al. discussed how to generate survival data based on the Cox model,
and considered only a subset of the available parametric survival models (Weibull, exponential).
We start by restricting attention to piecewise linear baseline functions, which make it easy to simulate
data that follows closely the baseline estimated from the data using semi- or
nonparametric models. This makes it straightforward to capture important aspects of the
data. We later return to extended Weibull baseline hazard models.

Different survival models can be cooked, and we here give recipes for
hazard and cumulative incidence based simulations. More recipes are
given in vignette about recurrent events. 

  - hazard based. 
    - Cox Regression  models.
       - piecewise constant baselines 
       - parametric Weibull models
  - cumulative incidence. 
    - Regression models with cloglog or logistic link. 
  - recurrent events (see recurrent events vignette).
    - Regression models with exp link, for rates or Ghosh-Lin type.


```{r}
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations
```

# Hazard based, Cox models  


Given a survival time $T$ with cumulative hazard $\Lambda(t)=\int_0^t \lambda(s) ds$, it
follows that, with $E \sim Exp(1)$ (exponential with rate 1), $\Lambda^{-1}(E)$ will have the
same distribution as $T$.

This provides the basis for simulations of survival times with a given hazard and is
a consequence of this simple calculation
$$
  P(\Lambda^{-1}(E) > t) = P(E > \Lambda(t)) = \exp( - \Lambda(t)) = P(T > t).
$$

Similarly if $T$ given $X$ has hazard on Cox form
$$
  \lambda_0(t) \exp( X^T \beta)
$$
where $\beta$ is a $p$-dimensional regression coefficient and $\lambda_0(t)$ a baseline
hazard function,
then it is useful to observe also that
$\Lambda^{-1}(E/HR)$ with $HR=\exp(X^T \beta)$ has the same distribution as $T$ given $X$.

Therefore, if the inverse of the cumulative hazard can be computed, we can generate survival times with
a specified hazard function. One useful observation is that for a piecewise linear continuous
cumulative hazard on an interval $[0,\tau]$, $\Lambda_l(t)$, it is easy to compute the inverse.

Further, any cumulative hazard can be approximated by a piecewise linear continuous
cumulative hazard, enabling simulation from this approximation. Recall that fitting
the Cox model to data yields a piecewise constant cumulative hazard and regression coefficients;
with these in hand we can approximate the piecewise constant Breslow estimator
by simply connecting the jump points with straight lines.

The engine is to simulate data with a given linear cumulative hazard. First generating survival data based on
the cumulative hazard `cumhaz`:

```{r}
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)
```

Now we fit a simple Cox model

```{r}
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim_phreg(cox1,n,data=bmt)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)

 ## modify regression coefficients 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim_phreg(cox10,n,data=bmt)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)
```

Multiple Cox models for cause-specific hazards can be combined. Below we draw the
covariates manually; alternatively, `sim_phregs` draws covariates directly from
the data.

```{r}
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
```

We now consider a fully nonparametric model with stratified baselines.

```{r}
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim_phreg(coxs,nsim,data=NULL,strata=strata)

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim_phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)
```

We now fit cause-specific hazard models with three causes (treating censoring as one of them)
and generate competing risks data with hazards taken from the fitted Cox models.
The following example includes stratified baselines for some of the models:

```{r,stratified}
 ## competing risks with phreg 
 cox0 <- phreg(Surv(time,cause==0)~tcell+platelet,data=bmt)
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
 coxs <- list(cox0,cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)

 ## verify cause-specific hazards match fitted model; increase n for better agreement
 scox0 <- phreg(Surv(time,cause==1)~tcell+platelet,data=dd)
 scox1 <- phreg(Surv(time,cause==2)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==3)~strata(tcell)+platelet,data=dd)
 cbind(cox0$coef,scox0$coef)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,3))
 plot(cox0); plot(scox0,add=TRUE,col=2); 
 plot(cox1); plot(scox1,add=TRUE,col=2); 
 plot(cox2); plot(scox2,add=TRUE,col=2); 
 
 ########################################
 ## second example 
 ########################################

 cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
 coxs <- list(cox1,cox2)
 dd <- sim_phregs(coxs,n,data=bmt)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=dd)
 scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE); 
 plot(cox2); plot(scox2,add=TRUE); 
```

One further example, again fully nonparametric:

```{r}
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim_phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~cause)
 scox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,cause==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)
```

**Summary of simulation functions**

| Function      | Purpose                                                     |
|---------------|-------------------------------------------------------------|
| `sim_phreg`   | Single `phreg` model; supports stratified baselines         |
| `sim_phregs`  | List of cause-specific `phreg` models; draws covariates automatically |


# Delayed entry

If $T$ given $X$ have hazard  on Cox form 
$$
  \lambda_0(t) \exp( X^T \beta)
$$
and we wish to generate data according to this hazard for those that are alive at time $s$, that is 
draw from the distribution of $T$ given $T>s$ (all given $X$ ), then we note that  
$$
\Lambda_0^{-1}( \Lambda_0(s) + E/HR)) 
$$
with $HR=\exp(X^T \beta))$ and with $E \sim Exp(1)$ has the distribution we are after. 

This is again a consequence of a simple calculation
$$
P_X\!\left(\Lambda^{-1}\!\left(\Lambda(s) + E/HR\right) > t\right)
= P_X\!\left(E > HR\!\left(\Lambda(t) - \Lambda(s)\right)\right)
= P_X(T > t \mid T > s).
$$

We here illustrate how to simulate from a competing risks model based on 
Cox hazards. 

```{r sim-phregs, label="entry-stratified"}
data(bmt)
cox0 <- phreg(Surv(time, cause == 0) ~ tcell + platelet+age,          data = bmt)
cox1 <- phreg(Surv(time, cause == 1) ~ tcell + platelet+age,          data = bmt)
cox2 <- phreg(Surv(time, cause == 2) ~ strata(tcell) + platelet+age,  data = bmt)

nsim <- 800
entry <- rbinom(nsim, 1, 0.5) * runif(nsim) * 60

dd <- sim_phreg(cox0,nsim, data = bmt,entry=entry)
scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
cbind(cox0$coef, scox0$coef)
par(mfrow = c(2, 2))
plot(cox0); plot(scox0, add = TRUE, col = 2)

dd <- sim_phregs(list(cox0, cox1, cox2), nsim, data = bmt,entry=entry)

scox0 <- phreg(Surv(entry,time, cause == 1) ~ tcell + platelet+age,         data = dd)
scox1 <- phreg(Surv(entry,time, cause == 2) ~ tcell + platelet+age,         data = dd)
scox2 <- phreg(Surv(entry,time, cause == 3) ~ strata(tcell) + platelet+age, data = dd)

cbind(cox0$coef, scox0$coef)
cbind(cox1$coef, scox1$coef)
cbind(cox2$coef, scox2$coef)

plot(cox0); plot(scox0, add = TRUE, col = 2)
plot(cox1); plot(scox1, add = TRUE, col = 2)
plot(cox2); plot(scox2, add = TRUE, col = 2)
```

# Parametric hazard models
While the semi‑parametric Cox model provides substantial flexibility for
simulating survival data, there are situations where a fully parametric
simulation model is convenient or preferable. Here we consider a Weibull model
 parametrized so that the cumulative hazard is given by $$\Lambda(t) = \lambda
 \cdot t^s$$ where $s$ is the **shape parameter**, and $\lambda$ the **rate
 parameter**. We allow regression on both parameters
 \begin{align*} \lambda :=
 \exp(\beta^\top X), \quad s := \exp(\gamma^\top Z) \end{align*}
  where $X$ and
 $Z$ are covariate vectors. Specifically, this opens up for exploring
 non‑proportional hazards when $s$ depends on covariates.

Revisiting the TRACE data example we can compare the predictions from the Cox
and the Weibull-Cox model stratified by `chf` and with a proportional hazard
effect of `age` 
```{r weibull1}
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)
```

To simulate data we can use the `rweibullcox()` function. Note that the
`stats::rweibull()` function gives a different parametrization where the
cumulative hazard is given by $H(t) = (t/b)^s$, i.e., with the same scale
parameter but where the scale parameter $b$ is related to the rate parameter we consider by $r := b^{-s}$.


```{r weibull_sim}
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)
```

All these steps are wrapped in the `simulate` method:
```{r}
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)
```


# Multistate models: The Illness Death model 

Using hazard-based simulation with delayed entry we can simulate data
from the general illness-death model. The cumulative hazards for each transition
must be specified.

We specify

 - $\Lambda_{12}(t)$ the cumulative hazard for $1 \rightarrow 2$ transitions
 - $\Lambda_{21}(t)$ the cumulative hazard for $2 \rightarrow 1$ transitions
 - $\Lambda_{13}(t)$ the cumulative hazard for $1 \rightarrow 3$ transitions
 - $\Lambda_{23}(t)$ the cumulative hazard for $2 \rightarrow 3$ transitions

Transitions are then generated using these hazards. Covariate effects can be included
via proportional hazards models by supplying hazard ratios for all components,
as well as an exponential censoring model. Dependence between transitions can be introduced via:

 - `dependence=0`: independence
 - `dependence=1`: all hazards share a common gamma-distributed frailty (variance `var.z`)


\begin{tikzpicture}[
    >=Stealth,
    node distance=4cm,
    state/.style={
        rectangle,
        draw=black,
        thick,
        minimum width=3cm,
        minimum height=1cm,
        align=center
    }
]

  % Top states
  \node[state] (H) {Healthy \\ (1)};
  \node[state] (I) [right=of H] {Ill \\ (2)};

  % Dead centered below
  \node[state] (D) at ($(H)!0.5!(I) + (0,-3)$) {Dead \\ (3)};

  % Two straight arrows between Healthy and Ill
  \draw[->, thick] 
    ($(H.east)+(0,0.15)$) -- ($(I.west)+(0,0.15)$)
    node[midway, above] {$\lambda_{12}$};

  \draw[->, thick] 
    ($(I.west)+(0,-0.15)$) -- ($(H.east)+(0,-0.15)$)
    node[midway, below] {$\lambda_{21}$};

  % Death transitions
  \draw[->, thick] (H) -- node[left] {$\lambda_{13}$} (D);
  \draw[->, thick] (I) -- node[right] {$\lambda_{23}$} (D);

\end{tikzpicture}

We pass the cumulative hazards for each transition to `simMultistate`
to simulate data from the model, then re-estimate the parameters on the simulated
data to validate the procedure.


```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- sim_multistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)

```


# Cumulative incidence 

In this section we discuss how to simulate competing risks data with a specified cumulative
incidence function. We consider for simplicity a competing risks model with two causes and denote the
cumulative incidence functions as $F_1(t,X) = P(T < t, \epsilon=1|X)$ and $F_2(t,X) = P(T < t, \epsilon=2|X)$,
given covariate $X$.

To generate data with the required cumulative incidence functions, a simple approach is to first determine
whether the subject experiences an event and, if so, from which cause; then draw the event time according to the
conditional distribution.

For simplicity we consider survival times in a fixed interval $[0,\tau]$. Given $X$:

  - first, flip a three-sided coin with probabilities $F_1(\tau,X)$, $F_2(\tau,X)$, $1-F_1(\tau,X)-F_2(\tau,X)$ to decide whether the subject survives or
     experiences one of the two causes.
  - second, draw the event time using the cumulative incidence distribution. The timing of a cause $j$ event is
       $T = \tilde F_j^{-1}(U,X)$ with $\tilde F_j(s,X) = F_j(s,X)/F_j(\tau,X)$ and $U$ uniform.

Then indeed $P(T \leq t, \epsilon=j|X) = F_j(t,X)$ for $j=1,2$.

We again note and use that if $\tilde F_j(s)$ and $F_j(s)$ are piecewise linear
continuous functions then the inverse is easy to compute. 

A couple of details worth noting:

 - the coin flip to determine cause is based on an underlying uniform, $pU$, which can be supplied and shared across subjects
     to generate dependence in the risk.
 - the uniform used for generating the timing, $U$, can also be supplied and shared across subjects to
    generate dependence in the timing.


## Cumulative incidence I

Here we simulate two causes of death with two binary covariates using a logistic link
\begin{align*}
F_1(t,X)  &= \frac{ \Lambda_1(t,\rho_1) exp(X^T  \beta_1)}{1+\Lambda_1(t,\rho_1) exp(X^T  \beta_1)}
\end{align*}
and $F_2$ here enforcing the sum condition $F_1+F_2 \leq 1$
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)} [ 1- F_1(\tau,X) ]
\end{align*}
or without the constraint
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta_2)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta_2)}.
\end{align*}
When the sum condition is not enforced through the construction, then 
it is enforced ad-hoc when drawing the cause of death. 

The baselines are given as $\Lambda_j(t) = \rho_1 (1- exp(-t/r_j))$ where $\rho_j$ 
and $r_j$ are positive constants, and here $\tau=6$. 

To simulate the survival time we use a piecewise linear approximation of the cumulative
incidence functions and will thus depends on some grid for linear approximation. Our linear 
approximation can be made arbitrarily close to any specific smooth cumulative incidence function.

The function `simul_cifs`

 - uses a time-scale $[0,6]$, which can obviously be rescaled.
 - takes regression coefficients as a single vector, with $\beta_1$ followed by $\beta_2$.
 - defaults to two binary covariates, but a covariate matrix $Z$ can be supplied (dimensions must match the coefficient vector).
 - uses exponential censoring with rate `rc=0.5`, which can be made covariate-dependent (`dependence=1`) with covariate effects given by
   `rcZ`, giving rate $rc \cdot \exp(Z^T rcZ)$ (dimensions must match). Censoring can be disabled by setting `rc=NULL`.


The function `sim_cifs` takes the output from `cifregFG` or `cifreg` and simulates using the
baselines and covariate effects stored in those objects.


```{r}
library(mets)
nsim <- 400
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul_cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
plot(ciff)
lines(attr(dats,"Lam1"))
```

We can also use the parameters based on fitted models 

```{r}
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
 cbind(cif2$coef,scif2$coef)
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)
```


## CIF Delayed entry

Now assume that given covariates $F_1(t;X) = P(T < t, \epsilon=1|X)$ and  $F_2(t;X) = P(T < t, \epsilon=2|X)$ are two 
cumulative incidence functions that satisfies the needed constraints. We wish to generate data that follows these two
piecewise linear cumulative incidence functions with delayed entry at time $s$.  Given delayed entry at time $s$ we 
should thus generate data that follows the cumulative incidence functions
$$
\tilde F_1(t,s;X)=   \frac{F_1(t;X) - F_1(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
and 
$$
\tilde F_2(t,s;X)=   \frac{F_2(t;X) - F_2(s;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
This can be done according to the recipe in the previous section.

 - First draw event type with conditional probabilities $\tilde F_j(t,sX)$ for $j=1,2$ and the remaining survivors
 - Second draw event time (timing) of chosen type with distribution  (still conditional on being a survivor at entry)
   \begin{align*}
  \frac{\tilde F_j(t,sX)}{\tilde F_j(\tau,X)} & = \frac{F_j(t,X)- F_j(s,X)}{F_j(\tau,X)-F_j(s,X)} \mbox{for} j=1,2.
   \end{align*}

If only $F_1$ is specified, the function assumes a pure survival setting with $F_2 \equiv 0$. Note also that 
given event type the timing is unaffected by the truncation probability.

For the cloglog link the Fine-Gray model the timing can be drawn as 
\begin{align*}
 \Lambda_0^{-1}[ -\log(1-U ( F_j(\tau,X)-F_j(s,X)) - F_j(s,X) ] \exp(- X^T \beta)
\end{align*}
and for the logit-link as 
\begin{align*}
 \Lambda_0^{-1}[ \exp(logit( U ( F_j(\tau,X)-F_j(s,X)) + F_j(s,X)))] \exp(- X^T \beta)
\end{align*}


```{r}
 data(bmt)
 ## two cloglog 
 cif1 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=1)
 cif2 <-  cifregFG(Event(time,cause)~tcell+platelet,data=bmt,cause=2)

 nsim <- 800
 entry <- rbinom(nsim,1,0.5)*runif(nsim)*60
 dd <- sim_cif(cif1,nsim,data=bmt,entry=entry)

 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

Combining two causes drawn with restriction $F_1+\tilde F_2 \leq 1$, thus modifying $\tilde F_2= F_2 (1-F_1(\tau))$ where 
$F_1$ and $F_2$ are the two input cumulative incidence models 

```{r}
 dd <- sim_cifs(list(cif1,cif2),nsim,data=bmt,entry=entry)

 ## logistic link; nonparametric Aalen-Johansen with delayed entry
 scif1 <-  cif(Event(entry,time,cause)~strata(tcell,platelet),data=dd,cause=1)
 plot(scif1); 
 ###
 pcif1 <- predict(cif1,expand.grid(tcell=0:1,platelet=0:1))
 plot(pcif1,add=TRUE)
```

Here we again combine two causes, now using parametric baselines via `simul_cifs`.
The baselines for $F_1$ and $F_2$ are returned as attributes; note that $F_2$ is modified
to satisfy the constraint $F_1 + F_2 \leq 1$ (remember that due to the restriction the $F_2$ model is modified).


```{r}
rho1 <- 0.3; rho2 <- 4
set.seed(100)
beta=c(0.3,-0.3,-0.3,0.3)
dep=0
rc <- 0.9

n <- nsim
entry <- rbinom(n,1,0.5)*runif(n)*6
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7),entry=entry) 
###
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,ylim=c(0,0.4))
###

## and without delayed entry for comparison
data <- simul_cifs(n,rho1,rho2,beta,bin=1,rc=0.5,rate=c(3,7)) 
scif1 <- cif(Event(entry,time,status)~strata(Z1,Z2),data)
plot(scif1,add=TRUE)

## true baseline cif, cloglog link 
baset <- attr(data,"Lam1")[,2]
timet <- attr(data,"Lam1")[,1]
F1base <- 1-exp(-baset)
lines(timet,F1base,lwd=3)
```


# Recurrent events

See also recurrent events vignette

 - `sim_recurrent_ts` simulates from the Two-Stage model where:
    - Rate of the terminal event among survivors is on Cox form (`phreg`)
    - Rate of recurrent events among survivors is on Cox form (`phreg`)
    - Marginal rate of recurrent events follows a Ghosh-Lin model (`recreg`)
    - Simulations are based on piecewise linear approximations on a grid
    - Events can be dependent via a gamma-distributed frailty
  - `sim_recurrentII`, `sim_recurrent`, `sim_recurrent_list`:
    - Frailty gamma models where the rates of recurrent events and the terminal event are specified via
       cumulative baselines and relative risk covariate effects, yielding a Cox model given the frailty and covariates
    - `simRecurrentList` supports multiple recurrent event types and multiple causes of death


Two-stage models

The following example fits Cox models for recurrent events and the terminal event on the `hfactioncpx12` dataset,
then simulates data from the estimated two-stage model and re-estimates to verify recovery of the parameters.

```{r}
library(mets)
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 200

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
 estimate(dr)

 simcoxcox <- sim_recurrent_ts(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)

```

Now with Ghosh-Lin and Cox marginals:

```{r}
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
 estimate(dr)

 simglcox <- sim_recurrent_ts(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(entry,time,status)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(entry,time,status==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
 plot(drs)
 plot(dr,add=TRUE)

```

We can also fit and simulate from stratified models:

```{r}
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)
hf$age <- rnorm(741)[hf$id]
hf$Z1 <- rbinom(741,1,0.5)[hf$id]

xr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400 
rr <- sim_recurrent_ts(xr,dr,n=n,data=hf)
rxr <- phreg(Surv(entry,time,status==1)~strata(x)+age+cluster(id),data=rr)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
estimate(rxr)
estimate(dr)
estimate(rdr)
plot(xr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
###

glr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=hf,death.code=2)
dr <- phreg(Surv(entry,time,status==2)~x+strata(Z1)+age+cluster(id),data=hf)
n <- 400
rr <- sim_recurrent_ts(glr,dr,n=n,data=hf)
rxr <- recreg(Event(entry,time,status)~strata(x)+age+cluster(id),data=rr,death.code=3)
rdr <- phreg(Surv(entry,time,status==3)~x+strata(Z1)+age+cluster(id),data=rr)
estimate(xr)
estimate(rxr)
estimate(dr)
estimate(rdr)
plot(glr); plot(rxr,add=TRUE)
plot(dr,add=TRUE,col=2); plot(rdr,add=TRUE,col=2)
```


Frailty models (simulations based on the rates/intensities) 

```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 400
 rr <- sim_recurrent(n,base1,death.cumhaz=dr)
 ###
 mets:::showfitsim(causes=1,rr,dr,base1,base1,which=1:2)

 rr <- sim_recurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
 mets:::showfitsim(causes=2,rr,dr,base1,base4,which=1:2)

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- sim_recurrent_list(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
 mets:::showfitsimList(rr,cumhaz,drl) 
```


# SessionInfo


```{r}
sessionInfo()
```

---
title: "Analysis of bivariate binomial data: Twin analysis"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Analysis of bivariate binomial data: Twin analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview 
==========

When analysing bivariate binomial data with the aim of learning about the
dependence present, possibly after adjusting for covariates, many
models are available.

   *  Random-effects logistic regression models covered elsewhere (`glmer` in `lme4`).

In the mets package you can fit the

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.


Typically it can be hard or impossible
to specify random effects models with special
structure among the parameters of the random effects. This is possible with
our models.

To be concrete about the model structure, assume that we have paired binomial
data $(Y_1, Y_2, X_1, X_2)$ where the responses are $Y_1, Y_2$ and the
covariates are $X_1, X_2$.

We start by giving a brief description of these different models.  First we
for bivariate data one can specify the marginal probability using logistic 
regression models 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2.
\]
These models can be estimated under working independence (Zeger and Liang, 1986).

A typical twin analysis will consist of looking at both

   *  Pairwise odds ratio model

   *  Bivariate Probit model
  
The additive gamma can be used for the same as the bivariate probit model but 
is more restrictive in terms of dependence structure, but is nevertheless 
still valuable to have also as a check of results of the bivariate probit
model. 


Biprobit with random effects
=============================

For these models we assume that given random effects $Z$ and a covariate vector
$V_{12}$ we have independent probit regression models
\[
\mbox{probit}(P(Y_i=1|X_i, Z)) = \alpha_i + X_i^T \beta + V_{12}^T Z,  \quad i=1,2.
\]
where $Z$ follows a bivariate normal distribution with covariance
$\Sigma$. The general covariance structure
$\Sigma$ makes the model very flexible.

We note that

 - Parameters $\beta$ are subject-specific
 - $\Sigma$ reflects the dependence structure

The more standard $\mbox{logit}$ link rather than the $\mbox{probit}$ link
is often used and implemented in, for example, Tutz and Hennevogl (1996). The advantage is that one obtains an odds-ratio interpretation
of the subject-specific effects, but numerical integration is then needed to
fit the model.

Pairwise odds ratio model 
------------------------


The pairwise odds ratio model specifies that given $X_1, X_2$
the marginal models are
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

The primary object of interest are the odds ratio between $Y_{1}$ and $Y_{2}$
\[
\gamma_{12} = \frac{ P(  Y_{ki} =1 , Y_{kj} =1) P(  Y_{ki} =0 , Y_{kj} =0) }{ 
  P(  Y_{ki} =1 , Y_{kj} =0) P(  Y_{ki} =0 , Y_{kj} =1) }
\]
given $X_{ki}$, $X_{kj}$, and $Z_{kji}$. 

We model the odds ratio with the regression 
\[
\gamma_{12} = \exp( Z_{12}^T \lambda)
\]
Where $Z_{12}$ are covariates that may influence the odds ratio
between $Y_{1}$ and $Y_{2}$ and contains the marginal covariates
\cite{carey-1993,dale1986global,palmgren1989,molenberghs1994marginal}. 
This odds-ratio is given covariates as well as marginal covariates. 
The odds-ratio and marginals specify the joint bivariate distribution via
the so-called Placckett-distribution. 

One way of fitting this model is the ALR (alternating logistic regression) algorithm,
described in several papers (Kuk, 2004; Kuk, 2007; Qaqish, 2012).
We estimate the parameters in a two-stage procedure:

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

This gives efficient estimates of the dependence parameters due to
orthogonality, though some efficiency may be gained for the marginal parameters
by using the full likelihood or iterative fitting such as ALR.


The pairwise odds-ratio model is very useful, but one does not have a random 
effects model. 


Additive gamma model 
----------------------

Again we operate under  marginal logistic regression models are 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

First with just one random effect $Z$ we assume that  conditional
on $Z$ the responses are independent  and follow the model 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )  
\]
where $\Psi$ is the laplace transform of $Z$ where we assume that
$Z$ is gamma distributed with variance $\lambda_{\bullet}^{-1}$ and mean 1. 
In general $\Psi(\lambda_1,\lambda_2)$ is the laplace transform of a Gamma distributed random 
effect with $Z$ with mean $\lambda_1/\lambda_2$ and variance $\lambda_1/\lambda_2^2$.

We fit this model by 

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

To deal with multiple random effects we consider random effects 
$Z_i  i=1,...,d$   such that  $Z_i$ is gamma distributed with 
 * mean: $\lambda_j/\lambda_{\bullet}$ 
 * variance: $\lambda_j/\lambda_{\bullet}^2$, where we define the scalar $\lambda_{\bullet}$ below. 

Now given a cluster-specific design vector $V_{12}$ we assume that 
\[
V_{12}^T Z
\]
is gamma distributed with mean 1 and variance $\lambda_{\bullet}^{-1}$ 
such that critically the random effect variance is the same for all clusters.
That is 
\[
 \lambda_{\bullet} = V_{12}^T (\lambda_1,...,\lambda_d)^T 
\]
We return to some specific models below, and show how to fit the ACE and AE 
model using this set-up. 

One last option in the model-specification is to specify how the 
parameters $\lambda_1,...,\lambda_d$ are related. We thus can specify a 
matrix $M$ of dimension $p \times d$ such that 
\[
 (\lambda_1,...,\lambda_d)^T  = M \theta
\]
where $\theta$ is d-dimensional.  If $M$ is diagonal we have no 
restrictions on parameters. 

This parametrization is obtained with the var.par=0 option that thus estimates
$\theta$.

The DEFAULT parametrization instead estimates the variances of the random effects (var.par=1)
via the parameters $\nu$ 
\[
 M \nu = ( \lambda_1/\lambda_{\bullet}^2, ...,\lambda_d/\lambda_{\bullet}^2)^T
\]


The basic modelling assumption is now that given random effects 
$Z=(Z_1,...,Z_d)$ we have independent probabilites 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -V_{12,i}^T Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )   i=1,2
\]

We fit this model by 

 - Estimating the marginal parameters via GEE
 - Using marginal estimates, estimate dependence parameters

Even though the model not formally in this formulation allows negative 
correlation in practice the parameters can be negative and this reflects
negative correlation. An advantage is that no numerical integration is 
needed. 


The twin-stutter data
------------------------

We consider the twin-stutter where for pairs of twins that are 
either dizygotic or monozygotic we have recorded whether the twins
are stuttering \cite{twinstut-ref}

We here consider MZ and same sex DZ twins. 

Looking at the data 


``` r
library(mets)
data(twinstut)
twinstut$binstut <- 1*(twinstut$stutter=="yes")
twinsall <- twinstut
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
head(twinstut)
#>    tvparnr zyg stutter    sex age nr binstut
#> 1        1  mz      no female  71  1       0
#> 2        1  mz      no female  71  2       0
#> 3        2  dz      no female  71  1       0
#> 8        5  mz      no female  71  1       0
#> 9        5  mz      no female  71  2       0
#> 11       7  dz      no   male  71  1       0
twinstut <- subset(twinstut,tvparnr < 3000)
```


Pairwise odds ratio model 
-------------------------

We start by fitting an overall dependence OR for both MZ and DZ even though 
the dependence is expected to be different across zygosity.

The first step is to fit the marginal model adjusting for marginal covariates. 
We here note that there is a rather strong gender effect in the risk of
stuttering. 


``` r
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
summary(margbin)
#> 
#> Call:
#> glm(formula = binstut ~ factor(sex) + age, family = binomial(), 
#>     data = twinstut)
#> 
#> Coefficients:
#>                 Estimate Std. Error z value Pr(>|z|)    
#> (Intercept)      0.03033    2.16166   0.014   0.9888    
#> factor(sex)male  0.80288    0.20272   3.961 7.48e-05 ***
#> age             -0.05471    0.03295  -1.660   0.0968 .  
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> (Dispersion parameter for binomial family taken to be 1)
#> 
#>     Null deviance: 940.43  on 2650  degrees of freedom
#> Residual deviance: 921.01  on 2648  degrees of freedom
#> AIC: 927.01
#> 
#> Number of Fisher Scoring iterations: 6
```

Now estimating the OR parameter. We see a strong dependence with an OR
at around 8 that is clearly significant. 


``` r
bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
                       clusters=twinstut$tvparnr,detail=0)
summary(bina)
#> Dependence parameter for Odds-Ratio (Plackett) model 
#> With log-link
#> $estimates
#>                theta       se
#> dependence1 1.859712 0.481585
#> 
#> $or
#>             Estimate Std.Err   2.5% 97.5% P-value
#> dependence1    6.422   3.093 0.3603 12.48 0.03785
#> 
#> $type
#> [1] "plackett"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

Now, and more interestingly, we consider an OR that depends on zygosity and
note that MZ twins have a much larger OR than DZ twins. This type of result is
somewhat complex to interpret: one possibility is a genetic effect, another is a
stronger shared environmental effect for MZ twins.



``` r
# design for OR dependence 
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
bin <- binomial_twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bin)
#> Dependence parameter for Odds-Ratio (Plackett) model 
#> With log-link
#> $estimates
#>                   theta        se
#> factor(zyg)dz 0.1624942 1.0050473
#> factor(zyg)mz 3.7460315 0.7259771
#> 
#> $or
#>               Estimate Std.Err    2.5%   97.5% P-value
#> factor(zyg)dz    1.176   1.182  -1.141   3.494  0.3197
#> factor(zyg)mz   42.353  30.747 -17.910 102.616  0.1684
#> 
#> $type
#> [1] "plackett"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```


We now consider further regression modelling of the OR structure by
considering possible interactions between sex and zygosity.
We see that MZ has a much higher dependence and that males have
a much lower dependence. We tested for interaction in this model and 
these were not significant. 
     

``` r
twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+factor(sex),data=twinstut)
bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)
#> Dependence parameter for Odds-Ratio (Plackett) model 
#> With log-link
#> $estimates
#>                     theta       se
#> factor(zyg)dz    1.238547 1.118919
#> factor(zyg)mz    6.078248 1.536564
#> factor(sex)male -3.233772 1.776972
#> 
#> $or
#>                  Estimate   Std.Err       2.5%     97.5% P-value
#> factor(zyg)dz     3.45059   3.86094   -4.11670   11.0179  0.3715
#> factor(zyg)mz   436.26434 670.34797 -877.59353 1750.1222  0.5152
#> factor(sex)male   0.03941   0.07003   -0.09784    0.1767  0.5736
#> 
#> $type
#> [1] "plackett"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```


Bivariate Probit model 
------------------------


``` r
library(mets)
data(twinstut)
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
twinstut$binstut <- 1*(twinstut$stutter=="yes")
head(twinstut)
#>    tvparnr zyg stutter    sex age nr binstut
#> 1        1  mz      no female  71  1       0
#> 2        1  mz      no female  71  2       0
#> 3        2  dz      no female  71  1       0
#> 8        5  mz      no female  71  1       0
#> 9        5  mz      no female  71  2       0
#> 11       7  dz      no   male  71  1       0
twinstut <- subset(twinstut,tvparnr < 2000)
```

First testing for same dependence in MZ and DZ  that we recommend doing by 
comparing the correlations of MZ and DZ twins. Apart from regression 
correction in the mean this is an un-structured model, and the useful
concordance and casewise concordance estimates can be reported from this
analysis. 


``` r
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="un")
summary(b1)
#> 
#>               Estimate  Std.Err        Z   p-value    
#> (Intercept)   -2.04432  0.10246 -19.9532 < 2.2e-16 ***
#> sexmale        0.41203  0.12392   3.3249 0.0008844 ***
#> atanh(rho) MZ  1.11748  0.43188   2.5875 0.0096683 ** 
#> atanh(rho) DZ  0.23000  0.25889   0.8884 0.3743295    
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#>  Total MZ/DZ Complete pairs MZ/DZ
#>  583/1192    212/354             
#> 
#>                            Estimate 2.5%     97.5%   
#> Tetrachoric correlation MZ  0.80669  0.26456  0.96139
#> Tetrachoric correlation DZ  0.22603 -0.27052  0.62759
#> 
#> MZ:
#>                      Estimate 2.5%     97.5%   
#> Concordance           0.00886  0.00272  0.02840
#> Casewise Concordance  0.43282  0.13214  0.79272
#> Marginal              0.02046  0.01258  0.03312
#> Rel.Recur.Risk       21.15321  2.70681 39.59960
#> log(OR)               4.15336  1.96005  6.34666
#> DZ:
#>                      Estimate 2.5%     97.5%   
#> Concordance           0.00128  0.00013  0.01240
#> Casewise Concordance  0.06251  0.00705  0.38497
#> Marginal              0.02046  0.01258  0.03312
#> Rel.Recur.Risk        3.05481 -3.12067  9.23029
#> log(OR)               1.20535 -1.09090  3.50161
#> 
#>                          Estimate 2.5% 97.5%
#> Broad-sense heritability   1      NaN  NaN
```


Polygenic modelling 
-------------------

   We now turn attention to specific polygenic modelling where special random 
   effects are used to specify ACE, AE, ADE models and so forth. This is very
   easy with the bptwin function. The key parts of the output are the sizes of 
   the genetic component A and the environmental component, and we can compare 
   with the results of the unstructed model above. Also formally we can test 
   if this submodel is acceptable by a likelihood ratio test. 



``` r
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ace")
summary(b1)
#> 
#>              Estimate   Std.Err        Z p-value    
#> (Intercept)  -4.20958   1.55369  -2.7094 0.00674 ** 
#> sexmale       0.85990   0.36387   2.3632 0.01812 *  
#> log(var(A))   1.17018   0.99695   1.1738 0.24049    
#> log(var(C)) -23.53038   1.87673 -12.5380 < 2e-16 ***
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#>  Total MZ/DZ Complete pairs MZ/DZ
#>  583/1192    212/354             
#> 
#>                    Estimate 2.5%     97.5%   
#> A                   0.76318  0.41002  1.11633
#> C                   0.00000  0.00000  0.00000
#> E                   0.23682 -0.11633  0.58998
#> MZ Tetrachoric Cor  0.76318  0.15672  0.95170
#> DZ Tetrachoric Cor  0.38159  0.19280  0.54313
#> 
#> MZ:
#>                      Estimate 2.5%     97.5%   
#> Concordance           0.00768  0.00201  0.02893
#> Casewise Concordance  0.37943  0.09711  0.77658
#> Marginal              0.02025  0.01241  0.03289
#> Rel.Recur.Risk       18.73516 -0.13385 37.60417
#> log(OR)               3.85127  1.58182  6.12072
#> DZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.00230  0.00077 0.00688
#> Casewise Concordance 0.11369  0.05266 0.22841
#> Marginal             0.02025  0.01241 0.03289
#> Rel.Recur.Risk       5.61371  2.19257 9.03484
#> log(OR)              1.92764  1.16374 2.69154
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.76318  0.41002 1.11633
```



``` r
b0 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ae")
summary(b0)
#> 
#>             Estimate  Std.Err       Z p-value   
#> (Intercept) -4.20958  1.55369 -2.7094 0.00674 **
#> sexmale      0.85990  0.36387  2.3632 0.01812 * 
#> log(var(A))  1.17018  0.99695  1.1738 0.24049   
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#>  Total MZ/DZ Complete pairs MZ/DZ
#>  583/1192    212/354             
#> 
#>                    Estimate 2.5%     97.5%   
#> A                   0.76318  0.41002  1.11633
#> E                   0.23682 -0.11633  0.58998
#> MZ Tetrachoric Cor  0.76318  0.15672  0.95170
#> DZ Tetrachoric Cor  0.38159  0.19280  0.54313
#> 
#> MZ:
#>                      Estimate 2.5%     97.5%   
#> Concordance           0.00768  0.00201  0.02893
#> Casewise Concordance  0.37943  0.09711  0.77658
#> Marginal              0.02025  0.01241  0.03289
#> Rel.Recur.Risk       18.73517 -0.13386 37.60419
#> log(OR)               3.85127  1.58182  6.12072
#> DZ:
#>                      Estimate 2.5%    97.5%  
#> Concordance          0.00230  0.00077 0.00688
#> Casewise Concordance 0.11369  0.05266 0.22841
#> Marginal             0.02025  0.01241 0.03289
#> Rel.Recur.Risk       5.61371  2.19257 9.03484
#> log(OR)              1.92764  1.16374 2.69154
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.76318  0.41002 1.11633
```

Additive gamma random effects 
-----------------------------

Fitting first a model with different size random effects for MZ and DZ. We 
note that as before in the OR and biprobit model the dependence is much
stronger for MZ twins. We also test if these are the same by parametrizing the
OR model with an intercept. This clearly shows a significant difference. 



``` r
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial_twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects 
#> With log-link
#> $estimates
#>                     theta        se
#> factor(zyg)dz -2.13348460 1.0821489
#> factor(zyg)mz -0.07740026 0.5790839
#> 
#> $vargam
#>               Estimate Std.Err    2.5%  97.5% P-value
#> factor(zyg)dz   0.1184  0.1282 -0.1327 0.3696 0.35544
#> factor(zyg)mz   0.9255  0.5360 -0.1249 1.9760 0.08419
#> 
#> $type
#> [1] "gamma"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"

# test for same dependence in MZ and DZ 
theta.des <- model.matrix( ~factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial_twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects 
#> With log-link
#> $estimates
#>                   theta       se
#> (Intercept)   -2.133485 1.082149
#> factor(zyg)mz  2.056084 1.227348
#> 
#> $vargam
#>               Estimate Std.Err     2.5%   97.5% P-value
#> (Intercept)     0.1184  0.1282  -0.1327  0.3696  0.3554
#> factor(zyg)mz   7.8153  9.5921 -10.9849 26.6155  0.4152
#> 
#> $type
#> [1] "gamma"
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

Polygenic modelling 
-----------------------

   First setting up the random effects design and
   the relationship between variance parameters.
   We see that the genetic random effect has size 1 for MZ and 0.5 for DZ subjects,
   who have shared and non-shared genetic components with variance 0.5 such that the total
   genetic variance is the same for all subjects. The shared environmental effect is the same for
   all. Thus two parameters with these constraints.


``` r
out <- twin_polygen_design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ace")
head(cbind(out$des.rv,twinstut$tvparnr),10)
#>    MZ DZ DZns1 DZns2 env   
#> 1   1  0     0     0   1  1
#> 2   1  0     0     0   1  1
#> 3   0  1     1     0   1  2
#> 8   1  0     0     0   1  5
#> 9   1  0     0     0   1  5
#> 11  0  1     1     0   1  7
#> 12  0  1     1     0   1  8
#> 13  0  1     0     1   1  8
#> 15  0  1     1     0   1 10
#> 18  0  1     1     0   1 12
out$pardes
#>      [,1] [,2]
#> [1,]  1.0    0
#> [2,]  0.5    0
#> [3,]  0.5    0
#> [4,]  0.5    0
#> [5,]  0.0    1
```


Now, fitting the ACE model, we see that the variance of the genetic, 
component, is 1.5 and the environmental variance is -0.5. Thus suggesting that 
the ACE model does not fit the data.  When the random design is given we 
automatically use the gamma fralty model. 


``` r
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin1 <- binomial_twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin1)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                  theta        se
#> dependence1  1.1700713 0.9350041
#> dependence2 -0.2552978 0.5910697
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err     2.5%  97.5% P-value
#> dependence1   1.2791  0.6039  0.09552 2.4626 0.03416
#> dependence2  -0.2791  0.6039 -1.46265 0.9045 0.64397
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err    2.5% 97.5% P-value
#> p1   0.9148  0.5352 -0.1343 1.964 0.08744
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```


For this model we estimate the concordance and casewise concordance as well 
as the marginal rates of stuttering for females. 


``` r
concordanceTwinACE(bintwin1,type="ace")
#> $MZ
#>                      Estimate  Std.Err     2.5%   97.5%   P-value
#> concordance          0.009738 0.004010 0.001878 0.01760 1.517e-02
#> casewise concordance 0.476141 0.190613 0.102547 0.84974 1.249e-02
#> marginal             0.020452 0.005083 0.010489 0.03041 5.733e-05
#> 
#> $DZ
#>                      Estimate  Std.Err       2.5%   97.5%   P-value
#> concordance          0.001301 0.001158 -0.0009681 0.00357 2.611e-01
#> casewise concordance 0.063604 0.057255 -0.0486137 0.17582 2.666e-01
#> marginal             0.020452 0.005083  0.0104894 0.03041 5.733e-05
```


The E component was not consistent with the fit of the data and we
now consider instead the AE model. 



``` r
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ae")

bintwin <- binomial_twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin)
#> Dependence parameter for Clayton-Oakes model
#> Variance of Gamma distributed random effects
#> $estimates
#>                 theta        se
#> dependence1 0.8485392 0.4941264
#> 
#> $type
#> [1] "clayton.oakes"
#> 
#> $h
#>             Estimate Std.Err 2.5% 97.5% P-value
#> dependence1        1       0    1     1       0
#> 
#> $vare
#> NULL
#> 
#> $vartot
#>    Estimate Std.Err    2.5% 97.5% P-value
#> p1   0.8485  0.4941 -0.1199 1.817 0.08593
#> 
#> attr(,"class")
#> [1] "summary.mets.twostage"
```

Again, the concordance can be computed: 


``` r
concordanceTwinACE(bintwin,type="ae")
#> $MZ
#>                      Estimate  Std.Err     2.5%   97.5%   P-value
#> concordance          0.009236 0.003778 0.001831 0.01664 1.450e-02
#> casewise concordance 0.451605 0.189131 0.080916 0.82229 1.695e-02
#> marginal             0.020452 0.005083 0.010489 0.03041 5.733e-05
#> 
#> $DZ
#>                      Estimate   Std.Err      2.5%    97.5%   P-value
#> concordance          0.001966 0.0007099 0.0005741 0.003357 5.628e-03
#> casewise concordance 0.096106 0.0196561 0.0575803 0.134631 1.012e-06
#> marginal             0.020452 0.0050831 0.0104894 0.030415 5.733e-05
```


SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         mvtnorm_1.3-7         
#> [13] numDeriv_2016.8-1.1    compiler_4.6.0         codetools_0.2-20      
#> [16] Rcpp_1.1.1-1.1         ucminf_1.2.3           future_1.70.0         
#> [19] lattice_0.22-9         digest_0.6.39          parallelly_1.47.0     
#> [22] parallel_4.6.0         splines_4.6.0          Matrix_1.7-5          
#> [25] tools_4.6.0            RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Haplotype Discrete Survival Models"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Haplotype Discrete Survival Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```


Haplotype Analysis for discrete TTP
===================================

Cycle-specific logistic regression of haplotype effects with
known haplotype probabilities. Given observed genotype $G$ and
unobserved haplotypes $H$, we marginalise over the possible
haplotypes using $P(H|G)$ supplied as input.

\begin{align*}
   S(t|x,G) & = E( S(t|x,H) | G)  = \sum_{h \in G} P(h|G) S(t|z,h)
\end{align*}
so survival can be computed by marginalising over possible $h$ given $g$.

Survival is based on logistic regression for the discrete hazard
function of the form
\begin{align*}
      \mbox{logit}(P(T=t \mid T \geq t, x, h)) & = \alpha_t + x(h)^T \beta
\end{align*}
where $x(h)$ is a regression design of $x$ and haplotypes $h=(h_1,h_2)$.

Simple binomial data can also be fitted using this function.

Standard errors are computed assuming that the haplotype probabilities are known.

	
We are particularly interested in the types haplotypes: 
```{r}
types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")

## some haplotypes frequencies for simulations 
data(haplo)
hapfreqs <- haplo$hapfreqs 
print(hapfreqs)
```

Among the haplotypes of interest we look up the frequencies and choose a baseline:

```{r}
www <-which(hapfreqs$haplotype %in% types)
hapfreqs$freq[www]

baseline=hapfreqs$haplotype[9]
baseline
```


We have cycle-specific data with $id$ and outcome $y$:

```{r}
haploX  <- haplo$haploX
dlist(haploX,.~id|id %in% c(1,4,7))
```
and a list of possible haplotypes for each id and their probabilities $p$
(the probabilities within each id sum to 1):

```{r}
ghaplos <- haplo$ghaplos
head(ghaplos)
```

The first id=1 has the haplotype fully observed, but id=2 has two possible haplotypes consistent with the
observed genotype for this id, the probabilities are 7\% and 93\%, respectively. 

With the baseline given above we can specify a regression design that gives an effect if a haplotype of the specified type
is present (`sm=0`), or an additive effect of haplotypes (`sm=1`):

```{r}
designftypes <- function(x,sm=0) {
hap1=x[1]
hap2=x[2]
if (sm==0) y <- 1*( (hap1==types) | (hap2==types))
if (sm==1) y <- 1*(hap1==types) + 1*(hap2==types)
return(y)
}

```

To fit the model we first construct a time design matrix (named `X`)
and supply the haplotype distributions for each id:

```{r}
haploX$time <- haploX$times
Xdes <- model.matrix(~factor(time),haploX)
colnames(Xdes) <- paste("X",1:ncol(Xdes),sep="")
X <- dkeep(haploX,~id+y+time)
X <- cbind(X,Xdes)
Haplos <- dkeep(ghaplos,~id+"haplo*"+p)
desnames=paste("X",1:6,sep="")   # six X's related to 6 cycles 
head(X)

```

We can now fit the model with the design given by the design function:
```{r}
out <- haplo_surv_discrete(X=X,y="y",time.name="time",
      Haplos=Haplos,desnames=desnames,designfunc=designftypes) 
names(out$coef) <- c(desnames,types)
out$coef
summary(out)
```

Haplotypes `"DCGCGCTCACG"` and `"DTCCGCTGACG"` give an increased hazard of pregnancy.

The data were generated with these true coefficients:

```{r}
tcoef=c(-1.93110204,-0.47531630,-0.04118204,-1.57872602,-0.22176426,-0.13836416,
0.88830288,0.60756224,0.39802821,0.32706859)

cbind(out$coef,tcoef)
```

The fitted design matrix can be found in the output:
```{r}
head(out$X,10)
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Analysis of bivariate binomial data: Twin analysis"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Analysis of bivariate binomial data: Twin analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Overview 
==========

When analysing bivariate binomial data with the aim of learning about the
dependence present, possibly after adjusting for covariates, many
models are available.

   *  Random-effects logistic regression models covered elsewhere (`glmer` in `lme4`).

In the mets package you can fit the

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.


Typically it can be hard or impossible
to specify random effects models with special
structure among the parameters of the random effects. This is possible with
our models.

To be concrete about the model structure, assume that we have paired binomial
data $(Y_1, Y_2, X_1, X_2)$ where the responses are $Y_1, Y_2$ and the
covariates are $X_1, X_2$.

We start by giving a brief description of these different models.  First we
for bivariate data one can specify the marginal probability using logistic 
regression models 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2.
\]
These models can be estimated under working independence (Zeger and Liang, 1986).

A typical twin analysis will consist of looking at both

   *  Pairwise odds ratio model

   *  Bivariate Probit model
  
The additive gamma can be used for the same as the bivariate probit model but 
is more restrictive in terms of dependence structure, but is nevertheless 
still valuable to have also as a check of results of the bivariate probit
model. 


Biprobit with random effects
=============================

For these models we assume that given random effects $Z$ and a covariate vector
$V_{12}$ we have independent probit regression models
\[
\mbox{probit}(P(Y_i=1|X_i, Z)) = \alpha_i + X_i^T \beta + V_{12}^T Z,  \quad i=1,2.
\]
where $Z$ follows a bivariate normal distribution with covariance
$\Sigma$. The general covariance structure
$\Sigma$ makes the model very flexible.

We note that

 - Parameters $\beta$ are subject-specific
 - $\Sigma$ reflects the dependence structure

The more standard $\mbox{logit}$ link rather than the $\mbox{probit}$ link
is often used and implemented in, for example, Tutz and Hennevogl (1996). The advantage is that one obtains an odds-ratio interpretation
of the subject-specific effects, but numerical integration is then needed to
fit the model.

Pairwise odds ratio model 
------------------------


The pairwise odds ratio model specifies that given $X_1, X_2$
the marginal models are
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

The primary object of interest are the odds ratio between $Y_{1}$ and $Y_{2}$
\[
\gamma_{12} = \frac{ P(  Y_{ki} =1 , Y_{kj} =1) P(  Y_{ki} =0 , Y_{kj} =0) }{ 
  P(  Y_{ki} =1 , Y_{kj} =0) P(  Y_{ki} =0 , Y_{kj} =1) }
\]
given $X_{ki}$, $X_{kj}$, and $Z_{kji}$. 

We model the odds ratio with the regression 
\[
\gamma_{12} = \exp( Z_{12}^T \lambda)
\]
Where $Z_{12}$ are covariates that may influence the odds ratio
between $Y_{1}$ and $Y_{2}$ and contains the marginal covariates
\cite{carey-1993,dale1986global,palmgren1989,molenberghs1994marginal}. 
This odds-ratio is given covariates as well as marginal covariates. 
The odds-ratio and marginals specify the joint bivariate distribution via
the so-called Placckett-distribution. 

One way of fitting this model is the ALR (alternating logistic regression) algorithm,
described in several papers (Kuk, 2004; Kuk, 2007; Qaqish, 2012).
We estimate the parameters in a two-stage procedure:

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

This gives efficient estimates of the dependence parameters due to
orthogonality, though some efficiency may be gained for the marginal parameters
by using the full likelihood or iterative fitting such as ALR.


The pairwise odds-ratio model is very useful, but one does not have a random 
effects model. 


Additive gamma model 
----------------------

Again we operate under  marginal logistic regression models are 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

First with just one random effect $Z$ we assume that  conditional
on $Z$ the responses are independent  and follow the model 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )  
\]
where $\Psi$ is the laplace transform of $Z$ where we assume that
$Z$ is gamma distributed with variance $\lambda_{\bullet}^{-1}$ and mean 1. 
In general $\Psi(\lambda_1,\lambda_2)$ is the laplace transform of a Gamma distributed random 
effect with $Z$ with mean $\lambda_1/\lambda_2$ and variance $\lambda_1/\lambda_2^2$.

We fit this model by 

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

To deal with multiple random effects we consider random effects 
$Z_i  i=1,...,d$   such that  $Z_i$ is gamma distributed with 
 * mean: $\lambda_j/\lambda_{\bullet}$ 
 * variance: $\lambda_j/\lambda_{\bullet}^2$, where we define the scalar $\lambda_{\bullet}$ below. 

Now given a cluster-specific design vector $V_{12}$ we assume that 
\[
V_{12}^T Z
\]
is gamma distributed with mean 1 and variance $\lambda_{\bullet}^{-1}$ 
such that critically the random effect variance is the same for all clusters.
That is 
\[
 \lambda_{\bullet} = V_{12}^T (\lambda_1,...,\lambda_d)^T 
\]
We return to some specific models below, and show how to fit the ACE and AE 
model using this set-up. 

One last option in the model-specification is to specify how the 
parameters $\lambda_1,...,\lambda_d$ are related. We thus can specify a 
matrix $M$ of dimension $p \times d$ such that 
\[
 (\lambda_1,...,\lambda_d)^T  = M \theta
\]
where $\theta$ is d-dimensional.  If $M$ is diagonal we have no 
restrictions on parameters. 

This parametrization is obtained with the var.par=0 option that thus estimates
$\theta$.

The DEFAULT parametrization instead estimates the variances of the random effects (var.par=1)
via the parameters $\nu$ 
\[
 M \nu = ( \lambda_1/\lambda_{\bullet}^2, ...,\lambda_d/\lambda_{\bullet}^2)^T
\]


The basic modelling assumption is now that given random effects 
$Z=(Z_1,...,Z_d)$ we have independent probabilites 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -V_{12,i}^T Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )   i=1,2
\]

We fit this model by 

 - Estimating the marginal parameters via GEE
 - Using marginal estimates, estimate dependence parameters

Even though the model not formally in this formulation allows negative 
correlation in practice the parameters can be negative and this reflects
negative correlation. An advantage is that no numerical integration is 
needed. 


The twin-stutter data
------------------------

We consider the twin-stutter where for pairs of twins that are 
either dizygotic or monozygotic we have recorded whether the twins
are stuttering \cite{twinstut-ref}

We here consider MZ and same sex DZ twins. 

Looking at the data 

```{r}
library(mets)
data(twinstut)
twinstut$binstut <- 1*(twinstut$stutter=="yes")
twinsall <- twinstut
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
head(twinstut)
twinstut <- subset(twinstut,tvparnr < 3000)
```


Pairwise odds ratio model 
-------------------------

We start by fitting an overall dependence OR for both MZ and DZ even though 
the dependence is expected to be different across zygosity.

The first step is to fit the marginal model adjusting for marginal covariates. 
We here note that there is a rather strong gender effect in the risk of
stuttering. 

```{r}
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
summary(margbin)
```

Now estimating the OR parameter. We see a strong dependence with an OR
at around 8 that is clearly significant. 

```{r twostage1}
bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
                       clusters=twinstut$tvparnr,detail=0)
summary(bina)
```

Now, and more interestingly, we consider an OR that depends on zygosity and
note that MZ twins have a much larger OR than DZ twins. This type of result is
somewhat complex to interpret: one possibility is a genetic effect, another is a
stronger shared environmental effect for MZ twins.


```{r twostage2}
# design for OR dependence 
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
bin <- binomial_twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bin)
```


We now consider further regression modelling of the OR structure by
considering possible interactions between sex and zygosity.
We see that MZ has a much higher dependence and that males have
a much lower dependence. We tested for interaction in this model and 
these were not significant. 
     
```{r twostage3}
twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+factor(sex),data=twinstut)
bina <- binomial_twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)
```


Bivariate Probit model 
------------------------

```{r}
library(mets)
data(twinstut)
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
twinstut$binstut <- 1*(twinstut$stutter=="yes")
head(twinstut)
twinstut <- subset(twinstut,tvparnr < 2000)
```

First testing for same dependence in MZ and DZ  that we recommend doing by 
comparing the correlations of MZ and DZ twins. Apart from regression 
correction in the mean this is an un-structured model, and the useful
concordance and casewise concordance estimates can be reported from this
analysis. 

```{r biprobit1}
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="un")
summary(b1)
```


Polygenic modelling 
-------------------

   We now turn attention to specific polygenic modelling where special random 
   effects are used to specify ACE, AE, ADE models and so forth. This is very
   easy with the bptwin function. The key parts of the output are the sizes of 
   the genetic component A and the environmental component, and we can compare 
   with the results of the unstructed model above. Also formally we can test 
   if this submodel is acceptable by a likelihood ratio test. 


```{r bptwin1}
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ace")
summary(b1)
```


```{r bptwin2}
b0 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ae")
summary(b0)
```

Additive gamma random effects 
-----------------------------

Fitting first a model with different size random effects for MZ and DZ. We 
note that as before in the OR and biprobit model the dependence is much
stronger for MZ twins. We also test if these are the same by parametrizing the
OR model with an intercept. This clearly shows a significant difference. 


```{r addgamma1}
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial_twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)

# test for same dependence in MZ and DZ 
theta.des <- model.matrix( ~factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial_twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)
```

Polygenic modelling 
-----------------------

   First setting up the random effects design and
   the relationship between variance parameters.
   We see that the genetic random effect has size 1 for MZ and 0.5 for DZ subjects,
   who have shared and non-shared genetic components with variance 0.5 such that the total
   genetic variance is the same for all subjects. The shared environmental effect is the same for
   all. Thus two parameters with these constraints.

```{r polygenic1}
out <- twin_polygen_design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ace")
head(cbind(out$des.rv,twinstut$tvparnr),10)
out$pardes
```


Now, fitting the ACE model, we see that the variance of the genetic, 
component, is 1.5 and the environmental variance is -0.5. Thus suggesting that 
the ACE model does not fit the data.  When the random design is given we 
automatically use the gamma fralty model. 

```{r polygenic2}
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin1 <- binomial_twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin1)
```


For this model we estimate the concordance and casewise concordance as well 
as the marginal rates of stuttering for females. 

```{r}
concordanceTwinACE(bintwin1,type="ace")
```


The E component was not consistent with the fit of the data and we
now consider instead the AE model. 


```{r polygenic_ae}
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ae")

bintwin <- binomial_twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin)
```

Again, the concordance can be computed: 

```{r}
concordanceTwinACE(bintwin,type="ae")
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Mediation Analysis for survival data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Mediation Analysis for survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="png",
  comment = "#>"  
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview 
========

Fit 

 * binomial-regression IPCW, binreg 
 * additive Lin-Ying model, aalenMets
 * cox model phreg 
 * standard logistic regression via binreg

in the context of mediation analysis using mediation weights as in the medFlex package. 
We thus fit natural effects models, that for example on the binary scale might state
that
\begin{align*}
\mbox{logit}(P(Y(x,M(x^*))=1| Z)  = \beta_0+ \beta_1 x + \beta_2 x^* + \beta_3^T Z,
\end{align*}
in this case the the Natural Direct Effect (NDE) for fixed covariates $Z$ is 
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NDE}} = \frac{\mbox{odds}(Y(1,M(x))|Z)}{\mbox{odds}(Y(0,M(x))|Z)}  = \exp(\beta_1),
\end{align*}
and the Natural Inderect Effect (NIE) for fixed covariates $Z$ is 
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NIE}} = \frac{\mbox{odds}(Y(x,M(1))|Z)}{\mbox{odds}(Y(x,M(0))|Z)} = \exp(\beta_2).
\end{align*}
See the medFlex package for additional discussion of the parametrization.

The mediator can  be

 * binomial using glm-binomial.
 * multnomial via the mlogit function of mets

Both mediator and exposure must be coded as factors.

In the below example these are

 * mediator: gp.f
 * exposure : dnr.f

and the outcome model is concerned with the risk/hazard of cause=2. 

The key is that the standard errors are computed using the i.i.d influence
functions and a Taylor expansion to deal with the uncertainty from the
mediation weights. 


Simulated Data 
==============

First we simulate some data that mimics that of Kumar et al 2012. 
This is data from multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR) Kumar et al (2012), "Trends in allogeneic stem cell transplantation for multiple myeloma: a CIBMTR
analysis".  The data used in this paper consist of patients transplanted from 1995 to
2005, and we compared the outcomes between transplant periods: 2001-2005 (N=488)
versus 1995-2000 (N=375). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1)) 
defined as death without relapse.
\cite{kumar-2012} considered the following risk covariates: 
transplant time period (gp (main interest of the study): 1 for transplanted in
2001-2005 versus 0 for transplanted in 1995-2000), donor type (dnr: 1 for Unrelated or
other related donor (N=280) versus 0 for HLA-identical sibling (N=584)), prior
autologous transplant (preauto: 1 for Auto+Allo transplant (N=399) versus 0 for
allogeneic transplant alone (N=465)) and time to transplant (ttt24: 1 for more than 24 months (N=289) versus 0 for less than or
equal to 24 months (N=575))). 

The interest is then on the effect of the period (gp) and the possible mediation
via the amount of unrealted or related donors (dnr). A somewhat artificial example ! 
All adjusted for other important counfounders. 

```{r}
 library(mets)
 runb <- 0
 options(warn=-1)
 set.seed(1000) # to control output in simulatins for p-values below.

n <- 200; k.boot <- 10; 

dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
          beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
    treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
dfactor(dat) <- dnr.f~dnr
dfactor(dat) <- gp.f~gp
drename(dat) <- ttt24~"ttt24*"
dat$id <- 1:n
dat$ftime <- 1
```


Mediation Weights 
=================

Then  compute the mediation weights  based on a mediation model

```{r}
weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
wdata <- medweight(fit,data=dat)
```

Binomial Regression 
===================

A simple multvariate regression of the probaibility of relapse at 50 months with both
exposure and mediator (given the other covariates)

```{r}
aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
summary(aaMss2)
```

Binomial regression IPCW Mediation Analysis 
===========================================

We first look at the probability of  relapse at 50 months

```{r, label=firstmodel}
### binomial regression ###########################################################
aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cause=2)
summary(aaMss)

ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```

So the  NDE is $1.40 (0.72,2.76)$ and the NIE is $1.32 (1.05,1.66)$. 

Mediation Analysis 
====================

We here also illustrate how to use the other models mentioned above. 

```{r, label=multiplemodels}
### lin-ying model ################################################################
aaMss <- aalenMets(Surv(time/100,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		   weights=wdata$weights)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### cox model ###############################################################################
aaMss <- phreg(Surv(time,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
	       weights=wdata$weights)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### Fine-Gray #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,propodds=NULL,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### logit model  #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### binomial outcome  ############################
aaMss <- binreg(Event(ftime,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cens.weights=1,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```


Multinomial regression
======================

Also works with mediator with more than two levels

 * meditor: wmi in 4 categories
 * exposure: age in 4 categories

```{r, label=multinom, cache=TRUE, eval=fullVignette}
data(tTRACE)
dcut(tTRACE) <- ~. 

weightmodel <- fit <- mlogit(wmicat.4 ~agecat.4+vf+chf,data=tTRACE,family=binomial)
wdata <- medweight(fit,data=tTRACE)

aaMss <- binreg(Event(time,status)~agecat.40+ agecat.41+ vf+chf+cluster(id),data=wdata,
		time=7,weights=wdata$weights,cause=9)
summary(aaMss)
MultMed <- mediatorSurv(aaMss,fit,data=tTRACE,wdata=wdata)
```

```{r  results="hide", echo=FALSE }
## To save time building the vignettes on CRAN, we cache time consuming computations
if (fullVignette) {
  MultMed[c('iid','iid.w','iid.surv')] <- NULL
  saveRDS(MultMed, "data/MultMed.rds")
} else {
  MultMed <- readRDS("data/MultMed.rds")
}
```

```{r}
summary(MultMed)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```


The `resmeanATE` or `resmeanIPCW`/`rmstIPCW` functions fit an exponential or linear link model with
IPCW adjustment at a specific time point for the restricted mean survival or years lost in
competing risks. The computation is linear in the data size, making it suitable for large datasets.
Influence functions are computed and available for downstream inference.

Key features:

  - censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument



RMST 
====

Regression for rmst outcome $E(T \wedge t | X) = exp(X^T \beta)$ based on IPCW adjustment for censoring, thus solving
the estimating equation 
\begin{align*}
 & X^T [ (T \wedge t) \frac{I(C > T \wedge t)}{G_C(T \wedge t,X)} - exp(X^T \beta) ] = 0 .
\end{align*}
This is done with the resmeanIPCW function. 
For a fully saturated model with a complete censoring model this is equal to the integral of the Kaplan-Meier estimator, as
illustrated below.

We can also compute the integral of the Kaplan-Meier or Cox-based survival estimator to
get the RMST (using the `resmean_phreg` function):
\[ \int_0^t S(s|X) ds. \]

For competing risks, the years lost can be computed via cumulative incidence functions (`cif_yearslost`)
or as an IPCW estimator, since
\[ E( I(\epsilon=1) ( t - T \wedge t ) | X) = \int_0^t F_1(s) ds. \]
For a fully saturated model with a complete censoring model these estimators are equivalent, as illustrated below.


```{r}
set.seed(101)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean_phreg(out1,times=30)
     summary(rm1)
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     ## same as integrated cumulative incidence 
     rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)
```

Based on the output from the IPCW estimators we can derive any measure of interest

```{r}
estimate(out)

measures <- function(p) {
 ratio1 <- p[1]/p[2]; ratio2 <- p[2]/p[1]; dif1 <- p[4]-p[1]; dif2 <- p[3]-p[1]
 m <- c(dif1,dif2,ratio1,ratio2)
 return(m)
}

 labs <- c("dif4-1","dif3-1","ratio 1/2","ratio 2/1")
 estimate(out,f=measures,labels=labs)
```

Average treatment effect
=========================

This section computes the average treatment effect for restricted mean survival and years lost in the competing risks setting.

```{r}
 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
		    treat.model=tcell~platelet)
 summary(out1)

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,time=40,
		    treat.model=tcell~platelet)
 summary(out2)
```



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Mediation Analysis for survival data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Mediation Analysis for survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="png",
  comment = "#>"  
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview 
========

We fit

 * binomial regression with IPCW: `binreg`
 * additive Lin-Ying model: `aalenMets`
 * Cox model: `phreg`
 * standard logistic regression via `binreg`

in the context of mediation analysis using mediation weights as in the `medFlex` package.
We thus fit natural effects models; for example, on the binary scale:
\begin{align*}
\mbox{logit}(P(Y(x,M(x^*))=1| Z)  = \beta_0+ \beta_1 x + \beta_2 x^* + \beta_3^T Z,
\end{align*}
In this case the Natural Direct Effect (NDE) for fixed covariates $Z$ is
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NDE}} = \frac{\mbox{odds}(Y(1,M(x))|Z)}{\mbox{odds}(Y(0,M(x))|Z)}  = \exp(\beta_1),
\end{align*}
and the Natural Indirect Effect (NIE) for fixed covariates $Z$ is
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NIE}} = \frac{\mbox{odds}(Y(x,M(1))|Z)}{\mbox{odds}(Y(x,M(0))|Z)} = \exp(\beta_2).
\end{align*}
See the `medFlex` package for additional discussion of the parametrisation.

The mediator can be:

 * binomial, using `glm` with family `binomial`.
 * multinomial, via the `mlogit` function in `mets`.

Both mediator and exposure must be coded as factors.

In the example below:

 * mediator: `gp.f`
 * exposure: `dnr.f`

The outcome model concerns the risk/hazard of cause 2.

Standard errors are computed using i.i.d. influence functions and a Taylor expansion to account for uncertainty in the
mediation weights.


Simulated Data 
==============

First we simulate data that mimics the dataset from Kumar et al. (2012),
which consists of multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR). The data cover patients transplanted from 1995 to
2005, comparing outcomes between transplant periods: 2001--2005 ($N=488$)
versus 1995--2000 ($N=375$). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1),
defined as death without relapse.
Kumar et al. (2012) considered the following risk covariates:
transplant period (`gp`, the main variable of interest: 1 for 2001--2005, 0 for 1995--2000),
donor type (`dnr`: 1 for unrelated or other related donor ($N=280$), 0 for HLA-identical sibling ($N=584$)),
prior autologous transplant (`preauto`: 1 for auto+allo transplant ($N=399$), 0 for allogeneic alone ($N=465$)),
and time to transplant (`ttt24`: 1 for more than 24 months ($N=289$), 0 for 24 months or less ($N=575$)).

The interest is in the effect of transplant period (`gp`) and possible mediation
via the proportion of unrelated or related donors (`dnr`) — a somewhat artificial example.
All analyses are adjusted for other important confounders.

```{r}
 library(mets)
 runb <- 0
 options(warn=-1)
 set.seed(1000) # to control output in simulations for p-values below.

n <- 200; k.boot <- 10; 

dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
          beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
    treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
dfactor(dat) <- dnr.f~dnr
dfactor(dat) <- gp.f~gp
drename(dat) <- ttt24~"ttt24*"
dat$id <- 1:n
dat$ftime <- 1
```


Mediation Weights
=================

We compute the mediation weights based on a model for the mediator:

```{r}
weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
wdata <- medweight(fit,data=dat)
```

Binomial Regression 
===================

A simple multivariate regression of the probability of relapse at 50 months with both
exposure and mediator (given the other covariates):

```{r}
aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
summary(aaMss2)
```

Binomial regression IPCW Mediation Analysis
===========================================

We first look at the probability of relapse at 50 months:

```{r, label=firstmodel}
### binomial regression ###########################################################
aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cause=2)
summary(aaMss)

ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```

The estimated NDE is $1.40$ (95% CI: $0.72$, $2.76$) and the NIE is $1.32$ (95% CI: $1.05$, $1.66$).

Mediation Analysis
====================

We illustrate how to use the other models mentioned above.

```{r, label=multiplemodels}
### lin-ying model ################################################################
aaMss <- aalenMets(Surv(time/100,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		   weights=wdata$weights)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### cox model ###############################################################################
aaMss <- phreg(Surv(time,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
	       weights=wdata$weights)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### Fine-Gray #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,propodds=NULL,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### logit model  #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### binomial outcome  ############################
aaMss <- binreg(Event(ftime,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cens.weights=1,cause=2)
summary(aaMss)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```


Multinomial regression
======================

Also works with a mediator with more than two levels:

 * mediator: `wmi` in 4 categories
 * exposure: `age` in 4 categories

```{r, label=multinom, cache=TRUE, eval=fullVignette}
library(mets)
data(tTRACE)
dcut(tTRACE) <- ~. 

weightmodel <- fit <- mlogit(wmicat.4 ~agecat.4+vf+chf,data=tTRACE,family=binomial)
wdata <- medweight(fit,data=tTRACE)

aaMss <- binreg(Event(time,status)~agecat.40+ agecat.41+ vf+chf+cluster(id),data=wdata,
		time=7,weights=wdata$weights,cause=9)
summary(aaMss)
MultMed <- mediatorSurv(aaMss,fit,data=tTRACE,wdata=wdata)
summary(MultMed)
```

```{r  results="hide", echo=FALSE }
## To save time building the vignettes on CRAN, we cache time consuming computations
if (fullVignette) {
  MultMed[c('iid','iid.w','iid.surv')] <- NULL
  saveRDS(MultMed, "data/MultMed.rds")
} else {
  MultMed <- readRDS("data/MultMed.rds")
}
```

```{r}
summary(MultMed)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Marginal modelling of clustered survival data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Marginal modelling of clustered survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="png",
  comment = "#>"  
)
library(mets)
```

Overview 
========

A basic component for our modelling of multivariate survival data is that many models are 
build around marginals that on Cox form. The marginal Cox model can be fitted efficiently
in the mets package, in particular the handling of strata and robust standard errors is 
optimized. 

The basic models assumes that each subject has a marginal on Cox-form
$$ 
\lambda_{g(k,i)}(t) \exp( X_{ki}^T \beta).
$$ 
where $g(k,i)$ gives the strata for the subject. 

We here discuss and show how to get robust standard errors of  

* the regression parameters

* the baseline 

and how to do goodness of fit test using 

* cumulative residuals score test 

First we generate some data from the Clayton-Oakes model, with $5$ members
in each cluster and a variance parameter at $2$  


```{r}
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in simulatins for p-values below.
 n <- 1000
 k <- 5
 theta <- 2
 data <- simClaytonOakes(n,k,theta,0.3,3)
```

The data is on has one subject per row.

* time : time of event 
* status : 1 for event and 0 for censoring 
* x : x is a binary covariate 
* cluster : cluster 


Now we fit the model and produce robust standard errors for both 
regression parameters and baseline.

First, recall that the baseline for strata $g$ is asymptotically equivalent to 
\begin{align}
\hat A_g(t) - A_g(t)  & = \sum_{k \in g} \left( \sum_{i: ki \in g} \int_0^t \frac{1}{S_{0,g}} dM_{ki}^g  - P^g(t) \beta_k \right) 
\end{align}
with $P^g(t) = \int_0^t E_g(s) d \hat \Lambda_g(s)$ the derivative of $\int_0^t 1/S_{0,g}(s) dN_{\cdot g}$
wrt to $\beta$, and 
\begin{align}
\hat \beta  - \beta  & = I(\tau)^{-1} \sum_k ( \sum_i \int_0^\tau (Z_{ki} - E_{g}) dM_{ki}^g ) =  \sum_k \beta_{k} 
\end{align}
with 
\begin{align}
 M_{ki}^g(t) &  = N_{ki}(t) - \int_0^t Y_{ki}(s) \exp( Z_{ki} \beta) d \Lambda_{g(k,i)}(t), \\
 \beta_{k} & =  I(\tau)^{-1} \sum_i \int_0^\tau (Z_{ki} - E_{g}) dM_{ki}^g 
\end{align}
the basic 0-mean processes, that are martingales in the iid setting,
and $I(t)$ is the derivative of the total score, $\hat U(t,\beta))$, 
with respect to $\beta$ evaluated at time $t$. 
       
The variance of the baseline of strata g  is estimated by 
\begin{align}
\sum_{k \in g} ( \sum_{i: ki \in g} \int_0^t \frac{1}{S_{0,g(k,i)}} d\hat M_{ki}^g - P^g(t) \beta_k )^2
\end{align}
that can be computed using the particular structure of 
\begin{align}
d \hat M_{ik}^g(t) & =  dN_{ik}(t) -  \frac{1}{S_{0,g(i,k)}} \exp(Z_{ik} \beta) dN_{g.}(t) 
\end{align}

This robust variance of the baseline and the iid decomposition for $\beta$ is
computed in mets as: 
```{r}
   out <- phreg(Surv(time,status)~x+cluster(cluster),data=data)
   summary(out)
   # robust standard errors attached to output
   rob <- robust.phreg(out)
```

We can get the iid decomposition of the $\hat \beta - \beta$ by 

```{r}
   # making iid decomposition of regression parameters
   betaiid <- iid(out)
   head(betaiid)
   # robust standard errors
   crossprod(betaiid)^.5
   # same as 
```

We now look at the plot  with robust standard errors 


```{r}
  bplot(rob,se=TRUE,robust=TRUE,col=3)
```

We can also make survival prediction with robust standard errors using the phreg.

```{r}
  pp <-  predict(out,data[1:20,],se=TRUE,robust=TRUE)
  plot(pp,se=TRUE,whichx=1:10)
```


Finally, just to check that we can recover the model we also estimate the dependence parameter 

```{r}
tt <- twostageMLE(out,data=data)
summary(tt)
```

Goodness of fit 
===============

The observed score process is given by 
\begin{align}
U(t,\hat \beta) & = \sum_k \sum_i \int_0^t (Z_{ki} - \hat E_g ) d \hat M_{ki}^g 
\end{align}
where $g$ is strata $g(k,i)$. The observed score has the iid decomposition 
\begin{align}
\hat U(t) = \sum_k \sum_i  \int_0^t (Z_{ki} - E_g) dM_{ki}^g  - 
	I(t)  \sum_k \beta_k 
\end{align}
where $\beta_k$ is the iid decomposition of the score process for the true $\beta$
\begin{align}
\beta_k  & =  I(\tau)^{-1} \sum_i \int_0^\tau (Z_{ki} - E_g ) d  M_{ki}^g 
\end{align}
and $I(t)$ is the derivative of the total score, $\hat U(t,\beta))$, 
with respect to $\beta$ evaluated at time $t$. 

This observed score can be resampled given it is on iid form in terms of 
clusters. 

Now using the cumulative score process for checking proportional 
hazards 
```{r}
gout <- gof(out)
gout
```

The p-value reflects wheter the observed score process is consistent with
the model. 

```{r}
  plot(gout)
```

Computational aspects
--------------------

The score processes can be resampled as in Lin, Wei, Ying (1993) using the martingale structure, such that the observed
score process is resampled by 
\begin{align}
  \sum_k \sum_i \int_0^t g_{ki} (Z_{ki} - E_g) dN_{ki}  - I(t)  I^{-1}(\tau) g_{ki} \int_0^{\tau} (Z_{ki} - E_g) dN_{ki} .
\end{align}
where $g_{ki}$ are i.i.d. standard normals. 


Based on the zero mean processes we more generally with clusters can 
resample the score process.
For resampling of score process we need
\begin{align}
U(t,\beta) & = \sum_k \sum_i g_k \int_0^t (Z_{ki} - E_g ) dM_{ki}^g 
\end{align}
where $g$ is strata. We write $g_k$ as $g_{ki}$ and thus repeating $g_k$ within each cluster.


Computations are done using that 
\begin{align*}
\int_0^t (Z_{ki} - E_{g}) dM_{ki}^g & = \int_0^t (Z_{ki} - E_{g}) dN_{ki}^g - \int_0^{t} (Z_{ki} - E_{g})  Y_{ki}(u) d\Lambda^g(u)  
\end{align*}
therefore and summing the compensator part with the $g_{ki}$ multipliers then gives for each strata $g$
\begin{align*}
   &  \int_0^t \frac{S_{1g}^w(u)}{S_{0g}(u)} dN_{g.}(v) -  \int_0^t E_{g}(u) \frac{S_{0g}^w(u)}{S_{0g}(u)} dN_{g.}(v)
\end{align*}
with  
\begin{align*}
S_{jg}^w(t) & =  \sum_{ki \in g} \exp(Z_{ki} \beta) Z_{ki}^j Y_{ki}(t) g_{ki}  \\
S_{jg}(t) & =  \sum_{ki \in g} \exp(Z_{ki} \beta) Z_{ki}^j Y_{ki}(t).
\end{align*}



Cluster stratified Cox models
==============================

For  clustered data it is possible to estimate the regression coefficient
within clusters by using Cox's partial likelihood stratified on clusters.

Note, here that the data is generated with a different subject specific structure, 
so we will not recover the $\beta$ at 0.3 and the model will not be
a proportional Cox model, we we would also expect to reject "proportionality" with
the gof-test. 

The model can be thought of as 
\[ 
\lambda_{g(k,i)} (t) \exp( X_{ki}^T \beta)
\] 
where $\lambda_g(t)$ is some cluster specific baseline. 

The regression coefficient $\beta$ can be estimated by using the 
partial likelihood for clusters. 

```{r}
 out <- phreg(Surv(time,status)~x+strata(cluster),data=data)
 summary(out)
```

SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Randomization for Cox Type rate models "
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Randomization for Cox Type rate models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Two-Stage Randomization for counting process outcomes 
========================================================

Specify rate models of $N_1(t)$. 

   - survival data
   - competing risks, cause specific hazards. 
   - recurrent events data


Under simple randomization we can estimate the rate Cox model  

  - \(  \lambda_0(t) \exp(A_0 \beta_0) \) 

Under two-stage randomization we can estimate the rate Cox model 

  - \( \lambda_0(t) \exp(A_0 \beta_0 + A_1(t) \beta_1 ) \) 


Starting point is that Cox's partial likelihood score can be used for estimating parameters
 \begin{align*}
   U(\beta) & = \int (A(t) - e(t)) dN_1(t)
 \end{align*}
 where $A(t)$ is the combined treatments over time. 

  - the solution will converge to $\beta^*$ the Struthers-Kalbfleisch solution of the score and will have robust standard errors Lin-Wei. 

 The estimator can be agumented in different ways using additional covariates at the time of randomization and a censoring augmentation. 
 The solved estimating eqution is 
 \begin{align*}
 \sum_i   U_i - AUG_0 - AUG_1 + AUG_C   = 0 
 \end{align*}
 using the covariates from augmentR0 to augment with 
 \begin{align*}
 AUG_0 = ( A_0 - \pi_0(X_0) ) X_0 \gamma_0
 \end{align*}
 where possibly $P(A_0=1|X_0)=\pi_0(X_0)$ but does not depend on covariates under randomization, 
 and furhter using the covariates from augmentR1, to augment with R indiciating that the
 randomization takes place or not,
 \begin{align*}
 AUG_1 = R ( A_1 - \pi_1(X_1))  X_1 \gamma_1
 \end{align*}
 and  the dynamic censoring augmenting  
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s) 
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimize the variance given the dynamic covariates specified by augmentC.

The propensity score models are always estimated unless it is requested to use some fixed number $\pi_0=1/2$ for example, but
always better to be adaptive and estimate $\pi_0$. Also $\gamma_0$ and $\gamma_1$ are estimated to reduce variance of $U_i$. 


  - The treatment's must be given as factors. 
  - Treatment for 2nd randomization may depend on response.
      - Treatment probabilities are estimated by default and uncertainty from this adjusted for.
      - treat.model must then typically allow for interaction with treatment number and covariates
  - Randomization augmentation for 1'st and 2'nd randomization possible. 
       - typesR=c("R0","R1","R01")
  - Censoring model possibly stratified on observed covariates (at time 0). 
     - default model is to stratify after randomization R0
     - cens.model can be specified
  - Censoring augmentation done dynamically over time  with time-dependent covariates. 
      - typesC=c("C","dynC"), C fixed coefficients and dynC dynamic
      - done for each strata in censoring model

 Standard errors are estimated using the influence function of all estimators and tests of differences 
 can therefore be computed subsequently.
 
  - variance adjustment for censoring augmentation computed subtracting variance gain
  - influence functions given for case with R0 only


The times of randomization is specified by 

   - treat.var  is "1" when a randomization is given 
     - default is to assume that all time-points corresponds to a treatment, the survival case without time-dependent
       covariates 
     - recurrent events situation must be specified as first record of each subject, see below example. 


Data must be given on start,stop,status survival format with

 - one code of status indicating events of interest
 - one code for the censorings


The phreg_rct can be used for counting process style data, and thus covers situations with 


  - recurrent events 
  - survival data
  - cause-specific hazards (competing risks)

and will in all cases compute augmentations  

  - dynamic censoring augmentation
     - dynC
  - RCT augmentation 
     - R0, R1 and R01



Simple Randomization: Lu-Tsiatis marginal Cox model
===================================================


```{r}
library(mets) 
set.seed(100)

## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z
 
out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~X)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X,cens.model=~+1)
```

Results consitent with speff of  library(speff2trial) 

```{r}
###library(speff2trial) 
library(mets)
data(ACTG175)
###
data <- ACTG175[ACTG175$arms==0 | ACTG175$arms==1, ]
data <- na.omit(data[,c("days","cens","arms","strat","cd40","cd80","age")])
data$days <- data$days+runif(nrow(data))*0.01
dfactor(data) <- arms.f~arms
notrun <- 1

if (notrun==0) { 
fit1 <- speffSurv(Surv(days,cens)~cd40+cd80+age,data=data,trt.id="arms",fixed=TRUE)
summary(fit1)
}
# 
# Treatment effect
#             Log HR       SE   LowerCI   UpperCI           p
# Prop Haz  -0.70375  0.12352  -0.94584  -0.46165  1.2162e-08
# Speff     -0.72430  0.12051  -0.96050  -0.48810  1.8533e-09

out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~cd40+cd80+age,augmentC=~cd40+cd80+age)
summary(out)
```

The study is actually block-randomized according (?)
so the standard should be computed with an adjustment that is 
equivalent to augmenting with this block as factor 

```{r}
dtable(data,~strat+arms)
dfactor(data) <- strat.f~strat
out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~strat.f)
summary(out)
```

Two-Stage Randomization CALGB-9823 for survival outcomes
=========================================================

We here illustrate some analysis of one SMART conducted by Cancer and
Leukemia Group B Protocol 8923 (CALGB 8923), Stone and others (2001). 
388 patients were randomized to an initial treatment of GM-CSF (A1 ) or standard chemotherapy
(A2 ). Patients with complete remission and informed consent to second stage were then re-randomized to
only cytarabine (B1 ) or cytarabine plus mitoxantrone (B2 ). 

We first compute the weighted risk-set  estimator based on estimated weights
\begin{align*}
\Lambda_{A1,B1}(t) & = \sum_i \int_0^t \frac{w_i(s)}{Y^w(s)} dN_i(s)
\end{align*}
where $w_i(s) = I(A0_i=A1) + (t>T_R) I(A1_i=B1)/\pi_1(X_i)$, that is 
1 when you start on treatment $A1$ and
then for those that changes to $B1$ at time $T_R$ then is scaled up 
with the proportion doing this. This is equivalent to
the IPTW (inverse probability of treatment weighted estimator). We estimate 
the treatment regimes $A1, B1$ and $A2, B1$ by letting $A10$ indicate those that are consistent with ending on $B1$. 
$A10$ then starts being $1$ and becomes $0$ if the subject is treated with $B2$, but stays $1$ if the subject is treated with $B1$. 
We can then look at the two strata where $A0=0,A10=1$ and $A0=1,A10=1$. Similary, for those that end being consistent with $B2$.
Thus defining $A11$ to start being $1$, then stays $1$ if $B2$ is taken, and becomes $0$ if the second randomization is $B1$.

  - the treatment models are for all time-points, unless  the weight.var variable is given  (1 for treatments, 0 otherwise) to accomodate a general start,stop format
  - the treatment model may also depend on a response value
  - standard errors are based on influence functions and is also computed for the baseline

We here use the propensity score model $P(A1=B1|A0)$ that uses the 
observed frequencies on arm $B1$ among those starting out on either $A1$ or $A2$.

```{r}
data(calgb8923)
calgt <- calgb8923

## tm <- At.f~factor(Count2)+age+sex+wbc
## tm <- At.f~factor(Count2)
tm <-  At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=1:10)
summary(pll0,times=1:10)
```

The propensity score mode can be extended to use covariates to get increased efficiency.
Note also that the propensity scores for $A0$ will cancel out in the different strata. 

We now illustrate how to fit a  Cox model of the form
\begin{align*}
  & \lambda_{A0}(t) \exp( B1(t) \beta_1 + B2(t) \beta_2)
\end{align*}
where $\beta_0$ is the effect of treatment $A2$ and the effect of $B1$ 

Now  comparing only those starting on A1/A2 to compare the effect of B1 versus B2

```{r}
library(mets)
data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1

## making time-dependent indicators of going to B1/B2
calgt$A10t <- calgt$A11t <- 0
calgt <- dtransform(calgt,A10t=1,A1==0 & Count2==1)
calgt <- dtransform(calgt,A11t=1,A1==1 & Count2==1)
calgt0 <- subset(calgt,A0==0)

ss0 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==0),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss0)

ss1 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==1),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss1)
```

and a more structured model with both A0 and A1, that does not seem very reasonable based on 
the above,

```{r}
ssf <- phreg_rct(Event(start,time,status)~A0.f+A10t+A11t+cluster(id),
	 data=calgt,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR0=~age+wbc+sex,augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
```



Recurrent events: Simple Randomization 
======================================

 Recurrents events simulation with death and censoring. 


```{r}
n <- 1000
beta <- 0.15; 
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- scalecumhaz(CPH_HPN_CRBSI$mechanical,0.5)

cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
ce <- 3; betao1 <- 0

varz <- 1; dep=4; X <- z <- rgamma(n,1/varz)*varz
Z0 <- NULL
px <- 0.5
if (betao1!=0) px <- lava::expit(betao1*X)      
A0 <- rbinom(n,1,px)
r1 <- exp(A0*beta[1])
rd <- exp( A0 * 0.15)
rc <- exp( A0 * 0 )
###
rr <-    mets:::simLUCox(n,base1,death.cumhaz=dr,r1=r1,Z0=X,dependence=dep,var.z=varz,cens=ce/5000)
rr$A0 <- A0[rr$id]
rr$z1 <- attr(rr,"z")[rr$id]
rr$lz1 <- log(rr$z1)
rr$X <- rr$lz1 
rr$lX <- rr$z1
rr$statusD <- rr$status
rr <- dtransform(rr,statusD=2,death==1)
rr <- count.history(rr)
rr$Z <- rr$A0
data <- rr
data$Z.f <- as.factor(data$Z)
data$treattime <- 0
data <- dtransform(data,treattime=1,lbnr__id==1)
dlist(data,start+stop+statusD+A0+z1+treattime+Count1~id|id %in% c(4,5))
```

Now we fit the model 

```{r}
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 treat.var="treattime",typesR=c("non","R0"),typesC=c("non","C","dynC"),
	 augmentR0=~z1,augmentC=~z1+Count1)
summary(fit2)
```

 - Censoring model was stratified on Z.f
 - treatment probabilities were estimated using the data


Twostage Randomization: Recurrent events 
=========================================



```{r}
n <- 500
beta=c(0.3,0.3);betatr=0.3;betac=0;betao=0;betao1=0;ce=3;fixed=1;sim=1;dep=4;varz=1;ztr=0; ce <- 3
## take possible frailty 
Z0 <- rgamma(n,1/varz)*varz
px0 <- 0.5; if (betao!=0) px0 <- expit(betao*Z0)
A0 <- rbinom(n,1,px0)
r1 <- exp(A0*beta[1])
#
px1 <- 0.5; if (betao1!=0) px1 <- expit(betao1*Z0)
A1 <- rbinom(n,1,px1)
r2 <- exp(A1*beta[2])
rtr <- exp(A0*betatr[1])
rr <-  mets:::simLUCox(n,base1,death.cumhaz=dr,cumhaz2=base1,rtr=rtr,betatr=0.3,A0=A0,Z0=Z0,
		r1=r1,r2=r2,dependence=dep,var.z=varz,cens=ce/5000,ztr=ztr)
rr$z1 <- attr(rr,"z")[rr$id]
rr$A1 <- A1[rr$id]
rr$A0 <- A0[rr$id]
rr$lz1 <- log(rr$z1)
rr <- count.history(rr,types=1:2)
rr$A1t <- 0
rr <- dtransform(rr,A1t=A1,Count2==1) 
rr$At.f <- rr$A0
rr$A0.f <- factor(rr$A0)
rr$A1.f <- factor(rr$A1)
rr <- dtransform(rr, At.f = A1, Count2 == 1)
rr$At.f <- factor(rr$At.f)
dfactor(rr)  <-  A0.f~A0
rr$treattime <- 0
rr <- dtransform(rr,treattime=1,lbnr__id==1)
rr$lagCount2 <- dlag(rr$Count2)
rr <- dtransform(rr,treattime=1,Count2==1 & (Count2!=lagCount2))
dlist(rr,start+stop+statusD+A0+A1+A1t+At.f+Count2+z1+treattime+Count1~id|id %in% c(5,10))
```

Now fitting the model and computing different augmentations (true values 0.3 and 0.3)

```{r}
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),treat.var="treattime",
	 treat.model=At.f~factor(Count2),
	 augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
```

  - treat.model has A0 and A1 coded as At.f and we here allow a model that depends on the randomization to
    be as adaptive as possible. 
  - for the observational case one can here also adjust for covarites. 
  - Censoring model was stratified on A0.f 


Causal assumptions for Twostage Randomization: Recurrent events 
===============================================================

We take interest in $N_1$ but also have death $N_d$. 

Now we need that given $X_0$ 

  - \( A_0 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | X_0 \) 
  - positivity \(  1 > \pi_0(X_0) > 0 \) 


and given $\bar X_1$ the history accumulated at time $T_R$ of 2nd randomization

  - \( A_1 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | \bar X_1 \) 
  - positivity \(  1 > \pi_1(X_1) > 0 \) 


and 

  - consistency 

to link the counterfactual quantities to observed data. 


We must use IPTW weighted Cox score and augment as before

In addition we need that the censoring is independent given for example $A_0$

  - Independent censoring 

To use the phreg_rct in this situation 

 - RCT=FALSE
 - propensity score models must be specified
 - marginal estimate is based on IPTW cox model phreg_IPTW
    - when the same model is used for the propensity scores and the augmentation models
      the augmentation models are not needed due to the adaptive nature from fitting the
      propensity score models. 


```{r}
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 typesR=c("non","R0"),typesC=c("non","C","dynC"),
         RCT=FALSE,treat.model=Z.f~z1,augmentR0=~z1,augmentC=~z1+Count1,
	 treat.var="treattime")
summary(fit2)
```

and for twostage randomization

```{r}
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
         treat.var="treattime",
	 RCT=FALSE,treat.model=At.f~z1*factor(Count2),
         augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Two-Stage Randomization for for Competing risks and Survival outcomes"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Two-Stage Randomization for for Competing risks and Survival outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Two-Stage Randomization for for Competing risks and Survival outcomes
=====================================================================

 Under two-stage randomization we can estimate the average treatment effect $E(Y(i,\bar k))$ of treatment regime $(i,\bar k)$. 

   - treatment A0=i
and 
     -  for all responses, randomization A1 = (k_1), so treatment k_1 
     -  response*A1 = (k_1, k_2), so treatment k_1 if response 1, and treatment k_2 if response 2.

 The estimator can be agumented in different ways: using the two randomizations and the dynamic censoring augmentatation.

Estimating  $\mu_{i,\bar k} = P(Y(i,\bar k,\epsilon=v) <= t)$, restricted mean
$E( \min(Y(i,\bar k),\tau))$  or years lost $E( I(\epsilon=v) \cdot (\tau  -  \min(Y(i,\bar k),\tau)))$ 
using IPCW weighted estimating equations : \\

 The solved estimating eqution is 
 \begin{align*}
 \sum_i  \frac{I(min(T_i,t) < G_i)}{G_c(min(T_i ,t))} I(T \leq t, \epsilon=1 ) - AUG_0 - AUG_1 + AUG_C  -  p(i,j)) = 0 
 \end{align*}
 using the covariates from augmentR0 to augment with 
 \begin{align*}
 AUG_0 = \frac{A_0(i) - \pi_0(i)}{ \pi_0(i)} X_0 \gamma_0
 \end{align*}
 and  using the covariates from augmentR1 to augment with 
 \begin{align*}
 AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{ \pi_1(j)} X_1 \gamma_1
 \end{align*}
 and  censoring augmenting  with 
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s) 
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimize the variance given the dynamic covariates specified by augmentC.


  - The treatment's must be given as factors. 
  - Treatment for 2nd randomization may depend on response.
      - Treatment probabilities are estimated by default and uncertainty from this adjusted for.
  - Randomization augmentation for 1'st and 2'nd randomization possible. 
  - Censoring model possibly stratified on observed covariates (at time 0). 
  - Censoring augmentation done dynamically over time  with time-dependent covariates. 

 Standard errors are estimated using the influence function  of all estimators and tests of differences can therefore be computed
 subsequently.


Data must be given on start,stop,status survival format with

 - one code of status indicating response, that is 2nd randomization
 - other codes defines the outcome of interest


```{r}
library(mets) 
set.seed(100)

n <- 200
ddf <- mets:::gsim(n,covs=1,null=0,cens=1,ce=1,betac=c(0.3,1))
true <- apply(ddf$TTt<2,2,mean)
true
datat <- ddf$datat
## set-random response on data, only relevant after status==2 
response <- rbinom(n,1,0.5)
datat$response <- as.factor(response[datat$id]*datat$Count2)
datat$A000 <- as.factor(1)
datat$A111 <- as.factor(1)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f,
		augmentR1=~X11+X12+TR, augmentR0=~X01+X02,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f))
bb

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01))

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]/p[2],p[3]/p[4]))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]-p[2],p[3]-p[4]))

```




```{r}
## 2 levels for each response , fixed weights 
datat$response.f <- as.factor(datat$response)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(0,0),pi0=0.5,pi1=0.5)
bb

## 2 levels for each response ,  estimated treat probabilities
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb


## 2 and 3 levels for each response , fixed weights 
datat$A1.23.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.23.f+response)
datat <- dtransform(datat,A1.23.f=2+rbinom(nrow(datat),1,0.5),
		    Count2==1 & A1.23.f==2 & response==0)
dtable(datat,~A1.23.f+response)
datat$A1.23.f <- as.factor(datat$A1.23.f)
dtable(datat,~A1.23.f+response|Count2==1)
###
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(1,0),pi1=c(0.3,0.5))
bb

## 2 and 3 levels for each response , estimated 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb

## 2 and 1 level for each response 
datat$A1.21.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.21.f+response|Count2==1)
datat <- dtransform(datat,A1.21.f=1,Count2==1 & response==1)
dtable(datat,~A1.21.f+response|Count2==1)
datat$A1.21.f <- as.factor(datat$A1.21.f)
dtable(datat,~A1.21.f+response|Count2==1)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb

## known weights 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,0),pi1=c(0.5,1))
bb
```



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "IPCW Cumulative Cost"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    # fig_width: 7.15
    # fig_height: 5.5 
header-includes: 
  - \usepackage{tikz}
vignette: >
  %\VignetteIndexEntry{IPCW Cumulative Cost}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview 
========

We here describe how to do regression modelling for cumulative cost
\begin{align*}
{\cal U}(t) & = \int_0^t Z(s) dN(s)
\end{align*}
where $N(s)$ is a counting process that registers the times at which the cost is realized and accumulated, 
and $Z(t)$ is the cost (or marks) at the event times. The counting process can be 
a mix of random and fixed times. The data would thus be represented in counting process format with
the marks/costs going along with the event times. There are many  additional uses of
such cumulative process, for example, when considering time-lost in a recurrent events setting, that we return to
below. 

We can estimate the marginal mean of the cumulative process
\begin{align*}
\nu(t) & = E ( {\cal U}(t) )  
\end{align*}
possibly for strata with standard errors based on the derived influence function. 

We provide semi-parametric regression modelling using the  proportional model 
\begin{align*}
E ( {\cal U}(t) | X) & = \Lambda_0(t) \exp( X^T \beta).
\end{align*}

In addition for a fixed time-point $t \in [0,\tau]$ we can estimate the mean given covariates
\begin{align*}
E ( {\cal U}(t) | X) & = \exp( X^T \beta) 
\end{align*}
where $\tau$ is some maximum follow-up time. 

 - These quantities are estimated in a setting with independent right-censoring given $X$, and based on IPCW adjusted estimating equations. 
    - similarly to the Ghosh-Lin model for recurrent events 
 - A terminal event can be specified. 

We also estimate the probability of exceeding thresholds over time
\begin{align*}
 P ( {\cal U}(t)  > k ) & = \mu_k(t),
\end{align*}
and in the situation with a terminal this is based on a derived competing risks data that 
keeps track of the competing terminal event. 

\begin{tikzpicture}[
    node distance=3.5cm,
    box/.style={draw, rectangle, minimum width=2.8cm, minimum height=1cm, align=center}
]
% Nodes
\node[box] (start) {At Risk};
\node[box, right of=start, yshift=2.4cm] (exceed) {Exceed-k};
\node[box, right of=start, yshift=-2.4cm] (death) {D};

% Arrows with hazard labels
\draw[->, thick] (start) -- node[below] {\hspace{2 cm} $F_{exceed-k}(t)$} (exceed);
\draw[->, thick] (start) -- node[below] {$F_{D}(t)$} (death);
\end{tikzpicture}

Regression modelling of this quantity is also possible using competing risks 
regression models, using for example, the cifreg function of mets. 


HF-action data
=============

Considering the HF-action data we simulate a severity score for each event. 


```{r}
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

proc_design <- mets:::proc_design
## marginal mean using formula  
outNZ <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 

outN <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
summary(outN,times=3) 
```

For comparison we also compute the IPCW estimates with and without marks at time 3, using the linear model, 
and note that they are identical. Standard errors are however based on different formula that are asymptotically 
equivalent, and we note that they are very similar.

```{r}
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
head(iid(outNZ3))

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
head(iid(outN3))
```

We also apply the semiparametric proportional cost model with IPCW adjustment: 

```{r}
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
plot(GL,add=TRUE,col=2)
```
Those treated have 14 \% lower cumulative severity and 11\%  lower number of expected events. 


Exceed threshold
=================

Finally, we also estimate the probability of exceeding a cumulative severity at 1,5,10

```{r}
ooNZ <- prob.exceed.recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
summary(ooNZ,times=3)
```

Cumulative time lost for recurrent events 
=========================================

The cumulative time lost for recurrent events has been defined as
\begin{align*}
  {\cal M}(t) = E[ \int_0^\tau (\tau-s) dN(s) ] = \int_0^\tau \mu(s) ds
\end{align*}
where $\mu(t) = E( N(t) )$ is the marginal mean of the recurrent events at time $t$. 


```{r}
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
head(iid(RecLost))
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Two-Stage Randomization for for Competing risks and Survival outcomes"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Two-Stage Randomization for for Competing risks and Survival outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Two-Stage Randomization for for Competing risks and Survival outcomes
=====================================================================

 Under two-stage randomization we can estimate the average treatment effect $E(Y(i,\bar k))$ of treatment regime $(i,\bar k)$. 

   - treatment A0=i
and 
     -  for all responses, randomization A1 = (k_1), so treatment k_1 
     -  response*A1 = (k_1, k_2), so treatment k_1 if response 1, and treatment k_2 if response 2.

 The estimator can be agumented in different ways: using the two randomizations and the dynamic censoring augmentatation.

Estimating  $\mu_{i,\bar k} = P(Y(i,\bar k,\epsilon=v) <= t)$, restricted mean
$E( \min(Y(i,\bar k),\tau))$  or years lost $E( I(\epsilon=v) \cdot (\tau  -  \min(Y(i,\bar k),\tau)))$ 
using IPCW weighted estimating equations : \\

 The solved estimating eqution is 
 \begin{align*}
 \sum_i  \frac{I(min(T_i,t) < G_i)}{G_c(min(T_i ,t))} I(T \leq t, \epsilon=1 ) - AUG_0 - AUG_1 + AUG_C  -  p(i,j)) = 0 
 \end{align*}
 using the covariates from augmentR0 to augment with 
 \begin{align*}
 AUG_0 = \frac{A_0(i) - \pi_0(i)}{ \pi_0(i)} X_0 \gamma_0
 \end{align*}
 and  using the covariates from augmentR1 to augment with 
 \begin{align*}
 AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{ \pi_1(j)} X_1 \gamma_1
 \end{align*}
 and  censoring augmenting  with 
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s) 
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimize the variance given the dynamic covariates specified by augmentC.


  - The treatment's must be given as factors. 
  - Treatment for 2nd randomization may depend on response.
      - Treatment probabilities are estimated by default and uncertainty from this adjusted for.
  - Randomization augmentation for 1'st and 2'nd randomization possible. 
  - Censoring model possibly stratified on observed covariates (at time 0). 
  - Censoring augmentation done dynamically over time  with time-dependent covariates. 

 Standard errors are estimated using the influence function  of all estimators and tests of differences can therefore be computed
 subsequently.


Data must be given on start,stop,status survival format with

 - one code of status indicating response, that is 2nd randomization
 - other codes defines the outcome of interest


```{r}
library(mets) 
set.seed(100)

n <- 200
ddf <- mets:::gsim(n,covs=1,null=0,cens=1,ce=1,betac=c(0.3,1))
true <- apply(ddf$TTt<2,2,mean)
true
datat <- ddf$datat
## set-random response on data, only relevant after status==2 
response <- rbinom(n,1,0.5)
datat$response <- as.factor(response[datat$id]*datat$Count2)
datat$A000 <- as.factor(1)
datat$A111 <- as.factor(1)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f,
		augmentR1=~X11+X12+TR, augmentR0=~X01+X02,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f))
bb

estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]/p[2],p[3]/p[4]))
estimate(coef=bb$riskG$riskG01[,1],vcov=crossprod(bb$riskG.iid$riskG01),f=function(p) c(p[1]-p[2],p[3]-p[4]))

```


```{r}

## 2 levels for each response , fixed weights 
datat$response.f <- as.factor(datat$response)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
		estpr=c(0,0),pi0=0.5,pi1=0.5)
bb

## 2 levels for each response ,  estimated treat probabilities
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
		treat.model0=A0.f~+1, treat.model1=A1.f~A0.f*response.f,
		augmentR0=~X01+X02, augmentR1=~X11+X12,
		augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),estpr=c(1,1))
bb


## 2 and 3 levels for each response , fixed weights 
datat$A1.23.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.23.f+response)
datat <- dtransform(datat,A1.23.f=2+rbinom(nrow(datat),1,0.5),
		    Count2==1 & A1.23.f==2 & response==0)
dtable(datat,~A1.23.f+response)
datat$A1.23.f <- as.factor(datat$A1.23.f)
dtable(datat,~A1.23.f+response|Count2==1)
###
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1,treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR, cens.model=~strata(A0.f),
        estpr=c(1,0),pi1=c(0.3,0.5))
bb


## 2 and 3 levels for each response , estimated 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.23.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb


## 2 and 1 level for each response 
datat$A1.21.f <- as.numeric(datat$A1.f)
dtable(datat,~A1.21.f+response|Count2==1)
datat <- dtransform(datat,A1.21.f=1,Count2==1 & response==1)
dtable(datat,~A1.21.f+response|Count2==1)
datat$A1.21.f <- as.factor(datat$A1.21.f)
dtable(datat,~A1.21.f+response|Count2==1)
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,cens.model=~strata(A0.f),
	estpr=c(1,1))
bb

## known weights 
bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),datat,time=2,cause=c(1),response.code=2,
	treat.model0=A0.f~+1, treat.model1=A1.21.f~A0.f*response.f,
	augmentR0=~X01+X02, augmentR1=~X11+X12,
	augmentC=~X01+X02+A11t+A12t+X11+X12+TR,
	cens.model=~strata(A0.f),estpr=c(1,0),pi1=c(0.5,1))
bb
```

Two-Stage Randomization CALGB-9823
==================================

We here illustrate some analysis of one SMART conducted by Cancer and
Leukemia Group B Protocol 8923 (CALGB 8923), Stone and others (2001). 
388 patients were randomized to an initial treatment of GM-CSF (A1 ) or standard chemotherapy
(A2 ). Patients with complete remission and informed consent to second stage were then re-randomized to
only cytarabine (B1 ) or cytarabine plus mitoxantrone (B2 ). 

We first compute the weighted risk-set  estimator based on estimated weights
\begin{align*}
\Lambda_{A1,B1}(t) & = \sum_i \int_0^t \frac{w_i(s)}{Y^w(s)} dN_i(s)
\end{align*}
where $w_i(s) = I(A0_i=A1) + (t>T_R) I(A1_i=B1)/\pi_1(X_i)$, that is 
1 when you start on treatment $A1$ and
then for those that changes to $B1$ at time $T_R$ then is scaled up 
with the proportion doing this. This is equivalent to
the IPTW (inverse probability of treatment weighted estimator). We estimate 
the treatment regimes $A1, B1$ and $A2, B1$ by letting $A10$ indicate those that are consistent with ending on $B1$. 
$A10$ then starts being $1$ and becomes $0$ if the subject is treated with $B2$, but stays $1$ if the subject is treated with $B1$. 
We can then look at the two strata where $A0=0,A10=1$ and $A0=1,A10=1$. Similary, for those that end being consistent with $B2$.
Thus defining $A11$ to start being $1$, then stays $1$ if $B2$ is taken, and becomes $0$ if the second randomization is $B1$.

  - the treatment models are for all time-points, unless  the weight.var variable is given  (1 for treatments, 0 otherwise) to accomodate a general start,stop format
  - the treatment model may also depend on a response value
  - standard errors are based on influence functions and is also computed for the baseline

We here use the propensity score model $P(A1=B1|A0)$ that uses the 
observed frequencies on arm $B1$ among those starting out on either $A1$ or $A2$.

```{r}
data(calgb8923)
calgt <- calgb8923

tm=At.f~factor(Count2)+age+sex+wbc
tm=At.f~factor(Count2)
tm=At.f~factor(Count2)*A0.f

head(calgt)
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)

summary(pll1,times=12)
summary(pll0,times=12)
```

The propensity score mode can be extended to use covariates to get increased efficiency.
Note also that the propensity scores for $A0$ will cancel out in the different strata. 



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "GEE cluster standard errors and predictions for glm objects" 
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{GEE cluster standard errors and predictions for glm objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Utility functions for GLM objects
=================================

Computing odds ratios with confidence intervals using GEE (sandwich)
standard errors:


``` r
set.seed(100)

library(mets)
data(bmt); 
bmt$id <- sample(1:100,408,replace=TRUE)

glm1 <- glm(tcell~platelet+age,bmt,family=binomial)
summaryGLM(glm1)
#> $coef
#>             Estimate Std.Err    2.5%   97.5%   P-value
#> (Intercept)  -2.4371  0.2225 -2.8732 -2.0009 6.481e-28
#> platelet      1.1368  0.3076  0.5340  1.7397 2.189e-04
#> age           0.5927  0.1551  0.2888  0.8966 1.319e-04
#> 
#> $or
#>               Estimate       2.5%     97.5%
#> (Intercept) 0.08741654 0.05651794 0.1352076
#> platelet    3.11688928 1.70573194 5.6955015
#> age         1.80895115 1.33489115 2.4513641
#> 
#> $fout
#> NULL

## GEE robust standard errors
summaryGLM(glm1,id=bmt$id)
#> $coef
#>             Estimate Std.Err    2.5%   97.5%   P-value
#> (Intercept)  -2.4371  0.2157 -2.8599 -2.0142 1.361e-29
#> platelet      1.1368  0.2830  0.5822  1.6914 5.877e-05
#> age           0.5927  0.1434  0.3117  0.8738 3.568e-05
#> 
#> $or
#>               Estimate       2.5%     97.5%
#> (Intercept) 0.08741654 0.05727471 0.1334211
#> platelet    3.11688928 1.79006045 5.4271903
#> age         1.80895115 1.36575550 2.3959664
#> 
#> $fout
#> NULL
```

Predictions are equally straightforward:


``` r
age <- seq(-2,2,by=0.1)
nd <- data.frame(platelet=0,age=seq(-2,2,by=0.1))
pnd <- predictGLM(glm1,nd)
head(pnd$pred)
#>      Estimate       2.5%      97.5%
#> p1 0.02601899 0.01115243 0.05951051
#> p2 0.02756409 0.01214068 0.06136414
#> p3 0.02919819 0.01321187 0.06328733
#> p4 0.03092608 0.01437206 0.06528441
#> p5 0.03275278 0.01562757 0.06736019
#> p6 0.03468351 0.01698493 0.06952008
plot(age,pnd$pred[,1],type="l",ylab="predictions",xlab="age",ylim=c(0,0.3))
plotConfRegion(age,pnd$pred[,2:3],col=2)
```

<img src="figure/glm-utility-unnamed-chunk-3-1.png" alt="GLM predicted probabilities with confidence region."  />

``` r
###matlines(age,pnd$pred[,-1],col=2,lty=1)
```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Recurrent events"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    # fig_width: 7.15
    # fig_height: 5.5 
header-includes: 
  - \usepackage{tikz}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Recurrent events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  #dev="svg",
  dpi=50,
  fig.width=7, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
)
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE")
library(mets)
```

Overview 
========

For recurrent events data it is often of interest to compute basic descriptive
quantities to get some basic understanding of the phenonmenon studied. 
We here demonstrate how one can compute: 

 * the marginal mean 
   * efficient marginal mean estimation with fast computation of standard errors
 * the Ghosh-Lin Cox type regression for the marginal mean, possibly with composite outcomes. 
   * efficient regression augmentation of the Ghosh-Lin model
   * clusters can be specfied
   * allows a stratified baseline 
 * the variance of a recurrent events process
 * the probability of exceeding k events
 * the two-stage recurrent events random effects model

We also show how to improve the efficiency of recurrents events marginal mean. 

In addition several tools can be used for simulating recurrent events and
bivariate recurrent events data, also with a possible terminating event: 

 * recurrent events with multiple 
   * event types 
   * Cox type rates 
 * with a terminal event with possibly multiple causes of death
   * Cox type rates 
 * frailty extenstions
 * the Ghosh-Lin model when the survival rate is on Cox form.
   * frailty extenstions
 * The general illness death model with  Cox models for all hazards. 

\begin{tikzpicture}[
    sharp corners=2pt,
    inner sep=4pt,
    node distance=0.9cm,
    >=latex,
    mynode/.style={draw,minimum height=0.9cm,minimum width=2.6cm}
]

\node[mynode] (healthy) {healthy $(h)$};
\node[mynode, right=of healthy] (e1) {1st event};
\node[mynode, right=of e1] (e2) {2nd event};

% Dots indicating continuation
\node[right=0.6cm of e2] (dots) {$\cdots$};

% Dead
\node[mynode, below=of e1] (dead) {dead $(d)$};

% Arrows along the event chain
\draw[->] (healthy) -- (e1);
\draw[->] (e1) -- (e2);
\draw[->] (e2) -- (dots);

% Arrows to 'dead'
\draw[->] (healthy) -- (dead);
\draw[->] (e1) -- (dead);
\draw[->] (e2) -- (dead);

\end{tikzpicture}


Simulation of recurrents events
==============================

We start by simulating some recurrent events data with two type of events with cumulative hazards 

 * $\Lambda_1(t)$ (rate among survivors)
 * $\Lambda_2(t)$ (rate among survivors)
 * $\Lambda_D(t)$

 where we consider types 1 and 2 and with a rate of the terminal event given by 
 $\Lambda_D(t)$. We let the events be independent, but could also specify a random effects
 structure to generate dependence. 

 When simulating data we can impose various random-effects structures to generate dependence  

* Dependence=0: The intensities can be independent.

* Dependence=1:  One gamma distributed random effects $Z$.  Then the intensities are 
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $Z \lambda_D(t)$

* Dependence=4:  One gamma distributed random effects $Z$.  Then the intensities are 
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $\lambda_D(t)$

* Dependence=2: We can draw normally distributed random effects $Z_1,Z_2,Z_d$ were the variance (var.z) 
  and correlation can be specified (cor.mat).  Then the intensities are 
    + $\exp(Z_1) \lambda_1(t)$
    + $\exp(Z_2) \lambda_2(t)$
    + $\exp(Z_3) \lambda_D(t)$

* Dependence=3: We can draw gamma distributed random effects $Z_1,Z_2,Z_d$ were the sum-structure can be speicifed via a matrix
   cor.mat. We compute $\tilde Z_j = \sum_k  Z_k^{cor.mat(j,k)}$ for $j=1,2,3$.  
   Then the intensities are
    + $\tilde Z_1 \lambda_1(t)$
    + $\tilde Z_2 \lambda_2(t)$
    + $\tilde Z_3 \lambda_D(t)$

We return to how to run the different set-ups later and start by simulating independent processes. 

The key functions are 

 * simRecurrent
   * simple simulation with only one event type and death 
 * simRecurrentII 
   * extended version with possibly multiple types of recurrent events (but rates can be 0) 
   * Allows Cox types rates with subject specific rates 
 * simRecurrentList
   * lists are allowed for multiple events and cause of death (competing risks)
   * Allows Cox types rates with subject specific rates 
 * sim.recurrent to simulate from Cox-Cox  (marginals) or Ghosh-Lin-Cox

In addition we can simulate data from the Ghosh-Lin model and where marginals of the rates among survivors are on
on Cox form 

 * simGLcox 
   * can simulate data from Ghosh-Lin model  (also simRecurrentCox)
   * with frailties
      * where survival model for terminal event is on Cox form 
   * can simulate data where rates among survivors are are con Cox form 
     * with frailties

see examples below for specific models. 


Utility functions 
=================

We here mention two utility functions 

 * tie.breaker for breaking ties among jump-times which is expected in the functions below.
 * count.history that counts the number of jumps previous for each subject that is $N_1(t-)$ and $N_2(t-)$. 
 

Marginal Mean 
=============

We start by estimating the marginal mean $E(N_1(t \wedge D))$ where $D$ is the timing of the terminal event. 
The marginal mean is the average number of events seen before time $t$. 

This is based on a  two rate models for 

 * the type 1 events  $\sim E(dN_1(t) | D > t)$
 * the terminal event $\sim E(dN_d(t) | D > t)$

and is defined as $\mu_1(t)=E(N_1(t))$ 
\begin{align}
   \int_0^t S(u) d R_1(u)	
\end{align}
where $S(t)=P(D \geq t)$ and $dR_1(t) = E(dN_1(t) | D > t)$ 

and can therefore be estimated by a 

 * Kaplan-Meier estimator, $\hat S(u)$ 
 * Nelson-Aalen estimator for $R_1(t)$

\begin{align}
  \hat R_1(t) & =   \sum_i \int_0^t  \frac{1}{Y_\bullet (s)}  dN_{1i}(s)
\end{align}
where $Y_{\bullet}(t)= \sum_i Y_i(t)$ such that the estimator is 
\begin{align}
  \hat \mu_1(t) & =    \int_0^t \hat S(u) d\hat R_1(u),
\end{align}
see Cook & Lawless (1997) and  Gosh & Lin (2000). 

The variance can be estimated based on the asymptotic expansion 
of $\hat \mu_1(t) - \mu_1(t)$
\begin{align*}
  & \sum_i \int_0^t \frac{S(s)}{\pi(s)} dM_{i1}  - \mu_1(t) \int_0^t  \frac{1}{\pi(s)} dM_i^d +  \int_0^t \frac{\mu_1(s) }{\pi(s)} dM_i^d,
\end{align*}

with mean-zero processes 

 * $M_i^d(t) = N_i^D(t)- \int_0^t Y_i(s) d \Lambda^D(s)$, 
 * $M_{i1}(t) = N_{i1}(t) - \int_0^t Y_{i}(s) dR_1(s)$. 

as described in Gosh & Lin (2000)


Generating data
==================

We start by generating some data to illustrate the computation of the marginal mean 

```{r}
library(mets)
set.seed(1000) # to control output in simulatins for p-values below.
```

```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- simRecurrent(200,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))
```

The status variable keeps track of the recurrent evnts and their type, and death the timing of 
death.

To compute the marginal mean we simly estimate the two rates functions of the 
number of events of interest and death by using the phreg function 
(to start without covariates). Then the estimates are combined with standard 
error computation in the recurrentMarginal function

```{r}
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust.phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrentMarginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)
```

We can also extract the estimate in different time-points 

```{r}
summary(out,times=c(1000,2000))
```

The marginal mean can also be estimated in the stratified case:

```{r}
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust.phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

Further, if we adjust for covariates for the two rates we can still do
predictions of marginal mean, what can be plotted is the baseline marginal mean, 
that is for the covariates equal to 0 for both models. Predictions for specific 
covariates can also be obtained with the recmarg (recurren marginal mean used 
solely for predictions without standard error computation). 

```{r}
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust.phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)
```

We here simulate multiple recurrent events processes with two causes of death causes  and exponential censoring with
rate $3/5000$, all processes are assumed independent (dependence=0)

```{r}
rr <- simRecurrentList(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))
```

Improving efficiency 
======================

To illustrate how the efficiency can be improved using heterogenity in the data, we
now simulate some data with strong heterogenity. The dynmamic augmentation is 
a regression on the history  for each subject consisting of the specified terms 
terms:  Nt, Nt2 (Nt squared), expNt (exp(-Nt)), NtexpNt (Nt*exp(-Nt)) or by simply
specifying these directly.  This was developed in Cortese and Scheike (2022).

```{r}
rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count.history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrentMarginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

times <- 500*(1:10)
###recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))
```

In the case where covariates might be important but we are still interested in the marginal mean 
we can also augment wrt these covariates 

```{r}
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::simGLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)

times <- seq(500,5000,by=500)
recEFF1x <- recurrentMarginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))

out <- recurrentMarginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)
```

Regression models for the marginal mean 
========================================

One can also do regression modelling , using the model
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
then Ghost-Lin suggested IPCW score equations that are implemented in the recreg function of mets. 

First we generate data that from a Ghosh-Lin model with regression coefficients 
$\beta=(-0.3,0.3)$ and the baseline given by base1, this is done under the assumption that the death 
rate given covariates is on Cox form with baseline dr: 

```{r}
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
summary(outs)$coef

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)
```

We note that for the extended censoring model we gain a little efficiency and that the estimates are close to the true values. 

Also possible to do IPCW regression at fixed time-point

```{r}
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
summary(outipcws)$coef
```

We can also do the Mao-Lin type composite outcome where we both count the recurrent events (cause 1) and deaths (cause 3)
for example 
\begin{align*}
E(N_1(t) + I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}

```{r}
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef
```

This can also be done with competing risks death 
\begin{align*}
E(w_1 N_1(t) + w_2 I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
and with weights  $w_1,w_2$ that follow the causes, here 1 and 3. 
We modify the data by changing some of the cause 3 deaths to cause 4

```{r}
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)
```

Predictions and standard errors can be computed via the iid decompositions of the baseline and the regression coefficients. We illustrate this
for the standard Ghosh-Lin model 

```{r}
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])
```

The Ghosh-Lin model can be made more efficient by the regression augmentation method.
First computing the augmentation and then in a second step the augmented estimator (Cortese and Scheike (2023)): 

```{r}
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef
```

We note that the simple augmentation improves the standard errors as expected. The data was generated assuming independence with
previous number of events so it would suffice to augment only with the covariates. 

Two-stage modelling 
====================

We simulate data with a terminal event on Cox form and recurrent events
satisfying the Ghosh-Lin model or having a rate on Cox form.

 - type=3 is Ghosh-Lin model for recurrent events and Cox for terminal event.
 - type=2 is Cox model for recurrent events among survivors and Cox for terminal event.
 - simulations based on time-grid to make linear approximations of cumulative hazards

Now we fit the two-stage model (the recreg must be called with twostage=TRUE)

```{r}
set.seed(100)
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
###
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
###
rr <- mets:::simGLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
glout$gradient
```

Standard errors are computed assuming that the parameters of out and outs are 
both known, and therefore propobly a bit to small. We could do a bootstrap to 
get more reliable standard errors. 


Simulations with specific structure
===================================

The function simGLcox can simulate data where the recurrent process has mean on Ghosh-Lin form. The key is that
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta) = \int_0^t S(t|X,Z) dR(t|X,Z)
\end{align*}
where $Z$ is a possible frailty. Therefore 
\begin{align*}
 R(t|X,Z) & = \frac{Z \Lambda_0(t)  \exp(X^T \beta) }{S(t|X,Z)}
\end{align*}
leads to a Ghosh-Lin model. We can choose the survival model to have Cox form among survivors by the option
model="twostage", otherwise model="frailty" uses the survival model with rate \( Z \lambda_d(t) rd \).
The $Z$ is gamma distributed  with a variance that can be specified. The simulations are based on 
a piecwise-linear approximation of the hazard functions for $S(t|X,Z)$ and $R(t|X,Z)$.


```{r}
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::simGLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
```

We can also simulate from models where the terminal event is on Cox form and the rate among survivors is on Cox form. 

 * $E(dN_1 | D>t, X) = \lambda_1(t) r_1$
 * $E(dN_d | D>t, X) = \lambda_d(t) r_d$

underlying these models we have a shared frailty model

```{r}
rr <- mets:::simGLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
estimate(recurrent)
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)
```

We can simulate data with underlying dependence fromm the two-stage model (simGLcox) or using 
simRecurrent random effects models, for Cox-Cox or Ghosh-Lin-Cox models.

Here with marginals on 
  - Cox- Cox form
  - Ghosh-Lin - Cox form 

Draws covariates from data and simulates data that has the marginals given. 

```{r}
simcoxcox <- sim.recurrent(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim.recurrent(recurrentGL,margsurv,n=10,data=rr)
```


Other marginal properties 
=========================

The mean is a useful summary measure but it is very easy and useful to look at other 
simple summary measures such as the probability of exceeding $k$ events 

 * $P(N_1^*(t) \ge k)$ 
   * cumulative incidence of $T_{k} = \inf \{ t: N_1^*(t)=k \}$ with competing $D$. 

that is thus equivalent to a certain cumulative incidence of $T_k$ occurring before $D$. We denote this
cumulative incidence as $\hat F_k(t)$. 

We note also that $N_1^*(t)^2$ can be written as
\begin{align*}
   \sum_{k=0}^K  \int_0^t I(D > s) I(N_1^*(s-)=k) f(k) dN_1^*(s)
\end{align*}
with $f(k)=(k+1)^2 - k^2$, such that its mean can be written as 
\begin{align*}
	\sum_{k=0}^K \int_0^t S(s) f(k) P(N_1^*(s-)= k  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k, D> s) 
\end{align*}
and estimated by
\begin{align*}
\tilde \mu_{1,2}(t) & = 
	\sum_{k=0}^K \int_0^t \hat S(s) f(k) 
	\frac{Y_{1\bullet}^k(s)}{Y_\bullet (s)} \frac{1}{Y_{1\bullet}^k(s)} d N_{1\bullet}^k(s)= \sum_{i=1}^n \int_0^t \hat S(s) f(N_{i1}(s-)) \frac{1}{Y_\bullet (s)} d N_{i1}(s),
\end{align*}
That is very similar to the "product-limit" estimator for $E( (N_1^*(t))^2 )$ 
\begin{align}
  \hat \mu_{1,2}(t) & =    \sum_{k=0}^K k^2 ( \hat F_{k}(t) - \hat F_{k+1}(t) ).
\end{align}

We use the esimator of the probabilty  of exceeding "k" events based on the fact that 
$I(N_1^*(t) \geq k)$ is  equivalent to 
\begin{align*}
	\int_0^t I(D > s) I(N_1^*(s-)=k-1) dN_1^*(s),
\end{align*}
suggesting that its mean can be computed as
\begin{align*}
\int_0^t S(s) P(N_1^*(s-)= k-1  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k-1, D> s) 
\end{align*}
and estimated by 
\begin{align*}
\tilde F_k(t) = \int_0^t \hat S(s)  \frac{Y_{1\bullet}^{k-1}(s)}{Y_\bullet (s)} 
          	\frac{1}{Y_{1\bullet}^{k-1}(s)} d N_{1\bullet}^{k-1}(s).
\end{align*}

To compute these estimators we use the prob.exceed.recurrent function

```{r}
rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count.history(rr)
dtable(rr,~statusD)

oo <- prob.exceed.recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)
```

We can also look at the mean and variance based on the estimators just described 

```{r}
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))
```


Multiple events 
================

We now generate recurrent events with two types of events. We start by
generating data as before where all events are independent. 

```{r}
rr <- simRecurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count.history(rr)
dtable(rr,~death+status)
```

Based on this we can estimate also the joint distribution function, that is
the probability that $(N_1(t) \geq k_1, N_2(t) \geq k_2)$

```{r}
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)
```

Looking at other simulations with dependence 
=============================================

Using the normally distributed random effects we plot 4 different settings. We have variance $0.5$ for all
random effects and change the correlation. We let the correlation between the random effect associated with
$N_1$ and $N_2$ be denoted $\rho_{12}$ and the correlation between the random effects 
associated between $N_j$ and $D$ the terminal event be denoted as $\rho_{j3}$, and organize all correlation
in a vector $\rho=(\rho_{12},\rho_{13},\rho_{23})$.

 * Scenario I $\rho=(0,0.0,0.0)$ Independence among all efects.

```{r, eval=FALSE}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

  par(mfrow=c(1,3))
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
  rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count.history(rr,types=1:2)
###  cor(attr(rr,"z"))
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  plot(coo,main ="Scenario I")
```
 * Scenario II $\rho=(0,0.5,0.5)$ Independence among survivors but dependence on terminal event 

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count.history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main ="Scenario II")
```

 * Scenario III $\rho=(0.5,0.5,0.5)$ Positive dependence among survivors and dependence on terminal event 

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # positive dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count.history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario III")
```

 * Scenario IV $\rho=(-0.4,0.5,0.5)$ Negative dependence among survivors and positive dependence on terminal event 

```{r, eval=FALSE}
  var.z <- c(0.5,0.5,0.5)
  # negative dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- simRecurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count.history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario IV")
```


SessionInfo
============


```{r}
sessionInfo()
```

---
title: "While Alive estimands for Recurrent Events"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{While Alive estimands for Recurrent Events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




While Alive estimands for Recurrent Events
==========================================

We consider two while-alive estimands for recurrent events data
\begin{align*}
 \frac{E(N(D \wedge t))}{E(D \wedge t)}
\end{align*}
and the mean of the subject specific events per time-unit 
\begin{align*}
 E( \frac{N(D \wedge t)}{D \wedge t} ) 
\end{align*}
for two treatment-groups in the case of an RCT. 
For the mean of events per time-unit it has been seen that when the sample size is small 
one can improve the finite sample properties by employing a transformation such as square or cube-root, and
thus consider 
\begin{align*}
 E( (\frac{N(D \wedge t)}{D \wedge t})^.33 ) 
\end{align*}


``` r
data(hfactioncpx12)

dtable(hfactioncpx12,~status)
#> 
#> status
#>    0    1    2 
#>  617 1391  124
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
#> While-Alive summaries:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate Std.Err  2.5% 97.5% P-value
#> treatment0    1.859 0.02108 1.817 1.900       0
#> treatment1    1.924 0.01502 1.894 1.953       0
#>  
#>                           Estimate Std.Err    2.5%    97.5% P-value
#> [treatment0] - [treat.... -0.06517 0.02588 -0.1159 -0.01444  0.0118
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#>  
#> chisq = 6.3405, df = 1, p-value = 0.0118
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err  2.5% 97.5%   P-value
#> treatment0    1.572 0.09573 1.384 1.759 1.375e-60
#> treatment1    1.453 0.10315 1.251 1.656 4.376e-45
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....   0.1185  0.1407 -0.1574 0.3943     0.4
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#>  
#> chisq = 0.7085, df = 1, p-value = 0.4
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>    Estimate Std.Err   2.5%  97.5%   P-value
#> p1   0.8457 0.05264 0.7425 0.9488 4.411e-58
#> p2   0.7555 0.05433 0.6490 0.8619 5.963e-44
#>  
#>             Estimate Std.Err     2.5%  97.5% P-value
#> [p1] - [p2]  0.09022 0.07565 -0.05805 0.2385   0.233
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] - [p2] = 0 
#>  
#> chisq = 1.4222, df = 1, p-value = 0.233
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err   2.5%  97.5%   P-value
#> treat0   1.0725  0.1222 0.8331 1.3119 1.645e-18
#> treat1   0.7552  0.0643 0.6291 0.8812 7.508e-32
#>  
#>                     Estimate Std.Err    2.5%  97.5% P-value
#> [treat0] - [treat1]   0.3173  0.1381 0.04675 0.5879 0.02153
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treat0] - [treat1] = 0 
#>  
#> chisq = 5.2837, df = 1, p-value = 0.02153

dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
		   death.code=2,trans=.333)
summary(dd,type="log")
#> While-Alive summaries, log-scale:  
#> 
#> RMST,  E(min(D,t)) 
#>            Estimate  Std.Err   2.5%  97.5% P-value
#> treatment0   0.6199 0.011340 0.5977 0.6421       0
#> treatment1   0.6543 0.007807 0.6390 0.6696       0
#>  
#>                           Estimate Std.Err     2.5%     97.5% P-value
#> [treatment0] - [treat.... -0.03446 0.01377 -0.06145 -0.007478 0.01231
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#>  
#> chisq = 6.2656, df = 1, p-value = 0.01231
#> mean events, E(N(min(D,t))): 
#>            Estimate Std.Err   2.5%  97.5%   P-value
#> treatment0   0.4523 0.06090 0.3329 0.5716 1.119e-13
#> treatment1   0.3739 0.07097 0.2348 0.5130 1.376e-07
#>  
#>                           Estimate Std.Err    2.5%  97.5% P-value
#> [treatment0] - [treat....  0.07835 0.09352 -0.1049 0.2616  0.4022
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treatment0] - [treatment1] = 0 
#>  
#> chisq = 0.7018, df = 1, p-value = 0.4022
#> _______________________________________________________ 
#> Ratio of means E(N(min(D,t)))/E(min(D,t)) 
#>    Estimate Std.Err    2.5%    97.5%   P-value
#> p1  -0.1676 0.06224 -0.2896 -0.04563 7.081e-03
#> p2  -0.2804 0.07192 -0.4214 -0.13947 9.651e-05
#>  
#>             Estimate Std.Err     2.5%  97.5% P-value
#> [p1] - [p2]   0.1128 0.09511 -0.07361 0.2992  0.2356
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] - [p2] = 0 
#>  
#> chisq = 1.4067, df = 1, p-value = 0.2356
#> _______________________________________________________ 
#> Mean of Events per time-unit E(N(min(D,t))/min(D,t)) 
#>        Estimate Std.Err    2.5%   97.5%   P-value
#> treat0  -0.3833 0.04939 -0.4801 -0.2865 8.487e-15
#> treat1  -0.5380 0.05666 -0.6491 -0.4270 2.191e-21
#>  
#>                     Estimate Std.Err     2.5%  97.5% P-value
#> [treat0] - [treat1]   0.1548 0.07517 0.007459 0.3021 0.03948
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treat0] - [treat1] = 0 
#>  
#> chisq = 4.2403, df = 1, p-value = 0.03948
```

We see that the ratio of means is not very different between groups, but that the subject-specific
mean of events per time-unit shows that those on the active treatment have fewer events per time-unit on average.

We can also fit a regression model for the mean of the subject-specific events per time-unit, here using the
exponential link.


``` r
hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id]
hfactioncpx12$sex <- rbinom(741,1,0.5)[hfactioncpx12$id]

dd <- WA_reg(Event(entry,time,status)~treatment+age+sex+cluster(id),hfactioncpx12,time=2,death.code=2)
summary(dd)
#>    n events
#>  741     86
#> 
#>  741 clusters
#> coeffients:
#>               Estimate    Std.Err       2.5%      97.5% P-value
#> (Intercept)  0.0059349  0.1018293 -0.1936468  0.2055166  0.9535
#> treatment1  -0.3475423  0.1399946 -0.6219268 -0.0731578  0.0130
#> age          0.0447745  0.0619333 -0.0766125  0.1661615  0.4697
#> sex          0.1252971  0.1557374 -0.1799426  0.4305367  0.4211
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  1.00595 0.82395 1.2282
#> treatment1   0.70642 0.53691 0.9295
#> age          1.04579 0.92625 1.1808
#> sex          1.13349 0.83532 1.5381
```

Composite outcomes involving death and marks
============================================

The event count can be generalised in various ways by using outcomes other than $N(D \wedge t)$, for example,
\begin{align*}
 \tilde N(D \wedge t) = \int_0^t I(D \geq s) M(s) dN(s)  +  \sum_j  M_j I(D \leq t,\epsilon=j) )
\end{align*}
where $M(s)$ are the marks associated with $N(s)$ and $M_j$ are marks associated with the different
causes of the terminal event. This provides an extension of the weighted composite outcomes measure of Mao \& Lin (2022).

The marks (or weights) can be stochastic, for example when counting hospital expenses, and are
stored as a vector in the data frame. The marks for the event times (defined through the causes) are then used.

Here we weight death by 2 and otherwise count recurrent events as before (with weight 1):


``` r
hfactioncpx12$marks <- runif(nrow(hfactioncpx12))

##ddmg <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
##cause=1:2,death.code=2,marks=hfactioncpx12$marks)
##summary(ddmg)

ddm <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),hfactioncpx12,time=2,
cause=1:2,death.code=2,marks=hfactioncpx12$status)

```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] prodlim_2026.03.11 timereg_2.0.7      survival_3.8-6     mets_1.3.10       
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              KernSmooth_2.23-26    
#>  [7] otel_0.2.0             data.table_1.18.4      glue_1.8.1            
#> [10] future.apply_1.20.2    listenv_0.10.1         lava_1.9.1            
#> [13] stats4_4.6.0           grid_4.6.0             evaluate_1.0.5        
#> [16] lifecycle_1.0.5        yaml_2.3.12            mvtnorm_1.3-7         
#> [19] numDeriv_2016.8-1.1    compiler_4.6.0         codetools_0.2-20      
#> [22] Rcpp_1.1.1-1.1         ucminf_1.2.3           future_1.70.0         
#> [25] lattice_0.22-9         digest_0.6.39          pillar_1.11.1         
#> [28] parallelly_1.47.0      parallel_4.6.0         Matrix_1.7-5          
#> [31] tools_4.6.0            RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "dUtility data-frame manipulations"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes 
    fig_width: 7.15
    fig_height: 5.5   
vignette: >
  %\VignetteIndexEntry{dUtility data-frame manipulations} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```



Simple data manipulation for data-frames 
==========================================

   *  Renaming variables, Deleting variables
   *  Looking at the data
   *  Making new variales for the analysis
   *  Making factors (groupings)
   *  Working with factors
   *  Making a factor from existing numeric variable and vice versa


Here are some key data-manipulation steps on a data-frame which is how we typically 
organize our data in R. 
After having read the data into R it will typically be a data-frame, if not we 
can force it to be a data-frame.   The basic idea of the utility functions is 
to get a simple and easy to type way of making simple data-manipulation on a 
data-frame much like what is possible in SAS or STATA.  

The functions, say,  dcut, dfactor and so on are all functions that basically does what 
the base R cut, factor do, but are easier to use in the context of data-frames 
and have additional functionality. 

```{r}
library(mets)
data(melanoma)
```

```{r}
is.data.frame(melanoma)
```

Here we work on the melanoma data that is already read into R and is 
a data-frame.


dUtility functions 
==================

The structure for all functions 
is  

*  dfunction(dataframe,y~x|ifcond,...)

to use the function on y in a dataframe grouped by x if 
condition ifcond is valid. The basic functions are 

Data processing
    * dsort
    * dreshape
    * dcut
    * drm, drename, ddrop, dkeep, dsubset
    * drelevel
    * dlag
    * dfactor, dnumeric

Data aggregation
    * dby, dby2
    * dscalar, deval, daggregate
    * dmean, dsd, dsum, dquantile, dcor
    * dtable, dcount

Data summaries
    * dhead, dtail, 
    * dsummary, 
    * dprint, dlist, dlevels, dunique


A generic function daggregate, daggr,  can be called with a function as the 
argument 



*  daggregate(dataframe,y~x|ifcond,fun=function,...)

without the grouping variable (x)

*  daggregate(dataframe,~y|ifcond,fun=function,...)

A useful feature is that y and x as well as the subset condition
can be specified using regular-expressions or by wildcards (default).
Here to illustrate this, we compute the means of certain variables. 

First just oveall 
```{r}
dmean(melanoma,~thick+I(log(thick)))
```

now only when days>500
```{r}
dmean(melanoma,~thick+I(log(thick))|I(days>500))
```

and now after sex but only when days>500

```{r}
dmean(melanoma,thick+I(log(thick))~sex|I(days>500))
```

and finally after quartiles of days (via the dcut function)

```{r}
dmean(melanoma,thick+I(log(thick))~I(dcut(days)))
```

or summary of all variables starting with "s" and that contains "a"

```{r}
dmean(melanoma,"s*"+"*a*"~sex|I(days>500))
```


Renaming, deleting, keeping, dropping  variables
===============================================

```{r}
melanoma=drename(melanoma,tykkelse~thick)
names(melanoma)
```

Deleting variables

```{r}
data(melanoma)
melanoma=drm(melanoma,~thick+sex)
names(melanoma)
```

or sas style

```{r}
data(melanoma)
melanoma=ddrop(melanoma,~thick+sex)
names(melanoma)
```

alternatively we can also keep certain variables

```{r}
data(melanoma)
melanoma=dkeep(melanoma,~thick+sex+status+days)
names(melanoma)
```

This can also be done with direct asignment 

```{r}
data(melanoma)
ddrop(melanoma) <- ~thick+sex
names(melanoma)
```

The dkeep function can also be used to re-ordering the variables in the data-frame

```{r}
data(melanoma)
names(melanoma)
melanoma=dkeep(melanoma,~days+status+.)
names(melanoma)
```


Looking at the data 
===================

```{r}
data(melanoma)
dstr(melanoma)
```


The data can in Rstudio be seen as a data-table but to list certain parts of 
the data in output window 

```{r}
dlist(melanoma)
```

```{r}
dlist(melanoma, ~.|sex==1)
```

```{r}
dlist(melanoma, ~ulc+days+thick+sex|sex==1)
```

Getting summaries

```{r}
dsummary(melanoma)
```

or for specfic variables

```{r}
dsummary(melanoma,~thick+status+sex)
```


Summaries in different groups (sex)

```{r}
dsummary(melanoma,thick+days+status~sex)
```


and only among those with thin-tumours or only females (sex==1)

```{r}
dsummary(melanoma,thick+days+status~sex|thick<97)
```

```{r}
dsummary(melanoma,thick+status~+1|sex==1)
```

or 

```{r}
dsummary(melanoma,~thick+status|sex==1)
```


To make more complex conditions need to use the I() 

```{r}
dsummary(melanoma,thick+days+status~sex|I(thick<97 & sex==1))
```

Tables between variables

```{r}
dtable(melanoma,~status+sex)
```

All bivariate tables

```{r}
dtable(melanoma,~status+sex+ulc,level=2)
```

All univariate tables

```{r}
dtable(melanoma,~status+sex+ulc,level=1)
```

and with new variables 

```{r}
dtable(melanoma,~status+sex+ulc+dcut(days)+I(days>300),level=1)
```


Sorting the data
===============

To sort the data 

```{r}
data(melanoma)
mel= dsort(melanoma,~days)
dsort(melanoma) <- ~days
head(mel)
```


and to sort after multiple variables increasing and decreasing 

```{r}
dsort(melanoma) <- ~days-status
head(melanoma)
```


Making new variales for the analysis
=====================================

To define a bunch of new covariates within a data-frame

```{r}
data(melanoma)
melanoma= transform(melanoma, thick2=thick^2, lthick=log(thick) ) 
dhead(melanoma)
```

When the above definitions are done using a condition this can be achieved 
using the dtransform function that extends transform with a possible condition  

```{r}
 melanoma=dtransform(melanoma,ll=thick*1.05^ulc,sex==1)  
 melanoma=dtransform(melanoma,ll=thick,sex!=1)  
 dmean(melanoma,ll~sex+ulc)
```


Making factors (groupings)
=============================

On the melanoma data the variable thick gives the thickness of the melanom tumour. For some analyses we would like to make a factor depending on the thickness. This can be done in several different ways

```{r}
melanoma=dcut(melanoma,~thick,breaks=c(0,200,500,800,2000))
```


New variable is named thickcat.0 by default.

To see levels of factors in data-frame

```{r}
dlevels(melanoma)
```


Checking group sizes 

```{r}
dtable(melanoma,~thickcat.0)
```


With adding to the data-frame directly 

```{r}
dcut(melanoma,breaks=c(0,200,500,800,2000)) <- gr.thick1~thick
dlevels(melanoma)
```

new variable is named thickcat.0 (after first cut-point), 
or to get quartiles with default names thick.cat.4 

```{r}
dcut(melanoma) <- ~ thick  # new variable is thickcat.4
dlevels(melanoma)
```

or median groups, here starting again with the original data, 

```{r}
data(melanoma)
dcut(melanoma,breaks=2) <- ~ thick  # new variable is thick.2
dlevels(melanoma)
```


to control new names

```{r}
data(melanoma)
mela= dcut(melanoma,thickcat4+dayscat4~thick+days,breaks=4)
dlevels(mela)
```

or

```{r}
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4+dayscat4~thick+days
dlevels(melanoma)
```

This can also be typed out more specifically

```{r}
melanoma$gthick = cut(melanoma$thick,breaks=c(0,200,500,800,2000))
melanoma$gthick = cut(melanoma$thick,breaks=quantile(melanoma$thick),include.lowest=TRUE)
```


Working with factors
====================

To see levels of covariates in data-frame

```{r}
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4~thick
dlevels(melanoma) 
```

To relevel the factor

```{r}
dtable(melanoma,~thickcat4)
melanoma = drelevel(melanoma,~thickcat4,ref="(194,356]")
dlevels(melanoma)
```


or to take the third level in the list of levels, same as above, 

```{r}
melanoma = drelevel(melanoma,~thickcat4,ref=2)
dlevels(melanoma)
```

To combine levels of a factor (first combinining first 3 groups into one)

```{r}
melanoma = drelevel(melanoma,~thickcat4,newlevels=1:3)
dlevels(melanoma)
```

or to combine groups 1 and 2 into one group and 3 and 4 into another

```{r}
dkeep(melanoma) <- ~thick+thickcat4
melanoma = drelevel(melanoma,gthick2~thickcat4,newlevels=list(1:2,3:4))
dlevels(melanoma)
```


Changing order of factor levels 

```{r}
dfactor(melanoma,levels=c(3,1,2,4)) <-  thickcat4.2~thickcat4
dlevel(melanoma,~ "thickcat4*")
dtable(melanoma,~thickcat4+thickcat4.2)
```

Combine levels but now control factor-level names 

```{r}
melanoma=drelevel(melanoma,gthick3~thickcat4,newlevels=list(group1.2=1:2,group3.4=3:4))
dlevels(melanoma)
```

Making a factor from existing numeric variable and vice versa
==============================================================

A numeric variable "status" with values 1,2,3 into a factor by

```{r}
data(melanoma)
melanoma = dfactor(melanoma,~status, labels=c("malignant-melanoma","censoring","dead-other"))
melanoma = dfactor(melanoma,sexl~sex,labels=c("females","males"))
dtable(melanoma,~sexl+status.f)
```

A gender factor with values "M", "F" can be converted into numerics by

```{r}
melanoma = dnumeric(melanoma,~sexl)
dstr(melanoma,"sex*")
dtable(melanoma,~'sex*',level=2)
```

SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Analysis of bivariate binomial data: Twin analysis"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Analysis of bivariate binomial data: Twin analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Overview 
==========

When looking at bivariate binomial data with the aim of learning about the 
dependence that is present, possibly after correcting for some covariates many
models are available. 

   *  Random-effects models logistic regression covered elsewhere (glmer in lme4).

in the mets package you can fit the 

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.


Typically it can be hard or impossible 
to specify random effects models with special 
structure among the parameters of the random effects. This is possible in
our models. 

To be concrete about the model structure assume that we have paired binomial 
data $(Y_1, Y_2, X_1, X_)2$ where the responses are $Y_1, Y_2$ and we
have covariates $X_1, X_2$.

We start by giving a brief description of these different models.  First we
for bivariate data one can specify the marginal probability using logistic 
regression models 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2.
\]
These model can be estimated under working independence 
\cite{zeger-liang-86}.  


A typical twin analysis will typically consist of  looking at both 

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
  
The additive gamma can be used for the same as the bivariate probit model but 
is more restrictive in terms of dependence structure, but is nevertheless 
still valuable to have also as a check of results of the bivariate probit
model. 


Biprobit with random effects
=============================

For these model we assume that given random effects $Z$ and a covariate vector 
$V_{12}$ we have independent logistic regression models 
\[
probit(P(Y_i=1|X_i, Z)) = \alpha_i + X_i^T \beta + V_{12}^T Z  i=1,2.
\]
where  Z$  is a bivariate normal distribution with some covariance 
$\Sigma$. The general covariance structure 
$\Sigma$ makes the model very flexible. 

We note that 

 - Paramters $\beta$  are subject specific
 - The $\Sigma$ will reflect dependence


The more standard link function $logit$ rather than the $probit$ link
is often used and implemented in for example 
\cite{mm}.  The advantage is that one now gets an odds-ratio interpretation 
of the subject specific effects, but one then needs  numerical integration to
fit the model. 

Pairwise odds ratio model 
------------------------


Now the pairwise odds ratio model the specifies that given $ X_1, X_2 $
the marginal models are 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

The primary object of interest are the odds ratio between $Y_{1}$ and $Y_{2}$
\[
\gamma_{12} = \frac{ P(  Y_{ki} =1 , Y_{kj} =1) P(  Y_{ki} =0 , Y_{kj} =0) }{ 
  P(  Y_{ki} =1 , Y_{kj} =0) P(  Y_{ki} =0 , Y_{kj} =1) }
\]
given $X_{ki}$, $X_{kj}$, and $Z_{kji}$. 

We model the odds ratio with the regression 
\[
\gamma_{12} = \exp( Z_{12}^T \lambda)
\]
Where $Z_{12}$ are some covarites that may influence the odds-ratio 
between between $Y_{1}$ and $Y_{2}$ and contains the marginal covariates,
\cite{carey-1993,dale1986global,palmgren1989,molenberghs1994marginal}. 
This odds-ratio is given covariates as well as marginal covariates. 
The odds-ratio and marginals specify the joint bivariate distribution via
the so-called Placckett-distribution. 

One way of fitting this model is the ALR algoritm, the alternating 
logistic regression ahd this has been described in several papers
\cite{kuk2004permutation,kuk2007hybrid,qaqish2012orthogonalized}.
We here simply estimate the parameters in a two stage-procedure

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

This gives efficient estimates of the dependence parameters because of
orthogonality, but some efficiency may be gained for the marginal parameters 
by using the full likelihood or iterative fitting such as for the ALR. 


The pairwise odds-ratio model is very useful, but one do not have a random 
effects model. 


Additive gamma model 
----------------------

Again we operate under  marginal logistic regression models are 
\[
logit(P(Y_i=1|X_i)) = \alpha_i + X_i^T \beta  i=1,2
\]

First with just one random effect $Z$ we assume that  conditional
on $Z$ the responses are independent  and follow the model 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )  
\]
where $\Psi$ is the laplace transform of $Z$ where we assume that
$Z$ is gamma distributed with variance $\lambda_{\bullet}^{-1}$ and mean 1. 
In general $\Psi(\lambda_1,\lambda_2)$ is the laplace transform of a Gamma distributed random 
effect with $Z$ with mean $\lambda_1/\lambda_2$ and variance $\lambda_1/\lambda_2^2$.

We fit this model by 

 * Estimating the marginal parameters via GEE
 * Using marginal estimates, estimate dependence parameters

To deal with multiple random effects we consider random effects 
$Z_i  i=1,...,d$   such that  $Z_i$ is gamma distributed with 
 * mean: $\lambda_j/\lambda_{\bullet}$ 
 * variance: $\lambda_j/\lambda_{\bullet}^2$, where we define the scalar $\lambda_{\bullet}$ below. 

Now given a cluster-specific design vector $V_{12}$ we assume that 
\[
V_{12}^T Z
\]
is gamma distributed with mean 1 and variance $\lambda_{\bullet}^{-1}$ 
such that critically the random effect variance is the same for all clusters.
That is 
\[
 \lambda_{\bullet} = V_{12}^T (\lambda_1,...,\lambda_d)^T 
\]
We return to some specific models below, and show how to fit the ACE and AE 
model using this set-up. 

One last option in the model-specification is to specify how the 
parameters $\lambda_1,...,\lambda_d$ are related. We thus can specify a 
matrix $M$ of dimension $p \times d$ such that 
\[
 (\lambda_1,...,\lambda_d)^T  = M \theta
\]
where $\theta$ is d-dimensional.  If $M$ is diagonal we have no 
restrictions on parameters. 

This parametrization is obtained with the var.par=0 option that thus estimates
$\theta$.

The DEFAULT parametrization instead estimates the variances of the random effecs (var.par=1)
via the parameters $\nu$ 
\[
 M \nu = ( \lambda_1/\lambda_{\bullet}^2, ...,\lambda_d/\lambda_{\bullet}^2)^T
\]


The basic modelling assumption is now that given random effects 
$Z=(Z_1,...,Z_d)$ we have independent probabilites 
\[
logit(P(Y_i=1|X_i,Z)) = exp( -V_{12,i}^T Z \cdot \Psi^{-1}(\lambda_{\bullet},\lambda_{\bullet},P(Y_i=1|X_i)) )   i=1,2
\]

We fit this model by 

 - Estimating the marginal parameters via GEE
 - Using marginal estimates, estimate dependence parameters

Even though the model not formaly in this formulation allows negative 
correlation in practice the paramters can be negative and this reflects
negative correlation. An advanatage is that no numerical integration is 
needed. 


The twin-stutter data
------------------------

We consider the twin-stutter where for pairs of twins that are 
either dizygotic or monozygotic we have recorded whether the twins
are stuttering \cite{twinstut-ref}

We here consider MZ and same sex DZ twins. 

Looking at the data 

```{r}
library(mets)
data(twinstut)
twinstut$binstut <- 1*(twinstut$stutter=="yes")
twinsall <- twinstut
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
head(twinstut)
```


Pairwise odds ratio model 
-------------------------

We start by fitting an overall dependence OR for both MZ and DZ even though 
the dependence is expected to be different across zygosity.

The first step is to fit the marginal model adjusting for marginal covariates. 
We here note that there is a rather strong gender effect in the risk of
stuttering. 

```{r}
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
summary(margbin)
```

Now estimating the OR parameter. We see a strong dependence with an OR
at around 8 that is clearly significant. 

```{r twostage1}
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
                       clusters=twinstut$tvparnr,detail=0)
summary(bina)
```

Now, and more interestingly, we consider an OR that depends on zygosity and
note that MZ have a much larger OR than DZ twins. This type of trait is 
somewhat complicated to interpret, but clearly, one option is that 
that there is a genetic effect, alternatively there might be
a stronger environmental effect for MZ twins. 


```{r twostage2}
# design for OR dependence 
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bin)
```


We now consider further regression modelling of the OR structure by
considering possible interactions between sex and zygozsity.
We see that MZ has a much higher dependence and that males have
a much lower dependence. We tested for interaction in this model and 
these were not significant. 
     
```{r twostage3}
twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+factor(sex),data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
                          clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)
```

Alternative syntax 
---------------------

We now demonstrate how the models can fitted jointly and with anohter
syntax, that ofcourse just fits the marginal model and subsequently fits
the pairwise OR model. 

First noticing as before that MZ twins have a much higher dependence. 
     
```{r altsyntax1}
 # refers to zygosity of first subject in eash pair : zyg1
 # could also use zyg2 (since zyg2=zyg1 within twinpair's)
 out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
                response="binstut",id="tvparnr",var.link=1,
                theta.formula=~-1+factor(zyg1))
summary(out)
```


Now considering all data and estimating separate effects for the OR
for opposite sex DZ twins and same sex twins. 
We here find that os twins are not markedly different from the same sex DZ 
twins. 
     
```{r osdesign}
 # refers to zygosity of first subject in eash pair : zyg1
 # could also use zyg2 (since zyg2=zyg1 within twinpair's))
 
 desfs<-function(x,num1="zyg1",num2="zyg2")
         c(x[num1]=="dz",x[num1]=="mz",x[num1]=="os")*1
     
 margbinall <- glm(binstut~factor(sex)+age,data=twinsall,family=binomial())
 out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
       data=twinsall,response="binstut",id="tvparnr",var.link=1,
       theta.formula=desfs,desnames=c("dz","mz","os"))
 summary(out3)
```

Bivariate Probit model 
------------------------

```{r}
library(mets)
data(twinstut)
twinstut <- subset(twinstut,zyg%in%c("mz","dz"))
twinstut$binstut <- 1*(twinstut$stutter=="yes")
head(twinstut)
```

First testing for same dependence in MZ and DZ  that we recommend doing by 
comparing the correlations of MZ and DZ twins. Apart from regression 
correction in the mean this is an un-structured model, and the useful
concordance and casewise concordance estimates can be reported from this
analysis. 

```{r biprobit1}
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="un")
summary(b1)
```


Polygenic modelling 
-------------------

   We now turn attention to specific polygenic modelling where special random 
   effects are used to specify ACE, AE, ADE models and so forth. This is very
   easy with the bptwin function. The key parts of the output are the sizes of 
   the genetic component A and the environmental component, and we can compare 
   with the results of the unstructed model above. Also formally we can test 
   if this submodel is acceptable by a likelihood ratio test. 


```{r bptwin1}
b1 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ace")
summary(b1)
```


```{r bptwin2}
b0 <- bptwin(binstut~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="ae")
summary(b0)
```

Additive gamma random effects 
-----------------------------

Fitting first a model with different size random effects for MZ and DZ. We 
note that as before in the OR and biprobit model the dependence is much
stronger for MZ twins. We also test if these are the same by parametrizing the
OR model with an intercept. This clearly shows a significant difference. 


```{r addgamma1}
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial.twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)

# test for same dependence in MZ and DZ 
theta.des <- model.matrix( ~factor(zyg),data=twinstut)
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin <- binomial.twostage(margbin,data=twinstut,model="gamma",
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=1,
     theta.des=theta.des)
summary(bintwin)
```

Polygenic modelling 
-----------------------

   First setting up the random effects design for the random effects and 
   the the relationship between variance parameters.
   We see that the genetic random effect has size one for MZ and 0.5 for DZ subjects, 
   that have shared and non-shared genetic components with variance 0.5 such that the total 
   genetic variance is the same for all subjects. The shared environmental effect is the samme for 
all. Thus two parameters with these bands. 

```{r polygenic1}
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ace")
head(cbind(out$des.rv,twinstut$tvparnr),10)
out$pardes
```


Now, fitting the ACE model, we see that the variance of the genetic, 
component, is 1.5 and the environmental variance is -0.5. Thus suggesting that 
the ACE model does not fit the data.  When the random design is given we 
automatically use the gamma fralty model. 

```{r polygenic2}
margbin <- glm(binstut~sex,data=twinstut,family=binomial())
bintwin1 <- binomial.twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin1)
```


For this model we estimate the concordance and casewise concordance as well 
as the marginal rates of stuttering for females. 

```{r}
concordanceTwinACE(bintwin1,type="ace")
```


The E component was not consistent with the fit of the data and we
now consider instead the AE model. 


```{r polygenic_ae}
out <- twin.polygen.design(twinstut,id="tvparnr",zygname="zyg",zyg="dz",type="ae")

bintwin <- binomial.twostage(margbin,data=twinstut,
     clusters=twinstut$tvparnr,detail=0,theta=c(0.1)/1,var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin)
```

Again, the concordance can be computed: 

```{r}
concordanceTwinACE(bintwin,type="ae")
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Cumulative Incidence Regression"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5    
vignette: >
  %\VignetteIndexEntry{Cumulative Incidence Regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

The cifreg function can fit the Fine-Gray model and the logit-link cumulative incidence function for the 
cause of interest for competing risks, and is completely scalable, that is, linear in the data. This includes
computation of standard errors that is also linear in data. In addition for the Fine-Gray model 
predictions can be provided standard errors for specific time-points based on influence functions 
for the baseline and the regression coeficients.  

To summarize

  - the baseline can be stratified 
  - the censoring weights can be strata dependent
  - predictions can be computed with standard errors (only for Fine-Gray)
  - computation time linear in data
     - including standard errors
  - only Fine-Gray: influence functions of baseline and regression coefficients computed and given by IC, iid and iidBaseline functions
  - clusters can be given and then cluster corrected standard errors are computed




Fine-Gray model
================

\cite{fine:gray:1999} considered a cumulative incidence on the form 
\begin{align*}
 F_1(t,X) & = P(T \leq t, \epsilon=1) = 1 - exp( - \Lambda_0(t) \exp(X^T \beta)).
\end{align*}

In the case of independent right-censoring with the censoring distribtion $G_c(t,X) = P(C > t | S(X))$ where
$S(X)$  is a set of strata defined from $X$, then an ubiased estimating equation is given by 
\begin{align*}
  U^{FG}_{n}(\beta) = \sum_{i=0}^{n} \int_0^{+\infty} \left( X_i- E_n(t,\beta) \right) w_i(t,X_i) dN_{1,i}(t)  \text{ where } 
	E_n(t,\beta)=\frac{\tilde S_1(t,\beta) }{\tilde S_0(t,\beta)},
\end{align*}
with \(w_i(t,X_i) = \frac{G_c(t,X_i)}{G_c(T_i \wedge t,X_i)} I( C_i > T_i \wedge t ) \) 
,$\tilde S_k(t,\beta) = \sum_{j=1}^n X_j^k \exp(X_j^T\beta) Y_{1,j}(t)$
for $k=0,1$, and with with $\tilde Y_{1,i}(t) = Y_{1,i}(t) w_i(t,X_i)$ for $i=1,...,n$.
$w_i(t)$ needs to be replaced by an estimator of the 
censoring distribution, and since it does not depend on $X$ the 
$\hat w_i(t) = \frac{\hat G_c(t,X_i)}{\hat G_c(T_i \wedge t,X_i)} I(C_i > T_i \wedge t)$ 
where $\hat G_c$ is the Kaplan-Meier estimator of the censoring distribution. 

First we simulate some competing risks data using some utility functions.

We simulate data with two causes based on the Fine-Gray model:
\begin{align}
F_1(t,X) &  = P(T\leq t, \epsilon=1|X)=( 1 - exp(-\Lambda_1(t) \exp(X^T \beta_1))) \\
F_2(t,X) & = P(T\leq t, \epsilon=2|X)= ( 1 - exp(-\Lambda_2(t) \exp(X^T \beta_2))) \cdot (1 - F_1(\infty,X))  
\end{align}
where the baselines are given as $\Lambda_j(t) = \rho_j (1- exp(-t/\nu_j))$ for $j=1,2$, and the 
$X$ being two independent binomials. 
Alternatively, one can also replace the FG-model with a logistic link 
$\mbox{expit}( \Lambda_j(t) + \exp(X^T \beta_j))$.

The advantage of the model is that it is easy to fit and to get standard errors, and that it is
quite flexible essentially being a Cox-model. On the downside 
is that the coefficients must be interpreted on $cloglog$-scale. 
Specifically, 
\begin{align}
\log(-\log( 1-F_1(t,X_1+1,X_2))) - \log(-\log( 1-F_1(t,X_1,X_2))) & =  \beta_1,
\end{align}
so the effect of an increase in $X_1$ is $\beta_1$ and leads to $1-F_1(t,X)$ on the $cloglog$ scale. 


```{r}
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in simulatins for p-values below.

 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 ## beta1=c(0.0,-0.1); beta2=c(-0.5,0.3)
 dats <- simul.cifs(n,rho1,rho2,beta,rc=0.5,rate=7)
 dtable(dats,~status)
 dsort(dats) <- ~time
```

We have a look at the non-parametric cumulative incidence curves

```{r}
 par(mfrow=c(1,2))
 cifs1 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=1)
 plot(cifs1)

 cifs2 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=2)
 plot(cifs2)
```

Now fitting the Fine-Gray model 

```{r}
 fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(fg)

 dd <- expand.grid(Z1=c(-1,1),Z2=0:1)
 pfg <- predict(fg,dd)
 plot(pfg,ylim=c(0,0.2))
```

and GOF based on cumulative residuals (Li et al. 2015) 

```{r}
gofFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
```

showing no problem with the proportionality of the model. 


SE's for the baseline and predictions of FG
===========================================

The standard errors reported for the FG-estimator are based on the i.i.d 
decompostion (influence functions) of the estimator that we give later. A similar decompostion
exist for the baseline and is needed when standard errors of predictions are computed. These
are a bit harder to compute for all time-points simultaneously, but they can be
obtained for specific timepoints jointly with the iid decomposition of the regression coefficients and
then used to get standard errors for predictions. 

We here plot the predictions with jittered confidence intervals for the predictions
at time point 5

```{r,pred-cif}
### predictions with CI based on iid decomposition of baseline and beta
fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
Biid <- iidBaseline(fg,time=5)
pfgse <- FGprediid(Biid,dd)
pfgse
plot(pfg,ylim=c(0,0.2))
for (i in 1:4) lines(c(5,5)+i/10,pfgse[i,3:4],col=i,lwd=2)
```

The iid decompostions are stored inside Biid, in addition we note that the 
iid decompostions for $\hat \beta - \beta_0$ are obtained by the command iid()

# Comparison 

We compare with the cmprsk function, that gives exactly the same, but without running it to avoid dependencies: 

```{r,cmprsk}
run <- 0
if (run==1) {
library(cmprsk)
mm <- model.matrix(~Z1+Z2,dats)[,-1]
cr <- with(dats,crr(time,status,mm))
cbind(cr$coef,diag(cr$var)^.5,fg$coef,fg$se.coef,cr$coef-fg$coef,diag(cr$var)^.5-fg$se.coef)
#          [,1]      [,2]       [,3]      [,4]          [,5]          [,6]
# Z1  0.6968603 0.3876029  0.6968603 0.3876029 -2.442491e-15 -2.553513e-15
# Z2 -0.8592892 0.6245258 -0.8592892 0.6245258 -2.997602e-15  1.776357e-15
}
```


When comparing with the results from the coxph based on setting up the data using
the finegray function, we get the same estimates but note that the standard errors
of the coxph is missing a term and therefore slightly different. When comparing to 
the estimates from coxph missing the additional censoring term we see that we 
get also the same standard errors

```{r,coxph}
if (run==1) {
 library(survival)
 dats$id <- 1:nrow(dats)
 dats$event <- factor(dats$status,0:2, labels=c("censor", "death", "other"))
 fgdats <- finegray(Surv(time,event)~.,data=dats)
 coxfg <- survival::coxph(Surv(fgstart, fgstop, fgstatus) ~ Z1+Z2 + cluster(id), weight=fgwt, data=fgdats)

 fg0 <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1,propodds=NULL)
 cbind( coxfg$coef,fg0$coef, coxfg$coef-fg0$coef)
#          [,1]       [,2]          [,3]
# Z1  0.6968603  0.6968603 -1.110223e-16
# Z2 -0.8592892 -0.8592892 -1.110223e-15
 cbind(diag(coxfg$var)^.5,fg0$se.coef,diag(coxfg$var)^.5-fg0$se.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3876029  0.0013099915
# [2,] 0.6241225 0.6245258 -0.0004033148
 cbind(diag(coxfg$var)^.5,fg0$se1.coef,diag(coxfg$var)^.5-fg0$se1.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3889129 -2.331468e-15
# [2,] 0.6241225 0.6241225  2.553513e-15
}
```

We also remove all censorings from the data to compare the estimates with those 
based on coxph, and observe that the estimates as well as the standard errors agree 

```{r,no-cens}
datsnc <- dtransform(dats,status=2,status==0)
dtable(datsnc,~status)
datsnc$id <- 1:n
datsnc$entry <- 0
max <- max(dats$time)+1
## for cause 2 add risk interaval 
datsnc2 <- subset(datsnc,status==2)
datsnc2 <- transform(datsnc2,entry=time)
datsnc2 <- transform(datsnc2,time=max)
datsncf <- rbind(datsnc,datsnc2)
#
cifnc <- cifreg(Event(time,status)~Z1+Z2,data=datsnc,cause=1,propodds=NULL)
cc <- phreg(Surv(entry,time,status==1)~Z1+Z2+cluster(id),datsncf)
cbind(cc$coef-cifnc$coef, diag(cc$var)^.5-diag(cifnc$var)^.5)
#            [,1]          [,2]
# Z1 1.332268e-15 -4.440892e-16
# Z2 4.218847e-15  2.220446e-16
```

the cmprsk also gives the same 

```{r}
if (run==1) {
 library(cmprsk)
 mm <- model.matrix(~Z1+Z2,datsnc)[,-1]
 cr <- with(datsnc,crr(time,status,mm))
 cbind(cc$coef-cr$coef, diag(cr$var)^.5-diag(cc$var)^.5)
#             [,1]         [,2]
# Z1 -4.218847e-15 1.443290e-15
# Z2  7.549517e-15 1.110223e-16
}
```


# Strata dependent Censoring weights 

We can improve efficiency and avoid bias by allowing the censoring weights to depend on 
the covariates 

```{r}
 fgcm <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1,cens.model=~strata(Z1,Z2))
 summary(fgcm)
 summary(fg)
```

We note that the standard errors are slightly smaller for the more efficient estimator.

The influence functions of the FG-estimator is given by \cite{fine:gray:1999},  
\begin{align*}
\phi_i^{FG}  & = \int (X_i- e(t))  \tilde w_i(t) dM_{i1}(t,X_i)  + \int \frac{q(t)}{\pi(t)} dM_{ic}(t),  \\
             & = \phi_i^{FG,1}  + \phi_i^{FG,2},
\end{align*}
where the first term is what would be achieved for a known censoring distribution, and the second term is
due to the variability from the Kaplan-Meier estimator. 
Where $M_{ic}(t) = N_{ic}(t) - \int_0^t Y_i(s) d\Lambda_c (s)$
with $M_{ic}$ the standard censoring martingale. 

The function $q(t)$ that reflects that the censoring only 
affects the terms related to cause "2" jumps, can be written as (see Appendix B2)
\begin{align*} 
	q(t) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(C > T)/G_c(T)) = E( H(t,X) F_2(t,X) ), 
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s) d \Lambda_1(s,X)$ and since $\pi(t)=E(Y(t))=S(t) G_c(t)$.

In the case where the censoring weights are stratified (based on $X$)
we get the influence functions related to the censoring term with 
\begin{align*} 
	q(t,X) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(T < C)/G_c(T,X) | X) = H(t,X) F_2(t,X), 
\end{align*}
so that the influence function becomes 
\begin{align*}
\int (X-e(t)) w(t) dM_1(t,X) +  \int H(t,X) \frac{F_2(t,X)}{S(t,X)} \frac{1}{G_c(t,X)} dM_c(t,X).
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s,X) d \Lambda_1(s,X)$.


Augmenting the FG-estimator 
===========================

Rather than using a larger censoring model we can also compute the augmentation term directly and
then fit the FG-model based on this augmentation term and do a couple of iterations 

```{r}
  fgaugS <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fg$E)
  summary(fgaugS)

  fgaugS2 <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fgaugS$E)
  summary(fgaugS2)

  fgaugS3 <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fgaugS2$E)
  summary(fgaugS3)
```
Again we note slightly smaller standard errors when augmenting the estimator. 

The function \verb+FG_AugmentCifstrata+ 
compute the augmentation term for fixed $E(t)$ based on the current $\hat \beta$
\begin{align*}
 U_n^{A} = \sum_{i=1}^n \int_{0}^{+\infty}  \frac{F_2(t,X_i)}{S(t,X_i)G_c(t,X_i)}  
	H(t,X_i,E,G_c,\Lambda_1)  dM_{ci}(t)
\end{align*}
using working models based on stratification to get $F_1^s$ and $F_2^s$ where the strata are given by
$strata()$ in the call.  Then fits the FG model so solve the 
\begin{align*}
 U_n^{A}(\beta_p)  + U^{FG}_{n}(\beta) = 0. 
\end{align*}

Then we may iterate to get a solution to the augmented score equation
\begin{align*}
 U_n^{A}(\beta_\infty)  + U^{FG}_{n}(\beta_\infty) = 0.
\end{align*}

The censoring model here is one overall Kaplan-Meier. 

The influence funtion for the augmented estimator is 
\begin{align*}
\int (X-e(t)) w(t) dM_1(t,X) +  \int H(t,X) \frac{F_2(t,X)}{S(t,X)} \frac{1}{G_c(t)} dM_c.
\end{align*}
and standard errors are based on this formula. 

Logistic-link 
================

```{r}
 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 dats <- simul.cifs(n,rho1,rho2,beta,rc=0.5,rate=7,type="logistic")
 dtable(dats,~status)
 dsort(dats) <- ~time
```

The model where 
\begin{align*}
 \mbox{logit}(F_1(t,X)) & =  \alpha(t) + X^T \beta
\end{align*}
that then leads to OR interpretation of the $F_1$, can also be fitted easily, however, the standard errors
are harder to compute and only approximative 
(assuming that the censoring weights are known) but this gives typically only a small error. 
In the ${{\bf timereg}}$-package the model can be 
fitted using different estimators that are more efficient using different weights but this is much slower.  

Fitting the model and getting OR's 

```{r}
 or <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(or)
```


Administrative Censoring
===========================

In the case with administrative censoring we can simply provide the 
risk-set given by the administrative censoring times for the Fine-Gray 
or logistic link cumulative incidence regression models. 

```{r}
library(mets)
rho1 <- 0.3; rho2 <- 5.9
set.seed(100)
n <- 100
beta=c(0.3,-0.3,-0.5,0.3)
rc <- 0.9
###
dats <- mets:::simul.cifsRA(n,rho1,rho2,beta,bin=1,rc=rc,rate=c(3,7))
dats$status07 <- dats$status
dats$status07[dats$status %in% c(0,7)] <- 0
tt <- seq(0,6,by=0.1)
base1 <- rho1*(1-exp(-tt/3))

ccA  <-  cifregFG(Event(timeA,statusA)~Z1+Z2,dats,
		  adm.cens.time=dats$censorA,no.codes=7)
estimate(ccA)
```

The Fine-Gray model can similarly be estimated using the modified risk-set and the phreg function

```{r}
dats$entry <- 0
dats$id <- 1:n
datA <- dats
datA2 <- subset(datA,statusA==2)
datA2$entry <- datA2$timeA
datA2$timeA <- datA2$censorA
datA2$statusA <- 0
datA <- rbind(datA,datA2)
ddA <- phreg(Event(entry,timeA,statusA==1)~Z1+Z2+cluster(id),datA)
estimate(ddA)

## also checking the cumulative baseline 
###plotl(tt,base1) 
###plot(ccA,add=TRUE,col=3)
###plot(ddA,col=2,add=TRUE)
```



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




The `resmeanATE` or `resmeanIPCW`/`rmstIPCW` functions fit an exponential or linear link model with
IPCW adjustment at a specific time point for the restricted mean survival or years lost in
competing risks. The computation is linear in the data size, making it suitable for large datasets.
Influence functions are computed and available for downstream inference.

Key features:

  - censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument



RMST 
====

Regression for rmst outcome $E(T \wedge t | X) = exp(X^T \beta)$ based on IPCW adjustment for censoring, thus solving
the estimating equation 
\begin{align*}
 & X^T [ (T \wedge t) \frac{I(C > T \wedge t)}{G_C(T \wedge t,X)} - exp(X^T \beta) ] = 0 .
\end{align*}
This is done with the resmeanIPCW function. 
For a fully saturated model with a complete censoring model this is equal to the integral of the Kaplan-Meier estimator, as
illustrated below.

We can also compute the integral of the Kaplan-Meier or Cox-based survival estimator to
get the RMST (using the `resmean_phreg` function):
\[ \int_0^t S(s|X) ds. \]

For competing risks, the years lost can be computed via cumulative incidence functions (`cif_yearslost`)
or as an IPCW estimator, since
\[ E( I(\epsilon=1) ( t - T \wedge t ) | X) = \int_0^t F_1(s) ds. \]
For a fully saturated model with a complete censoring model these estimators are equivalent, as illustrated below.



``` r
set.seed(101)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
#>    n events
#>  408    245
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  3.014321  0.065114  2.886700  3.141941  0.0000
#> tcell        0.106526  0.138268 -0.164473  0.377525  0.4410
#> platelet     0.247103  0.097337  0.056325  0.437880  0.0111
#> age         -0.185936  0.043566 -0.271324 -0.100548  0.0000
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 20.37524 17.93403 23.1488
#> tcell        1.11241  0.84834  1.4587
#> platelet     1.28031  1.05794  1.5494
#> age          0.83033  0.76237  0.9043
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
#>                        Estimate Std.Err  2.5% 97.5%   P-value
#> inttcell=0, platelet=0    13.60  0.8316 11.97 15.23 3.826e-60
#> inttcell=0, platelet=1    18.90  1.2696 16.41 21.39 3.997e-50
#> inttcell=1, platelet=0    16.19  2.4061 11.48 20.91 1.705e-11
#> inttcell=1, platelet=1    17.77  2.4536 12.96 22.58 4.463e-13
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean_phreg(out1,times=30)
     summary(rm1)
#>                     strata times    rmean  se.rmean    lower    upper
#> tcell=0, platelet=0      0    30 13.60295 0.8315418 12.06700 15.33439
#> tcell=0, platelet=1      1    30 18.90127 1.2693263 16.57021 21.56026
#> tcell=1, platelet=0      2    30 16.19121 2.4006185 12.10806 21.65131
#> tcell=1, platelet=1      3    30 17.76610 2.4421987 13.57008 23.25956
#>                     years.lost
#> tcell=0, platelet=0   16.39705
#> tcell=0, platelet=1   11.09873
#> tcell=1, platelet=0   13.80879
#> tcell=1, platelet=1   12.23390
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
#>                        Estimate Std.Err   2.5%  97.5%   P-value
#> inttcell=0, platelet=0   12.105  0.8508 10.438 13.773 6.162e-46
#> inttcell=0, platelet=1    6.884  1.1741  4.583  9.185 4.536e-09
#> inttcell=1, platelet=0    7.261  2.3533  2.648 11.873 2.033e-03
#> inttcell=1, platelet=1    5.780  2.0925  1.679  9.882 5.737e-03
     ## same as integrated cumulative incidence 
     rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)
#> $estimate
#> $estimate$intF_1
#>                     strata times    intF_1 se.intF_1 lower_intF_1 upper_intF_1
#> tcell=0, platelet=0      0    30 12.105125 0.8508102    10.547328    13.893001
#> tcell=0, platelet=1      1    30  6.884171 1.1740988     4.928092     9.616665
#> tcell=1, platelet=0      2    30  7.260755 2.3532867     3.846790    13.704561
#> tcell=1, platelet=1      3    30  5.780369 2.0924946     2.843285    11.751429
#> 
#> $estimate$intF_2
#>                     strata times   intF_2 se.intF_2 lower_intF_2 upper_intF_2
#> tcell=0, platelet=0      0    30 4.291930 0.6161439     3.239330     5.686565
#> tcell=0, platelet=1      1    30 4.214556 0.9057028     2.765857     6.422055
#> tcell=1, platelet=0      2    30 6.548034 1.9703340     3.630616    11.809770
#> tcell=1, platelet=1      3    30 6.453535 2.0815225     3.429661    12.143507
#> 
#> 
#> $total.years.lost
#> [1] 16.39705 11.09873 13.80879 12.23390

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)
```

<img src="figure/rmst-ate-unnamed-chunk-2-1.png" alt="Restricted mean survival and years lost by cause."  />

Based on the output from the IPCW estimators we can derive any measure of interest


``` r
estimate(out)
#>                        Estimate Std.Err   2.5%  97.5%   P-value
#> inttcell=0, platelet=0   12.105  0.8508 10.438 13.773 6.162e-46
#> inttcell=0, platelet=1    6.884  1.1741  4.583  9.185 4.536e-09
#> inttcell=1, platelet=0    7.261  2.3533  2.648 11.873 2.033e-03
#> inttcell=1, platelet=1    5.780  2.0925  1.679  9.882 5.737e-03

measures <- function(p) {
 ratio1 <- p[1]/p[2]; ratio2 <- p[2]/p[1]; dif1 <- p[4]-p[1]; dif2 <- p[3]-p[1]
 m <- c(dif1,dif2,ratio1,ratio2)
 return(m)
}

 labs <- c("dif4-1","dif3-1","ratio 1/2","ratio 2/1")
 estimate(out,f=measures,labels=labs)
#>           Estimate Std.Err     2.5%    97.5%   P-value
#> dif4-1     -6.3248  2.2589 -10.7520 -1.89748 5.111e-03
#> dif3-1     -4.8444  2.5024  -9.7489  0.06018 5.288e-02
#> ratio 1/2   1.7584  0.3244   1.1227  2.39414 5.924e-08
#> ratio 2/1   0.5687  0.1049   0.3631  0.77431 5.924e-08
```

Average treatment effect
=========================

This section computes the average treatment effect for restricted mean survival and years lost in the competing risks setting.


``` r
 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
#>    n events
#>  408    241
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.852670  0.062493  2.730187  2.975153  0.0000
#> tcell1       0.021401  0.122907 -0.219493  0.262295  0.8618
#> platelet     0.303489  0.090758  0.125606  0.481372  0.0008
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 17.33400 15.33575 19.5926
#> tcell1       1.02163  0.80293  1.2999
#> platelet     1.35458  1.13383  1.6183
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.26223  0.95926 17.38212 21.14234  0.0000
#> treat1    19.67890  2.22777 15.31255 24.04526  0.0000
#> treat:1-0  0.41667  2.41067 -4.30815  5.14150  0.8628
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    19.28131  0.95807 17.40352 21.15910  0.0000
#> treat1    20.34466  2.54101 15.36438 25.32494  0.0000
#> treat:1-0  1.06335  2.70979 -4.24773  6.37443  0.6948
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
		    treat.model=tcell~platelet)
 summary(out1)
#>    n events
#>  408    157
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  2.806295  0.069619  2.669845  2.942745  0.0000
#> tcell1      -0.374065  0.247669 -0.859488  0.111358  0.1310
#> platelet    -0.491736  0.164946 -0.815025 -0.168447  0.0029
#> 
#> exp(coeffients):
#>             Estimate     2.5%   97.5%
#> (Intercept) 16.54849 14.43773 18.9678
#> tcell1       0.68793  0.42338  1.1178
#> platelet     0.61156  0.44263  0.8450
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    14.53185  0.95707 12.65604 16.40767  0.0000
#> treat1     9.99693  2.37814  5.33587 14.65800  0.0000
#> treat:1-0 -4.53492  2.57516 -9.58213  0.51229  0.0782
#> 
#> Average Treatment effects (double robust) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     14.513742   0.958025  12.636048  16.391436  0.0000
#> treat1      9.365012   2.417032   4.627717  14.102307  0.0001
#> treat:1-0  -5.148730   2.597947 -10.240612  -0.056848  0.0475

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,time=40,
		    treat.model=tcell~platelet)
 summary(out2)
#>    n events
#>  408     84
#> 
#>  408 clusters
#> coeffients:
#>               Estimate    Std.Err       2.5%      97.5% P-value
#> (Intercept)  1.8266090  0.1312181  1.5694263  2.0837918  0.0000
#> tcell1       0.4751558  0.2403839  0.0040121  0.9462996  0.0481
#> platelet    -0.0090724  0.2168469 -0.4340845  0.4159397  0.9666
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  6.21278 4.80389 8.0349
#> tcell1       1.60826 1.00402 2.5762
#> platelet     0.99097 0.64786 1.5158
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     6.19518  0.71372  4.79631  7.59405  0.0000
#> treat1     9.96349  2.09256  5.86216 14.06482  0.0000
#> treat:1-0  3.76831  2.21654 -0.57602  8.11264  0.0891
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     6.20484  0.71392  4.80559  7.60410  0.0000
#> treat1    10.30065  2.21700  5.95542 14.64588  0.0000
#> treat:1-0  4.09581  2.32897 -0.46889  8.66050  0.0786
```



SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              KernSmooth_2.23-26    
#>  [7] otel_0.2.0             glue_1.8.1             future.apply_1.20.2   
#> [10] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [13] grid_4.6.0             evaluate_1.0.5         lifecycle_1.0.5       
#> [16] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [19] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [22] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [25] digest_0.6.39          pillar_1.11.1          parallelly_1.47.0     
#> [28] parallel_4.6.0         Matrix_1.7-5           tools_4.6.0           
#> [31] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "WIP: Cooking survival data, 5 minute recipes"
aouthor: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    toc: true
    # fig_width: 7.15
    # fig_height: 5.5        
header-includes: 
  - \usepackage{tikz}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Cooking survival data, 5 minutes recipes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"  
)
```

# Overview 

Simulation of survival data is important for both
theoretical and practical work. In a practical setting we might wish to
validate that standard errors are valid even in a rather small sample,
or validate that a complicated procedure is doing as intended. 
Therefore it is useful to have simple tools for generating survival data
that looks as much as possible like particular data. In a theoretical
setting we often are interested in evaluating the finite sample
properties of a new procedure in different settings that often are
motivated by a specific practical problem. The aim is 
provide such tools.

Bender et al. in a nice paper discussed how to generate
survival data based on the Cox model, and restricted attention to some
of the many useful parametric survival models (weibull, exponential).
We here use piecewise linear baseline functions that make it easy to simulate
data that follows closely the baseline given by the data using semi or 
nonparametric models. This makes it easy to capture important aspects of the
data. 

Different survival models can be cooked, and we here give recipes for
hazard and cumulative incidence based simulations. More recipes are
given in vignette about recurrent events. 

  - hazard based.
  - cumulative incidence.
  - recurrent events (see recurrent events vignette).


```{r}
 library(mets)
 options(warn=-1)
 set.seed(10) # to control output in simulations
```

# Hazard based, Cox models  


Given a survival time $T$ with cumulative hazard $\Lambda(t)=\int_0^t \lambda(s) ds$, it
follows that \cite{}
with $E \sim Exp(1)$ (exponential with rate 1), that $\Lambda^{-1}(E)$ will have the
same distribution as $T$.

This provides the basis for simulations of survival times with a given hazard  and is 
a consequence of this simple calculation
$$
  P(\Lambda^{-1}(E) > t) = P(E > \Lambda(t)) = \exp( - \Lambda(t)) = P(T > t).
$$

Similarly if $T$ given $X$ have hazard on Cox form 
$$
  \lambda_0(t) \exp( X^T \beta)
$$
where $\beta$ is a $p$-dimensional regression coefficient and $\lambda_0(t)$ a baseline 
hazard funcion, 
then it is useful to observe also that 
$\Lambda^{-1}(E/HR)$ with $HR=\exp(X^T \beta)$ has the same distribution as $T$ given $X$. 

Therefore if the inverse of the cumulative hazard can be computed we can generate survival with 
a specified hazard function. One useful observation is note that for a piecewise linear continuous
cumulative hazard on an interval $[0,\tau]$ $\Lambda_l(t)$ it is easy to compute the inverse. 

Further, we can approximate any cumulative hazard with a piecewise linear continous 
cumulative hazard and then simulate data according to this approximation. Recall that fitting
the Cox model to data will give a piecewise constant cumulative hazard and the regression coefficients so 
with these at hand we can first approximate the piecewise constant "Breslow"-estimator with a linear
upper (or lower bound) by simply connecting the values by 
straight lines. 

# Delayed entry 

If $T$ given $X$ have hazard  on Cox form 
$$
  \lambda_0(t) \exp( X^T \beta)
$$
and we wish to generate data according to this hazard for those that are alive at time $s$, that is 
draw from the distribution of $T$ given $T>s$ (all given $X$ ), then we note that  
$$
\Lambda_0^{-1}( \Lambda_0(s) + E/HR)) 
$$
with $HR=\exp(X^T \beta))$ and with $E \sim Exp(1)$ has the distributiion we are after. 

This is again a consequence of a simple calculation
$$
  P_X(\Lambda^{-1}(\Lambda(s)+ E/HR) > t) = P_X(E > HR( \Lambda(t) - \Lambda(s)) ) = P_X(T>t | T>s)
$$

The engine is to simulate data with a given linear cumulative hazard. First generating survival data based on
the cumulative hazard cumhaz:j

```{r}
 nsim <- 1000
 chaz <-  c(0,1,1.5,2,2.1)
 breaks <- c(0,10,   20,  30,   40)
 cumhaz <- cbind(breaks,chaz)
 X <- rbinom(nsim,1,0.5)
 beta <- 0.2
 rrcox <- exp(X * beta)
 
 pctime <- rchaz(cumhaz,n=nsim)
 pctimecox <- rchaz(cumhaz,rrcox)
```

Now looking at a simple cox model 

```{r}
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet+age,data=bmt)

 dd <- sim.phreg(cox1,n,data=bmt)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox1),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)

 ## changing the parameters 
 cox10 <- cox1
 cox10$coef <- c(0,0.4,0.3)
 dd <- sim.phreg(cox10,n,data=bmt)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~tcell+platelet+age,data=dd)
 cbind(coef(cox10),coef(scox1))
 par(mfrow=c(1,1))
 plot(scox1,col=2); plot(cox10,add=TRUE,col=1)
```


Multiple Cox models for cause specific hazards can be combined, and we start by drawing the 
covariates manually, below we just call the sim.phregs function that draws covariates from
the data, 

```{r}
 data(bmt); 
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

 X1 <- bmt[,c("tcell","platelet")]
 n <- nsim
 xid <- sample(1:nrow(X1),n,replace=TRUE)
 Z1 <- X1[xid,]
 Z2 <- X1[xid,]
 rr1 <- exp(as.matrix(Z1) %*% cox1$coef)
 rr2 <- exp(as.matrix(Z2) %*% cox2$coef)

 d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2)
 dd <- cbind(d,Z1)

 scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE,col=2)
 plot(cox2); plot(scox2,add=TRUE,col=2)
 cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)
```

Now fully nonparametric model with stratified baselines 

```{r}
 data(sTRACE)
 dtable(sTRACE,~chf+diabetes)
 coxs <-   phreg(Surv(time,status==9)~strata(diabetes,chf),data=sTRACE)
 strata <- sample(0:3,nsim,replace=TRUE)
 simb <- sim.phreg(coxs,nsim,data=NULL,strata=strata)

 cc <-   phreg(Surv(time,status)~strata(strata),data=simb)
 plot(coxs,col=1); plot(cc,add=TRUE,col=2)

 simb1 <- sim.phreg(coxs,nsim,data=sTRACE)
 cc1 <-   phreg(Surv(time,status)~strata(diabetes,chf),data=simb1)
 plot(cc1,add=TRUE,col=3)
```

We now fit cause-specific hazard models with 3 causes (censoring as one of them)
and generate competing risks data with hazards taken from the fitted Cox models. 
Here a situation with stratified baselines for some of the models:

```{stratified}
 ## r with phreg 
 cox0 <- phreg(Surv(time,cause==0)~tcell+platelet,data=bmt)
 cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
 coxs <- list(cox0,cox1,cox2)
 dd <- sim.phregs(coxs,n,data=bmt)

 ## checking that  cause specific hazards are as given, make n larger
 scox0 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
 scox1 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==3)~strata(tcell)+platelet,data=dd)
 cbind(cox0$coef,scox0$coef)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,3))
 plot(cox0); plot(scox0,add=TRUE,col=2); 
 plot(cox1); plot(scox1,add=TRUE,col=2); 
 plot(cox2); plot(scox2,add=TRUE,col=2); 
 
 ########################################
 ## second example 
 ########################################

 cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
 coxs <- list(cox1,cox2)
 dd <- sim.phregs(coxs,n,data=bmt)
 scox1 <- phreg(Surv(time,status==1)~strata(tcell)+platelet,data=dd)
 scox2 <- phreg(Surv(time,status==2)~tcell+strata(platelet),data=dd)
 cbind(cox1$coef,scox1$coef)
 cbind(cox2$coef,scox2$coef)
 par(mfrow=c(1,2))
 plot(cox1); plot(scox1,add=TRUE); 
 plot(cox2); plot(scox2,add=TRUE); 
```

 - sim.phreg for phreg, can deal with strata 
 - sim.phregs cause specific hazards on phreg form 


One more example  fully non-parametric 

```{r}
 library(mets)
 n <- nsim
 data(bmt)
 bmt$bmi <- rnorm(408)
 dcut(bmt) <- gage~age
 data <- bmt
 cox1 <- phreg(Surv(time,cause==1)~strata(tcell,platelet),data=bmt)
 cox2 <- phreg(Surv(time,cause==2)~strata(gage,tcell),data=bmt)
 cox3 <- phreg(Surv(time,cause==0)~strata(platelet)+bmi,data=bmt)
 coxs <- list(cox1,cox2,cox3)

 dd <- sim.phregs(coxs,n,data=bmt,extend=0.002)
 dtable(dd,~status)
 scox1 <- phreg(Surv(time,status==1)~strata(tcell,platelet),data=dd)
 scox2 <- phreg(Surv(time,status==2)~strata(gage,tcell),data=dd)
 scox3 <- phreg(Surv(time,status==3)~strata(platelet)+bmi,data=dd)
 cbind(coef(cox1),coef(scox1), coef(cox2),coef(scox2), coef(cox3),coef(scox3))
 par(mfrow=c(1,3))
 plot(scox1,col=2); plot(cox1,add=TRUE,col=1)
 plot(scox2,col=2); plot(cox2,add=TRUE,col=1)
 plot(scox3,col=2); plot(cox3,add=TRUE,col=1)
```



# Multistate models: The Illness Death model 

Using a hazard based simulation with delayed entry we can then simulate data 
from for example the general illness-death model. Here the cumulative hazards 
need to be specified.




\begin{tikzpicture}[
    >=Stealth,
    node distance=4cm,
    state/.style={
        rectangle,
        draw=black,
        thick,
        minimum width=3cm,
        minimum height=1cm,
        align=center
    }
]

  % Top states
  \node[state] (H) {Healthy \\ (1)};
  \node[state] (I) [right=of H] {Ill \\ (2)};

  % Dead centered below
  \node[state] (D) at ($(H)!0.5!(I) + (0,-3)$) {Dead \\ (3)};

  % Two straight arrows between Healthy and Ill
  \draw[->, thick] 
    ($(H.east)+(0,0.15)$) -- ($(I.west)+(0,0.15)$)
    node[midway, above] {$\lambda_{12}$};

  \draw[->, thick] 
    ($(I.west)+(0,-0.15)$) -- ($(H.east)+(0,-0.15)$)
    node[midway, below] {$\lambda_{21}$};

  % Death transitions
  \draw[->, thick] (H) -- node[left] {$\lambda_{13}$} (D);
  \draw[->, thick] (I) -- node[right] {$\lambda_{23}$} (D);

\end{tikzpicture}

We simply give the cumulative hazards for the different transitions to the function simMultistate
to simulate data from the model, subsequently we re-estimate the parameters based on the simulated
data to validate the procedure. 


```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical
 dr2 <- scalecumhaz(dr,1.5)
 cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

 iddata <- simMultistate(nsim,base1,base1,dr,dr2,cens=cens)
 dlist(iddata,.~id|id<3,n=0)
  
 ### estimating rates from simulated data  
 c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
 c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
 c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
 c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
 ###
 par(mfrow=c(2,2))
 plot(c0)
 lines(cens,col=2) 
 plot(c3,main="rates 1-> 3 , 2->3")
 lines(dr,col=1,lwd=2)
 lines(dr2,col=2,lwd=2)
 ###
 plot(c1,main="rate 1->2")
 lines(base1,lwd=2)
 ###
 plot(c2,main="rate 2->1")
 lines(base1,lwd=2)
 
```



# Cumulative incidence 

In this section we discuss how to simulate competing risks data that have a specfied cumulative 
incidence function.  We consider for simplicity a competing risks model with two causes and denote the
cumulative incidence curves as $F_1(t,X) = P(T < t, \epsilon=1|X)$ and  $F_2(t,X) = P(T < t, \epsilon=2|X)$.
Here given some covariate $X$.

To generate data with the required cumulative incidence functions a simple approach is to first figure out
if the subject dies and then from what cause, then finally draw the survival time according to the
conditional distribution.

For simplicity we consider survival times in a fixed interval $[0,\tau]$, and first flip a coin with 
and probabilities $1-F_1(\tau,X)-F_2(\tau,X)$ to decide if the subject is a survivor or dies. 
Then if subject dies we then flip a coin with probabilities $F_1(\tau,X)/(F_1(\tau,X)+F_2(\tau,X))$ and 
$F_2(\tau,X)/(F_1(\tau,X)+F_2(\tau,X))$ to decide if it is a cause $!$,  $\epsilon=1$, or a cause 2, $\epsilon=2$. 
Finally we draw the survival time using the cumulative incidence distribution.
The timing of a cause $j$ event is  thus 
$T = (\tilde F_1^{-1}(U,X)$ with $\tilde F_1(s,X) = F_1(s,X)/F_1(\tau,X)$ and $U$ is a uniform.

Then indeed $P(T \leq t, \epsilon=j|X) = F_j(t,X)$ for $j=1,2$.

We again note and use that if $\tilde F_j(s)$ and $F_j(s)$ are piecewise linear
continuous functions then the inverse is easy to compute. 


## Cumulative incidence I

We here simulate two causes of death with two binary covarites of logistic type
\begin{align*}
F_1(t,X)  &= \frac{ \Lambda_1(t,\rho_1) exp(X^T  \beta)}{1+\Lambda_1(t,\rho_1) exp(X^T  \beta)}
\end{align*}
and $F_2$ here enforcing the sum condition $F_1+F_2 \leq 1$
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta)} [ 1- F_1(\tau,X) ]
\end{align*}
or not 
\begin{align*}
F_2(t,X)  & =  \frac{ \Lambda_2(t,\rho_2) exp(X^T  \beta)}{1+\Lambda_2(t,\rho_2) exp(X^T  \beta)}
\end{align*}

The baselines are given as $\Lambda_j(t) = \rho_1 (1- exp(-t/r_j))$ where $\rho_j$ 
and $r_j$ are postive constants, and here $\tau=6$. 

To simulate the survival time we use a piecwise linear approximation of the cumulative
incidence functions and will thus depends on some grid for linear approximation. Our linear 
approximation can be made arbitrarily close to any specific smooth cumulative incidence function.


```{r}
library(mets)
nsim <- 100
rho1 <- 0.4; rho2 <- 2
beta <- c(0.3,-0.3,-0.3,0.3)

dats <- simul.cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="logistic")

par(mfrow=c(1,2))
# Fitting regression model with CIF logistic-link 
cif1 <- cifreg(Event(time,status)~Z1+Z2,dats)
summary(cif1)
plot(cif1)
lines(attr(dats,"Lam1"))

dats <- simul.cifs(nsim,rho1,rho2,beta,rc=0.5,depcens=0,type="cloglog")
ciff <- cifregFG(Event(time,status)~Z1+Z2,dats)
summary(ciff)
plot(ciff)
lines(attr(dats,"Lam1"))
```

We can also use the parameters based on fitted models 

```{r}
 data(bmt)
 ################################################################
 #  simulating several causes with specific cumulatives 
 ################################################################
 ## two logistic link models 
 cif1 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=1)
 cif2 <-  cifreg(Event(time,cause)~tcell+age,data=bmt,cause=2)

 dd <- sim.cifs(list(cif1,cif2),nsim,data=bmt)

 ## still logistic link 
 scif1 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=1)
 ## 2nd cause not on logistic form due to restriction
 scif2 <-  cifreg(Event(time,cause)~tcell+age,data=dd,cause=2)
    
 cbind(cif1$coef,scif1$coef)
 cbind(cif2$coef,scif2$coef)
 par(mfrow=c(1,2))   
 plot(cif1); plot(scif1,add=TRUE,col=2)
 plot(cif2); plot(scif2,add=TRUE,col=2)
```


## CIF Delayed entry

Now assume that given covariates $F_1(t;X) = P(T < t, \epsilon=1|X)$ and  $F_2(t;X) = P(T < t, \epsilon=2|X)$ are two 
cumulative incidence functions that satistifes the needed constraints. We wish to generate data that follows these two
piecewise linear cumulative indidence functions with delayed entry at time $s$.  We should thus
generate data that follows the cumulative incidence functions
$$
\tilde F_1(t,s;X)=   \frac{F_1(t;X) - F_1(s;;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
and 
$$
\tilde F_2(t,s;X)=   \frac{F_2(t;X) - F_2(s;;X)}{ 1 - F_1(s;X) - F_2(s;X)}
$$
this can be done according to the recipe in the previous section.  
To be specific (ignoring the $X$ in the formula)
$$
  F_1^{-1}( F_1(s) + U \cdot (1 - F_1(s;X) - F_2(s;X)) )
$$
where $U$ is a uniform, will have distribution given by $\tilde F_1(t,s)$.

# Recurrent events

See also recurrent events vignette

 - sim.recurrent can simulate based on the Two-Stage model where the the 
    - the rate of the terminal event among survivors in on Cox form (phreg)
      - the rate of the recurrent events among survivors is on Cox form (phreg)
      - the rate of the recurrent events is a  marginal Ghosh-Lin model (recreg)
    - the simulations is based on approximations with piecewise linear models based on a grid.
    - the events can be dependent via a frailty random effects (Gamma distributed)
  - simRecurrentII, simRecurrent, simRecurrentList
    - A frailty Gamma model where the rate of the  events and the terminal event are given based on 
       cumulative baselines and relative risk covariate effects. Thus ends up on Cox form  given the frailty and covariates.
    - simRecurrentList can take multiple recurrent events and multiple causes of death 


Two-stage models 

```{r}
 data(hfactioncpx12)
 hf <- hfactioncpx12
 hf$x <- as.numeric(hf$treatment) 
 n <- 1000

 ##  to fit Cox  models 
 xr <- phreg(Surv(entry,time,status==1)~treatment+cluster(id),data=hf)
 dr <- phreg(Surv(entry,time,status==2)~treatment+cluster(id),data=hf)
 estimate(xr)
 estimate(dr)

 simcoxcox <- sim.recurrent(xr,dr,n=n,data=hf)

 xrs <- phreg(Surv(start,stop,statusD==1)~treatment+cluster(id),data=simcoxcox)
 drs <- phreg(Surv(start,stop,statusD==3)~treatment+cluster(id),data=simcoxcox)
 estimate(xrs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(xrs); 
 plot(xr,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)

```

and a now with Ghosh-Lin and Cox marginals 

```{r}
 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 estimate(recGL)
 estimate(dr)

 simglcox <- sim.recurrent(recGL,dr,n=n,data=hf)

 simcoxcox <- sim.recurrent(xr,dr,n=n,data=hf)
 dtable(simcoxcox,~statusD)

 recGL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,death.code=2)
 simglcox <- sim.recurrent(recGL,dr,n=n,data=hf)

 GLs <- recreg(Event(start,stop,statusD)~treatment+cluster(id),data=simglcox,death.code=3)
 drs <- phreg(Surv(start,stop,statusD==3)~treatment+cluster(id),data=simglcox)
 estimate(GLs)
 estimate(drs)

 par(mfrow=c(1,2))
 plot(GLs); 
 plot(recGL,add=TRUE)
###
 plot(drs)
 plot(dr,add=TRUE)

```


Frailty models 

```{r}
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

 n <- 100
 rr <- simRecurrent(n,base1,death.cumhaz=dr)
 ###
 par(mfrow=c(1,3))
 showfitsim(causes=1,rr,dr,base1,base1,which=1:2)

 rr <- simRecurrentII(n,base1,base4,death.cumhaz=dr)
 dtable(rr,~death+status)
 showfitsim(causes=2,rr,dr,base1,base4,which=1:2)

 cumhaz <- list(base1,base1,base4)
 drl <- list(dr,base4)
 rr <- simRecurrentList(n,cumhaz,death.cumhaz=drl)
 dtable(rr,~death+status)
 showfitsimList(rr,cumhaz,drl) 
```


# Parametric models
While the semi‑parametric Cox model provides substantial flexibility for
simulating survival data, there are situations where a fully parametric
simulation model is convenient or preferable. Here we consider a Weibull model
 parametrized so that the cumulative hazard is given by $$\Lambda(t) = \lambda
 \cdot t^s$$ where $s$ is the **shape parameter**, and $\lambda$ the **rate
 parameter**. We allow regression on both parameters
 \begin{align*} \lambda :=
 \exp(\beta^\top X), \quad s := \exp(\gamma^\top Z) \end{align*}
  where $X$ and
 $Z$ are covariate vectors. Specifically, this opens up for exploring
 non‑proportional hazards when $s$ depends on covariates.

Revisiting the TRACE data example we can compare the predictions from the Cox
and the Weibull-Cox model stratified by `chf` and with a proportional hazard
effect of `age` 
```{r weibull1}
data(sTRACE, package = "mets")
dat <- sTRACE
cox1 <- phreg(Surv(time, status > 0) ~ strata(chf) + I(age - 67), data = sTRACE)
coxw <- phreg_weibull(Surv(time, status > 0) ~ chf + age,
    shape.formula = ~chf,
    data = sTRACE
    )
coxw

tt <- seq(0, max(sTRACE$time), length.out = 100)
newd <- data.frame(chf = c(1, 0), age=67)
pr <- predict(coxw, newdata = newd, times = tt, type="chaz")
plot(cox1, col = 1)
lines(tt, pr[, 1, 1], lty=2, lwd=2)
lines(tt, pr[, 1, 2], lty = 1, lwd = 2)
```

To simulate data we can use the `rweibullcox()` function. Note that the
`stats::rweibull()` function gives a different parametrization where the
cumulative hazard is given by $H(t) = (t/b)^s$, i.e., with the same scale
parameter but where the scale parameter $b$ is related to the rate parameter we consider by $r := b^{-s}$.


```{r weibull_sim}
n <- 5000
newd <- mets::dsample(size=n, sTRACE[,c("chf","age")]) # bootstrap covariates
lp <- predict(coxw, newdata=newd, type="lp") # linear-predictors
head(lp)

## simulate event times
tt <- rweibullcox(nrow(lp), rate = exp(lp[,1]), shape= exp(lp[,2]))

# censoring model
censw <- phreg_weibull(Surv(time, status==0) ~ 1, data=sTRACE)
censpar <- exp(coef(censw))
censtime <- pmin(8, rweibullcox(nrow(lp), censpar[1], censpar[2]))

# combined simulated data
newd <- transform(newd, time=pmin(tt, censtime), status=(tt<=censtime))
head(newd)

# estimate weibull model on new data
phreg_weibull(Surv(time,status) ~ chf + age, ~chf, data=newd)
```

All these steps are wrapped in the `simulate` method:
```{r}
# simulate(coxw, n = 5, cens.model = NULL, data=newd, var.names = c("time", "status"))
simulate(coxw, nsim = 5)
```


# SessionInfo


```{r}
sessionInfo()
```

---
title: "G-Computation or standardization for the Cox, Fine-Gray and binomial regression models for survival data" 
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{G-Computation or standardization for the Cox, Fine-Gray and binomial regression models for survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->




G-computation for the Cox and Fine-Gray models 
==============================================

We compute the standardized estimator (G-estimation) based on the Cox or the Fine-Gray model:
\[
\hat S(t,A=a) = n^{-1} \sum_i S(t,A=a,X_i) 
\]
and this estimator has influence function
\[
S(t,A=a,X_i) -  S(t,A=a)  + E( D_{A_0(t), \beta} S(t,A=a,X_i)  ) \epsilon_i(t)
\]
where $\epsilon_i(t)$ is the iid decomposition 
of the baseline and regression coefficients $(\hat A(t) - A(t), \hat \beta- \beta)$.

These estimates have a causal interpretation under the assumption
of no unmeasured confounders; even without the
causal assumption, standardisation can still be a useful summary measure.

We begin by looking at the cumulative incidence via a Fine-Gray model for both causes and plotting the
standardised cumulative incidence for cause 1.


``` r
library(mets)
set.seed(100)

data(bmt); 
bmt$time <- bmt$time+runif(nrow(bmt))*0.001
dfactor(bmt) <- tcell~tcell
bmt$event <- (bmt$cause!=0)*1

fg1 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=1)
Gfg1 <- survivalG(fg1,bmt,time=50)
summary(Gfg1)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.4331 0.02749 0.3793 0.4870 6.321e-56
#> risk1   0.2727 0.05863 0.1577 0.3876 3.313e-06
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err   2.5%    97.5% P-value
#> ps0  -0.1605 0.06353 -0.285 -0.03597 0.01153
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>       Estimate   Std.Err       2.5%       97.5%    P-value
#> ps0 -0.4628288 0.2212039 -0.8963806 -0.02927703 0.03641016
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.6295004 0.4080439 0.9711474

fg2 <- cifregFG(Event(time,cause)~tcell+platelet+age,bmt,cause=2)
summary(survivalG(fg2,bmt,time=50))
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.2127 0.02314 0.1674 0.2581 3.757e-20
#> risk1   0.3336 0.06799 0.2003 0.4668 9.281e-07
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err     2.5%  97.5% P-value
#> ps0   0.1208 0.07189 -0.02009 0.2617 0.09285
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>      Estimate   Std.Err         2.5%     97.5%   P-value
#> ps0 0.4497465 0.2313601 -0.003710973 0.9032039 0.0519046
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 1.5679146 0.9962959 2.4674960

cif1time <- survivalGtime(fg1,bmt)
plot(cif1time,type="risk"); 
```

<img src="figure/survival-ate-unnamed-chunk-2-1.png" alt="Standardized cumulative incidence for cause 1."  />

Now looking at the survival probability 


``` r
ss <- phreg(Surv(time,event)~tcell+platelet+age,bmt)
sss <- survivalG(ss,bmt,time=50)
summary(sss)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%    P-value
#> risk0   0.6539 0.02709 0.6008 0.7070 9.218e-129
#> risk1   0.5640 0.05971 0.4470 0.6811  3.531e-21
#> 
#> Average Treatment effect: difference (G-estimator) :
#>     Estimate Std.Err    2.5%   97.5% P-value
#> ps0 -0.08992  0.0629 -0.2132 0.03337  0.1529
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>       Estimate   Std.Err       2.5%      97.5%   P-value
#> ps0 -0.1479231 0.1095247 -0.3625876 0.06674132 0.1768263
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.8624974 0.6958733 1.0690189 
#> 
#> Average Treatment effect:  survival-difference (G-estimator) :
#>       Estimate    Std.Err        2.5%     97.5%   P-value
#> ps0 0.08991862 0.06290398 -0.03337092 0.2132082 0.1528725
#> 
#> Average Treatment effect: 1-G (survival)-ratio (G-estimator) :
#> log-ratio: 
#>      Estimate   Std.Err       2.5%     97.5%   P-value
#> ps0 0.2309818 0.1503867 -0.0637708 0.5257343 0.1245583
#> ratio: 
#> Estimate     2.5%    97.5% 
#> 1.259836 0.938220 1.691701

Gtime <- survivalGtime(ss,bmt)
plot(Gtime)
```

<img src="figure/survival-ate-unnamed-chunk-3-1.png" alt="Standardized survival probability over time."  />

G-computation for the binomial regression
==============================================

We compare with similar estimates using doubly robust
estimating equations via `binregATE`. The G-computation standardisation can also be computed using a specialised function that
uses less memory and is faster for large data: `binregG`.


``` r

## survival situation
sr1 <- binregATE(Event(time,event)~tcell+platelet+age,bmt,cause=1,
		 time=40, treat.model=tcell~platelet+age)
summary(sr1)
#>    n events
#>  408    241
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept)  0.676409  0.137007  0.407880  0.944939  0.0000
#> tcell1      -0.023675  0.346994 -0.703770  0.656420  0.9456
#> platelet    -0.492952  0.246158 -0.975412 -0.010492  0.0452
#> age          0.343939  0.115561  0.117444  0.570434  0.0029
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  1.96680 1.50363 2.5727
#> tcell1       0.97660 0.49472 1.9279
#> platelet     0.61082 0.37704 0.9896
#> age          1.41049 1.12462 1.7690
#> 
#> Average Treatment effects (G-formula) :
#>             Estimate    Std.Err       2.5%      97.5% P-value
#> treat0     0.6230976  0.0273827  0.5694284  0.6767667  0.0000
#> treat1     0.6177595  0.0731712  0.4743466  0.7611723  0.0000
#> treat:1-0 -0.0053381  0.0783973 -0.1589940  0.1483179  0.9457
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.622698  0.027460  0.568878  0.676518   0.000
#> treat1     0.637785  0.085242  0.470714  0.804857   0.000
#> treat:1-0  0.015087  0.089442 -0.160215  0.190389   0.866

## relative risk effect 
estimate(coef=sr1$riskDR,vcov=sr1$var.riskDR,f=function(p) p[2]/p[1],null=1)
#>        Estimate Std.Err   2.5% 97.5% P-value
#> treat1    1.024   0.144 0.7421 1.306  0.8664
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [treat1] = 1 
#>  
#> chisq = 0.0283, df = 1, p-value = 0.8664

## competing risks 
br1 <- binregATE(Event(time,cause)~tcell+platelet+age,bmt,cause=1,
		 time=40,treat.model=tcell~platelet+age)
summary(br1)
#>    n events
#>  408    157
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.191519  0.130883 -0.448044  0.065007  0.1434
#> tcell1      -0.712880  0.351489 -1.401786 -0.023974  0.0425
#> platelet    -0.531919  0.244495 -1.011119 -0.052718  0.0296
#> age          0.432939  0.107314  0.222607  0.643271  0.0001
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.82570 0.63888 1.0672
#> tcell1       0.49023 0.24616 0.9763
#> platelet     0.58748 0.36381 0.9486
#> age          1.54178 1.24933 1.9027
#> 
#> Average Treatment effects (G-formula) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.417746  0.027030  0.364768  0.470724  0.0000
#> treat1     0.267097  0.061849  0.145874  0.388319  0.0000
#> treat:1-0 -0.150649  0.067578 -0.283100 -0.018199  0.0258
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.417121  0.027112  0.363983  0.470259  0.0000
#> treat1     0.227776  0.061240  0.107748  0.347803  0.0002
#> treat:1-0 -0.189345  0.066600 -0.319878 -0.058812  0.0045
```

and using the specialized function 


``` r
br1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=40)
Gbr1 <- binregG(br1,bmt,Avalues=NULL)
summary(Gbr1)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.4177 0.02703 0.3648 0.4707 6.988e-54
#> risk1   0.2671 0.06185 0.1459 0.3883 1.571e-05
#> 
#> Average Treatment effect: difference (G-estimator) :
#>    Estimate Std.Err    2.5%   97.5% P-value
#> pa  -0.1506 0.06758 -0.2831 -0.0182  0.0258
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>      Estimate   Std.Err       2.5%      97.5%    P-value
#> pa -0.4472628 0.2406332 -0.9188953 0.02436964 0.06307095
#> ratio: 
#>  Estimate      2.5%     97.5% 
#> 0.6393758 0.3989595 1.0246690

## contrasting average age to 1+2-sd age, Avalues
Gbr2 <- binregG(br1,bmt,varname="age",Avalues=c(0,1,2))
summary(Gbr2)
#> G-estimator :
#>       Estimate Std.Err   2.5%  97.5%   P-value
#> risk0   0.3932 0.02537 0.3434 0.4429 3.738e-54
#> risk1   0.4964 0.03655 0.4248 0.5681 5.044e-42
#> risk2   0.5997 0.05531 0.4913 0.7081 2.136e-27
#> 
#> Average Treatment effect: difference (G-estimator) :
#>      Estimate Std.Err    2.5%  97.5%   P-value
#> pa     0.1033 0.02605 0.05222 0.1543 7.345e-05
#> pa.1   0.2066 0.04996 0.10863 0.3045 3.564e-05
#> 
#> Average Treatment effect: ratio (G-estimator) :
#> log-ratio: 
#>     Estimate    Std.Err      2.5%     97.5%      P-value
#> pa 0.2332376 0.05402806 0.1273445 0.3391307 1.581845e-05
#> pa 0.4222406 0.08691473 0.2518908 0.5925903 1.185167e-06
#> ratio: 
#>    Estimate     2.5%    97.5%
#> pa 1.262681 1.135808 1.403727
#> pa 1.525375 1.286456 1.808667
```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              KernSmooth_2.23-26    
#>  [7] otel_0.2.0             glue_1.8.1             future.apply_1.20.2   
#> [10] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [13] grid_4.6.0             evaluate_1.0.5         lifecycle_1.0.5       
#> [16] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [19] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [22] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [25] digest_0.6.39          pillar_1.11.1          parallelly_1.47.0     
#> [28] parallel_4.6.0         Matrix_1.7-5           tools_4.6.0           
#> [31] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "A practical guide to Human Genetics with Lifetime Data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
#output: pdf_document
output:
  rmarkdown::html_vignette:
    #fig_width: 7.15
    #fig_height: 5.5
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{A practical guide to Human Genetics with Lifetime Data} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, label=setup}
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
  )
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE") || length(list.files("data"))==0
saveobj <- function(obj, null_elements) {
  x <- get(obj, envir=parent.frame())
  if (length(null_elements)>0) {
    print(object.size(x))
    x[null_elements] <- NULL
    attributes(x)[null_elements] <- NULL
    print(object.size(x))
  }
  saveRDS(x, paste0("data/", obj, ".rds"), compress="xz")
}

library(mets)
 cols <- c("darkred","darkblue","black")
 ltys <- c(1,3,2)
 fig_w <- 5
 fig_h <- 5
 savefig <- TRUE
```

```{r  results="hide", echo=FALSE, eval=!fullVignette}
## To save time building the vignettes on CRAN, we cache time consuming computations
fitco1 <- readRDS("data/fitco1.rds")
fitco2 <- readRDS("data/fitco2.rds")
fitco3 <- readRDS("data/fitco3.rds")
fitco4 <- readRDS("data/fitco4.rds")
fitace <- readRDS("data/fitace.rds")
fitde <- readRDS("data/fitde.rds")
cse <- readRDS("data/cse.rds")
slr <- readRDS("data/slr.rds")
outacem <- readRDS("data/outacem.rds")
b0 <- readRDS("data/b0.rds")
b1 <- readRDS("data/b1.rds")
b2 <- readRDS("data/b2.rds")
a1 <- readRDS("data/a1.rds")
h2 <- readRDS("data/h2.rds")
concMZ <- readRDS("data/concMZ.rds")
s_mz_country <- readRDS("data/s_mz_country.rds")
s_dz_country <- readRDS("data/s_dz_country.rds")
```


This vignette demonstrates how to analyze familial resemblance for 
twins using the \texttt{mets} {\bf R}-package and is accompanying the
review by Scheike and Holst (2020). 

We consider a data-set in  that resembles the data of \cite{Hjelmborg2014} that
were based on the NorTwinCan a collaborative research project studying the  genetic and environmental components of prostate
cancer.  The data comprises around 18,000 DZ twins and 11,000 MZ twins.
It was a population based register study based on the Danish, Finnish, Norwegian, and Swedish twin registries.

We first illustrate a hazards based analysis to show how one would study 
dependence in survival data. This needs to be done under assumptions about independent 
competing risks
when the outcome of interest is observed subject to competing risks (here death). 

This seems reasonable here since the occurrence of cancer prior to death only contains weak
association with the risk of death for the other twin, and vice-versa. 

First looking at the data

```{r, label=data-prt}
 library(mets)
 set.seed(122)
 data(prt)
 
 dtable(prt,~status+cancer)
 dtable(prt,~zyg+country,level=1)
```

we see that there are 21283 censorings and 6997 deaths (prior to cancer) and
a total of 942 prostate cancers. 
Approximately half the data consist of DZ twins. 
In addition we see that there are around 10000 twins from Denmark and Sweden, and only 4000 from Norway and Finland, respectively.

Survival 
==========

Under assumption of random effects acting independently on different cause
specific hazards we can analyse competing risks data considering the
cause-specific hazard.  Typically, this can be questionable and the cumulative
incidence modelling below does not rely on this assumption. 

We consider the cause specific hazard of cancer in the competing risks
model with death and cancer.  

First estimating the marginal hazards for each country. 

```{r, label=survival-marginal}
 # Marginal Cox model here stratified on country without covariates 
 margph <- phreg(Surv(time,cancer)~strata(country)+cluster(id),data=prt)
 plot(margph)
```

We see that the marginal of Denmark in particular is quite different. 

Then we fit a two-stage random effects models with country specific 
marginals and random-effects variances that differ for MZ and DZ twins. 


```{r, label=survival-pairwise1, eval=fullVignette}
# Clayton-Oakes, MLE , overall variance
fitco1<-twostageMLE(margph,data=prt,theta=2.7)
```

```{r}
 summary(fitco1)
```

```{r, label=survival-pairwise2, eval=fullVignette}
fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)
```

```{r}
 summary(fitco2)
```

With different random effects for MZ and DZ
```{r label=survival-pairwise3, eval=fullVignette}
 mm <- model.matrix(~-1+factor(zyg),prt)
 fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)
```


```{r}
 summary(fitco3)
```

```{r label=survival-pairwise4, eval=fullVignette}
fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)
```

```{r}
 summary(fitco4)
 round(estimate(coef=fitco4$coef,vcov=fitco4$var.theta)$coefmat[,c(1,3:4)],2)

 ## mz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[2],0,K=1000)$mz.kendall
 ## dz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[1],0,K=1000)$mz.kendall
```


The dependence of MZ twins is much stronger, and is summarized by a 
variance at $`r round(fitco4$coef[2],2)`$ in contrast to the $DZ$ variance at $`r round(fitco4$coef[1],2)`$.


Now we look at the polygenic modelling for survival data, here applied to
the cause specific hazards. 

```{r, label=survival-polygenic1, eval=fullVignette}
 ### setting up design for random effects and parameters of random effects
 desace <- twin.polygen.design(prt,type="ace")

 ### ace model 
 fitace <- survival.twostage(margph,data=prt,theta=1,
       clusters=prt$id,var.link=0,model="clayton.oakes",
       numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
```

```{r}
 summary(fitace)
```

```{r label=survival-polygenic2, eval=fullVignette} 
 ### ace model with positive random effects variances
 # fitacee <- survival.twostage(margph,data=prt,theta=1,
 #      clusters=prt$id,var.link=1,model="clayton.oakes",
 #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
 #summary(fitacee)
 
 ### ae model 
 #desae <- twin.polygen.design(prt,type="ae")
 #fitae <- survival.twostage(margph,data=prt,theta=1,
 #      clusters=prt$id,var.link=0,model="clayton.oakes",
 #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
 #summary(fitae)

 ### de model 
 desde <- twin.polygen.design(prt,type="de")
 fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)                            

```{r}
summary(fitde)
```

The DE model  fits quite well. In summary all shared variance is due to 
genes and there is no suggestion of a shared environmental effect. 


Concordance and Casewise 
========================

First we estimate the concordance of joint prostate cancer. The two-twins are
censored at the same time, otherwise we would enforce this in the data by
artificially censor both twins at the first censoring time.  Given, however,
that we have the same-censoring assumption satisfied we can do the stanadar
Aalen-Johansen product-limit estimator of the concordance probabilities for MZ
and DZ twins.

For simplicity we do not do this for each country even though as we show there are big
differences between the countries. 


```{r}
prt <-  force.same.cens(prt,cause="status")

 dtable(prt,~status+cancer)
 dtable(prt,~status+country)
 dtable(prt,~zyg+country)
```


```{r, label=concordance}
 ## cumulative incidence with cluster standard errors.
 cif1 <- cif(Event(time,status)~strata(country)+cluster(id),prt,cause=2)
 plot(cif1,se=1)

 cifa <- cif(Event(time,status)~+1,prt,cause=2)

 ### concordance estimator, ignoring country differences. 
 p11 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2))
p11mz <- p11$model$"MZ"
p11dz <- p11$model$"DZ" 
```
 
```{r}
 par(mfrow=c(1,2))
 ## Concordance
 plot(p11mz,ylim=c(0,0.1));
 plot(p11dz,ylim=c(0,0.1));
```

Now we compare the concordance to the marginals to get a measure 
that takes the marginals into account when evaluating the strength 
of the association. 
     
```{r, label=concordance2}
 library(prodlim)
 outm <- prodlim(Hist(time,status)~+1,data=prt)

 cifzyg <- cif(Event(time,status)~+strata(zyg)+cluster(id),data=prt,cause=2)
 cifprt <- cif(Event(time,status)~country+cluster(id),data=prt,cause=2)
     
 times <- 70:100
 cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ")) ## cause is 2 (second cause) 
 cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))
    
 ### concordance for MZ and DZ twins<
 cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
 ccdz <- cc$model$"DZ"
 ccmz <- cc$model$"MZ"
     
 cdz <- casewise(ccdz,outm,cause.marg=2) 
 cmz <- casewise(ccmz,outm,cause.marg=2)

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,col=cols[2:1], lty=ltys[2:1], legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-MZ","concordance-DZ"),col=cols[1:2],lty=ltys[1:2])

 plot(cmz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,col=cols[c(1,3,3)],lty=ltys[c(1,3,3)],
      ylab="Casewise",xlab="Age")
  plot(cdz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(cols[2],NA,NA), lty=ltys[c(2,3,3)], add=TRUE)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[1]))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[2]))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=cols, lty=ltys, bg="white")

 summary(cdz)
 summary(cmz)

 Cpred(cmz$casewise,80)
 Cpred(cdz$casewise,80)
```


```{r, label=concordance3}

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-DZ","concordance-MZ"),col=c(1,2),lty=1)
 plot(cmz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,col=c(2,3,3),ylab="Casewise",xlab="Age",lty=c(1,3))
 plot(cdz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(1,3,3), add=TRUE, lty=c(2,3))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=c(2,1,3),lty=1)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=2))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=1))

```

The standard errors above are slightly off since they only reflect the 
uncertainty from the concordance estimation. This can be improved by
doing specific calculations for a specific time-point uisng the 
binomial regression function that gives and iid decomposition for the 
paramters.  We thus apply the binomial regression to estimate the
concordance as well as the marginal, and combine the iid decompositions
when estimating the standard error.  We also do this ignoring country differences. 


```{r, label=concordance4, eval=fullVignette}
 ### new version of Casewise for specific time-point based on binreg 
 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 newdata <- data.frame(zyg=c("DZ","MZ"),id=1)

 ## concordance 
 bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
 pconc <- predict(bcif1,newdata)

 ## marginal estimates
 mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
 mc <- predict(mbcif1,newdata)

 ### casewise with improved se's from log-scale 
 cse <- binregCasewise(bcif1,mbcif1)
```

```{r} 
 cse 
```

It can be useful also to simply model the concordance given covariates, and in
this case we might find it important to adjust for country, or to see if the
differences between MZ and DZ are comparable across contries even though
clearly DK has a much lower cumulative incidence of prostate cancer. 

```{r, label=semiparconc, eval=fullVignette}
 ### semi-parametric modelling of concordance 
 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
 regconc
 ### interaction test
 wald.test(regconc,coef.null=5:7)

 regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
 regconc

 ## logistic link 
 logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
 slr <- summary(logitregconc)
```

```{r}
slr
### library(Publish)
### publish(round(slr$exp.coef[,-c(2,5)],2),latex=TRUE,digits=2)
```

Competing risk using additive Gamma
====================================

Here we do the cumulative incidence random effects modelling 


```{r, label=additive_gamma, eval=fullVignette}
  times <- seq(50,90,length.out=5)
  cif1 <- comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

  desace <- twin.polygen.design(prt,type="ace")
 
  outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
  	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
         step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
  ##outacem$score
```

```{r}  
  summary(outacem)

 ###  variances
 estimate(coef=outacem$theta,vcov=outacem$var.theta,f=function(p) p/sum(p)^2)

 ## AE polygenic model
 # desae <- twin.polygen.design(prt,type="ae")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #    same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
 #        step=0.5,theta.des=desae$pardes,random.design=desae$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p)     p/sum(p)^2)

 ## AE polygenic model
 # desde <- twin.polygen.design(prt,type="de")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #   same.cens=TRUE,theta=c(0.35),var.link=0,
 #   step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

  times <- 90
  cif1 <- comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

 desde <- twin.polygen.design(prt,type="de")
 outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
	same.cens=TRUE,theta=c(0.35),var.link=0,
        step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 outaem$score
 summary(outaem)
 estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

```


Competing risk modeling using the Liabilty Threshold model 
===========================================================


First we fit the bivariate probit model (same marginals in MZ and DZ twins but different correlation parameter). Here we evaluate the risk of getting cancer before the last double cancer event (95 years)
```{r, label=probit1}
rm(prt)
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in
```

```{r b0, eval=fullVignette}
b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
```

```{r}
summary(b0)
```


Liability threshold model with ACE random effects structure
```{r, label=liability_ace1, eval=fullVignette}
b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
```

```{r}
summary(b1)
```

In this case the ACE model fits the data well - it is in fact indistinguishable from the flexible bivariate Probit model as seen by the IPCW weighted AIC measure
```{r}
AIC(b0, b1)
```

ACE model with marginal adjusted for country
```{r, label=liability_ace_country, eval=fullVignette}
b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
              cens.formula=Surv(time,status==0)~zyg+country, breaks=95)
```

```{r}
summary(b2)
```

```{r, label=bptime1, eval=fullVignette}
bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace", 
              cens.formula=Surv(time,status==0)~zyg,
              summary.function=function(x) x, breaks=tt)
h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))

```

```{r}
par(mfrow=c(1,2))
plot(tt, h2[,1], type="s", lty=1, col=cols[3], xlab="Age", ylab="Heritability", ylim=c(0,1))
confband(tt, h2[,2], h2[,3],polygon=TRUE, step=TRUE, col=Col(cols[3], 0.1), border=NA)
plot(tt, concMZ[,1], type="s", lty=1, col=cols[1], xlab="Age", ylab="Concordance", ylim=c(0,.1))
confband(tt, concMZ[,2], concMZ[,3],polygon=TRUE, step=TRUE, col=Col(cols[1], 0.1), border=NA)
```


Bivariate probit model at time different time points
```{r, label=biprobittime1}
system.time(a.mz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~1, pairs.only=TRUE,
                                breaks=tt))
system.time(a.dz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~1, pairs.only=TRUE,
                               breaks=tt))

#system.time(a.zyg <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, 
#                               cens.formula = Event(time,status==0)~1,
#                               eqmarg=FALSE, fix.cens.weight
#                               breaks=seq(75,100,by=10)))

a.mz
a.dz

plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance", ylim=c(0,0.07))
plot(a.mz, ylim=c(0,.07), col=cols[1], lty=ltys[1], legend=FALSE, add=TRUE)
plot(a.dz, col=cols[2], lty=ltys[2], add=TRUE)
```


Bivariate probit model adjusting for country
```{r, label=biprobittime2, eval=fullVignette}
a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
                                breaks=tt)
system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
                               breaks=tt))

s_mz_country <- summary(a.mz_country)
s_dz_country <- summary(a.dz_country)
```

```{r}
s_mz_country
s_dz_country
```



```{r, label=liability_ace_time1, eval=fullVignette}
## ACE model (time-varying) with and without adjustment for country
a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
                              zyg="zyg", DZ="DZ", 
                              cens.formula=Surv(time,status==0)~zyg,
                              breaks=tt)

#a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
#                              zyg="zyg", DZ="DZ", 
#                              #cens.formula=Surv(time,status==0)~country+zyg,
#                              breaks=tt)
```


```{r}
plot(a.mz, which=c(6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1], lty=ltys[1], legend=NULL, alpha=.1)
plot(a.dz, which=c(6), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, alpha=.1)
legend("topleft", c("MZ tetrachoric correlation", "DZ tetrachoric correlation"),
       col=cols, lty=ltys, lwd=2)

plot(a.mz, which=c(4), xlab="Age", ylab="Relative Recurrence Risk",
     ylim=c(1,20), col=cols[1], lty=ltys[1], legend=NULL, lwd=2, alpha=.1)
plot(a.dz, which=c(4), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, lwd=2, alpha=.1)
legend("topright", c("MZ relative recurrence risk", "DZ relative recurrence risk"),
       col=cols, lty=ltys, lwd=2)

plot(a1, which=c(5,6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1:2], lty=ltys[1:2], lwd=2, alpha=0.1,
     legend=c("MZ tetrachoric correlation", "DZ tetrachoric correlation"))

plot(a1, which=c(1), xlab="Age", ylim=c(0,1), col="black", lty=1, ylab="Heritability", legend=NULL, alpha=.1)
```

SessionInfo
============

```{r}
sessionInfo()
```


```{r  saveobj, results="hide", echo=FALSE, eval=fullVignette }
## To save time building the vignettes on CRAN, we cache time consuming computations

rms <- c('id','theta.iid','theta.des','marginal.trunc',
  'loglikeiid','marginal.surv','theta.iid.naive',
  'antclust','secluster','cluster.call','trunclikeiid',
  'logl.iid','score.iid','clusters')  
tmp <- lapply(as.list(paste0("fitco", 1:4)),
              function(x) saveobj(x, rms))

rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
saveobj("fitace", rms)
saveobj("fitde", rms)

saveobj("cse", NULL)
saveobj("slr", NULL)

rms <- c("theta.iid", "Clusters", "p11")
saveobj("outacem", rms)

rms <- c("model.frame", "score", "id", "logLik")
saveobj("b0", rms)
saveobj("b1", rms)
saveobj("b2", rms)
saveobj("a1", rms)
saveobj("h2", NULL)
saveobj("concMZ", NULL)
saveobj("s_mz_country", NULL)
saveobj("s_dz_country", NULL)
```

---
title: "Binomial Regression for Survival and Competing Risks Data"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Binomial Regression for Survival and Competing Risks Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Binomial Regression for censored data 
=====================================

The `binreg` function fits a logistic link model with IPCW adjustment for a specific time-point,
and can thus be used for both survival and competing risks data.
Computation is linear in data size, and influence functions are computed and available
for downstream calculations.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Details
=======

The binreg function does direct binomial regression for one time-point, $t$, fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta)  = F_1(t,X,\beta)
\end{align*}
based on an IPCW adjusted estimating equation (EE) with response  $Y(t)=I(T \leq t, \epsilon=1 )$
\begin{align*}
 U(\beta,\hat G_c) = &  X ( Y(t) \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)} - \mbox{expit}( X^T \beta)) = 0,
\end{align*}
with $G_c(t)=P(C>t)$, the  censoring survival distribution, and with $\Delta(t) = I( C_i > T_i \wedge t)$
the indicator of being uncensored at time $t$ (type="I"). 

The default type="II" is to augment with a censoring term, that is solve 
\begin{align*}
  &  U(\beta,\hat G_c) + \int_0^t X \frac{\hat E(Y(t)| T>u)}{\hat G_c(u)} d\hat M_c(u) =0 
\end{align*}
where $M_c(u)$ is the censoring martingale, this typically improves the performance. This is equivalent to the 
pseudo-value approach (see Overgaard (2025)).

The influence function for the type="II" estimator is 
\begin{align*}
    U(\beta,G_c) + \int_0^t X \frac{E(Y| T>u)}{G_c(u)} d M_c(u)  
    - \int_0^t  \frac{E(X| T>u) E(Y| T>u)}{G_c(u)} d M_c(u) - \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u)  
\end{align*}
and for type="I"
\begin{align*}
  &  U(\beta) + \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u).
\end{align*}
The means $E(X Y(t) | T>u)$ and $E(Y(t)| T>u)$ are estimated by IPCW estimators among survivors to get 
estimates of the influence functions.

The function logitIPCW instead considers 
\begin{align*}
 U^{glm}(\beta,\hat G_c) = & \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)}  X  ( Y(t) - \mbox{expit}( X^T \beta)) = 0.
\end{align*}
This score equation is quite similar to those of the binreg, and exactly the same when the censoring model is fully-nonparametric.

The logitIPCW has influence function 
\begin{align*}
  &  U^{glm}(\beta,G_c) + \int_0^t \frac{E( X ( Y - F_1(t,\beta)) | T>u)}{G_c(u)} d M_c(u)  
\end{align*}

Which estimator performs the best depends on the censoring distribution and it seems that the binreg with type="II" performs 
overall quite nicely (see Blanche et al (2023) and Overgaard (2025)).  For the full estimated censoring model all estimators have the
same influence function (see Blanche et al (2023)).

Additional functions logitATE, and binregATE computes the average treatment effect based on propensity and outcome modelling. 
We demonstrate this in another vignette. 

The functions logitATE/binregATE can be used when there is no censoring and we thus have simple binary outcome. 

The variance is based on a sandwich formula with IPCW adjustment (using the influence functions),
and `naive.var` is the variance under a known censoring model. The influence functions are stored in the output.
Clusters can be specified to obtain cluster-corrected standard errors.

Examples
========



``` r
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in random noise just below.
 data(bmt)
 bmt$time <- bmt$time+runif(nrow(bmt))*0.01
 bmt$id <- 1:408

 # logistic regression with IPCW binomial regression 
 out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
 summary(out)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.180338  0.126748 -0.428760  0.068084  0.1548
#> tcell       -0.418545  0.345480 -1.095675  0.258584  0.2257
#> platelet    -0.437644  0.240978 -0.909952  0.034665  0.0694
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.83499 0.65132 1.0705
#> tcell        0.65800 0.33431 1.2951
#> platelet     0.64556 0.40254 1.0353
```

We can also compute predictions using the estimates 

``` r
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
#>        pred         se     lower     upper
#> 1 0.3502406 0.04847582 0.2552280 0.4452533
#> 2 0.2618207 0.06969334 0.1252217 0.3984196
```

Further the censoring model can depend on strata 


``` r
 outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
 summary(outs)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.180697  0.127414 -0.430424  0.069030  0.1561
#> tcell       -0.365928  0.350632 -1.053154  0.321299  0.2967
#> platelet    -0.433494  0.240270 -0.904415  0.037428  0.0712
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.83469 0.65023 1.0715
#> tcell        0.69355 0.34884 1.3789
#> platelet     0.64824 0.40478 1.0381
```

Absolute risk differences and ratio
===================================

Now for illustrations we consider the absolute risk difference depending on tcell 


``` r
 outs <- binreg(Event(time,cause)~tcell,bmt,time=50,cens.model=~strata(tcell))
 summary(outs)
#>    n events
#>  408    160
#> 
#>  408 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -0.30054  0.11153 -0.51914 -0.08194  0.0070
#> tcell       -0.51741  0.33981 -1.18342  0.14860  0.1278
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.74042 0.59503 0.9213
#> tcell        0.59606 0.30623 1.1602
```

the risk difference is 


``` r
ps <-  predict(outs,data.frame(tcell=c(0,1)),se=TRUE)
ps
#>        pred         se     lower     upper
#> 1 0.4254253 0.02726306 0.3719897 0.4788609
#> 2 0.3061988 0.06819019 0.1725461 0.4398516
sum( c(1,-1) * ps[,1])
#> [1] 0.1192264
```

Getting the standard errors is straightforward since the two groups are
independent. If we had additionally adjusted for other covariates, however, we would need
to apply the delta theorem using the relevant covariances, along the lines of:


``` r
dd <- data.frame(tcell=c(0,1))
p <- predict(outs,dd)

riskdifratio <- function(p,contrast=c(1,-1)) {
   outs$coef <- p
   p <- predict(outs,dd)[,1]
   pd <- sum(contrast*p)
   r1 <- p[1]/p[2]
   r2 <- p[2]/p[1]
   return(c(pd,r1,r2))
}
     
estimate(outs,f=riskdifratio,dd,null=c(0,1,1))
#>    Estimate Std.Err     2.5%  97.5% P-value
#> p1   0.1192 0.07344 -0.02471 0.2632 0.10448
#> p2   1.3894 0.32197  0.75833 2.0204 0.22652
#> p3   0.7197 0.16679  0.39284 1.0467 0.09291
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] = 0
#>   [p2] = 1
#>   [p3] = 1 
#>  
#> chisq = 12.0249, df = 2, p-value = 0.002448
```

same as 


``` r
run <- 0
if (run==1) {
library(prodlim)
pl <- prodlim(Hist(time,cause)~tcell,bmt)
spl <- summary(pl,times=50,asMatrix=TRUE)
spl
}
```



SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "Discrete Interval Censored Survival Models"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{Discrete Interval Censored Survival Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Discrete Interval Censored survival times 
========================================

We consider the cumulative odds model for the probability of dying before time t: 
\begin{align*}
   \mbox{logit}(P(T \leq t | x)) & = \log(G(t)) + x^T \beta              \\
                P(T \leq t | x)  & =  \frac{G(t) exp( x^T \beta)}{1 + G(t) exp( x^T \beta) }          \\
                P(T >t | x)  & =  \frac{1}{1 + G(t) exp( x^T \beta) }          
\end{align*}

Input are intervals given by $]t_l,t_r]$ where t_r can be infinity
for right-censored intervals. When the data is discrete,
in contrast to grouping of continuous data, $]0,1]$ then
the intervals $]j,j+1]$ will be equivalent to an observation at j+1 (see below example).

Likelihood is maximized:
\begin{align*}
    \prod_i  P(T_i >t_{il} | x) - P(T_i> t_{ir}| x).            
\end{align*}

This model is also called the cumulative odds model 
\begin{align*}
     P(T \leq t | x)  & =  \frac{ G(t) exp( x^T \beta) }{1 + G(t) exp( x^T \beta) }.
\end{align*}
and $\beta$ captures the odds ratio for the probability of being before $t$.

The baseline is parametrized as 
\begin{align*}
       G(t)  & =  \sum_{j \leq t} \exp( \alpha_j ) 
\end{align*}

An important consequence of the model is that for all cut-points $t$ we have the same OR parameters for the 
OR of being early or later than $t$. 

Discrete TTP 
=============

First we look at some time to pregnancy data (simulated discrete survival data) that is
right-censored, and set it up to fit the cumulative odds model by 
constructing the intervals appropriately: 


``` r
library(mets)

data(ttpd) 
dtable(ttpd,~entry+time2)
#> 
#>       time2   1   2   3   4   5   6 Inf
#> entry                                  
#> 0           316   0   0   0   0   0   0
#> 1             0 133   0   0   0   0   0
#> 2             0   0 150   0   0   0   0
#> 3             0   0   0  23   0   0   0
#> 4             0   0   0   0  90   0   0
#> 5             0   0   0   0   0  68   0
#> 6             0   0   0   0   0   0 220
out <- interval_logitsurv_discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
summary(out)
#> $baseline
#>       Estimate Std.Err   2.5%   97.5%   P-value
#> time1  -2.0064  0.1523 -2.305 -1.7079 1.273e-39
#> time2  -2.1749  0.1599 -2.488 -1.8614 4.118e-42
#> time3  -1.4581  0.1544 -1.761 -1.1554 3.636e-21
#> time4  -2.9260  0.2453 -3.407 -2.4453 8.379e-33
#> time5  -1.2051  0.1706 -1.539 -0.8706 1.633e-12
#> time6  -0.9102  0.1860 -1.275 -0.5457 9.843e-07
#> 
#> $logor
#>    Estimate Std.Err    2.5%  97.5%   P-value
#> X1   0.9913  0.1179 0.76024 1.2223 4.100e-17
#> X2   0.6962  0.1162 0.46847 0.9238 2.064e-09
#> X3   0.3466  0.1159 0.11941 0.5738 2.788e-03
#> X4   0.3223  0.1151 0.09668 0.5478 5.111e-03
#> 
#> $or
#>    Estimate     2.5%    97.5%
#> X1 2.694610 2.138791 3.394874
#> X2 2.006032 1.597554 2.518953
#> X3 1.414239 1.126834 1.774950
#> X4 1.380231 1.101503 1.729490

dfactor(ttpd) <- entry.f~entry
out <- cumoddsreg(entry.f~X1+X2+X3+X4,ttpd)
summary(out)
#> $baseline
#>       Estimate Std.Err   2.5%   97.5%   P-value
#> time1  -2.0064  0.1523 -2.305 -1.7079 1.273e-39
#> time2  -2.1749  0.1599 -2.488 -1.8614 4.118e-42
#> time3  -1.4581  0.1544 -1.761 -1.1554 3.636e-21
#> time4  -2.9260  0.2453 -3.407 -2.4453 8.379e-33
#> time5  -1.2051  0.1706 -1.539 -0.8706 1.633e-12
#> time6  -0.9102  0.1860 -1.275 -0.5457 9.843e-07
#> 
#> $logor
#>    Estimate Std.Err    2.5%  97.5%   P-value
#> X1   0.9913  0.1179 0.76024 1.2223 4.100e-17
#> X2   0.6962  0.1162 0.46847 0.9238 2.064e-09
#> X3   0.3466  0.1159 0.11941 0.5738 2.788e-03
#> X4   0.3223  0.1151 0.09668 0.5478 5.111e-03
#> 
#> $or
#>    Estimate     2.5%    97.5%
#> X1 2.694610 2.138791 3.394874
#> X2 2.006032 1.597554 2.518953
#> X3 1.414239 1.126834 1.774950
#> X4 1.380231 1.101503 1.729490
```

We note that the probability of an event (pregnancy) is considerably higher for all covariates.

Now using this discrete survival model we simulate some data from this model 


``` r
set.seed(1000) # to control output in simulations for p-values below.
n <- 200
Z <- matrix(rbinom(n*4,1,0.5),n,4)
outsim <- simlogitSurvd(out$coef,Z)
outsim <- transform(outsim,left=time,right=time+1)
outsim <- dtransform(outsim,right=Inf,status==0)
outss <- interval_logitsurv_discrete(Interval(left,right)~+X1+X2+X3+X4,outsim)
summary(outss)
#> $baseline
#>       Estimate Std.Err    2.5%   97.5%   P-value
#> time1  -2.0154  0.3698 -2.7402 -1.2906 5.036e-08
#> time2  -1.5474  0.3473 -2.2281 -0.8666 8.385e-06
#> time3  -0.8119  0.3411 -1.4804 -0.1434 1.729e-02
#> time4  -2.0085  0.5102 -3.0084 -1.0086 8.248e-05
#> time5  -0.2185  0.3858 -0.9746  0.5376 5.711e-01
#> time6   0.2637  0.4618 -0.6415  1.1689 5.681e-01
#> 
#> $logor
#>    Estimate Std.Err    2.5%  97.5%   P-value
#> X1  1.27893  0.2804  0.7293 1.8286 5.106e-06
#> X2  0.39293  0.2635 -0.1235 0.9094 1.359e-01
#> X3 -0.09008  0.2524 -0.5847 0.4045 7.211e-01
#> X4  0.20766  0.2627 -0.3072 0.7225 4.292e-01
#> 
#> $or
#>    Estimate      2.5%    97.5%
#> X1 3.592796 2.0735647 6.225116
#> X2 1.481310 0.8838237 2.482711
#> X3 0.913858 0.5572845 1.498582
#> X4 1.230798 0.7355301 2.059553

pred <- predictlogitSurvd(out,se=TRUE)
plotSurvd(pred,se=TRUE)
```

<img src="figure/interval-discrete-survival-unnamed-chunk-3-1.png" alt="Discrete survival predictions with standard errors."  />

Finally, we look at some data and compare with the icenReg package that can also fit 
the proportional odds model for continuous or discrete data.
We make the data fully interval censored/discrete by
letting also exact observations be only observed to be in an interval.

We consider the interval censored survival times for time from onset 
of diabetes to diabetic nephropathy, then modify it to
observe only that the event times are in certain intervals. 



``` r
test <- 0 
if (test==1) {

require(icenReg)
data(IR_diabetes)
IRdia <- IR_diabetes
## removing fully observed data in continuous version, here making it a discrete observation 
IRdia <- dtransform(IRdia,left=left-1,left==right)
dtable(IRdia,~left+right,level=1)

ints <- with(IRdia,dInterval(left,right,cuts=c(0,5,10,20,30,40,Inf),show=TRUE) )
}
```

We note that the gender effect is equivalent for the two approaches. 



``` r
if (test==1) {
ints$Ileft <- ints$left
ints$Iright <- ints$right
IRdia <- cbind(IRdia,data.frame(Ileft=ints$Ileft,Iright=ints$Iright))
dtable(IRdia,~Ileft+Iright)
# 
#       Iright   1   2   3   4   5 Inf
# Ileft                               
# 0             10   1  34  25   4   0
# 1              0  55  19  17   1   1
# 2              0   0 393  16   4   0
# 3              0   0   0 127   1   0
# 4              0   0   0   0  21   0
# 5              0   0   0   0   0   2

outss <- interval.logitsurv.discrete(Interval(Ileft,Iright)~+gender,IRdia)
#            Estimate Std.Err    2.5%    97.5%   P-value
# time1        -3.934  0.3316 -4.5842 -3.28418 1.846e-32
# time2        -2.042  0.1693 -2.3742 -1.71038 1.710e-33
# time3         1.443  0.1481  1.1530  1.73340 1.911e-22
# time4         3.545  0.2629  3.0295  4.06008 1.976e-41
# time5         6.067  0.7757  4.5470  7.58784 5.217e-15
# gendermale   -0.385  0.1691 -0.7165 -0.05351 2.283e-02
summary(outss)
outss$ploglik
# [1] -646.1946

fit <- ic_sp(cbind(Ileft, Iright) ~ gender, data = IRdia, model = "po")
# 
# Model:  Proportional Odds
# Dependency structure assumed: Independence
# Baseline:  semi-parametric 
# Call: ic_sp(formula = cbind(Ileft, Iright) ~ gender, data = IRdia, 
#     model = "po")
# 
#            Estimate Exp(Est)
# gendermale    0.385     1.47
# 
# final llk =  -646.1946 
# Iterations =  6 
# Bootstrap Samples =  0 
# WARNING: only  0  bootstrap samples used for standard errors. 
# Suggest using more bootstrap samples for inference
summary(fit)

## sometimes NR-algorithm needs modifications of stepsize to run 
## outss <- interval_logitsurv_discrete(Interval(Ileft,Iright)~+gender,IRdia,control=list(trace=TRUE,stepsize=1.0))
}

```

This also agrees with the cumulative link regression of the `ordinal` package,
although the baseline is parametrised differently. Note that `clm` models the
probability of surviving rather than the probability of
dying. 


``` r

data(ttpd) 
dtable(ttpd,~entry+time2)
#> 
#>       time2   1   2   3   4   5   6 Inf
#> entry                                  
#> 0           316   0   0   0   0   0   0
#> 1             0 133   0   0   0   0   0
#> 2             0   0 150   0   0   0   0
#> 3             0   0   0  23   0   0   0
#> 4             0   0   0   0  90   0   0
#> 5             0   0   0   0   0  68   0
#> 6             0   0   0   0   0   0 220
ttpd <- dfactor(ttpd,fentry~entry)
out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
summary(out)
#> $baseline
#>       Estimate Std.Err   2.5%   97.5%   P-value
#> time1  -2.0064  0.1523 -2.305 -1.7079 1.273e-39
#> time2  -2.1749  0.1599 -2.488 -1.8614 4.118e-42
#> time3  -1.4581  0.1544 -1.761 -1.1554 3.636e-21
#> time4  -2.9260  0.2453 -3.407 -2.4453 8.379e-33
#> time5  -1.2051  0.1706 -1.539 -0.8706 1.633e-12
#> time6  -0.9102  0.1860 -1.275 -0.5457 9.843e-07
#> 
#> $logor
#>    Estimate Std.Err    2.5%  97.5%   P-value
#> X1   0.9913  0.1179 0.76024 1.2223 4.100e-17
#> X2   0.6962  0.1162 0.46847 0.9238 2.064e-09
#> X3   0.3466  0.1159 0.11941 0.5738 2.788e-03
#> X4   0.3223  0.1151 0.09668 0.5478 5.111e-03
#> 
#> $or
#>    Estimate     2.5%    97.5%
#> X1 2.694610 2.138791 3.394874
#> X2 2.006032 1.597554 2.518953
#> X3 1.414239 1.126834 1.774950
#> X4 1.380231 1.101503 1.729490

out$ploglik
#> [1] -1676.456

if (test==1) {
### library(ordinal)
### out1 <- clm(fentry~X1+X2+X3+X4,data=ttpd)
### summary(out1)

# formula: fentry ~ X1 + X2 + X3 + X4
# data:    ttpd
# 
#  link  threshold nobs logLik   AIC     niter max.grad cond.H 
#  logit flexible  1000 -1676.46 3372.91 6(2)  1.17e-12 5.3e+02
# 
# Coefficients:
#    Estimate Std. Error z value Pr(>|z|)    
# X1  -0.9913     0.1171  -8.465  < 2e-16 ***
# X2  -0.6962     0.1156  -6.021 1.74e-09 ***
# X3  -0.3466     0.1150  -3.013  0.00259 ** 
# X4  -0.3223     0.1147  -2.810  0.00495 ** 
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Threshold coefficients:
#     Estimate Std. Error z value
# 0|1  -2.0064     0.1461 -13.733
# 1|2  -1.3940     0.1396  -9.984
# 2|3  -0.7324     0.1347  -5.435
# 3|4  -0.6266     0.1343  -4.667
# 4|5  -0.1814     0.1333  -1.361
# 5|6   0.2123     0.1342   1.582
}

```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "dUtility data-frame manipulations"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes 
vignette: >
  %\VignetteIndexEntry{dUtility data-frame manipulations} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->





Simple data manipulation for data-frames 
==========================================

   *  Renaming variables, Deleting variables
   *  Looking at the data
   *  Making new variables for the analysis
   *  Making factors (groupings)
   *  Working with factors
   *  Making a factor from existing numeric variable and vice versa


Here are some key data-manipulation steps on a data frame, which is how we typically
organise our data in R.
After reading the data into R it will typically be a data frame; if not, we
can force it to be one. The basic idea of the utility functions is
to provide a simple and easy-to-type way of performing common data manipulations on a
data frame, much like what is possible in SAS or Stata.

The functions `dcut`, `dfactor`, and so on do essentially what
the base R `cut` and `factor` functions do, but are easier to use in the context of data frames
and have additional functionality.


``` r
library(mets)
data(melanoma)
```


``` r
is.data.frame(melanoma)
#> [1] TRUE
```

Here we work on the melanoma data, which has already been read into R and is
a data frame.


dUtility functions 
==================

The structure for all functions is

*  `dfunction(dataframe, y~x|ifcond, ...)`

to apply the function to `y` in a data frame, grouped by `x`, and only when
condition `ifcond` is satisfied. The basic functions are:

Data processing
    * dsort
    * dreshape
    * dcut
    * drm, drename, ddrop, dkeep, dsubset
    * drelevel
    * dlag
    * dfactor, dnumeric

Data aggregation
    * dby, dby2
    * dscalar, deval, daggregate
    * dmean, dsd, dsum, dquantile, dcor
    * dtable, dcount

Data summaries
    * dhead, dtail, 
    * dsummary, 
    * dprint, dlist, dlevels, dunique


A generic function daggregate, daggr,  can be called with a function as the 
argument:

*  `daggregate(dataframe, y~x|ifcond, fun=function, ...)`

or without the grouping variable (`x`):

*  `daggregate(dataframe, ~y|ifcond, fun=function, ...)`

A useful feature is that `y` and `x` as well as the subset condition
can be specified using regular expressions or wildcards (default).
Here we compute the means of certain variables.

First, overall:

``` r
dmean(melanoma,~thick+I(log(thick)))
#>         thick I(log(thick)) 
#>    291.985366      5.223341
```

now only when days>500

``` r
dmean(melanoma,~thick+I(log(thick))|I(days>500))
#>         thick I(log(thick)) 
#>    271.582011      5.168691
```

and now after sex but only when days>500


``` r
dmean(melanoma,thick+I(log(thick))~sex|I(days>500))
#>   sex    thick I(log(thick))
#> 1   0 242.9580      5.060086
#> 2   1 320.2429      5.353321
```

and finally after quartiles of days (via the dcut function)


``` r
dmean(melanoma,thick+I(log(thick))~I(dcut(days)))
#>         I(dcut(days))    thick I(log(thick))
#> 1       [10,1.52e+03] 482.1731      5.799525
#> 2    (1.52e+03,2e+03] 208.5490      4.987652
#> 3    (2e+03,3.04e+03] 223.2941      4.974759
#> 4 (3.04e+03,5.56e+03] 250.1961      5.120129
```

or summary of all variables starting with "s" and that contains "a"


``` r
dmean(melanoma,"s*"+"*a*"~sex|I(days>500))
#>   sex   status     days
#> 1   0 1.831933 2399.143
#> 2   1 1.714286 2169.800
```


Renaming, deleting, keeping, dropping  variables
===============================================


``` r
melanoma=drename(melanoma,tykkelse~thick)
names(melanoma)
#> [1] "no"       "status"   "days"     "ulc"      "tykkelse" "sex"
```

Deleting variables


``` r
data(melanoma)
melanoma=drm(melanoma,~thick+sex)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

or SAS style:


``` r
data(melanoma)
melanoma=ddrop(melanoma,~thick+sex)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

alternatively, we can keep certain variables:


``` r
data(melanoma)
melanoma=dkeep(melanoma,~thick+sex+status+days)
names(melanoma)
#> [1] "thick"  "sex"    "status" "days"
```

This can also be done with direct assignment 


``` r
data(melanoma)
ddrop(melanoma) <- ~thick+sex
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"
```

The dkeep function can also be used to re-ordering the variables in the data-frame


``` r
data(melanoma)
names(melanoma)
#> [1] "no"     "status" "days"   "ulc"    "thick"  "sex"
melanoma=dkeep(melanoma,~days+status+.)
names(melanoma)
#> [1] "days"   "status" "no"     "ulc"    "thick"  "sex"
```


Looking at the data 
===================


``` r
data(melanoma)
dstr(melanoma)
#> 'data.frame':	205 obs. of  6 variables:
#>  $ no    : int  789 13 97 16 21 469 685 7 932 944 ...
#>  $ status: int  3 3 2 3 1 1 1 1 3 1 ...
#>  $ days  : int  10 30 35 99 185 204 210 232 232 279 ...
#>  $ ulc   : int  1 0 0 0 1 1 1 1 1 1 ...
#>  $ thick : int  676 65 134 290 1208 484 516 1288 322 741 ...
#>  $ sex   : int  1 1 1 0 1 1 1 1 0 0 ...
```


The data can be viewed as a data table in RStudio, but to list certain parts of
the data in the output window:


``` r
dlist(melanoma)
#>     no  status days ulc thick sex
#> 1   789 3       10  1    676  1  
#> 2    13 3       30  0     65  1  
#> 3    97 2       35  0    134  1  
#> 4    16 3       99  0    290  0  
#> 5    21 1      185  1   1208  1  
#> ---                              
#> 201 317 2      4492 1   706   1  
#> 202 798 2      4668 0   612   0  
#> 203 806 2      4688 0    48   0  
#> 204 606 2      4926 0   226   0  
#> 205 328 2      5565 0   290   0
```


``` r
dlist(melanoma, ~.|sex==1)
#>     no  status days ulc thick
#> 1   789 3       10  1    676 
#> 2    13 3       30  0     65 
#> 3    97 2       35  0    134 
#> 5    21 1      185  1   1208 
#> 6   469 1      204  1    484 
#> ---                          
#> 191 445 2      3909 1   806  
#> 195 415 2      4119 0    65  
#> 197 175 2      4207 0    65  
#> 198 493 2      4310 0   210  
#> 201 317 2      4492 1   706
```


``` r
dlist(melanoma, ~ulc+days+thick+sex|sex==1)
#>     ulc days thick sex
#> 1   1    10   676  1  
#> 2   0    30    65  1  
#> 3   0    35   134  1  
#> 5   1   185  1208  1  
#> 6   1   204   484  1  
#> ---                   
#> 191 1   3909 806   1  
#> 195 0   4119  65   1  
#> 197 0   4207  65   1  
#> 198 0   4310 210   1  
#> 201 1   4492 706   1
```

Getting summaries


``` r
dsummary(melanoma)
#>        no            status          days           ulc            thick     
#>  Min.   :  2.0   Min.   :1.00   Min.   :  10   Min.   :0.000   Min.   :  10  
#>  1st Qu.:222.0   1st Qu.:1.00   1st Qu.:1525   1st Qu.:0.000   1st Qu.:  97  
#>  Median :469.0   Median :2.00   Median :2005   Median :0.000   Median : 194  
#>  Mean   :463.9   Mean   :1.79   Mean   :2153   Mean   :0.439   Mean   : 292  
#>  3rd Qu.:731.0   3rd Qu.:2.00   3rd Qu.:3042   3rd Qu.:1.000   3rd Qu.: 356  
#>  Max.   :992.0   Max.   :3.00   Max.   :5565   Max.   :1.000   Max.   :1742  
#>       sex        
#>  Min.   :0.0000  
#>  1st Qu.:0.0000  
#>  Median :0.0000  
#>  Mean   :0.3854  
#>  3rd Qu.:1.0000  
#>  Max.   :1.0000
```

or for specific variables


``` r
dsummary(melanoma,~thick+status+sex)
#>      thick          status          sex        
#>  Min.   :  10   Min.   :1.00   Min.   :0.0000  
#>  1st Qu.:  97   1st Qu.:1.00   1st Qu.:0.0000  
#>  Median : 194   Median :2.00   Median :0.0000  
#>  Mean   : 292   Mean   :1.79   Mean   :0.3854  
#>  3rd Qu.: 356   3rd Qu.:2.00   3rd Qu.:1.0000  
#>  Max.   :1742   Max.   :3.00   Max.   :1.0000
```


Summaries in different groups (sex)


``` r
dsummary(melanoma,thick+days+status~sex)
#> sex: 0
#>      thick             days          status     
#>  Min.   :  10.0   Min.   :  99   Min.   :1.000  
#>  1st Qu.:  97.0   1st Qu.:1636   1st Qu.:2.000  
#>  Median : 162.0   Median :2059   Median :2.000  
#>  Mean   : 248.6   Mean   :2283   Mean   :1.833  
#>  3rd Qu.: 306.0   3rd Qu.:3131   3rd Qu.:2.000  
#>  Max.   :1742.0   Max.   :5565   Max.   :3.000  
#> ------------------------------------------------------------ 
#> sex: 1
#>      thick             days          status     
#>  Min.   :  16.0   Min.   :  10   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1052   1st Qu.:1.000  
#>  Median : 258.0   Median :1860   Median :2.000  
#>  Mean   : 361.1   Mean   :1946   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2784   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :4492   Max.   :3.000
```


and only among those with thin-tumours or only females (sex==1)


``` r
dsummary(melanoma,thick+days+status~sex|thick<97)
#> sex: 0
#>      thick            days          status     
#>  Min.   :10.00   Min.   : 355   Min.   :1.000  
#>  1st Qu.:32.00   1st Qu.:1762   1st Qu.:2.000  
#>  Median :64.00   Median :2227   Median :2.000  
#>  Mean   :51.48   Mean   :2425   Mean   :2.034  
#>  3rd Qu.:65.00   3rd Qu.:3185   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4688   Max.   :3.000  
#> ------------------------------------------------------------ 
#> sex: 1
#>      thick            days          status     
#>  Min.   :16.00   Min.   :  30   Min.   :1.000  
#>  1st Qu.:30.00   1st Qu.:1820   1st Qu.:2.000  
#>  Median :65.00   Median :2886   Median :2.000  
#>  Mean   :55.75   Mean   :2632   Mean   :1.875  
#>  3rd Qu.:81.00   3rd Qu.:3328   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4207   Max.   :3.000
```


``` r
dsummary(melanoma,thick+status~+1|sex==1)
#>      thick            status     
#>  Min.   :  16.0   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1.000  
#>  Median : 258.0   Median :2.000  
#>  Mean   : 361.1   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :3.000
```

or 


``` r
dsummary(melanoma,~thick+status|sex==1)
#>      thick            status     
#>  Min.   :  16.0   Min.   :1.000  
#>  1st Qu.: 105.0   1st Qu.:1.000  
#>  Median : 258.0   Median :2.000  
#>  Mean   : 361.1   Mean   :1.722  
#>  3rd Qu.: 484.0   3rd Qu.:2.000  
#>  Max.   :1466.0   Max.   :3.000
```


For more complex conditions, use `I()`:


``` r
dsummary(melanoma,thick+days+status~sex|I(thick<97 & sex==1))
#> sex: 1
#>      thick            days          status     
#>  Min.   :16.00   Min.   :  30   Min.   :1.000  
#>  1st Qu.:30.00   1st Qu.:1820   1st Qu.:2.000  
#>  Median :65.00   Median :2886   Median :2.000  
#>  Mean   :55.75   Mean   :2632   Mean   :1.875  
#>  3rd Qu.:81.00   3rd Qu.:3328   3rd Qu.:2.000  
#>  Max.   :81.00   Max.   :4207   Max.   :3.000
```

Tables between variables


``` r
dtable(melanoma,~status+sex)
#> 
#>        sex  0  1
#> status          
#> 1          28 29
#> 2          91 43
#> 3           7  7
```

All bivariate tables


``` r
dtable(melanoma,~status+sex+ulc,level=2)
#> 
#>    status
#> sex  1  2  3
#>   0 28 91  7
#>   1 29 43  7
#> 
#>    status
#> ulc  1  2  3
#>   0 16 92  7
#>   1 41 42  7
#> 
#>    sex
#> ulc  0  1
#>   0 79 36
#>   1 47 43
```

All univariate tables


``` r
dtable(melanoma,~status+sex+ulc,level=1)
#> 
#> status
#>   1   2   3 
#>  57 134  14 
#> 
#> sex
#>   0   1 
#> 126  79 
#> 
#> ulc
#>   0   1 
#> 115  90
```

and with new variables 


``` r
dtable(melanoma,~status+sex+ulc+dcut(days)+I(days>300),level=1)
#> 
#> status
#>   1   2   3 
#>  57 134  14 
#> 
#> sex
#>   0   1 
#> 126  79 
#> 
#> ulc
#>   0   1 
#> 115  90 
#> 
#> dcut(days)
#>       [10,1.52e+03]    (1.52e+03,2e+03]    (2e+03,3.04e+03] (3.04e+03,5.56e+03] 
#>                  52                  51                  51                  51 
#> 
#> I(days > 300)
#> FALSE  TRUE 
#>    11   194
```


Sorting the data
===============

To sort the data 


``` r
data(melanoma)
mel= dsort(melanoma,~days)
dsort(melanoma) <- ~days
head(mel)
#>    no status days ulc thick sex
#> 1 789      3   10   1   676   1
#> 2  13      3   30   0    65   1
#> 3  97      2   35   0   134   1
#> 4  16      3   99   0   290   0
#> 5  21      1  185   1  1208   1
#> 6 469      1  204   1   484   1
```


and to sort after multiple variables increasing and decreasing 


``` r
dsort(melanoma) <- ~days-status
head(melanoma)
#>    no status days ulc thick sex
#> 1 789      3   10   1   676   1
#> 2  13      3   30   0    65   1
#> 3  97      2   35   0   134   1
#> 4  16      3   99   0   290   0
#> 5  21      1  185   1  1208   1
#> 6 469      1  204   1   484   1
```


Making new variables for the analysis
=====================================

To define a bunch of new covariates within a data-frame


``` r
data(melanoma)
melanoma= transform(melanoma, thick2=thick^2, lthick=log(thick) ) 
dhead(melanoma)
#>    no status days ulc thick sex  thick2   lthick
#> 1 789      3   10   1   676   1  456976 6.516193
#> 2  13      3   30   0    65   1    4225 4.174387
#> 3  97      2   35   0   134   1   17956 4.897840
#> 4  16      3   99   0   290   0   84100 5.669881
#> 5  21      1  185   1  1208   1 1459264 7.096721
#> 6 469      1  204   1   484   1  234256 6.182085
```

When the above definitions are done using a condition this can be achieved 
using the dtransform function that extends transform with a possible condition  


``` r
 melanoma=dtransform(melanoma,ll=thick*1.05^ulc,sex==1)  
 melanoma=dtransform(melanoma,ll=thick,sex!=1)  
 dmean(melanoma,ll~sex+ulc)
#>   sex ulc       ll
#> 1   0   0 173.7342
#> 2   1   0 197.3611
#> 3   0   1 374.5532
#> 4   1   1 523.1198
```


Making factors (groupings)
=============================

On the melanoma data, the variable `thick` gives the thickness of the melanoma tumour. For some analyses we would like to create a factor depending on the thickness. This can be done in several different ways:


``` r
melanoma=dcut(melanoma,~thick,breaks=c(0,200,500,800,2000))
```


New variable is named `thickcat.0` by default.

To see levels of factors in data-frame


``` r
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
```


Checking group sizes 


``` r
dtable(melanoma,~thickcat.0)
#> 
#> thickcat.0
#>     [0,200]   (200,500]   (500,800] (800,2e+03] 
#>         109          64          20          12
```


With adding to the data-frame directly 


``` r
dcut(melanoma,breaks=c(0,200,500,800,2000)) <- gr.thick1~thick
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> gr.thick1 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
```

new variable is named thickcat.0 (after first cut-point), 
or to get quartiles with default names thick.cat.4 


``` r
dcut(melanoma) <- ~ thick  # new variable is thickcat.4
dlevels(melanoma)
#> thickcat.0 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> gr.thick1 #levels=:4 
#> [1] "[0,200]"     "(200,500]"   "(500,800]"   "(800,2e+03]"
#> -----------------------------------------
#> thickcat.4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

or median groups, here starting again with the original data, 


``` r
data(melanoma)
dcut(melanoma,breaks=2) <- ~ thick  # new variable is thick.2
dlevels(melanoma)
#> thickcat.2 #levels=:2 
#> [1] "[10,194]"       "(194,1.74e+03]"
#> -----------------------------------------
```


to control new names


``` r
data(melanoma)
mela= dcut(melanoma,thickcat4+dayscat4~thick+days,breaks=4)
dlevels(mela)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> dayscat4 #levels=:4 
#> [1] "[10,1.52e+03]"       "(1.52e+03,2e+03]"    "(2e+03,3.04e+03]"   
#> [4] "(3.04e+03,5.56e+03]"
#> -----------------------------------------
```

or


``` r
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4+dayscat4~thick+days
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> dayscat4 #levels=:4 
#> [1] "[10,1.52e+03]"       "(1.52e+03,2e+03]"    "(2e+03,3.04e+03]"   
#> [4] "(3.04e+03,5.56e+03]"
#> -----------------------------------------
```

This can also be typed out more specifically


``` r
melanoma$gthick = cut(melanoma$thick,breaks=c(0,200,500,800,2000))
melanoma$gthick = cut(melanoma$thick,breaks=quantile(melanoma$thick),include.lowest=TRUE)
```


Working with factors
====================

To see levels of covariates in data-frame


``` r
data(melanoma)
dcut(melanoma,breaks=4) <- thickcat4~thick
dlevels(melanoma) 
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

To relevel the factor


``` r
dtable(melanoma,~thickcat4)
#> 
#> thickcat4
#>        [10,97]       (97,194]      (194,356] (356,1.74e+03] 
#>             56             53             45             51
melanoma = drelevel(melanoma,~thickcat4,ref="(194,356]")
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
```


or to take the third level in the list of levels, same as above, 


``` r
melanoma = drelevel(melanoma,~thickcat4,ref=2)
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(97,194]"       "[10,97]"        "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
```

To combine levels of a factor (first combining the first 3 groups into one):


``` r
melanoma = drelevel(melanoma,~thickcat4,newlevels=1:3)
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.(194,356] #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(97,194]"       "[10,97]"        "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.1:3 #levels=:2 
#> [1] "[10,97]-(194,356]" "(356,1.74e+03]"   
#> -----------------------------------------
```

or to combine groups 1 and 2 into one group and 3 and 4 into another


``` r
dkeep(melanoma) <- ~thick+thickcat4
melanoma = drelevel(melanoma,gthick2~thickcat4,newlevels=list(1:2,3:4))
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> gthick2 #levels=:2 
#> [1] "[10,97]-(97,194]"         "(194,356]-(356,1.74e+03]"
#> -----------------------------------------
```


Changing order of factor levels 


``` r
dfactor(melanoma,levels=c(3,1,2,4)) <-  thickcat4.2~thickcat4
dlevel(melanoma,~ "thickcat4*")
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
dtable(melanoma,~thickcat4+thickcat4.2)
#> 
#>                thickcat4.2 (194,356] [10,97] (97,194] (356,1.74e+03]
#> thickcat4                                                           
#> [10,97]                            0      56        0              0
#> (97,194]                           0       0       53              0
#> (194,356]                         45       0        0              0
#> (356,1.74e+03]                     0       0        0             51
```

Combine levels but now control factor-level names 


``` r
melanoma=drelevel(melanoma,gthick3~thickcat4,newlevels=list(group1.2=1:2,group3.4=3:4))
dlevels(melanoma)
#> thickcat4 #levels=:4 
#> [1] "[10,97]"        "(97,194]"       "(194,356]"      "(356,1.74e+03]"
#> -----------------------------------------
#> gthick2 #levels=:2 
#> [1] "[10,97]-(97,194]"         "(194,356]-(356,1.74e+03]"
#> -----------------------------------------
#> thickcat4.2 #levels=:4 
#> [1] "(194,356]"      "[10,97]"        "(97,194]"       "(356,1.74e+03]"
#> -----------------------------------------
#> gthick3 #levels=:2 
#> [1] "group1.2" "group3.4"
#> -----------------------------------------
```

Making a factor from existing numeric variable and vice versa
==============================================================

A numeric variable `status` with values 1, 2, 3 can be converted to a factor by:


``` r
data(melanoma)
melanoma = dfactor(melanoma,~status, labels=c("malignant-melanoma","censoring","dead-other"))
melanoma = dfactor(melanoma,sexl~sex,labels=c("females","males"))
dtable(melanoma,~sexl+status.f)
#> 
#>         status.f malignant-melanoma censoring dead-other
#> sexl                                                    
#> females                          28        91          7
#> males                            29        43          7
```

A gender factor with values `"M"`, `"F"` can be converted to numerics by:


``` r
melanoma = dnumeric(melanoma,~sexl)
dstr(melanoma,"sex*")
#> 'data.frame':	205 obs. of  3 variables:
#>  $ sex   : int  1 1 1 0 1 1 1 1 0 0 ...
#>  $ sexl  : Factor w/ 2 levels "females","males": 2 2 2 1 2 2 2 2 1 1 ...
#>  $ sexl.n: num  2 2 2 1 2 2 2 2 1 1 ...
dtable(melanoma,~'sex*',level=2)
#> 
#>          sex
#> sexl        0   1
#>   females 126   0
#>   males     0  79
#> 
#>       sex
#> sexl.n   0   1
#>      1 126   0
#>      2   0  79
#> 
#>       sexl
#> sexl.n females males
#>      1     126     0
#>      2       0    79
```

SessionInfo
============


``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] mets_1.3.10
#> 
#> loaded via a namespace (and not attached):
#>  [1] digest_0.6.39          numDeriv_2016.8-1.1    codetools_0.2-20      
#>  [4] RcppArmadillo_15.2.6-1 Matrix_1.7-5           xfun_0.57             
#>  [7] lattice_0.22-9         timereg_2.0.7          splines_4.6.0         
#> [10] lava_1.9.1             knitr_1.51             parallel_4.6.0        
#> [13] mvtnorm_1.3-7          cli_3.6.6              parallelly_1.47.0     
#> [16] grid_4.6.0             future_1.70.0          compiler_4.6.0        
#> [19] globals_0.19.1         tools_4.6.0            listenv_0.10.1        
#> [22] future.apply_1.20.2    evaluate_1.0.5         survival_3.8-6        
#> [25] Rcpp_1.1.1-1.1         otel_0.2.0             rlang_1.2.0
```

---
title: "A practical guide to Human Genetics with Lifetime Data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
#output: pdf_document
output:
  rmarkdown::html_vignette:
    #fig_width: 7.15
    #fig_height: 5.5
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{A practical guide to Human Genetics with Lifetime Data} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, label=setup}
knitr::opts_chunk$set(
  collapse = TRUE,
  ##dev="png",
  dpi=50,
  fig.width=7.15, fig.height=5.5,
  out.width="600px",
  fig.retina=1,
  comment = "#>"
  )
fullVignette <- Sys.getenv("_R_FULL_VIGNETTE_") %in% c("1","TRUE") || length(list.files("data"))==0
saveobj <- function(obj, null_elements) {
  x <- get(obj, envir=parent.frame())
  if (length(null_elements)>0) {
    print(object.size(x))
    x[null_elements] <- NULL
    attributes(x)[null_elements] <- NULL
    print(object.size(x))
  }
  saveRDS(x, paste0("data/", obj, ".rds"), compress="xz")
}

library(mets)
 cols <- c("darkred","darkblue","black")
 ltys <- c(1,3,2)
 fig_w <- 5
 fig_h <- 5
 savefig <- TRUE
```

```{r  results="hide", echo=FALSE, eval=!fullVignette}
## To save time building the vignettes on CRAN, we cache time consuming computations
fitco1 <- readRDS("data/fitco1.rds")
fitco2 <- readRDS("data/fitco2.rds")
fitco3 <- readRDS("data/fitco3.rds")
fitco4 <- readRDS("data/fitco4.rds")
fitace <- readRDS("data/fitace.rds")
fitde <- readRDS("data/fitde.rds")
cse <- readRDS("data/cse.rds")
slr <- readRDS("data/slr.rds")
outacem <- readRDS("data/outacem.rds")
b0 <- readRDS("data/b0.rds")
b1 <- readRDS("data/b1.rds")
b2 <- readRDS("data/b2.rds")
a1 <- readRDS("data/a1.rds")
h2 <- readRDS("data/h2.rds")
concMZ <- readRDS("data/concMZ.rds")
s_mz_country <- readRDS("data/s_mz_country.rds")
s_dz_country <- readRDS("data/s_dz_country.rds")
```


This vignette demonstrates how to analyze familial resemblance for 
twins using the \texttt{mets} {\bf R}-package and is accompanying the
review by Scheike and Holst (2020). 

We consider a data-set in  that resembles the data of \cite{Hjelmborg2014} that
were based on the NorTwinCan a collaborative research project studying the  genetic and environmental components of prostate
cancer.  The data comprises around 18,000 DZ twins and 11,000 MZ twins.
It was a population based register study based on the Danish, Finnish, Norwegian, and Swedish twin registries.

We first illustrate a hazards based analysis to show how one would study 
dependence in survival data. This needs to be done under assumptions about independent 
competing risks
when the outcome of interest is observed subject to competing risks (here death). 

This seems reasonable here since the occurrence of cancer prior to death only contains weak
association with the risk of death for the other twin, and vice-versa. 

First looking at the data

```{r, label=data-prt}
library(mets)
 set.seed(122)
 data(prt)
 
 dtable(prt,~status+cancer)
 dtable(prt,~zyg+country,level=1)
```

we see that there are 21283 censorings and 6997 deaths (prior to cancer) and
a total of 942 prostate cancers. 
Approximately half the data consist of DZ twins. 
In addition we see that there are around 10000 twins from Denmark and Sweden, and only 4000 from Norway and Finland, respectively.

Survival 
==========

Under assumption of random effects acting independently on different cause
specific hazards we can analyse competing risks data considering the
cause-specific hazard.  Typically, this can be questionable and the cumulative
incidence modelling below does not rely on this assumption. 

We consider the cause specific hazard of cancer in the competing risks
model with death and cancer.  

First estimating the marginal hazards for each country. 

```{r, label=survival-marginal}
 # Marginal Cox model here stratified on country without covariates 
 margph <- phreg(Surv(time,cancer)~strata(country)+cluster(id),data=prt)
 plot(margph)
```

We see that the marginal of Denmark in particular is quite different. 

Then we fit a two-stage random effects models with country specific 
marginals and random-effects variances that differ for MZ and DZ twins. 


```{r, label=survival-pairwise1, eval=fullVignette}
# Clayton-Oakes, MLE , overall variance
fitco1<-twostageMLE(margph,data=prt,theta=2.7)
```

```{r}
 summary(fitco1)
```

```{r, label=survival-pairwise2, eval=fullVignette}
fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)
```

```{r}
 summary(fitco2)
```

With different random effects for MZ and DZ
```{r label=survival-pairwise3, eval=fullVignette}
 mm <- model.matrix(~-1+factor(zyg),prt)
 fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)
```


```{r}
 summary(fitco3)
```

```{r label=survival-pairwise4, eval=fullVignette}
fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)
```

```{r}
 summary(fitco4)
 round(estimate(coef=fitco4$coef,vcov=fitco4$var.theta)$coefmat[,c(1,3:4)],2)

 ## mz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[2],0,K=1000)$mz.kendall
 ## dz kendalls tau
 kendall.ClaytonOakes.twin.ace(fitco4$theta[1],0,K=1000)$mz.kendall
```


The dependence of MZ twins is much stronger, and is summarized by a 
variance at $`r round(fitco4$coef[2],2)`$ in contrast to the $DZ$ variance at $`r round(fitco4$coef[1],2)`$.


Now we look at the polygenic modelling for survival data, here applied to
the cause specific hazards. 

```{r, label=survival-polygenic1, eval=fullVignette}
 ### setting up design for random effects and parameters of random effects
 desace <- twin.polygen.design(prt,type="ace")

 ### ace model 
 fitace <- survival.twostage(margph,data=prt,theta=1,
       clusters=prt$id,var.link=0,model="clayton.oakes",
       numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
```

```{r}
 summary(fitace)
```

```{r label=survival-polygenic2, eval=fullVignette} 
 ### ace model with positive random effects variances
 # fitacee <- survival.twostage(margph,data=prt,theta=1,
 #      clusters=prt$id,var.link=1,model="clayton.oakes",
 #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
 #summary(fitacee)
 
 ### ae model 
 #desae <- twin.polygen.design(prt,type="ae")
 #fitae <- survival.twostage(margph,data=prt,theta=1,
 #      clusters=prt$id,var.link=0,model="clayton.oakes",
 #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
 #summary(fitae)

 ### de model 
 desde <- twin.polygen.design(prt,type="de")
 fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)                            

```{r}
summary(fitde)
```

The DE model  fits quite well. In summary all shared variance is due to 
genes and there is no suggestion of a shared environmental effect. 


Concordance and Casewise 
========================

First we estimate the concordance of joint prostate cancer. The two-twins are
censored at the same time, otherwise we would enforce this in the data by
artificially censor both twins at the first censoring time.  Given, however,
that we have the same-censoring assumption satisfied we can do the stanadar
Aalen-Johansen product-limit estimator of the concordance probabilities for MZ
and DZ twins.

For simplicity we do not do this for each country even though as we show there are big
differences between the countries. 


```{r}
prt <-  force.same.cens(prt,cause="status")

 dtable(prt,~status+cancer)
 dtable(prt,~status+country)
 dtable(prt,~zyg+country)
```


```{r, label=concordance}
 ## cumulative incidence with cluster standard errors.
 cif1 <- cif(Event(time,status)~strata(country)+cluster(id),prt,cause=2)
 plot(cif1,se=1)

 cifa <- cif(Event(time,status)~+1,prt,cause=2)

 ### concordance estimator, ignoring country differences. 
 p11 <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2))
p11mz <- p11$model$"MZ"
p11dz <- p11$model$"DZ" 
```
 
```{r}
 par(mfrow=c(1,2))
 ## Concordance
 plot(p11mz,ylim=c(0,0.1));
 plot(p11dz,ylim=c(0,0.1));
```

Now we compare the concordance to the marginals to get a measure 
that takes the marginals into account when evaluating the strength 
of the association. 
     
```{r, label=concordance2}
 library(prodlim)
 outm <- prodlim(Hist(time,status)~+1,data=prt)

 cifzyg <- cif(Event(time,status)~+strata(zyg)+cluster(id),data=prt,cause=2)
 cifprt <- cif(Event(time,status)~country+cluster(id),data=prt,cause=2)
     
 times <- 70:100
 cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ")) ## cause is 2 (second cause) 
 cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))
    
 ### concordance for MZ and DZ twins<
 cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
 ccdz <- cc$model$"DZ"
 ccmz <- cc$model$"MZ"
     
 cdz <- casewise(ccdz,outm,cause.marg=2) 
 cmz <- casewise(ccmz,outm,cause.marg=2)

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,col=cols[2:1], lty=ltys[2:1], legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-MZ","concordance-DZ"),col=cols[1:2],lty=ltys[1:2])

 plot(cmz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,col=cols[c(1,3,3)],lty=ltys[c(1,3,3)],
      ylab="Casewise",xlab="Age")
  plot(cdz,ci=NULL,ylim=c(0,.8),xlim=c(70,97),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(cols[2],NA,NA), lty=ltys[c(2,3,3)], add=TRUE)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[1]))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=cols[2]))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=cols, lty=ltys, bg="white")

 summary(cdz)
 summary(cmz)

 timereg::Cpred(cmz$casewise,80)
 timereg::Cpred(cdz$casewise,80)
```


```{r, label=concordance3}

 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 conczyg <- cif(Event(time,status)~strata(zyg)+cluster(id),data=dd,cause=1)

 par(mfrow=c(1,2))
 plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance")
 legend("topleft",c("concordance-DZ","concordance-MZ"),col=c(1,2),lty=1)
 plot(cmz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,col=c(2,3,3),ylab="Casewise",xlab="Age",lty=c(1,3))
 plot(cdz,ci=NULL,ylim=c(0,0.6),xlim=c(70,100),legend=FALSE,ylab="Casewise",xlab="Age",
      col=c(1,3,3), add=TRUE, lty=c(2,3))
 legend("topleft",c("casewise-MZ","casewise-DZ","marginal"),col=c(2,1,3),lty=1)
 with(data.frame(cmz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=2))
 with(data.frame(cdz$casewise),plotConfRegionSE(time,casewise.conc,se.casewise,col=1))

```

The standard errors above are slightly off since they only reflect the 
uncertainty from the concordance estimation. This can be improved by
doing specific calculations for a specific time-point uisng the 
binomial regression function that gives and iid decomposition for the 
paramters.  We thus apply the binomial regression to estimate the
concordance as well as the marginal, and combine the iid decompositions
when estimating the standard error.  We also do this ignoring country differences. 


```{r, label=concordance4, eval=fullVignette}
 ### new version of Casewise for specific time-point based on binreg 
 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 newdata <- data.frame(zyg=c("DZ","MZ"),id=1)

 ## concordance 
 bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
 pconc <- predict(bcif1,newdata)

 ## marginal estimates
 mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
 mc <- predict(mbcif1,newdata)

 ### casewise with improved se's from log-scale 
 cse <- binregCasewise(bcif1,mbcif1)
```

```{r} 
 cse 
```

It can be useful also to simply model the concordance given covariates, and in
this case we might find it important to adjust for country, or to see if the
differences between MZ and DZ are comparable across contries even though
clearly DK has a much lower cumulative incidence of prostate cancer. 

```{r, label=semiparconc, eval=fullVignette}
 ### semi-parametric modelling of concordance 
 dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
 regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
 regconc
 ### interaction test
 wald.test(regconc,coef.null=5:7)

 regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
 regconc

 ## logistic link 
 logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
 slr <- summary(logitregconc)
```

```{r}
slr
### library(Publish)
### publish(round(slr$exp.coef[,-c(2,5)],2),latex=TRUE,digits=2)
```

Competing risk using additive Gamma
====================================

Here we do the cumulative incidence random effects modelling 


```{r, label=additive_gamma, eval=fullVignette}
  times <- seq(50,90,length.out=5)
  cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

  desace <- twin.polygen.design(prt,type="ace")
 
  outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
  	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
         step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
  ##outacem$score
```

```{r}  
  summary(outacem)

 ###  variances
 estimate(coef=outacem$theta,vcov=outacem$var.theta,f=function(p) p/sum(p)^2)

 ## AE polygenic model
 # desae <- twin.polygen.design(prt,type="ae")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #    same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
 #        step=0.5,theta.des=desae$pardes,random.design=desae$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p)     p/sum(p)^2)

 ## AE polygenic model
 # desde <- twin.polygen.design(prt,type="de")
 # outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
 #   same.cens=TRUE,theta=c(0.35),var.link=0,
 #   step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 # outaem$score
 # summary(outaem)
 # estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

  times <- 90
  cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
		   cause=2,times=times,max.clust=NULL)

  mm <- model.matrix(~-1+factor(zyg),prt)
  out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
		  theta.des=mm,same.cens=TRUE,step=0.5)
  summary(out1)
  round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)

 desde <- twin.polygen.design(prt,type="de")
 outaem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
	same.cens=TRUE,theta=c(0.35),var.link=0,
        step=0.5,theta.des=desde$pardes,random.design=desde$des.rv)
 outaem$score
 summary(outaem)
 estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)

```


Competing risk modeling using the Liabilty Threshold model 
===========================================================


First we fit the bivariate probit model (same marginals in MZ and DZ twins but different correlation parameter). Here we evaluate the risk of getting cancer before the last double cancer event (95 years)
```{r, label=probit1}
rm(prt)
data(prt)
prt0 <-  force.same.cens(prt, cause="status", cens.code=0, time="time", id="id")
prt0$country <- relevel(prt0$country, ref="Sweden")
prt_wide <- fast.reshape(prt0, id="id", num="num", varying=c("time","status","cancer"))
prt_time <- subset(prt_wide,  cancer1 & cancer2, select=c(time1, time2, zyg))
tau <- 95
tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in
```

```{r b0, eval=fullVignette}
b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
```

```{r}
summary(b0)
```


Liability threshold model with ACE random effects structure
```{r, label=liability_ace1, eval=fullVignette}
b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
              cens.formula=Surv(time,status==0)~zyg, breaks=tau)
```

```{r}
summary(b1)
```

In this case the ACE model fits the data well - it is in fact indistinguishable from the flexible bivariate Probit model as seen by the IPCW weighted AIC measure
```{r}
AIC(b0, b1)
```

ACE model with marginal adjusted for country
```{r, label=liability_ace_country, eval=fullVignette}
b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
              cens.formula=Surv(time,status==0)~zyg+country, breaks=95)
```

```{r}
summary(b2)
```

```{r, label=bptime1, eval=fullVignette}
bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace", 
              cens.formula=Surv(time,status==0)~zyg,
              summary.function=function(x) x, breaks=tt)
h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))

```

```{r}
par(mfrow=c(1,2))
plot(tt, h2[,1], type="s", lty=1, col=cols[3], xlab="Age", ylab="Heritability", ylim=c(0,1))
lava::confband(tt, h2[,2], h2[,3],polygon=TRUE, step=TRUE, col=lava::Col(cols[3], 0.1), border=NA)
plot(tt, concMZ[,1], type="s", lty=1, col=cols[1], xlab="Age", ylab="Concordance", ylim=c(0,.1))
lava::confband(tt, concMZ[,2], concMZ[,3],polygon=TRUE, step=TRUE, col=lava::Col(cols[1], 0.1), border=NA)
```


Bivariate probit model at time different time points
```{r, label=biprobittime1}
system.time(a.mz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~1, pairs.only=TRUE,
                                breaks=tt))
system.time(a.dz <- biprobit.time(cancer~1, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~1, pairs.only=TRUE,
                               breaks=tt))

#system.time(a.zyg <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, 
#                               cens.formula = Event(time,status==0)~1,
#                               eqmarg=FALSE, fix.cens.weight
#                               breaks=seq(75,100,by=10)))

a.mz
a.dz

plot(conczyg,se=TRUE,legend=FALSE,xlab="Age",ylab="Concordance", ylim=c(0,0.07))
plot(a.mz, ylim=c(0,.07), col=cols[1], lty=ltys[1], legend=FALSE, add=TRUE)
plot(a.dz, col=cols[2], lty=ltys[2], add=TRUE)
```


Bivariate probit model adjusting for country
```{r, label=biprobittime2, eval=fullVignette}
a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
                               cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
                                breaks=tt)
system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
                               cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
                               breaks=tt))

s_mz_country <- summary(a.mz_country)
s_dz_country <- summary(a.dz_country)
```

```{r}
s_mz_country
s_dz_country
```



```{r, label=liability_ace_time1, eval=fullVignette}
## ACE model (time-varying) with and without adjustment for country
a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
                              zyg="zyg", DZ="DZ", 
                              cens.formula=Surv(time,status==0)~zyg,
                              breaks=tt)

#a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
#                              zyg="zyg", DZ="DZ", 
#                              #cens.formula=Surv(time,status==0)~country+zyg,
#                              breaks=tt)
```


```{r}
plot(a.mz, which=c(6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1], lty=ltys[1], legend=NULL, alpha=.1)
plot(a.dz, which=c(6), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, alpha=.1)
legend("topleft", c("MZ tetrachoric correlation", "DZ tetrachoric correlation"),
       col=cols, lty=ltys, lwd=2)

plot(a.mz, which=c(4), xlab="Age", ylab="Relative Recurrence Risk",
     ylim=c(1,20), col=cols[1], lty=ltys[1], legend=NULL, lwd=2, alpha=.1)
plot(a.dz, which=c(4), col=cols[2], lty=ltys[2], legend=NULL, add=TRUE, lwd=2, alpha=.1)
legend("topright", c("MZ relative recurrence risk", "DZ relative recurrence risk"),
       col=cols, lty=ltys, lwd=2)

plot(a1, which=c(5,6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1:2], lty=ltys[1:2], lwd=2, alpha=0.1,
     legend=c("MZ tetrachoric correlation", "DZ tetrachoric correlation"))

plot(a1, which=c(1), xlab="Age", ylim=c(0,1), col="black", lty=1, ylab="Heritability", legend=NULL, alpha=.1)
```

SessionInfo
============

```{r}
sessionInfo()
```


```{r  saveobj, results="hide", echo=FALSE, eval=fullVignette }
## To save time building the vignettes on CRAN, we cache time consuming computations

rms <- c('id','theta.iid','theta.des','marginal.trunc',
  'loglikeiid','marginal.surv','theta.iid.naive',
  'antclust','secluster','cluster.call','trunclikeiid',
  'logl.iid','score.iid','clusters')  
tmp <- lapply(as.list(paste0("fitco", 1:4)),
              function(x) saveobj(x, rms))

rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
saveobj("fitace", rms)
saveobj("fitde", rms)

saveobj("cse", NULL)
saveobj("slr", NULL)

rms <- c("theta.iid", "Clusters", "p11")
saveobj("outacem", rms)

rms <- c("model.frame", "score", "id", "logLik")
saveobj("b0", rms)
saveobj("b1", rms)
saveobj("b2", rms)
saveobj("a1", rms)
saveobj("h2", NULL)
saveobj("concMZ", NULL)
saveobj("s_mz_country", NULL)
saveobj("s_dz_country", NULL)
```

---
title: "Recurrent events"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
header-includes: 
  - \usepackage{tikz}
  - \usepackage{adjustbox}
  - \usetikzlibrary{positioning, arrows.meta,calc}
vignette: >
  %\VignetteIndexEntry{Recurrent events}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview 
========

For recurrent events data it is often of interest to compute basic descriptive
quantities to gain some understanding of the phenomenon being studied.
We demonstrate how to compute:

 * the marginal mean 
   * AUC and logrank test for comparison of marginal means (also for special case with cumulative incidence)
   * efficient marginal mean estimation with fast computation of standard errors
 * the Ghosh-Lin Cox type regression for the marginal mean, possibly with composite outcomes. 
   * efficient regression augmentation of the Ghosh-Lin model
   * clusters can be specified
   * allows a stratified baseline 
 * the variance of a recurrent events process
 * the probability of exceeding k events
 * the two-stage recurrent events random effects model

In addition several tools can be used for simulating recurrent events and
bivariate recurrent events data, also with a possible terminating event: 

 * recurrent events with multiple 
   * event types 
   * Cox type rates 
 * with a terminal event with possibly multiple causes of death
   * Cox type rates 
 * frailty extensions
 * the Ghosh-Lin model when the survival rate is on Cox form.
   * frailty extensions
 * The general illness death model with  Cox models for all hazards. 

\begin{tikzpicture}[
    sharp corners=2pt,
    inner sep=4pt,
    node distance=0.9cm,
    >=latex,
    mynode/.style={draw,minimum height=0.9cm,minimum width=2.6cm}
]

\node[mynode] (healthy) {healthy $(h)$};
\node[mynode, right=of healthy] (e1) {1st event};
\node[mynode, right=of e1] (e2) {2nd event};

% Dots indicating continuation
\node[right=0.6cm of e2] (dots) {$\cdots$};

% Dead
\node[mynode, below=of e1] (dead) {dead $(d)$};

% Arrows along the event chain
\draw[->] (healthy) -- (e1);
\draw[->] (e1) -- (e2);
\draw[->] (e2) -- (dots);

% Arrows to 'dead'
\draw[->] (healthy) -- (dead);
\draw[->] (e1) -- (dead);
\draw[->] (e2) -- (dead);

\end{tikzpicture}


Simulation of recurrent events
==============================

We start by simulating some recurrent events data with two event types having cumulative hazards

 * $\Lambda_1(t)$ (rate among survivors)
 * $\Lambda_2(t)$ (rate among survivors)
 * $\Lambda_D(t)$

 where types 1 and 2 are considered and the terminal event rate is given by
 $\Lambda_D(t)$. The events are independent by default, but a random effects
 structure can also be specified to generate dependence.

 When simulating data, various random effects structures can be imposed to generate dependence:

* Dependence=0: The intensities are independent.

* Dependence=1: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $Z \lambda_D(t)$

* Dependence=4: One gamma-distributed random effect $Z$. The intensities are
    + $Z \lambda_1(t)$
    + $Z \lambda_2(t)$
    + $\lambda_D(t)$

* Dependence=2: Normally distributed random effects $Z_1,Z_2,Z_d$ with variance (`var.z`) 
  and correlation (`cor.mat`) that can be specified. The intensities are
    + $\exp(Z_1) \lambda_1(t)$
    + $\exp(Z_2) \lambda_2(t)$
    + $\exp(Z_3) \lambda_D(t)$

* Dependence=3: Gamma-distributed random effects $Z_1,Z_2,Z_d$ where the sum structure can be specified via a matrix
   `cor.mat`. We compute $\tilde Z_j = \sum_k  Z_k^{cor.mat(j,k)}$ for $j=1,2,3$.
   The intensities are
    + $\tilde Z_1 \lambda_1(t)$
    + $\tilde Z_2 \lambda_2(t)$
    + $\tilde Z_3 \lambda_D(t)$

We return to how to run the different set-ups later and start by simulating independent processes. 

The key simulation functions are:

 * `sim_recurrent`: simple simulation with one event type and death
 * `sim_recurrentII`: extended version with possibly multiple types of recurrent events (rates can be zero);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent_list`: lists allowed for multiple event types and causes of death (competing risks);
   allows Cox-type rates with subject-specific effects
 * `sim_recurrent`: simulates from Cox-Cox (marginals) or Ghosh-Lin-Cox models

Data can also be simulated from the Ghosh-Lin model where marginal rates among survivors are on Cox form:

 * `simGLcox` (also `simRecurrentCox`): simulates from the Ghosh-Lin model
   * with frailties, where the survival model for the terminal event is on Cox form
   * simulates data where rates among survivors are on Cox form, with or without frailties

see examples below for specific models. 


Utility functions
=================

Two utility functions worth noting:

 * `tie.breaker`: breaks ties among jump times, which is expected by the functions below.
 * `count_history`: counts the number of previous jumps for each subject, i.e. $N_1(t-)$ and $N_2(t-)$.

Marginal Mean 
=============

We start by estimating the marginal mean $E(N_1(t \wedge D))$ where $D$ is the time of the terminal event.
The marginal mean is the average number of events observed before time $t$.

This is based on a two-rate model for

 * the type 1 events: $E(dN_1(t) | D > t)$
 * the terminal event: $E(dN_d(t) | D > t)$

and is defined as $\mu_1(t)=E(N_1(t))$ 
\begin{align}
   \int_0^t S(u) d R_1(u)	
\end{align}
where $S(t)=P(D \geq t)$ and $dR_1(t) = E(dN_1(t) | D > t)$ 

and can therefore be estimated by a 

 * Kaplan-Meier estimator, $\hat S(u)$ 
 * Nelson-Aalen estimator for $R_1(t)$

\begin{align}
  \hat R_1(t) & =   \sum_i \int_0^t  \frac{1}{Y_\bullet (s)}  dN_{1i}(s)
\end{align}
where $Y_{\bullet}(t)= \sum_i Y_i(t)$ such that the estimator is 
\begin{align}
  \hat \mu_1(t) & =    \int_0^t \hat S(u) d\hat R_1(u),
\end{align}
see Cook & Lawless (1997) and Ghosh & Lin (2000).

The variance can be estimated based on the asymptotic expansion
of $\hat \mu_1(t) - \mu_1(t)$
\begin{align*}
  & \sum_i \int_0^t \frac{S(s)}{\pi(s)} dM_{i1}  - \mu_1(t) \int_0^t  \frac{1}{\pi(s)} dM_i^d +  \int_0^t \frac{\mu_1(s) }{\pi(s)} dM_i^d,
\end{align*}

with mean-zero processes 

 * $M_i^d(t) = N_i^D(t)- \int_0^t Y_i(s) d \Lambda^D(s)$, 
 * $M_{i1}(t) = N_{i1}(t) - \int_0^t Y_{i}(s) dR_1(s)$. 

as described in Ghosh & Lin (2000).


Generating data
==================

We start by generating some data to illustrate the computation of the marginal mean 


``` r
library(mets)
set.seed(1000) # to control output in simulations for p-values below.
```


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

rr <- sim_recurrent(400,base1,death.cumhaz=dr)
rr$x <- rnorm(nrow(rr)) 
rr$strata <- floor((rr$id-0.01)/100)
dlist(rr,.~id| id %in% c(1,7,9))
#> id: 1
#>      entry  time   status dtime fdeath death start  stop   x        strata
#> 1       0.0  451.1 1      3291  1      0        0.0  451.1 -0.39707 0     
#> 401   451.1  485.6 1      3291  1      0      451.1  485.6 -1.04577 0     
#> 679   485.6 1276.1 1      3291  1      0      485.6 1276.1  0.09424 0     
#> 867  1276.1 1278.8 1      3291  1      0     1276.1 1278.8  2.58113 0     
#> 1019 1278.8 1618.3 1      3291  1      0     1278.8 1618.3  0.00696 0     
#> 1148 1618.3 1752.9 1      3291  1      0     1618.3 1752.9 -0.81183 0     
#> 1256 1752.9 2466.3 1      3291  1      0     1752.9 2466.3 -0.52972 0     
#> 1344 2466.3 2550.8 1      3291  1      0     2466.3 2550.8  0.03179 0     
#> 1411 2550.8 3290.8 0      3291  1      1     2550.8 3290.8 -0.72717 0     
#> ------------------------------------------------------------ 
#> id: 7
#>   entry time  status dtime fdeath death start stop  x       strata
#> 7 0     658.3 0      658.3 1      1     0     658.3 -0.2209 0     
#> ------------------------------------------------------------ 
#> id: 9
#>     entry time  status dtime fdeath death start stop  x      strata
#> 9     0.0 433.5 1      505.3 1      0       0.0 433.5 0.3552 0     
#> 405 433.5 505.3 0      505.3 1      1     433.5 505.3 0.2355 0
```

The status variable keeps track of the recurrent events and their type, and death the timing of 
death.

To compute the marginal mean we simply estimate the two rates functions of the 
number of events of interest and death by using the phreg function 
(to start without covariates). Then the estimates are combined with standard 
error computation in the recurrent_marginal function


``` r
#  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status)~cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
# robust standard errors 
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(entry,time,status)~cluster(id),data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=2)
```

<img src="figure/recurrent-events-unnamed-chunk-4-1.png" alt="Death rate, event rate, and marginal mean."  />

We can also extract the estimate in different time-points 


``` r
summary(out,times=c(1000,2000))
#> [[1]]
#>     new.time     mean         se  CI-2.5% CI-97.5% strata
#> 495     1000 1.857035 0.08767256 1.692911 2.037072      0
#> 779     2000 3.802964 0.14690314 3.525668 4.102068      0
```

The marginal mean can also be estimated in the stratified case:


``` r
xr <- phreg(Surv(entry,time,status)~strata(strata)+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~strata(strata)+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <-   robust_phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrent_marginal(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

<img src="figure/recurrent-events-unnamed-chunk-6-1.png" alt="Stratified death rate, event rate, and marginal mean."  />

We can compare different marginal mean test (IPCW based) with a log-rank test  


``` r
test_logrankRecurrent(out)
#>    Estimate Std.Err   2.5%  97.5% P-value
#> p1   10.368   14.44 -17.93 38.661 0.47264
#> p2  -22.069   12.43 -46.43  2.288 0.07575
#> p3    2.019   15.72 -28.80 32.838 0.89785
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [p1] = 0
#>   [p2] = 0
#>   [p3] = 0 
#>  
#> chisq = 3.3471, df = 3, p-value = 0.3411

dd <- test_marginalMean(Event(entry,time,status)~strata(strata)+cluster(id),
			 data=rr,cause=1,death.code=2)
dd
#> coeffients:
#>                              p-value
#> time                       5079.7045
#> Pepe-Mori                         NA
#> Ratio-AUC                     0.2147
#> Proportionality               0.3865
#> Proportionality-score-test    0.3920
summary(dd)
#> coeffients:
#>                              p-value
#> time                       5079.7045
#> Pepe-Mori                         NA
#> Ratio-AUC                     0.2147
#> Proportionality               0.3865
#> Proportionality-score-test    0.3920
dd$RAUCl
#>    n events
#>  400   1184
#> 
#>  400 clusters
#> coeffients:
#>                          Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept)              24707.72  1498.20 21771.30 27644.15  0.0000
#> factor(strata__)strata=1 -3840.23  2028.83 -7816.67   136.20  0.0584
#> factor(strata__)strata=2 -1015.76  2026.74 -4988.10  2956.57  0.6162
#> factor(strata__)strata=3  -351.09  1948.38 -4169.84  3467.66  0.8570
```

The AUC suggest that strata 2 looses 3840 days less than strata 0. 


If we adjust for covariates for the two rates we can still do
predictions of marginal mean, what can be plotted is the baseline marginal mean, 
that is for the covariates equal to 0 for both models. Predictions for specific 
covariates can also be obtained with the recmarg (recurren marginal mean used 
solely for predictions without standard error computation). 


``` r
# cox case
xr <- phreg(Surv(entry,time,status)~x+cluster(id),data=rr)
xdr <- phreg(Surv(entry,time,death)~x+cluster(id),data=rr)
par(mfrow=c(1,3))
plot(xdr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
rxr <- robust_phreg(xr)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=1:2)

out <- recurrentMarginalPhreg(xr,xdr)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)
```

<img src="figure/recurrent-events-unnamed-chunk-8-1.png" alt="Cox-adjusted death rate, event rate, and marginal mean."  />

``` r

#### predictions witout se's 
###outX <- recmarg(xr,dr,Xr=1,Xd=1)
###plot(outX,add=TRUE,col=3)
```

We here simulate multiple recurrent events processes with two causes of death causes  and exponential censoring with
rate $3/5000$, all processes are assumed independent (dependence=0)


``` r
rr <- sim_recurrent_list(100,list(base1,base1,base4),death.cumhaz=list(dr,base4),cens=3/5000,dependence=0)
dtable(rr,~status+death,level=2)
#> 
#>      status
#> death   0   1   2   3
#>     0  37 135 165  12
#>     1  54   0   0   0
#>     2   9   0   0   0
mets:::showfitsimList(rr,list(base1,base1,base4),list(dr,base4))
```

<img src="figure/recurrent-events-unnamed-chunk-9-1.png" alt="Simulated vs true rates for multiple recurrent event types."  />

Improving efficiency 
======================

To illustrate how the efficiency can be improved using heterogenity in the data, we
now simulate some data with strong heterogenity. The dynamic augmentation is 
a regression on the history  for each subject consisting of the specified terms 
terms:  Nt, Nt2 (Nt squared), expNt (exp(-Nt)), NtexpNt (Nt*exp(-Nt)) or by simply
specifying these directly.  This was developed in Cortese and Scheike (2022).


``` r
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <-  count_history(rr)

rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
dtable(rr,~statusD+status+death,level=2,response=1)
#> 
#>       statusD
#> status   0   1   2   3
#>      0  83   0   0 117
#>      1   0 375   0   0
#>      2   0   0  44   0
#> 
#>      statusD
#> death   0   1   2   3
#>     0  83 375  44   0
#>     1   0   0   0 117

##xr <- phreg(Surv(start,stop,status==1)~cluster(id),data=rr)
##dr <- phreg(Surv(start,stop,death)~cluster(id),data=rr)
# marginal mean of expected number of recurrent events 
out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)

times <- 500*(1:10)
recEFF1 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
				   death.code=3,cause=1,augment.model=~Nt)
with( recEFF1, cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
#>       times       muP     semuP     muPAt   semuPAt          
#>  [1,]   500 0.8744004 0.1059068 0.8670372 0.1055614 0.9967382
#>  [2,]  1000 1.2696069 0.1458220 1.2621656 0.1447864 0.9928981
#>  [3,]  1500 1.7921498 0.2291966 1.7957473 0.2240253 0.9774373
#>  [4,]  2000 2.1216192 0.3047378 2.1595208 0.2937613 0.9639804
#>  [5,]  2500 2.4775306 0.3934784 2.5544338 0.3713749 0.9438254
#>  [6,]  3000 2.7947412 0.5183678 2.9101041 0.4625630 0.8923451
#>  [7,]  3500 3.0289847 0.6033927 3.1307552 0.5193403 0.8607004
#>  [8,]  4000 3.3163659 0.7183096 3.3392837 0.5769952 0.8032681
#>  [9,]  4500 3.6229966 0.8415333 3.5349425 0.6152266 0.7310781
#> [10,]  5000 3.8810150 0.9255727 3.6777784 0.6159133 0.6654402

times <- 500*(1:10)
###recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
###death.code=3,cause=1,augment.model=~Nt+Nt2+expNt+NtexpNt)
###with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))

recEFF14 <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,cens.code=0,
death.code=3,cause=1,augment.model=~Nt+I(Nt^2)+I(exp(-Nt))+ I( Nt*exp(-Nt)))
with(recEFF14,cbind(times,muP,semuP,muPAt,semuPAt,semuPAt/semuP))
#>       times       muP     semuP     muPAt   semuPAt          
#>  [1,]   500 0.8744004 0.1059068 0.8666135 0.1054672 0.9958484
#>  [2,]  1000 1.2696069 0.1458220 1.2643398 0.1444958 0.9909053
#>  [3,]  1500 1.7921498 0.2291966 1.8063914 0.2218548 0.9679670
#>  [4,]  2000 2.1216192 0.3047378 2.1433122 0.2888682 0.9479237
#>  [5,]  2500 2.4775306 0.3934784 2.4921659 0.3607250 0.9167593
#>  [6,]  3000 2.7947412 0.5183678 2.6962370 0.4264556 0.8226892
#>  [7,]  3500 3.0289847 0.6033927 2.7724842 0.4558824 0.7555319
#>  [8,]  4000 3.3163659 0.7183096 2.8716708 0.4752409 0.6616101
#>  [9,]  4500 3.6229966 0.8415333 2.9362144 0.4519118 0.5370100
#> [10,]  5000 3.8810150 0.9255727 2.9717225 0.3831618 0.4139726

plot(out,se=TRUE,ylab="marginal mean",col=2)
k <- 1
for (t in times) {
	ci1 <- c(recEFF1$muPAt[k]-1.96*recEFF1$semuPAt[k],
  	         recEFF1$muPAt[k]+1.96*recEFF1$semuPAt[k])
	ci2 <- c(recEFF1$muP[k]-1.96*recEFF1$semuP[k],
  	         recEFF1$muP[k]+1.96*recEFF1$semuP[k])
	lines(rep(t,2)-2,ci2,col=2,lty=1,lwd=2)
	lines(rep(t,2)+2,ci1,col=1,lty=1,lwd=2)
	k <- k+1
}
legend("bottomright",c("Eff-pred"),lty=1,col=c(1,3))
```

<img src="figure/recurrent-events-unnamed-chunk-10-1.png" alt="Augmented marginal mean estimates."  />

In the case where covariates might be important but we are still interested in the marginal mean 
we can also augment wrt these covariates 


``` r
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,fz,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])

dtable(rr,~statusD+status+death,level=2,response=1)
#> 
#>       statusD
#> status   0   1   3
#>      0  92   0 108
#>      1   0 529   0
#> 
#>      statusD
#> death   0   1   3
#>     0  92 314   0
#>     1   0 215 108

times <- seq(500,5000,by=500)
recEFF1x <- recurrent_marginalAIPCW(Event(start,stop,statusD)~cluster(id),data=rr,times=times,
				   cens.code=0,death.code=3,cause=1,augment.model=~X1+X2)
with(recEFF1x, cbind(muP,muPA,muPAt,semuP,semuPA,semuPAt,semuPAt/semuP))
#>            muP     muPA    muPAt      semuP     semuPA    semuPAt          
#>  [1,] 1.050317 1.052415 1.048871 0.08835005 0.08801111 0.08789822 0.9948859
#>  [2,] 1.843830 1.863996 1.838716 0.14730045 0.14606950 0.14556961 0.9882496
#>  [3,] 2.714728 2.768829 2.724566 0.24272224 0.23810724 0.23756192 0.9787398
#>  [4,] 3.546522 3.636583 3.591319 0.36401457 0.35109326 0.34874650 0.9580564
#>  [5,] 4.492684 4.658458 4.537357 0.51416355 0.50043401 0.49610099 0.9648700
#>  [6,] 5.080889 5.209681 5.047836 0.60551061 0.58818767 0.58285931 0.9625914
#>  [7,] 5.638012 5.701306 5.495962 0.70065049 0.67382134 0.66728557 0.9523801
#>  [8,] 6.895109 6.830233 6.447130 0.95745240 0.89622952 0.87919085 0.9182606
#>  [9,] 7.723651 6.794796 6.727887 1.15096828 0.99740795 0.96198463 0.8358046
#> [10,] 8.923607 7.170763 7.179372 1.45783461 1.07670415 1.01334938 0.6951059

out <- recurrent_marginal(Event(start,stop,statusD)~cluster(id),data=rr,cause=1,death.code=3)
summary(out,times=times)
#> [[1]]
#>     new.time     mean         se   CI-2.5%  CI-97.5% strata
#> 181      500 1.050317 0.08835005 0.8906755  1.238572      0
#> 279     1000 1.843830 0.14730045 1.5765935  2.156363      0
#> 355     1500 2.714728 0.24272224 2.2783527  3.234684      0
#> 413     2000 3.546522 0.36401457 2.9002498  4.336805      0
#> 462     2500 4.492684 0.51416355 3.5899669  5.622395      0
#> 480     3000 5.080889 0.60551061 4.0225230  6.417723      0
#> 492     3500 5.638012 0.70065049 4.4192133  7.192950      0
#> 514     4000 6.895109 0.95745240 5.2522287  9.051878      0
#> 523     4500 7.723651 1.15096828 5.7673685 10.343501      0
#> 530     5000 8.923607 1.45783461 6.4785990 12.291356      0
```

Regression models for the marginal mean 
========================================

One can also do regression modelling , using the model
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
then Ghosh-Lin suggested IPCW score equations that are implemented in the recreg function of mets. 

First we generate data that from a Ghosh-Lin model with regression coefficients 
$\beta=(-0.3,0.3)$ and the baseline given by base1, this is done under the assumption that the death 
rate given covariates is on Cox form with baseline dr: 


``` r
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])

out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
outs <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0,
		cens.model=~strata(X1,X2))
summary(out)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.39075785 0.3162028 0.10070725 0.2165394
#> X2 -0.09651179 0.3581337 0.09682266 0.7875562
summary(outs)$coef
#>      Estimate      S.E.    dU^-1/2   P-value
#> X1  0.2122279 0.2946370 0.10162550 0.4713386
#> X2 -0.1576775 0.3523691 0.09699948 0.6545299

## checking baseline
par(mfrow=c(1,1))
plot(out)
plot(outs,add=TRUE,col=2)
lines(scalecumhaz(base1,1),col=3,lwd=2)
```

<img src="figure/recurrent-events-unnamed-chunk-12-1.png" alt="Ghosh-Lin regression baseline estimates."  />

We note that for the extended censoring model we gain a little efficiency and that the estimates are close to the true values. 

Also possible to do IPCW regression at fixed time-point


``` r
outipcw  <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		cens.code=0,times=2000)
outipcws <- recregIPCW(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
	    cens.code=0,times=2000,cens.model=~strata(X1,X2))
summary(outipcw)$coef
#>               Estimate   Std.Err       2.5%     97.5%      P-value
#> (Intercept)  1.1590100 0.2775235  0.6150739 1.7029462 2.963428e-05
#> X1           0.2337768 0.2783519 -0.3117830 0.7793365 4.009866e-01
#> X2          -0.2114258 0.2758703 -0.7521217 0.3292701 4.434409e-01
summary(outipcws)$coef
#>               Estimate   Std.Err       2.5%     97.5%      P-value
#> (Intercept)  1.1681460 0.2766323  0.6259567 1.7103353 2.413511e-05
#> X1           0.2312122 0.2785043 -0.3146462 0.7770705 4.064300e-01
#> X2          -0.2321638 0.2747766 -0.7707160 0.3063885 3.981565e-01
```

We can also do the Mao-Lin type composite outcome where we both count the recurrent events (cause 1) and deaths (cause 3)
for example 
\begin{align*}
E(N_1(t) + I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}






``` r
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		death.code=3,cens.code=0)
summary(out)$coef
#>      Estimate      S.E.    dU^-1/2   P-value
#> X1  0.3677911 0.2642911 0.09289221 0.1640393
#> X2 -0.1056515 0.2995438 0.08962337 0.7243074
```

This can also be done with competing risks death 
\begin{align*}
E(w_1 N_1(t) + w_2 I(D<t,\epsilon=3) | X) &  = \Lambda_0(t)  \exp(X^T \beta)
\end{align*}
and with weights  $w_1,w_2$ that follow the causes, here 1 and 3. 
We modify the data by changing some of the cause 3 deaths to cause 4


``` r
rr$binf <- rbinom(nrow(rr),1,0.5) 
rr$statusDC <- rr$statusD
rr <- dtransform(rr,statusDC=4, statusD==3 & binf==0)
rr$weight <- 1
rr <- dtransform(rr,weight=2,statusDC==3)

outC  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		 death.code=c(3,4),cens.code=0)
summary(outC)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.40121614 0.2943567 0.09755943 0.1728740
#> X2 -0.09714896 0.3323313 0.09368958 0.7700377

outCW  <- recreg(Event(start,stop,statusDC)~X1+X2+cluster(id),data=rr,cause=c(1,3),
		  death.code=c(3,4),cens.code=0,wcomp=c(1,2))
summary(outCW)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.41043629 0.2765368 0.09469049 0.1377555
#> X2 -0.09770279 0.3106702 0.09084228 0.7531486

plot(out,ylab="Mean composite")
plot(outC,col=2,add=TRUE)
plot(outCW,col=3,add=TRUE)
```

<img src="figure/recurrent-events-unnamed-chunk-15-1.png" alt="Composite outcome baseline estimates."  />

Predictions and standard errors can be computed via the iid decompositions of the baseline and the regression coefficients. We illustrate this
for the standard Ghosh-Lin model 


``` r
out  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,cens.code=0)
summary(out)
#> 
#>    n events
#>  541    441
#> 
#>  100 clusters
#> coefficients:
#>     Estimate      S.E.   dU^-1/2 P-value
#> X1  0.390758  0.316203  0.100707  0.2165
#> X2 -0.096512  0.358134  0.096823  0.7876
#> 
#> exp(coefficients):
#>    Estimate    2.5% 97.5%
#> X1  1.47810 0.79534 2.747
#> X2  0.90800 0.45003 1.832
baseiid <- iidBaseline(out,time=3000)
GLprediid(baseiid,rr[1:5,])
#>          pred    se-log    lower    upper
#> [1,] 3.303778 0.2373066 2.074985 5.260253
#> [2,] 3.303778 0.2373066 2.074985 5.260253
#> [3,] 3.303778 0.2373066 2.074985 5.260253
#> [4,] 4.883316 0.2753957 2.846410 8.377841
#> [5,] 4.883316 0.2753957 2.846410 8.377841
```

The Ghosh-Lin model can be made more efficient by the regression augmentation method.
First computing the augmentation and then in a second step the augmented estimator (Cortese and Scheike (2023)): 


``` r
outA  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,cause=1,death.code=3,
		 cens.code=0,augment.model=~Nt+X1+X2)
summary(outA)$coef
#>       Estimate      S.E.    dU^-1/2   P-value
#> X1  0.16625713 0.2854224 0.09827540 0.5602333
#> X2 -0.06801445 0.3288907 0.09702398 0.8361664
```

We note that the simple augmentation improves the standard errors as expected. The data was generated assuming independence with
previous number of events so it would suffice to augment only with the covariates. 



Administrative censoring for the Ghosh-Lin model
===============================================

In the case of administative censoring with possible additional random censorering we can fit the Ghosh-Lin model using 
risk-set adjustment for the administrative censorering and IPCW adjustment for the  random cenouring. We illustrate it using 
simulated data from the two-stage model described below. 
The advantage of this procedure is that we do not rely on any modelling assumptions for the administrative censoring that will
often be the important part of the total censoring. 



``` r
library(mets)
rho1 <- 1; rho2 <- 0.5
rate <- c(1,1)
tt <- seq(0, 6, by = 0.01)
base1 <- cbind(tt,rho1 * (1 - exp(-tt/rate[1])))
drcumhaz <- cbind(tt,rho2 * (1 - exp(-tt/rate[2])))
base13 <- cpred(base1,c(1,3))[,2]

dats <- mets:::sim_GLRA(100,base1,drcumhaz,varz=1)
datsA <- dats[[1]]
dtable(datsA,~statusD)
#> 
#> statusD
#>  1  3  7 
#> 88 35 65
datsRA <- dats[[2]]
dtable(datsRA,~statusD)
#> 
#> statusD
#>  0  1  3  7 
#> 23 73 32 45
```

We get two data-sets, one with only administrative censoring datsA 
and one with additional random censoring datsRA. 
 
  - datsA: statusD has events (1) death (3) and administrative censorings at the  time censorA
  - datsRA: statusD has events (1) death (3),  administrative censorings (7) and random censorings (0). The time 
     of the administriative censoring is given by censorA

First we deal only  with the administriative censoring. 


``` r
   ## handling admin censoring with IPCW 
    outRR  <- recreg(Event(start,stop,statusD)~Z1+Z2+cluster(id), datsA, cause = 1, 
		       cens.code = c(7),death.code=3)
    estimate(outRR)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1   0.2244  0.1918 -0.1515 0.6004  0.2420
#> Z2  -0.2565  0.3637 -0.9694 0.4565  0.4808

    ## Full Adm-censoring statusA, timeA
    out0  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsA, cause = 1, 
		       cens.code = 9, death.code=3, adm.cens.time=datsA$censorA)
    estimate(out0)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1   0.2501  0.1930 -0.1283 0.6284  0.1952
#> Z2  -0.1517  0.3636 -0.8642 0.5609  0.6765
```

The additional random censoring can be handled by combining the two censoring times, or by handling the 
administrative censoring (via risk-set adjustment) and random censoring (via IPCW) separately


``` r
    ## std Right censoring  on combined censoring time 
    outR  <- recreg(Event(start,stop,statusD)~Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = c(0,7),death.code=3)
    estimate(outR)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3036  0.1724 -0.03434 0.6415 0.07827
#> Z2  -0.2023  0.3457 -0.87997 0.4753 0.55839

    ## Combined R-IPCW + Adm-censoring status, time
    out1  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		       cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA)
    estimate(out1)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3284  0.1732 -0.01105 0.6679 0.05794
#> Z2  -0.1040  0.3418 -0.77397 0.5660 0.76105

    ## Combined R-IPCW + Adm-censoring status + censoring modelling time
    out1c  <- recreg(Event(start,stop,statusD) ~ Z1 + Z2+cluster(id) , datsRA, cause = 1, 
		     cens.code = 0, death.code=3, adm.cens.time=datsRA$censorA,cens.model=~strata(Z1,Z2))
     estimate(out1c)
#>    Estimate Std.Err     2.5%  97.5% P-value
#> Z1   0.3468  0.1741  0.00545 0.6881 0.04645
#> Z2  -0.1024  0.3417 -0.77220 0.5674 0.76440
```

We have influence functions of all parameters and can thus also make predictions with confindence intervals

 - this is based on going through a grid of time-points if se=1
   - when se=0 the predictions are done for all jump-times 


``` r
predR0 <- predict(outR,data.frame(Z1=0:1,Z2=0))
plot(predR0,ylim=c(0,1.5))

predR <- predict(outR,data.frame(Z1=0:1,Z2=0),se=1)
plot(predR,se=1,add=TRUE)

pred1c <- predict(out1c,data.frame(Z1=0:1,Z2=0),se=1)
plot(pred1c,se=1,add=TRUE)
```

<img src="figure/recurrent-events-unnamed-chunk-21-1.png" alt="Ghosh-Lin predictions with confidence intervals."  />

The censoring weights for the random censoring and the combined censoring


``` r
km07 <- km(Event(start,stop,statusD %in% c(0,7))~strata(Z1,Z2),datsRA)
km0 <- km(Event(start,stop,statusD==0)~strata(Z1,Z2),datsRA)
plot(km07,col=1)
plot(km0,add=TRUE,col=2,lwd=2)
```

<img src="figure/recurrent-events-unnamed-chunk-22-1.png" alt="Censoring weight estimates by strata."  />


Two-stage modelling 
====================

We simulate data with a terminal event on Cox form and recurrent events
satisfying the Ghosh-Lin model or having a rate on Cox form.

 - type=3 is Ghosh-Lin model for recurrent events and Cox for terminal event.
 - type=2 is Cox model for recurrent events among survivors and Cox for terminal event.
 - simulations based on time-grid to make linear approximations of cumulative hazards

Now we fit the two-stage model (the recreg must be called with twostage=TRUE)


``` r
set.seed(100)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
n <- 200
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
fz <- NULL
## type=3 is cox-cox and type=2 is Ghosh-Lin/Cox model 
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,cens=1/5000,type=3) 
dtable(rr,~statusD)
#> 
#> statusD
#>   0   1   3 
#>  55 375 145
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
## cox/cox
tsout <- twostageREC(outs,out,data=rr)
summary(tsout)
#> Cox(recurrent)-Cox(terminal) intensity model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.14651 0.14126 0.86964 1.42339       0
#> 
#> var,shared:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.14651 0.14126 0.86964 1.42339       0
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=3,share=0.5) 
rr <- cbind(rr,X[rr$id+1,])
###
out  <- phreg(Event(start,stop,statusD==1)~X1+X2+cluster(id),data=rr)
outs <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
#
tsout <- twostageREC(outs,out,data=rr,model="shared")
summary(tsout)
#> Cox(recurrent)-Cox(terminal) intensity model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.07333 0.14096 0.79707 1.34960       0
#> share1       0.67344 0.12832 0.42193 0.92495       0
#> 
#> var,shared:
#>             Estimate    2.5%  97.5%
#> dependence1  1.07333 0.79707 1.3496
#> share1       0.67344 0.42193 0.9250
###
rr <- mets:::sim_GLcox(n,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,fz,cens=1/5000,type=2) 
rr <- cbind(rr,X[rr$id+1,])
outs  <- phreg(Event(start,stop,statusD==3)~X1+X2+cluster(id),data=rr)
outgl  <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),data=rr,twostage=TRUE,death.code=3)
##
## ghosh-lin/cox
glout <- twostageREC(outs,outgl,data=rr,theta=1)
summary(glout)
#> Ghosh-Lin(recurrent)-Cox(terminal) mean model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.04401 0.08988 0.86785 1.22017       0
#> 
#> var,shared:
#>             Estimate Std.Err    2.5%   97.5% P-value
#> dependence1  1.04401 0.08988 0.86785 1.22017       0
###
glout <- twostageREC(outs,outgl,data=rr,model="shared",theta=1,nu=0.9)
summary(glout)
#> Ghosh-Lin(recurrent)-Cox(terminal) mean model
#> 
#>  200 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> dependence1 1.192629 0.094129 1.008140 1.377117       0
#> share1      1.643844 0.176676 1.297566 1.990121       0
#> 
#> var,shared:
#>             Estimate   2.5%  97.5%
#> dependence1   1.1926 1.0081 1.3771
#> share1        1.6438 1.2976 1.9901
glout$gradient
#> [1] -1.823718e-08  2.506173e-08
```

Standard errors are computed assuming that the parameters of `out` and `outs` are
both known, and are therefore probably a little too small. A bootstrap could be used to
obtain more reliable standard errors.


Simulations with specific structure
===================================

The function `simGLcox` can simulate data where the recurrent process has a mean on Ghosh-Lin form. The key identity is that
\begin{align*}
E(N_1(t) | X) &  = \Lambda_0(t)  \exp(X^T \beta) = \int_0^t S(t|X,Z) dR(t|X,Z)
\end{align*}
where $Z$ is a possible frailty, so that
\begin{align*}
 R(t|X,Z) & = \frac{Z \Lambda_0(t)  \exp(X^T \beta) }{S(t|X,Z)}
\end{align*}
leads to a Ghosh-Lin model. The survival model can be specified to have Cox form among survivors via
`model="twostage"`; otherwise `model="frailty"` uses a survival model with rate $Z \lambda_d(t) r_d$.
The frailty $Z$ is gamma distributed with a variance that can be specified. Simulations are based on
a piecewise linear approximation of the hazard functions for $S(t|X,Z)$ and $R(t|X,Z)$.



``` r
n <- 100
X <- matrix(rbinom(n*2,1,0.5),n,2)
colnames(X) <- paste("X",1:2,sep="")
###
r1 <- exp( X %*% c(0.3,-0.3))
rd <- exp( X %*% c(0.3,-0.3))
rc <- exp( X %*% c(0,0))
rr <- mets:::sim_GLcox(n,base1,dr,var.z=0,r1=r1,rd=rd,rc=rc,model="twostage",cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
```

We can also simulate from models where the terminal event is on Cox form and the rate among survivors is on Cox form. 

 * $E(dN_1 | D>t, X) = \lambda_1(t) r_1$
 * $E(dN_d | D>t, X) = \lambda_d(t) r_d$

underlying these models we have a shared frailty model


``` r
rr <- mets:::sim_GLcox(100,base1,dr,var.z=1,r1=r1,rd=rd,rc=rc,type=3,cens=3/5000) 
rr <- cbind(rr,X[rr$id+1,])
margsurv <- phreg(Surv(start,stop,statusD==3)~X1+X2+cluster(id),rr)
recurrent <- phreg(Surv(start,stop,statusD==1)~X1+X2+cluster(id),rr)
estimate(margsurv)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> X1   0.2535  0.2553 -0.2469 0.7538  0.3208
#> X2  -0.2595  0.2637 -0.7764 0.2574  0.3252
estimate(recurrent)
#>    Estimate Std.Err     2.5%   97.5% P-value
#> X1   0.6675  0.3019  0.07583 1.25921 0.02702
#> X2  -0.4356  0.2637 -0.95247 0.08135 0.09864
par(mfrow=c(1,2)); 
plot(margsurv); lines(dr,col=3); 
plot(recurrent); lines(base1,col=3)
```

<img src="figure/recurrent-events-unnamed-chunk-25-1.png" alt="Cox rates for survivors vs true baselines."  />

We can also simulate data with underlying dependence from the two-stage model (`simGLcox`) or using
`simRecurrent` random effects models, for Cox-Cox or Ghosh-Lin-Cox models.

Here with marginals on Cox-Cox and Ghosh-Lin-Cox form, drawing covariates from data:


``` r
simcoxcox <- sim_recurrent_ts(recurrent,margsurv,n=10,data=rr)

recurrentGL <- recreg(Event(start,stop,statusD)~X1+X2+cluster(id),rr,death.code=3)
simglcox <- sim_recurrent_ts(recurrentGL,margsurv,n=10,data=rr)
```


Other marginal properties
=========================

The mean is a useful summary measure, but it is also easy and informative to examine other
simple summary measures such as the probability of exceeding $k$ events,

 * $P(N_1^*(t) \ge k)$
   * cumulative incidence of $T_{k} = \inf \{ t: N_1^*(t)=k \}$ with competing event $D$.

This is equivalent to a cumulative incidence of $T_k$ occurring before $D$, denoted $\hat F_k(t)$.

We note also that $N_1^*(t)^2$ can be written as
\begin{align*}
   \sum_{k=0}^K  \int_0^t I(D > s) I(N_1^*(s-)=k) f(k) dN_1^*(s)
\end{align*}
with $f(k)=(k+1)^2 - k^2$, such that its mean can be written as 
\begin{align*}
	\sum_{k=0}^K \int_0^t S(s) f(k) P(N_1^*(s-)= k  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k, D> s) 
\end{align*}
and estimated by
\begin{align*}
\tilde \mu_{1,2}(t) & = 
	\sum_{k=0}^K \int_0^t \hat S(s) f(k) 
	\frac{Y_{1\bullet}^k(s)}{Y_\bullet (s)} \frac{1}{Y_{1\bullet}^k(s)} d N_{1\bullet}^k(s)= \sum_{i=1}^n \int_0^t \hat S(s) f(N_{i1}(s-)) \frac{1}{Y_\bullet (s)} d N_{i1}(s),
\end{align*}
That is very similar to the "product-limit" estimator for $E( (N_1^*(t))^2 )$ 
\begin{align}
  \hat \mu_{1,2}(t) & =    \sum_{k=0}^K k^2 ( \hat F_{k}(t) - \hat F_{k+1}(t) ).
\end{align}

We use the estimator of the probability of exceeding $k$ events based on the fact that
$I(N_1^*(t) \geq k)$ is  equivalent to 
\begin{align*}
	\int_0^t I(D > s) I(N_1^*(s-)=k-1) dN_1^*(s),
\end{align*}
suggesting that its mean can be computed as
\begin{align*}
\int_0^t S(s) P(N_1^*(s-)= k-1  | D  \geq s) E( dN_1^*(s)  | N_1^*(s-)=k-1, D> s) 
\end{align*}
and estimated by 
\begin{align*}
\tilde F_k(t) = \int_0^t \hat S(s)  \frac{Y_{1\bullet}^{k-1}(s)}{Y_\bullet (s)} 
          	\frac{1}{Y_{1\bullet}^{k-1}(s)} d N_{1\bullet}^{k-1}(s).
\end{align*}

To compute these estimators we use the prob.exceed.recurrent function


``` r
rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,cens=3/5000,dependence=4,var.z=1)
rr <- transform(rr,statusD=status)
rr <- dtransform(rr,statusD=3,death==1)
rr <-  count_history(rr)
dtable(rr,~statusD)
#> 
#> statusD
#>   0   1   2   3 
#>  93 287  32 107

oo <- prob_exceed_recurrent(Event(entry,time,statusD)~cluster(id),rr,cause=1,death.code=3)
plot(oo,types=1:5)
```

<img src="figure/recurrent-events-unnamed-chunk-27-1.png" alt="Probability of exceeding k events over time."  />

We can also look at the mean and variance based on the estimators just described 


``` r
par(mfrow=c(1,2))
with(oo,plot(time,meanN,col=2,type="l"))
with(oo,plot(time,varN,type="l"))
```

<img src="figure/recurrent-events-unnamed-chunk-28-1.png" alt="Mean and variance of recurrent events process."  />


Multiple events
================

We now generate recurrent events data with two event types, starting with
independent events.


``` r
rr <- sim_recurrentII(200,base1,cumhaz2=base4,death.cumhaz=dr)
rr <-  count_history(rr)
dtable(rr,~death+status)
#> 
#>       status   0   1   2
#> death                   
#> 0             32 651  84
#> 1            168   0   0
```

Based on this we can estimate also the joint distribution function, that is
the probability that $(N_1(t) \geq k_1, N_2(t) \geq k_2)$


``` r
# Bivariate probability of exceeding 
## oo <- prob.exceedBiRecurrent(rr,1,2,exceed1=c(1,5),exceed2=c(1,2))
## with(oo, matplot(time,pe1e2,type="s"))
## nc <- ncol(oo$pe1e2)
## legend("topleft",legend=colnames(oo$pe1e2),lty=1:nc,col=1:nc)
```

Looking at simulations with dependence
=============================================

Using normally distributed random effects we illustrate four settings. We set variance $0.5$ for all
random effects and vary the correlation. We denote the correlation between the random effect associated with
$N_1$ and $N_2$ as $\rho_{12}$, and the correlation between the random effects
associated with $N_j$ and $D$ (the terminal event) as $\rho_{j3}$, organised in the vector $\rho=(\rho_{12},\rho_{13},\rho_{23})$.

 * Scenario I: $\rho=(0,0.0,0.0)$ — independence among all effects.


``` r
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- CPH_HPN_CRBSI$mechanical

  par(mfrow=c(1,3))
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.0), c(0.0, 1.0, 0.0), c(0.0, 0.0, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  cor(attr(rr,"z"))
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  plot(coo,main ="Scenario I")
```
 * Scenario II: $\rho=(0,0.5,0.5)$ — independence among survivors but dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # death related to  both causes in same way 
  cor.mat <- corM <- rbind(c(1.0, 0.0, 0.5), c(0.0, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main ="Scenario II")
```

 * Scenario III: $\rho=(0.5,0.5,0.5)$ — positive dependence among survivors and dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # positive dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, 0.5, 0.5), c(0.5, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario III")
```

 * Scenario IV: $\rho=(-0.4,0.5,0.5)$ — negative dependence among survivors and positive dependence on terminal event.


``` r
  var.z <- c(0.5,0.5,0.5)
  # negative dependence for N1 and N2 all related in same way
  cor.mat <- corM <- rbind(c(1.0, -0.4, 0.5), c(-0.4, 1.0, 0.5), c(0.5, 0.5, 1.0))
  rr <- sim_recurrentII(200,base1,base4,death.cumhaz=dr,var.z=var.z,cor.mat=cor.mat,dependence=2)
  rr <- count_history(rr,types=1:2)
###  coo <- covarianceRecurrent(rr,1,2,status="status",start="entry",stop="time")
###  par(mfrow=c(1,3))
###  plot(coo,main="Scenario IV")
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] splines   stats     graphics  grDevices utils     datasets  methods  
#> [8] base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              KernSmooth_2.23-26    
#>  [7] otel_0.2.0             glue_1.8.1             future.apply_1.20.2   
#> [10] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [13] grid_4.6.0             evaluate_1.0.5         lifecycle_1.0.5       
#> [16] yaml_2.3.12            mvtnorm_1.3-7          numDeriv_2016.8-1.1   
#> [19] compiler_4.6.0         codetools_0.2-20       Rcpp_1.1.1-1.1        
#> [22] ucminf_1.2.3           future_1.70.0          lattice_0.22-9        
#> [25] digest_0.6.39          pillar_1.11.1          parallelly_1.47.0     
#> [28] parallel_4.6.0         Matrix_1.7-5           tools_4.6.0           
#> [31] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Randomization for Cox Type rate models"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{Randomization for Cox Type rate models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Two-Stage Randomization for counting process outcomes
========================================================

Rate models are specified for $N_1(t)$, covering:

   - survival data
   - competing risks, cause-specific hazards
   - recurrent events data


Under simple randomization we can estimate the rate Cox model

  - \(  \lambda_0(t) \exp(A_0 \beta_0) \)

Under two-stage randomization we can estimate the rate Cox model

  - \( \lambda_0(t) \exp(A_0 \beta_0 + A_1(t) \beta_1 ) \)


The starting point is that Cox's partial likelihood score can be used for estimating parameters:
 \begin{align*}
   U(\beta) & = \int (A(t) - e(t)) dN_1(t)
 \end{align*}
 where $A(t)$ is the combined treatment vector over time.

  - The solution will converge to $\beta^*$, the Struthers-Kalbfleisch solution of the score, with robust Lin-Wei standard errors.

 The estimator can be augmented in different ways using additional covariates at the time of randomization and a censoring augmentation.
 The solved estimating equation is
 \begin{align*}
 \sum_i   U_i - AUG_0 - AUG_1 + AUG_C   = 0
 \end{align*}
 using covariates from `augmentR0` to augment with
 \begin{align*}
 AUG_0 = ( A_0 - \pi_0(X_0) ) X_0 \gamma_0
 \end{align*}
 where $P(A_0=1|X_0)=\pi_0(X_0)$ (which does not depend on covariates under randomization),
 and using covariates from `augmentR1` to augment with $R$ indicating whether the
 second randomization takes place:
 \begin{align*}
 AUG_1 = R ( A_1 - \pi_1(X_1))  X_1 \gamma_1
 \end{align*}
 and the dynamic censoring augmentation
 \begin{align*}
  AUG_C =  \int_0^t \gamma_c(s)^T (e(s) - \bar e(s))  \frac{1}{G_c(s) } dM_c(s)
 \end{align*}
 where $\gamma_c(s)$ is chosen to minimise the variance given the dynamic covariates specified by `augmentC`.

Propensity score models are always estimated unless a fixed value such as $\pi_0=1/2$ is requested, though
it is generally better to estimate $\pi_0$ adaptively. Similarly, $\gamma_0$ and $\gamma_1$ are estimated to reduce the variance of $U_i$.


  - Treatments must be given as factors.
  - The treatment for the 2nd randomization may depend on the response.
      - Treatment probabilities are estimated by default and uncertainty from this is adjusted for.
      - `treat.model` must typically allow for interaction between treatment number and covariates.
  - Randomization augmentation for the 1st and 2nd randomization is possible.
       - `typesR=c("R0","R1","R01")`
  - The censoring model can be stratified on observed baseline covariates.
     - Default model stratifies on randomization `R0`.
     - `cens.model` can be specified.
  - Censoring augmentation is done dynamically over time with time-dependent covariates.
      - `typesC=c("C","dynC")`: `C` for fixed coefficients and `dynC` for dynamic.
      - Applied separately within each stratum of the censoring model.

 Standard errors are estimated using the influence functions of all estimators, so tests of differences
 can be computed subsequently.

  - Variance adjustment for censoring augmentation is computed by subtracting the variance gain.
  - Influence functions are given for the case with `R0` only.


The times of randomization are specified by:

   - `treat.var` is `"1"` when a randomization occurs.
     - Default assumes all time points correspond to a treatment (the survival case without time-dependent covariates).
     - For recurrent events, this must be specified as the first record for each subject; see example below.


Data must be provided in start-stop-status survival format with:

 - one status code indicating events of interest
 - one code for censorings


The phreg_rct can be used for counting process style data, and thus covers situations with 


  - recurrent events 
  - survival data
  - cause-specific hazards (competing risks)

and will in all cases compute augmentations  

  - dynamic censoring augmentation
     - dynC
  - RCT augmentation 
     - R0, R1 and R01



Simple Randomization: Lu-Tsiatis marginal Cox model
===================================================



``` r
library(mets) 
set.seed(100)

## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z
 
out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
#>                 Estimate   Std.Err       2.5%     97.5%   P-value
#> Marginal-Z.f1 0.29263400 0.2739159 -0.2442313 0.8294993 0.2853693
#> R0_C:Z.f1     0.07166242 0.2234066 -0.3662065 0.5095313 0.7483838
#> R0_dynC:Z.f1  0.08321604 0.2221710 -0.3522312 0.5186633 0.7079889
#> attr(,"class")
#> [1] "summary.phreg_rct"
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~X)
###out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X,cens.model=~+1)
```

Results consistent with speff of  library(speff2trial) 


``` r
###library(speff2trial) 
library(mets)
data(ACTG175)
###
data <- ACTG175[ACTG175$arms==0 | ACTG175$arms==1, ]
data <- na.omit(data[,c("days","cens","arms","strat","cd40","cd80","age")])
data$days <- data$days+runif(nrow(data))*0.01
dfactor(data) <- arms.f~arms
notrun <- 1

if (notrun==0) { 
fit1 <- speffSurv(Surv(days,cens)~cd40+cd80+age,data=data,trt.id="arms",fixed=TRUE)
summary(fit1)
}
# 
# Treatment effect
#             Log HR       SE   LowerCI   UpperCI           p
# Prop Haz  -0.70375  0.12352  -0.94584  -0.46165  1.2162e-08
# Speff     -0.72430  0.12051  -0.96050  -0.48810  1.8533e-09

out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~cd40+cd80+age,augmentC=~cd40+cd80+age)
summary(out)
#>                    Estimate   Std.Err       2.5%      97.5%      P-value
#> Marginal-arms.f1 -0.7036460 0.1224406 -0.9436251 -0.4636669 9.092786e-09
#> R0_C:arms.f1     -0.7265342 0.1197607 -0.9612610 -0.4918075 1.306891e-09
#> R0_dynC:arms.f1  -0.7204699 0.1196158 -0.9549125 -0.4860272 1.710025e-09
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

The study is actually block-randomized,
so the standard errors should be computed with an adjustment equivalent to augmenting with the block as a factor:


``` r
dtable(data,~strat+arms)
#> 
#>       arms   0   1
#> strat             
#> 1          223 213
#> 2           96 106
#> 3          213 203
dfactor(data) <- strat.f~strat
out <- phreg_rct(Surv(days,cens)~arms.f,data=data,augmentR0=~strat.f)
summary(out)
#>                    Estimate   Std.Err       2.5%      97.5%      P-value
#> Marginal-arms.f1 -0.7036460 0.1224406 -0.9436251 -0.4636669 9.092786e-09
#> R0_none:arms.f1  -0.7009844 0.1217138 -0.9395390 -0.4624298 8.447051e-09
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

Two-Stage Randomization CALGB-9823 for survival outcomes
=========================================================

We illustrate an analysis of one SMART (Sequential Multiple Assignment Randomised Trial) conducted by Cancer and
Leukemia Group B Protocol 8923 (CALGB 8923), Stone and others (2001).
388 patients were randomised to an initial treatment of GM-CSF ($A_1$) or standard chemotherapy
($A_2$). Patients with complete remission and informed consent were then re-randomised to
cytarabine only ($B_1$) or cytarabine plus mitoxantrone ($B_2$).

We first compute the weighted risk-set  estimator based on estimated weights
\begin{align*}
\Lambda_{A1,B1}(t) & = \sum_i \int_0^t \frac{w_i(s)}{Y^w(s)} dN_i(s)
\end{align*}
where $w_i(s) = I(A0_i=A1) + (t>T_R) I(A1_i=B1)/\pi_1(X_i)$, that is 
1 when you start on treatment $A1$ and
then for those that changes to $B1$ at time $T_R$ then is scaled up 
with the proportion doing this. This is equivalent to
the IPTW (inverse probability of treatment weighted estimator). We estimate 
the treatment regimes $A1, B1$ and $A2, B1$ by letting $A10$ indicate those that are consistent with ending on $B1$. 
$A10$ then starts being $1$ and becomes $0$ if the subject is treated with $B2$, but stays $1$ if the subject is treated with $B1$. 
We can then look at the two strata where $A0=0,A10=1$ and $A0=1,A10=1$. Similarly, for those that end being consistent with $B2$.
Thus defining $A11$ to start being $1$, then stays $1$ if $B2$ is taken, and becomes $0$ if the second randomization is $B1$.

  - the treatment models are for all time-points, unless the `weight.var` variable is given (1 for treatments, 0 otherwise) to accommodate a general start-stop format
  - the treatment model may also depend on a response value
  - standard errors are based on influence functions and is also computed for the baseline

We here use the propensity score model $P(A1=B1|A0)$ that uses the 
observed frequencies on arm $B1$ among those starting out on either $A1$ or $A2$.


``` r
data(calgb8923)
calgt <- calgb8923

## tm <- At.f~factor(Count2)+age+sex+wbc
## tm <- At.f~factor(Count2)
tm <-  At.f~factor(Count2)*A0.f

head(calgt)
#>   id V X Z   TR R     U delta  stop age   wbc sex race      time status start
#> 1  1 0 0 0 0.00 0 13.33     1 13.33  64 128.0   1    1 13.338219      1  0.00
#> 2  2 1 1 0 0.00 0 17.80     1 17.80  71   4.3   2    1 17.802995      1  0.00
#> 3  3 1 0 0 0.00 0  1.27     1  1.27  71  43.6   2    1  1.271527      1  0.00
#> 4  4 1 0 1 0.00 0 24.77     1 24.77  63  72.3   2    1  0.730000      2  0.00
#> 5  4 1 0 1 0.73 1 24.77     1 24.77  63  72.3   2    1 24.772515      1  0.73
#> 6  5 0 1 0 0.00 0 10.37     1 10.37  65   1.4   1    1 10.374479      1  0.00
#>   A0.f A0 A1 A11 A12 A1.f A10 At.f lbnr__id Count1 Count2 consent trt2 trt1
#> 1    0  0  0   1   0    0   0    0        1      0      0      -1   -1    1
#> 2    1  1  0   1   0    0   0    1        1      0      0      -1   -1    2
#> 3    0  0  0   1   0    0   0    0        1      0      0      -1   -1    1
#> 4    0  0  0   1   0    0   0    0        1      0      0      -1   -1    1
#> 5    0  0  1   1   1    1   1    1        2      0      1       1    1    1
#> 6    1  1  0   1   0    0   0    1        1      0      0      -1   -1    2
ll0 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A10)+cluster(id),calgt,treat.model=tm)
pll0 <- predict(ll0,expand.grid(A0=0:1,A10=0,id=1))
ll1 <- phreg_IPTW(Event(start,time,status==1)~strata(A0,A11)+cluster(id),calgt,treat.model=tm)
pll1 <- predict(ll1,expand.grid(A0=0:1,A11=1,id=1))
plot(pll0,se=1,lwd=2,col=1:2,lty=1,xlab="time (months)",xlim=c(0,30))
plot(pll1,add=TRUE,col=3:4,se=1,lwd=2,lty=1,xlim=c(0,30))
abline(h=0.25)
legend("topright",c("A1B1","A2B1","A1B2","A2B2"),col=c(1,2,3,4),lty=1)
```

<img src="figure/phreg_rct-unnamed-chunk-5-1.png" alt="IPTW survival estimates for four treatment regimes."  />

``` r

summary(pll1,times=1:10)
#> Predictions of type 'surv'
#>   Showing subjects: 1, 2
#>   Showing times:    1, 2, 3, 4, 5, 6, 7, 8, 9, 10
#> 
#> -- Subject 1 --
#>  time   surv     se  lower  upper
#>     1 0.8023 0.0286 0.7481 0.8603
#>     2 0.7120 0.0310 0.6537 0.7754
#>     3 0.6676 0.0328 0.6062 0.7352
#>     4 0.6472 0.0327 0.5863 0.7145
#>     5 0.6370 0.0326 0.5763 0.7041
#>     6 0.6164 0.0323 0.5562 0.6831
#>     7 0.5770 0.0339 0.5143 0.6474
#>     8 0.5428 0.0349 0.4786 0.6156
#>     9 0.5155 0.0379 0.4463 0.5953
#>    10 0.5103 0.0381 0.4409 0.5906
#> 
#> -- Subject 2 --
#>  time   surv     se  lower  upper
#>     1 0.8568 0.0249 0.8094 0.9071
#>     2 0.7871 0.0282 0.7338 0.8444
#>     3 0.7456 0.0292 0.6906 0.8051
#>     4 0.7134 0.0313 0.6545 0.7775
#>     5 0.6879 0.0318 0.6284 0.7530
#>     6 0.6624 0.0321 0.6024 0.7283
#>     7 0.6401 0.0335 0.5778 0.7091
#>     8 0.6109 0.0360 0.5443 0.6858
#>     9 0.5646 0.0395 0.4923 0.6475
#>    10 0.5544 0.0396 0.4819 0.6377
summary(pll0,times=1:10)
#> Predictions of type 'surv'
#>   Showing subjects: 1, 2
#>   Showing times:    1, 2, 3, 4, 5, 6, 7, 8, 9, 10
#> 
#> -- Subject 1 --
#>  time   surv     se  lower  upper
#>     1 0.8017 0.0287 0.7473 0.8601
#>     2 0.7008 0.0336 0.6379 0.7700
#>     3 0.6523 0.0359 0.5855 0.7267
#>     4 0.6158 0.0375 0.5466 0.6938
#>     5 0.5924 0.0385 0.5215 0.6728
#>     6 0.5660 0.0391 0.4944 0.6479
#>     7 0.5330 0.0395 0.4609 0.6164
#>     8 0.4856 0.0408 0.4119 0.5725
#>     9 0.4751 0.0411 0.4010 0.5629
#>    10 0.4580 0.0416 0.3834 0.5472
#> 
#> -- Subject 2 --
#>  time   surv     se  lower  upper
#>     1 0.8561 0.0251 0.8083 0.9067
#>     2 0.7741 0.0305 0.7165 0.8363
#>     3 0.7154 0.0338 0.6520 0.7848
#>     4 0.6690 0.0366 0.6009 0.7448
#>     5 0.6272 0.0383 0.5564 0.7070
#>     6 0.5642 0.0408 0.4896 0.6502
#>     7 0.5413 0.0415 0.4658 0.6289
#>     8 0.5244 0.0420 0.4482 0.6136
#>     9 0.5014 0.0424 0.4249 0.5918
#>    10 0.4784 0.0427 0.4017 0.5699
```

The propensity score model can be extended to use covariates to get increased efficiency.
Note also that the propensity scores for $A0$ will cancel out in the different strata. 

We now illustrate how to fit a  Cox model of the form
\begin{align*}
  & \lambda_{A0}(t) \exp( B1(t) \beta_1 + B2(t) \beta_2)
\end{align*}
where $\beta_0$ is the effect of treatment $A2$ and the effect of $B1$ 

Now  comparing only those starting on A1/A2 to compare the effect of B1 versus B2


``` r
library(mets)
data(calgb8923)
calgt <- calgb8923
calgt$treatvar <- 1

## making time-dependent indicators of going to B1/B2
calgt$A10t <- calgt$A11t <- 0
calgt <- dtransform(calgt,A10t=1,A1==0 & Count2==1)
calgt <- dtransform(calgt,A11t=1,A1==1 & Count2==1)
calgt0 <- subset(calgt,A0==0)

ss0 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==0),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss0)
#>                Estimate   Std.Err      2.5%      97.5%      P-value
#> Marginal-A10t -1.570250 0.2433389 -2.047185 -1.0933143 1.097054e-10
#> Marginal-A11t -1.407287 0.2193924 -1.837289 -0.9772861 1.413090e-10
#> non_dynC:A10t -1.583146 0.2418997 -2.057260 -1.1090311 5.963963e-11
#> non_dynC:A11t -1.406682 0.2190539 -1.836020 -0.9773442 1.348264e-10
#> R1_non:A10t   -1.544312 0.2396152 -2.013949 -1.0746751 1.156250e-10
#> R1_non:A11t   -1.423064 0.2087465 -1.832199 -1.0139282 9.284039e-12
#> R1_dynC:A10t  -1.557021 0.2381534 -2.023793 -1.0902486 6.239330e-11
#> R1_dynC:A11t  -1.422360 0.2083906 -1.830798 -1.0139218 8.764919e-12
#> attr(,"class")
#> [1] "summary.phreg_rct"

ss1 <- phreg_rct(Event(start,time,status)~A10t+A11t+cluster(id),data=subset(calgt,A0==1),
	 typesR=c("non","R1"),typesC=c("non","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
summary(ss1)
#>                 Estimate   Std.Err      2.5%      97.5%      P-value
#> Marginal-A10t -0.8968608 0.2312067 -1.350018 -0.4437039 1.048683e-04
#> Marginal-A11t -0.9754528 0.2215523 -1.409687 -0.5412181 1.068580e-05
#> non_dynC:A10t -0.8312901 0.2263294 -1.274888 -0.3876925 2.397942e-04
#> non_dynC:A11t -1.0165973 0.2211108 -1.449967 -0.5832280 4.272177e-06
#> R1_non:A10t   -0.9310307 0.2299136 -1.381653 -0.4804083 5.133147e-05
#> R1_non:A11t   -0.9361199 0.2204289 -1.368153 -0.5040872 2.168342e-05
#> R1_dynC:A10t  -0.8634407 0.2250083 -1.304449 -0.4224326 1.243576e-04
#> R1_dynC:A11t  -0.9753885 0.2199851 -1.406551 -0.5442256 9.255029e-06
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

and a more structured model with both A0 and A1, that does not seem very reasonable based on 
the above,


``` r
ssf <- phreg_rct(Event(start,time,status)~A0.f+A10t+A11t+cluster(id),
	 data=calgt,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
	 treat.var="treatvar",treat.model=At.f~factor(Count2),
	 augmentR0=~age+wbc+sex,augmentR1=~age+wbc+sex+TR,augmentC=~age+wbc+sex+TR+Count2)
```



Recurrent events: Simple Randomization
======================================

Recurrent events simulation with death and censoring.



``` r
n <- 1000
beta <- 0.15; 
 data(CPH_HPN_CRBSI)
 dr <- CPH_HPN_CRBSI$terminal
 base1 <- CPH_HPN_CRBSI$crbsi 
 base4 <- scalecumhaz(CPH_HPN_CRBSI$mechanical,0.5)

cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))
ce <- 3; betao1 <- 0

varz <- 1; dep=4; X <- z <- rgamma(n,1/varz)*varz
Z0 <- NULL
px <- 0.5
if (betao1!=0) px <- lava::expit(betao1*X)      
A0 <- rbinom(n,1,px)
r1 <- exp(A0*beta[1])
rd <- exp( A0 * 0.15)
rc <- exp( A0 * 0 )
###
rr <-    mets:::simLUCox(n,base1,death.cumhaz=dr,r1=r1,Z0=X,dependence=dep,var.z=varz,cens=ce/5000)
rr$A0 <- A0[rr$id]
rr$z1 <- attr(rr,"z")[rr$id]
rr$lz1 <- log(rr$z1)
rr$X <- rr$lz1 
rr$lX <- rr$z1
rr$statusD <- rr$status
rr <- dtransform(rr,statusD=2,death==1)
rr <- count_history(rr)
rr$Z <- rr$A0
data <- rr
data$Z.f <- as.factor(data$Z)
data$treattime <- 0
data <- dtransform(data,treattime=1,lbnr__id==1)
dlist(data,start+stop+statusD+A0+z1+treattime+Count1~id|id %in% c(4,5))
#> id: 4
#>      start   stop    statusD A0 z1    treattime Count1
#> 4      0.000   9.565 1       0  0.471 1         0     
#> 1003   9.565 372.057 1       0  0.471 0         1     
#> 1468 372.057 389.831 0       0  0.471 0         2     
#> ------------------------------------------------------------ 
#> id: 5
#>   start stop  statusD A0 z1    treattime Count1
#> 5 0     213.9 2       1  2.338 1         0
```

Now we fit the model 


``` r
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 treat.var="treattime",typesR=c("non","R0"),typesC=c("non","C","dynC"),
	 augmentR0=~z1,augmentC=~z1+Count1)
summary(fit2)
#>                Estimate    Std.Err       2.5%     97.5%      P-value
#> Marginal-Z.f1 0.2870649 0.09632565 0.09827011 0.4758597 0.0028810700
#> non_C:Z.f1    0.2826049 0.09631924 0.09382262 0.4713871 0.0033457707
#> non_dynC:Z.f1 0.1926888 0.08864883 0.01894025 0.3664373 0.0297337758
#> R0_non:Z.f1   0.3110880 0.08049844 0.15331399 0.4688621 0.0001113067
#> R0_C:Z.f1     0.3066141 0.08049078 0.14885504 0.4643731 0.0001393570
#> R0_dynC:Z.f1  0.2164684 0.07113356 0.07704922 0.3558877 0.0023413376
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

 - Censoring model was stratified on Z.f
 - treatment probabilities were estimated using the data


Twostage Randomization: Recurrent events 
=========================================




``` r
n <- 500
beta=c(0.3,0.3);betatr=0.3;betac=0;betao=0;betao1=0;ce=3;fixed=1;sim=1;dep=4;varz=1;ztr=0; ce <- 3
## take possible frailty 
Z0 <- rgamma(n,1/varz)*varz
px0 <- 0.5; if (betao!=0) px0 <- expit(betao*Z0)
A0 <- rbinom(n,1,px0)
r1 <- exp(A0*beta[1])
#
px1 <- 0.5; if (betao1!=0) px1 <- expit(betao1*Z0)
A1 <- rbinom(n,1,px1)
r2 <- exp(A1*beta[2])
rtr <- exp(A0*betatr[1])
rr <-  mets:::simLUCox(n,base1,death.cumhaz=dr,cumhaz2=base1,rtr=rtr,betatr=0.3,A0=A0,Z0=Z0,
		r1=r1,r2=r2,dependence=dep,var.z=varz,cens=ce/5000,ztr=ztr)
rr$z1 <- attr(rr,"z")[rr$id]
rr$A1 <- A1[rr$id]
rr$A0 <- A0[rr$id]
rr$lz1 <- log(rr$z1)
rr <- count_history(rr,types=1:2)
rr$A1t <- 0
rr <- dtransform(rr,A1t=A1,Count2==1) 
rr$At.f <- rr$A0
rr$A0.f <- factor(rr$A0)
rr$A1.f <- factor(rr$A1)
rr <- dtransform(rr, At.f = A1, Count2 == 1)
rr$At.f <- factor(rr$At.f)
dfactor(rr)  <-  A0.f~A0
rr$treattime <- 0
rr <- dtransform(rr,treattime=1,lbnr__id==1)
rr$lagCount2 <- dlag(rr$Count2)
rr <- dtransform(rr,treattime=1,Count2==1 & (Count2!=lagCount2))
dlist(rr,start+stop+statusD+A0+A1+A1t+At.f+Count2+z1+treattime+Count1~id|id %in% c(5,10))
#> id: 5
#>   start stop  statusD A0 A1 A1t At.f Count2 z1     treattime Count1
#> 5 0     132.3 3       1  1  0   1    0      0.2316 1         0     
#> ------------------------------------------------------------ 
#> id: 10
#>     start stop    statusD A0 A1 A1t At.f Count2 z1      treattime Count1
#> 10   0.00   33.12 2       1  0  0   1    0      0.06891 1         0     
#> 509 33.12 1363.53 0       1  0  0   0    1      0.06891 1         0
```

Now fitting the model and computing different augmentations (true values 0.3 and 0.3)


``` r
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),treat.var="treattime",
	 treat.model=At.f~factor(Count2),
	 augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
#>                 Estimate   Std.Err        2.5%     97.5%      P-value
#> Marginal-A0.f1 0.3179631 0.1418023  0.04003574 0.5958904 0.0249420566
#> Marginal-A1t   0.3290147 0.1472363  0.04043683 0.6175925 0.0254434247
#> non_C:A0.f1    0.3002782 0.1391490  0.02755115 0.5730053 0.0309308283
#> non_C:A1t      0.4151190 0.1405664  0.13961382 0.6906241 0.0031451130
#> non_dynC:A0.f1 0.3104992 0.1314476  0.05286660 0.5681318 0.0181692114
#> non_dynC:A1t   0.4374223 0.1300991  0.18243265 0.6924119 0.0007731772
#> R0_non:A0.f1   0.4142867 0.1176773  0.18364338 0.6449300 0.0004306830
#> R0_non:A1t     0.3382185 0.1470378  0.05002979 0.6264072 0.0214360247
#> R0_C:A0.f1     0.3962505 0.1144662  0.17190089 0.6206002 0.0005367253
#> R0_C:A1t       0.4242165 0.1403584  0.14911904 0.6993140 0.0025079567
#> R0_dynC:A0.f1  0.4066624 0.1049692  0.20092652 0.6123983 0.0001070147
#> R0_dynC:A1t    0.4464941 0.1298744  0.19194497 0.7010432 0.0005862621
#> R1_non:A0.f1   0.3269008 0.1416813  0.04921043 0.6045911 0.0210383383
#> R1_non:A1t     0.2104421 0.1254571 -0.03544923 0.4563335 0.0934636201
#> R1_C:A0.f1     0.3092275 0.1390258  0.03674199 0.5817130 0.0261319083
#> R1_C:A1t       0.2976811 0.1175580  0.06727171 0.5280905 0.0113347106
#> R1_dynC:A0.f1  0.3194771 0.1313171  0.06210024 0.5768540 0.0149798139
#> R1_dynC:A1t    0.3202318 0.1048176  0.11479297 0.5256706 0.0022496121
#> R01_non:A0.f1  0.4092668 0.1176428  0.17869115 0.6398424 0.0005034879
#> R01_non:A1t    0.2280764 0.1243165 -0.01557936 0.4717322 0.0665584775
#> R01_C:A0.f1    0.3912859 0.1144307  0.16700576 0.6155659 0.0006275647
#> R01_C:A1t      0.3150865 0.1163399  0.08706445 0.5431086 0.0067623432
#> R01_dynC:A0.f1 0.4016977 0.1049305  0.19603760 0.6073577 0.0001290708
#> R01_dynC:A1t   0.3375831 0.1034497  0.13482542 0.5403408 0.0011013908
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

  - `treat.model` codes $A_0$ and $A_1$ as `At.f`; here we allow a model that depends on the randomization to
    be as adaptive as possible.
  - For the observational case one can also adjust for covariates here.
  - The censoring model was stratified on `A0.f`.


Causal assumptions for Two-Stage Randomization: Recurrent events
===============================================================

We are interested in $N_1$ and also have death $N_d$.

Given $X_0$, we require:

  - \( A_0 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | X_0 \)
  - positivity: \(  1 > \pi_0(X_0) > 0 \)

Given $\bar X_1$, the history accumulated up to the time $T_R$ of the second randomization:

  - \( A_1 \perp  N_1^0(), N_1^1(), N_d^0(), N_d^1() | \bar X_1 \)
  - positivity: \(  1 > \pi_1(X_1) > 0 \)

And consistency, to link counterfactual quantities to observed data.

We must use an IPTW-weighted Cox score and augment as before.

In addition we require that censoring is independent given, for example, $A_0$:

  - Independent censoring.

To use `phreg_rct` in this setting:

 - Set `RCT=FALSE`.
 - Propensity score models must be specified.
 - The marginal estimate is based on the IPTW Cox model `phreg_IPTW`.
    - When the same model is used for both the propensity scores and the augmentation models,
      the augmentation models are not needed due to the adaptive nature of fitting the
      propensity score models.



``` r
fit2 <- phreg_rct(Event(start,stop,statusD)~Z.f+cluster(id),data=data,
	 typesR=c("non","R0"),typesC=c("non","C","dynC"),
         RCT=FALSE,treat.model=Z.f~z1,augmentR0=~z1,augmentC=~z1+Count1,
	 treat.var="treattime")
summary(fit2)
#>                Estimate    Std.Err       2.5%     97.5%      P-value
#> Marginal-Z.f1 0.3111195 0.08058494 0.15317593 0.4690631 0.0001130326
#> non_C:Z.f1    0.3067348 0.08057748 0.14880581 0.4646637 0.0001408301
#> non_dynC:Z.f1 0.2169383 0.07119837 0.07739205 0.3564845 0.0023117164
#> R0_non:Z.f1   0.3111195 0.08058494 0.15317593 0.4690631 0.0001130326
#> R0_C:Z.f1     0.3067348 0.08057748 0.14880581 0.4646637 0.0001408301
#> R0_dynC:Z.f1  0.2169383 0.07119837 0.07739205 0.3564845 0.0023117164
#> attr(,"class")
#> [1] "summary.phreg_rct"
```

and for twostage randomization


``` r
sse <- phreg_rct(Event(start,time,statusD)~A0.f+A1t+cluster(id),data=rr,
	 typesR=c("non","R0","R1","R01"),typesC=c("non","C","dynC"),
         treat.var="treattime",
	 RCT=FALSE,treat.model=At.f~z1*factor(Count2),
         augmentR0=~z1,augmentR1=~z1,augmentC=~z1+Count1+A1t)
summary(sse)
#>                 Estimate    Std.Err        2.5%     97.5%      P-value
#> Marginal-A0.f1 0.3817476 0.13188068  0.12326622 0.6402290 0.0037958891
#> Marginal-A1t   0.2259319 0.12344157 -0.01600910 0.4678730 0.0672089296
#> non_C:A0.f1    0.3765255 0.12594711  0.12967369 0.6233773 0.0027938653
#> non_C:A1t      0.3187675 0.11256529  0.09814361 0.5393914 0.0046280182
#> non_dynC:A0.f1 0.3797091 0.11214290  0.15991306 0.5995051 0.0007093493
#> non_dynC:A1t   0.3514300 0.09297578  0.16920079 0.5336591 0.0001569535
#> R0_non:A0.f1   0.3817476 0.13188068  0.12326622 0.6402290 0.0037958891
#> R0_non:A1t     0.2259319 0.12344157 -0.01600910 0.4678730 0.0672089296
#> R0_C:A0.f1     0.3765255 0.12594711  0.12967369 0.6233773 0.0027938653
#> R0_C:A1t       0.3187675 0.11256529  0.09814361 0.5393914 0.0046280182
#> R0_dynC:A0.f1  0.3797091 0.11214290  0.15991306 0.5995051 0.0007093493
#> R0_dynC:A1t    0.3514300 0.09297578  0.16920079 0.5336591 0.0001569535
#> R1_non:A0.f1   0.3817476 0.13188068  0.12326622 0.6402290 0.0037958891
#> R1_non:A1t     0.2259319 0.12344157 -0.01600910 0.4678730 0.0672089296
#> R1_C:A0.f1     0.3765255 0.12594711  0.12967369 0.6233773 0.0027938653
#> R1_C:A1t       0.3187675 0.11256529  0.09814361 0.5393914 0.0046280182
#> R1_dynC:A0.f1  0.3797091 0.11214290  0.15991306 0.5995051 0.0007093493
#> R1_dynC:A1t    0.3514300 0.09297578  0.16920080 0.5336591 0.0001569535
#> R01_non:A0.f1  0.3817476 0.13185641  0.12331378 0.6401814 0.0037894525
#> R01_non:A1t    0.2259319 0.12307282 -0.01528637 0.4671502 0.0663934395
#> R01_C:A0.f1    0.3765255 0.12592170  0.12972349 0.6233275 0.0027883528
#> R01_C:A1t      0.3187675 0.11216079  0.09893641 0.5385986 0.0044823275
#> R01_dynC:A0.f1 0.3797091 0.11211436  0.15996900 0.5994492 0.0007071245
#> R01_dynC:A1t   0.3514300 0.09248564  0.17016144 0.5326985 0.0001447938
#> attr(,"class")
#> [1] "summary.phreg_rct"
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              otel_0.2.0            
#>  [7] glue_1.8.1             future.apply_1.20.2    listenv_0.10.1        
#> [10] lava_1.9.1             stats4_4.6.0           grid_4.6.0            
#> [13] evaluate_1.0.5         lifecycle_1.0.5        yaml_2.3.12           
#> [16] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [19] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [22] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [25] pillar_1.11.1          parallelly_1.47.0      parallel_4.6.0        
#> [28] splines_4.6.0          Matrix_1.7-5           tools_4.6.0           
#> [31] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Average treatment effect (ATE) for Competing risks and binary outcomes"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Competing risks and binary outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



The `binregATE` function estimates the average treatment effect for binary outcomes with
IPCW adjustment at a specific time-point, and can thus be used for survival or competing risks data.
Computation is linear in data size and influence functions are computed and available.

Key features:

  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors
  - computation time is linear in data size, including standard errors
  - cluster-corrected standard errors are available via the `clusters` argument


Average treatment effect 
=========================

First we simulate some data that mimics that of Kumar et al. (2012).
This is data from multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR). The data consisted of patients transplanted from 1995 to
2005, comparing outcomes between transplant periods: 2001--2005 ($N=488$)
versus 1995--2000 ($N=375$). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1) defined as death without relapse.
Kumar et al. (2012) considered the following risk covariates:
transplant time period (`gp`, main interest of the study: 1 for transplanted in
2001--2005 versus 0 for transplanted in 1995--2000), donor type (`dnr`: 1 for unrelated or
other related donor ($N=280$) versus 0 for HLA-identical sibling ($N=584$)), prior
autologous transplant (`preauto`: 1 for Auto+Allo transplant ($N=399$) versus 0 for
allogeneic transplant alone ($N=465$)) and time to transplant (`ttt24`: 1 for more than 24 months ($N=289$) versus 0 for less than or
equal to 24 months ($N=575$)).


We generate similar data by assuming that the two cumulative incidence curves are logistic and that
censoring depends on covariates via a Cox model, all wrapped in the `kumarsim` function.
The simulation does not enforce the constraint $F_1+F_2 < 1$, but as we increase the
sample size we still recover the parameters of cause 2.

We start by computing the average treatment effects for the transplant periods (`gp`), then consider
the possible interaction between `gp` and donor type (`dnr`) and estimate all contrasts between the risk estimates for all combinations
of `gp` and `dnr`. The key components are:

  - an outcome model (fitted by IPCW): `Event(...) ~ .....`
  - a propensity score model (using logistic regression): `treat.model=...`
  - an IPCW weight estimated using a stratified Kaplan-Meier model: `cens.model=~strata(..)`



``` r
library(mets) 
set.seed(100)
###
n <- 400
kumar <- kumarsim(n,depcens=1)
kumar$cause <- kumar$status
kumar$ttt24 <- kumar[,6]
dtable(kumar,~cause)
#> 
#> cause
#>   0   1   2 
#> 182  72 146
dfactor(kumar) <- gp.f~gp
kumar$id <- 1:n
kumar$idc <- sample(100,n,TRUE)
kumar$ids <- sample(n,n)
kumar$id2 <- sample(n,n)
kumar2 <- kumar[order(kumar$id2),]
kumar$int <- interaction(kumar$gp,kumar$dnr)
kumar2$int <- interaction(kumar2$gp,kumar2$dnr)
clust <- 0

b2 <- binregATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(b2)
#>    n events
#>  400    137
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.310227  0.221121 -1.743616 -0.876838  0.0000
#> gp.f1        0.898030  0.271691  0.365526  1.430535  0.0009
#> dnr          0.323059  0.271545 -0.209160  0.855278  0.2342
#> preauto      0.269177  0.278181 -0.276049  0.814402  0.3332
#> ttt24        0.496202  0.276045 -0.044836  1.037240  0.0722
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.26976 0.17489 0.4161
#> gp.f1        2.45476 1.44127 4.1809
#> dnr          1.38135 0.81127 2.3520
#> preauto      1.30889 0.75878 2.2578
#> ttt24        1.64247 0.95615 2.8214
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    0.293021 0.039148 0.216292 0.369750   0e+00
#> treat1    0.496481 0.041744 0.414665 0.578297   0e+00
#> treat:1-0 0.203460 0.060294 0.085285 0.321635   7e-04
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0    0.311984 0.042914 0.227875 0.396094   0e+00
#> treat1    0.512310 0.042454 0.429103 0.595518   0e+00
#> treat:1-0 0.200326 0.060585 0.081582 0.319070   9e-04

b5 <- binregATE(Event(time,cause)~int+preauto+ttt24,kumar,cause=2,
		treat.model=int~preauto+ttt24,cens.code=0,time=60)
summary(b5)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.115768  0.285147 -1.674646 -0.556890  0.0001
#> int1.0       0.674582  0.333967  0.020018  1.329146  0.0434
#> int0.1       0.034751  0.494229 -0.933920  1.003421  0.9439
#> int1.1       1.340278  0.431989  0.493594  2.186961  0.0019
#> preauto      0.298967  0.272324 -0.234779  0.832713  0.2723
#> ttt24        0.476755  0.286955 -0.085667  1.039177  0.0966
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.32766 0.18737 0.5730
#> int1.0       1.96321 1.02022 3.7778
#> int0.1       1.03536 0.39301 2.7276
#> int1.1       3.82010 1.63819 8.9081
#> preauto      1.34846 0.79075 2.2995
#> ttt24        1.61084 0.91790 2.8269
#> 
#> Average Treatment effects (G-formula) :
#>                 Estimate    Std.Err       2.5%      97.5% P-value
#> treat0.0       0.3124362  0.0586931  0.1973999  0.4274724  0.0000
#> treat1.0       0.4676985  0.0422018  0.3849846  0.5504125  0.0000
#> treat0.1       0.3197899  0.0896402  0.1440983  0.4954815  0.0004
#> treat1.1       0.6274521  0.0726051  0.4851488  0.7697554  0.0000
#> treat:1.0-0.0  0.1552624  0.0738341  0.0105502  0.2999745  0.0355
#> treat:0.1-0.0  0.0073537  0.1048541 -0.1981565  0.2128640  0.9441
#> treat:1.1-0.0  0.3150160  0.0978046  0.1233225  0.5067094  0.0013
#> treat:0.1-1.0 -0.1479086  0.1003040 -0.3445009  0.0486837  0.1403
#> treat:1.1-1.0  0.1597536  0.0845518 -0.0059649  0.3254721  0.0588
#> treat:1.1-0.1  0.3076622  0.1178093  0.0767602  0.5385643  0.0090
#> 
#> Average Treatment effects (double robust) :
#>                 Estimate    Std.Err       2.5%      97.5% P-value
#> treat0.0       0.3580588  0.0674640  0.2258318  0.4902858  0.0000
#> treat1.0       0.4871368  0.0501492  0.3888461  0.5854274  0.0000
#> treat0.1       0.3102783  0.1210473  0.0730299  0.5475266  0.0104
#> treat1.1       0.7645588  0.1887893  0.3945387  1.1345790  0.0001
#> treat:1.0-0.0  0.1290779  0.0831680 -0.0339284  0.2920843  0.1207
#> treat:0.1-0.0 -0.0477806  0.1400295 -0.3222333  0.2266722  0.7329
#> treat:1.1-0.0  0.4065000  0.2040550  0.0065596  0.8064404  0.0464
#> treat:0.1-1.0 -0.1768585  0.1315359 -0.4346641  0.0809471  0.1788
#> treat:1.1-1.0  0.2774221  0.1937486 -0.1023182  0.6571624  0.1522
#> treat:1.1-0.1  0.4542806  0.2102654  0.0421680  0.8663931  0.0307
```

We note that the estimates found using the large censoring model are very different from those
using the simple Kaplan-Meier weights that are severely biased for these data. This is due to a strong 
censoring dependence. 

The average treatment is around $0.17 = E(Y(1) - Y(0))$ at time 60 for the 
transplant period, under the standard causal assumptions. 
The 1/0 treatment variable used for the causal computation is found as the right hand side (rhs) of the treat.model 
or as the first argument on the rhs of the response model. 

The binregATE default uses binreg with its default to fit the working model and is 
recommended, but the logitIPCW and logitIPCWATE can also be used and are GLM-type 
IPCW weighted models (see binreg help page/vignette).  We can compare the logitIPCWATE/logitIPCW
with that of wglm and its ate function in the riskRegression package, and they agree. 


``` r
ib2 <- logitIPCWATE(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,
	treat.model=gp.f~dnr+preauto+ttt24,time=40,cens.model=~strata(gp,dnr))
summary(ib2)
#>    n events
#>  400    137
#> 
#>  400 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -1.27557  0.21917 -1.70514 -0.84601  0.0000
#> gp.f1        0.83337  0.25904  0.32566  1.34108  0.0013
#> dnr          0.36815  0.27656 -0.17390  0.91020  0.1831
#> preauto      0.48521  0.30772 -0.11791  1.08832  0.1148
#> ttt24        0.18958  0.30947 -0.41698  0.79614  0.5401
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.27927 0.18175 0.4291
#> gp.f1        2.30105 1.38494 3.8232
#> dnr          1.44506 0.84038 2.4848
#> preauto      1.62452 0.88878 2.9693
#> ttt24        1.20874 0.65904 2.2170
#> 
#> Average Treatment effects (G-formula) :
#>         Estimate  Std.Err     2.5%    97.5% P-value
#> treat-1 0.493059 0.040018 0.414625 0.571494  0.0000
#> treat-0 0.302695 0.040274 0.223759 0.381632  0.0000
#> p1      0.190364 0.058248 0.076200 0.304527  0.0011
#> 
#> Average Treatment effects (double robust) :
#>         Estimate  Std.Err     2.5%    97.5% P-value
#> treat-1 0.524459 0.043164 0.439860 0.609058   0e+00
#> treat-0 0.309211 0.043507 0.223938 0.394483   0e+00
#> p1      0.215248 0.061268 0.095165 0.335331   4e-04

ib5 <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,
	 time=60,cens.model=~strata(gp,dnr))
summary(ib5)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.142446  0.225267 -1.583960 -0.700931  0.0000
#> gp.f1        0.716372  0.267369  0.192339  1.240405  0.0074
#> dnr          0.551548  0.325499 -0.086418  1.189515  0.0902
#> preauto      0.748078  0.343802  0.074239  1.421917  0.0296
#> ttt24       -0.184844  0.373708 -0.917298  0.547610  0.6209
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.31904 0.20516 0.4961
#> gp.f1        2.04699 1.21208 3.4570
#> dnr          1.73594 0.91721 3.2855
#> preauto      2.11294 1.07706 4.1451
#> ttt24        0.83123 0.39960 1.7291

ibs <- logitIPCW(Event(time,cause)~gp.f+dnr+preauto+ttt24,kumar,cause=2,cens.code=0,time=60)
summary(ibs)
#>    n events
#>  400    142
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.294250  0.220215 -1.725864 -0.862636  0.0000
#> gp.f1        1.633772  0.294651  1.056267  2.211277  0.0000
#> dnr          0.020551  0.309235 -0.585538  0.626639  0.9470
#> preauto      0.547275  0.305442 -0.051381  1.145931  0.0732
#> ttt24        0.288357  0.309124 -0.317515  0.894228  0.3509
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.27410 0.17802 0.4220
#> gp.f1        5.12316 2.87562 9.1274
#> dnr          1.02076 0.55681 1.8713
#> preauto      1.72854 0.94992 3.1454
#> ttt24        1.33423 0.72796 2.4454

check <- 0
if (check==1) {
require(riskRegression)
e.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, formula.censor = Surv(time,cause==0)~+1, times = 60, data = kumar, product.limit=TRUE,cause=2)
summary(e.wglm)$coef
estimate(ibs)

es.wglm <- wglm( regressor.event=~gp.f+dnr+preauto+ttt24, 
formula.censor = Surv(time,cause==0)~strata(gp,dnr), times = 60, 
data = kumar, product.limit=TRUE,cause=2)
summary(es.wglm)$coef
estimate(ib5)
}
```

The cluster argument should not be used for the logitIPCWATE, but works for binregATE.


Average treatment for Competing risks data 
==========================================

The binreg function does direct binomial regression for one time-point, $t$, 
fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta),
\end{align*}
for possible right censored data. The estimation procedure 
is based on IPCW adjusted estimating equation (EE)
\begin{align*}
 U(\beta) = &  X \left( \Delta(t) I(T \leq t, \epsilon=1 )/G_c(T \wedge t) - \mbox{expit}( X^T beta) \right) = 0 
\end{align*}
where $G_c(t)=P(C > t)$, the censoring survival distribution, and with $\Delta(t) = I( C > T \wedge t)$ the indicator
of being uncensored at time $t$. 

The function logitIPCW instead considers the EE
\begin{align*}
 U(\beta) = &  X  \frac{\Delta(t)}{G_c(T \wedge t)} \left( I(T \leq t, \epsilon=1 ) - \mbox{expit}( X^T beta) \right) = 0.
\end{align*}
The two score equations are quite similar, and exactly the same when the censoring model is fully-nonparametric given $X$.

Additional functions logitATE, and binregATE computes the average treatment effect. We demonstrate their use below. 

The functions binregATE (recommended) and logitATE also works when there is no censoring and we thus have simple binary outcome. 

Variance is based on  sandwich formula with IPCW adjustment, 
and naive.var is variance 
under a known censoring model. The influence functions are stored in the 
output. Further, the standard errors can be
cluster corrected by specifying the relevant cluster for the working outcome model.

 - We estimate the average treatment effect of our binary response $I(T \leq t, \epsilon=1)$
   - Using a working logistic model for the response (possibly with a cluster specification)
   - Using a working logistic model for treatment given covariates
     - The binregATE can also handle a factor with more than two levels and then uses
        the mlogit multinomial regression function (of mets). 
   - Using a working model for censoring given covariates, this must be a stratified Kaplan-Meier.

If there are no censoring then the censoring weights are simply set to 1.

The average treatment effect is 
\begin{align*}
 E(Y(1) - Y(0)) 
\end{align*}
using counterfactual outcomes. 

We compute the simple G-estimator 
\begin{align*}
\sum  m_a(X_i) 
\end{align*}
to estimate the risk $E(Y(a))$. 

The DR-estimator instead uses the estimating equations that are double robust wrt 

   - A working logistic model for the response
   - A working logistic model for treatment given covariates

This is estimated using the estimator 
\begin{align*}
\sum \left[ \frac{A_i Y_i}{\pi_A(X_i)}-\frac{A_i - \pi_A(X_i)}{\pi_A(X_i)} m_1(X_i) \right]
   - \left[ \frac{(1-A_i) Y_i}{1-\pi_A(X_i)}+\frac{A_i - \pi_A(X_i)}{1-\pi_A(X_i)} m_0(X_i) \right]
\end{align*}
where

 - $A_i$ is treatment indicator
 - $\pi_A(X_i) = P(A_i=1|X_i)$ is treatment model 
 - $Y_i$ outcome, that in case of censoring is censoring adjusted $\tilde Y_i \Delta(t) /G_c(T_i- \wedge t)$
 - $\tilde Y_i = I(T_i \leq t, \epsilon_i=1)$ outcome before censoring.
 - $m_j(X_i)=P(Y_i=1| A_i=j,X_i)$ is outcome model, using binomial regression.

 The standard errors are then based on an iid decomposition using taylor-expansions 
 for the
 parameters of the treatment-model and the outcome-model, and the 
 censoring probability. 

 We need that the censoring model is correct, so it can be important to use a sufficiently 
 large censoring model as we also illustrate below. 

   - The censoring model can be specified by strata (used for `phreg`).

We also compute standard marginalization for average treatment effect  (called differenceG) 
\begin{align*}
\sum \left[  m_1(X_i) - m_0(X_i) \right]
\end{align*}
and again standard errors are based on the related influence functions and are also returned. 

For large data where there are more than 2 treatment groups the 
computations can be memory extensive when
there are many covariates due to the multinomial-regression model used for the 
propensity scores. Otherwise
the function (binregATE) will run for large data.


The ATE functions require that the treatment variable, given as the first variable on
the right-hand side of the outcome model, is a factor. The variable is also
identified from the left-hand side of the
treatment model (`treat.model`), which by default
assumes that treatment does not depend on any covariates.


Average treatment effect for binary or continuous responses
===========================================================

In the binary case a binary outcome is specified instead of the survival outcome, and as a
consequence no censoring adjustment is done.

 - the binary/numeric outcome must be a variable in the data frame

The following code runs the analysis (one can also use `binregATE` by coding `cause` without censoring values, i.e. setting `cens.code=2` and `time` large):


``` r
kumar$cause2 <- 1*(kumar$cause==2)

b3 <- logitATE(cause2~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)
#>    n events
#>  400    400
#> 
#>  400 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -1.219960  0.200830 -1.613580 -0.826341  0.0000
#> gp.f1        0.387595  0.243738 -0.090123  0.865314  0.1118
#> dnr          0.633992  0.241410  0.160837  1.107147  0.0086
#> preauto      0.139356  0.248680 -0.348049  0.626761  0.5752
#> ttt24        0.449527  0.243475 -0.027675  0.926730  0.0648
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.29524 0.19917 0.4376
#> gp.f1        1.47343 0.91382 2.3758
#> dnr          1.88512 1.17449 3.0257
#> preauto      1.14953 0.70606 1.8715
#> ttt24        1.56757 0.97270 2.5262
#> 
#> Average Treatment effects (G-formula) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.316084  0.037954  0.241695  0.390472  0.0000
#> treat1     0.400809  0.033395  0.335356  0.466262  0.0000
#> treat:1-0  0.084726  0.052651 -0.018468  0.187919  0.1076
#> 
#> Average Treatment effects (double robust) :
#>            Estimate   Std.Err      2.5%     97.5% P-value
#> treat0     0.343451  0.042284  0.260577  0.426325  0.0000
#> treat1     0.421615  0.033616  0.355729  0.487500  0.0000
#> treat:1-0  0.078164  0.053961 -0.027599  0.183926  0.1475

###library(targeted)
###b3a <- ate(cause2~gp.f|dnr+preauto+ttt24| dnr+preauto+ttt24,kumar,family=binomial)
###summary(b3a)

## calculate also relative risk
estimate(coef=b3$riskDR,vcov=b3$var.riskDR,f=function(p) p[1]/p[2])
#>        Estimate Std.Err   2.5% 97.5%   P-value
#> treat0   0.8146  0.1194 0.5807 1.049 8.831e-12
```

Or with continuous response using normal estimating equations


``` r
b3 <- normalATE(time~gp.f+dnr+preauto+ttt24,kumar,treat.model=gp.f~dnr+preauto+ttt24)
summary(b3)
#>    n events
#>  400    400
#> 
#>  400 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept)  43.4758   4.0796  35.4800  51.4716  0.0000
#> gp.f1       -27.2211   4.1379 -35.3313 -19.1109  0.0000
#> dnr           2.9485   4.4768  -5.8259  11.7228  0.5101
#> preauto      -2.8172   3.6191  -9.9105   4.2760  0.4363
#> ttt24        -1.9787   4.1285 -10.0704   6.1131  0.6318
#> 
#> 
#> Average Treatment effects (G-formula) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     42.3170   3.8362  34.7982  49.8357       0
#> treat1     15.0958   1.3284  12.4922  17.6994       0
#> treat:1-0 -27.2211   4.1379 -35.3313 -19.1109       0
#> 
#> Average Treatment effects (double robust) :
#>           Estimate  Std.Err     2.5%    97.5% P-value
#> treat0     42.3200   4.0417  34.3985  50.2415       0
#> treat1     15.1545   1.2580  12.6888  17.6201       0
#> treat:1-0 -27.1655   4.2405 -35.4768 -18.8543       0
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         mvtnorm_1.3-7         
#> [13] numDeriv_2016.8-1.1    compiler_4.6.0         codetools_0.2-20      
#> [16] Rcpp_1.1.1-1.1         ucminf_1.2.3           future_1.70.0         
#> [19] lattice_0.22-9         digest_0.6.39          parallelly_1.47.0     
#> [22] parallel_4.6.0         splines_4.6.0          Matrix_1.7-5          
#> [25] tools_4.6.0            RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5        
vignette: >
  %\VignetteIndexEntry{Average treatment effect (ATE) for Restricted mean survival and years lost of Competing risks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```


The resmeanATE or resmeanIPCW/rmstIPCW functions can fit an exponential/linear link model with 
IPCW adjustment for a specific time-point for the resticted mean survival or years lost for 
competing risks. 
The function can be used for large data and is completely scalable, that is, linear in the data. 
The function can thus be used for large data. 
A nice feature is that influcence functions are computed and are available.

In addition and to summarize

  - the censoring weights can be strata dependent
  - predictions can be computed with standard errors 
  - computation time is linear in data
     - including standard errors
  - clusters can be given and then cluster corrected standard errors are computed



RMST 
====

Regression for rmst outcome $E(T \wedge t | X) = exp(X^T \beta)$ based on IPCW adjustment for censoring, thus solving
the estimating equation 
\begin{align*}
 & X^T [ (T \wedge t) \frac{I(C > T \wedge t)}{G_C(T \wedge t,X)} - exp(X^T \beta) ] = 0 .
\end{align*}
This is done with the resmeanIPCW function. 
For fully saturated model with full censoring model this is equal to the integrals of the Kaplan-Meier estimators as
illustrated below. 

We can also compute the integral of the Kaplan-Meier or Cox based  survival estimator to
get the RMST (with the resmean.phreg function)
\[ \int_0^t S(s|X) ds \].

For competing risks the years lost can be computed via cumulative incidence functions (cif.yearslost)
or as IPCW estimator since  
\[ E( I(\epsilon=1) ( t - T \wedge t ) | X) = \int_0^t F_1(s) ds. \] 
For fully saturated model with full censoring model these estimators are equivalent as illustrated below.


```{r}
set.seed(101)

     data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

     # E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
     out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age,bmt,
                     time=50,cens.model=~strata(platelet),model="exp")
     summary(out)
     
      ### same as Kaplan-Meier for full censoring model 
     bmt$int <- with(bmt,strata(tcell,platelet))
     out <- resmeanIPCW(Event(time,cause!=0)~-1+int,bmt,time=30,
                                  cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)
     rm1 <- resmean.phreg(out1,times=30)
     summary(rm1)
     
     ## competing risks years-lost for cause 1  
     out <- resmeanIPCW(Event(time,cause)~-1+int,bmt,time=30,cause=1,
                                 cens.model=~strata(platelet,tcell),model="lin")
     estimate(out)
     ## same as integrated cumulative incidence 
     rmc1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=30,cause=1)
     summary(rmc1)

     ## plotting the years lost for different horizon's and the two causes 
     par(mfrow=c(1,3))
     plot(rm1,years.lost=TRUE,se=1)
     ## cause refers to column of cumhaz for the different causes
     plot(rmc1,cause=1,se=1)
     plot(rmc1,cause=2,se=1)
```

Based on the output from the IPCW estimators we can derive any measure of interest

```{r}
estimate(out)

measures <- function(p) {
 ratio1 <- p[1]/p[2]; ratio2 <- p[2]/p[1]; dif1 <- p[4]-p[1]; dif2 <- p[3]-p[1]
 m <- c(dif1,dif2,ratio1,ratio2)
 return(m)
}

 labs <- c("dif4-1","dif3-1","ratio 1/2","ratio 2/1")
 estimate(out,f=measures,labels=labs)
```

Average treatment effect 
=========================

Computes average treatment effect for restricted mean survival and years lost in competing risks situation

```{r}
 dfactor(bmt) <- tcell~tcell
 bmt$event <- (bmt$cause!=0)*1
 out <- resmeanATE(Event(time,event)~tcell+platelet,data=bmt,time=40,treat.model=tcell~platelet)
 summary(out)
 
 out1 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=1,time=40,
		    treat.model=tcell~platelet)
 summary(out1)

 out2 <- resmeanATE(Event(time,cause)~tcell+platelet,data=bmt,cause=2,time=40,
		    treat.model=tcell~platelet)
 summary(out2)
```



SessionInfo
============


```{r}
sessionInfo()
```

---
title: "Analysis of multivariate binomial data: family analysis"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5    
vignette: >
  %\VignetteIndexEntry{Analysis of multivariate binomial data: family analysis} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Overview 
=============

When looking at multivariate binomial data with the aim of learning about the 
dependence that is present, possibly after correcting for some covariates many
models are available. 

   *  Random-effects models logistic regression covered elsewhere (glmer in lme4).

in the mets package you can fit the 

   *  Pairwise odds ratio model

   *  Bivariate Probit model 
      + With random effects
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

   *  Additive gamma random effects model 
      + Special functionality for polygenic random effects modelling 
        such as ACE, ADE ,AE and so forth.

These last three models are all fitted in the mets package using composite 
likelihoods for pairs of data.  The models can be fitted specifically based 
on specifying which pairs one wants to use for the composite score. 

The models are described in futher details in the binomial-twin vignette. 


Simulated family data
====================

We start by simulating family data with and additive gamma structure
on ACE form.  Here 10000 families consisting of two parents and
two children. The response is ybin and there is one covariate x. 

```{r}
 library(mets)
 library(timereg)
 set.seed(100)
 data <- simbinClaytonOakes.family.ace(500,2,1,beta=NULL,alpha=NULL)
 data$number <- c(1,2,3,4)
 data$child <- 1*(data$number==3)
 head(data)
```

We fit the marginal models, and here find a covariate
effect at 0.3 for x. The marginals can be specified excatly as one
wants. 

```{r}
 aa <- margbin <- glm(ybin~x,data=data,family=binomial())
 summary(aa)
```

Additive gamma model 
====================

For the additive gamma of this type we set-up the random effects
included in such a family to make the ACE valid using some
special functions for this. 

The model is constructe with one enviromental effect shared by
all in the family and 8 genetic random effects with size (1/4) genetic
variance. Looking at the first family we see that the mother and
father both share half the genes with the children and that the
two children also share half their genes with this specification.
Below we also show an alternative specification of this model
using all pairs. 

```{r}
# make ace random effects design
out <- ace.family.design(data,member="type",id="cluster")
out$pardes
head(out$des.rv,4)
```


We can now fit the model calling the two-stage function

```{r}
# fitting ace model for family structure
ts <- binomial.twostage(margbin,data=data,clusters=data$cluster,
theta=c(2,1),
random.design=out$des.rv,theta.des=out$pardes)
summary(ts)
# true variance parameters
c(2,1)
# total variance 
3
```


Pairwise fitting 
=================

We now specify the same model via extracting all pairs.  
The random
effecs structure is typically simpler when just looking at pairs, but we start by
specifying the random effects jointly for whole family. 
A special function writes up all combinations of pairs. 
There are 6 pairs within each family, and we keep track of
who belongs to the different families. We first simply give the
pairs and we then should get the same result as before. 



```{r}
mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)
```

Now with the pairs we fit the model 

```{r}
tsp <- binomial.twostage(margbin,data=data,clusters=data$cluster,theta=c(2,1),detail=0,
        random.design=out$des.rv,theta.des=out$pardes,pairs=pairs)
summary(tsp)
```

Here a random sample of pairs are given instead and we get other estimates. 

```{r}
set.seed(100)
ssid <- sort(sample(1:nrow(pairs),nrow(pairs)/2))
tsd <- binomial.twostage(aa,data=data,clusters=data$cluster,
               theta=c(2,1),step=1.0,
               random.design=out$des.rv,iid=1,Nit=10,
  	           theta.des=out$pardes,pairs=pairs[ssid,])
summary(tsd)
```

To specify such a model when only the pairs are available we show
how to specify the model. We here use the same marginal "aa"
to make the results comparable. 
The marginal can also be fitted based on available data. 

We start by selecting the data related to the pairs, and sets up new id's and 
to start we specify the model using the full design with 9 random effects. 
Below we show how one can use with only the random effects needed for each 
pair, which is typically simpler.


```{r}
head(pairs[ssid,])
ids <- sort(unique(c(pairs[ssid,])))

pairsids <- c(pairs[ssid,])
pair.new <- matrix(fast.approx(ids,c(pairs[ssid,])),ncol=2)
head(pair.new)

dataid <- dsort(data[ids,],"cluster")
outid <- ace.family.design(dataid,member="type",id="cluster")
outid$pardes
head(outid$des.rv)
```

Now fitting the model with the data set up 

```{r}
aa <- glm(ybin~x,data=dataid,family=binomial())
tsdid <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
         theta=c(2,1),random.design=outid$des.rv,theta.des=outid$pardes,pairs=pair.new)
summary(tsdid)
```

We now specify the design specifically using the pairs. 
The random.design and design on the parameters
are now given for each pair, as a 3 dimensional matrix. 
with a direct specification of random.design and the 
design on the parameters theta.design.
In addition we need also to give the number of random effects for
each pair. These basic things are constructed by certain functions
for the ACE design. 


```{r}
pair.types <-  matrix(dataid[c(t(pair.new)),"type"],byrow=T,ncol=2)
head(pair.new,7)
head(pair.types,7)
###
theta.des  <- rbind( c(rbind(c(1,0),  c(1,0),  c(0,1),  c(0,0))),
		     c(rbind(c(0.5,0),c(0.5,0),c(0.5,0),c(0,1))))
random.des <- rbind( 
        c(1,0,1,0),c(0,1,1,0),
        c(1,1,0,1),c(1,0,1,1))
mf <- 1*(pair.types[,1]=="mother" & pair.types[,2]=="father")
##          pair, rv related to pairs,  theta.des related to pair 
pairs.new <- cbind(pair.new,(mf==1)*1+(mf==0)*3,(mf==1)*2+(mf==0)*4,(mf==1)*1+(mf==0)*2,(mf==1)*3+(mf==0)*4)
```

pairs.new is matrix with 

 * columns 1:2 giving the indeces of the  data points

 * columns 3:4 giving the indeces of the random.design for the different pairs

 * columns 5 giving the indeces of the theta.des written as rows 

 * columns 6 giving the number of random variables for this pair


The length of all rows of theta.des
are the maximum number of random effects $\times$ the number of parameters. 
These two numbers are given in the call. In this case 4 $\times$ 2. 
So theta.des has rows of length $8$, possibly including some 0's for rows
not relevant due to fewer random effects, as is the case here for pairs 
that do not share genetic effects.


For pair 1 that is a mother/farther pair, we see that they share 
1 environmental random effect of size 1. There are also two genetic
effects that are unshared between the two. 
So a total of 3 random effects are needed here. The theta.des relates the 3 
random effects to possible relationships in the parameters. Here the genetic
effects are full and so is the environmental effect. 
In contrast we also consider a mother/child pair that share half the genes, now
with random effects with (1/2) gene variance.  We there need 4 random effects,
2 non-shared half-gene, 1 shared half-gene, and one shared full environmental 
effect. 




```{r}
# 3 rvs here 
random.des
theta.des

head(pairs.new)
```

Now fitting the model, and we see that it is a lot quicker due to the
fewer random effects needed for pairs.  We need to also specify the number of
parameters in this case. 

```{r}
tsdid2 <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster, theta=c(2,1),
           random.design=random.des,theta.des=theta.des,pairs=pairs.new,dim.theta=2)
summary(tsdid2)
```

The same model can be specifed even simpler via the kinship
coefficient.  For this speicification there are 4 random effects for
each pair, but some have variance 0. The mother-father pair, here
shares a random effect with variance 0, and have two non-shared genetic 
effects with full variance, in addition to a fully shared environmental 
effect.

```{r}
kinship  <- rep(0.5,nrow(pair.types))
kinship[pair.types[,1]=="mother" & pair.types[,2]=="father"] <- 0
head(kinship,n=10)

out <- make.pairwise.design(pair.new,kinship,type="ace") 
```

```{r}
tsdid3 <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
             theta=c(2,1)/9,random.design=out$random.design,
             theta.des=out$theta.des,pairs=out$new.pairs,dim.theta=2)
summary(tsdid3)
```



Pairwise dependence modelling 
------------------------------


```{r}
library(mets)
set.seed(1000)
data <- simbinClaytonOakes.family.ace(500,2,1,beta=NULL,alpha=NULL)
head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

mm <- familycluster.index(data$cluster)
head(mm$familypairindex,n=20)
pairs <- mm$pairs
dim(pairs)
head(pairs,12)
```


```{r}
 dtypes <- interaction( data[pairs[,1],"type"], data[pairs[,2],"type"])
 dtypes <- droplevels(dtypes)
 table(dtypes)
 dm <- model.matrix(~-1+factor(dtypes))
```

Now with the pairs we fit the model 

```{r}
aa <- glm(ybin~x,data=data,family=binomial())

tsp <- binomial.twostage(aa,data=data, clusters=data$cluster,
		 theta.des=dm,pairs=cbind(pairs,1:nrow(dm)))
summary(tsp)
```


Pairwise odds ratio model 
=========================

To fit the pairwise odds-ratio model in the case of a pair-specification there 
are two options for fitting the model. 

1. One option is to set up some artificial data similar to twin data with
  + a pair-cluster-id  (clusters) 
  + with a cluster-id to get GEE type standard errors (se.cluster) 
2. We can also use the specify the design via the theta.des that is also a 
   matrix of dimension pairs x design with the design for POR model.  


Starting by the second option. We need to start by specify the design of
the odds-ratio of each pair. We set up the data and find all combinations 
within the pairs. Subsequently, we remove all the empty groups, by grouping
together the factor levels 4:9, and then we construct the design. 

```{r}
tdp <-cbind( dataid[pair.new[,1],],dataid[pair.new[,2],])
names(tdp) <- c(paste(names(dataid),"1",sep=""),
		paste(names(dataid),"2",sep=""))
tdp <-transform(tdp,tt=interaction(type1,type2))
dlevel(tdp)
drelevel(tdp,newlevels=list(mother.father=4:9)) <-  obs.types~tt
dtable(tdp,~tt+obs.types)
tdp <- model.matrix(~-1+factor(obs.types),tdp)
```

We then can fit the pairwise model using the pairs and the pair-design for
descrbing the OR. The results are consistent with the the ACE model as
the mother-father have a lower dependence as is due only the environmental
effects. All other combinations should have the same dependence as also seem
to be the case. 

To fit the OR model it is generally recommended to use the var.link to
use the parmetrization with log-odd-ratio regression.


```{r}
###porpair <- binomial.twostage(aa,data=dataid,clusters=dataid$cluster,
###           theta.des=tdp,pairs=pair.new,model="or",var.link=1)
###summary(porpair)
```


SessionInfo
============

```{r}
sessionInfo()
```

---
title: "Cumulative Incidence Regression"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
vignette: >
  %\VignetteIndexEntry{Cumulative Incidence Regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



The `cifreg` function fits the Fine-Gray model and the logit-link cumulative incidence
model for the cause of interest in competing risks settings. Computation is linear in
data size, making it suitable for large datasets.
For the Fine-Gray model, predictions with standard errors can be provided for specific
time-points based on influence functions for the baseline and the regression coefficients.

Key features:

  - the baseline can be stratified
  - the censoring weights can be stratum-dependent
  - predictions can be computed with standard errors (Fine-Gray only)
  - computation time is linear in data size, including standard errors
  - Fine-Gray only: influence functions of baseline and regression coefficients are computed and available via `IC`, `iid`, and `iidBaseline`
  - cluster-corrected standard errors are available via the `clusters` argument


Fine-Gray model
================

Fine and Gray (1999) considered a cumulative incidence of the form
\begin{align*}
 F_1(t,X) & = P(T \leq t, \epsilon=1) = 1 - \exp( - \Lambda_0(t) \exp(X^T \beta)).
\end{align*}

In the case of independent right-censoring with the censoring distribution $G_c(t,X) = P(C > t | S(X))$ where
$S(X)$ is a set of strata defined from $X$, then an unbiased estimating equation is given by
\begin{align*}
  U^{FG}_{n}(\beta) = \sum_{i=0}^{n} \int_0^{+\infty} \left( X_i- E_n(t,\beta) \right) w_i(t,X_i) dN_{1,i}(t)  \text{ where }
	E_n(t,\beta)=\frac{\tilde S_1(t,\beta) }{\tilde S_0(t,\beta)},
\end{align*}
with \(w_i(t,X_i) = \frac{G_c(t,X_i)}{G_c(T_i \wedge t,X_i)} I( C_i > T_i \wedge t ) \)
,$\tilde S_k(t,\beta) = \sum_{j=1}^n X_j^k \exp(X_j^T\beta) Y_{1,j}(t)$
for $k=0,1$, and with $\tilde Y_{1,i}(t) = Y_{1,i}(t) w_i(t,X_i)$ for $i=1,...,n$.
$w_i(t)$ needs to be replaced by an estimator of the
censoring distribution; since it does not depend on $X$, we use
$\hat w_i(t) = \frac{\hat G_c(t,X_i)}{\hat G_c(T_i \wedge t,X_i)} I(C_i > T_i \wedge t)$
where $\hat G_c$ is the Kaplan-Meier estimator of the censoring distribution.

First we simulate some competing risks data using some utility functions.

We simulate data with two causes based on the Fine-Gray model:
\begin{align}
F_1(t,X) &  = P(T\leq t, \epsilon=1|X)=( 1 - exp(-\Lambda_1(t) \exp(X^T \beta_1))) \\
F_2(t,X) & = P(T\leq t, \epsilon=2|X)= ( 1 - exp(-\Lambda_2(t) \exp(X^T \beta_2))) \cdot (1 - F_1(\infty,X))  
\end{align}
where the baselines are given as $\Lambda_j(t) = \rho_j (1- exp(-t/\nu_j))$ for $j=1,2$, and the 
$X$ being two independent binomials. 
Alternatively, one can also replace the FG-model with a logistic link 
$\mbox{expit}( \Lambda_j(t) + \exp(X^T \beta_j))$.

The advantage of the Fine-Gray model is that it is easy to fit, easy to obtain
standard errors for, and quite flexible. On the downside,
the coefficients must be interpreted on the $\mbox{cloglog}$ scale.
Specifically, 
\begin{align}
\log(-\log( 1-F_1(t,X_1+1,X_2))) - \log(-\log( 1-F_1(t,X_1,X_2))) & =  \beta_1,
\end{align}
so the effect of an increase in $X_1$ is $\beta_1$ and leads to $1-F_1(t,X)$ on the $cloglog$ scale. 



``` r
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in simulations for p-values below.

 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 ## beta1=c(0.0,-0.1); beta2=c(-0.5,0.3)
 dats <- simul_cifs(n,rho1,rho2,beta,rc=0.5,rate=7)
 dtable(dats,~status)
#> 
#> status
#>   0   1   2 
#> 127  12 261
 dsort(dats) <- ~time
```

We have a look at the non-parametric cumulative incidence curves


``` r
 par(mfrow=c(1,2))
 cifs1 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=1)
 plot(cifs1)

 cifs2 <- cif(Event(time,status)~strata(Z1,Z2),dats,cause=2)
 plot(cifs2)
```

<img src="figure/cifreg-unnamed-chunk-3-1.png" alt="Nonparametric cumulative incidence curves by covariate strata."  />

Now fitting the Fine-Gray model 


``` r
 fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(fg)
#> 
#>    n events
#>  400     12
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.69686  0.38760  0.38882  0.0722
#> Z2 -0.85929  0.62453  0.61478  0.1689
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  2.00744 0.93911 4.2911
#> Z2  0.42346 0.12451 1.4402

 dd <- expand.grid(Z1=c(-1,1),Z2=0:1)
 pfg <- predict(fg,dd)
 plot(pfg,ylim=c(0,0.2))
```

<img src="figure/cifreg-unnamed-chunk-4-1.png" alt="Fine-Gray model predicted cumulative incidence curves."  />

and GOF based on cumulative residuals (Li et al. 2015) 


``` r
gofFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
#> Cumulative score process test for Proportionality:
#>    Sup|U(t)|  pval
#> Z1  3.011461 0.124
#> Z2  1.373513 0.227
```

showing no problem with the proportionality of the model. 


SE's for the baseline and predictions of FG
===========================================

The standard errors reported for the Fine-Gray estimator are based on the i.i.d.
decomposition (influence functions) of the estimator. A similar decomposition
exists for the baseline and is needed when standard errors of predictions are computed. These
are somewhat harder to compute for all time-points simultaneously, but they can be
obtained for specific time-points jointly with the i.i.d. decomposition of the regression coefficients,
and then used to obtain standard errors for predictions.

We plot the predictions with confidence intervals for predictions at time point 5:


``` r
### predictions with CI based on iid decomposition of baseline and beta
fg <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1)
Biid <- iidBaseline(fg,time=5)
pfgse <- FGprediid(Biid,dd)
pfgse
#>            pred    se-log       lower     upper
#> [1,] 0.04253879 0.7418354 0.009938793 0.1820692
#> [2,] 0.16069100 0.3946377 0.074143886 0.3482633
#> [3,] 0.01823957 0.9410399 0.002884032 0.1153531
#> [4,] 0.07149610 0.4611261 0.028958169 0.1765199
plot(pfg,ylim=c(0,0.2))
for (i in 1:4) lines(c(5,5)+i/10,pfgse[i,3:4],col=i,lwd=2)
```

<img src="figure/cifreg-pred-cif-1.png" alt="Fine-Gray predictions with confidence intervals at time 5."  />

The i.i.d. decompositions are stored inside `Biid`; the
i.i.d. decomposition for $\hat \beta - \beta_0$ is obtained via the `iid()` function.

# Comparison 

We compare with the `cmprsk` function, which gives exactly the same results, but omit the code to avoid dependencies:


``` r
run <- 0
if (run==1) {
library(cmprsk)
mm <- model.matrix(~Z1+Z2,dats)[,-1]
cr <- with(dats,crr(time,status,mm))
cbind(cr$coef,diag(cr$var)^.5,fg$coef,fg$se.coef,cr$coef-fg$coef,diag(cr$var)^.5-fg$se.coef)
#          [,1]      [,2]       [,3]      [,4]          [,5]          [,6]
# Z1  0.6968603 0.3876029  0.6968603 0.3876029 -2.442491e-15 -2.553513e-15
# Z2 -0.8592892 0.6245258 -0.8592892 0.6245258 -2.997602e-15  1.776357e-15
}
```

When comparing with the results from `coxph` based on setting up the data using
the `finegray` function, we get the same estimates but note that the standard errors
from `coxph` are missing a term and therefore slightly different. When comparing to
the estimates from `coxph` without the additional censoring term, we also get
the same standard errors.


``` r
if (run==1) {
 library(survival)
 dats$id <- 1:nrow(dats)
 dats$event <- factor(dats$status,0:2, labels=c("censor", "death", "other"))
 fgdats <- finegray(Surv(time,event)~.,data=dats)
 coxfg <- survival::coxph(Surv(fgstart, fgstop, fgstatus) ~ Z1+Z2 + cluster(id), weight=fgwt, data=fgdats)

 fg0 <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1,propodds=NULL)
 cbind( coxfg$coef,fg0$coef, coxfg$coef-fg0$coef)
#          [,1]       [,2]          [,3]
# Z1  0.6968603  0.6968603 -1.110223e-16
# Z2 -0.8592892 -0.8592892 -1.110223e-15
 cbind(diag(coxfg$var)^.5,fg0$se.coef,diag(coxfg$var)^.5-fg0$se.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3876029  0.0013099915
# [2,] 0.6241225 0.6245258 -0.0004033148
 cbind(diag(coxfg$var)^.5,fg0$se1.coef,diag(coxfg$var)^.5-fg0$se1.coef)
#           [,1]      [,2]          [,3]
# [1,] 0.3889129 0.3889129 -2.331468e-15
# [2,] 0.6241225 0.6241225  2.553513e-15
}
```

We also remove all censorings from the data to compare the estimates with those
based on `coxph`, and observe that both the estimates and the standard errors agree.


``` r
datsnc <- dtransform(dats,status=2,status==0)
dtable(datsnc,~status)
#> 
#> status
#>   1   2 
#>  12 388
datsnc$id <- 1:n
datsnc$entry <- 0
max <- max(dats$time)+1
## for cause 2 add risk interaval 
datsnc2 <- subset(datsnc,status==2)
datsnc2 <- transform(datsnc2,entry=time)
datsnc2 <- transform(datsnc2,time=max)
datsncf <- rbind(datsnc,datsnc2)
#
cifnc <- cifreg(Event(time,status)~Z1+Z2,data=datsnc,cause=1,propodds=NULL)
cc <- phreg(Surv(entry,time,status==1)~Z1+Z2+cluster(id),datsncf)
cbind(cc$coef-cifnc$coef, diag(cc$var)^.5-diag(cifnc$var)^.5)
#>            [,1]          [,2]
#> Z1 1.221245e-15 -1.498801e-15
#> Z2 3.996803e-15  1.998401e-15
#            [,1]          [,2]
# Z1 1.332268e-15 -4.440892e-16
# Z2 4.218847e-15  2.220446e-16
```

the cmprsk also gives the same 


``` r
if (run==1) {
 library(cmprsk)
 mm <- model.matrix(~Z1+Z2,datsnc)[,-1]
 cr <- with(datsnc,crr(time,status,mm))
 cbind(cc$coef-cr$coef, diag(cr$var)^.5-diag(cc$var)^.5)
#             [,1]         [,2]
# Z1 -4.218847e-15 1.443290e-15
# Z2  7.549517e-15 1.110223e-16
}
```


# Strata dependent Censoring weights 

We can improve efficiency and reduce bias by allowing the censoring weights to depend on
the covariates.


``` r
 fgcm <- cifregFG(Event(time,status)~Z1+Z2,data=dats,cause=1,cens.model=~strata(Z1,Z2))
 summary(fgcm)
#> 
#>    n events
#>  400     12
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.54277  0.37188  0.39352  0.1444
#> Z2 -0.91846  0.61886  0.61447  0.1378
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.72077 0.83019 3.5667
#> Z2  0.39913 0.11867 1.3424
 summary(fg)
#> 
#>    n events
#>  400     12
#> 
#>  400 clusters
#> coefficients:
#>    Estimate     S.E.  dU^-1/2 P-value
#> Z1  0.69686  0.38760  0.38882  0.0722
#> Z2 -0.85929  0.62453  0.61478  0.1689
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  2.00744 0.93911 4.2911
#> Z2  0.42346 0.12451 1.4402
```

We note that the standard errors are slightly smaller for the more efficient estimator.

The influence functions of the Fine-Gray estimator are given by Fine and Gray (1999),
\begin{align*}
\phi_i^{FG}  & = \int (X_i- e(t))  \tilde w_i(t) dM_{i1}(t,X_i)  + \int \frac{q(t)}{\pi(t)} dM_{ic}(t),  \\
             & = \phi_i^{FG,1}  + \phi_i^{FG,2},
\end{align*}
where the first term is what would be achieved for a known censoring distribution, and the second term is
due to the variability from the Kaplan-Meier estimator. 
Where $M_{ic}(t) = N_{ic}(t) - \int_0^t Y_i(s) d\Lambda_c (s)$
with $M_{ic}$ the standard censoring martingale. 

The function $q(t)$ that reflects that the censoring only affects the terms related 
to cause "2" jumps, can be written as 
\begin{align*} 
	q(t) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(C > T)/G_c(T)) = E( H(t,X) F_2(t,X) ), 
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s) d \Lambda_1(s,X)$ and since $\pi(t)=E(Y(t))=S(t) G_c(t)$.

In the case where the censoring weights are stratified (based on $X$)
we get the influence functions related to the censoring term with 
\begin{align*} 
	q(t,X) & =   E( H(t,X) I(T  \leq t, \epsilon=2) I(T < C)/G_c(T,X) | X) = H(t,X) F_2(t,X), 
\end{align*}
so that the influence function becomes 
\begin{align*}
\int (X-e(t)) w(t) dM_1(t,X) +  \int H(t,X) \frac{F_2(t,X)}{S(t,X)} \frac{1}{G_c(t,X)} dM_c(t,X).
\end{align*}
with $H(t,X) =  \int_t^{\infty} (X- e(s))  G(s,X) d \Lambda_1(s,X)$.


Logistic-link 
================


``` r
 rho1 <- 0.2; rho2 <- 10
 n <- 400
 beta=c(0.0,-0.1,-0.5,0.3)
 dats <- simul_cifs(n,rho1,rho2,beta,rc=0.5,rate=7,type="logistic")
 dtable(dats,~status)
#> 
#> status
#>   0   1   2 
#> 166  16 218
 dsort(dats) <- ~time
```

The model
\begin{align*}
 \mbox{logit}(F_1(t,X)) & =  \alpha(t) + X^T \beta
\end{align*}
leads to an odds-ratio interpretation of $F_1$ and can be fitted easily; however, the standard errors
are harder to compute and only approximate
(assuming that the censoring weights are known), though this typically introduces only a small error.
In the `timereg` package the model can be
fitted using different estimators that are more efficient but considerably slower.

Fitting the model and getting OR's 


``` r
 or <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1)
 summary(or)
#> 
#>    n events
#>  400     16
#> 
#>  400 clusters
#> coefficients:
#>    Estimate    S.E. dU^-1/2 P-value
#> Z1  0.10017 0.25562 0.25215  0.6952
#> Z2  0.21763 0.50407 0.50346  0.6659
#> 
#> exp(coefficients):
#>    Estimate    2.5%  97.5%
#> Z1  1.10535 0.66976 1.8242
#> Z2  1.24313 0.46287 3.3387
```


Administrative Censoring
===========================

In the case with administrative censoring we can give the risk-set defined by the administrative censoring 
times for the Fine-Gray or logistic link cumulative incidence regression models. 

 - when an administrative censoring time is given, the risk-set is extended for all subjects experiencing the alternative event
 - if there are also additional random censorings, these can be adjusted for via IPCW weights, specified using `cens.code`
 - alternative causes are found by considering `cause`, `cens.code`, and the total number of values for the status variable; it can therefore be useful to also specify the other `death.code`(s)



``` r
library(mets)
rho1 <- 0.3; rho2 <- 5.9
set.seed(100)
n <- 100
beta=c(0.3,-0.3,-0.5,0.3)
rc <- 0.5
###
dats <- mets:::simul_cifsRA(n,rho1,rho2,beta,bin=1,rc=rc,rate=c(3,7))
dats$status07 <- dats$status
dats$status07[dats$status %in% c(0,7)] <- 0
tt <- seq(0,6,by=0.1)
base1 <- rho1*(1-exp(-tt/3))
dtable(dats,~status+statusA,level=1)
#> 
#> status
#>  0  1  2  7 
#> 17 16 58  9 
#> 
#> statusA
#>  1  2  7 
#> 21 64 15

## only admin censoring 
ccA  <-  cifregFG(Event(timeA,statusA)~Z1+Z2,dats,
		  adm.cens.time=dats$censorA,death.code=2)
estimate(ccA)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1  0.08665  0.2116 -0.3280 0.5014  0.6821
#> Z2  0.40535  0.4276 -0.4328 1.2435  0.3432

## admin and random censoring  via IPCW for C=min(C_A,C_R) 
ccAR_ipcw1  <-  cifregFG(Event(time,status)~Z1+Z2,dats,cens.code=c(0,7))
estimate(ccAR_ipcw1)
#>    Estimate Std.Err    2.5% 97.5% P-value
#> Z1  0.02131  0.2376 -0.4444 0.487  0.9285
#> Z2  0.32213  0.4844 -0.6272 1.271  0.5060

## admin and random censoring  via IPCW for C_R 
ccAR_ipcw2  <-  cifregFG(Event(time,status)~Z1+Z2,dats,cens.code=0,
		  adm.cens.time=dats$censorA,no.codes=7)
estimate(ccAR_ipcw2)
#>    Estimate Std.Err    2.5% 97.5% P-value
#> Z1  0.03723  0.2381 -0.4295 0.504  0.8758
#> Z2  0.34502  0.4859 -0.6074 1.297  0.4777
```

When there is only administrative censoring, the Fine-Gray model
can similarly be estimated using the modified risk-set and the
`phreg`/`coxph` function (equivalent to `ccA` above).


``` r
dats$entry <- 0
dats$id <- 1:n
datA <- dats
datA2 <- subset(datA,statusA==2)
datA2$entry <- datA2$timeA
datA2$timeA <- datA2$censorA
datA2$statusA <- 0
datA <- rbind(datA,datA2)
ddA <- phreg(Event(entry,timeA,statusA==1)~Z1+Z2+cluster(id),datA)
estimate(ddA)
#>    Estimate Std.Err    2.5%  97.5% P-value
#> Z1  0.08665  0.2116 -0.3280 0.5014  0.6821
#> Z2  0.40535  0.4276 -0.4328 1.2435  0.3432

## also checking the cumulative baseline 
###plotl(tt,base1) 
###plot(ccA,add=TRUE,col=3)
###plot(ddA,col=2,add=TRUE)
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
title: "IPCW Cumulative Cost"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: no
header-includes: 
  - \usepackage{tikz}
vignette: >
  %\VignetteIndexEntry{IPCW Cumulative Cost}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview
========

We describe how to perform regression modelling for cumulative cost
\begin{align*}
{\cal U}(t) & = \int_0^t Z(s) dN(s)
\end{align*}
where $N(s)$ is a counting process that registers the times at which costs are realised and accumulated,
and $Z(t)$ is the cost (or mark) at the event times. The counting process can be
a mix of random and fixed times, and the data are represented in counting process format with
the marks/costs attached to the event times. There are many additional uses of
such cumulative processes; for example, when considering time lost in a recurrent events setting, which we return to
below.

We can estimate the marginal mean of the cumulative process
\begin{align*}
\nu(t) & = E ( {\cal U}(t) )  
\end{align*}
possibly for strata with standard errors based on the derived influence function. 

We provide semi-parametric regression modelling using the  proportional model 
\begin{align*}
E ( {\cal U}(t) | X) & = \Lambda_0(t) \exp( X^T \beta).
\end{align*}

In addition for a fixed time-point $t \in [0,\tau]$ we can estimate the mean given covariates
\begin{align*}
E ( {\cal U}(t) | X) & = \exp( X^T \beta) 
\end{align*}
where $\tau$ is some maximum follow-up time. 

 - These quantities are estimated under independent right-censoring given $X$, using IPCW-adjusted estimating equations,
    - similarly to the Ghosh-Lin model for recurrent events.
 - A terminal event can be specified.

We also estimate the probability of exceeding thresholds over time
\begin{align*}
 P ( {\cal U}(t)  > k ) & = \mu_k(t),
\end{align*}
and in the setting with a terminal event this is based on a derived competing risks data structure that
keeps track of the competing terminal event.

\begin{tikzpicture}[
    node distance=3.5cm,
    box/.style={draw, rectangle, minimum width=2.8cm, minimum height=1cm, align=center}
]
% Nodes
\node[box] (start) {At Risk};
\node[box, right of=start, yshift=2.4cm] (exceed) {Exceed-k};
\node[box, right of=start, yshift=-2.4cm] (death) {D};

% Arrows with hazard labels
\draw[->, thick] (start) -- node[below] {\hspace{2 cm} $F_{exceed-k}(t)$} (exceed);
\draw[->, thick] (start) -- node[below] {$F_{D}(t)$} (death);
\end{tikzpicture}

Regression modelling of this quantity is also possible using competing risks
regression models, for example via the `cifreg` function in mets.


HF-action data
==============

Using the HF-action data, we simulate a severity score for each event.



``` r
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
set.seed(1)
hf$severity <- abs((5+rnorm(741)*2))[hf$id]

## marginal mean using formula  
outNZ <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id)
			 +marks(severity),hf,cause=1,death.code=2)
plot(outNZ,se=TRUE)
summary(outNZ,times=3) 
#> [[1]]
#>     new.time    mean        se CI-2.5% CI-97.5% strata
#> 682        3 10.3249 0.6044746  9.2056  11.5803      0
#> 
#> [[2]]
#>     new.time    mean        se  CI-2.5% CI-97.5% strata
#> 601        3 9.33613 0.6429995 8.157232 10.68541      1

outN <- recurrent_marginal(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
		       cause=1,death.code=2)
plot(outN,se=TRUE,add=TRUE)
```

<img src="figure/cumulative-cost-unnamed-chunk-2-1.png" alt="Marginal mean cumulative cost by treatment."  />

``` r
summary(outN,times=3) 
#> [[1]]
#>     new.time     mean        se  CI-2.5% CI-97.5% strata
#> 682        3 2.118496 0.1138572 1.906692 2.353829      0
#> 
#> [[2]]
#>     new.time     mean        se  CI-2.5% CI-97.5% strata
#> 601        3 1.924062 0.1216577 1.699801 2.177912      1
```

For comparison we also compute the IPCW estimates with and without marks at time 3 using the linear model,
and note that they are identical. Standard errors are based on different formulae that are asymptotically
equivalent, and we note that they are very similar.


``` r
outNZ3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(severity),data=hf,
		  cause=1,death.code=2,time=3,cens.model=~strata(treatment),model="lin")
summary(outNZ3)
#>    n events
#>  741   1281
#> 
#>  741 clusters
#> coeffients:
#>            Estimate  Std.Err     2.5%    97.5% P-value
#> treatment0 10.32490  0.60444  9.14022 11.50959       0
#> treatment1  9.33613  0.64295  8.07597 10.59629       0
head(iid(outNZ3))
#>            [,1]      [,2]
#> 1 -0.0013037837 0.0000000
#> 2  0.0089249278 0.0000000
#> 3  0.0000000000 0.0211858
#> 4 -0.0056613816 0.0000000
#> 5 -0.0005288663 0.0000000
#> 6 -0.0342554114 0.0000000

outN3 <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,cause=1,death.code=2,time=3,
		 cens.model=~strata(treatment),model="lin")
summary(outN3)
#>    n events
#>  741   1281
#> 
#>  741 clusters
#> coeffients:
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment0  2.11850 0.11385 1.89535 2.34164       0
#> treatment1  1.92406 0.12165 1.68564 2.16248       0
head(iid(outN3))
#>            [,1]        [,2]
#> 1  0.0004542472 0.000000000
#> 2  0.0009756994 0.000000000
#> 3  0.0000000000 0.009301496
#> 4 -0.0029668336 0.000000000
#> 5 -0.0001120764 0.000000000
#> 6 -0.0070693971 0.000000000
```

We also apply the semiparametric proportional cost model with IPCW adjustment: 


``` r
propNZ <- recreg(Event(entry,time,status)~treatment+marks(severity)+cluster(id),data=hf,cause=1,death.code=2)
summary(propNZ) 
#> 
#>     n events
#>  2132   1391
#> 
#>  741 clusters
#> coefficients:
#>             Estimate      S.E.   dU^-1/2 P-value
#> treatment1 -0.101014  0.089005  0.024304  0.2564
#> 
#> exp(coefficients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.90392 0.75922 1.0762
plot(propNZ,main="Baselines")
     
GL <- recreg(Event(entry,time,status)~treatment+cluster(id),hf,cause=1,death.code=2)
summary(GL)
#> 
#>     n events
#>  2132   1391
#> 
#>  741 clusters
#> coefficients:
#>             Estimate      S.E.   dU^-1/2 P-value
#> treatment1 -0.110404  0.078656  0.053776  0.1604
#> 
#> exp(coefficients):
#>            Estimate    2.5%  97.5%
#> treatment1  0.89547 0.76754 1.0447
plot(GL,add=TRUE,col=2)
```

<img src="figure/cumulative-cost-unnamed-chunk-4-1.png" alt="Proportional cost model baseline estimates."  />
Those treated have 14\% lower cumulative severity and 11\% lower expected number of events.


Exceed threshold
================

Finally, we estimate the probability of exceeding cumulative severity thresholds of 1, 5, and 10:


``` r
ooNZ <- prob_exceed_recurrent(Event(entry,time,status)~strata(treatment)+cluster(id)+marks(severity),data=hf,
			      cause=1,death.code=2,exceed=c(1,5,10,20))
plot(ooNZ,strata=1)
plot(ooNZ,strata=2,add=TRUE)
```

<img src="figure/cumulative-cost-unnamed-chunk-5-1.png" alt="Probability of exceeding cumulative cost thresholds."  />

``` r
summary(ooNZ,times=3)
#> $`0`
#> $`0`$prob
#>            times           
#>                3 2.99865085
#> N<1            3 0.06474819
#> exceed>=1      3 0.93525181
#> exceed>=5      3 0.90118229
#> exceed>=10     3 0.74537955
#> exceed>=20     3 0.49282394
#> 
#> $`0`$se
#>            times           
#>                3 2.99865085
#> N<1            3 0.02657120
#> exceed>=1      3 0.02657120
#> exceed>=5      3 0.03013468
#> exceed>=10     3 0.04466101
#> exceed>=20     3 0.05149605
#> 
#> $`0`$lower
#>      times          
#> [1,]     3 2.9986509
#> [2,]     3 0.1154033
#> [3,]     3 0.8845967
#> [4,]     3 0.8440133
#> [5,]     3 0.6627899
#> [6,]     3 0.4015580
#> 
#> $`0`$upper
#>      times           
#> [1,]     3 2.99865085
#> [2,]     3 0.01119234
#> [3,]     3 0.98880766
#> [4,]     3 0.96222364
#> [5,]     3 0.83826056
#> [6,]     3 0.60483284
#> 
#> 
#> $`1`
#> $`1`$prob
#>            times           
#>                3 2.99865085
#> N<1            3 0.01770732
#> exceed>=1      3 0.98229268
#> exceed>=5      3 0.94934557
#> exceed>=10     3 0.79560979
#> exceed>=20     3 0.53319063
#> 
#> $`1`$se
#>            times           
#>                3 2.99865085
#> N<1            3 0.01097663
#> exceed>=1      3 0.01097663
#> exceed>=5      3 0.01645312
#> exceed>=10     3 0.03727391
#> exceed>=20     3 0.05545584
#> 
#> $`1`$lower
#>      times           
#> [1,]     3 2.99865085
#> [2,]     3 0.03898725
#> [3,]     3 0.96101275
#> [4,]     3 0.91763959
#> [5,]     3 0.72580801
#> [6,]     3 0.43486168
#> 
#> $`1`$upper
#>      times          
#> [1,]     3 2.9986509
#> [2,]     3 0.0000000
#> [3,]     3 1.0000000
#> [4,]     3 0.9821470
#> [5,]     3 0.8721245
#> [6,]     3 0.6537533
```

Cumulative time lost for recurrent events
=========================================

The cumulative time lost for recurrent events is defined as
\begin{align*}
  {\cal M}(t) = E\left[ \int_0^\tau (\tau-s) dN(s) \right] = \int_0^\tau \mu(s) ds
\end{align*}
where $\mu(t) = E( N(t) )$ is the marginal mean of the recurrent events at time $t$.



``` r
hf$lost5 <- 5-hf$time

RecLost <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id)+marks(lost5),data=hf,
		   cause=1,death.code=2,time=5,cens.model=~strata(treatment),model="lin")
summary(RecLost)
#>    n events
#>  741   1391
#> 
#>  741 clusters
#> coeffients:
#>            Estimate Std.Err    2.5%   97.5% P-value
#> treatment0  8.58300 0.42951 7.74118 9.42482       0
#> treatment1  7.66234 0.46400 6.75292 8.57177       0
head(iid(RecLost))
#>            [,1]       [,2]
#> 1  0.0016920221 0.00000000
#> 2  0.0073388996 0.00000000
#> 3  0.0000000000 0.02120478
#> 4 -0.0095548150 0.00000000
#> 5 -0.0005696809 0.00000000
#> 6 -0.0201750011 0.00000000
```


SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] cli_3.6.6              knitr_1.51             rlang_1.2.0           
#>  [4] xfun_0.57              otel_0.2.0             future.apply_1.20.2   
#>  [7] listenv_0.10.1         lava_1.9.1             stats4_4.6.0          
#> [10] grid_4.6.0             evaluate_1.0.5         yaml_2.3.12           
#> [13] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [16] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [19] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [22] parallelly_1.47.0      parallel_4.6.0         splines_4.6.0         
#> [25] Matrix_1.7-5           tools_4.6.0            RcppArmadillo_15.2.6-1
#> [28] globals_0.19.1
```

---
# YAML header created by ox-ravel
title: Twin models
author: Klaus Holst & Thomas Scheike
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Twin models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->

This document provides a brief tutorial to analyzing twin data using the
**`mets`** package:



$$
\newcommand{\cov}{\mathbb{C}\text{ov}}
\newcommand{\cor}{\mathbb{C}\text{or}}
\newcommand{\var}{\mathbb{V}\text{ar}}
\newcommand{\E}{\mathbb{E}}
\newcommand{\unitfrac}[2]{#1/#2}
\newcommand{\n}{}
$$

 <!-- 
 The development version may be installed from *github*: 
 -->



# Twin analysis, continuous traits

  In the following we examine the heritability of Body Mass
Index\n{}<sup id="b71edfd9bc946c317f4a732845bcaf93"><a href="#korkeila_bmi_1991" title="Korkeila, Kaprio, Rissanen \&amp; Koskenvuo, {{E}ffects of gender and age on the heritability of body mass index}, {Int J Obes}, v(10), 647--654 (1991).">korkeila_bmi_1991</a></sup> <sup id="718839fcb6ade82ebb2d7de853582b80"><a href="#hjelmborg_bmi_2008" title="Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen \&amp; Kaprio, {{G}enetic influences on growth traits of {B}{M}{I}: a longitudinal study of adult twins}, {Obesity (Silver Spring)}, v(4), 847--852 (2008).">hjelmborg_bmi_2008</a></sup>, based on data
on self-reported BMI-values from a random sample of 11,411 same-sex
twins. First, we will load data


``` r
library(mets)
data("twinbmi")
head(twinbmi)
#>   tvparnr      bmi      age gender zyg id num
#> 1       1 26.33289 57.51212   male  DZ  1   1
#> 2       1 25.46939 57.51212   male  DZ  1   2
#> 3       2 28.65014 56.62696   male  MZ  2   1
#> 5       3 28.40909 57.73097   male  DZ  3   1
#> 7       4 27.25089 53.68683   male  DZ  4   1
#> 8       4 28.07504 53.68683   male  DZ  4   2
```

The data is on *long* format with one subject per row.

-   **`tvparnr`:** twin id
-   **`bmi`:** Body Mass Index (\(\mathrm{kg}/{\mathrm{m}^2}\))
-   **`age`:** Age (years)
-   **`gender`:** Gender factor (male,female)
-   **`zyg`:** zygosity (MZ, DZ)

We transpose the data allowing us to do pairwise analyses


``` r
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
head(twinwide)
#>    tvparnr     bmi1      age gender zyg id num     bmi2
#> 1        1 26.33289 57.51212   male  DZ  1   1 25.46939
#> 3        2 28.65014 56.62696   male  MZ  2   1       NA
#> 5        3 28.40909 57.73097   male  DZ  3   1       NA
#> 7        4 27.25089 53.68683   male  DZ  4   1 28.07504
#> 9        5 27.77778 52.55838   male  DZ  5   1       NA
#> 11       6 28.04282 52.52231   male  DZ  6   1 22.30936
```

Next we plot the association within each zygosity group

We show the log-transformed data here, which is slightly more symmetric
and more appropriate for the twin analysis (see Figure \@ref(fig:scatter1) and \@ref(fig:scatter2)).


``` r
mz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])
plot_twin(mz)
```

![Scatter plot of logarithmic BMI measurements in MZ twins](figure/quantitative-twin-scatter1-1.png)


``` r
dz <- log(subset(twinwide, zyg=="DZ")[,c("bmi1","bmi2")])
plot_twin(dz)
```

![Scatter plot of logarithmic BMI measurements in DZ twins](figure/quantitative-twin-scatter2-1.png)

The plots and raw association measures show considerably stronger
dependence in the MZ twins, indicating genetic influence on the trait.


``` r
cor.test(mz[,1],mz[,2], method="spearman")
#> 
#> 	Spearman's rank correlation rho
#> 
#> data:  mz[, 1] and mz[, 2]
#> S = 165457624, p-value < 2.2e-16
#> alternative hypothesis: true rho is not equal to 0
#> sample estimates:
#>       rho 
#> 0.6956209
```


``` r
cor.test(dz[,1],dz[,2], method="spearman")
#> 
#> 	Spearman's rank correlation rho
#> 
#> data:  dz[, 1] and dz[, 2]
#> S = 2162514570, p-value < 2.2e-16
#> alternative hypothesis: true rho is not equal to 0
#> sample estimates:
#>       rho 
#> 0.4012686
```

Next we examine the marginal distribution (GEE model with working independence):


``` r
l0 <- lm(bmi ~ gender + I(age-40), data=twinbmi)
estimate(l0, id=twinbmi$tvparnr)
#>             Estimate  Std.Err    2.5%   97.5%    P-value
#> (Intercept)  23.3687 0.054534 23.2618 23.4756  0.000e+00
#> gendermale    1.4077 0.073216  1.2642  1.5512  2.230e-82
#> I(age - 40)   0.1177 0.004787  0.1083  0.1271 1.499e-133
```


``` r
library("splines")
l1 <- lm(bmi ~ gender*ns(age,3), data=twinbmi)
marg1 <- estimate(l1, id=twinbmi$tvparnr)
```


``` r
dm <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("male"),
	    age=seq(33,61,length.out=50))
df <- lava::Expand(twinbmi,
	    bmi=0,
	    gender=c("female"),
	    age=seq(33,61,length.out=50))

plot(marg1, function(p) model.matrix(l1,data=dm)%*%p,
     data=dm["age"], ylab="BMI", xlab="Age",
     ylim=c(22,26.5))
plot(marg1, function(p) model.matrix(l1,data=df)%*%p,
     data=df["age"], col="red", add=TRUE)
legend("bottomright", c("Male","Female"),
       col=c("black","red"), lty=1, bty="n")
```

![Marginal association between BMI and Age for males and females.](figure/quantitative-twin-marg1-1.png)


## Polygenic model

We can decompose the trait into the following variance components

\begin{align*}
Y_i = A_i + D_i + C + E_i, \quad i=1,2
 \end{align*}

-   **\(A\):** Additive genetic effects of alleles
-   **\(D\):** Dominant genetic effects of alleles
-   **\(C\):** Shared environmental effects
-   **\(E\):** Unique environmental effects

Dissimilarity of MZ twins arises from unshared environmental effects
only, \(\cor(E_1,E_2)=0\) and

\begin{align*}
\cor(A_1^{MZ},A_2^{MZ}) = 1, \quad
\cor(D_1^{MZ},D_2^{MZ}) = 1,
\end{align*}

\begin{align*}
\cor(A_1^{DZ},A_2^{DZ}) = 0.5, \quad
\cor(D_1^{DZ},D_2^{DZ}) = 0.25,
\end{align*}

\begin{align*}
Y_i = A_i + C_i + D_i + E_i
\end{align*}

\begin{align*}
A_i \sim\mathcal{N}(0,\sigma_A^2), C_i
\sim\mathcal{N}(0,\sigma_C^2), D_i
\sim\mathcal{N}(0,\sigma_D^2),
E_i \sim\mathcal{N}(0,\sigma_E^2)
\end{align*}

\begin{gather*}
    \cov(Y_{1},Y_{2}) = \\
    \begin{pmatrix}
      \sigma_A^2 & 2\Phi\sigma_A^2 \\
      2\Phi\sigma_A^2 & \sigma_A^2
    \end{pmatrix} +
    \begin{pmatrix}
      \sigma_C^2 & \sigma_C^2 \\
      \sigma_C^2 & \sigma_C^2
  \end{pmatrix} +
    \begin{pmatrix}
      \sigma_D^2 & \Delta_{7}\sigma_D^2 \\
      \Delta_{7}\sigma_D^2 & \sigma_D^2
  \end{pmatrix} +
  \begin{pmatrix}
    \sigma_E^2 & 0 \\
    0 & \sigma_E^2
  \end{pmatrix}
\end{gather*}



``` r
dd <- na.omit(twinbmi)
```

Saturated model (different marginals in MZ and DZ twins and different marginals
for twin 1 and twin 2):

``` r
l0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="sat")
```

Different marginals for MZ and DZ twins (but same marginals within a pair)

``` r
lf <- twinlm(bmi ~ age+gender, data=dd,DZ="DZ", zyg="zyg", id="tvparnr", type="flex")
```
 
Same marginals but free correlation with MZ, DZ

``` r
lu <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="eqmarg")
estimate(lu)
#>                      Estimate  Std.Err    2.5%   97.5%    P-value
#> bmi.1@1               18.6037 0.251036 18.1116 19.0957  0.000e+00
#> bmi.1~age.1@1          0.1189 0.005635  0.1078  0.1299  9.177e-99
#> bmi.1~gendermale.1@1   1.3848 0.086573  1.2151  1.5544  1.376e-57
#> log(var)@1             2.4424 0.022095  2.3991  2.4857  0.000e+00
#> atanh(rhoMZ)@1         0.7803 0.036249  0.7092  0.8513 9.008e-103
#> atanh(rhoDZ)@2         0.2987 0.020953  0.2576  0.3397  4.288e-46
```

A formal test of genetic effects can be obtained by comparing the MZ and DZ correlation:

``` r
estimate(lu,lava::contr(5:6,6))
#>                           Estimate Std.Err   2.5%  97.5%   P-value
#> [atanh(rhoMZ)@1] - [a....   0.4816 0.04177 0.3997 0.5635 9.431e-31
#> ────────────────────────────────────────────────────────────
#> Null Hypothesis: 
#>   [atanh(rhoMZ)@1] - [atanh(rhoDZ)@2] = 0 
#>  
#> chisq = 132.9162, df = 1, p-value < 2.2e-16
```

We also consider the ACE model

``` r
ace0 <- twinlm(bmi ~ age+gender, data=dd, DZ="DZ", zyg="zyg", id="tvparnr", type="ace")
summary(ace0)
#>                  Estimate Std. Error Z value Pr(>|z|)
#> bmi            1.8599e+01 2.5576e-01  72.720   <2e-16
#> sd(A)          2.7270e+00 4.2658e-02  63.927   <2e-16
#> sd(C)          1.7123e-06 3.1064e-01   0.000        1
#> sd(E)          2.0276e+00 3.4787e-02  58.286   <2e-16
#> bmi~age        1.1892e-01 5.6246e-03  21.142   <2e-16
#> bmi~gendermale 1.3846e+00 8.8748e-02  15.601   <2e-16
#> 
#> MZ-pairs DZ-pairs 
#>     1483     2788 
#> 
#> Variance decomposition:
#>   Estimate 2.5%    97.5%  
#> A 0.64399  0.61793 0.67005
#> C 0.00000  0.00000 0.00000
#> E 0.35601  0.32995 0.38207
#> 
#> 
#>                          Estimate 2.5%    97.5%  
#> Broad-sense heritability 0.64399  0.61793 0.67005
#> 
#>                        Estimate 2.5%    97.5%  
#> Correlation within MZ: 0.64399  0.61718 0.66931
#> Correlation within DZ: 0.32200  0.30890 0.33497
#> 
#> 'log Lik.' -22019.66 (df=6)
#> AIC: 44051.32 
#> BIC: 44089.47
```


# Bibliography
<a id="korkeila_bmi_1991"></a>[korkeila_bmi_1991] Korkeila, Kaprio, Rissanen & Koskenvuo, Effects of gender and age on the heritability of body mass index, <i>Int J Obes</i>, <b>15(10)</b>, 647-654 (1991). [↩](#b71edfd9bc946c317f4a732845bcaf93)

<a id="hjelmborg_bmi_2008"></a>[hjelmborg_bmi_2008] Hjelmborg, Fagnani, Silventoinen, McGue, Korkeila, Christensen, Rissanen & Kaprio, Genetic influences on growth traits of BMI: a longitudinal study of adult twins, <i>Obesity (Silver Spring)</i>, <b>16(4)</b>, 847-852 (2008). [↩](#718839fcb6ade82ebb2d7de853582b80)

---
title: "Mediation Analysis for survival data"
author: Klaus Holst & Thomas Scheike
date: "2026-05-24"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{Mediation Analysis for survival data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<!-- DO NOT EDIT: auto-generated from the .Rmd.orig source -->



Overview 
========

We fit

 * binomial regression with IPCW: `binreg`
 * additive Lin-Ying model: `aalenMets`
 * Cox model: `phreg`
 * standard logistic regression via `binreg`

in the context of mediation analysis using mediation weights as in the `medFlex` package.
We thus fit natural effects models; for example, on the binary scale:
\begin{align*}
\mbox{logit}(P(Y(x,M(x^*))=1| Z)  = \beta_0+ \beta_1 x + \beta_2 x^* + \beta_3^T Z,
\end{align*}
In this case the Natural Direct Effect (NDE) for fixed covariates $Z$ is
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NDE}} = \frac{\mbox{odds}(Y(1,M(x))|Z)}{\mbox{odds}(Y(0,M(x))|Z)}  = \exp(\beta_1),
\end{align*}
and the Natural Indirect Effect (NIE) for fixed covariates $Z$ is
\begin{align*}
 \mbox{OR}_{1,0|Z}^{\mbox{NIE}} = \frac{\mbox{odds}(Y(x,M(1))|Z)}{\mbox{odds}(Y(x,M(0))|Z)} = \exp(\beta_2).
\end{align*}
See the `medFlex` package for additional discussion of the parametrisation.

The mediator can be:

 * binomial, using `glm` with family `binomial`.
 * multinomial, via the `mlogit` function in `mets`.

Both mediator and exposure must be coded as factors.

In the example below:

 * mediator: `gp.f`
 * exposure: `dnr.f`

The outcome model concerns the risk/hazard of cause 2.

Standard errors are computed using i.i.d. influence functions and a Taylor expansion to account for uncertainty in the
mediation weights.


Simulated Data 
==============

First we simulate data that mimics the dataset from Kumar et al. (2012),
which consists of multiple myeloma patients treated with allogeneic stem cell
transplantation from the Center for International Blood and Marrow Transplant Research
(CIBMTR). The data cover patients transplanted from 1995 to
2005, comparing outcomes between transplant periods: 2001--2005 ($N=488$)
versus 1995--2000 ($N=375$). The two competing events were
relapse (cause 2) and treatment-related mortality (TRM, cause 1),
defined as death without relapse.
Kumar et al. (2012) considered the following risk covariates:
transplant period (`gp`, the main variable of interest: 1 for 2001--2005, 0 for 1995--2000),
donor type (`dnr`: 1 for unrelated or other related donor ($N=280$), 0 for HLA-identical sibling ($N=584$)),
prior autologous transplant (`preauto`: 1 for auto+allo transplant ($N=399$), 0 for allogeneic alone ($N=465$)),
and time to transplant (`ttt24`: 1 for more than 24 months ($N=289$), 0 for 24 months or less ($N=575$)).

The interest is in the effect of transplant period (`gp`) and possible mediation
via the proportion of unrelated or related donors (`dnr`) — a somewhat artificial example.
All analyses are adjusted for other important confounders.


``` r
 library(mets)
 runb <- 0
 options(warn=-1)
 set.seed(1000) # to control output in simulations for p-values below.

n <- 200; k.boot <- 10; 

dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
          beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
    treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
dfactor(dat) <- dnr.f~dnr
dfactor(dat) <- gp.f~gp
drename(dat) <- ttt24~"ttt24*"
dat$id <- 1:n
dat$ftime <- 1
```


Mediation Weights
=================

We compute the mediation weights based on a model for the mediator:


``` r
weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
wdata <- medweight(fit,data=dat)
```

Binomial Regression 
===================

A simple multivariate regression of the probability of relapse at 50 months with both
exposure and mediator (given the other covariates):


``` r
aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
summary(aaMss2)
#>    n events
#>  200     97
#> 
#>  200 clusters
#> coeffients:
#>             Estimate  Std.Err     2.5%    97.5% P-value
#> (Intercept) -1.01508  0.31869 -1.63971 -0.39046  0.0014
#> gp           1.08533  0.34216  0.41471  1.75594  0.0015
#> dnr          0.51969  0.35757 -0.18113  1.22051  0.1461
#> preauto      0.39417  0.35936 -0.31017  1.09851  0.2727
#> ttt24        0.50469  0.38681 -0.25344  1.26283  0.1920
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.36237 0.19404 0.6767
#> gp           2.96041 1.51394 5.7889
#> dnr          1.68151 0.83433 3.3889
#> preauto      1.48316 0.73332 2.9997
#> ttt24        1.65648 0.77612 3.5354
```

Binomial regression IPCW Mediation Analysis
===========================================

We first look at the probability of relapse at 50 months:


``` r
### binomial regression ###########################################################
aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cause=2)
summary(aaMss)
#>    n events
#>  400    194
#> 
#>  200 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.535534  0.256218 -1.037712 -0.033356  0.0366
#> dnr.f01      0.375817  0.348618 -0.307462  1.059095  0.2810
#> dnr.f11      0.275383  0.071199  0.135836  0.414931  0.0001
#> preauto      0.588221  0.350437 -0.098624  1.275066  0.0932
#> ttt24        0.266179  0.363603 -0.446469  0.978827  0.4641
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.58536 0.35426 0.9672
#> dnr.f01      1.45618 0.73531 2.8838
#> dnr.f11      1.31704 1.14549 1.5143
#> preauto      1.80078 0.90608 3.5789
#> ttt24        1.30497 0.63988 2.6613

ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    194
#> 
#>  200 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.535534  0.254832 -1.034995 -0.036073  0.0356
#> dnr.f01      0.375817  0.317732 -0.246927  0.998560  0.2369
#> dnr.f11      0.275383  0.117175  0.045726  0.505041  0.0188
#> preauto      0.588221  0.346523 -0.090951  1.267394  0.0896
#> ttt24        0.266179  0.366361 -0.451875  0.984233  0.4675
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.58536 0.35523 0.9646
#> dnr.f01      1.45618 0.78120 2.7144
#> dnr.f11      1.31704 1.04679 1.6571
#> preauto      1.80078 0.91306 3.5516
#> ttt24        1.30497 0.63643 2.6758
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```

The estimated NDE is $1.40$ (95% CI: $0.72$, $2.76$) and the NIE is $1.32$ (95% CI: $1.05$, $1.66$).

Mediation Analysis
====================

We illustrate how to use the other models mentioned above.


``` r
### lin-ying model ################################################################
aaMss <- aalenMets(Surv(time/100,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		   weights=wdata$weights)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    196
#> coeffients:
#>          Estimate   Std.Err      2.5%     97.5% P-value
#> dnr.f01  1.169592  0.739323 -0.279454  2.618637  0.1137
#> dnr.f11  0.206757  0.131289 -0.050565  0.464078  0.1153
#> preauto  0.617537  0.504302 -0.370877  1.605950  0.2207
#> ttt24    0.457736  0.517822 -0.557175  1.472648  0.3767
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### cox model ###############################################################################
aaMss <- phreg(Surv(time,status==2)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
	       weights=wdata$weights)
summary(aaMss)
#> 
#>    n events
#>  400    196
#> coefficients:
#>         Estimate     S.E.  dU^-1/2 P-value
#> dnr.f01 0.414565 0.213724 0.157231  0.0524
#> dnr.f11 0.100656 0.039308 0.144971  0.0104
#> preauto 0.284460 0.232166 0.162375  0.2205
#> ttt24   0.185561 0.226044 0.160886  0.4117
#> 
#> exp(coefficients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.51371 0.99568 2.3013
#> dnr.f11  1.10590 1.02389 1.1945
#> preauto  1.32904 0.84318 2.0949
#> ttt24    1.20389 0.77300 1.8750
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    196
#> coeffients:
#>            Estimate     Std.Err        2.5%       97.5% P-value
#> dnr.f01  0.41456472  0.20869639  0.00552731  0.82360212  0.0470
#> dnr.f11  0.10065575  0.05121458  0.00027702  0.20103448  0.0494
#> preauto  0.28445952  0.23037280 -0.16706288  0.73598192  0.2169
#> ttt24    0.18556110  0.22549763 -0.25640614  0.62752835  0.4106
#> 
#> exp(coeffients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.51371 1.00554 2.2787
#> dnr.f11  1.10590 1.00028 1.2227
#> preauto  1.32904 0.84615 2.0875
#> ttt24    1.20389 0.77383 1.8730
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### Fine-Gray #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,propodds=NULL,cause=2)
summary(aaMss)
#> 
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coefficients:
#>         Estimate    S.E. dU^-1/2 P-value
#> dnr.f01  0.18943 0.21986 0.15855  0.3889
#> dnr.f11  0.18730 0.04083 0.14503  0.0000
#> preauto  0.41452 0.22783 0.16098  0.0688
#> ttt24    0.17304 0.22892 0.16308  0.4497
#> 
#> exp(coefficients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.20856 0.78545 1.8596
#> dnr.f11  1.20599 1.11324 1.3065
#> preauto  1.51364 0.96849 2.3656
#> ttt24    1.18892 0.75910 1.8621
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coeffients:
#>          Estimate   Std.Err      2.5%     97.5% P-value
#> dnr.f01  0.189426  0.233939 -0.269087  0.647939  0.4181
#> dnr.f11  0.187298  0.047733  0.093744  0.280853  0.0001
#> preauto  0.414517  0.230676 -0.037600  0.866634  0.0723
#> ttt24    0.173042  0.230810 -0.279338  0.625422  0.4534
#> 
#> exp(coeffients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.20856 0.76408 1.9116
#> dnr.f11  1.20599 1.09828 1.3243
#> preauto  1.51364 0.96310 2.3789
#> ttt24    1.18892 0.75628 1.8690
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### logit model  #############################################################3
aaMss <- cifreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		weights=wdata$weights,cause=2)
summary(aaMss)
#> 
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coefficients:
#>         Estimate     S.E.  dU^-1/2 P-value
#> dnr.f01 0.357168 0.339848 0.158937  0.2933
#> dnr.f11 0.272392 0.064166 0.145076  0.0000
#> preauto 0.657010 0.326082 0.160361  0.0439
#> ttt24   0.191333 0.353606 0.167443  0.5884
#> 
#> exp(coefficients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.42928 0.73424 2.7822
#> dnr.f11  1.31310 1.15792 1.4891
#> preauto  1.92902 1.01806 3.6551
#> ttt24    1.21086 0.60549 2.4215
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coeffients:
#>          Estimate   Std.Err      2.5%     97.5% P-value
#> dnr.f01  0.357168  0.351089 -0.330953  1.045289  0.3090
#> dnr.f11  0.272392  0.068131  0.138857  0.405927  0.0001
#> preauto  0.657010  0.328207  0.013736  1.300284  0.0453
#> ttt24    0.191333  0.356086 -0.506583  0.889250  0.5910
#> 
#> exp(coeffients):
#>         Estimate    2.5%  97.5%
#> dnr.f01  1.42928 0.71824 2.8442
#> dnr.f11  1.31310 1.14896 1.5007
#> preauto  1.92902 1.01383 3.6703
#> ttt24    1.21086 0.60255 2.4333
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}

### binomial outcome  ############################
aaMss <- binreg(Event(ftime,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),data=wdata,
		time=50,weights=wdata$weights,cens.weights=1,cause=2)
summary(aaMss)
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.674433  0.235285 -1.135583 -0.213284  0.0042
#> dnr.f01      0.221834  0.318264 -0.401952  0.845620  0.4858
#> dnr.f11      0.262722  0.060281  0.144572  0.380871  0.0000
#> preauto      0.578077  0.319091 -0.047331  1.203484  0.0700
#> ttt24        0.214442  0.328183 -0.428784  0.857669  0.5135
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.50944 0.32123 0.8079
#> dnr.f01      1.24836 0.66901 2.3294
#> dnr.f11      1.30046 1.15555 1.4636
#> preauto      1.78261 0.95377 3.3317
#> ttt24        1.23917 0.65130 2.3577
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.674433  0.235022 -1.135069 -0.213798  0.0041
#> dnr.f01      0.221834  0.286717 -0.340122  0.783789  0.4391
#> dnr.f11      0.262722  0.107508  0.052011  0.473432  0.0145
#> preauto      0.578077  0.315260 -0.039822  1.195975  0.0667
#> ttt24        0.214442  0.329107 -0.430596  0.859480  0.5147
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.50944 0.32140 0.8075
#> dnr.f01      1.24836 0.71168 2.1898
#> dnr.f11      1.30046 1.05339 1.6055
#> preauto      1.78261 0.96096 3.3068
#> ttt24        1.23917 0.65012 2.3619
if (runb>0) { bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=k.boot); summary(bll)}
```


Multinomial regression
======================

Also works with a mediator with more than two levels:

 * mediator: `wmi` in 4 categories
 * exposure: `age` in 4 categories


``` r
library(mets)
data(tTRACE)
dcut(tTRACE) <- ~. 

weightmodel <- fit <- mlogit(wmicat.4 ~agecat.4+vf+chf,data=tTRACE,family=binomial)
#> Error in `str2lang()`:
#> ! <text>:1:68: unexpected ','
#> 1: Surv(time,status)~strata(idrow)+cluster(id) + Intercept2+agecat4601,
#>                                                                        ^
wdata <- medweight(fit,data=tTRACE)
#> Error in `[.data.frame`:
#> ! undefined columns selected

aaMss <- binreg(Event(time,status)~agecat.40+ agecat.41+ vf+chf+cluster(id),data=wdata,
		time=7,weights=wdata$weights,cause=9)
#> Error:
#> ! object 'agecat.40' not found
summary(aaMss)
#>    n events
#>  400    196
#> 
#>  200 clusters
#> coeffients:
#>              Estimate   Std.Err      2.5%     97.5% P-value
#> (Intercept) -0.674433  0.235285 -1.135583 -0.213284  0.0042
#> dnr.f01      0.221834  0.318264 -0.401952  0.845620  0.4858
#> dnr.f11      0.262722  0.060281  0.144572  0.380871  0.0000
#> preauto      0.578077  0.319091 -0.047331  1.203484  0.0700
#> ttt24        0.214442  0.328183 -0.428784  0.857669  0.5135
#> 
#> exp(coeffients):
#>             Estimate    2.5%  97.5%
#> (Intercept)  0.50944 0.32123 0.8079
#> dnr.f01      1.24836 0.66901 2.3294
#> dnr.f11      1.30046 1.15555 1.4636
#> preauto      1.78261 0.95377 3.3317
#> ttt24        1.23917 0.65130 2.3577
MultMed <- mediatorSurv(aaMss,fit,data=tTRACE,wdata=wdata)
#> Error:
#> ! Mat::row(): index out of bounds
summary(MultMed)
#> Error:
#> ! object 'MultMed' not found
```


```
#> Error:
#> ! object 'MultMed' not found
```


``` r
summary(MultMed)
#> Error:
#> ! object 'MultMed' not found
```

SessionInfo
============



``` r
sessionInfo()
#> R version 4.6.0 (2026-04-24)
#> Platform: x86_64-pc-linux-gnu
#> Running under: Ubuntu 24.04.4 LTS
#> 
#> Matrix products: default
#> BLAS:   /home/kkzh/.asdf/installs/r/4.6.0/lib/R/lib/libRblas.so 
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.12.0  LAPACK version 3.12.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
#>  [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
#>  [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
#>  [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
#> [11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       
#> 
#> time zone: Europe/Copenhagen
#> tzcode source: system (glibc)
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] timereg_2.0.7  survival_3.8-6 mets_1.3.10   
#> 
#> loaded via a namespace (and not attached):
#>  [1] vctrs_0.7.3            cli_3.6.6              knitr_1.51            
#>  [4] rlang_1.2.0            xfun_0.57              otel_0.2.0            
#>  [7] glue_1.8.1             future.apply_1.20.2    listenv_0.10.1        
#> [10] lava_1.9.1             stats4_4.6.0           grid_4.6.0            
#> [13] evaluate_1.0.5         lifecycle_1.0.5        yaml_2.3.12           
#> [16] mvtnorm_1.3-7          numDeriv_2016.8-1.1    compiler_4.6.0        
#> [19] codetools_0.2-20       Rcpp_1.1.1-1.1         ucminf_1.2.3          
#> [22] future_1.70.0          lattice_0.22-9         digest_0.6.39         
#> [25] pillar_1.11.1          parallelly_1.47.0      parallel_4.6.0        
#> [28] splines_4.6.0          Matrix_1.7-5           tools_4.6.0           
#> [31] RcppArmadillo_15.2.6-1 globals_0.19.1
```

---
title: "Binomial Regression for Survival and Competing Risks Data"
author: Klaus Holst & Thomas Scheike
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7.15
    fig_height: 5.5 
vignette: >
  %\VignetteIndexEntry{Binomial Regression for Survival and Competing Risks Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(mets)
```

Binomial Regression for censored data 
=====================================

The binreg function can fit a logistic link model with IPCW adjustment for a specific time-point, 
and can thus be used for describing survival or competing risks data.
The function can be used for large data and is completely scalable, that is, linear in the data. 
A nice feature is that influcence functions are computed and are available, and can thus be
used for all other settings based on these parameters. 

In addition and to summarize

  - the censoring weights can be strata dependent
  - predictions can be computed with standard errors 
  - computation time linear in data
     - including standard errors
  - clusters can be given and then cluster corrected standard errors are computed


Details
=======

The binreg function does direct binomial regression for one time-point, $t$, fitting the model 
\begin{align*}
P(T \leq t, \epsilon=1 | X )  & = \mbox{expit}( X^T \beta)  = F_1(t,X,\beta)
\end{align*}
to an IPCW adjusted estmating equation (EE) with response  $Y(t)=I(T \leq t, \epsilon=1 )$
\begin{align*}
 U(\beta,\hat G_c) = &  X ( Y(t) \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)} - \mbox{expit}( X^T \beta)) = 0,
\end{align*}
with $G_c(t)=P(C>t)$, the  censoring survival distribution, and  with $\Delta(t) = I( C_i > T_i \wedge t)$
the indicator of being uncensored at time $t$ (type="I"). 
The default type="II" is to augment with a censoring term, that is solve 
\begin{align*}
  &  U(\beta,\hat G_c) + \int_0^t X \frac{\hat E(Y(t)| T>u)}{\hat G_c(u)} d\hat M_c(u) =0 
\end{align*}
where $M_c(u)$ is the censoring martingale, this typically improves the performance. This is equivlent to the 
pseudo-value approach (see Overgaard (2025)).

The influence function for the type="II" estimator is 
\begin{align*}
    U(\beta,G_c) + \int_0^t X \frac{E(Y| T>u)}{G_c(u)} d M_c(u)  
    - \int_0^t  \frac{E(X| T>u) E(Y| T>u)}{G_c(u)} d M_c(u) - \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u)  
\end{align*}
and for type="I"
\begin{align*}
  &  U(\beta) + \int_0^t \frac{E( X Y| T>u)}{G_c(u)} d M_c(u).
\end{align*}
The means $E(X Y(t) | T>u)$ and $E(Y(t)| T>u)$ are estimated by IPCW estimators among survivors to get 
estimates of the influence functions.

The function logitIPCW instead considers 
\begin{align*}
 U^{glm}(\beta,\hat G_c) = & \frac{ \Delta(t) }{\hat G_c(T_i \wedge t)}  X  ( Y(t) - \mbox{expit}( X^T \beta)) = 0.
\end{align*}
This score equation is quite similar to those of the binreg, and exactly the same when the censoring model is fully-nonparametric.

The logitIPCW has influence function 
\begin{align*}
  &  U^{glm}(\beta,G_c) + \int_0^t \frac{E( X ( Y - F_1(t,\beta)) | T>u)}{G_c(u)} d M_c(u)  
\end{align*}

Which estimator performs the best depends on the censoring distribution and it seems that the binreg with type="II" performs 
overall quite nicely (see Blanche et al (2023) and Overgaard (2024)).  For the full estimated censoring model all estimators have the
same influence function (see Blanche et al (2023)).

Additional functions logitATE, and binregATE computes the average treatment effect. 
We demonstrate this in another vignette. 

The functions logitATE/binregATE can be used there is no censoring and we thus have simple binary outcome. 

The variance is based on sandwich formula with IPCW adjustment (using the influence functions), 
and naive.var is the variance under known censoring model. The influence functions are stored in the output. 
Clusters can be specified to get cluster corrected standard errors. 

Examples
========


```{r}
 library(mets)
 options(warn=-1)
 set.seed(1000) # to control output in random noise just below.
 data(bmt)
 bmt$time <- bmt$time+runif(nrow(bmt))*0.01
 bmt$id <- 1:408

 # logistic regresion with IPCW binomial regression 
 out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
 summary(out)
```

We can also compute predictions using the estimates 
```{r}
 predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)
```

Further the censoring model can depend on strata 

```{r}
 outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
 summary(outs)
```

Absolute risk differences and ratio
===================================

Now for illustrations I wish to consider the absolute risk difference
depending on tcell 

```{r}
 outs <- binreg(Event(time,cause)~tcell,bmt,time=50,cens.model=~strata(tcell))
 summary(outs)
```

the risk difference is 

```{r}
ps <-  predict(outs,data.frame(tcell=c(0,1)),se=TRUE)
ps
sum( c(1,-1) * ps[,1])
```

Getting the standard errors are easy enough since the two-groups are
independent. In the case where we in addition had adjusted for other covariates, however, we would need 
the to apply the delta-theorem thus using the relevant covariances along the lines of 

```{r}
dd <- data.frame(tcell=c(0,1))
p <- predict(outs,dd)

riskdifratio <- function(p,contrast=c(1,-1)) {
   outs$coef <- p
   p <- predict(outs,dd)[,1]
   pd <- sum(contrast*p)
   r1 <- p[1]/p[2]
   r2 <- p[2]/p[1]
   return(c(pd,r1,r2))
}
     
estimate(outs,f=riskdifratio,dd,null=c(0,1,1))
```

same as 

```{r}
run <- 0
if (run==1) {
library(prodlim)
pl <- prodlim(Hist(time,cause)~tcell,bmt)
spl <- summary(pl,times=50,asMatrix=TRUE)
spl
}
```


Augmenting the Binomial Regression 
===================================

Rather than using a larger censoring model we can also compute an  augmentation term and
then fit the binomial regression model based on this augmentation term. 
Here we compute the augmentation based on stratified non-parametric estimates of $F_1(t,S(X))$, 
where $S(X)$ gives strata based on $X$ as a working model. 

 Computes  the augmentation term for each individual as well as the sum
\begin{align*}
 A & = \int_0^t H(u,X) \frac{1}{S^*(u,s)} \frac{1}{G_c(u)} dM_c(u)
\end{align*}
 with 
\begin{align*}
 H(u,X) & = F_1^*(t,S(X)) - F_1^*(u,S(X))
\end{align*}
using a KM for $G_c(t)$ and a working model for cumulative baseline
related to $F_1^*(t,s)$ and $s$ is strata, $S^*(t,s) = 1 - F_1^*(t,s) - F_2^*(t,s)$. 

Standard errors computed under assumption of correct but estimated $G_c(s)$ model.

```{r}
 data(bmt)
 dcut(bmt,breaks=2) <- ~age 
 out1<-BinAugmentCifstrata(Event(time,cause)~platelet+agecat.2+
			  strata(platelet,agecat.2),data=bmt,cause=1,time=40)
 summary(out1)

 out2<-BinAugmentCifstrata(Event(time,cause)~platelet+agecat.2+
     strata(platelet,agecat.2)+strataC(platelet),data=bmt,cause=1,time=40)
 summary(out2)
```


SessionInfo
============

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{cor.cif}
\alias{cor.cif}
\alias{or.cif}
\alias{rr.cif}
\title{Cross-odds-ratio, OR or RR risk regression for competing risks}
\usage{
cor.cif(
  cif,
  data,
  cause = NULL,
  times = NULL,
  cause1 = 1,
  cause2 = 1,
  cens.code = NULL,
  cens.model = "KM",
  Nit = 40,
  detail = 0,
  clusters = NULL,
  theta = NULL,
  theta.des = NULL,
  step = 1,
  sym = 0,
  weights = NULL,
  par.func = NULL,
  dpar.func = NULL,
  dimpar = NULL,
  score.method = "nlminb",
  same.cens = FALSE,
  censoring.weights = NULL,
  silent = 1,
  ...
)
}
\arguments{
\item{cif}{a model object from the timereg::comp.risk function with the
marginal cumulative incidence of cause1, i.e., the event of interest, and whose
odds the comparision is compared to the conditional odds given cause2}

\item{data}{a data.frame with the variables.}

\item{cause}{specifies the causes  related to the death
times, the value cens.code is the censoring value. When missing it comes from marginal cif.}

\item{times}{time-vector that specifies the times used for the estimating euqations for the cross-odds-ratio estimation.}

\item{cause1}{specificies the cause considered.}

\item{cause2}{specificies the cause that is conditioned on.}

\item{cens.code}{specificies the code for the censoring if NULL then uses the one from the marginal cif model.}

\item{cens.model}{specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"}

\item{Nit}{number of iterations for Newton-Raphson algorithm.}

\item{detail}{if 0 no details are printed during iterations, if 1 details are given.}

\item{clusters}{specifies the cluster structure.}

\item{theta}{specifies starting values for the cross-odds-ratio parameters of the model.}

\item{theta.des}{specifies a regression design for the cross-odds-ratio parameters.}

\item{step}{specifies the step size for the Newton-Raphson algorithm.}

\item{sym}{specifies if symmetry is used in the model.}

\item{weights}{weights for estimating equations.}

\item{par.func}{parfunc}

\item{dpar.func}{dparfunc}

\item{dimpar}{dimpar}

\item{score.method}{"nlminb", can also use "nr".}

\item{same.cens}{if true then censoring within clusters are assumed to be the same variable, default is independent censoring.}

\item{censoring.weights}{these probabilities are used for the bivariate censoring dist.}

\item{silent}{1 to suppress output about convergence related issues.}

\item{...}{Not used.}
}
\value{
returns an object of type 'cor'. With the following arguments:
\item{theta}{estimate of proportional odds parameters of model.}
\item{var.theta}{variance for gamma.  }
\item{hess}{the derivative of the used score.}
\item{score}{scores at final stage.}
\item{score}{scores at final stage.}
\item{theta.iid}{matrix of iid decomposition of parametric effects.}
}
\description{
Fits a parametric model for the log-cross-odds-ratio for the 
predictive effect of for the cumulative incidence curves for \eqn{T_1} 
experiencing cause i given that \eqn{T_2} has experienced a cause k :
\deqn{
\log(COR(i|k))  =  h(\theta,z_1,i,z_2,k,t)=_{default}  \theta^T z =  
}
with the log cross odds ratio being 
\deqn{
COR(i|k) = 
\frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
O(T_1 \leq t,cause_1=i)}  
}
the conditional odds divided by the unconditional odds, with the odds
being, respectively 
\deqn{
O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_1=k) = 
\frac{
P_x(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
P_x((T_1 \leq t,cause_1=i)^c | T_2 \leq t,cause_2=k)}
}
and 
\deqn{
O(T_1 \leq t,cause_1=i) = 
\frac{P_x(T_1 \leq t,cause_1=i )}{P_x((T_1 \leq t,cause_1=i)^c )}.
}
Here \eqn{B^c} is the complement event of \eqn{B},
\eqn{P_x} is the distribution given covariates 
(\eqn{x} are subject specific and \eqn{z} are cluster specific covariates), and 
\eqn{h()} is a function that is the simple identity 
\eqn{\theta^T z} by default.
}
\details{
The OR dependence measure is given by 
\deqn{
OR(i,k) = 
\log ( 
\frac{O(T_1 \leq t,cause_1=i | T_2 \leq t,cause_2=k)}{
O(T_1 \leq t,cause_1=i) | T_2 \leq t,cause_2=k)}  
}
This measure is numerically more stabile than the COR measure, and is symetric in i,k.

The RR dependence measure is given by 
\deqn{
RR(i,k) = 
\log ( 
\frac{P(T_1 \leq t,cause_1=i , T_2 \leq t,cause_2=k)}{
P(T_1 \leq t,cause_1=i) P(T_2 \leq t,cause_2=k)}  
}
This measure is numerically more stabile than the COR measure, and is symetric in i,k.

The model is fitted under symmetry (sym=1), i.e., such that it is assumed 
that \eqn{T_1} and \eqn{T_2} can be interchanged and leads to
the same cross-odd-ratio (i.e.
\eqn{COR(i|k) = COR(k|i))}, 
as would be expected for twins 
or without symmetry as might be the case with mothers and daughters (sym=0). 

\eqn{h()} may be specified as an R-function of the parameters, 
see example below, but the default is that it is simply \eqn{\theta^T z}.
}
\examples{
library("timereg")
data(multcif);
multcif$cause[multcif$cause==0] <- 2
zyg <- rep(rbinom(200,1,0.5),each=2)
theta.des <- model.matrix(~-1+factor(zyg))

times=seq(0.05,1,by=0.05) # to speed up computations use only these time-points
add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=1,
               n.sim=0,times=times,model="fg",max.clust=NULL)
add2 <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=multcif,cause=2,
               n.sim=0,times=times,model="fg",max.clust=NULL)

out1 <- cor.cif(add,data=multcif,cause1=1,cause2=1)
summary(out1)

out2 <- cor.cif(add,data=multcif,cause1=1,cause2=1,theta.des=theta.des)
summary(out2)

##out3 <- cor.cif(add,data=multcif,cause1=1,cause2=2,cif2=add2)
##summary(out3)
###########################################################
# investigating further models using parfunc and dparfunc
###########################################################
\donttest{ ## Reduce Ex.Timings
set.seed(100)
prt<-simnordic.random(2000,cordz=2,cormz=5)
prt$status <-prt$cause
table(prt$status)

times <- seq(40,100,by=10)
cifmod <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),data=prt,
                    cause=1,n.sim=0,
                    times=times,conservative=1,max.clust=NULL,model="fg")
theta.des <- model.matrix(~-1+factor(zyg),data=prt)

parfunc <- function(par,t,pardes)
{
par <- pardes \%*\% c(par[1],par[2]) +
       pardes \%*\% c( par[3]*(t-60)/12,par[4]*(t-60)/12)
par
}
head(parfunc(c(0.1,1,0.1,1),50,theta.des))

dparfunc <- function(par,t,pardes)
{
dpar <- cbind(pardes, t(t(pardes) * c( (t-60)/12,(t-60)/12)) )
dpar
}
head(dparfunc(c(0.1,1,0.1,1),50,theta.des))

names(prt)
or1 <- or.cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
              same.cens=TRUE,theta=c(0.6,1.1,0.1,0.1),
              par.func=parfunc,dpar.func=dparfunc,dimpar=4,
              score.method="nr",detail=1)
summary(or1)

 cor1 <- cor.cif(cifmod,data=prt,cause1=1,cause2=1,theta.des=theta.des,
                 same.cens=TRUE,theta=c(0.5,1.0,0.1,0.1),
                 par.func=parfunc,dpar.func=dparfunc,dimpar=4,
                 control=list(trace=TRUE),detail=1)
summary(cor1)

### piecewise contant OR model
gparfunc <- function(par,t,pardes)
{
	cuts <- c(0,80,90,120)
	grop <- diff(t<cuts)
paru  <- (pardes[,1]==1) * sum(grop*par[1:3]) +
    (pardes[,2]==1) * sum(grop*par[4:6])
paru
}

dgparfunc <- function(par,t,pardes)
{
	cuts <- c(0,80,90,120)
	grop <- diff(t<cuts)
par1 <- matrix(c(grop),nrow(pardes),length(grop),byrow=TRUE)
parmz <- par1* (pardes[,1]==1)
pardz <- (pardes[,2]==1) * par1
dpar <- cbind( parmz,pardz)
dpar
}
head(dgparfunc(rep(0.1,6),50,theta.des))
head(gparfunc(rep(0.1,6),50,theta.des))

or1g <- or.cif(cifmod,data=prt,cause1=1,cause2=1,
               theta.des=theta.des, same.cens=TRUE,
               par.func=gparfunc,dpar.func=dgparfunc,
               dimpar=6,score.method="nr",detail=1)
summary(or1g)
names(or1g)
head(or1g$theta.iid)
}
}
\references{
Cross odds ratio Modelling of dependence for
Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 

A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
Scheike, Zhang, Sun, Jensen (2010), Biometrika.
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/divide.conquer.R
\name{folds}
\alias{folds}
\title{Generate Random Fold Indices for Cross-Validation}
\usage{
folds(n, folds = 10)
}
\arguments{
\item{n}{number of observations.}

\item{folds}{number of folds (default 10).}
}
\value{
A list of integer vectors, each containing indices for one fold.
}
\description{
Splits \code{n} observations into random folds for cross-validation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{binregCasewise}
\alias{binregCasewise}
\title{Estimate Casewise Concordance Using Binomial Regression}
\usage{
binregCasewise(concbreg, margbreg, zygs = c("DZ", "MZ"), newdata = NULL, ...)
}
\arguments{
\item{concbreg}{Concordance object from \code{binreg}.}

\item{margbreg}{Marginal estimate object from \code{binreg}.}

\item{zygs}{Order of zygosity for estimation (e.g., \code{c("DZ","MZ")}).}

\item{newdata}{Data frame to give instead of \code{zygs}.}

\item{...}{Arguments passed to \code{estimate}.}
}
\value{
A list containing:
  \item{coef}{Exponentiated coefficients (ratios).}
  \item{logcoef}{Log-scale coefficients and standard errors.}
}
\description{
Estimates the casewise concordance based on concordance and marginal estimates 
obtained from \code{binreg} objects. Uses cluster-based IID for standard errors, 
which are often better than those from \code{casewise} (which can be conservative).
}
\examples{
data(prt)
prt <- force_same_cens(prt,cause="status")

dd <- bicompriskData(Event(time, status)~strata(zyg)+id(id), data=prt, cause=c(2, 2))
newdata <- data.frame(zyg=c("DZ","MZ"),id=1)

## concordance 
bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id), data=dd,
                time=80, cause=1, cens.model=~strata(zyg))
pconc <- predict(bcif1,newdata)

## marginal estimates 
mbcif1 <- binreg(Event(time,status)~cluster(id), data=prt, time=80, cause=2)
mc <- predict(mbcif1,newdata)

cse <- binregCasewise(bcif1,mbcif1)
cse
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score-mean-tests.R
\name{test_marginalMean}
\alias{test_marginalMean}
\title{Pepe-Mori Test for Marginal Mean Comparison}
\usage{
test_marginalMean(
  formula,
  data,
  cause = 1,
  cens.code = 0,
  ...,
  death.code = 2,
  death.code.prop = NULL,
  time = NULL,
  beta = NULL
)
}
\arguments{
\item{formula}{Formula with an \code{Event} object on the left-hand side and 
covariates (typically with \code{strata()} for group comparison) on the right. 
Can include \code{cluster(id)} for correlated data.}

\item{data}{Data frame containing all variables referenced in the formula.}

\item{cause}{Cause of interest (default 1).}

\item{cens.code}{Censoring code (default 0).}

\item{...}{Additional arguments passed to lower-level functions.}

\item{death.code}{Code for death (terminating event, default 2).}

\item{death.code.prop}{Code for other causes of death for Fine-Gray regression model.}

\item{time}{Upper limit for Pepe-Mori and AUC integrals. If NULL, defaults to 
the maximum event time for the cause of interest.}

\item{beta}{Starting values for the score test (default NULL, uses zeros).}
}
\value{
An object of class \code{"marginalTest"} containing:
  \item{pepe.mori}{Pepe-Mori test results with compare p-value.}
  \item{RatioAUC}{Ratio of AUC test results with compare p-value.}
  \item{difAUC}{Difference of AUC test results with compare p-value.}
  \item{prop.test}{Proportionality test results.}
  \item{score.test}{Score test results (equivalent to Gray's test).}
  \item{score.iid}{Influence function for the score test.}
  \item{time}{Upper time limit used.}
  \item{RAUCl, RAUCe}{Raw and transformed AUC estimates.}
}
\description{
Performs score-test type tests for proportionality of marginal means in competing 
risks data and recurrent events, as presented in Ghosh and Lin (2000). The test 
is based on an IPCW (Inverse Probability of Censoring Weighting) formulation.
}
\details{
The function computes several tests:
\enumerate{
  \item \strong{Pepe-Mori Test}: Tests for equality of marginal mean functions between groups.
  \item \strong{Ratio of AUC}: Compares the area under the curve of marginal means.
  \item \strong{Difference of AUC}: Tests for difference in areas under the curve.
  \item \strong{Score Test}: Tests for proportionality (equivalent to Gray's test for CIF).
  \item \strong{Proportionality Test}: Tests the proportional hazards assumption.
}

The Pepe-Mori test uses weights based on the number at risk in each group to 
construct a weighted integral of the difference in marginal means.
}
\examples{
data(bmt,package="mets")
bmt$time <- bmt$time+runif(nrow(bmt))*0.01
bmt$id <- 1:nrow(bmt)
dcut(bmt) <- age.f~age
     
fg=cifregFG(Event(time,cause)~tcell,data=bmt,cause=1)
 
## computing tests for difference  for CIF
pmt <- test_marginalMean(Event(time,cause)~strata(tcell)+cluster(id),data=bmt,cause=1,
			 death.code=1:2,death.code.prop=2,cens.code=0,time=40)
summary(pmt) 
 
pmt$pepe.mori
pmt$RatioAUC
pmt$prop.test
## score test equialent to Gray's test but variance estimated differently 
pmt$score.test
 
### age-groups  
pmt <- test_marginalMean(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1,
			 death.code=1:2,death.code.prop=2,cens.code=0)
summary(pmt) 
 
## having a look at the cumulative incidences 
cifs <- cif(Event(time,cause)~strata(age.f)+cluster(id),data=bmt,cause=1)
plot(cifs) 
 
## recurrent events   
data(hfactioncpx12)
hf <- hfactioncpx12
pmt <- test_marginalMean(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
			 cause=1,death.code=2,cens.code=0)
summary(pmt) 
 
}
\references{
Ghosh, D. and Lin, D. Y. (2000). Nonparametric Analysis of Recurrent Events and Death. Biometrics, 56, 554--562.
}
\seealso{
\code{\link{test_logrankRecurrent}}, \code{\link{test_conc}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{simRecurrentTS}
\alias{simRecurrentTS}
\title{Simulation of recurrent events data based on cumulative hazards: Two-stage model}
\usage{
simRecurrentTS(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz = NULL,
  nu = rep(1, 3),
  share1 = 0.3,
  vargamD = 2,
  vargam12 = 0.5,
  gap.time = FALSE,
  max.recurrent = 100,
  cens = NULL,
  ...
)
}
\arguments{
\item{n}{number of id's}

\item{cumhaz}{cumulative hazard of recurrent events}

\item{cumhaz2}{cumulative hazard of recurrent events  of type 2}

\item{death.cumhaz}{cumulative hazard of death}

\item{nu}{powers of random effects where nu > -1/shape}

\item{share1}{how random effect for death splits into two parts}

\item{vargamD}{variance of random effect  for death}

\item{vargam12}{shared random effect for N1 and N2}

\item{gap.time}{if true simulates gap-times with specified cumulative hazard}

\item{max.recurrent}{limits number recurrent events to 100}

\item{cens}{rate of censoring exponential distribution}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Simulation of recurrent events data based on cumulative hazards
}
\details{
Model is constructed such that marginals are on specified form by linear approximations
of cumulative hazards that are on a specific form to make them equivalent to marginals
after integrating out over survivors. Therefore E(dN_1 | D>t) = cumhaz, 
E(dN_2 | D>t) = cumhaz2,  and hazard of death is death.cumhazard 

Must give hazard of death and two recurrent events.  Hazard of death is death.cumhazard  two
event types and their dependence can be specified but the two recurrent events need
to share random effect. 

Random effect for  death Z.death=(Zd1+Zd2), Z1=(Zd1^nu1) Z12,  Z2=(Zd2^nu2) Z12^nu3
\deqn{Z.death=Zd1+Zd2}  gamma distributions 
\deqn{Zdj}  gamma distribution  with mean parameters (sharej), vargamD,  share2=1-share1
\deqn{Z12}  gamma distribution with mean 1 and variance vargam12
}
\examples{
########################################
## getting some rates to mimick 
########################################

data(base1cumhaz)
data(base4cumhaz)
data(drcumhaz)
dr <- drcumhaz
base1 <- base1cumhaz
base4 <- base4cumhaz

rr <- simRecurrentTS(1000,base1,base4,death.cumhaz=dr)
dtable(rr,~death+status)
showfitsim(causes=2,rr,dr,base1,base4)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{recurrentMarginal}
\alias{recurrentMarginal}
\alias{tie.breaker}
\alias{recmarg}
\alias{recurrentMarginalAIPCW}
\alias{recurrentMarginalPhreg}
\title{Fast recurrent marginal mean when death is possible}
\usage{
recurrentMarginal(formula, data, cause = 1, ..., death.code = 2)
}
\arguments{
\item{formula}{with Event object}

\item{data}{data frame for computation}

\item{cause}{of interest (1 default)}

\item{...}{Additional arguments to lower level funtions}

\item{death.code}{codes for death (terminating event, 2 default)}
}
\description{
Fast Marginal means of recurrent events using the Lin and Ghosh (2000) 
standard errors.  
Fitting two models for death and recurent events these are
combined to prducte the estimator 
\deqn{ \int_0^t  S(u|x=0) dR(u|x=0) } the mean number of recurrent events, here
\deqn{ S(u|x=0) }  is the probability of survival for the baseline group, and 
\deqn{ dR(u|x=0) }  is the hazard rate of an event among survivors for the baseline. 
Here \deqn{ S(u|x=0) }  is estimated by \deqn{ exp(-\Lambda_d(u|x=0) }  with 
 \deqn{\Lambda_d(u|x=0) } being the cumulative baseline for death.
}
\details{
Assumes no ties in the sense that jump times needs to be unique, this is particularly so for the stratified version.
}
\examples{
library(mets)
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment) 

##  to fit non-parametric models with just a baseline 
xr <- phreg(Surv(entry,time,status==1)~cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~cluster(id),data=hf)
par(mfrow=c(1,3))
plot(dr,se=TRUE)
title(main="death")
plot(xr,se=TRUE)
### robust standard errors 
rxr <-  robust.phreg(xr,fixbeta=1)
plot(rxr,se=TRUE,robust=TRUE,add=TRUE,col=4)

## marginal mean of expected number of recurrent events 
## out <- recurrentMarginalPhreg(xr,dr)
## summary(out,times=1:5) 

## marginal mean using formula  
outN <- recurrentMarginal(Event(entry,time,status)~cluster(id),hf,cause=1,death.code=2)
plot(outN,se=TRUE,col=2,add=TRUE)
summary(outN,times=1:5) 

########################################################################
###   with strata     ##################################################
########################################################################
out <- recurrentMarginal(Event(entry,time,status)~strata(treatment)+cluster(id),
                         data=hf,cause=1,death.code=2)
plot(out,se=TRUE,ylab="marginal mean",col=1:2)

summary(out,times=1:5) 

}
\references{
Cook, R. J. and Lawless, J. F. (1997) Marginal analysis of recurrent events and a terminating event. Statist. Med., 16, 911–924.
Ghosh and Lin (2002) Nonparametric Analysis of Recurrent events and death, Biometrics, 554--562.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-survival-haplo.R
\name{haplo.surv.discrete}
\alias{haplo.surv.discrete}
\alias{simTTP}
\alias{predictSurvd}
\alias{plotSurvd}
\title{Discrete time to event haplo type analysis}
\usage{
haplo.surv.discrete(
  X = NULL,
  y = "y",
  time.name = "time",
  Haplos = NULL,
  id = "id",
  desnames = NULL,
  designfunc = NULL,
  beta = NULL,
  no.opt = FALSE,
  method = "NR",
  stderr = TRUE,
  designMatrix = NULL,
  response = NULL,
  idhap = NULL,
  design.only = FALSE,
  covnames = NULL,
  fam = binomial,
  weights = NULL,
  offsets = NULL,
  idhapweights = NULL,
  ...
)
}
\arguments{
\item{X}{design matrix data-frame (sorted after id and time variable) with id time response  and desnames}

\item{y}{name of response (binary response with logistic link) from X}

\item{time.name}{to sort after time  for X}

\item{Haplos}{(data.frame with id, haplo1, haplo2 (haplotypes (h)) and  p=P(h|G)) haplotypes given as factor.}

\item{id}{name of id variale from X}

\item{desnames}{names for design matrix}

\item{designfunc}{function that computes design given haplotypes h=(h1,h2) x(h)}

\item{beta}{starting values}

\item{no.opt}{optimization TRUE/FALSE}

\item{method}{NR, nlm}

\item{stderr}{to return only estimate}

\item{designMatrix}{gives response and designMatrix directly not implemented (mush contain: p, id, idhap)}

\item{response}{gives response and design directly designMatrix not implemented}

\item{idhap}{name of id-hap variable to specify different haplotypes for different id}

\item{design.only}{to return only design matrices for haplo-type analyses.}

\item{covnames}{names of covariates to extract from object for regression}

\item{fam}{family of models, now binomial default and only option}

\item{weights}{weights following id for GLM}

\item{offsets}{following id  for GLM}

\item{idhapweights}{weights following id-hap for GLM (WIP)}

\item{...}{Additional arguments to lower level funtions lava::NR  optimizer or nlm}
}
\description{
Can be used for logistic regression when time variable is "1" for all id.
}
\details{
Cycle-specific logistic regression of haplo-type effects with known 
haplo-type probabilities. Given observed genotype G and unobserved haplotypes H
we here mix out over the possible haplotypes using that P(H|G) is provided. 

\deqn{
S(t|x,G)) = E( S(t|x,H) | G)  = \sum_{h \in G} P(h|G) S(t|z,h) 
}
so survival can be computed by mixing out over possible h given g.

Survival is based on logistic regression for the discrete hazard function of the
form 
\deqn{
logit(P(T=t| T \geq t, x,h)) = \alpha_t + x(h) \beta
}
where x(h) is a regression design of x and haplotypes \eqn{h=(h_1,h_2)}

Likelihood is maximized and standard errors assumes that P(H|G) is known. 

The design over the possible haplotypes is constructed by merging X with Haplos and  
can be viewed by design.only=TRUE
}
\examples{
## some haplotypes of interest
types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")

## some haplotypes frequencies for simulations 
data(hapfreqs)

www <-which(hapfreqs$haplotype \%in\% types)
hapfreqs$freq[www]

baseline=hapfreqs$haplotype[9]
baseline

designftypes <- function(x,sm=0) {# {{{
hap1=x[1]
hap2=x[2]
if (sm==0) y <- 1*( (hap1==types) | (hap2==types))
if (sm==1) y <- 1*(hap1==types) + 1*(hap2==types)
return(y)
}# }}}

tcoef=c(-1.93110204,-0.47531630,-0.04118204,-1.57872602,-0.22176426,-0.13836416,
0.88830288,0.60756224,0.39802821,0.32706859)

data(hHaplos)
data(haploX)

haploX$time <- haploX$times
Xdes <- model.matrix(~factor(time),haploX)
colnames(Xdes) <- paste("X",1:ncol(Xdes),sep="")
X <- dkeep(haploX,~id+y+time)
X <- cbind(X,Xdes)
Haplos <- dkeep(ghaplos,~id+"haplo*"+p)
desnames=paste("X",1:6,sep="")   # six X's related to 6 cycles 
out <- haplo.surv.discrete(X=X,y="y",time.name="time",
         Haplos=Haplos,desnames=desnames,designfunc=designftypes) 
names(out$coef) <- c(desnames,types)
out$coef
summary(out)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{robust.basehaz.phreg}
\alias{robust.basehaz.phreg}
\alias{summarybase.phreg}
\alias{conftype}
\title{Robust Baseline Hazard Standard Errors}
\usage{
robust.basehaz.phreg(x, type = "robust", fixbeta = NULL, ...)

summarybase.phreg(object, robust = FALSE, ...)

conftype(
  x,
  std.err,
  conf.type = c("log", "plain"),
  restrict = c("positive", "prob", "none"),
  conf.int = 0.95
)
}
\arguments{
\item{x}{point estimate(s).}

\item{type}{type of standard error (default \code{"robust"}).}

\item{fixbeta}{if non-NULL, fixes beta at given value.}

\item{...}{additional arguments.}

\item{object}{a \code{phreg} object.}

\item{robust}{logical; if TRUE, uses robust standard errors.}

\item{std.err}{standard error(s).}

\item{conf.type}{type of transformation: \code{"log"} or \code{"plain"}.}

\item{restrict}{restriction: \code{"positive"}, \code{"prob"}, or \code{"none"}.}

\item{conf.int}{confidence level (default 0.95).}
}
\value{
A list with \code{cumhaz}, \code{se.cumhaz}, and \code{strata}.

An object of class \code{"summary.recurrent"}.

A list with \code{upper}, \code{lower}, \code{conf.type}, and \code{conf.int}.
}
\description{
Computes robust (sandwich) standard errors for the cumulative baseline
hazard from a \code{phreg} object.

Summarizes cumulative baseline hazard estimates from a phreg object,
optionally with robust standard errors.

Computes confidence intervals using log or plain transformations,
with optional restrictions to positive values or probability scale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{dermalridges}
\alias{dermalridges}
\title{Dermal ridges data (families)}
\format{
Data on 50 families with ridge counts in left and right
hand for moter, father and each child. Family id in 'family' and
gender and child number in 'sex' and 'child'.
}
\source{
Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
}
\description{
Data on dermal ridge counts in left and right hand in (nuclear) families
}
\examples{
data(dermalridges)
fast.reshape(dermalridges,id="family",varying=c("child.left","child.right","sex"))
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop.strata.R
\name{drop.specials}
\alias{drop.specials}
\title{Remove Special Terms from a Formula}
\usage{
drop.specials(x, components, ...)
}
\arguments{
\item{x}{a formula object.}

\item{components}{character vector of special term names to remove.}

\item{...}{additional arguments passed to \code{lava::Specials}.}
}
\value{
The updated formula with an attribute \code{"variables"} containing
  the extracted variable names from removed specials.
}
\description{
Removes terms such as \code{strata()} or \code{cluster()} from a
formula/terms object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{drelevel}
\alias{drelevel}
\alias{dlevels}
\alias{dlevel}
\alias{dlev}
\alias{drelev}
\alias{dlev<-}
\alias{dlevel<-}
\alias{drelev<-}
\alias{drelevel<-}
\alias{dfactor}
\alias{dfactor<-}
\alias{dnumeric}
\alias{dnumeric<-}
\title{relev levels for data frames}
\usage{
drelevel(
  data,
  y = NULL,
  x = NULL,
  ref = NULL,
  newlevels = NULL,
  regex = mets.options()$regex,
  sep = NULL,
  overwrite = FALSE,
  ...
)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{name of variable, or fomula, or names of variables on data frame.}

\item{ref}{new reference variable}

\item{newlevels}{to combine levels of factor in data frame}

\item{regex}{for regular expressions.}

\item{sep}{seperator for naming of cut names.}

\item{overwrite}{to overwrite variable}

\item{...}{Optional additional arguments}
}
\description{
levels shows levels for variables in data frame, relevel relevels a factor in data.frame
}
\examples{

data(mena)
dstr(mena)
dfactor(mena)  <- ~twinnum
dnumeric(mena) <- ~twinnum.f

dstr(mena)

mena2 <- drelevel(mena,"cohort",ref="(1980,1982]")
mena2 <- drelevel(mena,~cohort,ref="(1980,1982]")
mena2 <- drelevel(mena,cohortII~cohort,ref="(1980,1982]")
dlevels(mena)
dlevels(mena2)
drelevel(mena,ref="(1975,1977]")  <-  ~cohort
drelevel(mena,ref="(1980,1982]")  <-  ~cohort
dlevels(mena,"coh*")
dtable(mena,"coh*",level=1)

### level 1 of zyg as baseline for new variable
drelevel(mena,ref=1) <- ~zyg
drelevel(mena,ref=c("DZ","[1973,1975]")) <- ~ zyg+cohort
drelevel(mena,ref=c("DZ","[1973,1975]")) <- zygdz+cohort.early~ zyg+cohort
### level 2 of zyg and cohort as baseline for new variables
drelevel(mena,ref=2) <- ~ zyg+cohort
dlevels(mena)

##################### combining factor levels with newlevels argument

dcut(mena,labels=c("I","II","III","IV")) <- cat4~agemena
dlevels(drelevel(mena,~cat4,newlevels=1:3))
dlevels(drelevel(mena,ncat4~cat4,newlevels=3:2))
drelevel(mena,newlevels=3:2) <- ncat4~cat4
dlevels(mena)

dlevels(drelevel(mena,nca4~cat4,newlevels=list(c(1,4),2:3)))

drelevel(mena,newlevels=list(c(1,4),2:3)) <- nca4..2 ~ cat4
dlevels(mena)

drelevel(mena,newlevels=list("I-III"=c("I","II","III"),"IV"="IV")) <- nca4..3 ~ cat4
dlevels(mena)

drelevel(mena,newlevels=list("I-III"=c("I","II","III"))) <- nca4..4 ~ cat4
dlevels(mena)

drelevel(mena,newlevels=list(group1=c("I","II","III"))) <- nca4..5 ~ cat4
dlevels(mena)

drelevel(mena,newlevels=list(g1=c("I","II","III"),g2="IV")) <- nca4..6 ~ cat4
dlevels(mena)

}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastreshape.R
\name{fast.reshape}
\alias{fast.reshape}
\alias{dreshape}
\title{Fast reshape}
\usage{
fast.reshape(
  data,
  varying,
  id,
  num,
  sep = "",
  keep,
  idname = "id",
  numname = "num",
  factor = FALSE,
  idcombine = TRUE,
  labelnum = FALSE,
  labels,
  regex = mets.options()$regex,
  dropid = FALSE,
  ...
)
}
\arguments{
\item{data}{data.frame or matrix}

\item{varying}{Vector of prefix-names of the time varying variables. Optional for Long->Wide reshaping.}

\item{id}{id-variable. If omitted then reshape Wide->Long.}

\item{num}{Optional number/time variable}

\item{sep}{String seperating prefix-name with number/time}

\item{keep}{Vector of column names to keep}

\item{idname}{Name of id-variable (Wide->Long)}

\item{numname}{Name of number-variable (Wide->Long)}

\item{factor}{If true all factors are kept (otherwise treated as character)}

\item{idcombine}{If TRUE and \code{id} is vector of several variables, the unique id is combined from all the variables.
Otherwise the first variable is only used as identifier.}

\item{labelnum}{If TRUE varying variables in wide format (going from long->wide) are labeled 1,2,3,... otherwise use 'num' variable. In long-format (going from wide->long) varying variables matching 'varying' prefix are only selected if their postfix is a number.}

\item{labels}{Optional labels for the number variable}

\item{regex}{Use regular expressions}

\item{dropid}{Drop id in long format (default FALSE)}

\item{...}{Optional additional arguments}
}
\description{
Fast reshape/tranpose of data
}
\examples{
m <- lava::lvm(c(y1,y2,y3,y4)~x)
d <- lava::sim(m,5)
d
fast.reshape(d,"y")
fast.reshape(fast.reshape(d,"y"),id="id")

##### From wide-format
(dd <- fast.reshape(d,"y"))
## Same with explicit setting new id and number variable/column names
## and seperator "" (default) and dropping x
fast.reshape(d,"y",idname="a",timevar="b",sep="",keep=c())
## Same with 'reshape' list-syntax
fast.reshape(d,list(c("y1","y2","y3","y4")),labelnum=TRUE)

##### From long-format
fast.reshape(dd,id="id")
## Restrict set up within-cluster varying variables
fast.reshape(dd,"y",id="id")
fast.reshape(dd,"y",id="id",keep="x",sep=".")

#####
x <- data.frame(id=c(5,5,6,6,7),y=1:5,x=1:5,tv=c(1,2,2,1,2))
x
(xw <- fast.reshape(x,id="id"))
(xl <- fast.reshape(xw,c("y","x"),idname="id2",keep=c()))
(xl <- fast.reshape(xw,c("y","x","tv")))
(xw2 <- fast.reshape(xl,id="id",num="num"))
fast.reshape(xw2,c("y","x"),idname="id")

### more generally:
### varying=list(c("ym","yf","yb1","yb2"), c("zm","zf","zb1","zb2"))
### varying=list(c("ym","yf","yb1","yb2")))

##### Family cluster example
d <- mets:::simBinFam(3)
d
fast.reshape(d,var="y")
fast.reshape(d,varying=list(c("ym","yf","yb1","yb2")))

d <- lava::sim(lava::lvm(~y1+y2+ya),10)
d
(dd <- fast.reshape(d,"y"))
fast.reshape(d,"y",labelnum=TRUE)
fast.reshape(dd,id="id",num="num")
fast.reshape(dd,id="id",num="num",labelnum=TRUE)
fast.reshape(d,c(a="y"),labelnum=TRUE) ## New column name


##### Unbalanced data
m <- lava::lvm(c(y1,y2,y3,y4)~ x+z1+z3+z5)
d <- lava::sim(m,3)
d
fast.reshape(d,c("y","z"))

##### not-varying syntax:
fast.reshape(d,-c("x"))

##### Automatically define varying variables from trailing digits
fast.reshape(d)

##### Prostate cancer example
data(prt)
head(prtw <- fast.reshape(prt,"cancer",id="id"))
ftable(cancer1~cancer2,data=prtw)
rm(prtw)
}
\author{
Thomas Scheike, Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{rchaz}
\alias{rchaz}
\alias{sim_rchaz}
\alias{lin_approx}
\title{Simulation of Piecewise Constant Hazard Model (Cox)}
\usage{
rchaz(cumhazard, rr, n = NULL, entry = NULL, cause = 1, extend = FALSE)
}
\arguments{
\item{cumhazard}{Cumulative hazard matrix (columns: time, cumulative hazard), or 
piece-constant rates for periods defined by the first column of input.}

\item{rr}{Relative risk vector for simulations, or alternatively when \code{rr=1} 
specify \code{n}.}

\item{n}{Number of simulations if \code{rr} not given.}

\item{entry}{Delayed entry time for simulations (optional).}

\item{cause}{Name/code of the event cause (default 1).}

\item{extend}{To extend piecewise constant with constant rate beyond the last break point. 
Default is \code{FALSE}. If \code{TRUE}, extends with average rate over time from 
cumulative. If numeric, uses the given rate.}
}
\value{
A data frame containing:
  \item{entry}{Entry times.}
  \item{time}{Event/censoring times.}
  \item{status}{Event status (1=event, 0=censored).}
  \item{rr}{Relative risks used.}
  
  Attributes include:
  \item{cumhaz}{The cumulative hazard used.}
  \item{extend.rate}{The extension rate if used.}
}
\description{
Simulates data from a piecewise constant baseline hazard that can also be of Cox type. 
Censors data at the highest value of the break points.
}
\details{
For a piecewise linear cumulative hazard, the inverse is easy to compute. With delayed 
entry \eqn{x}, we compute:
\deqn{\Lambda^{-1}(\Lambda(x) + E/RR)}
where \eqn{RR} are the relative risks and \eqn{E} is exponential with mean 1.
This quantity has survival function:
\deqn{P(T > t | T>x) = \exp(-RR (\Lambda(t) - \Lambda(x)))}
}
\examples{
chaz <-  c(0,1,1.5,2,2.1)
breaks <- c(0,10,   20,  30,   40)
cumhaz <- cbind(breaks,chaz)
n <- 10
X <- rbinom(n,1,0.5)
beta <- 0.2
rrcox <- exp(X * beta)

pctime <- rchaz(cumhaz,n=10)
pctimecox <- rchaz(cumhaz,rrcox,entry=runif(n))
}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipw.R
\name{ipw2}
\alias{ipw2}
\title{Inverse Probability of Censoring Weights}
\usage{
ipw2(
  data,
  times = NULL,
  entrytime = NULL,
  time = "time",
  cause = "cause",
  same.cens = FALSE,
  cluster = NULL,
  pairs = FALSE,
  strata = NULL,
  obs.only = TRUE,
  cens.formula = NULL,
  cens.code = 0,
  pair.cweight = "pcw",
  pair.tweight = "ptw",
  pair.weight = "weights",
  cname = "cweights",
  tname = "tweights",
  weight.name = "indi.weights",
  prec.factor = 100,
  ...
)
}
\arguments{
\item{data}{data frame}

\item{times}{possible time argument for speciying a maximum value of time tau=max(times), to specify when things are considered censored or not.}

\item{entrytime}{nam of entry-time for truncation.}

\item{time}{name of time variable on data frame.}

\item{cause}{name of cause indicator on data frame.}

\item{same.cens}{For clustered data, should same censoring be assumed and same truncation (bivariate probability calculated as mininum of the marginal probabilities)}

\item{cluster}{name of clustering variable}

\item{pairs}{For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)}

\item{strata}{name of strata variable to get weights stratified.}

\item{obs.only}{Return data with uncensored observations only}

\item{cens.formula}{model for Cox models for truncation and right censoring times.}

\item{cens.code}{censoring.code}

\item{pair.cweight}{Name of weight variable in the new data.frame for right censorig of pairs}

\item{pair.tweight}{Name of weight variable in the new data.frame for left truncation of pairs}

\item{pair.weight}{Name of weight variable in the new data.frame for right censoring and left truncation of pairs}

\item{cname}{Name of weight variable in the new data.frame for right censoring of individuals}

\item{tname}{Name of weight variable in the new data.frame for left truncation of individuals}

\item{weight.name}{Name of weight variable in the new data.frame for right censoring and left truncation of individuals}

\item{prec.factor}{To let tied censoring and truncation times come after the death times.}

\item{...}{Additional arguments to censoring model}
}
\description{
Internal function.
Calculates Inverse Probability of Censoring and Truncation 
Weights and adds them to a data.frame
}
\examples{
library("timereg")
set.seed(1)
d <- simnordic.random(5000,delayed=TRUE,ptrunc=0.7,
      cordz=0.5,cormz=2,lam0=0.3,country=FALSE)
d$strata <- as.numeric(d$country)+(d$zyg=="MZ")*4
times <- seq(60,100,by=10)
c1 <- timereg::comp.risk(Event(time,cause)~1+cluster(id),data=d,cause=1,
	model="fg",times=times,max.clust=NULL,n.sim=0)
mm=model.matrix(~-1+zyg,data=d)
out1<-random.cif(c1,data=d,cause1=1,cause2=1,same.cens=TRUE,theta.des=mm)
summary(out1)
pc1 <- predict(c1,X=1,se=0)
plot(pc1)

dl <- d[!d$truncated,]
dl <- ipw2(dl,cluster="id",same.cens=TRUE,time="time",entrytime="entry",cause="cause",
           strata="strata",prec.factor=100)
cl <- timereg::comp.risk(Event(time,cause)~+1+
		cluster(id),
		data=dl,cause=1,model="fg",
		weights=dl$indi.weights,cens.weights=rep(1,nrow(dl)),
            times=times,max.clust=NULL,n.sim=0)
pcl <- predict(cl,X=1,se=0)
lines(pcl$time,pcl$P1,col=2)
mm=model.matrix(~-1+factor(zyg),data=dl)
out2<-random.cif(cl,data=dl,cause1=1,cause2=1,theta.des=mm,
                 weights=dl$weights,censoring.weights=rep(1,nrow(dl)))
summary(out2)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/force.same.cens.R
\name{force.same.cens}
\alias{force.same.cens}
\alias{force_same_cens}
\title{Force Same Censoring Within Clusters}
\usage{
force.same.cens(
  data,
  id = "id",
  time = "time",
  cause = "cause",
  entrytime = NULL,
  cens.code = 0
)

force_same_cens(
  data,
  id = "id",
  time = "time",
  cause = "cause",
  entrytime = NULL,
  cens.code = 0
)
}
\arguments{
\item{data}{a data.frame in long format.}

\item{id}{name of the cluster/pair identifier column.}

\item{time}{name of the time variable.}

\item{cause}{name of the cause/status variable.}

\item{entrytime}{optional name of left-truncation time variable.}

\item{cens.code}{value indicating censoring in the cause variable.}
}
\value{
A data.frame with enforced same censoring.
}
\description{
Enforces the same censoring time within clusters (pairs) by censoring
both subjects at the minimum censoring time when censoring occurs first.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_multistateII}
\alias{sim_multistateII}
\title{Illness-Death Competing Risks with Two Causes of Death}
\usage{
sim_multistateII(
  cumhaz,
  death.cumhaz,
  death.cumhaz2,
  n = NULL,
  rr = NULL,
  rd = NULL,
  rd2 = NULL,
  gamma23 = 0,
  gamma24 = 0,
  early2 = 10000,
  gap.time = FALSE,
  max.recurrent = 100,
  cens = NULL,
  rrc = NULL,
  extend = TRUE,
  ...
)
}
\arguments{
\item{cumhaz}{Cumulative hazard from state 1 to 2.}

\item{death.cumhaz}{Cumulative hazard of death from state 1.}

\item{death.cumhaz2}{Cumulative hazard of death from state 2.}

\item{n}{Number of simulations.}

\item{rr}{Relative risks.}

\item{rd}{Relative risks for death from state 1.}

\item{rd2}{Relative risks for death from state 2.}

\item{gamma23}{Early effect parameters for death causes.}

\item{gamma24}{Early effect parameters for death causes.}

\item{early2}{Time threshold for early effect.}

\item{gap.time}{Gap time indicator.}

\item{max.recurrent}{Maximum recurrent events.}

\item{cens}{Censoring specification.}

\item{rrc}{Censoring relative risks.}

\item{extend}{Extend hazards.}

\item{...}{Additional arguments.}
}
\value{
Data frame with multi-state event history.
}
\description{
Simulates data from an illness-death model with two causes of death from the 
illness state. Covariate effects can be introduced via relative risk terms.
}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.twostage.R
\name{easy.binomial.twostage}
\alias{easy.binomial.twostage}
\title{Fits two-stage binomial for describing depdendence in binomial data
using marginals that are on logistic form using the binomial.twostage funcion, but
call is different and easier and the data manipulation is build into the function.
Useful in particular for family design data.}
\usage{
easy.binomial.twostage(
  margbin = NULL,
  data = parent.frame(),
  method = "nr",
  response = "response",
  id = "id",
  Nit = 60,
  detail = 0,
  silent = 1,
  weights = NULL,
  control = list(),
  theta = NULL,
  theta.formula = NULL,
  desnames = NULL,
  deshelp = 0,
  var.link = 1,
  iid = 1,
  step = 1,
  model = "plackett",
  marginal.p = NULL,
  strata = NULL,
  max.clust = NULL,
  se.clusters = NULL
)
}
\arguments{
\item{margbin}{Marginal binomial model}

\item{data}{data frame}

\item{method}{Scoring method}

\item{response}{name of response variable in data frame}

\item{id}{name of cluster variable in data frame}

\item{Nit}{Number of iterations}

\item{detail}{Detail for more output for iterations}

\item{silent}{Debug information}

\item{weights}{Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.}

\item{control}{Optimization arguments}

\item{theta}{Starting values for variance components}

\item{theta.formula}{design for depedence, either formula or design function}

\item{desnames}{names for dependence parameters}

\item{deshelp}{if 1 then prints out some data sets that are used, on on which the design function operates}

\item{var.link}{Link function for variance}

\item{iid}{Calculate i.i.d. decomposition}

\item{step}{Step size}

\item{model}{model}

\item{marginal.p}{vector of marginal probabilities}

\item{strata}{strata for fitting}

\item{max.clust}{max clusters used for i.i.d. decompostion}

\item{se.clusters}{clusters for iid decomposition for roubst standard errors}
}
\description{
If clusters contain more than two times, the algoritm uses a compososite likelihood
based on the pairwise bivariate models.
}
\details{
The reported standard errors are based on the estimated information from the
likelihood assuming that the marginals are known. This gives correct standard errors
in the case of the plackett distribution (OR model for dependence), but incorrect for
the clayton-oakes types model. The OR model is often known as the ALR model.
Our fitting procedures gives correct standard errors due to the ortogonality and is
fast.
}
\examples{
data(twinstut)
twinstut0 <- subset(twinstut, tvparnr<4000)
twinstut <- twinstut0
twinstut$binstut <- (twinstut$stutter=="yes")*1
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
		         clusters=twinstut$tvparnr,theta.des=theta.des,detail=0,
	                 method="nr")
summary(bin)
lava::estimate(coef=bin$theta,vcov=bin$var.theta,f=function(p) exp(p))

twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+cage,data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
		         clusters=twinstut$tvparnr,theta.des=theta.des,detail=0)
summary(bina)

theta.des <- model.matrix( ~-1+factor(zyg)+factor(zyg)*cage,data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
		         clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)

out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
                              response="binstut",id="tvparnr",var.link=1,
			          theta.formula=~-1+factor(zyg1))
summary(out)

## refers to zygosity of first subject in eash pair : zyg1
## could also use zyg2 (since zyg2=zyg1 within twinpair's))
## do not run t save time
# desfs <- function(x,num1="zyg1",namesdes=c("mz","dz","os"))
#     c(x[num1]=="mz",x[num1]=="dz",x[num1]=="os")*1
#
#out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
#                               data=twinstut, response="binstut",id="tvparnr",
#                               var.link=1,theta.formula=desfs,
#                               desnames=c("mz","dz","os"))
#summary(out3)

\donttest{ ## Reduce Ex.Timings
n <- 1000
set.seed(100)
dd <- simBinFam(n,beta=0.3)
binfam <- fast.reshape(dd,varying=c("age","x","y"))
## mother, father, children  (ordered)
head(binfam)

########### ########### ########### ########### ########### ###########
####  simple analyses of binomial family data
########### ########### ########### ########### ########### ###########
desfs <- function(x,num1="num1",num2="num2")
{
     pp <- 1*(((x[num1]=="m")*(x[num2]=="f"))|(x[num1]=="f")*(x[num2]=="m"))
     pc <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
     cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
     c(pp,pc,cc)
}

ud <- easy.binomial.twostage(y~+1,data=binfam,
     response="y",id="id",
     theta.formula=desfs,desnames=c("pp","pc","cc"))
summary(ud)

udx <- easy.binomial.twostage(y~+x,data=binfam,
     response="y",id="id",
     theta.formula=desfs,desnames=c("pp","pc","cc"))
summary(udx)

########### ########### ########### ########### ########### ###########
####  now allowing parent child POR to be different for mother and father
########### ########### ########### ########### ########### ###########

desfsi <- function(x,num1="num1",num2="num2")
{
    pp <- (x[num1]=="m")*(x[num2]=="f")*1
    mc <- (x[num1]=="m")*(x[num2]=="b1" | x[num2]=="b2")*1
    fc <- (x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
    cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
    c(pp,mc,fc,cc)
}

udi <- easy.binomial.twostage(y~+1,data=binfam,
     response="y",id="id",
     theta.formula=desfsi,desnames=c("pp","mother-child","father-child","cc"))
summary(udi)

##now looking to see if interactions with age or age influences marginal models
##converting factors to numeric to make all involved covariates numeric
##to use desfai2 rather then desfai that works on binfam

nbinfam <- binfam
nbinfam$num <- as.numeric(binfam$num)
head(nbinfam)

desfsai <- function(x,num1="num1",num2="num2")
{
    pp <- (x[num1]=="m")*(x[num2]=="f")*1
### av age for pp=1 i.e parent pairs
    agepp <- ((as.numeric(x["age1"])+as.numeric(x["age2"]))/2-30)*pp
    mc <- (x[num1]=="m")*(x[num2]=="b1" | x[num2]=="b2")*1
    fc <- (x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1
    cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1
    agecc <- ((as.numeric(x["age1"])+as.numeric(x["age2"]))/2-12)*cc
    c(pp,agepp,mc,fc,cc,agecc)
}

desfsai2 <- function(x,num1="num1",num2="num2")
{
    pp <- (x[num1]==1)*(x[num2]==2)*1
    agepp <- (((x["age1"]+x["age2"]))/2-30)*pp ### av age for pp=1 i.e parent pairs
    mc <- (x[num1]==1)*(x[num2]==3 | x[num2]==4)*1
    fc <- (x[num1]==2)*(x[num2]==3 | x[num2]==4)*1
    cc <- (x[num1]==3)*(x[num2]==3 | x[num2]==4)*1
    agecc <- ((x["age1"]+x["age2"])/2-12)*cc ### av age for children
    c(pp,agepp,mc,fc,cc,agecc)
}

udxai2 <- easy.binomial.twostage(y~+x+age,data=binfam,
     response="y",id="id",
     theta.formula=desfsai,
     desnames=c("pp","pp-age","mother-child","father-child","cc","cc-age"))
summary(udxai2)
}
}
\keyword{binomial}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratio.regression.R
\name{binregRatio}
\alias{binregRatio}
\alias{rmtlRatio}
\title{Percentage of Years Lost Due to a Cause Regression}
\usage{
binregRatio(
  formula,
  data,
  cause = 1,
  time = NULL,
  beta = NULL,
  type = c("III", "II", "I"),
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  cens.model = ~+1,
  se = TRUE,
  relative.to.causes = NULL,
  kaplan.meier = TRUE,
  cens.code = 0,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  outcome = c("rmtl", "cif"),
  model = c("logit", "exp", "lin"),
  Ydirect = NULL,
  ...
)
}
\arguments{
\item{formula}{Formula with an outcome (see \code{coxph}). The first covariate on the RHS 
is typically the treatment or group indicator. Can include \code{cluster(id)}.}

\item{data}{Data frame containing the variables.}

\item{cause}{Numeric code of the cause of interest.}

\item{time}{Time point \eqn{t} for the analysis. Required.}

\item{beta}{Starting values for optimization (default NULL, uses zeros).}

\item{type}{Type of estimator: \code{"I"} (IPCW only), \code{"II"} (IPCW + augmentation), 
or \code{"III"} (IPCW + complex augmentation). Default is \code{"III"}.}

\item{offset}{Offsets for the partial likelihood.}

\item{weights}{Weights for the score equations.}

\item{cens.weights}{External censoring weights (if provided, \code{cens.model} is ignored).}

\item{cens.model}{Formula for the censoring model (default \code{~+1}, stratified KM). 
Can include \code{strata()} for stratified censoring.}

\item{se}{Logical; if TRUE, computes standard errors based on IPCW (default TRUE).}

\item{relative.to.causes}{If not NULL, compares the RMTL of the specified \code{cause} 
to the RMTL of the causes in this vector (the denominator becomes the sum of these causes).}

\item{kaplan.meier}{Logical; if TRUE, uses Kaplan-Meier for IPCW weights; if FALSE, 
uses \eqn{\exp(-\text{cumulative hazard})}.}

\item{cens.code}{Censoring code (default 0).}

\item{no.opt}{Logical; if TRUE, skips optimization and uses \code{beta} directly.}

\item{method}{Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.}

\item{augmentation}{Initial augmentation term (used for type "II" and "III").}

\item{outcome}{Outcome type: \code{"rmtl"} (years lost) or \code{"cif"} (cumulative incidence).}

\item{model}{Link function: \code{"logit"} (default), \code{"exp"}, or \code{"lin"}.}

\item{Ydirect}{Matrix with two columns (numerator, denominator) to use directly as the response.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"binreg"} and \code{"ratio"} containing:
  \item{coef}{Coefficient estimates.}
  \item{se.coef}{Standard errors.}
  \item{var}{Variance-covariance matrix.}
  \item{iid}{Influence function decomposition (with censoring adjustment).}
  \item{iidI}{Influence function without censoring adjustment.}
  \item{naive.var}{Variance assuming known censoring.}
  \item{time}{Time point used.}
  \item{cause}{Cause of interest.}
  \item{Causes}{Set of causes considered in the denominator.}
  \item{Yipcw}{IPCW-adjusted response matrix.}
  \item{coefI, varI}{Results from the initial (type "I") fit.}
  \item{augmentation}{Final augmentation term used.}
}
\description{
Estimates the percentage of the restricted mean time lost (RMTL) that is attributable 
to a specific cause and models how covariates affect this percentage using IPCW regression.
}
\details{
Let the total years lost be \eqn{Y = t - \min(T, t)} and the years lost due to cause 1 be 
\eqn{Y_1 = I(\epsilon=1) (t - \min(T, t))}. The function models the ratio:
\deqn{ \text{logit}\left( \frac{E(Y_1 | X)}{E(Y | X)} \right) = X^T \beta }

Estimation is based on a binomial regression IPCW response estimating equation:
\deqn{ X \left( \Delta^{\text{ipcw}}(t) \left( Y \cdot \text{expit}(X^T \beta) - Y_1 \right) \right) = 0 }
where \eqn{\Delta^{\text{ipcw}}(t) = I(\min(t,T) < C) / G_c(\min(t,T))} is the IPCW adjustment.

The function supports three types of estimators:
\itemize{
  \item \code{"I"}: Classical outcome IPCW regression (no augmentation).
  \item \code{"II"}: Adds a censoring augmentation term \eqn{X \int E(Z(t)| T>s)/G_c(s) d \hat M_c} 
    to improve efficiency (requires an initial estimate of \eqn{\beta}).
  \item \code{"III"}: Adds a more complex augmentation term separating the expectations of 
    \eqn{Y} and \eqn{Y_1} for further efficiency gains.
}

The variance is based on the squared influence functions (IID). A "naive" variance 
(assuming known censoring) is also provided for comparison.
}
\examples{
data(bmt); bmt$time <- bmt$time+runif(408)*0.001

rmtl30 <- rmstIPCW(Event(time,cause!=0)~platelet+tcell+age, bmt, time=30, cause=1, outcome="rmtl")
rmtl301 <- rmstIPCW(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1)
rmtl302 <- rmstIPCW(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=2)

estimate(rmtl30)
estimate(rmtl301)
estimate(rmtl302)

## Percentage of total RMTL due to cause 1
rmtlratioI <- rmtlRatio(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1)
summary(rmtlratioI)

newdata <- data.frame(platelet=1, tcell=1, age=1)
pp <- predict(rmtlratioI, newdata)
pp

## Percentage of total cumulative incidence due to cause 1
cifratio <- binregRatio(Event(time,cause)~platelet+tcell+age, bmt, time=30, cause=1, model="cif")
summary(cifratio)
pp <- predict(cifratio, newdata)
pp
}
\references{
Scheike, T. & Tanaka, S. (2025). Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause. WIP.
}
\seealso{
\code{\link{resmeanIPCW}}, \code{\link{binreg}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{test.conc}
\alias{test.conc}
\title{Concordance test Compares two concordance estimates}
\usage{
test.conc(conc1, conc2, same.cluster = FALSE)
}
\arguments{
\item{conc1}{Concordance estimate of group 1}

\item{conc2}{Concordance estimate of group 2}

\item{same.cluster}{if FALSE then groups are independent, otherwise estimates are based on same data.}
}
\description{
.. content for description (no empty lines) ..
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{extendCums}
\alias{extendCums}
\title{Extend Cumulative Hazard Functions to Common Time Range}
\usage{
extendCums(cumA, cumB, extend = NULL)
}
\arguments{
\item{cumA}{a cumulative hazard matrix (two columns: time, cumulative hazard)
or a list of such matrices.}

\item{cumB}{an optional second cumulative hazard matrix. If provided it is
combined with \code{cumA} into a list.}

\item{extend}{numeric vector of hazard rates used for extrapolation, one per
cumulative hazard. If \code{NULL} (default), rates are estimated as
the ratio of the last cumulative hazard value to the last time point.}
}
\value{
A list of cumulative hazard matrices extended to the common maximum time.
}
\description{
Extends a collection of cumulative hazard functions so that they all
cover the same time range. Cumulative hazards that end before the
maximum observed time are extrapolated linearly using either a
user-supplied rate or the average hazard rate estimated from the
existing cumulative hazard.
}
\seealso{
\code{\link{rchaz}}, \code{\link{rcrisk}}, \code{\link{mets-simulation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gofM_phreg}
\alias{gofM_phreg}
\title{Goodness-of-Fit for Cox Covariates (Model Matrix)}
\usage{
gofM_phreg(
  formula,
  data,
  offset = NULL,
  weights = NULL,
  modelmatrix = NULL,
  n.sim = 1000,
  silent = 1,
  ...
)
}
\arguments{
\item{formula}{Formula for the Cox regression model.}

\item{data}{Data frame.}

\item{offset}{Offset vector.}

\item{weights}{Weights vector.}

\item{modelmatrix}{Matrix for cumulating residuals. Typically constructed using 
\code{cumContr()} or manually (e.g., quartiles of a continuous covariate).}

\item{n.sim}{Number of simulations (default 1000).}

\item{silent}{Logical; suppresses timing estimates if TRUE.}

\item{...}{Additional arguments passed to \code{phreg}.}
}
\value{
An object of class \code{"gof.phreg"} containing:
  \item{jumptimes}{Event times.}
  \item{supUsim}{Simulated supremum values.}
  \item{res}{Matrix with observed supremum and p-values for each column of \code{modelmatrix}.}
  \item{score}{Cumulative score process.}
  \item{simUt}{Simulated processes.}
  \item{Utlast, pval.last}{Supremum and p-value for the final time point (covariate direction).}
  \item{type}{Type of test ("modelmatrix").}
}
\description{
Tests the functional form of covariates in a Cox PH model by computing cumulative 
residuals against a user-specified model matrix. This helps detect non-linear effects 
or time-varying coefficients (interaction with time).
}
\details{
The test statistic is:
\deqn{ U(t) = \int_0^t K^T d \hat M }
where \eqn{K} is the model matrix (e.g., a set of basis functions for a continuous covariate) 
and \eqn{\hat M} are the martingale residuals.

P-values are based on the Lin, Wei, and Ying (1993) resampling method. The plot shows 
whether the residuals are consistent with the model across the range of the covariate.
}
\examples{
data(TRACE)
set.seed(1)
TRACEsam <- blocksample(TRACE, idvar="id", replace=FALSE, 100)
dcut(TRACEsam) <- ~. 
mm <- model.matrix(~-1+factor(wmicat.4), data=TRACEsam)
m1 <- gofM_phreg(Surv(time,status==9)~vf+chf+wmi, data=TRACEsam, modelmatrix=mm)
summary(m1)
if (interactive()) {
par(mfrow=c(2,2))
plot(m1)
}

## Cumulative sums in covariates via design matrix
mm <- mets:::cumContr(TRACEsam$wmi, breaks=10, equi=TRUE)
m1 <- gofM_phreg(Surv(time,status==9)~strata(vf)+chf+wmi, data=TRACEsam,
         modelmatrix=mm, silent=0)
summary(m1)
}
\references{
Lin, D. Y., Wei, L. J., & Ying, Z. (1993). Checking the Cox model with cumulative sums of martingale-based residuals. Biometrika, 80(3), 557-572.
}
\seealso{
\code{\link{gof.phreg}}, \code{\link{gofZ_phreg}}, \code{\link{cumContr}}
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{casewise.test}
\alias{casewise.test}
\alias{slope.process}
\alias{casewise.bin}
\title{Estimates the casewise concordance based on Concordance and marginal estimate using timereg and performs test for independence}
\usage{
casewise.test(conc, marg, test = "no-test", p = 0.01)
}
\arguments{
\item{conc}{Concordance}

\item{marg}{Marginal estimate}

\item{test}{Type of test for independence assumption. "conc" makes test on concordance scale and "case" means a test on the casewise concordance}

\item{p}{check that marginal probability is greater at some point than p}
}
\description{
Estimates the casewise concordance based on Concordance and marginal estimate using timereg and performs test for independence
}
\details{
Uses cluster based conservative standard errors for marginal and sometimes only the uncertainty of the concordance estimates. This works prettey well, alternatively one can use also the 
         funcions Casewise for a specific time point
}
\examples{
\donttest{ ## Reduce Ex.Timings
library("timereg")
data("prt",package="mets");
prt <- force.same.cens(prt,cause="status")

prt <- prt[which(prt$id \%in\% sample(unique(prt$id),7500)),]
### marginal cumulative incidence of prostate cancer
times <- seq(60,100,by=2)
outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,cause=2,times=times)

cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
cifdz <- predict(outm,X=1,uniform=0,resample.iid=1)

### concordance for MZ and DZ twins
cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                 data=prt,cause=c(2,2))
cdz <- cc$model$"DZ"
cmz <- cc$model$"MZ"

### To compute casewise cluster argument must be passed on,
###  here with a max of 100 to limit comp-time
outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,
                 cause=2,times=times,max.clust=100)
cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,
                cause=c(2,2),se.clusters=outm$clusters)
cdz <- cc$model$"DZ"
cmz <- cc$model$"MZ"

cdz <- casewise.test(cdz,cifmz,test="case") ## test based on casewise
cmz <- casewise.test(cmz,cifmz,test="conc") ## based on concordance

plot(cmz,ylim=c(0,0.7),xlim=c(60,100))
par(new=TRUE)
plot(cdz,ylim=c(0,0.7),xlim=c(60,100))

slope.process(cdz$casewise[,1],cdz$casewise[,2],iid=cdz$casewise.iid)

slope.process(cmz$casewise[,1],cmz$casewise[,2],iid=cmz$casewise.iid)

}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{survival.twostageCC}
\alias{survival.twostageCC}
\title{Twostage survival model for multivariate survival data}
\usage{
survival.twostageCC(
  margsurv,
  data = parent.frame(),
  method = "nr",
  Nit = 60,
  detail = 0,
  clusters = NULL,
  silent = 1,
  weights = NULL,
  control = list(),
  theta = NULL,
  theta.des = NULL,
  var.link = 1,
  baseline.iid = 1,
  step = 0.5,
  model = "clayton.oakes",
  marginal.trunc = NULL,
  marginal.survival = NULL,
  marginal.status = NULL,
  strata = NULL,
  se.clusters = NULL,
  numDeriv = 0,
  random.design = NULL,
  pairs = NULL,
  dim.theta = NULL,
  numDeriv.method = "simple",
  additive.gamma.sum = NULL,
  var.par = 1,
  cr.models = NULL,
  case.control = 0,
  ascertained = 0,
  shut.up = 0
)
}
\arguments{
\item{margsurv}{Marginal model}

\item{data}{data frame}

\item{method}{Scoring method "nr", "nlminb", "optimize", "nlm"}

\item{Nit}{Number of iterations}

\item{detail}{Detail}

\item{clusters}{Cluster variable}

\item{silent}{Debug information}

\item{weights}{Weights}

\item{control}{Optimization arguments}

\item{theta}{Starting values for variance components}

\item{theta.des}{design for dependence parameters, when pairs are given the indeces of the
theta-design for this pair, is given in pairs as column 5}

\item{var.link}{Link function for variance:  exp-link.}

\item{baseline.iid}{to adjust for baseline estimation, using phreg function on same data.}

\item{step}{Step size}

\item{model}{model}

\item{marginal.trunc}{marginal left truncation probabilities}

\item{marginal.survival}{optional vector of marginal survival probabilities}

\item{marginal.status}{related to marginal survival probabilities}

\item{strata}{strata for fitting, see example}

\item{se.clusters}{for clusters for se calculation with iid}

\item{numDeriv}{to get numDeriv version of second derivative, otherwise uses sum of squared score}

\item{random.design}{random effect design for additive gamma model, when pairs are given the
indeces of the pairs random.design rows are given as columns 3:4}

\item{pairs}{matrix with rows of indeces (two-columns) for the pairs considered in the pairwise
composite score, useful for case-control sampling when marginal is known.}

\item{dim.theta}{dimension of the theta parameter for pairs situation.}

\item{numDeriv.method}{uses simple to speed up things and second derivative not so important.}

\item{additive.gamma.sum}{for two.stage=0, this is specification of the lamtot in the models via
a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number
of theta parameters), when null then sums all parameters.}

\item{var.par}{is 1 for the default parametrization with the variances of the random effects,
var.par=0 specifies that the \eqn{\lambda_j}'s are used as parameters.}

\item{cr.models}{competing risks models for two.stage=0, should be given as a list with models for each cause}

\item{case.control}{assumes case control structure for "pairs" with second column being the probands,
when this options is used the twostage model is profiled out via the paired estimating equations for the survival model.}

\item{ascertained}{if the pair are sampled only when there is an event. This is in contrast to
case.control sampling where a proband is given. This can be combined with control probands. Pair-call
of twostage is needed  and second column of pairs are the first jump time with an event for ascertained pairs,
or time of control proband.}

\item{shut.up}{to make the program more silent in the context of iterative procedures for case-control}
}
\description{
older extended version of survival.twostage with  more options  and different maximizer.
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/restricted.mean.R
\name{resmeanIPCW}
\alias{resmeanIPCW}
\alias{rmstIPCW}
\title{Restricted IPCW Mean for Censored Survival Data}
\usage{
resmeanIPCW(formula, data, outcome = c("rmst", "rmtl"), ...)
}
\arguments{
\item{formula}{Formula with an \code{Event} outcome (e.g., \code{Event(time, cause)}).}

\item{data}{Data frame containing the variables.}

\item{outcome}{Outcome type: \code{"rmst"} (Restricted Mean Survival Time) or 
\code{"rmtl"} (Restricted Mean Time Lost).}

\item{...}{Additional arguments passed to \code{binreg}, such as \code{time}, \code{cause}, 
\code{cens.model}, \code{model}, \code{type}, etc.}
}
\value{
An object of class \code{"binreg"} containing:
  \item{coef}{Coefficient estimates.}
  \item{se}{Standard errors.}
  \item{var}{Variance-covariance matrix.}
  \item{iid}{Influence function decomposition.}
  \item{naive.var}{Variance under known censoring model (if applicable).}
  \item{time}{Time point used.}
  \item{outcome}{Type of outcome analyzed.}
}
\description{
Provides a fast implementation of Inverse Probability of Censoring Weighting (IPCW) 
regression for a single time point. It fits the model:
\deqn{ E( \min(T, t) | X ) = \exp( X^T \beta) }
or, in the case of competing risks data:
\deqn{ E( I(\epsilon=1) (t - \min(T, t)) | X ) = \exp( X^T \beta) }
which represents the "Years Lost Due to Cause" (RMTL).
}
\details{
The method solves the binomial regression IPCW response estimating equation:
\deqn{ X \left( \frac{\Delta(\min(T,t)) Y}{G_c(\min(T,t))} - \exp( X^T \beta) \right) = 0 }
where \eqn{\Delta(\min(T,t)) = I(\min(T,t) \leq C)} is the indicator of being uncensored 
at the time of interest.

When the status variable is binary, the outcome is assumed to be \eqn{Y = \min(T,t)} (RMST).
If the status has more than two levels (competing risks), the outcome is 
\eqn{Y = (t - \min(T,t)) I(\text{status}=\text{cause})} (RMTL for a specific cause).

The function supports:
\itemize{
  \item \strong{IPCW Adjustment}: Weights by the inverse of the censoring survival probability.
  \item \strong{Augmentation}: Can include an augmentation term (type="II" or "III") to improve efficiency 
    and robustness (Double Robust estimation).
  \item \strong{Variance Estimation}: Based on the influence function, including adjustments for 
    the estimation of the censoring model.
}
}
\examples{
data(bmt); bmt$time <- bmt$time+runif(nrow(bmt))*0.001

# E( min(T;t) | X ) = exp( a+b X) with IPCW estimation 
out <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
                time=50, cens.model=~strata(platelet), model="exp")
summary(out)

\donttest{ ## Reduce Ex.Timings
## Weighted GLM version (RMST)
out2 <- logitIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
            time=50, cens.model=~strata(platelet), model="exp", outcome="rmst")
summary(out2)

### Time-lost (RMTL)
outtl <- resmeanIPCW(Event(time,cause!=0)~tcell+platelet+age, bmt,
                time=50, cens.model=~strata(platelet), model="exp", outcome="rmtl")
summary(outtl)

### Same as Kaplan-Meier for full censoring model 
bmt$int <- with(bmt, strata(tcell, platelet))
out <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30,
                             cens.model=~strata(platelet, tcell), model="lin")
estimate(out)
out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet), data=bmt)
rm1 <- resmean_phreg(out1, times=30)
summary(rm1)

### Years lost regression
outl <- resmeanIPCW(Event(time,cause!=0)~-1+int, bmt, time=30, outcome="years-lost",
                             cens.model=~strata(platelet, tcell), model="lin")
estimate(outl)

## Competing risks years-lost for cause 1  
out <- resmeanIPCW(Event(time,cause)~-1+int, bmt, time=30, cause=1,
                            cens.model=~strata(platelet, tcell), model="lin")
estimate(out)
## Same as integrated cumulative incidence 
rmc1 <- cif_yearslost(Event(time,cause)~strata(tcell,platelet), data=bmt, times=30)
summary(rmc1)
}
}
\references{
Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.
}
\seealso{
\code{\link{binreg}}, \code{\link{resmeanATE}}, \code{\link{rmstIPCW}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{hfactioncpx12}
\alias{hfactioncpx12}
\title{hfaction, subset of block randmized study HF-ACtion from WA package}
\format{
Randomized study 
status : 1-event, 2-death, 0-censoring
treatment : 1/0
}
\source{
WA package, Connor et al. 2009
}
\description{
Data from HF-action trial slightly modified from WA package,
consisting of 741 nonischemic patients with baseline
cardiopulmonary test duration less than or equal to 12 minutes.
}
\examples{
data(hfactioncpx12)
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bptwin.R
\name{bptwin}
\alias{bptwin}
\alias{twinlm.time}
\alias{bptwin.time}
\title{Liability model for twin data}
\usage{
bptwin(
  x,
  data,
  id,
  zyg,
  DZ,
  group = NULL,
  num = NULL,
  weights = NULL,
  weights.fun = function(x) ifelse(any(x <= 0), 0, max(x)),
  strata = NULL,
  messages = 1,
  control = list(trace = 0),
  type = "ace",
  eqmean = TRUE,
  pairs.only = FALSE,
  samecens = TRUE,
  allmarg = samecens & !is.null(weights),
  stderr = TRUE,
  robustvar = TRUE,
  p,
  indiv = FALSE,
  constrain,
  varlink,
  ...
)
}
\arguments{
\item{x}{Formula specifying effects of covariates on the response.}

\item{data}{\code{data.frame} with one observation pr row. In
addition a column with the zygosity (DZ or MZ given as a factor) of
each individual much be
specified as well as a twin id variable giving a unique pair of
numbers/factors to each twin pair.}

\item{id}{The name of the column in the dataset containing the twin-id variable.}

\item{zyg}{The name of the column in the dataset containing the
zygosity variable.}

\item{DZ}{Character defining the level in the zyg variable
corresponding to the dyzogitic twins.}

\item{group}{Optional. Variable name defining group for interaction analysis (e.g., gender)}

\item{num}{Optional twin number variable}

\item{weights}{Weight matrix if needed by the chosen estimator (IPCW)}

\item{weights.fun}{Function defining a single weight each individual/cluster}

\item{strata}{Strata}

\item{messages}{Control amount of messages shown}

\item{control}{Control argument parsed on to the optimization routine. Starting values may be parsed as '\code{start}'.}

\item{type}{Character defining the type of analysis to be
performed. Should be a subset of "acde" (additive genetic factors, common
environmental factors, dominant
genetic factors, unique environmental factors).}

\item{eqmean}{Equal means (with type="cor")?}

\item{pairs.only}{Include complete pairs only?}

\item{samecens}{Same censoring}

\item{allmarg}{Should all marginal terms be included}

\item{stderr}{Should standard errors be calculated?}

\item{robustvar}{If TRUE robust (sandwich) variance estimates of the variance are used}

\item{p}{Parameter vector p in which to evaluate log-Likelihood and score function}

\item{indiv}{If TRUE the score and log-Likelihood contribution of each twin-pair}

\item{constrain}{Development argument}

\item{varlink}{Link function for variance parameters}

\item{...}{Additional arguments to lower level functions}
}
\description{
Liability-threshold model for twin data
}
\examples{
data(twinstut)
b0 <- bptwin(stutter~sex,
             data=droplevels(subset(twinstut,zyg\%in\%c("mz","dz"))),
             id="tvparnr",zyg="zyg",DZ="dz",type="ae")
summary(b0)
}
\seealso{
\code{\link{twinlm}}, \code{\link{twinlm.time}}, \code{\link{twinlm.strata}}, \code{\link{twinsim}}
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timereg-copy-package.R
\docType{data}
\name{bmt}
\alias{bmt}
\title{The Bone Marrow Transplant Data}
\format{
The data has 408 rows and 5 columns. \describe{ \item{cause}{a
numeric vector code.  Survival status. 1: dead from treatment related
causes, 2: relapse , 0: censored.} \item{time}{ a numeric vector. Survival
time.  } \item{platelet}{a numeric vector code. Plalelet 1: more than 100 x
\eqn{10^9} per L, 0: less.} \item{tcell}{a numeric vector. T-cell depleted
BMT 1:yes, 0:no.} \item{age}{a numeric vector code. Age of patient, scaled
and centered ((age-35)/15).} }
}
\source{
Simulated data
}
\description{
Bone marrow transplant data with 408 rows and 5 columns.
}
\examples{

data(bmt)
names(bmt)

}
\references{
NN
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Event.Split.R
\name{EventSplit}
\alias{EventSplit}
\title{Event split with two time-scales, time and gaptime}
\usage{
EventSplit(
  data,
  time = "time",
  status = "status",
  entry = "start",
  cuts = "cuts",
  name.id = "id",
  gaptime = NULL,
  gaptime.entry = NULL,
  cuttime = c("time", "gaptime"),
  cens.code = 0,
  order.id = TRUE
)
}
\arguments{
\item{data}{data to be split}

\item{time}{time variable.}

\item{status}{status variable.}

\item{entry}{name of entry variable.}

\item{cuts}{cuts variable or numeric cut (only one value)}

\item{name.id}{name of id variable.}

\item{gaptime}{gaptime variable.}

\item{gaptime.entry}{name of entry variable for gaptime.}

\item{cuttime}{to cut after time or gaptime}

\item{cens.code}{code for the censoring.}

\item{order.id}{order data after id and start.}
}
\description{
splits after cut times for the two time-scales.
}
\examples{
rr  <- data.frame(time=c(500,1000),start=c(0,500),status=c(1,1),id=c(1,1))
rr$gaptime <-  rr$time-rr$start
rr$gapstart <- 0

rr1 <- EventSplit(rr,cuts=600,cuttime="time",   gaptime="gaptime",gaptime.entry="gapstart")
rr2 <- EventSplit(rr1,cuts=100,cuttime="gaptime",gaptime="gaptime",gaptime.entry="gapstart")

dlist(rr1,start-time+status+gapstart+gaptime~id)
dlist(rr2,start-time+status+gapstart+gaptime~id)

}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.regression.R
\name{binregATE}
\alias{binregATE}
\alias{logitIPCWATE}
\alias{logitATE}
\alias{normalATE}
\title{Average Treatment Effect for Censored Competing Risks Data using Binomial Regression}
\usage{
binregATE(
  formula,
  data,
  cause = 1,
  time = NULL,
  beta = NULL,
  treat.model = ~+1,
  cens.model = ~+1,
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  se = TRUE,
  type = c("II", "I"),
  kaplan.meier = TRUE,
  cens.code = 0,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  outcome = c("cif", "rmst", "rmtl"),
  model = c("default", "logit", "exp", "lin"),
  Ydirect = NULL,
  typeATE = "II",
  ...
)
}
\arguments{
\item{formula}{A formula object specifying the outcome and covariates (see \code{coxph}). 
The first covariate should be the treatment variable coded as a factor.}

\item{data}{A data frame containing the variables in the formula.}

\item{cause}{Numeric scalar indicating the cause of interest for competing risks.}

\item{time}{Numeric scalar indicating the time point of interest.}

\item{beta}{Optional numeric vector of starting values for the coefficients.}

\item{treat.model}{A formula specifying the logistic treatment model given covariates 
(e.g., \code{treatment ~ covariate1 + covariate2}).}

\item{cens.model}{A formula specifying the censoring model. Only stratified Cox models 
without covariates are supported (e.g., \code{~ strata(group)}).}

\item{offset}{Optional numeric vector of offsets for the partial likelihood.}

\item{weights}{Optional numeric vector of weights for the score equations.}

\item{cens.weights}{Optional numeric vector of pre-calculated censoring weights. 
If \code{NULL}, weights are estimated internally.}

\item{se}{Logical. If \code{TRUE}, computes standard errors with IPCW adjustment. 
If \code{FALSE}, assumes IPCW weights are known.}

\item{type}{Character string. Either \code{"I"} (classic binomial regression) or 
\code{"II"} (adds augmentation term for efficiency).}

\item{kaplan.meier}{Logical. If \code{TRUE}, uses Kaplan-Meier for IPCW weights; 
otherwise uses \eqn{\exp(-\text{Baseline})}.}

\item{cens.code}{Numeric code representing censored observations in the status variable.}

\item{no.opt}{Logical. If \code{TRUE}, optimization is skipped and starting values are used.}

\item{method}{Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.}

\item{augmentation}{Optional numeric vector for augmenting binomial regression.}

\item{outcome}{Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function, 
\eqn{F(t|X)}), \code{"rmst"} (Restricted Mean Survival Time, \eqn{E(\min(T, t) | X)}), 
or \code{"rmtl"} (Restricted Mean Time Lost, \eqn{E(I(\epsilon=\text{cause})(t - \min(T,t)) | X)}).}

\item{model}{Character string. Link function for the outcome model: \code{"exp"} or 
\code{"lin"} (identity). For \code{"cif"}, \code{"logit"} is typically used.}

\item{Ydirect}{Optional numeric vector. Use this outcome Y with IPCW version instead of 
constructing one from \code{outcome}.}

\item{typeATE}{Character string. Either \code{"II"} (censoring augmentation of the 
estimating equation) or \code{"I"} (standard).}

\item{...}{Additional arguments passed to lower-level functions (e.g., \code{binreg} 
that fits the outcome model).}
}
\value{
An object of class \code{c("binreg", "ATE")} containing:
\item{coef}{Estimated coefficients from the outcome model.}
\item{riskDR}{Double-robust marginal risk estimates for each treatment level.}
\item{riskG}{G-formula marginal risk estimates for each treatment level.}
\item{difriskDR}{Difference in risks (ATE) using double-robust estimator.}
\item{difriskG}{Difference in risks (ATE) using G-formula estimator.}
\item{riskDR.iid, riskG.iid}{Influence functions for marginal risk estimates.}
\item{var, var.riskDR, var.riskG}{Variance-covariance matrices.}
\item{se.coef, se.riskDR, se.riskG}{Standard errors.}
}
\description{
Estimates the average treatment effect (ATE) \eqn{E(Y(1) - Y(0))} for censored 
competing risks data using binomial regression with Inverse Probability of 
Censoring Weighting (IPCW).
}
\details{
Under standard causal assumptions, the ATE can be estimated. These assumptions include:
\itemize{
  \item \strong{Consistency}: The observed outcome equals the potential outcome under the observed treatment.
  \item \strong{Ignorability}: \eqn{(Y(1), Y(0)) \perp A | X} (treatment assignment is independent of potential outcomes given covariates).
  \item \strong{Positivity}: All treatment levels have non-zero probability given covariates.
}

The first covariate in the competing risks regression model must be the treatment variable, 
which should be coded as a factor. If the factor has more than two levels, multinomial 
logistic regression (\code{mlogit}) is used for propensity score modeling. In the absence 
of censoring, this reduces to ordinary logistic regression.

The ATE is estimated using standard doubly robust estimating equations that are 
IPCW-censoring adjusted. As an alternative to binomial regression, 
\code{logitIPCWATE} provides an IPCW-weighted version of standard logistic regression.

When \code{typeATE = "II"}, the estimating equation is augmented with:
\deqn{ (A/\pi(X)) \int E( O(t) | T \geq t, S(X))/ G_c(t,S(X)) d \hat M_c(s) }
when estimating the mean outcome for the treated group.
}
\section{References}{

\itemize{
  \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
}
}

\examples{
data(bmt)
dfactor(bmt)  <-  ~.

brs <- binregATE(Event(time,cause)~tcell.f+platelet+age,bmt,time=50,cause=1,
  treat.model=tcell.f~platelet+age)
summary(brs)
head(brs$riskDR.iid)
head(brs$riskG.iid)

brsi <- binregATE(Event(time,cause)~tcell.f+tcell.f*platelet+tcell.f*age,bmt,time=50,cause=1,
  treat.model=tcell.f~platelet+age)
summary(brsi)
head(brs$riskDR.iid)
head(brs$riskG.iid)
}
\seealso{
\code{\link{binreg}}, \code{\link{logitIPCWATE}}, \code{\link{logitATE}}, 
  \code{\link{binregG}}

[kumarsim()] [kumarsimRCT()]
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mediator-survival.R
\name{mediatorSurv}
\alias{mediatorSurv}
\alias{BootmediatorSurv}
\title{Mediation analysis in survival context}
\usage{
mediatorSurv(
  survmodel,
  weightmodel,
  data = data,
  wdata = wdata,
  id = "id",
  silent = TRUE,
  ...
)
}
\arguments{
\item{survmodel}{with mediation model (binreg, aalenMets, phreg)}

\item{weightmodel}{mediation model}

\item{data}{for computations}

\item{wdata}{weighted data expansion for computations}

\item{id}{name of id variable, important for SE computations}

\item{silent}{to be silent}

\item{...}{Additional arguments to survival model}
}
\description{
Mediation analysis in survival context  with robust standard errors taking the weights into account
via influence function computations. Mediator and exposure must be factors.  This is based on numerical
derivative wrt parameters for weighting.  See vignette for more examples.
}
\examples{

n <- 400
dat <- kumarsimRCT(n,rho1=0.5,rho2=0.5,rct=2,censpar=c(0,0,0,0),
          beta = c(-0.67, 0.59, 0.55, 0.25, 0.98, 0.18, 0.45, 0.31),
    treatmodel = c(-0.18, 0.56, 0.56, 0.54),restrict=1)
dfactor(dat) <- dnr.f~dnr
dfactor(dat) <- gp.f~gp
drename(dat) <- ttt24~"ttt24*"
dat$id <- 1:n
dat$ftime <- 1

weightmodel <- fit <- glm(gp.f~dnr.f+preauto+ttt24,data=dat,family=binomial)
wdata <- medweight(fit,data=dat)

### fitting models with and without mediator
aaMss2 <- binreg(Event(time,status)~gp+dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)
aaMss22 <- binreg(Event(time,status)~dnr+preauto+ttt24+cluster(id),data=dat,time=50,cause=2)

### estimating direct and indirect effects (under strong strong assumptions) 
aaMss <- binreg(Event(time,status)~dnr.f0+dnr.f1+preauto+ttt24+cluster(id),
                data=wdata,time=50,weights=wdata$weights,cause=2)
## to compute standard errors , requires numDeriv
library(numDeriv)
ll <- mediatorSurv(aaMss,fit,data=dat,wdata=wdata)
summary(ll)
## not run bootstrap (to save time)
## bll <- BootmediatorSurv(aaMss,fit,data=dat,k.boot=500)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WA-estimands.R
\name{WA_reg}
\alias{WA_reg}
\title{While-Alive Regression for Recurrent Events}
\usage{
WA_reg(
  formula,
  data,
  time = NULL,
  cens.code = 0,
  cause = 1,
  death.code = 2,
  marks = NULL,
  ...,
  trans = 1
)
}
\arguments{
\item{formula}{Formula with regression design. The first covariate on the RHS must 
be the treatment factor. Can include other covariates and \code{cluster(id)}.}

\item{data}{Data frame.}

\item{time}{Time point \eqn{t} for estimation.}

\item{cens.code}{Censoring code.}

\item{cause}{Event cause code.}

\item{death.code}{Death code.}

\item{marks}{Marks for composite outcomes.}

\item{...}{Additional arguments passed to \code{binreg}.}

\item{trans}{Power transformation for the outcome (default 1).}
}
\value{
An object of class \code{"binreg"} containing coefficient estimates, 
  standard errors, confidence intervals, and influence functions for the 
  regression of the event rate per time alive.
}
\description{
Performs regression analysis for the "While-Alive" mean of events per time unit, 
defined as \eqn{Z(t) = N(\min(D,t)) / \min(D,t)}. This function models how covariates 
affect the rate of recurrent events per unit of time alive.
}
\details{
The estimation is based on IPCW (Inverse Probability of Censoring Weighting) and 
calls \code{binreg} after constructing the outcome variable. It supports double 
robust estimation if covariate augmentation is specified.
}
\examples{
data(hfactioncpx12)
hfactioncpx12$age <- rnorm(741)[hfactioncpx12$id] 
dtable(hfactioncpx12,~status)
## exp-link regression 
dd <- WA_reg(Event(entry,time,status)~treatment+age+cluster(id),data=hfactioncpx12,
                    time=2,death.code=2)
summary(dd)
}
\references{
Ragni, A., Martinussen, T., & Scheike, T. H. (2023). Nonparametric estimation of the Patient Weighted While-Alive Estimand. arXiv preprint.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timereg-copy-package.R
\docType{data}
\name{diabetes}
\alias{diabetes}
\title{The Diabetic Retinopathy Data}
\format{
This data frame contains the following columns: \describe{
\item{id}{a numeric vector. Patient code.} \item{agedx}{a numeric vector.
Age of patient at diagnosis.} \item{time}{a numeric vector. Survival time:
time to blindness or censoring.} \item{status}{ a numeric vector code.
Survival status. 1: blindness, 0: censored.} \item{trteye}{a numeric vector
code. Random eye selected for treatment. 1: left eye 2: right eye.}
\item{treat}{a numeric vector. 1: treatment 0: untreated.} \item{adult}{a
numeric vector code. 1: younger than 20, 2: older than 20.} }
}
\source{
Huster W.J. and Brookmeyer, R. and Self. S. (1989) MOdelling paired
survival data with covariates, Biometrics 45, 145-56.
}
\description{
The data was colleceted to test a laser treatment for delaying blindness in
patients with dibetic retinopathy. The subset of 197 patiens given in Huster
et al. (1989) is used.
}
\examples{

data(diabetes)
names(diabetes)

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blocksample.R
\name{blocksample}
\alias{blocksample}
\alias{dsample}
\title{Block sampling}
\usage{
blocksample(data, size, idvar = NULL, replace = TRUE, ...)
}
\arguments{
\item{data}{Data frame}

\item{size}{Size of samples}

\item{idvar}{Column defining the clusters}

\item{replace}{Logical indicating wether to sample with replacement}

\item{\dots}{additional arguments to lower level functions}
}
\value{
\code{data.frame}
}
\description{
Sample blockwise from clustered data
}
\details{
Original id is stored in the attribute 'id'
}
\examples{

d <- data.frame(x=rnorm(5), z=rnorm(5), id=c(4,10,10,5,5), v=rnorm(5))
(dd <- blocksample(d,size=20,~id))
attributes(dd)$id

\dontrun{
blocksample(data.table::data.table(d),1e6,~id)
}


d <- data.frame(x=c(1,rnorm(9)),
               z=rnorm(10),
               id=c(4,10,10,5,5,4,4,5,10,5),
               id2=c(1,1,2,1,2,1,1,1,1,2),
               v=rnorm(10))
dsample(d,~id, size=2)
dsample(d,.~id+id2)
dsample(d,x+z~id|x>0,size=5)

}
\author{
Klaus K. Holst
}
\keyword{models}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{twostageMLE}
\alias{twostageMLE}
\title{Twostage Survival Model Fitted by Pseudo MLE}
\usage{
twostageMLE(
  margsurv,
  data = parent.frame(),
  theta = NULL,
  theta.des = NULL,
  var.link = 0,
  method = "NR",
  no.opt = FALSE,
  weights = NULL,
  se.cluster = NULL,
  ...
)
}
\arguments{
\item{margsurv}{Marginal model from \code{phreg}.}

\item{data}{Data frame.}

\item{theta}{Starting values for variance components.}

\item{theta.des}{Design for dependence parameters; when pairs are given, this could be a
(pairs) \eqn{\times} (number of parameters) \eqn{\times} (max number random effects) matrix.}

\item{var.link}{Link function for variance; if 1 then uses exponential link.}

\item{method}{Type of optimizer; default is Newton-Raphson \code{"NR"}.}

\item{no.opt}{To not optimize, for example to get score and IID for specific theta.}

\item{weights}{Cluster-specific weights, but given with length equivalent to data-set; weights for score equations.}

\item{se.cluster}{Specifies how the influence functions are summed before squared when computing the variance.
Note that the id from the marginal model is used to construct MLE, and then these scores can be summed with the \code{se.cluster} argument.}

\item{...}{Arguments to be passed to optimizer.}
}
\value{
An object of class \code{"mets.twostage"} containing:
\item{theta}{Estimated dependence parameters.}
\item{coef}{Coefficients.}
\item{var.theta}{Variance of theta parameters.}
\item{robvar.theta}{Robust variance of theta parameters.}
\item{theta.iid}{Influence functions for theta.}
\item{theta.iid.naive}{Naive influence functions for theta.}
\item{loglike}{Log-likelihood value.}
}
\description{
Fits Clayton-Oakes clustered survival data using marginals that are on Cox form
in the likelihood for the dependence parameter as in Glidden (2000).
}
\details{
The dependence can be modelled via a regression structure for the independent
gamma distributed random effects and their variances that may depend on cluster covariates.
So:
\deqn{ \theta = h( z_j^T \alpha) }
where \eqn{z} is specified by \code{theta.des}. The link function can be the exponential
when \code{var.link=1}.
}
\examples{
data(diabetes)
dd <- phreg(Surv(time, status == 1) ~ treat + cluster(id), diabetes)
oo <- twostageMLE(dd, data = diabetes)
summary(oo)

theta.des <- model.matrix(~ -1 + factor(adult), diabetes)
oo <- twostageMLE(dd, data = diabetes, theta.des = theta.des)
summary(oo)
}
\references{
\itemize{
\item Measuring early or late dependence for bivariate twin data. Scheike, Holst, Hjelmborg (2015), LIDA.
\item Twostage modelling of additive gamma frailty models for survival data. Scheike and Holst, working paper.
\item Shih and Louis (1995) Inference on the association parameter in copula models for bivariate survival data, \emph{Biometrics}.
\item Glidden (2000), A Two-Stage estimator of the dependence parameter for the Clayton Oakes model, LIDA.
}
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{haplo}
\alias{haplo}
\title{haplo fun data}
\format{
hapfreqs : haplo frequencies
haploX:  covariates and response for haplo survival discrete survival
ghaplos:  haplo-types for subjects of haploX data
}
\source{
Estimated data
}
\description{
haplo fun data
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{ttpd}
\alias{ttpd}
\title{ttpd discrete survival data on interval form}
\source{
Simulated data
}
\description{
ttpd discrete survival data on interval form
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{plack.cif}
\alias{plack.cif}
\alias{plack.cif2}
\title{plack Computes concordance for or.cif based model, that is Plackett random effects model}
\usage{
plack.cif(cif1, cif2, object)
}
\arguments{
\item{cif1}{Cumulative incidence of first argument.}

\item{cif2}{Cumulative incidence of second argument.}

\item{object}{or.cif object with dependence parameters.}
}
\description{
.. content for description (no empty lines) ..
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifetable.R
\name{lifetable.matrix}
\alias{lifetable.matrix}
\alias{lifetable}
\alias{lifetable.formula}
\title{Life table}
\usage{
\method{lifetable}{matrix}(x, strata = list(), breaks = c(),
   weights=NULL, confint = FALSE, ...)

 \method{lifetable}{formula}(x, data=parent.frame(), breaks = c(),
   weights=NULL, confint = FALSE, ...)
}
\arguments{
\item{x}{time formula (Surv) or matrix/data.frame with columns time,status or entry,exit,status}

\item{strata}{strata}

\item{breaks}{time intervals}

\item{weights}{weights variable}

\item{confint}{if TRUE 95\% confidence limits are calculated}

\item{...}{additional arguments to lower level functions}

\item{data}{data.frame}
}
\description{
Create simple life table
}
\examples{
library(timereg)
data(TRACE)

d <- with(TRACE,lifetable(Surv(time,status==9)~sex+vf,breaks=c(0,0.2,0.5,8.5)))
summary(glm(events ~ offset(log(atrisk))+factor(int.end)*vf + sex*vf,
            data=d,poisson))
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pch.R
\name{rpch}
\alias{rpch}
\alias{ppch}
\title{Piecewise constant hazard distribution}
\usage{
rpch(n, lambda = 1, breaks = c(0, Inf))
}
\arguments{
\item{n}{sample size}

\item{lambda}{rate parameters}

\item{breaks}{time cut-points}
}
\description{
Piecewise constant hazard distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Dbvn.R
\name{Dbvn}
\alias{Dbvn}
\title{Derivatives of the bivariate normal cumulative distribution function}
\usage{
Dbvn(p,design=function(p,...) {
     return(list(mu=cbind(p[1],p[1]),
               dmu=cbind(1,1),
               S=matrix(c(p[2],p[3],p[3],p[4]),ncol=2),
               dS=rbind(c(1,0,0,0),c(0,1,1,0),c(0,0,0,1)))  )},                 
     Y=cbind(0,0))
}
\arguments{
\item{p}{Parameter vector}

\item{design}{Design function with defines mean, derivative of mean, variance,
and derivative of variance with respect to the parameter p}

\item{Y}{column vector where the CDF is evaluated}
}
\description{
Derivatives of the bivariate normal cumulative distribution function
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dreg.R
\name{dreg}
\alias{dreg}
\title{Regression for data frames with dutility call}
\usage{
dreg(
  data,
  y,
  x = NULL,
  z = NULL,
  x.oneatatime = TRUE,
  x.base.names = NULL,
  z.arg = c("clever", "base", "group", "condition"),
  fun. = lm,
  summary. = summary,
  regex = FALSE,
  convert = NULL,
  doSummary = TRUE,
  special = NULL,
  equal = TRUE,
  test = 1,
  ...
)
}
\arguments{
\item{data}{data frame}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{name of variable, or fomula, or names of variables on data frame.}

\item{z}{name of variable, or fomula, or names of variables on data frame.}

\item{x.oneatatime}{x's one at a time}

\item{x.base.names}{base covarirates}

\item{z.arg}{what is Z, c("clever","base","group","condition"), clever decides based on type of Z, base means that Z is used as fixed baseline covaraites for all X, group means the analyses is done based on groups of Z, and condition means that Z specifies a condition on the data}

\item{fun.}{function  lm is default}

\item{summary.}{summary to use}

\item{regex}{regex}

\item{convert}{convert}

\item{doSummary}{doSummary or not}

\item{special}{special's}

\item{equal}{to do pairwise stuff}

\item{test}{development argument}

\item{...}{Additional arguments for fun}
}
\description{
Regression for data frames with dutility call
}
\examples{
##'
data(iris)
dat <- iris
drename(dat) <- ~.
names(dat)
set.seed(1)
dat$time <- runif(nrow(dat))
dat$time1 <- runif(nrow(dat))
dat$status <- rbinom(nrow(dat),1,0.5)
dat$S1 <- with(dat, Surv(time,status))
dat$S2 <- with(dat, Surv(time1,status))
dat$id <- 1:nrow(dat)

mm <- dreg(dat, "*.length"~"*.width"|I(species=="setosa" & status==1))
mm <- dreg(dat, "*.length"~"*.width"|species+status)
mm <- dreg(dat, "*.length"~"*.width"|species)
mm <- dreg(dat, "*.length"~"*.width"|species+status,z.arg="group")

\donttest{ ## Reduce Ex.Timings
y <- "S*"~"*.width"
xs <- dreg(dat, y, fun.=phreg)
xs <- dreg(dat, y, fun.=survdiff)

y <- "S*"~"*.width"
xs <- dreg(dat, y, x.oneatatime=FALSE, fun.=phreg)

## under condition
y <- S1~"*.width"|I(species=="setosa" & sepal.width>3)
xs <- dreg(dat, y, z.arg="condition", fun.=phreg)
xs <- dreg(dat, y, fun.=phreg)

## under condition
y <- S1~"*.width"|species=="setosa"
xs <- dreg(dat, y, z.arg="condition", fun.=phreg)
xs <- dreg(dat, y, fun.=phreg)

## with baseline  after |
y <- S1~"*.width"|sepal.length
xs <- dreg(dat, y, fun.=phreg)

## by group by species, not working
y <- S1~"*.width"|species
ss <- split(dat, paste(dat$species, dat$status))

xs <- dreg(dat, y, fun.=phreg)

## species as base, species is factor so assumes that this is grouping
y <- S1~"*.width"|species
xs <- dreg(dat, y, z.arg="base", fun.=phreg)

##  background var after | and then one of x's at at time
y <- S1~"*.width"|status+"sepal*"
xs <- dreg(dat, y, fun.=phreg)

##  background var after | and then one of x's at at time
##y <- S1~"*.width"|status+"sepal*"
##xs <- dreg(dat, y, x.oneatatime=FALSE, fun.=phreg)
##xs <- dreg(dat, y, fun.=phreg)

##  background var after | and then one of x's at at time
##y <- S1~"*.width"+factor(species)
##xs <- dreg(dat, y, fun.=phreg)
##xs <- dreg(dat, y, fun.=phreg, x.oneatatime=FALSE)

y <- S1~"*.width"|factor(species)
xs <- dreg(dat, y, z.arg="base", fun.=phreg)

y <- S1~"*.width"|cluster(id)+factor(species)
xs <- dreg(dat, y, z.arg="base", fun.=phreg)
xs <- dreg(dat, y, z.arg="base", fun.=coxph)

## under condition with groups
y <- S1~"*.width"|I(sepal.length>4)
xs <- dreg(subset(dat, species=="setosa"), y,z.arg="group",fun.=phreg)

## under condition with groups
y <- S1~"*.width"+I(log(sepal.length))|I(sepal.length>4)
xs <- dreg(subset(dat, species=="setosa"), y,z.arg="group",fun.=phreg)

y <- S1~"*.width"+I(dcut(sepal.length))|I(sepal.length>4)
xs <- dreg(subset(dat,species=="setosa"), y,z.arg="group",fun.=phreg)

ff <- function(formula,data,...) {
 ss <- survfit(formula,data,...)
 kmplot(ss,...)
 return(ss)
}

if (interactive()) {
dcut(dat) <- ~"*.width"
y <- S1~"*.4"|I(sepal.length>4)
par(mfrow=c(1, 2))
xs <- dreg(dat, y, fun.=ff)
}
}

}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{test_logrankRecurrent}
\alias{test_logrankRecurrent}
\alias{logrankRecurrentBase}
\title{Logrank-type test for comparing recurrent event marginal means between groups}
\usage{
test_logrankRecurrent(
  recurrent,
  death,
  weight = c("I", "II", "III"),
  km = TRUE,
  start = 0,
  stop = NULL,
  at.risk = 5,
  cluster.id = NULL,
  ...
)
}
\arguments{
\item{recurrent}{Either a \code{"recurrent"} object returned by
\code{\link{recurrent_marginal}}, or a \code{"phreg"} object for the
recurrent event model (in which case \code{death} must also be supplied).}

\item{death}{A \code{"phreg"} object for the terminal event model. Required
when \code{recurrent} is a \code{"phreg"} object; ignored otherwise.}

\item{weight}{Character string specifying the weight scheme: \code{"I"},
\code{"II"}, or \code{"III"}. Default is \code{"I"}. For competing risks \code{"III"} 
 gives Gray's test with robust standard errors.}

\item{km}{Logical. If \code{TRUE} (default), the Kaplan-Meier estimator is
used for the survival probability \eqn{S(t)}; otherwise the Nelson-Aalen
estimator is used.}

\item{start}{Left truncation time for the integration. Default is \code{0}.}

\item{stop}{Right truncation time for the integration. Defaults to the last
observed jump time.}

\item{at.risk}{Minimum combined risk-set size below which the weight is set
to zero. Default is \code{5}.}

\item{cluster.id}{Optional vector of cluster identifiers for aggregating
influence functions across clusters before forming the test statistic.}

\item{...}{Currently unused.}
}
\value{
An object of class \code{"estimate"} (from the \pkg{lava} package)
  with the following components:
  \item{coef}{The weighted difference in marginal means between groups.}
  \item{se}{Robust standard error of the test statistic.}
  \item{lower, upper}{95\% confidence interval bounds.}
  \item{p.value}{Two-sided p-value for the null hypothesis of no difference.}
  The object also carries an \code{iid} attribute containing the
  subject-level influence function decomposition of the test statistic,
  which can be used for further inference or combination with other estimators.
}
\description{
Tests whether the marginal mean number of recurrent events differs across
groups (strata), extending the classical logrank test to the setting of
recurrent events with a competing terminal event. The test statistic is
\deqn{z = \int_0^\tau w(s)\bigl[d\hat\mu_1(s) - d\hat\mu_2(s)\bigr],}
where \eqn{w(s)} is a weight function and \eqn{\hat\mu_j(s)} is the estimated
marginal mean for group \eqn{j}. Variance is estimated robustly via the
influence functions of Ghosh and Lin (2000).
}
\details{
Three weight schemes are available:
\describe{
  \item{\code{"I"}}{(Default) \eqn{w(t) = R_1(t) R_2(t) / (R_1(t) + R_2(t))},
    where \eqn{R_j(t) = Y_j(t) / \hat S_j(t-)}. Analogous to the standard
    logrank weight.}
  \item{\code{"II"}}{\eqn{w(t) = Y_j(t)}, the raw risk-set size. Equivalent
    to using observed counts without survival adjustment.}
  \item{\code{"III"}}{A modified weight incorporating the cumulative incidence,
    analogous to Gray's test for competing risks.}
}
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12

## Test using two separate phreg models
xr <- phreg(Surv(entry, time, status == 1) ~ strata(treatment) + cluster(id), data = hf)
dr <- phreg(Surv(entry, time, status == 2) ~ strata(treatment) + cluster(id), data = hf)
out <- test_logrankRecurrent(xr, dr, stop = 5)
summary(out)

## Equivalently, using a recurrent_marginal object directly
outN <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
                           data = hf, cause = 1, death.code = 2)
test_logrankRecurrent(outN)

}
\references{
Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
and death. \emph{Biometrics}, 56, 554--562.
}
\seealso{
\code{\link{recurrent_marginal}}, \code{\link{logrankRecurrentBase}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{simMultistate}
\alias{simMultistate}
\alias{extendCums}
\title{Simulation of illness-death model}
\usage{
simMultistate(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz,
  death.cumhaz2,
  rr = NULL,
  rr2 = NULL,
  rd = NULL,
  rd2 = NULL,
  gap.time = FALSE,
  max.recurrent = 100,
  dependence = 0,
  var.z = 0.22,
  cor.mat = NULL,
  cens = NULL,
  ...
)
}
\arguments{
\item{n}{number of id's}

\item{cumhaz}{cumulative hazard of going from state 1 to 2.}

\item{cumhaz2}{cumulative hazard of going from state 2 to 1.}

\item{death.cumhaz}{cumulative hazard of death from state 1.}

\item{death.cumhaz2}{cumulative hazard of death from state 2.}

\item{rr}{relative risk adjustment for cumhaz}

\item{rr2}{relative risk adjustment for cumhaz2}

\item{rd}{relative risk adjustment for death.cumhaz}

\item{rd2}{relative risk adjustment for death.cumhaz2}

\item{gap.time}{if true simulates gap-times with specified cumulative hazard}

\item{max.recurrent}{limits number recurrent events to 100}

\item{dependence}{0:independence; 1:all share same random effect with variance var.z; 2:random effect exp(normal) with correlation structure from cor.mat; 3:additive gamma distributed random effects, z1= (z11+ z12)/2 such that mean is 1 , z2= (z11^cor.mat(1,2)+ z13)/2, z3= (z12^(cor.mat(2,3)+z13^cor.mat(1,3))/2, with z11 z12 z13 are gamma with mean and variance 1 , first random effect is z1 and for N1 second random effect is z2 and for N2 third random effect is for death}

\item{var.z}{variance of random effects}

\item{cor.mat}{correlation matrix for var.z variance of random effects}

\item{cens}{rate of censoring exponential distribution}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Simulation of illness-death model
}
\details{
simMultistate with different death intensities from states 1 and 2 

Must give cumulative hazards on some time-range
}
\examples{
########################################
## getting some rates to mimick 
########################################
data(base1cumhaz)
data(base4cumhaz)
data(drcumhaz)
dr <- drcumhaz
dr2 <- drcumhaz
dr2[,2] <- 1.5*drcumhaz[,2]
base1 <- base1cumhaz
base4 <- base4cumhaz
cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

iddata <- simMultistate(10000,base1,base1,dr,dr2,cens=cens)
dlist(iddata,.~id|id<3,n=0)
 
### estimating rates from simulated data  
c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
###
par(mfrow=c(2,3))
bplot(c0)
lines(cens,col=2) 
bplot(c3,main="rates 1-> 3 , 2->3")
lines(dr,col=1,lwd=2)
lines(dr2,col=2,lwd=2)
###
bplot(c1,main="rate 1->2")
lines(base1,lwd=2)
###
bplot(c2,main="rate 2->1")
lines(base1,lwd=2)
 
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterindex-reshape.R
\name{cluster.index}
\alias{cluster.index}
\alias{countID}
\alias{pairRisk}
\alias{mystrata}
\alias{mystrata2index}
\title{Finds subjects related to same cluster}
\usage{
cluster.index(
  clusters,
  index.type = FALSE,
  num = NULL,
  Rindex = 0,
  mat = NULL,
  return.all = FALSE,
  code.na = NA
)
}
\arguments{
\item{clusters}{list of indeces}

\item{index.type}{if TRUE then already list of integers of index.type}

\item{num}{to get numbering according to num-type in separate columns}

\item{Rindex}{index starts with 1, in C is it is 0}

\item{mat}{to return matrix of indeces}

\item{return.all}{return all arguments}

\item{code.na}{how to code missing values}
}
\description{
Finds subjects related to same cluster
}
\examples{
i<-c(1,1,2,2,1,3)
d<- cluster.index(i)
print(d)

type<-c("m","f","m","c","c","c")
d<- cluster.index(i,num=type,Rindex=1)
print(d)
}
\references{
Cluster indeces
}
\seealso{
familycluster.index familyclusterWithProbands.index
}
\author{
Klaus Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WA-estimands.R
\name{evalTerminal}
\alias{evalTerminal}
\title{Evaluates piece constant covariates at min(D,t) where D is a terminal event}
\usage{
evalTerminal(
  formula,
  data = data,
  death.code = 2,
  time = NULL,
  marks = NULL,
  mark.codes = NULL
)
}
\arguments{
\item{formula}{formula with 'Event' outcome and X to evaluate at min(D,t)}

\item{data}{data frame}

\item{death.code}{codes for death (terminating event, 2 default)}

\item{time}{for evaluation}

\item{marks}{for terminal events to add marks*I(D <=t ,epsilon "in" mark.codes)  to X(min(D,t))}

\item{mark.codes}{gives death codes for which to add mark value}
}
\description{
returns X(min(D,t)) and min(D,t) and their ratio. for censored observation 0. 
to use with the IPCW models implemented.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/restricted.mean.R
\name{resmeanATE}
\alias{resmeanATE}
\alias{rmstATE}
\title{Average Treatment Effect for Restricted Mean Time}
\usage{
resmeanATE(formula, data, model = "exp", outcome = c("rmst", "rmtl"), ...)
}
\arguments{
\item{formula}{Formula with an \code{Event} outcome. The first covariate must be the treatment factor.}

\item{data}{Data frame.}

\item{model}{Link function: \code{"exp"} (exponential) or \code{"lin"} (identity).}

\item{outcome}{Outcome type: \code{"rmst"} or \code{"rmtl"}.}

\item{...}{Additional arguments passed to \code{binregATE}, such as \code{time}, \code{treat.model}, 
\code{augmentR0}, \code{augmentC}, etc.}
}
\value{
An object of class \code{"binregATE"} containing:
  \item{riskG}{Simple IPCW estimator results.}
  \item{riskDR}{Double Robust estimator results.}
  \item{riskG.iid, riskDR.iid}{Influence functions.}
  \item{coef}{Treatment effect estimates.}
  \item{se}{Standard errors.}
}
\description{
Estimates the Average Treatment Effect (ATE) for Restricted Mean Survival Time (RMST) 
or Restricted Mean Time Lost (RMTL) in censored competing risks data using IPCW.
}
\details{
Under standard causal assumptions (Consistency, Ignorability, Positivity), the ATE is 
estimated as \eqn{E(Y(1) - Y(0))}, where \eqn{Y(a)} is the potential outcome under treatment \eqn{a}.
The method uses double robust estimating equations that are IPCW-adjusted for censoring.

The first covariate in the formula must be the treatment effect (a factor). If the factor 
has more than two levels, multinomial logistic regression (mlogit) is used for propensity 
score modeling.
}
\examples{
data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell

out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, 
                  treat.model=tcell~platelet, outcome="rmtl")
summary(out)

out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40,
                   treat.model=tcell~platelet, outcome="rmtl")
summary(out1)

}
\references{
Scheike, T. and Holst, K. K. (2024). Restricted mean time lost for survival and competing risks data using mets in R. WIP.

Scheike, T. and Tanaka, S. (2025). Restricted mean time lost ratio regression: Percentage of restricted mean time lost due to specific cause. WIP.
}
\seealso{
\code{\link{binregATE}}, \code{\link{ratioATE}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{summaryGLM}
\alias{summaryGLM}
\alias{predictGLM}
\title{Reporting OR (exp(coef)) from glm with binomial link and glm predictions}
\usage{
summaryGLM(object, id = NULL, fun = NULL, ...)
}
\arguments{
\item{object}{glm output}

\item{id}{possible id for cluster corrected standard errors}

\item{fun}{possible function for non-standard predictions based on object}

\item{...}{arguments of estimate of lava for example level=0.95}
}
\description{
Reporting OR from glm with binomial link  and glm predictions
}
\examples{
data(sTRACE)
sTRACE$id <- sample(1:100,nrow(sTRACE),replace=TRUE)

model <- glm(I(status==9)~sex+factor(diabetes)+age,data=sTRACE,family=binomial)
summaryGLM(model)
summaryGLM(model,id=sTRACE$id)

nd <- data.frame(sex=c(0,1),age=67,diabetes=1)
predictGLM(model,nd)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aalenfrailty.R
\name{simAalenFrailty}
\alias{simAalenFrailty}
\title{Simulate from the Aalen Frailty model}
\usage{
simAalenFrailty(
  n = 5000,
  theta = 0.3,
  K = 2,
  beta0 = 1.5,
  beta = 1,
  cens = 1.5,
  cuts = 0,
  ...
)
}
\arguments{
\item{n}{Number of observations in each cluster}

\item{theta}{Dependence paramter (variance of frailty)}

\item{K}{Number of clusters}

\item{beta0}{Baseline (intercept)}

\item{beta}{Effect (log hazard ratio) of covariate}

\item{cens}{Censoring rate}

\item{cuts}{time cuts}

\item{...}{Additional arguments}
}
\description{
Simulate observations from Aalen Frailty model with Gamma
distributed frailty and constant intensity.
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{survival.iterative}
\alias{survival.iterative}
\title{Survival model for multivariate survival data}
\usage{
survival.iterative(
  margsurv,
  data = parent.frame(),
  method = "nr",
  Nit = 60,
  detail = 0,
  clusters = NULL,
  silent = 1,
  weights = NULL,
  control = list(),
  theta = NULL,
  theta.des = NULL,
  var.link = 1,
  iid = 1,
  step = 0.5,
  model = "clayton.oakes",
  marginal.trunc = NULL,
  marginal.survival = NULL,
  marginal.status = NULL,
  strata = NULL,
  se.clusters = NULL,
  max.clust = NULL,
  numDeriv = 0,
  random.design = NULL,
  pairs = NULL,
  pairs.rvs = NULL,
  numDeriv.method = "simple",
  additive.gamma.sum = NULL,
  var.par = 1,
  cr.models = NULL,
  case.control = 0,
  ascertained = 0,
  shut.up = 0
)
}
\arguments{
\item{margsurv}{Marginal model}

\item{data}{data frame}

\item{method}{Scoring method "nr", "nlminb", "optimize", "nlm"}

\item{Nit}{Number of iterations}

\item{detail}{Detail}

\item{clusters}{Cluster variable}

\item{silent}{Debug information}

\item{weights}{Weights}

\item{control}{Optimization arguments}

\item{theta}{Starting values for variance components}

\item{theta.des}{design for dependence parameters, when pairs are given this is could be a
(pairs) x (numer of parameters)  x (max number random effects) matrix}

\item{var.link}{Link function for variance}

\item{iid}{Calculate i.i.d. decomposition}

\item{step}{Step size}

\item{model}{model}

\item{marginal.trunc}{marginal left truncation probabilities}

\item{marginal.survival}{optional vector of marginal survival probabilities}

\item{marginal.status}{related to marginal survival probabilities}

\item{strata}{strata for fitting, see example}

\item{se.clusters}{for clusters for se calculation with iid}

\item{max.clust}{max se.clusters for se calculation with iid}

\item{numDeriv}{to get numDeriv version of second derivative, otherwise uses sum of squared score}

\item{random.design}{random effect design for additive gamma model, when pairs are given this is
a (pairs) x (2) x (max number random effects) matrix, see pairs.rvs below}

\item{pairs}{matrix with rows of indeces (two-columns) for the pairs considered in the pairwise
composite score, useful for case-control sampling when marginal is known.}

\item{pairs.rvs}{for additive gamma model and random.design and theta.des are given as arrays,
this specifice number of random effects for each pair.}

\item{numDeriv.method}{uses simple to speed up things and second derivative not so important.}

\item{additive.gamma.sum}{for two.stage=0, this is specification of the lamtot in the models via
a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number
of theta parameters), when null then sums all parameters.}

\item{var.par}{is 1 for the default parametrization with the variances of the random effects,
var.par=0 specifies that the \eqn{\lambda_j}'s are used as parameters.}

\item{cr.models}{competing risks models for two.stage=0, should be given as a list with models for each cause}

\item{case.control}{assumes case control structure for "pairs" with second column being the probands,
when this options is used the twostage model is profiled out via the paired estimating equations for the
survival model.}

\item{ascertained}{if the pair are sampled only when there is an event. This is in contrast to
case.control sampling where a proband is given. This can be combined with control probands. Pair-call
of twostage is needed  and second column of pairs are the first jump time with an event for ascertained pairs,
or time of control proband.}

\item{shut.up}{to make the program more silent in the context of iterative procedures for case-control
and ascertained sampling}
}
\description{
Fits additive gamma frailty model
with additive hazard condtional on the random effects
\deqn{
\lambda_{ij} = (V_{ij}^T Z) (X_{ij}^T \alpha(t))
}
The baseline \eqn{\alpha(t)} is profiled out using
marginal modelling adjusted for the random effects structure as in Eriksson and Scheike (2015).
One advantage of the standard frailty model is that one can deal with competing risks
for this model.

For all models the
standard errors do not reflect this uncertainty of the baseline estimates, and might therefore be a bit to small.
To remedy this one can do bootstrapping.

If clusters contain more than two times, we use a composite likelihood
based on the pairwise bivariate models. Can also fit a additive gamma random
effects model described in detail below.

We allow a regression structure for the indenpendent gamma distributed
random effects  and their variances that may depend on cluster covariates. So
\deqn{
 \theta = z_j^T \alpha
}
where \eqn{z} is specified by theta.des
The reported standard errors are based on the estimated information from the
likelihood assuming that the marginals are known.

Can also fit a structured additive gamma random effects model, such
as the ACE, ADE model for survival data.

Now random.design specificies the random effects for each subject within a cluster. This is
a matrix of 1's and 0's with dimension n x d.  With d random effects.
For a cluster with two subjects, we let the random.design rows be
 \eqn{v_1} and \eqn{v_2}.
Such that the random effects for subject
1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}.
By construction subjects 1's random effect are Gamma distributed with
mean \eqn{\lambda_j/v_1^T \lambda}
and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed over all clusters
as it would be for the ACE model below.
The lamtot parameter may be specified separately for some sets of the parameter
is the additive.gamma.sum (ags) matrix is specified and then lamtot for the
j'th random effect is \eqn{ags_j^T \lambda}.

Based on these parameters the relative contribution (the heritability, h) is
equivalent to  the expected values of the random effects  \eqn{\lambda_j/v_1^T \lambda}

The DEFAULT parametrization uses the variances of the random effecs
\deqn{
\theta_j  = \lambda_j/(v_1^T \lambda)^2
}
For alternative parametrizations one can specify how the parameters relate to \eqn{\lambda_j}
with the function

Given the random effects the survival distributions with a cluster are independent and
on the form
\deqn{
P(T > t| x,z) = exp( -Z  A(t) \exp( Z^t beta))
}

The parameters \eqn{(\lambda_1,...,\lambda_d)}
are related to the parameters of the model
by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
\eqn{\lambda} parameters
with the (k) underlying \eqn{\theta} parameters
\deqn{
\lambda = theta.des  \theta
}
here using theta.des to specify these low-dimension association. Default is a diagonal matrix.

The case.control option that can be used with the pair specification of the pairwise parts
of the estimating equations. Here it is assumed that the second subject of each pair is the
proband.
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pmvn.R
\name{pmvn}
\alias{pmvn}
\alias{pbvn}
\alias{loglikMVN}
\alias{scoreMVN}
\alias{dmvn}
\alias{rmvn}
\title{Multivariate normal distribution function}
\usage{
pmvn(lower, upper, mu, sigma, cor = FALSE)
}
\arguments{
\item{lower}{lower limits}

\item{upper}{upper limits}

\item{mu}{mean vector}

\item{sigma}{variance matrix or vector of correlation coefficients}

\item{cor}{if TRUE sigma is treated as standardized (correlation matrix)}
}
\description{
Multivariate normal distribution function
}
\examples{
lower <- rbind(c(0,-Inf),c(-Inf,0))
upper <- rbind(c(Inf,0),c(0,Inf))
mu <- rbind(c(1,1),c(-1,1))
sigma <- diag(2)+1
pmvn(lower=lower,upper=upper,mu=mu,sigma=sigma)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{twinstut}
\alias{twinstut}
\title{Stutter data set}
\format{
tvparnr: twin-pair id
zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
stutter: stutter status (yes/no)
age: age
nr: number within twin-pair
}
\description{
Based on nation-wide questionnaire answers from 33,317 Danish twins
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recreg.R
\name{twostageREC}
\alias{twostageREC}
\title{Fitting of Two-Stage Recurrent Events Random Effects Model}
\usage{
twostageREC(
  margsurv,
  recurrent,
  data = parent.frame(),
  theta = NULL,
  model = c("full", "shared", "non-shared"),
  ghosh.lin = NULL,
  theta.des = NULL,
  var.link = 0,
  method = "NR",
  no.opt = FALSE,
  weights = NULL,
  se.cluster = NULL,
  fnu = NULL,
  nufix = 0,
  nu = NULL,
  numderiv = 1,
  derivmethod = c("simple", "Richardson"),
  ...
)
}
\arguments{
\item{margsurv}{Marginal model for the terminal event (object of class \code{"phreg"}).}

\item{recurrent}{Marginal model for recurrent events (object of class \code{"phreg"} or \code{"recreg"}).}

\item{data}{Data frame used for fitting.}

\item{theta}{Starting value for total variance of gamma frailty.}

\item{model}{Model type: \code{"full"} (fully shared), \code{"shared"} (partly shared), or \code{"non-shared"}.}

\item{ghosh.lin}{Logical; if \code{TRUE}, forces use of Ghosh-Lin marginals based on the recurrent model.}

\item{theta.des}{Regression design for variance parameters.}

\item{var.link}{Link function for variance (1 for exponential).}

\item{method}{Optimization method (default "NR").}

\item{no.opt}{Logical; if \code{TRUE}, skips optimization.}

\item{weights}{Weights.}

\item{se.cluster}{Clusters for SE calculation (GEE style).}

\item{fnu}{Function to transform \eqn{\nu} (amount shared).}

\item{nufix}{Logical; if \code{TRUE}, fixes the amount shared.}

\item{nu}{Starting value for the amount shared.}

\item{numderiv}{Logical; if \code{TRUE}, uses numerical derivatives.}

\item{derivmethod}{Method for numerical derivative.}

\item{...}{Arguments for the optimizer.}
}
\value{
An object of class \code{"twostageREC"} containing:
\item{coef}{Estimated coefficients.}
\item{var}{Variance-covariance matrix.}
\item{theta}{Dependence parameters.}
\item{model}{Model type.}
}
\description{
Fits a two-stage random effects model for recurrent events with a terminal event.
Marginal models (Cox or Ghosh-Lin) are fitted first and passed to this function.
}
\details{
Supports:
\itemize{
\item Cox/Cox marginals.
\item Cox/Ghosh-Lin marginals.
\item Fully shared, partly shared, or non-shared random effects.
}
}
\references{
Scheike (2026), Two-stage recurrent events random effects models, LIDA, to appear.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mediator-survival.R
\name{medweight}
\alias{medweight}
\title{Computes mediation weights}
\usage{
medweight(
  fit,
  data = data,
  var = NULL,
  name.weight = "weights",
  id.name = "id",
  ...
)
}
\arguments{
\item{fit}{either glm-binomial or mlogit (mets package)}

\item{data}{data frame with data}

\item{var}{is NULL reads mediator and exposure from formulae in the fit.}

\item{name.weight}{name of weights}

\item{id.name}{name of id variable, important for SE computations}

\item{...}{Additional arguments to}
}
\description{
Computes mediation weights for either binary or multinomial mediators.
The important part is that the influence functions can be obtained to compute standard errors.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{resmean_phreg}
\alias{resmean_phreg}
\alias{rmst_phreg}
\title{Restricted Mean for Stratified Kaplan-Meier or Cox Model}
\usage{
resmean_phreg(x, times = NULL, covs = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{"phreg"}.}

\item{times}{Possible times for which to report restricted mean. If \code{NULL}, reports for all event times.}

\item{covs}{Possible covariates for Cox model adjustment.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"resmean_phreg"} containing:
\item{rmst}{Matrix of restricted mean survival times.}
\item{se.rmst}{Standard errors for RMST.}
\item{intkmtimes}{Restricted mean at specified times.}
\item{years.lost}{Years lost (if applicable).}
}
\description{
Computes the Restricted Mean Survival Time (RMST) for stratified Kaplan-Meier or
stratified Cox models with martingale standard errors.
}
\details{
The standard error is computed using linear interpolation between standard errors at jump-times.
This allows plotting the restricted mean as a function of time.

Years lost can be computed based on this and decomposed into years lost for different causes
using the \code{cif_yearslost} function.
}
\examples{
data(bmt)
bmt$time <- bmt$time + runif(408) * 0.001
out1 <- phreg(Surv(time, cause != 0) ~ strata(tcell, platelet), data = bmt)

rm1 <- resmean_phreg(out1, times = 10 * (1:6))
summary(rm1)
e1 <- estimate(rm1)
par(mfrow = c(1, 2))
plot(rm1, se = 1)
plot(rm1, years.lost = TRUE, se = 1)

## Comparing populations
rm1 <- resmean_phreg(out1, times = 40)
e1 <- estimate(rm1)
estimate(e1, rbind(c(1, -1, 0, 0)))
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WA-estimands.R
\name{WA_recurrent}
\alias{WA_recurrent}
\title{While-Alive Estimands for Recurrent Events}
\usage{
WA_recurrent(
  formula,
  data,
  time = NULL,
  cens.code = 0,
  cause = 1,
  death.code = 2,
  trans = NULL,
  cens.formula = NULL,
  augmentR = NULL,
  augmentC = NULL,
  type = NULL,
  marks = NULL,
  ...
)
}
\arguments{
\item{formula}{Formula with an \code{Event} object. The first covariate on the RHS 
must be a factor representing the treatment group. Can include \code{cluster(id)}.}

\item{data}{Data frame containing all variables referenced in the formula.}

\item{time}{Time point \eqn{t} for estimation. If NULL, defaults to the maximum event time.}

\item{cens.code}{Numeric code for censoring (default 0).}

\item{cause}{Numeric code for the recurrent event of interest (default 1).}

\item{death.code}{Numeric code for the terminal event/death (default 2).}

\item{trans}{Power transformation for the mean of events per time-unit (default NULL, i.e., linear).}

\item{cens.formula}{Formula for the censoring model. Default is \code{~strata(treatment)}.}

\item{augmentR}{Formula for covariate augmentation in the randomization model 
(e.g., \code{~age+sex}). Improves efficiency.}

\item{augmentC}{Formula for covariate augmentation in the censoring model. 
Enables double robustness.}

\item{type}{Type of augmentation for the binomial regression call. Default is "I" 
if \code{augmentC} is given, otherwise "II".}

\item{marks}{Optional marks for composite outcome situations (e.g., distinguishing 
event types in a composite endpoint).}

\item{...}{Additional arguments passed to \code{binregATE}.}
}
\value{
An object of class \code{"WA"} containing:
  \item{RAW}{List of raw estimates: RMST, mean number of events, ratio of means, 
    and their log-transformed versions with standard errors.}
  \item{ET}{List of estimated treatment effects: risk difference for the mean rate 
    (\code{riskDR}) and optionally the augmented version (\code{riskDRC}).}
  \item{time}{The time point used for estimation.}
  \item{cause, death.code, cens.code}{Codes used.}
  \item{augmentR, augmentC}{Formulas used for augmentation.}
  
  The object includes influence functions (IID) for all estimators, allowing for 
  further variance calculations or combination with other estimators.
}
\description{
Computes the "While-Alive" estimands for recurrent events in the presence of a 
terminal event (death). These estimands address the challenge of defining meaningful 
treatment effects when death prevents further observation of recurrent events.
}
\details{
The function estimates two primary quantities:
\enumerate{
  \item \strong{Ratio of Means}: \deqn{E(N(\min(D,t))) / E(\min(D,t))}
    The expected number of events up to time \eqn{t} (censored by death \eqn{D}) 
    divided by the expected time alive up to \eqn{t}.
  \item \strong{Mean of Events per Time Unit}: \deqn{E(N(\min(D,t)) / \min(D,t))}
    The expected rate of events per unit of time alive.
}

Estimation is based on Inverse Probability of Censoring Weighting (IPCW) to handle 
administrative censoring and death. The method can be augmented with covariates 
(double robust estimation) to improve efficiency and robustness.
}
\examples{
data(hfactioncpx12)
dtable(hfactioncpx12,~status)
dd <- WA_recurrent(Event(entry,time,status)~treatment+cluster(id),data=hfactioncpx12,
                   time=2,death.code=2)
summary(dd)
}
\references{
Ragni, A., Martinussen, T., & Scheike, T. H. (2023). Nonparametric estimation of the Patient Weighted While-Alive Estimand. arXiv preprint.

Mao, L. (2023). Nonparametric inference of general while-alive estimands for recurrent events. Biometrics, 79(3), 1749–1760.

Schmidli, H., Roger, J. H., & Akacha, M. (2023). Estimands for recurrent event endpoints in the presence of a terminal event. Statistics in Biopharmaceutical Research, 15(2), 238–248.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{dlag}
\alias{dlag}
\alias{dlag<-}
\title{Lag operator}
\usage{
dlag(data, x, k = 1, combine = TRUE, simplify = TRUE, names, ...)
}
\arguments{
\item{data}{data.frame or vector}

\item{x}{optional column names or formula}

\item{k}{lag (vector of integers)}

\item{combine}{combine results with original data.frame}

\item{simplify}{Return vector if possible}

\item{names}{optional new column names}

\item{...}{additional arguments to lower level functions}
}
\description{
Lag operator
}
\examples{
d <- data.frame(y=1:10,x=c(10:1))
dlag(d,k=1:2)
dlag(d,~x,k=0:1)
dlag(d$x,k=1)
dlag(d$x,k=-1:2, names=letters[1:4])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{haploX}
\alias{haploX}
\title{haploX  covariates and response for haplo survival discrete survival}
\source{
Simulated data
}
\description{
haploX  covariates and response for haplo survival discrete survival
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_cif}
\alias{sim_cif}
\alias{sim_cifs}
\alias{subdist}
\alias{sim_subdist}
\alias{invsubdist}
\title{Simulation of Output from Cumulative Incidence Regression Model}
\usage{
sim_cif(
  cif,
  n,
  data = NULL,
  Z = NULL,
  rr = NULL,
  strata = NULL,
  drawZ = TRUE,
  cens = NULL,
  rrc = NULL,
  entry = NULL,
  Sentry = NULL,
  cumstart = c(0, 0),
  U = NULL,
  pU = NULL,
  type = NULL,
  extend = NULL,
  ...
)
}
\arguments{
\item{cif}{Output from \code{prop.odds.subdist} or \code{ccr} (\code{cmprsk}), 
or call \code{invsubdist} with cumulative and linear predictor.}

\item{n}{Number of simulations.}

\item{data}{Data frame to extract covariates.}

\item{Z}{Design matrix instead of data.}

\item{rr}{Relative risks.}

\item{strata}{Strata vector.}

\item{drawZ}{Logical; randomly sample from Z.}

\item{cens}{Censoring specification.}

\item{rrc}{Relative risks for censoring.}

\item{entry}{Delayed entry time.}

\item{Sentry}{Survival related to delayed entry.}

\item{cumstart}{Start cumulatives at time 0.}

\item{U}{Uniforms for drawing timing.}

\item{pU}{Uniforms for drawing event type.}

\item{type}{Model type: \code{"logistic"}, \code{"cloglog"}, or \code{"rr"}.}

\item{extend}{Extend piecewise constant with constant rate.}

\item{...}{Arguments for \code{sim_subdist}.}
}
\value{
Data frame with simulated event times, status, and covariates.
}
\description{
Simulates data that looks like fit from a fitted cumulative incidence model 
(Fine-Gray or logistic).
}
\examples{
data(bmt)
nsim <- 100

## logit cumulative incidence regression model 
cif <- cifreg(Event(time,cause)~platelet+age,data=bmt,cause=1)
estimate(cif)  
plot(cif,col=1)
simbmt <- sim_cif(cif,nsim,data=bmt)
dtable(simbmt,~cause)
scif <- cifreg(Event(time,cause)~platelet+age,data=simbmt,cause=1)
estimate(scif)
plot(scif,add=TRUE,col=2)

## Fine-Gray cloglog cumulative incidence regression model 
cif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
estimate(cif)  
plot(cif,col=1)
simbmt <- sim_cif(cif,nsim,data=bmt)
scif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=simbmt,cause=1)
estimate(scif)
plot(scif,add=TRUE,col=2)

################################################################
#  simulating several causes with specific cumulatives 
################################################################
cif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
cif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=bmt,cause=2)
cifss <-  list(cif1,cif2)
simbmt <- sim_cifs(list(cif1,cif2),nsim,data=bmt,extend=0.005)
scif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=simbmt,cause=1)
scif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=simbmt,cause=2)
cbind(cif1$coef,scif1$coef)   
## can be off due to restriction F1+F2<= 1    
cbind(cif2$coef,scif2$coef)   
    
par(mfrow=c(1,2))   
## Cause 1 follows the model    
plot(cif1); plot(scif1,add=TRUE,col=1:2,lwd=2)
# Cause 2:second cause is modified with restriction to satisfy F1+F2<= 1, so scaled down     
plot(cif2); plot(scif2,add=TRUE,col=1:2,lwd=2)

}
\seealso{
[simul_cifs()]
}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daggregate.R
\name{daggregate}
\alias{daggregate}
\alias{daggr}
\title{aggregating for for data frames}
\usage{
daggregate(
  data,
  y = NULL,
  x = NULL,
  subset,
  ...,
  fun = "summary",
  regex = mets.options()$regex,
  missing = FALSE,
  remove.empty = FALSE,
  matrix = FALSE,
  silent = FALSE,
  na.action = na.pass,
  convert = NULL
)
}
\arguments{
\item{data}{data.frame}

\item{y}{name of variable, or formula, or names of variables on data frame.}

\item{x}{name of variable, or formula, or names of variables on data frame.}

\item{subset}{subset expression}

\item{...}{additional arguments to lower level functions}

\item{fun}{function defining aggregation}

\item{regex}{interpret x,y as regular expressions}

\item{missing}{Missing used in groups (x)}

\item{remove.empty}{remove empty groups from output}

\item{matrix}{if TRUE a matrix is returned instead of an array}

\item{silent}{suppress messages}

\item{na.action}{How model.frame deals with 'NA's}

\item{convert}{if TRUE try to coerce result into matrix. Can also be a user-defined function}
}
\description{
aggregating for for data frames
}
\examples{
data("sTRACE",package="timereg")
daggregate(iris, "^.e.al", x="Species", fun=cor, regex=TRUE)
daggregate(iris, Sepal.Length+Petal.Length ~Species, fun=summary)
daggregate(iris, log(Sepal.Length)+I(Petal.Length>1.5) ~ Species,
                 fun=summary)
daggregate(iris, "*Length*", x="Species", fun=head)
daggregate(iris, "^.e.al", x="Species", fun=tail, regex=TRUE)
daggregate(sTRACE, status~ diabetes, fun=table)
daggregate(sTRACE, status~ diabetes+sex, fun=table)
daggregate(sTRACE, status + diabetes+sex ~ vf+I(wmi>1.4), fun=table)
daggregate(iris, "^.e.al", x="Species",regex=TRUE)
dlist(iris,Petal.Length+Sepal.Length ~ Species |Petal.Length>1.3 & Sepal.Length>5,
            n=list(1:3,-(3:1)))
daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5))
daggregate(iris, I(Sepal.Length>7)~Species | I(Petal.Length>1.5),
                 fun=table)

dsum(iris, .~Species, matrix=TRUE, missing=TRUE)

par(mfrow=c(1,2))
data(iris)
drename(iris) <- ~.
daggregate(iris,'sepal*'~species|species!="virginica",fun=plot)
daggregate(iris,'sepal*'~I(as.numeric(species))|I(as.numeric(species))!=1,fun=summary)

dnumeric(iris) <- ~species
daggregate(iris,'sepal*'~species.n|species.n!=1,fun=summary)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{sim_recurrentTS}
\alias{sim_recurrentTS}
\title{Simulate recurrent events from a two-stage model with structured gamma frailties}
\usage{
sim_recurrentTS(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz = NULL,
  nu = rep(1, 3),
  share1 = 0.3,
  vargamD = 2,
  vargam12 = 0.5,
  gap.time = FALSE,
  max.recurrent = 100,
  cens = NULL,
  ...
)
}
\arguments{
\item{n}{Number of subjects to simulate.}

\item{cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the target
marginal cumulative rate of the first recurrent event type.}

\item{cumhaz2}{Two-column matrix \code{(time, cumhaz)} giving the target
marginal cumulative rate of the second recurrent event type.}

\item{death.cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the
cumulative hazard of the terminal event.}

\item{nu}{Numeric vector of length 3: the powers \eqn{(\nu_1, \nu_2, \nu_3)}
applied to the frailty components (see Details). Must satisfy
\eqn{\nu_j > -1/\text{shape}}. Default is \code{rep(1, 3)}.}

\item{share1}{Proportion of the total death frailty variance assigned to the
first component \eqn{Z_{d1}}. The remainder goes to \eqn{Z_{d2}}. Must be
in \eqn{(0, 1)}. Default is \code{0.3}.}

\item{vargamD}{Total variance of the death frailty \eqn{Z_\text{death}}.
Default is \code{2}.}

\item{vargam12}{Variance of the shared recurrent-event frailty \eqn{Z_{12}}.
Default is \code{0.5}.}

\item{gap.time}{Logical. If \code{TRUE}, event times are drawn as gap times
rather than calendar times. Default is \code{FALSE}.}

\item{max.recurrent}{Maximum number of recurrent events per subject. Default
is \code{100}.}

\item{cens}{Rate of exponential censoring. If \code{NULL} (default), no
administrative censoring is applied.}

\item{...}{Further arguments passed to lower-level simulation functions.}
}
\value{
A data frame in counting-process format (one row per event interval
  per subject) with columns \code{id}, \code{start}, \code{stop},
  \code{entry}, \code{time}, \code{status}, and \code{death}. Attributes
  store the (possibly adjusted) cumulative hazards used in simulation and
  the frailty parameters.
}
\description{
Simulates recurrent event data with two event types and a terminal event,
using a parametric two-stage frailty model. The construction ensures that the
marginal rates are approximately correct: conditional on survival,
\eqn{E(dN_j \mid D > t) \approx} \code{cumhazj}, and the hazard of death
equals \code{death.cumhaz}.
}
\details{
The frailty structure uses three gamma random variables \eqn{Z_{d1}},
\eqn{Z_{d2}}, \eqn{Z_{12}} to induce dependence:
\deqn{Z_\text{death} = Z_{d1} + Z_{d2}, \quad
      Z_1 = Z_{d1}^{\nu_1} Z_{12}, \quad
      Z_2 = Z_{d2}^{\nu_2} Z_{12}^{\nu_3}.}
The parameters \code{share1} and \code{vargamD} control how the death frailty
splits between the two components, and \code{vargam12} controls the shared
recurrent-event frailty. Setting \eqn{\nu = (1,1,1)} with \code{share1 = 0.5}
gives a symmetric structure; varying \eqn{\nu} allows asymmetric dependence.
}
\examples{
data(CPH_HPN_CRBSI)
dr    <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi
base4 <- CPH_HPN_CRBSI$mechanical

rr <- sim_recurrentTS(1000, base1, base4, death.cumhaz = dr)
dtable(rr, ~death + status)
mets:::showfitsim(causes = 2, rr, dr, base1, base4)

}
\seealso{
\code{\link{sim_recurrentII}}, \code{\link{sim_recurrent_ts}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{back2timereg}
\alias{back2timereg}
\title{Convert to timereg object}
\usage{
back2timereg(obj)
}
\arguments{
\item{obj}{no use}
}
\description{
convert to timereg object
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{survival.twostage}
\alias{survival.twostage}
\alias{twostage.aalen}
\alias{twostage.cox.aalen}
\alias{twostage.coxph}
\alias{twostage.phreg}
\alias{randomDes}
\alias{readmargsurv}
\title{Twostage survival model for multivariate survival data}
\usage{
survival.twostage(
  margsurv,
  data = parent.frame(),
  method = "nr",
  detail = 0,
  clusters = NULL,
  silent = 1,
  weights = NULL,
  theta = NULL,
  theta.des = NULL,
  var.link = 1,
  baseline.iid = 1,
  model = "clayton.oakes",
  marginal.trunc = NULL,
  marginal.survival = NULL,
  strata = NULL,
  se.clusters = NULL,
  numDeriv = 1,
  random.design = NULL,
  pairs = NULL,
  dim.theta = NULL,
  numDeriv.method = "simple",
  additive.gamma.sum = NULL,
  var.par = 1,
  no.opt = FALSE,
  ...
)
}
\arguments{
\item{margsurv}{Marginal model}

\item{data}{data frame}

\item{method}{Scoring method "nr", for lava NR optimizer}

\item{detail}{Detail}

\item{clusters}{Cluster variable}

\item{silent}{Debug information}

\item{weights}{Weights}

\item{theta}{Starting values for variance components}

\item{theta.des}{design for dependence parameters, when pairs are given the indeces of the
theta-design for this pair, is given in pairs as column 5}

\item{var.link}{Link function for variance:  exp-link.}

\item{baseline.iid}{to adjust for baseline estimation, using phreg function on same data.}

\item{model}{model}

\item{marginal.trunc}{marginal left truncation probabilities}

\item{marginal.survival}{optional vector of marginal survival probabilities}

\item{strata}{strata for fitting, see example}

\item{se.clusters}{for clusters for se calculation with iid}

\item{numDeriv}{to get numDeriv version of second derivative, otherwise uses sum of squared scores for each pair}

\item{random.design}{random effect design for additive gamma model, when pairs are given the
indeces of the pairs random.design rows are given as columns 3:4}

\item{pairs}{matrix with rows of indeces (two-columns) for the pairs considered in the pairwise
composite score, useful for case-control sampling when marginal is known.}

\item{dim.theta}{dimension of the theta parameter for pairs situation.}

\item{numDeriv.method}{uses simple to speed up things and second derivative not so important.}

\item{additive.gamma.sum}{for two.stage=0, this is specification of the lamtot in the models via
a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number
of theta parameters), when null then sums all parameters.}

\item{var.par}{is 1 for the default parametrization with the variances of the random effects,
var.par=0 specifies that the \eqn{\lambda_j}'s are used as parameters.}

\item{no.opt}{for not optimizng}

\item{...}{Additional arguments to maximizer NR of lava.
and ascertained sampling}
}
\description{
Fits Clayton-Oakes or bivariate Plackett models for bivariate survival data
using marginals that are on Cox form. The dependence can be modelled via
\enumerate{
\item  Regression design on dependence parameter.
\item  Random effects, additive gamma model.
}

If clusters contain more than two subjects, we use a composite likelihood
based on the pairwise bivariate models, for full MLE see twostageMLE.

The two-stage model is constructed such that
given the gamma distributed random effects it is assumed that the survival functions
are indpendent, and that the marginal survival functions are on Cox form (or additive form)
\deqn{
P(T > t| x) = S(t|x)= exp( -exp(x^T \beta) A_0(t) )
}

One possibility is to model the variance within clusters via a regression design, and
then one can specify a regression structure for the independent gamma distributed
random effect for each cluster, such that the variance is given by
\deqn{
 \theta = h( z_j^T \alpha)
}
where \eqn{z} is specified by theta.des, and a possible link function var.link=1 will
will use the exponential link \eqn{h(x)=exp(x)}, and var.link=0 the identity link \eqn{h(x)=x}.
The reported standard errors are based on the estimated information from the
likelihood assuming that the marginals are known (unlike the twostageMLE and for the
additive gamma model below).

Can also fit a structured additive gamma random effects model, such
as the ACE, ADE model for survival data.  In this case the
random.design specificies the random effects for each subject within a cluster. This is
a matrix of 1's and 0's with dimension n x d.  With d random effects.
For a cluster with two subjects, we let the random.design rows be
 \eqn{v_1} and \eqn{v_2}.
Such that the random effects for subject
1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}.
By construction subjects 1's random effect are Gamma distributed with
mean \eqn{\lambda_j/v_1^T \lambda}
and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed within clusters
as it would be for the ACE model below.

Based on these parameters the relative contribution (the heritability, h) is
equivalent to  the expected values of the random effects: \eqn{\lambda_j/v_1^T \lambda}

The DEFAULT parametrization (var.par=1) uses the variances of the random effecs
\deqn{
\theta_j  = \lambda_j/(v_1^T \lambda)^2
}
For alternative parametrizations one can specify how the parameters relate to \eqn{\lambda_j}
with the argument var.par=0.

For both types of models the basic model assumptions are that
given the random effects of the clusters the survival distributions within a cluster
are independent and ' on the form
\deqn{
P(T > t| x,z) = exp( -Z \cdot Laplace^{-1}(lamtot,lamtot,S(t|x)) )
}
with the inverse laplace of the gamma distribution with mean 1 and variance 1/lamtot.

The parameters \eqn{(\lambda_1,...,\lambda_d)} are related to the parameters of the model
by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
\eqn{\lambda} parameters
with the (k) underlying \eqn{\theta} parameters
\deqn{
\lambda = theta.des  \theta
}
here using theta.des to specify these low-dimension association. Default is a diagonal matrix.
This can be used to make structural assumptions about the variances of the random-effects
as is needed for the ACE model for example.

The case.control option that can be used with the pair specification of the pairwise parts
of the estimating equations. Here it is assumed that the second subject of each pair is the proband.
}
\examples{
data(diabetes)

# Marginal Cox model  with treat as covariate
margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
### Clayton-Oakes, MLE
fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
summary(fitco1)

### Plackett model
mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
fitp <- survival.twostage(mph,data=diabetes,theta=3.0,Nit=40,
               clusters=diabetes$id,var.link=1,model="plackett")
summary(fitp)

### Clayton-Oakes
fitco2 <- survival.twostage(mph,data=diabetes,theta=0.0,detail=0,
                 clusters=diabetes$id,var.link=1,model="clayton.oakes")
summary(fitco2)
fitco3 <- survival.twostage(margph,data=diabetes,theta=1.0,detail=0,
                 clusters=diabetes$id,var.link=0,model="clayton.oakes")
summary(fitco3)

### without covariates but with stratafied
marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
fitpa <- survival.twostage(marg,data=diabetes,theta=1.0,
                clusters=diabetes$id,model="clayton.oakes")
summary(fitpa)

fitcoa <- survival.twostage(marg,data=diabetes,theta=1.0,clusters=diabetes$id,
                 model="clayton.oakes")
summary(fitcoa)

### Piecewise constant cross hazards ratio modelling
########################################################

d <- subset(simClaytonOakes(2000,2,0.5,0,stoptime=2,left=0),!truncated)
udp <- piecewise.twostage(c(0,0.5,2),data=d,method="optimize",
                          id="cluster",timevar="time",
                          status="status",model="clayton.oakes",silent=0)
summary(udp)

\donttest{ ## Reduce Ex.Timings
### Same model using the strata option, a bit slower
########################################################
## makes the survival pieces for different areas in the plane
##ud1=surv.boxarea(c(0,0),c(0.5,0.5),data=d,id="cluster",timevar="time",status="status")
##ud2=surv.boxarea(c(0,0.5),c(0.5,2),data=d,id="cluster",timevar="time",status="status")
##ud3=surv.boxarea(c(0.5,0),c(2,0.5),data=d,id="cluster",timevar="time",status="status")
##ud4=surv.boxarea(c(0.5,0.5),c(2,2),data=d,id="cluster",timevar="time",status="status")

## everything done in one call
ud <- piecewise.data(c(0,0.5,2),data=d,timevar="time",status="status",id="cluster")
ud$strata <- factor(ud$strata);
ud$intstrata <- factor(ud$intstrata)

## makes strata specific id variable to identify pairs within strata
## se's computed based on the id variable across strata "cluster"
ud$idstrata <- ud$id+(as.numeric(ud$strata)-1)*2000

marg2 <- timereg::aalen(Surv(boxtime,status)~-1+factor(num):factor(intstrata),
               data=ud,n.sim=0,robust=0)
tdes <- model.matrix(~-1+factor(strata),data=ud)
fitp2 <- survival.twostage(marg2,data=ud,se.clusters=ud$cluster,clusters=ud$idstrata,
                model="clayton.oakes",theta.des=tdes,step=0.5)
summary(fitp2)

### now fitting the model with symmetry, i.e. strata 2 and 3 same effect
ud$stratas <- ud$strata;
ud$stratas[ud$strata=="0.5-2,0-0.5"] <- "0-0.5,0.5-2"
tdes2 <- model.matrix(~-1+factor(stratas),data=ud)
fitp3 <- survival.twostage(marg2,data=ud,clusters=ud$idstrata,se.cluster=ud$cluster,
                model="clayton.oakes",theta.des=tdes2,step=0.5)
summary(fitp3)

### same model using strata option, a bit slower
fitp4 <- survival.twostage(marg2,data=ud,clusters=ud$cluster,se.cluster=ud$cluster,
                model="clayton.oakes",theta.des=tdes2,step=0.5,strata=ud$strata)
summary(fitp4)
}

\donttest{ ## Reduce Ex.Timings
### structured random effects model additive gamma ACE
### simulate structured two-stage additive gamma ACE model
data <- simClaytonOakes.twin.ace(4000,2,1,0,3)
out <- twin.polygen.design(data,id="cluster")
pardes <- out$pardes
pardes
des.rv <- out$des.rv
head(des.rv)
aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data,robust=0)
ts <- survival.twostage(aa,data=data,clusters=data$cluster,detail=0,
	       theta=c(2,1),var.link=0,step=0.5,
	       random.design=des.rv,theta.des=pardes)
summary(ts)
}

}
\references{
Twostage estimation of additive gamma frailty models for survival data.
Scheike (2019), work in progress

Shih and Louis (1995) Inference on the association parameter in copula models for bivariate
survival data, Biometrics, (1995).

Glidden (2000), A Two-Stage estimator of the dependence
parameter for the Clayton Oakes model, LIDA, (2000).

Measuring early or late dependence for bivariate twin data
Scheike, Holst, Hjelmborg (2015), LIDA

Estimating heritability for cause specific mortality based on twins studies
Scheike, Holst, Hjelmborg (2014), LIDA

Additive Gamma frailty models for competing risks data, Biometrics (2015)
Eriksson and Scheike (2015),
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{EVaddGam}
\alias{EVaddGam}
\title{Relative risk for additive gamma model}
\usage{
EVaddGam(theta, x1, x2, thetades, ags)
}
\arguments{
\item{theta}{theta}

\item{x1}{x1}

\item{x2}{x2}

\item{thetades}{thetades}

\item{ags}{ags}
}
\description{
Computes the relative risk for additive gamma model at time 0
}
\examples{
lam0 <- c(0.5,0.3)
pars <- c(1,1,1,1,0,1)
## genetic random effects, cause1, cause2 and overall
parg <- pars[c(1,3,5)]
## environmental random effects, cause1, cause2 and overall
parc <- pars[c(2,4,6)]

## simulate competing risks with two causes with hazards 0.5 and 0.3
## ace for each cause, and overall ace
out <- simCompete.twin.ace(10000,parg,parc,0,2,lam0=lam0,overall=1,all.sum=1)

## setting up design for running the model
mm <- familycluster.index(out$cluster)
head(mm$familypairindex,n=10)
pairs <- matrix(mm$familypairindex,ncol=2,byrow=TRUE)
tail(pairs,n=12)
#
kinship <- (out[pairs[,1],"zyg"]=="MZ")+ (out[pairs[,1],"zyg"]=="DZ")*0.5

# dout <- make.pairwise.design.competing(pairs,kinship,
#          type="ace",compete=length(lam0),overall=1)
# head(dout$ant.rvs)
## MZ
# dim(dout$theta.des)
# dout$random.design[,,1]
## DZ
# dout$theta.des[,,nrow(pairs)]
# dout$random.design[,,nrow(pairs)]
#
# thetades <- dout$theta.des[,,1]
# x <- dout$random.design[,,1]
# x
##EVaddGam(rep(1,6),x[1,],x[3,],thetades,matrix(1,18,6))

# thetades <- dout$theta.des[,,nrow(out)/2]
# x <- dout$random.design[,,nrow(out)/2]
##EVaddGam(rep(1,6),x[1,],x[4,],thetades,matrix(1,18,6))
}
\references{
Eriksson and Scheike (2015), Additive Gamma frailty models for competing risks data, Biometrics (2015)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recreg.R
\name{sim_GLcox}
\alias{sim_GLcox}
\title{Simulation of Two-Stage Recurrent Events Data}
\usage{
sim_GLcox(
  n,
  base1,
  drcumhaz,
  var.z = 0,
  r1 = NULL,
  rd = NULL,
  rc = NULL,
  fz = NULL,
  fdz = NULL,
  model = c("twostage", "frailty", "shared"),
  type = NULL,
  share = 1,
  cens = NULL,
  nmin = 100,
  nmax = 1000
)
}
\arguments{
\item{n}{Number of IDs.}

\item{base1}{Baseline cumulative hazard for recurrent events.}

\item{drcumhaz}{Baseline cumulative hazard for the terminal event.}

\item{var.z}{Variance of the gamma frailty.}

\item{r1}{Relative risk term for the recurrent event baseline.}

\item{rd}{Relative risk term for the terminal event.}

\item{rc}{Relative risk term for censoring.}

\item{fz}{Function for transformation of the frailty term.}

\item{fdz}{Function for transformation of the frailty term for death.}

\item{model}{Model type: \code{"twostage"}, \code{"frailty"}, or \code{"shared"}.}

\item{type}{Type of simulation (default depends on \code{model}).}

\item{share}{Proportion of shared random effects (for partially shared models).}

\item{cens}{Censoring rate for exponential censoring.}

\item{nmin}{Minimum number of points in the time grid (default 100).}

\item{nmax}{Maximum number of points in the time grid (default 1000).}
}
\value{
A data frame with simulated recurrent events and terminal events, including frailty terms.
}
\description{
Simulates data based on Cox/Cox or Cox/Ghosh-Lin structures for recurrent events with a terminal event.
}
\details{
\itemize{
\item \code{type=3}: Generates data from a Cox/Cox two-stage model.
\item \code{type=2}: Generates data from a Ghosh-Lin/Cox model.
}

If \code{var.z=0}, data is generated without dependence or frailty.
If \code{model="twostage"}, the default is to generate data from a Ghosh-Lin/Cox model.
If \code{type=3}, data is generated with marginal Cox models (Cox/Cox).

Simulation is based on a linear approximation of the hazard for two-stage models on a time grid.
The grid must be sufficiently fine.
}
\references{
Scheike (2025), Two-stage recurrent events random effects models, LIDA, to appear.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{recurrent_marginal}
\alias{recurrent_marginal}
\alias{recurrent_marginalAIPCW}
\alias{recurrentMarginalPhreg}
\alias{iidRecurrent}
\alias{recurrentMarginal}
\title{Marginal mean estimation for recurrent events with a terminal event}
\usage{
recurrent_marginal(formula, data, cause = 1, ..., death.code = 2, test = FALSE)

recurrentMarginal(formula, data, ...)
}
\arguments{
\item{formula}{A formula with an \code{\link{Event}} response on the left-hand
side, specifying entry time, exit time, and event status. The right-hand side
may include \code{cluster()} to identify subjects and \code{strata()} for a
stratified analysis. A \code{cluster()} term is required.}

\item{data}{A data frame containing all variables in \code{formula}.}

\item{cause}{Integer code(s) for the recurrent event of interest. Default is
\code{1}.}

\item{...}{Further arguments passed to \code{\link{phreg}}.}

\item{death.code}{Integer code(s) for the terminal event. Default is \code{2}.}

\item{test}{Logical. If \code{TRUE}, a logrank-type test comparing strata is
computed and stored as an attribute of the result. Default is \code{FALSE}.}
}
\value{
An object of class \code{"recurrent"} with the following components:
  \item{mu}{Estimated marginal mean \eqn{\mu(t)} at each jump time.}
  \item{se.mu}{Robust standard error of \code{mu}.}
  \item{times}{Jump times at which estimates are computed.}
  \item{St}{Marginal survival estimate \eqn{S(t)} at each jump time.}
  \item{cumhaz}{Two-column matrix of \code{(time, mu)}, suitable for plotting.}
  \item{se.cumhaz}{Two-column matrix of \code{(time, se.mu)}.}
  The object carries three attributes: \code{"logrank"} (the test result when
  \code{test = TRUE}, otherwise \code{NULL}), \code{"cause"}, and
  \code{"death.code"}.
}
\description{
Estimates the marginal mean number of recurrent events over time in the
presence of a competing terminal event (e.g. death), using the nonparametric
estimator of Ghosh and Lin (2000). Two proportional hazards models are fitted
internally—one for the recurrent event rate and one for the terminal
event—and combined to form the estimator
\deqn{\mu(t) = \int_0^t S(u-)\,dR(u),}
where \eqn{S(u)} is the marginal survival probability at the baseline covariate
level and \eqn{dR(u)} is the baseline recurrent event rate among survivors.
Robust (sandwich) standard errors are computed via the influence-function
approach of Ghosh and Lin (2000).
}
\details{
Jump times must be unique within each stratum. If ties are present, use
\code{\link{tie_breaker}} to resolve them before calling this function.
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)

## Fit nonparametric baseline models for recurrent events and death
xr <- phreg(Surv(entry, time, status == 1) ~ cluster(id), data = hf)
dr <- phreg(Surv(entry, time, status == 2) ~ cluster(id), data = hf)

par(mfrow = c(1, 3))
plot(dr, se = TRUE); title(main = "Death")
plot(xr, se = TRUE); title(main = "Recurrent events")

## Compare naive and robust standard errors for the recurrent event rate
rxr <- robust_phreg(xr, fixbeta = 1)
plot(rxr, se = TRUE, robust = TRUE, add = TRUE, col = 4)

## Marginal mean via formula interface
outN <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
                           data = hf, cause = 1, death.code = 2)
plot(outN, se = TRUE, col = 2, add = TRUE)
summary(outN, times = 1:5)

## Stratified analysis with logrank test
out <- recurrent_marginal(Event(entry, time, status) ~ strata(treatment) + cluster(id),
                          data = hf, cause = 1, death.code = 2, test = TRUE)
plot(out, se = TRUE, ylab = "Marginal mean", col = 1:2)
attr(out, "logrank")
summary(out, times = 1:5)

## Influence-function (iid) decomposition at a fixed time point
head(iid(outN, time = 3))

}
\references{
Cook, R. J. and Lawless, J. F. (1997). Marginal analysis of recurrent events
and a terminating event. \emph{Statistics in Medicine}, 16, 911--924.

Ghosh, D. and Lin, D. Y. (2000). Nonparametric analysis of recurrent events
and death. \emph{Biometrics}, 56, 554--562.
}
\seealso{
\code{\link{test_logrankRecurrent}}, \code{\link{tie_breaker}},
  \code{\link{prob_exceed_recurrent}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{easy.survival.twostage}
\alias{easy.survival.twostage}
\title{Wrapper for easy fitting of Clayton-Oakes or bivariate Plackett models for bivariate survival data}
\usage{
easy.survival.twostage(
  margsurv = NULL,
  data = parent.frame(),
  method = "nlminb",
  status = "status",
  time = "time",
  entry = NULL,
  id = "id",
  Nit = 60,
  detail = 0,
  silent = 1,
  weights = NULL,
  control = list(),
  theta = NULL,
  theta.formula = NULL,
  desnames = NULL,
  deshelp = 0,
  var.link = 1,
  step = 0.5,
  model = "plackett",
  marginal.surv = NULL,
  strata = NULL,
  se.clusters = NULL
)
}
\arguments{
\item{margsurv}{model}

\item{data}{data frame}

\item{method}{Scoring method}

\item{status}{Status at exit time}

\item{time}{Exit time}

\item{entry}{Entry time}

\item{id}{name of cluster variable in data frame}

\item{Nit}{Number of iterations}

\item{detail}{Detail for more output for iterations}

\item{silent}{Debug information}

\item{weights}{Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.}

\item{control}{Optimization arguments}

\item{theta}{Starting values for variance components}

\item{theta.formula}{design for depedence, either formula or design function}

\item{desnames}{names for dependence parameters}

\item{deshelp}{if 1 then prints out some data sets that are used, on on which the design function operates}

\item{var.link}{Link function for variance (exp link)}

\item{step}{Step size for newton-raphson}

\item{model}{plackett or clayton-oakes model}

\item{marginal.surv}{vector of marginal survival probabilities}

\item{strata}{strata for fitting}

\item{se.clusters}{clusters for iid decomposition for roubst standard errors}
}
\description{
Fits two-stage model for describing depdendence in survival data
using marginals that are on cox or aalen form using the twostage funcion, but
call is different and easier and the data manipulation  build into the function.
Useful in particular for family design data.
}
\details{
If clusters contain more than two times, the algoritm uses a composite likelihood
based on the pairwise bivariate models.

The reported standard errors are based on the estimated information from the
likelihood assuming that the marginals are known.
}
\examples{
library(mets)
data("prt",package="mets")
prtsam <- blocksample(prt,idvar="id",1e3,replace=FALSE)
margp <- coxph(Surv(time,status==1)~factor(country),data=prtsam)
fitco <- survival.twostage(margp,data=prtsam,clusters=prtsam$id)
summary(fitco)

des <- model.matrix(~-1+factor(zyg),data=prtsam);
fitco <- survival.twostage(margp,data=prtsam,theta.des=des,clusters=prtsam$id)
summary(fitco)
rm(prtsam)

dfam <- simSurvFam(1000)
dfam <- fast.reshape(dfam,var=c("x","time","status"))

desfs <- function(x,num1="num1",num2="num2")
{
pp <- (x[num1]=="m")*(x[num2]=="f")*1   ## mother-father
pc <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1 ## mother-child
cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1               ## child-child
c(pp,pc,cc)
}

marg <- coxph(Surv(time,status)~factor(num),data=dfam)
out3 <- easy.survival.twostage(marg,data=dfam,time="time",status="status",id="id",
             deshelp=0,
             method="nr",theta.formula=desfs,
             model="plackett",
             desnames=c("parent-parent","parent-child","child-cild"))
summary(out3)

}
\keyword{survival}
\keyword{twostage}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gofM.phreg}
\alias{gofM.phreg}
\title{GOF for Cox covariates in  PH regression}
\usage{
gofM.phreg(
  formula,
  data,
  offset = NULL,
  weights = NULL,
  modelmatrix = NULL,
  n.sim = 1000,
  silent = 1,
  ...
)
}
\arguments{
\item{formula}{formula for cox regression}

\item{data}{data for model}

\item{offset}{offset}

\item{weights}{weights}

\item{modelmatrix}{matrix for cumulating residuals}

\item{n.sim}{number of simulations for score processes}

\item{silent}{to keep it absolutely silent, otherwise timing estimate will be prduced for longer jobs.}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Cumulative residuals after model matrix for Cox PH regression
p-values based on Lin, Wei, Ying resampling.
}
\details{
That is, computes 
\deqn{
 U(t) = \int_0^t M^t d \hat M 
}
and resamples its asymptotic distribution. 

This will show if the residuals are consistent with the model. Typically,
M will be a design matrix for the continous covariates that gives for example
the quartiles, and then the plot will show if for the different quartiles of the covariate the risk
prediction is consistent over time  (time x covariate interaction).
}
\examples{
library(mets)
data(TRACE)
set.seed(1)
TRACEsam <- blocksample(TRACE,idvar="id",replace=FALSE,100)

dcut(TRACEsam)  <- ~. 
mm <- model.matrix(~-1+factor(wmicat.4),data=TRACEsam)
m1 <- gofM.phreg(Surv(time,status==9)~vf+chf+wmi,data=TRACEsam,modelmatrix=mm)
summary(m1)
if (interactive()) {
par(mfrow=c(2,2))
plot(m1)
}

m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,modelmatrix=mm) 
summary(m1)

## cumulative sums in covariates, via design matrix mm 
mm <- cumContr(TRACEsam$wmi,breaks=10,equi=TRUE)
m1 <- gofM.phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=TRACEsam,
		  modelmatrix=mm,silent=0)
summary(m1)

}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binregTSR.R
\name{binregTSR}
\alias{binregTSR}
\title{Two-Stage Randomization for Survival or Competing Risks Data}
\usage{
binregTSR(
  formula,
  data,
  cause = 1,
  time = NULL,
  cens.code = 0,
  response.code = NULL,
  augmentR0 = NULL,
  treat.model0 = ~+1,
  augmentR1 = NULL,
  treat.model1 = ~+1,
  augmentC = NULL,
  cens.model = ~+1,
  estpr = c(1, 1),
  response.name = NULL,
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  beta = NULL,
  kaplan.meier = TRUE,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  outcome = c("cif", "rmst", "rmst-cause"),
  model = "exp",
  Ydirect = NULL,
  return.dataw = 0,
  pi0 = 0.5,
  pi1 = 0.5,
  cens.time.fixed = 1,
  outcome.iid = 1,
  meanCs = 0,
  ...
)
}
\arguments{
\item{formula}{Formula with outcome (see \code{coxph}), typically \code{Event(entry,time,status)~+1+cluster(id)}.}

\item{data}{Data frame containing all variables.}

\item{cause}{Cause of interest for competing risks (default 1).}

\item{time}{Time point for estimation.}

\item{cens.code}{Censoring code (default 0).}

\item{response.code}{Code of status indicating response at which 2nd randomization occurs.}

\item{augmentR0}{Covariates for augmentation model of the first randomization.}

\item{treat.model0}{Logistic treatment model for the first randomization.}

\item{augmentR1}{Covariates for augmentation model of the second randomization.}

\item{treat.model1}{Logistic treatment model for the second randomization.}

\item{augmentC}{Covariates for censoring augmentation model.}

\item{cens.model}{Stratification for censoring model based on observed covariates.}

\item{estpr}{Logical; estimate randomization probabilities using model (default TRUE).}

\item{response.name}{Name of response variable (reads from \code{treat.model1} if NULL).}

\item{offset}{Not implemented.}

\item{weights}{Not implemented.}

\item{cens.weights}{Can be provided externally.}

\item{beta}{Starting values for optimization.}

\item{kaplan.meier}{Logical; use Kaplan-Meier for censoring weights rather than exp cumulative hazard.}

\item{no.opt}{Not implemented.}

\item{method}{Not implemented.}

\item{augmentation}{Not implemented.}

\item{outcome}{Outcome type: \code{"cif"} (cumulative incidence), \code{"rmst"} (restricted mean survival time), 
or \code{"rmst-cause"} (restricted mean time lost for cause).}

\item{model}{Not implemented, uses linear regression for augmentation.}

\item{Ydirect}{Use this Y instead of outcome constructed inside the program.}

\item{return.dataw}{Logical; return weighted data for all treatment regimes.}

\item{pi0}{Known randomization probabilities for first randomization.}

\item{pi1}{Known randomization probabilities for second randomization.}

\item{cens.time.fixed}{Logical; use time-dependent weights for censoring estimation.}

\item{outcome.iid}{Logical; get iid contribution from outcome model.}

\item{meanCs}{Logical; indicates censoring augmentation is centered by \code{CensAugment.times/n}.}

\item{...}{Additional arguments to lower-level functions.}
}
\value{
An object of class \code{"binregTSR"} containing:
  \item{riskG}{Simple estimator results (coefficient, SE).}
  \item{riskG0}{First randomization augmentation results.}
  \item{riskG1}{Second randomization augmentation results.}
  \item{riskG01}{Both randomizations augmentation results.}
  \item{riskG.iid}{Influence functions for all estimators.}
  \item{varG}{Variance-covariance matrices.}
  \item{MGc}{Censoring martingale contributions.}
  \item{CensAugment.times}{Censoring augmentation terms.}
  \item{dynCens.coef}{Dynamic censoring coefficients.}
  \item{dataW}{Weighted data (if \code{return.dataw=TRUE}).}
}
\description{
Estimates the average treatment effect \eqn{E(Y(i,j))} of treatment regime \eqn{(i,j)} 
under two-stage randomization. The estimator can be augmented using information from 
both randomizations and dynamic censoring augmentation to improve efficiency.
}
\details{
The method solves the estimating equation:
\deqn{ \frac{I(\min(T_i,t) < G_i)}{G_c(\min(T_i,t))} I(T \leq t, \epsilon=1) - AUG_0 - AUG_1 + AUG_C - p(i,j) = 0 }
where:
\itemize{
  \item \eqn{AUG_0 = \frac{A_0(i) - \pi_0(i)}{\pi_0(i)} X_0 \gamma_0} uses covariates from \code{augmentR0}
  \item \eqn{AUG_1 = \frac{A_0(i)}{\pi_0(i)} \frac{A_1(j) - \pi_1(j)}{\pi_1(j)} X_1 \gamma_1} uses covariates from \code{augmentR1}
  \item \eqn{AUG_C = \int_0^t \gamma_c(s)^T (e(s) - \bar e(s)) \frac{1}{G_c(s)} dM_c(s)} is the censoring augmentation
}

Standard errors are estimated using the influence function of all estimators, enabling 
tests of differences to be computed subsequently. The method handles both survival 
data and competing risks data, and supports multiple treatment levels.
}
\examples{
ddf <- mets:::gsim(200,covs=1,null=0,cens=1,ce=2)

bb <- binregTSR(Event(entry,time,status)~+1+cluster(id),ddf$datat,time=2,cause=c(1),
        cens.code=0,treat.model0=A0.f~+1,treat.model1=A1.f~A0.f,
        augmentR1=~X11+X12+TR,augmentR0=~X01+X02,
        augmentC=~A01+A02+X01+X02+A11t+A12t+X11+X12+TR,
        response.code=2)
summary(bb) 
}
\references{
Scheike, T. H. (2024). Two-stage randomization analysis for survival data. mets package documentation.
}
\seealso{
\code{\link{binreg}}, \code{\link{phreg_rct}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterindex-reshape.R
\name{familyclusterWithProbands.index}
\alias{familyclusterWithProbands.index}
\title{Finds all pairs within a cluster (famly)  with the proband (case/control)}
\usage{
familyclusterWithProbands.index(
  clusters,
  probands,
  index.type = FALSE,
  num = NULL,
  Rindex = 1
)
}
\arguments{
\item{clusters}{list of indeces giving the clusters (families)}

\item{probands}{list of 0,1 where 1 specifices which of the subjects that are probands}

\item{index.type}{argument passed to other functions}

\item{num}{argument passed to other functions}

\item{Rindex}{index starts with 1, in C is it is 0}
}
\description{
second column of pairs are the probands and the first column the related subjects
}
\examples{
i<-c(1,1,2,2,1,3)
p<-c(1,0,0,1,0,1)
d<- familyclusterWithProbands.index(i,p)
print(d)
}
\references{
Cluster indeces
}
\seealso{
familycluster.index cluster.index
}
\author{
Klaus Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim.cox}
\alias{sim.cox}
\alias{read.fit}
\alias{sim.base}
\alias{simulate.cox}
\title{Simulation of output from Cox model.}
\usage{
sim.cox(cox,n,data=NULL,cens=NULL,rrc=NULL,entry=NULL,...)
}
\arguments{
\item{cox}{output form coxph or cox.aalen model fitting cox model.}

\item{n}{number of simulations.}

\item{data}{to extract covariates for simulations (draws from observed
covariates).}

\item{cens}{specifies censoring model, if "is.matrix" then uses cumulative
hazard given, if "is.scalar" then uses rate for exponential, and if not
given then takes average rate of in simulated data from cox model.}

\item{rrc}{possible vector of relative risk for cox-type censoring.}

\item{entry}{delayed entry variable for simulation.}

\item{...}{arguments for rchaz, for example entry-time}
}
\description{
Simulates data that looks like fit from Cox model. Censor data automatically
for highest value of the event times by using cumulative hazard.
}
\examples{

data(sTRACE)
cox <-  coxph(Surv(time,status==9)~vf+chf+wmi,data=sTRACE)
sim1 <- sim.cox(cox,1000,data=sTRACE)
cc <- coxph(Surv(time,status)~vf+chf+wmi,data=sim1)
cbind(cox$coef,cc$coef)

cor(sim1[,c("vf","chf","wmi")])
cor(sTRACE[,c("vf","chf","wmi")])

cox <-  phreg(Surv(time, status==9)~vf+chf+wmi,data=sTRACE)
sim3 <- sim.cox(cox,1000,data=sTRACE)
cc <-  phreg(Surv(time, status)~vf+chf+wmi,data=sim3)
cbind(cox$coef,cc$coef)
plot(cox,se=TRUE)
plot(cc,add=TRUE,col=2)

cox <-  phreg(Surv(time,status==9)~strata(chf)+vf+wmi,data=sTRACE)
sim3 <- sim.cox(cox,100,data=sTRACE)
cc <-   phreg(Surv(time, status)~strata(chf)+vf+wmi,data=sim3)
cbind(cox$coef,cc$coef)
plot(cox)
plot(cc,add=TRUE,col=2)

}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timereg-copy-package.R
\docType{data}
\name{melanoma}
\alias{melanoma}
\title{The Melanoma Survival Data}
\format{
This data frame contains the following columns: \describe{
\item{no}{ a numeric vector. Patient code. } \item{status}{ a numeric vector
code. Survival status. 1: dead from melanoma, 2: alive, 3: dead from other
cause. } \item{days}{ a numeric vector. Survival time. } \item{ulc}{ a
numeric vector code. Ulceration, 1: present, 0: absent. } \item{thick}{ a
numeric vector. Tumour thickness (1/100 mm). } \item{sex}{ a numeric vector
code. 0: female, 1: male. } }
}
\source{
Andersen, P.K., Borgan O, Gill R.D., Keiding N. (1993),
\emph{Statistical Models Based on Counting Processes}, Springer-Verlag.

Drzewiecki, K.T., Ladefoged, C., and Christensen, H.E. (1980), Biopsy and
prognosis for cutaneous malignant melanoma in clinical stage I. Scand. J.
Plast. Reconstru. Surg. 14, 141-144.
}
\description{
The melanoma data frame has 205 rows and 7 columns.  It contains data
relating to survival of patients after operation for malignant melanoma
collected at Odense University Hospital by K.T.  Drzewiecki.
}
\examples{

data(melanoma)
names(melanoma)

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cifreg.R
\name{cifregFG}
\alias{cifregFG}
\title{Fine-Gray Cumulative Incidence Function Regression}
\usage{
cifregFG(formula, data, propodds = NULL, ...)
}
\arguments{
\item{formula}{Formula with an 'Event' outcome.}

\item{data}{Data frame.}

\item{propodds}{Set to \code{NULL} to fit the Fine-Gray model (default behavior of this function).}

\item{...}{Additional arguments passed to \code{cifreg}.}
}
\value{
An object of class \code{"cifreg"} (extending \code{"phreg"}) with \code{propodds=NULL}.
}
\description{
Convenience wrapper for \code{cifreg} that specifically fits the Fine-Gray model
by setting \code{propodds=NULL}. This provides subdistribution hazard ratio interpretations.
}
\seealso{
\code{\link{cifreg}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{random.cif}
\alias{random.cif}
\title{Random effects model for competing risks data}
\usage{
random.cif(
  cif,
  data,
  cause = NULL,
  cif2 = NULL,
  cause1 = 1,
  cause2 = 1,
  cens.code = NULL,
  cens.model = "KM",
  Nit = 40,
  detail = 0,
  clusters = NULL,
  theta = NULL,
  theta.des = NULL,
  sym = 1,
  step = 1,
  same.cens = FALSE,
  var.link = 0,
  score.method = "nr",
  entry = NULL,
  trunkp = 1,
  ...
)
}
\arguments{
\item{cif}{a model object from the comp.risk function with the 
marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
odds the comparision is made with respect to}

\item{data}{a data.frame with the variables.}

\item{cause}{specifies the causes  related to the death
times, the value cens.code is the censoring value.}

\item{cif2}{specificies model for cause2 if different from cause1.}

\item{cause1}{cause of first coordinate.}

\item{cause2}{cause of second coordinate.}

\item{cens.code}{specificies the code for the censoring if NULL then uses the one from the marginal cif model.}

\item{cens.model}{specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"}

\item{Nit}{number of iterations for Newton-Raphson algorithm.}

\item{detail}{if 0 no details are printed during iterations, if 1 details are given.}

\item{clusters}{specifies the cluster structure.}

\item{theta}{specifies starting values for the cross-odds-ratio parameters of the model.}

\item{theta.des}{specifies a regression design for the cross-odds-ratio parameters.}

\item{sym}{1 for symmetry 0 otherwise}

\item{step}{specifies the step size for the Newton-Raphson algorith.m}

\item{same.cens}{if true then censoring within clusters are assumed to be the same variable, default is independent censoring.}

\item{var.link}{if var.link=1 then var is on log-scale.}

\item{score.method}{default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.}

\item{entry}{entry-age in case of delayed entry. Then two causes must be given.}

\item{trunkp}{gives probability of survival for delayed entry, and related to entry-ages given above.}

\item{...}{extra arguments.}
}
\value{
returns an object of type 'cor'. With the following arguments:
\item{theta}{estimate of proportional odds parameters of model.}
\item{var.theta}{variance for gamma.  }
\item{hess}{the derivative of the used score.}
\item{score}{scores at final stage.}
\item{score}{scores at final stage.}
\item{theta.iid}{matrix of iid decomposition of parametric effects.}
}
\description{
Fits a random effects  model describing the dependence in the cumulative 
incidence curves for subjects within a cluster.  Given the gamma distributed
random effects it is assumed that the cumulative incidence curves are indpendent, and
that the marginal cumulative incidence curves are on the form
\deqn{
P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) exp(z^T \beta))
}
We allow a regression structure for the random effects variances that may depend on
cluster covariates.
}
\examples{
\donttest{ ## Reduce Ex.Timings
 d <- simnordic.random(5000,delayed=TRUE,cordz=0.5,cormz=2,lam0=0.3,country=TRUE)
 times <- seq(50,90,by=10)
 add1 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
 times=times,cause=1,max.clust=NULL)

 ### making group indidcator 
 mm <- model.matrix(~-1+factor(zyg),d)

 out1<-random.cif(add1,data=d,cause1=1,cause2=1,theta=1,same.cens=TRUE)
 summary(out1)

 out2<-random.cif(add1,data=d,cause1=1,cause2=1,theta=1,
		   theta.des=mm,same.cens=TRUE)
 summary(out2)

#########################################
##### 2 different causes
#########################################

 add2 <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
                  times=times,cause=2,max.clust=NULL)
 out3 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,sym=1,same.cens=TRUE)
 summary(out3) ## negative dependence

 out4 <- random.cif(add1,data=d,cause1=1,cause2=2,cif2=add2,theta.des=mm,sym=1,same.cens=TRUE)
 summary(out4) ## negative dependence
}
}
\references{
A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
Scheike, Zhang, Sun, Jensen (2010), Biometrika. 

Cross odds ratio Modelling of dependence for
Multivariate Competing Risks Data, Scheike and Sun (2012), work in progress.
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{survivalG}
\alias{survivalG}
\alias{survivalGtime}
\title{G-Estimator for Cox and Fine-Gray Models}
\usage{
survivalG(
  x,
  data,
  time = NULL,
  Avalues = NULL,
  varname = NULL,
  same.data = TRUE,
  First = FALSE
)
}
\arguments{
\item{x}{Object of class \code{"phreg"} or \code{"cifreg"}.}

\item{data}{Data frame for risk averaging. Must be part of the data used for fitting unless \code{same.data=FALSE}.}

\item{time}{Time point for estimation.}

\item{Avalues}{Values to compare for the first covariate \eqn{A}.}

\item{varname}{Name of the variable to be treated as the treatment/exposure variable (default is the first variable).}

\item{same.data}{Logical; assumes the same data is used for fitting and averaging.}

\item{First}{Logical; if \code{TRUE}, uses only the first record for G-averaging (useful for start-stop structures).}
}
\value{
An object of class \code{"survivalG"} containing:
\item{risk}{Standardized risk estimates.}
\item{risk.iid}{Influence functions for the risk estimates.}
\item{difference}{Pairwise differences in risks.}
\item{ratio}{Risk ratios.}
\item{survival.ratio}{Survival ratios (for \code{phreg}).}
\item{survival.difference}{Survival differences (for \code{phreg}).}
}
\description{
Computes the G-estimator (G-formula) for standardized survival or cumulative incidence estimates:
\deqn{ \hat S(t, A=a) = n^{-1} \sum_i \hat S(t, A=a, Z_i) }
}
\details{
Based on a \code{phreg} or \code{cifreg} object. Provides influence functions for these risk estimates,
allowing for standard error computation.

If the first covariate is a factor, contrasts between all levels are computed automatically.
If it is continuous, specific values must be provided via \code{Avalues}.
}
\examples{
data(bmt)
bmt$time <- bmt$time + runif(408) * 0.001
bmt$event <- (bmt$cause != 0) * 1
bmt$id <- 1:408
dfactor(bmt) <- tcell.f ~ tcell

# Fine-Gray model
fg1 <- cifreg(Event(time, cause) ~ tcell.f + platelet + age, bmt,
              cause = 1, cox.prep = TRUE, propodds = NULL)
summary(survivalG(fg1, bmt, 50))

\donttest{ ## Reduce Ex.Timings
# Cox model
ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age, bmt)
summary(survivalG(ss, bmt, 50))

# Stratified Cox model
ss <- phreg(Surv(time, event) ~ strata(tcell.f) + platelet + age, bmt)
summary(survivalG(ss, bmt, 50))

# Time-varying G-estimates
sst <- survivalGtime(ss, bmt, n = 50)
plot(sst)

# Among treated (specify id to link influence functions)
ss <- phreg(Surv(time, event) ~ tcell.f + platelet + age + cluster(id), bmt)
summary(survivalG(ss, subset(bmt, tcell == 1), 50))
}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim.cif}
\alias{sim.cif}
\alias{sim.cifs}
\alias{simul.cifs}
\alias{setup.cif}
\alias{subdist}
\alias{simsubdist}
\alias{invsubdist}
\title{Simulation of output from Cumulative incidence regression model}
\usage{
sim.cif(
  cif,
  n,
  data = NULL,
  Z = NULL,
  rr = NULL,
  strata = NULL,
  drawZ = TRUE,
  cens = NULL,
  rrc = NULL,
  cumstart = c(0, 0),
  U = NULL,
  pU = NULL,
  type = NULL,
  extend = NULL,
  ...
)
}
\arguments{
\item{cif}{output form prop.odds.subdist or ccr (cmprsk), can also call invsubdist with 
with cumulative and linear predictor}

\item{n}{number of simulations.}

\item{data}{to extract covariates for simulations (draws from observed
covariates).}

\item{Z}{to use these covariates for simulation rather than drawing new ones.}

\item{rr}{possible vector of relative risk for cox model.}

\item{strata}{possible vector of strata}

\item{drawZ}{to random sample from Z or not}

\item{cens}{specifies censoring model, if "is.matrix" then uses cumulative
hazard given, if "is.scalar" then uses rate for exponential, and if not
given then takes average rate of in simulated data from cox model.}

\item{rrc}{possible vector of relative risk for cox-type censoring.}

\item{cumstart}{to start cumulatives at time 0 in 0.}

\item{U}{uniforms to use for drawing of timing for cumulative incidence.}

\item{pU}{uniforms to use for drawing event type (F1,F2,1-F1-F2).}

\item{type}{of model logistic,cloglog,rr}

\item{extend}{to extend piecewise constant with constant rate. Default is average rate over time from cumulative (when TRUE), if numeric then uses given rate.}

\item{...}{arguments for simsubdist (for example Uniform variable for realizations)}
}
\description{
Simulates data that looks like fit from fitted cumulative incidence model
}
\examples{
library(mets)
data(bmt)
nsim <- 100

## logit cumulative incidence regression model 
cif <- cifreg(Event(time,cause)~platelet+age,data=bmt,cause=1)
estimate(cif)  
plot(cif,col=1)
simbmt <- sim.cif(cif,nsim,data=bmt)
dtable(simbmt,~status)
scif <- cifreg(Event(time,status)~platelet+age,data=simbmt,cause=1)
estimate(scif)
plot(scif,add=TRUE,col=2)

## Fine-Gray cloglog cumulative incidence regression model 
cif <- cifregFG(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
estimate(cif)  
plot(cif,col=1)
simbmt <- sim.cif(cif,nsim,data=bmt)
scif <- cifregFG(Event(time,status)~strata(tcell)+age,data=simbmt,cause=1)
estimate(scif)
plot(scif,add=TRUE,col=2)

################################################################
#  simulating several causes with specific cumulatives 
################################################################
cif1 <-  cifreg(Event(time,cause)~strata(tcell)+age,data=bmt,cause=1)
cif2 <-  cifreg(Event(time,cause)~strata(platelet)+tcell+age,data=bmt,cause=2)
cifss <-  list(cif1,cif2)
simbmt <- sim.cifs(list(cif1,cif2),nsim,data=bmt,extend=0.005)
dtable(simbmt,~status)
scif1 <-  cifreg(Event(time,status)~strata(tcell)+age,data=simbmt,cause=1)
scif2 <-  cifreg(Event(time,status)~strata(platelet)+tcell+age,data=simbmt,cause=2)
cbind(cif1$coef,scif1$coef)   
## can be off due to restriction F1+F2<= 1    
cbind(cif2$coef,scif2$coef)   
    
par(mfrow=c(1,2))   
## Cause 1 follows the model    
plot(cif1); plot(scif1,add=TRUE,col=1:2,lwd=2)
# Cause 2:second cause is modified with restriction to satisfy F1+F2<= 1, so scaled down     
plot(cif2); plot(scif2,add=TRUE,col=1:2,lwd=2)
   
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gofZ.phreg}
\alias{gofZ.phreg}
\alias{cumContr}
\title{GOF for Cox covariates in  PH regression}
\usage{
gofZ.phreg(
  formula,
  data,
  vars = NULL,
  offset = NULL,
  weights = NULL,
  breaks = 50,
  equi = FALSE,
  n.sim = 1000,
  silent = 1,
  ...
)
}
\arguments{
\item{formula}{formula for cox regression}

\item{data}{data for model}

\item{vars}{which variables to test for linearity}

\item{offset}{offset}

\item{weights}{weights}

\item{breaks}{number of breaks for cumulatives in covarirate direction}

\item{equi}{equidistant breaks  or not}

\item{n.sim}{number of simulations for score processes}

\item{silent}{to keep it absolutely silent, otherwise timing estimate will be prduced for longer jobs.}

\item{...}{Additional arguments to lower level funtions}
}
\description{
That is, computes 
\deqn{
 U(z,\tau) = \int_0^\tau M(z)^t d \hat M 
}
and resamples its asymptotic distribution.
}
\details{
This will show if the residuals are consistent with the model evaulated in the z covariate. 
M is here chosen based on a grid (z_1, ..., z_m) and the different columns are \eqn{I(Z_i \leq z_l)}.
for \eqn{l=1,...,m}. 
The process in z is resampled to find extreme values.  The time-points of evuluation is by default
50 points, chosen as 2%,4%,..., percentiles of the covariates.

The p-value is valid but depends on the chosen grid. When the number of break points are high
this will give the orginal test of Lin, Wei and Ying for linearity, that is also computed in 
the timereg package.
}
\examples{
library(mets)
data(TRACE)
set.seed(1)
TRACEsam <- blocksample(TRACE,idvar="id",replace=FALSE,100)

## cumulative sums in covariates, via design matrix mm
\donttest{ ## Reduce Ex.Timings
m1 <- gofZ.phreg(Surv(time,status==9)~strata(vf)+chf+wmi+age,data=TRACEsam)
summary(m1) 
plot(m1,type="z")
}
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{simRecurrent}
\alias{simRecurrent}
\title{Simulation of recurrent events data based on cumulative hazards for event and death process}
\usage{
simRecurrent(
  n,
  cumhaz,
  death.cumhaz = NULL,
  r1 = NULL,
  rd = NULL,
  rc = NULL,
  ...
)
}
\arguments{
\item{n}{number of id's}

\item{cumhaz}{cumulative hazard of recurrent events}

\item{death.cumhaz}{cumulative hazard of death}

\item{r1}{potential relative risk adjustment of rate}

\item{rd}{potential relative risk adjustment of rate}

\item{rc}{potential relative risk adjustment of rate}

\item{...}{Additional arguments to simRecurrentList}
}
\description{
Simulation of recurrent events data based on cumulative hazards for event and death process
}
\examples{
########################################
## getting some rates to mimick 
########################################
library(mets)
data(CPH_HPN_CRBSI)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
base4 <- CPH_HPN_CRBSI$mechanical

######################################################################
### simulating simple model that mimicks data 
######################################################################
rr <- simRecurrent(5,base1)
dlist(rr,.~id,n=0)
rr <- simRecurrent(5,base1,death.cumhaz=dr)
dlist(rr,.~id,n=0)

rr <- simRecurrent(100,base1,death.cumhaz=dr)
par(mfrow=c(1,3))
showfitsim(causes=1,rr,dr,base1,base1)
######################################################################
### simulating simple model 
### random effect for all causes (Z shared for death and recurrent) 
######################################################################
rr <- simRecurrent(100,base1,death.cumhaz=dr,dependence=1,var.z=0.4)
dtable(rr,~death+status)

######################################################################
### now with two event types and second type has same rate as death rate
######################################################################
set.seed(100)
rr <- simRecurrentII(100,base1,base4,death.cumhaz=dr)
dtable(rr,~death+status)
par(mfrow=c(2,2))
showfitsim(causes=2,rr,dr,base1,base4)

######################################################################
### now with three event types and two causes of death 
######################################################################
set.seed(100)
cumhaz <- list(base1,base1,base4)
drl <- list(dr,base4)
rr <- simRecurrentList(100,cumhaz,death.cumhaz=drl,dependence=0)
dtable(rr,~death+status)
showfitsimList(rr,cumhaz,drl) 

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doubleFGR.R
\name{doubleFGR}
\alias{doubleFGR}
\alias{bplotdFG}
\alias{predictdFG}
\title{Double CIF Fine-Gray model with two causes}
\usage{
doubleFGR(formula, data, offset = NULL, weights = NULL, X2 = NULL, ...)
}
\arguments{
\item{formula}{formula with 'Event'}

\item{data}{data frame}

\item{offset}{offsets for cox model}

\item{weights}{weights for Cox score equations}

\item{X2}{specifies the regression design for second CIF model}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Estimation based on derived hazards and recursive estimating equations.
fits two parametrizations
1)
 \deqn{
F_1(t,X) = 1 - \exp( - \exp( X^T \beta ) \Lambda_1(t))
}
and
 \deqn{
F_2(t,X_2) = 1 - \exp( -  \exp( X_2^T \beta_2 ) \Lambda_2(t))
}
or restricted version
2)
 \deqn{
F_1(t,X) = 1 - \exp( -  \exp( X^T \beta ) \Lambda_1(t))
}
and
 \deqn{
F_2(t,X_2,X) = ( 1 - \exp(  - \exp( X_2^T \beta_2 ) \Lambda_2(t)) ) (1 - F_1(\infty,X))
}
}
\examples{
res <- 0
data(bmt)
bmt$age2 <- bmt$age
newdata <- bmt[1:19,]
if (interactive()) par(mfrow=c(5,3))

## same X1 and X2
pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,restrict=res)
if (interactive()) {
  bplotdFG(pr2,cause=1)
  bplotdFG(pr2,cause=2,add=TRUE)
}
pp21 <- predictdFG(pr2,newdata=newdata)
pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}
pp21 <- predictdFG(pr2)
pp22 <- predictdFG(pr2,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}

pr2 <- doubleFGR(Event(time,cause)~strata(platelet),data=bmt,restrict=res)
if (interactive()) {
  bplotdFG(pr2,cause=1)
  bplotdFG(pr2,cause=2,add=TRUE)
}
pp21 <- predictdFG(pr2,newdata=newdata)
pp22 <- predictdFG(pr2,,newdata=newdata,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}
pp21 <- predictdFG(pr2)
pp22 <- predictdFG(pr2,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}

## different X1 and X2
pr2 <- doubleFGR(Event(time,cause)~age+platelet+age2,data=bmt,X2=3,restrict=res)
if (interactive()) {
  bplotdFG(pr2,cause=1)
  bplotdFG(pr2,cause=2,add=TRUE)
}
pp21 <- predictdFG(pr2,newdata=newdata)
pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}
pp21 <- predictdFG(pr2)
pp22 <- predictdFG(pr2,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}

### uden X1
pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,X2=1:2,restrict=res)
if (interactive()) {
  bplotdFG(pr2,cause=1)
  bplotdFG(pr2,cause=2,add=TRUE)
}
pp21 <- predictdFG(pr2,newdata=newdata)
pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}
pp21 <- predictdFG(pr2)
p22 <- predictdFG(pr2,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}

### without X2
pr2 <- doubleFGR(Event(time,cause)~age+platelet,data=bmt,X2=0,restrict=res)
if (interactive()) {
  bplotdFG(pr2,cause=1)
  bplotdFG(pr2,cause=2,add=TRUE)
}
pp21 <- predictdFG(pr2,newdata=newdata)
pp22 <- predictdFG(pr2,newdata=newdata,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}
pp21 <- predictdFG(pr2)
pp22 <- predictdFG(pr2,cause=2)
if (interactive()) {
  plot(pp21)
  plot(pp22,add=TRUE,col=2)
}

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{ACTG175}
\alias{ACTG175}
\title{ACTG175, block randmized study from speff2trial package}
\format{
Randomized study
}
\source{
Hammer et al. 1996, speff2trial package.
}
\description{
Data from speff2trial
}
\examples{
data(ACTG175)
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{dcut}
\alias{dcut}
\alias{dcut<-}
\alias{dunique}
\alias{drm}
\alias{drm<-}
\alias{dnames}
\alias{dnames<-}
\alias{drename}
\alias{drename<-}
\alias{dkeep}
\alias{dkeep<-}
\alias{ddrop}
\alias{ddrop<-}
\title{Cutting, sorting, rm (removing), rename for data frames}
\usage{
dcut(
  data,
  y = NULL,
  x = NULL,
  breaks = 4,
  probs = NULL,
  equi = FALSE,
  regex = mets.options()$regex,
  sep = NULL,
  na.rm = TRUE,
  labels = NULL,
  all = FALSE,
  ...
)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{name of variable, or fomula, or names of variables on data frame.}

\item{breaks}{number of breaks, for variables or vector of break points,}

\item{probs}{groups defined from quantiles}

\item{equi}{for equi-spaced breaks}

\item{regex}{for regular expressions.}

\item{sep}{seperator for naming of cut names.}

\item{na.rm}{to remove NA for grouping variables.}

\item{labels}{to use for cut groups}

\item{all}{to do all variables, even when breaks are not unique}

\item{...}{Optional additional arguments}
}
\description{
Cut variables, if breaks are given these are used, otherwise cuts into 
using group size given by probs, or equispace groups on range. Default 
is equally sized groups if possible
}
\examples{
data("sTRACE",package="timereg")
sTRACE$age2 <- sTRACE$age^2
sTRACE$age3 <- sTRACE$age^3

mm <- dcut(sTRACE,~age+wmi)
head(mm)

mm <- dcut(sTRACE,catage4+wmi4~age+wmi)
head(mm)

mm <- dcut(sTRACE,~age+wmi,breaks=c(2,4))
head(mm)

mm <- dcut(sTRACE,c("age","wmi"))
head(mm)

mm <- dcut(sTRACE,~.)
head(mm)

mm <- dcut(sTRACE,c("age","wmi"),breaks=c(2,4))
head(mm)

gx <- dcut(sTRACE$age)
head(gx)


## Removes all cuts variables with these names wildcards
mm1 <- drm(mm,c("*.2","*.4"))
head(mm1)

## wildcards, for age, age2, age4 and wmi
head(dcut(mm,c("a*","?m*")))

## with direct asignment
drm(mm) <- c("*.2","*.4")
head(mm)

dcut(mm) <- c("age","*m*")
dcut(mm) <- ageg1+wmig1~age+wmi
head(mm)

############################
## renaming
############################

head(mm)
drename(mm, ~Age+Wmi) <- c("wmi","age")
head(mm)
mm1 <- mm

## all names to lower
drename(mm1) <- ~.
head(mm1)

## A* to lower
mm2 <-  drename(mm,c("A*","W*"))
head(mm2)
drename(mm) <- "A*"
head(mm)

dd <- data.frame(A_1=1:2,B_1=1:2)
funn <- function(x) gsub("_",".",x)
drename(dd) <- ~.
drename(dd,fun=funn) <- ~.
names(dd)
}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{hfaction_cpx12}
\alias{hfaction_cpx12}
\title{hfaction, subset of block randmized study HF-ACtion from WA package}
\format{
Randomized study
}
\source{
WA package, Connor et al. 2009
}
\description{
Data from HF-action trial slightly modified from WA package
}
\examples{
data(hfaction_cpx12)
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{LinSpline}
\alias{LinSpline}
\title{Simple linear spline}
\usage{
LinSpline(x, knots, num = TRUE, name = "Spline")
}
\arguments{
\item{x}{variable to make into spline}

\item{knots}{cut points}

\item{num}{to give names x1 x2 and so forth}

\item{name}{name of spline expansion name.1 name.2 and so forth}
}
\description{
Simple linear spline
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg_rct.R
\name{phreg_rct}
\alias{phreg_rct}
\title{Lu-Tsiatis More Efficient Log-Rank for Randomized Studies with Baseline Covariates}
\usage{
phreg_rct(
  formula,
  data,
  cause = 1,
  cens.code = 0,
  typesR = c("R0", "R1", "R01"),
  typesC = c("C", "dynC"),
  weights = NULL,
  augmentR0 = NULL,
  augmentR1 = NULL,
  augmentC = NULL,
  treat.model = ~+1,
  RCT = TRUE,
  treat.var = NULL,
  km = TRUE,
  level = 0.95,
  cens.model = NULL,
  estpr = 1,
  pi0 = 0.5,
  base.augment = FALSE,
  return.augmentR0 = FALSE,
  mlogit = FALSE,
  ...
)
}
\arguments{
\item{formula}{Formula with \code{Surv} or \code{Event} outcome (see \code{coxph}) and 
treatment variable (randomization 0/1). The treatment variable must be the first 
covariate on the right-hand side.}

\item{data}{Data frame containing all variables referenced in the formula.}

\item{cause}{Numeric code for the event of interest in competing risks or recurrent events.}

\item{cens.code}{Numeric code for censoring in competing risks or recurrent events.}

\item{typesR}{Character vector specifying augmentation types for randomization 
(options: "R0" for baseline, "R1" for post-baseline, "R01" for both).}

\item{typesC}{Character vector specifying augmentation types for censoring 
(options: "C" for static, "dynC" for dynamic).}

\item{weights}{Weights to be used for \code{phreg}.}

\item{augmentR0}{Formula for the first randomization augmentation (e.g., \code{~age+sex}).}

\item{augmentR1}{Formula for the second randomization augmentation (e.g., \code{~age+sex}).}

\item{augmentC}{Formula for the censoring augmentation (e.g., \code{~age+sex}).}

\item{treat.model}{Propensity score model formula (default is \code{~+1}, assuming RCT).}

\item{RCT}{Logical; if FALSE, uses propensity score adjustment for marginal model.}

\item{treat.var}{Variable indicating treatment times in two-stage randomization.}

\item{km}{Logical; use Kaplan-Meier for censoring weights (stratified on treatment).}

\item{level}{Confidence level for intervals (default 0.95).}

\item{cens.model}{Censoring model formula (default is \code{~strata(treatment)}).}

\item{estpr}{Numeric code (1/0); estimate propensity scores or not (default TRUE).}

\item{pi0}{Fixed propensity scores for randomizations (if not estimating).}

\item{base.augment}{Logical; covariate augment baselines (only for R0 augmentation).}

\item{return.augmentR0}{Logical; return augmentation data.}

\item{mlogit}{Logical; use multinomial logistic regression for propensity scores.}

\item{...}{Additional arguments passed to \code{phreg} function.}
}
\value{
An object of class \code{"phreg_rct"} containing:
  \item{coefs}{Coefficient estimates for the treatment effect.}
  \item{iid}{Influence function (IID) decomposition for variance estimation.}
  \item{AugR0, AugR1, AugCdyn, AugClt}{Augmentation terms for different strategies.}
  \item{cumhaz}{Cumulative hazards.}
  \item{var}{Variance-covariance matrix.}
  \item{se}{Standard errors.}
  \item{call}{Original function call.}
  \item{formula}{Formula used.}
  \item{data}{The data used (if requested).}
  
  The object includes results for different augmentation combinations (R0, R1, R01, C, dynC).
}
\description{
Efficient implementation of the Lu-Tsiatis improvement using baseline covariates, 
extended to competing risks and recurrent events. The results are almost equivalent 
to the \code{speffSurv} function of the \code{speff2trial} package in the survival case. 
A dynamic censoring augmentation regression is also computed to gain additional 
efficiency from the censoring augmentation. The function handles two-stage 
randomizations and recurrent events (start,stop) with cluster structure.
}
\details{
The method improves the efficiency of the log-rank test by utilizing auxiliary 
baseline covariates to reduce variance, particularly useful in randomized clinical 
trials (RCTs) where covariate adjustment can increase power.
}
\examples{
## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)
dfactor(data) <- Z.f~Z

out <- phreg_rct(Surv(time,status)~Z.f,data=data,augmentR0=~X,augmentC=~factor(Z):X)
summary(out)
}
\references{
Lu, T. and Tsiatis, A. A. (2008), Improving the efficiency of the log-rank test using auxiliary covariates, Biometrika, 95, 679--694.

Scheike, T. H., Nerstroem, C. and Martinussen, T. (2026), Randomized clinical trials and the proportional hazards model for recurrent events, TEST.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.regression.R
\name{BinAugmentCifstrata}
\alias{BinAugmentCifstrata}
\title{Augmentation for Binomial regression based on stratified NPMLE Cif (Aalen-Johansen)}
\usage{
BinAugmentCifstrata(
  formula,
  data = data,
  cause = 1,
  cens.code = 0,
  km = TRUE,
  time = NULL,
  weights = NULL,
  offset = NULL,
  ...
)
}
\arguments{
\item{formula}{formula with 'Event', strata model for CIF given by strata, and strataC specifies censoring strata}

\item{data}{data frame}

\item{cause}{of interest}

\item{cens.code}{code of censoring}

\item{km}{to use Kaplan-Meier}

\item{time}{of interest}

\item{weights}{weights for estimating equations}

\item{offset}{offsets for logistic regression}

\item{...}{Additional arguments to binreg function.}
}
\description{
Computes  the augmentation term for each individual as well as the sum
\deqn{
A = \int_0^t H(u,X) \frac{1}{S^*(u,s)} \frac{1}{G_c(u)} dM_c(u)
}
with 
\deqn{
H(u,X) = F_1^*(t,s) - F_1^*(u,s)
}
using a KM for \deqn{G_c(t)} and a working model for cumulative baseline
related to \deqn{F_1^*(t,s)} and \deqn{s} is strata, \deqn{S^*(t,s) = 1 - F_1^*(t,s) - F_2^*(t,s)}.
}
\details{
Standard errors computed under assumption of correct \deqn{G_c(s)} model.
}
\examples{
data(bmt)
dcut(bmt,breaks=2) <- ~age 
out1<-BinAugmentCifstrata(Event(time,cause)~platelet+agecat.2+
		  strata(platelet,agecat.2),data=bmt,cause=1,time=40)
summary(out1)

out2<-BinAugmentCifstrata(Event(time,cause)~platelet+agecat.2+
    strata(platelet,agecat.2)+strataC(platelet),data=bmt,cause=1,time=40)
summary(out2)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dprint.R
\name{dprint}
\alias{dprint}
\alias{dlist}
\alias{dhead}
\alias{dtail}
\title{list, head, print, tail}
\usage{
dprint(data, y = NULL, n = 0, ..., x = NULL)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{n}{Index of observations to print (default c(1:nfirst, n-nlast:nlast)}

\item{...}{Optional additional arguments (nfirst,nlast, and print options)}

\item{x}{possible group variable}
}
\description{
listing for data frames
}
\examples{
m <- lava::lvm(letters)
d <- lava::sim(m, 20)

dlist(d,~a+b+c)
dlist(d,~a+b+c|a<0 & b>0)
## listing all : 
dlist(d,~a+b+c|a<0 & b>0,n=0)
dlist(d,a+b+c~I(d>0)|a<0 & b>0)
dlist(d,.~I(d>0)|a<0 & b>0)
dlist(d,~a+b+c|a<0 & b>0, nlast=0)
dlist(d,~a+b+c|a<0 & b>0, nfirst=3, nlast=3)
dlist(d,~a+b+c|a<0 & b>0, 1:5)
dlist(d,~a+b+c|a<0 & b>0, -(5:1))
dlist(d,~a+b+c|a<0 & b>0, list(1:5,50:55,-(5:1)))
dprint(d,a+b+c ~ I(d>0) |a<0 & b>0, list(1:5,50:55,-(5:1)))
}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-docs.R
\name{mets-simulation}
\alias{mets-simulation}
\alias{simrchaz}
\alias{simul_cifs}
\alias{simlogitSurvd}
\alias{kumarsim}
\alias{kumarsimRCT}
\alias{sim_ClaytonOakes_family_ace}
\alias{sim_ClaytonOakes_twin_ace}
\alias{sim_Compete_twin_ace}
\alias{sim_Compete_simple}
\alias{sim_Frailty_simple}
\alias{sim_BinPlack}
\alias{sim_BinFam}
\alias{sim_BinFam2}
\alias{sim_binClaytonOakes_family_ace}
\alias{sim_binClaytonOakes_twin_ace}
\alias{sim_binClaytonOakes_pairs}
\alias{sim_bptwin}
\alias{sim_SurvFam}
\alias{sim_nordictwin}
\alias{sim_nordic_random}
\alias{corsim_prostate_random}
\title{Simulation Helper Functions}
\description{
Internal simulation functions used for generating data from various
survival, competing risks, frailty, and twin/family models. These
functions are primarily intended for use in examples and testing.
}
\section{Survival and Competing Risks}{

\describe{
  \item{\code{simrchaz}}{Simulate from cumulative hazard via inverse CDF.}
  \item{\code{simul_cifs}}{Simulate competing risks data from cumulative incidence functions.}
  \item{\code{simlogitSurvd}}{Simulate survival data using logistic model.}
  \item{\code{kumarsim}}{Simulate competing risks data (Kumaraswamy-type).}
  \item{\code{kumarsimRCT}}{Simulate RCT competing risks data (Kumaraswamy-type).}
}
}

\section{Clayton-Oakes and Frailty Models}{

\describe{
  \item{\code{sim_ClaytonOakes_family_ace}}{Simulate family data from Clayton-Oakes ACE model.}
  \item{\code{sim_ClaytonOakes_twin_ace}}{Simulate twin data from Clayton-Oakes ACE model.}
  \item{\code{sim_Compete_twin_ace}}{Simulate twin competing risks data with ACE frailty.}
  \item{\code{sim_Compete_simple}}{Simulate competing risks data with shared frailty.}
  \item{\code{sim_Frailty_simple}}{Simulate survival data with shared frailty.}
  \item{\code{sim_SurvFam}}{Simulate family survival data with shared frailty.}
}
}

\section{Binomial Twin/Family Models}{

\describe{
  \item{\code{sim_BinPlack}}{Simulate paired binary data (Plackett model).}
  \item{\code{sim_BinFam}}{Simulate binary family data.}
  \item{\code{sim_BinFam2}}{Simulate binary family data (alternative parameterization).}
  \item{\code{sim_binClaytonOakes_family_ace}}{Simulate binary family data (Clayton-Oakes ACE).}
  \item{\code{sim_binClaytonOakes_twin_ace}}{Simulate binary twin data (Clayton-Oakes ACE).}
  \item{\code{sim_binClaytonOakes_pairs}}{Simulate binary paired data (Clayton-Oakes).}
  \item{\code{sim_bptwin}}{Simulate from a biprobit twin model.}
}
}

\section{Nordic Twin Studies}{

\describe{
  \item{\code{sim_nordictwin}}{Simulate Nordic twin registry data.}
  \item{\code{sim_nordic_random}}{Simulate Nordic twin data with random effects.}
  \item{\code{corsim_prostate_random}}{Simulate correlated prostate cancer data with random effects.}
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cifreg.R
\name{FG_AugmentCifstrata}
\alias{FG_AugmentCifstrata}
\alias{strataC}
\alias{simul.cifs}
\alias{setup.cif}
\alias{drop.strata}
\title{Augmentation for Fine-Gray model based on stratified NPMLE Cif (Aalen-Johansen)}
\usage{
FG_AugmentCifstrata(
  formula,
  data = data,
  E = NULL,
  cause = NULL,
  cens.code = 0,
  km = TRUE,
  case.weights = NULL,
  weights = NULL,
  offset = NULL,
  ...
)
}
\arguments{
\item{formula}{formula with 'Event', strata model for CIF given by strata, and strataC specifies censoring strata}

\item{data}{data frame}

\item{E}{from FG-model}

\item{cause}{of interest}

\item{cens.code}{code of censoring}

\item{km}{to use Kaplan-Meier}

\item{case.weights}{weights for FG score equations (that follow dN_1)}

\item{weights}{weights for FG score equations}

\item{offset}{offsets for FG   model}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Computes  the augmentation term for each individual as well as the sum
\deqn{
A(\beta) = \int H(t,X,\beta) \frac{F_2^*(t,s)}{S^*(t,s)} \frac{1}{G_c(t)} dM_c
}
with
\deqn{
H(t,X,\beta) = \int_t^\infty (X - E(\beta,t) ) G_c(t) d\Lambda_1^*i(t,s)
}
using a KM for \deqn{G_c(t)} and a working model for cumulative baseline
related to \deqn{F_1^*(t,s)} and \deqn{s} is strata,
\deqn{S^*(t,s) = 1 - F_1^*(t,s) - F_2^*(t,s)}, and
\deqn{E(\beta^p,t)} is given. Assumes that no strata for baseline of ine-Gay model that is augmented.
}
\details{
After a couple of iterations we end up with a solution of
\deqn{
\int (X - E(\beta) ) Y_1(t) w(t) dM_1 + A(\beta)
}
the augmented FG-score.

Standard errors computed under assumption of correct \deqn{G_c} model.
}
\examples{
set.seed(100)
rho1 <- 0.2; rho2 <- 10
n <- 400
beta=c(0.0,-0.1,-0.5,0.3)
dats <- simul.cifs(n,rho1,rho2,beta,rc=0.2)
dtable(dats,~status)
dsort(dats) <- ~time
fg <- cifreg(Event(time,status)~Z1+Z2,data=dats,cause=1,propodds=NULL)
summary(fg)

fgaugS <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fg$E)
summary(fgaugS)
fgaugS2 <- FG_AugmentCifstrata(Event(time,status)~Z1+Z2+strata(Z1,Z2),data=dats,cause=1,E=fgaugS$E)
summary(fgaugS2)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{multcif}
\alias{multcif}
\title{Multivariate Cumulative Incidence Function example data set}
\source{
Simulated data
}
\description{
Multivariate Cumulative Incidence Function example data set
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{sim.recurrent}
\alias{sim.recurrent}
\title{Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure}
\usage{
sim.recurrent(cox1,coxd=NULL,coxc=NULL,n=1,data=NULL,
type=c("default","cox-cox","gl-cox"),id="id",
varz=1,share=1,cens=0.001,scale1=1,scaled=1,dependence=NULL,...)
}
\arguments{
\item{cox1}{cox/ghosh-lin for recurrent events}

\item{coxd}{cox for terminal event (phreg)}

\item{coxc}{possible cox for censoring (phreg)}

\item{n}{number of id's}

\item{data}{on which the models are fitted (to draw covariates)}

\item{type}{to specify type of simulation, if not default}

\item{id}{name of id variable}

\item{varz}{dependence frailty}

\item{share}{to fit patly shared random effects model}

\item{cens}{censoring rate for exponential censoring}

\item{scale1}{to scale baseline of recurrent events model}

\item{scaled}{to scale baseline of terminal event}

\item{dependence}{if dependence different from NULL, then uses simRecurrentList based on models given}

\item{...}{Additional arguments to simGLcox, nmin, nmax regulates linear approximation grid}
}
\description{
Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure
}
\details{
Must specify two phreg objects, or a phreg and a recreg object, then simulates data from two-stage model
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment)
n <- 100
xr <- phreg(Surv(entry,time,status==1)~x+cluster(id),data=hf)
dr <- phreg(Surv(entry,time,status==2)~x+cluster(id),data=hf)
simcoxcox <- sim.recurrent(xr,dr,n=n,data=hf)
recGL <- recreg(Event(entry,time,status)~x+cluster(id),hf,death.code=2)
simglcox <- sim.recurrent(recGL,dr,n=n,data=hf)

}
\references{
Scheike (2024), Twostage recurrent events models, under review.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recreg.R
\name{recregIPCW}
\alias{recregIPCW}
\title{IPCW Estimator for Recurrent Events}
\usage{
recregIPCW(
  formula,
  data = data,
  cause = 1,
  cens.code = 0,
  death.code = 2,
  cens.model = ~1,
  km = TRUE,
  times = NULL,
  beta = NULL,
  offset = NULL,
  type = c("II", "I"),
  marks = NULL,
  weights = NULL,
  model = c("exp", "lin"),
  no.opt = FALSE,
  augmentation = NULL,
  method = "nr",
  se = TRUE,
  ...
)
}
\arguments{
\item{formula}{Formula with an 'Event' outcome.}

\item{data}{Data frame.}

\item{cause}{Cause of interest (default is 1).}

\item{cens.code}{Censoring code (default is 0).}

\item{death.code}{Death code (default is 2).}

\item{cens.model}{Formula for the censoring model (default is \code{~1}).}

\item{km}{Logical; if \code{TRUE}, uses Kaplan-Meier for censoring weights; otherwise uses Cox model.}

\item{times}{Time points for estimation (required).}

\item{beta}{Initial values for coefficients (optional).}

\item{offset}{Offsets.}

\item{type}{Type of estimator: \code{"II"} (default) or \code{"I"}.}

\item{marks}{Mark values.}

\item{weights}{Weights.}

\item{model}{Model type for the mean: \code{"exp"} (default) or \code{"lin"}.}

\item{no.opt}{Logical; if \code{TRUE}, skips optimization.}

\item{augmentation}{Augmentation terms.}

\item{method}{Optimization method (default is "nr").}

\item{se}{Logical; if \code{TRUE}, computes standard errors.}

\item{...}{Additional arguments.}
}
\value{
An object of class \code{"binreg"} (extending \code{"resmean"}) containing:
\item{coef}{Estimated coefficients.}
\item{var}{Variance-covariance matrix.}
\item{iid}{Influence functions.}
\item{times}{Time points.}
\item{Y}{Observed IPCW weighted counts (increment IPCW) \deqn{\int_0^t (1/G_c(s)) dN(s)}.}
}
\description{
Computes the Inverse Probability of Censoring Weighted (IPCW) estimator for the mean number of recurrent events.
Supports various estimators including the Ghosh-Lin and Lawless-Cook estimators.
}
\seealso{
\code{\link{recreg}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifetable.R
\name{eventpois}
\alias{eventpois}
\alias{pcif}
\title{Extract survival estimates from lifetable analysis}
\usage{
eventpois(
  object,
  ...,
  timevar,
  time,
  int.len,
  confint = FALSE,
  level = 0.95,
  individual = FALSE,
  length.out = 25
)
}
\arguments{
\item{object}{glm object (poisson regression)}

\item{...}{Contrast arguments}

\item{timevar}{Name of time variable}

\item{time}{Time points (optional)}

\item{int.len}{Time interval length (optional)}

\item{confint}{If TRUE confidence limits are supplied}

\item{level}{Level of confidence limits}

\item{individual}{Individual predictions}

\item{length.out}{Length of time vector}
}
\description{
Summary for survival analyses via the 'lifetable' function
}
\details{
Summary for survival analyses via the 'lifetable' function
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{prob_exceed_recurrent}
\alias{prob_exceed_recurrent}
\title{Estimate the probability of exceeding k recurrent events by time t}
\usage{
prob_exceed_recurrent(
  formula,
  data,
  cause = 1,
  death.code = 2,
  cens.code = 0,
  exceed = NULL,
  marks = NULL,
  all.cifs = FALSE,
  return.data = FALSE,
  conf.type = c("log", "plain"),
  level = 0.95,
  ...
)
}
\arguments{
\item{formula}{A formula with an \code{\link{Event}} response giving the
exit time and status (and optionally entry time). The right-hand side may
include \code{cluster()} and \code{strata()}.}

\item{data}{A data frame containing all variables in \code{formula}.}

\item{cause}{Integer code for the recurrent event of interest. Default is
\code{1}.}

\item{death.code}{Integer code for the terminal event. Default is \code{2}.}

\item{cens.code}{Integer code for censoring. Default is \code{0}.}

\item{exceed}{Integer vector of thresholds \eqn{k} to evaluate. If
\code{NULL} (default), all observed cumulative counts are used.}

\item{marks}{Optional numeric vector of event weights. If non-\code{NULL},
cumulative counts are formed as weighted sums of events rather than simple
counts. Must have the same length as \code{nrow(data)}.}

\item{all.cifs}{Logical. If \code{TRUE}, the fitted \code{cif} object for
each threshold is returned in \code{cif.exceed}. Default is \code{FALSE}.}

\item{return.data}{Logical. If \code{TRUE}, the constructed dataset for each
threshold is returned in \code{dataList}. Default is \code{FALSE}.}

\item{conf.type}{Type of confidence interval transformation: \code{"log"}
(default) or \code{"plain"}.}

\item{level}{Confidence level. Default is \code{0.95}.}

\item{...}{Further arguments passed to \code{\link{cif}}.}
}
\value{
An object of class \code{"exceed"} with the following components:
  \item{time}{Vector of evaluation time points.}
  \item{prob}{Array of dimension \code{(length(time), length(exceed) + 1,
    nstrata)} containing \eqn{P(N(t) \geq k)} for each threshold and stratum.
    The first column gives \eqn{P(N(t) < \text{exceed}[1])}.}
  \item{se.prob}{Standard errors of \code{prob}.}
  \item{lower, upper}{Pointwise confidence interval bounds.}
  \item{meanN}{Estimated mean number of events \eqn{E(N(t))} (single stratum
    only; \code{NULL} for stratified analyses).}
  \item{meanN2, varN}{Second moment and variance of \eqn{N(t)} (single
    stratum only).}
  \item{exceed}{Thresholds evaluated (excluding zero).}
  \item{cif.exceed}{List of fitted \code{cif} objects (if
    \code{all.cifs = TRUE}).}
  \item{dataList}{List of datasets for each threshold (if
    \code{return.data = TRUE}).}
  \item{nstrata, strata.levels, strata.name}{Stratification information.}
  Use \code{plot()} and \code{summary()} methods for visualisation and
  tabulation.
}
\description{
Estimates \eqn{P(N(t) \geq k)} as a function of time \eqn{t}, for a range of
thresholds \eqn{k}, in the presence of a terminal event (death). The estimator
is based on the cumulative incidence of "reaching \eqn{k} events", treating
death as a competing risk. Confidence intervals are computed on the log or
plain scale.
}
\details{
For each threshold \eqn{k} in \code{exceed}, the function identifies the
first time each subject reaches \eqn{k} events, then fits a competing risks
model (\code{\link{cif}}) with "reaching \eqn{k} events" as the event of
interest and death as the competing event. Strata are supported. When
\code{marks} is \code{NULL}, each event contributes equally; otherwise events
are weighted by their mark values before cumulative counts are formed.
}
\examples{
data(hfactioncpx12)
dtable(hfactioncpx12, ~status)

oo <- prob_exceed_recurrent(Event(entry, time, status) ~ cluster(id),
                             hfactioncpx12, cause = 1, death.code = 2)
plot(oo)
summary(oo, times = c(1, 2, 5))
}
\references{
Scheike, T. H., Eriksson, L., and Tribler, P. (2019). The mean, variance and
correlation for bivariate recurrent events with a terminal event.
\emph{Journal of the Royal Statistical Society, Series C}, 68(5).
}
\seealso{
\code{\link{recurrent_marginal}}, \code{\link{cif}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{phregR}
\alias{phregR}
\alias{FastCoxPLstrataR}
\title{Fast Cox PH regression and calculations done in R to make play and adjustments easy}
\usage{
phregR(formula, data, offset = NULL, weights = NULL, ...)
}
\arguments{
\item{formula}{formula with 'Surv' outcome (see \code{coxph})}

\item{data}{data frame}

\item{offset}{offsets for cox model}

\item{weights}{weights for Cox score equations}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Fast Cox PH regression with R implementation to play and adjust in R function: FastCoxPLstrataR
}
\details{
Robust variance is default variance with the summary. 

influence functions (iid) will follow numerical order of given cluster variable
so ordering after $id will give iid in order of data-set.
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pch.R
\name{rchazC}
\alias{rchazC}
\title{Piecewise constant hazard distribution}
\usage{
rchazC(base1, rr, entry)
}
\arguments{
\item{base1}{baseline}

\item{rr}{relative risk terms}

\item{entry}{entry times for left truncation}
}
\description{
Piecewise constant hazard distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.clayton.oakes.R
\name{sim_ClaytonOakes}
\alias{sim_ClaytonOakes}
\alias{sim_ClaytonOakesLam}
\title{Simulate from the Clayton-Oakes frailty model}
\usage{
sim_ClaytonOakes(
  K,
  n,
  eta,
  beta,
  stoptime,
  lam = 1,
  left = 0,
  pairleft = 0,
  trunc.prob = 0.5,
  same = 0
)
}
\arguments{
\item{K}{Number of clusters}

\item{n}{Number of observations in each cluster}

\item{eta}{variance}

\item{beta}{Effect (log hazard ratio) of covariate}

\item{stoptime}{Stopping time}

\item{lam}{constant hazard}

\item{left}{Left truncation}

\item{pairleft}{pairwise (1) left truncation or individual (0)}

\item{trunc.prob}{Truncation probability}

\item{same}{if 1 then left-truncation is same also for univariate truncation}
}
\description{
Simulate observations from the Clayton-Oakes copula model with
piecewise constant marginals.
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{np}
\alias{np}
\title{np data set}
\source{
Simulated data
}
\description{
np data set
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterindex-reshape.R
\name{coarse_clust}
\alias{coarse_clust}
\title{Coarsen Cluster Identifiers}
\usage{
coarse_clust(clusters, max.clust = 100)
}
\arguments{
\item{clusters}{vector of cluster identifiers.}

\item{max.clust}{maximum number of coarsened clusters.}
}
\value{
Integer vector of coarsened cluster indices (0-based).
}
\description{
Reduces the number of unique clusters by binning into quantile groups.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{waldTest}
\alias{waldTest}
\title{Wald test for model (type III test)}
\usage{
waldTest(object, ...)
}
\arguments{
\item{object, }{for example glm object that can be used with estimate}

\item{...}{arguments for estimate of lava for example id=data$id for cluster correction}
}
\description{
Wald test for model (type III test)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-survival-haplo.R
\name{haplo_surv_discrete}
\alias{haplo_surv_discrete}
\title{Discrete Time-to-Event Haplotype Analysis}
\usage{
haplo_surv_discrete(
  X = NULL,
  y = "y",
  time.name = "time",
  Haplos = NULL,
  id = "id",
  desnames = NULL,
  designfunc = NULL,
  beta = NULL,
  no.opt = FALSE,
  method = "NR",
  stderr = TRUE,
  designMatrix = NULL,
  response = NULL,
  idhap = NULL,
  design.only = FALSE,
  covnames = NULL,
  fam = binomial,
  weights = NULL,
  offsets = NULL,
  idhapweights = NULL,
  ...
)
}
\arguments{
\item{X}{Design matrix data frame (must be sorted by \code{id} and \code{time}) containing 
the ID, time variable, binary response, and covariates.}

\item{y}{Name of the response variable (binary, 0/1) in \code{X}.}

\item{time.name}{Name of the time variable in \code{X} used for sorting and cycle definition.}

\item{Haplos}{Data frame containing \code{id}, \code{haplo1}, \code{haplo2} (haplotypes as factors), 
and \code{p} (probability \eqn{P(H|G)}).}

\item{id}{Name of the ID variable in \code{X} and \code{Haplos}.}

\item{desnames}{Names of the covariate columns in \code{X} to be used in the design matrix.}

\item{designfunc}{Function that computes the design vector given haplotypes \eqn{h=(h_1, h_2)} 
and covariates. Must return a vector or matrix compatible with the model.}

\item{beta}{Starting values for the optimization (vector of length \eqn{p + k}, where \eqn{p} 
is the number of covariate effects and \eqn{k} is the number of time cycles).}

\item{no.opt}{Logical; if TRUE, skips optimization and returns estimates based on the 
provided \code{beta} (useful for initialization or diagnostics).}

\item{method}{Optimization method: \code{"NR"} (Newton-Raphson, default) or \code{"nlm"}.}

\item{stderr}{Logical; if FALSE, returns only the coefficient estimates.}

\item{designMatrix}{Alternative to \code{X} and \code{Haplos}: provides the response and 
design matrix directly (not fully implemented).}

\item{response}{Alternative to \code{X}: provides the response and design directly (not fully implemented).}

\item{idhap}{Name of the ID-haplotype variable to specify different haplotypes for different IDs.}

\item{design.only}{Logical; if TRUE, returns only the design matrices constructed for the analysis.}

\item{covnames}{Names of covariates to extract from the object for regression output.}

\item{fam}{Family of the model (default \code{binomial}, currently the only option).}

\item{weights}{Weights following ID for the GLM component.}

\item{offsets}{Offsets following ID for the GLM component.}

\item{idhapweights}{Weights following ID-haplotype for the GLM component (Work in Progress).}

\item{...}{Additional arguments passed to the optimizer (\code{lava::NR} or \code{nlm}).}
}
\value{
An object of class \code{"haplosurvd"} containing:
  \item{coef}{Estimated coefficients (baseline time effects and haplotype/covariate effects).}
  \item{se}{Standard errors of the coefficients.}
  \item{var}{Variance-covariance matrix.}
  \item{se.robust}{Robust standard errors (if available).}
  \item{iid}{Influence function (IID) decomposition.}
  \item{ploglik}{Log-likelihood at convergence.}
  \item{gradient, hessian}{Optimization results.}
  \item{Xhap, X, Haplos}{Data and design matrices used.}
  \item{nid, nidhap}{Number of IDs and ID-haplotype combinations.}
}
\description{
Performs cycle-specific logistic regression to estimate haplotype effects on discrete 
time-to-event data, accounting for phase ambiguity. Given observed genotypes \eqn{G} 
and unobserved haplotypes \eqn{H}, the method integrates (mixes out) over the possible 
haplotype configurations using the conditional probabilities \eqn{P(H|G)}.
}
\details{
The survival function is computed by averaging over the possible haplotypes:
\deqn{ S(t|x,G) = E[ S(t|x,H) | G ] = \sum_{h \in G} P(h|G) S(t|x,h) }

The discrete hazard function is modeled using logistic regression:
\deqn{ \text{logit}(P(T=t | T \geq t, x, h)) = \alpha_t + x(h) \beta }
where \eqn{\alpha_t} are time-specific intercepts (baseline hazards), \eqn{x(h)} is the 
regression design constructed from covariates and haplotypes \eqn{h=(h_1, h_2)}, and 
\eqn{\beta} are the regression coefficients.

The likelihood is maximized numerically. Standard errors are computed assuming that 
\eqn{P(H|G)} is known (i.e., ignoring the uncertainty in haplotype estimation).

The design matrix over possible haplotypes is constructed by merging the covariate 
data \eqn{X} with the haplotype probabilities \eqn{Haplos} and applying a user-defined 
\code{designfunc}.
}
\examples{
## Some haplotypes of interest
types <- c("DCGCGCTCACG","DTCCGCTGACG","ITCAGTTGACG","ITCCGCTGAGG")

## Some haplotype frequencies for simulations 
data(haplo)
hapfreqs <- haplo$hapfreqs 

www <- which(hapfreqs$haplotype \%in\% types)
hapfreqs$freq[www]

baseline <- hapfreqs$haplotype[9]
baseline

## Design function: indicator for presence of any 'types' haplotype
designftypes <- function(x, sm=0) {
  hap1 <- x[1]
  hap2 <- x[2]
  if (sm == 0) y <- 1 * ((hap1 == types) | (hap2 == types))
  if (sm == 1) y <- 1 * (hap1 == types) + 1 * (hap2 == types)
  return(y)
}

tcoef <- c(-1.93110204, -0.47531630, -0.04118204, -1.57872602, -0.22176426, -0.13836416,
           0.88830288, 0.60756224, 0.39802821, 0.32706859)

ghaplos <- haplo$ghaplos
haploX  <- haplo$haploX

haploX$time <- haploX$times
Xdes <- model.matrix(~ factor(time), haploX)
colnames(Xdes) <- paste("X", 1:ncol(Xdes), sep="")
X <- dkeep(haploX, ~ id + y + time)
X <- cbind(X, Xdes)
Haplos <- dkeep(ghaplos, ~ id + "haplo*" + p)
desnames <- paste("X", 1:6, sep="")   # Six X's related to 6 cycles 
out <- haplo_surv_discrete(X=X, y="y", time.name="time",
         Haplos=Haplos, desnames=desnames, designfunc=designftypes) 
names(out$coef) <- c(desnames, types)
out$coef
summary(out)
}
\references{
Scheike, T. H. (2024). Discrete time survival analysis with haplotype effects. mets package documentation.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{count.history}
\alias{count.history}
\alias{count.historyVar}
\title{Counts the number of previous events of two types for recurrent events processes}
\usage{
count.history(
  data,
  status = "status",
  id = "id",
  types = 1,
  names.count = "Count",
  lag = TRUE,
  multitype = FALSE,
  marks = NULL
)
}
\arguments{
\item{data}{data-frame}

\item{status}{name of status}

\item{id}{id}

\item{types}{types of the events (code) related to status (multiple values possible)}

\item{names.count}{name of Counts, for example Count1 Count2 when types=c(1,2)}

\item{lag}{if true counts previously observed, and if lag=FALSE counts up to know}

\item{multitype, }{if multitype is true then counts when status "in" types, otherwise counts for each value of type, types=c(1,2)}

\item{marks}{values related to status ("in" types), counts marks for types, only when multitype=TRUE}
}
\description{
Counts the number of previous events of two types for recurrent events processes
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12
dtable(hf,~status)
rr <-  count.history(hf,types=1:2,id="id",status="status")
dtable(rr,~"Count*"+status,level=1)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lifecourse.R
\name{lifecourse}
\alias{lifecourse}
\title{Life-course plot}
\usage{
lifecourse(
  formula,
  data,
  id = "id",
  group = NULL,
  type = "l",
  lty = 1,
  col = 1:10,
  alpha = 0.3,
  lwd = 1,
  recurrent.col = NULL,
  recurrent.lty = NULL,
  legend = NULL,
  pchlegend = NULL,
  by = NULL,
  status.legend = NULL,
  place.sl = "bottomright",
  xlab = "Time",
  ylab = "",
  add = FALSE,
  ...
)
}
\arguments{
\item{formula}{Formula (Event(start,slut,status) ~ ...)}

\item{data}{data.frame}

\item{id}{Id variable}

\item{group}{group variable}

\item{type}{Type (line 'l', stair 's', ...)}

\item{lty}{Line type}

\item{col}{Colour}

\item{alpha}{transparency (0-1)}

\item{lwd}{Line width}

\item{recurrent.col}{col of recurrence type}

\item{recurrent.lty}{lty's of  of recurrence type}

\item{legend}{position of optional id legend}

\item{pchlegend}{point type legends}

\item{by}{make separate plot for each level in 'by' (formula, name of column, or vector)}

\item{status.legend}{Status legend}

\item{place.sl}{Placement of status legend}

\item{xlab}{Label of X-axis}

\item{ylab}{Label of Y-axis}

\item{add}{Add to existing device}

\item{...}{Additional arguments to lower level arguments}
}
\description{
Life-course plot for event life data with recurrent events
}
\examples{
data = data.frame(id=c(1,1,1,2,2),start=c(0,1,2,3,4),slut=c(1,2,4,4,7),
                  type=c(1,2,3,2,3),status=c(0,1,2,1,2),group=c(1,1,1,2,2))
ll = lifecourse(Event(start,slut,status)~id,data,id="id")
ll = lifecourse(Event(start,slut,status)~id,data,id="id",recurrent.col="type")

ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2)
op <- par(mfrow=c(1,2))
ll = lifecourse(Event(start,slut,status)~id,data,id="id",by=~group)
par(op)
legends=c("censored","pregnant","married")
ll = lifecourse(Event(start,slut,status)~id,data,id="id",group=~group,col=1:2,status.legend=legends)

}
\author{
Thomas Scheike, Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{simRecurrentII}
\alias{simRecurrentII}
\alias{showfitsim}
\alias{covIntH1dM1IntH2dM2}
\alias{squareintHdM}
\alias{simRecurrentList}
\alias{showfitsimList}
\title{Simulation of recurrent events data based on cumulative hazards with two
types of recurrent events}
\usage{
simRecurrentII(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz = NULL,
  r1 = NULL,
  r2 = NULL,
  rd = NULL,
  rc = NULL,
  dependence = 0,
  var.z = 1,
  cor.mat = NULL,
  cens = NULL,
  gap.time = FALSE,
  max.recurrent = 100,
  ...
)
}
\arguments{
\item{n}{number of id's}

\item{cumhaz}{cumulative hazard of recurrent events}

\item{cumhaz2}{cumulative hazard of recurrent events  of type 2}

\item{death.cumhaz}{cumulative hazard of death}

\item{r1}{potential relative risk adjustment of rate}

\item{r2}{potential relative risk adjustment of rate}

\item{rd}{potential relative risk adjustment of rate}

\item{rc}{potential relative risk adjustment of rate}

\item{dependence}{0:independence; 1:all share same random effect with variance var.z; 2:random effect exp(normal) with correlation structure from cor.mat; 3:additive gamma distributed random effects, z1= (z11+ z12)/2 such that mean is 1 , z2= (z11^cor.mat(1,2)+ z13)/2, z3= (z12^(cor.mat(2,3)+z13^cor.mat(1,3))/2, with z11 z12 z13 are gamma with mean and variance 1 , first random effect is z1 and for N1 second random effect is z2 and for N2 third random effect is for death}

\item{var.z}{variance of random effects}

\item{cor.mat}{correlation matrix for var.z variance of random effects}

\item{cens}{rate of censoring exponential distribution}

\item{gap.time}{if true simulates gap-times with specified cumulative hazard}

\item{max.recurrent}{limits number recurrent events to 100}

\item{...}{Additional arguments to simRecurrentList}
}
\description{
Simulation of recurrent events data based on cumulative hazards
}
\details{
Must give cumulative hazard of death and possibly two recurrent events. 
Their dependence can be specified via random 
effects but the two recurrent events need to share the random effect, and can also be
specified via zzr. 
The terminal event may share this random effect (dependence=1) or not 
(dependence=4) and can be specified via zzr.
}
\examples{
########################################
## getting some rates to mimick 
########################################
library(mets)
data(CPH_HPN_CRBSI)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
base4 <- CPH_HPN_CRBSI$mechanical

######################################################################
### simulating simple model that mimicks data 
######################################################################
rr <- simRecurrent(5,base1)
dlist(rr,.~id,n=0)
rr <- simRecurrent(5,base1,death.cumhaz=dr)
dlist(rr,.~id,n=0)

rr <- simRecurrent(100,base1,death.cumhaz=dr)
par(mfrow=c(1,3))
showfitsim(causes=1,rr,dr,base1,base1)
######################################################################
### simulating simple model 
### random effect for all causes (Z shared for death and recurrent) 
######################################################################
rr <- simRecurrent(100,base1,death.cumhaz=dr,dependence=1,var.z=0.4)
dtable(rr,~death+status)

######################################################################
### now with two event types and second type has same rate as death rate
######################################################################
set.seed(100)
rr <- simRecurrentII(100,base1,base4,death.cumhaz=dr)
dtable(rr,~death+status)
par(mfrow=c(2,2))
showfitsim(causes=2,rr,dr,base1,base4)

######################################################################
### now with three event types and two causes of death 
######################################################################
set.seed(100)
cumhaz <- list(base1,base1,base4)
drl <- list(dr,base4)
rr <- simRecurrentList(100,cumhaz,death.cumhaz=drl,dependence=0)
dtable(rr,~death+status)
showfitsimList(rr,cumhaz,drl) 

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brier_binreg.R
\name{binregStrata}
\alias{binregStrata}
\title{Binomial Regression with Stratified Pseudo-Values and Censoring Adjustment}
\usage{
binregStrata(
  formula,
  data,
  cause = 1,
  time = NULL,
  beta = NULL,
  type = c("II", "I"),
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  cens.model = ~+1,
  se = TRUE,
  kaplan.meier = TRUE,
  cens.code = 0,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  outcome = c("cif", "rmst", "rmtl"),
  model = c("default", "logit", "exp", "lin"),
  Ydirect = NULL,
  strata = NULL,
  cv.fold = FALSE,
  ...
)
}
\arguments{
\item{formula}{A formula with a \code{\link[survival]{Surv}} or
\code{Event} object on the left-hand side, and covariates, optional
\code{cluster()}, \code{strata()}, \code{offset()}, and \code{weights()}
terms on the right-hand side.}

\item{data}{A \code{data.frame} containing the variables in \code{formula}.}

\item{cause}{Value(s) of the status variable that identify the event of
interest (cause 1 by default).}

\item{time}{The fixed time-point at which the CIF, RMST, or RMTL is
evaluated. Required.}

\item{beta}{Optional starting values for the regression coefficients.
Either a vector (recycled across strata) or a matrix with one row per
stratum.}

\item{type}{Character; either \code{"II"} (default) or \code{"I"}. When
\code{"II"}, an additional augmentation term and an extra component of
the censoring-related iid decomposition are computed.}

\item{offset}{Optional offset term; can also be specified via the formula.}

\item{weights}{Optional case weights; can also be specified via the
formula.}

\item{cens.weights}{Optional pre-computed inverse probability of censoring
weights. If supplied, the internal censoring model fit is skipped and
\code{se} is forced to \code{FALSE}.}

\item{cens.model}{A formula for the censoring model (right-hand side only,
fitted via \code{\link{phreg}}). Defaults to the Kaplan-Meier
(\code{~+1}).}

\item{se}{Logical. If \code{TRUE} (default), standard errors are computed,
including the contribution from estimating the censoring distribution.}

\item{kaplan.meier}{Logical. If \code{TRUE} (default), uses the
Kaplan-Meier estimator for censoring weights when the censoring model has
no covariates; otherwise a Cox-type estimator is used internally when
covariates are present.}

\item{cens.code}{The code in the status variable denoting censoring
(default 0).}

\item{no.opt}{Logical. If \code{TRUE}, no optimization is performed and the
object is evaluated at the supplied \code{beta}.}

\item{method}{Optimization method. Currently only \code{"nr"} (Newton-Raphson
via \code{lava::NR}) is supported.}

\item{augmentation}{Optional augmentation term(s) added to the score
equations, either a vector (recycled across strata) or a matrix with one
row per stratum.}

\item{outcome}{Character; the outcome scale to model: \code{"cif"}
(default), \code{"rmst"}, or \code{"rmtl"}.}

\item{model}{Character; the link function: \code{"default"} (chooses
\code{"logit"} for CIF and \code{"exp"} for RMST/RMTL), \code{"logit"},
\code{"exp"}, or \code{"lin"}.}

\item{Ydirect}{Optional, directly supplied response vector, overriding the
response otherwise constructed from \code{outcome}.}

\item{strata}{Optional numeric vector defining strata, used when strata are
not specified via \code{strata()} in the formula.}

\item{cv.fold}{Logical. If \code{TRUE}, for each stratum \code{s} the
regression coefficients are estimated using all observations \emph{not}
in stratum \code{s} (a cross-validation-style fit), rather than the
observations within stratum \code{s} (the default, \code{FALSE}).}

\item{...}{Additional arguments, currently passed as optimizer
\code{control} settings (e.g. \code{tol}, \code{stepsize}) to the
Newton-Raphson routine.}
}
\value{
An object of class \code{c("binregStrata","binreg")}, a list with
  components including:
  \item{coef}{Estimated regression coefficients (stacked across strata).}
  \item{iid}{Subject-level iid decomposition of the coefficients, adjusted
    for censoring estimation when \code{se = TRUE}.}
  \item{var, robvar}{Robust (sandwich) variance-covariance matrix.}
  \item{se.coef, se.robust}{Standard errors derived from \code{var}.}
  \item{hessian, ihessian}{Per-stratum Hessians and their (pseudo-)inverses.}
  \item{coef_list}{Per-stratum coefficient vectors.}
  \item{time, cause, cens.code, model, outcome}{Echoed arguments.}
  \item{Y, Yipcw, IPCW}{The constructed response, IPCW-weighted response,
    and IPCW weights.}
  \item{cens.weights, formC, cens.strata, cens.nstrata}{Censoring model
    weights and related information (when internally estimated).}
  \item{nstrata, strata_orig, strata_call, cv.fold}{Stratification
    information and whether the cross-validation fold scheme was used.}
  \item{n, nevent, ncluster, nid, id, name.id}{Sample size and
    identifier information.}
  \item{call, design}{The matched call and the design object.}
}
\description{
Fits a stratified binomial regression model for the cumulative incidence
function (CIF), restricted mean survival time (RMST), or restricted mean
time lost (RMTL), using inverse probability of censoring weighting (IPCW).
Each stratum may either be fit on its own observations or, when
\code{cv.fold = TRUE}, on the complement of its observations (i.e. a
leave-one-stratum-out / cross-validation style fit). Standard errors are
computed via an iid decomposition that accounts for the estimation of the
censoring distribution.
}
\details{
When \code{cv.fold = FALSE} (the standard case), the coefficients for
stratum \code{s} are estimated using only observations with
\code{strata == s}. When \code{cv.fold = TRUE}, the coefficients for
stratum \code{s} are instead estimated using observations with
\code{strata != s}, i.e. the complement; this also affects the
construction of the censoring-related iid terms (\code{MGCiid}) and the
per-stratum subject-level influence functions used for the robust
variance.

Censoring weights are computed via a Cox model (\code{\link{phreg}}) on
\code{cens.model}, evaluated at \code{pmin(exit, time)}, unless
\code{cens.weights} is supplied directly. When \code{se = TRUE}, the
additional variability coming from the estimation of the censoring
distribution is incorporated into the standard errors via a martingale-type
iid decomposition (\code{MGCiid}).
}
\examples{
\dontrun{
library(mets)
data(bmt)
out <- binregStrata(Event(time, cause) ~ tcell + strata(platelet),
                     data = bmt, cause = 1, time = 50)
summary(out)
}

}
\seealso{
\code{\link{phreg}}, \code{\link{binreg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.regression.R
\name{binreg}
\alias{binreg}
\alias{logitIPCW}
\alias{binregt}
\title{Binomial Regression for Censored Competing Risks Data}
\usage{
binreg(
  formula,
  data,
  cause = 1,
  time = NULL,
  beta = NULL,
  type = c("II", "I"),
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  cens.model = ~+1,
  se = TRUE,
  kaplan.meier = TRUE,
  cens.code = 0,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  outcome = c("cif", "rmst", "rmtl"),
  model = c("default", "logit", "exp", "lin"),
  Ydirect = NULL,
  ...
)
}
\arguments{
\item{formula}{A formula object specifying the outcome and covariates. 
The outcome must be an \code{Event} object (\code{Event(time, cause)}).}

\item{data}{A data frame containing the variables in the formula.}

\item{cause}{Numeric vector or scalar indicating the cause of interest for the competing risks.}

\item{time}{Numeric scalar indicating the time point of interest for the cumulative incidence.}

\item{beta}{Optional numeric vector of starting values for the coefficients. Defaults to zeros.}

\item{type}{Character string. Either \code{"I"} (classic binomial regression) or \code{"II"} (adds augmentation term).}

\item{offset}{Optional numeric vector of offsets for the linear predictor.}

\item{weights}{Optional numeric vector of weights for the score equations.}

\item{cens.weights}{Optional numeric vector of pre-calculated censoring weights. If NULL, they are estimated internally.}

\item{cens.model}{A formula specifying the censoring model. Defaults to \code{~+1}. Can include strata (e.g., \code{~strata(group)}).}

\item{se}{Logical. If TRUE, computes standard errors based on IPCW influence functions.}

\item{kaplan.meier}{Logical. If TRUE, uses Kaplan-Meier estimator for IPCW weights; otherwise uses exponential baseline.}

\item{cens.code}{Numeric code representing censored observations in the status variable. Defaults to 0.}

\item{no.opt}{Logical. If TRUE, optimization is skipped and starting values are used.}

\item{method}{Character string. Optimization method: \code{"nr"} (Newton-Raphson) or \code{"nlm"}.}

\item{augmentation}{Optional numeric vector for additional augmentation terms.}

\item{outcome}{Character string. Outcome type: \code{"cif"} (Cumulative Incidence Function), \code{"rmst"} (Restricted Mean Survival Time), or \code{"rmtl"} (Restricted Mean Time Lost).}

\item{model}{Character string. Link function: \code{"default"} (auto-selects based on outcome), \code{"logit"}, \code{"exp"}, or \code{"lin"} (identity).}

\item{Ydirect}{Optional numeric vector. If provided, this outcome is used instead of constructing one from \code{outcome}. Useful for custom IPCW adjustments.}

\item{...}{Additional arguments passed to lower-level optimization functions.}
}
\value{
An object of class \code{"binreg"} containing coefficients, variance-covariance matrix, 
  influence functions (\code{iid}), and model details.
}
\description{
Fits a binomial regression model for a specific time point in the presence of 
right-censored data and competing risks. This function implements the 
Inverse Probability of Censoring Weighted (IPCW) estimating equation approach.
}
\details{
The model estimates the probability:
\deqn{P(T \leq t, \epsilon=1 | X ) = \text{expit}( X^T \beta) }

Based on binomial regresion IPCW response estimating equation: 
\deqn{ X ( \Delta^{ipcw}(t) I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 }
where \deqn{\Delta^{ipcw}(t) = I((min(t,T)< C)/G_c(min(t,T)-)} is 
IPCW adjustment of the response \deqn{Y(t)= I(T \leq t, \epsilon=1 )}.  
Two types of estimators are available:
\itemize{
  \item \code{type="I"}: Solves the standard IPCW estimating equation.
  \item \code{type="II"}: Adds a censoring augmentation term for efficiency gains, 
  solving \deqn{X \int E(Y(t)| T>s)/G_c(s) d \hat M_c}.
}

Alternatively, \code{logitIPCW} performs a standard logistic regression with 
IPCW weights applied directly to the likelihood. Thus solving 
\deqn{ X  I(min(T_i,t) < G_i)/G_c(min(T_i ,t)) ( I(T \leq t, \epsilon=1 ) - expit( X^T beta)) = 0 } 
a standard logistic regression with IPCW weights.

The variance estimation is based on squared influence functions, with options 
for naive variance (assuming known censoring) and robust variance (accounting 
for censoring model estimation).

Censoring model may depend on strata (cens.model=~strata(gX)).
}
\section{References}{

\itemize{
  \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime data analysis}, 29, 441–482.
  \item Scheike TH, Zhang MJ, Gerds TA (2008). "Predicting cumulative incidence probability by direct binomial regression." \emph{Biometrika}, 95(1), 205–220.
}
}

\section{Outcome definition}{

The observed outcome \eqn{Y_i} constructed from \code{outcome} depends on whether
competing risks are present. Competing risks are detected automatically: the data are
considered to have competing risks when causes other than those specified in \code{cause}
are observed (i.e. \code{any(!(Causes \%in\% cause))}, where \code{Causes} are all
non-censoring event codes found in the data).

\strong{No competing risks} (all observed events belong to \code{cause}):
\itemize{
  \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i = 1)}. Cumulative incidence.
  \item \code{"rmst"}: \eqn{Y_i = \min(T_i, t)}. Restricted mean survival time.
  \item \code{"rmtl"}: \eqn{Y_i = t - \min(T_i, t)}. Restricted mean time lost.
}

\strong{Competing risks} (causes beyond \code{cause} are observed in the data):
\itemize{
  \item \code{"cif"}: \eqn{Y_i = I(T_i \leq t, \epsilon_i \in \code{cause})}. Cumulative
    incidence for the cause(s) of interest.
  \item \code{"rmst"} or \code{"rmtl"}: \eqn{Y_i = I(\epsilon_i \in \code{cause})(t - \min(T_i, t))}.
    Cause-specific years lost, accumulated only for subjects who experience an event in \code{cause}.
}

The default link function (\code{model = "default"}) is \code{"logit"} for \code{"cif"} and
\code{"exp"} for \code{"rmst"} and \code{"rmtl"}.
If \code{Ydirect} is supplied, outcome construction is bypassed entirely.
}

\examples{
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
# logistic regresion with IPCW binomial regression 
out <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50)
summary(out)
head(iid(out))

predict(out,data.frame(tcell=c(0,1),platelet=c(1,1)),se=TRUE)

outs <- binreg(Event(time,cause)~tcell+platelet,bmt,time=50,cens.model=~strata(tcell,platelet))
summary(outs)

## glm with IPCW weights 
outl <- logitIPCW(Event(time,cause)~tcell+platelet,bmt,time=50)
summary(outl)

##########################################
### risk-ratio of different causes #######
##########################################
data(bmt)
bmt$id <- 1:nrow(bmt)
bmt$status <- bmt$cause
bmt$strata <- 1
bmtdob <- bmt
bmtdob$strata <-2
bmtdob <- dtransform(bmtdob,status=1,cause==2)
bmtdob <- dtransform(bmtdob,status=2,cause==1)
###
bmtdob <- rbind(bmt,bmtdob)
dtable(bmtdob,cause+status~strata)

cif1 <- cif(Event(time,cause)~+1,bmt,cause=1)
cif2 <- cif(Event(time,cause)~+1,bmt,cause=2)
plot(cif1)
plot(cif2,add=TRUE,col=2)

cifs1 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=1,time=50)
cifs2 <- binreg(Event(time,cause)~tcell+platelet+age,bmt,cause=2,time=50)
summary(cifs1)
summary(cifs2)

cifdob <- binreg(Event(time,status)~-1+factor(strata)+
	 tcell*factor(strata)+platelet*factor(strata)+age*factor(strata)
	 +cluster(id),bmtdob,cause=1,time=50,cens.model=~strata(strata))
summary(cifdob)
head(iid(cifdob)) 

newdata <- data.frame(tcell=1,platelet=1,age=0,strata=1:2,id=1)
riskratio <- function(p) {
  cifdob$coef <- p
  p <- predict(cifdob,newdata,se=0)
  return(p[1]/p[2])
}
lava::estimate(cifdob,f=riskratio)

predict(cifdob,newdata)
(p1 <- predict(cifs1,newdata))
(p2 <- predict(cifs2,newdata))
p1[1,1]/p2[1,1]

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daggregate.R
\name{dcor}
\alias{dcor}
\alias{dsummary}
\alias{dstr}
\alias{dsubset}
\alias{dquantile}
\alias{dcount}
\alias{dmean}
\alias{dmeansd}
\alias{dscalar}
\alias{deval}
\alias{deval2}
\alias{dsum}
\alias{dsd}
\title{summary, tables, and correlations for data frames}
\usage{
dcor(data, y = NULL, x = NULL, use = "pairwise.complete.obs", ...)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{possible group variable}

\item{use}{how to handle missing values}

\item{...}{Optional additional arguments}
}
\description{
summary, tables, and correlations for data frames
}
\examples{
data("sTRACE",package="timereg")
dt<- sTRACE
dt$time2 <- dt$time^2
dt$wmi2 <- dt$wmi^2
head(dt)

dcor(dt)

dcor(dt,~time+wmi)
dcor(dt,~time+wmi,~vf+chf)
dcor(dt,time+wmi~vf+chf)

dcor(dt,c("time*","wmi*"),~vf+chf)
}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim.cause.cox}
\alias{sim.cause.cox}
\title{Simulation of cause specific from Cox models.}
\usage{
sim.cause.cox(coxs,n,data=NULL,cens=NULL,rrc=NULL,...)
}
\arguments{
\item{coxs}{list of cox models.}

\item{n}{number of simulations.}

\item{data}{to extract covariates for simulations (draws from observed
covariates).}

\item{cens}{specifies censoring model, if NULL then only censoring for 
  each cause at end of last event of this type. 
if "is.matrix" then uses cumulative. 
 hazard given, if "is.scalar" then uses rate for exponential, and if not
 given then takes average rate of in simulated data from cox model.
 But censoring can also be given as a cause.}

\item{rrc}{possible vector of relative risk for cox-type censoring.}

\item{...}{arguments for rchaz, for example entry-time}
}
\description{
Simulates data that looks like fit from cause specific Cox models. 
Censor data automatically. When censoring is given in the  list of causes this
will give censoring that looks like the data.  Covariates are drawn from data-set
with replacement. This gives covariates like the data.
}
\examples{
nsim <- 100

data(bmt)
# coxph          
cox1 <- coxph(Surv(time,cause==1)~tcell+platelet,data=bmt)
cox2 <- coxph(Surv(time,cause==2)~tcell+platelet,data=bmt)
coxs <- list(cox1,cox2)
dd <- sim.cause.cox(coxs,nsim,data=bmt)
scox1 <- coxph(Surv(time,status==1)~tcell+platelet,data=dd)
scox2 <- coxph(Surv(time,status==2)~tcell+platelet,data=dd)
cbind(cox1$coef,scox1$coef)
cbind(cox2$coef,scox2$coef)

data(bmt)
cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)
coxs <- list(cox1,cox2)
dd <- sim.cause.cox(coxs,nsim,data=bmt)
scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
cbind(cox1$coef,scox1$coef)
cbind(cox2$coef,scox2$coef)
par(mfrow=c(1,2))
plot(cox1); plot(scox1,add=TRUE); 
plot(cox2); plot(scox2,add=TRUE); 

cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet,data=bmt)
cox2 <- phreg(Surv(time,cause==2)~strata(tcell)+platelet,data=bmt)
coxs <- list(cox1,cox2)
dd <- sim.cause.cox(coxs,nsim,data=bmt)
scox1 <- phreg(Surv(time,status==1)~strata(tcell)+platelet,data=dd)
scox2 <- phreg(Surv(time,status==2)~strata(tcell)+platelet,data=dd)
cbind(cox1$coef,scox1$coef)
cbind(cox2$coef,scox2$coef)
par(mfrow=c(1,2))
plot(cox1); plot(scox1,add=TRUE); 
plot(cox2); plot(scox2,add=TRUE); 

}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efficient-binreg.R
\name{Effbinreg}
\alias{Effbinreg}
\title{Efficient IPCW for binary data}
\usage{
Effbinreg(
  formula,
  data,
  cause = 1,
  time = NULL,
  beta = NULL,
  offset = NULL,
  weights = NULL,
  cens.weights = NULL,
  cens.model = ~+1,
  se = TRUE,
  kaplan.meier = TRUE,
  cens.code = 0,
  no.opt = FALSE,
  method = "nr",
  augmentation = NULL,
  h = NULL,
  MCaugment = NULL,
  ...
)
}
\arguments{
\item{formula}{formula with outcome (see \code{coxph})}

\item{data}{data frame}

\item{cause}{cause of interest}

\item{time}{time of interest}

\item{beta}{starting values}

\item{offset}{offsets for partial likelihood}

\item{weights}{for score equations}

\item{cens.weights}{censoring weights}

\item{cens.model}{only stratified cox model without covariates}

\item{se}{to compute se's  based on IPCW}

\item{kaplan.meier}{uses Kaplan-Meier for IPCW in contrast to exp(-Baseline)}

\item{cens.code}{gives censoring code}

\item{no.opt}{to not optimize}

\item{method}{for optimization}

\item{augmentation}{to augment binomial regression}

\item{h}{h for estimating equation}

\item{MCaugment}{iid of h and censoring model}

\item{...}{Additional arguments to lower level funtions}

\item{model}{exp or linear}
}
\description{
Simple version of comp.risk function of timereg for just one time-point thus fitting the model 
\deqn{E(T \leq t | X ) = expit( X^T beta) }
}
\details{
Based on binomial regresion IPCW response estimating equation: 
\deqn{ X  ( \Delta (T \leq t)/G_c(T_i-) - expit( X^T beta)) = 0 }
for IPCW adjusted responses. 

Based on binomial regresion IPCW response estimating equation: 
\deqn{ h(X) X ( \Delta (T \leq t)/G_c(T_i-) - expit( X^T beta)) = 0 }
for IPCW adjusted responses where $h$ is given as an argument together with iid of censoring with h. 
By using appropriately the h argument we can also do the efficient IPCW estimator estimator this works 
the prepsurv and prepcif for survival or competing risks data. In this case also the censoring martingale 
should be given for variance calculation and this also comes out of the prepsurv or prepcif functions. 
(Experimental version at this stage).

Variance is based on  \deqn{ \sum w_i^2 } also with IPCW adjustment, and naive.var is variance 
under known censoring model. 

Censoring model may depend on strata.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipw.R
\name{ipw}
\alias{ipw}
\title{Inverse Probability of Censoring Weights}
\usage{
ipw(
  formula,
  data,
  cluster,
  same.cens = FALSE,
  obs.only = FALSE,
  weight.name = "w",
  trunc.prob = FALSE,
  weight.name2 = "wt",
  indi.weight = "pr",
  cens.model = "aalen",
  pairs = FALSE,
  theta.formula = ~1,
  ...
)
}
\arguments{
\item{formula}{Formula specifying the censoring model}

\item{data}{data frame}

\item{cluster}{clustering variable}

\item{same.cens}{For clustered data, should same censoring be assumed (bivariate probability calculated as mininum of the marginal probabilities)}

\item{obs.only}{Return data with uncensored observations only}

\item{weight.name}{Name of weight variable in the new data.frame}

\item{trunc.prob}{If TRUE truncation probabilities are also calculated and stored in 'weight.name2' (based on Clayton-Oakes gamma frailty model)}

\item{weight.name2}{Name of truncation probabilities}

\item{indi.weight}{Name of individual censoring weight  in the new data.frame}

\item{cens.model}{Censoring model (default Aalens additive model)}

\item{pairs}{For paired data (e.g. twins) only the complete pairs are returned (With pairs=TRUE)}

\item{theta.formula}{Model for the dependence parameter in the Clayton-Oakes model (truncation only)}

\item{...}{Additional arguments to censoring model}
}
\description{
Internal function.
Calculates Inverse Probability of Censoring
Weights (IPCW) and adds them to a data.frame
}
\examples{
\dontrun{
data("prt",package="mets")
prtw <- ipw(Surv(time,status==0)~country, data=prt[sample(nrow(prt),5000),],
            cluster="id",weight.name="w")
plot(0,type="n",xlim=range(prtw$time),ylim=c(0,1),xlab="Age",ylab="Probability")
count <- 0
for (l in unique(prtw$country)) {
    count <- count+1
    prtw <- prtw[order(prtw$time),]
    with(subset(prtw,country==l),
         lines(time,w,col=count,lwd=2))
}
legend("topright",legend=unique(prtw$country),col=1:4,pch=-1,lty=1)
}
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{sim_recurrent}
\alias{sim_recurrent}
\title{Simulate recurrent events with a single event type and a terminal event}
\usage{
sim_recurrent(
  n,
  cumhaz,
  death.cumhaz = NULL,
  r1 = NULL,
  rd = NULL,
  rc = NULL,
  ...
)
}
\arguments{
\item{n}{Number of subjects to simulate.}

\item{cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the cumulative
hazard of the recurrent event.}

\item{death.cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the
cumulative hazard of the terminal event. If \code{NULL}, no terminal event
is included.}

\item{r1}{Optional numeric vector of length \code{n} of subject-specific
relative risks for the recurrent event.}

\item{rd}{Optional numeric vector of length \code{n} of subject-specific
relative risks for the terminal event.}

\item{rc}{Optional numeric vector of length \code{n} of subject-specific
multipliers for the exponential censoring rate.}

\item{...}{Further arguments passed to \code{\link{sim_recurrent_list}},
including \code{dependence}, \code{var.z}, \code{gap.time}, and
\code{max.recurrent}.}
}
\description{
A convenience wrapper around \code{\link{sim_recurrentII}} for the common
case of a single recurrent event type. Frailty and censoring options are
passed through to \code{\link{sim_recurrent_list}}.
}
\examples{
data(CPH_HPN_CRBSI)
dr    <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi
base4 <- CPH_HPN_CRBSI$mechanical

## Single recurrent event type, with and without terminal event
rr <- sim_recurrent(5, base1)
dlist(rr, . ~ id, n = 0)

rr <- sim_recurrent(5, base1, death.cumhaz = dr)
dlist(rr, . ~ id, n = 0)

## Verify that estimated rates recover the true baselines (increase n for precision)
rr <- sim_recurrent(100, base1, death.cumhaz = dr)
par(mfrow = c(1, 3))
mets:::showfitsim(causes = 1, rr, dr, base1, base1)

## Shared frailty across all processes
rr <- sim_recurrent(100, base1, death.cumhaz = dr, dependence = 1, var.z = 0.4)
dtable(rr, ~death + status)

## Two event types; second type uses the mechanical complication rate
set.seed(100)
rr <- sim_recurrentII(100, base1, base4, death.cumhaz = dr)
dtable(rr, ~death + status)
par(mfrow = c(2, 2))
mets:::showfitsim(causes = 2, rr, dr, base1, base4)

## Three event types and two causes of death via sim_recurrent_list
set.seed(100)
cumhaz <- list(base1, base1, base4)
drl    <- list(dr, base4)
rr     <- sim_recurrent_list(100, cumhaz, death.cumhaz = drl, dependence = 0)
dtable(rr, ~death + status)
mets:::showfitsimList(rr, cumhaz, drl)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bicomprisk.R
\name{bicomprisk}
\alias{bicomprisk}
\alias{bicompriskData}
\title{Estimation of Concordance in Bivariate Competing Risks Data}
\usage{
bicomprisk(
  formula,
  data,
  cause = c(1, 1),
  cens = 0,
  causes,
  indiv,
  strata = NULL,
  id,
  num,
  max.clust = 1000,
  marg = NULL,
  se.clusters = NULL,
  wname = NULL,
  prodlim = FALSE,
  messages = TRUE,
  model,
  return.data = 0,
  uniform = 0,
  conservative = 1,
  resample.iid = 1,
  ...
)
}
\arguments{
\item{formula}{Formula with an \code{Event} object on the left-hand side. The right-hand 
side specifies the covariate structure, including \code{strata()} for grouping (e.g., MZ/DZ) 
and \code{id()} for pairing.}

\item{data}{Data frame containing the variables.}

\item{cause}{Vector of cause codes for which to estimate the bivariate cumulative incidence 
(default \code{c(1,1)}).}

\item{cens}{Censoring code (default 0).}

\item{causes}{Vector of all possible causes (optional, inferred from data if missing).}

\item{indiv}{Variable indicating individual within a pair (optional, inferred from \code{id}).}

\item{strata}{Variable for stratification (optional, can be specified in formula).}

\item{id}{Clustering variable (pair ID). Required.}

\item{num}{Variable for numbering individuals within pairs (optional, auto-generated if missing).}

\item{max.clust}{Maximum number of clusters to use for IID decomposition in \code{timereg::comp.risk}. 
If NULL, uses all clusters. Useful for large datasets to speed up computation.}

\item{marg}{Optional marginal cumulative incidence object (from \code{comp.risk}) to compute 
standard errors for same-cluster comparisons in subsequent \code{casewise.test()}.}

\item{se.clusters}{Vector of cluster indices or column name in \code{data} for standard error calculation. 
Defaults to the \code{id} variable.}

\item{wname}{Name of an additional weight variable for paired competing risks data.}

\item{prodlim}{Logical; if TRUE, uses the \code{prodlim} (Aalen-Johansen) estimator instead of 
the IPCW estimator based on \code{comp.risk}. These are equivalent in the absence of covariates.}

\item{messages}{Control amount of output (0 = silent, 1 = messages).}

\item{model}{Type of competing risk model for \code{comp.risk} (default "fg" for Fine-Gray).}

\item{return.data}{If 1, returns the reshaped data; if 2, returns only the data; otherwise returns the model.}

\item{uniform}{Logical; if TRUE, computes uniform standard errors based on resampling.}

\item{conservative}{Logical; if TRUE, uses conservative standard errors (recommended for large datasets).}

\item{resample.iid}{Logical; if TRUE, returns IID residual processes for further computations.}

\item{...}{Additional arguments passed to \code{timereg::comp.risk}.}
}
\value{
An object of class \code{"bicomprisk"} (or \code{"bicomprisk.strata"} if stratified) containing:
  \item{model}{List of fitted models for each stratum (or a single model).}
  \item{strata}{Names of strata (if applicable).}
  \item{N}{Number of strata (if applicable).}
  \item{time}{Event times.}
  \item{P1}{Bivariate cumulative incidence estimates.}
  \item{se.P1}{Standard errors of the estimates.}
  \item{P1.iid}{IID decomposition (if \code{resample.iid=TRUE}).}
  \item{clusters}{Cluster assignments (if \code{marg} or \code{se.clusters} provided).}
}
\description{
Estimates the bivariate cumulative incidence function (concordance) for paired 
data (e.g., twins, family members) in the presence of competing risks. The function 
handles both the IPCW (Inverse Probability of Censoring Weighting) estimator and 
the Aalen-Johansen estimator (via \code{prodlim}).
}
\details{
The concordance function \eqn{C(t)} is defined as the probability that both members 
of a pair experience the event of interest by time \eqn{t}:
\deqn{ C(t) = P(T_1 \leq t, T_2 \leq t, \epsilon_1 = k, \epsilon_2 = k) }
where \eqn{T_i} is the event time and \eqn{\epsilon_i} is the cause of failure for individual \eqn{i}.

The function supports:
\itemize{
  \item Stratified analysis (e.g., by zygosity in twin studies).
  \item Clustering for robust standard errors.
  \item Both IPCW and Aalen-Johansen estimation methods.
  \item Resampling for uniform standard errors.
  \item IID decomposition for further inference (e.g., casewise concordance tests).
}
}
\examples{
library("timereg")

## Simulated data example
prt <- sim_nordic_random(2000,delayed=TRUE,ptrunc=0.7,
	      cordz=0.5,cormz=2,lam0=0.3)
## Bivariate competing risk, concordance estimates
p11 <- bicomprisk(Event(time,cause)~strata(zyg)+id(id),data=prt,cause=c(1,1))

p11mz <- p11$model$"MZ"
p11dz <- p11$model$"DZ"
par(mfrow=c(1,2))
## Concordance
plot(p11mz,ylim=c(0,0.1));
plot(p11dz,ylim=c(0,0.1));

## Entry time, truncation weighting
### Other weighting procedure
prtl <-  prt[!prt$truncated,]
prt2 <- ipw2(prtl,cluster="id",same.cens=TRUE,
     time="time",cause="cause",entrytime="entry",
     pairs=TRUE,strata="zyg",obs.only=TRUE)

prt22 <- fast.reshape(prt2,id="id")

prt22$event <- (prt22$cause1==1)*(prt22$cause2==1)*1
prt22$timel <- pmax(prt22$time1,prt22$time2)
ipwc <- timereg::comp.risk(Event(timel,event)~-1+factor(zyg1),
  data=prt22,cause=1,n.sim=0,model="rcif2",times=50:90,
  weights=prt22$weights1,cens.weights=rep(1,nrow(prt22)))

p11wmz <- ipwc$cum[,2]
p11wdz <- ipwc$cum[,3]
lines(ipwc$cum[,1],p11wmz,col=3)
lines(ipwc$cum[,1],p11wdz,col=3)
}
\references{
Scheike, T. H.; Holst, K. K. & Hjelmborg, J. B. (2014). Estimating twin concordance for bivariate competing risks twin data. Statistics in Medicine, 33, 1193-1204.
}
\seealso{
\code{\link{test_casewise}}, \code{\link{casewise}}
}
\author{
Thomas Scheike, Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtransform.R
\name{dtransform}
\alias{dtransform}
\alias{dtransform<-}
\alias{dtrans}
\alias{dtrans<-}
\title{Transform that allows condition}
\usage{
dtransform(data, ...)
}
\arguments{
\item{data}{is data frame}

\item{...}{new variable definitions including possible if condition}
}
\description{
Defines new variables under condition for data frame
}
\examples{
data(mena)

xx <- dtransform(mena,ll=log(agemena)+twinnum)

xx <- dtransform(mena,ll=log(agemena)+twinnum,agemena<15)
xx <- dtransform(xx  ,ll=100+agemena,ll2=1000,agemena>15)
dsummary(xx,ll+ll2~I(agemena>15))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{sim_recurrentII}
\alias{sim_recurrentII}
\alias{sim_recurrent_list}
\title{Simulate recurrent events with two event types and a terminal event}
\usage{
sim_recurrentII(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz = NULL,
  r1 = NULL,
  r2 = NULL,
  rd = NULL,
  rc = NULL,
  dependence = 0,
  var.z = 1,
  cor.mat = NULL,
  cens = NULL,
  gap.time = FALSE,
  max.recurrent = 100,
  ...
)
}
\arguments{
\item{n}{Number of subjects to simulate.}

\item{cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the cumulative
hazard of the first type of recurrent event.}

\item{cumhaz2}{Two-column matrix \code{(time, cumhaz)} giving the cumulative
hazard of the second type of recurrent event.}

\item{death.cumhaz}{Two-column matrix \code{(time, cumhaz)} giving the
cumulative hazard of the terminal event. If \code{NULL}, no terminal event
is simulated and follow-up ends at the end of \code{cumhaz}.}

\item{r1}{Optional numeric vector of length \code{n} with subject-specific
relative risk multipliers for the first event type.}

\item{r2}{Optional numeric vector of length \code{n} with subject-specific
relative risk multipliers for the second event type.}

\item{rd}{Optional numeric vector of length \code{n} with subject-specific
relative risk multipliers for the terminal event.}

\item{rc}{Optional numeric vector of length \code{n} with subject-specific
multipliers for the exponential censoring rate.}

\item{dependence}{Integer specifying the frailty structure. One of \code{0}
(independence), \code{1} (shared gamma frailty), or \code{4} (shared frailty
for recurrent events only). See Details.}

\item{var.z}{Variance of the gamma-distributed frailty. Default is \code{1}.}

\item{cor.mat}{Correlation matrix for the random effects. Used when
\code{dependence = 2} (in \code{\link{sim_recurrent_list}}).}

\item{cens}{Rate of exponential censoring. If \code{NULL} (default), no
additional censoring is applied.}

\item{gap.time}{Logical. If \code{TRUE}, event times are drawn as gap times
(time since the last event) rather than calendar times. Default is
\code{FALSE}.}

\item{max.recurrent}{Maximum number of recurrent events allowed per subject.
Default is \code{100}.}

\item{...}{Further arguments passed to \code{\link{sim_recurrent_list}}.}
}
\value{
A data frame in counting-process format (one row per event interval
  per subject) with columns:
  \item{id}{Subject identifier.}
  \item{start, entry}{Interval start time.}
  \item{stop, time}{Interval end time (event or censoring time).}
  \item{status}{Event type at \code{stop}: \code{1} or \code{2} for a
    recurrent event of the corresponding type, \code{0} for censoring.}
  \item{death}{Indicator for a terminal event (\code{1}) or censoring/survival
    (\code{0}).}
  Attributes \code{"cumhaz"}, \code{"death.cumhaz"}, \code{"rr"}, and
  \code{"rd"} store the inputs used for simulation.
}
\description{
Simulates recurrent event data with up to two distinct event types and an
optional terminal event (death), based on user-supplied cumulative hazard
functions. Dependence between processes can be introduced via shared or
correlated gamma-distributed frailties.
}
\details{
The simulation proceeds by sequentially drawing the next event time from the
specified cumulative hazards, taking the minimum of the two recurrent event
times, and stopping each subject at death or administrative censoring.

Dependence between processes is controlled by \code{dependence}:
\describe{
  \item{\code{0}}{Independence: all subjects have frailty fixed at 1.}
  \item{\code{1}}{Shared frailty: all processes share a single
    gamma-distributed random effect with mean 1 and variance \code{var.z}.}
  \item{\code{4}}{Recurrent-event frailty only: the two recurrent event
    processes share a gamma frailty but the terminal event is independent.}
}
For more complex correlation structures across two event types and death, use
\code{\link{sim_recurrentTS}}.
}
\examples{
data(CPH_HPN_CRBSI)
dr    <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi
base4 <- CPH_HPN_CRBSI$mechanical

## Single recurrent event type, with and without terminal event
rr <- sim_recurrent(5, base1)
dlist(rr, . ~ id, n = 0)

rr <- sim_recurrent(5, base1, death.cumhaz = dr)
dlist(rr, . ~ id, n = 0)

## Verify that estimated rates recover the true baselines (increase n for precision)
rr <- sim_recurrent(100, base1, death.cumhaz = dr)
par(mfrow = c(1, 3))
mets:::showfitsim(causes = 1, rr, dr, base1, base1)

## Shared frailty across all processes
rr <- sim_recurrent(100, base1, death.cumhaz = dr, dependence = 1, var.z = 0.4)
dtable(rr, ~death + status)

## Two event types; second type uses the mechanical complication rate
set.seed(100)
rr <- sim_recurrentII(100, base1, base4, death.cumhaz = dr)
dtable(rr, ~death + status)
par(mfrow = c(2, 2))
mets:::showfitsim(causes = 2, rr, dr, base1, base4)

## Three event types and two causes of death via sim_recurrent_list
set.seed(100)
cumhaz <- list(base1, base1, base4)
drl    <- list(dr, base4)
rr     <- sim_recurrent_list(100, cumhaz, death.cumhaz = drl, dependence = 0)
dtable(rr, ~death + status)
mets:::showfitsimList(rr, cumhaz, drl)

}
\seealso{
\code{\link{sim_recurrent}}, \code{\link{sim_recurrent_list}},
  \code{\link{sim_recurrentTS}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{base44cumhaz}
\alias{base44cumhaz}
\title{rate of Occlusion/Thrombosis complication for catheter of HPN patients of Copenhagen}
\source{
Estimated data
}
\description{
rate of Occlusion/Thrombosis complication for catheter of HPN patients of Copenhagen
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dsort.R
\name{dsort}
\alias{dsort}
\alias{dsort2}
\alias{dsort<-}
\title{Sort data frame}
\usage{
dsort(data, x, ..., decreasing = FALSE, return.order = FALSE)
}
\arguments{
\item{data}{Data frame}

\item{x}{variable to order by}

\item{...}{additional variables to order by}

\item{decreasing}{sort order (vector of length x)}

\item{return.order}{return order}
}
\value{
data.frame
}
\description{
Sort data according to columns in data frame
}
\examples{
data(data="hubble",package="lava")
dsort(hubble, "sigma")
dsort(hubble, hubble$sigma,"v")
dsort(hubble,~sigma+v)
dsort(hubble,~sigma-v)

## with direct asignment
dsort(hubble) <- ~sigma-v
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastpattern.R
\name{fast.pattern}
\alias{fast.pattern}
\title{Fast pattern}
\usage{
fast.pattern(x, y, categories = 2, ...)
}
\arguments{
\item{x}{Matrix (binary) of patterns. Optionally if \code{y} is
also passed as argument, then the pattern matrix is defined as the
elements agreeing in the two matrices.}

\item{y}{Optional matrix argument with same dimensions as \code{x} (see above)}

\item{categories}{Default 2 (binary)}

\item{...}{Optional additional arguments}
}
\description{
Fast pattern
}
\examples{
X <- matrix(rbinom(100,1,0.5),ncol=4)
fast.pattern(X)

X <- matrix(rbinom(100,3,0.5),ncol=4)
fast.pattern(X,categories=4)
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\name{npc}
\alias{npc}
\alias{plotcr}
\alias{nonparcuminc}
\alias{simnordic}
\alias{corsim.prostate}
\alias{alpha2kendall}
\alias{alpha2spear}
\alias{coefmat}
\alias{piecewise.twostage}
\alias{surv.boxarea}
\alias{faster.reshape}
\alias{piecewise.data}
\alias{simBinPlack}
\alias{simBinFam}
\alias{simBinFam2}
\alias{simSurvFam}
\alias{corsim.prostate.random}
\alias{simnordic.random}
\alias{simCox}
\alias{sim}
\alias{grouptable}
\alias{jumptimes}
\alias{folds}
\alias{ace.family.design}
\alias{ascertained.pairs}
\alias{CCbinomial.twostage}
\alias{coarse.clust}
\alias{concordanceTwinACE}
\alias{concordanceTwostage}
\alias{fast.cluster}
\alias{force.same.cens}
\alias{ilap}
\alias{kendall.ClaytonOakes.twin.ace}
\alias{kendall.normal.twin.ace}
\alias{make.pairwise.design}
\alias{make.pairwise.design.competing}
\alias{matplot.mets.twostage}
\alias{object.defined}
\alias{p11.binomial.twostage.RV}
\alias{predictPairPlack}
\alias{simbinClaytonOakes.family.ace}
\alias{simbinClaytonOakes.pairs}
\alias{simbinClaytonOakes.twin.ace}
\alias{simClaytonOakes.family.ace}
\alias{simClaytonOakes.twin.ace}
\alias{simFrailty.simple}
\alias{simCompete.simple}
\alias{simCompete.twin.ace}
\alias{twin.polygen.design}
\alias{procform}
\alias{procform3}
\alias{procformdata}
\alias{drop.specials}
\title{For internal use}
\description{
For internal use
}
\author{
Klaus K. Holst
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{mena}
\alias{mena}
\title{Menarche data set}
\source{
Simulated data
}
\description{
Menarche data set
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{test_conc}
\alias{test_conc}
\title{Compare Two Concordance Estimates}
\usage{
test_conc(conc1, conc2, same.cluster = FALSE)
}
\arguments{
\item{conc1}{Concordance estimate of group 1.}

\item{conc2}{Concordance estimate of group 2.}

\item{same.cluster}{Logical; if FALSE, groups are assumed independent. If TRUE, 
estimates are based on the same data (e.g., paired twins).}
}
\value{
An object of class \code{"testconc"} containing:
  \item{test}{Matrix with cumulative difference, standard error, z-value, and p-value.}
  \item{mintime, maxtime}{Time range used.}
  \item{same.cluster}{Logical flag.}
}
\description{
Performs a Pepe-Mori type test to compare two concordance estimates (e.g., MZ vs DZ twins).
The test evaluates whether the concordance functions differ significantly over time.
}
\seealso{
\code{\link{test_casewise}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm-utils.R
\name{glm_IPTW}
\alias{glm_IPTW}
\title{IPTW GLM, Inverse Probaibilty of Treatment Weighted GLM}
\usage{
glm_IPTW(
  formula,
  data,
  treat.model = NULL,
  family = binomial(),
  id = NULL,
  weights = NULL,
  estpr = 1,
  pi0 = 0.5,
  ...
)
}
\arguments{
\item{formula}{for glm}

\item{data}{data frame for risk averaging}

\item{treat.model}{propensity score model (binary or multinomial)}

\item{family}{of glm (logistic regression)}

\item{id}{cluster id for standard errors}

\item{weights}{may be given, and then uses weights*w(A) as the weights}

\item{estpr}{to estimate propensity scores and get infuence function contribution to uncertainty}

\item{pi0}{fixed simple weights}

\item{...}{arguments for glm call}
}
\description{
Fits GLM model with treatment weights \deqn{ w(A)= \sum_a I(A=a)/P(A=a|X) }, computes
standard errors via influence functions that are returned as the IID argument. 
Propensity scores are fitted using either logistic regression (glm) or the multinomial model (mlogit) when more
than two categories for treatment. The treatment needs to be a factor and is identified on the rhs
of the "treat.model".
}
\details{
Also works with cluster argument.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{ghaplos}
\alias{ghaplos}
\title{ghaplos  haplo-types for subjects of haploX data}
\source{
Simulated data
}
\description{
ghaplos  haplo-types for subjects of haploX data
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{plot.phreg}
\alias{plot.phreg}
\alias{basehazplotO.phreg}
\alias{baseplot}
\alias{bplot}
\alias{basecumhaz}
\alias{plotConfRegion}
\alias{plotConfRegionSE}
\alias{plotstrata}
\alias{kmplot}
\alias{plotConfregion}
\title{Plotting the baselines of stratified Cox}
\usage{
\method{plot}{phreg}(x, ...)
}
\arguments{
\item{x}{phreg object}

\item{...}{Additional arguments to baseplot funtion}
}
\description{
Plotting the baselines of stratified Cox
}
\examples{
data(TRACE)
dcut(TRACE) <- ~.
out1 <- phreg(Surv(time,status==9)~vf+chf+strata(wmicat.4),data=TRACE)

par(mfrow=c(2,2))
plot(out1)
plot(out1,stratas=c(0,3))
plot(out1,stratas=c(0,3),col=2:3,lty=1:2,se=TRUE)
plot(out1,stratas=c(0),col=2,lty=2,se=TRUE,polygon=FALSE)
plot(out1,stratas=c(0),col=matrix(c(2,1,3),1,3),lty=matrix(c(1,2,3),1,3),se=TRUE,polygon=FALSE)
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv.boxarea.R
\name{surv_boxarea}
\alias{surv_boxarea}
\title{Bivariate Survival Data on Rectangular Regions}
\usage{
surv_boxarea(
  left.trunc,
  right.cens,
  data,
  timevar = "time",
  status = "status",
  id = "id",
  covars = NULL,
  covars.pairs = NULL,
  num = NULL,
  silent = 1,
  boxtimevar = "boxtime"
)
}
\arguments{
\item{left.trunc}{vector of length 2 giving left truncation times.}

\item{right.cens}{vector of length 2 giving right censoring times.}

\item{data}{a data.frame with the survival data.}

\item{timevar}{name of the time variable.}

\item{status}{name of the status variable.}

\item{id}{name of the cluster identifier.}

\item{covars}{optional covariate names.}

\item{covars.pairs}{optional pair-level covariate names.}

\item{num}{within-cluster numbering variable name.}

\item{silent}{verbosity level (1=silent).}

\item{boxtimevar}{name for the created box-time variable.}
}
\value{
A data.frame in long format restricted to the specified region.
}
\description{
Restricts bivariate survival data to a rectangular time region defined
by left-truncation and right-censoring boundaries, for use with
piecewise twostage models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim.phreg}
\alias{sim.phreg}
\alias{draw.phreg}
\alias{setup.phreg}
\title{Simulation of output from Cox model.}
\usage{
sim.phreg(
  cox,
  n,
  data = NULL,
  Z = NULL,
  rr = NULL,
  strata = NULL,
  entry = NULL,
  extend = NULL,
  cens = NULL,
  rrc = NULL,
  ...
)
}
\arguments{
\item{cox}{output form coxph or cox.aalen model fitting cox model.}

\item{n}{number of simulations.}

\item{data}{to extract covariates for simulations (draws from observed covariates).}

\item{Z}{give design matrix instead of data}

\item{rr}{possible vector of relative risk for cox model.}

\item{strata}{possible vector of strata}

\item{entry}{delayed entry variable for simulation.}

\item{extend}{to extend possible stratified baselines to largest end-point 
given then takes average rate of in simulated data from cox model.}

\item{cens}{specifies censoring model, if "is.matrix" then uses cumulative
hazard given, if "is.scalar" then uses rate for exponential, and if not}

\item{rrc}{possible vector of relative risk for cox-type censoring.}

\item{...}{arguments for rchaz, for example entry-time.}
}
\description{
Simulates data that looks like fit from Cox model. Censor data automatically
for highest value of the event times by using cumulative hazard.
}
\examples{
library(mets)
data(sTRACE)
nsim <- 100
coxs <-  phreg(Surv(time,status==9)~strata(chf)+vf+wmi,data=sTRACE)
set.seed(100)
sim3 <- sim.phreg(coxs,nsim,data=sTRACE)
head(sim3)
cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim3)
cbind(coxs$coef,cc$coef)
plot(coxs,col=1); plot(cc,add=TRUE,col=2)

Z <- sim3[,c("vf","chf","wmi")]
strata <- sim3[,c("chf")]
rr <- exp(as.matrix(Z[,-2]) \%*\% coef(coxs))
sim4 <- sim.phreg(coxs,nsim,data=NULL,rr=rr,strata=strata)
sim4 <- cbind(sim4,Z)
cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim4)
cbind(coxs$coef,cc$coef)
plot(coxs,col=1); plot(cc,add=TRUE,col=2)

}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twin.clustertrunc.r
\name{twin.clustertrunc}
\alias{twin.clustertrunc}
\title{Estimation of twostage model with cluster truncation in bivariate situation}
\usage{
twin.clustertrunc(
  survformula,
  data = parent.frame(),
  theta.des = NULL,
  clusters = NULL,
  var.link = 1,
  Nit = 10,
  final.fitting = FALSE,
  ...
)
}
\arguments{
\item{survformula}{Formula with survival model aalen or cox.aalen, some limitiation on model specification due to call of fast.reshape (so for example interactions and * and : do not work here, expand prior to call)}

\item{data}{Data frame}

\item{theta.des}{design for dependence parameters in two-stage model}

\item{clusters}{clustering variable for twins}

\item{var.link}{exp link for theta}

\item{Nit}{number of iteration}

\item{final.fitting}{TRUE to do final estimation with SE and ... arguments for marginal models}

\item{...}{Additional arguments to lower level functions}
}
\description{
Estimation of twostage model with cluster truncation in bivariate situation
}
\examples{
library("timereg")
data(diabetes)
v <- diabetes$time*runif(nrow(diabetes))*rbinom(nrow(diabetes),1,0.5)
diabetes$v <- v

aout <- twin.clustertrunc(Surv(v,time,status)~1+treat+adult,
		 data=diabetes,clusters="id")
aout$two        ## twostage output
par(mfrow=c(2,2))
plot(aout$marg) ## marginal model output

out <- twin.clustertrunc(Surv(v,time,status)~1+prop(treat)+prop(adult),
		 data=diabetes,clusters="id")
out$two        ## twostage output
plot(out$marg) ## marginal model output
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{aalenMets}
\alias{aalenMets}
\title{Fast Additive Hazards Model with Robust Standard Errors}
\usage{
aalenMets(formula, data = data, no.baseline = FALSE, ...)
}
\arguments{
\item{formula}{Formula with a 'Surv' outcome (similar to \code{coxph}).}

\item{data}{Data frame.}

\item{no.baseline}{Logical; if \code{TRUE}, fits the model without baseline hazard estimation.}

\item{...}{Additional arguments passed to \code{phreg}.}
}
\value{
An object of class \code{"aalenMets"} (extends \code{"phreg"}) containing:
\item{coef}{Estimated coefficients.}
\item{var}{Robust variance-covariance matrix.}
\item{iid}{Influence functions.}
\item{intZHZ}{Integrated ZHZ matrix.}
\item{gamma}{Coefficient estimates.}
}
\description{
Fits a fast Lin-Ying additive hazards model with a possibly stratified baseline.
Robust variance is the default variance estimate in the summary.
}
\details{
Influence functions (IID) follow the numerical order of the given cluster variable.
Ordering by \code{$id} aligns the IID terms with the dataset order.
}
\examples{
data(bmt)
bmt$time <- bmt$time + runif(408) * 0.001
out <- aalenMets(Surv(time, cause == 1) ~ tcell + platelet + age, data = bmt)
summary(out)

## Comparison with timereg::aalen
## out2 <- timereg::aalen(
##   Surv(time, cause == 1) ~ const(tcell) + const(platelet) + const(age),
##   data = bmt
## )
## summary(out2)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{sim_recurrent_ts}
\alias{sim_recurrent_ts}
\title{Simulate recurrent events from a two-stage Cox or Ghosh-Lin model}
\usage{
sim_recurrent_ts(
  cox1,
  coxd = NULL,
  n = 1,
  data = NULL,
  type = c("default", "cox-cox", "gl-cox"),
  id = "id",
  varz = 1,
  share = 1,
  cens = 0.001,
  scale1 = 1,
  scaled = 1,
  dependence = NULL,
  r1 = NULL,
  rd = NULL,
  rc = NULL,
  strata1 = NULL,
  stratad = NULL,
  death.code = 3,
  ...
)
}
\arguments{
\item{cox1}{A fitted \code{phreg} object for the recurrent event rate, or a
fitted \code{recreg} (Ghosh-Lin) object. The model type is detected
automatically from the class.}

\item{coxd}{A fitted \code{phreg} object for the terminal event. May be
\code{NULL} if no terminal event is modelled.}

\item{n}{Number of subjects to simulate. Default is \code{1}.}

\item{data}{The data frame on which \code{cox1} and \code{coxd} were fitted,
used to draw covariate values for the simulated subjects. If \code{NULL},
covariates must be supplied via \code{r1}, \code{rd}, \code{strata1}, and
\code{stratad}.}

\item{type}{Simulation type: \code{"default"} (auto-detected from class of
\code{cox1}), \code{"cox-cox"}, or \code{"gl-cox"}.}

\item{id}{Name of the subject identifier variable in \code{data}. Default is
\code{"id"}.}

\item{varz}{Variance of the frailty distribution in the two-stage model.
Default is \code{1}.}

\item{share}{Proportion of the shared frailty assigned to the recurrent event
process in the partial-sharing model. Default is \code{1}.}

\item{cens}{Rate of exponential censoring. Default is \code{0.001}.}

\item{scale1}{Scalar multiplier for the baseline cumulative hazard of the
recurrent event process. Default is \code{1}.}

\item{scaled}{Scalar multiplier for the baseline cumulative hazard of the
terminal event. Default is \code{1}.}

\item{dependence}{If non-\code{NULL}, falls back to
\code{\link{sim_recurrent_list}} with this frailty structure (see
\code{\link{sim_recurrentII}} for valid values). Default is \code{NULL}.}

\item{r1}{Optional numeric vector of length \code{n} of subject-specific
relative risks for the recurrent event, used when \code{data = NULL}.}

\item{rd}{Optional numeric vector of length \code{n} of subject-specific
relative risks for the terminal event, used when \code{data = NULL}.}

\item{rc}{Optional numeric vector of length \code{n} of subject-specific
censoring rate multipliers.}

\item{strata1}{Optional integer vector of length \code{n} specifying the
stratum index (0-based) for the recurrent event model, used when
\code{data = NULL}.}

\item{stratad}{Optional integer vector of length \code{n} specifying the
stratum index (0-based) for the terminal event model, used when
\code{data = NULL}.}

\item{death.code}{Integer status code used for the terminal event in the
output \code{status} column. Default is \code{3}.}

\item{...}{Further arguments passed to \code{\link{sim_GLcox}}, including
\code{nmin} and \code{nmax} for the linear approximation grid.}
}
\value{
A data frame in counting-process format with one row per event
  interval per subject. Column names match those in the original model
  formula (entry, exit, and status variables). Additional columns include
  \code{id} and the covariates drawn from \code{data} (if supplied). The
  terminal event is coded as \code{death.code} in the status variable;
  recurrent events are coded as \code{1}.
}
\description{
Simulates recurrent event data from a fitted two-stage model, where the
recurrent event process and the terminal event are each described by a
separate fitted model. The recurrent event model may be either a Cox
proportional hazards model (\code{phreg}) or a Ghosh-Lin marginal rate model
(\code{recreg}); the terminal event model must be a Cox model (\code{phreg}).
}
\details{
Covariates are drawn by bootstrap from \code{data} (if supplied), and
subject-specific relative risks and strata are derived from the fitted
model objects. Stratified baselines are fully supported. When
\code{dependence} is \code{NULL} (default), the simulation uses the
two-stage structure from \code{\link{sim_GLcox}}; setting \code{dependence}
to an integer falls back to \code{\link{sim_recurrent_list}} with the
corresponding frailty model.
}
\examples{
data(hfactioncpx12)
hf    <- hfactioncpx12
hf$x  <- as.numeric(hf$treatment)
n     <- 100

## Cox-Cox two-stage model
xr <- phreg(Surv(entry, time, status == 1) ~ x + cluster(id), data = hf)
dr <- phreg(Surv(entry, time, status == 2) ~ x + cluster(id), data = hf)
simcoxcox <- sim_recurrent_ts(xr, dr, n = n, data = hf, death.code = 2)

## Ghosh-Lin/Cox two-stage model
recGL  <- recreg(Event(entry, time, status) ~ x + cluster(id), hf, death.code = 2)
simglcox <- sim_recurrent_ts(recGL, dr, n = n, data = hf, death.code = 2)

}
\references{
Scheike, T. H. (2026). Two-stage recurrent events random effects models.
\emph{Lifetime Data Analysis}.
}
\seealso{
\code{\link{recurrent_marginal}}, \code{\link{sim_recurrent_list}},
  \code{\link{sim_GLcox}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_multistate}
\alias{sim_multistate}
\title{Simulation of Illness-Death Model}
\usage{
sim_multistate(
  n,
  cumhaz,
  cumhaz2,
  death.cumhaz,
  death.cumhaz2,
  rr12 = NULL,
  rr21 = NULL,
  rd13 = NULL,
  rd23 = NULL,
  rrc = NULL,
  gap.time = FALSE,
  max.recurrent = 100,
  dependence = 0,
  var.z = 0.5,
  cor.mat = NULL,
  cens = NULL,
  extend = TRUE,
  ...
)
}
\arguments{
\item{n}{Number of IDs.}

\item{cumhaz}{Cumulative hazard from state 1 to 2.}

\item{cumhaz2}{Cumulative hazard from state 2 to 1.}

\item{death.cumhaz}{Cumulative hazard of death from state 1.}

\item{death.cumhaz2}{Cumulative hazard of death from state 2.}

\item{rr12}{Relative risk for 1->2.}

\item{rr21}{Relative risk for 2->1.}

\item{rd13}{Relative risk for death 1->3.}

\item{rd23}{Relative risk for death 2->3.}

\item{rrc}{Relative risk for censoring.}

\item{gap.time}{Gap time indicator. If true simulates gap-times with specified cumulative hazard.}

\item{max.recurrent}{Maximum recurrent events.}

\item{dependence}{Dependence structure (0-3).}

\item{var.z}{Variance of random effects.}

\item{cor.mat}{Correlation matrix.}

\item{cens}{Censoring rate.}

\item{extend}{Extend hazards.}

\item{...}{Additional arguments.}
}
\value{
Data frame with multi-state event history.
}
\description{
Simulates data from a full illness-death model with reversible transitions 
and multiple causes of death. Supports various dependence structures via 
shared frailties.
}
\examples{
########################################
## getting some rates to mimick 
########################################
data(CPH_HPN_CRBSI)
dr <- CPH_HPN_CRBSI$terminal
base1 <- CPH_HPN_CRBSI$crbsi 
base4 <- CPH_HPN_CRBSI$mechanical
dr2 <- scalecumhaz(dr,1.5)
cens <- rbind(c(0,0),c(2000,0.5),c(5110,3))

iddata <- sim_multistate(100,base1,base1,dr,dr2,cens=cens)
dlist(iddata,.~id|id<3,n=0)
 
### estimating rates from simulated data  
c0 <- phreg(Surv(start,stop,status==0)~+1,iddata)
c3 <- phreg(Surv(start,stop,status==3)~+strata(from),iddata)
c1 <- phreg(Surv(start,stop,status==1)~+1,subset(iddata,from==2))
c2 <- phreg(Surv(start,stop,status==2)~+1,subset(iddata,from==1))
###
par(mfrow=c(2,3))
plot(c0)
lines(cens,col=2) 
plot(c3,main="rates 1-> 3 , 2->3")
lines(dr,col=1,lwd=2)
lines(dr2,col=2,lwd=2)
###
plot(c1,main="rate 1->2")
lines(base1,lwd=2)
###
plot(c2,main="rate 2->1")
lines(base1,lwd=2)
 
}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tetrachor.R
\name{tetrachoric}
\alias{tetrachoric}
\alias{or2prob}
\title{Estimate parameters from odds-ratio}
\usage{
tetrachoric(P, OR, approx = 0, ...)
}
\arguments{
\item{P}{Joint probabilities or marginals (if OR is given)}

\item{OR}{Odds-ratio}

\item{approx}{If TRUE an approximation of the tetrachoric correlation is used}

\item{...}{Additional arguments}
}
\description{
Calculate tetrachoric correlation of probabilities from odds-ratio
}
\examples{
tetrachoric(0.3,1.25) # Marginal p1=p2=0.3, OR=2
P <- matrix(c(0.1,0.2,0.2,0.5),2)
prod(diag(P))/prod(lava::revdiag(P))
##mets:::assoc(P)
tetrachoric(P)
or2prob(2,0.1)
or2prob(2,c(0.1,0.2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.twostage.R
\name{binomial.twostage}
\alias{binomial.twostage}
\alias{binomial.twostage.time}
\title{Fits Clayton-Oakes or bivariate Plackett (OR) models for binary data
using marginals that are on logistic form.
If clusters contain more than two times, the algoritm uses a compososite likelihood
based on all pairwise bivariate models.}
\usage{
binomial.twostage(
  margbin,
  data = parent.frame(),
  method = "nr",
  detail = 0,
  clusters = NULL,
  silent = 1,
  weights = NULL,
  theta = NULL,
  theta.des = NULL,
  var.link = 0,
  var.par = 1,
  var.func = NULL,
  iid = 1,
  notaylor = 1,
  model = "plackett",
  marginal.p = NULL,
  beta.iid = NULL,
  Dbeta.iid = NULL,
  strata = NULL,
  max.clust = NULL,
  se.clusters = NULL,
  numDeriv = 0,
  random.design = NULL,
  pairs = NULL,
  dim.theta = NULL,
  additive.gamma.sum = NULL,
  pair.ascertained = 0,
  case.control = 0,
  no.opt = FALSE,
  twostage = 1,
  beta = NULL,
  ...
)
}
\arguments{
\item{margbin}{Marginal binomial model}

\item{data}{data frame}

\item{method}{Scoring method "nr", for lava NR optimizer}

\item{detail}{Detail}

\item{clusters}{Cluster variable}

\item{silent}{Debug information}

\item{weights}{Weights for log-likelihood, can be used for each type of outcome in 2x2 tables.}

\item{theta}{Starting values for variance components}

\item{theta.des}{design for dependence parameters, when pairs are given the indeces of the
theta-design for this pair, is given in pairs as column 5}

\item{var.link}{Link function for variance}

\item{var.par}{parametrization}

\item{var.func}{when alternative parametrizations are used this function can specify how the paramters are related to the \eqn{\lambda_j}'s.}

\item{iid}{Calculate i.i.d. decomposition when iid>=1, when iid=2 then avoids adding the uncertainty for marginal paramters for additive gamma model (default).}

\item{notaylor}{Taylor expansion}

\item{model}{model}

\item{marginal.p}{vector of marginal probabilities}

\item{beta.iid}{iid decomposition of marginal probability  estimates for each subject, if based on GLM model this is computed.}

\item{Dbeta.iid}{derivatives of marginal model wrt marginal parameters, if based on GLM model this is computed.}

\item{strata}{strata for fitting: considers only pairs where both are from same strata}

\item{max.clust}{max clusters}

\item{se.clusters}{clusters for iid decomposition for roubst standard errors}

\item{numDeriv}{uses Fisher scoring aprox of second derivative if 0, otherwise numerical derivatives}

\item{random.design}{random effect design for additive gamma model, when pairs are given the
indeces of the pairs random.design rows are given as columns 3:4}

\item{pairs}{matrix with rows of indeces (two-columns) for the pairs considered in the pairwise composite score, useful for case-control sampling when marginal is known.}

\item{dim.theta}{dimension of theta when pairs and pairs specific design is given. That is when pairs has 6 columns.}

\item{additive.gamma.sum}{this is specification of the lamtot in the models via a matrix that is multiplied onto the parameters theta (dimensions=(number random effects x number of theta parameters), when null then sums all parameters. Default is a matrix of 1's}

\item{pair.ascertained}{if pairs are sampled only when there are events in the pair i.e. Y1+Y2>=1.}

\item{case.control}{if data is case control data for pair call, and here 2nd column of pairs are probands (cases or controls)}

\item{no.opt}{for not optimizing}

\item{twostage}{default twostage=1, to fit MLE use twostage=0}

\item{beta}{is starting value for beta for MLE version}

\item{...}{for NR of lava}
}
\description{
The pairwise pairwise odds ratio model provides an alternative to the alternating logistic
regression (ALR).
}
\details{
The reported standard errors are based on a cluster corrected score equations from the
pairwise likelihoods assuming that the marginals are known. This gives correct standard errors
in the case of the Odds-Ratio model (Plackett distribution) for dependence, but incorrect standard
errors for the Clayton-Oakes types model (that is also called "gamma"-frailty). For the additive gamma version of the
standard errors
are adjusted for the uncertainty in the marginal models via an iid deomposition using the iid() function of
lava. For the clayton oakes model that is not speicifed via the random effects these can be
fixed subsequently using the iid influence functions for the marginal model, but typically this does not
change much.

For the Clayton-Oakes version of the model, given the gamma distributed random effects it is
assumed that the probabilities are indpendent, and that the marginal survival functions are on logistic form
\deqn{
logit(P(Y=1|X)) = \alpha + x^T \beta
}
therefore conditional on the random effect the probability of the event is
\deqn{
logit(P(Y=1|X,Z)) = exp( -Z \cdot Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
}

Can also fit a structured additive gamma random effects model, such
the ACE, ADE model for survival data:

Now random.design specificies the random effects for each subject within a cluster. This is
a matrix of 1's and 0's with dimension n x d.  With d random effects.
For a cluster with two subjects, we let the random.design rows be
 \eqn{v_1} and \eqn{v_2}.
Such that the random effects for subject
1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
subjects 1's random effect are Gamma distributed with
mean \eqn{\lambda_j/v_1^T \lambda}
and variance \eqn{\lambda_j/(v_1^T \lambda)^2}. Note that the random effect
\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.
It is here asssumed that  \eqn{lamtot=v_1^T \lambda} is fixed over all clusters
as it would be for the ACE model below.

The DEFAULT parametrization uses the variances of the random effecs (var.par=1)
\deqn{
\theta_j  = \lambda_j/(v_1^T \lambda)^2
}

For alternative parametrizations (var.par=0) one can specify how the parameters relate
to \eqn{\lambda_j} with the function

Based on these parameters the relative contribution (the heritability, h) is
equivalent to  the expected values of the random effects  \eqn{\lambda_j/v_1^T \lambda}

Given the random effects the probabilities  are independent and on the form
\deqn{
logit(P(Y=1|X)) = exp( - Laplace^{-1}(lamtot,lamtot,P(Y=1|x)) )
}
with the inverse laplace of the gamma distribution with mean 1 and variance lamtot.

The parameters \eqn{(\lambda_1,...,\lambda_d)}
are related to the parameters of the model
by a regression construction \eqn{pard} (d x k), that links the \eqn{d}
\eqn{\lambda} parameters
with the (k) underlying \eqn{\theta} parameters
\deqn{
\lambda = theta.des  \theta
}
here using theta.des to specify these low-dimension association. Default is a diagonal matrix.
}
\examples{
data(twinstut)
twinstut0 <- subset(twinstut, tvparnr<4000)
twinstut <- twinstut0
twinstut$binstut <- (twinstut$stutter=="yes")*1
theta.des <- model.matrix( ~-1+factor(zyg),data=twinstut)
margbin <- glm(binstut~factor(sex)+age,data=twinstut,family=binomial())
bin <- binomial.twostage(margbin,data=twinstut,var.link=1,
         clusters=twinstut$tvparnr,theta.des=theta.des,detail=0)
summary(bin)

twinstut$cage <- scale(twinstut$age)
theta.des <- model.matrix( ~-1+factor(zyg)+cage,data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
		         clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)

theta.des <- model.matrix( ~-1+factor(zyg)+factor(zyg)*cage,data=twinstut)
bina <- binomial.twostage(margbin,data=twinstut,var.link=1,
		         clusters=twinstut$tvparnr,theta.des=theta.des)
summary(bina)

\donttest{ ## Reduce Ex.Timings
## refers to zygosity of first subject in eash pair : zyg1
## could also use zyg2 (since zyg2=zyg1 within twinpair's))
out <- easy.binomial.twostage(stutter~factor(sex)+age,data=twinstut,
                          response="binstut",id="tvparnr",var.link=1,
	             	      theta.formula=~-1+factor(zyg1))
summary(out)

## refers to zygosity of first subject in eash pair : zyg1
## could also use zyg2 (since zyg2=zyg1 within twinpair's))
desfs<-function(x,num1="zyg1",num2="zyg2")
    c(x[num1]=="dz",x[num1]=="mz",x[num1]=="os")*1

out3 <- easy.binomial.twostage(binstut~factor(sex)+age,
      data=twinstut,response="binstut",id="tvparnr",var.link=1,
      theta.formula=desfs,desnames=c("mz","dz","os"))
summary(out3)
}

### use of clayton oakes binomial additive gamma model
###########################################################
\donttest{ ## Reduce Ex.Timings
data <- simbinClaytonOakes.family.ace(10000,2,1,beta=NULL,alpha=NULL)
margbin <- glm(ybin~x,data=data,family=binomial())
margbin

head(data)
data$number <- c(1,2,3,4)
data$child <- 1*(data$number==3)

### make ace random effects design
out <- ace.family.design(data,member="type",id="cluster")
out$pardes
head(out$des.rv)

bints <- binomial.twostage(margbin,data=data,
     clusters=data$cluster,detail=0,var.par=1,
     theta=c(2,1),var.link=0,
     random.design=out$des.rv,theta.des=out$pardes)
summary(bints)

data <- simbinClaytonOakes.twin.ace(10000,2,1,beta=NULL,alpha=NULL)
out  <- twin.polygen.design(data,id="cluster",zygname="zygosity")
out$pardes
head(out$des.rv)
margbin <- glm(ybin~x,data=data,family=binomial())

bintwin <- binomial.twostage(margbin,data=data,
     clusters=data$cluster,var.par=1,
     theta=c(2,1),random.design=out$des.rv,theta.des=out$pardes)
summary(bintwin)
concordanceTwinACE(bintwin)
}

}
\references{
Two-stage binomial modelling
}
\author{
Thomas Scheike
}
\keyword{binomial}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtable.R
\name{dtable}
\alias{dtable}
\alias{dtab}
\title{tables for data frames}
\usage{
dtable(
  data,
  y = NULL,
  x = NULL,
  ...,
  level = -1,
  response = NULL,
  flat = TRUE,
  total = FALSE,
  prop = FALSE,
  summary = NULL
)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{name of variable, or fomula, or names of variables on data frame.}

\item{...}{Optional additional arguments}

\item{level}{1 for all marginal tables, 2 for all 2 by 2 tables, and null for the full table, possible versus group variable}

\item{response}{For level=2, only produce tables with columns given by 'response' (index)}

\item{flat}{produce flat tables}

\item{total}{add total counts/proportions}

\item{prop}{Proportions instead of counts (vector of margins)}

\item{summary}{summary function}
}
\description{
tables for data frames
}
\examples{
data("sTRACE",package="timereg")

dtable(sTRACE,~status)
dtable(sTRACE,~status+vf)
dtable(sTRACE,~status+vf,level=1)
dtable(sTRACE,~status+vf,~chf+diabetes)

dtable(sTRACE,c("*f*","status"),~diabetes)
dtable(sTRACE,c("*f*","status"),~diabetes, level=2)
dtable(sTRACE,c("*f*","status"),level=1)

dtable(sTRACE,~"*f*"+status,level=1)
dtable(sTRACE,~"*f*"+status+I(wmi>1.4)|age>60,level=2)
dtable(sTRACE,"*f*"+status~I(wmi>0.5)|age>60,level=1)
dtable(sTRACE,status~dcut(age))

dtable(sTRACE,~status+vf+sex|age>60)
dtable(sTRACE,status+vf+sex~+1|age>60, level=2)
dtable(sTRACE,.~status+vf+sex|age>60,level=1)
dtable(sTRACE,status+vf+sex~diabetes|age>60)
dtable(sTRACE,status+vf+sex~diabetes|age>60, flat=FALSE)

dtable(sTRACE,status+vf+sex~diabetes|age>60, level=1)
dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2)

dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=1, total=TRUE)
dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=2, total=TRUE)
dtable(sTRACE,status+vf+sex~diabetes|age>60, level=2, prop=1:2, summary=summary)

}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastcluster.R
\name{fast.cluster}
\alias{fast.cluster}
\title{Fast Cluster Index Conversion}
\usage{
fast.cluster(x, ...)
}
\arguments{
\item{x}{integer vector of cluster identifiers.}

\item{...}{additional arguments (not used).}
}
\value{
Integer vector of 0-based cluster indices.
}
\description{
Converts a cluster variable to consecutive numeric indices using compiled code.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{phreg_lt}
\alias{phreg_lt}
\title{Lu-Tsiatis More Efficient Log-Rank for Randomized studies with baseline covariates}
\usage{
phreg_lt(
  formula,
  data,
  augmentR = NULL,
  treat.model = ~+1,
  augmentC = NULL,
  km = TRUE,
  cens.code = 0,
  level = 0.95,
  cens.model = NULL,
  typeII = NULL,
  ...
)
}
\arguments{
\item{formula}{formula with 'Surv' outcome (see \code{coxph}) and treatment (randomization 0/1)}

\item{data}{data frame}

\item{augmentR}{formula for the randomization augmentation  (~age+sex)}

\item{treat.model}{propensity score model, default is ~+1, assuming RCT study}

\item{augmentC}{formula for the censoring augmentation  (~age+sex)}

\item{km}{use Kaplan-Meier for the censoring weights (stratified on treatment)}

\item{cens.code}{censoring code}

\item{level}{of confidence intervals}

\item{cens.model, }{default is censoring model ~strata(treatment) but any model can be used to make censoring martingales}

\item{typeII}{if 1 then computes also alternative formulae that are based on the censoring martingale rather than the robust processes of  Lu-Tsiatis computations.}

\item{...}{Additional arguments to phreg function}
}
\description{
Efficient implementation of the Lu-Tsiatis improvement using baseline covariates. Results
almost equivalent with the speffSurv function of the speff2trial function. A dynamic 
censoring augmentation regression is also computed to gain even more from the censoring augmentation.
}
\examples{
## Lu, Tsiatis simulation
data <- mets:::simLT(0.7,100)

out <- phreg_lt(Surv(time,status)~Z,data=data,augmentR=~X,augmentC=~factor(Z):X)
out$coefs
}
\references{
Lu, Tsiatis (2008), Improving the efficiency of the log-rank test using auxiliary covariates, Biometrika, 679--694
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twinlm.R
\name{twinlm}
\alias{twinlm}
\alias{twinlm.strata}
\title{Classic twin model for quantitative traits}
\usage{
twinlm(
  formula,
  data,
  id,
  zyg,
  DZ,
  group = NULL,
  group.equal = FALSE,
  strata = NULL,
  weights = NULL,
  type = c("ace"),
  twinnum = "twinnum",
  binary = FALSE,
  ordinal = 0,
  keep = weights,
  estimator = NULL,
  constrain = TRUE,
  control = list(),
  messages = 1,
  ...
)
}
\arguments{
\item{formula}{Formula specifying effects of covariates on the response}

\item{data}{\code{data.frame} with one observation pr row. In
addition a column with the zygosity (DZ or MZ given as a factor) of
each individual much be
specified as well as a twin id variable giving a unique pair of
numbers/factors to each twin pair}

\item{id}{The name of the column in the dataset containing the twin-id variable.}

\item{zyg}{The name of the column in the dataset containing the
zygosity variable}

\item{DZ}{Character defining the level in the zyg variable
corresponding to the dyzogitic twins. If this argument is missing,
the reference level (i.e. the first level) will be interpreted as
the dyzogitic twins}

\item{group}{Optional. Variable name defining group for interaction analysis (e.g., gender)}

\item{group.equal}{If TRUE marginals of groups are asummed to be the same}

\item{strata}{Strata variable name}

\item{weights}{Weights matrix if needed by the chosen estimator. For use
with Inverse Probability Weights}

\item{type}{Character defining the type of analysis to be
performed. Can be a subset of "aced" (additive genetic factors, common
environmental factors, unique environmental factors, dominant
genetic factors). Other choices are:
\itemize{
\item {"0" (or "sat"): Saturated model where twin 1 and twin 2 within each twin
       pair may have a different marginal distribution.}
\item {"1" (or "flex","zyg"): Within twin pairs the marginal distribution is
       the same, but the marginal distribution may differ between MZ and DZ
       twins. A free correlation structure within MZ and DZ twins. }
\item {"2" (or "u", "eqmarg"): All individuals have the same marginals
       but a free correlation structure within MZ and DZ twins.}
}
The default value is an additive polygenic model \code{type="ace"}.}

\item{twinnum}{The name of the column in the dataset numbering the
twins (1,2). If it does not exist in \code{data} it will
automatically be created.}

\item{binary}{If \code{TRUE} a liability model is fitted. Note that if the right-hand-side of the formula is a factor, character vector, og logical variable, then the liability model is automatically chosen (wrapper of the \code{bptwin} function).}

\item{ordinal}{If non-zero (number of bins) a liability model is fitted.}

\item{keep}{Vector of variables from \code{data} that are not
specified in \code{formula}, to be added to data.frame of the SEM}

\item{estimator}{Choice of estimator/model}

\item{constrain}{Development argument}

\item{control}{Control argument parsed on to the optimization routine}

\item{messages}{Control amount of messages shown}

\item{...}{Additional arguments parsed on to lower-level functions}
}
\value{
Returns an object of class \code{twinlm}.
}
\description{
Fits a classical twin model for quantitative traits.
}
\examples{
## Simulate data
set.seed(1)
d <- twinsim(1000,b1=c(1,-1),b2=c(),acde=c(1,1,0,1))
## E(y|z1,z2) = z1 - z2. var(A) = var(C) = var(E) = 1

## E.g to fit the data to an ACE-model without any confounders we simply write
ace <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id")
ace
## An AE-model could be fitted as
ae <- twinlm(y ~ 1, data=d, DZ="DZ", zyg="zyg", id="id", type="ae")
## LRT:
lava::compare(ae,ace)
## AIC
AIC(ae)-AIC(ace)
## To adjust for the covariates we simply alter the formula statement
ace2 <- twinlm(y ~ x1+x2, data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
## Summary/GOF
summary(ace2)
\donttest{ ## Reduce Ex.Timings
## An interaction could be analyzed as:
ace3 <- twinlm(y ~ x1+x2 + x1:I(x2<0), data=d, DZ="DZ", zyg="zyg", id="id", type="ace")
ace3
## Categorical variables are also supported 
d2 <- transform(d,x2cat=cut(x2,3,labels=c("Low","Med","High")))
ace4 <- twinlm(y ~ x1+x2cat, data=d2, DZ="DZ", zyg="zyg", id="id", type="ace")
}
}
\seealso{
\code{\link{bptwin}}, \code{\link{twinlm.time}}, \code{\link{twinlm.strata}}, \code{\link{twinsim}}
}
\author{
Klaus K. Holst
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{predict.phreg}
\alias{predict.phreg}
\title{Predictions from Proportional Hazards Model}
\usage{
\method{predict}{phreg}(
  object,
  newdata,
  times = NULL,
  individual.time = FALSE,
  tminus = FALSE,
  se = TRUE,
  robust = FALSE,
  conf.type = "log",
  conf.int = 0.95,
  km = FALSE,
  ...
)
}
\arguments{
\item{object}{Object of class \code{"phreg"}.}

\item{newdata}{Data frame for prediction. If \code{NULL}, predictions are made for the original data.}

\item{times}{Time points for prediction. Defaults to all unique event times in the model.}

\item{individual.time}{Logical; if \code{TRUE}, uses one time per subject (requires \code{newdata} and \code{times} to be same length).}

\item{tminus}{Logical; if \code{TRUE}, predicts at \eqn{t-} (strictly before \eqn{t}).}

\item{se}{Logical; if \code{TRUE}, computes standard errors and confidence intervals.}

\item{robust}{Logical; if \code{TRUE}, uses robust standard errors (default for most functions).}

\item{conf.type}{Transformation for survival estimates: \code{"log"} (default) or \code{"plain"}.}

\item{conf.int}{Confidence level (default 0.95).}

\item{km}{Logical; if \code{TRUE}, uses Kaplan-Meier product-limit for baseline; otherwise uses exponential of cumulative baseline.}

\item{...}{Additional arguments for plotting functions.}
}
\value{
An object of class \code{"predictphreg"} containing:
\item{surr}{Matrix of survival probabilities.}
\item{cumhaz}{Matrix of cumulative hazards.}
\item{cif}{Matrix of cumulative incidence functions (if applicable).}
\item{times}{Vector of time points.}
\item{surv.upper, surv.lower}{Confidence bounds for survival.}
\item{RR}{Relative risks.}
}
\description{
Computes predictions for survival probability, cumulative hazard, or risk at specified
time points for new data or existing data. Includes standard errors and confidence intervals.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{count_history}
\alias{count_history}
\title{Compute cumulative event counts as time-dependent covariates}
\usage{
count_history(
  data,
  status = "status",
  id = "id",
  types = 1,
  names.count = "Count",
  lag = TRUE,
  multitype = FALSE,
  marks = NULL
)
}
\arguments{
\item{data}{A data frame in counting-process format, with one row per event
interval per subject.}

\item{status}{Name of the column containing event status codes. Default is
\code{"status"}.}

\item{id}{Name of the column containing subject identifiers. Default is
\code{"id"}.}

\item{types}{Integer vector of status codes to count. Each value in
\code{types} generates one new count column (when \code{multitype = FALSE})
or contributes to a single combined count (when \code{multitype = TRUE}).
Default is \code{1}.}

\item{names.count}{Prefix for the names of the new count columns. The
status code is appended, e.g. \code{"Count1"}, \code{"Count2"}.
Default is \code{"Count"}.}

\item{lag}{Logical. If \code{TRUE} (default), the count at each row is the
number of events strictly before the current time (\eqn{N(t-)}). If
\code{FALSE}, events at the current time are included (\eqn{N(t)}).}

\item{multitype}{Logical. If \code{TRUE}, events with status in \code{types}
are aggregated into a single count column, optionally weighted by
\code{marks}. If \code{FALSE} (default), a separate count column is created
for each value in \code{types}.}

\item{marks}{Optional numeric vector of weights applied to events when
\code{multitype = TRUE}. If \code{NULL} (default), each event has weight 1.}
}
\value{
The input data frame \code{data} with one or more new integer columns
  appended. With \code{multitype = FALSE}, columns are named
  \code{paste0(names.count, k)} for each \code{k} in \code{types}; with
  \code{multitype = TRUE}, a single column named
  \code{paste0(names.count, types[1])} is added. An internal bookkeeping
  column \code{lbnr__id} is also added.
}
\description{
For each subject and each row of \code{data}, counts the number of prior
events of specified types in the recurrent event history. The resulting
count columns can be used as time-dependent covariates in subsequent models,
e.g. to capture event-history dependence in the recurrent event rate.
}
\details{
When \code{lag = TRUE} (default), the count at each row reflects events that
occurred strictly before the current time point (i.e. \eqn{N(t-)}), making
it suitable as a left-continuous covariate in counting-process models.
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12
dtable(hf, ~status)

## Separate counts for event types 1 and 2
rr <- count_history(hf, types = 1:2, id = "id", status = "status")
dtable(rr, ~"Count*" + status, level = 1)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg_weibull.R
\name{phreg_weibull}
\alias{phreg_weibull}
\alias{phreg.par}
\title{Weibull-Cox regression}
\usage{
phreg_weibull(
  formula,
  shape.formula = ~1,
  data,
  save.data = TRUE,
  control = list()
)
}
\arguments{
\item{formula}{Formula for proportional hazards. The right-handside must be
an [Event] or [Surv] object (with right-censoring and possibly delayed
entry).}

\item{shape.formula}{Formula for shape parameter}

\item{data}{data.frame}

\item{save.data}{if TRUE the data.frame is stored in the model object (for
predictions and simulations)}

\item{control}{control arguments to optimization routine [stats::nlmbin]}
}
\value{
`phreg.par` object
}
\description{
Fits a Cox-Weibull with cumulative hazard given by \deqn{
  \Lambda(t) = \lambda \cdot t^s } where \eqn{s} is the shape parameter, and
  \eqn{\lambda} the rate parameter. We here allow a regression model for
  both parameters \deqn{\lambda := \exp(\beta^\top X)} \deqn{s :=e
  \exp(\gamma^\top Z)} as defined by `formula` and `shape.formula`
  respectively.
}
\details{
The parametrization
}
\examples{
data(sTRACE, package="mets")
sTRACE$entry <- 0
fit1 <- phreg_weibull(Event(entry, time, status == 9) ~ age,
             shape.formula = ~age, data = sTRACE)
tt <- seq(0,10, length.out=100)
pr1 <- predict(fit1, newdata = sTRACE[1, ], times = tt)
fit2 <- phreg(Event(time, status == 9) ~ age, data = sTRACE)
pr2 <- predict(fit2, newdata = sTRACE[1, ], se = FALSE)
if (interactive()) {
   plot(pr2$times, pr2$surv, type="s")
   lines(tt, pr1[,1,1], col="red", lwd=2)
}
}
\seealso{
[mets::phreg()]
}
\author{
Klaus Kähler Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{twinbmi}
\alias{twinbmi}
\title{BMI data set}
\format{
Self-reported BMI-values on 11,411 subjects

tvparnr: twin id
bmi: BMI (m/kg^2)
age: Age
gender: (male/female)
zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
}
\description{
BMI data set
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{migr}
\alias{migr}
\title{Migraine data}
\description{
Migraine data
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{phreg}
\alias{phreg}
\alias{robust_phreg}
\title{Fast Cox Proportional Hazards Regression}
\usage{
phreg(formula, data, offset = NULL, weights = NULL, ...)
}
\arguments{
\item{formula}{Formula with a 'Surv' or 'Event' outcome (similar to \code{coxph}).}

\item{data}{Data frame containing the variables.}

\item{offset}{Offsets for the Cox model linear predictor.}

\item{weights}{Weights for the Cox score equations.}

\item{...}{Additional arguments passed to lower-level functions (e.g., optimization controls).}
}
\value{
An object of class \code{"phreg"} containing:
\item{coef}{Vector of estimated coefficients.}
\item{var}{Robust variance-covariance matrix.}
\item{beta.iid}{Influence functions for the regression coefficients.}
\item{cumhaz}{Matrix of cumulative hazard estimates (time, cumhaz).}
\item{se.cumhaz}{Matrix of standard errors for the cumulative hazard.}
\item{cox.prep}{List containing preprocessed data for the Cox model.}
\item{opt}{Optimization results (if optimization was performed).}
\item{call}{The matched call.}
}
\description{
Fits a Cox proportional hazards model using a fast C++ backend.
Robust variance (sandwich estimator) is the default variance estimate in the summary.
}
\details{
The influence functions (IID decomposition) follow the numerical order of the given cluster variable.
Ordering the results by `$id` will align the IID terms with the original dataset order.
}
\examples{
data(TRACE)
dcut(TRACE) <- ~.
# Fit model with clustering
out1 <- phreg(Surv(time, status == 9) ~ wmi + age + strata(vf, chf) + cluster(id), data = TRACE)
summary(out1)

# Plotting baselines
par(mfrow = c(1, 2))
plot(out1)

# Computing robust variance for baseline
rob1 <- robust_phreg(out1)
plot(rob1, se = TRUE, robust = TRUE)

# IID decomposition for regression parameters
head(iid(out1))

# IID decomposition for baseline at a specific time-point
Aiiid <- iid(out1, time = 30)
head(Aiiid)

# Combined IID decomposition (beta and baseline)
dd <- iidBaseline(out1, time = 30)
head(dd$beta.iid)
head(dd$base.iid)

# Stratified model
outs <- phreg(Surv(time, status == 9) ~ strata(vf, wmicat.4) + cluster(id), data = TRACE)
summary(outs)
par(mfrow = c(1, 2))
plot(outs)
}
\seealso{
\code{\link{plot.phreg}}, \code{\link{predict.phreg}}, \code{\link{robust_phreg}}
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{CPH_HPN_CRBSI}
\alias{CPH_HPN_CRBSI}
\title{Rates for HPN program for patients of Copenhagen Cohort}
\format{
crbsi: cumulative rate of catheter related bloodstream infection in HPN patients of Copenhagen
 mechanical: cumulative rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen
trombo: cumulative rate of Occlusion/Thrombosis complication for catheter of HPN patients of Copenhagen
terminal: rate of terminal event, patients leaving the HPN program
}
\source{
Estimated data
}
\description{
Rates for HPN program for patients of Copenhagen Cohort
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{covarianceRecurrent}
\alias{covarianceRecurrent}
\alias{plot.covariace.recurrent}
\alias{covarianceRecurrentS}
\alias{Bootcovariancerecurrence}
\alias{BootcovariancerecurrenceS}
\title{Estimation of covariance for bivariate recurrent events with terminal event}
\usage{
covarianceRecurrent(
  data,
  type1,
  type2,
  status = "status",
  death = "death",
  start = "start",
  stop = "stop",
  id = "id",
  names.count = "Count"
)
}
\arguments{
\item{data}{data-frame}

\item{type1}{type of first event (code) related to status}

\item{type2}{type of second event (code) related to status}

\item{status}{name of status}

\item{death}{name of death indicator}

\item{start}{start stop call of Hist() of prodlim}

\item{stop}{start stop call of Hist() of prodlim}

\item{id}{id}

\item{names.count}{name of count for number of previous event of different types, here generated by count.history()}
}
\description{
Estimation of probability of more that k events for recurrent events process
where there is terminal event
}
\examples{

########################################
## getting some data to work on 
########################################
data(base1cumhaz)
data(base4cumhaz)
data(drcumhaz)
dr <- drcumhaz
base1 <- base1cumhaz
base4 <- base4cumhaz
rr <- simRecurrentII(1000,base1,cumhaz2=base4,death.cumhaz=dr)
rr <- count.history(rr)
rr$strata <- 1
dtable(rr,~death+status)

covrp <- covarianceRecurrent(rr,1,2,status="status",death="death",
                        start="entry",stop="time",id="id",names.count="Count")
par(mfrow=c(1,3)) 
plot(covrp)

### with strata, each strata in matrix column, provides basis for fast Bootstrap
covrpS <- covarianceRecurrentS(rr,1,2,status="status",death="death",
        start="entry",stop="time",strata="strata",id="id",names.count="Count")

}
\references{
Scheike, Eriksson, Tribler (2019) 
            The mean, variance and correlation for bivariate recurrent events
            with a terminal event,  JRSS-C
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{survival-helpers}
\alias{survival-helpers}
\alias{survival.twostage}
\alias{alpha2kendall}
\alias{alpha2spear}
\alias{piecewise_twostage}
\alias{piecewise_data}
\alias{matplot.mets.twostage}
\title{Survival Twostage Helpers}
\usage{
survival.twostage(x, ...)

matplot.mets.twostage(object, ...)

alpha2spear(theta, link = 1)

alpha2kendall(theta, link = 0)

piecewise_twostage(
  cut1,
  cut2,
  data = parent.frame(),
  timevar = "time",
  status = "status",
  id = "id",
  covars = NULL,
  covars.pairs = NULL,
  num = NULL,
  method = "optimize",
  Nit = 100,
  detail = 0,
  silent = 1,
  weights = NULL,
  control = list(),
  theta = NULL,
  theta.des = NULL,
  var.link = 1,
  step = 0.5,
  model = "plackett",
  data.return = 0
)

piecewise_data(
  cut1,
  cut2,
  data = parent.frame(),
  timevar = "time",
  status = "status",
  id = "id",
  covars = NULL,
  covars.pairs = NULL,
  num = NULL,
  silent = 1
)
}
\arguments{
\item{x}{a marginal model object (for \code{survival.twostage}).}

\item{...}{additional arguments.}

\item{object}{a twostage model object (for matplot method).}

\item{theta}{initial dependence parameter values.}

\item{link}{if 1, parameters are on log scale (for alpha2kendall/alpha2spear).}

\item{cut1}{vector of cut points for the first time axis.}

\item{cut2}{vector of cut points for the second time axis.}

\item{data}{a data.frame with the survival data.}

\item{timevar}{character name of the time variable.}

\item{status}{character name of the status variable.}

\item{id}{name of the cluster identifier column.}

\item{covars}{optional character vector of covariate names.}

\item{covars.pairs}{optional covariates at the pair level.}

\item{num}{character name of the within-cluster number variable.}

\item{method}{optimization method.}

\item{Nit}{maximum number of iterations.}

\item{detail}{level of detail in output.}

\item{silent}{level of verbosity (1=silent).}

\item{weights}{optional weights.}

\item{control}{optimization control list.}

\item{theta.des}{theta design matrix.}

\item{var.link}{if 1, log-link for variance parameters.}

\item{step}{step size for optimization.}

\item{model}{dependence model: \code{"plackett"} or \code{"clayton.oakes"}.}

\item{data.return}{if 1, return data with model fits.}
}
\description{
Helper functions for the twostage survival dependence models.
}
\details{
\code{survival.twostage} is an alias for \code{survival_twostage}.

\code{alpha2kendall} converts the Clayton-Oakes alpha parameter to Kendall's tau.

\code{alpha2spear} converts the Clayton-Oakes alpha parameter to Spearman's rho.

\code{piecewise_twostage} fits twostage models on piecewise time intervals.

\code{piecewise_data} prepares data for piecewise twostage analysis.

\code{matplot.mets.twostage} produces matplot of twostage baseline estimates.
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterindex-reshape.R
\name{familycluster.index}
\alias{familycluster.index}
\title{Finds all pairs within a cluster (family)}
\usage{
familycluster.index(clusters, index.type = FALSE, num = NULL, Rindex = 1)
}
\arguments{
\item{clusters}{list of indeces}

\item{index.type}{argument of cluster index}

\item{num}{num}

\item{Rindex}{index starts with 1 in R, and 0 in C}
}
\description{
Finds all pairs within a cluster (family)
}
\examples{
i<-c(1,1,2,2,1,3)
d<- familycluster.index(i)
print(d)
}
\references{
Cluster indeces
}
\seealso{
cluster.index familyclusterWithProbands.index
}
\author{
Klaus Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastapprox.R
\name{fast.approx}
\alias{fast.approx}
\alias{indexstrata}
\alias{predictCumhaz}
\alias{cpred}
\title{Fast approximation}
\usage{
fast.approx(
  time,
  new.time,
  equal = FALSE,
  type = c("nearest", "right", "left"),
  sorted = FALSE,
  ...
)
}
\arguments{
\item{time}{Original ordered time points}

\item{new.time}{New time points}

\item{equal}{If TRUE a list is returned with additional element}

\item{type}{Type of matching, nearest index, nearest greater than
or equal (right), number of elements smaller than y otherwise
the closest value above new.time is returned.}

\item{sorted}{Set to true if new.time is already sorted}

\item{...}{Optional additional arguments}
}
\description{
Fast approximation
}
\examples{
id <- c(1,1,2,2,7,7,10,10)
fast.approx(unique(id),id)

t <- 0:6
n <- c(-1,0,0.1,0.9,1,1.1,1.2,6,6.5)
fast.approx(t,n,type="left")
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wild-phreg.R
\name{Bootphreg}
\alias{Bootphreg}
\alias{pred.cif.boot}
\title{Wild bootstrap for Cox PH regression}
\usage{
Bootphreg(
  formula,
  data,
  offset = NULL,
  weights = NULL,
  B = 1000,
  type = c("exp", "poisson", "normal"),
  ...
)
}
\arguments{
\item{formula}{formula with 'Surv' outcome (see \code{coxph})}

\item{data}{data frame}

\item{offset}{offsets for cox model}

\item{weights}{weights for Cox score equations}

\item{B}{bootstraps}

\item{type}{distribution for multiplier}

\item{...}{Additional arguments to lower level funtions}
}
\description{
wild bootstrap for uniform bands for Cox models
}
\examples{

 n <- 100
 x <- 4*rnorm(n)
 time1 <- 2*rexp(n)/exp(x*0.3)
 time2 <- 2*rexp(n)/exp(x*(-0.3))
 status <- ifelse(time1<time2,1,2)
 time <- pmin(time1,time2)
 rbin <- rbinom(n,1,0.5)
 cc <-rexp(n)*(rbin==1)+(rbin==0)*rep(3,n)
 status <- ifelse(time < cc,status,0)
 time  <- ifelse(time < cc,time,cc)
 data <- data.frame(time=time,status=status,x=x)

 b1 <- Bootphreg(Surv(time,status==1)~x,data,B=1000)
 b2 <- Bootphreg(Surv(time,status==2)~x,data,B=1000)
 c1 <- phreg(Surv(time,status==1)~x,data)
 c2 <- phreg(Surv(time,status==2)~x,data)

 ### exp to make all bootstraps positive
 out <- pred.cif.boot(b1,b2,c1,c2,gplot=0)

 cif.true <- (1-exp(-out$time))*.5
 with(out,plot(time,cif,ylim=c(0,1),type="l"))
 lines(out$time,cif.true,col=3)
 with(out,plotConfRegion(time,band.EE,col=1))
 with(out,plotConfRegion(time,band.EE.log,col=3))
 with(out,plotConfRegion(time,band.EE.log.o,col=2))

}
\references{
Wild bootstrap based confidence intervals for multiplicative hazards models, 
Dobler, Pauly, and Scheike (2018),
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timereg-copy-package.R
\docType{data}
\name{TRACE}
\alias{TRACE}
\alias{sTRACE}
\alias{tTRACE}
\title{The TRACE study group of myocardial infarction}
\format{
This data frame contains the following columns: \describe{
\item{id}{a numeric vector. Patient code. } \item{status}{ a numeric vector
code. Survival status. 9: dead from myocardial infarction, 0: alive, 7: dead
from other causes.  } \item{time}{ a numeric vector. Survival time in years.
} \item{chf}{ a numeric vector code. Clinical heart pump failure, 1:
present, 0: absent. } \item{diabetes}{ a numeric vector code. Diabetes, 1:
present, 0: absent. } \item{vf}{ a numeric vector code. Ventricular
fibrillation, 1: present, 0: absent. } \item{wmi}{ a numeric vector.
Measure of heart pumping effect based on ultrasound measurements where 2 is
normal and 0 is worst. } \item{sex}{ a numeric vector code. 1: female, 0:
male. } \item{age}{ a numeric vector code. Age of patient. } }
}
\source{
The TRACE study group.

Jensen, G.V., Torp-Pedersen, C., Hildebrandt, P., Kober, L., F. E. Nielsen,
Melchior, T., Joen, T. and P. K. Andersen (1997), Does in-hospital
ventricular fibrillation affect prognosis after myocardial infarction?,
European Heart Journal 18, 919--924.
}
\description{
The TRACE data frame contains 1877 patients and is a subset of a data set
consisting of approximately 6000 patients.  It contains data relating
survival of patients after myocardial infarction to various risk factors.
}
\details{
sTRACE is a subsample consisting of 300 patients.

tTRACE is a subsample consisting of 1000 patients.
}
\examples{

data(TRACE)
names(TRACE)

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{dermalridgesMZ}
\alias{dermalridgesMZ}
\title{Dermal ridges data (monozygotic twins)}
\format{
Data on dermal ridge counts (left and right hand) in 18
monozygotic twin pairs.
}
\source{
Sarah B. Holt (1952). Genetics of dermal ridges: bilateral
asymmetry in finger ridge-counts.  Annals of Eugenics 17 (1),
pp.211--231. DOI: 10.1111/j.1469-1809.1952.tb02513.x
}
\description{
Data on dermal ridge counts in left and right hand in (nuclear) families
}
\examples{
data(dermalridgesMZ)
fast.reshape(dermalridgesMZ,id="id",varying=c("left","right"))
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{summary.cor}
\alias{summary.cor}
\title{Summary for dependence models for competing risks}
\usage{
\method{summary}{cor}(object, marg.cif = NULL, marg.cif2 = NULL, digits = 3, ...)
}
\arguments{
\item{object}{object from cor_cif rr_cif or or_cif for dependence between competing risks data for two causes.}

\item{marg.cif}{a number that gives the cumulative incidence in one time point for which concordance and 
casewise concordance are computed.}

\item{marg.cif2}{the cumulative incidence for cause 2 for concordance and 
casewise concordance are computed. Default is that it is the same as marg.cif.}

\item{digits}{digits in output.}

\item{...}{Additional arguments.}
}
\value{
prints summary for dependence model. 
\item{casewise}{gives casewise concordance that is, probability of cause 2 (related to cif2) given that cause 1 (related to cif1)
	has occured.}
\item{concordance}{gives concordance that is, probability of cause 2 (related to cif2) and cause 1 (related to cif1).}
\item{cif1}{cumulative incidence for cause1.}
\item{cif2}{cumulative incidence for cause1.}
}
\description{
Computes concordance and casewise concordance for dependence models for competing risks 
models of the type cor_cif, rr_cif or or_cif for the given cumulative incidences and the different dependence
measures in the object.
}
\examples{
## library("timereg")
## data("multcif",package="mets") # simulated data 
## multcif$cause[multcif$cause==0] <- 2
##  
## times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
## add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),
##                           data=multcif,n.sim=0,times=times,cause=1)
###
## out1<- cor_cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
## summary(out1)
## 
## pad <- predict(add,X=1,se=0,uniform=0)
## summary(out1,marg.cif=pad)
}
\references{
Cross odds ratio Modelling of dependence for
Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 

A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
Scheike, Zhang, Sun, Jensen (2010), Biometrika.
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterindex-reshape.R
\name{faster.reshape}
\alias{faster.reshape}
\title{Fast Reshape from Long to Wide Format}
\usage{
faster.reshape(data, clusters, index.type = FALSE, num = NULL, Rindex = 1)
}
\arguments{
\item{data}{a matrix or data.frame to reshape.}

\item{clusters}{vector of cluster identifiers, or column name in data.}

\item{index.type}{logical; if TRUE, clusters are already 0-based indices.}

\item{num}{optional within-cluster numbering variable.}

\item{Rindex}{if 1, use R (1-based) indexing.}
}
\value{
A wide-format matrix.
}
\description{
Reshapes clustered long-format data to wide format efficiently using
compiled code.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dby.R
\name{dby}
\alias{dby}
\alias{dby<-}
\alias{dby2}
\alias{dby2<-}
\alias{dbyr}
\title{Calculate summary statistics grouped by}
\usage{
dby(
  data,
  INPUT,
  ...,
  ID = NULL,
  ORDER = NULL,
  SUBSET = NULL,
  SORT = 0,
  COMBINE = !REDUCE,
  NOCHECK = FALSE,
  ARGS = NULL,
  NAMES,
  COLUMN = FALSE,
  REDUCE = FALSE,
  REGEX = mets.options()$regex,
  ALL = TRUE
)
}
\arguments{
\item{data}{Data.frame}

\item{INPUT}{Input variables (character or formula)}

\item{...}{functions}

\item{ID}{id variable}

\item{ORDER}{(optional) order variable}

\item{SUBSET}{(optional) subset expression}

\item{SORT}{sort order (id+order variable)}

\item{COMBINE}{If TRUE result is appended to data}

\item{NOCHECK}{No sorting or check for missing data}

\item{ARGS}{Optional list of arguments to functions (...)}

\item{NAMES}{Optional vector of column names}

\item{COLUMN}{If TRUE do the calculations for each column}

\item{REDUCE}{Reduce number of redundant rows}

\item{REGEX}{Allow regular expressions}

\item{ALL}{if FALSE only the subset will be returned}
}
\description{
Calculate summary statistics grouped by variable
}
\details{
Calculate summary statistics grouped by


dby2 for column-wise calculations
}
\examples{
n <- 4
k <- c(3,rbinom(n-1,3,0.5)+1)
N <- sum(k)
d <- data.frame(y=rnorm(N),x=rnorm(N),id=rep(seq(n),k),num=unlist(sapply(k,seq)))
d2 <- d[sample(nrow(d)),]

dby(d2, y~id, mean)
dby(d2, y~id + order(num), cumsum)

dby(d,y ~ id + order(num), dlag)
dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2))
dby(d,y ~ id + order(num), dlag, ARGS=list(k=1:2), NAMES=c("l1","l2"))

dby(d, y~id + order(num), mean=mean, csum=cumsum, n=length)
dby(d2, y~id + order(num), a=cumsum, b=mean, N=length, l1=function(x) c(NA,x)[-length(x)])

dby(d, y~id + order(num), nn=seq_along, n=length)
dby(d, y~id + order(num), nn=seq_along, n=length)

d <- d[,1:4]
dby(d, x<0) <- list(z=mean)
d <- dby(d, is.na(z), z=1)

f <- function(x) apply(x,1,min)
dby(d, y+x~id, min=f)

dby(d,y+x~id+order(num), function(x) x)

f <- function(x) { cbind(cumsum(x[,1]),cumsum(x[,2]))/sum(x)}
dby(d, y+x~id, f)

## column-wise
a <- d
dby2(a, mean, median, REGEX=TRUE) <- '^[y|x]'~id
a
## wildcards 
dby2(a,'y*'+'x*'~id,mean) 


## subset
dby(d, x<0) <- list(z=NA)
d
dby(d, y~id|x>-1, v=mean,z=1)
dby(d, y+x~id|x>-1, mean, median, COLUMN=TRUE)

dby2(d, y+x~id|x>0, mean, REDUCE=TRUE)

dby(d,y~id|x<0,mean,ALL=FALSE)

a <- iris
a <- dby(a,y=1)
dby(a,Species=="versicolor") <- list(y=2)
}
\author{
Klaus K. Holst and Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dutils.R
\name{dspline}
\alias{dspline}
\alias{dspline<-}
\title{Simple linear spline}
\usage{
dspline(
  data,
  y = NULL,
  x = NULL,
  breaks = 4,
  probs = NULL,
  equi = FALSE,
  regex = mets.options()$regex,
  sep = NULL,
  na.rm = TRUE,
  labels = NULL,
  all = FALSE,
  ...
)
}
\arguments{
\item{data}{if x is formula or names for data frame then data frame is needed.}

\item{y}{name of variable, or fomula, or names of variables on data frame.}

\item{x}{name of variable, or fomula, or names of variables on data frame.}

\item{breaks}{number of breaks, for variables or vector of break points,}

\item{probs}{groups defined from quantiles}

\item{equi}{for equi-spaced breaks}

\item{regex}{for regular expressions.}

\item{sep}{seperator for naming of cut names.}

\item{na.rm}{to remove NA for grouping variables.}

\item{labels}{to use for cut groups}

\item{all}{to do all variables, even when breaks are not unique}

\item{...}{Optional additional arguments}
}
\description{
Constructs simple linear spline  on a data frame using the formula syntax of dutils
that is adds (x-cuti)* (x>cuti) to the data-set for each knot of the spline. 
The full spline is thus given by x and spline variables added to the data-set.
}
\examples{
data(TRACE)
TRACE <- dspline(TRACE,~wmi,breaks=c(1,1.3,1.7))
cca <- coxph(Surv(time,status==9)~age+vf+chf+wmi,data=TRACE)
cca2 <- coxph(Surv(time,status==9)~age+wmi+vf+chf+wmi.spline1+wmi.spline2+wmi.spline3,data=TRACE)
anova(cca,cca2)

nd=data.frame(age=50,vf=0,chf=0,wmi=seq(0.4,3,by=0.01))
nd <- dspline(nd,~wmi,breaks=c(1,1.3,1.7))
pl <- predict(cca2,newdata=nd)
plot(nd$wmi,pl,type="l")

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{tie_breaker}
\alias{tie_breaker}
\title{Break ties in event times for recurrent event data}
\usage{
tie_breaker(
  data,
  stop = "time",
  start = "entry",
  status = "status",
  id = NULL,
  cause = NULL,
  cens.code = 0,
  exit.unique = TRUE,
  ddt = NULL,
  seed = NULL
)
}
\arguments{
\item{data}{A data frame in counting-process format, sorted by subject and
time.}

\item{stop}{Name of the column containing interval exit (stop) times.
Default is \code{"time"}.}

\item{start}{Name of the column containing interval entry (start) times,
used to update the following row when \code{id} is supplied. Default is
\code{"entry"}.}

\item{status}{Name of the column containing event status codes. Default is
\code{"status"}.}

\item{id}{Name of the column containing subject identifiers. If supplied,
the start time of the next interval for the same subject is adjusted to
match the perturbed stop time, preserving interval continuity. Default is
\code{NULL} (no adjustment made).}

\item{cause}{Integer vector of status codes that identify events (non-censored
rows). If \code{NULL} (default), all non-censoring status values are treated
as events.}

\item{cens.code}{Integer code(s) for censoring. Rows with this status are
never perturbed. Default is \code{0}.}

\item{exit.unique}{Logical. If \code{TRUE} (default), an event time is
considered tied whenever it coincides with \emph{any} exit time in the
data (including censored rows). If \code{FALSE}, only ties among event rows
are resolved.}

\item{ddt}{Maximum perturbation size. Tied event times are shifted by a
uniform draw on \eqn{[0, \text{ddt}]}. If \code{NULL} (default), \code{ddt}
is set to half the smallest positive gap between any two consecutive exit
times in the data.}

\item{seed}{Optional integer passed to \code{set.seed()} before drawing the
perturbations, making results reproducible. Default is \code{NULL} (no seed
set).}
}
\value{
The input data frame \code{data} with tied event exit times
  perturbed. If \code{id} is supplied, an additional logical column
  \code{tiebreaker} marks rows whose start time was adjusted as a consequence
  of a perturbation in the preceding row.
}
\description{
Resolves tied event times in a counting-process dataset by adding a small
random perturbation to duplicated exit times of event rows. This is a
preprocessing step required by \code{\link{recurrent_marginal}} and related
functions, which assume unique jump times within each stratum.
}
\details{
A tie is defined as an event exit time (rows where \code{status} is a cause
code) that coincides with another exit time in the dataset. When
\code{exit.unique = TRUE} (default), a tie is flagged whenever an event time
also appears among any other exit times (censored or event). When
\code{exit.unique = FALSE}, only exact ties between two event rows are
resolved.

Tied event times are perturbed by adding \eqn{U \cdot \delta} where
\eqn{U \sim \text{Uniform}(0, 1)} and \eqn{\delta} is \code{ddt} (defaulting
to half the smallest observed positive gap between consecutive exit times).
When subject IDs are provided via \code{id}, the corresponding interval start
time of the immediately following row for the same subject is updated to
maintain a valid counting-process structure, and a logical \code{tiebreaker}
column is added to flag affected rows.
}
\examples{
data(hfactioncpx12)
hf <- hfactioncpx12

## Check for ties in event exit times
ev <- hf[hf$status == 1, ]
any(duplicated(ev$time))

## Resolve ties before fitting the marginal mean model
hf_clean <- tie_breaker(hf, stop = "time", start = "entry",
                         status = "status", id = "id",
                         cause = 1, cens.code = 0)

out <- recurrent_marginal(Event(entry, time, status) ~ cluster(id),
                           data = hf_clean, cause = 1, death.code = 2)
summary(out, times = 1:5)

}
\seealso{
\code{\link{recurrent_marginal}}, \code{\link{test_logrankRecurrent}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/divide.conquer.R
\name{divide.conquer}
\alias{divide.conquer}
\title{Split a data set and run function}
\usage{
divide.conquer(func = NULL, data, size, splits, id = NULL, ...)
}
\arguments{
\item{func}{called function}

\item{data}{data-frame}

\item{size}{size of splits}

\item{splits}{number of splits (ignored if size is given)}

\item{id}{optional cluster variable}

\item{...}{Additional arguments to lower level functions}
}
\description{
Split a data set and run function
}
\examples{
## avoid dependency on timereg
## library(timereg)
## data(TRACE)
## res <- divide.conquer(prop.odds,TRACE,
## 	     formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=200)
}
\author{
Thomas Scheike, Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostage.R
\name{survival_twostage}
\alias{survival_twostage}
\alias{twostage_aalen}
\alias{twostage_cox.aalen}
\alias{twostage_coxph}
\alias{twostage_phreg}
\alias{randomDes}
\alias{readmargsurv}
\title{Twostage Survival Model for Multivariate Survival Data}
\usage{
survival_twostage(
  margsurv,
  data = NULL,
  method = "nr",
  detail = 0,
  clusters = NULL,
  silent = 1,
  weights = NULL,
  theta = NULL,
  theta.des = NULL,
  var.link = 1,
  baseline.iid = 1,
  model = "clayton.oakes",
  marginal.trunc = NULL,
  marginal.survival = NULL,
  strata = NULL,
  se.clusters = NULL,
  numDeriv = 1,
  random.design = NULL,
  pairs = NULL,
  dim.theta = NULL,
  numDeriv.method = "simple",
  additive.gamma.sum = NULL,
  var.par = 1,
  no.opt = FALSE,
  ...
)
}
\arguments{
\item{margsurv}{Marginal model object.}

\item{data}{Data frame (must be given).}

\item{method}{Scoring method: \code{"nr"} for lava NR optimizer.}

\item{detail}{Detail level for output.}

\item{clusters}{Cluster variable.}

\item{silent}{Debug information level.}

\item{weights}{Weights for score equations.}

\item{theta}{Starting values for variance components.}

\item{theta.des}{Design for dependence parameters; when pairs are given, the indices of the
theta-design for this pair are given in pairs as column 5.}

\item{var.link}{Link function for variance: 1 for exponential link.}

\item{baseline.iid}{To adjust for baseline estimation, using \code{phreg} function on same data.}

\item{model}{Model type: \code{"clayton.oakes"} or \code{"plackett"}.}

\item{marginal.trunc}{Marginal left truncation probabilities.}

\item{marginal.survival}{Optional vector of marginal survival probabilities.}

\item{strata}{Strata for fitting (see examples).}

\item{se.clusters}{Clusters for SE calculation with IID.}

\item{numDeriv}{To get numDeriv version of second derivative; otherwise uses sum of squared scores for each pair.}

\item{random.design}{Random effect design for additive gamma model; when pairs are given, the indices of the
pairs' random.design rows are given as columns 3:4.}

\item{pairs}{Matrix with rows of indices (two-columns) for the pairs considered in the pairwise
composite score; useful for case-control sampling when marginal is known.}

\item{dim.theta}{Dimension of the theta parameter for pairs situation.}

\item{numDeriv.method}{Method for numerical derivative (e.g., \code{"simple"} to speed up things).}

\item{additive.gamma.sum}{For \code{two.stage=0}, this is specification of the \code{lamtot} in the models via
a matrix that is multiplied onto the parameters theta (dimensions = number of random effects \eqn{\times} number of theta parameters);
when \code{NULL}, sums all parameters.}

\item{var.par}{Is 1 for the default parametrization with the variances of the random effects;
\code{var.par=0} specifies that the \eqn{\lambda_j}'s are used as parameters.}

\item{no.opt}{For not optimizing.}

\item{...}{Additional arguments to maximizer NR of lava.}
}
\value{
An object of class \code{"mets.twostage"} containing:
\item{theta}{Estimated dependence parameters.}
\item{coef}{Coefficients.}
\item{score}{Score vector.}
\item{hess}{Hessian matrix.}
\item{hessi}{Inverse Hessian matrix.}
\item{var.theta}{Variance of theta parameters.}
\item{robvar.theta}{Robust variance of theta parameters.}
\item{loglike}{Log-likelihood value.}
\item{theta.iid}{Influence functions for theta.}
\item{model}{Model type used.}
}
\description{
Fits Clayton-Oakes or bivariate Plackett models for bivariate survival data
using marginals that are on Cox form. The dependence can be modelled via:
\enumerate{
\item Regression design on dependence parameter.
\item Random effects, additive gamma model.
}
}
\details{
If clusters contain more than two subjects, a composite likelihood based on
pairwise bivariate models is used (for full MLE see \code{twostageMLE}).

The two-stage model is constructed such that, given gamma distributed random effects,
the survival functions are assumed independent, and the marginal survival functions
are on Cox form:
\deqn{ P(T > t| x) = S(t|x)= \exp( -\exp(x^T \beta) A_0(t) ) }

One possibility is to model the variance within clusters via a regression design,
specifying a regression structure for the independent gamma distributed random
effect for each cluster, such that the variance is given by:
\deqn{ \theta = h( z_j^T \alpha) }
where \eqn{z} is specified by \code{theta.des}, and a possible link function
\code{var.link=1} will use the exponential link \eqn{h(x)=\exp(x)}, and
\code{var.link=0} the identity link \eqn{h(x)=x}.

The reported standard errors are based on the estimated information from the
likelihood assuming that the marginals are known (unlike \code{twostageMLE}
and for the additive gamma model below).

Can also fit a structured additive gamma random effects model, such as the
ACE, ADE model for survival data. In this case, the \code{random.design}
specifies the random effects for each subject within a cluster. This is a
matrix of 1's and 0's with dimension \eqn{n \times d} (for \eqn{d} random effects).

For a cluster with two subjects, the \code{random.design} rows are \eqn{v_1}
and \eqn{v_2}. Such that the random effect for subject 1 is \deqn{v_1^T
(Z_1,...,Z_d)}, for \eqn{d} random effects. Each random effect has an
associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction,
subject 1's random effect is Gamma distributed with mean
\eqn{\lambda_j/v_1^T \lambda} and variance \eqn{\lambda_j/(v_1^T
\lambda)^2}. Note that the random effect \eqn{v_1^T (Z_1,...,Z_d)} has mean
1 and variance \eqn{1/(v_1^T \lambda)}. It is assumed that \eqn{lamtot=v_1^T
\lambda} is fixed within clusters as it would be for the ACE model.

Based on these parameters, the relative contribution (the heritability, \eqn{h}) is
equivalent to the expected values of the random effects: \eqn{\lambda_j/v_1^T \lambda}.

The DEFAULT parametrization (\code{var.par=1}) uses the variances of the random effects:
\deqn{ \theta_j  = \lambda_j/(v_1^T \lambda)^2 }
For alternative parametrizations, specify how the parameters relate to \eqn{\lambda_j}
with the argument \code{var.par=0}.

For both types of models, the basic model assumptions are that given the
random effects of the clusters, the survival distributions within a cluster
are independent and on the form: \deqn{ P(T > t| x,z) = \exp( -Z \cdot
\text{Laplace}^{-1}(lamtot,lamtot,S(t|x)) ) } with the inverse Laplace of
the gamma distribution with mean 1 and variance \eqn{1/lamtot}.

The parameters \eqn{(\lambda_1,...,\lambda_d)} are related to the parameters of the model
by a regression construction \code{pard} (\eqn{d \times k}), that links the \eqn{d}
\eqn{\lambda} parameters with the \eqn{k} underlying \eqn{\theta} parameters:
\deqn{ \lambda = theta.des \times \theta }
here using \code{theta.des} to specify these low-dimension associations. Default is a diagonal matrix.
This can be used to make structural assumptions about the variances of the random-effects
as is needed for the ACE model for example.

The \code{case.control} option can be used with the pair specification of the pairwise parts
of the estimating equations. Here it is assumed that the second subject of each pair is the proband.
}
\examples{
data(diabetes)

# Marginal Cox model  with treat as covariate
margph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
### Clayton-Oakes, MLE
fitco1<-twostageMLE(margph,data=diabetes,theta=1.0)
summary(fitco1)

\donttest{ ## Reduce Ex.Timings
### Plackett model
mph <- phreg(Surv(time,status)~treat+cluster(id),data=diabetes)
fitp <- survival_twostage(mph,data=diabetes,theta=3.0,Nit=40,
               clusters=diabetes$id,var.link=1,model="plackett")
summary(fitp)

### Clayton-Oakes
fitco2 <- survival_twostage(mph,data=diabetes,theta=0.0,detail=0,
                 clusters=diabetes$id,var.link=1,model="clayton.oakes")
summary(fitco2)
fitco3 <- survival_twostage(margph,data=diabetes,theta=1.0,detail=0,
             clusters=diabetes$id,var.link=0,model="clayton.oakes")
summary(fitco3)

### without covariates but with stratafied
marg <- phreg(Surv(time,status)~+strata(treat)+cluster(id),data=diabetes)
fitpa <- survival_twostage(marg,data=diabetes,theta=1.0,
                clusters=diabetes$id,model="clayton.oakes")
summary(fitpa)


### Piecewise constant cross hazards ratio modelling
########################################################

d <- subset(sim_ClaytonOakes(1000,2,0.5,0,stoptime=2,left=0),!truncated)
udp <- piecewise_twostage(c(0,0.5,2),data=d,method="optimize",
                          id="cluster",timevar="time",
                          status="status",model="clayton.oakes",silent=0)
summary(udp)
}

\donttest{ ## Reduce Ex.Timings
### Same model using the strata option, a bit slower
########################################################
## makes the survival pieces for different areas in the plane
##ud1=surv_boxarea(c(0,0),c(0.5,0.5),data=d,id="cluster",timevar="time",status="status")
##ud2=surv_boxarea(c(0,0.5),c(0.5,2),data=d,id="cluster",timevar="time",status="status")
##ud3=surv_boxarea(c(0.5,0),c(2,0.5),data=d,id="cluster",timevar="time",status="status")
##ud4=surv_boxarea(c(0.5,0.5),c(2,2),data=d,id="cluster",timevar="time",status="status")

## everything done in one call
ud <- piecewise_data(c(0,0.5,2),data=d,timevar="time",status="status",id="cluster")
ud$strata <- factor(ud$strata);
ud$intstrata <- factor(ud$intstrata)

## makes strata specific id variable to identify pairs within strata
## se's computed based on the id variable across strata "cluster"
ud$idstrata <- ud$id+(as.numeric(ud$strata)-1)*2000

marg2 <- timereg::aalen(Surv(boxtime,status)~-1+factor(num):factor(intstrata),
               data=ud,n.sim=0,robust=0)
tdes <- model.matrix(~-1+factor(strata),data=ud)
fitp2 <- survival_twostage(marg2,data=ud,se.clusters=ud$cluster,clusters=ud$idstrata,
                model="clayton.oakes",theta.des=tdes,step=0.5)
summary(fitp2)

### now fitting the model with symmetry, i.e. strata 2 and 3 same effect
ud$stratas <- ud$strata;
ud$stratas[ud$strata=="0.5-2,0-0.5"] <- "0-0.5,0.5-2"
tdes2 <- model.matrix(~-1+factor(stratas),data=ud)
fitp3 <- survival_twostage(marg2,data=ud,clusters=ud$idstrata,se.cluster=ud$cluster,
                model="clayton.oakes",theta.des=tdes2,step=0.5)
summary(fitp3)

### same model using strata option, a bit slower
fitp4 <- survival_twostage(marg2,data=ud,clusters=ud$cluster,se.cluster=ud$cluster,
                model="clayton.oakes",theta.des=tdes2,step=0.5,strata=ud$strata)
summary(fitp4)
}

\donttest{ ## Reduce Ex.Timings
### structured random effects model additive gamma ACE
### simulate structured two-stage additive gamma ACE model
data <- sim_ClaytonOakes_twin_ace(2000,2,1,0,3)
out <- twin_polygen_design(data,id="cluster")
pardes <- out$pardes
pardes
des.rv <- out$des.rv
head(des.rv)
aa <- phreg(Surv(time,status)~x+cluster(cluster),data=data,robust=0)
ts <- survival_twostage(aa,data=data,clusters=data$cluster,detail=0,
	       theta=c(2,1),var.link=0,step=0.5,
	       random.design=des.rv,theta.des=pardes)
summary(ts)
}
}
\references{
\itemize{
\item Twostage estimation of additive gamma frailty models for survival data. Scheike (2019), work in progress.
\item Shih and Louis (1995) Inference on the association parameter in copula models for bivariate survival data, \emph{Biometrics}.
\item Glidden (2000), A Two-Stage estimator of the dependence parameter for the Clayton Oakes model, LIDA.
\item Measuring early or late dependence for bivariate twin data. Scheike, Holst, Hjelmborg (2015), LIDA.
\item Estimating heritability for cause specific mortality based on twins studies. Scheike, Holst, Hjelmborg (2014), LIDA.
\item Additive Gamma frailty models for competing risks data. Eriksson and Scheike (2015), \emph{Biometrics}.
}
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_phregs}
\alias{sim_phregs}
\title{Simulation of Cause-Specific Cox Models}
\usage{
sim_phregs(
  coxs,
  n,
  data = NULL,
  rr = NULL,
  strata = NULL,
  entry = NULL,
  extend = TRUE,
  cens = NULL,
  rrc = NULL,
  ...
)
}
\arguments{
\item{coxs}{List of Cox models.}

\item{n}{Number of simulations.}

\item{data}{Data frame to extract covariates.}

\item{rr}{Relative risks.}

\item{strata}{Strata vector.}

\item{entry}{Delayed entry.}

\item{extend}{Extend baselines to largest endpoint.}

\item{cens}{Censoring specification.}

\item{rrc}{Relative risks for censoring.}

\item{...}{Arguments for \code{rchaz}.}
}
\value{
Data frame with simulated event times, status, and covariates.
}
\description{
Simulates data that looks like fit from cause-specific Cox models. Censors data 
automatically. When censoring is given in the list of causes, this provides censoring 
that looks like the data.
}
\examples{
data(bmt)
nsim <- 100; 

cox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet+age,data=bmt)
cox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=bmt)
coxs <- list(cox1,cox2)
## just calls sim_phregs !
dd <- sim_phregs(coxs,nsim,data=bmt,extend=c(0.001))
scox1 <- phreg(Surv(time,cause==1)~strata(tcell)+platelet+age,data=dd)
scox2 <- phreg(Surv(time,cause==2)~tcell+strata(platelet),data=dd)

cbind(cox1$coef,scox1$coef)
cbind(cox2$coef,scox2$coef)
par(mfrow=c(1,2))
plot(cox1); plot(scox1,add=TRUE); 
plot(cox2); plot(scox2,add=TRUE); 

}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.regression.R
\name{binregG}
\alias{binregG}
\title{G-Estimator for Binomial Regression Model (Standardized Estimates)}
\usage{
binregG(x, data, Avalues = NULL, varname = NULL)
}
\arguments{
\item{x}{An object of class \code{"binreg"} obtained from \code{binreg()} or \code{logitIPCW()}.}

\item{data}{A data frame containing the covariates to be used for averaging the risk 
estimates. This should ideally be the same data used to fit the model, or a 
representative sample.}

\item{Avalues}{Numeric or factor vector specifying the values of the first covariate 
(\eqn{A}) for which to compute standardized risks. 
\itemize{
  \item If the first covariate is a factor and \code{Avalues} is \code{NULL}, 
    all levels of the factor are used.
  \item If the first covariate is continuous, \code{Avalues} must be provided.
}}

\item{varname}{Optional character string specifying the name of the variable to be 
treated as the treatment/exposure variable. If \code{NULL}, the first variable 
in the model formula is used.}
}
\value{
An object of class \code{"survivalG"} containing:
\item{risk}{A table of standardized risk estimates for each value of \code{Avalues}.}
\item{risk.iid}{Influence functions for the standardized risk estimates.}
\item{difference}{Pairwise differences in risks between levels of \code{A}.}
\item{ratio}{Risk ratios between levels of \code{A}.}
\item{vcov}{Variance-covariance matrix of the risk estimates.}
\item{model}{The link function used in the original model.}
}
\description{
Computes the G-estimator (G-formula) for standardized risk estimates based on a 
fitted \code{binreg} object. The G-estimator standardizes predictions over the 
covariate distribution in the data:
\deqn{ \hat F(t, A=a) = n^{-1} \sum_{i=1}^n \hat F(t, A=a, Z_i) }
}
\details{
This function assumes that the first covariate in the original model formula 
represents the treatment or exposure variable (\eqn{A}). It calculates the 
marginal risk for specified values of \eqn{A} by averaging the conditional 
predictions over the observed covariate distribution \eqn{Z}.

The function returns influence functions for these risk estimates, allowing 
for the computation of standard errors and confidence intervals.

If the first covariate is a factor, contrasts between all levels are computed 
automatically. If it is continuous, specific values must be provided via 
\code{Avalues}.
}
\section{References}{

\itemize{
  \item Blanche PF, Holt A, Scheike T (2022). "On logistic regression with right censored data, with or without competing risks, and its use for estimating treatment effects." \emph{Lifetime Data Analysis}, 29, 441–482.
}
}

\examples{
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
bmt$event <- (bmt$cause!=0)*1

b1 <- binreg(Event(time,cause)~age+tcell+platelet,bmt,cause=1,time=50)
sb1 <- binregG(b1,bmt,Avalues=c(0,1,2))
summary(sb1)
}
\seealso{
\code{\link{binreg}}, \code{\link{binregATE}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recreg.R
\name{recreg}
\alias{recreg}
\alias{marks}
\alias{scalecumhaz}
\alias{GLprediid}
\alias{IIDrecreg}
\title{Recurrent Events Regression with Terminal Event}
\usage{
recreg(
  formula,
  data,
  cause = 1,
  death.code = 2,
  cens.code = 0,
  cens.model = ~1,
  weights = NULL,
  offset = NULL,
  Gc = NULL,
  wcomp = NULL,
  marks = NULL,
  augmentation.type = c("lindyn.augment", "lin.augment"),
  ...
)
}
\arguments{
\item{formula}{Formula with an 'Event' outcome.}

\item{data}{Data frame containing the variables.}

\item{cause}{Cause of interest (default is 1).}

\item{death.code}{Codes for the terminal event/death (default is 2).}

\item{cens.code}{Code for censoring (default is 0).}

\item{cens.model}{Formula for a stratified Cox model without covariates used to estimate censoring probabilities.}

\item{weights}{Weights for the score equations.}

\item{offset}{Offsets for the model.}

\item{Gc}{Censoring weights for the time argument. If \code{NULL}, these are calculated using a Kaplan-Meier estimator
(should then provide \eqn{G_c(T_i-)}).}

\item{wcomp}{Weights for composite outcomes (e.g., when \code{cause=c(1,3)}, \code{wcomp} might be \code{c(1,2)}).}

\item{marks}{A mark value vector from the data frame, specifying the mark value at all events.}

\item{augmentation.type}{Type of augmentation when an augmentation model is given (options: \code{"lindyn.augment"}, \code{"lin.augment"}).}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"recreg"} (extending \code{"phreg"}) containing:
\item{coef}{Estimated coefficients.}
\item{var}{Robust variance-covariance matrix.}
\item{iid}{Influence functions for the coefficients.}
\item{cumhaz}{Cumulative hazard estimates.}
\item{se.cumhaz}{Standard errors for cumulative hazard.}
}
\description{
Fits a Ghosh-Lin IPCW (Inverse Probability of Censoring Weighted) Cox-type model for recurrent events
in the presence of a terminal event (e.g., death).
}
\details{
For the Cox-type model, the expectation is modeled as:
\deqn{ E(dN_1(t)|X) = \mu_0(t) dt \exp(X^T \beta) }
by solving Cox-type IPCW weighted score equations:
\deqn{ \int (Z - E(t)) w(t) dN_1(t) }
where \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))},
\deqn{E(t) = S_1(t)/S_0(t)}, and \deqn{S_j(t) = \sum X_i^j w_i(t) \exp(X_i^T \beta)}.

The IID decomposition of the beta coefficients takes the form:
\deqn{ \int (Z - E) w(t) dM_1 + \int q(s)/p(s) dM_c }
and is returned as the \code{iid} component.

Events, deaths, and censorings are specified via a start-stop structure and the \code{Event} call.
The function identifies these via a status vector and cause codes, censoring codes (\code{cens.code}),
and death codes (\code{death.code}). See examples and vignettes for details.
}
\examples{
## data with no ties
data(hfactioncpx12)
hf <- hfactioncpx12
hf$x <- as.numeric(hf$treatment) 
dd <- data.frame(treatment=levels(hf$treatment),id=1)

gl <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,cause=1,death.code=2)
summary(gl)
head(iid(gl))
pgl <- predict(gl,dd,se=1); 
plot(pgl,se=1)

## censoring stratified after treatment 
gls <- recreg(Event(entry,time,status)~treatment+cluster(id),data=hf,
cause=1,death.code=2,cens.model=~strata(treatment))
summary(gls)

glss <- recreg(Event(entry,time,status)~strata(treatment)+cluster(id),data=hf,
cause=1,death.code=2,cens.model=~strata(treatment))
summary(glss)
plot(glss)

## IPCW at 2 years 
ll2 <- recregIPCW(Event(entry,time,status)~treatment+cluster(id),data=hf,
cause=1,death.code=2,time=2,cens.model=~strata(treatment))
summary(ll2)

ll2i <- recregIPCW(Event(entry,time,status)~-1+treatment+cluster(id),data=hf,
cause=1,death.code=2,time=2,cens.model=~strata(treatment))
summary(ll2i)
}
\seealso{
\code{\link{recregIPCW}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gofG.phreg}
\alias{gofG.phreg}
\title{Stratified baseline graphical GOF test for Cox covariates in PH regression}
\usage{
gofG.phreg(x, sim = 0, silent = 1, lm = TRUE, ...)
}
\arguments{
\item{x}{phreg object}

\item{sim}{to simulate som variation from cox model to put on graph}

\item{silent}{to keep it absolutely silent}

\item{lm}{addd line to plot, regressing the cumulatives on each other}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Looks at stratified baseline in Cox model and plots all baselines versus each
other to see if lines are straight, with 50 resample versions under the 
assumptiosn that the stratified Cox is correct
}
\examples{
data(tTRACE)

m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+wmi,data=tTRACE) 
m2 <- phreg(Surv(time,status==9)~vf+strata(chf)+wmi,data=tTRACE) 
par(mfrow=c(2,2))

gofG.phreg(m1)
gofG.phreg(m2)

bplot(m1,log="y")
bplot(m2,log="y")
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinomialreg.R
\name{mlogit}
\alias{mlogit}
\alias{predict}
\title{Multinomial Regression Based on phreg}
\usage{
mlogit(formula, data, offset = NULL, weights = NULL, fix.X = FALSE, ...)
}
\arguments{
\item{formula}{Formula with the outcome (similar to \code{coxph}). The outcome must be a factor.}

\item{data}{Data frame containing the variables.}

\item{offset}{Offsets for the partial likelihood.}

\item{weights}{Weights for the score equations.}

\item{fix.X}{Logical; if \code{TRUE}, forces the same coefficients for all categories (except intercepts).}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"mlogit"} (extending \code{"phreg"}) containing:
\item{coef}{Matrix of estimated coefficients (rows correspond to categories, columns to covariates).}
\item{var}{Robust variance-covariance matrix.}
\item{iid}{Influence functions for the coefficients.}
\item{nlev}{Number of levels in the outcome.}
\item{px}{Number of covariates.}
}
\description{
Fits a multinomial regression model for a categorical outcome with \eqn{K} levels:
\deqn{ P_i = \frac{ \exp( X^\top \beta_i ) }{ \sum_{j=1}^K \exp( X^\top \beta_j ) } }
for \eqn{i=1, \dots, K}, where \eqn{\beta_1 = 0} (baseline category).
}
\details{
This ensures that \eqn{\sum_j P_j = 1}. The model is fitted using the \code{phreg} function
by expanding the data into a long format with strata for each category.

The coefficients represent the log-Relative-Risk (log-RR) relative to the baseline group
(the first level of the factor, which can be reset using \code{relevel}).

Standard errors are computed based on the sandwich form:
\deqn{ D U^{-1} \left( \sum U_i^2 \right) D U^{-1} }
where \eqn{U} is the score vector and \eqn{D} is the derivative matrix.

Influence functions (possibly robust) can be obtained via the \code{iid()} function.
The response variable should be a factor.

Can also fit a cumulative odds model as a special case of \code{interval_logitsurv_discrete}.
}
\examples{
data(bmt)
bmt$id <- sample(200,408,replace=TRUE)
dfactor(bmt) <- cause1f~cause
drelevel(bmt,ref=3) <- cause3f~cause
dlevels(bmt)

mreg <- mlogit(cause1f~+1+cluster(id),bmt)
summary(mreg)
head(iid(mreg))
dim(iid(mreg))

mreg <- mlogit(cause1f~tcell+platelet,bmt)
summary(mreg)
head(iid(mreg))
dim(iid(mreg))

mreg3 <- mlogit(cause3f~tcell+platelet,bmt)
summary(mreg3)

## inverse information standard errors 
lava::estimate(coef=mreg3$coef,vcov=mreg3$II)

## predictions based on seen response or not 
## all probabilities
head(predict(mreg,response=FALSE))
head(predict(mreg))
## using newdata 
newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1),cause1f=c("2","2","0"))
## only probability of seen response 
predict(mreg,newdata)
## without response
predict(mreg,newdata,response=FALSE)
## given indexx of P(Y=j)
predict(mreg,newdata,Y=c(1,2,3))
##  reponse not given 
newdata <- data.frame(tcell=c(1,1,1),platelet=c(0,1,1))
predict(mreg,newdata)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{rcrisk}
\alias{rcrisk}
\alias{rchazl}
\title{Simulation of Piecewise constant hazard models with two causes (Cox).}
\usage{
rcrisk(
  cumA,
  cumB,
  rr1 = NULL,
  rr2 = NULL,
  n = NULL,
  cens = NULL,
  rrc = NULL,
  extend = TRUE,
  causes = NULL,
  ...
)
}
\arguments{
\item{cumA}{cumulative hazard of cause 1, or list of multiple cumulative hazards}

\item{cumB}{cumulative hazard of cause 2 or NULL when cumA is a list}

\item{rr1}{number of simulations or vector of relative risk for simuations, or matrix with columns equal to number of hazards in list}

\item{rr2}{number of simulations or vector of relative risk for simuations.}

\item{n}{number of simulation if rr not given, must be given when rr is not given}

\item{cens}{to censor further , rate or cumumlative hazard}

\item{rrc}{retlativ risk for censoring.}

\item{extend}{to extend the cumulative hazards to largest end-point}

\item{causes}{to assign status values for each of the causes, vector of integers}

\item{...}{arguments for rchaz}
}
\description{
Simulates data from piecwise constant baseline hazard that can also be of
Cox type. Censor data at highest value of the break points for either of the
cumulatives, see also sim.phregs
}
\examples{
library(mets)
data(bmt); 
n <- 100
cox1 <- phreg(Surv(time,cause==1)~tcell+platelet,data=bmt)
cox2 <- phreg(Surv(time,cause==2)~tcell+platelet,data=bmt)

X1 <- bmt[,c("tcell","platelet")]
xid <- sample(1:nrow(X1),n,replace=TRUE)
Z1 <- X1[xid,]
Z2 <- X1[xid,]
rr1 <- exp(as.matrix(Z1) \%*\% cox1$coef)
rr2 <- exp(as.matrix(Z2) \%*\% cox2$coef)

d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=2/70)
dd <- cbind(d,Z1)

d <-  rcrisk(cox1$cum,cox2$cum,rr1,rr2,cens=cbind(c(1,30,68),c(.01,1,3)))
dd <- cbind(d,Z1)

par(mfrow=c(1,3))
scox0 <- phreg(Surv(time,status==0)~tcell+platelet,data=dd)
plot(scox0); lines(cbind(c(1,30,68),c(.01,1,3)),col=2)
##
scox1 <- phreg(Surv(time,status==1)~tcell+platelet,data=dd)
scox2 <- phreg(Surv(time,status==2)~tcell+platelet,data=dd)
plot(cox1); plot(scox1,add=TRUE,col=2)
plot(cox2); plot(scox2,add=TRUE,col=2)
cbind(cox1$coef,scox1$coef,cox2$coef,scox2$coef)

# 3 causes and censoring 
d3 <-  rcrisk(list(cox1$cum,cox2$cum,cox1$cum),NULL,n=100,cens=cbind(c(1,30,68),c(.01,1,3)))
dtable(d3,~status)

}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.regression.R
\name{IC.binreg}
\alias{IC.binreg}
\title{Influence curve components for binomial regression ATE}
\usage{
\method{IC}{binreg}(x, type = c("coef", "DR", "G"), ...)
}
\arguments{
\item{x}{An object of class \code{ATE}, or an object containing an
\code{iid} component.}

\item{type}{Character vector specifying which influence curve components
to return. Possible values are:
\describe{
  \item{coef}{Influence curve for regression coefficients}
  \item{DR}{Doubly robust ATE influence curve}
  \item{G}{G-computation ATE influence curve}
}
If \code{x} inherits from \code{ATE} and \code{type} is missing,
the default is \code{c("DR", "G")}.}

\item{...}{Currently unused.}
}
\value{
A numeric matrix obtained by column-binding the selected
influence curve components. Each column corresponds to one element
of \code{type}, with column names preserved.
}
\description{
Extracts and scales influence function (IC) components from a fitted
binomial regression ATE object or raw IC matrix. Depending on the object
class and requested `type`, returns one or more components combined by
column-binding.
}
\details{
For objects of class \code{ATE}, the function extracts and rescales
the stored influence curve components:

\itemize{
  \item \code{x$iid} for coefficient ICs
  \item \code{x$riskDR.iid} for doubly robust ATE ICs
  \item \code{x$riskG.iid} for G-computation ICs
}

Each component is multiplied by its sample size (\code{NROW(.)}).

For non-\code{ATE} objects, the function returns
\code{NROW(x$iid) * x$iid}.
}
\examples{
\dontrun{
# default ATE behavior (DR + G)
IC.binreg(fit)

# only DR component
IC.binreg(fit, "DR")

# multiple components
IC.binreg(fit, c("coef", "DR"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gof.phreg}
\alias{gof.phreg}
\title{Goodness-of-Fit for Cox PH Regression (Proportionality)}
\usage{
\method{gof}{phreg}(object, n.sim = 1000, silent = 1, robust = NULL, ...)
}
\arguments{
\item{object}{A fitted \code{phreg} object (from \code{mets} or \code{survival}).}

\item{n.sim}{Number of simulations for the resampling procedure (default 1000).}

\item{silent}{Logical; if TRUE, suppresses timing estimates for long jobs.}

\item{robust}{Logical; if TRUE, uses robust martingale-based simulations. 
If NULL, defaults to TRUE if a cluster term is detected in the model call.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"gof.phreg"} containing:
  \item{jumptimes}{Event times used in the process.}
  \item{supUsim}{Matrix of simulated supremum values for each covariate.}
  \item{res}{Matrix with observed supremum (\code{Sup|U(t)|}) and p-values.}
  \item{supU}{Observed supremum values.}
  \item{pvals}{Vector of p-values for each covariate.}
  \item{score}{Cumulative score process values over time.}
  \item{simUt}{Simulated score processes.}
  \item{type}{Type of test performed ("prop").}
  \item{robust}{Logical flag indicating if robust simulation was used.}
}
\description{
Performs cumulative score process residual tests for the proportional hazards (PH) 
assumption in Cox regression. The test statistics are based on the cumulative 
score process:
\deqn{ U(t) = \int_0^t (X_i - E(t) ) d \hat M_i(s) }
where \eqn{\hat M_i(s)} are the martingale residuals.
}
\details{
P-values are computed using the Lin, Wei, and Ying (1993) resampling method, which 
simulates the asymptotic distribution of the supremum of the score process under 
the null hypothesis of proportional hazards.

The function supports two types of simulation:
\itemize{
  \item \strong{Standard}: Uses \eqn{dN_i} (counting process increments) for simulation.
  \item \strong{Robust}: Uses \eqn{\hat M_i(t)} (martingale residuals) adjusted for clustering 
    if a \code{cluster()} term is present in the model.
}
}
\examples{
data(sTRACE)

m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes, data=sTRACE) 
gg <- gof(m1)
gg
par(mfrow=c(1,3))
plot(gg)

m1 <- phreg(Surv(time,status==9)~strata(vf)+chf+diabetes, data=sTRACE) 
gg <- gof(m1)

## Robust simulations with cluster
sTRACE$id <- 1:500
m1 <- phreg(Surv(time,status==9)~vf+chf+diabetes+cluster(id), data=sTRACE) 
gg <- gof(m1)
gg
}
\references{
Lin, D. Y., Wei, L. J., & Ying, Z. (1993). Checking the Cox model with cumulative sums of martingale-based residuals. Biometrika, 80(3), 557-572.
}
\seealso{
\code{\link{gofM_phreg}}, \code{\link{gofZ_phreg}}
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{concordanceCor}
\alias{concordanceCor}
\alias{concordance.cor}
\title{Concordance Computes concordance and casewise concordance}
\usage{
concordanceCor(
  object,
  cif1,
  cif2 = NULL,
  messages = TRUE,
  model = NULL,
  coefs = NULL,
  ...
)
}
\arguments{
\item{object}{Output from the cor.cif, rr.cif or or.cif function}

\item{cif1}{Marginal cumulative incidence}

\item{cif2}{Marginal cumulative incidence of other cause (cause2) if  it is different from cause1}

\item{messages}{To print messages}

\item{model}{Specfifies wich model that is considered if object not given.}

\item{coefs}{Specfifies  dependence parameters if object is not given.}

\item{...}{Extra arguments, not used.}
}
\description{
Concordance for Twins
}
\details{
The concordance is the probability that both twins have experienced the 
event of interest and is defined as 
\deqn{
 cor(t) = P(T_1 \leq t, \epsilon_1 =1 , T_2 \leq t, \epsilon_2=1) 
 }

Similarly, the casewise concordance is 
\deqn{
 casewise(t) = \frac{cor(t)}{P(T_1 \leq t, \epsilon_1=1) }
}
that is the probability that twin "2" has the event given that twins "1" has.
}
\references{
Estimating twin concordance for bivariate competing risks twin data
Thomas H. Scheike, Klaus K. Holst and Jacob B. Hjelmborg, 
Statistics in Medicine 2014, 1193-1204

Estimating Twin Pair Concordance for Age of Onset.
Thomas H. Scheike, Jacob V B Hjelmborg, Klaus K. Holst, 2015 
in Behavior genetics DOI:10.1007/s10519-015-9729-3
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{mets.options}
\alias{mets.options}
\title{Set global options for \code{mets}}
\usage{
mets.options(...)
}
\arguments{
\item{...}{Arguments}
}
\value{
\code{list} of parameters
}
\description{
Extract and set global parameters of \code{mets}.
}
\details{
\itemize{
  \item \code{regex}: If TRUE character vectors will be interpreted as regular expressions (\code{dby}, \code{dcut}, ...)
  \item \code{silent}: Set to \code{FALSE} to disable various output messages
}
}
\examples{
\dontrun{
mets.options(regex=TRUE)
}
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.clayton.oakes.R
\name{simClaytonOakesWei}
\alias{simClaytonOakesWei}
\title{Simulate from the Clayton-Oakes frailty model}
\usage{
simClaytonOakesWei(
  K,
  n,
  eta,
  beta,
  stoptime,
  weiscale = 1,
  weishape = 2,
  left = 0,
  pairleft = 0
)
}
\arguments{
\item{K}{Number of clusters}

\item{n}{Number of observations in each cluster}

\item{eta}{1/variance}

\item{beta}{Effect (log hazard ratio) of covariate}

\item{stoptime}{Stopping time}

\item{weiscale}{weibull scale parameter}

\item{weishape}{weibull shape parameter}

\item{left}{Left truncation}

\item{pairleft}{pairwise (1) left truncation or individual (0)}
}
\description{
Simulate observations from the Clayton-Oakes copula model with
Weibull type baseline and Cox marginals.
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{cif}
\alias{cif}
\title{Cumulative Incidence with Robust Standard Errors}
\usage{
cif(formula, data = data, cause = 1, cens.code = 0, death.code = NULL, ...)
}
\arguments{
\item{formula}{Formula with 'Event' outcome and \code{strata} (only!).}

\item{data}{Data frame.}

\item{cause}{Cause of interest (default is \code{NULL}, which looks at all causes).}

\item{cens.code}{Censoring code (default is \code{"0"}).}

\item{death.code}{Alternative to \code{cens.code}; specifies codes of death.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"cif"} (extends \code{"phreg"}) containing:
\item{cumhaz}{Cumulative incidence estimates.}
\item{se.cumhaz}{Standard errors.}
\item{cause}{Cause of interest.}
}
\description{
Computes cumulative incidence functions with robust standard errors.
}
\examples{
data(bmt)
bmt$cluster <- sample(1:100, 408, replace = TRUE)
out1 <- cif(Event(time, cause) ~ +1, data = bmt, cause = 1)
out2 <- cif(Event(time, cause) ~ +1 + cluster(cluster), data = bmt, cause = 1)

par(mfrow = c(1, 2))
plot(out1, se = TRUE)
plot(out2, se = TRUE)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{drcumhaz}
\alias{drcumhaz}
\title{Rate for leaving HPN program for patients of Copenhagen}
\source{
Estimated data
}
\description{
Rate for leaving HPN program for patients of Copenhagen
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{print.casewise}
\alias{print.casewise}
\title{prints Concordance test}
\usage{
\method{print}{casewise}(x, digits = 3, ...)
}
\arguments{
\item{x}{output from casewise.test}

\item{digits}{number of digits}

\item{\dots}{Additional arguments to lower level functions}
}
\description{
prints Concordance test
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{resmean.phreg}
\alias{resmean.phreg}
\alias{cif.yearslost}
\alias{rmst.phreg}
\title{Restricted mean for stratified Kaplan-Meier or Cox model with martingale standard errors}
\usage{
resmean.phreg(x, times = NULL, covs = NULL, ...)
}
\arguments{
\item{x}{phreg object}

\item{times}{possible times for which to report restricted mean}

\item{covs}{possible covariate for Cox model}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Restricted mean for stratified Kaplan-Meier or stratified Cox with martingale 
standard error. Standard error is computed using linear interpolation between 
standard errors at jump-times. Plots gives restricted mean at all times. 
Years lost can be computed based on this and decomposed into years lost for
different causes using the cif.yearslost function that is based on  
integrating the cumulative incidence functions.  
One particular feature of these functions are that the restricted mean and years-lost are 
computed for all event times as functions and can be plotted/viewed.  When times are given and beyond
the last event time withn a strata the curves are extrapolated using the estimates of 
cumulative incidence.
}
\examples{
data(bmt); bmt$time <- bmt$time+runif(408)*0.001
out1 <- phreg(Surv(time,cause!=0)~strata(tcell,platelet),data=bmt)

rm1 <- resmean.phreg(out1,times=10*(1:6))
summary(rm1)
par(mfrow=c(1,2))
plot(rm1,se=1)
plot(rm1,years.lost=TRUE,se=1)

## years.lost decomposed into causes
drm1 <- cif.yearslost(Event(time,cause)~strata(tcell,platelet),data=bmt,times=10*(1:6))
summary(drm1)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-nordic-twin.R
\name{ilap}
\alias{ilap}
\title{Inverse Laplace Transform Helper}
\usage{
ilap(theta, t)
}
\arguments{
\item{theta}{frailty parameter.}

\item{t}{value at which to evaluate.}
}
\value{
Numeric value of the inverse Laplace transform.
}
\description{
Computes the inverse Laplace transform for gamma frailty simulation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twinsim.R
\name{twinsim}
\alias{twinsim}
\title{Simulate twin data}
\usage{
twinsim(
  nMZ = 100,
  nDZ = nMZ,
  b1 = c(),
  b2 = c(),
  mu = 0,
  acde = c(1, 1, 0, 1),
  randomslope = NULL,
  threshold = 0,
  cens = FALSE,
  wide = FALSE,
  ...
)
}
\arguments{
\item{nMZ}{Number of monozygotic twin pairs}

\item{nDZ}{Number of dizygotic twin pairs}

\item{b1}{Effect of covariates (labelled x1,x2,...) of type 1. One
distinct covariate value for each twin/individual.}

\item{b2}{Effect of covariates (labelled g1,g2,...) of type 2. One
covariate value for each twin pair.}

\item{mu}{Intercept parameter.}

\item{acde}{Variance of random effects (in the order A,C,D,E)}

\item{randomslope}{Logical indicating wether to include random slopes of
the variance components w.r.t. x1,x2,...}

\item{threshold}{Treshold used to define binary outcome y0}

\item{cens}{Logical variable indicating whether to censor outcome}

\item{wide}{Logical indicating if wide data format should be returned}

\item{...}{Additional arguments parsed on to lower-level functions}
}
\description{
Simulate twin data from a linear normal ACE/ADE/AE model.
}
\seealso{
\code{\link{twinlm}}
}
\author{
Klaus K. Holst
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biprobit.R
\name{biprobit}
\alias{biprobit}
\alias{biprobit.vector}
\alias{biprobit.time}
\title{Bivariate Probit model}
\usage{
biprobit(
  x,
  data,
  id,
  rho = ~1,
  num = NULL,
  strata = NULL,
  eqmarg = TRUE,
  indep = FALSE,
  weights = NULL,
  weights.fun = function(x) ifelse(any(x <= 0), 0, max(x)),
  randomeffect = FALSE,
  vcov = "robust",
  pairs.only = FALSE,
  allmarg = !is.null(weights),
  control = list(trace = 0),
  messages = 1,
  constrain = NULL,
  table = pairs.only,
  p = NULL,
  ...
)
}
\arguments{
\item{x}{formula (or vector)}

\item{data}{data.frame}

\item{id}{The name of the column in the dataset containing the cluster id-variable.}

\item{rho}{Formula specifying the regression model for the dependence parameter}

\item{num}{Optional name of order variable}

\item{strata}{Strata}

\item{eqmarg}{If TRUE same marginals are assumed (exchangeable)}

\item{indep}{Independence}

\item{weights}{Weights}

\item{weights.fun}{Function defining the bivariate weight in each cluster}

\item{randomeffect}{If TRUE a random effect model is used (otherwise correlation parameter is estimated allowing for both negative and positive dependence)}

\item{vcov}{Type of standard errors to be calculated}

\item{pairs.only}{Include complete pairs only?}

\item{allmarg}{Should all marginal terms be included}

\item{control}{Control argument parsed on to the optimization routine. Starting values may be parsed as '\code{start}'.}

\item{messages}{Control amount of messages shown}

\item{constrain}{Vector of parameter constraints (NA where free). Use this to set an offset.}

\item{table}{Type of estimation procedure}

\item{p}{Parameter vector p in which to evaluate log-Likelihood and score function}

\item{...}{Optional arguments}
}
\description{
Bivariate Probit model
}
\examples{
data(prt)
prt0 <- subset(prt,country=="Denmark")
a <- biprobit(cancer~1+zyg, ~1+zyg, data=prt0, id="id")
b <- biprobit(cancer~1+zyg, ~1+zyg, data=prt0, id="id",pairs.only=TRUE)
predict(b,newdata=lava::Expand(prt,zyg=c("MZ")))
predict(b,newdata=lava::Expand(prt,zyg=c("MZ","DZ")))

\donttest{ ## Reduce Ex.Timings
n <- 2e3
x <- sort(runif(n, -1, 1))
y <- rmvn(n, c(0,0), rho=cbind(tanh(x)))>0
d <- data.frame(y1=y[,1], y2=y[,2], x=x)
dd <- fast.reshape(d)

a <- biprobit(y~1+x,rho=~1+x,data=dd,id="id")
summary(a, mean.contrast=c(1,.5), cor.contrast=c(1,.5))
with(predict(a,data.frame(x=seq(-1,1,by=.1))), plot(p00~x,type="l"))

pp <- predict(a,data.frame(x=seq(-1,1,by=.1)),which=c(1))
plot(pp[,1]~pp$x, type="l", xlab="x", ylab="Concordance", lwd=2, xaxs="i")
lava::confband(pp$x,pp[,2],pp[,3],polygon=TRUE,lty=0,col=lava::Col(1))

pp <- predict(a,data.frame(x=seq(-1,1,by=.1)),which=c(9)) ## rho
plot(pp[,1]~pp$x, type="l", xlab="x", ylab="Correlation", lwd=2, xaxs="i")
lava::confband(pp$x,pp[,2],pp[,3],polygon=TRUE,lty=0,col=lava::Col(1))
with(pp, lines(x,tanh(-x),lwd=2,lty=2))

xp <- seq(-1,1,length.out=6); delta <- mean(diff(xp))
a2 <- biprobit(y~1+x,rho=~1+I(cut(x,breaks=xp)),data=dd,id="id")
pp2 <- predict(a2,data.frame(x=xp[-1]-delta/2),which=c(9)) ## rho
lava::confband(pp2$x,pp2[,2],pp2[,3],center=pp2[,1])


}

## Time
\dontrun{
    a <- biprobit.time(cancer~1, rho=~1+zyg, id="id", data=prt, eqmarg=TRUE,
                       cens.formula=Surv(time,status==0)~1,
                       breaks=seq(75,100,by=3),fix.censweights=TRUE)

    a <- biprobit.time2(cancer~1+zyg, rho=~1+zyg, id="id", data=prt0, eqmarg=TRUE,
                       cens.formula=Surv(time,status==0)~zyg,
                       breaks=100)

    #a1 <- biprobit.time2(cancer~1, rho=~1, id="id", data=subset(prt0,zyg=="MZ"), eqmarg=TRUE,
    #                   cens.formula=Surv(time,status==0)~1,
    #                   breaks=100,pairs.only=TRUE)

    #a2 <- biprobit.time2(cancer~1, rho=~1, id="id", data=subset(prt0,zyg=="DZ"), eqmarg=TRUE,
    #                    cens.formula=Surv(time,status==0)~1,
    #                    breaks=100,pairs.only=TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{basehazplot.phreg}
\alias{basehazplot.phreg}
\alias{bplot}
\alias{basecumhaz}
\alias{plotConfRegion}
\alias{plotConfRegionSE}
\alias{plotstrata}
\title{Plotting the baslines of stratified Cox}
\usage{
basehazplot.phreg(
  x,
  se = FALSE,
  time = NULL,
  add = FALSE,
  ylim = NULL,
  xlim = NULL,
  lty = NULL,
  col = NULL,
  lwd = NULL,
  legend = TRUE,
  ylab = NULL,
  xlab = NULL,
  polygon = TRUE,
  level = 0.95,
  stratas = NULL,
  robust = FALSE,
  conf.type = c("plain", "log"),
  ...
)
}
\arguments{
\item{x}{phreg object}

\item{se}{to include standard errors}

\item{time}{to plot for specific time variables}

\item{add}{to add to previous plot}

\item{ylim}{to give ylim}

\item{xlim}{to give xlim}

\item{lty}{to specify lty of components}

\item{col}{to specify col of components}

\item{lwd}{to specify lwd of components}

\item{legend}{to specify col of components}

\item{ylab}{to specify ylab}

\item{xlab}{to specify xlab}

\item{polygon}{to get standard error in shaded form}

\item{level}{of standard errors}

\item{stratas}{wich strata to plot}

\item{robust}{to use robust standard errors if possible}

\item{conf.type}{"plain" or "log" transformed}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Plotting the baselines of stratified Cox
}
\examples{
data(TRACE)
dcut(TRACE) <- ~.
out1 <- phreg(Surv(time,status==9)~vf+chf+strata(wmicat.4),data=TRACE)

par(mfrow=c(2,2))
bplot(out1)
bplot(out1,stratas=c(0,3))
bplot(out1,stratas=c(0,3),col=2:3,lty=1:2,se=TRUE)
bplot(out1,stratas=c(0),col=2,lty=2,se=TRUE,polygon=FALSE)
bplot(out1,stratas=c(0),col=matrix(c(2,1,3),1,3),lty=matrix(c(1,2,3),1,3),se=TRUE,polygon=FALSE)
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brier_binreg.R
\name{print.summary_binregCV_multi}
\alias{print.summary_binregCV_multi}
\title{Print method for summary_binregCV_multi objects}
\usage{
\method{print}{summary_binregCV_multi}(x, digits = 4, ...)
}
\arguments{
\item{x}{A `summary_binregCV_multi` object.}

\item{digits}{Number of digits to round to.}

\item{...}{Not currently used.}
}
\description{
Print method for summary_binregCV_multi objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{cif_yearslost}
\alias{cif_yearslost}
\title{Restricted Mean Time Lost for Competing Risks}
\usage{
cif_yearslost(formula, data = data, cens.code = 0, times = NULL, ...)
}
\arguments{
\item{formula}{Formula for \code{phreg} object with \code{strata} to indicate strata, or \code{+1} if no strata.}

\item{data}{Data frame for calculations.}

\item{cens.code}{Censoring code (needed to separate event codes from censorings).}

\item{times}{Possible times for which to report restricted mean. Summary displays estimates for these times.}

\item{...}{Additional arguments passed to \code{phreg}.}
}
\value{
An object of class \code{"resmean_phreg"} containing:
\item{cumhaz}{Matrix of cumulative hazards (years lost).}
\item{se.cumhaz}{Standard errors.}
\item{intF1times}{Years lost at specified times.}
\item{causes}{Vector of cause codes.}
}
\description{
Computes the Restricted Mean Time Lost (RMTL) for competing risks based on the
integrated Aalen-Johansen estimator.
}
\details{
A set of time points can be given to be returned in the summary, but the function
computes years-lost for all event times and can be plotted/viewed.
The RMTL for a specific time-point can also be computed using the \code{rmstIPCW} function.
}
\examples{
data(bmt)
bmt$time <- bmt$time + runif(408) * 0.001

## Years lost decomposed into causes
drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = c(40, 50))
par(mfrow = c(1, 2))
plot(drm1, cause = 1, se = 1)
plot(drm1, cause = 2, se = 1)
summary(drm1)
estimate(drm1, cause = 1)
estimate(drm1, cause = 2)

## Comparing populations
drm1 <- cif_yearslost(Event(time, cause) ~ strata(tcell, platelet), data = bmt, times = 40)
summary(drm1, contrast = list(1:4))
e1 <- estimate(drm1)
estimate(e1, rbind(c(1, -1, 0, 0)))
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/claytonakes.R
\name{ClaytonOakes}
\alias{ClaytonOakes}
\title{Clayton-Oakes model with piece-wise constant hazards}
\usage{
ClaytonOakes(
  formula,
  data = parent.frame(),
  cluster,
  var.formula = ~1,
  cuts = NULL,
  type = "piecewise",
  start,
  control = list(),
  var.invlink = exp,
  ...
)
}
\arguments{
\item{formula}{formula specifying the marginal proportional (piecewise constant) hazard structure with the right-hand-side being a survival object (Surv) specifying the entry time (optional), the follow-up time, and event/censoring status at follow-up. The clustering can be specified using the special function \code{cluster} (see example below).}

\item{data}{Data frame}

\item{cluster}{Variable defining the clustering (if not given in the formula)}

\item{var.formula}{Formula specifying the variance component structure (if not given via the cluster special function in the formula) using a linear model with log-link.}

\item{cuts}{Cut points defining the piecewise constant hazard}

\item{type}{when equal to \code{two.stage}, the Clayton-Oakes-Glidden estimator will be calculated via the \code{timereg} package}

\item{start}{Optional starting values}

\item{control}{Control parameters to the optimization routine}

\item{var.invlink}{Inverse link function for variance structure model}

\item{...}{Additional arguments}
}
\description{
Clayton-Oakes frailty model
}
\examples{
set.seed(1)
d <- subset(simClaytonOakes(500,4,2,1,stoptime=2,left=2),truncated)
e <- ClaytonOakes(survival::Surv(lefttime,time,status)~x+cluster(~1,cluster),
                  cuts=c(0,0.5,1,2),data=d)
e

d2 <- simClaytonOakes(500,4,2,1,stoptime=2,left=0)
d2$z <- rep(1,nrow(d2)); d2$z[d2$cluster\%in\%sample(d2$cluster,100)] <- 0
## Marginal=Cox Proportional Hazards model:
ts <- ClaytonOakes(survival::Surv(time,status)~timereg::prop(x)+cluster(~1,cluster),
                   data=d2,type="two.stage")
## Marginal=Aalens additive model:
ts2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),
                    data=d2,type="two.stage")
## Marginal=Piecewise constant:
e2 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~-1+factor(z),cluster),
                   cuts=c(0,0.5,1,2),data=d2)
e2


e0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~-1+factor(z),cluster),
                   cuts=c(0,0.5,1,2),data=d2)
ts0 <- ClaytonOakes(survival::Surv(time,status)~cluster(~1,cluster),
                   data=d2,type="two.stage")
plot(ts0)
plot(e0,add=TRUE)

e3 <- ClaytonOakes(survival::Surv(time,status)~x+cluster(~1,cluster),cuts=c(0,0.5,1,2),
                   data=d,var.invlink=identity)
e3
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{base4cumhaz}
\alias{base4cumhaz}
\title{rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen}
\source{
Estimated data
}
\description{
rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/divide.conquer.R
\name{divide.conquer.timereg}
\alias{divide.conquer.timereg}
\title{Split a data set and run function from timereg and aggregate}
\usage{
divide.conquer.timereg(func = NULL, data, size, ...)
}
\arguments{
\item{func}{called function}

\item{data}{data-frame}

\item{size}{size of splits}

\item{...}{Additional arguments to lower level functions}
}
\description{
Split a data set and run function of cox-aalen type and aggregate results
}
\examples{
## library(timereg)
## data(TRACE)
## a <- divide.conquer.timereg(prop.odds,TRACE,
##                            formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=200)
## coef(a)
## a2 <- divide.conquer.timereg(prop.odds,TRACE,
##                              formula=Event(time,status==9)~chf+vf+age,n.sim=0,size=500)
## coef(a2)
## 
##if (interactive()) {
##par(mfrow=c(1,1))
##plot(a,xlim=c(0,8),ylim=c(0,0.01))
##par(new=TRUE)
##plot(a2,xlim=c(0,8),ylim=c(0,0.01))
##}
}
\author{
Thomas Scheike, Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cifreg.R
\name{cifreg}
\alias{cifreg}
\alias{vecAllStrata}
\alias{diffstrata}
\alias{FGprediid}
\alias{indexstratarightR}
\alias{gofFG}
\title{Cumulative Incidence Function (CIF) Regression}
\usage{
cifreg(
  formula,
  data,
  propodds = 1,
  cause = 1,
  cens.code = 0,
  no.codes = NULL,
  death.code = NULL,
  ...
)
}
\arguments{
\item{formula}{Formula with an 'Event' outcome.}

\item{data}{Data frame containing the variables.}

\item{propodds}{Logical; if \code{1} (default), fits the logit link model. If \code{NULL}, fits the Fine-Gray model.}

\item{cause}{Cause of interest (default is 1).}

\item{cens.code}{Code for censoring (default is 0).}

\item{no.codes}{Event codes to be ignored when identifying competing causes (useful for administrative censoring).}

\item{death.code}{Codes for death (terminal events). If \code{NULL}, defaults to all remaining codes (excluding \code{cause}, \code{cens.code}, and \code{no.codes}).}

\item{...}{Additional arguments passed to \code{recreg}.}
}
\value{
An object of class \code{"cifreg"} (extending \code{"phreg"}) containing:
\item{coef}{Estimated coefficients.}
\item{var}{Robust variance-covariance matrix.}
\item{iid}{Influence functions for the coefficients.}
\item{cumhaz}{Cumulative incidence estimates.}
\item{propodds}{Indicator of the link function used.}
}
\description{
Fits a regression model for the cumulative incidence function (CIF) in the presence of competing risks.
Supports two link functions:
\itemize{
\item \code{propodds=1} (default): Logistic link model (logit of CIF), providing Odds Ratio (OR) interpretations.
\item \code{propodds=NULL}: Fine-Gray (cloglog) regression model, providing subdistribution hazard ratio interpretations.
}
}
\details{
For the Fine-Gray model, the score equations are:
\deqn{ \int (X - E(t)) Y_1(t) w(t) dM_1 }
summed over clusters and returned as \code{iid.naive} (multiplied by the inverse of the second derivative).
Here, \deqn{w(t) = G(t) (I(T_i \wedge t < C_i)/G_c(T_i \wedge t))},
\deqn{E(t) = S_1(t)/S_0(t)}, and \deqn{S_j(t) = \sum X_i^j Y_{i1}(t) w_i(t) \exp(X_i^T \beta)}.

The full influence function (IID decomposition) for the beta coefficients includes a censoring term:
\deqn{ \int (X - E(t)) Y_1(t) w(t) dM_1 + \int q(s)/p(s) dM_c }
which is returned as the \code{iid} component.

For the logistic link model, standard errors may be slightly underestimated because uncertainty from the
recursive baseline estimation is not fully accounted for. For smaller datasets, it is recommended to use
the \code{prop.odds.subdist} function from the \pkg{timereg} package (which uses more efficient weights)
or to bootstrap the standard errors.
}
\examples{
## data with no ties
data(bmt,package="mets")
bmt$time <- bmt$time+runif(nrow(bmt))*0.01
bmt$id <- 1:nrow(bmt)

## logistic link  OR interpretation
or=cifreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
summary(or)
par(mfrow=c(1,2))
plot(or)
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
por <- predict(or,nd)
plot(por)

## approximate standard errors 
por <-mets:::predict.phreg(or,nd)
plot(por,se=1)

## Fine-Gray model
fg=cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
summary(fg)
##fg=recreg(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1,death.code=2)
##summary(fg)
plot(fg)
nd <- data.frame(tcell=c(1,0),platelet=0,age=0)
pfg <- predict(fg,nd,se=1)
plot(pfg,se=1)

## bt <- iidBaseline(fg,time=30)
## bt <- IIDrecreg(fg$cox.prep,fg,time=30)

## not run to avoid timing issues
## gofFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)

sfg <- cifregFG(Event(time,cause)~strata(tcell)+platelet+age,data=bmt,cause=1)
summary(sfg)
plot(sfg)

### predictions with CI based on iid decomposition of baseline and beta
### these are used in the predict function above
fg <- cifregFG(Event(time,cause)~tcell+platelet+age,data=bmt,cause=1)
Biid <- iidBaseline(fg,time=20)
pfg1 <- FGprediid(Biid,nd)
pfg1
}
\seealso{
\code{\link{cifregFG}}, \code{\link{recreg}}, \code{\link{gofFG}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{iidBaseline}
\alias{iidBaseline}
\title{Influence Functions or IID Decomposition of Baseline}
\usage{
iidBaseline(
  object,
  time = NULL,
  ft = NULL,
  fixbeta = NULL,
  beta.iid = NULL,
  tminus = FALSE,
  ...
)
}
\arguments{
\item{object}{Object of class \code{"phreg"}, \code{"recreg"}, or \code{"cifregFG"}.}

\item{time}{Time point for baseline IID (required).}

\item{ft}{Function to compute IID of baseline integrated against \eqn{f(t)}.}

\item{fixbeta}{Logical; if \code{TRUE}, fixes the coefficients (useful for specific tests).}

\item{beta.iid}{Optional matrix of beta influence functions to use.}

\item{tminus}{Logical; if \code{TRUE}, computes predictions at \eqn{t-} (strictly before \eqn{t}), useful for IPCW techniques.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"iidBaseline"} containing:
\item{time}{Time point.}
\item{base.iid}{Influence functions for the baseline.}
\item{beta.iid}{Influence functions for the regression coefficients.}
\item{cumhaz.time}{Cumulative hazard at the specified time.}
\item{strata}{Strata indices.}
}
\description{
Computes the influence functions for the baseline cumulative hazard (and optionally
regression coefficients) for \code{phreg}, \code{recreg}, or \code{cifregFG} objects.
}
\details{
The decomposition is based on the formula:
\deqn{ \sum_i \int_0^t \frac{f(s)}{S_0(s)} dM_{ki}(s) - P(t) \beta_k }
where \eqn{k} denotes the stratum and \eqn{i} the cluster.
}
\seealso{
\code{\link{phreg}}, \code{\link{recreg}}, \code{\link{cifreg}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{Grandom.cif}
\alias{Grandom.cif}
\title{Additive Random effects model for competing risks data for polygenetic modelling}
\usage{
Grandom.cif(
  cif,
  data,
  cause = NULL,
  cif2 = NULL,
  times = NULL,
  cause1 = 1,
  cause2 = 1,
  cens.code = NULL,
  cens.model = "KM",
  Nit = 40,
  detail = 0,
  clusters = NULL,
  theta = NULL,
  theta.des = NULL,
  weights = NULL,
  step = 1,
  sym = 0,
  same.cens = FALSE,
  censoring.weights = NULL,
  silent = 1,
  var.link = 0,
  score.method = "nr",
  entry = NULL,
  estimator = 1,
  trunkp = 1,
  admin.cens = NULL,
  random.design = NULL,
  ...
)
}
\arguments{
\item{cif}{a model object from the timereg::comp.risk function with the
marginal cumulative incidence of cause2, i.e., the event that is conditioned on, and whose
odds the comparision is made with respect to}

\item{data}{a data.frame with the variables.}

\item{cause}{specifies the causes  related to the death
times, the value cens.code is the censoring value.}

\item{cif2}{specificies model for cause2 if different from cause1.}

\item{times}{time points}

\item{cause1}{cause of first coordinate.}

\item{cause2}{cause of second coordinate.}

\item{cens.code}{specificies the code for the censoring if NULL then uses the one from the marginal cif model.}

\item{cens.model}{specified which model to use for the ICPW, KM is Kaplan-Meier alternatively it may be "cox"}

\item{Nit}{number of iterations for Newton-Raphson algorithm.}

\item{detail}{if 0 no details are printed during iterations, if 1 details are given.}

\item{clusters}{specifies the cluster structure.}

\item{theta}{specifies starting values for the cross-odds-ratio parameters of the model.}

\item{theta.des}{specifies a regression design for the cross-odds-ratio parameters.}

\item{weights}{weights for score equations.}

\item{step}{specifies the step size for the Newton-Raphson algorith.m}

\item{sym}{1 for symmetri and 0 otherwise}

\item{same.cens}{if true then censoring within clusters are assumed to be the same variable, default is independent censoring.}

\item{censoring.weights}{Censoring probabilities}

\item{silent}{debug information}

\item{var.link}{if var.link=1 then var is on log-scale.}

\item{score.method}{default uses "nlminb" optimzer, alternatively, use the "nr" algorithm.}

\item{entry}{entry-age in case of delayed entry. Then two causes must be given.}

\item{estimator}{estimator}

\item{trunkp}{gives probability of survival for delayed entry, and related to entry-ages given above.}

\item{admin.cens}{Administrative censoring}

\item{random.design}{specifies a regression design of 0/1's for the random effects.}

\item{...}{extra arguments.}
}
\value{
returns an object of type 'random.cif'. With the following arguments:
\item{theta}{estimate of parameters of model.}
\item{var.theta}{variance for gamma.  }
\item{hess}{the derivative of the used score.}
\item{score}{scores at final stage.}
\item{theta.iid}{matrix of iid decomposition of parametric effects.}
}
\description{
Fits a random effects  model describing the dependence in the cumulative 
incidence curves for subjects within a cluster.  Given the gamma distributed
random effects it is assumed that the cumulative incidence curves 
are indpendent, and that the marginal cumulative incidence curves 
are on additive form
\deqn{
P(T \leq t, cause=1 | x,z) = P_1(t,x,z) = 1- exp( -x^T A(t) - t z^T \beta)
}
}
\details{
We allow a regression structure for the indenpendent gamma distributed 
random effects  and their variances that may depend on cluster covariates.

random.design specificies the random effects for each subject within a cluster. This is
a matrix of 1's and 0's with dimension n x d.  With d random effects. 
For a cluster with two subjects, we let the random.design rows be 
\eqn{v_1} and \eqn{v_2}. 
Such that the random effects for subject 
1 is \deqn{v_1^T (Z_1,...,Z_d)}, for d random effects. Each random effect
has an associated parameter \eqn{(\lambda_1,...,\lambda_d)}. By construction
subjects 1's random effect are Gamma distributed with 
mean \eqn{\lambda_1/v_1^T \lambda}
and variance \eqn{\lambda_1/(v_1^T \lambda)^2}. Note that the random effect 
\eqn{v_1^T (Z_1,...,Z_d)} has mean 1 and variance \eqn{1/(v_1^T \lambda)}.

The parameters \eqn{(\lambda_1,...,\lambda_d)}
are related to the parameters of the model
by a regression construction \eqn{pard} (d x k), that links the \eqn{d} 
\eqn{\lambda} parameters
with the (k) underlying \eqn{\theta} parameters 
\deqn{
\lambda = pard \theta 
}
}
\examples{
\donttest{ ## Reduce Ex.Timings
 d <- simnordic.random(5000,delayed=TRUE,
       cordz=1.0,cormz=2,lam0=0.3,country=TRUE)
 times <- seq(50,90,by=10)
 addm <- timereg::comp.risk(Event(time,cause)~-1+factor(country)+cluster(id),data=d,
 times=times,cause=1,max.clust=NULL)

 ### making group indidcator 
 mm <- model.matrix(~-1+factor(zyg),d)

 out1m<-random.cif(addm,data=d,cause1=1,cause2=1,theta=1,
		   theta.des=mm,same.cens=TRUE)
 summary(out1m)
 
 ## this model can also be formulated as a random effects model 
 ## but with different parameters
 out2m<-Grandom.cif(addm,data=d,cause1=1,cause2=1,
		    theta=c(0.5,1),step=1.0,
		    random.design=mm,same.cens=TRUE)
 summary(out2m)
 1/out2m$theta
 out1m$theta
 
 ####################################################################
 ################### ACE modelling of twin data #####################
 ####################################################################
 ### assume that zygbin gives the zygosity of mono and dizygotic twins
 ### 0 for mono and 1 for dizygotic twins. We now formulate and AC model
 zygbin <- d$zyg=="DZ"

 n <- nrow(d)
 ### random effects for each cluster
 des.rv <- cbind(mm,(zygbin==1)*rep(c(1,0)),(zygbin==1)*rep(c(0,1)),1)
 ### design making parameters half the variance for dizygotic components
 pardes <- rbind(c(1,0), c(0.5,0),c(0.5,0), c(0.5,0), c(0,1))

 outacem <-Grandom.cif(addm,data=d,cause1=1,cause2=1,
		same.cens=TRUE,theta=c(0.35,0.15),
            step=1.0,theta.des=pardes,random.design=des.rv)
 summary(outacem)

}
}
\references{
A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
Scheike, Zhang, Sun, Jensen (2010), Biometrika.

Cross odds ratio Modelling of dependence for
Multivariate Competing Risks Data, Scheike and Sun (2013), Biostatitistics.

Scheike, Holst, Hjelmborg (2014),  LIDA,  
Estimating heritability for cause specific hazards based on twin data
}
\author{
Thomas Scheike
}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brier_binreg.R
\name{summary.binregCV}
\alias{summary.binregCV}
\title{Summary method for binregCV objects}
\usage{
\method{summary}{binregCV}(
  object,
  transform = NULL,
  conf.level = 0.95,
  type = c("cv", "both"),
  ...
)
}
\arguments{
\item{object}{A \code{binregCV} object, as returned by
\code{\link{brier_binreg}} for a single evaluation time.}

\item{transform}{Optional back-transformation function applied to Brier
scores and confidence limits, e.g. \code{exp} to undo the internal log
scale.}

\item{conf.level}{Confidence level for the reported intervals (default
0.95).}

\item{type}{Either \code{"cv"} (default) for cross-validated Brier scores
only, or \code{"both"} to also include apparent (in-sample) scores.}

\item{...}{Not currently used.}
}
\description{
Summary method for binregCV objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brier_binreg.R
\name{brier_binreg}
\alias{brier_binreg}
\title{Cross-validated Brier score for competing risks, RMST and RMTL regression
using stratified leave-fold-out fits (\code{binregStrata})}
\usage{
brier_binreg(
  formula,
  data,
  time,
  fold = 5,
  rhs = NULL,
  rhs0 = ~+1,
  cens.model = ~+1,
  brier.cens.model = ~+1,
  brier.args = list(),
  cv.split.index = NULL,
  ...
)
}
\arguments{
\item{formula}{Two-sided formula of the form
\code{Event(time, status) ~ covariates + cluster(id)} or
\code{Event(time, status, cause) ~ covariates + cluster(id)}.
The \code{cluster(id)} term is optional; if absent an internal
identifier \code{id__} is added automatically assuming independent
observations.}

\item{data}{A \code{data.frame} containing all variables in
\code{formula} and \code{rhs}.}

\item{time}{Scalar or numeric vector of evaluation time points. When a
vector is supplied the function loops over time points reusing the same
fold structure and returns a named list of \code{binregCV} objects with
class \code{"binregCV_list"}.}

\item{fold}{Integer. Number of cross-validation folds / strata
(default 5).}

\item{rhs}{Named list of one-sided formulas specifying candidate models,
e.g. \code{list(small = ~Z1, full = ~Z1 + Z2)}. If \code{NULL} (default),
the right-hand side of \code{formula} (minus any \code{cluster()} term)
is used as a single candidate named \code{"model1"}.}

\item{rhs0}{One-sided formula for the null (intercept-only) model
(default \code{~+1}).}

\item{cens.model}{Censoring model passed to \code{binregStrata} for the
CV fold fits (default \code{~+1}). Since folds are handled internally
by \code{binregStrata}'s \code{strata}/\code{cv.fold} mechanism, this
does \emph{not} need to include \code{strata(fold)}.}

\item{brier.cens.model}{Censoring model passed to \code{resmeanIPCW} when
computing Brier scores. Default \code{~+1} gives Kaplan-Meier weights.
Supply a stratified model, e.g. \code{~strata(Z1, Z2)}, for
covariate-adjusted IPCW weights.}

\item{brier.args}{Named list of additional arguments forwarded to every
\code{resmeanIPCW} call (beyond \code{formula}, \code{data},
\code{time}, \code{Ydirect}, \code{cens.model}, and \code{model} which
are set internally). Typically empty.}

\item{cv.split.index}{List from \code{folds(n, fold)} or a vector of fold
indices (1:fold) for each observation, to fix the cross-validation
folds. \code{NULL} (default) generates folds internally via
\code{folds(n, fold)}.}

\item{\dots}{Additional arguments passed to every \code{binreg} /
\code{binregStrata} call, e.g. \code{cause = 2}, \code{outcome = "rmst"},
or \code{offset}.}
}
\value{
An object of class \code{"binregCV"} (single time point) or
  \code{"binregCV_list"} (multiple time points), with the following
  structure:
\describe{
  \item{\code{null}}{List with \code{coef}, \code{se}, and \code{iid} for
    the null model. \code{coef} and \code{se} are named vectors with
    entries \code{bbrier} (apparent, log scale) and \code{bbrierCV}
    (cross-validated, log scale). \code{iid} contains the
    influence-function matrix for \code{bbrierCV} with delta-method
    correction for the stratified CV prediction step and the log
    transform.}
  \item{\code{models}}{Named list, one element per entry in \code{rhs}.
    Each element contains \code{coef} (\code{brier}, \code{brierCV}, log
    scale), \code{se}, \code{dcoef} (differences from null: log Brier
    ratios), \code{sed}, and \code{iid} (influence-function matrix for
    \code{brierCV}, log scale, with CV delta-method correction).}
  \item{\code{iid_mat}}{Matrix of dimension \eqn{n \times (1 +
    \code{length(rhs)})} with columns \code{null} and one per entry of
    \code{rhs}, each the log-scale CV Brier influence function for the
    corresponding model. Used by \code{\link{estimate.binregCV}} for joint
    inference.}
  \item{\code{cv.split.index}}{List of fold index vectors used.}
  \item{\code{rhs}, \code{rhs0}}{Processed candidate and null formulas
    (cluster terms removed).}
  \item{\code{model}}{Link function read from the \code{binreg} fit:
    \code{"logit"}, \code{"exp"}, or \code{"lin"}.}
  \item{\code{call}, \code{time}, \code{fold}, \code{cens.model},
    \code{brier.cens.model}}{Call and settings for printing and
    downstream use.}
}
}
\description{
Fits candidate models via \code{\link[mets]{binregStrata}} with
\code{cv.fold = TRUE} and evaluates each using apparent (in-sample) and
\emph{k}-fold cross-validated IPCW Brier scores relative to a null model.
Supports all outcome types available in \code{binreg}/\code{binregStrata}:
cause-specific cumulative incidence (competing risks), restricted mean
survival time (RMST), and restricted mean time lost (RMTL).
}
\details{
Cross-validation is delegated directly to
\code{\link[mets]{binregStrata}} via its \code{strata} and
\code{cv.fold = TRUE} arguments: each stratum (fold) gets its own
coefficient vector \eqn{\beta_s}, estimated on the complement of that
fold. Out-of-fold predictions are obtained by routing each observation to
the \eqn{\beta_s} corresponding to its own fold, which was trained on the
other folds.

The fold membership is stored in an internal \code{fold} column (1-based)
in \code{data} and passed to \code{binregStrata} as a 0-based
\code{strata} vector. Censoring weights for the underlying competing-risks
model are estimated according to \code{cens.model} as usual; since strata
already define the folds, \code{cens.model} need not include
\code{strata(fold)} explicitly.

Brier scores are computed on the \strong{linear scale} via
\code{resmeanIPCW(..., model = "lin")} and then log-transformed using
\code{lava::estimate(..., f = log)} with the delta method. All stored
coefficients, standard errors, and influence functions are therefore on
the \strong{log scale}. Use \code{summary(fit, transform = exp)} or
\code{estimate(fit, f = exp)} to back-transform to the original Brier
scale.

The delta-method correction for the cross-validated influence function
accounts for the block structure of \code{binregStrata}'s influence
function (\eqn{n_{id} \times (\code{nstrata} \times p)}, one
\eqn{p}-column block per stratum). For each stratum \eqn{s}, only
observations belonging to fold \eqn{s} have a non-zero derivative of
their out-of-fold prediction with respect to \eqn{\beta_s}, since
\eqn{\beta_s} is exactly the coefficient used to predict fold \eqn{s}.
The full gradient is therefore block-sparse, with one non-zero
\eqn{p}-length block per stratum, and the correction reduces to a single
matrix product against \code{iid(outff_fit)}.

Left-truncation (delayed entry) is not supported and will raise an error.
Specifically, \code{Event(entry, time, status)} is rejected; use
\code{Event(time, status)} or \code{Event(time, status, cause)}.
}
\section{Outcome types}{

The function supports all outcome types provided by \code{binreg}/
\code{binregStrata}:
\describe{
  \item{Competing risks CIF}{Default; use
    \code{Event(time, status, cause)} and pass \code{cause} via \code{...}.
    The Brier score measures squared error for the cause-specific CIF.}
  \item{RMST}{Pass \code{outcome = "rmst"} via \code{...}. The Brier score
    measures squared error for the restricted mean survival time up to
    \code{time}.}
  \item{RMTL}{Pass \code{outcome = "rmtl"} via \code{...}. The Brier score
    measures squared error for the restricted mean time lost.}
}
}

\section{Scale and inference}{

All output is on the log scale. To obtain Brier scores on the original
scale with confidence intervals, use:
\preformatted{
  summary(fit, transform = exp)
  estimate(fit, f = exp)
}
Differences on the log scale correspond to log Brier ratios; exponentiate
to obtain Brier ratios. For absolute differences on the original scale,
compute contrasts after \code{estimate(fit, f = exp)}.
}

\examples{
\dontrun{
library(mets)
data(bmt)
bmt$id <- seq_len(nrow(bmt))

## --- competing risks CIF/Survival (default) ---
fit <- brier_binreg(
  Event(time, cause) ~ tcell + platelet + age + cluster(id),
  data = bmt, time = 50,
  rhs = list(small = ~age, full = ~tcell + platelet + age)
)
summary(fit)                        ## log scale
summary(fit, transform = exp)       ## Brier scale

## multiple time points
fitm <- brier_binreg(
  Event(time, cause) ~ tcell + platelet + age + cluster(id),
  data = bmt, time = c(50, 60),
  rhs = list(small = ~age, full = ~tcell + platelet + age)
)
summary(fitm, transform = exp)

## joint inference across models via lava::estimate
e <- estimate(fitm)
e
estimate(e, f = exp)               ## back-transform

## stratified IPCW weights for Brier score
fit_strat <- brier_binreg(
  Event(time, cause) ~ tcell + platelet + age + cluster(id),
  data = bmt, time = 50,
  rhs = list(small = ~age, full = ~tcell + platelet + age),
  brier.cens.model = ~strata(tcell)
)

## --- RMST --- survival case only
fit_rmst <- brier_binreg(
  Event(time, cause) ~ tcell + platelet + age + cluster(id),
  data = bmt, time = 50, cause = 1:2,
  rhs    = list(small = ~age, full = ~tcell + platelet + age),
  outcome = "rmst"
)
summary(fit_rmst, transform = exp)

## --- RMTL --- competing risks or survival (RMTL)
fit_rmtl <- brier_binreg(
  Event(time, cause) ~ tcell + platelet + age + cluster(id),
  data = bmt, time = 50,
  rhs    = list(small = ~age, full = ~tcell + platelet + age),
  outcome = "rmtl"
)
summary(fit_rmtl, transform = exp)
}
}
\seealso{
\code{\link{estimate.binregCV}} for joint inference across models and
  time points;
  \code{\link{summary.binregCV}} and \code{\link{print.summary_binregCV_multi}}
  for formatted output;
  \code{\link[mets]{binregStrata}} for the underlying stratified
  regression model;
  \code{\link[mets]{resmeanIPCW}} for IPCW mean estimation;
  \code{\link[lava]{estimate}} for delta-method inference.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{rchazl}
\alias{rchazl}
\title{Multiple Cause Piecewise Constant Hazard Simulation}
\usage{
rchazl(cumhaz, rr, causes = NULL, ...)
}
\arguments{
\item{cumhaz}{List of cumulative hazard matrices, one for each cause.}

\item{rr}{Matrix of relative risks (rows = subjects, columns = causes).}

\item{causes}{Vector of cause codes to assign (default NULL, uses 1,2,...).}

\item{...}{Additional arguments passed to \code{rchaz}.}
}
\value{
A data frame with event times and status indicating the cause of the first event.
}
\description{
Simulates data from multiple piecewise constant baseline hazards for competing risks. 
Takes the minimum of all cause-specific event times and assigns the corresponding cause.
}
\seealso{
\code{\link{rchaz}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binomial.twostage.R, R/sim.clayton.oakes.R,
%   R/twostage.R
\name{p11_binomial_twostage_RV}
\alias{p11_binomial_twostage_RV}
\alias{concordanceTwostage}
\alias{concordanceTwinACE}
\alias{kendall_ClaytonOakes_twin_ace}
\alias{kendall.ClaytonOakes.twin.ace}
\alias{kendall_normal_twin_ace}
\alias{ascertained_pairs}
\alias{twin-design}
\alias{twin_polygen_design}
\alias{twin.polygen.design}
\alias{ace_family_design}
\alias{make_pairwise_design}
\alias{p11.binomial.twostage.RV}
\title{Concordance Probability from Twostage Model}
\usage{
p11_binomial_twostage_RV(
  theta,
  rv1,
  rv2,
  p1,
  p2,
  pardes,
  ags = NULL,
  link = 0,
  i = 1,
  j = 1
)

concordanceTwostage(
  theta,
  p,
  rv1,
  rv2,
  theta.des,
  additive.gamma.sum = NULL,
  link = 0,
  var.par = 0,
  ...
)

concordanceTwinACE(
  object,
  rv1 = NULL,
  rv2 = NULL,
  xmarg = NULL,
  type = "ace",
  ...
)

kendall_ClaytonOakes_twin_ace(parg, parc, K = 10000, test = 0)

kendall.ClaytonOakes.twin.ace(x, y, ...)

kendall_normal_twin_ace(parg, parc, K = 10000)

ascertained_pairs(pairs, data, cr.models, bin = FALSE)

twin.polygen.design(x, ...)

ace_family_design(
  data,
  id = "id",
  member = "type",
  mother = "mother",
  father = "father",
  child = "child",
  child1 = "child",
  type = "ace",
  ...
)

make_pairwise_design(pairs, kinship, type = "ace")
}
\arguments{
\item{theta}{dependence parameter vector.}

\item{rv1}{random-effects design for subject 1.}

\item{rv2}{random-effects design for subject 2.}

\item{p1}{marginal probability for subject 1.}

\item{p2}{marginal probability for subject 2.}

\item{pardes}{parameter design matrix.}

\item{ags}{additive gamma sum matrix (optional).}

\item{link}{link function indicator (0 = identity, 1 = log).}

\item{i}{index for subject 1.}

\item{j}{index for subject 2.}

\item{p}{matrix of marginal probabilities (n x 2).}

\item{theta.des}{parameter design matrix linking theta to lambda parameters.}

\item{additive.gamma.sum}{optional matrix for additive gamma sums.}

\item{var.par}{if 1, parameters are rescaled by sum squared.}

\item{...}{additional arguments.}

\item{object}{a binomial twostage model object.}

\item{xmarg}{optional covariate values for marginal probabilities.}

\item{type}{model type: \code{"ace"}, \code{"ade"}, \code{"ae"}, \code{"de"},
\code{"dce"}, or \code{"un"}.}

\item{parg}{genetic variance parameter (gamma shape for genetic component).}

\item{parc}{common environment variance parameter (gamma shape for environment).}

\item{K}{number of simulated twin pairs (multiplied by 2 internally).}

\item{test}{if 1, prints diagnostic correlations.}

\item{x}{passed as \code{parg} (alias wrapper).}

\item{y}{passed as \code{parc} (alias wrapper).}

\item{pairs}{matrix of pair indices (n x 2).}

\item{data}{a data.frame with twin/family data.}

\item{cr.models}{formula specifying time and status variables.}

\item{bin}{logical; if TRUE uses binary (prevalence) ordering rather than time ordering.}

\item{id}{character name of the cluster (pair) identifier column.}

\item{member}{character name of the family member type column.}

\item{mother}{value identifying mothers in the member column.}

\item{father}{value identifying fathers in the member column.}

\item{child}{value identifying children in the member column.}

\item{child1}{column name distinguishing first child from second.}

\item{kinship}{vector of kinship coefficients for each pair.}
}
\value{
A list of concordance tables, one per pair, each containing
  \code{pmat} (2x2 probability matrix), \code{casewise} (casewise concordance),
  and \code{marg} (marginal probabilities).

A list of concordance tables per zygosity group.

A list with components:
  \item{pardes}{parameter design matrix linking random effects to variance parameters.}
  \item{des.rv}{random-effects design matrix for subjects.}
}
\description{
Computes concordance probability (joint probability of both subjects
experiencing the event) given dependence parameters and random-effect
variance structures from a twostage model.

Computes concordance probabilities for twin ACE/ADE models from a
binomial twostage object.

Functions for constructing random-effects design matrices for twin and
family models. These designs specify the genetic (A), dominance (D),
common environment (C), and unique environment (E) variance components.
}
\details{
\code{twin_polygen_design} creates a polygenic random-effects design for
twin pairs, distinguishing MZ and DZ twins.

\code{twin.polygen.design} is an alias for \code{twin_polygen_design}.

\code{ace_family_design} creates designs for nuclear families (mother,
father, children).

\code{make_pairwise_design} creates pairwise random-effects designs for
arbitrary kinship structures.

\code{concordanceTwostage} computes concordance probabilities from a
twostage model.

\code{concordanceTwinACE} computes concordance from a twin ACE model.

\code{kendall_ClaytonOakes_twin_ace} and \code{kendall_normal_twin_ace}
compute Kendall's tau for Clayton-Oakes and normal-frailty twin ACE models
respectively.

\code{ascertained_pairs} identifies ascertained (affected) pairs in
clustered survival data.

\code{p11_binomial_twostage_RV} computes the joint probability P(T1<=t, T2<=t)
for the additive gamma binary random effects model.
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R, R/npc.R, R/plotcr.R
\name{rr_cif}
\alias{rr_cif}
\alias{or_cif}
\alias{random.cif}
\alias{Grandom.cif}
\alias{predictPairPlack}
\alias{cif-nonpar}
\alias{npc}
\alias{nonparcuminc}
\alias{plotcr}
\title{Non-parametric Cumulative Incidence Functions}
\usage{
rr_cif(
  cif,
  data,
  cause = NULL,
  cif2 = NULL,
  times = NULL,
  cause1 = 1,
  cause2 = 1,
  cens.code = NULL,
  cens.model = "KM",
  Nit = 40,
  detail = 0,
  clusters = NULL,
  theta = NULL,
  theta.des = NULL,
  step = 1,
  sym = 0,
  weights = NULL,
  same.cens = FALSE,
  censoring.weights = NULL,
  silent = 1,
  par.func = NULL,
  dpar.func = NULL,
  dimpar = NULL,
  score.method = "nlminb",
  entry = NULL,
  estimator = 1,
  trunkp = 1,
  admin.cens = NULL,
  ...
)

or_cif(
  cif,
  data,
  cause = NULL,
  cif2 = NULL,
  times = NULL,
  cause1 = 1,
  cause2 = 1,
  cens.code = NULL,
  cens.model = "KM",
  Nit = 40,
  detail = 0,
  clusters = NULL,
  theta = NULL,
  theta.des = NULL,
  step = 1,
  sym = 0,
  weights = NULL,
  same.cens = FALSE,
  censoring.weights = NULL,
  silent = 1,
  par.func = NULL,
  dpar.func = NULL,
  dimpar = NULL,
  score.method = "nlminb",
  entry = NULL,
  estimator = 1,
  trunkp = 1,
  admin.cens = NULL,
  ...
)

random.cif(cif, ...)

Grandom.cif(cif, ...)

predictPairPlack(cif1, cif2, status1, status2, theta)

npc(T, cause, same.cens = TRUE, sep = FALSE)

nonparcuminc(t, status, cens = 0)

plotcr(
  x,
  col,
  lty,
  legend = TRUE,
  which = 1:2,
  cause = 1:2,
  ask = prod(par("mfcol")) < length(which) && dev.interactive(),
  ...
)
}
\arguments{
\item{cif}{a cumulative incidence model object (from timereg).}

\item{data}{a data.frame with the variables.}

\item{cause}{causes to plot.}

\item{cif2}{optional second CIF model if different from first.}

\item{times}{time points for evaluation.}

\item{cause1}{cause for first coordinate.}

\item{cause2}{cause for second coordinate.}

\item{cens.code}{censoring code value.}

\item{cens.model}{censoring model type (default \code{"KM"}).}

\item{Nit}{maximum number of iterations.}

\item{detail}{level of output detail.}

\item{clusters}{cluster variable name or vector.}

\item{theta}{dependence parameter(s).}

\item{theta.des}{design matrix for theta.}

\item{step}{step size for optimization.}

\item{sym}{if 1, symmetric dependence structure.}

\item{weights}{optional weights.}

\item{same.cens}{logical; if TRUE, uses joint censoring weights.}

\item{censoring.weights}{optional pre-computed censoring weights.}

\item{silent}{verbosity level.}

\item{par.func}{optional parameter function.}

\item{dpar.func}{optional derivative of parameter function.}

\item{dimpar}{dimension of parameter vector.}

\item{score.method}{optimization method (default \code{"nlminb"}).}

\item{entry}{optional entry time variable.}

\item{estimator}{estimator type.}

\item{trunkp}{truncation probability.}

\item{admin.cens}{administrative censoring time.}

\item{...}{additional arguments.}

\item{cif1}{CIF values for subject 1 (for \code{predictPairPlack}).}

\item{status1}{status for subject 1.}

\item{status2}{status for subject 2.}

\item{T}{matrix with columns: time1, time2, status1, status2 (for \code{npc}).}

\item{sep}{logical; if TRUE, uses separate censoring models for each subject.}

\item{t}{vector of event/censoring times (for \code{nonparcuminc}).}

\item{status}{vector of status codes (for \code{nonparcuminc}).}

\item{cens}{censoring code (default 0).}

\item{x}{data matrix or competing risks object.}

\item{col}{colors for curves.}

\item{lty}{line types for curves.}

\item{legend}{logical; if TRUE, add legend.}

\item{which}{which plots to show.}

\item{ask}{logical; if TRUE, prompt before new page.}
}
\value{
For \code{npc}: matrix with columns (time, cumulative incidence).
  For \code{nonparcuminc}: matrix with time and cause-specific cumulative incidences.
}
\description{
Functions for computing and visualizing non-parametric cumulative incidence
estimates, as well as dependence measures (odds ratio, relative risk) for
bivariate competing risks data.
}
\details{
\code{npc} computes bivariate non-parametric cumulative incidence using
inverse-probability-of-censoring weights.

\code{nonparcuminc} computes univariate non-parametric cumulative incidence
for multiple causes.

\code{plotcr} plots cumulative incidence curves for competing risks using
the prodlim package.

\code{or_cif} fits an odds-ratio model for bivariate cumulative incidence.

\code{rr_cif} fits a relative-risk model for bivariate cumulative incidence.

\code{random.cif} and \code{Grandom.cif} are aliases for \code{random_cif}
and \code{Grandom_cif} (random effects CIF models).

\code{predictPairPlack} predicts pairwise joint probabilities under a
Plackett (odds-ratio) dependence model.

\code{matplot.mets.twostage} produces matrix-plots of concordance over time
from a twostage object.
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinomialreg.R
\name{predict.mlogit}
\alias{predict.mlogit}
\title{Predictions from Multinomial Regression}
\usage{
\method{predict}{mlogit}(
  object,
  newdata,
  se = TRUE,
  response = TRUE,
  Y = NULL,
  level = 0.95,
  ...
)
}
\arguments{
\item{object}{Object of class \code{"mlogit"}.}

\item{newdata}{Data frame for prediction. If missing, predictions are made for the original data.}

\item{se}{Logical; if \code{TRUE}, computes standard errors and confidence intervals.}

\item{response}{Logical; if \code{TRUE} (default), returns probabilities only for the observed response
(if \code{newdata} contains the response). If \code{FALSE}, returns probabilities for all categories.}

\item{Y}{Vector of outcome levels to predict probabilities for (optional).}

\item{level}{Confidence level for intervals (default 0.95).}

\item{...}{Additional arguments.}
}
\value{
A matrix or data frame containing:
\item{pred}{Predicted probabilities.}
\item{se}{Standard errors (if \code{se=TRUE}).}
\item{lower, upper}{Confidence intervals (if \code{se=TRUE}).}

If \code{response=FALSE}, columns correspond to the levels of the outcome factor.
}
\description{
Computes predicted probabilities for the categorical outcome based on a \code{mlogit} object.
Can return probabilities for all categories or only for the observed response.
}
\seealso{
\code{\link{mlogit}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{calgb8923}
\alias{calgb8923}
\title{CALGB 8923, twostage randomization SMART design}
\format{
Data from smart design
id: id of subject
status : 1-death, 2-response for second randomization, 0-censoring
A0 : treament at first randomization
A1 : treament at second randomization
At.f : treament given at record (A0 or A1)
TR : time of response 
sex : 0-males, 1-females 
consent: 1 if agrees to 2nd randomization, censored if not
R: 1 if response 
trt1: A0
trt2: A1
}
\source{
https://github.com/ycchao/code_Joint_model_SMART
}
\description{
Data from CALGB 8923
}
\examples{
data(calgb8923)
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/riskRegression.R
\name{predictRisk}
\alias{predictRisk}
\title{Risk predictions to work with riskRegression package}
\usage{
predictRisk(object, ...)
}
\arguments{
\item{object}{phreg/binreg/cifreg object}

\item{...}{additional arguments to lower level functions}
}
\description{
Risk predictions to work with riskRegression package
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/riskRegression.R
\name{predictRisk.binreg}
\alias{predictRisk.binreg}
\alias{predictRisk.phreg}
\alias{predictRisk.cifreg}
\alias{predictRisk.cifregFG}
\alias{predictRisk.recreg}
\title{Risk predictions to work with riskRegression package}
\usage{
\method{predictRisk}{binreg}(object, newdata, cause, times = NULL, ...)
}
\arguments{
\item{object}{phreg/binreg/cifreg object}

\item{newdata}{data.frame on which to make new predictions}

\item{cause}{cause (cif) to predict}

\item{times}{times for predictions}

\item{...}{additional arguments to lower level functions}
}
\description{
Risk predictions to work with riskRegression package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recreg.R
\name{simGLcox}
\alias{simGLcox}
\title{Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure}
\usage{
simGLcox(
  n,
  base1,
  drcumhaz,
  var.z = 0,
  r1 = NULL,
  rd = NULL,
  rc = NULL,
  fz = NULL,
  fdz = NULL,
  model = c("twostage", "frailty", "shared"),
  type = NULL,
  share = 1,
  cens = NULL,
  nmin = 100,
  nmax = 1000
)
}
\arguments{
\item{n}{number of id's}

\item{base1}{baseline for cox/ghosh-lin models}

\item{drcumhaz}{baseline for terminal event}

\item{var.z}{variance of gamma frailty}

\item{r1}{relative risk term for baseline}

\item{rd}{relative risk term for terminal event}

\item{rc}{relative risk term for censorings}

\item{fz}{possible transformation (function) of frailty term}

\item{fdz}{possible transformation (function) of frailty term for death}

\item{model}{twostage, frailty, shared (partly shared two-stage model)}

\item{type}{type of simulation, default is decided based on model}

\item{share}{to fit patly shared random effects model}

\item{cens}{censoring rate for exponential censoring}

\item{nmin}{default 100, at least nmin or number of rows of the two-baselines max(nmin,nrow(base1),nrow(drcumhaz)) points in time-grid from 0 to maximum time for base1}

\item{nmax}{default 1000, at most nmax points in time-grid}
}
\description{
Simulation of two-stage recurrent events data based on Cox/Cox or Cox/Ghosh-Lin structure. type=3 will generate
Cox/Cox twostage mode, type=2 will generate Ghosh-Lin/Cox model. 
If the variance is var.z=0, then generates data without any dependence or frailty. If model="twostage" then default is to generate data from Ghosh-Lin/Cox model, and
if type=3 then will generate data with marginal Cox models (Cox/Cox). 
Simulation based on linear aproximation of hazard for two-stage models based on grid on time-scale. Must be sufficientyly fine.
}
\details{
Must specify baselines of recurrent events and terminal event and possible covariate effects.
}
\references{
Scheike (2025), Two-stage recurrent events random effects models, LIDA, to appear
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aalenfrailty.R
\name{aalenfrailty}
\alias{aalenfrailty}
\title{Aalen frailty model}
\usage{
aalenfrailty(time, status, X, id, theta, B = NULL, ...)
}
\arguments{
\item{time}{Time variable}

\item{status}{Status variable (0,1)}

\item{X}{Covariate design matrix}

\item{id}{cluster variable}

\item{theta}{list of thetas (returns score evaluated here), or
starting point for optimization (defaults to magic number 0.1)}

\item{B}{(optional) Cumulative coefficients (update theta by fixing B)}

\item{...}{Additional arguments to lower level functions}
}
\value{
Parameter estimates
}
\description{
Additive hazards model with (gamma) frailty
}
\details{
Aalen frailty model
}
\examples{
library("timereg")
dd <- simAalenFrailty(5000)
f <- ~1##+x
X <- model.matrix(f,dd) ## design matrix for non-parametric terms
system.time(out<-timereg::aalen(update(f,Surv(time,status)~.),dd,n.sim=0,robust=0))
dix <- which(dd$status==1)
t1 <- system.time(bb <- .Call("Bhat",as.integer(dd$status),
                              X,0.2,as.integer(dd$id),NULL,NULL,
                              PACKAGE="mets"))
spec <- 1
##plot(out,spec=spec)
## plot(dd$time[dix],bb$B2[,spec],col="red",type="s",
##      ylim=c(0,max(dd$time)*c(beta0,beta)[spec]))
## abline(a=0,b=c(beta0,beta)[spec])
##'

\dontrun{
thetas <- seq(0.1,2,length.out=10)
Us <- unlist(aalenfrailty(dd$time,dd$status,X,dd$id,as.list(thetas)))
##plot(thetas,Us,type="l",ylim=c(-.5,1)); abline(h=0,lty=2); abline(v=theta,lty=2)
op <- aalenfrailty(dd$time,dd$status,X,dd$id)
op
}
}
\author{
Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jumptimes.R
\name{jumptimes}
\alias{jumptimes}
\title{Extract Event (Jump) Times}
\usage{
jumptimes(
  time,
  status = TRUE,
  id,
  cause,
  sample,
  sample.all = TRUE,
  strata = NULL,
  num = NULL,
  ...
)
}
\arguments{
\item{time}{vector of event/censoring times.}

\item{status}{vector of status indicators (default TRUE = event).}

\item{id}{optional cluster identifier for concordant pair times.}

\item{cause}{optional cause value to select.}

\item{sample}{optional maximum number of time points to return.}

\item{sample.all}{logical; if TRUE and sampling, include remaining times.}

\item{strata}{not used.}

\item{num}{optional within-cluster numbering variable.}

\item{...}{additional arguments.}
}
\value{
Sorted vector of event times.
}
\description{
Extracts unique event times from survival data, optionally restricting
to concordant pairs or specific causes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{phreg_IPTW}
\alias{phreg_IPTW}
\title{IPTW Cox Regression (Inverse Probability of Treatment Weighted)}
\usage{
phreg_IPTW(
  formula,
  data,
  treat.model = NULL,
  treat.var = NULL,
  weights = NULL,
  estpr = 1,
  pi0 = 0.5,
  se.cluster = NULL,
  ...
)
}
\arguments{
\item{formula}{Formula for \code{phreg}.}

\item{data}{Data frame for risk averaging.}

\item{treat.model}{Propensity score model (binary or multinomial).}

\item{treat.var}{A 1/0 variable indicating when treatment is given (for time-dependent weights).}

\item{weights}{Optional weights to multiply with the IPTW weights.}

\item{estpr}{(=1, default) to estimate propensity scores and include their uncertainty in the influence function.}

\item{pi0}{Fixed simple weights (if \code{estpr=0}).}

\item{se.cluster}{To compute GEE-type standard errors when additional cluster structure is present.}

\item{...}{Arguments for \code{phreg} call.}
}
\value{
An object of class \code{"phreg"} with additional IPTW components:
\item{IID}{Influence functions including propensity score uncertainty.}
\item{iptw}{IPTW weights used.}
\item{naive.var}{Naive variance ignoring propensity score uncertainty.}
}
\description{
Fits a Cox model with treatment weights \deqn{w(A) = \sum_a I(A=a)/\pi(a|X)}, where
\deqn{\pi(a|X) = P(A=a|X)}.
}
\details{
Standard errors are computed via influence functions that are returned as the IID argument.
Propensity scores are fitted using either logistic regression (\code{glm}) or the multinomial
model (\code{mlogit}) when there are more than two treatment categories.

The treatment variable must be a factor and is identified on the RHS of the \code{treat.model}.
Recurrent events can be considered with a start-stop structure, requiring \code{cluster(id)}.
Robust standard errors are computed in all cases.

Time-dependent propensity score weights can be computed when \code{treat.var} is used. This weight
be 1 at the time of first (A_0) and 2nd treatment (A_1), then uses weights \deqn{w_0(A_0) * w_1(A_1)^{t>T_r}} where \deqn{T_r} is
time of 2nd randomization. The weights are constructed using a \code{glm} or \code{mlogit} model based on the data where 
\code{treat.var=1}. The propensity score can be constructed for any number of treatments in a similar manner.
}
\examples{
data <- mets:::simLT(0.7, 100, beta = 0.3, betac = 0, ce = 1, betao = 0.3)
dfactor(data) <- Z.f ~ Z
out <- phreg_IPTW(Surv(time, status) ~ Z.f, data = data, treat.model = Z.f ~ X)
summary(out)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discrete-survival-haplo.R
\name{interval.logitsurv.discrete}
\alias{interval.logitsurv.discrete}
\alias{Interval}
\alias{dInterval}
\alias{simlogitSurvd}
\alias{predictlogitSurvd}
\alias{cumODDS}
\title{Discrete time to event interval censored data}
\usage{
interval.logitsurv.discrete(
  formula,
  data,
  beta = NULL,
  no.opt = FALSE,
  method = "NR",
  stderr = TRUE,
  weights = NULL,
  offsets = NULL,
  exp.link = 1,
  increment = 1,
  ...
)
}
\arguments{
\item{formula}{formula}

\item{data}{data}

\item{beta}{starting values}

\item{no.opt}{optimization TRUE/FALSE}

\item{method}{NR, nlm}

\item{stderr}{to return only estimate}

\item{weights}{weights following id for GLM}

\item{offsets}{following id  for GLM}

\item{exp.link}{parametrize increments exp(alpha) > 0}

\item{increment}{using increments dG(t)=exp(alpha) as parameters}

\item{...}{Additional arguments to lower level funtions lava::NR  optimizer or nlm}
}
\description{
\deqn{
   logit(P(T >t | x)) = log(G(t)) + x \beta
}
\deqn{
   P(T >t | x) =  \frac{1}{1 + G(t) exp( x \beta) }
}
}
\details{
This is thus also the cumulative odds model, since 
\deqn{
   P(T \leq t | x) =  \frac{G(t) \exp(x \beta) }{1 + G(t) exp( x \beta) }
}

The baseline \eqn{G(t)} is written as \eqn{cumsum(exp(\alpha))} and this is not the standard
parametrization that takes log of \eqn{G(t)} as the parameters.

Input are intervals given by ]t_l,t_r] where t_r can be infinity for right-censored intervals 
When truly discrete ]0,1] will be an observation at 1, and  ]j,j+1] will be an observation at j+1

Likelihood is maximized:
\deqn{
 \prod  P(T_i >t_{il} | x) - P(T_i> t_{ir}| x) 
}
}
\examples{
data(ttpd) 
dtable(ttpd,~entry+time2)
out <- interval.logitsurv.discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
summary(out)

pred <- predictlogitSurvd(out,se=FALSE)
plotSurvd(pred)

}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{test_casewise}
\alias{test_casewise}
\title{Test for Independence Using Casewise Concordance}
\usage{
test_casewise(conc, marg, test = "no-test", p = 0.01)
}
\arguments{
\item{conc}{An object containing concordance estimates (e.g., from \code{bicomprisk}). 
Must contain \code{time}, \code{P1}, \code{se.P1}, and ideally \code{P1.iid}.}

\item{marg}{An object containing marginal cumulative incidence estimates (e.g., from \code{comp.risk} or \code{prodlim}). 
Must contain \code{time}, \code{P1}, \code{se.P1}, and ideally \code{P1.iid}.}

\item{test}{Type of test: \code{"case"} (test on casewise concordance) or \code{"conc"} (test on concordance). 
Default is \code{"no-test"}.}

\item{p}{Threshold for checking that marginal probability is greater than this value at some point 
(default 0.01). Used to avoid division by near-zero probabilities.}
}
\value{
An object of class \code{"casewise"} containing:
  \item{casewise}{Matrix with time, casewise concordance, and standard errors.}
  \item{marg}{Matrix with time, marginal CIF, and standard errors.}
  \item{conc}{Matrix with time, concordance, and standard errors.}
  \item{casewise.iid}{Influence function decomposition for casewise concordance.}
  \item{test}{Test results (if \code{test} is specified): Pepe-Mori type test statistics, 
    standard errors, z-values, and p-values.}
  \item{mintime, maxtime}{Time range used for the analysis.}
  \item{same.cluster}{Logical indicating if clusters were assumed identical.}
  \item{testtype}{Type of test performed.}
}
\description{
Estimates the casewise concordance based on concordance and marginal estimates, 
and performs tests for the independence assumption. This is particularly useful 
in twin studies to assess genetic vs. environmental contributions to disease risk.
}
\details{
The casewise concordance is defined as:
\deqn{ C(t) = \frac{P(T_1 \leq t, T_2 \leq t)}{P(T_1 \leq t)} }
where the numerator is the joint probability of both twins having the event by time \eqn{t}, 
and the denominator is the marginal probability.

The function supports two types of tests:
\itemize{
  \item \code{"case"}: Tests on the casewise concordance scale (difference between observed and expected under independence).
  \item \code{"conc"}: Tests on the concordance scale (difference between observed concordance and squared marginal).
}

Standard errors are computed using cluster-based conservative estimates or influence 
functions (IID) if available from the input objects.
}
\examples{
\donttest{ ## Reduce Ex.Timings
library("timereg")
data("prt",package="mets");
prt <- force_same_cens(prt,cause="status")

prt <- prt[which(prt$id \%in\% sample(unique(prt$id),7500)),]
### marginal cumulative incidence of prostate cancer
times <- seq(60,100,by=2)
outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,cause=2,times=times)

cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
cifdz <- predict(outm,X=1,uniform=0,resample.iid=1)

### concordance for MZ and DZ twins
cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),
                 data=prt,cause=c(2,2))
cdz <- cc$model$"DZ"
cmz <- cc$model$"MZ"

### To compute casewise cluster argument must be passed on,
###  here with a max of 100 to limit comp-time
outm <- timereg::comp.risk(Event(time,status)~+1,data=prt,
                 cause=2,times=times,max.clust=100)
cifmz <- predict(outm,X=1,uniform=0,resample.iid=1)
cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,
                cause=c(2,2),se.clusters=outm$clusters)
cdz <- cc$model$"DZ"
cmz <- cc$model$"MZ"

cdz <- test_casewise(cdz,cifmz,test="case") ## test based on casewise
cmz <- test_casewise(cmz,cifmz,test="conc") ## based on concordance

plot(cmz,ylim=c(0,0.7),xlim=c(60,100))
par(new=TRUE)
plot(cdz,ylim=c(0,0.7),xlim=c(60,100))

}
}
\references{
Scheike, T. H. (2024). Casewise concordance estimation and testing. mets package documentation.
}
\seealso{
\code{\link{bicomprisk}}, \code{\link{casewise}}, \code{\link{test_conc}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{package}
\name{mets-package}
\alias{mets-package}
\alias{mets}
\title{mets: Analysis of Multivariate Event Times}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Implementation of various statistical models for multivariate event history data \doi{10.1007/s10985-013-9244-x}. Including multivariate cumulative incidence models \doi{10.1002/sim.6016}, and bivariate random effects probit models (Liability models) \doi{10.1016/j.csda.2015.01.014}. Modern methods for survival analysis, including regression modelling (Cox, Fine-Gray, Ghosh-Lin, Binomial regression) with fast computation of influence functions.

Implementation of various statistical models for multivariate
event history data. Including multivariate cumulative incidence models,
and bivariate random effects probit models (Liability models)
}
\examples{

## To appear

}
\seealso{
Useful links:
\itemize{
  \item \url{https://kkholst.github.io/mets/}
  \item Report bugs at \url{https://github.com/kkholst/mets/issues}
}

}
\author{
\strong{Maintainer}: Klaus K. Holst \email{klaus@holst.it}

Authors:
\itemize{
  \item Thomas Scheike
}


Klaus K. Holst and Thomas Scheike
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg_weibull.R
\name{rweibullcox}
\alias{rweibullcox}
\title{Simulate observations from a Weibull distribution}
\usage{
rweibullcox(n, rate, shape)
}
\arguments{
\item{n}{(integer) number of observations}

\item{rate}{(numeric) rate parameter (can be a vector of size n)}

\item{shape}{(numeric) shape parameter (can be a vector of size n)}
}
\description{
Simulate observations from the model with cumulative hazard
  given by \deqn{\Lambda(t) = \lambda\cdot t^s} where \eqn{\lambda} is the
  \emph{rate parameter} and \eqn{s} is the \emph{shape parameter}.
}
\details{
[stats::rweibull()] uses a different parametrization with
  cumulative hazard given by
\deqn{H(t) = (t/b)^a,}
i.e., the shape is the same \eqn{a:=s} but the scale paramter \eqn{b}
is related to rate paramter \eqn{r} by
\deqn{r := b^{-a}}
}
\seealso{
[stats::rweibull()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_twin.R
\name{plot_twin}
\alias{plot_twin}
\title{Scatter plot function}
\usage{
plot_twin(
  data,
  marginal.args = list(),
  kernsmooth.args = list(),
  xlab,
  ylab,
  col = "black",
  col2 = "lightblue",
  alpha = 0.3,
  grid = TRUE,
  side.plot = TRUE,
  ...
)
}
\arguments{
\item{data}{bivariate data to plot (data.frame or matrix with 2 columns)}

\item{marginal.args}{argumemts to marginal estimator (`density` continuous
data, `barplot` for categorical )}

\item{kernsmooth.args}{arguments to 2d-kernel smoother}

\item{xlab}{x-axis label}

\item{ylab}{y-axis label}

\item{col}{color of points}

\item{col2}{color of contour / density plot}

\item{alpha}{transparency level of points}

\item{grid}{should grid be added to the plot}

\item{side.plot}{If TRUE subplots of the marginal distributions are added to
the plot}

\item{...}{arguments to lower level plot functions}
}
\description{
Scatterplot with contours of the (kernel) estimated density
}
\examples{
data("twinbmi", package="mets")
twinwide <- fast.reshape(twinbmi, id="tvparnr",varying=c("bmi"))
datamz <- log(subset(twinwide, zyg=="MZ")[,c("bmi1","bmi2")])

# continuous variables
plot_twin(datamz)

# categorical variables
datamz2 <- datamz
datamz2[, 1] <- cut(datamz[, 1], 4)
datamz2[, 2] <- cut(datamz[, 2], 4)
plot_twin(datamz2, color = TRUE)

# survival variables
cens1 <- rbinom(nrow(datamz), 1, 0.5)
cens2 <- rbinom(nrow(datamz), 1, 0.5)
datamz2[, 1] <- Event(datamz[, 1], cens1)
datamz2[, 2] <- suppressWarnings(Event(datamz[, 2], cens2))
plot_twin(datamz2)

rm(datamz, datamz2, cens1, cens2)
}
\author{
Klaus Kähler Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{strata-numeric}
\alias{strata-numeric}
\alias{sumstrata}
\alias{cumsumstrata}
\alias{revcumsumstrata}
\alias{revcumsum}
\alias{matdoubleindex}
\alias{mdi}
\title{Stratified Cumulative and Summary Operations}
\usage{
sumstrata(x, strata, nstrata)

cumsumstrata(x, strata, nstrata)

revcumsumstrata(x, strata, nstrata)

revcumsum(x)

matdoubleindex(x, rows, cols, xvec = NULL)

mdi(x, ...)
}
\arguments{
\item{x}{numeric vector (or matrix for \code{matdoubleindex}).}

\item{strata}{integer vector of strata indices (0-based).}

\item{nstrata}{number of distinct strata.}

\item{rows}{row indices for \code{matdoubleindex}.}

\item{cols}{column indices for \code{matdoubleindex}.}

\item{xvec}{optional values to assign at indexed positions.}

\item{...}{additional arguments (for \code{mdi}).}
}
\value{
Numeric vector of the same length as \code{x} (or modified matrix).
}
\description{
Low-level helper functions for computing sums, cumulative sums, and
reverse cumulative sums within strata groups, as well as matrix
double-indexing.
}
\author{
Klaus K. Holst, Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof-phreg.R
\name{gofZ_phreg}
\alias{gofZ_phreg}
\alias{cumContr}
\title{Goodness-of-Fit for Cox Covariates (Linearity)}
\usage{
gofZ_phreg(
  formula,
  data,
  vars = NULL,
  offset = NULL,
  weights = NULL,
  breaks = 50,
  equi = FALSE,
  n.sim = 1000,
  silent = 1,
  ...
)
}
\arguments{
\item{formula}{Formula for the Cox regression.}

\item{data}{Data frame.}

\item{vars}{Vector of variable names to test. If NULL, automatically detects continuous 
covariates with more than 2 levels.}

\item{offset}{Offset vector.}

\item{weights}{Weights vector.}

\item{breaks}{Number of break points for the grid (default 50).}

\item{equi}{Logical; if TRUE, uses equidistant breaks; if FALSE, uses quantiles.}

\item{n.sim}{Number of simulations (default 1000).}

\item{silent}{Logical; suppresses timing estimates.}

\item{...}{Additional arguments passed to \code{gofM_phreg}.}
}
\value{
An object of class \code{"gof.phreg"} with type "Zmodelmatrix" containing:
  \item{res}{Matrix of p-values for each tested variable.}
  \item{Zres}{List of \code{gof.phreg} objects, one for each variable.}
  \item{type}{Type of test ("Zmodelmatrix").}
}
\description{
Tests the functional form of continuous covariates in a Cox PH model to check for 
linearity. It computes cumulative residuals evaluated at a grid of covariate values \eqn{z}.
}
\details{
The test statistic is:
\deqn{ U(z, \tau) = \int_0^\tau K(z)^T d \hat M }
where \eqn{K(z)} is a design matrix based on indicator functions \eqn{I(Z_i \leq z_l)} 
for a grid of points \eqn{z_l}.

The p-value is valid but depends on the chosen grid. As the number of break points increases, 
this test converges to the original Lin, Wei, and Ying test for linearity.
}
\examples{
data(TRACE)
set.seed(1)
TRACEsam <- blocksample(TRACE, idvar="id", replace=FALSE, 100)

## Test linearity of continuous covariates
\donttest{ ## Reduce Ex.Timings
m1 <- gofZ_phreg(Surv(time,status==9)~strata(vf)+chf+wmi+age, data=TRACEsam)
summary(m1) 
plot(m1, type="z")
}
}
\seealso{
\code{\link{gofM_phreg}}, \code{\link{cumContr}}
}
\author{
Thomas Scheike and Klaus K. Holst
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brier_binreg.R
\name{estimate.binregCV}
\alias{estimate.binregCV}
\title{Joint estimate across CV models for a binregCV object}
\usage{
\method{estimate}{binregCV}(x, f = NULL, ...)
}
\arguments{
\item{x}{A `binregCV` object.}

\item{f}{Optional transformation function passed to `lava::estimate`.}

\item{...}{Not currently used.}
}
\description{
Joint estimate across CV models for a binregCV object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{grouptable}
\alias{grouptable}
\title{Create Group Contingency Table from Clustered Data}
\usage{
grouptable(
  data,
  id,
  group,
  var,
  lower = TRUE,
  labels,
  order,
  group.labels,
  group.order,
  combine = " & ",
  ...
)
}
\arguments{
\item{data}{a data.frame.}

\item{id}{name of the cluster/pair identifier column.}

\item{group}{name of the grouping variable (e.g., zygosity).}

\item{var}{name of the outcome variable to tabulate.}

\item{lower}{logical; if TRUE, fold upper triangle into lower.}

\item{labels}{optional labels for levels of \code{var}.}

\item{order}{optional ordering of factor levels.}

\item{group.labels}{optional labels for the groups.}

\item{group.order}{optional ordering of groups.}

\item{combine}{separator for combining two groups (default \code{" & "}).}

\item{...}{additional arguments.}
}
\value{
A table or list of tables.
}
\description{
Creates a contingency table by group from paired/clustered data,
optionally combining lower and upper triangles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{prob.exceed.recurrent}
\alias{prob.exceed.recurrent}
\alias{summaryRecurrentobject}
\alias{summaryTimeobject}
\title{Estimation of probability of more that k events for recurrent events process}
\usage{
prob.exceed.recurrent(
  formula,
  data,
  cause = 1,
  death.code = 2,
  cens.code = 0,
  exceed = NULL,
  marks = NULL,
  all.cifs = FALSE,
  return.data = FALSE,
  conf.type = c("log", "plain"),
  level = 0.95,
  ...
)
}
\arguments{
\item{formula}{formula}

\item{data}{data-frame}

\item{cause}{of interest}

\item{death.code}{for status}

\item{cens.code}{censoring codes}

\item{exceed}{values (if not given then all observed values)}

\item{marks}{may be give for jump-times and then exceed values needs to be specified}

\item{all.cifs}{if true then returns list of all fitted objects in cif.exceed}

\item{return.data}{if true then returns list of data for fitting the different excess thresholds}

\item{conf.type}{type of confidence interval c("log","plain")}

\item{level}{of confidence intervals default is 0.95}

\item{...}{Additional arguments to lower level funtions}
}
\description{
Estimation of probability of more that k events for recurrent events process
where there is terminal event, based on this also estimate of variance of recurrent events. The estimator is based on cumulative incidence of exceeding "k" events.
In contrast the probability of exceeding k events can also be computed as a counting process integral.
}
\examples{
library(mets)
data(hfactioncpx12)
dtable(hfactioncpx12,~status)

oo <- prob.exceed.recurrent(Event(entry,time,status)~cluster(id),
        hfactioncpx12,cause=1,death.code=2)
plot(oo)
summary(oo,times=c(1,2,5))

}
\references{
Scheike, Eriksson, Tribler (2019), The mean, variance and correlation for bivariate 
                                       recurrent events with a terminal event, JRSS-C
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{casewise_bin}
\alias{casewise_bin}
\title{Casewise Concordance from Concordant/Discordant Counts}
\usage{
casewise_bin(nc, nd)
}
\arguments{
\item{nc}{number of concordant pairs.}

\item{nd}{number of discordant pairs.}
}
\value{
A list with \code{p.casewise} (estimated probability) and
  \code{ci.casewise} (confidence interval).
}
\description{
Computes casewise concordance probability and confidence interval from
counts of concordant and discordant pairs using a binomial GLM.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/riskRegression.R
\name{predictRisk.phreg}
\alias{predictRisk.phreg}
\alias{predictRisk.cifreg}
\alias{predictRisk.binreg}
\title{Risk predictions to work with riskRegression package}
\usage{
predictRisk.phreg(object, newdata, times = NULL)
}
\arguments{
\item{object}{phreg/binreg/cifreg object}

\item{newdata}{newdata}

\item{times}{times for predictions}
}
\description{
Risk predictions to work with riskRegression package
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_phreg}
\alias{sim_phreg}
\title{Simulation of Output from Cox Model}
\usage{
sim_phreg(
  cox,
  n,
  data = NULL,
  Z = NULL,
  rr = NULL,
  strata = NULL,
  entry = NULL,
  extend = TRUE,
  cens = NULL,
  rrc = NULL,
  ...
)
}
\arguments{
\item{cox}{Output from \code{coxph} or \code{phreg} model fitting.}

\item{n}{Number of simulations.}

\item{data}{Data frame to extract covariates for simulations (draws from observed covariates).}

\item{Z}{Design matrix instead of data.}

\item{rr}{Vector of relative risks for Cox model.}

\item{strata}{Vector of strata.}

\item{entry}{Delayed entry variable for simulation.}

\item{extend}{Extend possible stratified baselines to largest endpoint.}

\item{cens}{Censoring specification (matrix = cumulative hazard, scalar = rate).}

\item{rrc}{Relative risks for Cox-type censoring.}

\item{...}{Arguments for \code{rchaz} (e.g., entry-time).}
}
\value{
Data frame with simulated event times, status, and covariates.
}
\description{
Simulates data that looks like fit from a Cox model. Automatically censors data 
for the highest value of the event times by using cumulative hazard.
}
\examples{
data(sTRACE)
nsim <- 100
coxs <-  phreg(Surv(time,status==9)~strata(chf)+vf+wmi,data=sTRACE)
set.seed(100)
sim3 <- sim_phreg(coxs,nsim,data=sTRACE)
head(sim3)
cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim3)
cbind(coxs$coef,cc$coef)
plot(coxs,col=1); plot(cc,add=TRUE,col=2)

Z <- sim3[,c("vf","chf","wmi")]
strata <- sim3[,c("chf")]
rr <- exp(as.matrix(Z[,-2]) \%*\% coef(coxs))
sim4 <- sim_phreg(coxs,nsim,data=NULL,rr=rr,strata=strata)
sim4 <- cbind(sim4,Z)
cc <-   phreg(Surv(time,status)~strata(chf)+vf+wmi,data=sim4)
cbind(coxs$coef,cc$coef)
plot(coxs,col=1); plot(cc,add=TRUE,col=2)

}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{logitSurv}
\alias{logitSurv}
\title{Proportional Odds Survival Model}
\usage{
logitSurv(formula, data, offset = NULL, weights = NULL, ...)
}
\arguments{
\item{formula}{Formula with 'Surv' outcome (similar to \code{coxph}).}

\item{data}{Data frame.}

\item{offset}{Offsets for \eqn{\exp(x \beta)} terms.}

\item{weights}{Weights for score equations.}

\item{...}{Additional arguments passed to lower-level functions.}
}
\value{
An object of class \code{"phreg"} with \code{propodds=1}.
}
\description{
Fits a semiparametric proportional odds model where:
\deqn{ \mbox{logit}(S(t|x)) = \log(\Lambda(t)) + x \beta }
Thus, covariate effects represent the odds ratio (OR) of survival.
}
\details{
This is equivalent to using a hazards model:
\deqn{ Z \lambda(t) \exp(x \beta) }
where \eqn{Z} is gamma distributed with mean and variance 1.
}
\examples{
data(TRACE)
dcut(TRACE) <- ~.
out1 <- logitSurv(Surv(time, status == 9) ~ vf + chf + strata(wmicat.4), data = TRACE)
summary(out1)
gof(out1)
plot(out1)
}
\references{
Eriksson, Frank, Li, Jianing, Scheike, Thomas, and Zhang, Mei-Jie (2015).
"The proportional odds cumulative incidence model for competing risks."
\emph{Biometrics}, 71(3), 687--695.
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/restricted.mean.R
\name{ratioATE}
\alias{ratioATE}
\title{Ratio of Average Treatment Effects}
\usage{
ratioATE(rmtl, rmtl1, h = NULL, null = 1)
}
\arguments{
\item{rmtl}{Object containing the ATE for the total RMTL (from \code{resmeanATE}).}

\item{rmtl1}{Object containing the ATE for the specific cause RMTL (from \code{resmeanATE}).}

\item{h}{Transformation function (e.g., \code{log}) applied to the ratio. Default is identity.}

\item{null}{Value under the null hypothesis for the ratio (default 1).}
}
\value{
A list containing:
  \item{ratioG}{Ratio based on the simple IPCW estimator.}
  \item{ratioDR}{Ratio based on the double robust estimator.}
}
\description{
Computes the ratio of two Average Treatment Effects (ATEs), typically comparing the 
ATE for a specific cause (e.g., RMTL due to cause 1) to the ATE for the total RMTL.
}
\details{
The function transforms the estimates (e.g., using log) to compute the ratio and 
its standard error using the delta method and the joint influence functions.
}
\examples{
data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, outcome="rmtl")
out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40, outcome="rmtl")
ratioATE(out, out1, h=log)
}
\seealso{
\code{\link{resmeanATE}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recurrent.marginal.R
\name{summaryTimeobject}
\alias{summaryTimeobject}
\title{Summarize a Time-Varying Estimate with Confidence Bands}
\usage{
summaryTimeobject(
  mutimes,
  mu,
  se.mu = NULL,
  times = NULL,
  type = "log",
  level = 0.95,
  ...
)
}
\arguments{
\item{mutimes}{vector of estimation time points.}

\item{mu}{vector or matrix of estimates.}

\item{se.mu}{vector or matrix of standard errors (optional).}

\item{times}{time points at which to evaluate (default: \code{mutimes}).}

\item{type}{confidence interval type: \code{"log"} or \code{"plain"}.}

\item{level}{confidence level (default 0.95).}

\item{...}{additional arguments.}
}
\value{
A data.frame with columns: times, mean, se-mean, CI-2.5\%, CI-97.5\%.
}
\description{
Extracts estimates at specified time points with confidence intervals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{IC.phreg}
\alias{IC.phreg}
\title{Influence Functions for phreg objects}
\usage{
\method{IC}{phreg}(x, type = "robust", all = FALSE, time = NULL, baseline = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{"phreg"}.}

\item{type}{Type of influence function: \code{"robust"} (default) or \code{"martingale"}.}

\item{all}{Logical; if \code{TRUE}, returns both beta and baseline influence functions.}

\item{time}{Time point for baseline influence function (required if baseline is requested).}

\item{baseline}{Arguments for baseline estimation.}

\item{...}{Additional arguments.}
}
\value{
A matrix of influence functions. If \code{all=TRUE}, columns correspond to
regression coefficients and baseline cumulative hazard. Attributes include \code{coef}
and \code{time}.
}
\description{
Computes the influence functions (IID decomposition) for the regression coefficients
and/or the baseline cumulative hazard at a specific time point.
}
\seealso{
\code{\link{phreg}}, \code{\link{iidBaseline}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{iid}
\alias{IC}
\alias{twostage}
\alias{estimate}
\alias{gof}
\alias{strata}
\alias{Surv}
\alias{cluster}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{lava}{\code{\link[lava:estimate]{estimate()}}, \code{\link[lava:gof]{gof()}}, \code{\link[lava:IC]{IC()}}, \code{\link[lava:iid]{iid()}}, \code{\link[lava:twostage]{twostage()}}}

  \item{survival}{\code{\link[survival:cluster]{cluster()}}, \code{\link[survival:strata]{strata()}}, \code{\link[survival:Surv]{Surv()}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval-cumoddsreg.r
\name{interval_logitsurv_discrete}
\alias{interval_logitsurv_discrete}
\alias{Interval}
\alias{dInterval}
\alias{predictlogitSurvd}
\alias{predictSurvd}
\alias{plotSurvd}
\title{Discrete Time-to-Event Analysis with Interval Censoring}
\usage{
interval_logitsurv_discrete(
  formula,
  data,
  beta = NULL,
  no.opt = FALSE,
  method = "NR",
  stderr = TRUE,
  weights = NULL,
  offsets = NULL,
  exp.link = 1,
  increment = 1,
  ...
)
}
\arguments{
\item{formula}{Formula with an \code{Interval} object (e.g., \code{Interval(entry, time)}) 
on the left-hand side and covariates on the right. Can include \code{cluster()} for 
correlated data.}

\item{data}{Data frame containing the variables in the formula.}

\item{beta}{Starting values for the optimization (vector of length \eqn{p + k}, where 
\eqn{p} is the number of covariates and \eqn{k} is the number of time intervals).}

\item{no.opt}{Logical; if TRUE, skips optimization and returns estimates based on 
the provided \code{beta} (useful for initialization).}

\item{method}{Optimization method: \code{"NR"} (Newton-Raphson, default) or \code{"nlm"}.}

\item{stderr}{Logical; if FALSE, returns only the coefficient estimates.}

\item{weights}{Observation weights (follows ID).}

\item{offsets}{Offsets (follows ID).}

\item{exp.link}{Logical; if TRUE, parameterizes increments as \eqn{\exp(\alpha) > 0}.}

\item{increment}{Logical; if TRUE, uses increments \eqn{dG(t) = \exp(\alpha)} as parameters.}

\item{...}{Additional arguments passed to the optimizer (\code{lava::NR} or \code{nlm}).}
}
\value{
An object of class \code{"cumoddsreg"} containing:
  \item{coef}{Estimated coefficients (baseline time effects and covariate effects).}
  \item{se.coef}{Standard errors of the coefficients.}
  \item{var}{Variance-covariance matrix.}
  \item{iid}{Influence function (IID) decomposition for robust variance estimation.}
  \item{ntimes}{Number of distinct time intervals.}
  \item{utimes}{Unique time points.}
  \item{ploglik}{Log-likelihood at convergence.}
  \item{gradient, hessian}{Optimization results.}
  \item{call}{Original function call.}
}
\description{
Fits a cumulative odds model for discrete time-to-event data, handling interval 
censoring where the event time is known only to lie within an interval \eqn{(t_l, t_r]}.
The model assumes:
\deqn{ \text{logit}(P(T \leq t | x)) = \log(G(t)) + x \beta }
where \eqn{G(t)} is the baseline cumulative odds function and \eqn{\beta} are the 
regression coefficients. This is equivalent to:
\deqn{ P(T \leq t | x) = \frac{G(t) \exp(x \beta)}{1 + G(t) \exp(x \beta)} }
}
\details{
The baseline \eqn{G(t)} is parameterized as the cumulative sum of exponentials 
(\eqn{G(t) = \sum \exp(\alpha)}), ensuring positivity. The regression coefficients 
describe the log-odds of the event occurring by time \eqn{t}.

The likelihood is maximized over the observed intervals:
\deqn{ L = \prod_i [ P(T_i > t_{il} | x_i) - P(T_i > t_{ir} | x_i) ] }
where \eqn{t_{il}} and \eqn{t_{ir}} are the left and right endpoints of the interval 
for subject \eqn{i}. Right-censored intervals have \eqn{t_{ir} = \infty}.
}
\examples{
data(ttpd) 
dtable(ttpd,~entry+time2)

out <- interval_logitsurv_discrete(Interval(entry,time2)~X1+X2+X3+X4,ttpd)
summary(out)
head(iid(out)) 

pred <- predictlogitSurvd(out,se=FALSE)
plotSurvd(pred)

ttpd <- dfactor(ttpd,fentry~entry)
out <- cumoddsreg(fentry~X1+X2+X3+X4,ttpd)
summary(out)
}
\references{
Scheike, T. H. (2024). Discrete time survival analysis with interval censoring. mets package documentation.
}
\seealso{
\code{\link{cumoddsreg}}, \code{\link{predictlogitSurvd}}, \code{\link{simlogitSurvd}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval-cumoddsreg.r
\name{cumoddsreg}
\alias{cumoddsreg}
\title{Cumulative Odds Regression for Discrete Time Data}
\usage{
cumoddsreg(formula, data, ...)
}
\arguments{
\item{formula}{Formula with a factor response on the left-hand side (representing 
the event time) and covariates on the right.}

\item{data}{Data frame.}

\item{...}{Arguments passed to \code{interval_logitsurv_discrete}.}
}
\value{
An object of class \code{"cumoddsreg"} with the same structure as 
  \code{interval_logitsurv_discrete}.
}
\description{
A wrapper function for \code{interval_logitsurv_discrete} that simplifies the 
interface for discrete time-to-event data where the event time is observed exactly 
or as a factor level. It converts a factor response into an interval-censored format 
internally.
}
\seealso{
\code{\link{interval_logitsurv_discrete}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{casewise}
\alias{casewise}
\title{Estimate Casewise Concordance from prodlim Objects}
\usage{
casewise(conc, marg, cause.marg)
}
\arguments{
\item{conc}{Concordance object from \code{prodlim} (output of \code{bicomprisk} with \code{prodlim=TRUE}).}

\item{marg}{Marginal cumulative incidence object from \code{prodlim} (output of \code{prodlim}).}

\item{cause.marg}{Specifies which cause should be used for the marginal CIF based on the \code{Event} object.}
}
\value{
An object of class \code{"casewise"} containing:
  \item{casewise}{Matrix with time, casewise concordance, and standard errors.}
  \item{marg}{Matrix with time, marginal CIF, and standard errors.}
  \item{concordance}{Matrix with time, concordance, and standard errors.}
  \item{timer}{Time points used.}
  \item{P1, se.P1}{Extracted concordance values and SEs.}
}
\description{
Estimates the casewise concordance based on concordance and marginal estimates 
derived from \code{prodlim} objects. Unlike \code{test_casewise}, this function 
does not perform hypothesis testing but focuses on estimation and plotting.
}
\examples{
\donttest{ ## Reduce Ex.Timings
library(prodlim)
data(prt);
prt <- force_same_cens(prt,cause="status")

### marginal cumulative incidence of prostate cancer
outm <- prodlim(Hist(time,status)~+1,data=prt)

times <- 60:100
cifmz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="MZ"))
cifdz <- predict(outm,cause=2,time=times,newdata=data.frame(zyg="DZ"))

### concordance for MZ and DZ twins
cc <- bicomprisk(Event(time,status)~strata(zyg)+id(id),data=prt,cause=c(2,2),prodlim=TRUE)
cdz <- cc$model$"DZ"
cmz <- cc$model$"MZ"

cdz <- casewise(cdz,outm,cause.marg=2) 
cmz <- casewise(cmz,outm,cause.marg=2)

plot(cmz,ci=NULL,ylim=c(0,0.5),xlim=c(60,100),legend=TRUE,col=c(3,2,1))
par(new=TRUE)
plot(cdz,ci=NULL,ylim=c(0,0.5),xlim=c(60,100),legend=TRUE)
summary(cdz)
summary(cmz)
}
}
\seealso{
\code{\link{test_casewise}}, \code{\link{bicomprisk}}
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{km}
\alias{km}
\title{Kaplan-Meier with Robust Standard Errors}
\usage{
km(formula, data = data, km = TRUE, ...)
}
\arguments{
\item{formula}{Formula with 'Surv' or 'Event' outcome.}

\item{data}{Data frame.}

\item{km}{Logical; if \code{TRUE}, returns Kaplan-Meier estimates; otherwise returns Nelson-Aalen based estimates.}

\item{...}{Additional arguments passed to \code{phreg}.}
}
\value{
An object of class \code{"km"} (extends \code{"predictphreg"}) containing:
\item{surr}{Survival probabilities.}
\item{se.surv}{Standard errors.}
\item{lower, upper}{Confidence intervals.}
}
\description{
Computes Kaplan-Meier estimates with robust standard errors.
Robust variance is the default and is obtained from the \code{predict} call.
}
\examples{
data(sTRACE)
sTRACE$cluster <- sample(1:100, 500, replace = TRUE)
out1 <- km(Surv(time, status == 9) ~ strata(vf, chf), data = sTRACE)
out2 <- km(Surv(time, status == 9) ~ strata(vf, chf) + cluster(cluster), data = sTRACE)

summary(out1, times = 1:3)
summary(out2, times = 1:3)

par(mfrow = c(1, 2))
plot(out1, se = TRUE)
plot(out2, se = TRUE)
}
\author{
Thomas Scheike
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm-utils.R
\name{binreg_IPTW}
\alias{binreg_IPTW}
\title{IPTW logistic regression, Inverse Probabibilty of Treatment Weighted binreg}
\usage{
binreg_IPTW(
  formula,
  data,
  treat.model = NULL,
  weights = NULL,
  estpr = 1,
  pi0 = 0.5,
  ...
)
}
\arguments{
\item{formula}{for binreg}

\item{data}{data-frame for estimation}

\item{treat.model}{propensity score model (binary or multinomial)}

\item{weights}{may be given, and then uses weights*w(A) as the weights}

\item{estpr}{to estimate propensity scores and get infuence function contribution to uncertainty}

\item{pi0}{fixed simple weights}

\item{...}{arguments for  binreg call}
}
\description{
Fits logistic regression model with treatment weights \deqn{ w(A)= \sum_a I(A=a)/P(A=a|X) }, computes
standard errors via influence functions that are returned as the IID argument. 
Propensity scores are fitted using either logistic regression (glm) or the multinomial model (mlogit) when more
than two categories for treatment. The treatment needs to be a factor and is identified on the rhs
of the "treat.model". Can handle right censored binreg type estimating equations with IPTW weights.
}
\details{
Also works with cluster argument.
}
\examples{

data(bmt)
dfactor(bmt) <- platelet.f~platelet
## logistic modelling of cumulative incidence 
gg <- binreg_IPTW(Event(time,cause)~platelet.f+age,bmt,
	       treat.model=platelet.f~age,time=30)
summary(gg)
head(iid(gg))

## logistic modelling  
gg <- binreg_IPTW(tcell~platelet.f+age,bmt,
	       treat.model=platelet.f~age)
summary(gg)
}
\author{
Thomas Scheike
}
